/* This file was generated by upbc (the upb compiler) from the input
 * file:
 *
 *     envoy/api/v2/eds.proto
 *
 * Do not edit -- your changes will be discarded when the file is
 * regenerated. */

#include <stddef.h>
#include "upb/msg.h"
#include "envoy/api/v2/eds.upb.h"
#include "envoy/api/v2/discovery.upb.h"
#include "envoy/api/v2/endpoint/endpoint.upb.h"
#include "envoy/type/percent.upb.h"
#include "google/api/annotations.upb.h"
#include "validate/validate.upb.h"
#include "gogoproto/gogo.upb.h"
#include "google/protobuf/wrappers.upb.h"

#include "upb/port_def.inc"

static const upb_msglayout *const envoy_api_v2_ClusterLoadAssignment_submsgs[2] = {
  &envoy_api_v2_ClusterLoadAssignment_Policy_msginit,
  &envoy_api_v2_endpoint_LocalityLbEndpoints_msginit,
};

static const upb_msglayout_field envoy_api_v2_ClusterLoadAssignment__fields[3] = {
  {1, UPB_SIZE(0, 0), 0, 0, 9, 1},
  {2, UPB_SIZE(12, 24), 0, 1, 11, 3},
  {4, UPB_SIZE(8, 16), 0, 0, 11, 1},
};

const upb_msglayout envoy_api_v2_ClusterLoadAssignment_msginit = {
  &envoy_api_v2_ClusterLoadAssignment_submsgs[0],
  &envoy_api_v2_ClusterLoadAssignment__fields[0],
  UPB_SIZE(16, 32), 3, false,
};

static const upb_msglayout *const envoy_api_v2_ClusterLoadAssignment_Policy_submsgs[2] = {
  &envoy_api_v2_ClusterLoadAssignment_Policy_DropOverload_msginit,
  &google_protobuf_UInt32Value_msginit,
};

static const upb_msglayout_field envoy_api_v2_ClusterLoadAssignment_Policy__fields[2] = {
  {2, UPB_SIZE(4, 8), 0, 0, 11, 3},
  {3, UPB_SIZE(0, 0), 0, 1, 11, 1},
};

const upb_msglayout envoy_api_v2_ClusterLoadAssignment_Policy_msginit = {
  &envoy_api_v2_ClusterLoadAssignment_Policy_submsgs[0],
  &envoy_api_v2_ClusterLoadAssignment_Policy__fields[0],
  UPB_SIZE(8, 16), 2, false,
};

static const upb_msglayout *const envoy_api_v2_ClusterLoadAssignment_Policy_DropOverload_submsgs[1] = {
  &envoy_type_FractionalPercent_msginit,
};

static const upb_msglayout_field envoy_api_v2_ClusterLoadAssignment_Policy_DropOverload__fields[2] = {
  {1, UPB_SIZE(0, 0), 0, 0, 9, 1},
  {2, UPB_SIZE(8, 16), 0, 0, 11, 1},
};

const upb_msglayout envoy_api_v2_ClusterLoadAssignment_Policy_DropOverload_msginit = {
  &envoy_api_v2_ClusterLoadAssignment_Policy_DropOverload_submsgs[0],
  &envoy_api_v2_ClusterLoadAssignment_Policy_DropOverload__fields[0],
  UPB_SIZE(16, 32), 2, false,
};

#include "upb/port_undef.inc"

