/* This file was generated by upbc (the upb compiler) from the input
 * file:
 *
 *     google/rpc/status.proto
 *
 * Do not edit -- your changes will be discarded when the file is
 * regenerated. */

#include <stddef.h>
#include "upb/msg.h"
#include "google/rpc/status.upb.h"
#include "google/protobuf/any.upb.h"

#include "upb/port_def.inc"

static const upb_msglayout *const google_rpc_Status_submsgs[1] = {
  &google_protobuf_Any_msginit,
};

static const upb_msglayout_field google_rpc_Status__fields[3] = {
  {1, UPB_SIZE(0, 0), 0, 0, 5, 1},
  {2, UPB_SIZE(4, 8), 0, 0, 9, 1},
  {3, UPB_SIZE(12, 24), 0, 0, 11, 3},
};

const upb_msglayout google_rpc_Status_msginit = {
  &google_rpc_Status_submsgs[0],
  &google_rpc_Status__fields[0],
  UPB_SIZE(16, 32), 3, false,
};

#include "upb/port_undef.inc"

