/* Generated by Cython 0.28.4 */

/* BEGIN: Cython Metadata
{
    "distutils": {
        "define_macros": [
            [
                "OPENSSL_NO_ASM", 
                1
            ], 
            [
                "_WIN32_WINNT", 
                1536
            ], 
            [
                "GPR_BACKWARDS_COMPATIBILITY_MODE", 
                1
            ], 
            [
                "HAVE_CONFIG_H", 
                1
            ], 
            [
                "GRPC_ENABLE_FORK_SUPPORT", 
                1
            ], 
            [
                "PyMODINIT_FUNC", 
                "extern \"C\" __attribute__((visibility (\"default\"))) void"
            ], 
            [
                "GRPC_POSIX_FORK_ALLOW_PTHREAD_ATFORK", 
                1
            ]
        ], 
        "depends": [
            "include/grpc/byte_buffer_reader.h", 
            "include/grpc/compression.h", 
            "include/grpc/grpc.h", 
            "include/grpc/grpc_security.h", 
            "include/grpc/impl/codegen/compression_types.h", 
            "include/grpc/impl/codegen/propagation_bits.h", 
            "include/grpc/impl/codegen/slice.h", 
            "include/grpc/support/alloc.h", 
            "src/core/lib/iomgr/error.h", 
            "src/core/lib/iomgr/gevent_util.h", 
            "src/core/lib/iomgr/iomgr_custom.h", 
            "src/core/lib/iomgr/pollset_custom.h", 
            "src/core/lib/iomgr/resolve_address.h", 
            "src/core/lib/iomgr/resolve_address_custom.h", 
            "src/core/lib/iomgr/sockaddr.h", 
            "src/core/lib/iomgr/sockaddr_utils.h", 
            "src/core/lib/iomgr/tcp_custom.h", 
            "src/core/lib/iomgr/timer_custom.h"
        ], 
        "extra_compile_args": [
            "-std=c++11", 
            "-std=gnu99", 
            "-fvisibility=hidden", 
            "-fno-wrapv", 
            "-fno-exceptions", 
            "-DPB_FIELD_32BIT"
        ], 
        "extra_link_args": [
            "-lpthread", 
            "-Wl,-wrap,memcpy", 
            "-static-libgcc"
        ], 
        "include_dirs": [
            "src/python/grpcio", 
            "include", 
            ".", 
            "third_party/boringssl/include", 
            "third_party/zlib", 
            "third_party/nanopb", 
            "third_party/cares", 
            "third_party/cares/cares", 
            "third_party/cares/config_linux", 
            "third_party/address_sorting/include"
        ], 
        "language": "c++", 
        "libraries": [
            "rt", 
            "m"
        ], 
        "name": "grpc._cython.cygrpc", 
        "sources": [
            "src/python/grpcio/grpc/_cython/cygrpc.pyx", 
            "third_party/address_sorting/address_sorting.c", 
            "third_party/address_sorting/address_sorting_posix.c", 
            "third_party/address_sorting/address_sorting_windows.c", 
            "src/core/lib/gpr/alloc.cc", 
            "src/core/lib/gpr/atm.cc", 
            "src/core/lib/gpr/cpu_iphone.cc", 
            "src/core/lib/gpr/cpu_linux.cc", 
            "src/core/lib/gpr/cpu_posix.cc", 
            "src/core/lib/gpr/cpu_windows.cc", 
            "src/core/lib/gpr/env_linux.cc", 
            "src/core/lib/gpr/env_posix.cc", 
            "src/core/lib/gpr/env_windows.cc", 
            "src/core/lib/gpr/host_port.cc", 
            "src/core/lib/gpr/log.cc", 
            "src/core/lib/gpr/log_android.cc", 
            "src/core/lib/gpr/log_linux.cc", 
            "src/core/lib/gpr/log_posix.cc", 
            "src/core/lib/gpr/log_windows.cc", 
            "src/core/lib/gpr/mpscq.cc", 
            "src/core/lib/gpr/murmur_hash.cc", 
            "src/core/lib/gpr/string.cc", 
            "src/core/lib/gpr/string_posix.cc", 
            "src/core/lib/gpr/string_util_windows.cc", 
            "src/core/lib/gpr/string_windows.cc", 
            "src/core/lib/gpr/sync.cc", 
            "src/core/lib/gpr/sync_posix.cc", 
            "src/core/lib/gpr/sync_windows.cc", 
            "src/core/lib/gpr/time.cc", 
            "src/core/lib/gpr/time_posix.cc", 
            "src/core/lib/gpr/time_precise.cc", 
            "src/core/lib/gpr/time_windows.cc", 
            "src/core/lib/gpr/tls_pthread.cc", 
            "src/core/lib/gpr/tmpfile_msys.cc", 
            "src/core/lib/gpr/tmpfile_posix.cc", 
            "src/core/lib/gpr/tmpfile_windows.cc", 
            "src/core/lib/gpr/wrap_memcpy.cc", 
            "src/core/lib/gprpp/arena.cc", 
            "src/core/lib/gprpp/fork.cc", 
            "src/core/lib/gprpp/global_config_env.cc", 
            "src/core/lib/gprpp/thd_posix.cc", 
            "src/core/lib/gprpp/thd_windows.cc", 
            "src/core/lib/profiling/basic_timers.cc", 
            "src/core/lib/profiling/stap_timers.cc", 
            "src/core/lib/surface/init.cc", 
            "src/core/lib/avl/avl.cc", 
            "src/core/lib/backoff/backoff.cc", 
            "src/core/lib/channel/channel_args.cc", 
            "src/core/lib/channel/channel_stack.cc", 
            "src/core/lib/channel/channel_stack_builder.cc", 
            "src/core/lib/channel/channel_trace.cc", 
            "src/core/lib/channel/channelz.cc", 
            "src/core/lib/channel/channelz_registry.cc", 
            "src/core/lib/channel/connected_channel.cc", 
            "src/core/lib/channel/handshaker.cc", 
            "src/core/lib/channel/handshaker_registry.cc", 
            "src/core/lib/channel/status_util.cc", 
            "src/core/lib/compression/compression.cc", 
            "src/core/lib/compression/compression_args.cc", 
            "src/core/lib/compression/compression_internal.cc", 
            "src/core/lib/compression/message_compress.cc", 
            "src/core/lib/compression/stream_compression.cc", 
            "src/core/lib/compression/stream_compression_gzip.cc", 
            "src/core/lib/compression/stream_compression_identity.cc", 
            "src/core/lib/debug/stats.cc", 
            "src/core/lib/debug/stats_data.cc", 
            "src/core/lib/http/format_request.cc", 
            "src/core/lib/http/httpcli.cc", 
            "src/core/lib/http/parser.cc", 
            "src/core/lib/iomgr/buffer_list.cc", 
            "src/core/lib/iomgr/call_combiner.cc", 
            "src/core/lib/iomgr/cfstream_handle.cc", 
            "src/core/lib/iomgr/combiner.cc", 
            "src/core/lib/iomgr/endpoint.cc", 
            "src/core/lib/iomgr/endpoint_cfstream.cc", 
            "src/core/lib/iomgr/endpoint_pair_posix.cc", 
            "src/core/lib/iomgr/endpoint_pair_uv.cc", 
            "src/core/lib/iomgr/endpoint_pair_windows.cc", 
            "src/core/lib/iomgr/error.cc", 
            "src/core/lib/iomgr/error_cfstream.cc", 
            "src/core/lib/iomgr/ev_epoll1_linux.cc", 
            "src/core/lib/iomgr/ev_epollex_linux.cc", 
            "src/core/lib/iomgr/ev_poll_posix.cc", 
            "src/core/lib/iomgr/ev_posix.cc", 
            "src/core/lib/iomgr/ev_windows.cc", 
            "src/core/lib/iomgr/exec_ctx.cc", 
            "src/core/lib/iomgr/executor.cc", 
            "src/core/lib/iomgr/fork_posix.cc", 
            "src/core/lib/iomgr/fork_windows.cc", 
            "src/core/lib/iomgr/gethostname_fallback.cc", 
            "src/core/lib/iomgr/gethostname_host_name_max.cc", 
            "src/core/lib/iomgr/gethostname_sysconf.cc", 
            "src/core/lib/iomgr/grpc_if_nametoindex_posix.cc", 
            "src/core/lib/iomgr/grpc_if_nametoindex_unsupported.cc", 
            "src/core/lib/iomgr/internal_errqueue.cc", 
            "src/core/lib/iomgr/iocp_windows.cc", 
            "src/core/lib/iomgr/iomgr.cc", 
            "src/core/lib/iomgr/iomgr_custom.cc", 
            "src/core/lib/iomgr/iomgr_internal.cc", 
            "src/core/lib/iomgr/iomgr_posix.cc", 
            "src/core/lib/iomgr/iomgr_posix_cfstream.cc", 
            "src/core/lib/iomgr/iomgr_uv.cc", 
            "src/core/lib/iomgr/iomgr_windows.cc", 
            "src/core/lib/iomgr/is_epollexclusive_available.cc", 
            "src/core/lib/iomgr/load_file.cc", 
            "src/core/lib/iomgr/lockfree_event.cc", 
            "src/core/lib/iomgr/polling_entity.cc", 
            "src/core/lib/iomgr/pollset.cc", 
            "src/core/lib/iomgr/pollset_custom.cc", 
            "src/core/lib/iomgr/pollset_set.cc", 
            "src/core/lib/iomgr/pollset_set_custom.cc", 
            "src/core/lib/iomgr/pollset_set_windows.cc", 
            "src/core/lib/iomgr/pollset_uv.cc", 
            "src/core/lib/iomgr/pollset_windows.cc", 
            "src/core/lib/iomgr/resolve_address.cc", 
            "src/core/lib/iomgr/resolve_address_custom.cc", 
            "src/core/lib/iomgr/resolve_address_posix.cc", 
            "src/core/lib/iomgr/resolve_address_windows.cc", 
            "src/core/lib/iomgr/resource_quota.cc", 
            "src/core/lib/iomgr/sockaddr_utils.cc", 
            "src/core/lib/iomgr/socket_factory_posix.cc", 
            "src/core/lib/iomgr/socket_mutator.cc", 
            "src/core/lib/iomgr/socket_utils_common_posix.cc", 
            "src/core/lib/iomgr/socket_utils_linux.cc", 
            "src/core/lib/iomgr/socket_utils_posix.cc", 
            "src/core/lib/iomgr/socket_utils_uv.cc", 
            "src/core/lib/iomgr/socket_utils_windows.cc", 
            "src/core/lib/iomgr/socket_windows.cc", 
            "src/core/lib/iomgr/tcp_client.cc", 
            "src/core/lib/iomgr/tcp_client_cfstream.cc", 
            "src/core/lib/iomgr/tcp_client_custom.cc", 
            "src/core/lib/iomgr/tcp_client_posix.cc", 
            "src/core/lib/iomgr/tcp_client_windows.cc", 
            "src/core/lib/iomgr/tcp_custom.cc", 
            "src/core/lib/iomgr/tcp_posix.cc", 
            "src/core/lib/iomgr/tcp_server.cc", 
            "src/core/lib/iomgr/tcp_server_custom.cc", 
            "src/core/lib/iomgr/tcp_server_posix.cc", 
            "src/core/lib/iomgr/tcp_server_utils_posix_common.cc", 
            "src/core/lib/iomgr/tcp_server_utils_posix_ifaddrs.cc", 
            "src/core/lib/iomgr/tcp_server_utils_posix_noifaddrs.cc", 
            "src/core/lib/iomgr/tcp_server_windows.cc", 
            "src/core/lib/iomgr/tcp_uv.cc", 
            "src/core/lib/iomgr/tcp_windows.cc", 
            "src/core/lib/iomgr/time_averaged_stats.cc", 
            "src/core/lib/iomgr/timer.cc", 
            "src/core/lib/iomgr/timer_custom.cc", 
            "src/core/lib/iomgr/timer_generic.cc", 
            "src/core/lib/iomgr/timer_heap.cc", 
            "src/core/lib/iomgr/timer_manager.cc", 
            "src/core/lib/iomgr/timer_uv.cc", 
            "src/core/lib/iomgr/udp_server.cc", 
            "src/core/lib/iomgr/unix_sockets_posix.cc", 
            "src/core/lib/iomgr/unix_sockets_posix_noop.cc", 
            "src/core/lib/iomgr/wakeup_fd_eventfd.cc", 
            "src/core/lib/iomgr/wakeup_fd_nospecial.cc", 
            "src/core/lib/iomgr/wakeup_fd_pipe.cc", 
            "src/core/lib/iomgr/wakeup_fd_posix.cc", 
            "src/core/lib/json/json.cc", 
            "src/core/lib/json/json_reader.cc", 
            "src/core/lib/json/json_string.cc", 
            "src/core/lib/json/json_writer.cc", 
            "src/core/lib/slice/b64.cc", 
            "src/core/lib/slice/percent_encoding.cc", 
            "src/core/lib/slice/slice.cc", 
            "src/core/lib/slice/slice_buffer.cc", 
            "src/core/lib/slice/slice_intern.cc", 
            "src/core/lib/slice/slice_string_helpers.cc", 
            "src/core/lib/surface/api_trace.cc", 
            "src/core/lib/surface/byte_buffer.cc", 
            "src/core/lib/surface/byte_buffer_reader.cc", 
            "src/core/lib/surface/call.cc", 
            "src/core/lib/surface/call_details.cc", 
            "src/core/lib/surface/call_log_batch.cc", 
            "src/core/lib/surface/channel.cc", 
            "src/core/lib/surface/channel_init.cc", 
            "src/core/lib/surface/channel_ping.cc", 
            "src/core/lib/surface/channel_stack_type.cc", 
            "src/core/lib/surface/completion_queue.cc", 
            "src/core/lib/surface/completion_queue_factory.cc", 
            "src/core/lib/surface/event_string.cc", 
            "src/core/lib/surface/lame_client.cc", 
            "src/core/lib/surface/metadata_array.cc", 
            "src/core/lib/surface/server.cc", 
            "src/core/lib/surface/validate_metadata.cc", 
            "src/core/lib/surface/version.cc", 
            "src/core/lib/transport/bdp_estimator.cc", 
            "src/core/lib/transport/byte_stream.cc", 
            "src/core/lib/transport/connectivity_state.cc", 
            "src/core/lib/transport/error_utils.cc", 
            "src/core/lib/transport/metadata.cc", 
            "src/core/lib/transport/metadata_batch.cc", 
            "src/core/lib/transport/pid_controller.cc", 
            "src/core/lib/transport/static_metadata.cc", 
            "src/core/lib/transport/status_conversion.cc", 
            "src/core/lib/transport/status_metadata.cc", 
            "src/core/lib/transport/timeout_encoding.cc", 
            "src/core/lib/transport/transport.cc", 
            "src/core/lib/transport/transport_op_string.cc", 
            "src/core/lib/uri/uri_parser.cc", 
            "src/core/lib/debug/trace.cc", 
            "src/core/ext/transport/chttp2/server/secure/server_secure_chttp2.cc", 
            "src/core/ext/transport/chttp2/transport/bin_decoder.cc", 
            "src/core/ext/transport/chttp2/transport/bin_encoder.cc", 
            "src/core/ext/transport/chttp2/transport/chttp2_plugin.cc", 
            "src/core/ext/transport/chttp2/transport/chttp2_transport.cc", 
            "src/core/ext/transport/chttp2/transport/context_list.cc", 
            "src/core/ext/transport/chttp2/transport/flow_control.cc", 
            "src/core/ext/transport/chttp2/transport/frame_data.cc", 
            "src/core/ext/transport/chttp2/transport/frame_goaway.cc", 
            "src/core/ext/transport/chttp2/transport/frame_ping.cc", 
            "src/core/ext/transport/chttp2/transport/frame_rst_stream.cc", 
            "src/core/ext/transport/chttp2/transport/frame_settings.cc", 
            "src/core/ext/transport/chttp2/transport/frame_window_update.cc", 
            "src/core/ext/transport/chttp2/transport/hpack_encoder.cc", 
            "src/core/ext/transport/chttp2/transport/hpack_parser.cc", 
            "src/core/ext/transport/chttp2/transport/hpack_table.cc", 
            "src/core/ext/transport/chttp2/transport/http2_settings.cc", 
            "src/core/ext/transport/chttp2/transport/huffsyms.cc", 
            "src/core/ext/transport/chttp2/transport/incoming_metadata.cc", 
            "src/core/ext/transport/chttp2/transport/parsing.cc", 
            "src/core/ext/transport/chttp2/transport/stream_lists.cc", 
            "src/core/ext/transport/chttp2/transport/stream_map.cc", 
            "src/core/ext/transport/chttp2/transport/varint.cc", 
            "src/core/ext/transport/chttp2/transport/writing.cc", 
            "src/core/ext/transport/chttp2/alpn/alpn.cc", 
            "src/core/ext/filters/http/client/http_client_filter.cc", 
            "src/core/ext/filters/http/http_filters_plugin.cc", 
            "src/core/ext/filters/http/message_compress/message_compress_filter.cc", 
            "src/core/ext/filters/http/server/http_server_filter.cc", 
            "src/core/lib/http/httpcli_security_connector.cc", 
            "src/core/lib/security/context/security_context.cc", 
            "src/core/lib/security/credentials/alts/alts_credentials.cc", 
            "src/core/lib/security/credentials/composite/composite_credentials.cc", 
            "src/core/lib/security/credentials/credentials.cc", 
            "src/core/lib/security/credentials/credentials_metadata.cc", 
            "src/core/lib/security/credentials/fake/fake_credentials.cc", 
            "src/core/lib/security/credentials/google_default/credentials_generic.cc", 
            "src/core/lib/security/credentials/google_default/google_default_credentials.cc", 
            "src/core/lib/security/credentials/iam/iam_credentials.cc", 
            "src/core/lib/security/credentials/jwt/json_token.cc", 
            "src/core/lib/security/credentials/jwt/jwt_credentials.cc", 
            "src/core/lib/security/credentials/jwt/jwt_verifier.cc", 
            "src/core/lib/security/credentials/local/local_credentials.cc", 
            "src/core/lib/security/credentials/oauth2/oauth2_credentials.cc", 
            "src/core/lib/security/credentials/plugin/plugin_credentials.cc", 
            "src/core/lib/security/credentials/ssl/ssl_credentials.cc", 
            "src/core/lib/security/credentials/tls/grpc_tls_credentials_options.cc", 
            "src/core/lib/security/credentials/tls/spiffe_credentials.cc", 
            "src/core/lib/security/security_connector/alts/alts_security_connector.cc", 
            "src/core/lib/security/security_connector/fake/fake_security_connector.cc", 
            "src/core/lib/security/security_connector/load_system_roots_fallback.cc", 
            "src/core/lib/security/security_connector/load_system_roots_linux.cc", 
            "src/core/lib/security/security_connector/local/local_security_connector.cc", 
            "src/core/lib/security/security_connector/security_connector.cc", 
            "src/core/lib/security/security_connector/ssl/ssl_security_connector.cc", 
            "src/core/lib/security/security_connector/ssl_utils.cc", 
            "src/core/lib/security/security_connector/tls/spiffe_security_connector.cc", 
            "src/core/lib/security/transport/client_auth_filter.cc", 
            "src/core/lib/security/transport/secure_endpoint.cc", 
            "src/core/lib/security/transport/security_handshaker.cc", 
            "src/core/lib/security/transport/server_auth_filter.cc", 
            "src/core/lib/security/transport/target_authority_table.cc", 
            "src/core/lib/security/transport/tsi_error.cc", 
            "src/core/lib/security/util/json_util.cc", 
            "src/core/lib/surface/init_secure.cc", 
            "src/core/tsi/alts/crypt/aes_gcm.cc", 
            "src/core/tsi/alts/crypt/gsec.cc", 
            "src/core/tsi/alts/frame_protector/alts_counter.cc", 
            "src/core/tsi/alts/frame_protector/alts_crypter.cc", 
            "src/core/tsi/alts/frame_protector/alts_frame_protector.cc", 
            "src/core/tsi/alts/frame_protector/alts_record_protocol_crypter_common.cc", 
            "src/core/tsi/alts/frame_protector/alts_seal_privacy_integrity_crypter.cc", 
            "src/core/tsi/alts/frame_protector/alts_unseal_privacy_integrity_crypter.cc", 
            "src/core/tsi/alts/frame_protector/frame_handler.cc", 
            "src/core/tsi/alts/handshaker/alts_handshaker_client.cc", 
            "src/core/tsi/alts/handshaker/alts_shared_resource.cc", 
            "src/core/tsi/alts/handshaker/alts_tsi_handshaker.cc", 
            "src/core/tsi/alts/zero_copy_frame_protector/alts_grpc_integrity_only_record_protocol.cc", 
            "src/core/tsi/alts/zero_copy_frame_protector/alts_grpc_privacy_integrity_record_protocol.cc", 
            "src/core/tsi/alts/zero_copy_frame_protector/alts_grpc_record_protocol_common.cc", 
            "src/core/tsi/alts/zero_copy_frame_protector/alts_iovec_record_protocol.cc", 
            "src/core/tsi/alts/zero_copy_frame_protector/alts_zero_copy_grpc_protector.cc", 
            "src/core/lib/security/credentials/alts/check_gcp_environment.cc", 
            "src/core/lib/security/credentials/alts/check_gcp_environment_linux.cc", 
            "src/core/lib/security/credentials/alts/check_gcp_environment_no_op.cc", 
            "src/core/lib/security/credentials/alts/check_gcp_environment_windows.cc", 
            "src/core/lib/security/credentials/alts/grpc_alts_credentials_client_options.cc", 
            "src/core/lib/security/credentials/alts/grpc_alts_credentials_options.cc", 
            "src/core/lib/security/credentials/alts/grpc_alts_credentials_server_options.cc", 
            "src/core/tsi/alts/handshaker/alts_handshaker_service_api.cc", 
            "src/core/tsi/alts/handshaker/alts_handshaker_service_api_util.cc", 
            "src/core/tsi/alts/handshaker/alts_tsi_utils.cc", 
            "src/core/tsi/alts/handshaker/transport_security_common_api.cc", 
            "src/core/tsi/alts/handshaker/altscontext.pb.c", 
            "src/core/tsi/alts/handshaker/handshaker.pb.c", 
            "src/core/tsi/alts/handshaker/transport_security_common.pb.c", 
            "third_party/nanopb/pb_common.c", 
            "third_party/nanopb/pb_decode.c", 
            "third_party/nanopb/pb_encode.c", 
            "src/core/tsi/transport_security.cc", 
            "src/core/ext/transport/chttp2/client/insecure/channel_create.cc", 
            "src/core/ext/transport/chttp2/client/insecure/channel_create_posix.cc", 
            "src/core/ext/transport/chttp2/client/authority.cc", 
            "src/core/ext/transport/chttp2/client/chttp2_connector.cc", 
            "src/core/ext/filters/client_channel/backup_poller.cc", 
            "src/core/ext/filters/client_channel/channel_connectivity.cc", 
            "src/core/ext/filters/client_channel/client_channel.cc", 
            "src/core/ext/filters/client_channel/client_channel_channelz.cc", 
            "src/core/ext/filters/client_channel/client_channel_factory.cc", 
            "src/core/ext/filters/client_channel/client_channel_plugin.cc", 
            "src/core/ext/filters/client_channel/connector.cc", 
            "src/core/ext/filters/client_channel/global_subchannel_pool.cc", 
            "src/core/ext/filters/client_channel/health/health_check_client.cc", 
            "src/core/ext/filters/client_channel/http_connect_handshaker.cc", 
            "src/core/ext/filters/client_channel/http_proxy.cc", 
            "src/core/ext/filters/client_channel/lb_policy.cc", 
            "src/core/ext/filters/client_channel/lb_policy_registry.cc", 
            "src/core/ext/filters/client_channel/local_subchannel_pool.cc", 
            "src/core/ext/filters/client_channel/parse_address.cc", 
            "src/core/ext/filters/client_channel/proxy_mapper.cc", 
            "src/core/ext/filters/client_channel/proxy_mapper_registry.cc", 
            "src/core/ext/filters/client_channel/resolver.cc", 
            "src/core/ext/filters/client_channel/resolver_registry.cc", 
            "src/core/ext/filters/client_channel/resolver_result_parsing.cc", 
            "src/core/ext/filters/client_channel/resolving_lb_policy.cc", 
            "src/core/ext/filters/client_channel/retry_throttle.cc", 
            "src/core/ext/filters/client_channel/server_address.cc", 
            "src/core/ext/filters/client_channel/service_config.cc", 
            "src/core/ext/filters/client_channel/subchannel.cc", 
            "src/core/ext/filters/client_channel/subchannel_pool_interface.cc", 
            "src/core/ext/filters/deadline/deadline_filter.cc", 
            "src/core/ext/filters/client_channel/health/health.pb.c", 
            "src/core/tsi/fake_transport_security.cc", 
            "src/core/tsi/local_transport_security.cc", 
            "src/core/tsi/ssl/session_cache/ssl_session_boringssl.cc", 
            "src/core/tsi/ssl/session_cache/ssl_session_cache.cc", 
            "src/core/tsi/ssl/session_cache/ssl_session_openssl.cc", 
            "src/core/tsi/ssl_transport_security.cc", 
            "src/core/tsi/transport_security_grpc.cc", 
            "src/core/ext/transport/chttp2/server/chttp2_server.cc", 
            "src/core/ext/transport/chttp2/client/secure/secure_channel_create.cc", 
            "src/core/ext/transport/chttp2/server/insecure/server_chttp2.cc", 
            "src/core/ext/transport/chttp2/server/insecure/server_chttp2_posix.cc", 
            "src/core/ext/transport/inproc/inproc_plugin.cc", 
            "src/core/ext/transport/inproc/inproc_transport.cc", 
            "src/core/ext/filters/client_channel/lb_policy/grpclb/client_load_reporting_filter.cc", 
            "src/core/ext/filters/client_channel/lb_policy/grpclb/grpclb.cc", 
            "src/core/ext/filters/client_channel/lb_policy/grpclb/grpclb_channel_secure.cc", 
            "src/core/ext/filters/client_channel/lb_policy/grpclb/grpclb_client_stats.cc", 
            "src/core/ext/filters/client_channel/lb_policy/grpclb/load_balancer_api.cc", 
            "src/core/ext/filters/client_channel/resolver/fake/fake_resolver.cc", 
            "src/core/ext/filters/client_channel/lb_policy/grpclb/proto/grpc/lb/v1/google/protobuf/duration.pb.c", 
            "src/core/ext/filters/client_channel/lb_policy/grpclb/proto/grpc/lb/v1/google/protobuf/timestamp.pb.c", 
            "src/core/ext/filters/client_channel/lb_policy/grpclb/proto/grpc/lb/v1/load_balancer.pb.c", 
            "src/core/ext/filters/client_channel/lb_policy/xds/xds.cc", 
            "src/core/ext/filters/client_channel/lb_policy/xds/xds_channel_secure.cc", 
            "src/core/ext/filters/client_channel/lb_policy/xds/xds_client_stats.cc", 
            "src/core/ext/filters/client_channel/lb_policy/xds/xds_load_balancer_api.cc", 
            "src/core/ext/filters/client_channel/lb_policy/pick_first/pick_first.cc", 
            "src/core/ext/filters/client_channel/lb_policy/round_robin/round_robin.cc", 
            "src/core/ext/filters/client_channel/resolver/dns/c_ares/dns_resolver_ares.cc", 
            "src/core/ext/filters/client_channel/resolver/dns/c_ares/grpc_ares_ev_driver.cc", 
            "src/core/ext/filters/client_channel/resolver/dns/c_ares/grpc_ares_ev_driver_libuv.cc", 
            "src/core/ext/filters/client_channel/resolver/dns/c_ares/grpc_ares_ev_driver_posix.cc", 
            "src/core/ext/filters/client_channel/resolver/dns/c_ares/grpc_ares_ev_driver_windows.cc", 
            "src/core/ext/filters/client_channel/resolver/dns/c_ares/grpc_ares_wrapper.cc", 
            "src/core/ext/filters/client_channel/resolver/dns/c_ares/grpc_ares_wrapper_fallback.cc", 
            "src/core/ext/filters/client_channel/resolver/dns/c_ares/grpc_ares_wrapper_libuv.cc", 
            "src/core/ext/filters/client_channel/resolver/dns/c_ares/grpc_ares_wrapper_posix.cc", 
            "src/core/ext/filters/client_channel/resolver/dns/c_ares/grpc_ares_wrapper_windows.cc", 
            "src/core/ext/filters/client_channel/resolver/dns/dns_resolver_selection.cc", 
            "src/core/ext/filters/client_channel/resolver/dns/native/dns_resolver.cc", 
            "src/core/ext/filters/client_channel/resolver/sockaddr/sockaddr_resolver.cc", 
            "src/core/ext/filters/census/grpc_context.cc", 
            "src/core/ext/filters/max_age/max_age_filter.cc", 
            "src/core/ext/filters/message_size/message_size_filter.cc", 
            "src/core/ext/filters/http/client_authority_filter.cc", 
            "src/core/ext/filters/workarounds/workaround_cronet_compression_filter.cc", 
            "src/core/ext/filters/workarounds/workaround_utils.cc", 
            "src/core/plugin_registry/grpc_plugin_registry.cc", 
            "src/boringssl/err_data.c", 
            "third_party/boringssl/crypto/asn1/a_bitstr.c", 
            "third_party/boringssl/crypto/asn1/a_bool.c", 
            "third_party/boringssl/crypto/asn1/a_d2i_fp.c", 
            "third_party/boringssl/crypto/asn1/a_dup.c", 
            "third_party/boringssl/crypto/asn1/a_enum.c", 
            "third_party/boringssl/crypto/asn1/a_gentm.c", 
            "third_party/boringssl/crypto/asn1/a_i2d_fp.c", 
            "third_party/boringssl/crypto/asn1/a_int.c", 
            "third_party/boringssl/crypto/asn1/a_mbstr.c", 
            "third_party/boringssl/crypto/asn1/a_object.c", 
            "third_party/boringssl/crypto/asn1/a_octet.c", 
            "third_party/boringssl/crypto/asn1/a_print.c", 
            "third_party/boringssl/crypto/asn1/a_strnid.c", 
            "third_party/boringssl/crypto/asn1/a_time.c", 
            "third_party/boringssl/crypto/asn1/a_type.c", 
            "third_party/boringssl/crypto/asn1/a_utctm.c", 
            "third_party/boringssl/crypto/asn1/a_utf8.c", 
            "third_party/boringssl/crypto/asn1/asn1_lib.c", 
            "third_party/boringssl/crypto/asn1/asn1_par.c", 
            "third_party/boringssl/crypto/asn1/asn_pack.c", 
            "third_party/boringssl/crypto/asn1/f_enum.c", 
            "third_party/boringssl/crypto/asn1/f_int.c", 
            "third_party/boringssl/crypto/asn1/f_string.c", 
            "third_party/boringssl/crypto/asn1/tasn_dec.c", 
            "third_party/boringssl/crypto/asn1/tasn_enc.c", 
            "third_party/boringssl/crypto/asn1/tasn_fre.c", 
            "third_party/boringssl/crypto/asn1/tasn_new.c", 
            "third_party/boringssl/crypto/asn1/tasn_typ.c", 
            "third_party/boringssl/crypto/asn1/tasn_utl.c", 
            "third_party/boringssl/crypto/asn1/time_support.c", 
            "third_party/boringssl/crypto/base64/base64.c", 
            "third_party/boringssl/crypto/bio/bio.c", 
            "third_party/boringssl/crypto/bio/bio_mem.c", 
            "third_party/boringssl/crypto/bio/connect.c", 
            "third_party/boringssl/crypto/bio/fd.c", 
            "third_party/boringssl/crypto/bio/file.c", 
            "third_party/boringssl/crypto/bio/hexdump.c", 
            "third_party/boringssl/crypto/bio/pair.c", 
            "third_party/boringssl/crypto/bio/printf.c", 
            "third_party/boringssl/crypto/bio/socket.c", 
            "third_party/boringssl/crypto/bio/socket_helper.c", 
            "third_party/boringssl/crypto/bn_extra/bn_asn1.c", 
            "third_party/boringssl/crypto/bn_extra/convert.c", 
            "third_party/boringssl/crypto/buf/buf.c", 
            "third_party/boringssl/crypto/bytestring/asn1_compat.c", 
            "third_party/boringssl/crypto/bytestring/ber.c", 
            "third_party/boringssl/crypto/bytestring/cbb.c", 
            "third_party/boringssl/crypto/bytestring/cbs.c", 
            "third_party/boringssl/crypto/chacha/chacha.c", 
            "third_party/boringssl/crypto/cipher_extra/cipher_extra.c", 
            "third_party/boringssl/crypto/cipher_extra/derive_key.c", 
            "third_party/boringssl/crypto/cipher_extra/e_aesccm.c", 
            "third_party/boringssl/crypto/cipher_extra/e_aesctrhmac.c", 
            "third_party/boringssl/crypto/cipher_extra/e_aesgcmsiv.c", 
            "third_party/boringssl/crypto/cipher_extra/e_chacha20poly1305.c", 
            "third_party/boringssl/crypto/cipher_extra/e_null.c", 
            "third_party/boringssl/crypto/cipher_extra/e_rc2.c", 
            "third_party/boringssl/crypto/cipher_extra/e_rc4.c", 
            "third_party/boringssl/crypto/cipher_extra/e_ssl3.c", 
            "third_party/boringssl/crypto/cipher_extra/e_tls.c", 
            "third_party/boringssl/crypto/cipher_extra/tls_cbc.c", 
            "third_party/boringssl/crypto/cmac/cmac.c", 
            "third_party/boringssl/crypto/conf/conf.c", 
            "third_party/boringssl/crypto/cpu-aarch64-fuchsia.c", 
            "third_party/boringssl/crypto/cpu-aarch64-linux.c", 
            "third_party/boringssl/crypto/cpu-arm-linux.c", 
            "third_party/boringssl/crypto/cpu-arm.c", 
            "third_party/boringssl/crypto/cpu-intel.c", 
            "third_party/boringssl/crypto/cpu-ppc64le.c", 
            "third_party/boringssl/crypto/crypto.c", 
            "third_party/boringssl/crypto/curve25519/spake25519.c", 
            "third_party/boringssl/crypto/dh/check.c", 
            "third_party/boringssl/crypto/dh/dh.c", 
            "third_party/boringssl/crypto/dh/dh_asn1.c", 
            "third_party/boringssl/crypto/dh/params.c", 
            "third_party/boringssl/crypto/digest_extra/digest_extra.c", 
            "third_party/boringssl/crypto/dsa/dsa.c", 
            "third_party/boringssl/crypto/dsa/dsa_asn1.c", 
            "third_party/boringssl/crypto/ec_extra/ec_asn1.c", 
            "third_party/boringssl/crypto/ecdh/ecdh.c", 
            "third_party/boringssl/crypto/ecdsa_extra/ecdsa_asn1.c", 
            "third_party/boringssl/crypto/engine/engine.c", 
            "third_party/boringssl/crypto/err/err.c", 
            "third_party/boringssl/crypto/evp/digestsign.c", 
            "third_party/boringssl/crypto/evp/evp.c", 
            "third_party/boringssl/crypto/evp/evp_asn1.c", 
            "third_party/boringssl/crypto/evp/evp_ctx.c", 
            "third_party/boringssl/crypto/evp/p_dsa_asn1.c", 
            "third_party/boringssl/crypto/evp/p_ec.c", 
            "third_party/boringssl/crypto/evp/p_ec_asn1.c", 
            "third_party/boringssl/crypto/evp/p_ed25519.c", 
            "third_party/boringssl/crypto/evp/p_ed25519_asn1.c", 
            "third_party/boringssl/crypto/evp/p_rsa.c", 
            "third_party/boringssl/crypto/evp/p_rsa_asn1.c", 
            "third_party/boringssl/crypto/evp/pbkdf.c", 
            "third_party/boringssl/crypto/evp/print.c", 
            "third_party/boringssl/crypto/evp/scrypt.c", 
            "third_party/boringssl/crypto/evp/sign.c", 
            "third_party/boringssl/crypto/ex_data.c", 
            "third_party/boringssl/crypto/fipsmodule/bcm.c", 
            "third_party/boringssl/crypto/fipsmodule/is_fips.c", 
            "third_party/boringssl/crypto/hkdf/hkdf.c", 
            "third_party/boringssl/crypto/lhash/lhash.c", 
            "third_party/boringssl/crypto/mem.c", 
            "third_party/boringssl/crypto/obj/obj.c", 
            "third_party/boringssl/crypto/obj/obj_xref.c", 
            "third_party/boringssl/crypto/pem/pem_all.c", 
            "third_party/boringssl/crypto/pem/pem_info.c", 
            "third_party/boringssl/crypto/pem/pem_lib.c", 
            "third_party/boringssl/crypto/pem/pem_oth.c", 
            "third_party/boringssl/crypto/pem/pem_pk8.c", 
            "third_party/boringssl/crypto/pem/pem_pkey.c", 
            "third_party/boringssl/crypto/pem/pem_x509.c", 
            "third_party/boringssl/crypto/pem/pem_xaux.c", 
            "third_party/boringssl/crypto/pkcs7/pkcs7.c", 
            "third_party/boringssl/crypto/pkcs7/pkcs7_x509.c", 
            "third_party/boringssl/crypto/pkcs8/p5_pbev2.c", 
            "third_party/boringssl/crypto/pkcs8/pkcs8.c", 
            "third_party/boringssl/crypto/pkcs8/pkcs8_x509.c", 
            "third_party/boringssl/crypto/poly1305/poly1305.c", 
            "third_party/boringssl/crypto/poly1305/poly1305_arm.c", 
            "third_party/boringssl/crypto/poly1305/poly1305_vec.c", 
            "third_party/boringssl/crypto/pool/pool.c", 
            "third_party/boringssl/crypto/rand_extra/deterministic.c", 
            "third_party/boringssl/crypto/rand_extra/forkunsafe.c", 
            "third_party/boringssl/crypto/rand_extra/fuchsia.c", 
            "third_party/boringssl/crypto/rand_extra/rand_extra.c", 
            "third_party/boringssl/crypto/rand_extra/windows.c", 
            "third_party/boringssl/crypto/rc4/rc4.c", 
            "third_party/boringssl/crypto/refcount_c11.c", 
            "third_party/boringssl/crypto/refcount_lock.c", 
            "third_party/boringssl/crypto/rsa_extra/rsa_asn1.c", 
            "third_party/boringssl/crypto/stack/stack.c", 
            "third_party/boringssl/crypto/thread.c", 
            "third_party/boringssl/crypto/thread_none.c", 
            "third_party/boringssl/crypto/thread_pthread.c", 
            "third_party/boringssl/crypto/thread_win.c", 
            "third_party/boringssl/crypto/x509/a_digest.c", 
            "third_party/boringssl/crypto/x509/a_sign.c", 
            "third_party/boringssl/crypto/x509/a_strex.c", 
            "third_party/boringssl/crypto/x509/a_verify.c", 
            "third_party/boringssl/crypto/x509/algorithm.c", 
            "third_party/boringssl/crypto/x509/asn1_gen.c", 
            "third_party/boringssl/crypto/x509/by_dir.c", 
            "third_party/boringssl/crypto/x509/by_file.c", 
            "third_party/boringssl/crypto/x509/i2d_pr.c", 
            "third_party/boringssl/crypto/x509/rsa_pss.c", 
            "third_party/boringssl/crypto/x509/t_crl.c", 
            "third_party/boringssl/crypto/x509/t_req.c", 
            "third_party/boringssl/crypto/x509/t_x509.c", 
            "third_party/boringssl/crypto/x509/t_x509a.c", 
            "third_party/boringssl/crypto/x509/x509.c", 
            "third_party/boringssl/crypto/x509/x509_att.c", 
            "third_party/boringssl/crypto/x509/x509_cmp.c", 
            "third_party/boringssl/crypto/x509/x509_d2.c", 
            "third_party/boringssl/crypto/x509/x509_def.c", 
            "third_party/boringssl/crypto/x509/x509_ext.c", 
            "third_party/boringssl/crypto/x509/x509_lu.c", 
            "third_party/boringssl/crypto/x509/x509_obj.c", 
            "third_party/boringssl/crypto/x509/x509_r2x.c", 
            "third_party/boringssl/crypto/x509/x509_req.c", 
            "third_party/boringssl/crypto/x509/x509_set.c", 
            "third_party/boringssl/crypto/x509/x509_trs.c", 
            "third_party/boringssl/crypto/x509/x509_txt.c", 
            "third_party/boringssl/crypto/x509/x509_v3.c", 
            "third_party/boringssl/crypto/x509/x509_vfy.c", 
            "third_party/boringssl/crypto/x509/x509_vpm.c", 
            "third_party/boringssl/crypto/x509/x509cset.c", 
            "third_party/boringssl/crypto/x509/x509name.c", 
            "third_party/boringssl/crypto/x509/x509rset.c", 
            "third_party/boringssl/crypto/x509/x509spki.c", 
            "third_party/boringssl/crypto/x509/x_algor.c", 
            "third_party/boringssl/crypto/x509/x_all.c", 
            "third_party/boringssl/crypto/x509/x_attrib.c", 
            "third_party/boringssl/crypto/x509/x_crl.c", 
            "third_party/boringssl/crypto/x509/x_exten.c", 
            "third_party/boringssl/crypto/x509/x_info.c", 
            "third_party/boringssl/crypto/x509/x_name.c", 
            "third_party/boringssl/crypto/x509/x_pkey.c", 
            "third_party/boringssl/crypto/x509/x_pubkey.c", 
            "third_party/boringssl/crypto/x509/x_req.c", 
            "third_party/boringssl/crypto/x509/x_sig.c", 
            "third_party/boringssl/crypto/x509/x_spki.c", 
            "third_party/boringssl/crypto/x509/x_val.c", 
            "third_party/boringssl/crypto/x509/x_x509.c", 
            "third_party/boringssl/crypto/x509/x_x509a.c", 
            "third_party/boringssl/crypto/x509v3/pcy_cache.c", 
            "third_party/boringssl/crypto/x509v3/pcy_data.c", 
            "third_party/boringssl/crypto/x509v3/pcy_lib.c", 
            "third_party/boringssl/crypto/x509v3/pcy_map.c", 
            "third_party/boringssl/crypto/x509v3/pcy_node.c", 
            "third_party/boringssl/crypto/x509v3/pcy_tree.c", 
            "third_party/boringssl/crypto/x509v3/v3_akey.c", 
            "third_party/boringssl/crypto/x509v3/v3_akeya.c", 
            "third_party/boringssl/crypto/x509v3/v3_alt.c", 
            "third_party/boringssl/crypto/x509v3/v3_bcons.c", 
            "third_party/boringssl/crypto/x509v3/v3_bitst.c", 
            "third_party/boringssl/crypto/x509v3/v3_conf.c", 
            "third_party/boringssl/crypto/x509v3/v3_cpols.c", 
            "third_party/boringssl/crypto/x509v3/v3_crld.c", 
            "third_party/boringssl/crypto/x509v3/v3_enum.c", 
            "third_party/boringssl/crypto/x509v3/v3_extku.c", 
            "third_party/boringssl/crypto/x509v3/v3_genn.c", 
            "third_party/boringssl/crypto/x509v3/v3_ia5.c", 
            "third_party/boringssl/crypto/x509v3/v3_info.c", 
            "third_party/boringssl/crypto/x509v3/v3_int.c", 
            "third_party/boringssl/crypto/x509v3/v3_lib.c", 
            "third_party/boringssl/crypto/x509v3/v3_ncons.c", 
            "third_party/boringssl/crypto/x509v3/v3_pci.c", 
            "third_party/boringssl/crypto/x509v3/v3_pcia.c", 
            "third_party/boringssl/crypto/x509v3/v3_pcons.c", 
            "third_party/boringssl/crypto/x509v3/v3_pku.c", 
            "third_party/boringssl/crypto/x509v3/v3_pmaps.c", 
            "third_party/boringssl/crypto/x509v3/v3_prn.c", 
            "third_party/boringssl/crypto/x509v3/v3_purp.c", 
            "third_party/boringssl/crypto/x509v3/v3_skey.c", 
            "third_party/boringssl/crypto/x509v3/v3_sxnet.c", 
            "third_party/boringssl/crypto/x509v3/v3_utl.c", 
            "third_party/boringssl/ssl/bio_ssl.cc", 
            "third_party/boringssl/ssl/custom_extensions.cc", 
            "third_party/boringssl/ssl/d1_both.cc", 
            "third_party/boringssl/ssl/d1_lib.cc", 
            "third_party/boringssl/ssl/d1_pkt.cc", 
            "third_party/boringssl/ssl/d1_srtp.cc", 
            "third_party/boringssl/ssl/dtls_method.cc", 
            "third_party/boringssl/ssl/dtls_record.cc", 
            "third_party/boringssl/ssl/handoff.cc", 
            "third_party/boringssl/ssl/handshake.cc", 
            "third_party/boringssl/ssl/handshake_client.cc", 
            "third_party/boringssl/ssl/handshake_server.cc", 
            "third_party/boringssl/ssl/s3_both.cc", 
            "third_party/boringssl/ssl/s3_lib.cc", 
            "third_party/boringssl/ssl/s3_pkt.cc", 
            "third_party/boringssl/ssl/ssl_aead_ctx.cc", 
            "third_party/boringssl/ssl/ssl_asn1.cc", 
            "third_party/boringssl/ssl/ssl_buffer.cc", 
            "third_party/boringssl/ssl/ssl_cert.cc", 
            "third_party/boringssl/ssl/ssl_cipher.cc", 
            "third_party/boringssl/ssl/ssl_file.cc", 
            "third_party/boringssl/ssl/ssl_key_share.cc", 
            "third_party/boringssl/ssl/ssl_lib.cc", 
            "third_party/boringssl/ssl/ssl_privkey.cc", 
            "third_party/boringssl/ssl/ssl_session.cc", 
            "third_party/boringssl/ssl/ssl_stat.cc", 
            "third_party/boringssl/ssl/ssl_transcript.cc", 
            "third_party/boringssl/ssl/ssl_versions.cc", 
            "third_party/boringssl/ssl/ssl_x509.cc", 
            "third_party/boringssl/ssl/t1_enc.cc", 
            "third_party/boringssl/ssl/t1_lib.cc", 
            "third_party/boringssl/ssl/tls13_both.cc", 
            "third_party/boringssl/ssl/tls13_client.cc", 
            "third_party/boringssl/ssl/tls13_enc.cc", 
            "third_party/boringssl/ssl/tls13_server.cc", 
            "third_party/boringssl/ssl/tls_method.cc", 
            "third_party/boringssl/ssl/tls_record.cc", 
            "third_party/boringssl/third_party/fiat/curve25519.c", 
            "third_party/zlib/adler32.c", 
            "third_party/zlib/compress.c", 
            "third_party/zlib/crc32.c", 
            "third_party/zlib/deflate.c", 
            "third_party/zlib/gzclose.c", 
            "third_party/zlib/gzlib.c", 
            "third_party/zlib/gzread.c", 
            "third_party/zlib/gzwrite.c", 
            "third_party/zlib/infback.c", 
            "third_party/zlib/inffast.c", 
            "third_party/zlib/inflate.c", 
            "third_party/zlib/inftrees.c", 
            "third_party/zlib/trees.c", 
            "third_party/zlib/uncompr.c", 
            "third_party/zlib/zutil.c", 
            "third_party/cares/cares/ares__close_sockets.c", 
            "third_party/cares/cares/ares__get_hostent.c", 
            "third_party/cares/cares/ares__read_line.c", 
            "third_party/cares/cares/ares__timeval.c", 
            "third_party/cares/cares/ares_cancel.c", 
            "third_party/cares/cares/ares_create_query.c", 
            "third_party/cares/cares/ares_data.c", 
            "third_party/cares/cares/ares_destroy.c", 
            "third_party/cares/cares/ares_expand_name.c", 
            "third_party/cares/cares/ares_expand_string.c", 
            "third_party/cares/cares/ares_fds.c", 
            "third_party/cares/cares/ares_free_hostent.c", 
            "third_party/cares/cares/ares_free_string.c", 
            "third_party/cares/cares/ares_getenv.c", 
            "third_party/cares/cares/ares_gethostbyaddr.c", 
            "third_party/cares/cares/ares_gethostbyname.c", 
            "third_party/cares/cares/ares_getnameinfo.c", 
            "third_party/cares/cares/ares_getopt.c", 
            "third_party/cares/cares/ares_getsock.c", 
            "third_party/cares/cares/ares_init.c", 
            "third_party/cares/cares/ares_library_init.c", 
            "third_party/cares/cares/ares_llist.c", 
            "third_party/cares/cares/ares_mkquery.c", 
            "third_party/cares/cares/ares_nowarn.c", 
            "third_party/cares/cares/ares_options.c", 
            "third_party/cares/cares/ares_parse_a_reply.c", 
            "third_party/cares/cares/ares_parse_aaaa_reply.c", 
            "third_party/cares/cares/ares_parse_mx_reply.c", 
            "third_party/cares/cares/ares_parse_naptr_reply.c", 
            "third_party/cares/cares/ares_parse_ns_reply.c", 
            "third_party/cares/cares/ares_parse_ptr_reply.c", 
            "third_party/cares/cares/ares_parse_soa_reply.c", 
            "third_party/cares/cares/ares_parse_srv_reply.c", 
            "third_party/cares/cares/ares_parse_txt_reply.c", 
            "third_party/cares/cares/ares_platform.c", 
            "third_party/cares/cares/ares_process.c", 
            "third_party/cares/cares/ares_query.c", 
            "third_party/cares/cares/ares_search.c", 
            "third_party/cares/cares/ares_send.c", 
            "third_party/cares/cares/ares_strcasecmp.c", 
            "third_party/cares/cares/ares_strdup.c", 
            "third_party/cares/cares/ares_strerror.c", 
            "third_party/cares/cares/ares_strsplit.c", 
            "third_party/cares/cares/ares_timeout.c", 
            "third_party/cares/cares/ares_version.c", 
            "third_party/cares/cares/ares_writev.c", 
            "third_party/cares/cares/bitncmp.c", 
            "third_party/cares/cares/inet_net_pton.c", 
            "third_party/cares/cares/inet_ntop.c", 
            "third_party/cares/cares/windows_port.c"
        ]
    }, 
    "module_name": "grpc._cython.cygrpc"
}
END: Cython Metadata */

#define PY_SSIZE_T_CLEAN
#include "Python.h"
#ifndef Py_PYTHON_H
    #error Python headers needed to compile C extensions, please install development version of Python.
#elif PY_VERSION_HEX < 0x02060000 || (0x03000000 <= PY_VERSION_HEX && PY_VERSION_HEX < 0x03030000)
    #error Cython requires Python 2.6+ or Python 3.3+.
#else
#define CYTHON_ABI "0_28_4"
#define CYTHON_FUTURE_DIVISION 0
#include <stddef.h>
#ifndef offsetof
  #define offsetof(type, member) ( (size_t) & ((type*)0) -> member )
#endif
#if !defined(WIN32) && !defined(MS_WINDOWS)
  #ifndef __stdcall
    #define __stdcall
  #endif
  #ifndef __cdecl
    #define __cdecl
  #endif
  #ifndef __fastcall
    #define __fastcall
  #endif
#endif
#ifndef DL_IMPORT
  #define DL_IMPORT(t) t
#endif
#ifndef DL_EXPORT
  #define DL_EXPORT(t) t
#endif
#define __PYX_COMMA ,
#ifndef HAVE_LONG_LONG
  #if PY_VERSION_HEX >= 0x02070000
    #define HAVE_LONG_LONG
  #endif
#endif
#ifndef PY_LONG_LONG
  #define PY_LONG_LONG LONG_LONG
#endif
#ifndef Py_HUGE_VAL
  #define Py_HUGE_VAL HUGE_VAL
#endif
#ifdef PYPY_VERSION
  #define CYTHON_COMPILING_IN_PYPY 1
  #define CYTHON_COMPILING_IN_PYSTON 0
  #define CYTHON_COMPILING_IN_CPYTHON 0
  #undef CYTHON_USE_TYPE_SLOTS
  #define CYTHON_USE_TYPE_SLOTS 0
  #undef CYTHON_USE_PYTYPE_LOOKUP
  #define CYTHON_USE_PYTYPE_LOOKUP 0
  #if PY_VERSION_HEX < 0x03050000
    #undef CYTHON_USE_ASYNC_SLOTS
    #define CYTHON_USE_ASYNC_SLOTS 0
  #elif !defined(CYTHON_USE_ASYNC_SLOTS)
    #define CYTHON_USE_ASYNC_SLOTS 1
  #endif
  #undef CYTHON_USE_PYLIST_INTERNALS
  #define CYTHON_USE_PYLIST_INTERNALS 0
  #undef CYTHON_USE_UNICODE_INTERNALS
  #define CYTHON_USE_UNICODE_INTERNALS 0
  #undef CYTHON_USE_UNICODE_WRITER
  #define CYTHON_USE_UNICODE_WRITER 0
  #undef CYTHON_USE_PYLONG_INTERNALS
  #define CYTHON_USE_PYLONG_INTERNALS 0
  #undef CYTHON_AVOID_BORROWED_REFS
  #define CYTHON_AVOID_BORROWED_REFS 1
  #undef CYTHON_ASSUME_SAFE_MACROS
  #define CYTHON_ASSUME_SAFE_MACROS 0
  #undef CYTHON_UNPACK_METHODS
  #define CYTHON_UNPACK_METHODS 0
  #undef CYTHON_FAST_THREAD_STATE
  #define CYTHON_FAST_THREAD_STATE 0
  #undef CYTHON_FAST_PYCALL
  #define CYTHON_FAST_PYCALL 0
  #undef CYTHON_PEP489_MULTI_PHASE_INIT
  #define CYTHON_PEP489_MULTI_PHASE_INIT 0
  #undef CYTHON_USE_TP_FINALIZE
  #define CYTHON_USE_TP_FINALIZE 0
#elif defined(PYSTON_VERSION)
  #define CYTHON_COMPILING_IN_PYPY 0
  #define CYTHON_COMPILING_IN_PYSTON 1
  #define CYTHON_COMPILING_IN_CPYTHON 0
  #ifndef CYTHON_USE_TYPE_SLOTS
    #define CYTHON_USE_TYPE_SLOTS 1
  #endif
  #undef CYTHON_USE_PYTYPE_LOOKUP
  #define CYTHON_USE_PYTYPE_LOOKUP 0
  #undef CYTHON_USE_ASYNC_SLOTS
  #define CYTHON_USE_ASYNC_SLOTS 0
  #undef CYTHON_USE_PYLIST_INTERNALS
  #define CYTHON_USE_PYLIST_INTERNALS 0
  #ifndef CYTHON_USE_UNICODE_INTERNALS
    #define CYTHON_USE_UNICODE_INTERNALS 1
  #endif
  #undef CYTHON_USE_UNICODE_WRITER
  #define CYTHON_USE_UNICODE_WRITER 0
  #undef CYTHON_USE_PYLONG_INTERNALS
  #define CYTHON_USE_PYLONG_INTERNALS 0
  #ifndef CYTHON_AVOID_BORROWED_REFS
    #define CYTHON_AVOID_BORROWED_REFS 0
  #endif
  #ifndef CYTHON_ASSUME_SAFE_MACROS
    #define CYTHON_ASSUME_SAFE_MACROS 1
  #endif
  #ifndef CYTHON_UNPACK_METHODS
    #define CYTHON_UNPACK_METHODS 1
  #endif
  #undef CYTHON_FAST_THREAD_STATE
  #define CYTHON_FAST_THREAD_STATE 0
  #undef CYTHON_FAST_PYCALL
  #define CYTHON_FAST_PYCALL 0
  #undef CYTHON_PEP489_MULTI_PHASE_INIT
  #define CYTHON_PEP489_MULTI_PHASE_INIT 0
  #undef CYTHON_USE_TP_FINALIZE
  #define CYTHON_USE_TP_FINALIZE 0
#else
  #define CYTHON_COMPILING_IN_PYPY 0
  #define CYTHON_COMPILING_IN_PYSTON 0
  #define CYTHON_COMPILING_IN_CPYTHON 1
  #ifndef CYTHON_USE_TYPE_SLOTS
    #define CYTHON_USE_TYPE_SLOTS 1
  #endif
  #if PY_VERSION_HEX < 0x02070000
    #undef CYTHON_USE_PYTYPE_LOOKUP
    #define CYTHON_USE_PYTYPE_LOOKUP 0
  #elif !defined(CYTHON_USE_PYTYPE_LOOKUP)
    #define CYTHON_USE_PYTYPE_LOOKUP 1
  #endif
  #if PY_MAJOR_VERSION < 3
    #undef CYTHON_USE_ASYNC_SLOTS
    #define CYTHON_USE_ASYNC_SLOTS 0
  #elif !defined(CYTHON_USE_ASYNC_SLOTS)
    #define CYTHON_USE_ASYNC_SLOTS 1
  #endif
  #if PY_VERSION_HEX < 0x02070000
    #undef CYTHON_USE_PYLONG_INTERNALS
    #define CYTHON_USE_PYLONG_INTERNALS 0
  #elif !defined(CYTHON_USE_PYLONG_INTERNALS)
    #define CYTHON_USE_PYLONG_INTERNALS 1
  #endif
  #ifndef CYTHON_USE_PYLIST_INTERNALS
    #define CYTHON_USE_PYLIST_INTERNALS 1
  #endif
  #ifndef CYTHON_USE_UNICODE_INTERNALS
    #define CYTHON_USE_UNICODE_INTERNALS 1
  #endif
  #if PY_VERSION_HEX < 0x030300F0
    #undef CYTHON_USE_UNICODE_WRITER
    #define CYTHON_USE_UNICODE_WRITER 0
  #elif !defined(CYTHON_USE_UNICODE_WRITER)
    #define CYTHON_USE_UNICODE_WRITER 1
  #endif
  #ifndef CYTHON_AVOID_BORROWED_REFS
    #define CYTHON_AVOID_BORROWED_REFS 0
  #endif
  #ifndef CYTHON_ASSUME_SAFE_MACROS
    #define CYTHON_ASSUME_SAFE_MACROS 1
  #endif
  #ifndef CYTHON_UNPACK_METHODS
    #define CYTHON_UNPACK_METHODS 1
  #endif
  #ifndef CYTHON_FAST_THREAD_STATE
    #define CYTHON_FAST_THREAD_STATE 1
  #endif
  #ifndef CYTHON_FAST_PYCALL
    #define CYTHON_FAST_PYCALL 1
  #endif
  #ifndef CYTHON_PEP489_MULTI_PHASE_INIT
    #define CYTHON_PEP489_MULTI_PHASE_INIT (0 && PY_VERSION_HEX >= 0x03050000)
  #endif
  #ifndef CYTHON_USE_TP_FINALIZE
    #define CYTHON_USE_TP_FINALIZE (PY_VERSION_HEX >= 0x030400a1)
  #endif
#endif
#if !defined(CYTHON_FAST_PYCCALL)
#define CYTHON_FAST_PYCCALL  (CYTHON_FAST_PYCALL && PY_VERSION_HEX >= 0x030600B1)
#endif
#if CYTHON_USE_PYLONG_INTERNALS
  #include "longintrepr.h"
  #undef SHIFT
  #undef BASE
  #undef MASK
#endif
#ifndef __has_attribute
  #define __has_attribute(x) 0
#endif
#ifndef __has_cpp_attribute
  #define __has_cpp_attribute(x) 0
#endif
#ifndef CYTHON_RESTRICT
  #if defined(__GNUC__)
    #define CYTHON_RESTRICT __restrict__
  #elif defined(_MSC_VER) && _MSC_VER >= 1400
    #define CYTHON_RESTRICT __restrict
  #elif defined (__STDC_VERSION__) && __STDC_VERSION__ >= 199901L
    #define CYTHON_RESTRICT restrict
  #else
    #define CYTHON_RESTRICT
  #endif
#endif
#ifndef CYTHON_UNUSED
# if defined(__GNUC__)
#   if !(defined(__cplusplus)) || (__GNUC__ > 3 || (__GNUC__ == 3 && __GNUC_MINOR__ >= 4))
#     define CYTHON_UNUSED __attribute__ ((__unused__))
#   else
#     define CYTHON_UNUSED
#   endif
# elif defined(__ICC) || (defined(__INTEL_COMPILER) && !defined(_MSC_VER))
#   define CYTHON_UNUSED __attribute__ ((__unused__))
# else
#   define CYTHON_UNUSED
# endif
#endif
#ifndef CYTHON_MAYBE_UNUSED_VAR
#  if defined(__cplusplus)
     template<class T> void CYTHON_MAYBE_UNUSED_VAR( const T& ) { }
#  else
#    define CYTHON_MAYBE_UNUSED_VAR(x) (void)(x)
#  endif
#endif
#ifndef CYTHON_NCP_UNUSED
# if CYTHON_COMPILING_IN_CPYTHON
#  define CYTHON_NCP_UNUSED
# else
#  define CYTHON_NCP_UNUSED CYTHON_UNUSED
# endif
#endif
#define __Pyx_void_to_None(void_result) ((void)(void_result), Py_INCREF(Py_None), Py_None)
#ifdef _MSC_VER
    #ifndef _MSC_STDINT_H_
        #if _MSC_VER < 1300
           typedef unsigned char     uint8_t;
           typedef unsigned int      uint32_t;
        #else
           typedef unsigned __int8   uint8_t;
           typedef unsigned __int32  uint32_t;
        #endif
    #endif
#else
   #include <stdint.h>
#endif
#ifndef CYTHON_FALLTHROUGH
  #if defined(__cplusplus) && __cplusplus >= 201103L
    #if __has_cpp_attribute(fallthrough)
      #define CYTHON_FALLTHROUGH [[fallthrough]]
    #elif __has_cpp_attribute(clang::fallthrough)
      #define CYTHON_FALLTHROUGH [[clang::fallthrough]]
    #elif __has_cpp_attribute(gnu::fallthrough)
      #define CYTHON_FALLTHROUGH [[gnu::fallthrough]]
    #endif
  #endif
  #ifndef CYTHON_FALLTHROUGH
    #if __has_attribute(fallthrough)
      #define CYTHON_FALLTHROUGH __attribute__((fallthrough))
    #else
      #define CYTHON_FALLTHROUGH
    #endif
  #endif
  #if defined(__clang__ ) && defined(__apple_build_version__)
    #if __apple_build_version__ < 7000000
      #undef  CYTHON_FALLTHROUGH
      #define CYTHON_FALLTHROUGH
    #endif
  #endif
#endif

#ifndef __cplusplus
  #error "Cython files generated with the C++ option must be compiled with a C++ compiler."
#endif
#ifndef CYTHON_INLINE
  #if defined(__clang__)
    #define CYTHON_INLINE __inline__ __attribute__ ((__unused__))
  #else
    #define CYTHON_INLINE inline
  #endif
#endif
template<typename T>
void __Pyx_call_destructor(T& x) {
    x.~T();
}
template<typename T>
class __Pyx_FakeReference {
  public:
    __Pyx_FakeReference() : ptr(NULL) { }
    __Pyx_FakeReference(const T& ref) : ptr(const_cast<T*>(&ref)) { }
    T *operator->() { return ptr; }
    T *operator&() { return ptr; }
    operator T&() { return *ptr; }
    template<typename U> bool operator ==(U other) { return *ptr == other; }
    template<typename U> bool operator !=(U other) { return *ptr != other; }
  private:
    T *ptr;
};

#if CYTHON_COMPILING_IN_PYPY && PY_VERSION_HEX < 0x02070600 && !defined(Py_OptimizeFlag)
  #define Py_OptimizeFlag 0
#endif
#define __PYX_BUILD_PY_SSIZE_T "n"
#define CYTHON_FORMAT_SSIZE_T "z"
#if PY_MAJOR_VERSION < 3
  #define __Pyx_BUILTIN_MODULE_NAME "__builtin__"
  #define __Pyx_PyCode_New(a, k, l, s, f, code, c, n, v, fv, cell, fn, name, fline, lnos)\
          PyCode_New(a+k, l, s, f, code, c, n, v, fv, cell, fn, name, fline, lnos)
  #define __Pyx_DefaultClassType PyClass_Type
#else
  #define __Pyx_BUILTIN_MODULE_NAME "builtins"
  #define __Pyx_PyCode_New(a, k, l, s, f, code, c, n, v, fv, cell, fn, name, fline, lnos)\
          PyCode_New(a, k, l, s, f, code, c, n, v, fv, cell, fn, name, fline, lnos)
  #define __Pyx_DefaultClassType PyType_Type
#endif
#ifndef Py_TPFLAGS_CHECKTYPES
  #define Py_TPFLAGS_CHECKTYPES 0
#endif
#ifndef Py_TPFLAGS_HAVE_INDEX
  #define Py_TPFLAGS_HAVE_INDEX 0
#endif
#ifndef Py_TPFLAGS_HAVE_NEWBUFFER
  #define Py_TPFLAGS_HAVE_NEWBUFFER 0
#endif
#ifndef Py_TPFLAGS_HAVE_FINALIZE
  #define Py_TPFLAGS_HAVE_FINALIZE 0
#endif
#if PY_VERSION_HEX <= 0x030700A3 || !defined(METH_FASTCALL)
  #ifndef METH_FASTCALL
     #define METH_FASTCALL 0x80
  #endif
  typedef PyObject *(*__Pyx_PyCFunctionFast) (PyObject *self, PyObject *const *args, Py_ssize_t nargs);
  typedef PyObject *(*__Pyx_PyCFunctionFastWithKeywords) (PyObject *self, PyObject *const *args,
                                                          Py_ssize_t nargs, PyObject *kwnames);
#else
  #define __Pyx_PyCFunctionFast _PyCFunctionFast
  #define __Pyx_PyCFunctionFastWithKeywords _PyCFunctionFastWithKeywords
#endif
#if CYTHON_FAST_PYCCALL
#define __Pyx_PyFastCFunction_Check(func)\
    ((PyCFunction_Check(func) && (METH_FASTCALL == (PyCFunction_GET_FLAGS(func) & ~(METH_CLASS | METH_STATIC | METH_COEXIST | METH_KEYWORDS)))))
#else
#define __Pyx_PyFastCFunction_Check(func) 0
#endif
#if CYTHON_COMPILING_IN_PYPY && !defined(PyObject_Malloc)
  #define PyObject_Malloc(s)   PyMem_Malloc(s)
  #define PyObject_Free(p)     PyMem_Free(p)
  #define PyObject_Realloc(p)  PyMem_Realloc(p)
#endif
#if CYTHON_COMPILING_IN_PYSTON
  #define __Pyx_PyCode_HasFreeVars(co)  PyCode_HasFreeVars(co)
  #define __Pyx_PyFrame_SetLineNumber(frame, lineno) PyFrame_SetLineNumber(frame, lineno)
#else
  #define __Pyx_PyCode_HasFreeVars(co)  (PyCode_GetNumFree(co) > 0)
  #define __Pyx_PyFrame_SetLineNumber(frame, lineno)  (frame)->f_lineno = (lineno)
#endif
#if !CYTHON_FAST_THREAD_STATE || PY_VERSION_HEX < 0x02070000
  #define __Pyx_PyThreadState_Current PyThreadState_GET()
#elif PY_VERSION_HEX >= 0x03060000
  #define __Pyx_PyThreadState_Current _PyThreadState_UncheckedGet()
#elif PY_VERSION_HEX >= 0x03000000
  #define __Pyx_PyThreadState_Current PyThreadState_GET()
#else
  #define __Pyx_PyThreadState_Current _PyThreadState_Current
#endif
#if PY_VERSION_HEX < 0x030700A2 && !defined(PyThread_tss_create) && !defined(Py_tss_NEEDS_INIT)
#include "pythread.h"
#define Py_tss_NEEDS_INIT 0
typedef int Py_tss_t;
static CYTHON_INLINE int PyThread_tss_create(Py_tss_t *key) {
  *key = PyThread_create_key();
  return 0; // PyThread_create_key reports success always
}
static CYTHON_INLINE Py_tss_t * PyThread_tss_alloc(void) {
  Py_tss_t *key = (Py_tss_t *)PyObject_Malloc(sizeof(Py_tss_t));
  *key = Py_tss_NEEDS_INIT;
  return key;
}
static CYTHON_INLINE void PyThread_tss_free(Py_tss_t *key) {
  PyObject_Free(key);
}
static CYTHON_INLINE int PyThread_tss_is_created(Py_tss_t *key) {
  return *key != Py_tss_NEEDS_INIT;
}
static CYTHON_INLINE void PyThread_tss_delete(Py_tss_t *key) {
  PyThread_delete_key(*key);
  *key = Py_tss_NEEDS_INIT;
}
static CYTHON_INLINE int PyThread_tss_set(Py_tss_t *key, void *value) {
  return PyThread_set_key_value(*key, value);
}
static CYTHON_INLINE void * PyThread_tss_get(Py_tss_t *key) {
  return PyThread_get_key_value(*key);
}
#endif // TSS (Thread Specific Storage) API
#if CYTHON_COMPILING_IN_CPYTHON || defined(_PyDict_NewPresized)
#define __Pyx_PyDict_NewPresized(n)  ((n <= 8) ? PyDict_New() : _PyDict_NewPresized(n))
#else
#define __Pyx_PyDict_NewPresized(n)  PyDict_New()
#endif
#if PY_MAJOR_VERSION >= 3 || CYTHON_FUTURE_DIVISION
  #define __Pyx_PyNumber_Divide(x,y)         PyNumber_TrueDivide(x,y)
  #define __Pyx_PyNumber_InPlaceDivide(x,y)  PyNumber_InPlaceTrueDivide(x,y)
#else
  #define __Pyx_PyNumber_Divide(x,y)         PyNumber_Divide(x,y)
  #define __Pyx_PyNumber_InPlaceDivide(x,y)  PyNumber_InPlaceDivide(x,y)
#endif
#if CYTHON_COMPILING_IN_CPYTHON && PY_VERSION_HEX >= 0x030500A1 && CYTHON_USE_UNICODE_INTERNALS
#define __Pyx_PyDict_GetItemStr(dict, name)  _PyDict_GetItem_KnownHash(dict, name, ((PyASCIIObject *) name)->hash)
#else
#define __Pyx_PyDict_GetItemStr(dict, name)  PyDict_GetItem(dict, name)
#endif
#if PY_VERSION_HEX > 0x03030000 && defined(PyUnicode_KIND)
  #define CYTHON_PEP393_ENABLED 1
  #define __Pyx_PyUnicode_READY(op)       (likely(PyUnicode_IS_READY(op)) ?\
                                              0 : _PyUnicode_Ready((PyObject *)(op)))
  #define __Pyx_PyUnicode_GET_LENGTH(u)   PyUnicode_GET_LENGTH(u)
  #define __Pyx_PyUnicode_READ_CHAR(u, i) PyUnicode_READ_CHAR(u, i)
  #define __Pyx_PyUnicode_MAX_CHAR_VALUE(u)   PyUnicode_MAX_CHAR_VALUE(u)
  #define __Pyx_PyUnicode_KIND(u)         PyUnicode_KIND(u)
  #define __Pyx_PyUnicode_DATA(u)         PyUnicode_DATA(u)
  #define __Pyx_PyUnicode_READ(k, d, i)   PyUnicode_READ(k, d, i)
  #define __Pyx_PyUnicode_WRITE(k, d, i, ch)  PyUnicode_WRITE(k, d, i, ch)
  #define __Pyx_PyUnicode_IS_TRUE(u)      (0 != (likely(PyUnicode_IS_READY(u)) ? PyUnicode_GET_LENGTH(u) : PyUnicode_GET_SIZE(u)))
#else
  #define CYTHON_PEP393_ENABLED 0
  #define PyUnicode_1BYTE_KIND  1
  #define PyUnicode_2BYTE_KIND  2
  #define PyUnicode_4BYTE_KIND  4
  #define __Pyx_PyUnicode_READY(op)       (0)
  #define __Pyx_PyUnicode_GET_LENGTH(u)   PyUnicode_GET_SIZE(u)
  #define __Pyx_PyUnicode_READ_CHAR(u, i) ((Py_UCS4)(PyUnicode_AS_UNICODE(u)[i]))
  #define __Pyx_PyUnicode_MAX_CHAR_VALUE(u)   ((sizeof(Py_UNICODE) == 2) ? 65535 : 1114111)
  #define __Pyx_PyUnicode_KIND(u)         (sizeof(Py_UNICODE))
  #define __Pyx_PyUnicode_DATA(u)         ((void*)PyUnicode_AS_UNICODE(u))
  #define __Pyx_PyUnicode_READ(k, d, i)   ((void)(k), (Py_UCS4)(((Py_UNICODE*)d)[i]))
  #define __Pyx_PyUnicode_WRITE(k, d, i, ch)  (((void)(k)), ((Py_UNICODE*)d)[i] = ch)
  #define __Pyx_PyUnicode_IS_TRUE(u)      (0 != PyUnicode_GET_SIZE(u))
#endif
#if CYTHON_COMPILING_IN_PYPY
  #define __Pyx_PyUnicode_Concat(a, b)      PyNumber_Add(a, b)
  #define __Pyx_PyUnicode_ConcatSafe(a, b)  PyNumber_Add(a, b)
#else
  #define __Pyx_PyUnicode_Concat(a, b)      PyUnicode_Concat(a, b)
  #define __Pyx_PyUnicode_ConcatSafe(a, b)  ((unlikely((a) == Py_None) || unlikely((b) == Py_None)) ?\
      PyNumber_Add(a, b) : __Pyx_PyUnicode_Concat(a, b))
#endif
#if CYTHON_COMPILING_IN_PYPY && !defined(PyUnicode_Contains)
  #define PyUnicode_Contains(u, s)  PySequence_Contains(u, s)
#endif
#if CYTHON_COMPILING_IN_PYPY && !defined(PyByteArray_Check)
  #define PyByteArray_Check(obj)  PyObject_TypeCheck(obj, &PyByteArray_Type)
#endif
#if CYTHON_COMPILING_IN_PYPY && !defined(PyObject_Format)
  #define PyObject_Format(obj, fmt)  PyObject_CallMethod(obj, "__format__", "O", fmt)
#endif
#define __Pyx_PyString_FormatSafe(a, b)   ((unlikely((a) == Py_None)) ? PyNumber_Remainder(a, b) : __Pyx_PyString_Format(a, b))
#define __Pyx_PyUnicode_FormatSafe(a, b)  ((unlikely((a) == Py_None)) ? PyNumber_Remainder(a, b) : PyUnicode_Format(a, b))
#if PY_MAJOR_VERSION >= 3
  #define __Pyx_PyString_Format(a, b)  PyUnicode_Format(a, b)
#else
  #define __Pyx_PyString_Format(a, b)  PyString_Format(a, b)
#endif
#if PY_MAJOR_VERSION < 3 && !defined(PyObject_ASCII)
  #define PyObject_ASCII(o)            PyObject_Repr(o)
#endif
#if PY_MAJOR_VERSION >= 3
  #define PyBaseString_Type            PyUnicode_Type
  #define PyStringObject               PyUnicodeObject
  #define PyString_Type                PyUnicode_Type
  #define PyString_Check               PyUnicode_Check
  #define PyString_CheckExact          PyUnicode_CheckExact
  #define PyObject_Unicode             PyObject_Str
#endif
#if PY_MAJOR_VERSION >= 3
  #define __Pyx_PyBaseString_Check(obj) PyUnicode_Check(obj)
  #define __Pyx_PyBaseString_CheckExact(obj) PyUnicode_CheckExact(obj)
#else
  #define __Pyx_PyBaseString_Check(obj) (PyString_Check(obj) || PyUnicode_Check(obj))
  #define __Pyx_PyBaseString_CheckExact(obj) (PyString_CheckExact(obj) || PyUnicode_CheckExact(obj))
#endif
#ifndef PySet_CheckExact
  #define PySet_CheckExact(obj)        (Py_TYPE(obj) == &PySet_Type)
#endif
#if CYTHON_ASSUME_SAFE_MACROS
  #define __Pyx_PySequence_SIZE(seq)  Py_SIZE(seq)
#else
  #define __Pyx_PySequence_SIZE(seq)  PySequence_Size(seq)
#endif
#if PY_MAJOR_VERSION >= 3
  #define PyIntObject                  PyLongObject
  #define PyInt_Type                   PyLong_Type
  #define PyInt_Check(op)              PyLong_Check(op)
  #define PyInt_CheckExact(op)         PyLong_CheckExact(op)
  #define PyInt_FromString             PyLong_FromString
  #define PyInt_FromUnicode            PyLong_FromUnicode
  #define PyInt_FromLong               PyLong_FromLong
  #define PyInt_FromSize_t             PyLong_FromSize_t
  #define PyInt_FromSsize_t            PyLong_FromSsize_t
  #define PyInt_AsLong                 PyLong_AsLong
  #define PyInt_AS_LONG                PyLong_AS_LONG
  #define PyInt_AsSsize_t              PyLong_AsSsize_t
  #define PyInt_AsUnsignedLongMask     PyLong_AsUnsignedLongMask
  #define PyInt_AsUnsignedLongLongMask PyLong_AsUnsignedLongLongMask
  #define PyNumber_Int                 PyNumber_Long
#endif
#if PY_MAJOR_VERSION >= 3
  #define PyBoolObject                 PyLongObject
#endif
#if PY_MAJOR_VERSION >= 3 && CYTHON_COMPILING_IN_PYPY
  #ifndef PyUnicode_InternFromString
    #define PyUnicode_InternFromString(s) PyUnicode_FromString(s)
  #endif
#endif
#if PY_VERSION_HEX < 0x030200A4
  typedef long Py_hash_t;
  #define __Pyx_PyInt_FromHash_t PyInt_FromLong
  #define __Pyx_PyInt_AsHash_t   PyInt_AsLong
#else
  #define __Pyx_PyInt_FromHash_t PyInt_FromSsize_t
  #define __Pyx_PyInt_AsHash_t   PyInt_AsSsize_t
#endif
#if PY_MAJOR_VERSION >= 3
  #define __Pyx_PyMethod_New(func, self, klass) ((self) ? PyMethod_New(func, self) : (Py_INCREF(func), func))
#else
  #define __Pyx_PyMethod_New(func, self, klass) PyMethod_New(func, self, klass)
#endif
#if CYTHON_USE_ASYNC_SLOTS
  #if PY_VERSION_HEX >= 0x030500B1
    #define __Pyx_PyAsyncMethodsStruct PyAsyncMethods
    #define __Pyx_PyType_AsAsync(obj) (Py_TYPE(obj)->tp_as_async)
  #else
    #define __Pyx_PyType_AsAsync(obj) ((__Pyx_PyAsyncMethodsStruct*) (Py_TYPE(obj)->tp_reserved))
  #endif
#else
  #define __Pyx_PyType_AsAsync(obj) NULL
#endif
#ifndef __Pyx_PyAsyncMethodsStruct
    typedef struct {
        unaryfunc am_await;
        unaryfunc am_aiter;
        unaryfunc am_anext;
    } __Pyx_PyAsyncMethodsStruct;
#endif

#if defined(WIN32) || defined(MS_WINDOWS)
  #define _USE_MATH_DEFINES
#endif
#include <math.h>
#ifdef NAN
#define __PYX_NAN() ((float) NAN)
#else
static CYTHON_INLINE float __PYX_NAN() {
  float value;
  memset(&value, 0xFF, sizeof(value));
  return value;
}
#endif
#if defined(__CYGWIN__) && defined(_LDBL_EQ_DBL)
#define __Pyx_truncl trunc
#else
#define __Pyx_truncl truncl
#endif


#define __PYX_ERR(f_index, lineno, Ln_error) \
{ \
  __pyx_filename = __pyx_f[f_index]; __pyx_lineno = lineno; __pyx_clineno = __LINE__; goto Ln_error; \
}

#ifndef __PYX_EXTERN_C
  #ifdef __cplusplus
    #define __PYX_EXTERN_C extern "C"
  #else
    #define __PYX_EXTERN_C extern
  #endif
#endif

#define __PYX_HAVE__grpc___cython__cygrpc
#define __PYX_HAVE_API__grpc___cython__cygrpc
/* Early includes */
#include <stddef.h>
#include <time.h>
#include "grpc/support/alloc.h"
#include "grpc/byte_buffer_reader.h"
#include "grpc/grpc.h"
#include "grpc/grpc_security.h"
#include "grpc/compression.h"
#include "grpc/impl/codegen/compression_types.h"
#include "grpc/impl/codegen/propagation_bits.h"
#include "grpc/impl/codegen/slice.h"
#include "src/core/lib/iomgr/error.h"
#include "src/core/lib/iomgr/gevent_util.h"
#include "src/core/lib/iomgr/sockaddr.h"
#include "src/core/lib/iomgr/resolve_address.h"
#include "src/core/lib/iomgr/resolve_address_custom.h"
#include "src/core/lib/iomgr/tcp_custom.h"
#include "src/core/lib/iomgr/timer_custom.h"
#include "src/core/lib/iomgr/pollset_custom.h"
#include "src/core/lib/iomgr/iomgr_custom.h"
#include "src/core/lib/iomgr/sockaddr_utils.h"
#include "pthread.h"
#include <string.h>
#include <stdio.h>
#include "pythread.h"
#include <stdlib.h>
#ifdef _OPENMP
#include <omp.h>
#endif /* _OPENMP */

#if defined(PYREX_WITHOUT_ASSERTIONS) && !defined(CYTHON_WITHOUT_ASSERTIONS)
#define CYTHON_WITHOUT_ASSERTIONS
#endif

typedef struct {PyObject **p; const char *s; const Py_ssize_t n; const char* encoding;
                const char is_unicode; const char is_str; const char intern; } __Pyx_StringTabEntry;

#define __PYX_DEFAULT_STRING_ENCODING_IS_ASCII 0
#define __PYX_DEFAULT_STRING_ENCODING_IS_DEFAULT 0
#define __PYX_DEFAULT_STRING_ENCODING ""
#define __Pyx_PyObject_FromString __Pyx_PyBytes_FromString
#define __Pyx_PyObject_FromStringAndSize __Pyx_PyBytes_FromStringAndSize
#define __Pyx_uchar_cast(c) ((unsigned char)c)
#define __Pyx_long_cast(x) ((long)x)
#define __Pyx_fits_Py_ssize_t(v, type, is_signed)  (\
    (sizeof(type) < sizeof(Py_ssize_t))  ||\
    (sizeof(type) > sizeof(Py_ssize_t) &&\
          likely(v < (type)PY_SSIZE_T_MAX ||\
                 v == (type)PY_SSIZE_T_MAX)  &&\
          (!is_signed || likely(v > (type)PY_SSIZE_T_MIN ||\
                                v == (type)PY_SSIZE_T_MIN)))  ||\
    (sizeof(type) == sizeof(Py_ssize_t) &&\
          (is_signed || likely(v < (type)PY_SSIZE_T_MAX ||\
                               v == (type)PY_SSIZE_T_MAX)))  )
#if defined (__cplusplus) && __cplusplus >= 201103L
    #include <cstdlib>
    #define __Pyx_sst_abs(value) std::abs(value)
#elif SIZEOF_INT >= SIZEOF_SIZE_T
    #define __Pyx_sst_abs(value) abs(value)
#elif SIZEOF_LONG >= SIZEOF_SIZE_T
    #define __Pyx_sst_abs(value) labs(value)
#elif defined (_MSC_VER)
    #define __Pyx_sst_abs(value) ((Py_ssize_t)_abs64(value))
#elif defined (__STDC_VERSION__) && __STDC_VERSION__ >= 199901L
    #define __Pyx_sst_abs(value) llabs(value)
#elif defined (__GNUC__)
    #define __Pyx_sst_abs(value) __builtin_llabs(value)
#else
    #define __Pyx_sst_abs(value) ((value<0) ? -value : value)
#endif
static CYTHON_INLINE const char* __Pyx_PyObject_AsString(PyObject*);
static CYTHON_INLINE const char* __Pyx_PyObject_AsStringAndSize(PyObject*, Py_ssize_t* length);
#define __Pyx_PyByteArray_FromString(s) PyByteArray_FromStringAndSize((const char*)s, strlen((const char*)s))
#define __Pyx_PyByteArray_FromStringAndSize(s, l) PyByteArray_FromStringAndSize((const char*)s, l)
#define __Pyx_PyBytes_FromString        PyBytes_FromString
#define __Pyx_PyBytes_FromStringAndSize PyBytes_FromStringAndSize
static CYTHON_INLINE PyObject* __Pyx_PyUnicode_FromString(const char*);
#if PY_MAJOR_VERSION < 3
    #define __Pyx_PyStr_FromString        __Pyx_PyBytes_FromString
    #define __Pyx_PyStr_FromStringAndSize __Pyx_PyBytes_FromStringAndSize
#else
    #define __Pyx_PyStr_FromString        __Pyx_PyUnicode_FromString
    #define __Pyx_PyStr_FromStringAndSize __Pyx_PyUnicode_FromStringAndSize
#endif
#define __Pyx_PyBytes_AsWritableString(s)     ((char*) PyBytes_AS_STRING(s))
#define __Pyx_PyBytes_AsWritableSString(s)    ((signed char*) PyBytes_AS_STRING(s))
#define __Pyx_PyBytes_AsWritableUString(s)    ((unsigned char*) PyBytes_AS_STRING(s))
#define __Pyx_PyBytes_AsString(s)     ((const char*) PyBytes_AS_STRING(s))
#define __Pyx_PyBytes_AsSString(s)    ((const signed char*) PyBytes_AS_STRING(s))
#define __Pyx_PyBytes_AsUString(s)    ((const unsigned char*) PyBytes_AS_STRING(s))
#define __Pyx_PyObject_AsWritableString(s)    ((char*) __Pyx_PyObject_AsString(s))
#define __Pyx_PyObject_AsWritableSString(s)    ((signed char*) __Pyx_PyObject_AsString(s))
#define __Pyx_PyObject_AsWritableUString(s)    ((unsigned char*) __Pyx_PyObject_AsString(s))
#define __Pyx_PyObject_AsSString(s)    ((const signed char*) __Pyx_PyObject_AsString(s))
#define __Pyx_PyObject_AsUString(s)    ((const unsigned char*) __Pyx_PyObject_AsString(s))
#define __Pyx_PyObject_FromCString(s)  __Pyx_PyObject_FromString((const char*)s)
#define __Pyx_PyBytes_FromCString(s)   __Pyx_PyBytes_FromString((const char*)s)
#define __Pyx_PyByteArray_FromCString(s)   __Pyx_PyByteArray_FromString((const char*)s)
#define __Pyx_PyStr_FromCString(s)     __Pyx_PyStr_FromString((const char*)s)
#define __Pyx_PyUnicode_FromCString(s) __Pyx_PyUnicode_FromString((const char*)s)
static CYTHON_INLINE size_t __Pyx_Py_UNICODE_strlen(const Py_UNICODE *u) {
    const Py_UNICODE *u_end = u;
    while (*u_end++) ;
    return (size_t)(u_end - u - 1);
}
#define __Pyx_PyUnicode_FromUnicode(u)       PyUnicode_FromUnicode(u, __Pyx_Py_UNICODE_strlen(u))
#define __Pyx_PyUnicode_FromUnicodeAndLength PyUnicode_FromUnicode
#define __Pyx_PyUnicode_AsUnicode            PyUnicode_AsUnicode
#define __Pyx_NewRef(obj) (Py_INCREF(obj), obj)
#define __Pyx_Owned_Py_None(b) __Pyx_NewRef(Py_None)
static CYTHON_INLINE PyObject * __Pyx_PyBool_FromLong(long b);
static CYTHON_INLINE int __Pyx_PyObject_IsTrue(PyObject*);
static CYTHON_INLINE PyObject* __Pyx_PyNumber_IntOrLong(PyObject* x);
#define __Pyx_PySequence_Tuple(obj)\
    (likely(PyTuple_CheckExact(obj)) ? __Pyx_NewRef(obj) : PySequence_Tuple(obj))
static CYTHON_INLINE Py_ssize_t __Pyx_PyIndex_AsSsize_t(PyObject*);
static CYTHON_INLINE PyObject * __Pyx_PyInt_FromSize_t(size_t);
#if CYTHON_ASSUME_SAFE_MACROS
#define __pyx_PyFloat_AsDouble(x) (PyFloat_CheckExact(x) ? PyFloat_AS_DOUBLE(x) : PyFloat_AsDouble(x))
#else
#define __pyx_PyFloat_AsDouble(x) PyFloat_AsDouble(x)
#endif
#define __pyx_PyFloat_AsFloat(x) ((float) __pyx_PyFloat_AsDouble(x))
#if PY_MAJOR_VERSION >= 3
#define __Pyx_PyNumber_Int(x) (PyLong_CheckExact(x) ? __Pyx_NewRef(x) : PyNumber_Long(x))
#else
#define __Pyx_PyNumber_Int(x) (PyInt_CheckExact(x) ? __Pyx_NewRef(x) : PyNumber_Int(x))
#endif
#define __Pyx_PyNumber_Float(x) (PyFloat_CheckExact(x) ? __Pyx_NewRef(x) : PyNumber_Float(x))
#if PY_MAJOR_VERSION < 3 && __PYX_DEFAULT_STRING_ENCODING_IS_ASCII
static int __Pyx_sys_getdefaultencoding_not_ascii;
static int __Pyx_init_sys_getdefaultencoding_params(void) {
    PyObject* sys;
    PyObject* default_encoding = NULL;
    PyObject* ascii_chars_u = NULL;
    PyObject* ascii_chars_b = NULL;
    const char* default_encoding_c;
    sys = PyImport_ImportModule("sys");
    if (!sys) goto bad;
    default_encoding = PyObject_CallMethod(sys, (char*) "getdefaultencoding", NULL);
    Py_DECREF(sys);
    if (!default_encoding) goto bad;
    default_encoding_c = PyBytes_AsString(default_encoding);
    if (!default_encoding_c) goto bad;
    if (strcmp(default_encoding_c, "ascii") == 0) {
        __Pyx_sys_getdefaultencoding_not_ascii = 0;
    } else {
        char ascii_chars[128];
        int c;
        for (c = 0; c < 128; c++) {
            ascii_chars[c] = c;
        }
        __Pyx_sys_getdefaultencoding_not_ascii = 1;
        ascii_chars_u = PyUnicode_DecodeASCII(ascii_chars, 128, NULL);
        if (!ascii_chars_u) goto bad;
        ascii_chars_b = PyUnicode_AsEncodedString(ascii_chars_u, default_encoding_c, NULL);
        if (!ascii_chars_b || !PyBytes_Check(ascii_chars_b) || memcmp(ascii_chars, PyBytes_AS_STRING(ascii_chars_b), 128) != 0) {
            PyErr_Format(
                PyExc_ValueError,
                "This module compiled with c_string_encoding=ascii, but default encoding '%.200s' is not a superset of ascii.",
                default_encoding_c);
            goto bad;
        }
        Py_DECREF(ascii_chars_u);
        Py_DECREF(ascii_chars_b);
    }
    Py_DECREF(default_encoding);
    return 0;
bad:
    Py_XDECREF(default_encoding);
    Py_XDECREF(ascii_chars_u);
    Py_XDECREF(ascii_chars_b);
    return -1;
}
#endif
#if __PYX_DEFAULT_STRING_ENCODING_IS_DEFAULT && PY_MAJOR_VERSION >= 3
#define __Pyx_PyUnicode_FromStringAndSize(c_str, size) PyUnicode_DecodeUTF8(c_str, size, NULL)
#else
#define __Pyx_PyUnicode_FromStringAndSize(c_str, size) PyUnicode_Decode(c_str, size, __PYX_DEFAULT_STRING_ENCODING, NULL)
#if __PYX_DEFAULT_STRING_ENCODING_IS_DEFAULT
static char* __PYX_DEFAULT_STRING_ENCODING;
static int __Pyx_init_sys_getdefaultencoding_params(void) {
    PyObject* sys;
    PyObject* default_encoding = NULL;
    char* default_encoding_c;
    sys = PyImport_ImportModule("sys");
    if (!sys) goto bad;
    default_encoding = PyObject_CallMethod(sys, (char*) (const char*) "getdefaultencoding", NULL);
    Py_DECREF(sys);
    if (!default_encoding) goto bad;
    default_encoding_c = PyBytes_AsString(default_encoding);
    if (!default_encoding_c) goto bad;
    __PYX_DEFAULT_STRING_ENCODING = (char*) malloc(strlen(default_encoding_c));
    if (!__PYX_DEFAULT_STRING_ENCODING) goto bad;
    strcpy(__PYX_DEFAULT_STRING_ENCODING, default_encoding_c);
    Py_DECREF(default_encoding);
    return 0;
bad:
    Py_XDECREF(default_encoding);
    return -1;
}
#endif
#endif


/* Test for GCC > 2.95 */
#if defined(__GNUC__)     && (__GNUC__ > 2 || (__GNUC__ == 2 && (__GNUC_MINOR__ > 95)))
  #define likely(x)   __builtin_expect(!!(x), 1)
  #define unlikely(x) __builtin_expect(!!(x), 0)
#else /* !__GNUC__ or GCC < 2.95 */
  #define likely(x)   (x)
  #define unlikely(x) (x)
#endif /* __GNUC__ */
static CYTHON_INLINE void __Pyx_pretend_to_initialize(void* ptr) { (void)ptr; }

static PyObject *__pyx_m = NULL;
static PyObject *__pyx_d;
static PyObject *__pyx_b;
static PyObject *__pyx_cython_runtime = NULL;
static PyObject *__pyx_empty_tuple;
static PyObject *__pyx_empty_bytes;
static PyObject *__pyx_empty_unicode;
static int __pyx_lineno;
static int __pyx_clineno = 0;
static const char * __pyx_cfilenm= __FILE__;
static const char *__pyx_filename;


static const char *__pyx_f[] = {
  "src/python/grpcio/grpc/_cython/_cygrpc/fork_posix.pyx.pxi",
  "src/python/grpcio/grpc/_cython/_cygrpc/grpc_string.pyx.pxi",
  "src/python/grpcio/grpc/_cython/_cygrpc/arguments.pyx.pxi",
  "src/python/grpcio/grpc/_cython/_cygrpc/call.pyx.pxi",
  "src/python/grpcio/grpc/_cython/_cygrpc/credentials.pyx.pxi",
  "src/python/grpcio/grpc/_cython/_cygrpc/server.pyx.pxi",
  "src/python/grpcio/grpc/_cython/_cygrpc/grpc_gevent.pyx.pxi",
  "stringsource",
  "src/python/grpcio/grpc/_cython/_cygrpc/channel.pyx.pxi",
  "src/python/grpcio/grpc/_cython/_cygrpc/channelz.pyx.pxi",
  "src/python/grpcio/grpc/_cython/_cygrpc/completion_queue.pyx.pxi",
  "src/python/grpcio/grpc/_cython/_cygrpc/event.pyx.pxi",
  "src/python/grpcio/grpc/_cython/_cygrpc/event.pxd.pxi",
  "src/python/grpcio/grpc/_cython/_cygrpc/metadata.pyx.pxi",
  "src/python/grpcio/grpc/_cython/_cygrpc/operation.pyx.pxi",
  "src/python/grpcio/grpc/_cython/_cygrpc/operation.pxd.pxi",
  "src/python/grpcio/grpc/_cython/_cygrpc/records.pyx.pxi",
  "src/python/grpcio/grpc/_cython/_cygrpc/security.pyx.pxi",
  "src/python/grpcio/grpc/_cython/_cygrpc/tag.pyx.pxi",
  "src/python/grpcio/grpc/_cython/_cygrpc/time.pyx.pxi",
  "src/python/grpcio/grpc/_cython/_cygrpc/_hooks.pyx.pxi",
  "src/python/grpcio/grpc/_cython/cygrpc.pyx",
  "src/python/grpcio/grpc/_cython/_cygrpc/vtable.pyx.pxi",
  "type.pxd",
  "bool.pxd",
  "complex.pxd",
  "src/python/grpcio/grpc/_cython/_cygrpc/propagation_bits.pyx.pxi",
};
/* NoFastGil.proto */
#define __Pyx_PyGILState_Ensure PyGILState_Ensure
#define __Pyx_PyGILState_Release PyGILState_Release
#define __Pyx_FastGIL_Remember()
#define __Pyx_FastGIL_Forget()
#define __Pyx_FastGilFuncInit()

/* ForceInitThreads.proto */
#ifndef __PYX_FORCE_INIT_THREADS
  #define __PYX_FORCE_INIT_THREADS 0
#endif


/* "src/python/grpcio/grpc/_cython/_cygrpc/grpc.pxi":17
 * cimport libc.time
 * 
 * ctypedef          ssize_t   intptr_t             # <<<<<<<<<<<<<<
 * ctypedef          size_t    uintptr_t
 * ctypedef   signed char      int8_t
 */
typedef Py_ssize_t __pyx_t_4grpc_7_cython_6cygrpc_intptr_t;

/* "src/python/grpcio/grpc/_cython/_cygrpc/grpc.pxi":18
 * 
 * ctypedef          ssize_t   intptr_t
 * ctypedef          size_t    uintptr_t             # <<<<<<<<<<<<<<
 * ctypedef   signed char      int8_t
 * ctypedef   signed short     int16_t
 */
typedef size_t __pyx_t_4grpc_7_cython_6cygrpc_uintptr_t;

/* "src/python/grpcio/grpc/_cython/_cygrpc/grpc.pxi":19
 * ctypedef          ssize_t   intptr_t
 * ctypedef          size_t    uintptr_t
 * ctypedef   signed char      int8_t             # <<<<<<<<<<<<<<
 * ctypedef   signed short     int16_t
 * ctypedef   signed int       int32_t
 */
typedef signed char __pyx_t_4grpc_7_cython_6cygrpc_int8_t;

/* "src/python/grpcio/grpc/_cython/_cygrpc/grpc.pxi":20
 * ctypedef          size_t    uintptr_t
 * ctypedef   signed char      int8_t
 * ctypedef   signed short     int16_t             # <<<<<<<<<<<<<<
 * ctypedef   signed int       int32_t
 * ctypedef   signed long long int64_t
 */
typedef short __pyx_t_4grpc_7_cython_6cygrpc_int16_t;

/* "src/python/grpcio/grpc/_cython/_cygrpc/grpc.pxi":21
 * ctypedef   signed char      int8_t
 * ctypedef   signed short     int16_t
 * ctypedef   signed int       int32_t             # <<<<<<<<<<<<<<
 * ctypedef   signed long long int64_t
 * ctypedef unsigned char      uint8_t
 */
typedef int __pyx_t_4grpc_7_cython_6cygrpc_int32_t;

/* "src/python/grpcio/grpc/_cython/_cygrpc/grpc.pxi":22
 * ctypedef   signed short     int16_t
 * ctypedef   signed int       int32_t
 * ctypedef   signed long long int64_t             # <<<<<<<<<<<<<<
 * ctypedef unsigned char      uint8_t
 * ctypedef unsigned short     uint16_t
 */
typedef PY_LONG_LONG __pyx_t_4grpc_7_cython_6cygrpc_int64_t;

/* "src/python/grpcio/grpc/_cython/_cygrpc/grpc.pxi":23
 * ctypedef   signed int       int32_t
 * ctypedef   signed long long int64_t
 * ctypedef unsigned char      uint8_t             # <<<<<<<<<<<<<<
 * ctypedef unsigned short     uint16_t
 * ctypedef unsigned int       uint32_t
 */
typedef unsigned char __pyx_t_4grpc_7_cython_6cygrpc_uint8_t;

/* "src/python/grpcio/grpc/_cython/_cygrpc/grpc.pxi":24
 * ctypedef   signed long long int64_t
 * ctypedef unsigned char      uint8_t
 * ctypedef unsigned short     uint16_t             # <<<<<<<<<<<<<<
 * ctypedef unsigned int       uint32_t
 * ctypedef unsigned long long uint64_t
 */
typedef unsigned short __pyx_t_4grpc_7_cython_6cygrpc_uint16_t;

/* "src/python/grpcio/grpc/_cython/_cygrpc/grpc.pxi":25
 * ctypedef unsigned char      uint8_t
 * ctypedef unsigned short     uint16_t
 * ctypedef unsigned int       uint32_t             # <<<<<<<<<<<<<<
 * ctypedef unsigned long long uint64_t
 * 
 */
typedef unsigned int __pyx_t_4grpc_7_cython_6cygrpc_uint32_t;

/* "src/python/grpcio/grpc/_cython/_cygrpc/grpc.pxi":26
 * ctypedef unsigned short     uint16_t
 * ctypedef unsigned int       uint32_t
 * ctypedef unsigned long long uint64_t             # <<<<<<<<<<<<<<
 * 
 * 
 */
typedef unsigned PY_LONG_LONG __pyx_t_4grpc_7_cython_6cygrpc_uint64_t;

/*--- Type declarations ---*/
struct __pyx_obj_4grpc_7_cython_6cygrpc__ChannelArg;
struct __pyx_obj_4grpc_7_cython_6cygrpc__ChannelArgs;
struct __pyx_obj_4grpc_7_cython_6cygrpc_Call;
struct __pyx_obj_4grpc_7_cython_6cygrpc__CallState;
struct __pyx_obj_4grpc_7_cython_6cygrpc__ChannelState;
struct __pyx_obj_4grpc_7_cython_6cygrpc_IntegratedCall;
struct __pyx_obj_4grpc_7_cython_6cygrpc_SegregatedCall;
struct __pyx_obj_4grpc_7_cython_6cygrpc_Channel;
struct __pyx_obj_4grpc_7_cython_6cygrpc_CallCredentials;
struct __pyx_obj_4grpc_7_cython_6cygrpc_MetadataPluginCallCredentials;
struct __pyx_obj_4grpc_7_cython_6cygrpc_CompositeCallCredentials;
struct __pyx_obj_4grpc_7_cython_6cygrpc_ChannelCredentials;
struct __pyx_obj_4grpc_7_cython_6cygrpc_SSLSessionCacheLRU;
struct __pyx_obj_4grpc_7_cython_6cygrpc_SSLChannelCredentials;
struct __pyx_obj_4grpc_7_cython_6cygrpc_CompositeChannelCredentials;
struct __pyx_obj_4grpc_7_cython_6cygrpc_ServerCertificateConfig;
struct __pyx_obj_4grpc_7_cython_6cygrpc_ServerCredentials;
struct __pyx_obj_4grpc_7_cython_6cygrpc_CompletionQueue;
struct __pyx_obj_4grpc_7_cython_6cygrpc_ConnectivityEvent;
struct __pyx_obj_4grpc_7_cython_6cygrpc_RequestCallEvent;
struct __pyx_obj_4grpc_7_cython_6cygrpc_BatchOperationEvent;
struct __pyx_obj_4grpc_7_cython_6cygrpc_ServerShutdownEvent;
struct __pyx_obj_4grpc_7_cython_6cygrpc_Operation;
struct __pyx_obj_4grpc_7_cython_6cygrpc_SendInitialMetadataOperation;
struct __pyx_obj_4grpc_7_cython_6cygrpc_SendMessageOperation;
struct __pyx_obj_4grpc_7_cython_6cygrpc_SendCloseFromClientOperation;
struct __pyx_obj_4grpc_7_cython_6cygrpc_SendStatusFromServerOperation;
struct __pyx_obj_4grpc_7_cython_6cygrpc_ReceiveInitialMetadataOperation;
struct __pyx_obj_4grpc_7_cython_6cygrpc_ReceiveMessageOperation;
struct __pyx_obj_4grpc_7_cython_6cygrpc_ReceiveStatusOnClientOperation;
struct __pyx_obj_4grpc_7_cython_6cygrpc_ReceiveCloseOnServerOperation;
struct __pyx_obj_4grpc_7_cython_6cygrpc_CallDetails;
struct __pyx_obj_4grpc_7_cython_6cygrpc_SslPemKeyCertPair;
struct __pyx_obj_4grpc_7_cython_6cygrpc_CompressionOptions;
struct __pyx_obj_4grpc_7_cython_6cygrpc_Server;
struct __pyx_obj_4grpc_7_cython_6cygrpc__Tag;
struct __pyx_obj_4grpc_7_cython_6cygrpc__ConnectivityTag;
struct __pyx_obj_4grpc_7_cython_6cygrpc__RequestCallTag;
struct __pyx_obj_4grpc_7_cython_6cygrpc__BatchOperationTag;
struct __pyx_obj_4grpc_7_cython_6cygrpc__ServerShutdownTag;
struct __pyx_obj_4grpc_7_cython_6cygrpc__VTable;
struct __pyx_obj_4grpc_7_cython_6cygrpc_TimerWrapper;
struct __pyx_obj_4grpc_7_cython_6cygrpc_SocketWrapper;
struct __pyx_obj_4grpc_7_cython_6cygrpc_ResolveWrapper;
struct __pyx_obj_4grpc_7_cython_6cygrpc__GrpcArgWrapper;
struct __pyx_obj_4grpc_7_cython_6cygrpc_CensusContext;
struct __pyx_obj_4grpc_7_cython_6cygrpc___pyx_scope_struct____pyx_f_4grpc_7_cython_6cygrpc__integrated_call;
struct __pyx_obj_4grpc_7_cython_6cygrpc___pyx_scope_struct_1_next_event;
struct __pyx_obj_4grpc_7_cython_6cygrpc___pyx_scope_struct_2___pyx_f_4grpc_7_cython_6cygrpc__segregated_call;
struct __pyx_obj_4grpc_7_cython_6cygrpc___pyx_scope_struct_3_next_call_event;
struct __pyx_obj_4grpc_7_cython_6cygrpc___pyx_scope_struct_4___pyx_f_4grpc_7_cython_6cygrpc__get_metadata;
struct __pyx_obj_4grpc_7_cython_6cygrpc___pyx_scope_struct_5___pyx_f_4grpc_7_cython_6cygrpc__metadata;
struct __pyx_obj_4grpc_7_cython_6cygrpc___pyx_scope_struct_6_genexpr;
struct __pyx_obj_4grpc_7_cython_6cygrpc___pyx_scope_struct_7___pyx_f_4grpc_7_cython_6cygrpc_tuples_to_resolvaddr;
struct __pyx_obj_4grpc_7_cython_6cygrpc___pyx_scope_struct_8_genexpr;
struct __pyx_obj_4grpc_7_cython_6cygrpc___pyx_scope_struct_9___init__;

/* "src/python/grpcio/grpc/_cython/_cygrpc/arguments.pxd.pxi":22
 * 
 * 
 * cdef class _ChannelArg:             # <<<<<<<<<<<<<<
 * 
 *   cdef grpc_arg c_argument
 */
struct __pyx_obj_4grpc_7_cython_6cygrpc__ChannelArg {
  PyObject_HEAD
  struct __pyx_vtabstruct_4grpc_7_cython_6cygrpc__ChannelArg *__pyx_vtab;
  grpc_arg c_argument;
};


/* "src/python/grpcio/grpc/_cython/_cygrpc/arguments.pxd.pxi":29
 * 
 * 
 * cdef class _ChannelArgs:             # <<<<<<<<<<<<<<
 * 
 *   cdef readonly tuple _arguments
 */
struct __pyx_obj_4grpc_7_cython_6cygrpc__ChannelArgs {
  PyObject_HEAD
  struct __pyx_vtabstruct_4grpc_7_cython_6cygrpc__ChannelArgs *__pyx_vtab;
  PyObject *_arguments;
  PyObject *_channel_args;
  PyObject *_references;
  grpc_channel_args _c_arguments;
};


/* "src/python/grpcio/grpc/_cython/_cygrpc/call.pxd.pxi":16
 * 
 * 
 * cdef class Call:             # <<<<<<<<<<<<<<
 * 
 *   cdef grpc_call *c_call
 */
struct __pyx_obj_4grpc_7_cython_6cygrpc_Call {
  PyObject_HEAD
  grpc_call *c_call;
  PyObject *references;
};


/* "src/python/grpcio/grpc/_cython/_cygrpc/channel.pxd.pxi":25
 * 
 * 
 * cdef class _CallState:             # <<<<<<<<<<<<<<
 * 
 *   cdef grpc_call *c_call
 */
struct __pyx_obj_4grpc_7_cython_6cygrpc__CallState {
  PyObject_HEAD
  grpc_call *c_call;
  PyObject *due;
};


/* "src/python/grpcio/grpc/_cython/_cygrpc/channel.pxd.pxi":31
 * 
 * 
 * cdef class _ChannelState:             # <<<<<<<<<<<<<<
 * 
 *   cdef object condition
 */
struct __pyx_obj_4grpc_7_cython_6cygrpc__ChannelState {
  PyObject_HEAD
  PyObject *condition;
  grpc_channel *c_channel;
  PyObject *open;
  PyObject *closed_reason;
  PyObject *integrated_call_states;
  grpc_completion_queue *c_call_completion_queue;
  PyObject *segregated_call_states;
  PyObject *connectivity_due;
  grpc_completion_queue *c_connectivity_completion_queue;
};


/* "src/python/grpcio/grpc/_cython/_cygrpc/channel.pxd.pxi":56
 * 
 * 
 * cdef class IntegratedCall:             # <<<<<<<<<<<<<<
 * 
 *   cdef _ChannelState _channel_state
 */
struct __pyx_obj_4grpc_7_cython_6cygrpc_IntegratedCall {
  PyObject_HEAD
  struct __pyx_obj_4grpc_7_cython_6cygrpc__ChannelState *_channel_state;
  struct __pyx_obj_4grpc_7_cython_6cygrpc__CallState *_call_state;
};


/* "src/python/grpcio/grpc/_cython/_cygrpc/channel.pxd.pxi":62
 * 
 * 
 * cdef class SegregatedCall:             # <<<<<<<<<<<<<<
 * 
 *   cdef _ChannelState _channel_state
 */
struct __pyx_obj_4grpc_7_cython_6cygrpc_SegregatedCall {
  PyObject_HEAD
  struct __pyx_obj_4grpc_7_cython_6cygrpc__ChannelState *_channel_state;
  struct __pyx_obj_4grpc_7_cython_6cygrpc__CallState *_call_state;
  grpc_completion_queue *_c_completion_queue;
};


/* "src/python/grpcio/grpc/_cython/_cygrpc/channel.pxd.pxi":69
 * 
 * 
 * cdef class Channel:             # <<<<<<<<<<<<<<
 * 
 *   cdef _ChannelState _state
 */
struct __pyx_obj_4grpc_7_cython_6cygrpc_Channel {
  PyObject_HEAD
  struct __pyx_obj_4grpc_7_cython_6cygrpc__ChannelState *_state;
  struct __pyx_obj_4grpc_7_cython_6cygrpc__VTable *_vtable;
  PyObject *_arguments;
};


/* "src/python/grpcio/grpc/_cython/_cygrpc/credentials.pxd.pxi":16
 * 
 * 
 * cdef class CallCredentials:             # <<<<<<<<<<<<<<
 * 
 *   cdef grpc_call_credentials *c(self) except *
 */
struct __pyx_obj_4grpc_7_cython_6cygrpc_CallCredentials {
  PyObject_HEAD
  struct __pyx_vtabstruct_4grpc_7_cython_6cygrpc_CallCredentials *__pyx_vtab;
  grpc_call_credentials *c_credentials;
};


/* "src/python/grpcio/grpc/_cython/_cygrpc/credentials.pxd.pxi":34
 * 
 * 
 * cdef class MetadataPluginCallCredentials(CallCredentials):             # <<<<<<<<<<<<<<
 * 
 *   cdef readonly object _metadata_plugin
 */
struct __pyx_obj_4grpc_7_cython_6cygrpc_MetadataPluginCallCredentials {
  struct __pyx_obj_4grpc_7_cython_6cygrpc_CallCredentials __pyx_base;
  PyObject *_metadata_plugin;
  PyObject *_name;
};


/* "src/python/grpcio/grpc/_cython/_cygrpc/credentials.pxd.pxi":45
 * 
 * 
 * cdef class CompositeCallCredentials(CallCredentials):             # <<<<<<<<<<<<<<
 * 
 *   cdef readonly tuple _call_credentialses
 */
struct __pyx_obj_4grpc_7_cython_6cygrpc_CompositeCallCredentials {
  struct __pyx_obj_4grpc_7_cython_6cygrpc_CallCredentials __pyx_base;
  PyObject *_call_credentialses;
};


/* "src/python/grpcio/grpc/_cython/_cygrpc/credentials.pxd.pxi":52
 * 
 * 
 * cdef class ChannelCredentials:             # <<<<<<<<<<<<<<
 * 
 *   cdef grpc_channel_credentials *c(self) except *
 */
struct __pyx_obj_4grpc_7_cython_6cygrpc_ChannelCredentials {
  PyObject_HEAD
  struct __pyx_vtabstruct_4grpc_7_cython_6cygrpc_ChannelCredentials *__pyx_vtab;
};


/* "src/python/grpcio/grpc/_cython/_cygrpc/credentials.pxd.pxi":57
 * 
 * 
 * cdef class SSLSessionCacheLRU:             # <<<<<<<<<<<<<<
 * 
 *   cdef grpc_ssl_session_cache *_cache
 */
struct __pyx_obj_4grpc_7_cython_6cygrpc_SSLSessionCacheLRU {
  PyObject_HEAD
  grpc_ssl_session_cache *_cache;
};


/* "src/python/grpcio/grpc/_cython/_cygrpc/credentials.pxd.pxi":62
 * 
 * 
 * cdef class SSLChannelCredentials(ChannelCredentials):             # <<<<<<<<<<<<<<
 * 
 *   cdef readonly object _pem_root_certificates
 */
struct __pyx_obj_4grpc_7_cython_6cygrpc_SSLChannelCredentials {
  struct __pyx_obj_4grpc_7_cython_6cygrpc_ChannelCredentials __pyx_base;
  PyObject *_pem_root_certificates;
  PyObject *_private_key;
  PyObject *_certificate_chain;
};


/* "src/python/grpcio/grpc/_cython/_cygrpc/credentials.pxd.pxi":71
 * 
 * 
 * cdef class CompositeChannelCredentials(ChannelCredentials):             # <<<<<<<<<<<<<<
 * 
 *   cdef readonly tuple _call_credentialses
 */
struct __pyx_obj_4grpc_7_cython_6cygrpc_CompositeChannelCredentials {
  struct __pyx_obj_4grpc_7_cython_6cygrpc_ChannelCredentials __pyx_base;
  PyObject *_call_credentialses;
  struct __pyx_obj_4grpc_7_cython_6cygrpc_ChannelCredentials *_channel_credentials;
};


/* "src/python/grpcio/grpc/_cython/_cygrpc/credentials.pxd.pxi":79
 * 
 * 
 * cdef class ServerCertificateConfig:             # <<<<<<<<<<<<<<
 * 
 *   cdef grpc_ssl_server_certificate_config *c_cert_config
 */
struct __pyx_obj_4grpc_7_cython_6cygrpc_ServerCertificateConfig {
  PyObject_HEAD
  grpc_ssl_server_certificate_config *c_cert_config;
  char const *c_pem_root_certs;
  grpc_ssl_pem_key_cert_pair *c_ssl_pem_key_cert_pairs;
  size_t c_ssl_pem_key_cert_pairs_count;
  PyObject *references;
};


/* "src/python/grpcio/grpc/_cython/_cygrpc/credentials.pxd.pxi":88
 * 
 * 
 * cdef class ServerCredentials:             # <<<<<<<<<<<<<<
 * 
 *   cdef grpc_server_credentials *c_credentials
 */
struct __pyx_obj_4grpc_7_cython_6cygrpc_ServerCredentials {
  PyObject_HEAD
  grpc_server_credentials *c_credentials;
  grpc_ssl_pem_key_cert_pair *c_ssl_pem_key_cert_pairs;
  size_t c_ssl_pem_key_cert_pairs_count;
  PyObject *references;
  PyObject *initial_cert_config;
  PyObject *cert_config_fetcher;
  int initial_cert_config_fetched;
};


/* "src/python/grpcio/grpc/_cython/_cygrpc/completion_queue.pxd.pxi":22
 * 
 * 
 * cdef class CompletionQueue:             # <<<<<<<<<<<<<<
 * 
 *   cdef grpc_completion_queue *c_completion_queue
 */
struct __pyx_obj_4grpc_7_cython_6cygrpc_CompletionQueue {
  PyObject_HEAD
  struct __pyx_vtabstruct_4grpc_7_cython_6cygrpc_CompletionQueue *__pyx_vtab;
  grpc_completion_queue *c_completion_queue;
  int is_shutting_down;
  int is_shutdown;
};


/* "src/python/grpcio/grpc/_cython/_cygrpc/event.pxd.pxi":16
 * 
 * 
 * cdef class ConnectivityEvent:             # <<<<<<<<<<<<<<
 * 
 *   cdef readonly grpc_completion_type completion_type
 */
struct __pyx_obj_4grpc_7_cython_6cygrpc_ConnectivityEvent {
  PyObject_HEAD
  grpc_completion_type completion_type;
  int success;
  PyObject *tag;
};


/* "src/python/grpcio/grpc/_cython/_cygrpc/event.pxd.pxi":23
 * 
 * 
 * cdef class RequestCallEvent:             # <<<<<<<<<<<<<<
 * 
 *   cdef readonly grpc_completion_type completion_type
 */
struct __pyx_obj_4grpc_7_cython_6cygrpc_RequestCallEvent {
  PyObject_HEAD
  grpc_completion_type completion_type;
  int success;
  PyObject *tag;
  struct __pyx_obj_4grpc_7_cython_6cygrpc_Call *call;
  struct __pyx_obj_4grpc_7_cython_6cygrpc_CallDetails *call_details;
  PyObject *invocation_metadata;
};


/* "src/python/grpcio/grpc/_cython/_cygrpc/event.pxd.pxi":33
 * 
 * 
 * cdef class BatchOperationEvent:             # <<<<<<<<<<<<<<
 * 
 *   cdef readonly grpc_completion_type completion_type
 */
struct __pyx_obj_4grpc_7_cython_6cygrpc_BatchOperationEvent {
  PyObject_HEAD
  grpc_completion_type completion_type;
  int success;
  PyObject *tag;
  PyObject *batch_operations;
};


/* "src/python/grpcio/grpc/_cython/_cygrpc/event.pxd.pxi":41
 * 
 * 
 * cdef class ServerShutdownEvent:             # <<<<<<<<<<<<<<
 * 
 *   cdef readonly grpc_completion_type completion_type
 */
struct __pyx_obj_4grpc_7_cython_6cygrpc_ServerShutdownEvent {
  PyObject_HEAD
  grpc_completion_type completion_type;
  int success;
  PyObject *tag;
};


/* "src/python/grpcio/grpc/_cython/_cygrpc/operation.pxd.pxi":16
 * 
 * 
 * cdef class Operation:             # <<<<<<<<<<<<<<
 * 
 *   cdef void c(self) except *
 */
struct __pyx_obj_4grpc_7_cython_6cygrpc_Operation {
  PyObject_HEAD
  struct __pyx_vtabstruct_4grpc_7_cython_6cygrpc_Operation *__pyx_vtab;
  grpc_op c_op;
};


/* "src/python/grpcio/grpc/_cython/_cygrpc/operation.pxd.pxi":25
 * 
 * 
 * cdef class SendInitialMetadataOperation(Operation):             # <<<<<<<<<<<<<<
 * 
 *   cdef readonly object _initial_metadata;
 */
struct __pyx_obj_4grpc_7_cython_6cygrpc_SendInitialMetadataOperation {
  struct __pyx_obj_4grpc_7_cython_6cygrpc_Operation __pyx_base;
  PyObject *_initial_metadata;
  int _flags;
  grpc_metadata *_c_initial_metadata;
  size_t _c_initial_metadata_count;
};


/* "src/python/grpcio/grpc/_cython/_cygrpc/operation.pxd.pxi":36
 * 
 * 
 * cdef class SendMessageOperation(Operation):             # <<<<<<<<<<<<<<
 * 
 *   cdef readonly bytes _message
 */
struct __pyx_obj_4grpc_7_cython_6cygrpc_SendMessageOperation {
  struct __pyx_obj_4grpc_7_cython_6cygrpc_Operation __pyx_base;
  PyObject *_message;
  int _flags;
  grpc_byte_buffer *_c_message_byte_buffer;
};


/* "src/python/grpcio/grpc/_cython/_cygrpc/operation.pxd.pxi":46
 * 
 * 
 * cdef class SendCloseFromClientOperation(Operation):             # <<<<<<<<<<<<<<
 * 
 *   cdef readonly int _flags
 */
struct __pyx_obj_4grpc_7_cython_6cygrpc_SendCloseFromClientOperation {
  struct __pyx_obj_4grpc_7_cython_6cygrpc_Operation __pyx_base;
  int _flags;
};


/* "src/python/grpcio/grpc/_cython/_cygrpc/operation.pxd.pxi":54
 * 
 * 
 * cdef class SendStatusFromServerOperation(Operation):             # <<<<<<<<<<<<<<
 * 
 *   cdef readonly object _trailing_metadata
 */
struct __pyx_obj_4grpc_7_cython_6cygrpc_SendStatusFromServerOperation {
  struct __pyx_obj_4grpc_7_cython_6cygrpc_Operation __pyx_base;
  PyObject *_trailing_metadata;
  PyObject *_code;
  PyObject *_details;
  int _flags;
  grpc_metadata *_c_trailing_metadata;
  size_t _c_trailing_metadata_count;
  grpc_slice _c_details;
};


/* "src/python/grpcio/grpc/_cython/_cygrpc/operation.pxd.pxi":68
 * 
 * 
 * cdef class ReceiveInitialMetadataOperation(Operation):             # <<<<<<<<<<<<<<
 * 
 *   cdef readonly int _flags
 */
struct __pyx_obj_4grpc_7_cython_6cygrpc_ReceiveInitialMetadataOperation {
  struct __pyx_obj_4grpc_7_cython_6cygrpc_Operation __pyx_base;
  int _flags;
  PyObject *_initial_metadata;
  grpc_metadata_array _c_initial_metadata;
};


/* "src/python/grpcio/grpc/_cython/_cygrpc/operation.pxd.pxi":78
 * 
 * 
 * cdef class ReceiveMessageOperation(Operation):             # <<<<<<<<<<<<<<
 * 
 *   cdef readonly int _flags
 */
struct __pyx_obj_4grpc_7_cython_6cygrpc_ReceiveMessageOperation {
  struct __pyx_obj_4grpc_7_cython_6cygrpc_Operation __pyx_base;
  int _flags;
  grpc_byte_buffer *_c_message_byte_buffer;
  PyObject *_message;
};


/* "src/python/grpcio/grpc/_cython/_cygrpc/operation.pxd.pxi":88
 * 
 * 
 * cdef class ReceiveStatusOnClientOperation(Operation):             # <<<<<<<<<<<<<<
 * 
 *   cdef readonly int _flags
 */
struct __pyx_obj_4grpc_7_cython_6cygrpc_ReceiveStatusOnClientOperation {
  struct __pyx_obj_4grpc_7_cython_6cygrpc_Operation __pyx_base;
  int _flags;
  grpc_metadata_array _c_trailing_metadata;
  grpc_status_code _c_code;
  grpc_slice _c_details;
  char const *_c_error_string;
  PyObject *_trailing_metadata;
  PyObject *_code;
  PyObject *_details;
  PyObject *_error_string;
};


/* "src/python/grpcio/grpc/_cython/_cygrpc/operation.pxd.pxi":104
 * 
 * 
 * cdef class ReceiveCloseOnServerOperation(Operation):             # <<<<<<<<<<<<<<
 * 
 *   cdef readonly int _flags
 */
struct __pyx_obj_4grpc_7_cython_6cygrpc_ReceiveCloseOnServerOperation {
  struct __pyx_obj_4grpc_7_cython_6cygrpc_Operation __pyx_base;
  int _flags;
  PyObject *_cancelled;
  int _c_cancelled;
};


/* "src/python/grpcio/grpc/_cython/_cygrpc/records.pxd.pxi":21
 * 
 * 
 * cdef class CallDetails:             # <<<<<<<<<<<<<<
 * 
 *   cdef grpc_call_details c_details
 */
struct __pyx_obj_4grpc_7_cython_6cygrpc_CallDetails {
  PyObject_HEAD
  grpc_call_details c_details;
};


/* "src/python/grpcio/grpc/_cython/_cygrpc/records.pxd.pxi":26
 * 
 * 
 * cdef class SslPemKeyCertPair:             # <<<<<<<<<<<<<<
 * 
 *   cdef grpc_ssl_pem_key_cert_pair c_pair
 */
struct __pyx_obj_4grpc_7_cython_6cygrpc_SslPemKeyCertPair {
  PyObject_HEAD
  grpc_ssl_pem_key_cert_pair c_pair;
  PyObject *private_key;
  PyObject *certificate_chain;
};


/* "src/python/grpcio/grpc/_cython/_cygrpc/records.pxd.pxi":32
 * 
 * 
 * cdef class CompressionOptions:             # <<<<<<<<<<<<<<
 * 
 *   cdef grpc_compression_options c_options
 */
struct __pyx_obj_4grpc_7_cython_6cygrpc_CompressionOptions {
  PyObject_HEAD
  grpc_compression_options c_options;
};


/* "src/python/grpcio/grpc/_cython/_cygrpc/server.pxd.pxi":15
 * # limitations under the License.
 * 
 * cdef class Server:             # <<<<<<<<<<<<<<
 * 
 *   cdef grpc_server *c_server
 */
struct __pyx_obj_4grpc_7_cython_6cygrpc_Server {
  PyObject_HEAD
  struct __pyx_vtabstruct_4grpc_7_cython_6cygrpc_Server *__pyx_vtab;
  grpc_server *c_server;
  struct __pyx_obj_4grpc_7_cython_6cygrpc__VTable *_vtable;
  int is_started;
  int is_shutting_down;
  int is_shutdown;
  struct __pyx_obj_4grpc_7_cython_6cygrpc_CompletionQueue *backup_shutdown_queue;
  PyObject *references;
  PyObject *registered_completion_queues;
};


/* "src/python/grpcio/grpc/_cython/_cygrpc/tag.pxd.pxi":16
 * 
 * 
 * cdef class _Tag:             # <<<<<<<<<<<<<<
 * 
 *   cdef object event(self, grpc_event c_event)
 */
struct __pyx_obj_4grpc_7_cython_6cygrpc__Tag {
  PyObject_HEAD
  struct __pyx_vtabstruct_4grpc_7_cython_6cygrpc__Tag *__pyx_vtab;
};


/* "src/python/grpcio/grpc/_cython/_cygrpc/tag.pxd.pxi":21
 * 
 * 
 * cdef class _ConnectivityTag(_Tag):             # <<<<<<<<<<<<<<
 * 
 *   cdef readonly object _user_tag
 */
struct __pyx_obj_4grpc_7_cython_6cygrpc__ConnectivityTag {
  struct __pyx_obj_4grpc_7_cython_6cygrpc__Tag __pyx_base;
  PyObject *_user_tag;
};


/* "src/python/grpcio/grpc/_cython/_cygrpc/tag.pxd.pxi":28
 * 
 * 
 * cdef class _RequestCallTag(_Tag):             # <<<<<<<<<<<<<<
 * 
 *   cdef readonly object _user_tag
 */
struct __pyx_obj_4grpc_7_cython_6cygrpc__RequestCallTag {
  struct __pyx_obj_4grpc_7_cython_6cygrpc__Tag __pyx_base;
  PyObject *_user_tag;
  struct __pyx_obj_4grpc_7_cython_6cygrpc_Call *call;
  struct __pyx_obj_4grpc_7_cython_6cygrpc__VTable *_vtable;
  struct __pyx_obj_4grpc_7_cython_6cygrpc_CallDetails *call_details;
  grpc_metadata_array c_invocation_metadata;
};


/* "src/python/grpcio/grpc/_cython/_cygrpc/tag.pxd.pxi":40
 * 
 * 
 * cdef class _BatchOperationTag(_Tag):             # <<<<<<<<<<<<<<
 * 
 *   cdef object _user_tag
 */
struct __pyx_obj_4grpc_7_cython_6cygrpc__BatchOperationTag {
  struct __pyx_obj_4grpc_7_cython_6cygrpc__Tag __pyx_base;
  PyObject *_user_tag;
  PyObject *_operations;
  PyObject *_retained_call;
  grpc_op *c_ops;
  size_t c_nops;
};


/* "src/python/grpcio/grpc/_cython/_cygrpc/tag.pxd.pxi":52
 * 
 * 
 * cdef class _ServerShutdownTag(_Tag):             # <<<<<<<<<<<<<<
 * 
 *   cdef readonly object _user_tag
 */
struct __pyx_obj_4grpc_7_cython_6cygrpc__ServerShutdownTag {
  struct __pyx_obj_4grpc_7_cython_6cygrpc__Tag __pyx_base;
  PyObject *_user_tag;
  struct __pyx_obj_4grpc_7_cython_6cygrpc_Server *_shutting_down_server;
};


/* "src/python/grpcio/grpc/_cython/_cygrpc/vtable.pxd.pxi":25
 * 
 * 
 * cdef class _VTable:             # <<<<<<<<<<<<<<
 *   cdef grpc_arg_pointer_vtable c_vtable
 */
struct __pyx_obj_4grpc_7_cython_6cygrpc__VTable {
  PyObject_HEAD
  grpc_arg_pointer_vtable c_vtable;
};


/* "src/python/grpcio/grpc/_cython/_cygrpc/grpc_gevent.pxd.pxi":125
 * 
 * 
 * cdef class TimerWrapper:             # <<<<<<<<<<<<<<
 * 
 *   cdef grpc_custom_timer *c_timer
 */
struct __pyx_obj_4grpc_7_cython_6cygrpc_TimerWrapper {
  PyObject_HEAD
  struct grpc_custom_timer *c_timer;
  PyObject *timer;
  PyObject *event;
};


/* "src/python/grpcio/grpc/_cython/_cygrpc/grpc_gevent.pxd.pxi":131
 *   cdef object event
 * 
 * cdef class SocketWrapper:             # <<<<<<<<<<<<<<
 *   cdef object sockopts
 *   cdef object socket
 */
struct __pyx_obj_4grpc_7_cython_6cygrpc_SocketWrapper {
  PyObject_HEAD
  PyObject *sockopts;
  PyObject *socket;
  PyObject *closed;
  struct grpc_custom_socket *c_socket;
  char *c_buffer;
  size_t len;
  struct grpc_custom_socket *accepting_socket;
  grpc_custom_connect_callback connect_cb;
  grpc_custom_write_callback write_cb;
  grpc_custom_read_callback read_cb;
  grpc_custom_accept_callback accept_cb;
  grpc_custom_close_callback close_cb;
};


/* "src/python/grpcio/grpc/_cython/_cygrpc/grpc_gevent.pxd.pxi":147
 * 
 * 
 * cdef class ResolveWrapper:             # <<<<<<<<<<<<<<
 *   cdef grpc_custom_resolver *c_resolver
 *   cdef char* c_host
 */
struct __pyx_obj_4grpc_7_cython_6cygrpc_ResolveWrapper {
  PyObject_HEAD
  struct grpc_custom_resolver *c_resolver;
  char *c_host;
  char *c_port;
};


/* "src/python/grpcio/grpc/_cython/_cygrpc/arguments.pyx.pxi":18
 * 
 * 
 * cdef class _GrpcArgWrapper:             # <<<<<<<<<<<<<<
 * 
 *   cdef grpc_arg arg
 */
struct __pyx_obj_4grpc_7_cython_6cygrpc__GrpcArgWrapper {
  PyObject_HEAD
  grpc_arg arg;
};


/* "src/python/grpcio/grpc/_cython/_cygrpc/_hooks.pyx.pxi":28
 *   pass
 * 
 * cdef class CensusContext:             # <<<<<<<<<<<<<<
 *   pass
 * 
 */
struct __pyx_obj_4grpc_7_cython_6cygrpc_CensusContext {
  PyObject_HEAD
};


/* "src/python/grpcio/grpc/_cython/_cygrpc/channel.pyx.pxi":262
 * 
 * 
 * cdef IntegratedCall _integrated_call(             # <<<<<<<<<<<<<<
 *     _ChannelState state, int flags, method, host, object deadline,
 *     object metadata, CallCredentials credentials, operationses_and_user_tags,
 */
struct __pyx_obj_4grpc_7_cython_6cygrpc___pyx_scope_struct____pyx_f_4grpc_7_cython_6cygrpc__integrated_call {
  PyObject_HEAD
  struct __pyx_obj_4grpc_7_cython_6cygrpc__CallState *__pyx_v_call_state;
  struct __pyx_obj_4grpc_7_cython_6cygrpc__ChannelState *__pyx_v_state;
};


/* "src/python/grpcio/grpc/_cython/_cygrpc/channel.pyx.pxi":306
 *     _cancel(self._channel_state, self._call_state, code, details)
 * 
 *   def next_event(self):             # <<<<<<<<<<<<<<
 *     def on_success(tag):
 *       _process_segregated_call_tag(
 */
struct __pyx_obj_4grpc_7_cython_6cygrpc___pyx_scope_struct_1_next_event {
  PyObject_HEAD
  struct __pyx_obj_4grpc_7_cython_6cygrpc_SegregatedCall *__pyx_v_self;
};


/* "src/python/grpcio/grpc/_cython/_cygrpc/channel.pyx.pxi":314
 * 
 * 
 * cdef SegregatedCall _segregated_call(             # <<<<<<<<<<<<<<
 *     _ChannelState state, int flags, method, host, object deadline,
 *     object metadata, CallCredentials credentials, operationses_and_user_tags,
 */
struct __pyx_obj_4grpc_7_cython_6cygrpc___pyx_scope_struct_2___pyx_f_4grpc_7_cython_6cygrpc__segregated_call {
  PyObject_HEAD
  struct __pyx_obj_4grpc_7_cython_6cygrpc__CallState *__pyx_v_call_state;
  struct __pyx_obj_4grpc_7_cython_6cygrpc__ChannelState *__pyx_v_state;
};


/* "src/python/grpcio/grpc/_cython/_cygrpc/channel.pyx.pxi":456
 *         operationses_and_tags, context)
 * 
 *   def next_call_event(self):             # <<<<<<<<<<<<<<
 *     def on_success(tag):
 *       if tag is not None:
 */
struct __pyx_obj_4grpc_7_cython_6cygrpc___pyx_scope_struct_3_next_call_event {
  PyObject_HEAD
  struct __pyx_obj_4grpc_7_cython_6cygrpc_Channel *__pyx_v_self;
};


/* "src/python/grpcio/grpc/_cython/_cygrpc/credentials.pyx.pxi":40
 * 
 * 
 * cdef int _get_metadata(             # <<<<<<<<<<<<<<
 *     void *state, grpc_auth_metadata_context context,
 *     grpc_credentials_plugin_metadata_cb cb, void *user_data,
 */
struct __pyx_obj_4grpc_7_cython_6cygrpc___pyx_scope_struct_4___pyx_f_4grpc_7_cython_6cygrpc__get_metadata {
  PyObject_HEAD
  grpc_metadata *__pyx_v_c_metadata;
  grpc_credentials_plugin_metadata_cb __pyx_v_cb;
  size_t __pyx_v_metadata_count;
  void *__pyx_v_user_data;
};


/* "src/python/grpcio/grpc/_cython/_cygrpc/metadata.pyx.pxi":63
 * 
 * 
 * cdef tuple _metadata(grpc_metadata_array *c_metadata_array):             # <<<<<<<<<<<<<<
 *   return tuple(
 *       _metadatum(
 */
struct __pyx_obj_4grpc_7_cython_6cygrpc___pyx_scope_struct_5___pyx_f_4grpc_7_cython_6cygrpc__metadata {
  PyObject_HEAD
  grpc_metadata_array *__pyx_v_c_metadata_array;
};


/* "src/python/grpcio/grpc/_cython/_cygrpc/metadata.pyx.pxi":65
 * cdef tuple _metadata(grpc_metadata_array *c_metadata_array):
 *   return tuple(
 *       _metadatum(             # <<<<<<<<<<<<<<
 *           c_metadata_array.metadata[index].key,
 *           c_metadata_array.metadata[index].value)
 */
struct __pyx_obj_4grpc_7_cython_6cygrpc___pyx_scope_struct_6_genexpr {
  PyObject_HEAD
  struct __pyx_obj_4grpc_7_cython_6cygrpc___pyx_scope_struct_5___pyx_f_4grpc_7_cython_6cygrpc__metadata *__pyx_outer_scope;
  size_t __pyx_v_index;
  size_t __pyx_t_0;
  size_t __pyx_t_1;
  size_t __pyx_t_2;
};


/* "src/python/grpcio/grpc/_cython/_cygrpc/grpc_gevent.pyx.pxi":59
 *   return grpc_sockaddr_get_uri_scheme(&c_addr) == b'ipv4'
 * 
 * cdef grpc_resolved_addresses* tuples_to_resolvaddr(tups) except *:             # <<<<<<<<<<<<<<
 *   cdef grpc_resolved_addresses* addresses
 *   tups_set = set((tup[4][0], tup[4][1]) for tup in tups)
 */
struct __pyx_obj_4grpc_7_cython_6cygrpc___pyx_scope_struct_7___pyx_f_4grpc_7_cython_6cygrpc_tuples_to_resolvaddr {
  PyObject_HEAD
  PyObject *__pyx_v_tups;
};


/* "src/python/grpcio/grpc/_cython/_cygrpc/grpc_gevent.pyx.pxi":61
 * cdef grpc_resolved_addresses* tuples_to_resolvaddr(tups) except *:
 *   cdef grpc_resolved_addresses* addresses
 *   tups_set = set((tup[4][0], tup[4][1]) for tup in tups)             # <<<<<<<<<<<<<<
 *   addresses = <grpc_resolved_addresses*> malloc(sizeof(grpc_resolved_addresses))
 *   addresses.naddrs = len(tups_set)
 */
struct __pyx_obj_4grpc_7_cython_6cygrpc___pyx_scope_struct_8_genexpr {
  PyObject_HEAD
  struct __pyx_obj_4grpc_7_cython_6cygrpc___pyx_scope_struct_7___pyx_f_4grpc_7_cython_6cygrpc_tuples_to_resolvaddr *__pyx_outer_scope;
  PyObject *__pyx_v_tup;
};


/* "src/python/grpcio/grpc/_cython/_cygrpc/fork_posix.pyx.pxi":102
 * 
 * class ForkManagedThread(object):
 *     def __init__(self, target, args=()):             # <<<<<<<<<<<<<<
 *         if _GRPC_ENABLE_FORK_SUPPORT:
 *             def managed_target(*args):
 */
struct __pyx_obj_4grpc_7_cython_6cygrpc___pyx_scope_struct_9___init__ {
  PyObject_HEAD
  PyObject *__pyx_v_target;
};



/* "src/python/grpcio/grpc/_cython/_cygrpc/arguments.pyx.pxi":34
 * 
 * 
 * cdef class _ChannelArg:             # <<<<<<<<<<<<<<
 * 
 *   cdef void c(self, argument, _VTable vtable, references) except *:
 */

struct __pyx_vtabstruct_4grpc_7_cython_6cygrpc__ChannelArg {
  void (*c)(struct __pyx_obj_4grpc_7_cython_6cygrpc__ChannelArg *, PyObject *, struct __pyx_obj_4grpc_7_cython_6cygrpc__VTable *, PyObject *);
};
static struct __pyx_vtabstruct_4grpc_7_cython_6cygrpc__ChannelArg *__pyx_vtabptr_4grpc_7_cython_6cygrpc__ChannelArg;


/* "src/python/grpcio/grpc/_cython/_cygrpc/arguments.pyx.pxi":66
 * 
 * 
 * cdef class _ChannelArgs:             # <<<<<<<<<<<<<<
 * 
 *   def __cinit__(self, arguments, _VTable vtable not None):
 */

struct __pyx_vtabstruct_4grpc_7_cython_6cygrpc__ChannelArgs {
  grpc_channel_args *(*c_args)(struct __pyx_obj_4grpc_7_cython_6cygrpc__ChannelArgs *);
};
static struct __pyx_vtabstruct_4grpc_7_cython_6cygrpc__ChannelArgs *__pyx_vtabptr_4grpc_7_cython_6cygrpc__ChannelArgs;


/* "src/python/grpcio/grpc/_cython/_cygrpc/credentials.pyx.pxi":34
 * 
 * 
 * cdef class CallCredentials:             # <<<<<<<<<<<<<<
 * 
 *   cdef grpc_call_credentials *c(self) except *:
 */

struct __pyx_vtabstruct_4grpc_7_cython_6cygrpc_CallCredentials {
  grpc_call_credentials *(*c)(struct __pyx_obj_4grpc_7_cython_6cygrpc_CallCredentials *);
};
static struct __pyx_vtabstruct_4grpc_7_cython_6cygrpc_CallCredentials *__pyx_vtabptr_4grpc_7_cython_6cygrpc_CallCredentials;


/* "src/python/grpcio/grpc/_cython/_cygrpc/credentials.pyx.pxi":65
 * 
 * 
 * cdef class MetadataPluginCallCredentials(CallCredentials):             # <<<<<<<<<<<<<<
 * 
 *   def __cinit__(self, metadata_plugin, name):
 */

struct __pyx_vtabstruct_4grpc_7_cython_6cygrpc_MetadataPluginCallCredentials {
  struct __pyx_vtabstruct_4grpc_7_cython_6cygrpc_CallCredentials __pyx_base;
};
static struct __pyx_vtabstruct_4grpc_7_cython_6cygrpc_MetadataPluginCallCredentials *__pyx_vtabptr_4grpc_7_cython_6cygrpc_MetadataPluginCallCredentials;


/* "src/python/grpcio/grpc/_cython/_cygrpc/credentials.pyx.pxi":99
 * 
 * 
 * cdef class CompositeCallCredentials(CallCredentials):             # <<<<<<<<<<<<<<
 * 
 *   def __cinit__(self, call_credentialses):
 */

struct __pyx_vtabstruct_4grpc_7_cython_6cygrpc_CompositeCallCredentials {
  struct __pyx_vtabstruct_4grpc_7_cython_6cygrpc_CallCredentials __pyx_base;
};
static struct __pyx_vtabstruct_4grpc_7_cython_6cygrpc_CompositeCallCredentials *__pyx_vtabptr_4grpc_7_cython_6cygrpc_CompositeCallCredentials;


/* "src/python/grpcio/grpc/_cython/_cygrpc/credentials.pyx.pxi":108
 * 
 * 
 * cdef class ChannelCredentials:             # <<<<<<<<<<<<<<
 * 
 *   cdef grpc_channel_credentials *c(self) except *:
 */

struct __pyx_vtabstruct_4grpc_7_cython_6cygrpc_ChannelCredentials {
  grpc_channel_credentials *(*c)(struct __pyx_obj_4grpc_7_cython_6cygrpc_ChannelCredentials *);
};
static struct __pyx_vtabstruct_4grpc_7_cython_6cygrpc_ChannelCredentials *__pyx_vtabptr_4grpc_7_cython_6cygrpc_ChannelCredentials;


/* "src/python/grpcio/grpc/_cython/_cygrpc/credentials.pyx.pxi":129
 * 
 * 
 * cdef class SSLChannelCredentials(ChannelCredentials):             # <<<<<<<<<<<<<<
 * 
 *   def __cinit__(self, pem_root_certificates, private_key, certificate_chain):
 */

struct __pyx_vtabstruct_4grpc_7_cython_6cygrpc_SSLChannelCredentials {
  struct __pyx_vtabstruct_4grpc_7_cython_6cygrpc_ChannelCredentials __pyx_base;
};
static struct __pyx_vtabstruct_4grpc_7_cython_6cygrpc_SSLChannelCredentials *__pyx_vtabptr_4grpc_7_cython_6cygrpc_SSLChannelCredentials;


/* "src/python/grpcio/grpc/_cython/_cygrpc/credentials.pyx.pxi":161
 * 
 * 
 * cdef class CompositeChannelCredentials(ChannelCredentials):             # <<<<<<<<<<<<<<
 * 
 *   def __cinit__(self, call_credentialses, channel_credentials):
 */

struct __pyx_vtabstruct_4grpc_7_cython_6cygrpc_CompositeChannelCredentials {
  struct __pyx_vtabstruct_4grpc_7_cython_6cygrpc_ChannelCredentials __pyx_base;
};
static struct __pyx_vtabstruct_4grpc_7_cython_6cygrpc_CompositeChannelCredentials *__pyx_vtabptr_4grpc_7_cython_6cygrpc_CompositeChannelCredentials;


/* "src/python/grpcio/grpc/_cython/_cygrpc/completion_queue.pyx.pxi":71
 * 
 * 
 * cdef class CompletionQueue:             # <<<<<<<<<<<<<<
 * 
 *   def __cinit__(self, shutdown_cq=False):
 */

struct __pyx_vtabstruct_4grpc_7_cython_6cygrpc_CompletionQueue {
  PyObject *(*_interpret_event)(struct __pyx_obj_4grpc_7_cython_6cygrpc_CompletionQueue *, grpc_event);
};
static struct __pyx_vtabstruct_4grpc_7_cython_6cygrpc_CompletionQueue *__pyx_vtabptr_4grpc_7_cython_6cygrpc_CompletionQueue;


/* "src/python/grpcio/grpc/_cython/_cygrpc/operation.pyx.pxi":16
 * 
 * 
 * cdef class Operation:             # <<<<<<<<<<<<<<
 * 
 *   cdef void c(self) except *:
 */

struct __pyx_vtabstruct_4grpc_7_cython_6cygrpc_Operation {
  void (*c)(struct __pyx_obj_4grpc_7_cython_6cygrpc_Operation *);
  void (*un_c)(struct __pyx_obj_4grpc_7_cython_6cygrpc_Operation *);
};
static struct __pyx_vtabstruct_4grpc_7_cython_6cygrpc_Operation *__pyx_vtabptr_4grpc_7_cython_6cygrpc_Operation;


/* "src/python/grpcio/grpc/_cython/_cygrpc/operation.pyx.pxi":25
 * 
 * 
 * cdef class SendInitialMetadataOperation(Operation):             # <<<<<<<<<<<<<<
 * 
 *   def __cinit__(self, initial_metadata, flags):
 */

struct __pyx_vtabstruct_4grpc_7_cython_6cygrpc_SendInitialMetadataOperation {
  struct __pyx_vtabstruct_4grpc_7_cython_6cygrpc_Operation __pyx_base;
};
static struct __pyx_vtabstruct_4grpc_7_cython_6cygrpc_SendInitialMetadataOperation *__pyx_vtabptr_4grpc_7_cython_6cygrpc_SendInitialMetadataOperation;


/* "src/python/grpcio/grpc/_cython/_cygrpc/operation.pyx.pxi":49
 * 
 * 
 * cdef class SendMessageOperation(Operation):             # <<<<<<<<<<<<<<
 * 
 *   def __cinit__(self, bytes message, int flags):
 */

struct __pyx_vtabstruct_4grpc_7_cython_6cygrpc_SendMessageOperation {
  struct __pyx_vtabstruct_4grpc_7_cython_6cygrpc_Operation __pyx_base;
};
static struct __pyx_vtabstruct_4grpc_7_cython_6cygrpc_SendMessageOperation *__pyx_vtabptr_4grpc_7_cython_6cygrpc_SendMessageOperation;


/* "src/python/grpcio/grpc/_cython/_cygrpc/operation.pyx.pxi":72
 * 
 * 
 * cdef class SendCloseFromClientOperation(Operation):             # <<<<<<<<<<<<<<
 * 
 *   def __cinit__(self, int flags):
 */

struct __pyx_vtabstruct_4grpc_7_cython_6cygrpc_SendCloseFromClientOperation {
  struct __pyx_vtabstruct_4grpc_7_cython_6cygrpc_Operation __pyx_base;
};
static struct __pyx_vtabstruct_4grpc_7_cython_6cygrpc_SendCloseFromClientOperation *__pyx_vtabptr_4grpc_7_cython_6cygrpc_SendCloseFromClientOperation;


/* "src/python/grpcio/grpc/_cython/_cygrpc/operation.pyx.pxi":88
 * 
 * 
 * cdef class SendStatusFromServerOperation(Operation):             # <<<<<<<<<<<<<<
 * 
 *   def __cinit__(self, trailing_metadata, code, object details, int flags):
 */

struct __pyx_vtabstruct_4grpc_7_cython_6cygrpc_SendStatusFromServerOperation {
  struct __pyx_vtabstruct_4grpc_7_cython_6cygrpc_Operation __pyx_base;
};
static struct __pyx_vtabstruct_4grpc_7_cython_6cygrpc_SendStatusFromServerOperation *__pyx_vtabptr_4grpc_7_cython_6cygrpc_SendStatusFromServerOperation;


/* "src/python/grpcio/grpc/_cython/_cygrpc/operation.pyx.pxi":119
 * 
 * 
 * cdef class ReceiveInitialMetadataOperation(Operation):             # <<<<<<<<<<<<<<
 * 
 *   def __cinit__(self, flags):
 */

struct __pyx_vtabstruct_4grpc_7_cython_6cygrpc_ReceiveInitialMetadataOperation {
  struct __pyx_vtabstruct_4grpc_7_cython_6cygrpc_Operation __pyx_base;
};
static struct __pyx_vtabstruct_4grpc_7_cython_6cygrpc_ReceiveInitialMetadataOperation *__pyx_vtabptr_4grpc_7_cython_6cygrpc_ReceiveInitialMetadataOperation;


/* "src/python/grpcio/grpc/_cython/_cygrpc/operation.pyx.pxi":142
 * 
 * 
 * cdef class ReceiveMessageOperation(Operation):             # <<<<<<<<<<<<<<
 * 
 *   def __cinit__(self, flags):
 */

struct __pyx_vtabstruct_4grpc_7_cython_6cygrpc_ReceiveMessageOperation {
  struct __pyx_vtabstruct_4grpc_7_cython_6cygrpc_Operation __pyx_base;
};
static struct __pyx_vtabstruct_4grpc_7_cython_6cygrpc_ReceiveMessageOperation *__pyx_vtabptr_4grpc_7_cython_6cygrpc_ReceiveMessageOperation;


/* "src/python/grpcio/grpc/_cython/_cygrpc/operation.pyx.pxi":184
 * 
 * 
 * cdef class ReceiveStatusOnClientOperation(Operation):             # <<<<<<<<<<<<<<
 * 
 *   def __cinit__(self, flags):
 */

struct __pyx_vtabstruct_4grpc_7_cython_6cygrpc_ReceiveStatusOnClientOperation {
  struct __pyx_vtabstruct_4grpc_7_cython_6cygrpc_Operation __pyx_base;
};
static struct __pyx_vtabstruct_4grpc_7_cython_6cygrpc_ReceiveStatusOnClientOperation *__pyx_vtabptr_4grpc_7_cython_6cygrpc_ReceiveStatusOnClientOperation;


/* "src/python/grpcio/grpc/_cython/_cygrpc/operation.pyx.pxi":230
 * 
 * 
 * cdef class ReceiveCloseOnServerOperation(Operation):             # <<<<<<<<<<<<<<
 * 
 *   def __cinit__(self, flags):
 */

struct __pyx_vtabstruct_4grpc_7_cython_6cygrpc_ReceiveCloseOnServerOperation {
  struct __pyx_vtabstruct_4grpc_7_cython_6cygrpc_Operation __pyx_base;
};
static struct __pyx_vtabstruct_4grpc_7_cython_6cygrpc_ReceiveCloseOnServerOperation *__pyx_vtabptr_4grpc_7_cython_6cygrpc_ReceiveCloseOnServerOperation;


/* "src/python/grpcio/grpc/_cython/_cygrpc/server.pyx.pxi":24
 * 
 * 
 * cdef class Server:             # <<<<<<<<<<<<<<
 * 
 *   def __cinit__(self, object arguments):
 */

struct __pyx_vtabstruct_4grpc_7_cython_6cygrpc_Server {
  PyObject *(*_c_shutdown)(struct __pyx_obj_4grpc_7_cython_6cygrpc_Server *, struct __pyx_obj_4grpc_7_cython_6cygrpc_CompletionQueue *, PyObject *);
  PyObject *(*notify_shutdown_complete)(struct __pyx_obj_4grpc_7_cython_6cygrpc_Server *);
};
static struct __pyx_vtabstruct_4grpc_7_cython_6cygrpc_Server *__pyx_vtabptr_4grpc_7_cython_6cygrpc_Server;


/* "src/python/grpcio/grpc/_cython/_cygrpc/tag.pyx.pxi":16
 * 
 * 
 * cdef class _Tag:             # <<<<<<<<<<<<<<
 * 
 *   cdef object event(self, grpc_event c_event):
 */

struct __pyx_vtabstruct_4grpc_7_cython_6cygrpc__Tag {
  PyObject *(*event)(struct __pyx_obj_4grpc_7_cython_6cygrpc__Tag *, grpc_event);
};
static struct __pyx_vtabstruct_4grpc_7_cython_6cygrpc__Tag *__pyx_vtabptr_4grpc_7_cython_6cygrpc__Tag;


/* "src/python/grpcio/grpc/_cython/_cygrpc/tag.pyx.pxi":22
 * 
 * 
 * cdef class _ConnectivityTag(_Tag):             # <<<<<<<<<<<<<<
 * 
 *   def __cinit__(self, user_tag):
 */

struct __pyx_vtabstruct_4grpc_7_cython_6cygrpc__ConnectivityTag {
  struct __pyx_vtabstruct_4grpc_7_cython_6cygrpc__Tag __pyx_base;
  struct __pyx_obj_4grpc_7_cython_6cygrpc_ConnectivityEvent *(*event)(struct __pyx_obj_4grpc_7_cython_6cygrpc__ConnectivityTag *, grpc_event);
};
static struct __pyx_vtabstruct_4grpc_7_cython_6cygrpc__ConnectivityTag *__pyx_vtabptr_4grpc_7_cython_6cygrpc__ConnectivityTag;


/* "src/python/grpcio/grpc/_cython/_cygrpc/tag.pyx.pxi":31
 * 
 * 
 * cdef class _RequestCallTag(_Tag):             # <<<<<<<<<<<<<<
 * 
 *   def __cinit__(self, user_tag, _VTable vtable not None):
 */

struct __pyx_vtabstruct_4grpc_7_cython_6cygrpc__RequestCallTag {
  struct __pyx_vtabstruct_4grpc_7_cython_6cygrpc__Tag __pyx_base;
  void (*prepare)(struct __pyx_obj_4grpc_7_cython_6cygrpc__RequestCallTag *);
  struct __pyx_obj_4grpc_7_cython_6cygrpc_RequestCallEvent *(*event)(struct __pyx_obj_4grpc_7_cython_6cygrpc__RequestCallTag *, grpc_event);
};
static struct __pyx_vtabstruct_4grpc_7_cython_6cygrpc__RequestCallTag *__pyx_vtabptr_4grpc_7_cython_6cygrpc__RequestCallTag;


/* "src/python/grpcio/grpc/_cython/_cygrpc/tag.pyx.pxi":52
 * 
 * 
 * cdef class _BatchOperationTag:             # <<<<<<<<<<<<<<
 * 
 *   def __cinit__(self, user_tag, operations, call):
 */

struct __pyx_vtabstruct_4grpc_7_cython_6cygrpc__BatchOperationTag {
  struct __pyx_vtabstruct_4grpc_7_cython_6cygrpc__Tag __pyx_base;
  void (*prepare)(struct __pyx_obj_4grpc_7_cython_6cygrpc__BatchOperationTag *);
  struct __pyx_obj_4grpc_7_cython_6cygrpc_BatchOperationEvent *(*event)(struct __pyx_obj_4grpc_7_cython_6cygrpc__BatchOperationTag *, grpc_event);
};
static struct __pyx_vtabstruct_4grpc_7_cython_6cygrpc__BatchOperationTag *__pyx_vtabptr_4grpc_7_cython_6cygrpc__BatchOperationTag;


/* "src/python/grpcio/grpc/_cython/_cygrpc/tag.pyx.pxi":81
 * 
 * 
 * cdef class _ServerShutdownTag(_Tag):             # <<<<<<<<<<<<<<
 * 
 *   def __cinit__(self, user_tag, shutting_down_server):
 */

struct __pyx_vtabstruct_4grpc_7_cython_6cygrpc__ServerShutdownTag {
  struct __pyx_vtabstruct_4grpc_7_cython_6cygrpc__Tag __pyx_base;
  struct __pyx_obj_4grpc_7_cython_6cygrpc_ServerShutdownEvent *(*event)(struct __pyx_obj_4grpc_7_cython_6cygrpc__ServerShutdownTag *, grpc_event);
};
static struct __pyx_vtabstruct_4grpc_7_cython_6cygrpc__ServerShutdownTag *__pyx_vtabptr_4grpc_7_cython_6cygrpc__ServerShutdownTag;

/* --- Runtime support code (head) --- */
/* Refnanny.proto */
#ifndef CYTHON_REFNANNY
  #define CYTHON_REFNANNY 0
#endif
#if CYTHON_REFNANNY
  typedef struct {
    void (*INCREF)(void*, PyObject*, int);
    void (*DECREF)(void*, PyObject*, int);
    void (*GOTREF)(void*, PyObject*, int);
    void (*GIVEREF)(void*, PyObject*, int);
    void* (*SetupContext)(const char*, int, const char*);
    void (*FinishContext)(void**);
  } __Pyx_RefNannyAPIStruct;
  static __Pyx_RefNannyAPIStruct *__Pyx_RefNanny = NULL;
  static __Pyx_RefNannyAPIStruct *__Pyx_RefNannyImportAPI(const char *modname);
  #define __Pyx_RefNannyDeclarations void *__pyx_refnanny = NULL;
#ifdef WITH_THREAD
  #define __Pyx_RefNannySetupContext(name, acquire_gil)\
          if (acquire_gil) {\
              PyGILState_STATE __pyx_gilstate_save = PyGILState_Ensure();\
              __pyx_refnanny = __Pyx_RefNanny->SetupContext((name), __LINE__, __FILE__);\
              PyGILState_Release(__pyx_gilstate_save);\
          } else {\
              __pyx_refnanny = __Pyx_RefNanny->SetupContext((name), __LINE__, __FILE__);\
          }
#else
  #define __Pyx_RefNannySetupContext(name, acquire_gil)\
          __pyx_refnanny = __Pyx_RefNanny->SetupContext((name), __LINE__, __FILE__)
#endif
  #define __Pyx_RefNannyFinishContext()\
          __Pyx_RefNanny->FinishContext(&__pyx_refnanny)
  #define __Pyx_INCREF(r)  __Pyx_RefNanny->INCREF(__pyx_refnanny, (PyObject *)(r), __LINE__)
  #define __Pyx_DECREF(r)  __Pyx_RefNanny->DECREF(__pyx_refnanny, (PyObject *)(r), __LINE__)
  #define __Pyx_GOTREF(r)  __Pyx_RefNanny->GOTREF(__pyx_refnanny, (PyObject *)(r), __LINE__)
  #define __Pyx_GIVEREF(r) __Pyx_RefNanny->GIVEREF(__pyx_refnanny, (PyObject *)(r), __LINE__)
  #define __Pyx_XINCREF(r)  do { if((r) != NULL) {__Pyx_INCREF(r); }} while(0)
  #define __Pyx_XDECREF(r)  do { if((r) != NULL) {__Pyx_DECREF(r); }} while(0)
  #define __Pyx_XGOTREF(r)  do { if((r) != NULL) {__Pyx_GOTREF(r); }} while(0)
  #define __Pyx_XGIVEREF(r) do { if((r) != NULL) {__Pyx_GIVEREF(r);}} while(0)
#else
  #define __Pyx_RefNannyDeclarations
  #define __Pyx_RefNannySetupContext(name, acquire_gil)
  #define __Pyx_RefNannyFinishContext()
  #define __Pyx_INCREF(r) Py_INCREF(r)
  #define __Pyx_DECREF(r) Py_DECREF(r)
  #define __Pyx_GOTREF(r)
  #define __Pyx_GIVEREF(r)
  #define __Pyx_XINCREF(r) Py_XINCREF(r)
  #define __Pyx_XDECREF(r) Py_XDECREF(r)
  #define __Pyx_XGOTREF(r)
  #define __Pyx_XGIVEREF(r)
#endif
#define __Pyx_XDECREF_SET(r, v) do {\
        PyObject *tmp = (PyObject *) r;\
        r = v; __Pyx_XDECREF(tmp);\
    } while (0)
#define __Pyx_DECREF_SET(r, v) do {\
        PyObject *tmp = (PyObject *) r;\
        r = v; __Pyx_DECREF(tmp);\
    } while (0)
#define __Pyx_CLEAR(r)    do { PyObject* tmp = ((PyObject*)(r)); r = NULL; __Pyx_DECREF(tmp);} while(0)
#define __Pyx_XCLEAR(r)   do { if((r) != NULL) {PyObject* tmp = ((PyObject*)(r)); r = NULL; __Pyx_DECREF(tmp);}} while(0)

/* PyObjectGetAttrStr.proto */
#if CYTHON_USE_TYPE_SLOTS
static CYTHON_INLINE PyObject* __Pyx_PyObject_GetAttrStr(PyObject* obj, PyObject* attr_name);
#else
#define __Pyx_PyObject_GetAttrStr(o,n) PyObject_GetAttr(o,n)
#endif

/* GetBuiltinName.proto */
static PyObject *__Pyx_GetBuiltinName(PyObject *name);

/* PyObjectCall.proto */
#if CYTHON_COMPILING_IN_CPYTHON
static CYTHON_INLINE PyObject* __Pyx_PyObject_Call(PyObject *func, PyObject *arg, PyObject *kw);
#else
#define __Pyx_PyObject_Call(func, arg, kw) PyObject_Call(func, arg, kw)
#endif

/* PyCFunctionFastCall.proto */
#if CYTHON_FAST_PYCCALL
static CYTHON_INLINE PyObject *__Pyx_PyCFunction_FastCall(PyObject *func, PyObject **args, Py_ssize_t nargs);
#else
#define __Pyx_PyCFunction_FastCall(func, args, nargs)  (assert(0), NULL)
#endif

/* PyFunctionFastCall.proto */
#if CYTHON_FAST_PYCALL
#define __Pyx_PyFunction_FastCall(func, args, nargs)\
    __Pyx_PyFunction_FastCallDict((func), (args), (nargs), NULL)
#if 1 || PY_VERSION_HEX < 0x030600B1
static PyObject *__Pyx_PyFunction_FastCallDict(PyObject *func, PyObject **args, int nargs, PyObject *kwargs);
#else
#define __Pyx_PyFunction_FastCallDict(func, args, nargs, kwargs) _PyFunction_FastCallDict(func, args, nargs, kwargs)
#endif
#endif

/* PyObjectCallMethO.proto */
#if CYTHON_COMPILING_IN_CPYTHON
static CYTHON_INLINE PyObject* __Pyx_PyObject_CallMethO(PyObject *func, PyObject *arg);
#endif

/* PyObjectCallOneArg.proto */
static CYTHON_INLINE PyObject* __Pyx_PyObject_CallOneArg(PyObject *func, PyObject *arg);

/* PyThreadStateGet.proto */
#if CYTHON_FAST_THREAD_STATE
#define __Pyx_PyThreadState_declare  PyThreadState *__pyx_tstate;
#define __Pyx_PyThreadState_assign  __pyx_tstate = __Pyx_PyThreadState_Current;
#define __Pyx_PyErr_Occurred()  __pyx_tstate->curexc_type
#else
#define __Pyx_PyThreadState_declare
#define __Pyx_PyThreadState_assign
#define __Pyx_PyErr_Occurred()  PyErr_Occurred()
#endif

/* PyErrFetchRestore.proto */
#if CYTHON_FAST_THREAD_STATE
#define __Pyx_PyErr_Clear() __Pyx_ErrRestore(NULL, NULL, NULL)
#define __Pyx_ErrRestoreWithState(type, value, tb)  __Pyx_ErrRestoreInState(PyThreadState_GET(), type, value, tb)
#define __Pyx_ErrFetchWithState(type, value, tb)    __Pyx_ErrFetchInState(PyThreadState_GET(), type, value, tb)
#define __Pyx_ErrRestore(type, value, tb)  __Pyx_ErrRestoreInState(__pyx_tstate, type, value, tb)
#define __Pyx_ErrFetch(type, value, tb)    __Pyx_ErrFetchInState(__pyx_tstate, type, value, tb)
static CYTHON_INLINE void __Pyx_ErrRestoreInState(PyThreadState *tstate, PyObject *type, PyObject *value, PyObject *tb);
static CYTHON_INLINE void __Pyx_ErrFetchInState(PyThreadState *tstate, PyObject **type, PyObject **value, PyObject **tb);
#if CYTHON_COMPILING_IN_CPYTHON
#define __Pyx_PyErr_SetNone(exc) (Py_INCREF(exc), __Pyx_ErrRestore((exc), NULL, NULL))
#else
#define __Pyx_PyErr_SetNone(exc) PyErr_SetNone(exc)
#endif
#else
#define __Pyx_PyErr_Clear() PyErr_Clear()
#define __Pyx_PyErr_SetNone(exc) PyErr_SetNone(exc)
#define __Pyx_ErrRestoreWithState(type, value, tb)  PyErr_Restore(type, value, tb)
#define __Pyx_ErrFetchWithState(type, value, tb)  PyErr_Fetch(type, value, tb)
#define __Pyx_ErrRestoreInState(tstate, type, value, tb)  PyErr_Restore(type, value, tb)
#define __Pyx_ErrFetchInState(tstate, type, value, tb)  PyErr_Fetch(type, value, tb)
#define __Pyx_ErrRestore(type, value, tb)  PyErr_Restore(type, value, tb)
#define __Pyx_ErrFetch(type, value, tb)  PyErr_Fetch(type, value, tb)
#endif

/* RaiseException.proto */
static void __Pyx_Raise(PyObject *type, PyObject *value, PyObject *tb, PyObject *cause);

/* decode_c_string_utf16.proto */
static CYTHON_INLINE PyObject *__Pyx_PyUnicode_DecodeUTF16(const char *s, Py_ssize_t size, const char *errors) {
    int byteorder = 0;
    return PyUnicode_DecodeUTF16(s, size, errors, &byteorder);
}
static CYTHON_INLINE PyObject *__Pyx_PyUnicode_DecodeUTF16LE(const char *s, Py_ssize_t size, const char *errors) {
    int byteorder = -1;
    return PyUnicode_DecodeUTF16(s, size, errors, &byteorder);
}
static CYTHON_INLINE PyObject *__Pyx_PyUnicode_DecodeUTF16BE(const char *s, Py_ssize_t size, const char *errors) {
    int byteorder = 1;
    return PyUnicode_DecodeUTF16(s, size, errors, &byteorder);
}

/* decode_c_bytes.proto */
static CYTHON_INLINE PyObject* __Pyx_decode_c_bytes(
         const char* cstring, Py_ssize_t length, Py_ssize_t start, Py_ssize_t stop,
         const char* encoding, const char* errors,
         PyObject* (*decode_func)(const char *s, Py_ssize_t size, const char *errors));

/* decode_bytes.proto */
static CYTHON_INLINE PyObject* __Pyx_decode_bytes(
         PyObject* string, Py_ssize_t start, Py_ssize_t stop,
         const char* encoding, const char* errors,
         PyObject* (*decode_func)(const char *s, Py_ssize_t size, const char *errors)) {
    return __Pyx_decode_c_bytes(
        PyBytes_AS_STRING(string), PyBytes_GET_SIZE(string),
        start, stop, encoding, errors, decode_func);
}

/* SaveResetException.proto */
#if CYTHON_FAST_THREAD_STATE
#define __Pyx_ExceptionSave(type, value, tb)  __Pyx__ExceptionSave(__pyx_tstate, type, value, tb)
static CYTHON_INLINE void __Pyx__ExceptionSave(PyThreadState *tstate, PyObject **type, PyObject **value, PyObject **tb);
#define __Pyx_ExceptionReset(type, value, tb)  __Pyx__ExceptionReset(__pyx_tstate, type, value, tb)
static CYTHON_INLINE void __Pyx__ExceptionReset(PyThreadState *tstate, PyObject *type, PyObject *value, PyObject *tb);
#else
#define __Pyx_ExceptionSave(type, value, tb)   PyErr_GetExcInfo(type, value, tb)
#define __Pyx_ExceptionReset(type, value, tb)  PyErr_SetExcInfo(type, value, tb)
#endif

/* PyErrExceptionMatches.proto */
#if CYTHON_FAST_THREAD_STATE
#define __Pyx_PyErr_ExceptionMatches(err) __Pyx_PyErr_ExceptionMatchesInState(__pyx_tstate, err)
static CYTHON_INLINE int __Pyx_PyErr_ExceptionMatchesInState(PyThreadState* tstate, PyObject* err);
#else
#define __Pyx_PyErr_ExceptionMatches(err)  PyErr_ExceptionMatches(err)
#endif

/* GetException.proto */
#if CYTHON_FAST_THREAD_STATE
#define __Pyx_GetException(type, value, tb)  __Pyx__GetException(__pyx_tstate, type, value, tb)
static int __Pyx__GetException(PyThreadState *tstate, PyObject **type, PyObject **value, PyObject **tb);
#else
static int __Pyx_GetException(PyObject **type, PyObject **value, PyObject **tb);
#endif

/* GetModuleGlobalName.proto */
static CYTHON_INLINE PyObject *__Pyx_GetModuleGlobalName(PyObject *name);

/* PyObjectCallNoArg.proto */
#if CYTHON_COMPILING_IN_CPYTHON
static CYTHON_INLINE PyObject* __Pyx_PyObject_CallNoArg(PyObject *func);
#else
#define __Pyx_PyObject_CallNoArg(func) __Pyx_PyObject_Call(func, __pyx_empty_tuple, NULL)
#endif

/* GetItemInt.proto */
#define __Pyx_GetItemInt(o, i, type, is_signed, to_py_func, is_list, wraparound, boundscheck)\
    (__Pyx_fits_Py_ssize_t(i, type, is_signed) ?\
    __Pyx_GetItemInt_Fast(o, (Py_ssize_t)i, is_list, wraparound, boundscheck) :\
    (is_list ? (PyErr_SetString(PyExc_IndexError, "list index out of range"), (PyObject*)NULL) :\
               __Pyx_GetItemInt_Generic(o, to_py_func(i))))
#define __Pyx_GetItemInt_List(o, i, type, is_signed, to_py_func, is_list, wraparound, boundscheck)\
    (__Pyx_fits_Py_ssize_t(i, type, is_signed) ?\
    __Pyx_GetItemInt_List_Fast(o, (Py_ssize_t)i, wraparound, boundscheck) :\
    (PyErr_SetString(PyExc_IndexError, "list index out of range"), (PyObject*)NULL))
static CYTHON_INLINE PyObject *__Pyx_GetItemInt_List_Fast(PyObject *o, Py_ssize_t i,
                                                              int wraparound, int boundscheck);
#define __Pyx_GetItemInt_Tuple(o, i, type, is_signed, to_py_func, is_list, wraparound, boundscheck)\
    (__Pyx_fits_Py_ssize_t(i, type, is_signed) ?\
    __Pyx_GetItemInt_Tuple_Fast(o, (Py_ssize_t)i, wraparound, boundscheck) :\
    (PyErr_SetString(PyExc_IndexError, "tuple index out of range"), (PyObject*)NULL))
static CYTHON_INLINE PyObject *__Pyx_GetItemInt_Tuple_Fast(PyObject *o, Py_ssize_t i,
                                                              int wraparound, int boundscheck);
static PyObject *__Pyx_GetItemInt_Generic(PyObject *o, PyObject* j);
static CYTHON_INLINE PyObject *__Pyx_GetItemInt_Fast(PyObject *o, Py_ssize_t i,
                                                     int is_list, int wraparound, int boundscheck);

/* ExtTypeTest.proto */
static CYTHON_INLINE int __Pyx_TypeTest(PyObject *obj, PyTypeObject *type);

/* WriteUnraisableException.proto */
static void __Pyx_WriteUnraisable(const char *name, int clineno,
                                  int lineno, const char *filename,
                                  int full_traceback, int nogil);

/* RaiseTooManyValuesToUnpack.proto */
static CYTHON_INLINE void __Pyx_RaiseTooManyValuesError(Py_ssize_t expected);

/* RaiseNeedMoreValuesToUnpack.proto */
static CYTHON_INLINE void __Pyx_RaiseNeedMoreValuesError(Py_ssize_t index);

/* IterFinish.proto */
static CYTHON_INLINE int __Pyx_IterFinish(void);

/* UnpackItemEndCheck.proto */
static int __Pyx_IternextUnpackEndCheck(PyObject *retval, Py_ssize_t expected);

/* ListAppend.proto */
#if CYTHON_USE_PYLIST_INTERNALS && CYTHON_ASSUME_SAFE_MACROS
static CYTHON_INLINE int __Pyx_PyList_Append(PyObject* list, PyObject* x) {
    PyListObject* L = (PyListObject*) list;
    Py_ssize_t len = Py_SIZE(list);
    if (likely(L->allocated > len) & likely(len > (L->allocated >> 1))) {
        Py_INCREF(x);
        PyList_SET_ITEM(list, len, x);
        Py_SIZE(list) = len+1;
        return 0;
    }
    return PyList_Append(list, x);
}
#else
#define __Pyx_PyList_Append(L,x) PyList_Append(L,x)
#endif

/* PyObjectCallMethod1.proto */
static PyObject* __Pyx_PyObject_CallMethod1(PyObject* obj, PyObject* method_name, PyObject* arg);
static PyObject* __Pyx__PyObject_CallMethod1(PyObject* method, PyObject* arg);

/* append.proto */
static CYTHON_INLINE int __Pyx_PyObject_Append(PyObject* L, PyObject* x);

/* GetAttr.proto */
static CYTHON_INLINE PyObject *__Pyx_GetAttr(PyObject *, PyObject *);

/* HasAttr.proto */
static CYTHON_INLINE int __Pyx_HasAttr(PyObject *, PyObject *);

/* RaiseArgTupleInvalid.proto */
static void __Pyx_RaiseArgtupleInvalid(const char* func_name, int exact,
    Py_ssize_t num_min, Py_ssize_t num_max, Py_ssize_t num_found);

/* RaiseDoubleKeywords.proto */
static void __Pyx_RaiseDoubleKeywordsError(const char* func_name, PyObject* kw_name);

/* ParseKeywords.proto */
static int __Pyx_ParseOptionalKeywords(PyObject *kwds, PyObject **argnames[],\
    PyObject *kwds2, PyObject *values[], Py_ssize_t num_pos_args,\
    const char* function_name);

/* ArgTypeTest.proto */
#define __Pyx_ArgTypeTest(obj, type, none_allowed, name, exact)\
    ((likely((Py_TYPE(obj) == type) | (none_allowed && (obj == Py_None)))) ? 1 :\
        __Pyx__ArgTypeTest(obj, type, name, exact))
static int __Pyx__ArgTypeTest(PyObject *obj, PyTypeObject *type, const char *name, int exact);

/* PyIntBinop.proto */
#if !CYTHON_COMPILING_IN_PYPY
static PyObject* __Pyx_PyInt_AddObjC(PyObject *op1, PyObject *op2, long intval, int inplace);
#else
#define __Pyx_PyInt_AddObjC(op1, op2, intval, inplace)\
    (inplace ? PyNumber_InPlaceAdd(op1, op2) : PyNumber_Add(op1, op2))
#endif

/* KeywordStringCheck.proto */
static int __Pyx_CheckKeywordStrings(PyObject *kwdict, const char* function_name, int kw_allowed);

/* PyObjectLookupSpecial.proto */
#if CYTHON_USE_PYTYPE_LOOKUP && CYTHON_USE_TYPE_SLOTS
static CYTHON_INLINE PyObject* __Pyx_PyObject_LookupSpecial(PyObject* obj, PyObject* attr_name) {
    PyObject *res;
    PyTypeObject *tp = Py_TYPE(obj);
#if PY_MAJOR_VERSION < 3
    if (unlikely(PyInstance_Check(obj)))
        return __Pyx_PyObject_GetAttrStr(obj, attr_name);
#endif
    res = _PyType_Lookup(tp, attr_name);
    if (likely(res)) {
        descrgetfunc f = Py_TYPE(res)->tp_descr_get;
        if (!f) {
            Py_INCREF(res);
        } else {
            res = f(res, obj, (PyObject *)tp);
        }
    } else {
        PyErr_SetObject(PyExc_AttributeError, attr_name);
    }
    return res;
}
#else
#define __Pyx_PyObject_LookupSpecial(o,n) __Pyx_PyObject_GetAttrStr(o,n)
#endif

/* RaiseNoneIterError.proto */
static CYTHON_INLINE void __Pyx_RaiseNoneNotIterableError(void);

/* py_dict_pop.proto */
static CYTHON_INLINE PyObject *__Pyx_PyDict_Pop(PyObject *d, PyObject *key, PyObject *default_value);

/* UnpackUnboundCMethod.proto */
typedef struct {
    PyObject *type;
    PyObject **method_name;
    PyCFunction func;
    PyObject *method;
    int flag;
} __Pyx_CachedCFunction;

/* CallUnboundCMethod2.proto */
static PyObject* __Pyx__CallUnboundCMethod2(__Pyx_CachedCFunction* cfunc, PyObject* self, PyObject* arg1, PyObject* arg2);
#if CYTHON_COMPILING_IN_CPYTHON && PY_VERSION_HEX >= 0x030600B1
static CYTHON_INLINE PyObject *__Pyx_CallUnboundCMethod2(__Pyx_CachedCFunction *cfunc, PyObject *self, PyObject *arg1, PyObject *arg2);
#else
#define __Pyx_CallUnboundCMethod2(cfunc, self, arg1, arg2)  __Pyx__CallUnboundCMethod2(cfunc, self, arg1, arg2)
#endif

/* CallUnboundCMethod1.proto */
static PyObject* __Pyx__CallUnboundCMethod1(__Pyx_CachedCFunction* cfunc, PyObject* self, PyObject* arg);
#if CYTHON_COMPILING_IN_CPYTHON
static CYTHON_INLINE PyObject* __Pyx_CallUnboundCMethod1(__Pyx_CachedCFunction* cfunc, PyObject* self, PyObject* arg);
#else
#define __Pyx_CallUnboundCMethod1(cfunc, self, arg)  __Pyx__CallUnboundCMethod1(cfunc, self, arg)
#endif

/* pyfrozenset_new.proto */
static CYTHON_INLINE PyObject* __Pyx_PyFrozenSet_New(PyObject* it);

/* py_set_remove.proto */
static CYTHON_INLINE int __Pyx_PySet_Remove(PyObject *set, PyObject *key);

/* None.proto */
static CYTHON_INLINE void __Pyx_RaiseClosureNameError(const char *varname);

/* FetchCommonType.proto */
static PyTypeObject* __Pyx_FetchCommonType(PyTypeObject* type);

/* CythonFunction.proto */
#define __Pyx_CyFunction_USED 1
#define __Pyx_CYFUNCTION_STATICMETHOD  0x01
#define __Pyx_CYFUNCTION_CLASSMETHOD   0x02
#define __Pyx_CYFUNCTION_CCLASS        0x04
#define __Pyx_CyFunction_GetClosure(f)\
    (((__pyx_CyFunctionObject *) (f))->func_closure)
#define __Pyx_CyFunction_GetClassObj(f)\
    (((__pyx_CyFunctionObject *) (f))->func_classobj)
#define __Pyx_CyFunction_Defaults(type, f)\
    ((type *)(((__pyx_CyFunctionObject *) (f))->defaults))
#define __Pyx_CyFunction_SetDefaultsGetter(f, g)\
    ((__pyx_CyFunctionObject *) (f))->defaults_getter = (g)
typedef struct {
    PyCFunctionObject func;
#if PY_VERSION_HEX < 0x030500A0
    PyObject *func_weakreflist;
#endif
    PyObject *func_dict;
    PyObject *func_name;
    PyObject *func_qualname;
    PyObject *func_doc;
    PyObject *func_globals;
    PyObject *func_code;
    PyObject *func_closure;
    PyObject *func_classobj;
    void *defaults;
    int defaults_pyobjects;
    int flags;
    PyObject *defaults_tuple;
    PyObject *defaults_kwdict;
    PyObject *(*defaults_getter)(PyObject *);
    PyObject *func_annotations;
} __pyx_CyFunctionObject;
static PyTypeObject *__pyx_CyFunctionType = 0;
#define __Pyx_CyFunction_NewEx(ml, flags, qualname, self, module, globals, code)\
    __Pyx_CyFunction_New(__pyx_CyFunctionType, ml, flags, qualname, self, module, globals, code)
static PyObject *__Pyx_CyFunction_New(PyTypeObject *, PyMethodDef *ml,
                                      int flags, PyObject* qualname,
                                      PyObject *self,
                                      PyObject *module, PyObject *globals,
                                      PyObject* code);
static CYTHON_INLINE void *__Pyx_CyFunction_InitDefaults(PyObject *m,
                                                         size_t size,
                                                         int pyobjects);
static CYTHON_INLINE void __Pyx_CyFunction_SetDefaultsTuple(PyObject *m,
                                                            PyObject *tuple);
static CYTHON_INLINE void __Pyx_CyFunction_SetDefaultsKwDict(PyObject *m,
                                                             PyObject *dict);
static CYTHON_INLINE void __Pyx_CyFunction_SetAnnotationsDict(PyObject *m,
                                                              PyObject *dict);
static int __pyx_CyFunction_init(void);

/* set_iter.proto */
static CYTHON_INLINE PyObject* __Pyx_set_iterator(PyObject* iterable, int is_set,
                                                  Py_ssize_t* p_orig_length, int* p_source_is_set);
static CYTHON_INLINE int __Pyx_set_iter_next(
        PyObject* iter_obj, Py_ssize_t orig_length,
        Py_ssize_t* ppos, PyObject **value,
        int source_is_set);

/* py_dict_values.proto */
static CYTHON_INLINE PyObject* __Pyx_PyDict_Values(PyObject* d);

/* CallUnboundCMethod0.proto */
static PyObject* __Pyx__CallUnboundCMethod0(__Pyx_CachedCFunction* cfunc, PyObject* self);
#if CYTHON_COMPILING_IN_CPYTHON
#define __Pyx_CallUnboundCMethod0(cfunc, self)\
    (likely((cfunc)->func) ?\
        (likely((cfunc)->flag == METH_NOARGS) ?  (*((cfunc)->func))(self, NULL) :\
         (PY_VERSION_HEX >= 0x030600B1 && likely((cfunc)->flag == METH_FASTCALL) ?\
            (PY_VERSION_HEX >= 0x030700A0 ?\
                (*(__Pyx_PyCFunctionFast)(cfunc)->func)(self, &__pyx_empty_tuple, 0) :\
                (*(__Pyx_PyCFunctionFastWithKeywords)(cfunc)->func)(self, &__pyx_empty_tuple, 0, NULL)) :\
          (PY_VERSION_HEX >= 0x030700A0 && (cfunc)->flag == (METH_FASTCALL | METH_KEYWORDS) ?\
            (*(__Pyx_PyCFunctionFastWithKeywords)(cfunc)->func)(self, &__pyx_empty_tuple, 0, NULL) :\
            (likely((cfunc)->flag == (METH_VARARGS | METH_KEYWORDS)) ?  ((*(PyCFunctionWithKeywords)(cfunc)->func)(self, __pyx_empty_tuple, NULL)) :\
               ((cfunc)->flag == METH_VARARGS ?  (*((cfunc)->func))(self, __pyx_empty_tuple) :\
               __Pyx__CallUnboundCMethod0(cfunc, self)))))) :\
        __Pyx__CallUnboundCMethod0(cfunc, self))
#else
#define __Pyx_CallUnboundCMethod0(cfunc, self)  __Pyx__CallUnboundCMethod0(cfunc, self)
#endif

/* PyFloatBinop.proto */
#if !CYTHON_COMPILING_IN_PYPY
static PyObject* __Pyx_PyFloat_AddObjC(PyObject *op1, PyObject *op2, double floatval, int inplace);
#else
#define __Pyx_PyFloat_AddObjC(op1, op2, floatval, inplace)\
    (inplace ? PyNumber_InPlaceAdd(op1, op2) : PyNumber_Add(op1, op2))
#endif

/* IterNext.proto */
#define __Pyx_PyIter_Next(obj) __Pyx_PyIter_Next2(obj, NULL)
static CYTHON_INLINE PyObject *__Pyx_PyIter_Next2(PyObject *, PyObject *);

/* GetAttr3.proto */
static CYTHON_INLINE PyObject *__Pyx_GetAttr3(PyObject *, PyObject *, PyObject *);

/* CallableCheck.proto */
#if CYTHON_USE_TYPE_SLOTS && PY_MAJOR_VERSION >= 3
#define __Pyx_PyCallable_Check(obj)   ((obj)->ob_type->tp_call != NULL)
#else
#define __Pyx_PyCallable_Check(obj)   PyCallable_Check(obj)
#endif

/* PyIntBinop.proto */
#if !CYTHON_COMPILING_IN_PYPY
static PyObject* __Pyx_PyInt_EqObjC(PyObject *op1, PyObject *op2, long intval, int inplace);
#else
#define __Pyx_PyInt_EqObjC(op1, op2, intval, inplace)\
    PyObject_RichCompare(op1, op2, Py_EQ)
    #endif

/* IncludeStringH.proto */
#include <string.h>

/* BytesEquals.proto */
static CYTHON_INLINE int __Pyx_PyBytes_Equals(PyObject* s1, PyObject* s2, int equals);

/* SliceObject.proto */
static CYTHON_INLINE PyObject* __Pyx_PyObject_GetSlice(
        PyObject* obj, Py_ssize_t cstart, Py_ssize_t cstop,
        PyObject** py_start, PyObject** py_stop, PyObject** py_slice,
        int has_cstart, int has_cstop, int wraparound);

/* PyDictContains.proto */
static CYTHON_INLINE int __Pyx_PyDict_ContainsTF(PyObject* item, PyObject* dict, int eq) {
    int result = PyDict_Contains(dict, item);
    return unlikely(result < 0) ? result : (result == (eq == Py_EQ));
}

/* DictGetItem.proto */
#if PY_MAJOR_VERSION >= 3 && !CYTHON_COMPILING_IN_PYPY
static PyObject *__Pyx_PyDict_GetItem(PyObject *d, PyObject* key);
#define __Pyx_PyObject_Dict_GetItem(obj, name)\
    (likely(PyDict_CheckExact(obj)) ?\
     __Pyx_PyDict_GetItem(obj, name) : PyObject_GetItem(obj, name))
#else
#define __Pyx_PyDict_GetItem(d, key) PyObject_GetItem(d, key)
#define __Pyx_PyObject_Dict_GetItem(obj, name)  PyObject_GetItem(obj, name)
#endif

/* PySequenceContains.proto */
static CYTHON_INLINE int __Pyx_PySequence_ContainsTF(PyObject* item, PyObject* seq, int eq) {
    int result = PySequence_Contains(seq, item);
    return unlikely(result < 0) ? result : (result == (eq == Py_EQ));
}

/* StringJoin.proto */
#if PY_MAJOR_VERSION < 3
#define __Pyx_PyString_Join __Pyx_PyBytes_Join
#define __Pyx_PyBaseString_Join(s, v) (PyUnicode_CheckExact(s) ? PyUnicode_Join(s, v) : __Pyx_PyBytes_Join(s, v))
#else
#define __Pyx_PyString_Join PyUnicode_Join
#define __Pyx_PyBaseString_Join PyUnicode_Join
#endif
#if CYTHON_COMPILING_IN_CPYTHON
    #if PY_MAJOR_VERSION < 3
    #define __Pyx_PyBytes_Join _PyString_Join
    #else
    #define __Pyx_PyBytes_Join _PyBytes_Join
    #endif
#else
static CYTHON_INLINE PyObject* __Pyx_PyBytes_Join(PyObject* sep, PyObject* values);
#endif

/* SetItemInt.proto */
#define __Pyx_SetItemInt(o, i, v, type, is_signed, to_py_func, is_list, wraparound, boundscheck)\
    (__Pyx_fits_Py_ssize_t(i, type, is_signed) ?\
    __Pyx_SetItemInt_Fast(o, (Py_ssize_t)i, v, is_list, wraparound, boundscheck) :\
    (is_list ? (PyErr_SetString(PyExc_IndexError, "list assignment index out of range"), -1) :\
               __Pyx_SetItemInt_Generic(o, to_py_func(i), v)))
static int __Pyx_SetItemInt_Generic(PyObject *o, PyObject *j, PyObject *v);
static CYTHON_INLINE int __Pyx_SetItemInt_Fast(PyObject *o, Py_ssize_t i, PyObject *v,
                                               int is_list, int wraparound, int boundscheck);

/* FastTypeChecks.proto */
#if CYTHON_COMPILING_IN_CPYTHON
#define __Pyx_TypeCheck(obj, type) __Pyx_IsSubtype(Py_TYPE(obj), (PyTypeObject *)type)
static CYTHON_INLINE int __Pyx_IsSubtype(PyTypeObject *a, PyTypeObject *b);
static CYTHON_INLINE int __Pyx_PyErr_GivenExceptionMatches(PyObject *err, PyObject *type);
static CYTHON_INLINE int __Pyx_PyErr_GivenExceptionMatches2(PyObject *err, PyObject *type1, PyObject *type2);
#else
#define __Pyx_TypeCheck(obj, type) PyObject_TypeCheck(obj, (PyTypeObject *)type)
#define __Pyx_PyErr_GivenExceptionMatches(err, type) PyErr_GivenExceptionMatches(err, type)
#define __Pyx_PyErr_GivenExceptionMatches2(err, type1, type2) (PyErr_GivenExceptionMatches(err, type1) || PyErr_GivenExceptionMatches(err, type2))
#endif
#define __Pyx_PyException_Check(obj) __Pyx_TypeCheck(obj, PyExc_Exception)

/* Import.proto */
static PyObject *__Pyx_Import(PyObject *name, PyObject *from_list, int level);

/* PyObjectSetAttrStr.proto */
#if CYTHON_USE_TYPE_SLOTS
#define __Pyx_PyObject_DelAttrStr(o,n) __Pyx_PyObject_SetAttrStr(o, n, NULL)
static CYTHON_INLINE int __Pyx_PyObject_SetAttrStr(PyObject* obj, PyObject* attr_name, PyObject* value);
#else
#define __Pyx_PyObject_DelAttrStr(o,n)   PyObject_DelAttr(o,n)
#define __Pyx_PyObject_SetAttrStr(o,n,v) PyObject_SetAttr(o,n,v)
#endif

/* SwapException.proto */
#if CYTHON_FAST_THREAD_STATE
#define __Pyx_ExceptionSwap(type, value, tb)  __Pyx__ExceptionSwap(__pyx_tstate, type, value, tb)
static CYTHON_INLINE void __Pyx__ExceptionSwap(PyThreadState *tstate, PyObject **type, PyObject **value, PyObject **tb);
#else
static CYTHON_INLINE void __Pyx_ExceptionSwap(PyObject **type, PyObject **value, PyObject **tb);
#endif

/* PyIntBinop.proto */
#if !CYTHON_COMPILING_IN_PYPY
static PyObject* __Pyx_PyInt_SubtractObjC(PyObject *op1, PyObject *op2, long intval, int inplace);
#else
#define __Pyx_PyInt_SubtractObjC(op1, op2, intval, inplace)\
    (inplace ? PyNumber_InPlaceSubtract(op1, op2) : PyNumber_Subtract(op1, op2))
#endif

/* ImportFrom.proto */
static PyObject* __Pyx_ImportFrom(PyObject* module, PyObject* name);

/* CallNextTpTraverse.proto */
static int __Pyx_call_next_tp_traverse(PyObject* obj, visitproc v, void *a, traverseproc current_tp_traverse);

/* CallNextTpClear.proto */
static void __Pyx_call_next_tp_clear(PyObject* obj, inquiry current_tp_dealloc);

/* PyObject_GenericGetAttrNoDict.proto */
#if CYTHON_USE_TYPE_SLOTS && CYTHON_USE_PYTYPE_LOOKUP && PY_VERSION_HEX < 0x03070000
static CYTHON_INLINE PyObject* __Pyx_PyObject_GenericGetAttrNoDict(PyObject* obj, PyObject* attr_name);
#else
#define __Pyx_PyObject_GenericGetAttrNoDict PyObject_GenericGetAttr
#endif

/* PyObject_GenericGetAttr.proto */
#if CYTHON_USE_TYPE_SLOTS && CYTHON_USE_PYTYPE_LOOKUP && PY_VERSION_HEX < 0x03070000
static PyObject* __Pyx_PyObject_GenericGetAttr(PyObject* obj, PyObject* attr_name);
#else
#define __Pyx_PyObject_GenericGetAttr PyObject_GenericGetAttr
#endif

/* SetVTable.proto */
static int __Pyx_SetVtable(PyObject *dict, void *vtable);

/* SetupReduce.proto */
static int __Pyx_setup_reduce(PyObject* type_obj);

/* SetNameInClass.proto */
#if CYTHON_COMPILING_IN_CPYTHON && PY_VERSION_HEX >= 0x030500A1
#define __Pyx_SetNameInClass(ns, name, value)\
    (likely(PyDict_CheckExact(ns)) ? _PyDict_SetItem_KnownHash(ns, name, value, ((PyASCIIObject *) name)->hash) : PyObject_SetItem(ns, name, value))
#elif CYTHON_COMPILING_IN_CPYTHON
#define __Pyx_SetNameInClass(ns, name, value)\
    (likely(PyDict_CheckExact(ns)) ? PyDict_SetItem(ns, name, value) : PyObject_SetItem(ns, name, value))
#else
#define __Pyx_SetNameInClass(ns, name, value)  PyObject_SetItem(ns, name, value)
#endif

/* CalculateMetaclass.proto */
static PyObject *__Pyx_CalculateMetaclass(PyTypeObject *metaclass, PyObject *bases);

/* Py3ClassCreate.proto */
static PyObject *__Pyx_Py3MetaclassPrepare(PyObject *metaclass, PyObject *bases, PyObject *name, PyObject *qualname,
                                           PyObject *mkw, PyObject *modname, PyObject *doc);
static PyObject *__Pyx_Py3ClassCreate(PyObject *metaclass, PyObject *name, PyObject *bases, PyObject *dict,
                                      PyObject *mkw, int calculate_metaclass, int allow_py2_metaclass);

/* CLineInTraceback.proto */
#ifdef CYTHON_CLINE_IN_TRACEBACK
#define __Pyx_CLineForTraceback(tstate, c_line)  (((CYTHON_CLINE_IN_TRACEBACK)) ? c_line : 0)
#else
static int __Pyx_CLineForTraceback(PyThreadState *tstate, int c_line);
#endif

/* CodeObjectCache.proto */
typedef struct {
    PyCodeObject* code_object;
    int code_line;
} __Pyx_CodeObjectCacheEntry;
struct __Pyx_CodeObjectCache {
    int count;
    int max_count;
    __Pyx_CodeObjectCacheEntry* entries;
};
static struct __Pyx_CodeObjectCache __pyx_code_cache = {0,0,NULL};
static int __pyx_bisect_code_objects(__Pyx_CodeObjectCacheEntry* entries, int count, int code_line);
static PyCodeObject *__pyx_find_code_object(int code_line);
static void __pyx_insert_code_object(int code_line, PyCodeObject* code_object);

/* AddTraceback.proto */
static void __Pyx_AddTraceback(const char *funcname, int c_line,
                               int py_line, const char *filename);

/* CIntToPy.proto */
static CYTHON_INLINE PyObject* __Pyx_PyInt_From_int(int value);

/* CIntToPy.proto */
static CYTHON_INLINE PyObject* __Pyx_PyInt_From_grpc_connectivity_state(grpc_connectivity_state value);

/* CIntToPy.proto */
static CYTHON_INLINE PyObject* __Pyx_PyInt_From_grpc_status_code(grpc_status_code value);

/* CIntToPy.proto */
static CYTHON_INLINE PyObject* __Pyx_PyInt_From_grpc_call_error(grpc_call_error value);

/* CIntToPy.proto */
static CYTHON_INLINE PyObject* __Pyx_PyInt_From_grpc_completion_type(grpc_completion_type value);

/* CIntToPy.proto */
static CYTHON_INLINE PyObject* __Pyx_PyInt_From_grpc_op_type(grpc_op_type value);

/* CIntToPy.proto */
static CYTHON_INLINE PyObject* __Pyx_PyInt_From_grpc_compression_algorithm(grpc_compression_algorithm value);

/* CIntToPy.proto */
static CYTHON_INLINE PyObject* __Pyx_PyInt_From_grpc_compression_level(grpc_compression_level value);

/* CIntToPy.proto */
static CYTHON_INLINE PyObject* __Pyx_PyInt_From_long(long value);

/* CIntToPy.proto */
static CYTHON_INLINE PyObject* __Pyx_PyInt_From_unsigned_int(unsigned int value);

/* CIntFromPy.proto */
static CYTHON_INLINE grpc_status_code __Pyx_PyInt_As_grpc_status_code(PyObject *);

/* CIntFromPy.proto */
static CYTHON_INLINE int __Pyx_PyInt_As_int(PyObject *);

/* CIntFromPy.proto */
static CYTHON_INLINE grpc_connectivity_state __Pyx_PyInt_As_grpc_connectivity_state(PyObject *);

/* CIntFromPy.proto */
static CYTHON_INLINE grpc_completion_type __Pyx_PyInt_As_grpc_completion_type(PyObject *);

/* CIntFromPy.proto */
static CYTHON_INLINE grpc_compression_algorithm __Pyx_PyInt_As_grpc_compression_algorithm(PyObject *);

/* CIntFromPy.proto */
static CYTHON_INLINE long __Pyx_PyInt_As_long(PyObject *);

/* CIntFromPy.proto */
static CYTHON_INLINE grpc_call_error __Pyx_PyInt_As_grpc_call_error(PyObject *);

/* CIntFromPy.proto */
static CYTHON_INLINE size_t __Pyx_PyInt_As_size_t(PyObject *);

/* CoroutineBase.proto */
typedef PyObject *(*__pyx_coroutine_body_t)(PyObject *, PyThreadState *, PyObject *);
typedef struct {
    PyObject_HEAD
    __pyx_coroutine_body_t body;
    PyObject *closure;
    PyObject *exc_type;
    PyObject *exc_value;
    PyObject *exc_traceback;
    PyObject *gi_weakreflist;
    PyObject *classobj;
    PyObject *yieldfrom;
    PyObject *gi_name;
    PyObject *gi_qualname;
    PyObject *gi_modulename;
    PyObject *gi_code;
    int resume_label;
    char is_running;
} __pyx_CoroutineObject;
static __pyx_CoroutineObject *__Pyx__Coroutine_New(
    PyTypeObject *type, __pyx_coroutine_body_t body, PyObject *code, PyObject *closure,
    PyObject *name, PyObject *qualname, PyObject *module_name);
static __pyx_CoroutineObject *__Pyx__Coroutine_NewInit(
            __pyx_CoroutineObject *gen, __pyx_coroutine_body_t body, PyObject *code, PyObject *closure,
            PyObject *name, PyObject *qualname, PyObject *module_name);
static int __Pyx_Coroutine_clear(PyObject *self);
static PyObject *__Pyx_Coroutine_Send(PyObject *self, PyObject *value);
static PyObject *__Pyx_Coroutine_Close(PyObject *self);
static PyObject *__Pyx_Coroutine_Throw(PyObject *gen, PyObject *args);
#define __Pyx_Coroutine_SwapException(self) {\
    __Pyx_ExceptionSwap(&(self)->exc_type, &(self)->exc_value, &(self)->exc_traceback);\
    __Pyx_Coroutine_ResetFrameBackpointer(self);\
    }
#define __Pyx_Coroutine_ResetAndClearException(self) {\
    __Pyx_ExceptionReset((self)->exc_type, (self)->exc_value, (self)->exc_traceback);\
    (self)->exc_type = (self)->exc_value = (self)->exc_traceback = NULL;\
    }
#if CYTHON_FAST_THREAD_STATE
#define __Pyx_PyGen_FetchStopIterationValue(pvalue)\
    __Pyx_PyGen__FetchStopIterationValue(__pyx_tstate, pvalue)
#else
#define __Pyx_PyGen_FetchStopIterationValue(pvalue)\
    __Pyx_PyGen__FetchStopIterationValue(__Pyx_PyThreadState_Current, pvalue)
#endif
static int __Pyx_PyGen__FetchStopIterationValue(PyThreadState *tstate, PyObject **pvalue);
static CYTHON_INLINE void __Pyx_Coroutine_ResetFrameBackpointer(__pyx_CoroutineObject *self);

/* PatchModuleWithCoroutine.proto */
static PyObject* __Pyx_Coroutine_patch_module(PyObject* module, const char* py_code);

/* PatchGeneratorABC.proto */
static int __Pyx_patch_abc(void);

/* Generator.proto */
#define __Pyx_Generator_USED
static PyTypeObject *__pyx_GeneratorType = 0;
#define __Pyx_Generator_CheckExact(obj) (Py_TYPE(obj) == __pyx_GeneratorType)
#define __Pyx_Generator_New(body, code, closure, name, qualname, module_name)\
    __Pyx__Coroutine_New(__pyx_GeneratorType, body, code, closure, name, qualname, module_name)
static PyObject *__Pyx_Generator_Next(PyObject *self);
static int __pyx_Generator_init(void);

/* CheckBinaryVersion.proto */
static int __Pyx_check_binary_version(void);

/* FunctionExport.proto */
static int __Pyx_ExportFunction(const char *name, void (*f)(void), const char *sig);

/* PyIdentifierFromString.proto */
#if !defined(__Pyx_PyIdentifier_FromString)
#if PY_MAJOR_VERSION < 3
  #define __Pyx_PyIdentifier_FromString(s) PyString_FromString(s)
#else
  #define __Pyx_PyIdentifier_FromString(s) PyUnicode_FromString(s)
#endif
#endif

/* ModuleImport.proto */
static PyObject *__Pyx_ImportModule(const char *name);

/* TypeImport.proto */
static PyTypeObject *__Pyx_ImportType(const char *module_name, const char *class_name, size_t size, int strict);

/* InitStrings.proto */
static int __Pyx_InitStrings(__Pyx_StringTabEntry *t);

static void __pyx_f_4grpc_7_cython_6cygrpc_11_ChannelArg_c(struct __pyx_obj_4grpc_7_cython_6cygrpc__ChannelArg *__pyx_v_self, PyObject *__pyx_v_argument, struct __pyx_obj_4grpc_7_cython_6cygrpc__VTable *__pyx_v_vtable, PyObject *__pyx_v_references); /* proto*/
static grpc_channel_args *__pyx_f_4grpc_7_cython_6cygrpc_12_ChannelArgs_c_args(struct __pyx_obj_4grpc_7_cython_6cygrpc__ChannelArgs *__pyx_v_self); /* proto*/
static grpc_call_credentials *__pyx_f_4grpc_7_cython_6cygrpc_15CallCredentials_c(CYTHON_UNUSED struct __pyx_obj_4grpc_7_cython_6cygrpc_CallCredentials *__pyx_v_self); /* proto*/
static grpc_call_credentials *__pyx_f_4grpc_7_cython_6cygrpc_29MetadataPluginCallCredentials_c(struct __pyx_obj_4grpc_7_cython_6cygrpc_MetadataPluginCallCredentials *__pyx_v_self); /* proto*/
static grpc_call_credentials *__pyx_f_4grpc_7_cython_6cygrpc_24CompositeCallCredentials_c(struct __pyx_obj_4grpc_7_cython_6cygrpc_CompositeCallCredentials *__pyx_v_self); /* proto*/
static grpc_channel_credentials *__pyx_f_4grpc_7_cython_6cygrpc_18ChannelCredentials_c(CYTHON_UNUSED struct __pyx_obj_4grpc_7_cython_6cygrpc_ChannelCredentials *__pyx_v_self); /* proto*/
static grpc_channel_credentials *__pyx_f_4grpc_7_cython_6cygrpc_21SSLChannelCredentials_c(struct __pyx_obj_4grpc_7_cython_6cygrpc_SSLChannelCredentials *__pyx_v_self); /* proto*/
static grpc_channel_credentials *__pyx_f_4grpc_7_cython_6cygrpc_27CompositeChannelCredentials_c(struct __pyx_obj_4grpc_7_cython_6cygrpc_CompositeChannelCredentials *__pyx_v_self); /* proto*/
static PyObject *__pyx_f_4grpc_7_cython_6cygrpc_15CompletionQueue__interpret_event(struct __pyx_obj_4grpc_7_cython_6cygrpc_CompletionQueue *__pyx_v_self, grpc_event __pyx_v_c_event); /* proto*/
static void __pyx_f_4grpc_7_cython_6cygrpc_9Operation_c(CYTHON_UNUSED struct __pyx_obj_4grpc_7_cython_6cygrpc_Operation *__pyx_v_self); /* proto*/
static void __pyx_f_4grpc_7_cython_6cygrpc_9Operation_un_c(CYTHON_UNUSED struct __pyx_obj_4grpc_7_cython_6cygrpc_Operation *__pyx_v_self); /* proto*/
static void __pyx_f_4grpc_7_cython_6cygrpc_28SendInitialMetadataOperation_c(struct __pyx_obj_4grpc_7_cython_6cygrpc_SendInitialMetadataOperation *__pyx_v_self); /* proto*/
static void __pyx_f_4grpc_7_cython_6cygrpc_28SendInitialMetadataOperation_un_c(struct __pyx_obj_4grpc_7_cython_6cygrpc_SendInitialMetadataOperation *__pyx_v_self); /* proto*/
static void __pyx_f_4grpc_7_cython_6cygrpc_20SendMessageOperation_c(struct __pyx_obj_4grpc_7_cython_6cygrpc_SendMessageOperation *__pyx_v_self); /* proto*/
static void __pyx_f_4grpc_7_cython_6cygrpc_20SendMessageOperation_un_c(struct __pyx_obj_4grpc_7_cython_6cygrpc_SendMessageOperation *__pyx_v_self); /* proto*/
static void __pyx_f_4grpc_7_cython_6cygrpc_28SendCloseFromClientOperation_c(struct __pyx_obj_4grpc_7_cython_6cygrpc_SendCloseFromClientOperation *__pyx_v_self); /* proto*/
static void __pyx_f_4grpc_7_cython_6cygrpc_28SendCloseFromClientOperation_un_c(CYTHON_UNUSED struct __pyx_obj_4grpc_7_cython_6cygrpc_SendCloseFromClientOperation *__pyx_v_self); /* proto*/
static void __pyx_f_4grpc_7_cython_6cygrpc_29SendStatusFromServerOperation_c(struct __pyx_obj_4grpc_7_cython_6cygrpc_SendStatusFromServerOperation *__pyx_v_self); /* proto*/
static void __pyx_f_4grpc_7_cython_6cygrpc_29SendStatusFromServerOperation_un_c(struct __pyx_obj_4grpc_7_cython_6cygrpc_SendStatusFromServerOperation *__pyx_v_self); /* proto*/
static void __pyx_f_4grpc_7_cython_6cygrpc_31ReceiveInitialMetadataOperation_c(struct __pyx_obj_4grpc_7_cython_6cygrpc_ReceiveInitialMetadataOperation *__pyx_v_self); /* proto*/
static void __pyx_f_4grpc_7_cython_6cygrpc_31ReceiveInitialMetadataOperation_un_c(struct __pyx_obj_4grpc_7_cython_6cygrpc_ReceiveInitialMetadataOperation *__pyx_v_self); /* proto*/
static void __pyx_f_4grpc_7_cython_6cygrpc_23ReceiveMessageOperation_c(struct __pyx_obj_4grpc_7_cython_6cygrpc_ReceiveMessageOperation *__pyx_v_self); /* proto*/
static void __pyx_f_4grpc_7_cython_6cygrpc_23ReceiveMessageOperation_un_c(struct __pyx_obj_4grpc_7_cython_6cygrpc_ReceiveMessageOperation *__pyx_v_self); /* proto*/
static void __pyx_f_4grpc_7_cython_6cygrpc_30ReceiveStatusOnClientOperation_c(struct __pyx_obj_4grpc_7_cython_6cygrpc_ReceiveStatusOnClientOperation *__pyx_v_self); /* proto*/
static void __pyx_f_4grpc_7_cython_6cygrpc_30ReceiveStatusOnClientOperation_un_c(struct __pyx_obj_4grpc_7_cython_6cygrpc_ReceiveStatusOnClientOperation *__pyx_v_self); /* proto*/
static void __pyx_f_4grpc_7_cython_6cygrpc_29ReceiveCloseOnServerOperation_c(struct __pyx_obj_4grpc_7_cython_6cygrpc_ReceiveCloseOnServerOperation *__pyx_v_self); /* proto*/
static void __pyx_f_4grpc_7_cython_6cygrpc_29ReceiveCloseOnServerOperation_un_c(struct __pyx_obj_4grpc_7_cython_6cygrpc_ReceiveCloseOnServerOperation *__pyx_v_self); /* proto*/
static PyObject *__pyx_f_4grpc_7_cython_6cygrpc_6Server__c_shutdown(struct __pyx_obj_4grpc_7_cython_6cygrpc_Server *__pyx_v_self, struct __pyx_obj_4grpc_7_cython_6cygrpc_CompletionQueue *__pyx_v_queue, PyObject *__pyx_v_tag); /* proto*/
static PyObject *__pyx_f_4grpc_7_cython_6cygrpc_6Server_notify_shutdown_complete(struct __pyx_obj_4grpc_7_cython_6cygrpc_Server *__pyx_v_self); /* proto*/
static PyObject *__pyx_f_4grpc_7_cython_6cygrpc_4_Tag_event(CYTHON_UNUSED struct __pyx_obj_4grpc_7_cython_6cygrpc__Tag *__pyx_v_self, CYTHON_UNUSED grpc_event __pyx_v_c_event); /* proto*/
struct __pyx_obj_4grpc_7_cython_6cygrpc_ConnectivityEvent *__pyx_f_4grpc_7_cython_6cygrpc_16_ConnectivityTag_event(struct __pyx_obj_4grpc_7_cython_6cygrpc__ConnectivityTag *__pyx_v_self, grpc_event __pyx_v_c_event); /* proto*/
struct __pyx_obj_4grpc_7_cython_6cygrpc_ConnectivityEvent *__pyx_f_4grpc_7_cython_6cygrpc_16_ConnectivityTag_event__pyx_wrap_1(struct __pyx_obj_4grpc_7_cython_6cygrpc__ConnectivityTag *__pyx_v_self, grpc_event __pyx_v_c_event); /* proto*/
static void __pyx_f_4grpc_7_cython_6cygrpc_15_RequestCallTag_prepare(struct __pyx_obj_4grpc_7_cython_6cygrpc__RequestCallTag *__pyx_v_self); /* proto*/
struct __pyx_obj_4grpc_7_cython_6cygrpc_RequestCallEvent *__pyx_f_4grpc_7_cython_6cygrpc_15_RequestCallTag_event(struct __pyx_obj_4grpc_7_cython_6cygrpc__RequestCallTag *__pyx_v_self, grpc_event __pyx_v_c_event); /* proto*/
struct __pyx_obj_4grpc_7_cython_6cygrpc_RequestCallEvent *__pyx_f_4grpc_7_cython_6cygrpc_15_RequestCallTag_event__pyx_wrap_1(struct __pyx_obj_4grpc_7_cython_6cygrpc__RequestCallTag *__pyx_v_self, grpc_event __pyx_v_c_event); /* proto*/
static void __pyx_f_4grpc_7_cython_6cygrpc_18_BatchOperationTag_prepare(struct __pyx_obj_4grpc_7_cython_6cygrpc__BatchOperationTag *__pyx_v_self); /* proto*/
struct __pyx_obj_4grpc_7_cython_6cygrpc_BatchOperationEvent *__pyx_f_4grpc_7_cython_6cygrpc_18_BatchOperationTag_event(struct __pyx_obj_4grpc_7_cython_6cygrpc__BatchOperationTag *__pyx_v_self, grpc_event __pyx_v_c_event); /* proto*/
struct __pyx_obj_4grpc_7_cython_6cygrpc_BatchOperationEvent *__pyx_f_4grpc_7_cython_6cygrpc_18_BatchOperationTag_event__pyx_wrap_1(struct __pyx_obj_4grpc_7_cython_6cygrpc__BatchOperationTag *__pyx_v_self, grpc_event __pyx_v_c_event); /* proto*/
struct __pyx_obj_4grpc_7_cython_6cygrpc_ServerShutdownEvent *__pyx_f_4grpc_7_cython_6cygrpc_18_ServerShutdownTag_event(struct __pyx_obj_4grpc_7_cython_6cygrpc__ServerShutdownTag *__pyx_v_self, grpc_event __pyx_v_c_event); /* proto*/
struct __pyx_obj_4grpc_7_cython_6cygrpc_ServerShutdownEvent *__pyx_f_4grpc_7_cython_6cygrpc_18_ServerShutdownTag_event__pyx_wrap_1(struct __pyx_obj_4grpc_7_cython_6cygrpc__ServerShutdownTag *__pyx_v_self, grpc_event __pyx_v_c_event); /* proto*/

/* Module declarations from 'libc.stddef' */

/* Module declarations from 'libc.time' */

/* Module declarations from 'libc' */

/* Module declarations from 'cpython.version' */

/* Module declarations from '__builtin__' */

/* Module declarations from 'cpython.type' */
static PyTypeObject *__pyx_ptype_7cpython_4type_type = 0;

/* Module declarations from 'libc.string' */

/* Module declarations from 'libc.stdio' */

/* Module declarations from 'cpython.object' */

/* Module declarations from 'cpython.ref' */

/* Module declarations from 'cpython.exc' */

/* Module declarations from 'cpython.module' */

/* Module declarations from 'cpython.mem' */

/* Module declarations from 'cpython.tuple' */

/* Module declarations from 'cpython.list' */

/* Module declarations from 'cpython.sequence' */

/* Module declarations from 'cpython.mapping' */

/* Module declarations from 'cpython.iterator' */

/* Module declarations from 'cpython.number' */

/* Module declarations from 'cpython.int' */

/* Module declarations from '__builtin__' */

/* Module declarations from 'cpython.bool' */
static PyTypeObject *__pyx_ptype_7cpython_4bool_bool = 0;

/* Module declarations from 'cpython.long' */

/* Module declarations from 'cpython.float' */

/* Module declarations from '__builtin__' */

/* Module declarations from 'cpython.complex' */
static PyTypeObject *__pyx_ptype_7cpython_7complex_complex = 0;

/* Module declarations from 'cpython.string' */

/* Module declarations from 'cpython.unicode' */

/* Module declarations from 'cpython.dict' */

/* Module declarations from 'cpython.instance' */

/* Module declarations from 'cpython.function' */

/* Module declarations from 'cpython.method' */

/* Module declarations from 'cpython.weakref' */

/* Module declarations from 'cpython.getargs' */

/* Module declarations from 'cpython.pythread' */

/* Module declarations from 'cpython.pystate' */

/* Module declarations from 'cpython.cobject' */

/* Module declarations from 'cpython.oldbuffer' */

/* Module declarations from 'cpython.set' */

/* Module declarations from 'cpython.buffer' */

/* Module declarations from 'cpython.bytes' */

/* Module declarations from 'cpython.pycapsule' */

/* Module declarations from 'cpython' */

/* Module declarations from 'libc.stdlib' */

/* Module declarations from 'grpc._cython.cygrpc' */
static PyTypeObject *__pyx_ptype_4grpc_7_cython_6cygrpc__ChannelArg = 0;
static PyTypeObject *__pyx_ptype_4grpc_7_cython_6cygrpc__ChannelArgs = 0;
static PyTypeObject *__pyx_ptype_4grpc_7_cython_6cygrpc_Call = 0;
static PyTypeObject *__pyx_ptype_4grpc_7_cython_6cygrpc__CallState = 0;
static PyTypeObject *__pyx_ptype_4grpc_7_cython_6cygrpc__ChannelState = 0;
static PyTypeObject *__pyx_ptype_4grpc_7_cython_6cygrpc_IntegratedCall = 0;
static PyTypeObject *__pyx_ptype_4grpc_7_cython_6cygrpc_SegregatedCall = 0;
static PyTypeObject *__pyx_ptype_4grpc_7_cython_6cygrpc_Channel = 0;
static PyTypeObject *__pyx_ptype_4grpc_7_cython_6cygrpc_CallCredentials = 0;
static PyTypeObject *__pyx_ptype_4grpc_7_cython_6cygrpc_MetadataPluginCallCredentials = 0;
static PyTypeObject *__pyx_ptype_4grpc_7_cython_6cygrpc_CompositeCallCredentials = 0;
static PyTypeObject *__pyx_ptype_4grpc_7_cython_6cygrpc_ChannelCredentials = 0;
static PyTypeObject *__pyx_ptype_4grpc_7_cython_6cygrpc_SSLSessionCacheLRU = 0;
static PyTypeObject *__pyx_ptype_4grpc_7_cython_6cygrpc_SSLChannelCredentials = 0;
static PyTypeObject *__pyx_ptype_4grpc_7_cython_6cygrpc_CompositeChannelCredentials = 0;
static PyTypeObject *__pyx_ptype_4grpc_7_cython_6cygrpc_ServerCertificateConfig = 0;
static PyTypeObject *__pyx_ptype_4grpc_7_cython_6cygrpc_ServerCredentials = 0;
static PyTypeObject *__pyx_ptype_4grpc_7_cython_6cygrpc_CompletionQueue = 0;
static PyTypeObject *__pyx_ptype_4grpc_7_cython_6cygrpc_ConnectivityEvent = 0;
static PyTypeObject *__pyx_ptype_4grpc_7_cython_6cygrpc_RequestCallEvent = 0;
static PyTypeObject *__pyx_ptype_4grpc_7_cython_6cygrpc_BatchOperationEvent = 0;
static PyTypeObject *__pyx_ptype_4grpc_7_cython_6cygrpc_ServerShutdownEvent = 0;
static PyTypeObject *__pyx_ptype_4grpc_7_cython_6cygrpc_Operation = 0;
static PyTypeObject *__pyx_ptype_4grpc_7_cython_6cygrpc_SendInitialMetadataOperation = 0;
static PyTypeObject *__pyx_ptype_4grpc_7_cython_6cygrpc_SendMessageOperation = 0;
static PyTypeObject *__pyx_ptype_4grpc_7_cython_6cygrpc_SendCloseFromClientOperation = 0;
static PyTypeObject *__pyx_ptype_4grpc_7_cython_6cygrpc_SendStatusFromServerOperation = 0;
static PyTypeObject *__pyx_ptype_4grpc_7_cython_6cygrpc_ReceiveInitialMetadataOperation = 0;
static PyTypeObject *__pyx_ptype_4grpc_7_cython_6cygrpc_ReceiveMessageOperation = 0;
static PyTypeObject *__pyx_ptype_4grpc_7_cython_6cygrpc_ReceiveStatusOnClientOperation = 0;
static PyTypeObject *__pyx_ptype_4grpc_7_cython_6cygrpc_ReceiveCloseOnServerOperation = 0;
static PyTypeObject *__pyx_ptype_4grpc_7_cython_6cygrpc_CallDetails = 0;
static PyTypeObject *__pyx_ptype_4grpc_7_cython_6cygrpc_SslPemKeyCertPair = 0;
static PyTypeObject *__pyx_ptype_4grpc_7_cython_6cygrpc_CompressionOptions = 0;
static PyTypeObject *__pyx_ptype_4grpc_7_cython_6cygrpc_Server = 0;
static PyTypeObject *__pyx_ptype_4grpc_7_cython_6cygrpc__Tag = 0;
static PyTypeObject *__pyx_ptype_4grpc_7_cython_6cygrpc__ConnectivityTag = 0;
static PyTypeObject *__pyx_ptype_4grpc_7_cython_6cygrpc__RequestCallTag = 0;
static PyTypeObject *__pyx_ptype_4grpc_7_cython_6cygrpc__BatchOperationTag = 0;
static PyTypeObject *__pyx_ptype_4grpc_7_cython_6cygrpc__ServerShutdownTag = 0;
static PyTypeObject *__pyx_ptype_4grpc_7_cython_6cygrpc__VTable = 0;
static PyTypeObject *__pyx_ptype_4grpc_7_cython_6cygrpc_TimerWrapper = 0;
static PyTypeObject *__pyx_ptype_4grpc_7_cython_6cygrpc_SocketWrapper = 0;
static PyTypeObject *__pyx_ptype_4grpc_7_cython_6cygrpc_ResolveWrapper = 0;
static PyTypeObject *__pyx_ptype_4grpc_7_cython_6cygrpc__GrpcArgWrapper = 0;
static PyTypeObject *__pyx_ptype_4grpc_7_cython_6cygrpc_CensusContext = 0;
static PyTypeObject *__pyx_ptype_4grpc_7_cython_6cygrpc___pyx_scope_struct____pyx_f_4grpc_7_cython_6cygrpc__integrated_call = 0;
static PyTypeObject *__pyx_ptype_4grpc_7_cython_6cygrpc___pyx_scope_struct_1_next_event = 0;
static PyTypeObject *__pyx_ptype_4grpc_7_cython_6cygrpc___pyx_scope_struct_2___pyx_f_4grpc_7_cython_6cygrpc__segregated_call = 0;
static PyTypeObject *__pyx_ptype_4grpc_7_cython_6cygrpc___pyx_scope_struct_3_next_call_event = 0;
static PyTypeObject *__pyx_ptype_4grpc_7_cython_6cygrpc___pyx_scope_struct_4___pyx_f_4grpc_7_cython_6cygrpc__get_metadata = 0;
static PyTypeObject *__pyx_ptype_4grpc_7_cython_6cygrpc___pyx_scope_struct_5___pyx_f_4grpc_7_cython_6cygrpc__metadata = 0;
static PyTypeObject *__pyx_ptype_4grpc_7_cython_6cygrpc___pyx_scope_struct_6_genexpr = 0;
static PyTypeObject *__pyx_ptype_4grpc_7_cython_6cygrpc___pyx_scope_struct_7___pyx_f_4grpc_7_cython_6cygrpc_tuples_to_resolvaddr = 0;
static PyTypeObject *__pyx_ptype_4grpc_7_cython_6cygrpc___pyx_scope_struct_8_genexpr = 0;
static PyTypeObject *__pyx_ptype_4grpc_7_cython_6cygrpc___pyx_scope_struct_9___init__ = 0;
static int __pyx_v_4grpc_7_cython_6cygrpc__INTERRUPT_CHECK_PERIOD_MS;
static struct grpc_socket_vtable __pyx_v_4grpc_7_cython_6cygrpc_gevent_socket_vtable;
static struct grpc_custom_resolver_vtable __pyx_v_4grpc_7_cython_6cygrpc_gevent_resolver_vtable;
static struct grpc_custom_timer_vtable __pyx_v_4grpc_7_cython_6cygrpc_gevent_timer_vtable;
static struct grpc_custom_poller_vtable __pyx_v_4grpc_7_cython_6cygrpc_gevent_pollset_vtable;
static PyObject *__pyx_f_4grpc_7_cython_6cygrpc__check_call_error_no_metadata(PyObject *); /*proto*/
static PyObject *__pyx_f_4grpc_7_cython_6cygrpc__check_and_raise_call_error_no_metadata(PyObject *); /*proto*/
static int __pyx_f_4grpc_7_cython_6cygrpc__get_metadata(void *, grpc_auth_metadata_context, grpc_credentials_plugin_metadata_cb, void *, grpc_metadata *, size_t *, grpc_status_code *, char const **); /*proto*/
static void __pyx_f_4grpc_7_cython_6cygrpc__destroy(void *); /*proto*/
static grpc_call_credentials *__pyx_f_4grpc_7_cython_6cygrpc__composition(PyObject *); /*proto*/
static grpc_event __pyx_f_4grpc_7_cython_6cygrpc__next(grpc_completion_queue *, PyObject *); /*proto*/
static PyObject *__pyx_f_4grpc_7_cython_6cygrpc__interpret_event(grpc_event); /*proto*/
static void __pyx_f_4grpc_7_cython_6cygrpc__store_c_metadata(PyObject *, grpc_metadata **, size_t *); /*proto*/
static void __pyx_f_4grpc_7_cython_6cygrpc__release_c_metadata(grpc_metadata *, int); /*proto*/
static PyObject *__pyx_f_4grpc_7_cython_6cygrpc__metadatum(grpc_slice, grpc_slice); /*proto*/
static PyObject *__pyx_f_4grpc_7_cython_6cygrpc__metadata(grpc_metadata_array *); /*proto*/
static PyObject *__pyx_f_4grpc_7_cython_6cygrpc__slice_bytes(grpc_slice); /*proto*/
static grpc_slice __pyx_f_4grpc_7_cython_6cygrpc__slice_from_bytes(PyObject *); /*proto*/
static grpc_ssl_roots_override_result __pyx_f_4grpc_7_cython_6cygrpc_ssl_roots_override_callback(char **); /*proto*/
static gpr_timespec __pyx_f_4grpc_7_cython_6cygrpc__timespec_from_time(PyObject *); /*proto*/
static double __pyx_f_4grpc_7_cython_6cygrpc__time_from_timespec(gpr_timespec); /*proto*/
static void *__pyx_f_4grpc_7_cython_6cygrpc__copy_pointer(void *); /*proto*/
static void __pyx_f_4grpc_7_cython_6cygrpc__destroy_pointer(void *); /*proto*/
static int __pyx_f_4grpc_7_cython_6cygrpc__compare_pointer(void *, void *); /*proto*/
static PyObject *__pyx_f_4grpc_7_cython_6cygrpc__custom_op_on_c_call(int, grpc_call *); /*proto*/
static void __pyx_f_4grpc_7_cython_6cygrpc___prefork(void); /*proto*/
static void __pyx_f_4grpc_7_cython_6cygrpc___postfork_parent(void); /*proto*/
static void __pyx_f_4grpc_7_cython_6cygrpc___postfork_child(void); /*proto*/
static PyObject *__pyx_f_4grpc_7_cython_6cygrpc_str_to_bytes(PyObject *); /*proto*/
static PyObject *__pyx_f_4grpc_7_cython_6cygrpc__encode(PyObject *); /*proto*/
static PyObject *__pyx_f_4grpc_7_cython_6cygrpc__decode(PyObject *); /*proto*/
static PyObject *__pyx_f_4grpc_7_cython_6cygrpc__call_error_metadata(PyObject *); /*proto*/
static PyObject *__pyx_f_4grpc_7_cython_6cygrpc__call_error_no_metadata(PyObject *); /*proto*/
static PyObject *__pyx_f_4grpc_7_cython_6cygrpc__call_error(PyObject *, PyObject *); /*proto*/
static void __pyx_f_4grpc_7_cython_6cygrpc__raise_call_error_no_metadata(PyObject *); /*proto*/
static void __pyx_f_4grpc_7_cython_6cygrpc__raise_call_error(PyObject *, PyObject *); /*proto*/
static PyObject *__pyx_f_4grpc_7_cython_6cygrpc__destroy_c_completion_queue(grpc_completion_queue *); /*proto*/
static PyObject *__pyx_f_4grpc_7_cython_6cygrpc__operate(grpc_call *, PyObject *, PyObject *); /*proto*/
static PyObject *__pyx_f_4grpc_7_cython_6cygrpc__operate_from_integrated_call(struct __pyx_obj_4grpc_7_cython_6cygrpc__ChannelState *, struct __pyx_obj_4grpc_7_cython_6cygrpc__CallState *, PyObject *, PyObject *); /*proto*/
static PyObject *__pyx_f_4grpc_7_cython_6cygrpc__operate_from_segregated_call(struct __pyx_obj_4grpc_7_cython_6cygrpc__ChannelState *, struct __pyx_obj_4grpc_7_cython_6cygrpc__CallState *, PyObject *, PyObject *); /*proto*/
static PyObject *__pyx_f_4grpc_7_cython_6cygrpc__cancel(struct __pyx_obj_4grpc_7_cython_6cygrpc__ChannelState *, struct __pyx_obj_4grpc_7_cython_6cygrpc__CallState *, grpc_status_code, PyObject *); /*proto*/
static PyObject *__pyx_f_4grpc_7_cython_6cygrpc__next_call_event(struct __pyx_obj_4grpc_7_cython_6cygrpc__ChannelState *, grpc_completion_queue *, PyObject *, PyObject *); /*proto*/
static void __pyx_f_4grpc_7_cython_6cygrpc__call(struct __pyx_obj_4grpc_7_cython_6cygrpc__ChannelState *, struct __pyx_obj_4grpc_7_cython_6cygrpc__CallState *, grpc_completion_queue *, PyObject *, int, PyObject *, PyObject *, PyObject *, struct __pyx_obj_4grpc_7_cython_6cygrpc_CallCredentials *, PyObject *, PyObject *, PyObject *); /*proto*/
static void __pyx_f_4grpc_7_cython_6cygrpc__process_integrated_call_tag(struct __pyx_obj_4grpc_7_cython_6cygrpc__ChannelState *, struct __pyx_obj_4grpc_7_cython_6cygrpc__BatchOperationTag *); /*proto*/
static struct __pyx_obj_4grpc_7_cython_6cygrpc_IntegratedCall *__pyx_f_4grpc_7_cython_6cygrpc__integrated_call(struct __pyx_obj_4grpc_7_cython_6cygrpc__ChannelState *, int, PyObject *, PyObject *, PyObject *, PyObject *, struct __pyx_obj_4grpc_7_cython_6cygrpc_CallCredentials *, PyObject *, PyObject *); /*proto*/
static PyObject *__pyx_f_4grpc_7_cython_6cygrpc__process_segregated_call_tag(struct __pyx_obj_4grpc_7_cython_6cygrpc__ChannelState *, struct __pyx_obj_4grpc_7_cython_6cygrpc__CallState *, grpc_completion_queue *, struct __pyx_obj_4grpc_7_cython_6cygrpc__BatchOperationTag *); /*proto*/
static struct __pyx_obj_4grpc_7_cython_6cygrpc_SegregatedCall *__pyx_f_4grpc_7_cython_6cygrpc__segregated_call(struct __pyx_obj_4grpc_7_cython_6cygrpc__ChannelState *, int, PyObject *, PyObject *, PyObject *, PyObject *, struct __pyx_obj_4grpc_7_cython_6cygrpc_CallCredentials *, PyObject *, PyObject *); /*proto*/
static PyObject *__pyx_f_4grpc_7_cython_6cygrpc__watch_connectivity_state(struct __pyx_obj_4grpc_7_cython_6cygrpc__ChannelState *, grpc_connectivity_state, PyObject *); /*proto*/
static PyObject *__pyx_f_4grpc_7_cython_6cygrpc__close(struct __pyx_obj_4grpc_7_cython_6cygrpc_Channel *, grpc_status_code, PyObject *, PyObject *); /*proto*/
static PyObject *__pyx_f_4grpc_7_cython_6cygrpc__calls_drained(struct __pyx_obj_4grpc_7_cython_6cygrpc__ChannelState *); /*proto*/
static char const *__pyx_f_4grpc_7_cython_6cygrpc__get_c_pem_root_certs(PyObject *); /*proto*/
static grpc_ssl_pem_key_cert_pair *__pyx_f_4grpc_7_cython_6cygrpc__create_c_ssl_pem_key_cert_pairs(PyObject *); /*proto*/
static grpc_ssl_certificate_config_reload_status __pyx_f_4grpc_7_cython_6cygrpc__server_cert_config_fetcher_wrapper(void *, grpc_ssl_server_certificate_config **); /*proto*/
static PyObject *__pyx_f_4grpc_7_cython_6cygrpc__latent_event(grpc_completion_queue *, PyObject *); /*proto*/
static struct grpc_error *__pyx_f_4grpc_7_cython_6cygrpc_grpc_error_none(void); /*proto*/
static struct grpc_error *__pyx_f_4grpc_7_cython_6cygrpc_socket_error(PyObject *, PyObject *); /*proto*/
static PyObject *__pyx_f_4grpc_7_cython_6cygrpc_resolved_addr_to_tuple(grpc_resolved_address *); /*proto*/
static PyObject *__pyx_f_4grpc_7_cython_6cygrpc_sockaddr_to_tuple(grpc_sockaddr const *, size_t); /*proto*/
static PyObject *__pyx_f_4grpc_7_cython_6cygrpc_sockaddr_is_ipv4(grpc_sockaddr const *, size_t); /*proto*/
static grpc_resolved_addresses *__pyx_f_4grpc_7_cython_6cygrpc_tuples_to_resolvaddr(PyObject *); /*proto*/
static struct grpc_error *__pyx_f_4grpc_7_cython_6cygrpc_socket_init(struct grpc_custom_socket *, int); /*proto*/
static PyObject *__pyx_f_4grpc_7_cython_6cygrpc_socket_connect_async_cython(struct __pyx_obj_4grpc_7_cython_6cygrpc_SocketWrapper *, PyObject *); /*proto*/
static void __pyx_f_4grpc_7_cython_6cygrpc_socket_connect(struct grpc_custom_socket *, grpc_sockaddr const *, size_t, grpc_custom_connect_callback); /*proto*/
static void __pyx_f_4grpc_7_cython_6cygrpc_socket_destroy(struct grpc_custom_socket *); /*proto*/
static void __pyx_f_4grpc_7_cython_6cygrpc_socket_shutdown(struct grpc_custom_socket *); /*proto*/
static void __pyx_f_4grpc_7_cython_6cygrpc_socket_close(struct grpc_custom_socket *, grpc_custom_close_callback); /*proto*/
static PyObject *__pyx_f_4grpc_7_cython_6cygrpc_socket_write_async_cython(struct __pyx_obj_4grpc_7_cython_6cygrpc_SocketWrapper *, PyObject *); /*proto*/
static void __pyx_f_4grpc_7_cython_6cygrpc_socket_write(struct grpc_custom_socket *, struct grpc_slice_buffer *, grpc_custom_write_callback); /*proto*/
static PyObject *__pyx_f_4grpc_7_cython_6cygrpc_socket_read_async_cython(struct __pyx_obj_4grpc_7_cython_6cygrpc_SocketWrapper *); /*proto*/
static void __pyx_f_4grpc_7_cython_6cygrpc_socket_read(struct grpc_custom_socket *, char *, size_t, grpc_custom_read_callback); /*proto*/
static struct grpc_error *__pyx_f_4grpc_7_cython_6cygrpc_socket_getpeername(struct grpc_custom_socket *, grpc_sockaddr const *, int *); /*proto*/
static struct grpc_error *__pyx_f_4grpc_7_cython_6cygrpc_socket_getsockname(struct grpc_custom_socket *, grpc_sockaddr const *, int *); /*proto*/
static struct grpc_error *__pyx_f_4grpc_7_cython_6cygrpc_socket_bind(struct grpc_custom_socket *, grpc_sockaddr const *, size_t, int); /*proto*/
static struct grpc_error *__pyx_f_4grpc_7_cython_6cygrpc_socket_listen(struct grpc_custom_socket *); /*proto*/
static void __pyx_f_4grpc_7_cython_6cygrpc_accept_callback_cython(struct __pyx_obj_4grpc_7_cython_6cygrpc_SocketWrapper *); /*proto*/
static void __pyx_f_4grpc_7_cython_6cygrpc_socket_accept(struct grpc_custom_socket *, struct grpc_custom_socket *, grpc_custom_accept_callback); /*proto*/
static PyObject *__pyx_f_4grpc_7_cython_6cygrpc_socket_resolve_async_cython(struct __pyx_obj_4grpc_7_cython_6cygrpc_ResolveWrapper *); /*proto*/
static void __pyx_f_4grpc_7_cython_6cygrpc_socket_resolve_async(struct grpc_custom_resolver *, char *, char *); /*proto*/
static struct grpc_error *__pyx_f_4grpc_7_cython_6cygrpc_socket_resolve(char *, char *, grpc_resolved_addresses **); /*proto*/
static void __pyx_f_4grpc_7_cython_6cygrpc_timer_start(struct grpc_custom_timer *); /*proto*/
static void __pyx_f_4grpc_7_cython_6cygrpc_timer_stop(struct grpc_custom_timer *); /*proto*/
static void __pyx_f_4grpc_7_cython_6cygrpc_init_loop(void); /*proto*/
static void __pyx_f_4grpc_7_cython_6cygrpc_destroy_loop(void); /*proto*/
static void __pyx_f_4grpc_7_cython_6cygrpc_kick_loop(void); /*proto*/
static void __pyx_f_4grpc_7_cython_6cygrpc_run_loop(size_t); /*proto*/
static PyObject *__pyx_f_4grpc_7_cython_6cygrpc__initialize(void); /*proto*/
static PyObject *__pyx_f_4grpc_7_cython_6cygrpc___pyx_unpickle_ChannelCredentials__set_state(struct __pyx_obj_4grpc_7_cython_6cygrpc_ChannelCredentials *, PyObject *); /*proto*/
static PyObject *__pyx_f_4grpc_7_cython_6cygrpc___pyx_unpickle__Tag__set_state(struct __pyx_obj_4grpc_7_cython_6cygrpc__Tag *, PyObject *); /*proto*/
static PyObject *__pyx_f_4grpc_7_cython_6cygrpc___pyx_unpickle_CensusContext__set_state(struct __pyx_obj_4grpc_7_cython_6cygrpc_CensusContext *, PyObject *); /*proto*/
#define __Pyx_MODULE_NAME "grpc._cython.cygrpc"
extern int __pyx_module_is_main_grpc___cython__cygrpc;
int __pyx_module_is_main_grpc___cython__cygrpc = 0;

/* Implementation of 'grpc._cython.cygrpc' */
static PyObject *__pyx_builtin_object;
static PyObject *__pyx_builtin_TypeError;
static PyObject *__pyx_builtin_UnicodeDecodeError;
static PyObject *__pyx_builtin_enumerate;
static PyObject *__pyx_builtin_ValueError;
static PyObject *__pyx_builtin_NotImplementedError;
static PyObject *__pyx_builtin_range;
static PyObject *__pyx_builtin_RuntimeError;
static PyObject *__pyx_builtin_IOError;
static PyObject *__pyx_builtin_AttributeError;
static const char __pyx_k_0[] = "0";
static const char __pyx_k_1[] = "1";
static const char __pyx_k_s[] = "s";
static const char __pyx_k__2[] = "";
static const char __pyx_k_cb[] = "cb";
static const char __pyx_k_ok[] = "ok";
static const char __pyx_k_os[] = "os";
static const char __pyx_k_YES[] = "YES";
static const char __pyx_k_Yes[] = "Yes";
static const char __pyx_k_add[] = "add";
static const char __pyx_k_bin[] = "-bin";
static const char __pyx_k_doc[] = "__doc__";
static const char __pyx_k_get[] = "get";
static const char __pyx_k_hub[] = "hub";
static const char __pyx_k_int[] = "__int__";
static const char __pyx_k_key[] = "key";
static const char __pyx_k_low[] = "low";
static const char __pyx_k_new[] = "__new__";
static const char __pyx_k_pop[] = "pop";
static const char __pyx_k_set[] = "set";
static const char __pyx_k_sys[] = "sys";
static const char __pyx_k_tag[] = "tag";
static const char __pyx_k_yes[] = "yes";
static const char __pyx_k_Lock[] = "Lock";
static const char __pyx_k_TRUE[] = "TRUE";
static const char __pyx_k_True[] = "True";
static const char __pyx_k__131[] = ":";
static const char __pyx_k__132[] = "[";
static const char __pyx_k__134[] = "]";
static const char __pyx_k__136[] = "{}";
static const char __pyx_k_args[] = "args";
static const char __pyx_k_bind[] = "bind";
static const char __pyx_k_call[] = "call";
static const char __pyx_k_code[] = "code";
static const char __pyx_k_dict[] = "__dict__";
static const char __pyx_k_exit[] = "__exit__";
static const char __pyx_k_grpc[] = "grpc";
static const char __pyx_k_gzip[] = "gzip";
static const char __pyx_k_high[] = "high";
static const char __pyx_k_host[] = "host";
static const char __pyx_k_idle[] = "idle";
static const char __pyx_k_init[] = "__init__";
static const char __pyx_k_ipv4[] = "ipv4";
static const char __pyx_k_join[] = "join";
static const char __pyx_k_loop[] = "loop";
static const char __pyx_k_main[] = "__main__";
static const char __pyx_k_name[] = "name";
static const char __pyx_k_none[] = "none";
static const char __pyx_k_poll[] = "poll";
static const char __pyx_k_pool[] = "pool";
static const char __pyx_k_recv[] = "recv";
static const char __pyx_k_self[] = "self";
static const char __pyx_k_send[] = "send";
static const char __pyx_k_stop[] = "stop";
static const char __pyx_k_test[] = "__test__";
static const char __pyx_k_time[] = "time";
static const char __pyx_k_true[] = "true";
static const char __pyx_k_type[] = "type";
static const char __pyx_k_wait[] = "wait";
static const char __pyx_k_Event[] = "Event";
static const char __pyx_k_Group[] = "Group";
static const char __pyx_k_ascii[] = "ascii";
static const char __pyx_k_c_key[] = "c_key";
static const char __pyx_k_clear[] = "clear";
static const char __pyx_k_close[] = "close";
static const char __pyx_k_enter[] = "__enter__";
static const char __pyx_k_errno[] = "errno";
static const char __pyx_k_error[] = "error";
static const char __pyx_k_event[] = "event";
static const char __pyx_k_flags[] = "flags";
static const char __pyx_k_lower[] = "lower";
static const char __pyx_k_queue[] = "queue";
static const char __pyx_k_range[] = "range";
static const char __pyx_k_ready[] = "ready";
static const char __pyx_k_sleep[] = "sleep";
static const char __pyx_k_spawn[] = "spawn";
static const char __pyx_k_start[] = "start";
static const char __pyx_k_throw[] = "throw";
static const char __pyx_k_timer[] = "timer";
static const char __pyx_k_value[] = "value";
static const char __pyx_k_LOGGER[] = "_LOGGER";
static const char __pyx_k_Thread[] = "Thread";
static const char __pyx_k_accept[] = "accept";
static const char __pyx_k_append[] = "append";
static const char __pyx_k_cygrpc[] = ".cygrpc";
static const char __pyx_k_daemon[] = "daemon";
static const char __pyx_k_encode[] = "encode";
static const char __pyx_k_exit_2[] = "_exit";
static const char __pyx_k_failed[] = "{} failed: {}";
static const char __pyx_k_format[] = "format";
static const char __pyx_k_g_pool[] = "g_pool";
static const char __pyx_k_gevent[] = "gevent";
static const char __pyx_k_import[] = "__import__";
static const char __pyx_k_listen[] = "listen";
static const char __pyx_k_lstrip[] = "lstrip";
static const char __pyx_k_medium[] = "medium";
static const char __pyx_k_method[] = "method";
static const char __pyx_k_module[] = "__module__";
static const char __pyx_k_name_2[] = "__name__";
static const char __pyx_k_object[] = "object";
static const char __pyx_k_pickle[] = "pickle";
static const char __pyx_k_reduce[] = "__reduce__";
static const char __pyx_k_rstrip[] = "rstrip";
static const char __pyx_k_socket[] = "socket";
static const char __pyx_k_status[] = "status";
static const char __pyx_k_target[] = "target";
static const char __pyx_k_thread[] = "_thread";
static const char __pyx_k_update[] = "update";
static const char __pyx_k_values[] = "values";
static const char __pyx_k_vtable[] = "vtable";
static const char __pyx_k_0_0_0_0[] = "0.0.0.0";
static const char __pyx_k_AF_INET[] = "AF_INET";
static const char __pyx_k_IOError[] = "IOError";
static const char __pyx_k_aborted[] = "aborted";
static const char __pyx_k_address[] = "address";
static const char __pyx_k_cb_func[] = "cb_func";
static const char __pyx_k_channel[] = "channel";
static const char __pyx_k_connect[] = "connect";
static const char __pyx_k_context[] = "context";
static const char __pyx_k_deflate[] = "deflate";
static const char __pyx_k_details[] = "details";
static const char __pyx_k_discard[] = "discard";
static const char __pyx_k_environ[] = "environ";
static const char __pyx_k_g_event[] = "g_event";
static const char __pyx_k_genexpr[] = "genexpr";
static const char __pyx_k_get_hub[] = "get_hub";
static const char __pyx_k_logging[] = "logging";
static const char __pyx_k_message[] = "message";
static const char __pyx_k_os_path[] = "os.path";
static const char __pyx_k_pkgutil[] = "pkgutil";
static const char __pyx_k_prepare[] = "__prepare__";
static const char __pyx_k_sendmsg[] = "sendmsg";
static const char __pyx_k_success[] = "success";
static const char __pyx_k_unknown[] = "unknown";
static const char __pyx_k_AF_INET6[] = "AF_INET6";
static const char __pyx_k_ENOTCONN[] = "ENOTCONN";
static const char __pyx_k_EX_USAGE[] = "EX_USAGE";
static const char __pyx_k_callback[] = "callback";
static const char __pyx_k_capacity[] = "capacity";
static const char __pyx_k_channels[] = "channels";
static const char __pyx_k_daemonic[] = "daemonic";
static const char __pyx_k_deadline[] = "deadline";
static const char __pyx_k_end_time[] = "end_time";
static const char __pyx_k_endswith[] = "endswith";
static const char __pyx_k_exc_info[] = "exc_info";
static const char __pyx_k_gaierror[] = "gaierror";
static const char __pyx_k_get_data[] = "get_data";
static const char __pyx_k_getstate[] = "__getstate__";
static const char __pyx_k_gevent_g[] = "gevent_g";
static const char __pyx_k_greenlet[] = "greenlet";
static const char __pyx_k_internal[] = "internal";
static const char __pyx_k_is_valid[] = "is_valid";
static const char __pyx_k_metadata[] = "metadata";
static const char __pyx_k_property[] = "property";
static const char __pyx_k_pyx_type[] = "__pyx_type";
static const char __pyx_k_qualname[] = "__qualname__";
static const char __pyx_k_setstate[] = "__setstate__";
static const char __pyx_k_shutdown[] = "shutdown";
static const char __pyx_k_user_tag[] = "user_tag";
static const char __pyx_k_CallError[] = "CallError";
static const char __pyx_k_Condition[] = "Condition";
static const char __pyx_k_ForkState[] = "_ForkState";
static const char __pyx_k_Metadatum[] = "_Metadatum";
static const char __pyx_k_SHUT_RDWR[] = "SHUT_RDWR";
static const char __pyx_k_TypeError[] = "TypeError";
static const char __pyx_k_WriteFlag[] = "WriteFlag";
static const char __pyx_k_algorithm[] = "algorithm";
static const char __pyx_k_arguments[] = "arguments";
static const char __pyx_k_c_options[] = "c_options";
static const char __pyx_k_cancelled[] = "cancelled";
static const char __pyx_k_condition[] = "_condition";
static const char __pyx_k_data_loss[] = "data_loss";
static const char __pyx_k_decrement[] = "decrement";
static const char __pyx_k_enumerate[] = "enumerate";
static const char __pyx_k_exception[] = "exception";
static const char __pyx_k_getLogger[] = "getLogger";
static const char __pyx_k_increment[] = "increment";
static const char __pyx_k_metaclass[] = "__metaclass__";
static const char __pyx_k_not_found[] = "not_found";
static const char __pyx_k_on_finish[] = "on_finish";
static const char __pyx_k_pyx_state[] = "__pyx_state";
static const char __pyx_k_reduce_ex[] = "__reduce_ex__";
static const char __pyx_k_server_id[] = "server_id";
static const char __pyx_k_setDaemon[] = "setDaemon";
static const char __pyx_k_socket_id[] = "socket_id";
static const char __pyx_k_threading[] = "threading";
static const char __pyx_k_used_mask[] = "used_mask";
static const char __pyx_k_wait_time[] = "wait_time";
static const char __pyx_k_SOL_SOCKET[] = "SOL_SOCKET";
static const char __pyx_k_StatusCode[] = "StatusCode";
static const char __pyx_k_ValueError[] = "ValueError";
static const char __pyx_k_addr_tuple[] = "addr_tuple";
static const char __pyx_k_call_queue[] = "call_queue";
static const char __pyx_k_call_state[] = "call_state";
static const char __pyx_k_census_ctx[] = "census_ctx";
static const char __pyx_k_channel_id[] = "channel_id";
static const char __pyx_k_connecting[] = "connecting";
static const char __pyx_k_error_code[] = "error_code";
static const char __pyx_k_fork_epoch[] = "fork_epoch";
static const char __pyx_k_fork_state[] = "_fork_state";
static const char __pyx_k_gevent_hub[] = "gevent_hub";
static const char __pyx_k_identities[] = "identities";
static const char __pyx_k_namedtuple[] = "namedtuple";
static const char __pyx_k_notify_all[] = "notify_all";
static const char __pyx_k_on_success[] = "on_success";
static const char __pyx_k_operations[] = "operations";
static const char __pyx_k_properties[] = "properties";
static const char __pyx_k_pyx_result[] = "__pyx_result";
static const char __pyx_k_pyx_vtable[] = "__pyx_vtable__";
static const char __pyx_k_setsockopt[] = "setsockopt";
static const char __pyx_k_IPPROTO_TCP[] = "IPPROTO_TCP";
static const char __pyx_k_PickleError[] = "PickleError";
static const char __pyx_k_TCP_NODELAY[] = "TCP_NODELAY";
static const char __pyx_k_TRUE_VALUES[] = "_TRUE_VALUES";
static const char __pyx_k_buffer_hint[] = "buffer_hint";
static const char __pyx_k_cert_config[] = "cert_config";
static const char __pyx_k_collections[] = "collections";
static const char __pyx_k_credentials[] = "credentials";
static const char __pyx_k_getaddrinfo[] = "getaddrinfo";
static const char __pyx_k_getpeername[] = "getpeername";
static const char __pyx_k_getsockname[] = "getsockname";
static const char __pyx_k_gevent_pool[] = "gevent.pool";
static const char __pyx_k_max_results[] = "max_results";
static const char __pyx_k_no_compress[] = "no_compress";
static const char __pyx_k_not_invoked[] = "not_invoked";
static const char __pyx_k_private_key[] = "private_key";
static const char __pyx_k_retain_self[] = "retain_self";
static const char __pyx_k_shutdown_cq[] = "shutdown_cq";
static const char __pyx_k_start_batch[] = "_start_batch";
static const char __pyx_k_started_tag[] = "started_tag";
static const char __pyx_k_unavailable[] = "unavailable";
static const char __pyx_k_write_bytes[] = "write_bytes";
static const char __pyx_k_RuntimeError[] = "RuntimeError";
static const char __pyx_k_SO_REUSEADDR[] = "SO_REUSEADDR";
static const char __pyx_k_auth_context[] = "auth_context";
static const char __pyx_k_call_details[] = "call_details";
static const char __pyx_k_gevent_event[] = "gevent_event";
static const char __pyx_k_gevent_hub_2[] = "gevent.hub";
static const char __pyx_k_out_of_range[] = "out_of_range";
static const char __pyx_k_pyx_checksum[] = "__pyx_checksum";
static const char __pyx_k_send_message[] = "send_message";
static const char __pyx_k_server_queue[] = "server_queue";
static const char __pyx_k_started_tags[] = "started_tags";
static const char __pyx_k_stringsource[] = "stringsource";
static const char __pyx_k_timeout_secs[] = "timeout_secs";
static const char __pyx_k_ChannelArgKey[] = "ChannelArgKey";
static const char __pyx_k_OperationType[] = "OperationType";
static const char __pyx_k_applysockopts[] = "applysockopts";
static const char __pyx_k_c_cert_config[] = "c_cert_config";
static const char __pyx_k_callback_func[] = "callback_func";
static const char __pyx_k_channel_state[] = "channel_state";
static const char __pyx_k_close_on_fork[] = "_close_on_fork";
static const char __pyx_k_enable_census[] = "enable_census";
static const char __pyx_k_error_details[] = "error_details";
static const char __pyx_k_gevent_socket[] = "gevent_socket";
static const char __pyx_k_invalid_flags[] = "invalid_flags";
static const char __pyx_k_not_on_client[] = "not_on_client";
static const char __pyx_k_not_on_server[] = "not_on_server";
static const char __pyx_k_queue_timeout[] = "queue_timeout";
static const char __pyx_k_reduce_cython[] = "__reduce_cython__";
static const char __pyx_k_subchannel_id[] = "subchannel_id";
static const char __pyx_k_unimplemented[] = "unimplemented";
static const char __pyx_k_AttributeError[] = "AttributeError";
static const char __pyx_k_CompletionType[] = "CompletionType";
static const char __pyx_k_already_exists[] = "already_exists";
static const char __pyx_k_c_returned_str[] = "c_returned_str";
static const char __pyx_k_get_fork_epoch[] = "get_fork_epoch";
static const char __pyx_k_gevent_event_2[] = "gevent.event";
static const char __pyx_k_managed_target[] = "managed_target";
static const char __pyx_k_pem_root_certs[] = "pem_root_certs";
static const char __pyx_k_queue_shutdown[] = "queue_shutdown";
static const char __pyx_k_socket_sendmsg[] = "socket_sendmsg";
static const char __pyx_k_socket_wrapper[] = "socket_wrapper";
static const char __pyx_k_spawn_greenlet[] = "_spawn_greenlet";
static const char __pyx_k_wait_for_ready[] = "wait_for_ready";
static const char __pyx_k_already_invoked[] = "already_invoked";
static const char __pyx_k_completion_type[] = "completion_type";
static const char __pyx_k_gevent_socket_2[] = "gevent.socket";
static const char __pyx_k_metadata_plugin[] = "metadata_plugin";
static const char __pyx_k_next_call_event[] = "next_call_event";
static const char __pyx_k_peer_identities[] = "peer_identities";
static const char __pyx_k_py_auth_context[] = "py_auth_context";
static const char __pyx_k_pyx_PickleError[] = "__pyx_PickleError";
static const char __pyx_k_receive_message[] = "receive_message";
static const char __pyx_k_resolve_wrapper[] = "resolve_wrapper";
static const char __pyx_k_setstate_cython[] = "__setstate_cython__";
static const char __pyx_k_start_server_id[] = "start_server_id";
static const char __pyx_k_start_socket_id[] = "start_socket_id";
static const char __pyx_k_unauthenticated[] = "unauthenticated";
static const char __pyx_k_CompressionLevel[] = "CompressionLevel";
static const char __pyx_k_Expected_str_not[] = "Expected str, not {}";
static const char __pyx_k_ForkState___init[] = "_ForkState.__init__";
static const char __pyx_k_already_accepted[] = "already_accepted";
static const char __pyx_k_already_finished[] = "already_finished";
static const char __pyx_k_batch_operations[] = "batch_operations";
static const char __pyx_k_c_pem_root_certs[] = "c_pem_root_certs";
static const char __pyx_k_fork_in_progress[] = "fork_in_progress";
static const char __pyx_k_init_grpc_gevent[] = "init_grpc_gevent";
static const char __pyx_k_initial_metadata[] = "initial_metadata";
static const char __pyx_k_invalid_argument[] = "invalid_argument";
static const char __pyx_k_invalid_metadata[] = "invalid_metadata";
static const char __pyx_k_start_channel_id[] = "start_channel_id";
static const char __pyx_k_ActiveThreadCount[] = "_ActiveThreadCount";
static const char __pyx_k_ConnectivityState[] = "ConnectivityState";
static const char __pyx_k_ForkManagedThread[] = "ForkManagedThread";
static const char __pyx_k_certificate_chain[] = "certificate_chain";
static const char __pyx_k_deadline_exceeded[] = "deadline_exceeded";
static const char __pyx_k_default_authority[] = "default_authority";
static const char __pyx_k_force_client_auth[] = "force_client_auth";
static const char __pyx_k_peer_identity_key[] = "peer_identity_key";
static const char __pyx_k_permission_denied[] = "permission_denied";
static const char __pyx_k_pyx_unpickle__Tag[] = "__pyx_unpickle__Tag";
static const char __pyx_k_socket_read_async[] = "socket_read_async";
static const char __pyx_k_ssl_session_cache[] = "ssl_session_cache";
static const char __pyx_k_trailing_metadata[] = "trailing_metadata";
static const char __pyx_k_transient_failure[] = "transient_failure";
static const char __pyx_k_uninstall_context[] = "uninstall_context";
static const char __pyx_k_UnicodeDecodeError[] = "UnicodeDecodeError";
static const char __pyx_k_await_zero_threads[] = "await_zero_threads";
static const char __pyx_k_call_credentialses[] = "call_credentialses";
static const char __pyx_k_cline_in_traceback[] = "cline_in_traceback";
static const char __pyx_k_num_active_threads[] = "_num_active_threads";
static const char __pyx_k_operation_complete[] = "operation_complete";
static const char __pyx_k_pem_key_cert_pairs[] = "pem_key_cert_pairs";
static const char __pyx_k_queue_must_be_live[] = "queue must be live";
static const char __pyx_k_resource_exhausted[] = "resource_exhausted";
static const char __pyx_k_server_credentials[] = "server_credentials";
static const char __pyx_k_socket_write_async[] = "socket_write_async";
static const char __pyx_k_Cannot_invoke_RPC_s[] = "Cannot invoke RPC: %s";
static const char __pyx_k_NotImplementedError[] = "NotImplementedError";
static const char __pyx_k_active_thread_count[] = "active_thread_count";
static const char __pyx_k_async_callback_func[] = "async_callback_func";
static const char __pyx_k_cert_config_fetcher[] = "cert_config_fetcher";
static const char __pyx_k_channel_credentials[] = "channel_credentials";
static const char __pyx_k_channelz_get_server[] = "channelz_get_server";
static const char __pyx_k_channelz_get_socket[] = "channelz_get_socket";
static const char __pyx_k_failed_precondition[] = "failed_precondition";
static const char __pyx_k_fork_handler_failed[] = "_fork_handler_failed";
static const char __pyx_k_grpc__cython_cygrpc[] = "grpc._cython.cygrpc";
static const char __pyx_k_initial_cert_config[] = "initial_cert_config";
static const char __pyx_k_invocation_metadata[] = "invocation_metadata";
static const char __pyx_k_last_observed_state[] = "last_observed_state";
static const char __pyx_k_socket_accept_async[] = "socket_accept_async";
static const char __pyx_k_too_many_operations[] = "too_many_operations";
static const char __pyx_k_CompressionAlgorithm[] = "CompressionAlgorithm";
static const char __pyx_k_InitialMetadataFlags[] = "InitialMetadataFlags";
static const char __pyx_k_PropagationConstants[] = "PropagationConstants";
static const char __pyx_k_build_census_context[] = "build_census_context";
static const char __pyx_k_channelz_get_channel[] = "channelz_get_channel";
static const char __pyx_k_channelz_get_servers[] = "channelz_get_servers";
static const char __pyx_k_reset_postfork_child[] = "reset_postfork_child";
static const char __pyx_k_shutting_down_server[] = "shutting_down_server";
static const char __pyx_k_socket_connect_async[] = "socket_connect_async";
static const char __pyx_k_spawn_callback_async[] = "_spawn_callback_async";
static const char __pyx_k_Invalid_encoding_on_s[] = "Invalid encoding on %s";
static const char __pyx_k_credentials_roots_pem[] = "_credentials/roots.pem";
static const char __pyx_k_fork_register_channel[] = "fork_register_channel";
static const char __pyx_k_operationses_and_tags[] = "operationses_and_tags";
static const char __pyx_k_pem_root_certificates[] = "pem_root_certificates";
static const char __pyx_k_send_initial_metadata[] = "send_initial_metadata";
static const char __pyx_k_ForkManagedThread_join[] = "ForkManagedThread.join";
static const char __pyx_k_max_concurrent_streams[] = "max_concurrent_streams";
static const char __pyx_k_metadata_was_invalid_s[] = "metadata was invalid: %s";
static const char __pyx_k_send_close_from_client[] = "send_close_from_client";
static const char __pyx_k_server_credentials_ssl[] = "server_credentials_ssl";
static const char __pyx_k_ForkManagedThread_start[] = "ForkManagedThread.start";
static const char __pyx_k_GRPC_PROPAGATE_DEADLINE[] = "GRPC_PROPAGATE_DEADLINE";
static const char __pyx_k_GRPC_PROPAGATE_DEFAULTS[] = "GRPC_PROPAGATE_DEFAULTS";
static const char __pyx_k_channelz_get_subchannel[] = "channelz_get_subchannel";
static const char __pyx_k_fork_handler_registered[] = "fork_handler_registered";
static const char __pyx_k_fork_unregister_channel[] = "fork_unregister_channel";
static const char __pyx_k_is_fork_support_enabled[] = "is_fork_support_enabled";
static const char __pyx_k_max_send_message_length[] = "max_send_message_length";
static const char __pyx_k_metadata_locals_genexpr[] = "_metadata.<locals>.genexpr";
static const char __pyx_k_postfork_state_to_reset[] = "postfork_state_to_reset";
static const char __pyx_k_receive_close_on_server[] = "receive_close_on_server";
static const char __pyx_k_send_status_from_server[] = "send_status_from_server";
static const char __pyx_k_set_async_callback_func[] = "set_async_callback_func";
static const char __pyx_k_ActiveThreadCount___init[] = "_ActiveThreadCount.__init__";
static const char __pyx_k_ForkManagedThread___init[] = "ForkManagedThread.__init__";
static const char __pyx_k_GRPC_ENABLE_FORK_SUPPORT[] = "_GRPC_ENABLE_FORK_SUPPORT";
static const char __pyx_k_postfork_states_to_reset[] = "postfork_states_to_reset";
static const char __pyx_k_receive_initial_metadata[] = "receive_initial_metadata";
static const char __pyx_k_receive_status_on_client[] = "receive_status_on_client";
static const char __pyx_k_spawn_callback_in_thread[] = "_spawn_callback_in_thread";
static const char __pyx_k_ssl_target_name_override[] = "ssl_target_name_override";
static const char __pyx_k_block_if_fork_in_progress[] = "block_if_fork_in_progress";
static const char __pyx_k_certificate_configuration[] = "_certificate_configuration";
static const char __pyx_k_channelz_get_top_channels[] = "channelz_get_top_channels";
static const char __pyx_k_get_deadline_from_context[] = "get_deadline_from_context";
static const char __pyx_k_primary_user_agent_string[] = "primary_user_agent_string";
static const char __pyx_k_register_completion_queue[] = "register_completion_queue";
static const char __pyx_k_GRPC_ENABLE_FORK_SUPPORT_2[] = "GRPC_ENABLE_FORK_SUPPORT";
static const char __pyx_k_Internal_gRPC_call_error_d[] = "Internal gRPC call error %d. ";
static const char __pyx_k_compression_algorithm_name[] = "compression_algorithm_name";
static const char __pyx_k_fork_in_progress_condition[] = "fork_in_progress_condition";
static const char __pyx_k_max_receive_message_length[] = "max_receive_message_length";
static const char __pyx_k_pyx_unpickle_CensusContext[] = "__pyx_unpickle_CensusContext";
static const char __pyx_k_set_census_context_on_call[] = "set_census_context_on_call";
static const char __pyx_k_ActiveThreadCount_decrement[] = "_ActiveThreadCount.decrement";
static const char __pyx_k_ActiveThreadCount_increment[] = "_ActiveThreadCount.increment";
static const char __pyx_k_ForkManagedThread_setDaemon[] = "ForkManagedThread.setDaemon";
static const char __pyx_k_GRPC_PROPAGATE_CANCELLATION[] = "GRPC_PROPAGATE_CANCELLATION";
static const char __pyx_k_channelz_get_server_sockets[] = "channelz_get_server_sockets";
static const char __pyx_k_fork_handlers_and_grpc_init[] = "fork_handlers_and_grpc_init";
static const char __pyx_k_secondary_user_agent_string[] = "secondary_user_agent_string";
static const char __pyx_k_socket_resolve_async_python[] = "socket_resolve_async_python";
static const char __pyx_k_enter_user_request_generator[] = "enter_user_request_generator";
static const char __pyx_k_fork_handler_registered_lock[] = "fork_handler_registered_lock";
static const char __pyx_k_get_metadata_locals_callback[] = "_get_metadata.<locals>.callback";
static const char __pyx_k_next_event_locals_on_success[] = "next_event.<locals>.on_success";
static const char __pyx_k_AWAIT_THREADS_TIMEOUT_SECONDS[] = "_AWAIT_THREADS_TIMEOUT_SECONDS";
static const char __pyx_k_http2_initial_sequence_number[] = "http2_initial_sequence_number";
static const char __pyx_k_server_certificate_config_ssl[] = "server_certificate_config_ssl";
static const char __pyx_k_the_server_hasn_t_started_yet[] = "the server hasn't started yet";
static const char __pyx_k_wait_for_ready_explicitly_set[] = "wait_for_ready_explicitly_set";
static const char __pyx_k_ServerCertificateConfiguration[] = "ServerCertificateConfiguration";
static const char __pyx_k_pyx_unpickle_ChannelCredential[] = "__pyx_unpickle_ChannelCredentials";
static const char __pyx_k_src_python_grpcio_grpc__cython[] = "src/python/grpcio/grpc/_cython/_cygrpc/channel.pyx.pxi";
static const char __pyx_k_the_server_has_already_started[] = "the server has already started";
static const char __pyx_k_ActiveThreadCount_await_zero_th[] = "_ActiveThreadCount.await_zero_threads";
static const char __pyx_k_Failed_to_get_the_server_please[] = "Failed to get the server, please ensure your server_id==%s is valid";
static const char __pyx_k_Failed_to_get_the_socket_please[] = "Failed to get the socket, please ensure your socket_id==%s is valid";
static const char __pyx_k_INTERNAL_CALL_ERROR_MESSAGE_FOR[] = "_INTERNAL_CALL_ERROR_MESSAGE_FORMAT";
static const char __pyx_k_No_custom_hooks_are_implemented[] = "No custom hooks are implemented";
static const char __pyx_k_init_grpc_gevent_locals_cb_func[] = "init_grpc_gevent.<locals>.cb_func";
static const char __pyx_k_integrated_call_locals_on_succe[] = "_integrated_call.<locals>.on_success";
static const char __pyx_k_segregated_call_locals_on_succe[] = "_segregated_call.<locals>.on_success";
static const char __pyx_k_self_arg_cannot_be_converted_to[] = "self.arg cannot be converted to a Python object for pickling";
static const char __pyx_k_Cannot_invoke_RPC_on_closed_chan[] = "Cannot invoke RPC on closed channel!";
static const char __pyx_k_Error_fetching_certificate_confi[] = "Error fetching certificate config";
static const char __pyx_k_Exiting_child_due_to_raised_exce[] = "Exiting child due to raised exception";
static const char __pyx_k_Expected_bytes_str_or_unicode_no[] = "Expected bytes, str, or unicode, not {}";
static const char __pyx_k_Expected_int_bytes_or_behavior_g[] = "Expected int, bytes, or behavior, got {}";
static const char __pyx_k_Failed_to_get_server_sockets_ple[] = "Failed to get server sockets, please ensure your server_id==%s and start_socket_id==%s and max_results==%s is valid";
static const char __pyx_k_Failed_to_get_servers_please_ens[] = "Failed to get servers, please ensure your start_server_id==%s is valid";
static const char __pyx_k_Failed_to_get_the_channel_please[] = "Failed to get the channel, please ensure your channel_id==%s is valid";
static const char __pyx_k_Failed_to_get_the_subchannel_ple[] = "Failed to get the subchannel, please ensure your subchannel_id==%s is valid";
static const char __pyx_k_Failed_to_get_top_channels_pleas[] = "Failed to get top channels, please ensure your start_channel_id==%s is valid";
static const char __pyx_k_Failed_to_shutdown_gRPC_Core_aft[] = "Failed to shutdown gRPC Core after fork()";
static const char __pyx_k_Failed_to_shutdown_gRPC_Python_t[] = "Failed to shutdown gRPC Python threads prior to fork. Behavior after fork will be undefined.";
static const char __pyx_k_ForkManagedThread___init___local[] = "ForkManagedThread.__init__.<locals>.managed_target";
static const char __pyx_k_GRPC_COMPRESSION_CHANNEL_DEFAULT[] = "GRPC_COMPRESSION_CHANNEL_DEFAULT_ALGORITHM";
static const char __pyx_k_GRPC_COMPRESSION_REQUEST_ALGORIT[] = "GRPC_COMPRESSION_REQUEST_ALGORITHM_MD_KEY";
static const char __pyx_k_GRPC_PROPAGATE_CENSUS_STATS_CONT[] = "GRPC_PROPAGATE_CENSUS_STATS_CONTEXT";
static const char __pyx_k_GRPC_PROPAGATE_CENSUS_TRACING_CO[] = "GRPC_PROPAGATE_CENSUS_TRACING_CONTEXT";
static const char __pyx_k_Incompatible_checksums_s_vs_0xd4[] = "Incompatible checksums (%s vs 0xd41d8cd = ())";
static const char __pyx_k_Please_report_to_https_github_co[] = "Please report to https://github.com/grpc/grpc/issues";
static const char __pyx_k_cannot_register_completion_queue[] = "cannot register completion queues after start";
static const char __pyx_k_cert_config_fetcher_must_be_call[] = "cert_config_fetcher must be callable";
static const char __pyx_k_expected_certificate_to_be_bytes[] = "expected certificate to be bytes, got %s";
static const char __pyx_k_expected_pem_key_cert_pairs_to_b[] = "expected pem_key_cert_pairs to be sequence of SslPemKeyCertPair";
static const char __pyx_k_expected_registered_completion_q[] = "expected registered completion queue";
static const char __pyx_k_grpc_python__cygrpc__GrpcArgWrap[] = "grpc.python._cygrpc._GrpcArgWrapper";
static const char __pyx_k_if_error_code_is_specified_so_mu[] = "if error_code is specified, so must details (and vice-versa)";
static const char __pyx_k_initial_cert_config_must_be_a_gr[] = "initial_cert_config must be a grpc.ServerCertificateConfiguration";
static const char __pyx_k_install_context_from_request_cal[] = "install_context_from_request_call_event";
static const char __pyx_k_internal_error_user_data_must_be[] = "internal error: user_data must be specified";
static const char __pyx_k_invalid_call_object_cannot_be_us[] = "invalid call object cannot be used from Python";
static const char __pyx_k_next_call_event_locals_on_succes[] = "next_call_event.<locals>.on_success";
static const char __pyx_k_no_default___reduce___due_to_non[] = "no default __reduce__ due to non-trivial __cinit__";
static const char __pyx_k_queue_must_be_shutting_down_to_b[] = "queue must be shutting down to be cleared";
static const char __pyx_k_return_from_user_request_generat[] = "return_from_user_request_generator";
static const char __pyx_k_self_c_argument_cannot_be_conver[] = "self.c_argument cannot be converted to a Python object for pickling";
static const char __pyx_k_self_c_credentials_cannot_be_con[] = "self.c_credentials cannot be converted to a Python object for pickling";
static const char __pyx_k_self_c_op_cannot_be_converted_to[] = "self.c_op cannot be converted to a Python object for pickling";
static const char __pyx_k_server_credentials_ssl_dynamic_c[] = "server_credentials_ssl_dynamic_cert_config";
static const char __pyx_k_server_must_be_started_and_not_s[] = "server must be started and not shutting down";
static const char __pyx_k_server_queue_must_be_a_registere[] = "server_queue must be a registered completion queue";
static const char __pyx_k_src_python_grpcio_grpc__cython_2[] = "src/python/grpcio/grpc/_cython/_cygrpc/credentials.pyx.pxi";
static const char __pyx_k_src_python_grpcio_grpc__cython_3[] = "src/python/grpcio/grpc/_cython/_cygrpc/grpc_gevent.pyx.pxi";
static const char __pyx_k_src_python_grpcio_grpc__cython_4[] = "src/python/grpcio/grpc/_cython/_cygrpc/fork_posix.pyx.pxi";
static const char __pyx_k_src_python_grpcio_grpc__cython_5[] = "src/python/grpcio/grpc/_cython/_cygrpc/channelz.pyx.pxi";
static const char __pyx_k_src_python_grpcio_grpc__cython_6[] = "src/python/grpcio/grpc/_cython/_cygrpc/records.pyx.pxi";
static const char __pyx_k_src_python_grpcio_grpc__cython_7[] = "src/python/grpcio/grpc/_cython/_cygrpc/security.pyx.pxi";
static const char __pyx_k_src_python_grpcio_grpc__cython_8[] = "src/python/grpcio/grpc/_cython/_cygrpc/_hooks.pyx.pxi";
static const char __pyx_k_the_server_must_be_shutting_down[] = "the server must be shutting down to cancel all calls";
static const char __pyx_k_tuples_to_resolvaddr_locals_gene[] = "tuples_to_resolvaddr.<locals>.genexpr";
static const char __pyx_k_Error_fetching_certificate_confi_2[] = "Error fetching certificate configuration: certificate configuration must be of type grpc.ServerCertificateConfiguration, not %s";
static PyObject *__pyx_kp_s_0;
static PyObject *__pyx_kp_s_0_0_0_0;
static PyObject *__pyx_kp_s_1;
static PyObject *__pyx_n_s_AF_INET;
static PyObject *__pyx_n_s_AF_INET6;
static PyObject *__pyx_n_s_AWAIT_THREADS_TIMEOUT_SECONDS;
static PyObject *__pyx_n_s_ActiveThreadCount;
static PyObject *__pyx_n_s_ActiveThreadCount___init;
static PyObject *__pyx_n_s_ActiveThreadCount_await_zero_th;
static PyObject *__pyx_n_s_ActiveThreadCount_decrement;
static PyObject *__pyx_n_s_ActiveThreadCount_increment;
static PyObject *__pyx_n_s_AttributeError;
static PyObject *__pyx_n_s_CallError;
static PyObject *__pyx_kp_s_Cannot_invoke_RPC_on_closed_chan;
static PyObject *__pyx_kp_s_Cannot_invoke_RPC_s;
static PyObject *__pyx_n_s_ChannelArgKey;
static PyObject *__pyx_n_s_CompletionType;
static PyObject *__pyx_n_s_CompressionAlgorithm;
static PyObject *__pyx_n_s_CompressionLevel;
static PyObject *__pyx_n_s_Condition;
static PyObject *__pyx_n_s_ConnectivityState;
static PyObject *__pyx_n_s_ENOTCONN;
static PyObject *__pyx_n_s_EX_USAGE;
static PyObject *__pyx_kp_s_Error_fetching_certificate_confi;
static PyObject *__pyx_kp_s_Error_fetching_certificate_confi_2;
static PyObject *__pyx_n_s_Event;
static PyObject *__pyx_kp_s_Exiting_child_due_to_raised_exce;
static PyObject *__pyx_kp_s_Expected_bytes_str_or_unicode_no;
static PyObject *__pyx_kp_s_Expected_int_bytes_or_behavior_g;
static PyObject *__pyx_kp_s_Expected_str_not;
static PyObject *__pyx_kp_s_Failed_to_get_server_sockets_ple;
static PyObject *__pyx_kp_s_Failed_to_get_servers_please_ens;
static PyObject *__pyx_kp_s_Failed_to_get_the_channel_please;
static PyObject *__pyx_kp_s_Failed_to_get_the_server_please;
static PyObject *__pyx_kp_s_Failed_to_get_the_socket_please;
static PyObject *__pyx_kp_s_Failed_to_get_the_subchannel_ple;
static PyObject *__pyx_kp_s_Failed_to_get_top_channels_pleas;
static PyObject *__pyx_kp_s_Failed_to_shutdown_gRPC_Core_aft;
static PyObject *__pyx_kp_s_Failed_to_shutdown_gRPC_Python_t;
static PyObject *__pyx_n_s_ForkManagedThread;
static PyObject *__pyx_n_s_ForkManagedThread___init;
static PyObject *__pyx_n_s_ForkManagedThread___init___local;
static PyObject *__pyx_n_s_ForkManagedThread_join;
static PyObject *__pyx_n_s_ForkManagedThread_setDaemon;
static PyObject *__pyx_n_s_ForkManagedThread_start;
static PyObject *__pyx_n_s_ForkState;
static PyObject *__pyx_n_s_ForkState___init;
static PyObject *__pyx_n_s_GRPC_COMPRESSION_CHANNEL_DEFAULT;
static PyObject *__pyx_n_s_GRPC_COMPRESSION_REQUEST_ALGORIT;
static PyObject *__pyx_n_s_GRPC_ENABLE_FORK_SUPPORT;
static PyObject *__pyx_n_s_GRPC_ENABLE_FORK_SUPPORT_2;
static PyObject *__pyx_n_s_GRPC_PROPAGATE_CANCELLATION;
static PyObject *__pyx_n_s_GRPC_PROPAGATE_CENSUS_STATS_CONT;
static PyObject *__pyx_n_s_GRPC_PROPAGATE_CENSUS_TRACING_CO;
static PyObject *__pyx_n_s_GRPC_PROPAGATE_DEADLINE;
static PyObject *__pyx_n_s_GRPC_PROPAGATE_DEFAULTS;
static PyObject *__pyx_n_s_Group;
static PyObject *__pyx_n_s_INTERNAL_CALL_ERROR_MESSAGE_FOR;
static PyObject *__pyx_n_s_IOError;
static PyObject *__pyx_n_s_IPPROTO_TCP;
static PyObject *__pyx_kp_s_Incompatible_checksums_s_vs_0xd4;
static PyObject *__pyx_n_s_InitialMetadataFlags;
static PyObject *__pyx_kp_s_Internal_gRPC_call_error_d;
static PyObject *__pyx_kp_s_Invalid_encoding_on_s;
static PyObject *__pyx_n_s_LOGGER;
static PyObject *__pyx_n_s_Lock;
static PyObject *__pyx_n_s_Metadatum;
static PyObject *__pyx_kp_s_No_custom_hooks_are_implemented;
static PyObject *__pyx_n_s_NotImplementedError;
static PyObject *__pyx_n_s_OperationType;
static PyObject *__pyx_n_s_PickleError;
static PyObject *__pyx_kp_s_Please_report_to_https_github_co;
static PyObject *__pyx_n_s_PropagationConstants;
static PyObject *__pyx_n_s_RuntimeError;
static PyObject *__pyx_n_s_SHUT_RDWR;
static PyObject *__pyx_n_s_SOL_SOCKET;
static PyObject *__pyx_n_s_SO_REUSEADDR;
static PyObject *__pyx_n_s_ServerCertificateConfiguration;
static PyObject *__pyx_n_s_StatusCode;
static PyObject *__pyx_n_s_TCP_NODELAY;
static PyObject *__pyx_n_s_TRUE;
static PyObject *__pyx_n_s_TRUE_VALUES;
static PyObject *__pyx_n_s_Thread;
static PyObject *__pyx_n_s_True;
static PyObject *__pyx_n_s_TypeError;
static PyObject *__pyx_n_s_UnicodeDecodeError;
static PyObject *__pyx_n_s_ValueError;
static PyObject *__pyx_n_s_WriteFlag;
static PyObject *__pyx_n_s_YES;
static PyObject *__pyx_n_s_Yes;
static PyObject *__pyx_kp_s__131;
static PyObject *__pyx_kp_s__132;
static PyObject *__pyx_kp_s__134;
static PyObject *__pyx_kp_s__136;
static PyObject *__pyx_kp_b__2;
static PyObject *__pyx_kp_s__2;
static PyObject *__pyx_n_s_aborted;
static PyObject *__pyx_n_s_accept;
static PyObject *__pyx_n_s_active_thread_count;
static PyObject *__pyx_n_s_add;
static PyObject *__pyx_n_s_addr_tuple;
static PyObject *__pyx_n_s_address;
static PyObject *__pyx_n_s_algorithm;
static PyObject *__pyx_n_s_already_accepted;
static PyObject *__pyx_n_s_already_exists;
static PyObject *__pyx_n_s_already_finished;
static PyObject *__pyx_n_s_already_invoked;
static PyObject *__pyx_n_s_append;
static PyObject *__pyx_n_s_applysockopts;
static PyObject *__pyx_n_s_args;
static PyObject *__pyx_n_s_arguments;
static PyObject *__pyx_n_s_ascii;
static PyObject *__pyx_n_s_async_callback_func;
static PyObject *__pyx_n_s_auth_context;
static PyObject *__pyx_n_s_await_zero_threads;
static PyObject *__pyx_n_s_batch_operations;
static PyObject *__pyx_kp_b_bin;
static PyObject *__pyx_n_s_bind;
static PyObject *__pyx_n_s_block_if_fork_in_progress;
static PyObject *__pyx_n_s_buffer_hint;
static PyObject *__pyx_n_s_build_census_context;
static PyObject *__pyx_n_s_c_cert_config;
static PyObject *__pyx_n_s_c_key;
static PyObject *__pyx_n_s_c_options;
static PyObject *__pyx_n_s_c_pem_root_certs;
static PyObject *__pyx_n_s_c_returned_str;
static PyObject *__pyx_n_s_call;
static PyObject *__pyx_n_s_call_credentialses;
static PyObject *__pyx_n_s_call_details;
static PyObject *__pyx_n_s_call_queue;
static PyObject *__pyx_n_s_call_state;
static PyObject *__pyx_n_s_callback;
static PyObject *__pyx_n_s_callback_func;
static PyObject *__pyx_n_s_cancelled;
static PyObject *__pyx_kp_s_cannot_register_completion_queue;
static PyObject *__pyx_n_s_capacity;
static PyObject *__pyx_n_s_cb;
static PyObject *__pyx_n_s_cb_func;
static PyObject *__pyx_n_s_census_ctx;
static PyObject *__pyx_n_s_cert_config;
static PyObject *__pyx_n_s_cert_config_fetcher;
static PyObject *__pyx_kp_s_cert_config_fetcher_must_be_call;
static PyObject *__pyx_n_s_certificate_chain;
static PyObject *__pyx_n_s_certificate_configuration;
static PyObject *__pyx_n_s_channel;
static PyObject *__pyx_n_s_channel_credentials;
static PyObject *__pyx_n_s_channel_id;
static PyObject *__pyx_n_s_channel_state;
static PyObject *__pyx_n_s_channels;
static PyObject *__pyx_n_s_channelz_get_channel;
static PyObject *__pyx_n_s_channelz_get_server;
static PyObject *__pyx_n_s_channelz_get_server_sockets;
static PyObject *__pyx_n_s_channelz_get_servers;
static PyObject *__pyx_n_s_channelz_get_socket;
static PyObject *__pyx_n_s_channelz_get_subchannel;
static PyObject *__pyx_n_s_channelz_get_top_channels;
static PyObject *__pyx_n_s_clear;
static PyObject *__pyx_n_s_cline_in_traceback;
static PyObject *__pyx_n_s_close;
static PyObject *__pyx_n_s_close_on_fork;
static PyObject *__pyx_n_s_code;
static PyObject *__pyx_n_s_collections;
static PyObject *__pyx_n_s_completion_type;
static PyObject *__pyx_n_s_compression_algorithm_name;
static PyObject *__pyx_n_s_condition;
static PyObject *__pyx_n_s_connect;
static PyObject *__pyx_n_s_connecting;
static PyObject *__pyx_n_s_context;
static PyObject *__pyx_n_s_credentials;
static PyObject *__pyx_kp_s_credentials_roots_pem;
static PyObject *__pyx_kp_s_cygrpc;
static PyObject *__pyx_n_s_daemon;
static PyObject *__pyx_n_s_daemonic;
static PyObject *__pyx_n_s_data_loss;
static PyObject *__pyx_n_s_deadline;
static PyObject *__pyx_n_s_deadline_exceeded;
static PyObject *__pyx_n_s_decrement;
static PyObject *__pyx_n_s_default_authority;
static PyObject *__pyx_n_s_deflate;
static PyObject *__pyx_n_s_details;
static PyObject *__pyx_n_s_dict;
static PyObject *__pyx_n_s_discard;
static PyObject *__pyx_n_s_doc;
static PyObject *__pyx_n_s_enable_census;
static PyObject *__pyx_n_s_encode;
static PyObject *__pyx_n_s_end_time;
static PyObject *__pyx_n_s_endswith;
static PyObject *__pyx_n_s_enter;
static PyObject *__pyx_n_s_enter_user_request_generator;
static PyObject *__pyx_n_s_enumerate;
static PyObject *__pyx_n_s_environ;
static PyObject *__pyx_n_s_errno;
static PyObject *__pyx_n_s_error;
static PyObject *__pyx_n_s_error_code;
static PyObject *__pyx_n_s_error_details;
static PyObject *__pyx_n_s_event;
static PyObject *__pyx_n_s_exc_info;
static PyObject *__pyx_n_s_exception;
static PyObject *__pyx_n_s_exit;
static PyObject *__pyx_n_s_exit_2;
static PyObject *__pyx_kp_s_expected_certificate_to_be_bytes;
static PyObject *__pyx_kp_s_expected_pem_key_cert_pairs_to_b;
static PyObject *__pyx_kp_s_expected_registered_completion_q;
static PyObject *__pyx_kp_s_failed;
static PyObject *__pyx_n_s_failed_precondition;
static PyObject *__pyx_n_s_flags;
static PyObject *__pyx_n_s_force_client_auth;
static PyObject *__pyx_n_s_fork_epoch;
static PyObject *__pyx_n_s_fork_handler_failed;
static PyObject *__pyx_n_s_fork_handler_registered;
static PyObject *__pyx_n_s_fork_handler_registered_lock;
static PyObject *__pyx_n_s_fork_handlers_and_grpc_init;
static PyObject *__pyx_n_s_fork_in_progress;
static PyObject *__pyx_n_s_fork_in_progress_condition;
static PyObject *__pyx_n_s_fork_register_channel;
static PyObject *__pyx_n_s_fork_state;
static PyObject *__pyx_n_s_fork_unregister_channel;
static PyObject *__pyx_n_s_format;
static PyObject *__pyx_n_s_g_event;
static PyObject *__pyx_n_s_g_pool;
static PyObject *__pyx_n_s_gaierror;
static PyObject *__pyx_n_s_genexpr;
static PyObject *__pyx_n_s_get;
static PyObject *__pyx_n_s_getLogger;
static PyObject *__pyx_n_s_get_data;
static PyObject *__pyx_n_s_get_deadline_from_context;
static PyObject *__pyx_n_s_get_fork_epoch;
static PyObject *__pyx_n_s_get_hub;
static PyObject *__pyx_n_s_get_metadata_locals_callback;
static PyObject *__pyx_n_s_getaddrinfo;
static PyObject *__pyx_n_s_getpeername;
static PyObject *__pyx_n_s_getsockname;
static PyObject *__pyx_n_s_getstate;
static PyObject *__pyx_n_s_gevent;
static PyObject *__pyx_n_s_gevent_event;
static PyObject *__pyx_n_s_gevent_event_2;
static PyObject *__pyx_n_s_gevent_g;
static PyObject *__pyx_n_s_gevent_hub;
static PyObject *__pyx_n_s_gevent_hub_2;
static PyObject *__pyx_n_s_gevent_pool;
static PyObject *__pyx_n_s_gevent_socket;
static PyObject *__pyx_n_s_gevent_socket_2;
static PyObject *__pyx_n_s_greenlet;
static PyObject *__pyx_n_s_grpc;
static PyObject *__pyx_n_s_grpc__cython_cygrpc;
static PyObject *__pyx_kp_s_grpc_python__cygrpc__GrpcArgWrap;
static PyObject *__pyx_n_s_gzip;
static PyObject *__pyx_n_s_high;
static PyObject *__pyx_n_s_host;
static PyObject *__pyx_n_s_http2_initial_sequence_number;
static PyObject *__pyx_n_s_hub;
static PyObject *__pyx_n_s_identities;
static PyObject *__pyx_n_s_idle;
static PyObject *__pyx_kp_s_if_error_code_is_specified_so_mu;
static PyObject *__pyx_n_s_import;
static PyObject *__pyx_n_s_increment;
static PyObject *__pyx_n_s_init;
static PyObject *__pyx_n_s_init_grpc_gevent;
static PyObject *__pyx_n_s_init_grpc_gevent_locals_cb_func;
static PyObject *__pyx_n_s_initial_cert_config;
static PyObject *__pyx_kp_s_initial_cert_config_must_be_a_gr;
static PyObject *__pyx_n_s_initial_metadata;
static PyObject *__pyx_n_s_install_context_from_request_cal;
static PyObject *__pyx_n_s_int;
static PyObject *__pyx_n_s_integrated_call_locals_on_succe;
static PyObject *__pyx_n_s_internal;
static PyObject *__pyx_kp_s_internal_error_user_data_must_be;
static PyObject *__pyx_n_s_invalid_argument;
static PyObject *__pyx_kp_s_invalid_call_object_cannot_be_us;
static PyObject *__pyx_n_s_invalid_flags;
static PyObject *__pyx_n_s_invalid_metadata;
static PyObject *__pyx_n_s_invocation_metadata;
static PyObject *__pyx_n_b_ipv4;
static PyObject *__pyx_n_s_is_fork_support_enabled;
static PyObject *__pyx_n_s_is_valid;
static PyObject *__pyx_n_s_join;
static PyObject *__pyx_n_s_key;
static PyObject *__pyx_n_s_last_observed_state;
static PyObject *__pyx_n_s_listen;
static PyObject *__pyx_n_s_logging;
static PyObject *__pyx_n_s_loop;
static PyObject *__pyx_n_s_low;
static PyObject *__pyx_n_s_lower;
static PyObject *__pyx_n_s_lstrip;
static PyObject *__pyx_n_s_main;
static PyObject *__pyx_n_s_managed_target;
static PyObject *__pyx_n_s_max_concurrent_streams;
static PyObject *__pyx_n_s_max_receive_message_length;
static PyObject *__pyx_n_s_max_results;
static PyObject *__pyx_n_s_max_send_message_length;
static PyObject *__pyx_n_s_medium;
static PyObject *__pyx_n_s_message;
static PyObject *__pyx_n_s_metaclass;
static PyObject *__pyx_n_s_metadata;
static PyObject *__pyx_n_s_metadata_locals_genexpr;
static PyObject *__pyx_n_s_metadata_plugin;
static PyObject *__pyx_kp_s_metadata_was_invalid_s;
static PyObject *__pyx_n_s_method;
static PyObject *__pyx_n_s_module;
static PyObject *__pyx_n_s_name;
static PyObject *__pyx_n_s_name_2;
static PyObject *__pyx_n_s_namedtuple;
static PyObject *__pyx_n_s_new;
static PyObject *__pyx_n_s_next_call_event;
static PyObject *__pyx_n_s_next_call_event_locals_on_succes;
static PyObject *__pyx_n_s_next_event_locals_on_success;
static PyObject *__pyx_n_s_no_compress;
static PyObject *__pyx_kp_s_no_default___reduce___due_to_non;
static PyObject *__pyx_n_s_none;
static PyObject *__pyx_n_s_not_found;
static PyObject *__pyx_n_s_not_invoked;
static PyObject *__pyx_n_s_not_on_client;
static PyObject *__pyx_n_s_not_on_server;
static PyObject *__pyx_n_s_notify_all;
static PyObject *__pyx_n_s_num_active_threads;
static PyObject *__pyx_n_s_object;
static PyObject *__pyx_n_s_ok;
static PyObject *__pyx_n_s_on_finish;
static PyObject *__pyx_n_s_on_success;
static PyObject *__pyx_n_s_operation_complete;
static PyObject *__pyx_n_s_operations;
static PyObject *__pyx_n_s_operationses_and_tags;
static PyObject *__pyx_n_s_os;
static PyObject *__pyx_n_s_os_path;
static PyObject *__pyx_n_s_out_of_range;
static PyObject *__pyx_n_s_peer_identities;
static PyObject *__pyx_n_s_peer_identity_key;
static PyObject *__pyx_n_s_pem_key_cert_pairs;
static PyObject *__pyx_n_s_pem_root_certificates;
static PyObject *__pyx_n_s_pem_root_certs;
static PyObject *__pyx_n_s_permission_denied;
static PyObject *__pyx_n_s_pickle;
static PyObject *__pyx_n_s_pkgutil;
static PyObject *__pyx_n_s_poll;
static PyObject *__pyx_n_s_pool;
static PyObject *__pyx_n_s_pop;
static PyObject *__pyx_n_s_postfork_state_to_reset;
static PyObject *__pyx_n_s_postfork_states_to_reset;
static PyObject *__pyx_n_s_prepare;
static PyObject *__pyx_n_s_primary_user_agent_string;
static PyObject *__pyx_n_s_private_key;
static PyObject *__pyx_n_s_properties;
static PyObject *__pyx_n_s_property;
static PyObject *__pyx_n_s_py_auth_context;
static PyObject *__pyx_n_s_pyx_PickleError;
static PyObject *__pyx_n_s_pyx_checksum;
static PyObject *__pyx_n_s_pyx_result;
static PyObject *__pyx_n_s_pyx_state;
static PyObject *__pyx_n_s_pyx_type;
static PyObject *__pyx_n_s_pyx_unpickle_CensusContext;
static PyObject *__pyx_n_s_pyx_unpickle_ChannelCredential;
static PyObject *__pyx_n_s_pyx_unpickle__Tag;
static PyObject *__pyx_n_s_pyx_vtable;
static PyObject *__pyx_n_s_qualname;
static PyObject *__pyx_n_s_queue;
static PyObject *__pyx_kp_s_queue_must_be_live;
static PyObject *__pyx_kp_s_queue_must_be_shutting_down_to_b;
static PyObject *__pyx_n_s_queue_shutdown;
static PyObject *__pyx_n_s_queue_timeout;
static PyObject *__pyx_n_s_range;
static PyObject *__pyx_n_s_ready;
static PyObject *__pyx_n_s_receive_close_on_server;
static PyObject *__pyx_n_s_receive_initial_metadata;
static PyObject *__pyx_n_s_receive_message;
static PyObject *__pyx_n_s_receive_status_on_client;
static PyObject *__pyx_n_s_recv;
static PyObject *__pyx_n_s_reduce;
static PyObject *__pyx_n_s_reduce_cython;
static PyObject *__pyx_n_s_reduce_ex;
static PyObject *__pyx_n_s_register_completion_queue;
static PyObject *__pyx_n_s_reset_postfork_child;
static PyObject *__pyx_n_s_resolve_wrapper;
static PyObject *__pyx_n_s_resource_exhausted;
static PyObject *__pyx_n_s_retain_self;
static PyObject *__pyx_n_s_return_from_user_request_generat;
static PyObject *__pyx_n_s_rstrip;
static PyObject *__pyx_n_s_s;
static PyObject *__pyx_n_s_secondary_user_agent_string;
static PyObject *__pyx_n_s_segregated_call_locals_on_succe;
static PyObject *__pyx_n_s_self;
static PyObject *__pyx_kp_s_self_arg_cannot_be_converted_to;
static PyObject *__pyx_kp_s_self_c_argument_cannot_be_conver;
static PyObject *__pyx_kp_s_self_c_credentials_cannot_be_con;
static PyObject *__pyx_kp_s_self_c_op_cannot_be_converted_to;
static PyObject *__pyx_n_s_send;
static PyObject *__pyx_n_s_send_close_from_client;
static PyObject *__pyx_n_s_send_initial_metadata;
static PyObject *__pyx_n_s_send_message;
static PyObject *__pyx_n_s_send_status_from_server;
static PyObject *__pyx_n_s_sendmsg;
static PyObject *__pyx_n_s_server_certificate_config_ssl;
static PyObject *__pyx_n_s_server_credentials;
static PyObject *__pyx_n_s_server_credentials_ssl;
static PyObject *__pyx_n_s_server_credentials_ssl_dynamic_c;
static PyObject *__pyx_n_s_server_id;
static PyObject *__pyx_kp_s_server_must_be_started_and_not_s;
static PyObject *__pyx_n_s_server_queue;
static PyObject *__pyx_kp_s_server_queue_must_be_a_registere;
static PyObject *__pyx_n_s_set;
static PyObject *__pyx_n_s_setDaemon;
static PyObject *__pyx_n_s_set_async_callback_func;
static PyObject *__pyx_n_s_set_census_context_on_call;
static PyObject *__pyx_n_s_setsockopt;
static PyObject *__pyx_n_s_setstate;
static PyObject *__pyx_n_s_setstate_cython;
static PyObject *__pyx_n_s_shutdown;
static PyObject *__pyx_n_s_shutdown_cq;
static PyObject *__pyx_n_s_shutting_down_server;
static PyObject *__pyx_n_s_sleep;
static PyObject *__pyx_n_s_socket;
static PyObject *__pyx_n_s_socket_accept_async;
static PyObject *__pyx_n_s_socket_connect_async;
static PyObject *__pyx_n_s_socket_id;
static PyObject *__pyx_n_s_socket_read_async;
static PyObject *__pyx_n_s_socket_resolve_async_python;
static PyObject *__pyx_n_s_socket_sendmsg;
static PyObject *__pyx_n_s_socket_wrapper;
static PyObject *__pyx_n_s_socket_write_async;
static PyObject *__pyx_n_s_spawn;
static PyObject *__pyx_n_s_spawn_callback_async;
static PyObject *__pyx_n_s_spawn_callback_in_thread;
static PyObject *__pyx_n_s_spawn_greenlet;
static PyObject *__pyx_kp_s_src_python_grpcio_grpc__cython;
static PyObject *__pyx_kp_s_src_python_grpcio_grpc__cython_2;
static PyObject *__pyx_kp_s_src_python_grpcio_grpc__cython_3;
static PyObject *__pyx_kp_s_src_python_grpcio_grpc__cython_4;
static PyObject *__pyx_kp_s_src_python_grpcio_grpc__cython_5;
static PyObject *__pyx_kp_s_src_python_grpcio_grpc__cython_6;
static PyObject *__pyx_kp_s_src_python_grpcio_grpc__cython_7;
static PyObject *__pyx_kp_s_src_python_grpcio_grpc__cython_8;
static PyObject *__pyx_n_s_ssl_session_cache;
static PyObject *__pyx_n_s_ssl_target_name_override;
static PyObject *__pyx_n_s_start;
static PyObject *__pyx_n_s_start_batch;
static PyObject *__pyx_n_s_start_channel_id;
static PyObject *__pyx_n_s_start_server_id;
static PyObject *__pyx_n_s_start_socket_id;
static PyObject *__pyx_n_s_started_tag;
static PyObject *__pyx_n_s_started_tags;
static PyObject *__pyx_n_s_status;
static PyObject *__pyx_n_s_stop;
static PyObject *__pyx_kp_s_stringsource;
static PyObject *__pyx_n_s_subchannel_id;
static PyObject *__pyx_n_s_success;
static PyObject *__pyx_n_s_sys;
static PyObject *__pyx_n_s_tag;
static PyObject *__pyx_n_s_target;
static PyObject *__pyx_n_s_test;
static PyObject *__pyx_kp_s_the_server_has_already_started;
static PyObject *__pyx_kp_s_the_server_hasn_t_started_yet;
static PyObject *__pyx_kp_s_the_server_must_be_shutting_down;
static PyObject *__pyx_n_s_thread;
static PyObject *__pyx_n_s_threading;
static PyObject *__pyx_n_s_throw;
static PyObject *__pyx_n_s_time;
static PyObject *__pyx_n_s_timeout_secs;
static PyObject *__pyx_n_s_timer;
static PyObject *__pyx_n_s_too_many_operations;
static PyObject *__pyx_n_s_trailing_metadata;
static PyObject *__pyx_n_s_transient_failure;
static PyObject *__pyx_n_s_true;
static PyObject *__pyx_n_s_tuples_to_resolvaddr_locals_gene;
static PyObject *__pyx_n_s_type;
static PyObject *__pyx_n_s_unauthenticated;
static PyObject *__pyx_n_s_unavailable;
static PyObject *__pyx_n_s_unimplemented;
static PyObject *__pyx_n_s_uninstall_context;
static PyObject *__pyx_n_s_unknown;
static PyObject *__pyx_n_s_update;
static PyObject *__pyx_n_s_used_mask;
static PyObject *__pyx_n_s_user_tag;
static PyObject *__pyx_n_s_value;
static PyObject *__pyx_n_s_values;
static PyObject *__pyx_n_s_vtable;
static PyObject *__pyx_n_s_wait;
static PyObject *__pyx_n_s_wait_for_ready;
static PyObject *__pyx_n_s_wait_for_ready_explicitly_set;
static PyObject *__pyx_n_s_wait_time;
static PyObject *__pyx_n_s_write_bytes;
static PyObject *__pyx_n_s_yes;
static PyObject *__pyx_pf_4grpc_7_cython_6cygrpc_15_GrpcArgWrapper___reduce_cython__(CYTHON_UNUSED struct __pyx_obj_4grpc_7_cython_6cygrpc__GrpcArgWrapper *__pyx_v_self); /* proto */
static PyObject *__pyx_pf_4grpc_7_cython_6cygrpc_15_GrpcArgWrapper_2__setstate_cython__(CYTHON_UNUSED struct __pyx_obj_4grpc_7_cython_6cygrpc__GrpcArgWrapper *__pyx_v_self, CYTHON_UNUSED PyObject *__pyx_v___pyx_state); /* proto */
static PyObject *__pyx_pf_4grpc_7_cython_6cygrpc_11_ChannelArg___reduce_cython__(CYTHON_UNUSED struct __pyx_obj_4grpc_7_cython_6cygrpc__ChannelArg *__pyx_v_self); /* proto */
static PyObject *__pyx_pf_4grpc_7_cython_6cygrpc_11_ChannelArg_2__setstate_cython__(CYTHON_UNUSED struct __pyx_obj_4grpc_7_cython_6cygrpc__ChannelArg *__pyx_v_self, CYTHON_UNUSED PyObject *__pyx_v___pyx_state); /* proto */
static int __pyx_pf_4grpc_7_cython_6cygrpc_12_ChannelArgs___cinit__(struct __pyx_obj_4grpc_7_cython_6cygrpc__ChannelArgs *__pyx_v_self, PyObject *__pyx_v_arguments, struct __pyx_obj_4grpc_7_cython_6cygrpc__VTable *__pyx_v_vtable); /* proto */
static void __pyx_pf_4grpc_7_cython_6cygrpc_12_ChannelArgs_2__dealloc__(struct __pyx_obj_4grpc_7_cython_6cygrpc__ChannelArgs *__pyx_v_self); /* proto */
static PyObject *__pyx_pf_4grpc_7_cython_6cygrpc_12_ChannelArgs_10_arguments___get__(struct __pyx_obj_4grpc_7_cython_6cygrpc__ChannelArgs *__pyx_v_self); /* proto */
static PyObject *__pyx_pf_4grpc_7_cython_6cygrpc_12_ChannelArgs_11_references___get__(struct __pyx_obj_4grpc_7_cython_6cygrpc__ChannelArgs *__pyx_v_self); /* proto */
static PyObject *__pyx_pf_4grpc_7_cython_6cygrpc_12_ChannelArgs_4__reduce_cython__(CYTHON_UNUSED struct __pyx_obj_4grpc_7_cython_6cygrpc__ChannelArgs *__pyx_v_self); /* proto */
static PyObject *__pyx_pf_4grpc_7_cython_6cygrpc_12_ChannelArgs_6__setstate_cython__(CYTHON_UNUSED struct __pyx_obj_4grpc_7_cython_6cygrpc__ChannelArgs *__pyx_v_self, CYTHON_UNUSED PyObject *__pyx_v___pyx_state); /* proto */
static int __pyx_pf_4grpc_7_cython_6cygrpc_4Call___cinit__(struct __pyx_obj_4grpc_7_cython_6cygrpc_Call *__pyx_v_self, struct __pyx_obj_4grpc_7_cython_6cygrpc__VTable *__pyx_v_vtable); /* proto */
static PyObject *__pyx_pf_4grpc_7_cython_6cygrpc_4Call_2_start_batch(struct __pyx_obj_4grpc_7_cython_6cygrpc_Call *__pyx_v_self, PyObject *__pyx_v_operations, PyObject *__pyx_v_tag, PyObject *__pyx_v_retain_self); /* proto */
static PyObject *__pyx_pf_4grpc_7_cython_6cygrpc_4Call_4start_client_batch(struct __pyx_obj_4grpc_7_cython_6cygrpc_Call *__pyx_v_self, PyObject *__pyx_v_operations, PyObject *__pyx_v_tag); /* proto */
static PyObject *__pyx_pf_4grpc_7_cython_6cygrpc_4Call_6start_server_batch(struct __pyx_obj_4grpc_7_cython_6cygrpc_Call *__pyx_v_self, PyObject *__pyx_v_operations, PyObject *__pyx_v_tag); /* proto */
static PyObject *__pyx_pf_4grpc_7_cython_6cygrpc_4Call_8cancel(struct __pyx_obj_4grpc_7_cython_6cygrpc_Call *__pyx_v_self, grpc_status_code __pyx_v_error_code, PyObject *__pyx_v_details); /* proto */
static PyObject *__pyx_pf_4grpc_7_cython_6cygrpc_4Call_10set_credentials(struct __pyx_obj_4grpc_7_cython_6cygrpc_Call *__pyx_v_self, struct __pyx_obj_4grpc_7_cython_6cygrpc_CallCredentials *__pyx_v_call_credentials); /* proto */
static PyObject *__pyx_pf_4grpc_7_cython_6cygrpc_4Call_12peer(struct __pyx_obj_4grpc_7_cython_6cygrpc_Call *__pyx_v_self); /* proto */
static void __pyx_pf_4grpc_7_cython_6cygrpc_4Call_14__dealloc__(struct __pyx_obj_4grpc_7_cython_6cygrpc_Call *__pyx_v_self); /* proto */
static PyObject *__pyx_pf_4grpc_7_cython_6cygrpc_4Call_8is_valid___get__(struct __pyx_obj_4grpc_7_cython_6cygrpc_Call *__pyx_v_self); /* proto */
static PyObject *__pyx_pf_4grpc_7_cython_6cygrpc_4Call_16_custom_op_on_c_call(struct __pyx_obj_4grpc_7_cython_6cygrpc_Call *__pyx_v_self, int __pyx_v_op); /* proto */
static PyObject *__pyx_pf_4grpc_7_cython_6cygrpc_4Call_18__reduce_cython__(CYTHON_UNUSED struct __pyx_obj_4grpc_7_cython_6cygrpc_Call *__pyx_v_self); /* proto */
static PyObject *__pyx_pf_4grpc_7_cython_6cygrpc_4Call_20__setstate_cython__(CYTHON_UNUSED struct __pyx_obj_4grpc_7_cython_6cygrpc_Call *__pyx_v_self, CYTHON_UNUSED PyObject *__pyx_v___pyx_state); /* proto */
static int __pyx_pf_4grpc_7_cython_6cygrpc_10_CallState___cinit__(struct __pyx_obj_4grpc_7_cython_6cygrpc__CallState *__pyx_v_self); /* proto */
static PyObject *__pyx_pf_4grpc_7_cython_6cygrpc_10_CallState_2__reduce_cython__(CYTHON_UNUSED struct __pyx_obj_4grpc_7_cython_6cygrpc__CallState *__pyx_v_self); /* proto */
static PyObject *__pyx_pf_4grpc_7_cython_6cygrpc_10_CallState_4__setstate_cython__(CYTHON_UNUSED struct __pyx_obj_4grpc_7_cython_6cygrpc__CallState *__pyx_v_self, CYTHON_UNUSED PyObject *__pyx_v___pyx_state); /* proto */
static int __pyx_pf_4grpc_7_cython_6cygrpc_13_ChannelState___cinit__(struct __pyx_obj_4grpc_7_cython_6cygrpc__ChannelState *__pyx_v_self); /* proto */
static PyObject *__pyx_pf_4grpc_7_cython_6cygrpc_13_ChannelState_2__reduce_cython__(CYTHON_UNUSED struct __pyx_obj_4grpc_7_cython_6cygrpc__ChannelState *__pyx_v_self); /* proto */
static PyObject *__pyx_pf_4grpc_7_cython_6cygrpc_13_ChannelState_4__setstate_cython__(CYTHON_UNUSED struct __pyx_obj_4grpc_7_cython_6cygrpc__ChannelState *__pyx_v_self, CYTHON_UNUSED PyObject *__pyx_v___pyx_state); /* proto */
static int __pyx_pf_4grpc_7_cython_6cygrpc_14IntegratedCall___cinit__(struct __pyx_obj_4grpc_7_cython_6cygrpc_IntegratedCall *__pyx_v_self, struct __pyx_obj_4grpc_7_cython_6cygrpc__ChannelState *__pyx_v_channel_state, struct __pyx_obj_4grpc_7_cython_6cygrpc__CallState *__pyx_v_call_state); /* proto */
static PyObject *__pyx_pf_4grpc_7_cython_6cygrpc_14IntegratedCall_2operate(struct __pyx_obj_4grpc_7_cython_6cygrpc_IntegratedCall *__pyx_v_self, PyObject *__pyx_v_operations, PyObject *__pyx_v_tag); /* proto */
static PyObject *__pyx_pf_4grpc_7_cython_6cygrpc_14IntegratedCall_4cancel(struct __pyx_obj_4grpc_7_cython_6cygrpc_IntegratedCall *__pyx_v_self, PyObject *__pyx_v_code, PyObject *__pyx_v_details); /* proto */
static PyObject *__pyx_pf_4grpc_7_cython_6cygrpc_14IntegratedCall_6__reduce_cython__(CYTHON_UNUSED struct __pyx_obj_4grpc_7_cython_6cygrpc_IntegratedCall *__pyx_v_self); /* proto */
static PyObject *__pyx_pf_4grpc_7_cython_6cygrpc_14IntegratedCall_8__setstate_cython__(CYTHON_UNUSED struct __pyx_obj_4grpc_7_cython_6cygrpc_IntegratedCall *__pyx_v_self, CYTHON_UNUSED PyObject *__pyx_v___pyx_state); /* proto */
static PyObject *__pyx_pf_4grpc_7_cython_6cygrpc_16_integrated_call_on_success(PyObject *__pyx_self, PyObject *__pyx_v_started_tags); /* proto */
static int __pyx_pf_4grpc_7_cython_6cygrpc_14SegregatedCall___cinit__(struct __pyx_obj_4grpc_7_cython_6cygrpc_SegregatedCall *__pyx_v_self, struct __pyx_obj_4grpc_7_cython_6cygrpc__ChannelState *__pyx_v_channel_state, struct __pyx_obj_4grpc_7_cython_6cygrpc__CallState *__pyx_v_call_state); /* proto */
static PyObject *__pyx_pf_4grpc_7_cython_6cygrpc_14SegregatedCall_2operate(struct __pyx_obj_4grpc_7_cython_6cygrpc_SegregatedCall *__pyx_v_self, PyObject *__pyx_v_operations, PyObject *__pyx_v_tag); /* proto */
static PyObject *__pyx_pf_4grpc_7_cython_6cygrpc_14SegregatedCall_4cancel(struct __pyx_obj_4grpc_7_cython_6cygrpc_SegregatedCall *__pyx_v_self, PyObject *__pyx_v_code, PyObject *__pyx_v_details); /* proto */
static PyObject *__pyx_pf_4grpc_7_cython_6cygrpc_14SegregatedCall_10next_event_on_success(PyObject *__pyx_self, PyObject *__pyx_v_tag); /* proto */
static PyObject *__pyx_pf_4grpc_7_cython_6cygrpc_14SegregatedCall_6next_event(struct __pyx_obj_4grpc_7_cython_6cygrpc_SegregatedCall *__pyx_v_self); /* proto */
static PyObject *__pyx_pf_4grpc_7_cython_6cygrpc_14SegregatedCall_8__reduce_cython__(CYTHON_UNUSED struct __pyx_obj_4grpc_7_cython_6cygrpc_SegregatedCall *__pyx_v_self); /* proto */
static PyObject *__pyx_pf_4grpc_7_cython_6cygrpc_14SegregatedCall_10__setstate_cython__(CYTHON_UNUSED struct __pyx_obj_4grpc_7_cython_6cygrpc_SegregatedCall *__pyx_v_self, CYTHON_UNUSED PyObject *__pyx_v___pyx_state); /* proto */
static PyObject *__pyx_pf_4grpc_7_cython_6cygrpc_16_segregated_call_on_success(PyObject *__pyx_self, CYTHON_UNUSED PyObject *__pyx_v_started_tags); /* proto */
static int __pyx_pf_4grpc_7_cython_6cygrpc_7Channel___cinit__(struct __pyx_obj_4grpc_7_cython_6cygrpc_Channel *__pyx_v_self, PyObject *__pyx_v_target, PyObject *__pyx_v_arguments, struct __pyx_obj_4grpc_7_cython_6cygrpc_ChannelCredentials *__pyx_v_channel_credentials); /* proto */
static PyObject *__pyx_pf_4grpc_7_cython_6cygrpc_7Channel_2target(struct __pyx_obj_4grpc_7_cython_6cygrpc_Channel *__pyx_v_self); /* proto */
static PyObject *__pyx_pf_4grpc_7_cython_6cygrpc_7Channel_4integrated_call(struct __pyx_obj_4grpc_7_cython_6cygrpc_Channel *__pyx_v_self, int __pyx_v_flags, PyObject *__pyx_v_method, PyObject *__pyx_v_host, PyObject *__pyx_v_deadline, PyObject *__pyx_v_metadata, struct __pyx_obj_4grpc_7_cython_6cygrpc_CallCredentials *__pyx_v_credentials, PyObject *__pyx_v_operationses_and_tags, PyObject *__pyx_v_context); /* proto */
static PyObject *__pyx_pf_4grpc_7_cython_6cygrpc_7Channel_15next_call_event_on_success(PyObject *__pyx_self, PyObject *__pyx_v_tag); /* proto */
static PyObject *__pyx_pf_4grpc_7_cython_6cygrpc_7Channel_6next_call_event(struct __pyx_obj_4grpc_7_cython_6cygrpc_Channel *__pyx_v_self); /* proto */
static PyObject *__pyx_pf_4grpc_7_cython_6cygrpc_7Channel_8segregated_call(struct __pyx_obj_4grpc_7_cython_6cygrpc_Channel *__pyx_v_self, int __pyx_v_flags, PyObject *__pyx_v_method, PyObject *__pyx_v_host, PyObject *__pyx_v_deadline, PyObject *__pyx_v_metadata, struct __pyx_obj_4grpc_7_cython_6cygrpc_CallCredentials *__pyx_v_credentials, PyObject *__pyx_v_operationses_and_tags, PyObject *__pyx_v_context); /* proto */
static PyObject *__pyx_pf_4grpc_7_cython_6cygrpc_7Channel_10check_connectivity_state(struct __pyx_obj_4grpc_7_cython_6cygrpc_Channel *__pyx_v_self, int __pyx_v_try_to_connect); /* proto */
static PyObject *__pyx_pf_4grpc_7_cython_6cygrpc_7Channel_12watch_connectivity_state(struct __pyx_obj_4grpc_7_cython_6cygrpc_Channel *__pyx_v_self, grpc_connectivity_state __pyx_v_last_observed_state, PyObject *__pyx_v_deadline); /* proto */
static PyObject *__pyx_pf_4grpc_7_cython_6cygrpc_7Channel_14close(struct __pyx_obj_4grpc_7_cython_6cygrpc_Channel *__pyx_v_self, PyObject *__pyx_v_code, PyObject *__pyx_v_details); /* proto */
static PyObject *__pyx_pf_4grpc_7_cython_6cygrpc_7Channel_16close_on_fork(struct __pyx_obj_4grpc_7_cython_6cygrpc_Channel *__pyx_v_self, PyObject *__pyx_v_code, PyObject *__pyx_v_details); /* proto */
static PyObject *__pyx_pf_4grpc_7_cython_6cygrpc_7Channel_18__reduce_cython__(CYTHON_UNUSED struct __pyx_obj_4grpc_7_cython_6cygrpc_Channel *__pyx_v_self); /* proto */
static PyObject *__pyx_pf_4grpc_7_cython_6cygrpc_7Channel_20__setstate_cython__(CYTHON_UNUSED struct __pyx_obj_4grpc_7_cython_6cygrpc_Channel *__pyx_v_self, CYTHON_UNUSED PyObject *__pyx_v___pyx_state); /* proto */
static PyObject *__pyx_pf_4grpc_7_cython_6cygrpc_channelz_get_top_channels(CYTHON_UNUSED PyObject *__pyx_self, PyObject *__pyx_v_start_channel_id); /* proto */
static PyObject *__pyx_pf_4grpc_7_cython_6cygrpc_2channelz_get_servers(CYTHON_UNUSED PyObject *__pyx_self, PyObject *__pyx_v_start_server_id); /* proto */
static PyObject *__pyx_pf_4grpc_7_cython_6cygrpc_4channelz_get_server(CYTHON_UNUSED PyObject *__pyx_self, PyObject *__pyx_v_server_id); /* proto */
static PyObject *__pyx_pf_4grpc_7_cython_6cygrpc_6channelz_get_server_sockets(CYTHON_UNUSED PyObject *__pyx_self, PyObject *__pyx_v_server_id, PyObject *__pyx_v_start_socket_id, PyObject *__pyx_v_max_results); /* proto */
static PyObject *__pyx_pf_4grpc_7_cython_6cygrpc_8channelz_get_channel(CYTHON_UNUSED PyObject *__pyx_self, PyObject *__pyx_v_channel_id); /* proto */
static PyObject *__pyx_pf_4grpc_7_cython_6cygrpc_10channelz_get_subchannel(CYTHON_UNUSED PyObject *__pyx_self, PyObject *__pyx_v_subchannel_id); /* proto */
static PyObject *__pyx_pf_4grpc_7_cython_6cygrpc_12channelz_get_socket(CYTHON_UNUSED PyObject *__pyx_self, PyObject *__pyx_v_socket_id); /* proto */
static PyObject *__pyx_pf_4grpc_7_cython_6cygrpc_14_spawn_callback_in_thread(CYTHON_UNUSED PyObject *__pyx_self, PyObject *__pyx_v_cb_func, PyObject *__pyx_v_args); /* proto */
static PyObject *__pyx_pf_4grpc_7_cython_6cygrpc_16set_async_callback_func(CYTHON_UNUSED PyObject *__pyx_self, PyObject *__pyx_v_callback_func); /* proto */
static PyObject *__pyx_pf_4grpc_7_cython_6cygrpc_18_spawn_callback_async(CYTHON_UNUSED PyObject *__pyx_self, PyObject *__pyx_v_callback, PyObject *__pyx_v_args); /* proto */
static PyObject *__pyx_pf_4grpc_7_cython_6cygrpc_15CallCredentials___reduce_cython__(CYTHON_UNUSED struct __pyx_obj_4grpc_7_cython_6cygrpc_CallCredentials *__pyx_v_self); /* proto */
static PyObject *__pyx_pf_4grpc_7_cython_6cygrpc_15CallCredentials_2__setstate_cython__(CYTHON_UNUSED struct __pyx_obj_4grpc_7_cython_6cygrpc_CallCredentials *__pyx_v_self, CYTHON_UNUSED PyObject *__pyx_v___pyx_state); /* proto */
static PyObject *__pyx_pf_4grpc_7_cython_6cygrpc_13_get_metadata_callback(PyObject *__pyx_self, PyObject *__pyx_v_metadata, grpc_status_code __pyx_v_status, PyObject *__pyx_v_error_details); /* proto */
static int __pyx_pf_4grpc_7_cython_6cygrpc_29MetadataPluginCallCredentials___cinit__(struct __pyx_obj_4grpc_7_cython_6cygrpc_MetadataPluginCallCredentials *__pyx_v_self, PyObject *__pyx_v_metadata_plugin, PyObject *__pyx_v_name); /* proto */
static PyObject *__pyx_pf_4grpc_7_cython_6cygrpc_29MetadataPluginCallCredentials_16_metadata_plugin___get__(struct __pyx_obj_4grpc_7_cython_6cygrpc_MetadataPluginCallCredentials *__pyx_v_self); /* proto */
static PyObject *__pyx_pf_4grpc_7_cython_6cygrpc_29MetadataPluginCallCredentials_5_name___get__(struct __pyx_obj_4grpc_7_cython_6cygrpc_MetadataPluginCallCredentials *__pyx_v_self); /* proto */
static PyObject *__pyx_pf_4grpc_7_cython_6cygrpc_29MetadataPluginCallCredentials_2__reduce_cython__(CYTHON_UNUSED struct __pyx_obj_4grpc_7_cython_6cygrpc_MetadataPluginCallCredentials *__pyx_v_self); /* proto */
static PyObject *__pyx_pf_4grpc_7_cython_6cygrpc_29MetadataPluginCallCredentials_4__setstate_cython__(CYTHON_UNUSED struct __pyx_obj_4grpc_7_cython_6cygrpc_MetadataPluginCallCredentials *__pyx_v_self, CYTHON_UNUSED PyObject *__pyx_v___pyx_state); /* proto */
static int __pyx_pf_4grpc_7_cython_6cygrpc_24CompositeCallCredentials___cinit__(struct __pyx_obj_4grpc_7_cython_6cygrpc_CompositeCallCredentials *__pyx_v_self, PyObject *__pyx_v_call_credentialses); /* proto */
static PyObject *__pyx_pf_4grpc_7_cython_6cygrpc_24CompositeCallCredentials_19_call_credentialses___get__(struct __pyx_obj_4grpc_7_cython_6cygrpc_CompositeCallCredentials *__pyx_v_self); /* proto */
static PyObject *__pyx_pf_4grpc_7_cython_6cygrpc_24CompositeCallCredentials_2__reduce_cython__(CYTHON_UNUSED struct __pyx_obj_4grpc_7_cython_6cygrpc_CompositeCallCredentials *__pyx_v_self); /* proto */
static PyObject *__pyx_pf_4grpc_7_cython_6cygrpc_24CompositeCallCredentials_4__setstate_cython__(CYTHON_UNUSED struct __pyx_obj_4grpc_7_cython_6cygrpc_CompositeCallCredentials *__pyx_v_self, CYTHON_UNUSED PyObject *__pyx_v___pyx_state); /* proto */
static PyObject *__pyx_pf_4grpc_7_cython_6cygrpc_18ChannelCredentials___reduce_cython__(struct __pyx_obj_4grpc_7_cython_6cygrpc_ChannelCredentials *__pyx_v_self); /* proto */
static PyObject *__pyx_pf_4grpc_7_cython_6cygrpc_18ChannelCredentials_2__setstate_cython__(struct __pyx_obj_4grpc_7_cython_6cygrpc_ChannelCredentials *__pyx_v_self, PyObject *__pyx_v___pyx_state); /* proto */
static int __pyx_pf_4grpc_7_cython_6cygrpc_18SSLSessionCacheLRU___cinit__(struct __pyx_obj_4grpc_7_cython_6cygrpc_SSLSessionCacheLRU *__pyx_v_self, PyObject *__pyx_v_capacity); /* proto */
static PyObject *__pyx_pf_4grpc_7_cython_6cygrpc_18SSLSessionCacheLRU_2__int__(struct __pyx_obj_4grpc_7_cython_6cygrpc_SSLSessionCacheLRU *__pyx_v_self); /* proto */
static void __pyx_pf_4grpc_7_cython_6cygrpc_18SSLSessionCacheLRU_4__dealloc__(struct __pyx_obj_4grpc_7_cython_6cygrpc_SSLSessionCacheLRU *__pyx_v_self); /* proto */
static PyObject *__pyx_pf_4grpc_7_cython_6cygrpc_18SSLSessionCacheLRU_6__reduce_cython__(CYTHON_UNUSED struct __pyx_obj_4grpc_7_cython_6cygrpc_SSLSessionCacheLRU *__pyx_v_self); /* proto */
static PyObject *__pyx_pf_4grpc_7_cython_6cygrpc_18SSLSessionCacheLRU_8__setstate_cython__(CYTHON_UNUSED struct __pyx_obj_4grpc_7_cython_6cygrpc_SSLSessionCacheLRU *__pyx_v_self, CYTHON_UNUSED PyObject *__pyx_v___pyx_state); /* proto */
static int __pyx_pf_4grpc_7_cython_6cygrpc_21SSLChannelCredentials___cinit__(struct __pyx_obj_4grpc_7_cython_6cygrpc_SSLChannelCredentials *__pyx_v_self, PyObject *__pyx_v_pem_root_certificates, PyObject *__pyx_v_private_key, PyObject *__pyx_v_certificate_chain); /* proto */
static PyObject *__pyx_pf_4grpc_7_cython_6cygrpc_21SSLChannelCredentials_22_pem_root_certificates___get__(struct __pyx_obj_4grpc_7_cython_6cygrpc_SSLChannelCredentials *__pyx_v_self); /* proto */
static PyObject *__pyx_pf_4grpc_7_cython_6cygrpc_21SSLChannelCredentials_12_private_key___get__(struct __pyx_obj_4grpc_7_cython_6cygrpc_SSLChannelCredentials *__pyx_v_self); /* proto */
static PyObject *__pyx_pf_4grpc_7_cython_6cygrpc_21SSLChannelCredentials_18_certificate_chain___get__(struct __pyx_obj_4grpc_7_cython_6cygrpc_SSLChannelCredentials *__pyx_v_self); /* proto */
static PyObject *__pyx_pf_4grpc_7_cython_6cygrpc_21SSLChannelCredentials_2__reduce_cython__(CYTHON_UNUSED struct __pyx_obj_4grpc_7_cython_6cygrpc_SSLChannelCredentials *__pyx_v_self); /* proto */
static PyObject *__pyx_pf_4grpc_7_cython_6cygrpc_21SSLChannelCredentials_4__setstate_cython__(CYTHON_UNUSED struct __pyx_obj_4grpc_7_cython_6cygrpc_SSLChannelCredentials *__pyx_v_self, CYTHON_UNUSED PyObject *__pyx_v___pyx_state); /* proto */
static int __pyx_pf_4grpc_7_cython_6cygrpc_27CompositeChannelCredentials___cinit__(struct __pyx_obj_4grpc_7_cython_6cygrpc_CompositeChannelCredentials *__pyx_v_self, PyObject *__pyx_v_call_credentialses, PyObject *__pyx_v_channel_credentials); /* proto */
static PyObject *__pyx_pf_4grpc_7_cython_6cygrpc_27CompositeChannelCredentials_19_call_credentialses___get__(struct __pyx_obj_4grpc_7_cython_6cygrpc_CompositeChannelCredentials *__pyx_v_self); /* proto */
static PyObject *__pyx_pf_4grpc_7_cython_6cygrpc_27CompositeChannelCredentials_20_channel_credentials___get__(struct __pyx_obj_4grpc_7_cython_6cygrpc_CompositeChannelCredentials *__pyx_v_self); /* proto */
static PyObject *__pyx_pf_4grpc_7_cython_6cygrpc_27CompositeChannelCredentials_2__reduce_cython__(CYTHON_UNUSED struct __pyx_obj_4grpc_7_cython_6cygrpc_CompositeChannelCredentials *__pyx_v_self); /* proto */
static PyObject *__pyx_pf_4grpc_7_cython_6cygrpc_27CompositeChannelCredentials_4__setstate_cython__(CYTHON_UNUSED struct __pyx_obj_4grpc_7_cython_6cygrpc_CompositeChannelCredentials *__pyx_v_self, CYTHON_UNUSED PyObject *__pyx_v___pyx_state); /* proto */
static int __pyx_pf_4grpc_7_cython_6cygrpc_23ServerCertificateConfig___cinit__(struct __pyx_obj_4grpc_7_cython_6cygrpc_ServerCertificateConfig *__pyx_v_self); /* proto */
static void __pyx_pf_4grpc_7_cython_6cygrpc_23ServerCertificateConfig_2__dealloc__(struct __pyx_obj_4grpc_7_cython_6cygrpc_ServerCertificateConfig *__pyx_v_self); /* proto */
static PyObject *__pyx_pf_4grpc_7_cython_6cygrpc_23ServerCertificateConfig_4__reduce_cython__(CYTHON_UNUSED struct __pyx_obj_4grpc_7_cython_6cygrpc_ServerCertificateConfig *__pyx_v_self); /* proto */
static PyObject *__pyx_pf_4grpc_7_cython_6cygrpc_23ServerCertificateConfig_6__setstate_cython__(CYTHON_UNUSED struct __pyx_obj_4grpc_7_cython_6cygrpc_ServerCertificateConfig *__pyx_v_self, CYTHON_UNUSED PyObject *__pyx_v___pyx_state); /* proto */
static int __pyx_pf_4grpc_7_cython_6cygrpc_17ServerCredentials___cinit__(struct __pyx_obj_4grpc_7_cython_6cygrpc_ServerCredentials *__pyx_v_self); /* proto */
static void __pyx_pf_4grpc_7_cython_6cygrpc_17ServerCredentials_2__dealloc__(struct __pyx_obj_4grpc_7_cython_6cygrpc_ServerCredentials *__pyx_v_self); /* proto */
static PyObject *__pyx_pf_4grpc_7_cython_6cygrpc_17ServerCredentials_4__reduce_cython__(CYTHON_UNUSED struct __pyx_obj_4grpc_7_cython_6cygrpc_ServerCredentials *__pyx_v_self); /* proto */
static PyObject *__pyx_pf_4grpc_7_cython_6cygrpc_17ServerCredentials_6__setstate_cython__(CYTHON_UNUSED struct __pyx_obj_4grpc_7_cython_6cygrpc_ServerCredentials *__pyx_v_self, CYTHON_UNUSED PyObject *__pyx_v___pyx_state); /* proto */
static PyObject *__pyx_pf_4grpc_7_cython_6cygrpc_20server_credentials_ssl(CYTHON_UNUSED PyObject *__pyx_self, PyObject *__pyx_v_pem_root_certs, PyObject *__pyx_v_pem_key_cert_pairs, int __pyx_v_force_client_auth); /* proto */
static PyObject *__pyx_pf_4grpc_7_cython_6cygrpc_22server_certificate_config_ssl(CYTHON_UNUSED PyObject *__pyx_self, PyObject *__pyx_v_pem_root_certs, PyObject *__pyx_v_pem_key_cert_pairs); /* proto */
static PyObject *__pyx_pf_4grpc_7_cython_6cygrpc_24server_credentials_ssl_dynamic_cert_config(CYTHON_UNUSED PyObject *__pyx_self, PyObject *__pyx_v_initial_cert_config, PyObject *__pyx_v_cert_config_fetcher, int __pyx_v_force_client_auth); /* proto */
static int __pyx_pf_4grpc_7_cython_6cygrpc_15CompletionQueue___cinit__(struct __pyx_obj_4grpc_7_cython_6cygrpc_CompletionQueue *__pyx_v_self, PyObject *__pyx_v_shutdown_cq); /* proto */
static PyObject *__pyx_pf_4grpc_7_cython_6cygrpc_15CompletionQueue_2poll(struct __pyx_obj_4grpc_7_cython_6cygrpc_CompletionQueue *__pyx_v_self, PyObject *__pyx_v_deadline); /* proto */
static PyObject *__pyx_pf_4grpc_7_cython_6cygrpc_15CompletionQueue_4shutdown(struct __pyx_obj_4grpc_7_cython_6cygrpc_CompletionQueue *__pyx_v_self); /* proto */
static PyObject *__pyx_pf_4grpc_7_cython_6cygrpc_15CompletionQueue_6clear(struct __pyx_obj_4grpc_7_cython_6cygrpc_CompletionQueue *__pyx_v_self); /* proto */
static void __pyx_pf_4grpc_7_cython_6cygrpc_15CompletionQueue_8__dealloc__(struct __pyx_obj_4grpc_7_cython_6cygrpc_CompletionQueue *__pyx_v_self); /* proto */
static PyObject *__pyx_pf_4grpc_7_cython_6cygrpc_15CompletionQueue_10__reduce_cython__(CYTHON_UNUSED struct __pyx_obj_4grpc_7_cython_6cygrpc_CompletionQueue *__pyx_v_self); /* proto */
static PyObject *__pyx_pf_4grpc_7_cython_6cygrpc_15CompletionQueue_12__setstate_cython__(CYTHON_UNUSED struct __pyx_obj_4grpc_7_cython_6cygrpc_CompletionQueue *__pyx_v_self, CYTHON_UNUSED PyObject *__pyx_v___pyx_state); /* proto */
static int __pyx_pf_4grpc_7_cython_6cygrpc_17ConnectivityEvent___cinit__(struct __pyx_obj_4grpc_7_cython_6cygrpc_ConnectivityEvent *__pyx_v_self, grpc_completion_type __pyx_v_completion_type, int __pyx_v_success, PyObject *__pyx_v_tag); /* proto */
static PyObject *__pyx_pf_4grpc_7_cython_6cygrpc_17ConnectivityEvent_15completion_type___get__(struct __pyx_obj_4grpc_7_cython_6cygrpc_ConnectivityEvent *__pyx_v_self); /* proto */
static PyObject *__pyx_pf_4grpc_7_cython_6cygrpc_17ConnectivityEvent_7success___get__(struct __pyx_obj_4grpc_7_cython_6cygrpc_ConnectivityEvent *__pyx_v_self); /* proto */
static PyObject *__pyx_pf_4grpc_7_cython_6cygrpc_17ConnectivityEvent_3tag___get__(struct __pyx_obj_4grpc_7_cython_6cygrpc_ConnectivityEvent *__pyx_v_self); /* proto */
static PyObject *__pyx_pf_4grpc_7_cython_6cygrpc_17ConnectivityEvent_2__reduce_cython__(CYTHON_UNUSED struct __pyx_obj_4grpc_7_cython_6cygrpc_ConnectivityEvent *__pyx_v_self); /* proto */
static PyObject *__pyx_pf_4grpc_7_cython_6cygrpc_17ConnectivityEvent_4__setstate_cython__(CYTHON_UNUSED struct __pyx_obj_4grpc_7_cython_6cygrpc_ConnectivityEvent *__pyx_v_self, CYTHON_UNUSED PyObject *__pyx_v___pyx_state); /* proto */
static int __pyx_pf_4grpc_7_cython_6cygrpc_16RequestCallEvent___cinit__(struct __pyx_obj_4grpc_7_cython_6cygrpc_RequestCallEvent *__pyx_v_self, grpc_completion_type __pyx_v_completion_type, int __pyx_v_success, PyObject *__pyx_v_tag, struct __pyx_obj_4grpc_7_cython_6cygrpc_Call *__pyx_v_call, struct __pyx_obj_4grpc_7_cython_6cygrpc_CallDetails *__pyx_v_call_details, PyObject *__pyx_v_invocation_metadata); /* proto */
static PyObject *__pyx_pf_4grpc_7_cython_6cygrpc_16RequestCallEvent_15completion_type___get__(struct __pyx_obj_4grpc_7_cython_6cygrpc_RequestCallEvent *__pyx_v_self); /* proto */
static PyObject *__pyx_pf_4grpc_7_cython_6cygrpc_16RequestCallEvent_7success___get__(struct __pyx_obj_4grpc_7_cython_6cygrpc_RequestCallEvent *__pyx_v_self); /* proto */
static PyObject *__pyx_pf_4grpc_7_cython_6cygrpc_16RequestCallEvent_3tag___get__(struct __pyx_obj_4grpc_7_cython_6cygrpc_RequestCallEvent *__pyx_v_self); /* proto */
static PyObject *__pyx_pf_4grpc_7_cython_6cygrpc_16RequestCallEvent_4call___get__(struct __pyx_obj_4grpc_7_cython_6cygrpc_RequestCallEvent *__pyx_v_self); /* proto */
static PyObject *__pyx_pf_4grpc_7_cython_6cygrpc_16RequestCallEvent_12call_details___get__(struct __pyx_obj_4grpc_7_cython_6cygrpc_RequestCallEvent *__pyx_v_self); /* proto */
static PyObject *__pyx_pf_4grpc_7_cython_6cygrpc_16RequestCallEvent_19invocation_metadata___get__(struct __pyx_obj_4grpc_7_cython_6cygrpc_RequestCallEvent *__pyx_v_self); /* proto */
static PyObject *__pyx_pf_4grpc_7_cython_6cygrpc_16RequestCallEvent_2__reduce_cython__(CYTHON_UNUSED struct __pyx_obj_4grpc_7_cython_6cygrpc_RequestCallEvent *__pyx_v_self); /* proto */
static PyObject *__pyx_pf_4grpc_7_cython_6cygrpc_16RequestCallEvent_4__setstate_cython__(CYTHON_UNUSED struct __pyx_obj_4grpc_7_cython_6cygrpc_RequestCallEvent *__pyx_v_self, CYTHON_UNUSED PyObject *__pyx_v___pyx_state); /* proto */
static int __pyx_pf_4grpc_7_cython_6cygrpc_19BatchOperationEvent___cinit__(struct __pyx_obj_4grpc_7_cython_6cygrpc_BatchOperationEvent *__pyx_v_self, grpc_completion_type __pyx_v_completion_type, int __pyx_v_success, PyObject *__pyx_v_tag, PyObject *__pyx_v_batch_operations); /* proto */
static PyObject *__pyx_pf_4grpc_7_cython_6cygrpc_19BatchOperationEvent_15completion_type___get__(struct __pyx_obj_4grpc_7_cython_6cygrpc_BatchOperationEvent *__pyx_v_self); /* proto */
static PyObject *__pyx_pf_4grpc_7_cython_6cygrpc_19BatchOperationEvent_7success___get__(struct __pyx_obj_4grpc_7_cython_6cygrpc_BatchOperationEvent *__pyx_v_self); /* proto */
static PyObject *__pyx_pf_4grpc_7_cython_6cygrpc_19BatchOperationEvent_3tag___get__(struct __pyx_obj_4grpc_7_cython_6cygrpc_BatchOperationEvent *__pyx_v_self); /* proto */
static PyObject *__pyx_pf_4grpc_7_cython_6cygrpc_19BatchOperationEvent_16batch_operations___get__(struct __pyx_obj_4grpc_7_cython_6cygrpc_BatchOperationEvent *__pyx_v_self); /* proto */
static PyObject *__pyx_pf_4grpc_7_cython_6cygrpc_19BatchOperationEvent_2__reduce_cython__(CYTHON_UNUSED struct __pyx_obj_4grpc_7_cython_6cygrpc_BatchOperationEvent *__pyx_v_self); /* proto */
static PyObject *__pyx_pf_4grpc_7_cython_6cygrpc_19BatchOperationEvent_4__setstate_cython__(CYTHON_UNUSED struct __pyx_obj_4grpc_7_cython_6cygrpc_BatchOperationEvent *__pyx_v_self, CYTHON_UNUSED PyObject *__pyx_v___pyx_state); /* proto */
static int __pyx_pf_4grpc_7_cython_6cygrpc_19ServerShutdownEvent___cinit__(struct __pyx_obj_4grpc_7_cython_6cygrpc_ServerShutdownEvent *__pyx_v_self, grpc_completion_type __pyx_v_completion_type, int __pyx_v_success, PyObject *__pyx_v_tag); /* proto */
static PyObject *__pyx_pf_4grpc_7_cython_6cygrpc_19ServerShutdownEvent_15completion_type___get__(struct __pyx_obj_4grpc_7_cython_6cygrpc_ServerShutdownEvent *__pyx_v_self); /* proto */
static PyObject *__pyx_pf_4grpc_7_cython_6cygrpc_19ServerShutdownEvent_7success___get__(struct __pyx_obj_4grpc_7_cython_6cygrpc_ServerShutdownEvent *__pyx_v_self); /* proto */
static PyObject *__pyx_pf_4grpc_7_cython_6cygrpc_19ServerShutdownEvent_3tag___get__(struct __pyx_obj_4grpc_7_cython_6cygrpc_ServerShutdownEvent *__pyx_v_self); /* proto */
static PyObject *__pyx_pf_4grpc_7_cython_6cygrpc_19ServerShutdownEvent_2__reduce_cython__(CYTHON_UNUSED struct __pyx_obj_4grpc_7_cython_6cygrpc_ServerShutdownEvent *__pyx_v_self); /* proto */
static PyObject *__pyx_pf_4grpc_7_cython_6cygrpc_19ServerShutdownEvent_4__setstate_cython__(CYTHON_UNUSED struct __pyx_obj_4grpc_7_cython_6cygrpc_ServerShutdownEvent *__pyx_v_self, CYTHON_UNUSED PyObject *__pyx_v___pyx_state); /* proto */
static PyObject *__pyx_pf_4grpc_7_cython_6cygrpc_9_metadata_genexpr(PyObject *__pyx_self); /* proto */
static PyObject *__pyx_pf_4grpc_7_cython_6cygrpc_9Operation___reduce_cython__(CYTHON_UNUSED struct __pyx_obj_4grpc_7_cython_6cygrpc_Operation *__pyx_v_self); /* proto */
static PyObject *__pyx_pf_4grpc_7_cython_6cygrpc_9Operation_2__setstate_cython__(CYTHON_UNUSED struct __pyx_obj_4grpc_7_cython_6cygrpc_Operation *__pyx_v_self, CYTHON_UNUSED PyObject *__pyx_v___pyx_state); /* proto */
static int __pyx_pf_4grpc_7_cython_6cygrpc_28SendInitialMetadataOperation___cinit__(struct __pyx_obj_4grpc_7_cython_6cygrpc_SendInitialMetadataOperation *__pyx_v_self, PyObject *__pyx_v_initial_metadata, PyObject *__pyx_v_flags); /* proto */
static PyObject *__pyx_pf_4grpc_7_cython_6cygrpc_28SendInitialMetadataOperation_2type(CYTHON_UNUSED struct __pyx_obj_4grpc_7_cython_6cygrpc_SendInitialMetadataOperation *__pyx_v_self); /* proto */
static PyObject *__pyx_pf_4grpc_7_cython_6cygrpc_28SendInitialMetadataOperation_17_initial_metadata___get__(struct __pyx_obj_4grpc_7_cython_6cygrpc_SendInitialMetadataOperation *__pyx_v_self); /* proto */
static PyObject *__pyx_pf_4grpc_7_cython_6cygrpc_28SendInitialMetadataOperation_6_flags___get__(struct __pyx_obj_4grpc_7_cython_6cygrpc_SendInitialMetadataOperation *__pyx_v_self); /* proto */
static PyObject *__pyx_pf_4grpc_7_cython_6cygrpc_28SendInitialMetadataOperation_4__reduce_cython__(CYTHON_UNUSED struct __pyx_obj_4grpc_7_cython_6cygrpc_SendInitialMetadataOperation *__pyx_v_self); /* proto */
static PyObject *__pyx_pf_4grpc_7_cython_6cygrpc_28SendInitialMetadataOperation_6__setstate_cython__(CYTHON_UNUSED struct __pyx_obj_4grpc_7_cython_6cygrpc_SendInitialMetadataOperation *__pyx_v_self, CYTHON_UNUSED PyObject *__pyx_v___pyx_state); /* proto */
static int __pyx_pf_4grpc_7_cython_6cygrpc_20SendMessageOperation___cinit__(struct __pyx_obj_4grpc_7_cython_6cygrpc_SendMessageOperation *__pyx_v_self, PyObject *__pyx_v_message, int __pyx_v_flags); /* proto */
static PyObject *__pyx_pf_4grpc_7_cython_6cygrpc_20SendMessageOperation_2type(CYTHON_UNUSED struct __pyx_obj_4grpc_7_cython_6cygrpc_SendMessageOperation *__pyx_v_self); /* proto */
static PyObject *__pyx_pf_4grpc_7_cython_6cygrpc_20SendMessageOperation_8_message___get__(struct __pyx_obj_4grpc_7_cython_6cygrpc_SendMessageOperation *__pyx_v_self); /* proto */
static PyObject *__pyx_pf_4grpc_7_cython_6cygrpc_20SendMessageOperation_6_flags___get__(struct __pyx_obj_4grpc_7_cython_6cygrpc_SendMessageOperation *__pyx_v_self); /* proto */
static PyObject *__pyx_pf_4grpc_7_cython_6cygrpc_20SendMessageOperation_4__reduce_cython__(CYTHON_UNUSED struct __pyx_obj_4grpc_7_cython_6cygrpc_SendMessageOperation *__pyx_v_self); /* proto */
static PyObject *__pyx_pf_4grpc_7_cython_6cygrpc_20SendMessageOperation_6__setstate_cython__(CYTHON_UNUSED struct __pyx_obj_4grpc_7_cython_6cygrpc_SendMessageOperation *__pyx_v_self, CYTHON_UNUSED PyObject *__pyx_v___pyx_state); /* proto */
static int __pyx_pf_4grpc_7_cython_6cygrpc_28SendCloseFromClientOperation___cinit__(struct __pyx_obj_4grpc_7_cython_6cygrpc_SendCloseFromClientOperation *__pyx_v_self, int __pyx_v_flags); /* proto */
static PyObject *__pyx_pf_4grpc_7_cython_6cygrpc_28SendCloseFromClientOperation_2type(CYTHON_UNUSED struct __pyx_obj_4grpc_7_cython_6cygrpc_SendCloseFromClientOperation *__pyx_v_self); /* proto */
static PyObject *__pyx_pf_4grpc_7_cython_6cygrpc_28SendCloseFromClientOperation_6_flags___get__(struct __pyx_obj_4grpc_7_cython_6cygrpc_SendCloseFromClientOperation *__pyx_v_self); /* proto */
static PyObject *__pyx_pf_4grpc_7_cython_6cygrpc_28SendCloseFromClientOperation_4__reduce_cython__(CYTHON_UNUSED struct __pyx_obj_4grpc_7_cython_6cygrpc_SendCloseFromClientOperation *__pyx_v_self); /* proto */
static PyObject *__pyx_pf_4grpc_7_cython_6cygrpc_28SendCloseFromClientOperation_6__setstate_cython__(CYTHON_UNUSED struct __pyx_obj_4grpc_7_cython_6cygrpc_SendCloseFromClientOperation *__pyx_v_self, CYTHON_UNUSED PyObject *__pyx_v___pyx_state); /* proto */
static int __pyx_pf_4grpc_7_cython_6cygrpc_29SendStatusFromServerOperation___cinit__(struct __pyx_obj_4grpc_7_cython_6cygrpc_SendStatusFromServerOperation *__pyx_v_self, PyObject *__pyx_v_trailing_metadata, PyObject *__pyx_v_code, PyObject *__pyx_v_details, int __pyx_v_flags); /* proto */
static PyObject *__pyx_pf_4grpc_7_cython_6cygrpc_29SendStatusFromServerOperation_2type(CYTHON_UNUSED struct __pyx_obj_4grpc_7_cython_6cygrpc_SendStatusFromServerOperation *__pyx_v_self); /* proto */
static PyObject *__pyx_pf_4grpc_7_cython_6cygrpc_29SendStatusFromServerOperation_18_trailing_metadata___get__(struct __pyx_obj_4grpc_7_cython_6cygrpc_SendStatusFromServerOperation *__pyx_v_self); /* proto */
static PyObject *__pyx_pf_4grpc_7_cython_6cygrpc_29SendStatusFromServerOperation_5_code___get__(struct __pyx_obj_4grpc_7_cython_6cygrpc_SendStatusFromServerOperation *__pyx_v_self); /* proto */
static PyObject *__pyx_pf_4grpc_7_cython_6cygrpc_29SendStatusFromServerOperation_8_details___get__(struct __pyx_obj_4grpc_7_cython_6cygrpc_SendStatusFromServerOperation *__pyx_v_self); /* proto */
static PyObject *__pyx_pf_4grpc_7_cython_6cygrpc_29SendStatusFromServerOperation_6_flags___get__(struct __pyx_obj_4grpc_7_cython_6cygrpc_SendStatusFromServerOperation *__pyx_v_self); /* proto */
static PyObject *__pyx_pf_4grpc_7_cython_6cygrpc_29SendStatusFromServerOperation_4__reduce_cython__(CYTHON_UNUSED struct __pyx_obj_4grpc_7_cython_6cygrpc_SendStatusFromServerOperation *__pyx_v_self); /* proto */
static PyObject *__pyx_pf_4grpc_7_cython_6cygrpc_29SendStatusFromServerOperation_6__setstate_cython__(CYTHON_UNUSED struct __pyx_obj_4grpc_7_cython_6cygrpc_SendStatusFromServerOperation *__pyx_v_self, CYTHON_UNUSED PyObject *__pyx_v___pyx_state); /* proto */
static int __pyx_pf_4grpc_7_cython_6cygrpc_31ReceiveInitialMetadataOperation___cinit__(struct __pyx_obj_4grpc_7_cython_6cygrpc_ReceiveInitialMetadataOperation *__pyx_v_self, PyObject *__pyx_v_flags); /* proto */
static PyObject *__pyx_pf_4grpc_7_cython_6cygrpc_31ReceiveInitialMetadataOperation_2type(CYTHON_UNUSED struct __pyx_obj_4grpc_7_cython_6cygrpc_ReceiveInitialMetadataOperation *__pyx_v_self); /* proto */
static PyObject *__pyx_pf_4grpc_7_cython_6cygrpc_31ReceiveInitialMetadataOperation_4initial_metadata(struct __pyx_obj_4grpc_7_cython_6cygrpc_ReceiveInitialMetadataOperation *__pyx_v_self); /* proto */
static PyObject *__pyx_pf_4grpc_7_cython_6cygrpc_31ReceiveInitialMetadataOperation_6_flags___get__(struct __pyx_obj_4grpc_7_cython_6cygrpc_ReceiveInitialMetadataOperation *__pyx_v_self); /* proto */
static PyObject *__pyx_pf_4grpc_7_cython_6cygrpc_31ReceiveInitialMetadataOperation_6__reduce_cython__(CYTHON_UNUSED struct __pyx_obj_4grpc_7_cython_6cygrpc_ReceiveInitialMetadataOperation *__pyx_v_self); /* proto */
static PyObject *__pyx_pf_4grpc_7_cython_6cygrpc_31ReceiveInitialMetadataOperation_8__setstate_cython__(CYTHON_UNUSED struct __pyx_obj_4grpc_7_cython_6cygrpc_ReceiveInitialMetadataOperation *__pyx_v_self, CYTHON_UNUSED PyObject *__pyx_v___pyx_state); /* proto */
static int __pyx_pf_4grpc_7_cython_6cygrpc_23ReceiveMessageOperation___cinit__(struct __pyx_obj_4grpc_7_cython_6cygrpc_ReceiveMessageOperation *__pyx_v_self, PyObject *__pyx_v_flags); /* proto */
static PyObject *__pyx_pf_4grpc_7_cython_6cygrpc_23ReceiveMessageOperation_2type(CYTHON_UNUSED struct __pyx_obj_4grpc_7_cython_6cygrpc_ReceiveMessageOperation *__pyx_v_self); /* proto */
static PyObject *__pyx_pf_4grpc_7_cython_6cygrpc_23ReceiveMessageOperation_4message(struct __pyx_obj_4grpc_7_cython_6cygrpc_ReceiveMessageOperation *__pyx_v_self); /* proto */
static PyObject *__pyx_pf_4grpc_7_cython_6cygrpc_23ReceiveMessageOperation_6_flags___get__(struct __pyx_obj_4grpc_7_cython_6cygrpc_ReceiveMessageOperation *__pyx_v_self); /* proto */
static PyObject *__pyx_pf_4grpc_7_cython_6cygrpc_23ReceiveMessageOperation_6__reduce_cython__(CYTHON_UNUSED struct __pyx_obj_4grpc_7_cython_6cygrpc_ReceiveMessageOperation *__pyx_v_self); /* proto */
static PyObject *__pyx_pf_4grpc_7_cython_6cygrpc_23ReceiveMessageOperation_8__setstate_cython__(CYTHON_UNUSED struct __pyx_obj_4grpc_7_cython_6cygrpc_ReceiveMessageOperation *__pyx_v_self, CYTHON_UNUSED PyObject *__pyx_v___pyx_state); /* proto */
static int __pyx_pf_4grpc_7_cython_6cygrpc_30ReceiveStatusOnClientOperation___cinit__(struct __pyx_obj_4grpc_7_cython_6cygrpc_ReceiveStatusOnClientOperation *__pyx_v_self, PyObject *__pyx_v_flags); /* proto */
static PyObject *__pyx_pf_4grpc_7_cython_6cygrpc_30ReceiveStatusOnClientOperation_2type(CYTHON_UNUSED struct __pyx_obj_4grpc_7_cython_6cygrpc_ReceiveStatusOnClientOperation *__pyx_v_self); /* proto */
static PyObject *__pyx_pf_4grpc_7_cython_6cygrpc_30ReceiveStatusOnClientOperation_4trailing_metadata(struct __pyx_obj_4grpc_7_cython_6cygrpc_ReceiveStatusOnClientOperation *__pyx_v_self); /* proto */
static PyObject *__pyx_pf_4grpc_7_cython_6cygrpc_30ReceiveStatusOnClientOperation_6code(struct __pyx_obj_4grpc_7_cython_6cygrpc_ReceiveStatusOnClientOperation *__pyx_v_self); /* proto */
static PyObject *__pyx_pf_4grpc_7_cython_6cygrpc_30ReceiveStatusOnClientOperation_8details(struct __pyx_obj_4grpc_7_cython_6cygrpc_ReceiveStatusOnClientOperation *__pyx_v_self); /* proto */
static PyObject *__pyx_pf_4grpc_7_cython_6cygrpc_30ReceiveStatusOnClientOperation_10error_string(struct __pyx_obj_4grpc_7_cython_6cygrpc_ReceiveStatusOnClientOperation *__pyx_v_self); /* proto */
static PyObject *__pyx_pf_4grpc_7_cython_6cygrpc_30ReceiveStatusOnClientOperation_6_flags___get__(struct __pyx_obj_4grpc_7_cython_6cygrpc_ReceiveStatusOnClientOperation *__pyx_v_self); /* proto */
static PyObject *__pyx_pf_4grpc_7_cython_6cygrpc_30ReceiveStatusOnClientOperation_12__reduce_cython__(CYTHON_UNUSED struct __pyx_obj_4grpc_7_cython_6cygrpc_ReceiveStatusOnClientOperation *__pyx_v_self); /* proto */
static PyObject *__pyx_pf_4grpc_7_cython_6cygrpc_30ReceiveStatusOnClientOperation_14__setstate_cython__(CYTHON_UNUSED struct __pyx_obj_4grpc_7_cython_6cygrpc_ReceiveStatusOnClientOperation *__pyx_v_self, CYTHON_UNUSED PyObject *__pyx_v___pyx_state); /* proto */
static int __pyx_pf_4grpc_7_cython_6cygrpc_29ReceiveCloseOnServerOperation___cinit__(struct __pyx_obj_4grpc_7_cython_6cygrpc_ReceiveCloseOnServerOperation *__pyx_v_self, PyObject *__pyx_v_flags); /* proto */
static PyObject *__pyx_pf_4grpc_7_cython_6cygrpc_29ReceiveCloseOnServerOperation_2type(CYTHON_UNUSED struct __pyx_obj_4grpc_7_cython_6cygrpc_ReceiveCloseOnServerOperation *__pyx_v_self); /* proto */
static PyObject *__pyx_pf_4grpc_7_cython_6cygrpc_29ReceiveCloseOnServerOperation_4cancelled(struct __pyx_obj_4grpc_7_cython_6cygrpc_ReceiveCloseOnServerOperation *__pyx_v_self); /* proto */
static PyObject *__pyx_pf_4grpc_7_cython_6cygrpc_29ReceiveCloseOnServerOperation_6_flags___get__(struct __pyx_obj_4grpc_7_cython_6cygrpc_ReceiveCloseOnServerOperation *__pyx_v_self); /* proto */
static PyObject *__pyx_pf_4grpc_7_cython_6cygrpc_29ReceiveCloseOnServerOperation_6__reduce_cython__(CYTHON_UNUSED struct __pyx_obj_4grpc_7_cython_6cygrpc_ReceiveCloseOnServerOperation *__pyx_v_self); /* proto */
static PyObject *__pyx_pf_4grpc_7_cython_6cygrpc_29ReceiveCloseOnServerOperation_8__setstate_cython__(CYTHON_UNUSED struct __pyx_obj_4grpc_7_cython_6cygrpc_ReceiveCloseOnServerOperation *__pyx_v_self, CYTHON_UNUSED PyObject *__pyx_v___pyx_state); /* proto */
static int __pyx_pf_4grpc_7_cython_6cygrpc_11CallDetails___cinit__(struct __pyx_obj_4grpc_7_cython_6cygrpc_CallDetails *__pyx_v_self); /* proto */
static void __pyx_pf_4grpc_7_cython_6cygrpc_11CallDetails_2__dealloc__(struct __pyx_obj_4grpc_7_cython_6cygrpc_CallDetails *__pyx_v_self); /* proto */
static PyObject *__pyx_pf_4grpc_7_cython_6cygrpc_11CallDetails_6method___get__(struct __pyx_obj_4grpc_7_cython_6cygrpc_CallDetails *__pyx_v_self); /* proto */
static PyObject *__pyx_pf_4grpc_7_cython_6cygrpc_11CallDetails_4host___get__(struct __pyx_obj_4grpc_7_cython_6cygrpc_CallDetails *__pyx_v_self); /* proto */
static PyObject *__pyx_pf_4grpc_7_cython_6cygrpc_11CallDetails_8deadline___get__(struct __pyx_obj_4grpc_7_cython_6cygrpc_CallDetails *__pyx_v_self); /* proto */
static PyObject *__pyx_pf_4grpc_7_cython_6cygrpc_11CallDetails_4__reduce_cython__(CYTHON_UNUSED struct __pyx_obj_4grpc_7_cython_6cygrpc_CallDetails *__pyx_v_self); /* proto */
static PyObject *__pyx_pf_4grpc_7_cython_6cygrpc_11CallDetails_6__setstate_cython__(CYTHON_UNUSED struct __pyx_obj_4grpc_7_cython_6cygrpc_CallDetails *__pyx_v_self, CYTHON_UNUSED PyObject *__pyx_v___pyx_state); /* proto */
static int __pyx_pf_4grpc_7_cython_6cygrpc_17SslPemKeyCertPair___cinit__(struct __pyx_obj_4grpc_7_cython_6cygrpc_SslPemKeyCertPair *__pyx_v_self, PyObject *__pyx_v_private_key, PyObject *__pyx_v_certificate_chain); /* proto */
static PyObject *__pyx_pf_4grpc_7_cython_6cygrpc_17SslPemKeyCertPair_11private_key___get__(struct __pyx_obj_4grpc_7_cython_6cygrpc_SslPemKeyCertPair *__pyx_v_self); /* proto */
static PyObject *__pyx_pf_4grpc_7_cython_6cygrpc_17SslPemKeyCertPair_17certificate_chain___get__(struct __pyx_obj_4grpc_7_cython_6cygrpc_SslPemKeyCertPair *__pyx_v_self); /* proto */
static PyObject *__pyx_pf_4grpc_7_cython_6cygrpc_17SslPemKeyCertPair_2__reduce_cython__(CYTHON_UNUSED struct __pyx_obj_4grpc_7_cython_6cygrpc_SslPemKeyCertPair *__pyx_v_self); /* proto */
static PyObject *__pyx_pf_4grpc_7_cython_6cygrpc_17SslPemKeyCertPair_4__setstate_cython__(CYTHON_UNUSED struct __pyx_obj_4grpc_7_cython_6cygrpc_SslPemKeyCertPair *__pyx_v_self, CYTHON_UNUSED PyObject *__pyx_v___pyx_state); /* proto */
static int __pyx_pf_4grpc_7_cython_6cygrpc_18CompressionOptions___cinit__(struct __pyx_obj_4grpc_7_cython_6cygrpc_CompressionOptions *__pyx_v_self); /* proto */
static PyObject *__pyx_pf_4grpc_7_cython_6cygrpc_18CompressionOptions_2enable_algorithm(struct __pyx_obj_4grpc_7_cython_6cygrpc_CompressionOptions *__pyx_v_self, grpc_compression_algorithm __pyx_v_algorithm); /* proto */
static PyObject *__pyx_pf_4grpc_7_cython_6cygrpc_18CompressionOptions_4disable_algorithm(struct __pyx_obj_4grpc_7_cython_6cygrpc_CompressionOptions *__pyx_v_self, grpc_compression_algorithm __pyx_v_algorithm); /* proto */
static PyObject *__pyx_pf_4grpc_7_cython_6cygrpc_18CompressionOptions_6is_algorithm_enabled(struct __pyx_obj_4grpc_7_cython_6cygrpc_CompressionOptions *__pyx_v_self, grpc_compression_algorithm __pyx_v_algorithm); /* proto */
static PyObject *__pyx_pf_4grpc_7_cython_6cygrpc_18CompressionOptions_8to_channel_arg(struct __pyx_obj_4grpc_7_cython_6cygrpc_CompressionOptions *__pyx_v_self); /* proto */
static PyObject *__pyx_pf_4grpc_7_cython_6cygrpc_18CompressionOptions_10__reduce_cython__(CYTHON_UNUSED struct __pyx_obj_4grpc_7_cython_6cygrpc_CompressionOptions *__pyx_v_self); /* proto */
static PyObject *__pyx_pf_4grpc_7_cython_6cygrpc_18CompressionOptions_12__setstate_cython__(CYTHON_UNUSED struct __pyx_obj_4grpc_7_cython_6cygrpc_CompressionOptions *__pyx_v_self, CYTHON_UNUSED PyObject *__pyx_v___pyx_state); /* proto */
static PyObject *__pyx_pf_4grpc_7_cython_6cygrpc_26compression_algorithm_name(CYTHON_UNUSED PyObject *__pyx_self, grpc_compression_algorithm __pyx_v_algorithm); /* proto */
static PyObject *__pyx_pf_4grpc_7_cython_6cygrpc_28peer_identities(CYTHON_UNUSED PyObject *__pyx_self, struct __pyx_obj_4grpc_7_cython_6cygrpc_Call *__pyx_v_call); /* proto */
static PyObject *__pyx_pf_4grpc_7_cython_6cygrpc_30peer_identity_key(CYTHON_UNUSED PyObject *__pyx_self, struct __pyx_obj_4grpc_7_cython_6cygrpc_Call *__pyx_v_call); /* proto */
static PyObject *__pyx_pf_4grpc_7_cython_6cygrpc_32auth_context(CYTHON_UNUSED PyObject *__pyx_self, struct __pyx_obj_4grpc_7_cython_6cygrpc_Call *__pyx_v_call); /* proto */
static int __pyx_pf_4grpc_7_cython_6cygrpc_6Server___cinit__(struct __pyx_obj_4grpc_7_cython_6cygrpc_Server *__pyx_v_self, PyObject *__pyx_v_arguments); /* proto */
static PyObject *__pyx_pf_4grpc_7_cython_6cygrpc_6Server_2request_call(struct __pyx_obj_4grpc_7_cython_6cygrpc_Server *__pyx_v_self, struct __pyx_obj_4grpc_7_cython_6cygrpc_CompletionQueue *__pyx_v_call_queue, struct __pyx_obj_4grpc_7_cython_6cygrpc_CompletionQueue *__pyx_v_server_queue, PyObject *__pyx_v_tag); /* proto */
static PyObject *__pyx_pf_4grpc_7_cython_6cygrpc_6Server_4register_completion_queue(struct __pyx_obj_4grpc_7_cython_6cygrpc_Server *__pyx_v_self, struct __pyx_obj_4grpc_7_cython_6cygrpc_CompletionQueue *__pyx_v_queue); /* proto */
static PyObject *__pyx_pf_4grpc_7_cython_6cygrpc_6Server_6start(struct __pyx_obj_4grpc_7_cython_6cygrpc_Server *__pyx_v_self); /* proto */
static PyObject *__pyx_pf_4grpc_7_cython_6cygrpc_6Server_8add_http2_port(struct __pyx_obj_4grpc_7_cython_6cygrpc_Server *__pyx_v_self, PyObject *__pyx_v_address, struct __pyx_obj_4grpc_7_cython_6cygrpc_ServerCredentials *__pyx_v_server_credentials); /* proto */
static PyObject *__pyx_pf_4grpc_7_cython_6cygrpc_6Server_10shutdown(struct __pyx_obj_4grpc_7_cython_6cygrpc_Server *__pyx_v_self, struct __pyx_obj_4grpc_7_cython_6cygrpc_CompletionQueue *__pyx_v_queue, PyObject *__pyx_v_tag); /* proto */
static PyObject *__pyx_pf_4grpc_7_cython_6cygrpc_6Server_12cancel_all_calls(struct __pyx_obj_4grpc_7_cython_6cygrpc_Server *__pyx_v_self); /* proto */
static PyObject *__pyx_pf_4grpc_7_cython_6cygrpc_6Server_14destroy(struct __pyx_obj_4grpc_7_cython_6cygrpc_Server *__pyx_v_self); /* proto */
static void __pyx_pf_4grpc_7_cython_6cygrpc_6Server_16__dealloc__(struct __pyx_obj_4grpc_7_cython_6cygrpc_Server *__pyx_v_self); /* proto */
static PyObject *__pyx_pf_4grpc_7_cython_6cygrpc_6Server_18__reduce_cython__(CYTHON_UNUSED struct __pyx_obj_4grpc_7_cython_6cygrpc_Server *__pyx_v_self); /* proto */
static PyObject *__pyx_pf_4grpc_7_cython_6cygrpc_6Server_20__setstate_cython__(CYTHON_UNUSED struct __pyx_obj_4grpc_7_cython_6cygrpc_Server *__pyx_v_self, CYTHON_UNUSED PyObject *__pyx_v___pyx_state); /* proto */
static PyObject *__pyx_pf_4grpc_7_cython_6cygrpc_4_Tag___reduce_cython__(struct __pyx_obj_4grpc_7_cython_6cygrpc__Tag *__pyx_v_self); /* proto */
static PyObject *__pyx_pf_4grpc_7_cython_6cygrpc_4_Tag_2__setstate_cython__(struct __pyx_obj_4grpc_7_cython_6cygrpc__Tag *__pyx_v_self, PyObject *__pyx_v___pyx_state); /* proto */
static int __pyx_pf_4grpc_7_cython_6cygrpc_16_ConnectivityTag___cinit__(struct __pyx_obj_4grpc_7_cython_6cygrpc__ConnectivityTag *__pyx_v_self, PyObject *__pyx_v_user_tag); /* proto */
static PyObject *__pyx_pf_4grpc_7_cython_6cygrpc_16_ConnectivityTag_9_user_tag___get__(struct __pyx_obj_4grpc_7_cython_6cygrpc__ConnectivityTag *__pyx_v_self); /* proto */
static PyObject *__pyx_pf_4grpc_7_cython_6cygrpc_16_ConnectivityTag_2__reduce_cython__(CYTHON_UNUSED struct __pyx_obj_4grpc_7_cython_6cygrpc__ConnectivityTag *__pyx_v_self); /* proto */
static PyObject *__pyx_pf_4grpc_7_cython_6cygrpc_16_ConnectivityTag_4__setstate_cython__(CYTHON_UNUSED struct __pyx_obj_4grpc_7_cython_6cygrpc__ConnectivityTag *__pyx_v_self, CYTHON_UNUSED PyObject *__pyx_v___pyx_state); /* proto */
static int __pyx_pf_4grpc_7_cython_6cygrpc_15_RequestCallTag___cinit__(struct __pyx_obj_4grpc_7_cython_6cygrpc__RequestCallTag *__pyx_v_self, PyObject *__pyx_v_user_tag, struct __pyx_obj_4grpc_7_cython_6cygrpc__VTable *__pyx_v_vtable); /* proto */
static PyObject *__pyx_pf_4grpc_7_cython_6cygrpc_15_RequestCallTag_9_user_tag___get__(struct __pyx_obj_4grpc_7_cython_6cygrpc__RequestCallTag *__pyx_v_self); /* proto */
static PyObject *__pyx_pf_4grpc_7_cython_6cygrpc_15_RequestCallTag_2__reduce_cython__(CYTHON_UNUSED struct __pyx_obj_4grpc_7_cython_6cygrpc__RequestCallTag *__pyx_v_self); /* proto */
static PyObject *__pyx_pf_4grpc_7_cython_6cygrpc_15_RequestCallTag_4__setstate_cython__(CYTHON_UNUSED struct __pyx_obj_4grpc_7_cython_6cygrpc__RequestCallTag *__pyx_v_self, CYTHON_UNUSED PyObject *__pyx_v___pyx_state); /* proto */
static int __pyx_pf_4grpc_7_cython_6cygrpc_18_BatchOperationTag___cinit__(struct __pyx_obj_4grpc_7_cython_6cygrpc__BatchOperationTag *__pyx_v_self, PyObject *__pyx_v_user_tag, PyObject *__pyx_v_operations, PyObject *__pyx_v_call); /* proto */
static PyObject *__pyx_pf_4grpc_7_cython_6cygrpc_18_BatchOperationTag_11_operations___get__(struct __pyx_obj_4grpc_7_cython_6cygrpc__BatchOperationTag *__pyx_v_self); /* proto */
static PyObject *__pyx_pf_4grpc_7_cython_6cygrpc_18_BatchOperationTag_14_retained_call___get__(struct __pyx_obj_4grpc_7_cython_6cygrpc__BatchOperationTag *__pyx_v_self); /* proto */
static PyObject *__pyx_pf_4grpc_7_cython_6cygrpc_18_BatchOperationTag_2__reduce_cython__(CYTHON_UNUSED struct __pyx_obj_4grpc_7_cython_6cygrpc__BatchOperationTag *__pyx_v_self); /* proto */
static PyObject *__pyx_pf_4grpc_7_cython_6cygrpc_18_BatchOperationTag_4__setstate_cython__(CYTHON_UNUSED struct __pyx_obj_4grpc_7_cython_6cygrpc__BatchOperationTag *__pyx_v_self, CYTHON_UNUSED PyObject *__pyx_v___pyx_state); /* proto */
static int __pyx_pf_4grpc_7_cython_6cygrpc_18_ServerShutdownTag___cinit__(struct __pyx_obj_4grpc_7_cython_6cygrpc__ServerShutdownTag *__pyx_v_self, PyObject *__pyx_v_user_tag, PyObject *__pyx_v_shutting_down_server); /* proto */
static PyObject *__pyx_pf_4grpc_7_cython_6cygrpc_18_ServerShutdownTag_9_user_tag___get__(struct __pyx_obj_4grpc_7_cython_6cygrpc__ServerShutdownTag *__pyx_v_self); /* proto */
static PyObject *__pyx_pf_4grpc_7_cython_6cygrpc_18_ServerShutdownTag_21_shutting_down_server___get__(struct __pyx_obj_4grpc_7_cython_6cygrpc__ServerShutdownTag *__pyx_v_self); /* proto */
static PyObject *__pyx_pf_4grpc_7_cython_6cygrpc_18_ServerShutdownTag_2__reduce_cython__(CYTHON_UNUSED struct __pyx_obj_4grpc_7_cython_6cygrpc__ServerShutdownTag *__pyx_v_self); /* proto */
static PyObject *__pyx_pf_4grpc_7_cython_6cygrpc_18_ServerShutdownTag_4__setstate_cython__(CYTHON_UNUSED struct __pyx_obj_4grpc_7_cython_6cygrpc__ServerShutdownTag *__pyx_v_self, CYTHON_UNUSED PyObject *__pyx_v___pyx_state); /* proto */
static int __pyx_pf_4grpc_7_cython_6cygrpc_7_VTable___cinit__(struct __pyx_obj_4grpc_7_cython_6cygrpc__VTable *__pyx_v_self); /* proto */
static PyObject *__pyx_pf_4grpc_7_cython_6cygrpc_7_VTable_2__reduce_cython__(CYTHON_UNUSED struct __pyx_obj_4grpc_7_cython_6cygrpc__VTable *__pyx_v_self); /* proto */
static PyObject *__pyx_pf_4grpc_7_cython_6cygrpc_7_VTable_4__setstate_cython__(CYTHON_UNUSED struct __pyx_obj_4grpc_7_cython_6cygrpc__VTable *__pyx_v_self, CYTHON_UNUSED PyObject *__pyx_v___pyx_state); /* proto */
static PyObject *__pyx_pf_4grpc_7_cython_6cygrpc_34install_context_from_request_call_event(CYTHON_UNUSED PyObject *__pyx_self, CYTHON_UNUSED struct __pyx_obj_4grpc_7_cython_6cygrpc_RequestCallEvent *__pyx_v_event); /* proto */
static PyObject *__pyx_pf_4grpc_7_cython_6cygrpc_36uninstall_context(CYTHON_UNUSED PyObject *__pyx_self); /* proto */
static PyObject *__pyx_pf_4grpc_7_cython_6cygrpc_38build_census_context(CYTHON_UNUSED PyObject *__pyx_self); /* proto */
static PyObject *__pyx_pf_4grpc_7_cython_6cygrpc_13CensusContext___reduce_cython__(struct __pyx_obj_4grpc_7_cython_6cygrpc_CensusContext *__pyx_v_self); /* proto */
static PyObject *__pyx_pf_4grpc_7_cython_6cygrpc_13CensusContext_2__setstate_cython__(struct __pyx_obj_4grpc_7_cython_6cygrpc_CensusContext *__pyx_v_self, PyObject *__pyx_v___pyx_state); /* proto */
static PyObject *__pyx_pf_4grpc_7_cython_6cygrpc_40set_census_context_on_call(CYTHON_UNUSED PyObject *__pyx_self, CYTHON_UNUSED struct __pyx_obj_4grpc_7_cython_6cygrpc__CallState *__pyx_v_call_state, CYTHON_UNUSED struct __pyx_obj_4grpc_7_cython_6cygrpc_CensusContext *__pyx_v_census_ctx); /* proto */
static PyObject *__pyx_pf_4grpc_7_cython_6cygrpc_42get_deadline_from_context(CYTHON_UNUSED PyObject *__pyx_self); /* proto */
static PyObject *__pyx_pf_4grpc_7_cython_6cygrpc_20tuples_to_resolvaddr_genexpr(PyObject *__pyx_self); /* proto */
static PyObject *__pyx_pf_4grpc_7_cython_6cygrpc_44_spawn_greenlet(CYTHON_UNUSED PyObject *__pyx_self, PyObject *__pyx_v_args); /* proto */
static int __pyx_pf_4grpc_7_cython_6cygrpc_13SocketWrapper___cinit__(struct __pyx_obj_4grpc_7_cython_6cygrpc_SocketWrapper *__pyx_v_self); /* proto */
static PyObject *__pyx_pf_4grpc_7_cython_6cygrpc_13SocketWrapper_2__reduce_cython__(CYTHON_UNUSED struct __pyx_obj_4grpc_7_cython_6cygrpc_SocketWrapper *__pyx_v_self); /* proto */
static PyObject *__pyx_pf_4grpc_7_cython_6cygrpc_13SocketWrapper_4__setstate_cython__(CYTHON_UNUSED struct __pyx_obj_4grpc_7_cython_6cygrpc_SocketWrapper *__pyx_v_self, CYTHON_UNUSED PyObject *__pyx_v___pyx_state); /* proto */
static PyObject *__pyx_pf_4grpc_7_cython_6cygrpc_46socket_connect_async(CYTHON_UNUSED PyObject *__pyx_self, PyObject *__pyx_v_socket_wrapper, PyObject *__pyx_v_addr_tuple); /* proto */
static PyObject *__pyx_pf_4grpc_7_cython_6cygrpc_48socket_sendmsg(CYTHON_UNUSED PyObject *__pyx_self, PyObject *__pyx_v_socket, PyObject *__pyx_v_write_bytes); /* proto */
static PyObject *__pyx_pf_4grpc_7_cython_6cygrpc_50socket_write_async(CYTHON_UNUSED PyObject *__pyx_self, PyObject *__pyx_v_socket_wrapper, PyObject *__pyx_v_write_bytes); /* proto */
static PyObject *__pyx_pf_4grpc_7_cython_6cygrpc_52socket_read_async(CYTHON_UNUSED PyObject *__pyx_self, PyObject *__pyx_v_socket_wrapper); /* proto */
static PyObject *__pyx_pf_4grpc_7_cython_6cygrpc_54applysockopts(CYTHON_UNUSED PyObject *__pyx_self, PyObject *__pyx_v_s); /* proto */
static PyObject *__pyx_pf_4grpc_7_cython_6cygrpc_56socket_accept_async(CYTHON_UNUSED PyObject *__pyx_self, PyObject *__pyx_v_s); /* proto */
static int __pyx_pf_4grpc_7_cython_6cygrpc_14ResolveWrapper___cinit__(struct __pyx_obj_4grpc_7_cython_6cygrpc_ResolveWrapper *__pyx_v_self); /* proto */
static PyObject *__pyx_pf_4grpc_7_cython_6cygrpc_14ResolveWrapper_2__reduce_cython__(CYTHON_UNUSED struct __pyx_obj_4grpc_7_cython_6cygrpc_ResolveWrapper *__pyx_v_self); /* proto */
static PyObject *__pyx_pf_4grpc_7_cython_6cygrpc_14ResolveWrapper_4__setstate_cython__(CYTHON_UNUSED struct __pyx_obj_4grpc_7_cython_6cygrpc_ResolveWrapper *__pyx_v_self, CYTHON_UNUSED PyObject *__pyx_v___pyx_state); /* proto */
static PyObject *__pyx_pf_4grpc_7_cython_6cygrpc_58socket_resolve_async_python(CYTHON_UNUSED PyObject *__pyx_self, PyObject *__pyx_v_resolve_wrapper); /* proto */
static int __pyx_pf_4grpc_7_cython_6cygrpc_12TimerWrapper___cinit__(struct __pyx_obj_4grpc_7_cython_6cygrpc_TimerWrapper *__pyx_v_self, PyObject *__pyx_v_deadline); /* proto */
static PyObject *__pyx_pf_4grpc_7_cython_6cygrpc_12TimerWrapper_2start(struct __pyx_obj_4grpc_7_cython_6cygrpc_TimerWrapper *__pyx_v_self); /* proto */
static PyObject *__pyx_pf_4grpc_7_cython_6cygrpc_12TimerWrapper_4on_finish(struct __pyx_obj_4grpc_7_cython_6cygrpc_TimerWrapper *__pyx_v_self); /* proto */
static PyObject *__pyx_pf_4grpc_7_cython_6cygrpc_12TimerWrapper_6stop(struct __pyx_obj_4grpc_7_cython_6cygrpc_TimerWrapper *__pyx_v_self); /* proto */
static PyObject *__pyx_pf_4grpc_7_cython_6cygrpc_12TimerWrapper_8__reduce_cython__(CYTHON_UNUSED struct __pyx_obj_4grpc_7_cython_6cygrpc_TimerWrapper *__pyx_v_self); /* proto */
static PyObject *__pyx_pf_4grpc_7_cython_6cygrpc_12TimerWrapper_10__setstate_cython__(CYTHON_UNUSED struct __pyx_obj_4grpc_7_cython_6cygrpc_TimerWrapper *__pyx_v_self, CYTHON_UNUSED PyObject *__pyx_v___pyx_state); /* proto */
static PyObject *__pyx_pf_4grpc_7_cython_6cygrpc_16init_grpc_gevent_cb_func(CYTHON_UNUSED PyObject *__pyx_self, PyObject *__pyx_v_cb, PyObject *__pyx_v_args); /* proto */
static PyObject *__pyx_pf_4grpc_7_cython_6cygrpc_60init_grpc_gevent(CYTHON_UNUSED PyObject *__pyx_self); /* proto */
static PyObject *__pyx_pf_4grpc_7_cython_6cygrpc_62fork_handlers_and_grpc_init(CYTHON_UNUSED PyObject *__pyx_self); /* proto */
static PyObject *__pyx_pf_4grpc_7_cython_6cygrpc_17ForkManagedThread_8__init___managed_target(PyObject *__pyx_self, PyObject *__pyx_v_args); /* proto */
static PyObject *__pyx_pf_4grpc_7_cython_6cygrpc_17ForkManagedThread___init__(CYTHON_UNUSED PyObject *__pyx_self, PyObject *__pyx_v_self, PyObject *__pyx_v_target, PyObject *__pyx_v_args); /* proto */
static PyObject *__pyx_pf_4grpc_7_cython_6cygrpc_17ForkManagedThread_2setDaemon(CYTHON_UNUSED PyObject *__pyx_self, PyObject *__pyx_v_self, PyObject *__pyx_v_daemonic); /* proto */
static PyObject *__pyx_pf_4grpc_7_cython_6cygrpc_17ForkManagedThread_4start(CYTHON_UNUSED PyObject *__pyx_self, PyObject *__pyx_v_self); /* proto */
static PyObject *__pyx_pf_4grpc_7_cython_6cygrpc_17ForkManagedThread_6join(CYTHON_UNUSED PyObject *__pyx_self, PyObject *__pyx_v_self); /* proto */
static PyObject *__pyx_pf_4grpc_7_cython_6cygrpc_64block_if_fork_in_progress(CYTHON_UNUSED PyObject *__pyx_self, PyObject *__pyx_v_postfork_state_to_reset); /* proto */
static PyObject *__pyx_pf_4grpc_7_cython_6cygrpc_66enter_user_request_generator(CYTHON_UNUSED PyObject *__pyx_self); /* proto */
static PyObject *__pyx_pf_4grpc_7_cython_6cygrpc_68return_from_user_request_generator(CYTHON_UNUSED PyObject *__pyx_self); /* proto */
static PyObject *__pyx_pf_4grpc_7_cython_6cygrpc_70get_fork_epoch(CYTHON_UNUSED PyObject *__pyx_self); /* proto */
static PyObject *__pyx_pf_4grpc_7_cython_6cygrpc_72is_fork_support_enabled(CYTHON_UNUSED PyObject *__pyx_self); /* proto */
static PyObject *__pyx_pf_4grpc_7_cython_6cygrpc_74fork_register_channel(CYTHON_UNUSED PyObject *__pyx_self, PyObject *__pyx_v_channel); /* proto */
static PyObject *__pyx_pf_4grpc_7_cython_6cygrpc_76fork_unregister_channel(CYTHON_UNUSED PyObject *__pyx_self, PyObject *__pyx_v_channel); /* proto */
static PyObject *__pyx_pf_4grpc_7_cython_6cygrpc_18_ActiveThreadCount___init__(CYTHON_UNUSED PyObject *__pyx_self, PyObject *__pyx_v_self); /* proto */
static PyObject *__pyx_pf_4grpc_7_cython_6cygrpc_18_ActiveThreadCount_2increment(CYTHON_UNUSED PyObject *__pyx_self, PyObject *__pyx_v_self); /* proto */
static PyObject *__pyx_pf_4grpc_7_cython_6cygrpc_18_ActiveThreadCount_4decrement(CYTHON_UNUSED PyObject *__pyx_self, PyObject *__pyx_v_self); /* proto */
static PyObject *__pyx_pf_4grpc_7_cython_6cygrpc_18_ActiveThreadCount_6await_zero_threads(CYTHON_UNUSED PyObject *__pyx_self, PyObject *__pyx_v_self, PyObject *__pyx_v_timeout_secs); /* proto */
static PyObject *__pyx_pf_4grpc_7_cython_6cygrpc_10_ForkState___init__(CYTHON_UNUSED PyObject *__pyx_self, PyObject *__pyx_v_self); /* proto */
static PyObject *__pyx_pf_4grpc_7_cython_6cygrpc_78__pyx_unpickle_ChannelCredentials(CYTHON_UNUSED PyObject *__pyx_self, PyObject *__pyx_v___pyx_type, long __pyx_v___pyx_checksum, PyObject *__pyx_v___pyx_state); /* proto */
static PyObject *__pyx_pf_4grpc_7_cython_6cygrpc_80__pyx_unpickle__Tag(CYTHON_UNUSED PyObject *__pyx_self, PyObject *__pyx_v___pyx_type, long __pyx_v___pyx_checksum, PyObject *__pyx_v___pyx_state); /* proto */
static PyObject *__pyx_pf_4grpc_7_cython_6cygrpc_82__pyx_unpickle_CensusContext(CYTHON_UNUSED PyObject *__pyx_self, PyObject *__pyx_v___pyx_type, long __pyx_v___pyx_checksum, PyObject *__pyx_v___pyx_state); /* proto */
static PyObject *__pyx_tp_new_4grpc_7_cython_6cygrpc__ChannelArg(PyTypeObject *t, PyObject *a, PyObject *k); /*proto*/
static PyObject *__pyx_tp_new_4grpc_7_cython_6cygrpc__ChannelArgs(PyTypeObject *t, PyObject *a, PyObject *k); /*proto*/
static PyObject *__pyx_tp_new_4grpc_7_cython_6cygrpc_Call(PyTypeObject *t, PyObject *a, PyObject *k); /*proto*/
static PyObject *__pyx_tp_new_4grpc_7_cython_6cygrpc__CallState(PyTypeObject *t, PyObject *a, PyObject *k); /*proto*/
static PyObject *__pyx_tp_new_4grpc_7_cython_6cygrpc__ChannelState(PyTypeObject *t, PyObject *a, PyObject *k); /*proto*/
static PyObject *__pyx_tp_new_4grpc_7_cython_6cygrpc_IntegratedCall(PyTypeObject *t, PyObject *a, PyObject *k); /*proto*/
static PyObject *__pyx_tp_new_4grpc_7_cython_6cygrpc_SegregatedCall(PyTypeObject *t, PyObject *a, PyObject *k); /*proto*/
static PyObject *__pyx_tp_new_4grpc_7_cython_6cygrpc_Channel(PyTypeObject *t, PyObject *a, PyObject *k); /*proto*/
static PyObject *__pyx_tp_new_4grpc_7_cython_6cygrpc_CallCredentials(PyTypeObject *t, PyObject *a, PyObject *k); /*proto*/
static PyObject *__pyx_tp_new_4grpc_7_cython_6cygrpc_MetadataPluginCallCredentials(PyTypeObject *t, PyObject *a, PyObject *k); /*proto*/
static PyObject *__pyx_tp_new_4grpc_7_cython_6cygrpc_CompositeCallCredentials(PyTypeObject *t, PyObject *a, PyObject *k); /*proto*/
static PyObject *__pyx_tp_new_4grpc_7_cython_6cygrpc_ChannelCredentials(PyTypeObject *t, PyObject *a, PyObject *k); /*proto*/
static PyObject *__pyx_tp_new_4grpc_7_cython_6cygrpc_SSLSessionCacheLRU(PyTypeObject *t, PyObject *a, PyObject *k); /*proto*/
static PyObject *__pyx_tp_new_4grpc_7_cython_6cygrpc_SSLChannelCredentials(PyTypeObject *t, PyObject *a, PyObject *k); /*proto*/
static PyObject *__pyx_tp_new_4grpc_7_cython_6cygrpc_CompositeChannelCredentials(PyTypeObject *t, PyObject *a, PyObject *k); /*proto*/
static PyObject *__pyx_tp_new_4grpc_7_cython_6cygrpc_ServerCertificateConfig(PyTypeObject *t, PyObject *a, PyObject *k); /*proto*/
static PyObject *__pyx_tp_new_4grpc_7_cython_6cygrpc_ServerCredentials(PyTypeObject *t, PyObject *a, PyObject *k); /*proto*/
static PyObject *__pyx_tp_new_4grpc_7_cython_6cygrpc_CompletionQueue(PyTypeObject *t, PyObject *a, PyObject *k); /*proto*/
static PyObject *__pyx_tp_new_4grpc_7_cython_6cygrpc_ConnectivityEvent(PyTypeObject *t, PyObject *a, PyObject *k); /*proto*/
static PyObject *__pyx_tp_new_4grpc_7_cython_6cygrpc_RequestCallEvent(PyTypeObject *t, PyObject *a, PyObject *k); /*proto*/
static PyObject *__pyx_tp_new_4grpc_7_cython_6cygrpc_BatchOperationEvent(PyTypeObject *t, PyObject *a, PyObject *k); /*proto*/
static PyObject *__pyx_tp_new_4grpc_7_cython_6cygrpc_ServerShutdownEvent(PyTypeObject *t, PyObject *a, PyObject *k); /*proto*/
static PyObject *__pyx_tp_new_4grpc_7_cython_6cygrpc_Operation(PyTypeObject *t, PyObject *a, PyObject *k); /*proto*/
static PyObject *__pyx_tp_new_4grpc_7_cython_6cygrpc_SendInitialMetadataOperation(PyTypeObject *t, PyObject *a, PyObject *k); /*proto*/
static PyObject *__pyx_tp_new_4grpc_7_cython_6cygrpc_SendMessageOperation(PyTypeObject *t, PyObject *a, PyObject *k); /*proto*/
static PyObject *__pyx_tp_new_4grpc_7_cython_6cygrpc_SendCloseFromClientOperation(PyTypeObject *t, PyObject *a, PyObject *k); /*proto*/
static PyObject *__pyx_tp_new_4grpc_7_cython_6cygrpc_SendStatusFromServerOperation(PyTypeObject *t, PyObject *a, PyObject *k); /*proto*/
static PyObject *__pyx_tp_new_4grpc_7_cython_6cygrpc_ReceiveInitialMetadataOperation(PyTypeObject *t, PyObject *a, PyObject *k); /*proto*/
static PyObject *__pyx_tp_new_4grpc_7_cython_6cygrpc_ReceiveMessageOperation(PyTypeObject *t, PyObject *a, PyObject *k); /*proto*/
static PyObject *__pyx_tp_new_4grpc_7_cython_6cygrpc_ReceiveStatusOnClientOperation(PyTypeObject *t, PyObject *a, PyObject *k); /*proto*/
static PyObject *__pyx_tp_new_4grpc_7_cython_6cygrpc_ReceiveCloseOnServerOperation(PyTypeObject *t, PyObject *a, PyObject *k); /*proto*/
static PyObject *__pyx_tp_new_4grpc_7_cython_6cygrpc_CallDetails(PyTypeObject *t, PyObject *a, PyObject *k); /*proto*/
static PyObject *__pyx_tp_new_4grpc_7_cython_6cygrpc_SslPemKeyCertPair(PyTypeObject *t, PyObject *a, PyObject *k); /*proto*/
static PyObject *__pyx_tp_new_4grpc_7_cython_6cygrpc_CompressionOptions(PyTypeObject *t, PyObject *a, PyObject *k); /*proto*/
static PyObject *__pyx_tp_new_4grpc_7_cython_6cygrpc_Server(PyTypeObject *t, PyObject *a, PyObject *k); /*proto*/
static PyObject *__pyx_tp_new_4grpc_7_cython_6cygrpc__Tag(PyTypeObject *t, PyObject *a, PyObject *k); /*proto*/
static PyObject *__pyx_tp_new_4grpc_7_cython_6cygrpc__ConnectivityTag(PyTypeObject *t, PyObject *a, PyObject *k); /*proto*/
static PyObject *__pyx_tp_new_4grpc_7_cython_6cygrpc__RequestCallTag(PyTypeObject *t, PyObject *a, PyObject *k); /*proto*/
static PyObject *__pyx_tp_new_4grpc_7_cython_6cygrpc__BatchOperationTag(PyTypeObject *t, PyObject *a, PyObject *k); /*proto*/
static PyObject *__pyx_tp_new_4grpc_7_cython_6cygrpc__ServerShutdownTag(PyTypeObject *t, PyObject *a, PyObject *k); /*proto*/
static PyObject *__pyx_tp_new_4grpc_7_cython_6cygrpc__VTable(PyTypeObject *t, PyObject *a, PyObject *k); /*proto*/
static PyObject *__pyx_tp_new_4grpc_7_cython_6cygrpc_TimerWrapper(PyTypeObject *t, PyObject *a, PyObject *k); /*proto*/
static PyObject *__pyx_tp_new_4grpc_7_cython_6cygrpc_SocketWrapper(PyTypeObject *t, PyObject *a, PyObject *k); /*proto*/
static PyObject *__pyx_tp_new_4grpc_7_cython_6cygrpc_ResolveWrapper(PyTypeObject *t, PyObject *a, PyObject *k); /*proto*/
static PyObject *__pyx_tp_new_4grpc_7_cython_6cygrpc__GrpcArgWrapper(PyTypeObject *t, PyObject *a, PyObject *k); /*proto*/
static PyObject *__pyx_tp_new_4grpc_7_cython_6cygrpc_CensusContext(PyTypeObject *t, PyObject *a, PyObject *k); /*proto*/
static PyObject *__pyx_tp_new_4grpc_7_cython_6cygrpc___pyx_scope_struct____pyx_f_4grpc_7_cython_6cygrpc__integrated_call(PyTypeObject *t, PyObject *a, PyObject *k); /*proto*/
static PyObject *__pyx_tp_new_4grpc_7_cython_6cygrpc___pyx_scope_struct_1_next_event(PyTypeObject *t, PyObject *a, PyObject *k); /*proto*/
static PyObject *__pyx_tp_new_4grpc_7_cython_6cygrpc___pyx_scope_struct_2___pyx_f_4grpc_7_cython_6cygrpc__segregated_call(PyTypeObject *t, PyObject *a, PyObject *k); /*proto*/
static PyObject *__pyx_tp_new_4grpc_7_cython_6cygrpc___pyx_scope_struct_3_next_call_event(PyTypeObject *t, PyObject *a, PyObject *k); /*proto*/
static PyObject *__pyx_tp_new_4grpc_7_cython_6cygrpc___pyx_scope_struct_4___pyx_f_4grpc_7_cython_6cygrpc__get_metadata(PyTypeObject *t, PyObject *a, PyObject *k); /*proto*/
static PyObject *__pyx_tp_new_4grpc_7_cython_6cygrpc___pyx_scope_struct_5___pyx_f_4grpc_7_cython_6cygrpc__metadata(PyTypeObject *t, PyObject *a, PyObject *k); /*proto*/
static PyObject *__pyx_tp_new_4grpc_7_cython_6cygrpc___pyx_scope_struct_6_genexpr(PyTypeObject *t, PyObject *a, PyObject *k); /*proto*/
static PyObject *__pyx_tp_new_4grpc_7_cython_6cygrpc___pyx_scope_struct_7___pyx_f_4grpc_7_cython_6cygrpc_tuples_to_resolvaddr(PyTypeObject *t, PyObject *a, PyObject *k); /*proto*/
static PyObject *__pyx_tp_new_4grpc_7_cython_6cygrpc___pyx_scope_struct_8_genexpr(PyTypeObject *t, PyObject *a, PyObject *k); /*proto*/
static PyObject *__pyx_tp_new_4grpc_7_cython_6cygrpc___pyx_scope_struct_9___init__(PyTypeObject *t, PyObject *a, PyObject *k); /*proto*/
static __Pyx_CachedCFunction __pyx_umethod_PyDict_Type_pop = {0, &__pyx_n_s_pop, 0, 0, 0};
static __Pyx_CachedCFunction __pyx_umethod_PyDict_Type_values = {0, &__pyx_n_s_values, 0, 0, 0};
static PyObject *__pyx_float_1_0;
static PyObject *__pyx_int_0;
static PyObject *__pyx_int_1;
static PyObject *__pyx_int_5;
static PyObject *__pyx_int_50;
static PyObject *__pyx_int_222419149;
static grpc_status_code __pyx_k__11;
static PyObject *__pyx_tuple_;
static PyObject *__pyx_tuple__3;
static PyObject *__pyx_tuple__4;
static PyObject *__pyx_tuple__5;
static PyObject *__pyx_tuple__6;
static PyObject *__pyx_tuple__7;
static PyObject *__pyx_tuple__8;
static PyObject *__pyx_tuple__9;
static PyObject *__pyx_tuple__10;
static PyObject *__pyx_tuple__12;
static PyObject *__pyx_tuple__13;
static PyObject *__pyx_tuple__14;
static PyObject *__pyx_tuple__15;
static PyObject *__pyx_tuple__16;
static PyObject *__pyx_tuple__17;
static PyObject *__pyx_tuple__18;
static PyObject *__pyx_tuple__19;
static PyObject *__pyx_tuple__20;
static PyObject *__pyx_tuple__21;
static PyObject *__pyx_tuple__22;
static PyObject *__pyx_tuple__23;
static PyObject *__pyx_tuple__24;
static PyObject *__pyx_tuple__25;
static PyObject *__pyx_tuple__26;
static PyObject *__pyx_tuple__27;
static PyObject *__pyx_tuple__28;
static PyObject *__pyx_tuple__29;
static PyObject *__pyx_tuple__31;
static PyObject *__pyx_tuple__33;
static PyObject *__pyx_tuple__34;
static PyObject *__pyx_tuple__35;
static PyObject *__pyx_tuple__37;
static PyObject *__pyx_tuple__38;
static PyObject *__pyx_tuple__39;
static PyObject *__pyx_tuple__40;
static PyObject *__pyx_tuple__41;
static PyObject *__pyx_tuple__42;
static PyObject *__pyx_tuple__43;
static PyObject *__pyx_tuple__44;
static PyObject *__pyx_tuple__46;
static PyObject *__pyx_tuple__47;
static PyObject *__pyx_tuple__48;
static PyObject *__pyx_tuple__49;
static PyObject *__pyx_tuple__50;
static PyObject *__pyx_tuple__51;
static PyObject *__pyx_tuple__52;
static PyObject *__pyx_tuple__54;
static PyObject *__pyx_tuple__55;
static PyObject *__pyx_tuple__56;
static PyObject *__pyx_tuple__57;
static PyObject *__pyx_tuple__58;
static PyObject *__pyx_tuple__59;
static PyObject *__pyx_tuple__60;
static PyObject *__pyx_tuple__61;
static PyObject *__pyx_tuple__62;
static PyObject *__pyx_tuple__63;
static PyObject *__pyx_tuple__64;
static PyObject *__pyx_tuple__65;
static PyObject *__pyx_tuple__66;
static PyObject *__pyx_tuple__67;
static PyObject *__pyx_tuple__68;
static PyObject *__pyx_tuple__69;
static PyObject *__pyx_tuple__70;
static PyObject *__pyx_tuple__71;
static PyObject *__pyx_tuple__72;
static PyObject *__pyx_tuple__73;
static PyObject *__pyx_tuple__74;
static PyObject *__pyx_tuple__75;
static PyObject *__pyx_tuple__76;
static PyObject *__pyx_tuple__77;
static PyObject *__pyx_tuple__78;
static PyObject *__pyx_tuple__79;
static PyObject *__pyx_tuple__80;
static PyObject *__pyx_tuple__81;
static PyObject *__pyx_tuple__82;
static PyObject *__pyx_tuple__83;
static PyObject *__pyx_tuple__84;
static PyObject *__pyx_tuple__85;
static PyObject *__pyx_tuple__86;
static PyObject *__pyx_tuple__87;
static PyObject *__pyx_tuple__88;
static PyObject *__pyx_tuple__89;
static PyObject *__pyx_tuple__90;
static PyObject *__pyx_tuple__91;
static PyObject *__pyx_tuple__92;
static PyObject *__pyx_tuple__93;
static PyObject *__pyx_tuple__94;
static PyObject *__pyx_tuple__95;
static PyObject *__pyx_tuple__96;
static PyObject *__pyx_tuple__97;
static PyObject *__pyx_tuple__98;
static PyObject *__pyx_tuple__99;
static PyObject *__pyx_slice__139;
static PyObject *__pyx_tuple__100;
static PyObject *__pyx_tuple__101;
static PyObject *__pyx_tuple__102;
static PyObject *__pyx_tuple__103;
static PyObject *__pyx_tuple__104;
static PyObject *__pyx_tuple__105;
static PyObject *__pyx_tuple__106;
static PyObject *__pyx_tuple__107;
static PyObject *__pyx_tuple__108;
static PyObject *__pyx_tuple__109;
static PyObject *__pyx_tuple__110;
static PyObject *__pyx_tuple__111;
static PyObject *__pyx_tuple__112;
static PyObject *__pyx_tuple__113;
static PyObject *__pyx_tuple__114;
static PyObject *__pyx_tuple__115;
static PyObject *__pyx_tuple__116;
static PyObject *__pyx_tuple__117;
static PyObject *__pyx_tuple__118;
static PyObject *__pyx_tuple__119;
static PyObject *__pyx_tuple__120;
static PyObject *__pyx_tuple__121;
static PyObject *__pyx_tuple__122;
static PyObject *__pyx_tuple__123;
static PyObject *__pyx_tuple__124;
static PyObject *__pyx_tuple__125;
static PyObject *__pyx_tuple__126;
static PyObject *__pyx_tuple__127;
static PyObject *__pyx_tuple__128;
static PyObject *__pyx_tuple__129;
static PyObject *__pyx_tuple__130;
static PyObject *__pyx_tuple__133;
static PyObject *__pyx_tuple__135;
static PyObject *__pyx_tuple__137;
static PyObject *__pyx_tuple__138;
static PyObject *__pyx_tuple__140;
static PyObject *__pyx_tuple__141;
static PyObject *__pyx_tuple__142;
static PyObject *__pyx_tuple__143;
static PyObject *__pyx_tuple__144;
static PyObject *__pyx_tuple__145;
static PyObject *__pyx_tuple__146;
static PyObject *__pyx_tuple__148;
static PyObject *__pyx_tuple__149;
static PyObject *__pyx_tuple__150;
static PyObject *__pyx_tuple__151;
static PyObject *__pyx_tuple__152;
static PyObject *__pyx_tuple__153;
static PyObject *__pyx_tuple__154;
static PyObject *__pyx_tuple__155;
static PyObject *__pyx_tuple__157;
static PyObject *__pyx_tuple__158;
static PyObject *__pyx_tuple__159;
static PyObject *__pyx_tuple__160;
static PyObject *__pyx_tuple__161;
static PyObject *__pyx_tuple__162;
static PyObject *__pyx_tuple__163;
static PyObject *__pyx_tuple__165;
static PyObject *__pyx_tuple__167;
static PyObject *__pyx_tuple__169;
static PyObject *__pyx_tuple__171;
static PyObject *__pyx_tuple__173;
static PyObject *__pyx_tuple__175;
static PyObject *__pyx_tuple__177;
static PyObject *__pyx_tuple__179;
static PyObject *__pyx_tuple__181;
static PyObject *__pyx_tuple__183;
static PyObject *__pyx_tuple__185;
static PyObject *__pyx_tuple__187;
static PyObject *__pyx_tuple__189;
static PyObject *__pyx_tuple__190;
static PyObject *__pyx_tuple__191;
static PyObject *__pyx_tuple__193;
static PyObject *__pyx_tuple__195;
static PyObject *__pyx_tuple__197;
static PyObject *__pyx_tuple__199;
static PyObject *__pyx_tuple__203;
static PyObject *__pyx_tuple__206;
static PyObject *__pyx_tuple__208;
static PyObject *__pyx_tuple__210;
static PyObject *__pyx_tuple__212;
static PyObject *__pyx_tuple__214;
static PyObject *__pyx_tuple__216;
static PyObject *__pyx_tuple__218;
static PyObject *__pyx_tuple__220;
static PyObject *__pyx_tuple__222;
static PyObject *__pyx_tuple__224;
static PyObject *__pyx_tuple__226;
static PyObject *__pyx_tuple__227;
static PyObject *__pyx_tuple__229;
static PyObject *__pyx_tuple__230;
static PyObject *__pyx_tuple__232;
static PyObject *__pyx_tuple__234;
static PyObject *__pyx_tuple__236;
static PyObject *__pyx_tuple__242;
static PyObject *__pyx_tuple__244;
static PyObject *__pyx_tuple__246;
static PyObject *__pyx_tuple__247;
static PyObject *__pyx_tuple__249;
static PyObject *__pyx_tuple__251;
static PyObject *__pyx_tuple__253;
static PyObject *__pyx_tuple__255;
static PyObject *__pyx_tuple__256;
static PyObject *__pyx_tuple__258;
static PyObject *__pyx_tuple__260;
static PyObject *__pyx_tuple__262;
static PyObject *__pyx_codeobj__30;
static PyObject *__pyx_codeobj__32;
static PyObject *__pyx_codeobj__36;
static PyObject *__pyx_codeobj__45;
static PyObject *__pyx_codeobj__53;
static PyObject *__pyx_codeobj__147;
static PyObject *__pyx_codeobj__156;
static PyObject *__pyx_codeobj__164;
static PyObject *__pyx_codeobj__166;
static PyObject *__pyx_codeobj__168;
static PyObject *__pyx_codeobj__170;
static PyObject *__pyx_codeobj__172;
static PyObject *__pyx_codeobj__174;
static PyObject *__pyx_codeobj__176;
static PyObject *__pyx_codeobj__178;
static PyObject *__pyx_codeobj__180;
static PyObject *__pyx_codeobj__182;
static PyObject *__pyx_codeobj__184;
static PyObject *__pyx_codeobj__186;
static PyObject *__pyx_codeobj__188;
static PyObject *__pyx_codeobj__192;
static PyObject *__pyx_codeobj__194;
static PyObject *__pyx_codeobj__196;
static PyObject *__pyx_codeobj__198;
static PyObject *__pyx_codeobj__200;
static PyObject *__pyx_codeobj__201;
static PyObject *__pyx_codeobj__202;
static PyObject *__pyx_codeobj__204;
static PyObject *__pyx_codeobj__205;
static PyObject *__pyx_codeobj__207;
static PyObject *__pyx_codeobj__209;
static PyObject *__pyx_codeobj__211;
static PyObject *__pyx_codeobj__213;
static PyObject *__pyx_codeobj__215;
static PyObject *__pyx_codeobj__217;
static PyObject *__pyx_codeobj__219;
static PyObject *__pyx_codeobj__221;
static PyObject *__pyx_codeobj__223;
static PyObject *__pyx_codeobj__225;
static PyObject *__pyx_codeobj__228;
static PyObject *__pyx_codeobj__231;
static PyObject *__pyx_codeobj__233;
static PyObject *__pyx_codeobj__235;
static PyObject *__pyx_codeobj__237;
static PyObject *__pyx_codeobj__238;
static PyObject *__pyx_codeobj__239;
static PyObject *__pyx_codeobj__240;
static PyObject *__pyx_codeobj__241;
static PyObject *__pyx_codeobj__243;
static PyObject *__pyx_codeobj__245;
static PyObject *__pyx_codeobj__248;
static PyObject *__pyx_codeobj__250;
static PyObject *__pyx_codeobj__252;
static PyObject *__pyx_codeobj__254;
static PyObject *__pyx_codeobj__257;
static PyObject *__pyx_codeobj__259;
static PyObject *__pyx_codeobj__261;
static PyObject *__pyx_codeobj__263;
/* Late includes */

/* "src/python/grpcio/grpc/_cython/_cygrpc/grpc_string.pyx.pxi":21
 * # This function will ascii encode unicode string inputs if neccesary.
 * # In Python3, unicode strings are the default str type.
 * cdef bytes str_to_bytes(object s):             # <<<<<<<<<<<<<<
 *   if s is None or isinstance(s, bytes):
 *     return s
 */

static PyObject *__pyx_f_4grpc_7_cython_6cygrpc_str_to_bytes(PyObject *__pyx_v_s) {
  PyObject *__pyx_r = NULL;
  __Pyx_RefNannyDeclarations
  int __pyx_t_1;
  int __pyx_t_2;
  int __pyx_t_3;
  PyObject *__pyx_t_4 = NULL;
  PyObject *__pyx_t_5 = NULL;
  PyObject *__pyx_t_6 = NULL;
  PyObject *__pyx_t_7 = NULL;
  __Pyx_RefNannySetupContext("str_to_bytes", 0);

  /* "src/python/grpcio/grpc/_cython/_cygrpc/grpc_string.pyx.pxi":22
 * # In Python3, unicode strings are the default str type.
 * cdef bytes str_to_bytes(object s):
 *   if s is None or isinstance(s, bytes):             # <<<<<<<<<<<<<<
 *     return s
 *   elif isinstance(s, unicode):
 */
  __pyx_t_2 = (__pyx_v_s == Py_None);
  __pyx_t_3 = (__pyx_t_2 != 0);
  if (!__pyx_t_3) {
  } else {
    __pyx_t_1 = __pyx_t_3;
    goto __pyx_L4_bool_binop_done;
  }
  __pyx_t_3 = PyBytes_Check(__pyx_v_s); 
  __pyx_t_2 = (__pyx_t_3 != 0);
  __pyx_t_1 = __pyx_t_2;
  __pyx_L4_bool_binop_done:;
  if (__pyx_t_1) {

    /* "src/python/grpcio/grpc/_cython/_cygrpc/grpc_string.pyx.pxi":23
 * cdef bytes str_to_bytes(object s):
 *   if s is None or isinstance(s, bytes):
 *     return s             # <<<<<<<<<<<<<<
 *   elif isinstance(s, unicode):
 *     return s.encode('ascii')
 */
    __Pyx_XDECREF(__pyx_r);
    if (!(likely(PyBytes_CheckExact(__pyx_v_s))||((__pyx_v_s) == Py_None)||(PyErr_Format(PyExc_TypeError, "Expected %.16s, got %.200s", "bytes", Py_TYPE(__pyx_v_s)->tp_name), 0))) __PYX_ERR(1, 23, __pyx_L1_error)
    __Pyx_INCREF(__pyx_v_s);
    __pyx_r = ((PyObject*)__pyx_v_s);
    goto __pyx_L0;

    /* "src/python/grpcio/grpc/_cython/_cygrpc/grpc_string.pyx.pxi":22
 * # In Python3, unicode strings are the default str type.
 * cdef bytes str_to_bytes(object s):
 *   if s is None or isinstance(s, bytes):             # <<<<<<<<<<<<<<
 *     return s
 *   elif isinstance(s, unicode):
 */
  }

  /* "src/python/grpcio/grpc/_cython/_cygrpc/grpc_string.pyx.pxi":24
 *   if s is None or isinstance(s, bytes):
 *     return s
 *   elif isinstance(s, unicode):             # <<<<<<<<<<<<<<
 *     return s.encode('ascii')
 *   else:
 */
  __pyx_t_1 = PyUnicode_Check(__pyx_v_s); 
  __pyx_t_2 = (__pyx_t_1 != 0);
  if (likely(__pyx_t_2)) {

    /* "src/python/grpcio/grpc/_cython/_cygrpc/grpc_string.pyx.pxi":25
 *     return s
 *   elif isinstance(s, unicode):
 *     return s.encode('ascii')             # <<<<<<<<<<<<<<
 *   else:
 *     raise TypeError('Expected bytes, str, or unicode, not {}'.format(type(s)))
 */
    __Pyx_XDECREF(__pyx_r);
    __pyx_t_4 = __Pyx_PyObject_GetAttrStr(__pyx_v_s, __pyx_n_s_encode); if (unlikely(!__pyx_t_4)) __PYX_ERR(1, 25, __pyx_L1_error)
    __Pyx_GOTREF(__pyx_t_4);
    __pyx_t_5 = __Pyx_PyObject_Call(__pyx_t_4, __pyx_tuple_, NULL); if (unlikely(!__pyx_t_5)) __PYX_ERR(1, 25, __pyx_L1_error)
    __Pyx_GOTREF(__pyx_t_5);
    __Pyx_DECREF(__pyx_t_4); __pyx_t_4 = 0;
    if (!(likely(PyBytes_CheckExact(__pyx_t_5))||((__pyx_t_5) == Py_None)||(PyErr_Format(PyExc_TypeError, "Expected %.16s, got %.200s", "bytes", Py_TYPE(__pyx_t_5)->tp_name), 0))) __PYX_ERR(1, 25, __pyx_L1_error)
    __pyx_r = ((PyObject*)__pyx_t_5);
    __pyx_t_5 = 0;
    goto __pyx_L0;

    /* "src/python/grpcio/grpc/_cython/_cygrpc/grpc_string.pyx.pxi":24
 *   if s is None or isinstance(s, bytes):
 *     return s
 *   elif isinstance(s, unicode):             # <<<<<<<<<<<<<<
 *     return s.encode('ascii')
 *   else:
 */
  }

  /* "src/python/grpcio/grpc/_cython/_cygrpc/grpc_string.pyx.pxi":27
 *     return s.encode('ascii')
 *   else:
 *     raise TypeError('Expected bytes, str, or unicode, not {}'.format(type(s)))             # <<<<<<<<<<<<<<
 * 
 * 
 */
  /*else*/ {
    __pyx_t_4 = __Pyx_PyObject_GetAttrStr(__pyx_kp_s_Expected_bytes_str_or_unicode_no, __pyx_n_s_format); if (unlikely(!__pyx_t_4)) __PYX_ERR(1, 27, __pyx_L1_error)
    __Pyx_GOTREF(__pyx_t_4);
    __pyx_t_6 = NULL;
    if (CYTHON_UNPACK_METHODS && likely(PyMethod_Check(__pyx_t_4))) {
      __pyx_t_6 = PyMethod_GET_SELF(__pyx_t_4);
      if (likely(__pyx_t_6)) {
        PyObject* function = PyMethod_GET_FUNCTION(__pyx_t_4);
        __Pyx_INCREF(__pyx_t_6);
        __Pyx_INCREF(function);
        __Pyx_DECREF_SET(__pyx_t_4, function);
      }
    }
    if (!__pyx_t_6) {
      __pyx_t_5 = __Pyx_PyObject_CallOneArg(__pyx_t_4, ((PyObject *)Py_TYPE(__pyx_v_s))); if (unlikely(!__pyx_t_5)) __PYX_ERR(1, 27, __pyx_L1_error)
      __Pyx_GOTREF(__pyx_t_5);
    } else {
      #if CYTHON_FAST_PYCALL
      if (PyFunction_Check(__pyx_t_4)) {
        PyObject *__pyx_temp[2] = {__pyx_t_6, ((PyObject *)Py_TYPE(__pyx_v_s))};
        __pyx_t_5 = __Pyx_PyFunction_FastCall(__pyx_t_4, __pyx_temp+1-1, 1+1); if (unlikely(!__pyx_t_5)) __PYX_ERR(1, 27, __pyx_L1_error)
        __Pyx_XDECREF(__pyx_t_6); __pyx_t_6 = 0;
        __Pyx_GOTREF(__pyx_t_5);
      } else
      #endif
      #if CYTHON_FAST_PYCCALL
      if (__Pyx_PyFastCFunction_Check(__pyx_t_4)) {
        PyObject *__pyx_temp[2] = {__pyx_t_6, ((PyObject *)Py_TYPE(__pyx_v_s))};
        __pyx_t_5 = __Pyx_PyCFunction_FastCall(__pyx_t_4, __pyx_temp+1-1, 1+1); if (unlikely(!__pyx_t_5)) __PYX_ERR(1, 27, __pyx_L1_error)
        __Pyx_XDECREF(__pyx_t_6); __pyx_t_6 = 0;
        __Pyx_GOTREF(__pyx_t_5);
      } else
      #endif
      {
        __pyx_t_7 = PyTuple_New(1+1); if (unlikely(!__pyx_t_7)) __PYX_ERR(1, 27, __pyx_L1_error)
        __Pyx_GOTREF(__pyx_t_7);
        __Pyx_GIVEREF(__pyx_t_6); PyTuple_SET_ITEM(__pyx_t_7, 0, __pyx_t_6); __pyx_t_6 = NULL;
        __Pyx_INCREF(((PyObject *)Py_TYPE(__pyx_v_s)));
        __Pyx_GIVEREF(((PyObject *)Py_TYPE(__pyx_v_s)));
        PyTuple_SET_ITEM(__pyx_t_7, 0+1, ((PyObject *)Py_TYPE(__pyx_v_s)));
        __pyx_t_5 = __Pyx_PyObject_Call(__pyx_t_4, __pyx_t_7, NULL); if (unlikely(!__pyx_t_5)) __PYX_ERR(1, 27, __pyx_L1_error)
        __Pyx_GOTREF(__pyx_t_5);
        __Pyx_DECREF(__pyx_t_7); __pyx_t_7 = 0;
      }
    }
    __Pyx_DECREF(__pyx_t_4); __pyx_t_4 = 0;
    __pyx_t_4 = __Pyx_PyObject_CallOneArg(__pyx_builtin_TypeError, __pyx_t_5); if (unlikely(!__pyx_t_4)) __PYX_ERR(1, 27, __pyx_L1_error)
    __Pyx_GOTREF(__pyx_t_4);
    __Pyx_DECREF(__pyx_t_5); __pyx_t_5 = 0;
    __Pyx_Raise(__pyx_t_4, 0, 0, 0);
    __Pyx_DECREF(__pyx_t_4); __pyx_t_4 = 0;
    __PYX_ERR(1, 27, __pyx_L1_error)
  }

  /* "src/python/grpcio/grpc/_cython/_cygrpc/grpc_string.pyx.pxi":21
 * # This function will ascii encode unicode string inputs if neccesary.
 * # In Python3, unicode strings are the default str type.
 * cdef bytes str_to_bytes(object s):             # <<<<<<<<<<<<<<
 *   if s is None or isinstance(s, bytes):
 *     return s
 */

  /* function exit code */
  __pyx_L1_error:;
  __Pyx_XDECREF(__pyx_t_4);
  __Pyx_XDECREF(__pyx_t_5);
  __Pyx_XDECREF(__pyx_t_6);
  __Pyx_XDECREF(__pyx_t_7);
  __Pyx_AddTraceback("grpc._cython.cygrpc.str_to_bytes", __pyx_clineno, __pyx_lineno, __pyx_filename);
  __pyx_r = 0;
  __pyx_L0:;
  __Pyx_XGIVEREF(__pyx_r);
  __Pyx_RefNannyFinishContext();
  return __pyx_r;
}

/* "src/python/grpcio/grpc/_cython/_cygrpc/grpc_string.pyx.pxi":35
 * # parameter rather than "object"), but would it be nice for our users? Right
 * # now we haven't yet heard from enough users to know one way or another.
 * cdef bytes _encode(object string_or_none):             # <<<<<<<<<<<<<<
 *   if string_or_none is None:
 *     return b''
 */

static PyObject *__pyx_f_4grpc_7_cython_6cygrpc__encode(PyObject *__pyx_v_string_or_none) {
  PyObject *__pyx_r = NULL;
  __Pyx_RefNannyDeclarations
  int __pyx_t_1;
  int __pyx_t_2;
  PyObject *__pyx_t_3 = NULL;
  PyObject *__pyx_t_4 = NULL;
  PyObject *__pyx_t_5 = NULL;
  PyObject *__pyx_t_6 = NULL;
  __Pyx_RefNannySetupContext("_encode", 0);

  /* "src/python/grpcio/grpc/_cython/_cygrpc/grpc_string.pyx.pxi":36
 * # now we haven't yet heard from enough users to know one way or another.
 * cdef bytes _encode(object string_or_none):
 *   if string_or_none is None:             # <<<<<<<<<<<<<<
 *     return b''
 *   elif isinstance(string_or_none, (bytes,)):
 */
  __pyx_t_1 = (__pyx_v_string_or_none == Py_None);
  __pyx_t_2 = (__pyx_t_1 != 0);
  if (__pyx_t_2) {

    /* "src/python/grpcio/grpc/_cython/_cygrpc/grpc_string.pyx.pxi":37
 * cdef bytes _encode(object string_or_none):
 *   if string_or_none is None:
 *     return b''             # <<<<<<<<<<<<<<
 *   elif isinstance(string_or_none, (bytes,)):
 *     return <bytes>string_or_none
 */
    __Pyx_XDECREF(__pyx_r);
    __Pyx_INCREF(__pyx_kp_b__2);
    __pyx_r = __pyx_kp_b__2;
    goto __pyx_L0;

    /* "src/python/grpcio/grpc/_cython/_cygrpc/grpc_string.pyx.pxi":36
 * # now we haven't yet heard from enough users to know one way or another.
 * cdef bytes _encode(object string_or_none):
 *   if string_or_none is None:             # <<<<<<<<<<<<<<
 *     return b''
 *   elif isinstance(string_or_none, (bytes,)):
 */
  }

  /* "src/python/grpcio/grpc/_cython/_cygrpc/grpc_string.pyx.pxi":38
 *   if string_or_none is None:
 *     return b''
 *   elif isinstance(string_or_none, (bytes,)):             # <<<<<<<<<<<<<<
 *     return <bytes>string_or_none
 *   elif isinstance(string_or_none, (unicode,)):
 */
  __pyx_t_2 = PyBytes_Check(__pyx_v_string_or_none); 
  __pyx_t_1 = (__pyx_t_2 != 0);
  if (__pyx_t_1) {

    /* "src/python/grpcio/grpc/_cython/_cygrpc/grpc_string.pyx.pxi":39
 *     return b''
 *   elif isinstance(string_or_none, (bytes,)):
 *     return <bytes>string_or_none             # <<<<<<<<<<<<<<
 *   elif isinstance(string_or_none, (unicode,)):
 *     return string_or_none.encode('ascii')
 */
    __Pyx_XDECREF(__pyx_r);
    __Pyx_INCREF(((PyObject*)__pyx_v_string_or_none));
    __pyx_r = ((PyObject*)__pyx_v_string_or_none);
    goto __pyx_L0;

    /* "src/python/grpcio/grpc/_cython/_cygrpc/grpc_string.pyx.pxi":38
 *   if string_or_none is None:
 *     return b''
 *   elif isinstance(string_or_none, (bytes,)):             # <<<<<<<<<<<<<<
 *     return <bytes>string_or_none
 *   elif isinstance(string_or_none, (unicode,)):
 */
  }

  /* "src/python/grpcio/grpc/_cython/_cygrpc/grpc_string.pyx.pxi":40
 *   elif isinstance(string_or_none, (bytes,)):
 *     return <bytes>string_or_none
 *   elif isinstance(string_or_none, (unicode,)):             # <<<<<<<<<<<<<<
 *     return string_or_none.encode('ascii')
 *   else:
 */
  __pyx_t_1 = PyUnicode_Check(__pyx_v_string_or_none); 
  __pyx_t_2 = (__pyx_t_1 != 0);
  if (likely(__pyx_t_2)) {

    /* "src/python/grpcio/grpc/_cython/_cygrpc/grpc_string.pyx.pxi":41
 *     return <bytes>string_or_none
 *   elif isinstance(string_or_none, (unicode,)):
 *     return string_or_none.encode('ascii')             # <<<<<<<<<<<<<<
 *   else:
 *     raise TypeError('Expected str, not {}'.format(type(string_or_none)))
 */
    __Pyx_XDECREF(__pyx_r);
    __pyx_t_3 = __Pyx_PyObject_GetAttrStr(__pyx_v_string_or_none, __pyx_n_s_encode); if (unlikely(!__pyx_t_3)) __PYX_ERR(1, 41, __pyx_L1_error)
    __Pyx_GOTREF(__pyx_t_3);
    __pyx_t_4 = __Pyx_PyObject_Call(__pyx_t_3, __pyx_tuple__3, NULL); if (unlikely(!__pyx_t_4)) __PYX_ERR(1, 41, __pyx_L1_error)
    __Pyx_GOTREF(__pyx_t_4);
    __Pyx_DECREF(__pyx_t_3); __pyx_t_3 = 0;
    if (!(likely(PyBytes_CheckExact(__pyx_t_4))||((__pyx_t_4) == Py_None)||(PyErr_Format(PyExc_TypeError, "Expected %.16s, got %.200s", "bytes", Py_TYPE(__pyx_t_4)->tp_name), 0))) __PYX_ERR(1, 41, __pyx_L1_error)
    __pyx_r = ((PyObject*)__pyx_t_4);
    __pyx_t_4 = 0;
    goto __pyx_L0;

    /* "src/python/grpcio/grpc/_cython/_cygrpc/grpc_string.pyx.pxi":40
 *   elif isinstance(string_or_none, (bytes,)):
 *     return <bytes>string_or_none
 *   elif isinstance(string_or_none, (unicode,)):             # <<<<<<<<<<<<<<
 *     return string_or_none.encode('ascii')
 *   else:
 */
  }

  /* "src/python/grpcio/grpc/_cython/_cygrpc/grpc_string.pyx.pxi":43
 *     return string_or_none.encode('ascii')
 *   else:
 *     raise TypeError('Expected str, not {}'.format(type(string_or_none)))             # <<<<<<<<<<<<<<
 * 
 * 
 */
  /*else*/ {
    __pyx_t_3 = __Pyx_PyObject_GetAttrStr(__pyx_kp_s_Expected_str_not, __pyx_n_s_format); if (unlikely(!__pyx_t_3)) __PYX_ERR(1, 43, __pyx_L1_error)
    __Pyx_GOTREF(__pyx_t_3);
    __pyx_t_5 = NULL;
    if (CYTHON_UNPACK_METHODS && likely(PyMethod_Check(__pyx_t_3))) {
      __pyx_t_5 = PyMethod_GET_SELF(__pyx_t_3);
      if (likely(__pyx_t_5)) {
        PyObject* function = PyMethod_GET_FUNCTION(__pyx_t_3);
        __Pyx_INCREF(__pyx_t_5);
        __Pyx_INCREF(function);
        __Pyx_DECREF_SET(__pyx_t_3, function);
      }
    }
    if (!__pyx_t_5) {
      __pyx_t_4 = __Pyx_PyObject_CallOneArg(__pyx_t_3, ((PyObject *)Py_TYPE(__pyx_v_string_or_none))); if (unlikely(!__pyx_t_4)) __PYX_ERR(1, 43, __pyx_L1_error)
      __Pyx_GOTREF(__pyx_t_4);
    } else {
      #if CYTHON_FAST_PYCALL
      if (PyFunction_Check(__pyx_t_3)) {
        PyObject *__pyx_temp[2] = {__pyx_t_5, ((PyObject *)Py_TYPE(__pyx_v_string_or_none))};
        __pyx_t_4 = __Pyx_PyFunction_FastCall(__pyx_t_3, __pyx_temp+1-1, 1+1); if (unlikely(!__pyx_t_4)) __PYX_ERR(1, 43, __pyx_L1_error)
        __Pyx_XDECREF(__pyx_t_5); __pyx_t_5 = 0;
        __Pyx_GOTREF(__pyx_t_4);
      } else
      #endif
      #if CYTHON_FAST_PYCCALL
      if (__Pyx_PyFastCFunction_Check(__pyx_t_3)) {
        PyObject *__pyx_temp[2] = {__pyx_t_5, ((PyObject *)Py_TYPE(__pyx_v_string_or_none))};
        __pyx_t_4 = __Pyx_PyCFunction_FastCall(__pyx_t_3, __pyx_temp+1-1, 1+1); if (unlikely(!__pyx_t_4)) __PYX_ERR(1, 43, __pyx_L1_error)
        __Pyx_XDECREF(__pyx_t_5); __pyx_t_5 = 0;
        __Pyx_GOTREF(__pyx_t_4);
      } else
      #endif
      {
        __pyx_t_6 = PyTuple_New(1+1); if (unlikely(!__pyx_t_6)) __PYX_ERR(1, 43, __pyx_L1_error)
        __Pyx_GOTREF(__pyx_t_6);
        __Pyx_GIVEREF(__pyx_t_5); PyTuple_SET_ITEM(__pyx_t_6, 0, __pyx_t_5); __pyx_t_5 = NULL;
        __Pyx_INCREF(((PyObject *)Py_TYPE(__pyx_v_string_or_none)));
        __Pyx_GIVEREF(((PyObject *)Py_TYPE(__pyx_v_string_or_none)));
        PyTuple_SET_ITEM(__pyx_t_6, 0+1, ((PyObject *)Py_TYPE(__pyx_v_string_or_none)));
        __pyx_t_4 = __Pyx_PyObject_Call(__pyx_t_3, __pyx_t_6, NULL); if (unlikely(!__pyx_t_4)) __PYX_ERR(1, 43, __pyx_L1_error)
        __Pyx_GOTREF(__pyx_t_4);
        __Pyx_DECREF(__pyx_t_6); __pyx_t_6 = 0;
      }
    }
    __Pyx_DECREF(__pyx_t_3); __pyx_t_3 = 0;
    __pyx_t_3 = __Pyx_PyObject_CallOneArg(__pyx_builtin_TypeError, __pyx_t_4); if (unlikely(!__pyx_t_3)) __PYX_ERR(1, 43, __pyx_L1_error)
    __Pyx_GOTREF(__pyx_t_3);
    __Pyx_DECREF(__pyx_t_4); __pyx_t_4 = 0;
    __Pyx_Raise(__pyx_t_3, 0, 0, 0);
    __Pyx_DECREF(__pyx_t_3); __pyx_t_3 = 0;
    __PYX_ERR(1, 43, __pyx_L1_error)
  }

  /* "src/python/grpcio/grpc/_cython/_cygrpc/grpc_string.pyx.pxi":35
 * # parameter rather than "object"), but would it be nice for our users? Right
 * # now we haven't yet heard from enough users to know one way or another.
 * cdef bytes _encode(object string_or_none):             # <<<<<<<<<<<<<<
 *   if string_or_none is None:
 *     return b''
 */

  /* function exit code */
  __pyx_L1_error:;
  __Pyx_XDECREF(__pyx_t_3);
  __Pyx_XDECREF(__pyx_t_4);
  __Pyx_XDECREF(__pyx_t_5);
  __Pyx_XDECREF(__pyx_t_6);
  __Pyx_AddTraceback("grpc._cython.cygrpc._encode", __pyx_clineno, __pyx_lineno, __pyx_filename);
  __pyx_r = 0;
  __pyx_L0:;
  __Pyx_XGIVEREF(__pyx_r);
  __Pyx_RefNannyFinishContext();
  return __pyx_r;
}

/* "src/python/grpcio/grpc/_cython/_cygrpc/grpc_string.pyx.pxi":46
 * 
 * 
 * cdef str _decode(bytes bytestring):             # <<<<<<<<<<<<<<
 *     if isinstance(bytestring, (str,)):
 *         return <str>bytestring
 */

static PyObject *__pyx_f_4grpc_7_cython_6cygrpc__decode(PyObject *__pyx_v_bytestring) {
  PyObject *__pyx_r = NULL;
  __Pyx_RefNannyDeclarations
  int __pyx_t_1;
  int __pyx_t_2;
  PyObject *__pyx_t_3 = NULL;
  PyObject *__pyx_t_4 = NULL;
  PyObject *__pyx_t_5 = NULL;
  PyObject *__pyx_t_6 = NULL;
  int __pyx_t_7;
  PyObject *__pyx_t_8 = NULL;
  PyObject *__pyx_t_9 = NULL;
  PyObject *__pyx_t_10 = NULL;
  PyObject *__pyx_t_11 = NULL;
  PyObject *__pyx_t_12 = NULL;
  PyObject *__pyx_t_13 = NULL;
  __Pyx_RefNannySetupContext("_decode", 0);

  /* "src/python/grpcio/grpc/_cython/_cygrpc/grpc_string.pyx.pxi":47
 * 
 * cdef str _decode(bytes bytestring):
 *     if isinstance(bytestring, (str,)):             # <<<<<<<<<<<<<<
 *         return <str>bytestring
 *     else:
 */
  __pyx_t_1 = PyString_Check(__pyx_v_bytestring); 
  __pyx_t_2 = (__pyx_t_1 != 0);
  if (__pyx_t_2) {

    /* "src/python/grpcio/grpc/_cython/_cygrpc/grpc_string.pyx.pxi":48
 * cdef str _decode(bytes bytestring):
 *     if isinstance(bytestring, (str,)):
 *         return <str>bytestring             # <<<<<<<<<<<<<<
 *     else:
 *         try:
 */
    __Pyx_XDECREF(__pyx_r);
    __Pyx_INCREF(((PyObject*)__pyx_v_bytestring));
    __pyx_r = ((PyObject*)__pyx_v_bytestring);
    goto __pyx_L0;

    /* "src/python/grpcio/grpc/_cython/_cygrpc/grpc_string.pyx.pxi":47
 * 
 * cdef str _decode(bytes bytestring):
 *     if isinstance(bytestring, (str,)):             # <<<<<<<<<<<<<<
 *         return <str>bytestring
 *     else:
 */
  }

  /* "src/python/grpcio/grpc/_cython/_cygrpc/grpc_string.pyx.pxi":50
 *         return <str>bytestring
 *     else:
 *         try:             # <<<<<<<<<<<<<<
 *             return bytestring.decode('utf8')
 *         except UnicodeDecodeError:
 */
  /*else*/ {
    {
      __Pyx_PyThreadState_declare
      __Pyx_PyThreadState_assign
      __Pyx_ExceptionSave(&__pyx_t_3, &__pyx_t_4, &__pyx_t_5);
      __Pyx_XGOTREF(__pyx_t_3);
      __Pyx_XGOTREF(__pyx_t_4);
      __Pyx_XGOTREF(__pyx_t_5);
      /*try:*/ {

        /* "src/python/grpcio/grpc/_cython/_cygrpc/grpc_string.pyx.pxi":51
 *     else:
 *         try:
 *             return bytestring.decode('utf8')             # <<<<<<<<<<<<<<
 *         except UnicodeDecodeError:
 *             _LOGGER.exception('Invalid encoding on %s', bytestring)
 */
        __Pyx_XDECREF(__pyx_r);
        if (unlikely(__pyx_v_bytestring == Py_None)) {
          PyErr_Format(PyExc_AttributeError, "'NoneType' object has no attribute '%.30s'", "decode");
          __PYX_ERR(1, 51, __pyx_L4_error)
        }
        __pyx_t_6 = __Pyx_decode_bytes(__pyx_v_bytestring, 0, PY_SSIZE_T_MAX, NULL, NULL, PyUnicode_DecodeUTF8); if (unlikely(!__pyx_t_6)) __PYX_ERR(1, 51, __pyx_L4_error)
        __Pyx_GOTREF(__pyx_t_6);
        if (!(likely(PyString_CheckExact(__pyx_t_6))||(PyErr_Format(PyExc_TypeError, "Expected %.16s, got %.200s", "str", Py_TYPE(__pyx_t_6)->tp_name), 0))) __PYX_ERR(1, 51, __pyx_L4_error)
        __pyx_r = ((PyObject*)__pyx_t_6);
        __pyx_t_6 = 0;
        goto __pyx_L8_try_return;

        /* "src/python/grpcio/grpc/_cython/_cygrpc/grpc_string.pyx.pxi":50
 *         return <str>bytestring
 *     else:
 *         try:             # <<<<<<<<<<<<<<
 *             return bytestring.decode('utf8')
 *         except UnicodeDecodeError:
 */
      }
      __pyx_L4_error:;
      __Pyx_XDECREF(__pyx_t_6); __pyx_t_6 = 0;

      /* "src/python/grpcio/grpc/_cython/_cygrpc/grpc_string.pyx.pxi":52
 *         try:
 *             return bytestring.decode('utf8')
 *         except UnicodeDecodeError:             # <<<<<<<<<<<<<<
 *             _LOGGER.exception('Invalid encoding on %s', bytestring)
 *             return bytestring.decode('latin1')
 */
      __pyx_t_7 = __Pyx_PyErr_ExceptionMatches(__pyx_builtin_UnicodeDecodeError);
      if (__pyx_t_7) {
        __Pyx_AddTraceback("grpc._cython.cygrpc._decode", __pyx_clineno, __pyx_lineno, __pyx_filename);
        if (__Pyx_GetException(&__pyx_t_6, &__pyx_t_8, &__pyx_t_9) < 0) __PYX_ERR(1, 52, __pyx_L6_except_error)
        __Pyx_GOTREF(__pyx_t_6);
        __Pyx_GOTREF(__pyx_t_8);
        __Pyx_GOTREF(__pyx_t_9);

        /* "src/python/grpcio/grpc/_cython/_cygrpc/grpc_string.pyx.pxi":53
 *             return bytestring.decode('utf8')
 *         except UnicodeDecodeError:
 *             _LOGGER.exception('Invalid encoding on %s', bytestring)             # <<<<<<<<<<<<<<
 *             return bytestring.decode('latin1')
 */
        __pyx_t_11 = __Pyx_GetModuleGlobalName(__pyx_n_s_LOGGER); if (unlikely(!__pyx_t_11)) __PYX_ERR(1, 53, __pyx_L6_except_error)
        __Pyx_GOTREF(__pyx_t_11);
        __pyx_t_12 = __Pyx_PyObject_GetAttrStr(__pyx_t_11, __pyx_n_s_exception); if (unlikely(!__pyx_t_12)) __PYX_ERR(1, 53, __pyx_L6_except_error)
        __Pyx_GOTREF(__pyx_t_12);
        __Pyx_DECREF(__pyx_t_11); __pyx_t_11 = 0;
        __pyx_t_11 = NULL;
        __pyx_t_7 = 0;
        if (CYTHON_UNPACK_METHODS && unlikely(PyMethod_Check(__pyx_t_12))) {
          __pyx_t_11 = PyMethod_GET_SELF(__pyx_t_12);
          if (likely(__pyx_t_11)) {
            PyObject* function = PyMethod_GET_FUNCTION(__pyx_t_12);
            __Pyx_INCREF(__pyx_t_11);
            __Pyx_INCREF(function);
            __Pyx_DECREF_SET(__pyx_t_12, function);
            __pyx_t_7 = 1;
          }
        }
        #if CYTHON_FAST_PYCALL
        if (PyFunction_Check(__pyx_t_12)) {
          PyObject *__pyx_temp[3] = {__pyx_t_11, __pyx_kp_s_Invalid_encoding_on_s, __pyx_v_bytestring};
          __pyx_t_10 = __Pyx_PyFunction_FastCall(__pyx_t_12, __pyx_temp+1-__pyx_t_7, 2+__pyx_t_7); if (unlikely(!__pyx_t_10)) __PYX_ERR(1, 53, __pyx_L6_except_error)
          __Pyx_XDECREF(__pyx_t_11); __pyx_t_11 = 0;
          __Pyx_GOTREF(__pyx_t_10);
        } else
        #endif
        #if CYTHON_FAST_PYCCALL
        if (__Pyx_PyFastCFunction_Check(__pyx_t_12)) {
          PyObject *__pyx_temp[3] = {__pyx_t_11, __pyx_kp_s_Invalid_encoding_on_s, __pyx_v_bytestring};
          __pyx_t_10 = __Pyx_PyCFunction_FastCall(__pyx_t_12, __pyx_temp+1-__pyx_t_7, 2+__pyx_t_7); if (unlikely(!__pyx_t_10)) __PYX_ERR(1, 53, __pyx_L6_except_error)
          __Pyx_XDECREF(__pyx_t_11); __pyx_t_11 = 0;
          __Pyx_GOTREF(__pyx_t_10);
        } else
        #endif
        {
          __pyx_t_13 = PyTuple_New(2+__pyx_t_7); if (unlikely(!__pyx_t_13)) __PYX_ERR(1, 53, __pyx_L6_except_error)
          __Pyx_GOTREF(__pyx_t_13);
          if (__pyx_t_11) {
            __Pyx_GIVEREF(__pyx_t_11); PyTuple_SET_ITEM(__pyx_t_13, 0, __pyx_t_11); __pyx_t_11 = NULL;
          }
          __Pyx_INCREF(__pyx_kp_s_Invalid_encoding_on_s);
          __Pyx_GIVEREF(__pyx_kp_s_Invalid_encoding_on_s);
          PyTuple_SET_ITEM(__pyx_t_13, 0+__pyx_t_7, __pyx_kp_s_Invalid_encoding_on_s);
          __Pyx_INCREF(__pyx_v_bytestring);
          __Pyx_GIVEREF(__pyx_v_bytestring);
          PyTuple_SET_ITEM(__pyx_t_13, 1+__pyx_t_7, __pyx_v_bytestring);
          __pyx_t_10 = __Pyx_PyObject_Call(__pyx_t_12, __pyx_t_13, NULL); if (unlikely(!__pyx_t_10)) __PYX_ERR(1, 53, __pyx_L6_except_error)
          __Pyx_GOTREF(__pyx_t_10);
          __Pyx_DECREF(__pyx_t_13); __pyx_t_13 = 0;
        }
        __Pyx_DECREF(__pyx_t_12); __pyx_t_12 = 0;
        __Pyx_DECREF(__pyx_t_10); __pyx_t_10 = 0;

        /* "src/python/grpcio/grpc/_cython/_cygrpc/grpc_string.pyx.pxi":54
 *         except UnicodeDecodeError:
 *             _LOGGER.exception('Invalid encoding on %s', bytestring)
 *             return bytestring.decode('latin1')             # <<<<<<<<<<<<<<
 */
        __Pyx_XDECREF(__pyx_r);
        if (unlikely(__pyx_v_bytestring == Py_None)) {
          PyErr_Format(PyExc_AttributeError, "'NoneType' object has no attribute '%.30s'", "decode");
          __PYX_ERR(1, 54, __pyx_L6_except_error)
        }
        __pyx_t_10 = __Pyx_decode_bytes(__pyx_v_bytestring, 0, PY_SSIZE_T_MAX, NULL, NULL, PyUnicode_DecodeLatin1); if (unlikely(!__pyx_t_10)) __PYX_ERR(1, 54, __pyx_L6_except_error)
        __Pyx_GOTREF(__pyx_t_10);
        if (!(likely(PyString_CheckExact(__pyx_t_10))||(PyErr_Format(PyExc_TypeError, "Expected %.16s, got %.200s", "str", Py_TYPE(__pyx_t_10)->tp_name), 0))) __PYX_ERR(1, 54, __pyx_L6_except_error)
        __pyx_r = ((PyObject*)__pyx_t_10);
        __pyx_t_10 = 0;
        __Pyx_DECREF(__pyx_t_6); __pyx_t_6 = 0;
        __Pyx_DECREF(__pyx_t_8); __pyx_t_8 = 0;
        __Pyx_DECREF(__pyx_t_9); __pyx_t_9 = 0;
        goto __pyx_L7_except_return;
      }
      goto __pyx_L6_except_error;
      __pyx_L6_except_error:;

      /* "src/python/grpcio/grpc/_cython/_cygrpc/grpc_string.pyx.pxi":50
 *         return <str>bytestring
 *     else:
 *         try:             # <<<<<<<<<<<<<<
 *             return bytestring.decode('utf8')
 *         except UnicodeDecodeError:
 */
      __Pyx_XGIVEREF(__pyx_t_3);
      __Pyx_XGIVEREF(__pyx_t_4);
      __Pyx_XGIVEREF(__pyx_t_5);
      __Pyx_ExceptionReset(__pyx_t_3, __pyx_t_4, __pyx_t_5);
      goto __pyx_L1_error;
      __pyx_L8_try_return:;
      __Pyx_XGIVEREF(__pyx_t_3);
      __Pyx_XGIVEREF(__pyx_t_4);
      __Pyx_XGIVEREF(__pyx_t_5);
      __Pyx_ExceptionReset(__pyx_t_3, __pyx_t_4, __pyx_t_5);
      goto __pyx_L0;
      __pyx_L7_except_return:;
      __Pyx_XGIVEREF(__pyx_t_3);
      __Pyx_XGIVEREF(__pyx_t_4);
      __Pyx_XGIVEREF(__pyx_t_5);
      __Pyx_ExceptionReset(__pyx_t_3, __pyx_t_4, __pyx_t_5);
      goto __pyx_L0;
    }
  }

  /* "src/python/grpcio/grpc/_cython/_cygrpc/grpc_string.pyx.pxi":46
 * 
 * 
 * cdef str _decode(bytes bytestring):             # <<<<<<<<<<<<<<
 *     if isinstance(bytestring, (str,)):
 *         return <str>bytestring
 */

  /* function exit code */
  __pyx_L1_error:;
  __Pyx_XDECREF(__pyx_t_6);
  __Pyx_XDECREF(__pyx_t_8);
  __Pyx_XDECREF(__pyx_t_9);
  __Pyx_XDECREF(__pyx_t_10);
  __Pyx_XDECREF(__pyx_t_11);
  __Pyx_XDECREF(__pyx_t_12);
  __Pyx_XDECREF(__pyx_t_13);
  __Pyx_AddTraceback("grpc._cython.cygrpc._decode", __pyx_clineno, __pyx_lineno, __pyx_filename);
  __pyx_r = 0;
  __pyx_L0:;
  __Pyx_XGIVEREF(__pyx_r);
  __Pyx_RefNannyFinishContext();
  return __pyx_r;
}

/* "(tree fragment)":1
 * def __reduce_cython__(self):             # <<<<<<<<<<<<<<
 *     raise TypeError("self.arg cannot be converted to a Python object for pickling")
 * def __setstate_cython__(self, __pyx_state):
 */

/* Python wrapper */
static PyObject *__pyx_pw_4grpc_7_cython_6cygrpc_15_GrpcArgWrapper_1__reduce_cython__(PyObject *__pyx_v_self, CYTHON_UNUSED PyObject *unused); /*proto*/
static PyObject *__pyx_pw_4grpc_7_cython_6cygrpc_15_GrpcArgWrapper_1__reduce_cython__(PyObject *__pyx_v_self, CYTHON_UNUSED PyObject *unused) {
  PyObject *__pyx_r = 0;
  __Pyx_RefNannyDeclarations
  __Pyx_RefNannySetupContext("__reduce_cython__ (wrapper)", 0);
  __pyx_r = __pyx_pf_4grpc_7_cython_6cygrpc_15_GrpcArgWrapper___reduce_cython__(((struct __pyx_obj_4grpc_7_cython_6cygrpc__GrpcArgWrapper *)__pyx_v_self));

  /* function exit code */
  __Pyx_RefNannyFinishContext();
  return __pyx_r;
}

static PyObject *__pyx_pf_4grpc_7_cython_6cygrpc_15_GrpcArgWrapper___reduce_cython__(CYTHON_UNUSED struct __pyx_obj_4grpc_7_cython_6cygrpc__GrpcArgWrapper *__pyx_v_self) {
  PyObject *__pyx_r = NULL;
  __Pyx_RefNannyDeclarations
  PyObject *__pyx_t_1 = NULL;
  __Pyx_RefNannySetupContext("__reduce_cython__", 0);

  /* "(tree fragment)":2
 * def __reduce_cython__(self):
 *     raise TypeError("self.arg cannot be converted to a Python object for pickling")             # <<<<<<<<<<<<<<
 * def __setstate_cython__(self, __pyx_state):
 *     raise TypeError("self.arg cannot be converted to a Python object for pickling")
 */
  __pyx_t_1 = __Pyx_PyObject_Call(__pyx_builtin_TypeError, __pyx_tuple__4, NULL); if (unlikely(!__pyx_t_1)) __PYX_ERR(7, 2, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_1);
  __Pyx_Raise(__pyx_t_1, 0, 0, 0);
  __Pyx_DECREF(__pyx_t_1); __pyx_t_1 = 0;
  __PYX_ERR(7, 2, __pyx_L1_error)

  /* "(tree fragment)":1
 * def __reduce_cython__(self):             # <<<<<<<<<<<<<<
 *     raise TypeError("self.arg cannot be converted to a Python object for pickling")
 * def __setstate_cython__(self, __pyx_state):
 */

  /* function exit code */
  __pyx_L1_error:;
  __Pyx_XDECREF(__pyx_t_1);
  __Pyx_AddTraceback("grpc._cython.cygrpc._GrpcArgWrapper.__reduce_cython__", __pyx_clineno, __pyx_lineno, __pyx_filename);
  __pyx_r = NULL;
  __Pyx_XGIVEREF(__pyx_r);
  __Pyx_RefNannyFinishContext();
  return __pyx_r;
}

/* "(tree fragment)":3
 * def __reduce_cython__(self):
 *     raise TypeError("self.arg cannot be converted to a Python object for pickling")
 * def __setstate_cython__(self, __pyx_state):             # <<<<<<<<<<<<<<
 *     raise TypeError("self.arg cannot be converted to a Python object for pickling")
 */

/* Python wrapper */
static PyObject *__pyx_pw_4grpc_7_cython_6cygrpc_15_GrpcArgWrapper_3__setstate_cython__(PyObject *__pyx_v_self, PyObject *__pyx_v___pyx_state); /*proto*/
static PyObject *__pyx_pw_4grpc_7_cython_6cygrpc_15_GrpcArgWrapper_3__setstate_cython__(PyObject *__pyx_v_self, PyObject *__pyx_v___pyx_state) {
  PyObject *__pyx_r = 0;
  __Pyx_RefNannyDeclarations
  __Pyx_RefNannySetupContext("__setstate_cython__ (wrapper)", 0);
  __pyx_r = __pyx_pf_4grpc_7_cython_6cygrpc_15_GrpcArgWrapper_2__setstate_cython__(((struct __pyx_obj_4grpc_7_cython_6cygrpc__GrpcArgWrapper *)__pyx_v_self), ((PyObject *)__pyx_v___pyx_state));

  /* function exit code */
  __Pyx_RefNannyFinishContext();
  return __pyx_r;
}

static PyObject *__pyx_pf_4grpc_7_cython_6cygrpc_15_GrpcArgWrapper_2__setstate_cython__(CYTHON_UNUSED struct __pyx_obj_4grpc_7_cython_6cygrpc__GrpcArgWrapper *__pyx_v_self, CYTHON_UNUSED PyObject *__pyx_v___pyx_state) {
  PyObject *__pyx_r = NULL;
  __Pyx_RefNannyDeclarations
  PyObject *__pyx_t_1 = NULL;
  __Pyx_RefNannySetupContext("__setstate_cython__", 0);

  /* "(tree fragment)":4
 *     raise TypeError("self.arg cannot be converted to a Python object for pickling")
 * def __setstate_cython__(self, __pyx_state):
 *     raise TypeError("self.arg cannot be converted to a Python object for pickling")             # <<<<<<<<<<<<<<
 */
  __pyx_t_1 = __Pyx_PyObject_Call(__pyx_builtin_TypeError, __pyx_tuple__5, NULL); if (unlikely(!__pyx_t_1)) __PYX_ERR(7, 4, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_1);
  __Pyx_Raise(__pyx_t_1, 0, 0, 0);
  __Pyx_DECREF(__pyx_t_1); __pyx_t_1 = 0;
  __PYX_ERR(7, 4, __pyx_L1_error)

  /* "(tree fragment)":3
 * def __reduce_cython__(self):
 *     raise TypeError("self.arg cannot be converted to a Python object for pickling")
 * def __setstate_cython__(self, __pyx_state):             # <<<<<<<<<<<<<<
 *     raise TypeError("self.arg cannot be converted to a Python object for pickling")
 */

  /* function exit code */
  __pyx_L1_error:;
  __Pyx_XDECREF(__pyx_t_1);
  __Pyx_AddTraceback("grpc._cython.cygrpc._GrpcArgWrapper.__setstate_cython__", __pyx_clineno, __pyx_lineno, __pyx_filename);
  __pyx_r = NULL;
  __Pyx_XGIVEREF(__pyx_r);
  __Pyx_RefNannyFinishContext();
  return __pyx_r;
}

/* "src/python/grpcio/grpc/_cython/_cygrpc/arguments.pyx.pxi":23
 * 
 * 
 * cdef tuple _wrap_grpc_arg(grpc_arg arg):             # <<<<<<<<<<<<<<
 *   wrapped = _GrpcArgWrapper()
 *   wrapped.arg = arg
 */

static PyObject *__pyx_f_4grpc_7_cython_6cygrpc__wrap_grpc_arg(grpc_arg __pyx_v_arg) {
  struct __pyx_obj_4grpc_7_cython_6cygrpc__GrpcArgWrapper *__pyx_v_wrapped = NULL;
  PyObject *__pyx_r = NULL;
  __Pyx_RefNannyDeclarations
  PyObject *__pyx_t_1 = NULL;
  __Pyx_RefNannySetupContext("_wrap_grpc_arg", 0);

  /* "src/python/grpcio/grpc/_cython/_cygrpc/arguments.pyx.pxi":24
 * 
 * cdef tuple _wrap_grpc_arg(grpc_arg arg):
 *   wrapped = _GrpcArgWrapper()             # <<<<<<<<<<<<<<
 *   wrapped.arg = arg
 *   return ("grpc.python._cygrpc._GrpcArgWrapper", wrapped)
 */
  __pyx_t_1 = __Pyx_PyObject_CallNoArg(((PyObject *)__pyx_ptype_4grpc_7_cython_6cygrpc__GrpcArgWrapper)); if (unlikely(!__pyx_t_1)) __PYX_ERR(2, 24, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_1);
  __pyx_v_wrapped = ((struct __pyx_obj_4grpc_7_cython_6cygrpc__GrpcArgWrapper *)__pyx_t_1);
  __pyx_t_1 = 0;

  /* "src/python/grpcio/grpc/_cython/_cygrpc/arguments.pyx.pxi":25
 * cdef tuple _wrap_grpc_arg(grpc_arg arg):
 *   wrapped = _GrpcArgWrapper()
 *   wrapped.arg = arg             # <<<<<<<<<<<<<<
 *   return ("grpc.python._cygrpc._GrpcArgWrapper", wrapped)
 * 
 */
  __pyx_v_wrapped->arg = __pyx_v_arg;

  /* "src/python/grpcio/grpc/_cython/_cygrpc/arguments.pyx.pxi":26
 *   wrapped = _GrpcArgWrapper()
 *   wrapped.arg = arg
 *   return ("grpc.python._cygrpc._GrpcArgWrapper", wrapped)             # <<<<<<<<<<<<<<
 * 
 * 
 */
  __Pyx_XDECREF(__pyx_r);
  __pyx_t_1 = PyTuple_New(2); if (unlikely(!__pyx_t_1)) __PYX_ERR(2, 26, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_1);
  __Pyx_INCREF(__pyx_kp_s_grpc_python__cygrpc__GrpcArgWrap);
  __Pyx_GIVEREF(__pyx_kp_s_grpc_python__cygrpc__GrpcArgWrap);
  PyTuple_SET_ITEM(__pyx_t_1, 0, __pyx_kp_s_grpc_python__cygrpc__GrpcArgWrap);
  __Pyx_INCREF(((PyObject *)__pyx_v_wrapped));
  __Pyx_GIVEREF(((PyObject *)__pyx_v_wrapped));
  PyTuple_SET_ITEM(__pyx_t_1, 1, ((PyObject *)__pyx_v_wrapped));
  __pyx_r = ((PyObject*)__pyx_t_1);
  __pyx_t_1 = 0;
  goto __pyx_L0;

  /* "src/python/grpcio/grpc/_cython/_cygrpc/arguments.pyx.pxi":23
 * 
 * 
 * cdef tuple _wrap_grpc_arg(grpc_arg arg):             # <<<<<<<<<<<<<<
 *   wrapped = _GrpcArgWrapper()
 *   wrapped.arg = arg
 */

  /* function exit code */
  __pyx_L1_error:;
  __Pyx_XDECREF(__pyx_t_1);
  __Pyx_AddTraceback("grpc._cython.cygrpc._wrap_grpc_arg", __pyx_clineno, __pyx_lineno, __pyx_filename);
  __pyx_r = 0;
  __pyx_L0:;
  __Pyx_XDECREF((PyObject *)__pyx_v_wrapped);
  __Pyx_XGIVEREF(__pyx_r);
  __Pyx_RefNannyFinishContext();
  return __pyx_r;
}

/* "src/python/grpcio/grpc/_cython/_cygrpc/arguments.pyx.pxi":29
 * 
 * 
 * cdef grpc_arg _unwrap_grpc_arg(tuple wrapped_arg):             # <<<<<<<<<<<<<<
 *   cdef _GrpcArgWrapper wrapped = wrapped_arg[1]
 *   return wrapped.arg
 */

static grpc_arg __pyx_f_4grpc_7_cython_6cygrpc__unwrap_grpc_arg(PyObject *__pyx_v_wrapped_arg) {
  struct __pyx_obj_4grpc_7_cython_6cygrpc__GrpcArgWrapper *__pyx_v_wrapped = 0;
  grpc_arg __pyx_r;
  __Pyx_RefNannyDeclarations
  PyObject *__pyx_t_1 = NULL;
  __Pyx_RefNannySetupContext("_unwrap_grpc_arg", 0);

  /* "src/python/grpcio/grpc/_cython/_cygrpc/arguments.pyx.pxi":30
 * 
 * cdef grpc_arg _unwrap_grpc_arg(tuple wrapped_arg):
 *   cdef _GrpcArgWrapper wrapped = wrapped_arg[1]             # <<<<<<<<<<<<<<
 *   return wrapped.arg
 * 
 */
  if (unlikely(__pyx_v_wrapped_arg == Py_None)) {
    PyErr_SetString(PyExc_TypeError, "'NoneType' object is not subscriptable");
    __PYX_ERR(2, 30, __pyx_L1_error)
  }
  __pyx_t_1 = __Pyx_GetItemInt_Tuple(__pyx_v_wrapped_arg, 1, long, 1, __Pyx_PyInt_From_long, 0, 0, 1); if (unlikely(!__pyx_t_1)) __PYX_ERR(2, 30, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_1);
  if (!(likely(((__pyx_t_1) == Py_None) || likely(__Pyx_TypeTest(__pyx_t_1, __pyx_ptype_4grpc_7_cython_6cygrpc__GrpcArgWrapper))))) __PYX_ERR(2, 30, __pyx_L1_error)
  __pyx_v_wrapped = ((struct __pyx_obj_4grpc_7_cython_6cygrpc__GrpcArgWrapper *)__pyx_t_1);
  __pyx_t_1 = 0;

  /* "src/python/grpcio/grpc/_cython/_cygrpc/arguments.pyx.pxi":31
 * cdef grpc_arg _unwrap_grpc_arg(tuple wrapped_arg):
 *   cdef _GrpcArgWrapper wrapped = wrapped_arg[1]
 *   return wrapped.arg             # <<<<<<<<<<<<<<
 * 
 * 
 */
  __pyx_r = __pyx_v_wrapped->arg;
  goto __pyx_L0;

  /* "src/python/grpcio/grpc/_cython/_cygrpc/arguments.pyx.pxi":29
 * 
 * 
 * cdef grpc_arg _unwrap_grpc_arg(tuple wrapped_arg):             # <<<<<<<<<<<<<<
 *   cdef _GrpcArgWrapper wrapped = wrapped_arg[1]
 *   return wrapped.arg
 */

  /* function exit code */
  __pyx_L1_error:;
  __Pyx_XDECREF(__pyx_t_1);
  __Pyx_WriteUnraisable("grpc._cython.cygrpc._unwrap_grpc_arg", __pyx_clineno, __pyx_lineno, __pyx_filename, 1, 0);
  __Pyx_pretend_to_initialize(&__pyx_r);
  __pyx_L0:;
  __Pyx_XDECREF((PyObject *)__pyx_v_wrapped);
  __Pyx_RefNannyFinishContext();
  return __pyx_r;
}

/* "src/python/grpcio/grpc/_cython/_cygrpc/arguments.pyx.pxi":36
 * cdef class _ChannelArg:
 * 
 *   cdef void c(self, argument, _VTable vtable, references) except *:             # <<<<<<<<<<<<<<
 *     key, value = argument
 *     cdef bytes encoded_key = _encode(key)
 */

static void __pyx_f_4grpc_7_cython_6cygrpc_11_ChannelArg_c(struct __pyx_obj_4grpc_7_cython_6cygrpc__ChannelArg *__pyx_v_self, PyObject *__pyx_v_argument, struct __pyx_obj_4grpc_7_cython_6cygrpc__VTable *__pyx_v_vtable, PyObject *__pyx_v_references) {
  PyObject *__pyx_v_key = NULL;
  PyObject *__pyx_v_value = NULL;
  PyObject *__pyx_v_encoded_key = 0;
  PyObject *__pyx_v_encoded_value = NULL;
  __Pyx_RefNannyDeclarations
  PyObject *__pyx_t_1 = NULL;
  PyObject *__pyx_t_2 = NULL;
  PyObject *__pyx_t_3 = NULL;
  PyObject *(*__pyx_t_4)(PyObject *);
  int __pyx_t_5;
  int __pyx_t_6;
  int __pyx_t_7;
  char *__pyx_t_8;
  int __pyx_t_9;
  int __pyx_t_10;
  grpc_arg __pyx_t_11;
  __pyx_t_4grpc_7_cython_6cygrpc_intptr_t __pyx_t_12;
  PyObject *__pyx_t_13 = NULL;
  __Pyx_RefNannySetupContext("c", 0);

  /* "src/python/grpcio/grpc/_cython/_cygrpc/arguments.pyx.pxi":37
 * 
 *   cdef void c(self, argument, _VTable vtable, references) except *:
 *     key, value = argument             # <<<<<<<<<<<<<<
 *     cdef bytes encoded_key = _encode(key)
 *     if encoded_key is not key:
 */
  if ((likely(PyTuple_CheckExact(__pyx_v_argument))) || (PyList_CheckExact(__pyx_v_argument))) {
    PyObject* sequence = __pyx_v_argument;
    Py_ssize_t size = __Pyx_PySequence_SIZE(sequence);
    if (unlikely(size != 2)) {
      if (size > 2) __Pyx_RaiseTooManyValuesError(2);
      else if (size >= 0) __Pyx_RaiseNeedMoreValuesError(size);
      __PYX_ERR(2, 37, __pyx_L1_error)
    }
    #if CYTHON_ASSUME_SAFE_MACROS && !CYTHON_AVOID_BORROWED_REFS
    if (likely(PyTuple_CheckExact(sequence))) {
      __pyx_t_1 = PyTuple_GET_ITEM(sequence, 0); 
      __pyx_t_2 = PyTuple_GET_ITEM(sequence, 1); 
    } else {
      __pyx_t_1 = PyList_GET_ITEM(sequence, 0); 
      __pyx_t_2 = PyList_GET_ITEM(sequence, 1); 
    }
    __Pyx_INCREF(__pyx_t_1);
    __Pyx_INCREF(__pyx_t_2);
    #else
    __pyx_t_1 = PySequence_ITEM(sequence, 0); if (unlikely(!__pyx_t_1)) __PYX_ERR(2, 37, __pyx_L1_error)
    __Pyx_GOTREF(__pyx_t_1);
    __pyx_t_2 = PySequence_ITEM(sequence, 1); if (unlikely(!__pyx_t_2)) __PYX_ERR(2, 37, __pyx_L1_error)
    __Pyx_GOTREF(__pyx_t_2);
    #endif
  } else {
    Py_ssize_t index = -1;
    __pyx_t_3 = PyObject_GetIter(__pyx_v_argument); if (unlikely(!__pyx_t_3)) __PYX_ERR(2, 37, __pyx_L1_error)
    __Pyx_GOTREF(__pyx_t_3);
    __pyx_t_4 = Py_TYPE(__pyx_t_3)->tp_iternext;
    index = 0; __pyx_t_1 = __pyx_t_4(__pyx_t_3); if (unlikely(!__pyx_t_1)) goto __pyx_L3_unpacking_failed;
    __Pyx_GOTREF(__pyx_t_1);
    index = 1; __pyx_t_2 = __pyx_t_4(__pyx_t_3); if (unlikely(!__pyx_t_2)) goto __pyx_L3_unpacking_failed;
    __Pyx_GOTREF(__pyx_t_2);
    if (__Pyx_IternextUnpackEndCheck(__pyx_t_4(__pyx_t_3), 2) < 0) __PYX_ERR(2, 37, __pyx_L1_error)
    __pyx_t_4 = NULL;
    __Pyx_DECREF(__pyx_t_3); __pyx_t_3 = 0;
    goto __pyx_L4_unpacking_done;
    __pyx_L3_unpacking_failed:;
    __Pyx_DECREF(__pyx_t_3); __pyx_t_3 = 0;
    __pyx_t_4 = NULL;
    if (__Pyx_IterFinish() == 0) __Pyx_RaiseNeedMoreValuesError(index);
    __PYX_ERR(2, 37, __pyx_L1_error)
    __pyx_L4_unpacking_done:;
  }
  __pyx_v_key = __pyx_t_1;
  __pyx_t_1 = 0;
  __pyx_v_value = __pyx_t_2;
  __pyx_t_2 = 0;

  /* "src/python/grpcio/grpc/_cython/_cygrpc/arguments.pyx.pxi":38
 *   cdef void c(self, argument, _VTable vtable, references) except *:
 *     key, value = argument
 *     cdef bytes encoded_key = _encode(key)             # <<<<<<<<<<<<<<
 *     if encoded_key is not key:
 *       references.append(encoded_key)
 */
  __pyx_t_2 = __pyx_f_4grpc_7_cython_6cygrpc__encode(__pyx_v_key); if (unlikely(!__pyx_t_2)) __PYX_ERR(2, 38, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_2);
  __pyx_v_encoded_key = ((PyObject*)__pyx_t_2);
  __pyx_t_2 = 0;

  /* "src/python/grpcio/grpc/_cython/_cygrpc/arguments.pyx.pxi":39
 *     key, value = argument
 *     cdef bytes encoded_key = _encode(key)
 *     if encoded_key is not key:             # <<<<<<<<<<<<<<
 *       references.append(encoded_key)
 *     self.c_argument.key = encoded_key
 */
  __pyx_t_5 = (__pyx_v_encoded_key != ((PyObject*)__pyx_v_key));
  __pyx_t_6 = (__pyx_t_5 != 0);
  if (__pyx_t_6) {

    /* "src/python/grpcio/grpc/_cython/_cygrpc/arguments.pyx.pxi":40
 *     cdef bytes encoded_key = _encode(key)
 *     if encoded_key is not key:
 *       references.append(encoded_key)             # <<<<<<<<<<<<<<
 *     self.c_argument.key = encoded_key
 *     if isinstance(value, int):
 */
    __pyx_t_7 = __Pyx_PyObject_Append(__pyx_v_references, __pyx_v_encoded_key); if (unlikely(__pyx_t_7 == ((int)-1))) __PYX_ERR(2, 40, __pyx_L1_error)

    /* "src/python/grpcio/grpc/_cython/_cygrpc/arguments.pyx.pxi":39
 *     key, value = argument
 *     cdef bytes encoded_key = _encode(key)
 *     if encoded_key is not key:             # <<<<<<<<<<<<<<
 *       references.append(encoded_key)
 *     self.c_argument.key = encoded_key
 */
  }

  /* "src/python/grpcio/grpc/_cython/_cygrpc/arguments.pyx.pxi":41
 *     if encoded_key is not key:
 *       references.append(encoded_key)
 *     self.c_argument.key = encoded_key             # <<<<<<<<<<<<<<
 *     if isinstance(value, int):
 *       self.c_argument.type = GRPC_ARG_INTEGER
 */
  if (unlikely(__pyx_v_encoded_key == Py_None)) {
    PyErr_SetString(PyExc_TypeError, "expected bytes, NoneType found");
    __PYX_ERR(2, 41, __pyx_L1_error)
  }
  __pyx_t_8 = __Pyx_PyBytes_AsWritableString(__pyx_v_encoded_key); if (unlikely((!__pyx_t_8) && PyErr_Occurred())) __PYX_ERR(2, 41, __pyx_L1_error)
  __pyx_v_self->c_argument.key = __pyx_t_8;

  /* "src/python/grpcio/grpc/_cython/_cygrpc/arguments.pyx.pxi":42
 *       references.append(encoded_key)
 *     self.c_argument.key = encoded_key
 *     if isinstance(value, int):             # <<<<<<<<<<<<<<
 *       self.c_argument.type = GRPC_ARG_INTEGER
 *       self.c_argument.value.integer = value
 */
  __pyx_t_6 = PyInt_Check(__pyx_v_value); 
  __pyx_t_5 = (__pyx_t_6 != 0);
  if (__pyx_t_5) {

    /* "src/python/grpcio/grpc/_cython/_cygrpc/arguments.pyx.pxi":43
 *     self.c_argument.key = encoded_key
 *     if isinstance(value, int):
 *       self.c_argument.type = GRPC_ARG_INTEGER             # <<<<<<<<<<<<<<
 *       self.c_argument.value.integer = value
 *     elif isinstance(value, (bytes, str, unicode,)):
 */
    __pyx_v_self->c_argument.type = GRPC_ARG_INTEGER;

    /* "src/python/grpcio/grpc/_cython/_cygrpc/arguments.pyx.pxi":44
 *     if isinstance(value, int):
 *       self.c_argument.type = GRPC_ARG_INTEGER
 *       self.c_argument.value.integer = value             # <<<<<<<<<<<<<<
 *     elif isinstance(value, (bytes, str, unicode,)):
 *       self.c_argument.type = GRPC_ARG_STRING
 */
    __pyx_t_9 = __Pyx_PyInt_As_int(__pyx_v_value); if (unlikely((__pyx_t_9 == (int)-1) && PyErr_Occurred())) __PYX_ERR(2, 44, __pyx_L1_error)
    __pyx_v_self->c_argument.value.integer = __pyx_t_9;

    /* "src/python/grpcio/grpc/_cython/_cygrpc/arguments.pyx.pxi":42
 *       references.append(encoded_key)
 *     self.c_argument.key = encoded_key
 *     if isinstance(value, int):             # <<<<<<<<<<<<<<
 *       self.c_argument.type = GRPC_ARG_INTEGER
 *       self.c_argument.value.integer = value
 */
    goto __pyx_L6;
  }

  /* "src/python/grpcio/grpc/_cython/_cygrpc/arguments.pyx.pxi":45
 *       self.c_argument.type = GRPC_ARG_INTEGER
 *       self.c_argument.value.integer = value
 *     elif isinstance(value, (bytes, str, unicode,)):             # <<<<<<<<<<<<<<
 *       self.c_argument.type = GRPC_ARG_STRING
 *       encoded_value = _encode(value)
 */
  __pyx_t_6 = PyBytes_Check(__pyx_v_value); 
  __pyx_t_10 = (__pyx_t_6 != 0);
  if (!__pyx_t_10) {
  } else {
    __pyx_t_5 = __pyx_t_10;
    goto __pyx_L7_bool_binop_done;
  }
  __pyx_t_10 = PyString_Check(__pyx_v_value); 
  __pyx_t_6 = (__pyx_t_10 != 0);
  if (!__pyx_t_6) {
  } else {
    __pyx_t_5 = __pyx_t_6;
    goto __pyx_L7_bool_binop_done;
  }
  __pyx_t_6 = PyUnicode_Check(__pyx_v_value); 
  __pyx_t_10 = (__pyx_t_6 != 0);
  __pyx_t_5 = __pyx_t_10;
  __pyx_L7_bool_binop_done:;
  __pyx_t_10 = (__pyx_t_5 != 0);
  if (__pyx_t_10) {

    /* "src/python/grpcio/grpc/_cython/_cygrpc/arguments.pyx.pxi":46
 *       self.c_argument.value.integer = value
 *     elif isinstance(value, (bytes, str, unicode,)):
 *       self.c_argument.type = GRPC_ARG_STRING             # <<<<<<<<<<<<<<
 *       encoded_value = _encode(value)
 *       if encoded_value is not value:
 */
    __pyx_v_self->c_argument.type = GRPC_ARG_STRING;

    /* "src/python/grpcio/grpc/_cython/_cygrpc/arguments.pyx.pxi":47
 *     elif isinstance(value, (bytes, str, unicode,)):
 *       self.c_argument.type = GRPC_ARG_STRING
 *       encoded_value = _encode(value)             # <<<<<<<<<<<<<<
 *       if encoded_value is not value:
 *         references.append(encoded_value)
 */
    __pyx_t_2 = __pyx_f_4grpc_7_cython_6cygrpc__encode(__pyx_v_value); if (unlikely(!__pyx_t_2)) __PYX_ERR(2, 47, __pyx_L1_error)
    __Pyx_GOTREF(__pyx_t_2);
    __pyx_v_encoded_value = ((PyObject*)__pyx_t_2);
    __pyx_t_2 = 0;

    /* "src/python/grpcio/grpc/_cython/_cygrpc/arguments.pyx.pxi":48
 *       self.c_argument.type = GRPC_ARG_STRING
 *       encoded_value = _encode(value)
 *       if encoded_value is not value:             # <<<<<<<<<<<<<<
 *         references.append(encoded_value)
 *       self.c_argument.value.string = encoded_value
 */
    __pyx_t_10 = (__pyx_v_encoded_value != ((PyObject*)__pyx_v_value));
    __pyx_t_5 = (__pyx_t_10 != 0);
    if (__pyx_t_5) {

      /* "src/python/grpcio/grpc/_cython/_cygrpc/arguments.pyx.pxi":49
 *       encoded_value = _encode(value)
 *       if encoded_value is not value:
 *         references.append(encoded_value)             # <<<<<<<<<<<<<<
 *       self.c_argument.value.string = encoded_value
 *     elif isinstance(value, _GrpcArgWrapper):
 */
      __pyx_t_7 = __Pyx_PyObject_Append(__pyx_v_references, __pyx_v_encoded_value); if (unlikely(__pyx_t_7 == ((int)-1))) __PYX_ERR(2, 49, __pyx_L1_error)

      /* "src/python/grpcio/grpc/_cython/_cygrpc/arguments.pyx.pxi":48
 *       self.c_argument.type = GRPC_ARG_STRING
 *       encoded_value = _encode(value)
 *       if encoded_value is not value:             # <<<<<<<<<<<<<<
 *         references.append(encoded_value)
 *       self.c_argument.value.string = encoded_value
 */
    }

    /* "src/python/grpcio/grpc/_cython/_cygrpc/arguments.pyx.pxi":50
 *       if encoded_value is not value:
 *         references.append(encoded_value)
 *       self.c_argument.value.string = encoded_value             # <<<<<<<<<<<<<<
 *     elif isinstance(value, _GrpcArgWrapper):
 *       self.c_argument = (<_GrpcArgWrapper>value).arg
 */
    if (unlikely(__pyx_v_encoded_value == Py_None)) {
      PyErr_SetString(PyExc_TypeError, "expected bytes, NoneType found");
      __PYX_ERR(2, 50, __pyx_L1_error)
    }
    __pyx_t_8 = __Pyx_PyBytes_AsWritableString(__pyx_v_encoded_value); if (unlikely((!__pyx_t_8) && PyErr_Occurred())) __PYX_ERR(2, 50, __pyx_L1_error)
    __pyx_v_self->c_argument.value.string = __pyx_t_8;

    /* "src/python/grpcio/grpc/_cython/_cygrpc/arguments.pyx.pxi":45
 *       self.c_argument.type = GRPC_ARG_INTEGER
 *       self.c_argument.value.integer = value
 *     elif isinstance(value, (bytes, str, unicode,)):             # <<<<<<<<<<<<<<
 *       self.c_argument.type = GRPC_ARG_STRING
 *       encoded_value = _encode(value)
 */
    goto __pyx_L6;
  }

  /* "src/python/grpcio/grpc/_cython/_cygrpc/arguments.pyx.pxi":51
 *         references.append(encoded_value)
 *       self.c_argument.value.string = encoded_value
 *     elif isinstance(value, _GrpcArgWrapper):             # <<<<<<<<<<<<<<
 *       self.c_argument = (<_GrpcArgWrapper>value).arg
 *     elif hasattr(value, '__int__'):
 */
  __pyx_t_5 = __Pyx_TypeCheck(__pyx_v_value, __pyx_ptype_4grpc_7_cython_6cygrpc__GrpcArgWrapper); 
  __pyx_t_10 = (__pyx_t_5 != 0);
  if (__pyx_t_10) {

    /* "src/python/grpcio/grpc/_cython/_cygrpc/arguments.pyx.pxi":52
 *       self.c_argument.value.string = encoded_value
 *     elif isinstance(value, _GrpcArgWrapper):
 *       self.c_argument = (<_GrpcArgWrapper>value).arg             # <<<<<<<<<<<<<<
 *     elif hasattr(value, '__int__'):
 *       # Pointer objects must override __int__() to return
 */
    __pyx_t_11 = ((struct __pyx_obj_4grpc_7_cython_6cygrpc__GrpcArgWrapper *)__pyx_v_value)->arg;
    __pyx_v_self->c_argument = __pyx_t_11;

    /* "src/python/grpcio/grpc/_cython/_cygrpc/arguments.pyx.pxi":51
 *         references.append(encoded_value)
 *       self.c_argument.value.string = encoded_value
 *     elif isinstance(value, _GrpcArgWrapper):             # <<<<<<<<<<<<<<
 *       self.c_argument = (<_GrpcArgWrapper>value).arg
 *     elif hasattr(value, '__int__'):
 */
    goto __pyx_L6;
  }

  /* "src/python/grpcio/grpc/_cython/_cygrpc/arguments.pyx.pxi":53
 *     elif isinstance(value, _GrpcArgWrapper):
 *       self.c_argument = (<_GrpcArgWrapper>value).arg
 *     elif hasattr(value, '__int__'):             # <<<<<<<<<<<<<<
 *       # Pointer objects must override __int__() to return
 *       # the underlying C address (Python ints are word size). The
 */
  __pyx_t_10 = __Pyx_HasAttr(__pyx_v_value, __pyx_n_s_int); if (unlikely(__pyx_t_10 == ((int)-1))) __PYX_ERR(2, 53, __pyx_L1_error)
  __pyx_t_5 = (__pyx_t_10 != 0);
  if (likely(__pyx_t_5)) {

    /* "src/python/grpcio/grpc/_cython/_cygrpc/arguments.pyx.pxi":58
 *       # lifecycle of the pointer is fixed to the lifecycle of the
 *       # python object wrapping it.
 *       self.c_argument.type = GRPC_ARG_POINTER             # <<<<<<<<<<<<<<
 *       self.c_argument.value.pointer.vtable = &vtable.c_vtable
 *       self.c_argument.value.pointer.address = <void*>(<intptr_t>int(value))
 */
    __pyx_v_self->c_argument.type = GRPC_ARG_POINTER;

    /* "src/python/grpcio/grpc/_cython/_cygrpc/arguments.pyx.pxi":59
 *       # python object wrapping it.
 *       self.c_argument.type = GRPC_ARG_POINTER
 *       self.c_argument.value.pointer.vtable = &vtable.c_vtable             # <<<<<<<<<<<<<<
 *       self.c_argument.value.pointer.address = <void*>(<intptr_t>int(value))
 *     else:
 */
    __pyx_v_self->c_argument.value.pointer.vtable = (&__pyx_v_vtable->c_vtable);

    /* "src/python/grpcio/grpc/_cython/_cygrpc/arguments.pyx.pxi":60
 *       self.c_argument.type = GRPC_ARG_POINTER
 *       self.c_argument.value.pointer.vtable = &vtable.c_vtable
 *       self.c_argument.value.pointer.address = <void*>(<intptr_t>int(value))             # <<<<<<<<<<<<<<
 *     else:
 *       raise TypeError(
 */
    __pyx_t_2 = __Pyx_PyNumber_Int(__pyx_v_value); if (unlikely(!__pyx_t_2)) __PYX_ERR(2, 60, __pyx_L1_error)
    __Pyx_GOTREF(__pyx_t_2);
    __pyx_t_12 = PyInt_AsSsize_t(__pyx_t_2); if (unlikely((__pyx_t_12 == (Py_ssize_t)-1) && PyErr_Occurred())) __PYX_ERR(2, 60, __pyx_L1_error)
    __Pyx_DECREF(__pyx_t_2); __pyx_t_2 = 0;
    __pyx_v_self->c_argument.value.pointer.p = ((void *)((__pyx_t_4grpc_7_cython_6cygrpc_intptr_t)__pyx_t_12));

    /* "src/python/grpcio/grpc/_cython/_cygrpc/arguments.pyx.pxi":53
 *     elif isinstance(value, _GrpcArgWrapper):
 *       self.c_argument = (<_GrpcArgWrapper>value).arg
 *     elif hasattr(value, '__int__'):             # <<<<<<<<<<<<<<
 *       # Pointer objects must override __int__() to return
 *       # the underlying C address (Python ints are word size). The
 */
    goto __pyx_L6;
  }

  /* "src/python/grpcio/grpc/_cython/_cygrpc/arguments.pyx.pxi":62
 *       self.c_argument.value.pointer.address = <void*>(<intptr_t>int(value))
 *     else:
 *       raise TypeError(             # <<<<<<<<<<<<<<
 *           'Expected int, bytes, or behavior, got {}'.format(type(value)))
 * 
 */
  /*else*/ {

    /* "src/python/grpcio/grpc/_cython/_cygrpc/arguments.pyx.pxi":63
 *     else:
 *       raise TypeError(
 *           'Expected int, bytes, or behavior, got {}'.format(type(value)))             # <<<<<<<<<<<<<<
 * 
 * 
 */
    __pyx_t_1 = __Pyx_PyObject_GetAttrStr(__pyx_kp_s_Expected_int_bytes_or_behavior_g, __pyx_n_s_format); if (unlikely(!__pyx_t_1)) __PYX_ERR(2, 63, __pyx_L1_error)
    __Pyx_GOTREF(__pyx_t_1);
    __pyx_t_3 = NULL;
    if (CYTHON_UNPACK_METHODS && likely(PyMethod_Check(__pyx_t_1))) {
      __pyx_t_3 = PyMethod_GET_SELF(__pyx_t_1);
      if (likely(__pyx_t_3)) {
        PyObject* function = PyMethod_GET_FUNCTION(__pyx_t_1);
        __Pyx_INCREF(__pyx_t_3);
        __Pyx_INCREF(function);
        __Pyx_DECREF_SET(__pyx_t_1, function);
      }
    }
    if (!__pyx_t_3) {
      __pyx_t_2 = __Pyx_PyObject_CallOneArg(__pyx_t_1, ((PyObject *)Py_TYPE(__pyx_v_value))); if (unlikely(!__pyx_t_2)) __PYX_ERR(2, 63, __pyx_L1_error)
      __Pyx_GOTREF(__pyx_t_2);
    } else {
      #if CYTHON_FAST_PYCALL
      if (PyFunction_Check(__pyx_t_1)) {
        PyObject *__pyx_temp[2] = {__pyx_t_3, ((PyObject *)Py_TYPE(__pyx_v_value))};
        __pyx_t_2 = __Pyx_PyFunction_FastCall(__pyx_t_1, __pyx_temp+1-1, 1+1); if (unlikely(!__pyx_t_2)) __PYX_ERR(2, 63, __pyx_L1_error)
        __Pyx_XDECREF(__pyx_t_3); __pyx_t_3 = 0;
        __Pyx_GOTREF(__pyx_t_2);
      } else
      #endif
      #if CYTHON_FAST_PYCCALL
      if (__Pyx_PyFastCFunction_Check(__pyx_t_1)) {
        PyObject *__pyx_temp[2] = {__pyx_t_3, ((PyObject *)Py_TYPE(__pyx_v_value))};
        __pyx_t_2 = __Pyx_PyCFunction_FastCall(__pyx_t_1, __pyx_temp+1-1, 1+1); if (unlikely(!__pyx_t_2)) __PYX_ERR(2, 63, __pyx_L1_error)
        __Pyx_XDECREF(__pyx_t_3); __pyx_t_3 = 0;
        __Pyx_GOTREF(__pyx_t_2);
      } else
      #endif
      {
        __pyx_t_13 = PyTuple_New(1+1); if (unlikely(!__pyx_t_13)) __PYX_ERR(2, 63, __pyx_L1_error)
        __Pyx_GOTREF(__pyx_t_13);
        __Pyx_GIVEREF(__pyx_t_3); PyTuple_SET_ITEM(__pyx_t_13, 0, __pyx_t_3); __pyx_t_3 = NULL;
        __Pyx_INCREF(((PyObject *)Py_TYPE(__pyx_v_value)));
        __Pyx_GIVEREF(((PyObject *)Py_TYPE(__pyx_v_value)));
        PyTuple_SET_ITEM(__pyx_t_13, 0+1, ((PyObject *)Py_TYPE(__pyx_v_value)));
        __pyx_t_2 = __Pyx_PyObject_Call(__pyx_t_1, __pyx_t_13, NULL); if (unlikely(!__pyx_t_2)) __PYX_ERR(2, 63, __pyx_L1_error)
        __Pyx_GOTREF(__pyx_t_2);
        __Pyx_DECREF(__pyx_t_13); __pyx_t_13 = 0;
      }
    }
    __Pyx_DECREF(__pyx_t_1); __pyx_t_1 = 0;

    /* "src/python/grpcio/grpc/_cython/_cygrpc/arguments.pyx.pxi":62
 *       self.c_argument.value.pointer.address = <void*>(<intptr_t>int(value))
 *     else:
 *       raise TypeError(             # <<<<<<<<<<<<<<
 *           'Expected int, bytes, or behavior, got {}'.format(type(value)))
 * 
 */
    __pyx_t_1 = __Pyx_PyObject_CallOneArg(__pyx_builtin_TypeError, __pyx_t_2); if (unlikely(!__pyx_t_1)) __PYX_ERR(2, 62, __pyx_L1_error)
    __Pyx_GOTREF(__pyx_t_1);
    __Pyx_DECREF(__pyx_t_2); __pyx_t_2 = 0;
    __Pyx_Raise(__pyx_t_1, 0, 0, 0);
    __Pyx_DECREF(__pyx_t_1); __pyx_t_1 = 0;
    __PYX_ERR(2, 62, __pyx_L1_error)
  }
  __pyx_L6:;

  /* "src/python/grpcio/grpc/_cython/_cygrpc/arguments.pyx.pxi":36
 * cdef class _ChannelArg:
 * 
 *   cdef void c(self, argument, _VTable vtable, references) except *:             # <<<<<<<<<<<<<<
 *     key, value = argument
 *     cdef bytes encoded_key = _encode(key)
 */

  /* function exit code */
  goto __pyx_L0;
  __pyx_L1_error:;
  __Pyx_XDECREF(__pyx_t_1);
  __Pyx_XDECREF(__pyx_t_2);
  __Pyx_XDECREF(__pyx_t_3);
  __Pyx_XDECREF(__pyx_t_13);
  __Pyx_AddTraceback("grpc._cython.cygrpc._ChannelArg.c", __pyx_clineno, __pyx_lineno, __pyx_filename);
  __pyx_L0:;
  __Pyx_XDECREF(__pyx_v_key);
  __Pyx_XDECREF(__pyx_v_value);
  __Pyx_XDECREF(__pyx_v_encoded_key);
  __Pyx_XDECREF(__pyx_v_encoded_value);
  __Pyx_RefNannyFinishContext();
}

/* "(tree fragment)":1
 * def __reduce_cython__(self):             # <<<<<<<<<<<<<<
 *     raise TypeError("self.c_argument cannot be converted to a Python object for pickling")
 * def __setstate_cython__(self, __pyx_state):
 */

/* Python wrapper */
static PyObject *__pyx_pw_4grpc_7_cython_6cygrpc_11_ChannelArg_1__reduce_cython__(PyObject *__pyx_v_self, CYTHON_UNUSED PyObject *unused); /*proto*/
static PyObject *__pyx_pw_4grpc_7_cython_6cygrpc_11_ChannelArg_1__reduce_cython__(PyObject *__pyx_v_self, CYTHON_UNUSED PyObject *unused) {
  PyObject *__pyx_r = 0;
  __Pyx_RefNannyDeclarations
  __Pyx_RefNannySetupContext("__reduce_cython__ (wrapper)", 0);
  __pyx_r = __pyx_pf_4grpc_7_cython_6cygrpc_11_ChannelArg___reduce_cython__(((struct __pyx_obj_4grpc_7_cython_6cygrpc__ChannelArg *)__pyx_v_self));

  /* function exit code */
  __Pyx_RefNannyFinishContext();
  return __pyx_r;
}

static PyObject *__pyx_pf_4grpc_7_cython_6cygrpc_11_ChannelArg___reduce_cython__(CYTHON_UNUSED struct __pyx_obj_4grpc_7_cython_6cygrpc__ChannelArg *__pyx_v_self) {
  PyObject *__pyx_r = NULL;
  __Pyx_RefNannyDeclarations
  PyObject *__pyx_t_1 = NULL;
  __Pyx_RefNannySetupContext("__reduce_cython__", 0);

  /* "(tree fragment)":2
 * def __reduce_cython__(self):
 *     raise TypeError("self.c_argument cannot be converted to a Python object for pickling")             # <<<<<<<<<<<<<<
 * def __setstate_cython__(self, __pyx_state):
 *     raise TypeError("self.c_argument cannot be converted to a Python object for pickling")
 */
  __pyx_t_1 = __Pyx_PyObject_Call(__pyx_builtin_TypeError, __pyx_tuple__6, NULL); if (unlikely(!__pyx_t_1)) __PYX_ERR(7, 2, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_1);
  __Pyx_Raise(__pyx_t_1, 0, 0, 0);
  __Pyx_DECREF(__pyx_t_1); __pyx_t_1 = 0;
  __PYX_ERR(7, 2, __pyx_L1_error)

  /* "(tree fragment)":1
 * def __reduce_cython__(self):             # <<<<<<<<<<<<<<
 *     raise TypeError("self.c_argument cannot be converted to a Python object for pickling")
 * def __setstate_cython__(self, __pyx_state):
 */

  /* function exit code */
  __pyx_L1_error:;
  __Pyx_XDECREF(__pyx_t_1);
  __Pyx_AddTraceback("grpc._cython.cygrpc._ChannelArg.__reduce_cython__", __pyx_clineno, __pyx_lineno, __pyx_filename);
  __pyx_r = NULL;
  __Pyx_XGIVEREF(__pyx_r);
  __Pyx_RefNannyFinishContext();
  return __pyx_r;
}

/* "(tree fragment)":3
 * def __reduce_cython__(self):
 *     raise TypeError("self.c_argument cannot be converted to a Python object for pickling")
 * def __setstate_cython__(self, __pyx_state):             # <<<<<<<<<<<<<<
 *     raise TypeError("self.c_argument cannot be converted to a Python object for pickling")
 */

/* Python wrapper */
static PyObject *__pyx_pw_4grpc_7_cython_6cygrpc_11_ChannelArg_3__setstate_cython__(PyObject *__pyx_v_self, PyObject *__pyx_v___pyx_state); /*proto*/
static PyObject *__pyx_pw_4grpc_7_cython_6cygrpc_11_ChannelArg_3__setstate_cython__(PyObject *__pyx_v_self, PyObject *__pyx_v___pyx_state) {
  PyObject *__pyx_r = 0;
  __Pyx_RefNannyDeclarations
  __Pyx_RefNannySetupContext("__setstate_cython__ (wrapper)", 0);
  __pyx_r = __pyx_pf_4grpc_7_cython_6cygrpc_11_ChannelArg_2__setstate_cython__(((struct __pyx_obj_4grpc_7_cython_6cygrpc__ChannelArg *)__pyx_v_self), ((PyObject *)__pyx_v___pyx_state));

  /* function exit code */
  __Pyx_RefNannyFinishContext();
  return __pyx_r;
}

static PyObject *__pyx_pf_4grpc_7_cython_6cygrpc_11_ChannelArg_2__setstate_cython__(CYTHON_UNUSED struct __pyx_obj_4grpc_7_cython_6cygrpc__ChannelArg *__pyx_v_self, CYTHON_UNUSED PyObject *__pyx_v___pyx_state) {
  PyObject *__pyx_r = NULL;
  __Pyx_RefNannyDeclarations
  PyObject *__pyx_t_1 = NULL;
  __Pyx_RefNannySetupContext("__setstate_cython__", 0);

  /* "(tree fragment)":4
 *     raise TypeError("self.c_argument cannot be converted to a Python object for pickling")
 * def __setstate_cython__(self, __pyx_state):
 *     raise TypeError("self.c_argument cannot be converted to a Python object for pickling")             # <<<<<<<<<<<<<<
 */
  __pyx_t_1 = __Pyx_PyObject_Call(__pyx_builtin_TypeError, __pyx_tuple__7, NULL); if (unlikely(!__pyx_t_1)) __PYX_ERR(7, 4, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_1);
  __Pyx_Raise(__pyx_t_1, 0, 0, 0);
  __Pyx_DECREF(__pyx_t_1); __pyx_t_1 = 0;
  __PYX_ERR(7, 4, __pyx_L1_error)

  /* "(tree fragment)":3
 * def __reduce_cython__(self):
 *     raise TypeError("self.c_argument cannot be converted to a Python object for pickling")
 * def __setstate_cython__(self, __pyx_state):             # <<<<<<<<<<<<<<
 *     raise TypeError("self.c_argument cannot be converted to a Python object for pickling")
 */

  /* function exit code */
  __pyx_L1_error:;
  __Pyx_XDECREF(__pyx_t_1);
  __Pyx_AddTraceback("grpc._cython.cygrpc._ChannelArg.__setstate_cython__", __pyx_clineno, __pyx_lineno, __pyx_filename);
  __pyx_r = NULL;
  __Pyx_XGIVEREF(__pyx_r);
  __Pyx_RefNannyFinishContext();
  return __pyx_r;
}

/* "src/python/grpcio/grpc/_cython/_cygrpc/arguments.pyx.pxi":68
 * cdef class _ChannelArgs:
 * 
 *   def __cinit__(self, arguments, _VTable vtable not None):             # <<<<<<<<<<<<<<
 *     self._arguments = () if arguments is None else tuple(arguments)
 *     self._channel_args = []
 */

/* Python wrapper */
static int __pyx_pw_4grpc_7_cython_6cygrpc_12_ChannelArgs_1__cinit__(PyObject *__pyx_v_self, PyObject *__pyx_args, PyObject *__pyx_kwds); /*proto*/
static int __pyx_pw_4grpc_7_cython_6cygrpc_12_ChannelArgs_1__cinit__(PyObject *__pyx_v_self, PyObject *__pyx_args, PyObject *__pyx_kwds) {
  PyObject *__pyx_v_arguments = 0;
  struct __pyx_obj_4grpc_7_cython_6cygrpc__VTable *__pyx_v_vtable = 0;
  int __pyx_r;
  __Pyx_RefNannyDeclarations
  __Pyx_RefNannySetupContext("__cinit__ (wrapper)", 0);
  {
    static PyObject **__pyx_pyargnames[] = {&__pyx_n_s_arguments,&__pyx_n_s_vtable,0};
    PyObject* values[2] = {0,0};
    if (unlikely(__pyx_kwds)) {
      Py_ssize_t kw_args;
      const Py_ssize_t pos_args = PyTuple_GET_SIZE(__pyx_args);
      switch (pos_args) {
        case  2: values[1] = PyTuple_GET_ITEM(__pyx_args, 1);
        CYTHON_FALLTHROUGH;
        case  1: values[0] = PyTuple_GET_ITEM(__pyx_args, 0);
        CYTHON_FALLTHROUGH;
        case  0: break;
        default: goto __pyx_L5_argtuple_error;
      }
      kw_args = PyDict_Size(__pyx_kwds);
      switch (pos_args) {
        case  0:
        if (likely((values[0] = __Pyx_PyDict_GetItemStr(__pyx_kwds, __pyx_n_s_arguments)) != 0)) kw_args--;
        else goto __pyx_L5_argtuple_error;
        CYTHON_FALLTHROUGH;
        case  1:
        if (likely((values[1] = __Pyx_PyDict_GetItemStr(__pyx_kwds, __pyx_n_s_vtable)) != 0)) kw_args--;
        else {
          __Pyx_RaiseArgtupleInvalid("__cinit__", 1, 2, 2, 1); __PYX_ERR(2, 68, __pyx_L3_error)
        }
      }
      if (unlikely(kw_args > 0)) {
        if (unlikely(__Pyx_ParseOptionalKeywords(__pyx_kwds, __pyx_pyargnames, 0, values, pos_args, "__cinit__") < 0)) __PYX_ERR(2, 68, __pyx_L3_error)
      }
    } else if (PyTuple_GET_SIZE(__pyx_args) != 2) {
      goto __pyx_L5_argtuple_error;
    } else {
      values[0] = PyTuple_GET_ITEM(__pyx_args, 0);
      values[1] = PyTuple_GET_ITEM(__pyx_args, 1);
    }
    __pyx_v_arguments = values[0];
    __pyx_v_vtable = ((struct __pyx_obj_4grpc_7_cython_6cygrpc__VTable *)values[1]);
  }
  goto __pyx_L4_argument_unpacking_done;
  __pyx_L5_argtuple_error:;
  __Pyx_RaiseArgtupleInvalid("__cinit__", 1, 2, 2, PyTuple_GET_SIZE(__pyx_args)); __PYX_ERR(2, 68, __pyx_L3_error)
  __pyx_L3_error:;
  __Pyx_AddTraceback("grpc._cython.cygrpc._ChannelArgs.__cinit__", __pyx_clineno, __pyx_lineno, __pyx_filename);
  __Pyx_RefNannyFinishContext();
  return -1;
  __pyx_L4_argument_unpacking_done:;
  if (unlikely(!__Pyx_ArgTypeTest(((PyObject *)__pyx_v_vtable), __pyx_ptype_4grpc_7_cython_6cygrpc__VTable, 0, "vtable", 0))) __PYX_ERR(2, 68, __pyx_L1_error)
  __pyx_r = __pyx_pf_4grpc_7_cython_6cygrpc_12_ChannelArgs___cinit__(((struct __pyx_obj_4grpc_7_cython_6cygrpc__ChannelArgs *)__pyx_v_self), __pyx_v_arguments, __pyx_v_vtable);

  /* function exit code */
  goto __pyx_L0;
  __pyx_L1_error:;
  __pyx_r = -1;
  __pyx_L0:;
  __Pyx_RefNannyFinishContext();
  return __pyx_r;
}

static int __pyx_pf_4grpc_7_cython_6cygrpc_12_ChannelArgs___cinit__(struct __pyx_obj_4grpc_7_cython_6cygrpc__ChannelArgs *__pyx_v_self, PyObject *__pyx_v_arguments, struct __pyx_obj_4grpc_7_cython_6cygrpc__VTable *__pyx_v_vtable) {
  PyObject *__pyx_v_index = NULL;
  PyObject *__pyx_v_argument = NULL;
  struct __pyx_obj_4grpc_7_cython_6cygrpc__ChannelArg *__pyx_v_channel_arg = NULL;
  int __pyx_r;
  __Pyx_RefNannyDeclarations
  PyObject *__pyx_t_1 = NULL;
  int __pyx_t_2;
  PyObject *__pyx_t_3 = NULL;
  Py_ssize_t __pyx_t_4;
  PyObject *__pyx_t_5 = NULL;
  grpc_arg __pyx_t_6;
  Py_ssize_t __pyx_t_7;
  int __pyx_t_8;
  __Pyx_RefNannySetupContext("__cinit__", 0);

  /* "src/python/grpcio/grpc/_cython/_cygrpc/arguments.pyx.pxi":69
 * 
 *   def __cinit__(self, arguments, _VTable vtable not None):
 *     self._arguments = () if arguments is None else tuple(arguments)             # <<<<<<<<<<<<<<
 *     self._channel_args = []
 *     self._references = []
 */
  __pyx_t_2 = (__pyx_v_arguments == Py_None);
  if ((__pyx_t_2 != 0)) {
    __Pyx_INCREF(__pyx_empty_tuple);
    __pyx_t_1 = __pyx_empty_tuple;
  } else {
    __pyx_t_3 = __Pyx_PySequence_Tuple(__pyx_v_arguments); if (unlikely(!__pyx_t_3)) __PYX_ERR(2, 69, __pyx_L1_error)
    __Pyx_GOTREF(__pyx_t_3);
    __pyx_t_1 = __pyx_t_3;
    __pyx_t_3 = 0;
  }
  __Pyx_GIVEREF(__pyx_t_1);
  __Pyx_GOTREF(__pyx_v_self->_arguments);
  __Pyx_DECREF(__pyx_v_self->_arguments);
  __pyx_v_self->_arguments = ((PyObject*)__pyx_t_1);
  __pyx_t_1 = 0;

  /* "src/python/grpcio/grpc/_cython/_cygrpc/arguments.pyx.pxi":70
 *   def __cinit__(self, arguments, _VTable vtable not None):
 *     self._arguments = () if arguments is None else tuple(arguments)
 *     self._channel_args = []             # <<<<<<<<<<<<<<
 *     self._references = []
 *     self._c_arguments.arguments_length = len(self._arguments)
 */
  __pyx_t_1 = PyList_New(0); if (unlikely(!__pyx_t_1)) __PYX_ERR(2, 70, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_1);
  __Pyx_GIVEREF(__pyx_t_1);
  __Pyx_GOTREF(__pyx_v_self->_channel_args);
  __Pyx_DECREF(__pyx_v_self->_channel_args);
  __pyx_v_self->_channel_args = ((PyObject*)__pyx_t_1);
  __pyx_t_1 = 0;

  /* "src/python/grpcio/grpc/_cython/_cygrpc/arguments.pyx.pxi":71
 *     self._arguments = () if arguments is None else tuple(arguments)
 *     self._channel_args = []
 *     self._references = []             # <<<<<<<<<<<<<<
 *     self._c_arguments.arguments_length = len(self._arguments)
 *     if self._c_arguments.arguments_length != 0:
 */
  __pyx_t_1 = PyList_New(0); if (unlikely(!__pyx_t_1)) __PYX_ERR(2, 71, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_1);
  __Pyx_GIVEREF(__pyx_t_1);
  __Pyx_GOTREF(__pyx_v_self->_references);
  __Pyx_DECREF(__pyx_v_self->_references);
  __pyx_v_self->_references = ((PyObject*)__pyx_t_1);
  __pyx_t_1 = 0;

  /* "src/python/grpcio/grpc/_cython/_cygrpc/arguments.pyx.pxi":72
 *     self._channel_args = []
 *     self._references = []
 *     self._c_arguments.arguments_length = len(self._arguments)             # <<<<<<<<<<<<<<
 *     if self._c_arguments.arguments_length != 0:
 *       self._c_arguments.arguments = <grpc_arg *>gpr_malloc(
 */
  __pyx_t_1 = __pyx_v_self->_arguments;
  __Pyx_INCREF(__pyx_t_1);
  if (unlikely(__pyx_t_1 == Py_None)) {
    PyErr_SetString(PyExc_TypeError, "object of type 'NoneType' has no len()");
    __PYX_ERR(2, 72, __pyx_L1_error)
  }
  __pyx_t_4 = PyTuple_GET_SIZE(__pyx_t_1); if (unlikely(__pyx_t_4 == ((Py_ssize_t)-1))) __PYX_ERR(2, 72, __pyx_L1_error)
  __Pyx_DECREF(__pyx_t_1); __pyx_t_1 = 0;
  __pyx_v_self->_c_arguments.num_args = __pyx_t_4;

  /* "src/python/grpcio/grpc/_cython/_cygrpc/arguments.pyx.pxi":73
 *     self._references = []
 *     self._c_arguments.arguments_length = len(self._arguments)
 *     if self._c_arguments.arguments_length != 0:             # <<<<<<<<<<<<<<
 *       self._c_arguments.arguments = <grpc_arg *>gpr_malloc(
 *           self._c_arguments.arguments_length * sizeof(grpc_arg))
 */
  __pyx_t_2 = ((__pyx_v_self->_c_arguments.num_args != 0) != 0);
  if (__pyx_t_2) {

    /* "src/python/grpcio/grpc/_cython/_cygrpc/arguments.pyx.pxi":74
 *     self._c_arguments.arguments_length = len(self._arguments)
 *     if self._c_arguments.arguments_length != 0:
 *       self._c_arguments.arguments = <grpc_arg *>gpr_malloc(             # <<<<<<<<<<<<<<
 *           self._c_arguments.arguments_length * sizeof(grpc_arg))
 *       for index, argument in enumerate(self._arguments):
 */
    __pyx_v_self->_c_arguments.args = ((grpc_arg *)gpr_malloc((__pyx_v_self->_c_arguments.num_args * (sizeof(grpc_arg)))));

    /* "src/python/grpcio/grpc/_cython/_cygrpc/arguments.pyx.pxi":76
 *       self._c_arguments.arguments = <grpc_arg *>gpr_malloc(
 *           self._c_arguments.arguments_length * sizeof(grpc_arg))
 *       for index, argument in enumerate(self._arguments):             # <<<<<<<<<<<<<<
 *         channel_arg = _ChannelArg()
 *         channel_arg.c(argument, vtable, self._references)
 */
    __Pyx_INCREF(__pyx_int_0);
    __pyx_t_1 = __pyx_int_0;
    __pyx_t_3 = __pyx_v_self->_arguments; __Pyx_INCREF(__pyx_t_3); __pyx_t_4 = 0;
    for (;;) {
      if (__pyx_t_4 >= PyTuple_GET_SIZE(__pyx_t_3)) break;
      #if CYTHON_ASSUME_SAFE_MACROS && !CYTHON_AVOID_BORROWED_REFS
      __pyx_t_5 = PyTuple_GET_ITEM(__pyx_t_3, __pyx_t_4); __Pyx_INCREF(__pyx_t_5); __pyx_t_4++; if (unlikely(0 < 0)) __PYX_ERR(2, 76, __pyx_L1_error)
      #else
      __pyx_t_5 = PySequence_ITEM(__pyx_t_3, __pyx_t_4); __pyx_t_4++; if (unlikely(!__pyx_t_5)) __PYX_ERR(2, 76, __pyx_L1_error)
      __Pyx_GOTREF(__pyx_t_5);
      #endif
      __Pyx_XDECREF_SET(__pyx_v_argument, __pyx_t_5);
      __pyx_t_5 = 0;
      __Pyx_INCREF(__pyx_t_1);
      __Pyx_XDECREF_SET(__pyx_v_index, __pyx_t_1);
      __pyx_t_5 = __Pyx_PyInt_AddObjC(__pyx_t_1, __pyx_int_1, 1, 0); if (unlikely(!__pyx_t_5)) __PYX_ERR(2, 76, __pyx_L1_error)
      __Pyx_GOTREF(__pyx_t_5);
      __Pyx_DECREF(__pyx_t_1);
      __pyx_t_1 = __pyx_t_5;
      __pyx_t_5 = 0;

      /* "src/python/grpcio/grpc/_cython/_cygrpc/arguments.pyx.pxi":77
 *           self._c_arguments.arguments_length * sizeof(grpc_arg))
 *       for index, argument in enumerate(self._arguments):
 *         channel_arg = _ChannelArg()             # <<<<<<<<<<<<<<
 *         channel_arg.c(argument, vtable, self._references)
 *         self._c_arguments.arguments[index] = channel_arg.c_argument
 */
      __pyx_t_5 = __Pyx_PyObject_CallNoArg(((PyObject *)__pyx_ptype_4grpc_7_cython_6cygrpc__ChannelArg)); if (unlikely(!__pyx_t_5)) __PYX_ERR(2, 77, __pyx_L1_error)
      __Pyx_GOTREF(__pyx_t_5);
      __Pyx_XDECREF_SET(__pyx_v_channel_arg, ((struct __pyx_obj_4grpc_7_cython_6cygrpc__ChannelArg *)__pyx_t_5));
      __pyx_t_5 = 0;

      /* "src/python/grpcio/grpc/_cython/_cygrpc/arguments.pyx.pxi":78
 *       for index, argument in enumerate(self._arguments):
 *         channel_arg = _ChannelArg()
 *         channel_arg.c(argument, vtable, self._references)             # <<<<<<<<<<<<<<
 *         self._c_arguments.arguments[index] = channel_arg.c_argument
 *         self._channel_args.append(channel_arg)
 */
      __pyx_t_5 = __pyx_v_self->_references;
      __Pyx_INCREF(__pyx_t_5);
      ((struct __pyx_vtabstruct_4grpc_7_cython_6cygrpc__ChannelArg *)__pyx_v_channel_arg->__pyx_vtab)->c(__pyx_v_channel_arg, __pyx_v_argument, __pyx_v_vtable, __pyx_t_5); if (unlikely(PyErr_Occurred())) __PYX_ERR(2, 78, __pyx_L1_error)
      __Pyx_DECREF(__pyx_t_5); __pyx_t_5 = 0;

      /* "src/python/grpcio/grpc/_cython/_cygrpc/arguments.pyx.pxi":79
 *         channel_arg = _ChannelArg()
 *         channel_arg.c(argument, vtable, self._references)
 *         self._c_arguments.arguments[index] = channel_arg.c_argument             # <<<<<<<<<<<<<<
 *         self._channel_args.append(channel_arg)
 * 
 */
      __pyx_t_6 = __pyx_v_channel_arg->c_argument;
      __pyx_t_7 = __Pyx_PyIndex_AsSsize_t(__pyx_v_index); if (unlikely((__pyx_t_7 == (Py_ssize_t)-1) && PyErr_Occurred())) __PYX_ERR(2, 79, __pyx_L1_error)
      (__pyx_v_self->_c_arguments.args[__pyx_t_7]) = __pyx_t_6;

      /* "src/python/grpcio/grpc/_cython/_cygrpc/arguments.pyx.pxi":80
 *         channel_arg.c(argument, vtable, self._references)
 *         self._c_arguments.arguments[index] = channel_arg.c_argument
 *         self._channel_args.append(channel_arg)             # <<<<<<<<<<<<<<
 * 
 *   cdef grpc_channel_args *c_args(self) except *:
 */
      if (unlikely(__pyx_v_self->_channel_args == Py_None)) {
        PyErr_Format(PyExc_AttributeError, "'NoneType' object has no attribute '%.30s'", "append");
        __PYX_ERR(2, 80, __pyx_L1_error)
      }
      __pyx_t_8 = __Pyx_PyList_Append(__pyx_v_self->_channel_args, ((PyObject *)__pyx_v_channel_arg)); if (unlikely(__pyx_t_8 == ((int)-1))) __PYX_ERR(2, 80, __pyx_L1_error)

      /* "src/python/grpcio/grpc/_cython/_cygrpc/arguments.pyx.pxi":76
 *       self._c_arguments.arguments = <grpc_arg *>gpr_malloc(
 *           self._c_arguments.arguments_length * sizeof(grpc_arg))
 *       for index, argument in enumerate(self._arguments):             # <<<<<<<<<<<<<<
 *         channel_arg = _ChannelArg()
 *         channel_arg.c(argument, vtable, self._references)
 */
    }
    __Pyx_DECREF(__pyx_t_3); __pyx_t_3 = 0;
    __Pyx_DECREF(__pyx_t_1); __pyx_t_1 = 0;

    /* "src/python/grpcio/grpc/_cython/_cygrpc/arguments.pyx.pxi":73
 *     self._references = []
 *     self._c_arguments.arguments_length = len(self._arguments)
 *     if self._c_arguments.arguments_length != 0:             # <<<<<<<<<<<<<<
 *       self._c_arguments.arguments = <grpc_arg *>gpr_malloc(
 *           self._c_arguments.arguments_length * sizeof(grpc_arg))
 */
  }

  /* "src/python/grpcio/grpc/_cython/_cygrpc/arguments.pyx.pxi":68
 * cdef class _ChannelArgs:
 * 
 *   def __cinit__(self, arguments, _VTable vtable not None):             # <<<<<<<<<<<<<<
 *     self._arguments = () if arguments is None else tuple(arguments)
 *     self._channel_args = []
 */

  /* function exit code */
  __pyx_r = 0;
  goto __pyx_L0;
  __pyx_L1_error:;
  __Pyx_XDECREF(__pyx_t_1);
  __Pyx_XDECREF(__pyx_t_3);
  __Pyx_XDECREF(__pyx_t_5);
  __Pyx_AddTraceback("grpc._cython.cygrpc._ChannelArgs.__cinit__", __pyx_clineno, __pyx_lineno, __pyx_filename);
  __pyx_r = -1;
  __pyx_L0:;
  __Pyx_XDECREF(__pyx_v_index);
  __Pyx_XDECREF(__pyx_v_argument);
  __Pyx_XDECREF((PyObject *)__pyx_v_channel_arg);
  __Pyx_RefNannyFinishContext();
  return __pyx_r;
}

/* "src/python/grpcio/grpc/_cython/_cygrpc/arguments.pyx.pxi":82
 *         self._channel_args.append(channel_arg)
 * 
 *   cdef grpc_channel_args *c_args(self) except *:             # <<<<<<<<<<<<<<
 *     return &self._c_arguments
 * 
 */

static grpc_channel_args *__pyx_f_4grpc_7_cython_6cygrpc_12_ChannelArgs_c_args(struct __pyx_obj_4grpc_7_cython_6cygrpc__ChannelArgs *__pyx_v_self) {
  grpc_channel_args *__pyx_r;
  __Pyx_RefNannyDeclarations
  __Pyx_RefNannySetupContext("c_args", 0);

  /* "src/python/grpcio/grpc/_cython/_cygrpc/arguments.pyx.pxi":83
 * 
 *   cdef grpc_channel_args *c_args(self) except *:
 *     return &self._c_arguments             # <<<<<<<<<<<<<<
 * 
 *   def __dealloc__(self):
 */
  __pyx_r = (&__pyx_v_self->_c_arguments);
  goto __pyx_L0;

  /* "src/python/grpcio/grpc/_cython/_cygrpc/arguments.pyx.pxi":82
 *         self._channel_args.append(channel_arg)
 * 
 *   cdef grpc_channel_args *c_args(self) except *:             # <<<<<<<<<<<<<<
 *     return &self._c_arguments
 * 
 */

  /* function exit code */
  __pyx_L0:;
  __Pyx_RefNannyFinishContext();
  return __pyx_r;
}

/* "src/python/grpcio/grpc/_cython/_cygrpc/arguments.pyx.pxi":85
 *     return &self._c_arguments
 * 
 *   def __dealloc__(self):             # <<<<<<<<<<<<<<
 *     if self._c_arguments.arguments != NULL:
 *       gpr_free(self._c_arguments.arguments)
 */

/* Python wrapper */
static void __pyx_pw_4grpc_7_cython_6cygrpc_12_ChannelArgs_3__dealloc__(PyObject *__pyx_v_self); /*proto*/
static void __pyx_pw_4grpc_7_cython_6cygrpc_12_ChannelArgs_3__dealloc__(PyObject *__pyx_v_self) {
  __Pyx_RefNannyDeclarations
  __Pyx_RefNannySetupContext("__dealloc__ (wrapper)", 0);
  __pyx_pf_4grpc_7_cython_6cygrpc_12_ChannelArgs_2__dealloc__(((struct __pyx_obj_4grpc_7_cython_6cygrpc__ChannelArgs *)__pyx_v_self));

  /* function exit code */
  __Pyx_RefNannyFinishContext();
}

static void __pyx_pf_4grpc_7_cython_6cygrpc_12_ChannelArgs_2__dealloc__(struct __pyx_obj_4grpc_7_cython_6cygrpc__ChannelArgs *__pyx_v_self) {
  __Pyx_RefNannyDeclarations
  int __pyx_t_1;
  __Pyx_RefNannySetupContext("__dealloc__", 0);

  /* "src/python/grpcio/grpc/_cython/_cygrpc/arguments.pyx.pxi":86
 * 
 *   def __dealloc__(self):
 *     if self._c_arguments.arguments != NULL:             # <<<<<<<<<<<<<<
 *       gpr_free(self._c_arguments.arguments)
 */
  __pyx_t_1 = ((__pyx_v_self->_c_arguments.args != NULL) != 0);
  if (__pyx_t_1) {

    /* "src/python/grpcio/grpc/_cython/_cygrpc/arguments.pyx.pxi":87
 *   def __dealloc__(self):
 *     if self._c_arguments.arguments != NULL:
 *       gpr_free(self._c_arguments.arguments)             # <<<<<<<<<<<<<<
 */
    gpr_free(__pyx_v_self->_c_arguments.args);

    /* "src/python/grpcio/grpc/_cython/_cygrpc/arguments.pyx.pxi":86
 * 
 *   def __dealloc__(self):
 *     if self._c_arguments.arguments != NULL:             # <<<<<<<<<<<<<<
 *       gpr_free(self._c_arguments.arguments)
 */
  }

  /* "src/python/grpcio/grpc/_cython/_cygrpc/arguments.pyx.pxi":85
 *     return &self._c_arguments
 * 
 *   def __dealloc__(self):             # <<<<<<<<<<<<<<
 *     if self._c_arguments.arguments != NULL:
 *       gpr_free(self._c_arguments.arguments)
 */

  /* function exit code */
  __Pyx_RefNannyFinishContext();
}

/* "src/python/grpcio/grpc/_cython/_cygrpc/arguments.pxd.pxi":31
 * cdef class _ChannelArgs:
 * 
 *   cdef readonly tuple _arguments             # <<<<<<<<<<<<<<
 *   cdef list _channel_args
 *   cdef readonly list _references
 */

/* Python wrapper */
static PyObject *__pyx_pw_4grpc_7_cython_6cygrpc_12_ChannelArgs_10_arguments_1__get__(PyObject *__pyx_v_self); /*proto*/
static PyObject *__pyx_pw_4grpc_7_cython_6cygrpc_12_ChannelArgs_10_arguments_1__get__(PyObject *__pyx_v_self) {
  PyObject *__pyx_r = 0;
  __Pyx_RefNannyDeclarations
  __Pyx_RefNannySetupContext("__get__ (wrapper)", 0);
  __pyx_r = __pyx_pf_4grpc_7_cython_6cygrpc_12_ChannelArgs_10_arguments___get__(((struct __pyx_obj_4grpc_7_cython_6cygrpc__ChannelArgs *)__pyx_v_self));

  /* function exit code */
  __Pyx_RefNannyFinishContext();
  return __pyx_r;
}

static PyObject *__pyx_pf_4grpc_7_cython_6cygrpc_12_ChannelArgs_10_arguments___get__(struct __pyx_obj_4grpc_7_cython_6cygrpc__ChannelArgs *__pyx_v_self) {
  PyObject *__pyx_r = NULL;
  __Pyx_RefNannyDeclarations
  __Pyx_RefNannySetupContext("__get__", 0);
  __Pyx_XDECREF(__pyx_r);
  __Pyx_INCREF(__pyx_v_self->_arguments);
  __pyx_r = __pyx_v_self->_arguments;
  goto __pyx_L0;

  /* function exit code */
  __pyx_L0:;
  __Pyx_XGIVEREF(__pyx_r);
  __Pyx_RefNannyFinishContext();
  return __pyx_r;
}

/* "src/python/grpcio/grpc/_cython/_cygrpc/arguments.pxd.pxi":33
 *   cdef readonly tuple _arguments
 *   cdef list _channel_args
 *   cdef readonly list _references             # <<<<<<<<<<<<<<
 *   cdef grpc_channel_args _c_arguments
 * 
 */

/* Python wrapper */
static PyObject *__pyx_pw_4grpc_7_cython_6cygrpc_12_ChannelArgs_11_references_1__get__(PyObject *__pyx_v_self); /*proto*/
static PyObject *__pyx_pw_4grpc_7_cython_6cygrpc_12_ChannelArgs_11_references_1__get__(PyObject *__pyx_v_self) {
  PyObject *__pyx_r = 0;
  __Pyx_RefNannyDeclarations
  __Pyx_RefNannySetupContext("__get__ (wrapper)", 0);
  __pyx_r = __pyx_pf_4grpc_7_cython_6cygrpc_12_ChannelArgs_11_references___get__(((struct __pyx_obj_4grpc_7_cython_6cygrpc__ChannelArgs *)__pyx_v_self));

  /* function exit code */
  __Pyx_RefNannyFinishContext();
  return __pyx_r;
}

static PyObject *__pyx_pf_4grpc_7_cython_6cygrpc_12_ChannelArgs_11_references___get__(struct __pyx_obj_4grpc_7_cython_6cygrpc__ChannelArgs *__pyx_v_self) {
  PyObject *__pyx_r = NULL;
  __Pyx_RefNannyDeclarations
  __Pyx_RefNannySetupContext("__get__", 0);
  __Pyx_XDECREF(__pyx_r);
  __Pyx_INCREF(__pyx_v_self->_references);
  __pyx_r = __pyx_v_self->_references;
  goto __pyx_L0;

  /* function exit code */
  __pyx_L0:;
  __Pyx_XGIVEREF(__pyx_r);
  __Pyx_RefNannyFinishContext();
  return __pyx_r;
}

/* "(tree fragment)":1
 * def __reduce_cython__(self):             # <<<<<<<<<<<<<<
 *     raise TypeError("no default __reduce__ due to non-trivial __cinit__")
 * def __setstate_cython__(self, __pyx_state):
 */

/* Python wrapper */
static PyObject *__pyx_pw_4grpc_7_cython_6cygrpc_12_ChannelArgs_5__reduce_cython__(PyObject *__pyx_v_self, CYTHON_UNUSED PyObject *unused); /*proto*/
static PyObject *__pyx_pw_4grpc_7_cython_6cygrpc_12_ChannelArgs_5__reduce_cython__(PyObject *__pyx_v_self, CYTHON_UNUSED PyObject *unused) {
  PyObject *__pyx_r = 0;
  __Pyx_RefNannyDeclarations
  __Pyx_RefNannySetupContext("__reduce_cython__ (wrapper)", 0);
  __pyx_r = __pyx_pf_4grpc_7_cython_6cygrpc_12_ChannelArgs_4__reduce_cython__(((struct __pyx_obj_4grpc_7_cython_6cygrpc__ChannelArgs *)__pyx_v_self));

  /* function exit code */
  __Pyx_RefNannyFinishContext();
  return __pyx_r;
}

static PyObject *__pyx_pf_4grpc_7_cython_6cygrpc_12_ChannelArgs_4__reduce_cython__(CYTHON_UNUSED struct __pyx_obj_4grpc_7_cython_6cygrpc__ChannelArgs *__pyx_v_self) {
  PyObject *__pyx_r = NULL;
  __Pyx_RefNannyDeclarations
  PyObject *__pyx_t_1 = NULL;
  __Pyx_RefNannySetupContext("__reduce_cython__", 0);

  /* "(tree fragment)":2
 * def __reduce_cython__(self):
 *     raise TypeError("no default __reduce__ due to non-trivial __cinit__")             # <<<<<<<<<<<<<<
 * def __setstate_cython__(self, __pyx_state):
 *     raise TypeError("no default __reduce__ due to non-trivial __cinit__")
 */
  __pyx_t_1 = __Pyx_PyObject_Call(__pyx_builtin_TypeError, __pyx_tuple__8, NULL); if (unlikely(!__pyx_t_1)) __PYX_ERR(7, 2, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_1);
  __Pyx_Raise(__pyx_t_1, 0, 0, 0);
  __Pyx_DECREF(__pyx_t_1); __pyx_t_1 = 0;
  __PYX_ERR(7, 2, __pyx_L1_error)

  /* "(tree fragment)":1
 * def __reduce_cython__(self):             # <<<<<<<<<<<<<<
 *     raise TypeError("no default __reduce__ due to non-trivial __cinit__")
 * def __setstate_cython__(self, __pyx_state):
 */

  /* function exit code */
  __pyx_L1_error:;
  __Pyx_XDECREF(__pyx_t_1);
  __Pyx_AddTraceback("grpc._cython.cygrpc._ChannelArgs.__reduce_cython__", __pyx_clineno, __pyx_lineno, __pyx_filename);
  __pyx_r = NULL;
  __Pyx_XGIVEREF(__pyx_r);
  __Pyx_RefNannyFinishContext();
  return __pyx_r;
}

/* "(tree fragment)":3
 * def __reduce_cython__(self):
 *     raise TypeError("no default __reduce__ due to non-trivial __cinit__")
 * def __setstate_cython__(self, __pyx_state):             # <<<<<<<<<<<<<<
 *     raise TypeError("no default __reduce__ due to non-trivial __cinit__")
 */

/* Python wrapper */
static PyObject *__pyx_pw_4grpc_7_cython_6cygrpc_12_ChannelArgs_7__setstate_cython__(PyObject *__pyx_v_self, PyObject *__pyx_v___pyx_state); /*proto*/
static PyObject *__pyx_pw_4grpc_7_cython_6cygrpc_12_ChannelArgs_7__setstate_cython__(PyObject *__pyx_v_self, PyObject *__pyx_v___pyx_state) {
  PyObject *__pyx_r = 0;
  __Pyx_RefNannyDeclarations
  __Pyx_RefNannySetupContext("__setstate_cython__ (wrapper)", 0);
  __pyx_r = __pyx_pf_4grpc_7_cython_6cygrpc_12_ChannelArgs_6__setstate_cython__(((struct __pyx_obj_4grpc_7_cython_6cygrpc__ChannelArgs *)__pyx_v_self), ((PyObject *)__pyx_v___pyx_state));

  /* function exit code */
  __Pyx_RefNannyFinishContext();
  return __pyx_r;
}

static PyObject *__pyx_pf_4grpc_7_cython_6cygrpc_12_ChannelArgs_6__setstate_cython__(CYTHON_UNUSED struct __pyx_obj_4grpc_7_cython_6cygrpc__ChannelArgs *__pyx_v_self, CYTHON_UNUSED PyObject *__pyx_v___pyx_state) {
  PyObject *__pyx_r = NULL;
  __Pyx_RefNannyDeclarations
  PyObject *__pyx_t_1 = NULL;
  __Pyx_RefNannySetupContext("__setstate_cython__", 0);

  /* "(tree fragment)":4
 *     raise TypeError("no default __reduce__ due to non-trivial __cinit__")
 * def __setstate_cython__(self, __pyx_state):
 *     raise TypeError("no default __reduce__ due to non-trivial __cinit__")             # <<<<<<<<<<<<<<
 */
  __pyx_t_1 = __Pyx_PyObject_Call(__pyx_builtin_TypeError, __pyx_tuple__9, NULL); if (unlikely(!__pyx_t_1)) __PYX_ERR(7, 4, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_1);
  __Pyx_Raise(__pyx_t_1, 0, 0, 0);
  __Pyx_DECREF(__pyx_t_1); __pyx_t_1 = 0;
  __PYX_ERR(7, 4, __pyx_L1_error)

  /* "(tree fragment)":3
 * def __reduce_cython__(self):
 *     raise TypeError("no default __reduce__ due to non-trivial __cinit__")
 * def __setstate_cython__(self, __pyx_state):             # <<<<<<<<<<<<<<
 *     raise TypeError("no default __reduce__ due to non-trivial __cinit__")
 */

  /* function exit code */
  __pyx_L1_error:;
  __Pyx_XDECREF(__pyx_t_1);
  __Pyx_AddTraceback("grpc._cython.cygrpc._ChannelArgs.__setstate_cython__", __pyx_clineno, __pyx_lineno, __pyx_filename);
  __pyx_r = NULL;
  __Pyx_XGIVEREF(__pyx_r);
  __Pyx_RefNannyFinishContext();
  return __pyx_r;
}

/* "src/python/grpcio/grpc/_cython/_cygrpc/call.pyx.pxi":20
 * cdef class Call:
 * 
 *   def __cinit__(self, _VTable vtable not None):             # <<<<<<<<<<<<<<
 *     # Create an *empty* call
 *     fork_handlers_and_grpc_init()
 */

/* Python wrapper */
static int __pyx_pw_4grpc_7_cython_6cygrpc_4Call_1__cinit__(PyObject *__pyx_v_self, PyObject *__pyx_args, PyObject *__pyx_kwds); /*proto*/
static int __pyx_pw_4grpc_7_cython_6cygrpc_4Call_1__cinit__(PyObject *__pyx_v_self, PyObject *__pyx_args, PyObject *__pyx_kwds) {
  struct __pyx_obj_4grpc_7_cython_6cygrpc__VTable *__pyx_v_vtable = 0;
  int __pyx_r;
  __Pyx_RefNannyDeclarations
  __Pyx_RefNannySetupContext("__cinit__ (wrapper)", 0);
  {
    static PyObject **__pyx_pyargnames[] = {&__pyx_n_s_vtable,0};
    PyObject* values[1] = {0};
    if (unlikely(__pyx_kwds)) {
      Py_ssize_t kw_args;
      const Py_ssize_t pos_args = PyTuple_GET_SIZE(__pyx_args);
      switch (pos_args) {
        case  1: values[0] = PyTuple_GET_ITEM(__pyx_args, 0);
        CYTHON_FALLTHROUGH;
        case  0: break;
        default: goto __pyx_L5_argtuple_error;
      }
      kw_args = PyDict_Size(__pyx_kwds);
      switch (pos_args) {
        case  0:
        if (likely((values[0] = __Pyx_PyDict_GetItemStr(__pyx_kwds, __pyx_n_s_vtable)) != 0)) kw_args--;
        else goto __pyx_L5_argtuple_error;
      }
      if (unlikely(kw_args > 0)) {
        if (unlikely(__Pyx_ParseOptionalKeywords(__pyx_kwds, __pyx_pyargnames, 0, values, pos_args, "__cinit__") < 0)) __PYX_ERR(3, 20, __pyx_L3_error)
      }
    } else if (PyTuple_GET_SIZE(__pyx_args) != 1) {
      goto __pyx_L5_argtuple_error;
    } else {
      values[0] = PyTuple_GET_ITEM(__pyx_args, 0);
    }
    __pyx_v_vtable = ((struct __pyx_obj_4grpc_7_cython_6cygrpc__VTable *)values[0]);
  }
  goto __pyx_L4_argument_unpacking_done;
  __pyx_L5_argtuple_error:;
  __Pyx_RaiseArgtupleInvalid("__cinit__", 1, 1, 1, PyTuple_GET_SIZE(__pyx_args)); __PYX_ERR(3, 20, __pyx_L3_error)
  __pyx_L3_error:;
  __Pyx_AddTraceback("grpc._cython.cygrpc.Call.__cinit__", __pyx_clineno, __pyx_lineno, __pyx_filename);
  __Pyx_RefNannyFinishContext();
  return -1;
  __pyx_L4_argument_unpacking_done:;
  if (unlikely(!__Pyx_ArgTypeTest(((PyObject *)__pyx_v_vtable), __pyx_ptype_4grpc_7_cython_6cygrpc__VTable, 0, "vtable", 0))) __PYX_ERR(3, 20, __pyx_L1_error)
  __pyx_r = __pyx_pf_4grpc_7_cython_6cygrpc_4Call___cinit__(((struct __pyx_obj_4grpc_7_cython_6cygrpc_Call *)__pyx_v_self), __pyx_v_vtable);

  /* function exit code */
  goto __pyx_L0;
  __pyx_L1_error:;
  __pyx_r = -1;
  __pyx_L0:;
  __Pyx_RefNannyFinishContext();
  return __pyx_r;
}

static int __pyx_pf_4grpc_7_cython_6cygrpc_4Call___cinit__(struct __pyx_obj_4grpc_7_cython_6cygrpc_Call *__pyx_v_self, struct __pyx_obj_4grpc_7_cython_6cygrpc__VTable *__pyx_v_vtable) {
  int __pyx_r;
  __Pyx_RefNannyDeclarations
  PyObject *__pyx_t_1 = NULL;
  PyObject *__pyx_t_2 = NULL;
  PyObject *__pyx_t_3 = NULL;
  __Pyx_RefNannySetupContext("__cinit__", 0);

  /* "src/python/grpcio/grpc/_cython/_cygrpc/call.pyx.pxi":22
 *   def __cinit__(self, _VTable vtable not None):
 *     # Create an *empty* call
 *     fork_handlers_and_grpc_init()             # <<<<<<<<<<<<<<
 *     self.c_call = NULL
 *     self.references = [vtable]
 */
  __pyx_t_2 = __Pyx_GetModuleGlobalName(__pyx_n_s_fork_handlers_and_grpc_init); if (unlikely(!__pyx_t_2)) __PYX_ERR(3, 22, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_2);
  __pyx_t_3 = NULL;
  if (CYTHON_UNPACK_METHODS && unlikely(PyMethod_Check(__pyx_t_2))) {
    __pyx_t_3 = PyMethod_GET_SELF(__pyx_t_2);
    if (likely(__pyx_t_3)) {
      PyObject* function = PyMethod_GET_FUNCTION(__pyx_t_2);
      __Pyx_INCREF(__pyx_t_3);
      __Pyx_INCREF(function);
      __Pyx_DECREF_SET(__pyx_t_2, function);
    }
  }
  if (__pyx_t_3) {
    __pyx_t_1 = __Pyx_PyObject_CallOneArg(__pyx_t_2, __pyx_t_3); if (unlikely(!__pyx_t_1)) __PYX_ERR(3, 22, __pyx_L1_error)
    __Pyx_DECREF(__pyx_t_3); __pyx_t_3 = 0;
  } else {
    __pyx_t_1 = __Pyx_PyObject_CallNoArg(__pyx_t_2); if (unlikely(!__pyx_t_1)) __PYX_ERR(3, 22, __pyx_L1_error)
  }
  __Pyx_GOTREF(__pyx_t_1);
  __Pyx_DECREF(__pyx_t_2); __pyx_t_2 = 0;
  __Pyx_DECREF(__pyx_t_1); __pyx_t_1 = 0;

  /* "src/python/grpcio/grpc/_cython/_cygrpc/call.pyx.pxi":23
 *     # Create an *empty* call
 *     fork_handlers_and_grpc_init()
 *     self.c_call = NULL             # <<<<<<<<<<<<<<
 *     self.references = [vtable]
 * 
 */
  __pyx_v_self->c_call = NULL;

  /* "src/python/grpcio/grpc/_cython/_cygrpc/call.pyx.pxi":24
 *     fork_handlers_and_grpc_init()
 *     self.c_call = NULL
 *     self.references = [vtable]             # <<<<<<<<<<<<<<
 * 
 *   def _start_batch(self, operations, tag, retain_self):
 */
  __pyx_t_1 = PyList_New(1); if (unlikely(!__pyx_t_1)) __PYX_ERR(3, 24, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_1);
  __Pyx_INCREF(((PyObject *)__pyx_v_vtable));
  __Pyx_GIVEREF(((PyObject *)__pyx_v_vtable));
  PyList_SET_ITEM(__pyx_t_1, 0, ((PyObject *)__pyx_v_vtable));
  __Pyx_GIVEREF(__pyx_t_1);
  __Pyx_GOTREF(__pyx_v_self->references);
  __Pyx_DECREF(__pyx_v_self->references);
  __pyx_v_self->references = ((PyObject*)__pyx_t_1);
  __pyx_t_1 = 0;

  /* "src/python/grpcio/grpc/_cython/_cygrpc/call.pyx.pxi":20
 * cdef class Call:
 * 
 *   def __cinit__(self, _VTable vtable not None):             # <<<<<<<<<<<<<<
 *     # Create an *empty* call
 *     fork_handlers_and_grpc_init()
 */

  /* function exit code */
  __pyx_r = 0;
  goto __pyx_L0;
  __pyx_L1_error:;
  __Pyx_XDECREF(__pyx_t_1);
  __Pyx_XDECREF(__pyx_t_2);
  __Pyx_XDECREF(__pyx_t_3);
  __Pyx_AddTraceback("grpc._cython.cygrpc.Call.__cinit__", __pyx_clineno, __pyx_lineno, __pyx_filename);
  __pyx_r = -1;
  __pyx_L0:;
  __Pyx_RefNannyFinishContext();
  return __pyx_r;
}

/* "src/python/grpcio/grpc/_cython/_cygrpc/call.pyx.pxi":26
 *     self.references = [vtable]
 * 
 *   def _start_batch(self, operations, tag, retain_self):             # <<<<<<<<<<<<<<
 *     if not self.is_valid:
 *       raise ValueError("invalid call object cannot be used from Python")
 */

/* Python wrapper */
static PyObject *__pyx_pw_4grpc_7_cython_6cygrpc_4Call_3_start_batch(PyObject *__pyx_v_self, PyObject *__pyx_args, PyObject *__pyx_kwds); /*proto*/
static PyObject *__pyx_pw_4grpc_7_cython_6cygrpc_4Call_3_start_batch(PyObject *__pyx_v_self, PyObject *__pyx_args, PyObject *__pyx_kwds) {
  PyObject *__pyx_v_operations = 0;
  PyObject *__pyx_v_tag = 0;
  PyObject *__pyx_v_retain_self = 0;
  PyObject *__pyx_r = 0;
  __Pyx_RefNannyDeclarations
  __Pyx_RefNannySetupContext("_start_batch (wrapper)", 0);
  {
    static PyObject **__pyx_pyargnames[] = {&__pyx_n_s_operations,&__pyx_n_s_tag,&__pyx_n_s_retain_self,0};
    PyObject* values[3] = {0,0,0};
    if (unlikely(__pyx_kwds)) {
      Py_ssize_t kw_args;
      const Py_ssize_t pos_args = PyTuple_GET_SIZE(__pyx_args);
      switch (pos_args) {
        case  3: values[2] = PyTuple_GET_ITEM(__pyx_args, 2);
        CYTHON_FALLTHROUGH;
        case  2: values[1] = PyTuple_GET_ITEM(__pyx_args, 1);
        CYTHON_FALLTHROUGH;
        case  1: values[0] = PyTuple_GET_ITEM(__pyx_args, 0);
        CYTHON_FALLTHROUGH;
        case  0: break;
        default: goto __pyx_L5_argtuple_error;
      }
      kw_args = PyDict_Size(__pyx_kwds);
      switch (pos_args) {
        case  0:
        if (likely((values[0] = __Pyx_PyDict_GetItemStr(__pyx_kwds, __pyx_n_s_operations)) != 0)) kw_args--;
        else goto __pyx_L5_argtuple_error;
        CYTHON_FALLTHROUGH;
        case  1:
        if (likely((values[1] = __Pyx_PyDict_GetItemStr(__pyx_kwds, __pyx_n_s_tag)) != 0)) kw_args--;
        else {
          __Pyx_RaiseArgtupleInvalid("_start_batch", 1, 3, 3, 1); __PYX_ERR(3, 26, __pyx_L3_error)
        }
        CYTHON_FALLTHROUGH;
        case  2:
        if (likely((values[2] = __Pyx_PyDict_GetItemStr(__pyx_kwds, __pyx_n_s_retain_self)) != 0)) kw_args--;
        else {
          __Pyx_RaiseArgtupleInvalid("_start_batch", 1, 3, 3, 2); __PYX_ERR(3, 26, __pyx_L3_error)
        }
      }
      if (unlikely(kw_args > 0)) {
        if (unlikely(__Pyx_ParseOptionalKeywords(__pyx_kwds, __pyx_pyargnames, 0, values, pos_args, "_start_batch") < 0)) __PYX_ERR(3, 26, __pyx_L3_error)
      }
    } else if (PyTuple_GET_SIZE(__pyx_args) != 3) {
      goto __pyx_L5_argtuple_error;
    } else {
      values[0] = PyTuple_GET_ITEM(__pyx_args, 0);
      values[1] = PyTuple_GET_ITEM(__pyx_args, 1);
      values[2] = PyTuple_GET_ITEM(__pyx_args, 2);
    }
    __pyx_v_operations = values[0];
    __pyx_v_tag = values[1];
    __pyx_v_retain_self = values[2];
  }
  goto __pyx_L4_argument_unpacking_done;
  __pyx_L5_argtuple_error:;
  __Pyx_RaiseArgtupleInvalid("_start_batch", 1, 3, 3, PyTuple_GET_SIZE(__pyx_args)); __PYX_ERR(3, 26, __pyx_L3_error)
  __pyx_L3_error:;
  __Pyx_AddTraceback("grpc._cython.cygrpc.Call._start_batch", __pyx_clineno, __pyx_lineno, __pyx_filename);
  __Pyx_RefNannyFinishContext();
  return NULL;
  __pyx_L4_argument_unpacking_done:;
  __pyx_r = __pyx_pf_4grpc_7_cython_6cygrpc_4Call_2_start_batch(((struct __pyx_obj_4grpc_7_cython_6cygrpc_Call *)__pyx_v_self), __pyx_v_operations, __pyx_v_tag, __pyx_v_retain_self);

  /* function exit code */
  __Pyx_RefNannyFinishContext();
  return __pyx_r;
}

static PyObject *__pyx_pf_4grpc_7_cython_6cygrpc_4Call_2_start_batch(struct __pyx_obj_4grpc_7_cython_6cygrpc_Call *__pyx_v_self, PyObject *__pyx_v_operations, PyObject *__pyx_v_tag, PyObject *__pyx_v_retain_self) {
  struct __pyx_obj_4grpc_7_cython_6cygrpc__BatchOperationTag *__pyx_v_batch_operation_tag = 0;
  grpc_call_error __pyx_v_error;
  PyObject *__pyx_r = NULL;
  __Pyx_RefNannyDeclarations
  PyObject *__pyx_t_1 = NULL;
  int __pyx_t_2;
  int __pyx_t_3;
  PyObject *__pyx_t_4 = NULL;
  __Pyx_RefNannySetupContext("_start_batch", 0);

  /* "src/python/grpcio/grpc/_cython/_cygrpc/call.pyx.pxi":27
 * 
 *   def _start_batch(self, operations, tag, retain_self):
 *     if not self.is_valid:             # <<<<<<<<<<<<<<
 *       raise ValueError("invalid call object cannot be used from Python")
 *     cdef _BatchOperationTag batch_operation_tag = _BatchOperationTag(
 */
  __pyx_t_1 = __Pyx_PyObject_GetAttrStr(((PyObject *)__pyx_v_self), __pyx_n_s_is_valid); if (unlikely(!__pyx_t_1)) __PYX_ERR(3, 27, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_1);
  __pyx_t_2 = __Pyx_PyObject_IsTrue(__pyx_t_1); if (unlikely(__pyx_t_2 < 0)) __PYX_ERR(3, 27, __pyx_L1_error)
  __Pyx_DECREF(__pyx_t_1); __pyx_t_1 = 0;
  __pyx_t_3 = ((!__pyx_t_2) != 0);
  if (unlikely(__pyx_t_3)) {

    /* "src/python/grpcio/grpc/_cython/_cygrpc/call.pyx.pxi":28
 *   def _start_batch(self, operations, tag, retain_self):
 *     if not self.is_valid:
 *       raise ValueError("invalid call object cannot be used from Python")             # <<<<<<<<<<<<<<
 *     cdef _BatchOperationTag batch_operation_tag = _BatchOperationTag(
 *         tag, operations, self if retain_self else None)
 */
    __pyx_t_1 = __Pyx_PyObject_Call(__pyx_builtin_ValueError, __pyx_tuple__10, NULL); if (unlikely(!__pyx_t_1)) __PYX_ERR(3, 28, __pyx_L1_error)
    __Pyx_GOTREF(__pyx_t_1);
    __Pyx_Raise(__pyx_t_1, 0, 0, 0);
    __Pyx_DECREF(__pyx_t_1); __pyx_t_1 = 0;
    __PYX_ERR(3, 28, __pyx_L1_error)

    /* "src/python/grpcio/grpc/_cython/_cygrpc/call.pyx.pxi":27
 * 
 *   def _start_batch(self, operations, tag, retain_self):
 *     if not self.is_valid:             # <<<<<<<<<<<<<<
 *       raise ValueError("invalid call object cannot be used from Python")
 *     cdef _BatchOperationTag batch_operation_tag = _BatchOperationTag(
 */
  }

  /* "src/python/grpcio/grpc/_cython/_cygrpc/call.pyx.pxi":30
 *       raise ValueError("invalid call object cannot be used from Python")
 *     cdef _BatchOperationTag batch_operation_tag = _BatchOperationTag(
 *         tag, operations, self if retain_self else None)             # <<<<<<<<<<<<<<
 *     batch_operation_tag.prepare()
 *     cpython.Py_INCREF(batch_operation_tag)
 */
  __pyx_t_3 = __Pyx_PyObject_IsTrue(__pyx_v_retain_self); if (unlikely(__pyx_t_3 < 0)) __PYX_ERR(3, 30, __pyx_L1_error)
  if (__pyx_t_3) {
    __Pyx_INCREF(((PyObject *)__pyx_v_self));
    __pyx_t_1 = ((PyObject *)__pyx_v_self);
  } else {
    __Pyx_INCREF(Py_None);
    __pyx_t_1 = Py_None;
  }

  /* "src/python/grpcio/grpc/_cython/_cygrpc/call.pyx.pxi":29
 *     if not self.is_valid:
 *       raise ValueError("invalid call object cannot be used from Python")
 *     cdef _BatchOperationTag batch_operation_tag = _BatchOperationTag(             # <<<<<<<<<<<<<<
 *         tag, operations, self if retain_self else None)
 *     batch_operation_tag.prepare()
 */
  __pyx_t_4 = PyTuple_New(3); if (unlikely(!__pyx_t_4)) __PYX_ERR(3, 29, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_4);
  __Pyx_INCREF(__pyx_v_tag);
  __Pyx_GIVEREF(__pyx_v_tag);
  PyTuple_SET_ITEM(__pyx_t_4, 0, __pyx_v_tag);
  __Pyx_INCREF(__pyx_v_operations);
  __Pyx_GIVEREF(__pyx_v_operations);
  PyTuple_SET_ITEM(__pyx_t_4, 1, __pyx_v_operations);
  __Pyx_GIVEREF(__pyx_t_1);
  PyTuple_SET_ITEM(__pyx_t_4, 2, __pyx_t_1);
  __pyx_t_1 = 0;
  __pyx_t_1 = __Pyx_PyObject_Call(((PyObject *)__pyx_ptype_4grpc_7_cython_6cygrpc__BatchOperationTag), __pyx_t_4, NULL); if (unlikely(!__pyx_t_1)) __PYX_ERR(3, 29, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_1);
  __Pyx_DECREF(__pyx_t_4); __pyx_t_4 = 0;
  __pyx_v_batch_operation_tag = ((struct __pyx_obj_4grpc_7_cython_6cygrpc__BatchOperationTag *)__pyx_t_1);
  __pyx_t_1 = 0;

  /* "src/python/grpcio/grpc/_cython/_cygrpc/call.pyx.pxi":31
 *     cdef _BatchOperationTag batch_operation_tag = _BatchOperationTag(
 *         tag, operations, self if retain_self else None)
 *     batch_operation_tag.prepare()             # <<<<<<<<<<<<<<
 *     cpython.Py_INCREF(batch_operation_tag)
 *     cdef grpc_call_error error
 */
  ((struct __pyx_vtabstruct_4grpc_7_cython_6cygrpc__BatchOperationTag *)__pyx_v_batch_operation_tag->__pyx_base.__pyx_vtab)->prepare(__pyx_v_batch_operation_tag); if (unlikely(PyErr_Occurred())) __PYX_ERR(3, 31, __pyx_L1_error)

  /* "src/python/grpcio/grpc/_cython/_cygrpc/call.pyx.pxi":32
 *         tag, operations, self if retain_self else None)
 *     batch_operation_tag.prepare()
 *     cpython.Py_INCREF(batch_operation_tag)             # <<<<<<<<<<<<<<
 *     cdef grpc_call_error error
 *     with nogil:
 */
  Py_INCREF(((PyObject *)__pyx_v_batch_operation_tag));

  /* "src/python/grpcio/grpc/_cython/_cygrpc/call.pyx.pxi":34
 *     cpython.Py_INCREF(batch_operation_tag)
 *     cdef grpc_call_error error
 *     with nogil:             # <<<<<<<<<<<<<<
 *       error = grpc_call_start_batch(
 *           self.c_call, batch_operation_tag.c_ops, batch_operation_tag.c_nops,
 */
  {
      #ifdef WITH_THREAD
      PyThreadState *_save;
      Py_UNBLOCK_THREADS
      __Pyx_FastGIL_Remember();
      #endif
      /*try:*/ {

        /* "src/python/grpcio/grpc/_cython/_cygrpc/call.pyx.pxi":35
 *     cdef grpc_call_error error
 *     with nogil:
 *       error = grpc_call_start_batch(             # <<<<<<<<<<<<<<
 *           self.c_call, batch_operation_tag.c_ops, batch_operation_tag.c_nops,
 *           <cpython.PyObject *>batch_operation_tag, NULL)
 */
        __pyx_v_error = grpc_call_start_batch(__pyx_v_self->c_call, __pyx_v_batch_operation_tag->c_ops, __pyx_v_batch_operation_tag->c_nops, ((PyObject *)__pyx_v_batch_operation_tag), NULL);
      }

      /* "src/python/grpcio/grpc/_cython/_cygrpc/call.pyx.pxi":34
 *     cpython.Py_INCREF(batch_operation_tag)
 *     cdef grpc_call_error error
 *     with nogil:             # <<<<<<<<<<<<<<
 *       error = grpc_call_start_batch(
 *           self.c_call, batch_operation_tag.c_ops, batch_operation_tag.c_nops,
 */
      /*finally:*/ {
        /*normal exit:*/{
          #ifdef WITH_THREAD
          __Pyx_FastGIL_Forget();
          Py_BLOCK_THREADS
          #endif
          goto __pyx_L6;
        }
        __pyx_L6:;
      }
  }

  /* "src/python/grpcio/grpc/_cython/_cygrpc/call.pyx.pxi":38
 *           self.c_call, batch_operation_tag.c_ops, batch_operation_tag.c_nops,
 *           <cpython.PyObject *>batch_operation_tag, NULL)
 *     return error             # <<<<<<<<<<<<<<
 * 
 *   def start_client_batch(self, operations, tag):
 */
  __Pyx_XDECREF(__pyx_r);
  __pyx_t_1 = __Pyx_PyInt_From_grpc_call_error(__pyx_v_error); if (unlikely(!__pyx_t_1)) __PYX_ERR(3, 38, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_1);
  __pyx_r = __pyx_t_1;
  __pyx_t_1 = 0;
  goto __pyx_L0;

  /* "src/python/grpcio/grpc/_cython/_cygrpc/call.pyx.pxi":26
 *     self.references = [vtable]
 * 
 *   def _start_batch(self, operations, tag, retain_self):             # <<<<<<<<<<<<<<
 *     if not self.is_valid:
 *       raise ValueError("invalid call object cannot be used from Python")
 */

  /* function exit code */
  __pyx_L1_error:;
  __Pyx_XDECREF(__pyx_t_1);
  __Pyx_XDECREF(__pyx_t_4);
  __Pyx_AddTraceback("grpc._cython.cygrpc.Call._start_batch", __pyx_clineno, __pyx_lineno, __pyx_filename);
  __pyx_r = NULL;
  __pyx_L0:;
  __Pyx_XDECREF((PyObject *)__pyx_v_batch_operation_tag);
  __Pyx_XGIVEREF(__pyx_r);
  __Pyx_RefNannyFinishContext();
  return __pyx_r;
}

/* "src/python/grpcio/grpc/_cython/_cygrpc/call.pyx.pxi":40
 *     return error
 * 
 *   def start_client_batch(self, operations, tag):             # <<<<<<<<<<<<<<
 *     # We don't reference this call in the operations tag because
 *     # it should be cancelled when it goes out of scope
 */

/* Python wrapper */
static PyObject *__pyx_pw_4grpc_7_cython_6cygrpc_4Call_5start_client_batch(PyObject *__pyx_v_self, PyObject *__pyx_args, PyObject *__pyx_kwds); /*proto*/
static PyObject *__pyx_pw_4grpc_7_cython_6cygrpc_4Call_5start_client_batch(PyObject *__pyx_v_self, PyObject *__pyx_args, PyObject *__pyx_kwds) {
  PyObject *__pyx_v_operations = 0;
  PyObject *__pyx_v_tag = 0;
  PyObject *__pyx_r = 0;
  __Pyx_RefNannyDeclarations
  __Pyx_RefNannySetupContext("start_client_batch (wrapper)", 0);
  {
    static PyObject **__pyx_pyargnames[] = {&__pyx_n_s_operations,&__pyx_n_s_tag,0};
    PyObject* values[2] = {0,0};
    if (unlikely(__pyx_kwds)) {
      Py_ssize_t kw_args;
      const Py_ssize_t pos_args = PyTuple_GET_SIZE(__pyx_args);
      switch (pos_args) {
        case  2: values[1] = PyTuple_GET_ITEM(__pyx_args, 1);
        CYTHON_FALLTHROUGH;
        case  1: values[0] = PyTuple_GET_ITEM(__pyx_args, 0);
        CYTHON_FALLTHROUGH;
        case  0: break;
        default: goto __pyx_L5_argtuple_error;
      }
      kw_args = PyDict_Size(__pyx_kwds);
      switch (pos_args) {
        case  0:
        if (likely((values[0] = __Pyx_PyDict_GetItemStr(__pyx_kwds, __pyx_n_s_operations)) != 0)) kw_args--;
        else goto __pyx_L5_argtuple_error;
        CYTHON_FALLTHROUGH;
        case  1:
        if (likely((values[1] = __Pyx_PyDict_GetItemStr(__pyx_kwds, __pyx_n_s_tag)) != 0)) kw_args--;
        else {
          __Pyx_RaiseArgtupleInvalid("start_client_batch", 1, 2, 2, 1); __PYX_ERR(3, 40, __pyx_L3_error)
        }
      }
      if (unlikely(kw_args > 0)) {
        if (unlikely(__Pyx_ParseOptionalKeywords(__pyx_kwds, __pyx_pyargnames, 0, values, pos_args, "start_client_batch") < 0)) __PYX_ERR(3, 40, __pyx_L3_error)
      }
    } else if (PyTuple_GET_SIZE(__pyx_args) != 2) {
      goto __pyx_L5_argtuple_error;
    } else {
      values[0] = PyTuple_GET_ITEM(__pyx_args, 0);
      values[1] = PyTuple_GET_ITEM(__pyx_args, 1);
    }
    __pyx_v_operations = values[0];
    __pyx_v_tag = values[1];
  }
  goto __pyx_L4_argument_unpacking_done;
  __pyx_L5_argtuple_error:;
  __Pyx_RaiseArgtupleInvalid("start_client_batch", 1, 2, 2, PyTuple_GET_SIZE(__pyx_args)); __PYX_ERR(3, 40, __pyx_L3_error)
  __pyx_L3_error:;
  __Pyx_AddTraceback("grpc._cython.cygrpc.Call.start_client_batch", __pyx_clineno, __pyx_lineno, __pyx_filename);
  __Pyx_RefNannyFinishContext();
  return NULL;
  __pyx_L4_argument_unpacking_done:;
  __pyx_r = __pyx_pf_4grpc_7_cython_6cygrpc_4Call_4start_client_batch(((struct __pyx_obj_4grpc_7_cython_6cygrpc_Call *)__pyx_v_self), __pyx_v_operations, __pyx_v_tag);

  /* function exit code */
  __Pyx_RefNannyFinishContext();
  return __pyx_r;
}

static PyObject *__pyx_pf_4grpc_7_cython_6cygrpc_4Call_4start_client_batch(struct __pyx_obj_4grpc_7_cython_6cygrpc_Call *__pyx_v_self, PyObject *__pyx_v_operations, PyObject *__pyx_v_tag) {
  PyObject *__pyx_r = NULL;
  __Pyx_RefNannyDeclarations
  PyObject *__pyx_t_1 = NULL;
  PyObject *__pyx_t_2 = NULL;
  PyObject *__pyx_t_3 = NULL;
  int __pyx_t_4;
  PyObject *__pyx_t_5 = NULL;
  __Pyx_RefNannySetupContext("start_client_batch", 0);

  /* "src/python/grpcio/grpc/_cython/_cygrpc/call.pyx.pxi":43
 *     # We don't reference this call in the operations tag because
 *     # it should be cancelled when it goes out of scope
 *     return self._start_batch(operations, tag, False)             # <<<<<<<<<<<<<<
 * 
 *   def start_server_batch(self, operations, tag):
 */
  __Pyx_XDECREF(__pyx_r);
  __pyx_t_2 = __Pyx_PyObject_GetAttrStr(((PyObject *)__pyx_v_self), __pyx_n_s_start_batch); if (unlikely(!__pyx_t_2)) __PYX_ERR(3, 43, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_2);
  __pyx_t_3 = NULL;
  __pyx_t_4 = 0;
  if (CYTHON_UNPACK_METHODS && likely(PyMethod_Check(__pyx_t_2))) {
    __pyx_t_3 = PyMethod_GET_SELF(__pyx_t_2);
    if (likely(__pyx_t_3)) {
      PyObject* function = PyMethod_GET_FUNCTION(__pyx_t_2);
      __Pyx_INCREF(__pyx_t_3);
      __Pyx_INCREF(function);
      __Pyx_DECREF_SET(__pyx_t_2, function);
      __pyx_t_4 = 1;
    }
  }
  #if CYTHON_FAST_PYCALL
  if (PyFunction_Check(__pyx_t_2)) {
    PyObject *__pyx_temp[4] = {__pyx_t_3, __pyx_v_operations, __pyx_v_tag, Py_False};
    __pyx_t_1 = __Pyx_PyFunction_FastCall(__pyx_t_2, __pyx_temp+1-__pyx_t_4, 3+__pyx_t_4); if (unlikely(!__pyx_t_1)) __PYX_ERR(3, 43, __pyx_L1_error)
    __Pyx_XDECREF(__pyx_t_3); __pyx_t_3 = 0;
    __Pyx_GOTREF(__pyx_t_1);
  } else
  #endif
  #if CYTHON_FAST_PYCCALL
  if (__Pyx_PyFastCFunction_Check(__pyx_t_2)) {
    PyObject *__pyx_temp[4] = {__pyx_t_3, __pyx_v_operations, __pyx_v_tag, Py_False};
    __pyx_t_1 = __Pyx_PyCFunction_FastCall(__pyx_t_2, __pyx_temp+1-__pyx_t_4, 3+__pyx_t_4); if (unlikely(!__pyx_t_1)) __PYX_ERR(3, 43, __pyx_L1_error)
    __Pyx_XDECREF(__pyx_t_3); __pyx_t_3 = 0;
    __Pyx_GOTREF(__pyx_t_1);
  } else
  #endif
  {
    __pyx_t_5 = PyTuple_New(3+__pyx_t_4); if (unlikely(!__pyx_t_5)) __PYX_ERR(3, 43, __pyx_L1_error)
    __Pyx_GOTREF(__pyx_t_5);
    if (__pyx_t_3) {
      __Pyx_GIVEREF(__pyx_t_3); PyTuple_SET_ITEM(__pyx_t_5, 0, __pyx_t_3); __pyx_t_3 = NULL;
    }
    __Pyx_INCREF(__pyx_v_operations);
    __Pyx_GIVEREF(__pyx_v_operations);
    PyTuple_SET_ITEM(__pyx_t_5, 0+__pyx_t_4, __pyx_v_operations);
    __Pyx_INCREF(__pyx_v_tag);
    __Pyx_GIVEREF(__pyx_v_tag);
    PyTuple_SET_ITEM(__pyx_t_5, 1+__pyx_t_4, __pyx_v_tag);
    __Pyx_INCREF(Py_False);
    __Pyx_GIVEREF(Py_False);
    PyTuple_SET_ITEM(__pyx_t_5, 2+__pyx_t_4, Py_False);
    __pyx_t_1 = __Pyx_PyObject_Call(__pyx_t_2, __pyx_t_5, NULL); if (unlikely(!__pyx_t_1)) __PYX_ERR(3, 43, __pyx_L1_error)
    __Pyx_GOTREF(__pyx_t_1);
    __Pyx_DECREF(__pyx_t_5); __pyx_t_5 = 0;
  }
  __Pyx_DECREF(__pyx_t_2); __pyx_t_2 = 0;
  __pyx_r = __pyx_t_1;
  __pyx_t_1 = 0;
  goto __pyx_L0;

  /* "src/python/grpcio/grpc/_cython/_cygrpc/call.pyx.pxi":40
 *     return error
 * 
 *   def start_client_batch(self, operations, tag):             # <<<<<<<<<<<<<<
 *     # We don't reference this call in the operations tag because
 *     # it should be cancelled when it goes out of scope
 */

  /* function exit code */
  __pyx_L1_error:;
  __Pyx_XDECREF(__pyx_t_1);
  __Pyx_XDECREF(__pyx_t_2);
  __Pyx_XDECREF(__pyx_t_3);
  __Pyx_XDECREF(__pyx_t_5);
  __Pyx_AddTraceback("grpc._cython.cygrpc.Call.start_client_batch", __pyx_clineno, __pyx_lineno, __pyx_filename);
  __pyx_r = NULL;
  __pyx_L0:;
  __Pyx_XGIVEREF(__pyx_r);
  __Pyx_RefNannyFinishContext();
  return __pyx_r;
}

/* "src/python/grpcio/grpc/_cython/_cygrpc/call.pyx.pxi":45
 *     return self._start_batch(operations, tag, False)
 * 
 *   def start_server_batch(self, operations, tag):             # <<<<<<<<<<<<<<
 *     return self._start_batch(operations, tag, True)
 * 
 */

/* Python wrapper */
static PyObject *__pyx_pw_4grpc_7_cython_6cygrpc_4Call_7start_server_batch(PyObject *__pyx_v_self, PyObject *__pyx_args, PyObject *__pyx_kwds); /*proto*/
static PyObject *__pyx_pw_4grpc_7_cython_6cygrpc_4Call_7start_server_batch(PyObject *__pyx_v_self, PyObject *__pyx_args, PyObject *__pyx_kwds) {
  PyObject *__pyx_v_operations = 0;
  PyObject *__pyx_v_tag = 0;
  PyObject *__pyx_r = 0;
  __Pyx_RefNannyDeclarations
  __Pyx_RefNannySetupContext("start_server_batch (wrapper)", 0);
  {
    static PyObject **__pyx_pyargnames[] = {&__pyx_n_s_operations,&__pyx_n_s_tag,0};
    PyObject* values[2] = {0,0};
    if (unlikely(__pyx_kwds)) {
      Py_ssize_t kw_args;
      const Py_ssize_t pos_args = PyTuple_GET_SIZE(__pyx_args);
      switch (pos_args) {
        case  2: values[1] = PyTuple_GET_ITEM(__pyx_args, 1);
        CYTHON_FALLTHROUGH;
        case  1: values[0] = PyTuple_GET_ITEM(__pyx_args, 0);
        CYTHON_FALLTHROUGH;
        case  0: break;
        default: goto __pyx_L5_argtuple_error;
      }
      kw_args = PyDict_Size(__pyx_kwds);
      switch (pos_args) {
        case  0:
        if (likely((values[0] = __Pyx_PyDict_GetItemStr(__pyx_kwds, __pyx_n_s_operations)) != 0)) kw_args--;
        else goto __pyx_L5_argtuple_error;
        CYTHON_FALLTHROUGH;
        case  1:
        if (likely((values[1] = __Pyx_PyDict_GetItemStr(__pyx_kwds, __pyx_n_s_tag)) != 0)) kw_args--;
        else {
          __Pyx_RaiseArgtupleInvalid("start_server_batch", 1, 2, 2, 1); __PYX_ERR(3, 45, __pyx_L3_error)
        }
      }
      if (unlikely(kw_args > 0)) {
        if (unlikely(__Pyx_ParseOptionalKeywords(__pyx_kwds, __pyx_pyargnames, 0, values, pos_args, "start_server_batch") < 0)) __PYX_ERR(3, 45, __pyx_L3_error)
      }
    } else if (PyTuple_GET_SIZE(__pyx_args) != 2) {
      goto __pyx_L5_argtuple_error;
    } else {
      values[0] = PyTuple_GET_ITEM(__pyx_args, 0);
      values[1] = PyTuple_GET_ITEM(__pyx_args, 1);
    }
    __pyx_v_operations = values[0];
    __pyx_v_tag = values[1];
  }
  goto __pyx_L4_argument_unpacking_done;
  __pyx_L5_argtuple_error:;
  __Pyx_RaiseArgtupleInvalid("start_server_batch", 1, 2, 2, PyTuple_GET_SIZE(__pyx_args)); __PYX_ERR(3, 45, __pyx_L3_error)
  __pyx_L3_error:;
  __Pyx_AddTraceback("grpc._cython.cygrpc.Call.start_server_batch", __pyx_clineno, __pyx_lineno, __pyx_filename);
  __Pyx_RefNannyFinishContext();
  return NULL;
  __pyx_L4_argument_unpacking_done:;
  __pyx_r = __pyx_pf_4grpc_7_cython_6cygrpc_4Call_6start_server_batch(((struct __pyx_obj_4grpc_7_cython_6cygrpc_Call *)__pyx_v_self), __pyx_v_operations, __pyx_v_tag);

  /* function exit code */
  __Pyx_RefNannyFinishContext();
  return __pyx_r;
}

static PyObject *__pyx_pf_4grpc_7_cython_6cygrpc_4Call_6start_server_batch(struct __pyx_obj_4grpc_7_cython_6cygrpc_Call *__pyx_v_self, PyObject *__pyx_v_operations, PyObject *__pyx_v_tag) {
  PyObject *__pyx_r = NULL;
  __Pyx_RefNannyDeclarations
  PyObject *__pyx_t_1 = NULL;
  PyObject *__pyx_t_2 = NULL;
  PyObject *__pyx_t_3 = NULL;
  int __pyx_t_4;
  PyObject *__pyx_t_5 = NULL;
  __Pyx_RefNannySetupContext("start_server_batch", 0);

  /* "src/python/grpcio/grpc/_cython/_cygrpc/call.pyx.pxi":46
 * 
 *   def start_server_batch(self, operations, tag):
 *     return self._start_batch(operations, tag, True)             # <<<<<<<<<<<<<<
 * 
 *   def cancel(
 */
  __Pyx_XDECREF(__pyx_r);
  __pyx_t_2 = __Pyx_PyObject_GetAttrStr(((PyObject *)__pyx_v_self), __pyx_n_s_start_batch); if (unlikely(!__pyx_t_2)) __PYX_ERR(3, 46, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_2);
  __pyx_t_3 = NULL;
  __pyx_t_4 = 0;
  if (CYTHON_UNPACK_METHODS && likely(PyMethod_Check(__pyx_t_2))) {
    __pyx_t_3 = PyMethod_GET_SELF(__pyx_t_2);
    if (likely(__pyx_t_3)) {
      PyObject* function = PyMethod_GET_FUNCTION(__pyx_t_2);
      __Pyx_INCREF(__pyx_t_3);
      __Pyx_INCREF(function);
      __Pyx_DECREF_SET(__pyx_t_2, function);
      __pyx_t_4 = 1;
    }
  }
  #if CYTHON_FAST_PYCALL
  if (PyFunction_Check(__pyx_t_2)) {
    PyObject *__pyx_temp[4] = {__pyx_t_3, __pyx_v_operations, __pyx_v_tag, Py_True};
    __pyx_t_1 = __Pyx_PyFunction_FastCall(__pyx_t_2, __pyx_temp+1-__pyx_t_4, 3+__pyx_t_4); if (unlikely(!__pyx_t_1)) __PYX_ERR(3, 46, __pyx_L1_error)
    __Pyx_XDECREF(__pyx_t_3); __pyx_t_3 = 0;
    __Pyx_GOTREF(__pyx_t_1);
  } else
  #endif
  #if CYTHON_FAST_PYCCALL
  if (__Pyx_PyFastCFunction_Check(__pyx_t_2)) {
    PyObject *__pyx_temp[4] = {__pyx_t_3, __pyx_v_operations, __pyx_v_tag, Py_True};
    __pyx_t_1 = __Pyx_PyCFunction_FastCall(__pyx_t_2, __pyx_temp+1-__pyx_t_4, 3+__pyx_t_4); if (unlikely(!__pyx_t_1)) __PYX_ERR(3, 46, __pyx_L1_error)
    __Pyx_XDECREF(__pyx_t_3); __pyx_t_3 = 0;
    __Pyx_GOTREF(__pyx_t_1);
  } else
  #endif
  {
    __pyx_t_5 = PyTuple_New(3+__pyx_t_4); if (unlikely(!__pyx_t_5)) __PYX_ERR(3, 46, __pyx_L1_error)
    __Pyx_GOTREF(__pyx_t_5);
    if (__pyx_t_3) {
      __Pyx_GIVEREF(__pyx_t_3); PyTuple_SET_ITEM(__pyx_t_5, 0, __pyx_t_3); __pyx_t_3 = NULL;
    }
    __Pyx_INCREF(__pyx_v_operations);
    __Pyx_GIVEREF(__pyx_v_operations);
    PyTuple_SET_ITEM(__pyx_t_5, 0+__pyx_t_4, __pyx_v_operations);
    __Pyx_INCREF(__pyx_v_tag);
    __Pyx_GIVEREF(__pyx_v_tag);
    PyTuple_SET_ITEM(__pyx_t_5, 1+__pyx_t_4, __pyx_v_tag);
    __Pyx_INCREF(Py_True);
    __Pyx_GIVEREF(Py_True);
    PyTuple_SET_ITEM(__pyx_t_5, 2+__pyx_t_4, Py_True);
    __pyx_t_1 = __Pyx_PyObject_Call(__pyx_t_2, __pyx_t_5, NULL); if (unlikely(!__pyx_t_1)) __PYX_ERR(3, 46, __pyx_L1_error)
    __Pyx_GOTREF(__pyx_t_1);
    __Pyx_DECREF(__pyx_t_5); __pyx_t_5 = 0;
  }
  __Pyx_DECREF(__pyx_t_2); __pyx_t_2 = 0;
  __pyx_r = __pyx_t_1;
  __pyx_t_1 = 0;
  goto __pyx_L0;

  /* "src/python/grpcio/grpc/_cython/_cygrpc/call.pyx.pxi":45
 *     return self._start_batch(operations, tag, False)
 * 
 *   def start_server_batch(self, operations, tag):             # <<<<<<<<<<<<<<
 *     return self._start_batch(operations, tag, True)
 * 
 */

  /* function exit code */
  __pyx_L1_error:;
  __Pyx_XDECREF(__pyx_t_1);
  __Pyx_XDECREF(__pyx_t_2);
  __Pyx_XDECREF(__pyx_t_3);
  __Pyx_XDECREF(__pyx_t_5);
  __Pyx_AddTraceback("grpc._cython.cygrpc.Call.start_server_batch", __pyx_clineno, __pyx_lineno, __pyx_filename);
  __pyx_r = NULL;
  __pyx_L0:;
  __Pyx_XGIVEREF(__pyx_r);
  __Pyx_RefNannyFinishContext();
  return __pyx_r;
}

/* "src/python/grpcio/grpc/_cython/_cygrpc/call.pyx.pxi":48
 *     return self._start_batch(operations, tag, True)
 * 
 *   def cancel(             # <<<<<<<<<<<<<<
 *       self, grpc_status_code error_code=GRPC_STATUS__DO_NOT_USE,
 *       details=None):
 */

/* Python wrapper */
static PyObject *__pyx_pw_4grpc_7_cython_6cygrpc_4Call_9cancel(PyObject *__pyx_v_self, PyObject *__pyx_args, PyObject *__pyx_kwds); /*proto*/
static PyObject *__pyx_pw_4grpc_7_cython_6cygrpc_4Call_9cancel(PyObject *__pyx_v_self, PyObject *__pyx_args, PyObject *__pyx_kwds) {
  grpc_status_code __pyx_v_error_code;
  PyObject *__pyx_v_details = 0;
  PyObject *__pyx_r = 0;
  __Pyx_RefNannyDeclarations
  __Pyx_RefNannySetupContext("cancel (wrapper)", 0);
  {
    static PyObject **__pyx_pyargnames[] = {&__pyx_n_s_error_code,&__pyx_n_s_details,0};
    PyObject* values[2] = {0,0};

    /* "src/python/grpcio/grpc/_cython/_cygrpc/call.pyx.pxi":50
 *   def cancel(
 *       self, grpc_status_code error_code=GRPC_STATUS__DO_NOT_USE,
 *       details=None):             # <<<<<<<<<<<<<<
 *     details = str_to_bytes(details)
 *     if not self.is_valid:
 */
    values[1] = ((PyObject *)Py_None);
    if (unlikely(__pyx_kwds)) {
      Py_ssize_t kw_args;
      const Py_ssize_t pos_args = PyTuple_GET_SIZE(__pyx_args);
      switch (pos_args) {
        case  2: values[1] = PyTuple_GET_ITEM(__pyx_args, 1);
        CYTHON_FALLTHROUGH;
        case  1: values[0] = PyTuple_GET_ITEM(__pyx_args, 0);
        CYTHON_FALLTHROUGH;
        case  0: break;
        default: goto __pyx_L5_argtuple_error;
      }
      kw_args = PyDict_Size(__pyx_kwds);
      switch (pos_args) {
        case  0:
        if (kw_args > 0) {
          PyObject* value = __Pyx_PyDict_GetItemStr(__pyx_kwds, __pyx_n_s_error_code);
          if (value) { values[0] = value; kw_args--; }
        }
        CYTHON_FALLTHROUGH;
        case  1:
        if (kw_args > 0) {
          PyObject* value = __Pyx_PyDict_GetItemStr(__pyx_kwds, __pyx_n_s_details);
          if (value) { values[1] = value; kw_args--; }
        }
      }
      if (unlikely(kw_args > 0)) {
        if (unlikely(__Pyx_ParseOptionalKeywords(__pyx_kwds, __pyx_pyargnames, 0, values, pos_args, "cancel") < 0)) __PYX_ERR(3, 48, __pyx_L3_error)
      }
    } else {
      switch (PyTuple_GET_SIZE(__pyx_args)) {
        case  2: values[1] = PyTuple_GET_ITEM(__pyx_args, 1);
        CYTHON_FALLTHROUGH;
        case  1: values[0] = PyTuple_GET_ITEM(__pyx_args, 0);
        CYTHON_FALLTHROUGH;
        case  0: break;
        default: goto __pyx_L5_argtuple_error;
      }
    }
    if (values[0]) {
      __pyx_v_error_code = ((grpc_status_code)__Pyx_PyInt_As_grpc_status_code(values[0])); if (unlikely(PyErr_Occurred())) __PYX_ERR(3, 49, __pyx_L3_error)
    } else {
      __pyx_v_error_code = __pyx_k__11;
    }
    __pyx_v_details = values[1];
  }
  goto __pyx_L4_argument_unpacking_done;
  __pyx_L5_argtuple_error:;
  __Pyx_RaiseArgtupleInvalid("cancel", 0, 0, 2, PyTuple_GET_SIZE(__pyx_args)); __PYX_ERR(3, 48, __pyx_L3_error)
  __pyx_L3_error:;
  __Pyx_AddTraceback("grpc._cython.cygrpc.Call.cancel", __pyx_clineno, __pyx_lineno, __pyx_filename);
  __Pyx_RefNannyFinishContext();
  return NULL;
  __pyx_L4_argument_unpacking_done:;
  __pyx_r = __pyx_pf_4grpc_7_cython_6cygrpc_4Call_8cancel(((struct __pyx_obj_4grpc_7_cython_6cygrpc_Call *)__pyx_v_self), __pyx_v_error_code, __pyx_v_details);

  /* "src/python/grpcio/grpc/_cython/_cygrpc/call.pyx.pxi":48
 *     return self._start_batch(operations, tag, True)
 * 
 *   def cancel(             # <<<<<<<<<<<<<<
 *       self, grpc_status_code error_code=GRPC_STATUS__DO_NOT_USE,
 *       details=None):
 */

  /* function exit code */
  __Pyx_RefNannyFinishContext();
  return __pyx_r;
}

static PyObject *__pyx_pf_4grpc_7_cython_6cygrpc_4Call_8cancel(struct __pyx_obj_4grpc_7_cython_6cygrpc_Call *__pyx_v_self, grpc_status_code __pyx_v_error_code, PyObject *__pyx_v_details) {
  grpc_call_error __pyx_v_result;
  char *__pyx_v_c_details;
  PyObject *__pyx_r = NULL;
  __Pyx_RefNannyDeclarations
  PyObject *__pyx_t_1 = NULL;
  int __pyx_t_2;
  int __pyx_t_3;
  int __pyx_t_4;
  char *__pyx_t_5;
  __Pyx_RefNannySetupContext("cancel", 0);
  __Pyx_INCREF(__pyx_v_details);

  /* "src/python/grpcio/grpc/_cython/_cygrpc/call.pyx.pxi":51
 *       self, grpc_status_code error_code=GRPC_STATUS__DO_NOT_USE,
 *       details=None):
 *     details = str_to_bytes(details)             # <<<<<<<<<<<<<<
 *     if not self.is_valid:
 *       raise ValueError("invalid call object cannot be used from Python")
 */
  __pyx_t_1 = __pyx_f_4grpc_7_cython_6cygrpc_str_to_bytes(__pyx_v_details); if (unlikely(!__pyx_t_1)) __PYX_ERR(3, 51, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_1);
  __Pyx_DECREF_SET(__pyx_v_details, __pyx_t_1);
  __pyx_t_1 = 0;

  /* "src/python/grpcio/grpc/_cython/_cygrpc/call.pyx.pxi":52
 *       details=None):
 *     details = str_to_bytes(details)
 *     if not self.is_valid:             # <<<<<<<<<<<<<<
 *       raise ValueError("invalid call object cannot be used from Python")
 *     if (details is None) != (error_code == GRPC_STATUS__DO_NOT_USE):
 */
  __pyx_t_1 = __Pyx_PyObject_GetAttrStr(((PyObject *)__pyx_v_self), __pyx_n_s_is_valid); if (unlikely(!__pyx_t_1)) __PYX_ERR(3, 52, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_1);
  __pyx_t_2 = __Pyx_PyObject_IsTrue(__pyx_t_1); if (unlikely(__pyx_t_2 < 0)) __PYX_ERR(3, 52, __pyx_L1_error)
  __Pyx_DECREF(__pyx_t_1); __pyx_t_1 = 0;
  __pyx_t_3 = ((!__pyx_t_2) != 0);
  if (unlikely(__pyx_t_3)) {

    /* "src/python/grpcio/grpc/_cython/_cygrpc/call.pyx.pxi":53
 *     details = str_to_bytes(details)
 *     if not self.is_valid:
 *       raise ValueError("invalid call object cannot be used from Python")             # <<<<<<<<<<<<<<
 *     if (details is None) != (error_code == GRPC_STATUS__DO_NOT_USE):
 *       raise ValueError("if error_code is specified, so must details "
 */
    __pyx_t_1 = __Pyx_PyObject_Call(__pyx_builtin_ValueError, __pyx_tuple__12, NULL); if (unlikely(!__pyx_t_1)) __PYX_ERR(3, 53, __pyx_L1_error)
    __Pyx_GOTREF(__pyx_t_1);
    __Pyx_Raise(__pyx_t_1, 0, 0, 0);
    __Pyx_DECREF(__pyx_t_1); __pyx_t_1 = 0;
    __PYX_ERR(3, 53, __pyx_L1_error)

    /* "src/python/grpcio/grpc/_cython/_cygrpc/call.pyx.pxi":52
 *       details=None):
 *     details = str_to_bytes(details)
 *     if not self.is_valid:             # <<<<<<<<<<<<<<
 *       raise ValueError("invalid call object cannot be used from Python")
 *     if (details is None) != (error_code == GRPC_STATUS__DO_NOT_USE):
 */
  }

  /* "src/python/grpcio/grpc/_cython/_cygrpc/call.pyx.pxi":54
 *     if not self.is_valid:
 *       raise ValueError("invalid call object cannot be used from Python")
 *     if (details is None) != (error_code == GRPC_STATUS__DO_NOT_USE):             # <<<<<<<<<<<<<<
 *       raise ValueError("if error_code is specified, so must details "
 *                        "(and vice-versa)")
 */
  __pyx_t_3 = (__pyx_v_details == Py_None);
  __pyx_t_2 = ((__pyx_t_3 != (__pyx_v_error_code == GRPC_STATUS__DO_NOT_USE)) != 0);
  if (unlikely(__pyx_t_2)) {

    /* "src/python/grpcio/grpc/_cython/_cygrpc/call.pyx.pxi":55
 *       raise ValueError("invalid call object cannot be used from Python")
 *     if (details is None) != (error_code == GRPC_STATUS__DO_NOT_USE):
 *       raise ValueError("if error_code is specified, so must details "             # <<<<<<<<<<<<<<
 *                        "(and vice-versa)")
 *     cdef grpc_call_error result
 */
    __pyx_t_1 = __Pyx_PyObject_Call(__pyx_builtin_ValueError, __pyx_tuple__13, NULL); if (unlikely(!__pyx_t_1)) __PYX_ERR(3, 55, __pyx_L1_error)
    __Pyx_GOTREF(__pyx_t_1);
    __Pyx_Raise(__pyx_t_1, 0, 0, 0);
    __Pyx_DECREF(__pyx_t_1); __pyx_t_1 = 0;
    __PYX_ERR(3, 55, __pyx_L1_error)

    /* "src/python/grpcio/grpc/_cython/_cygrpc/call.pyx.pxi":54
 *     if not self.is_valid:
 *       raise ValueError("invalid call object cannot be used from Python")
 *     if (details is None) != (error_code == GRPC_STATUS__DO_NOT_USE):             # <<<<<<<<<<<<<<
 *       raise ValueError("if error_code is specified, so must details "
 *                        "(and vice-versa)")
 */
  }

  /* "src/python/grpcio/grpc/_cython/_cygrpc/call.pyx.pxi":58
 *                        "(and vice-versa)")
 *     cdef grpc_call_error result
 *     cdef char *c_details = NULL             # <<<<<<<<<<<<<<
 *     if error_code != GRPC_STATUS__DO_NOT_USE:
 *       self.references.append(details)
 */
  __pyx_v_c_details = NULL;

  /* "src/python/grpcio/grpc/_cython/_cygrpc/call.pyx.pxi":59
 *     cdef grpc_call_error result
 *     cdef char *c_details = NULL
 *     if error_code != GRPC_STATUS__DO_NOT_USE:             # <<<<<<<<<<<<<<
 *       self.references.append(details)
 *       c_details = details
 */
  __pyx_t_2 = ((__pyx_v_error_code != GRPC_STATUS__DO_NOT_USE) != 0);
  if (__pyx_t_2) {

    /* "src/python/grpcio/grpc/_cython/_cygrpc/call.pyx.pxi":60
 *     cdef char *c_details = NULL
 *     if error_code != GRPC_STATUS__DO_NOT_USE:
 *       self.references.append(details)             # <<<<<<<<<<<<<<
 *       c_details = details
 *       with nogil:
 */
    if (unlikely(__pyx_v_self->references == Py_None)) {
      PyErr_Format(PyExc_AttributeError, "'NoneType' object has no attribute '%.30s'", "append");
      __PYX_ERR(3, 60, __pyx_L1_error)
    }
    __pyx_t_4 = __Pyx_PyList_Append(__pyx_v_self->references, __pyx_v_details); if (unlikely(__pyx_t_4 == ((int)-1))) __PYX_ERR(3, 60, __pyx_L1_error)

    /* "src/python/grpcio/grpc/_cython/_cygrpc/call.pyx.pxi":61
 *     if error_code != GRPC_STATUS__DO_NOT_USE:
 *       self.references.append(details)
 *       c_details = details             # <<<<<<<<<<<<<<
 *       with nogil:
 *         result = grpc_call_cancel_with_status(
 */
    __pyx_t_5 = __Pyx_PyObject_AsWritableString(__pyx_v_details); if (unlikely((!__pyx_t_5) && PyErr_Occurred())) __PYX_ERR(3, 61, __pyx_L1_error)
    __pyx_v_c_details = __pyx_t_5;

    /* "src/python/grpcio/grpc/_cython/_cygrpc/call.pyx.pxi":62
 *       self.references.append(details)
 *       c_details = details
 *       with nogil:             # <<<<<<<<<<<<<<
 *         result = grpc_call_cancel_with_status(
 *             self.c_call, error_code, c_details, NULL)
 */
    {
        #ifdef WITH_THREAD
        PyThreadState *_save;
        Py_UNBLOCK_THREADS
        __Pyx_FastGIL_Remember();
        #endif
        /*try:*/ {

          /* "src/python/grpcio/grpc/_cython/_cygrpc/call.pyx.pxi":63
 *       c_details = details
 *       with nogil:
 *         result = grpc_call_cancel_with_status(             # <<<<<<<<<<<<<<
 *             self.c_call, error_code, c_details, NULL)
 *       return result
 */
          __pyx_v_result = grpc_call_cancel_with_status(__pyx_v_self->c_call, __pyx_v_error_code, __pyx_v_c_details, NULL);
        }

        /* "src/python/grpcio/grpc/_cython/_cygrpc/call.pyx.pxi":62
 *       self.references.append(details)
 *       c_details = details
 *       with nogil:             # <<<<<<<<<<<<<<
 *         result = grpc_call_cancel_with_status(
 *             self.c_call, error_code, c_details, NULL)
 */
        /*finally:*/ {
          /*normal exit:*/{
            #ifdef WITH_THREAD
            __Pyx_FastGIL_Forget();
            Py_BLOCK_THREADS
            #endif
            goto __pyx_L8;
          }
          __pyx_L8:;
        }
    }

    /* "src/python/grpcio/grpc/_cython/_cygrpc/call.pyx.pxi":65
 *         result = grpc_call_cancel_with_status(
 *             self.c_call, error_code, c_details, NULL)
 *       return result             # <<<<<<<<<<<<<<
 *     else:
 *       with nogil:
 */
    __Pyx_XDECREF(__pyx_r);
    __pyx_t_1 = __Pyx_PyInt_From_grpc_call_error(__pyx_v_result); if (unlikely(!__pyx_t_1)) __PYX_ERR(3, 65, __pyx_L1_error)
    __Pyx_GOTREF(__pyx_t_1);
    __pyx_r = __pyx_t_1;
    __pyx_t_1 = 0;
    goto __pyx_L0;

    /* "src/python/grpcio/grpc/_cython/_cygrpc/call.pyx.pxi":59
 *     cdef grpc_call_error result
 *     cdef char *c_details = NULL
 *     if error_code != GRPC_STATUS__DO_NOT_USE:             # <<<<<<<<<<<<<<
 *       self.references.append(details)
 *       c_details = details
 */
  }

  /* "src/python/grpcio/grpc/_cython/_cygrpc/call.pyx.pxi":67
 *       return result
 *     else:
 *       with nogil:             # <<<<<<<<<<<<<<
 *         result = grpc_call_cancel(self.c_call, NULL)
 *       return result
 */
  /*else*/ {
    {
        #ifdef WITH_THREAD
        PyThreadState *_save;
        Py_UNBLOCK_THREADS
        __Pyx_FastGIL_Remember();
        #endif
        /*try:*/ {

          /* "src/python/grpcio/grpc/_cython/_cygrpc/call.pyx.pxi":68
 *     else:
 *       with nogil:
 *         result = grpc_call_cancel(self.c_call, NULL)             # <<<<<<<<<<<<<<
 *       return result
 * 
 */
          __pyx_v_result = grpc_call_cancel(__pyx_v_self->c_call, NULL);
        }

        /* "src/python/grpcio/grpc/_cython/_cygrpc/call.pyx.pxi":67
 *       return result
 *     else:
 *       with nogil:             # <<<<<<<<<<<<<<
 *         result = grpc_call_cancel(self.c_call, NULL)
 *       return result
 */
        /*finally:*/ {
          /*normal exit:*/{
            #ifdef WITH_THREAD
            __Pyx_FastGIL_Forget();
            Py_BLOCK_THREADS
            #endif
            goto __pyx_L11;
          }
          __pyx_L11:;
        }
    }

    /* "src/python/grpcio/grpc/_cython/_cygrpc/call.pyx.pxi":69
 *       with nogil:
 *         result = grpc_call_cancel(self.c_call, NULL)
 *       return result             # <<<<<<<<<<<<<<
 * 
 *   def set_credentials(self, CallCredentials call_credentials not None):
 */
    __Pyx_XDECREF(__pyx_r);
    __pyx_t_1 = __Pyx_PyInt_From_grpc_call_error(__pyx_v_result); if (unlikely(!__pyx_t_1)) __PYX_ERR(3, 69, __pyx_L1_error)
    __Pyx_GOTREF(__pyx_t_1);
    __pyx_r = __pyx_t_1;
    __pyx_t_1 = 0;
    goto __pyx_L0;
  }

  /* "src/python/grpcio/grpc/_cython/_cygrpc/call.pyx.pxi":48
 *     return self._start_batch(operations, tag, True)
 * 
 *   def cancel(             # <<<<<<<<<<<<<<
 *       self, grpc_status_code error_code=GRPC_STATUS__DO_NOT_USE,
 *       details=None):
 */

  /* function exit code */
  __pyx_L1_error:;
  __Pyx_XDECREF(__pyx_t_1);
  __Pyx_AddTraceback("grpc._cython.cygrpc.Call.cancel", __pyx_clineno, __pyx_lineno, __pyx_filename);
  __pyx_r = NULL;
  __pyx_L0:;
  __Pyx_XDECREF(__pyx_v_details);
  __Pyx_XGIVEREF(__pyx_r);
  __Pyx_RefNannyFinishContext();
  return __pyx_r;
}

/* "src/python/grpcio/grpc/_cython/_cygrpc/call.pyx.pxi":71
 *       return result
 * 
 *   def set_credentials(self, CallCredentials call_credentials not None):             # <<<<<<<<<<<<<<
 *     cdef grpc_call_credentials *c_call_credentials = call_credentials.c()
 *     cdef grpc_call_error call_error = grpc_call_set_credentials(
 */

/* Python wrapper */
static PyObject *__pyx_pw_4grpc_7_cython_6cygrpc_4Call_11set_credentials(PyObject *__pyx_v_self, PyObject *__pyx_v_call_credentials); /*proto*/
static PyObject *__pyx_pw_4grpc_7_cython_6cygrpc_4Call_11set_credentials(PyObject *__pyx_v_self, PyObject *__pyx_v_call_credentials) {
  PyObject *__pyx_r = 0;
  __Pyx_RefNannyDeclarations
  __Pyx_RefNannySetupContext("set_credentials (wrapper)", 0);
  if (unlikely(!__Pyx_ArgTypeTest(((PyObject *)__pyx_v_call_credentials), __pyx_ptype_4grpc_7_cython_6cygrpc_CallCredentials, 0, "call_credentials", 0))) __PYX_ERR(3, 71, __pyx_L1_error)
  __pyx_r = __pyx_pf_4grpc_7_cython_6cygrpc_4Call_10set_credentials(((struct __pyx_obj_4grpc_7_cython_6cygrpc_Call *)__pyx_v_self), ((struct __pyx_obj_4grpc_7_cython_6cygrpc_CallCredentials *)__pyx_v_call_credentials));

  /* function exit code */
  goto __pyx_L0;
  __pyx_L1_error:;
  __pyx_r = NULL;
  __pyx_L0:;
  __Pyx_RefNannyFinishContext();
  return __pyx_r;
}

static PyObject *__pyx_pf_4grpc_7_cython_6cygrpc_4Call_10set_credentials(struct __pyx_obj_4grpc_7_cython_6cygrpc_Call *__pyx_v_self, struct __pyx_obj_4grpc_7_cython_6cygrpc_CallCredentials *__pyx_v_call_credentials) {
  grpc_call_credentials *__pyx_v_c_call_credentials;
  grpc_call_error __pyx_v_call_error;
  PyObject *__pyx_r = NULL;
  __Pyx_RefNannyDeclarations
  grpc_call_credentials *__pyx_t_1;
  PyObject *__pyx_t_2 = NULL;
  __Pyx_RefNannySetupContext("set_credentials", 0);

  /* "src/python/grpcio/grpc/_cython/_cygrpc/call.pyx.pxi":72
 * 
 *   def set_credentials(self, CallCredentials call_credentials not None):
 *     cdef grpc_call_credentials *c_call_credentials = call_credentials.c()             # <<<<<<<<<<<<<<
 *     cdef grpc_call_error call_error = grpc_call_set_credentials(
 *         self.c_call, c_call_credentials)
 */
  __pyx_t_1 = ((struct __pyx_vtabstruct_4grpc_7_cython_6cygrpc_CallCredentials *)__pyx_v_call_credentials->__pyx_vtab)->c(__pyx_v_call_credentials); if (unlikely(PyErr_Occurred())) __PYX_ERR(3, 72, __pyx_L1_error)
  __pyx_v_c_call_credentials = __pyx_t_1;

  /* "src/python/grpcio/grpc/_cython/_cygrpc/call.pyx.pxi":73
 *   def set_credentials(self, CallCredentials call_credentials not None):
 *     cdef grpc_call_credentials *c_call_credentials = call_credentials.c()
 *     cdef grpc_call_error call_error = grpc_call_set_credentials(             # <<<<<<<<<<<<<<
 *         self.c_call, c_call_credentials)
 *     grpc_call_credentials_release(c_call_credentials)
 */
  __pyx_v_call_error = grpc_call_set_credentials(__pyx_v_self->c_call, __pyx_v_c_call_credentials);

  /* "src/python/grpcio/grpc/_cython/_cygrpc/call.pyx.pxi":75
 *     cdef grpc_call_error call_error = grpc_call_set_credentials(
 *         self.c_call, c_call_credentials)
 *     grpc_call_credentials_release(c_call_credentials)             # <<<<<<<<<<<<<<
 *     return call_error
 * 
 */
  grpc_call_credentials_release(__pyx_v_c_call_credentials);

  /* "src/python/grpcio/grpc/_cython/_cygrpc/call.pyx.pxi":76
 *         self.c_call, c_call_credentials)
 *     grpc_call_credentials_release(c_call_credentials)
 *     return call_error             # <<<<<<<<<<<<<<
 * 
 *   def peer(self):
 */
  __Pyx_XDECREF(__pyx_r);
  __pyx_t_2 = __Pyx_PyInt_From_grpc_call_error(__pyx_v_call_error); if (unlikely(!__pyx_t_2)) __PYX_ERR(3, 76, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_2);
  __pyx_r = __pyx_t_2;
  __pyx_t_2 = 0;
  goto __pyx_L0;

  /* "src/python/grpcio/grpc/_cython/_cygrpc/call.pyx.pxi":71
 *       return result
 * 
 *   def set_credentials(self, CallCredentials call_credentials not None):             # <<<<<<<<<<<<<<
 *     cdef grpc_call_credentials *c_call_credentials = call_credentials.c()
 *     cdef grpc_call_error call_error = grpc_call_set_credentials(
 */

  /* function exit code */
  __pyx_L1_error:;
  __Pyx_XDECREF(__pyx_t_2);
  __Pyx_AddTraceback("grpc._cython.cygrpc.Call.set_credentials", __pyx_clineno, __pyx_lineno, __pyx_filename);
  __pyx_r = NULL;
  __pyx_L0:;
  __Pyx_XGIVEREF(__pyx_r);
  __Pyx_RefNannyFinishContext();
  return __pyx_r;
}

/* "src/python/grpcio/grpc/_cython/_cygrpc/call.pyx.pxi":78
 *     return call_error
 * 
 *   def peer(self):             # <<<<<<<<<<<<<<
 *     cdef char *peer = NULL
 *     with nogil:
 */

/* Python wrapper */
static PyObject *__pyx_pw_4grpc_7_cython_6cygrpc_4Call_13peer(PyObject *__pyx_v_self, CYTHON_UNUSED PyObject *unused); /*proto*/
static PyObject *__pyx_pw_4grpc_7_cython_6cygrpc_4Call_13peer(PyObject *__pyx_v_self, CYTHON_UNUSED PyObject *unused) {
  PyObject *__pyx_r = 0;
  __Pyx_RefNannyDeclarations
  __Pyx_RefNannySetupContext("peer (wrapper)", 0);
  __pyx_r = __pyx_pf_4grpc_7_cython_6cygrpc_4Call_12peer(((struct __pyx_obj_4grpc_7_cython_6cygrpc_Call *)__pyx_v_self));

  /* function exit code */
  __Pyx_RefNannyFinishContext();
  return __pyx_r;
}

static PyObject *__pyx_pf_4grpc_7_cython_6cygrpc_4Call_12peer(struct __pyx_obj_4grpc_7_cython_6cygrpc_Call *__pyx_v_self) {
  char *__pyx_v_peer;
  PyObject *__pyx_v_result = NULL;
  PyObject *__pyx_r = NULL;
  __Pyx_RefNannyDeclarations
  PyObject *__pyx_t_1 = NULL;
  PyObject *__pyx_t_2 = NULL;
  __Pyx_RefNannySetupContext("peer", 0);

  /* "src/python/grpcio/grpc/_cython/_cygrpc/call.pyx.pxi":79
 * 
 *   def peer(self):
 *     cdef char *peer = NULL             # <<<<<<<<<<<<<<
 *     with nogil:
 *       peer = grpc_call_get_peer(self.c_call)
 */
  __pyx_v_peer = NULL;

  /* "src/python/grpcio/grpc/_cython/_cygrpc/call.pyx.pxi":80
 *   def peer(self):
 *     cdef char *peer = NULL
 *     with nogil:             # <<<<<<<<<<<<<<
 *       peer = grpc_call_get_peer(self.c_call)
 *     result = <bytes>peer
 */
  {
      #ifdef WITH_THREAD
      PyThreadState *_save;
      Py_UNBLOCK_THREADS
      __Pyx_FastGIL_Remember();
      #endif
      /*try:*/ {

        /* "src/python/grpcio/grpc/_cython/_cygrpc/call.pyx.pxi":81
 *     cdef char *peer = NULL
 *     with nogil:
 *       peer = grpc_call_get_peer(self.c_call)             # <<<<<<<<<<<<<<
 *     result = <bytes>peer
 *     with nogil:
 */
        __pyx_v_peer = grpc_call_get_peer(__pyx_v_self->c_call);
      }

      /* "src/python/grpcio/grpc/_cython/_cygrpc/call.pyx.pxi":80
 *   def peer(self):
 *     cdef char *peer = NULL
 *     with nogil:             # <<<<<<<<<<<<<<
 *       peer = grpc_call_get_peer(self.c_call)
 *     result = <bytes>peer
 */
      /*finally:*/ {
        /*normal exit:*/{
          #ifdef WITH_THREAD
          __Pyx_FastGIL_Forget();
          Py_BLOCK_THREADS
          #endif
          goto __pyx_L5;
        }
        __pyx_L5:;
      }
  }

  /* "src/python/grpcio/grpc/_cython/_cygrpc/call.pyx.pxi":82
 *     with nogil:
 *       peer = grpc_call_get_peer(self.c_call)
 *     result = <bytes>peer             # <<<<<<<<<<<<<<
 *     with nogil:
 *       gpr_free(peer)
 */
  __pyx_t_1 = __Pyx_PyBytes_FromString(__pyx_v_peer); if (unlikely(!__pyx_t_1)) __PYX_ERR(3, 82, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_1);
  __pyx_t_2 = __pyx_t_1;
  __Pyx_INCREF(__pyx_t_2);
  __Pyx_DECREF(__pyx_t_1); __pyx_t_1 = 0;
  __pyx_v_result = ((PyObject*)__pyx_t_2);
  __pyx_t_2 = 0;

  /* "src/python/grpcio/grpc/_cython/_cygrpc/call.pyx.pxi":83
 *       peer = grpc_call_get_peer(self.c_call)
 *     result = <bytes>peer
 *     with nogil:             # <<<<<<<<<<<<<<
 *       gpr_free(peer)
 *     return result
 */
  {
      #ifdef WITH_THREAD
      PyThreadState *_save;
      Py_UNBLOCK_THREADS
      __Pyx_FastGIL_Remember();
      #endif
      /*try:*/ {

        /* "src/python/grpcio/grpc/_cython/_cygrpc/call.pyx.pxi":84
 *     result = <bytes>peer
 *     with nogil:
 *       gpr_free(peer)             # <<<<<<<<<<<<<<
 *     return result
 * 
 */
        gpr_free(__pyx_v_peer);
      }

      /* "src/python/grpcio/grpc/_cython/_cygrpc/call.pyx.pxi":83
 *       peer = grpc_call_get_peer(self.c_call)
 *     result = <bytes>peer
 *     with nogil:             # <<<<<<<<<<<<<<
 *       gpr_free(peer)
 *     return result
 */
      /*finally:*/ {
        /*normal exit:*/{
          #ifdef WITH_THREAD
          __Pyx_FastGIL_Forget();
          Py_BLOCK_THREADS
          #endif
          goto __pyx_L8;
        }
        __pyx_L8:;
      }
  }

  /* "src/python/grpcio/grpc/_cython/_cygrpc/call.pyx.pxi":85
 *     with nogil:
 *       gpr_free(peer)
 *     return result             # <<<<<<<<<<<<<<
 * 
 *   def __dealloc__(self):
 */
  __Pyx_XDECREF(__pyx_r);
  __Pyx_INCREF(__pyx_v_result);
  __pyx_r = __pyx_v_result;
  goto __pyx_L0;

  /* "src/python/grpcio/grpc/_cython/_cygrpc/call.pyx.pxi":78
 *     return call_error
 * 
 *   def peer(self):             # <<<<<<<<<<<<<<
 *     cdef char *peer = NULL
 *     with nogil:
 */

  /* function exit code */
  __pyx_L1_error:;
  __Pyx_XDECREF(__pyx_t_1);
  __Pyx_XDECREF(__pyx_t_2);
  __Pyx_AddTraceback("grpc._cython.cygrpc.Call.peer", __pyx_clineno, __pyx_lineno, __pyx_filename);
  __pyx_r = NULL;
  __pyx_L0:;
  __Pyx_XDECREF(__pyx_v_result);
  __Pyx_XGIVEREF(__pyx_r);
  __Pyx_RefNannyFinishContext();
  return __pyx_r;
}

/* "src/python/grpcio/grpc/_cython/_cygrpc/call.pyx.pxi":87
 *     return result
 * 
 *   def __dealloc__(self):             # <<<<<<<<<<<<<<
 *     with nogil:
 *       if self.c_call != NULL:
 */

/* Python wrapper */
static void __pyx_pw_4grpc_7_cython_6cygrpc_4Call_15__dealloc__(PyObject *__pyx_v_self); /*proto*/
static void __pyx_pw_4grpc_7_cython_6cygrpc_4Call_15__dealloc__(PyObject *__pyx_v_self) {
  __Pyx_RefNannyDeclarations
  __Pyx_RefNannySetupContext("__dealloc__ (wrapper)", 0);
  __pyx_pf_4grpc_7_cython_6cygrpc_4Call_14__dealloc__(((struct __pyx_obj_4grpc_7_cython_6cygrpc_Call *)__pyx_v_self));

  /* function exit code */
  __Pyx_RefNannyFinishContext();
}

static void __pyx_pf_4grpc_7_cython_6cygrpc_4Call_14__dealloc__(struct __pyx_obj_4grpc_7_cython_6cygrpc_Call *__pyx_v_self) {
  __Pyx_RefNannyDeclarations
  int __pyx_t_1;
  __Pyx_RefNannySetupContext("__dealloc__", 0);

  /* "src/python/grpcio/grpc/_cython/_cygrpc/call.pyx.pxi":88
 * 
 *   def __dealloc__(self):
 *     with nogil:             # <<<<<<<<<<<<<<
 *       if self.c_call != NULL:
 *         grpc_call_unref(self.c_call)
 */
  {
      #ifdef WITH_THREAD
      PyThreadState *_save;
      Py_UNBLOCK_THREADS
      __Pyx_FastGIL_Remember();
      #endif
      /*try:*/ {

        /* "src/python/grpcio/grpc/_cython/_cygrpc/call.pyx.pxi":89
 *   def __dealloc__(self):
 *     with nogil:
 *       if self.c_call != NULL:             # <<<<<<<<<<<<<<
 *         grpc_call_unref(self.c_call)
 *       grpc_shutdown_blocking()
 */
        __pyx_t_1 = ((__pyx_v_self->c_call != NULL) != 0);
        if (__pyx_t_1) {

          /* "src/python/grpcio/grpc/_cython/_cygrpc/call.pyx.pxi":90
 *     with nogil:
 *       if self.c_call != NULL:
 *         grpc_call_unref(self.c_call)             # <<<<<<<<<<<<<<
 *       grpc_shutdown_blocking()
 * 
 */
          grpc_call_unref(__pyx_v_self->c_call);

          /* "src/python/grpcio/grpc/_cython/_cygrpc/call.pyx.pxi":89
 *   def __dealloc__(self):
 *     with nogil:
 *       if self.c_call != NULL:             # <<<<<<<<<<<<<<
 *         grpc_call_unref(self.c_call)
 *       grpc_shutdown_blocking()
 */
        }

        /* "src/python/grpcio/grpc/_cython/_cygrpc/call.pyx.pxi":91
 *       if self.c_call != NULL:
 *         grpc_call_unref(self.c_call)
 *       grpc_shutdown_blocking()             # <<<<<<<<<<<<<<
 * 
 *   # The object *should* always be valid from Python. Used for debugging.
 */
        grpc_shutdown_blocking();
      }

      /* "src/python/grpcio/grpc/_cython/_cygrpc/call.pyx.pxi":88
 * 
 *   def __dealloc__(self):
 *     with nogil:             # <<<<<<<<<<<<<<
 *       if self.c_call != NULL:
 *         grpc_call_unref(self.c_call)
 */
      /*finally:*/ {
        /*normal exit:*/{
          #ifdef WITH_THREAD
          __Pyx_FastGIL_Forget();
          Py_BLOCK_THREADS
          #endif
          goto __pyx_L5;
        }
        __pyx_L5:;
      }
  }

  /* "src/python/grpcio/grpc/_cython/_cygrpc/call.pyx.pxi":87
 *     return result
 * 
 *   def __dealloc__(self):             # <<<<<<<<<<<<<<
 *     with nogil:
 *       if self.c_call != NULL:
 */

  /* function exit code */
  __Pyx_RefNannyFinishContext();
}

/* "src/python/grpcio/grpc/_cython/_cygrpc/call.pyx.pxi":95
 *   # The object *should* always be valid from Python. Used for debugging.
 *   @property
 *   def is_valid(self):             # <<<<<<<<<<<<<<
 *     return self.c_call != NULL
 * 
 */

/* Python wrapper */
static PyObject *__pyx_pw_4grpc_7_cython_6cygrpc_4Call_8is_valid_1__get__(PyObject *__pyx_v_self); /*proto*/
static PyObject *__pyx_pw_4grpc_7_cython_6cygrpc_4Call_8is_valid_1__get__(PyObject *__pyx_v_self) {
  PyObject *__pyx_r = 0;
  __Pyx_RefNannyDeclarations
  __Pyx_RefNannySetupContext("__get__ (wrapper)", 0);
  __pyx_r = __pyx_pf_4grpc_7_cython_6cygrpc_4Call_8is_valid___get__(((struct __pyx_obj_4grpc_7_cython_6cygrpc_Call *)__pyx_v_self));

  /* function exit code */
  __Pyx_RefNannyFinishContext();
  return __pyx_r;
}

static PyObject *__pyx_pf_4grpc_7_cython_6cygrpc_4Call_8is_valid___get__(struct __pyx_obj_4grpc_7_cython_6cygrpc_Call *__pyx_v_self) {
  PyObject *__pyx_r = NULL;
  __Pyx_RefNannyDeclarations
  PyObject *__pyx_t_1 = NULL;
  __Pyx_RefNannySetupContext("__get__", 0);

  /* "src/python/grpcio/grpc/_cython/_cygrpc/call.pyx.pxi":96
 *   @property
 *   def is_valid(self):
 *     return self.c_call != NULL             # <<<<<<<<<<<<<<
 * 
 *   def _custom_op_on_c_call(self, int op):
 */
  __Pyx_XDECREF(__pyx_r);
  __pyx_t_1 = __Pyx_PyBool_FromLong((__pyx_v_self->c_call != NULL)); if (unlikely(!__pyx_t_1)) __PYX_ERR(3, 96, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_1);
  __pyx_r = __pyx_t_1;
  __pyx_t_1 = 0;
  goto __pyx_L0;

  /* "src/python/grpcio/grpc/_cython/_cygrpc/call.pyx.pxi":95
 *   # The object *should* always be valid from Python. Used for debugging.
 *   @property
 *   def is_valid(self):             # <<<<<<<<<<<<<<
 *     return self.c_call != NULL
 * 
 */

  /* function exit code */
  __pyx_L1_error:;
  __Pyx_XDECREF(__pyx_t_1);
  __Pyx_AddTraceback("grpc._cython.cygrpc.Call.is_valid.__get__", __pyx_clineno, __pyx_lineno, __pyx_filename);
  __pyx_r = NULL;
  __pyx_L0:;
  __Pyx_XGIVEREF(__pyx_r);
  __Pyx_RefNannyFinishContext();
  return __pyx_r;
}

/* "src/python/grpcio/grpc/_cython/_cygrpc/call.pyx.pxi":98
 *     return self.c_call != NULL
 * 
 *   def _custom_op_on_c_call(self, int op):             # <<<<<<<<<<<<<<
 *     return _custom_op_on_c_call(op, self.c_call)
 */

/* Python wrapper */
static PyObject *__pyx_pw_4grpc_7_cython_6cygrpc_4Call_17_custom_op_on_c_call(PyObject *__pyx_v_self, PyObject *__pyx_arg_op); /*proto*/
static PyObject *__pyx_pw_4grpc_7_cython_6cygrpc_4Call_17_custom_op_on_c_call(PyObject *__pyx_v_self, PyObject *__pyx_arg_op) {
  int __pyx_v_op;
  PyObject *__pyx_r = 0;
  __Pyx_RefNannyDeclarations
  __Pyx_RefNannySetupContext("_custom_op_on_c_call (wrapper)", 0);
  assert(__pyx_arg_op); {
    __pyx_v_op = __Pyx_PyInt_As_int(__pyx_arg_op); if (unlikely((__pyx_v_op == (int)-1) && PyErr_Occurred())) __PYX_ERR(3, 98, __pyx_L3_error)
  }
  goto __pyx_L4_argument_unpacking_done;
  __pyx_L3_error:;
  __Pyx_AddTraceback("grpc._cython.cygrpc.Call._custom_op_on_c_call", __pyx_clineno, __pyx_lineno, __pyx_filename);
  __Pyx_RefNannyFinishContext();
  return NULL;
  __pyx_L4_argument_unpacking_done:;
  __pyx_r = __pyx_pf_4grpc_7_cython_6cygrpc_4Call_16_custom_op_on_c_call(((struct __pyx_obj_4grpc_7_cython_6cygrpc_Call *)__pyx_v_self), ((int)__pyx_v_op));

  /* function exit code */
  __Pyx_RefNannyFinishContext();
  return __pyx_r;
}

static PyObject *__pyx_pf_4grpc_7_cython_6cygrpc_4Call_16_custom_op_on_c_call(struct __pyx_obj_4grpc_7_cython_6cygrpc_Call *__pyx_v_self, int __pyx_v_op) {
  PyObject *__pyx_r = NULL;
  __Pyx_RefNannyDeclarations
  PyObject *__pyx_t_1 = NULL;
  __Pyx_RefNannySetupContext("_custom_op_on_c_call", 0);

  /* "src/python/grpcio/grpc/_cython/_cygrpc/call.pyx.pxi":99
 * 
 *   def _custom_op_on_c_call(self, int op):
 *     return _custom_op_on_c_call(op, self.c_call)             # <<<<<<<<<<<<<<
 */
  __Pyx_XDECREF(__pyx_r);
  __pyx_t_1 = __pyx_f_4grpc_7_cython_6cygrpc__custom_op_on_c_call(__pyx_v_op, __pyx_v_self->c_call); if (unlikely(!__pyx_t_1)) __PYX_ERR(3, 99, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_1);
  __pyx_r = __pyx_t_1;
  __pyx_t_1 = 0;
  goto __pyx_L0;

  /* "src/python/grpcio/grpc/_cython/_cygrpc/call.pyx.pxi":98
 *     return self.c_call != NULL
 * 
 *   def _custom_op_on_c_call(self, int op):             # <<<<<<<<<<<<<<
 *     return _custom_op_on_c_call(op, self.c_call)
 */

  /* function exit code */
  __pyx_L1_error:;
  __Pyx_XDECREF(__pyx_t_1);
  __Pyx_AddTraceback("grpc._cython.cygrpc.Call._custom_op_on_c_call", __pyx_clineno, __pyx_lineno, __pyx_filename);
  __pyx_r = NULL;
  __pyx_L0:;
  __Pyx_XGIVEREF(__pyx_r);
  __Pyx_RefNannyFinishContext();
  return __pyx_r;
}

/* "(tree fragment)":1
 * def __reduce_cython__(self):             # <<<<<<<<<<<<<<
 *     raise TypeError("no default __reduce__ due to non-trivial __cinit__")
 * def __setstate_cython__(self, __pyx_state):
 */

/* Python wrapper */
static PyObject *__pyx_pw_4grpc_7_cython_6cygrpc_4Call_19__reduce_cython__(PyObject *__pyx_v_self, CYTHON_UNUSED PyObject *unused); /*proto*/
static PyObject *__pyx_pw_4grpc_7_cython_6cygrpc_4Call_19__reduce_cython__(PyObject *__pyx_v_self, CYTHON_UNUSED PyObject *unused) {
  PyObject *__pyx_r = 0;
  __Pyx_RefNannyDeclarations
  __Pyx_RefNannySetupContext("__reduce_cython__ (wrapper)", 0);
  __pyx_r = __pyx_pf_4grpc_7_cython_6cygrpc_4Call_18__reduce_cython__(((struct __pyx_obj_4grpc_7_cython_6cygrpc_Call *)__pyx_v_self));

  /* function exit code */
  __Pyx_RefNannyFinishContext();
  return __pyx_r;
}

static PyObject *__pyx_pf_4grpc_7_cython_6cygrpc_4Call_18__reduce_cython__(CYTHON_UNUSED struct __pyx_obj_4grpc_7_cython_6cygrpc_Call *__pyx_v_self) {
  PyObject *__pyx_r = NULL;
  __Pyx_RefNannyDeclarations
  PyObject *__pyx_t_1 = NULL;
  __Pyx_RefNannySetupContext("__reduce_cython__", 0);

  /* "(tree fragment)":2
 * def __reduce_cython__(self):
 *     raise TypeError("no default __reduce__ due to non-trivial __cinit__")             # <<<<<<<<<<<<<<
 * def __setstate_cython__(self, __pyx_state):
 *     raise TypeError("no default __reduce__ due to non-trivial __cinit__")
 */
  __pyx_t_1 = __Pyx_PyObject_Call(__pyx_builtin_TypeError, __pyx_tuple__14, NULL); if (unlikely(!__pyx_t_1)) __PYX_ERR(7, 2, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_1);
  __Pyx_Raise(__pyx_t_1, 0, 0, 0);
  __Pyx_DECREF(__pyx_t_1); __pyx_t_1 = 0;
  __PYX_ERR(7, 2, __pyx_L1_error)

  /* "(tree fragment)":1
 * def __reduce_cython__(self):             # <<<<<<<<<<<<<<
 *     raise TypeError("no default __reduce__ due to non-trivial __cinit__")
 * def __setstate_cython__(self, __pyx_state):
 */

  /* function exit code */
  __pyx_L1_error:;
  __Pyx_XDECREF(__pyx_t_1);
  __Pyx_AddTraceback("grpc._cython.cygrpc.Call.__reduce_cython__", __pyx_clineno, __pyx_lineno, __pyx_filename);
  __pyx_r = NULL;
  __Pyx_XGIVEREF(__pyx_r);
  __Pyx_RefNannyFinishContext();
  return __pyx_r;
}

/* "(tree fragment)":3
 * def __reduce_cython__(self):
 *     raise TypeError("no default __reduce__ due to non-trivial __cinit__")
 * def __setstate_cython__(self, __pyx_state):             # <<<<<<<<<<<<<<
 *     raise TypeError("no default __reduce__ due to non-trivial __cinit__")
 */

/* Python wrapper */
static PyObject *__pyx_pw_4grpc_7_cython_6cygrpc_4Call_21__setstate_cython__(PyObject *__pyx_v_self, PyObject *__pyx_v___pyx_state); /*proto*/
static PyObject *__pyx_pw_4grpc_7_cython_6cygrpc_4Call_21__setstate_cython__(PyObject *__pyx_v_self, PyObject *__pyx_v___pyx_state) {
  PyObject *__pyx_r = 0;
  __Pyx_RefNannyDeclarations
  __Pyx_RefNannySetupContext("__setstate_cython__ (wrapper)", 0);
  __pyx_r = __pyx_pf_4grpc_7_cython_6cygrpc_4Call_20__setstate_cython__(((struct __pyx_obj_4grpc_7_cython_6cygrpc_Call *)__pyx_v_self), ((PyObject *)__pyx_v___pyx_state));

  /* function exit code */
  __Pyx_RefNannyFinishContext();
  return __pyx_r;
}

static PyObject *__pyx_pf_4grpc_7_cython_6cygrpc_4Call_20__setstate_cython__(CYTHON_UNUSED struct __pyx_obj_4grpc_7_cython_6cygrpc_Call *__pyx_v_self, CYTHON_UNUSED PyObject *__pyx_v___pyx_state) {
  PyObject *__pyx_r = NULL;
  __Pyx_RefNannyDeclarations
  PyObject *__pyx_t_1 = NULL;
  __Pyx_RefNannySetupContext("__setstate_cython__", 0);

  /* "(tree fragment)":4
 *     raise TypeError("no default __reduce__ due to non-trivial __cinit__")
 * def __setstate_cython__(self, __pyx_state):
 *     raise TypeError("no default __reduce__ due to non-trivial __cinit__")             # <<<<<<<<<<<<<<
 */
  __pyx_t_1 = __Pyx_PyObject_Call(__pyx_builtin_TypeError, __pyx_tuple__15, NULL); if (unlikely(!__pyx_t_1)) __PYX_ERR(7, 4, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_1);
  __Pyx_Raise(__pyx_t_1, 0, 0, 0);
  __Pyx_DECREF(__pyx_t_1); __pyx_t_1 = 0;
  __PYX_ERR(7, 4, __pyx_L1_error)

  /* "(tree fragment)":3
 * def __reduce_cython__(self):
 *     raise TypeError("no default __reduce__ due to non-trivial __cinit__")
 * def __setstate_cython__(self, __pyx_state):             # <<<<<<<<<<<<<<
 *     raise TypeError("no default __reduce__ due to non-trivial __cinit__")
 */

  /* function exit code */
  __pyx_L1_error:;
  __Pyx_XDECREF(__pyx_t_1);
  __Pyx_AddTraceback("grpc._cython.cygrpc.Call.__setstate_cython__", __pyx_clineno, __pyx_lineno, __pyx_filename);
  __pyx_r = NULL;
  __Pyx_XGIVEREF(__pyx_r);
  __Pyx_RefNannyFinishContext();
  return __pyx_r;
}

/* "src/python/grpcio/grpc/_cython/_cygrpc/channel.pyx.pxi":25
 * 
 * 
 * cdef str _call_error_metadata(metadata):             # <<<<<<<<<<<<<<
 *   return 'metadata was invalid: %s' % metadata
 * 
 */

static PyObject *__pyx_f_4grpc_7_cython_6cygrpc__call_error_metadata(PyObject *__pyx_v_metadata) {
  PyObject *__pyx_r = NULL;
  __Pyx_RefNannyDeclarations
  PyObject *__pyx_t_1 = NULL;
  __Pyx_RefNannySetupContext("_call_error_metadata", 0);

  /* "src/python/grpcio/grpc/_cython/_cygrpc/channel.pyx.pxi":26
 * 
 * cdef str _call_error_metadata(metadata):
 *   return 'metadata was invalid: %s' % metadata             # <<<<<<<<<<<<<<
 * 
 * 
 */
  __Pyx_XDECREF(__pyx_r);
  __pyx_t_1 = __Pyx_PyString_Format(__pyx_kp_s_metadata_was_invalid_s, __pyx_v_metadata); if (unlikely(!__pyx_t_1)) __PYX_ERR(8, 26, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_1);
  if (!(likely(PyString_CheckExact(__pyx_t_1))||(PyErr_Format(PyExc_TypeError, "Expected %.16s, got %.200s", "str", Py_TYPE(__pyx_t_1)->tp_name), 0))) __PYX_ERR(8, 26, __pyx_L1_error)
  __pyx_r = ((PyObject*)__pyx_t_1);
  __pyx_t_1 = 0;
  goto __pyx_L0;

  /* "src/python/grpcio/grpc/_cython/_cygrpc/channel.pyx.pxi":25
 * 
 * 
 * cdef str _call_error_metadata(metadata):             # <<<<<<<<<<<<<<
 *   return 'metadata was invalid: %s' % metadata
 * 
 */

  /* function exit code */
  __pyx_L1_error:;
  __Pyx_XDECREF(__pyx_t_1);
  __Pyx_AddTraceback("grpc._cython.cygrpc._call_error_metadata", __pyx_clineno, __pyx_lineno, __pyx_filename);
  __pyx_r = 0;
  __pyx_L0:;
  __Pyx_XGIVEREF(__pyx_r);
  __Pyx_RefNannyFinishContext();
  return __pyx_r;
}

/* "src/python/grpcio/grpc/_cython/_cygrpc/channel.pyx.pxi":29
 * 
 * 
 * cdef str _call_error_no_metadata(c_call_error):             # <<<<<<<<<<<<<<
 *   return _INTERNAL_CALL_ERROR_MESSAGE_FORMAT % c_call_error
 * 
 */

static PyObject *__pyx_f_4grpc_7_cython_6cygrpc__call_error_no_metadata(PyObject *__pyx_v_c_call_error) {
  PyObject *__pyx_r = NULL;
  __Pyx_RefNannyDeclarations
  PyObject *__pyx_t_1 = NULL;
  PyObject *__pyx_t_2 = NULL;
  __Pyx_RefNannySetupContext("_call_error_no_metadata", 0);

  /* "src/python/grpcio/grpc/_cython/_cygrpc/channel.pyx.pxi":30
 * 
 * cdef str _call_error_no_metadata(c_call_error):
 *   return _INTERNAL_CALL_ERROR_MESSAGE_FORMAT % c_call_error             # <<<<<<<<<<<<<<
 * 
 * 
 */
  __Pyx_XDECREF(__pyx_r);
  __pyx_t_1 = __Pyx_GetModuleGlobalName(__pyx_n_s_INTERNAL_CALL_ERROR_MESSAGE_FOR); if (unlikely(!__pyx_t_1)) __PYX_ERR(8, 30, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_1);
  __pyx_t_2 = PyNumber_Remainder(__pyx_t_1, __pyx_v_c_call_error); if (unlikely(!__pyx_t_2)) __PYX_ERR(8, 30, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_2);
  __Pyx_DECREF(__pyx_t_1); __pyx_t_1 = 0;
  if (!(likely(PyString_CheckExact(__pyx_t_2))||((__pyx_t_2) == Py_None)||(PyErr_Format(PyExc_TypeError, "Expected %.16s, got %.200s", "str", Py_TYPE(__pyx_t_2)->tp_name), 0))) __PYX_ERR(8, 30, __pyx_L1_error)
  __pyx_r = ((PyObject*)__pyx_t_2);
  __pyx_t_2 = 0;
  goto __pyx_L0;

  /* "src/python/grpcio/grpc/_cython/_cygrpc/channel.pyx.pxi":29
 * 
 * 
 * cdef str _call_error_no_metadata(c_call_error):             # <<<<<<<<<<<<<<
 *   return _INTERNAL_CALL_ERROR_MESSAGE_FORMAT % c_call_error
 * 
 */

  /* function exit code */
  __pyx_L1_error:;
  __Pyx_XDECREF(__pyx_t_1);
  __Pyx_XDECREF(__pyx_t_2);
  __Pyx_AddTraceback("grpc._cython.cygrpc._call_error_no_metadata", __pyx_clineno, __pyx_lineno, __pyx_filename);
  __pyx_r = 0;
  __pyx_L0:;
  __Pyx_XGIVEREF(__pyx_r);
  __Pyx_RefNannyFinishContext();
  return __pyx_r;
}

/* "src/python/grpcio/grpc/_cython/_cygrpc/channel.pyx.pxi":33
 * 
 * 
 * cdef str _call_error(c_call_error, metadata):             # <<<<<<<<<<<<<<
 *   if c_call_error == GRPC_CALL_ERROR_INVALID_METADATA:
 *     return _call_error_metadata(metadata)
 */

static PyObject *__pyx_f_4grpc_7_cython_6cygrpc__call_error(PyObject *__pyx_v_c_call_error, PyObject *__pyx_v_metadata) {
  PyObject *__pyx_r = NULL;
  __Pyx_RefNannyDeclarations
  PyObject *__pyx_t_1 = NULL;
  PyObject *__pyx_t_2 = NULL;
  int __pyx_t_3;
  __Pyx_RefNannySetupContext("_call_error", 0);

  /* "src/python/grpcio/grpc/_cython/_cygrpc/channel.pyx.pxi":34
 * 
 * cdef str _call_error(c_call_error, metadata):
 *   if c_call_error == GRPC_CALL_ERROR_INVALID_METADATA:             # <<<<<<<<<<<<<<
 *     return _call_error_metadata(metadata)
 *   else:
 */
  __pyx_t_1 = __Pyx_PyInt_From_grpc_call_error(GRPC_CALL_ERROR_INVALID_METADATA); if (unlikely(!__pyx_t_1)) __PYX_ERR(8, 34, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_1);
  __pyx_t_2 = PyObject_RichCompare(__pyx_v_c_call_error, __pyx_t_1, Py_EQ); __Pyx_XGOTREF(__pyx_t_2); if (unlikely(!__pyx_t_2)) __PYX_ERR(8, 34, __pyx_L1_error)
  __Pyx_DECREF(__pyx_t_1); __pyx_t_1 = 0;
  __pyx_t_3 = __Pyx_PyObject_IsTrue(__pyx_t_2); if (unlikely(__pyx_t_3 < 0)) __PYX_ERR(8, 34, __pyx_L1_error)
  __Pyx_DECREF(__pyx_t_2); __pyx_t_2 = 0;
  if (__pyx_t_3) {

    /* "src/python/grpcio/grpc/_cython/_cygrpc/channel.pyx.pxi":35
 * cdef str _call_error(c_call_error, metadata):
 *   if c_call_error == GRPC_CALL_ERROR_INVALID_METADATA:
 *     return _call_error_metadata(metadata)             # <<<<<<<<<<<<<<
 *   else:
 *     return _call_error_no_metadata(c_call_error)
 */
    __Pyx_XDECREF(__pyx_r);
    __pyx_t_2 = __pyx_f_4grpc_7_cython_6cygrpc__call_error_metadata(__pyx_v_metadata); if (unlikely(!__pyx_t_2)) __PYX_ERR(8, 35, __pyx_L1_error)
    __Pyx_GOTREF(__pyx_t_2);
    __pyx_r = ((PyObject*)__pyx_t_2);
    __pyx_t_2 = 0;
    goto __pyx_L0;

    /* "src/python/grpcio/grpc/_cython/_cygrpc/channel.pyx.pxi":34
 * 
 * cdef str _call_error(c_call_error, metadata):
 *   if c_call_error == GRPC_CALL_ERROR_INVALID_METADATA:             # <<<<<<<<<<<<<<
 *     return _call_error_metadata(metadata)
 *   else:
 */
  }

  /* "src/python/grpcio/grpc/_cython/_cygrpc/channel.pyx.pxi":37
 *     return _call_error_metadata(metadata)
 *   else:
 *     return _call_error_no_metadata(c_call_error)             # <<<<<<<<<<<<<<
 * 
 * 
 */
  /*else*/ {
    __Pyx_XDECREF(__pyx_r);
    __pyx_t_2 = __pyx_f_4grpc_7_cython_6cygrpc__call_error_no_metadata(__pyx_v_c_call_error); if (unlikely(!__pyx_t_2)) __PYX_ERR(8, 37, __pyx_L1_error)
    __Pyx_GOTREF(__pyx_t_2);
    __pyx_r = ((PyObject*)__pyx_t_2);
    __pyx_t_2 = 0;
    goto __pyx_L0;
  }

  /* "src/python/grpcio/grpc/_cython/_cygrpc/channel.pyx.pxi":33
 * 
 * 
 * cdef str _call_error(c_call_error, metadata):             # <<<<<<<<<<<<<<
 *   if c_call_error == GRPC_CALL_ERROR_INVALID_METADATA:
 *     return _call_error_metadata(metadata)
 */

  /* function exit code */
  __pyx_L1_error:;
  __Pyx_XDECREF(__pyx_t_1);
  __Pyx_XDECREF(__pyx_t_2);
  __Pyx_AddTraceback("grpc._cython.cygrpc._call_error", __pyx_clineno, __pyx_lineno, __pyx_filename);
  __pyx_r = 0;
  __pyx_L0:;
  __Pyx_XGIVEREF(__pyx_r);
  __Pyx_RefNannyFinishContext();
  return __pyx_r;
}

/* "src/python/grpcio/grpc/_cython/_cygrpc/channel.pyx.pxi":40
 * 
 * 
 * cdef _check_call_error_no_metadata(c_call_error):             # <<<<<<<<<<<<<<
 *   if c_call_error != GRPC_CALL_OK:
 *     return _INTERNAL_CALL_ERROR_MESSAGE_FORMAT % c_call_error
 */

static PyObject *__pyx_f_4grpc_7_cython_6cygrpc__check_call_error_no_metadata(PyObject *__pyx_v_c_call_error) {
  PyObject *__pyx_r = NULL;
  __Pyx_RefNannyDeclarations
  PyObject *__pyx_t_1 = NULL;
  PyObject *__pyx_t_2 = NULL;
  int __pyx_t_3;
  __Pyx_RefNannySetupContext("_check_call_error_no_metadata", 0);

  /* "src/python/grpcio/grpc/_cython/_cygrpc/channel.pyx.pxi":41
 * 
 * cdef _check_call_error_no_metadata(c_call_error):
 *   if c_call_error != GRPC_CALL_OK:             # <<<<<<<<<<<<<<
 *     return _INTERNAL_CALL_ERROR_MESSAGE_FORMAT % c_call_error
 *   else:
 */
  __pyx_t_1 = __Pyx_PyInt_From_grpc_call_error(GRPC_CALL_OK); if (unlikely(!__pyx_t_1)) __PYX_ERR(8, 41, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_1);
  __pyx_t_2 = PyObject_RichCompare(__pyx_v_c_call_error, __pyx_t_1, Py_NE); __Pyx_XGOTREF(__pyx_t_2); if (unlikely(!__pyx_t_2)) __PYX_ERR(8, 41, __pyx_L1_error)
  __Pyx_DECREF(__pyx_t_1); __pyx_t_1 = 0;
  __pyx_t_3 = __Pyx_PyObject_IsTrue(__pyx_t_2); if (unlikely(__pyx_t_3 < 0)) __PYX_ERR(8, 41, __pyx_L1_error)
  __Pyx_DECREF(__pyx_t_2); __pyx_t_2 = 0;
  if (__pyx_t_3) {

    /* "src/python/grpcio/grpc/_cython/_cygrpc/channel.pyx.pxi":42
 * cdef _check_call_error_no_metadata(c_call_error):
 *   if c_call_error != GRPC_CALL_OK:
 *     return _INTERNAL_CALL_ERROR_MESSAGE_FORMAT % c_call_error             # <<<<<<<<<<<<<<
 *   else:
 *     return None
 */
    __Pyx_XDECREF(__pyx_r);
    __pyx_t_2 = __Pyx_GetModuleGlobalName(__pyx_n_s_INTERNAL_CALL_ERROR_MESSAGE_FOR); if (unlikely(!__pyx_t_2)) __PYX_ERR(8, 42, __pyx_L1_error)
    __Pyx_GOTREF(__pyx_t_2);
    __pyx_t_1 = PyNumber_Remainder(__pyx_t_2, __pyx_v_c_call_error); if (unlikely(!__pyx_t_1)) __PYX_ERR(8, 42, __pyx_L1_error)
    __Pyx_GOTREF(__pyx_t_1);
    __Pyx_DECREF(__pyx_t_2); __pyx_t_2 = 0;
    __pyx_r = __pyx_t_1;
    __pyx_t_1 = 0;
    goto __pyx_L0;

    /* "src/python/grpcio/grpc/_cython/_cygrpc/channel.pyx.pxi":41
 * 
 * cdef _check_call_error_no_metadata(c_call_error):
 *   if c_call_error != GRPC_CALL_OK:             # <<<<<<<<<<<<<<
 *     return _INTERNAL_CALL_ERROR_MESSAGE_FORMAT % c_call_error
 *   else:
 */
  }

  /* "src/python/grpcio/grpc/_cython/_cygrpc/channel.pyx.pxi":44
 *     return _INTERNAL_CALL_ERROR_MESSAGE_FORMAT % c_call_error
 *   else:
 *     return None             # <<<<<<<<<<<<<<
 * 
 * 
 */
  /*else*/ {
    __Pyx_XDECREF(__pyx_r);
    __pyx_r = Py_None; __Pyx_INCREF(Py_None);
    goto __pyx_L0;
  }

  /* "src/python/grpcio/grpc/_cython/_cygrpc/channel.pyx.pxi":40
 * 
 * 
 * cdef _check_call_error_no_metadata(c_call_error):             # <<<<<<<<<<<<<<
 *   if c_call_error != GRPC_CALL_OK:
 *     return _INTERNAL_CALL_ERROR_MESSAGE_FORMAT % c_call_error
 */

  /* function exit code */
  __pyx_L1_error:;
  __Pyx_XDECREF(__pyx_t_1);
  __Pyx_XDECREF(__pyx_t_2);
  __Pyx_AddTraceback("grpc._cython.cygrpc._check_call_error_no_metadata", __pyx_clineno, __pyx_lineno, __pyx_filename);
  __pyx_r = 0;
  __pyx_L0:;
  __Pyx_XGIVEREF(__pyx_r);
  __Pyx_RefNannyFinishContext();
  return __pyx_r;
}

/* "src/python/grpcio/grpc/_cython/_cygrpc/channel.pyx.pxi":47
 * 
 * 
 * cdef _check_and_raise_call_error_no_metadata(c_call_error):             # <<<<<<<<<<<<<<
 *   error = _check_call_error_no_metadata(c_call_error)
 *   if error is not None:
 */

static PyObject *__pyx_f_4grpc_7_cython_6cygrpc__check_and_raise_call_error_no_metadata(PyObject *__pyx_v_c_call_error) {
  PyObject *__pyx_v_error = NULL;
  PyObject *__pyx_r = NULL;
  __Pyx_RefNannyDeclarations
  PyObject *__pyx_t_1 = NULL;
  int __pyx_t_2;
  int __pyx_t_3;
  __Pyx_RefNannySetupContext("_check_and_raise_call_error_no_metadata", 0);

  /* "src/python/grpcio/grpc/_cython/_cygrpc/channel.pyx.pxi":48
 * 
 * cdef _check_and_raise_call_error_no_metadata(c_call_error):
 *   error = _check_call_error_no_metadata(c_call_error)             # <<<<<<<<<<<<<<
 *   if error is not None:
 *     raise ValueError(error)
 */
  __pyx_t_1 = __pyx_f_4grpc_7_cython_6cygrpc__check_call_error_no_metadata(__pyx_v_c_call_error); if (unlikely(!__pyx_t_1)) __PYX_ERR(8, 48, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_1);
  __pyx_v_error = __pyx_t_1;
  __pyx_t_1 = 0;

  /* "src/python/grpcio/grpc/_cython/_cygrpc/channel.pyx.pxi":49
 * cdef _check_and_raise_call_error_no_metadata(c_call_error):
 *   error = _check_call_error_no_metadata(c_call_error)
 *   if error is not None:             # <<<<<<<<<<<<<<
 *     raise ValueError(error)
 * 
 */
  __pyx_t_2 = (__pyx_v_error != Py_None);
  __pyx_t_3 = (__pyx_t_2 != 0);
  if (unlikely(__pyx_t_3)) {

    /* "src/python/grpcio/grpc/_cython/_cygrpc/channel.pyx.pxi":50
 *   error = _check_call_error_no_metadata(c_call_error)
 *   if error is not None:
 *     raise ValueError(error)             # <<<<<<<<<<<<<<
 * 
 * 
 */
    __pyx_t_1 = __Pyx_PyObject_CallOneArg(__pyx_builtin_ValueError, __pyx_v_error); if (unlikely(!__pyx_t_1)) __PYX_ERR(8, 50, __pyx_L1_error)
    __Pyx_GOTREF(__pyx_t_1);
    __Pyx_Raise(__pyx_t_1, 0, 0, 0);
    __Pyx_DECREF(__pyx_t_1); __pyx_t_1 = 0;
    __PYX_ERR(8, 50, __pyx_L1_error)

    /* "src/python/grpcio/grpc/_cython/_cygrpc/channel.pyx.pxi":49
 * cdef _check_and_raise_call_error_no_metadata(c_call_error):
 *   error = _check_call_error_no_metadata(c_call_error)
 *   if error is not None:             # <<<<<<<<<<<<<<
 *     raise ValueError(error)
 * 
 */
  }

  /* "src/python/grpcio/grpc/_cython/_cygrpc/channel.pyx.pxi":47
 * 
 * 
 * cdef _check_and_raise_call_error_no_metadata(c_call_error):             # <<<<<<<<<<<<<<
 *   error = _check_call_error_no_metadata(c_call_error)
 *   if error is not None:
 */

  /* function exit code */
  __pyx_r = Py_None; __Pyx_INCREF(Py_None);
  goto __pyx_L0;
  __pyx_L1_error:;
  __Pyx_XDECREF(__pyx_t_1);
  __Pyx_AddTraceback("grpc._cython.cygrpc._check_and_raise_call_error_no_metadata", __pyx_clineno, __pyx_lineno, __pyx_filename);
  __pyx_r = 0;
  __pyx_L0:;
  __Pyx_XDECREF(__pyx_v_error);
  __Pyx_XGIVEREF(__pyx_r);
  __Pyx_RefNannyFinishContext();
  return __pyx_r;
}

/* "src/python/grpcio/grpc/_cython/_cygrpc/channel.pyx.pxi":53
 * 
 * 
 * cdef _check_call_error(c_call_error, metadata):             # <<<<<<<<<<<<<<
 *   if c_call_error == GRPC_CALL_ERROR_INVALID_METADATA:
 *     return _call_error_metadata(metadata)
 */

static PyObject *__pyx_f_4grpc_7_cython_6cygrpc__check_call_error(PyObject *__pyx_v_c_call_error, PyObject *__pyx_v_metadata) {
  PyObject *__pyx_r = NULL;
  __Pyx_RefNannyDeclarations
  PyObject *__pyx_t_1 = NULL;
  PyObject *__pyx_t_2 = NULL;
  int __pyx_t_3;
  __Pyx_RefNannySetupContext("_check_call_error", 0);

  /* "src/python/grpcio/grpc/_cython/_cygrpc/channel.pyx.pxi":54
 * 
 * cdef _check_call_error(c_call_error, metadata):
 *   if c_call_error == GRPC_CALL_ERROR_INVALID_METADATA:             # <<<<<<<<<<<<<<
 *     return _call_error_metadata(metadata)
 *   else:
 */
  __pyx_t_1 = __Pyx_PyInt_From_grpc_call_error(GRPC_CALL_ERROR_INVALID_METADATA); if (unlikely(!__pyx_t_1)) __PYX_ERR(8, 54, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_1);
  __pyx_t_2 = PyObject_RichCompare(__pyx_v_c_call_error, __pyx_t_1, Py_EQ); __Pyx_XGOTREF(__pyx_t_2); if (unlikely(!__pyx_t_2)) __PYX_ERR(8, 54, __pyx_L1_error)
  __Pyx_DECREF(__pyx_t_1); __pyx_t_1 = 0;
  __pyx_t_3 = __Pyx_PyObject_IsTrue(__pyx_t_2); if (unlikely(__pyx_t_3 < 0)) __PYX_ERR(8, 54, __pyx_L1_error)
  __Pyx_DECREF(__pyx_t_2); __pyx_t_2 = 0;
  if (__pyx_t_3) {

    /* "src/python/grpcio/grpc/_cython/_cygrpc/channel.pyx.pxi":55
 * cdef _check_call_error(c_call_error, metadata):
 *   if c_call_error == GRPC_CALL_ERROR_INVALID_METADATA:
 *     return _call_error_metadata(metadata)             # <<<<<<<<<<<<<<
 *   else:
 *     return _check_call_error_no_metadata(c_call_error)
 */
    __Pyx_XDECREF(__pyx_r);
    __pyx_t_2 = __pyx_f_4grpc_7_cython_6cygrpc__call_error_metadata(__pyx_v_metadata); if (unlikely(!__pyx_t_2)) __PYX_ERR(8, 55, __pyx_L1_error)
    __Pyx_GOTREF(__pyx_t_2);
    __pyx_r = __pyx_t_2;
    __pyx_t_2 = 0;
    goto __pyx_L0;

    /* "src/python/grpcio/grpc/_cython/_cygrpc/channel.pyx.pxi":54
 * 
 * cdef _check_call_error(c_call_error, metadata):
 *   if c_call_error == GRPC_CALL_ERROR_INVALID_METADATA:             # <<<<<<<<<<<<<<
 *     return _call_error_metadata(metadata)
 *   else:
 */
  }

  /* "src/python/grpcio/grpc/_cython/_cygrpc/channel.pyx.pxi":57
 *     return _call_error_metadata(metadata)
 *   else:
 *     return _check_call_error_no_metadata(c_call_error)             # <<<<<<<<<<<<<<
 * 
 * 
 */
  /*else*/ {
    __Pyx_XDECREF(__pyx_r);
    __pyx_t_2 = __pyx_f_4grpc_7_cython_6cygrpc__check_call_error_no_metadata(__pyx_v_c_call_error); if (unlikely(!__pyx_t_2)) __PYX_ERR(8, 57, __pyx_L1_error)
    __Pyx_GOTREF(__pyx_t_2);
    __pyx_r = __pyx_t_2;
    __pyx_t_2 = 0;
    goto __pyx_L0;
  }

  /* "src/python/grpcio/grpc/_cython/_cygrpc/channel.pyx.pxi":53
 * 
 * 
 * cdef _check_call_error(c_call_error, metadata):             # <<<<<<<<<<<<<<
 *   if c_call_error == GRPC_CALL_ERROR_INVALID_METADATA:
 *     return _call_error_metadata(metadata)
 */

  /* function exit code */
  __pyx_L1_error:;
  __Pyx_XDECREF(__pyx_t_1);
  __Pyx_XDECREF(__pyx_t_2);
  __Pyx_AddTraceback("grpc._cython.cygrpc._check_call_error", __pyx_clineno, __pyx_lineno, __pyx_filename);
  __pyx_r = 0;
  __pyx_L0:;
  __Pyx_XGIVEREF(__pyx_r);
  __Pyx_RefNannyFinishContext();
  return __pyx_r;
}

/* "src/python/grpcio/grpc/_cython/_cygrpc/channel.pyx.pxi":60
 * 
 * 
 * cdef void _raise_call_error_no_metadata(c_call_error) except *:             # <<<<<<<<<<<<<<
 *   raise ValueError(_call_error_no_metadata(c_call_error))
 * 
 */

static void __pyx_f_4grpc_7_cython_6cygrpc__raise_call_error_no_metadata(PyObject *__pyx_v_c_call_error) {
  __Pyx_RefNannyDeclarations
  PyObject *__pyx_t_1 = NULL;
  PyObject *__pyx_t_2 = NULL;
  __Pyx_RefNannySetupContext("_raise_call_error_no_metadata", 0);

  /* "src/python/grpcio/grpc/_cython/_cygrpc/channel.pyx.pxi":61
 * 
 * cdef void _raise_call_error_no_metadata(c_call_error) except *:
 *   raise ValueError(_call_error_no_metadata(c_call_error))             # <<<<<<<<<<<<<<
 * 
 * 
 */
  __pyx_t_1 = __pyx_f_4grpc_7_cython_6cygrpc__call_error_no_metadata(__pyx_v_c_call_error); if (unlikely(!__pyx_t_1)) __PYX_ERR(8, 61, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_1);
  __pyx_t_2 = __Pyx_PyObject_CallOneArg(__pyx_builtin_ValueError, __pyx_t_1); if (unlikely(!__pyx_t_2)) __PYX_ERR(8, 61, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_2);
  __Pyx_DECREF(__pyx_t_1); __pyx_t_1 = 0;
  __Pyx_Raise(__pyx_t_2, 0, 0, 0);
  __Pyx_DECREF(__pyx_t_2); __pyx_t_2 = 0;
  __PYX_ERR(8, 61, __pyx_L1_error)

  /* "src/python/grpcio/grpc/_cython/_cygrpc/channel.pyx.pxi":60
 * 
 * 
 * cdef void _raise_call_error_no_metadata(c_call_error) except *:             # <<<<<<<<<<<<<<
 *   raise ValueError(_call_error_no_metadata(c_call_error))
 * 
 */

  /* function exit code */
  __pyx_L1_error:;
  __Pyx_XDECREF(__pyx_t_1);
  __Pyx_XDECREF(__pyx_t_2);
  __Pyx_AddTraceback("grpc._cython.cygrpc._raise_call_error_no_metadata", __pyx_clineno, __pyx_lineno, __pyx_filename);
  __Pyx_RefNannyFinishContext();
}

/* "src/python/grpcio/grpc/_cython/_cygrpc/channel.pyx.pxi":64
 * 
 * 
 * cdef void _raise_call_error(c_call_error, metadata) except *:             # <<<<<<<<<<<<<<
 *   raise ValueError(_call_error(c_call_error, metadata))
 * 
 */

static void __pyx_f_4grpc_7_cython_6cygrpc__raise_call_error(PyObject *__pyx_v_c_call_error, PyObject *__pyx_v_metadata) {
  __Pyx_RefNannyDeclarations
  PyObject *__pyx_t_1 = NULL;
  PyObject *__pyx_t_2 = NULL;
  __Pyx_RefNannySetupContext("_raise_call_error", 0);

  /* "src/python/grpcio/grpc/_cython/_cygrpc/channel.pyx.pxi":65
 * 
 * cdef void _raise_call_error(c_call_error, metadata) except *:
 *   raise ValueError(_call_error(c_call_error, metadata))             # <<<<<<<<<<<<<<
 * 
 * 
 */
  __pyx_t_1 = __pyx_f_4grpc_7_cython_6cygrpc__call_error(__pyx_v_c_call_error, __pyx_v_metadata); if (unlikely(!__pyx_t_1)) __PYX_ERR(8, 65, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_1);
  __pyx_t_2 = __Pyx_PyObject_CallOneArg(__pyx_builtin_ValueError, __pyx_t_1); if (unlikely(!__pyx_t_2)) __PYX_ERR(8, 65, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_2);
  __Pyx_DECREF(__pyx_t_1); __pyx_t_1 = 0;
  __Pyx_Raise(__pyx_t_2, 0, 0, 0);
  __Pyx_DECREF(__pyx_t_2); __pyx_t_2 = 0;
  __PYX_ERR(8, 65, __pyx_L1_error)

  /* "src/python/grpcio/grpc/_cython/_cygrpc/channel.pyx.pxi":64
 * 
 * 
 * cdef void _raise_call_error(c_call_error, metadata) except *:             # <<<<<<<<<<<<<<
 *   raise ValueError(_call_error(c_call_error, metadata))
 * 
 */

  /* function exit code */
  __pyx_L1_error:;
  __Pyx_XDECREF(__pyx_t_1);
  __Pyx_XDECREF(__pyx_t_2);
  __Pyx_AddTraceback("grpc._cython.cygrpc._raise_call_error", __pyx_clineno, __pyx_lineno, __pyx_filename);
  __Pyx_RefNannyFinishContext();
}

/* "src/python/grpcio/grpc/_cython/_cygrpc/channel.pyx.pxi":68
 * 
 * 
 * cdef _destroy_c_completion_queue(grpc_completion_queue *c_completion_queue):             # <<<<<<<<<<<<<<
 *   grpc_completion_queue_shutdown(c_completion_queue)
 *   grpc_completion_queue_destroy(c_completion_queue)
 */

static PyObject *__pyx_f_4grpc_7_cython_6cygrpc__destroy_c_completion_queue(grpc_completion_queue *__pyx_v_c_completion_queue) {
  PyObject *__pyx_r = NULL;
  __Pyx_RefNannyDeclarations
  __Pyx_RefNannySetupContext("_destroy_c_completion_queue", 0);

  /* "src/python/grpcio/grpc/_cython/_cygrpc/channel.pyx.pxi":69
 * 
 * cdef _destroy_c_completion_queue(grpc_completion_queue *c_completion_queue):
 *   grpc_completion_queue_shutdown(c_completion_queue)             # <<<<<<<<<<<<<<
 *   grpc_completion_queue_destroy(c_completion_queue)
 * 
 */
  grpc_completion_queue_shutdown(__pyx_v_c_completion_queue);

  /* "src/python/grpcio/grpc/_cython/_cygrpc/channel.pyx.pxi":70
 * cdef _destroy_c_completion_queue(grpc_completion_queue *c_completion_queue):
 *   grpc_completion_queue_shutdown(c_completion_queue)
 *   grpc_completion_queue_destroy(c_completion_queue)             # <<<<<<<<<<<<<<
 * 
 * 
 */
  grpc_completion_queue_destroy(__pyx_v_c_completion_queue);

  /* "src/python/grpcio/grpc/_cython/_cygrpc/channel.pyx.pxi":68
 * 
 * 
 * cdef _destroy_c_completion_queue(grpc_completion_queue *c_completion_queue):             # <<<<<<<<<<<<<<
 *   grpc_completion_queue_shutdown(c_completion_queue)
 *   grpc_completion_queue_destroy(c_completion_queue)
 */

  /* function exit code */
  __pyx_r = Py_None; __Pyx_INCREF(Py_None);
  __Pyx_XGIVEREF(__pyx_r);
  __Pyx_RefNannyFinishContext();
  return __pyx_r;
}

/* "src/python/grpcio/grpc/_cython/_cygrpc/channel.pyx.pxi":75
 * cdef class _CallState:
 * 
 *   def __cinit__(self):             # <<<<<<<<<<<<<<
 *     self.due = set()
 * 
 */

/* Python wrapper */
static int __pyx_pw_4grpc_7_cython_6cygrpc_10_CallState_1__cinit__(PyObject *__pyx_v_self, PyObject *__pyx_args, PyObject *__pyx_kwds); /*proto*/
static int __pyx_pw_4grpc_7_cython_6cygrpc_10_CallState_1__cinit__(PyObject *__pyx_v_self, PyObject *__pyx_args, PyObject *__pyx_kwds) {
  int __pyx_r;
  __Pyx_RefNannyDeclarations
  __Pyx_RefNannySetupContext("__cinit__ (wrapper)", 0);
  if (unlikely(PyTuple_GET_SIZE(__pyx_args) > 0)) {
    __Pyx_RaiseArgtupleInvalid("__cinit__", 1, 0, 0, PyTuple_GET_SIZE(__pyx_args)); return -1;}
  if (unlikely(__pyx_kwds) && unlikely(PyDict_Size(__pyx_kwds) > 0) && unlikely(!__Pyx_CheckKeywordStrings(__pyx_kwds, "__cinit__", 0))) return -1;
  __pyx_r = __pyx_pf_4grpc_7_cython_6cygrpc_10_CallState___cinit__(((struct __pyx_obj_4grpc_7_cython_6cygrpc__CallState *)__pyx_v_self));

  /* function exit code */
  __Pyx_RefNannyFinishContext();
  return __pyx_r;
}

static int __pyx_pf_4grpc_7_cython_6cygrpc_10_CallState___cinit__(struct __pyx_obj_4grpc_7_cython_6cygrpc__CallState *__pyx_v_self) {
  int __pyx_r;
  __Pyx_RefNannyDeclarations
  PyObject *__pyx_t_1 = NULL;
  __Pyx_RefNannySetupContext("__cinit__", 0);

  /* "src/python/grpcio/grpc/_cython/_cygrpc/channel.pyx.pxi":76
 * 
 *   def __cinit__(self):
 *     self.due = set()             # <<<<<<<<<<<<<<
 * 
 * 
 */
  __pyx_t_1 = PySet_New(0); if (unlikely(!__pyx_t_1)) __PYX_ERR(8, 76, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_1);
  __Pyx_GIVEREF(__pyx_t_1);
  __Pyx_GOTREF(__pyx_v_self->due);
  __Pyx_DECREF(__pyx_v_self->due);
  __pyx_v_self->due = ((PyObject*)__pyx_t_1);
  __pyx_t_1 = 0;

  /* "src/python/grpcio/grpc/_cython/_cygrpc/channel.pyx.pxi":75
 * cdef class _CallState:
 * 
 *   def __cinit__(self):             # <<<<<<<<<<<<<<
 *     self.due = set()
 * 
 */

  /* function exit code */
  __pyx_r = 0;
  goto __pyx_L0;
  __pyx_L1_error:;
  __Pyx_XDECREF(__pyx_t_1);
  __Pyx_AddTraceback("grpc._cython.cygrpc._CallState.__cinit__", __pyx_clineno, __pyx_lineno, __pyx_filename);
  __pyx_r = -1;
  __pyx_L0:;
  __Pyx_RefNannyFinishContext();
  return __pyx_r;
}

/* "(tree fragment)":1
 * def __reduce_cython__(self):             # <<<<<<<<<<<<<<
 *     raise TypeError("no default __reduce__ due to non-trivial __cinit__")
 * def __setstate_cython__(self, __pyx_state):
 */

/* Python wrapper */
static PyObject *__pyx_pw_4grpc_7_cython_6cygrpc_10_CallState_3__reduce_cython__(PyObject *__pyx_v_self, CYTHON_UNUSED PyObject *unused); /*proto*/
static PyObject *__pyx_pw_4grpc_7_cython_6cygrpc_10_CallState_3__reduce_cython__(PyObject *__pyx_v_self, CYTHON_UNUSED PyObject *unused) {
  PyObject *__pyx_r = 0;
  __Pyx_RefNannyDeclarations
  __Pyx_RefNannySetupContext("__reduce_cython__ (wrapper)", 0);
  __pyx_r = __pyx_pf_4grpc_7_cython_6cygrpc_10_CallState_2__reduce_cython__(((struct __pyx_obj_4grpc_7_cython_6cygrpc__CallState *)__pyx_v_self));

  /* function exit code */
  __Pyx_RefNannyFinishContext();
  return __pyx_r;
}

static PyObject *__pyx_pf_4grpc_7_cython_6cygrpc_10_CallState_2__reduce_cython__(CYTHON_UNUSED struct __pyx_obj_4grpc_7_cython_6cygrpc__CallState *__pyx_v_self) {
  PyObject *__pyx_r = NULL;
  __Pyx_RefNannyDeclarations
  PyObject *__pyx_t_1 = NULL;
  __Pyx_RefNannySetupContext("__reduce_cython__", 0);

  /* "(tree fragment)":2
 * def __reduce_cython__(self):
 *     raise TypeError("no default __reduce__ due to non-trivial __cinit__")             # <<<<<<<<<<<<<<
 * def __setstate_cython__(self, __pyx_state):
 *     raise TypeError("no default __reduce__ due to non-trivial __cinit__")
 */
  __pyx_t_1 = __Pyx_PyObject_Call(__pyx_builtin_TypeError, __pyx_tuple__16, NULL); if (unlikely(!__pyx_t_1)) __PYX_ERR(7, 2, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_1);
  __Pyx_Raise(__pyx_t_1, 0, 0, 0);
  __Pyx_DECREF(__pyx_t_1); __pyx_t_1 = 0;
  __PYX_ERR(7, 2, __pyx_L1_error)

  /* "(tree fragment)":1
 * def __reduce_cython__(self):             # <<<<<<<<<<<<<<
 *     raise TypeError("no default __reduce__ due to non-trivial __cinit__")
 * def __setstate_cython__(self, __pyx_state):
 */

  /* function exit code */
  __pyx_L1_error:;
  __Pyx_XDECREF(__pyx_t_1);
  __Pyx_AddTraceback("grpc._cython.cygrpc._CallState.__reduce_cython__", __pyx_clineno, __pyx_lineno, __pyx_filename);
  __pyx_r = NULL;
  __Pyx_XGIVEREF(__pyx_r);
  __Pyx_RefNannyFinishContext();
  return __pyx_r;
}

/* "(tree fragment)":3
 * def __reduce_cython__(self):
 *     raise TypeError("no default __reduce__ due to non-trivial __cinit__")
 * def __setstate_cython__(self, __pyx_state):             # <<<<<<<<<<<<<<
 *     raise TypeError("no default __reduce__ due to non-trivial __cinit__")
 */

/* Python wrapper */
static PyObject *__pyx_pw_4grpc_7_cython_6cygrpc_10_CallState_5__setstate_cython__(PyObject *__pyx_v_self, PyObject *__pyx_v___pyx_state); /*proto*/
static PyObject *__pyx_pw_4grpc_7_cython_6cygrpc_10_CallState_5__setstate_cython__(PyObject *__pyx_v_self, PyObject *__pyx_v___pyx_state) {
  PyObject *__pyx_r = 0;
  __Pyx_RefNannyDeclarations
  __Pyx_RefNannySetupContext("__setstate_cython__ (wrapper)", 0);
  __pyx_r = __pyx_pf_4grpc_7_cython_6cygrpc_10_CallState_4__setstate_cython__(((struct __pyx_obj_4grpc_7_cython_6cygrpc__CallState *)__pyx_v_self), ((PyObject *)__pyx_v___pyx_state));

  /* function exit code */
  __Pyx_RefNannyFinishContext();
  return __pyx_r;
}

static PyObject *__pyx_pf_4grpc_7_cython_6cygrpc_10_CallState_4__setstate_cython__(CYTHON_UNUSED struct __pyx_obj_4grpc_7_cython_6cygrpc__CallState *__pyx_v_self, CYTHON_UNUSED PyObject *__pyx_v___pyx_state) {
  PyObject *__pyx_r = NULL;
  __Pyx_RefNannyDeclarations
  PyObject *__pyx_t_1 = NULL;
  __Pyx_RefNannySetupContext("__setstate_cython__", 0);

  /* "(tree fragment)":4
 *     raise TypeError("no default __reduce__ due to non-trivial __cinit__")
 * def __setstate_cython__(self, __pyx_state):
 *     raise TypeError("no default __reduce__ due to non-trivial __cinit__")             # <<<<<<<<<<<<<<
 */
  __pyx_t_1 = __Pyx_PyObject_Call(__pyx_builtin_TypeError, __pyx_tuple__17, NULL); if (unlikely(!__pyx_t_1)) __PYX_ERR(7, 4, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_1);
  __Pyx_Raise(__pyx_t_1, 0, 0, 0);
  __Pyx_DECREF(__pyx_t_1); __pyx_t_1 = 0;
  __PYX_ERR(7, 4, __pyx_L1_error)

  /* "(tree fragment)":3
 * def __reduce_cython__(self):
 *     raise TypeError("no default __reduce__ due to non-trivial __cinit__")
 * def __setstate_cython__(self, __pyx_state):             # <<<<<<<<<<<<<<
 *     raise TypeError("no default __reduce__ due to non-trivial __cinit__")
 */

  /* function exit code */
  __pyx_L1_error:;
  __Pyx_XDECREF(__pyx_t_1);
  __Pyx_AddTraceback("grpc._cython.cygrpc._CallState.__setstate_cython__", __pyx_clineno, __pyx_lineno, __pyx_filename);
  __pyx_r = NULL;
  __Pyx_XGIVEREF(__pyx_r);
  __Pyx_RefNannyFinishContext();
  return __pyx_r;
}

/* "src/python/grpcio/grpc/_cython/_cygrpc/channel.pyx.pxi":81
 * cdef class _ChannelState:
 * 
 *   def __cinit__(self):             # <<<<<<<<<<<<<<
 *     self.condition = threading.Condition()
 *     self.open = True
 */

/* Python wrapper */
static int __pyx_pw_4grpc_7_cython_6cygrpc_13_ChannelState_1__cinit__(PyObject *__pyx_v_self, PyObject *__pyx_args, PyObject *__pyx_kwds); /*proto*/
static int __pyx_pw_4grpc_7_cython_6cygrpc_13_ChannelState_1__cinit__(PyObject *__pyx_v_self, PyObject *__pyx_args, PyObject *__pyx_kwds) {
  int __pyx_r;
  __Pyx_RefNannyDeclarations
  __Pyx_RefNannySetupContext("__cinit__ (wrapper)", 0);
  if (unlikely(PyTuple_GET_SIZE(__pyx_args) > 0)) {
    __Pyx_RaiseArgtupleInvalid("__cinit__", 1, 0, 0, PyTuple_GET_SIZE(__pyx_args)); return -1;}
  if (unlikely(__pyx_kwds) && unlikely(PyDict_Size(__pyx_kwds) > 0) && unlikely(!__Pyx_CheckKeywordStrings(__pyx_kwds, "__cinit__", 0))) return -1;
  __pyx_r = __pyx_pf_4grpc_7_cython_6cygrpc_13_ChannelState___cinit__(((struct __pyx_obj_4grpc_7_cython_6cygrpc__ChannelState *)__pyx_v_self));

  /* function exit code */
  __Pyx_RefNannyFinishContext();
  return __pyx_r;
}

static int __pyx_pf_4grpc_7_cython_6cygrpc_13_ChannelState___cinit__(struct __pyx_obj_4grpc_7_cython_6cygrpc__ChannelState *__pyx_v_self) {
  int __pyx_r;
  __Pyx_RefNannyDeclarations
  PyObject *__pyx_t_1 = NULL;
  PyObject *__pyx_t_2 = NULL;
  PyObject *__pyx_t_3 = NULL;
  __Pyx_RefNannySetupContext("__cinit__", 0);

  /* "src/python/grpcio/grpc/_cython/_cygrpc/channel.pyx.pxi":82
 * 
 *   def __cinit__(self):
 *     self.condition = threading.Condition()             # <<<<<<<<<<<<<<
 *     self.open = True
 *     self.integrated_call_states = {}
 */
  __pyx_t_2 = __Pyx_GetModuleGlobalName(__pyx_n_s_threading); if (unlikely(!__pyx_t_2)) __PYX_ERR(8, 82, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_2);
  __pyx_t_3 = __Pyx_PyObject_GetAttrStr(__pyx_t_2, __pyx_n_s_Condition); if (unlikely(!__pyx_t_3)) __PYX_ERR(8, 82, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_3);
  __Pyx_DECREF(__pyx_t_2); __pyx_t_2 = 0;
  __pyx_t_2 = NULL;
  if (CYTHON_UNPACK_METHODS && unlikely(PyMethod_Check(__pyx_t_3))) {
    __pyx_t_2 = PyMethod_GET_SELF(__pyx_t_3);
    if (likely(__pyx_t_2)) {
      PyObject* function = PyMethod_GET_FUNCTION(__pyx_t_3);
      __Pyx_INCREF(__pyx_t_2);
      __Pyx_INCREF(function);
      __Pyx_DECREF_SET(__pyx_t_3, function);
    }
  }
  if (__pyx_t_2) {
    __pyx_t_1 = __Pyx_PyObject_CallOneArg(__pyx_t_3, __pyx_t_2); if (unlikely(!__pyx_t_1)) __PYX_ERR(8, 82, __pyx_L1_error)
    __Pyx_DECREF(__pyx_t_2); __pyx_t_2 = 0;
  } else {
    __pyx_t_1 = __Pyx_PyObject_CallNoArg(__pyx_t_3); if (unlikely(!__pyx_t_1)) __PYX_ERR(8, 82, __pyx_L1_error)
  }
  __Pyx_GOTREF(__pyx_t_1);
  __Pyx_DECREF(__pyx_t_3); __pyx_t_3 = 0;
  __Pyx_GIVEREF(__pyx_t_1);
  __Pyx_GOTREF(__pyx_v_self->condition);
  __Pyx_DECREF(__pyx_v_self->condition);
  __pyx_v_self->condition = __pyx_t_1;
  __pyx_t_1 = 0;

  /* "src/python/grpcio/grpc/_cython/_cygrpc/channel.pyx.pxi":83
 *   def __cinit__(self):
 *     self.condition = threading.Condition()
 *     self.open = True             # <<<<<<<<<<<<<<
 *     self.integrated_call_states = {}
 *     self.segregated_call_states = set()
 */
  __Pyx_INCREF(Py_True);
  __Pyx_GIVEREF(Py_True);
  __Pyx_GOTREF(__pyx_v_self->open);
  __Pyx_DECREF(__pyx_v_self->open);
  __pyx_v_self->open = Py_True;

  /* "src/python/grpcio/grpc/_cython/_cygrpc/channel.pyx.pxi":84
 *     self.condition = threading.Condition()
 *     self.open = True
 *     self.integrated_call_states = {}             # <<<<<<<<<<<<<<
 *     self.segregated_call_states = set()
 *     self.connectivity_due = set()
 */
  __pyx_t_1 = __Pyx_PyDict_NewPresized(0); if (unlikely(!__pyx_t_1)) __PYX_ERR(8, 84, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_1);
  __Pyx_GIVEREF(__pyx_t_1);
  __Pyx_GOTREF(__pyx_v_self->integrated_call_states);
  __Pyx_DECREF(__pyx_v_self->integrated_call_states);
  __pyx_v_self->integrated_call_states = ((PyObject*)__pyx_t_1);
  __pyx_t_1 = 0;

  /* "src/python/grpcio/grpc/_cython/_cygrpc/channel.pyx.pxi":85
 *     self.open = True
 *     self.integrated_call_states = {}
 *     self.segregated_call_states = set()             # <<<<<<<<<<<<<<
 *     self.connectivity_due = set()
 *     self.closed_reason = None
 */
  __pyx_t_1 = PySet_New(0); if (unlikely(!__pyx_t_1)) __PYX_ERR(8, 85, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_1);
  __Pyx_GIVEREF(__pyx_t_1);
  __Pyx_GOTREF(__pyx_v_self->segregated_call_states);
  __Pyx_DECREF(__pyx_v_self->segregated_call_states);
  __pyx_v_self->segregated_call_states = ((PyObject*)__pyx_t_1);
  __pyx_t_1 = 0;

  /* "src/python/grpcio/grpc/_cython/_cygrpc/channel.pyx.pxi":86
 *     self.integrated_call_states = {}
 *     self.segregated_call_states = set()
 *     self.connectivity_due = set()             # <<<<<<<<<<<<<<
 *     self.closed_reason = None
 * 
 */
  __pyx_t_1 = PySet_New(0); if (unlikely(!__pyx_t_1)) __PYX_ERR(8, 86, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_1);
  __Pyx_GIVEREF(__pyx_t_1);
  __Pyx_GOTREF(__pyx_v_self->connectivity_due);
  __Pyx_DECREF(__pyx_v_self->connectivity_due);
  __pyx_v_self->connectivity_due = ((PyObject*)__pyx_t_1);
  __pyx_t_1 = 0;

  /* "src/python/grpcio/grpc/_cython/_cygrpc/channel.pyx.pxi":87
 *     self.segregated_call_states = set()
 *     self.connectivity_due = set()
 *     self.closed_reason = None             # <<<<<<<<<<<<<<
 * 
 * 
 */
  __Pyx_INCREF(Py_None);
  __Pyx_GIVEREF(Py_None);
  __Pyx_GOTREF(__pyx_v_self->closed_reason);
  __Pyx_DECREF(__pyx_v_self->closed_reason);
  __pyx_v_self->closed_reason = Py_None;

  /* "src/python/grpcio/grpc/_cython/_cygrpc/channel.pyx.pxi":81
 * cdef class _ChannelState:
 * 
 *   def __cinit__(self):             # <<<<<<<<<<<<<<
 *     self.condition = threading.Condition()
 *     self.open = True
 */

  /* function exit code */
  __pyx_r = 0;
  goto __pyx_L0;
  __pyx_L1_error:;
  __Pyx_XDECREF(__pyx_t_1);
  __Pyx_XDECREF(__pyx_t_2);
  __Pyx_XDECREF(__pyx_t_3);
  __Pyx_AddTraceback("grpc._cython.cygrpc._ChannelState.__cinit__", __pyx_clineno, __pyx_lineno, __pyx_filename);
  __pyx_r = -1;
  __pyx_L0:;
  __Pyx_RefNannyFinishContext();
  return __pyx_r;
}

/* "(tree fragment)":1
 * def __reduce_cython__(self):             # <<<<<<<<<<<<<<
 *     raise TypeError("no default __reduce__ due to non-trivial __cinit__")
 * def __setstate_cython__(self, __pyx_state):
 */

/* Python wrapper */
static PyObject *__pyx_pw_4grpc_7_cython_6cygrpc_13_ChannelState_3__reduce_cython__(PyObject *__pyx_v_self, CYTHON_UNUSED PyObject *unused); /*proto*/
static PyObject *__pyx_pw_4grpc_7_cython_6cygrpc_13_ChannelState_3__reduce_cython__(PyObject *__pyx_v_self, CYTHON_UNUSED PyObject *unused) {
  PyObject *__pyx_r = 0;
  __Pyx_RefNannyDeclarations
  __Pyx_RefNannySetupContext("__reduce_cython__ (wrapper)", 0);
  __pyx_r = __pyx_pf_4grpc_7_cython_6cygrpc_13_ChannelState_2__reduce_cython__(((struct __pyx_obj_4grpc_7_cython_6cygrpc__ChannelState *)__pyx_v_self));

  /* function exit code */
  __Pyx_RefNannyFinishContext();
  return __pyx_r;
}

static PyObject *__pyx_pf_4grpc_7_cython_6cygrpc_13_ChannelState_2__reduce_cython__(CYTHON_UNUSED struct __pyx_obj_4grpc_7_cython_6cygrpc__ChannelState *__pyx_v_self) {
  PyObject *__pyx_r = NULL;
  __Pyx_RefNannyDeclarations
  PyObject *__pyx_t_1 = NULL;
  __Pyx_RefNannySetupContext("__reduce_cython__", 0);

  /* "(tree fragment)":2
 * def __reduce_cython__(self):
 *     raise TypeError("no default __reduce__ due to non-trivial __cinit__")             # <<<<<<<<<<<<<<
 * def __setstate_cython__(self, __pyx_state):
 *     raise TypeError("no default __reduce__ due to non-trivial __cinit__")
 */
  __pyx_t_1 = __Pyx_PyObject_Call(__pyx_builtin_TypeError, __pyx_tuple__18, NULL); if (unlikely(!__pyx_t_1)) __PYX_ERR(7, 2, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_1);
  __Pyx_Raise(__pyx_t_1, 0, 0, 0);
  __Pyx_DECREF(__pyx_t_1); __pyx_t_1 = 0;
  __PYX_ERR(7, 2, __pyx_L1_error)

  /* "(tree fragment)":1
 * def __reduce_cython__(self):             # <<<<<<<<<<<<<<
 *     raise TypeError("no default __reduce__ due to non-trivial __cinit__")
 * def __setstate_cython__(self, __pyx_state):
 */

  /* function exit code */
  __pyx_L1_error:;
  __Pyx_XDECREF(__pyx_t_1);
  __Pyx_AddTraceback("grpc._cython.cygrpc._ChannelState.__reduce_cython__", __pyx_clineno, __pyx_lineno, __pyx_filename);
  __pyx_r = NULL;
  __Pyx_XGIVEREF(__pyx_r);
  __Pyx_RefNannyFinishContext();
  return __pyx_r;
}

/* "(tree fragment)":3
 * def __reduce_cython__(self):
 *     raise TypeError("no default __reduce__ due to non-trivial __cinit__")
 * def __setstate_cython__(self, __pyx_state):             # <<<<<<<<<<<<<<
 *     raise TypeError("no default __reduce__ due to non-trivial __cinit__")
 */

/* Python wrapper */
static PyObject *__pyx_pw_4grpc_7_cython_6cygrpc_13_ChannelState_5__setstate_cython__(PyObject *__pyx_v_self, PyObject *__pyx_v___pyx_state); /*proto*/
static PyObject *__pyx_pw_4grpc_7_cython_6cygrpc_13_ChannelState_5__setstate_cython__(PyObject *__pyx_v_self, PyObject *__pyx_v___pyx_state) {
  PyObject *__pyx_r = 0;
  __Pyx_RefNannyDeclarations
  __Pyx_RefNannySetupContext("__setstate_cython__ (wrapper)", 0);
  __pyx_r = __pyx_pf_4grpc_7_cython_6cygrpc_13_ChannelState_4__setstate_cython__(((struct __pyx_obj_4grpc_7_cython_6cygrpc__ChannelState *)__pyx_v_self), ((PyObject *)__pyx_v___pyx_state));

  /* function exit code */
  __Pyx_RefNannyFinishContext();
  return __pyx_r;
}

static PyObject *__pyx_pf_4grpc_7_cython_6cygrpc_13_ChannelState_4__setstate_cython__(CYTHON_UNUSED struct __pyx_obj_4grpc_7_cython_6cygrpc__ChannelState *__pyx_v_self, CYTHON_UNUSED PyObject *__pyx_v___pyx_state) {
  PyObject *__pyx_r = NULL;
  __Pyx_RefNannyDeclarations
  PyObject *__pyx_t_1 = NULL;
  __Pyx_RefNannySetupContext("__setstate_cython__", 0);

  /* "(tree fragment)":4
 *     raise TypeError("no default __reduce__ due to non-trivial __cinit__")
 * def __setstate_cython__(self, __pyx_state):
 *     raise TypeError("no default __reduce__ due to non-trivial __cinit__")             # <<<<<<<<<<<<<<
 */
  __pyx_t_1 = __Pyx_PyObject_Call(__pyx_builtin_TypeError, __pyx_tuple__19, NULL); if (unlikely(!__pyx_t_1)) __PYX_ERR(7, 4, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_1);
  __Pyx_Raise(__pyx_t_1, 0, 0, 0);
  __Pyx_DECREF(__pyx_t_1); __pyx_t_1 = 0;
  __PYX_ERR(7, 4, __pyx_L1_error)

  /* "(tree fragment)":3
 * def __reduce_cython__(self):
 *     raise TypeError("no default __reduce__ due to non-trivial __cinit__")
 * def __setstate_cython__(self, __pyx_state):             # <<<<<<<<<<<<<<
 *     raise TypeError("no default __reduce__ due to non-trivial __cinit__")
 */

  /* function exit code */
  __pyx_L1_error:;
  __Pyx_XDECREF(__pyx_t_1);
  __Pyx_AddTraceback("grpc._cython.cygrpc._ChannelState.__setstate_cython__", __pyx_clineno, __pyx_lineno, __pyx_filename);
  __pyx_r = NULL;
  __Pyx_XGIVEREF(__pyx_r);
  __Pyx_RefNannyFinishContext();
  return __pyx_r;
}

/* "src/python/grpcio/grpc/_cython/_cygrpc/channel.pyx.pxi":90
 * 
 * 
 * cdef tuple _operate(grpc_call *c_call, object operations, object user_tag):             # <<<<<<<<<<<<<<
 *   cdef grpc_call_error c_call_error
 *   cdef _BatchOperationTag tag = _BatchOperationTag(user_tag, operations, None)
 */

static PyObject *__pyx_f_4grpc_7_cython_6cygrpc__operate(grpc_call *__pyx_v_c_call, PyObject *__pyx_v_operations, PyObject *__pyx_v_user_tag) {
  grpc_call_error __pyx_v_c_call_error;
  struct __pyx_obj_4grpc_7_cython_6cygrpc__BatchOperationTag *__pyx_v_tag = 0;
  PyObject *__pyx_r = NULL;
  __Pyx_RefNannyDeclarations
  PyObject *__pyx_t_1 = NULL;
  PyObject *__pyx_t_2 = NULL;
  __Pyx_RefNannySetupContext("_operate", 0);

  /* "src/python/grpcio/grpc/_cython/_cygrpc/channel.pyx.pxi":92
 * cdef tuple _operate(grpc_call *c_call, object operations, object user_tag):
 *   cdef grpc_call_error c_call_error
 *   cdef _BatchOperationTag tag = _BatchOperationTag(user_tag, operations, None)             # <<<<<<<<<<<<<<
 *   tag.prepare()
 *   cpython.Py_INCREF(tag)
 */
  __pyx_t_1 = PyTuple_New(3); if (unlikely(!__pyx_t_1)) __PYX_ERR(8, 92, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_1);
  __Pyx_INCREF(__pyx_v_user_tag);
  __Pyx_GIVEREF(__pyx_v_user_tag);
  PyTuple_SET_ITEM(__pyx_t_1, 0, __pyx_v_user_tag);
  __Pyx_INCREF(__pyx_v_operations);
  __Pyx_GIVEREF(__pyx_v_operations);
  PyTuple_SET_ITEM(__pyx_t_1, 1, __pyx_v_operations);
  __Pyx_INCREF(Py_None);
  __Pyx_GIVEREF(Py_None);
  PyTuple_SET_ITEM(__pyx_t_1, 2, Py_None);
  __pyx_t_2 = __Pyx_PyObject_Call(((PyObject *)__pyx_ptype_4grpc_7_cython_6cygrpc__BatchOperationTag), __pyx_t_1, NULL); if (unlikely(!__pyx_t_2)) __PYX_ERR(8, 92, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_2);
  __Pyx_DECREF(__pyx_t_1); __pyx_t_1 = 0;
  __pyx_v_tag = ((struct __pyx_obj_4grpc_7_cython_6cygrpc__BatchOperationTag *)__pyx_t_2);
  __pyx_t_2 = 0;

  /* "src/python/grpcio/grpc/_cython/_cygrpc/channel.pyx.pxi":93
 *   cdef grpc_call_error c_call_error
 *   cdef _BatchOperationTag tag = _BatchOperationTag(user_tag, operations, None)
 *   tag.prepare()             # <<<<<<<<<<<<<<
 *   cpython.Py_INCREF(tag)
 *   with nogil:
 */
  ((struct __pyx_vtabstruct_4grpc_7_cython_6cygrpc__BatchOperationTag *)__pyx_v_tag->__pyx_base.__pyx_vtab)->prepare(__pyx_v_tag); if (unlikely(PyErr_Occurred())) __PYX_ERR(8, 93, __pyx_L1_error)

  /* "src/python/grpcio/grpc/_cython/_cygrpc/channel.pyx.pxi":94
 *   cdef _BatchOperationTag tag = _BatchOperationTag(user_tag, operations, None)
 *   tag.prepare()
 *   cpython.Py_INCREF(tag)             # <<<<<<<<<<<<<<
 *   with nogil:
 *     c_call_error = grpc_call_start_batch(
 */
  Py_INCREF(((PyObject *)__pyx_v_tag));

  /* "src/python/grpcio/grpc/_cython/_cygrpc/channel.pyx.pxi":95
 *   tag.prepare()
 *   cpython.Py_INCREF(tag)
 *   with nogil:             # <<<<<<<<<<<<<<
 *     c_call_error = grpc_call_start_batch(
 *         c_call, tag.c_ops, tag.c_nops, <cpython.PyObject *>tag, NULL)
 */
  {
      #ifdef WITH_THREAD
      PyThreadState *_save;
      Py_UNBLOCK_THREADS
      __Pyx_FastGIL_Remember();
      #endif
      /*try:*/ {

        /* "src/python/grpcio/grpc/_cython/_cygrpc/channel.pyx.pxi":96
 *   cpython.Py_INCREF(tag)
 *   with nogil:
 *     c_call_error = grpc_call_start_batch(             # <<<<<<<<<<<<<<
 *         c_call, tag.c_ops, tag.c_nops, <cpython.PyObject *>tag, NULL)
 *   return c_call_error, tag
 */
        __pyx_v_c_call_error = grpc_call_start_batch(__pyx_v_c_call, __pyx_v_tag->c_ops, __pyx_v_tag->c_nops, ((PyObject *)__pyx_v_tag), NULL);
      }

      /* "src/python/grpcio/grpc/_cython/_cygrpc/channel.pyx.pxi":95
 *   tag.prepare()
 *   cpython.Py_INCREF(tag)
 *   with nogil:             # <<<<<<<<<<<<<<
 *     c_call_error = grpc_call_start_batch(
 *         c_call, tag.c_ops, tag.c_nops, <cpython.PyObject *>tag, NULL)
 */
      /*finally:*/ {
        /*normal exit:*/{
          #ifdef WITH_THREAD
          __Pyx_FastGIL_Forget();
          Py_BLOCK_THREADS
          #endif
          goto __pyx_L5;
        }
        __pyx_L5:;
      }
  }

  /* "src/python/grpcio/grpc/_cython/_cygrpc/channel.pyx.pxi":98
 *     c_call_error = grpc_call_start_batch(
 *         c_call, tag.c_ops, tag.c_nops, <cpython.PyObject *>tag, NULL)
 *   return c_call_error, tag             # <<<<<<<<<<<<<<
 * 
 * 
 */
  __Pyx_XDECREF(__pyx_r);
  __pyx_t_2 = __Pyx_PyInt_From_grpc_call_error(__pyx_v_c_call_error); if (unlikely(!__pyx_t_2)) __PYX_ERR(8, 98, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_2);
  __pyx_t_1 = PyTuple_New(2); if (unlikely(!__pyx_t_1)) __PYX_ERR(8, 98, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_1);
  __Pyx_GIVEREF(__pyx_t_2);
  PyTuple_SET_ITEM(__pyx_t_1, 0, __pyx_t_2);
  __Pyx_INCREF(((PyObject *)__pyx_v_tag));
  __Pyx_GIVEREF(((PyObject *)__pyx_v_tag));
  PyTuple_SET_ITEM(__pyx_t_1, 1, ((PyObject *)__pyx_v_tag));
  __pyx_t_2 = 0;
  __pyx_r = ((PyObject*)__pyx_t_1);
  __pyx_t_1 = 0;
  goto __pyx_L0;

  /* "src/python/grpcio/grpc/_cython/_cygrpc/channel.pyx.pxi":90
 * 
 * 
 * cdef tuple _operate(grpc_call *c_call, object operations, object user_tag):             # <<<<<<<<<<<<<<
 *   cdef grpc_call_error c_call_error
 *   cdef _BatchOperationTag tag = _BatchOperationTag(user_tag, operations, None)
 */

  /* function exit code */
  __pyx_L1_error:;
  __Pyx_XDECREF(__pyx_t_1);
  __Pyx_XDECREF(__pyx_t_2);
  __Pyx_AddTraceback("grpc._cython.cygrpc._operate", __pyx_clineno, __pyx_lineno, __pyx_filename);
  __pyx_r = 0;
  __pyx_L0:;
  __Pyx_XDECREF((PyObject *)__pyx_v_tag);
  __Pyx_XGIVEREF(__pyx_r);
  __Pyx_RefNannyFinishContext();
  return __pyx_r;
}

/* "src/python/grpcio/grpc/_cython/_cygrpc/channel.pyx.pxi":101
 * 
 * 
 * cdef object _operate_from_integrated_call(             # <<<<<<<<<<<<<<
 *     _ChannelState channel_state, _CallState call_state, object operations,
 *     object user_tag):
 */

static PyObject *__pyx_f_4grpc_7_cython_6cygrpc__operate_from_integrated_call(struct __pyx_obj_4grpc_7_cython_6cygrpc__ChannelState *__pyx_v_channel_state, struct __pyx_obj_4grpc_7_cython_6cygrpc__CallState *__pyx_v_call_state, PyObject *__pyx_v_operations, PyObject *__pyx_v_user_tag) {
  grpc_call_error __pyx_v_c_call_error;
  struct __pyx_obj_4grpc_7_cython_6cygrpc__BatchOperationTag *__pyx_v_tag = 0;
  PyObject *__pyx_r = NULL;
  __Pyx_RefNannyDeclarations
  PyObject *__pyx_t_1 = NULL;
  PyObject *__pyx_t_2 = NULL;
  PyObject *__pyx_t_3 = NULL;
  PyObject *__pyx_t_4 = NULL;
  PyObject *__pyx_t_5 = NULL;
  PyObject *__pyx_t_6 = NULL;
  PyObject *__pyx_t_7 = NULL;
  int __pyx_t_8;
  grpc_call_error __pyx_t_9;
  int __pyx_t_10;
  PyObject *__pyx_t_11 = NULL;
  PyObject *__pyx_t_12 = NULL;
  int __pyx_t_13;
  __Pyx_RefNannySetupContext("_operate_from_integrated_call", 0);

  /* "src/python/grpcio/grpc/_cython/_cygrpc/channel.pyx.pxi":106
 *   cdef grpc_call_error c_call_error
 *   cdef _BatchOperationTag tag
 *   with channel_state.condition:             # <<<<<<<<<<<<<<
 *     if call_state.due:
 *       c_call_error, tag = _operate(call_state.c_call, operations, user_tag)
 */
  /*with:*/ {
    __pyx_t_1 = __Pyx_PyObject_LookupSpecial(__pyx_v_channel_state->condition, __pyx_n_s_exit); if (unlikely(!__pyx_t_1)) __PYX_ERR(8, 106, __pyx_L1_error)
    __Pyx_GOTREF(__pyx_t_1);
    __pyx_t_3 = __Pyx_PyObject_LookupSpecial(__pyx_v_channel_state->condition, __pyx_n_s_enter); if (unlikely(!__pyx_t_3)) __PYX_ERR(8, 106, __pyx_L3_error)
    __Pyx_GOTREF(__pyx_t_3);
    __pyx_t_4 = NULL;
    if (CYTHON_UNPACK_METHODS && likely(PyMethod_Check(__pyx_t_3))) {
      __pyx_t_4 = PyMethod_GET_SELF(__pyx_t_3);
      if (likely(__pyx_t_4)) {
        PyObject* function = PyMethod_GET_FUNCTION(__pyx_t_3);
        __Pyx_INCREF(__pyx_t_4);
        __Pyx_INCREF(function);
        __Pyx_DECREF_SET(__pyx_t_3, function);
      }
    }
    if (__pyx_t_4) {
      __pyx_t_2 = __Pyx_PyObject_CallOneArg(__pyx_t_3, __pyx_t_4); if (unlikely(!__pyx_t_2)) __PYX_ERR(8, 106, __pyx_L3_error)
      __Pyx_DECREF(__pyx_t_4); __pyx_t_4 = 0;
    } else {
      __pyx_t_2 = __Pyx_PyObject_CallNoArg(__pyx_t_3); if (unlikely(!__pyx_t_2)) __PYX_ERR(8, 106, __pyx_L3_error)
    }
    __Pyx_GOTREF(__pyx_t_2);
    __Pyx_DECREF(__pyx_t_3); __pyx_t_3 = 0;
    __Pyx_DECREF(__pyx_t_2); __pyx_t_2 = 0;
    /*try:*/ {
      {
        __Pyx_PyThreadState_declare
        __Pyx_PyThreadState_assign
        __Pyx_ExceptionSave(&__pyx_t_5, &__pyx_t_6, &__pyx_t_7);
        __Pyx_XGOTREF(__pyx_t_5);
        __Pyx_XGOTREF(__pyx_t_6);
        __Pyx_XGOTREF(__pyx_t_7);
        /*try:*/ {

          /* "src/python/grpcio/grpc/_cython/_cygrpc/channel.pyx.pxi":107
 *   cdef _BatchOperationTag tag
 *   with channel_state.condition:
 *     if call_state.due:             # <<<<<<<<<<<<<<
 *       c_call_error, tag = _operate(call_state.c_call, operations, user_tag)
 *       if c_call_error == GRPC_CALL_OK:
 */
          __pyx_t_8 = (__pyx_v_call_state->due != Py_None)&&(PySet_GET_SIZE(__pyx_v_call_state->due) != 0);
          if (__pyx_t_8) {

            /* "src/python/grpcio/grpc/_cython/_cygrpc/channel.pyx.pxi":108
 *   with channel_state.condition:
 *     if call_state.due:
 *       c_call_error, tag = _operate(call_state.c_call, operations, user_tag)             # <<<<<<<<<<<<<<
 *       if c_call_error == GRPC_CALL_OK:
 *         call_state.due.add(tag)
 */
            __pyx_t_2 = __pyx_f_4grpc_7_cython_6cygrpc__operate(__pyx_v_call_state->c_call, __pyx_v_operations, __pyx_v_user_tag); if (unlikely(!__pyx_t_2)) __PYX_ERR(8, 108, __pyx_L7_error)
            __Pyx_GOTREF(__pyx_t_2);
            if (likely(__pyx_t_2 != Py_None)) {
              PyObject* sequence = __pyx_t_2;
              Py_ssize_t size = __Pyx_PySequence_SIZE(sequence);
              if (unlikely(size != 2)) {
                if (size > 2) __Pyx_RaiseTooManyValuesError(2);
                else if (size >= 0) __Pyx_RaiseNeedMoreValuesError(size);
                __PYX_ERR(8, 108, __pyx_L7_error)
              }
              #if CYTHON_ASSUME_SAFE_MACROS && !CYTHON_AVOID_BORROWED_REFS
              __pyx_t_3 = PyTuple_GET_ITEM(sequence, 0); 
              __pyx_t_4 = PyTuple_GET_ITEM(sequence, 1); 
              __Pyx_INCREF(__pyx_t_3);
              __Pyx_INCREF(__pyx_t_4);
              #else
              __pyx_t_3 = PySequence_ITEM(sequence, 0); if (unlikely(!__pyx_t_3)) __PYX_ERR(8, 108, __pyx_L7_error)
              __Pyx_GOTREF(__pyx_t_3);
              __pyx_t_4 = PySequence_ITEM(sequence, 1); if (unlikely(!__pyx_t_4)) __PYX_ERR(8, 108, __pyx_L7_error)
              __Pyx_GOTREF(__pyx_t_4);
              #endif
              __Pyx_DECREF(__pyx_t_2); __pyx_t_2 = 0;
            } else {
              __Pyx_RaiseNoneNotIterableError(); __PYX_ERR(8, 108, __pyx_L7_error)
            }
            __pyx_t_9 = ((grpc_call_error)__Pyx_PyInt_As_grpc_call_error(__pyx_t_3)); if (unlikely(PyErr_Occurred())) __PYX_ERR(8, 108, __pyx_L7_error)
            __Pyx_DECREF(__pyx_t_3); __pyx_t_3 = 0;
            if (!(likely(((__pyx_t_4) == Py_None) || likely(__Pyx_TypeTest(__pyx_t_4, __pyx_ptype_4grpc_7_cython_6cygrpc__BatchOperationTag))))) __PYX_ERR(8, 108, __pyx_L7_error)
            __pyx_v_c_call_error = __pyx_t_9;
            __pyx_v_tag = ((struct __pyx_obj_4grpc_7_cython_6cygrpc__BatchOperationTag *)__pyx_t_4);
            __pyx_t_4 = 0;

            /* "src/python/grpcio/grpc/_cython/_cygrpc/channel.pyx.pxi":109
 *     if call_state.due:
 *       c_call_error, tag = _operate(call_state.c_call, operations, user_tag)
 *       if c_call_error == GRPC_CALL_OK:             # <<<<<<<<<<<<<<
 *         call_state.due.add(tag)
 *         channel_state.integrated_call_states[tag] = call_state
 */
            __pyx_t_8 = ((__pyx_v_c_call_error == GRPC_CALL_OK) != 0);
            if (__pyx_t_8) {

              /* "src/python/grpcio/grpc/_cython/_cygrpc/channel.pyx.pxi":110
 *       c_call_error, tag = _operate(call_state.c_call, operations, user_tag)
 *       if c_call_error == GRPC_CALL_OK:
 *         call_state.due.add(tag)             # <<<<<<<<<<<<<<
 *         channel_state.integrated_call_states[tag] = call_state
 *         return True
 */
              if (unlikely(__pyx_v_call_state->due == Py_None)) {
                PyErr_Format(PyExc_AttributeError, "'NoneType' object has no attribute '%.30s'", "add");
                __PYX_ERR(8, 110, __pyx_L7_error)
              }
              __pyx_t_10 = PySet_Add(__pyx_v_call_state->due, ((PyObject *)__pyx_v_tag)); if (unlikely(__pyx_t_10 == ((int)-1))) __PYX_ERR(8, 110, __pyx_L7_error)

              /* "src/python/grpcio/grpc/_cython/_cygrpc/channel.pyx.pxi":111
 *       if c_call_error == GRPC_CALL_OK:
 *         call_state.due.add(tag)
 *         channel_state.integrated_call_states[tag] = call_state             # <<<<<<<<<<<<<<
 *         return True
 *       else:
 */
              if (unlikely(__pyx_v_channel_state->integrated_call_states == Py_None)) {
                PyErr_SetString(PyExc_TypeError, "'NoneType' object is not subscriptable");
                __PYX_ERR(8, 111, __pyx_L7_error)
              }
              if (unlikely(PyDict_SetItem(__pyx_v_channel_state->integrated_call_states, ((PyObject *)__pyx_v_tag), ((PyObject *)__pyx_v_call_state)) < 0)) __PYX_ERR(8, 111, __pyx_L7_error)

              /* "src/python/grpcio/grpc/_cython/_cygrpc/channel.pyx.pxi":112
 *         call_state.due.add(tag)
 *         channel_state.integrated_call_states[tag] = call_state
 *         return True             # <<<<<<<<<<<<<<
 *       else:
 *         _raise_call_error_no_metadata(c_call_error)
 */
              __Pyx_XDECREF(__pyx_r);
              __Pyx_INCREF(Py_True);
              __pyx_r = Py_True;
              goto __pyx_L11_try_return;

              /* "src/python/grpcio/grpc/_cython/_cygrpc/channel.pyx.pxi":109
 *     if call_state.due:
 *       c_call_error, tag = _operate(call_state.c_call, operations, user_tag)
 *       if c_call_error == GRPC_CALL_OK:             # <<<<<<<<<<<<<<
 *         call_state.due.add(tag)
 *         channel_state.integrated_call_states[tag] = call_state
 */
            }

            /* "src/python/grpcio/grpc/_cython/_cygrpc/channel.pyx.pxi":114
 *         return True
 *       else:
 *         _raise_call_error_no_metadata(c_call_error)             # <<<<<<<<<<<<<<
 *     else:
 *       return False
 */
            /*else*/ {
              __pyx_t_2 = __Pyx_PyInt_From_grpc_call_error(__pyx_v_c_call_error); if (unlikely(!__pyx_t_2)) __PYX_ERR(8, 114, __pyx_L7_error)
              __Pyx_GOTREF(__pyx_t_2);
              __pyx_f_4grpc_7_cython_6cygrpc__raise_call_error_no_metadata(__pyx_t_2); if (unlikely(PyErr_Occurred())) __PYX_ERR(8, 114, __pyx_L7_error)
              __Pyx_DECREF(__pyx_t_2); __pyx_t_2 = 0;
            }

            /* "src/python/grpcio/grpc/_cython/_cygrpc/channel.pyx.pxi":107
 *   cdef _BatchOperationTag tag
 *   with channel_state.condition:
 *     if call_state.due:             # <<<<<<<<<<<<<<
 *       c_call_error, tag = _operate(call_state.c_call, operations, user_tag)
 *       if c_call_error == GRPC_CALL_OK:
 */
            goto __pyx_L13;
          }

          /* "src/python/grpcio/grpc/_cython/_cygrpc/channel.pyx.pxi":116
 *         _raise_call_error_no_metadata(c_call_error)
 *     else:
 *       return False             # <<<<<<<<<<<<<<
 * 
 * 
 */
          /*else*/ {
            __Pyx_XDECREF(__pyx_r);
            __Pyx_INCREF(Py_False);
            __pyx_r = Py_False;
            goto __pyx_L11_try_return;
          }
          __pyx_L13:;

          /* "src/python/grpcio/grpc/_cython/_cygrpc/channel.pyx.pxi":106
 *   cdef grpc_call_error c_call_error
 *   cdef _BatchOperationTag tag
 *   with channel_state.condition:             # <<<<<<<<<<<<<<
 *     if call_state.due:
 *       c_call_error, tag = _operate(call_state.c_call, operations, user_tag)
 */
        }
        __Pyx_XDECREF(__pyx_t_5); __pyx_t_5 = 0;
        __Pyx_XDECREF(__pyx_t_6); __pyx_t_6 = 0;
        __Pyx_XDECREF(__pyx_t_7); __pyx_t_7 = 0;
        goto __pyx_L12_try_end;
        __pyx_L7_error:;
        __Pyx_XDECREF(__pyx_t_3); __pyx_t_3 = 0;
        __Pyx_XDECREF(__pyx_t_4); __pyx_t_4 = 0;
        __Pyx_XDECREF(__pyx_t_2); __pyx_t_2 = 0;
        /*except:*/ {
          __Pyx_AddTraceback("grpc._cython.cygrpc._operate_from_integrated_call", __pyx_clineno, __pyx_lineno, __pyx_filename);
          if (__Pyx_GetException(&__pyx_t_2, &__pyx_t_4, &__pyx_t_3) < 0) __PYX_ERR(8, 106, __pyx_L9_except_error)
          __Pyx_GOTREF(__pyx_t_2);
          __Pyx_GOTREF(__pyx_t_4);
          __Pyx_GOTREF(__pyx_t_3);
          __pyx_t_11 = PyTuple_Pack(3, __pyx_t_2, __pyx_t_4, __pyx_t_3); if (unlikely(!__pyx_t_11)) __PYX_ERR(8, 106, __pyx_L9_except_error)
          __Pyx_GOTREF(__pyx_t_11);
          __pyx_t_12 = __Pyx_PyObject_Call(__pyx_t_1, __pyx_t_11, NULL);
          __Pyx_DECREF(__pyx_t_1); __pyx_t_1 = 0;
          __Pyx_DECREF(__pyx_t_11); __pyx_t_11 = 0;
          if (unlikely(!__pyx_t_12)) __PYX_ERR(8, 106, __pyx_L9_except_error)
          __Pyx_GOTREF(__pyx_t_12);
          __pyx_t_8 = __Pyx_PyObject_IsTrue(__pyx_t_12);
          __Pyx_DECREF(__pyx_t_12); __pyx_t_12 = 0;
          if (__pyx_t_8 < 0) __PYX_ERR(8, 106, __pyx_L9_except_error)
          __pyx_t_13 = ((!(__pyx_t_8 != 0)) != 0);
          if (__pyx_t_13) {
            __Pyx_GIVEREF(__pyx_t_2);
            __Pyx_GIVEREF(__pyx_t_4);
            __Pyx_XGIVEREF(__pyx_t_3);
            __Pyx_ErrRestoreWithState(__pyx_t_2, __pyx_t_4, __pyx_t_3);
            __pyx_t_2 = 0; __pyx_t_4 = 0; __pyx_t_3 = 0; 
            __PYX_ERR(8, 106, __pyx_L9_except_error)
          }
          __Pyx_DECREF(__pyx_t_2); __pyx_t_2 = 0;
          __Pyx_DECREF(__pyx_t_4); __pyx_t_4 = 0;
          __Pyx_DECREF(__pyx_t_3); __pyx_t_3 = 0;
          goto __pyx_L8_exception_handled;
        }
        __pyx_L9_except_error:;
        __Pyx_XGIVEREF(__pyx_t_5);
        __Pyx_XGIVEREF(__pyx_t_6);
        __Pyx_XGIVEREF(__pyx_t_7);
        __Pyx_ExceptionReset(__pyx_t_5, __pyx_t_6, __pyx_t_7);
        goto __pyx_L1_error;
        __pyx_L11_try_return:;
        __Pyx_XGIVEREF(__pyx_t_5);
        __Pyx_XGIVEREF(__pyx_t_6);
        __Pyx_XGIVEREF(__pyx_t_7);
        __Pyx_ExceptionReset(__pyx_t_5, __pyx_t_6, __pyx_t_7);
        goto __pyx_L4_return;
        __pyx_L8_exception_handled:;
        __Pyx_XGIVEREF(__pyx_t_5);
        __Pyx_XGIVEREF(__pyx_t_6);
        __Pyx_XGIVEREF(__pyx_t_7);
        __Pyx_ExceptionReset(__pyx_t_5, __pyx_t_6, __pyx_t_7);
        __pyx_L12_try_end:;
      }
    }
    /*finally:*/ {
      /*normal exit:*/{
        if (__pyx_t_1) {
          __pyx_t_7 = __Pyx_PyObject_Call(__pyx_t_1, __pyx_tuple__20, NULL);
          __Pyx_DECREF(__pyx_t_1); __pyx_t_1 = 0;
          if (unlikely(!__pyx_t_7)) __PYX_ERR(8, 106, __pyx_L1_error)
          __Pyx_GOTREF(__pyx_t_7);
          __Pyx_DECREF(__pyx_t_7); __pyx_t_7 = 0;
        }
        goto __pyx_L6;
      }
      __pyx_L4_return: {
        __pyx_t_7 = __pyx_r;
        __pyx_r = 0;
        if (__pyx_t_1) {
          __pyx_t_6 = __Pyx_PyObject_Call(__pyx_t_1, __pyx_tuple__21, NULL);
          __Pyx_DECREF(__pyx_t_1); __pyx_t_1 = 0;
          if (unlikely(!__pyx_t_6)) __PYX_ERR(8, 106, __pyx_L1_error)
          __Pyx_GOTREF(__pyx_t_6);
          __Pyx_DECREF(__pyx_t_6); __pyx_t_6 = 0;
        }
        __pyx_r = __pyx_t_7;
        __pyx_t_7 = 0;
        goto __pyx_L0;
      }
      __pyx_L6:;
    }
    goto __pyx_L18;
    __pyx_L3_error:;
    __Pyx_DECREF(__pyx_t_1); __pyx_t_1 = 0;
    goto __pyx_L1_error;
    __pyx_L18:;
  }

  /* "src/python/grpcio/grpc/_cython/_cygrpc/channel.pyx.pxi":101
 * 
 * 
 * cdef object _operate_from_integrated_call(             # <<<<<<<<<<<<<<
 *     _ChannelState channel_state, _CallState call_state, object operations,
 *     object user_tag):
 */

  /* function exit code */
  __pyx_r = Py_None; __Pyx_INCREF(Py_None);
  goto __pyx_L0;
  __pyx_L1_error:;
  __Pyx_XDECREF(__pyx_t_2);
  __Pyx_XDECREF(__pyx_t_3);
  __Pyx_XDECREF(__pyx_t_4);
  __Pyx_XDECREF(__pyx_t_11);
  __Pyx_AddTraceback("grpc._cython.cygrpc._operate_from_integrated_call", __pyx_clineno, __pyx_lineno, __pyx_filename);
  __pyx_r = 0;
  __pyx_L0:;
  __Pyx_XDECREF((PyObject *)__pyx_v_tag);
  __Pyx_XGIVEREF(__pyx_r);
  __Pyx_RefNannyFinishContext();
  return __pyx_r;
}

/* "src/python/grpcio/grpc/_cython/_cygrpc/channel.pyx.pxi":119
 * 
 * 
 * cdef object _operate_from_segregated_call(             # <<<<<<<<<<<<<<
 *     _ChannelState channel_state, _CallState call_state, object operations,
 *     object user_tag):
 */

static PyObject *__pyx_f_4grpc_7_cython_6cygrpc__operate_from_segregated_call(struct __pyx_obj_4grpc_7_cython_6cygrpc__ChannelState *__pyx_v_channel_state, struct __pyx_obj_4grpc_7_cython_6cygrpc__CallState *__pyx_v_call_state, PyObject *__pyx_v_operations, PyObject *__pyx_v_user_tag) {
  grpc_call_error __pyx_v_c_call_error;
  struct __pyx_obj_4grpc_7_cython_6cygrpc__BatchOperationTag *__pyx_v_tag = 0;
  PyObject *__pyx_r = NULL;
  __Pyx_RefNannyDeclarations
  PyObject *__pyx_t_1 = NULL;
  PyObject *__pyx_t_2 = NULL;
  PyObject *__pyx_t_3 = NULL;
  PyObject *__pyx_t_4 = NULL;
  PyObject *__pyx_t_5 = NULL;
  PyObject *__pyx_t_6 = NULL;
  PyObject *__pyx_t_7 = NULL;
  int __pyx_t_8;
  grpc_call_error __pyx_t_9;
  int __pyx_t_10;
  PyObject *__pyx_t_11 = NULL;
  PyObject *__pyx_t_12 = NULL;
  int __pyx_t_13;
  __Pyx_RefNannySetupContext("_operate_from_segregated_call", 0);

  /* "src/python/grpcio/grpc/_cython/_cygrpc/channel.pyx.pxi":124
 *   cdef grpc_call_error c_call_error
 *   cdef _BatchOperationTag tag
 *   with channel_state.condition:             # <<<<<<<<<<<<<<
 *     if call_state.due:
 *       c_call_error, tag = _operate(call_state.c_call, operations, user_tag)
 */
  /*with:*/ {
    __pyx_t_1 = __Pyx_PyObject_LookupSpecial(__pyx_v_channel_state->condition, __pyx_n_s_exit); if (unlikely(!__pyx_t_1)) __PYX_ERR(8, 124, __pyx_L1_error)
    __Pyx_GOTREF(__pyx_t_1);
    __pyx_t_3 = __Pyx_PyObject_LookupSpecial(__pyx_v_channel_state->condition, __pyx_n_s_enter); if (unlikely(!__pyx_t_3)) __PYX_ERR(8, 124, __pyx_L3_error)
    __Pyx_GOTREF(__pyx_t_3);
    __pyx_t_4 = NULL;
    if (CYTHON_UNPACK_METHODS && likely(PyMethod_Check(__pyx_t_3))) {
      __pyx_t_4 = PyMethod_GET_SELF(__pyx_t_3);
      if (likely(__pyx_t_4)) {
        PyObject* function = PyMethod_GET_FUNCTION(__pyx_t_3);
        __Pyx_INCREF(__pyx_t_4);
        __Pyx_INCREF(function);
        __Pyx_DECREF_SET(__pyx_t_3, function);
      }
    }
    if (__pyx_t_4) {
      __pyx_t_2 = __Pyx_PyObject_CallOneArg(__pyx_t_3, __pyx_t_4); if (unlikely(!__pyx_t_2)) __PYX_ERR(8, 124, __pyx_L3_error)
      __Pyx_DECREF(__pyx_t_4); __pyx_t_4 = 0;
    } else {
      __pyx_t_2 = __Pyx_PyObject_CallNoArg(__pyx_t_3); if (unlikely(!__pyx_t_2)) __PYX_ERR(8, 124, __pyx_L3_error)
    }
    __Pyx_GOTREF(__pyx_t_2);
    __Pyx_DECREF(__pyx_t_3); __pyx_t_3 = 0;
    __Pyx_DECREF(__pyx_t_2); __pyx_t_2 = 0;
    /*try:*/ {
      {
        __Pyx_PyThreadState_declare
        __Pyx_PyThreadState_assign
        __Pyx_ExceptionSave(&__pyx_t_5, &__pyx_t_6, &__pyx_t_7);
        __Pyx_XGOTREF(__pyx_t_5);
        __Pyx_XGOTREF(__pyx_t_6);
        __Pyx_XGOTREF(__pyx_t_7);
        /*try:*/ {

          /* "src/python/grpcio/grpc/_cython/_cygrpc/channel.pyx.pxi":125
 *   cdef _BatchOperationTag tag
 *   with channel_state.condition:
 *     if call_state.due:             # <<<<<<<<<<<<<<
 *       c_call_error, tag = _operate(call_state.c_call, operations, user_tag)
 *       if c_call_error == GRPC_CALL_OK:
 */
          __pyx_t_8 = (__pyx_v_call_state->due != Py_None)&&(PySet_GET_SIZE(__pyx_v_call_state->due) != 0);
          if (__pyx_t_8) {

            /* "src/python/grpcio/grpc/_cython/_cygrpc/channel.pyx.pxi":126
 *   with channel_state.condition:
 *     if call_state.due:
 *       c_call_error, tag = _operate(call_state.c_call, operations, user_tag)             # <<<<<<<<<<<<<<
 *       if c_call_error == GRPC_CALL_OK:
 *         call_state.due.add(tag)
 */
            __pyx_t_2 = __pyx_f_4grpc_7_cython_6cygrpc__operate(__pyx_v_call_state->c_call, __pyx_v_operations, __pyx_v_user_tag); if (unlikely(!__pyx_t_2)) __PYX_ERR(8, 126, __pyx_L7_error)
            __Pyx_GOTREF(__pyx_t_2);
            if (likely(__pyx_t_2 != Py_None)) {
              PyObject* sequence = __pyx_t_2;
              Py_ssize_t size = __Pyx_PySequence_SIZE(sequence);
              if (unlikely(size != 2)) {
                if (size > 2) __Pyx_RaiseTooManyValuesError(2);
                else if (size >= 0) __Pyx_RaiseNeedMoreValuesError(size);
                __PYX_ERR(8, 126, __pyx_L7_error)
              }
              #if CYTHON_ASSUME_SAFE_MACROS && !CYTHON_AVOID_BORROWED_REFS
              __pyx_t_3 = PyTuple_GET_ITEM(sequence, 0); 
              __pyx_t_4 = PyTuple_GET_ITEM(sequence, 1); 
              __Pyx_INCREF(__pyx_t_3);
              __Pyx_INCREF(__pyx_t_4);
              #else
              __pyx_t_3 = PySequence_ITEM(sequence, 0); if (unlikely(!__pyx_t_3)) __PYX_ERR(8, 126, __pyx_L7_error)
              __Pyx_GOTREF(__pyx_t_3);
              __pyx_t_4 = PySequence_ITEM(sequence, 1); if (unlikely(!__pyx_t_4)) __PYX_ERR(8, 126, __pyx_L7_error)
              __Pyx_GOTREF(__pyx_t_4);
              #endif
              __Pyx_DECREF(__pyx_t_2); __pyx_t_2 = 0;
            } else {
              __Pyx_RaiseNoneNotIterableError(); __PYX_ERR(8, 126, __pyx_L7_error)
            }
            __pyx_t_9 = ((grpc_call_error)__Pyx_PyInt_As_grpc_call_error(__pyx_t_3)); if (unlikely(PyErr_Occurred())) __PYX_ERR(8, 126, __pyx_L7_error)
            __Pyx_DECREF(__pyx_t_3); __pyx_t_3 = 0;
            if (!(likely(((__pyx_t_4) == Py_None) || likely(__Pyx_TypeTest(__pyx_t_4, __pyx_ptype_4grpc_7_cython_6cygrpc__BatchOperationTag))))) __PYX_ERR(8, 126, __pyx_L7_error)
            __pyx_v_c_call_error = __pyx_t_9;
            __pyx_v_tag = ((struct __pyx_obj_4grpc_7_cython_6cygrpc__BatchOperationTag *)__pyx_t_4);
            __pyx_t_4 = 0;

            /* "src/python/grpcio/grpc/_cython/_cygrpc/channel.pyx.pxi":127
 *     if call_state.due:
 *       c_call_error, tag = _operate(call_state.c_call, operations, user_tag)
 *       if c_call_error == GRPC_CALL_OK:             # <<<<<<<<<<<<<<
 *         call_state.due.add(tag)
 *         return True
 */
            __pyx_t_8 = ((__pyx_v_c_call_error == GRPC_CALL_OK) != 0);
            if (__pyx_t_8) {

              /* "src/python/grpcio/grpc/_cython/_cygrpc/channel.pyx.pxi":128
 *       c_call_error, tag = _operate(call_state.c_call, operations, user_tag)
 *       if c_call_error == GRPC_CALL_OK:
 *         call_state.due.add(tag)             # <<<<<<<<<<<<<<
 *         return True
 *       else:
 */
              if (unlikely(__pyx_v_call_state->due == Py_None)) {
                PyErr_Format(PyExc_AttributeError, "'NoneType' object has no attribute '%.30s'", "add");
                __PYX_ERR(8, 128, __pyx_L7_error)
              }
              __pyx_t_10 = PySet_Add(__pyx_v_call_state->due, ((PyObject *)__pyx_v_tag)); if (unlikely(__pyx_t_10 == ((int)-1))) __PYX_ERR(8, 128, __pyx_L7_error)

              /* "src/python/grpcio/grpc/_cython/_cygrpc/channel.pyx.pxi":129
 *       if c_call_error == GRPC_CALL_OK:
 *         call_state.due.add(tag)
 *         return True             # <<<<<<<<<<<<<<
 *       else:
 *         _raise_call_error_no_metadata(c_call_error)
 */
              __Pyx_XDECREF(__pyx_r);
              __Pyx_INCREF(Py_True);
              __pyx_r = Py_True;
              goto __pyx_L11_try_return;

              /* "src/python/grpcio/grpc/_cython/_cygrpc/channel.pyx.pxi":127
 *     if call_state.due:
 *       c_call_error, tag = _operate(call_state.c_call, operations, user_tag)
 *       if c_call_error == GRPC_CALL_OK:             # <<<<<<<<<<<<<<
 *         call_state.due.add(tag)
 *         return True
 */
            }

            /* "src/python/grpcio/grpc/_cython/_cygrpc/channel.pyx.pxi":131
 *         return True
 *       else:
 *         _raise_call_error_no_metadata(c_call_error)             # <<<<<<<<<<<<<<
 *     else:
 *       return False
 */
            /*else*/ {
              __pyx_t_2 = __Pyx_PyInt_From_grpc_call_error(__pyx_v_c_call_error); if (unlikely(!__pyx_t_2)) __PYX_ERR(8, 131, __pyx_L7_error)
              __Pyx_GOTREF(__pyx_t_2);
              __pyx_f_4grpc_7_cython_6cygrpc__raise_call_error_no_metadata(__pyx_t_2); if (unlikely(PyErr_Occurred())) __PYX_ERR(8, 131, __pyx_L7_error)
              __Pyx_DECREF(__pyx_t_2); __pyx_t_2 = 0;
            }

            /* "src/python/grpcio/grpc/_cython/_cygrpc/channel.pyx.pxi":125
 *   cdef _BatchOperationTag tag
 *   with channel_state.condition:
 *     if call_state.due:             # <<<<<<<<<<<<<<
 *       c_call_error, tag = _operate(call_state.c_call, operations, user_tag)
 *       if c_call_error == GRPC_CALL_OK:
 */
            goto __pyx_L13;
          }

          /* "src/python/grpcio/grpc/_cython/_cygrpc/channel.pyx.pxi":133
 *         _raise_call_error_no_metadata(c_call_error)
 *     else:
 *       return False             # <<<<<<<<<<<<<<
 * 
 * 
 */
          /*else*/ {
            __Pyx_XDECREF(__pyx_r);
            __Pyx_INCREF(Py_False);
            __pyx_r = Py_False;
            goto __pyx_L11_try_return;
          }
          __pyx_L13:;

          /* "src/python/grpcio/grpc/_cython/_cygrpc/channel.pyx.pxi":124
 *   cdef grpc_call_error c_call_error
 *   cdef _BatchOperationTag tag
 *   with channel_state.condition:             # <<<<<<<<<<<<<<
 *     if call_state.due:
 *       c_call_error, tag = _operate(call_state.c_call, operations, user_tag)
 */
        }
        __Pyx_XDECREF(__pyx_t_5); __pyx_t_5 = 0;
        __Pyx_XDECREF(__pyx_t_6); __pyx_t_6 = 0;
        __Pyx_XDECREF(__pyx_t_7); __pyx_t_7 = 0;
        goto __pyx_L12_try_end;
        __pyx_L7_error:;
        __Pyx_XDECREF(__pyx_t_3); __pyx_t_3 = 0;
        __Pyx_XDECREF(__pyx_t_4); __pyx_t_4 = 0;
        __Pyx_XDECREF(__pyx_t_2); __pyx_t_2 = 0;
        /*except:*/ {
          __Pyx_AddTraceback("grpc._cython.cygrpc._operate_from_segregated_call", __pyx_clineno, __pyx_lineno, __pyx_filename);
          if (__Pyx_GetException(&__pyx_t_2, &__pyx_t_4, &__pyx_t_3) < 0) __PYX_ERR(8, 124, __pyx_L9_except_error)
          __Pyx_GOTREF(__pyx_t_2);
          __Pyx_GOTREF(__pyx_t_4);
          __Pyx_GOTREF(__pyx_t_3);
          __pyx_t_11 = PyTuple_Pack(3, __pyx_t_2, __pyx_t_4, __pyx_t_3); if (unlikely(!__pyx_t_11)) __PYX_ERR(8, 124, __pyx_L9_except_error)
          __Pyx_GOTREF(__pyx_t_11);
          __pyx_t_12 = __Pyx_PyObject_Call(__pyx_t_1, __pyx_t_11, NULL);
          __Pyx_DECREF(__pyx_t_1); __pyx_t_1 = 0;
          __Pyx_DECREF(__pyx_t_11); __pyx_t_11 = 0;
          if (unlikely(!__pyx_t_12)) __PYX_ERR(8, 124, __pyx_L9_except_error)
          __Pyx_GOTREF(__pyx_t_12);
          __pyx_t_8 = __Pyx_PyObject_IsTrue(__pyx_t_12);
          __Pyx_DECREF(__pyx_t_12); __pyx_t_12 = 0;
          if (__pyx_t_8 < 0) __PYX_ERR(8, 124, __pyx_L9_except_error)
          __pyx_t_13 = ((!(__pyx_t_8 != 0)) != 0);
          if (__pyx_t_13) {
            __Pyx_GIVEREF(__pyx_t_2);
            __Pyx_GIVEREF(__pyx_t_4);
            __Pyx_XGIVEREF(__pyx_t_3);
            __Pyx_ErrRestoreWithState(__pyx_t_2, __pyx_t_4, __pyx_t_3);
            __pyx_t_2 = 0; __pyx_t_4 = 0; __pyx_t_3 = 0; 
            __PYX_ERR(8, 124, __pyx_L9_except_error)
          }
          __Pyx_DECREF(__pyx_t_2); __pyx_t_2 = 0;
          __Pyx_DECREF(__pyx_t_4); __pyx_t_4 = 0;
          __Pyx_DECREF(__pyx_t_3); __pyx_t_3 = 0;
          goto __pyx_L8_exception_handled;
        }
        __pyx_L9_except_error:;
        __Pyx_XGIVEREF(__pyx_t_5);
        __Pyx_XGIVEREF(__pyx_t_6);
        __Pyx_XGIVEREF(__pyx_t_7);
        __Pyx_ExceptionReset(__pyx_t_5, __pyx_t_6, __pyx_t_7);
        goto __pyx_L1_error;
        __pyx_L11_try_return:;
        __Pyx_XGIVEREF(__pyx_t_5);
        __Pyx_XGIVEREF(__pyx_t_6);
        __Pyx_XGIVEREF(__pyx_t_7);
        __Pyx_ExceptionReset(__pyx_t_5, __pyx_t_6, __pyx_t_7);
        goto __pyx_L4_return;
        __pyx_L8_exception_handled:;
        __Pyx_XGIVEREF(__pyx_t_5);
        __Pyx_XGIVEREF(__pyx_t_6);
        __Pyx_XGIVEREF(__pyx_t_7);
        __Pyx_ExceptionReset(__pyx_t_5, __pyx_t_6, __pyx_t_7);
        __pyx_L12_try_end:;
      }
    }
    /*finally:*/ {
      /*normal exit:*/{
        if (__pyx_t_1) {
          __pyx_t_7 = __Pyx_PyObject_Call(__pyx_t_1, __pyx_tuple__22, NULL);
          __Pyx_DECREF(__pyx_t_1); __pyx_t_1 = 0;
          if (unlikely(!__pyx_t_7)) __PYX_ERR(8, 124, __pyx_L1_error)
          __Pyx_GOTREF(__pyx_t_7);
          __Pyx_DECREF(__pyx_t_7); __pyx_t_7 = 0;
        }
        goto __pyx_L6;
      }
      __pyx_L4_return: {
        __pyx_t_7 = __pyx_r;
        __pyx_r = 0;
        if (__pyx_t_1) {
          __pyx_t_6 = __Pyx_PyObject_Call(__pyx_t_1, __pyx_tuple__23, NULL);
          __Pyx_DECREF(__pyx_t_1); __pyx_t_1 = 0;
          if (unlikely(!__pyx_t_6)) __PYX_ERR(8, 124, __pyx_L1_error)
          __Pyx_GOTREF(__pyx_t_6);
          __Pyx_DECREF(__pyx_t_6); __pyx_t_6 = 0;
        }
        __pyx_r = __pyx_t_7;
        __pyx_t_7 = 0;
        goto __pyx_L0;
      }
      __pyx_L6:;
    }
    goto __pyx_L18;
    __pyx_L3_error:;
    __Pyx_DECREF(__pyx_t_1); __pyx_t_1 = 0;
    goto __pyx_L1_error;
    __pyx_L18:;
  }

  /* "src/python/grpcio/grpc/_cython/_cygrpc/channel.pyx.pxi":119
 * 
 * 
 * cdef object _operate_from_segregated_call(             # <<<<<<<<<<<<<<
 *     _ChannelState channel_state, _CallState call_state, object operations,
 *     object user_tag):
 */

  /* function exit code */
  __pyx_r = Py_None; __Pyx_INCREF(Py_None);
  goto __pyx_L0;
  __pyx_L1_error:;
  __Pyx_XDECREF(__pyx_t_2);
  __Pyx_XDECREF(__pyx_t_3);
  __Pyx_XDECREF(__pyx_t_4);
  __Pyx_XDECREF(__pyx_t_11);
  __Pyx_AddTraceback("grpc._cython.cygrpc._operate_from_segregated_call", __pyx_clineno, __pyx_lineno, __pyx_filename);
  __pyx_r = 0;
  __pyx_L0:;
  __Pyx_XDECREF((PyObject *)__pyx_v_tag);
  __Pyx_XGIVEREF(__pyx_r);
  __Pyx_RefNannyFinishContext();
  return __pyx_r;
}

/* "src/python/grpcio/grpc/_cython/_cygrpc/channel.pyx.pxi":136
 * 
 * 
 * cdef _cancel(             # <<<<<<<<<<<<<<
 *     _ChannelState channel_state, _CallState call_state, grpc_status_code code,
 *     str details):
 */

static PyObject *__pyx_f_4grpc_7_cython_6cygrpc__cancel(struct __pyx_obj_4grpc_7_cython_6cygrpc__ChannelState *__pyx_v_channel_state, struct __pyx_obj_4grpc_7_cython_6cygrpc__CallState *__pyx_v_call_state, grpc_status_code __pyx_v_code, PyObject *__pyx_v_details) {
  grpc_call_error __pyx_v_c_call_error;
  PyObject *__pyx_r = NULL;
  __Pyx_RefNannyDeclarations
  PyObject *__pyx_t_1 = NULL;
  PyObject *__pyx_t_2 = NULL;
  PyObject *__pyx_t_3 = NULL;
  PyObject *__pyx_t_4 = NULL;
  PyObject *__pyx_t_5 = NULL;
  PyObject *__pyx_t_6 = NULL;
  PyObject *__pyx_t_7 = NULL;
  int __pyx_t_8;
  char const *__pyx_t_9;
  PyObject *__pyx_t_10 = NULL;
  PyObject *__pyx_t_11 = NULL;
  int __pyx_t_12;
  __Pyx_RefNannySetupContext("_cancel", 0);

  /* "src/python/grpcio/grpc/_cython/_cygrpc/channel.pyx.pxi":140
 *     str details):
 *   cdef grpc_call_error c_call_error
 *   with channel_state.condition:             # <<<<<<<<<<<<<<
 *     if call_state.due:
 *       c_call_error = grpc_call_cancel_with_status(
 */
  /*with:*/ {
    __pyx_t_1 = __Pyx_PyObject_LookupSpecial(__pyx_v_channel_state->condition, __pyx_n_s_exit); if (unlikely(!__pyx_t_1)) __PYX_ERR(8, 140, __pyx_L1_error)
    __Pyx_GOTREF(__pyx_t_1);
    __pyx_t_3 = __Pyx_PyObject_LookupSpecial(__pyx_v_channel_state->condition, __pyx_n_s_enter); if (unlikely(!__pyx_t_3)) __PYX_ERR(8, 140, __pyx_L3_error)
    __Pyx_GOTREF(__pyx_t_3);
    __pyx_t_4 = NULL;
    if (CYTHON_UNPACK_METHODS && likely(PyMethod_Check(__pyx_t_3))) {
      __pyx_t_4 = PyMethod_GET_SELF(__pyx_t_3);
      if (likely(__pyx_t_4)) {
        PyObject* function = PyMethod_GET_FUNCTION(__pyx_t_3);
        __Pyx_INCREF(__pyx_t_4);
        __Pyx_INCREF(function);
        __Pyx_DECREF_SET(__pyx_t_3, function);
      }
    }
    if (__pyx_t_4) {
      __pyx_t_2 = __Pyx_PyObject_CallOneArg(__pyx_t_3, __pyx_t_4); if (unlikely(!__pyx_t_2)) __PYX_ERR(8, 140, __pyx_L3_error)
      __Pyx_DECREF(__pyx_t_4); __pyx_t_4 = 0;
    } else {
      __pyx_t_2 = __Pyx_PyObject_CallNoArg(__pyx_t_3); if (unlikely(!__pyx_t_2)) __PYX_ERR(8, 140, __pyx_L3_error)
    }
    __Pyx_GOTREF(__pyx_t_2);
    __Pyx_DECREF(__pyx_t_3); __pyx_t_3 = 0;
    __Pyx_DECREF(__pyx_t_2); __pyx_t_2 = 0;
    /*try:*/ {
      {
        __Pyx_PyThreadState_declare
        __Pyx_PyThreadState_assign
        __Pyx_ExceptionSave(&__pyx_t_5, &__pyx_t_6, &__pyx_t_7);
        __Pyx_XGOTREF(__pyx_t_5);
        __Pyx_XGOTREF(__pyx_t_6);
        __Pyx_XGOTREF(__pyx_t_7);
        /*try:*/ {

          /* "src/python/grpcio/grpc/_cython/_cygrpc/channel.pyx.pxi":141
 *   cdef grpc_call_error c_call_error
 *   with channel_state.condition:
 *     if call_state.due:             # <<<<<<<<<<<<<<
 *       c_call_error = grpc_call_cancel_with_status(
 *           call_state.c_call, code, _encode(details), NULL)
 */
          __pyx_t_8 = (__pyx_v_call_state->due != Py_None)&&(PySet_GET_SIZE(__pyx_v_call_state->due) != 0);
          if (__pyx_t_8) {

            /* "src/python/grpcio/grpc/_cython/_cygrpc/channel.pyx.pxi":143
 *     if call_state.due:
 *       c_call_error = grpc_call_cancel_with_status(
 *           call_state.c_call, code, _encode(details), NULL)             # <<<<<<<<<<<<<<
 *       _check_and_raise_call_error_no_metadata(c_call_error)
 * 
 */
            __pyx_t_2 = __pyx_f_4grpc_7_cython_6cygrpc__encode(__pyx_v_details); if (unlikely(!__pyx_t_2)) __PYX_ERR(8, 143, __pyx_L7_error)
            __Pyx_GOTREF(__pyx_t_2);
            if (unlikely(__pyx_t_2 == Py_None)) {
              PyErr_SetString(PyExc_TypeError, "expected bytes, NoneType found");
              __PYX_ERR(8, 143, __pyx_L7_error)
            }
            __pyx_t_9 = __Pyx_PyBytes_AsString(__pyx_t_2); if (unlikely((!__pyx_t_9) && PyErr_Occurred())) __PYX_ERR(8, 143, __pyx_L7_error)

            /* "src/python/grpcio/grpc/_cython/_cygrpc/channel.pyx.pxi":142
 *   with channel_state.condition:
 *     if call_state.due:
 *       c_call_error = grpc_call_cancel_with_status(             # <<<<<<<<<<<<<<
 *           call_state.c_call, code, _encode(details), NULL)
 *       _check_and_raise_call_error_no_metadata(c_call_error)
 */
            __pyx_v_c_call_error = grpc_call_cancel_with_status(__pyx_v_call_state->c_call, __pyx_v_code, __pyx_t_9, NULL);
            __Pyx_DECREF(__pyx_t_2); __pyx_t_2 = 0;

            /* "src/python/grpcio/grpc/_cython/_cygrpc/channel.pyx.pxi":144
 *       c_call_error = grpc_call_cancel_with_status(
 *           call_state.c_call, code, _encode(details), NULL)
 *       _check_and_raise_call_error_no_metadata(c_call_error)             # <<<<<<<<<<<<<<
 * 
 * 
 */
            __pyx_t_2 = __Pyx_PyInt_From_grpc_call_error(__pyx_v_c_call_error); if (unlikely(!__pyx_t_2)) __PYX_ERR(8, 144, __pyx_L7_error)
            __Pyx_GOTREF(__pyx_t_2);
            __pyx_t_3 = __pyx_f_4grpc_7_cython_6cygrpc__check_and_raise_call_error_no_metadata(__pyx_t_2); if (unlikely(!__pyx_t_3)) __PYX_ERR(8, 144, __pyx_L7_error)
            __Pyx_GOTREF(__pyx_t_3);
            __Pyx_DECREF(__pyx_t_2); __pyx_t_2 = 0;
            __Pyx_DECREF(__pyx_t_3); __pyx_t_3 = 0;

            /* "src/python/grpcio/grpc/_cython/_cygrpc/channel.pyx.pxi":141
 *   cdef grpc_call_error c_call_error
 *   with channel_state.condition:
 *     if call_state.due:             # <<<<<<<<<<<<<<
 *       c_call_error = grpc_call_cancel_with_status(
 *           call_state.c_call, code, _encode(details), NULL)
 */
          }

          /* "src/python/grpcio/grpc/_cython/_cygrpc/channel.pyx.pxi":140
 *     str details):
 *   cdef grpc_call_error c_call_error
 *   with channel_state.condition:             # <<<<<<<<<<<<<<
 *     if call_state.due:
 *       c_call_error = grpc_call_cancel_with_status(
 */
        }
        __Pyx_XDECREF(__pyx_t_5); __pyx_t_5 = 0;
        __Pyx_XDECREF(__pyx_t_6); __pyx_t_6 = 0;
        __Pyx_XDECREF(__pyx_t_7); __pyx_t_7 = 0;
        goto __pyx_L12_try_end;
        __pyx_L7_error:;
        __Pyx_XDECREF(__pyx_t_4); __pyx_t_4 = 0;
        __Pyx_XDECREF(__pyx_t_2); __pyx_t_2 = 0;
        __Pyx_XDECREF(__pyx_t_3); __pyx_t_3 = 0;
        /*except:*/ {
          __Pyx_AddTraceback("grpc._cython.cygrpc._cancel", __pyx_clineno, __pyx_lineno, __pyx_filename);
          if (__Pyx_GetException(&__pyx_t_3, &__pyx_t_2, &__pyx_t_4) < 0) __PYX_ERR(8, 140, __pyx_L9_except_error)
          __Pyx_GOTREF(__pyx_t_3);
          __Pyx_GOTREF(__pyx_t_2);
          __Pyx_GOTREF(__pyx_t_4);
          __pyx_t_10 = PyTuple_Pack(3, __pyx_t_3, __pyx_t_2, __pyx_t_4); if (unlikely(!__pyx_t_10)) __PYX_ERR(8, 140, __pyx_L9_except_error)
          __Pyx_GOTREF(__pyx_t_10);
          __pyx_t_11 = __Pyx_PyObject_Call(__pyx_t_1, __pyx_t_10, NULL);
          __Pyx_DECREF(__pyx_t_1); __pyx_t_1 = 0;
          __Pyx_DECREF(__pyx_t_10); __pyx_t_10 = 0;
          if (unlikely(!__pyx_t_11)) __PYX_ERR(8, 140, __pyx_L9_except_error)
          __Pyx_GOTREF(__pyx_t_11);
          __pyx_t_8 = __Pyx_PyObject_IsTrue(__pyx_t_11);
          __Pyx_DECREF(__pyx_t_11); __pyx_t_11 = 0;
          if (__pyx_t_8 < 0) __PYX_ERR(8, 140, __pyx_L9_except_error)
          __pyx_t_12 = ((!(__pyx_t_8 != 0)) != 0);
          if (__pyx_t_12) {
            __Pyx_GIVEREF(__pyx_t_3);
            __Pyx_GIVEREF(__pyx_t_2);
            __Pyx_XGIVEREF(__pyx_t_4);
            __Pyx_ErrRestoreWithState(__pyx_t_3, __pyx_t_2, __pyx_t_4);
            __pyx_t_3 = 0; __pyx_t_2 = 0; __pyx_t_4 = 0; 
            __PYX_ERR(8, 140, __pyx_L9_except_error)
          }
          __Pyx_DECREF(__pyx_t_3); __pyx_t_3 = 0;
          __Pyx_DECREF(__pyx_t_2); __pyx_t_2 = 0;
          __Pyx_DECREF(__pyx_t_4); __pyx_t_4 = 0;
          goto __pyx_L8_exception_handled;
        }
        __pyx_L9_except_error:;
        __Pyx_XGIVEREF(__pyx_t_5);
        __Pyx_XGIVEREF(__pyx_t_6);
        __Pyx_XGIVEREF(__pyx_t_7);
        __Pyx_ExceptionReset(__pyx_t_5, __pyx_t_6, __pyx_t_7);
        goto __pyx_L1_error;
        __pyx_L8_exception_handled:;
        __Pyx_XGIVEREF(__pyx_t_5);
        __Pyx_XGIVEREF(__pyx_t_6);
        __Pyx_XGIVEREF(__pyx_t_7);
        __Pyx_ExceptionReset(__pyx_t_5, __pyx_t_6, __pyx_t_7);
        __pyx_L12_try_end:;
      }
    }
    /*finally:*/ {
      /*normal exit:*/{
        if (__pyx_t_1) {
          __pyx_t_7 = __Pyx_PyObject_Call(__pyx_t_1, __pyx_tuple__24, NULL);
          __Pyx_DECREF(__pyx_t_1); __pyx_t_1 = 0;
          if (unlikely(!__pyx_t_7)) __PYX_ERR(8, 140, __pyx_L1_error)
          __Pyx_GOTREF(__pyx_t_7);
          __Pyx_DECREF(__pyx_t_7); __pyx_t_7 = 0;
        }
        goto __pyx_L6;
      }
      __pyx_L6:;
    }
    goto __pyx_L17;
    __pyx_L3_error:;
    __Pyx_DECREF(__pyx_t_1); __pyx_t_1 = 0;
    goto __pyx_L1_error;
    __pyx_L17:;
  }

  /* "src/python/grpcio/grpc/_cython/_cygrpc/channel.pyx.pxi":136
 * 
 * 
 * cdef _cancel(             # <<<<<<<<<<<<<<
 *     _ChannelState channel_state, _CallState call_state, grpc_status_code code,
 *     str details):
 */

  /* function exit code */
  __pyx_r = Py_None; __Pyx_INCREF(Py_None);
  goto __pyx_L0;
  __pyx_L1_error:;
  __Pyx_XDECREF(__pyx_t_2);
  __Pyx_XDECREF(__pyx_t_3);
  __Pyx_XDECREF(__pyx_t_4);
  __Pyx_XDECREF(__pyx_t_10);
  __Pyx_AddTraceback("grpc._cython.cygrpc._cancel", __pyx_clineno, __pyx_lineno, __pyx_filename);
  __pyx_r = 0;
  __pyx_L0:;
  __Pyx_XGIVEREF(__pyx_r);
  __Pyx_RefNannyFinishContext();
  return __pyx_r;
}

/* "src/python/grpcio/grpc/_cython/_cygrpc/channel.pyx.pxi":147
 * 
 * 
 * cdef _next_call_event(             # <<<<<<<<<<<<<<
 *     _ChannelState channel_state, grpc_completion_queue *c_completion_queue,
 *     on_success, deadline):
 */

static PyObject *__pyx_f_4grpc_7_cython_6cygrpc__next_call_event(struct __pyx_obj_4grpc_7_cython_6cygrpc__ChannelState *__pyx_v_channel_state, grpc_completion_queue *__pyx_v_c_completion_queue, PyObject *__pyx_v_on_success, PyObject *__pyx_v_deadline) {
  PyObject *__pyx_v_tag = NULL;
  PyObject *__pyx_v_event = NULL;
  PyObject *__pyx_r = NULL;
  __Pyx_RefNannyDeclarations
  PyObject *__pyx_t_1 = NULL;
  PyObject *__pyx_t_2 = NULL;
  PyObject *__pyx_t_3 = NULL;
  PyObject *__pyx_t_4 = NULL;
  PyObject *(*__pyx_t_5)(PyObject *);
  PyObject *__pyx_t_6 = NULL;
  PyObject *__pyx_t_7 = NULL;
  PyObject *__pyx_t_8 = NULL;
  PyObject *__pyx_t_9 = NULL;
  PyObject *__pyx_t_10 = NULL;
  int __pyx_t_11;
  int __pyx_t_12;
  __Pyx_RefNannySetupContext("_next_call_event", 0);

  /* "src/python/grpcio/grpc/_cython/_cygrpc/channel.pyx.pxi":150
 *     _ChannelState channel_state, grpc_completion_queue *c_completion_queue,
 *     on_success, deadline):
 *   tag, event = _latent_event(c_completion_queue, deadline)             # <<<<<<<<<<<<<<
 *   with channel_state.condition:
 *     on_success(tag)
 */
  __pyx_t_1 = __pyx_f_4grpc_7_cython_6cygrpc__latent_event(__pyx_v_c_completion_queue, __pyx_v_deadline); if (unlikely(!__pyx_t_1)) __PYX_ERR(8, 150, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_1);
  if ((likely(PyTuple_CheckExact(__pyx_t_1))) || (PyList_CheckExact(__pyx_t_1))) {
    PyObject* sequence = __pyx_t_1;
    Py_ssize_t size = __Pyx_PySequence_SIZE(sequence);
    if (unlikely(size != 2)) {
      if (size > 2) __Pyx_RaiseTooManyValuesError(2);
      else if (size >= 0) __Pyx_RaiseNeedMoreValuesError(size);
      __PYX_ERR(8, 150, __pyx_L1_error)
    }
    #if CYTHON_ASSUME_SAFE_MACROS && !CYTHON_AVOID_BORROWED_REFS
    if (likely(PyTuple_CheckExact(sequence))) {
      __pyx_t_2 = PyTuple_GET_ITEM(sequence, 0); 
      __pyx_t_3 = PyTuple_GET_ITEM(sequence, 1); 
    } else {
      __pyx_t_2 = PyList_GET_ITEM(sequence, 0); 
      __pyx_t_3 = PyList_GET_ITEM(sequence, 1); 
    }
    __Pyx_INCREF(__pyx_t_2);
    __Pyx_INCREF(__pyx_t_3);
    #else
    __pyx_t_2 = PySequence_ITEM(sequence, 0); if (unlikely(!__pyx_t_2)) __PYX_ERR(8, 150, __pyx_L1_error)
    __Pyx_GOTREF(__pyx_t_2);
    __pyx_t_3 = PySequence_ITEM(sequence, 1); if (unlikely(!__pyx_t_3)) __PYX_ERR(8, 150, __pyx_L1_error)
    __Pyx_GOTREF(__pyx_t_3);
    #endif
    __Pyx_DECREF(__pyx_t_1); __pyx_t_1 = 0;
  } else {
    Py_ssize_t index = -1;
    __pyx_t_4 = PyObject_GetIter(__pyx_t_1); if (unlikely(!__pyx_t_4)) __PYX_ERR(8, 150, __pyx_L1_error)
    __Pyx_GOTREF(__pyx_t_4);
    __Pyx_DECREF(__pyx_t_1); __pyx_t_1 = 0;
    __pyx_t_5 = Py_TYPE(__pyx_t_4)->tp_iternext;
    index = 0; __pyx_t_2 = __pyx_t_5(__pyx_t_4); if (unlikely(!__pyx_t_2)) goto __pyx_L3_unpacking_failed;
    __Pyx_GOTREF(__pyx_t_2);
    index = 1; __pyx_t_3 = __pyx_t_5(__pyx_t_4); if (unlikely(!__pyx_t_3)) goto __pyx_L3_unpacking_failed;
    __Pyx_GOTREF(__pyx_t_3);
    if (__Pyx_IternextUnpackEndCheck(__pyx_t_5(__pyx_t_4), 2) < 0) __PYX_ERR(8, 150, __pyx_L1_error)
    __pyx_t_5 = NULL;
    __Pyx_DECREF(__pyx_t_4); __pyx_t_4 = 0;
    goto __pyx_L4_unpacking_done;
    __pyx_L3_unpacking_failed:;
    __Pyx_DECREF(__pyx_t_4); __pyx_t_4 = 0;
    __pyx_t_5 = NULL;
    if (__Pyx_IterFinish() == 0) __Pyx_RaiseNeedMoreValuesError(index);
    __PYX_ERR(8, 150, __pyx_L1_error)
    __pyx_L4_unpacking_done:;
  }
  __pyx_v_tag = __pyx_t_2;
  __pyx_t_2 = 0;
  __pyx_v_event = __pyx_t_3;
  __pyx_t_3 = 0;

  /* "src/python/grpcio/grpc/_cython/_cygrpc/channel.pyx.pxi":151
 *     on_success, deadline):
 *   tag, event = _latent_event(c_completion_queue, deadline)
 *   with channel_state.condition:             # <<<<<<<<<<<<<<
 *     on_success(tag)
 *     channel_state.condition.notify_all()
 */
  /*with:*/ {
    __pyx_t_6 = __Pyx_PyObject_LookupSpecial(__pyx_v_channel_state->condition, __pyx_n_s_exit); if (unlikely(!__pyx_t_6)) __PYX_ERR(8, 151, __pyx_L1_error)
    __Pyx_GOTREF(__pyx_t_6);
    __pyx_t_3 = __Pyx_PyObject_LookupSpecial(__pyx_v_channel_state->condition, __pyx_n_s_enter); if (unlikely(!__pyx_t_3)) __PYX_ERR(8, 151, __pyx_L5_error)
    __Pyx_GOTREF(__pyx_t_3);
    __pyx_t_2 = NULL;
    if (CYTHON_UNPACK_METHODS && likely(PyMethod_Check(__pyx_t_3))) {
      __pyx_t_2 = PyMethod_GET_SELF(__pyx_t_3);
      if (likely(__pyx_t_2)) {
        PyObject* function = PyMethod_GET_FUNCTION(__pyx_t_3);
        __Pyx_INCREF(__pyx_t_2);
        __Pyx_INCREF(function);
        __Pyx_DECREF_SET(__pyx_t_3, function);
      }
    }
    if (__pyx_t_2) {
      __pyx_t_1 = __Pyx_PyObject_CallOneArg(__pyx_t_3, __pyx_t_2); if (unlikely(!__pyx_t_1)) __PYX_ERR(8, 151, __pyx_L5_error)
      __Pyx_DECREF(__pyx_t_2); __pyx_t_2 = 0;
    } else {
      __pyx_t_1 = __Pyx_PyObject_CallNoArg(__pyx_t_3); if (unlikely(!__pyx_t_1)) __PYX_ERR(8, 151, __pyx_L5_error)
    }
    __Pyx_GOTREF(__pyx_t_1);
    __Pyx_DECREF(__pyx_t_3); __pyx_t_3 = 0;
    __Pyx_DECREF(__pyx_t_1); __pyx_t_1 = 0;
    /*try:*/ {
      {
        __Pyx_PyThreadState_declare
        __Pyx_PyThreadState_assign
        __Pyx_ExceptionSave(&__pyx_t_7, &__pyx_t_8, &__pyx_t_9);
        __Pyx_XGOTREF(__pyx_t_7);
        __Pyx_XGOTREF(__pyx_t_8);
        __Pyx_XGOTREF(__pyx_t_9);
        /*try:*/ {

          /* "src/python/grpcio/grpc/_cython/_cygrpc/channel.pyx.pxi":152
 *   tag, event = _latent_event(c_completion_queue, deadline)
 *   with channel_state.condition:
 *     on_success(tag)             # <<<<<<<<<<<<<<
 *     channel_state.condition.notify_all()
 *   return event
 */
          __Pyx_INCREF(__pyx_v_on_success);
          __pyx_t_3 = __pyx_v_on_success; __pyx_t_2 = NULL;
          if (CYTHON_UNPACK_METHODS && unlikely(PyMethod_Check(__pyx_t_3))) {
            __pyx_t_2 = PyMethod_GET_SELF(__pyx_t_3);
            if (likely(__pyx_t_2)) {
              PyObject* function = PyMethod_GET_FUNCTION(__pyx_t_3);
              __Pyx_INCREF(__pyx_t_2);
              __Pyx_INCREF(function);
              __Pyx_DECREF_SET(__pyx_t_3, function);
            }
          }
          if (!__pyx_t_2) {
            __pyx_t_1 = __Pyx_PyObject_CallOneArg(__pyx_t_3, __pyx_v_tag); if (unlikely(!__pyx_t_1)) __PYX_ERR(8, 152, __pyx_L9_error)
            __Pyx_GOTREF(__pyx_t_1);
          } else {
            #if CYTHON_FAST_PYCALL
            if (PyFunction_Check(__pyx_t_3)) {
              PyObject *__pyx_temp[2] = {__pyx_t_2, __pyx_v_tag};
              __pyx_t_1 = __Pyx_PyFunction_FastCall(__pyx_t_3, __pyx_temp+1-1, 1+1); if (unlikely(!__pyx_t_1)) __PYX_ERR(8, 152, __pyx_L9_error)
              __Pyx_XDECREF(__pyx_t_2); __pyx_t_2 = 0;
              __Pyx_GOTREF(__pyx_t_1);
            } else
            #endif
            #if CYTHON_FAST_PYCCALL
            if (__Pyx_PyFastCFunction_Check(__pyx_t_3)) {
              PyObject *__pyx_temp[2] = {__pyx_t_2, __pyx_v_tag};
              __pyx_t_1 = __Pyx_PyCFunction_FastCall(__pyx_t_3, __pyx_temp+1-1, 1+1); if (unlikely(!__pyx_t_1)) __PYX_ERR(8, 152, __pyx_L9_error)
              __Pyx_XDECREF(__pyx_t_2); __pyx_t_2 = 0;
              __Pyx_GOTREF(__pyx_t_1);
            } else
            #endif
            {
              __pyx_t_4 = PyTuple_New(1+1); if (unlikely(!__pyx_t_4)) __PYX_ERR(8, 152, __pyx_L9_error)
              __Pyx_GOTREF(__pyx_t_4);
              __Pyx_GIVEREF(__pyx_t_2); PyTuple_SET_ITEM(__pyx_t_4, 0, __pyx_t_2); __pyx_t_2 = NULL;
              __Pyx_INCREF(__pyx_v_tag);
              __Pyx_GIVEREF(__pyx_v_tag);
              PyTuple_SET_ITEM(__pyx_t_4, 0+1, __pyx_v_tag);
              __pyx_t_1 = __Pyx_PyObject_Call(__pyx_t_3, __pyx_t_4, NULL); if (unlikely(!__pyx_t_1)) __PYX_ERR(8, 152, __pyx_L9_error)
              __Pyx_GOTREF(__pyx_t_1);
              __Pyx_DECREF(__pyx_t_4); __pyx_t_4 = 0;
            }
          }
          __Pyx_DECREF(__pyx_t_3); __pyx_t_3 = 0;
          __Pyx_DECREF(__pyx_t_1); __pyx_t_1 = 0;

          /* "src/python/grpcio/grpc/_cython/_cygrpc/channel.pyx.pxi":153
 *   with channel_state.condition:
 *     on_success(tag)
 *     channel_state.condition.notify_all()             # <<<<<<<<<<<<<<
 *   return event
 * 
 */
          __pyx_t_3 = __Pyx_PyObject_GetAttrStr(__pyx_v_channel_state->condition, __pyx_n_s_notify_all); if (unlikely(!__pyx_t_3)) __PYX_ERR(8, 153, __pyx_L9_error)
          __Pyx_GOTREF(__pyx_t_3);
          __pyx_t_4 = NULL;
          if (CYTHON_UNPACK_METHODS && likely(PyMethod_Check(__pyx_t_3))) {
            __pyx_t_4 = PyMethod_GET_SELF(__pyx_t_3);
            if (likely(__pyx_t_4)) {
              PyObject* function = PyMethod_GET_FUNCTION(__pyx_t_3);
              __Pyx_INCREF(__pyx_t_4);
              __Pyx_INCREF(function);
              __Pyx_DECREF_SET(__pyx_t_3, function);
            }
          }
          if (__pyx_t_4) {
            __pyx_t_1 = __Pyx_PyObject_CallOneArg(__pyx_t_3, __pyx_t_4); if (unlikely(!__pyx_t_1)) __PYX_ERR(8, 153, __pyx_L9_error)
            __Pyx_DECREF(__pyx_t_4); __pyx_t_4 = 0;
          } else {
            __pyx_t_1 = __Pyx_PyObject_CallNoArg(__pyx_t_3); if (unlikely(!__pyx_t_1)) __PYX_ERR(8, 153, __pyx_L9_error)
          }
          __Pyx_GOTREF(__pyx_t_1);
          __Pyx_DECREF(__pyx_t_3); __pyx_t_3 = 0;
          __Pyx_DECREF(__pyx_t_1); __pyx_t_1 = 0;

          /* "src/python/grpcio/grpc/_cython/_cygrpc/channel.pyx.pxi":151
 *     on_success, deadline):
 *   tag, event = _latent_event(c_completion_queue, deadline)
 *   with channel_state.condition:             # <<<<<<<<<<<<<<
 *     on_success(tag)
 *     channel_state.condition.notify_all()
 */
        }
        __Pyx_XDECREF(__pyx_t_7); __pyx_t_7 = 0;
        __Pyx_XDECREF(__pyx_t_8); __pyx_t_8 = 0;
        __Pyx_XDECREF(__pyx_t_9); __pyx_t_9 = 0;
        goto __pyx_L14_try_end;
        __pyx_L9_error:;
        __Pyx_XDECREF(__pyx_t_2); __pyx_t_2 = 0;
        __Pyx_XDECREF(__pyx_t_4); __pyx_t_4 = 0;
        __Pyx_XDECREF(__pyx_t_3); __pyx_t_3 = 0;
        __Pyx_XDECREF(__pyx_t_1); __pyx_t_1 = 0;
        /*except:*/ {
          __Pyx_AddTraceback("grpc._cython.cygrpc._next_call_event", __pyx_clineno, __pyx_lineno, __pyx_filename);
          if (__Pyx_GetException(&__pyx_t_1, &__pyx_t_3, &__pyx_t_4) < 0) __PYX_ERR(8, 151, __pyx_L11_except_error)
          __Pyx_GOTREF(__pyx_t_1);
          __Pyx_GOTREF(__pyx_t_3);
          __Pyx_GOTREF(__pyx_t_4);
          __pyx_t_2 = PyTuple_Pack(3, __pyx_t_1, __pyx_t_3, __pyx_t_4); if (unlikely(!__pyx_t_2)) __PYX_ERR(8, 151, __pyx_L11_except_error)
          __Pyx_GOTREF(__pyx_t_2);
          __pyx_t_10 = __Pyx_PyObject_Call(__pyx_t_6, __pyx_t_2, NULL);
          __Pyx_DECREF(__pyx_t_6); __pyx_t_6 = 0;
          __Pyx_DECREF(__pyx_t_2); __pyx_t_2 = 0;
          if (unlikely(!__pyx_t_10)) __PYX_ERR(8, 151, __pyx_L11_except_error)
          __Pyx_GOTREF(__pyx_t_10);
          __pyx_t_11 = __Pyx_PyObject_IsTrue(__pyx_t_10);
          __Pyx_DECREF(__pyx_t_10); __pyx_t_10 = 0;
          if (__pyx_t_11 < 0) __PYX_ERR(8, 151, __pyx_L11_except_error)
          __pyx_t_12 = ((!(__pyx_t_11 != 0)) != 0);
          if (__pyx_t_12) {
            __Pyx_GIVEREF(__pyx_t_1);
            __Pyx_GIVEREF(__pyx_t_3);
            __Pyx_XGIVEREF(__pyx_t_4);
            __Pyx_ErrRestoreWithState(__pyx_t_1, __pyx_t_3, __pyx_t_4);
            __pyx_t_1 = 0; __pyx_t_3 = 0; __pyx_t_4 = 0; 
            __PYX_ERR(8, 151, __pyx_L11_except_error)
          }
          __Pyx_DECREF(__pyx_t_1); __pyx_t_1 = 0;
          __Pyx_DECREF(__pyx_t_3); __pyx_t_3 = 0;
          __Pyx_DECREF(__pyx_t_4); __pyx_t_4 = 0;
          goto __pyx_L10_exception_handled;
        }
        __pyx_L11_except_error:;
        __Pyx_XGIVEREF(__pyx_t_7);
        __Pyx_XGIVEREF(__pyx_t_8);
        __Pyx_XGIVEREF(__pyx_t_9);
        __Pyx_ExceptionReset(__pyx_t_7, __pyx_t_8, __pyx_t_9);
        goto __pyx_L1_error;
        __pyx_L10_exception_handled:;
        __Pyx_XGIVEREF(__pyx_t_7);
        __Pyx_XGIVEREF(__pyx_t_8);
        __Pyx_XGIVEREF(__pyx_t_9);
        __Pyx_ExceptionReset(__pyx_t_7, __pyx_t_8, __pyx_t_9);
        __pyx_L14_try_end:;
      }
    }
    /*finally:*/ {
      /*normal exit:*/{
        if (__pyx_t_6) {
          __pyx_t_9 = __Pyx_PyObject_Call(__pyx_t_6, __pyx_tuple__25, NULL);
          __Pyx_DECREF(__pyx_t_6); __pyx_t_6 = 0;
          if (unlikely(!__pyx_t_9)) __PYX_ERR(8, 151, __pyx_L1_error)
          __Pyx_GOTREF(__pyx_t_9);
          __Pyx_DECREF(__pyx_t_9); __pyx_t_9 = 0;
        }
        goto __pyx_L8;
      }
      __pyx_L8:;
    }
    goto __pyx_L18;
    __pyx_L5_error:;
    __Pyx_DECREF(__pyx_t_6); __pyx_t_6 = 0;
    goto __pyx_L1_error;
    __pyx_L18:;
  }

  /* "src/python/grpcio/grpc/_cython/_cygrpc/channel.pyx.pxi":154
 *     on_success(tag)
 *     channel_state.condition.notify_all()
 *   return event             # <<<<<<<<<<<<<<
 * 
 * 
 */
  __Pyx_XDECREF(__pyx_r);
  __Pyx_INCREF(__pyx_v_event);
  __pyx_r = __pyx_v_event;
  goto __pyx_L0;

  /* "src/python/grpcio/grpc/_cython/_cygrpc/channel.pyx.pxi":147
 * 
 * 
 * cdef _next_call_event(             # <<<<<<<<<<<<<<
 *     _ChannelState channel_state, grpc_completion_queue *c_completion_queue,
 *     on_success, deadline):
 */

  /* function exit code */
  __pyx_L1_error:;
  __Pyx_XDECREF(__pyx_t_1);
  __Pyx_XDECREF(__pyx_t_2);
  __Pyx_XDECREF(__pyx_t_3);
  __Pyx_XDECREF(__pyx_t_4);
  __Pyx_AddTraceback("grpc._cython.cygrpc._next_call_event", __pyx_clineno, __pyx_lineno, __pyx_filename);
  __pyx_r = 0;
  __pyx_L0:;
  __Pyx_XDECREF(__pyx_v_tag);
  __Pyx_XDECREF(__pyx_v_event);
  __Pyx_XGIVEREF(__pyx_r);
  __Pyx_RefNannyFinishContext();
  return __pyx_r;
}

/* "src/python/grpcio/grpc/_cython/_cygrpc/channel.pyx.pxi":158
 * 
 * # TODO(https://github.com/grpc/grpc/issues/14569): This could be a lot simpler.
 * cdef void _call(             # <<<<<<<<<<<<<<
 *     _ChannelState channel_state, _CallState call_state,
 *     grpc_completion_queue *c_completion_queue, on_success, int flags, method,
 */

static void __pyx_f_4grpc_7_cython_6cygrpc__call(struct __pyx_obj_4grpc_7_cython_6cygrpc__ChannelState *__pyx_v_channel_state, struct __pyx_obj_4grpc_7_cython_6cygrpc__CallState *__pyx_v_call_state, grpc_completion_queue *__pyx_v_c_completion_queue, PyObject *__pyx_v_on_success, int __pyx_v_flags, PyObject *__pyx_v_method, PyObject *__pyx_v_host, PyObject *__pyx_v_deadline, struct __pyx_obj_4grpc_7_cython_6cygrpc_CallCredentials *__pyx_v_credentials, PyObject *__pyx_v_operationses_and_user_tags, PyObject *__pyx_v_metadata, PyObject *__pyx_v_context) {
  grpc_slice __pyx_v_method_slice;
  grpc_slice __pyx_v_host_slice;
  grpc_slice *__pyx_v_host_slice_ptr;
  grpc_call_credentials *__pyx_v_c_call_credentials;
  grpc_call_error __pyx_v_c_call_error;
  PyObject *__pyx_v_started_tags = NULL;
  PyObject *__pyx_v_operations = NULL;
  PyObject *__pyx_v_user_tag = NULL;
  PyObject *__pyx_v_tag = NULL;
  __Pyx_RefNannyDeclarations
  PyObject *__pyx_t_1 = NULL;
  PyObject *__pyx_t_2 = NULL;
  PyObject *__pyx_t_3 = NULL;
  PyObject *__pyx_t_4 = NULL;
  PyObject *__pyx_t_5 = NULL;
  PyObject *__pyx_t_6 = NULL;
  PyObject *__pyx_t_7 = NULL;
  int __pyx_t_8;
  int __pyx_t_9;
  gpr_timespec __pyx_t_10;
  int __pyx_t_11;
  PyObject *__pyx_t_12 = NULL;
  grpc_call_credentials *__pyx_t_13;
  Py_ssize_t __pyx_t_14;
  PyObject *(*__pyx_t_15)(PyObject *);
  PyObject *__pyx_t_16 = NULL;
  PyObject *(*__pyx_t_17)(PyObject *);
  grpc_call_error __pyx_t_18;
  int __pyx_t_19;
  PyObject *__pyx_t_20 = NULL;
  __Pyx_RefNannySetupContext("_call", 0);

  /* "src/python/grpcio/grpc/_cython/_cygrpc/channel.pyx.pxi":198
 *   cdef tuple error_and_wrapper_tag
 *   cdef _BatchOperationTag wrapper_tag
 *   with channel_state.condition:             # <<<<<<<<<<<<<<
 *     if channel_state.open:
 *       method_slice = _slice_from_bytes(method)
 */
  /*with:*/ {
    __pyx_t_1 = __Pyx_PyObject_LookupSpecial(__pyx_v_channel_state->condition, __pyx_n_s_exit); if (unlikely(!__pyx_t_1)) __PYX_ERR(8, 198, __pyx_L1_error)
    __Pyx_GOTREF(__pyx_t_1);
    __pyx_t_3 = __Pyx_PyObject_LookupSpecial(__pyx_v_channel_state->condition, __pyx_n_s_enter); if (unlikely(!__pyx_t_3)) __PYX_ERR(8, 198, __pyx_L3_error)
    __Pyx_GOTREF(__pyx_t_3);
    __pyx_t_4 = NULL;
    if (CYTHON_UNPACK_METHODS && likely(PyMethod_Check(__pyx_t_3))) {
      __pyx_t_4 = PyMethod_GET_SELF(__pyx_t_3);
      if (likely(__pyx_t_4)) {
        PyObject* function = PyMethod_GET_FUNCTION(__pyx_t_3);
        __Pyx_INCREF(__pyx_t_4);
        __Pyx_INCREF(function);
        __Pyx_DECREF_SET(__pyx_t_3, function);
      }
    }
    if (__pyx_t_4) {
      __pyx_t_2 = __Pyx_PyObject_CallOneArg(__pyx_t_3, __pyx_t_4); if (unlikely(!__pyx_t_2)) __PYX_ERR(8, 198, __pyx_L3_error)
      __Pyx_DECREF(__pyx_t_4); __pyx_t_4 = 0;
    } else {
      __pyx_t_2 = __Pyx_PyObject_CallNoArg(__pyx_t_3); if (unlikely(!__pyx_t_2)) __PYX_ERR(8, 198, __pyx_L3_error)
    }
    __Pyx_GOTREF(__pyx_t_2);
    __Pyx_DECREF(__pyx_t_3); __pyx_t_3 = 0;
    __Pyx_DECREF(__pyx_t_2); __pyx_t_2 = 0;
    /*try:*/ {
      {
        __Pyx_PyThreadState_declare
        __Pyx_PyThreadState_assign
        __Pyx_ExceptionSave(&__pyx_t_5, &__pyx_t_6, &__pyx_t_7);
        __Pyx_XGOTREF(__pyx_t_5);
        __Pyx_XGOTREF(__pyx_t_6);
        __Pyx_XGOTREF(__pyx_t_7);
        /*try:*/ {

          /* "src/python/grpcio/grpc/_cython/_cygrpc/channel.pyx.pxi":199
 *   cdef _BatchOperationTag wrapper_tag
 *   with channel_state.condition:
 *     if channel_state.open:             # <<<<<<<<<<<<<<
 *       method_slice = _slice_from_bytes(method)
 *       if host is None:
 */
          __pyx_t_8 = __Pyx_PyObject_IsTrue(__pyx_v_channel_state->open); if (unlikely(__pyx_t_8 < 0)) __PYX_ERR(8, 199, __pyx_L7_error)
          if (likely(__pyx_t_8)) {

            /* "src/python/grpcio/grpc/_cython/_cygrpc/channel.pyx.pxi":200
 *   with channel_state.condition:
 *     if channel_state.open:
 *       method_slice = _slice_from_bytes(method)             # <<<<<<<<<<<<<<
 *       if host is None:
 *         host_slice_ptr = NULL
 */
            if (!(likely(PyBytes_CheckExact(__pyx_v_method))||((__pyx_v_method) == Py_None)||(PyErr_Format(PyExc_TypeError, "Expected %.16s, got %.200s", "bytes", Py_TYPE(__pyx_v_method)->tp_name), 0))) __PYX_ERR(8, 200, __pyx_L7_error)
            __pyx_v_method_slice = __pyx_f_4grpc_7_cython_6cygrpc__slice_from_bytes(((PyObject*)__pyx_v_method));

            /* "src/python/grpcio/grpc/_cython/_cygrpc/channel.pyx.pxi":201
 *     if channel_state.open:
 *       method_slice = _slice_from_bytes(method)
 *       if host is None:             # <<<<<<<<<<<<<<
 *         host_slice_ptr = NULL
 *       else:
 */
            __pyx_t_8 = (__pyx_v_host == Py_None);
            __pyx_t_9 = (__pyx_t_8 != 0);
            if (__pyx_t_9) {

              /* "src/python/grpcio/grpc/_cython/_cygrpc/channel.pyx.pxi":202
 *       method_slice = _slice_from_bytes(method)
 *       if host is None:
 *         host_slice_ptr = NULL             # <<<<<<<<<<<<<<
 *       else:
 *         host_slice = _slice_from_bytes(host)
 */
              __pyx_v_host_slice_ptr = NULL;

              /* "src/python/grpcio/grpc/_cython/_cygrpc/channel.pyx.pxi":201
 *     if channel_state.open:
 *       method_slice = _slice_from_bytes(method)
 *       if host is None:             # <<<<<<<<<<<<<<
 *         host_slice_ptr = NULL
 *       else:
 */
              goto __pyx_L14;
            }

            /* "src/python/grpcio/grpc/_cython/_cygrpc/channel.pyx.pxi":204
 *         host_slice_ptr = NULL
 *       else:
 *         host_slice = _slice_from_bytes(host)             # <<<<<<<<<<<<<<
 *         host_slice_ptr = &host_slice
 *       call_state.c_call = grpc_channel_create_call(
 */
            /*else*/ {
              if (!(likely(PyBytes_CheckExact(__pyx_v_host))||((__pyx_v_host) == Py_None)||(PyErr_Format(PyExc_TypeError, "Expected %.16s, got %.200s", "bytes", Py_TYPE(__pyx_v_host)->tp_name), 0))) __PYX_ERR(8, 204, __pyx_L7_error)
              __pyx_v_host_slice = __pyx_f_4grpc_7_cython_6cygrpc__slice_from_bytes(((PyObject*)__pyx_v_host));

              /* "src/python/grpcio/grpc/_cython/_cygrpc/channel.pyx.pxi":205
 *       else:
 *         host_slice = _slice_from_bytes(host)
 *         host_slice_ptr = &host_slice             # <<<<<<<<<<<<<<
 *       call_state.c_call = grpc_channel_create_call(
 *           channel_state.c_channel, NULL, flags,
 */
              __pyx_v_host_slice_ptr = (&__pyx_v_host_slice);
            }
            __pyx_L14:;

            /* "src/python/grpcio/grpc/_cython/_cygrpc/channel.pyx.pxi":209
 *           channel_state.c_channel, NULL, flags,
 *           c_completion_queue, method_slice, host_slice_ptr,
 *           _timespec_from_time(deadline), NULL)             # <<<<<<<<<<<<<<
 *       grpc_slice_unref(method_slice)
 *       if host_slice_ptr:
 */
            __pyx_t_10 = __pyx_f_4grpc_7_cython_6cygrpc__timespec_from_time(__pyx_v_deadline); if (unlikely(PyErr_Occurred())) __PYX_ERR(8, 209, __pyx_L7_error)

            /* "src/python/grpcio/grpc/_cython/_cygrpc/channel.pyx.pxi":206
 *         host_slice = _slice_from_bytes(host)
 *         host_slice_ptr = &host_slice
 *       call_state.c_call = grpc_channel_create_call(             # <<<<<<<<<<<<<<
 *           channel_state.c_channel, NULL, flags,
 *           c_completion_queue, method_slice, host_slice_ptr,
 */
            __pyx_v_call_state->c_call = grpc_channel_create_call(__pyx_v_channel_state->c_channel, NULL, __pyx_v_flags, __pyx_v_c_completion_queue, __pyx_v_method_slice, __pyx_v_host_slice_ptr, __pyx_t_10, NULL);

            /* "src/python/grpcio/grpc/_cython/_cygrpc/channel.pyx.pxi":210
 *           c_completion_queue, method_slice, host_slice_ptr,
 *           _timespec_from_time(deadline), NULL)
 *       grpc_slice_unref(method_slice)             # <<<<<<<<<<<<<<
 *       if host_slice_ptr:
 *         grpc_slice_unref(host_slice)
 */
            grpc_slice_unref(__pyx_v_method_slice);

            /* "src/python/grpcio/grpc/_cython/_cygrpc/channel.pyx.pxi":211
 *           _timespec_from_time(deadline), NULL)
 *       grpc_slice_unref(method_slice)
 *       if host_slice_ptr:             # <<<<<<<<<<<<<<
 *         grpc_slice_unref(host_slice)
 *       if context is not None:
 */
            __pyx_t_9 = (__pyx_v_host_slice_ptr != 0);
            if (__pyx_t_9) {

              /* "src/python/grpcio/grpc/_cython/_cygrpc/channel.pyx.pxi":212
 *       grpc_slice_unref(method_slice)
 *       if host_slice_ptr:
 *         grpc_slice_unref(host_slice)             # <<<<<<<<<<<<<<
 *       if context is not None:
 *         set_census_context_on_call(call_state, context)
 */
              grpc_slice_unref(__pyx_v_host_slice);

              /* "src/python/grpcio/grpc/_cython/_cygrpc/channel.pyx.pxi":211
 *           _timespec_from_time(deadline), NULL)
 *       grpc_slice_unref(method_slice)
 *       if host_slice_ptr:             # <<<<<<<<<<<<<<
 *         grpc_slice_unref(host_slice)
 *       if context is not None:
 */
            }

            /* "src/python/grpcio/grpc/_cython/_cygrpc/channel.pyx.pxi":213
 *       if host_slice_ptr:
 *         grpc_slice_unref(host_slice)
 *       if context is not None:             # <<<<<<<<<<<<<<
 *         set_census_context_on_call(call_state, context)
 *       if credentials is not None:
 */
            __pyx_t_9 = (__pyx_v_context != Py_None);
            __pyx_t_8 = (__pyx_t_9 != 0);
            if (__pyx_t_8) {

              /* "src/python/grpcio/grpc/_cython/_cygrpc/channel.pyx.pxi":214
 *         grpc_slice_unref(host_slice)
 *       if context is not None:
 *         set_census_context_on_call(call_state, context)             # <<<<<<<<<<<<<<
 *       if credentials is not None:
 *         c_call_credentials = credentials.c()
 */
              __pyx_t_3 = __Pyx_GetModuleGlobalName(__pyx_n_s_set_census_context_on_call); if (unlikely(!__pyx_t_3)) __PYX_ERR(8, 214, __pyx_L7_error)
              __Pyx_GOTREF(__pyx_t_3);
              __pyx_t_4 = NULL;
              __pyx_t_11 = 0;
              if (CYTHON_UNPACK_METHODS && unlikely(PyMethod_Check(__pyx_t_3))) {
                __pyx_t_4 = PyMethod_GET_SELF(__pyx_t_3);
                if (likely(__pyx_t_4)) {
                  PyObject* function = PyMethod_GET_FUNCTION(__pyx_t_3);
                  __Pyx_INCREF(__pyx_t_4);
                  __Pyx_INCREF(function);
                  __Pyx_DECREF_SET(__pyx_t_3, function);
                  __pyx_t_11 = 1;
                }
              }
              #if CYTHON_FAST_PYCALL
              if (PyFunction_Check(__pyx_t_3)) {
                PyObject *__pyx_temp[3] = {__pyx_t_4, ((PyObject *)__pyx_v_call_state), __pyx_v_context};
                __pyx_t_2 = __Pyx_PyFunction_FastCall(__pyx_t_3, __pyx_temp+1-__pyx_t_11, 2+__pyx_t_11); if (unlikely(!__pyx_t_2)) __PYX_ERR(8, 214, __pyx_L7_error)
                __Pyx_XDECREF(__pyx_t_4); __pyx_t_4 = 0;
                __Pyx_GOTREF(__pyx_t_2);
              } else
              #endif
              #if CYTHON_FAST_PYCCALL
              if (__Pyx_PyFastCFunction_Check(__pyx_t_3)) {
                PyObject *__pyx_temp[3] = {__pyx_t_4, ((PyObject *)__pyx_v_call_state), __pyx_v_context};
                __pyx_t_2 = __Pyx_PyCFunction_FastCall(__pyx_t_3, __pyx_temp+1-__pyx_t_11, 2+__pyx_t_11); if (unlikely(!__pyx_t_2)) __PYX_ERR(8, 214, __pyx_L7_error)
                __Pyx_XDECREF(__pyx_t_4); __pyx_t_4 = 0;
                __Pyx_GOTREF(__pyx_t_2);
              } else
              #endif
              {
                __pyx_t_12 = PyTuple_New(2+__pyx_t_11); if (unlikely(!__pyx_t_12)) __PYX_ERR(8, 214, __pyx_L7_error)
                __Pyx_GOTREF(__pyx_t_12);
                if (__pyx_t_4) {
                  __Pyx_GIVEREF(__pyx_t_4); PyTuple_SET_ITEM(__pyx_t_12, 0, __pyx_t_4); __pyx_t_4 = NULL;
                }
                __Pyx_INCREF(((PyObject *)__pyx_v_call_state));
                __Pyx_GIVEREF(((PyObject *)__pyx_v_call_state));
                PyTuple_SET_ITEM(__pyx_t_12, 0+__pyx_t_11, ((PyObject *)__pyx_v_call_state));
                __Pyx_INCREF(__pyx_v_context);
                __Pyx_GIVEREF(__pyx_v_context);
                PyTuple_SET_ITEM(__pyx_t_12, 1+__pyx_t_11, __pyx_v_context);
                __pyx_t_2 = __Pyx_PyObject_Call(__pyx_t_3, __pyx_t_12, NULL); if (unlikely(!__pyx_t_2)) __PYX_ERR(8, 214, __pyx_L7_error)
                __Pyx_GOTREF(__pyx_t_2);
                __Pyx_DECREF(__pyx_t_12); __pyx_t_12 = 0;
              }
              __Pyx_DECREF(__pyx_t_3); __pyx_t_3 = 0;
              __Pyx_DECREF(__pyx_t_2); __pyx_t_2 = 0;

              /* "src/python/grpcio/grpc/_cython/_cygrpc/channel.pyx.pxi":213
 *       if host_slice_ptr:
 *         grpc_slice_unref(host_slice)
 *       if context is not None:             # <<<<<<<<<<<<<<
 *         set_census_context_on_call(call_state, context)
 *       if credentials is not None:
 */
            }

            /* "src/python/grpcio/grpc/_cython/_cygrpc/channel.pyx.pxi":215
 *       if context is not None:
 *         set_census_context_on_call(call_state, context)
 *       if credentials is not None:             # <<<<<<<<<<<<<<
 *         c_call_credentials = credentials.c()
 *         c_call_error = grpc_call_set_credentials(
 */
            __pyx_t_8 = (((PyObject *)__pyx_v_credentials) != Py_None);
            __pyx_t_9 = (__pyx_t_8 != 0);
            if (__pyx_t_9) {

              /* "src/python/grpcio/grpc/_cython/_cygrpc/channel.pyx.pxi":216
 *         set_census_context_on_call(call_state, context)
 *       if credentials is not None:
 *         c_call_credentials = credentials.c()             # <<<<<<<<<<<<<<
 *         c_call_error = grpc_call_set_credentials(
 *             call_state.c_call, c_call_credentials)
 */
              __pyx_t_13 = ((struct __pyx_vtabstruct_4grpc_7_cython_6cygrpc_CallCredentials *)__pyx_v_credentials->__pyx_vtab)->c(__pyx_v_credentials); if (unlikely(PyErr_Occurred())) __PYX_ERR(8, 216, __pyx_L7_error)
              __pyx_v_c_call_credentials = __pyx_t_13;

              /* "src/python/grpcio/grpc/_cython/_cygrpc/channel.pyx.pxi":217
 *       if credentials is not None:
 *         c_call_credentials = credentials.c()
 *         c_call_error = grpc_call_set_credentials(             # <<<<<<<<<<<<<<
 *             call_state.c_call, c_call_credentials)
 *         grpc_call_credentials_release(c_call_credentials)
 */
              __pyx_v_c_call_error = grpc_call_set_credentials(__pyx_v_call_state->c_call, __pyx_v_c_call_credentials);

              /* "src/python/grpcio/grpc/_cython/_cygrpc/channel.pyx.pxi":219
 *         c_call_error = grpc_call_set_credentials(
 *             call_state.c_call, c_call_credentials)
 *         grpc_call_credentials_release(c_call_credentials)             # <<<<<<<<<<<<<<
 *         if c_call_error != GRPC_CALL_OK:
 *           grpc_call_unref(call_state.c_call)
 */
              grpc_call_credentials_release(__pyx_v_c_call_credentials);

              /* "src/python/grpcio/grpc/_cython/_cygrpc/channel.pyx.pxi":220
 *             call_state.c_call, c_call_credentials)
 *         grpc_call_credentials_release(c_call_credentials)
 *         if c_call_error != GRPC_CALL_OK:             # <<<<<<<<<<<<<<
 *           grpc_call_unref(call_state.c_call)
 *           call_state.c_call = NULL
 */
              __pyx_t_9 = ((__pyx_v_c_call_error != GRPC_CALL_OK) != 0);
              if (__pyx_t_9) {

                /* "src/python/grpcio/grpc/_cython/_cygrpc/channel.pyx.pxi":221
 *         grpc_call_credentials_release(c_call_credentials)
 *         if c_call_error != GRPC_CALL_OK:
 *           grpc_call_unref(call_state.c_call)             # <<<<<<<<<<<<<<
 *           call_state.c_call = NULL
 *           _raise_call_error_no_metadata(c_call_error)
 */
                grpc_call_unref(__pyx_v_call_state->c_call);

                /* "src/python/grpcio/grpc/_cython/_cygrpc/channel.pyx.pxi":222
 *         if c_call_error != GRPC_CALL_OK:
 *           grpc_call_unref(call_state.c_call)
 *           call_state.c_call = NULL             # <<<<<<<<<<<<<<
 *           _raise_call_error_no_metadata(c_call_error)
 *       started_tags = set()
 */
                __pyx_v_call_state->c_call = NULL;

                /* "src/python/grpcio/grpc/_cython/_cygrpc/channel.pyx.pxi":223
 *           grpc_call_unref(call_state.c_call)
 *           call_state.c_call = NULL
 *           _raise_call_error_no_metadata(c_call_error)             # <<<<<<<<<<<<<<
 *       started_tags = set()
 *       for operations, user_tag in operationses_and_user_tags:
 */
                __pyx_t_2 = __Pyx_PyInt_From_grpc_call_error(__pyx_v_c_call_error); if (unlikely(!__pyx_t_2)) __PYX_ERR(8, 223, __pyx_L7_error)
                __Pyx_GOTREF(__pyx_t_2);
                __pyx_f_4grpc_7_cython_6cygrpc__raise_call_error_no_metadata(__pyx_t_2); if (unlikely(PyErr_Occurred())) __PYX_ERR(8, 223, __pyx_L7_error)
                __Pyx_DECREF(__pyx_t_2); __pyx_t_2 = 0;

                /* "src/python/grpcio/grpc/_cython/_cygrpc/channel.pyx.pxi":220
 *             call_state.c_call, c_call_credentials)
 *         grpc_call_credentials_release(c_call_credentials)
 *         if c_call_error != GRPC_CALL_OK:             # <<<<<<<<<<<<<<
 *           grpc_call_unref(call_state.c_call)
 *           call_state.c_call = NULL
 */
              }

              /* "src/python/grpcio/grpc/_cython/_cygrpc/channel.pyx.pxi":215
 *       if context is not None:
 *         set_census_context_on_call(call_state, context)
 *       if credentials is not None:             # <<<<<<<<<<<<<<
 *         c_call_credentials = credentials.c()
 *         c_call_error = grpc_call_set_credentials(
 */
            }

            /* "src/python/grpcio/grpc/_cython/_cygrpc/channel.pyx.pxi":224
 *           call_state.c_call = NULL
 *           _raise_call_error_no_metadata(c_call_error)
 *       started_tags = set()             # <<<<<<<<<<<<<<
 *       for operations, user_tag in operationses_and_user_tags:
 *         c_call_error, tag = _operate(call_state.c_call, operations, user_tag)
 */
            __pyx_t_2 = PySet_New(0); if (unlikely(!__pyx_t_2)) __PYX_ERR(8, 224, __pyx_L7_error)
            __Pyx_GOTREF(__pyx_t_2);
            __pyx_v_started_tags = ((PyObject*)__pyx_t_2);
            __pyx_t_2 = 0;

            /* "src/python/grpcio/grpc/_cython/_cygrpc/channel.pyx.pxi":225
 *           _raise_call_error_no_metadata(c_call_error)
 *       started_tags = set()
 *       for operations, user_tag in operationses_and_user_tags:             # <<<<<<<<<<<<<<
 *         c_call_error, tag = _operate(call_state.c_call, operations, user_tag)
 *         if c_call_error == GRPC_CALL_OK:
 */
            if (likely(PyList_CheckExact(__pyx_v_operationses_and_user_tags)) || PyTuple_CheckExact(__pyx_v_operationses_and_user_tags)) {
              __pyx_t_2 = __pyx_v_operationses_and_user_tags; __Pyx_INCREF(__pyx_t_2); __pyx_t_14 = 0;
              __pyx_t_15 = NULL;
            } else {
              __pyx_t_14 = -1; __pyx_t_2 = PyObject_GetIter(__pyx_v_operationses_and_user_tags); if (unlikely(!__pyx_t_2)) __PYX_ERR(8, 225, __pyx_L7_error)
              __Pyx_GOTREF(__pyx_t_2);
              __pyx_t_15 = Py_TYPE(__pyx_t_2)->tp_iternext; if (unlikely(!__pyx_t_15)) __PYX_ERR(8, 225, __pyx_L7_error)
            }
            for (;;) {
              if (likely(!__pyx_t_15)) {
                if (likely(PyList_CheckExact(__pyx_t_2))) {
                  if (__pyx_t_14 >= PyList_GET_SIZE(__pyx_t_2)) break;
                  #if CYTHON_ASSUME_SAFE_MACROS && !CYTHON_AVOID_BORROWED_REFS
                  __pyx_t_3 = PyList_GET_ITEM(__pyx_t_2, __pyx_t_14); __Pyx_INCREF(__pyx_t_3); __pyx_t_14++; if (unlikely(0 < 0)) __PYX_ERR(8, 225, __pyx_L7_error)
                  #else
                  __pyx_t_3 = PySequence_ITEM(__pyx_t_2, __pyx_t_14); __pyx_t_14++; if (unlikely(!__pyx_t_3)) __PYX_ERR(8, 225, __pyx_L7_error)
                  __Pyx_GOTREF(__pyx_t_3);
                  #endif
                } else {
                  if (__pyx_t_14 >= PyTuple_GET_SIZE(__pyx_t_2)) break;
                  #if CYTHON_ASSUME_SAFE_MACROS && !CYTHON_AVOID_BORROWED_REFS
                  __pyx_t_3 = PyTuple_GET_ITEM(__pyx_t_2, __pyx_t_14); __Pyx_INCREF(__pyx_t_3); __pyx_t_14++; if (unlikely(0 < 0)) __PYX_ERR(8, 225, __pyx_L7_error)
                  #else
                  __pyx_t_3 = PySequence_ITEM(__pyx_t_2, __pyx_t_14); __pyx_t_14++; if (unlikely(!__pyx_t_3)) __PYX_ERR(8, 225, __pyx_L7_error)
                  __Pyx_GOTREF(__pyx_t_3);
                  #endif
                }
              } else {
                __pyx_t_3 = __pyx_t_15(__pyx_t_2);
                if (unlikely(!__pyx_t_3)) {
                  PyObject* exc_type = PyErr_Occurred();
                  if (exc_type) {
                    if (likely(__Pyx_PyErr_GivenExceptionMatches(exc_type, PyExc_StopIteration))) PyErr_Clear();
                    else __PYX_ERR(8, 225, __pyx_L7_error)
                  }
                  break;
                }
                __Pyx_GOTREF(__pyx_t_3);
              }
              if ((likely(PyTuple_CheckExact(__pyx_t_3))) || (PyList_CheckExact(__pyx_t_3))) {
                PyObject* sequence = __pyx_t_3;
                Py_ssize_t size = __Pyx_PySequence_SIZE(sequence);
                if (unlikely(size != 2)) {
                  if (size > 2) __Pyx_RaiseTooManyValuesError(2);
                  else if (size >= 0) __Pyx_RaiseNeedMoreValuesError(size);
                  __PYX_ERR(8, 225, __pyx_L7_error)
                }
                #if CYTHON_ASSUME_SAFE_MACROS && !CYTHON_AVOID_BORROWED_REFS
                if (likely(PyTuple_CheckExact(sequence))) {
                  __pyx_t_12 = PyTuple_GET_ITEM(sequence, 0); 
                  __pyx_t_4 = PyTuple_GET_ITEM(sequence, 1); 
                } else {
                  __pyx_t_12 = PyList_GET_ITEM(sequence, 0); 
                  __pyx_t_4 = PyList_GET_ITEM(sequence, 1); 
                }
                __Pyx_INCREF(__pyx_t_12);
                __Pyx_INCREF(__pyx_t_4);
                #else
                __pyx_t_12 = PySequence_ITEM(sequence, 0); if (unlikely(!__pyx_t_12)) __PYX_ERR(8, 225, __pyx_L7_error)
                __Pyx_GOTREF(__pyx_t_12);
                __pyx_t_4 = PySequence_ITEM(sequence, 1); if (unlikely(!__pyx_t_4)) __PYX_ERR(8, 225, __pyx_L7_error)
                __Pyx_GOTREF(__pyx_t_4);
                #endif
                __Pyx_DECREF(__pyx_t_3); __pyx_t_3 = 0;
              } else {
                Py_ssize_t index = -1;
                __pyx_t_16 = PyObject_GetIter(__pyx_t_3); if (unlikely(!__pyx_t_16)) __PYX_ERR(8, 225, __pyx_L7_error)
                __Pyx_GOTREF(__pyx_t_16);
                __Pyx_DECREF(__pyx_t_3); __pyx_t_3 = 0;
                __pyx_t_17 = Py_TYPE(__pyx_t_16)->tp_iternext;
                index = 0; __pyx_t_12 = __pyx_t_17(__pyx_t_16); if (unlikely(!__pyx_t_12)) goto __pyx_L21_unpacking_failed;
                __Pyx_GOTREF(__pyx_t_12);
                index = 1; __pyx_t_4 = __pyx_t_17(__pyx_t_16); if (unlikely(!__pyx_t_4)) goto __pyx_L21_unpacking_failed;
                __Pyx_GOTREF(__pyx_t_4);
                if (__Pyx_IternextUnpackEndCheck(__pyx_t_17(__pyx_t_16), 2) < 0) __PYX_ERR(8, 225, __pyx_L7_error)
                __pyx_t_17 = NULL;
                __Pyx_DECREF(__pyx_t_16); __pyx_t_16 = 0;
                goto __pyx_L22_unpacking_done;
                __pyx_L21_unpacking_failed:;
                __Pyx_DECREF(__pyx_t_16); __pyx_t_16 = 0;
                __pyx_t_17 = NULL;
                if (__Pyx_IterFinish() == 0) __Pyx_RaiseNeedMoreValuesError(index);
                __PYX_ERR(8, 225, __pyx_L7_error)
                __pyx_L22_unpacking_done:;
              }
              __Pyx_XDECREF_SET(__pyx_v_operations, __pyx_t_12);
              __pyx_t_12 = 0;
              __Pyx_XDECREF_SET(__pyx_v_user_tag, __pyx_t_4);
              __pyx_t_4 = 0;

              /* "src/python/grpcio/grpc/_cython/_cygrpc/channel.pyx.pxi":226
 *       started_tags = set()
 *       for operations, user_tag in operationses_and_user_tags:
 *         c_call_error, tag = _operate(call_state.c_call, operations, user_tag)             # <<<<<<<<<<<<<<
 *         if c_call_error == GRPC_CALL_OK:
 *           started_tags.add(tag)
 */
              __pyx_t_3 = __pyx_f_4grpc_7_cython_6cygrpc__operate(__pyx_v_call_state->c_call, __pyx_v_operations, __pyx_v_user_tag); if (unlikely(!__pyx_t_3)) __PYX_ERR(8, 226, __pyx_L7_error)
              __Pyx_GOTREF(__pyx_t_3);
              if (likely(__pyx_t_3 != Py_None)) {
                PyObject* sequence = __pyx_t_3;
                Py_ssize_t size = __Pyx_PySequence_SIZE(sequence);
                if (unlikely(size != 2)) {
                  if (size > 2) __Pyx_RaiseTooManyValuesError(2);
                  else if (size >= 0) __Pyx_RaiseNeedMoreValuesError(size);
                  __PYX_ERR(8, 226, __pyx_L7_error)
                }
                #if CYTHON_ASSUME_SAFE_MACROS && !CYTHON_AVOID_BORROWED_REFS
                __pyx_t_4 = PyTuple_GET_ITEM(sequence, 0); 
                __pyx_t_12 = PyTuple_GET_ITEM(sequence, 1); 
                __Pyx_INCREF(__pyx_t_4);
                __Pyx_INCREF(__pyx_t_12);
                #else
                __pyx_t_4 = PySequence_ITEM(sequence, 0); if (unlikely(!__pyx_t_4)) __PYX_ERR(8, 226, __pyx_L7_error)
                __Pyx_GOTREF(__pyx_t_4);
                __pyx_t_12 = PySequence_ITEM(sequence, 1); if (unlikely(!__pyx_t_12)) __PYX_ERR(8, 226, __pyx_L7_error)
                __Pyx_GOTREF(__pyx_t_12);
                #endif
                __Pyx_DECREF(__pyx_t_3); __pyx_t_3 = 0;
              } else {
                __Pyx_RaiseNoneNotIterableError(); __PYX_ERR(8, 226, __pyx_L7_error)
              }
              __pyx_t_18 = ((grpc_call_error)__Pyx_PyInt_As_grpc_call_error(__pyx_t_4)); if (unlikely(PyErr_Occurred())) __PYX_ERR(8, 226, __pyx_L7_error)
              __Pyx_DECREF(__pyx_t_4); __pyx_t_4 = 0;
              __pyx_v_c_call_error = __pyx_t_18;
              __Pyx_XDECREF_SET(__pyx_v_tag, __pyx_t_12);
              __pyx_t_12 = 0;

              /* "src/python/grpcio/grpc/_cython/_cygrpc/channel.pyx.pxi":227
 *       for operations, user_tag in operationses_and_user_tags:
 *         c_call_error, tag = _operate(call_state.c_call, operations, user_tag)
 *         if c_call_error == GRPC_CALL_OK:             # <<<<<<<<<<<<<<
 *           started_tags.add(tag)
 *         else:
 */
              __pyx_t_9 = ((__pyx_v_c_call_error == GRPC_CALL_OK) != 0);
              if (__pyx_t_9) {

                /* "src/python/grpcio/grpc/_cython/_cygrpc/channel.pyx.pxi":228
 *         c_call_error, tag = _operate(call_state.c_call, operations, user_tag)
 *         if c_call_error == GRPC_CALL_OK:
 *           started_tags.add(tag)             # <<<<<<<<<<<<<<
 *         else:
 *           grpc_call_cancel(call_state.c_call, NULL)
 */
                __pyx_t_19 = PySet_Add(__pyx_v_started_tags, __pyx_v_tag); if (unlikely(__pyx_t_19 == ((int)-1))) __PYX_ERR(8, 228, __pyx_L7_error)

                /* "src/python/grpcio/grpc/_cython/_cygrpc/channel.pyx.pxi":227
 *       for operations, user_tag in operationses_and_user_tags:
 *         c_call_error, tag = _operate(call_state.c_call, operations, user_tag)
 *         if c_call_error == GRPC_CALL_OK:             # <<<<<<<<<<<<<<
 *           started_tags.add(tag)
 *         else:
 */
                goto __pyx_L23;
              }

              /* "src/python/grpcio/grpc/_cython/_cygrpc/channel.pyx.pxi":230
 *           started_tags.add(tag)
 *         else:
 *           grpc_call_cancel(call_state.c_call, NULL)             # <<<<<<<<<<<<<<
 *           grpc_call_unref(call_state.c_call)
 *           call_state.c_call = NULL
 */
              /*else*/ {
                (void)(grpc_call_cancel(__pyx_v_call_state->c_call, NULL));

                /* "src/python/grpcio/grpc/_cython/_cygrpc/channel.pyx.pxi":231
 *         else:
 *           grpc_call_cancel(call_state.c_call, NULL)
 *           grpc_call_unref(call_state.c_call)             # <<<<<<<<<<<<<<
 *           call_state.c_call = NULL
 *           _raise_call_error(c_call_error, metadata)
 */
                grpc_call_unref(__pyx_v_call_state->c_call);

                /* "src/python/grpcio/grpc/_cython/_cygrpc/channel.pyx.pxi":232
 *           grpc_call_cancel(call_state.c_call, NULL)
 *           grpc_call_unref(call_state.c_call)
 *           call_state.c_call = NULL             # <<<<<<<<<<<<<<
 *           _raise_call_error(c_call_error, metadata)
 *       else:
 */
                __pyx_v_call_state->c_call = NULL;

                /* "src/python/grpcio/grpc/_cython/_cygrpc/channel.pyx.pxi":233
 *           grpc_call_unref(call_state.c_call)
 *           call_state.c_call = NULL
 *           _raise_call_error(c_call_error, metadata)             # <<<<<<<<<<<<<<
 *       else:
 *         call_state.due.update(started_tags)
 */
                __pyx_t_3 = __Pyx_PyInt_From_grpc_call_error(__pyx_v_c_call_error); if (unlikely(!__pyx_t_3)) __PYX_ERR(8, 233, __pyx_L7_error)
                __Pyx_GOTREF(__pyx_t_3);
                __pyx_f_4grpc_7_cython_6cygrpc__raise_call_error(__pyx_t_3, __pyx_v_metadata); if (unlikely(PyErr_Occurred())) __PYX_ERR(8, 233, __pyx_L7_error)
                __Pyx_DECREF(__pyx_t_3); __pyx_t_3 = 0;
              }
              __pyx_L23:;

              /* "src/python/grpcio/grpc/_cython/_cygrpc/channel.pyx.pxi":225
 *           _raise_call_error_no_metadata(c_call_error)
 *       started_tags = set()
 *       for operations, user_tag in operationses_and_user_tags:             # <<<<<<<<<<<<<<
 *         c_call_error, tag = _operate(call_state.c_call, operations, user_tag)
 *         if c_call_error == GRPC_CALL_OK:
 */
            }
            /*else*/ {

              /* "src/python/grpcio/grpc/_cython/_cygrpc/channel.pyx.pxi":235
 *           _raise_call_error(c_call_error, metadata)
 *       else:
 *         call_state.due.update(started_tags)             # <<<<<<<<<<<<<<
 *         on_success(started_tags)
 *     else:
 */
              __pyx_t_12 = __Pyx_PyObject_GetAttrStr(__pyx_v_call_state->due, __pyx_n_s_update); if (unlikely(!__pyx_t_12)) __PYX_ERR(8, 235, __pyx_L7_error)
              __Pyx_GOTREF(__pyx_t_12);
              __pyx_t_4 = NULL;
              if (CYTHON_UNPACK_METHODS && likely(PyMethod_Check(__pyx_t_12))) {
                __pyx_t_4 = PyMethod_GET_SELF(__pyx_t_12);
                if (likely(__pyx_t_4)) {
                  PyObject* function = PyMethod_GET_FUNCTION(__pyx_t_12);
                  __Pyx_INCREF(__pyx_t_4);
                  __Pyx_INCREF(function);
                  __Pyx_DECREF_SET(__pyx_t_12, function);
                }
              }
              if (!__pyx_t_4) {
                __pyx_t_3 = __Pyx_PyObject_CallOneArg(__pyx_t_12, __pyx_v_started_tags); if (unlikely(!__pyx_t_3)) __PYX_ERR(8, 235, __pyx_L7_error)
                __Pyx_GOTREF(__pyx_t_3);
              } else {
                #if CYTHON_FAST_PYCALL
                if (PyFunction_Check(__pyx_t_12)) {
                  PyObject *__pyx_temp[2] = {__pyx_t_4, __pyx_v_started_tags};
                  __pyx_t_3 = __Pyx_PyFunction_FastCall(__pyx_t_12, __pyx_temp+1-1, 1+1); if (unlikely(!__pyx_t_3)) __PYX_ERR(8, 235, __pyx_L7_error)
                  __Pyx_XDECREF(__pyx_t_4); __pyx_t_4 = 0;
                  __Pyx_GOTREF(__pyx_t_3);
                } else
                #endif
                #if CYTHON_FAST_PYCCALL
                if (__Pyx_PyFastCFunction_Check(__pyx_t_12)) {
                  PyObject *__pyx_temp[2] = {__pyx_t_4, __pyx_v_started_tags};
                  __pyx_t_3 = __Pyx_PyCFunction_FastCall(__pyx_t_12, __pyx_temp+1-1, 1+1); if (unlikely(!__pyx_t_3)) __PYX_ERR(8, 235, __pyx_L7_error)
                  __Pyx_XDECREF(__pyx_t_4); __pyx_t_4 = 0;
                  __Pyx_GOTREF(__pyx_t_3);
                } else
                #endif
                {
                  __pyx_t_16 = PyTuple_New(1+1); if (unlikely(!__pyx_t_16)) __PYX_ERR(8, 235, __pyx_L7_error)
                  __Pyx_GOTREF(__pyx_t_16);
                  __Pyx_GIVEREF(__pyx_t_4); PyTuple_SET_ITEM(__pyx_t_16, 0, __pyx_t_4); __pyx_t_4 = NULL;
                  __Pyx_INCREF(__pyx_v_started_tags);
                  __Pyx_GIVEREF(__pyx_v_started_tags);
                  PyTuple_SET_ITEM(__pyx_t_16, 0+1, __pyx_v_started_tags);
                  __pyx_t_3 = __Pyx_PyObject_Call(__pyx_t_12, __pyx_t_16, NULL); if (unlikely(!__pyx_t_3)) __PYX_ERR(8, 235, __pyx_L7_error)
                  __Pyx_GOTREF(__pyx_t_3);
                  __Pyx_DECREF(__pyx_t_16); __pyx_t_16 = 0;
                }
              }
              __Pyx_DECREF(__pyx_t_12); __pyx_t_12 = 0;
              __Pyx_DECREF(__pyx_t_3); __pyx_t_3 = 0;

              /* "src/python/grpcio/grpc/_cython/_cygrpc/channel.pyx.pxi":236
 *       else:
 *         call_state.due.update(started_tags)
 *         on_success(started_tags)             # <<<<<<<<<<<<<<
 *     else:
 *       raise ValueError('Cannot invoke RPC: %s' % channel_state.closed_reason)
 */
              __Pyx_INCREF(__pyx_v_on_success);
              __pyx_t_12 = __pyx_v_on_success; __pyx_t_16 = NULL;
              if (CYTHON_UNPACK_METHODS && unlikely(PyMethod_Check(__pyx_t_12))) {
                __pyx_t_16 = PyMethod_GET_SELF(__pyx_t_12);
                if (likely(__pyx_t_16)) {
                  PyObject* function = PyMethod_GET_FUNCTION(__pyx_t_12);
                  __Pyx_INCREF(__pyx_t_16);
                  __Pyx_INCREF(function);
                  __Pyx_DECREF_SET(__pyx_t_12, function);
                }
              }
              if (!__pyx_t_16) {
                __pyx_t_3 = __Pyx_PyObject_CallOneArg(__pyx_t_12, __pyx_v_started_tags); if (unlikely(!__pyx_t_3)) __PYX_ERR(8, 236, __pyx_L7_error)
                __Pyx_GOTREF(__pyx_t_3);
              } else {
                #if CYTHON_FAST_PYCALL
                if (PyFunction_Check(__pyx_t_12)) {
                  PyObject *__pyx_temp[2] = {__pyx_t_16, __pyx_v_started_tags};
                  __pyx_t_3 = __Pyx_PyFunction_FastCall(__pyx_t_12, __pyx_temp+1-1, 1+1); if (unlikely(!__pyx_t_3)) __PYX_ERR(8, 236, __pyx_L7_error)
                  __Pyx_XDECREF(__pyx_t_16); __pyx_t_16 = 0;
                  __Pyx_GOTREF(__pyx_t_3);
                } else
                #endif
                #if CYTHON_FAST_PYCCALL
                if (__Pyx_PyFastCFunction_Check(__pyx_t_12)) {
                  PyObject *__pyx_temp[2] = {__pyx_t_16, __pyx_v_started_tags};
                  __pyx_t_3 = __Pyx_PyCFunction_FastCall(__pyx_t_12, __pyx_temp+1-1, 1+1); if (unlikely(!__pyx_t_3)) __PYX_ERR(8, 236, __pyx_L7_error)
                  __Pyx_XDECREF(__pyx_t_16); __pyx_t_16 = 0;
                  __Pyx_GOTREF(__pyx_t_3);
                } else
                #endif
                {
                  __pyx_t_4 = PyTuple_New(1+1); if (unlikely(!__pyx_t_4)) __PYX_ERR(8, 236, __pyx_L7_error)
                  __Pyx_GOTREF(__pyx_t_4);
                  __Pyx_GIVEREF(__pyx_t_16); PyTuple_SET_ITEM(__pyx_t_4, 0, __pyx_t_16); __pyx_t_16 = NULL;
                  __Pyx_INCREF(__pyx_v_started_tags);
                  __Pyx_GIVEREF(__pyx_v_started_tags);
                  PyTuple_SET_ITEM(__pyx_t_4, 0+1, __pyx_v_started_tags);
                  __pyx_t_3 = __Pyx_PyObject_Call(__pyx_t_12, __pyx_t_4, NULL); if (unlikely(!__pyx_t_3)) __PYX_ERR(8, 236, __pyx_L7_error)
                  __Pyx_GOTREF(__pyx_t_3);
                  __Pyx_DECREF(__pyx_t_4); __pyx_t_4 = 0;
                }
              }
              __Pyx_DECREF(__pyx_t_12); __pyx_t_12 = 0;
              __Pyx_DECREF(__pyx_t_3); __pyx_t_3 = 0;
            }

            /* "src/python/grpcio/grpc/_cython/_cygrpc/channel.pyx.pxi":225
 *           _raise_call_error_no_metadata(c_call_error)
 *       started_tags = set()
 *       for operations, user_tag in operationses_and_user_tags:             # <<<<<<<<<<<<<<
 *         c_call_error, tag = _operate(call_state.c_call, operations, user_tag)
 *         if c_call_error == GRPC_CALL_OK:
 */
            __Pyx_DECREF(__pyx_t_2); __pyx_t_2 = 0;

            /* "src/python/grpcio/grpc/_cython/_cygrpc/channel.pyx.pxi":199
 *   cdef _BatchOperationTag wrapper_tag
 *   with channel_state.condition:
 *     if channel_state.open:             # <<<<<<<<<<<<<<
 *       method_slice = _slice_from_bytes(method)
 *       if host is None:
 */
            goto __pyx_L13;
          }

          /* "src/python/grpcio/grpc/_cython/_cygrpc/channel.pyx.pxi":238
 *         on_success(started_tags)
 *     else:
 *       raise ValueError('Cannot invoke RPC: %s' % channel_state.closed_reason)             # <<<<<<<<<<<<<<
 * cdef void _process_integrated_call_tag(
 *     _ChannelState state, _BatchOperationTag tag) except *:
 */
          /*else*/ {
            __pyx_t_2 = __Pyx_PyString_Format(__pyx_kp_s_Cannot_invoke_RPC_s, __pyx_v_channel_state->closed_reason); if (unlikely(!__pyx_t_2)) __PYX_ERR(8, 238, __pyx_L7_error)
            __Pyx_GOTREF(__pyx_t_2);
            __pyx_t_3 = __Pyx_PyObject_CallOneArg(__pyx_builtin_ValueError, __pyx_t_2); if (unlikely(!__pyx_t_3)) __PYX_ERR(8, 238, __pyx_L7_error)
            __Pyx_GOTREF(__pyx_t_3);
            __Pyx_DECREF(__pyx_t_2); __pyx_t_2 = 0;
            __Pyx_Raise(__pyx_t_3, 0, 0, 0);
            __Pyx_DECREF(__pyx_t_3); __pyx_t_3 = 0;
            __PYX_ERR(8, 238, __pyx_L7_error)
          }
          __pyx_L13:;

          /* "src/python/grpcio/grpc/_cython/_cygrpc/channel.pyx.pxi":198
 *   cdef tuple error_and_wrapper_tag
 *   cdef _BatchOperationTag wrapper_tag
 *   with channel_state.condition:             # <<<<<<<<<<<<<<
 *     if channel_state.open:
 *       method_slice = _slice_from_bytes(method)
 */
        }
        __Pyx_XDECREF(__pyx_t_5); __pyx_t_5 = 0;
        __Pyx_XDECREF(__pyx_t_6); __pyx_t_6 = 0;
        __Pyx_XDECREF(__pyx_t_7); __pyx_t_7 = 0;
        goto __pyx_L12_try_end;
        __pyx_L7_error:;
        __Pyx_XDECREF(__pyx_t_16); __pyx_t_16 = 0;
        __Pyx_XDECREF(__pyx_t_4); __pyx_t_4 = 0;
        __Pyx_XDECREF(__pyx_t_12); __pyx_t_12 = 0;
        __Pyx_XDECREF(__pyx_t_2); __pyx_t_2 = 0;
        __Pyx_XDECREF(__pyx_t_3); __pyx_t_3 = 0;
        /*except:*/ {
          __Pyx_AddTraceback("grpc._cython.cygrpc._call", __pyx_clineno, __pyx_lineno, __pyx_filename);
          if (__Pyx_GetException(&__pyx_t_3, &__pyx_t_2, &__pyx_t_12) < 0) __PYX_ERR(8, 198, __pyx_L9_except_error)
          __Pyx_GOTREF(__pyx_t_3);
          __Pyx_GOTREF(__pyx_t_2);
          __Pyx_GOTREF(__pyx_t_12);
          __pyx_t_4 = PyTuple_Pack(3, __pyx_t_3, __pyx_t_2, __pyx_t_12); if (unlikely(!__pyx_t_4)) __PYX_ERR(8, 198, __pyx_L9_except_error)
          __Pyx_GOTREF(__pyx_t_4);
          __pyx_t_20 = __Pyx_PyObject_Call(__pyx_t_1, __pyx_t_4, NULL);
          __Pyx_DECREF(__pyx_t_1); __pyx_t_1 = 0;
          __Pyx_DECREF(__pyx_t_4); __pyx_t_4 = 0;
          if (unlikely(!__pyx_t_20)) __PYX_ERR(8, 198, __pyx_L9_except_error)
          __Pyx_GOTREF(__pyx_t_20);
          __pyx_t_9 = __Pyx_PyObject_IsTrue(__pyx_t_20);
          __Pyx_DECREF(__pyx_t_20); __pyx_t_20 = 0;
          if (__pyx_t_9 < 0) __PYX_ERR(8, 198, __pyx_L9_except_error)
          __pyx_t_8 = ((!(__pyx_t_9 != 0)) != 0);
          if (__pyx_t_8) {
            __Pyx_GIVEREF(__pyx_t_3);
            __Pyx_GIVEREF(__pyx_t_2);
            __Pyx_XGIVEREF(__pyx_t_12);
            __Pyx_ErrRestoreWithState(__pyx_t_3, __pyx_t_2, __pyx_t_12);
            __pyx_t_3 = 0; __pyx_t_2 = 0; __pyx_t_12 = 0; 
            __PYX_ERR(8, 198, __pyx_L9_except_error)
          }
          __Pyx_DECREF(__pyx_t_3); __pyx_t_3 = 0;
          __Pyx_DECREF(__pyx_t_2); __pyx_t_2 = 0;
          __Pyx_DECREF(__pyx_t_12); __pyx_t_12 = 0;
          goto __pyx_L8_exception_handled;
        }
        __pyx_L9_except_error:;
        __Pyx_XGIVEREF(__pyx_t_5);
        __Pyx_XGIVEREF(__pyx_t_6);
        __Pyx_XGIVEREF(__pyx_t_7);
        __Pyx_ExceptionReset(__pyx_t_5, __pyx_t_6, __pyx_t_7);
        goto __pyx_L1_error;
        __pyx_L8_exception_handled:;
        __Pyx_XGIVEREF(__pyx_t_5);
        __Pyx_XGIVEREF(__pyx_t_6);
        __Pyx_XGIVEREF(__pyx_t_7);
        __Pyx_ExceptionReset(__pyx_t_5, __pyx_t_6, __pyx_t_7);
        __pyx_L12_try_end:;
      }
    }
    /*finally:*/ {
      /*normal exit:*/{
        if (__pyx_t_1) {
          __pyx_t_7 = __Pyx_PyObject_Call(__pyx_t_1, __pyx_tuple__26, NULL);
          __Pyx_DECREF(__pyx_t_1); __pyx_t_1 = 0;
          if (unlikely(!__pyx_t_7)) __PYX_ERR(8, 198, __pyx_L1_error)
          __Pyx_GOTREF(__pyx_t_7);
          __Pyx_DECREF(__pyx_t_7); __pyx_t_7 = 0;
        }
        goto __pyx_L6;
      }
      __pyx_L6:;
    }
    goto __pyx_L28;
    __pyx_L3_error:;
    __Pyx_DECREF(__pyx_t_1); __pyx_t_1 = 0;
    goto __pyx_L1_error;
    __pyx_L28:;
  }

  /* "src/python/grpcio/grpc/_cython/_cygrpc/channel.pyx.pxi":158
 * 
 * # TODO(https://github.com/grpc/grpc/issues/14569): This could be a lot simpler.
 * cdef void _call(             # <<<<<<<<<<<<<<
 *     _ChannelState channel_state, _CallState call_state,
 *     grpc_completion_queue *c_completion_queue, on_success, int flags, method,
 */

  /* function exit code */
  goto __pyx_L0;
  __pyx_L1_error:;
  __Pyx_XDECREF(__pyx_t_2);
  __Pyx_XDECREF(__pyx_t_3);
  __Pyx_XDECREF(__pyx_t_4);
  __Pyx_XDECREF(__pyx_t_12);
  __Pyx_XDECREF(__pyx_t_16);
  __Pyx_AddTraceback("grpc._cython.cygrpc._call", __pyx_clineno, __pyx_lineno, __pyx_filename);
  __pyx_L0:;
  __Pyx_XDECREF(__pyx_v_started_tags);
  __Pyx_XDECREF(__pyx_v_operations);
  __Pyx_XDECREF(__pyx_v_user_tag);
  __Pyx_XDECREF(__pyx_v_tag);
  __Pyx_RefNannyFinishContext();
}

/* "src/python/grpcio/grpc/_cython/_cygrpc/channel.pyx.pxi":239
 *     else:
 *       raise ValueError('Cannot invoke RPC: %s' % channel_state.closed_reason)
 * cdef void _process_integrated_call_tag(             # <<<<<<<<<<<<<<
 *     _ChannelState state, _BatchOperationTag tag) except *:
 *   cdef _CallState call_state = state.integrated_call_states.pop(tag)
 */

static void __pyx_f_4grpc_7_cython_6cygrpc__process_integrated_call_tag(struct __pyx_obj_4grpc_7_cython_6cygrpc__ChannelState *__pyx_v_state, struct __pyx_obj_4grpc_7_cython_6cygrpc__BatchOperationTag *__pyx_v_tag) {
  struct __pyx_obj_4grpc_7_cython_6cygrpc__CallState *__pyx_v_call_state = 0;
  __Pyx_RefNannyDeclarations
  PyObject *__pyx_t_1 = NULL;
  int __pyx_t_2;
  int __pyx_t_3;
  int __pyx_t_4;
  __Pyx_RefNannySetupContext("_process_integrated_call_tag", 0);

  /* "src/python/grpcio/grpc/_cython/_cygrpc/channel.pyx.pxi":241
 * cdef void _process_integrated_call_tag(
 *     _ChannelState state, _BatchOperationTag tag) except *:
 *   cdef _CallState call_state = state.integrated_call_states.pop(tag)             # <<<<<<<<<<<<<<
 *   call_state.due.remove(tag)
 *   if not call_state.due:
 */
  if (unlikely(__pyx_v_state->integrated_call_states == Py_None)) {
    PyErr_Format(PyExc_AttributeError, "'NoneType' object has no attribute '%.30s'", "pop");
    __PYX_ERR(8, 241, __pyx_L1_error)
  }
  __pyx_t_1 = __Pyx_PyDict_Pop(__pyx_v_state->integrated_call_states, ((PyObject *)__pyx_v_tag), ((PyObject *)NULL)); if (unlikely(!__pyx_t_1)) __PYX_ERR(8, 241, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_1);
  if (!(likely(((__pyx_t_1) == Py_None) || likely(__Pyx_TypeTest(__pyx_t_1, __pyx_ptype_4grpc_7_cython_6cygrpc__CallState))))) __PYX_ERR(8, 241, __pyx_L1_error)
  __pyx_v_call_state = ((struct __pyx_obj_4grpc_7_cython_6cygrpc__CallState *)__pyx_t_1);
  __pyx_t_1 = 0;

  /* "src/python/grpcio/grpc/_cython/_cygrpc/channel.pyx.pxi":242
 *     _ChannelState state, _BatchOperationTag tag) except *:
 *   cdef _CallState call_state = state.integrated_call_states.pop(tag)
 *   call_state.due.remove(tag)             # <<<<<<<<<<<<<<
 *   if not call_state.due:
 *     grpc_call_unref(call_state.c_call)
 */
  if (unlikely(__pyx_v_call_state->due == Py_None)) {
    PyErr_Format(PyExc_AttributeError, "'NoneType' object has no attribute '%.30s'", "remove");
    __PYX_ERR(8, 242, __pyx_L1_error)
  }
  __pyx_t_2 = __Pyx_PySet_Remove(__pyx_v_call_state->due, ((PyObject *)__pyx_v_tag)); if (unlikely(__pyx_t_2 == ((int)-1))) __PYX_ERR(8, 242, __pyx_L1_error)

  /* "src/python/grpcio/grpc/_cython/_cygrpc/channel.pyx.pxi":243
 *   cdef _CallState call_state = state.integrated_call_states.pop(tag)
 *   call_state.due.remove(tag)
 *   if not call_state.due:             # <<<<<<<<<<<<<<
 *     grpc_call_unref(call_state.c_call)
 *     call_state.c_call = NULL
 */
  __pyx_t_3 = (__pyx_v_call_state->due != Py_None)&&(PySet_GET_SIZE(__pyx_v_call_state->due) != 0);
  __pyx_t_4 = ((!__pyx_t_3) != 0);
  if (__pyx_t_4) {

    /* "src/python/grpcio/grpc/_cython/_cygrpc/channel.pyx.pxi":244
 *   call_state.due.remove(tag)
 *   if not call_state.due:
 *     grpc_call_unref(call_state.c_call)             # <<<<<<<<<<<<<<
 *     call_state.c_call = NULL
 * 
 */
    grpc_call_unref(__pyx_v_call_state->c_call);

    /* "src/python/grpcio/grpc/_cython/_cygrpc/channel.pyx.pxi":245
 *   if not call_state.due:
 *     grpc_call_unref(call_state.c_call)
 *     call_state.c_call = NULL             # <<<<<<<<<<<<<<
 * 
 * 
 */
    __pyx_v_call_state->c_call = NULL;

    /* "src/python/grpcio/grpc/_cython/_cygrpc/channel.pyx.pxi":243
 *   cdef _CallState call_state = state.integrated_call_states.pop(tag)
 *   call_state.due.remove(tag)
 *   if not call_state.due:             # <<<<<<<<<<<<<<
 *     grpc_call_unref(call_state.c_call)
 *     call_state.c_call = NULL
 */
  }

  /* "src/python/grpcio/grpc/_cython/_cygrpc/channel.pyx.pxi":239
 *     else:
 *       raise ValueError('Cannot invoke RPC: %s' % channel_state.closed_reason)
 * cdef void _process_integrated_call_tag(             # <<<<<<<<<<<<<<
 *     _ChannelState state, _BatchOperationTag tag) except *:
 *   cdef _CallState call_state = state.integrated_call_states.pop(tag)
 */

  /* function exit code */
  goto __pyx_L0;
  __pyx_L1_error:;
  __Pyx_XDECREF(__pyx_t_1);
  __Pyx_AddTraceback("grpc._cython.cygrpc._process_integrated_call_tag", __pyx_clineno, __pyx_lineno, __pyx_filename);
  __pyx_L0:;
  __Pyx_XDECREF((PyObject *)__pyx_v_call_state);
  __Pyx_RefNannyFinishContext();
}

/* "src/python/grpcio/grpc/_cython/_cygrpc/channel.pyx.pxi":250
 * cdef class IntegratedCall:
 * 
 *   def __cinit__(self, _ChannelState channel_state, _CallState call_state):             # <<<<<<<<<<<<<<
 *     self._channel_state = channel_state
 *     self._call_state = call_state
 */

/* Python wrapper */
static int __pyx_pw_4grpc_7_cython_6cygrpc_14IntegratedCall_1__cinit__(PyObject *__pyx_v_self, PyObject *__pyx_args, PyObject *__pyx_kwds); /*proto*/
static int __pyx_pw_4grpc_7_cython_6cygrpc_14IntegratedCall_1__cinit__(PyObject *__pyx_v_self, PyObject *__pyx_args, PyObject *__pyx_kwds) {
  struct __pyx_obj_4grpc_7_cython_6cygrpc__ChannelState *__pyx_v_channel_state = 0;
  struct __pyx_obj_4grpc_7_cython_6cygrpc__CallState *__pyx_v_call_state = 0;
  int __pyx_r;
  __Pyx_RefNannyDeclarations
  __Pyx_RefNannySetupContext("__cinit__ (wrapper)", 0);
  {
    static PyObject **__pyx_pyargnames[] = {&__pyx_n_s_channel_state,&__pyx_n_s_call_state,0};
    PyObject* values[2] = {0,0};
    if (unlikely(__pyx_kwds)) {
      Py_ssize_t kw_args;
      const Py_ssize_t pos_args = PyTuple_GET_SIZE(__pyx_args);
      switch (pos_args) {
        case  2: values[1] = PyTuple_GET_ITEM(__pyx_args, 1);
        CYTHON_FALLTHROUGH;
        case  1: values[0] = PyTuple_GET_ITEM(__pyx_args, 0);
        CYTHON_FALLTHROUGH;
        case  0: break;
        default: goto __pyx_L5_argtuple_error;
      }
      kw_args = PyDict_Size(__pyx_kwds);
      switch (pos_args) {
        case  0:
        if (likely((values[0] = __Pyx_PyDict_GetItemStr(__pyx_kwds, __pyx_n_s_channel_state)) != 0)) kw_args--;
        else goto __pyx_L5_argtuple_error;
        CYTHON_FALLTHROUGH;
        case  1:
        if (likely((values[1] = __Pyx_PyDict_GetItemStr(__pyx_kwds, __pyx_n_s_call_state)) != 0)) kw_args--;
        else {
          __Pyx_RaiseArgtupleInvalid("__cinit__", 1, 2, 2, 1); __PYX_ERR(8, 250, __pyx_L3_error)
        }
      }
      if (unlikely(kw_args > 0)) {
        if (unlikely(__Pyx_ParseOptionalKeywords(__pyx_kwds, __pyx_pyargnames, 0, values, pos_args, "__cinit__") < 0)) __PYX_ERR(8, 250, __pyx_L3_error)
      }
    } else if (PyTuple_GET_SIZE(__pyx_args) != 2) {
      goto __pyx_L5_argtuple_error;
    } else {
      values[0] = PyTuple_GET_ITEM(__pyx_args, 0);
      values[1] = PyTuple_GET_ITEM(__pyx_args, 1);
    }
    __pyx_v_channel_state = ((struct __pyx_obj_4grpc_7_cython_6cygrpc__ChannelState *)values[0]);
    __pyx_v_call_state = ((struct __pyx_obj_4grpc_7_cython_6cygrpc__CallState *)values[1]);
  }
  goto __pyx_L4_argument_unpacking_done;
  __pyx_L5_argtuple_error:;
  __Pyx_RaiseArgtupleInvalid("__cinit__", 1, 2, 2, PyTuple_GET_SIZE(__pyx_args)); __PYX_ERR(8, 250, __pyx_L3_error)
  __pyx_L3_error:;
  __Pyx_AddTraceback("grpc._cython.cygrpc.IntegratedCall.__cinit__", __pyx_clineno, __pyx_lineno, __pyx_filename);
  __Pyx_RefNannyFinishContext();
  return -1;
  __pyx_L4_argument_unpacking_done:;
  if (unlikely(!__Pyx_ArgTypeTest(((PyObject *)__pyx_v_channel_state), __pyx_ptype_4grpc_7_cython_6cygrpc__ChannelState, 1, "channel_state", 0))) __PYX_ERR(8, 250, __pyx_L1_error)
  if (unlikely(!__Pyx_ArgTypeTest(((PyObject *)__pyx_v_call_state), __pyx_ptype_4grpc_7_cython_6cygrpc__CallState, 1, "call_state", 0))) __PYX_ERR(8, 250, __pyx_L1_error)
  __pyx_r = __pyx_pf_4grpc_7_cython_6cygrpc_14IntegratedCall___cinit__(((struct __pyx_obj_4grpc_7_cython_6cygrpc_IntegratedCall *)__pyx_v_self), __pyx_v_channel_state, __pyx_v_call_state);

  /* function exit code */
  goto __pyx_L0;
  __pyx_L1_error:;
  __pyx_r = -1;
  __pyx_L0:;
  __Pyx_RefNannyFinishContext();
  return __pyx_r;
}

static int __pyx_pf_4grpc_7_cython_6cygrpc_14IntegratedCall___cinit__(struct __pyx_obj_4grpc_7_cython_6cygrpc_IntegratedCall *__pyx_v_self, struct __pyx_obj_4grpc_7_cython_6cygrpc__ChannelState *__pyx_v_channel_state, struct __pyx_obj_4grpc_7_cython_6cygrpc__CallState *__pyx_v_call_state) {
  int __pyx_r;
  __Pyx_RefNannyDeclarations
  __Pyx_RefNannySetupContext("__cinit__", 0);

  /* "src/python/grpcio/grpc/_cython/_cygrpc/channel.pyx.pxi":251
 * 
 *   def __cinit__(self, _ChannelState channel_state, _CallState call_state):
 *     self._channel_state = channel_state             # <<<<<<<<<<<<<<
 *     self._call_state = call_state
 * 
 */
  __Pyx_INCREF(((PyObject *)__pyx_v_channel_state));
  __Pyx_GIVEREF(((PyObject *)__pyx_v_channel_state));
  __Pyx_GOTREF(__pyx_v_self->_channel_state);
  __Pyx_DECREF(((PyObject *)__pyx_v_self->_channel_state));
  __pyx_v_self->_channel_state = __pyx_v_channel_state;

  /* "src/python/grpcio/grpc/_cython/_cygrpc/channel.pyx.pxi":252
 *   def __cinit__(self, _ChannelState channel_state, _CallState call_state):
 *     self._channel_state = channel_state
 *     self._call_state = call_state             # <<<<<<<<<<<<<<
 * 
 *   def operate(self, operations, tag):
 */
  __Pyx_INCREF(((PyObject *)__pyx_v_call_state));
  __Pyx_GIVEREF(((PyObject *)__pyx_v_call_state));
  __Pyx_GOTREF(__pyx_v_self->_call_state);
  __Pyx_DECREF(((PyObject *)__pyx_v_self->_call_state));
  __pyx_v_self->_call_state = __pyx_v_call_state;

  /* "src/python/grpcio/grpc/_cython/_cygrpc/channel.pyx.pxi":250
 * cdef class IntegratedCall:
 * 
 *   def __cinit__(self, _ChannelState channel_state, _CallState call_state):             # <<<<<<<<<<<<<<
 *     self._channel_state = channel_state
 *     self._call_state = call_state
 */

  /* function exit code */
  __pyx_r = 0;
  __Pyx_RefNannyFinishContext();
  return __pyx_r;
}

/* "src/python/grpcio/grpc/_cython/_cygrpc/channel.pyx.pxi":254
 *     self._call_state = call_state
 * 
 *   def operate(self, operations, tag):             # <<<<<<<<<<<<<<
 *     return _operate_from_integrated_call(
 *         self._channel_state, self._call_state, operations, tag)
 */

/* Python wrapper */
static PyObject *__pyx_pw_4grpc_7_cython_6cygrpc_14IntegratedCall_3operate(PyObject *__pyx_v_self, PyObject *__pyx_args, PyObject *__pyx_kwds); /*proto*/
static PyObject *__pyx_pw_4grpc_7_cython_6cygrpc_14IntegratedCall_3operate(PyObject *__pyx_v_self, PyObject *__pyx_args, PyObject *__pyx_kwds) {
  PyObject *__pyx_v_operations = 0;
  PyObject *__pyx_v_tag = 0;
  PyObject *__pyx_r = 0;
  __Pyx_RefNannyDeclarations
  __Pyx_RefNannySetupContext("operate (wrapper)", 0);
  {
    static PyObject **__pyx_pyargnames[] = {&__pyx_n_s_operations,&__pyx_n_s_tag,0};
    PyObject* values[2] = {0,0};
    if (unlikely(__pyx_kwds)) {
      Py_ssize_t kw_args;
      const Py_ssize_t pos_args = PyTuple_GET_SIZE(__pyx_args);
      switch (pos_args) {
        case  2: values[1] = PyTuple_GET_ITEM(__pyx_args, 1);
        CYTHON_FALLTHROUGH;
        case  1: values[0] = PyTuple_GET_ITEM(__pyx_args, 0);
        CYTHON_FALLTHROUGH;
        case  0: break;
        default: goto __pyx_L5_argtuple_error;
      }
      kw_args = PyDict_Size(__pyx_kwds);
      switch (pos_args) {
        case  0:
        if (likely((values[0] = __Pyx_PyDict_GetItemStr(__pyx_kwds, __pyx_n_s_operations)) != 0)) kw_args--;
        else goto __pyx_L5_argtuple_error;
        CYTHON_FALLTHROUGH;
        case  1:
        if (likely((values[1] = __Pyx_PyDict_GetItemStr(__pyx_kwds, __pyx_n_s_tag)) != 0)) kw_args--;
        else {
          __Pyx_RaiseArgtupleInvalid("operate", 1, 2, 2, 1); __PYX_ERR(8, 254, __pyx_L3_error)
        }
      }
      if (unlikely(kw_args > 0)) {
        if (unlikely(__Pyx_ParseOptionalKeywords(__pyx_kwds, __pyx_pyargnames, 0, values, pos_args, "operate") < 0)) __PYX_ERR(8, 254, __pyx_L3_error)
      }
    } else if (PyTuple_GET_SIZE(__pyx_args) != 2) {
      goto __pyx_L5_argtuple_error;
    } else {
      values[0] = PyTuple_GET_ITEM(__pyx_args, 0);
      values[1] = PyTuple_GET_ITEM(__pyx_args, 1);
    }
    __pyx_v_operations = values[0];
    __pyx_v_tag = values[1];
  }
  goto __pyx_L4_argument_unpacking_done;
  __pyx_L5_argtuple_error:;
  __Pyx_RaiseArgtupleInvalid("operate", 1, 2, 2, PyTuple_GET_SIZE(__pyx_args)); __PYX_ERR(8, 254, __pyx_L3_error)
  __pyx_L3_error:;
  __Pyx_AddTraceback("grpc._cython.cygrpc.IntegratedCall.operate", __pyx_clineno, __pyx_lineno, __pyx_filename);
  __Pyx_RefNannyFinishContext();
  return NULL;
  __pyx_L4_argument_unpacking_done:;
  __pyx_r = __pyx_pf_4grpc_7_cython_6cygrpc_14IntegratedCall_2operate(((struct __pyx_obj_4grpc_7_cython_6cygrpc_IntegratedCall *)__pyx_v_self), __pyx_v_operations, __pyx_v_tag);

  /* function exit code */
  __Pyx_RefNannyFinishContext();
  return __pyx_r;
}

static PyObject *__pyx_pf_4grpc_7_cython_6cygrpc_14IntegratedCall_2operate(struct __pyx_obj_4grpc_7_cython_6cygrpc_IntegratedCall *__pyx_v_self, PyObject *__pyx_v_operations, PyObject *__pyx_v_tag) {
  PyObject *__pyx_r = NULL;
  __Pyx_RefNannyDeclarations
  PyObject *__pyx_t_1 = NULL;
  PyObject *__pyx_t_2 = NULL;
  PyObject *__pyx_t_3 = NULL;
  __Pyx_RefNannySetupContext("operate", 0);

  /* "src/python/grpcio/grpc/_cython/_cygrpc/channel.pyx.pxi":255
 * 
 *   def operate(self, operations, tag):
 *     return _operate_from_integrated_call(             # <<<<<<<<<<<<<<
 *         self._channel_state, self._call_state, operations, tag)
 * 
 */
  __Pyx_XDECREF(__pyx_r);

  /* "src/python/grpcio/grpc/_cython/_cygrpc/channel.pyx.pxi":256
 *   def operate(self, operations, tag):
 *     return _operate_from_integrated_call(
 *         self._channel_state, self._call_state, operations, tag)             # <<<<<<<<<<<<<<
 * 
 *   def cancel(self, code, details):
 */
  __pyx_t_1 = ((PyObject *)__pyx_v_self->_channel_state);
  __Pyx_INCREF(__pyx_t_1);
  __pyx_t_2 = ((PyObject *)__pyx_v_self->_call_state);
  __Pyx_INCREF(__pyx_t_2);

  /* "src/python/grpcio/grpc/_cython/_cygrpc/channel.pyx.pxi":255
 * 
 *   def operate(self, operations, tag):
 *     return _operate_from_integrated_call(             # <<<<<<<<<<<<<<
 *         self._channel_state, self._call_state, operations, tag)
 * 
 */
  __pyx_t_3 = __pyx_f_4grpc_7_cython_6cygrpc__operate_from_integrated_call(((struct __pyx_obj_4grpc_7_cython_6cygrpc__ChannelState *)__pyx_t_1), ((struct __pyx_obj_4grpc_7_cython_6cygrpc__CallState *)__pyx_t_2), __pyx_v_operations, __pyx_v_tag); if (unlikely(!__pyx_t_3)) __PYX_ERR(8, 255, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_3);
  __Pyx_DECREF(__pyx_t_1); __pyx_t_1 = 0;
  __Pyx_DECREF(__pyx_t_2); __pyx_t_2 = 0;
  __pyx_r = __pyx_t_3;
  __pyx_t_3 = 0;
  goto __pyx_L0;

  /* "src/python/grpcio/grpc/_cython/_cygrpc/channel.pyx.pxi":254
 *     self._call_state = call_state
 * 
 *   def operate(self, operations, tag):             # <<<<<<<<<<<<<<
 *     return _operate_from_integrated_call(
 *         self._channel_state, self._call_state, operations, tag)
 */

  /* function exit code */
  __pyx_L1_error:;
  __Pyx_XDECREF(__pyx_t_1);
  __Pyx_XDECREF(__pyx_t_2);
  __Pyx_XDECREF(__pyx_t_3);
  __Pyx_AddTraceback("grpc._cython.cygrpc.IntegratedCall.operate", __pyx_clineno, __pyx_lineno, __pyx_filename);
  __pyx_r = NULL;
  __pyx_L0:;
  __Pyx_XGIVEREF(__pyx_r);
  __Pyx_RefNannyFinishContext();
  return __pyx_r;
}

/* "src/python/grpcio/grpc/_cython/_cygrpc/channel.pyx.pxi":258
 *         self._channel_state, self._call_state, operations, tag)
 * 
 *   def cancel(self, code, details):             # <<<<<<<<<<<<<<
 *     _cancel(self._channel_state, self._call_state, code, details)
 * 
 */

/* Python wrapper */
static PyObject *__pyx_pw_4grpc_7_cython_6cygrpc_14IntegratedCall_5cancel(PyObject *__pyx_v_self, PyObject *__pyx_args, PyObject *__pyx_kwds); /*proto*/
static PyObject *__pyx_pw_4grpc_7_cython_6cygrpc_14IntegratedCall_5cancel(PyObject *__pyx_v_self, PyObject *__pyx_args, PyObject *__pyx_kwds) {
  PyObject *__pyx_v_code = 0;
  PyObject *__pyx_v_details = 0;
  PyObject *__pyx_r = 0;
  __Pyx_RefNannyDeclarations
  __Pyx_RefNannySetupContext("cancel (wrapper)", 0);
  {
    static PyObject **__pyx_pyargnames[] = {&__pyx_n_s_code,&__pyx_n_s_details,0};
    PyObject* values[2] = {0,0};
    if (unlikely(__pyx_kwds)) {
      Py_ssize_t kw_args;
      const Py_ssize_t pos_args = PyTuple_GET_SIZE(__pyx_args);
      switch (pos_args) {
        case  2: values[1] = PyTuple_GET_ITEM(__pyx_args, 1);
        CYTHON_FALLTHROUGH;
        case  1: values[0] = PyTuple_GET_ITEM(__pyx_args, 0);
        CYTHON_FALLTHROUGH;
        case  0: break;
        default: goto __pyx_L5_argtuple_error;
      }
      kw_args = PyDict_Size(__pyx_kwds);
      switch (pos_args) {
        case  0:
        if (likely((values[0] = __Pyx_PyDict_GetItemStr(__pyx_kwds, __pyx_n_s_code)) != 0)) kw_args--;
        else goto __pyx_L5_argtuple_error;
        CYTHON_FALLTHROUGH;
        case  1:
        if (likely((values[1] = __Pyx_PyDict_GetItemStr(__pyx_kwds, __pyx_n_s_details)) != 0)) kw_args--;
        else {
          __Pyx_RaiseArgtupleInvalid("cancel", 1, 2, 2, 1); __PYX_ERR(8, 258, __pyx_L3_error)
        }
      }
      if (unlikely(kw_args > 0)) {
        if (unlikely(__Pyx_ParseOptionalKeywords(__pyx_kwds, __pyx_pyargnames, 0, values, pos_args, "cancel") < 0)) __PYX_ERR(8, 258, __pyx_L3_error)
      }
    } else if (PyTuple_GET_SIZE(__pyx_args) != 2) {
      goto __pyx_L5_argtuple_error;
    } else {
      values[0] = PyTuple_GET_ITEM(__pyx_args, 0);
      values[1] = PyTuple_GET_ITEM(__pyx_args, 1);
    }
    __pyx_v_code = values[0];
    __pyx_v_details = values[1];
  }
  goto __pyx_L4_argument_unpacking_done;
  __pyx_L5_argtuple_error:;
  __Pyx_RaiseArgtupleInvalid("cancel", 1, 2, 2, PyTuple_GET_SIZE(__pyx_args)); __PYX_ERR(8, 258, __pyx_L3_error)
  __pyx_L3_error:;
  __Pyx_AddTraceback("grpc._cython.cygrpc.IntegratedCall.cancel", __pyx_clineno, __pyx_lineno, __pyx_filename);
  __Pyx_RefNannyFinishContext();
  return NULL;
  __pyx_L4_argument_unpacking_done:;
  __pyx_r = __pyx_pf_4grpc_7_cython_6cygrpc_14IntegratedCall_4cancel(((struct __pyx_obj_4grpc_7_cython_6cygrpc_IntegratedCall *)__pyx_v_self), __pyx_v_code, __pyx_v_details);

  /* function exit code */
  __Pyx_RefNannyFinishContext();
  return __pyx_r;
}

static PyObject *__pyx_pf_4grpc_7_cython_6cygrpc_14IntegratedCall_4cancel(struct __pyx_obj_4grpc_7_cython_6cygrpc_IntegratedCall *__pyx_v_self, PyObject *__pyx_v_code, PyObject *__pyx_v_details) {
  PyObject *__pyx_r = NULL;
  __Pyx_RefNannyDeclarations
  PyObject *__pyx_t_1 = NULL;
  PyObject *__pyx_t_2 = NULL;
  grpc_status_code __pyx_t_3;
  PyObject *__pyx_t_4 = NULL;
  __Pyx_RefNannySetupContext("cancel", 0);

  /* "src/python/grpcio/grpc/_cython/_cygrpc/channel.pyx.pxi":259
 * 
 *   def cancel(self, code, details):
 *     _cancel(self._channel_state, self._call_state, code, details)             # <<<<<<<<<<<<<<
 * 
 * 
 */
  __pyx_t_1 = ((PyObject *)__pyx_v_self->_channel_state);
  __Pyx_INCREF(__pyx_t_1);
  __pyx_t_2 = ((PyObject *)__pyx_v_self->_call_state);
  __Pyx_INCREF(__pyx_t_2);
  __pyx_t_3 = ((grpc_status_code)__Pyx_PyInt_As_grpc_status_code(__pyx_v_code)); if (unlikely(PyErr_Occurred())) __PYX_ERR(8, 259, __pyx_L1_error)
  if (!(likely(PyString_CheckExact(__pyx_v_details))||((__pyx_v_details) == Py_None)||(PyErr_Format(PyExc_TypeError, "Expected %.16s, got %.200s", "str", Py_TYPE(__pyx_v_details)->tp_name), 0))) __PYX_ERR(8, 259, __pyx_L1_error)
  __pyx_t_4 = __pyx_f_4grpc_7_cython_6cygrpc__cancel(((struct __pyx_obj_4grpc_7_cython_6cygrpc__ChannelState *)__pyx_t_1), ((struct __pyx_obj_4grpc_7_cython_6cygrpc__CallState *)__pyx_t_2), __pyx_t_3, ((PyObject*)__pyx_v_details)); if (unlikely(!__pyx_t_4)) __PYX_ERR(8, 259, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_4);
  __Pyx_DECREF(__pyx_t_1); __pyx_t_1 = 0;
  __Pyx_DECREF(__pyx_t_2); __pyx_t_2 = 0;
  __Pyx_DECREF(__pyx_t_4); __pyx_t_4 = 0;

  /* "src/python/grpcio/grpc/_cython/_cygrpc/channel.pyx.pxi":258
 *         self._channel_state, self._call_state, operations, tag)
 * 
 *   def cancel(self, code, details):             # <<<<<<<<<<<<<<
 *     _cancel(self._channel_state, self._call_state, code, details)
 * 
 */

  /* function exit code */
  __pyx_r = Py_None; __Pyx_INCREF(Py_None);
  goto __pyx_L0;
  __pyx_L1_error:;
  __Pyx_XDECREF(__pyx_t_1);
  __Pyx_XDECREF(__pyx_t_2);
  __Pyx_XDECREF(__pyx_t_4);
  __Pyx_AddTraceback("grpc._cython.cygrpc.IntegratedCall.cancel", __pyx_clineno, __pyx_lineno, __pyx_filename);
  __pyx_r = NULL;
  __pyx_L0:;
  __Pyx_XGIVEREF(__pyx_r);
  __Pyx_RefNannyFinishContext();
  return __pyx_r;
}

/* "(tree fragment)":1
 * def __reduce_cython__(self):             # <<<<<<<<<<<<<<
 *     raise TypeError("no default __reduce__ due to non-trivial __cinit__")
 * def __setstate_cython__(self, __pyx_state):
 */

/* Python wrapper */
static PyObject *__pyx_pw_4grpc_7_cython_6cygrpc_14IntegratedCall_7__reduce_cython__(PyObject *__pyx_v_self, CYTHON_UNUSED PyObject *unused); /*proto*/
static PyObject *__pyx_pw_4grpc_7_cython_6cygrpc_14IntegratedCall_7__reduce_cython__(PyObject *__pyx_v_self, CYTHON_UNUSED PyObject *unused) {
  PyObject *__pyx_r = 0;
  __Pyx_RefNannyDeclarations
  __Pyx_RefNannySetupContext("__reduce_cython__ (wrapper)", 0);
  __pyx_r = __pyx_pf_4grpc_7_cython_6cygrpc_14IntegratedCall_6__reduce_cython__(((struct __pyx_obj_4grpc_7_cython_6cygrpc_IntegratedCall *)__pyx_v_self));

  /* function exit code */
  __Pyx_RefNannyFinishContext();
  return __pyx_r;
}

static PyObject *__pyx_pf_4grpc_7_cython_6cygrpc_14IntegratedCall_6__reduce_cython__(CYTHON_UNUSED struct __pyx_obj_4grpc_7_cython_6cygrpc_IntegratedCall *__pyx_v_self) {
  PyObject *__pyx_r = NULL;
  __Pyx_RefNannyDeclarations
  PyObject *__pyx_t_1 = NULL;
  __Pyx_RefNannySetupContext("__reduce_cython__", 0);

  /* "(tree fragment)":2
 * def __reduce_cython__(self):
 *     raise TypeError("no default __reduce__ due to non-trivial __cinit__")             # <<<<<<<<<<<<<<
 * def __setstate_cython__(self, __pyx_state):
 *     raise TypeError("no default __reduce__ due to non-trivial __cinit__")
 */
  __pyx_t_1 = __Pyx_PyObject_Call(__pyx_builtin_TypeError, __pyx_tuple__27, NULL); if (unlikely(!__pyx_t_1)) __PYX_ERR(7, 2, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_1);
  __Pyx_Raise(__pyx_t_1, 0, 0, 0);
  __Pyx_DECREF(__pyx_t_1); __pyx_t_1 = 0;
  __PYX_ERR(7, 2, __pyx_L1_error)

  /* "(tree fragment)":1
 * def __reduce_cython__(self):             # <<<<<<<<<<<<<<
 *     raise TypeError("no default __reduce__ due to non-trivial __cinit__")
 * def __setstate_cython__(self, __pyx_state):
 */

  /* function exit code */
  __pyx_L1_error:;
  __Pyx_XDECREF(__pyx_t_1);
  __Pyx_AddTraceback("grpc._cython.cygrpc.IntegratedCall.__reduce_cython__", __pyx_clineno, __pyx_lineno, __pyx_filename);
  __pyx_r = NULL;
  __Pyx_XGIVEREF(__pyx_r);
  __Pyx_RefNannyFinishContext();
  return __pyx_r;
}

/* "(tree fragment)":3
 * def __reduce_cython__(self):
 *     raise TypeError("no default __reduce__ due to non-trivial __cinit__")
 * def __setstate_cython__(self, __pyx_state):             # <<<<<<<<<<<<<<
 *     raise TypeError("no default __reduce__ due to non-trivial __cinit__")
 */

/* Python wrapper */
static PyObject *__pyx_pw_4grpc_7_cython_6cygrpc_14IntegratedCall_9__setstate_cython__(PyObject *__pyx_v_self, PyObject *__pyx_v___pyx_state); /*proto*/
static PyObject *__pyx_pw_4grpc_7_cython_6cygrpc_14IntegratedCall_9__setstate_cython__(PyObject *__pyx_v_self, PyObject *__pyx_v___pyx_state) {
  PyObject *__pyx_r = 0;
  __Pyx_RefNannyDeclarations
  __Pyx_RefNannySetupContext("__setstate_cython__ (wrapper)", 0);
  __pyx_r = __pyx_pf_4grpc_7_cython_6cygrpc_14IntegratedCall_8__setstate_cython__(((struct __pyx_obj_4grpc_7_cython_6cygrpc_IntegratedCall *)__pyx_v_self), ((PyObject *)__pyx_v___pyx_state));

  /* function exit code */
  __Pyx_RefNannyFinishContext();
  return __pyx_r;
}

static PyObject *__pyx_pf_4grpc_7_cython_6cygrpc_14IntegratedCall_8__setstate_cython__(CYTHON_UNUSED struct __pyx_obj_4grpc_7_cython_6cygrpc_IntegratedCall *__pyx_v_self, CYTHON_UNUSED PyObject *__pyx_v___pyx_state) {
  PyObject *__pyx_r = NULL;
  __Pyx_RefNannyDeclarations
  PyObject *__pyx_t_1 = NULL;
  __Pyx_RefNannySetupContext("__setstate_cython__", 0);

  /* "(tree fragment)":4
 *     raise TypeError("no default __reduce__ due to non-trivial __cinit__")
 * def __setstate_cython__(self, __pyx_state):
 *     raise TypeError("no default __reduce__ due to non-trivial __cinit__")             # <<<<<<<<<<<<<<
 */
  __pyx_t_1 = __Pyx_PyObject_Call(__pyx_builtin_TypeError, __pyx_tuple__28, NULL); if (unlikely(!__pyx_t_1)) __PYX_ERR(7, 4, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_1);
  __Pyx_Raise(__pyx_t_1, 0, 0, 0);
  __Pyx_DECREF(__pyx_t_1); __pyx_t_1 = 0;
  __PYX_ERR(7, 4, __pyx_L1_error)

  /* "(tree fragment)":3
 * def __reduce_cython__(self):
 *     raise TypeError("no default __reduce__ due to non-trivial __cinit__")
 * def __setstate_cython__(self, __pyx_state):             # <<<<<<<<<<<<<<
 *     raise TypeError("no default __reduce__ due to non-trivial __cinit__")
 */

  /* function exit code */
  __pyx_L1_error:;
  __Pyx_XDECREF(__pyx_t_1);
  __Pyx_AddTraceback("grpc._cython.cygrpc.IntegratedCall.__setstate_cython__", __pyx_clineno, __pyx_lineno, __pyx_filename);
  __pyx_r = NULL;
  __Pyx_XGIVEREF(__pyx_r);
  __Pyx_RefNannyFinishContext();
  return __pyx_r;
}

/* "src/python/grpcio/grpc/_cython/_cygrpc/channel.pyx.pxi":268
 *   call_state = _CallState()
 * 
 *   def on_success(started_tags):             # <<<<<<<<<<<<<<
 *     for started_tag in started_tags:
 *       state.integrated_call_states[started_tag] = call_state
 */

/* Python wrapper */
static PyObject *__pyx_pw_4grpc_7_cython_6cygrpc_16_integrated_call_1on_success(PyObject *__pyx_self, PyObject *__pyx_v_started_tags); /*proto*/
static PyMethodDef __pyx_mdef_4grpc_7_cython_6cygrpc_16_integrated_call_1on_success = {"on_success", (PyCFunction)__pyx_pw_4grpc_7_cython_6cygrpc_16_integrated_call_1on_success, METH_O, 0};
static PyObject *__pyx_pw_4grpc_7_cython_6cygrpc_16_integrated_call_1on_success(PyObject *__pyx_self, PyObject *__pyx_v_started_tags) {
  PyObject *__pyx_r = 0;
  __Pyx_RefNannyDeclarations
  __Pyx_RefNannySetupContext("on_success (wrapper)", 0);
  __pyx_r = __pyx_pf_4grpc_7_cython_6cygrpc_16_integrated_call_on_success(__pyx_self, ((PyObject *)__pyx_v_started_tags));

  /* function exit code */
  __Pyx_RefNannyFinishContext();
  return __pyx_r;
}

static PyObject *__pyx_pf_4grpc_7_cython_6cygrpc_16_integrated_call_on_success(PyObject *__pyx_self, PyObject *__pyx_v_started_tags) {
  struct __pyx_obj_4grpc_7_cython_6cygrpc___pyx_scope_struct____pyx_f_4grpc_7_cython_6cygrpc__integrated_call *__pyx_cur_scope;
  struct __pyx_obj_4grpc_7_cython_6cygrpc___pyx_scope_struct____pyx_f_4grpc_7_cython_6cygrpc__integrated_call *__pyx_outer_scope;
  PyObject *__pyx_v_started_tag = NULL;
  PyObject *__pyx_r = NULL;
  __Pyx_RefNannyDeclarations
  PyObject *__pyx_t_1 = NULL;
  Py_ssize_t __pyx_t_2;
  PyObject *(*__pyx_t_3)(PyObject *);
  PyObject *__pyx_t_4 = NULL;
  __Pyx_RefNannySetupContext("on_success", 0);
  __pyx_outer_scope = (struct __pyx_obj_4grpc_7_cython_6cygrpc___pyx_scope_struct____pyx_f_4grpc_7_cython_6cygrpc__integrated_call *) __Pyx_CyFunction_GetClosure(__pyx_self);
  __pyx_cur_scope = __pyx_outer_scope;

  /* "src/python/grpcio/grpc/_cython/_cygrpc/channel.pyx.pxi":269
 * 
 *   def on_success(started_tags):
 *     for started_tag in started_tags:             # <<<<<<<<<<<<<<
 *       state.integrated_call_states[started_tag] = call_state
 * 
 */
  if (likely(PyList_CheckExact(__pyx_v_started_tags)) || PyTuple_CheckExact(__pyx_v_started_tags)) {
    __pyx_t_1 = __pyx_v_started_tags; __Pyx_INCREF(__pyx_t_1); __pyx_t_2 = 0;
    __pyx_t_3 = NULL;
  } else {
    __pyx_t_2 = -1; __pyx_t_1 = PyObject_GetIter(__pyx_v_started_tags); if (unlikely(!__pyx_t_1)) __PYX_ERR(8, 269, __pyx_L1_error)
    __Pyx_GOTREF(__pyx_t_1);
    __pyx_t_3 = Py_TYPE(__pyx_t_1)->tp_iternext; if (unlikely(!__pyx_t_3)) __PYX_ERR(8, 269, __pyx_L1_error)
  }
  for (;;) {
    if (likely(!__pyx_t_3)) {
      if (likely(PyList_CheckExact(__pyx_t_1))) {
        if (__pyx_t_2 >= PyList_GET_SIZE(__pyx_t_1)) break;
        #if CYTHON_ASSUME_SAFE_MACROS && !CYTHON_AVOID_BORROWED_REFS
        __pyx_t_4 = PyList_GET_ITEM(__pyx_t_1, __pyx_t_2); __Pyx_INCREF(__pyx_t_4); __pyx_t_2++; if (unlikely(0 < 0)) __PYX_ERR(8, 269, __pyx_L1_error)
        #else
        __pyx_t_4 = PySequence_ITEM(__pyx_t_1, __pyx_t_2); __pyx_t_2++; if (unlikely(!__pyx_t_4)) __PYX_ERR(8, 269, __pyx_L1_error)
        __Pyx_GOTREF(__pyx_t_4);
        #endif
      } else {
        if (__pyx_t_2 >= PyTuple_GET_SIZE(__pyx_t_1)) break;
        #if CYTHON_ASSUME_SAFE_MACROS && !CYTHON_AVOID_BORROWED_REFS
        __pyx_t_4 = PyTuple_GET_ITEM(__pyx_t_1, __pyx_t_2); __Pyx_INCREF(__pyx_t_4); __pyx_t_2++; if (unlikely(0 < 0)) __PYX_ERR(8, 269, __pyx_L1_error)
        #else
        __pyx_t_4 = PySequence_ITEM(__pyx_t_1, __pyx_t_2); __pyx_t_2++; if (unlikely(!__pyx_t_4)) __PYX_ERR(8, 269, __pyx_L1_error)
        __Pyx_GOTREF(__pyx_t_4);
        #endif
      }
    } else {
      __pyx_t_4 = __pyx_t_3(__pyx_t_1);
      if (unlikely(!__pyx_t_4)) {
        PyObject* exc_type = PyErr_Occurred();
        if (exc_type) {
          if (likely(__Pyx_PyErr_GivenExceptionMatches(exc_type, PyExc_StopIteration))) PyErr_Clear();
          else __PYX_ERR(8, 269, __pyx_L1_error)
        }
        break;
      }
      __Pyx_GOTREF(__pyx_t_4);
    }
    __Pyx_XDECREF_SET(__pyx_v_started_tag, __pyx_t_4);
    __pyx_t_4 = 0;

    /* "src/python/grpcio/grpc/_cython/_cygrpc/channel.pyx.pxi":270
 *   def on_success(started_tags):
 *     for started_tag in started_tags:
 *       state.integrated_call_states[started_tag] = call_state             # <<<<<<<<<<<<<<
 * 
 *   _call(
 */
    if (unlikely(!__pyx_cur_scope->__pyx_v_call_state)) { __Pyx_RaiseClosureNameError("call_state"); __PYX_ERR(8, 270, __pyx_L1_error) }
    if (unlikely(!__pyx_cur_scope->__pyx_v_state)) { __Pyx_RaiseClosureNameError("state"); __PYX_ERR(8, 270, __pyx_L1_error) }
    if (unlikely(__pyx_cur_scope->__pyx_v_state->integrated_call_states == Py_None)) {
      PyErr_SetString(PyExc_TypeError, "'NoneType' object is not subscriptable");
      __PYX_ERR(8, 270, __pyx_L1_error)
    }
    if (unlikely(PyDict_SetItem(__pyx_cur_scope->__pyx_v_state->integrated_call_states, __pyx_v_started_tag, ((PyObject *)__pyx_cur_scope->__pyx_v_call_state)) < 0)) __PYX_ERR(8, 270, __pyx_L1_error)

    /* "src/python/grpcio/grpc/_cython/_cygrpc/channel.pyx.pxi":269
 * 
 *   def on_success(started_tags):
 *     for started_tag in started_tags:             # <<<<<<<<<<<<<<
 *       state.integrated_call_states[started_tag] = call_state
 * 
 */
  }
  __Pyx_DECREF(__pyx_t_1); __pyx_t_1 = 0;

  /* "src/python/grpcio/grpc/_cython/_cygrpc/channel.pyx.pxi":268
 *   call_state = _CallState()
 * 
 *   def on_success(started_tags):             # <<<<<<<<<<<<<<
 *     for started_tag in started_tags:
 *       state.integrated_call_states[started_tag] = call_state
 */

  /* function exit code */
  __pyx_r = Py_None; __Pyx_INCREF(Py_None);
  goto __pyx_L0;
  __pyx_L1_error:;
  __Pyx_XDECREF(__pyx_t_1);
  __Pyx_XDECREF(__pyx_t_4);
  __Pyx_AddTraceback("grpc._cython.cygrpc._integrated_call.on_success", __pyx_clineno, __pyx_lineno, __pyx_filename);
  __pyx_r = NULL;
  __pyx_L0:;
  __Pyx_XDECREF(__pyx_v_started_tag);
  __Pyx_XGIVEREF(__pyx_r);
  __Pyx_RefNannyFinishContext();
  return __pyx_r;
}

/* "src/python/grpcio/grpc/_cython/_cygrpc/channel.pyx.pxi":262
 * 
 * 
 * cdef IntegratedCall _integrated_call(             # <<<<<<<<<<<<<<
 *     _ChannelState state, int flags, method, host, object deadline,
 *     object metadata, CallCredentials credentials, operationses_and_user_tags,
 */

static struct __pyx_obj_4grpc_7_cython_6cygrpc_IntegratedCall *__pyx_f_4grpc_7_cython_6cygrpc__integrated_call(struct __pyx_obj_4grpc_7_cython_6cygrpc__ChannelState *__pyx_v_state, int __pyx_v_flags, PyObject *__pyx_v_method, PyObject *__pyx_v_host, PyObject *__pyx_v_deadline, PyObject *__pyx_v_metadata, struct __pyx_obj_4grpc_7_cython_6cygrpc_CallCredentials *__pyx_v_credentials, PyObject *__pyx_v_operationses_and_user_tags, PyObject *__pyx_v_context) {
  struct __pyx_obj_4grpc_7_cython_6cygrpc___pyx_scope_struct____pyx_f_4grpc_7_cython_6cygrpc__integrated_call *__pyx_cur_scope;
  PyObject *__pyx_v_on_success = 0;
  struct __pyx_obj_4grpc_7_cython_6cygrpc_IntegratedCall *__pyx_r = NULL;
  __Pyx_RefNannyDeclarations
  PyObject *__pyx_t_1 = NULL;
  PyObject *__pyx_t_2 = NULL;
  __Pyx_RefNannySetupContext("_integrated_call", 0);
  __pyx_cur_scope = (struct __pyx_obj_4grpc_7_cython_6cygrpc___pyx_scope_struct____pyx_f_4grpc_7_cython_6cygrpc__integrated_call *)__pyx_tp_new_4grpc_7_cython_6cygrpc___pyx_scope_struct____pyx_f_4grpc_7_cython_6cygrpc__integrated_call(__pyx_ptype_4grpc_7_cython_6cygrpc___pyx_scope_struct____pyx_f_4grpc_7_cython_6cygrpc__integrated_call, __pyx_empty_tuple, NULL);
  if (unlikely(!__pyx_cur_scope)) {
    __pyx_cur_scope = ((struct __pyx_obj_4grpc_7_cython_6cygrpc___pyx_scope_struct____pyx_f_4grpc_7_cython_6cygrpc__integrated_call *)Py_None);
    __Pyx_INCREF(Py_None);
    __PYX_ERR(8, 262, __pyx_L1_error)
  } else {
    __Pyx_GOTREF(__pyx_cur_scope);
  }
  __pyx_cur_scope->__pyx_v_state = __pyx_v_state;
  __Pyx_INCREF((PyObject *)__pyx_cur_scope->__pyx_v_state);
  __Pyx_GIVEREF((PyObject *)__pyx_cur_scope->__pyx_v_state);

  /* "src/python/grpcio/grpc/_cython/_cygrpc/channel.pyx.pxi":266
 *     object metadata, CallCredentials credentials, operationses_and_user_tags,
 *     object context):
 *   call_state = _CallState()             # <<<<<<<<<<<<<<
 * 
 *   def on_success(started_tags):
 */
  __pyx_t_1 = __Pyx_PyObject_CallNoArg(((PyObject *)__pyx_ptype_4grpc_7_cython_6cygrpc__CallState)); if (unlikely(!__pyx_t_1)) __PYX_ERR(8, 266, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_1);
  __Pyx_GIVEREF(__pyx_t_1);
  __pyx_cur_scope->__pyx_v_call_state = ((struct __pyx_obj_4grpc_7_cython_6cygrpc__CallState *)__pyx_t_1);
  __pyx_t_1 = 0;

  /* "src/python/grpcio/grpc/_cython/_cygrpc/channel.pyx.pxi":268
 *   call_state = _CallState()
 * 
 *   def on_success(started_tags):             # <<<<<<<<<<<<<<
 *     for started_tag in started_tags:
 *       state.integrated_call_states[started_tag] = call_state
 */
  __pyx_t_1 = __Pyx_CyFunction_NewEx(&__pyx_mdef_4grpc_7_cython_6cygrpc_16_integrated_call_1on_success, 0, __pyx_n_s_integrated_call_locals_on_succe, ((PyObject*)__pyx_cur_scope), __pyx_n_s_grpc__cython_cygrpc, __pyx_d, ((PyObject *)__pyx_codeobj__30)); if (unlikely(!__pyx_t_1)) __PYX_ERR(8, 268, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_1);
  __pyx_v_on_success = __pyx_t_1;
  __pyx_t_1 = 0;

  /* "src/python/grpcio/grpc/_cython/_cygrpc/channel.pyx.pxi":273
 * 
 *   _call(
 *       state, call_state, state.c_call_completion_queue, on_success, flags,             # <<<<<<<<<<<<<<
 *       method, host, deadline, credentials, operationses_and_user_tags, metadata, context)
 * 
 */
  __pyx_t_1 = ((PyObject *)__pyx_cur_scope->__pyx_v_state);
  __Pyx_INCREF(__pyx_t_1);
  __pyx_t_2 = ((PyObject *)__pyx_cur_scope->__pyx_v_call_state);
  __Pyx_INCREF(__pyx_t_2);

  /* "src/python/grpcio/grpc/_cython/_cygrpc/channel.pyx.pxi":272
 *       state.integrated_call_states[started_tag] = call_state
 * 
 *   _call(             # <<<<<<<<<<<<<<
 *       state, call_state, state.c_call_completion_queue, on_success, flags,
 *       method, host, deadline, credentials, operationses_and_user_tags, metadata, context)
 */
  __pyx_f_4grpc_7_cython_6cygrpc__call(((struct __pyx_obj_4grpc_7_cython_6cygrpc__ChannelState *)__pyx_t_1), ((struct __pyx_obj_4grpc_7_cython_6cygrpc__CallState *)__pyx_t_2), __pyx_cur_scope->__pyx_v_state->c_call_completion_queue, __pyx_v_on_success, __pyx_v_flags, __pyx_v_method, __pyx_v_host, __pyx_v_deadline, __pyx_v_credentials, __pyx_v_operationses_and_user_tags, __pyx_v_metadata, __pyx_v_context); if (unlikely(PyErr_Occurred())) __PYX_ERR(8, 272, __pyx_L1_error)
  __Pyx_DECREF(__pyx_t_1); __pyx_t_1 = 0;
  __Pyx_DECREF(__pyx_t_2); __pyx_t_2 = 0;

  /* "src/python/grpcio/grpc/_cython/_cygrpc/channel.pyx.pxi":276
 *       method, host, deadline, credentials, operationses_and_user_tags, metadata, context)
 * 
 *   return IntegratedCall(state, call_state)             # <<<<<<<<<<<<<<
 * 
 * 
 */
  __Pyx_XDECREF(((PyObject *)__pyx_r));
  __pyx_t_2 = PyTuple_New(2); if (unlikely(!__pyx_t_2)) __PYX_ERR(8, 276, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_2);
  __Pyx_INCREF(((PyObject *)__pyx_cur_scope->__pyx_v_state));
  __Pyx_GIVEREF(((PyObject *)__pyx_cur_scope->__pyx_v_state));
  PyTuple_SET_ITEM(__pyx_t_2, 0, ((PyObject *)__pyx_cur_scope->__pyx_v_state));
  __Pyx_INCREF(((PyObject *)__pyx_cur_scope->__pyx_v_call_state));
  __Pyx_GIVEREF(((PyObject *)__pyx_cur_scope->__pyx_v_call_state));
  PyTuple_SET_ITEM(__pyx_t_2, 1, ((PyObject *)__pyx_cur_scope->__pyx_v_call_state));
  __pyx_t_1 = __Pyx_PyObject_Call(((PyObject *)__pyx_ptype_4grpc_7_cython_6cygrpc_IntegratedCall), __pyx_t_2, NULL); if (unlikely(!__pyx_t_1)) __PYX_ERR(8, 276, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_1);
  __Pyx_DECREF(__pyx_t_2); __pyx_t_2 = 0;
  __pyx_r = ((struct __pyx_obj_4grpc_7_cython_6cygrpc_IntegratedCall *)__pyx_t_1);
  __pyx_t_1 = 0;
  goto __pyx_L0;

  /* "src/python/grpcio/grpc/_cython/_cygrpc/channel.pyx.pxi":262
 * 
 * 
 * cdef IntegratedCall _integrated_call(             # <<<<<<<<<<<<<<
 *     _ChannelState state, int flags, method, host, object deadline,
 *     object metadata, CallCredentials credentials, operationses_and_user_tags,
 */

  /* function exit code */
  __pyx_L1_error:;
  __Pyx_XDECREF(__pyx_t_1);
  __Pyx_XDECREF(__pyx_t_2);
  __Pyx_AddTraceback("grpc._cython.cygrpc._integrated_call", __pyx_clineno, __pyx_lineno, __pyx_filename);
  __pyx_r = 0;
  __pyx_L0:;
  __Pyx_XDECREF(__pyx_v_on_success);
  __Pyx_DECREF(((PyObject *)__pyx_cur_scope));
  __Pyx_XGIVEREF((PyObject *)__pyx_r);
  __Pyx_RefNannyFinishContext();
  return __pyx_r;
}

/* "src/python/grpcio/grpc/_cython/_cygrpc/channel.pyx.pxi":279
 * 
 * 
 * cdef object _process_segregated_call_tag(             # <<<<<<<<<<<<<<
 *     _ChannelState state, _CallState call_state,
 *     grpc_completion_queue *c_completion_queue, _BatchOperationTag tag):
 */

static PyObject *__pyx_f_4grpc_7_cython_6cygrpc__process_segregated_call_tag(struct __pyx_obj_4grpc_7_cython_6cygrpc__ChannelState *__pyx_v_state, struct __pyx_obj_4grpc_7_cython_6cygrpc__CallState *__pyx_v_call_state, grpc_completion_queue *__pyx_v_c_completion_queue, struct __pyx_obj_4grpc_7_cython_6cygrpc__BatchOperationTag *__pyx_v_tag) {
  PyObject *__pyx_r = NULL;
  __Pyx_RefNannyDeclarations
  int __pyx_t_1;
  int __pyx_t_2;
  int __pyx_t_3;
  PyObject *__pyx_t_4 = NULL;
  __Pyx_RefNannySetupContext("_process_segregated_call_tag", 0);

  /* "src/python/grpcio/grpc/_cython/_cygrpc/channel.pyx.pxi":282
 *     _ChannelState state, _CallState call_state,
 *     grpc_completion_queue *c_completion_queue, _BatchOperationTag tag):
 *   call_state.due.remove(tag)             # <<<<<<<<<<<<<<
 *   if not call_state.due:
 *     grpc_call_unref(call_state.c_call)
 */
  if (unlikely(__pyx_v_call_state->due == Py_None)) {
    PyErr_Format(PyExc_AttributeError, "'NoneType' object has no attribute '%.30s'", "remove");
    __PYX_ERR(8, 282, __pyx_L1_error)
  }
  __pyx_t_1 = __Pyx_PySet_Remove(__pyx_v_call_state->due, ((PyObject *)__pyx_v_tag)); if (unlikely(__pyx_t_1 == ((int)-1))) __PYX_ERR(8, 282, __pyx_L1_error)

  /* "src/python/grpcio/grpc/_cython/_cygrpc/channel.pyx.pxi":283
 *     grpc_completion_queue *c_completion_queue, _BatchOperationTag tag):
 *   call_state.due.remove(tag)
 *   if not call_state.due:             # <<<<<<<<<<<<<<
 *     grpc_call_unref(call_state.c_call)
 *     call_state.c_call = NULL
 */
  __pyx_t_2 = (__pyx_v_call_state->due != Py_None)&&(PySet_GET_SIZE(__pyx_v_call_state->due) != 0);
  __pyx_t_3 = ((!__pyx_t_2) != 0);
  if (__pyx_t_3) {

    /* "src/python/grpcio/grpc/_cython/_cygrpc/channel.pyx.pxi":284
 *   call_state.due.remove(tag)
 *   if not call_state.due:
 *     grpc_call_unref(call_state.c_call)             # <<<<<<<<<<<<<<
 *     call_state.c_call = NULL
 *     state.segregated_call_states.remove(call_state)
 */
    grpc_call_unref(__pyx_v_call_state->c_call);

    /* "src/python/grpcio/grpc/_cython/_cygrpc/channel.pyx.pxi":285
 *   if not call_state.due:
 *     grpc_call_unref(call_state.c_call)
 *     call_state.c_call = NULL             # <<<<<<<<<<<<<<
 *     state.segregated_call_states.remove(call_state)
 *     _destroy_c_completion_queue(c_completion_queue)
 */
    __pyx_v_call_state->c_call = NULL;

    /* "src/python/grpcio/grpc/_cython/_cygrpc/channel.pyx.pxi":286
 *     grpc_call_unref(call_state.c_call)
 *     call_state.c_call = NULL
 *     state.segregated_call_states.remove(call_state)             # <<<<<<<<<<<<<<
 *     _destroy_c_completion_queue(c_completion_queue)
 *     return True
 */
    if (unlikely(__pyx_v_state->segregated_call_states == Py_None)) {
      PyErr_Format(PyExc_AttributeError, "'NoneType' object has no attribute '%.30s'", "remove");
      __PYX_ERR(8, 286, __pyx_L1_error)
    }
    __pyx_t_1 = __Pyx_PySet_Remove(__pyx_v_state->segregated_call_states, ((PyObject *)__pyx_v_call_state)); if (unlikely(__pyx_t_1 == ((int)-1))) __PYX_ERR(8, 286, __pyx_L1_error)

    /* "src/python/grpcio/grpc/_cython/_cygrpc/channel.pyx.pxi":287
 *     call_state.c_call = NULL
 *     state.segregated_call_states.remove(call_state)
 *     _destroy_c_completion_queue(c_completion_queue)             # <<<<<<<<<<<<<<
 *     return True
 *   else:
 */
    __pyx_t_4 = __pyx_f_4grpc_7_cython_6cygrpc__destroy_c_completion_queue(__pyx_v_c_completion_queue); if (unlikely(!__pyx_t_4)) __PYX_ERR(8, 287, __pyx_L1_error)
    __Pyx_GOTREF(__pyx_t_4);
    __Pyx_DECREF(__pyx_t_4); __pyx_t_4 = 0;

    /* "src/python/grpcio/grpc/_cython/_cygrpc/channel.pyx.pxi":288
 *     state.segregated_call_states.remove(call_state)
 *     _destroy_c_completion_queue(c_completion_queue)
 *     return True             # <<<<<<<<<<<<<<
 *   else:
 *     return False
 */
    __Pyx_XDECREF(__pyx_r);
    __Pyx_INCREF(Py_True);
    __pyx_r = Py_True;
    goto __pyx_L0;

    /* "src/python/grpcio/grpc/_cython/_cygrpc/channel.pyx.pxi":283
 *     grpc_completion_queue *c_completion_queue, _BatchOperationTag tag):
 *   call_state.due.remove(tag)
 *   if not call_state.due:             # <<<<<<<<<<<<<<
 *     grpc_call_unref(call_state.c_call)
 *     call_state.c_call = NULL
 */
  }

  /* "src/python/grpcio/grpc/_cython/_cygrpc/channel.pyx.pxi":290
 *     return True
 *   else:
 *     return False             # <<<<<<<<<<<<<<
 * 
 * 
 */
  /*else*/ {
    __Pyx_XDECREF(__pyx_r);
    __Pyx_INCREF(Py_False);
    __pyx_r = Py_False;
    goto __pyx_L0;
  }

  /* "src/python/grpcio/grpc/_cython/_cygrpc/channel.pyx.pxi":279
 * 
 * 
 * cdef object _process_segregated_call_tag(             # <<<<<<<<<<<<<<
 *     _ChannelState state, _CallState call_state,
 *     grpc_completion_queue *c_completion_queue, _BatchOperationTag tag):
 */

  /* function exit code */
  __pyx_L1_error:;
  __Pyx_XDECREF(__pyx_t_4);
  __Pyx_AddTraceback("grpc._cython.cygrpc._process_segregated_call_tag", __pyx_clineno, __pyx_lineno, __pyx_filename);
  __pyx_r = 0;
  __pyx_L0:;
  __Pyx_XGIVEREF(__pyx_r);
  __Pyx_RefNannyFinishContext();
  return __pyx_r;
}

/* "src/python/grpcio/grpc/_cython/_cygrpc/channel.pyx.pxi":295
 * cdef class SegregatedCall:
 * 
 *   def __cinit__(self, _ChannelState channel_state, _CallState call_state):             # <<<<<<<<<<<<<<
 *     self._channel_state = channel_state
 *     self._call_state = call_state
 */

/* Python wrapper */
static int __pyx_pw_4grpc_7_cython_6cygrpc_14SegregatedCall_1__cinit__(PyObject *__pyx_v_self, PyObject *__pyx_args, PyObject *__pyx_kwds); /*proto*/
static int __pyx_pw_4grpc_7_cython_6cygrpc_14SegregatedCall_1__cinit__(PyObject *__pyx_v_self, PyObject *__pyx_args, PyObject *__pyx_kwds) {
  struct __pyx_obj_4grpc_7_cython_6cygrpc__ChannelState *__pyx_v_channel_state = 0;
  struct __pyx_obj_4grpc_7_cython_6cygrpc__CallState *__pyx_v_call_state = 0;
  int __pyx_r;
  __Pyx_RefNannyDeclarations
  __Pyx_RefNannySetupContext("__cinit__ (wrapper)", 0);
  {
    static PyObject **__pyx_pyargnames[] = {&__pyx_n_s_channel_state,&__pyx_n_s_call_state,0};
    PyObject* values[2] = {0,0};
    if (unlikely(__pyx_kwds)) {
      Py_ssize_t kw_args;
      const Py_ssize_t pos_args = PyTuple_GET_SIZE(__pyx_args);
      switch (pos_args) {
        case  2: values[1] = PyTuple_GET_ITEM(__pyx_args, 1);
        CYTHON_FALLTHROUGH;
        case  1: values[0] = PyTuple_GET_ITEM(__pyx_args, 0);
        CYTHON_FALLTHROUGH;
        case  0: break;
        default: goto __pyx_L5_argtuple_error;
      }
      kw_args = PyDict_Size(__pyx_kwds);
      switch (pos_args) {
        case  0:
        if (likely((values[0] = __Pyx_PyDict_GetItemStr(__pyx_kwds, __pyx_n_s_channel_state)) != 0)) kw_args--;
        else goto __pyx_L5_argtuple_error;
        CYTHON_FALLTHROUGH;
        case  1:
        if (likely((values[1] = __Pyx_PyDict_GetItemStr(__pyx_kwds, __pyx_n_s_call_state)) != 0)) kw_args--;
        else {
          __Pyx_RaiseArgtupleInvalid("__cinit__", 1, 2, 2, 1); __PYX_ERR(8, 295, __pyx_L3_error)
        }
      }
      if (unlikely(kw_args > 0)) {
        if (unlikely(__Pyx_ParseOptionalKeywords(__pyx_kwds, __pyx_pyargnames, 0, values, pos_args, "__cinit__") < 0)) __PYX_ERR(8, 295, __pyx_L3_error)
      }
    } else if (PyTuple_GET_SIZE(__pyx_args) != 2) {
      goto __pyx_L5_argtuple_error;
    } else {
      values[0] = PyTuple_GET_ITEM(__pyx_args, 0);
      values[1] = PyTuple_GET_ITEM(__pyx_args, 1);
    }
    __pyx_v_channel_state = ((struct __pyx_obj_4grpc_7_cython_6cygrpc__ChannelState *)values[0]);
    __pyx_v_call_state = ((struct __pyx_obj_4grpc_7_cython_6cygrpc__CallState *)values[1]);
  }
  goto __pyx_L4_argument_unpacking_done;
  __pyx_L5_argtuple_error:;
  __Pyx_RaiseArgtupleInvalid("__cinit__", 1, 2, 2, PyTuple_GET_SIZE(__pyx_args)); __PYX_ERR(8, 295, __pyx_L3_error)
  __pyx_L3_error:;
  __Pyx_AddTraceback("grpc._cython.cygrpc.SegregatedCall.__cinit__", __pyx_clineno, __pyx_lineno, __pyx_filename);
  __Pyx_RefNannyFinishContext();
  return -1;
  __pyx_L4_argument_unpacking_done:;
  if (unlikely(!__Pyx_ArgTypeTest(((PyObject *)__pyx_v_channel_state), __pyx_ptype_4grpc_7_cython_6cygrpc__ChannelState, 1, "channel_state", 0))) __PYX_ERR(8, 295, __pyx_L1_error)
  if (unlikely(!__Pyx_ArgTypeTest(((PyObject *)__pyx_v_call_state), __pyx_ptype_4grpc_7_cython_6cygrpc__CallState, 1, "call_state", 0))) __PYX_ERR(8, 295, __pyx_L1_error)
  __pyx_r = __pyx_pf_4grpc_7_cython_6cygrpc_14SegregatedCall___cinit__(((struct __pyx_obj_4grpc_7_cython_6cygrpc_SegregatedCall *)__pyx_v_self), __pyx_v_channel_state, __pyx_v_call_state);

  /* function exit code */
  goto __pyx_L0;
  __pyx_L1_error:;
  __pyx_r = -1;
  __pyx_L0:;
  __Pyx_RefNannyFinishContext();
  return __pyx_r;
}

static int __pyx_pf_4grpc_7_cython_6cygrpc_14SegregatedCall___cinit__(struct __pyx_obj_4grpc_7_cython_6cygrpc_SegregatedCall *__pyx_v_self, struct __pyx_obj_4grpc_7_cython_6cygrpc__ChannelState *__pyx_v_channel_state, struct __pyx_obj_4grpc_7_cython_6cygrpc__CallState *__pyx_v_call_state) {
  int __pyx_r;
  __Pyx_RefNannyDeclarations
  __Pyx_RefNannySetupContext("__cinit__", 0);

  /* "src/python/grpcio/grpc/_cython/_cygrpc/channel.pyx.pxi":296
 * 
 *   def __cinit__(self, _ChannelState channel_state, _CallState call_state):
 *     self._channel_state = channel_state             # <<<<<<<<<<<<<<
 *     self._call_state = call_state
 * 
 */
  __Pyx_INCREF(((PyObject *)__pyx_v_channel_state));
  __Pyx_GIVEREF(((PyObject *)__pyx_v_channel_state));
  __Pyx_GOTREF(__pyx_v_self->_channel_state);
  __Pyx_DECREF(((PyObject *)__pyx_v_self->_channel_state));
  __pyx_v_self->_channel_state = __pyx_v_channel_state;

  /* "src/python/grpcio/grpc/_cython/_cygrpc/channel.pyx.pxi":297
 *   def __cinit__(self, _ChannelState channel_state, _CallState call_state):
 *     self._channel_state = channel_state
 *     self._call_state = call_state             # <<<<<<<<<<<<<<
 * 
 *   def operate(self, operations, tag):
 */
  __Pyx_INCREF(((PyObject *)__pyx_v_call_state));
  __Pyx_GIVEREF(((PyObject *)__pyx_v_call_state));
  __Pyx_GOTREF(__pyx_v_self->_call_state);
  __Pyx_DECREF(((PyObject *)__pyx_v_self->_call_state));
  __pyx_v_self->_call_state = __pyx_v_call_state;

  /* "src/python/grpcio/grpc/_cython/_cygrpc/channel.pyx.pxi":295
 * cdef class SegregatedCall:
 * 
 *   def __cinit__(self, _ChannelState channel_state, _CallState call_state):             # <<<<<<<<<<<<<<
 *     self._channel_state = channel_state
 *     self._call_state = call_state
 */

  /* function exit code */
  __pyx_r = 0;
  __Pyx_RefNannyFinishContext();
  return __pyx_r;
}

/* "src/python/grpcio/grpc/_cython/_cygrpc/channel.pyx.pxi":299
 *     self._call_state = call_state
 * 
 *   def operate(self, operations, tag):             # <<<<<<<<<<<<<<
 *     return _operate_from_segregated_call(
 *         self._channel_state, self._call_state, operations, tag)
 */

/* Python wrapper */
static PyObject *__pyx_pw_4grpc_7_cython_6cygrpc_14SegregatedCall_3operate(PyObject *__pyx_v_self, PyObject *__pyx_args, PyObject *__pyx_kwds); /*proto*/
static PyObject *__pyx_pw_4grpc_7_cython_6cygrpc_14SegregatedCall_3operate(PyObject *__pyx_v_self, PyObject *__pyx_args, PyObject *__pyx_kwds) {
  PyObject *__pyx_v_operations = 0;
  PyObject *__pyx_v_tag = 0;
  PyObject *__pyx_r = 0;
  __Pyx_RefNannyDeclarations
  __Pyx_RefNannySetupContext("operate (wrapper)", 0);
  {
    static PyObject **__pyx_pyargnames[] = {&__pyx_n_s_operations,&__pyx_n_s_tag,0};
    PyObject* values[2] = {0,0};
    if (unlikely(__pyx_kwds)) {
      Py_ssize_t kw_args;
      const Py_ssize_t pos_args = PyTuple_GET_SIZE(__pyx_args);
      switch (pos_args) {
        case  2: values[1] = PyTuple_GET_ITEM(__pyx_args, 1);
        CYTHON_FALLTHROUGH;
        case  1: values[0] = PyTuple_GET_ITEM(__pyx_args, 0);
        CYTHON_FALLTHROUGH;
        case  0: break;
        default: goto __pyx_L5_argtuple_error;
      }
      kw_args = PyDict_Size(__pyx_kwds);
      switch (pos_args) {
        case  0:
        if (likely((values[0] = __Pyx_PyDict_GetItemStr(__pyx_kwds, __pyx_n_s_operations)) != 0)) kw_args--;
        else goto __pyx_L5_argtuple_error;
        CYTHON_FALLTHROUGH;
        case  1:
        if (likely((values[1] = __Pyx_PyDict_GetItemStr(__pyx_kwds, __pyx_n_s_tag)) != 0)) kw_args--;
        else {
          __Pyx_RaiseArgtupleInvalid("operate", 1, 2, 2, 1); __PYX_ERR(8, 299, __pyx_L3_error)
        }
      }
      if (unlikely(kw_args > 0)) {
        if (unlikely(__Pyx_ParseOptionalKeywords(__pyx_kwds, __pyx_pyargnames, 0, values, pos_args, "operate") < 0)) __PYX_ERR(8, 299, __pyx_L3_error)
      }
    } else if (PyTuple_GET_SIZE(__pyx_args) != 2) {
      goto __pyx_L5_argtuple_error;
    } else {
      values[0] = PyTuple_GET_ITEM(__pyx_args, 0);
      values[1] = PyTuple_GET_ITEM(__pyx_args, 1);
    }
    __pyx_v_operations = values[0];
    __pyx_v_tag = values[1];
  }
  goto __pyx_L4_argument_unpacking_done;
  __pyx_L5_argtuple_error:;
  __Pyx_RaiseArgtupleInvalid("operate", 1, 2, 2, PyTuple_GET_SIZE(__pyx_args)); __PYX_ERR(8, 299, __pyx_L3_error)
  __pyx_L3_error:;
  __Pyx_AddTraceback("grpc._cython.cygrpc.SegregatedCall.operate", __pyx_clineno, __pyx_lineno, __pyx_filename);
  __Pyx_RefNannyFinishContext();
  return NULL;
  __pyx_L4_argument_unpacking_done:;
  __pyx_r = __pyx_pf_4grpc_7_cython_6cygrpc_14SegregatedCall_2operate(((struct __pyx_obj_4grpc_7_cython_6cygrpc_SegregatedCall *)__pyx_v_self), __pyx_v_operations, __pyx_v_tag);

  /* function exit code */
  __Pyx_RefNannyFinishContext();
  return __pyx_r;
}

static PyObject *__pyx_pf_4grpc_7_cython_6cygrpc_14SegregatedCall_2operate(struct __pyx_obj_4grpc_7_cython_6cygrpc_SegregatedCall *__pyx_v_self, PyObject *__pyx_v_operations, PyObject *__pyx_v_tag) {
  PyObject *__pyx_r = NULL;
  __Pyx_RefNannyDeclarations
  PyObject *__pyx_t_1 = NULL;
  PyObject *__pyx_t_2 = NULL;
  PyObject *__pyx_t_3 = NULL;
  __Pyx_RefNannySetupContext("operate", 0);

  /* "src/python/grpcio/grpc/_cython/_cygrpc/channel.pyx.pxi":300
 * 
 *   def operate(self, operations, tag):
 *     return _operate_from_segregated_call(             # <<<<<<<<<<<<<<
 *         self._channel_state, self._call_state, operations, tag)
 * 
 */
  __Pyx_XDECREF(__pyx_r);

  /* "src/python/grpcio/grpc/_cython/_cygrpc/channel.pyx.pxi":301
 *   def operate(self, operations, tag):
 *     return _operate_from_segregated_call(
 *         self._channel_state, self._call_state, operations, tag)             # <<<<<<<<<<<<<<
 * 
 *   def cancel(self, code, details):
 */
  __pyx_t_1 = ((PyObject *)__pyx_v_self->_channel_state);
  __Pyx_INCREF(__pyx_t_1);
  __pyx_t_2 = ((PyObject *)__pyx_v_self->_call_state);
  __Pyx_INCREF(__pyx_t_2);

  /* "src/python/grpcio/grpc/_cython/_cygrpc/channel.pyx.pxi":300
 * 
 *   def operate(self, operations, tag):
 *     return _operate_from_segregated_call(             # <<<<<<<<<<<<<<
 *         self._channel_state, self._call_state, operations, tag)
 * 
 */
  __pyx_t_3 = __pyx_f_4grpc_7_cython_6cygrpc__operate_from_segregated_call(((struct __pyx_obj_4grpc_7_cython_6cygrpc__ChannelState *)__pyx_t_1), ((struct __pyx_obj_4grpc_7_cython_6cygrpc__CallState *)__pyx_t_2), __pyx_v_operations, __pyx_v_tag); if (unlikely(!__pyx_t_3)) __PYX_ERR(8, 300, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_3);
  __Pyx_DECREF(__pyx_t_1); __pyx_t_1 = 0;
  __Pyx_DECREF(__pyx_t_2); __pyx_t_2 = 0;
  __pyx_r = __pyx_t_3;
  __pyx_t_3 = 0;
  goto __pyx_L0;

  /* "src/python/grpcio/grpc/_cython/_cygrpc/channel.pyx.pxi":299
 *     self._call_state = call_state
 * 
 *   def operate(self, operations, tag):             # <<<<<<<<<<<<<<
 *     return _operate_from_segregated_call(
 *         self._channel_state, self._call_state, operations, tag)
 */

  /* function exit code */
  __pyx_L1_error:;
  __Pyx_XDECREF(__pyx_t_1);
  __Pyx_XDECREF(__pyx_t_2);
  __Pyx_XDECREF(__pyx_t_3);
  __Pyx_AddTraceback("grpc._cython.cygrpc.SegregatedCall.operate", __pyx_clineno, __pyx_lineno, __pyx_filename);
  __pyx_r = NULL;
  __pyx_L0:;
  __Pyx_XGIVEREF(__pyx_r);
  __Pyx_RefNannyFinishContext();
  return __pyx_r;
}

/* "src/python/grpcio/grpc/_cython/_cygrpc/channel.pyx.pxi":303
 *         self._channel_state, self._call_state, operations, tag)
 * 
 *   def cancel(self, code, details):             # <<<<<<<<<<<<<<
 *     _cancel(self._channel_state, self._call_state, code, details)
 * 
 */

/* Python wrapper */
static PyObject *__pyx_pw_4grpc_7_cython_6cygrpc_14SegregatedCall_5cancel(PyObject *__pyx_v_self, PyObject *__pyx_args, PyObject *__pyx_kwds); /*proto*/
static PyObject *__pyx_pw_4grpc_7_cython_6cygrpc_14SegregatedCall_5cancel(PyObject *__pyx_v_self, PyObject *__pyx_args, PyObject *__pyx_kwds) {
  PyObject *__pyx_v_code = 0;
  PyObject *__pyx_v_details = 0;
  PyObject *__pyx_r = 0;
  __Pyx_RefNannyDeclarations
  __Pyx_RefNannySetupContext("cancel (wrapper)", 0);
  {
    static PyObject **__pyx_pyargnames[] = {&__pyx_n_s_code,&__pyx_n_s_details,0};
    PyObject* values[2] = {0,0};
    if (unlikely(__pyx_kwds)) {
      Py_ssize_t kw_args;
      const Py_ssize_t pos_args = PyTuple_GET_SIZE(__pyx_args);
      switch (pos_args) {
        case  2: values[1] = PyTuple_GET_ITEM(__pyx_args, 1);
        CYTHON_FALLTHROUGH;
        case  1: values[0] = PyTuple_GET_ITEM(__pyx_args, 0);
        CYTHON_FALLTHROUGH;
        case  0: break;
        default: goto __pyx_L5_argtuple_error;
      }
      kw_args = PyDict_Size(__pyx_kwds);
      switch (pos_args) {
        case  0:
        if (likely((values[0] = __Pyx_PyDict_GetItemStr(__pyx_kwds, __pyx_n_s_code)) != 0)) kw_args--;
        else goto __pyx_L5_argtuple_error;
        CYTHON_FALLTHROUGH;
        case  1:
        if (likely((values[1] = __Pyx_PyDict_GetItemStr(__pyx_kwds, __pyx_n_s_details)) != 0)) kw_args--;
        else {
          __Pyx_RaiseArgtupleInvalid("cancel", 1, 2, 2, 1); __PYX_ERR(8, 303, __pyx_L3_error)
        }
      }
      if (unlikely(kw_args > 0)) {
        if (unlikely(__Pyx_ParseOptionalKeywords(__pyx_kwds, __pyx_pyargnames, 0, values, pos_args, "cancel") < 0)) __PYX_ERR(8, 303, __pyx_L3_error)
      }
    } else if (PyTuple_GET_SIZE(__pyx_args) != 2) {
      goto __pyx_L5_argtuple_error;
    } else {
      values[0] = PyTuple_GET_ITEM(__pyx_args, 0);
      values[1] = PyTuple_GET_ITEM(__pyx_args, 1);
    }
    __pyx_v_code = values[0];
    __pyx_v_details = values[1];
  }
  goto __pyx_L4_argument_unpacking_done;
  __pyx_L5_argtuple_error:;
  __Pyx_RaiseArgtupleInvalid("cancel", 1, 2, 2, PyTuple_GET_SIZE(__pyx_args)); __PYX_ERR(8, 303, __pyx_L3_error)
  __pyx_L3_error:;
  __Pyx_AddTraceback("grpc._cython.cygrpc.SegregatedCall.cancel", __pyx_clineno, __pyx_lineno, __pyx_filename);
  __Pyx_RefNannyFinishContext();
  return NULL;
  __pyx_L4_argument_unpacking_done:;
  __pyx_r = __pyx_pf_4grpc_7_cython_6cygrpc_14SegregatedCall_4cancel(((struct __pyx_obj_4grpc_7_cython_6cygrpc_SegregatedCall *)__pyx_v_self), __pyx_v_code, __pyx_v_details);

  /* function exit code */
  __Pyx_RefNannyFinishContext();
  return __pyx_r;
}

static PyObject *__pyx_pf_4grpc_7_cython_6cygrpc_14SegregatedCall_4cancel(struct __pyx_obj_4grpc_7_cython_6cygrpc_SegregatedCall *__pyx_v_self, PyObject *__pyx_v_code, PyObject *__pyx_v_details) {
  PyObject *__pyx_r = NULL;
  __Pyx_RefNannyDeclarations
  PyObject *__pyx_t_1 = NULL;
  PyObject *__pyx_t_2 = NULL;
  grpc_status_code __pyx_t_3;
  PyObject *__pyx_t_4 = NULL;
  __Pyx_RefNannySetupContext("cancel", 0);

  /* "src/python/grpcio/grpc/_cython/_cygrpc/channel.pyx.pxi":304
 * 
 *   def cancel(self, code, details):
 *     _cancel(self._channel_state, self._call_state, code, details)             # <<<<<<<<<<<<<<
 * 
 *   def next_event(self):
 */
  __pyx_t_1 = ((PyObject *)__pyx_v_self->_channel_state);
  __Pyx_INCREF(__pyx_t_1);
  __pyx_t_2 = ((PyObject *)__pyx_v_self->_call_state);
  __Pyx_INCREF(__pyx_t_2);
  __pyx_t_3 = ((grpc_status_code)__Pyx_PyInt_As_grpc_status_code(__pyx_v_code)); if (unlikely(PyErr_Occurred())) __PYX_ERR(8, 304, __pyx_L1_error)
  if (!(likely(PyString_CheckExact(__pyx_v_details))||((__pyx_v_details) == Py_None)||(PyErr_Format(PyExc_TypeError, "Expected %.16s, got %.200s", "str", Py_TYPE(__pyx_v_details)->tp_name), 0))) __PYX_ERR(8, 304, __pyx_L1_error)
  __pyx_t_4 = __pyx_f_4grpc_7_cython_6cygrpc__cancel(((struct __pyx_obj_4grpc_7_cython_6cygrpc__ChannelState *)__pyx_t_1), ((struct __pyx_obj_4grpc_7_cython_6cygrpc__CallState *)__pyx_t_2), __pyx_t_3, ((PyObject*)__pyx_v_details)); if (unlikely(!__pyx_t_4)) __PYX_ERR(8, 304, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_4);
  __Pyx_DECREF(__pyx_t_1); __pyx_t_1 = 0;
  __Pyx_DECREF(__pyx_t_2); __pyx_t_2 = 0;
  __Pyx_DECREF(__pyx_t_4); __pyx_t_4 = 0;

  /* "src/python/grpcio/grpc/_cython/_cygrpc/channel.pyx.pxi":303
 *         self._channel_state, self._call_state, operations, tag)
 * 
 *   def cancel(self, code, details):             # <<<<<<<<<<<<<<
 *     _cancel(self._channel_state, self._call_state, code, details)
 * 
 */

  /* function exit code */
  __pyx_r = Py_None; __Pyx_INCREF(Py_None);
  goto __pyx_L0;
  __pyx_L1_error:;
  __Pyx_XDECREF(__pyx_t_1);
  __Pyx_XDECREF(__pyx_t_2);
  __Pyx_XDECREF(__pyx_t_4);
  __Pyx_AddTraceback("grpc._cython.cygrpc.SegregatedCall.cancel", __pyx_clineno, __pyx_lineno, __pyx_filename);
  __pyx_r = NULL;
  __pyx_L0:;
  __Pyx_XGIVEREF(__pyx_r);
  __Pyx_RefNannyFinishContext();
  return __pyx_r;
}

/* "src/python/grpcio/grpc/_cython/_cygrpc/channel.pyx.pxi":306
 *     _cancel(self._channel_state, self._call_state, code, details)
 * 
 *   def next_event(self):             # <<<<<<<<<<<<<<
 *     def on_success(tag):
 *       _process_segregated_call_tag(
 */

/* Python wrapper */
static PyObject *__pyx_pw_4grpc_7_cython_6cygrpc_14SegregatedCall_7next_event(PyObject *__pyx_v_self, CYTHON_UNUSED PyObject *unused); /*proto*/
static PyObject *__pyx_pw_4grpc_7_cython_6cygrpc_14SegregatedCall_7next_event(PyObject *__pyx_v_self, CYTHON_UNUSED PyObject *unused) {
  PyObject *__pyx_r = 0;
  __Pyx_RefNannyDeclarations
  __Pyx_RefNannySetupContext("next_event (wrapper)", 0);
  __pyx_r = __pyx_pf_4grpc_7_cython_6cygrpc_14SegregatedCall_6next_event(((struct __pyx_obj_4grpc_7_cython_6cygrpc_SegregatedCall *)__pyx_v_self));

  /* function exit code */
  __Pyx_RefNannyFinishContext();
  return __pyx_r;
}

/* "src/python/grpcio/grpc/_cython/_cygrpc/channel.pyx.pxi":307
 * 
 *   def next_event(self):
 *     def on_success(tag):             # <<<<<<<<<<<<<<
 *       _process_segregated_call_tag(
 *         self._channel_state, self._call_state, self._c_completion_queue, tag)
 */

/* Python wrapper */
static PyObject *__pyx_pw_4grpc_7_cython_6cygrpc_14SegregatedCall_10next_event_1on_success(PyObject *__pyx_self, PyObject *__pyx_v_tag); /*proto*/
static PyMethodDef __pyx_mdef_4grpc_7_cython_6cygrpc_14SegregatedCall_10next_event_1on_success = {"on_success", (PyCFunction)__pyx_pw_4grpc_7_cython_6cygrpc_14SegregatedCall_10next_event_1on_success, METH_O, 0};
static PyObject *__pyx_pw_4grpc_7_cython_6cygrpc_14SegregatedCall_10next_event_1on_success(PyObject *__pyx_self, PyObject *__pyx_v_tag) {
  PyObject *__pyx_r = 0;
  __Pyx_RefNannyDeclarations
  __Pyx_RefNannySetupContext("on_success (wrapper)", 0);
  __pyx_r = __pyx_pf_4grpc_7_cython_6cygrpc_14SegregatedCall_10next_event_on_success(__pyx_self, ((PyObject *)__pyx_v_tag));

  /* function exit code */
  __Pyx_RefNannyFinishContext();
  return __pyx_r;
}

static PyObject *__pyx_pf_4grpc_7_cython_6cygrpc_14SegregatedCall_10next_event_on_success(PyObject *__pyx_self, PyObject *__pyx_v_tag) {
  struct __pyx_obj_4grpc_7_cython_6cygrpc___pyx_scope_struct_1_next_event *__pyx_cur_scope;
  struct __pyx_obj_4grpc_7_cython_6cygrpc___pyx_scope_struct_1_next_event *__pyx_outer_scope;
  PyObject *__pyx_r = NULL;
  __Pyx_RefNannyDeclarations
  PyObject *__pyx_t_1 = NULL;
  PyObject *__pyx_t_2 = NULL;
  PyObject *__pyx_t_3 = NULL;
  __Pyx_RefNannySetupContext("on_success", 0);
  __pyx_outer_scope = (struct __pyx_obj_4grpc_7_cython_6cygrpc___pyx_scope_struct_1_next_event *) __Pyx_CyFunction_GetClosure(__pyx_self);
  __pyx_cur_scope = __pyx_outer_scope;

  /* "src/python/grpcio/grpc/_cython/_cygrpc/channel.pyx.pxi":309
 *     def on_success(tag):
 *       _process_segregated_call_tag(
 *         self._channel_state, self._call_state, self._c_completion_queue, tag)             # <<<<<<<<<<<<<<
 *     return _next_call_event(
 *         self._channel_state, self._c_completion_queue, on_success, None)
 */
  if (unlikely(!__pyx_cur_scope->__pyx_v_self)) { __Pyx_RaiseClosureNameError("self"); __PYX_ERR(8, 309, __pyx_L1_error) }
  __pyx_t_1 = ((PyObject *)__pyx_cur_scope->__pyx_v_self->_channel_state);
  __Pyx_INCREF(__pyx_t_1);
  if (unlikely(!__pyx_cur_scope->__pyx_v_self)) { __Pyx_RaiseClosureNameError("self"); __PYX_ERR(8, 309, __pyx_L1_error) }
  __pyx_t_2 = ((PyObject *)__pyx_cur_scope->__pyx_v_self->_call_state);
  __Pyx_INCREF(__pyx_t_2);
  if (unlikely(!__pyx_cur_scope->__pyx_v_self)) { __Pyx_RaiseClosureNameError("self"); __PYX_ERR(8, 309, __pyx_L1_error) }
  if (!(likely(((__pyx_v_tag) == Py_None) || likely(__Pyx_TypeTest(__pyx_v_tag, __pyx_ptype_4grpc_7_cython_6cygrpc__BatchOperationTag))))) __PYX_ERR(8, 309, __pyx_L1_error)

  /* "src/python/grpcio/grpc/_cython/_cygrpc/channel.pyx.pxi":308
 *   def next_event(self):
 *     def on_success(tag):
 *       _process_segregated_call_tag(             # <<<<<<<<<<<<<<
 *         self._channel_state, self._call_state, self._c_completion_queue, tag)
 *     return _next_call_event(
 */
  __pyx_t_3 = __pyx_f_4grpc_7_cython_6cygrpc__process_segregated_call_tag(((struct __pyx_obj_4grpc_7_cython_6cygrpc__ChannelState *)__pyx_t_1), ((struct __pyx_obj_4grpc_7_cython_6cygrpc__CallState *)__pyx_t_2), __pyx_cur_scope->__pyx_v_self->_c_completion_queue, ((struct __pyx_obj_4grpc_7_cython_6cygrpc__BatchOperationTag *)__pyx_v_tag)); if (unlikely(!__pyx_t_3)) __PYX_ERR(8, 308, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_3);
  __Pyx_DECREF(__pyx_t_1); __pyx_t_1 = 0;
  __Pyx_DECREF(__pyx_t_2); __pyx_t_2 = 0;
  __Pyx_DECREF(__pyx_t_3); __pyx_t_3 = 0;

  /* "src/python/grpcio/grpc/_cython/_cygrpc/channel.pyx.pxi":307
 * 
 *   def next_event(self):
 *     def on_success(tag):             # <<<<<<<<<<<<<<
 *       _process_segregated_call_tag(
 *         self._channel_state, self._call_state, self._c_completion_queue, tag)
 */

  /* function exit code */
  __pyx_r = Py_None; __Pyx_INCREF(Py_None);
  goto __pyx_L0;
  __pyx_L1_error:;
  __Pyx_XDECREF(__pyx_t_1);
  __Pyx_XDECREF(__pyx_t_2);
  __Pyx_XDECREF(__pyx_t_3);
  __Pyx_AddTraceback("grpc._cython.cygrpc.SegregatedCall.next_event.on_success", __pyx_clineno, __pyx_lineno, __pyx_filename);
  __pyx_r = NULL;
  __pyx_L0:;
  __Pyx_XGIVEREF(__pyx_r);
  __Pyx_RefNannyFinishContext();
  return __pyx_r;
}

/* "src/python/grpcio/grpc/_cython/_cygrpc/channel.pyx.pxi":306
 *     _cancel(self._channel_state, self._call_state, code, details)
 * 
 *   def next_event(self):             # <<<<<<<<<<<<<<
 *     def on_success(tag):
 *       _process_segregated_call_tag(
 */

static PyObject *__pyx_pf_4grpc_7_cython_6cygrpc_14SegregatedCall_6next_event(struct __pyx_obj_4grpc_7_cython_6cygrpc_SegregatedCall *__pyx_v_self) {
  struct __pyx_obj_4grpc_7_cython_6cygrpc___pyx_scope_struct_1_next_event *__pyx_cur_scope;
  PyObject *__pyx_v_on_success = 0;
  PyObject *__pyx_r = NULL;
  __Pyx_RefNannyDeclarations
  PyObject *__pyx_t_1 = NULL;
  PyObject *__pyx_t_2 = NULL;
  __Pyx_RefNannySetupContext("next_event", 0);
  __pyx_cur_scope = (struct __pyx_obj_4grpc_7_cython_6cygrpc___pyx_scope_struct_1_next_event *)__pyx_tp_new_4grpc_7_cython_6cygrpc___pyx_scope_struct_1_next_event(__pyx_ptype_4grpc_7_cython_6cygrpc___pyx_scope_struct_1_next_event, __pyx_empty_tuple, NULL);
  if (unlikely(!__pyx_cur_scope)) {
    __pyx_cur_scope = ((struct __pyx_obj_4grpc_7_cython_6cygrpc___pyx_scope_struct_1_next_event *)Py_None);
    __Pyx_INCREF(Py_None);
    __PYX_ERR(8, 306, __pyx_L1_error)
  } else {
    __Pyx_GOTREF(__pyx_cur_scope);
  }
  __pyx_cur_scope->__pyx_v_self = __pyx_v_self;
  __Pyx_INCREF((PyObject *)__pyx_cur_scope->__pyx_v_self);
  __Pyx_GIVEREF((PyObject *)__pyx_cur_scope->__pyx_v_self);

  /* "src/python/grpcio/grpc/_cython/_cygrpc/channel.pyx.pxi":307
 * 
 *   def next_event(self):
 *     def on_success(tag):             # <<<<<<<<<<<<<<
 *       _process_segregated_call_tag(
 *         self._channel_state, self._call_state, self._c_completion_queue, tag)
 */
  __pyx_t_1 = __Pyx_CyFunction_NewEx(&__pyx_mdef_4grpc_7_cython_6cygrpc_14SegregatedCall_10next_event_1on_success, 0, __pyx_n_s_next_event_locals_on_success, ((PyObject*)__pyx_cur_scope), __pyx_n_s_grpc__cython_cygrpc, __pyx_d, ((PyObject *)__pyx_codeobj__32)); if (unlikely(!__pyx_t_1)) __PYX_ERR(8, 307, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_1);
  __pyx_v_on_success = __pyx_t_1;
  __pyx_t_1 = 0;

  /* "src/python/grpcio/grpc/_cython/_cygrpc/channel.pyx.pxi":310
 *       _process_segregated_call_tag(
 *         self._channel_state, self._call_state, self._c_completion_queue, tag)
 *     return _next_call_event(             # <<<<<<<<<<<<<<
 *         self._channel_state, self._c_completion_queue, on_success, None)
 * 
 */
  __Pyx_XDECREF(__pyx_r);

  /* "src/python/grpcio/grpc/_cython/_cygrpc/channel.pyx.pxi":311
 *         self._channel_state, self._call_state, self._c_completion_queue, tag)
 *     return _next_call_event(
 *         self._channel_state, self._c_completion_queue, on_success, None)             # <<<<<<<<<<<<<<
 * 
 * 
 */
  __pyx_t_1 = ((PyObject *)__pyx_cur_scope->__pyx_v_self->_channel_state);
  __Pyx_INCREF(__pyx_t_1);

  /* "src/python/grpcio/grpc/_cython/_cygrpc/channel.pyx.pxi":310
 *       _process_segregated_call_tag(
 *         self._channel_state, self._call_state, self._c_completion_queue, tag)
 *     return _next_call_event(             # <<<<<<<<<<<<<<
 *         self._channel_state, self._c_completion_queue, on_success, None)
 * 
 */
  __pyx_t_2 = __pyx_f_4grpc_7_cython_6cygrpc__next_call_event(((struct __pyx_obj_4grpc_7_cython_6cygrpc__ChannelState *)__pyx_t_1), __pyx_cur_scope->__pyx_v_self->_c_completion_queue, __pyx_v_on_success, Py_None); if (unlikely(!__pyx_t_2)) __PYX_ERR(8, 310, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_2);
  __Pyx_DECREF(__pyx_t_1); __pyx_t_1 = 0;
  __pyx_r = __pyx_t_2;
  __pyx_t_2 = 0;
  goto __pyx_L0;

  /* "src/python/grpcio/grpc/_cython/_cygrpc/channel.pyx.pxi":306
 *     _cancel(self._channel_state, self._call_state, code, details)
 * 
 *   def next_event(self):             # <<<<<<<<<<<<<<
 *     def on_success(tag):
 *       _process_segregated_call_tag(
 */

  /* function exit code */
  __pyx_L1_error:;
  __Pyx_XDECREF(__pyx_t_1);
  __Pyx_XDECREF(__pyx_t_2);
  __Pyx_AddTraceback("grpc._cython.cygrpc.SegregatedCall.next_event", __pyx_clineno, __pyx_lineno, __pyx_filename);
  __pyx_r = NULL;
  __pyx_L0:;
  __Pyx_XDECREF(__pyx_v_on_success);
  __Pyx_DECREF(((PyObject *)__pyx_cur_scope));
  __Pyx_XGIVEREF(__pyx_r);
  __Pyx_RefNannyFinishContext();
  return __pyx_r;
}

/* "(tree fragment)":1
 * def __reduce_cython__(self):             # <<<<<<<<<<<<<<
 *     raise TypeError("no default __reduce__ due to non-trivial __cinit__")
 * def __setstate_cython__(self, __pyx_state):
 */

/* Python wrapper */
static PyObject *__pyx_pw_4grpc_7_cython_6cygrpc_14SegregatedCall_9__reduce_cython__(PyObject *__pyx_v_self, CYTHON_UNUSED PyObject *unused); /*proto*/
static PyObject *__pyx_pw_4grpc_7_cython_6cygrpc_14SegregatedCall_9__reduce_cython__(PyObject *__pyx_v_self, CYTHON_UNUSED PyObject *unused) {
  PyObject *__pyx_r = 0;
  __Pyx_RefNannyDeclarations
  __Pyx_RefNannySetupContext("__reduce_cython__ (wrapper)", 0);
  __pyx_r = __pyx_pf_4grpc_7_cython_6cygrpc_14SegregatedCall_8__reduce_cython__(((struct __pyx_obj_4grpc_7_cython_6cygrpc_SegregatedCall *)__pyx_v_self));

  /* function exit code */
  __Pyx_RefNannyFinishContext();
  return __pyx_r;
}

static PyObject *__pyx_pf_4grpc_7_cython_6cygrpc_14SegregatedCall_8__reduce_cython__(CYTHON_UNUSED struct __pyx_obj_4grpc_7_cython_6cygrpc_SegregatedCall *__pyx_v_self) {
  PyObject *__pyx_r = NULL;
  __Pyx_RefNannyDeclarations
  PyObject *__pyx_t_1 = NULL;
  __Pyx_RefNannySetupContext("__reduce_cython__", 0);

  /* "(tree fragment)":2
 * def __reduce_cython__(self):
 *     raise TypeError("no default __reduce__ due to non-trivial __cinit__")             # <<<<<<<<<<<<<<
 * def __setstate_cython__(self, __pyx_state):
 *     raise TypeError("no default __reduce__ due to non-trivial __cinit__")
 */
  __pyx_t_1 = __Pyx_PyObject_Call(__pyx_builtin_TypeError, __pyx_tuple__33, NULL); if (unlikely(!__pyx_t_1)) __PYX_ERR(7, 2, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_1);
  __Pyx_Raise(__pyx_t_1, 0, 0, 0);
  __Pyx_DECREF(__pyx_t_1); __pyx_t_1 = 0;
  __PYX_ERR(7, 2, __pyx_L1_error)

  /* "(tree fragment)":1
 * def __reduce_cython__(self):             # <<<<<<<<<<<<<<
 *     raise TypeError("no default __reduce__ due to non-trivial __cinit__")
 * def __setstate_cython__(self, __pyx_state):
 */

  /* function exit code */
  __pyx_L1_error:;
  __Pyx_XDECREF(__pyx_t_1);
  __Pyx_AddTraceback("grpc._cython.cygrpc.SegregatedCall.__reduce_cython__", __pyx_clineno, __pyx_lineno, __pyx_filename);
  __pyx_r = NULL;
  __Pyx_XGIVEREF(__pyx_r);
  __Pyx_RefNannyFinishContext();
  return __pyx_r;
}

/* "(tree fragment)":3
 * def __reduce_cython__(self):
 *     raise TypeError("no default __reduce__ due to non-trivial __cinit__")
 * def __setstate_cython__(self, __pyx_state):             # <<<<<<<<<<<<<<
 *     raise TypeError("no default __reduce__ due to non-trivial __cinit__")
 */

/* Python wrapper */
static PyObject *__pyx_pw_4grpc_7_cython_6cygrpc_14SegregatedCall_11__setstate_cython__(PyObject *__pyx_v_self, PyObject *__pyx_v___pyx_state); /*proto*/
static PyObject *__pyx_pw_4grpc_7_cython_6cygrpc_14SegregatedCall_11__setstate_cython__(PyObject *__pyx_v_self, PyObject *__pyx_v___pyx_state) {
  PyObject *__pyx_r = 0;
  __Pyx_RefNannyDeclarations
  __Pyx_RefNannySetupContext("__setstate_cython__ (wrapper)", 0);
  __pyx_r = __pyx_pf_4grpc_7_cython_6cygrpc_14SegregatedCall_10__setstate_cython__(((struct __pyx_obj_4grpc_7_cython_6cygrpc_SegregatedCall *)__pyx_v_self), ((PyObject *)__pyx_v___pyx_state));

  /* function exit code */
  __Pyx_RefNannyFinishContext();
  return __pyx_r;
}

static PyObject *__pyx_pf_4grpc_7_cython_6cygrpc_14SegregatedCall_10__setstate_cython__(CYTHON_UNUSED struct __pyx_obj_4grpc_7_cython_6cygrpc_SegregatedCall *__pyx_v_self, CYTHON_UNUSED PyObject *__pyx_v___pyx_state) {
  PyObject *__pyx_r = NULL;
  __Pyx_RefNannyDeclarations
  PyObject *__pyx_t_1 = NULL;
  __Pyx_RefNannySetupContext("__setstate_cython__", 0);

  /* "(tree fragment)":4
 *     raise TypeError("no default __reduce__ due to non-trivial __cinit__")
 * def __setstate_cython__(self, __pyx_state):
 *     raise TypeError("no default __reduce__ due to non-trivial __cinit__")             # <<<<<<<<<<<<<<
 */
  __pyx_t_1 = __Pyx_PyObject_Call(__pyx_builtin_TypeError, __pyx_tuple__34, NULL); if (unlikely(!__pyx_t_1)) __PYX_ERR(7, 4, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_1);
  __Pyx_Raise(__pyx_t_1, 0, 0, 0);
  __Pyx_DECREF(__pyx_t_1); __pyx_t_1 = 0;
  __PYX_ERR(7, 4, __pyx_L1_error)

  /* "(tree fragment)":3
 * def __reduce_cython__(self):
 *     raise TypeError("no default __reduce__ due to non-trivial __cinit__")
 * def __setstate_cython__(self, __pyx_state):             # <<<<<<<<<<<<<<
 *     raise TypeError("no default __reduce__ due to non-trivial __cinit__")
 */

  /* function exit code */
  __pyx_L1_error:;
  __Pyx_XDECREF(__pyx_t_1);
  __Pyx_AddTraceback("grpc._cython.cygrpc.SegregatedCall.__setstate_cython__", __pyx_clineno, __pyx_lineno, __pyx_filename);
  __pyx_r = NULL;
  __Pyx_XGIVEREF(__pyx_r);
  __Pyx_RefNannyFinishContext();
  return __pyx_r;
}

/* "src/python/grpcio/grpc/_cython/_cygrpc/channel.pyx.pxi":322
 *   cdef grpc_completion_queue *c_completion_queue
 * 
 *   def on_success(started_tags):             # <<<<<<<<<<<<<<
 *     state.segregated_call_states.add(call_state)
 * 
 */

/* Python wrapper */
static PyObject *__pyx_pw_4grpc_7_cython_6cygrpc_16_segregated_call_1on_success(PyObject *__pyx_self, PyObject *__pyx_v_started_tags); /*proto*/
static PyMethodDef __pyx_mdef_4grpc_7_cython_6cygrpc_16_segregated_call_1on_success = {"on_success", (PyCFunction)__pyx_pw_4grpc_7_cython_6cygrpc_16_segregated_call_1on_success, METH_O, 0};
static PyObject *__pyx_pw_4grpc_7_cython_6cygrpc_16_segregated_call_1on_success(PyObject *__pyx_self, PyObject *__pyx_v_started_tags) {
  PyObject *__pyx_r = 0;
  __Pyx_RefNannyDeclarations
  __Pyx_RefNannySetupContext("on_success (wrapper)", 0);
  __pyx_r = __pyx_pf_4grpc_7_cython_6cygrpc_16_segregated_call_on_success(__pyx_self, ((PyObject *)__pyx_v_started_tags));

  /* function exit code */
  __Pyx_RefNannyFinishContext();
  return __pyx_r;
}

static PyObject *__pyx_pf_4grpc_7_cython_6cygrpc_16_segregated_call_on_success(PyObject *__pyx_self, CYTHON_UNUSED PyObject *__pyx_v_started_tags) {
  struct __pyx_obj_4grpc_7_cython_6cygrpc___pyx_scope_struct_2___pyx_f_4grpc_7_cython_6cygrpc__segregated_call *__pyx_cur_scope;
  struct __pyx_obj_4grpc_7_cython_6cygrpc___pyx_scope_struct_2___pyx_f_4grpc_7_cython_6cygrpc__segregated_call *__pyx_outer_scope;
  PyObject *__pyx_r = NULL;
  __Pyx_RefNannyDeclarations
  PyObject *__pyx_t_1 = NULL;
  int __pyx_t_2;
  __Pyx_RefNannySetupContext("on_success", 0);
  __pyx_outer_scope = (struct __pyx_obj_4grpc_7_cython_6cygrpc___pyx_scope_struct_2___pyx_f_4grpc_7_cython_6cygrpc__segregated_call *) __Pyx_CyFunction_GetClosure(__pyx_self);
  __pyx_cur_scope = __pyx_outer_scope;

  /* "src/python/grpcio/grpc/_cython/_cygrpc/channel.pyx.pxi":323
 * 
 *   def on_success(started_tags):
 *     state.segregated_call_states.add(call_state)             # <<<<<<<<<<<<<<
 * 
 *   with state.condition:
 */
  if (unlikely(!__pyx_cur_scope->__pyx_v_state)) { __Pyx_RaiseClosureNameError("state"); __PYX_ERR(8, 323, __pyx_L1_error) }
  if (unlikely(__pyx_cur_scope->__pyx_v_state->segregated_call_states == Py_None)) {
    PyErr_Format(PyExc_AttributeError, "'NoneType' object has no attribute '%.30s'", "add");
    __PYX_ERR(8, 323, __pyx_L1_error)
  }
  if (unlikely(!__pyx_cur_scope->__pyx_v_call_state)) { __Pyx_RaiseClosureNameError("call_state"); __PYX_ERR(8, 323, __pyx_L1_error) }
  __pyx_t_1 = ((PyObject *)__pyx_cur_scope->__pyx_v_call_state);
  __Pyx_INCREF(__pyx_t_1);
  __pyx_t_2 = PySet_Add(__pyx_cur_scope->__pyx_v_state->segregated_call_states, __pyx_t_1); if (unlikely(__pyx_t_2 == ((int)-1))) __PYX_ERR(8, 323, __pyx_L1_error)
  __Pyx_DECREF(__pyx_t_1); __pyx_t_1 = 0;

  /* "src/python/grpcio/grpc/_cython/_cygrpc/channel.pyx.pxi":322
 *   cdef grpc_completion_queue *c_completion_queue
 * 
 *   def on_success(started_tags):             # <<<<<<<<<<<<<<
 *     state.segregated_call_states.add(call_state)
 * 
 */

  /* function exit code */
  __pyx_r = Py_None; __Pyx_INCREF(Py_None);
  goto __pyx_L0;
  __pyx_L1_error:;
  __Pyx_XDECREF(__pyx_t_1);
  __Pyx_AddTraceback("grpc._cython.cygrpc._segregated_call.on_success", __pyx_clineno, __pyx_lineno, __pyx_filename);
  __pyx_r = NULL;
  __pyx_L0:;
  __Pyx_XGIVEREF(__pyx_r);
  __Pyx_RefNannyFinishContext();
  return __pyx_r;
}

/* "src/python/grpcio/grpc/_cython/_cygrpc/channel.pyx.pxi":314
 * 
 * 
 * cdef SegregatedCall _segregated_call(             # <<<<<<<<<<<<<<
 *     _ChannelState state, int flags, method, host, object deadline,
 *     object metadata, CallCredentials credentials, operationses_and_user_tags,
 */

static struct __pyx_obj_4grpc_7_cython_6cygrpc_SegregatedCall *__pyx_f_4grpc_7_cython_6cygrpc__segregated_call(struct __pyx_obj_4grpc_7_cython_6cygrpc__ChannelState *__pyx_v_state, int __pyx_v_flags, PyObject *__pyx_v_method, PyObject *__pyx_v_host, PyObject *__pyx_v_deadline, PyObject *__pyx_v_metadata, struct __pyx_obj_4grpc_7_cython_6cygrpc_CallCredentials *__pyx_v_credentials, PyObject *__pyx_v_operationses_and_user_tags, PyObject *__pyx_v_context) {
  struct __pyx_obj_4grpc_7_cython_6cygrpc___pyx_scope_struct_2___pyx_f_4grpc_7_cython_6cygrpc__segregated_call *__pyx_cur_scope;
  struct __pyx_obj_4grpc_7_cython_6cygrpc_SegregatedCall *__pyx_v_segregated_call = 0;
  grpc_completion_queue *__pyx_v_c_completion_queue;
  PyObject *__pyx_v_on_success = 0;
  struct __pyx_obj_4grpc_7_cython_6cygrpc_SegregatedCall *__pyx_r = NULL;
  __Pyx_RefNannyDeclarations
  PyObject *__pyx_t_1 = NULL;
  PyObject *__pyx_t_2 = NULL;
  PyObject *__pyx_t_3 = NULL;
  PyObject *__pyx_t_4 = NULL;
  PyObject *__pyx_t_5 = NULL;
  PyObject *__pyx_t_6 = NULL;
  PyObject *__pyx_t_7 = NULL;
  int __pyx_t_8;
  PyObject *__pyx_t_9 = NULL;
  PyObject *__pyx_t_10 = NULL;
  int __pyx_t_11;
  __Pyx_RefNannySetupContext("_segregated_call", 0);
  __pyx_cur_scope = (struct __pyx_obj_4grpc_7_cython_6cygrpc___pyx_scope_struct_2___pyx_f_4grpc_7_cython_6cygrpc__segregated_call *)__pyx_tp_new_4grpc_7_cython_6cygrpc___pyx_scope_struct_2___pyx_f_4grpc_7_cython_6cygrpc__segregated_call(__pyx_ptype_4grpc_7_cython_6cygrpc___pyx_scope_struct_2___pyx_f_4grpc_7_cython_6cygrpc__segregated_call, __pyx_empty_tuple, NULL);
  if (unlikely(!__pyx_cur_scope)) {
    __pyx_cur_scope = ((struct __pyx_obj_4grpc_7_cython_6cygrpc___pyx_scope_struct_2___pyx_f_4grpc_7_cython_6cygrpc__segregated_call *)Py_None);
    __Pyx_INCREF(Py_None);
    __PYX_ERR(8, 314, __pyx_L1_error)
  } else {
    __Pyx_GOTREF(__pyx_cur_scope);
  }
  __pyx_cur_scope->__pyx_v_state = __pyx_v_state;
  __Pyx_INCREF((PyObject *)__pyx_cur_scope->__pyx_v_state);
  __Pyx_GIVEREF((PyObject *)__pyx_cur_scope->__pyx_v_state);

  /* "src/python/grpcio/grpc/_cython/_cygrpc/channel.pyx.pxi":318
 *     object metadata, CallCredentials credentials, operationses_and_user_tags,
 *     object context):
 *   cdef _CallState call_state = _CallState()             # <<<<<<<<<<<<<<
 *   cdef SegregatedCall segregated_call
 *   cdef grpc_completion_queue *c_completion_queue
 */
  __pyx_t_1 = __Pyx_PyObject_CallNoArg(((PyObject *)__pyx_ptype_4grpc_7_cython_6cygrpc__CallState)); if (unlikely(!__pyx_t_1)) __PYX_ERR(8, 318, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_1);
  __Pyx_GIVEREF(__pyx_t_1);
  __pyx_cur_scope->__pyx_v_call_state = ((struct __pyx_obj_4grpc_7_cython_6cygrpc__CallState *)__pyx_t_1);
  __pyx_t_1 = 0;

  /* "src/python/grpcio/grpc/_cython/_cygrpc/channel.pyx.pxi":322
 *   cdef grpc_completion_queue *c_completion_queue
 * 
 *   def on_success(started_tags):             # <<<<<<<<<<<<<<
 *     state.segregated_call_states.add(call_state)
 * 
 */
  __pyx_t_1 = __Pyx_CyFunction_NewEx(&__pyx_mdef_4grpc_7_cython_6cygrpc_16_segregated_call_1on_success, 0, __pyx_n_s_segregated_call_locals_on_succe, ((PyObject*)__pyx_cur_scope), __pyx_n_s_grpc__cython_cygrpc, __pyx_d, ((PyObject *)__pyx_codeobj__36)); if (unlikely(!__pyx_t_1)) __PYX_ERR(8, 322, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_1);
  __pyx_v_on_success = __pyx_t_1;
  __pyx_t_1 = 0;

  /* "src/python/grpcio/grpc/_cython/_cygrpc/channel.pyx.pxi":325
 *     state.segregated_call_states.add(call_state)
 * 
 *   with state.condition:             # <<<<<<<<<<<<<<
 *     if state.open:
 *       c_completion_queue = (grpc_completion_queue_create_for_next(NULL))
 */
  /*with:*/ {
    __pyx_t_2 = __Pyx_PyObject_LookupSpecial(__pyx_cur_scope->__pyx_v_state->condition, __pyx_n_s_exit); if (unlikely(!__pyx_t_2)) __PYX_ERR(8, 325, __pyx_L1_error)
    __Pyx_GOTREF(__pyx_t_2);
    __pyx_t_3 = __Pyx_PyObject_LookupSpecial(__pyx_cur_scope->__pyx_v_state->condition, __pyx_n_s_enter); if (unlikely(!__pyx_t_3)) __PYX_ERR(8, 325, __pyx_L3_error)
    __Pyx_GOTREF(__pyx_t_3);
    __pyx_t_4 = NULL;
    if (CYTHON_UNPACK_METHODS && likely(PyMethod_Check(__pyx_t_3))) {
      __pyx_t_4 = PyMethod_GET_SELF(__pyx_t_3);
      if (likely(__pyx_t_4)) {
        PyObject* function = PyMethod_GET_FUNCTION(__pyx_t_3);
        __Pyx_INCREF(__pyx_t_4);
        __Pyx_INCREF(function);
        __Pyx_DECREF_SET(__pyx_t_3, function);
      }
    }
    if (__pyx_t_4) {
      __pyx_t_1 = __Pyx_PyObject_CallOneArg(__pyx_t_3, __pyx_t_4); if (unlikely(!__pyx_t_1)) __PYX_ERR(8, 325, __pyx_L3_error)
      __Pyx_DECREF(__pyx_t_4); __pyx_t_4 = 0;
    } else {
      __pyx_t_1 = __Pyx_PyObject_CallNoArg(__pyx_t_3); if (unlikely(!__pyx_t_1)) __PYX_ERR(8, 325, __pyx_L3_error)
    }
    __Pyx_GOTREF(__pyx_t_1);
    __Pyx_DECREF(__pyx_t_3); __pyx_t_3 = 0;
    __Pyx_DECREF(__pyx_t_1); __pyx_t_1 = 0;
    /*try:*/ {
      {
        __Pyx_PyThreadState_declare
        __Pyx_PyThreadState_assign
        __Pyx_ExceptionSave(&__pyx_t_5, &__pyx_t_6, &__pyx_t_7);
        __Pyx_XGOTREF(__pyx_t_5);
        __Pyx_XGOTREF(__pyx_t_6);
        __Pyx_XGOTREF(__pyx_t_7);
        /*try:*/ {

          /* "src/python/grpcio/grpc/_cython/_cygrpc/channel.pyx.pxi":326
 * 
 *   with state.condition:
 *     if state.open:             # <<<<<<<<<<<<<<
 *       c_completion_queue = (grpc_completion_queue_create_for_next(NULL))
 *     else:
 */
          __pyx_t_8 = __Pyx_PyObject_IsTrue(__pyx_cur_scope->__pyx_v_state->open); if (unlikely(__pyx_t_8 < 0)) __PYX_ERR(8, 326, __pyx_L7_error)
          if (likely(__pyx_t_8)) {

            /* "src/python/grpcio/grpc/_cython/_cygrpc/channel.pyx.pxi":327
 *   with state.condition:
 *     if state.open:
 *       c_completion_queue = (grpc_completion_queue_create_for_next(NULL))             # <<<<<<<<<<<<<<
 *     else:
 *       raise ValueError('Cannot invoke RPC on closed channel!')
 */
            __pyx_v_c_completion_queue = grpc_completion_queue_create_for_next(NULL);

            /* "src/python/grpcio/grpc/_cython/_cygrpc/channel.pyx.pxi":326
 * 
 *   with state.condition:
 *     if state.open:             # <<<<<<<<<<<<<<
 *       c_completion_queue = (grpc_completion_queue_create_for_next(NULL))
 *     else:
 */
            goto __pyx_L13;
          }

          /* "src/python/grpcio/grpc/_cython/_cygrpc/channel.pyx.pxi":329
 *       c_completion_queue = (grpc_completion_queue_create_for_next(NULL))
 *     else:
 *       raise ValueError('Cannot invoke RPC on closed channel!')             # <<<<<<<<<<<<<<
 * 
 *   try:
 */
          /*else*/ {
            __pyx_t_1 = __Pyx_PyObject_Call(__pyx_builtin_ValueError, __pyx_tuple__37, NULL); if (unlikely(!__pyx_t_1)) __PYX_ERR(8, 329, __pyx_L7_error)
            __Pyx_GOTREF(__pyx_t_1);
            __Pyx_Raise(__pyx_t_1, 0, 0, 0);
            __Pyx_DECREF(__pyx_t_1); __pyx_t_1 = 0;
            __PYX_ERR(8, 329, __pyx_L7_error)
          }
          __pyx_L13:;

          /* "src/python/grpcio/grpc/_cython/_cygrpc/channel.pyx.pxi":325
 *     state.segregated_call_states.add(call_state)
 * 
 *   with state.condition:             # <<<<<<<<<<<<<<
 *     if state.open:
 *       c_completion_queue = (grpc_completion_queue_create_for_next(NULL))
 */
        }
        __Pyx_XDECREF(__pyx_t_5); __pyx_t_5 = 0;
        __Pyx_XDECREF(__pyx_t_6); __pyx_t_6 = 0;
        __Pyx_XDECREF(__pyx_t_7); __pyx_t_7 = 0;
        goto __pyx_L12_try_end;
        __pyx_L7_error:;
        __Pyx_XDECREF(__pyx_t_4); __pyx_t_4 = 0;
        __Pyx_XDECREF(__pyx_t_3); __pyx_t_3 = 0;
        __Pyx_XDECREF(__pyx_t_1); __pyx_t_1 = 0;
        /*except:*/ {
          __Pyx_AddTraceback("grpc._cython.cygrpc._segregated_call", __pyx_clineno, __pyx_lineno, __pyx_filename);
          if (__Pyx_GetException(&__pyx_t_1, &__pyx_t_3, &__pyx_t_4) < 0) __PYX_ERR(8, 325, __pyx_L9_except_error)
          __Pyx_GOTREF(__pyx_t_1);
          __Pyx_GOTREF(__pyx_t_3);
          __Pyx_GOTREF(__pyx_t_4);
          __pyx_t_9 = PyTuple_Pack(3, __pyx_t_1, __pyx_t_3, __pyx_t_4); if (unlikely(!__pyx_t_9)) __PYX_ERR(8, 325, __pyx_L9_except_error)
          __Pyx_GOTREF(__pyx_t_9);
          __pyx_t_10 = __Pyx_PyObject_Call(__pyx_t_2, __pyx_t_9, NULL);
          __Pyx_DECREF(__pyx_t_2); __pyx_t_2 = 0;
          __Pyx_DECREF(__pyx_t_9); __pyx_t_9 = 0;
          if (unlikely(!__pyx_t_10)) __PYX_ERR(8, 325, __pyx_L9_except_error)
          __Pyx_GOTREF(__pyx_t_10);
          __pyx_t_8 = __Pyx_PyObject_IsTrue(__pyx_t_10);
          __Pyx_DECREF(__pyx_t_10); __pyx_t_10 = 0;
          if (__pyx_t_8 < 0) __PYX_ERR(8, 325, __pyx_L9_except_error)
          __pyx_t_11 = ((!(__pyx_t_8 != 0)) != 0);
          if (__pyx_t_11) {
            __Pyx_GIVEREF(__pyx_t_1);
            __Pyx_GIVEREF(__pyx_t_3);
            __Pyx_XGIVEREF(__pyx_t_4);
            __Pyx_ErrRestoreWithState(__pyx_t_1, __pyx_t_3, __pyx_t_4);
            __pyx_t_1 = 0; __pyx_t_3 = 0; __pyx_t_4 = 0; 
            __PYX_ERR(8, 325, __pyx_L9_except_error)
          }
          __Pyx_DECREF(__pyx_t_1); __pyx_t_1 = 0;
          __Pyx_DECREF(__pyx_t_3); __pyx_t_3 = 0;
          __Pyx_DECREF(__pyx_t_4); __pyx_t_4 = 0;
          goto __pyx_L8_exception_handled;
        }
        __pyx_L9_except_error:;
        __Pyx_XGIVEREF(__pyx_t_5);
        __Pyx_XGIVEREF(__pyx_t_6);
        __Pyx_XGIVEREF(__pyx_t_7);
        __Pyx_ExceptionReset(__pyx_t_5, __pyx_t_6, __pyx_t_7);
        goto __pyx_L1_error;
        __pyx_L8_exception_handled:;
        __Pyx_XGIVEREF(__pyx_t_5);
        __Pyx_XGIVEREF(__pyx_t_6);
        __Pyx_XGIVEREF(__pyx_t_7);
        __Pyx_ExceptionReset(__pyx_t_5, __pyx_t_6, __pyx_t_7);
        __pyx_L12_try_end:;
      }
    }
    /*finally:*/ {
      /*normal exit:*/{
        if (__pyx_t_2) {
          __pyx_t_7 = __Pyx_PyObject_Call(__pyx_t_2, __pyx_tuple__38, NULL);
          __Pyx_DECREF(__pyx_t_2); __pyx_t_2 = 0;
          if (unlikely(!__pyx_t_7)) __PYX_ERR(8, 325, __pyx_L1_error)
          __Pyx_GOTREF(__pyx_t_7);
          __Pyx_DECREF(__pyx_t_7); __pyx_t_7 = 0;
        }
        goto __pyx_L6;
      }
      __pyx_L6:;
    }
    goto __pyx_L17;
    __pyx_L3_error:;
    __Pyx_DECREF(__pyx_t_2); __pyx_t_2 = 0;
    goto __pyx_L1_error;
    __pyx_L17:;
  }

  /* "src/python/grpcio/grpc/_cython/_cygrpc/channel.pyx.pxi":331
 *       raise ValueError('Cannot invoke RPC on closed channel!')
 * 
 *   try:             # <<<<<<<<<<<<<<
 *     _call(
 *         state, call_state, c_completion_queue, on_success, flags, method, host,
 */
  {
    __Pyx_PyThreadState_declare
    __Pyx_PyThreadState_assign
    __Pyx_ExceptionSave(&__pyx_t_2, &__pyx_t_7, &__pyx_t_6);
    __Pyx_XGOTREF(__pyx_t_2);
    __Pyx_XGOTREF(__pyx_t_7);
    __Pyx_XGOTREF(__pyx_t_6);
    /*try:*/ {

      /* "src/python/grpcio/grpc/_cython/_cygrpc/channel.pyx.pxi":333
 *   try:
 *     _call(
 *         state, call_state, c_completion_queue, on_success, flags, method, host,             # <<<<<<<<<<<<<<
 *         deadline, credentials, operationses_and_user_tags, metadata,
 *         context)
 */
      __pyx_t_4 = ((PyObject *)__pyx_cur_scope->__pyx_v_state);
      __Pyx_INCREF(__pyx_t_4);
      __pyx_t_3 = ((PyObject *)__pyx_cur_scope->__pyx_v_call_state);
      __Pyx_INCREF(__pyx_t_3);

      /* "src/python/grpcio/grpc/_cython/_cygrpc/channel.pyx.pxi":332
 * 
 *   try:
 *     _call(             # <<<<<<<<<<<<<<
 *         state, call_state, c_completion_queue, on_success, flags, method, host,
 *         deadline, credentials, operationses_and_user_tags, metadata,
 */
      __pyx_f_4grpc_7_cython_6cygrpc__call(((struct __pyx_obj_4grpc_7_cython_6cygrpc__ChannelState *)__pyx_t_4), ((struct __pyx_obj_4grpc_7_cython_6cygrpc__CallState *)__pyx_t_3), __pyx_v_c_completion_queue, __pyx_v_on_success, __pyx_v_flags, __pyx_v_method, __pyx_v_host, __pyx_v_deadline, __pyx_v_credentials, __pyx_v_operationses_and_user_tags, __pyx_v_metadata, __pyx_v_context); if (unlikely(PyErr_Occurred())) __PYX_ERR(8, 332, __pyx_L18_error)
      __Pyx_DECREF(__pyx_t_4); __pyx_t_4 = 0;
      __Pyx_DECREF(__pyx_t_3); __pyx_t_3 = 0;

      /* "src/python/grpcio/grpc/_cython/_cygrpc/channel.pyx.pxi":331
 *       raise ValueError('Cannot invoke RPC on closed channel!')
 * 
 *   try:             # <<<<<<<<<<<<<<
 *     _call(
 *         state, call_state, c_completion_queue, on_success, flags, method, host,
 */
    }
    __Pyx_XDECREF(__pyx_t_2); __pyx_t_2 = 0;
    __Pyx_XDECREF(__pyx_t_7); __pyx_t_7 = 0;
    __Pyx_XDECREF(__pyx_t_6); __pyx_t_6 = 0;
    goto __pyx_L23_try_end;
    __pyx_L18_error:;
    __Pyx_XDECREF(__pyx_t_9); __pyx_t_9 = 0;
    __Pyx_XDECREF(__pyx_t_1); __pyx_t_1 = 0;
    __Pyx_XDECREF(__pyx_t_4); __pyx_t_4 = 0;
    __Pyx_XDECREF(__pyx_t_3); __pyx_t_3 = 0;

    /* "src/python/grpcio/grpc/_cython/_cygrpc/channel.pyx.pxi":336
 *         deadline, credentials, operationses_and_user_tags, metadata,
 *         context)
 *   except:             # <<<<<<<<<<<<<<
 *     _destroy_c_completion_queue(c_completion_queue)
 *     raise
 */
    /*except:*/ {
      __Pyx_AddTraceback("grpc._cython.cygrpc._segregated_call", __pyx_clineno, __pyx_lineno, __pyx_filename);
      if (__Pyx_GetException(&__pyx_t_3, &__pyx_t_4, &__pyx_t_1) < 0) __PYX_ERR(8, 336, __pyx_L20_except_error)
      __Pyx_GOTREF(__pyx_t_3);
      __Pyx_GOTREF(__pyx_t_4);
      __Pyx_GOTREF(__pyx_t_1);

      /* "src/python/grpcio/grpc/_cython/_cygrpc/channel.pyx.pxi":337
 *         context)
 *   except:
 *     _destroy_c_completion_queue(c_completion_queue)             # <<<<<<<<<<<<<<
 *     raise
 * 
 */
      __pyx_t_9 = __pyx_f_4grpc_7_cython_6cygrpc__destroy_c_completion_queue(__pyx_v_c_completion_queue); if (unlikely(!__pyx_t_9)) __PYX_ERR(8, 337, __pyx_L20_except_error)
      __Pyx_GOTREF(__pyx_t_9);
      __Pyx_DECREF(__pyx_t_9); __pyx_t_9 = 0;

      /* "src/python/grpcio/grpc/_cython/_cygrpc/channel.pyx.pxi":338
 *   except:
 *     _destroy_c_completion_queue(c_completion_queue)
 *     raise             # <<<<<<<<<<<<<<
 * 
 *   segregated_call = SegregatedCall(state, call_state)
 */
      __Pyx_GIVEREF(__pyx_t_3);
      __Pyx_GIVEREF(__pyx_t_4);
      __Pyx_XGIVEREF(__pyx_t_1);
      __Pyx_ErrRestoreWithState(__pyx_t_3, __pyx_t_4, __pyx_t_1);
      __pyx_t_3 = 0; __pyx_t_4 = 0; __pyx_t_1 = 0; 
      __PYX_ERR(8, 338, __pyx_L20_except_error)
    }
    __pyx_L20_except_error:;

    /* "src/python/grpcio/grpc/_cython/_cygrpc/channel.pyx.pxi":331
 *       raise ValueError('Cannot invoke RPC on closed channel!')
 * 
 *   try:             # <<<<<<<<<<<<<<
 *     _call(
 *         state, call_state, c_completion_queue, on_success, flags, method, host,
 */
    __Pyx_XGIVEREF(__pyx_t_2);
    __Pyx_XGIVEREF(__pyx_t_7);
    __Pyx_XGIVEREF(__pyx_t_6);
    __Pyx_ExceptionReset(__pyx_t_2, __pyx_t_7, __pyx_t_6);
    goto __pyx_L1_error;
    __pyx_L23_try_end:;
  }

  /* "src/python/grpcio/grpc/_cython/_cygrpc/channel.pyx.pxi":340
 *     raise
 * 
 *   segregated_call = SegregatedCall(state, call_state)             # <<<<<<<<<<<<<<
 *   segregated_call._c_completion_queue = c_completion_queue
 *   return segregated_call
 */
  __pyx_t_1 = PyTuple_New(2); if (unlikely(!__pyx_t_1)) __PYX_ERR(8, 340, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_1);
  __Pyx_INCREF(((PyObject *)__pyx_cur_scope->__pyx_v_state));
  __Pyx_GIVEREF(((PyObject *)__pyx_cur_scope->__pyx_v_state));
  PyTuple_SET_ITEM(__pyx_t_1, 0, ((PyObject *)__pyx_cur_scope->__pyx_v_state));
  __Pyx_INCREF(((PyObject *)__pyx_cur_scope->__pyx_v_call_state));
  __Pyx_GIVEREF(((PyObject *)__pyx_cur_scope->__pyx_v_call_state));
  PyTuple_SET_ITEM(__pyx_t_1, 1, ((PyObject *)__pyx_cur_scope->__pyx_v_call_state));
  __pyx_t_4 = __Pyx_PyObject_Call(((PyObject *)__pyx_ptype_4grpc_7_cython_6cygrpc_SegregatedCall), __pyx_t_1, NULL); if (unlikely(!__pyx_t_4)) __PYX_ERR(8, 340, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_4);
  __Pyx_DECREF(__pyx_t_1); __pyx_t_1 = 0;
  __pyx_v_segregated_call = ((struct __pyx_obj_4grpc_7_cython_6cygrpc_SegregatedCall *)__pyx_t_4);
  __pyx_t_4 = 0;

  /* "src/python/grpcio/grpc/_cython/_cygrpc/channel.pyx.pxi":341
 * 
 *   segregated_call = SegregatedCall(state, call_state)
 *   segregated_call._c_completion_queue = c_completion_queue             # <<<<<<<<<<<<<<
 *   return segregated_call
 * 
 */
  __pyx_v_segregated_call->_c_completion_queue = __pyx_v_c_completion_queue;

  /* "src/python/grpcio/grpc/_cython/_cygrpc/channel.pyx.pxi":342
 *   segregated_call = SegregatedCall(state, call_state)
 *   segregated_call._c_completion_queue = c_completion_queue
 *   return segregated_call             # <<<<<<<<<<<<<<
 * 
 * 
 */
  __Pyx_XDECREF(((PyObject *)__pyx_r));
  __Pyx_INCREF(((PyObject *)__pyx_v_segregated_call));
  __pyx_r = __pyx_v_segregated_call;
  goto __pyx_L0;

  /* "src/python/grpcio/grpc/_cython/_cygrpc/channel.pyx.pxi":314
 * 
 * 
 * cdef SegregatedCall _segregated_call(             # <<<<<<<<<<<<<<
 *     _ChannelState state, int flags, method, host, object deadline,
 *     object metadata, CallCredentials credentials, operationses_and_user_tags,
 */

  /* function exit code */
  __pyx_L1_error:;
  __Pyx_XDECREF(__pyx_t_1);
  __Pyx_XDECREF(__pyx_t_3);
  __Pyx_XDECREF(__pyx_t_4);
  __Pyx_XDECREF(__pyx_t_9);
  __Pyx_AddTraceback("grpc._cython.cygrpc._segregated_call", __pyx_clineno, __pyx_lineno, __pyx_filename);
  __pyx_r = 0;
  __pyx_L0:;
  __Pyx_XDECREF((PyObject *)__pyx_v_segregated_call);
  __Pyx_XDECREF(__pyx_v_on_success);
  __Pyx_DECREF(((PyObject *)__pyx_cur_scope));
  __Pyx_XGIVEREF((PyObject *)__pyx_r);
  __Pyx_RefNannyFinishContext();
  return __pyx_r;
}

/* "src/python/grpcio/grpc/_cython/_cygrpc/channel.pyx.pxi":345
 * 
 * 
 * cdef object _watch_connectivity_state(             # <<<<<<<<<<<<<<
 *     _ChannelState state, grpc_connectivity_state last_observed_state,
 *     object deadline):
 */

static PyObject *__pyx_f_4grpc_7_cython_6cygrpc__watch_connectivity_state(struct __pyx_obj_4grpc_7_cython_6cygrpc__ChannelState *__pyx_v_state, grpc_connectivity_state __pyx_v_last_observed_state, PyObject *__pyx_v_deadline) {
  struct __pyx_obj_4grpc_7_cython_6cygrpc__ConnectivityTag *__pyx_v_tag = 0;
  PyObject *__pyx_v_completed_tag = NULL;
  PyObject *__pyx_v_event = NULL;
  PyObject *__pyx_r = NULL;
  __Pyx_RefNannyDeclarations
  PyObject *__pyx_t_1 = NULL;
  PyObject *__pyx_t_2 = NULL;
  PyObject *__pyx_t_3 = NULL;
  PyObject *__pyx_t_4 = NULL;
  PyObject *__pyx_t_5 = NULL;
  PyObject *__pyx_t_6 = NULL;
  PyObject *__pyx_t_7 = NULL;
  int __pyx_t_8;
  gpr_timespec __pyx_t_9;
  int __pyx_t_10;
  PyObject *__pyx_t_11 = NULL;
  PyObject *__pyx_t_12 = NULL;
  int __pyx_t_13;
  PyObject *(*__pyx_t_14)(PyObject *);
  __Pyx_RefNannySetupContext("_watch_connectivity_state", 0);

  /* "src/python/grpcio/grpc/_cython/_cygrpc/channel.pyx.pxi":348
 *     _ChannelState state, grpc_connectivity_state last_observed_state,
 *     object deadline):
 *   cdef _ConnectivityTag tag = _ConnectivityTag(object())             # <<<<<<<<<<<<<<
 *   with state.condition:
 *     if state.open:
 */
  __pyx_t_1 = __Pyx_PyObject_CallNoArg(__pyx_builtin_object); if (unlikely(!__pyx_t_1)) __PYX_ERR(8, 348, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_1);
  __pyx_t_2 = __Pyx_PyObject_CallOneArg(((PyObject *)__pyx_ptype_4grpc_7_cython_6cygrpc__ConnectivityTag), __pyx_t_1); if (unlikely(!__pyx_t_2)) __PYX_ERR(8, 348, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_2);
  __Pyx_DECREF(__pyx_t_1); __pyx_t_1 = 0;
  __pyx_v_tag = ((struct __pyx_obj_4grpc_7_cython_6cygrpc__ConnectivityTag *)__pyx_t_2);
  __pyx_t_2 = 0;

  /* "src/python/grpcio/grpc/_cython/_cygrpc/channel.pyx.pxi":349
 *     object deadline):
 *   cdef _ConnectivityTag tag = _ConnectivityTag(object())
 *   with state.condition:             # <<<<<<<<<<<<<<
 *     if state.open:
 *       cpython.Py_INCREF(tag)
 */
  /*with:*/ {
    __pyx_t_3 = __Pyx_PyObject_LookupSpecial(__pyx_v_state->condition, __pyx_n_s_exit); if (unlikely(!__pyx_t_3)) __PYX_ERR(8, 349, __pyx_L1_error)
    __Pyx_GOTREF(__pyx_t_3);
    __pyx_t_1 = __Pyx_PyObject_LookupSpecial(__pyx_v_state->condition, __pyx_n_s_enter); if (unlikely(!__pyx_t_1)) __PYX_ERR(8, 349, __pyx_L3_error)
    __Pyx_GOTREF(__pyx_t_1);
    __pyx_t_4 = NULL;
    if (CYTHON_UNPACK_METHODS && likely(PyMethod_Check(__pyx_t_1))) {
      __pyx_t_4 = PyMethod_GET_SELF(__pyx_t_1);
      if (likely(__pyx_t_4)) {
        PyObject* function = PyMethod_GET_FUNCTION(__pyx_t_1);
        __Pyx_INCREF(__pyx_t_4);
        __Pyx_INCREF(function);
        __Pyx_DECREF_SET(__pyx_t_1, function);
      }
    }
    if (__pyx_t_4) {
      __pyx_t_2 = __Pyx_PyObject_CallOneArg(__pyx_t_1, __pyx_t_4); if (unlikely(!__pyx_t_2)) __PYX_ERR(8, 349, __pyx_L3_error)
      __Pyx_DECREF(__pyx_t_4); __pyx_t_4 = 0;
    } else {
      __pyx_t_2 = __Pyx_PyObject_CallNoArg(__pyx_t_1); if (unlikely(!__pyx_t_2)) __PYX_ERR(8, 349, __pyx_L3_error)
    }
    __Pyx_GOTREF(__pyx_t_2);
    __Pyx_DECREF(__pyx_t_1); __pyx_t_1 = 0;
    __Pyx_DECREF(__pyx_t_2); __pyx_t_2 = 0;
    /*try:*/ {
      {
        __Pyx_PyThreadState_declare
        __Pyx_PyThreadState_assign
        __Pyx_ExceptionSave(&__pyx_t_5, &__pyx_t_6, &__pyx_t_7);
        __Pyx_XGOTREF(__pyx_t_5);
        __Pyx_XGOTREF(__pyx_t_6);
        __Pyx_XGOTREF(__pyx_t_7);
        /*try:*/ {

          /* "src/python/grpcio/grpc/_cython/_cygrpc/channel.pyx.pxi":350
 *   cdef _ConnectivityTag tag = _ConnectivityTag(object())
 *   with state.condition:
 *     if state.open:             # <<<<<<<<<<<<<<
 *       cpython.Py_INCREF(tag)
 *       grpc_channel_watch_connectivity_state(
 */
          __pyx_t_8 = __Pyx_PyObject_IsTrue(__pyx_v_state->open); if (unlikely(__pyx_t_8 < 0)) __PYX_ERR(8, 350, __pyx_L7_error)
          if (likely(__pyx_t_8)) {

            /* "src/python/grpcio/grpc/_cython/_cygrpc/channel.pyx.pxi":351
 *   with state.condition:
 *     if state.open:
 *       cpython.Py_INCREF(tag)             # <<<<<<<<<<<<<<
 *       grpc_channel_watch_connectivity_state(
 *           state.c_channel, last_observed_state, _timespec_from_time(deadline),
 */
            Py_INCREF(((PyObject *)__pyx_v_tag));

            /* "src/python/grpcio/grpc/_cython/_cygrpc/channel.pyx.pxi":353
 *       cpython.Py_INCREF(tag)
 *       grpc_channel_watch_connectivity_state(
 *           state.c_channel, last_observed_state, _timespec_from_time(deadline),             # <<<<<<<<<<<<<<
 *           state.c_connectivity_completion_queue, <cpython.PyObject *>tag)
 *       state.connectivity_due.add(tag)
 */
            __pyx_t_9 = __pyx_f_4grpc_7_cython_6cygrpc__timespec_from_time(__pyx_v_deadline); if (unlikely(PyErr_Occurred())) __PYX_ERR(8, 353, __pyx_L7_error)

            /* "src/python/grpcio/grpc/_cython/_cygrpc/channel.pyx.pxi":352
 *     if state.open:
 *       cpython.Py_INCREF(tag)
 *       grpc_channel_watch_connectivity_state(             # <<<<<<<<<<<<<<
 *           state.c_channel, last_observed_state, _timespec_from_time(deadline),
 *           state.c_connectivity_completion_queue, <cpython.PyObject *>tag)
 */
            grpc_channel_watch_connectivity_state(__pyx_v_state->c_channel, __pyx_v_last_observed_state, __pyx_t_9, __pyx_v_state->c_connectivity_completion_queue, ((PyObject *)__pyx_v_tag));

            /* "src/python/grpcio/grpc/_cython/_cygrpc/channel.pyx.pxi":355
 *           state.c_channel, last_observed_state, _timespec_from_time(deadline),
 *           state.c_connectivity_completion_queue, <cpython.PyObject *>tag)
 *       state.connectivity_due.add(tag)             # <<<<<<<<<<<<<<
 *     else:
 *       raise ValueError('Cannot invoke RPC: %s' % state.closed_reason)
 */
            if (unlikely(__pyx_v_state->connectivity_due == Py_None)) {
              PyErr_Format(PyExc_AttributeError, "'NoneType' object has no attribute '%.30s'", "add");
              __PYX_ERR(8, 355, __pyx_L7_error)
            }
            __pyx_t_10 = PySet_Add(__pyx_v_state->connectivity_due, ((PyObject *)__pyx_v_tag)); if (unlikely(__pyx_t_10 == ((int)-1))) __PYX_ERR(8, 355, __pyx_L7_error)

            /* "src/python/grpcio/grpc/_cython/_cygrpc/channel.pyx.pxi":350
 *   cdef _ConnectivityTag tag = _ConnectivityTag(object())
 *   with state.condition:
 *     if state.open:             # <<<<<<<<<<<<<<
 *       cpython.Py_INCREF(tag)
 *       grpc_channel_watch_connectivity_state(
 */
            goto __pyx_L13;
          }

          /* "src/python/grpcio/grpc/_cython/_cygrpc/channel.pyx.pxi":357
 *       state.connectivity_due.add(tag)
 *     else:
 *       raise ValueError('Cannot invoke RPC: %s' % state.closed_reason)             # <<<<<<<<<<<<<<
 *   completed_tag, event = _latent_event(
 *       state.c_connectivity_completion_queue, None)
 */
          /*else*/ {
            __pyx_t_2 = __Pyx_PyString_Format(__pyx_kp_s_Cannot_invoke_RPC_s, __pyx_v_state->closed_reason); if (unlikely(!__pyx_t_2)) __PYX_ERR(8, 357, __pyx_L7_error)
            __Pyx_GOTREF(__pyx_t_2);
            __pyx_t_1 = __Pyx_PyObject_CallOneArg(__pyx_builtin_ValueError, __pyx_t_2); if (unlikely(!__pyx_t_1)) __PYX_ERR(8, 357, __pyx_L7_error)
            __Pyx_GOTREF(__pyx_t_1);
            __Pyx_DECREF(__pyx_t_2); __pyx_t_2 = 0;
            __Pyx_Raise(__pyx_t_1, 0, 0, 0);
            __Pyx_DECREF(__pyx_t_1); __pyx_t_1 = 0;
            __PYX_ERR(8, 357, __pyx_L7_error)
          }
          __pyx_L13:;

          /* "src/python/grpcio/grpc/_cython/_cygrpc/channel.pyx.pxi":349
 *     object deadline):
 *   cdef _ConnectivityTag tag = _ConnectivityTag(object())
 *   with state.condition:             # <<<<<<<<<<<<<<
 *     if state.open:
 *       cpython.Py_INCREF(tag)
 */
        }
        __Pyx_XDECREF(__pyx_t_5); __pyx_t_5 = 0;
        __Pyx_XDECREF(__pyx_t_6); __pyx_t_6 = 0;
        __Pyx_XDECREF(__pyx_t_7); __pyx_t_7 = 0;
        goto __pyx_L12_try_end;
        __pyx_L7_error:;
        __Pyx_XDECREF(__pyx_t_4); __pyx_t_4 = 0;
        __Pyx_XDECREF(__pyx_t_2); __pyx_t_2 = 0;
        __Pyx_XDECREF(__pyx_t_1); __pyx_t_1 = 0;
        /*except:*/ {
          __Pyx_AddTraceback("grpc._cython.cygrpc._watch_connectivity_state", __pyx_clineno, __pyx_lineno, __pyx_filename);
          if (__Pyx_GetException(&__pyx_t_1, &__pyx_t_2, &__pyx_t_4) < 0) __PYX_ERR(8, 349, __pyx_L9_except_error)
          __Pyx_GOTREF(__pyx_t_1);
          __Pyx_GOTREF(__pyx_t_2);
          __Pyx_GOTREF(__pyx_t_4);
          __pyx_t_11 = PyTuple_Pack(3, __pyx_t_1, __pyx_t_2, __pyx_t_4); if (unlikely(!__pyx_t_11)) __PYX_ERR(8, 349, __pyx_L9_except_error)
          __Pyx_GOTREF(__pyx_t_11);
          __pyx_t_12 = __Pyx_PyObject_Call(__pyx_t_3, __pyx_t_11, NULL);
          __Pyx_DECREF(__pyx_t_3); __pyx_t_3 = 0;
          __Pyx_DECREF(__pyx_t_11); __pyx_t_11 = 0;
          if (unlikely(!__pyx_t_12)) __PYX_ERR(8, 349, __pyx_L9_except_error)
          __Pyx_GOTREF(__pyx_t_12);
          __pyx_t_8 = __Pyx_PyObject_IsTrue(__pyx_t_12);
          __Pyx_DECREF(__pyx_t_12); __pyx_t_12 = 0;
          if (__pyx_t_8 < 0) __PYX_ERR(8, 349, __pyx_L9_except_error)
          __pyx_t_13 = ((!(__pyx_t_8 != 0)) != 0);
          if (__pyx_t_13) {
            __Pyx_GIVEREF(__pyx_t_1);
            __Pyx_GIVEREF(__pyx_t_2);
            __Pyx_XGIVEREF(__pyx_t_4);
            __Pyx_ErrRestoreWithState(__pyx_t_1, __pyx_t_2, __pyx_t_4);
            __pyx_t_1 = 0; __pyx_t_2 = 0; __pyx_t_4 = 0; 
            __PYX_ERR(8, 349, __pyx_L9_except_error)
          }
          __Pyx_DECREF(__pyx_t_1); __pyx_t_1 = 0;
          __Pyx_DECREF(__pyx_t_2); __pyx_t_2 = 0;
          __Pyx_DECREF(__pyx_t_4); __pyx_t_4 = 0;
          goto __pyx_L8_exception_handled;
        }
        __pyx_L9_except_error:;
        __Pyx_XGIVEREF(__pyx_t_5);
        __Pyx_XGIVEREF(__pyx_t_6);
        __Pyx_XGIVEREF(__pyx_t_7);
        __Pyx_ExceptionReset(__pyx_t_5, __pyx_t_6, __pyx_t_7);
        goto __pyx_L1_error;
        __pyx_L8_exception_handled:;
        __Pyx_XGIVEREF(__pyx_t_5);
        __Pyx_XGIVEREF(__pyx_t_6);
        __Pyx_XGIVEREF(__pyx_t_7);
        __Pyx_ExceptionReset(__pyx_t_5, __pyx_t_6, __pyx_t_7);
        __pyx_L12_try_end:;
      }
    }
    /*finally:*/ {
      /*normal exit:*/{
        if (__pyx_t_3) {
          __pyx_t_7 = __Pyx_PyObject_Call(__pyx_t_3, __pyx_tuple__39, NULL);
          __Pyx_DECREF(__pyx_t_3); __pyx_t_3 = 0;
          if (unlikely(!__pyx_t_7)) __PYX_ERR(8, 349, __pyx_L1_error)
          __Pyx_GOTREF(__pyx_t_7);
          __Pyx_DECREF(__pyx_t_7); __pyx_t_7 = 0;
        }
        goto __pyx_L6;
      }
      __pyx_L6:;
    }
    goto __pyx_L17;
    __pyx_L3_error:;
    __Pyx_DECREF(__pyx_t_3); __pyx_t_3 = 0;
    goto __pyx_L1_error;
    __pyx_L17:;
  }

  /* "src/python/grpcio/grpc/_cython/_cygrpc/channel.pyx.pxi":358
 *     else:
 *       raise ValueError('Cannot invoke RPC: %s' % state.closed_reason)
 *   completed_tag, event = _latent_event(             # <<<<<<<<<<<<<<
 *       state.c_connectivity_completion_queue, None)
 *   with state.condition:
 */
  __pyx_t_4 = __pyx_f_4grpc_7_cython_6cygrpc__latent_event(__pyx_v_state->c_connectivity_completion_queue, Py_None); if (unlikely(!__pyx_t_4)) __PYX_ERR(8, 358, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_4);
  if ((likely(PyTuple_CheckExact(__pyx_t_4))) || (PyList_CheckExact(__pyx_t_4))) {
    PyObject* sequence = __pyx_t_4;
    Py_ssize_t size = __Pyx_PySequence_SIZE(sequence);
    if (unlikely(size != 2)) {
      if (size > 2) __Pyx_RaiseTooManyValuesError(2);
      else if (size >= 0) __Pyx_RaiseNeedMoreValuesError(size);
      __PYX_ERR(8, 358, __pyx_L1_error)
    }
    #if CYTHON_ASSUME_SAFE_MACROS && !CYTHON_AVOID_BORROWED_REFS
    if (likely(PyTuple_CheckExact(sequence))) {
      __pyx_t_2 = PyTuple_GET_ITEM(sequence, 0); 
      __pyx_t_1 = PyTuple_GET_ITEM(sequence, 1); 
    } else {
      __pyx_t_2 = PyList_GET_ITEM(sequence, 0); 
      __pyx_t_1 = PyList_GET_ITEM(sequence, 1); 
    }
    __Pyx_INCREF(__pyx_t_2);
    __Pyx_INCREF(__pyx_t_1);
    #else
    __pyx_t_2 = PySequence_ITEM(sequence, 0); if (unlikely(!__pyx_t_2)) __PYX_ERR(8, 358, __pyx_L1_error)
    __Pyx_GOTREF(__pyx_t_2);
    __pyx_t_1 = PySequence_ITEM(sequence, 1); if (unlikely(!__pyx_t_1)) __PYX_ERR(8, 358, __pyx_L1_error)
    __Pyx_GOTREF(__pyx_t_1);
    #endif
    __Pyx_DECREF(__pyx_t_4); __pyx_t_4 = 0;
  } else {
    Py_ssize_t index = -1;
    __pyx_t_11 = PyObject_GetIter(__pyx_t_4); if (unlikely(!__pyx_t_11)) __PYX_ERR(8, 358, __pyx_L1_error)
    __Pyx_GOTREF(__pyx_t_11);
    __Pyx_DECREF(__pyx_t_4); __pyx_t_4 = 0;
    __pyx_t_14 = Py_TYPE(__pyx_t_11)->tp_iternext;
    index = 0; __pyx_t_2 = __pyx_t_14(__pyx_t_11); if (unlikely(!__pyx_t_2)) goto __pyx_L18_unpacking_failed;
    __Pyx_GOTREF(__pyx_t_2);
    index = 1; __pyx_t_1 = __pyx_t_14(__pyx_t_11); if (unlikely(!__pyx_t_1)) goto __pyx_L18_unpacking_failed;
    __Pyx_GOTREF(__pyx_t_1);
    if (__Pyx_IternextUnpackEndCheck(__pyx_t_14(__pyx_t_11), 2) < 0) __PYX_ERR(8, 358, __pyx_L1_error)
    __pyx_t_14 = NULL;
    __Pyx_DECREF(__pyx_t_11); __pyx_t_11 = 0;
    goto __pyx_L19_unpacking_done;
    __pyx_L18_unpacking_failed:;
    __Pyx_DECREF(__pyx_t_11); __pyx_t_11 = 0;
    __pyx_t_14 = NULL;
    if (__Pyx_IterFinish() == 0) __Pyx_RaiseNeedMoreValuesError(index);
    __PYX_ERR(8, 358, __pyx_L1_error)
    __pyx_L19_unpacking_done:;
  }
  __pyx_v_completed_tag = __pyx_t_2;
  __pyx_t_2 = 0;
  __pyx_v_event = __pyx_t_1;
  __pyx_t_1 = 0;

  /* "src/python/grpcio/grpc/_cython/_cygrpc/channel.pyx.pxi":360
 *   completed_tag, event = _latent_event(
 *       state.c_connectivity_completion_queue, None)
 *   with state.condition:             # <<<<<<<<<<<<<<
 *     state.connectivity_due.remove(completed_tag)
 *     state.condition.notify_all()
 */
  /*with:*/ {
    __pyx_t_3 = __Pyx_PyObject_LookupSpecial(__pyx_v_state->condition, __pyx_n_s_exit); if (unlikely(!__pyx_t_3)) __PYX_ERR(8, 360, __pyx_L1_error)
    __Pyx_GOTREF(__pyx_t_3);
    __pyx_t_1 = __Pyx_PyObject_LookupSpecial(__pyx_v_state->condition, __pyx_n_s_enter); if (unlikely(!__pyx_t_1)) __PYX_ERR(8, 360, __pyx_L20_error)
    __Pyx_GOTREF(__pyx_t_1);
    __pyx_t_2 = NULL;
    if (CYTHON_UNPACK_METHODS && likely(PyMethod_Check(__pyx_t_1))) {
      __pyx_t_2 = PyMethod_GET_SELF(__pyx_t_1);
      if (likely(__pyx_t_2)) {
        PyObject* function = PyMethod_GET_FUNCTION(__pyx_t_1);
        __Pyx_INCREF(__pyx_t_2);
        __Pyx_INCREF(function);
        __Pyx_DECREF_SET(__pyx_t_1, function);
      }
    }
    if (__pyx_t_2) {
      __pyx_t_4 = __Pyx_PyObject_CallOneArg(__pyx_t_1, __pyx_t_2); if (unlikely(!__pyx_t_4)) __PYX_ERR(8, 360, __pyx_L20_error)
      __Pyx_DECREF(__pyx_t_2); __pyx_t_2 = 0;
    } else {
      __pyx_t_4 = __Pyx_PyObject_CallNoArg(__pyx_t_1); if (unlikely(!__pyx_t_4)) __PYX_ERR(8, 360, __pyx_L20_error)
    }
    __Pyx_GOTREF(__pyx_t_4);
    __Pyx_DECREF(__pyx_t_1); __pyx_t_1 = 0;
    __Pyx_DECREF(__pyx_t_4); __pyx_t_4 = 0;
    /*try:*/ {
      {
        __Pyx_PyThreadState_declare
        __Pyx_PyThreadState_assign
        __Pyx_ExceptionSave(&__pyx_t_7, &__pyx_t_6, &__pyx_t_5);
        __Pyx_XGOTREF(__pyx_t_7);
        __Pyx_XGOTREF(__pyx_t_6);
        __Pyx_XGOTREF(__pyx_t_5);
        /*try:*/ {

          /* "src/python/grpcio/grpc/_cython/_cygrpc/channel.pyx.pxi":361
 *       state.c_connectivity_completion_queue, None)
 *   with state.condition:
 *     state.connectivity_due.remove(completed_tag)             # <<<<<<<<<<<<<<
 *     state.condition.notify_all()
 *   return event
 */
          if (unlikely(__pyx_v_state->connectivity_due == Py_None)) {
            PyErr_Format(PyExc_AttributeError, "'NoneType' object has no attribute '%.30s'", "remove");
            __PYX_ERR(8, 361, __pyx_L24_error)
          }
          __pyx_t_10 = __Pyx_PySet_Remove(__pyx_v_state->connectivity_due, __pyx_v_completed_tag); if (unlikely(__pyx_t_10 == ((int)-1))) __PYX_ERR(8, 361, __pyx_L24_error)

          /* "src/python/grpcio/grpc/_cython/_cygrpc/channel.pyx.pxi":362
 *   with state.condition:
 *     state.connectivity_due.remove(completed_tag)
 *     state.condition.notify_all()             # <<<<<<<<<<<<<<
 *   return event
 * 
 */
          __pyx_t_1 = __Pyx_PyObject_GetAttrStr(__pyx_v_state->condition, __pyx_n_s_notify_all); if (unlikely(!__pyx_t_1)) __PYX_ERR(8, 362, __pyx_L24_error)
          __Pyx_GOTREF(__pyx_t_1);
          __pyx_t_2 = NULL;
          if (CYTHON_UNPACK_METHODS && likely(PyMethod_Check(__pyx_t_1))) {
            __pyx_t_2 = PyMethod_GET_SELF(__pyx_t_1);
            if (likely(__pyx_t_2)) {
              PyObject* function = PyMethod_GET_FUNCTION(__pyx_t_1);
              __Pyx_INCREF(__pyx_t_2);
              __Pyx_INCREF(function);
              __Pyx_DECREF_SET(__pyx_t_1, function);
            }
          }
          if (__pyx_t_2) {
            __pyx_t_4 = __Pyx_PyObject_CallOneArg(__pyx_t_1, __pyx_t_2); if (unlikely(!__pyx_t_4)) __PYX_ERR(8, 362, __pyx_L24_error)
            __Pyx_DECREF(__pyx_t_2); __pyx_t_2 = 0;
          } else {
            __pyx_t_4 = __Pyx_PyObject_CallNoArg(__pyx_t_1); if (unlikely(!__pyx_t_4)) __PYX_ERR(8, 362, __pyx_L24_error)
          }
          __Pyx_GOTREF(__pyx_t_4);
          __Pyx_DECREF(__pyx_t_1); __pyx_t_1 = 0;
          __Pyx_DECREF(__pyx_t_4); __pyx_t_4 = 0;

          /* "src/python/grpcio/grpc/_cython/_cygrpc/channel.pyx.pxi":360
 *   completed_tag, event = _latent_event(
 *       state.c_connectivity_completion_queue, None)
 *   with state.condition:             # <<<<<<<<<<<<<<
 *     state.connectivity_due.remove(completed_tag)
 *     state.condition.notify_all()
 */
        }
        __Pyx_XDECREF(__pyx_t_7); __pyx_t_7 = 0;
        __Pyx_XDECREF(__pyx_t_6); __pyx_t_6 = 0;
        __Pyx_XDECREF(__pyx_t_5); __pyx_t_5 = 0;
        goto __pyx_L29_try_end;
        __pyx_L24_error:;
        __Pyx_XDECREF(__pyx_t_11); __pyx_t_11 = 0;
        __Pyx_XDECREF(__pyx_t_2); __pyx_t_2 = 0;
        __Pyx_XDECREF(__pyx_t_1); __pyx_t_1 = 0;
        __Pyx_XDECREF(__pyx_t_4); __pyx_t_4 = 0;
        /*except:*/ {
          __Pyx_AddTraceback("grpc._cython.cygrpc._watch_connectivity_state", __pyx_clineno, __pyx_lineno, __pyx_filename);
          if (__Pyx_GetException(&__pyx_t_4, &__pyx_t_1, &__pyx_t_2) < 0) __PYX_ERR(8, 360, __pyx_L26_except_error)
          __Pyx_GOTREF(__pyx_t_4);
          __Pyx_GOTREF(__pyx_t_1);
          __Pyx_GOTREF(__pyx_t_2);
          __pyx_t_11 = PyTuple_Pack(3, __pyx_t_4, __pyx_t_1, __pyx_t_2); if (unlikely(!__pyx_t_11)) __PYX_ERR(8, 360, __pyx_L26_except_error)
          __Pyx_GOTREF(__pyx_t_11);
          __pyx_t_12 = __Pyx_PyObject_Call(__pyx_t_3, __pyx_t_11, NULL);
          __Pyx_DECREF(__pyx_t_3); __pyx_t_3 = 0;
          __Pyx_DECREF(__pyx_t_11); __pyx_t_11 = 0;
          if (unlikely(!__pyx_t_12)) __PYX_ERR(8, 360, __pyx_L26_except_error)
          __Pyx_GOTREF(__pyx_t_12);
          __pyx_t_13 = __Pyx_PyObject_IsTrue(__pyx_t_12);
          __Pyx_DECREF(__pyx_t_12); __pyx_t_12 = 0;
          if (__pyx_t_13 < 0) __PYX_ERR(8, 360, __pyx_L26_except_error)
          __pyx_t_8 = ((!(__pyx_t_13 != 0)) != 0);
          if (__pyx_t_8) {
            __Pyx_GIVEREF(__pyx_t_4);
            __Pyx_GIVEREF(__pyx_t_1);
            __Pyx_XGIVEREF(__pyx_t_2);
            __Pyx_ErrRestoreWithState(__pyx_t_4, __pyx_t_1, __pyx_t_2);
            __pyx_t_4 = 0; __pyx_t_1 = 0; __pyx_t_2 = 0; 
            __PYX_ERR(8, 360, __pyx_L26_except_error)
          }
          __Pyx_DECREF(__pyx_t_4); __pyx_t_4 = 0;
          __Pyx_DECREF(__pyx_t_1); __pyx_t_1 = 0;
          __Pyx_DECREF(__pyx_t_2); __pyx_t_2 = 0;
          goto __pyx_L25_exception_handled;
        }
        __pyx_L26_except_error:;
        __Pyx_XGIVEREF(__pyx_t_7);
        __Pyx_XGIVEREF(__pyx_t_6);
        __Pyx_XGIVEREF(__pyx_t_5);
        __Pyx_ExceptionReset(__pyx_t_7, __pyx_t_6, __pyx_t_5);
        goto __pyx_L1_error;
        __pyx_L25_exception_handled:;
        __Pyx_XGIVEREF(__pyx_t_7);
        __Pyx_XGIVEREF(__pyx_t_6);
        __Pyx_XGIVEREF(__pyx_t_5);
        __Pyx_ExceptionReset(__pyx_t_7, __pyx_t_6, __pyx_t_5);
        __pyx_L29_try_end:;
      }
    }
    /*finally:*/ {
      /*normal exit:*/{
        if (__pyx_t_3) {
          __pyx_t_5 = __Pyx_PyObject_Call(__pyx_t_3, __pyx_tuple__40, NULL);
          __Pyx_DECREF(__pyx_t_3); __pyx_t_3 = 0;
          if (unlikely(!__pyx_t_5)) __PYX_ERR(8, 360, __pyx_L1_error)
          __Pyx_GOTREF(__pyx_t_5);
          __Pyx_DECREF(__pyx_t_5); __pyx_t_5 = 0;
        }
        goto __pyx_L23;
      }
      __pyx_L23:;
    }
    goto __pyx_L33;
    __pyx_L20_error:;
    __Pyx_DECREF(__pyx_t_3); __pyx_t_3 = 0;
    goto __pyx_L1_error;
    __pyx_L33:;
  }

  /* "src/python/grpcio/grpc/_cython/_cygrpc/channel.pyx.pxi":363
 *     state.connectivity_due.remove(completed_tag)
 *     state.condition.notify_all()
 *   return event             # <<<<<<<<<<<<<<
 * 
 * 
 */
  __Pyx_XDECREF(__pyx_r);
  __Pyx_INCREF(__pyx_v_event);
  __pyx_r = __pyx_v_event;
  goto __pyx_L0;

  /* "src/python/grpcio/grpc/_cython/_cygrpc/channel.pyx.pxi":345
 * 
 * 
 * cdef object _watch_connectivity_state(             # <<<<<<<<<<<<<<
 *     _ChannelState state, grpc_connectivity_state last_observed_state,
 *     object deadline):
 */

  /* function exit code */
  __pyx_L1_error:;
  __Pyx_XDECREF(__pyx_t_1);
  __Pyx_XDECREF(__pyx_t_2);
  __Pyx_XDECREF(__pyx_t_4);
  __Pyx_XDECREF(__pyx_t_11);
  __Pyx_AddTraceback("grpc._cython.cygrpc._watch_connectivity_state", __pyx_clineno, __pyx_lineno, __pyx_filename);
  __pyx_r = 0;
  __pyx_L0:;
  __Pyx_XDECREF((PyObject *)__pyx_v_tag);
  __Pyx_XDECREF(__pyx_v_completed_tag);
  __Pyx_XDECREF(__pyx_v_event);
  __Pyx_XGIVEREF(__pyx_r);
  __Pyx_RefNannyFinishContext();
  return __pyx_r;
}

/* "src/python/grpcio/grpc/_cython/_cygrpc/channel.pyx.pxi":366
 * 
 * 
 * cdef _close(Channel channel, grpc_status_code code, object details,             # <<<<<<<<<<<<<<
 *     drain_calls):
 *   cdef _ChannelState state = channel._state
 */

static PyObject *__pyx_f_4grpc_7_cython_6cygrpc__close(struct __pyx_obj_4grpc_7_cython_6cygrpc_Channel *__pyx_v_channel, grpc_status_code __pyx_v_code, PyObject *__pyx_v_details, PyObject *__pyx_v_drain_calls) {
  struct __pyx_obj_4grpc_7_cython_6cygrpc__ChannelState *__pyx_v_state = 0;
  struct __pyx_obj_4grpc_7_cython_6cygrpc__CallState *__pyx_v_call_state = 0;
  PyObject *__pyx_v_encoded_details = NULL;
  PyObject *__pyx_v_event = NULL;
  PyObject *__pyx_r = NULL;
  __Pyx_RefNannyDeclarations
  PyObject *__pyx_t_1 = NULL;
  PyObject *__pyx_t_2 = NULL;
  PyObject *__pyx_t_3 = NULL;
  PyObject *__pyx_t_4 = NULL;
  PyObject *__pyx_t_5 = NULL;
  PyObject *__pyx_t_6 = NULL;
  PyObject *__pyx_t_7 = NULL;
  int __pyx_t_8;
  Py_ssize_t __pyx_t_9;
  Py_ssize_t __pyx_t_10;
  int __pyx_t_11;
  int __pyx_t_12;
  char const *__pyx_t_13;
  int __pyx_t_14;
  PyObject *__pyx_t_15 = NULL;
  PyObject *__pyx_t_16 = NULL;
  __Pyx_RefNannySetupContext("_close", 0);

  /* "src/python/grpcio/grpc/_cython/_cygrpc/channel.pyx.pxi":368
 * cdef _close(Channel channel, grpc_status_code code, object details,
 *     drain_calls):
 *   cdef _ChannelState state = channel._state             # <<<<<<<<<<<<<<
 *   cdef _CallState call_state
 *   encoded_details = _encode(details)
 */
  __pyx_t_1 = ((PyObject *)__pyx_v_channel->_state);
  __Pyx_INCREF(__pyx_t_1);
  __pyx_v_state = ((struct __pyx_obj_4grpc_7_cython_6cygrpc__ChannelState *)__pyx_t_1);
  __pyx_t_1 = 0;

  /* "src/python/grpcio/grpc/_cython/_cygrpc/channel.pyx.pxi":370
 *   cdef _ChannelState state = channel._state
 *   cdef _CallState call_state
 *   encoded_details = _encode(details)             # <<<<<<<<<<<<<<
 *   with state.condition:
 *     if state.open:
 */
  __pyx_t_1 = __pyx_f_4grpc_7_cython_6cygrpc__encode(__pyx_v_details); if (unlikely(!__pyx_t_1)) __PYX_ERR(8, 370, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_1);
  __pyx_v_encoded_details = ((PyObject*)__pyx_t_1);
  __pyx_t_1 = 0;

  /* "src/python/grpcio/grpc/_cython/_cygrpc/channel.pyx.pxi":371
 *   cdef _CallState call_state
 *   encoded_details = _encode(details)
 *   with state.condition:             # <<<<<<<<<<<<<<
 *     if state.open:
 *       state.open = False
 */
  /*with:*/ {
    __pyx_t_2 = __Pyx_PyObject_LookupSpecial(__pyx_v_state->condition, __pyx_n_s_exit); if (unlikely(!__pyx_t_2)) __PYX_ERR(8, 371, __pyx_L1_error)
    __Pyx_GOTREF(__pyx_t_2);
    __pyx_t_3 = __Pyx_PyObject_LookupSpecial(__pyx_v_state->condition, __pyx_n_s_enter); if (unlikely(!__pyx_t_3)) __PYX_ERR(8, 371, __pyx_L3_error)
    __Pyx_GOTREF(__pyx_t_3);
    __pyx_t_4 = NULL;
    if (CYTHON_UNPACK_METHODS && likely(PyMethod_Check(__pyx_t_3))) {
      __pyx_t_4 = PyMethod_GET_SELF(__pyx_t_3);
      if (likely(__pyx_t_4)) {
        PyObject* function = PyMethod_GET_FUNCTION(__pyx_t_3);
        __Pyx_INCREF(__pyx_t_4);
        __Pyx_INCREF(function);
        __Pyx_DECREF_SET(__pyx_t_3, function);
      }
    }
    if (__pyx_t_4) {
      __pyx_t_1 = __Pyx_PyObject_CallOneArg(__pyx_t_3, __pyx_t_4); if (unlikely(!__pyx_t_1)) __PYX_ERR(8, 371, __pyx_L3_error)
      __Pyx_DECREF(__pyx_t_4); __pyx_t_4 = 0;
    } else {
      __pyx_t_1 = __Pyx_PyObject_CallNoArg(__pyx_t_3); if (unlikely(!__pyx_t_1)) __PYX_ERR(8, 371, __pyx_L3_error)
    }
    __Pyx_GOTREF(__pyx_t_1);
    __Pyx_DECREF(__pyx_t_3); __pyx_t_3 = 0;
    __Pyx_DECREF(__pyx_t_1); __pyx_t_1 = 0;
    /*try:*/ {
      {
        __Pyx_PyThreadState_declare
        __Pyx_PyThreadState_assign
        __Pyx_ExceptionSave(&__pyx_t_5, &__pyx_t_6, &__pyx_t_7);
        __Pyx_XGOTREF(__pyx_t_5);
        __Pyx_XGOTREF(__pyx_t_6);
        __Pyx_XGOTREF(__pyx_t_7);
        /*try:*/ {

          /* "src/python/grpcio/grpc/_cython/_cygrpc/channel.pyx.pxi":372
 *   encoded_details = _encode(details)
 *   with state.condition:
 *     if state.open:             # <<<<<<<<<<<<<<
 *       state.open = False
 *       state.closed_reason = details
 */
          __pyx_t_8 = __Pyx_PyObject_IsTrue(__pyx_v_state->open); if (unlikely(__pyx_t_8 < 0)) __PYX_ERR(8, 372, __pyx_L7_error)
          if (__pyx_t_8) {

            /* "src/python/grpcio/grpc/_cython/_cygrpc/channel.pyx.pxi":373
 *   with state.condition:
 *     if state.open:
 *       state.open = False             # <<<<<<<<<<<<<<
 *       state.closed_reason = details
 *       for call_state in set(state.integrated_call_states.values()):
 */
            __Pyx_INCREF(Py_False);
            __Pyx_GIVEREF(Py_False);
            __Pyx_GOTREF(__pyx_v_state->open);
            __Pyx_DECREF(__pyx_v_state->open);
            __pyx_v_state->open = Py_False;

            /* "src/python/grpcio/grpc/_cython/_cygrpc/channel.pyx.pxi":374
 *     if state.open:
 *       state.open = False
 *       state.closed_reason = details             # <<<<<<<<<<<<<<
 *       for call_state in set(state.integrated_call_states.values()):
 *         grpc_call_cancel_with_status(
 */
            __Pyx_INCREF(__pyx_v_details);
            __Pyx_GIVEREF(__pyx_v_details);
            __Pyx_GOTREF(__pyx_v_state->closed_reason);
            __Pyx_DECREF(__pyx_v_state->closed_reason);
            __pyx_v_state->closed_reason = __pyx_v_details;

            /* "src/python/grpcio/grpc/_cython/_cygrpc/channel.pyx.pxi":375
 *       state.open = False
 *       state.closed_reason = details
 *       for call_state in set(state.integrated_call_states.values()):             # <<<<<<<<<<<<<<
 *         grpc_call_cancel_with_status(
 *             call_state.c_call, code, encoded_details, NULL)
 */
            __pyx_t_9 = 0;
            if (unlikely(__pyx_v_state->integrated_call_states == Py_None)) {
              PyErr_Format(PyExc_AttributeError, "'NoneType' object has no attribute '%.30s'", "values");
              __PYX_ERR(8, 375, __pyx_L7_error)
            }
            __pyx_t_3 = __Pyx_PyDict_Values(__pyx_v_state->integrated_call_states); if (unlikely(!__pyx_t_3)) __PYX_ERR(8, 375, __pyx_L7_error)
            __Pyx_GOTREF(__pyx_t_3);
            __pyx_t_4 = PySet_New(__pyx_t_3); if (unlikely(!__pyx_t_4)) __PYX_ERR(8, 375, __pyx_L7_error)
            __Pyx_GOTREF(__pyx_t_4);
            __Pyx_DECREF(__pyx_t_3); __pyx_t_3 = 0;
            __pyx_t_3 = __Pyx_set_iterator(__pyx_t_4, 1, (&__pyx_t_10), (&__pyx_t_11)); if (unlikely(!__pyx_t_3)) __PYX_ERR(8, 375, __pyx_L7_error)
            __Pyx_GOTREF(__pyx_t_3);
            __Pyx_DECREF(__pyx_t_4); __pyx_t_4 = 0;
            __Pyx_XDECREF(__pyx_t_1);
            __pyx_t_1 = __pyx_t_3;
            __pyx_t_3 = 0;
            while (1) {
              __pyx_t_12 = __Pyx_set_iter_next(__pyx_t_1, __pyx_t_10, &__pyx_t_9, &__pyx_t_3, __pyx_t_11);
              if (unlikely(__pyx_t_12 == 0)) break;
              if (unlikely(__pyx_t_12 == -1)) __PYX_ERR(8, 375, __pyx_L7_error)
              __Pyx_GOTREF(__pyx_t_3);
              if (!(likely(((__pyx_t_3) == Py_None) || likely(__Pyx_TypeTest(__pyx_t_3, __pyx_ptype_4grpc_7_cython_6cygrpc__CallState))))) __PYX_ERR(8, 375, __pyx_L7_error)
              __Pyx_XDECREF_SET(__pyx_v_call_state, ((struct __pyx_obj_4grpc_7_cython_6cygrpc__CallState *)__pyx_t_3));
              __pyx_t_3 = 0;

              /* "src/python/grpcio/grpc/_cython/_cygrpc/channel.pyx.pxi":377
 *       for call_state in set(state.integrated_call_states.values()):
 *         grpc_call_cancel_with_status(
 *             call_state.c_call, code, encoded_details, NULL)             # <<<<<<<<<<<<<<
 *       for call_state in state.segregated_call_states:
 *         grpc_call_cancel_with_status(
 */
              if (unlikely(__pyx_v_encoded_details == Py_None)) {
                PyErr_SetString(PyExc_TypeError, "expected bytes, NoneType found");
                __PYX_ERR(8, 377, __pyx_L7_error)
              }
              __pyx_t_13 = __Pyx_PyBytes_AsString(__pyx_v_encoded_details); if (unlikely((!__pyx_t_13) && PyErr_Occurred())) __PYX_ERR(8, 377, __pyx_L7_error)

              /* "src/python/grpcio/grpc/_cython/_cygrpc/channel.pyx.pxi":376
 *       state.closed_reason = details
 *       for call_state in set(state.integrated_call_states.values()):
 *         grpc_call_cancel_with_status(             # <<<<<<<<<<<<<<
 *             call_state.c_call, code, encoded_details, NULL)
 *       for call_state in state.segregated_call_states:
 */
              (void)(grpc_call_cancel_with_status(__pyx_v_call_state->c_call, __pyx_v_code, __pyx_t_13, NULL));
            }
            __Pyx_DECREF(__pyx_t_1); __pyx_t_1 = 0;

            /* "src/python/grpcio/grpc/_cython/_cygrpc/channel.pyx.pxi":378
 *         grpc_call_cancel_with_status(
 *             call_state.c_call, code, encoded_details, NULL)
 *       for call_state in state.segregated_call_states:             # <<<<<<<<<<<<<<
 *         grpc_call_cancel_with_status(
 *             call_state.c_call, code, encoded_details, NULL)
 */
            __pyx_t_10 = 0;
            __pyx_t_3 = __Pyx_set_iterator(__pyx_v_state->segregated_call_states, 1, (&__pyx_t_9), (&__pyx_t_11)); if (unlikely(!__pyx_t_3)) __PYX_ERR(8, 378, __pyx_L7_error)
            __Pyx_GOTREF(__pyx_t_3);
            __Pyx_XDECREF(__pyx_t_1);
            __pyx_t_1 = __pyx_t_3;
            __pyx_t_3 = 0;
            while (1) {
              __pyx_t_12 = __Pyx_set_iter_next(__pyx_t_1, __pyx_t_9, &__pyx_t_10, &__pyx_t_3, __pyx_t_11);
              if (unlikely(__pyx_t_12 == 0)) break;
              if (unlikely(__pyx_t_12 == -1)) __PYX_ERR(8, 378, __pyx_L7_error)
              __Pyx_GOTREF(__pyx_t_3);
              if (!(likely(((__pyx_t_3) == Py_None) || likely(__Pyx_TypeTest(__pyx_t_3, __pyx_ptype_4grpc_7_cython_6cygrpc__CallState))))) __PYX_ERR(8, 378, __pyx_L7_error)
              __Pyx_XDECREF_SET(__pyx_v_call_state, ((struct __pyx_obj_4grpc_7_cython_6cygrpc__CallState *)__pyx_t_3));
              __pyx_t_3 = 0;

              /* "src/python/grpcio/grpc/_cython/_cygrpc/channel.pyx.pxi":380
 *       for call_state in state.segregated_call_states:
 *         grpc_call_cancel_with_status(
 *             call_state.c_call, code, encoded_details, NULL)             # <<<<<<<<<<<<<<
 *       # TODO(https://github.com/grpc/grpc/issues/3064): Cancel connectivity
 *       # watching.
 */
              if (unlikely(__pyx_v_encoded_details == Py_None)) {
                PyErr_SetString(PyExc_TypeError, "expected bytes, NoneType found");
                __PYX_ERR(8, 380, __pyx_L7_error)
              }
              __pyx_t_13 = __Pyx_PyBytes_AsString(__pyx_v_encoded_details); if (unlikely((!__pyx_t_13) && PyErr_Occurred())) __PYX_ERR(8, 380, __pyx_L7_error)

              /* "src/python/grpcio/grpc/_cython/_cygrpc/channel.pyx.pxi":379
 *             call_state.c_call, code, encoded_details, NULL)
 *       for call_state in state.segregated_call_states:
 *         grpc_call_cancel_with_status(             # <<<<<<<<<<<<<<
 *             call_state.c_call, code, encoded_details, NULL)
 *       # TODO(https://github.com/grpc/grpc/issues/3064): Cancel connectivity
 */
              (void)(grpc_call_cancel_with_status(__pyx_v_call_state->c_call, __pyx_v_code, __pyx_t_13, NULL));
            }
            __Pyx_DECREF(__pyx_t_1); __pyx_t_1 = 0;

            /* "src/python/grpcio/grpc/_cython/_cygrpc/channel.pyx.pxi":384
 *       # watching.
 * 
 *       if drain_calls:             # <<<<<<<<<<<<<<
 *         while not _calls_drained(state):
 *           event = channel.next_call_event()
 */
            __pyx_t_8 = __Pyx_PyObject_IsTrue(__pyx_v_drain_calls); if (unlikely(__pyx_t_8 < 0)) __PYX_ERR(8, 384, __pyx_L7_error)
            if (__pyx_t_8) {

              /* "src/python/grpcio/grpc/_cython/_cygrpc/channel.pyx.pxi":385
 * 
 *       if drain_calls:
 *         while not _calls_drained(state):             # <<<<<<<<<<<<<<
 *           event = channel.next_call_event()
 *           if event.completion_type == CompletionType.queue_timeout:
 */
              while (1) {
                __pyx_t_1 = __pyx_f_4grpc_7_cython_6cygrpc__calls_drained(__pyx_v_state); if (unlikely(!__pyx_t_1)) __PYX_ERR(8, 385, __pyx_L7_error)
                __Pyx_GOTREF(__pyx_t_1);
                __pyx_t_8 = __Pyx_PyObject_IsTrue(__pyx_t_1); if (unlikely(__pyx_t_8 < 0)) __PYX_ERR(8, 385, __pyx_L7_error)
                __Pyx_DECREF(__pyx_t_1); __pyx_t_1 = 0;
                __pyx_t_14 = ((!__pyx_t_8) != 0);
                if (!__pyx_t_14) break;

                /* "src/python/grpcio/grpc/_cython/_cygrpc/channel.pyx.pxi":386
 *       if drain_calls:
 *         while not _calls_drained(state):
 *           event = channel.next_call_event()             # <<<<<<<<<<<<<<
 *           if event.completion_type == CompletionType.queue_timeout:
 *               continue
 */
                __pyx_t_3 = __Pyx_PyObject_GetAttrStr(((PyObject *)__pyx_v_channel), __pyx_n_s_next_call_event); if (unlikely(!__pyx_t_3)) __PYX_ERR(8, 386, __pyx_L7_error)
                __Pyx_GOTREF(__pyx_t_3);
                __pyx_t_4 = NULL;
                if (CYTHON_UNPACK_METHODS && likely(PyMethod_Check(__pyx_t_3))) {
                  __pyx_t_4 = PyMethod_GET_SELF(__pyx_t_3);
                  if (likely(__pyx_t_4)) {
                    PyObject* function = PyMethod_GET_FUNCTION(__pyx_t_3);
                    __Pyx_INCREF(__pyx_t_4);
                    __Pyx_INCREF(function);
                    __Pyx_DECREF_SET(__pyx_t_3, function);
                  }
                }
                if (__pyx_t_4) {
                  __pyx_t_1 = __Pyx_PyObject_CallOneArg(__pyx_t_3, __pyx_t_4); if (unlikely(!__pyx_t_1)) __PYX_ERR(8, 386, __pyx_L7_error)
                  __Pyx_DECREF(__pyx_t_4); __pyx_t_4 = 0;
                } else {
                  __pyx_t_1 = __Pyx_PyObject_CallNoArg(__pyx_t_3); if (unlikely(!__pyx_t_1)) __PYX_ERR(8, 386, __pyx_L7_error)
                }
                __Pyx_GOTREF(__pyx_t_1);
                __Pyx_DECREF(__pyx_t_3); __pyx_t_3 = 0;
                __Pyx_XDECREF_SET(__pyx_v_event, __pyx_t_1);
                __pyx_t_1 = 0;

                /* "src/python/grpcio/grpc/_cython/_cygrpc/channel.pyx.pxi":387
 *         while not _calls_drained(state):
 *           event = channel.next_call_event()
 *           if event.completion_type == CompletionType.queue_timeout:             # <<<<<<<<<<<<<<
 *               continue
 *           event.tag(event)
 */
                __pyx_t_1 = __Pyx_PyObject_GetAttrStr(__pyx_v_event, __pyx_n_s_completion_type); if (unlikely(!__pyx_t_1)) __PYX_ERR(8, 387, __pyx_L7_error)
                __Pyx_GOTREF(__pyx_t_1);
                __pyx_t_3 = __Pyx_GetModuleGlobalName(__pyx_n_s_CompletionType); if (unlikely(!__pyx_t_3)) __PYX_ERR(8, 387, __pyx_L7_error)
                __Pyx_GOTREF(__pyx_t_3);
                __pyx_t_4 = __Pyx_PyObject_GetAttrStr(__pyx_t_3, __pyx_n_s_queue_timeout); if (unlikely(!__pyx_t_4)) __PYX_ERR(8, 387, __pyx_L7_error)
                __Pyx_GOTREF(__pyx_t_4);
                __Pyx_DECREF(__pyx_t_3); __pyx_t_3 = 0;
                __pyx_t_3 = PyObject_RichCompare(__pyx_t_1, __pyx_t_4, Py_EQ); __Pyx_XGOTREF(__pyx_t_3); if (unlikely(!__pyx_t_3)) __PYX_ERR(8, 387, __pyx_L7_error)
                __Pyx_DECREF(__pyx_t_1); __pyx_t_1 = 0;
                __Pyx_DECREF(__pyx_t_4); __pyx_t_4 = 0;
                __pyx_t_14 = __Pyx_PyObject_IsTrue(__pyx_t_3); if (unlikely(__pyx_t_14 < 0)) __PYX_ERR(8, 387, __pyx_L7_error)
                __Pyx_DECREF(__pyx_t_3); __pyx_t_3 = 0;
                if (__pyx_t_14) {

                  /* "src/python/grpcio/grpc/_cython/_cygrpc/channel.pyx.pxi":388
 *           event = channel.next_call_event()
 *           if event.completion_type == CompletionType.queue_timeout:
 *               continue             # <<<<<<<<<<<<<<
 *           event.tag(event)
 *       else:
 */
                  goto __pyx_L19_continue;

                  /* "src/python/grpcio/grpc/_cython/_cygrpc/channel.pyx.pxi":387
 *         while not _calls_drained(state):
 *           event = channel.next_call_event()
 *           if event.completion_type == CompletionType.queue_timeout:             # <<<<<<<<<<<<<<
 *               continue
 *           event.tag(event)
 */
                }

                /* "src/python/grpcio/grpc/_cython/_cygrpc/channel.pyx.pxi":389
 *           if event.completion_type == CompletionType.queue_timeout:
 *               continue
 *           event.tag(event)             # <<<<<<<<<<<<<<
 *       else:
 *         while state.integrated_call_states:
 */
                __pyx_t_4 = __Pyx_PyObject_GetAttrStr(__pyx_v_event, __pyx_n_s_tag); if (unlikely(!__pyx_t_4)) __PYX_ERR(8, 389, __pyx_L7_error)
                __Pyx_GOTREF(__pyx_t_4);
                __pyx_t_1 = NULL;
                if (CYTHON_UNPACK_METHODS && likely(PyMethod_Check(__pyx_t_4))) {
                  __pyx_t_1 = PyMethod_GET_SELF(__pyx_t_4);
                  if (likely(__pyx_t_1)) {
                    PyObject* function = PyMethod_GET_FUNCTION(__pyx_t_4);
                    __Pyx_INCREF(__pyx_t_1);
                    __Pyx_INCREF(function);
                    __Pyx_DECREF_SET(__pyx_t_4, function);
                  }
                }
                if (!__pyx_t_1) {
                  __pyx_t_3 = __Pyx_PyObject_CallOneArg(__pyx_t_4, __pyx_v_event); if (unlikely(!__pyx_t_3)) __PYX_ERR(8, 389, __pyx_L7_error)
                  __Pyx_GOTREF(__pyx_t_3);
                } else {
                  #if CYTHON_FAST_PYCALL
                  if (PyFunction_Check(__pyx_t_4)) {
                    PyObject *__pyx_temp[2] = {__pyx_t_1, __pyx_v_event};
                    __pyx_t_3 = __Pyx_PyFunction_FastCall(__pyx_t_4, __pyx_temp+1-1, 1+1); if (unlikely(!__pyx_t_3)) __PYX_ERR(8, 389, __pyx_L7_error)
                    __Pyx_XDECREF(__pyx_t_1); __pyx_t_1 = 0;
                    __Pyx_GOTREF(__pyx_t_3);
                  } else
                  #endif
                  #if CYTHON_FAST_PYCCALL
                  if (__Pyx_PyFastCFunction_Check(__pyx_t_4)) {
                    PyObject *__pyx_temp[2] = {__pyx_t_1, __pyx_v_event};
                    __pyx_t_3 = __Pyx_PyCFunction_FastCall(__pyx_t_4, __pyx_temp+1-1, 1+1); if (unlikely(!__pyx_t_3)) __PYX_ERR(8, 389, __pyx_L7_error)
                    __Pyx_XDECREF(__pyx_t_1); __pyx_t_1 = 0;
                    __Pyx_GOTREF(__pyx_t_3);
                  } else
                  #endif
                  {
                    __pyx_t_15 = PyTuple_New(1+1); if (unlikely(!__pyx_t_15)) __PYX_ERR(8, 389, __pyx_L7_error)
                    __Pyx_GOTREF(__pyx_t_15);
                    __Pyx_GIVEREF(__pyx_t_1); PyTuple_SET_ITEM(__pyx_t_15, 0, __pyx_t_1); __pyx_t_1 = NULL;
                    __Pyx_INCREF(__pyx_v_event);
                    __Pyx_GIVEREF(__pyx_v_event);
                    PyTuple_SET_ITEM(__pyx_t_15, 0+1, __pyx_v_event);
                    __pyx_t_3 = __Pyx_PyObject_Call(__pyx_t_4, __pyx_t_15, NULL); if (unlikely(!__pyx_t_3)) __PYX_ERR(8, 389, __pyx_L7_error)
                    __Pyx_GOTREF(__pyx_t_3);
                    __Pyx_DECREF(__pyx_t_15); __pyx_t_15 = 0;
                  }
                }
                __Pyx_DECREF(__pyx_t_4); __pyx_t_4 = 0;
                __Pyx_DECREF(__pyx_t_3); __pyx_t_3 = 0;
                __pyx_L19_continue:;
              }

              /* "src/python/grpcio/grpc/_cython/_cygrpc/channel.pyx.pxi":384
 *       # watching.
 * 
 *       if drain_calls:             # <<<<<<<<<<<<<<
 *         while not _calls_drained(state):
 *           event = channel.next_call_event()
 */
              goto __pyx_L18;
            }

            /* "src/python/grpcio/grpc/_cython/_cygrpc/channel.pyx.pxi":391
 *           event.tag(event)
 *       else:
 *         while state.integrated_call_states:             # <<<<<<<<<<<<<<
 *           state.condition.wait()
 *         while state.segregated_call_states:
 */
            /*else*/ {
              while (1) {
                __pyx_t_14 = __Pyx_PyObject_IsTrue(__pyx_v_state->integrated_call_states); if (unlikely(__pyx_t_14 < 0)) __PYX_ERR(8, 391, __pyx_L7_error)
                if (!__pyx_t_14) break;

                /* "src/python/grpcio/grpc/_cython/_cygrpc/channel.pyx.pxi":392
 *       else:
 *         while state.integrated_call_states:
 *           state.condition.wait()             # <<<<<<<<<<<<<<
 *         while state.segregated_call_states:
 *           state.condition.wait()
 */
                __pyx_t_4 = __Pyx_PyObject_GetAttrStr(__pyx_v_state->condition, __pyx_n_s_wait); if (unlikely(!__pyx_t_4)) __PYX_ERR(8, 392, __pyx_L7_error)
                __Pyx_GOTREF(__pyx_t_4);
                __pyx_t_15 = NULL;
                if (CYTHON_UNPACK_METHODS && likely(PyMethod_Check(__pyx_t_4))) {
                  __pyx_t_15 = PyMethod_GET_SELF(__pyx_t_4);
                  if (likely(__pyx_t_15)) {
                    PyObject* function = PyMethod_GET_FUNCTION(__pyx_t_4);
                    __Pyx_INCREF(__pyx_t_15);
                    __Pyx_INCREF(function);
                    __Pyx_DECREF_SET(__pyx_t_4, function);
                  }
                }
                if (__pyx_t_15) {
                  __pyx_t_3 = __Pyx_PyObject_CallOneArg(__pyx_t_4, __pyx_t_15); if (unlikely(!__pyx_t_3)) __PYX_ERR(8, 392, __pyx_L7_error)
                  __Pyx_DECREF(__pyx_t_15); __pyx_t_15 = 0;
                } else {
                  __pyx_t_3 = __Pyx_PyObject_CallNoArg(__pyx_t_4); if (unlikely(!__pyx_t_3)) __PYX_ERR(8, 392, __pyx_L7_error)
                }
                __Pyx_GOTREF(__pyx_t_3);
                __Pyx_DECREF(__pyx_t_4); __pyx_t_4 = 0;
                __Pyx_DECREF(__pyx_t_3); __pyx_t_3 = 0;
              }

              /* "src/python/grpcio/grpc/_cython/_cygrpc/channel.pyx.pxi":393
 *         while state.integrated_call_states:
 *           state.condition.wait()
 *         while state.segregated_call_states:             # <<<<<<<<<<<<<<
 *           state.condition.wait()
 *         while state.connectivity_due:
 */
              while (1) {
                __pyx_t_14 = (__pyx_v_state->segregated_call_states != Py_None)&&(PySet_GET_SIZE(__pyx_v_state->segregated_call_states) != 0);
                if (!__pyx_t_14) break;

                /* "src/python/grpcio/grpc/_cython/_cygrpc/channel.pyx.pxi":394
 *           state.condition.wait()
 *         while state.segregated_call_states:
 *           state.condition.wait()             # <<<<<<<<<<<<<<
 *         while state.connectivity_due:
 *           state.condition.wait()
 */
                __pyx_t_4 = __Pyx_PyObject_GetAttrStr(__pyx_v_state->condition, __pyx_n_s_wait); if (unlikely(!__pyx_t_4)) __PYX_ERR(8, 394, __pyx_L7_error)
                __Pyx_GOTREF(__pyx_t_4);
                __pyx_t_15 = NULL;
                if (CYTHON_UNPACK_METHODS && likely(PyMethod_Check(__pyx_t_4))) {
                  __pyx_t_15 = PyMethod_GET_SELF(__pyx_t_4);
                  if (likely(__pyx_t_15)) {
                    PyObject* function = PyMethod_GET_FUNCTION(__pyx_t_4);
                    __Pyx_INCREF(__pyx_t_15);
                    __Pyx_INCREF(function);
                    __Pyx_DECREF_SET(__pyx_t_4, function);
                  }
                }
                if (__pyx_t_15) {
                  __pyx_t_3 = __Pyx_PyObject_CallOneArg(__pyx_t_4, __pyx_t_15); if (unlikely(!__pyx_t_3)) __PYX_ERR(8, 394, __pyx_L7_error)
                  __Pyx_DECREF(__pyx_t_15); __pyx_t_15 = 0;
                } else {
                  __pyx_t_3 = __Pyx_PyObject_CallNoArg(__pyx_t_4); if (unlikely(!__pyx_t_3)) __PYX_ERR(8, 394, __pyx_L7_error)
                }
                __Pyx_GOTREF(__pyx_t_3);
                __Pyx_DECREF(__pyx_t_4); __pyx_t_4 = 0;
                __Pyx_DECREF(__pyx_t_3); __pyx_t_3 = 0;
              }

              /* "src/python/grpcio/grpc/_cython/_cygrpc/channel.pyx.pxi":395
 *         while state.segregated_call_states:
 *           state.condition.wait()
 *         while state.connectivity_due:             # <<<<<<<<<<<<<<
 *           state.condition.wait()
 * 
 */
              while (1) {
                __pyx_t_14 = (__pyx_v_state->connectivity_due != Py_None)&&(PySet_GET_SIZE(__pyx_v_state->connectivity_due) != 0);
                if (!__pyx_t_14) break;

                /* "src/python/grpcio/grpc/_cython/_cygrpc/channel.pyx.pxi":396
 *           state.condition.wait()
 *         while state.connectivity_due:
 *           state.condition.wait()             # <<<<<<<<<<<<<<
 * 
 *       _destroy_c_completion_queue(state.c_call_completion_queue)
 */
                __pyx_t_4 = __Pyx_PyObject_GetAttrStr(__pyx_v_state->condition, __pyx_n_s_wait); if (unlikely(!__pyx_t_4)) __PYX_ERR(8, 396, __pyx_L7_error)
                __Pyx_GOTREF(__pyx_t_4);
                __pyx_t_15 = NULL;
                if (CYTHON_UNPACK_METHODS && likely(PyMethod_Check(__pyx_t_4))) {
                  __pyx_t_15 = PyMethod_GET_SELF(__pyx_t_4);
                  if (likely(__pyx_t_15)) {
                    PyObject* function = PyMethod_GET_FUNCTION(__pyx_t_4);
                    __Pyx_INCREF(__pyx_t_15);
                    __Pyx_INCREF(function);
                    __Pyx_DECREF_SET(__pyx_t_4, function);
                  }
                }
                if (__pyx_t_15) {
                  __pyx_t_3 = __Pyx_PyObject_CallOneArg(__pyx_t_4, __pyx_t_15); if (unlikely(!__pyx_t_3)) __PYX_ERR(8, 396, __pyx_L7_error)
                  __Pyx_DECREF(__pyx_t_15); __pyx_t_15 = 0;
                } else {
                  __pyx_t_3 = __Pyx_PyObject_CallNoArg(__pyx_t_4); if (unlikely(!__pyx_t_3)) __PYX_ERR(8, 396, __pyx_L7_error)
                }
                __Pyx_GOTREF(__pyx_t_3);
                __Pyx_DECREF(__pyx_t_4); __pyx_t_4 = 0;
                __Pyx_DECREF(__pyx_t_3); __pyx_t_3 = 0;
              }
            }
            __pyx_L18:;

            /* "src/python/grpcio/grpc/_cython/_cygrpc/channel.pyx.pxi":398
 *           state.condition.wait()
 * 
 *       _destroy_c_completion_queue(state.c_call_completion_queue)             # <<<<<<<<<<<<<<
 *       _destroy_c_completion_queue(state.c_connectivity_completion_queue)
 *       grpc_channel_destroy(state.c_channel)
 */
            __pyx_t_3 = __pyx_f_4grpc_7_cython_6cygrpc__destroy_c_completion_queue(__pyx_v_state->c_call_completion_queue); if (unlikely(!__pyx_t_3)) __PYX_ERR(8, 398, __pyx_L7_error)
            __Pyx_GOTREF(__pyx_t_3);
            __Pyx_DECREF(__pyx_t_3); __pyx_t_3 = 0;

            /* "src/python/grpcio/grpc/_cython/_cygrpc/channel.pyx.pxi":399
 * 
 *       _destroy_c_completion_queue(state.c_call_completion_queue)
 *       _destroy_c_completion_queue(state.c_connectivity_completion_queue)             # <<<<<<<<<<<<<<
 *       grpc_channel_destroy(state.c_channel)
 *       state.c_channel = NULL
 */
            __pyx_t_3 = __pyx_f_4grpc_7_cython_6cygrpc__destroy_c_completion_queue(__pyx_v_state->c_connectivity_completion_queue); if (unlikely(!__pyx_t_3)) __PYX_ERR(8, 399, __pyx_L7_error)
            __Pyx_GOTREF(__pyx_t_3);
            __Pyx_DECREF(__pyx_t_3); __pyx_t_3 = 0;

            /* "src/python/grpcio/grpc/_cython/_cygrpc/channel.pyx.pxi":400
 *       _destroy_c_completion_queue(state.c_call_completion_queue)
 *       _destroy_c_completion_queue(state.c_connectivity_completion_queue)
 *       grpc_channel_destroy(state.c_channel)             # <<<<<<<<<<<<<<
 *       state.c_channel = NULL
 *       grpc_shutdown_blocking()
 */
            grpc_channel_destroy(__pyx_v_state->c_channel);

            /* "src/python/grpcio/grpc/_cython/_cygrpc/channel.pyx.pxi":401
 *       _destroy_c_completion_queue(state.c_connectivity_completion_queue)
 *       grpc_channel_destroy(state.c_channel)
 *       state.c_channel = NULL             # <<<<<<<<<<<<<<
 *       grpc_shutdown_blocking()
 *       state.condition.notify_all()
 */
            __pyx_v_state->c_channel = NULL;

            /* "src/python/grpcio/grpc/_cython/_cygrpc/channel.pyx.pxi":402
 *       grpc_channel_destroy(state.c_channel)
 *       state.c_channel = NULL
 *       grpc_shutdown_blocking()             # <<<<<<<<<<<<<<
 *       state.condition.notify_all()
 *     else:
 */
            grpc_shutdown_blocking();

            /* "src/python/grpcio/grpc/_cython/_cygrpc/channel.pyx.pxi":403
 *       state.c_channel = NULL
 *       grpc_shutdown_blocking()
 *       state.condition.notify_all()             # <<<<<<<<<<<<<<
 *     else:
 *       # Another call to close already completed in the past or is currently
 */
            __pyx_t_4 = __Pyx_PyObject_GetAttrStr(__pyx_v_state->condition, __pyx_n_s_notify_all); if (unlikely(!__pyx_t_4)) __PYX_ERR(8, 403, __pyx_L7_error)
            __Pyx_GOTREF(__pyx_t_4);
            __pyx_t_15 = NULL;
            if (CYTHON_UNPACK_METHODS && likely(PyMethod_Check(__pyx_t_4))) {
              __pyx_t_15 = PyMethod_GET_SELF(__pyx_t_4);
              if (likely(__pyx_t_15)) {
                PyObject* function = PyMethod_GET_FUNCTION(__pyx_t_4);
                __Pyx_INCREF(__pyx_t_15);
                __Pyx_INCREF(function);
                __Pyx_DECREF_SET(__pyx_t_4, function);
              }
            }
            if (__pyx_t_15) {
              __pyx_t_3 = __Pyx_PyObject_CallOneArg(__pyx_t_4, __pyx_t_15); if (unlikely(!__pyx_t_3)) __PYX_ERR(8, 403, __pyx_L7_error)
              __Pyx_DECREF(__pyx_t_15); __pyx_t_15 = 0;
            } else {
              __pyx_t_3 = __Pyx_PyObject_CallNoArg(__pyx_t_4); if (unlikely(!__pyx_t_3)) __PYX_ERR(8, 403, __pyx_L7_error)
            }
            __Pyx_GOTREF(__pyx_t_3);
            __Pyx_DECREF(__pyx_t_4); __pyx_t_4 = 0;
            __Pyx_DECREF(__pyx_t_3); __pyx_t_3 = 0;

            /* "src/python/grpcio/grpc/_cython/_cygrpc/channel.pyx.pxi":372
 *   encoded_details = _encode(details)
 *   with state.condition:
 *     if state.open:             # <<<<<<<<<<<<<<
 *       state.open = False
 *       state.closed_reason = details
 */
            goto __pyx_L13;
          }

          /* "src/python/grpcio/grpc/_cython/_cygrpc/channel.pyx.pxi":407
 *       # Another call to close already completed in the past or is currently
 *       # being executed in another thread.
 *       while state.c_channel != NULL:             # <<<<<<<<<<<<<<
 *         state.condition.wait()
 * 
 */
          /*else*/ {
            while (1) {
              __pyx_t_14 = ((__pyx_v_state->c_channel != NULL) != 0);
              if (!__pyx_t_14) break;

              /* "src/python/grpcio/grpc/_cython/_cygrpc/channel.pyx.pxi":408
 *       # being executed in another thread.
 *       while state.c_channel != NULL:
 *         state.condition.wait()             # <<<<<<<<<<<<<<
 * 
 * 
 */
              __pyx_t_4 = __Pyx_PyObject_GetAttrStr(__pyx_v_state->condition, __pyx_n_s_wait); if (unlikely(!__pyx_t_4)) __PYX_ERR(8, 408, __pyx_L7_error)
              __Pyx_GOTREF(__pyx_t_4);
              __pyx_t_15 = NULL;
              if (CYTHON_UNPACK_METHODS && likely(PyMethod_Check(__pyx_t_4))) {
                __pyx_t_15 = PyMethod_GET_SELF(__pyx_t_4);
                if (likely(__pyx_t_15)) {
                  PyObject* function = PyMethod_GET_FUNCTION(__pyx_t_4);
                  __Pyx_INCREF(__pyx_t_15);
                  __Pyx_INCREF(function);
                  __Pyx_DECREF_SET(__pyx_t_4, function);
                }
              }
              if (__pyx_t_15) {
                __pyx_t_3 = __Pyx_PyObject_CallOneArg(__pyx_t_4, __pyx_t_15); if (unlikely(!__pyx_t_3)) __PYX_ERR(8, 408, __pyx_L7_error)
                __Pyx_DECREF(__pyx_t_15); __pyx_t_15 = 0;
              } else {
                __pyx_t_3 = __Pyx_PyObject_CallNoArg(__pyx_t_4); if (unlikely(!__pyx_t_3)) __PYX_ERR(8, 408, __pyx_L7_error)
              }
              __Pyx_GOTREF(__pyx_t_3);
              __Pyx_DECREF(__pyx_t_4); __pyx_t_4 = 0;
              __Pyx_DECREF(__pyx_t_3); __pyx_t_3 = 0;
            }
          }
          __pyx_L13:;

          /* "src/python/grpcio/grpc/_cython/_cygrpc/channel.pyx.pxi":371
 *   cdef _CallState call_state
 *   encoded_details = _encode(details)
 *   with state.condition:             # <<<<<<<<<<<<<<
 *     if state.open:
 *       state.open = False
 */
        }
        __Pyx_XDECREF(__pyx_t_5); __pyx_t_5 = 0;
        __Pyx_XDECREF(__pyx_t_6); __pyx_t_6 = 0;
        __Pyx_XDECREF(__pyx_t_7); __pyx_t_7 = 0;
        goto __pyx_L12_try_end;
        __pyx_L7_error:;
        __Pyx_XDECREF(__pyx_t_1); __pyx_t_1 = 0;
        __Pyx_XDECREF(__pyx_t_15); __pyx_t_15 = 0;
        __Pyx_XDECREF(__pyx_t_4); __pyx_t_4 = 0;
        __Pyx_XDECREF(__pyx_t_3); __pyx_t_3 = 0;
        /*except:*/ {
          __Pyx_AddTraceback("grpc._cython.cygrpc._close", __pyx_clineno, __pyx_lineno, __pyx_filename);
          if (__Pyx_GetException(&__pyx_t_3, &__pyx_t_4, &__pyx_t_15) < 0) __PYX_ERR(8, 371, __pyx_L9_except_error)
          __Pyx_GOTREF(__pyx_t_3);
          __Pyx_GOTREF(__pyx_t_4);
          __Pyx_GOTREF(__pyx_t_15);
          __pyx_t_1 = PyTuple_Pack(3, __pyx_t_3, __pyx_t_4, __pyx_t_15); if (unlikely(!__pyx_t_1)) __PYX_ERR(8, 371, __pyx_L9_except_error)
          __Pyx_GOTREF(__pyx_t_1);
          __pyx_t_16 = __Pyx_PyObject_Call(__pyx_t_2, __pyx_t_1, NULL);
          __Pyx_DECREF(__pyx_t_2); __pyx_t_2 = 0;
          __Pyx_DECREF(__pyx_t_1); __pyx_t_1 = 0;
          if (unlikely(!__pyx_t_16)) __PYX_ERR(8, 371, __pyx_L9_except_error)
          __Pyx_GOTREF(__pyx_t_16);
          __pyx_t_14 = __Pyx_PyObject_IsTrue(__pyx_t_16);
          __Pyx_DECREF(__pyx_t_16); __pyx_t_16 = 0;
          if (__pyx_t_14 < 0) __PYX_ERR(8, 371, __pyx_L9_except_error)
          __pyx_t_8 = ((!(__pyx_t_14 != 0)) != 0);
          if (__pyx_t_8) {
            __Pyx_GIVEREF(__pyx_t_3);
            __Pyx_GIVEREF(__pyx_t_4);
            __Pyx_XGIVEREF(__pyx_t_15);
            __Pyx_ErrRestoreWithState(__pyx_t_3, __pyx_t_4, __pyx_t_15);
            __pyx_t_3 = 0; __pyx_t_4 = 0; __pyx_t_15 = 0; 
            __PYX_ERR(8, 371, __pyx_L9_except_error)
          }
          __Pyx_DECREF(__pyx_t_3); __pyx_t_3 = 0;
          __Pyx_DECREF(__pyx_t_4); __pyx_t_4 = 0;
          __Pyx_DECREF(__pyx_t_15); __pyx_t_15 = 0;
          goto __pyx_L8_exception_handled;
        }
        __pyx_L9_except_error:;
        __Pyx_XGIVEREF(__pyx_t_5);
        __Pyx_XGIVEREF(__pyx_t_6);
        __Pyx_XGIVEREF(__pyx_t_7);
        __Pyx_ExceptionReset(__pyx_t_5, __pyx_t_6, __pyx_t_7);
        goto __pyx_L1_error;
        __pyx_L8_exception_handled:;
        __Pyx_XGIVEREF(__pyx_t_5);
        __Pyx_XGIVEREF(__pyx_t_6);
        __Pyx_XGIVEREF(__pyx_t_7);
        __Pyx_ExceptionReset(__pyx_t_5, __pyx_t_6, __pyx_t_7);
        __pyx_L12_try_end:;
      }
    }
    /*finally:*/ {
      /*normal exit:*/{
        if (__pyx_t_2) {
          __pyx_t_7 = __Pyx_PyObject_Call(__pyx_t_2, __pyx_tuple__41, NULL);
          __Pyx_DECREF(__pyx_t_2); __pyx_t_2 = 0;
          if (unlikely(!__pyx_t_7)) __PYX_ERR(8, 371, __pyx_L1_error)
          __Pyx_GOTREF(__pyx_t_7);
          __Pyx_DECREF(__pyx_t_7); __pyx_t_7 = 0;
        }
        goto __pyx_L6;
      }
      __pyx_L6:;
    }
    goto __pyx_L33;
    __pyx_L3_error:;
    __Pyx_DECREF(__pyx_t_2); __pyx_t_2 = 0;
    goto __pyx_L1_error;
    __pyx_L33:;
  }

  /* "src/python/grpcio/grpc/_cython/_cygrpc/channel.pyx.pxi":366
 * 
 * 
 * cdef _close(Channel channel, grpc_status_code code, object details,             # <<<<<<<<<<<<<<
 *     drain_calls):
 *   cdef _ChannelState state = channel._state
 */

  /* function exit code */
  __pyx_r = Py_None; __Pyx_INCREF(Py_None);
  goto __pyx_L0;
  __pyx_L1_error:;
  __Pyx_XDECREF(__pyx_t_1);
  __Pyx_XDECREF(__pyx_t_3);
  __Pyx_XDECREF(__pyx_t_4);
  __Pyx_XDECREF(__pyx_t_15);
  __Pyx_AddTraceback("grpc._cython.cygrpc._close", __pyx_clineno, __pyx_lineno, __pyx_filename);
  __pyx_r = 0;
  __pyx_L0:;
  __Pyx_XDECREF((PyObject *)__pyx_v_state);
  __Pyx_XDECREF((PyObject *)__pyx_v_call_state);
  __Pyx_XDECREF(__pyx_v_encoded_details);
  __Pyx_XDECREF(__pyx_v_event);
  __Pyx_XGIVEREF(__pyx_r);
  __Pyx_RefNannyFinishContext();
  return __pyx_r;
}

/* "src/python/grpcio/grpc/_cython/_cygrpc/channel.pyx.pxi":411
 * 
 * 
 * cdef _calls_drained(_ChannelState state):             # <<<<<<<<<<<<<<
 *   return not (state.integrated_call_states or state.segregated_call_states or
 *               state.connectivity_due)
 */

static PyObject *__pyx_f_4grpc_7_cython_6cygrpc__calls_drained(struct __pyx_obj_4grpc_7_cython_6cygrpc__ChannelState *__pyx_v_state) {
  PyObject *__pyx_r = NULL;
  __Pyx_RefNannyDeclarations
  int __pyx_t_1;
  int __pyx_t_2;
  PyObject *__pyx_t_3 = NULL;
  __Pyx_RefNannySetupContext("_calls_drained", 0);

  /* "src/python/grpcio/grpc/_cython/_cygrpc/channel.pyx.pxi":412
 * 
 * cdef _calls_drained(_ChannelState state):
 *   return not (state.integrated_call_states or state.segregated_call_states or             # <<<<<<<<<<<<<<
 *               state.connectivity_due)
 * 
 */
  __Pyx_XDECREF(__pyx_r);
  __pyx_t_2 = __Pyx_PyObject_IsTrue(__pyx_v_state->integrated_call_states); if (unlikely(__pyx_t_2 < 0)) __PYX_ERR(8, 412, __pyx_L1_error)
  if (!__pyx_t_2) {
  } else {
    __pyx_t_1 = __pyx_t_2;
    goto __pyx_L3_bool_binop_done;
  }
  __pyx_t_2 = (__pyx_v_state->segregated_call_states != Py_None)&&(PySet_GET_SIZE(__pyx_v_state->segregated_call_states) != 0);
  if (!__pyx_t_2) {
  } else {
    __pyx_t_1 = __pyx_t_2;
    goto __pyx_L3_bool_binop_done;
  }

  /* "src/python/grpcio/grpc/_cython/_cygrpc/channel.pyx.pxi":413
 * cdef _calls_drained(_ChannelState state):
 *   return not (state.integrated_call_states or state.segregated_call_states or
 *               state.connectivity_due)             # <<<<<<<<<<<<<<
 * 
 * cdef class Channel:
 */
  __pyx_t_2 = (__pyx_v_state->connectivity_due != Py_None)&&(PySet_GET_SIZE(__pyx_v_state->connectivity_due) != 0);
  __pyx_t_1 = __pyx_t_2;
  __pyx_L3_bool_binop_done:;

  /* "src/python/grpcio/grpc/_cython/_cygrpc/channel.pyx.pxi":412
 * 
 * cdef _calls_drained(_ChannelState state):
 *   return not (state.integrated_call_states or state.segregated_call_states or             # <<<<<<<<<<<<<<
 *               state.connectivity_due)
 * 
 */
  __pyx_t_3 = __Pyx_PyBool_FromLong((!__pyx_t_1)); if (unlikely(!__pyx_t_3)) __PYX_ERR(8, 412, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_3);
  __pyx_r = __pyx_t_3;
  __pyx_t_3 = 0;
  goto __pyx_L0;

  /* "src/python/grpcio/grpc/_cython/_cygrpc/channel.pyx.pxi":411
 * 
 * 
 * cdef _calls_drained(_ChannelState state):             # <<<<<<<<<<<<<<
 *   return not (state.integrated_call_states or state.segregated_call_states or
 *               state.connectivity_due)
 */

  /* function exit code */
  __pyx_L1_error:;
  __Pyx_XDECREF(__pyx_t_3);
  __Pyx_AddTraceback("grpc._cython.cygrpc._calls_drained", __pyx_clineno, __pyx_lineno, __pyx_filename);
  __pyx_r = 0;
  __pyx_L0:;
  __Pyx_XGIVEREF(__pyx_r);
  __Pyx_RefNannyFinishContext();
  return __pyx_r;
}

/* "src/python/grpcio/grpc/_cython/_cygrpc/channel.pyx.pxi":417
 * cdef class Channel:
 * 
 *   def __cinit__(             # <<<<<<<<<<<<<<
 *       self, bytes target, object arguments,
 *       ChannelCredentials channel_credentials):
 */

/* Python wrapper */
static int __pyx_pw_4grpc_7_cython_6cygrpc_7Channel_1__cinit__(PyObject *__pyx_v_self, PyObject *__pyx_args, PyObject *__pyx_kwds); /*proto*/
static int __pyx_pw_4grpc_7_cython_6cygrpc_7Channel_1__cinit__(PyObject *__pyx_v_self, PyObject *__pyx_args, PyObject *__pyx_kwds) {
  PyObject *__pyx_v_target = 0;
  PyObject *__pyx_v_arguments = 0;
  struct __pyx_obj_4grpc_7_cython_6cygrpc_ChannelCredentials *__pyx_v_channel_credentials = 0;
  int __pyx_r;
  __Pyx_RefNannyDeclarations
  __Pyx_RefNannySetupContext("__cinit__ (wrapper)", 0);
  {
    static PyObject **__pyx_pyargnames[] = {&__pyx_n_s_target,&__pyx_n_s_arguments,&__pyx_n_s_channel_credentials,0};
    PyObject* values[3] = {0,0,0};
    if (unlikely(__pyx_kwds)) {
      Py_ssize_t kw_args;
      const Py_ssize_t pos_args = PyTuple_GET_SIZE(__pyx_args);
      switch (pos_args) {
        case  3: values[2] = PyTuple_GET_ITEM(__pyx_args, 2);
        CYTHON_FALLTHROUGH;
        case  2: values[1] = PyTuple_GET_ITEM(__pyx_args, 1);
        CYTHON_FALLTHROUGH;
        case  1: values[0] = PyTuple_GET_ITEM(__pyx_args, 0);
        CYTHON_FALLTHROUGH;
        case  0: break;
        default: goto __pyx_L5_argtuple_error;
      }
      kw_args = PyDict_Size(__pyx_kwds);
      switch (pos_args) {
        case  0:
        if (likely((values[0] = __Pyx_PyDict_GetItemStr(__pyx_kwds, __pyx_n_s_target)) != 0)) kw_args--;
        else goto __pyx_L5_argtuple_error;
        CYTHON_FALLTHROUGH;
        case  1:
        if (likely((values[1] = __Pyx_PyDict_GetItemStr(__pyx_kwds, __pyx_n_s_arguments)) != 0)) kw_args--;
        else {
          __Pyx_RaiseArgtupleInvalid("__cinit__", 1, 3, 3, 1); __PYX_ERR(8, 417, __pyx_L3_error)
        }
        CYTHON_FALLTHROUGH;
        case  2:
        if (likely((values[2] = __Pyx_PyDict_GetItemStr(__pyx_kwds, __pyx_n_s_channel_credentials)) != 0)) kw_args--;
        else {
          __Pyx_RaiseArgtupleInvalid("__cinit__", 1, 3, 3, 2); __PYX_ERR(8, 417, __pyx_L3_error)
        }
      }
      if (unlikely(kw_args > 0)) {
        if (unlikely(__Pyx_ParseOptionalKeywords(__pyx_kwds, __pyx_pyargnames, 0, values, pos_args, "__cinit__") < 0)) __PYX_ERR(8, 417, __pyx_L3_error)
      }
    } else if (PyTuple_GET_SIZE(__pyx_args) != 3) {
      goto __pyx_L5_argtuple_error;
    } else {
      values[0] = PyTuple_GET_ITEM(__pyx_args, 0);
      values[1] = PyTuple_GET_ITEM(__pyx_args, 1);
      values[2] = PyTuple_GET_ITEM(__pyx_args, 2);
    }
    __pyx_v_target = ((PyObject*)values[0]);
    __pyx_v_arguments = values[1];
    __pyx_v_channel_credentials = ((struct __pyx_obj_4grpc_7_cython_6cygrpc_ChannelCredentials *)values[2]);
  }
  goto __pyx_L4_argument_unpacking_done;
  __pyx_L5_argtuple_error:;
  __Pyx_RaiseArgtupleInvalid("__cinit__", 1, 3, 3, PyTuple_GET_SIZE(__pyx_args)); __PYX_ERR(8, 417, __pyx_L3_error)
  __pyx_L3_error:;
  __Pyx_AddTraceback("grpc._cython.cygrpc.Channel.__cinit__", __pyx_clineno, __pyx_lineno, __pyx_filename);
  __Pyx_RefNannyFinishContext();
  return -1;
  __pyx_L4_argument_unpacking_done:;
  if (unlikely(!__Pyx_ArgTypeTest(((PyObject *)__pyx_v_target), (&PyBytes_Type), 1, "target", 1))) __PYX_ERR(8, 418, __pyx_L1_error)
  if (unlikely(!__Pyx_ArgTypeTest(((PyObject *)__pyx_v_channel_credentials), __pyx_ptype_4grpc_7_cython_6cygrpc_ChannelCredentials, 1, "channel_credentials", 0))) __PYX_ERR(8, 419, __pyx_L1_error)
  __pyx_r = __pyx_pf_4grpc_7_cython_6cygrpc_7Channel___cinit__(((struct __pyx_obj_4grpc_7_cython_6cygrpc_Channel *)__pyx_v_self), __pyx_v_target, __pyx_v_arguments, __pyx_v_channel_credentials);

  /* function exit code */
  goto __pyx_L0;
  __pyx_L1_error:;
  __pyx_r = -1;
  __pyx_L0:;
  __Pyx_RefNannyFinishContext();
  return __pyx_r;
}

static int __pyx_pf_4grpc_7_cython_6cygrpc_7Channel___cinit__(struct __pyx_obj_4grpc_7_cython_6cygrpc_Channel *__pyx_v_self, PyObject *__pyx_v_target, PyObject *__pyx_v_arguments, struct __pyx_obj_4grpc_7_cython_6cygrpc_ChannelCredentials *__pyx_v_channel_credentials) {
  struct __pyx_obj_4grpc_7_cython_6cygrpc__ChannelArgs *__pyx_v_channel_args = 0;
  grpc_channel_credentials *__pyx_v_c_channel_credentials;
  int __pyx_r;
  __Pyx_RefNannyDeclarations
  PyObject *__pyx_t_1 = NULL;
  int __pyx_t_2;
  PyObject *__pyx_t_3 = NULL;
  PyObject *__pyx_t_4 = NULL;
  int __pyx_t_5;
  char *__pyx_t_6;
  grpc_channel_args *__pyx_t_7;
  grpc_channel_credentials *__pyx_t_8;
  __Pyx_RefNannySetupContext("__cinit__", 0);
  __Pyx_INCREF(__pyx_v_arguments);

  /* "src/python/grpcio/grpc/_cython/_cygrpc/channel.pyx.pxi":420
 *       self, bytes target, object arguments,
 *       ChannelCredentials channel_credentials):
 *     arguments = () if arguments is None else tuple(arguments)             # <<<<<<<<<<<<<<
 *     fork_handlers_and_grpc_init()
 *     self._state = _ChannelState()
 */
  __pyx_t_2 = (__pyx_v_arguments == Py_None);
  if ((__pyx_t_2 != 0)) {
    __Pyx_INCREF(__pyx_empty_tuple);
    __pyx_t_1 = __pyx_empty_tuple;
  } else {
    __pyx_t_3 = __Pyx_PySequence_Tuple(__pyx_v_arguments); if (unlikely(!__pyx_t_3)) __PYX_ERR(8, 420, __pyx_L1_error)
    __Pyx_GOTREF(__pyx_t_3);
    __pyx_t_1 = __pyx_t_3;
    __pyx_t_3 = 0;
  }
  __Pyx_DECREF_SET(__pyx_v_arguments, __pyx_t_1);
  __pyx_t_1 = 0;

  /* "src/python/grpcio/grpc/_cython/_cygrpc/channel.pyx.pxi":421
 *       ChannelCredentials channel_credentials):
 *     arguments = () if arguments is None else tuple(arguments)
 *     fork_handlers_and_grpc_init()             # <<<<<<<<<<<<<<
 *     self._state = _ChannelState()
 *     self._state.c_call_completion_queue = (
 */
  __pyx_t_3 = __Pyx_GetModuleGlobalName(__pyx_n_s_fork_handlers_and_grpc_init); if (unlikely(!__pyx_t_3)) __PYX_ERR(8, 421, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_3);
  __pyx_t_4 = NULL;
  if (CYTHON_UNPACK_METHODS && unlikely(PyMethod_Check(__pyx_t_3))) {
    __pyx_t_4 = PyMethod_GET_SELF(__pyx_t_3);
    if (likely(__pyx_t_4)) {
      PyObject* function = PyMethod_GET_FUNCTION(__pyx_t_3);
      __Pyx_INCREF(__pyx_t_4);
      __Pyx_INCREF(function);
      __Pyx_DECREF_SET(__pyx_t_3, function);
    }
  }
  if (__pyx_t_4) {
    __pyx_t_1 = __Pyx_PyObject_CallOneArg(__pyx_t_3, __pyx_t_4); if (unlikely(!__pyx_t_1)) __PYX_ERR(8, 421, __pyx_L1_error)
    __Pyx_DECREF(__pyx_t_4); __pyx_t_4 = 0;
  } else {
    __pyx_t_1 = __Pyx_PyObject_CallNoArg(__pyx_t_3); if (unlikely(!__pyx_t_1)) __PYX_ERR(8, 421, __pyx_L1_error)
  }
  __Pyx_GOTREF(__pyx_t_1);
  __Pyx_DECREF(__pyx_t_3); __pyx_t_3 = 0;
  __Pyx_DECREF(__pyx_t_1); __pyx_t_1 = 0;

  /* "src/python/grpcio/grpc/_cython/_cygrpc/channel.pyx.pxi":422
 *     arguments = () if arguments is None else tuple(arguments)
 *     fork_handlers_and_grpc_init()
 *     self._state = _ChannelState()             # <<<<<<<<<<<<<<
 *     self._state.c_call_completion_queue = (
 *         grpc_completion_queue_create_for_next(NULL))
 */
  __pyx_t_1 = __Pyx_PyObject_CallNoArg(((PyObject *)__pyx_ptype_4grpc_7_cython_6cygrpc__ChannelState)); if (unlikely(!__pyx_t_1)) __PYX_ERR(8, 422, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_1);
  __Pyx_GIVEREF(__pyx_t_1);
  __Pyx_GOTREF(__pyx_v_self->_state);
  __Pyx_DECREF(((PyObject *)__pyx_v_self->_state));
  __pyx_v_self->_state = ((struct __pyx_obj_4grpc_7_cython_6cygrpc__ChannelState *)__pyx_t_1);
  __pyx_t_1 = 0;

  /* "src/python/grpcio/grpc/_cython/_cygrpc/channel.pyx.pxi":423
 *     fork_handlers_and_grpc_init()
 *     self._state = _ChannelState()
 *     self._state.c_call_completion_queue = (             # <<<<<<<<<<<<<<
 *         grpc_completion_queue_create_for_next(NULL))
 *     self._state.c_connectivity_completion_queue = (
 */
  __pyx_v_self->_state->c_call_completion_queue = grpc_completion_queue_create_for_next(NULL);

  /* "src/python/grpcio/grpc/_cython/_cygrpc/channel.pyx.pxi":425
 *     self._state.c_call_completion_queue = (
 *         grpc_completion_queue_create_for_next(NULL))
 *     self._state.c_connectivity_completion_queue = (             # <<<<<<<<<<<<<<
 *         grpc_completion_queue_create_for_next(NULL))
 *     self._arguments = arguments
 */
  __pyx_v_self->_state->c_connectivity_completion_queue = grpc_completion_queue_create_for_next(NULL);

  /* "src/python/grpcio/grpc/_cython/_cygrpc/channel.pyx.pxi":427
 *     self._state.c_connectivity_completion_queue = (
 *         grpc_completion_queue_create_for_next(NULL))
 *     self._arguments = arguments             # <<<<<<<<<<<<<<
 *     self._vtable = _VTable()
 *     cdef _ChannelArgs channel_args = _ChannelArgs(
 */
  if (!(likely(PyTuple_CheckExact(__pyx_v_arguments))||((__pyx_v_arguments) == Py_None)||(PyErr_Format(PyExc_TypeError, "Expected %.16s, got %.200s", "tuple", Py_TYPE(__pyx_v_arguments)->tp_name), 0))) __PYX_ERR(8, 427, __pyx_L1_error)
  __pyx_t_1 = __pyx_v_arguments;
  __Pyx_INCREF(__pyx_t_1);
  __Pyx_GIVEREF(__pyx_t_1);
  __Pyx_GOTREF(__pyx_v_self->_arguments);
  __Pyx_DECREF(__pyx_v_self->_arguments);
  __pyx_v_self->_arguments = ((PyObject*)__pyx_t_1);
  __pyx_t_1 = 0;

  /* "src/python/grpcio/grpc/_cython/_cygrpc/channel.pyx.pxi":428
 *         grpc_completion_queue_create_for_next(NULL))
 *     self._arguments = arguments
 *     self._vtable = _VTable()             # <<<<<<<<<<<<<<
 *     cdef _ChannelArgs channel_args = _ChannelArgs(
 *         arguments, self._vtable)
 */
  __pyx_t_1 = __Pyx_PyObject_CallNoArg(((PyObject *)__pyx_ptype_4grpc_7_cython_6cygrpc__VTable)); if (unlikely(!__pyx_t_1)) __PYX_ERR(8, 428, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_1);
  __Pyx_GIVEREF(__pyx_t_1);
  __Pyx_GOTREF(__pyx_v_self->_vtable);
  __Pyx_DECREF(((PyObject *)__pyx_v_self->_vtable));
  __pyx_v_self->_vtable = ((struct __pyx_obj_4grpc_7_cython_6cygrpc__VTable *)__pyx_t_1);
  __pyx_t_1 = 0;

  /* "src/python/grpcio/grpc/_cython/_cygrpc/channel.pyx.pxi":429
 *     self._arguments = arguments
 *     self._vtable = _VTable()
 *     cdef _ChannelArgs channel_args = _ChannelArgs(             # <<<<<<<<<<<<<<
 *         arguments, self._vtable)
 *     if channel_credentials is None:
 */
  __pyx_t_1 = PyTuple_New(2); if (unlikely(!__pyx_t_1)) __PYX_ERR(8, 429, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_1);
  __Pyx_INCREF(__pyx_v_arguments);
  __Pyx_GIVEREF(__pyx_v_arguments);
  PyTuple_SET_ITEM(__pyx_t_1, 0, __pyx_v_arguments);
  __Pyx_INCREF(((PyObject *)__pyx_v_self->_vtable));
  __Pyx_GIVEREF(((PyObject *)__pyx_v_self->_vtable));
  PyTuple_SET_ITEM(__pyx_t_1, 1, ((PyObject *)__pyx_v_self->_vtable));
  __pyx_t_3 = __Pyx_PyObject_Call(((PyObject *)__pyx_ptype_4grpc_7_cython_6cygrpc__ChannelArgs), __pyx_t_1, NULL); if (unlikely(!__pyx_t_3)) __PYX_ERR(8, 429, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_3);
  __Pyx_DECREF(__pyx_t_1); __pyx_t_1 = 0;
  __pyx_v_channel_args = ((struct __pyx_obj_4grpc_7_cython_6cygrpc__ChannelArgs *)__pyx_t_3);
  __pyx_t_3 = 0;

  /* "src/python/grpcio/grpc/_cython/_cygrpc/channel.pyx.pxi":431
 *     cdef _ChannelArgs channel_args = _ChannelArgs(
 *         arguments, self._vtable)
 *     if channel_credentials is None:             # <<<<<<<<<<<<<<
 *       self._state.c_channel = grpc_insecure_channel_create(
 *           <char *>target, channel_args.c_args(), NULL)
 */
  __pyx_t_2 = (((PyObject *)__pyx_v_channel_credentials) == Py_None);
  __pyx_t_5 = (__pyx_t_2 != 0);
  if (__pyx_t_5) {

    /* "src/python/grpcio/grpc/_cython/_cygrpc/channel.pyx.pxi":433
 *     if channel_credentials is None:
 *       self._state.c_channel = grpc_insecure_channel_create(
 *           <char *>target, channel_args.c_args(), NULL)             # <<<<<<<<<<<<<<
 *     else:
 *       c_channel_credentials = channel_credentials.c()
 */
    if (unlikely(__pyx_v_target == Py_None)) {
      PyErr_SetString(PyExc_TypeError, "expected bytes, NoneType found");
      __PYX_ERR(8, 433, __pyx_L1_error)
    }
    __pyx_t_6 = __Pyx_PyBytes_AsWritableString(__pyx_v_target); if (unlikely((!__pyx_t_6) && PyErr_Occurred())) __PYX_ERR(8, 433, __pyx_L1_error)
    __pyx_t_7 = ((struct __pyx_vtabstruct_4grpc_7_cython_6cygrpc__ChannelArgs *)__pyx_v_channel_args->__pyx_vtab)->c_args(__pyx_v_channel_args); if (unlikely(PyErr_Occurred())) __PYX_ERR(8, 433, __pyx_L1_error)

    /* "src/python/grpcio/grpc/_cython/_cygrpc/channel.pyx.pxi":432
 *         arguments, self._vtable)
 *     if channel_credentials is None:
 *       self._state.c_channel = grpc_insecure_channel_create(             # <<<<<<<<<<<<<<
 *           <char *>target, channel_args.c_args(), NULL)
 *     else:
 */
    __pyx_v_self->_state->c_channel = grpc_insecure_channel_create(((char *)__pyx_t_6), __pyx_t_7, NULL);

    /* "src/python/grpcio/grpc/_cython/_cygrpc/channel.pyx.pxi":431
 *     cdef _ChannelArgs channel_args = _ChannelArgs(
 *         arguments, self._vtable)
 *     if channel_credentials is None:             # <<<<<<<<<<<<<<
 *       self._state.c_channel = grpc_insecure_channel_create(
 *           <char *>target, channel_args.c_args(), NULL)
 */
    goto __pyx_L3;
  }

  /* "src/python/grpcio/grpc/_cython/_cygrpc/channel.pyx.pxi":435
 *           <char *>target, channel_args.c_args(), NULL)
 *     else:
 *       c_channel_credentials = channel_credentials.c()             # <<<<<<<<<<<<<<
 *       self._state.c_channel = grpc_secure_channel_create(
 *           c_channel_credentials, <char *>target, channel_args.c_args(), NULL)
 */
  /*else*/ {
    __pyx_t_8 = ((struct __pyx_vtabstruct_4grpc_7_cython_6cygrpc_ChannelCredentials *)__pyx_v_channel_credentials->__pyx_vtab)->c(__pyx_v_channel_credentials); if (unlikely(PyErr_Occurred())) __PYX_ERR(8, 435, __pyx_L1_error)
    __pyx_v_c_channel_credentials = __pyx_t_8;

    /* "src/python/grpcio/grpc/_cython/_cygrpc/channel.pyx.pxi":437
 *       c_channel_credentials = channel_credentials.c()
 *       self._state.c_channel = grpc_secure_channel_create(
 *           c_channel_credentials, <char *>target, channel_args.c_args(), NULL)             # <<<<<<<<<<<<<<
 *       grpc_channel_credentials_release(c_channel_credentials)
 * 
 */
    if (unlikely(__pyx_v_target == Py_None)) {
      PyErr_SetString(PyExc_TypeError, "expected bytes, NoneType found");
      __PYX_ERR(8, 437, __pyx_L1_error)
    }
    __pyx_t_6 = __Pyx_PyBytes_AsWritableString(__pyx_v_target); if (unlikely((!__pyx_t_6) && PyErr_Occurred())) __PYX_ERR(8, 437, __pyx_L1_error)
    __pyx_t_7 = ((struct __pyx_vtabstruct_4grpc_7_cython_6cygrpc__ChannelArgs *)__pyx_v_channel_args->__pyx_vtab)->c_args(__pyx_v_channel_args); if (unlikely(PyErr_Occurred())) __PYX_ERR(8, 437, __pyx_L1_error)

    /* "src/python/grpcio/grpc/_cython/_cygrpc/channel.pyx.pxi":436
 *     else:
 *       c_channel_credentials = channel_credentials.c()
 *       self._state.c_channel = grpc_secure_channel_create(             # <<<<<<<<<<<<<<
 *           c_channel_credentials, <char *>target, channel_args.c_args(), NULL)
 *       grpc_channel_credentials_release(c_channel_credentials)
 */
    __pyx_v_self->_state->c_channel = grpc_secure_channel_create(__pyx_v_c_channel_credentials, ((char *)__pyx_t_6), __pyx_t_7, NULL);

    /* "src/python/grpcio/grpc/_cython/_cygrpc/channel.pyx.pxi":438
 *       self._state.c_channel = grpc_secure_channel_create(
 *           c_channel_credentials, <char *>target, channel_args.c_args(), NULL)
 *       grpc_channel_credentials_release(c_channel_credentials)             # <<<<<<<<<<<<<<
 * 
 *   def target(self):
 */
    grpc_channel_credentials_release(__pyx_v_c_channel_credentials);
  }
  __pyx_L3:;

  /* "src/python/grpcio/grpc/_cython/_cygrpc/channel.pyx.pxi":417
 * cdef class Channel:
 * 
 *   def __cinit__(             # <<<<<<<<<<<<<<
 *       self, bytes target, object arguments,
 *       ChannelCredentials channel_credentials):
 */

  /* function exit code */
  __pyx_r = 0;
  goto __pyx_L0;
  __pyx_L1_error:;
  __Pyx_XDECREF(__pyx_t_1);
  __Pyx_XDECREF(__pyx_t_3);
  __Pyx_XDECREF(__pyx_t_4);
  __Pyx_AddTraceback("grpc._cython.cygrpc.Channel.__cinit__", __pyx_clineno, __pyx_lineno, __pyx_filename);
  __pyx_r = -1;
  __pyx_L0:;
  __Pyx_XDECREF((PyObject *)__pyx_v_channel_args);
  __Pyx_XDECREF(__pyx_v_arguments);
  __Pyx_RefNannyFinishContext();
  return __pyx_r;
}

/* "src/python/grpcio/grpc/_cython/_cygrpc/channel.pyx.pxi":440
 *       grpc_channel_credentials_release(c_channel_credentials)
 * 
 *   def target(self):             # <<<<<<<<<<<<<<
 *     cdef char *c_target
 *     with self._state.condition:
 */

/* Python wrapper */
static PyObject *__pyx_pw_4grpc_7_cython_6cygrpc_7Channel_3target(PyObject *__pyx_v_self, CYTHON_UNUSED PyObject *unused); /*proto*/
static PyObject *__pyx_pw_4grpc_7_cython_6cygrpc_7Channel_3target(PyObject *__pyx_v_self, CYTHON_UNUSED PyObject *unused) {
  PyObject *__pyx_r = 0;
  __Pyx_RefNannyDeclarations
  __Pyx_RefNannySetupContext("target (wrapper)", 0);
  __pyx_r = __pyx_pf_4grpc_7_cython_6cygrpc_7Channel_2target(((struct __pyx_obj_4grpc_7_cython_6cygrpc_Channel *)__pyx_v_self));

  /* function exit code */
  __Pyx_RefNannyFinishContext();
  return __pyx_r;
}

static PyObject *__pyx_pf_4grpc_7_cython_6cygrpc_7Channel_2target(struct __pyx_obj_4grpc_7_cython_6cygrpc_Channel *__pyx_v_self) {
  char *__pyx_v_c_target;
  PyObject *__pyx_v_target = NULL;
  PyObject *__pyx_r = NULL;
  __Pyx_RefNannyDeclarations
  PyObject *__pyx_t_1 = NULL;
  PyObject *__pyx_t_2 = NULL;
  PyObject *__pyx_t_3 = NULL;
  PyObject *__pyx_t_4 = NULL;
  PyObject *__pyx_t_5 = NULL;
  PyObject *__pyx_t_6 = NULL;
  PyObject *__pyx_t_7 = NULL;
  PyObject *__pyx_t_8 = NULL;
  PyObject *__pyx_t_9 = NULL;
  int __pyx_t_10;
  int __pyx_t_11;
  __Pyx_RefNannySetupContext("target", 0);

  /* "src/python/grpcio/grpc/_cython/_cygrpc/channel.pyx.pxi":442
 *   def target(self):
 *     cdef char *c_target
 *     with self._state.condition:             # <<<<<<<<<<<<<<
 *       c_target = grpc_channel_get_target(self._state.c_channel)
 *       target = <bytes>c_target
 */
  /*with:*/ {
    __pyx_t_1 = __Pyx_PyObject_LookupSpecial(__pyx_v_self->_state->condition, __pyx_n_s_exit); if (unlikely(!__pyx_t_1)) __PYX_ERR(8, 442, __pyx_L1_error)
    __Pyx_GOTREF(__pyx_t_1);
    __pyx_t_3 = __Pyx_PyObject_LookupSpecial(__pyx_v_self->_state->condition, __pyx_n_s_enter); if (unlikely(!__pyx_t_3)) __PYX_ERR(8, 442, __pyx_L3_error)
    __Pyx_GOTREF(__pyx_t_3);
    __pyx_t_4 = NULL;
    if (CYTHON_UNPACK_METHODS && likely(PyMethod_Check(__pyx_t_3))) {
      __pyx_t_4 = PyMethod_GET_SELF(__pyx_t_3);
      if (likely(__pyx_t_4)) {
        PyObject* function = PyMethod_GET_FUNCTION(__pyx_t_3);
        __Pyx_INCREF(__pyx_t_4);
        __Pyx_INCREF(function);
        __Pyx_DECREF_SET(__pyx_t_3, function);
      }
    }
    if (__pyx_t_4) {
      __pyx_t_2 = __Pyx_PyObject_CallOneArg(__pyx_t_3, __pyx_t_4); if (unlikely(!__pyx_t_2)) __PYX_ERR(8, 442, __pyx_L3_error)
      __Pyx_DECREF(__pyx_t_4); __pyx_t_4 = 0;
    } else {
      __pyx_t_2 = __Pyx_PyObject_CallNoArg(__pyx_t_3); if (unlikely(!__pyx_t_2)) __PYX_ERR(8, 442, __pyx_L3_error)
    }
    __Pyx_GOTREF(__pyx_t_2);
    __Pyx_DECREF(__pyx_t_3); __pyx_t_3 = 0;
    __Pyx_DECREF(__pyx_t_2); __pyx_t_2 = 0;
    /*try:*/ {
      {
        __Pyx_PyThreadState_declare
        __Pyx_PyThreadState_assign
        __Pyx_ExceptionSave(&__pyx_t_5, &__pyx_t_6, &__pyx_t_7);
        __Pyx_XGOTREF(__pyx_t_5);
        __Pyx_XGOTREF(__pyx_t_6);
        __Pyx_XGOTREF(__pyx_t_7);
        /*try:*/ {

          /* "src/python/grpcio/grpc/_cython/_cygrpc/channel.pyx.pxi":443
 *     cdef char *c_target
 *     with self._state.condition:
 *       c_target = grpc_channel_get_target(self._state.c_channel)             # <<<<<<<<<<<<<<
 *       target = <bytes>c_target
 *       gpr_free(c_target)
 */
          __pyx_v_c_target = grpc_channel_get_target(__pyx_v_self->_state->c_channel);

          /* "src/python/grpcio/grpc/_cython/_cygrpc/channel.pyx.pxi":444
 *     with self._state.condition:
 *       c_target = grpc_channel_get_target(self._state.c_channel)
 *       target = <bytes>c_target             # <<<<<<<<<<<<<<
 *       gpr_free(c_target)
 *       return target
 */
          __pyx_t_2 = __Pyx_PyBytes_FromString(__pyx_v_c_target); if (unlikely(!__pyx_t_2)) __PYX_ERR(8, 444, __pyx_L7_error)
          __Pyx_GOTREF(__pyx_t_2);
          __pyx_t_3 = __pyx_t_2;
          __Pyx_INCREF(__pyx_t_3);
          __Pyx_DECREF(__pyx_t_2); __pyx_t_2 = 0;
          __pyx_v_target = ((PyObject*)__pyx_t_3);
          __pyx_t_3 = 0;

          /* "src/python/grpcio/grpc/_cython/_cygrpc/channel.pyx.pxi":445
 *       c_target = grpc_channel_get_target(self._state.c_channel)
 *       target = <bytes>c_target
 *       gpr_free(c_target)             # <<<<<<<<<<<<<<
 *       return target
 * 
 */
          gpr_free(__pyx_v_c_target);

          /* "src/python/grpcio/grpc/_cython/_cygrpc/channel.pyx.pxi":446
 *       target = <bytes>c_target
 *       gpr_free(c_target)
 *       return target             # <<<<<<<<<<<<<<
 * 
 *   def integrated_call(
 */
          __Pyx_XDECREF(__pyx_r);
          __Pyx_INCREF(__pyx_v_target);
          __pyx_r = __pyx_v_target;
          goto __pyx_L11_try_return;

          /* "src/python/grpcio/grpc/_cython/_cygrpc/channel.pyx.pxi":442
 *   def target(self):
 *     cdef char *c_target
 *     with self._state.condition:             # <<<<<<<<<<<<<<
 *       c_target = grpc_channel_get_target(self._state.c_channel)
 *       target = <bytes>c_target
 */
        }
        __pyx_L7_error:;
        __Pyx_XDECREF(__pyx_t_4); __pyx_t_4 = 0;
        __Pyx_XDECREF(__pyx_t_2); __pyx_t_2 = 0;
        __Pyx_XDECREF(__pyx_t_3); __pyx_t_3 = 0;
        /*except:*/ {
          __Pyx_AddTraceback("grpc._cython.cygrpc.Channel.target", __pyx_clineno, __pyx_lineno, __pyx_filename);
          if (__Pyx_GetException(&__pyx_t_3, &__pyx_t_2, &__pyx_t_4) < 0) __PYX_ERR(8, 442, __pyx_L9_except_error)
          __Pyx_GOTREF(__pyx_t_3);
          __Pyx_GOTREF(__pyx_t_2);
          __Pyx_GOTREF(__pyx_t_4);
          __pyx_t_8 = PyTuple_Pack(3, __pyx_t_3, __pyx_t_2, __pyx_t_4); if (unlikely(!__pyx_t_8)) __PYX_ERR(8, 442, __pyx_L9_except_error)
          __Pyx_GOTREF(__pyx_t_8);
          __pyx_t_9 = __Pyx_PyObject_Call(__pyx_t_1, __pyx_t_8, NULL);
          __Pyx_DECREF(__pyx_t_1); __pyx_t_1 = 0;
          __Pyx_DECREF(__pyx_t_8); __pyx_t_8 = 0;
          if (unlikely(!__pyx_t_9)) __PYX_ERR(8, 442, __pyx_L9_except_error)
          __Pyx_GOTREF(__pyx_t_9);
          __pyx_t_10 = __Pyx_PyObject_IsTrue(__pyx_t_9);
          __Pyx_DECREF(__pyx_t_9); __pyx_t_9 = 0;
          if (__pyx_t_10 < 0) __PYX_ERR(8, 442, __pyx_L9_except_error)
          __pyx_t_11 = ((!(__pyx_t_10 != 0)) != 0);
          if (__pyx_t_11) {
            __Pyx_GIVEREF(__pyx_t_3);
            __Pyx_GIVEREF(__pyx_t_2);
            __Pyx_XGIVEREF(__pyx_t_4);
            __Pyx_ErrRestoreWithState(__pyx_t_3, __pyx_t_2, __pyx_t_4);
            __pyx_t_3 = 0; __pyx_t_2 = 0; __pyx_t_4 = 0; 
            __PYX_ERR(8, 442, __pyx_L9_except_error)
          }
          __Pyx_DECREF(__pyx_t_3); __pyx_t_3 = 0;
          __Pyx_DECREF(__pyx_t_2); __pyx_t_2 = 0;
          __Pyx_DECREF(__pyx_t_4); __pyx_t_4 = 0;
          goto __pyx_L8_exception_handled;
        }
        __pyx_L9_except_error:;
        __Pyx_XGIVEREF(__pyx_t_5);
        __Pyx_XGIVEREF(__pyx_t_6);
        __Pyx_XGIVEREF(__pyx_t_7);
        __Pyx_ExceptionReset(__pyx_t_5, __pyx_t_6, __pyx_t_7);
        goto __pyx_L1_error;
        __pyx_L11_try_return:;
        __Pyx_XGIVEREF(__pyx_t_5);
        __Pyx_XGIVEREF(__pyx_t_6);
        __Pyx_XGIVEREF(__pyx_t_7);
        __Pyx_ExceptionReset(__pyx_t_5, __pyx_t_6, __pyx_t_7);
        goto __pyx_L4_return;
        __pyx_L8_exception_handled:;
        __Pyx_XGIVEREF(__pyx_t_5);
        __Pyx_XGIVEREF(__pyx_t_6);
        __Pyx_XGIVEREF(__pyx_t_7);
        __Pyx_ExceptionReset(__pyx_t_5, __pyx_t_6, __pyx_t_7);
      }
    }
    /*finally:*/ {
      /*normal exit:*/{
        if (__pyx_t_1) {
          __pyx_t_7 = __Pyx_PyObject_Call(__pyx_t_1, __pyx_tuple__42, NULL);
          __Pyx_DECREF(__pyx_t_1); __pyx_t_1 = 0;
          if (unlikely(!__pyx_t_7)) __PYX_ERR(8, 442, __pyx_L1_error)
          __Pyx_GOTREF(__pyx_t_7);
          __Pyx_DECREF(__pyx_t_7); __pyx_t_7 = 0;
        }
        goto __pyx_L6;
      }
      __pyx_L4_return: {
        __pyx_t_7 = __pyx_r;
        __pyx_r = 0;
        if (__pyx_t_1) {
          __pyx_t_6 = __Pyx_PyObject_Call(__pyx_t_1, __pyx_tuple__43, NULL);
          __Pyx_DECREF(__pyx_t_1); __pyx_t_1 = 0;
          if (unlikely(!__pyx_t_6)) __PYX_ERR(8, 442, __pyx_L1_error)
          __Pyx_GOTREF(__pyx_t_6);
          __Pyx_DECREF(__pyx_t_6); __pyx_t_6 = 0;
        }
        __pyx_r = __pyx_t_7;
        __pyx_t_7 = 0;
        goto __pyx_L0;
      }
      __pyx_L6:;
    }
    goto __pyx_L16;
    __pyx_L3_error:;
    __Pyx_DECREF(__pyx_t_1); __pyx_t_1 = 0;
    goto __pyx_L1_error;
    __pyx_L16:;
  }

  /* "src/python/grpcio/grpc/_cython/_cygrpc/channel.pyx.pxi":440
 *       grpc_channel_credentials_release(c_channel_credentials)
 * 
 *   def target(self):             # <<<<<<<<<<<<<<
 *     cdef char *c_target
 *     with self._state.condition:
 */

  /* function exit code */
  __pyx_r = Py_None; __Pyx_INCREF(Py_None);
  goto __pyx_L0;
  __pyx_L1_error:;
  __Pyx_XDECREF(__pyx_t_2);
  __Pyx_XDECREF(__pyx_t_3);
  __Pyx_XDECREF(__pyx_t_4);
  __Pyx_XDECREF(__pyx_t_8);
  __Pyx_AddTraceback("grpc._cython.cygrpc.Channel.target", __pyx_clineno, __pyx_lineno, __pyx_filename);
  __pyx_r = NULL;
  __pyx_L0:;
  __Pyx_XDECREF(__pyx_v_target);
  __Pyx_XGIVEREF(__pyx_r);
  __Pyx_RefNannyFinishContext();
  return __pyx_r;
}

/* "src/python/grpcio/grpc/_cython/_cygrpc/channel.pyx.pxi":448
 *       return target
 * 
 *   def integrated_call(             # <<<<<<<<<<<<<<
 *       self, int flags, method, host, object deadline, object metadata,
 *       CallCredentials credentials, operationses_and_tags,
 */

/* Python wrapper */
static PyObject *__pyx_pw_4grpc_7_cython_6cygrpc_7Channel_5integrated_call(PyObject *__pyx_v_self, PyObject *__pyx_args, PyObject *__pyx_kwds); /*proto*/
static PyObject *__pyx_pw_4grpc_7_cython_6cygrpc_7Channel_5integrated_call(PyObject *__pyx_v_self, PyObject *__pyx_args, PyObject *__pyx_kwds) {
  int __pyx_v_flags;
  PyObject *__pyx_v_method = 0;
  PyObject *__pyx_v_host = 0;
  PyObject *__pyx_v_deadline = 0;
  PyObject *__pyx_v_metadata = 0;
  struct __pyx_obj_4grpc_7_cython_6cygrpc_CallCredentials *__pyx_v_credentials = 0;
  PyObject *__pyx_v_operationses_and_tags = 0;
  PyObject *__pyx_v_context = 0;
  PyObject *__pyx_r = 0;
  __Pyx_RefNannyDeclarations
  __Pyx_RefNannySetupContext("integrated_call (wrapper)", 0);
  {
    static PyObject **__pyx_pyargnames[] = {&__pyx_n_s_flags,&__pyx_n_s_method,&__pyx_n_s_host,&__pyx_n_s_deadline,&__pyx_n_s_metadata,&__pyx_n_s_credentials,&__pyx_n_s_operationses_and_tags,&__pyx_n_s_context,0};
    PyObject* values[8] = {0,0,0,0,0,0,0,0};

    /* "src/python/grpcio/grpc/_cython/_cygrpc/channel.pyx.pxi":451
 *       self, int flags, method, host, object deadline, object metadata,
 *       CallCredentials credentials, operationses_and_tags,
 *       object context = None):             # <<<<<<<<<<<<<<
 *     return _integrated_call(
 *         self._state, flags, method, host, deadline, metadata, credentials,
 */
    values[7] = ((PyObject *)Py_None);
    if (unlikely(__pyx_kwds)) {
      Py_ssize_t kw_args;
      const Py_ssize_t pos_args = PyTuple_GET_SIZE(__pyx_args);
      switch (pos_args) {
        case  8: values[7] = PyTuple_GET_ITEM(__pyx_args, 7);
        CYTHON_FALLTHROUGH;
        case  7: values[6] = PyTuple_GET_ITEM(__pyx_args, 6);
        CYTHON_FALLTHROUGH;
        case  6: values[5] = PyTuple_GET_ITEM(__pyx_args, 5);
        CYTHON_FALLTHROUGH;
        case  5: values[4] = PyTuple_GET_ITEM(__pyx_args, 4);
        CYTHON_FALLTHROUGH;
        case  4: values[3] = PyTuple_GET_ITEM(__pyx_args, 3);
        CYTHON_FALLTHROUGH;
        case  3: values[2] = PyTuple_GET_ITEM(__pyx_args, 2);
        CYTHON_FALLTHROUGH;
        case  2: values[1] = PyTuple_GET_ITEM(__pyx_args, 1);
        CYTHON_FALLTHROUGH;
        case  1: values[0] = PyTuple_GET_ITEM(__pyx_args, 0);
        CYTHON_FALLTHROUGH;
        case  0: break;
        default: goto __pyx_L5_argtuple_error;
      }
      kw_args = PyDict_Size(__pyx_kwds);
      switch (pos_args) {
        case  0:
        if (likely((values[0] = __Pyx_PyDict_GetItemStr(__pyx_kwds, __pyx_n_s_flags)) != 0)) kw_args--;
        else goto __pyx_L5_argtuple_error;
        CYTHON_FALLTHROUGH;
        case  1:
        if (likely((values[1] = __Pyx_PyDict_GetItemStr(__pyx_kwds, __pyx_n_s_method)) != 0)) kw_args--;
        else {
          __Pyx_RaiseArgtupleInvalid("integrated_call", 0, 7, 8, 1); __PYX_ERR(8, 448, __pyx_L3_error)
        }
        CYTHON_FALLTHROUGH;
        case  2:
        if (likely((values[2] = __Pyx_PyDict_GetItemStr(__pyx_kwds, __pyx_n_s_host)) != 0)) kw_args--;
        else {
          __Pyx_RaiseArgtupleInvalid("integrated_call", 0, 7, 8, 2); __PYX_ERR(8, 448, __pyx_L3_error)
        }
        CYTHON_FALLTHROUGH;
        case  3:
        if (likely((values[3] = __Pyx_PyDict_GetItemStr(__pyx_kwds, __pyx_n_s_deadline)) != 0)) kw_args--;
        else {
          __Pyx_RaiseArgtupleInvalid("integrated_call", 0, 7, 8, 3); __PYX_ERR(8, 448, __pyx_L3_error)
        }
        CYTHON_FALLTHROUGH;
        case  4:
        if (likely((values[4] = __Pyx_PyDict_GetItemStr(__pyx_kwds, __pyx_n_s_metadata)) != 0)) kw_args--;
        else {
          __Pyx_RaiseArgtupleInvalid("integrated_call", 0, 7, 8, 4); __PYX_ERR(8, 448, __pyx_L3_error)
        }
        CYTHON_FALLTHROUGH;
        case  5:
        if (likely((values[5] = __Pyx_PyDict_GetItemStr(__pyx_kwds, __pyx_n_s_credentials)) != 0)) kw_args--;
        else {
          __Pyx_RaiseArgtupleInvalid("integrated_call", 0, 7, 8, 5); __PYX_ERR(8, 448, __pyx_L3_error)
        }
        CYTHON_FALLTHROUGH;
        case  6:
        if (likely((values[6] = __Pyx_PyDict_GetItemStr(__pyx_kwds, __pyx_n_s_operationses_and_tags)) != 0)) kw_args--;
        else {
          __Pyx_RaiseArgtupleInvalid("integrated_call", 0, 7, 8, 6); __PYX_ERR(8, 448, __pyx_L3_error)
        }
        CYTHON_FALLTHROUGH;
        case  7:
        if (kw_args > 0) {
          PyObject* value = __Pyx_PyDict_GetItemStr(__pyx_kwds, __pyx_n_s_context);
          if (value) { values[7] = value; kw_args--; }
        }
      }
      if (unlikely(kw_args > 0)) {
        if (unlikely(__Pyx_ParseOptionalKeywords(__pyx_kwds, __pyx_pyargnames, 0, values, pos_args, "integrated_call") < 0)) __PYX_ERR(8, 448, __pyx_L3_error)
      }
    } else {
      switch (PyTuple_GET_SIZE(__pyx_args)) {
        case  8: values[7] = PyTuple_GET_ITEM(__pyx_args, 7);
        CYTHON_FALLTHROUGH;
        case  7: values[6] = PyTuple_GET_ITEM(__pyx_args, 6);
        values[5] = PyTuple_GET_ITEM(__pyx_args, 5);
        values[4] = PyTuple_GET_ITEM(__pyx_args, 4);
        values[3] = PyTuple_GET_ITEM(__pyx_args, 3);
        values[2] = PyTuple_GET_ITEM(__pyx_args, 2);
        values[1] = PyTuple_GET_ITEM(__pyx_args, 1);
        values[0] = PyTuple_GET_ITEM(__pyx_args, 0);
        break;
        default: goto __pyx_L5_argtuple_error;
      }
    }
    __pyx_v_flags = __Pyx_PyInt_As_int(values[0]); if (unlikely((__pyx_v_flags == (int)-1) && PyErr_Occurred())) __PYX_ERR(8, 449, __pyx_L3_error)
    __pyx_v_method = values[1];
    __pyx_v_host = values[2];
    __pyx_v_deadline = values[3];
    __pyx_v_metadata = values[4];
    __pyx_v_credentials = ((struct __pyx_obj_4grpc_7_cython_6cygrpc_CallCredentials *)values[5]);
    __pyx_v_operationses_and_tags = values[6];
    __pyx_v_context = values[7];
  }
  goto __pyx_L4_argument_unpacking_done;
  __pyx_L5_argtuple_error:;
  __Pyx_RaiseArgtupleInvalid("integrated_call", 0, 7, 8, PyTuple_GET_SIZE(__pyx_args)); __PYX_ERR(8, 448, __pyx_L3_error)
  __pyx_L3_error:;
  __Pyx_AddTraceback("grpc._cython.cygrpc.Channel.integrated_call", __pyx_clineno, __pyx_lineno, __pyx_filename);
  __Pyx_RefNannyFinishContext();
  return NULL;
  __pyx_L4_argument_unpacking_done:;
  if (unlikely(!__Pyx_ArgTypeTest(((PyObject *)__pyx_v_credentials), __pyx_ptype_4grpc_7_cython_6cygrpc_CallCredentials, 1, "credentials", 0))) __PYX_ERR(8, 450, __pyx_L1_error)
  __pyx_r = __pyx_pf_4grpc_7_cython_6cygrpc_7Channel_4integrated_call(((struct __pyx_obj_4grpc_7_cython_6cygrpc_Channel *)__pyx_v_self), __pyx_v_flags, __pyx_v_method, __pyx_v_host, __pyx_v_deadline, __pyx_v_metadata, __pyx_v_credentials, __pyx_v_operationses_and_tags, __pyx_v_context);

  /* "src/python/grpcio/grpc/_cython/_cygrpc/channel.pyx.pxi":448
 *       return target
 * 
 *   def integrated_call(             # <<<<<<<<<<<<<<
 *       self, int flags, method, host, object deadline, object metadata,
 *       CallCredentials credentials, operationses_and_tags,
 */

  /* function exit code */
  goto __pyx_L0;
  __pyx_L1_error:;
  __pyx_r = NULL;
  __pyx_L0:;
  __Pyx_RefNannyFinishContext();
  return __pyx_r;
}

static PyObject *__pyx_pf_4grpc_7_cython_6cygrpc_7Channel_4integrated_call(struct __pyx_obj_4grpc_7_cython_6cygrpc_Channel *__pyx_v_self, int __pyx_v_flags, PyObject *__pyx_v_method, PyObject *__pyx_v_host, PyObject *__pyx_v_deadline, PyObject *__pyx_v_metadata, struct __pyx_obj_4grpc_7_cython_6cygrpc_CallCredentials *__pyx_v_credentials, PyObject *__pyx_v_operationses_and_tags, PyObject *__pyx_v_context) {
  PyObject *__pyx_r = NULL;
  __Pyx_RefNannyDeclarations
  PyObject *__pyx_t_1 = NULL;
  PyObject *__pyx_t_2 = NULL;
  __Pyx_RefNannySetupContext("integrated_call", 0);

  /* "src/python/grpcio/grpc/_cython/_cygrpc/channel.pyx.pxi":452
 *       CallCredentials credentials, operationses_and_tags,
 *       object context = None):
 *     return _integrated_call(             # <<<<<<<<<<<<<<
 *         self._state, flags, method, host, deadline, metadata, credentials,
 *         operationses_and_tags, context)
 */
  __Pyx_XDECREF(__pyx_r);

  /* "src/python/grpcio/grpc/_cython/_cygrpc/channel.pyx.pxi":453
 *       object context = None):
 *     return _integrated_call(
 *         self._state, flags, method, host, deadline, metadata, credentials,             # <<<<<<<<<<<<<<
 *         operationses_and_tags, context)
 * 
 */
  __pyx_t_1 = ((PyObject *)__pyx_v_self->_state);
  __Pyx_INCREF(__pyx_t_1);

  /* "src/python/grpcio/grpc/_cython/_cygrpc/channel.pyx.pxi":452
 *       CallCredentials credentials, operationses_and_tags,
 *       object context = None):
 *     return _integrated_call(             # <<<<<<<<<<<<<<
 *         self._state, flags, method, host, deadline, metadata, credentials,
 *         operationses_and_tags, context)
 */
  __pyx_t_2 = ((PyObject *)__pyx_f_4grpc_7_cython_6cygrpc__integrated_call(((struct __pyx_obj_4grpc_7_cython_6cygrpc__ChannelState *)__pyx_t_1), __pyx_v_flags, __pyx_v_method, __pyx_v_host, __pyx_v_deadline, __pyx_v_metadata, __pyx_v_credentials, __pyx_v_operationses_and_tags, __pyx_v_context)); if (unlikely(!__pyx_t_2)) __PYX_ERR(8, 452, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_2);
  __Pyx_DECREF(__pyx_t_1); __pyx_t_1 = 0;
  __pyx_r = __pyx_t_2;
  __pyx_t_2 = 0;
  goto __pyx_L0;

  /* "src/python/grpcio/grpc/_cython/_cygrpc/channel.pyx.pxi":448
 *       return target
 * 
 *   def integrated_call(             # <<<<<<<<<<<<<<
 *       self, int flags, method, host, object deadline, object metadata,
 *       CallCredentials credentials, operationses_and_tags,
 */

  /* function exit code */
  __pyx_L1_error:;
  __Pyx_XDECREF(__pyx_t_1);
  __Pyx_XDECREF(__pyx_t_2);
  __Pyx_AddTraceback("grpc._cython.cygrpc.Channel.integrated_call", __pyx_clineno, __pyx_lineno, __pyx_filename);
  __pyx_r = NULL;
  __pyx_L0:;
  __Pyx_XGIVEREF(__pyx_r);
  __Pyx_RefNannyFinishContext();
  return __pyx_r;
}

/* "src/python/grpcio/grpc/_cython/_cygrpc/channel.pyx.pxi":456
 *         operationses_and_tags, context)
 * 
 *   def next_call_event(self):             # <<<<<<<<<<<<<<
 *     def on_success(tag):
 *       if tag is not None:
 */

/* Python wrapper */
static PyObject *__pyx_pw_4grpc_7_cython_6cygrpc_7Channel_7next_call_event(PyObject *__pyx_v_self, CYTHON_UNUSED PyObject *unused); /*proto*/
static PyObject *__pyx_pw_4grpc_7_cython_6cygrpc_7Channel_7next_call_event(PyObject *__pyx_v_self, CYTHON_UNUSED PyObject *unused) {
  PyObject *__pyx_r = 0;
  __Pyx_RefNannyDeclarations
  __Pyx_RefNannySetupContext("next_call_event (wrapper)", 0);
  __pyx_r = __pyx_pf_4grpc_7_cython_6cygrpc_7Channel_6next_call_event(((struct __pyx_obj_4grpc_7_cython_6cygrpc_Channel *)__pyx_v_self));

  /* function exit code */
  __Pyx_RefNannyFinishContext();
  return __pyx_r;
}

/* "src/python/grpcio/grpc/_cython/_cygrpc/channel.pyx.pxi":457
 * 
 *   def next_call_event(self):
 *     def on_success(tag):             # <<<<<<<<<<<<<<
 *       if tag is not None:
 *         _process_integrated_call_tag(self._state, tag)
 */

/* Python wrapper */
static PyObject *__pyx_pw_4grpc_7_cython_6cygrpc_7Channel_15next_call_event_1on_success(PyObject *__pyx_self, PyObject *__pyx_v_tag); /*proto*/
static PyMethodDef __pyx_mdef_4grpc_7_cython_6cygrpc_7Channel_15next_call_event_1on_success = {"on_success", (PyCFunction)__pyx_pw_4grpc_7_cython_6cygrpc_7Channel_15next_call_event_1on_success, METH_O, 0};
static PyObject *__pyx_pw_4grpc_7_cython_6cygrpc_7Channel_15next_call_event_1on_success(PyObject *__pyx_self, PyObject *__pyx_v_tag) {
  PyObject *__pyx_r = 0;
  __Pyx_RefNannyDeclarations
  __Pyx_RefNannySetupContext("on_success (wrapper)", 0);
  __pyx_r = __pyx_pf_4grpc_7_cython_6cygrpc_7Channel_15next_call_event_on_success(__pyx_self, ((PyObject *)__pyx_v_tag));

  /* function exit code */
  __Pyx_RefNannyFinishContext();
  return __pyx_r;
}

static PyObject *__pyx_pf_4grpc_7_cython_6cygrpc_7Channel_15next_call_event_on_success(PyObject *__pyx_self, PyObject *__pyx_v_tag) {
  struct __pyx_obj_4grpc_7_cython_6cygrpc___pyx_scope_struct_3_next_call_event *__pyx_cur_scope;
  struct __pyx_obj_4grpc_7_cython_6cygrpc___pyx_scope_struct_3_next_call_event *__pyx_outer_scope;
  PyObject *__pyx_r = NULL;
  __Pyx_RefNannyDeclarations
  int __pyx_t_1;
  int __pyx_t_2;
  PyObject *__pyx_t_3 = NULL;
  __Pyx_RefNannySetupContext("on_success", 0);
  __pyx_outer_scope = (struct __pyx_obj_4grpc_7_cython_6cygrpc___pyx_scope_struct_3_next_call_event *) __Pyx_CyFunction_GetClosure(__pyx_self);
  __pyx_cur_scope = __pyx_outer_scope;

  /* "src/python/grpcio/grpc/_cython/_cygrpc/channel.pyx.pxi":458
 *   def next_call_event(self):
 *     def on_success(tag):
 *       if tag is not None:             # <<<<<<<<<<<<<<
 *         _process_integrated_call_tag(self._state, tag)
 *     if is_fork_support_enabled():
 */
  __pyx_t_1 = (__pyx_v_tag != Py_None);
  __pyx_t_2 = (__pyx_t_1 != 0);
  if (__pyx_t_2) {

    /* "src/python/grpcio/grpc/_cython/_cygrpc/channel.pyx.pxi":459
 *     def on_success(tag):
 *       if tag is not None:
 *         _process_integrated_call_tag(self._state, tag)             # <<<<<<<<<<<<<<
 *     if is_fork_support_enabled():
 *       queue_deadline = time.time() + 1.0
 */
    if (unlikely(!__pyx_cur_scope->__pyx_v_self)) { __Pyx_RaiseClosureNameError("self"); __PYX_ERR(8, 459, __pyx_L1_error) }
    __pyx_t_3 = ((PyObject *)__pyx_cur_scope->__pyx_v_self->_state);
    __Pyx_INCREF(__pyx_t_3);
    if (!(likely(((__pyx_v_tag) == Py_None) || likely(__Pyx_TypeTest(__pyx_v_tag, __pyx_ptype_4grpc_7_cython_6cygrpc__BatchOperationTag))))) __PYX_ERR(8, 459, __pyx_L1_error)
    __pyx_f_4grpc_7_cython_6cygrpc__process_integrated_call_tag(((struct __pyx_obj_4grpc_7_cython_6cygrpc__ChannelState *)__pyx_t_3), ((struct __pyx_obj_4grpc_7_cython_6cygrpc__BatchOperationTag *)__pyx_v_tag)); if (unlikely(PyErr_Occurred())) __PYX_ERR(8, 459, __pyx_L1_error)
    __Pyx_DECREF(__pyx_t_3); __pyx_t_3 = 0;

    /* "src/python/grpcio/grpc/_cython/_cygrpc/channel.pyx.pxi":458
 *   def next_call_event(self):
 *     def on_success(tag):
 *       if tag is not None:             # <<<<<<<<<<<<<<
 *         _process_integrated_call_tag(self._state, tag)
 *     if is_fork_support_enabled():
 */
  }

  /* "src/python/grpcio/grpc/_cython/_cygrpc/channel.pyx.pxi":457
 * 
 *   def next_call_event(self):
 *     def on_success(tag):             # <<<<<<<<<<<<<<
 *       if tag is not None:
 *         _process_integrated_call_tag(self._state, tag)
 */

  /* function exit code */
  __pyx_r = Py_None; __Pyx_INCREF(Py_None);
  goto __pyx_L0;
  __pyx_L1_error:;
  __Pyx_XDECREF(__pyx_t_3);
  __Pyx_AddTraceback("grpc._cython.cygrpc.Channel.next_call_event.on_success", __pyx_clineno, __pyx_lineno, __pyx_filename);
  __pyx_r = NULL;
  __pyx_L0:;
  __Pyx_XGIVEREF(__pyx_r);
  __Pyx_RefNannyFinishContext();
  return __pyx_r;
}

/* "src/python/grpcio/grpc/_cython/_cygrpc/channel.pyx.pxi":456
 *         operationses_and_tags, context)
 * 
 *   def next_call_event(self):             # <<<<<<<<<<<<<<
 *     def on_success(tag):
 *       if tag is not None:
 */

static PyObject *__pyx_pf_4grpc_7_cython_6cygrpc_7Channel_6next_call_event(struct __pyx_obj_4grpc_7_cython_6cygrpc_Channel *__pyx_v_self) {
  struct __pyx_obj_4grpc_7_cython_6cygrpc___pyx_scope_struct_3_next_call_event *__pyx_cur_scope;
  PyObject *__pyx_v_on_success = 0;
  PyObject *__pyx_v_queue_deadline = NULL;
  PyObject *__pyx_r = NULL;
  __Pyx_RefNannyDeclarations
  PyObject *__pyx_t_1 = NULL;
  PyObject *__pyx_t_2 = NULL;
  PyObject *__pyx_t_3 = NULL;
  int __pyx_t_4;
  __Pyx_RefNannySetupContext("next_call_event", 0);
  __pyx_cur_scope = (struct __pyx_obj_4grpc_7_cython_6cygrpc___pyx_scope_struct_3_next_call_event *)__pyx_tp_new_4grpc_7_cython_6cygrpc___pyx_scope_struct_3_next_call_event(__pyx_ptype_4grpc_7_cython_6cygrpc___pyx_scope_struct_3_next_call_event, __pyx_empty_tuple, NULL);
  if (unlikely(!__pyx_cur_scope)) {
    __pyx_cur_scope = ((struct __pyx_obj_4grpc_7_cython_6cygrpc___pyx_scope_struct_3_next_call_event *)Py_None);
    __Pyx_INCREF(Py_None);
    __PYX_ERR(8, 456, __pyx_L1_error)
  } else {
    __Pyx_GOTREF(__pyx_cur_scope);
  }
  __pyx_cur_scope->__pyx_v_self = __pyx_v_self;
  __Pyx_INCREF((PyObject *)__pyx_cur_scope->__pyx_v_self);
  __Pyx_GIVEREF((PyObject *)__pyx_cur_scope->__pyx_v_self);

  /* "src/python/grpcio/grpc/_cython/_cygrpc/channel.pyx.pxi":457
 * 
 *   def next_call_event(self):
 *     def on_success(tag):             # <<<<<<<<<<<<<<
 *       if tag is not None:
 *         _process_integrated_call_tag(self._state, tag)
 */
  __pyx_t_1 = __Pyx_CyFunction_NewEx(&__pyx_mdef_4grpc_7_cython_6cygrpc_7Channel_15next_call_event_1on_success, 0, __pyx_n_s_next_call_event_locals_on_succes, ((PyObject*)__pyx_cur_scope), __pyx_n_s_grpc__cython_cygrpc, __pyx_d, ((PyObject *)__pyx_codeobj__45)); if (unlikely(!__pyx_t_1)) __PYX_ERR(8, 457, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_1);
  __pyx_v_on_success = __pyx_t_1;
  __pyx_t_1 = 0;

  /* "src/python/grpcio/grpc/_cython/_cygrpc/channel.pyx.pxi":460
 *       if tag is not None:
 *         _process_integrated_call_tag(self._state, tag)
 *     if is_fork_support_enabled():             # <<<<<<<<<<<<<<
 *       queue_deadline = time.time() + 1.0
 *     else:
 */
  __pyx_t_2 = __Pyx_GetModuleGlobalName(__pyx_n_s_is_fork_support_enabled); if (unlikely(!__pyx_t_2)) __PYX_ERR(8, 460, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_2);
  __pyx_t_3 = NULL;
  if (CYTHON_UNPACK_METHODS && unlikely(PyMethod_Check(__pyx_t_2))) {
    __pyx_t_3 = PyMethod_GET_SELF(__pyx_t_2);
    if (likely(__pyx_t_3)) {
      PyObject* function = PyMethod_GET_FUNCTION(__pyx_t_2);
      __Pyx_INCREF(__pyx_t_3);
      __Pyx_INCREF(function);
      __Pyx_DECREF_SET(__pyx_t_2, function);
    }
  }
  if (__pyx_t_3) {
    __pyx_t_1 = __Pyx_PyObject_CallOneArg(__pyx_t_2, __pyx_t_3); if (unlikely(!__pyx_t_1)) __PYX_ERR(8, 460, __pyx_L1_error)
    __Pyx_DECREF(__pyx_t_3); __pyx_t_3 = 0;
  } else {
    __pyx_t_1 = __Pyx_PyObject_CallNoArg(__pyx_t_2); if (unlikely(!__pyx_t_1)) __PYX_ERR(8, 460, __pyx_L1_error)
  }
  __Pyx_GOTREF(__pyx_t_1);
  __Pyx_DECREF(__pyx_t_2); __pyx_t_2 = 0;
  __pyx_t_4 = __Pyx_PyObject_IsTrue(__pyx_t_1); if (unlikely(__pyx_t_4 < 0)) __PYX_ERR(8, 460, __pyx_L1_error)
  __Pyx_DECREF(__pyx_t_1); __pyx_t_1 = 0;
  if (__pyx_t_4) {

    /* "src/python/grpcio/grpc/_cython/_cygrpc/channel.pyx.pxi":461
 *         _process_integrated_call_tag(self._state, tag)
 *     if is_fork_support_enabled():
 *       queue_deadline = time.time() + 1.0             # <<<<<<<<<<<<<<
 *     else:
 *       queue_deadline = None
 */
    __pyx_t_2 = __Pyx_GetModuleGlobalName(__pyx_n_s_time); if (unlikely(!__pyx_t_2)) __PYX_ERR(8, 461, __pyx_L1_error)
    __Pyx_GOTREF(__pyx_t_2);
    __pyx_t_3 = __Pyx_PyObject_GetAttrStr(__pyx_t_2, __pyx_n_s_time); if (unlikely(!__pyx_t_3)) __PYX_ERR(8, 461, __pyx_L1_error)
    __Pyx_GOTREF(__pyx_t_3);
    __Pyx_DECREF(__pyx_t_2); __pyx_t_2 = 0;
    __pyx_t_2 = NULL;
    if (CYTHON_UNPACK_METHODS && unlikely(PyMethod_Check(__pyx_t_3))) {
      __pyx_t_2 = PyMethod_GET_SELF(__pyx_t_3);
      if (likely(__pyx_t_2)) {
        PyObject* function = PyMethod_GET_FUNCTION(__pyx_t_3);
        __Pyx_INCREF(__pyx_t_2);
        __Pyx_INCREF(function);
        __Pyx_DECREF_SET(__pyx_t_3, function);
      }
    }
    if (__pyx_t_2) {
      __pyx_t_1 = __Pyx_PyObject_CallOneArg(__pyx_t_3, __pyx_t_2); if (unlikely(!__pyx_t_1)) __PYX_ERR(8, 461, __pyx_L1_error)
      __Pyx_DECREF(__pyx_t_2); __pyx_t_2 = 0;
    } else {
      __pyx_t_1 = __Pyx_PyObject_CallNoArg(__pyx_t_3); if (unlikely(!__pyx_t_1)) __PYX_ERR(8, 461, __pyx_L1_error)
    }
    __Pyx_GOTREF(__pyx_t_1);
    __Pyx_DECREF(__pyx_t_3); __pyx_t_3 = 0;
    __pyx_t_3 = __Pyx_PyFloat_AddObjC(__pyx_t_1, __pyx_float_1_0, 1.0, 0); if (unlikely(!__pyx_t_3)) __PYX_ERR(8, 461, __pyx_L1_error)
    __Pyx_GOTREF(__pyx_t_3);
    __Pyx_DECREF(__pyx_t_1); __pyx_t_1 = 0;
    __pyx_v_queue_deadline = __pyx_t_3;
    __pyx_t_3 = 0;

    /* "src/python/grpcio/grpc/_cython/_cygrpc/channel.pyx.pxi":460
 *       if tag is not None:
 *         _process_integrated_call_tag(self._state, tag)
 *     if is_fork_support_enabled():             # <<<<<<<<<<<<<<
 *       queue_deadline = time.time() + 1.0
 *     else:
 */
    goto __pyx_L3;
  }

  /* "src/python/grpcio/grpc/_cython/_cygrpc/channel.pyx.pxi":463
 *       queue_deadline = time.time() + 1.0
 *     else:
 *       queue_deadline = None             # <<<<<<<<<<<<<<
 *     return _next_call_event(self._state, self._state.c_call_completion_queue,
 *                             on_success, queue_deadline)
 */
  /*else*/ {
    __Pyx_INCREF(Py_None);
    __pyx_v_queue_deadline = Py_None;
  }
  __pyx_L3:;

  /* "src/python/grpcio/grpc/_cython/_cygrpc/channel.pyx.pxi":464
 *     else:
 *       queue_deadline = None
 *     return _next_call_event(self._state, self._state.c_call_completion_queue,             # <<<<<<<<<<<<<<
 *                             on_success, queue_deadline)
 * 
 */
  __Pyx_XDECREF(__pyx_r);
  __pyx_t_3 = ((PyObject *)__pyx_cur_scope->__pyx_v_self->_state);
  __Pyx_INCREF(__pyx_t_3);

  /* "src/python/grpcio/grpc/_cython/_cygrpc/channel.pyx.pxi":465
 *       queue_deadline = None
 *     return _next_call_event(self._state, self._state.c_call_completion_queue,
 *                             on_success, queue_deadline)             # <<<<<<<<<<<<<<
 * 
 *   def segregated_call(
 */
  __pyx_t_1 = __pyx_f_4grpc_7_cython_6cygrpc__next_call_event(((struct __pyx_obj_4grpc_7_cython_6cygrpc__ChannelState *)__pyx_t_3), __pyx_cur_scope->__pyx_v_self->_state->c_call_completion_queue, __pyx_v_on_success, __pyx_v_queue_deadline); if (unlikely(!__pyx_t_1)) __PYX_ERR(8, 464, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_1);
  __Pyx_DECREF(__pyx_t_3); __pyx_t_3 = 0;
  __pyx_r = __pyx_t_1;
  __pyx_t_1 = 0;
  goto __pyx_L0;

  /* "src/python/grpcio/grpc/_cython/_cygrpc/channel.pyx.pxi":456
 *         operationses_and_tags, context)
 * 
 *   def next_call_event(self):             # <<<<<<<<<<<<<<
 *     def on_success(tag):
 *       if tag is not None:
 */

  /* function exit code */
  __pyx_L1_error:;
  __Pyx_XDECREF(__pyx_t_1);
  __Pyx_XDECREF(__pyx_t_2);
  __Pyx_XDECREF(__pyx_t_3);
  __Pyx_AddTraceback("grpc._cython.cygrpc.Channel.next_call_event", __pyx_clineno, __pyx_lineno, __pyx_filename);
  __pyx_r = NULL;
  __pyx_L0:;
  __Pyx_XDECREF(__pyx_v_on_success);
  __Pyx_XDECREF(__pyx_v_queue_deadline);
  __Pyx_DECREF(((PyObject *)__pyx_cur_scope));
  __Pyx_XGIVEREF(__pyx_r);
  __Pyx_RefNannyFinishContext();
  return __pyx_r;
}

/* "src/python/grpcio/grpc/_cython/_cygrpc/channel.pyx.pxi":467
 *                             on_success, queue_deadline)
 * 
 *   def segregated_call(             # <<<<<<<<<<<<<<
 *       self, int flags, method, host, object deadline, object metadata,
 *       CallCredentials credentials, operationses_and_tags,
 */

/* Python wrapper */
static PyObject *__pyx_pw_4grpc_7_cython_6cygrpc_7Channel_9segregated_call(PyObject *__pyx_v_self, PyObject *__pyx_args, PyObject *__pyx_kwds); /*proto*/
static PyObject *__pyx_pw_4grpc_7_cython_6cygrpc_7Channel_9segregated_call(PyObject *__pyx_v_self, PyObject *__pyx_args, PyObject *__pyx_kwds) {
  int __pyx_v_flags;
  PyObject *__pyx_v_method = 0;
  PyObject *__pyx_v_host = 0;
  PyObject *__pyx_v_deadline = 0;
  PyObject *__pyx_v_metadata = 0;
  struct __pyx_obj_4grpc_7_cython_6cygrpc_CallCredentials *__pyx_v_credentials = 0;
  PyObject *__pyx_v_operationses_and_tags = 0;
  PyObject *__pyx_v_context = 0;
  PyObject *__pyx_r = 0;
  __Pyx_RefNannyDeclarations
  __Pyx_RefNannySetupContext("segregated_call (wrapper)", 0);
  {
    static PyObject **__pyx_pyargnames[] = {&__pyx_n_s_flags,&__pyx_n_s_method,&__pyx_n_s_host,&__pyx_n_s_deadline,&__pyx_n_s_metadata,&__pyx_n_s_credentials,&__pyx_n_s_operationses_and_tags,&__pyx_n_s_context,0};
    PyObject* values[8] = {0,0,0,0,0,0,0,0};

    /* "src/python/grpcio/grpc/_cython/_cygrpc/channel.pyx.pxi":470
 *       self, int flags, method, host, object deadline, object metadata,
 *       CallCredentials credentials, operationses_and_tags,
 *       object context = None):             # <<<<<<<<<<<<<<
 *     return _segregated_call(
 *         self._state, flags, method, host, deadline, metadata, credentials,
 */
    values[7] = ((PyObject *)Py_None);
    if (unlikely(__pyx_kwds)) {
      Py_ssize_t kw_args;
      const Py_ssize_t pos_args = PyTuple_GET_SIZE(__pyx_args);
      switch (pos_args) {
        case  8: values[7] = PyTuple_GET_ITEM(__pyx_args, 7);
        CYTHON_FALLTHROUGH;
        case  7: values[6] = PyTuple_GET_ITEM(__pyx_args, 6);
        CYTHON_FALLTHROUGH;
        case  6: values[5] = PyTuple_GET_ITEM(__pyx_args, 5);
        CYTHON_FALLTHROUGH;
        case  5: values[4] = PyTuple_GET_ITEM(__pyx_args, 4);
        CYTHON_FALLTHROUGH;
        case  4: values[3] = PyTuple_GET_ITEM(__pyx_args, 3);
        CYTHON_FALLTHROUGH;
        case  3: values[2] = PyTuple_GET_ITEM(__pyx_args, 2);
        CYTHON_FALLTHROUGH;
        case  2: values[1] = PyTuple_GET_ITEM(__pyx_args, 1);
        CYTHON_FALLTHROUGH;
        case  1: values[0] = PyTuple_GET_ITEM(__pyx_args, 0);
        CYTHON_FALLTHROUGH;
        case  0: break;
        default: goto __pyx_L5_argtuple_error;
      }
      kw_args = PyDict_Size(__pyx_kwds);
      switch (pos_args) {
        case  0:
        if (likely((values[0] = __Pyx_PyDict_GetItemStr(__pyx_kwds, __pyx_n_s_flags)) != 0)) kw_args--;
        else goto __pyx_L5_argtuple_error;
        CYTHON_FALLTHROUGH;
        case  1:
        if (likely((values[1] = __Pyx_PyDict_GetItemStr(__pyx_kwds, __pyx_n_s_method)) != 0)) kw_args--;
        else {
          __Pyx_RaiseArgtupleInvalid("segregated_call", 0, 7, 8, 1); __PYX_ERR(8, 467, __pyx_L3_error)
        }
        CYTHON_FALLTHROUGH;
        case  2:
        if (likely((values[2] = __Pyx_PyDict_GetItemStr(__pyx_kwds, __pyx_n_s_host)) != 0)) kw_args--;
        else {
          __Pyx_RaiseArgtupleInvalid("segregated_call", 0, 7, 8, 2); __PYX_ERR(8, 467, __pyx_L3_error)
        }
        CYTHON_FALLTHROUGH;
        case  3:
        if (likely((values[3] = __Pyx_PyDict_GetItemStr(__pyx_kwds, __pyx_n_s_deadline)) != 0)) kw_args--;
        else {
          __Pyx_RaiseArgtupleInvalid("segregated_call", 0, 7, 8, 3); __PYX_ERR(8, 467, __pyx_L3_error)
        }
        CYTHON_FALLTHROUGH;
        case  4:
        if (likely((values[4] = __Pyx_PyDict_GetItemStr(__pyx_kwds, __pyx_n_s_metadata)) != 0)) kw_args--;
        else {
          __Pyx_RaiseArgtupleInvalid("segregated_call", 0, 7, 8, 4); __PYX_ERR(8, 467, __pyx_L3_error)
        }
        CYTHON_FALLTHROUGH;
        case  5:
        if (likely((values[5] = __Pyx_PyDict_GetItemStr(__pyx_kwds, __pyx_n_s_credentials)) != 0)) kw_args--;
        else {
          __Pyx_RaiseArgtupleInvalid("segregated_call", 0, 7, 8, 5); __PYX_ERR(8, 467, __pyx_L3_error)
        }
        CYTHON_FALLTHROUGH;
        case  6:
        if (likely((values[6] = __Pyx_PyDict_GetItemStr(__pyx_kwds, __pyx_n_s_operationses_and_tags)) != 0)) kw_args--;
        else {
          __Pyx_RaiseArgtupleInvalid("segregated_call", 0, 7, 8, 6); __PYX_ERR(8, 467, __pyx_L3_error)
        }
        CYTHON_FALLTHROUGH;
        case  7:
        if (kw_args > 0) {
          PyObject* value = __Pyx_PyDict_GetItemStr(__pyx_kwds, __pyx_n_s_context);
          if (value) { values[7] = value; kw_args--; }
        }
      }
      if (unlikely(kw_args > 0)) {
        if (unlikely(__Pyx_ParseOptionalKeywords(__pyx_kwds, __pyx_pyargnames, 0, values, pos_args, "segregated_call") < 0)) __PYX_ERR(8, 467, __pyx_L3_error)
      }
    } else {
      switch (PyTuple_GET_SIZE(__pyx_args)) {
        case  8: values[7] = PyTuple_GET_ITEM(__pyx_args, 7);
        CYTHON_FALLTHROUGH;
        case  7: values[6] = PyTuple_GET_ITEM(__pyx_args, 6);
        values[5] = PyTuple_GET_ITEM(__pyx_args, 5);
        values[4] = PyTuple_GET_ITEM(__pyx_args, 4);
        values[3] = PyTuple_GET_ITEM(__pyx_args, 3);
        values[2] = PyTuple_GET_ITEM(__pyx_args, 2);
        values[1] = PyTuple_GET_ITEM(__pyx_args, 1);
        values[0] = PyTuple_GET_ITEM(__pyx_args, 0);
        break;
        default: goto __pyx_L5_argtuple_error;
      }
    }
    __pyx_v_flags = __Pyx_PyInt_As_int(values[0]); if (unlikely((__pyx_v_flags == (int)-1) && PyErr_Occurred())) __PYX_ERR(8, 468, __pyx_L3_error)
    __pyx_v_method = values[1];
    __pyx_v_host = values[2];
    __pyx_v_deadline = values[3];
    __pyx_v_metadata = values[4];
    __pyx_v_credentials = ((struct __pyx_obj_4grpc_7_cython_6cygrpc_CallCredentials *)values[5]);
    __pyx_v_operationses_and_tags = values[6];
    __pyx_v_context = values[7];
  }
  goto __pyx_L4_argument_unpacking_done;
  __pyx_L5_argtuple_error:;
  __Pyx_RaiseArgtupleInvalid("segregated_call", 0, 7, 8, PyTuple_GET_SIZE(__pyx_args)); __PYX_ERR(8, 467, __pyx_L3_error)
  __pyx_L3_error:;
  __Pyx_AddTraceback("grpc._cython.cygrpc.Channel.segregated_call", __pyx_clineno, __pyx_lineno, __pyx_filename);
  __Pyx_RefNannyFinishContext();
  return NULL;
  __pyx_L4_argument_unpacking_done:;
  if (unlikely(!__Pyx_ArgTypeTest(((PyObject *)__pyx_v_credentials), __pyx_ptype_4grpc_7_cython_6cygrpc_CallCredentials, 1, "credentials", 0))) __PYX_ERR(8, 469, __pyx_L1_error)
  __pyx_r = __pyx_pf_4grpc_7_cython_6cygrpc_7Channel_8segregated_call(((struct __pyx_obj_4grpc_7_cython_6cygrpc_Channel *)__pyx_v_self), __pyx_v_flags, __pyx_v_method, __pyx_v_host, __pyx_v_deadline, __pyx_v_metadata, __pyx_v_credentials, __pyx_v_operationses_and_tags, __pyx_v_context);

  /* "src/python/grpcio/grpc/_cython/_cygrpc/channel.pyx.pxi":467
 *                             on_success, queue_deadline)
 * 
 *   def segregated_call(             # <<<<<<<<<<<<<<
 *       self, int flags, method, host, object deadline, object metadata,
 *       CallCredentials credentials, operationses_and_tags,
 */

  /* function exit code */
  goto __pyx_L0;
  __pyx_L1_error:;
  __pyx_r = NULL;
  __pyx_L0:;
  __Pyx_RefNannyFinishContext();
  return __pyx_r;
}

static PyObject *__pyx_pf_4grpc_7_cython_6cygrpc_7Channel_8segregated_call(struct __pyx_obj_4grpc_7_cython_6cygrpc_Channel *__pyx_v_self, int __pyx_v_flags, PyObject *__pyx_v_method, PyObject *__pyx_v_host, PyObject *__pyx_v_deadline, PyObject *__pyx_v_metadata, struct __pyx_obj_4grpc_7_cython_6cygrpc_CallCredentials *__pyx_v_credentials, PyObject *__pyx_v_operationses_and_tags, PyObject *__pyx_v_context) {
  PyObject *__pyx_r = NULL;
  __Pyx_RefNannyDeclarations
  PyObject *__pyx_t_1 = NULL;
  PyObject *__pyx_t_2 = NULL;
  __Pyx_RefNannySetupContext("segregated_call", 0);

  /* "src/python/grpcio/grpc/_cython/_cygrpc/channel.pyx.pxi":471
 *       CallCredentials credentials, operationses_and_tags,
 *       object context = None):
 *     return _segregated_call(             # <<<<<<<<<<<<<<
 *         self._state, flags, method, host, deadline, metadata, credentials,
 *         operationses_and_tags, context)
 */
  __Pyx_XDECREF(__pyx_r);

  /* "src/python/grpcio/grpc/_cython/_cygrpc/channel.pyx.pxi":472
 *       object context = None):
 *     return _segregated_call(
 *         self._state, flags, method, host, deadline, metadata, credentials,             # <<<<<<<<<<<<<<
 *         operationses_and_tags, context)
 * 
 */
  __pyx_t_1 = ((PyObject *)__pyx_v_self->_state);
  __Pyx_INCREF(__pyx_t_1);

  /* "src/python/grpcio/grpc/_cython/_cygrpc/channel.pyx.pxi":471
 *       CallCredentials credentials, operationses_and_tags,
 *       object context = None):
 *     return _segregated_call(             # <<<<<<<<<<<<<<
 *         self._state, flags, method, host, deadline, metadata, credentials,
 *         operationses_and_tags, context)
 */
  __pyx_t_2 = ((PyObject *)__pyx_f_4grpc_7_cython_6cygrpc__segregated_call(((struct __pyx_obj_4grpc_7_cython_6cygrpc__ChannelState *)__pyx_t_1), __pyx_v_flags, __pyx_v_method, __pyx_v_host, __pyx_v_deadline, __pyx_v_metadata, __pyx_v_credentials, __pyx_v_operationses_and_tags, __pyx_v_context)); if (unlikely(!__pyx_t_2)) __PYX_ERR(8, 471, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_2);
  __Pyx_DECREF(__pyx_t_1); __pyx_t_1 = 0;
  __pyx_r = __pyx_t_2;
  __pyx_t_2 = 0;
  goto __pyx_L0;

  /* "src/python/grpcio/grpc/_cython/_cygrpc/channel.pyx.pxi":467
 *                             on_success, queue_deadline)
 * 
 *   def segregated_call(             # <<<<<<<<<<<<<<
 *       self, int flags, method, host, object deadline, object metadata,
 *       CallCredentials credentials, operationses_and_tags,
 */

  /* function exit code */
  __pyx_L1_error:;
  __Pyx_XDECREF(__pyx_t_1);
  __Pyx_XDECREF(__pyx_t_2);
  __Pyx_AddTraceback("grpc._cython.cygrpc.Channel.segregated_call", __pyx_clineno, __pyx_lineno, __pyx_filename);
  __pyx_r = NULL;
  __pyx_L0:;
  __Pyx_XGIVEREF(__pyx_r);
  __Pyx_RefNannyFinishContext();
  return __pyx_r;
}

/* "src/python/grpcio/grpc/_cython/_cygrpc/channel.pyx.pxi":475
 *         operationses_and_tags, context)
 * 
 *   def check_connectivity_state(self, bint try_to_connect):             # <<<<<<<<<<<<<<
 *     with self._state.condition:
 *       if self._state.open:
 */

/* Python wrapper */
static PyObject *__pyx_pw_4grpc_7_cython_6cygrpc_7Channel_11check_connectivity_state(PyObject *__pyx_v_self, PyObject *__pyx_arg_try_to_connect); /*proto*/
static PyObject *__pyx_pw_4grpc_7_cython_6cygrpc_7Channel_11check_connectivity_state(PyObject *__pyx_v_self, PyObject *__pyx_arg_try_to_connect) {
  int __pyx_v_try_to_connect;
  PyObject *__pyx_r = 0;
  __Pyx_RefNannyDeclarations
  __Pyx_RefNannySetupContext("check_connectivity_state (wrapper)", 0);
  assert(__pyx_arg_try_to_connect); {
    __pyx_v_try_to_connect = __Pyx_PyObject_IsTrue(__pyx_arg_try_to_connect); if (unlikely((__pyx_v_try_to_connect == (int)-1) && PyErr_Occurred())) __PYX_ERR(8, 475, __pyx_L3_error)
  }
  goto __pyx_L4_argument_unpacking_done;
  __pyx_L3_error:;
  __Pyx_AddTraceback("grpc._cython.cygrpc.Channel.check_connectivity_state", __pyx_clineno, __pyx_lineno, __pyx_filename);
  __Pyx_RefNannyFinishContext();
  return NULL;
  __pyx_L4_argument_unpacking_done:;
  __pyx_r = __pyx_pf_4grpc_7_cython_6cygrpc_7Channel_10check_connectivity_state(((struct __pyx_obj_4grpc_7_cython_6cygrpc_Channel *)__pyx_v_self), ((int)__pyx_v_try_to_connect));

  /* function exit code */
  __Pyx_RefNannyFinishContext();
  return __pyx_r;
}

static PyObject *__pyx_pf_4grpc_7_cython_6cygrpc_7Channel_10check_connectivity_state(struct __pyx_obj_4grpc_7_cython_6cygrpc_Channel *__pyx_v_self, int __pyx_v_try_to_connect) {
  PyObject *__pyx_r = NULL;
  __Pyx_RefNannyDeclarations
  PyObject *__pyx_t_1 = NULL;
  PyObject *__pyx_t_2 = NULL;
  PyObject *__pyx_t_3 = NULL;
  PyObject *__pyx_t_4 = NULL;
  PyObject *__pyx_t_5 = NULL;
  PyObject *__pyx_t_6 = NULL;
  PyObject *__pyx_t_7 = NULL;
  int __pyx_t_8;
  PyObject *__pyx_t_9 = NULL;
  PyObject *__pyx_t_10 = NULL;
  int __pyx_t_11;
  __Pyx_RefNannySetupContext("check_connectivity_state", 0);

  /* "src/python/grpcio/grpc/_cython/_cygrpc/channel.pyx.pxi":476
 * 
 *   def check_connectivity_state(self, bint try_to_connect):
 *     with self._state.condition:             # <<<<<<<<<<<<<<
 *       if self._state.open:
 *         return grpc_channel_check_connectivity_state(
 */
  /*with:*/ {
    __pyx_t_1 = __Pyx_PyObject_LookupSpecial(__pyx_v_self->_state->condition, __pyx_n_s_exit); if (unlikely(!__pyx_t_1)) __PYX_ERR(8, 476, __pyx_L1_error)
    __Pyx_GOTREF(__pyx_t_1);
    __pyx_t_3 = __Pyx_PyObject_LookupSpecial(__pyx_v_self->_state->condition, __pyx_n_s_enter); if (unlikely(!__pyx_t_3)) __PYX_ERR(8, 476, __pyx_L3_error)
    __Pyx_GOTREF(__pyx_t_3);
    __pyx_t_4 = NULL;
    if (CYTHON_UNPACK_METHODS && likely(PyMethod_Check(__pyx_t_3))) {
      __pyx_t_4 = PyMethod_GET_SELF(__pyx_t_3);
      if (likely(__pyx_t_4)) {
        PyObject* function = PyMethod_GET_FUNCTION(__pyx_t_3);
        __Pyx_INCREF(__pyx_t_4);
        __Pyx_INCREF(function);
        __Pyx_DECREF_SET(__pyx_t_3, function);
      }
    }
    if (__pyx_t_4) {
      __pyx_t_2 = __Pyx_PyObject_CallOneArg(__pyx_t_3, __pyx_t_4); if (unlikely(!__pyx_t_2)) __PYX_ERR(8, 476, __pyx_L3_error)
      __Pyx_DECREF(__pyx_t_4); __pyx_t_4 = 0;
    } else {
      __pyx_t_2 = __Pyx_PyObject_CallNoArg(__pyx_t_3); if (unlikely(!__pyx_t_2)) __PYX_ERR(8, 476, __pyx_L3_error)
    }
    __Pyx_GOTREF(__pyx_t_2);
    __Pyx_DECREF(__pyx_t_3); __pyx_t_3 = 0;
    __Pyx_DECREF(__pyx_t_2); __pyx_t_2 = 0;
    /*try:*/ {
      {
        __Pyx_PyThreadState_declare
        __Pyx_PyThreadState_assign
        __Pyx_ExceptionSave(&__pyx_t_5, &__pyx_t_6, &__pyx_t_7);
        __Pyx_XGOTREF(__pyx_t_5);
        __Pyx_XGOTREF(__pyx_t_6);
        __Pyx_XGOTREF(__pyx_t_7);
        /*try:*/ {

          /* "src/python/grpcio/grpc/_cython/_cygrpc/channel.pyx.pxi":477
 *   def check_connectivity_state(self, bint try_to_connect):
 *     with self._state.condition:
 *       if self._state.open:             # <<<<<<<<<<<<<<
 *         return grpc_channel_check_connectivity_state(
 *             self._state.c_channel, try_to_connect)
 */
          __pyx_t_8 = __Pyx_PyObject_IsTrue(__pyx_v_self->_state->open); if (unlikely(__pyx_t_8 < 0)) __PYX_ERR(8, 477, __pyx_L7_error)
          if (likely(__pyx_t_8)) {

            /* "src/python/grpcio/grpc/_cython/_cygrpc/channel.pyx.pxi":478
 *     with self._state.condition:
 *       if self._state.open:
 *         return grpc_channel_check_connectivity_state(             # <<<<<<<<<<<<<<
 *             self._state.c_channel, try_to_connect)
 *       else:
 */
            __Pyx_XDECREF(__pyx_r);

            /* "src/python/grpcio/grpc/_cython/_cygrpc/channel.pyx.pxi":479
 *       if self._state.open:
 *         return grpc_channel_check_connectivity_state(
 *             self._state.c_channel, try_to_connect)             # <<<<<<<<<<<<<<
 *       else:
 *         raise ValueError('Cannot invoke RPC: %s' % self._state.closed_reason)
 */
            __pyx_t_2 = __Pyx_PyInt_From_grpc_connectivity_state(grpc_channel_check_connectivity_state(__pyx_v_self->_state->c_channel, __pyx_v_try_to_connect)); if (unlikely(!__pyx_t_2)) __PYX_ERR(8, 478, __pyx_L7_error)
            __Pyx_GOTREF(__pyx_t_2);
            __pyx_r = __pyx_t_2;
            __pyx_t_2 = 0;
            goto __pyx_L11_try_return;

            /* "src/python/grpcio/grpc/_cython/_cygrpc/channel.pyx.pxi":477
 *   def check_connectivity_state(self, bint try_to_connect):
 *     with self._state.condition:
 *       if self._state.open:             # <<<<<<<<<<<<<<
 *         return grpc_channel_check_connectivity_state(
 *             self._state.c_channel, try_to_connect)
 */
          }

          /* "src/python/grpcio/grpc/_cython/_cygrpc/channel.pyx.pxi":481
 *             self._state.c_channel, try_to_connect)
 *       else:
 *         raise ValueError('Cannot invoke RPC: %s' % self._state.closed_reason)             # <<<<<<<<<<<<<<
 * 
 *   def watch_connectivity_state(
 */
          /*else*/ {
            __pyx_t_2 = __Pyx_PyString_Format(__pyx_kp_s_Cannot_invoke_RPC_s, __pyx_v_self->_state->closed_reason); if (unlikely(!__pyx_t_2)) __PYX_ERR(8, 481, __pyx_L7_error)
            __Pyx_GOTREF(__pyx_t_2);
            __pyx_t_3 = __Pyx_PyObject_CallOneArg(__pyx_builtin_ValueError, __pyx_t_2); if (unlikely(!__pyx_t_3)) __PYX_ERR(8, 481, __pyx_L7_error)
            __Pyx_GOTREF(__pyx_t_3);
            __Pyx_DECREF(__pyx_t_2); __pyx_t_2 = 0;
            __Pyx_Raise(__pyx_t_3, 0, 0, 0);
            __Pyx_DECREF(__pyx_t_3); __pyx_t_3 = 0;
            __PYX_ERR(8, 481, __pyx_L7_error)
          }

          /* "src/python/grpcio/grpc/_cython/_cygrpc/channel.pyx.pxi":476
 * 
 *   def check_connectivity_state(self, bint try_to_connect):
 *     with self._state.condition:             # <<<<<<<<<<<<<<
 *       if self._state.open:
 *         return grpc_channel_check_connectivity_state(
 */
        }
        __pyx_L7_error:;
        __Pyx_XDECREF(__pyx_t_4); __pyx_t_4 = 0;
        __Pyx_XDECREF(__pyx_t_2); __pyx_t_2 = 0;
        __Pyx_XDECREF(__pyx_t_3); __pyx_t_3 = 0;
        /*except:*/ {
          __Pyx_AddTraceback("grpc._cython.cygrpc.Channel.check_connectivity_state", __pyx_clineno, __pyx_lineno, __pyx_filename);
          if (__Pyx_GetException(&__pyx_t_3, &__pyx_t_2, &__pyx_t_4) < 0) __PYX_ERR(8, 476, __pyx_L9_except_error)
          __Pyx_GOTREF(__pyx_t_3);
          __Pyx_GOTREF(__pyx_t_2);
          __Pyx_GOTREF(__pyx_t_4);
          __pyx_t_9 = PyTuple_Pack(3, __pyx_t_3, __pyx_t_2, __pyx_t_4); if (unlikely(!__pyx_t_9)) __PYX_ERR(8, 476, __pyx_L9_except_error)
          __Pyx_GOTREF(__pyx_t_9);
          __pyx_t_10 = __Pyx_PyObject_Call(__pyx_t_1, __pyx_t_9, NULL);
          __Pyx_DECREF(__pyx_t_1); __pyx_t_1 = 0;
          __Pyx_DECREF(__pyx_t_9); __pyx_t_9 = 0;
          if (unlikely(!__pyx_t_10)) __PYX_ERR(8, 476, __pyx_L9_except_error)
          __Pyx_GOTREF(__pyx_t_10);
          __pyx_t_8 = __Pyx_PyObject_IsTrue(__pyx_t_10);
          __Pyx_DECREF(__pyx_t_10); __pyx_t_10 = 0;
          if (__pyx_t_8 < 0) __PYX_ERR(8, 476, __pyx_L9_except_error)
          __pyx_t_11 = ((!(__pyx_t_8 != 0)) != 0);
          if (__pyx_t_11) {
            __Pyx_GIVEREF(__pyx_t_3);
            __Pyx_GIVEREF(__pyx_t_2);
            __Pyx_XGIVEREF(__pyx_t_4);
            __Pyx_ErrRestoreWithState(__pyx_t_3, __pyx_t_2, __pyx_t_4);
            __pyx_t_3 = 0; __pyx_t_2 = 0; __pyx_t_4 = 0; 
            __PYX_ERR(8, 476, __pyx_L9_except_error)
          }
          __Pyx_DECREF(__pyx_t_3); __pyx_t_3 = 0;
          __Pyx_DECREF(__pyx_t_2); __pyx_t_2 = 0;
          __Pyx_DECREF(__pyx_t_4); __pyx_t_4 = 0;
          goto __pyx_L8_exception_handled;
        }
        __pyx_L9_except_error:;
        __Pyx_XGIVEREF(__pyx_t_5);
        __Pyx_XGIVEREF(__pyx_t_6);
        __Pyx_XGIVEREF(__pyx_t_7);
        __Pyx_ExceptionReset(__pyx_t_5, __pyx_t_6, __pyx_t_7);
        goto __pyx_L1_error;
        __pyx_L11_try_return:;
        __Pyx_XGIVEREF(__pyx_t_5);
        __Pyx_XGIVEREF(__pyx_t_6);
        __Pyx_XGIVEREF(__pyx_t_7);
        __Pyx_ExceptionReset(__pyx_t_5, __pyx_t_6, __pyx_t_7);
        goto __pyx_L4_return;
        __pyx_L8_exception_handled:;
        __Pyx_XGIVEREF(__pyx_t_5);
        __Pyx_XGIVEREF(__pyx_t_6);
        __Pyx_XGIVEREF(__pyx_t_7);
        __Pyx_ExceptionReset(__pyx_t_5, __pyx_t_6, __pyx_t_7);
      }
    }
    /*finally:*/ {
      /*normal exit:*/{
        if (__pyx_t_1) {
          __pyx_t_7 = __Pyx_PyObject_Call(__pyx_t_1, __pyx_tuple__46, NULL);
          __Pyx_DECREF(__pyx_t_1); __pyx_t_1 = 0;
          if (unlikely(!__pyx_t_7)) __PYX_ERR(8, 476, __pyx_L1_error)
          __Pyx_GOTREF(__pyx_t_7);
          __Pyx_DECREF(__pyx_t_7); __pyx_t_7 = 0;
        }
        goto __pyx_L6;
      }
      __pyx_L4_return: {
        __pyx_t_7 = __pyx_r;
        __pyx_r = 0;
        if (__pyx_t_1) {
          __pyx_t_6 = __Pyx_PyObject_Call(__pyx_t_1, __pyx_tuple__47, NULL);
          __Pyx_DECREF(__pyx_t_1); __pyx_t_1 = 0;
          if (unlikely(!__pyx_t_6)) __PYX_ERR(8, 476, __pyx_L1_error)
          __Pyx_GOTREF(__pyx_t_6);
          __Pyx_DECREF(__pyx_t_6); __pyx_t_6 = 0;
        }
        __pyx_r = __pyx_t_7;
        __pyx_t_7 = 0;
        goto __pyx_L0;
      }
      __pyx_L6:;
    }
    goto __pyx_L17;
    __pyx_L3_error:;
    __Pyx_DECREF(__pyx_t_1); __pyx_t_1 = 0;
    goto __pyx_L1_error;
    __pyx_L17:;
  }

  /* "src/python/grpcio/grpc/_cython/_cygrpc/channel.pyx.pxi":475
 *         operationses_and_tags, context)
 * 
 *   def check_connectivity_state(self, bint try_to_connect):             # <<<<<<<<<<<<<<
 *     with self._state.condition:
 *       if self._state.open:
 */

  /* function exit code */
  __pyx_r = Py_None; __Pyx_INCREF(Py_None);
  goto __pyx_L0;
  __pyx_L1_error:;
  __Pyx_XDECREF(__pyx_t_2);
  __Pyx_XDECREF(__pyx_t_3);
  __Pyx_XDECREF(__pyx_t_4);
  __Pyx_XDECREF(__pyx_t_9);
  __Pyx_AddTraceback("grpc._cython.cygrpc.Channel.check_connectivity_state", __pyx_clineno, __pyx_lineno, __pyx_filename);
  __pyx_r = NULL;
  __pyx_L0:;
  __Pyx_XGIVEREF(__pyx_r);
  __Pyx_RefNannyFinishContext();
  return __pyx_r;
}

/* "src/python/grpcio/grpc/_cython/_cygrpc/channel.pyx.pxi":483
 *         raise ValueError('Cannot invoke RPC: %s' % self._state.closed_reason)
 * 
 *   def watch_connectivity_state(             # <<<<<<<<<<<<<<
 *       self, grpc_connectivity_state last_observed_state, object deadline):
 *     return _watch_connectivity_state(self._state, last_observed_state, deadline)
 */

/* Python wrapper */
static PyObject *__pyx_pw_4grpc_7_cython_6cygrpc_7Channel_13watch_connectivity_state(PyObject *__pyx_v_self, PyObject *__pyx_args, PyObject *__pyx_kwds); /*proto*/
static PyObject *__pyx_pw_4grpc_7_cython_6cygrpc_7Channel_13watch_connectivity_state(PyObject *__pyx_v_self, PyObject *__pyx_args, PyObject *__pyx_kwds) {
  grpc_connectivity_state __pyx_v_last_observed_state;
  PyObject *__pyx_v_deadline = 0;
  PyObject *__pyx_r = 0;
  __Pyx_RefNannyDeclarations
  __Pyx_RefNannySetupContext("watch_connectivity_state (wrapper)", 0);
  {
    static PyObject **__pyx_pyargnames[] = {&__pyx_n_s_last_observed_state,&__pyx_n_s_deadline,0};
    PyObject* values[2] = {0,0};
    if (unlikely(__pyx_kwds)) {
      Py_ssize_t kw_args;
      const Py_ssize_t pos_args = PyTuple_GET_SIZE(__pyx_args);
      switch (pos_args) {
        case  2: values[1] = PyTuple_GET_ITEM(__pyx_args, 1);
        CYTHON_FALLTHROUGH;
        case  1: values[0] = PyTuple_GET_ITEM(__pyx_args, 0);
        CYTHON_FALLTHROUGH;
        case  0: break;
        default: goto __pyx_L5_argtuple_error;
      }
      kw_args = PyDict_Size(__pyx_kwds);
      switch (pos_args) {
        case  0:
        if (likely((values[0] = __Pyx_PyDict_GetItemStr(__pyx_kwds, __pyx_n_s_last_observed_state)) != 0)) kw_args--;
        else goto __pyx_L5_argtuple_error;
        CYTHON_FALLTHROUGH;
        case  1:
        if (likely((values[1] = __Pyx_PyDict_GetItemStr(__pyx_kwds, __pyx_n_s_deadline)) != 0)) kw_args--;
        else {
          __Pyx_RaiseArgtupleInvalid("watch_connectivity_state", 1, 2, 2, 1); __PYX_ERR(8, 483, __pyx_L3_error)
        }
      }
      if (unlikely(kw_args > 0)) {
        if (unlikely(__Pyx_ParseOptionalKeywords(__pyx_kwds, __pyx_pyargnames, 0, values, pos_args, "watch_connectivity_state") < 0)) __PYX_ERR(8, 483, __pyx_L3_error)
      }
    } else if (PyTuple_GET_SIZE(__pyx_args) != 2) {
      goto __pyx_L5_argtuple_error;
    } else {
      values[0] = PyTuple_GET_ITEM(__pyx_args, 0);
      values[1] = PyTuple_GET_ITEM(__pyx_args, 1);
    }
    __pyx_v_last_observed_state = ((grpc_connectivity_state)__Pyx_PyInt_As_grpc_connectivity_state(values[0])); if (unlikely(PyErr_Occurred())) __PYX_ERR(8, 484, __pyx_L3_error)
    __pyx_v_deadline = values[1];
  }
  goto __pyx_L4_argument_unpacking_done;
  __pyx_L5_argtuple_error:;
  __Pyx_RaiseArgtupleInvalid("watch_connectivity_state", 1, 2, 2, PyTuple_GET_SIZE(__pyx_args)); __PYX_ERR(8, 483, __pyx_L3_error)
  __pyx_L3_error:;
  __Pyx_AddTraceback("grpc._cython.cygrpc.Channel.watch_connectivity_state", __pyx_clineno, __pyx_lineno, __pyx_filename);
  __Pyx_RefNannyFinishContext();
  return NULL;
  __pyx_L4_argument_unpacking_done:;
  __pyx_r = __pyx_pf_4grpc_7_cython_6cygrpc_7Channel_12watch_connectivity_state(((struct __pyx_obj_4grpc_7_cython_6cygrpc_Channel *)__pyx_v_self), __pyx_v_last_observed_state, __pyx_v_deadline);

  /* function exit code */
  __Pyx_RefNannyFinishContext();
  return __pyx_r;
}

static PyObject *__pyx_pf_4grpc_7_cython_6cygrpc_7Channel_12watch_connectivity_state(struct __pyx_obj_4grpc_7_cython_6cygrpc_Channel *__pyx_v_self, grpc_connectivity_state __pyx_v_last_observed_state, PyObject *__pyx_v_deadline) {
  PyObject *__pyx_r = NULL;
  __Pyx_RefNannyDeclarations
  PyObject *__pyx_t_1 = NULL;
  PyObject *__pyx_t_2 = NULL;
  __Pyx_RefNannySetupContext("watch_connectivity_state", 0);

  /* "src/python/grpcio/grpc/_cython/_cygrpc/channel.pyx.pxi":485
 *   def watch_connectivity_state(
 *       self, grpc_connectivity_state last_observed_state, object deadline):
 *     return _watch_connectivity_state(self._state, last_observed_state, deadline)             # <<<<<<<<<<<<<<
 * 
 *   def close(self, code, details):
 */
  __Pyx_XDECREF(__pyx_r);
  __pyx_t_1 = ((PyObject *)__pyx_v_self->_state);
  __Pyx_INCREF(__pyx_t_1);
  __pyx_t_2 = __pyx_f_4grpc_7_cython_6cygrpc__watch_connectivity_state(((struct __pyx_obj_4grpc_7_cython_6cygrpc__ChannelState *)__pyx_t_1), __pyx_v_last_observed_state, __pyx_v_deadline); if (unlikely(!__pyx_t_2)) __PYX_ERR(8, 485, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_2);
  __Pyx_DECREF(__pyx_t_1); __pyx_t_1 = 0;
  __pyx_r = __pyx_t_2;
  __pyx_t_2 = 0;
  goto __pyx_L0;

  /* "src/python/grpcio/grpc/_cython/_cygrpc/channel.pyx.pxi":483
 *         raise ValueError('Cannot invoke RPC: %s' % self._state.closed_reason)
 * 
 *   def watch_connectivity_state(             # <<<<<<<<<<<<<<
 *       self, grpc_connectivity_state last_observed_state, object deadline):
 *     return _watch_connectivity_state(self._state, last_observed_state, deadline)
 */

  /* function exit code */
  __pyx_L1_error:;
  __Pyx_XDECREF(__pyx_t_1);
  __Pyx_XDECREF(__pyx_t_2);
  __Pyx_AddTraceback("grpc._cython.cygrpc.Channel.watch_connectivity_state", __pyx_clineno, __pyx_lineno, __pyx_filename);
  __pyx_r = NULL;
  __pyx_L0:;
  __Pyx_XGIVEREF(__pyx_r);
  __Pyx_RefNannyFinishContext();
  return __pyx_r;
}

/* "src/python/grpcio/grpc/_cython/_cygrpc/channel.pyx.pxi":487
 *     return _watch_connectivity_state(self._state, last_observed_state, deadline)
 * 
 *   def close(self, code, details):             # <<<<<<<<<<<<<<
 *     _close(self, code, details, False)
 * 
 */

/* Python wrapper */
static PyObject *__pyx_pw_4grpc_7_cython_6cygrpc_7Channel_15close(PyObject *__pyx_v_self, PyObject *__pyx_args, PyObject *__pyx_kwds); /*proto*/
static PyObject *__pyx_pw_4grpc_7_cython_6cygrpc_7Channel_15close(PyObject *__pyx_v_self, PyObject *__pyx_args, PyObject *__pyx_kwds) {
  PyObject *__pyx_v_code = 0;
  PyObject *__pyx_v_details = 0;
  PyObject *__pyx_r = 0;
  __Pyx_RefNannyDeclarations
  __Pyx_RefNannySetupContext("close (wrapper)", 0);
  {
    static PyObject **__pyx_pyargnames[] = {&__pyx_n_s_code,&__pyx_n_s_details,0};
    PyObject* values[2] = {0,0};
    if (unlikely(__pyx_kwds)) {
      Py_ssize_t kw_args;
      const Py_ssize_t pos_args = PyTuple_GET_SIZE(__pyx_args);
      switch (pos_args) {
        case  2: values[1] = PyTuple_GET_ITEM(__pyx_args, 1);
        CYTHON_FALLTHROUGH;
        case  1: values[0] = PyTuple_GET_ITEM(__pyx_args, 0);
        CYTHON_FALLTHROUGH;
        case  0: break;
        default: goto __pyx_L5_argtuple_error;
      }
      kw_args = PyDict_Size(__pyx_kwds);
      switch (pos_args) {
        case  0:
        if (likely((values[0] = __Pyx_PyDict_GetItemStr(__pyx_kwds, __pyx_n_s_code)) != 0)) kw_args--;
        else goto __pyx_L5_argtuple_error;
        CYTHON_FALLTHROUGH;
        case  1:
        if (likely((values[1] = __Pyx_PyDict_GetItemStr(__pyx_kwds, __pyx_n_s_details)) != 0)) kw_args--;
        else {
          __Pyx_RaiseArgtupleInvalid("close", 1, 2, 2, 1); __PYX_ERR(8, 487, __pyx_L3_error)
        }
      }
      if (unlikely(kw_args > 0)) {
        if (unlikely(__Pyx_ParseOptionalKeywords(__pyx_kwds, __pyx_pyargnames, 0, values, pos_args, "close") < 0)) __PYX_ERR(8, 487, __pyx_L3_error)
      }
    } else if (PyTuple_GET_SIZE(__pyx_args) != 2) {
      goto __pyx_L5_argtuple_error;
    } else {
      values[0] = PyTuple_GET_ITEM(__pyx_args, 0);
      values[1] = PyTuple_GET_ITEM(__pyx_args, 1);
    }
    __pyx_v_code = values[0];
    __pyx_v_details = values[1];
  }
  goto __pyx_L4_argument_unpacking_done;
  __pyx_L5_argtuple_error:;
  __Pyx_RaiseArgtupleInvalid("close", 1, 2, 2, PyTuple_GET_SIZE(__pyx_args)); __PYX_ERR(8, 487, __pyx_L3_error)
  __pyx_L3_error:;
  __Pyx_AddTraceback("grpc._cython.cygrpc.Channel.close", __pyx_clineno, __pyx_lineno, __pyx_filename);
  __Pyx_RefNannyFinishContext();
  return NULL;
  __pyx_L4_argument_unpacking_done:;
  __pyx_r = __pyx_pf_4grpc_7_cython_6cygrpc_7Channel_14close(((struct __pyx_obj_4grpc_7_cython_6cygrpc_Channel *)__pyx_v_self), __pyx_v_code, __pyx_v_details);

  /* function exit code */
  __Pyx_RefNannyFinishContext();
  return __pyx_r;
}

static PyObject *__pyx_pf_4grpc_7_cython_6cygrpc_7Channel_14close(struct __pyx_obj_4grpc_7_cython_6cygrpc_Channel *__pyx_v_self, PyObject *__pyx_v_code, PyObject *__pyx_v_details) {
  PyObject *__pyx_r = NULL;
  __Pyx_RefNannyDeclarations
  grpc_status_code __pyx_t_1;
  PyObject *__pyx_t_2 = NULL;
  __Pyx_RefNannySetupContext("close", 0);

  /* "src/python/grpcio/grpc/_cython/_cygrpc/channel.pyx.pxi":488
 * 
 *   def close(self, code, details):
 *     _close(self, code, details, False)             # <<<<<<<<<<<<<<
 * 
 *   def close_on_fork(self, code, details):
 */
  __pyx_t_1 = ((grpc_status_code)__Pyx_PyInt_As_grpc_status_code(__pyx_v_code)); if (unlikely(PyErr_Occurred())) __PYX_ERR(8, 488, __pyx_L1_error)
  __pyx_t_2 = __pyx_f_4grpc_7_cython_6cygrpc__close(__pyx_v_self, __pyx_t_1, __pyx_v_details, Py_False); if (unlikely(!__pyx_t_2)) __PYX_ERR(8, 488, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_2);
  __Pyx_DECREF(__pyx_t_2); __pyx_t_2 = 0;

  /* "src/python/grpcio/grpc/_cython/_cygrpc/channel.pyx.pxi":487
 *     return _watch_connectivity_state(self._state, last_observed_state, deadline)
 * 
 *   def close(self, code, details):             # <<<<<<<<<<<<<<
 *     _close(self, code, details, False)
 * 
 */

  /* function exit code */
  __pyx_r = Py_None; __Pyx_INCREF(Py_None);
  goto __pyx_L0;
  __pyx_L1_error:;
  __Pyx_XDECREF(__pyx_t_2);
  __Pyx_AddTraceback("grpc._cython.cygrpc.Channel.close", __pyx_clineno, __pyx_lineno, __pyx_filename);
  __pyx_r = NULL;
  __pyx_L0:;
  __Pyx_XGIVEREF(__pyx_r);
  __Pyx_RefNannyFinishContext();
  return __pyx_r;
}

/* "src/python/grpcio/grpc/_cython/_cygrpc/channel.pyx.pxi":490
 *     _close(self, code, details, False)
 * 
 *   def close_on_fork(self, code, details):             # <<<<<<<<<<<<<<
 *     _close(self, code, details, True)
 */

/* Python wrapper */
static PyObject *__pyx_pw_4grpc_7_cython_6cygrpc_7Channel_17close_on_fork(PyObject *__pyx_v_self, PyObject *__pyx_args, PyObject *__pyx_kwds); /*proto*/
static PyObject *__pyx_pw_4grpc_7_cython_6cygrpc_7Channel_17close_on_fork(PyObject *__pyx_v_self, PyObject *__pyx_args, PyObject *__pyx_kwds) {
  PyObject *__pyx_v_code = 0;
  PyObject *__pyx_v_details = 0;
  PyObject *__pyx_r = 0;
  __Pyx_RefNannyDeclarations
  __Pyx_RefNannySetupContext("close_on_fork (wrapper)", 0);
  {
    static PyObject **__pyx_pyargnames[] = {&__pyx_n_s_code,&__pyx_n_s_details,0};
    PyObject* values[2] = {0,0};
    if (unlikely(__pyx_kwds)) {
      Py_ssize_t kw_args;
      const Py_ssize_t pos_args = PyTuple_GET_SIZE(__pyx_args);
      switch (pos_args) {
        case  2: values[1] = PyTuple_GET_ITEM(__pyx_args, 1);
        CYTHON_FALLTHROUGH;
        case  1: values[0] = PyTuple_GET_ITEM(__pyx_args, 0);
        CYTHON_FALLTHROUGH;
        case  0: break;
        default: goto __pyx_L5_argtuple_error;
      }
      kw_args = PyDict_Size(__pyx_kwds);
      switch (pos_args) {
        case  0:
        if (likely((values[0] = __Pyx_PyDict_GetItemStr(__pyx_kwds, __pyx_n_s_code)) != 0)) kw_args--;
        else goto __pyx_L5_argtuple_error;
        CYTHON_FALLTHROUGH;
        case  1:
        if (likely((values[1] = __Pyx_PyDict_GetItemStr(__pyx_kwds, __pyx_n_s_details)) != 0)) kw_args--;
        else {
          __Pyx_RaiseArgtupleInvalid("close_on_fork", 1, 2, 2, 1); __PYX_ERR(8, 490, __pyx_L3_error)
        }
      }
      if (unlikely(kw_args > 0)) {
        if (unlikely(__Pyx_ParseOptionalKeywords(__pyx_kwds, __pyx_pyargnames, 0, values, pos_args, "close_on_fork") < 0)) __PYX_ERR(8, 490, __pyx_L3_error)
      }
    } else if (PyTuple_GET_SIZE(__pyx_args) != 2) {
      goto __pyx_L5_argtuple_error;
    } else {
      values[0] = PyTuple_GET_ITEM(__pyx_args, 0);
      values[1] = PyTuple_GET_ITEM(__pyx_args, 1);
    }
    __pyx_v_code = values[0];
    __pyx_v_details = values[1];
  }
  goto __pyx_L4_argument_unpacking_done;
  __pyx_L5_argtuple_error:;
  __Pyx_RaiseArgtupleInvalid("close_on_fork", 1, 2, 2, PyTuple_GET_SIZE(__pyx_args)); __PYX_ERR(8, 490, __pyx_L3_error)
  __pyx_L3_error:;
  __Pyx_AddTraceback("grpc._cython.cygrpc.Channel.close_on_fork", __pyx_clineno, __pyx_lineno, __pyx_filename);
  __Pyx_RefNannyFinishContext();
  return NULL;
  __pyx_L4_argument_unpacking_done:;
  __pyx_r = __pyx_pf_4grpc_7_cython_6cygrpc_7Channel_16close_on_fork(((struct __pyx_obj_4grpc_7_cython_6cygrpc_Channel *)__pyx_v_self), __pyx_v_code, __pyx_v_details);

  /* function exit code */
  __Pyx_RefNannyFinishContext();
  return __pyx_r;
}

static PyObject *__pyx_pf_4grpc_7_cython_6cygrpc_7Channel_16close_on_fork(struct __pyx_obj_4grpc_7_cython_6cygrpc_Channel *__pyx_v_self, PyObject *__pyx_v_code, PyObject *__pyx_v_details) {
  PyObject *__pyx_r = NULL;
  __Pyx_RefNannyDeclarations
  grpc_status_code __pyx_t_1;
  PyObject *__pyx_t_2 = NULL;
  __Pyx_RefNannySetupContext("close_on_fork", 0);

  /* "src/python/grpcio/grpc/_cython/_cygrpc/channel.pyx.pxi":491
 * 
 *   def close_on_fork(self, code, details):
 *     _close(self, code, details, True)             # <<<<<<<<<<<<<<
 */
  __pyx_t_1 = ((grpc_status_code)__Pyx_PyInt_As_grpc_status_code(__pyx_v_code)); if (unlikely(PyErr_Occurred())) __PYX_ERR(8, 491, __pyx_L1_error)
  __pyx_t_2 = __pyx_f_4grpc_7_cython_6cygrpc__close(__pyx_v_self, __pyx_t_1, __pyx_v_details, Py_True); if (unlikely(!__pyx_t_2)) __PYX_ERR(8, 491, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_2);
  __Pyx_DECREF(__pyx_t_2); __pyx_t_2 = 0;

  /* "src/python/grpcio/grpc/_cython/_cygrpc/channel.pyx.pxi":490
 *     _close(self, code, details, False)
 * 
 *   def close_on_fork(self, code, details):             # <<<<<<<<<<<<<<
 *     _close(self, code, details, True)
 */

  /* function exit code */
  __pyx_r = Py_None; __Pyx_INCREF(Py_None);
  goto __pyx_L0;
  __pyx_L1_error:;
  __Pyx_XDECREF(__pyx_t_2);
  __Pyx_AddTraceback("grpc._cython.cygrpc.Channel.close_on_fork", __pyx_clineno, __pyx_lineno, __pyx_filename);
  __pyx_r = NULL;
  __pyx_L0:;
  __Pyx_XGIVEREF(__pyx_r);
  __Pyx_RefNannyFinishContext();
  return __pyx_r;
}

/* "(tree fragment)":1
 * def __reduce_cython__(self):             # <<<<<<<<<<<<<<
 *     raise TypeError("no default __reduce__ due to non-trivial __cinit__")
 * def __setstate_cython__(self, __pyx_state):
 */

/* Python wrapper */
static PyObject *__pyx_pw_4grpc_7_cython_6cygrpc_7Channel_19__reduce_cython__(PyObject *__pyx_v_self, CYTHON_UNUSED PyObject *unused); /*proto*/
static PyObject *__pyx_pw_4grpc_7_cython_6cygrpc_7Channel_19__reduce_cython__(PyObject *__pyx_v_self, CYTHON_UNUSED PyObject *unused) {
  PyObject *__pyx_r = 0;
  __Pyx_RefNannyDeclarations
  __Pyx_RefNannySetupContext("__reduce_cython__ (wrapper)", 0);
  __pyx_r = __pyx_pf_4grpc_7_cython_6cygrpc_7Channel_18__reduce_cython__(((struct __pyx_obj_4grpc_7_cython_6cygrpc_Channel *)__pyx_v_self));

  /* function exit code */
  __Pyx_RefNannyFinishContext();
  return __pyx_r;
}

static PyObject *__pyx_pf_4grpc_7_cython_6cygrpc_7Channel_18__reduce_cython__(CYTHON_UNUSED struct __pyx_obj_4grpc_7_cython_6cygrpc_Channel *__pyx_v_self) {
  PyObject *__pyx_r = NULL;
  __Pyx_RefNannyDeclarations
  PyObject *__pyx_t_1 = NULL;
  __Pyx_RefNannySetupContext("__reduce_cython__", 0);

  /* "(tree fragment)":2
 * def __reduce_cython__(self):
 *     raise TypeError("no default __reduce__ due to non-trivial __cinit__")             # <<<<<<<<<<<<<<
 * def __setstate_cython__(self, __pyx_state):
 *     raise TypeError("no default __reduce__ due to non-trivial __cinit__")
 */
  __pyx_t_1 = __Pyx_PyObject_Call(__pyx_builtin_TypeError, __pyx_tuple__48, NULL); if (unlikely(!__pyx_t_1)) __PYX_ERR(7, 2, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_1);
  __Pyx_Raise(__pyx_t_1, 0, 0, 0);
  __Pyx_DECREF(__pyx_t_1); __pyx_t_1 = 0;
  __PYX_ERR(7, 2, __pyx_L1_error)

  /* "(tree fragment)":1
 * def __reduce_cython__(self):             # <<<<<<<<<<<<<<
 *     raise TypeError("no default __reduce__ due to non-trivial __cinit__")
 * def __setstate_cython__(self, __pyx_state):
 */

  /* function exit code */
  __pyx_L1_error:;
  __Pyx_XDECREF(__pyx_t_1);
  __Pyx_AddTraceback("grpc._cython.cygrpc.Channel.__reduce_cython__", __pyx_clineno, __pyx_lineno, __pyx_filename);
  __pyx_r = NULL;
  __Pyx_XGIVEREF(__pyx_r);
  __Pyx_RefNannyFinishContext();
  return __pyx_r;
}

/* "(tree fragment)":3
 * def __reduce_cython__(self):
 *     raise TypeError("no default __reduce__ due to non-trivial __cinit__")
 * def __setstate_cython__(self, __pyx_state):             # <<<<<<<<<<<<<<
 *     raise TypeError("no default __reduce__ due to non-trivial __cinit__")
 */

/* Python wrapper */
static PyObject *__pyx_pw_4grpc_7_cython_6cygrpc_7Channel_21__setstate_cython__(PyObject *__pyx_v_self, PyObject *__pyx_v___pyx_state); /*proto*/
static PyObject *__pyx_pw_4grpc_7_cython_6cygrpc_7Channel_21__setstate_cython__(PyObject *__pyx_v_self, PyObject *__pyx_v___pyx_state) {
  PyObject *__pyx_r = 0;
  __Pyx_RefNannyDeclarations
  __Pyx_RefNannySetupContext("__setstate_cython__ (wrapper)", 0);
  __pyx_r = __pyx_pf_4grpc_7_cython_6cygrpc_7Channel_20__setstate_cython__(((struct __pyx_obj_4grpc_7_cython_6cygrpc_Channel *)__pyx_v_self), ((PyObject *)__pyx_v___pyx_state));

  /* function exit code */
  __Pyx_RefNannyFinishContext();
  return __pyx_r;
}

static PyObject *__pyx_pf_4grpc_7_cython_6cygrpc_7Channel_20__setstate_cython__(CYTHON_UNUSED struct __pyx_obj_4grpc_7_cython_6cygrpc_Channel *__pyx_v_self, CYTHON_UNUSED PyObject *__pyx_v___pyx_state) {
  PyObject *__pyx_r = NULL;
  __Pyx_RefNannyDeclarations
  PyObject *__pyx_t_1 = NULL;
  __Pyx_RefNannySetupContext("__setstate_cython__", 0);

  /* "(tree fragment)":4
 *     raise TypeError("no default __reduce__ due to non-trivial __cinit__")
 * def __setstate_cython__(self, __pyx_state):
 *     raise TypeError("no default __reduce__ due to non-trivial __cinit__")             # <<<<<<<<<<<<<<
 */
  __pyx_t_1 = __Pyx_PyObject_Call(__pyx_builtin_TypeError, __pyx_tuple__49, NULL); if (unlikely(!__pyx_t_1)) __PYX_ERR(7, 4, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_1);
  __Pyx_Raise(__pyx_t_1, 0, 0, 0);
  __Pyx_DECREF(__pyx_t_1); __pyx_t_1 = 0;
  __PYX_ERR(7, 4, __pyx_L1_error)

  /* "(tree fragment)":3
 * def __reduce_cython__(self):
 *     raise TypeError("no default __reduce__ due to non-trivial __cinit__")
 * def __setstate_cython__(self, __pyx_state):             # <<<<<<<<<<<<<<
 *     raise TypeError("no default __reduce__ due to non-trivial __cinit__")
 */

  /* function exit code */
  __pyx_L1_error:;
  __Pyx_XDECREF(__pyx_t_1);
  __Pyx_AddTraceback("grpc._cython.cygrpc.Channel.__setstate_cython__", __pyx_clineno, __pyx_lineno, __pyx_filename);
  __pyx_r = NULL;
  __Pyx_XGIVEREF(__pyx_r);
  __Pyx_RefNannyFinishContext();
  return __pyx_r;
}

/* "src/python/grpcio/grpc/_cython/_cygrpc/channelz.pyx.pxi":16
 * 
 * 
 * def channelz_get_top_channels(start_channel_id):             # <<<<<<<<<<<<<<
 *     cdef char *c_returned_str = grpc_channelz_get_top_channels(
 *         start_channel_id,
 */

/* Python wrapper */
static PyObject *__pyx_pw_4grpc_7_cython_6cygrpc_1channelz_get_top_channels(PyObject *__pyx_self, PyObject *__pyx_v_start_channel_id); /*proto*/
static PyMethodDef __pyx_mdef_4grpc_7_cython_6cygrpc_1channelz_get_top_channels = {"channelz_get_top_channels", (PyCFunction)__pyx_pw_4grpc_7_cython_6cygrpc_1channelz_get_top_channels, METH_O, 0};
static PyObject *__pyx_pw_4grpc_7_cython_6cygrpc_1channelz_get_top_channels(PyObject *__pyx_self, PyObject *__pyx_v_start_channel_id) {
  PyObject *__pyx_r = 0;
  __Pyx_RefNannyDeclarations
  __Pyx_RefNannySetupContext("channelz_get_top_channels (wrapper)", 0);
  __pyx_r = __pyx_pf_4grpc_7_cython_6cygrpc_channelz_get_top_channels(__pyx_self, ((PyObject *)__pyx_v_start_channel_id));

  /* function exit code */
  __Pyx_RefNannyFinishContext();
  return __pyx_r;
}

static PyObject *__pyx_pf_4grpc_7_cython_6cygrpc_channelz_get_top_channels(CYTHON_UNUSED PyObject *__pyx_self, PyObject *__pyx_v_start_channel_id) {
  char *__pyx_v_c_returned_str;
  PyObject *__pyx_r = NULL;
  __Pyx_RefNannyDeclarations
  __pyx_t_4grpc_7_cython_6cygrpc_intptr_t __pyx_t_1;
  int __pyx_t_2;
  PyObject *__pyx_t_3 = NULL;
  PyObject *__pyx_t_4 = NULL;
  __Pyx_RefNannySetupContext("channelz_get_top_channels", 0);

  /* "src/python/grpcio/grpc/_cython/_cygrpc/channelz.pyx.pxi":18
 * def channelz_get_top_channels(start_channel_id):
 *     cdef char *c_returned_str = grpc_channelz_get_top_channels(
 *         start_channel_id,             # <<<<<<<<<<<<<<
 *     )
 *     if c_returned_str == NULL:
 */
  __pyx_t_1 = PyInt_AsSsize_t(__pyx_v_start_channel_id); if (unlikely((__pyx_t_1 == (Py_ssize_t)-1) && PyErr_Occurred())) __PYX_ERR(9, 18, __pyx_L1_error)

  /* "src/python/grpcio/grpc/_cython/_cygrpc/channelz.pyx.pxi":17
 * 
 * def channelz_get_top_channels(start_channel_id):
 *     cdef char *c_returned_str = grpc_channelz_get_top_channels(             # <<<<<<<<<<<<<<
 *         start_channel_id,
 *     )
 */
  __pyx_v_c_returned_str = grpc_channelz_get_top_channels(__pyx_t_1);

  /* "src/python/grpcio/grpc/_cython/_cygrpc/channelz.pyx.pxi":20
 *         start_channel_id,
 *     )
 *     if c_returned_str == NULL:             # <<<<<<<<<<<<<<
 *         raise ValueError('Failed to get top channels, please ensure your' \
 *                          ' start_channel_id==%s is valid' % start_channel_id)
 */
  __pyx_t_2 = ((__pyx_v_c_returned_str == NULL) != 0);
  if (unlikely(__pyx_t_2)) {

    /* "src/python/grpcio/grpc/_cython/_cygrpc/channelz.pyx.pxi":22
 *     if c_returned_str == NULL:
 *         raise ValueError('Failed to get top channels, please ensure your' \
 *                          ' start_channel_id==%s is valid' % start_channel_id)             # <<<<<<<<<<<<<<
 *     return c_returned_str
 * 
 */
    __pyx_t_3 = __Pyx_PyString_Format(__pyx_kp_s_Failed_to_get_top_channels_pleas, __pyx_v_start_channel_id); if (unlikely(!__pyx_t_3)) __PYX_ERR(9, 22, __pyx_L1_error)
    __Pyx_GOTREF(__pyx_t_3);

    /* "src/python/grpcio/grpc/_cython/_cygrpc/channelz.pyx.pxi":21
 *     )
 *     if c_returned_str == NULL:
 *         raise ValueError('Failed to get top channels, please ensure your' \             # <<<<<<<<<<<<<<
 *                          ' start_channel_id==%s is valid' % start_channel_id)
 *     return c_returned_str
 */
    __pyx_t_4 = __Pyx_PyObject_CallOneArg(__pyx_builtin_ValueError, __pyx_t_3); if (unlikely(!__pyx_t_4)) __PYX_ERR(9, 21, __pyx_L1_error)
    __Pyx_GOTREF(__pyx_t_4);
    __Pyx_DECREF(__pyx_t_3); __pyx_t_3 = 0;
    __Pyx_Raise(__pyx_t_4, 0, 0, 0);
    __Pyx_DECREF(__pyx_t_4); __pyx_t_4 = 0;
    __PYX_ERR(9, 21, __pyx_L1_error)

    /* "src/python/grpcio/grpc/_cython/_cygrpc/channelz.pyx.pxi":20
 *         start_channel_id,
 *     )
 *     if c_returned_str == NULL:             # <<<<<<<<<<<<<<
 *         raise ValueError('Failed to get top channels, please ensure your' \
 *                          ' start_channel_id==%s is valid' % start_channel_id)
 */
  }

  /* "src/python/grpcio/grpc/_cython/_cygrpc/channelz.pyx.pxi":23
 *         raise ValueError('Failed to get top channels, please ensure your' \
 *                          ' start_channel_id==%s is valid' % start_channel_id)
 *     return c_returned_str             # <<<<<<<<<<<<<<
 * 
 * def channelz_get_servers(start_server_id):
 */
  __Pyx_XDECREF(__pyx_r);
  __pyx_t_4 = __Pyx_PyBytes_FromString(__pyx_v_c_returned_str); if (unlikely(!__pyx_t_4)) __PYX_ERR(9, 23, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_4);
  __pyx_r = __pyx_t_4;
  __pyx_t_4 = 0;
  goto __pyx_L0;

  /* "src/python/grpcio/grpc/_cython/_cygrpc/channelz.pyx.pxi":16
 * 
 * 
 * def channelz_get_top_channels(start_channel_id):             # <<<<<<<<<<<<<<
 *     cdef char *c_returned_str = grpc_channelz_get_top_channels(
 *         start_channel_id,
 */

  /* function exit code */
  __pyx_L1_error:;
  __Pyx_XDECREF(__pyx_t_3);
  __Pyx_XDECREF(__pyx_t_4);
  __Pyx_AddTraceback("grpc._cython.cygrpc.channelz_get_top_channels", __pyx_clineno, __pyx_lineno, __pyx_filename);
  __pyx_r = NULL;
  __pyx_L0:;
  __Pyx_XGIVEREF(__pyx_r);
  __Pyx_RefNannyFinishContext();
  return __pyx_r;
}

/* "src/python/grpcio/grpc/_cython/_cygrpc/channelz.pyx.pxi":25
 *     return c_returned_str
 * 
 * def channelz_get_servers(start_server_id):             # <<<<<<<<<<<<<<
 *     cdef char *c_returned_str = grpc_channelz_get_servers(start_server_id)
 *     if c_returned_str == NULL:
 */

/* Python wrapper */
static PyObject *__pyx_pw_4grpc_7_cython_6cygrpc_3channelz_get_servers(PyObject *__pyx_self, PyObject *__pyx_v_start_server_id); /*proto*/
static PyMethodDef __pyx_mdef_4grpc_7_cython_6cygrpc_3channelz_get_servers = {"channelz_get_servers", (PyCFunction)__pyx_pw_4grpc_7_cython_6cygrpc_3channelz_get_servers, METH_O, 0};
static PyObject *__pyx_pw_4grpc_7_cython_6cygrpc_3channelz_get_servers(PyObject *__pyx_self, PyObject *__pyx_v_start_server_id) {
  PyObject *__pyx_r = 0;
  __Pyx_RefNannyDeclarations
  __Pyx_RefNannySetupContext("channelz_get_servers (wrapper)", 0);
  __pyx_r = __pyx_pf_4grpc_7_cython_6cygrpc_2channelz_get_servers(__pyx_self, ((PyObject *)__pyx_v_start_server_id));

  /* function exit code */
  __Pyx_RefNannyFinishContext();
  return __pyx_r;
}

static PyObject *__pyx_pf_4grpc_7_cython_6cygrpc_2channelz_get_servers(CYTHON_UNUSED PyObject *__pyx_self, PyObject *__pyx_v_start_server_id) {
  char *__pyx_v_c_returned_str;
  PyObject *__pyx_r = NULL;
  __Pyx_RefNannyDeclarations
  __pyx_t_4grpc_7_cython_6cygrpc_intptr_t __pyx_t_1;
  int __pyx_t_2;
  PyObject *__pyx_t_3 = NULL;
  PyObject *__pyx_t_4 = NULL;
  __Pyx_RefNannySetupContext("channelz_get_servers", 0);

  /* "src/python/grpcio/grpc/_cython/_cygrpc/channelz.pyx.pxi":26
 * 
 * def channelz_get_servers(start_server_id):
 *     cdef char *c_returned_str = grpc_channelz_get_servers(start_server_id)             # <<<<<<<<<<<<<<
 *     if c_returned_str == NULL:
 *         raise ValueError('Failed to get servers, please ensure your' \
 */
  __pyx_t_1 = PyInt_AsSsize_t(__pyx_v_start_server_id); if (unlikely((__pyx_t_1 == (Py_ssize_t)-1) && PyErr_Occurred())) __PYX_ERR(9, 26, __pyx_L1_error)
  __pyx_v_c_returned_str = grpc_channelz_get_servers(__pyx_t_1);

  /* "src/python/grpcio/grpc/_cython/_cygrpc/channelz.pyx.pxi":27
 * def channelz_get_servers(start_server_id):
 *     cdef char *c_returned_str = grpc_channelz_get_servers(start_server_id)
 *     if c_returned_str == NULL:             # <<<<<<<<<<<<<<
 *         raise ValueError('Failed to get servers, please ensure your' \
 *                          ' start_server_id==%s is valid' % start_server_id)
 */
  __pyx_t_2 = ((__pyx_v_c_returned_str == NULL) != 0);
  if (unlikely(__pyx_t_2)) {

    /* "src/python/grpcio/grpc/_cython/_cygrpc/channelz.pyx.pxi":29
 *     if c_returned_str == NULL:
 *         raise ValueError('Failed to get servers, please ensure your' \
 *                          ' start_server_id==%s is valid' % start_server_id)             # <<<<<<<<<<<<<<
 *     return c_returned_str
 * 
 */
    __pyx_t_3 = __Pyx_PyString_Format(__pyx_kp_s_Failed_to_get_servers_please_ens, __pyx_v_start_server_id); if (unlikely(!__pyx_t_3)) __PYX_ERR(9, 29, __pyx_L1_error)
    __Pyx_GOTREF(__pyx_t_3);

    /* "src/python/grpcio/grpc/_cython/_cygrpc/channelz.pyx.pxi":28
 *     cdef char *c_returned_str = grpc_channelz_get_servers(start_server_id)
 *     if c_returned_str == NULL:
 *         raise ValueError('Failed to get servers, please ensure your' \             # <<<<<<<<<<<<<<
 *                          ' start_server_id==%s is valid' % start_server_id)
 *     return c_returned_str
 */
    __pyx_t_4 = __Pyx_PyObject_CallOneArg(__pyx_builtin_ValueError, __pyx_t_3); if (unlikely(!__pyx_t_4)) __PYX_ERR(9, 28, __pyx_L1_error)
    __Pyx_GOTREF(__pyx_t_4);
    __Pyx_DECREF(__pyx_t_3); __pyx_t_3 = 0;
    __Pyx_Raise(__pyx_t_4, 0, 0, 0);
    __Pyx_DECREF(__pyx_t_4); __pyx_t_4 = 0;
    __PYX_ERR(9, 28, __pyx_L1_error)

    /* "src/python/grpcio/grpc/_cython/_cygrpc/channelz.pyx.pxi":27
 * def channelz_get_servers(start_server_id):
 *     cdef char *c_returned_str = grpc_channelz_get_servers(start_server_id)
 *     if c_returned_str == NULL:             # <<<<<<<<<<<<<<
 *         raise ValueError('Failed to get servers, please ensure your' \
 *                          ' start_server_id==%s is valid' % start_server_id)
 */
  }

  /* "src/python/grpcio/grpc/_cython/_cygrpc/channelz.pyx.pxi":30
 *         raise ValueError('Failed to get servers, please ensure your' \
 *                          ' start_server_id==%s is valid' % start_server_id)
 *     return c_returned_str             # <<<<<<<<<<<<<<
 * 
 * def channelz_get_server(server_id):
 */
  __Pyx_XDECREF(__pyx_r);
  __pyx_t_4 = __Pyx_PyBytes_FromString(__pyx_v_c_returned_str); if (unlikely(!__pyx_t_4)) __PYX_ERR(9, 30, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_4);
  __pyx_r = __pyx_t_4;
  __pyx_t_4 = 0;
  goto __pyx_L0;

  /* "src/python/grpcio/grpc/_cython/_cygrpc/channelz.pyx.pxi":25
 *     return c_returned_str
 * 
 * def channelz_get_servers(start_server_id):             # <<<<<<<<<<<<<<
 *     cdef char *c_returned_str = grpc_channelz_get_servers(start_server_id)
 *     if c_returned_str == NULL:
 */

  /* function exit code */
  __pyx_L1_error:;
  __Pyx_XDECREF(__pyx_t_3);
  __Pyx_XDECREF(__pyx_t_4);
  __Pyx_AddTraceback("grpc._cython.cygrpc.channelz_get_servers", __pyx_clineno, __pyx_lineno, __pyx_filename);
  __pyx_r = NULL;
  __pyx_L0:;
  __Pyx_XGIVEREF(__pyx_r);
  __Pyx_RefNannyFinishContext();
  return __pyx_r;
}

/* "src/python/grpcio/grpc/_cython/_cygrpc/channelz.pyx.pxi":32
 *     return c_returned_str
 * 
 * def channelz_get_server(server_id):             # <<<<<<<<<<<<<<
 *     cdef char *c_returned_str = grpc_channelz_get_server(server_id)
 *     if c_returned_str == NULL:
 */

/* Python wrapper */
static PyObject *__pyx_pw_4grpc_7_cython_6cygrpc_5channelz_get_server(PyObject *__pyx_self, PyObject *__pyx_v_server_id); /*proto*/
static PyMethodDef __pyx_mdef_4grpc_7_cython_6cygrpc_5channelz_get_server = {"channelz_get_server", (PyCFunction)__pyx_pw_4grpc_7_cython_6cygrpc_5channelz_get_server, METH_O, 0};
static PyObject *__pyx_pw_4grpc_7_cython_6cygrpc_5channelz_get_server(PyObject *__pyx_self, PyObject *__pyx_v_server_id) {
  PyObject *__pyx_r = 0;
  __Pyx_RefNannyDeclarations
  __Pyx_RefNannySetupContext("channelz_get_server (wrapper)", 0);
  __pyx_r = __pyx_pf_4grpc_7_cython_6cygrpc_4channelz_get_server(__pyx_self, ((PyObject *)__pyx_v_server_id));

  /* function exit code */
  __Pyx_RefNannyFinishContext();
  return __pyx_r;
}

static PyObject *__pyx_pf_4grpc_7_cython_6cygrpc_4channelz_get_server(CYTHON_UNUSED PyObject *__pyx_self, PyObject *__pyx_v_server_id) {
  char *__pyx_v_c_returned_str;
  PyObject *__pyx_r = NULL;
  __Pyx_RefNannyDeclarations
  __pyx_t_4grpc_7_cython_6cygrpc_intptr_t __pyx_t_1;
  int __pyx_t_2;
  PyObject *__pyx_t_3 = NULL;
  PyObject *__pyx_t_4 = NULL;
  __Pyx_RefNannySetupContext("channelz_get_server", 0);

  /* "src/python/grpcio/grpc/_cython/_cygrpc/channelz.pyx.pxi":33
 * 
 * def channelz_get_server(server_id):
 *     cdef char *c_returned_str = grpc_channelz_get_server(server_id)             # <<<<<<<<<<<<<<
 *     if c_returned_str == NULL:
 *         raise ValueError('Failed to get the server, please ensure your' \
 */
  __pyx_t_1 = PyInt_AsSsize_t(__pyx_v_server_id); if (unlikely((__pyx_t_1 == (Py_ssize_t)-1) && PyErr_Occurred())) __PYX_ERR(9, 33, __pyx_L1_error)
  __pyx_v_c_returned_str = grpc_channelz_get_server(__pyx_t_1);

  /* "src/python/grpcio/grpc/_cython/_cygrpc/channelz.pyx.pxi":34
 * def channelz_get_server(server_id):
 *     cdef char *c_returned_str = grpc_channelz_get_server(server_id)
 *     if c_returned_str == NULL:             # <<<<<<<<<<<<<<
 *         raise ValueError('Failed to get the server, please ensure your' \
 *                          ' server_id==%s is valid' % server_id)
 */
  __pyx_t_2 = ((__pyx_v_c_returned_str == NULL) != 0);
  if (unlikely(__pyx_t_2)) {

    /* "src/python/grpcio/grpc/_cython/_cygrpc/channelz.pyx.pxi":36
 *     if c_returned_str == NULL:
 *         raise ValueError('Failed to get the server, please ensure your' \
 *                          ' server_id==%s is valid' % server_id)             # <<<<<<<<<<<<<<
 *     return c_returned_str
 * 
 */
    __pyx_t_3 = __Pyx_PyString_Format(__pyx_kp_s_Failed_to_get_the_server_please, __pyx_v_server_id); if (unlikely(!__pyx_t_3)) __PYX_ERR(9, 36, __pyx_L1_error)
    __Pyx_GOTREF(__pyx_t_3);

    /* "src/python/grpcio/grpc/_cython/_cygrpc/channelz.pyx.pxi":35
 *     cdef char *c_returned_str = grpc_channelz_get_server(server_id)
 *     if c_returned_str == NULL:
 *         raise ValueError('Failed to get the server, please ensure your' \             # <<<<<<<<<<<<<<
 *                          ' server_id==%s is valid' % server_id)
 *     return c_returned_str
 */
    __pyx_t_4 = __Pyx_PyObject_CallOneArg(__pyx_builtin_ValueError, __pyx_t_3); if (unlikely(!__pyx_t_4)) __PYX_ERR(9, 35, __pyx_L1_error)
    __Pyx_GOTREF(__pyx_t_4);
    __Pyx_DECREF(__pyx_t_3); __pyx_t_3 = 0;
    __Pyx_Raise(__pyx_t_4, 0, 0, 0);
    __Pyx_DECREF(__pyx_t_4); __pyx_t_4 = 0;
    __PYX_ERR(9, 35, __pyx_L1_error)

    /* "src/python/grpcio/grpc/_cython/_cygrpc/channelz.pyx.pxi":34
 * def channelz_get_server(server_id):
 *     cdef char *c_returned_str = grpc_channelz_get_server(server_id)
 *     if c_returned_str == NULL:             # <<<<<<<<<<<<<<
 *         raise ValueError('Failed to get the server, please ensure your' \
 *                          ' server_id==%s is valid' % server_id)
 */
  }

  /* "src/python/grpcio/grpc/_cython/_cygrpc/channelz.pyx.pxi":37
 *         raise ValueError('Failed to get the server, please ensure your' \
 *                          ' server_id==%s is valid' % server_id)
 *     return c_returned_str             # <<<<<<<<<<<<<<
 * 
 * def channelz_get_server_sockets(server_id, start_socket_id, max_results):
 */
  __Pyx_XDECREF(__pyx_r);
  __pyx_t_4 = __Pyx_PyBytes_FromString(__pyx_v_c_returned_str); if (unlikely(!__pyx_t_4)) __PYX_ERR(9, 37, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_4);
  __pyx_r = __pyx_t_4;
  __pyx_t_4 = 0;
  goto __pyx_L0;

  /* "src/python/grpcio/grpc/_cython/_cygrpc/channelz.pyx.pxi":32
 *     return c_returned_str
 * 
 * def channelz_get_server(server_id):             # <<<<<<<<<<<<<<
 *     cdef char *c_returned_str = grpc_channelz_get_server(server_id)
 *     if c_returned_str == NULL:
 */

  /* function exit code */
  __pyx_L1_error:;
  __Pyx_XDECREF(__pyx_t_3);
  __Pyx_XDECREF(__pyx_t_4);
  __Pyx_AddTraceback("grpc._cython.cygrpc.channelz_get_server", __pyx_clineno, __pyx_lineno, __pyx_filename);
  __pyx_r = NULL;
  __pyx_L0:;
  __Pyx_XGIVEREF(__pyx_r);
  __Pyx_RefNannyFinishContext();
  return __pyx_r;
}

/* "src/python/grpcio/grpc/_cython/_cygrpc/channelz.pyx.pxi":39
 *     return c_returned_str
 * 
 * def channelz_get_server_sockets(server_id, start_socket_id, max_results):             # <<<<<<<<<<<<<<
 *     cdef char *c_returned_str = grpc_channelz_get_server_sockets(
 *         server_id,
 */

/* Python wrapper */
static PyObject *__pyx_pw_4grpc_7_cython_6cygrpc_7channelz_get_server_sockets(PyObject *__pyx_self, PyObject *__pyx_args, PyObject *__pyx_kwds); /*proto*/
static PyMethodDef __pyx_mdef_4grpc_7_cython_6cygrpc_7channelz_get_server_sockets = {"channelz_get_server_sockets", (PyCFunction)__pyx_pw_4grpc_7_cython_6cygrpc_7channelz_get_server_sockets, METH_VARARGS|METH_KEYWORDS, 0};
static PyObject *__pyx_pw_4grpc_7_cython_6cygrpc_7channelz_get_server_sockets(PyObject *__pyx_self, PyObject *__pyx_args, PyObject *__pyx_kwds) {
  PyObject *__pyx_v_server_id = 0;
  PyObject *__pyx_v_start_socket_id = 0;
  PyObject *__pyx_v_max_results = 0;
  PyObject *__pyx_r = 0;
  __Pyx_RefNannyDeclarations
  __Pyx_RefNannySetupContext("channelz_get_server_sockets (wrapper)", 0);
  {
    static PyObject **__pyx_pyargnames[] = {&__pyx_n_s_server_id,&__pyx_n_s_start_socket_id,&__pyx_n_s_max_results,0};
    PyObject* values[3] = {0,0,0};
    if (unlikely(__pyx_kwds)) {
      Py_ssize_t kw_args;
      const Py_ssize_t pos_args = PyTuple_GET_SIZE(__pyx_args);
      switch (pos_args) {
        case  3: values[2] = PyTuple_GET_ITEM(__pyx_args, 2);
        CYTHON_FALLTHROUGH;
        case  2: values[1] = PyTuple_GET_ITEM(__pyx_args, 1);
        CYTHON_FALLTHROUGH;
        case  1: values[0] = PyTuple_GET_ITEM(__pyx_args, 0);
        CYTHON_FALLTHROUGH;
        case  0: break;
        default: goto __pyx_L5_argtuple_error;
      }
      kw_args = PyDict_Size(__pyx_kwds);
      switch (pos_args) {
        case  0:
        if (likely((values[0] = __Pyx_PyDict_GetItemStr(__pyx_kwds, __pyx_n_s_server_id)) != 0)) kw_args--;
        else goto __pyx_L5_argtuple_error;
        CYTHON_FALLTHROUGH;
        case  1:
        if (likely((values[1] = __Pyx_PyDict_GetItemStr(__pyx_kwds, __pyx_n_s_start_socket_id)) != 0)) kw_args--;
        else {
          __Pyx_RaiseArgtupleInvalid("channelz_get_server_sockets", 1, 3, 3, 1); __PYX_ERR(9, 39, __pyx_L3_error)
        }
        CYTHON_FALLTHROUGH;
        case  2:
        if (likely((values[2] = __Pyx_PyDict_GetItemStr(__pyx_kwds, __pyx_n_s_max_results)) != 0)) kw_args--;
        else {
          __Pyx_RaiseArgtupleInvalid("channelz_get_server_sockets", 1, 3, 3, 2); __PYX_ERR(9, 39, __pyx_L3_error)
        }
      }
      if (unlikely(kw_args > 0)) {
        if (unlikely(__Pyx_ParseOptionalKeywords(__pyx_kwds, __pyx_pyargnames, 0, values, pos_args, "channelz_get_server_sockets") < 0)) __PYX_ERR(9, 39, __pyx_L3_error)
      }
    } else if (PyTuple_GET_SIZE(__pyx_args) != 3) {
      goto __pyx_L5_argtuple_error;
    } else {
      values[0] = PyTuple_GET_ITEM(__pyx_args, 0);
      values[1] = PyTuple_GET_ITEM(__pyx_args, 1);
      values[2] = PyTuple_GET_ITEM(__pyx_args, 2);
    }
    __pyx_v_server_id = values[0];
    __pyx_v_start_socket_id = values[1];
    __pyx_v_max_results = values[2];
  }
  goto __pyx_L4_argument_unpacking_done;
  __pyx_L5_argtuple_error:;
  __Pyx_RaiseArgtupleInvalid("channelz_get_server_sockets", 1, 3, 3, PyTuple_GET_SIZE(__pyx_args)); __PYX_ERR(9, 39, __pyx_L3_error)
  __pyx_L3_error:;
  __Pyx_AddTraceback("grpc._cython.cygrpc.channelz_get_server_sockets", __pyx_clineno, __pyx_lineno, __pyx_filename);
  __Pyx_RefNannyFinishContext();
  return NULL;
  __pyx_L4_argument_unpacking_done:;
  __pyx_r = __pyx_pf_4grpc_7_cython_6cygrpc_6channelz_get_server_sockets(__pyx_self, __pyx_v_server_id, __pyx_v_start_socket_id, __pyx_v_max_results);

  /* function exit code */
  __Pyx_RefNannyFinishContext();
  return __pyx_r;
}

static PyObject *__pyx_pf_4grpc_7_cython_6cygrpc_6channelz_get_server_sockets(CYTHON_UNUSED PyObject *__pyx_self, PyObject *__pyx_v_server_id, PyObject *__pyx_v_start_socket_id, PyObject *__pyx_v_max_results) {
  char *__pyx_v_c_returned_str;
  PyObject *__pyx_r = NULL;
  __Pyx_RefNannyDeclarations
  __pyx_t_4grpc_7_cython_6cygrpc_intptr_t __pyx_t_1;
  __pyx_t_4grpc_7_cython_6cygrpc_intptr_t __pyx_t_2;
  __pyx_t_4grpc_7_cython_6cygrpc_intptr_t __pyx_t_3;
  int __pyx_t_4;
  PyObject *__pyx_t_5 = NULL;
  PyObject *__pyx_t_6 = NULL;
  __Pyx_RefNannySetupContext("channelz_get_server_sockets", 0);

  /* "src/python/grpcio/grpc/_cython/_cygrpc/channelz.pyx.pxi":41
 * def channelz_get_server_sockets(server_id, start_socket_id, max_results):
 *     cdef char *c_returned_str = grpc_channelz_get_server_sockets(
 *         server_id,             # <<<<<<<<<<<<<<
 *         start_socket_id,
 *         max_results,
 */
  __pyx_t_1 = PyInt_AsSsize_t(__pyx_v_server_id); if (unlikely((__pyx_t_1 == (Py_ssize_t)-1) && PyErr_Occurred())) __PYX_ERR(9, 41, __pyx_L1_error)

  /* "src/python/grpcio/grpc/_cython/_cygrpc/channelz.pyx.pxi":42
 *     cdef char *c_returned_str = grpc_channelz_get_server_sockets(
 *         server_id,
 *         start_socket_id,             # <<<<<<<<<<<<<<
 *         max_results,
 *     )
 */
  __pyx_t_2 = PyInt_AsSsize_t(__pyx_v_start_socket_id); if (unlikely((__pyx_t_2 == (Py_ssize_t)-1) && PyErr_Occurred())) __PYX_ERR(9, 42, __pyx_L1_error)

  /* "src/python/grpcio/grpc/_cython/_cygrpc/channelz.pyx.pxi":43
 *         server_id,
 *         start_socket_id,
 *         max_results,             # <<<<<<<<<<<<<<
 *     )
 *     if c_returned_str == NULL:
 */
  __pyx_t_3 = PyInt_AsSsize_t(__pyx_v_max_results); if (unlikely((__pyx_t_3 == (Py_ssize_t)-1) && PyErr_Occurred())) __PYX_ERR(9, 43, __pyx_L1_error)

  /* "src/python/grpcio/grpc/_cython/_cygrpc/channelz.pyx.pxi":40
 * 
 * def channelz_get_server_sockets(server_id, start_socket_id, max_results):
 *     cdef char *c_returned_str = grpc_channelz_get_server_sockets(             # <<<<<<<<<<<<<<
 *         server_id,
 *         start_socket_id,
 */
  __pyx_v_c_returned_str = grpc_channelz_get_server_sockets(__pyx_t_1, __pyx_t_2, __pyx_t_3);

  /* "src/python/grpcio/grpc/_cython/_cygrpc/channelz.pyx.pxi":45
 *         max_results,
 *     )
 *     if c_returned_str == NULL:             # <<<<<<<<<<<<<<
 *         raise ValueError('Failed to get server sockets, please ensure your' \
 *                          ' server_id==%s and start_socket_id==%s and' \
 */
  __pyx_t_4 = ((__pyx_v_c_returned_str == NULL) != 0);
  if (unlikely(__pyx_t_4)) {

    /* "src/python/grpcio/grpc/_cython/_cygrpc/channelz.pyx.pxi":49
 *                          ' server_id==%s and start_socket_id==%s and' \
 *                          ' max_results==%s is valid' %
 *                          (server_id, start_socket_id, max_results))             # <<<<<<<<<<<<<<
 *     return c_returned_str
 * 
 */
    __pyx_t_5 = PyTuple_New(3); if (unlikely(!__pyx_t_5)) __PYX_ERR(9, 49, __pyx_L1_error)
    __Pyx_GOTREF(__pyx_t_5);
    __Pyx_INCREF(__pyx_v_server_id);
    __Pyx_GIVEREF(__pyx_v_server_id);
    PyTuple_SET_ITEM(__pyx_t_5, 0, __pyx_v_server_id);
    __Pyx_INCREF(__pyx_v_start_socket_id);
    __Pyx_GIVEREF(__pyx_v_start_socket_id);
    PyTuple_SET_ITEM(__pyx_t_5, 1, __pyx_v_start_socket_id);
    __Pyx_INCREF(__pyx_v_max_results);
    __Pyx_GIVEREF(__pyx_v_max_results);
    PyTuple_SET_ITEM(__pyx_t_5, 2, __pyx_v_max_results);

    /* "src/python/grpcio/grpc/_cython/_cygrpc/channelz.pyx.pxi":48
 *         raise ValueError('Failed to get server sockets, please ensure your' \
 *                          ' server_id==%s and start_socket_id==%s and' \
 *                          ' max_results==%s is valid' %             # <<<<<<<<<<<<<<
 *                          (server_id, start_socket_id, max_results))
 *     return c_returned_str
 */
    __pyx_t_6 = __Pyx_PyString_Format(__pyx_kp_s_Failed_to_get_server_sockets_ple, __pyx_t_5); if (unlikely(!__pyx_t_6)) __PYX_ERR(9, 48, __pyx_L1_error)
    __Pyx_GOTREF(__pyx_t_6);
    __Pyx_DECREF(__pyx_t_5); __pyx_t_5 = 0;

    /* "src/python/grpcio/grpc/_cython/_cygrpc/channelz.pyx.pxi":46
 *     )
 *     if c_returned_str == NULL:
 *         raise ValueError('Failed to get server sockets, please ensure your' \             # <<<<<<<<<<<<<<
 *                          ' server_id==%s and start_socket_id==%s and' \
 *                          ' max_results==%s is valid' %
 */
    __pyx_t_5 = __Pyx_PyObject_CallOneArg(__pyx_builtin_ValueError, __pyx_t_6); if (unlikely(!__pyx_t_5)) __PYX_ERR(9, 46, __pyx_L1_error)
    __Pyx_GOTREF(__pyx_t_5);
    __Pyx_DECREF(__pyx_t_6); __pyx_t_6 = 0;
    __Pyx_Raise(__pyx_t_5, 0, 0, 0);
    __Pyx_DECREF(__pyx_t_5); __pyx_t_5 = 0;
    __PYX_ERR(9, 46, __pyx_L1_error)

    /* "src/python/grpcio/grpc/_cython/_cygrpc/channelz.pyx.pxi":45
 *         max_results,
 *     )
 *     if c_returned_str == NULL:             # <<<<<<<<<<<<<<
 *         raise ValueError('Failed to get server sockets, please ensure your' \
 *                          ' server_id==%s and start_socket_id==%s and' \
 */
  }

  /* "src/python/grpcio/grpc/_cython/_cygrpc/channelz.pyx.pxi":50
 *                          ' max_results==%s is valid' %
 *                          (server_id, start_socket_id, max_results))
 *     return c_returned_str             # <<<<<<<<<<<<<<
 * 
 * def channelz_get_channel(channel_id):
 */
  __Pyx_XDECREF(__pyx_r);
  __pyx_t_5 = __Pyx_PyBytes_FromString(__pyx_v_c_returned_str); if (unlikely(!__pyx_t_5)) __PYX_ERR(9, 50, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_5);
  __pyx_r = __pyx_t_5;
  __pyx_t_5 = 0;
  goto __pyx_L0;

  /* "src/python/grpcio/grpc/_cython/_cygrpc/channelz.pyx.pxi":39
 *     return c_returned_str
 * 
 * def channelz_get_server_sockets(server_id, start_socket_id, max_results):             # <<<<<<<<<<<<<<
 *     cdef char *c_returned_str = grpc_channelz_get_server_sockets(
 *         server_id,
 */

  /* function exit code */
  __pyx_L1_error:;
  __Pyx_XDECREF(__pyx_t_5);
  __Pyx_XDECREF(__pyx_t_6);
  __Pyx_AddTraceback("grpc._cython.cygrpc.channelz_get_server_sockets", __pyx_clineno, __pyx_lineno, __pyx_filename);
  __pyx_r = NULL;
  __pyx_L0:;
  __Pyx_XGIVEREF(__pyx_r);
  __Pyx_RefNannyFinishContext();
  return __pyx_r;
}

/* "src/python/grpcio/grpc/_cython/_cygrpc/channelz.pyx.pxi":52
 *     return c_returned_str
 * 
 * def channelz_get_channel(channel_id):             # <<<<<<<<<<<<<<
 *     cdef char *c_returned_str = grpc_channelz_get_channel(channel_id)
 *     if c_returned_str == NULL:
 */

/* Python wrapper */
static PyObject *__pyx_pw_4grpc_7_cython_6cygrpc_9channelz_get_channel(PyObject *__pyx_self, PyObject *__pyx_v_channel_id); /*proto*/
static PyMethodDef __pyx_mdef_4grpc_7_cython_6cygrpc_9channelz_get_channel = {"channelz_get_channel", (PyCFunction)__pyx_pw_4grpc_7_cython_6cygrpc_9channelz_get_channel, METH_O, 0};
static PyObject *__pyx_pw_4grpc_7_cython_6cygrpc_9channelz_get_channel(PyObject *__pyx_self, PyObject *__pyx_v_channel_id) {
  PyObject *__pyx_r = 0;
  __Pyx_RefNannyDeclarations
  __Pyx_RefNannySetupContext("channelz_get_channel (wrapper)", 0);
  __pyx_r = __pyx_pf_4grpc_7_cython_6cygrpc_8channelz_get_channel(__pyx_self, ((PyObject *)__pyx_v_channel_id));

  /* function exit code */
  __Pyx_RefNannyFinishContext();
  return __pyx_r;
}

static PyObject *__pyx_pf_4grpc_7_cython_6cygrpc_8channelz_get_channel(CYTHON_UNUSED PyObject *__pyx_self, PyObject *__pyx_v_channel_id) {
  char *__pyx_v_c_returned_str;
  PyObject *__pyx_r = NULL;
  __Pyx_RefNannyDeclarations
  __pyx_t_4grpc_7_cython_6cygrpc_intptr_t __pyx_t_1;
  int __pyx_t_2;
  PyObject *__pyx_t_3 = NULL;
  PyObject *__pyx_t_4 = NULL;
  __Pyx_RefNannySetupContext("channelz_get_channel", 0);

  /* "src/python/grpcio/grpc/_cython/_cygrpc/channelz.pyx.pxi":53
 * 
 * def channelz_get_channel(channel_id):
 *     cdef char *c_returned_str = grpc_channelz_get_channel(channel_id)             # <<<<<<<<<<<<<<
 *     if c_returned_str == NULL:
 *         raise ValueError('Failed to get the channel, please ensure your' \
 */
  __pyx_t_1 = PyInt_AsSsize_t(__pyx_v_channel_id); if (unlikely((__pyx_t_1 == (Py_ssize_t)-1) && PyErr_Occurred())) __PYX_ERR(9, 53, __pyx_L1_error)
  __pyx_v_c_returned_str = grpc_channelz_get_channel(__pyx_t_1);

  /* "src/python/grpcio/grpc/_cython/_cygrpc/channelz.pyx.pxi":54
 * def channelz_get_channel(channel_id):
 *     cdef char *c_returned_str = grpc_channelz_get_channel(channel_id)
 *     if c_returned_str == NULL:             # <<<<<<<<<<<<<<
 *         raise ValueError('Failed to get the channel, please ensure your' \
 *                          ' channel_id==%s is valid' % (channel_id))
 */
  __pyx_t_2 = ((__pyx_v_c_returned_str == NULL) != 0);
  if (unlikely(__pyx_t_2)) {

    /* "src/python/grpcio/grpc/_cython/_cygrpc/channelz.pyx.pxi":56
 *     if c_returned_str == NULL:
 *         raise ValueError('Failed to get the channel, please ensure your' \
 *                          ' channel_id==%s is valid' % (channel_id))             # <<<<<<<<<<<<<<
 *     return c_returned_str
 * 
 */
    __pyx_t_3 = __Pyx_PyString_Format(__pyx_kp_s_Failed_to_get_the_channel_please, __pyx_v_channel_id); if (unlikely(!__pyx_t_3)) __PYX_ERR(9, 56, __pyx_L1_error)
    __Pyx_GOTREF(__pyx_t_3);

    /* "src/python/grpcio/grpc/_cython/_cygrpc/channelz.pyx.pxi":55
 *     cdef char *c_returned_str = grpc_channelz_get_channel(channel_id)
 *     if c_returned_str == NULL:
 *         raise ValueError('Failed to get the channel, please ensure your' \             # <<<<<<<<<<<<<<
 *                          ' channel_id==%s is valid' % (channel_id))
 *     return c_returned_str
 */
    __pyx_t_4 = __Pyx_PyObject_CallOneArg(__pyx_builtin_ValueError, __pyx_t_3); if (unlikely(!__pyx_t_4)) __PYX_ERR(9, 55, __pyx_L1_error)
    __Pyx_GOTREF(__pyx_t_4);
    __Pyx_DECREF(__pyx_t_3); __pyx_t_3 = 0;
    __Pyx_Raise(__pyx_t_4, 0, 0, 0);
    __Pyx_DECREF(__pyx_t_4); __pyx_t_4 = 0;
    __PYX_ERR(9, 55, __pyx_L1_error)

    /* "src/python/grpcio/grpc/_cython/_cygrpc/channelz.pyx.pxi":54
 * def channelz_get_channel(channel_id):
 *     cdef char *c_returned_str = grpc_channelz_get_channel(channel_id)
 *     if c_returned_str == NULL:             # <<<<<<<<<<<<<<
 *         raise ValueError('Failed to get the channel, please ensure your' \
 *                          ' channel_id==%s is valid' % (channel_id))
 */
  }

  /* "src/python/grpcio/grpc/_cython/_cygrpc/channelz.pyx.pxi":57
 *         raise ValueError('Failed to get the channel, please ensure your' \
 *                          ' channel_id==%s is valid' % (channel_id))
 *     return c_returned_str             # <<<<<<<<<<<<<<
 * 
 * def channelz_get_subchannel(subchannel_id):
 */
  __Pyx_XDECREF(__pyx_r);
  __pyx_t_4 = __Pyx_PyBytes_FromString(__pyx_v_c_returned_str); if (unlikely(!__pyx_t_4)) __PYX_ERR(9, 57, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_4);
  __pyx_r = __pyx_t_4;
  __pyx_t_4 = 0;
  goto __pyx_L0;

  /* "src/python/grpcio/grpc/_cython/_cygrpc/channelz.pyx.pxi":52
 *     return c_returned_str
 * 
 * def channelz_get_channel(channel_id):             # <<<<<<<<<<<<<<
 *     cdef char *c_returned_str = grpc_channelz_get_channel(channel_id)
 *     if c_returned_str == NULL:
 */

  /* function exit code */
  __pyx_L1_error:;
  __Pyx_XDECREF(__pyx_t_3);
  __Pyx_XDECREF(__pyx_t_4);
  __Pyx_AddTraceback("grpc._cython.cygrpc.channelz_get_channel", __pyx_clineno, __pyx_lineno, __pyx_filename);
  __pyx_r = NULL;
  __pyx_L0:;
  __Pyx_XGIVEREF(__pyx_r);
  __Pyx_RefNannyFinishContext();
  return __pyx_r;
}

/* "src/python/grpcio/grpc/_cython/_cygrpc/channelz.pyx.pxi":59
 *     return c_returned_str
 * 
 * def channelz_get_subchannel(subchannel_id):             # <<<<<<<<<<<<<<
 *     cdef char *c_returned_str = grpc_channelz_get_subchannel(subchannel_id)
 *     if c_returned_str == NULL:
 */

/* Python wrapper */
static PyObject *__pyx_pw_4grpc_7_cython_6cygrpc_11channelz_get_subchannel(PyObject *__pyx_self, PyObject *__pyx_v_subchannel_id); /*proto*/
static PyMethodDef __pyx_mdef_4grpc_7_cython_6cygrpc_11channelz_get_subchannel = {"channelz_get_subchannel", (PyCFunction)__pyx_pw_4grpc_7_cython_6cygrpc_11channelz_get_subchannel, METH_O, 0};
static PyObject *__pyx_pw_4grpc_7_cython_6cygrpc_11channelz_get_subchannel(PyObject *__pyx_self, PyObject *__pyx_v_subchannel_id) {
  PyObject *__pyx_r = 0;
  __Pyx_RefNannyDeclarations
  __Pyx_RefNannySetupContext("channelz_get_subchannel (wrapper)", 0);
  __pyx_r = __pyx_pf_4grpc_7_cython_6cygrpc_10channelz_get_subchannel(__pyx_self, ((PyObject *)__pyx_v_subchannel_id));

  /* function exit code */
  __Pyx_RefNannyFinishContext();
  return __pyx_r;
}

static PyObject *__pyx_pf_4grpc_7_cython_6cygrpc_10channelz_get_subchannel(CYTHON_UNUSED PyObject *__pyx_self, PyObject *__pyx_v_subchannel_id) {
  char *__pyx_v_c_returned_str;
  PyObject *__pyx_r = NULL;
  __Pyx_RefNannyDeclarations
  __pyx_t_4grpc_7_cython_6cygrpc_intptr_t __pyx_t_1;
  int __pyx_t_2;
  PyObject *__pyx_t_3 = NULL;
  PyObject *__pyx_t_4 = NULL;
  __Pyx_RefNannySetupContext("channelz_get_subchannel", 0);

  /* "src/python/grpcio/grpc/_cython/_cygrpc/channelz.pyx.pxi":60
 * 
 * def channelz_get_subchannel(subchannel_id):
 *     cdef char *c_returned_str = grpc_channelz_get_subchannel(subchannel_id)             # <<<<<<<<<<<<<<
 *     if c_returned_str == NULL:
 *         raise ValueError('Failed to get the subchannel, please ensure your' \
 */
  __pyx_t_1 = PyInt_AsSsize_t(__pyx_v_subchannel_id); if (unlikely((__pyx_t_1 == (Py_ssize_t)-1) && PyErr_Occurred())) __PYX_ERR(9, 60, __pyx_L1_error)
  __pyx_v_c_returned_str = grpc_channelz_get_subchannel(__pyx_t_1);

  /* "src/python/grpcio/grpc/_cython/_cygrpc/channelz.pyx.pxi":61
 * def channelz_get_subchannel(subchannel_id):
 *     cdef char *c_returned_str = grpc_channelz_get_subchannel(subchannel_id)
 *     if c_returned_str == NULL:             # <<<<<<<<<<<<<<
 *         raise ValueError('Failed to get the subchannel, please ensure your' \
 *                          ' subchannel_id==%s is valid' % (subchannel_id))
 */
  __pyx_t_2 = ((__pyx_v_c_returned_str == NULL) != 0);
  if (unlikely(__pyx_t_2)) {

    /* "src/python/grpcio/grpc/_cython/_cygrpc/channelz.pyx.pxi":63
 *     if c_returned_str == NULL:
 *         raise ValueError('Failed to get the subchannel, please ensure your' \
 *                          ' subchannel_id==%s is valid' % (subchannel_id))             # <<<<<<<<<<<<<<
 *     return c_returned_str
 * 
 */
    __pyx_t_3 = __Pyx_PyString_Format(__pyx_kp_s_Failed_to_get_the_subchannel_ple, __pyx_v_subchannel_id); if (unlikely(!__pyx_t_3)) __PYX_ERR(9, 63, __pyx_L1_error)
    __Pyx_GOTREF(__pyx_t_3);

    /* "src/python/grpcio/grpc/_cython/_cygrpc/channelz.pyx.pxi":62
 *     cdef char *c_returned_str = grpc_channelz_get_subchannel(subchannel_id)
 *     if c_returned_str == NULL:
 *         raise ValueError('Failed to get the subchannel, please ensure your' \             # <<<<<<<<<<<<<<
 *                          ' subchannel_id==%s is valid' % (subchannel_id))
 *     return c_returned_str
 */
    __pyx_t_4 = __Pyx_PyObject_CallOneArg(__pyx_builtin_ValueError, __pyx_t_3); if (unlikely(!__pyx_t_4)) __PYX_ERR(9, 62, __pyx_L1_error)
    __Pyx_GOTREF(__pyx_t_4);
    __Pyx_DECREF(__pyx_t_3); __pyx_t_3 = 0;
    __Pyx_Raise(__pyx_t_4, 0, 0, 0);
    __Pyx_DECREF(__pyx_t_4); __pyx_t_4 = 0;
    __PYX_ERR(9, 62, __pyx_L1_error)

    /* "src/python/grpcio/grpc/_cython/_cygrpc/channelz.pyx.pxi":61
 * def channelz_get_subchannel(subchannel_id):
 *     cdef char *c_returned_str = grpc_channelz_get_subchannel(subchannel_id)
 *     if c_returned_str == NULL:             # <<<<<<<<<<<<<<
 *         raise ValueError('Failed to get the subchannel, please ensure your' \
 *                          ' subchannel_id==%s is valid' % (subchannel_id))
 */
  }

  /* "src/python/grpcio/grpc/_cython/_cygrpc/channelz.pyx.pxi":64
 *         raise ValueError('Failed to get the subchannel, please ensure your' \
 *                          ' subchannel_id==%s is valid' % (subchannel_id))
 *     return c_returned_str             # <<<<<<<<<<<<<<
 * 
 * def channelz_get_socket(socket_id):
 */
  __Pyx_XDECREF(__pyx_r);
  __pyx_t_4 = __Pyx_PyBytes_FromString(__pyx_v_c_returned_str); if (unlikely(!__pyx_t_4)) __PYX_ERR(9, 64, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_4);
  __pyx_r = __pyx_t_4;
  __pyx_t_4 = 0;
  goto __pyx_L0;

  /* "src/python/grpcio/grpc/_cython/_cygrpc/channelz.pyx.pxi":59
 *     return c_returned_str
 * 
 * def channelz_get_subchannel(subchannel_id):             # <<<<<<<<<<<<<<
 *     cdef char *c_returned_str = grpc_channelz_get_subchannel(subchannel_id)
 *     if c_returned_str == NULL:
 */

  /* function exit code */
  __pyx_L1_error:;
  __Pyx_XDECREF(__pyx_t_3);
  __Pyx_XDECREF(__pyx_t_4);
  __Pyx_AddTraceback("grpc._cython.cygrpc.channelz_get_subchannel", __pyx_clineno, __pyx_lineno, __pyx_filename);
  __pyx_r = NULL;
  __pyx_L0:;
  __Pyx_XGIVEREF(__pyx_r);
  __Pyx_RefNannyFinishContext();
  return __pyx_r;
}

/* "src/python/grpcio/grpc/_cython/_cygrpc/channelz.pyx.pxi":66
 *     return c_returned_str
 * 
 * def channelz_get_socket(socket_id):             # <<<<<<<<<<<<<<
 *     cdef char *c_returned_str = grpc_channelz_get_socket(socket_id)
 *     if c_returned_str == NULL:
 */

/* Python wrapper */
static PyObject *__pyx_pw_4grpc_7_cython_6cygrpc_13channelz_get_socket(PyObject *__pyx_self, PyObject *__pyx_v_socket_id); /*proto*/
static PyMethodDef __pyx_mdef_4grpc_7_cython_6cygrpc_13channelz_get_socket = {"channelz_get_socket", (PyCFunction)__pyx_pw_4grpc_7_cython_6cygrpc_13channelz_get_socket, METH_O, 0};
static PyObject *__pyx_pw_4grpc_7_cython_6cygrpc_13channelz_get_socket(PyObject *__pyx_self, PyObject *__pyx_v_socket_id) {
  PyObject *__pyx_r = 0;
  __Pyx_RefNannyDeclarations
  __Pyx_RefNannySetupContext("channelz_get_socket (wrapper)", 0);
  __pyx_r = __pyx_pf_4grpc_7_cython_6cygrpc_12channelz_get_socket(__pyx_self, ((PyObject *)__pyx_v_socket_id));

  /* function exit code */
  __Pyx_RefNannyFinishContext();
  return __pyx_r;
}

static PyObject *__pyx_pf_4grpc_7_cython_6cygrpc_12channelz_get_socket(CYTHON_UNUSED PyObject *__pyx_self, PyObject *__pyx_v_socket_id) {
  char *__pyx_v_c_returned_str;
  PyObject *__pyx_r = NULL;
  __Pyx_RefNannyDeclarations
  __pyx_t_4grpc_7_cython_6cygrpc_intptr_t __pyx_t_1;
  int __pyx_t_2;
  PyObject *__pyx_t_3 = NULL;
  PyObject *__pyx_t_4 = NULL;
  __Pyx_RefNannySetupContext("channelz_get_socket", 0);

  /* "src/python/grpcio/grpc/_cython/_cygrpc/channelz.pyx.pxi":67
 * 
 * def channelz_get_socket(socket_id):
 *     cdef char *c_returned_str = grpc_channelz_get_socket(socket_id)             # <<<<<<<<<<<<<<
 *     if c_returned_str == NULL:
 *         raise ValueError('Failed to get the socket, please ensure your' \
 */
  __pyx_t_1 = PyInt_AsSsize_t(__pyx_v_socket_id); if (unlikely((__pyx_t_1 == (Py_ssize_t)-1) && PyErr_Occurred())) __PYX_ERR(9, 67, __pyx_L1_error)
  __pyx_v_c_returned_str = grpc_channelz_get_socket(__pyx_t_1);

  /* "src/python/grpcio/grpc/_cython/_cygrpc/channelz.pyx.pxi":68
 * def channelz_get_socket(socket_id):
 *     cdef char *c_returned_str = grpc_channelz_get_socket(socket_id)
 *     if c_returned_str == NULL:             # <<<<<<<<<<<<<<
 *         raise ValueError('Failed to get the socket, please ensure your' \
 *                          ' socket_id==%s is valid' % (socket_id))
 */
  __pyx_t_2 = ((__pyx_v_c_returned_str == NULL) != 0);
  if (unlikely(__pyx_t_2)) {

    /* "src/python/grpcio/grpc/_cython/_cygrpc/channelz.pyx.pxi":70
 *     if c_returned_str == NULL:
 *         raise ValueError('Failed to get the socket, please ensure your' \
 *                          ' socket_id==%s is valid' % (socket_id))             # <<<<<<<<<<<<<<
 *     return c_returned_str
 */
    __pyx_t_3 = __Pyx_PyString_Format(__pyx_kp_s_Failed_to_get_the_socket_please, __pyx_v_socket_id); if (unlikely(!__pyx_t_3)) __PYX_ERR(9, 70, __pyx_L1_error)
    __Pyx_GOTREF(__pyx_t_3);

    /* "src/python/grpcio/grpc/_cython/_cygrpc/channelz.pyx.pxi":69
 *     cdef char *c_returned_str = grpc_channelz_get_socket(socket_id)
 *     if c_returned_str == NULL:
 *         raise ValueError('Failed to get the socket, please ensure your' \             # <<<<<<<<<<<<<<
 *                          ' socket_id==%s is valid' % (socket_id))
 *     return c_returned_str
 */
    __pyx_t_4 = __Pyx_PyObject_CallOneArg(__pyx_builtin_ValueError, __pyx_t_3); if (unlikely(!__pyx_t_4)) __PYX_ERR(9, 69, __pyx_L1_error)
    __Pyx_GOTREF(__pyx_t_4);
    __Pyx_DECREF(__pyx_t_3); __pyx_t_3 = 0;
    __Pyx_Raise(__pyx_t_4, 0, 0, 0);
    __Pyx_DECREF(__pyx_t_4); __pyx_t_4 = 0;
    __PYX_ERR(9, 69, __pyx_L1_error)

    /* "src/python/grpcio/grpc/_cython/_cygrpc/channelz.pyx.pxi":68
 * def channelz_get_socket(socket_id):
 *     cdef char *c_returned_str = grpc_channelz_get_socket(socket_id)
 *     if c_returned_str == NULL:             # <<<<<<<<<<<<<<
 *         raise ValueError('Failed to get the socket, please ensure your' \
 *                          ' socket_id==%s is valid' % (socket_id))
 */
  }

  /* "src/python/grpcio/grpc/_cython/_cygrpc/channelz.pyx.pxi":71
 *         raise ValueError('Failed to get the socket, please ensure your' \
 *                          ' socket_id==%s is valid' % (socket_id))
 *     return c_returned_str             # <<<<<<<<<<<<<<
 */
  __Pyx_XDECREF(__pyx_r);
  __pyx_t_4 = __Pyx_PyBytes_FromString(__pyx_v_c_returned_str); if (unlikely(!__pyx_t_4)) __PYX_ERR(9, 71, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_4);
  __pyx_r = __pyx_t_4;
  __pyx_t_4 = 0;
  goto __pyx_L0;

  /* "src/python/grpcio/grpc/_cython/_cygrpc/channelz.pyx.pxi":66
 *     return c_returned_str
 * 
 * def channelz_get_socket(socket_id):             # <<<<<<<<<<<<<<
 *     cdef char *c_returned_str = grpc_channelz_get_socket(socket_id)
 *     if c_returned_str == NULL:
 */

  /* function exit code */
  __pyx_L1_error:;
  __Pyx_XDECREF(__pyx_t_3);
  __Pyx_XDECREF(__pyx_t_4);
  __Pyx_AddTraceback("grpc._cython.cygrpc.channelz_get_socket", __pyx_clineno, __pyx_lineno, __pyx_filename);
  __pyx_r = NULL;
  __pyx_L0:;
  __Pyx_XGIVEREF(__pyx_r);
  __Pyx_RefNannyFinishContext();
  return __pyx_r;
}

/* "src/python/grpcio/grpc/_cython/_cygrpc/credentials.pyx.pxi":21
 * 
 * 
 * def _spawn_callback_in_thread(cb_func, args):             # <<<<<<<<<<<<<<
 *   ForkManagedThread(target=cb_func, args=args).start()
 * 
 */

/* Python wrapper */
static PyObject *__pyx_pw_4grpc_7_cython_6cygrpc_15_spawn_callback_in_thread(PyObject *__pyx_self, PyObject *__pyx_args, PyObject *__pyx_kwds); /*proto*/
static PyMethodDef __pyx_mdef_4grpc_7_cython_6cygrpc_15_spawn_callback_in_thread = {"_spawn_callback_in_thread", (PyCFunction)__pyx_pw_4grpc_7_cython_6cygrpc_15_spawn_callback_in_thread, METH_VARARGS|METH_KEYWORDS, 0};
static PyObject *__pyx_pw_4grpc_7_cython_6cygrpc_15_spawn_callback_in_thread(PyObject *__pyx_self, PyObject *__pyx_args, PyObject *__pyx_kwds) {
  PyObject *__pyx_v_cb_func = 0;
  PyObject *__pyx_v_args = 0;
  PyObject *__pyx_r = 0;
  __Pyx_RefNannyDeclarations
  __Pyx_RefNannySetupContext("_spawn_callback_in_thread (wrapper)", 0);
  {
    static PyObject **__pyx_pyargnames[] = {&__pyx_n_s_cb_func,&__pyx_n_s_args,0};
    PyObject* values[2] = {0,0};
    if (unlikely(__pyx_kwds)) {
      Py_ssize_t kw_args;
      const Py_ssize_t pos_args = PyTuple_GET_SIZE(__pyx_args);
      switch (pos_args) {
        case  2: values[1] = PyTuple_GET_ITEM(__pyx_args, 1);
        CYTHON_FALLTHROUGH;
        case  1: values[0] = PyTuple_GET_ITEM(__pyx_args, 0);
        CYTHON_FALLTHROUGH;
        case  0: break;
        default: goto __pyx_L5_argtuple_error;
      }
      kw_args = PyDict_Size(__pyx_kwds);
      switch (pos_args) {
        case  0:
        if (likely((values[0] = __Pyx_PyDict_GetItemStr(__pyx_kwds, __pyx_n_s_cb_func)) != 0)) kw_args--;
        else goto __pyx_L5_argtuple_error;
        CYTHON_FALLTHROUGH;
        case  1:
        if (likely((values[1] = __Pyx_PyDict_GetItemStr(__pyx_kwds, __pyx_n_s_args)) != 0)) kw_args--;
        else {
          __Pyx_RaiseArgtupleInvalid("_spawn_callback_in_thread", 1, 2, 2, 1); __PYX_ERR(4, 21, __pyx_L3_error)
        }
      }
      if (unlikely(kw_args > 0)) {
        if (unlikely(__Pyx_ParseOptionalKeywords(__pyx_kwds, __pyx_pyargnames, 0, values, pos_args, "_spawn_callback_in_thread") < 0)) __PYX_ERR(4, 21, __pyx_L3_error)
      }
    } else if (PyTuple_GET_SIZE(__pyx_args) != 2) {
      goto __pyx_L5_argtuple_error;
    } else {
      values[0] = PyTuple_GET_ITEM(__pyx_args, 0);
      values[1] = PyTuple_GET_ITEM(__pyx_args, 1);
    }
    __pyx_v_cb_func = values[0];
    __pyx_v_args = values[1];
  }
  goto __pyx_L4_argument_unpacking_done;
  __pyx_L5_argtuple_error:;
  __Pyx_RaiseArgtupleInvalid("_spawn_callback_in_thread", 1, 2, 2, PyTuple_GET_SIZE(__pyx_args)); __PYX_ERR(4, 21, __pyx_L3_error)
  __pyx_L3_error:;
  __Pyx_AddTraceback("grpc._cython.cygrpc._spawn_callback_in_thread", __pyx_clineno, __pyx_lineno, __pyx_filename);
  __Pyx_RefNannyFinishContext();
  return NULL;
  __pyx_L4_argument_unpacking_done:;
  __pyx_r = __pyx_pf_4grpc_7_cython_6cygrpc_14_spawn_callback_in_thread(__pyx_self, __pyx_v_cb_func, __pyx_v_args);

  /* function exit code */
  __Pyx_RefNannyFinishContext();
  return __pyx_r;
}

static PyObject *__pyx_pf_4grpc_7_cython_6cygrpc_14_spawn_callback_in_thread(CYTHON_UNUSED PyObject *__pyx_self, PyObject *__pyx_v_cb_func, PyObject *__pyx_v_args) {
  PyObject *__pyx_r = NULL;
  __Pyx_RefNannyDeclarations
  PyObject *__pyx_t_1 = NULL;
  PyObject *__pyx_t_2 = NULL;
  PyObject *__pyx_t_3 = NULL;
  PyObject *__pyx_t_4 = NULL;
  __Pyx_RefNannySetupContext("_spawn_callback_in_thread", 0);

  /* "src/python/grpcio/grpc/_cython/_cygrpc/credentials.pyx.pxi":22
 * 
 * def _spawn_callback_in_thread(cb_func, args):
 *   ForkManagedThread(target=cb_func, args=args).start()             # <<<<<<<<<<<<<<
 * 
 * async_callback_func = _spawn_callback_in_thread
 */
  __pyx_t_2 = __Pyx_GetModuleGlobalName(__pyx_n_s_ForkManagedThread); if (unlikely(!__pyx_t_2)) __PYX_ERR(4, 22, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_2);
  __pyx_t_3 = __Pyx_PyDict_NewPresized(2); if (unlikely(!__pyx_t_3)) __PYX_ERR(4, 22, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_3);
  if (PyDict_SetItem(__pyx_t_3, __pyx_n_s_target, __pyx_v_cb_func) < 0) __PYX_ERR(4, 22, __pyx_L1_error)
  if (PyDict_SetItem(__pyx_t_3, __pyx_n_s_args, __pyx_v_args) < 0) __PYX_ERR(4, 22, __pyx_L1_error)
  __pyx_t_4 = __Pyx_PyObject_Call(__pyx_t_2, __pyx_empty_tuple, __pyx_t_3); if (unlikely(!__pyx_t_4)) __PYX_ERR(4, 22, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_4);
  __Pyx_DECREF(__pyx_t_2); __pyx_t_2 = 0;
  __Pyx_DECREF(__pyx_t_3); __pyx_t_3 = 0;
  __pyx_t_3 = __Pyx_PyObject_GetAttrStr(__pyx_t_4, __pyx_n_s_start); if (unlikely(!__pyx_t_3)) __PYX_ERR(4, 22, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_3);
  __Pyx_DECREF(__pyx_t_4); __pyx_t_4 = 0;
  __pyx_t_4 = NULL;
  if (CYTHON_UNPACK_METHODS && likely(PyMethod_Check(__pyx_t_3))) {
    __pyx_t_4 = PyMethod_GET_SELF(__pyx_t_3);
    if (likely(__pyx_t_4)) {
      PyObject* function = PyMethod_GET_FUNCTION(__pyx_t_3);
      __Pyx_INCREF(__pyx_t_4);
      __Pyx_INCREF(function);
      __Pyx_DECREF_SET(__pyx_t_3, function);
    }
  }
  if (__pyx_t_4) {
    __pyx_t_1 = __Pyx_PyObject_CallOneArg(__pyx_t_3, __pyx_t_4); if (unlikely(!__pyx_t_1)) __PYX_ERR(4, 22, __pyx_L1_error)
    __Pyx_DECREF(__pyx_t_4); __pyx_t_4 = 0;
  } else {
    __pyx_t_1 = __Pyx_PyObject_CallNoArg(__pyx_t_3); if (unlikely(!__pyx_t_1)) __PYX_ERR(4, 22, __pyx_L1_error)
  }
  __Pyx_GOTREF(__pyx_t_1);
  __Pyx_DECREF(__pyx_t_3); __pyx_t_3 = 0;
  __Pyx_DECREF(__pyx_t_1); __pyx_t_1 = 0;

  /* "src/python/grpcio/grpc/_cython/_cygrpc/credentials.pyx.pxi":21
 * 
 * 
 * def _spawn_callback_in_thread(cb_func, args):             # <<<<<<<<<<<<<<
 *   ForkManagedThread(target=cb_func, args=args).start()
 * 
 */

  /* function exit code */
  __pyx_r = Py_None; __Pyx_INCREF(Py_None);
  goto __pyx_L0;
  __pyx_L1_error:;
  __Pyx_XDECREF(__pyx_t_1);
  __Pyx_XDECREF(__pyx_t_2);
  __Pyx_XDECREF(__pyx_t_3);
  __Pyx_XDECREF(__pyx_t_4);
  __Pyx_AddTraceback("grpc._cython.cygrpc._spawn_callback_in_thread", __pyx_clineno, __pyx_lineno, __pyx_filename);
  __pyx_r = NULL;
  __pyx_L0:;
  __Pyx_XGIVEREF(__pyx_r);
  __Pyx_RefNannyFinishContext();
  return __pyx_r;
}

/* "src/python/grpcio/grpc/_cython/_cygrpc/credentials.pyx.pxi":26
 * async_callback_func = _spawn_callback_in_thread
 * 
 * def set_async_callback_func(callback_func):             # <<<<<<<<<<<<<<
 *   global async_callback_func
 *   async_callback_func = callback_func
 */

/* Python wrapper */
static PyObject *__pyx_pw_4grpc_7_cython_6cygrpc_17set_async_callback_func(PyObject *__pyx_self, PyObject *__pyx_v_callback_func); /*proto*/
static PyMethodDef __pyx_mdef_4grpc_7_cython_6cygrpc_17set_async_callback_func = {"set_async_callback_func", (PyCFunction)__pyx_pw_4grpc_7_cython_6cygrpc_17set_async_callback_func, METH_O, 0};
static PyObject *__pyx_pw_4grpc_7_cython_6cygrpc_17set_async_callback_func(PyObject *__pyx_self, PyObject *__pyx_v_callback_func) {
  PyObject *__pyx_r = 0;
  __Pyx_RefNannyDeclarations
  __Pyx_RefNannySetupContext("set_async_callback_func (wrapper)", 0);
  __pyx_r = __pyx_pf_4grpc_7_cython_6cygrpc_16set_async_callback_func(__pyx_self, ((PyObject *)__pyx_v_callback_func));

  /* function exit code */
  __Pyx_RefNannyFinishContext();
  return __pyx_r;
}

static PyObject *__pyx_pf_4grpc_7_cython_6cygrpc_16set_async_callback_func(CYTHON_UNUSED PyObject *__pyx_self, PyObject *__pyx_v_callback_func) {
  PyObject *__pyx_r = NULL;
  __Pyx_RefNannyDeclarations
  __Pyx_RefNannySetupContext("set_async_callback_func", 0);

  /* "src/python/grpcio/grpc/_cython/_cygrpc/credentials.pyx.pxi":28
 * def set_async_callback_func(callback_func):
 *   global async_callback_func
 *   async_callback_func = callback_func             # <<<<<<<<<<<<<<
 * 
 * def _spawn_callback_async(callback, args):
 */
  if (PyDict_SetItem(__pyx_d, __pyx_n_s_async_callback_func, __pyx_v_callback_func) < 0) __PYX_ERR(4, 28, __pyx_L1_error)

  /* "src/python/grpcio/grpc/_cython/_cygrpc/credentials.pyx.pxi":26
 * async_callback_func = _spawn_callback_in_thread
 * 
 * def set_async_callback_func(callback_func):             # <<<<<<<<<<<<<<
 *   global async_callback_func
 *   async_callback_func = callback_func
 */

  /* function exit code */
  __pyx_r = Py_None; __Pyx_INCREF(Py_None);
  goto __pyx_L0;
  __pyx_L1_error:;
  __Pyx_AddTraceback("grpc._cython.cygrpc.set_async_callback_func", __pyx_clineno, __pyx_lineno, __pyx_filename);
  __pyx_r = NULL;
  __pyx_L0:;
  __Pyx_XGIVEREF(__pyx_r);
  __Pyx_RefNannyFinishContext();
  return __pyx_r;
}

/* "src/python/grpcio/grpc/_cython/_cygrpc/credentials.pyx.pxi":30
 *   async_callback_func = callback_func
 * 
 * def _spawn_callback_async(callback, args):             # <<<<<<<<<<<<<<
 *   async_callback_func(callback, args)
 * 
 */

/* Python wrapper */
static PyObject *__pyx_pw_4grpc_7_cython_6cygrpc_19_spawn_callback_async(PyObject *__pyx_self, PyObject *__pyx_args, PyObject *__pyx_kwds); /*proto*/
static PyMethodDef __pyx_mdef_4grpc_7_cython_6cygrpc_19_spawn_callback_async = {"_spawn_callback_async", (PyCFunction)__pyx_pw_4grpc_7_cython_6cygrpc_19_spawn_callback_async, METH_VARARGS|METH_KEYWORDS, 0};
static PyObject *__pyx_pw_4grpc_7_cython_6cygrpc_19_spawn_callback_async(PyObject *__pyx_self, PyObject *__pyx_args, PyObject *__pyx_kwds) {
  PyObject *__pyx_v_callback = 0;
  PyObject *__pyx_v_args = 0;
  PyObject *__pyx_r = 0;
  __Pyx_RefNannyDeclarations
  __Pyx_RefNannySetupContext("_spawn_callback_async (wrapper)", 0);
  {
    static PyObject **__pyx_pyargnames[] = {&__pyx_n_s_callback,&__pyx_n_s_args,0};
    PyObject* values[2] = {0,0};
    if (unlikely(__pyx_kwds)) {
      Py_ssize_t kw_args;
      const Py_ssize_t pos_args = PyTuple_GET_SIZE(__pyx_args);
      switch (pos_args) {
        case  2: values[1] = PyTuple_GET_ITEM(__pyx_args, 1);
        CYTHON_FALLTHROUGH;
        case  1: values[0] = PyTuple_GET_ITEM(__pyx_args, 0);
        CYTHON_FALLTHROUGH;
        case  0: break;
        default: goto __pyx_L5_argtuple_error;
      }
      kw_args = PyDict_Size(__pyx_kwds);
      switch (pos_args) {
        case  0:
        if (likely((values[0] = __Pyx_PyDict_GetItemStr(__pyx_kwds, __pyx_n_s_callback)) != 0)) kw_args--;
        else goto __pyx_L5_argtuple_error;
        CYTHON_FALLTHROUGH;
        case  1:
        if (likely((values[1] = __Pyx_PyDict_GetItemStr(__pyx_kwds, __pyx_n_s_args)) != 0)) kw_args--;
        else {
          __Pyx_RaiseArgtupleInvalid("_spawn_callback_async", 1, 2, 2, 1); __PYX_ERR(4, 30, __pyx_L3_error)
        }
      }
      if (unlikely(kw_args > 0)) {
        if (unlikely(__Pyx_ParseOptionalKeywords(__pyx_kwds, __pyx_pyargnames, 0, values, pos_args, "_spawn_callback_async") < 0)) __PYX_ERR(4, 30, __pyx_L3_error)
      }
    } else if (PyTuple_GET_SIZE(__pyx_args) != 2) {
      goto __pyx_L5_argtuple_error;
    } else {
      values[0] = PyTuple_GET_ITEM(__pyx_args, 0);
      values[1] = PyTuple_GET_ITEM(__pyx_args, 1);
    }
    __pyx_v_callback = values[0];
    __pyx_v_args = values[1];
  }
  goto __pyx_L4_argument_unpacking_done;
  __pyx_L5_argtuple_error:;
  __Pyx_RaiseArgtupleInvalid("_spawn_callback_async", 1, 2, 2, PyTuple_GET_SIZE(__pyx_args)); __PYX_ERR(4, 30, __pyx_L3_error)
  __pyx_L3_error:;
  __Pyx_AddTraceback("grpc._cython.cygrpc._spawn_callback_async", __pyx_clineno, __pyx_lineno, __pyx_filename);
  __Pyx_RefNannyFinishContext();
  return NULL;
  __pyx_L4_argument_unpacking_done:;
  __pyx_r = __pyx_pf_4grpc_7_cython_6cygrpc_18_spawn_callback_async(__pyx_self, __pyx_v_callback, __pyx_v_args);

  /* function exit code */
  __Pyx_RefNannyFinishContext();
  return __pyx_r;
}

static PyObject *__pyx_pf_4grpc_7_cython_6cygrpc_18_spawn_callback_async(CYTHON_UNUSED PyObject *__pyx_self, PyObject *__pyx_v_callback, PyObject *__pyx_v_args) {
  PyObject *__pyx_r = NULL;
  __Pyx_RefNannyDeclarations
  PyObject *__pyx_t_1 = NULL;
  PyObject *__pyx_t_2 = NULL;
  PyObject *__pyx_t_3 = NULL;
  int __pyx_t_4;
  PyObject *__pyx_t_5 = NULL;
  __Pyx_RefNannySetupContext("_spawn_callback_async", 0);

  /* "src/python/grpcio/grpc/_cython/_cygrpc/credentials.pyx.pxi":31
 * 
 * def _spawn_callback_async(callback, args):
 *   async_callback_func(callback, args)             # <<<<<<<<<<<<<<
 * 
 * 
 */
  __pyx_t_2 = __Pyx_GetModuleGlobalName(__pyx_n_s_async_callback_func); if (unlikely(!__pyx_t_2)) __PYX_ERR(4, 31, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_2);
  __pyx_t_3 = NULL;
  __pyx_t_4 = 0;
  if (CYTHON_UNPACK_METHODS && unlikely(PyMethod_Check(__pyx_t_2))) {
    __pyx_t_3 = PyMethod_GET_SELF(__pyx_t_2);
    if (likely(__pyx_t_3)) {
      PyObject* function = PyMethod_GET_FUNCTION(__pyx_t_2);
      __Pyx_INCREF(__pyx_t_3);
      __Pyx_INCREF(function);
      __Pyx_DECREF_SET(__pyx_t_2, function);
      __pyx_t_4 = 1;
    }
  }
  #if CYTHON_FAST_PYCALL
  if (PyFunction_Check(__pyx_t_2)) {
    PyObject *__pyx_temp[3] = {__pyx_t_3, __pyx_v_callback, __pyx_v_args};
    __pyx_t_1 = __Pyx_PyFunction_FastCall(__pyx_t_2, __pyx_temp+1-__pyx_t_4, 2+__pyx_t_4); if (unlikely(!__pyx_t_1)) __PYX_ERR(4, 31, __pyx_L1_error)
    __Pyx_XDECREF(__pyx_t_3); __pyx_t_3 = 0;
    __Pyx_GOTREF(__pyx_t_1);
  } else
  #endif
  #if CYTHON_FAST_PYCCALL
  if (__Pyx_PyFastCFunction_Check(__pyx_t_2)) {
    PyObject *__pyx_temp[3] = {__pyx_t_3, __pyx_v_callback, __pyx_v_args};
    __pyx_t_1 = __Pyx_PyCFunction_FastCall(__pyx_t_2, __pyx_temp+1-__pyx_t_4, 2+__pyx_t_4); if (unlikely(!__pyx_t_1)) __PYX_ERR(4, 31, __pyx_L1_error)
    __Pyx_XDECREF(__pyx_t_3); __pyx_t_3 = 0;
    __Pyx_GOTREF(__pyx_t_1);
  } else
  #endif
  {
    __pyx_t_5 = PyTuple_New(2+__pyx_t_4); if (unlikely(!__pyx_t_5)) __PYX_ERR(4, 31, __pyx_L1_error)
    __Pyx_GOTREF(__pyx_t_5);
    if (__pyx_t_3) {
      __Pyx_GIVEREF(__pyx_t_3); PyTuple_SET_ITEM(__pyx_t_5, 0, __pyx_t_3); __pyx_t_3 = NULL;
    }
    __Pyx_INCREF(__pyx_v_callback);
    __Pyx_GIVEREF(__pyx_v_callback);
    PyTuple_SET_ITEM(__pyx_t_5, 0+__pyx_t_4, __pyx_v_callback);
    __Pyx_INCREF(__pyx_v_args);
    __Pyx_GIVEREF(__pyx_v_args);
    PyTuple_SET_ITEM(__pyx_t_5, 1+__pyx_t_4, __pyx_v_args);
    __pyx_t_1 = __Pyx_PyObject_Call(__pyx_t_2, __pyx_t_5, NULL); if (unlikely(!__pyx_t_1)) __PYX_ERR(4, 31, __pyx_L1_error)
    __Pyx_GOTREF(__pyx_t_1);
    __Pyx_DECREF(__pyx_t_5); __pyx_t_5 = 0;
  }
  __Pyx_DECREF(__pyx_t_2); __pyx_t_2 = 0;
  __Pyx_DECREF(__pyx_t_1); __pyx_t_1 = 0;

  /* "src/python/grpcio/grpc/_cython/_cygrpc/credentials.pyx.pxi":30
 *   async_callback_func = callback_func
 * 
 * def _spawn_callback_async(callback, args):             # <<<<<<<<<<<<<<
 *   async_callback_func(callback, args)
 * 
 */

  /* function exit code */
  __pyx_r = Py_None; __Pyx_INCREF(Py_None);
  goto __pyx_L0;
  __pyx_L1_error:;
  __Pyx_XDECREF(__pyx_t_1);
  __Pyx_XDECREF(__pyx_t_2);
  __Pyx_XDECREF(__pyx_t_3);
  __Pyx_XDECREF(__pyx_t_5);
  __Pyx_AddTraceback("grpc._cython.cygrpc._spawn_callback_async", __pyx_clineno, __pyx_lineno, __pyx_filename);
  __pyx_r = NULL;
  __pyx_L0:;
  __Pyx_XGIVEREF(__pyx_r);
  __Pyx_RefNannyFinishContext();
  return __pyx_r;
}

/* "src/python/grpcio/grpc/_cython/_cygrpc/credentials.pyx.pxi":36
 * cdef class CallCredentials:
 * 
 *   cdef grpc_call_credentials *c(self) except *:             # <<<<<<<<<<<<<<
 *     raise NotImplementedError()
 * 
 */

static grpc_call_credentials *__pyx_f_4grpc_7_cython_6cygrpc_15CallCredentials_c(CYTHON_UNUSED struct __pyx_obj_4grpc_7_cython_6cygrpc_CallCredentials *__pyx_v_self) {
  grpc_call_credentials *__pyx_r;
  __Pyx_RefNannyDeclarations
  PyObject *__pyx_t_1 = NULL;
  __Pyx_RefNannySetupContext("c", 0);

  /* "src/python/grpcio/grpc/_cython/_cygrpc/credentials.pyx.pxi":37
 * 
 *   cdef grpc_call_credentials *c(self) except *:
 *     raise NotImplementedError()             # <<<<<<<<<<<<<<
 * 
 * 
 */
  __pyx_t_1 = __Pyx_PyObject_CallNoArg(__pyx_builtin_NotImplementedError); if (unlikely(!__pyx_t_1)) __PYX_ERR(4, 37, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_1);
  __Pyx_Raise(__pyx_t_1, 0, 0, 0);
  __Pyx_DECREF(__pyx_t_1); __pyx_t_1 = 0;
  __PYX_ERR(4, 37, __pyx_L1_error)

  /* "src/python/grpcio/grpc/_cython/_cygrpc/credentials.pyx.pxi":36
 * cdef class CallCredentials:
 * 
 *   cdef grpc_call_credentials *c(self) except *:             # <<<<<<<<<<<<<<
 *     raise NotImplementedError()
 * 
 */

  /* function exit code */
  __pyx_L1_error:;
  __Pyx_XDECREF(__pyx_t_1);
  __Pyx_AddTraceback("grpc._cython.cygrpc.CallCredentials.c", __pyx_clineno, __pyx_lineno, __pyx_filename);
  __pyx_r = 0;
  __Pyx_RefNannyFinishContext();
  return __pyx_r;
}

/* "(tree fragment)":1
 * def __reduce_cython__(self):             # <<<<<<<<<<<<<<
 *     raise TypeError("self.c_credentials cannot be converted to a Python object for pickling")
 * def __setstate_cython__(self, __pyx_state):
 */

/* Python wrapper */
static PyObject *__pyx_pw_4grpc_7_cython_6cygrpc_15CallCredentials_1__reduce_cython__(PyObject *__pyx_v_self, CYTHON_UNUSED PyObject *unused); /*proto*/
static PyObject *__pyx_pw_4grpc_7_cython_6cygrpc_15CallCredentials_1__reduce_cython__(PyObject *__pyx_v_self, CYTHON_UNUSED PyObject *unused) {
  PyObject *__pyx_r = 0;
  __Pyx_RefNannyDeclarations
  __Pyx_RefNannySetupContext("__reduce_cython__ (wrapper)", 0);
  __pyx_r = __pyx_pf_4grpc_7_cython_6cygrpc_15CallCredentials___reduce_cython__(((struct __pyx_obj_4grpc_7_cython_6cygrpc_CallCredentials *)__pyx_v_self));

  /* function exit code */
  __Pyx_RefNannyFinishContext();
  return __pyx_r;
}

static PyObject *__pyx_pf_4grpc_7_cython_6cygrpc_15CallCredentials___reduce_cython__(CYTHON_UNUSED struct __pyx_obj_4grpc_7_cython_6cygrpc_CallCredentials *__pyx_v_self) {
  PyObject *__pyx_r = NULL;
  __Pyx_RefNannyDeclarations
  PyObject *__pyx_t_1 = NULL;
  __Pyx_RefNannySetupContext("__reduce_cython__", 0);

  /* "(tree fragment)":2
 * def __reduce_cython__(self):
 *     raise TypeError("self.c_credentials cannot be converted to a Python object for pickling")             # <<<<<<<<<<<<<<
 * def __setstate_cython__(self, __pyx_state):
 *     raise TypeError("self.c_credentials cannot be converted to a Python object for pickling")
 */
  __pyx_t_1 = __Pyx_PyObject_Call(__pyx_builtin_TypeError, __pyx_tuple__50, NULL); if (unlikely(!__pyx_t_1)) __PYX_ERR(7, 2, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_1);
  __Pyx_Raise(__pyx_t_1, 0, 0, 0);
  __Pyx_DECREF(__pyx_t_1); __pyx_t_1 = 0;
  __PYX_ERR(7, 2, __pyx_L1_error)

  /* "(tree fragment)":1
 * def __reduce_cython__(self):             # <<<<<<<<<<<<<<
 *     raise TypeError("self.c_credentials cannot be converted to a Python object for pickling")
 * def __setstate_cython__(self, __pyx_state):
 */

  /* function exit code */
  __pyx_L1_error:;
  __Pyx_XDECREF(__pyx_t_1);
  __Pyx_AddTraceback("grpc._cython.cygrpc.CallCredentials.__reduce_cython__", __pyx_clineno, __pyx_lineno, __pyx_filename);
  __pyx_r = NULL;
  __Pyx_XGIVEREF(__pyx_r);
  __Pyx_RefNannyFinishContext();
  return __pyx_r;
}

/* "(tree fragment)":3
 * def __reduce_cython__(self):
 *     raise TypeError("self.c_credentials cannot be converted to a Python object for pickling")
 * def __setstate_cython__(self, __pyx_state):             # <<<<<<<<<<<<<<
 *     raise TypeError("self.c_credentials cannot be converted to a Python object for pickling")
 */

/* Python wrapper */
static PyObject *__pyx_pw_4grpc_7_cython_6cygrpc_15CallCredentials_3__setstate_cython__(PyObject *__pyx_v_self, PyObject *__pyx_v___pyx_state); /*proto*/
static PyObject *__pyx_pw_4grpc_7_cython_6cygrpc_15CallCredentials_3__setstate_cython__(PyObject *__pyx_v_self, PyObject *__pyx_v___pyx_state) {
  PyObject *__pyx_r = 0;
  __Pyx_RefNannyDeclarations
  __Pyx_RefNannySetupContext("__setstate_cython__ (wrapper)", 0);
  __pyx_r = __pyx_pf_4grpc_7_cython_6cygrpc_15CallCredentials_2__setstate_cython__(((struct __pyx_obj_4grpc_7_cython_6cygrpc_CallCredentials *)__pyx_v_self), ((PyObject *)__pyx_v___pyx_state));

  /* function exit code */
  __Pyx_RefNannyFinishContext();
  return __pyx_r;
}

static PyObject *__pyx_pf_4grpc_7_cython_6cygrpc_15CallCredentials_2__setstate_cython__(CYTHON_UNUSED struct __pyx_obj_4grpc_7_cython_6cygrpc_CallCredentials *__pyx_v_self, CYTHON_UNUSED PyObject *__pyx_v___pyx_state) {
  PyObject *__pyx_r = NULL;
  __Pyx_RefNannyDeclarations
  PyObject *__pyx_t_1 = NULL;
  __Pyx_RefNannySetupContext("__setstate_cython__", 0);

  /* "(tree fragment)":4
 *     raise TypeError("self.c_credentials cannot be converted to a Python object for pickling")
 * def __setstate_cython__(self, __pyx_state):
 *     raise TypeError("self.c_credentials cannot be converted to a Python object for pickling")             # <<<<<<<<<<<<<<
 */
  __pyx_t_1 = __Pyx_PyObject_Call(__pyx_builtin_TypeError, __pyx_tuple__51, NULL); if (unlikely(!__pyx_t_1)) __PYX_ERR(7, 4, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_1);
  __Pyx_Raise(__pyx_t_1, 0, 0, 0);
  __Pyx_DECREF(__pyx_t_1); __pyx_t_1 = 0;
  __PYX_ERR(7, 4, __pyx_L1_error)

  /* "(tree fragment)":3
 * def __reduce_cython__(self):
 *     raise TypeError("self.c_credentials cannot be converted to a Python object for pickling")
 * def __setstate_cython__(self, __pyx_state):             # <<<<<<<<<<<<<<
 *     raise TypeError("self.c_credentials cannot be converted to a Python object for pickling")
 */

  /* function exit code */
  __pyx_L1_error:;
  __Pyx_XDECREF(__pyx_t_1);
  __Pyx_AddTraceback("grpc._cython.cygrpc.CallCredentials.__setstate_cython__", __pyx_clineno, __pyx_lineno, __pyx_filename);
  __pyx_r = NULL;
  __Pyx_XGIVEREF(__pyx_r);
  __Pyx_RefNannyFinishContext();
  return __pyx_r;
}

/* "src/python/grpcio/grpc/_cython/_cygrpc/credentials.pyx.pxi":48
 *   cdef size_t metadata_count
 *   cdef grpc_metadata *c_metadata
 *   def callback(metadata, grpc_status_code status, bytes error_details):             # <<<<<<<<<<<<<<
 *     if status == StatusCode.ok:
 *       _store_c_metadata(metadata, &c_metadata, &metadata_count)
 */

/* Python wrapper */
static PyObject *__pyx_pw_4grpc_7_cython_6cygrpc_13_get_metadata_1callback(PyObject *__pyx_self, PyObject *__pyx_args, PyObject *__pyx_kwds); /*proto*/
static PyMethodDef __pyx_mdef_4grpc_7_cython_6cygrpc_13_get_metadata_1callback = {"callback", (PyCFunction)__pyx_pw_4grpc_7_cython_6cygrpc_13_get_metadata_1callback, METH_VARARGS|METH_KEYWORDS, 0};
static PyObject *__pyx_pw_4grpc_7_cython_6cygrpc_13_get_metadata_1callback(PyObject *__pyx_self, PyObject *__pyx_args, PyObject *__pyx_kwds) {
  PyObject *__pyx_v_metadata = 0;
  grpc_status_code __pyx_v_status;
  PyObject *__pyx_v_error_details = 0;
  PyObject *__pyx_r = 0;
  __Pyx_RefNannyDeclarations
  __Pyx_RefNannySetupContext("callback (wrapper)", 0);
  {
    static PyObject **__pyx_pyargnames[] = {&__pyx_n_s_metadata,&__pyx_n_s_status,&__pyx_n_s_error_details,0};
    PyObject* values[3] = {0,0,0};
    if (unlikely(__pyx_kwds)) {
      Py_ssize_t kw_args;
      const Py_ssize_t pos_args = PyTuple_GET_SIZE(__pyx_args);
      switch (pos_args) {
        case  3: values[2] = PyTuple_GET_ITEM(__pyx_args, 2);
        CYTHON_FALLTHROUGH;
        case  2: values[1] = PyTuple_GET_ITEM(__pyx_args, 1);
        CYTHON_FALLTHROUGH;
        case  1: values[0] = PyTuple_GET_ITEM(__pyx_args, 0);
        CYTHON_FALLTHROUGH;
        case  0: break;
        default: goto __pyx_L5_argtuple_error;
      }
      kw_args = PyDict_Size(__pyx_kwds);
      switch (pos_args) {
        case  0:
        if (likely((values[0] = __Pyx_PyDict_GetItemStr(__pyx_kwds, __pyx_n_s_metadata)) != 0)) kw_args--;
        else goto __pyx_L5_argtuple_error;
        CYTHON_FALLTHROUGH;
        case  1:
        if (likely((values[1] = __Pyx_PyDict_GetItemStr(__pyx_kwds, __pyx_n_s_status)) != 0)) kw_args--;
        else {
          __Pyx_RaiseArgtupleInvalid("callback", 1, 3, 3, 1); __PYX_ERR(4, 48, __pyx_L3_error)
        }
        CYTHON_FALLTHROUGH;
        case  2:
        if (likely((values[2] = __Pyx_PyDict_GetItemStr(__pyx_kwds, __pyx_n_s_error_details)) != 0)) kw_args--;
        else {
          __Pyx_RaiseArgtupleInvalid("callback", 1, 3, 3, 2); __PYX_ERR(4, 48, __pyx_L3_error)
        }
      }
      if (unlikely(kw_args > 0)) {
        if (unlikely(__Pyx_ParseOptionalKeywords(__pyx_kwds, __pyx_pyargnames, 0, values, pos_args, "callback") < 0)) __PYX_ERR(4, 48, __pyx_L3_error)
      }
    } else if (PyTuple_GET_SIZE(__pyx_args) != 3) {
      goto __pyx_L5_argtuple_error;
    } else {
      values[0] = PyTuple_GET_ITEM(__pyx_args, 0);
      values[1] = PyTuple_GET_ITEM(__pyx_args, 1);
      values[2] = PyTuple_GET_ITEM(__pyx_args, 2);
    }
    __pyx_v_metadata = values[0];
    __pyx_v_status = ((grpc_status_code)__Pyx_PyInt_As_grpc_status_code(values[1])); if (unlikely(PyErr_Occurred())) __PYX_ERR(4, 48, __pyx_L3_error)
    __pyx_v_error_details = ((PyObject*)values[2]);
  }
  goto __pyx_L4_argument_unpacking_done;
  __pyx_L5_argtuple_error:;
  __Pyx_RaiseArgtupleInvalid("callback", 1, 3, 3, PyTuple_GET_SIZE(__pyx_args)); __PYX_ERR(4, 48, __pyx_L3_error)
  __pyx_L3_error:;
  __Pyx_AddTraceback("grpc._cython.cygrpc._get_metadata.callback", __pyx_clineno, __pyx_lineno, __pyx_filename);
  __Pyx_RefNannyFinishContext();
  return NULL;
  __pyx_L4_argument_unpacking_done:;
  if (unlikely(!__Pyx_ArgTypeTest(((PyObject *)__pyx_v_error_details), (&PyBytes_Type), 1, "error_details", 1))) __PYX_ERR(4, 48, __pyx_L1_error)
  __pyx_r = __pyx_pf_4grpc_7_cython_6cygrpc_13_get_metadata_callback(__pyx_self, __pyx_v_metadata, __pyx_v_status, __pyx_v_error_details);

  /* function exit code */
  goto __pyx_L0;
  __pyx_L1_error:;
  __pyx_r = NULL;
  __pyx_L0:;
  __Pyx_RefNannyFinishContext();
  return __pyx_r;
}

static PyObject *__pyx_pf_4grpc_7_cython_6cygrpc_13_get_metadata_callback(PyObject *__pyx_self, PyObject *__pyx_v_metadata, grpc_status_code __pyx_v_status, PyObject *__pyx_v_error_details) {
  struct __pyx_obj_4grpc_7_cython_6cygrpc___pyx_scope_struct_4___pyx_f_4grpc_7_cython_6cygrpc__get_metadata *__pyx_cur_scope;
  struct __pyx_obj_4grpc_7_cython_6cygrpc___pyx_scope_struct_4___pyx_f_4grpc_7_cython_6cygrpc__get_metadata *__pyx_outer_scope;
  PyObject *__pyx_r = NULL;
  __Pyx_RefNannyDeclarations
  PyObject *__pyx_t_1 = NULL;
  PyObject *__pyx_t_2 = NULL;
  PyObject *__pyx_t_3 = NULL;
  int __pyx_t_4;
  char const *__pyx_t_5;
  __Pyx_RefNannySetupContext("callback", 0);
  __pyx_outer_scope = (struct __pyx_obj_4grpc_7_cython_6cygrpc___pyx_scope_struct_4___pyx_f_4grpc_7_cython_6cygrpc__get_metadata *) __Pyx_CyFunction_GetClosure(__pyx_self);
  __pyx_cur_scope = __pyx_outer_scope;

  /* "src/python/grpcio/grpc/_cython/_cygrpc/credentials.pyx.pxi":49
 *   cdef grpc_metadata *c_metadata
 *   def callback(metadata, grpc_status_code status, bytes error_details):
 *     if status == StatusCode.ok:             # <<<<<<<<<<<<<<
 *       _store_c_metadata(metadata, &c_metadata, &metadata_count)
 *       cb(user_data, c_metadata, metadata_count, status, NULL)
 */
  __pyx_t_1 = __Pyx_PyInt_From_grpc_status_code(__pyx_v_status); if (unlikely(!__pyx_t_1)) __PYX_ERR(4, 49, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_1);
  __pyx_t_2 = __Pyx_GetModuleGlobalName(__pyx_n_s_StatusCode); if (unlikely(!__pyx_t_2)) __PYX_ERR(4, 49, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_2);
  __pyx_t_3 = __Pyx_PyObject_GetAttrStr(__pyx_t_2, __pyx_n_s_ok); if (unlikely(!__pyx_t_3)) __PYX_ERR(4, 49, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_3);
  __Pyx_DECREF(__pyx_t_2); __pyx_t_2 = 0;
  __pyx_t_2 = PyObject_RichCompare(__pyx_t_1, __pyx_t_3, Py_EQ); __Pyx_XGOTREF(__pyx_t_2); if (unlikely(!__pyx_t_2)) __PYX_ERR(4, 49, __pyx_L1_error)
  __Pyx_DECREF(__pyx_t_1); __pyx_t_1 = 0;
  __Pyx_DECREF(__pyx_t_3); __pyx_t_3 = 0;
  __pyx_t_4 = __Pyx_PyObject_IsTrue(__pyx_t_2); if (unlikely(__pyx_t_4 < 0)) __PYX_ERR(4, 49, __pyx_L1_error)
  __Pyx_DECREF(__pyx_t_2); __pyx_t_2 = 0;
  if (__pyx_t_4) {

    /* "src/python/grpcio/grpc/_cython/_cygrpc/credentials.pyx.pxi":50
 *   def callback(metadata, grpc_status_code status, bytes error_details):
 *     if status == StatusCode.ok:
 *       _store_c_metadata(metadata, &c_metadata, &metadata_count)             # <<<<<<<<<<<<<<
 *       cb(user_data, c_metadata, metadata_count, status, NULL)
 *       _release_c_metadata(c_metadata, metadata_count)
 */
    __pyx_f_4grpc_7_cython_6cygrpc__store_c_metadata(__pyx_v_metadata, (&__pyx_cur_scope->__pyx_v_c_metadata), (&__pyx_cur_scope->__pyx_v_metadata_count)); if (unlikely(PyErr_Occurred())) __PYX_ERR(4, 50, __pyx_L1_error)

    /* "src/python/grpcio/grpc/_cython/_cygrpc/credentials.pyx.pxi":51
 *     if status == StatusCode.ok:
 *       _store_c_metadata(metadata, &c_metadata, &metadata_count)
 *       cb(user_data, c_metadata, metadata_count, status, NULL)             # <<<<<<<<<<<<<<
 *       _release_c_metadata(c_metadata, metadata_count)
 *     else:
 */
    __pyx_cur_scope->__pyx_v_cb(__pyx_cur_scope->__pyx_v_user_data, __pyx_cur_scope->__pyx_v_c_metadata, __pyx_cur_scope->__pyx_v_metadata_count, __pyx_v_status, NULL);

    /* "src/python/grpcio/grpc/_cython/_cygrpc/credentials.pyx.pxi":52
 *       _store_c_metadata(metadata, &c_metadata, &metadata_count)
 *       cb(user_data, c_metadata, metadata_count, status, NULL)
 *       _release_c_metadata(c_metadata, metadata_count)             # <<<<<<<<<<<<<<
 *     else:
 *       cb(user_data, NULL, 0, status, error_details)
 */
    __pyx_f_4grpc_7_cython_6cygrpc__release_c_metadata(__pyx_cur_scope->__pyx_v_c_metadata, __pyx_cur_scope->__pyx_v_metadata_count); if (unlikely(PyErr_Occurred())) __PYX_ERR(4, 52, __pyx_L1_error)

    /* "src/python/grpcio/grpc/_cython/_cygrpc/credentials.pyx.pxi":49
 *   cdef grpc_metadata *c_metadata
 *   def callback(metadata, grpc_status_code status, bytes error_details):
 *     if status == StatusCode.ok:             # <<<<<<<<<<<<<<
 *       _store_c_metadata(metadata, &c_metadata, &metadata_count)
 *       cb(user_data, c_metadata, metadata_count, status, NULL)
 */
    goto __pyx_L3;
  }

  /* "src/python/grpcio/grpc/_cython/_cygrpc/credentials.pyx.pxi":54
 *       _release_c_metadata(c_metadata, metadata_count)
 *     else:
 *       cb(user_data, NULL, 0, status, error_details)             # <<<<<<<<<<<<<<
 *   args = context.service_url, context.method_name, callback,
 *   _spawn_callback_async(<object>state, args)
 */
  /*else*/ {
    if (unlikely(__pyx_v_error_details == Py_None)) {
      PyErr_SetString(PyExc_TypeError, "expected bytes, NoneType found");
      __PYX_ERR(4, 54, __pyx_L1_error)
    }
    __pyx_t_5 = __Pyx_PyBytes_AsString(__pyx_v_error_details); if (unlikely((!__pyx_t_5) && PyErr_Occurred())) __PYX_ERR(4, 54, __pyx_L1_error)
    __pyx_cur_scope->__pyx_v_cb(__pyx_cur_scope->__pyx_v_user_data, NULL, 0, __pyx_v_status, __pyx_t_5);
  }
  __pyx_L3:;

  /* "src/python/grpcio/grpc/_cython/_cygrpc/credentials.pyx.pxi":48
 *   cdef size_t metadata_count
 *   cdef grpc_metadata *c_metadata
 *   def callback(metadata, grpc_status_code status, bytes error_details):             # <<<<<<<<<<<<<<
 *     if status == StatusCode.ok:
 *       _store_c_metadata(metadata, &c_metadata, &metadata_count)
 */

  /* function exit code */
  __pyx_r = Py_None; __Pyx_INCREF(Py_None);
  goto __pyx_L0;
  __pyx_L1_error:;
  __Pyx_XDECREF(__pyx_t_1);
  __Pyx_XDECREF(__pyx_t_2);
  __Pyx_XDECREF(__pyx_t_3);
  __Pyx_AddTraceback("grpc._cython.cygrpc._get_metadata.callback", __pyx_clineno, __pyx_lineno, __pyx_filename);
  __pyx_r = NULL;
  __pyx_L0:;
  __Pyx_XGIVEREF(__pyx_r);
  __Pyx_RefNannyFinishContext();
  return __pyx_r;
}

/* "src/python/grpcio/grpc/_cython/_cygrpc/credentials.pyx.pxi":40
 * 
 * 
 * cdef int _get_metadata(             # <<<<<<<<<<<<<<
 *     void *state, grpc_auth_metadata_context context,
 *     grpc_credentials_plugin_metadata_cb cb, void *user_data,
 */

static int __pyx_f_4grpc_7_cython_6cygrpc__get_metadata(void *__pyx_v_state, grpc_auth_metadata_context __pyx_v_context, grpc_credentials_plugin_metadata_cb __pyx_v_cb, void *__pyx_v_user_data, CYTHON_UNUSED grpc_metadata *__pyx_v_creds_md, CYTHON_UNUSED size_t *__pyx_v_num_creds_md, CYTHON_UNUSED grpc_status_code *__pyx_v_status, CYTHON_UNUSED char const **__pyx_v_error_details) {
  struct __pyx_obj_4grpc_7_cython_6cygrpc___pyx_scope_struct_4___pyx_f_4grpc_7_cython_6cygrpc__get_metadata *__pyx_cur_scope;
  PyObject *__pyx_v_callback = 0;
  PyObject *__pyx_v_args = NULL;
  int __pyx_r;
  __Pyx_RefNannyDeclarations
  PyObject *__pyx_t_1 = NULL;
  PyObject *__pyx_t_2 = NULL;
  PyObject *__pyx_t_3 = NULL;
  int __pyx_t_4;
  PyObject *__pyx_t_5 = NULL;
  #ifdef WITH_THREAD
  PyGILState_STATE __pyx_gilstate_save = __Pyx_PyGILState_Ensure();
  #endif
  __Pyx_RefNannySetupContext("_get_metadata", 0);
  __pyx_cur_scope = (struct __pyx_obj_4grpc_7_cython_6cygrpc___pyx_scope_struct_4___pyx_f_4grpc_7_cython_6cygrpc__get_metadata *)__pyx_tp_new_4grpc_7_cython_6cygrpc___pyx_scope_struct_4___pyx_f_4grpc_7_cython_6cygrpc__get_metadata(__pyx_ptype_4grpc_7_cython_6cygrpc___pyx_scope_struct_4___pyx_f_4grpc_7_cython_6cygrpc__get_metadata, __pyx_empty_tuple, NULL);
  if (unlikely(!__pyx_cur_scope)) {
    __pyx_cur_scope = ((struct __pyx_obj_4grpc_7_cython_6cygrpc___pyx_scope_struct_4___pyx_f_4grpc_7_cython_6cygrpc__get_metadata *)Py_None);
    __Pyx_INCREF(Py_None);
    __PYX_ERR(4, 40, __pyx_L1_error)
  } else {
    __Pyx_GOTREF(__pyx_cur_scope);
  }
  __pyx_cur_scope->__pyx_v_cb = __pyx_v_cb;
  __pyx_cur_scope->__pyx_v_user_data = __pyx_v_user_data;

  /* "src/python/grpcio/grpc/_cython/_cygrpc/credentials.pyx.pxi":48
 *   cdef size_t metadata_count
 *   cdef grpc_metadata *c_metadata
 *   def callback(metadata, grpc_status_code status, bytes error_details):             # <<<<<<<<<<<<<<
 *     if status == StatusCode.ok:
 *       _store_c_metadata(metadata, &c_metadata, &metadata_count)
 */
  __pyx_t_1 = __Pyx_CyFunction_NewEx(&__pyx_mdef_4grpc_7_cython_6cygrpc_13_get_metadata_1callback, 0, __pyx_n_s_get_metadata_locals_callback, ((PyObject*)__pyx_cur_scope), __pyx_n_s_grpc__cython_cygrpc, __pyx_d, ((PyObject *)__pyx_codeobj__53)); if (unlikely(!__pyx_t_1)) __PYX_ERR(4, 48, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_1);
  __pyx_v_callback = __pyx_t_1;
  __pyx_t_1 = 0;

  /* "src/python/grpcio/grpc/_cython/_cygrpc/credentials.pyx.pxi":55
 *     else:
 *       cb(user_data, NULL, 0, status, error_details)
 *   args = context.service_url, context.method_name, callback,             # <<<<<<<<<<<<<<
 *   _spawn_callback_async(<object>state, args)
 *   return 0  # Asynchronous return
 */
  __pyx_t_1 = __Pyx_PyBytes_FromString(__pyx_v_context.service_url); if (unlikely(!__pyx_t_1)) __PYX_ERR(4, 55, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_1);
  __pyx_t_2 = __Pyx_PyBytes_FromString(__pyx_v_context.method_name); if (unlikely(!__pyx_t_2)) __PYX_ERR(4, 55, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_2);
  __pyx_t_3 = PyTuple_New(3); if (unlikely(!__pyx_t_3)) __PYX_ERR(4, 55, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_3);
  __Pyx_GIVEREF(__pyx_t_1);
  PyTuple_SET_ITEM(__pyx_t_3, 0, __pyx_t_1);
  __Pyx_GIVEREF(__pyx_t_2);
  PyTuple_SET_ITEM(__pyx_t_3, 1, __pyx_t_2);
  __Pyx_INCREF(__pyx_v_callback);
  __Pyx_GIVEREF(__pyx_v_callback);
  PyTuple_SET_ITEM(__pyx_t_3, 2, __pyx_v_callback);
  __pyx_t_1 = 0;
  __pyx_t_2 = 0;
  __pyx_v_args = ((PyObject*)__pyx_t_3);
  __pyx_t_3 = 0;

  /* "src/python/grpcio/grpc/_cython/_cygrpc/credentials.pyx.pxi":56
 *       cb(user_data, NULL, 0, status, error_details)
 *   args = context.service_url, context.method_name, callback,
 *   _spawn_callback_async(<object>state, args)             # <<<<<<<<<<<<<<
 *   return 0  # Asynchronous return
 * 
 */
  __pyx_t_2 = __Pyx_GetModuleGlobalName(__pyx_n_s_spawn_callback_async); if (unlikely(!__pyx_t_2)) __PYX_ERR(4, 56, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_2);
  __pyx_t_1 = NULL;
  __pyx_t_4 = 0;
  if (CYTHON_UNPACK_METHODS && unlikely(PyMethod_Check(__pyx_t_2))) {
    __pyx_t_1 = PyMethod_GET_SELF(__pyx_t_2);
    if (likely(__pyx_t_1)) {
      PyObject* function = PyMethod_GET_FUNCTION(__pyx_t_2);
      __Pyx_INCREF(__pyx_t_1);
      __Pyx_INCREF(function);
      __Pyx_DECREF_SET(__pyx_t_2, function);
      __pyx_t_4 = 1;
    }
  }
  #if CYTHON_FAST_PYCALL
  if (PyFunction_Check(__pyx_t_2)) {
    PyObject *__pyx_temp[3] = {__pyx_t_1, ((PyObject *)__pyx_v_state), __pyx_v_args};
    __pyx_t_3 = __Pyx_PyFunction_FastCall(__pyx_t_2, __pyx_temp+1-__pyx_t_4, 2+__pyx_t_4); if (unlikely(!__pyx_t_3)) __PYX_ERR(4, 56, __pyx_L1_error)
    __Pyx_XDECREF(__pyx_t_1); __pyx_t_1 = 0;
    __Pyx_GOTREF(__pyx_t_3);
  } else
  #endif
  #if CYTHON_FAST_PYCCALL
  if (__Pyx_PyFastCFunction_Check(__pyx_t_2)) {
    PyObject *__pyx_temp[3] = {__pyx_t_1, ((PyObject *)__pyx_v_state), __pyx_v_args};
    __pyx_t_3 = __Pyx_PyCFunction_FastCall(__pyx_t_2, __pyx_temp+1-__pyx_t_4, 2+__pyx_t_4); if (unlikely(!__pyx_t_3)) __PYX_ERR(4, 56, __pyx_L1_error)
    __Pyx_XDECREF(__pyx_t_1); __pyx_t_1 = 0;
    __Pyx_GOTREF(__pyx_t_3);
  } else
  #endif
  {
    __pyx_t_5 = PyTuple_New(2+__pyx_t_4); if (unlikely(!__pyx_t_5)) __PYX_ERR(4, 56, __pyx_L1_error)
    __Pyx_GOTREF(__pyx_t_5);
    if (__pyx_t_1) {
      __Pyx_GIVEREF(__pyx_t_1); PyTuple_SET_ITEM(__pyx_t_5, 0, __pyx_t_1); __pyx_t_1 = NULL;
    }
    __Pyx_INCREF(((PyObject *)__pyx_v_state));
    __Pyx_GIVEREF(((PyObject *)__pyx_v_state));
    PyTuple_SET_ITEM(__pyx_t_5, 0+__pyx_t_4, ((PyObject *)__pyx_v_state));
    __Pyx_INCREF(__pyx_v_args);
    __Pyx_GIVEREF(__pyx_v_args);
    PyTuple_SET_ITEM(__pyx_t_5, 1+__pyx_t_4, __pyx_v_args);
    __pyx_t_3 = __Pyx_PyObject_Call(__pyx_t_2, __pyx_t_5, NULL); if (unlikely(!__pyx_t_3)) __PYX_ERR(4, 56, __pyx_L1_error)
    __Pyx_GOTREF(__pyx_t_3);
    __Pyx_DECREF(__pyx_t_5); __pyx_t_5 = 0;
  }
  __Pyx_DECREF(__pyx_t_2); __pyx_t_2 = 0;
  __Pyx_DECREF(__pyx_t_3); __pyx_t_3 = 0;

  /* "src/python/grpcio/grpc/_cython/_cygrpc/credentials.pyx.pxi":57
 *   args = context.service_url, context.method_name, callback,
 *   _spawn_callback_async(<object>state, args)
 *   return 0  # Asynchronous return             # <<<<<<<<<<<<<<
 * 
 * 
 */
  __pyx_r = 0;
  goto __pyx_L0;

  /* "src/python/grpcio/grpc/_cython/_cygrpc/credentials.pyx.pxi":40
 * 
 * 
 * cdef int _get_metadata(             # <<<<<<<<<<<<<<
 *     void *state, grpc_auth_metadata_context context,
 *     grpc_credentials_plugin_metadata_cb cb, void *user_data,
 */

  /* function exit code */
  __pyx_L1_error:;
  __Pyx_XDECREF(__pyx_t_1);
  __Pyx_XDECREF(__pyx_t_2);
  __Pyx_XDECREF(__pyx_t_3);
  __Pyx_XDECREF(__pyx_t_5);
  __Pyx_AddTraceback("grpc._cython.cygrpc._get_metadata", __pyx_clineno, __pyx_lineno, __pyx_filename);
  __pyx_r = -1;
  __pyx_L0:;
  __Pyx_XDECREF(__pyx_v_callback);
  __Pyx_XDECREF(__pyx_v_args);
  __Pyx_DECREF(((PyObject *)__pyx_cur_scope));
  __Pyx_RefNannyFinishContext();
  #ifdef WITH_THREAD
  __Pyx_PyGILState_Release(__pyx_gilstate_save);
  #endif
  return __pyx_r;
}

/* "src/python/grpcio/grpc/_cython/_cygrpc/credentials.pyx.pxi":60
 * 
 * 
 * cdef void _destroy(void *state) except * with gil:             # <<<<<<<<<<<<<<
 *   cpython.Py_DECREF(<object>state)
 *   grpc_shutdown_blocking()
 */

static void __pyx_f_4grpc_7_cython_6cygrpc__destroy(void *__pyx_v_state) {
  __Pyx_RefNannyDeclarations
  #ifdef WITH_THREAD
  PyGILState_STATE __pyx_gilstate_save = __Pyx_PyGILState_Ensure();
  #endif
  __Pyx_RefNannySetupContext("_destroy", 0);

  /* "src/python/grpcio/grpc/_cython/_cygrpc/credentials.pyx.pxi":61
 * 
 * cdef void _destroy(void *state) except * with gil:
 *   cpython.Py_DECREF(<object>state)             # <<<<<<<<<<<<<<
 *   grpc_shutdown_blocking()
 * 
 */
  Py_DECREF(((PyObject *)__pyx_v_state));

  /* "src/python/grpcio/grpc/_cython/_cygrpc/credentials.pyx.pxi":62
 * cdef void _destroy(void *state) except * with gil:
 *   cpython.Py_DECREF(<object>state)
 *   grpc_shutdown_blocking()             # <<<<<<<<<<<<<<
 * 
 * 
 */
  grpc_shutdown_blocking();

  /* "src/python/grpcio/grpc/_cython/_cygrpc/credentials.pyx.pxi":60
 * 
 * 
 * cdef void _destroy(void *state) except * with gil:             # <<<<<<<<<<<<<<
 *   cpython.Py_DECREF(<object>state)
 *   grpc_shutdown_blocking()
 */

  /* function exit code */
  __Pyx_RefNannyFinishContext();
  #ifdef WITH_THREAD
  __Pyx_PyGILState_Release(__pyx_gilstate_save);
  #endif
}

/* "src/python/grpcio/grpc/_cython/_cygrpc/credentials.pyx.pxi":67
 * cdef class MetadataPluginCallCredentials(CallCredentials):
 * 
 *   def __cinit__(self, metadata_plugin, name):             # <<<<<<<<<<<<<<
 *     self._metadata_plugin = metadata_plugin
 *     self._name = name
 */

/* Python wrapper */
static int __pyx_pw_4grpc_7_cython_6cygrpc_29MetadataPluginCallCredentials_1__cinit__(PyObject *__pyx_v_self, PyObject *__pyx_args, PyObject *__pyx_kwds); /*proto*/
static int __pyx_pw_4grpc_7_cython_6cygrpc_29MetadataPluginCallCredentials_1__cinit__(PyObject *__pyx_v_self, PyObject *__pyx_args, PyObject *__pyx_kwds) {
  PyObject *__pyx_v_metadata_plugin = 0;
  PyObject *__pyx_v_name = 0;
  int __pyx_r;
  __Pyx_RefNannyDeclarations
  __Pyx_RefNannySetupContext("__cinit__ (wrapper)", 0);
  {
    static PyObject **__pyx_pyargnames[] = {&__pyx_n_s_metadata_plugin,&__pyx_n_s_name,0};
    PyObject* values[2] = {0,0};
    if (unlikely(__pyx_kwds)) {
      Py_ssize_t kw_args;
      const Py_ssize_t pos_args = PyTuple_GET_SIZE(__pyx_args);
      switch (pos_args) {
        case  2: values[1] = PyTuple_GET_ITEM(__pyx_args, 1);
        CYTHON_FALLTHROUGH;
        case  1: values[0] = PyTuple_GET_ITEM(__pyx_args, 0);
        CYTHON_FALLTHROUGH;
        case  0: break;
        default: goto __pyx_L5_argtuple_error;
      }
      kw_args = PyDict_Size(__pyx_kwds);
      switch (pos_args) {
        case  0:
        if (likely((values[0] = __Pyx_PyDict_GetItemStr(__pyx_kwds, __pyx_n_s_metadata_plugin)) != 0)) kw_args--;
        else goto __pyx_L5_argtuple_error;
        CYTHON_FALLTHROUGH;
        case  1:
        if (likely((values[1] = __Pyx_PyDict_GetItemStr(__pyx_kwds, __pyx_n_s_name)) != 0)) kw_args--;
        else {
          __Pyx_RaiseArgtupleInvalid("__cinit__", 1, 2, 2, 1); __PYX_ERR(4, 67, __pyx_L3_error)
        }
      }
      if (unlikely(kw_args > 0)) {
        if (unlikely(__Pyx_ParseOptionalKeywords(__pyx_kwds, __pyx_pyargnames, 0, values, pos_args, "__cinit__") < 0)) __PYX_ERR(4, 67, __pyx_L3_error)
      }
    } else if (PyTuple_GET_SIZE(__pyx_args) != 2) {
      goto __pyx_L5_argtuple_error;
    } else {
      values[0] = PyTuple_GET_ITEM(__pyx_args, 0);
      values[1] = PyTuple_GET_ITEM(__pyx_args, 1);
    }
    __pyx_v_metadata_plugin = values[0];
    __pyx_v_name = values[1];
  }
  goto __pyx_L4_argument_unpacking_done;
  __pyx_L5_argtuple_error:;
  __Pyx_RaiseArgtupleInvalid("__cinit__", 1, 2, 2, PyTuple_GET_SIZE(__pyx_args)); __PYX_ERR(4, 67, __pyx_L3_error)
  __pyx_L3_error:;
  __Pyx_AddTraceback("grpc._cython.cygrpc.MetadataPluginCallCredentials.__cinit__", __pyx_clineno, __pyx_lineno, __pyx_filename);
  __Pyx_RefNannyFinishContext();
  return -1;
  __pyx_L4_argument_unpacking_done:;
  __pyx_r = __pyx_pf_4grpc_7_cython_6cygrpc_29MetadataPluginCallCredentials___cinit__(((struct __pyx_obj_4grpc_7_cython_6cygrpc_MetadataPluginCallCredentials *)__pyx_v_self), __pyx_v_metadata_plugin, __pyx_v_name);

  /* function exit code */
  __Pyx_RefNannyFinishContext();
  return __pyx_r;
}

static int __pyx_pf_4grpc_7_cython_6cygrpc_29MetadataPluginCallCredentials___cinit__(struct __pyx_obj_4grpc_7_cython_6cygrpc_MetadataPluginCallCredentials *__pyx_v_self, PyObject *__pyx_v_metadata_plugin, PyObject *__pyx_v_name) {
  int __pyx_r;
  __Pyx_RefNannyDeclarations
  PyObject *__pyx_t_1 = NULL;
  __Pyx_RefNannySetupContext("__cinit__", 0);

  /* "src/python/grpcio/grpc/_cython/_cygrpc/credentials.pyx.pxi":68
 * 
 *   def __cinit__(self, metadata_plugin, name):
 *     self._metadata_plugin = metadata_plugin             # <<<<<<<<<<<<<<
 *     self._name = name
 * 
 */
  __Pyx_INCREF(__pyx_v_metadata_plugin);
  __Pyx_GIVEREF(__pyx_v_metadata_plugin);
  __Pyx_GOTREF(__pyx_v_self->_metadata_plugin);
  __Pyx_DECREF(__pyx_v_self->_metadata_plugin);
  __pyx_v_self->_metadata_plugin = __pyx_v_metadata_plugin;

  /* "src/python/grpcio/grpc/_cython/_cygrpc/credentials.pyx.pxi":69
 *   def __cinit__(self, metadata_plugin, name):
 *     self._metadata_plugin = metadata_plugin
 *     self._name = name             # <<<<<<<<<<<<<<
 * 
 *   cdef grpc_call_credentials *c(self) except *:
 */
  if (!(likely(PyBytes_CheckExact(__pyx_v_name))||((__pyx_v_name) == Py_None)||(PyErr_Format(PyExc_TypeError, "Expected %.16s, got %.200s", "bytes", Py_TYPE(__pyx_v_name)->tp_name), 0))) __PYX_ERR(4, 69, __pyx_L1_error)
  __pyx_t_1 = __pyx_v_name;
  __Pyx_INCREF(__pyx_t_1);
  __Pyx_GIVEREF(__pyx_t_1);
  __Pyx_GOTREF(__pyx_v_self->_name);
  __Pyx_DECREF(__pyx_v_self->_name);
  __pyx_v_self->_name = ((PyObject*)__pyx_t_1);
  __pyx_t_1 = 0;

  /* "src/python/grpcio/grpc/_cython/_cygrpc/credentials.pyx.pxi":67
 * cdef class MetadataPluginCallCredentials(CallCredentials):
 * 
 *   def __cinit__(self, metadata_plugin, name):             # <<<<<<<<<<<<<<
 *     self._metadata_plugin = metadata_plugin
 *     self._name = name
 */

  /* function exit code */
  __pyx_r = 0;
  goto __pyx_L0;
  __pyx_L1_error:;
  __Pyx_XDECREF(__pyx_t_1);
  __Pyx_AddTraceback("grpc._cython.cygrpc.MetadataPluginCallCredentials.__cinit__", __pyx_clineno, __pyx_lineno, __pyx_filename);
  __pyx_r = -1;
  __pyx_L0:;
  __Pyx_RefNannyFinishContext();
  return __pyx_r;
}

/* "src/python/grpcio/grpc/_cython/_cygrpc/credentials.pyx.pxi":71
 *     self._name = name
 * 
 *   cdef grpc_call_credentials *c(self) except *:             # <<<<<<<<<<<<<<
 *     cdef grpc_metadata_credentials_plugin c_metadata_plugin
 *     c_metadata_plugin.get_metadata = _get_metadata
 */

static grpc_call_credentials *__pyx_f_4grpc_7_cython_6cygrpc_29MetadataPluginCallCredentials_c(struct __pyx_obj_4grpc_7_cython_6cygrpc_MetadataPluginCallCredentials *__pyx_v_self) {
  grpc_metadata_credentials_plugin __pyx_v_c_metadata_plugin;
  grpc_call_credentials *__pyx_r;
  __Pyx_RefNannyDeclarations
  char const *__pyx_t_1;
  PyObject *__pyx_t_2 = NULL;
  PyObject *__pyx_t_3 = NULL;
  PyObject *__pyx_t_4 = NULL;
  __Pyx_RefNannySetupContext("c", 0);

  /* "src/python/grpcio/grpc/_cython/_cygrpc/credentials.pyx.pxi":73
 *   cdef grpc_call_credentials *c(self) except *:
 *     cdef grpc_metadata_credentials_plugin c_metadata_plugin
 *     c_metadata_plugin.get_metadata = _get_metadata             # <<<<<<<<<<<<<<
 *     c_metadata_plugin.destroy = _destroy
 *     c_metadata_plugin.state = <void *>self._metadata_plugin
 */
  __pyx_v_c_metadata_plugin.get_metadata = __pyx_f_4grpc_7_cython_6cygrpc__get_metadata;

  /* "src/python/grpcio/grpc/_cython/_cygrpc/credentials.pyx.pxi":74
 *     cdef grpc_metadata_credentials_plugin c_metadata_plugin
 *     c_metadata_plugin.get_metadata = _get_metadata
 *     c_metadata_plugin.destroy = _destroy             # <<<<<<<<<<<<<<
 *     c_metadata_plugin.state = <void *>self._metadata_plugin
 *     c_metadata_plugin.type = self._name
 */
  __pyx_v_c_metadata_plugin.destroy = __pyx_f_4grpc_7_cython_6cygrpc__destroy;

  /* "src/python/grpcio/grpc/_cython/_cygrpc/credentials.pyx.pxi":75
 *     c_metadata_plugin.get_metadata = _get_metadata
 *     c_metadata_plugin.destroy = _destroy
 *     c_metadata_plugin.state = <void *>self._metadata_plugin             # <<<<<<<<<<<<<<
 *     c_metadata_plugin.type = self._name
 *     cpython.Py_INCREF(self._metadata_plugin)
 */
  __pyx_v_c_metadata_plugin.state = ((void *)__pyx_v_self->_metadata_plugin);

  /* "src/python/grpcio/grpc/_cython/_cygrpc/credentials.pyx.pxi":76
 *     c_metadata_plugin.destroy = _destroy
 *     c_metadata_plugin.state = <void *>self._metadata_plugin
 *     c_metadata_plugin.type = self._name             # <<<<<<<<<<<<<<
 *     cpython.Py_INCREF(self._metadata_plugin)
 *     fork_handlers_and_grpc_init()
 */
  if (unlikely(__pyx_v_self->_name == Py_None)) {
    PyErr_SetString(PyExc_TypeError, "expected bytes, NoneType found");
    __PYX_ERR(4, 76, __pyx_L1_error)
  }
  __pyx_t_1 = __Pyx_PyBytes_AsString(__pyx_v_self->_name); if (unlikely((!__pyx_t_1) && PyErr_Occurred())) __PYX_ERR(4, 76, __pyx_L1_error)
  __pyx_v_c_metadata_plugin.type = __pyx_t_1;

  /* "src/python/grpcio/grpc/_cython/_cygrpc/credentials.pyx.pxi":77
 *     c_metadata_plugin.state = <void *>self._metadata_plugin
 *     c_metadata_plugin.type = self._name
 *     cpython.Py_INCREF(self._metadata_plugin)             # <<<<<<<<<<<<<<
 *     fork_handlers_and_grpc_init()
 *     return grpc_metadata_credentials_create_from_plugin(c_metadata_plugin, NULL)
 */
  __pyx_t_2 = __pyx_v_self->_metadata_plugin;
  __Pyx_INCREF(__pyx_t_2);
  Py_INCREF(__pyx_t_2);
  __Pyx_DECREF(__pyx_t_2); __pyx_t_2 = 0;

  /* "src/python/grpcio/grpc/_cython/_cygrpc/credentials.pyx.pxi":78
 *     c_metadata_plugin.type = self._name
 *     cpython.Py_INCREF(self._metadata_plugin)
 *     fork_handlers_and_grpc_init()             # <<<<<<<<<<<<<<
 *     return grpc_metadata_credentials_create_from_plugin(c_metadata_plugin, NULL)
 * 
 */
  __pyx_t_3 = __Pyx_GetModuleGlobalName(__pyx_n_s_fork_handlers_and_grpc_init); if (unlikely(!__pyx_t_3)) __PYX_ERR(4, 78, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_3);
  __pyx_t_4 = NULL;
  if (CYTHON_UNPACK_METHODS && unlikely(PyMethod_Check(__pyx_t_3))) {
    __pyx_t_4 = PyMethod_GET_SELF(__pyx_t_3);
    if (likely(__pyx_t_4)) {
      PyObject* function = PyMethod_GET_FUNCTION(__pyx_t_3);
      __Pyx_INCREF(__pyx_t_4);
      __Pyx_INCREF(function);
      __Pyx_DECREF_SET(__pyx_t_3, function);
    }
  }
  if (__pyx_t_4) {
    __pyx_t_2 = __Pyx_PyObject_CallOneArg(__pyx_t_3, __pyx_t_4); if (unlikely(!__pyx_t_2)) __PYX_ERR(4, 78, __pyx_L1_error)
    __Pyx_DECREF(__pyx_t_4); __pyx_t_4 = 0;
  } else {
    __pyx_t_2 = __Pyx_PyObject_CallNoArg(__pyx_t_3); if (unlikely(!__pyx_t_2)) __PYX_ERR(4, 78, __pyx_L1_error)
  }
  __Pyx_GOTREF(__pyx_t_2);
  __Pyx_DECREF(__pyx_t_3); __pyx_t_3 = 0;
  __Pyx_DECREF(__pyx_t_2); __pyx_t_2 = 0;

  /* "src/python/grpcio/grpc/_cython/_cygrpc/credentials.pyx.pxi":79
 *     cpython.Py_INCREF(self._metadata_plugin)
 *     fork_handlers_and_grpc_init()
 *     return grpc_metadata_credentials_create_from_plugin(c_metadata_plugin, NULL)             # <<<<<<<<<<<<<<
 * 
 * 
 */
  __pyx_r = grpc_metadata_credentials_create_from_plugin(__pyx_v_c_metadata_plugin, NULL);
  goto __pyx_L0;

  /* "src/python/grpcio/grpc/_cython/_cygrpc/credentials.pyx.pxi":71
 *     self._name = name
 * 
 *   cdef grpc_call_credentials *c(self) except *:             # <<<<<<<<<<<<<<
 *     cdef grpc_metadata_credentials_plugin c_metadata_plugin
 *     c_metadata_plugin.get_metadata = _get_metadata
 */

  /* function exit code */
  __pyx_L1_error:;
  __Pyx_XDECREF(__pyx_t_2);
  __Pyx_XDECREF(__pyx_t_3);
  __Pyx_XDECREF(__pyx_t_4);
  __Pyx_AddTraceback("grpc._cython.cygrpc.MetadataPluginCallCredentials.c", __pyx_clineno, __pyx_lineno, __pyx_filename);
  __pyx_r = 0;
  __pyx_L0:;
  __Pyx_RefNannyFinishContext();
  return __pyx_r;
}

/* "src/python/grpcio/grpc/_cython/_cygrpc/credentials.pxd.pxi":36
 * cdef class MetadataPluginCallCredentials(CallCredentials):
 * 
 *   cdef readonly object _metadata_plugin             # <<<<<<<<<<<<<<
 *   cdef readonly bytes _name
 * 
 */

/* Python wrapper */
static PyObject *__pyx_pw_4grpc_7_cython_6cygrpc_29MetadataPluginCallCredentials_16_metadata_plugin_1__get__(PyObject *__pyx_v_self); /*proto*/
static PyObject *__pyx_pw_4grpc_7_cython_6cygrpc_29MetadataPluginCallCredentials_16_metadata_plugin_1__get__(PyObject *__pyx_v_self) {
  PyObject *__pyx_r = 0;
  __Pyx_RefNannyDeclarations
  __Pyx_RefNannySetupContext("__get__ (wrapper)", 0);
  __pyx_r = __pyx_pf_4grpc_7_cython_6cygrpc_29MetadataPluginCallCredentials_16_metadata_plugin___get__(((struct __pyx_obj_4grpc_7_cython_6cygrpc_MetadataPluginCallCredentials *)__pyx_v_self));

  /* function exit code */
  __Pyx_RefNannyFinishContext();
  return __pyx_r;
}

static PyObject *__pyx_pf_4grpc_7_cython_6cygrpc_29MetadataPluginCallCredentials_16_metadata_plugin___get__(struct __pyx_obj_4grpc_7_cython_6cygrpc_MetadataPluginCallCredentials *__pyx_v_self) {
  PyObject *__pyx_r = NULL;
  __Pyx_RefNannyDeclarations
  __Pyx_RefNannySetupContext("__get__", 0);
  __Pyx_XDECREF(__pyx_r);
  __Pyx_INCREF(__pyx_v_self->_metadata_plugin);
  __pyx_r = __pyx_v_self->_metadata_plugin;
  goto __pyx_L0;

  /* function exit code */
  __pyx_L0:;
  __Pyx_XGIVEREF(__pyx_r);
  __Pyx_RefNannyFinishContext();
  return __pyx_r;
}

/* "src/python/grpcio/grpc/_cython/_cygrpc/credentials.pxd.pxi":37
 * 
 *   cdef readonly object _metadata_plugin
 *   cdef readonly bytes _name             # <<<<<<<<<<<<<<
 * 
 *   cdef grpc_call_credentials *c(self) except *
 */

/* Python wrapper */
static PyObject *__pyx_pw_4grpc_7_cython_6cygrpc_29MetadataPluginCallCredentials_5_name_1__get__(PyObject *__pyx_v_self); /*proto*/
static PyObject *__pyx_pw_4grpc_7_cython_6cygrpc_29MetadataPluginCallCredentials_5_name_1__get__(PyObject *__pyx_v_self) {
  PyObject *__pyx_r = 0;
  __Pyx_RefNannyDeclarations
  __Pyx_RefNannySetupContext("__get__ (wrapper)", 0);
  __pyx_r = __pyx_pf_4grpc_7_cython_6cygrpc_29MetadataPluginCallCredentials_5_name___get__(((struct __pyx_obj_4grpc_7_cython_6cygrpc_MetadataPluginCallCredentials *)__pyx_v_self));

  /* function exit code */
  __Pyx_RefNannyFinishContext();
  return __pyx_r;
}

static PyObject *__pyx_pf_4grpc_7_cython_6cygrpc_29MetadataPluginCallCredentials_5_name___get__(struct __pyx_obj_4grpc_7_cython_6cygrpc_MetadataPluginCallCredentials *__pyx_v_self) {
  PyObject *__pyx_r = NULL;
  __Pyx_RefNannyDeclarations
  __Pyx_RefNannySetupContext("__get__", 0);
  __Pyx_XDECREF(__pyx_r);
  __Pyx_INCREF(__pyx_v_self->_name);
  __pyx_r = __pyx_v_self->_name;
  goto __pyx_L0;

  /* function exit code */
  __pyx_L0:;
  __Pyx_XGIVEREF(__pyx_r);
  __Pyx_RefNannyFinishContext();
  return __pyx_r;
}

/* "(tree fragment)":1
 * def __reduce_cython__(self):             # <<<<<<<<<<<<<<
 *     raise TypeError("no default __reduce__ due to non-trivial __cinit__")
 * def __setstate_cython__(self, __pyx_state):
 */

/* Python wrapper */
static PyObject *__pyx_pw_4grpc_7_cython_6cygrpc_29MetadataPluginCallCredentials_3__reduce_cython__(PyObject *__pyx_v_self, CYTHON_UNUSED PyObject *unused); /*proto*/
static PyObject *__pyx_pw_4grpc_7_cython_6cygrpc_29MetadataPluginCallCredentials_3__reduce_cython__(PyObject *__pyx_v_self, CYTHON_UNUSED PyObject *unused) {
  PyObject *__pyx_r = 0;
  __Pyx_RefNannyDeclarations
  __Pyx_RefNannySetupContext("__reduce_cython__ (wrapper)", 0);
  __pyx_r = __pyx_pf_4grpc_7_cython_6cygrpc_29MetadataPluginCallCredentials_2__reduce_cython__(((struct __pyx_obj_4grpc_7_cython_6cygrpc_MetadataPluginCallCredentials *)__pyx_v_self));

  /* function exit code */
  __Pyx_RefNannyFinishContext();
  return __pyx_r;
}

static PyObject *__pyx_pf_4grpc_7_cython_6cygrpc_29MetadataPluginCallCredentials_2__reduce_cython__(CYTHON_UNUSED struct __pyx_obj_4grpc_7_cython_6cygrpc_MetadataPluginCallCredentials *__pyx_v_self) {
  PyObject *__pyx_r = NULL;
  __Pyx_RefNannyDeclarations
  PyObject *__pyx_t_1 = NULL;
  __Pyx_RefNannySetupContext("__reduce_cython__", 0);

  /* "(tree fragment)":2
 * def __reduce_cython__(self):
 *     raise TypeError("no default __reduce__ due to non-trivial __cinit__")             # <<<<<<<<<<<<<<
 * def __setstate_cython__(self, __pyx_state):
 *     raise TypeError("no default __reduce__ due to non-trivial __cinit__")
 */
  __pyx_t_1 = __Pyx_PyObject_Call(__pyx_builtin_TypeError, __pyx_tuple__54, NULL); if (unlikely(!__pyx_t_1)) __PYX_ERR(7, 2, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_1);
  __Pyx_Raise(__pyx_t_1, 0, 0, 0);
  __Pyx_DECREF(__pyx_t_1); __pyx_t_1 = 0;
  __PYX_ERR(7, 2, __pyx_L1_error)

  /* "(tree fragment)":1
 * def __reduce_cython__(self):             # <<<<<<<<<<<<<<
 *     raise TypeError("no default __reduce__ due to non-trivial __cinit__")
 * def __setstate_cython__(self, __pyx_state):
 */

  /* function exit code */
  __pyx_L1_error:;
  __Pyx_XDECREF(__pyx_t_1);
  __Pyx_AddTraceback("grpc._cython.cygrpc.MetadataPluginCallCredentials.__reduce_cython__", __pyx_clineno, __pyx_lineno, __pyx_filename);
  __pyx_r = NULL;
  __Pyx_XGIVEREF(__pyx_r);
  __Pyx_RefNannyFinishContext();
  return __pyx_r;
}

/* "(tree fragment)":3
 * def __reduce_cython__(self):
 *     raise TypeError("no default __reduce__ due to non-trivial __cinit__")
 * def __setstate_cython__(self, __pyx_state):             # <<<<<<<<<<<<<<
 *     raise TypeError("no default __reduce__ due to non-trivial __cinit__")
 */

/* Python wrapper */
static PyObject *__pyx_pw_4grpc_7_cython_6cygrpc_29MetadataPluginCallCredentials_5__setstate_cython__(PyObject *__pyx_v_self, PyObject *__pyx_v___pyx_state); /*proto*/
static PyObject *__pyx_pw_4grpc_7_cython_6cygrpc_29MetadataPluginCallCredentials_5__setstate_cython__(PyObject *__pyx_v_self, PyObject *__pyx_v___pyx_state) {
  PyObject *__pyx_r = 0;
  __Pyx_RefNannyDeclarations
  __Pyx_RefNannySetupContext("__setstate_cython__ (wrapper)", 0);
  __pyx_r = __pyx_pf_4grpc_7_cython_6cygrpc_29MetadataPluginCallCredentials_4__setstate_cython__(((struct __pyx_obj_4grpc_7_cython_6cygrpc_MetadataPluginCallCredentials *)__pyx_v_self), ((PyObject *)__pyx_v___pyx_state));

  /* function exit code */
  __Pyx_RefNannyFinishContext();
  return __pyx_r;
}

static PyObject *__pyx_pf_4grpc_7_cython_6cygrpc_29MetadataPluginCallCredentials_4__setstate_cython__(CYTHON_UNUSED struct __pyx_obj_4grpc_7_cython_6cygrpc_MetadataPluginCallCredentials *__pyx_v_self, CYTHON_UNUSED PyObject *__pyx_v___pyx_state) {
  PyObject *__pyx_r = NULL;
  __Pyx_RefNannyDeclarations
  PyObject *__pyx_t_1 = NULL;
  __Pyx_RefNannySetupContext("__setstate_cython__", 0);

  /* "(tree fragment)":4
 *     raise TypeError("no default __reduce__ due to non-trivial __cinit__")
 * def __setstate_cython__(self, __pyx_state):
 *     raise TypeError("no default __reduce__ due to non-trivial __cinit__")             # <<<<<<<<<<<<<<
 */
  __pyx_t_1 = __Pyx_PyObject_Call(__pyx_builtin_TypeError, __pyx_tuple__55, NULL); if (unlikely(!__pyx_t_1)) __PYX_ERR(7, 4, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_1);
  __Pyx_Raise(__pyx_t_1, 0, 0, 0);
  __Pyx_DECREF(__pyx_t_1); __pyx_t_1 = 0;
  __PYX_ERR(7, 4, __pyx_L1_error)

  /* "(tree fragment)":3
 * def __reduce_cython__(self):
 *     raise TypeError("no default __reduce__ due to non-trivial __cinit__")
 * def __setstate_cython__(self, __pyx_state):             # <<<<<<<<<<<<<<
 *     raise TypeError("no default __reduce__ due to non-trivial __cinit__")
 */

  /* function exit code */
  __pyx_L1_error:;
  __Pyx_XDECREF(__pyx_t_1);
  __Pyx_AddTraceback("grpc._cython.cygrpc.MetadataPluginCallCredentials.__setstate_cython__", __pyx_clineno, __pyx_lineno, __pyx_filename);
  __pyx_r = NULL;
  __Pyx_XGIVEREF(__pyx_r);
  __Pyx_RefNannyFinishContext();
  return __pyx_r;
}

/* "src/python/grpcio/grpc/_cython/_cygrpc/credentials.pyx.pxi":82
 * 
 * 
 * cdef grpc_call_credentials *_composition(call_credentialses):             # <<<<<<<<<<<<<<
 *   call_credentials_iterator = iter(call_credentialses)
 *   cdef CallCredentials composition = next(call_credentials_iterator)
 */

static grpc_call_credentials *__pyx_f_4grpc_7_cython_6cygrpc__composition(PyObject *__pyx_v_call_credentialses) {
  PyObject *__pyx_v_call_credentials_iterator = NULL;
  struct __pyx_obj_4grpc_7_cython_6cygrpc_CallCredentials *__pyx_v_composition = 0;
  grpc_call_credentials *__pyx_v_c_composition;
  struct __pyx_obj_4grpc_7_cython_6cygrpc_CallCredentials *__pyx_v_additional_call_credentials = 0;
  grpc_call_credentials *__pyx_v_c_additional_call_credentials;
  grpc_call_credentials *__pyx_v_c_next_composition;
  grpc_call_credentials *__pyx_r;
  __Pyx_RefNannyDeclarations
  PyObject *__pyx_t_1 = NULL;
  grpc_call_credentials *__pyx_t_2;
  Py_ssize_t __pyx_t_3;
  PyObject *(*__pyx_t_4)(PyObject *);
  PyObject *__pyx_t_5 = NULL;
  __Pyx_RefNannySetupContext("_composition", 0);

  /* "src/python/grpcio/grpc/_cython/_cygrpc/credentials.pyx.pxi":83
 * 
 * cdef grpc_call_credentials *_composition(call_credentialses):
 *   call_credentials_iterator = iter(call_credentialses)             # <<<<<<<<<<<<<<
 *   cdef CallCredentials composition = next(call_credentials_iterator)
 *   cdef grpc_call_credentials *c_composition = composition.c()
 */
  __pyx_t_1 = PyObject_GetIter(__pyx_v_call_credentialses); if (unlikely(!__pyx_t_1)) __PYX_ERR(4, 83, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_1);
  __pyx_v_call_credentials_iterator = __pyx_t_1;
  __pyx_t_1 = 0;

  /* "src/python/grpcio/grpc/_cython/_cygrpc/credentials.pyx.pxi":84
 * cdef grpc_call_credentials *_composition(call_credentialses):
 *   call_credentials_iterator = iter(call_credentialses)
 *   cdef CallCredentials composition = next(call_credentials_iterator)             # <<<<<<<<<<<<<<
 *   cdef grpc_call_credentials *c_composition = composition.c()
 *   cdef CallCredentials additional_call_credentials
 */
  __pyx_t_1 = __Pyx_PyIter_Next(__pyx_v_call_credentials_iterator); if (unlikely(!__pyx_t_1)) __PYX_ERR(4, 84, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_1);
  if (!(likely(((__pyx_t_1) == Py_None) || likely(__Pyx_TypeTest(__pyx_t_1, __pyx_ptype_4grpc_7_cython_6cygrpc_CallCredentials))))) __PYX_ERR(4, 84, __pyx_L1_error)
  __pyx_v_composition = ((struct __pyx_obj_4grpc_7_cython_6cygrpc_CallCredentials *)__pyx_t_1);
  __pyx_t_1 = 0;

  /* "src/python/grpcio/grpc/_cython/_cygrpc/credentials.pyx.pxi":85
 *   call_credentials_iterator = iter(call_credentialses)
 *   cdef CallCredentials composition = next(call_credentials_iterator)
 *   cdef grpc_call_credentials *c_composition = composition.c()             # <<<<<<<<<<<<<<
 *   cdef CallCredentials additional_call_credentials
 *   cdef grpc_call_credentials *c_additional_call_credentials
 */
  __pyx_t_2 = ((struct __pyx_vtabstruct_4grpc_7_cython_6cygrpc_CallCredentials *)__pyx_v_composition->__pyx_vtab)->c(__pyx_v_composition); if (unlikely(PyErr_Occurred())) __PYX_ERR(4, 85, __pyx_L1_error)
  __pyx_v_c_composition = __pyx_t_2;

  /* "src/python/grpcio/grpc/_cython/_cygrpc/credentials.pyx.pxi":89
 *   cdef grpc_call_credentials *c_additional_call_credentials
 *   cdef grpc_call_credentials *c_next_composition
 *   for additional_call_credentials in call_credentials_iterator:             # <<<<<<<<<<<<<<
 *     c_additional_call_credentials = additional_call_credentials.c()
 *     c_next_composition = grpc_composite_call_credentials_create(
 */
  if (likely(PyList_CheckExact(__pyx_v_call_credentials_iterator)) || PyTuple_CheckExact(__pyx_v_call_credentials_iterator)) {
    __pyx_t_1 = __pyx_v_call_credentials_iterator; __Pyx_INCREF(__pyx_t_1); __pyx_t_3 = 0;
    __pyx_t_4 = NULL;
  } else {
    __pyx_t_3 = -1; __pyx_t_1 = PyObject_GetIter(__pyx_v_call_credentials_iterator); if (unlikely(!__pyx_t_1)) __PYX_ERR(4, 89, __pyx_L1_error)
    __Pyx_GOTREF(__pyx_t_1);
    __pyx_t_4 = Py_TYPE(__pyx_t_1)->tp_iternext; if (unlikely(!__pyx_t_4)) __PYX_ERR(4, 89, __pyx_L1_error)
  }
  for (;;) {
    if (likely(!__pyx_t_4)) {
      if (likely(PyList_CheckExact(__pyx_t_1))) {
        if (__pyx_t_3 >= PyList_GET_SIZE(__pyx_t_1)) break;
        #if CYTHON_ASSUME_SAFE_MACROS && !CYTHON_AVOID_BORROWED_REFS
        __pyx_t_5 = PyList_GET_ITEM(__pyx_t_1, __pyx_t_3); __Pyx_INCREF(__pyx_t_5); __pyx_t_3++; if (unlikely(0 < 0)) __PYX_ERR(4, 89, __pyx_L1_error)
        #else
        __pyx_t_5 = PySequence_ITEM(__pyx_t_1, __pyx_t_3); __pyx_t_3++; if (unlikely(!__pyx_t_5)) __PYX_ERR(4, 89, __pyx_L1_error)
        __Pyx_GOTREF(__pyx_t_5);
        #endif
      } else {
        if (__pyx_t_3 >= PyTuple_GET_SIZE(__pyx_t_1)) break;
        #if CYTHON_ASSUME_SAFE_MACROS && !CYTHON_AVOID_BORROWED_REFS
        __pyx_t_5 = PyTuple_GET_ITEM(__pyx_t_1, __pyx_t_3); __Pyx_INCREF(__pyx_t_5); __pyx_t_3++; if (unlikely(0 < 0)) __PYX_ERR(4, 89, __pyx_L1_error)
        #else
        __pyx_t_5 = PySequence_ITEM(__pyx_t_1, __pyx_t_3); __pyx_t_3++; if (unlikely(!__pyx_t_5)) __PYX_ERR(4, 89, __pyx_L1_error)
        __Pyx_GOTREF(__pyx_t_5);
        #endif
      }
    } else {
      __pyx_t_5 = __pyx_t_4(__pyx_t_1);
      if (unlikely(!__pyx_t_5)) {
        PyObject* exc_type = PyErr_Occurred();
        if (exc_type) {
          if (likely(__Pyx_PyErr_GivenExceptionMatches(exc_type, PyExc_StopIteration))) PyErr_Clear();
          else __PYX_ERR(4, 89, __pyx_L1_error)
        }
        break;
      }
      __Pyx_GOTREF(__pyx_t_5);
    }
    if (!(likely(((__pyx_t_5) == Py_None) || likely(__Pyx_TypeTest(__pyx_t_5, __pyx_ptype_4grpc_7_cython_6cygrpc_CallCredentials))))) __PYX_ERR(4, 89, __pyx_L1_error)
    __Pyx_XDECREF_SET(__pyx_v_additional_call_credentials, ((struct __pyx_obj_4grpc_7_cython_6cygrpc_CallCredentials *)__pyx_t_5));
    __pyx_t_5 = 0;

    /* "src/python/grpcio/grpc/_cython/_cygrpc/credentials.pyx.pxi":90
 *   cdef grpc_call_credentials *c_next_composition
 *   for additional_call_credentials in call_credentials_iterator:
 *     c_additional_call_credentials = additional_call_credentials.c()             # <<<<<<<<<<<<<<
 *     c_next_composition = grpc_composite_call_credentials_create(
 *         c_composition, c_additional_call_credentials, NULL)
 */
    __pyx_t_2 = ((struct __pyx_vtabstruct_4grpc_7_cython_6cygrpc_CallCredentials *)__pyx_v_additional_call_credentials->__pyx_vtab)->c(__pyx_v_additional_call_credentials); if (unlikely(PyErr_Occurred())) __PYX_ERR(4, 90, __pyx_L1_error)
    __pyx_v_c_additional_call_credentials = __pyx_t_2;

    /* "src/python/grpcio/grpc/_cython/_cygrpc/credentials.pyx.pxi":91
 *   for additional_call_credentials in call_credentials_iterator:
 *     c_additional_call_credentials = additional_call_credentials.c()
 *     c_next_composition = grpc_composite_call_credentials_create(             # <<<<<<<<<<<<<<
 *         c_composition, c_additional_call_credentials, NULL)
 *     grpc_call_credentials_release(c_composition)
 */
    __pyx_v_c_next_composition = grpc_composite_call_credentials_create(__pyx_v_c_composition, __pyx_v_c_additional_call_credentials, NULL);

    /* "src/python/grpcio/grpc/_cython/_cygrpc/credentials.pyx.pxi":93
 *     c_next_composition = grpc_composite_call_credentials_create(
 *         c_composition, c_additional_call_credentials, NULL)
 *     grpc_call_credentials_release(c_composition)             # <<<<<<<<<<<<<<
 *     grpc_call_credentials_release(c_additional_call_credentials)
 *     c_composition = c_next_composition
 */
    grpc_call_credentials_release(__pyx_v_c_composition);

    /* "src/python/grpcio/grpc/_cython/_cygrpc/credentials.pyx.pxi":94
 *         c_composition, c_additional_call_credentials, NULL)
 *     grpc_call_credentials_release(c_composition)
 *     grpc_call_credentials_release(c_additional_call_credentials)             # <<<<<<<<<<<<<<
 *     c_composition = c_next_composition
 *   return c_composition
 */
    grpc_call_credentials_release(__pyx_v_c_additional_call_credentials);

    /* "src/python/grpcio/grpc/_cython/_cygrpc/credentials.pyx.pxi":95
 *     grpc_call_credentials_release(c_composition)
 *     grpc_call_credentials_release(c_additional_call_credentials)
 *     c_composition = c_next_composition             # <<<<<<<<<<<<<<
 *   return c_composition
 * 
 */
    __pyx_v_c_composition = __pyx_v_c_next_composition;

    /* "src/python/grpcio/grpc/_cython/_cygrpc/credentials.pyx.pxi":89
 *   cdef grpc_call_credentials *c_additional_call_credentials
 *   cdef grpc_call_credentials *c_next_composition
 *   for additional_call_credentials in call_credentials_iterator:             # <<<<<<<<<<<<<<
 *     c_additional_call_credentials = additional_call_credentials.c()
 *     c_next_composition = grpc_composite_call_credentials_create(
 */
  }
  __Pyx_DECREF(__pyx_t_1); __pyx_t_1 = 0;

  /* "src/python/grpcio/grpc/_cython/_cygrpc/credentials.pyx.pxi":96
 *     grpc_call_credentials_release(c_additional_call_credentials)
 *     c_composition = c_next_composition
 *   return c_composition             # <<<<<<<<<<<<<<
 * 
 * 
 */
  __pyx_r = __pyx_v_c_composition;
  goto __pyx_L0;

  /* "src/python/grpcio/grpc/_cython/_cygrpc/credentials.pyx.pxi":82
 * 
 * 
 * cdef grpc_call_credentials *_composition(call_credentialses):             # <<<<<<<<<<<<<<
 *   call_credentials_iterator = iter(call_credentialses)
 *   cdef CallCredentials composition = next(call_credentials_iterator)
 */

  /* function exit code */
  __pyx_L1_error:;
  __Pyx_XDECREF(__pyx_t_1);
  __Pyx_XDECREF(__pyx_t_5);
  __Pyx_WriteUnraisable("grpc._cython.cygrpc._composition", __pyx_clineno, __pyx_lineno, __pyx_filename, 1, 0);
  __pyx_r = 0;
  __pyx_L0:;
  __Pyx_XDECREF(__pyx_v_call_credentials_iterator);
  __Pyx_XDECREF((PyObject *)__pyx_v_composition);
  __Pyx_XDECREF((PyObject *)__pyx_v_additional_call_credentials);
  __Pyx_RefNannyFinishContext();
  return __pyx_r;
}

/* "src/python/grpcio/grpc/_cython/_cygrpc/credentials.pyx.pxi":101
 * cdef class CompositeCallCredentials(CallCredentials):
 * 
 *   def __cinit__(self, call_credentialses):             # <<<<<<<<<<<<<<
 *     self._call_credentialses = call_credentialses
 * 
 */

/* Python wrapper */
static int __pyx_pw_4grpc_7_cython_6cygrpc_24CompositeCallCredentials_1__cinit__(PyObject *__pyx_v_self, PyObject *__pyx_args, PyObject *__pyx_kwds); /*proto*/
static int __pyx_pw_4grpc_7_cython_6cygrpc_24CompositeCallCredentials_1__cinit__(PyObject *__pyx_v_self, PyObject *__pyx_args, PyObject *__pyx_kwds) {
  PyObject *__pyx_v_call_credentialses = 0;
  int __pyx_r;
  __Pyx_RefNannyDeclarations
  __Pyx_RefNannySetupContext("__cinit__ (wrapper)", 0);
  {
    static PyObject **__pyx_pyargnames[] = {&__pyx_n_s_call_credentialses,0};
    PyObject* values[1] = {0};
    if (unlikely(__pyx_kwds)) {
      Py_ssize_t kw_args;
      const Py_ssize_t pos_args = PyTuple_GET_SIZE(__pyx_args);
      switch (pos_args) {
        case  1: values[0] = PyTuple_GET_ITEM(__pyx_args, 0);
        CYTHON_FALLTHROUGH;
        case  0: break;
        default: goto __pyx_L5_argtuple_error;
      }
      kw_args = PyDict_Size(__pyx_kwds);
      switch (pos_args) {
        case  0:
        if (likely((values[0] = __Pyx_PyDict_GetItemStr(__pyx_kwds, __pyx_n_s_call_credentialses)) != 0)) kw_args--;
        else goto __pyx_L5_argtuple_error;
      }
      if (unlikely(kw_args > 0)) {
        if (unlikely(__Pyx_ParseOptionalKeywords(__pyx_kwds, __pyx_pyargnames, 0, values, pos_args, "__cinit__") < 0)) __PYX_ERR(4, 101, __pyx_L3_error)
      }
    } else if (PyTuple_GET_SIZE(__pyx_args) != 1) {
      goto __pyx_L5_argtuple_error;
    } else {
      values[0] = PyTuple_GET_ITEM(__pyx_args, 0);
    }
    __pyx_v_call_credentialses = values[0];
  }
  goto __pyx_L4_argument_unpacking_done;
  __pyx_L5_argtuple_error:;
  __Pyx_RaiseArgtupleInvalid("__cinit__", 1, 1, 1, PyTuple_GET_SIZE(__pyx_args)); __PYX_ERR(4, 101, __pyx_L3_error)
  __pyx_L3_error:;
  __Pyx_AddTraceback("grpc._cython.cygrpc.CompositeCallCredentials.__cinit__", __pyx_clineno, __pyx_lineno, __pyx_filename);
  __Pyx_RefNannyFinishContext();
  return -1;
  __pyx_L4_argument_unpacking_done:;
  __pyx_r = __pyx_pf_4grpc_7_cython_6cygrpc_24CompositeCallCredentials___cinit__(((struct __pyx_obj_4grpc_7_cython_6cygrpc_CompositeCallCredentials *)__pyx_v_self), __pyx_v_call_credentialses);

  /* function exit code */
  __Pyx_RefNannyFinishContext();
  return __pyx_r;
}

static int __pyx_pf_4grpc_7_cython_6cygrpc_24CompositeCallCredentials___cinit__(struct __pyx_obj_4grpc_7_cython_6cygrpc_CompositeCallCredentials *__pyx_v_self, PyObject *__pyx_v_call_credentialses) {
  int __pyx_r;
  __Pyx_RefNannyDeclarations
  PyObject *__pyx_t_1 = NULL;
  __Pyx_RefNannySetupContext("__cinit__", 0);

  /* "src/python/grpcio/grpc/_cython/_cygrpc/credentials.pyx.pxi":102
 * 
 *   def __cinit__(self, call_credentialses):
 *     self._call_credentialses = call_credentialses             # <<<<<<<<<<<<<<
 * 
 *   cdef grpc_call_credentials *c(self) except *:
 */
  if (!(likely(PyTuple_CheckExact(__pyx_v_call_credentialses))||((__pyx_v_call_credentialses) == Py_None)||(PyErr_Format(PyExc_TypeError, "Expected %.16s, got %.200s", "tuple", Py_TYPE(__pyx_v_call_credentialses)->tp_name), 0))) __PYX_ERR(4, 102, __pyx_L1_error)
  __pyx_t_1 = __pyx_v_call_credentialses;
  __Pyx_INCREF(__pyx_t_1);
  __Pyx_GIVEREF(__pyx_t_1);
  __Pyx_GOTREF(__pyx_v_self->_call_credentialses);
  __Pyx_DECREF(__pyx_v_self->_call_credentialses);
  __pyx_v_self->_call_credentialses = ((PyObject*)__pyx_t_1);
  __pyx_t_1 = 0;

  /* "src/python/grpcio/grpc/_cython/_cygrpc/credentials.pyx.pxi":101
 * cdef class CompositeCallCredentials(CallCredentials):
 * 
 *   def __cinit__(self, call_credentialses):             # <<<<<<<<<<<<<<
 *     self._call_credentialses = call_credentialses
 * 
 */

  /* function exit code */
  __pyx_r = 0;
  goto __pyx_L0;
  __pyx_L1_error:;
  __Pyx_XDECREF(__pyx_t_1);
  __Pyx_AddTraceback("grpc._cython.cygrpc.CompositeCallCredentials.__cinit__", __pyx_clineno, __pyx_lineno, __pyx_filename);
  __pyx_r = -1;
  __pyx_L0:;
  __Pyx_RefNannyFinishContext();
  return __pyx_r;
}

/* "src/python/grpcio/grpc/_cython/_cygrpc/credentials.pyx.pxi":104
 *     self._call_credentialses = call_credentialses
 * 
 *   cdef grpc_call_credentials *c(self) except *:             # <<<<<<<<<<<<<<
 *     return _composition(self._call_credentialses)
 * 
 */

static grpc_call_credentials *__pyx_f_4grpc_7_cython_6cygrpc_24CompositeCallCredentials_c(struct __pyx_obj_4grpc_7_cython_6cygrpc_CompositeCallCredentials *__pyx_v_self) {
  grpc_call_credentials *__pyx_r;
  __Pyx_RefNannyDeclarations
  PyObject *__pyx_t_1 = NULL;
  __Pyx_RefNannySetupContext("c", 0);

  /* "src/python/grpcio/grpc/_cython/_cygrpc/credentials.pyx.pxi":105
 * 
 *   cdef grpc_call_credentials *c(self) except *:
 *     return _composition(self._call_credentialses)             # <<<<<<<<<<<<<<
 * 
 * 
 */
  __pyx_t_1 = __pyx_v_self->_call_credentialses;
  __Pyx_INCREF(__pyx_t_1);
  __pyx_r = __pyx_f_4grpc_7_cython_6cygrpc__composition(__pyx_t_1);
  __Pyx_DECREF(__pyx_t_1); __pyx_t_1 = 0;
  goto __pyx_L0;

  /* "src/python/grpcio/grpc/_cython/_cygrpc/credentials.pyx.pxi":104
 *     self._call_credentialses = call_credentialses
 * 
 *   cdef grpc_call_credentials *c(self) except *:             # <<<<<<<<<<<<<<
 *     return _composition(self._call_credentialses)
 * 
 */

  /* function exit code */
  __pyx_L0:;
  __Pyx_RefNannyFinishContext();
  return __pyx_r;
}

/* "src/python/grpcio/grpc/_cython/_cygrpc/credentials.pxd.pxi":47
 * cdef class CompositeCallCredentials(CallCredentials):
 * 
 *   cdef readonly tuple _call_credentialses             # <<<<<<<<<<<<<<
 * 
 *   cdef grpc_call_credentials *c(self) except *
 */

/* Python wrapper */
static PyObject *__pyx_pw_4grpc_7_cython_6cygrpc_24CompositeCallCredentials_19_call_credentialses_1__get__(PyObject *__pyx_v_self); /*proto*/
static PyObject *__pyx_pw_4grpc_7_cython_6cygrpc_24CompositeCallCredentials_19_call_credentialses_1__get__(PyObject *__pyx_v_self) {
  PyObject *__pyx_r = 0;
  __Pyx_RefNannyDeclarations
  __Pyx_RefNannySetupContext("__get__ (wrapper)", 0);
  __pyx_r = __pyx_pf_4grpc_7_cython_6cygrpc_24CompositeCallCredentials_19_call_credentialses___get__(((struct __pyx_obj_4grpc_7_cython_6cygrpc_CompositeCallCredentials *)__pyx_v_self));

  /* function exit code */
  __Pyx_RefNannyFinishContext();
  return __pyx_r;
}

static PyObject *__pyx_pf_4grpc_7_cython_6cygrpc_24CompositeCallCredentials_19_call_credentialses___get__(struct __pyx_obj_4grpc_7_cython_6cygrpc_CompositeCallCredentials *__pyx_v_self) {
  PyObject *__pyx_r = NULL;
  __Pyx_RefNannyDeclarations
  __Pyx_RefNannySetupContext("__get__", 0);
  __Pyx_XDECREF(__pyx_r);
  __Pyx_INCREF(__pyx_v_self->_call_credentialses);
  __pyx_r = __pyx_v_self->_call_credentialses;
  goto __pyx_L0;

  /* function exit code */
  __pyx_L0:;
  __Pyx_XGIVEREF(__pyx_r);
  __Pyx_RefNannyFinishContext();
  return __pyx_r;
}

/* "(tree fragment)":1
 * def __reduce_cython__(self):             # <<<<<<<<<<<<<<
 *     raise TypeError("no default __reduce__ due to non-trivial __cinit__")
 * def __setstate_cython__(self, __pyx_state):
 */

/* Python wrapper */
static PyObject *__pyx_pw_4grpc_7_cython_6cygrpc_24CompositeCallCredentials_3__reduce_cython__(PyObject *__pyx_v_self, CYTHON_UNUSED PyObject *unused); /*proto*/
static PyObject *__pyx_pw_4grpc_7_cython_6cygrpc_24CompositeCallCredentials_3__reduce_cython__(PyObject *__pyx_v_self, CYTHON_UNUSED PyObject *unused) {
  PyObject *__pyx_r = 0;
  __Pyx_RefNannyDeclarations
  __Pyx_RefNannySetupContext("__reduce_cython__ (wrapper)", 0);
  __pyx_r = __pyx_pf_4grpc_7_cython_6cygrpc_24CompositeCallCredentials_2__reduce_cython__(((struct __pyx_obj_4grpc_7_cython_6cygrpc_CompositeCallCredentials *)__pyx_v_self));

  /* function exit code */
  __Pyx_RefNannyFinishContext();
  return __pyx_r;
}

static PyObject *__pyx_pf_4grpc_7_cython_6cygrpc_24CompositeCallCredentials_2__reduce_cython__(CYTHON_UNUSED struct __pyx_obj_4grpc_7_cython_6cygrpc_CompositeCallCredentials *__pyx_v_self) {
  PyObject *__pyx_r = NULL;
  __Pyx_RefNannyDeclarations
  PyObject *__pyx_t_1 = NULL;
  __Pyx_RefNannySetupContext("__reduce_cython__", 0);

  /* "(tree fragment)":2
 * def __reduce_cython__(self):
 *     raise TypeError("no default __reduce__ due to non-trivial __cinit__")             # <<<<<<<<<<<<<<
 * def __setstate_cython__(self, __pyx_state):
 *     raise TypeError("no default __reduce__ due to non-trivial __cinit__")
 */
  __pyx_t_1 = __Pyx_PyObject_Call(__pyx_builtin_TypeError, __pyx_tuple__56, NULL); if (unlikely(!__pyx_t_1)) __PYX_ERR(7, 2, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_1);
  __Pyx_Raise(__pyx_t_1, 0, 0, 0);
  __Pyx_DECREF(__pyx_t_1); __pyx_t_1 = 0;
  __PYX_ERR(7, 2, __pyx_L1_error)

  /* "(tree fragment)":1
 * def __reduce_cython__(self):             # <<<<<<<<<<<<<<
 *     raise TypeError("no default __reduce__ due to non-trivial __cinit__")
 * def __setstate_cython__(self, __pyx_state):
 */

  /* function exit code */
  __pyx_L1_error:;
  __Pyx_XDECREF(__pyx_t_1);
  __Pyx_AddTraceback("grpc._cython.cygrpc.CompositeCallCredentials.__reduce_cython__", __pyx_clineno, __pyx_lineno, __pyx_filename);
  __pyx_r = NULL;
  __Pyx_XGIVEREF(__pyx_r);
  __Pyx_RefNannyFinishContext();
  return __pyx_r;
}

/* "(tree fragment)":3
 * def __reduce_cython__(self):
 *     raise TypeError("no default __reduce__ due to non-trivial __cinit__")
 * def __setstate_cython__(self, __pyx_state):             # <<<<<<<<<<<<<<
 *     raise TypeError("no default __reduce__ due to non-trivial __cinit__")
 */

/* Python wrapper */
static PyObject *__pyx_pw_4grpc_7_cython_6cygrpc_24CompositeCallCredentials_5__setstate_cython__(PyObject *__pyx_v_self, PyObject *__pyx_v___pyx_state); /*proto*/
static PyObject *__pyx_pw_4grpc_7_cython_6cygrpc_24CompositeCallCredentials_5__setstate_cython__(PyObject *__pyx_v_self, PyObject *__pyx_v___pyx_state) {
  PyObject *__pyx_r = 0;
  __Pyx_RefNannyDeclarations
  __Pyx_RefNannySetupContext("__setstate_cython__ (wrapper)", 0);
  __pyx_r = __pyx_pf_4grpc_7_cython_6cygrpc_24CompositeCallCredentials_4__setstate_cython__(((struct __pyx_obj_4grpc_7_cython_6cygrpc_CompositeCallCredentials *)__pyx_v_self), ((PyObject *)__pyx_v___pyx_state));

  /* function exit code */
  __Pyx_RefNannyFinishContext();
  return __pyx_r;
}

static PyObject *__pyx_pf_4grpc_7_cython_6cygrpc_24CompositeCallCredentials_4__setstate_cython__(CYTHON_UNUSED struct __pyx_obj_4grpc_7_cython_6cygrpc_CompositeCallCredentials *__pyx_v_self, CYTHON_UNUSED PyObject *__pyx_v___pyx_state) {
  PyObject *__pyx_r = NULL;
  __Pyx_RefNannyDeclarations
  PyObject *__pyx_t_1 = NULL;
  __Pyx_RefNannySetupContext("__setstate_cython__", 0);

  /* "(tree fragment)":4
 *     raise TypeError("no default __reduce__ due to non-trivial __cinit__")
 * def __setstate_cython__(self, __pyx_state):
 *     raise TypeError("no default __reduce__ due to non-trivial __cinit__")             # <<<<<<<<<<<<<<
 */
  __pyx_t_1 = __Pyx_PyObject_Call(__pyx_builtin_TypeError, __pyx_tuple__57, NULL); if (unlikely(!__pyx_t_1)) __PYX_ERR(7, 4, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_1);
  __Pyx_Raise(__pyx_t_1, 0, 0, 0);
  __Pyx_DECREF(__pyx_t_1); __pyx_t_1 = 0;
  __PYX_ERR(7, 4, __pyx_L1_error)

  /* "(tree fragment)":3
 * def __reduce_cython__(self):
 *     raise TypeError("no default __reduce__ due to non-trivial __cinit__")
 * def __setstate_cython__(self, __pyx_state):             # <<<<<<<<<<<<<<
 *     raise TypeError("no default __reduce__ due to non-trivial __cinit__")
 */

  /* function exit code */
  __pyx_L1_error:;
  __Pyx_XDECREF(__pyx_t_1);
  __Pyx_AddTraceback("grpc._cython.cygrpc.CompositeCallCredentials.__setstate_cython__", __pyx_clineno, __pyx_lineno, __pyx_filename);
  __pyx_r = NULL;
  __Pyx_XGIVEREF(__pyx_r);
  __Pyx_RefNannyFinishContext();
  return __pyx_r;
}

/* "src/python/grpcio/grpc/_cython/_cygrpc/credentials.pyx.pxi":110
 * cdef class ChannelCredentials:
 * 
 *   cdef grpc_channel_credentials *c(self) except *:             # <<<<<<<<<<<<<<
 *     raise NotImplementedError()
 * 
 */

static grpc_channel_credentials *__pyx_f_4grpc_7_cython_6cygrpc_18ChannelCredentials_c(CYTHON_UNUSED struct __pyx_obj_4grpc_7_cython_6cygrpc_ChannelCredentials *__pyx_v_self) {
  grpc_channel_credentials *__pyx_r;
  __Pyx_RefNannyDeclarations
  PyObject *__pyx_t_1 = NULL;
  __Pyx_RefNannySetupContext("c", 0);

  /* "src/python/grpcio/grpc/_cython/_cygrpc/credentials.pyx.pxi":111
 * 
 *   cdef grpc_channel_credentials *c(self) except *:
 *     raise NotImplementedError()             # <<<<<<<<<<<<<<
 * 
 * 
 */
  __pyx_t_1 = __Pyx_PyObject_CallNoArg(__pyx_builtin_NotImplementedError); if (unlikely(!__pyx_t_1)) __PYX_ERR(4, 111, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_1);
  __Pyx_Raise(__pyx_t_1, 0, 0, 0);
  __Pyx_DECREF(__pyx_t_1); __pyx_t_1 = 0;
  __PYX_ERR(4, 111, __pyx_L1_error)

  /* "src/python/grpcio/grpc/_cython/_cygrpc/credentials.pyx.pxi":110
 * cdef class ChannelCredentials:
 * 
 *   cdef grpc_channel_credentials *c(self) except *:             # <<<<<<<<<<<<<<
 *     raise NotImplementedError()
 * 
 */

  /* function exit code */
  __pyx_L1_error:;
  __Pyx_XDECREF(__pyx_t_1);
  __Pyx_AddTraceback("grpc._cython.cygrpc.ChannelCredentials.c", __pyx_clineno, __pyx_lineno, __pyx_filename);
  __pyx_r = 0;
  __Pyx_RefNannyFinishContext();
  return __pyx_r;
}

/* "(tree fragment)":1
 * def __reduce_cython__(self):             # <<<<<<<<<<<<<<
 *     cdef bint use_setstate
 *     state = ()
 */

/* Python wrapper */
static PyObject *__pyx_pw_4grpc_7_cython_6cygrpc_18ChannelCredentials_1__reduce_cython__(PyObject *__pyx_v_self, CYTHON_UNUSED PyObject *unused); /*proto*/
static PyObject *__pyx_pw_4grpc_7_cython_6cygrpc_18ChannelCredentials_1__reduce_cython__(PyObject *__pyx_v_self, CYTHON_UNUSED PyObject *unused) {
  PyObject *__pyx_r = 0;
  __Pyx_RefNannyDeclarations
  __Pyx_RefNannySetupContext("__reduce_cython__ (wrapper)", 0);
  __pyx_r = __pyx_pf_4grpc_7_cython_6cygrpc_18ChannelCredentials___reduce_cython__(((struct __pyx_obj_4grpc_7_cython_6cygrpc_ChannelCredentials *)__pyx_v_self));

  /* function exit code */
  __Pyx_RefNannyFinishContext();
  return __pyx_r;
}

static PyObject *__pyx_pf_4grpc_7_cython_6cygrpc_18ChannelCredentials___reduce_cython__(struct __pyx_obj_4grpc_7_cython_6cygrpc_ChannelCredentials *__pyx_v_self) {
  int __pyx_v_use_setstate;
  PyObject *__pyx_v_state = NULL;
  PyObject *__pyx_v__dict = NULL;
  PyObject *__pyx_r = NULL;
  __Pyx_RefNannyDeclarations
  PyObject *__pyx_t_1 = NULL;
  int __pyx_t_2;
  int __pyx_t_3;
  PyObject *__pyx_t_4 = NULL;
  PyObject *__pyx_t_5 = NULL;
  __Pyx_RefNannySetupContext("__reduce_cython__", 0);

  /* "(tree fragment)":3
 * def __reduce_cython__(self):
 *     cdef bint use_setstate
 *     state = ()             # <<<<<<<<<<<<<<
 *     _dict = getattr(self, '__dict__', None)
 *     if _dict is not None:
 */
  __Pyx_INCREF(__pyx_empty_tuple);
  __pyx_v_state = __pyx_empty_tuple;

  /* "(tree fragment)":4
 *     cdef bint use_setstate
 *     state = ()
 *     _dict = getattr(self, '__dict__', None)             # <<<<<<<<<<<<<<
 *     if _dict is not None:
 *         state += (_dict,)
 */
  __pyx_t_1 = __Pyx_GetAttr3(((PyObject *)__pyx_v_self), __pyx_n_s_dict, Py_None); if (unlikely(!__pyx_t_1)) __PYX_ERR(7, 4, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_1);
  __pyx_v__dict = __pyx_t_1;
  __pyx_t_1 = 0;

  /* "(tree fragment)":5
 *     state = ()
 *     _dict = getattr(self, '__dict__', None)
 *     if _dict is not None:             # <<<<<<<<<<<<<<
 *         state += (_dict,)
 *         use_setstate = True
 */
  __pyx_t_2 = (__pyx_v__dict != Py_None);
  __pyx_t_3 = (__pyx_t_2 != 0);
  if (__pyx_t_3) {

    /* "(tree fragment)":6
 *     _dict = getattr(self, '__dict__', None)
 *     if _dict is not None:
 *         state += (_dict,)             # <<<<<<<<<<<<<<
 *         use_setstate = True
 *     else:
 */
    __pyx_t_1 = PyTuple_New(1); if (unlikely(!__pyx_t_1)) __PYX_ERR(7, 6, __pyx_L1_error)
    __Pyx_GOTREF(__pyx_t_1);
    __Pyx_INCREF(__pyx_v__dict);
    __Pyx_GIVEREF(__pyx_v__dict);
    PyTuple_SET_ITEM(__pyx_t_1, 0, __pyx_v__dict);
    __pyx_t_4 = PyNumber_InPlaceAdd(__pyx_v_state, __pyx_t_1); if (unlikely(!__pyx_t_4)) __PYX_ERR(7, 6, __pyx_L1_error)
    __Pyx_GOTREF(__pyx_t_4);
    __Pyx_DECREF(__pyx_t_1); __pyx_t_1 = 0;
    __Pyx_DECREF_SET(__pyx_v_state, ((PyObject*)__pyx_t_4));
    __pyx_t_4 = 0;

    /* "(tree fragment)":7
 *     if _dict is not None:
 *         state += (_dict,)
 *         use_setstate = True             # <<<<<<<<<<<<<<
 *     else:
 *         use_setstate = False
 */
    __pyx_v_use_setstate = 1;

    /* "(tree fragment)":5
 *     state = ()
 *     _dict = getattr(self, '__dict__', None)
 *     if _dict is not None:             # <<<<<<<<<<<<<<
 *         state += (_dict,)
 *         use_setstate = True
 */
    goto __pyx_L3;
  }

  /* "(tree fragment)":9
 *         use_setstate = True
 *     else:
 *         use_setstate = False             # <<<<<<<<<<<<<<
 *     if use_setstate:
 *         return __pyx_unpickle_ChannelCredentials, (type(self), 0xd41d8cd, None), state
 */
  /*else*/ {
    __pyx_v_use_setstate = 0;
  }
  __pyx_L3:;

  /* "(tree fragment)":10
 *     else:
 *         use_setstate = False
 *     if use_setstate:             # <<<<<<<<<<<<<<
 *         return __pyx_unpickle_ChannelCredentials, (type(self), 0xd41d8cd, None), state
 *     else:
 */
  __pyx_t_3 = (__pyx_v_use_setstate != 0);
  if (__pyx_t_3) {

    /* "(tree fragment)":11
 *         use_setstate = False
 *     if use_setstate:
 *         return __pyx_unpickle_ChannelCredentials, (type(self), 0xd41d8cd, None), state             # <<<<<<<<<<<<<<
 *     else:
 *         return __pyx_unpickle_ChannelCredentials, (type(self), 0xd41d8cd, state)
 */
    __Pyx_XDECREF(__pyx_r);
    __pyx_t_4 = __Pyx_GetModuleGlobalName(__pyx_n_s_pyx_unpickle_ChannelCredential); if (unlikely(!__pyx_t_4)) __PYX_ERR(7, 11, __pyx_L1_error)
    __Pyx_GOTREF(__pyx_t_4);
    __pyx_t_1 = PyTuple_New(3); if (unlikely(!__pyx_t_1)) __PYX_ERR(7, 11, __pyx_L1_error)
    __Pyx_GOTREF(__pyx_t_1);
    __Pyx_INCREF(((PyObject *)Py_TYPE(((PyObject *)__pyx_v_self))));
    __Pyx_GIVEREF(((PyObject *)Py_TYPE(((PyObject *)__pyx_v_self))));
    PyTuple_SET_ITEM(__pyx_t_1, 0, ((PyObject *)Py_TYPE(((PyObject *)__pyx_v_self))));
    __Pyx_INCREF(__pyx_int_222419149);
    __Pyx_GIVEREF(__pyx_int_222419149);
    PyTuple_SET_ITEM(__pyx_t_1, 1, __pyx_int_222419149);
    __Pyx_INCREF(Py_None);
    __Pyx_GIVEREF(Py_None);
    PyTuple_SET_ITEM(__pyx_t_1, 2, Py_None);
    __pyx_t_5 = PyTuple_New(3); if (unlikely(!__pyx_t_5)) __PYX_ERR(7, 11, __pyx_L1_error)
    __Pyx_GOTREF(__pyx_t_5);
    __Pyx_GIVEREF(__pyx_t_4);
    PyTuple_SET_ITEM(__pyx_t_5, 0, __pyx_t_4);
    __Pyx_GIVEREF(__pyx_t_1);
    PyTuple_SET_ITEM(__pyx_t_5, 1, __pyx_t_1);
    __Pyx_INCREF(__pyx_v_state);
    __Pyx_GIVEREF(__pyx_v_state);
    PyTuple_SET_ITEM(__pyx_t_5, 2, __pyx_v_state);
    __pyx_t_4 = 0;
    __pyx_t_1 = 0;
    __pyx_r = __pyx_t_5;
    __pyx_t_5 = 0;
    goto __pyx_L0;

    /* "(tree fragment)":10
 *     else:
 *         use_setstate = False
 *     if use_setstate:             # <<<<<<<<<<<<<<
 *         return __pyx_unpickle_ChannelCredentials, (type(self), 0xd41d8cd, None), state
 *     else:
 */
  }

  /* "(tree fragment)":13
 *         return __pyx_unpickle_ChannelCredentials, (type(self), 0xd41d8cd, None), state
 *     else:
 *         return __pyx_unpickle_ChannelCredentials, (type(self), 0xd41d8cd, state)             # <<<<<<<<<<<<<<
 * def __setstate_cython__(self, __pyx_state):
 *     __pyx_unpickle_ChannelCredentials__set_state(self, __pyx_state)
 */
  /*else*/ {
    __Pyx_XDECREF(__pyx_r);
    __pyx_t_5 = __Pyx_GetModuleGlobalName(__pyx_n_s_pyx_unpickle_ChannelCredential); if (unlikely(!__pyx_t_5)) __PYX_ERR(7, 13, __pyx_L1_error)
    __Pyx_GOTREF(__pyx_t_5);
    __pyx_t_1 = PyTuple_New(3); if (unlikely(!__pyx_t_1)) __PYX_ERR(7, 13, __pyx_L1_error)
    __Pyx_GOTREF(__pyx_t_1);
    __Pyx_INCREF(((PyObject *)Py_TYPE(((PyObject *)__pyx_v_self))));
    __Pyx_GIVEREF(((PyObject *)Py_TYPE(((PyObject *)__pyx_v_self))));
    PyTuple_SET_ITEM(__pyx_t_1, 0, ((PyObject *)Py_TYPE(((PyObject *)__pyx_v_self))));
    __Pyx_INCREF(__pyx_int_222419149);
    __Pyx_GIVEREF(__pyx_int_222419149);
    PyTuple_SET_ITEM(__pyx_t_1, 1, __pyx_int_222419149);
    __Pyx_INCREF(__pyx_v_state);
    __Pyx_GIVEREF(__pyx_v_state);
    PyTuple_SET_ITEM(__pyx_t_1, 2, __pyx_v_state);
    __pyx_t_4 = PyTuple_New(2); if (unlikely(!__pyx_t_4)) __PYX_ERR(7, 13, __pyx_L1_error)
    __Pyx_GOTREF(__pyx_t_4);
    __Pyx_GIVEREF(__pyx_t_5);
    PyTuple_SET_ITEM(__pyx_t_4, 0, __pyx_t_5);
    __Pyx_GIVEREF(__pyx_t_1);
    PyTuple_SET_ITEM(__pyx_t_4, 1, __pyx_t_1);
    __pyx_t_5 = 0;
    __pyx_t_1 = 0;
    __pyx_r = __pyx_t_4;
    __pyx_t_4 = 0;
    goto __pyx_L0;
  }

  /* "(tree fragment)":1
 * def __reduce_cython__(self):             # <<<<<<<<<<<<<<
 *     cdef bint use_setstate
 *     state = ()
 */

  /* function exit code */
  __pyx_L1_error:;
  __Pyx_XDECREF(__pyx_t_1);
  __Pyx_XDECREF(__pyx_t_4);
  __Pyx_XDECREF(__pyx_t_5);
  __Pyx_AddTraceback("grpc._cython.cygrpc.ChannelCredentials.__reduce_cython__", __pyx_clineno, __pyx_lineno, __pyx_filename);
  __pyx_r = NULL;
  __pyx_L0:;
  __Pyx_XDECREF(__pyx_v_state);
  __Pyx_XDECREF(__pyx_v__dict);
  __Pyx_XGIVEREF(__pyx_r);
  __Pyx_RefNannyFinishContext();
  return __pyx_r;
}

/* "(tree fragment)":14
 *     else:
 *         return __pyx_unpickle_ChannelCredentials, (type(self), 0xd41d8cd, state)
 * def __setstate_cython__(self, __pyx_state):             # <<<<<<<<<<<<<<
 *     __pyx_unpickle_ChannelCredentials__set_state(self, __pyx_state)
 */

/* Python wrapper */
static PyObject *__pyx_pw_4grpc_7_cython_6cygrpc_18ChannelCredentials_3__setstate_cython__(PyObject *__pyx_v_self, PyObject *__pyx_v___pyx_state); /*proto*/
static PyObject *__pyx_pw_4grpc_7_cython_6cygrpc_18ChannelCredentials_3__setstate_cython__(PyObject *__pyx_v_self, PyObject *__pyx_v___pyx_state) {
  PyObject *__pyx_r = 0;
  __Pyx_RefNannyDeclarations
  __Pyx_RefNannySetupContext("__setstate_cython__ (wrapper)", 0);
  __pyx_r = __pyx_pf_4grpc_7_cython_6cygrpc_18ChannelCredentials_2__setstate_cython__(((struct __pyx_obj_4grpc_7_cython_6cygrpc_ChannelCredentials *)__pyx_v_self), ((PyObject *)__pyx_v___pyx_state));

  /* function exit code */
  __Pyx_RefNannyFinishContext();
  return __pyx_r;
}

static PyObject *__pyx_pf_4grpc_7_cython_6cygrpc_18ChannelCredentials_2__setstate_cython__(struct __pyx_obj_4grpc_7_cython_6cygrpc_ChannelCredentials *__pyx_v_self, PyObject *__pyx_v___pyx_state) {
  PyObject *__pyx_r = NULL;
  __Pyx_RefNannyDeclarations
  PyObject *__pyx_t_1 = NULL;
  __Pyx_RefNannySetupContext("__setstate_cython__", 0);

  /* "(tree fragment)":15
 *         return __pyx_unpickle_ChannelCredentials, (type(self), 0xd41d8cd, state)
 * def __setstate_cython__(self, __pyx_state):
 *     __pyx_unpickle_ChannelCredentials__set_state(self, __pyx_state)             # <<<<<<<<<<<<<<
 */
  if (!(likely(PyTuple_CheckExact(__pyx_v___pyx_state))||((__pyx_v___pyx_state) == Py_None)||(PyErr_Format(PyExc_TypeError, "Expected %.16s, got %.200s", "tuple", Py_TYPE(__pyx_v___pyx_state)->tp_name), 0))) __PYX_ERR(7, 15, __pyx_L1_error)
  __pyx_t_1 = __pyx_f_4grpc_7_cython_6cygrpc___pyx_unpickle_ChannelCredentials__set_state(__pyx_v_self, ((PyObject*)__pyx_v___pyx_state)); if (unlikely(!__pyx_t_1)) __PYX_ERR(7, 15, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_1);
  __Pyx_DECREF(__pyx_t_1); __pyx_t_1 = 0;

  /* "(tree fragment)":14
 *     else:
 *         return __pyx_unpickle_ChannelCredentials, (type(self), 0xd41d8cd, state)
 * def __setstate_cython__(self, __pyx_state):             # <<<<<<<<<<<<<<
 *     __pyx_unpickle_ChannelCredentials__set_state(self, __pyx_state)
 */

  /* function exit code */
  __pyx_r = Py_None; __Pyx_INCREF(Py_None);
  goto __pyx_L0;
  __pyx_L1_error:;
  __Pyx_XDECREF(__pyx_t_1);
  __Pyx_AddTraceback("grpc._cython.cygrpc.ChannelCredentials.__setstate_cython__", __pyx_clineno, __pyx_lineno, __pyx_filename);
  __pyx_r = NULL;
  __pyx_L0:;
  __Pyx_XGIVEREF(__pyx_r);
  __Pyx_RefNannyFinishContext();
  return __pyx_r;
}

/* "src/python/grpcio/grpc/_cython/_cygrpc/credentials.pyx.pxi":116
 * cdef class SSLSessionCacheLRU:
 * 
 *   def __cinit__(self, capacity):             # <<<<<<<<<<<<<<
 *     fork_handlers_and_grpc_init()
 *     self._cache = grpc_ssl_session_cache_create_lru(capacity)
 */

/* Python wrapper */
static int __pyx_pw_4grpc_7_cython_6cygrpc_18SSLSessionCacheLRU_1__cinit__(PyObject *__pyx_v_self, PyObject *__pyx_args, PyObject *__pyx_kwds); /*proto*/
static int __pyx_pw_4grpc_7_cython_6cygrpc_18SSLSessionCacheLRU_1__cinit__(PyObject *__pyx_v_self, PyObject *__pyx_args, PyObject *__pyx_kwds) {
  PyObject *__pyx_v_capacity = 0;
  int __pyx_r;
  __Pyx_RefNannyDeclarations
  __Pyx_RefNannySetupContext("__cinit__ (wrapper)", 0);
  {
    static PyObject **__pyx_pyargnames[] = {&__pyx_n_s_capacity,0};
    PyObject* values[1] = {0};
    if (unlikely(__pyx_kwds)) {
      Py_ssize_t kw_args;
      const Py_ssize_t pos_args = PyTuple_GET_SIZE(__pyx_args);
      switch (pos_args) {
        case  1: values[0] = PyTuple_GET_ITEM(__pyx_args, 0);
        CYTHON_FALLTHROUGH;
        case  0: break;
        default: goto __pyx_L5_argtuple_error;
      }
      kw_args = PyDict_Size(__pyx_kwds);
      switch (pos_args) {
        case  0:
        if (likely((values[0] = __Pyx_PyDict_GetItemStr(__pyx_kwds, __pyx_n_s_capacity)) != 0)) kw_args--;
        else goto __pyx_L5_argtuple_error;
      }
      if (unlikely(kw_args > 0)) {
        if (unlikely(__Pyx_ParseOptionalKeywords(__pyx_kwds, __pyx_pyargnames, 0, values, pos_args, "__cinit__") < 0)) __PYX_ERR(4, 116, __pyx_L3_error)
      }
    } else if (PyTuple_GET_SIZE(__pyx_args) != 1) {
      goto __pyx_L5_argtuple_error;
    } else {
      values[0] = PyTuple_GET_ITEM(__pyx_args, 0);
    }
    __pyx_v_capacity = values[0];
  }
  goto __pyx_L4_argument_unpacking_done;
  __pyx_L5_argtuple_error:;
  __Pyx_RaiseArgtupleInvalid("__cinit__", 1, 1, 1, PyTuple_GET_SIZE(__pyx_args)); __PYX_ERR(4, 116, __pyx_L3_error)
  __pyx_L3_error:;
  __Pyx_AddTraceback("grpc._cython.cygrpc.SSLSessionCacheLRU.__cinit__", __pyx_clineno, __pyx_lineno, __pyx_filename);
  __Pyx_RefNannyFinishContext();
  return -1;
  __pyx_L4_argument_unpacking_done:;
  __pyx_r = __pyx_pf_4grpc_7_cython_6cygrpc_18SSLSessionCacheLRU___cinit__(((struct __pyx_obj_4grpc_7_cython_6cygrpc_SSLSessionCacheLRU *)__pyx_v_self), __pyx_v_capacity);

  /* function exit code */
  __Pyx_RefNannyFinishContext();
  return __pyx_r;
}

static int __pyx_pf_4grpc_7_cython_6cygrpc_18SSLSessionCacheLRU___cinit__(struct __pyx_obj_4grpc_7_cython_6cygrpc_SSLSessionCacheLRU *__pyx_v_self, PyObject *__pyx_v_capacity) {
  int __pyx_r;
  __Pyx_RefNannyDeclarations
  PyObject *__pyx_t_1 = NULL;
  PyObject *__pyx_t_2 = NULL;
  PyObject *__pyx_t_3 = NULL;
  size_t __pyx_t_4;
  __Pyx_RefNannySetupContext("__cinit__", 0);

  /* "src/python/grpcio/grpc/_cython/_cygrpc/credentials.pyx.pxi":117
 * 
 *   def __cinit__(self, capacity):
 *     fork_handlers_and_grpc_init()             # <<<<<<<<<<<<<<
 *     self._cache = grpc_ssl_session_cache_create_lru(capacity)
 * 
 */
  __pyx_t_2 = __Pyx_GetModuleGlobalName(__pyx_n_s_fork_handlers_and_grpc_init); if (unlikely(!__pyx_t_2)) __PYX_ERR(4, 117, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_2);
  __pyx_t_3 = NULL;
  if (CYTHON_UNPACK_METHODS && unlikely(PyMethod_Check(__pyx_t_2))) {
    __pyx_t_3 = PyMethod_GET_SELF(__pyx_t_2);
    if (likely(__pyx_t_3)) {
      PyObject* function = PyMethod_GET_FUNCTION(__pyx_t_2);
      __Pyx_INCREF(__pyx_t_3);
      __Pyx_INCREF(function);
      __Pyx_DECREF_SET(__pyx_t_2, function);
    }
  }
  if (__pyx_t_3) {
    __pyx_t_1 = __Pyx_PyObject_CallOneArg(__pyx_t_2, __pyx_t_3); if (unlikely(!__pyx_t_1)) __PYX_ERR(4, 117, __pyx_L1_error)
    __Pyx_DECREF(__pyx_t_3); __pyx_t_3 = 0;
  } else {
    __pyx_t_1 = __Pyx_PyObject_CallNoArg(__pyx_t_2); if (unlikely(!__pyx_t_1)) __PYX_ERR(4, 117, __pyx_L1_error)
  }
  __Pyx_GOTREF(__pyx_t_1);
  __Pyx_DECREF(__pyx_t_2); __pyx_t_2 = 0;
  __Pyx_DECREF(__pyx_t_1); __pyx_t_1 = 0;

  /* "src/python/grpcio/grpc/_cython/_cygrpc/credentials.pyx.pxi":118
 *   def __cinit__(self, capacity):
 *     fork_handlers_and_grpc_init()
 *     self._cache = grpc_ssl_session_cache_create_lru(capacity)             # <<<<<<<<<<<<<<
 * 
 *   def __int__(self):
 */
  __pyx_t_4 = __Pyx_PyInt_As_size_t(__pyx_v_capacity); if (unlikely((__pyx_t_4 == (size_t)-1) && PyErr_Occurred())) __PYX_ERR(4, 118, __pyx_L1_error)
  __pyx_v_self->_cache = grpc_ssl_session_cache_create_lru(__pyx_t_4);

  /* "src/python/grpcio/grpc/_cython/_cygrpc/credentials.pyx.pxi":116
 * cdef class SSLSessionCacheLRU:
 * 
 *   def __cinit__(self, capacity):             # <<<<<<<<<<<<<<
 *     fork_handlers_and_grpc_init()
 *     self._cache = grpc_ssl_session_cache_create_lru(capacity)
 */

  /* function exit code */
  __pyx_r = 0;
  goto __pyx_L0;
  __pyx_L1_error:;
  __Pyx_XDECREF(__pyx_t_1);
  __Pyx_XDECREF(__pyx_t_2);
  __Pyx_XDECREF(__pyx_t_3);
  __Pyx_AddTraceback("grpc._cython.cygrpc.SSLSessionCacheLRU.__cinit__", __pyx_clineno, __pyx_lineno, __pyx_filename);
  __pyx_r = -1;
  __pyx_L0:;
  __Pyx_RefNannyFinishContext();
  return __pyx_r;
}

/* "src/python/grpcio/grpc/_cython/_cygrpc/credentials.pyx.pxi":120
 *     self._cache = grpc_ssl_session_cache_create_lru(capacity)
 * 
 *   def __int__(self):             # <<<<<<<<<<<<<<
 *     return <uintptr_t>self._cache
 * 
 */

/* Python wrapper */
static PyObject *__pyx_pw_4grpc_7_cython_6cygrpc_18SSLSessionCacheLRU_3__int__(PyObject *__pyx_v_self); /*proto*/
static PyObject *__pyx_pw_4grpc_7_cython_6cygrpc_18SSLSessionCacheLRU_3__int__(PyObject *__pyx_v_self) {
  PyObject *__pyx_r = 0;
  __Pyx_RefNannyDeclarations
  __Pyx_RefNannySetupContext("__int__ (wrapper)", 0);
  __pyx_r = __pyx_pf_4grpc_7_cython_6cygrpc_18SSLSessionCacheLRU_2__int__(((struct __pyx_obj_4grpc_7_cython_6cygrpc_SSLSessionCacheLRU *)__pyx_v_self));

  /* function exit code */
  __Pyx_RefNannyFinishContext();
  return __pyx_r;
}

static PyObject *__pyx_pf_4grpc_7_cython_6cygrpc_18SSLSessionCacheLRU_2__int__(struct __pyx_obj_4grpc_7_cython_6cygrpc_SSLSessionCacheLRU *__pyx_v_self) {
  PyObject *__pyx_r = NULL;
  __Pyx_RefNannyDeclarations
  PyObject *__pyx_t_1 = NULL;
  __Pyx_RefNannySetupContext("__int__", 0);

  /* "src/python/grpcio/grpc/_cython/_cygrpc/credentials.pyx.pxi":121
 * 
 *   def __int__(self):
 *     return <uintptr_t>self._cache             # <<<<<<<<<<<<<<
 * 
 *   def __dealloc__(self):
 */
  __Pyx_XDECREF(__pyx_r);
  __pyx_t_1 = __Pyx_PyInt_FromSize_t(((__pyx_t_4grpc_7_cython_6cygrpc_uintptr_t)__pyx_v_self->_cache)); if (unlikely(!__pyx_t_1)) __PYX_ERR(4, 121, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_1);
  __pyx_r = __pyx_t_1;
  __pyx_t_1 = 0;
  goto __pyx_L0;

  /* "src/python/grpcio/grpc/_cython/_cygrpc/credentials.pyx.pxi":120
 *     self._cache = grpc_ssl_session_cache_create_lru(capacity)
 * 
 *   def __int__(self):             # <<<<<<<<<<<<<<
 *     return <uintptr_t>self._cache
 * 
 */

  /* function exit code */
  __pyx_L1_error:;
  __Pyx_XDECREF(__pyx_t_1);
  __Pyx_AddTraceback("grpc._cython.cygrpc.SSLSessionCacheLRU.__int__", __pyx_clineno, __pyx_lineno, __pyx_filename);
  __pyx_r = NULL;
  __pyx_L0:;
  __Pyx_XGIVEREF(__pyx_r);
  __Pyx_RefNannyFinishContext();
  return __pyx_r;
}

/* "src/python/grpcio/grpc/_cython/_cygrpc/credentials.pyx.pxi":123
 *     return <uintptr_t>self._cache
 * 
 *   def __dealloc__(self):             # <<<<<<<<<<<<<<
 *     if self._cache != NULL:
 *         grpc_ssl_session_cache_destroy(self._cache)
 */

/* Python wrapper */
static void __pyx_pw_4grpc_7_cython_6cygrpc_18SSLSessionCacheLRU_5__dealloc__(PyObject *__pyx_v_self); /*proto*/
static void __pyx_pw_4grpc_7_cython_6cygrpc_18SSLSessionCacheLRU_5__dealloc__(PyObject *__pyx_v_self) {
  __Pyx_RefNannyDeclarations
  __Pyx_RefNannySetupContext("__dealloc__ (wrapper)", 0);
  __pyx_pf_4grpc_7_cython_6cygrpc_18SSLSessionCacheLRU_4__dealloc__(((struct __pyx_obj_4grpc_7_cython_6cygrpc_SSLSessionCacheLRU *)__pyx_v_self));

  /* function exit code */
  __Pyx_RefNannyFinishContext();
}

static void __pyx_pf_4grpc_7_cython_6cygrpc_18SSLSessionCacheLRU_4__dealloc__(struct __pyx_obj_4grpc_7_cython_6cygrpc_SSLSessionCacheLRU *__pyx_v_self) {
  __Pyx_RefNannyDeclarations
  int __pyx_t_1;
  __Pyx_RefNannySetupContext("__dealloc__", 0);

  /* "src/python/grpcio/grpc/_cython/_cygrpc/credentials.pyx.pxi":124
 * 
 *   def __dealloc__(self):
 *     if self._cache != NULL:             # <<<<<<<<<<<<<<
 *         grpc_ssl_session_cache_destroy(self._cache)
 *     grpc_shutdown_blocking()
 */
  __pyx_t_1 = ((__pyx_v_self->_cache != NULL) != 0);
  if (__pyx_t_1) {

    /* "src/python/grpcio/grpc/_cython/_cygrpc/credentials.pyx.pxi":125
 *   def __dealloc__(self):
 *     if self._cache != NULL:
 *         grpc_ssl_session_cache_destroy(self._cache)             # <<<<<<<<<<<<<<
 *     grpc_shutdown_blocking()
 * 
 */
    grpc_ssl_session_cache_destroy(__pyx_v_self->_cache);

    /* "src/python/grpcio/grpc/_cython/_cygrpc/credentials.pyx.pxi":124
 * 
 *   def __dealloc__(self):
 *     if self._cache != NULL:             # <<<<<<<<<<<<<<
 *         grpc_ssl_session_cache_destroy(self._cache)
 *     grpc_shutdown_blocking()
 */
  }

  /* "src/python/grpcio/grpc/_cython/_cygrpc/credentials.pyx.pxi":126
 *     if self._cache != NULL:
 *         grpc_ssl_session_cache_destroy(self._cache)
 *     grpc_shutdown_blocking()             # <<<<<<<<<<<<<<
 * 
 * 
 */
  grpc_shutdown_blocking();

  /* "src/python/grpcio/grpc/_cython/_cygrpc/credentials.pyx.pxi":123
 *     return <uintptr_t>self._cache
 * 
 *   def __dealloc__(self):             # <<<<<<<<<<<<<<
 *     if self._cache != NULL:
 *         grpc_ssl_session_cache_destroy(self._cache)
 */

  /* function exit code */
  __Pyx_RefNannyFinishContext();
}

/* "(tree fragment)":1
 * def __reduce_cython__(self):             # <<<<<<<<<<<<<<
 *     raise TypeError("no default __reduce__ due to non-trivial __cinit__")
 * def __setstate_cython__(self, __pyx_state):
 */

/* Python wrapper */
static PyObject *__pyx_pw_4grpc_7_cython_6cygrpc_18SSLSessionCacheLRU_7__reduce_cython__(PyObject *__pyx_v_self, CYTHON_UNUSED PyObject *unused); /*proto*/
static PyObject *__pyx_pw_4grpc_7_cython_6cygrpc_18SSLSessionCacheLRU_7__reduce_cython__(PyObject *__pyx_v_self, CYTHON_UNUSED PyObject *unused) {
  PyObject *__pyx_r = 0;
  __Pyx_RefNannyDeclarations
  __Pyx_RefNannySetupContext("__reduce_cython__ (wrapper)", 0);
  __pyx_r = __pyx_pf_4grpc_7_cython_6cygrpc_18SSLSessionCacheLRU_6__reduce_cython__(((struct __pyx_obj_4grpc_7_cython_6cygrpc_SSLSessionCacheLRU *)__pyx_v_self));

  /* function exit code */
  __Pyx_RefNannyFinishContext();
  return __pyx_r;
}

static PyObject *__pyx_pf_4grpc_7_cython_6cygrpc_18SSLSessionCacheLRU_6__reduce_cython__(CYTHON_UNUSED struct __pyx_obj_4grpc_7_cython_6cygrpc_SSLSessionCacheLRU *__pyx_v_self) {
  PyObject *__pyx_r = NULL;
  __Pyx_RefNannyDeclarations
  PyObject *__pyx_t_1 = NULL;
  __Pyx_RefNannySetupContext("__reduce_cython__", 0);

  /* "(tree fragment)":2
 * def __reduce_cython__(self):
 *     raise TypeError("no default __reduce__ due to non-trivial __cinit__")             # <<<<<<<<<<<<<<
 * def __setstate_cython__(self, __pyx_state):
 *     raise TypeError("no default __reduce__ due to non-trivial __cinit__")
 */
  __pyx_t_1 = __Pyx_PyObject_Call(__pyx_builtin_TypeError, __pyx_tuple__58, NULL); if (unlikely(!__pyx_t_1)) __PYX_ERR(7, 2, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_1);
  __Pyx_Raise(__pyx_t_1, 0, 0, 0);
  __Pyx_DECREF(__pyx_t_1); __pyx_t_1 = 0;
  __PYX_ERR(7, 2, __pyx_L1_error)

  /* "(tree fragment)":1
 * def __reduce_cython__(self):             # <<<<<<<<<<<<<<
 *     raise TypeError("no default __reduce__ due to non-trivial __cinit__")
 * def __setstate_cython__(self, __pyx_state):
 */

  /* function exit code */
  __pyx_L1_error:;
  __Pyx_XDECREF(__pyx_t_1);
  __Pyx_AddTraceback("grpc._cython.cygrpc.SSLSessionCacheLRU.__reduce_cython__", __pyx_clineno, __pyx_lineno, __pyx_filename);
  __pyx_r = NULL;
  __Pyx_XGIVEREF(__pyx_r);
  __Pyx_RefNannyFinishContext();
  return __pyx_r;
}

/* "(tree fragment)":3
 * def __reduce_cython__(self):
 *     raise TypeError("no default __reduce__ due to non-trivial __cinit__")
 * def __setstate_cython__(self, __pyx_state):             # <<<<<<<<<<<<<<
 *     raise TypeError("no default __reduce__ due to non-trivial __cinit__")
 */

/* Python wrapper */
static PyObject *__pyx_pw_4grpc_7_cython_6cygrpc_18SSLSessionCacheLRU_9__setstate_cython__(PyObject *__pyx_v_self, PyObject *__pyx_v___pyx_state); /*proto*/
static PyObject *__pyx_pw_4grpc_7_cython_6cygrpc_18SSLSessionCacheLRU_9__setstate_cython__(PyObject *__pyx_v_self, PyObject *__pyx_v___pyx_state) {
  PyObject *__pyx_r = 0;
  __Pyx_RefNannyDeclarations
  __Pyx_RefNannySetupContext("__setstate_cython__ (wrapper)", 0);
  __pyx_r = __pyx_pf_4grpc_7_cython_6cygrpc_18SSLSessionCacheLRU_8__setstate_cython__(((struct __pyx_obj_4grpc_7_cython_6cygrpc_SSLSessionCacheLRU *)__pyx_v_self), ((PyObject *)__pyx_v___pyx_state));

  /* function exit code */
  __Pyx_RefNannyFinishContext();
  return __pyx_r;
}

static PyObject *__pyx_pf_4grpc_7_cython_6cygrpc_18SSLSessionCacheLRU_8__setstate_cython__(CYTHON_UNUSED struct __pyx_obj_4grpc_7_cython_6cygrpc_SSLSessionCacheLRU *__pyx_v_self, CYTHON_UNUSED PyObject *__pyx_v___pyx_state) {
  PyObject *__pyx_r = NULL;
  __Pyx_RefNannyDeclarations
  PyObject *__pyx_t_1 = NULL;
  __Pyx_RefNannySetupContext("__setstate_cython__", 0);

  /* "(tree fragment)":4
 *     raise TypeError("no default __reduce__ due to non-trivial __cinit__")
 * def __setstate_cython__(self, __pyx_state):
 *     raise TypeError("no default __reduce__ due to non-trivial __cinit__")             # <<<<<<<<<<<<<<
 */
  __pyx_t_1 = __Pyx_PyObject_Call(__pyx_builtin_TypeError, __pyx_tuple__59, NULL); if (unlikely(!__pyx_t_1)) __PYX_ERR(7, 4, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_1);
  __Pyx_Raise(__pyx_t_1, 0, 0, 0);
  __Pyx_DECREF(__pyx_t_1); __pyx_t_1 = 0;
  __PYX_ERR(7, 4, __pyx_L1_error)

  /* "(tree fragment)":3
 * def __reduce_cython__(self):
 *     raise TypeError("no default __reduce__ due to non-trivial __cinit__")
 * def __setstate_cython__(self, __pyx_state):             # <<<<<<<<<<<<<<
 *     raise TypeError("no default __reduce__ due to non-trivial __cinit__")
 */

  /* function exit code */
  __pyx_L1_error:;
  __Pyx_XDECREF(__pyx_t_1);
  __Pyx_AddTraceback("grpc._cython.cygrpc.SSLSessionCacheLRU.__setstate_cython__", __pyx_clineno, __pyx_lineno, __pyx_filename);
  __pyx_r = NULL;
  __Pyx_XGIVEREF(__pyx_r);
  __Pyx_RefNannyFinishContext();
  return __pyx_r;
}

/* "src/python/grpcio/grpc/_cython/_cygrpc/credentials.pyx.pxi":131
 * cdef class SSLChannelCredentials(ChannelCredentials):
 * 
 *   def __cinit__(self, pem_root_certificates, private_key, certificate_chain):             # <<<<<<<<<<<<<<
 *     if pem_root_certificates is not None and not isinstance(pem_root_certificates, bytes):
 *       raise TypeError('expected certificate to be bytes, got %s' % (type(pem_root_certificates)))
 */

/* Python wrapper */
static int __pyx_pw_4grpc_7_cython_6cygrpc_21SSLChannelCredentials_1__cinit__(PyObject *__pyx_v_self, PyObject *__pyx_args, PyObject *__pyx_kwds); /*proto*/
static int __pyx_pw_4grpc_7_cython_6cygrpc_21SSLChannelCredentials_1__cinit__(PyObject *__pyx_v_self, PyObject *__pyx_args, PyObject *__pyx_kwds) {
  PyObject *__pyx_v_pem_root_certificates = 0;
  PyObject *__pyx_v_private_key = 0;
  PyObject *__pyx_v_certificate_chain = 0;
  int __pyx_r;
  __Pyx_RefNannyDeclarations
  __Pyx_RefNannySetupContext("__cinit__ (wrapper)", 0);
  {
    static PyObject **__pyx_pyargnames[] = {&__pyx_n_s_pem_root_certificates,&__pyx_n_s_private_key,&__pyx_n_s_certificate_chain,0};
    PyObject* values[3] = {0,0,0};
    if (unlikely(__pyx_kwds)) {
      Py_ssize_t kw_args;
      const Py_ssize_t pos_args = PyTuple_GET_SIZE(__pyx_args);
      switch (pos_args) {
        case  3: values[2] = PyTuple_GET_ITEM(__pyx_args, 2);
        CYTHON_FALLTHROUGH;
        case  2: values[1] = PyTuple_GET_ITEM(__pyx_args, 1);
        CYTHON_FALLTHROUGH;
        case  1: values[0] = PyTuple_GET_ITEM(__pyx_args, 0);
        CYTHON_FALLTHROUGH;
        case  0: break;
        default: goto __pyx_L5_argtuple_error;
      }
      kw_args = PyDict_Size(__pyx_kwds);
      switch (pos_args) {
        case  0:
        if (likely((values[0] = __Pyx_PyDict_GetItemStr(__pyx_kwds, __pyx_n_s_pem_root_certificates)) != 0)) kw_args--;
        else goto __pyx_L5_argtuple_error;
        CYTHON_FALLTHROUGH;
        case  1:
        if (likely((values[1] = __Pyx_PyDict_GetItemStr(__pyx_kwds, __pyx_n_s_private_key)) != 0)) kw_args--;
        else {
          __Pyx_RaiseArgtupleInvalid("__cinit__", 1, 3, 3, 1); __PYX_ERR(4, 131, __pyx_L3_error)
        }
        CYTHON_FALLTHROUGH;
        case  2:
        if (likely((values[2] = __Pyx_PyDict_GetItemStr(__pyx_kwds, __pyx_n_s_certificate_chain)) != 0)) kw_args--;
        else {
          __Pyx_RaiseArgtupleInvalid("__cinit__", 1, 3, 3, 2); __PYX_ERR(4, 131, __pyx_L3_error)
        }
      }
      if (unlikely(kw_args > 0)) {
        if (unlikely(__Pyx_ParseOptionalKeywords(__pyx_kwds, __pyx_pyargnames, 0, values, pos_args, "__cinit__") < 0)) __PYX_ERR(4, 131, __pyx_L3_error)
      }
    } else if (PyTuple_GET_SIZE(__pyx_args) != 3) {
      goto __pyx_L5_argtuple_error;
    } else {
      values[0] = PyTuple_GET_ITEM(__pyx_args, 0);
      values[1] = PyTuple_GET_ITEM(__pyx_args, 1);
      values[2] = PyTuple_GET_ITEM(__pyx_args, 2);
    }
    __pyx_v_pem_root_certificates = values[0];
    __pyx_v_private_key = values[1];
    __pyx_v_certificate_chain = values[2];
  }
  goto __pyx_L4_argument_unpacking_done;
  __pyx_L5_argtuple_error:;
  __Pyx_RaiseArgtupleInvalid("__cinit__", 1, 3, 3, PyTuple_GET_SIZE(__pyx_args)); __PYX_ERR(4, 131, __pyx_L3_error)
  __pyx_L3_error:;
  __Pyx_AddTraceback("grpc._cython.cygrpc.SSLChannelCredentials.__cinit__", __pyx_clineno, __pyx_lineno, __pyx_filename);
  __Pyx_RefNannyFinishContext();
  return -1;
  __pyx_L4_argument_unpacking_done:;
  __pyx_r = __pyx_pf_4grpc_7_cython_6cygrpc_21SSLChannelCredentials___cinit__(((struct __pyx_obj_4grpc_7_cython_6cygrpc_SSLChannelCredentials *)__pyx_v_self), __pyx_v_pem_root_certificates, __pyx_v_private_key, __pyx_v_certificate_chain);

  /* function exit code */
  __Pyx_RefNannyFinishContext();
  return __pyx_r;
}

static int __pyx_pf_4grpc_7_cython_6cygrpc_21SSLChannelCredentials___cinit__(struct __pyx_obj_4grpc_7_cython_6cygrpc_SSLChannelCredentials *__pyx_v_self, PyObject *__pyx_v_pem_root_certificates, PyObject *__pyx_v_private_key, PyObject *__pyx_v_certificate_chain) {
  int __pyx_r;
  __Pyx_RefNannyDeclarations
  int __pyx_t_1;
  int __pyx_t_2;
  int __pyx_t_3;
  PyObject *__pyx_t_4 = NULL;
  PyObject *__pyx_t_5 = NULL;
  __Pyx_RefNannySetupContext("__cinit__", 0);

  /* "src/python/grpcio/grpc/_cython/_cygrpc/credentials.pyx.pxi":132
 * 
 *   def __cinit__(self, pem_root_certificates, private_key, certificate_chain):
 *     if pem_root_certificates is not None and not isinstance(pem_root_certificates, bytes):             # <<<<<<<<<<<<<<
 *       raise TypeError('expected certificate to be bytes, got %s' % (type(pem_root_certificates)))
 *     self._pem_root_certificates = pem_root_certificates
 */
  __pyx_t_2 = (__pyx_v_pem_root_certificates != Py_None);
  __pyx_t_3 = (__pyx_t_2 != 0);
  if (__pyx_t_3) {
  } else {
    __pyx_t_1 = __pyx_t_3;
    goto __pyx_L4_bool_binop_done;
  }
  __pyx_t_3 = PyBytes_Check(__pyx_v_pem_root_certificates); 
  __pyx_t_2 = ((!(__pyx_t_3 != 0)) != 0);
  __pyx_t_1 = __pyx_t_2;
  __pyx_L4_bool_binop_done:;
  if (unlikely(__pyx_t_1)) {

    /* "src/python/grpcio/grpc/_cython/_cygrpc/credentials.pyx.pxi":133
 *   def __cinit__(self, pem_root_certificates, private_key, certificate_chain):
 *     if pem_root_certificates is not None and not isinstance(pem_root_certificates, bytes):
 *       raise TypeError('expected certificate to be bytes, got %s' % (type(pem_root_certificates)))             # <<<<<<<<<<<<<<
 *     self._pem_root_certificates = pem_root_certificates
 *     self._private_key = private_key
 */
    __pyx_t_4 = __Pyx_PyString_Format(__pyx_kp_s_expected_certificate_to_be_bytes, ((PyObject *)Py_TYPE(__pyx_v_pem_root_certificates))); if (unlikely(!__pyx_t_4)) __PYX_ERR(4, 133, __pyx_L1_error)
    __Pyx_GOTREF(__pyx_t_4);
    __pyx_t_5 = __Pyx_PyObject_CallOneArg(__pyx_builtin_TypeError, __pyx_t_4); if (unlikely(!__pyx_t_5)) __PYX_ERR(4, 133, __pyx_L1_error)
    __Pyx_GOTREF(__pyx_t_5);
    __Pyx_DECREF(__pyx_t_4); __pyx_t_4 = 0;
    __Pyx_Raise(__pyx_t_5, 0, 0, 0);
    __Pyx_DECREF(__pyx_t_5); __pyx_t_5 = 0;
    __PYX_ERR(4, 133, __pyx_L1_error)

    /* "src/python/grpcio/grpc/_cython/_cygrpc/credentials.pyx.pxi":132
 * 
 *   def __cinit__(self, pem_root_certificates, private_key, certificate_chain):
 *     if pem_root_certificates is not None and not isinstance(pem_root_certificates, bytes):             # <<<<<<<<<<<<<<
 *       raise TypeError('expected certificate to be bytes, got %s' % (type(pem_root_certificates)))
 *     self._pem_root_certificates = pem_root_certificates
 */
  }

  /* "src/python/grpcio/grpc/_cython/_cygrpc/credentials.pyx.pxi":134
 *     if pem_root_certificates is not None and not isinstance(pem_root_certificates, bytes):
 *       raise TypeError('expected certificate to be bytes, got %s' % (type(pem_root_certificates)))
 *     self._pem_root_certificates = pem_root_certificates             # <<<<<<<<<<<<<<
 *     self._private_key = private_key
 *     self._certificate_chain = certificate_chain
 */
  __Pyx_INCREF(__pyx_v_pem_root_certificates);
  __Pyx_GIVEREF(__pyx_v_pem_root_certificates);
  __Pyx_GOTREF(__pyx_v_self->_pem_root_certificates);
  __Pyx_DECREF(__pyx_v_self->_pem_root_certificates);
  __pyx_v_self->_pem_root_certificates = __pyx_v_pem_root_certificates;

  /* "src/python/grpcio/grpc/_cython/_cygrpc/credentials.pyx.pxi":135
 *       raise TypeError('expected certificate to be bytes, got %s' % (type(pem_root_certificates)))
 *     self._pem_root_certificates = pem_root_certificates
 *     self._private_key = private_key             # <<<<<<<<<<<<<<
 *     self._certificate_chain = certificate_chain
 * 
 */
  __Pyx_INCREF(__pyx_v_private_key);
  __Pyx_GIVEREF(__pyx_v_private_key);
  __Pyx_GOTREF(__pyx_v_self->_private_key);
  __Pyx_DECREF(__pyx_v_self->_private_key);
  __pyx_v_self->_private_key = __pyx_v_private_key;

  /* "src/python/grpcio/grpc/_cython/_cygrpc/credentials.pyx.pxi":136
 *     self._pem_root_certificates = pem_root_certificates
 *     self._private_key = private_key
 *     self._certificate_chain = certificate_chain             # <<<<<<<<<<<<<<
 * 
 *   cdef grpc_channel_credentials *c(self) except *:
 */
  __Pyx_INCREF(__pyx_v_certificate_chain);
  __Pyx_GIVEREF(__pyx_v_certificate_chain);
  __Pyx_GOTREF(__pyx_v_self->_certificate_chain);
  __Pyx_DECREF(__pyx_v_self->_certificate_chain);
  __pyx_v_self->_certificate_chain = __pyx_v_certificate_chain;

  /* "src/python/grpcio/grpc/_cython/_cygrpc/credentials.pyx.pxi":131
 * cdef class SSLChannelCredentials(ChannelCredentials):
 * 
 *   def __cinit__(self, pem_root_certificates, private_key, certificate_chain):             # <<<<<<<<<<<<<<
 *     if pem_root_certificates is not None and not isinstance(pem_root_certificates, bytes):
 *       raise TypeError('expected certificate to be bytes, got %s' % (type(pem_root_certificates)))
 */

  /* function exit code */
  __pyx_r = 0;
  goto __pyx_L0;
  __pyx_L1_error:;
  __Pyx_XDECREF(__pyx_t_4);
  __Pyx_XDECREF(__pyx_t_5);
  __Pyx_AddTraceback("grpc._cython.cygrpc.SSLChannelCredentials.__cinit__", __pyx_clineno, __pyx_lineno, __pyx_filename);
  __pyx_r = -1;
  __pyx_L0:;
  __Pyx_RefNannyFinishContext();
  return __pyx_r;
}

/* "src/python/grpcio/grpc/_cython/_cygrpc/credentials.pyx.pxi":138
 *     self._certificate_chain = certificate_chain
 * 
 *   cdef grpc_channel_credentials *c(self) except *:             # <<<<<<<<<<<<<<
 *     cdef const char *c_pem_root_certificates
 *     cdef grpc_ssl_pem_key_cert_pair c_pem_key_certificate_pair
 */

static grpc_channel_credentials *__pyx_f_4grpc_7_cython_6cygrpc_21SSLChannelCredentials_c(struct __pyx_obj_4grpc_7_cython_6cygrpc_SSLChannelCredentials *__pyx_v_self) {
  char const *__pyx_v_c_pem_root_certificates;
  grpc_ssl_pem_key_cert_pair __pyx_v_c_pem_key_certificate_pair;
  grpc_channel_credentials *__pyx_r;
  __Pyx_RefNannyDeclarations
  int __pyx_t_1;
  int __pyx_t_2;
  char const *__pyx_t_3;
  int __pyx_t_4;
  char const *__pyx_t_5;
  char const *__pyx_t_6;
  __Pyx_RefNannySetupContext("c", 0);

  /* "src/python/grpcio/grpc/_cython/_cygrpc/credentials.pyx.pxi":141
 *     cdef const char *c_pem_root_certificates
 *     cdef grpc_ssl_pem_key_cert_pair c_pem_key_certificate_pair
 *     if self._pem_root_certificates is None:             # <<<<<<<<<<<<<<
 *       c_pem_root_certificates = NULL
 *     else:
 */
  __pyx_t_1 = (__pyx_v_self->_pem_root_certificates == Py_None);
  __pyx_t_2 = (__pyx_t_1 != 0);
  if (__pyx_t_2) {

    /* "src/python/grpcio/grpc/_cython/_cygrpc/credentials.pyx.pxi":142
 *     cdef grpc_ssl_pem_key_cert_pair c_pem_key_certificate_pair
 *     if self._pem_root_certificates is None:
 *       c_pem_root_certificates = NULL             # <<<<<<<<<<<<<<
 *     else:
 *       c_pem_root_certificates = self._pem_root_certificates
 */
    __pyx_v_c_pem_root_certificates = NULL;

    /* "src/python/grpcio/grpc/_cython/_cygrpc/credentials.pyx.pxi":141
 *     cdef const char *c_pem_root_certificates
 *     cdef grpc_ssl_pem_key_cert_pair c_pem_key_certificate_pair
 *     if self._pem_root_certificates is None:             # <<<<<<<<<<<<<<
 *       c_pem_root_certificates = NULL
 *     else:
 */
    goto __pyx_L3;
  }

  /* "src/python/grpcio/grpc/_cython/_cygrpc/credentials.pyx.pxi":144
 *       c_pem_root_certificates = NULL
 *     else:
 *       c_pem_root_certificates = self._pem_root_certificates             # <<<<<<<<<<<<<<
 *     if self._private_key is None and self._certificate_chain is None:
 *       return grpc_ssl_credentials_create(
 */
  /*else*/ {
    __pyx_t_3 = __Pyx_PyObject_AsString(__pyx_v_self->_pem_root_certificates); if (unlikely((!__pyx_t_3) && PyErr_Occurred())) __PYX_ERR(4, 144, __pyx_L1_error)
    __pyx_v_c_pem_root_certificates = __pyx_t_3;
  }
  __pyx_L3:;

  /* "src/python/grpcio/grpc/_cython/_cygrpc/credentials.pyx.pxi":145
 *     else:
 *       c_pem_root_certificates = self._pem_root_certificates
 *     if self._private_key is None and self._certificate_chain is None:             # <<<<<<<<<<<<<<
 *       return grpc_ssl_credentials_create(
 *           c_pem_root_certificates, NULL, NULL, NULL)
 */
  __pyx_t_1 = (__pyx_v_self->_private_key == Py_None);
  __pyx_t_4 = (__pyx_t_1 != 0);
  if (__pyx_t_4) {
  } else {
    __pyx_t_2 = __pyx_t_4;
    goto __pyx_L5_bool_binop_done;
  }
  __pyx_t_4 = (__pyx_v_self->_certificate_chain == Py_None);
  __pyx_t_1 = (__pyx_t_4 != 0);
  __pyx_t_2 = __pyx_t_1;
  __pyx_L5_bool_binop_done:;
  if (__pyx_t_2) {

    /* "src/python/grpcio/grpc/_cython/_cygrpc/credentials.pyx.pxi":146
 *       c_pem_root_certificates = self._pem_root_certificates
 *     if self._private_key is None and self._certificate_chain is None:
 *       return grpc_ssl_credentials_create(             # <<<<<<<<<<<<<<
 *           c_pem_root_certificates, NULL, NULL, NULL)
 *     else:
 */
    __pyx_r = grpc_ssl_credentials_create(__pyx_v_c_pem_root_certificates, NULL, NULL, NULL);
    goto __pyx_L0;

    /* "src/python/grpcio/grpc/_cython/_cygrpc/credentials.pyx.pxi":145
 *     else:
 *       c_pem_root_certificates = self._pem_root_certificates
 *     if self._private_key is None and self._certificate_chain is None:             # <<<<<<<<<<<<<<
 *       return grpc_ssl_credentials_create(
 *           c_pem_root_certificates, NULL, NULL, NULL)
 */
  }

  /* "src/python/grpcio/grpc/_cython/_cygrpc/credentials.pyx.pxi":149
 *           c_pem_root_certificates, NULL, NULL, NULL)
 *     else:
 *       if self._private_key:             # <<<<<<<<<<<<<<
 *         c_pem_key_certificate_pair.private_key = self._private_key
 *       else:
 */
  /*else*/ {
    __pyx_t_2 = __Pyx_PyObject_IsTrue(__pyx_v_self->_private_key); if (unlikely(__pyx_t_2 < 0)) __PYX_ERR(4, 149, __pyx_L1_error)
    if (__pyx_t_2) {

      /* "src/python/grpcio/grpc/_cython/_cygrpc/credentials.pyx.pxi":150
 *     else:
 *       if self._private_key:
 *         c_pem_key_certificate_pair.private_key = self._private_key             # <<<<<<<<<<<<<<
 *       else:
 *         c_pem_key_certificate_pair.private_key = NULL
 */
      __pyx_t_5 = __Pyx_PyObject_AsString(__pyx_v_self->_private_key); if (unlikely((!__pyx_t_5) && PyErr_Occurred())) __PYX_ERR(4, 150, __pyx_L1_error)
      __pyx_v_c_pem_key_certificate_pair.private_key = __pyx_t_5;

      /* "src/python/grpcio/grpc/_cython/_cygrpc/credentials.pyx.pxi":149
 *           c_pem_root_certificates, NULL, NULL, NULL)
 *     else:
 *       if self._private_key:             # <<<<<<<<<<<<<<
 *         c_pem_key_certificate_pair.private_key = self._private_key
 *       else:
 */
      goto __pyx_L7;
    }

    /* "src/python/grpcio/grpc/_cython/_cygrpc/credentials.pyx.pxi":152
 *         c_pem_key_certificate_pair.private_key = self._private_key
 *       else:
 *         c_pem_key_certificate_pair.private_key = NULL             # <<<<<<<<<<<<<<
 *       if self._certificate_chain:
 *         c_pem_key_certificate_pair.certificate_chain = self._certificate_chain
 */
    /*else*/ {
      __pyx_v_c_pem_key_certificate_pair.private_key = NULL;
    }
    __pyx_L7:;

    /* "src/python/grpcio/grpc/_cython/_cygrpc/credentials.pyx.pxi":153
 *       else:
 *         c_pem_key_certificate_pair.private_key = NULL
 *       if self._certificate_chain:             # <<<<<<<<<<<<<<
 *         c_pem_key_certificate_pair.certificate_chain = self._certificate_chain
 *       else:
 */
    __pyx_t_2 = __Pyx_PyObject_IsTrue(__pyx_v_self->_certificate_chain); if (unlikely(__pyx_t_2 < 0)) __PYX_ERR(4, 153, __pyx_L1_error)
    if (__pyx_t_2) {

      /* "src/python/grpcio/grpc/_cython/_cygrpc/credentials.pyx.pxi":154
 *         c_pem_key_certificate_pair.private_key = NULL
 *       if self._certificate_chain:
 *         c_pem_key_certificate_pair.certificate_chain = self._certificate_chain             # <<<<<<<<<<<<<<
 *       else:
 *         c_pem_key_certificate_pair.certificate_chain = NULL
 */
      __pyx_t_6 = __Pyx_PyObject_AsString(__pyx_v_self->_certificate_chain); if (unlikely((!__pyx_t_6) && PyErr_Occurred())) __PYX_ERR(4, 154, __pyx_L1_error)
      __pyx_v_c_pem_key_certificate_pair.cert_chain = __pyx_t_6;

      /* "src/python/grpcio/grpc/_cython/_cygrpc/credentials.pyx.pxi":153
 *       else:
 *         c_pem_key_certificate_pair.private_key = NULL
 *       if self._certificate_chain:             # <<<<<<<<<<<<<<
 *         c_pem_key_certificate_pair.certificate_chain = self._certificate_chain
 *       else:
 */
      goto __pyx_L8;
    }

    /* "src/python/grpcio/grpc/_cython/_cygrpc/credentials.pyx.pxi":156
 *         c_pem_key_certificate_pair.certificate_chain = self._certificate_chain
 *       else:
 *         c_pem_key_certificate_pair.certificate_chain = NULL             # <<<<<<<<<<<<<<
 *       return grpc_ssl_credentials_create(
 *           c_pem_root_certificates, &c_pem_key_certificate_pair, NULL, NULL)
 */
    /*else*/ {
      __pyx_v_c_pem_key_certificate_pair.cert_chain = NULL;
    }
    __pyx_L8:;

    /* "src/python/grpcio/grpc/_cython/_cygrpc/credentials.pyx.pxi":157
 *       else:
 *         c_pem_key_certificate_pair.certificate_chain = NULL
 *       return grpc_ssl_credentials_create(             # <<<<<<<<<<<<<<
 *           c_pem_root_certificates, &c_pem_key_certificate_pair, NULL, NULL)
 * 
 */
    __pyx_r = grpc_ssl_credentials_create(__pyx_v_c_pem_root_certificates, (&__pyx_v_c_pem_key_certificate_pair), NULL, NULL);
    goto __pyx_L0;
  }

  /* "src/python/grpcio/grpc/_cython/_cygrpc/credentials.pyx.pxi":138
 *     self._certificate_chain = certificate_chain
 * 
 *   cdef grpc_channel_credentials *c(self) except *:             # <<<<<<<<<<<<<<
 *     cdef const char *c_pem_root_certificates
 *     cdef grpc_ssl_pem_key_cert_pair c_pem_key_certificate_pair
 */

  /* function exit code */
  __pyx_L1_error:;
  __Pyx_AddTraceback("grpc._cython.cygrpc.SSLChannelCredentials.c", __pyx_clineno, __pyx_lineno, __pyx_filename);
  __pyx_r = 0;
  __pyx_L0:;
  __Pyx_RefNannyFinishContext();
  return __pyx_r;
}

/* "src/python/grpcio/grpc/_cython/_cygrpc/credentials.pxd.pxi":64
 * cdef class SSLChannelCredentials(ChannelCredentials):
 * 
 *   cdef readonly object _pem_root_certificates             # <<<<<<<<<<<<<<
 *   cdef readonly object _private_key
 *   cdef readonly object _certificate_chain
 */

/* Python wrapper */
static PyObject *__pyx_pw_4grpc_7_cython_6cygrpc_21SSLChannelCredentials_22_pem_root_certificates_1__get__(PyObject *__pyx_v_self); /*proto*/
static PyObject *__pyx_pw_4grpc_7_cython_6cygrpc_21SSLChannelCredentials_22_pem_root_certificates_1__get__(PyObject *__pyx_v_self) {
  PyObject *__pyx_r = 0;
  __Pyx_RefNannyDeclarations
  __Pyx_RefNannySetupContext("__get__ (wrapper)", 0);
  __pyx_r = __pyx_pf_4grpc_7_cython_6cygrpc_21SSLChannelCredentials_22_pem_root_certificates___get__(((struct __pyx_obj_4grpc_7_cython_6cygrpc_SSLChannelCredentials *)__pyx_v_self));

  /* function exit code */
  __Pyx_RefNannyFinishContext();
  return __pyx_r;
}

static PyObject *__pyx_pf_4grpc_7_cython_6cygrpc_21SSLChannelCredentials_22_pem_root_certificates___get__(struct __pyx_obj_4grpc_7_cython_6cygrpc_SSLChannelCredentials *__pyx_v_self) {
  PyObject *__pyx_r = NULL;
  __Pyx_RefNannyDeclarations
  __Pyx_RefNannySetupContext("__get__", 0);
  __Pyx_XDECREF(__pyx_r);
  __Pyx_INCREF(__pyx_v_self->_pem_root_certificates);
  __pyx_r = __pyx_v_self->_pem_root_certificates;
  goto __pyx_L0;

  /* function exit code */
  __pyx_L0:;
  __Pyx_XGIVEREF(__pyx_r);
  __Pyx_RefNannyFinishContext();
  return __pyx_r;
}

/* "src/python/grpcio/grpc/_cython/_cygrpc/credentials.pxd.pxi":65
 * 
 *   cdef readonly object _pem_root_certificates
 *   cdef readonly object _private_key             # <<<<<<<<<<<<<<
 *   cdef readonly object _certificate_chain
 * 
 */

/* Python wrapper */
static PyObject *__pyx_pw_4grpc_7_cython_6cygrpc_21SSLChannelCredentials_12_private_key_1__get__(PyObject *__pyx_v_self); /*proto*/
static PyObject *__pyx_pw_4grpc_7_cython_6cygrpc_21SSLChannelCredentials_12_private_key_1__get__(PyObject *__pyx_v_self) {
  PyObject *__pyx_r = 0;
  __Pyx_RefNannyDeclarations
  __Pyx_RefNannySetupContext("__get__ (wrapper)", 0);
  __pyx_r = __pyx_pf_4grpc_7_cython_6cygrpc_21SSLChannelCredentials_12_private_key___get__(((struct __pyx_obj_4grpc_7_cython_6cygrpc_SSLChannelCredentials *)__pyx_v_self));

  /* function exit code */
  __Pyx_RefNannyFinishContext();
  return __pyx_r;
}

static PyObject *__pyx_pf_4grpc_7_cython_6cygrpc_21SSLChannelCredentials_12_private_key___get__(struct __pyx_obj_4grpc_7_cython_6cygrpc_SSLChannelCredentials *__pyx_v_self) {
  PyObject *__pyx_r = NULL;
  __Pyx_RefNannyDeclarations
  __Pyx_RefNannySetupContext("__get__", 0);
  __Pyx_XDECREF(__pyx_r);
  __Pyx_INCREF(__pyx_v_self->_private_key);
  __pyx_r = __pyx_v_self->_private_key;
  goto __pyx_L0;

  /* function exit code */
  __pyx_L0:;
  __Pyx_XGIVEREF(__pyx_r);
  __Pyx_RefNannyFinishContext();
  return __pyx_r;
}

/* "src/python/grpcio/grpc/_cython/_cygrpc/credentials.pxd.pxi":66
 *   cdef readonly object _pem_root_certificates
 *   cdef readonly object _private_key
 *   cdef readonly object _certificate_chain             # <<<<<<<<<<<<<<
 * 
 *   cdef grpc_channel_credentials *c(self) except *
 */

/* Python wrapper */
static PyObject *__pyx_pw_4grpc_7_cython_6cygrpc_21SSLChannelCredentials_18_certificate_chain_1__get__(PyObject *__pyx_v_self); /*proto*/
static PyObject *__pyx_pw_4grpc_7_cython_6cygrpc_21SSLChannelCredentials_18_certificate_chain_1__get__(PyObject *__pyx_v_self) {
  PyObject *__pyx_r = 0;
  __Pyx_RefNannyDeclarations
  __Pyx_RefNannySetupContext("__get__ (wrapper)", 0);
  __pyx_r = __pyx_pf_4grpc_7_cython_6cygrpc_21SSLChannelCredentials_18_certificate_chain___get__(((struct __pyx_obj_4grpc_7_cython_6cygrpc_SSLChannelCredentials *)__pyx_v_self));

  /* function exit code */
  __Pyx_RefNannyFinishContext();
  return __pyx_r;
}

static PyObject *__pyx_pf_4grpc_7_cython_6cygrpc_21SSLChannelCredentials_18_certificate_chain___get__(struct __pyx_obj_4grpc_7_cython_6cygrpc_SSLChannelCredentials *__pyx_v_self) {
  PyObject *__pyx_r = NULL;
  __Pyx_RefNannyDeclarations
  __Pyx_RefNannySetupContext("__get__", 0);
  __Pyx_XDECREF(__pyx_r);
  __Pyx_INCREF(__pyx_v_self->_certificate_chain);
  __pyx_r = __pyx_v_self->_certificate_chain;
  goto __pyx_L0;

  /* function exit code */
  __pyx_L0:;
  __Pyx_XGIVEREF(__pyx_r);
  __Pyx_RefNannyFinishContext();
  return __pyx_r;
}

/* "(tree fragment)":1
 * def __reduce_cython__(self):             # <<<<<<<<<<<<<<
 *     raise TypeError("no default __reduce__ due to non-trivial __cinit__")
 * def __setstate_cython__(self, __pyx_state):
 */

/* Python wrapper */
static PyObject *__pyx_pw_4grpc_7_cython_6cygrpc_21SSLChannelCredentials_3__reduce_cython__(PyObject *__pyx_v_self, CYTHON_UNUSED PyObject *unused); /*proto*/
static PyObject *__pyx_pw_4grpc_7_cython_6cygrpc_21SSLChannelCredentials_3__reduce_cython__(PyObject *__pyx_v_self, CYTHON_UNUSED PyObject *unused) {
  PyObject *__pyx_r = 0;
  __Pyx_RefNannyDeclarations
  __Pyx_RefNannySetupContext("__reduce_cython__ (wrapper)", 0);
  __pyx_r = __pyx_pf_4grpc_7_cython_6cygrpc_21SSLChannelCredentials_2__reduce_cython__(((struct __pyx_obj_4grpc_7_cython_6cygrpc_SSLChannelCredentials *)__pyx_v_self));

  /* function exit code */
  __Pyx_RefNannyFinishContext();
  return __pyx_r;
}

static PyObject *__pyx_pf_4grpc_7_cython_6cygrpc_21SSLChannelCredentials_2__reduce_cython__(CYTHON_UNUSED struct __pyx_obj_4grpc_7_cython_6cygrpc_SSLChannelCredentials *__pyx_v_self) {
  PyObject *__pyx_r = NULL;
  __Pyx_RefNannyDeclarations
  PyObject *__pyx_t_1 = NULL;
  __Pyx_RefNannySetupContext("__reduce_cython__", 0);

  /* "(tree fragment)":2
 * def __reduce_cython__(self):
 *     raise TypeError("no default __reduce__ due to non-trivial __cinit__")             # <<<<<<<<<<<<<<
 * def __setstate_cython__(self, __pyx_state):
 *     raise TypeError("no default __reduce__ due to non-trivial __cinit__")
 */
  __pyx_t_1 = __Pyx_PyObject_Call(__pyx_builtin_TypeError, __pyx_tuple__60, NULL); if (unlikely(!__pyx_t_1)) __PYX_ERR(7, 2, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_1);
  __Pyx_Raise(__pyx_t_1, 0, 0, 0);
  __Pyx_DECREF(__pyx_t_1); __pyx_t_1 = 0;
  __PYX_ERR(7, 2, __pyx_L1_error)

  /* "(tree fragment)":1
 * def __reduce_cython__(self):             # <<<<<<<<<<<<<<
 *     raise TypeError("no default __reduce__ due to non-trivial __cinit__")
 * def __setstate_cython__(self, __pyx_state):
 */

  /* function exit code */
  __pyx_L1_error:;
  __Pyx_XDECREF(__pyx_t_1);
  __Pyx_AddTraceback("grpc._cython.cygrpc.SSLChannelCredentials.__reduce_cython__", __pyx_clineno, __pyx_lineno, __pyx_filename);
  __pyx_r = NULL;
  __Pyx_XGIVEREF(__pyx_r);
  __Pyx_RefNannyFinishContext();
  return __pyx_r;
}

/* "(tree fragment)":3
 * def __reduce_cython__(self):
 *     raise TypeError("no default __reduce__ due to non-trivial __cinit__")
 * def __setstate_cython__(self, __pyx_state):             # <<<<<<<<<<<<<<
 *     raise TypeError("no default __reduce__ due to non-trivial __cinit__")
 */

/* Python wrapper */
static PyObject *__pyx_pw_4grpc_7_cython_6cygrpc_21SSLChannelCredentials_5__setstate_cython__(PyObject *__pyx_v_self, PyObject *__pyx_v___pyx_state); /*proto*/
static PyObject *__pyx_pw_4grpc_7_cython_6cygrpc_21SSLChannelCredentials_5__setstate_cython__(PyObject *__pyx_v_self, PyObject *__pyx_v___pyx_state) {
  PyObject *__pyx_r = 0;
  __Pyx_RefNannyDeclarations
  __Pyx_RefNannySetupContext("__setstate_cython__ (wrapper)", 0);
  __pyx_r = __pyx_pf_4grpc_7_cython_6cygrpc_21SSLChannelCredentials_4__setstate_cython__(((struct __pyx_obj_4grpc_7_cython_6cygrpc_SSLChannelCredentials *)__pyx_v_self), ((PyObject *)__pyx_v___pyx_state));

  /* function exit code */
  __Pyx_RefNannyFinishContext();
  return __pyx_r;
}

static PyObject *__pyx_pf_4grpc_7_cython_6cygrpc_21SSLChannelCredentials_4__setstate_cython__(CYTHON_UNUSED struct __pyx_obj_4grpc_7_cython_6cygrpc_SSLChannelCredentials *__pyx_v_self, CYTHON_UNUSED PyObject *__pyx_v___pyx_state) {
  PyObject *__pyx_r = NULL;
  __Pyx_RefNannyDeclarations
  PyObject *__pyx_t_1 = NULL;
  __Pyx_RefNannySetupContext("__setstate_cython__", 0);

  /* "(tree fragment)":4
 *     raise TypeError("no default __reduce__ due to non-trivial __cinit__")
 * def __setstate_cython__(self, __pyx_state):
 *     raise TypeError("no default __reduce__ due to non-trivial __cinit__")             # <<<<<<<<<<<<<<
 */
  __pyx_t_1 = __Pyx_PyObject_Call(__pyx_builtin_TypeError, __pyx_tuple__61, NULL); if (unlikely(!__pyx_t_1)) __PYX_ERR(7, 4, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_1);
  __Pyx_Raise(__pyx_t_1, 0, 0, 0);
  __Pyx_DECREF(__pyx_t_1); __pyx_t_1 = 0;
  __PYX_ERR(7, 4, __pyx_L1_error)

  /* "(tree fragment)":3
 * def __reduce_cython__(self):
 *     raise TypeError("no default __reduce__ due to non-trivial __cinit__")
 * def __setstate_cython__(self, __pyx_state):             # <<<<<<<<<<<<<<
 *     raise TypeError("no default __reduce__ due to non-trivial __cinit__")
 */

  /* function exit code */
  __pyx_L1_error:;
  __Pyx_XDECREF(__pyx_t_1);
  __Pyx_AddTraceback("grpc._cython.cygrpc.SSLChannelCredentials.__setstate_cython__", __pyx_clineno, __pyx_lineno, __pyx_filename);
  __pyx_r = NULL;
  __Pyx_XGIVEREF(__pyx_r);
  __Pyx_RefNannyFinishContext();
  return __pyx_r;
}

/* "src/python/grpcio/grpc/_cython/_cygrpc/credentials.pyx.pxi":163
 * cdef class CompositeChannelCredentials(ChannelCredentials):
 * 
 *   def __cinit__(self, call_credentialses, channel_credentials):             # <<<<<<<<<<<<<<
 *     self._call_credentialses = call_credentialses
 *     self._channel_credentials = channel_credentials
 */

/* Python wrapper */
static int __pyx_pw_4grpc_7_cython_6cygrpc_27CompositeChannelCredentials_1__cinit__(PyObject *__pyx_v_self, PyObject *__pyx_args, PyObject *__pyx_kwds); /*proto*/
static int __pyx_pw_4grpc_7_cython_6cygrpc_27CompositeChannelCredentials_1__cinit__(PyObject *__pyx_v_self, PyObject *__pyx_args, PyObject *__pyx_kwds) {
  PyObject *__pyx_v_call_credentialses = 0;
  PyObject *__pyx_v_channel_credentials = 0;
  int __pyx_r;
  __Pyx_RefNannyDeclarations
  __Pyx_RefNannySetupContext("__cinit__ (wrapper)", 0);
  {
    static PyObject **__pyx_pyargnames[] = {&__pyx_n_s_call_credentialses,&__pyx_n_s_channel_credentials,0};
    PyObject* values[2] = {0,0};
    if (unlikely(__pyx_kwds)) {
      Py_ssize_t kw_args;
      const Py_ssize_t pos_args = PyTuple_GET_SIZE(__pyx_args);
      switch (pos_args) {
        case  2: values[1] = PyTuple_GET_ITEM(__pyx_args, 1);
        CYTHON_FALLTHROUGH;
        case  1: values[0] = PyTuple_GET_ITEM(__pyx_args, 0);
        CYTHON_FALLTHROUGH;
        case  0: break;
        default: goto __pyx_L5_argtuple_error;
      }
      kw_args = PyDict_Size(__pyx_kwds);
      switch (pos_args) {
        case  0:
        if (likely((values[0] = __Pyx_PyDict_GetItemStr(__pyx_kwds, __pyx_n_s_call_credentialses)) != 0)) kw_args--;
        else goto __pyx_L5_argtuple_error;
        CYTHON_FALLTHROUGH;
        case  1:
        if (likely((values[1] = __Pyx_PyDict_GetItemStr(__pyx_kwds, __pyx_n_s_channel_credentials)) != 0)) kw_args--;
        else {
          __Pyx_RaiseArgtupleInvalid("__cinit__", 1, 2, 2, 1); __PYX_ERR(4, 163, __pyx_L3_error)
        }
      }
      if (unlikely(kw_args > 0)) {
        if (unlikely(__Pyx_ParseOptionalKeywords(__pyx_kwds, __pyx_pyargnames, 0, values, pos_args, "__cinit__") < 0)) __PYX_ERR(4, 163, __pyx_L3_error)
      }
    } else if (PyTuple_GET_SIZE(__pyx_args) != 2) {
      goto __pyx_L5_argtuple_error;
    } else {
      values[0] = PyTuple_GET_ITEM(__pyx_args, 0);
      values[1] = PyTuple_GET_ITEM(__pyx_args, 1);
    }
    __pyx_v_call_credentialses = values[0];
    __pyx_v_channel_credentials = values[1];
  }
  goto __pyx_L4_argument_unpacking_done;
  __pyx_L5_argtuple_error:;
  __Pyx_RaiseArgtupleInvalid("__cinit__", 1, 2, 2, PyTuple_GET_SIZE(__pyx_args)); __PYX_ERR(4, 163, __pyx_L3_error)
  __pyx_L3_error:;
  __Pyx_AddTraceback("grpc._cython.cygrpc.CompositeChannelCredentials.__cinit__", __pyx_clineno, __pyx_lineno, __pyx_filename);
  __Pyx_RefNannyFinishContext();
  return -1;
  __pyx_L4_argument_unpacking_done:;
  __pyx_r = __pyx_pf_4grpc_7_cython_6cygrpc_27CompositeChannelCredentials___cinit__(((struct __pyx_obj_4grpc_7_cython_6cygrpc_CompositeChannelCredentials *)__pyx_v_self), __pyx_v_call_credentialses, __pyx_v_channel_credentials);

  /* function exit code */
  __Pyx_RefNannyFinishContext();
  return __pyx_r;
}

static int __pyx_pf_4grpc_7_cython_6cygrpc_27CompositeChannelCredentials___cinit__(struct __pyx_obj_4grpc_7_cython_6cygrpc_CompositeChannelCredentials *__pyx_v_self, PyObject *__pyx_v_call_credentialses, PyObject *__pyx_v_channel_credentials) {
  int __pyx_r;
  __Pyx_RefNannyDeclarations
  PyObject *__pyx_t_1 = NULL;
  __Pyx_RefNannySetupContext("__cinit__", 0);

  /* "src/python/grpcio/grpc/_cython/_cygrpc/credentials.pyx.pxi":164
 * 
 *   def __cinit__(self, call_credentialses, channel_credentials):
 *     self._call_credentialses = call_credentialses             # <<<<<<<<<<<<<<
 *     self._channel_credentials = channel_credentials
 * 
 */
  if (!(likely(PyTuple_CheckExact(__pyx_v_call_credentialses))||((__pyx_v_call_credentialses) == Py_None)||(PyErr_Format(PyExc_TypeError, "Expected %.16s, got %.200s", "tuple", Py_TYPE(__pyx_v_call_credentialses)->tp_name), 0))) __PYX_ERR(4, 164, __pyx_L1_error)
  __pyx_t_1 = __pyx_v_call_credentialses;
  __Pyx_INCREF(__pyx_t_1);
  __Pyx_GIVEREF(__pyx_t_1);
  __Pyx_GOTREF(__pyx_v_self->_call_credentialses);
  __Pyx_DECREF(__pyx_v_self->_call_credentialses);
  __pyx_v_self->_call_credentialses = ((PyObject*)__pyx_t_1);
  __pyx_t_1 = 0;

  /* "src/python/grpcio/grpc/_cython/_cygrpc/credentials.pyx.pxi":165
 *   def __cinit__(self, call_credentialses, channel_credentials):
 *     self._call_credentialses = call_credentialses
 *     self._channel_credentials = channel_credentials             # <<<<<<<<<<<<<<
 * 
 *   cdef grpc_channel_credentials *c(self) except *:
 */
  if (!(likely(((__pyx_v_channel_credentials) == Py_None) || likely(__Pyx_TypeTest(__pyx_v_channel_credentials, __pyx_ptype_4grpc_7_cython_6cygrpc_ChannelCredentials))))) __PYX_ERR(4, 165, __pyx_L1_error)
  __pyx_t_1 = __pyx_v_channel_credentials;
  __Pyx_INCREF(__pyx_t_1);
  __Pyx_GIVEREF(__pyx_t_1);
  __Pyx_GOTREF(__pyx_v_self->_channel_credentials);
  __Pyx_DECREF(((PyObject *)__pyx_v_self->_channel_credentials));
  __pyx_v_self->_channel_credentials = ((struct __pyx_obj_4grpc_7_cython_6cygrpc_ChannelCredentials *)__pyx_t_1);
  __pyx_t_1 = 0;

  /* "src/python/grpcio/grpc/_cython/_cygrpc/credentials.pyx.pxi":163
 * cdef class CompositeChannelCredentials(ChannelCredentials):
 * 
 *   def __cinit__(self, call_credentialses, channel_credentials):             # <<<<<<<<<<<<<<
 *     self._call_credentialses = call_credentialses
 *     self._channel_credentials = channel_credentials
 */

  /* function exit code */
  __pyx_r = 0;
  goto __pyx_L0;
  __pyx_L1_error:;
  __Pyx_XDECREF(__pyx_t_1);
  __Pyx_AddTraceback("grpc._cython.cygrpc.CompositeChannelCredentials.__cinit__", __pyx_clineno, __pyx_lineno, __pyx_filename);
  __pyx_r = -1;
  __pyx_L0:;
  __Pyx_RefNannyFinishContext();
  return __pyx_r;
}

/* "src/python/grpcio/grpc/_cython/_cygrpc/credentials.pyx.pxi":167
 *     self._channel_credentials = channel_credentials
 * 
 *   cdef grpc_channel_credentials *c(self) except *:             # <<<<<<<<<<<<<<
 *     cdef grpc_channel_credentials *c_channel_credentials
 *     c_channel_credentials = self._channel_credentials.c()
 */

static grpc_channel_credentials *__pyx_f_4grpc_7_cython_6cygrpc_27CompositeChannelCredentials_c(struct __pyx_obj_4grpc_7_cython_6cygrpc_CompositeChannelCredentials *__pyx_v_self) {
  grpc_channel_credentials *__pyx_v_c_channel_credentials;
  grpc_call_credentials *__pyx_v_c_call_credentials_composition;
  grpc_channel_credentials *__pyx_v_c_composition;
  grpc_channel_credentials *__pyx_r;
  __Pyx_RefNannyDeclarations
  grpc_channel_credentials *__pyx_t_1;
  PyObject *__pyx_t_2 = NULL;
  __Pyx_RefNannySetupContext("c", 0);

  /* "src/python/grpcio/grpc/_cython/_cygrpc/credentials.pyx.pxi":169
 *   cdef grpc_channel_credentials *c(self) except *:
 *     cdef grpc_channel_credentials *c_channel_credentials
 *     c_channel_credentials = self._channel_credentials.c()             # <<<<<<<<<<<<<<
 *     cdef grpc_call_credentials *c_call_credentials_composition = _composition(
 *         self._call_credentialses)
 */
  __pyx_t_1 = ((struct __pyx_vtabstruct_4grpc_7_cython_6cygrpc_ChannelCredentials *)__pyx_v_self->_channel_credentials->__pyx_vtab)->c(__pyx_v_self->_channel_credentials); if (unlikely(PyErr_Occurred())) __PYX_ERR(4, 169, __pyx_L1_error)
  __pyx_v_c_channel_credentials = __pyx_t_1;

  /* "src/python/grpcio/grpc/_cython/_cygrpc/credentials.pyx.pxi":171
 *     c_channel_credentials = self._channel_credentials.c()
 *     cdef grpc_call_credentials *c_call_credentials_composition = _composition(
 *         self._call_credentialses)             # <<<<<<<<<<<<<<
 *     cdef grpc_channel_credentials *composition
 *     c_composition = grpc_composite_channel_credentials_create(
 */
  __pyx_t_2 = __pyx_v_self->_call_credentialses;
  __Pyx_INCREF(__pyx_t_2);

  /* "src/python/grpcio/grpc/_cython/_cygrpc/credentials.pyx.pxi":170
 *     cdef grpc_channel_credentials *c_channel_credentials
 *     c_channel_credentials = self._channel_credentials.c()
 *     cdef grpc_call_credentials *c_call_credentials_composition = _composition(             # <<<<<<<<<<<<<<
 *         self._call_credentialses)
 *     cdef grpc_channel_credentials *composition
 */
  __pyx_v_c_call_credentials_composition = __pyx_f_4grpc_7_cython_6cygrpc__composition(__pyx_t_2);
  __Pyx_DECREF(__pyx_t_2); __pyx_t_2 = 0;

  /* "src/python/grpcio/grpc/_cython/_cygrpc/credentials.pyx.pxi":173
 *         self._call_credentialses)
 *     cdef grpc_channel_credentials *composition
 *     c_composition = grpc_composite_channel_credentials_create(             # <<<<<<<<<<<<<<
 *         c_channel_credentials, c_call_credentials_composition, NULL)
 *     grpc_channel_credentials_release(c_channel_credentials)
 */
  __pyx_v_c_composition = grpc_composite_channel_credentials_create(__pyx_v_c_channel_credentials, __pyx_v_c_call_credentials_composition, NULL);

  /* "src/python/grpcio/grpc/_cython/_cygrpc/credentials.pyx.pxi":175
 *     c_composition = grpc_composite_channel_credentials_create(
 *         c_channel_credentials, c_call_credentials_composition, NULL)
 *     grpc_channel_credentials_release(c_channel_credentials)             # <<<<<<<<<<<<<<
 *     grpc_call_credentials_release(c_call_credentials_composition)
 *     return c_composition
 */
  grpc_channel_credentials_release(__pyx_v_c_channel_credentials);

  /* "src/python/grpcio/grpc/_cython/_cygrpc/credentials.pyx.pxi":176
 *         c_channel_credentials, c_call_credentials_composition, NULL)
 *     grpc_channel_credentials_release(c_channel_credentials)
 *     grpc_call_credentials_release(c_call_credentials_composition)             # <<<<<<<<<<<<<<
 *     return c_composition
 * 
 */
  grpc_call_credentials_release(__pyx_v_c_call_credentials_composition);

  /* "src/python/grpcio/grpc/_cython/_cygrpc/credentials.pyx.pxi":177
 *     grpc_channel_credentials_release(c_channel_credentials)
 *     grpc_call_credentials_release(c_call_credentials_composition)
 *     return c_composition             # <<<<<<<<<<<<<<
 * 
 * 
 */
  __pyx_r = __pyx_v_c_composition;
  goto __pyx_L0;

  /* "src/python/grpcio/grpc/_cython/_cygrpc/credentials.pyx.pxi":167
 *     self._channel_credentials = channel_credentials
 * 
 *   cdef grpc_channel_credentials *c(self) except *:             # <<<<<<<<<<<<<<
 *     cdef grpc_channel_credentials *c_channel_credentials
 *     c_channel_credentials = self._channel_credentials.c()
 */

  /* function exit code */
  __pyx_L1_error:;
  __Pyx_XDECREF(__pyx_t_2);
  __Pyx_AddTraceback("grpc._cython.cygrpc.CompositeChannelCredentials.c", __pyx_clineno, __pyx_lineno, __pyx_filename);
  __pyx_r = 0;
  __pyx_L0:;
  __Pyx_RefNannyFinishContext();
  return __pyx_r;
}

/* "src/python/grpcio/grpc/_cython/_cygrpc/credentials.pxd.pxi":73
 * cdef class CompositeChannelCredentials(ChannelCredentials):
 * 
 *   cdef readonly tuple _call_credentialses             # <<<<<<<<<<<<<<
 *   cdef readonly ChannelCredentials _channel_credentials
 * 
 */

/* Python wrapper */
static PyObject *__pyx_pw_4grpc_7_cython_6cygrpc_27CompositeChannelCredentials_19_call_credentialses_1__get__(PyObject *__pyx_v_self); /*proto*/
static PyObject *__pyx_pw_4grpc_7_cython_6cygrpc_27CompositeChannelCredentials_19_call_credentialses_1__get__(PyObject *__pyx_v_self) {
  PyObject *__pyx_r = 0;
  __Pyx_RefNannyDeclarations
  __Pyx_RefNannySetupContext("__get__ (wrapper)", 0);
  __pyx_r = __pyx_pf_4grpc_7_cython_6cygrpc_27CompositeChannelCredentials_19_call_credentialses___get__(((struct __pyx_obj_4grpc_7_cython_6cygrpc_CompositeChannelCredentials *)__pyx_v_self));

  /* function exit code */
  __Pyx_RefNannyFinishContext();
  return __pyx_r;
}

static PyObject *__pyx_pf_4grpc_7_cython_6cygrpc_27CompositeChannelCredentials_19_call_credentialses___get__(struct __pyx_obj_4grpc_7_cython_6cygrpc_CompositeChannelCredentials *__pyx_v_self) {
  PyObject *__pyx_r = NULL;
  __Pyx_RefNannyDeclarations
  __Pyx_RefNannySetupContext("__get__", 0);
  __Pyx_XDECREF(__pyx_r);
  __Pyx_INCREF(__pyx_v_self->_call_credentialses);
  __pyx_r = __pyx_v_self->_call_credentialses;
  goto __pyx_L0;

  /* function exit code */
  __pyx_L0:;
  __Pyx_XGIVEREF(__pyx_r);
  __Pyx_RefNannyFinishContext();
  return __pyx_r;
}

/* "src/python/grpcio/grpc/_cython/_cygrpc/credentials.pxd.pxi":74
 * 
 *   cdef readonly tuple _call_credentialses
 *   cdef readonly ChannelCredentials _channel_credentials             # <<<<<<<<<<<<<<
 * 
 *   cdef grpc_channel_credentials *c(self) except *
 */

/* Python wrapper */
static PyObject *__pyx_pw_4grpc_7_cython_6cygrpc_27CompositeChannelCredentials_20_channel_credentials_1__get__(PyObject *__pyx_v_self); /*proto*/
static PyObject *__pyx_pw_4grpc_7_cython_6cygrpc_27CompositeChannelCredentials_20_channel_credentials_1__get__(PyObject *__pyx_v_self) {
  PyObject *__pyx_r = 0;
  __Pyx_RefNannyDeclarations
  __Pyx_RefNannySetupContext("__get__ (wrapper)", 0);
  __pyx_r = __pyx_pf_4grpc_7_cython_6cygrpc_27CompositeChannelCredentials_20_channel_credentials___get__(((struct __pyx_obj_4grpc_7_cython_6cygrpc_CompositeChannelCredentials *)__pyx_v_self));

  /* function exit code */
  __Pyx_RefNannyFinishContext();
  return __pyx_r;
}

static PyObject *__pyx_pf_4grpc_7_cython_6cygrpc_27CompositeChannelCredentials_20_channel_credentials___get__(struct __pyx_obj_4grpc_7_cython_6cygrpc_CompositeChannelCredentials *__pyx_v_self) {
  PyObject *__pyx_r = NULL;
  __Pyx_RefNannyDeclarations
  __Pyx_RefNannySetupContext("__get__", 0);
  __Pyx_XDECREF(__pyx_r);
  __Pyx_INCREF(((PyObject *)__pyx_v_self->_channel_credentials));
  __pyx_r = ((PyObject *)__pyx_v_self->_channel_credentials);
  goto __pyx_L0;

  /* function exit code */
  __pyx_L0:;
  __Pyx_XGIVEREF(__pyx_r);
  __Pyx_RefNannyFinishContext();
  return __pyx_r;
}

/* "(tree fragment)":1
 * def __reduce_cython__(self):             # <<<<<<<<<<<<<<
 *     raise TypeError("no default __reduce__ due to non-trivial __cinit__")
 * def __setstate_cython__(self, __pyx_state):
 */

/* Python wrapper */
static PyObject *__pyx_pw_4grpc_7_cython_6cygrpc_27CompositeChannelCredentials_3__reduce_cython__(PyObject *__pyx_v_self, CYTHON_UNUSED PyObject *unused); /*proto*/
static PyObject *__pyx_pw_4grpc_7_cython_6cygrpc_27CompositeChannelCredentials_3__reduce_cython__(PyObject *__pyx_v_self, CYTHON_UNUSED PyObject *unused) {
  PyObject *__pyx_r = 0;
  __Pyx_RefNannyDeclarations
  __Pyx_RefNannySetupContext("__reduce_cython__ (wrapper)", 0);
  __pyx_r = __pyx_pf_4grpc_7_cython_6cygrpc_27CompositeChannelCredentials_2__reduce_cython__(((struct __pyx_obj_4grpc_7_cython_6cygrpc_CompositeChannelCredentials *)__pyx_v_self));

  /* function exit code */
  __Pyx_RefNannyFinishContext();
  return __pyx_r;
}

static PyObject *__pyx_pf_4grpc_7_cython_6cygrpc_27CompositeChannelCredentials_2__reduce_cython__(CYTHON_UNUSED struct __pyx_obj_4grpc_7_cython_6cygrpc_CompositeChannelCredentials *__pyx_v_self) {
  PyObject *__pyx_r = NULL;
  __Pyx_RefNannyDeclarations
  PyObject *__pyx_t_1 = NULL;
  __Pyx_RefNannySetupContext("__reduce_cython__", 0);

  /* "(tree fragment)":2
 * def __reduce_cython__(self):
 *     raise TypeError("no default __reduce__ due to non-trivial __cinit__")             # <<<<<<<<<<<<<<
 * def __setstate_cython__(self, __pyx_state):
 *     raise TypeError("no default __reduce__ due to non-trivial __cinit__")
 */
  __pyx_t_1 = __Pyx_PyObject_Call(__pyx_builtin_TypeError, __pyx_tuple__62, NULL); if (unlikely(!__pyx_t_1)) __PYX_ERR(7, 2, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_1);
  __Pyx_Raise(__pyx_t_1, 0, 0, 0);
  __Pyx_DECREF(__pyx_t_1); __pyx_t_1 = 0;
  __PYX_ERR(7, 2, __pyx_L1_error)

  /* "(tree fragment)":1
 * def __reduce_cython__(self):             # <<<<<<<<<<<<<<
 *     raise TypeError("no default __reduce__ due to non-trivial __cinit__")
 * def __setstate_cython__(self, __pyx_state):
 */

  /* function exit code */
  __pyx_L1_error:;
  __Pyx_XDECREF(__pyx_t_1);
  __Pyx_AddTraceback("grpc._cython.cygrpc.CompositeChannelCredentials.__reduce_cython__", __pyx_clineno, __pyx_lineno, __pyx_filename);
  __pyx_r = NULL;
  __Pyx_XGIVEREF(__pyx_r);
  __Pyx_RefNannyFinishContext();
  return __pyx_r;
}

/* "(tree fragment)":3
 * def __reduce_cython__(self):
 *     raise TypeError("no default __reduce__ due to non-trivial __cinit__")
 * def __setstate_cython__(self, __pyx_state):             # <<<<<<<<<<<<<<
 *     raise TypeError("no default __reduce__ due to non-trivial __cinit__")
 */

/* Python wrapper */
static PyObject *__pyx_pw_4grpc_7_cython_6cygrpc_27CompositeChannelCredentials_5__setstate_cython__(PyObject *__pyx_v_self, PyObject *__pyx_v___pyx_state); /*proto*/
static PyObject *__pyx_pw_4grpc_7_cython_6cygrpc_27CompositeChannelCredentials_5__setstate_cython__(PyObject *__pyx_v_self, PyObject *__pyx_v___pyx_state) {
  PyObject *__pyx_r = 0;
  __Pyx_RefNannyDeclarations
  __Pyx_RefNannySetupContext("__setstate_cython__ (wrapper)", 0);
  __pyx_r = __pyx_pf_4grpc_7_cython_6cygrpc_27CompositeChannelCredentials_4__setstate_cython__(((struct __pyx_obj_4grpc_7_cython_6cygrpc_CompositeChannelCredentials *)__pyx_v_self), ((PyObject *)__pyx_v___pyx_state));

  /* function exit code */
  __Pyx_RefNannyFinishContext();
  return __pyx_r;
}

static PyObject *__pyx_pf_4grpc_7_cython_6cygrpc_27CompositeChannelCredentials_4__setstate_cython__(CYTHON_UNUSED struct __pyx_obj_4grpc_7_cython_6cygrpc_CompositeChannelCredentials *__pyx_v_self, CYTHON_UNUSED PyObject *__pyx_v___pyx_state) {
  PyObject *__pyx_r = NULL;
  __Pyx_RefNannyDeclarations
  PyObject *__pyx_t_1 = NULL;
  __Pyx_RefNannySetupContext("__setstate_cython__", 0);

  /* "(tree fragment)":4
 *     raise TypeError("no default __reduce__ due to non-trivial __cinit__")
 * def __setstate_cython__(self, __pyx_state):
 *     raise TypeError("no default __reduce__ due to non-trivial __cinit__")             # <<<<<<<<<<<<<<
 */
  __pyx_t_1 = __Pyx_PyObject_Call(__pyx_builtin_TypeError, __pyx_tuple__63, NULL); if (unlikely(!__pyx_t_1)) __PYX_ERR(7, 4, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_1);
  __Pyx_Raise(__pyx_t_1, 0, 0, 0);
  __Pyx_DECREF(__pyx_t_1); __pyx_t_1 = 0;
  __PYX_ERR(7, 4, __pyx_L1_error)

  /* "(tree fragment)":3
 * def __reduce_cython__(self):
 *     raise TypeError("no default __reduce__ due to non-trivial __cinit__")
 * def __setstate_cython__(self, __pyx_state):             # <<<<<<<<<<<<<<
 *     raise TypeError("no default __reduce__ due to non-trivial __cinit__")
 */

  /* function exit code */
  __pyx_L1_error:;
  __Pyx_XDECREF(__pyx_t_1);
  __Pyx_AddTraceback("grpc._cython.cygrpc.CompositeChannelCredentials.__setstate_cython__", __pyx_clineno, __pyx_lineno, __pyx_filename);
  __pyx_r = NULL;
  __Pyx_XGIVEREF(__pyx_r);
  __Pyx_RefNannyFinishContext();
  return __pyx_r;
}

/* "src/python/grpcio/grpc/_cython/_cygrpc/credentials.pyx.pxi":182
 * cdef class ServerCertificateConfig:
 * 
 *   def __cinit__(self):             # <<<<<<<<<<<<<<
 *     fork_handlers_and_grpc_init()
 *     self.c_cert_config = NULL
 */

/* Python wrapper */
static int __pyx_pw_4grpc_7_cython_6cygrpc_23ServerCertificateConfig_1__cinit__(PyObject *__pyx_v_self, PyObject *__pyx_args, PyObject *__pyx_kwds); /*proto*/
static int __pyx_pw_4grpc_7_cython_6cygrpc_23ServerCertificateConfig_1__cinit__(PyObject *__pyx_v_self, PyObject *__pyx_args, PyObject *__pyx_kwds) {
  int __pyx_r;
  __Pyx_RefNannyDeclarations
  __Pyx_RefNannySetupContext("__cinit__ (wrapper)", 0);
  if (unlikely(PyTuple_GET_SIZE(__pyx_args) > 0)) {
    __Pyx_RaiseArgtupleInvalid("__cinit__", 1, 0, 0, PyTuple_GET_SIZE(__pyx_args)); return -1;}
  if (unlikely(__pyx_kwds) && unlikely(PyDict_Size(__pyx_kwds) > 0) && unlikely(!__Pyx_CheckKeywordStrings(__pyx_kwds, "__cinit__", 0))) return -1;
  __pyx_r = __pyx_pf_4grpc_7_cython_6cygrpc_23ServerCertificateConfig___cinit__(((struct __pyx_obj_4grpc_7_cython_6cygrpc_ServerCertificateConfig *)__pyx_v_self));

  /* function exit code */
  __Pyx_RefNannyFinishContext();
  return __pyx_r;
}

static int __pyx_pf_4grpc_7_cython_6cygrpc_23ServerCertificateConfig___cinit__(struct __pyx_obj_4grpc_7_cython_6cygrpc_ServerCertificateConfig *__pyx_v_self) {
  int __pyx_r;
  __Pyx_RefNannyDeclarations
  PyObject *__pyx_t_1 = NULL;
  PyObject *__pyx_t_2 = NULL;
  PyObject *__pyx_t_3 = NULL;
  __Pyx_RefNannySetupContext("__cinit__", 0);

  /* "src/python/grpcio/grpc/_cython/_cygrpc/credentials.pyx.pxi":183
 * 
 *   def __cinit__(self):
 *     fork_handlers_and_grpc_init()             # <<<<<<<<<<<<<<
 *     self.c_cert_config = NULL
 *     self.c_pem_root_certs = NULL
 */
  __pyx_t_2 = __Pyx_GetModuleGlobalName(__pyx_n_s_fork_handlers_and_grpc_init); if (unlikely(!__pyx_t_2)) __PYX_ERR(4, 183, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_2);
  __pyx_t_3 = NULL;
  if (CYTHON_UNPACK_METHODS && unlikely(PyMethod_Check(__pyx_t_2))) {
    __pyx_t_3 = PyMethod_GET_SELF(__pyx_t_2);
    if (likely(__pyx_t_3)) {
      PyObject* function = PyMethod_GET_FUNCTION(__pyx_t_2);
      __Pyx_INCREF(__pyx_t_3);
      __Pyx_INCREF(function);
      __Pyx_DECREF_SET(__pyx_t_2, function);
    }
  }
  if (__pyx_t_3) {
    __pyx_t_1 = __Pyx_PyObject_CallOneArg(__pyx_t_2, __pyx_t_3); if (unlikely(!__pyx_t_1)) __PYX_ERR(4, 183, __pyx_L1_error)
    __Pyx_DECREF(__pyx_t_3); __pyx_t_3 = 0;
  } else {
    __pyx_t_1 = __Pyx_PyObject_CallNoArg(__pyx_t_2); if (unlikely(!__pyx_t_1)) __PYX_ERR(4, 183, __pyx_L1_error)
  }
  __Pyx_GOTREF(__pyx_t_1);
  __Pyx_DECREF(__pyx_t_2); __pyx_t_2 = 0;
  __Pyx_DECREF(__pyx_t_1); __pyx_t_1 = 0;

  /* "src/python/grpcio/grpc/_cython/_cygrpc/credentials.pyx.pxi":184
 *   def __cinit__(self):
 *     fork_handlers_and_grpc_init()
 *     self.c_cert_config = NULL             # <<<<<<<<<<<<<<
 *     self.c_pem_root_certs = NULL
 *     self.c_ssl_pem_key_cert_pairs = NULL
 */
  __pyx_v_self->c_cert_config = NULL;

  /* "src/python/grpcio/grpc/_cython/_cygrpc/credentials.pyx.pxi":185
 *     fork_handlers_and_grpc_init()
 *     self.c_cert_config = NULL
 *     self.c_pem_root_certs = NULL             # <<<<<<<<<<<<<<
 *     self.c_ssl_pem_key_cert_pairs = NULL
 *     self.references = []
 */
  __pyx_v_self->c_pem_root_certs = NULL;

  /* "src/python/grpcio/grpc/_cython/_cygrpc/credentials.pyx.pxi":186
 *     self.c_cert_config = NULL
 *     self.c_pem_root_certs = NULL
 *     self.c_ssl_pem_key_cert_pairs = NULL             # <<<<<<<<<<<<<<
 *     self.references = []
 * 
 */
  __pyx_v_self->c_ssl_pem_key_cert_pairs = NULL;

  /* "src/python/grpcio/grpc/_cython/_cygrpc/credentials.pyx.pxi":187
 *     self.c_pem_root_certs = NULL
 *     self.c_ssl_pem_key_cert_pairs = NULL
 *     self.references = []             # <<<<<<<<<<<<<<
 * 
 *   def __dealloc__(self):
 */
  __pyx_t_1 = PyList_New(0); if (unlikely(!__pyx_t_1)) __PYX_ERR(4, 187, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_1);
  __Pyx_GIVEREF(__pyx_t_1);
  __Pyx_GOTREF(__pyx_v_self->references);
  __Pyx_DECREF(__pyx_v_self->references);
  __pyx_v_self->references = ((PyObject*)__pyx_t_1);
  __pyx_t_1 = 0;

  /* "src/python/grpcio/grpc/_cython/_cygrpc/credentials.pyx.pxi":182
 * cdef class ServerCertificateConfig:
 * 
 *   def __cinit__(self):             # <<<<<<<<<<<<<<
 *     fork_handlers_and_grpc_init()
 *     self.c_cert_config = NULL
 */

  /* function exit code */
  __pyx_r = 0;
  goto __pyx_L0;
  __pyx_L1_error:;
  __Pyx_XDECREF(__pyx_t_1);
  __Pyx_XDECREF(__pyx_t_2);
  __Pyx_XDECREF(__pyx_t_3);
  __Pyx_AddTraceback("grpc._cython.cygrpc.ServerCertificateConfig.__cinit__", __pyx_clineno, __pyx_lineno, __pyx_filename);
  __pyx_r = -1;
  __pyx_L0:;
  __Pyx_RefNannyFinishContext();
  return __pyx_r;
}

/* "src/python/grpcio/grpc/_cython/_cygrpc/credentials.pyx.pxi":189
 *     self.references = []
 * 
 *   def __dealloc__(self):             # <<<<<<<<<<<<<<
 *     grpc_ssl_server_certificate_config_destroy(self.c_cert_config)
 *     gpr_free(self.c_ssl_pem_key_cert_pairs)
 */

/* Python wrapper */
static void __pyx_pw_4grpc_7_cython_6cygrpc_23ServerCertificateConfig_3__dealloc__(PyObject *__pyx_v_self); /*proto*/
static void __pyx_pw_4grpc_7_cython_6cygrpc_23ServerCertificateConfig_3__dealloc__(PyObject *__pyx_v_self) {
  __Pyx_RefNannyDeclarations
  __Pyx_RefNannySetupContext("__dealloc__ (wrapper)", 0);
  __pyx_pf_4grpc_7_cython_6cygrpc_23ServerCertificateConfig_2__dealloc__(((struct __pyx_obj_4grpc_7_cython_6cygrpc_ServerCertificateConfig *)__pyx_v_self));

  /* function exit code */
  __Pyx_RefNannyFinishContext();
}

static void __pyx_pf_4grpc_7_cython_6cygrpc_23ServerCertificateConfig_2__dealloc__(struct __pyx_obj_4grpc_7_cython_6cygrpc_ServerCertificateConfig *__pyx_v_self) {
  __Pyx_RefNannyDeclarations
  __Pyx_RefNannySetupContext("__dealloc__", 0);

  /* "src/python/grpcio/grpc/_cython/_cygrpc/credentials.pyx.pxi":190
 * 
 *   def __dealloc__(self):
 *     grpc_ssl_server_certificate_config_destroy(self.c_cert_config)             # <<<<<<<<<<<<<<
 *     gpr_free(self.c_ssl_pem_key_cert_pairs)
 *     grpc_shutdown_blocking()
 */
  grpc_ssl_server_certificate_config_destroy(__pyx_v_self->c_cert_config);

  /* "src/python/grpcio/grpc/_cython/_cygrpc/credentials.pyx.pxi":191
 *   def __dealloc__(self):
 *     grpc_ssl_server_certificate_config_destroy(self.c_cert_config)
 *     gpr_free(self.c_ssl_pem_key_cert_pairs)             # <<<<<<<<<<<<<<
 *     grpc_shutdown_blocking()
 * 
 */
  gpr_free(__pyx_v_self->c_ssl_pem_key_cert_pairs);

  /* "src/python/grpcio/grpc/_cython/_cygrpc/credentials.pyx.pxi":192
 *     grpc_ssl_server_certificate_config_destroy(self.c_cert_config)
 *     gpr_free(self.c_ssl_pem_key_cert_pairs)
 *     grpc_shutdown_blocking()             # <<<<<<<<<<<<<<
 * 
 * 
 */
  grpc_shutdown_blocking();

  /* "src/python/grpcio/grpc/_cython/_cygrpc/credentials.pyx.pxi":189
 *     self.references = []
 * 
 *   def __dealloc__(self):             # <<<<<<<<<<<<<<
 *     grpc_ssl_server_certificate_config_destroy(self.c_cert_config)
 *     gpr_free(self.c_ssl_pem_key_cert_pairs)
 */

  /* function exit code */
  __Pyx_RefNannyFinishContext();
}

/* "(tree fragment)":1
 * def __reduce_cython__(self):             # <<<<<<<<<<<<<<
 *     raise TypeError("no default __reduce__ due to non-trivial __cinit__")
 * def __setstate_cython__(self, __pyx_state):
 */

/* Python wrapper */
static PyObject *__pyx_pw_4grpc_7_cython_6cygrpc_23ServerCertificateConfig_5__reduce_cython__(PyObject *__pyx_v_self, CYTHON_UNUSED PyObject *unused); /*proto*/
static PyObject *__pyx_pw_4grpc_7_cython_6cygrpc_23ServerCertificateConfig_5__reduce_cython__(PyObject *__pyx_v_self, CYTHON_UNUSED PyObject *unused) {
  PyObject *__pyx_r = 0;
  __Pyx_RefNannyDeclarations
  __Pyx_RefNannySetupContext("__reduce_cython__ (wrapper)", 0);
  __pyx_r = __pyx_pf_4grpc_7_cython_6cygrpc_23ServerCertificateConfig_4__reduce_cython__(((struct __pyx_obj_4grpc_7_cython_6cygrpc_ServerCertificateConfig *)__pyx_v_self));

  /* function exit code */
  __Pyx_RefNannyFinishContext();
  return __pyx_r;
}

static PyObject *__pyx_pf_4grpc_7_cython_6cygrpc_23ServerCertificateConfig_4__reduce_cython__(CYTHON_UNUSED struct __pyx_obj_4grpc_7_cython_6cygrpc_ServerCertificateConfig *__pyx_v_self) {
  PyObject *__pyx_r = NULL;
  __Pyx_RefNannyDeclarations
  PyObject *__pyx_t_1 = NULL;
  __Pyx_RefNannySetupContext("__reduce_cython__", 0);

  /* "(tree fragment)":2
 * def __reduce_cython__(self):
 *     raise TypeError("no default __reduce__ due to non-trivial __cinit__")             # <<<<<<<<<<<<<<
 * def __setstate_cython__(self, __pyx_state):
 *     raise TypeError("no default __reduce__ due to non-trivial __cinit__")
 */
  __pyx_t_1 = __Pyx_PyObject_Call(__pyx_builtin_TypeError, __pyx_tuple__64, NULL); if (unlikely(!__pyx_t_1)) __PYX_ERR(7, 2, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_1);
  __Pyx_Raise(__pyx_t_1, 0, 0, 0);
  __Pyx_DECREF(__pyx_t_1); __pyx_t_1 = 0;
  __PYX_ERR(7, 2, __pyx_L1_error)

  /* "(tree fragment)":1
 * def __reduce_cython__(self):             # <<<<<<<<<<<<<<
 *     raise TypeError("no default __reduce__ due to non-trivial __cinit__")
 * def __setstate_cython__(self, __pyx_state):
 */

  /* function exit code */
  __pyx_L1_error:;
  __Pyx_XDECREF(__pyx_t_1);
  __Pyx_AddTraceback("grpc._cython.cygrpc.ServerCertificateConfig.__reduce_cython__", __pyx_clineno, __pyx_lineno, __pyx_filename);
  __pyx_r = NULL;
  __Pyx_XGIVEREF(__pyx_r);
  __Pyx_RefNannyFinishContext();
  return __pyx_r;
}

/* "(tree fragment)":3
 * def __reduce_cython__(self):
 *     raise TypeError("no default __reduce__ due to non-trivial __cinit__")
 * def __setstate_cython__(self, __pyx_state):             # <<<<<<<<<<<<<<
 *     raise TypeError("no default __reduce__ due to non-trivial __cinit__")
 */

/* Python wrapper */
static PyObject *__pyx_pw_4grpc_7_cython_6cygrpc_23ServerCertificateConfig_7__setstate_cython__(PyObject *__pyx_v_self, PyObject *__pyx_v___pyx_state); /*proto*/
static PyObject *__pyx_pw_4grpc_7_cython_6cygrpc_23ServerCertificateConfig_7__setstate_cython__(PyObject *__pyx_v_self, PyObject *__pyx_v___pyx_state) {
  PyObject *__pyx_r = 0;
  __Pyx_RefNannyDeclarations
  __Pyx_RefNannySetupContext("__setstate_cython__ (wrapper)", 0);
  __pyx_r = __pyx_pf_4grpc_7_cython_6cygrpc_23ServerCertificateConfig_6__setstate_cython__(((struct __pyx_obj_4grpc_7_cython_6cygrpc_ServerCertificateConfig *)__pyx_v_self), ((PyObject *)__pyx_v___pyx_state));

  /* function exit code */
  __Pyx_RefNannyFinishContext();
  return __pyx_r;
}

static PyObject *__pyx_pf_4grpc_7_cython_6cygrpc_23ServerCertificateConfig_6__setstate_cython__(CYTHON_UNUSED struct __pyx_obj_4grpc_7_cython_6cygrpc_ServerCertificateConfig *__pyx_v_self, CYTHON_UNUSED PyObject *__pyx_v___pyx_state) {
  PyObject *__pyx_r = NULL;
  __Pyx_RefNannyDeclarations
  PyObject *__pyx_t_1 = NULL;
  __Pyx_RefNannySetupContext("__setstate_cython__", 0);

  /* "(tree fragment)":4
 *     raise TypeError("no default __reduce__ due to non-trivial __cinit__")
 * def __setstate_cython__(self, __pyx_state):
 *     raise TypeError("no default __reduce__ due to non-trivial __cinit__")             # <<<<<<<<<<<<<<
 */
  __pyx_t_1 = __Pyx_PyObject_Call(__pyx_builtin_TypeError, __pyx_tuple__65, NULL); if (unlikely(!__pyx_t_1)) __PYX_ERR(7, 4, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_1);
  __Pyx_Raise(__pyx_t_1, 0, 0, 0);
  __Pyx_DECREF(__pyx_t_1); __pyx_t_1 = 0;
  __PYX_ERR(7, 4, __pyx_L1_error)

  /* "(tree fragment)":3
 * def __reduce_cython__(self):
 *     raise TypeError("no default __reduce__ due to non-trivial __cinit__")
 * def __setstate_cython__(self, __pyx_state):             # <<<<<<<<<<<<<<
 *     raise TypeError("no default __reduce__ due to non-trivial __cinit__")
 */

  /* function exit code */
  __pyx_L1_error:;
  __Pyx_XDECREF(__pyx_t_1);
  __Pyx_AddTraceback("grpc._cython.cygrpc.ServerCertificateConfig.__setstate_cython__", __pyx_clineno, __pyx_lineno, __pyx_filename);
  __pyx_r = NULL;
  __Pyx_XGIVEREF(__pyx_r);
  __Pyx_RefNannyFinishContext();
  return __pyx_r;
}

/* "src/python/grpcio/grpc/_cython/_cygrpc/credentials.pyx.pxi":197
 * cdef class ServerCredentials:
 * 
 *   def __cinit__(self):             # <<<<<<<<<<<<<<
 *     fork_handlers_and_grpc_init()
 *     self.c_credentials = NULL
 */

/* Python wrapper */
static int __pyx_pw_4grpc_7_cython_6cygrpc_17ServerCredentials_1__cinit__(PyObject *__pyx_v_self, PyObject *__pyx_args, PyObject *__pyx_kwds); /*proto*/
static int __pyx_pw_4grpc_7_cython_6cygrpc_17ServerCredentials_1__cinit__(PyObject *__pyx_v_self, PyObject *__pyx_args, PyObject *__pyx_kwds) {
  int __pyx_r;
  __Pyx_RefNannyDeclarations
  __Pyx_RefNannySetupContext("__cinit__ (wrapper)", 0);
  if (unlikely(PyTuple_GET_SIZE(__pyx_args) > 0)) {
    __Pyx_RaiseArgtupleInvalid("__cinit__", 1, 0, 0, PyTuple_GET_SIZE(__pyx_args)); return -1;}
  if (unlikely(__pyx_kwds) && unlikely(PyDict_Size(__pyx_kwds) > 0) && unlikely(!__Pyx_CheckKeywordStrings(__pyx_kwds, "__cinit__", 0))) return -1;
  __pyx_r = __pyx_pf_4grpc_7_cython_6cygrpc_17ServerCredentials___cinit__(((struct __pyx_obj_4grpc_7_cython_6cygrpc_ServerCredentials *)__pyx_v_self));

  /* function exit code */
  __Pyx_RefNannyFinishContext();
  return __pyx_r;
}

static int __pyx_pf_4grpc_7_cython_6cygrpc_17ServerCredentials___cinit__(struct __pyx_obj_4grpc_7_cython_6cygrpc_ServerCredentials *__pyx_v_self) {
  int __pyx_r;
  __Pyx_RefNannyDeclarations
  PyObject *__pyx_t_1 = NULL;
  PyObject *__pyx_t_2 = NULL;
  PyObject *__pyx_t_3 = NULL;
  __Pyx_RefNannySetupContext("__cinit__", 0);

  /* "src/python/grpcio/grpc/_cython/_cygrpc/credentials.pyx.pxi":198
 * 
 *   def __cinit__(self):
 *     fork_handlers_and_grpc_init()             # <<<<<<<<<<<<<<
 *     self.c_credentials = NULL
 *     self.references = []
 */
  __pyx_t_2 = __Pyx_GetModuleGlobalName(__pyx_n_s_fork_handlers_and_grpc_init); if (unlikely(!__pyx_t_2)) __PYX_ERR(4, 198, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_2);
  __pyx_t_3 = NULL;
  if (CYTHON_UNPACK_METHODS && unlikely(PyMethod_Check(__pyx_t_2))) {
    __pyx_t_3 = PyMethod_GET_SELF(__pyx_t_2);
    if (likely(__pyx_t_3)) {
      PyObject* function = PyMethod_GET_FUNCTION(__pyx_t_2);
      __Pyx_INCREF(__pyx_t_3);
      __Pyx_INCREF(function);
      __Pyx_DECREF_SET(__pyx_t_2, function);
    }
  }
  if (__pyx_t_3) {
    __pyx_t_1 = __Pyx_PyObject_CallOneArg(__pyx_t_2, __pyx_t_3); if (unlikely(!__pyx_t_1)) __PYX_ERR(4, 198, __pyx_L1_error)
    __Pyx_DECREF(__pyx_t_3); __pyx_t_3 = 0;
  } else {
    __pyx_t_1 = __Pyx_PyObject_CallNoArg(__pyx_t_2); if (unlikely(!__pyx_t_1)) __PYX_ERR(4, 198, __pyx_L1_error)
  }
  __Pyx_GOTREF(__pyx_t_1);
  __Pyx_DECREF(__pyx_t_2); __pyx_t_2 = 0;
  __Pyx_DECREF(__pyx_t_1); __pyx_t_1 = 0;

  /* "src/python/grpcio/grpc/_cython/_cygrpc/credentials.pyx.pxi":199
 *   def __cinit__(self):
 *     fork_handlers_and_grpc_init()
 *     self.c_credentials = NULL             # <<<<<<<<<<<<<<
 *     self.references = []
 *     self.initial_cert_config = None
 */
  __pyx_v_self->c_credentials = NULL;

  /* "src/python/grpcio/grpc/_cython/_cygrpc/credentials.pyx.pxi":200
 *     fork_handlers_and_grpc_init()
 *     self.c_credentials = NULL
 *     self.references = []             # <<<<<<<<<<<<<<
 *     self.initial_cert_config = None
 *     self.cert_config_fetcher = None
 */
  __pyx_t_1 = PyList_New(0); if (unlikely(!__pyx_t_1)) __PYX_ERR(4, 200, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_1);
  __Pyx_GIVEREF(__pyx_t_1);
  __Pyx_GOTREF(__pyx_v_self->references);
  __Pyx_DECREF(__pyx_v_self->references);
  __pyx_v_self->references = ((PyObject*)__pyx_t_1);
  __pyx_t_1 = 0;

  /* "src/python/grpcio/grpc/_cython/_cygrpc/credentials.pyx.pxi":201
 *     self.c_credentials = NULL
 *     self.references = []
 *     self.initial_cert_config = None             # <<<<<<<<<<<<<<
 *     self.cert_config_fetcher = None
 *     self.initial_cert_config_fetched = False
 */
  __Pyx_INCREF(Py_None);
  __Pyx_GIVEREF(Py_None);
  __Pyx_GOTREF(__pyx_v_self->initial_cert_config);
  __Pyx_DECREF(__pyx_v_self->initial_cert_config);
  __pyx_v_self->initial_cert_config = Py_None;

  /* "src/python/grpcio/grpc/_cython/_cygrpc/credentials.pyx.pxi":202
 *     self.references = []
 *     self.initial_cert_config = None
 *     self.cert_config_fetcher = None             # <<<<<<<<<<<<<<
 *     self.initial_cert_config_fetched = False
 * 
 */
  __Pyx_INCREF(Py_None);
  __Pyx_GIVEREF(Py_None);
  __Pyx_GOTREF(__pyx_v_self->cert_config_fetcher);
  __Pyx_DECREF(__pyx_v_self->cert_config_fetcher);
  __pyx_v_self->cert_config_fetcher = Py_None;

  /* "src/python/grpcio/grpc/_cython/_cygrpc/credentials.pyx.pxi":203
 *     self.initial_cert_config = None
 *     self.cert_config_fetcher = None
 *     self.initial_cert_config_fetched = False             # <<<<<<<<<<<<<<
 * 
 *   def __dealloc__(self):
 */
  __pyx_v_self->initial_cert_config_fetched = 0;

  /* "src/python/grpcio/grpc/_cython/_cygrpc/credentials.pyx.pxi":197
 * cdef class ServerCredentials:
 * 
 *   def __cinit__(self):             # <<<<<<<<<<<<<<
 *     fork_handlers_and_grpc_init()
 *     self.c_credentials = NULL
 */

  /* function exit code */
  __pyx_r = 0;
  goto __pyx_L0;
  __pyx_L1_error:;
  __Pyx_XDECREF(__pyx_t_1);
  __Pyx_XDECREF(__pyx_t_2);
  __Pyx_XDECREF(__pyx_t_3);
  __Pyx_AddTraceback("grpc._cython.cygrpc.ServerCredentials.__cinit__", __pyx_clineno, __pyx_lineno, __pyx_filename);
  __pyx_r = -1;
  __pyx_L0:;
  __Pyx_RefNannyFinishContext();
  return __pyx_r;
}

/* "src/python/grpcio/grpc/_cython/_cygrpc/credentials.pyx.pxi":205
 *     self.initial_cert_config_fetched = False
 * 
 *   def __dealloc__(self):             # <<<<<<<<<<<<<<
 *     if self.c_credentials != NULL:
 *       grpc_server_credentials_release(self.c_credentials)
 */

/* Python wrapper */
static void __pyx_pw_4grpc_7_cython_6cygrpc_17ServerCredentials_3__dealloc__(PyObject *__pyx_v_self); /*proto*/
static void __pyx_pw_4grpc_7_cython_6cygrpc_17ServerCredentials_3__dealloc__(PyObject *__pyx_v_self) {
  __Pyx_RefNannyDeclarations
  __Pyx_RefNannySetupContext("__dealloc__ (wrapper)", 0);
  __pyx_pf_4grpc_7_cython_6cygrpc_17ServerCredentials_2__dealloc__(((struct __pyx_obj_4grpc_7_cython_6cygrpc_ServerCredentials *)__pyx_v_self));

  /* function exit code */
  __Pyx_RefNannyFinishContext();
}

static void __pyx_pf_4grpc_7_cython_6cygrpc_17ServerCredentials_2__dealloc__(struct __pyx_obj_4grpc_7_cython_6cygrpc_ServerCredentials *__pyx_v_self) {
  __Pyx_RefNannyDeclarations
  int __pyx_t_1;
  __Pyx_RefNannySetupContext("__dealloc__", 0);

  /* "src/python/grpcio/grpc/_cython/_cygrpc/credentials.pyx.pxi":206
 * 
 *   def __dealloc__(self):
 *     if self.c_credentials != NULL:             # <<<<<<<<<<<<<<
 *       grpc_server_credentials_release(self.c_credentials)
 *     grpc_shutdown_blocking()
 */
  __pyx_t_1 = ((__pyx_v_self->c_credentials != NULL) != 0);
  if (__pyx_t_1) {

    /* "src/python/grpcio/grpc/_cython/_cygrpc/credentials.pyx.pxi":207
 *   def __dealloc__(self):
 *     if self.c_credentials != NULL:
 *       grpc_server_credentials_release(self.c_credentials)             # <<<<<<<<<<<<<<
 *     grpc_shutdown_blocking()
 * 
 */
    grpc_server_credentials_release(__pyx_v_self->c_credentials);

    /* "src/python/grpcio/grpc/_cython/_cygrpc/credentials.pyx.pxi":206
 * 
 *   def __dealloc__(self):
 *     if self.c_credentials != NULL:             # <<<<<<<<<<<<<<
 *       grpc_server_credentials_release(self.c_credentials)
 *     grpc_shutdown_blocking()
 */
  }

  /* "src/python/grpcio/grpc/_cython/_cygrpc/credentials.pyx.pxi":208
 *     if self.c_credentials != NULL:
 *       grpc_server_credentials_release(self.c_credentials)
 *     grpc_shutdown_blocking()             # <<<<<<<<<<<<<<
 * 
 * cdef const char* _get_c_pem_root_certs(pem_root_certs):
 */
  grpc_shutdown_blocking();

  /* "src/python/grpcio/grpc/_cython/_cygrpc/credentials.pyx.pxi":205
 *     self.initial_cert_config_fetched = False
 * 
 *   def __dealloc__(self):             # <<<<<<<<<<<<<<
 *     if self.c_credentials != NULL:
 *       grpc_server_credentials_release(self.c_credentials)
 */

  /* function exit code */
  __Pyx_RefNannyFinishContext();
}

/* "(tree fragment)":1
 * def __reduce_cython__(self):             # <<<<<<<<<<<<<<
 *     raise TypeError("no default __reduce__ due to non-trivial __cinit__")
 * def __setstate_cython__(self, __pyx_state):
 */

/* Python wrapper */
static PyObject *__pyx_pw_4grpc_7_cython_6cygrpc_17ServerCredentials_5__reduce_cython__(PyObject *__pyx_v_self, CYTHON_UNUSED PyObject *unused); /*proto*/
static PyObject *__pyx_pw_4grpc_7_cython_6cygrpc_17ServerCredentials_5__reduce_cython__(PyObject *__pyx_v_self, CYTHON_UNUSED PyObject *unused) {
  PyObject *__pyx_r = 0;
  __Pyx_RefNannyDeclarations
  __Pyx_RefNannySetupContext("__reduce_cython__ (wrapper)", 0);
  __pyx_r = __pyx_pf_4grpc_7_cython_6cygrpc_17ServerCredentials_4__reduce_cython__(((struct __pyx_obj_4grpc_7_cython_6cygrpc_ServerCredentials *)__pyx_v_self));

  /* function exit code */
  __Pyx_RefNannyFinishContext();
  return __pyx_r;
}

static PyObject *__pyx_pf_4grpc_7_cython_6cygrpc_17ServerCredentials_4__reduce_cython__(CYTHON_UNUSED struct __pyx_obj_4grpc_7_cython_6cygrpc_ServerCredentials *__pyx_v_self) {
  PyObject *__pyx_r = NULL;
  __Pyx_RefNannyDeclarations
  PyObject *__pyx_t_1 = NULL;
  __Pyx_RefNannySetupContext("__reduce_cython__", 0);

  /* "(tree fragment)":2
 * def __reduce_cython__(self):
 *     raise TypeError("no default __reduce__ due to non-trivial __cinit__")             # <<<<<<<<<<<<<<
 * def __setstate_cython__(self, __pyx_state):
 *     raise TypeError("no default __reduce__ due to non-trivial __cinit__")
 */
  __pyx_t_1 = __Pyx_PyObject_Call(__pyx_builtin_TypeError, __pyx_tuple__66, NULL); if (unlikely(!__pyx_t_1)) __PYX_ERR(7, 2, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_1);
  __Pyx_Raise(__pyx_t_1, 0, 0, 0);
  __Pyx_DECREF(__pyx_t_1); __pyx_t_1 = 0;
  __PYX_ERR(7, 2, __pyx_L1_error)

  /* "(tree fragment)":1
 * def __reduce_cython__(self):             # <<<<<<<<<<<<<<
 *     raise TypeError("no default __reduce__ due to non-trivial __cinit__")
 * def __setstate_cython__(self, __pyx_state):
 */

  /* function exit code */
  __pyx_L1_error:;
  __Pyx_XDECREF(__pyx_t_1);
  __Pyx_AddTraceback("grpc._cython.cygrpc.ServerCredentials.__reduce_cython__", __pyx_clineno, __pyx_lineno, __pyx_filename);
  __pyx_r = NULL;
  __Pyx_XGIVEREF(__pyx_r);
  __Pyx_RefNannyFinishContext();
  return __pyx_r;
}

/* "(tree fragment)":3
 * def __reduce_cython__(self):
 *     raise TypeError("no default __reduce__ due to non-trivial __cinit__")
 * def __setstate_cython__(self, __pyx_state):             # <<<<<<<<<<<<<<
 *     raise TypeError("no default __reduce__ due to non-trivial __cinit__")
 */

/* Python wrapper */
static PyObject *__pyx_pw_4grpc_7_cython_6cygrpc_17ServerCredentials_7__setstate_cython__(PyObject *__pyx_v_self, PyObject *__pyx_v___pyx_state); /*proto*/
static PyObject *__pyx_pw_4grpc_7_cython_6cygrpc_17ServerCredentials_7__setstate_cython__(PyObject *__pyx_v_self, PyObject *__pyx_v___pyx_state) {
  PyObject *__pyx_r = 0;
  __Pyx_RefNannyDeclarations
  __Pyx_RefNannySetupContext("__setstate_cython__ (wrapper)", 0);
  __pyx_r = __pyx_pf_4grpc_7_cython_6cygrpc_17ServerCredentials_6__setstate_cython__(((struct __pyx_obj_4grpc_7_cython_6cygrpc_ServerCredentials *)__pyx_v_self), ((PyObject *)__pyx_v___pyx_state));

  /* function exit code */
  __Pyx_RefNannyFinishContext();
  return __pyx_r;
}

static PyObject *__pyx_pf_4grpc_7_cython_6cygrpc_17ServerCredentials_6__setstate_cython__(CYTHON_UNUSED struct __pyx_obj_4grpc_7_cython_6cygrpc_ServerCredentials *__pyx_v_self, CYTHON_UNUSED PyObject *__pyx_v___pyx_state) {
  PyObject *__pyx_r = NULL;
  __Pyx_RefNannyDeclarations
  PyObject *__pyx_t_1 = NULL;
  __Pyx_RefNannySetupContext("__setstate_cython__", 0);

  /* "(tree fragment)":4
 *     raise TypeError("no default __reduce__ due to non-trivial __cinit__")
 * def __setstate_cython__(self, __pyx_state):
 *     raise TypeError("no default __reduce__ due to non-trivial __cinit__")             # <<<<<<<<<<<<<<
 */
  __pyx_t_1 = __Pyx_PyObject_Call(__pyx_builtin_TypeError, __pyx_tuple__67, NULL); if (unlikely(!__pyx_t_1)) __PYX_ERR(7, 4, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_1);
  __Pyx_Raise(__pyx_t_1, 0, 0, 0);
  __Pyx_DECREF(__pyx_t_1); __pyx_t_1 = 0;
  __PYX_ERR(7, 4, __pyx_L1_error)

  /* "(tree fragment)":3
 * def __reduce_cython__(self):
 *     raise TypeError("no default __reduce__ due to non-trivial __cinit__")
 * def __setstate_cython__(self, __pyx_state):             # <<<<<<<<<<<<<<
 *     raise TypeError("no default __reduce__ due to non-trivial __cinit__")
 */

  /* function exit code */
  __pyx_L1_error:;
  __Pyx_XDECREF(__pyx_t_1);
  __Pyx_AddTraceback("grpc._cython.cygrpc.ServerCredentials.__setstate_cython__", __pyx_clineno, __pyx_lineno, __pyx_filename);
  __pyx_r = NULL;
  __Pyx_XGIVEREF(__pyx_r);
  __Pyx_RefNannyFinishContext();
  return __pyx_r;
}

/* "src/python/grpcio/grpc/_cython/_cygrpc/credentials.pyx.pxi":210
 *     grpc_shutdown_blocking()
 * 
 * cdef const char* _get_c_pem_root_certs(pem_root_certs):             # <<<<<<<<<<<<<<
 *   if pem_root_certs is None:
 *     return NULL
 */

static char const *__pyx_f_4grpc_7_cython_6cygrpc__get_c_pem_root_certs(PyObject *__pyx_v_pem_root_certs) {
  char const *__pyx_r;
  __Pyx_RefNannyDeclarations
  int __pyx_t_1;
  int __pyx_t_2;
  char const *__pyx_t_3;
  __Pyx_RefNannySetupContext("_get_c_pem_root_certs", 0);

  /* "src/python/grpcio/grpc/_cython/_cygrpc/credentials.pyx.pxi":211
 * 
 * cdef const char* _get_c_pem_root_certs(pem_root_certs):
 *   if pem_root_certs is None:             # <<<<<<<<<<<<<<
 *     return NULL
 *   else:
 */
  __pyx_t_1 = (__pyx_v_pem_root_certs == Py_None);
  __pyx_t_2 = (__pyx_t_1 != 0);
  if (__pyx_t_2) {

    /* "src/python/grpcio/grpc/_cython/_cygrpc/credentials.pyx.pxi":212
 * cdef const char* _get_c_pem_root_certs(pem_root_certs):
 *   if pem_root_certs is None:
 *     return NULL             # <<<<<<<<<<<<<<
 *   else:
 *     return pem_root_certs
 */
    __pyx_r = NULL;
    goto __pyx_L0;

    /* "src/python/grpcio/grpc/_cython/_cygrpc/credentials.pyx.pxi":211
 * 
 * cdef const char* _get_c_pem_root_certs(pem_root_certs):
 *   if pem_root_certs is None:             # <<<<<<<<<<<<<<
 *     return NULL
 *   else:
 */
  }

  /* "src/python/grpcio/grpc/_cython/_cygrpc/credentials.pyx.pxi":214
 *     return NULL
 *   else:
 *     return pem_root_certs             # <<<<<<<<<<<<<<
 * 
 * cdef grpc_ssl_pem_key_cert_pair* _create_c_ssl_pem_key_cert_pairs(pem_key_cert_pairs):
 */
  /*else*/ {
    __pyx_t_3 = __Pyx_PyObject_AsString(__pyx_v_pem_root_certs); if (unlikely((!__pyx_t_3) && PyErr_Occurred())) __PYX_ERR(4, 214, __pyx_L1_error)
    __pyx_r = __pyx_t_3;
    goto __pyx_L0;
  }

  /* "src/python/grpcio/grpc/_cython/_cygrpc/credentials.pyx.pxi":210
 *     grpc_shutdown_blocking()
 * 
 * cdef const char* _get_c_pem_root_certs(pem_root_certs):             # <<<<<<<<<<<<<<
 *   if pem_root_certs is None:
 *     return NULL
 */

  /* function exit code */
  __pyx_L1_error:;
  __Pyx_WriteUnraisable("grpc._cython.cygrpc._get_c_pem_root_certs", __pyx_clineno, __pyx_lineno, __pyx_filename, 1, 0);
  __pyx_r = 0;
  __pyx_L0:;
  __Pyx_RefNannyFinishContext();
  return __pyx_r;
}

/* "src/python/grpcio/grpc/_cython/_cygrpc/credentials.pyx.pxi":216
 *     return pem_root_certs
 * 
 * cdef grpc_ssl_pem_key_cert_pair* _create_c_ssl_pem_key_cert_pairs(pem_key_cert_pairs):             # <<<<<<<<<<<<<<
 *   # return a malloc'ed grpc_ssl_pem_key_cert_pair from a _list_ of SslPemKeyCertPair
 *   for pair in pem_key_cert_pairs:
 */

static grpc_ssl_pem_key_cert_pair *__pyx_f_4grpc_7_cython_6cygrpc__create_c_ssl_pem_key_cert_pairs(PyObject *__pyx_v_pem_key_cert_pairs) {
  PyObject *__pyx_v_pair = NULL;
  size_t __pyx_v_c_ssl_pem_key_cert_pairs_count;
  grpc_ssl_pem_key_cert_pair *__pyx_v_c_ssl_pem_key_cert_pairs;
  size_t __pyx_v_i;
  grpc_ssl_pem_key_cert_pair *__pyx_r;
  __Pyx_RefNannyDeclarations
  PyObject *__pyx_t_1 = NULL;
  Py_ssize_t __pyx_t_2;
  PyObject *(*__pyx_t_3)(PyObject *);
  PyObject *__pyx_t_4 = NULL;
  int __pyx_t_5;
  int __pyx_t_6;
  size_t __pyx_t_7;
  size_t __pyx_t_8;
  size_t __pyx_t_9;
  grpc_ssl_pem_key_cert_pair __pyx_t_10;
  __Pyx_RefNannySetupContext("_create_c_ssl_pem_key_cert_pairs", 0);

  /* "src/python/grpcio/grpc/_cython/_cygrpc/credentials.pyx.pxi":218
 * cdef grpc_ssl_pem_key_cert_pair* _create_c_ssl_pem_key_cert_pairs(pem_key_cert_pairs):
 *   # return a malloc'ed grpc_ssl_pem_key_cert_pair from a _list_ of SslPemKeyCertPair
 *   for pair in pem_key_cert_pairs:             # <<<<<<<<<<<<<<
 *     if not isinstance(pair, SslPemKeyCertPair):
 *       raise TypeError("expected pem_key_cert_pairs to be sequence of "
 */
  if (likely(PyList_CheckExact(__pyx_v_pem_key_cert_pairs)) || PyTuple_CheckExact(__pyx_v_pem_key_cert_pairs)) {
    __pyx_t_1 = __pyx_v_pem_key_cert_pairs; __Pyx_INCREF(__pyx_t_1); __pyx_t_2 = 0;
    __pyx_t_3 = NULL;
  } else {
    __pyx_t_2 = -1; __pyx_t_1 = PyObject_GetIter(__pyx_v_pem_key_cert_pairs); if (unlikely(!__pyx_t_1)) __PYX_ERR(4, 218, __pyx_L1_error)
    __Pyx_GOTREF(__pyx_t_1);
    __pyx_t_3 = Py_TYPE(__pyx_t_1)->tp_iternext; if (unlikely(!__pyx_t_3)) __PYX_ERR(4, 218, __pyx_L1_error)
  }
  for (;;) {
    if (likely(!__pyx_t_3)) {
      if (likely(PyList_CheckExact(__pyx_t_1))) {
        if (__pyx_t_2 >= PyList_GET_SIZE(__pyx_t_1)) break;
        #if CYTHON_ASSUME_SAFE_MACROS && !CYTHON_AVOID_BORROWED_REFS
        __pyx_t_4 = PyList_GET_ITEM(__pyx_t_1, __pyx_t_2); __Pyx_INCREF(__pyx_t_4); __pyx_t_2++; if (unlikely(0 < 0)) __PYX_ERR(4, 218, __pyx_L1_error)
        #else
        __pyx_t_4 = PySequence_ITEM(__pyx_t_1, __pyx_t_2); __pyx_t_2++; if (unlikely(!__pyx_t_4)) __PYX_ERR(4, 218, __pyx_L1_error)
        __Pyx_GOTREF(__pyx_t_4);
        #endif
      } else {
        if (__pyx_t_2 >= PyTuple_GET_SIZE(__pyx_t_1)) break;
        #if CYTHON_ASSUME_SAFE_MACROS && !CYTHON_AVOID_BORROWED_REFS
        __pyx_t_4 = PyTuple_GET_ITEM(__pyx_t_1, __pyx_t_2); __Pyx_INCREF(__pyx_t_4); __pyx_t_2++; if (unlikely(0 < 0)) __PYX_ERR(4, 218, __pyx_L1_error)
        #else
        __pyx_t_4 = PySequence_ITEM(__pyx_t_1, __pyx_t_2); __pyx_t_2++; if (unlikely(!__pyx_t_4)) __PYX_ERR(4, 218, __pyx_L1_error)
        __Pyx_GOTREF(__pyx_t_4);
        #endif
      }
    } else {
      __pyx_t_4 = __pyx_t_3(__pyx_t_1);
      if (unlikely(!__pyx_t_4)) {
        PyObject* exc_type = PyErr_Occurred();
        if (exc_type) {
          if (likely(__Pyx_PyErr_GivenExceptionMatches(exc_type, PyExc_StopIteration))) PyErr_Clear();
          else __PYX_ERR(4, 218, __pyx_L1_error)
        }
        break;
      }
      __Pyx_GOTREF(__pyx_t_4);
    }
    __Pyx_XDECREF_SET(__pyx_v_pair, __pyx_t_4);
    __pyx_t_4 = 0;

    /* "src/python/grpcio/grpc/_cython/_cygrpc/credentials.pyx.pxi":219
 *   # return a malloc'ed grpc_ssl_pem_key_cert_pair from a _list_ of SslPemKeyCertPair
 *   for pair in pem_key_cert_pairs:
 *     if not isinstance(pair, SslPemKeyCertPair):             # <<<<<<<<<<<<<<
 *       raise TypeError("expected pem_key_cert_pairs to be sequence of "
 *                       "SslPemKeyCertPair")
 */
    __pyx_t_5 = __Pyx_TypeCheck(__pyx_v_pair, __pyx_ptype_4grpc_7_cython_6cygrpc_SslPemKeyCertPair); 
    __pyx_t_6 = ((!(__pyx_t_5 != 0)) != 0);
    if (unlikely(__pyx_t_6)) {

      /* "src/python/grpcio/grpc/_cython/_cygrpc/credentials.pyx.pxi":220
 *   for pair in pem_key_cert_pairs:
 *     if not isinstance(pair, SslPemKeyCertPair):
 *       raise TypeError("expected pem_key_cert_pairs to be sequence of "             # <<<<<<<<<<<<<<
 *                       "SslPemKeyCertPair")
 *   cdef size_t c_ssl_pem_key_cert_pairs_count = len(pem_key_cert_pairs)
 */
      __pyx_t_4 = __Pyx_PyObject_Call(__pyx_builtin_TypeError, __pyx_tuple__68, NULL); if (unlikely(!__pyx_t_4)) __PYX_ERR(4, 220, __pyx_L1_error)
      __Pyx_GOTREF(__pyx_t_4);
      __Pyx_Raise(__pyx_t_4, 0, 0, 0);
      __Pyx_DECREF(__pyx_t_4); __pyx_t_4 = 0;
      __PYX_ERR(4, 220, __pyx_L1_error)

      /* "src/python/grpcio/grpc/_cython/_cygrpc/credentials.pyx.pxi":219
 *   # return a malloc'ed grpc_ssl_pem_key_cert_pair from a _list_ of SslPemKeyCertPair
 *   for pair in pem_key_cert_pairs:
 *     if not isinstance(pair, SslPemKeyCertPair):             # <<<<<<<<<<<<<<
 *       raise TypeError("expected pem_key_cert_pairs to be sequence of "
 *                       "SslPemKeyCertPair")
 */
    }

    /* "src/python/grpcio/grpc/_cython/_cygrpc/credentials.pyx.pxi":218
 * cdef grpc_ssl_pem_key_cert_pair* _create_c_ssl_pem_key_cert_pairs(pem_key_cert_pairs):
 *   # return a malloc'ed grpc_ssl_pem_key_cert_pair from a _list_ of SslPemKeyCertPair
 *   for pair in pem_key_cert_pairs:             # <<<<<<<<<<<<<<
 *     if not isinstance(pair, SslPemKeyCertPair):
 *       raise TypeError("expected pem_key_cert_pairs to be sequence of "
 */
  }
  __Pyx_DECREF(__pyx_t_1); __pyx_t_1 = 0;

  /* "src/python/grpcio/grpc/_cython/_cygrpc/credentials.pyx.pxi":222
 *       raise TypeError("expected pem_key_cert_pairs to be sequence of "
 *                       "SslPemKeyCertPair")
 *   cdef size_t c_ssl_pem_key_cert_pairs_count = len(pem_key_cert_pairs)             # <<<<<<<<<<<<<<
 *   cdef grpc_ssl_pem_key_cert_pair* c_ssl_pem_key_cert_pairs = NULL
 *   with nogil:
 */
  __pyx_t_2 = PyObject_Length(__pyx_v_pem_key_cert_pairs); if (unlikely(__pyx_t_2 == ((Py_ssize_t)-1))) __PYX_ERR(4, 222, __pyx_L1_error)
  __pyx_v_c_ssl_pem_key_cert_pairs_count = __pyx_t_2;

  /* "src/python/grpcio/grpc/_cython/_cygrpc/credentials.pyx.pxi":223
 *                       "SslPemKeyCertPair")
 *   cdef size_t c_ssl_pem_key_cert_pairs_count = len(pem_key_cert_pairs)
 *   cdef grpc_ssl_pem_key_cert_pair* c_ssl_pem_key_cert_pairs = NULL             # <<<<<<<<<<<<<<
 *   with nogil:
 *     c_ssl_pem_key_cert_pairs = (
 */
  __pyx_v_c_ssl_pem_key_cert_pairs = NULL;

  /* "src/python/grpcio/grpc/_cython/_cygrpc/credentials.pyx.pxi":224
 *   cdef size_t c_ssl_pem_key_cert_pairs_count = len(pem_key_cert_pairs)
 *   cdef grpc_ssl_pem_key_cert_pair* c_ssl_pem_key_cert_pairs = NULL
 *   with nogil:             # <<<<<<<<<<<<<<
 *     c_ssl_pem_key_cert_pairs = (
 *       <grpc_ssl_pem_key_cert_pair *>gpr_malloc(
 */
  {
      #ifdef WITH_THREAD
      PyThreadState *_save;
      Py_UNBLOCK_THREADS
      __Pyx_FastGIL_Remember();
      #endif
      /*try:*/ {

        /* "src/python/grpcio/grpc/_cython/_cygrpc/credentials.pyx.pxi":226
 *   with nogil:
 *     c_ssl_pem_key_cert_pairs = (
 *       <grpc_ssl_pem_key_cert_pair *>gpr_malloc(             # <<<<<<<<<<<<<<
 *         sizeof(grpc_ssl_pem_key_cert_pair) * c_ssl_pem_key_cert_pairs_count))
 *   for i in range(c_ssl_pem_key_cert_pairs_count):
 */
        __pyx_v_c_ssl_pem_key_cert_pairs = ((grpc_ssl_pem_key_cert_pair *)gpr_malloc(((sizeof(grpc_ssl_pem_key_cert_pair)) * __pyx_v_c_ssl_pem_key_cert_pairs_count)));
      }

      /* "src/python/grpcio/grpc/_cython/_cygrpc/credentials.pyx.pxi":224
 *   cdef size_t c_ssl_pem_key_cert_pairs_count = len(pem_key_cert_pairs)
 *   cdef grpc_ssl_pem_key_cert_pair* c_ssl_pem_key_cert_pairs = NULL
 *   with nogil:             # <<<<<<<<<<<<<<
 *     c_ssl_pem_key_cert_pairs = (
 *       <grpc_ssl_pem_key_cert_pair *>gpr_malloc(
 */
      /*finally:*/ {
        /*normal exit:*/{
          #ifdef WITH_THREAD
          __Pyx_FastGIL_Forget();
          Py_BLOCK_THREADS
          #endif
          goto __pyx_L8;
        }
        __pyx_L8:;
      }
  }

  /* "src/python/grpcio/grpc/_cython/_cygrpc/credentials.pyx.pxi":228
 *       <grpc_ssl_pem_key_cert_pair *>gpr_malloc(
 *         sizeof(grpc_ssl_pem_key_cert_pair) * c_ssl_pem_key_cert_pairs_count))
 *   for i in range(c_ssl_pem_key_cert_pairs_count):             # <<<<<<<<<<<<<<
 *     c_ssl_pem_key_cert_pairs[i] = (
 *       (<SslPemKeyCertPair>pem_key_cert_pairs[i]).c_pair)
 */
  __pyx_t_7 = __pyx_v_c_ssl_pem_key_cert_pairs_count;
  __pyx_t_8 = __pyx_t_7;
  for (__pyx_t_9 = 0; __pyx_t_9 < __pyx_t_8; __pyx_t_9+=1) {
    __pyx_v_i = __pyx_t_9;

    /* "src/python/grpcio/grpc/_cython/_cygrpc/credentials.pyx.pxi":230
 *   for i in range(c_ssl_pem_key_cert_pairs_count):
 *     c_ssl_pem_key_cert_pairs[i] = (
 *       (<SslPemKeyCertPair>pem_key_cert_pairs[i]).c_pair)             # <<<<<<<<<<<<<<
 *   return c_ssl_pem_key_cert_pairs
 * 
 */
    __pyx_t_1 = __Pyx_GetItemInt(__pyx_v_pem_key_cert_pairs, __pyx_v_i, size_t, 0, __Pyx_PyInt_FromSize_t, 0, 0, 1); if (unlikely(!__pyx_t_1)) __PYX_ERR(4, 230, __pyx_L1_error)
    __Pyx_GOTREF(__pyx_t_1);
    __pyx_t_10 = ((struct __pyx_obj_4grpc_7_cython_6cygrpc_SslPemKeyCertPair *)__pyx_t_1)->c_pair;
    __Pyx_DECREF(__pyx_t_1); __pyx_t_1 = 0;

    /* "src/python/grpcio/grpc/_cython/_cygrpc/credentials.pyx.pxi":229
 *         sizeof(grpc_ssl_pem_key_cert_pair) * c_ssl_pem_key_cert_pairs_count))
 *   for i in range(c_ssl_pem_key_cert_pairs_count):
 *     c_ssl_pem_key_cert_pairs[i] = (             # <<<<<<<<<<<<<<
 *       (<SslPemKeyCertPair>pem_key_cert_pairs[i]).c_pair)
 *   return c_ssl_pem_key_cert_pairs
 */
    (__pyx_v_c_ssl_pem_key_cert_pairs[__pyx_v_i]) = __pyx_t_10;
  }

  /* "src/python/grpcio/grpc/_cython/_cygrpc/credentials.pyx.pxi":231
 *     c_ssl_pem_key_cert_pairs[i] = (
 *       (<SslPemKeyCertPair>pem_key_cert_pairs[i]).c_pair)
 *   return c_ssl_pem_key_cert_pairs             # <<<<<<<<<<<<<<
 * 
 * def server_credentials_ssl(pem_root_certs, pem_key_cert_pairs,
 */
  __pyx_r = __pyx_v_c_ssl_pem_key_cert_pairs;
  goto __pyx_L0;

  /* "src/python/grpcio/grpc/_cython/_cygrpc/credentials.pyx.pxi":216
 *     return pem_root_certs
 * 
 * cdef grpc_ssl_pem_key_cert_pair* _create_c_ssl_pem_key_cert_pairs(pem_key_cert_pairs):             # <<<<<<<<<<<<<<
 *   # return a malloc'ed grpc_ssl_pem_key_cert_pair from a _list_ of SslPemKeyCertPair
 *   for pair in pem_key_cert_pairs:
 */

  /* function exit code */
  __pyx_L1_error:;
  __Pyx_XDECREF(__pyx_t_1);
  __Pyx_XDECREF(__pyx_t_4);
  __Pyx_WriteUnraisable("grpc._cython.cygrpc._create_c_ssl_pem_key_cert_pairs", __pyx_clineno, __pyx_lineno, __pyx_filename, 1, 0);
  __pyx_r = 0;
  __pyx_L0:;
  __Pyx_XDECREF(__pyx_v_pair);
  __Pyx_RefNannyFinishContext();
  return __pyx_r;
}

/* "src/python/grpcio/grpc/_cython/_cygrpc/credentials.pyx.pxi":233
 *   return c_ssl_pem_key_cert_pairs
 * 
 * def server_credentials_ssl(pem_root_certs, pem_key_cert_pairs,             # <<<<<<<<<<<<<<
 *                            bint force_client_auth):
 *   pem_root_certs = str_to_bytes(pem_root_certs)
 */

/* Python wrapper */
static PyObject *__pyx_pw_4grpc_7_cython_6cygrpc_21server_credentials_ssl(PyObject *__pyx_self, PyObject *__pyx_args, PyObject *__pyx_kwds); /*proto*/
static PyMethodDef __pyx_mdef_4grpc_7_cython_6cygrpc_21server_credentials_ssl = {"server_credentials_ssl", (PyCFunction)__pyx_pw_4grpc_7_cython_6cygrpc_21server_credentials_ssl, METH_VARARGS|METH_KEYWORDS, 0};
static PyObject *__pyx_pw_4grpc_7_cython_6cygrpc_21server_credentials_ssl(PyObject *__pyx_self, PyObject *__pyx_args, PyObject *__pyx_kwds) {
  PyObject *__pyx_v_pem_root_certs = 0;
  PyObject *__pyx_v_pem_key_cert_pairs = 0;
  int __pyx_v_force_client_auth;
  PyObject *__pyx_r = 0;
  __Pyx_RefNannyDeclarations
  __Pyx_RefNannySetupContext("server_credentials_ssl (wrapper)", 0);
  {
    static PyObject **__pyx_pyargnames[] = {&__pyx_n_s_pem_root_certs,&__pyx_n_s_pem_key_cert_pairs,&__pyx_n_s_force_client_auth,0};
    PyObject* values[3] = {0,0,0};
    if (unlikely(__pyx_kwds)) {
      Py_ssize_t kw_args;
      const Py_ssize_t pos_args = PyTuple_GET_SIZE(__pyx_args);
      switch (pos_args) {
        case  3: values[2] = PyTuple_GET_ITEM(__pyx_args, 2);
        CYTHON_FALLTHROUGH;
        case  2: values[1] = PyTuple_GET_ITEM(__pyx_args, 1);
        CYTHON_FALLTHROUGH;
        case  1: values[0] = PyTuple_GET_ITEM(__pyx_args, 0);
        CYTHON_FALLTHROUGH;
        case  0: break;
        default: goto __pyx_L5_argtuple_error;
      }
      kw_args = PyDict_Size(__pyx_kwds);
      switch (pos_args) {
        case  0:
        if (likely((values[0] = __Pyx_PyDict_GetItemStr(__pyx_kwds, __pyx_n_s_pem_root_certs)) != 0)) kw_args--;
        else goto __pyx_L5_argtuple_error;
        CYTHON_FALLTHROUGH;
        case  1:
        if (likely((values[1] = __Pyx_PyDict_GetItemStr(__pyx_kwds, __pyx_n_s_pem_key_cert_pairs)) != 0)) kw_args--;
        else {
          __Pyx_RaiseArgtupleInvalid("server_credentials_ssl", 1, 3, 3, 1); __PYX_ERR(4, 233, __pyx_L3_error)
        }
        CYTHON_FALLTHROUGH;
        case  2:
        if (likely((values[2] = __Pyx_PyDict_GetItemStr(__pyx_kwds, __pyx_n_s_force_client_auth)) != 0)) kw_args--;
        else {
          __Pyx_RaiseArgtupleInvalid("server_credentials_ssl", 1, 3, 3, 2); __PYX_ERR(4, 233, __pyx_L3_error)
        }
      }
      if (unlikely(kw_args > 0)) {
        if (unlikely(__Pyx_ParseOptionalKeywords(__pyx_kwds, __pyx_pyargnames, 0, values, pos_args, "server_credentials_ssl") < 0)) __PYX_ERR(4, 233, __pyx_L3_error)
      }
    } else if (PyTuple_GET_SIZE(__pyx_args) != 3) {
      goto __pyx_L5_argtuple_error;
    } else {
      values[0] = PyTuple_GET_ITEM(__pyx_args, 0);
      values[1] = PyTuple_GET_ITEM(__pyx_args, 1);
      values[2] = PyTuple_GET_ITEM(__pyx_args, 2);
    }
    __pyx_v_pem_root_certs = values[0];
    __pyx_v_pem_key_cert_pairs = values[1];
    __pyx_v_force_client_auth = __Pyx_PyObject_IsTrue(values[2]); if (unlikely((__pyx_v_force_client_auth == (int)-1) && PyErr_Occurred())) __PYX_ERR(4, 234, __pyx_L3_error)
  }
  goto __pyx_L4_argument_unpacking_done;
  __pyx_L5_argtuple_error:;
  __Pyx_RaiseArgtupleInvalid("server_credentials_ssl", 1, 3, 3, PyTuple_GET_SIZE(__pyx_args)); __PYX_ERR(4, 233, __pyx_L3_error)
  __pyx_L3_error:;
  __Pyx_AddTraceback("grpc._cython.cygrpc.server_credentials_ssl", __pyx_clineno, __pyx_lineno, __pyx_filename);
  __Pyx_RefNannyFinishContext();
  return NULL;
  __pyx_L4_argument_unpacking_done:;
  __pyx_r = __pyx_pf_4grpc_7_cython_6cygrpc_20server_credentials_ssl(__pyx_self, __pyx_v_pem_root_certs, __pyx_v_pem_key_cert_pairs, __pyx_v_force_client_auth);

  /* function exit code */
  __Pyx_RefNannyFinishContext();
  return __pyx_r;
}

static PyObject *__pyx_pf_4grpc_7_cython_6cygrpc_20server_credentials_ssl(CYTHON_UNUSED PyObject *__pyx_self, PyObject *__pyx_v_pem_root_certs, PyObject *__pyx_v_pem_key_cert_pairs, int __pyx_v_force_client_auth) {
  struct __pyx_obj_4grpc_7_cython_6cygrpc_ServerCredentials *__pyx_v_credentials = 0;
  char const *__pyx_v_c_pem_root_certs;
  grpc_ssl_server_certificate_config *__pyx_v_c_cert_config;
  grpc_ssl_server_credentials_options *__pyx_v_c_options;
  PyObject *__pyx_r = NULL;
  __Pyx_RefNannyDeclarations
  PyObject *__pyx_t_1 = NULL;
  int __pyx_t_2;
  Py_ssize_t __pyx_t_3;
  grpc_ssl_client_certificate_request_type __pyx_t_4;
  __Pyx_RefNannySetupContext("server_credentials_ssl", 0);
  __Pyx_INCREF(__pyx_v_pem_root_certs);
  __Pyx_INCREF(__pyx_v_pem_key_cert_pairs);

  /* "src/python/grpcio/grpc/_cython/_cygrpc/credentials.pyx.pxi":235
 * def server_credentials_ssl(pem_root_certs, pem_key_cert_pairs,
 *                            bint force_client_auth):
 *   pem_root_certs = str_to_bytes(pem_root_certs)             # <<<<<<<<<<<<<<
 *   pem_key_cert_pairs = list(pem_key_cert_pairs)
 *   cdef ServerCredentials credentials = ServerCredentials()
 */
  __pyx_t_1 = __pyx_f_4grpc_7_cython_6cygrpc_str_to_bytes(__pyx_v_pem_root_certs); if (unlikely(!__pyx_t_1)) __PYX_ERR(4, 235, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_1);
  __Pyx_DECREF_SET(__pyx_v_pem_root_certs, __pyx_t_1);
  __pyx_t_1 = 0;

  /* "src/python/grpcio/grpc/_cython/_cygrpc/credentials.pyx.pxi":236
 *                            bint force_client_auth):
 *   pem_root_certs = str_to_bytes(pem_root_certs)
 *   pem_key_cert_pairs = list(pem_key_cert_pairs)             # <<<<<<<<<<<<<<
 *   cdef ServerCredentials credentials = ServerCredentials()
 *   credentials.references.append(pem_root_certs)
 */
  __pyx_t_1 = PySequence_List(__pyx_v_pem_key_cert_pairs); if (unlikely(!__pyx_t_1)) __PYX_ERR(4, 236, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_1);
  __Pyx_DECREF_SET(__pyx_v_pem_key_cert_pairs, __pyx_t_1);
  __pyx_t_1 = 0;

  /* "src/python/grpcio/grpc/_cython/_cygrpc/credentials.pyx.pxi":237
 *   pem_root_certs = str_to_bytes(pem_root_certs)
 *   pem_key_cert_pairs = list(pem_key_cert_pairs)
 *   cdef ServerCredentials credentials = ServerCredentials()             # <<<<<<<<<<<<<<
 *   credentials.references.append(pem_root_certs)
 *   credentials.references.append(pem_key_cert_pairs)
 */
  __pyx_t_1 = __Pyx_PyObject_CallNoArg(((PyObject *)__pyx_ptype_4grpc_7_cython_6cygrpc_ServerCredentials)); if (unlikely(!__pyx_t_1)) __PYX_ERR(4, 237, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_1);
  __pyx_v_credentials = ((struct __pyx_obj_4grpc_7_cython_6cygrpc_ServerCredentials *)__pyx_t_1);
  __pyx_t_1 = 0;

  /* "src/python/grpcio/grpc/_cython/_cygrpc/credentials.pyx.pxi":238
 *   pem_key_cert_pairs = list(pem_key_cert_pairs)
 *   cdef ServerCredentials credentials = ServerCredentials()
 *   credentials.references.append(pem_root_certs)             # <<<<<<<<<<<<<<
 *   credentials.references.append(pem_key_cert_pairs)
 *   cdef const char * c_pem_root_certs = _get_c_pem_root_certs(pem_root_certs)
 */
  if (unlikely(__pyx_v_credentials->references == Py_None)) {
    PyErr_Format(PyExc_AttributeError, "'NoneType' object has no attribute '%.30s'", "append");
    __PYX_ERR(4, 238, __pyx_L1_error)
  }
  __pyx_t_2 = __Pyx_PyList_Append(__pyx_v_credentials->references, __pyx_v_pem_root_certs); if (unlikely(__pyx_t_2 == ((int)-1))) __PYX_ERR(4, 238, __pyx_L1_error)

  /* "src/python/grpcio/grpc/_cython/_cygrpc/credentials.pyx.pxi":239
 *   cdef ServerCredentials credentials = ServerCredentials()
 *   credentials.references.append(pem_root_certs)
 *   credentials.references.append(pem_key_cert_pairs)             # <<<<<<<<<<<<<<
 *   cdef const char * c_pem_root_certs = _get_c_pem_root_certs(pem_root_certs)
 *   credentials.c_ssl_pem_key_cert_pairs_count = len(pem_key_cert_pairs)
 */
  if (unlikely(__pyx_v_credentials->references == Py_None)) {
    PyErr_Format(PyExc_AttributeError, "'NoneType' object has no attribute '%.30s'", "append");
    __PYX_ERR(4, 239, __pyx_L1_error)
  }
  __pyx_t_2 = __Pyx_PyList_Append(__pyx_v_credentials->references, __pyx_v_pem_key_cert_pairs); if (unlikely(__pyx_t_2 == ((int)-1))) __PYX_ERR(4, 239, __pyx_L1_error)

  /* "src/python/grpcio/grpc/_cython/_cygrpc/credentials.pyx.pxi":240
 *   credentials.references.append(pem_root_certs)
 *   credentials.references.append(pem_key_cert_pairs)
 *   cdef const char * c_pem_root_certs = _get_c_pem_root_certs(pem_root_certs)             # <<<<<<<<<<<<<<
 *   credentials.c_ssl_pem_key_cert_pairs_count = len(pem_key_cert_pairs)
 *   credentials.c_ssl_pem_key_cert_pairs = _create_c_ssl_pem_key_cert_pairs(pem_key_cert_pairs)
 */
  __pyx_v_c_pem_root_certs = __pyx_f_4grpc_7_cython_6cygrpc__get_c_pem_root_certs(__pyx_v_pem_root_certs);

  /* "src/python/grpcio/grpc/_cython/_cygrpc/credentials.pyx.pxi":241
 *   credentials.references.append(pem_key_cert_pairs)
 *   cdef const char * c_pem_root_certs = _get_c_pem_root_certs(pem_root_certs)
 *   credentials.c_ssl_pem_key_cert_pairs_count = len(pem_key_cert_pairs)             # <<<<<<<<<<<<<<
 *   credentials.c_ssl_pem_key_cert_pairs = _create_c_ssl_pem_key_cert_pairs(pem_key_cert_pairs)
 *   cdef grpc_ssl_server_certificate_config *c_cert_config = NULL
 */
  __pyx_t_3 = PyObject_Length(__pyx_v_pem_key_cert_pairs); if (unlikely(__pyx_t_3 == ((Py_ssize_t)-1))) __PYX_ERR(4, 241, __pyx_L1_error)
  __pyx_v_credentials->c_ssl_pem_key_cert_pairs_count = __pyx_t_3;

  /* "src/python/grpcio/grpc/_cython/_cygrpc/credentials.pyx.pxi":242
 *   cdef const char * c_pem_root_certs = _get_c_pem_root_certs(pem_root_certs)
 *   credentials.c_ssl_pem_key_cert_pairs_count = len(pem_key_cert_pairs)
 *   credentials.c_ssl_pem_key_cert_pairs = _create_c_ssl_pem_key_cert_pairs(pem_key_cert_pairs)             # <<<<<<<<<<<<<<
 *   cdef grpc_ssl_server_certificate_config *c_cert_config = NULL
 *   c_cert_config = grpc_ssl_server_certificate_config_create(
 */
  __pyx_v_credentials->c_ssl_pem_key_cert_pairs = __pyx_f_4grpc_7_cython_6cygrpc__create_c_ssl_pem_key_cert_pairs(__pyx_v_pem_key_cert_pairs);

  /* "src/python/grpcio/grpc/_cython/_cygrpc/credentials.pyx.pxi":243
 *   credentials.c_ssl_pem_key_cert_pairs_count = len(pem_key_cert_pairs)
 *   credentials.c_ssl_pem_key_cert_pairs = _create_c_ssl_pem_key_cert_pairs(pem_key_cert_pairs)
 *   cdef grpc_ssl_server_certificate_config *c_cert_config = NULL             # <<<<<<<<<<<<<<
 *   c_cert_config = grpc_ssl_server_certificate_config_create(
 *     c_pem_root_certs, credentials.c_ssl_pem_key_cert_pairs,
 */
  __pyx_v_c_cert_config = NULL;

  /* "src/python/grpcio/grpc/_cython/_cygrpc/credentials.pyx.pxi":244
 *   credentials.c_ssl_pem_key_cert_pairs = _create_c_ssl_pem_key_cert_pairs(pem_key_cert_pairs)
 *   cdef grpc_ssl_server_certificate_config *c_cert_config = NULL
 *   c_cert_config = grpc_ssl_server_certificate_config_create(             # <<<<<<<<<<<<<<
 *     c_pem_root_certs, credentials.c_ssl_pem_key_cert_pairs,
 *     credentials.c_ssl_pem_key_cert_pairs_count)
 */
  __pyx_v_c_cert_config = grpc_ssl_server_certificate_config_create(__pyx_v_c_pem_root_certs, __pyx_v_credentials->c_ssl_pem_key_cert_pairs, __pyx_v_credentials->c_ssl_pem_key_cert_pairs_count);

  /* "src/python/grpcio/grpc/_cython/_cygrpc/credentials.pyx.pxi":247
 *     c_pem_root_certs, credentials.c_ssl_pem_key_cert_pairs,
 *     credentials.c_ssl_pem_key_cert_pairs_count)
 *   cdef grpc_ssl_server_credentials_options* c_options = NULL             # <<<<<<<<<<<<<<
 *   # C-core assumes ownership of c_cert_config
 *   c_options = grpc_ssl_server_credentials_create_options_using_config(
 */
  __pyx_v_c_options = NULL;

  /* "src/python/grpcio/grpc/_cython/_cygrpc/credentials.pyx.pxi":251
 *   c_options = grpc_ssl_server_credentials_create_options_using_config(
 *     GRPC_SSL_REQUEST_AND_REQUIRE_CLIENT_CERTIFICATE_AND_VERIFY
 *     if force_client_auth else             # <<<<<<<<<<<<<<
 *     GRPC_SSL_DONT_REQUEST_CLIENT_CERTIFICATE,
 *     c_cert_config)
 */
  if ((__pyx_v_force_client_auth != 0)) {

    /* "src/python/grpcio/grpc/_cython/_cygrpc/credentials.pyx.pxi":250
 *   # C-core assumes ownership of c_cert_config
 *   c_options = grpc_ssl_server_credentials_create_options_using_config(
 *     GRPC_SSL_REQUEST_AND_REQUIRE_CLIENT_CERTIFICATE_AND_VERIFY             # <<<<<<<<<<<<<<
 *     if force_client_auth else
 *     GRPC_SSL_DONT_REQUEST_CLIENT_CERTIFICATE,
 */
    __pyx_t_4 = GRPC_SSL_REQUEST_AND_REQUIRE_CLIENT_CERTIFICATE_AND_VERIFY;
  } else {

    /* "src/python/grpcio/grpc/_cython/_cygrpc/credentials.pyx.pxi":252
 *     GRPC_SSL_REQUEST_AND_REQUIRE_CLIENT_CERTIFICATE_AND_VERIFY
 *     if force_client_auth else
 *     GRPC_SSL_DONT_REQUEST_CLIENT_CERTIFICATE,             # <<<<<<<<<<<<<<
 *     c_cert_config)
 *   # C-core assumes ownership of c_options
 */
    __pyx_t_4 = GRPC_SSL_DONT_REQUEST_CLIENT_CERTIFICATE;
  }

  /* "src/python/grpcio/grpc/_cython/_cygrpc/credentials.pyx.pxi":249
 *   cdef grpc_ssl_server_credentials_options* c_options = NULL
 *   # C-core assumes ownership of c_cert_config
 *   c_options = grpc_ssl_server_credentials_create_options_using_config(             # <<<<<<<<<<<<<<
 *     GRPC_SSL_REQUEST_AND_REQUIRE_CLIENT_CERTIFICATE_AND_VERIFY
 *     if force_client_auth else
 */
  __pyx_v_c_options = grpc_ssl_server_credentials_create_options_using_config(__pyx_t_4, __pyx_v_c_cert_config);

  /* "src/python/grpcio/grpc/_cython/_cygrpc/credentials.pyx.pxi":255
 *     c_cert_config)
 *   # C-core assumes ownership of c_options
 *   credentials.c_credentials = grpc_ssl_server_credentials_create_with_options(c_options)             # <<<<<<<<<<<<<<
 *   return credentials
 * 
 */
  __pyx_v_credentials->c_credentials = grpc_ssl_server_credentials_create_with_options(__pyx_v_c_options);

  /* "src/python/grpcio/grpc/_cython/_cygrpc/credentials.pyx.pxi":256
 *   # C-core assumes ownership of c_options
 *   credentials.c_credentials = grpc_ssl_server_credentials_create_with_options(c_options)
 *   return credentials             # <<<<<<<<<<<<<<
 * 
 * def server_certificate_config_ssl(pem_root_certs, pem_key_cert_pairs):
 */
  __Pyx_XDECREF(__pyx_r);
  __Pyx_INCREF(((PyObject *)__pyx_v_credentials));
  __pyx_r = ((PyObject *)__pyx_v_credentials);
  goto __pyx_L0;

  /* "src/python/grpcio/grpc/_cython/_cygrpc/credentials.pyx.pxi":233
 *   return c_ssl_pem_key_cert_pairs
 * 
 * def server_credentials_ssl(pem_root_certs, pem_key_cert_pairs,             # <<<<<<<<<<<<<<
 *                            bint force_client_auth):
 *   pem_root_certs = str_to_bytes(pem_root_certs)
 */

  /* function exit code */
  __pyx_L1_error:;
  __Pyx_XDECREF(__pyx_t_1);
  __Pyx_AddTraceback("grpc._cython.cygrpc.server_credentials_ssl", __pyx_clineno, __pyx_lineno, __pyx_filename);
  __pyx_r = NULL;
  __pyx_L0:;
  __Pyx_XDECREF((PyObject *)__pyx_v_credentials);
  __Pyx_XDECREF(__pyx_v_pem_root_certs);
  __Pyx_XDECREF(__pyx_v_pem_key_cert_pairs);
  __Pyx_XGIVEREF(__pyx_r);
  __Pyx_RefNannyFinishContext();
  return __pyx_r;
}

/* "src/python/grpcio/grpc/_cython/_cygrpc/credentials.pyx.pxi":258
 *   return credentials
 * 
 * def server_certificate_config_ssl(pem_root_certs, pem_key_cert_pairs):             # <<<<<<<<<<<<<<
 *   pem_root_certs = str_to_bytes(pem_root_certs)
 *   pem_key_cert_pairs = list(pem_key_cert_pairs)
 */

/* Python wrapper */
static PyObject *__pyx_pw_4grpc_7_cython_6cygrpc_23server_certificate_config_ssl(PyObject *__pyx_self, PyObject *__pyx_args, PyObject *__pyx_kwds); /*proto*/
static PyMethodDef __pyx_mdef_4grpc_7_cython_6cygrpc_23server_certificate_config_ssl = {"server_certificate_config_ssl", (PyCFunction)__pyx_pw_4grpc_7_cython_6cygrpc_23server_certificate_config_ssl, METH_VARARGS|METH_KEYWORDS, 0};
static PyObject *__pyx_pw_4grpc_7_cython_6cygrpc_23server_certificate_config_ssl(PyObject *__pyx_self, PyObject *__pyx_args, PyObject *__pyx_kwds) {
  PyObject *__pyx_v_pem_root_certs = 0;
  PyObject *__pyx_v_pem_key_cert_pairs = 0;
  PyObject *__pyx_r = 0;
  __Pyx_RefNannyDeclarations
  __Pyx_RefNannySetupContext("server_certificate_config_ssl (wrapper)", 0);
  {
    static PyObject **__pyx_pyargnames[] = {&__pyx_n_s_pem_root_certs,&__pyx_n_s_pem_key_cert_pairs,0};
    PyObject* values[2] = {0,0};
    if (unlikely(__pyx_kwds)) {
      Py_ssize_t kw_args;
      const Py_ssize_t pos_args = PyTuple_GET_SIZE(__pyx_args);
      switch (pos_args) {
        case  2: values[1] = PyTuple_GET_ITEM(__pyx_args, 1);
        CYTHON_FALLTHROUGH;
        case  1: values[0] = PyTuple_GET_ITEM(__pyx_args, 0);
        CYTHON_FALLTHROUGH;
        case  0: break;
        default: goto __pyx_L5_argtuple_error;
      }
      kw_args = PyDict_Size(__pyx_kwds);
      switch (pos_args) {
        case  0:
        if (likely((values[0] = __Pyx_PyDict_GetItemStr(__pyx_kwds, __pyx_n_s_pem_root_certs)) != 0)) kw_args--;
        else goto __pyx_L5_argtuple_error;
        CYTHON_FALLTHROUGH;
        case  1:
        if (likely((values[1] = __Pyx_PyDict_GetItemStr(__pyx_kwds, __pyx_n_s_pem_key_cert_pairs)) != 0)) kw_args--;
        else {
          __Pyx_RaiseArgtupleInvalid("server_certificate_config_ssl", 1, 2, 2, 1); __PYX_ERR(4, 258, __pyx_L3_error)
        }
      }
      if (unlikely(kw_args > 0)) {
        if (unlikely(__Pyx_ParseOptionalKeywords(__pyx_kwds, __pyx_pyargnames, 0, values, pos_args, "server_certificate_config_ssl") < 0)) __PYX_ERR(4, 258, __pyx_L3_error)
      }
    } else if (PyTuple_GET_SIZE(__pyx_args) != 2) {
      goto __pyx_L5_argtuple_error;
    } else {
      values[0] = PyTuple_GET_ITEM(__pyx_args, 0);
      values[1] = PyTuple_GET_ITEM(__pyx_args, 1);
    }
    __pyx_v_pem_root_certs = values[0];
    __pyx_v_pem_key_cert_pairs = values[1];
  }
  goto __pyx_L4_argument_unpacking_done;
  __pyx_L5_argtuple_error:;
  __Pyx_RaiseArgtupleInvalid("server_certificate_config_ssl", 1, 2, 2, PyTuple_GET_SIZE(__pyx_args)); __PYX_ERR(4, 258, __pyx_L3_error)
  __pyx_L3_error:;
  __Pyx_AddTraceback("grpc._cython.cygrpc.server_certificate_config_ssl", __pyx_clineno, __pyx_lineno, __pyx_filename);
  __Pyx_RefNannyFinishContext();
  return NULL;
  __pyx_L4_argument_unpacking_done:;
  __pyx_r = __pyx_pf_4grpc_7_cython_6cygrpc_22server_certificate_config_ssl(__pyx_self, __pyx_v_pem_root_certs, __pyx_v_pem_key_cert_pairs);

  /* function exit code */
  __Pyx_RefNannyFinishContext();
  return __pyx_r;
}

static PyObject *__pyx_pf_4grpc_7_cython_6cygrpc_22server_certificate_config_ssl(CYTHON_UNUSED PyObject *__pyx_self, PyObject *__pyx_v_pem_root_certs, PyObject *__pyx_v_pem_key_cert_pairs) {
  struct __pyx_obj_4grpc_7_cython_6cygrpc_ServerCertificateConfig *__pyx_v_cert_config = 0;
  PyObject *__pyx_r = NULL;
  __Pyx_RefNannyDeclarations
  PyObject *__pyx_t_1 = NULL;
  int __pyx_t_2;
  Py_ssize_t __pyx_t_3;
  __Pyx_RefNannySetupContext("server_certificate_config_ssl", 0);
  __Pyx_INCREF(__pyx_v_pem_root_certs);
  __Pyx_INCREF(__pyx_v_pem_key_cert_pairs);

  /* "src/python/grpcio/grpc/_cython/_cygrpc/credentials.pyx.pxi":259
 * 
 * def server_certificate_config_ssl(pem_root_certs, pem_key_cert_pairs):
 *   pem_root_certs = str_to_bytes(pem_root_certs)             # <<<<<<<<<<<<<<
 *   pem_key_cert_pairs = list(pem_key_cert_pairs)
 *   cdef ServerCertificateConfig cert_config = ServerCertificateConfig()
 */
  __pyx_t_1 = __pyx_f_4grpc_7_cython_6cygrpc_str_to_bytes(__pyx_v_pem_root_certs); if (unlikely(!__pyx_t_1)) __PYX_ERR(4, 259, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_1);
  __Pyx_DECREF_SET(__pyx_v_pem_root_certs, __pyx_t_1);
  __pyx_t_1 = 0;

  /* "src/python/grpcio/grpc/_cython/_cygrpc/credentials.pyx.pxi":260
 * def server_certificate_config_ssl(pem_root_certs, pem_key_cert_pairs):
 *   pem_root_certs = str_to_bytes(pem_root_certs)
 *   pem_key_cert_pairs = list(pem_key_cert_pairs)             # <<<<<<<<<<<<<<
 *   cdef ServerCertificateConfig cert_config = ServerCertificateConfig()
 *   cert_config.references.append(pem_root_certs)
 */
  __pyx_t_1 = PySequence_List(__pyx_v_pem_key_cert_pairs); if (unlikely(!__pyx_t_1)) __PYX_ERR(4, 260, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_1);
  __Pyx_DECREF_SET(__pyx_v_pem_key_cert_pairs, __pyx_t_1);
  __pyx_t_1 = 0;

  /* "src/python/grpcio/grpc/_cython/_cygrpc/credentials.pyx.pxi":261
 *   pem_root_certs = str_to_bytes(pem_root_certs)
 *   pem_key_cert_pairs = list(pem_key_cert_pairs)
 *   cdef ServerCertificateConfig cert_config = ServerCertificateConfig()             # <<<<<<<<<<<<<<
 *   cert_config.references.append(pem_root_certs)
 *   cert_config.references.append(pem_key_cert_pairs)
 */
  __pyx_t_1 = __Pyx_PyObject_CallNoArg(((PyObject *)__pyx_ptype_4grpc_7_cython_6cygrpc_ServerCertificateConfig)); if (unlikely(!__pyx_t_1)) __PYX_ERR(4, 261, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_1);
  __pyx_v_cert_config = ((struct __pyx_obj_4grpc_7_cython_6cygrpc_ServerCertificateConfig *)__pyx_t_1);
  __pyx_t_1 = 0;

  /* "src/python/grpcio/grpc/_cython/_cygrpc/credentials.pyx.pxi":262
 *   pem_key_cert_pairs = list(pem_key_cert_pairs)
 *   cdef ServerCertificateConfig cert_config = ServerCertificateConfig()
 *   cert_config.references.append(pem_root_certs)             # <<<<<<<<<<<<<<
 *   cert_config.references.append(pem_key_cert_pairs)
 *   cert_config.c_pem_root_certs = _get_c_pem_root_certs(pem_root_certs)
 */
  if (unlikely(__pyx_v_cert_config->references == Py_None)) {
    PyErr_Format(PyExc_AttributeError, "'NoneType' object has no attribute '%.30s'", "append");
    __PYX_ERR(4, 262, __pyx_L1_error)
  }
  __pyx_t_2 = __Pyx_PyList_Append(__pyx_v_cert_config->references, __pyx_v_pem_root_certs); if (unlikely(__pyx_t_2 == ((int)-1))) __PYX_ERR(4, 262, __pyx_L1_error)

  /* "src/python/grpcio/grpc/_cython/_cygrpc/credentials.pyx.pxi":263
 *   cdef ServerCertificateConfig cert_config = ServerCertificateConfig()
 *   cert_config.references.append(pem_root_certs)
 *   cert_config.references.append(pem_key_cert_pairs)             # <<<<<<<<<<<<<<
 *   cert_config.c_pem_root_certs = _get_c_pem_root_certs(pem_root_certs)
 *   cert_config.c_ssl_pem_key_cert_pairs_count = len(pem_key_cert_pairs)
 */
  if (unlikely(__pyx_v_cert_config->references == Py_None)) {
    PyErr_Format(PyExc_AttributeError, "'NoneType' object has no attribute '%.30s'", "append");
    __PYX_ERR(4, 263, __pyx_L1_error)
  }
  __pyx_t_2 = __Pyx_PyList_Append(__pyx_v_cert_config->references, __pyx_v_pem_key_cert_pairs); if (unlikely(__pyx_t_2 == ((int)-1))) __PYX_ERR(4, 263, __pyx_L1_error)

  /* "src/python/grpcio/grpc/_cython/_cygrpc/credentials.pyx.pxi":264
 *   cert_config.references.append(pem_root_certs)
 *   cert_config.references.append(pem_key_cert_pairs)
 *   cert_config.c_pem_root_certs = _get_c_pem_root_certs(pem_root_certs)             # <<<<<<<<<<<<<<
 *   cert_config.c_ssl_pem_key_cert_pairs_count = len(pem_key_cert_pairs)
 *   cert_config.c_ssl_pem_key_cert_pairs = _create_c_ssl_pem_key_cert_pairs(pem_key_cert_pairs)
 */
  __pyx_v_cert_config->c_pem_root_certs = __pyx_f_4grpc_7_cython_6cygrpc__get_c_pem_root_certs(__pyx_v_pem_root_certs);

  /* "src/python/grpcio/grpc/_cython/_cygrpc/credentials.pyx.pxi":265
 *   cert_config.references.append(pem_key_cert_pairs)
 *   cert_config.c_pem_root_certs = _get_c_pem_root_certs(pem_root_certs)
 *   cert_config.c_ssl_pem_key_cert_pairs_count = len(pem_key_cert_pairs)             # <<<<<<<<<<<<<<
 *   cert_config.c_ssl_pem_key_cert_pairs = _create_c_ssl_pem_key_cert_pairs(pem_key_cert_pairs)
 *   cert_config.c_cert_config = grpc_ssl_server_certificate_config_create(
 */
  __pyx_t_3 = PyObject_Length(__pyx_v_pem_key_cert_pairs); if (unlikely(__pyx_t_3 == ((Py_ssize_t)-1))) __PYX_ERR(4, 265, __pyx_L1_error)
  __pyx_v_cert_config->c_ssl_pem_key_cert_pairs_count = __pyx_t_3;

  /* "src/python/grpcio/grpc/_cython/_cygrpc/credentials.pyx.pxi":266
 *   cert_config.c_pem_root_certs = _get_c_pem_root_certs(pem_root_certs)
 *   cert_config.c_ssl_pem_key_cert_pairs_count = len(pem_key_cert_pairs)
 *   cert_config.c_ssl_pem_key_cert_pairs = _create_c_ssl_pem_key_cert_pairs(pem_key_cert_pairs)             # <<<<<<<<<<<<<<
 *   cert_config.c_cert_config = grpc_ssl_server_certificate_config_create(
 *     cert_config.c_pem_root_certs, cert_config.c_ssl_pem_key_cert_pairs,
 */
  __pyx_v_cert_config->c_ssl_pem_key_cert_pairs = __pyx_f_4grpc_7_cython_6cygrpc__create_c_ssl_pem_key_cert_pairs(__pyx_v_pem_key_cert_pairs);

  /* "src/python/grpcio/grpc/_cython/_cygrpc/credentials.pyx.pxi":267
 *   cert_config.c_ssl_pem_key_cert_pairs_count = len(pem_key_cert_pairs)
 *   cert_config.c_ssl_pem_key_cert_pairs = _create_c_ssl_pem_key_cert_pairs(pem_key_cert_pairs)
 *   cert_config.c_cert_config = grpc_ssl_server_certificate_config_create(             # <<<<<<<<<<<<<<
 *     cert_config.c_pem_root_certs, cert_config.c_ssl_pem_key_cert_pairs,
 *     cert_config.c_ssl_pem_key_cert_pairs_count)
 */
  __pyx_v_cert_config->c_cert_config = grpc_ssl_server_certificate_config_create(__pyx_v_cert_config->c_pem_root_certs, __pyx_v_cert_config->c_ssl_pem_key_cert_pairs, __pyx_v_cert_config->c_ssl_pem_key_cert_pairs_count);

  /* "src/python/grpcio/grpc/_cython/_cygrpc/credentials.pyx.pxi":270
 *     cert_config.c_pem_root_certs, cert_config.c_ssl_pem_key_cert_pairs,
 *     cert_config.c_ssl_pem_key_cert_pairs_count)
 *   return cert_config             # <<<<<<<<<<<<<<
 * 
 * def server_credentials_ssl_dynamic_cert_config(initial_cert_config,
 */
  __Pyx_XDECREF(__pyx_r);
  __Pyx_INCREF(((PyObject *)__pyx_v_cert_config));
  __pyx_r = ((PyObject *)__pyx_v_cert_config);
  goto __pyx_L0;

  /* "src/python/grpcio/grpc/_cython/_cygrpc/credentials.pyx.pxi":258
 *   return credentials
 * 
 * def server_certificate_config_ssl(pem_root_certs, pem_key_cert_pairs):             # <<<<<<<<<<<<<<
 *   pem_root_certs = str_to_bytes(pem_root_certs)
 *   pem_key_cert_pairs = list(pem_key_cert_pairs)
 */

  /* function exit code */
  __pyx_L1_error:;
  __Pyx_XDECREF(__pyx_t_1);
  __Pyx_AddTraceback("grpc._cython.cygrpc.server_certificate_config_ssl", __pyx_clineno, __pyx_lineno, __pyx_filename);
  __pyx_r = NULL;
  __pyx_L0:;
  __Pyx_XDECREF((PyObject *)__pyx_v_cert_config);
  __Pyx_XDECREF(__pyx_v_pem_root_certs);
  __Pyx_XDECREF(__pyx_v_pem_key_cert_pairs);
  __Pyx_XGIVEREF(__pyx_r);
  __Pyx_RefNannyFinishContext();
  return __pyx_r;
}

/* "src/python/grpcio/grpc/_cython/_cygrpc/credentials.pyx.pxi":272
 *   return cert_config
 * 
 * def server_credentials_ssl_dynamic_cert_config(initial_cert_config,             # <<<<<<<<<<<<<<
 *                                                cert_config_fetcher,
 *                                                bint force_client_auth):
 */

/* Python wrapper */
static PyObject *__pyx_pw_4grpc_7_cython_6cygrpc_25server_credentials_ssl_dynamic_cert_config(PyObject *__pyx_self, PyObject *__pyx_args, PyObject *__pyx_kwds); /*proto*/
static PyMethodDef __pyx_mdef_4grpc_7_cython_6cygrpc_25server_credentials_ssl_dynamic_cert_config = {"server_credentials_ssl_dynamic_cert_config", (PyCFunction)__pyx_pw_4grpc_7_cython_6cygrpc_25server_credentials_ssl_dynamic_cert_config, METH_VARARGS|METH_KEYWORDS, 0};
static PyObject *__pyx_pw_4grpc_7_cython_6cygrpc_25server_credentials_ssl_dynamic_cert_config(PyObject *__pyx_self, PyObject *__pyx_args, PyObject *__pyx_kwds) {
  PyObject *__pyx_v_initial_cert_config = 0;
  PyObject *__pyx_v_cert_config_fetcher = 0;
  int __pyx_v_force_client_auth;
  PyObject *__pyx_r = 0;
  __Pyx_RefNannyDeclarations
  __Pyx_RefNannySetupContext("server_credentials_ssl_dynamic_cert_config (wrapper)", 0);
  {
    static PyObject **__pyx_pyargnames[] = {&__pyx_n_s_initial_cert_config,&__pyx_n_s_cert_config_fetcher,&__pyx_n_s_force_client_auth,0};
    PyObject* values[3] = {0,0,0};
    if (unlikely(__pyx_kwds)) {
      Py_ssize_t kw_args;
      const Py_ssize_t pos_args = PyTuple_GET_SIZE(__pyx_args);
      switch (pos_args) {
        case  3: values[2] = PyTuple_GET_ITEM(__pyx_args, 2);
        CYTHON_FALLTHROUGH;
        case  2: values[1] = PyTuple_GET_ITEM(__pyx_args, 1);
        CYTHON_FALLTHROUGH;
        case  1: values[0] = PyTuple_GET_ITEM(__pyx_args, 0);
        CYTHON_FALLTHROUGH;
        case  0: break;
        default: goto __pyx_L5_argtuple_error;
      }
      kw_args = PyDict_Size(__pyx_kwds);
      switch (pos_args) {
        case  0:
        if (likely((values[0] = __Pyx_PyDict_GetItemStr(__pyx_kwds, __pyx_n_s_initial_cert_config)) != 0)) kw_args--;
        else goto __pyx_L5_argtuple_error;
        CYTHON_FALLTHROUGH;
        case  1:
        if (likely((values[1] = __Pyx_PyDict_GetItemStr(__pyx_kwds, __pyx_n_s_cert_config_fetcher)) != 0)) kw_args--;
        else {
          __Pyx_RaiseArgtupleInvalid("server_credentials_ssl_dynamic_cert_config", 1, 3, 3, 1); __PYX_ERR(4, 272, __pyx_L3_error)
        }
        CYTHON_FALLTHROUGH;
        case  2:
        if (likely((values[2] = __Pyx_PyDict_GetItemStr(__pyx_kwds, __pyx_n_s_force_client_auth)) != 0)) kw_args--;
        else {
          __Pyx_RaiseArgtupleInvalid("server_credentials_ssl_dynamic_cert_config", 1, 3, 3, 2); __PYX_ERR(4, 272, __pyx_L3_error)
        }
      }
      if (unlikely(kw_args > 0)) {
        if (unlikely(__Pyx_ParseOptionalKeywords(__pyx_kwds, __pyx_pyargnames, 0, values, pos_args, "server_credentials_ssl_dynamic_cert_config") < 0)) __PYX_ERR(4, 272, __pyx_L3_error)
      }
    } else if (PyTuple_GET_SIZE(__pyx_args) != 3) {
      goto __pyx_L5_argtuple_error;
    } else {
      values[0] = PyTuple_GET_ITEM(__pyx_args, 0);
      values[1] = PyTuple_GET_ITEM(__pyx_args, 1);
      values[2] = PyTuple_GET_ITEM(__pyx_args, 2);
    }
    __pyx_v_initial_cert_config = values[0];
    __pyx_v_cert_config_fetcher = values[1];
    __pyx_v_force_client_auth = __Pyx_PyObject_IsTrue(values[2]); if (unlikely((__pyx_v_force_client_auth == (int)-1) && PyErr_Occurred())) __PYX_ERR(4, 274, __pyx_L3_error)
  }
  goto __pyx_L4_argument_unpacking_done;
  __pyx_L5_argtuple_error:;
  __Pyx_RaiseArgtupleInvalid("server_credentials_ssl_dynamic_cert_config", 1, 3, 3, PyTuple_GET_SIZE(__pyx_args)); __PYX_ERR(4, 272, __pyx_L3_error)
  __pyx_L3_error:;
  __Pyx_AddTraceback("grpc._cython.cygrpc.server_credentials_ssl_dynamic_cert_config", __pyx_clineno, __pyx_lineno, __pyx_filename);
  __Pyx_RefNannyFinishContext();
  return NULL;
  __pyx_L4_argument_unpacking_done:;
  __pyx_r = __pyx_pf_4grpc_7_cython_6cygrpc_24server_credentials_ssl_dynamic_cert_config(__pyx_self, __pyx_v_initial_cert_config, __pyx_v_cert_config_fetcher, __pyx_v_force_client_auth);

  /* function exit code */
  __Pyx_RefNannyFinishContext();
  return __pyx_r;
}

static PyObject *__pyx_pf_4grpc_7_cython_6cygrpc_24server_credentials_ssl_dynamic_cert_config(CYTHON_UNUSED PyObject *__pyx_self, PyObject *__pyx_v_initial_cert_config, PyObject *__pyx_v_cert_config_fetcher, int __pyx_v_force_client_auth) {
  struct __pyx_obj_4grpc_7_cython_6cygrpc_ServerCredentials *__pyx_v_credentials = 0;
  grpc_ssl_server_credentials_options *__pyx_v_c_options;
  PyObject *__pyx_r = NULL;
  __Pyx_RefNannyDeclarations
  PyObject *__pyx_t_1 = NULL;
  PyObject *__pyx_t_2 = NULL;
  int __pyx_t_3;
  int __pyx_t_4;
  grpc_ssl_client_certificate_request_type __pyx_t_5;
  __Pyx_RefNannySetupContext("server_credentials_ssl_dynamic_cert_config", 0);

  /* "src/python/grpcio/grpc/_cython/_cygrpc/credentials.pyx.pxi":275
 *                                                cert_config_fetcher,
 *                                                bint force_client_auth):
 *   if not isinstance(initial_cert_config, grpc.ServerCertificateConfiguration):             # <<<<<<<<<<<<<<
 *     raise TypeError(
 *         'initial_cert_config must be a grpc.ServerCertificateConfiguration')
 */
  __pyx_t_1 = __Pyx_GetModuleGlobalName(__pyx_n_s_grpc); if (unlikely(!__pyx_t_1)) __PYX_ERR(4, 275, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_1);
  __pyx_t_2 = __Pyx_PyObject_GetAttrStr(__pyx_t_1, __pyx_n_s_ServerCertificateConfiguration); if (unlikely(!__pyx_t_2)) __PYX_ERR(4, 275, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_2);
  __Pyx_DECREF(__pyx_t_1); __pyx_t_1 = 0;
  __pyx_t_3 = PyObject_IsInstance(__pyx_v_initial_cert_config, __pyx_t_2); if (unlikely(__pyx_t_3 == ((int)-1))) __PYX_ERR(4, 275, __pyx_L1_error)
  __Pyx_DECREF(__pyx_t_2); __pyx_t_2 = 0;
  __pyx_t_4 = ((!(__pyx_t_3 != 0)) != 0);
  if (unlikely(__pyx_t_4)) {

    /* "src/python/grpcio/grpc/_cython/_cygrpc/credentials.pyx.pxi":276
 *                                                bint force_client_auth):
 *   if not isinstance(initial_cert_config, grpc.ServerCertificateConfiguration):
 *     raise TypeError(             # <<<<<<<<<<<<<<
 *         'initial_cert_config must be a grpc.ServerCertificateConfiguration')
 *   if not callable(cert_config_fetcher):
 */
    __pyx_t_2 = __Pyx_PyObject_Call(__pyx_builtin_TypeError, __pyx_tuple__69, NULL); if (unlikely(!__pyx_t_2)) __PYX_ERR(4, 276, __pyx_L1_error)
    __Pyx_GOTREF(__pyx_t_2);
    __Pyx_Raise(__pyx_t_2, 0, 0, 0);
    __Pyx_DECREF(__pyx_t_2); __pyx_t_2 = 0;
    __PYX_ERR(4, 276, __pyx_L1_error)

    /* "src/python/grpcio/grpc/_cython/_cygrpc/credentials.pyx.pxi":275
 *                                                cert_config_fetcher,
 *                                                bint force_client_auth):
 *   if not isinstance(initial_cert_config, grpc.ServerCertificateConfiguration):             # <<<<<<<<<<<<<<
 *     raise TypeError(
 *         'initial_cert_config must be a grpc.ServerCertificateConfiguration')
 */
  }

  /* "src/python/grpcio/grpc/_cython/_cygrpc/credentials.pyx.pxi":278
 *     raise TypeError(
 *         'initial_cert_config must be a grpc.ServerCertificateConfiguration')
 *   if not callable(cert_config_fetcher):             # <<<<<<<<<<<<<<
 *     raise TypeError('cert_config_fetcher must be callable')
 *   cdef ServerCredentials credentials = ServerCredentials()
 */
  __pyx_t_4 = __Pyx_PyCallable_Check(__pyx_v_cert_config_fetcher); if (unlikely(__pyx_t_4 == ((int)-1))) __PYX_ERR(4, 278, __pyx_L1_error)
  __pyx_t_3 = ((!(__pyx_t_4 != 0)) != 0);
  if (unlikely(__pyx_t_3)) {

    /* "src/python/grpcio/grpc/_cython/_cygrpc/credentials.pyx.pxi":279
 *         'initial_cert_config must be a grpc.ServerCertificateConfiguration')
 *   if not callable(cert_config_fetcher):
 *     raise TypeError('cert_config_fetcher must be callable')             # <<<<<<<<<<<<<<
 *   cdef ServerCredentials credentials = ServerCredentials()
 *   credentials.initial_cert_config = initial_cert_config
 */
    __pyx_t_2 = __Pyx_PyObject_Call(__pyx_builtin_TypeError, __pyx_tuple__70, NULL); if (unlikely(!__pyx_t_2)) __PYX_ERR(4, 279, __pyx_L1_error)
    __Pyx_GOTREF(__pyx_t_2);
    __Pyx_Raise(__pyx_t_2, 0, 0, 0);
    __Pyx_DECREF(__pyx_t_2); __pyx_t_2 = 0;
    __PYX_ERR(4, 279, __pyx_L1_error)

    /* "src/python/grpcio/grpc/_cython/_cygrpc/credentials.pyx.pxi":278
 *     raise TypeError(
 *         'initial_cert_config must be a grpc.ServerCertificateConfiguration')
 *   if not callable(cert_config_fetcher):             # <<<<<<<<<<<<<<
 *     raise TypeError('cert_config_fetcher must be callable')
 *   cdef ServerCredentials credentials = ServerCredentials()
 */
  }

  /* "src/python/grpcio/grpc/_cython/_cygrpc/credentials.pyx.pxi":280
 *   if not callable(cert_config_fetcher):
 *     raise TypeError('cert_config_fetcher must be callable')
 *   cdef ServerCredentials credentials = ServerCredentials()             # <<<<<<<<<<<<<<
 *   credentials.initial_cert_config = initial_cert_config
 *   credentials.cert_config_fetcher = cert_config_fetcher
 */
  __pyx_t_2 = __Pyx_PyObject_CallNoArg(((PyObject *)__pyx_ptype_4grpc_7_cython_6cygrpc_ServerCredentials)); if (unlikely(!__pyx_t_2)) __PYX_ERR(4, 280, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_2);
  __pyx_v_credentials = ((struct __pyx_obj_4grpc_7_cython_6cygrpc_ServerCredentials *)__pyx_t_2);
  __pyx_t_2 = 0;

  /* "src/python/grpcio/grpc/_cython/_cygrpc/credentials.pyx.pxi":281
 *     raise TypeError('cert_config_fetcher must be callable')
 *   cdef ServerCredentials credentials = ServerCredentials()
 *   credentials.initial_cert_config = initial_cert_config             # <<<<<<<<<<<<<<
 *   credentials.cert_config_fetcher = cert_config_fetcher
 *   cdef grpc_ssl_server_credentials_options* c_options = NULL
 */
  __Pyx_INCREF(__pyx_v_initial_cert_config);
  __Pyx_GIVEREF(__pyx_v_initial_cert_config);
  __Pyx_GOTREF(__pyx_v_credentials->initial_cert_config);
  __Pyx_DECREF(__pyx_v_credentials->initial_cert_config);
  __pyx_v_credentials->initial_cert_config = __pyx_v_initial_cert_config;

  /* "src/python/grpcio/grpc/_cython/_cygrpc/credentials.pyx.pxi":282
 *   cdef ServerCredentials credentials = ServerCredentials()
 *   credentials.initial_cert_config = initial_cert_config
 *   credentials.cert_config_fetcher = cert_config_fetcher             # <<<<<<<<<<<<<<
 *   cdef grpc_ssl_server_credentials_options* c_options = NULL
 *   c_options = grpc_ssl_server_credentials_create_options_using_config_fetcher(
 */
  __Pyx_INCREF(__pyx_v_cert_config_fetcher);
  __Pyx_GIVEREF(__pyx_v_cert_config_fetcher);
  __Pyx_GOTREF(__pyx_v_credentials->cert_config_fetcher);
  __Pyx_DECREF(__pyx_v_credentials->cert_config_fetcher);
  __pyx_v_credentials->cert_config_fetcher = __pyx_v_cert_config_fetcher;

  /* "src/python/grpcio/grpc/_cython/_cygrpc/credentials.pyx.pxi":283
 *   credentials.initial_cert_config = initial_cert_config
 *   credentials.cert_config_fetcher = cert_config_fetcher
 *   cdef grpc_ssl_server_credentials_options* c_options = NULL             # <<<<<<<<<<<<<<
 *   c_options = grpc_ssl_server_credentials_create_options_using_config_fetcher(
 *     GRPC_SSL_REQUEST_AND_REQUIRE_CLIENT_CERTIFICATE_AND_VERIFY
 */
  __pyx_v_c_options = NULL;

  /* "src/python/grpcio/grpc/_cython/_cygrpc/credentials.pyx.pxi":286
 *   c_options = grpc_ssl_server_credentials_create_options_using_config_fetcher(
 *     GRPC_SSL_REQUEST_AND_REQUIRE_CLIENT_CERTIFICATE_AND_VERIFY
 *     if force_client_auth else             # <<<<<<<<<<<<<<
 *     GRPC_SSL_DONT_REQUEST_CLIENT_CERTIFICATE,
 *     _server_cert_config_fetcher_wrapper,
 */
  if ((__pyx_v_force_client_auth != 0)) {

    /* "src/python/grpcio/grpc/_cython/_cygrpc/credentials.pyx.pxi":285
 *   cdef grpc_ssl_server_credentials_options* c_options = NULL
 *   c_options = grpc_ssl_server_credentials_create_options_using_config_fetcher(
 *     GRPC_SSL_REQUEST_AND_REQUIRE_CLIENT_CERTIFICATE_AND_VERIFY             # <<<<<<<<<<<<<<
 *     if force_client_auth else
 *     GRPC_SSL_DONT_REQUEST_CLIENT_CERTIFICATE,
 */
    __pyx_t_5 = GRPC_SSL_REQUEST_AND_REQUIRE_CLIENT_CERTIFICATE_AND_VERIFY;
  } else {

    /* "src/python/grpcio/grpc/_cython/_cygrpc/credentials.pyx.pxi":287
 *     GRPC_SSL_REQUEST_AND_REQUIRE_CLIENT_CERTIFICATE_AND_VERIFY
 *     if force_client_auth else
 *     GRPC_SSL_DONT_REQUEST_CLIENT_CERTIFICATE,             # <<<<<<<<<<<<<<
 *     _server_cert_config_fetcher_wrapper,
 *     <void*>credentials)
 */
    __pyx_t_5 = GRPC_SSL_DONT_REQUEST_CLIENT_CERTIFICATE;
  }

  /* "src/python/grpcio/grpc/_cython/_cygrpc/credentials.pyx.pxi":284
 *   credentials.cert_config_fetcher = cert_config_fetcher
 *   cdef grpc_ssl_server_credentials_options* c_options = NULL
 *   c_options = grpc_ssl_server_credentials_create_options_using_config_fetcher(             # <<<<<<<<<<<<<<
 *     GRPC_SSL_REQUEST_AND_REQUIRE_CLIENT_CERTIFICATE_AND_VERIFY
 *     if force_client_auth else
 */
  __pyx_v_c_options = grpc_ssl_server_credentials_create_options_using_config_fetcher(__pyx_t_5, __pyx_f_4grpc_7_cython_6cygrpc__server_cert_config_fetcher_wrapper, ((void *)__pyx_v_credentials));

  /* "src/python/grpcio/grpc/_cython/_cygrpc/credentials.pyx.pxi":291
 *     <void*>credentials)
 *   # C-core assumes ownership of c_options
 *   credentials.c_credentials = grpc_ssl_server_credentials_create_with_options(c_options)             # <<<<<<<<<<<<<<
 *   return credentials
 * 
 */
  __pyx_v_credentials->c_credentials = grpc_ssl_server_credentials_create_with_options(__pyx_v_c_options);

  /* "src/python/grpcio/grpc/_cython/_cygrpc/credentials.pyx.pxi":292
 *   # C-core assumes ownership of c_options
 *   credentials.c_credentials = grpc_ssl_server_credentials_create_with_options(c_options)
 *   return credentials             # <<<<<<<<<<<<<<
 * 
 * cdef grpc_ssl_certificate_config_reload_status _server_cert_config_fetcher_wrapper(
 */
  __Pyx_XDECREF(__pyx_r);
  __Pyx_INCREF(((PyObject *)__pyx_v_credentials));
  __pyx_r = ((PyObject *)__pyx_v_credentials);
  goto __pyx_L0;

  /* "src/python/grpcio/grpc/_cython/_cygrpc/credentials.pyx.pxi":272
 *   return cert_config
 * 
 * def server_credentials_ssl_dynamic_cert_config(initial_cert_config,             # <<<<<<<<<<<<<<
 *                                                cert_config_fetcher,
 *                                                bint force_client_auth):
 */

  /* function exit code */
  __pyx_L1_error:;
  __Pyx_XDECREF(__pyx_t_1);
  __Pyx_XDECREF(__pyx_t_2);
  __Pyx_AddTraceback("grpc._cython.cygrpc.server_credentials_ssl_dynamic_cert_config", __pyx_clineno, __pyx_lineno, __pyx_filename);
  __pyx_r = NULL;
  __pyx_L0:;
  __Pyx_XDECREF((PyObject *)__pyx_v_credentials);
  __Pyx_XGIVEREF(__pyx_r);
  __Pyx_RefNannyFinishContext();
  return __pyx_r;
}

/* "src/python/grpcio/grpc/_cython/_cygrpc/credentials.pyx.pxi":294
 *   return credentials
 * 
 * cdef grpc_ssl_certificate_config_reload_status _server_cert_config_fetcher_wrapper(             # <<<<<<<<<<<<<<
 *         void* user_data, grpc_ssl_server_certificate_config **config) with gil:
 *   # This is a credentials.ServerCertificateConfig
 */

static grpc_ssl_certificate_config_reload_status __pyx_f_4grpc_7_cython_6cygrpc__server_cert_config_fetcher_wrapper(void *__pyx_v_user_data, grpc_ssl_server_certificate_config **__pyx_v_config) {
  struct __pyx_obj_4grpc_7_cython_6cygrpc_ServerCertificateConfig *__pyx_v_cert_config = 0;
  struct __pyx_obj_4grpc_7_cython_6cygrpc_ServerCredentials *__pyx_v_credentials = NULL;
  PyObject *__pyx_v_user_cb = NULL;
  PyObject *__pyx_v_cert_config_wrapper = NULL;
  grpc_ssl_certificate_config_reload_status __pyx_r;
  __Pyx_RefNannyDeclarations
  int __pyx_t_1;
  PyObject *__pyx_t_2 = NULL;
  PyObject *__pyx_t_3 = NULL;
  PyObject *__pyx_t_4 = NULL;
  PyObject *__pyx_t_5 = NULL;
  PyObject *__pyx_t_6 = NULL;
  PyObject *__pyx_t_7 = NULL;
  int __pyx_t_8;
  PyObject *__pyx_t_9 = NULL;
  PyObject *__pyx_t_10 = NULL;
  int __pyx_t_11;
  #ifdef WITH_THREAD
  PyGILState_STATE __pyx_gilstate_save = __Pyx_PyGILState_Ensure();
  #endif
  __Pyx_RefNannySetupContext("_server_cert_config_fetcher_wrapper", 0);

  /* "src/python/grpcio/grpc/_cython/_cygrpc/credentials.pyx.pxi":297
 *         void* user_data, grpc_ssl_server_certificate_config **config) with gil:
 *   # This is a credentials.ServerCertificateConfig
 *   cdef ServerCertificateConfig cert_config = None             # <<<<<<<<<<<<<<
 *   if not user_data:
 *     raise ValueError('internal error: user_data must be specified')
 */
  __Pyx_INCREF(Py_None);
  __pyx_v_cert_config = ((struct __pyx_obj_4grpc_7_cython_6cygrpc_ServerCertificateConfig *)Py_None);

  /* "src/python/grpcio/grpc/_cython/_cygrpc/credentials.pyx.pxi":298
 *   # This is a credentials.ServerCertificateConfig
 *   cdef ServerCertificateConfig cert_config = None
 *   if not user_data:             # <<<<<<<<<<<<<<
 *     raise ValueError('internal error: user_data must be specified')
 *   credentials = <ServerCredentials>user_data
 */
  __pyx_t_1 = ((!(__pyx_v_user_data != 0)) != 0);
  if (unlikely(__pyx_t_1)) {

    /* "src/python/grpcio/grpc/_cython/_cygrpc/credentials.pyx.pxi":299
 *   cdef ServerCertificateConfig cert_config = None
 *   if not user_data:
 *     raise ValueError('internal error: user_data must be specified')             # <<<<<<<<<<<<<<
 *   credentials = <ServerCredentials>user_data
 *   if not credentials.initial_cert_config_fetched:
 */
    __pyx_t_2 = __Pyx_PyObject_Call(__pyx_builtin_ValueError, __pyx_tuple__71, NULL); if (unlikely(!__pyx_t_2)) __PYX_ERR(4, 299, __pyx_L1_error)
    __Pyx_GOTREF(__pyx_t_2);
    __Pyx_Raise(__pyx_t_2, 0, 0, 0);
    __Pyx_DECREF(__pyx_t_2); __pyx_t_2 = 0;
    __PYX_ERR(4, 299, __pyx_L1_error)

    /* "src/python/grpcio/grpc/_cython/_cygrpc/credentials.pyx.pxi":298
 *   # This is a credentials.ServerCertificateConfig
 *   cdef ServerCertificateConfig cert_config = None
 *   if not user_data:             # <<<<<<<<<<<<<<
 *     raise ValueError('internal error: user_data must be specified')
 *   credentials = <ServerCredentials>user_data
 */
  }

  /* "src/python/grpcio/grpc/_cython/_cygrpc/credentials.pyx.pxi":300
 *   if not user_data:
 *     raise ValueError('internal error: user_data must be specified')
 *   credentials = <ServerCredentials>user_data             # <<<<<<<<<<<<<<
 *   if not credentials.initial_cert_config_fetched:
 *     # C-core is asking for the initial cert config
 */
  __pyx_t_2 = ((PyObject *)__pyx_v_user_data);
  __Pyx_INCREF(__pyx_t_2);
  __pyx_v_credentials = ((struct __pyx_obj_4grpc_7_cython_6cygrpc_ServerCredentials *)__pyx_t_2);
  __pyx_t_2 = 0;

  /* "src/python/grpcio/grpc/_cython/_cygrpc/credentials.pyx.pxi":301
 *     raise ValueError('internal error: user_data must be specified')
 *   credentials = <ServerCredentials>user_data
 *   if not credentials.initial_cert_config_fetched:             # <<<<<<<<<<<<<<
 *     # C-core is asking for the initial cert config
 *     credentials.initial_cert_config_fetched = True
 */
  __pyx_t_1 = ((!(__pyx_v_credentials->initial_cert_config_fetched != 0)) != 0);
  if (__pyx_t_1) {

    /* "src/python/grpcio/grpc/_cython/_cygrpc/credentials.pyx.pxi":303
 *   if not credentials.initial_cert_config_fetched:
 *     # C-core is asking for the initial cert config
 *     credentials.initial_cert_config_fetched = True             # <<<<<<<<<<<<<<
 *     cert_config = credentials.initial_cert_config._certificate_configuration
 *   else:
 */
    __pyx_v_credentials->initial_cert_config_fetched = 1;

    /* "src/python/grpcio/grpc/_cython/_cygrpc/credentials.pyx.pxi":304
 *     # C-core is asking for the initial cert config
 *     credentials.initial_cert_config_fetched = True
 *     cert_config = credentials.initial_cert_config._certificate_configuration             # <<<<<<<<<<<<<<
 *   else:
 *     user_cb = credentials.cert_config_fetcher
 */
    __pyx_t_2 = __Pyx_PyObject_GetAttrStr(__pyx_v_credentials->initial_cert_config, __pyx_n_s_certificate_configuration); if (unlikely(!__pyx_t_2)) __PYX_ERR(4, 304, __pyx_L1_error)
    __Pyx_GOTREF(__pyx_t_2);
    if (!(likely(((__pyx_t_2) == Py_None) || likely(__Pyx_TypeTest(__pyx_t_2, __pyx_ptype_4grpc_7_cython_6cygrpc_ServerCertificateConfig))))) __PYX_ERR(4, 304, __pyx_L1_error)
    __Pyx_DECREF_SET(__pyx_v_cert_config, ((struct __pyx_obj_4grpc_7_cython_6cygrpc_ServerCertificateConfig *)__pyx_t_2));
    __pyx_t_2 = 0;

    /* "src/python/grpcio/grpc/_cython/_cygrpc/credentials.pyx.pxi":301
 *     raise ValueError('internal error: user_data must be specified')
 *   credentials = <ServerCredentials>user_data
 *   if not credentials.initial_cert_config_fetched:             # <<<<<<<<<<<<<<
 *     # C-core is asking for the initial cert config
 *     credentials.initial_cert_config_fetched = True
 */
    goto __pyx_L4;
  }

  /* "src/python/grpcio/grpc/_cython/_cygrpc/credentials.pyx.pxi":306
 *     cert_config = credentials.initial_cert_config._certificate_configuration
 *   else:
 *     user_cb = credentials.cert_config_fetcher             # <<<<<<<<<<<<<<
 *     try:
 *       cert_config_wrapper = user_cb()
 */
  /*else*/ {
    __pyx_t_2 = __pyx_v_credentials->cert_config_fetcher;
    __Pyx_INCREF(__pyx_t_2);
    __pyx_v_user_cb = __pyx_t_2;
    __pyx_t_2 = 0;

    /* "src/python/grpcio/grpc/_cython/_cygrpc/credentials.pyx.pxi":307
 *   else:
 *     user_cb = credentials.cert_config_fetcher
 *     try:             # <<<<<<<<<<<<<<
 *       cert_config_wrapper = user_cb()
 *     except Exception:
 */
    {
      __Pyx_PyThreadState_declare
      __Pyx_PyThreadState_assign
      __Pyx_ExceptionSave(&__pyx_t_3, &__pyx_t_4, &__pyx_t_5);
      __Pyx_XGOTREF(__pyx_t_3);
      __Pyx_XGOTREF(__pyx_t_4);
      __Pyx_XGOTREF(__pyx_t_5);
      /*try:*/ {

        /* "src/python/grpcio/grpc/_cython/_cygrpc/credentials.pyx.pxi":308
 *     user_cb = credentials.cert_config_fetcher
 *     try:
 *       cert_config_wrapper = user_cb()             # <<<<<<<<<<<<<<
 *     except Exception:
 *       _LOGGER.exception('Error fetching certificate config')
 */
        __Pyx_INCREF(__pyx_v_user_cb);
        __pyx_t_6 = __pyx_v_user_cb; __pyx_t_7 = NULL;
        if (CYTHON_UNPACK_METHODS && likely(PyMethod_Check(__pyx_t_6))) {
          __pyx_t_7 = PyMethod_GET_SELF(__pyx_t_6);
          if (likely(__pyx_t_7)) {
            PyObject* function = PyMethod_GET_FUNCTION(__pyx_t_6);
            __Pyx_INCREF(__pyx_t_7);
            __Pyx_INCREF(function);
            __Pyx_DECREF_SET(__pyx_t_6, function);
          }
        }
        if (__pyx_t_7) {
          __pyx_t_2 = __Pyx_PyObject_CallOneArg(__pyx_t_6, __pyx_t_7); if (unlikely(!__pyx_t_2)) __PYX_ERR(4, 308, __pyx_L5_error)
          __Pyx_DECREF(__pyx_t_7); __pyx_t_7 = 0;
        } else {
          __pyx_t_2 = __Pyx_PyObject_CallNoArg(__pyx_t_6); if (unlikely(!__pyx_t_2)) __PYX_ERR(4, 308, __pyx_L5_error)
        }
        __Pyx_GOTREF(__pyx_t_2);
        __Pyx_DECREF(__pyx_t_6); __pyx_t_6 = 0;
        __pyx_v_cert_config_wrapper = __pyx_t_2;
        __pyx_t_2 = 0;

        /* "src/python/grpcio/grpc/_cython/_cygrpc/credentials.pyx.pxi":307
 *   else:
 *     user_cb = credentials.cert_config_fetcher
 *     try:             # <<<<<<<<<<<<<<
 *       cert_config_wrapper = user_cb()
 *     except Exception:
 */
      }
      __Pyx_XDECREF(__pyx_t_3); __pyx_t_3 = 0;
      __Pyx_XDECREF(__pyx_t_4); __pyx_t_4 = 0;
      __Pyx_XDECREF(__pyx_t_5); __pyx_t_5 = 0;
      goto __pyx_L10_try_end;
      __pyx_L5_error:;
      __Pyx_XDECREF(__pyx_t_7); __pyx_t_7 = 0;
      __Pyx_XDECREF(__pyx_t_6); __pyx_t_6 = 0;
      __Pyx_XDECREF(__pyx_t_2); __pyx_t_2 = 0;

      /* "src/python/grpcio/grpc/_cython/_cygrpc/credentials.pyx.pxi":309
 *     try:
 *       cert_config_wrapper = user_cb()
 *     except Exception:             # <<<<<<<<<<<<<<
 *       _LOGGER.exception('Error fetching certificate config')
 *       return GRPC_SSL_CERTIFICATE_CONFIG_RELOAD_FAIL
 */
      __pyx_t_8 = __Pyx_PyErr_ExceptionMatches(((PyObject *)(&((PyTypeObject*)PyExc_Exception)[0])));
      if (__pyx_t_8) {
        __Pyx_AddTraceback("grpc._cython.cygrpc._server_cert_config_fetcher_wrapper", __pyx_clineno, __pyx_lineno, __pyx_filename);
        if (__Pyx_GetException(&__pyx_t_2, &__pyx_t_6, &__pyx_t_7) < 0) __PYX_ERR(4, 309, __pyx_L7_except_error)
        __Pyx_GOTREF(__pyx_t_2);
        __Pyx_GOTREF(__pyx_t_6);
        __Pyx_GOTREF(__pyx_t_7);

        /* "src/python/grpcio/grpc/_cython/_cygrpc/credentials.pyx.pxi":310
 *       cert_config_wrapper = user_cb()
 *     except Exception:
 *       _LOGGER.exception('Error fetching certificate config')             # <<<<<<<<<<<<<<
 *       return GRPC_SSL_CERTIFICATE_CONFIG_RELOAD_FAIL
 *     if cert_config_wrapper is None:
 */
        __pyx_t_9 = __Pyx_GetModuleGlobalName(__pyx_n_s_LOGGER); if (unlikely(!__pyx_t_9)) __PYX_ERR(4, 310, __pyx_L7_except_error)
        __Pyx_GOTREF(__pyx_t_9);
        __pyx_t_10 = __Pyx_PyObject_GetAttrStr(__pyx_t_9, __pyx_n_s_exception); if (unlikely(!__pyx_t_10)) __PYX_ERR(4, 310, __pyx_L7_except_error)
        __Pyx_GOTREF(__pyx_t_10);
        __Pyx_DECREF(__pyx_t_9); __pyx_t_9 = 0;
        __pyx_t_9 = __Pyx_PyObject_Call(__pyx_t_10, __pyx_tuple__72, NULL); if (unlikely(!__pyx_t_9)) __PYX_ERR(4, 310, __pyx_L7_except_error)
        __Pyx_GOTREF(__pyx_t_9);
        __Pyx_DECREF(__pyx_t_10); __pyx_t_10 = 0;
        __Pyx_DECREF(__pyx_t_9); __pyx_t_9 = 0;

        /* "src/python/grpcio/grpc/_cython/_cygrpc/credentials.pyx.pxi":311
 *     except Exception:
 *       _LOGGER.exception('Error fetching certificate config')
 *       return GRPC_SSL_CERTIFICATE_CONFIG_RELOAD_FAIL             # <<<<<<<<<<<<<<
 *     if cert_config_wrapper is None:
 *       return GRPC_SSL_CERTIFICATE_CONFIG_RELOAD_UNCHANGED
 */
        __pyx_r = GRPC_SSL_CERTIFICATE_CONFIG_RELOAD_FAIL;
        __Pyx_DECREF(__pyx_t_2); __pyx_t_2 = 0;
        __Pyx_DECREF(__pyx_t_6); __pyx_t_6 = 0;
        __Pyx_DECREF(__pyx_t_7); __pyx_t_7 = 0;
        goto __pyx_L8_except_return;
      }
      goto __pyx_L7_except_error;
      __pyx_L7_except_error:;

      /* "src/python/grpcio/grpc/_cython/_cygrpc/credentials.pyx.pxi":307
 *   else:
 *     user_cb = credentials.cert_config_fetcher
 *     try:             # <<<<<<<<<<<<<<
 *       cert_config_wrapper = user_cb()
 *     except Exception:
 */
      __Pyx_XGIVEREF(__pyx_t_3);
      __Pyx_XGIVEREF(__pyx_t_4);
      __Pyx_XGIVEREF(__pyx_t_5);
      __Pyx_ExceptionReset(__pyx_t_3, __pyx_t_4, __pyx_t_5);
      goto __pyx_L1_error;
      __pyx_L8_except_return:;
      __Pyx_XGIVEREF(__pyx_t_3);
      __Pyx_XGIVEREF(__pyx_t_4);
      __Pyx_XGIVEREF(__pyx_t_5);
      __Pyx_ExceptionReset(__pyx_t_3, __pyx_t_4, __pyx_t_5);
      goto __pyx_L0;
      __pyx_L10_try_end:;
    }

    /* "src/python/grpcio/grpc/_cython/_cygrpc/credentials.pyx.pxi":312
 *       _LOGGER.exception('Error fetching certificate config')
 *       return GRPC_SSL_CERTIFICATE_CONFIG_RELOAD_FAIL
 *     if cert_config_wrapper is None:             # <<<<<<<<<<<<<<
 *       return GRPC_SSL_CERTIFICATE_CONFIG_RELOAD_UNCHANGED
 *     elif not isinstance(
 */
    __pyx_t_1 = (__pyx_v_cert_config_wrapper == Py_None);
    __pyx_t_11 = (__pyx_t_1 != 0);
    if (__pyx_t_11) {

      /* "src/python/grpcio/grpc/_cython/_cygrpc/credentials.pyx.pxi":313
 *       return GRPC_SSL_CERTIFICATE_CONFIG_RELOAD_FAIL
 *     if cert_config_wrapper is None:
 *       return GRPC_SSL_CERTIFICATE_CONFIG_RELOAD_UNCHANGED             # <<<<<<<<<<<<<<
 *     elif not isinstance(
 *         cert_config_wrapper, grpc.ServerCertificateConfiguration):
 */
      __pyx_r = GRPC_SSL_CERTIFICATE_CONFIG_RELOAD_UNCHANGED;
      goto __pyx_L0;

      /* "src/python/grpcio/grpc/_cython/_cygrpc/credentials.pyx.pxi":312
 *       _LOGGER.exception('Error fetching certificate config')
 *       return GRPC_SSL_CERTIFICATE_CONFIG_RELOAD_FAIL
 *     if cert_config_wrapper is None:             # <<<<<<<<<<<<<<
 *       return GRPC_SSL_CERTIFICATE_CONFIG_RELOAD_UNCHANGED
 *     elif not isinstance(
 */
    }

    /* "src/python/grpcio/grpc/_cython/_cygrpc/credentials.pyx.pxi":315
 *       return GRPC_SSL_CERTIFICATE_CONFIG_RELOAD_UNCHANGED
 *     elif not isinstance(
 *         cert_config_wrapper, grpc.ServerCertificateConfiguration):             # <<<<<<<<<<<<<<
 *       _LOGGER.error(
 *           'Error fetching certificate configuration: certificate '
 */
    __pyx_t_7 = __Pyx_GetModuleGlobalName(__pyx_n_s_grpc); if (unlikely(!__pyx_t_7)) __PYX_ERR(4, 315, __pyx_L1_error)
    __Pyx_GOTREF(__pyx_t_7);
    __pyx_t_6 = __Pyx_PyObject_GetAttrStr(__pyx_t_7, __pyx_n_s_ServerCertificateConfiguration); if (unlikely(!__pyx_t_6)) __PYX_ERR(4, 315, __pyx_L1_error)
    __Pyx_GOTREF(__pyx_t_6);
    __Pyx_DECREF(__pyx_t_7); __pyx_t_7 = 0;

    /* "src/python/grpcio/grpc/_cython/_cygrpc/credentials.pyx.pxi":314
 *     if cert_config_wrapper is None:
 *       return GRPC_SSL_CERTIFICATE_CONFIG_RELOAD_UNCHANGED
 *     elif not isinstance(             # <<<<<<<<<<<<<<
 *         cert_config_wrapper, grpc.ServerCertificateConfiguration):
 *       _LOGGER.error(
 */
    __pyx_t_11 = PyObject_IsInstance(__pyx_v_cert_config_wrapper, __pyx_t_6); if (unlikely(__pyx_t_11 == ((int)-1))) __PYX_ERR(4, 314, __pyx_L1_error)
    __Pyx_DECREF(__pyx_t_6); __pyx_t_6 = 0;
    __pyx_t_1 = ((!(__pyx_t_11 != 0)) != 0);
    if (__pyx_t_1) {

      /* "src/python/grpcio/grpc/_cython/_cygrpc/credentials.pyx.pxi":316
 *     elif not isinstance(
 *         cert_config_wrapper, grpc.ServerCertificateConfiguration):
 *       _LOGGER.error(             # <<<<<<<<<<<<<<
 *           'Error fetching certificate configuration: certificate '
 *           'configuration must be of type grpc.ServerCertificateConfiguration, '
 */
      __pyx_t_7 = __Pyx_GetModuleGlobalName(__pyx_n_s_LOGGER); if (unlikely(!__pyx_t_7)) __PYX_ERR(4, 316, __pyx_L1_error)
      __Pyx_GOTREF(__pyx_t_7);
      __pyx_t_2 = __Pyx_PyObject_GetAttrStr(__pyx_t_7, __pyx_n_s_error); if (unlikely(!__pyx_t_2)) __PYX_ERR(4, 316, __pyx_L1_error)
      __Pyx_GOTREF(__pyx_t_2);
      __Pyx_DECREF(__pyx_t_7); __pyx_t_7 = 0;

      /* "src/python/grpcio/grpc/_cython/_cygrpc/credentials.pyx.pxi":319
 *           'Error fetching certificate configuration: certificate '
 *           'configuration must be of type grpc.ServerCertificateConfiguration, '
 *           'not %s' % type(cert_config_wrapper).__name__)             # <<<<<<<<<<<<<<
 *       return GRPC_SSL_CERTIFICATE_CONFIG_RELOAD_FAIL
 *     else:
 */
      __pyx_t_7 = __Pyx_PyObject_GetAttrStr(((PyObject *)Py_TYPE(__pyx_v_cert_config_wrapper)), __pyx_n_s_name_2); if (unlikely(!__pyx_t_7)) __PYX_ERR(4, 319, __pyx_L1_error)
      __Pyx_GOTREF(__pyx_t_7);
      __pyx_t_9 = __Pyx_PyString_Format(__pyx_kp_s_Error_fetching_certificate_confi_2, __pyx_t_7); if (unlikely(!__pyx_t_9)) __PYX_ERR(4, 319, __pyx_L1_error)
      __Pyx_GOTREF(__pyx_t_9);
      __Pyx_DECREF(__pyx_t_7); __pyx_t_7 = 0;
      __pyx_t_7 = NULL;
      if (CYTHON_UNPACK_METHODS && unlikely(PyMethod_Check(__pyx_t_2))) {
        __pyx_t_7 = PyMethod_GET_SELF(__pyx_t_2);
        if (likely(__pyx_t_7)) {
          PyObject* function = PyMethod_GET_FUNCTION(__pyx_t_2);
          __Pyx_INCREF(__pyx_t_7);
          __Pyx_INCREF(function);
          __Pyx_DECREF_SET(__pyx_t_2, function);
        }
      }
      if (!__pyx_t_7) {
        __pyx_t_6 = __Pyx_PyObject_CallOneArg(__pyx_t_2, __pyx_t_9); if (unlikely(!__pyx_t_6)) __PYX_ERR(4, 316, __pyx_L1_error)
        __Pyx_DECREF(__pyx_t_9); __pyx_t_9 = 0;
        __Pyx_GOTREF(__pyx_t_6);
      } else {
        #if CYTHON_FAST_PYCALL
        if (PyFunction_Check(__pyx_t_2)) {
          PyObject *__pyx_temp[2] = {__pyx_t_7, __pyx_t_9};
          __pyx_t_6 = __Pyx_PyFunction_FastCall(__pyx_t_2, __pyx_temp+1-1, 1+1); if (unlikely(!__pyx_t_6)) __PYX_ERR(4, 316, __pyx_L1_error)
          __Pyx_XDECREF(__pyx_t_7); __pyx_t_7 = 0;
          __Pyx_GOTREF(__pyx_t_6);
          __Pyx_DECREF(__pyx_t_9); __pyx_t_9 = 0;
        } else
        #endif
        #if CYTHON_FAST_PYCCALL
        if (__Pyx_PyFastCFunction_Check(__pyx_t_2)) {
          PyObject *__pyx_temp[2] = {__pyx_t_7, __pyx_t_9};
          __pyx_t_6 = __Pyx_PyCFunction_FastCall(__pyx_t_2, __pyx_temp+1-1, 1+1); if (unlikely(!__pyx_t_6)) __PYX_ERR(4, 316, __pyx_L1_error)
          __Pyx_XDECREF(__pyx_t_7); __pyx_t_7 = 0;
          __Pyx_GOTREF(__pyx_t_6);
          __Pyx_DECREF(__pyx_t_9); __pyx_t_9 = 0;
        } else
        #endif
        {
          __pyx_t_10 = PyTuple_New(1+1); if (unlikely(!__pyx_t_10)) __PYX_ERR(4, 316, __pyx_L1_error)
          __Pyx_GOTREF(__pyx_t_10);
          __Pyx_GIVEREF(__pyx_t_7); PyTuple_SET_ITEM(__pyx_t_10, 0, __pyx_t_7); __pyx_t_7 = NULL;
          __Pyx_GIVEREF(__pyx_t_9);
          PyTuple_SET_ITEM(__pyx_t_10, 0+1, __pyx_t_9);
          __pyx_t_9 = 0;
          __pyx_t_6 = __Pyx_PyObject_Call(__pyx_t_2, __pyx_t_10, NULL); if (unlikely(!__pyx_t_6)) __PYX_ERR(4, 316, __pyx_L1_error)
          __Pyx_GOTREF(__pyx_t_6);
          __Pyx_DECREF(__pyx_t_10); __pyx_t_10 = 0;
        }
      }
      __Pyx_DECREF(__pyx_t_2); __pyx_t_2 = 0;
      __Pyx_DECREF(__pyx_t_6); __pyx_t_6 = 0;

      /* "src/python/grpcio/grpc/_cython/_cygrpc/credentials.pyx.pxi":320
 *           'configuration must be of type grpc.ServerCertificateConfiguration, '
 *           'not %s' % type(cert_config_wrapper).__name__)
 *       return GRPC_SSL_CERTIFICATE_CONFIG_RELOAD_FAIL             # <<<<<<<<<<<<<<
 *     else:
 *       cert_config = cert_config_wrapper._certificate_configuration
 */
      __pyx_r = GRPC_SSL_CERTIFICATE_CONFIG_RELOAD_FAIL;
      goto __pyx_L0;

      /* "src/python/grpcio/grpc/_cython/_cygrpc/credentials.pyx.pxi":314
 *     if cert_config_wrapper is None:
 *       return GRPC_SSL_CERTIFICATE_CONFIG_RELOAD_UNCHANGED
 *     elif not isinstance(             # <<<<<<<<<<<<<<
 *         cert_config_wrapper, grpc.ServerCertificateConfiguration):
 *       _LOGGER.error(
 */
    }

    /* "src/python/grpcio/grpc/_cython/_cygrpc/credentials.pyx.pxi":322
 *       return GRPC_SSL_CERTIFICATE_CONFIG_RELOAD_FAIL
 *     else:
 *       cert_config = cert_config_wrapper._certificate_configuration             # <<<<<<<<<<<<<<
 *   config[0] = <grpc_ssl_server_certificate_config*>cert_config.c_cert_config
 *   # our caller will assume ownership of memory, so we have to recreate
 */
    /*else*/ {
      __pyx_t_6 = __Pyx_PyObject_GetAttrStr(__pyx_v_cert_config_wrapper, __pyx_n_s_certificate_configuration); if (unlikely(!__pyx_t_6)) __PYX_ERR(4, 322, __pyx_L1_error)
      __Pyx_GOTREF(__pyx_t_6);
      if (!(likely(((__pyx_t_6) == Py_None) || likely(__Pyx_TypeTest(__pyx_t_6, __pyx_ptype_4grpc_7_cython_6cygrpc_ServerCertificateConfig))))) __PYX_ERR(4, 322, __pyx_L1_error)
      __Pyx_DECREF_SET(__pyx_v_cert_config, ((struct __pyx_obj_4grpc_7_cython_6cygrpc_ServerCertificateConfig *)__pyx_t_6));
      __pyx_t_6 = 0;
    }
  }
  __pyx_L4:;

  /* "src/python/grpcio/grpc/_cython/_cygrpc/credentials.pyx.pxi":323
 *     else:
 *       cert_config = cert_config_wrapper._certificate_configuration
 *   config[0] = <grpc_ssl_server_certificate_config*>cert_config.c_cert_config             # <<<<<<<<<<<<<<
 *   # our caller will assume ownership of memory, so we have to recreate
 *   # a copy of c_cert_config here
 */
  (__pyx_v_config[0]) = ((grpc_ssl_server_certificate_config *)__pyx_v_cert_config->c_cert_config);

  /* "src/python/grpcio/grpc/_cython/_cygrpc/credentials.pyx.pxi":326
 *   # our caller will assume ownership of memory, so we have to recreate
 *   # a copy of c_cert_config here
 *   cert_config.c_cert_config = grpc_ssl_server_certificate_config_create(             # <<<<<<<<<<<<<<
 *       cert_config.c_pem_root_certs, cert_config.c_ssl_pem_key_cert_pairs,
 *       cert_config.c_ssl_pem_key_cert_pairs_count)
 */
  __pyx_v_cert_config->c_cert_config = grpc_ssl_server_certificate_config_create(__pyx_v_cert_config->c_pem_root_certs, __pyx_v_cert_config->c_ssl_pem_key_cert_pairs, __pyx_v_cert_config->c_ssl_pem_key_cert_pairs_count);

  /* "src/python/grpcio/grpc/_cython/_cygrpc/credentials.pyx.pxi":329
 *       cert_config.c_pem_root_certs, cert_config.c_ssl_pem_key_cert_pairs,
 *       cert_config.c_ssl_pem_key_cert_pairs_count)
 *   return GRPC_SSL_CERTIFICATE_CONFIG_RELOAD_NEW             # <<<<<<<<<<<<<<
 * 
 */
  __pyx_r = GRPC_SSL_CERTIFICATE_CONFIG_RELOAD_NEW;
  goto __pyx_L0;

  /* "src/python/grpcio/grpc/_cython/_cygrpc/credentials.pyx.pxi":294
 *   return credentials
 * 
 * cdef grpc_ssl_certificate_config_reload_status _server_cert_config_fetcher_wrapper(             # <<<<<<<<<<<<<<
 *         void* user_data, grpc_ssl_server_certificate_config **config) with gil:
 *   # This is a credentials.ServerCertificateConfig
 */

  /* function exit code */
  __pyx_L1_error:;
  __Pyx_XDECREF(__pyx_t_2);
  __Pyx_XDECREF(__pyx_t_6);
  __Pyx_XDECREF(__pyx_t_7);
  __Pyx_XDECREF(__pyx_t_9);
  __Pyx_XDECREF(__pyx_t_10);
  __Pyx_WriteUnraisable("grpc._cython.cygrpc._server_cert_config_fetcher_wrapper", __pyx_clineno, __pyx_lineno, __pyx_filename, 1, 0);
  __pyx_r = (grpc_ssl_certificate_config_reload_status) 0;
  __pyx_L0:;
  __Pyx_XDECREF((PyObject *)__pyx_v_cert_config);
  __Pyx_XDECREF((PyObject *)__pyx_v_credentials);
  __Pyx_XDECREF(__pyx_v_user_cb);
  __Pyx_XDECREF(__pyx_v_cert_config_wrapper);
  __Pyx_RefNannyFinishContext();
  #ifdef WITH_THREAD
  __Pyx_PyGILState_Release(__pyx_gilstate_save);
  #endif
  return __pyx_r;
}

/* "src/python/grpcio/grpc/_cython/_cygrpc/completion_queue.pyx.pxi":23
 * 
 * 
 * cdef grpc_event _next(grpc_completion_queue *c_completion_queue, deadline) except *:             # <<<<<<<<<<<<<<
 *   cdef gpr_timespec c_increment
 *   cdef gpr_timespec c_timeout
 */

static grpc_event __pyx_f_4grpc_7_cython_6cygrpc__next(grpc_completion_queue *__pyx_v_c_completion_queue, PyObject *__pyx_v_deadline) {
  gpr_timespec __pyx_v_c_increment;
  gpr_timespec __pyx_v_c_timeout;
  gpr_timespec __pyx_v_c_deadline;
  grpc_event __pyx_v_c_event;
  grpc_event __pyx_r;
  __Pyx_RefNannyDeclarations
  int __pyx_t_1;
  int __pyx_t_2;
  gpr_timespec __pyx_t_3;
  int __pyx_t_4;
  __Pyx_RefNannySetupContext("_next", 0);

  /* "src/python/grpcio/grpc/_cython/_cygrpc/completion_queue.pyx.pxi":27
 *   cdef gpr_timespec c_timeout
 *   cdef gpr_timespec c_deadline
 *   c_increment = gpr_time_from_millis(_INTERRUPT_CHECK_PERIOD_MS, GPR_TIMESPAN)             # <<<<<<<<<<<<<<
 *   if deadline is None:
 *     c_deadline = gpr_inf_future(GPR_CLOCK_REALTIME)
 */
  __pyx_v_c_increment = gpr_time_from_millis(__pyx_v_4grpc_7_cython_6cygrpc__INTERRUPT_CHECK_PERIOD_MS, GPR_TIMESPAN);

  /* "src/python/grpcio/grpc/_cython/_cygrpc/completion_queue.pyx.pxi":28
 *   cdef gpr_timespec c_deadline
 *   c_increment = gpr_time_from_millis(_INTERRUPT_CHECK_PERIOD_MS, GPR_TIMESPAN)
 *   if deadline is None:             # <<<<<<<<<<<<<<
 *     c_deadline = gpr_inf_future(GPR_CLOCK_REALTIME)
 *   else:
 */
  __pyx_t_1 = (__pyx_v_deadline == Py_None);
  __pyx_t_2 = (__pyx_t_1 != 0);
  if (__pyx_t_2) {

    /* "src/python/grpcio/grpc/_cython/_cygrpc/completion_queue.pyx.pxi":29
 *   c_increment = gpr_time_from_millis(_INTERRUPT_CHECK_PERIOD_MS, GPR_TIMESPAN)
 *   if deadline is None:
 *     c_deadline = gpr_inf_future(GPR_CLOCK_REALTIME)             # <<<<<<<<<<<<<<
 *   else:
 *     c_deadline = _timespec_from_time(deadline)
 */
    __pyx_v_c_deadline = gpr_inf_future(GPR_CLOCK_REALTIME);

    /* "src/python/grpcio/grpc/_cython/_cygrpc/completion_queue.pyx.pxi":28
 *   cdef gpr_timespec c_deadline
 *   c_increment = gpr_time_from_millis(_INTERRUPT_CHECK_PERIOD_MS, GPR_TIMESPAN)
 *   if deadline is None:             # <<<<<<<<<<<<<<
 *     c_deadline = gpr_inf_future(GPR_CLOCK_REALTIME)
 *   else:
 */
    goto __pyx_L3;
  }

  /* "src/python/grpcio/grpc/_cython/_cygrpc/completion_queue.pyx.pxi":31
 *     c_deadline = gpr_inf_future(GPR_CLOCK_REALTIME)
 *   else:
 *     c_deadline = _timespec_from_time(deadline)             # <<<<<<<<<<<<<<
 * 
 *   while True:
 */
  /*else*/ {
    __pyx_t_3 = __pyx_f_4grpc_7_cython_6cygrpc__timespec_from_time(__pyx_v_deadline); if (unlikely(PyErr_Occurred())) __PYX_ERR(10, 31, __pyx_L1_error)
    __pyx_v_c_deadline = __pyx_t_3;
  }
  __pyx_L3:;

  /* "src/python/grpcio/grpc/_cython/_cygrpc/completion_queue.pyx.pxi":33
 *     c_deadline = _timespec_from_time(deadline)
 * 
 *   while True:             # <<<<<<<<<<<<<<
 *     with nogil:
 *       c_timeout = gpr_time_add(gpr_now(GPR_CLOCK_REALTIME), c_increment)
 */
  while (1) {

    /* "src/python/grpcio/grpc/_cython/_cygrpc/completion_queue.pyx.pxi":34
 * 
 *   while True:
 *     with nogil:             # <<<<<<<<<<<<<<
 *       c_timeout = gpr_time_add(gpr_now(GPR_CLOCK_REALTIME), c_increment)
 *       if gpr_time_cmp(c_timeout, c_deadline) > 0:
 */
    {
        #ifdef WITH_THREAD
        PyThreadState *_save;
        Py_UNBLOCK_THREADS
        __Pyx_FastGIL_Remember();
        #endif
        /*try:*/ {

          /* "src/python/grpcio/grpc/_cython/_cygrpc/completion_queue.pyx.pxi":35
 *   while True:
 *     with nogil:
 *       c_timeout = gpr_time_add(gpr_now(GPR_CLOCK_REALTIME), c_increment)             # <<<<<<<<<<<<<<
 *       if gpr_time_cmp(c_timeout, c_deadline) > 0:
 *         c_timeout = c_deadline
 */
          __pyx_v_c_timeout = gpr_time_add(gpr_now(GPR_CLOCK_REALTIME), __pyx_v_c_increment);

          /* "src/python/grpcio/grpc/_cython/_cygrpc/completion_queue.pyx.pxi":36
 *     with nogil:
 *       c_timeout = gpr_time_add(gpr_now(GPR_CLOCK_REALTIME), c_increment)
 *       if gpr_time_cmp(c_timeout, c_deadline) > 0:             # <<<<<<<<<<<<<<
 *         c_timeout = c_deadline
 * 
 */
          __pyx_t_2 = ((gpr_time_cmp(__pyx_v_c_timeout, __pyx_v_c_deadline) > 0) != 0);
          if (__pyx_t_2) {

            /* "src/python/grpcio/grpc/_cython/_cygrpc/completion_queue.pyx.pxi":37
 *       c_timeout = gpr_time_add(gpr_now(GPR_CLOCK_REALTIME), c_increment)
 *       if gpr_time_cmp(c_timeout, c_deadline) > 0:
 *         c_timeout = c_deadline             # <<<<<<<<<<<<<<
 * 
 *       c_event = grpc_completion_queue_next(c_completion_queue, c_timeout, NULL)
 */
            __pyx_v_c_timeout = __pyx_v_c_deadline;

            /* "src/python/grpcio/grpc/_cython/_cygrpc/completion_queue.pyx.pxi":36
 *     with nogil:
 *       c_timeout = gpr_time_add(gpr_now(GPR_CLOCK_REALTIME), c_increment)
 *       if gpr_time_cmp(c_timeout, c_deadline) > 0:             # <<<<<<<<<<<<<<
 *         c_timeout = c_deadline
 * 
 */
          }

          /* "src/python/grpcio/grpc/_cython/_cygrpc/completion_queue.pyx.pxi":39
 *         c_timeout = c_deadline
 * 
 *       c_event = grpc_completion_queue_next(c_completion_queue, c_timeout, NULL)             # <<<<<<<<<<<<<<
 * 
 *       if (c_event.type != GRPC_QUEUE_TIMEOUT or
 */
          __pyx_v_c_event = grpc_completion_queue_next(__pyx_v_c_completion_queue, __pyx_v_c_timeout, NULL);

          /* "src/python/grpcio/grpc/_cython/_cygrpc/completion_queue.pyx.pxi":41
 *       c_event = grpc_completion_queue_next(c_completion_queue, c_timeout, NULL)
 * 
 *       if (c_event.type != GRPC_QUEUE_TIMEOUT or             # <<<<<<<<<<<<<<
 *           gpr_time_cmp(c_timeout, c_deadline) == 0):
 *         break
 */
          __pyx_t_1 = ((__pyx_v_c_event.type != GRPC_QUEUE_TIMEOUT) != 0);
          if (!__pyx_t_1) {
          } else {
            __pyx_t_2 = __pyx_t_1;
            goto __pyx_L13_bool_binop_done;
          }

          /* "src/python/grpcio/grpc/_cython/_cygrpc/completion_queue.pyx.pxi":42
 * 
 *       if (c_event.type != GRPC_QUEUE_TIMEOUT or
 *           gpr_time_cmp(c_timeout, c_deadline) == 0):             # <<<<<<<<<<<<<<
 *         break
 * 
 */
          __pyx_t_1 = ((gpr_time_cmp(__pyx_v_c_timeout, __pyx_v_c_deadline) == 0) != 0);
          __pyx_t_2 = __pyx_t_1;
          __pyx_L13_bool_binop_done:;

          /* "src/python/grpcio/grpc/_cython/_cygrpc/completion_queue.pyx.pxi":41
 *       c_event = grpc_completion_queue_next(c_completion_queue, c_timeout, NULL)
 * 
 *       if (c_event.type != GRPC_QUEUE_TIMEOUT or             # <<<<<<<<<<<<<<
 *           gpr_time_cmp(c_timeout, c_deadline) == 0):
 *         break
 */
          if (__pyx_t_2) {

            /* "src/python/grpcio/grpc/_cython/_cygrpc/completion_queue.pyx.pxi":43
 *       if (c_event.type != GRPC_QUEUE_TIMEOUT or
 *           gpr_time_cmp(c_timeout, c_deadline) == 0):
 *         break             # <<<<<<<<<<<<<<
 * 
 *     # Handle any signals
 */
            goto __pyx_L7_break;

            /* "src/python/grpcio/grpc/_cython/_cygrpc/completion_queue.pyx.pxi":41
 *       c_event = grpc_completion_queue_next(c_completion_queue, c_timeout, NULL)
 * 
 *       if (c_event.type != GRPC_QUEUE_TIMEOUT or             # <<<<<<<<<<<<<<
 *           gpr_time_cmp(c_timeout, c_deadline) == 0):
 *         break
 */
          }
        }

        /* "src/python/grpcio/grpc/_cython/_cygrpc/completion_queue.pyx.pxi":34
 * 
 *   while True:
 *     with nogil:             # <<<<<<<<<<<<<<
 *       c_timeout = gpr_time_add(gpr_now(GPR_CLOCK_REALTIME), c_increment)
 *       if gpr_time_cmp(c_timeout, c_deadline) > 0:
 */
        /*finally:*/ {
          /*normal exit:*/{
            #ifdef WITH_THREAD
            __Pyx_FastGIL_Forget();
            Py_BLOCK_THREADS
            #endif
            goto __pyx_L10;
          }
          __pyx_L7_break: {
            #ifdef WITH_THREAD
            __Pyx_FastGIL_Forget();
            Py_BLOCK_THREADS
            #endif
            goto __pyx_L5_break;
          }
          __pyx_L10:;
        }
    }

    /* "src/python/grpcio/grpc/_cython/_cygrpc/completion_queue.pyx.pxi":46
 * 
 *     # Handle any signals
 *     cpython.PyErr_CheckSignals()             # <<<<<<<<<<<<<<
 *   return c_event
 * 
 */
    __pyx_t_4 = PyErr_CheckSignals(); if (unlikely(__pyx_t_4 == ((int)-1))) __PYX_ERR(10, 46, __pyx_L1_error)
  }
  __pyx_L5_break:;

  /* "src/python/grpcio/grpc/_cython/_cygrpc/completion_queue.pyx.pxi":47
 *     # Handle any signals
 *     cpython.PyErr_CheckSignals()
 *   return c_event             # <<<<<<<<<<<<<<
 * 
 * 
 */
  __pyx_r = __pyx_v_c_event;
  goto __pyx_L0;

  /* "src/python/grpcio/grpc/_cython/_cygrpc/completion_queue.pyx.pxi":23
 * 
 * 
 * cdef grpc_event _next(grpc_completion_queue *c_completion_queue, deadline) except *:             # <<<<<<<<<<<<<<
 *   cdef gpr_timespec c_increment
 *   cdef gpr_timespec c_timeout
 */

  /* function exit code */
  __pyx_L1_error:;
  __Pyx_AddTraceback("grpc._cython.cygrpc._next", __pyx_clineno, __pyx_lineno, __pyx_filename);
  __Pyx_pretend_to_initialize(&__pyx_r);
  __pyx_L0:;
  __Pyx_RefNannyFinishContext();
  return __pyx_r;
}

/* "src/python/grpcio/grpc/_cython/_cygrpc/completion_queue.pyx.pxi":50
 * 
 * 
 * cdef _interpret_event(grpc_event c_event):             # <<<<<<<<<<<<<<
 *   cdef _Tag tag
 *   if c_event.type == GRPC_QUEUE_TIMEOUT:
 */

static PyObject *__pyx_f_4grpc_7_cython_6cygrpc__interpret_event(grpc_event __pyx_v_c_event) {
  struct __pyx_obj_4grpc_7_cython_6cygrpc__Tag *__pyx_v_tag = 0;
  PyObject *__pyx_r = NULL;
  __Pyx_RefNannyDeclarations
  PyObject *__pyx_t_1 = NULL;
  PyObject *__pyx_t_2 = NULL;
  __Pyx_RefNannySetupContext("_interpret_event", 0);

  /* "src/python/grpcio/grpc/_cython/_cygrpc/completion_queue.pyx.pxi":52
 * cdef _interpret_event(grpc_event c_event):
 *   cdef _Tag tag
 *   if c_event.type == GRPC_QUEUE_TIMEOUT:             # <<<<<<<<<<<<<<
 *     # TODO(ericgribkoff) Do not coopt ConnectivityEvent here.
 *     return None, ConnectivityEvent(GRPC_QUEUE_TIMEOUT, False, None)
 */
  switch (__pyx_v_c_event.type) {
    case GRPC_QUEUE_TIMEOUT:

    /* "src/python/grpcio/grpc/_cython/_cygrpc/completion_queue.pyx.pxi":54
 *   if c_event.type == GRPC_QUEUE_TIMEOUT:
 *     # TODO(ericgribkoff) Do not coopt ConnectivityEvent here.
 *     return None, ConnectivityEvent(GRPC_QUEUE_TIMEOUT, False, None)             # <<<<<<<<<<<<<<
 *   elif c_event.type == GRPC_QUEUE_SHUTDOWN:
 *     # NOTE(nathaniel): For now we coopt ConnectivityEvent here.
 */
    __Pyx_XDECREF(__pyx_r);
    __pyx_t_1 = __Pyx_PyInt_From_grpc_completion_type(GRPC_QUEUE_TIMEOUT); if (unlikely(!__pyx_t_1)) __PYX_ERR(10, 54, __pyx_L1_error)
    __Pyx_GOTREF(__pyx_t_1);
    __pyx_t_2 = PyTuple_New(3); if (unlikely(!__pyx_t_2)) __PYX_ERR(10, 54, __pyx_L1_error)
    __Pyx_GOTREF(__pyx_t_2);
    __Pyx_GIVEREF(__pyx_t_1);
    PyTuple_SET_ITEM(__pyx_t_2, 0, __pyx_t_1);
    __Pyx_INCREF(Py_False);
    __Pyx_GIVEREF(Py_False);
    PyTuple_SET_ITEM(__pyx_t_2, 1, Py_False);
    __Pyx_INCREF(Py_None);
    __Pyx_GIVEREF(Py_None);
    PyTuple_SET_ITEM(__pyx_t_2, 2, Py_None);
    __pyx_t_1 = 0;
    __pyx_t_1 = __Pyx_PyObject_Call(((PyObject *)__pyx_ptype_4grpc_7_cython_6cygrpc_ConnectivityEvent), __pyx_t_2, NULL); if (unlikely(!__pyx_t_1)) __PYX_ERR(10, 54, __pyx_L1_error)
    __Pyx_GOTREF(__pyx_t_1);
    __Pyx_DECREF(__pyx_t_2); __pyx_t_2 = 0;
    __pyx_t_2 = PyTuple_New(2); if (unlikely(!__pyx_t_2)) __PYX_ERR(10, 54, __pyx_L1_error)
    __Pyx_GOTREF(__pyx_t_2);
    __Pyx_INCREF(Py_None);
    __Pyx_GIVEREF(Py_None);
    PyTuple_SET_ITEM(__pyx_t_2, 0, Py_None);
    __Pyx_GIVEREF(__pyx_t_1);
    PyTuple_SET_ITEM(__pyx_t_2, 1, __pyx_t_1);
    __pyx_t_1 = 0;
    __pyx_r = __pyx_t_2;
    __pyx_t_2 = 0;
    goto __pyx_L0;

    /* "src/python/grpcio/grpc/_cython/_cygrpc/completion_queue.pyx.pxi":52
 * cdef _interpret_event(grpc_event c_event):
 *   cdef _Tag tag
 *   if c_event.type == GRPC_QUEUE_TIMEOUT:             # <<<<<<<<<<<<<<
 *     # TODO(ericgribkoff) Do not coopt ConnectivityEvent here.
 *     return None, ConnectivityEvent(GRPC_QUEUE_TIMEOUT, False, None)
 */
    break;

    /* "src/python/grpcio/grpc/_cython/_cygrpc/completion_queue.pyx.pxi":55
 *     # TODO(ericgribkoff) Do not coopt ConnectivityEvent here.
 *     return None, ConnectivityEvent(GRPC_QUEUE_TIMEOUT, False, None)
 *   elif c_event.type == GRPC_QUEUE_SHUTDOWN:             # <<<<<<<<<<<<<<
 *     # NOTE(nathaniel): For now we coopt ConnectivityEvent here.
 *     return None, ConnectivityEvent(GRPC_QUEUE_SHUTDOWN, False, None)
 */
    case GRPC_QUEUE_SHUTDOWN:

    /* "src/python/grpcio/grpc/_cython/_cygrpc/completion_queue.pyx.pxi":57
 *   elif c_event.type == GRPC_QUEUE_SHUTDOWN:
 *     # NOTE(nathaniel): For now we coopt ConnectivityEvent here.
 *     return None, ConnectivityEvent(GRPC_QUEUE_SHUTDOWN, False, None)             # <<<<<<<<<<<<<<
 *   else:
 *     tag = <_Tag>c_event.tag
 */
    __Pyx_XDECREF(__pyx_r);
    __pyx_t_2 = __Pyx_PyInt_From_grpc_completion_type(GRPC_QUEUE_SHUTDOWN); if (unlikely(!__pyx_t_2)) __PYX_ERR(10, 57, __pyx_L1_error)
    __Pyx_GOTREF(__pyx_t_2);
    __pyx_t_1 = PyTuple_New(3); if (unlikely(!__pyx_t_1)) __PYX_ERR(10, 57, __pyx_L1_error)
    __Pyx_GOTREF(__pyx_t_1);
    __Pyx_GIVEREF(__pyx_t_2);
    PyTuple_SET_ITEM(__pyx_t_1, 0, __pyx_t_2);
    __Pyx_INCREF(Py_False);
    __Pyx_GIVEREF(Py_False);
    PyTuple_SET_ITEM(__pyx_t_1, 1, Py_False);
    __Pyx_INCREF(Py_None);
    __Pyx_GIVEREF(Py_None);
    PyTuple_SET_ITEM(__pyx_t_1, 2, Py_None);
    __pyx_t_2 = 0;
    __pyx_t_2 = __Pyx_PyObject_Call(((PyObject *)__pyx_ptype_4grpc_7_cython_6cygrpc_ConnectivityEvent), __pyx_t_1, NULL); if (unlikely(!__pyx_t_2)) __PYX_ERR(10, 57, __pyx_L1_error)
    __Pyx_GOTREF(__pyx_t_2);
    __Pyx_DECREF(__pyx_t_1); __pyx_t_1 = 0;
    __pyx_t_1 = PyTuple_New(2); if (unlikely(!__pyx_t_1)) __PYX_ERR(10, 57, __pyx_L1_error)
    __Pyx_GOTREF(__pyx_t_1);
    __Pyx_INCREF(Py_None);
    __Pyx_GIVEREF(Py_None);
    PyTuple_SET_ITEM(__pyx_t_1, 0, Py_None);
    __Pyx_GIVEREF(__pyx_t_2);
    PyTuple_SET_ITEM(__pyx_t_1, 1, __pyx_t_2);
    __pyx_t_2 = 0;
    __pyx_r = __pyx_t_1;
    __pyx_t_1 = 0;
    goto __pyx_L0;

    /* "src/python/grpcio/grpc/_cython/_cygrpc/completion_queue.pyx.pxi":55
 *     # TODO(ericgribkoff) Do not coopt ConnectivityEvent here.
 *     return None, ConnectivityEvent(GRPC_QUEUE_TIMEOUT, False, None)
 *   elif c_event.type == GRPC_QUEUE_SHUTDOWN:             # <<<<<<<<<<<<<<
 *     # NOTE(nathaniel): For now we coopt ConnectivityEvent here.
 *     return None, ConnectivityEvent(GRPC_QUEUE_SHUTDOWN, False, None)
 */
    break;
    default:

    /* "src/python/grpcio/grpc/_cython/_cygrpc/completion_queue.pyx.pxi":59
 *     return None, ConnectivityEvent(GRPC_QUEUE_SHUTDOWN, False, None)
 *   else:
 *     tag = <_Tag>c_event.tag             # <<<<<<<<<<<<<<
 *     # We receive event tags only after they've been inc-ref'd elsewhere in
 *     # the code.
 */
    __pyx_t_1 = ((PyObject *)__pyx_v_c_event.tag);
    __Pyx_INCREF(__pyx_t_1);
    __pyx_v_tag = ((struct __pyx_obj_4grpc_7_cython_6cygrpc__Tag *)__pyx_t_1);
    __pyx_t_1 = 0;

    /* "src/python/grpcio/grpc/_cython/_cygrpc/completion_queue.pyx.pxi":62
 *     # We receive event tags only after they've been inc-ref'd elsewhere in
 *     # the code.
 *     cpython.Py_DECREF(tag)             # <<<<<<<<<<<<<<
 *     return tag, tag.event(c_event)
 * 
 */
    Py_DECREF(((PyObject *)__pyx_v_tag));

    /* "src/python/grpcio/grpc/_cython/_cygrpc/completion_queue.pyx.pxi":63
 *     # the code.
 *     cpython.Py_DECREF(tag)
 *     return tag, tag.event(c_event)             # <<<<<<<<<<<<<<
 * 
 * 
 */
    __Pyx_XDECREF(__pyx_r);
    __pyx_t_1 = ((struct __pyx_vtabstruct_4grpc_7_cython_6cygrpc__Tag *)__pyx_v_tag->__pyx_vtab)->event(__pyx_v_tag, __pyx_v_c_event); if (unlikely(!__pyx_t_1)) __PYX_ERR(10, 63, __pyx_L1_error)
    __Pyx_GOTREF(__pyx_t_1);
    __pyx_t_2 = PyTuple_New(2); if (unlikely(!__pyx_t_2)) __PYX_ERR(10, 63, __pyx_L1_error)
    __Pyx_GOTREF(__pyx_t_2);
    __Pyx_INCREF(((PyObject *)__pyx_v_tag));
    __Pyx_GIVEREF(((PyObject *)__pyx_v_tag));
    PyTuple_SET_ITEM(__pyx_t_2, 0, ((PyObject *)__pyx_v_tag));
    __Pyx_GIVEREF(__pyx_t_1);
    PyTuple_SET_ITEM(__pyx_t_2, 1, __pyx_t_1);
    __pyx_t_1 = 0;
    __pyx_r = __pyx_t_2;
    __pyx_t_2 = 0;
    goto __pyx_L0;
    break;
  }

  /* "src/python/grpcio/grpc/_cython/_cygrpc/completion_queue.pyx.pxi":50
 * 
 * 
 * cdef _interpret_event(grpc_event c_event):             # <<<<<<<<<<<<<<
 *   cdef _Tag tag
 *   if c_event.type == GRPC_QUEUE_TIMEOUT:
 */

  /* function exit code */
  __pyx_L1_error:;
  __Pyx_XDECREF(__pyx_t_1);
  __Pyx_XDECREF(__pyx_t_2);
  __Pyx_AddTraceback("grpc._cython.cygrpc._interpret_event", __pyx_clineno, __pyx_lineno, __pyx_filename);
  __pyx_r = 0;
  __pyx_L0:;
  __Pyx_XDECREF((PyObject *)__pyx_v_tag);
  __Pyx_XGIVEREF(__pyx_r);
  __Pyx_RefNannyFinishContext();
  return __pyx_r;
}

/* "src/python/grpcio/grpc/_cython/_cygrpc/completion_queue.pyx.pxi":66
 * 
 * 
 * cdef _latent_event(grpc_completion_queue *c_completion_queue, object deadline):             # <<<<<<<<<<<<<<
 *   cdef grpc_event c_event = _next(c_completion_queue, deadline)
 *   return _interpret_event(c_event)
 */

static PyObject *__pyx_f_4grpc_7_cython_6cygrpc__latent_event(grpc_completion_queue *__pyx_v_c_completion_queue, PyObject *__pyx_v_deadline) {
  grpc_event __pyx_v_c_event;
  PyObject *__pyx_r = NULL;
  __Pyx_RefNannyDeclarations
  grpc_event __pyx_t_1;
  PyObject *__pyx_t_2 = NULL;
  __Pyx_RefNannySetupContext("_latent_event", 0);

  /* "src/python/grpcio/grpc/_cython/_cygrpc/completion_queue.pyx.pxi":67
 * 
 * cdef _latent_event(grpc_completion_queue *c_completion_queue, object deadline):
 *   cdef grpc_event c_event = _next(c_completion_queue, deadline)             # <<<<<<<<<<<<<<
 *   return _interpret_event(c_event)
 * 
 */
  __pyx_t_1 = __pyx_f_4grpc_7_cython_6cygrpc__next(__pyx_v_c_completion_queue, __pyx_v_deadline); if (unlikely(PyErr_Occurred())) __PYX_ERR(10, 67, __pyx_L1_error)
  __pyx_v_c_event = __pyx_t_1;

  /* "src/python/grpcio/grpc/_cython/_cygrpc/completion_queue.pyx.pxi":68
 * cdef _latent_event(grpc_completion_queue *c_completion_queue, object deadline):
 *   cdef grpc_event c_event = _next(c_completion_queue, deadline)
 *   return _interpret_event(c_event)             # <<<<<<<<<<<<<<
 * 
 * 
 */
  __Pyx_XDECREF(__pyx_r);
  __pyx_t_2 = __pyx_f_4grpc_7_cython_6cygrpc__interpret_event(__pyx_v_c_event); if (unlikely(!__pyx_t_2)) __PYX_ERR(10, 68, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_2);
  __pyx_r = __pyx_t_2;
  __pyx_t_2 = 0;
  goto __pyx_L0;

  /* "src/python/grpcio/grpc/_cython/_cygrpc/completion_queue.pyx.pxi":66
 * 
 * 
 * cdef _latent_event(grpc_completion_queue *c_completion_queue, object deadline):             # <<<<<<<<<<<<<<
 *   cdef grpc_event c_event = _next(c_completion_queue, deadline)
 *   return _interpret_event(c_event)
 */

  /* function exit code */
  __pyx_L1_error:;
  __Pyx_XDECREF(__pyx_t_2);
  __Pyx_AddTraceback("grpc._cython.cygrpc._latent_event", __pyx_clineno, __pyx_lineno, __pyx_filename);
  __pyx_r = 0;
  __pyx_L0:;
  __Pyx_XGIVEREF(__pyx_r);
  __Pyx_RefNannyFinishContext();
  return __pyx_r;
}

/* "src/python/grpcio/grpc/_cython/_cygrpc/completion_queue.pyx.pxi":73
 * cdef class CompletionQueue:
 * 
 *   def __cinit__(self, shutdown_cq=False):             # <<<<<<<<<<<<<<
 *     cdef grpc_completion_queue_attributes c_attrs
 *     fork_handlers_and_grpc_init()
 */

/* Python wrapper */
static int __pyx_pw_4grpc_7_cython_6cygrpc_15CompletionQueue_1__cinit__(PyObject *__pyx_v_self, PyObject *__pyx_args, PyObject *__pyx_kwds); /*proto*/
static int __pyx_pw_4grpc_7_cython_6cygrpc_15CompletionQueue_1__cinit__(PyObject *__pyx_v_self, PyObject *__pyx_args, PyObject *__pyx_kwds) {
  PyObject *__pyx_v_shutdown_cq = 0;
  int __pyx_r;
  __Pyx_RefNannyDeclarations
  __Pyx_RefNannySetupContext("__cinit__ (wrapper)", 0);
  {
    static PyObject **__pyx_pyargnames[] = {&__pyx_n_s_shutdown_cq,0};
    PyObject* values[1] = {0};
    values[0] = ((PyObject *)Py_False);
    if (unlikely(__pyx_kwds)) {
      Py_ssize_t kw_args;
      const Py_ssize_t pos_args = PyTuple_GET_SIZE(__pyx_args);
      switch (pos_args) {
        case  1: values[0] = PyTuple_GET_ITEM(__pyx_args, 0);
        CYTHON_FALLTHROUGH;
        case  0: break;
        default: goto __pyx_L5_argtuple_error;
      }
      kw_args = PyDict_Size(__pyx_kwds);
      switch (pos_args) {
        case  0:
        if (kw_args > 0) {
          PyObject* value = __Pyx_PyDict_GetItemStr(__pyx_kwds, __pyx_n_s_shutdown_cq);
          if (value) { values[0] = value; kw_args--; }
        }
      }
      if (unlikely(kw_args > 0)) {
        if (unlikely(__Pyx_ParseOptionalKeywords(__pyx_kwds, __pyx_pyargnames, 0, values, pos_args, "__cinit__") < 0)) __PYX_ERR(10, 73, __pyx_L3_error)
      }
    } else {
      switch (PyTuple_GET_SIZE(__pyx_args)) {
        case  1: values[0] = PyTuple_GET_ITEM(__pyx_args, 0);
        CYTHON_FALLTHROUGH;
        case  0: break;
        default: goto __pyx_L5_argtuple_error;
      }
    }
    __pyx_v_shutdown_cq = values[0];
  }
  goto __pyx_L4_argument_unpacking_done;
  __pyx_L5_argtuple_error:;
  __Pyx_RaiseArgtupleInvalid("__cinit__", 0, 0, 1, PyTuple_GET_SIZE(__pyx_args)); __PYX_ERR(10, 73, __pyx_L3_error)
  __pyx_L3_error:;
  __Pyx_AddTraceback("grpc._cython.cygrpc.CompletionQueue.__cinit__", __pyx_clineno, __pyx_lineno, __pyx_filename);
  __Pyx_RefNannyFinishContext();
  return -1;
  __pyx_L4_argument_unpacking_done:;
  __pyx_r = __pyx_pf_4grpc_7_cython_6cygrpc_15CompletionQueue___cinit__(((struct __pyx_obj_4grpc_7_cython_6cygrpc_CompletionQueue *)__pyx_v_self), __pyx_v_shutdown_cq);

  /* function exit code */
  __Pyx_RefNannyFinishContext();
  return __pyx_r;
}

static int __pyx_pf_4grpc_7_cython_6cygrpc_15CompletionQueue___cinit__(struct __pyx_obj_4grpc_7_cython_6cygrpc_CompletionQueue *__pyx_v_self, PyObject *__pyx_v_shutdown_cq) {
  grpc_completion_queue_attributes __pyx_v_c_attrs;
  int __pyx_r;
  __Pyx_RefNannyDeclarations
  PyObject *__pyx_t_1 = NULL;
  PyObject *__pyx_t_2 = NULL;
  PyObject *__pyx_t_3 = NULL;
  int __pyx_t_4;
  __Pyx_RefNannySetupContext("__cinit__", 0);

  /* "src/python/grpcio/grpc/_cython/_cygrpc/completion_queue.pyx.pxi":75
 *   def __cinit__(self, shutdown_cq=False):
 *     cdef grpc_completion_queue_attributes c_attrs
 *     fork_handlers_and_grpc_init()             # <<<<<<<<<<<<<<
 *     if shutdown_cq:
 *       c_attrs.version = 1
 */
  __pyx_t_2 = __Pyx_GetModuleGlobalName(__pyx_n_s_fork_handlers_and_grpc_init); if (unlikely(!__pyx_t_2)) __PYX_ERR(10, 75, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_2);
  __pyx_t_3 = NULL;
  if (CYTHON_UNPACK_METHODS && unlikely(PyMethod_Check(__pyx_t_2))) {
    __pyx_t_3 = PyMethod_GET_SELF(__pyx_t_2);
    if (likely(__pyx_t_3)) {
      PyObject* function = PyMethod_GET_FUNCTION(__pyx_t_2);
      __Pyx_INCREF(__pyx_t_3);
      __Pyx_INCREF(function);
      __Pyx_DECREF_SET(__pyx_t_2, function);
    }
  }
  if (__pyx_t_3) {
    __pyx_t_1 = __Pyx_PyObject_CallOneArg(__pyx_t_2, __pyx_t_3); if (unlikely(!__pyx_t_1)) __PYX_ERR(10, 75, __pyx_L1_error)
    __Pyx_DECREF(__pyx_t_3); __pyx_t_3 = 0;
  } else {
    __pyx_t_1 = __Pyx_PyObject_CallNoArg(__pyx_t_2); if (unlikely(!__pyx_t_1)) __PYX_ERR(10, 75, __pyx_L1_error)
  }
  __Pyx_GOTREF(__pyx_t_1);
  __Pyx_DECREF(__pyx_t_2); __pyx_t_2 = 0;
  __Pyx_DECREF(__pyx_t_1); __pyx_t_1 = 0;

  /* "src/python/grpcio/grpc/_cython/_cygrpc/completion_queue.pyx.pxi":76
 *     cdef grpc_completion_queue_attributes c_attrs
 *     fork_handlers_and_grpc_init()
 *     if shutdown_cq:             # <<<<<<<<<<<<<<
 *       c_attrs.version = 1
 *       c_attrs.cq_completion_type = GRPC_CQ_NEXT
 */
  __pyx_t_4 = __Pyx_PyObject_IsTrue(__pyx_v_shutdown_cq); if (unlikely(__pyx_t_4 < 0)) __PYX_ERR(10, 76, __pyx_L1_error)
  if (__pyx_t_4) {

    /* "src/python/grpcio/grpc/_cython/_cygrpc/completion_queue.pyx.pxi":77
 *     fork_handlers_and_grpc_init()
 *     if shutdown_cq:
 *       c_attrs.version = 1             # <<<<<<<<<<<<<<
 *       c_attrs.cq_completion_type = GRPC_CQ_NEXT
 *       c_attrs.cq_polling_type = GRPC_CQ_NON_LISTENING
 */
    __pyx_v_c_attrs.version = 1;

    /* "src/python/grpcio/grpc/_cython/_cygrpc/completion_queue.pyx.pxi":78
 *     if shutdown_cq:
 *       c_attrs.version = 1
 *       c_attrs.cq_completion_type = GRPC_CQ_NEXT             # <<<<<<<<<<<<<<
 *       c_attrs.cq_polling_type = GRPC_CQ_NON_LISTENING
 *       self.c_completion_queue = grpc_completion_queue_create(
 */
    __pyx_v_c_attrs.cq_completion_type = GRPC_CQ_NEXT;

    /* "src/python/grpcio/grpc/_cython/_cygrpc/completion_queue.pyx.pxi":79
 *       c_attrs.version = 1
 *       c_attrs.cq_completion_type = GRPC_CQ_NEXT
 *       c_attrs.cq_polling_type = GRPC_CQ_NON_LISTENING             # <<<<<<<<<<<<<<
 *       self.c_completion_queue = grpc_completion_queue_create(
 *           grpc_completion_queue_factory_lookup(&c_attrs), &c_attrs, NULL);
 */
    __pyx_v_c_attrs.cq_polling_type = GRPC_CQ_NON_LISTENING;

    /* "src/python/grpcio/grpc/_cython/_cygrpc/completion_queue.pyx.pxi":80
 *       c_attrs.cq_completion_type = GRPC_CQ_NEXT
 *       c_attrs.cq_polling_type = GRPC_CQ_NON_LISTENING
 *       self.c_completion_queue = grpc_completion_queue_create(             # <<<<<<<<<<<<<<
 *           grpc_completion_queue_factory_lookup(&c_attrs), &c_attrs, NULL);
 *     else:
 */
    __pyx_v_self->c_completion_queue = grpc_completion_queue_create(grpc_completion_queue_factory_lookup((&__pyx_v_c_attrs)), (&__pyx_v_c_attrs), NULL);

    /* "src/python/grpcio/grpc/_cython/_cygrpc/completion_queue.pyx.pxi":76
 *     cdef grpc_completion_queue_attributes c_attrs
 *     fork_handlers_and_grpc_init()
 *     if shutdown_cq:             # <<<<<<<<<<<<<<
 *       c_attrs.version = 1
 *       c_attrs.cq_completion_type = GRPC_CQ_NEXT
 */
    goto __pyx_L3;
  }

  /* "src/python/grpcio/grpc/_cython/_cygrpc/completion_queue.pyx.pxi":83
 *           grpc_completion_queue_factory_lookup(&c_attrs), &c_attrs, NULL);
 *     else:
 *       self.c_completion_queue = grpc_completion_queue_create_for_next(NULL)             # <<<<<<<<<<<<<<
 *     self.is_shutting_down = False
 *     self.is_shutdown = False
 */
  /*else*/ {
    __pyx_v_self->c_completion_queue = grpc_completion_queue_create_for_next(NULL);
  }
  __pyx_L3:;

  /* "src/python/grpcio/grpc/_cython/_cygrpc/completion_queue.pyx.pxi":84
 *     else:
 *       self.c_completion_queue = grpc_completion_queue_create_for_next(NULL)
 *     self.is_shutting_down = False             # <<<<<<<<<<<<<<
 *     self.is_shutdown = False
 * 
 */
  __pyx_v_self->is_shutting_down = 0;

  /* "src/python/grpcio/grpc/_cython/_cygrpc/completion_queue.pyx.pxi":85
 *       self.c_completion_queue = grpc_completion_queue_create_for_next(NULL)
 *     self.is_shutting_down = False
 *     self.is_shutdown = False             # <<<<<<<<<<<<<<
 * 
 *   cdef _interpret_event(self, grpc_event c_event):
 */
  __pyx_v_self->is_shutdown = 0;

  /* "src/python/grpcio/grpc/_cython/_cygrpc/completion_queue.pyx.pxi":73
 * cdef class CompletionQueue:
 * 
 *   def __cinit__(self, shutdown_cq=False):             # <<<<<<<<<<<<<<
 *     cdef grpc_completion_queue_attributes c_attrs
 *     fork_handlers_and_grpc_init()
 */

  /* function exit code */
  __pyx_r = 0;
  goto __pyx_L0;
  __pyx_L1_error:;
  __Pyx_XDECREF(__pyx_t_1);
  __Pyx_XDECREF(__pyx_t_2);
  __Pyx_XDECREF(__pyx_t_3);
  __Pyx_AddTraceback("grpc._cython.cygrpc.CompletionQueue.__cinit__", __pyx_clineno, __pyx_lineno, __pyx_filename);
  __pyx_r = -1;
  __pyx_L0:;
  __Pyx_RefNannyFinishContext();
  return __pyx_r;
}

/* "src/python/grpcio/grpc/_cython/_cygrpc/completion_queue.pyx.pxi":87
 *     self.is_shutdown = False
 * 
 *   cdef _interpret_event(self, grpc_event c_event):             # <<<<<<<<<<<<<<
 *     unused_tag, event = _interpret_event(c_event)
 *     if event.completion_type == GRPC_QUEUE_SHUTDOWN:
 */

static PyObject *__pyx_f_4grpc_7_cython_6cygrpc_15CompletionQueue__interpret_event(struct __pyx_obj_4grpc_7_cython_6cygrpc_CompletionQueue *__pyx_v_self, grpc_event __pyx_v_c_event) {
  CYTHON_UNUSED PyObject *__pyx_v_unused_tag = NULL;
  PyObject *__pyx_v_event = NULL;
  PyObject *__pyx_r = NULL;
  __Pyx_RefNannyDeclarations
  PyObject *__pyx_t_1 = NULL;
  PyObject *__pyx_t_2 = NULL;
  PyObject *__pyx_t_3 = NULL;
  PyObject *__pyx_t_4 = NULL;
  PyObject *(*__pyx_t_5)(PyObject *);
  int __pyx_t_6;
  __Pyx_RefNannySetupContext("_interpret_event", 0);

  /* "src/python/grpcio/grpc/_cython/_cygrpc/completion_queue.pyx.pxi":88
 * 
 *   cdef _interpret_event(self, grpc_event c_event):
 *     unused_tag, event = _interpret_event(c_event)             # <<<<<<<<<<<<<<
 *     if event.completion_type == GRPC_QUEUE_SHUTDOWN:
 *       self.is_shutdown = True
 */
  __pyx_t_1 = __pyx_f_4grpc_7_cython_6cygrpc__interpret_event(__pyx_v_c_event); if (unlikely(!__pyx_t_1)) __PYX_ERR(10, 88, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_1);
  if ((likely(PyTuple_CheckExact(__pyx_t_1))) || (PyList_CheckExact(__pyx_t_1))) {
    PyObject* sequence = __pyx_t_1;
    Py_ssize_t size = __Pyx_PySequence_SIZE(sequence);
    if (unlikely(size != 2)) {
      if (size > 2) __Pyx_RaiseTooManyValuesError(2);
      else if (size >= 0) __Pyx_RaiseNeedMoreValuesError(size);
      __PYX_ERR(10, 88, __pyx_L1_error)
    }
    #if CYTHON_ASSUME_SAFE_MACROS && !CYTHON_AVOID_BORROWED_REFS
    if (likely(PyTuple_CheckExact(sequence))) {
      __pyx_t_2 = PyTuple_GET_ITEM(sequence, 0); 
      __pyx_t_3 = PyTuple_GET_ITEM(sequence, 1); 
    } else {
      __pyx_t_2 = PyList_GET_ITEM(sequence, 0); 
      __pyx_t_3 = PyList_GET_ITEM(sequence, 1); 
    }
    __Pyx_INCREF(__pyx_t_2);
    __Pyx_INCREF(__pyx_t_3);
    #else
    __pyx_t_2 = PySequence_ITEM(sequence, 0); if (unlikely(!__pyx_t_2)) __PYX_ERR(10, 88, __pyx_L1_error)
    __Pyx_GOTREF(__pyx_t_2);
    __pyx_t_3 = PySequence_ITEM(sequence, 1); if (unlikely(!__pyx_t_3)) __PYX_ERR(10, 88, __pyx_L1_error)
    __Pyx_GOTREF(__pyx_t_3);
    #endif
    __Pyx_DECREF(__pyx_t_1); __pyx_t_1 = 0;
  } else {
    Py_ssize_t index = -1;
    __pyx_t_4 = PyObject_GetIter(__pyx_t_1); if (unlikely(!__pyx_t_4)) __PYX_ERR(10, 88, __pyx_L1_error)
    __Pyx_GOTREF(__pyx_t_4);
    __Pyx_DECREF(__pyx_t_1); __pyx_t_1 = 0;
    __pyx_t_5 = Py_TYPE(__pyx_t_4)->tp_iternext;
    index = 0; __pyx_t_2 = __pyx_t_5(__pyx_t_4); if (unlikely(!__pyx_t_2)) goto __pyx_L3_unpacking_failed;
    __Pyx_GOTREF(__pyx_t_2);
    index = 1; __pyx_t_3 = __pyx_t_5(__pyx_t_4); if (unlikely(!__pyx_t_3)) goto __pyx_L3_unpacking_failed;
    __Pyx_GOTREF(__pyx_t_3);
    if (__Pyx_IternextUnpackEndCheck(__pyx_t_5(__pyx_t_4), 2) < 0) __PYX_ERR(10, 88, __pyx_L1_error)
    __pyx_t_5 = NULL;
    __Pyx_DECREF(__pyx_t_4); __pyx_t_4 = 0;
    goto __pyx_L4_unpacking_done;
    __pyx_L3_unpacking_failed:;
    __Pyx_DECREF(__pyx_t_4); __pyx_t_4 = 0;
    __pyx_t_5 = NULL;
    if (__Pyx_IterFinish() == 0) __Pyx_RaiseNeedMoreValuesError(index);
    __PYX_ERR(10, 88, __pyx_L1_error)
    __pyx_L4_unpacking_done:;
  }
  __pyx_v_unused_tag = __pyx_t_2;
  __pyx_t_2 = 0;
  __pyx_v_event = __pyx_t_3;
  __pyx_t_3 = 0;

  /* "src/python/grpcio/grpc/_cython/_cygrpc/completion_queue.pyx.pxi":89
 *   cdef _interpret_event(self, grpc_event c_event):
 *     unused_tag, event = _interpret_event(c_event)
 *     if event.completion_type == GRPC_QUEUE_SHUTDOWN:             # <<<<<<<<<<<<<<
 *       self.is_shutdown = True
 *     return event
 */
  __pyx_t_1 = __Pyx_PyObject_GetAttrStr(__pyx_v_event, __pyx_n_s_completion_type); if (unlikely(!__pyx_t_1)) __PYX_ERR(10, 89, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_1);
  __pyx_t_3 = __Pyx_PyInt_From_grpc_completion_type(GRPC_QUEUE_SHUTDOWN); if (unlikely(!__pyx_t_3)) __PYX_ERR(10, 89, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_3);
  __pyx_t_2 = PyObject_RichCompare(__pyx_t_1, __pyx_t_3, Py_EQ); __Pyx_XGOTREF(__pyx_t_2); if (unlikely(!__pyx_t_2)) __PYX_ERR(10, 89, __pyx_L1_error)
  __Pyx_DECREF(__pyx_t_1); __pyx_t_1 = 0;
  __Pyx_DECREF(__pyx_t_3); __pyx_t_3 = 0;
  __pyx_t_6 = __Pyx_PyObject_IsTrue(__pyx_t_2); if (unlikely(__pyx_t_6 < 0)) __PYX_ERR(10, 89, __pyx_L1_error)
  __Pyx_DECREF(__pyx_t_2); __pyx_t_2 = 0;
  if (__pyx_t_6) {

    /* "src/python/grpcio/grpc/_cython/_cygrpc/completion_queue.pyx.pxi":90
 *     unused_tag, event = _interpret_event(c_event)
 *     if event.completion_type == GRPC_QUEUE_SHUTDOWN:
 *       self.is_shutdown = True             # <<<<<<<<<<<<<<
 *     return event
 * 
 */
    __pyx_v_self->is_shutdown = 1;

    /* "src/python/grpcio/grpc/_cython/_cygrpc/completion_queue.pyx.pxi":89
 *   cdef _interpret_event(self, grpc_event c_event):
 *     unused_tag, event = _interpret_event(c_event)
 *     if event.completion_type == GRPC_QUEUE_SHUTDOWN:             # <<<<<<<<<<<<<<
 *       self.is_shutdown = True
 *     return event
 */
  }

  /* "src/python/grpcio/grpc/_cython/_cygrpc/completion_queue.pyx.pxi":91
 *     if event.completion_type == GRPC_QUEUE_SHUTDOWN:
 *       self.is_shutdown = True
 *     return event             # <<<<<<<<<<<<<<
 * 
 *   # We name this 'poll' to avoid problems with CPython's expectations for
 */
  __Pyx_XDECREF(__pyx_r);
  __Pyx_INCREF(__pyx_v_event);
  __pyx_r = __pyx_v_event;
  goto __pyx_L0;

  /* "src/python/grpcio/grpc/_cython/_cygrpc/completion_queue.pyx.pxi":87
 *     self.is_shutdown = False
 * 
 *   cdef _interpret_event(self, grpc_event c_event):             # <<<<<<<<<<<<<<
 *     unused_tag, event = _interpret_event(c_event)
 *     if event.completion_type == GRPC_QUEUE_SHUTDOWN:
 */

  /* function exit code */
  __pyx_L1_error:;
  __Pyx_XDECREF(__pyx_t_1);
  __Pyx_XDECREF(__pyx_t_2);
  __Pyx_XDECREF(__pyx_t_3);
  __Pyx_XDECREF(__pyx_t_4);
  __Pyx_AddTraceback("grpc._cython.cygrpc.CompletionQueue._interpret_event", __pyx_clineno, __pyx_lineno, __pyx_filename);
  __pyx_r = 0;
  __pyx_L0:;
  __Pyx_XDECREF(__pyx_v_unused_tag);
  __Pyx_XDECREF(__pyx_v_event);
  __Pyx_XGIVEREF(__pyx_r);
  __Pyx_RefNannyFinishContext();
  return __pyx_r;
}

/* "src/python/grpcio/grpc/_cython/_cygrpc/completion_queue.pyx.pxi":95
 *   # We name this 'poll' to avoid problems with CPython's expectations for
 *   # 'special' methods (like next and __next__).
 *   def poll(self, deadline=None):             # <<<<<<<<<<<<<<
 *     return self._interpret_event(_next(self.c_completion_queue, deadline))
 * 
 */

/* Python wrapper */
static PyObject *__pyx_pw_4grpc_7_cython_6cygrpc_15CompletionQueue_3poll(PyObject *__pyx_v_self, PyObject *__pyx_args, PyObject *__pyx_kwds); /*proto*/
static PyObject *__pyx_pw_4grpc_7_cython_6cygrpc_15CompletionQueue_3poll(PyObject *__pyx_v_self, PyObject *__pyx_args, PyObject *__pyx_kwds) {
  PyObject *__pyx_v_deadline = 0;
  PyObject *__pyx_r = 0;
  __Pyx_RefNannyDeclarations
  __Pyx_RefNannySetupContext("poll (wrapper)", 0);
  {
    static PyObject **__pyx_pyargnames[] = {&__pyx_n_s_deadline,0};
    PyObject* values[1] = {0};
    values[0] = ((PyObject *)Py_None);
    if (unlikely(__pyx_kwds)) {
      Py_ssize_t kw_args;
      const Py_ssize_t pos_args = PyTuple_GET_SIZE(__pyx_args);
      switch (pos_args) {
        case  1: values[0] = PyTuple_GET_ITEM(__pyx_args, 0);
        CYTHON_FALLTHROUGH;
        case  0: break;
        default: goto __pyx_L5_argtuple_error;
      }
      kw_args = PyDict_Size(__pyx_kwds);
      switch (pos_args) {
        case  0:
        if (kw_args > 0) {
          PyObject* value = __Pyx_PyDict_GetItemStr(__pyx_kwds, __pyx_n_s_deadline);
          if (value) { values[0] = value; kw_args--; }
        }
      }
      if (unlikely(kw_args > 0)) {
        if (unlikely(__Pyx_ParseOptionalKeywords(__pyx_kwds, __pyx_pyargnames, 0, values, pos_args, "poll") < 0)) __PYX_ERR(10, 95, __pyx_L3_error)
      }
    } else {
      switch (PyTuple_GET_SIZE(__pyx_args)) {
        case  1: values[0] = PyTuple_GET_ITEM(__pyx_args, 0);
        CYTHON_FALLTHROUGH;
        case  0: break;
        default: goto __pyx_L5_argtuple_error;
      }
    }
    __pyx_v_deadline = values[0];
  }
  goto __pyx_L4_argument_unpacking_done;
  __pyx_L5_argtuple_error:;
  __Pyx_RaiseArgtupleInvalid("poll", 0, 0, 1, PyTuple_GET_SIZE(__pyx_args)); __PYX_ERR(10, 95, __pyx_L3_error)
  __pyx_L3_error:;
  __Pyx_AddTraceback("grpc._cython.cygrpc.CompletionQueue.poll", __pyx_clineno, __pyx_lineno, __pyx_filename);
  __Pyx_RefNannyFinishContext();
  return NULL;
  __pyx_L4_argument_unpacking_done:;
  __pyx_r = __pyx_pf_4grpc_7_cython_6cygrpc_15CompletionQueue_2poll(((struct __pyx_obj_4grpc_7_cython_6cygrpc_CompletionQueue *)__pyx_v_self), __pyx_v_deadline);

  /* function exit code */
  __Pyx_RefNannyFinishContext();
  return __pyx_r;
}

static PyObject *__pyx_pf_4grpc_7_cython_6cygrpc_15CompletionQueue_2poll(struct __pyx_obj_4grpc_7_cython_6cygrpc_CompletionQueue *__pyx_v_self, PyObject *__pyx_v_deadline) {
  PyObject *__pyx_r = NULL;
  __Pyx_RefNannyDeclarations
  grpc_event __pyx_t_1;
  PyObject *__pyx_t_2 = NULL;
  __Pyx_RefNannySetupContext("poll", 0);

  /* "src/python/grpcio/grpc/_cython/_cygrpc/completion_queue.pyx.pxi":96
 *   # 'special' methods (like next and __next__).
 *   def poll(self, deadline=None):
 *     return self._interpret_event(_next(self.c_completion_queue, deadline))             # <<<<<<<<<<<<<<
 * 
 *   def shutdown(self):
 */
  __Pyx_XDECREF(__pyx_r);
  __pyx_t_1 = __pyx_f_4grpc_7_cython_6cygrpc__next(__pyx_v_self->c_completion_queue, __pyx_v_deadline); if (unlikely(PyErr_Occurred())) __PYX_ERR(10, 96, __pyx_L1_error)
  __pyx_t_2 = ((struct __pyx_vtabstruct_4grpc_7_cython_6cygrpc_CompletionQueue *)__pyx_v_self->__pyx_vtab)->_interpret_event(__pyx_v_self, __pyx_t_1); if (unlikely(!__pyx_t_2)) __PYX_ERR(10, 96, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_2);
  __pyx_r = __pyx_t_2;
  __pyx_t_2 = 0;
  goto __pyx_L0;

  /* "src/python/grpcio/grpc/_cython/_cygrpc/completion_queue.pyx.pxi":95
 *   # We name this 'poll' to avoid problems with CPython's expectations for
 *   # 'special' methods (like next and __next__).
 *   def poll(self, deadline=None):             # <<<<<<<<<<<<<<
 *     return self._interpret_event(_next(self.c_completion_queue, deadline))
 * 
 */

  /* function exit code */
  __pyx_L1_error:;
  __Pyx_XDECREF(__pyx_t_2);
  __Pyx_AddTraceback("grpc._cython.cygrpc.CompletionQueue.poll", __pyx_clineno, __pyx_lineno, __pyx_filename);
  __pyx_r = NULL;
  __pyx_L0:;
  __Pyx_XGIVEREF(__pyx_r);
  __Pyx_RefNannyFinishContext();
  return __pyx_r;
}

/* "src/python/grpcio/grpc/_cython/_cygrpc/completion_queue.pyx.pxi":98
 *     return self._interpret_event(_next(self.c_completion_queue, deadline))
 * 
 *   def shutdown(self):             # <<<<<<<<<<<<<<
 *     with nogil:
 *       grpc_completion_queue_shutdown(self.c_completion_queue)
 */

/* Python wrapper */
static PyObject *__pyx_pw_4grpc_7_cython_6cygrpc_15CompletionQueue_5shutdown(PyObject *__pyx_v_self, CYTHON_UNUSED PyObject *unused); /*proto*/
static PyObject *__pyx_pw_4grpc_7_cython_6cygrpc_15CompletionQueue_5shutdown(PyObject *__pyx_v_self, CYTHON_UNUSED PyObject *unused) {
  PyObject *__pyx_r = 0;
  __Pyx_RefNannyDeclarations
  __Pyx_RefNannySetupContext("shutdown (wrapper)", 0);
  __pyx_r = __pyx_pf_4grpc_7_cython_6cygrpc_15CompletionQueue_4shutdown(((struct __pyx_obj_4grpc_7_cython_6cygrpc_CompletionQueue *)__pyx_v_self));

  /* function exit code */
  __Pyx_RefNannyFinishContext();
  return __pyx_r;
}

static PyObject *__pyx_pf_4grpc_7_cython_6cygrpc_15CompletionQueue_4shutdown(struct __pyx_obj_4grpc_7_cython_6cygrpc_CompletionQueue *__pyx_v_self) {
  PyObject *__pyx_r = NULL;
  __Pyx_RefNannyDeclarations
  __Pyx_RefNannySetupContext("shutdown", 0);

  /* "src/python/grpcio/grpc/_cython/_cygrpc/completion_queue.pyx.pxi":99
 * 
 *   def shutdown(self):
 *     with nogil:             # <<<<<<<<<<<<<<
 *       grpc_completion_queue_shutdown(self.c_completion_queue)
 *     self.is_shutting_down = True
 */
  {
      #ifdef WITH_THREAD
      PyThreadState *_save;
      Py_UNBLOCK_THREADS
      __Pyx_FastGIL_Remember();
      #endif
      /*try:*/ {

        /* "src/python/grpcio/grpc/_cython/_cygrpc/completion_queue.pyx.pxi":100
 *   def shutdown(self):
 *     with nogil:
 *       grpc_completion_queue_shutdown(self.c_completion_queue)             # <<<<<<<<<<<<<<
 *     self.is_shutting_down = True
 * 
 */
        grpc_completion_queue_shutdown(__pyx_v_self->c_completion_queue);
      }

      /* "src/python/grpcio/grpc/_cython/_cygrpc/completion_queue.pyx.pxi":99
 * 
 *   def shutdown(self):
 *     with nogil:             # <<<<<<<<<<<<<<
 *       grpc_completion_queue_shutdown(self.c_completion_queue)
 *     self.is_shutting_down = True
 */
      /*finally:*/ {
        /*normal exit:*/{
          #ifdef WITH_THREAD
          __Pyx_FastGIL_Forget();
          Py_BLOCK_THREADS
          #endif
          goto __pyx_L5;
        }
        __pyx_L5:;
      }
  }

  /* "src/python/grpcio/grpc/_cython/_cygrpc/completion_queue.pyx.pxi":101
 *     with nogil:
 *       grpc_completion_queue_shutdown(self.c_completion_queue)
 *     self.is_shutting_down = True             # <<<<<<<<<<<<<<
 * 
 *   def clear(self):
 */
  __pyx_v_self->is_shutting_down = 1;

  /* "src/python/grpcio/grpc/_cython/_cygrpc/completion_queue.pyx.pxi":98
 *     return self._interpret_event(_next(self.c_completion_queue, deadline))
 * 
 *   def shutdown(self):             # <<<<<<<<<<<<<<
 *     with nogil:
 *       grpc_completion_queue_shutdown(self.c_completion_queue)
 */

  /* function exit code */
  __pyx_r = Py_None; __Pyx_INCREF(Py_None);
  __Pyx_XGIVEREF(__pyx_r);
  __Pyx_RefNannyFinishContext();
  return __pyx_r;
}

/* "src/python/grpcio/grpc/_cython/_cygrpc/completion_queue.pyx.pxi":103
 *     self.is_shutting_down = True
 * 
 *   def clear(self):             # <<<<<<<<<<<<<<
 *     if not self.is_shutting_down:
 *       raise ValueError('queue must be shutting down to be cleared')
 */

/* Python wrapper */
static PyObject *__pyx_pw_4grpc_7_cython_6cygrpc_15CompletionQueue_7clear(PyObject *__pyx_v_self, CYTHON_UNUSED PyObject *unused); /*proto*/
static PyObject *__pyx_pw_4grpc_7_cython_6cygrpc_15CompletionQueue_7clear(PyObject *__pyx_v_self, CYTHON_UNUSED PyObject *unused) {
  PyObject *__pyx_r = 0;
  __Pyx_RefNannyDeclarations
  __Pyx_RefNannySetupContext("clear (wrapper)", 0);
  __pyx_r = __pyx_pf_4grpc_7_cython_6cygrpc_15CompletionQueue_6clear(((struct __pyx_obj_4grpc_7_cython_6cygrpc_CompletionQueue *)__pyx_v_self));

  /* function exit code */
  __Pyx_RefNannyFinishContext();
  return __pyx_r;
}

static PyObject *__pyx_pf_4grpc_7_cython_6cygrpc_15CompletionQueue_6clear(struct __pyx_obj_4grpc_7_cython_6cygrpc_CompletionQueue *__pyx_v_self) {
  PyObject *__pyx_r = NULL;
  __Pyx_RefNannyDeclarations
  int __pyx_t_1;
  PyObject *__pyx_t_2 = NULL;
  PyObject *__pyx_t_3 = NULL;
  PyObject *__pyx_t_4 = NULL;
  __Pyx_RefNannySetupContext("clear", 0);

  /* "src/python/grpcio/grpc/_cython/_cygrpc/completion_queue.pyx.pxi":104
 * 
 *   def clear(self):
 *     if not self.is_shutting_down:             # <<<<<<<<<<<<<<
 *       raise ValueError('queue must be shutting down to be cleared')
 *     while self.poll().type != GRPC_QUEUE_SHUTDOWN:
 */
  __pyx_t_1 = ((!(__pyx_v_self->is_shutting_down != 0)) != 0);
  if (unlikely(__pyx_t_1)) {

    /* "src/python/grpcio/grpc/_cython/_cygrpc/completion_queue.pyx.pxi":105
 *   def clear(self):
 *     if not self.is_shutting_down:
 *       raise ValueError('queue must be shutting down to be cleared')             # <<<<<<<<<<<<<<
 *     while self.poll().type != GRPC_QUEUE_SHUTDOWN:
 *       pass
 */
    __pyx_t_2 = __Pyx_PyObject_Call(__pyx_builtin_ValueError, __pyx_tuple__73, NULL); if (unlikely(!__pyx_t_2)) __PYX_ERR(10, 105, __pyx_L1_error)
    __Pyx_GOTREF(__pyx_t_2);
    __Pyx_Raise(__pyx_t_2, 0, 0, 0);
    __Pyx_DECREF(__pyx_t_2); __pyx_t_2 = 0;
    __PYX_ERR(10, 105, __pyx_L1_error)

    /* "src/python/grpcio/grpc/_cython/_cygrpc/completion_queue.pyx.pxi":104
 * 
 *   def clear(self):
 *     if not self.is_shutting_down:             # <<<<<<<<<<<<<<
 *       raise ValueError('queue must be shutting down to be cleared')
 *     while self.poll().type != GRPC_QUEUE_SHUTDOWN:
 */
  }

  /* "src/python/grpcio/grpc/_cython/_cygrpc/completion_queue.pyx.pxi":106
 *     if not self.is_shutting_down:
 *       raise ValueError('queue must be shutting down to be cleared')
 *     while self.poll().type != GRPC_QUEUE_SHUTDOWN:             # <<<<<<<<<<<<<<
 *       pass
 * 
 */
  while (1) {
    __pyx_t_3 = __Pyx_PyObject_GetAttrStr(((PyObject *)__pyx_v_self), __pyx_n_s_poll); if (unlikely(!__pyx_t_3)) __PYX_ERR(10, 106, __pyx_L1_error)
    __Pyx_GOTREF(__pyx_t_3);
    __pyx_t_4 = NULL;
    if (CYTHON_UNPACK_METHODS && likely(PyMethod_Check(__pyx_t_3))) {
      __pyx_t_4 = PyMethod_GET_SELF(__pyx_t_3);
      if (likely(__pyx_t_4)) {
        PyObject* function = PyMethod_GET_FUNCTION(__pyx_t_3);
        __Pyx_INCREF(__pyx_t_4);
        __Pyx_INCREF(function);
        __Pyx_DECREF_SET(__pyx_t_3, function);
      }
    }
    if (__pyx_t_4) {
      __pyx_t_2 = __Pyx_PyObject_CallOneArg(__pyx_t_3, __pyx_t_4); if (unlikely(!__pyx_t_2)) __PYX_ERR(10, 106, __pyx_L1_error)
      __Pyx_DECREF(__pyx_t_4); __pyx_t_4 = 0;
    } else {
      __pyx_t_2 = __Pyx_PyObject_CallNoArg(__pyx_t_3); if (unlikely(!__pyx_t_2)) __PYX_ERR(10, 106, __pyx_L1_error)
    }
    __Pyx_GOTREF(__pyx_t_2);
    __Pyx_DECREF(__pyx_t_3); __pyx_t_3 = 0;
    __pyx_t_3 = __Pyx_PyObject_GetAttrStr(__pyx_t_2, __pyx_n_s_type); if (unlikely(!__pyx_t_3)) __PYX_ERR(10, 106, __pyx_L1_error)
    __Pyx_GOTREF(__pyx_t_3);
    __Pyx_DECREF(__pyx_t_2); __pyx_t_2 = 0;
    __pyx_t_2 = __Pyx_PyInt_From_grpc_completion_type(GRPC_QUEUE_SHUTDOWN); if (unlikely(!__pyx_t_2)) __PYX_ERR(10, 106, __pyx_L1_error)
    __Pyx_GOTREF(__pyx_t_2);
    __pyx_t_4 = PyObject_RichCompare(__pyx_t_3, __pyx_t_2, Py_NE); __Pyx_XGOTREF(__pyx_t_4); if (unlikely(!__pyx_t_4)) __PYX_ERR(10, 106, __pyx_L1_error)
    __Pyx_DECREF(__pyx_t_3); __pyx_t_3 = 0;
    __Pyx_DECREF(__pyx_t_2); __pyx_t_2 = 0;
    __pyx_t_1 = __Pyx_PyObject_IsTrue(__pyx_t_4); if (unlikely(__pyx_t_1 < 0)) __PYX_ERR(10, 106, __pyx_L1_error)
    __Pyx_DECREF(__pyx_t_4); __pyx_t_4 = 0;
    if (!__pyx_t_1) break;
  }

  /* "src/python/grpcio/grpc/_cython/_cygrpc/completion_queue.pyx.pxi":103
 *     self.is_shutting_down = True
 * 
 *   def clear(self):             # <<<<<<<<<<<<<<
 *     if not self.is_shutting_down:
 *       raise ValueError('queue must be shutting down to be cleared')
 */

  /* function exit code */
  __pyx_r = Py_None; __Pyx_INCREF(Py_None);
  goto __pyx_L0;
  __pyx_L1_error:;
  __Pyx_XDECREF(__pyx_t_2);
  __Pyx_XDECREF(__pyx_t_3);
  __Pyx_XDECREF(__pyx_t_4);
  __Pyx_AddTraceback("grpc._cython.cygrpc.CompletionQueue.clear", __pyx_clineno, __pyx_lineno, __pyx_filename);
  __pyx_r = NULL;
  __pyx_L0:;
  __Pyx_XGIVEREF(__pyx_r);
  __Pyx_RefNannyFinishContext();
  return __pyx_r;
}

/* "src/python/grpcio/grpc/_cython/_cygrpc/completion_queue.pyx.pxi":109
 *       pass
 * 
 *   def __dealloc__(self):             # <<<<<<<<<<<<<<
 *     cdef gpr_timespec c_deadline
 *     c_deadline = gpr_inf_future(GPR_CLOCK_REALTIME)
 */

/* Python wrapper */
static void __pyx_pw_4grpc_7_cython_6cygrpc_15CompletionQueue_9__dealloc__(PyObject *__pyx_v_self); /*proto*/
static void __pyx_pw_4grpc_7_cython_6cygrpc_15CompletionQueue_9__dealloc__(PyObject *__pyx_v_self) {
  __Pyx_RefNannyDeclarations
  __Pyx_RefNannySetupContext("__dealloc__ (wrapper)", 0);
  __pyx_pf_4grpc_7_cython_6cygrpc_15CompletionQueue_8__dealloc__(((struct __pyx_obj_4grpc_7_cython_6cygrpc_CompletionQueue *)__pyx_v_self));

  /* function exit code */
  __Pyx_RefNannyFinishContext();
}

static void __pyx_pf_4grpc_7_cython_6cygrpc_15CompletionQueue_8__dealloc__(struct __pyx_obj_4grpc_7_cython_6cygrpc_CompletionQueue *__pyx_v_self) {
  gpr_timespec __pyx_v_c_deadline;
  grpc_event __pyx_v_event;
  __Pyx_RefNannyDeclarations
  int __pyx_t_1;
  PyObject *__pyx_t_2 = NULL;
  __Pyx_RefNannySetupContext("__dealloc__", 0);

  /* "src/python/grpcio/grpc/_cython/_cygrpc/completion_queue.pyx.pxi":111
 *   def __dealloc__(self):
 *     cdef gpr_timespec c_deadline
 *     c_deadline = gpr_inf_future(GPR_CLOCK_REALTIME)             # <<<<<<<<<<<<<<
 *     if self.c_completion_queue != NULL:
 *       # Ensure shutdown
 */
  __pyx_v_c_deadline = gpr_inf_future(GPR_CLOCK_REALTIME);

  /* "src/python/grpcio/grpc/_cython/_cygrpc/completion_queue.pyx.pxi":112
 *     cdef gpr_timespec c_deadline
 *     c_deadline = gpr_inf_future(GPR_CLOCK_REALTIME)
 *     if self.c_completion_queue != NULL:             # <<<<<<<<<<<<<<
 *       # Ensure shutdown
 *       if not self.is_shutting_down:
 */
  __pyx_t_1 = ((__pyx_v_self->c_completion_queue != NULL) != 0);
  if (__pyx_t_1) {

    /* "src/python/grpcio/grpc/_cython/_cygrpc/completion_queue.pyx.pxi":114
 *     if self.c_completion_queue != NULL:
 *       # Ensure shutdown
 *       if not self.is_shutting_down:             # <<<<<<<<<<<<<<
 *         grpc_completion_queue_shutdown(self.c_completion_queue)
 *       # Pump the queue (All outstanding calls should have been cancelled)
 */
    __pyx_t_1 = ((!(__pyx_v_self->is_shutting_down != 0)) != 0);
    if (__pyx_t_1) {

      /* "src/python/grpcio/grpc/_cython/_cygrpc/completion_queue.pyx.pxi":115
 *       # Ensure shutdown
 *       if not self.is_shutting_down:
 *         grpc_completion_queue_shutdown(self.c_completion_queue)             # <<<<<<<<<<<<<<
 *       # Pump the queue (All outstanding calls should have been cancelled)
 *       while not self.is_shutdown:
 */
      grpc_completion_queue_shutdown(__pyx_v_self->c_completion_queue);

      /* "src/python/grpcio/grpc/_cython/_cygrpc/completion_queue.pyx.pxi":114
 *     if self.c_completion_queue != NULL:
 *       # Ensure shutdown
 *       if not self.is_shutting_down:             # <<<<<<<<<<<<<<
 *         grpc_completion_queue_shutdown(self.c_completion_queue)
 *       # Pump the queue (All outstanding calls should have been cancelled)
 */
    }

    /* "src/python/grpcio/grpc/_cython/_cygrpc/completion_queue.pyx.pxi":117
 *         grpc_completion_queue_shutdown(self.c_completion_queue)
 *       # Pump the queue (All outstanding calls should have been cancelled)
 *       while not self.is_shutdown:             # <<<<<<<<<<<<<<
 *         event = grpc_completion_queue_next(
 *             self.c_completion_queue, c_deadline, NULL)
 */
    while (1) {
      __pyx_t_1 = ((!(__pyx_v_self->is_shutdown != 0)) != 0);
      if (!__pyx_t_1) break;

      /* "src/python/grpcio/grpc/_cython/_cygrpc/completion_queue.pyx.pxi":118
 *       # Pump the queue (All outstanding calls should have been cancelled)
 *       while not self.is_shutdown:
 *         event = grpc_completion_queue_next(             # <<<<<<<<<<<<<<
 *             self.c_completion_queue, c_deadline, NULL)
 *         self._interpret_event(event)
 */
      __pyx_v_event = grpc_completion_queue_next(__pyx_v_self->c_completion_queue, __pyx_v_c_deadline, NULL);

      /* "src/python/grpcio/grpc/_cython/_cygrpc/completion_queue.pyx.pxi":120
 *         event = grpc_completion_queue_next(
 *             self.c_completion_queue, c_deadline, NULL)
 *         self._interpret_event(event)             # <<<<<<<<<<<<<<
 *       grpc_completion_queue_destroy(self.c_completion_queue)
 *     grpc_shutdown_blocking()
 */
      __pyx_t_2 = ((struct __pyx_vtabstruct_4grpc_7_cython_6cygrpc_CompletionQueue *)__pyx_v_self->__pyx_vtab)->_interpret_event(__pyx_v_self, __pyx_v_event); if (unlikely(!__pyx_t_2)) __PYX_ERR(10, 120, __pyx_L1_error)
      __Pyx_GOTREF(__pyx_t_2);
      __Pyx_DECREF(__pyx_t_2); __pyx_t_2 = 0;
    }

    /* "src/python/grpcio/grpc/_cython/_cygrpc/completion_queue.pyx.pxi":121
 *             self.c_completion_queue, c_deadline, NULL)
 *         self._interpret_event(event)
 *       grpc_completion_queue_destroy(self.c_completion_queue)             # <<<<<<<<<<<<<<
 *     grpc_shutdown_blocking()
 */
    grpc_completion_queue_destroy(__pyx_v_self->c_completion_queue);

    /* "src/python/grpcio/grpc/_cython/_cygrpc/completion_queue.pyx.pxi":112
 *     cdef gpr_timespec c_deadline
 *     c_deadline = gpr_inf_future(GPR_CLOCK_REALTIME)
 *     if self.c_completion_queue != NULL:             # <<<<<<<<<<<<<<
 *       # Ensure shutdown
 *       if not self.is_shutting_down:
 */
  }

  /* "src/python/grpcio/grpc/_cython/_cygrpc/completion_queue.pyx.pxi":122
 *         self._interpret_event(event)
 *       grpc_completion_queue_destroy(self.c_completion_queue)
 *     grpc_shutdown_blocking()             # <<<<<<<<<<<<<<
 */
  grpc_shutdown_blocking();

  /* "src/python/grpcio/grpc/_cython/_cygrpc/completion_queue.pyx.pxi":109
 *       pass
 * 
 *   def __dealloc__(self):             # <<<<<<<<<<<<<<
 *     cdef gpr_timespec c_deadline
 *     c_deadline = gpr_inf_future(GPR_CLOCK_REALTIME)
 */

  /* function exit code */
  goto __pyx_L0;
  __pyx_L1_error:;
  __Pyx_XDECREF(__pyx_t_2);
  __Pyx_WriteUnraisable("grpc._cython.cygrpc.CompletionQueue.__dealloc__", __pyx_clineno, __pyx_lineno, __pyx_filename, 1, 0);
  __pyx_L0:;
  __Pyx_RefNannyFinishContext();
}

/* "(tree fragment)":1
 * def __reduce_cython__(self):             # <<<<<<<<<<<<<<
 *     raise TypeError("no default __reduce__ due to non-trivial __cinit__")
 * def __setstate_cython__(self, __pyx_state):
 */

/* Python wrapper */
static PyObject *__pyx_pw_4grpc_7_cython_6cygrpc_15CompletionQueue_11__reduce_cython__(PyObject *__pyx_v_self, CYTHON_UNUSED PyObject *unused); /*proto*/
static PyObject *__pyx_pw_4grpc_7_cython_6cygrpc_15CompletionQueue_11__reduce_cython__(PyObject *__pyx_v_self, CYTHON_UNUSED PyObject *unused) {
  PyObject *__pyx_r = 0;
  __Pyx_RefNannyDeclarations
  __Pyx_RefNannySetupContext("__reduce_cython__ (wrapper)", 0);
  __pyx_r = __pyx_pf_4grpc_7_cython_6cygrpc_15CompletionQueue_10__reduce_cython__(((struct __pyx_obj_4grpc_7_cython_6cygrpc_CompletionQueue *)__pyx_v_self));

  /* function exit code */
  __Pyx_RefNannyFinishContext();
  return __pyx_r;
}

static PyObject *__pyx_pf_4grpc_7_cython_6cygrpc_15CompletionQueue_10__reduce_cython__(CYTHON_UNUSED struct __pyx_obj_4grpc_7_cython_6cygrpc_CompletionQueue *__pyx_v_self) {
  PyObject *__pyx_r = NULL;
  __Pyx_RefNannyDeclarations
  PyObject *__pyx_t_1 = NULL;
  __Pyx_RefNannySetupContext("__reduce_cython__", 0);

  /* "(tree fragment)":2
 * def __reduce_cython__(self):
 *     raise TypeError("no default __reduce__ due to non-trivial __cinit__")             # <<<<<<<<<<<<<<
 * def __setstate_cython__(self, __pyx_state):
 *     raise TypeError("no default __reduce__ due to non-trivial __cinit__")
 */
  __pyx_t_1 = __Pyx_PyObject_Call(__pyx_builtin_TypeError, __pyx_tuple__74, NULL); if (unlikely(!__pyx_t_1)) __PYX_ERR(7, 2, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_1);
  __Pyx_Raise(__pyx_t_1, 0, 0, 0);
  __Pyx_DECREF(__pyx_t_1); __pyx_t_1 = 0;
  __PYX_ERR(7, 2, __pyx_L1_error)

  /* "(tree fragment)":1
 * def __reduce_cython__(self):             # <<<<<<<<<<<<<<
 *     raise TypeError("no default __reduce__ due to non-trivial __cinit__")
 * def __setstate_cython__(self, __pyx_state):
 */

  /* function exit code */
  __pyx_L1_error:;
  __Pyx_XDECREF(__pyx_t_1);
  __Pyx_AddTraceback("grpc._cython.cygrpc.CompletionQueue.__reduce_cython__", __pyx_clineno, __pyx_lineno, __pyx_filename);
  __pyx_r = NULL;
  __Pyx_XGIVEREF(__pyx_r);
  __Pyx_RefNannyFinishContext();
  return __pyx_r;
}

/* "(tree fragment)":3
 * def __reduce_cython__(self):
 *     raise TypeError("no default __reduce__ due to non-trivial __cinit__")
 * def __setstate_cython__(self, __pyx_state):             # <<<<<<<<<<<<<<
 *     raise TypeError("no default __reduce__ due to non-trivial __cinit__")
 */

/* Python wrapper */
static PyObject *__pyx_pw_4grpc_7_cython_6cygrpc_15CompletionQueue_13__setstate_cython__(PyObject *__pyx_v_self, PyObject *__pyx_v___pyx_state); /*proto*/
static PyObject *__pyx_pw_4grpc_7_cython_6cygrpc_15CompletionQueue_13__setstate_cython__(PyObject *__pyx_v_self, PyObject *__pyx_v___pyx_state) {
  PyObject *__pyx_r = 0;
  __Pyx_RefNannyDeclarations
  __Pyx_RefNannySetupContext("__setstate_cython__ (wrapper)", 0);
  __pyx_r = __pyx_pf_4grpc_7_cython_6cygrpc_15CompletionQueue_12__setstate_cython__(((struct __pyx_obj_4grpc_7_cython_6cygrpc_CompletionQueue *)__pyx_v_self), ((PyObject *)__pyx_v___pyx_state));

  /* function exit code */
  __Pyx_RefNannyFinishContext();
  return __pyx_r;
}

static PyObject *__pyx_pf_4grpc_7_cython_6cygrpc_15CompletionQueue_12__setstate_cython__(CYTHON_UNUSED struct __pyx_obj_4grpc_7_cython_6cygrpc_CompletionQueue *__pyx_v_self, CYTHON_UNUSED PyObject *__pyx_v___pyx_state) {
  PyObject *__pyx_r = NULL;
  __Pyx_RefNannyDeclarations
  PyObject *__pyx_t_1 = NULL;
  __Pyx_RefNannySetupContext("__setstate_cython__", 0);

  /* "(tree fragment)":4
 *     raise TypeError("no default __reduce__ due to non-trivial __cinit__")
 * def __setstate_cython__(self, __pyx_state):
 *     raise TypeError("no default __reduce__ due to non-trivial __cinit__")             # <<<<<<<<<<<<<<
 */
  __pyx_t_1 = __Pyx_PyObject_Call(__pyx_builtin_TypeError, __pyx_tuple__75, NULL); if (unlikely(!__pyx_t_1)) __PYX_ERR(7, 4, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_1);
  __Pyx_Raise(__pyx_t_1, 0, 0, 0);
  __Pyx_DECREF(__pyx_t_1); __pyx_t_1 = 0;
  __PYX_ERR(7, 4, __pyx_L1_error)

  /* "(tree fragment)":3
 * def __reduce_cython__(self):
 *     raise TypeError("no default __reduce__ due to non-trivial __cinit__")
 * def __setstate_cython__(self, __pyx_state):             # <<<<<<<<<<<<<<
 *     raise TypeError("no default __reduce__ due to non-trivial __cinit__")
 */

  /* function exit code */
  __pyx_L1_error:;
  __Pyx_XDECREF(__pyx_t_1);
  __Pyx_AddTraceback("grpc._cython.cygrpc.CompletionQueue.__setstate_cython__", __pyx_clineno, __pyx_lineno, __pyx_filename);
  __pyx_r = NULL;
  __Pyx_XGIVEREF(__pyx_r);
  __Pyx_RefNannyFinishContext();
  return __pyx_r;
}

/* "src/python/grpcio/grpc/_cython/_cygrpc/event.pyx.pxi":18
 * cdef class ConnectivityEvent:
 * 
 *   def __cinit__(             # <<<<<<<<<<<<<<
 *       self, grpc_completion_type completion_type, bint success, object tag):
 *     self.completion_type = completion_type
 */

/* Python wrapper */
static int __pyx_pw_4grpc_7_cython_6cygrpc_17ConnectivityEvent_1__cinit__(PyObject *__pyx_v_self, PyObject *__pyx_args, PyObject *__pyx_kwds); /*proto*/
static int __pyx_pw_4grpc_7_cython_6cygrpc_17ConnectivityEvent_1__cinit__(PyObject *__pyx_v_self, PyObject *__pyx_args, PyObject *__pyx_kwds) {
  grpc_completion_type __pyx_v_completion_type;
  int __pyx_v_success;
  PyObject *__pyx_v_tag = 0;
  int __pyx_r;
  __Pyx_RefNannyDeclarations
  __Pyx_RefNannySetupContext("__cinit__ (wrapper)", 0);
  {
    static PyObject **__pyx_pyargnames[] = {&__pyx_n_s_completion_type,&__pyx_n_s_success,&__pyx_n_s_tag,0};
    PyObject* values[3] = {0,0,0};
    if (unlikely(__pyx_kwds)) {
      Py_ssize_t kw_args;
      const Py_ssize_t pos_args = PyTuple_GET_SIZE(__pyx_args);
      switch (pos_args) {
        case  3: values[2] = PyTuple_GET_ITEM(__pyx_args, 2);
        CYTHON_FALLTHROUGH;
        case  2: values[1] = PyTuple_GET_ITEM(__pyx_args, 1);
        CYTHON_FALLTHROUGH;
        case  1: values[0] = PyTuple_GET_ITEM(__pyx_args, 0);
        CYTHON_FALLTHROUGH;
        case  0: break;
        default: goto __pyx_L5_argtuple_error;
      }
      kw_args = PyDict_Size(__pyx_kwds);
      switch (pos_args) {
        case  0:
        if (likely((values[0] = __Pyx_PyDict_GetItemStr(__pyx_kwds, __pyx_n_s_completion_type)) != 0)) kw_args--;
        else goto __pyx_L5_argtuple_error;
        CYTHON_FALLTHROUGH;
        case  1:
        if (likely((values[1] = __Pyx_PyDict_GetItemStr(__pyx_kwds, __pyx_n_s_success)) != 0)) kw_args--;
        else {
          __Pyx_RaiseArgtupleInvalid("__cinit__", 1, 3, 3, 1); __PYX_ERR(11, 18, __pyx_L3_error)
        }
        CYTHON_FALLTHROUGH;
        case  2:
        if (likely((values[2] = __Pyx_PyDict_GetItemStr(__pyx_kwds, __pyx_n_s_tag)) != 0)) kw_args--;
        else {
          __Pyx_RaiseArgtupleInvalid("__cinit__", 1, 3, 3, 2); __PYX_ERR(11, 18, __pyx_L3_error)
        }
      }
      if (unlikely(kw_args > 0)) {
        if (unlikely(__Pyx_ParseOptionalKeywords(__pyx_kwds, __pyx_pyargnames, 0, values, pos_args, "__cinit__") < 0)) __PYX_ERR(11, 18, __pyx_L3_error)
      }
    } else if (PyTuple_GET_SIZE(__pyx_args) != 3) {
      goto __pyx_L5_argtuple_error;
    } else {
      values[0] = PyTuple_GET_ITEM(__pyx_args, 0);
      values[1] = PyTuple_GET_ITEM(__pyx_args, 1);
      values[2] = PyTuple_GET_ITEM(__pyx_args, 2);
    }
    __pyx_v_completion_type = ((grpc_completion_type)__Pyx_PyInt_As_grpc_completion_type(values[0])); if (unlikely(PyErr_Occurred())) __PYX_ERR(11, 19, __pyx_L3_error)
    __pyx_v_success = __Pyx_PyObject_IsTrue(values[1]); if (unlikely((__pyx_v_success == (int)-1) && PyErr_Occurred())) __PYX_ERR(11, 19, __pyx_L3_error)
    __pyx_v_tag = values[2];
  }
  goto __pyx_L4_argument_unpacking_done;
  __pyx_L5_argtuple_error:;
  __Pyx_RaiseArgtupleInvalid("__cinit__", 1, 3, 3, PyTuple_GET_SIZE(__pyx_args)); __PYX_ERR(11, 18, __pyx_L3_error)
  __pyx_L3_error:;
  __Pyx_AddTraceback("grpc._cython.cygrpc.ConnectivityEvent.__cinit__", __pyx_clineno, __pyx_lineno, __pyx_filename);
  __Pyx_RefNannyFinishContext();
  return -1;
  __pyx_L4_argument_unpacking_done:;
  __pyx_r = __pyx_pf_4grpc_7_cython_6cygrpc_17ConnectivityEvent___cinit__(((struct __pyx_obj_4grpc_7_cython_6cygrpc_ConnectivityEvent *)__pyx_v_self), __pyx_v_completion_type, __pyx_v_success, __pyx_v_tag);

  /* function exit code */
  __Pyx_RefNannyFinishContext();
  return __pyx_r;
}

static int __pyx_pf_4grpc_7_cython_6cygrpc_17ConnectivityEvent___cinit__(struct __pyx_obj_4grpc_7_cython_6cygrpc_ConnectivityEvent *__pyx_v_self, grpc_completion_type __pyx_v_completion_type, int __pyx_v_success, PyObject *__pyx_v_tag) {
  int __pyx_r;
  __Pyx_RefNannyDeclarations
  __Pyx_RefNannySetupContext("__cinit__", 0);

  /* "src/python/grpcio/grpc/_cython/_cygrpc/event.pyx.pxi":20
 *   def __cinit__(
 *       self, grpc_completion_type completion_type, bint success, object tag):
 *     self.completion_type = completion_type             # <<<<<<<<<<<<<<
 *     self.success = success
 *     self.tag = tag
 */
  __pyx_v_self->completion_type = __pyx_v_completion_type;

  /* "src/python/grpcio/grpc/_cython/_cygrpc/event.pyx.pxi":21
 *       self, grpc_completion_type completion_type, bint success, object tag):
 *     self.completion_type = completion_type
 *     self.success = success             # <<<<<<<<<<<<<<
 *     self.tag = tag
 * 
 */
  __pyx_v_self->success = __pyx_v_success;

  /* "src/python/grpcio/grpc/_cython/_cygrpc/event.pyx.pxi":22
 *     self.completion_type = completion_type
 *     self.success = success
 *     self.tag = tag             # <<<<<<<<<<<<<<
 * 
 * 
 */
  __Pyx_INCREF(__pyx_v_tag);
  __Pyx_GIVEREF(__pyx_v_tag);
  __Pyx_GOTREF(__pyx_v_self->tag);
  __Pyx_DECREF(__pyx_v_self->tag);
  __pyx_v_self->tag = __pyx_v_tag;

  /* "src/python/grpcio/grpc/_cython/_cygrpc/event.pyx.pxi":18
 * cdef class ConnectivityEvent:
 * 
 *   def __cinit__(             # <<<<<<<<<<<<<<
 *       self, grpc_completion_type completion_type, bint success, object tag):
 *     self.completion_type = completion_type
 */

  /* function exit code */
  __pyx_r = 0;
  __Pyx_RefNannyFinishContext();
  return __pyx_r;
}

/* "src/python/grpcio/grpc/_cython/_cygrpc/event.pxd.pxi":18
 * cdef class ConnectivityEvent:
 * 
 *   cdef readonly grpc_completion_type completion_type             # <<<<<<<<<<<<<<
 *   cdef readonly bint success
 *   cdef readonly object tag
 */

/* Python wrapper */
static PyObject *__pyx_pw_4grpc_7_cython_6cygrpc_17ConnectivityEvent_15completion_type_1__get__(PyObject *__pyx_v_self); /*proto*/
static PyObject *__pyx_pw_4grpc_7_cython_6cygrpc_17ConnectivityEvent_15completion_type_1__get__(PyObject *__pyx_v_self) {
  PyObject *__pyx_r = 0;
  __Pyx_RefNannyDeclarations
  __Pyx_RefNannySetupContext("__get__ (wrapper)", 0);
  __pyx_r = __pyx_pf_4grpc_7_cython_6cygrpc_17ConnectivityEvent_15completion_type___get__(((struct __pyx_obj_4grpc_7_cython_6cygrpc_ConnectivityEvent *)__pyx_v_self));

  /* function exit code */
  __Pyx_RefNannyFinishContext();
  return __pyx_r;
}

static PyObject *__pyx_pf_4grpc_7_cython_6cygrpc_17ConnectivityEvent_15completion_type___get__(struct __pyx_obj_4grpc_7_cython_6cygrpc_ConnectivityEvent *__pyx_v_self) {
  PyObject *__pyx_r = NULL;
  __Pyx_RefNannyDeclarations
  PyObject *__pyx_t_1 = NULL;
  __Pyx_RefNannySetupContext("__get__", 0);
  __Pyx_XDECREF(__pyx_r);
  __pyx_t_1 = __Pyx_PyInt_From_grpc_completion_type(__pyx_v_self->completion_type); if (unlikely(!__pyx_t_1)) __PYX_ERR(12, 18, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_1);
  __pyx_r = __pyx_t_1;
  __pyx_t_1 = 0;
  goto __pyx_L0;

  /* function exit code */
  __pyx_L1_error:;
  __Pyx_XDECREF(__pyx_t_1);
  __Pyx_AddTraceback("grpc._cython.cygrpc.ConnectivityEvent.completion_type.__get__", __pyx_clineno, __pyx_lineno, __pyx_filename);
  __pyx_r = NULL;
  __pyx_L0:;
  __Pyx_XGIVEREF(__pyx_r);
  __Pyx_RefNannyFinishContext();
  return __pyx_r;
}

/* "src/python/grpcio/grpc/_cython/_cygrpc/event.pxd.pxi":19
 * 
 *   cdef readonly grpc_completion_type completion_type
 *   cdef readonly bint success             # <<<<<<<<<<<<<<
 *   cdef readonly object tag
 * 
 */

/* Python wrapper */
static PyObject *__pyx_pw_4grpc_7_cython_6cygrpc_17ConnectivityEvent_7success_1__get__(PyObject *__pyx_v_self); /*proto*/
static PyObject *__pyx_pw_4grpc_7_cython_6cygrpc_17ConnectivityEvent_7success_1__get__(PyObject *__pyx_v_self) {
  PyObject *__pyx_r = 0;
  __Pyx_RefNannyDeclarations
  __Pyx_RefNannySetupContext("__get__ (wrapper)", 0);
  __pyx_r = __pyx_pf_4grpc_7_cython_6cygrpc_17ConnectivityEvent_7success___get__(((struct __pyx_obj_4grpc_7_cython_6cygrpc_ConnectivityEvent *)__pyx_v_self));

  /* function exit code */
  __Pyx_RefNannyFinishContext();
  return __pyx_r;
}

static PyObject *__pyx_pf_4grpc_7_cython_6cygrpc_17ConnectivityEvent_7success___get__(struct __pyx_obj_4grpc_7_cython_6cygrpc_ConnectivityEvent *__pyx_v_self) {
  PyObject *__pyx_r = NULL;
  __Pyx_RefNannyDeclarations
  PyObject *__pyx_t_1 = NULL;
  __Pyx_RefNannySetupContext("__get__", 0);
  __Pyx_XDECREF(__pyx_r);
  __pyx_t_1 = __Pyx_PyBool_FromLong(__pyx_v_self->success); if (unlikely(!__pyx_t_1)) __PYX_ERR(12, 19, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_1);
  __pyx_r = __pyx_t_1;
  __pyx_t_1 = 0;
  goto __pyx_L0;

  /* function exit code */
  __pyx_L1_error:;
  __Pyx_XDECREF(__pyx_t_1);
  __Pyx_AddTraceback("grpc._cython.cygrpc.ConnectivityEvent.success.__get__", __pyx_clineno, __pyx_lineno, __pyx_filename);
  __pyx_r = NULL;
  __pyx_L0:;
  __Pyx_XGIVEREF(__pyx_r);
  __Pyx_RefNannyFinishContext();
  return __pyx_r;
}

/* "src/python/grpcio/grpc/_cython/_cygrpc/event.pxd.pxi":20
 *   cdef readonly grpc_completion_type completion_type
 *   cdef readonly bint success
 *   cdef readonly object tag             # <<<<<<<<<<<<<<
 * 
 * 
 */

/* Python wrapper */
static PyObject *__pyx_pw_4grpc_7_cython_6cygrpc_17ConnectivityEvent_3tag_1__get__(PyObject *__pyx_v_self); /*proto*/
static PyObject *__pyx_pw_4grpc_7_cython_6cygrpc_17ConnectivityEvent_3tag_1__get__(PyObject *__pyx_v_self) {
  PyObject *__pyx_r = 0;
  __Pyx_RefNannyDeclarations
  __Pyx_RefNannySetupContext("__get__ (wrapper)", 0);
  __pyx_r = __pyx_pf_4grpc_7_cython_6cygrpc_17ConnectivityEvent_3tag___get__(((struct __pyx_obj_4grpc_7_cython_6cygrpc_ConnectivityEvent *)__pyx_v_self));

  /* function exit code */
  __Pyx_RefNannyFinishContext();
  return __pyx_r;
}

static PyObject *__pyx_pf_4grpc_7_cython_6cygrpc_17ConnectivityEvent_3tag___get__(struct __pyx_obj_4grpc_7_cython_6cygrpc_ConnectivityEvent *__pyx_v_self) {
  PyObject *__pyx_r = NULL;
  __Pyx_RefNannyDeclarations
  __Pyx_RefNannySetupContext("__get__", 0);
  __Pyx_XDECREF(__pyx_r);
  __Pyx_INCREF(__pyx_v_self->tag);
  __pyx_r = __pyx_v_self->tag;
  goto __pyx_L0;

  /* function exit code */
  __pyx_L0:;
  __Pyx_XGIVEREF(__pyx_r);
  __Pyx_RefNannyFinishContext();
  return __pyx_r;
}

/* "(tree fragment)":1
 * def __reduce_cython__(self):             # <<<<<<<<<<<<<<
 *     raise TypeError("no default __reduce__ due to non-trivial __cinit__")
 * def __setstate_cython__(self, __pyx_state):
 */

/* Python wrapper */
static PyObject *__pyx_pw_4grpc_7_cython_6cygrpc_17ConnectivityEvent_3__reduce_cython__(PyObject *__pyx_v_self, CYTHON_UNUSED PyObject *unused); /*proto*/
static PyObject *__pyx_pw_4grpc_7_cython_6cygrpc_17ConnectivityEvent_3__reduce_cython__(PyObject *__pyx_v_self, CYTHON_UNUSED PyObject *unused) {
  PyObject *__pyx_r = 0;
  __Pyx_RefNannyDeclarations
  __Pyx_RefNannySetupContext("__reduce_cython__ (wrapper)", 0);
  __pyx_r = __pyx_pf_4grpc_7_cython_6cygrpc_17ConnectivityEvent_2__reduce_cython__(((struct __pyx_obj_4grpc_7_cython_6cygrpc_ConnectivityEvent *)__pyx_v_self));

  /* function exit code */
  __Pyx_RefNannyFinishContext();
  return __pyx_r;
}

static PyObject *__pyx_pf_4grpc_7_cython_6cygrpc_17ConnectivityEvent_2__reduce_cython__(CYTHON_UNUSED struct __pyx_obj_4grpc_7_cython_6cygrpc_ConnectivityEvent *__pyx_v_self) {
  PyObject *__pyx_r = NULL;
  __Pyx_RefNannyDeclarations
  PyObject *__pyx_t_1 = NULL;
  __Pyx_RefNannySetupContext("__reduce_cython__", 0);

  /* "(tree fragment)":2
 * def __reduce_cython__(self):
 *     raise TypeError("no default __reduce__ due to non-trivial __cinit__")             # <<<<<<<<<<<<<<
 * def __setstate_cython__(self, __pyx_state):
 *     raise TypeError("no default __reduce__ due to non-trivial __cinit__")
 */
  __pyx_t_1 = __Pyx_PyObject_Call(__pyx_builtin_TypeError, __pyx_tuple__76, NULL); if (unlikely(!__pyx_t_1)) __PYX_ERR(7, 2, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_1);
  __Pyx_Raise(__pyx_t_1, 0, 0, 0);
  __Pyx_DECREF(__pyx_t_1); __pyx_t_1 = 0;
  __PYX_ERR(7, 2, __pyx_L1_error)

  /* "(tree fragment)":1
 * def __reduce_cython__(self):             # <<<<<<<<<<<<<<
 *     raise TypeError("no default __reduce__ due to non-trivial __cinit__")
 * def __setstate_cython__(self, __pyx_state):
 */

  /* function exit code */
  __pyx_L1_error:;
  __Pyx_XDECREF(__pyx_t_1);
  __Pyx_AddTraceback("grpc._cython.cygrpc.ConnectivityEvent.__reduce_cython__", __pyx_clineno, __pyx_lineno, __pyx_filename);
  __pyx_r = NULL;
  __Pyx_XGIVEREF(__pyx_r);
  __Pyx_RefNannyFinishContext();
  return __pyx_r;
}

/* "(tree fragment)":3
 * def __reduce_cython__(self):
 *     raise TypeError("no default __reduce__ due to non-trivial __cinit__")
 * def __setstate_cython__(self, __pyx_state):             # <<<<<<<<<<<<<<
 *     raise TypeError("no default __reduce__ due to non-trivial __cinit__")
 */

/* Python wrapper */
static PyObject *__pyx_pw_4grpc_7_cython_6cygrpc_17ConnectivityEvent_5__setstate_cython__(PyObject *__pyx_v_self, PyObject *__pyx_v___pyx_state); /*proto*/
static PyObject *__pyx_pw_4grpc_7_cython_6cygrpc_17ConnectivityEvent_5__setstate_cython__(PyObject *__pyx_v_self, PyObject *__pyx_v___pyx_state) {
  PyObject *__pyx_r = 0;
  __Pyx_RefNannyDeclarations
  __Pyx_RefNannySetupContext("__setstate_cython__ (wrapper)", 0);
  __pyx_r = __pyx_pf_4grpc_7_cython_6cygrpc_17ConnectivityEvent_4__setstate_cython__(((struct __pyx_obj_4grpc_7_cython_6cygrpc_ConnectivityEvent *)__pyx_v_self), ((PyObject *)__pyx_v___pyx_state));

  /* function exit code */
  __Pyx_RefNannyFinishContext();
  return __pyx_r;
}

static PyObject *__pyx_pf_4grpc_7_cython_6cygrpc_17ConnectivityEvent_4__setstate_cython__(CYTHON_UNUSED struct __pyx_obj_4grpc_7_cython_6cygrpc_ConnectivityEvent *__pyx_v_self, CYTHON_UNUSED PyObject *__pyx_v___pyx_state) {
  PyObject *__pyx_r = NULL;
  __Pyx_RefNannyDeclarations
  PyObject *__pyx_t_1 = NULL;
  __Pyx_RefNannySetupContext("__setstate_cython__", 0);

  /* "(tree fragment)":4
 *     raise TypeError("no default __reduce__ due to non-trivial __cinit__")
 * def __setstate_cython__(self, __pyx_state):
 *     raise TypeError("no default __reduce__ due to non-trivial __cinit__")             # <<<<<<<<<<<<<<
 */
  __pyx_t_1 = __Pyx_PyObject_Call(__pyx_builtin_TypeError, __pyx_tuple__77, NULL); if (unlikely(!__pyx_t_1)) __PYX_ERR(7, 4, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_1);
  __Pyx_Raise(__pyx_t_1, 0, 0, 0);
  __Pyx_DECREF(__pyx_t_1); __pyx_t_1 = 0;
  __PYX_ERR(7, 4, __pyx_L1_error)

  /* "(tree fragment)":3
 * def __reduce_cython__(self):
 *     raise TypeError("no default __reduce__ due to non-trivial __cinit__")
 * def __setstate_cython__(self, __pyx_state):             # <<<<<<<<<<<<<<
 *     raise TypeError("no default __reduce__ due to non-trivial __cinit__")
 */

  /* function exit code */
  __pyx_L1_error:;
  __Pyx_XDECREF(__pyx_t_1);
  __Pyx_AddTraceback("grpc._cython.cygrpc.ConnectivityEvent.__setstate_cython__", __pyx_clineno, __pyx_lineno, __pyx_filename);
  __pyx_r = NULL;
  __Pyx_XGIVEREF(__pyx_r);
  __Pyx_RefNannyFinishContext();
  return __pyx_r;
}

/* "src/python/grpcio/grpc/_cython/_cygrpc/event.pyx.pxi":27
 * cdef class RequestCallEvent:
 * 
 *   def __cinit__(             # <<<<<<<<<<<<<<
 *       self, grpc_completion_type completion_type, bint success, object tag,
 *       Call call, CallDetails call_details, tuple invocation_metadata):
 */

/* Python wrapper */
static int __pyx_pw_4grpc_7_cython_6cygrpc_16RequestCallEvent_1__cinit__(PyObject *__pyx_v_self, PyObject *__pyx_args, PyObject *__pyx_kwds); /*proto*/
static int __pyx_pw_4grpc_7_cython_6cygrpc_16RequestCallEvent_1__cinit__(PyObject *__pyx_v_self, PyObject *__pyx_args, PyObject *__pyx_kwds) {
  grpc_completion_type __pyx_v_completion_type;
  int __pyx_v_success;
  PyObject *__pyx_v_tag = 0;
  struct __pyx_obj_4grpc_7_cython_6cygrpc_Call *__pyx_v_call = 0;
  struct __pyx_obj_4grpc_7_cython_6cygrpc_CallDetails *__pyx_v_call_details = 0;
  PyObject *__pyx_v_invocation_metadata = 0;
  int __pyx_r;
  __Pyx_RefNannyDeclarations
  __Pyx_RefNannySetupContext("__cinit__ (wrapper)", 0);
  {
    static PyObject **__pyx_pyargnames[] = {&__pyx_n_s_completion_type,&__pyx_n_s_success,&__pyx_n_s_tag,&__pyx_n_s_call,&__pyx_n_s_call_details,&__pyx_n_s_invocation_metadata,0};
    PyObject* values[6] = {0,0,0,0,0,0};
    if (unlikely(__pyx_kwds)) {
      Py_ssize_t kw_args;
      const Py_ssize_t pos_args = PyTuple_GET_SIZE(__pyx_args);
      switch (pos_args) {
        case  6: values[5] = PyTuple_GET_ITEM(__pyx_args, 5);
        CYTHON_FALLTHROUGH;
        case  5: values[4] = PyTuple_GET_ITEM(__pyx_args, 4);
        CYTHON_FALLTHROUGH;
        case  4: values[3] = PyTuple_GET_ITEM(__pyx_args, 3);
        CYTHON_FALLTHROUGH;
        case  3: values[2] = PyTuple_GET_ITEM(__pyx_args, 2);
        CYTHON_FALLTHROUGH;
        case  2: values[1] = PyTuple_GET_ITEM(__pyx_args, 1);
        CYTHON_FALLTHROUGH;
        case  1: values[0] = PyTuple_GET_ITEM(__pyx_args, 0);
        CYTHON_FALLTHROUGH;
        case  0: break;
        default: goto __pyx_L5_argtuple_error;
      }
      kw_args = PyDict_Size(__pyx_kwds);
      switch (pos_args) {
        case  0:
        if (likely((values[0] = __Pyx_PyDict_GetItemStr(__pyx_kwds, __pyx_n_s_completion_type)) != 0)) kw_args--;
        else goto __pyx_L5_argtuple_error;
        CYTHON_FALLTHROUGH;
        case  1:
        if (likely((values[1] = __Pyx_PyDict_GetItemStr(__pyx_kwds, __pyx_n_s_success)) != 0)) kw_args--;
        else {
          __Pyx_RaiseArgtupleInvalid("__cinit__", 1, 6, 6, 1); __PYX_ERR(11, 27, __pyx_L3_error)
        }
        CYTHON_FALLTHROUGH;
        case  2:
        if (likely((values[2] = __Pyx_PyDict_GetItemStr(__pyx_kwds, __pyx_n_s_tag)) != 0)) kw_args--;
        else {
          __Pyx_RaiseArgtupleInvalid("__cinit__", 1, 6, 6, 2); __PYX_ERR(11, 27, __pyx_L3_error)
        }
        CYTHON_FALLTHROUGH;
        case  3:
        if (likely((values[3] = __Pyx_PyDict_GetItemStr(__pyx_kwds, __pyx_n_s_call)) != 0)) kw_args--;
        else {
          __Pyx_RaiseArgtupleInvalid("__cinit__", 1, 6, 6, 3); __PYX_ERR(11, 27, __pyx_L3_error)
        }
        CYTHON_FALLTHROUGH;
        case  4:
        if (likely((values[4] = __Pyx_PyDict_GetItemStr(__pyx_kwds, __pyx_n_s_call_details)) != 0)) kw_args--;
        else {
          __Pyx_RaiseArgtupleInvalid("__cinit__", 1, 6, 6, 4); __PYX_ERR(11, 27, __pyx_L3_error)
        }
        CYTHON_FALLTHROUGH;
        case  5:
        if (likely((values[5] = __Pyx_PyDict_GetItemStr(__pyx_kwds, __pyx_n_s_invocation_metadata)) != 0)) kw_args--;
        else {
          __Pyx_RaiseArgtupleInvalid("__cinit__", 1, 6, 6, 5); __PYX_ERR(11, 27, __pyx_L3_error)
        }
      }
      if (unlikely(kw_args > 0)) {
        if (unlikely(__Pyx_ParseOptionalKeywords(__pyx_kwds, __pyx_pyargnames, 0, values, pos_args, "__cinit__") < 0)) __PYX_ERR(11, 27, __pyx_L3_error)
      }
    } else if (PyTuple_GET_SIZE(__pyx_args) != 6) {
      goto __pyx_L5_argtuple_error;
    } else {
      values[0] = PyTuple_GET_ITEM(__pyx_args, 0);
      values[1] = PyTuple_GET_ITEM(__pyx_args, 1);
      values[2] = PyTuple_GET_ITEM(__pyx_args, 2);
      values[3] = PyTuple_GET_ITEM(__pyx_args, 3);
      values[4] = PyTuple_GET_ITEM(__pyx_args, 4);
      values[5] = PyTuple_GET_ITEM(__pyx_args, 5);
    }
    __pyx_v_completion_type = ((grpc_completion_type)__Pyx_PyInt_As_grpc_completion_type(values[0])); if (unlikely(PyErr_Occurred())) __PYX_ERR(11, 28, __pyx_L3_error)
    __pyx_v_success = __Pyx_PyObject_IsTrue(values[1]); if (unlikely((__pyx_v_success == (int)-1) && PyErr_Occurred())) __PYX_ERR(11, 28, __pyx_L3_error)
    __pyx_v_tag = values[2];
    __pyx_v_call = ((struct __pyx_obj_4grpc_7_cython_6cygrpc_Call *)values[3]);
    __pyx_v_call_details = ((struct __pyx_obj_4grpc_7_cython_6cygrpc_CallDetails *)values[4]);
    __pyx_v_invocation_metadata = ((PyObject*)values[5]);
  }
  goto __pyx_L4_argument_unpacking_done;
  __pyx_L5_argtuple_error:;
  __Pyx_RaiseArgtupleInvalid("__cinit__", 1, 6, 6, PyTuple_GET_SIZE(__pyx_args)); __PYX_ERR(11, 27, __pyx_L3_error)
  __pyx_L3_error:;
  __Pyx_AddTraceback("grpc._cython.cygrpc.RequestCallEvent.__cinit__", __pyx_clineno, __pyx_lineno, __pyx_filename);
  __Pyx_RefNannyFinishContext();
  return -1;
  __pyx_L4_argument_unpacking_done:;
  if (unlikely(!__Pyx_ArgTypeTest(((PyObject *)__pyx_v_call), __pyx_ptype_4grpc_7_cython_6cygrpc_Call, 1, "call", 0))) __PYX_ERR(11, 29, __pyx_L1_error)
  if (unlikely(!__Pyx_ArgTypeTest(((PyObject *)__pyx_v_call_details), __pyx_ptype_4grpc_7_cython_6cygrpc_CallDetails, 1, "call_details", 0))) __PYX_ERR(11, 29, __pyx_L1_error)
  if (unlikely(!__Pyx_ArgTypeTest(((PyObject *)__pyx_v_invocation_metadata), (&PyTuple_Type), 1, "invocation_metadata", 1))) __PYX_ERR(11, 29, __pyx_L1_error)
  __pyx_r = __pyx_pf_4grpc_7_cython_6cygrpc_16RequestCallEvent___cinit__(((struct __pyx_obj_4grpc_7_cython_6cygrpc_RequestCallEvent *)__pyx_v_self), __pyx_v_completion_type, __pyx_v_success, __pyx_v_tag, __pyx_v_call, __pyx_v_call_details, __pyx_v_invocation_metadata);

  /* function exit code */
  goto __pyx_L0;
  __pyx_L1_error:;
  __pyx_r = -1;
  __pyx_L0:;
  __Pyx_RefNannyFinishContext();
  return __pyx_r;
}

static int __pyx_pf_4grpc_7_cython_6cygrpc_16RequestCallEvent___cinit__(struct __pyx_obj_4grpc_7_cython_6cygrpc_RequestCallEvent *__pyx_v_self, grpc_completion_type __pyx_v_completion_type, int __pyx_v_success, PyObject *__pyx_v_tag, struct __pyx_obj_4grpc_7_cython_6cygrpc_Call *__pyx_v_call, struct __pyx_obj_4grpc_7_cython_6cygrpc_CallDetails *__pyx_v_call_details, PyObject *__pyx_v_invocation_metadata) {
  int __pyx_r;
  __Pyx_RefNannyDeclarations
  __Pyx_RefNannySetupContext("__cinit__", 0);

  /* "src/python/grpcio/grpc/_cython/_cygrpc/event.pyx.pxi":30
 *       self, grpc_completion_type completion_type, bint success, object tag,
 *       Call call, CallDetails call_details, tuple invocation_metadata):
 *     self.completion_type = completion_type             # <<<<<<<<<<<<<<
 *     self.success = success
 *     self.tag = tag
 */
  __pyx_v_self->completion_type = __pyx_v_completion_type;

  /* "src/python/grpcio/grpc/_cython/_cygrpc/event.pyx.pxi":31
 *       Call call, CallDetails call_details, tuple invocation_metadata):
 *     self.completion_type = completion_type
 *     self.success = success             # <<<<<<<<<<<<<<
 *     self.tag = tag
 *     self.call = call
 */
  __pyx_v_self->success = __pyx_v_success;

  /* "src/python/grpcio/grpc/_cython/_cygrpc/event.pyx.pxi":32
 *     self.completion_type = completion_type
 *     self.success = success
 *     self.tag = tag             # <<<<<<<<<<<<<<
 *     self.call = call
 *     self.call_details = call_details
 */
  __Pyx_INCREF(__pyx_v_tag);
  __Pyx_GIVEREF(__pyx_v_tag);
  __Pyx_GOTREF(__pyx_v_self->tag);
  __Pyx_DECREF(__pyx_v_self->tag);
  __pyx_v_self->tag = __pyx_v_tag;

  /* "src/python/grpcio/grpc/_cython/_cygrpc/event.pyx.pxi":33
 *     self.success = success
 *     self.tag = tag
 *     self.call = call             # <<<<<<<<<<<<<<
 *     self.call_details = call_details
 *     self.invocation_metadata = invocation_metadata
 */
  __Pyx_INCREF(((PyObject *)__pyx_v_call));
  __Pyx_GIVEREF(((PyObject *)__pyx_v_call));
  __Pyx_GOTREF(__pyx_v_self->call);
  __Pyx_DECREF(((PyObject *)__pyx_v_self->call));
  __pyx_v_self->call = __pyx_v_call;

  /* "src/python/grpcio/grpc/_cython/_cygrpc/event.pyx.pxi":34
 *     self.tag = tag
 *     self.call = call
 *     self.call_details = call_details             # <<<<<<<<<<<<<<
 *     self.invocation_metadata = invocation_metadata
 * 
 */
  __Pyx_INCREF(((PyObject *)__pyx_v_call_details));
  __Pyx_GIVEREF(((PyObject *)__pyx_v_call_details));
  __Pyx_GOTREF(__pyx_v_self->call_details);
  __Pyx_DECREF(((PyObject *)__pyx_v_self->call_details));
  __pyx_v_self->call_details = __pyx_v_call_details;

  /* "src/python/grpcio/grpc/_cython/_cygrpc/event.pyx.pxi":35
 *     self.call = call
 *     self.call_details = call_details
 *     self.invocation_metadata = invocation_metadata             # <<<<<<<<<<<<<<
 * 
 * 
 */
  __Pyx_INCREF(__pyx_v_invocation_metadata);
  __Pyx_GIVEREF(__pyx_v_invocation_metadata);
  __Pyx_GOTREF(__pyx_v_self->invocation_metadata);
  __Pyx_DECREF(__pyx_v_self->invocation_metadata);
  __pyx_v_self->invocation_metadata = __pyx_v_invocation_metadata;

  /* "src/python/grpcio/grpc/_cython/_cygrpc/event.pyx.pxi":27
 * cdef class RequestCallEvent:
 * 
 *   def __cinit__(             # <<<<<<<<<<<<<<
 *       self, grpc_completion_type completion_type, bint success, object tag,
 *       Call call, CallDetails call_details, tuple invocation_metadata):
 */

  /* function exit code */
  __pyx_r = 0;
  __Pyx_RefNannyFinishContext();
  return __pyx_r;
}

/* "src/python/grpcio/grpc/_cython/_cygrpc/event.pxd.pxi":25
 * cdef class RequestCallEvent:
 * 
 *   cdef readonly grpc_completion_type completion_type             # <<<<<<<<<<<<<<
 *   cdef readonly bint success
 *   cdef readonly object tag
 */

/* Python wrapper */
static PyObject *__pyx_pw_4grpc_7_cython_6cygrpc_16RequestCallEvent_15completion_type_1__get__(PyObject *__pyx_v_self); /*proto*/
static PyObject *__pyx_pw_4grpc_7_cython_6cygrpc_16RequestCallEvent_15completion_type_1__get__(PyObject *__pyx_v_self) {
  PyObject *__pyx_r = 0;
  __Pyx_RefNannyDeclarations
  __Pyx_RefNannySetupContext("__get__ (wrapper)", 0);
  __pyx_r = __pyx_pf_4grpc_7_cython_6cygrpc_16RequestCallEvent_15completion_type___get__(((struct __pyx_obj_4grpc_7_cython_6cygrpc_RequestCallEvent *)__pyx_v_self));

  /* function exit code */
  __Pyx_RefNannyFinishContext();
  return __pyx_r;
}

static PyObject *__pyx_pf_4grpc_7_cython_6cygrpc_16RequestCallEvent_15completion_type___get__(struct __pyx_obj_4grpc_7_cython_6cygrpc_RequestCallEvent *__pyx_v_self) {
  PyObject *__pyx_r = NULL;
  __Pyx_RefNannyDeclarations
  PyObject *__pyx_t_1 = NULL;
  __Pyx_RefNannySetupContext("__get__", 0);
  __Pyx_XDECREF(__pyx_r);
  __pyx_t_1 = __Pyx_PyInt_From_grpc_completion_type(__pyx_v_self->completion_type); if (unlikely(!__pyx_t_1)) __PYX_ERR(12, 25, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_1);
  __pyx_r = __pyx_t_1;
  __pyx_t_1 = 0;
  goto __pyx_L0;

  /* function exit code */
  __pyx_L1_error:;
  __Pyx_XDECREF(__pyx_t_1);
  __Pyx_AddTraceback("grpc._cython.cygrpc.RequestCallEvent.completion_type.__get__", __pyx_clineno, __pyx_lineno, __pyx_filename);
  __pyx_r = NULL;
  __pyx_L0:;
  __Pyx_XGIVEREF(__pyx_r);
  __Pyx_RefNannyFinishContext();
  return __pyx_r;
}

/* "src/python/grpcio/grpc/_cython/_cygrpc/event.pxd.pxi":26
 * 
 *   cdef readonly grpc_completion_type completion_type
 *   cdef readonly bint success             # <<<<<<<<<<<<<<
 *   cdef readonly object tag
 *   cdef readonly Call call
 */

/* Python wrapper */
static PyObject *__pyx_pw_4grpc_7_cython_6cygrpc_16RequestCallEvent_7success_1__get__(PyObject *__pyx_v_self); /*proto*/
static PyObject *__pyx_pw_4grpc_7_cython_6cygrpc_16RequestCallEvent_7success_1__get__(PyObject *__pyx_v_self) {
  PyObject *__pyx_r = 0;
  __Pyx_RefNannyDeclarations
  __Pyx_RefNannySetupContext("__get__ (wrapper)", 0);
  __pyx_r = __pyx_pf_4grpc_7_cython_6cygrpc_16RequestCallEvent_7success___get__(((struct __pyx_obj_4grpc_7_cython_6cygrpc_RequestCallEvent *)__pyx_v_self));

  /* function exit code */
  __Pyx_RefNannyFinishContext();
  return __pyx_r;
}

static PyObject *__pyx_pf_4grpc_7_cython_6cygrpc_16RequestCallEvent_7success___get__(struct __pyx_obj_4grpc_7_cython_6cygrpc_RequestCallEvent *__pyx_v_self) {
  PyObject *__pyx_r = NULL;
  __Pyx_RefNannyDeclarations
  PyObject *__pyx_t_1 = NULL;
  __Pyx_RefNannySetupContext("__get__", 0);
  __Pyx_XDECREF(__pyx_r);
  __pyx_t_1 = __Pyx_PyBool_FromLong(__pyx_v_self->success); if (unlikely(!__pyx_t_1)) __PYX_ERR(12, 26, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_1);
  __pyx_r = __pyx_t_1;
  __pyx_t_1 = 0;
  goto __pyx_L0;

  /* function exit code */
  __pyx_L1_error:;
  __Pyx_XDECREF(__pyx_t_1);
  __Pyx_AddTraceback("grpc._cython.cygrpc.RequestCallEvent.success.__get__", __pyx_clineno, __pyx_lineno, __pyx_filename);
  __pyx_r = NULL;
  __pyx_L0:;
  __Pyx_XGIVEREF(__pyx_r);
  __Pyx_RefNannyFinishContext();
  return __pyx_r;
}

/* "src/python/grpcio/grpc/_cython/_cygrpc/event.pxd.pxi":27
 *   cdef readonly grpc_completion_type completion_type
 *   cdef readonly bint success
 *   cdef readonly object tag             # <<<<<<<<<<<<<<
 *   cdef readonly Call call
 *   cdef readonly CallDetails call_details
 */

/* Python wrapper */
static PyObject *__pyx_pw_4grpc_7_cython_6cygrpc_16RequestCallEvent_3tag_1__get__(PyObject *__pyx_v_self); /*proto*/
static PyObject *__pyx_pw_4grpc_7_cython_6cygrpc_16RequestCallEvent_3tag_1__get__(PyObject *__pyx_v_self) {
  PyObject *__pyx_r = 0;
  __Pyx_RefNannyDeclarations
  __Pyx_RefNannySetupContext("__get__ (wrapper)", 0);
  __pyx_r = __pyx_pf_4grpc_7_cython_6cygrpc_16RequestCallEvent_3tag___get__(((struct __pyx_obj_4grpc_7_cython_6cygrpc_RequestCallEvent *)__pyx_v_self));

  /* function exit code */
  __Pyx_RefNannyFinishContext();
  return __pyx_r;
}

static PyObject *__pyx_pf_4grpc_7_cython_6cygrpc_16RequestCallEvent_3tag___get__(struct __pyx_obj_4grpc_7_cython_6cygrpc_RequestCallEvent *__pyx_v_self) {
  PyObject *__pyx_r = NULL;
  __Pyx_RefNannyDeclarations
  __Pyx_RefNannySetupContext("__get__", 0);
  __Pyx_XDECREF(__pyx_r);
  __Pyx_INCREF(__pyx_v_self->tag);
  __pyx_r = __pyx_v_self->tag;
  goto __pyx_L0;

  /* function exit code */
  __pyx_L0:;
  __Pyx_XGIVEREF(__pyx_r);
  __Pyx_RefNannyFinishContext();
  return __pyx_r;
}

/* "src/python/grpcio/grpc/_cython/_cygrpc/event.pxd.pxi":28
 *   cdef readonly bint success
 *   cdef readonly object tag
 *   cdef readonly Call call             # <<<<<<<<<<<<<<
 *   cdef readonly CallDetails call_details
 *   cdef readonly tuple invocation_metadata
 */

/* Python wrapper */
static PyObject *__pyx_pw_4grpc_7_cython_6cygrpc_16RequestCallEvent_4call_1__get__(PyObject *__pyx_v_self); /*proto*/
static PyObject *__pyx_pw_4grpc_7_cython_6cygrpc_16RequestCallEvent_4call_1__get__(PyObject *__pyx_v_self) {
  PyObject *__pyx_r = 0;
  __Pyx_RefNannyDeclarations
  __Pyx_RefNannySetupContext("__get__ (wrapper)", 0);
  __pyx_r = __pyx_pf_4grpc_7_cython_6cygrpc_16RequestCallEvent_4call___get__(((struct __pyx_obj_4grpc_7_cython_6cygrpc_RequestCallEvent *)__pyx_v_self));

  /* function exit code */
  __Pyx_RefNannyFinishContext();
  return __pyx_r;
}

static PyObject *__pyx_pf_4grpc_7_cython_6cygrpc_16RequestCallEvent_4call___get__(struct __pyx_obj_4grpc_7_cython_6cygrpc_RequestCallEvent *__pyx_v_self) {
  PyObject *__pyx_r = NULL;
  __Pyx_RefNannyDeclarations
  __Pyx_RefNannySetupContext("__get__", 0);
  __Pyx_XDECREF(__pyx_r);
  __Pyx_INCREF(((PyObject *)__pyx_v_self->call));
  __pyx_r = ((PyObject *)__pyx_v_self->call);
  goto __pyx_L0;

  /* function exit code */
  __pyx_L0:;
  __Pyx_XGIVEREF(__pyx_r);
  __Pyx_RefNannyFinishContext();
  return __pyx_r;
}

/* "src/python/grpcio/grpc/_cython/_cygrpc/event.pxd.pxi":29
 *   cdef readonly object tag
 *   cdef readonly Call call
 *   cdef readonly CallDetails call_details             # <<<<<<<<<<<<<<
 *   cdef readonly tuple invocation_metadata
 * 
 */

/* Python wrapper */
static PyObject *__pyx_pw_4grpc_7_cython_6cygrpc_16RequestCallEvent_12call_details_1__get__(PyObject *__pyx_v_self); /*proto*/
static PyObject *__pyx_pw_4grpc_7_cython_6cygrpc_16RequestCallEvent_12call_details_1__get__(PyObject *__pyx_v_self) {
  PyObject *__pyx_r = 0;
  __Pyx_RefNannyDeclarations
  __Pyx_RefNannySetupContext("__get__ (wrapper)", 0);
  __pyx_r = __pyx_pf_4grpc_7_cython_6cygrpc_16RequestCallEvent_12call_details___get__(((struct __pyx_obj_4grpc_7_cython_6cygrpc_RequestCallEvent *)__pyx_v_self));

  /* function exit code */
  __Pyx_RefNannyFinishContext();
  return __pyx_r;
}

static PyObject *__pyx_pf_4grpc_7_cython_6cygrpc_16RequestCallEvent_12call_details___get__(struct __pyx_obj_4grpc_7_cython_6cygrpc_RequestCallEvent *__pyx_v_self) {
  PyObject *__pyx_r = NULL;
  __Pyx_RefNannyDeclarations
  __Pyx_RefNannySetupContext("__get__", 0);
  __Pyx_XDECREF(__pyx_r);
  __Pyx_INCREF(((PyObject *)__pyx_v_self->call_details));
  __pyx_r = ((PyObject *)__pyx_v_self->call_details);
  goto __pyx_L0;

  /* function exit code */
  __pyx_L0:;
  __Pyx_XGIVEREF(__pyx_r);
  __Pyx_RefNannyFinishContext();
  return __pyx_r;
}

/* "src/python/grpcio/grpc/_cython/_cygrpc/event.pxd.pxi":30
 *   cdef readonly Call call
 *   cdef readonly CallDetails call_details
 *   cdef readonly tuple invocation_metadata             # <<<<<<<<<<<<<<
 * 
 * 
 */

/* Python wrapper */
static PyObject *__pyx_pw_4grpc_7_cython_6cygrpc_16RequestCallEvent_19invocation_metadata_1__get__(PyObject *__pyx_v_self); /*proto*/
static PyObject *__pyx_pw_4grpc_7_cython_6cygrpc_16RequestCallEvent_19invocation_metadata_1__get__(PyObject *__pyx_v_self) {
  PyObject *__pyx_r = 0;
  __Pyx_RefNannyDeclarations
  __Pyx_RefNannySetupContext("__get__ (wrapper)", 0);
  __pyx_r = __pyx_pf_4grpc_7_cython_6cygrpc_16RequestCallEvent_19invocation_metadata___get__(((struct __pyx_obj_4grpc_7_cython_6cygrpc_RequestCallEvent *)__pyx_v_self));

  /* function exit code */
  __Pyx_RefNannyFinishContext();
  return __pyx_r;
}

static PyObject *__pyx_pf_4grpc_7_cython_6cygrpc_16RequestCallEvent_19invocation_metadata___get__(struct __pyx_obj_4grpc_7_cython_6cygrpc_RequestCallEvent *__pyx_v_self) {
  PyObject *__pyx_r = NULL;
  __Pyx_RefNannyDeclarations
  __Pyx_RefNannySetupContext("__get__", 0);
  __Pyx_XDECREF(__pyx_r);
  __Pyx_INCREF(__pyx_v_self->invocation_metadata);
  __pyx_r = __pyx_v_self->invocation_metadata;
  goto __pyx_L0;

  /* function exit code */
  __pyx_L0:;
  __Pyx_XGIVEREF(__pyx_r);
  __Pyx_RefNannyFinishContext();
  return __pyx_r;
}

/* "(tree fragment)":1
 * def __reduce_cython__(self):             # <<<<<<<<<<<<<<
 *     raise TypeError("no default __reduce__ due to non-trivial __cinit__")
 * def __setstate_cython__(self, __pyx_state):
 */

/* Python wrapper */
static PyObject *__pyx_pw_4grpc_7_cython_6cygrpc_16RequestCallEvent_3__reduce_cython__(PyObject *__pyx_v_self, CYTHON_UNUSED PyObject *unused); /*proto*/
static PyObject *__pyx_pw_4grpc_7_cython_6cygrpc_16RequestCallEvent_3__reduce_cython__(PyObject *__pyx_v_self, CYTHON_UNUSED PyObject *unused) {
  PyObject *__pyx_r = 0;
  __Pyx_RefNannyDeclarations
  __Pyx_RefNannySetupContext("__reduce_cython__ (wrapper)", 0);
  __pyx_r = __pyx_pf_4grpc_7_cython_6cygrpc_16RequestCallEvent_2__reduce_cython__(((struct __pyx_obj_4grpc_7_cython_6cygrpc_RequestCallEvent *)__pyx_v_self));

  /* function exit code */
  __Pyx_RefNannyFinishContext();
  return __pyx_r;
}

static PyObject *__pyx_pf_4grpc_7_cython_6cygrpc_16RequestCallEvent_2__reduce_cython__(CYTHON_UNUSED struct __pyx_obj_4grpc_7_cython_6cygrpc_RequestCallEvent *__pyx_v_self) {
  PyObject *__pyx_r = NULL;
  __Pyx_RefNannyDeclarations
  PyObject *__pyx_t_1 = NULL;
  __Pyx_RefNannySetupContext("__reduce_cython__", 0);

  /* "(tree fragment)":2
 * def __reduce_cython__(self):
 *     raise TypeError("no default __reduce__ due to non-trivial __cinit__")             # <<<<<<<<<<<<<<
 * def __setstate_cython__(self, __pyx_state):
 *     raise TypeError("no default __reduce__ due to non-trivial __cinit__")
 */
  __pyx_t_1 = __Pyx_PyObject_Call(__pyx_builtin_TypeError, __pyx_tuple__78, NULL); if (unlikely(!__pyx_t_1)) __PYX_ERR(7, 2, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_1);
  __Pyx_Raise(__pyx_t_1, 0, 0, 0);
  __Pyx_DECREF(__pyx_t_1); __pyx_t_1 = 0;
  __PYX_ERR(7, 2, __pyx_L1_error)

  /* "(tree fragment)":1
 * def __reduce_cython__(self):             # <<<<<<<<<<<<<<
 *     raise TypeError("no default __reduce__ due to non-trivial __cinit__")
 * def __setstate_cython__(self, __pyx_state):
 */

  /* function exit code */
  __pyx_L1_error:;
  __Pyx_XDECREF(__pyx_t_1);
  __Pyx_AddTraceback("grpc._cython.cygrpc.RequestCallEvent.__reduce_cython__", __pyx_clineno, __pyx_lineno, __pyx_filename);
  __pyx_r = NULL;
  __Pyx_XGIVEREF(__pyx_r);
  __Pyx_RefNannyFinishContext();
  return __pyx_r;
}

/* "(tree fragment)":3
 * def __reduce_cython__(self):
 *     raise TypeError("no default __reduce__ due to non-trivial __cinit__")
 * def __setstate_cython__(self, __pyx_state):             # <<<<<<<<<<<<<<
 *     raise TypeError("no default __reduce__ due to non-trivial __cinit__")
 */

/* Python wrapper */
static PyObject *__pyx_pw_4grpc_7_cython_6cygrpc_16RequestCallEvent_5__setstate_cython__(PyObject *__pyx_v_self, PyObject *__pyx_v___pyx_state); /*proto*/
static PyObject *__pyx_pw_4grpc_7_cython_6cygrpc_16RequestCallEvent_5__setstate_cython__(PyObject *__pyx_v_self, PyObject *__pyx_v___pyx_state) {
  PyObject *__pyx_r = 0;
  __Pyx_RefNannyDeclarations
  __Pyx_RefNannySetupContext("__setstate_cython__ (wrapper)", 0);
  __pyx_r = __pyx_pf_4grpc_7_cython_6cygrpc_16RequestCallEvent_4__setstate_cython__(((struct __pyx_obj_4grpc_7_cython_6cygrpc_RequestCallEvent *)__pyx_v_self), ((PyObject *)__pyx_v___pyx_state));

  /* function exit code */
  __Pyx_RefNannyFinishContext();
  return __pyx_r;
}

static PyObject *__pyx_pf_4grpc_7_cython_6cygrpc_16RequestCallEvent_4__setstate_cython__(CYTHON_UNUSED struct __pyx_obj_4grpc_7_cython_6cygrpc_RequestCallEvent *__pyx_v_self, CYTHON_UNUSED PyObject *__pyx_v___pyx_state) {
  PyObject *__pyx_r = NULL;
  __Pyx_RefNannyDeclarations
  PyObject *__pyx_t_1 = NULL;
  __Pyx_RefNannySetupContext("__setstate_cython__", 0);

  /* "(tree fragment)":4
 *     raise TypeError("no default __reduce__ due to non-trivial __cinit__")
 * def __setstate_cython__(self, __pyx_state):
 *     raise TypeError("no default __reduce__ due to non-trivial __cinit__")             # <<<<<<<<<<<<<<
 */
  __pyx_t_1 = __Pyx_PyObject_Call(__pyx_builtin_TypeError, __pyx_tuple__79, NULL); if (unlikely(!__pyx_t_1)) __PYX_ERR(7, 4, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_1);
  __Pyx_Raise(__pyx_t_1, 0, 0, 0);
  __Pyx_DECREF(__pyx_t_1); __pyx_t_1 = 0;
  __PYX_ERR(7, 4, __pyx_L1_error)

  /* "(tree fragment)":3
 * def __reduce_cython__(self):
 *     raise TypeError("no default __reduce__ due to non-trivial __cinit__")
 * def __setstate_cython__(self, __pyx_state):             # <<<<<<<<<<<<<<
 *     raise TypeError("no default __reduce__ due to non-trivial __cinit__")
 */

  /* function exit code */
  __pyx_L1_error:;
  __Pyx_XDECREF(__pyx_t_1);
  __Pyx_AddTraceback("grpc._cython.cygrpc.RequestCallEvent.__setstate_cython__", __pyx_clineno, __pyx_lineno, __pyx_filename);
  __pyx_r = NULL;
  __Pyx_XGIVEREF(__pyx_r);
  __Pyx_RefNannyFinishContext();
  return __pyx_r;
}

/* "src/python/grpcio/grpc/_cython/_cygrpc/event.pyx.pxi":40
 * cdef class BatchOperationEvent:
 * 
 *   def __cinit__(             # <<<<<<<<<<<<<<
 *       self, grpc_completion_type completion_type, bint success, object tag,
 *       object batch_operations):
 */

/* Python wrapper */
static int __pyx_pw_4grpc_7_cython_6cygrpc_19BatchOperationEvent_1__cinit__(PyObject *__pyx_v_self, PyObject *__pyx_args, PyObject *__pyx_kwds); /*proto*/
static int __pyx_pw_4grpc_7_cython_6cygrpc_19BatchOperationEvent_1__cinit__(PyObject *__pyx_v_self, PyObject *__pyx_args, PyObject *__pyx_kwds) {
  grpc_completion_type __pyx_v_completion_type;
  int __pyx_v_success;
  PyObject *__pyx_v_tag = 0;
  PyObject *__pyx_v_batch_operations = 0;
  int __pyx_r;
  __Pyx_RefNannyDeclarations
  __Pyx_RefNannySetupContext("__cinit__ (wrapper)", 0);
  {
    static PyObject **__pyx_pyargnames[] = {&__pyx_n_s_completion_type,&__pyx_n_s_success,&__pyx_n_s_tag,&__pyx_n_s_batch_operations,0};
    PyObject* values[4] = {0,0,0,0};
    if (unlikely(__pyx_kwds)) {
      Py_ssize_t kw_args;
      const Py_ssize_t pos_args = PyTuple_GET_SIZE(__pyx_args);
      switch (pos_args) {
        case  4: values[3] = PyTuple_GET_ITEM(__pyx_args, 3);
        CYTHON_FALLTHROUGH;
        case  3: values[2] = PyTuple_GET_ITEM(__pyx_args, 2);
        CYTHON_FALLTHROUGH;
        case  2: values[1] = PyTuple_GET_ITEM(__pyx_args, 1);
        CYTHON_FALLTHROUGH;
        case  1: values[0] = PyTuple_GET_ITEM(__pyx_args, 0);
        CYTHON_FALLTHROUGH;
        case  0: break;
        default: goto __pyx_L5_argtuple_error;
      }
      kw_args = PyDict_Size(__pyx_kwds);
      switch (pos_args) {
        case  0:
        if (likely((values[0] = __Pyx_PyDict_GetItemStr(__pyx_kwds, __pyx_n_s_completion_type)) != 0)) kw_args--;
        else goto __pyx_L5_argtuple_error;
        CYTHON_FALLTHROUGH;
        case  1:
        if (likely((values[1] = __Pyx_PyDict_GetItemStr(__pyx_kwds, __pyx_n_s_success)) != 0)) kw_args--;
        else {
          __Pyx_RaiseArgtupleInvalid("__cinit__", 1, 4, 4, 1); __PYX_ERR(11, 40, __pyx_L3_error)
        }
        CYTHON_FALLTHROUGH;
        case  2:
        if (likely((values[2] = __Pyx_PyDict_GetItemStr(__pyx_kwds, __pyx_n_s_tag)) != 0)) kw_args--;
        else {
          __Pyx_RaiseArgtupleInvalid("__cinit__", 1, 4, 4, 2); __PYX_ERR(11, 40, __pyx_L3_error)
        }
        CYTHON_FALLTHROUGH;
        case  3:
        if (likely((values[3] = __Pyx_PyDict_GetItemStr(__pyx_kwds, __pyx_n_s_batch_operations)) != 0)) kw_args--;
        else {
          __Pyx_RaiseArgtupleInvalid("__cinit__", 1, 4, 4, 3); __PYX_ERR(11, 40, __pyx_L3_error)
        }
      }
      if (unlikely(kw_args > 0)) {
        if (unlikely(__Pyx_ParseOptionalKeywords(__pyx_kwds, __pyx_pyargnames, 0, values, pos_args, "__cinit__") < 0)) __PYX_ERR(11, 40, __pyx_L3_error)
      }
    } else if (PyTuple_GET_SIZE(__pyx_args) != 4) {
      goto __pyx_L5_argtuple_error;
    } else {
      values[0] = PyTuple_GET_ITEM(__pyx_args, 0);
      values[1] = PyTuple_GET_ITEM(__pyx_args, 1);
      values[2] = PyTuple_GET_ITEM(__pyx_args, 2);
      values[3] = PyTuple_GET_ITEM(__pyx_args, 3);
    }
    __pyx_v_completion_type = ((grpc_completion_type)__Pyx_PyInt_As_grpc_completion_type(values[0])); if (unlikely(PyErr_Occurred())) __PYX_ERR(11, 41, __pyx_L3_error)
    __pyx_v_success = __Pyx_PyObject_IsTrue(values[1]); if (unlikely((__pyx_v_success == (int)-1) && PyErr_Occurred())) __PYX_ERR(11, 41, __pyx_L3_error)
    __pyx_v_tag = values[2];
    __pyx_v_batch_operations = values[3];
  }
  goto __pyx_L4_argument_unpacking_done;
  __pyx_L5_argtuple_error:;
  __Pyx_RaiseArgtupleInvalid("__cinit__", 1, 4, 4, PyTuple_GET_SIZE(__pyx_args)); __PYX_ERR(11, 40, __pyx_L3_error)
  __pyx_L3_error:;
  __Pyx_AddTraceback("grpc._cython.cygrpc.BatchOperationEvent.__cinit__", __pyx_clineno, __pyx_lineno, __pyx_filename);
  __Pyx_RefNannyFinishContext();
  return -1;
  __pyx_L4_argument_unpacking_done:;
  __pyx_r = __pyx_pf_4grpc_7_cython_6cygrpc_19BatchOperationEvent___cinit__(((struct __pyx_obj_4grpc_7_cython_6cygrpc_BatchOperationEvent *)__pyx_v_self), __pyx_v_completion_type, __pyx_v_success, __pyx_v_tag, __pyx_v_batch_operations);

  /* function exit code */
  __Pyx_RefNannyFinishContext();
  return __pyx_r;
}

static int __pyx_pf_4grpc_7_cython_6cygrpc_19BatchOperationEvent___cinit__(struct __pyx_obj_4grpc_7_cython_6cygrpc_BatchOperationEvent *__pyx_v_self, grpc_completion_type __pyx_v_completion_type, int __pyx_v_success, PyObject *__pyx_v_tag, PyObject *__pyx_v_batch_operations) {
  int __pyx_r;
  __Pyx_RefNannyDeclarations
  __Pyx_RefNannySetupContext("__cinit__", 0);

  /* "src/python/grpcio/grpc/_cython/_cygrpc/event.pyx.pxi":43
 *       self, grpc_completion_type completion_type, bint success, object tag,
 *       object batch_operations):
 *     self.completion_type = completion_type             # <<<<<<<<<<<<<<
 *     self.success = success
 *     self.tag = tag
 */
  __pyx_v_self->completion_type = __pyx_v_completion_type;

  /* "src/python/grpcio/grpc/_cython/_cygrpc/event.pyx.pxi":44
 *       object batch_operations):
 *     self.completion_type = completion_type
 *     self.success = success             # <<<<<<<<<<<<<<
 *     self.tag = tag
 *     self.batch_operations = batch_operations
 */
  __pyx_v_self->success = __pyx_v_success;

  /* "src/python/grpcio/grpc/_cython/_cygrpc/event.pyx.pxi":45
 *     self.completion_type = completion_type
 *     self.success = success
 *     self.tag = tag             # <<<<<<<<<<<<<<
 *     self.batch_operations = batch_operations
 * 
 */
  __Pyx_INCREF(__pyx_v_tag);
  __Pyx_GIVEREF(__pyx_v_tag);
  __Pyx_GOTREF(__pyx_v_self->tag);
  __Pyx_DECREF(__pyx_v_self->tag);
  __pyx_v_self->tag = __pyx_v_tag;

  /* "src/python/grpcio/grpc/_cython/_cygrpc/event.pyx.pxi":46
 *     self.success = success
 *     self.tag = tag
 *     self.batch_operations = batch_operations             # <<<<<<<<<<<<<<
 * 
 * 
 */
  __Pyx_INCREF(__pyx_v_batch_operations);
  __Pyx_GIVEREF(__pyx_v_batch_operations);
  __Pyx_GOTREF(__pyx_v_self->batch_operations);
  __Pyx_DECREF(__pyx_v_self->batch_operations);
  __pyx_v_self->batch_operations = __pyx_v_batch_operations;

  /* "src/python/grpcio/grpc/_cython/_cygrpc/event.pyx.pxi":40
 * cdef class BatchOperationEvent:
 * 
 *   def __cinit__(             # <<<<<<<<<<<<<<
 *       self, grpc_completion_type completion_type, bint success, object tag,
 *       object batch_operations):
 */

  /* function exit code */
  __pyx_r = 0;
  __Pyx_RefNannyFinishContext();
  return __pyx_r;
}

/* "src/python/grpcio/grpc/_cython/_cygrpc/event.pxd.pxi":35
 * cdef class BatchOperationEvent:
 * 
 *   cdef readonly grpc_completion_type completion_type             # <<<<<<<<<<<<<<
 *   cdef readonly bint success
 *   cdef readonly object tag
 */

/* Python wrapper */
static PyObject *__pyx_pw_4grpc_7_cython_6cygrpc_19BatchOperationEvent_15completion_type_1__get__(PyObject *__pyx_v_self); /*proto*/
static PyObject *__pyx_pw_4grpc_7_cython_6cygrpc_19BatchOperationEvent_15completion_type_1__get__(PyObject *__pyx_v_self) {
  PyObject *__pyx_r = 0;
  __Pyx_RefNannyDeclarations
  __Pyx_RefNannySetupContext("__get__ (wrapper)", 0);
  __pyx_r = __pyx_pf_4grpc_7_cython_6cygrpc_19BatchOperationEvent_15completion_type___get__(((struct __pyx_obj_4grpc_7_cython_6cygrpc_BatchOperationEvent *)__pyx_v_self));

  /* function exit code */
  __Pyx_RefNannyFinishContext();
  return __pyx_r;
}

static PyObject *__pyx_pf_4grpc_7_cython_6cygrpc_19BatchOperationEvent_15completion_type___get__(struct __pyx_obj_4grpc_7_cython_6cygrpc_BatchOperationEvent *__pyx_v_self) {
  PyObject *__pyx_r = NULL;
  __Pyx_RefNannyDeclarations
  PyObject *__pyx_t_1 = NULL;
  __Pyx_RefNannySetupContext("__get__", 0);
  __Pyx_XDECREF(__pyx_r);
  __pyx_t_1 = __Pyx_PyInt_From_grpc_completion_type(__pyx_v_self->completion_type); if (unlikely(!__pyx_t_1)) __PYX_ERR(12, 35, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_1);
  __pyx_r = __pyx_t_1;
  __pyx_t_1 = 0;
  goto __pyx_L0;

  /* function exit code */
  __pyx_L1_error:;
  __Pyx_XDECREF(__pyx_t_1);
  __Pyx_AddTraceback("grpc._cython.cygrpc.BatchOperationEvent.completion_type.__get__", __pyx_clineno, __pyx_lineno, __pyx_filename);
  __pyx_r = NULL;
  __pyx_L0:;
  __Pyx_XGIVEREF(__pyx_r);
  __Pyx_RefNannyFinishContext();
  return __pyx_r;
}

/* "src/python/grpcio/grpc/_cython/_cygrpc/event.pxd.pxi":36
 * 
 *   cdef readonly grpc_completion_type completion_type
 *   cdef readonly bint success             # <<<<<<<<<<<<<<
 *   cdef readonly object tag
 *   cdef readonly object batch_operations
 */

/* Python wrapper */
static PyObject *__pyx_pw_4grpc_7_cython_6cygrpc_19BatchOperationEvent_7success_1__get__(PyObject *__pyx_v_self); /*proto*/
static PyObject *__pyx_pw_4grpc_7_cython_6cygrpc_19BatchOperationEvent_7success_1__get__(PyObject *__pyx_v_self) {
  PyObject *__pyx_r = 0;
  __Pyx_RefNannyDeclarations
  __Pyx_RefNannySetupContext("__get__ (wrapper)", 0);
  __pyx_r = __pyx_pf_4grpc_7_cython_6cygrpc_19BatchOperationEvent_7success___get__(((struct __pyx_obj_4grpc_7_cython_6cygrpc_BatchOperationEvent *)__pyx_v_self));

  /* function exit code */
  __Pyx_RefNannyFinishContext();
  return __pyx_r;
}

static PyObject *__pyx_pf_4grpc_7_cython_6cygrpc_19BatchOperationEvent_7success___get__(struct __pyx_obj_4grpc_7_cython_6cygrpc_BatchOperationEvent *__pyx_v_self) {
  PyObject *__pyx_r = NULL;
  __Pyx_RefNannyDeclarations
  PyObject *__pyx_t_1 = NULL;
  __Pyx_RefNannySetupContext("__get__", 0);
  __Pyx_XDECREF(__pyx_r);
  __pyx_t_1 = __Pyx_PyBool_FromLong(__pyx_v_self->success); if (unlikely(!__pyx_t_1)) __PYX_ERR(12, 36, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_1);
  __pyx_r = __pyx_t_1;
  __pyx_t_1 = 0;
  goto __pyx_L0;

  /* function exit code */
  __pyx_L1_error:;
  __Pyx_XDECREF(__pyx_t_1);
  __Pyx_AddTraceback("grpc._cython.cygrpc.BatchOperationEvent.success.__get__", __pyx_clineno, __pyx_lineno, __pyx_filename);
  __pyx_r = NULL;
  __pyx_L0:;
  __Pyx_XGIVEREF(__pyx_r);
  __Pyx_RefNannyFinishContext();
  return __pyx_r;
}

/* "src/python/grpcio/grpc/_cython/_cygrpc/event.pxd.pxi":37
 *   cdef readonly grpc_completion_type completion_type
 *   cdef readonly bint success
 *   cdef readonly object tag             # <<<<<<<<<<<<<<
 *   cdef readonly object batch_operations
 * 
 */

/* Python wrapper */
static PyObject *__pyx_pw_4grpc_7_cython_6cygrpc_19BatchOperationEvent_3tag_1__get__(PyObject *__pyx_v_self); /*proto*/
static PyObject *__pyx_pw_4grpc_7_cython_6cygrpc_19BatchOperationEvent_3tag_1__get__(PyObject *__pyx_v_self) {
  PyObject *__pyx_r = 0;
  __Pyx_RefNannyDeclarations
  __Pyx_RefNannySetupContext("__get__ (wrapper)", 0);
  __pyx_r = __pyx_pf_4grpc_7_cython_6cygrpc_19BatchOperationEvent_3tag___get__(((struct __pyx_obj_4grpc_7_cython_6cygrpc_BatchOperationEvent *)__pyx_v_self));

  /* function exit code */
  __Pyx_RefNannyFinishContext();
  return __pyx_r;
}

static PyObject *__pyx_pf_4grpc_7_cython_6cygrpc_19BatchOperationEvent_3tag___get__(struct __pyx_obj_4grpc_7_cython_6cygrpc_BatchOperationEvent *__pyx_v_self) {
  PyObject *__pyx_r = NULL;
  __Pyx_RefNannyDeclarations
  __Pyx_RefNannySetupContext("__get__", 0);
  __Pyx_XDECREF(__pyx_r);
  __Pyx_INCREF(__pyx_v_self->tag);
  __pyx_r = __pyx_v_self->tag;
  goto __pyx_L0;

  /* function exit code */
  __pyx_L0:;
  __Pyx_XGIVEREF(__pyx_r);
  __Pyx_RefNannyFinishContext();
  return __pyx_r;
}

/* "src/python/grpcio/grpc/_cython/_cygrpc/event.pxd.pxi":38
 *   cdef readonly bint success
 *   cdef readonly object tag
 *   cdef readonly object batch_operations             # <<<<<<<<<<<<<<
 * 
 * 
 */

/* Python wrapper */
static PyObject *__pyx_pw_4grpc_7_cython_6cygrpc_19BatchOperationEvent_16batch_operations_1__get__(PyObject *__pyx_v_self); /*proto*/
static PyObject *__pyx_pw_4grpc_7_cython_6cygrpc_19BatchOperationEvent_16batch_operations_1__get__(PyObject *__pyx_v_self) {
  PyObject *__pyx_r = 0;
  __Pyx_RefNannyDeclarations
  __Pyx_RefNannySetupContext("__get__ (wrapper)", 0);
  __pyx_r = __pyx_pf_4grpc_7_cython_6cygrpc_19BatchOperationEvent_16batch_operations___get__(((struct __pyx_obj_4grpc_7_cython_6cygrpc_BatchOperationEvent *)__pyx_v_self));

  /* function exit code */
  __Pyx_RefNannyFinishContext();
  return __pyx_r;
}

static PyObject *__pyx_pf_4grpc_7_cython_6cygrpc_19BatchOperationEvent_16batch_operations___get__(struct __pyx_obj_4grpc_7_cython_6cygrpc_BatchOperationEvent *__pyx_v_self) {
  PyObject *__pyx_r = NULL;
  __Pyx_RefNannyDeclarations
  __Pyx_RefNannySetupContext("__get__", 0);
  __Pyx_XDECREF(__pyx_r);
  __Pyx_INCREF(__pyx_v_self->batch_operations);
  __pyx_r = __pyx_v_self->batch_operations;
  goto __pyx_L0;

  /* function exit code */
  __pyx_L0:;
  __Pyx_XGIVEREF(__pyx_r);
  __Pyx_RefNannyFinishContext();
  return __pyx_r;
}

/* "(tree fragment)":1
 * def __reduce_cython__(self):             # <<<<<<<<<<<<<<
 *     raise TypeError("no default __reduce__ due to non-trivial __cinit__")
 * def __setstate_cython__(self, __pyx_state):
 */

/* Python wrapper */
static PyObject *__pyx_pw_4grpc_7_cython_6cygrpc_19BatchOperationEvent_3__reduce_cython__(PyObject *__pyx_v_self, CYTHON_UNUSED PyObject *unused); /*proto*/
static PyObject *__pyx_pw_4grpc_7_cython_6cygrpc_19BatchOperationEvent_3__reduce_cython__(PyObject *__pyx_v_self, CYTHON_UNUSED PyObject *unused) {
  PyObject *__pyx_r = 0;
  __Pyx_RefNannyDeclarations
  __Pyx_RefNannySetupContext("__reduce_cython__ (wrapper)", 0);
  __pyx_r = __pyx_pf_4grpc_7_cython_6cygrpc_19BatchOperationEvent_2__reduce_cython__(((struct __pyx_obj_4grpc_7_cython_6cygrpc_BatchOperationEvent *)__pyx_v_self));

  /* function exit code */
  __Pyx_RefNannyFinishContext();
  return __pyx_r;
}

static PyObject *__pyx_pf_4grpc_7_cython_6cygrpc_19BatchOperationEvent_2__reduce_cython__(CYTHON_UNUSED struct __pyx_obj_4grpc_7_cython_6cygrpc_BatchOperationEvent *__pyx_v_self) {
  PyObject *__pyx_r = NULL;
  __Pyx_RefNannyDeclarations
  PyObject *__pyx_t_1 = NULL;
  __Pyx_RefNannySetupContext("__reduce_cython__", 0);

  /* "(tree fragment)":2
 * def __reduce_cython__(self):
 *     raise TypeError("no default __reduce__ due to non-trivial __cinit__")             # <<<<<<<<<<<<<<
 * def __setstate_cython__(self, __pyx_state):
 *     raise TypeError("no default __reduce__ due to non-trivial __cinit__")
 */
  __pyx_t_1 = __Pyx_PyObject_Call(__pyx_builtin_TypeError, __pyx_tuple__80, NULL); if (unlikely(!__pyx_t_1)) __PYX_ERR(7, 2, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_1);
  __Pyx_Raise(__pyx_t_1, 0, 0, 0);
  __Pyx_DECREF(__pyx_t_1); __pyx_t_1 = 0;
  __PYX_ERR(7, 2, __pyx_L1_error)

  /* "(tree fragment)":1
 * def __reduce_cython__(self):             # <<<<<<<<<<<<<<
 *     raise TypeError("no default __reduce__ due to non-trivial __cinit__")
 * def __setstate_cython__(self, __pyx_state):
 */

  /* function exit code */
  __pyx_L1_error:;
  __Pyx_XDECREF(__pyx_t_1);
  __Pyx_AddTraceback("grpc._cython.cygrpc.BatchOperationEvent.__reduce_cython__", __pyx_clineno, __pyx_lineno, __pyx_filename);
  __pyx_r = NULL;
  __Pyx_XGIVEREF(__pyx_r);
  __Pyx_RefNannyFinishContext();
  return __pyx_r;
}

/* "(tree fragment)":3
 * def __reduce_cython__(self):
 *     raise TypeError("no default __reduce__ due to non-trivial __cinit__")
 * def __setstate_cython__(self, __pyx_state):             # <<<<<<<<<<<<<<
 *     raise TypeError("no default __reduce__ due to non-trivial __cinit__")
 */

/* Python wrapper */
static PyObject *__pyx_pw_4grpc_7_cython_6cygrpc_19BatchOperationEvent_5__setstate_cython__(PyObject *__pyx_v_self, PyObject *__pyx_v___pyx_state); /*proto*/
static PyObject *__pyx_pw_4grpc_7_cython_6cygrpc_19BatchOperationEvent_5__setstate_cython__(PyObject *__pyx_v_self, PyObject *__pyx_v___pyx_state) {
  PyObject *__pyx_r = 0;
  __Pyx_RefNannyDeclarations
  __Pyx_RefNannySetupContext("__setstate_cython__ (wrapper)", 0);
  __pyx_r = __pyx_pf_4grpc_7_cython_6cygrpc_19BatchOperationEvent_4__setstate_cython__(((struct __pyx_obj_4grpc_7_cython_6cygrpc_BatchOperationEvent *)__pyx_v_self), ((PyObject *)__pyx_v___pyx_state));

  /* function exit code */
  __Pyx_RefNannyFinishContext();
  return __pyx_r;
}

static PyObject *__pyx_pf_4grpc_7_cython_6cygrpc_19BatchOperationEvent_4__setstate_cython__(CYTHON_UNUSED struct __pyx_obj_4grpc_7_cython_6cygrpc_BatchOperationEvent *__pyx_v_self, CYTHON_UNUSED PyObject *__pyx_v___pyx_state) {
  PyObject *__pyx_r = NULL;
  __Pyx_RefNannyDeclarations
  PyObject *__pyx_t_1 = NULL;
  __Pyx_RefNannySetupContext("__setstate_cython__", 0);

  /* "(tree fragment)":4
 *     raise TypeError("no default __reduce__ due to non-trivial __cinit__")
 * def __setstate_cython__(self, __pyx_state):
 *     raise TypeError("no default __reduce__ due to non-trivial __cinit__")             # <<<<<<<<<<<<<<
 */
  __pyx_t_1 = __Pyx_PyObject_Call(__pyx_builtin_TypeError, __pyx_tuple__81, NULL); if (unlikely(!__pyx_t_1)) __PYX_ERR(7, 4, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_1);
  __Pyx_Raise(__pyx_t_1, 0, 0, 0);
  __Pyx_DECREF(__pyx_t_1); __pyx_t_1 = 0;
  __PYX_ERR(7, 4, __pyx_L1_error)

  /* "(tree fragment)":3
 * def __reduce_cython__(self):
 *     raise TypeError("no default __reduce__ due to non-trivial __cinit__")
 * def __setstate_cython__(self, __pyx_state):             # <<<<<<<<<<<<<<
 *     raise TypeError("no default __reduce__ due to non-trivial __cinit__")
 */

  /* function exit code */
  __pyx_L1_error:;
  __Pyx_XDECREF(__pyx_t_1);
  __Pyx_AddTraceback("grpc._cython.cygrpc.BatchOperationEvent.__setstate_cython__", __pyx_clineno, __pyx_lineno, __pyx_filename);
  __pyx_r = NULL;
  __Pyx_XGIVEREF(__pyx_r);
  __Pyx_RefNannyFinishContext();
  return __pyx_r;
}

/* "src/python/grpcio/grpc/_cython/_cygrpc/event.pyx.pxi":51
 * cdef class ServerShutdownEvent:
 * 
 *   def __cinit__(             # <<<<<<<<<<<<<<
 *       self, grpc_completion_type completion_type, bint success, object tag):
 *     self.completion_type = completion_type
 */

/* Python wrapper */
static int __pyx_pw_4grpc_7_cython_6cygrpc_19ServerShutdownEvent_1__cinit__(PyObject *__pyx_v_self, PyObject *__pyx_args, PyObject *__pyx_kwds); /*proto*/
static int __pyx_pw_4grpc_7_cython_6cygrpc_19ServerShutdownEvent_1__cinit__(PyObject *__pyx_v_self, PyObject *__pyx_args, PyObject *__pyx_kwds) {
  grpc_completion_type __pyx_v_completion_type;
  int __pyx_v_success;
  PyObject *__pyx_v_tag = 0;
  int __pyx_r;
  __Pyx_RefNannyDeclarations
  __Pyx_RefNannySetupContext("__cinit__ (wrapper)", 0);
  {
    static PyObject **__pyx_pyargnames[] = {&__pyx_n_s_completion_type,&__pyx_n_s_success,&__pyx_n_s_tag,0};
    PyObject* values[3] = {0,0,0};
    if (unlikely(__pyx_kwds)) {
      Py_ssize_t kw_args;
      const Py_ssize_t pos_args = PyTuple_GET_SIZE(__pyx_args);
      switch (pos_args) {
        case  3: values[2] = PyTuple_GET_ITEM(__pyx_args, 2);
        CYTHON_FALLTHROUGH;
        case  2: values[1] = PyTuple_GET_ITEM(__pyx_args, 1);
        CYTHON_FALLTHROUGH;
        case  1: values[0] = PyTuple_GET_ITEM(__pyx_args, 0);
        CYTHON_FALLTHROUGH;
        case  0: break;
        default: goto __pyx_L5_argtuple_error;
      }
      kw_args = PyDict_Size(__pyx_kwds);
      switch (pos_args) {
        case  0:
        if (likely((values[0] = __Pyx_PyDict_GetItemStr(__pyx_kwds, __pyx_n_s_completion_type)) != 0)) kw_args--;
        else goto __pyx_L5_argtuple_error;
        CYTHON_FALLTHROUGH;
        case  1:
        if (likely((values[1] = __Pyx_PyDict_GetItemStr(__pyx_kwds, __pyx_n_s_success)) != 0)) kw_args--;
        else {
          __Pyx_RaiseArgtupleInvalid("__cinit__", 1, 3, 3, 1); __PYX_ERR(11, 51, __pyx_L3_error)
        }
        CYTHON_FALLTHROUGH;
        case  2:
        if (likely((values[2] = __Pyx_PyDict_GetItemStr(__pyx_kwds, __pyx_n_s_tag)) != 0)) kw_args--;
        else {
          __Pyx_RaiseArgtupleInvalid("__cinit__", 1, 3, 3, 2); __PYX_ERR(11, 51, __pyx_L3_error)
        }
      }
      if (unlikely(kw_args > 0)) {
        if (unlikely(__Pyx_ParseOptionalKeywords(__pyx_kwds, __pyx_pyargnames, 0, values, pos_args, "__cinit__") < 0)) __PYX_ERR(11, 51, __pyx_L3_error)
      }
    } else if (PyTuple_GET_SIZE(__pyx_args) != 3) {
      goto __pyx_L5_argtuple_error;
    } else {
      values[0] = PyTuple_GET_ITEM(__pyx_args, 0);
      values[1] = PyTuple_GET_ITEM(__pyx_args, 1);
      values[2] = PyTuple_GET_ITEM(__pyx_args, 2);
    }
    __pyx_v_completion_type = ((grpc_completion_type)__Pyx_PyInt_As_grpc_completion_type(values[0])); if (unlikely(PyErr_Occurred())) __PYX_ERR(11, 52, __pyx_L3_error)
    __pyx_v_success = __Pyx_PyObject_IsTrue(values[1]); if (unlikely((__pyx_v_success == (int)-1) && PyErr_Occurred())) __PYX_ERR(11, 52, __pyx_L3_error)
    __pyx_v_tag = values[2];
  }
  goto __pyx_L4_argument_unpacking_done;
  __pyx_L5_argtuple_error:;
  __Pyx_RaiseArgtupleInvalid("__cinit__", 1, 3, 3, PyTuple_GET_SIZE(__pyx_args)); __PYX_ERR(11, 51, __pyx_L3_error)
  __pyx_L3_error:;
  __Pyx_AddTraceback("grpc._cython.cygrpc.ServerShutdownEvent.__cinit__", __pyx_clineno, __pyx_lineno, __pyx_filename);
  __Pyx_RefNannyFinishContext();
  return -1;
  __pyx_L4_argument_unpacking_done:;
  __pyx_r = __pyx_pf_4grpc_7_cython_6cygrpc_19ServerShutdownEvent___cinit__(((struct __pyx_obj_4grpc_7_cython_6cygrpc_ServerShutdownEvent *)__pyx_v_self), __pyx_v_completion_type, __pyx_v_success, __pyx_v_tag);

  /* function exit code */
  __Pyx_RefNannyFinishContext();
  return __pyx_r;
}

static int __pyx_pf_4grpc_7_cython_6cygrpc_19ServerShutdownEvent___cinit__(struct __pyx_obj_4grpc_7_cython_6cygrpc_ServerShutdownEvent *__pyx_v_self, grpc_completion_type __pyx_v_completion_type, int __pyx_v_success, PyObject *__pyx_v_tag) {
  int __pyx_r;
  __Pyx_RefNannyDeclarations
  __Pyx_RefNannySetupContext("__cinit__", 0);

  /* "src/python/grpcio/grpc/_cython/_cygrpc/event.pyx.pxi":53
 *   def __cinit__(
 *       self, grpc_completion_type completion_type, bint success, object tag):
 *     self.completion_type = completion_type             # <<<<<<<<<<<<<<
 *     self.success = success
 *     self.tag = tag
 */
  __pyx_v_self->completion_type = __pyx_v_completion_type;

  /* "src/python/grpcio/grpc/_cython/_cygrpc/event.pyx.pxi":54
 *       self, grpc_completion_type completion_type, bint success, object tag):
 *     self.completion_type = completion_type
 *     self.success = success             # <<<<<<<<<<<<<<
 *     self.tag = tag
 */
  __pyx_v_self->success = __pyx_v_success;

  /* "src/python/grpcio/grpc/_cython/_cygrpc/event.pyx.pxi":55
 *     self.completion_type = completion_type
 *     self.success = success
 *     self.tag = tag             # <<<<<<<<<<<<<<
 */
  __Pyx_INCREF(__pyx_v_tag);
  __Pyx_GIVEREF(__pyx_v_tag);
  __Pyx_GOTREF(__pyx_v_self->tag);
  __Pyx_DECREF(__pyx_v_self->tag);
  __pyx_v_self->tag = __pyx_v_tag;

  /* "src/python/grpcio/grpc/_cython/_cygrpc/event.pyx.pxi":51
 * cdef class ServerShutdownEvent:
 * 
 *   def __cinit__(             # <<<<<<<<<<<<<<
 *       self, grpc_completion_type completion_type, bint success, object tag):
 *     self.completion_type = completion_type
 */

  /* function exit code */
  __pyx_r = 0;
  __Pyx_RefNannyFinishContext();
  return __pyx_r;
}

/* "src/python/grpcio/grpc/_cython/_cygrpc/event.pxd.pxi":43
 * cdef class ServerShutdownEvent:
 * 
 *   cdef readonly grpc_completion_type completion_type             # <<<<<<<<<<<<<<
 *   cdef readonly bint success
 *   cdef readonly object tag
 */

/* Python wrapper */
static PyObject *__pyx_pw_4grpc_7_cython_6cygrpc_19ServerShutdownEvent_15completion_type_1__get__(PyObject *__pyx_v_self); /*proto*/
static PyObject *__pyx_pw_4grpc_7_cython_6cygrpc_19ServerShutdownEvent_15completion_type_1__get__(PyObject *__pyx_v_self) {
  PyObject *__pyx_r = 0;
  __Pyx_RefNannyDeclarations
  __Pyx_RefNannySetupContext("__get__ (wrapper)", 0);
  __pyx_r = __pyx_pf_4grpc_7_cython_6cygrpc_19ServerShutdownEvent_15completion_type___get__(((struct __pyx_obj_4grpc_7_cython_6cygrpc_ServerShutdownEvent *)__pyx_v_self));

  /* function exit code */
  __Pyx_RefNannyFinishContext();
  return __pyx_r;
}

static PyObject *__pyx_pf_4grpc_7_cython_6cygrpc_19ServerShutdownEvent_15completion_type___get__(struct __pyx_obj_4grpc_7_cython_6cygrpc_ServerShutdownEvent *__pyx_v_self) {
  PyObject *__pyx_r = NULL;
  __Pyx_RefNannyDeclarations
  PyObject *__pyx_t_1 = NULL;
  __Pyx_RefNannySetupContext("__get__", 0);
  __Pyx_XDECREF(__pyx_r);
  __pyx_t_1 = __Pyx_PyInt_From_grpc_completion_type(__pyx_v_self->completion_type); if (unlikely(!__pyx_t_1)) __PYX_ERR(12, 43, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_1);
  __pyx_r = __pyx_t_1;
  __pyx_t_1 = 0;
  goto __pyx_L0;

  /* function exit code */
  __pyx_L1_error:;
  __Pyx_XDECREF(__pyx_t_1);
  __Pyx_AddTraceback("grpc._cython.cygrpc.ServerShutdownEvent.completion_type.__get__", __pyx_clineno, __pyx_lineno, __pyx_filename);
  __pyx_r = NULL;
  __pyx_L0:;
  __Pyx_XGIVEREF(__pyx_r);
  __Pyx_RefNannyFinishContext();
  return __pyx_r;
}

/* "src/python/grpcio/grpc/_cython/_cygrpc/event.pxd.pxi":44
 * 
 *   cdef readonly grpc_completion_type completion_type
 *   cdef readonly bint success             # <<<<<<<<<<<<<<
 *   cdef readonly object tag
 */

/* Python wrapper */
static PyObject *__pyx_pw_4grpc_7_cython_6cygrpc_19ServerShutdownEvent_7success_1__get__(PyObject *__pyx_v_self); /*proto*/
static PyObject *__pyx_pw_4grpc_7_cython_6cygrpc_19ServerShutdownEvent_7success_1__get__(PyObject *__pyx_v_self) {
  PyObject *__pyx_r = 0;
  __Pyx_RefNannyDeclarations
  __Pyx_RefNannySetupContext("__get__ (wrapper)", 0);
  __pyx_r = __pyx_pf_4grpc_7_cython_6cygrpc_19ServerShutdownEvent_7success___get__(((struct __pyx_obj_4grpc_7_cython_6cygrpc_ServerShutdownEvent *)__pyx_v_self));

  /* function exit code */
  __Pyx_RefNannyFinishContext();
  return __pyx_r;
}

static PyObject *__pyx_pf_4grpc_7_cython_6cygrpc_19ServerShutdownEvent_7success___get__(struct __pyx_obj_4grpc_7_cython_6cygrpc_ServerShutdownEvent *__pyx_v_self) {
  PyObject *__pyx_r = NULL;
  __Pyx_RefNannyDeclarations
  PyObject *__pyx_t_1 = NULL;
  __Pyx_RefNannySetupContext("__get__", 0);
  __Pyx_XDECREF(__pyx_r);
  __pyx_t_1 = __Pyx_PyBool_FromLong(__pyx_v_self->success); if (unlikely(!__pyx_t_1)) __PYX_ERR(12, 44, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_1);
  __pyx_r = __pyx_t_1;
  __pyx_t_1 = 0;
  goto __pyx_L0;

  /* function exit code */
  __pyx_L1_error:;
  __Pyx_XDECREF(__pyx_t_1);
  __Pyx_AddTraceback("grpc._cython.cygrpc.ServerShutdownEvent.success.__get__", __pyx_clineno, __pyx_lineno, __pyx_filename);
  __pyx_r = NULL;
  __pyx_L0:;
  __Pyx_XGIVEREF(__pyx_r);
  __Pyx_RefNannyFinishContext();
  return __pyx_r;
}

/* "src/python/grpcio/grpc/_cython/_cygrpc/event.pxd.pxi":45
 *   cdef readonly grpc_completion_type completion_type
 *   cdef readonly bint success
 *   cdef readonly object tag             # <<<<<<<<<<<<<<
 */

/* Python wrapper */
static PyObject *__pyx_pw_4grpc_7_cython_6cygrpc_19ServerShutdownEvent_3tag_1__get__(PyObject *__pyx_v_self); /*proto*/
static PyObject *__pyx_pw_4grpc_7_cython_6cygrpc_19ServerShutdownEvent_3tag_1__get__(PyObject *__pyx_v_self) {
  PyObject *__pyx_r = 0;
  __Pyx_RefNannyDeclarations
  __Pyx_RefNannySetupContext("__get__ (wrapper)", 0);
  __pyx_r = __pyx_pf_4grpc_7_cython_6cygrpc_19ServerShutdownEvent_3tag___get__(((struct __pyx_obj_4grpc_7_cython_6cygrpc_ServerShutdownEvent *)__pyx_v_self));

  /* function exit code */
  __Pyx_RefNannyFinishContext();
  return __pyx_r;
}

static PyObject *__pyx_pf_4grpc_7_cython_6cygrpc_19ServerShutdownEvent_3tag___get__(struct __pyx_obj_4grpc_7_cython_6cygrpc_ServerShutdownEvent *__pyx_v_self) {
  PyObject *__pyx_r = NULL;
  __Pyx_RefNannyDeclarations
  __Pyx_RefNannySetupContext("__get__", 0);
  __Pyx_XDECREF(__pyx_r);
  __Pyx_INCREF(__pyx_v_self->tag);
  __pyx_r = __pyx_v_self->tag;
  goto __pyx_L0;

  /* function exit code */
  __pyx_L0:;
  __Pyx_XGIVEREF(__pyx_r);
  __Pyx_RefNannyFinishContext();
  return __pyx_r;
}

/* "(tree fragment)":1
 * def __reduce_cython__(self):             # <<<<<<<<<<<<<<
 *     raise TypeError("no default __reduce__ due to non-trivial __cinit__")
 * def __setstate_cython__(self, __pyx_state):
 */

/* Python wrapper */
static PyObject *__pyx_pw_4grpc_7_cython_6cygrpc_19ServerShutdownEvent_3__reduce_cython__(PyObject *__pyx_v_self, CYTHON_UNUSED PyObject *unused); /*proto*/
static PyObject *__pyx_pw_4grpc_7_cython_6cygrpc_19ServerShutdownEvent_3__reduce_cython__(PyObject *__pyx_v_self, CYTHON_UNUSED PyObject *unused) {
  PyObject *__pyx_r = 0;
  __Pyx_RefNannyDeclarations
  __Pyx_RefNannySetupContext("__reduce_cython__ (wrapper)", 0);
  __pyx_r = __pyx_pf_4grpc_7_cython_6cygrpc_19ServerShutdownEvent_2__reduce_cython__(((struct __pyx_obj_4grpc_7_cython_6cygrpc_ServerShutdownEvent *)__pyx_v_self));

  /* function exit code */
  __Pyx_RefNannyFinishContext();
  return __pyx_r;
}

static PyObject *__pyx_pf_4grpc_7_cython_6cygrpc_19ServerShutdownEvent_2__reduce_cython__(CYTHON_UNUSED struct __pyx_obj_4grpc_7_cython_6cygrpc_ServerShutdownEvent *__pyx_v_self) {
  PyObject *__pyx_r = NULL;
  __Pyx_RefNannyDeclarations
  PyObject *__pyx_t_1 = NULL;
  __Pyx_RefNannySetupContext("__reduce_cython__", 0);

  /* "(tree fragment)":2
 * def __reduce_cython__(self):
 *     raise TypeError("no default __reduce__ due to non-trivial __cinit__")             # <<<<<<<<<<<<<<
 * def __setstate_cython__(self, __pyx_state):
 *     raise TypeError("no default __reduce__ due to non-trivial __cinit__")
 */
  __pyx_t_1 = __Pyx_PyObject_Call(__pyx_builtin_TypeError, __pyx_tuple__82, NULL); if (unlikely(!__pyx_t_1)) __PYX_ERR(7, 2, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_1);
  __Pyx_Raise(__pyx_t_1, 0, 0, 0);
  __Pyx_DECREF(__pyx_t_1); __pyx_t_1 = 0;
  __PYX_ERR(7, 2, __pyx_L1_error)

  /* "(tree fragment)":1
 * def __reduce_cython__(self):             # <<<<<<<<<<<<<<
 *     raise TypeError("no default __reduce__ due to non-trivial __cinit__")
 * def __setstate_cython__(self, __pyx_state):
 */

  /* function exit code */
  __pyx_L1_error:;
  __Pyx_XDECREF(__pyx_t_1);
  __Pyx_AddTraceback("grpc._cython.cygrpc.ServerShutdownEvent.__reduce_cython__", __pyx_clineno, __pyx_lineno, __pyx_filename);
  __pyx_r = NULL;
  __Pyx_XGIVEREF(__pyx_r);
  __Pyx_RefNannyFinishContext();
  return __pyx_r;
}

/* "(tree fragment)":3
 * def __reduce_cython__(self):
 *     raise TypeError("no default __reduce__ due to non-trivial __cinit__")
 * def __setstate_cython__(self, __pyx_state):             # <<<<<<<<<<<<<<
 *     raise TypeError("no default __reduce__ due to non-trivial __cinit__")
 */

/* Python wrapper */
static PyObject *__pyx_pw_4grpc_7_cython_6cygrpc_19ServerShutdownEvent_5__setstate_cython__(PyObject *__pyx_v_self, PyObject *__pyx_v___pyx_state); /*proto*/
static PyObject *__pyx_pw_4grpc_7_cython_6cygrpc_19ServerShutdownEvent_5__setstate_cython__(PyObject *__pyx_v_self, PyObject *__pyx_v___pyx_state) {
  PyObject *__pyx_r = 0;
  __Pyx_RefNannyDeclarations
  __Pyx_RefNannySetupContext("__setstate_cython__ (wrapper)", 0);
  __pyx_r = __pyx_pf_4grpc_7_cython_6cygrpc_19ServerShutdownEvent_4__setstate_cython__(((struct __pyx_obj_4grpc_7_cython_6cygrpc_ServerShutdownEvent *)__pyx_v_self), ((PyObject *)__pyx_v___pyx_state));

  /* function exit code */
  __Pyx_RefNannyFinishContext();
  return __pyx_r;
}

static PyObject *__pyx_pf_4grpc_7_cython_6cygrpc_19ServerShutdownEvent_4__setstate_cython__(CYTHON_UNUSED struct __pyx_obj_4grpc_7_cython_6cygrpc_ServerShutdownEvent *__pyx_v_self, CYTHON_UNUSED PyObject *__pyx_v___pyx_state) {
  PyObject *__pyx_r = NULL;
  __Pyx_RefNannyDeclarations
  PyObject *__pyx_t_1 = NULL;
  __Pyx_RefNannySetupContext("__setstate_cython__", 0);

  /* "(tree fragment)":4
 *     raise TypeError("no default __reduce__ due to non-trivial __cinit__")
 * def __setstate_cython__(self, __pyx_state):
 *     raise TypeError("no default __reduce__ due to non-trivial __cinit__")             # <<<<<<<<<<<<<<
 */
  __pyx_t_1 = __Pyx_PyObject_Call(__pyx_builtin_TypeError, __pyx_tuple__83, NULL); if (unlikely(!__pyx_t_1)) __PYX_ERR(7, 4, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_1);
  __Pyx_Raise(__pyx_t_1, 0, 0, 0);
  __Pyx_DECREF(__pyx_t_1); __pyx_t_1 = 0;
  __PYX_ERR(7, 4, __pyx_L1_error)

  /* "(tree fragment)":3
 * def __reduce_cython__(self):
 *     raise TypeError("no default __reduce__ due to non-trivial __cinit__")
 * def __setstate_cython__(self, __pyx_state):             # <<<<<<<<<<<<<<
 *     raise TypeError("no default __reduce__ due to non-trivial __cinit__")
 */

  /* function exit code */
  __pyx_L1_error:;
  __Pyx_XDECREF(__pyx_t_1);
  __Pyx_AddTraceback("grpc._cython.cygrpc.ServerShutdownEvent.__setstate_cython__", __pyx_clineno, __pyx_lineno, __pyx_filename);
  __pyx_r = NULL;
  __Pyx_XGIVEREF(__pyx_r);
  __Pyx_RefNannyFinishContext();
  return __pyx_r;
}

/* "src/python/grpcio/grpc/_cython/_cygrpc/metadata.pyx.pxi":27
 * 
 * 
 * cdef void _store_c_metadata(             # <<<<<<<<<<<<<<
 *     metadata, grpc_metadata **c_metadata, size_t *c_count) except *:
 *   if metadata is None:
 */

static void __pyx_f_4grpc_7_cython_6cygrpc__store_c_metadata(PyObject *__pyx_v_metadata, grpc_metadata **__pyx_v_c_metadata, size_t *__pyx_v_c_count) {
  PyObject *__pyx_v_metadatum_count = NULL;
  PyObject *__pyx_v_index = NULL;
  PyObject *__pyx_v_key = NULL;
  PyObject *__pyx_v_value = NULL;
  PyObject *__pyx_v_encoded_key = NULL;
  PyObject *__pyx_v_encoded_value = NULL;
  __Pyx_RefNannyDeclarations
  int __pyx_t_1;
  int __pyx_t_2;
  Py_ssize_t __pyx_t_3;
  PyObject *__pyx_t_4 = NULL;
  size_t __pyx_t_5;
  PyObject *__pyx_t_6 = NULL;
  PyObject *(*__pyx_t_7)(PyObject *);
  PyObject *__pyx_t_8 = NULL;
  PyObject *__pyx_t_9 = NULL;
  PyObject *__pyx_t_10 = NULL;
  PyObject *__pyx_t_11 = NULL;
  PyObject *(*__pyx_t_12)(PyObject *);
  Py_ssize_t __pyx_t_13;
  __Pyx_RefNannySetupContext("_store_c_metadata", 0);

  /* "src/python/grpcio/grpc/_cython/_cygrpc/metadata.pyx.pxi":29
 * cdef void _store_c_metadata(
 *     metadata, grpc_metadata **c_metadata, size_t *c_count) except *:
 *   if metadata is None:             # <<<<<<<<<<<<<<
 *     c_count[0] = 0
 *     c_metadata[0] = NULL
 */
  __pyx_t_1 = (__pyx_v_metadata == Py_None);
  __pyx_t_2 = (__pyx_t_1 != 0);
  if (__pyx_t_2) {

    /* "src/python/grpcio/grpc/_cython/_cygrpc/metadata.pyx.pxi":30
 *     metadata, grpc_metadata **c_metadata, size_t *c_count) except *:
 *   if metadata is None:
 *     c_count[0] = 0             # <<<<<<<<<<<<<<
 *     c_metadata[0] = NULL
 *   else:
 */
    (__pyx_v_c_count[0]) = 0;

    /* "src/python/grpcio/grpc/_cython/_cygrpc/metadata.pyx.pxi":31
 *   if metadata is None:
 *     c_count[0] = 0
 *     c_metadata[0] = NULL             # <<<<<<<<<<<<<<
 *   else:
 *     metadatum_count = len(metadata)
 */
    (__pyx_v_c_metadata[0]) = NULL;

    /* "src/python/grpcio/grpc/_cython/_cygrpc/metadata.pyx.pxi":29
 * cdef void _store_c_metadata(
 *     metadata, grpc_metadata **c_metadata, size_t *c_count) except *:
 *   if metadata is None:             # <<<<<<<<<<<<<<
 *     c_count[0] = 0
 *     c_metadata[0] = NULL
 */
    goto __pyx_L3;
  }

  /* "src/python/grpcio/grpc/_cython/_cygrpc/metadata.pyx.pxi":33
 *     c_metadata[0] = NULL
 *   else:
 *     metadatum_count = len(metadata)             # <<<<<<<<<<<<<<
 *     if metadatum_count == 0:
 *       c_count[0] = 0
 */
  /*else*/ {
    __pyx_t_3 = PyObject_Length(__pyx_v_metadata); if (unlikely(__pyx_t_3 == ((Py_ssize_t)-1))) __PYX_ERR(13, 33, __pyx_L1_error)
    __pyx_t_4 = PyInt_FromSsize_t(__pyx_t_3); if (unlikely(!__pyx_t_4)) __PYX_ERR(13, 33, __pyx_L1_error)
    __Pyx_GOTREF(__pyx_t_4);
    __pyx_v_metadatum_count = __pyx_t_4;
    __pyx_t_4 = 0;

    /* "src/python/grpcio/grpc/_cython/_cygrpc/metadata.pyx.pxi":34
 *   else:
 *     metadatum_count = len(metadata)
 *     if metadatum_count == 0:             # <<<<<<<<<<<<<<
 *       c_count[0] = 0
 *       c_metadata[0] = NULL
 */
    __pyx_t_4 = __Pyx_PyInt_EqObjC(__pyx_v_metadatum_count, __pyx_int_0, 0, 0); if (unlikely(!__pyx_t_4)) __PYX_ERR(13, 34, __pyx_L1_error)
    __Pyx_GOTREF(__pyx_t_4);
    __pyx_t_2 = __Pyx_PyObject_IsTrue(__pyx_t_4); if (unlikely(__pyx_t_2 < 0)) __PYX_ERR(13, 34, __pyx_L1_error)
    __Pyx_DECREF(__pyx_t_4); __pyx_t_4 = 0;
    if (__pyx_t_2) {

      /* "src/python/grpcio/grpc/_cython/_cygrpc/metadata.pyx.pxi":35
 *     metadatum_count = len(metadata)
 *     if metadatum_count == 0:
 *       c_count[0] = 0             # <<<<<<<<<<<<<<
 *       c_metadata[0] = NULL
 *     else:
 */
      (__pyx_v_c_count[0]) = 0;

      /* "src/python/grpcio/grpc/_cython/_cygrpc/metadata.pyx.pxi":36
 *     if metadatum_count == 0:
 *       c_count[0] = 0
 *       c_metadata[0] = NULL             # <<<<<<<<<<<<<<
 *     else:
 *       c_count[0] = metadatum_count
 */
      (__pyx_v_c_metadata[0]) = NULL;

      /* "src/python/grpcio/grpc/_cython/_cygrpc/metadata.pyx.pxi":34
 *   else:
 *     metadatum_count = len(metadata)
 *     if metadatum_count == 0:             # <<<<<<<<<<<<<<
 *       c_count[0] = 0
 *       c_metadata[0] = NULL
 */
      goto __pyx_L4;
    }

    /* "src/python/grpcio/grpc/_cython/_cygrpc/metadata.pyx.pxi":38
 *       c_metadata[0] = NULL
 *     else:
 *       c_count[0] = metadatum_count             # <<<<<<<<<<<<<<
 *       c_metadata[0] = <grpc_metadata *>gpr_malloc(
 *           metadatum_count * sizeof(grpc_metadata))
 */
    /*else*/ {
      __pyx_t_5 = __Pyx_PyInt_As_size_t(__pyx_v_metadatum_count); if (unlikely((__pyx_t_5 == (size_t)-1) && PyErr_Occurred())) __PYX_ERR(13, 38, __pyx_L1_error)
      (__pyx_v_c_count[0]) = __pyx_t_5;

      /* "src/python/grpcio/grpc/_cython/_cygrpc/metadata.pyx.pxi":40
 *       c_count[0] = metadatum_count
 *       c_metadata[0] = <grpc_metadata *>gpr_malloc(
 *           metadatum_count * sizeof(grpc_metadata))             # <<<<<<<<<<<<<<
 *       for index, (key, value) in enumerate(metadata):
 *         encoded_key = _encode(key)
 */
      __pyx_t_4 = __Pyx_PyInt_FromSize_t((sizeof(grpc_metadata))); if (unlikely(!__pyx_t_4)) __PYX_ERR(13, 40, __pyx_L1_error)
      __Pyx_GOTREF(__pyx_t_4);
      __pyx_t_6 = PyNumber_Multiply(__pyx_v_metadatum_count, __pyx_t_4); if (unlikely(!__pyx_t_6)) __PYX_ERR(13, 40, __pyx_L1_error)
      __Pyx_GOTREF(__pyx_t_6);
      __Pyx_DECREF(__pyx_t_4); __pyx_t_4 = 0;
      __pyx_t_5 = __Pyx_PyInt_As_size_t(__pyx_t_6); if (unlikely((__pyx_t_5 == (size_t)-1) && PyErr_Occurred())) __PYX_ERR(13, 40, __pyx_L1_error)
      __Pyx_DECREF(__pyx_t_6); __pyx_t_6 = 0;

      /* "src/python/grpcio/grpc/_cython/_cygrpc/metadata.pyx.pxi":39
 *     else:
 *       c_count[0] = metadatum_count
 *       c_metadata[0] = <grpc_metadata *>gpr_malloc(             # <<<<<<<<<<<<<<
 *           metadatum_count * sizeof(grpc_metadata))
 *       for index, (key, value) in enumerate(metadata):
 */
      (__pyx_v_c_metadata[0]) = ((grpc_metadata *)gpr_malloc(__pyx_t_5));

      /* "src/python/grpcio/grpc/_cython/_cygrpc/metadata.pyx.pxi":41
 *       c_metadata[0] = <grpc_metadata *>gpr_malloc(
 *           metadatum_count * sizeof(grpc_metadata))
 *       for index, (key, value) in enumerate(metadata):             # <<<<<<<<<<<<<<
 *         encoded_key = _encode(key)
 *         encoded_value = value if encoded_key[-4:] == b'-bin' else _encode(value)
 */
      __Pyx_INCREF(__pyx_int_0);
      __pyx_t_6 = __pyx_int_0;
      if (likely(PyList_CheckExact(__pyx_v_metadata)) || PyTuple_CheckExact(__pyx_v_metadata)) {
        __pyx_t_4 = __pyx_v_metadata; __Pyx_INCREF(__pyx_t_4); __pyx_t_3 = 0;
        __pyx_t_7 = NULL;
      } else {
        __pyx_t_3 = -1; __pyx_t_4 = PyObject_GetIter(__pyx_v_metadata); if (unlikely(!__pyx_t_4)) __PYX_ERR(13, 41, __pyx_L1_error)
        __Pyx_GOTREF(__pyx_t_4);
        __pyx_t_7 = Py_TYPE(__pyx_t_4)->tp_iternext; if (unlikely(!__pyx_t_7)) __PYX_ERR(13, 41, __pyx_L1_error)
      }
      for (;;) {
        if (likely(!__pyx_t_7)) {
          if (likely(PyList_CheckExact(__pyx_t_4))) {
            if (__pyx_t_3 >= PyList_GET_SIZE(__pyx_t_4)) break;
            #if CYTHON_ASSUME_SAFE_MACROS && !CYTHON_AVOID_BORROWED_REFS
            __pyx_t_8 = PyList_GET_ITEM(__pyx_t_4, __pyx_t_3); __Pyx_INCREF(__pyx_t_8); __pyx_t_3++; if (unlikely(0 < 0)) __PYX_ERR(13, 41, __pyx_L1_error)
            #else
            __pyx_t_8 = PySequence_ITEM(__pyx_t_4, __pyx_t_3); __pyx_t_3++; if (unlikely(!__pyx_t_8)) __PYX_ERR(13, 41, __pyx_L1_error)
            __Pyx_GOTREF(__pyx_t_8);
            #endif
          } else {
            if (__pyx_t_3 >= PyTuple_GET_SIZE(__pyx_t_4)) break;
            #if CYTHON_ASSUME_SAFE_MACROS && !CYTHON_AVOID_BORROWED_REFS
            __pyx_t_8 = PyTuple_GET_ITEM(__pyx_t_4, __pyx_t_3); __Pyx_INCREF(__pyx_t_8); __pyx_t_3++; if (unlikely(0 < 0)) __PYX_ERR(13, 41, __pyx_L1_error)
            #else
            __pyx_t_8 = PySequence_ITEM(__pyx_t_4, __pyx_t_3); __pyx_t_3++; if (unlikely(!__pyx_t_8)) __PYX_ERR(13, 41, __pyx_L1_error)
            __Pyx_GOTREF(__pyx_t_8);
            #endif
          }
        } else {
          __pyx_t_8 = __pyx_t_7(__pyx_t_4);
          if (unlikely(!__pyx_t_8)) {
            PyObject* exc_type = PyErr_Occurred();
            if (exc_type) {
              if (likely(__Pyx_PyErr_GivenExceptionMatches(exc_type, PyExc_StopIteration))) PyErr_Clear();
              else __PYX_ERR(13, 41, __pyx_L1_error)
            }
            break;
          }
          __Pyx_GOTREF(__pyx_t_8);
        }
        if ((likely(PyTuple_CheckExact(__pyx_t_8))) || (PyList_CheckExact(__pyx_t_8))) {
          PyObject* sequence = __pyx_t_8;
          Py_ssize_t size = __Pyx_PySequence_SIZE(sequence);
          if (unlikely(size != 2)) {
            if (size > 2) __Pyx_RaiseTooManyValuesError(2);
            else if (size >= 0) __Pyx_RaiseNeedMoreValuesError(size);
            __PYX_ERR(13, 41, __pyx_L1_error)
          }
          #if CYTHON_ASSUME_SAFE_MACROS && !CYTHON_AVOID_BORROWED_REFS
          if (likely(PyTuple_CheckExact(sequence))) {
            __pyx_t_9 = PyTuple_GET_ITEM(sequence, 0); 
            __pyx_t_10 = PyTuple_GET_ITEM(sequence, 1); 
          } else {
            __pyx_t_9 = PyList_GET_ITEM(sequence, 0); 
            __pyx_t_10 = PyList_GET_ITEM(sequence, 1); 
          }
          __Pyx_INCREF(__pyx_t_9);
          __Pyx_INCREF(__pyx_t_10);
          #else
          __pyx_t_9 = PySequence_ITEM(sequence, 0); if (unlikely(!__pyx_t_9)) __PYX_ERR(13, 41, __pyx_L1_error)
          __Pyx_GOTREF(__pyx_t_9);
          __pyx_t_10 = PySequence_ITEM(sequence, 1); if (unlikely(!__pyx_t_10)) __PYX_ERR(13, 41, __pyx_L1_error)
          __Pyx_GOTREF(__pyx_t_10);
          #endif
          __Pyx_DECREF(__pyx_t_8); __pyx_t_8 = 0;
        } else {
          Py_ssize_t index = -1;
          __pyx_t_11 = PyObject_GetIter(__pyx_t_8); if (unlikely(!__pyx_t_11)) __PYX_ERR(13, 41, __pyx_L1_error)
          __Pyx_GOTREF(__pyx_t_11);
          __Pyx_DECREF(__pyx_t_8); __pyx_t_8 = 0;
          __pyx_t_12 = Py_TYPE(__pyx_t_11)->tp_iternext;
          index = 0; __pyx_t_9 = __pyx_t_12(__pyx_t_11); if (unlikely(!__pyx_t_9)) goto __pyx_L7_unpacking_failed;
          __Pyx_GOTREF(__pyx_t_9);
          index = 1; __pyx_t_10 = __pyx_t_12(__pyx_t_11); if (unlikely(!__pyx_t_10)) goto __pyx_L7_unpacking_failed;
          __Pyx_GOTREF(__pyx_t_10);
          if (__Pyx_IternextUnpackEndCheck(__pyx_t_12(__pyx_t_11), 2) < 0) __PYX_ERR(13, 41, __pyx_L1_error)
          __pyx_t_12 = NULL;
          __Pyx_DECREF(__pyx_t_11); __pyx_t_11 = 0;
          goto __pyx_L8_unpacking_done;
          __pyx_L7_unpacking_failed:;
          __Pyx_DECREF(__pyx_t_11); __pyx_t_11 = 0;
          __pyx_t_12 = NULL;
          if (__Pyx_IterFinish() == 0) __Pyx_RaiseNeedMoreValuesError(index);
          __PYX_ERR(13, 41, __pyx_L1_error)
          __pyx_L8_unpacking_done:;
        }
        __Pyx_XDECREF_SET(__pyx_v_key, __pyx_t_9);
        __pyx_t_9 = 0;
        __Pyx_XDECREF_SET(__pyx_v_value, __pyx_t_10);
        __pyx_t_10 = 0;
        __Pyx_INCREF(__pyx_t_6);
        __Pyx_XDECREF_SET(__pyx_v_index, __pyx_t_6);
        __pyx_t_8 = __Pyx_PyInt_AddObjC(__pyx_t_6, __pyx_int_1, 1, 0); if (unlikely(!__pyx_t_8)) __PYX_ERR(13, 41, __pyx_L1_error)
        __Pyx_GOTREF(__pyx_t_8);
        __Pyx_DECREF(__pyx_t_6);
        __pyx_t_6 = __pyx_t_8;
        __pyx_t_8 = 0;

        /* "src/python/grpcio/grpc/_cython/_cygrpc/metadata.pyx.pxi":42
 *           metadatum_count * sizeof(grpc_metadata))
 *       for index, (key, value) in enumerate(metadata):
 *         encoded_key = _encode(key)             # <<<<<<<<<<<<<<
 *         encoded_value = value if encoded_key[-4:] == b'-bin' else _encode(value)
 *         c_metadata[0][index].key = _slice_from_bytes(encoded_key)
 */
        __pyx_t_8 = __pyx_f_4grpc_7_cython_6cygrpc__encode(__pyx_v_key); if (unlikely(!__pyx_t_8)) __PYX_ERR(13, 42, __pyx_L1_error)
        __Pyx_GOTREF(__pyx_t_8);
        __Pyx_XDECREF_SET(__pyx_v_encoded_key, ((PyObject*)__pyx_t_8));
        __pyx_t_8 = 0;

        /* "src/python/grpcio/grpc/_cython/_cygrpc/metadata.pyx.pxi":43
 *       for index, (key, value) in enumerate(metadata):
 *         encoded_key = _encode(key)
 *         encoded_value = value if encoded_key[-4:] == b'-bin' else _encode(value)             # <<<<<<<<<<<<<<
 *         c_metadata[0][index].key = _slice_from_bytes(encoded_key)
 *         c_metadata[0][index].value = _slice_from_bytes(encoded_value)
 */
        if (unlikely(__pyx_v_encoded_key == Py_None)) {
          PyErr_SetString(PyExc_TypeError, "'NoneType' object is not subscriptable");
          __PYX_ERR(13, 43, __pyx_L1_error)
        }
        __pyx_t_10 = PySequence_GetSlice(__pyx_v_encoded_key, -4L, PY_SSIZE_T_MAX); if (unlikely(!__pyx_t_10)) __PYX_ERR(13, 43, __pyx_L1_error)
        __Pyx_GOTREF(__pyx_t_10);
        __pyx_t_2 = (__Pyx_PyBytes_Equals(__pyx_t_10, __pyx_kp_b_bin, Py_EQ)); if (unlikely(__pyx_t_2 < 0)) __PYX_ERR(13, 43, __pyx_L1_error)
        __Pyx_DECREF(__pyx_t_10); __pyx_t_10 = 0;
        if ((__pyx_t_2 != 0)) {
          __Pyx_INCREF(__pyx_v_value);
          __pyx_t_8 = __pyx_v_value;
        } else {
          __pyx_t_10 = __pyx_f_4grpc_7_cython_6cygrpc__encode(__pyx_v_value); if (unlikely(!__pyx_t_10)) __PYX_ERR(13, 43, __pyx_L1_error)
          __Pyx_GOTREF(__pyx_t_10);
          __pyx_t_8 = __pyx_t_10;
          __pyx_t_10 = 0;
        }
        __Pyx_XDECREF_SET(__pyx_v_encoded_value, __pyx_t_8);
        __pyx_t_8 = 0;

        /* "src/python/grpcio/grpc/_cython/_cygrpc/metadata.pyx.pxi":44
 *         encoded_key = _encode(key)
 *         encoded_value = value if encoded_key[-4:] == b'-bin' else _encode(value)
 *         c_metadata[0][index].key = _slice_from_bytes(encoded_key)             # <<<<<<<<<<<<<<
 *         c_metadata[0][index].value = _slice_from_bytes(encoded_value)
 * 
 */
        __pyx_t_13 = __Pyx_PyIndex_AsSsize_t(__pyx_v_index); if (unlikely((__pyx_t_13 == (Py_ssize_t)-1) && PyErr_Occurred())) __PYX_ERR(13, 44, __pyx_L1_error)
        ((__pyx_v_c_metadata[0])[__pyx_t_13]).key = __pyx_f_4grpc_7_cython_6cygrpc__slice_from_bytes(__pyx_v_encoded_key);

        /* "src/python/grpcio/grpc/_cython/_cygrpc/metadata.pyx.pxi":45
 *         encoded_value = value if encoded_key[-4:] == b'-bin' else _encode(value)
 *         c_metadata[0][index].key = _slice_from_bytes(encoded_key)
 *         c_metadata[0][index].value = _slice_from_bytes(encoded_value)             # <<<<<<<<<<<<<<
 * 
 * 
 */
        if (!(likely(PyBytes_CheckExact(__pyx_v_encoded_value))||((__pyx_v_encoded_value) == Py_None)||(PyErr_Format(PyExc_TypeError, "Expected %.16s, got %.200s", "bytes", Py_TYPE(__pyx_v_encoded_value)->tp_name), 0))) __PYX_ERR(13, 45, __pyx_L1_error)
        __pyx_t_13 = __Pyx_PyIndex_AsSsize_t(__pyx_v_index); if (unlikely((__pyx_t_13 == (Py_ssize_t)-1) && PyErr_Occurred())) __PYX_ERR(13, 45, __pyx_L1_error)
        ((__pyx_v_c_metadata[0])[__pyx_t_13]).value = __pyx_f_4grpc_7_cython_6cygrpc__slice_from_bytes(((PyObject*)__pyx_v_encoded_value));

        /* "src/python/grpcio/grpc/_cython/_cygrpc/metadata.pyx.pxi":41
 *       c_metadata[0] = <grpc_metadata *>gpr_malloc(
 *           metadatum_count * sizeof(grpc_metadata))
 *       for index, (key, value) in enumerate(metadata):             # <<<<<<<<<<<<<<
 *         encoded_key = _encode(key)
 *         encoded_value = value if encoded_key[-4:] == b'-bin' else _encode(value)
 */
      }
      __Pyx_DECREF(__pyx_t_4); __pyx_t_4 = 0;
      __Pyx_DECREF(__pyx_t_6); __pyx_t_6 = 0;
    }
    __pyx_L4:;
  }
  __pyx_L3:;

  /* "src/python/grpcio/grpc/_cython/_cygrpc/metadata.pyx.pxi":27
 * 
 * 
 * cdef void _store_c_metadata(             # <<<<<<<<<<<<<<
 *     metadata, grpc_metadata **c_metadata, size_t *c_count) except *:
 *   if metadata is None:
 */

  /* function exit code */
  goto __pyx_L0;
  __pyx_L1_error:;
  __Pyx_XDECREF(__pyx_t_4);
  __Pyx_XDECREF(__pyx_t_6);
  __Pyx_XDECREF(__pyx_t_8);
  __Pyx_XDECREF(__pyx_t_9);
  __Pyx_XDECREF(__pyx_t_10);
  __Pyx_XDECREF(__pyx_t_11);
  __Pyx_AddTraceback("grpc._cython.cygrpc._store_c_metadata", __pyx_clineno, __pyx_lineno, __pyx_filename);
  __pyx_L0:;
  __Pyx_XDECREF(__pyx_v_metadatum_count);
  __Pyx_XDECREF(__pyx_v_index);
  __Pyx_XDECREF(__pyx_v_key);
  __Pyx_XDECREF(__pyx_v_value);
  __Pyx_XDECREF(__pyx_v_encoded_key);
  __Pyx_XDECREF(__pyx_v_encoded_value);
  __Pyx_RefNannyFinishContext();
}

/* "src/python/grpcio/grpc/_cython/_cygrpc/metadata.pyx.pxi":48
 * 
 * 
 * cdef void _release_c_metadata(grpc_metadata *c_metadata, int count) except *:             # <<<<<<<<<<<<<<
 *   if 0 < count:
 *     for index in range(count):
 */

static void __pyx_f_4grpc_7_cython_6cygrpc__release_c_metadata(grpc_metadata *__pyx_v_c_metadata, int __pyx_v_count) {
  int __pyx_v_index;
  __Pyx_RefNannyDeclarations
  int __pyx_t_1;
  int __pyx_t_2;
  int __pyx_t_3;
  int __pyx_t_4;
  __Pyx_RefNannySetupContext("_release_c_metadata", 0);

  /* "src/python/grpcio/grpc/_cython/_cygrpc/metadata.pyx.pxi":49
 * 
 * cdef void _release_c_metadata(grpc_metadata *c_metadata, int count) except *:
 *   if 0 < count:             # <<<<<<<<<<<<<<
 *     for index in range(count):
 *       grpc_slice_unref(c_metadata[index].key)
 */
  __pyx_t_1 = ((0 < __pyx_v_count) != 0);
  if (__pyx_t_1) {

    /* "src/python/grpcio/grpc/_cython/_cygrpc/metadata.pyx.pxi":50
 * cdef void _release_c_metadata(grpc_metadata *c_metadata, int count) except *:
 *   if 0 < count:
 *     for index in range(count):             # <<<<<<<<<<<<<<
 *       grpc_slice_unref(c_metadata[index].key)
 *       grpc_slice_unref(c_metadata[index].value)
 */
    __pyx_t_2 = __pyx_v_count;
    __pyx_t_3 = __pyx_t_2;
    for (__pyx_t_4 = 0; __pyx_t_4 < __pyx_t_3; __pyx_t_4+=1) {
      __pyx_v_index = __pyx_t_4;

      /* "src/python/grpcio/grpc/_cython/_cygrpc/metadata.pyx.pxi":51
 *   if 0 < count:
 *     for index in range(count):
 *       grpc_slice_unref(c_metadata[index].key)             # <<<<<<<<<<<<<<
 *       grpc_slice_unref(c_metadata[index].value)
 *     gpr_free(c_metadata)
 */
      grpc_slice_unref((__pyx_v_c_metadata[__pyx_v_index]).key);

      /* "src/python/grpcio/grpc/_cython/_cygrpc/metadata.pyx.pxi":52
 *     for index in range(count):
 *       grpc_slice_unref(c_metadata[index].key)
 *       grpc_slice_unref(c_metadata[index].value)             # <<<<<<<<<<<<<<
 *     gpr_free(c_metadata)
 * 
 */
      grpc_slice_unref((__pyx_v_c_metadata[__pyx_v_index]).value);
    }

    /* "src/python/grpcio/grpc/_cython/_cygrpc/metadata.pyx.pxi":53
 *       grpc_slice_unref(c_metadata[index].key)
 *       grpc_slice_unref(c_metadata[index].value)
 *     gpr_free(c_metadata)             # <<<<<<<<<<<<<<
 * 
 * 
 */
    gpr_free(__pyx_v_c_metadata);

    /* "src/python/grpcio/grpc/_cython/_cygrpc/metadata.pyx.pxi":49
 * 
 * cdef void _release_c_metadata(grpc_metadata *c_metadata, int count) except *:
 *   if 0 < count:             # <<<<<<<<<<<<<<
 *     for index in range(count):
 *       grpc_slice_unref(c_metadata[index].key)
 */
  }

  /* "src/python/grpcio/grpc/_cython/_cygrpc/metadata.pyx.pxi":48
 * 
 * 
 * cdef void _release_c_metadata(grpc_metadata *c_metadata, int count) except *:             # <<<<<<<<<<<<<<
 *   if 0 < count:
 *     for index in range(count):
 */

  /* function exit code */
  __Pyx_RefNannyFinishContext();
}

/* "src/python/grpcio/grpc/_cython/_cygrpc/metadata.pyx.pxi":56
 * 
 * 
 * cdef tuple _metadatum(grpc_slice key_slice, grpc_slice value_slice):             # <<<<<<<<<<<<<<
 *   cdef bytes key = _slice_bytes(key_slice)
 *   cdef bytes value = _slice_bytes(value_slice)
 */

static PyObject *__pyx_f_4grpc_7_cython_6cygrpc__metadatum(grpc_slice __pyx_v_key_slice, grpc_slice __pyx_v_value_slice) {
  PyObject *__pyx_v_key = 0;
  PyObject *__pyx_v_value = 0;
  PyObject *__pyx_r = NULL;
  __Pyx_RefNannyDeclarations
  PyObject *__pyx_t_1 = NULL;
  PyObject *__pyx_t_2 = NULL;
  PyObject *__pyx_t_3 = NULL;
  PyObject *__pyx_t_4 = NULL;
  PyObject *__pyx_t_5 = NULL;
  int __pyx_t_6;
  int __pyx_t_7;
  PyObject *__pyx_t_8 = NULL;
  __Pyx_RefNannySetupContext("_metadatum", 0);

  /* "src/python/grpcio/grpc/_cython/_cygrpc/metadata.pyx.pxi":57
 * 
 * cdef tuple _metadatum(grpc_slice key_slice, grpc_slice value_slice):
 *   cdef bytes key = _slice_bytes(key_slice)             # <<<<<<<<<<<<<<
 *   cdef bytes value = _slice_bytes(value_slice)
 *   return <tuple>_Metadatum(
 */
  __pyx_t_1 = __pyx_f_4grpc_7_cython_6cygrpc__slice_bytes(__pyx_v_key_slice); if (unlikely(!__pyx_t_1)) __PYX_ERR(13, 57, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_1);
  __pyx_v_key = ((PyObject*)__pyx_t_1);
  __pyx_t_1 = 0;

  /* "src/python/grpcio/grpc/_cython/_cygrpc/metadata.pyx.pxi":58
 * cdef tuple _metadatum(grpc_slice key_slice, grpc_slice value_slice):
 *   cdef bytes key = _slice_bytes(key_slice)
 *   cdef bytes value = _slice_bytes(value_slice)             # <<<<<<<<<<<<<<
 *   return <tuple>_Metadatum(
 *       _decode(key), value if key[-4:] == b'-bin' else _decode(value))
 */
  __pyx_t_1 = __pyx_f_4grpc_7_cython_6cygrpc__slice_bytes(__pyx_v_value_slice); if (unlikely(!__pyx_t_1)) __PYX_ERR(13, 58, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_1);
  __pyx_v_value = ((PyObject*)__pyx_t_1);
  __pyx_t_1 = 0;

  /* "src/python/grpcio/grpc/_cython/_cygrpc/metadata.pyx.pxi":59
 *   cdef bytes key = _slice_bytes(key_slice)
 *   cdef bytes value = _slice_bytes(value_slice)
 *   return <tuple>_Metadatum(             # <<<<<<<<<<<<<<
 *       _decode(key), value if key[-4:] == b'-bin' else _decode(value))
 * 
 */
  __Pyx_XDECREF(__pyx_r);
  __pyx_t_2 = __Pyx_GetModuleGlobalName(__pyx_n_s_Metadatum); if (unlikely(!__pyx_t_2)) __PYX_ERR(13, 59, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_2);

  /* "src/python/grpcio/grpc/_cython/_cygrpc/metadata.pyx.pxi":60
 *   cdef bytes value = _slice_bytes(value_slice)
 *   return <tuple>_Metadatum(
 *       _decode(key), value if key[-4:] == b'-bin' else _decode(value))             # <<<<<<<<<<<<<<
 * 
 * 
 */
  __pyx_t_3 = __pyx_f_4grpc_7_cython_6cygrpc__decode(__pyx_v_key); if (unlikely(!__pyx_t_3)) __PYX_ERR(13, 60, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_3);
  if (unlikely(__pyx_v_key == Py_None)) {
    PyErr_SetString(PyExc_TypeError, "'NoneType' object is not subscriptable");
    __PYX_ERR(13, 60, __pyx_L1_error)
  }
  __pyx_t_5 = PySequence_GetSlice(__pyx_v_key, -4L, PY_SSIZE_T_MAX); if (unlikely(!__pyx_t_5)) __PYX_ERR(13, 60, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_5);
  __pyx_t_6 = (__Pyx_PyBytes_Equals(__pyx_t_5, __pyx_kp_b_bin, Py_EQ)); if (unlikely(__pyx_t_6 < 0)) __PYX_ERR(13, 60, __pyx_L1_error)
  __Pyx_DECREF(__pyx_t_5); __pyx_t_5 = 0;
  if ((__pyx_t_6 != 0)) {
    __Pyx_INCREF(__pyx_v_value);
    __pyx_t_4 = __pyx_v_value;
  } else {
    __pyx_t_5 = __pyx_f_4grpc_7_cython_6cygrpc__decode(__pyx_v_value); if (unlikely(!__pyx_t_5)) __PYX_ERR(13, 60, __pyx_L1_error)
    __Pyx_GOTREF(__pyx_t_5);
    __pyx_t_4 = __pyx_t_5;
    __pyx_t_5 = 0;
  }
  __pyx_t_5 = NULL;
  __pyx_t_7 = 0;
  if (CYTHON_UNPACK_METHODS && unlikely(PyMethod_Check(__pyx_t_2))) {
    __pyx_t_5 = PyMethod_GET_SELF(__pyx_t_2);
    if (likely(__pyx_t_5)) {
      PyObject* function = PyMethod_GET_FUNCTION(__pyx_t_2);
      __Pyx_INCREF(__pyx_t_5);
      __Pyx_INCREF(function);
      __Pyx_DECREF_SET(__pyx_t_2, function);
      __pyx_t_7 = 1;
    }
  }
  #if CYTHON_FAST_PYCALL
  if (PyFunction_Check(__pyx_t_2)) {
    PyObject *__pyx_temp[3] = {__pyx_t_5, __pyx_t_3, __pyx_t_4};
    __pyx_t_1 = __Pyx_PyFunction_FastCall(__pyx_t_2, __pyx_temp+1-__pyx_t_7, 2+__pyx_t_7); if (unlikely(!__pyx_t_1)) __PYX_ERR(13, 59, __pyx_L1_error)
    __Pyx_XDECREF(__pyx_t_5); __pyx_t_5 = 0;
    __Pyx_GOTREF(__pyx_t_1);
    __Pyx_DECREF(__pyx_t_3); __pyx_t_3 = 0;
    __Pyx_DECREF(__pyx_t_4); __pyx_t_4 = 0;
  } else
  #endif
  #if CYTHON_FAST_PYCCALL
  if (__Pyx_PyFastCFunction_Check(__pyx_t_2)) {
    PyObject *__pyx_temp[3] = {__pyx_t_5, __pyx_t_3, __pyx_t_4};
    __pyx_t_1 = __Pyx_PyCFunction_FastCall(__pyx_t_2, __pyx_temp+1-__pyx_t_7, 2+__pyx_t_7); if (unlikely(!__pyx_t_1)) __PYX_ERR(13, 59, __pyx_L1_error)
    __Pyx_XDECREF(__pyx_t_5); __pyx_t_5 = 0;
    __Pyx_GOTREF(__pyx_t_1);
    __Pyx_DECREF(__pyx_t_3); __pyx_t_3 = 0;
    __Pyx_DECREF(__pyx_t_4); __pyx_t_4 = 0;
  } else
  #endif
  {
    __pyx_t_8 = PyTuple_New(2+__pyx_t_7); if (unlikely(!__pyx_t_8)) __PYX_ERR(13, 59, __pyx_L1_error)
    __Pyx_GOTREF(__pyx_t_8);
    if (__pyx_t_5) {
      __Pyx_GIVEREF(__pyx_t_5); PyTuple_SET_ITEM(__pyx_t_8, 0, __pyx_t_5); __pyx_t_5 = NULL;
    }
    __Pyx_GIVEREF(__pyx_t_3);
    PyTuple_SET_ITEM(__pyx_t_8, 0+__pyx_t_7, __pyx_t_3);
    __Pyx_GIVEREF(__pyx_t_4);
    PyTuple_SET_ITEM(__pyx_t_8, 1+__pyx_t_7, __pyx_t_4);
    __pyx_t_3 = 0;
    __pyx_t_4 = 0;
    __pyx_t_1 = __Pyx_PyObject_Call(__pyx_t_2, __pyx_t_8, NULL); if (unlikely(!__pyx_t_1)) __PYX_ERR(13, 59, __pyx_L1_error)
    __Pyx_GOTREF(__pyx_t_1);
    __Pyx_DECREF(__pyx_t_8); __pyx_t_8 = 0;
  }
  __Pyx_DECREF(__pyx_t_2); __pyx_t_2 = 0;

  /* "src/python/grpcio/grpc/_cython/_cygrpc/metadata.pyx.pxi":59
 *   cdef bytes key = _slice_bytes(key_slice)
 *   cdef bytes value = _slice_bytes(value_slice)
 *   return <tuple>_Metadatum(             # <<<<<<<<<<<<<<
 *       _decode(key), value if key[-4:] == b'-bin' else _decode(value))
 * 
 */
  __Pyx_INCREF(((PyObject*)__pyx_t_1));
  __pyx_r = ((PyObject*)__pyx_t_1);
  __Pyx_DECREF(__pyx_t_1); __pyx_t_1 = 0;
  goto __pyx_L0;

  /* "src/python/grpcio/grpc/_cython/_cygrpc/metadata.pyx.pxi":56
 * 
 * 
 * cdef tuple _metadatum(grpc_slice key_slice, grpc_slice value_slice):             # <<<<<<<<<<<<<<
 *   cdef bytes key = _slice_bytes(key_slice)
 *   cdef bytes value = _slice_bytes(value_slice)
 */

  /* function exit code */
  __pyx_L1_error:;
  __Pyx_XDECREF(__pyx_t_1);
  __Pyx_XDECREF(__pyx_t_2);
  __Pyx_XDECREF(__pyx_t_3);
  __Pyx_XDECREF(__pyx_t_4);
  __Pyx_XDECREF(__pyx_t_5);
  __Pyx_XDECREF(__pyx_t_8);
  __Pyx_AddTraceback("grpc._cython.cygrpc._metadatum", __pyx_clineno, __pyx_lineno, __pyx_filename);
  __pyx_r = 0;
  __pyx_L0:;
  __Pyx_XDECREF(__pyx_v_key);
  __Pyx_XDECREF(__pyx_v_value);
  __Pyx_XGIVEREF(__pyx_r);
  __Pyx_RefNannyFinishContext();
  return __pyx_r;
}
static PyObject *__pyx_gb_4grpc_7_cython_6cygrpc_9_metadata_2generator(__pyx_CoroutineObject *__pyx_generator, CYTHON_UNUSED PyThreadState *__pyx_tstate, PyObject *__pyx_sent_value); /* proto */

/* "src/python/grpcio/grpc/_cython/_cygrpc/metadata.pyx.pxi":65
 * cdef tuple _metadata(grpc_metadata_array *c_metadata_array):
 *   return tuple(
 *       _metadatum(             # <<<<<<<<<<<<<<
 *           c_metadata_array.metadata[index].key,
 *           c_metadata_array.metadata[index].value)
 */

static PyObject *__pyx_pf_4grpc_7_cython_6cygrpc_9_metadata_genexpr(PyObject *__pyx_self) {
  struct __pyx_obj_4grpc_7_cython_6cygrpc___pyx_scope_struct_6_genexpr *__pyx_cur_scope;
  PyObject *__pyx_r = NULL;
  __Pyx_RefNannyDeclarations
  __Pyx_RefNannySetupContext("genexpr", 0);
  __pyx_cur_scope = (struct __pyx_obj_4grpc_7_cython_6cygrpc___pyx_scope_struct_6_genexpr *)__pyx_tp_new_4grpc_7_cython_6cygrpc___pyx_scope_struct_6_genexpr(__pyx_ptype_4grpc_7_cython_6cygrpc___pyx_scope_struct_6_genexpr, __pyx_empty_tuple, NULL);
  if (unlikely(!__pyx_cur_scope)) {
    __pyx_cur_scope = ((struct __pyx_obj_4grpc_7_cython_6cygrpc___pyx_scope_struct_6_genexpr *)Py_None);
    __Pyx_INCREF(Py_None);
    __PYX_ERR(13, 65, __pyx_L1_error)
  } else {
    __Pyx_GOTREF(__pyx_cur_scope);
  }
  __pyx_cur_scope->__pyx_outer_scope = (struct __pyx_obj_4grpc_7_cython_6cygrpc___pyx_scope_struct_5___pyx_f_4grpc_7_cython_6cygrpc__metadata *) __pyx_self;
  __Pyx_INCREF(((PyObject *)__pyx_cur_scope->__pyx_outer_scope));
  __Pyx_GIVEREF(__pyx_cur_scope->__pyx_outer_scope);
  {
    __pyx_CoroutineObject *gen = __Pyx_Generator_New((__pyx_coroutine_body_t) __pyx_gb_4grpc_7_cython_6cygrpc_9_metadata_2generator, NULL, (PyObject *) __pyx_cur_scope, __pyx_n_s_genexpr, __pyx_n_s_metadata_locals_genexpr, __pyx_n_s_grpc__cython_cygrpc); if (unlikely(!gen)) __PYX_ERR(13, 65, __pyx_L1_error)
    __Pyx_DECREF(__pyx_cur_scope);
    __Pyx_RefNannyFinishContext();
    return (PyObject *) gen;
  }

  /* function exit code */
  __pyx_L1_error:;
  __Pyx_AddTraceback("grpc._cython.cygrpc._metadata.genexpr", __pyx_clineno, __pyx_lineno, __pyx_filename);
  __pyx_r = NULL;
  __Pyx_DECREF(((PyObject *)__pyx_cur_scope));
  __Pyx_XGIVEREF(__pyx_r);
  __Pyx_RefNannyFinishContext();
  return __pyx_r;
}

static PyObject *__pyx_gb_4grpc_7_cython_6cygrpc_9_metadata_2generator(__pyx_CoroutineObject *__pyx_generator, CYTHON_UNUSED PyThreadState *__pyx_tstate, PyObject *__pyx_sent_value) /* generator body */
{
  struct __pyx_obj_4grpc_7_cython_6cygrpc___pyx_scope_struct_6_genexpr *__pyx_cur_scope = ((struct __pyx_obj_4grpc_7_cython_6cygrpc___pyx_scope_struct_6_genexpr *)__pyx_generator->closure);
  PyObject *__pyx_r = NULL;
  size_t __pyx_t_1;
  size_t __pyx_t_2;
  size_t __pyx_t_3;
  PyObject *__pyx_t_4 = NULL;
  __Pyx_RefNannyDeclarations
  __Pyx_RefNannySetupContext("genexpr", 0);
  switch (__pyx_generator->resume_label) {
    case 0: goto __pyx_L3_first_run;
    case 1: goto __pyx_L6_resume_from_yield;
    default: /* CPython raises the right error here */
    __Pyx_RefNannyFinishContext();
    return NULL;
  }
  __pyx_L3_first_run:;
  if (unlikely(!__pyx_sent_value)) __PYX_ERR(13, 65, __pyx_L1_error)

  /* "src/python/grpcio/grpc/_cython/_cygrpc/metadata.pyx.pxi":68
 *           c_metadata_array.metadata[index].key,
 *           c_metadata_array.metadata[index].value)
 *       for index in range(c_metadata_array.count))             # <<<<<<<<<<<<<<
 */
  __pyx_t_1 = __pyx_cur_scope->__pyx_outer_scope->__pyx_v_c_metadata_array->count;
  __pyx_t_2 = __pyx_t_1;
  for (__pyx_t_3 = 0; __pyx_t_3 < __pyx_t_2; __pyx_t_3+=1) {
    __pyx_cur_scope->__pyx_v_index = __pyx_t_3;

    /* "src/python/grpcio/grpc/_cython/_cygrpc/metadata.pyx.pxi":65
 * cdef tuple _metadata(grpc_metadata_array *c_metadata_array):
 *   return tuple(
 *       _metadatum(             # <<<<<<<<<<<<<<
 *           c_metadata_array.metadata[index].key,
 *           c_metadata_array.metadata[index].value)
 */
    __pyx_t_4 = __pyx_f_4grpc_7_cython_6cygrpc__metadatum((__pyx_cur_scope->__pyx_outer_scope->__pyx_v_c_metadata_array->metadata[__pyx_cur_scope->__pyx_v_index]).key, (__pyx_cur_scope->__pyx_outer_scope->__pyx_v_c_metadata_array->metadata[__pyx_cur_scope->__pyx_v_index]).value); if (unlikely(!__pyx_t_4)) __PYX_ERR(13, 65, __pyx_L1_error)
    __Pyx_GOTREF(__pyx_t_4);
    __pyx_r = __pyx_t_4;
    __pyx_t_4 = 0;
    __pyx_cur_scope->__pyx_t_0 = __pyx_t_1;
    __pyx_cur_scope->__pyx_t_1 = __pyx_t_2;
    __pyx_cur_scope->__pyx_t_2 = __pyx_t_3;
    __Pyx_XGIVEREF(__pyx_r);
    __Pyx_RefNannyFinishContext();
    __Pyx_Coroutine_ResetAndClearException(__pyx_generator);
    /* return from generator, yielding value */
    __pyx_generator->resume_label = 1;
    return __pyx_r;
    __pyx_L6_resume_from_yield:;
    __pyx_t_1 = __pyx_cur_scope->__pyx_t_0;
    __pyx_t_2 = __pyx_cur_scope->__pyx_t_1;
    __pyx_t_3 = __pyx_cur_scope->__pyx_t_2;
    if (unlikely(!__pyx_sent_value)) __PYX_ERR(13, 65, __pyx_L1_error)
  }
  CYTHON_MAYBE_UNUSED_VAR(__pyx_cur_scope);

  /* function exit code */
  PyErr_SetNone(PyExc_StopIteration);
  goto __pyx_L0;
  __pyx_L1_error:;
  __Pyx_XDECREF(__pyx_t_4);
  __Pyx_AddTraceback("genexpr", __pyx_clineno, __pyx_lineno, __pyx_filename);
  __pyx_L0:;
  __Pyx_XDECREF(__pyx_r); __pyx_r = 0;
  __Pyx_Coroutine_ResetAndClearException(__pyx_generator);
  __pyx_generator->resume_label = -1;
  __Pyx_Coroutine_clear((PyObject*)__pyx_generator);
  __Pyx_RefNannyFinishContext();
  return __pyx_r;
}

/* "src/python/grpcio/grpc/_cython/_cygrpc/metadata.pyx.pxi":63
 * 
 * 
 * cdef tuple _metadata(grpc_metadata_array *c_metadata_array):             # <<<<<<<<<<<<<<
 *   return tuple(
 *       _metadatum(
 */

static PyObject *__pyx_f_4grpc_7_cython_6cygrpc__metadata(grpc_metadata_array *__pyx_v_c_metadata_array) {
  struct __pyx_obj_4grpc_7_cython_6cygrpc___pyx_scope_struct_5___pyx_f_4grpc_7_cython_6cygrpc__metadata *__pyx_cur_scope;
  PyObject *__pyx_r = NULL;
  __Pyx_RefNannyDeclarations
  PyObject *__pyx_t_1 = NULL;
  PyObject *__pyx_t_2 = NULL;
  __Pyx_RefNannySetupContext("_metadata", 0);
  __pyx_cur_scope = (struct __pyx_obj_4grpc_7_cython_6cygrpc___pyx_scope_struct_5___pyx_f_4grpc_7_cython_6cygrpc__metadata *)__pyx_tp_new_4grpc_7_cython_6cygrpc___pyx_scope_struct_5___pyx_f_4grpc_7_cython_6cygrpc__metadata(__pyx_ptype_4grpc_7_cython_6cygrpc___pyx_scope_struct_5___pyx_f_4grpc_7_cython_6cygrpc__metadata, __pyx_empty_tuple, NULL);
  if (unlikely(!__pyx_cur_scope)) {
    __pyx_cur_scope = ((struct __pyx_obj_4grpc_7_cython_6cygrpc___pyx_scope_struct_5___pyx_f_4grpc_7_cython_6cygrpc__metadata *)Py_None);
    __Pyx_INCREF(Py_None);
    __PYX_ERR(13, 63, __pyx_L1_error)
  } else {
    __Pyx_GOTREF(__pyx_cur_scope);
  }
  __pyx_cur_scope->__pyx_v_c_metadata_array = __pyx_v_c_metadata_array;

  /* "src/python/grpcio/grpc/_cython/_cygrpc/metadata.pyx.pxi":64
 * 
 * cdef tuple _metadata(grpc_metadata_array *c_metadata_array):
 *   return tuple(             # <<<<<<<<<<<<<<
 *       _metadatum(
 *           c_metadata_array.metadata[index].key,
 */
  __Pyx_XDECREF(__pyx_r);

  /* "src/python/grpcio/grpc/_cython/_cygrpc/metadata.pyx.pxi":65
 * cdef tuple _metadata(grpc_metadata_array *c_metadata_array):
 *   return tuple(
 *       _metadatum(             # <<<<<<<<<<<<<<
 *           c_metadata_array.metadata[index].key,
 *           c_metadata_array.metadata[index].value)
 */
  __pyx_t_1 = __pyx_pf_4grpc_7_cython_6cygrpc_9_metadata_genexpr(((PyObject*)__pyx_cur_scope)); if (unlikely(!__pyx_t_1)) __PYX_ERR(13, 65, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_1);

  /* "src/python/grpcio/grpc/_cython/_cygrpc/metadata.pyx.pxi":64
 * 
 * cdef tuple _metadata(grpc_metadata_array *c_metadata_array):
 *   return tuple(             # <<<<<<<<<<<<<<
 *       _metadatum(
 *           c_metadata_array.metadata[index].key,
 */
  __pyx_t_2 = __Pyx_PySequence_Tuple(__pyx_t_1); if (unlikely(!__pyx_t_2)) __PYX_ERR(13, 64, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_2);
  __Pyx_DECREF(__pyx_t_1); __pyx_t_1 = 0;
  __pyx_r = ((PyObject*)__pyx_t_2);
  __pyx_t_2 = 0;
  goto __pyx_L0;

  /* "src/python/grpcio/grpc/_cython/_cygrpc/metadata.pyx.pxi":63
 * 
 * 
 * cdef tuple _metadata(grpc_metadata_array *c_metadata_array):             # <<<<<<<<<<<<<<
 *   return tuple(
 *       _metadatum(
 */

  /* function exit code */
  __pyx_L1_error:;
  __Pyx_XDECREF(__pyx_t_1);
  __Pyx_XDECREF(__pyx_t_2);
  __Pyx_AddTraceback("grpc._cython.cygrpc._metadata", __pyx_clineno, __pyx_lineno, __pyx_filename);
  __pyx_r = 0;
  __pyx_L0:;
  __Pyx_DECREF(((PyObject *)__pyx_cur_scope));
  __Pyx_XGIVEREF(__pyx_r);
  __Pyx_RefNannyFinishContext();
  return __pyx_r;
}

/* "src/python/grpcio/grpc/_cython/_cygrpc/operation.pyx.pxi":18
 * cdef class Operation:
 * 
 *   cdef void c(self) except *:             # <<<<<<<<<<<<<<
 *     raise NotImplementedError()
 * 
 */

static void __pyx_f_4grpc_7_cython_6cygrpc_9Operation_c(CYTHON_UNUSED struct __pyx_obj_4grpc_7_cython_6cygrpc_Operation *__pyx_v_self) {
  __Pyx_RefNannyDeclarations
  PyObject *__pyx_t_1 = NULL;
  __Pyx_RefNannySetupContext("c", 0);

  /* "src/python/grpcio/grpc/_cython/_cygrpc/operation.pyx.pxi":19
 * 
 *   cdef void c(self) except *:
 *     raise NotImplementedError()             # <<<<<<<<<<<<<<
 * 
 *   cdef void un_c(self) except *:
 */
  __pyx_t_1 = __Pyx_PyObject_CallNoArg(__pyx_builtin_NotImplementedError); if (unlikely(!__pyx_t_1)) __PYX_ERR(14, 19, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_1);
  __Pyx_Raise(__pyx_t_1, 0, 0, 0);
  __Pyx_DECREF(__pyx_t_1); __pyx_t_1 = 0;
  __PYX_ERR(14, 19, __pyx_L1_error)

  /* "src/python/grpcio/grpc/_cython/_cygrpc/operation.pyx.pxi":18
 * cdef class Operation:
 * 
 *   cdef void c(self) except *:             # <<<<<<<<<<<<<<
 *     raise NotImplementedError()
 * 
 */

  /* function exit code */
  __pyx_L1_error:;
  __Pyx_XDECREF(__pyx_t_1);
  __Pyx_AddTraceback("grpc._cython.cygrpc.Operation.c", __pyx_clineno, __pyx_lineno, __pyx_filename);
  __Pyx_RefNannyFinishContext();
}

/* "src/python/grpcio/grpc/_cython/_cygrpc/operation.pyx.pxi":21
 *     raise NotImplementedError()
 * 
 *   cdef void un_c(self) except *:             # <<<<<<<<<<<<<<
 *     raise NotImplementedError()
 * 
 */

static void __pyx_f_4grpc_7_cython_6cygrpc_9Operation_un_c(CYTHON_UNUSED struct __pyx_obj_4grpc_7_cython_6cygrpc_Operation *__pyx_v_self) {
  __Pyx_RefNannyDeclarations
  PyObject *__pyx_t_1 = NULL;
  __Pyx_RefNannySetupContext("un_c", 0);

  /* "src/python/grpcio/grpc/_cython/_cygrpc/operation.pyx.pxi":22
 * 
 *   cdef void un_c(self) except *:
 *     raise NotImplementedError()             # <<<<<<<<<<<<<<
 * 
 * 
 */
  __pyx_t_1 = __Pyx_PyObject_CallNoArg(__pyx_builtin_NotImplementedError); if (unlikely(!__pyx_t_1)) __PYX_ERR(14, 22, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_1);
  __Pyx_Raise(__pyx_t_1, 0, 0, 0);
  __Pyx_DECREF(__pyx_t_1); __pyx_t_1 = 0;
  __PYX_ERR(14, 22, __pyx_L1_error)

  /* "src/python/grpcio/grpc/_cython/_cygrpc/operation.pyx.pxi":21
 *     raise NotImplementedError()
 * 
 *   cdef void un_c(self) except *:             # <<<<<<<<<<<<<<
 *     raise NotImplementedError()
 * 
 */

  /* function exit code */
  __pyx_L1_error:;
  __Pyx_XDECREF(__pyx_t_1);
  __Pyx_AddTraceback("grpc._cython.cygrpc.Operation.un_c", __pyx_clineno, __pyx_lineno, __pyx_filename);
  __Pyx_RefNannyFinishContext();
}

/* "(tree fragment)":1
 * def __reduce_cython__(self):             # <<<<<<<<<<<<<<
 *     raise TypeError("self.c_op cannot be converted to a Python object for pickling")
 * def __setstate_cython__(self, __pyx_state):
 */

/* Python wrapper */
static PyObject *__pyx_pw_4grpc_7_cython_6cygrpc_9Operation_1__reduce_cython__(PyObject *__pyx_v_self, CYTHON_UNUSED PyObject *unused); /*proto*/
static PyObject *__pyx_pw_4grpc_7_cython_6cygrpc_9Operation_1__reduce_cython__(PyObject *__pyx_v_self, CYTHON_UNUSED PyObject *unused) {
  PyObject *__pyx_r = 0;
  __Pyx_RefNannyDeclarations
  __Pyx_RefNannySetupContext("__reduce_cython__ (wrapper)", 0);
  __pyx_r = __pyx_pf_4grpc_7_cython_6cygrpc_9Operation___reduce_cython__(((struct __pyx_obj_4grpc_7_cython_6cygrpc_Operation *)__pyx_v_self));

  /* function exit code */
  __Pyx_RefNannyFinishContext();
  return __pyx_r;
}

static PyObject *__pyx_pf_4grpc_7_cython_6cygrpc_9Operation___reduce_cython__(CYTHON_UNUSED struct __pyx_obj_4grpc_7_cython_6cygrpc_Operation *__pyx_v_self) {
  PyObject *__pyx_r = NULL;
  __Pyx_RefNannyDeclarations
  PyObject *__pyx_t_1 = NULL;
  __Pyx_RefNannySetupContext("__reduce_cython__", 0);

  /* "(tree fragment)":2
 * def __reduce_cython__(self):
 *     raise TypeError("self.c_op cannot be converted to a Python object for pickling")             # <<<<<<<<<<<<<<
 * def __setstate_cython__(self, __pyx_state):
 *     raise TypeError("self.c_op cannot be converted to a Python object for pickling")
 */
  __pyx_t_1 = __Pyx_PyObject_Call(__pyx_builtin_TypeError, __pyx_tuple__84, NULL); if (unlikely(!__pyx_t_1)) __PYX_ERR(7, 2, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_1);
  __Pyx_Raise(__pyx_t_1, 0, 0, 0);
  __Pyx_DECREF(__pyx_t_1); __pyx_t_1 = 0;
  __PYX_ERR(7, 2, __pyx_L1_error)

  /* "(tree fragment)":1
 * def __reduce_cython__(self):             # <<<<<<<<<<<<<<
 *     raise TypeError("self.c_op cannot be converted to a Python object for pickling")
 * def __setstate_cython__(self, __pyx_state):
 */

  /* function exit code */
  __pyx_L1_error:;
  __Pyx_XDECREF(__pyx_t_1);
  __Pyx_AddTraceback("grpc._cython.cygrpc.Operation.__reduce_cython__", __pyx_clineno, __pyx_lineno, __pyx_filename);
  __pyx_r = NULL;
  __Pyx_XGIVEREF(__pyx_r);
  __Pyx_RefNannyFinishContext();
  return __pyx_r;
}

/* "(tree fragment)":3
 * def __reduce_cython__(self):
 *     raise TypeError("self.c_op cannot be converted to a Python object for pickling")
 * def __setstate_cython__(self, __pyx_state):             # <<<<<<<<<<<<<<
 *     raise TypeError("self.c_op cannot be converted to a Python object for pickling")
 */

/* Python wrapper */
static PyObject *__pyx_pw_4grpc_7_cython_6cygrpc_9Operation_3__setstate_cython__(PyObject *__pyx_v_self, PyObject *__pyx_v___pyx_state); /*proto*/
static PyObject *__pyx_pw_4grpc_7_cython_6cygrpc_9Operation_3__setstate_cython__(PyObject *__pyx_v_self, PyObject *__pyx_v___pyx_state) {
  PyObject *__pyx_r = 0;
  __Pyx_RefNannyDeclarations
  __Pyx_RefNannySetupContext("__setstate_cython__ (wrapper)", 0);
  __pyx_r = __pyx_pf_4grpc_7_cython_6cygrpc_9Operation_2__setstate_cython__(((struct __pyx_obj_4grpc_7_cython_6cygrpc_Operation *)__pyx_v_self), ((PyObject *)__pyx_v___pyx_state));

  /* function exit code */
  __Pyx_RefNannyFinishContext();
  return __pyx_r;
}

static PyObject *__pyx_pf_4grpc_7_cython_6cygrpc_9Operation_2__setstate_cython__(CYTHON_UNUSED struct __pyx_obj_4grpc_7_cython_6cygrpc_Operation *__pyx_v_self, CYTHON_UNUSED PyObject *__pyx_v___pyx_state) {
  PyObject *__pyx_r = NULL;
  __Pyx_RefNannyDeclarations
  PyObject *__pyx_t_1 = NULL;
  __Pyx_RefNannySetupContext("__setstate_cython__", 0);

  /* "(tree fragment)":4
 *     raise TypeError("self.c_op cannot be converted to a Python object for pickling")
 * def __setstate_cython__(self, __pyx_state):
 *     raise TypeError("self.c_op cannot be converted to a Python object for pickling")             # <<<<<<<<<<<<<<
 */
  __pyx_t_1 = __Pyx_PyObject_Call(__pyx_builtin_TypeError, __pyx_tuple__85, NULL); if (unlikely(!__pyx_t_1)) __PYX_ERR(7, 4, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_1);
  __Pyx_Raise(__pyx_t_1, 0, 0, 0);
  __Pyx_DECREF(__pyx_t_1); __pyx_t_1 = 0;
  __PYX_ERR(7, 4, __pyx_L1_error)

  /* "(tree fragment)":3
 * def __reduce_cython__(self):
 *     raise TypeError("self.c_op cannot be converted to a Python object for pickling")
 * def __setstate_cython__(self, __pyx_state):             # <<<<<<<<<<<<<<
 *     raise TypeError("self.c_op cannot be converted to a Python object for pickling")
 */

  /* function exit code */
  __pyx_L1_error:;
  __Pyx_XDECREF(__pyx_t_1);
  __Pyx_AddTraceback("grpc._cython.cygrpc.Operation.__setstate_cython__", __pyx_clineno, __pyx_lineno, __pyx_filename);
  __pyx_r = NULL;
  __Pyx_XGIVEREF(__pyx_r);
  __Pyx_RefNannyFinishContext();
  return __pyx_r;
}

/* "src/python/grpcio/grpc/_cython/_cygrpc/operation.pyx.pxi":27
 * cdef class SendInitialMetadataOperation(Operation):
 * 
 *   def __cinit__(self, initial_metadata, flags):             # <<<<<<<<<<<<<<
 *     self._initial_metadata = initial_metadata
 *     self._flags = flags
 */

/* Python wrapper */
static int __pyx_pw_4grpc_7_cython_6cygrpc_28SendInitialMetadataOperation_1__cinit__(PyObject *__pyx_v_self, PyObject *__pyx_args, PyObject *__pyx_kwds); /*proto*/
static int __pyx_pw_4grpc_7_cython_6cygrpc_28SendInitialMetadataOperation_1__cinit__(PyObject *__pyx_v_self, PyObject *__pyx_args, PyObject *__pyx_kwds) {
  PyObject *__pyx_v_initial_metadata = 0;
  PyObject *__pyx_v_flags = 0;
  int __pyx_r;
  __Pyx_RefNannyDeclarations
  __Pyx_RefNannySetupContext("__cinit__ (wrapper)", 0);
  {
    static PyObject **__pyx_pyargnames[] = {&__pyx_n_s_initial_metadata,&__pyx_n_s_flags,0};
    PyObject* values[2] = {0,0};
    if (unlikely(__pyx_kwds)) {
      Py_ssize_t kw_args;
      const Py_ssize_t pos_args = PyTuple_GET_SIZE(__pyx_args);
      switch (pos_args) {
        case  2: values[1] = PyTuple_GET_ITEM(__pyx_args, 1);
        CYTHON_FALLTHROUGH;
        case  1: values[0] = PyTuple_GET_ITEM(__pyx_args, 0);
        CYTHON_FALLTHROUGH;
        case  0: break;
        default: goto __pyx_L5_argtuple_error;
      }
      kw_args = PyDict_Size(__pyx_kwds);
      switch (pos_args) {
        case  0:
        if (likely((values[0] = __Pyx_PyDict_GetItemStr(__pyx_kwds, __pyx_n_s_initial_metadata)) != 0)) kw_args--;
        else goto __pyx_L5_argtuple_error;
        CYTHON_FALLTHROUGH;
        case  1:
        if (likely((values[1] = __Pyx_PyDict_GetItemStr(__pyx_kwds, __pyx_n_s_flags)) != 0)) kw_args--;
        else {
          __Pyx_RaiseArgtupleInvalid("__cinit__", 1, 2, 2, 1); __PYX_ERR(14, 27, __pyx_L3_error)
        }
      }
      if (unlikely(kw_args > 0)) {
        if (unlikely(__Pyx_ParseOptionalKeywords(__pyx_kwds, __pyx_pyargnames, 0, values, pos_args, "__cinit__") < 0)) __PYX_ERR(14, 27, __pyx_L3_error)
      }
    } else if (PyTuple_GET_SIZE(__pyx_args) != 2) {
      goto __pyx_L5_argtuple_error;
    } else {
      values[0] = PyTuple_GET_ITEM(__pyx_args, 0);
      values[1] = PyTuple_GET_ITEM(__pyx_args, 1);
    }
    __pyx_v_initial_metadata = values[0];
    __pyx_v_flags = values[1];
  }
  goto __pyx_L4_argument_unpacking_done;
  __pyx_L5_argtuple_error:;
  __Pyx_RaiseArgtupleInvalid("__cinit__", 1, 2, 2, PyTuple_GET_SIZE(__pyx_args)); __PYX_ERR(14, 27, __pyx_L3_error)
  __pyx_L3_error:;
  __Pyx_AddTraceback("grpc._cython.cygrpc.SendInitialMetadataOperation.__cinit__", __pyx_clineno, __pyx_lineno, __pyx_filename);
  __Pyx_RefNannyFinishContext();
  return -1;
  __pyx_L4_argument_unpacking_done:;
  __pyx_r = __pyx_pf_4grpc_7_cython_6cygrpc_28SendInitialMetadataOperation___cinit__(((struct __pyx_obj_4grpc_7_cython_6cygrpc_SendInitialMetadataOperation *)__pyx_v_self), __pyx_v_initial_metadata, __pyx_v_flags);

  /* function exit code */
  __Pyx_RefNannyFinishContext();
  return __pyx_r;
}

static int __pyx_pf_4grpc_7_cython_6cygrpc_28SendInitialMetadataOperation___cinit__(struct __pyx_obj_4grpc_7_cython_6cygrpc_SendInitialMetadataOperation *__pyx_v_self, PyObject *__pyx_v_initial_metadata, PyObject *__pyx_v_flags) {
  int __pyx_r;
  __Pyx_RefNannyDeclarations
  int __pyx_t_1;
  __Pyx_RefNannySetupContext("__cinit__", 0);

  /* "src/python/grpcio/grpc/_cython/_cygrpc/operation.pyx.pxi":28
 * 
 *   def __cinit__(self, initial_metadata, flags):
 *     self._initial_metadata = initial_metadata             # <<<<<<<<<<<<<<
 *     self._flags = flags
 * 
 */
  __Pyx_INCREF(__pyx_v_initial_metadata);
  __Pyx_GIVEREF(__pyx_v_initial_metadata);
  __Pyx_GOTREF(__pyx_v_self->_initial_metadata);
  __Pyx_DECREF(__pyx_v_self->_initial_metadata);
  __pyx_v_self->_initial_metadata = __pyx_v_initial_metadata;

  /* "src/python/grpcio/grpc/_cython/_cygrpc/operation.pyx.pxi":29
 *   def __cinit__(self, initial_metadata, flags):
 *     self._initial_metadata = initial_metadata
 *     self._flags = flags             # <<<<<<<<<<<<<<
 * 
 *   def type(self):
 */
  __pyx_t_1 = __Pyx_PyInt_As_int(__pyx_v_flags); if (unlikely((__pyx_t_1 == (int)-1) && PyErr_Occurred())) __PYX_ERR(14, 29, __pyx_L1_error)
  __pyx_v_self->_flags = __pyx_t_1;

  /* "src/python/grpcio/grpc/_cython/_cygrpc/operation.pyx.pxi":27
 * cdef class SendInitialMetadataOperation(Operation):
 * 
 *   def __cinit__(self, initial_metadata, flags):             # <<<<<<<<<<<<<<
 *     self._initial_metadata = initial_metadata
 *     self._flags = flags
 */

  /* function exit code */
  __pyx_r = 0;
  goto __pyx_L0;
  __pyx_L1_error:;
  __Pyx_AddTraceback("grpc._cython.cygrpc.SendInitialMetadataOperation.__cinit__", __pyx_clineno, __pyx_lineno, __pyx_filename);
  __pyx_r = -1;
  __pyx_L0:;
  __Pyx_RefNannyFinishContext();
  return __pyx_r;
}

/* "src/python/grpcio/grpc/_cython/_cygrpc/operation.pyx.pxi":31
 *     self._flags = flags
 * 
 *   def type(self):             # <<<<<<<<<<<<<<
 *     return GRPC_OP_SEND_INITIAL_METADATA
 * 
 */

/* Python wrapper */
static PyObject *__pyx_pw_4grpc_7_cython_6cygrpc_28SendInitialMetadataOperation_3type(PyObject *__pyx_v_self, CYTHON_UNUSED PyObject *unused); /*proto*/
static PyObject *__pyx_pw_4grpc_7_cython_6cygrpc_28SendInitialMetadataOperation_3type(PyObject *__pyx_v_self, CYTHON_UNUSED PyObject *unused) {
  PyObject *__pyx_r = 0;
  __Pyx_RefNannyDeclarations
  __Pyx_RefNannySetupContext("type (wrapper)", 0);
  __pyx_r = __pyx_pf_4grpc_7_cython_6cygrpc_28SendInitialMetadataOperation_2type(((struct __pyx_obj_4grpc_7_cython_6cygrpc_SendInitialMetadataOperation *)__pyx_v_self));

  /* function exit code */
  __Pyx_RefNannyFinishContext();
  return __pyx_r;
}

static PyObject *__pyx_pf_4grpc_7_cython_6cygrpc_28SendInitialMetadataOperation_2type(CYTHON_UNUSED struct __pyx_obj_4grpc_7_cython_6cygrpc_SendInitialMetadataOperation *__pyx_v_self) {
  PyObject *__pyx_r = NULL;
  __Pyx_RefNannyDeclarations
  PyObject *__pyx_t_1 = NULL;
  __Pyx_RefNannySetupContext("type", 0);

  /* "src/python/grpcio/grpc/_cython/_cygrpc/operation.pyx.pxi":32
 * 
 *   def type(self):
 *     return GRPC_OP_SEND_INITIAL_METADATA             # <<<<<<<<<<<<<<
 * 
 *   cdef void c(self) except *:
 */
  __Pyx_XDECREF(__pyx_r);
  __pyx_t_1 = __Pyx_PyInt_From_grpc_op_type(GRPC_OP_SEND_INITIAL_METADATA); if (unlikely(!__pyx_t_1)) __PYX_ERR(14, 32, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_1);
  __pyx_r = __pyx_t_1;
  __pyx_t_1 = 0;
  goto __pyx_L0;

  /* "src/python/grpcio/grpc/_cython/_cygrpc/operation.pyx.pxi":31
 *     self._flags = flags
 * 
 *   def type(self):             # <<<<<<<<<<<<<<
 *     return GRPC_OP_SEND_INITIAL_METADATA
 * 
 */

  /* function exit code */
  __pyx_L1_error:;
  __Pyx_XDECREF(__pyx_t_1);
  __Pyx_AddTraceback("grpc._cython.cygrpc.SendInitialMetadataOperation.type", __pyx_clineno, __pyx_lineno, __pyx_filename);
  __pyx_r = NULL;
  __pyx_L0:;
  __Pyx_XGIVEREF(__pyx_r);
  __Pyx_RefNannyFinishContext();
  return __pyx_r;
}

/* "src/python/grpcio/grpc/_cython/_cygrpc/operation.pyx.pxi":34
 *     return GRPC_OP_SEND_INITIAL_METADATA
 * 
 *   cdef void c(self) except *:             # <<<<<<<<<<<<<<
 *     self.c_op.type = GRPC_OP_SEND_INITIAL_METADATA
 *     self.c_op.flags = self._flags
 */

static void __pyx_f_4grpc_7_cython_6cygrpc_28SendInitialMetadataOperation_c(struct __pyx_obj_4grpc_7_cython_6cygrpc_SendInitialMetadataOperation *__pyx_v_self) {
  __Pyx_RefNannyDeclarations
  int __pyx_t_1;
  PyObject *__pyx_t_2 = NULL;
  grpc_metadata *__pyx_t_3;
  size_t __pyx_t_4;
  __Pyx_RefNannySetupContext("c", 0);

  /* "src/python/grpcio/grpc/_cython/_cygrpc/operation.pyx.pxi":35
 * 
 *   cdef void c(self) except *:
 *     self.c_op.type = GRPC_OP_SEND_INITIAL_METADATA             # <<<<<<<<<<<<<<
 *     self.c_op.flags = self._flags
 *     _store_c_metadata(
 */
  __pyx_v_self->__pyx_base.c_op.op = GRPC_OP_SEND_INITIAL_METADATA;

  /* "src/python/grpcio/grpc/_cython/_cygrpc/operation.pyx.pxi":36
 *   cdef void c(self) except *:
 *     self.c_op.type = GRPC_OP_SEND_INITIAL_METADATA
 *     self.c_op.flags = self._flags             # <<<<<<<<<<<<<<
 *     _store_c_metadata(
 *         self._initial_metadata, &self._c_initial_metadata,
 */
  __pyx_t_1 = __pyx_v_self->_flags;
  __pyx_v_self->__pyx_base.c_op.flags = __pyx_t_1;

  /* "src/python/grpcio/grpc/_cython/_cygrpc/operation.pyx.pxi":38
 *     self.c_op.flags = self._flags
 *     _store_c_metadata(
 *         self._initial_metadata, &self._c_initial_metadata,             # <<<<<<<<<<<<<<
 *         &self._c_initial_metadata_count)
 *     self.c_op.data.send_initial_metadata.metadata = self._c_initial_metadata
 */
  __pyx_t_2 = __pyx_v_self->_initial_metadata;
  __Pyx_INCREF(__pyx_t_2);

  /* "src/python/grpcio/grpc/_cython/_cygrpc/operation.pyx.pxi":37
 *     self.c_op.type = GRPC_OP_SEND_INITIAL_METADATA
 *     self.c_op.flags = self._flags
 *     _store_c_metadata(             # <<<<<<<<<<<<<<
 *         self._initial_metadata, &self._c_initial_metadata,
 *         &self._c_initial_metadata_count)
 */
  __pyx_f_4grpc_7_cython_6cygrpc__store_c_metadata(__pyx_t_2, (&__pyx_v_self->_c_initial_metadata), (&__pyx_v_self->_c_initial_metadata_count)); if (unlikely(PyErr_Occurred())) __PYX_ERR(14, 37, __pyx_L1_error)
  __Pyx_DECREF(__pyx_t_2); __pyx_t_2 = 0;

  /* "src/python/grpcio/grpc/_cython/_cygrpc/operation.pyx.pxi":40
 *         self._initial_metadata, &self._c_initial_metadata,
 *         &self._c_initial_metadata_count)
 *     self.c_op.data.send_initial_metadata.metadata = self._c_initial_metadata             # <<<<<<<<<<<<<<
 *     self.c_op.data.send_initial_metadata.count = self._c_initial_metadata_count
 *     self.c_op.data.send_initial_metadata.maybe_compression_level.is_set = 0
 */
  __pyx_t_3 = __pyx_v_self->_c_initial_metadata;
  __pyx_v_self->__pyx_base.c_op.data.send_initial_metadata.metadata = __pyx_t_3;

  /* "src/python/grpcio/grpc/_cython/_cygrpc/operation.pyx.pxi":41
 *         &self._c_initial_metadata_count)
 *     self.c_op.data.send_initial_metadata.metadata = self._c_initial_metadata
 *     self.c_op.data.send_initial_metadata.count = self._c_initial_metadata_count             # <<<<<<<<<<<<<<
 *     self.c_op.data.send_initial_metadata.maybe_compression_level.is_set = 0
 * 
 */
  __pyx_t_4 = __pyx_v_self->_c_initial_metadata_count;
  __pyx_v_self->__pyx_base.c_op.data.send_initial_metadata.count = __pyx_t_4;

  /* "src/python/grpcio/grpc/_cython/_cygrpc/operation.pyx.pxi":42
 *     self.c_op.data.send_initial_metadata.metadata = self._c_initial_metadata
 *     self.c_op.data.send_initial_metadata.count = self._c_initial_metadata_count
 *     self.c_op.data.send_initial_metadata.maybe_compression_level.is_set = 0             # <<<<<<<<<<<<<<
 * 
 *   cdef void un_c(self) except *:
 */
  __pyx_v_self->__pyx_base.c_op.data.send_initial_metadata.maybe_compression_level.is_set = 0;

  /* "src/python/grpcio/grpc/_cython/_cygrpc/operation.pyx.pxi":34
 *     return GRPC_OP_SEND_INITIAL_METADATA
 * 
 *   cdef void c(self) except *:             # <<<<<<<<<<<<<<
 *     self.c_op.type = GRPC_OP_SEND_INITIAL_METADATA
 *     self.c_op.flags = self._flags
 */

  /* function exit code */
  goto __pyx_L0;
  __pyx_L1_error:;
  __Pyx_XDECREF(__pyx_t_2);
  __Pyx_AddTraceback("grpc._cython.cygrpc.SendInitialMetadataOperation.c", __pyx_clineno, __pyx_lineno, __pyx_filename);
  __pyx_L0:;
  __Pyx_RefNannyFinishContext();
}

/* "src/python/grpcio/grpc/_cython/_cygrpc/operation.pyx.pxi":44
 *     self.c_op.data.send_initial_metadata.maybe_compression_level.is_set = 0
 * 
 *   cdef void un_c(self) except *:             # <<<<<<<<<<<<<<
 *     _release_c_metadata(
 *         self._c_initial_metadata, self._c_initial_metadata_count)
 */

static void __pyx_f_4grpc_7_cython_6cygrpc_28SendInitialMetadataOperation_un_c(struct __pyx_obj_4grpc_7_cython_6cygrpc_SendInitialMetadataOperation *__pyx_v_self) {
  __Pyx_RefNannyDeclarations
  __Pyx_RefNannySetupContext("un_c", 0);

  /* "src/python/grpcio/grpc/_cython/_cygrpc/operation.pyx.pxi":45
 * 
 *   cdef void un_c(self) except *:
 *     _release_c_metadata(             # <<<<<<<<<<<<<<
 *         self._c_initial_metadata, self._c_initial_metadata_count)
 * 
 */
  __pyx_f_4grpc_7_cython_6cygrpc__release_c_metadata(__pyx_v_self->_c_initial_metadata, __pyx_v_self->_c_initial_metadata_count); if (unlikely(PyErr_Occurred())) __PYX_ERR(14, 45, __pyx_L1_error)

  /* "src/python/grpcio/grpc/_cython/_cygrpc/operation.pyx.pxi":44
 *     self.c_op.data.send_initial_metadata.maybe_compression_level.is_set = 0
 * 
 *   cdef void un_c(self) except *:             # <<<<<<<<<<<<<<
 *     _release_c_metadata(
 *         self._c_initial_metadata, self._c_initial_metadata_count)
 */

  /* function exit code */
  goto __pyx_L0;
  __pyx_L1_error:;
  __Pyx_AddTraceback("grpc._cython.cygrpc.SendInitialMetadataOperation.un_c", __pyx_clineno, __pyx_lineno, __pyx_filename);
  __pyx_L0:;
  __Pyx_RefNannyFinishContext();
}

/* "src/python/grpcio/grpc/_cython/_cygrpc/operation.pxd.pxi":27
 * cdef class SendInitialMetadataOperation(Operation):
 * 
 *   cdef readonly object _initial_metadata;             # <<<<<<<<<<<<<<
 *   cdef readonly int _flags
 *   cdef grpc_metadata *_c_initial_metadata
 */

/* Python wrapper */
static PyObject *__pyx_pw_4grpc_7_cython_6cygrpc_28SendInitialMetadataOperation_17_initial_metadata_1__get__(PyObject *__pyx_v_self); /*proto*/
static PyObject *__pyx_pw_4grpc_7_cython_6cygrpc_28SendInitialMetadataOperation_17_initial_metadata_1__get__(PyObject *__pyx_v_self) {
  PyObject *__pyx_r = 0;
  __Pyx_RefNannyDeclarations
  __Pyx_RefNannySetupContext("__get__ (wrapper)", 0);
  __pyx_r = __pyx_pf_4grpc_7_cython_6cygrpc_28SendInitialMetadataOperation_17_initial_metadata___get__(((struct __pyx_obj_4grpc_7_cython_6cygrpc_SendInitialMetadataOperation *)__pyx_v_self));

  /* function exit code */
  __Pyx_RefNannyFinishContext();
  return __pyx_r;
}

static PyObject *__pyx_pf_4grpc_7_cython_6cygrpc_28SendInitialMetadataOperation_17_initial_metadata___get__(struct __pyx_obj_4grpc_7_cython_6cygrpc_SendInitialMetadataOperation *__pyx_v_self) {
  PyObject *__pyx_r = NULL;
  __Pyx_RefNannyDeclarations
  __Pyx_RefNannySetupContext("__get__", 0);
  __Pyx_XDECREF(__pyx_r);
  __Pyx_INCREF(__pyx_v_self->_initial_metadata);
  __pyx_r = __pyx_v_self->_initial_metadata;
  goto __pyx_L0;

  /* function exit code */
  __pyx_L0:;
  __Pyx_XGIVEREF(__pyx_r);
  __Pyx_RefNannyFinishContext();
  return __pyx_r;
}

/* "src/python/grpcio/grpc/_cython/_cygrpc/operation.pxd.pxi":28
 * 
 *   cdef readonly object _initial_metadata;
 *   cdef readonly int _flags             # <<<<<<<<<<<<<<
 *   cdef grpc_metadata *_c_initial_metadata
 *   cdef size_t _c_initial_metadata_count
 */

/* Python wrapper */
static PyObject *__pyx_pw_4grpc_7_cython_6cygrpc_28SendInitialMetadataOperation_6_flags_1__get__(PyObject *__pyx_v_self); /*proto*/
static PyObject *__pyx_pw_4grpc_7_cython_6cygrpc_28SendInitialMetadataOperation_6_flags_1__get__(PyObject *__pyx_v_self) {
  PyObject *__pyx_r = 0;
  __Pyx_RefNannyDeclarations
  __Pyx_RefNannySetupContext("__get__ (wrapper)", 0);
  __pyx_r = __pyx_pf_4grpc_7_cython_6cygrpc_28SendInitialMetadataOperation_6_flags___get__(((struct __pyx_obj_4grpc_7_cython_6cygrpc_SendInitialMetadataOperation *)__pyx_v_self));

  /* function exit code */
  __Pyx_RefNannyFinishContext();
  return __pyx_r;
}

static PyObject *__pyx_pf_4grpc_7_cython_6cygrpc_28SendInitialMetadataOperation_6_flags___get__(struct __pyx_obj_4grpc_7_cython_6cygrpc_SendInitialMetadataOperation *__pyx_v_self) {
  PyObject *__pyx_r = NULL;
  __Pyx_RefNannyDeclarations
  PyObject *__pyx_t_1 = NULL;
  __Pyx_RefNannySetupContext("__get__", 0);
  __Pyx_XDECREF(__pyx_r);
  __pyx_t_1 = __Pyx_PyInt_From_int(__pyx_v_self->_flags); if (unlikely(!__pyx_t_1)) __PYX_ERR(15, 28, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_1);
  __pyx_r = __pyx_t_1;
  __pyx_t_1 = 0;
  goto __pyx_L0;

  /* function exit code */
  __pyx_L1_error:;
  __Pyx_XDECREF(__pyx_t_1);
  __Pyx_AddTraceback("grpc._cython.cygrpc.SendInitialMetadataOperation._flags.__get__", __pyx_clineno, __pyx_lineno, __pyx_filename);
  __pyx_r = NULL;
  __pyx_L0:;
  __Pyx_XGIVEREF(__pyx_r);
  __Pyx_RefNannyFinishContext();
  return __pyx_r;
}

/* "(tree fragment)":1
 * def __reduce_cython__(self):             # <<<<<<<<<<<<<<
 *     raise TypeError("no default __reduce__ due to non-trivial __cinit__")
 * def __setstate_cython__(self, __pyx_state):
 */

/* Python wrapper */
static PyObject *__pyx_pw_4grpc_7_cython_6cygrpc_28SendInitialMetadataOperation_5__reduce_cython__(PyObject *__pyx_v_self, CYTHON_UNUSED PyObject *unused); /*proto*/
static PyObject *__pyx_pw_4grpc_7_cython_6cygrpc_28SendInitialMetadataOperation_5__reduce_cython__(PyObject *__pyx_v_self, CYTHON_UNUSED PyObject *unused) {
  PyObject *__pyx_r = 0;
  __Pyx_RefNannyDeclarations
  __Pyx_RefNannySetupContext("__reduce_cython__ (wrapper)", 0);
  __pyx_r = __pyx_pf_4grpc_7_cython_6cygrpc_28SendInitialMetadataOperation_4__reduce_cython__(((struct __pyx_obj_4grpc_7_cython_6cygrpc_SendInitialMetadataOperation *)__pyx_v_self));

  /* function exit code */
  __Pyx_RefNannyFinishContext();
  return __pyx_r;
}

static PyObject *__pyx_pf_4grpc_7_cython_6cygrpc_28SendInitialMetadataOperation_4__reduce_cython__(CYTHON_UNUSED struct __pyx_obj_4grpc_7_cython_6cygrpc_SendInitialMetadataOperation *__pyx_v_self) {
  PyObject *__pyx_r = NULL;
  __Pyx_RefNannyDeclarations
  PyObject *__pyx_t_1 = NULL;
  __Pyx_RefNannySetupContext("__reduce_cython__", 0);

  /* "(tree fragment)":2
 * def __reduce_cython__(self):
 *     raise TypeError("no default __reduce__ due to non-trivial __cinit__")             # <<<<<<<<<<<<<<
 * def __setstate_cython__(self, __pyx_state):
 *     raise TypeError("no default __reduce__ due to non-trivial __cinit__")
 */
  __pyx_t_1 = __Pyx_PyObject_Call(__pyx_builtin_TypeError, __pyx_tuple__86, NULL); if (unlikely(!__pyx_t_1)) __PYX_ERR(7, 2, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_1);
  __Pyx_Raise(__pyx_t_1, 0, 0, 0);
  __Pyx_DECREF(__pyx_t_1); __pyx_t_1 = 0;
  __PYX_ERR(7, 2, __pyx_L1_error)

  /* "(tree fragment)":1
 * def __reduce_cython__(self):             # <<<<<<<<<<<<<<
 *     raise TypeError("no default __reduce__ due to non-trivial __cinit__")
 * def __setstate_cython__(self, __pyx_state):
 */

  /* function exit code */
  __pyx_L1_error:;
  __Pyx_XDECREF(__pyx_t_1);
  __Pyx_AddTraceback("grpc._cython.cygrpc.SendInitialMetadataOperation.__reduce_cython__", __pyx_clineno, __pyx_lineno, __pyx_filename);
  __pyx_r = NULL;
  __Pyx_XGIVEREF(__pyx_r);
  __Pyx_RefNannyFinishContext();
  return __pyx_r;
}

/* "(tree fragment)":3
 * def __reduce_cython__(self):
 *     raise TypeError("no default __reduce__ due to non-trivial __cinit__")
 * def __setstate_cython__(self, __pyx_state):             # <<<<<<<<<<<<<<
 *     raise TypeError("no default __reduce__ due to non-trivial __cinit__")
 */

/* Python wrapper */
static PyObject *__pyx_pw_4grpc_7_cython_6cygrpc_28SendInitialMetadataOperation_7__setstate_cython__(PyObject *__pyx_v_self, PyObject *__pyx_v___pyx_state); /*proto*/
static PyObject *__pyx_pw_4grpc_7_cython_6cygrpc_28SendInitialMetadataOperation_7__setstate_cython__(PyObject *__pyx_v_self, PyObject *__pyx_v___pyx_state) {
  PyObject *__pyx_r = 0;
  __Pyx_RefNannyDeclarations
  __Pyx_RefNannySetupContext("__setstate_cython__ (wrapper)", 0);
  __pyx_r = __pyx_pf_4grpc_7_cython_6cygrpc_28SendInitialMetadataOperation_6__setstate_cython__(((struct __pyx_obj_4grpc_7_cython_6cygrpc_SendInitialMetadataOperation *)__pyx_v_self), ((PyObject *)__pyx_v___pyx_state));

  /* function exit code */
  __Pyx_RefNannyFinishContext();
  return __pyx_r;
}

static PyObject *__pyx_pf_4grpc_7_cython_6cygrpc_28SendInitialMetadataOperation_6__setstate_cython__(CYTHON_UNUSED struct __pyx_obj_4grpc_7_cython_6cygrpc_SendInitialMetadataOperation *__pyx_v_self, CYTHON_UNUSED PyObject *__pyx_v___pyx_state) {
  PyObject *__pyx_r = NULL;
  __Pyx_RefNannyDeclarations
  PyObject *__pyx_t_1 = NULL;
  __Pyx_RefNannySetupContext("__setstate_cython__", 0);

  /* "(tree fragment)":4
 *     raise TypeError("no default __reduce__ due to non-trivial __cinit__")
 * def __setstate_cython__(self, __pyx_state):
 *     raise TypeError("no default __reduce__ due to non-trivial __cinit__")             # <<<<<<<<<<<<<<
 */
  __pyx_t_1 = __Pyx_PyObject_Call(__pyx_builtin_TypeError, __pyx_tuple__87, NULL); if (unlikely(!__pyx_t_1)) __PYX_ERR(7, 4, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_1);
  __Pyx_Raise(__pyx_t_1, 0, 0, 0);
  __Pyx_DECREF(__pyx_t_1); __pyx_t_1 = 0;
  __PYX_ERR(7, 4, __pyx_L1_error)

  /* "(tree fragment)":3
 * def __reduce_cython__(self):
 *     raise TypeError("no default __reduce__ due to non-trivial __cinit__")
 * def __setstate_cython__(self, __pyx_state):             # <<<<<<<<<<<<<<
 *     raise TypeError("no default __reduce__ due to non-trivial __cinit__")
 */

  /* function exit code */
  __pyx_L1_error:;
  __Pyx_XDECREF(__pyx_t_1);
  __Pyx_AddTraceback("grpc._cython.cygrpc.SendInitialMetadataOperation.__setstate_cython__", __pyx_clineno, __pyx_lineno, __pyx_filename);
  __pyx_r = NULL;
  __Pyx_XGIVEREF(__pyx_r);
  __Pyx_RefNannyFinishContext();
  return __pyx_r;
}

/* "src/python/grpcio/grpc/_cython/_cygrpc/operation.pyx.pxi":51
 * cdef class SendMessageOperation(Operation):
 * 
 *   def __cinit__(self, bytes message, int flags):             # <<<<<<<<<<<<<<
 *     self._message = message
 *     self._flags = flags
 */

/* Python wrapper */
static int __pyx_pw_4grpc_7_cython_6cygrpc_20SendMessageOperation_1__cinit__(PyObject *__pyx_v_self, PyObject *__pyx_args, PyObject *__pyx_kwds); /*proto*/
static int __pyx_pw_4grpc_7_cython_6cygrpc_20SendMessageOperation_1__cinit__(PyObject *__pyx_v_self, PyObject *__pyx_args, PyObject *__pyx_kwds) {
  PyObject *__pyx_v_message = 0;
  int __pyx_v_flags;
  int __pyx_r;
  __Pyx_RefNannyDeclarations
  __Pyx_RefNannySetupContext("__cinit__ (wrapper)", 0);
  {
    static PyObject **__pyx_pyargnames[] = {&__pyx_n_s_message,&__pyx_n_s_flags,0};
    PyObject* values[2] = {0,0};
    if (unlikely(__pyx_kwds)) {
      Py_ssize_t kw_args;
      const Py_ssize_t pos_args = PyTuple_GET_SIZE(__pyx_args);
      switch (pos_args) {
        case  2: values[1] = PyTuple_GET_ITEM(__pyx_args, 1);
        CYTHON_FALLTHROUGH;
        case  1: values[0] = PyTuple_GET_ITEM(__pyx_args, 0);
        CYTHON_FALLTHROUGH;
        case  0: break;
        default: goto __pyx_L5_argtuple_error;
      }
      kw_args = PyDict_Size(__pyx_kwds);
      switch (pos_args) {
        case  0:
        if (likely((values[0] = __Pyx_PyDict_GetItemStr(__pyx_kwds, __pyx_n_s_message)) != 0)) kw_args--;
        else goto __pyx_L5_argtuple_error;
        CYTHON_FALLTHROUGH;
        case  1:
        if (likely((values[1] = __Pyx_PyDict_GetItemStr(__pyx_kwds, __pyx_n_s_flags)) != 0)) kw_args--;
        else {
          __Pyx_RaiseArgtupleInvalid("__cinit__", 1, 2, 2, 1); __PYX_ERR(14, 51, __pyx_L3_error)
        }
      }
      if (unlikely(kw_args > 0)) {
        if (unlikely(__Pyx_ParseOptionalKeywords(__pyx_kwds, __pyx_pyargnames, 0, values, pos_args, "__cinit__") < 0)) __PYX_ERR(14, 51, __pyx_L3_error)
      }
    } else if (PyTuple_GET_SIZE(__pyx_args) != 2) {
      goto __pyx_L5_argtuple_error;
    } else {
      values[0] = PyTuple_GET_ITEM(__pyx_args, 0);
      values[1] = PyTuple_GET_ITEM(__pyx_args, 1);
    }
    __pyx_v_message = ((PyObject*)values[0]);
    __pyx_v_flags = __Pyx_PyInt_As_int(values[1]); if (unlikely((__pyx_v_flags == (int)-1) && PyErr_Occurred())) __PYX_ERR(14, 51, __pyx_L3_error)
  }
  goto __pyx_L4_argument_unpacking_done;
  __pyx_L5_argtuple_error:;
  __Pyx_RaiseArgtupleInvalid("__cinit__", 1, 2, 2, PyTuple_GET_SIZE(__pyx_args)); __PYX_ERR(14, 51, __pyx_L3_error)
  __pyx_L3_error:;
  __Pyx_AddTraceback("grpc._cython.cygrpc.SendMessageOperation.__cinit__", __pyx_clineno, __pyx_lineno, __pyx_filename);
  __Pyx_RefNannyFinishContext();
  return -1;
  __pyx_L4_argument_unpacking_done:;
  if (unlikely(!__Pyx_ArgTypeTest(((PyObject *)__pyx_v_message), (&PyBytes_Type), 1, "message", 1))) __PYX_ERR(14, 51, __pyx_L1_error)
  __pyx_r = __pyx_pf_4grpc_7_cython_6cygrpc_20SendMessageOperation___cinit__(((struct __pyx_obj_4grpc_7_cython_6cygrpc_SendMessageOperation *)__pyx_v_self), __pyx_v_message, __pyx_v_flags);

  /* function exit code */
  goto __pyx_L0;
  __pyx_L1_error:;
  __pyx_r = -1;
  __pyx_L0:;
  __Pyx_RefNannyFinishContext();
  return __pyx_r;
}

static int __pyx_pf_4grpc_7_cython_6cygrpc_20SendMessageOperation___cinit__(struct __pyx_obj_4grpc_7_cython_6cygrpc_SendMessageOperation *__pyx_v_self, PyObject *__pyx_v_message, int __pyx_v_flags) {
  int __pyx_r;
  __Pyx_RefNannyDeclarations
  __Pyx_RefNannySetupContext("__cinit__", 0);

  /* "src/python/grpcio/grpc/_cython/_cygrpc/operation.pyx.pxi":52
 * 
 *   def __cinit__(self, bytes message, int flags):
 *     self._message = message             # <<<<<<<<<<<<<<
 *     self._flags = flags
 * 
 */
  __Pyx_INCREF(__pyx_v_message);
  __Pyx_GIVEREF(__pyx_v_message);
  __Pyx_GOTREF(__pyx_v_self->_message);
  __Pyx_DECREF(__pyx_v_self->_message);
  __pyx_v_self->_message = __pyx_v_message;

  /* "src/python/grpcio/grpc/_cython/_cygrpc/operation.pyx.pxi":53
 *   def __cinit__(self, bytes message, int flags):
 *     self._message = message
 *     self._flags = flags             # <<<<<<<<<<<<<<
 * 
 *   def type(self):
 */
  __pyx_v_self->_flags = __pyx_v_flags;

  /* "src/python/grpcio/grpc/_cython/_cygrpc/operation.pyx.pxi":51
 * cdef class SendMessageOperation(Operation):
 * 
 *   def __cinit__(self, bytes message, int flags):             # <<<<<<<<<<<<<<
 *     self._message = message
 *     self._flags = flags
 */

  /* function exit code */
  __pyx_r = 0;
  __Pyx_RefNannyFinishContext();
  return __pyx_r;
}

/* "src/python/grpcio/grpc/_cython/_cygrpc/operation.pyx.pxi":55
 *     self._flags = flags
 * 
 *   def type(self):             # <<<<<<<<<<<<<<
 *     return GRPC_OP_SEND_MESSAGE
 * 
 */

/* Python wrapper */
static PyObject *__pyx_pw_4grpc_7_cython_6cygrpc_20SendMessageOperation_3type(PyObject *__pyx_v_self, CYTHON_UNUSED PyObject *unused); /*proto*/
static PyObject *__pyx_pw_4grpc_7_cython_6cygrpc_20SendMessageOperation_3type(PyObject *__pyx_v_self, CYTHON_UNUSED PyObject *unused) {
  PyObject *__pyx_r = 0;
  __Pyx_RefNannyDeclarations
  __Pyx_RefNannySetupContext("type (wrapper)", 0);
  __pyx_r = __pyx_pf_4grpc_7_cython_6cygrpc_20SendMessageOperation_2type(((struct __pyx_obj_4grpc_7_cython_6cygrpc_SendMessageOperation *)__pyx_v_self));

  /* function exit code */
  __Pyx_RefNannyFinishContext();
  return __pyx_r;
}

static PyObject *__pyx_pf_4grpc_7_cython_6cygrpc_20SendMessageOperation_2type(CYTHON_UNUSED struct __pyx_obj_4grpc_7_cython_6cygrpc_SendMessageOperation *__pyx_v_self) {
  PyObject *__pyx_r = NULL;
  __Pyx_RefNannyDeclarations
  PyObject *__pyx_t_1 = NULL;
  __Pyx_RefNannySetupContext("type", 0);

  /* "src/python/grpcio/grpc/_cython/_cygrpc/operation.pyx.pxi":56
 * 
 *   def type(self):
 *     return GRPC_OP_SEND_MESSAGE             # <<<<<<<<<<<<<<
 * 
 *   cdef void c(self) except *:
 */
  __Pyx_XDECREF(__pyx_r);
  __pyx_t_1 = __Pyx_PyInt_From_grpc_op_type(GRPC_OP_SEND_MESSAGE); if (unlikely(!__pyx_t_1)) __PYX_ERR(14, 56, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_1);
  __pyx_r = __pyx_t_1;
  __pyx_t_1 = 0;
  goto __pyx_L0;

  /* "src/python/grpcio/grpc/_cython/_cygrpc/operation.pyx.pxi":55
 *     self._flags = flags
 * 
 *   def type(self):             # <<<<<<<<<<<<<<
 *     return GRPC_OP_SEND_MESSAGE
 * 
 */

  /* function exit code */
  __pyx_L1_error:;
  __Pyx_XDECREF(__pyx_t_1);
  __Pyx_AddTraceback("grpc._cython.cygrpc.SendMessageOperation.type", __pyx_clineno, __pyx_lineno, __pyx_filename);
  __pyx_r = NULL;
  __pyx_L0:;
  __Pyx_XGIVEREF(__pyx_r);
  __Pyx_RefNannyFinishContext();
  return __pyx_r;
}

/* "src/python/grpcio/grpc/_cython/_cygrpc/operation.pyx.pxi":58
 *     return GRPC_OP_SEND_MESSAGE
 * 
 *   cdef void c(self) except *:             # <<<<<<<<<<<<<<
 *     self.c_op.type = GRPC_OP_SEND_MESSAGE
 *     self.c_op.flags = self._flags
 */

static void __pyx_f_4grpc_7_cython_6cygrpc_20SendMessageOperation_c(struct __pyx_obj_4grpc_7_cython_6cygrpc_SendMessageOperation *__pyx_v_self) {
  grpc_slice __pyx_v_message_slice;
  __Pyx_RefNannyDeclarations
  int __pyx_t_1;
  char const *__pyx_t_2;
  PyObject *__pyx_t_3 = NULL;
  Py_ssize_t __pyx_t_4;
  grpc_byte_buffer *__pyx_t_5;
  __Pyx_RefNannySetupContext("c", 0);

  /* "src/python/grpcio/grpc/_cython/_cygrpc/operation.pyx.pxi":59
 * 
 *   cdef void c(self) except *:
 *     self.c_op.type = GRPC_OP_SEND_MESSAGE             # <<<<<<<<<<<<<<
 *     self.c_op.flags = self._flags
 *     cdef grpc_slice message_slice = grpc_slice_from_copied_buffer(
 */
  __pyx_v_self->__pyx_base.c_op.op = GRPC_OP_SEND_MESSAGE;

  /* "src/python/grpcio/grpc/_cython/_cygrpc/operation.pyx.pxi":60
 *   cdef void c(self) except *:
 *     self.c_op.type = GRPC_OP_SEND_MESSAGE
 *     self.c_op.flags = self._flags             # <<<<<<<<<<<<<<
 *     cdef grpc_slice message_slice = grpc_slice_from_copied_buffer(
 *         self._message, len(self._message))
 */
  __pyx_t_1 = __pyx_v_self->_flags;
  __pyx_v_self->__pyx_base.c_op.flags = __pyx_t_1;

  /* "src/python/grpcio/grpc/_cython/_cygrpc/operation.pyx.pxi":62
 *     self.c_op.flags = self._flags
 *     cdef grpc_slice message_slice = grpc_slice_from_copied_buffer(
 *         self._message, len(self._message))             # <<<<<<<<<<<<<<
 *     self._c_message_byte_buffer = grpc_raw_byte_buffer_create(
 *         &message_slice, 1)
 */
  if (unlikely(__pyx_v_self->_message == Py_None)) {
    PyErr_SetString(PyExc_TypeError, "expected bytes, NoneType found");
    __PYX_ERR(14, 62, __pyx_L1_error)
  }
  __pyx_t_2 = __Pyx_PyBytes_AsString(__pyx_v_self->_message); if (unlikely((!__pyx_t_2) && PyErr_Occurred())) __PYX_ERR(14, 62, __pyx_L1_error)
  __pyx_t_3 = __pyx_v_self->_message;
  __Pyx_INCREF(__pyx_t_3);
  if (unlikely(__pyx_t_3 == Py_None)) {
    PyErr_SetString(PyExc_TypeError, "object of type 'NoneType' has no len()");
    __PYX_ERR(14, 62, __pyx_L1_error)
  }
  __pyx_t_4 = PyBytes_GET_SIZE(__pyx_t_3); if (unlikely(__pyx_t_4 == ((Py_ssize_t)-1))) __PYX_ERR(14, 62, __pyx_L1_error)
  __Pyx_DECREF(__pyx_t_3); __pyx_t_3 = 0;

  /* "src/python/grpcio/grpc/_cython/_cygrpc/operation.pyx.pxi":61
 *     self.c_op.type = GRPC_OP_SEND_MESSAGE
 *     self.c_op.flags = self._flags
 *     cdef grpc_slice message_slice = grpc_slice_from_copied_buffer(             # <<<<<<<<<<<<<<
 *         self._message, len(self._message))
 *     self._c_message_byte_buffer = grpc_raw_byte_buffer_create(
 */
  __pyx_v_message_slice = grpc_slice_from_copied_buffer(__pyx_t_2, __pyx_t_4);

  /* "src/python/grpcio/grpc/_cython/_cygrpc/operation.pyx.pxi":63
 *     cdef grpc_slice message_slice = grpc_slice_from_copied_buffer(
 *         self._message, len(self._message))
 *     self._c_message_byte_buffer = grpc_raw_byte_buffer_create(             # <<<<<<<<<<<<<<
 *         &message_slice, 1)
 *     grpc_slice_unref(message_slice)
 */
  __pyx_v_self->_c_message_byte_buffer = grpc_raw_byte_buffer_create((&__pyx_v_message_slice), 1);

  /* "src/python/grpcio/grpc/_cython/_cygrpc/operation.pyx.pxi":65
 *     self._c_message_byte_buffer = grpc_raw_byte_buffer_create(
 *         &message_slice, 1)
 *     grpc_slice_unref(message_slice)             # <<<<<<<<<<<<<<
 *     self.c_op.data.send_message.send_message = self._c_message_byte_buffer
 * 
 */
  grpc_slice_unref(__pyx_v_message_slice);

  /* "src/python/grpcio/grpc/_cython/_cygrpc/operation.pyx.pxi":66
 *         &message_slice, 1)
 *     grpc_slice_unref(message_slice)
 *     self.c_op.data.send_message.send_message = self._c_message_byte_buffer             # <<<<<<<<<<<<<<
 * 
 *   cdef void un_c(self) except *:
 */
  __pyx_t_5 = __pyx_v_self->_c_message_byte_buffer;
  __pyx_v_self->__pyx_base.c_op.data.send_message.send_message = __pyx_t_5;

  /* "src/python/grpcio/grpc/_cython/_cygrpc/operation.pyx.pxi":58
 *     return GRPC_OP_SEND_MESSAGE
 * 
 *   cdef void c(self) except *:             # <<<<<<<<<<<<<<
 *     self.c_op.type = GRPC_OP_SEND_MESSAGE
 *     self.c_op.flags = self._flags
 */

  /* function exit code */
  goto __pyx_L0;
  __pyx_L1_error:;
  __Pyx_XDECREF(__pyx_t_3);
  __Pyx_AddTraceback("grpc._cython.cygrpc.SendMessageOperation.c", __pyx_clineno, __pyx_lineno, __pyx_filename);
  __pyx_L0:;
  __Pyx_RefNannyFinishContext();
}

/* "src/python/grpcio/grpc/_cython/_cygrpc/operation.pyx.pxi":68
 *     self.c_op.data.send_message.send_message = self._c_message_byte_buffer
 * 
 *   cdef void un_c(self) except *:             # <<<<<<<<<<<<<<
 *     grpc_byte_buffer_destroy(self._c_message_byte_buffer)
 * 
 */

static void __pyx_f_4grpc_7_cython_6cygrpc_20SendMessageOperation_un_c(struct __pyx_obj_4grpc_7_cython_6cygrpc_SendMessageOperation *__pyx_v_self) {
  __Pyx_RefNannyDeclarations
  __Pyx_RefNannySetupContext("un_c", 0);

  /* "src/python/grpcio/grpc/_cython/_cygrpc/operation.pyx.pxi":69
 * 
 *   cdef void un_c(self) except *:
 *     grpc_byte_buffer_destroy(self._c_message_byte_buffer)             # <<<<<<<<<<<<<<
 * 
 * 
 */
  grpc_byte_buffer_destroy(__pyx_v_self->_c_message_byte_buffer);

  /* "src/python/grpcio/grpc/_cython/_cygrpc/operation.pyx.pxi":68
 *     self.c_op.data.send_message.send_message = self._c_message_byte_buffer
 * 
 *   cdef void un_c(self) except *:             # <<<<<<<<<<<<<<
 *     grpc_byte_buffer_destroy(self._c_message_byte_buffer)
 * 
 */

  /* function exit code */
  __Pyx_RefNannyFinishContext();
}

/* "src/python/grpcio/grpc/_cython/_cygrpc/operation.pxd.pxi":38
 * cdef class SendMessageOperation(Operation):
 * 
 *   cdef readonly bytes _message             # <<<<<<<<<<<<<<
 *   cdef readonly int _flags
 *   cdef grpc_byte_buffer *_c_message_byte_buffer
 */

/* Python wrapper */
static PyObject *__pyx_pw_4grpc_7_cython_6cygrpc_20SendMessageOperation_8_message_1__get__(PyObject *__pyx_v_self); /*proto*/
static PyObject *__pyx_pw_4grpc_7_cython_6cygrpc_20SendMessageOperation_8_message_1__get__(PyObject *__pyx_v_self) {
  PyObject *__pyx_r = 0;
  __Pyx_RefNannyDeclarations
  __Pyx_RefNannySetupContext("__get__ (wrapper)", 0);
  __pyx_r = __pyx_pf_4grpc_7_cython_6cygrpc_20SendMessageOperation_8_message___get__(((struct __pyx_obj_4grpc_7_cython_6cygrpc_SendMessageOperation *)__pyx_v_self));

  /* function exit code */
  __Pyx_RefNannyFinishContext();
  return __pyx_r;
}

static PyObject *__pyx_pf_4grpc_7_cython_6cygrpc_20SendMessageOperation_8_message___get__(struct __pyx_obj_4grpc_7_cython_6cygrpc_SendMessageOperation *__pyx_v_self) {
  PyObject *__pyx_r = NULL;
  __Pyx_RefNannyDeclarations
  __Pyx_RefNannySetupContext("__get__", 0);
  __Pyx_XDECREF(__pyx_r);
  __Pyx_INCREF(__pyx_v_self->_message);
  __pyx_r = __pyx_v_self->_message;
  goto __pyx_L0;

  /* function exit code */
  __pyx_L0:;
  __Pyx_XGIVEREF(__pyx_r);
  __Pyx_RefNannyFinishContext();
  return __pyx_r;
}

/* "src/python/grpcio/grpc/_cython/_cygrpc/operation.pxd.pxi":39
 * 
 *   cdef readonly bytes _message
 *   cdef readonly int _flags             # <<<<<<<<<<<<<<
 *   cdef grpc_byte_buffer *_c_message_byte_buffer
 * 
 */

/* Python wrapper */
static PyObject *__pyx_pw_4grpc_7_cython_6cygrpc_20SendMessageOperation_6_flags_1__get__(PyObject *__pyx_v_self); /*proto*/
static PyObject *__pyx_pw_4grpc_7_cython_6cygrpc_20SendMessageOperation_6_flags_1__get__(PyObject *__pyx_v_self) {
  PyObject *__pyx_r = 0;
  __Pyx_RefNannyDeclarations
  __Pyx_RefNannySetupContext("__get__ (wrapper)", 0);
  __pyx_r = __pyx_pf_4grpc_7_cython_6cygrpc_20SendMessageOperation_6_flags___get__(((struct __pyx_obj_4grpc_7_cython_6cygrpc_SendMessageOperation *)__pyx_v_self));

  /* function exit code */
  __Pyx_RefNannyFinishContext();
  return __pyx_r;
}

static PyObject *__pyx_pf_4grpc_7_cython_6cygrpc_20SendMessageOperation_6_flags___get__(struct __pyx_obj_4grpc_7_cython_6cygrpc_SendMessageOperation *__pyx_v_self) {
  PyObject *__pyx_r = NULL;
  __Pyx_RefNannyDeclarations
  PyObject *__pyx_t_1 = NULL;
  __Pyx_RefNannySetupContext("__get__", 0);
  __Pyx_XDECREF(__pyx_r);
  __pyx_t_1 = __Pyx_PyInt_From_int(__pyx_v_self->_flags); if (unlikely(!__pyx_t_1)) __PYX_ERR(15, 39, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_1);
  __pyx_r = __pyx_t_1;
  __pyx_t_1 = 0;
  goto __pyx_L0;

  /* function exit code */
  __pyx_L1_error:;
  __Pyx_XDECREF(__pyx_t_1);
  __Pyx_AddTraceback("grpc._cython.cygrpc.SendMessageOperation._flags.__get__", __pyx_clineno, __pyx_lineno, __pyx_filename);
  __pyx_r = NULL;
  __pyx_L0:;
  __Pyx_XGIVEREF(__pyx_r);
  __Pyx_RefNannyFinishContext();
  return __pyx_r;
}

/* "(tree fragment)":1
 * def __reduce_cython__(self):             # <<<<<<<<<<<<<<
 *     raise TypeError("no default __reduce__ due to non-trivial __cinit__")
 * def __setstate_cython__(self, __pyx_state):
 */

/* Python wrapper */
static PyObject *__pyx_pw_4grpc_7_cython_6cygrpc_20SendMessageOperation_5__reduce_cython__(PyObject *__pyx_v_self, CYTHON_UNUSED PyObject *unused); /*proto*/
static PyObject *__pyx_pw_4grpc_7_cython_6cygrpc_20SendMessageOperation_5__reduce_cython__(PyObject *__pyx_v_self, CYTHON_UNUSED PyObject *unused) {
  PyObject *__pyx_r = 0;
  __Pyx_RefNannyDeclarations
  __Pyx_RefNannySetupContext("__reduce_cython__ (wrapper)", 0);
  __pyx_r = __pyx_pf_4grpc_7_cython_6cygrpc_20SendMessageOperation_4__reduce_cython__(((struct __pyx_obj_4grpc_7_cython_6cygrpc_SendMessageOperation *)__pyx_v_self));

  /* function exit code */
  __Pyx_RefNannyFinishContext();
  return __pyx_r;
}

static PyObject *__pyx_pf_4grpc_7_cython_6cygrpc_20SendMessageOperation_4__reduce_cython__(CYTHON_UNUSED struct __pyx_obj_4grpc_7_cython_6cygrpc_SendMessageOperation *__pyx_v_self) {
  PyObject *__pyx_r = NULL;
  __Pyx_RefNannyDeclarations
  PyObject *__pyx_t_1 = NULL;
  __Pyx_RefNannySetupContext("__reduce_cython__", 0);

  /* "(tree fragment)":2
 * def __reduce_cython__(self):
 *     raise TypeError("no default __reduce__ due to non-trivial __cinit__")             # <<<<<<<<<<<<<<
 * def __setstate_cython__(self, __pyx_state):
 *     raise TypeError("no default __reduce__ due to non-trivial __cinit__")
 */
  __pyx_t_1 = __Pyx_PyObject_Call(__pyx_builtin_TypeError, __pyx_tuple__88, NULL); if (unlikely(!__pyx_t_1)) __PYX_ERR(7, 2, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_1);
  __Pyx_Raise(__pyx_t_1, 0, 0, 0);
  __Pyx_DECREF(__pyx_t_1); __pyx_t_1 = 0;
  __PYX_ERR(7, 2, __pyx_L1_error)

  /* "(tree fragment)":1
 * def __reduce_cython__(self):             # <<<<<<<<<<<<<<
 *     raise TypeError("no default __reduce__ due to non-trivial __cinit__")
 * def __setstate_cython__(self, __pyx_state):
 */

  /* function exit code */
  __pyx_L1_error:;
  __Pyx_XDECREF(__pyx_t_1);
  __Pyx_AddTraceback("grpc._cython.cygrpc.SendMessageOperation.__reduce_cython__", __pyx_clineno, __pyx_lineno, __pyx_filename);
  __pyx_r = NULL;
  __Pyx_XGIVEREF(__pyx_r);
  __Pyx_RefNannyFinishContext();
  return __pyx_r;
}

/* "(tree fragment)":3
 * def __reduce_cython__(self):
 *     raise TypeError("no default __reduce__ due to non-trivial __cinit__")
 * def __setstate_cython__(self, __pyx_state):             # <<<<<<<<<<<<<<
 *     raise TypeError("no default __reduce__ due to non-trivial __cinit__")
 */

/* Python wrapper */
static PyObject *__pyx_pw_4grpc_7_cython_6cygrpc_20SendMessageOperation_7__setstate_cython__(PyObject *__pyx_v_self, PyObject *__pyx_v___pyx_state); /*proto*/
static PyObject *__pyx_pw_4grpc_7_cython_6cygrpc_20SendMessageOperation_7__setstate_cython__(PyObject *__pyx_v_self, PyObject *__pyx_v___pyx_state) {
  PyObject *__pyx_r = 0;
  __Pyx_RefNannyDeclarations
  __Pyx_RefNannySetupContext("__setstate_cython__ (wrapper)", 0);
  __pyx_r = __pyx_pf_4grpc_7_cython_6cygrpc_20SendMessageOperation_6__setstate_cython__(((struct __pyx_obj_4grpc_7_cython_6cygrpc_SendMessageOperation *)__pyx_v_self), ((PyObject *)__pyx_v___pyx_state));

  /* function exit code */
  __Pyx_RefNannyFinishContext();
  return __pyx_r;
}

static PyObject *__pyx_pf_4grpc_7_cython_6cygrpc_20SendMessageOperation_6__setstate_cython__(CYTHON_UNUSED struct __pyx_obj_4grpc_7_cython_6cygrpc_SendMessageOperation *__pyx_v_self, CYTHON_UNUSED PyObject *__pyx_v___pyx_state) {
  PyObject *__pyx_r = NULL;
  __Pyx_RefNannyDeclarations
  PyObject *__pyx_t_1 = NULL;
  __Pyx_RefNannySetupContext("__setstate_cython__", 0);

  /* "(tree fragment)":4
 *     raise TypeError("no default __reduce__ due to non-trivial __cinit__")
 * def __setstate_cython__(self, __pyx_state):
 *     raise TypeError("no default __reduce__ due to non-trivial __cinit__")             # <<<<<<<<<<<<<<
 */
  __pyx_t_1 = __Pyx_PyObject_Call(__pyx_builtin_TypeError, __pyx_tuple__89, NULL); if (unlikely(!__pyx_t_1)) __PYX_ERR(7, 4, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_1);
  __Pyx_Raise(__pyx_t_1, 0, 0, 0);
  __Pyx_DECREF(__pyx_t_1); __pyx_t_1 = 0;
  __PYX_ERR(7, 4, __pyx_L1_error)

  /* "(tree fragment)":3
 * def __reduce_cython__(self):
 *     raise TypeError("no default __reduce__ due to non-trivial __cinit__")
 * def __setstate_cython__(self, __pyx_state):             # <<<<<<<<<<<<<<
 *     raise TypeError("no default __reduce__ due to non-trivial __cinit__")
 */

  /* function exit code */
  __pyx_L1_error:;
  __Pyx_XDECREF(__pyx_t_1);
  __Pyx_AddTraceback("grpc._cython.cygrpc.SendMessageOperation.__setstate_cython__", __pyx_clineno, __pyx_lineno, __pyx_filename);
  __pyx_r = NULL;
  __Pyx_XGIVEREF(__pyx_r);
  __Pyx_RefNannyFinishContext();
  return __pyx_r;
}

/* "src/python/grpcio/grpc/_cython/_cygrpc/operation.pyx.pxi":74
 * cdef class SendCloseFromClientOperation(Operation):
 * 
 *   def __cinit__(self, int flags):             # <<<<<<<<<<<<<<
 *     self._flags = flags
 * 
 */

/* Python wrapper */
static int __pyx_pw_4grpc_7_cython_6cygrpc_28SendCloseFromClientOperation_1__cinit__(PyObject *__pyx_v_self, PyObject *__pyx_args, PyObject *__pyx_kwds); /*proto*/
static int __pyx_pw_4grpc_7_cython_6cygrpc_28SendCloseFromClientOperation_1__cinit__(PyObject *__pyx_v_self, PyObject *__pyx_args, PyObject *__pyx_kwds) {
  int __pyx_v_flags;
  int __pyx_r;
  __Pyx_RefNannyDeclarations
  __Pyx_RefNannySetupContext("__cinit__ (wrapper)", 0);
  {
    static PyObject **__pyx_pyargnames[] = {&__pyx_n_s_flags,0};
    PyObject* values[1] = {0};
    if (unlikely(__pyx_kwds)) {
      Py_ssize_t kw_args;
      const Py_ssize_t pos_args = PyTuple_GET_SIZE(__pyx_args);
      switch (pos_args) {
        case  1: values[0] = PyTuple_GET_ITEM(__pyx_args, 0);
        CYTHON_FALLTHROUGH;
        case  0: break;
        default: goto __pyx_L5_argtuple_error;
      }
      kw_args = PyDict_Size(__pyx_kwds);
      switch (pos_args) {
        case  0:
        if (likely((values[0] = __Pyx_PyDict_GetItemStr(__pyx_kwds, __pyx_n_s_flags)) != 0)) kw_args--;
        else goto __pyx_L5_argtuple_error;
      }
      if (unlikely(kw_args > 0)) {
        if (unlikely(__Pyx_ParseOptionalKeywords(__pyx_kwds, __pyx_pyargnames, 0, values, pos_args, "__cinit__") < 0)) __PYX_ERR(14, 74, __pyx_L3_error)
      }
    } else if (PyTuple_GET_SIZE(__pyx_args) != 1) {
      goto __pyx_L5_argtuple_error;
    } else {
      values[0] = PyTuple_GET_ITEM(__pyx_args, 0);
    }
    __pyx_v_flags = __Pyx_PyInt_As_int(values[0]); if (unlikely((__pyx_v_flags == (int)-1) && PyErr_Occurred())) __PYX_ERR(14, 74, __pyx_L3_error)
  }
  goto __pyx_L4_argument_unpacking_done;
  __pyx_L5_argtuple_error:;
  __Pyx_RaiseArgtupleInvalid("__cinit__", 1, 1, 1, PyTuple_GET_SIZE(__pyx_args)); __PYX_ERR(14, 74, __pyx_L3_error)
  __pyx_L3_error:;
  __Pyx_AddTraceback("grpc._cython.cygrpc.SendCloseFromClientOperation.__cinit__", __pyx_clineno, __pyx_lineno, __pyx_filename);
  __Pyx_RefNannyFinishContext();
  return -1;
  __pyx_L4_argument_unpacking_done:;
  __pyx_r = __pyx_pf_4grpc_7_cython_6cygrpc_28SendCloseFromClientOperation___cinit__(((struct __pyx_obj_4grpc_7_cython_6cygrpc_SendCloseFromClientOperation *)__pyx_v_self), __pyx_v_flags);

  /* function exit code */
  __Pyx_RefNannyFinishContext();
  return __pyx_r;
}

static int __pyx_pf_4grpc_7_cython_6cygrpc_28SendCloseFromClientOperation___cinit__(struct __pyx_obj_4grpc_7_cython_6cygrpc_SendCloseFromClientOperation *__pyx_v_self, int __pyx_v_flags) {
  int __pyx_r;
  __Pyx_RefNannyDeclarations
  __Pyx_RefNannySetupContext("__cinit__", 0);

  /* "src/python/grpcio/grpc/_cython/_cygrpc/operation.pyx.pxi":75
 * 
 *   def __cinit__(self, int flags):
 *     self._flags = flags             # <<<<<<<<<<<<<<
 * 
 *   def type(self):
 */
  __pyx_v_self->_flags = __pyx_v_flags;

  /* "src/python/grpcio/grpc/_cython/_cygrpc/operation.pyx.pxi":74
 * cdef class SendCloseFromClientOperation(Operation):
 * 
 *   def __cinit__(self, int flags):             # <<<<<<<<<<<<<<
 *     self._flags = flags
 * 
 */

  /* function exit code */
  __pyx_r = 0;
  __Pyx_RefNannyFinishContext();
  return __pyx_r;
}

/* "src/python/grpcio/grpc/_cython/_cygrpc/operation.pyx.pxi":77
 *     self._flags = flags
 * 
 *   def type(self):             # <<<<<<<<<<<<<<
 *     return GRPC_OP_SEND_CLOSE_FROM_CLIENT
 * 
 */

/* Python wrapper */
static PyObject *__pyx_pw_4grpc_7_cython_6cygrpc_28SendCloseFromClientOperation_3type(PyObject *__pyx_v_self, CYTHON_UNUSED PyObject *unused); /*proto*/
static PyObject *__pyx_pw_4grpc_7_cython_6cygrpc_28SendCloseFromClientOperation_3type(PyObject *__pyx_v_self, CYTHON_UNUSED PyObject *unused) {
  PyObject *__pyx_r = 0;
  __Pyx_RefNannyDeclarations
  __Pyx_RefNannySetupContext("type (wrapper)", 0);
  __pyx_r = __pyx_pf_4grpc_7_cython_6cygrpc_28SendCloseFromClientOperation_2type(((struct __pyx_obj_4grpc_7_cython_6cygrpc_SendCloseFromClientOperation *)__pyx_v_self));

  /* function exit code */
  __Pyx_RefNannyFinishContext();
  return __pyx_r;
}

static PyObject *__pyx_pf_4grpc_7_cython_6cygrpc_28SendCloseFromClientOperation_2type(CYTHON_UNUSED struct __pyx_obj_4grpc_7_cython_6cygrpc_SendCloseFromClientOperation *__pyx_v_self) {
  PyObject *__pyx_r = NULL;
  __Pyx_RefNannyDeclarations
  PyObject *__pyx_t_1 = NULL;
  __Pyx_RefNannySetupContext("type", 0);

  /* "src/python/grpcio/grpc/_cython/_cygrpc/operation.pyx.pxi":78
 * 
 *   def type(self):
 *     return GRPC_OP_SEND_CLOSE_FROM_CLIENT             # <<<<<<<<<<<<<<
 * 
 *   cdef void c(self) except *:
 */
  __Pyx_XDECREF(__pyx_r);
  __pyx_t_1 = __Pyx_PyInt_From_grpc_op_type(GRPC_OP_SEND_CLOSE_FROM_CLIENT); if (unlikely(!__pyx_t_1)) __PYX_ERR(14, 78, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_1);
  __pyx_r = __pyx_t_1;
  __pyx_t_1 = 0;
  goto __pyx_L0;

  /* "src/python/grpcio/grpc/_cython/_cygrpc/operation.pyx.pxi":77
 *     self._flags = flags
 * 
 *   def type(self):             # <<<<<<<<<<<<<<
 *     return GRPC_OP_SEND_CLOSE_FROM_CLIENT
 * 
 */

  /* function exit code */
  __pyx_L1_error:;
  __Pyx_XDECREF(__pyx_t_1);
  __Pyx_AddTraceback("grpc._cython.cygrpc.SendCloseFromClientOperation.type", __pyx_clineno, __pyx_lineno, __pyx_filename);
  __pyx_r = NULL;
  __pyx_L0:;
  __Pyx_XGIVEREF(__pyx_r);
  __Pyx_RefNannyFinishContext();
  return __pyx_r;
}

/* "src/python/grpcio/grpc/_cython/_cygrpc/operation.pyx.pxi":80
 *     return GRPC_OP_SEND_CLOSE_FROM_CLIENT
 * 
 *   cdef void c(self) except *:             # <<<<<<<<<<<<<<
 *     self.c_op.type = GRPC_OP_SEND_CLOSE_FROM_CLIENT
 *     self.c_op.flags = self._flags
 */

static void __pyx_f_4grpc_7_cython_6cygrpc_28SendCloseFromClientOperation_c(struct __pyx_obj_4grpc_7_cython_6cygrpc_SendCloseFromClientOperation *__pyx_v_self) {
  __Pyx_RefNannyDeclarations
  int __pyx_t_1;
  __Pyx_RefNannySetupContext("c", 0);

  /* "src/python/grpcio/grpc/_cython/_cygrpc/operation.pyx.pxi":81
 * 
 *   cdef void c(self) except *:
 *     self.c_op.type = GRPC_OP_SEND_CLOSE_FROM_CLIENT             # <<<<<<<<<<<<<<
 *     self.c_op.flags = self._flags
 * 
 */
  __pyx_v_self->__pyx_base.c_op.op = GRPC_OP_SEND_CLOSE_FROM_CLIENT;

  /* "src/python/grpcio/grpc/_cython/_cygrpc/operation.pyx.pxi":82
 *   cdef void c(self) except *:
 *     self.c_op.type = GRPC_OP_SEND_CLOSE_FROM_CLIENT
 *     self.c_op.flags = self._flags             # <<<<<<<<<<<<<<
 * 
 *   cdef void un_c(self) except *:
 */
  __pyx_t_1 = __pyx_v_self->_flags;
  __pyx_v_self->__pyx_base.c_op.flags = __pyx_t_1;

  /* "src/python/grpcio/grpc/_cython/_cygrpc/operation.pyx.pxi":80
 *     return GRPC_OP_SEND_CLOSE_FROM_CLIENT
 * 
 *   cdef void c(self) except *:             # <<<<<<<<<<<<<<
 *     self.c_op.type = GRPC_OP_SEND_CLOSE_FROM_CLIENT
 *     self.c_op.flags = self._flags
 */

  /* function exit code */
  __Pyx_RefNannyFinishContext();
}

/* "src/python/grpcio/grpc/_cython/_cygrpc/operation.pyx.pxi":84
 *     self.c_op.flags = self._flags
 * 
 *   cdef void un_c(self) except *:             # <<<<<<<<<<<<<<
 *     pass
 * 
 */

static void __pyx_f_4grpc_7_cython_6cygrpc_28SendCloseFromClientOperation_un_c(CYTHON_UNUSED struct __pyx_obj_4grpc_7_cython_6cygrpc_SendCloseFromClientOperation *__pyx_v_self) {
  __Pyx_RefNannyDeclarations
  __Pyx_RefNannySetupContext("un_c", 0);

  /* function exit code */
  __Pyx_RefNannyFinishContext();
}

/* "src/python/grpcio/grpc/_cython/_cygrpc/operation.pxd.pxi":48
 * cdef class SendCloseFromClientOperation(Operation):
 * 
 *   cdef readonly int _flags             # <<<<<<<<<<<<<<
 * 
 *   cdef void c(self) except *
 */

/* Python wrapper */
static PyObject *__pyx_pw_4grpc_7_cython_6cygrpc_28SendCloseFromClientOperation_6_flags_1__get__(PyObject *__pyx_v_self); /*proto*/
static PyObject *__pyx_pw_4grpc_7_cython_6cygrpc_28SendCloseFromClientOperation_6_flags_1__get__(PyObject *__pyx_v_self) {
  PyObject *__pyx_r = 0;
  __Pyx_RefNannyDeclarations
  __Pyx_RefNannySetupContext("__get__ (wrapper)", 0);
  __pyx_r = __pyx_pf_4grpc_7_cython_6cygrpc_28SendCloseFromClientOperation_6_flags___get__(((struct __pyx_obj_4grpc_7_cython_6cygrpc_SendCloseFromClientOperation *)__pyx_v_self));

  /* function exit code */
  __Pyx_RefNannyFinishContext();
  return __pyx_r;
}

static PyObject *__pyx_pf_4grpc_7_cython_6cygrpc_28SendCloseFromClientOperation_6_flags___get__(struct __pyx_obj_4grpc_7_cython_6cygrpc_SendCloseFromClientOperation *__pyx_v_self) {
  PyObject *__pyx_r = NULL;
  __Pyx_RefNannyDeclarations
  PyObject *__pyx_t_1 = NULL;
  __Pyx_RefNannySetupContext("__get__", 0);
  __Pyx_XDECREF(__pyx_r);
  __pyx_t_1 = __Pyx_PyInt_From_int(__pyx_v_self->_flags); if (unlikely(!__pyx_t_1)) __PYX_ERR(15, 48, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_1);
  __pyx_r = __pyx_t_1;
  __pyx_t_1 = 0;
  goto __pyx_L0;

  /* function exit code */
  __pyx_L1_error:;
  __Pyx_XDECREF(__pyx_t_1);
  __Pyx_AddTraceback("grpc._cython.cygrpc.SendCloseFromClientOperation._flags.__get__", __pyx_clineno, __pyx_lineno, __pyx_filename);
  __pyx_r = NULL;
  __pyx_L0:;
  __Pyx_XGIVEREF(__pyx_r);
  __Pyx_RefNannyFinishContext();
  return __pyx_r;
}

/* "(tree fragment)":1
 * def __reduce_cython__(self):             # <<<<<<<<<<<<<<
 *     raise TypeError("no default __reduce__ due to non-trivial __cinit__")
 * def __setstate_cython__(self, __pyx_state):
 */

/* Python wrapper */
static PyObject *__pyx_pw_4grpc_7_cython_6cygrpc_28SendCloseFromClientOperation_5__reduce_cython__(PyObject *__pyx_v_self, CYTHON_UNUSED PyObject *unused); /*proto*/
static PyObject *__pyx_pw_4grpc_7_cython_6cygrpc_28SendCloseFromClientOperation_5__reduce_cython__(PyObject *__pyx_v_self, CYTHON_UNUSED PyObject *unused) {
  PyObject *__pyx_r = 0;
  __Pyx_RefNannyDeclarations
  __Pyx_RefNannySetupContext("__reduce_cython__ (wrapper)", 0);
  __pyx_r = __pyx_pf_4grpc_7_cython_6cygrpc_28SendCloseFromClientOperation_4__reduce_cython__(((struct __pyx_obj_4grpc_7_cython_6cygrpc_SendCloseFromClientOperation *)__pyx_v_self));

  /* function exit code */
  __Pyx_RefNannyFinishContext();
  return __pyx_r;
}

static PyObject *__pyx_pf_4grpc_7_cython_6cygrpc_28SendCloseFromClientOperation_4__reduce_cython__(CYTHON_UNUSED struct __pyx_obj_4grpc_7_cython_6cygrpc_SendCloseFromClientOperation *__pyx_v_self) {
  PyObject *__pyx_r = NULL;
  __Pyx_RefNannyDeclarations
  PyObject *__pyx_t_1 = NULL;
  __Pyx_RefNannySetupContext("__reduce_cython__", 0);

  /* "(tree fragment)":2
 * def __reduce_cython__(self):
 *     raise TypeError("no default __reduce__ due to non-trivial __cinit__")             # <<<<<<<<<<<<<<
 * def __setstate_cython__(self, __pyx_state):
 *     raise TypeError("no default __reduce__ due to non-trivial __cinit__")
 */
  __pyx_t_1 = __Pyx_PyObject_Call(__pyx_builtin_TypeError, __pyx_tuple__90, NULL); if (unlikely(!__pyx_t_1)) __PYX_ERR(7, 2, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_1);
  __Pyx_Raise(__pyx_t_1, 0, 0, 0);
  __Pyx_DECREF(__pyx_t_1); __pyx_t_1 = 0;
  __PYX_ERR(7, 2, __pyx_L1_error)

  /* "(tree fragment)":1
 * def __reduce_cython__(self):             # <<<<<<<<<<<<<<
 *     raise TypeError("no default __reduce__ due to non-trivial __cinit__")
 * def __setstate_cython__(self, __pyx_state):
 */

  /* function exit code */
  __pyx_L1_error:;
  __Pyx_XDECREF(__pyx_t_1);
  __Pyx_AddTraceback("grpc._cython.cygrpc.SendCloseFromClientOperation.__reduce_cython__", __pyx_clineno, __pyx_lineno, __pyx_filename);
  __pyx_r = NULL;
  __Pyx_XGIVEREF(__pyx_r);
  __Pyx_RefNannyFinishContext();
  return __pyx_r;
}

/* "(tree fragment)":3
 * def __reduce_cython__(self):
 *     raise TypeError("no default __reduce__ due to non-trivial __cinit__")
 * def __setstate_cython__(self, __pyx_state):             # <<<<<<<<<<<<<<
 *     raise TypeError("no default __reduce__ due to non-trivial __cinit__")
 */

/* Python wrapper */
static PyObject *__pyx_pw_4grpc_7_cython_6cygrpc_28SendCloseFromClientOperation_7__setstate_cython__(PyObject *__pyx_v_self, PyObject *__pyx_v___pyx_state); /*proto*/
static PyObject *__pyx_pw_4grpc_7_cython_6cygrpc_28SendCloseFromClientOperation_7__setstate_cython__(PyObject *__pyx_v_self, PyObject *__pyx_v___pyx_state) {
  PyObject *__pyx_r = 0;
  __Pyx_RefNannyDeclarations
  __Pyx_RefNannySetupContext("__setstate_cython__ (wrapper)", 0);
  __pyx_r = __pyx_pf_4grpc_7_cython_6cygrpc_28SendCloseFromClientOperation_6__setstate_cython__(((struct __pyx_obj_4grpc_7_cython_6cygrpc_SendCloseFromClientOperation *)__pyx_v_self), ((PyObject *)__pyx_v___pyx_state));

  /* function exit code */
  __Pyx_RefNannyFinishContext();
  return __pyx_r;
}

static PyObject *__pyx_pf_4grpc_7_cython_6cygrpc_28SendCloseFromClientOperation_6__setstate_cython__(CYTHON_UNUSED struct __pyx_obj_4grpc_7_cython_6cygrpc_SendCloseFromClientOperation *__pyx_v_self, CYTHON_UNUSED PyObject *__pyx_v___pyx_state) {
  PyObject *__pyx_r = NULL;
  __Pyx_RefNannyDeclarations
  PyObject *__pyx_t_1 = NULL;
  __Pyx_RefNannySetupContext("__setstate_cython__", 0);

  /* "(tree fragment)":4
 *     raise TypeError("no default __reduce__ due to non-trivial __cinit__")
 * def __setstate_cython__(self, __pyx_state):
 *     raise TypeError("no default __reduce__ due to non-trivial __cinit__")             # <<<<<<<<<<<<<<
 */
  __pyx_t_1 = __Pyx_PyObject_Call(__pyx_builtin_TypeError, __pyx_tuple__91, NULL); if (unlikely(!__pyx_t_1)) __PYX_ERR(7, 4, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_1);
  __Pyx_Raise(__pyx_t_1, 0, 0, 0);
  __Pyx_DECREF(__pyx_t_1); __pyx_t_1 = 0;
  __PYX_ERR(7, 4, __pyx_L1_error)

  /* "(tree fragment)":3
 * def __reduce_cython__(self):
 *     raise TypeError("no default __reduce__ due to non-trivial __cinit__")
 * def __setstate_cython__(self, __pyx_state):             # <<<<<<<<<<<<<<
 *     raise TypeError("no default __reduce__ due to non-trivial __cinit__")
 */

  /* function exit code */
  __pyx_L1_error:;
  __Pyx_XDECREF(__pyx_t_1);
  __Pyx_AddTraceback("grpc._cython.cygrpc.SendCloseFromClientOperation.__setstate_cython__", __pyx_clineno, __pyx_lineno, __pyx_filename);
  __pyx_r = NULL;
  __Pyx_XGIVEREF(__pyx_r);
  __Pyx_RefNannyFinishContext();
  return __pyx_r;
}

/* "src/python/grpcio/grpc/_cython/_cygrpc/operation.pyx.pxi":90
 * cdef class SendStatusFromServerOperation(Operation):
 * 
 *   def __cinit__(self, trailing_metadata, code, object details, int flags):             # <<<<<<<<<<<<<<
 *     self._trailing_metadata = trailing_metadata
 *     self._code = code
 */

/* Python wrapper */
static int __pyx_pw_4grpc_7_cython_6cygrpc_29SendStatusFromServerOperation_1__cinit__(PyObject *__pyx_v_self, PyObject *__pyx_args, PyObject *__pyx_kwds); /*proto*/
static int __pyx_pw_4grpc_7_cython_6cygrpc_29SendStatusFromServerOperation_1__cinit__(PyObject *__pyx_v_self, PyObject *__pyx_args, PyObject *__pyx_kwds) {
  PyObject *__pyx_v_trailing_metadata = 0;
  PyObject *__pyx_v_code = 0;
  PyObject *__pyx_v_details = 0;
  int __pyx_v_flags;
  int __pyx_r;
  __Pyx_RefNannyDeclarations
  __Pyx_RefNannySetupContext("__cinit__ (wrapper)", 0);
  {
    static PyObject **__pyx_pyargnames[] = {&__pyx_n_s_trailing_metadata,&__pyx_n_s_code,&__pyx_n_s_details,&__pyx_n_s_flags,0};
    PyObject* values[4] = {0,0,0,0};
    if (unlikely(__pyx_kwds)) {
      Py_ssize_t kw_args;
      const Py_ssize_t pos_args = PyTuple_GET_SIZE(__pyx_args);
      switch (pos_args) {
        case  4: values[3] = PyTuple_GET_ITEM(__pyx_args, 3);
        CYTHON_FALLTHROUGH;
        case  3: values[2] = PyTuple_GET_ITEM(__pyx_args, 2);
        CYTHON_FALLTHROUGH;
        case  2: values[1] = PyTuple_GET_ITEM(__pyx_args, 1);
        CYTHON_FALLTHROUGH;
        case  1: values[0] = PyTuple_GET_ITEM(__pyx_args, 0);
        CYTHON_FALLTHROUGH;
        case  0: break;
        default: goto __pyx_L5_argtuple_error;
      }
      kw_args = PyDict_Size(__pyx_kwds);
      switch (pos_args) {
        case  0:
        if (likely((values[0] = __Pyx_PyDict_GetItemStr(__pyx_kwds, __pyx_n_s_trailing_metadata)) != 0)) kw_args--;
        else goto __pyx_L5_argtuple_error;
        CYTHON_FALLTHROUGH;
        case  1:
        if (likely((values[1] = __Pyx_PyDict_GetItemStr(__pyx_kwds, __pyx_n_s_code)) != 0)) kw_args--;
        else {
          __Pyx_RaiseArgtupleInvalid("__cinit__", 1, 4, 4, 1); __PYX_ERR(14, 90, __pyx_L3_error)
        }
        CYTHON_FALLTHROUGH;
        case  2:
        if (likely((values[2] = __Pyx_PyDict_GetItemStr(__pyx_kwds, __pyx_n_s_details)) != 0)) kw_args--;
        else {
          __Pyx_RaiseArgtupleInvalid("__cinit__", 1, 4, 4, 2); __PYX_ERR(14, 90, __pyx_L3_error)
        }
        CYTHON_FALLTHROUGH;
        case  3:
        if (likely((values[3] = __Pyx_PyDict_GetItemStr(__pyx_kwds, __pyx_n_s_flags)) != 0)) kw_args--;
        else {
          __Pyx_RaiseArgtupleInvalid("__cinit__", 1, 4, 4, 3); __PYX_ERR(14, 90, __pyx_L3_error)
        }
      }
      if (unlikely(kw_args > 0)) {
        if (unlikely(__Pyx_ParseOptionalKeywords(__pyx_kwds, __pyx_pyargnames, 0, values, pos_args, "__cinit__") < 0)) __PYX_ERR(14, 90, __pyx_L3_error)
      }
    } else if (PyTuple_GET_SIZE(__pyx_args) != 4) {
      goto __pyx_L5_argtuple_error;
    } else {
      values[0] = PyTuple_GET_ITEM(__pyx_args, 0);
      values[1] = PyTuple_GET_ITEM(__pyx_args, 1);
      values[2] = PyTuple_GET_ITEM(__pyx_args, 2);
      values[3] = PyTuple_GET_ITEM(__pyx_args, 3);
    }
    __pyx_v_trailing_metadata = values[0];
    __pyx_v_code = values[1];
    __pyx_v_details = values[2];
    __pyx_v_flags = __Pyx_PyInt_As_int(values[3]); if (unlikely((__pyx_v_flags == (int)-1) && PyErr_Occurred())) __PYX_ERR(14, 90, __pyx_L3_error)
  }
  goto __pyx_L4_argument_unpacking_done;
  __pyx_L5_argtuple_error:;
  __Pyx_RaiseArgtupleInvalid("__cinit__", 1, 4, 4, PyTuple_GET_SIZE(__pyx_args)); __PYX_ERR(14, 90, __pyx_L3_error)
  __pyx_L3_error:;
  __Pyx_AddTraceback("grpc._cython.cygrpc.SendStatusFromServerOperation.__cinit__", __pyx_clineno, __pyx_lineno, __pyx_filename);
  __Pyx_RefNannyFinishContext();
  return -1;
  __pyx_L4_argument_unpacking_done:;
  __pyx_r = __pyx_pf_4grpc_7_cython_6cygrpc_29SendStatusFromServerOperation___cinit__(((struct __pyx_obj_4grpc_7_cython_6cygrpc_SendStatusFromServerOperation *)__pyx_v_self), __pyx_v_trailing_metadata, __pyx_v_code, __pyx_v_details, __pyx_v_flags);

  /* function exit code */
  __Pyx_RefNannyFinishContext();
  return __pyx_r;
}

static int __pyx_pf_4grpc_7_cython_6cygrpc_29SendStatusFromServerOperation___cinit__(struct __pyx_obj_4grpc_7_cython_6cygrpc_SendStatusFromServerOperation *__pyx_v_self, PyObject *__pyx_v_trailing_metadata, PyObject *__pyx_v_code, PyObject *__pyx_v_details, int __pyx_v_flags) {
  int __pyx_r;
  __Pyx_RefNannyDeclarations
  __Pyx_RefNannySetupContext("__cinit__", 0);

  /* "src/python/grpcio/grpc/_cython/_cygrpc/operation.pyx.pxi":91
 * 
 *   def __cinit__(self, trailing_metadata, code, object details, int flags):
 *     self._trailing_metadata = trailing_metadata             # <<<<<<<<<<<<<<
 *     self._code = code
 *     self._details = details
 */
  __Pyx_INCREF(__pyx_v_trailing_metadata);
  __Pyx_GIVEREF(__pyx_v_trailing_metadata);
  __Pyx_GOTREF(__pyx_v_self->_trailing_metadata);
  __Pyx_DECREF(__pyx_v_self->_trailing_metadata);
  __pyx_v_self->_trailing_metadata = __pyx_v_trailing_metadata;

  /* "src/python/grpcio/grpc/_cython/_cygrpc/operation.pyx.pxi":92
 *   def __cinit__(self, trailing_metadata, code, object details, int flags):
 *     self._trailing_metadata = trailing_metadata
 *     self._code = code             # <<<<<<<<<<<<<<
 *     self._details = details
 *     self._flags = flags
 */
  __Pyx_INCREF(__pyx_v_code);
  __Pyx_GIVEREF(__pyx_v_code);
  __Pyx_GOTREF(__pyx_v_self->_code);
  __Pyx_DECREF(__pyx_v_self->_code);
  __pyx_v_self->_code = __pyx_v_code;

  /* "src/python/grpcio/grpc/_cython/_cygrpc/operation.pyx.pxi":93
 *     self._trailing_metadata = trailing_metadata
 *     self._code = code
 *     self._details = details             # <<<<<<<<<<<<<<
 *     self._flags = flags
 * 
 */
  __Pyx_INCREF(__pyx_v_details);
  __Pyx_GIVEREF(__pyx_v_details);
  __Pyx_GOTREF(__pyx_v_self->_details);
  __Pyx_DECREF(__pyx_v_self->_details);
  __pyx_v_self->_details = __pyx_v_details;

  /* "src/python/grpcio/grpc/_cython/_cygrpc/operation.pyx.pxi":94
 *     self._code = code
 *     self._details = details
 *     self._flags = flags             # <<<<<<<<<<<<<<
 * 
 *   def type(self):
 */
  __pyx_v_self->_flags = __pyx_v_flags;

  /* "src/python/grpcio/grpc/_cython/_cygrpc/operation.pyx.pxi":90
 * cdef class SendStatusFromServerOperation(Operation):
 * 
 *   def __cinit__(self, trailing_metadata, code, object details, int flags):             # <<<<<<<<<<<<<<
 *     self._trailing_metadata = trailing_metadata
 *     self._code = code
 */

  /* function exit code */
  __pyx_r = 0;
  __Pyx_RefNannyFinishContext();
  return __pyx_r;
}

/* "src/python/grpcio/grpc/_cython/_cygrpc/operation.pyx.pxi":96
 *     self._flags = flags
 * 
 *   def type(self):             # <<<<<<<<<<<<<<
 *     return GRPC_OP_SEND_STATUS_FROM_SERVER
 * 
 */

/* Python wrapper */
static PyObject *__pyx_pw_4grpc_7_cython_6cygrpc_29SendStatusFromServerOperation_3type(PyObject *__pyx_v_self, CYTHON_UNUSED PyObject *unused); /*proto*/
static PyObject *__pyx_pw_4grpc_7_cython_6cygrpc_29SendStatusFromServerOperation_3type(PyObject *__pyx_v_self, CYTHON_UNUSED PyObject *unused) {
  PyObject *__pyx_r = 0;
  __Pyx_RefNannyDeclarations
  __Pyx_RefNannySetupContext("type (wrapper)", 0);
  __pyx_r = __pyx_pf_4grpc_7_cython_6cygrpc_29SendStatusFromServerOperation_2type(((struct __pyx_obj_4grpc_7_cython_6cygrpc_SendStatusFromServerOperation *)__pyx_v_self));

  /* function exit code */
  __Pyx_RefNannyFinishContext();
  return __pyx_r;
}

static PyObject *__pyx_pf_4grpc_7_cython_6cygrpc_29SendStatusFromServerOperation_2type(CYTHON_UNUSED struct __pyx_obj_4grpc_7_cython_6cygrpc_SendStatusFromServerOperation *__pyx_v_self) {
  PyObject *__pyx_r = NULL;
  __Pyx_RefNannyDeclarations
  PyObject *__pyx_t_1 = NULL;
  __Pyx_RefNannySetupContext("type", 0);

  /* "src/python/grpcio/grpc/_cython/_cygrpc/operation.pyx.pxi":97
 * 
 *   def type(self):
 *     return GRPC_OP_SEND_STATUS_FROM_SERVER             # <<<<<<<<<<<<<<
 * 
 *   cdef void c(self) except *:
 */
  __Pyx_XDECREF(__pyx_r);
  __pyx_t_1 = __Pyx_PyInt_From_grpc_op_type(GRPC_OP_SEND_STATUS_FROM_SERVER); if (unlikely(!__pyx_t_1)) __PYX_ERR(14, 97, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_1);
  __pyx_r = __pyx_t_1;
  __pyx_t_1 = 0;
  goto __pyx_L0;

  /* "src/python/grpcio/grpc/_cython/_cygrpc/operation.pyx.pxi":96
 *     self._flags = flags
 * 
 *   def type(self):             # <<<<<<<<<<<<<<
 *     return GRPC_OP_SEND_STATUS_FROM_SERVER
 * 
 */

  /* function exit code */
  __pyx_L1_error:;
  __Pyx_XDECREF(__pyx_t_1);
  __Pyx_AddTraceback("grpc._cython.cygrpc.SendStatusFromServerOperation.type", __pyx_clineno, __pyx_lineno, __pyx_filename);
  __pyx_r = NULL;
  __pyx_L0:;
  __Pyx_XGIVEREF(__pyx_r);
  __Pyx_RefNannyFinishContext();
  return __pyx_r;
}

/* "src/python/grpcio/grpc/_cython/_cygrpc/operation.pyx.pxi":99
 *     return GRPC_OP_SEND_STATUS_FROM_SERVER
 * 
 *   cdef void c(self) except *:             # <<<<<<<<<<<<<<
 *     self.c_op.type = GRPC_OP_SEND_STATUS_FROM_SERVER
 *     self.c_op.flags = self._flags
 */

static void __pyx_f_4grpc_7_cython_6cygrpc_29SendStatusFromServerOperation_c(struct __pyx_obj_4grpc_7_cython_6cygrpc_SendStatusFromServerOperation *__pyx_v_self) {
  __Pyx_RefNannyDeclarations
  int __pyx_t_1;
  PyObject *__pyx_t_2 = NULL;
  grpc_metadata *__pyx_t_3;
  size_t __pyx_t_4;
  grpc_status_code __pyx_t_5;
  PyObject *__pyx_t_6 = NULL;
  __Pyx_RefNannySetupContext("c", 0);

  /* "src/python/grpcio/grpc/_cython/_cygrpc/operation.pyx.pxi":100
 * 
 *   cdef void c(self) except *:
 *     self.c_op.type = GRPC_OP_SEND_STATUS_FROM_SERVER             # <<<<<<<<<<<<<<
 *     self.c_op.flags = self._flags
 *     _store_c_metadata(
 */
  __pyx_v_self->__pyx_base.c_op.op = GRPC_OP_SEND_STATUS_FROM_SERVER;

  /* "src/python/grpcio/grpc/_cython/_cygrpc/operation.pyx.pxi":101
 *   cdef void c(self) except *:
 *     self.c_op.type = GRPC_OP_SEND_STATUS_FROM_SERVER
 *     self.c_op.flags = self._flags             # <<<<<<<<<<<<<<
 *     _store_c_metadata(
 *         self._trailing_metadata, &self._c_trailing_metadata,
 */
  __pyx_t_1 = __pyx_v_self->_flags;
  __pyx_v_self->__pyx_base.c_op.flags = __pyx_t_1;

  /* "src/python/grpcio/grpc/_cython/_cygrpc/operation.pyx.pxi":103
 *     self.c_op.flags = self._flags
 *     _store_c_metadata(
 *         self._trailing_metadata, &self._c_trailing_metadata,             # <<<<<<<<<<<<<<
 *         &self._c_trailing_metadata_count)
 *     self.c_op.data.send_status_from_server.trailing_metadata = (
 */
  __pyx_t_2 = __pyx_v_self->_trailing_metadata;
  __Pyx_INCREF(__pyx_t_2);

  /* "src/python/grpcio/grpc/_cython/_cygrpc/operation.pyx.pxi":102
 *     self.c_op.type = GRPC_OP_SEND_STATUS_FROM_SERVER
 *     self.c_op.flags = self._flags
 *     _store_c_metadata(             # <<<<<<<<<<<<<<
 *         self._trailing_metadata, &self._c_trailing_metadata,
 *         &self._c_trailing_metadata_count)
 */
  __pyx_f_4grpc_7_cython_6cygrpc__store_c_metadata(__pyx_t_2, (&__pyx_v_self->_c_trailing_metadata), (&__pyx_v_self->_c_trailing_metadata_count)); if (unlikely(PyErr_Occurred())) __PYX_ERR(14, 102, __pyx_L1_error)
  __Pyx_DECREF(__pyx_t_2); __pyx_t_2 = 0;

  /* "src/python/grpcio/grpc/_cython/_cygrpc/operation.pyx.pxi":106
 *         &self._c_trailing_metadata_count)
 *     self.c_op.data.send_status_from_server.trailing_metadata = (
 *         self._c_trailing_metadata)             # <<<<<<<<<<<<<<
 *     self.c_op.data.send_status_from_server.trailing_metadata_count = (
 *         self._c_trailing_metadata_count)
 */
  __pyx_t_3 = __pyx_v_self->_c_trailing_metadata;

  /* "src/python/grpcio/grpc/_cython/_cygrpc/operation.pyx.pxi":105
 *         self._trailing_metadata, &self._c_trailing_metadata,
 *         &self._c_trailing_metadata_count)
 *     self.c_op.data.send_status_from_server.trailing_metadata = (             # <<<<<<<<<<<<<<
 *         self._c_trailing_metadata)
 *     self.c_op.data.send_status_from_server.trailing_metadata_count = (
 */
  __pyx_v_self->__pyx_base.c_op.data.send_status_from_server.trailing_metadata = __pyx_t_3;

  /* "src/python/grpcio/grpc/_cython/_cygrpc/operation.pyx.pxi":108
 *         self._c_trailing_metadata)
 *     self.c_op.data.send_status_from_server.trailing_metadata_count = (
 *         self._c_trailing_metadata_count)             # <<<<<<<<<<<<<<
 *     self.c_op.data.send_status_from_server.status = self._code
 *     self._c_details = _slice_from_bytes(_encode(self._details))
 */
  __pyx_t_4 = __pyx_v_self->_c_trailing_metadata_count;

  /* "src/python/grpcio/grpc/_cython/_cygrpc/operation.pyx.pxi":107
 *     self.c_op.data.send_status_from_server.trailing_metadata = (
 *         self._c_trailing_metadata)
 *     self.c_op.data.send_status_from_server.trailing_metadata_count = (             # <<<<<<<<<<<<<<
 *         self._c_trailing_metadata_count)
 *     self.c_op.data.send_status_from_server.status = self._code
 */
  __pyx_v_self->__pyx_base.c_op.data.send_status_from_server.trailing_metadata_count = __pyx_t_4;

  /* "src/python/grpcio/grpc/_cython/_cygrpc/operation.pyx.pxi":109
 *     self.c_op.data.send_status_from_server.trailing_metadata_count = (
 *         self._c_trailing_metadata_count)
 *     self.c_op.data.send_status_from_server.status = self._code             # <<<<<<<<<<<<<<
 *     self._c_details = _slice_from_bytes(_encode(self._details))
 *     self.c_op.data.send_status_from_server.status_details = &self._c_details
 */
  __pyx_t_5 = ((grpc_status_code)__Pyx_PyInt_As_grpc_status_code(__pyx_v_self->_code)); if (unlikely(PyErr_Occurred())) __PYX_ERR(14, 109, __pyx_L1_error)
  __pyx_v_self->__pyx_base.c_op.data.send_status_from_server.status = __pyx_t_5;

  /* "src/python/grpcio/grpc/_cython/_cygrpc/operation.pyx.pxi":110
 *         self._c_trailing_metadata_count)
 *     self.c_op.data.send_status_from_server.status = self._code
 *     self._c_details = _slice_from_bytes(_encode(self._details))             # <<<<<<<<<<<<<<
 *     self.c_op.data.send_status_from_server.status_details = &self._c_details
 * 
 */
  __pyx_t_2 = __pyx_v_self->_details;
  __Pyx_INCREF(__pyx_t_2);
  __pyx_t_6 = __pyx_f_4grpc_7_cython_6cygrpc__encode(__pyx_t_2); if (unlikely(!__pyx_t_6)) __PYX_ERR(14, 110, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_6);
  __Pyx_DECREF(__pyx_t_2); __pyx_t_2 = 0;
  __pyx_v_self->_c_details = __pyx_f_4grpc_7_cython_6cygrpc__slice_from_bytes(((PyObject*)__pyx_t_6));
  __Pyx_DECREF(__pyx_t_6); __pyx_t_6 = 0;

  /* "src/python/grpcio/grpc/_cython/_cygrpc/operation.pyx.pxi":111
 *     self.c_op.data.send_status_from_server.status = self._code
 *     self._c_details = _slice_from_bytes(_encode(self._details))
 *     self.c_op.data.send_status_from_server.status_details = &self._c_details             # <<<<<<<<<<<<<<
 * 
 *   cdef void un_c(self) except *:
 */
  __pyx_v_self->__pyx_base.c_op.data.send_status_from_server.status_details = (&__pyx_v_self->_c_details);

  /* "src/python/grpcio/grpc/_cython/_cygrpc/operation.pyx.pxi":99
 *     return GRPC_OP_SEND_STATUS_FROM_SERVER
 * 
 *   cdef void c(self) except *:             # <<<<<<<<<<<<<<
 *     self.c_op.type = GRPC_OP_SEND_STATUS_FROM_SERVER
 *     self.c_op.flags = self._flags
 */

  /* function exit code */
  goto __pyx_L0;
  __pyx_L1_error:;
  __Pyx_XDECREF(__pyx_t_2);
  __Pyx_XDECREF(__pyx_t_6);
  __Pyx_AddTraceback("grpc._cython.cygrpc.SendStatusFromServerOperation.c", __pyx_clineno, __pyx_lineno, __pyx_filename);
  __pyx_L0:;
  __Pyx_RefNannyFinishContext();
}

/* "src/python/grpcio/grpc/_cython/_cygrpc/operation.pyx.pxi":113
 *     self.c_op.data.send_status_from_server.status_details = &self._c_details
 * 
 *   cdef void un_c(self) except *:             # <<<<<<<<<<<<<<
 *     grpc_slice_unref(self._c_details)
 *     _release_c_metadata(
 */

static void __pyx_f_4grpc_7_cython_6cygrpc_29SendStatusFromServerOperation_un_c(struct __pyx_obj_4grpc_7_cython_6cygrpc_SendStatusFromServerOperation *__pyx_v_self) {
  __Pyx_RefNannyDeclarations
  __Pyx_RefNannySetupContext("un_c", 0);

  /* "src/python/grpcio/grpc/_cython/_cygrpc/operation.pyx.pxi":114
 * 
 *   cdef void un_c(self) except *:
 *     grpc_slice_unref(self._c_details)             # <<<<<<<<<<<<<<
 *     _release_c_metadata(
 *         self._c_trailing_metadata, self._c_trailing_metadata_count)
 */
  grpc_slice_unref(__pyx_v_self->_c_details);

  /* "src/python/grpcio/grpc/_cython/_cygrpc/operation.pyx.pxi":115
 *   cdef void un_c(self) except *:
 *     grpc_slice_unref(self._c_details)
 *     _release_c_metadata(             # <<<<<<<<<<<<<<
 *         self._c_trailing_metadata, self._c_trailing_metadata_count)
 * 
 */
  __pyx_f_4grpc_7_cython_6cygrpc__release_c_metadata(__pyx_v_self->_c_trailing_metadata, __pyx_v_self->_c_trailing_metadata_count); if (unlikely(PyErr_Occurred())) __PYX_ERR(14, 115, __pyx_L1_error)

  /* "src/python/grpcio/grpc/_cython/_cygrpc/operation.pyx.pxi":113
 *     self.c_op.data.send_status_from_server.status_details = &self._c_details
 * 
 *   cdef void un_c(self) except *:             # <<<<<<<<<<<<<<
 *     grpc_slice_unref(self._c_details)
 *     _release_c_metadata(
 */

  /* function exit code */
  goto __pyx_L0;
  __pyx_L1_error:;
  __Pyx_AddTraceback("grpc._cython.cygrpc.SendStatusFromServerOperation.un_c", __pyx_clineno, __pyx_lineno, __pyx_filename);
  __pyx_L0:;
  __Pyx_RefNannyFinishContext();
}

/* "src/python/grpcio/grpc/_cython/_cygrpc/operation.pxd.pxi":56
 * cdef class SendStatusFromServerOperation(Operation):
 * 
 *   cdef readonly object _trailing_metadata             # <<<<<<<<<<<<<<
 *   cdef readonly object _code
 *   cdef readonly object _details
 */

/* Python wrapper */
static PyObject *__pyx_pw_4grpc_7_cython_6cygrpc_29SendStatusFromServerOperation_18_trailing_metadata_1__get__(PyObject *__pyx_v_self); /*proto*/
static PyObject *__pyx_pw_4grpc_7_cython_6cygrpc_29SendStatusFromServerOperation_18_trailing_metadata_1__get__(PyObject *__pyx_v_self) {
  PyObject *__pyx_r = 0;
  __Pyx_RefNannyDeclarations
  __Pyx_RefNannySetupContext("__get__ (wrapper)", 0);
  __pyx_r = __pyx_pf_4grpc_7_cython_6cygrpc_29SendStatusFromServerOperation_18_trailing_metadata___get__(((struct __pyx_obj_4grpc_7_cython_6cygrpc_SendStatusFromServerOperation *)__pyx_v_self));

  /* function exit code */
  __Pyx_RefNannyFinishContext();
  return __pyx_r;
}

static PyObject *__pyx_pf_4grpc_7_cython_6cygrpc_29SendStatusFromServerOperation_18_trailing_metadata___get__(struct __pyx_obj_4grpc_7_cython_6cygrpc_SendStatusFromServerOperation *__pyx_v_self) {
  PyObject *__pyx_r = NULL;
  __Pyx_RefNannyDeclarations
  __Pyx_RefNannySetupContext("__get__", 0);
  __Pyx_XDECREF(__pyx_r);
  __Pyx_INCREF(__pyx_v_self->_trailing_metadata);
  __pyx_r = __pyx_v_self->_trailing_metadata;
  goto __pyx_L0;

  /* function exit code */
  __pyx_L0:;
  __Pyx_XGIVEREF(__pyx_r);
  __Pyx_RefNannyFinishContext();
  return __pyx_r;
}

/* "src/python/grpcio/grpc/_cython/_cygrpc/operation.pxd.pxi":57
 * 
 *   cdef readonly object _trailing_metadata
 *   cdef readonly object _code             # <<<<<<<<<<<<<<
 *   cdef readonly object _details
 *   cdef readonly int _flags
 */

/* Python wrapper */
static PyObject *__pyx_pw_4grpc_7_cython_6cygrpc_29SendStatusFromServerOperation_5_code_1__get__(PyObject *__pyx_v_self); /*proto*/
static PyObject *__pyx_pw_4grpc_7_cython_6cygrpc_29SendStatusFromServerOperation_5_code_1__get__(PyObject *__pyx_v_self) {
  PyObject *__pyx_r = 0;
  __Pyx_RefNannyDeclarations
  __Pyx_RefNannySetupContext("__get__ (wrapper)", 0);
  __pyx_r = __pyx_pf_4grpc_7_cython_6cygrpc_29SendStatusFromServerOperation_5_code___get__(((struct __pyx_obj_4grpc_7_cython_6cygrpc_SendStatusFromServerOperation *)__pyx_v_self));

  /* function exit code */
  __Pyx_RefNannyFinishContext();
  return __pyx_r;
}

static PyObject *__pyx_pf_4grpc_7_cython_6cygrpc_29SendStatusFromServerOperation_5_code___get__(struct __pyx_obj_4grpc_7_cython_6cygrpc_SendStatusFromServerOperation *__pyx_v_self) {
  PyObject *__pyx_r = NULL;
  __Pyx_RefNannyDeclarations
  __Pyx_RefNannySetupContext("__get__", 0);
  __Pyx_XDECREF(__pyx_r);
  __Pyx_INCREF(__pyx_v_self->_code);
  __pyx_r = __pyx_v_self->_code;
  goto __pyx_L0;

  /* function exit code */
  __pyx_L0:;
  __Pyx_XGIVEREF(__pyx_r);
  __Pyx_RefNannyFinishContext();
  return __pyx_r;
}

/* "src/python/grpcio/grpc/_cython/_cygrpc/operation.pxd.pxi":58
 *   cdef readonly object _trailing_metadata
 *   cdef readonly object _code
 *   cdef readonly object _details             # <<<<<<<<<<<<<<
 *   cdef readonly int _flags
 *   cdef grpc_metadata *_c_trailing_metadata
 */

/* Python wrapper */
static PyObject *__pyx_pw_4grpc_7_cython_6cygrpc_29SendStatusFromServerOperation_8_details_1__get__(PyObject *__pyx_v_self); /*proto*/
static PyObject *__pyx_pw_4grpc_7_cython_6cygrpc_29SendStatusFromServerOperation_8_details_1__get__(PyObject *__pyx_v_self) {
  PyObject *__pyx_r = 0;
  __Pyx_RefNannyDeclarations
  __Pyx_RefNannySetupContext("__get__ (wrapper)", 0);
  __pyx_r = __pyx_pf_4grpc_7_cython_6cygrpc_29SendStatusFromServerOperation_8_details___get__(((struct __pyx_obj_4grpc_7_cython_6cygrpc_SendStatusFromServerOperation *)__pyx_v_self));

  /* function exit code */
  __Pyx_RefNannyFinishContext();
  return __pyx_r;
}

static PyObject *__pyx_pf_4grpc_7_cython_6cygrpc_29SendStatusFromServerOperation_8_details___get__(struct __pyx_obj_4grpc_7_cython_6cygrpc_SendStatusFromServerOperation *__pyx_v_self) {
  PyObject *__pyx_r = NULL;
  __Pyx_RefNannyDeclarations
  __Pyx_RefNannySetupContext("__get__", 0);
  __Pyx_XDECREF(__pyx_r);
  __Pyx_INCREF(__pyx_v_self->_details);
  __pyx_r = __pyx_v_self->_details;
  goto __pyx_L0;

  /* function exit code */
  __pyx_L0:;
  __Pyx_XGIVEREF(__pyx_r);
  __Pyx_RefNannyFinishContext();
  return __pyx_r;
}

/* "src/python/grpcio/grpc/_cython/_cygrpc/operation.pxd.pxi":59
 *   cdef readonly object _code
 *   cdef readonly object _details
 *   cdef readonly int _flags             # <<<<<<<<<<<<<<
 *   cdef grpc_metadata *_c_trailing_metadata
 *   cdef size_t _c_trailing_metadata_count
 */

/* Python wrapper */
static PyObject *__pyx_pw_4grpc_7_cython_6cygrpc_29SendStatusFromServerOperation_6_flags_1__get__(PyObject *__pyx_v_self); /*proto*/
static PyObject *__pyx_pw_4grpc_7_cython_6cygrpc_29SendStatusFromServerOperation_6_flags_1__get__(PyObject *__pyx_v_self) {
  PyObject *__pyx_r = 0;
  __Pyx_RefNannyDeclarations
  __Pyx_RefNannySetupContext("__get__ (wrapper)", 0);
  __pyx_r = __pyx_pf_4grpc_7_cython_6cygrpc_29SendStatusFromServerOperation_6_flags___get__(((struct __pyx_obj_4grpc_7_cython_6cygrpc_SendStatusFromServerOperation *)__pyx_v_self));

  /* function exit code */
  __Pyx_RefNannyFinishContext();
  return __pyx_r;
}

static PyObject *__pyx_pf_4grpc_7_cython_6cygrpc_29SendStatusFromServerOperation_6_flags___get__(struct __pyx_obj_4grpc_7_cython_6cygrpc_SendStatusFromServerOperation *__pyx_v_self) {
  PyObject *__pyx_r = NULL;
  __Pyx_RefNannyDeclarations
  PyObject *__pyx_t_1 = NULL;
  __Pyx_RefNannySetupContext("__get__", 0);
  __Pyx_XDECREF(__pyx_r);
  __pyx_t_1 = __Pyx_PyInt_From_int(__pyx_v_self->_flags); if (unlikely(!__pyx_t_1)) __PYX_ERR(15, 59, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_1);
  __pyx_r = __pyx_t_1;
  __pyx_t_1 = 0;
  goto __pyx_L0;

  /* function exit code */
  __pyx_L1_error:;
  __Pyx_XDECREF(__pyx_t_1);
  __Pyx_AddTraceback("grpc._cython.cygrpc.SendStatusFromServerOperation._flags.__get__", __pyx_clineno, __pyx_lineno, __pyx_filename);
  __pyx_r = NULL;
  __pyx_L0:;
  __Pyx_XGIVEREF(__pyx_r);
  __Pyx_RefNannyFinishContext();
  return __pyx_r;
}

/* "(tree fragment)":1
 * def __reduce_cython__(self):             # <<<<<<<<<<<<<<
 *     raise TypeError("no default __reduce__ due to non-trivial __cinit__")
 * def __setstate_cython__(self, __pyx_state):
 */

/* Python wrapper */
static PyObject *__pyx_pw_4grpc_7_cython_6cygrpc_29SendStatusFromServerOperation_5__reduce_cython__(PyObject *__pyx_v_self, CYTHON_UNUSED PyObject *unused); /*proto*/
static PyObject *__pyx_pw_4grpc_7_cython_6cygrpc_29SendStatusFromServerOperation_5__reduce_cython__(PyObject *__pyx_v_self, CYTHON_UNUSED PyObject *unused) {
  PyObject *__pyx_r = 0;
  __Pyx_RefNannyDeclarations
  __Pyx_RefNannySetupContext("__reduce_cython__ (wrapper)", 0);
  __pyx_r = __pyx_pf_4grpc_7_cython_6cygrpc_29SendStatusFromServerOperation_4__reduce_cython__(((struct __pyx_obj_4grpc_7_cython_6cygrpc_SendStatusFromServerOperation *)__pyx_v_self));

  /* function exit code */
  __Pyx_RefNannyFinishContext();
  return __pyx_r;
}

static PyObject *__pyx_pf_4grpc_7_cython_6cygrpc_29SendStatusFromServerOperation_4__reduce_cython__(CYTHON_UNUSED struct __pyx_obj_4grpc_7_cython_6cygrpc_SendStatusFromServerOperation *__pyx_v_self) {
  PyObject *__pyx_r = NULL;
  __Pyx_RefNannyDeclarations
  PyObject *__pyx_t_1 = NULL;
  __Pyx_RefNannySetupContext("__reduce_cython__", 0);

  /* "(tree fragment)":2
 * def __reduce_cython__(self):
 *     raise TypeError("no default __reduce__ due to non-trivial __cinit__")             # <<<<<<<<<<<<<<
 * def __setstate_cython__(self, __pyx_state):
 *     raise TypeError("no default __reduce__ due to non-trivial __cinit__")
 */
  __pyx_t_1 = __Pyx_PyObject_Call(__pyx_builtin_TypeError, __pyx_tuple__92, NULL); if (unlikely(!__pyx_t_1)) __PYX_ERR(7, 2, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_1);
  __Pyx_Raise(__pyx_t_1, 0, 0, 0);
  __Pyx_DECREF(__pyx_t_1); __pyx_t_1 = 0;
  __PYX_ERR(7, 2, __pyx_L1_error)

  /* "(tree fragment)":1
 * def __reduce_cython__(self):             # <<<<<<<<<<<<<<
 *     raise TypeError("no default __reduce__ due to non-trivial __cinit__")
 * def __setstate_cython__(self, __pyx_state):
 */

  /* function exit code */
  __pyx_L1_error:;
  __Pyx_XDECREF(__pyx_t_1);
  __Pyx_AddTraceback("grpc._cython.cygrpc.SendStatusFromServerOperation.__reduce_cython__", __pyx_clineno, __pyx_lineno, __pyx_filename);
  __pyx_r = NULL;
  __Pyx_XGIVEREF(__pyx_r);
  __Pyx_RefNannyFinishContext();
  return __pyx_r;
}

/* "(tree fragment)":3
 * def __reduce_cython__(self):
 *     raise TypeError("no default __reduce__ due to non-trivial __cinit__")
 * def __setstate_cython__(self, __pyx_state):             # <<<<<<<<<<<<<<
 *     raise TypeError("no default __reduce__ due to non-trivial __cinit__")
 */

/* Python wrapper */
static PyObject *__pyx_pw_4grpc_7_cython_6cygrpc_29SendStatusFromServerOperation_7__setstate_cython__(PyObject *__pyx_v_self, PyObject *__pyx_v___pyx_state); /*proto*/
static PyObject *__pyx_pw_4grpc_7_cython_6cygrpc_29SendStatusFromServerOperation_7__setstate_cython__(PyObject *__pyx_v_self, PyObject *__pyx_v___pyx_state) {
  PyObject *__pyx_r = 0;
  __Pyx_RefNannyDeclarations
  __Pyx_RefNannySetupContext("__setstate_cython__ (wrapper)", 0);
  __pyx_r = __pyx_pf_4grpc_7_cython_6cygrpc_29SendStatusFromServerOperation_6__setstate_cython__(((struct __pyx_obj_4grpc_7_cython_6cygrpc_SendStatusFromServerOperation *)__pyx_v_self), ((PyObject *)__pyx_v___pyx_state));

  /* function exit code */
  __Pyx_RefNannyFinishContext();
  return __pyx_r;
}

static PyObject *__pyx_pf_4grpc_7_cython_6cygrpc_29SendStatusFromServerOperation_6__setstate_cython__(CYTHON_UNUSED struct __pyx_obj_4grpc_7_cython_6cygrpc_SendStatusFromServerOperation *__pyx_v_self, CYTHON_UNUSED PyObject *__pyx_v___pyx_state) {
  PyObject *__pyx_r = NULL;
  __Pyx_RefNannyDeclarations
  PyObject *__pyx_t_1 = NULL;
  __Pyx_RefNannySetupContext("__setstate_cython__", 0);

  /* "(tree fragment)":4
 *     raise TypeError("no default __reduce__ due to non-trivial __cinit__")
 * def __setstate_cython__(self, __pyx_state):
 *     raise TypeError("no default __reduce__ due to non-trivial __cinit__")             # <<<<<<<<<<<<<<
 */
  __pyx_t_1 = __Pyx_PyObject_Call(__pyx_builtin_TypeError, __pyx_tuple__93, NULL); if (unlikely(!__pyx_t_1)) __PYX_ERR(7, 4, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_1);
  __Pyx_Raise(__pyx_t_1, 0, 0, 0);
  __Pyx_DECREF(__pyx_t_1); __pyx_t_1 = 0;
  __PYX_ERR(7, 4, __pyx_L1_error)

  /* "(tree fragment)":3
 * def __reduce_cython__(self):
 *     raise TypeError("no default __reduce__ due to non-trivial __cinit__")
 * def __setstate_cython__(self, __pyx_state):             # <<<<<<<<<<<<<<
 *     raise TypeError("no default __reduce__ due to non-trivial __cinit__")
 */

  /* function exit code */
  __pyx_L1_error:;
  __Pyx_XDECREF(__pyx_t_1);
  __Pyx_AddTraceback("grpc._cython.cygrpc.SendStatusFromServerOperation.__setstate_cython__", __pyx_clineno, __pyx_lineno, __pyx_filename);
  __pyx_r = NULL;
  __Pyx_XGIVEREF(__pyx_r);
  __Pyx_RefNannyFinishContext();
  return __pyx_r;
}

/* "src/python/grpcio/grpc/_cython/_cygrpc/operation.pyx.pxi":121
 * cdef class ReceiveInitialMetadataOperation(Operation):
 * 
 *   def __cinit__(self, flags):             # <<<<<<<<<<<<<<
 *     self._flags = flags
 * 
 */

/* Python wrapper */
static int __pyx_pw_4grpc_7_cython_6cygrpc_31ReceiveInitialMetadataOperation_1__cinit__(PyObject *__pyx_v_self, PyObject *__pyx_args, PyObject *__pyx_kwds); /*proto*/
static int __pyx_pw_4grpc_7_cython_6cygrpc_31ReceiveInitialMetadataOperation_1__cinit__(PyObject *__pyx_v_self, PyObject *__pyx_args, PyObject *__pyx_kwds) {
  PyObject *__pyx_v_flags = 0;
  int __pyx_r;
  __Pyx_RefNannyDeclarations
  __Pyx_RefNannySetupContext("__cinit__ (wrapper)", 0);
  {
    static PyObject **__pyx_pyargnames[] = {&__pyx_n_s_flags,0};
    PyObject* values[1] = {0};
    if (unlikely(__pyx_kwds)) {
      Py_ssize_t kw_args;
      const Py_ssize_t pos_args = PyTuple_GET_SIZE(__pyx_args);
      switch (pos_args) {
        case  1: values[0] = PyTuple_GET_ITEM(__pyx_args, 0);
        CYTHON_FALLTHROUGH;
        case  0: break;
        default: goto __pyx_L5_argtuple_error;
      }
      kw_args = PyDict_Size(__pyx_kwds);
      switch (pos_args) {
        case  0:
        if (likely((values[0] = __Pyx_PyDict_GetItemStr(__pyx_kwds, __pyx_n_s_flags)) != 0)) kw_args--;
        else goto __pyx_L5_argtuple_error;
      }
      if (unlikely(kw_args > 0)) {
        if (unlikely(__Pyx_ParseOptionalKeywords(__pyx_kwds, __pyx_pyargnames, 0, values, pos_args, "__cinit__") < 0)) __PYX_ERR(14, 121, __pyx_L3_error)
      }
    } else if (PyTuple_GET_SIZE(__pyx_args) != 1) {
      goto __pyx_L5_argtuple_error;
    } else {
      values[0] = PyTuple_GET_ITEM(__pyx_args, 0);
    }
    __pyx_v_flags = values[0];
  }
  goto __pyx_L4_argument_unpacking_done;
  __pyx_L5_argtuple_error:;
  __Pyx_RaiseArgtupleInvalid("__cinit__", 1, 1, 1, PyTuple_GET_SIZE(__pyx_args)); __PYX_ERR(14, 121, __pyx_L3_error)
  __pyx_L3_error:;
  __Pyx_AddTraceback("grpc._cython.cygrpc.ReceiveInitialMetadataOperation.__cinit__", __pyx_clineno, __pyx_lineno, __pyx_filename);
  __Pyx_RefNannyFinishContext();
  return -1;
  __pyx_L4_argument_unpacking_done:;
  __pyx_r = __pyx_pf_4grpc_7_cython_6cygrpc_31ReceiveInitialMetadataOperation___cinit__(((struct __pyx_obj_4grpc_7_cython_6cygrpc_ReceiveInitialMetadataOperation *)__pyx_v_self), __pyx_v_flags);

  /* function exit code */
  __Pyx_RefNannyFinishContext();
  return __pyx_r;
}

static int __pyx_pf_4grpc_7_cython_6cygrpc_31ReceiveInitialMetadataOperation___cinit__(struct __pyx_obj_4grpc_7_cython_6cygrpc_ReceiveInitialMetadataOperation *__pyx_v_self, PyObject *__pyx_v_flags) {
  int __pyx_r;
  __Pyx_RefNannyDeclarations
  int __pyx_t_1;
  __Pyx_RefNannySetupContext("__cinit__", 0);

  /* "src/python/grpcio/grpc/_cython/_cygrpc/operation.pyx.pxi":122
 * 
 *   def __cinit__(self, flags):
 *     self._flags = flags             # <<<<<<<<<<<<<<
 * 
 *   def type(self):
 */
  __pyx_t_1 = __Pyx_PyInt_As_int(__pyx_v_flags); if (unlikely((__pyx_t_1 == (int)-1) && PyErr_Occurred())) __PYX_ERR(14, 122, __pyx_L1_error)
  __pyx_v_self->_flags = __pyx_t_1;

  /* "src/python/grpcio/grpc/_cython/_cygrpc/operation.pyx.pxi":121
 * cdef class ReceiveInitialMetadataOperation(Operation):
 * 
 *   def __cinit__(self, flags):             # <<<<<<<<<<<<<<
 *     self._flags = flags
 * 
 */

  /* function exit code */
  __pyx_r = 0;
  goto __pyx_L0;
  __pyx_L1_error:;
  __Pyx_AddTraceback("grpc._cython.cygrpc.ReceiveInitialMetadataOperation.__cinit__", __pyx_clineno, __pyx_lineno, __pyx_filename);
  __pyx_r = -1;
  __pyx_L0:;
  __Pyx_RefNannyFinishContext();
  return __pyx_r;
}

/* "src/python/grpcio/grpc/_cython/_cygrpc/operation.pyx.pxi":124
 *     self._flags = flags
 * 
 *   def type(self):             # <<<<<<<<<<<<<<
 *     return GRPC_OP_RECV_INITIAL_METADATA
 * 
 */

/* Python wrapper */
static PyObject *__pyx_pw_4grpc_7_cython_6cygrpc_31ReceiveInitialMetadataOperation_3type(PyObject *__pyx_v_self, CYTHON_UNUSED PyObject *unused); /*proto*/
static PyObject *__pyx_pw_4grpc_7_cython_6cygrpc_31ReceiveInitialMetadataOperation_3type(PyObject *__pyx_v_self, CYTHON_UNUSED PyObject *unused) {
  PyObject *__pyx_r = 0;
  __Pyx_RefNannyDeclarations
  __Pyx_RefNannySetupContext("type (wrapper)", 0);
  __pyx_r = __pyx_pf_4grpc_7_cython_6cygrpc_31ReceiveInitialMetadataOperation_2type(((struct __pyx_obj_4grpc_7_cython_6cygrpc_ReceiveInitialMetadataOperation *)__pyx_v_self));

  /* function exit code */
  __Pyx_RefNannyFinishContext();
  return __pyx_r;
}

static PyObject *__pyx_pf_4grpc_7_cython_6cygrpc_31ReceiveInitialMetadataOperation_2type(CYTHON_UNUSED struct __pyx_obj_4grpc_7_cython_6cygrpc_ReceiveInitialMetadataOperation *__pyx_v_self) {
  PyObject *__pyx_r = NULL;
  __Pyx_RefNannyDeclarations
  PyObject *__pyx_t_1 = NULL;
  __Pyx_RefNannySetupContext("type", 0);

  /* "src/python/grpcio/grpc/_cython/_cygrpc/operation.pyx.pxi":125
 * 
 *   def type(self):
 *     return GRPC_OP_RECV_INITIAL_METADATA             # <<<<<<<<<<<<<<
 * 
 *   cdef void c(self) except *:
 */
  __Pyx_XDECREF(__pyx_r);
  __pyx_t_1 = __Pyx_PyInt_From_grpc_op_type(GRPC_OP_RECV_INITIAL_METADATA); if (unlikely(!__pyx_t_1)) __PYX_ERR(14, 125, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_1);
  __pyx_r = __pyx_t_1;
  __pyx_t_1 = 0;
  goto __pyx_L0;

  /* "src/python/grpcio/grpc/_cython/_cygrpc/operation.pyx.pxi":124
 *     self._flags = flags
 * 
 *   def type(self):             # <<<<<<<<<<<<<<
 *     return GRPC_OP_RECV_INITIAL_METADATA
 * 
 */

  /* function exit code */
  __pyx_L1_error:;
  __Pyx_XDECREF(__pyx_t_1);
  __Pyx_AddTraceback("grpc._cython.cygrpc.ReceiveInitialMetadataOperation.type", __pyx_clineno, __pyx_lineno, __pyx_filename);
  __pyx_r = NULL;
  __pyx_L0:;
  __Pyx_XGIVEREF(__pyx_r);
  __Pyx_RefNannyFinishContext();
  return __pyx_r;
}

/* "src/python/grpcio/grpc/_cython/_cygrpc/operation.pyx.pxi":127
 *     return GRPC_OP_RECV_INITIAL_METADATA
 * 
 *   cdef void c(self) except *:             # <<<<<<<<<<<<<<
 *     self.c_op.type = GRPC_OP_RECV_INITIAL_METADATA
 *     self.c_op.flags = self._flags
 */

static void __pyx_f_4grpc_7_cython_6cygrpc_31ReceiveInitialMetadataOperation_c(struct __pyx_obj_4grpc_7_cython_6cygrpc_ReceiveInitialMetadataOperation *__pyx_v_self) {
  __Pyx_RefNannyDeclarations
  int __pyx_t_1;
  __Pyx_RefNannySetupContext("c", 0);

  /* "src/python/grpcio/grpc/_cython/_cygrpc/operation.pyx.pxi":128
 * 
 *   cdef void c(self) except *:
 *     self.c_op.type = GRPC_OP_RECV_INITIAL_METADATA             # <<<<<<<<<<<<<<
 *     self.c_op.flags = self._flags
 *     grpc_metadata_array_init(&self._c_initial_metadata)
 */
  __pyx_v_self->__pyx_base.c_op.op = GRPC_OP_RECV_INITIAL_METADATA;

  /* "src/python/grpcio/grpc/_cython/_cygrpc/operation.pyx.pxi":129
 *   cdef void c(self) except *:
 *     self.c_op.type = GRPC_OP_RECV_INITIAL_METADATA
 *     self.c_op.flags = self._flags             # <<<<<<<<<<<<<<
 *     grpc_metadata_array_init(&self._c_initial_metadata)
 *     self.c_op.data.receive_initial_metadata.receive_initial_metadata = (
 */
  __pyx_t_1 = __pyx_v_self->_flags;
  __pyx_v_self->__pyx_base.c_op.flags = __pyx_t_1;

  /* "src/python/grpcio/grpc/_cython/_cygrpc/operation.pyx.pxi":130
 *     self.c_op.type = GRPC_OP_RECV_INITIAL_METADATA
 *     self.c_op.flags = self._flags
 *     grpc_metadata_array_init(&self._c_initial_metadata)             # <<<<<<<<<<<<<<
 *     self.c_op.data.receive_initial_metadata.receive_initial_metadata = (
 *         &self._c_initial_metadata)
 */
  grpc_metadata_array_init((&__pyx_v_self->_c_initial_metadata));

  /* "src/python/grpcio/grpc/_cython/_cygrpc/operation.pyx.pxi":131
 *     self.c_op.flags = self._flags
 *     grpc_metadata_array_init(&self._c_initial_metadata)
 *     self.c_op.data.receive_initial_metadata.receive_initial_metadata = (             # <<<<<<<<<<<<<<
 *         &self._c_initial_metadata)
 * 
 */
  __pyx_v_self->__pyx_base.c_op.data.recv_initial_metadata.recv_initial_metadata = (&__pyx_v_self->_c_initial_metadata);

  /* "src/python/grpcio/grpc/_cython/_cygrpc/operation.pyx.pxi":127
 *     return GRPC_OP_RECV_INITIAL_METADATA
 * 
 *   cdef void c(self) except *:             # <<<<<<<<<<<<<<
 *     self.c_op.type = GRPC_OP_RECV_INITIAL_METADATA
 *     self.c_op.flags = self._flags
 */

  /* function exit code */
  __Pyx_RefNannyFinishContext();
}

/* "src/python/grpcio/grpc/_cython/_cygrpc/operation.pyx.pxi":134
 *         &self._c_initial_metadata)
 * 
 *   cdef void un_c(self) except *:             # <<<<<<<<<<<<<<
 *     self._initial_metadata = _metadata(&self._c_initial_metadata)
 *     grpc_metadata_array_destroy(&self._c_initial_metadata)
 */

static void __pyx_f_4grpc_7_cython_6cygrpc_31ReceiveInitialMetadataOperation_un_c(struct __pyx_obj_4grpc_7_cython_6cygrpc_ReceiveInitialMetadataOperation *__pyx_v_self) {
  __Pyx_RefNannyDeclarations
  PyObject *__pyx_t_1 = NULL;
  __Pyx_RefNannySetupContext("un_c", 0);

  /* "src/python/grpcio/grpc/_cython/_cygrpc/operation.pyx.pxi":135
 * 
 *   cdef void un_c(self) except *:
 *     self._initial_metadata = _metadata(&self._c_initial_metadata)             # <<<<<<<<<<<<<<
 *     grpc_metadata_array_destroy(&self._c_initial_metadata)
 * 
 */
  __pyx_t_1 = __pyx_f_4grpc_7_cython_6cygrpc__metadata((&__pyx_v_self->_c_initial_metadata)); if (unlikely(!__pyx_t_1)) __PYX_ERR(14, 135, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_1);
  __Pyx_GIVEREF(__pyx_t_1);
  __Pyx_GOTREF(__pyx_v_self->_initial_metadata);
  __Pyx_DECREF(__pyx_v_self->_initial_metadata);
  __pyx_v_self->_initial_metadata = ((PyObject*)__pyx_t_1);
  __pyx_t_1 = 0;

  /* "src/python/grpcio/grpc/_cython/_cygrpc/operation.pyx.pxi":136
 *   cdef void un_c(self) except *:
 *     self._initial_metadata = _metadata(&self._c_initial_metadata)
 *     grpc_metadata_array_destroy(&self._c_initial_metadata)             # <<<<<<<<<<<<<<
 * 
 *   def initial_metadata(self):
 */
  grpc_metadata_array_destroy((&__pyx_v_self->_c_initial_metadata));

  /* "src/python/grpcio/grpc/_cython/_cygrpc/operation.pyx.pxi":134
 *         &self._c_initial_metadata)
 * 
 *   cdef void un_c(self) except *:             # <<<<<<<<<<<<<<
 *     self._initial_metadata = _metadata(&self._c_initial_metadata)
 *     grpc_metadata_array_destroy(&self._c_initial_metadata)
 */

  /* function exit code */
  goto __pyx_L0;
  __pyx_L1_error:;
  __Pyx_XDECREF(__pyx_t_1);
  __Pyx_AddTraceback("grpc._cython.cygrpc.ReceiveInitialMetadataOperation.un_c", __pyx_clineno, __pyx_lineno, __pyx_filename);
  __pyx_L0:;
  __Pyx_RefNannyFinishContext();
}

/* "src/python/grpcio/grpc/_cython/_cygrpc/operation.pyx.pxi":138
 *     grpc_metadata_array_destroy(&self._c_initial_metadata)
 * 
 *   def initial_metadata(self):             # <<<<<<<<<<<<<<
 *     return self._initial_metadata
 * 
 */

/* Python wrapper */
static PyObject *__pyx_pw_4grpc_7_cython_6cygrpc_31ReceiveInitialMetadataOperation_5initial_metadata(PyObject *__pyx_v_self, CYTHON_UNUSED PyObject *unused); /*proto*/
static PyObject *__pyx_pw_4grpc_7_cython_6cygrpc_31ReceiveInitialMetadataOperation_5initial_metadata(PyObject *__pyx_v_self, CYTHON_UNUSED PyObject *unused) {
  PyObject *__pyx_r = 0;
  __Pyx_RefNannyDeclarations
  __Pyx_RefNannySetupContext("initial_metadata (wrapper)", 0);
  __pyx_r = __pyx_pf_4grpc_7_cython_6cygrpc_31ReceiveInitialMetadataOperation_4initial_metadata(((struct __pyx_obj_4grpc_7_cython_6cygrpc_ReceiveInitialMetadataOperation *)__pyx_v_self));

  /* function exit code */
  __Pyx_RefNannyFinishContext();
  return __pyx_r;
}

static PyObject *__pyx_pf_4grpc_7_cython_6cygrpc_31ReceiveInitialMetadataOperation_4initial_metadata(struct __pyx_obj_4grpc_7_cython_6cygrpc_ReceiveInitialMetadataOperation *__pyx_v_self) {
  PyObject *__pyx_r = NULL;
  __Pyx_RefNannyDeclarations
  __Pyx_RefNannySetupContext("initial_metadata", 0);

  /* "src/python/grpcio/grpc/_cython/_cygrpc/operation.pyx.pxi":139
 * 
 *   def initial_metadata(self):
 *     return self._initial_metadata             # <<<<<<<<<<<<<<
 * 
 * 
 */
  __Pyx_XDECREF(__pyx_r);
  __Pyx_INCREF(__pyx_v_self->_initial_metadata);
  __pyx_r = __pyx_v_self->_initial_metadata;
  goto __pyx_L0;

  /* "src/python/grpcio/grpc/_cython/_cygrpc/operation.pyx.pxi":138
 *     grpc_metadata_array_destroy(&self._c_initial_metadata)
 * 
 *   def initial_metadata(self):             # <<<<<<<<<<<<<<
 *     return self._initial_metadata
 * 
 */

  /* function exit code */
  __pyx_L0:;
  __Pyx_XGIVEREF(__pyx_r);
  __Pyx_RefNannyFinishContext();
  return __pyx_r;
}

/* "src/python/grpcio/grpc/_cython/_cygrpc/operation.pxd.pxi":70
 * cdef class ReceiveInitialMetadataOperation(Operation):
 * 
 *   cdef readonly int _flags             # <<<<<<<<<<<<<<
 *   cdef tuple _initial_metadata
 *   cdef grpc_metadata_array _c_initial_metadata
 */

/* Python wrapper */
static PyObject *__pyx_pw_4grpc_7_cython_6cygrpc_31ReceiveInitialMetadataOperation_6_flags_1__get__(PyObject *__pyx_v_self); /*proto*/
static PyObject *__pyx_pw_4grpc_7_cython_6cygrpc_31ReceiveInitialMetadataOperation_6_flags_1__get__(PyObject *__pyx_v_self) {
  PyObject *__pyx_r = 0;
  __Pyx_RefNannyDeclarations
  __Pyx_RefNannySetupContext("__get__ (wrapper)", 0);
  __pyx_r = __pyx_pf_4grpc_7_cython_6cygrpc_31ReceiveInitialMetadataOperation_6_flags___get__(((struct __pyx_obj_4grpc_7_cython_6cygrpc_ReceiveInitialMetadataOperation *)__pyx_v_self));

  /* function exit code */
  __Pyx_RefNannyFinishContext();
  return __pyx_r;
}

static PyObject *__pyx_pf_4grpc_7_cython_6cygrpc_31ReceiveInitialMetadataOperation_6_flags___get__(struct __pyx_obj_4grpc_7_cython_6cygrpc_ReceiveInitialMetadataOperation *__pyx_v_self) {
  PyObject *__pyx_r = NULL;
  __Pyx_RefNannyDeclarations
  PyObject *__pyx_t_1 = NULL;
  __Pyx_RefNannySetupContext("__get__", 0);
  __Pyx_XDECREF(__pyx_r);
  __pyx_t_1 = __Pyx_PyInt_From_int(__pyx_v_self->_flags); if (unlikely(!__pyx_t_1)) __PYX_ERR(15, 70, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_1);
  __pyx_r = __pyx_t_1;
  __pyx_t_1 = 0;
  goto __pyx_L0;

  /* function exit code */
  __pyx_L1_error:;
  __Pyx_XDECREF(__pyx_t_1);
  __Pyx_AddTraceback("grpc._cython.cygrpc.ReceiveInitialMetadataOperation._flags.__get__", __pyx_clineno, __pyx_lineno, __pyx_filename);
  __pyx_r = NULL;
  __pyx_L0:;
  __Pyx_XGIVEREF(__pyx_r);
  __Pyx_RefNannyFinishContext();
  return __pyx_r;
}

/* "(tree fragment)":1
 * def __reduce_cython__(self):             # <<<<<<<<<<<<<<
 *     raise TypeError("no default __reduce__ due to non-trivial __cinit__")
 * def __setstate_cython__(self, __pyx_state):
 */

/* Python wrapper */
static PyObject *__pyx_pw_4grpc_7_cython_6cygrpc_31ReceiveInitialMetadataOperation_7__reduce_cython__(PyObject *__pyx_v_self, CYTHON_UNUSED PyObject *unused); /*proto*/
static PyObject *__pyx_pw_4grpc_7_cython_6cygrpc_31ReceiveInitialMetadataOperation_7__reduce_cython__(PyObject *__pyx_v_self, CYTHON_UNUSED PyObject *unused) {
  PyObject *__pyx_r = 0;
  __Pyx_RefNannyDeclarations
  __Pyx_RefNannySetupContext("__reduce_cython__ (wrapper)", 0);
  __pyx_r = __pyx_pf_4grpc_7_cython_6cygrpc_31ReceiveInitialMetadataOperation_6__reduce_cython__(((struct __pyx_obj_4grpc_7_cython_6cygrpc_ReceiveInitialMetadataOperation *)__pyx_v_self));

  /* function exit code */
  __Pyx_RefNannyFinishContext();
  return __pyx_r;
}

static PyObject *__pyx_pf_4grpc_7_cython_6cygrpc_31ReceiveInitialMetadataOperation_6__reduce_cython__(CYTHON_UNUSED struct __pyx_obj_4grpc_7_cython_6cygrpc_ReceiveInitialMetadataOperation *__pyx_v_self) {
  PyObject *__pyx_r = NULL;
  __Pyx_RefNannyDeclarations
  PyObject *__pyx_t_1 = NULL;
  __Pyx_RefNannySetupContext("__reduce_cython__", 0);

  /* "(tree fragment)":2
 * def __reduce_cython__(self):
 *     raise TypeError("no default __reduce__ due to non-trivial __cinit__")             # <<<<<<<<<<<<<<
 * def __setstate_cython__(self, __pyx_state):
 *     raise TypeError("no default __reduce__ due to non-trivial __cinit__")
 */
  __pyx_t_1 = __Pyx_PyObject_Call(__pyx_builtin_TypeError, __pyx_tuple__94, NULL); if (unlikely(!__pyx_t_1)) __PYX_ERR(7, 2, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_1);
  __Pyx_Raise(__pyx_t_1, 0, 0, 0);
  __Pyx_DECREF(__pyx_t_1); __pyx_t_1 = 0;
  __PYX_ERR(7, 2, __pyx_L1_error)

  /* "(tree fragment)":1
 * def __reduce_cython__(self):             # <<<<<<<<<<<<<<
 *     raise TypeError("no default __reduce__ due to non-trivial __cinit__")
 * def __setstate_cython__(self, __pyx_state):
 */

  /* function exit code */
  __pyx_L1_error:;
  __Pyx_XDECREF(__pyx_t_1);
  __Pyx_AddTraceback("grpc._cython.cygrpc.ReceiveInitialMetadataOperation.__reduce_cython__", __pyx_clineno, __pyx_lineno, __pyx_filename);
  __pyx_r = NULL;
  __Pyx_XGIVEREF(__pyx_r);
  __Pyx_RefNannyFinishContext();
  return __pyx_r;
}

/* "(tree fragment)":3
 * def __reduce_cython__(self):
 *     raise TypeError("no default __reduce__ due to non-trivial __cinit__")
 * def __setstate_cython__(self, __pyx_state):             # <<<<<<<<<<<<<<
 *     raise TypeError("no default __reduce__ due to non-trivial __cinit__")
 */

/* Python wrapper */
static PyObject *__pyx_pw_4grpc_7_cython_6cygrpc_31ReceiveInitialMetadataOperation_9__setstate_cython__(PyObject *__pyx_v_self, PyObject *__pyx_v___pyx_state); /*proto*/
static PyObject *__pyx_pw_4grpc_7_cython_6cygrpc_31ReceiveInitialMetadataOperation_9__setstate_cython__(PyObject *__pyx_v_self, PyObject *__pyx_v___pyx_state) {
  PyObject *__pyx_r = 0;
  __Pyx_RefNannyDeclarations
  __Pyx_RefNannySetupContext("__setstate_cython__ (wrapper)", 0);
  __pyx_r = __pyx_pf_4grpc_7_cython_6cygrpc_31ReceiveInitialMetadataOperation_8__setstate_cython__(((struct __pyx_obj_4grpc_7_cython_6cygrpc_ReceiveInitialMetadataOperation *)__pyx_v_self), ((PyObject *)__pyx_v___pyx_state));

  /* function exit code */
  __Pyx_RefNannyFinishContext();
  return __pyx_r;
}

static PyObject *__pyx_pf_4grpc_7_cython_6cygrpc_31ReceiveInitialMetadataOperation_8__setstate_cython__(CYTHON_UNUSED struct __pyx_obj_4grpc_7_cython_6cygrpc_ReceiveInitialMetadataOperation *__pyx_v_self, CYTHON_UNUSED PyObject *__pyx_v___pyx_state) {
  PyObject *__pyx_r = NULL;
  __Pyx_RefNannyDeclarations
  PyObject *__pyx_t_1 = NULL;
  __Pyx_RefNannySetupContext("__setstate_cython__", 0);

  /* "(tree fragment)":4
 *     raise TypeError("no default __reduce__ due to non-trivial __cinit__")
 * def __setstate_cython__(self, __pyx_state):
 *     raise TypeError("no default __reduce__ due to non-trivial __cinit__")             # <<<<<<<<<<<<<<
 */
  __pyx_t_1 = __Pyx_PyObject_Call(__pyx_builtin_TypeError, __pyx_tuple__95, NULL); if (unlikely(!__pyx_t_1)) __PYX_ERR(7, 4, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_1);
  __Pyx_Raise(__pyx_t_1, 0, 0, 0);
  __Pyx_DECREF(__pyx_t_1); __pyx_t_1 = 0;
  __PYX_ERR(7, 4, __pyx_L1_error)

  /* "(tree fragment)":3
 * def __reduce_cython__(self):
 *     raise TypeError("no default __reduce__ due to non-trivial __cinit__")
 * def __setstate_cython__(self, __pyx_state):             # <<<<<<<<<<<<<<
 *     raise TypeError("no default __reduce__ due to non-trivial __cinit__")
 */

  /* function exit code */
  __pyx_L1_error:;
  __Pyx_XDECREF(__pyx_t_1);
  __Pyx_AddTraceback("grpc._cython.cygrpc.ReceiveInitialMetadataOperation.__setstate_cython__", __pyx_clineno, __pyx_lineno, __pyx_filename);
  __pyx_r = NULL;
  __Pyx_XGIVEREF(__pyx_r);
  __Pyx_RefNannyFinishContext();
  return __pyx_r;
}

/* "src/python/grpcio/grpc/_cython/_cygrpc/operation.pyx.pxi":144
 * cdef class ReceiveMessageOperation(Operation):
 * 
 *   def __cinit__(self, flags):             # <<<<<<<<<<<<<<
 *     self._flags = flags
 * 
 */

/* Python wrapper */
static int __pyx_pw_4grpc_7_cython_6cygrpc_23ReceiveMessageOperation_1__cinit__(PyObject *__pyx_v_self, PyObject *__pyx_args, PyObject *__pyx_kwds); /*proto*/
static int __pyx_pw_4grpc_7_cython_6cygrpc_23ReceiveMessageOperation_1__cinit__(PyObject *__pyx_v_self, PyObject *__pyx_args, PyObject *__pyx_kwds) {
  PyObject *__pyx_v_flags = 0;
  int __pyx_r;
  __Pyx_RefNannyDeclarations
  __Pyx_RefNannySetupContext("__cinit__ (wrapper)", 0);
  {
    static PyObject **__pyx_pyargnames[] = {&__pyx_n_s_flags,0};
    PyObject* values[1] = {0};
    if (unlikely(__pyx_kwds)) {
      Py_ssize_t kw_args;
      const Py_ssize_t pos_args = PyTuple_GET_SIZE(__pyx_args);
      switch (pos_args) {
        case  1: values[0] = PyTuple_GET_ITEM(__pyx_args, 0);
        CYTHON_FALLTHROUGH;
        case  0: break;
        default: goto __pyx_L5_argtuple_error;
      }
      kw_args = PyDict_Size(__pyx_kwds);
      switch (pos_args) {
        case  0:
        if (likely((values[0] = __Pyx_PyDict_GetItemStr(__pyx_kwds, __pyx_n_s_flags)) != 0)) kw_args--;
        else goto __pyx_L5_argtuple_error;
      }
      if (unlikely(kw_args > 0)) {
        if (unlikely(__Pyx_ParseOptionalKeywords(__pyx_kwds, __pyx_pyargnames, 0, values, pos_args, "__cinit__") < 0)) __PYX_ERR(14, 144, __pyx_L3_error)
      }
    } else if (PyTuple_GET_SIZE(__pyx_args) != 1) {
      goto __pyx_L5_argtuple_error;
    } else {
      values[0] = PyTuple_GET_ITEM(__pyx_args, 0);
    }
    __pyx_v_flags = values[0];
  }
  goto __pyx_L4_argument_unpacking_done;
  __pyx_L5_argtuple_error:;
  __Pyx_RaiseArgtupleInvalid("__cinit__", 1, 1, 1, PyTuple_GET_SIZE(__pyx_args)); __PYX_ERR(14, 144, __pyx_L3_error)
  __pyx_L3_error:;
  __Pyx_AddTraceback("grpc._cython.cygrpc.ReceiveMessageOperation.__cinit__", __pyx_clineno, __pyx_lineno, __pyx_filename);
  __Pyx_RefNannyFinishContext();
  return -1;
  __pyx_L4_argument_unpacking_done:;
  __pyx_r = __pyx_pf_4grpc_7_cython_6cygrpc_23ReceiveMessageOperation___cinit__(((struct __pyx_obj_4grpc_7_cython_6cygrpc_ReceiveMessageOperation *)__pyx_v_self), __pyx_v_flags);

  /* function exit code */
  __Pyx_RefNannyFinishContext();
  return __pyx_r;
}

static int __pyx_pf_4grpc_7_cython_6cygrpc_23ReceiveMessageOperation___cinit__(struct __pyx_obj_4grpc_7_cython_6cygrpc_ReceiveMessageOperation *__pyx_v_self, PyObject *__pyx_v_flags) {
  int __pyx_r;
  __Pyx_RefNannyDeclarations
  int __pyx_t_1;
  __Pyx_RefNannySetupContext("__cinit__", 0);

  /* "src/python/grpcio/grpc/_cython/_cygrpc/operation.pyx.pxi":145
 * 
 *   def __cinit__(self, flags):
 *     self._flags = flags             # <<<<<<<<<<<<<<
 * 
 *   def type(self):
 */
  __pyx_t_1 = __Pyx_PyInt_As_int(__pyx_v_flags); if (unlikely((__pyx_t_1 == (int)-1) && PyErr_Occurred())) __PYX_ERR(14, 145, __pyx_L1_error)
  __pyx_v_self->_flags = __pyx_t_1;

  /* "src/python/grpcio/grpc/_cython/_cygrpc/operation.pyx.pxi":144
 * cdef class ReceiveMessageOperation(Operation):
 * 
 *   def __cinit__(self, flags):             # <<<<<<<<<<<<<<
 *     self._flags = flags
 * 
 */

  /* function exit code */
  __pyx_r = 0;
  goto __pyx_L0;
  __pyx_L1_error:;
  __Pyx_AddTraceback("grpc._cython.cygrpc.ReceiveMessageOperation.__cinit__", __pyx_clineno, __pyx_lineno, __pyx_filename);
  __pyx_r = -1;
  __pyx_L0:;
  __Pyx_RefNannyFinishContext();
  return __pyx_r;
}

/* "src/python/grpcio/grpc/_cython/_cygrpc/operation.pyx.pxi":147
 *     self._flags = flags
 * 
 *   def type(self):             # <<<<<<<<<<<<<<
 *     return GRPC_OP_RECV_MESSAGE
 * 
 */

/* Python wrapper */
static PyObject *__pyx_pw_4grpc_7_cython_6cygrpc_23ReceiveMessageOperation_3type(PyObject *__pyx_v_self, CYTHON_UNUSED PyObject *unused); /*proto*/
static PyObject *__pyx_pw_4grpc_7_cython_6cygrpc_23ReceiveMessageOperation_3type(PyObject *__pyx_v_self, CYTHON_UNUSED PyObject *unused) {
  PyObject *__pyx_r = 0;
  __Pyx_RefNannyDeclarations
  __Pyx_RefNannySetupContext("type (wrapper)", 0);
  __pyx_r = __pyx_pf_4grpc_7_cython_6cygrpc_23ReceiveMessageOperation_2type(((struct __pyx_obj_4grpc_7_cython_6cygrpc_ReceiveMessageOperation *)__pyx_v_self));

  /* function exit code */
  __Pyx_RefNannyFinishContext();
  return __pyx_r;
}

static PyObject *__pyx_pf_4grpc_7_cython_6cygrpc_23ReceiveMessageOperation_2type(CYTHON_UNUSED struct __pyx_obj_4grpc_7_cython_6cygrpc_ReceiveMessageOperation *__pyx_v_self) {
  PyObject *__pyx_r = NULL;
  __Pyx_RefNannyDeclarations
  PyObject *__pyx_t_1 = NULL;
  __Pyx_RefNannySetupContext("type", 0);

  /* "src/python/grpcio/grpc/_cython/_cygrpc/operation.pyx.pxi":148
 * 
 *   def type(self):
 *     return GRPC_OP_RECV_MESSAGE             # <<<<<<<<<<<<<<
 * 
 *   cdef void c(self) except *:
 */
  __Pyx_XDECREF(__pyx_r);
  __pyx_t_1 = __Pyx_PyInt_From_grpc_op_type(GRPC_OP_RECV_MESSAGE); if (unlikely(!__pyx_t_1)) __PYX_ERR(14, 148, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_1);
  __pyx_r = __pyx_t_1;
  __pyx_t_1 = 0;
  goto __pyx_L0;

  /* "src/python/grpcio/grpc/_cython/_cygrpc/operation.pyx.pxi":147
 *     self._flags = flags
 * 
 *   def type(self):             # <<<<<<<<<<<<<<
 *     return GRPC_OP_RECV_MESSAGE
 * 
 */

  /* function exit code */
  __pyx_L1_error:;
  __Pyx_XDECREF(__pyx_t_1);
  __Pyx_AddTraceback("grpc._cython.cygrpc.ReceiveMessageOperation.type", __pyx_clineno, __pyx_lineno, __pyx_filename);
  __pyx_r = NULL;
  __pyx_L0:;
  __Pyx_XGIVEREF(__pyx_r);
  __Pyx_RefNannyFinishContext();
  return __pyx_r;
}

/* "src/python/grpcio/grpc/_cython/_cygrpc/operation.pyx.pxi":150
 *     return GRPC_OP_RECV_MESSAGE
 * 
 *   cdef void c(self) except *:             # <<<<<<<<<<<<<<
 *     self.c_op.type = GRPC_OP_RECV_MESSAGE
 *     self.c_op.flags = self._flags
 */

static void __pyx_f_4grpc_7_cython_6cygrpc_23ReceiveMessageOperation_c(struct __pyx_obj_4grpc_7_cython_6cygrpc_ReceiveMessageOperation *__pyx_v_self) {
  __Pyx_RefNannyDeclarations
  int __pyx_t_1;
  __Pyx_RefNannySetupContext("c", 0);

  /* "src/python/grpcio/grpc/_cython/_cygrpc/operation.pyx.pxi":151
 * 
 *   cdef void c(self) except *:
 *     self.c_op.type = GRPC_OP_RECV_MESSAGE             # <<<<<<<<<<<<<<
 *     self.c_op.flags = self._flags
 *     self.c_op.data.receive_message.receive_message = (
 */
  __pyx_v_self->__pyx_base.c_op.op = GRPC_OP_RECV_MESSAGE;

  /* "src/python/grpcio/grpc/_cython/_cygrpc/operation.pyx.pxi":152
 *   cdef void c(self) except *:
 *     self.c_op.type = GRPC_OP_RECV_MESSAGE
 *     self.c_op.flags = self._flags             # <<<<<<<<<<<<<<
 *     self.c_op.data.receive_message.receive_message = (
 *         &self._c_message_byte_buffer)
 */
  __pyx_t_1 = __pyx_v_self->_flags;
  __pyx_v_self->__pyx_base.c_op.flags = __pyx_t_1;

  /* "src/python/grpcio/grpc/_cython/_cygrpc/operation.pyx.pxi":153
 *     self.c_op.type = GRPC_OP_RECV_MESSAGE
 *     self.c_op.flags = self._flags
 *     self.c_op.data.receive_message.receive_message = (             # <<<<<<<<<<<<<<
 *         &self._c_message_byte_buffer)
 * 
 */
  __pyx_v_self->__pyx_base.c_op.data.recv_message.recv_message = (&__pyx_v_self->_c_message_byte_buffer);

  /* "src/python/grpcio/grpc/_cython/_cygrpc/operation.pyx.pxi":150
 *     return GRPC_OP_RECV_MESSAGE
 * 
 *   cdef void c(self) except *:             # <<<<<<<<<<<<<<
 *     self.c_op.type = GRPC_OP_RECV_MESSAGE
 *     self.c_op.flags = self._flags
 */

  /* function exit code */
  __Pyx_RefNannyFinishContext();
}

/* "src/python/grpcio/grpc/_cython/_cygrpc/operation.pyx.pxi":156
 *         &self._c_message_byte_buffer)
 * 
 *   cdef void un_c(self) except *:             # <<<<<<<<<<<<<<
 *     cdef grpc_byte_buffer_reader message_reader
 *     cdef bint message_reader_status
 */

static void __pyx_f_4grpc_7_cython_6cygrpc_23ReceiveMessageOperation_un_c(struct __pyx_obj_4grpc_7_cython_6cygrpc_ReceiveMessageOperation *__pyx_v_self) {
  struct grpc_byte_buffer_reader __pyx_v_message_reader;
  int __pyx_v_message_reader_status;
  grpc_slice __pyx_v_message_slice;
  size_t __pyx_v_message_slice_length;
  void *__pyx_v_message_slice_pointer;
  PyObject *__pyx_v_message = NULL;
  __Pyx_RefNannyDeclarations
  int __pyx_t_1;
  PyObject *__pyx_t_2 = NULL;
  PyObject *__pyx_t_3 = NULL;
  __Pyx_RefNannySetupContext("un_c", 0);

  /* "src/python/grpcio/grpc/_cython/_cygrpc/operation.pyx.pxi":162
 *     cdef size_t message_slice_length
 *     cdef void *message_slice_pointer
 *     if self._c_message_byte_buffer != NULL:             # <<<<<<<<<<<<<<
 *       message_reader_status = grpc_byte_buffer_reader_init(
 *           &message_reader, self._c_message_byte_buffer)
 */
  __pyx_t_1 = ((__pyx_v_self->_c_message_byte_buffer != NULL) != 0);
  if (__pyx_t_1) {

    /* "src/python/grpcio/grpc/_cython/_cygrpc/operation.pyx.pxi":163
 *     cdef void *message_slice_pointer
 *     if self._c_message_byte_buffer != NULL:
 *       message_reader_status = grpc_byte_buffer_reader_init(             # <<<<<<<<<<<<<<
 *           &message_reader, self._c_message_byte_buffer)
 *       if message_reader_status:
 */
    __pyx_v_message_reader_status = grpc_byte_buffer_reader_init((&__pyx_v_message_reader), __pyx_v_self->_c_message_byte_buffer);

    /* "src/python/grpcio/grpc/_cython/_cygrpc/operation.pyx.pxi":165
 *       message_reader_status = grpc_byte_buffer_reader_init(
 *           &message_reader, self._c_message_byte_buffer)
 *       if message_reader_status:             # <<<<<<<<<<<<<<
 *         message = bytearray()
 *         while grpc_byte_buffer_reader_next(&message_reader, &message_slice):
 */
    __pyx_t_1 = (__pyx_v_message_reader_status != 0);
    if (__pyx_t_1) {

      /* "src/python/grpcio/grpc/_cython/_cygrpc/operation.pyx.pxi":166
 *           &message_reader, self._c_message_byte_buffer)
 *       if message_reader_status:
 *         message = bytearray()             # <<<<<<<<<<<<<<
 *         while grpc_byte_buffer_reader_next(&message_reader, &message_slice):
 *           message_slice_pointer = grpc_slice_start_ptr(message_slice)
 */
      __pyx_t_2 = __Pyx_PyObject_CallNoArg(((PyObject *)(&PyByteArray_Type))); if (unlikely(!__pyx_t_2)) __PYX_ERR(14, 166, __pyx_L1_error)
      __Pyx_GOTREF(__pyx_t_2);
      __pyx_v_message = ((PyObject*)__pyx_t_2);
      __pyx_t_2 = 0;

      /* "src/python/grpcio/grpc/_cython/_cygrpc/operation.pyx.pxi":167
 *       if message_reader_status:
 *         message = bytearray()
 *         while grpc_byte_buffer_reader_next(&message_reader, &message_slice):             # <<<<<<<<<<<<<<
 *           message_slice_pointer = grpc_slice_start_ptr(message_slice)
 *           message_slice_length = grpc_slice_length(message_slice)
 */
      while (1) {
        __pyx_t_1 = (grpc_byte_buffer_reader_next((&__pyx_v_message_reader), (&__pyx_v_message_slice)) != 0);
        if (!__pyx_t_1) break;

        /* "src/python/grpcio/grpc/_cython/_cygrpc/operation.pyx.pxi":168
 *         message = bytearray()
 *         while grpc_byte_buffer_reader_next(&message_reader, &message_slice):
 *           message_slice_pointer = grpc_slice_start_ptr(message_slice)             # <<<<<<<<<<<<<<
 *           message_slice_length = grpc_slice_length(message_slice)
 *           message += (<char *>message_slice_pointer)[:message_slice_length]
 */
        __pyx_v_message_slice_pointer = GRPC_SLICE_START_PTR(__pyx_v_message_slice);

        /* "src/python/grpcio/grpc/_cython/_cygrpc/operation.pyx.pxi":169
 *         while grpc_byte_buffer_reader_next(&message_reader, &message_slice):
 *           message_slice_pointer = grpc_slice_start_ptr(message_slice)
 *           message_slice_length = grpc_slice_length(message_slice)             # <<<<<<<<<<<<<<
 *           message += (<char *>message_slice_pointer)[:message_slice_length]
 *           grpc_slice_unref(message_slice)
 */
        __pyx_v_message_slice_length = GRPC_SLICE_LENGTH(__pyx_v_message_slice);

        /* "src/python/grpcio/grpc/_cython/_cygrpc/operation.pyx.pxi":170
 *           message_slice_pointer = grpc_slice_start_ptr(message_slice)
 *           message_slice_length = grpc_slice_length(message_slice)
 *           message += (<char *>message_slice_pointer)[:message_slice_length]             # <<<<<<<<<<<<<<
 *           grpc_slice_unref(message_slice)
 *         grpc_byte_buffer_reader_destroy(&message_reader)
 */
        __pyx_t_2 = __Pyx_PyBytes_FromStringAndSize(((char *)__pyx_v_message_slice_pointer) + 0, __pyx_v_message_slice_length - 0); if (unlikely(!__pyx_t_2)) __PYX_ERR(14, 170, __pyx_L1_error)
        __Pyx_GOTREF(__pyx_t_2);
        __pyx_t_3 = PyNumber_InPlaceAdd(__pyx_v_message, __pyx_t_2); if (unlikely(!__pyx_t_3)) __PYX_ERR(14, 170, __pyx_L1_error)
        __Pyx_GOTREF(__pyx_t_3);
        __Pyx_DECREF(__pyx_t_2); __pyx_t_2 = 0;
        __Pyx_DECREF_SET(__pyx_v_message, ((PyObject*)__pyx_t_3));
        __pyx_t_3 = 0;

        /* "src/python/grpcio/grpc/_cython/_cygrpc/operation.pyx.pxi":171
 *           message_slice_length = grpc_slice_length(message_slice)
 *           message += (<char *>message_slice_pointer)[:message_slice_length]
 *           grpc_slice_unref(message_slice)             # <<<<<<<<<<<<<<
 *         grpc_byte_buffer_reader_destroy(&message_reader)
 *         self._message = bytes(message)
 */
        grpc_slice_unref(__pyx_v_message_slice);
      }

      /* "src/python/grpcio/grpc/_cython/_cygrpc/operation.pyx.pxi":172
 *           message += (<char *>message_slice_pointer)[:message_slice_length]
 *           grpc_slice_unref(message_slice)
 *         grpc_byte_buffer_reader_destroy(&message_reader)             # <<<<<<<<<<<<<<
 *         self._message = bytes(message)
 *       else:
 */
      grpc_byte_buffer_reader_destroy((&__pyx_v_message_reader));

      /* "src/python/grpcio/grpc/_cython/_cygrpc/operation.pyx.pxi":173
 *           grpc_slice_unref(message_slice)
 *         grpc_byte_buffer_reader_destroy(&message_reader)
 *         self._message = bytes(message)             # <<<<<<<<<<<<<<
 *       else:
 *         self._message = None
 */
      __pyx_t_3 = __Pyx_PyObject_CallOneArg(((PyObject *)(&PyBytes_Type)), __pyx_v_message); if (unlikely(!__pyx_t_3)) __PYX_ERR(14, 173, __pyx_L1_error)
      __Pyx_GOTREF(__pyx_t_3);
      __Pyx_GIVEREF(__pyx_t_3);
      __Pyx_GOTREF(__pyx_v_self->_message);
      __Pyx_DECREF(__pyx_v_self->_message);
      __pyx_v_self->_message = ((PyObject*)__pyx_t_3);
      __pyx_t_3 = 0;

      /* "src/python/grpcio/grpc/_cython/_cygrpc/operation.pyx.pxi":165
 *       message_reader_status = grpc_byte_buffer_reader_init(
 *           &message_reader, self._c_message_byte_buffer)
 *       if message_reader_status:             # <<<<<<<<<<<<<<
 *         message = bytearray()
 *         while grpc_byte_buffer_reader_next(&message_reader, &message_slice):
 */
      goto __pyx_L4;
    }

    /* "src/python/grpcio/grpc/_cython/_cygrpc/operation.pyx.pxi":175
 *         self._message = bytes(message)
 *       else:
 *         self._message = None             # <<<<<<<<<<<<<<
 *       grpc_byte_buffer_destroy(self._c_message_byte_buffer)
 *     else:
 */
    /*else*/ {
      __Pyx_INCREF(Py_None);
      __Pyx_GIVEREF(Py_None);
      __Pyx_GOTREF(__pyx_v_self->_message);
      __Pyx_DECREF(__pyx_v_self->_message);
      __pyx_v_self->_message = ((PyObject*)Py_None);
    }
    __pyx_L4:;

    /* "src/python/grpcio/grpc/_cython/_cygrpc/operation.pyx.pxi":176
 *       else:
 *         self._message = None
 *       grpc_byte_buffer_destroy(self._c_message_byte_buffer)             # <<<<<<<<<<<<<<
 *     else:
 *       self._message = None
 */
    grpc_byte_buffer_destroy(__pyx_v_self->_c_message_byte_buffer);

    /* "src/python/grpcio/grpc/_cython/_cygrpc/operation.pyx.pxi":162
 *     cdef size_t message_slice_length
 *     cdef void *message_slice_pointer
 *     if self._c_message_byte_buffer != NULL:             # <<<<<<<<<<<<<<
 *       message_reader_status = grpc_byte_buffer_reader_init(
 *           &message_reader, self._c_message_byte_buffer)
 */
    goto __pyx_L3;
  }

  /* "src/python/grpcio/grpc/_cython/_cygrpc/operation.pyx.pxi":178
 *       grpc_byte_buffer_destroy(self._c_message_byte_buffer)
 *     else:
 *       self._message = None             # <<<<<<<<<<<<<<
 * 
 *   def message(self):
 */
  /*else*/ {
    __Pyx_INCREF(Py_None);
    __Pyx_GIVEREF(Py_None);
    __Pyx_GOTREF(__pyx_v_self->_message);
    __Pyx_DECREF(__pyx_v_self->_message);
    __pyx_v_self->_message = ((PyObject*)Py_None);
  }
  __pyx_L3:;

  /* "src/python/grpcio/grpc/_cython/_cygrpc/operation.pyx.pxi":156
 *         &self._c_message_byte_buffer)
 * 
 *   cdef void un_c(self) except *:             # <<<<<<<<<<<<<<
 *     cdef grpc_byte_buffer_reader message_reader
 *     cdef bint message_reader_status
 */

  /* function exit code */
  goto __pyx_L0;
  __pyx_L1_error:;
  __Pyx_XDECREF(__pyx_t_2);
  __Pyx_XDECREF(__pyx_t_3);
  __Pyx_AddTraceback("grpc._cython.cygrpc.ReceiveMessageOperation.un_c", __pyx_clineno, __pyx_lineno, __pyx_filename);
  __pyx_L0:;
  __Pyx_XDECREF(__pyx_v_message);
  __Pyx_RefNannyFinishContext();
}

/* "src/python/grpcio/grpc/_cython/_cygrpc/operation.pyx.pxi":180
 *       self._message = None
 * 
 *   def message(self):             # <<<<<<<<<<<<<<
 *     return self._message
 * 
 */

/* Python wrapper */
static PyObject *__pyx_pw_4grpc_7_cython_6cygrpc_23ReceiveMessageOperation_5message(PyObject *__pyx_v_self, CYTHON_UNUSED PyObject *unused); /*proto*/
static PyObject *__pyx_pw_4grpc_7_cython_6cygrpc_23ReceiveMessageOperation_5message(PyObject *__pyx_v_self, CYTHON_UNUSED PyObject *unused) {
  PyObject *__pyx_r = 0;
  __Pyx_RefNannyDeclarations
  __Pyx_RefNannySetupContext("message (wrapper)", 0);
  __pyx_r = __pyx_pf_4grpc_7_cython_6cygrpc_23ReceiveMessageOperation_4message(((struct __pyx_obj_4grpc_7_cython_6cygrpc_ReceiveMessageOperation *)__pyx_v_self));

  /* function exit code */
  __Pyx_RefNannyFinishContext();
  return __pyx_r;
}

static PyObject *__pyx_pf_4grpc_7_cython_6cygrpc_23ReceiveMessageOperation_4message(struct __pyx_obj_4grpc_7_cython_6cygrpc_ReceiveMessageOperation *__pyx_v_self) {
  PyObject *__pyx_r = NULL;
  __Pyx_RefNannyDeclarations
  __Pyx_RefNannySetupContext("message", 0);

  /* "src/python/grpcio/grpc/_cython/_cygrpc/operation.pyx.pxi":181
 * 
 *   def message(self):
 *     return self._message             # <<<<<<<<<<<<<<
 * 
 * 
 */
  __Pyx_XDECREF(__pyx_r);
  __Pyx_INCREF(__pyx_v_self->_message);
  __pyx_r = __pyx_v_self->_message;
  goto __pyx_L0;

  /* "src/python/grpcio/grpc/_cython/_cygrpc/operation.pyx.pxi":180
 *       self._message = None
 * 
 *   def message(self):             # <<<<<<<<<<<<<<
 *     return self._message
 * 
 */

  /* function exit code */
  __pyx_L0:;
  __Pyx_XGIVEREF(__pyx_r);
  __Pyx_RefNannyFinishContext();
  return __pyx_r;
}

/* "src/python/grpcio/grpc/_cython/_cygrpc/operation.pxd.pxi":80
 * cdef class ReceiveMessageOperation(Operation):
 * 
 *   cdef readonly int _flags             # <<<<<<<<<<<<<<
 *   cdef grpc_byte_buffer *_c_message_byte_buffer
 *   cdef bytes _message
 */

/* Python wrapper */
static PyObject *__pyx_pw_4grpc_7_cython_6cygrpc_23ReceiveMessageOperation_6_flags_1__get__(PyObject *__pyx_v_self); /*proto*/
static PyObject *__pyx_pw_4grpc_7_cython_6cygrpc_23ReceiveMessageOperation_6_flags_1__get__(PyObject *__pyx_v_self) {
  PyObject *__pyx_r = 0;
  __Pyx_RefNannyDeclarations
  __Pyx_RefNannySetupContext("__get__ (wrapper)", 0);
  __pyx_r = __pyx_pf_4grpc_7_cython_6cygrpc_23ReceiveMessageOperation_6_flags___get__(((struct __pyx_obj_4grpc_7_cython_6cygrpc_ReceiveMessageOperation *)__pyx_v_self));

  /* function exit code */
  __Pyx_RefNannyFinishContext();
  return __pyx_r;
}

static PyObject *__pyx_pf_4grpc_7_cython_6cygrpc_23ReceiveMessageOperation_6_flags___get__(struct __pyx_obj_4grpc_7_cython_6cygrpc_ReceiveMessageOperation *__pyx_v_self) {
  PyObject *__pyx_r = NULL;
  __Pyx_RefNannyDeclarations
  PyObject *__pyx_t_1 = NULL;
  __Pyx_RefNannySetupContext("__get__", 0);
  __Pyx_XDECREF(__pyx_r);
  __pyx_t_1 = __Pyx_PyInt_From_int(__pyx_v_self->_flags); if (unlikely(!__pyx_t_1)) __PYX_ERR(15, 80, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_1);
  __pyx_r = __pyx_t_1;
  __pyx_t_1 = 0;
  goto __pyx_L0;

  /* function exit code */
  __pyx_L1_error:;
  __Pyx_XDECREF(__pyx_t_1);
  __Pyx_AddTraceback("grpc._cython.cygrpc.ReceiveMessageOperation._flags.__get__", __pyx_clineno, __pyx_lineno, __pyx_filename);
  __pyx_r = NULL;
  __pyx_L0:;
  __Pyx_XGIVEREF(__pyx_r);
  __Pyx_RefNannyFinishContext();
  return __pyx_r;
}

/* "(tree fragment)":1
 * def __reduce_cython__(self):             # <<<<<<<<<<<<<<
 *     raise TypeError("no default __reduce__ due to non-trivial __cinit__")
 * def __setstate_cython__(self, __pyx_state):
 */

/* Python wrapper */
static PyObject *__pyx_pw_4grpc_7_cython_6cygrpc_23ReceiveMessageOperation_7__reduce_cython__(PyObject *__pyx_v_self, CYTHON_UNUSED PyObject *unused); /*proto*/
static PyObject *__pyx_pw_4grpc_7_cython_6cygrpc_23ReceiveMessageOperation_7__reduce_cython__(PyObject *__pyx_v_self, CYTHON_UNUSED PyObject *unused) {
  PyObject *__pyx_r = 0;
  __Pyx_RefNannyDeclarations
  __Pyx_RefNannySetupContext("__reduce_cython__ (wrapper)", 0);
  __pyx_r = __pyx_pf_4grpc_7_cython_6cygrpc_23ReceiveMessageOperation_6__reduce_cython__(((struct __pyx_obj_4grpc_7_cython_6cygrpc_ReceiveMessageOperation *)__pyx_v_self));

  /* function exit code */
  __Pyx_RefNannyFinishContext();
  return __pyx_r;
}

static PyObject *__pyx_pf_4grpc_7_cython_6cygrpc_23ReceiveMessageOperation_6__reduce_cython__(CYTHON_UNUSED struct __pyx_obj_4grpc_7_cython_6cygrpc_ReceiveMessageOperation *__pyx_v_self) {
  PyObject *__pyx_r = NULL;
  __Pyx_RefNannyDeclarations
  PyObject *__pyx_t_1 = NULL;
  __Pyx_RefNannySetupContext("__reduce_cython__", 0);

  /* "(tree fragment)":2
 * def __reduce_cython__(self):
 *     raise TypeError("no default __reduce__ due to non-trivial __cinit__")             # <<<<<<<<<<<<<<
 * def __setstate_cython__(self, __pyx_state):
 *     raise TypeError("no default __reduce__ due to non-trivial __cinit__")
 */
  __pyx_t_1 = __Pyx_PyObject_Call(__pyx_builtin_TypeError, __pyx_tuple__96, NULL); if (unlikely(!__pyx_t_1)) __PYX_ERR(7, 2, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_1);
  __Pyx_Raise(__pyx_t_1, 0, 0, 0);
  __Pyx_DECREF(__pyx_t_1); __pyx_t_1 = 0;
  __PYX_ERR(7, 2, __pyx_L1_error)

  /* "(tree fragment)":1
 * def __reduce_cython__(self):             # <<<<<<<<<<<<<<
 *     raise TypeError("no default __reduce__ due to non-trivial __cinit__")
 * def __setstate_cython__(self, __pyx_state):
 */

  /* function exit code */
  __pyx_L1_error:;
  __Pyx_XDECREF(__pyx_t_1);
  __Pyx_AddTraceback("grpc._cython.cygrpc.ReceiveMessageOperation.__reduce_cython__", __pyx_clineno, __pyx_lineno, __pyx_filename);
  __pyx_r = NULL;
  __Pyx_XGIVEREF(__pyx_r);
  __Pyx_RefNannyFinishContext();
  return __pyx_r;
}

/* "(tree fragment)":3
 * def __reduce_cython__(self):
 *     raise TypeError("no default __reduce__ due to non-trivial __cinit__")
 * def __setstate_cython__(self, __pyx_state):             # <<<<<<<<<<<<<<
 *     raise TypeError("no default __reduce__ due to non-trivial __cinit__")
 */

/* Python wrapper */
static PyObject *__pyx_pw_4grpc_7_cython_6cygrpc_23ReceiveMessageOperation_9__setstate_cython__(PyObject *__pyx_v_self, PyObject *__pyx_v___pyx_state); /*proto*/
static PyObject *__pyx_pw_4grpc_7_cython_6cygrpc_23ReceiveMessageOperation_9__setstate_cython__(PyObject *__pyx_v_self, PyObject *__pyx_v___pyx_state) {
  PyObject *__pyx_r = 0;
  __Pyx_RefNannyDeclarations
  __Pyx_RefNannySetupContext("__setstate_cython__ (wrapper)", 0);
  __pyx_r = __pyx_pf_4grpc_7_cython_6cygrpc_23ReceiveMessageOperation_8__setstate_cython__(((struct __pyx_obj_4grpc_7_cython_6cygrpc_ReceiveMessageOperation *)__pyx_v_self), ((PyObject *)__pyx_v___pyx_state));

  /* function exit code */
  __Pyx_RefNannyFinishContext();
  return __pyx_r;
}

static PyObject *__pyx_pf_4grpc_7_cython_6cygrpc_23ReceiveMessageOperation_8__setstate_cython__(CYTHON_UNUSED struct __pyx_obj_4grpc_7_cython_6cygrpc_ReceiveMessageOperation *__pyx_v_self, CYTHON_UNUSED PyObject *__pyx_v___pyx_state) {
  PyObject *__pyx_r = NULL;
  __Pyx_RefNannyDeclarations
  PyObject *__pyx_t_1 = NULL;
  __Pyx_RefNannySetupContext("__setstate_cython__", 0);

  /* "(tree fragment)":4
 *     raise TypeError("no default __reduce__ due to non-trivial __cinit__")
 * def __setstate_cython__(self, __pyx_state):
 *     raise TypeError("no default __reduce__ due to non-trivial __cinit__")             # <<<<<<<<<<<<<<
 */
  __pyx_t_1 = __Pyx_PyObject_Call(__pyx_builtin_TypeError, __pyx_tuple__97, NULL); if (unlikely(!__pyx_t_1)) __PYX_ERR(7, 4, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_1);
  __Pyx_Raise(__pyx_t_1, 0, 0, 0);
  __Pyx_DECREF(__pyx_t_1); __pyx_t_1 = 0;
  __PYX_ERR(7, 4, __pyx_L1_error)

  /* "(tree fragment)":3
 * def __reduce_cython__(self):
 *     raise TypeError("no default __reduce__ due to non-trivial __cinit__")
 * def __setstate_cython__(self, __pyx_state):             # <<<<<<<<<<<<<<
 *     raise TypeError("no default __reduce__ due to non-trivial __cinit__")
 */

  /* function exit code */
  __pyx_L1_error:;
  __Pyx_XDECREF(__pyx_t_1);
  __Pyx_AddTraceback("grpc._cython.cygrpc.ReceiveMessageOperation.__setstate_cython__", __pyx_clineno, __pyx_lineno, __pyx_filename);
  __pyx_r = NULL;
  __Pyx_XGIVEREF(__pyx_r);
  __Pyx_RefNannyFinishContext();
  return __pyx_r;
}

/* "src/python/grpcio/grpc/_cython/_cygrpc/operation.pyx.pxi":186
 * cdef class ReceiveStatusOnClientOperation(Operation):
 * 
 *   def __cinit__(self, flags):             # <<<<<<<<<<<<<<
 *     self._flags = flags
 * 
 */

/* Python wrapper */
static int __pyx_pw_4grpc_7_cython_6cygrpc_30ReceiveStatusOnClientOperation_1__cinit__(PyObject *__pyx_v_self, PyObject *__pyx_args, PyObject *__pyx_kwds); /*proto*/
static int __pyx_pw_4grpc_7_cython_6cygrpc_30ReceiveStatusOnClientOperation_1__cinit__(PyObject *__pyx_v_self, PyObject *__pyx_args, PyObject *__pyx_kwds) {
  PyObject *__pyx_v_flags = 0;
  int __pyx_r;
  __Pyx_RefNannyDeclarations
  __Pyx_RefNannySetupContext("__cinit__ (wrapper)", 0);
  {
    static PyObject **__pyx_pyargnames[] = {&__pyx_n_s_flags,0};
    PyObject* values[1] = {0};
    if (unlikely(__pyx_kwds)) {
      Py_ssize_t kw_args;
      const Py_ssize_t pos_args = PyTuple_GET_SIZE(__pyx_args);
      switch (pos_args) {
        case  1: values[0] = PyTuple_GET_ITEM(__pyx_args, 0);
        CYTHON_FALLTHROUGH;
        case  0: break;
        default: goto __pyx_L5_argtuple_error;
      }
      kw_args = PyDict_Size(__pyx_kwds);
      switch (pos_args) {
        case  0:
        if (likely((values[0] = __Pyx_PyDict_GetItemStr(__pyx_kwds, __pyx_n_s_flags)) != 0)) kw_args--;
        else goto __pyx_L5_argtuple_error;
      }
      if (unlikely(kw_args > 0)) {
        if (unlikely(__Pyx_ParseOptionalKeywords(__pyx_kwds, __pyx_pyargnames, 0, values, pos_args, "__cinit__") < 0)) __PYX_ERR(14, 186, __pyx_L3_error)
      }
    } else if (PyTuple_GET_SIZE(__pyx_args) != 1) {
      goto __pyx_L5_argtuple_error;
    } else {
      values[0] = PyTuple_GET_ITEM(__pyx_args, 0);
    }
    __pyx_v_flags = values[0];
  }
  goto __pyx_L4_argument_unpacking_done;
  __pyx_L5_argtuple_error:;
  __Pyx_RaiseArgtupleInvalid("__cinit__", 1, 1, 1, PyTuple_GET_SIZE(__pyx_args)); __PYX_ERR(14, 186, __pyx_L3_error)
  __pyx_L3_error:;
  __Pyx_AddTraceback("grpc._cython.cygrpc.ReceiveStatusOnClientOperation.__cinit__", __pyx_clineno, __pyx_lineno, __pyx_filename);
  __Pyx_RefNannyFinishContext();
  return -1;
  __pyx_L4_argument_unpacking_done:;
  __pyx_r = __pyx_pf_4grpc_7_cython_6cygrpc_30ReceiveStatusOnClientOperation___cinit__(((struct __pyx_obj_4grpc_7_cython_6cygrpc_ReceiveStatusOnClientOperation *)__pyx_v_self), __pyx_v_flags);

  /* function exit code */
  __Pyx_RefNannyFinishContext();
  return __pyx_r;
}

static int __pyx_pf_4grpc_7_cython_6cygrpc_30ReceiveStatusOnClientOperation___cinit__(struct __pyx_obj_4grpc_7_cython_6cygrpc_ReceiveStatusOnClientOperation *__pyx_v_self, PyObject *__pyx_v_flags) {
  int __pyx_r;
  __Pyx_RefNannyDeclarations
  int __pyx_t_1;
  __Pyx_RefNannySetupContext("__cinit__", 0);

  /* "src/python/grpcio/grpc/_cython/_cygrpc/operation.pyx.pxi":187
 * 
 *   def __cinit__(self, flags):
 *     self._flags = flags             # <<<<<<<<<<<<<<
 * 
 *   def type(self):
 */
  __pyx_t_1 = __Pyx_PyInt_As_int(__pyx_v_flags); if (unlikely((__pyx_t_1 == (int)-1) && PyErr_Occurred())) __PYX_ERR(14, 187, __pyx_L1_error)
  __pyx_v_self->_flags = __pyx_t_1;

  /* "src/python/grpcio/grpc/_cython/_cygrpc/operation.pyx.pxi":186
 * cdef class ReceiveStatusOnClientOperation(Operation):
 * 
 *   def __cinit__(self, flags):             # <<<<<<<<<<<<<<
 *     self._flags = flags
 * 
 */

  /* function exit code */
  __pyx_r = 0;
  goto __pyx_L0;
  __pyx_L1_error:;
  __Pyx_AddTraceback("grpc._cython.cygrpc.ReceiveStatusOnClientOperation.__cinit__", __pyx_clineno, __pyx_lineno, __pyx_filename);
  __pyx_r = -1;
  __pyx_L0:;
  __Pyx_RefNannyFinishContext();
  return __pyx_r;
}

/* "src/python/grpcio/grpc/_cython/_cygrpc/operation.pyx.pxi":189
 *     self._flags = flags
 * 
 *   def type(self):             # <<<<<<<<<<<<<<
 *     return GRPC_OP_RECV_STATUS_ON_CLIENT
 * 
 */

/* Python wrapper */
static PyObject *__pyx_pw_4grpc_7_cython_6cygrpc_30ReceiveStatusOnClientOperation_3type(PyObject *__pyx_v_self, CYTHON_UNUSED PyObject *unused); /*proto*/
static PyObject *__pyx_pw_4grpc_7_cython_6cygrpc_30ReceiveStatusOnClientOperation_3type(PyObject *__pyx_v_self, CYTHON_UNUSED PyObject *unused) {
  PyObject *__pyx_r = 0;
  __Pyx_RefNannyDeclarations
  __Pyx_RefNannySetupContext("type (wrapper)", 0);
  __pyx_r = __pyx_pf_4grpc_7_cython_6cygrpc_30ReceiveStatusOnClientOperation_2type(((struct __pyx_obj_4grpc_7_cython_6cygrpc_ReceiveStatusOnClientOperation *)__pyx_v_self));

  /* function exit code */
  __Pyx_RefNannyFinishContext();
  return __pyx_r;
}

static PyObject *__pyx_pf_4grpc_7_cython_6cygrpc_30ReceiveStatusOnClientOperation_2type(CYTHON_UNUSED struct __pyx_obj_4grpc_7_cython_6cygrpc_ReceiveStatusOnClientOperation *__pyx_v_self) {
  PyObject *__pyx_r = NULL;
  __Pyx_RefNannyDeclarations
  PyObject *__pyx_t_1 = NULL;
  __Pyx_RefNannySetupContext("type", 0);

  /* "src/python/grpcio/grpc/_cython/_cygrpc/operation.pyx.pxi":190
 * 
 *   def type(self):
 *     return GRPC_OP_RECV_STATUS_ON_CLIENT             # <<<<<<<<<<<<<<
 * 
 *   cdef void c(self) except *:
 */
  __Pyx_XDECREF(__pyx_r);
  __pyx_t_1 = __Pyx_PyInt_From_grpc_op_type(GRPC_OP_RECV_STATUS_ON_CLIENT); if (unlikely(!__pyx_t_1)) __PYX_ERR(14, 190, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_1);
  __pyx_r = __pyx_t_1;
  __pyx_t_1 = 0;
  goto __pyx_L0;

  /* "src/python/grpcio/grpc/_cython/_cygrpc/operation.pyx.pxi":189
 *     self._flags = flags
 * 
 *   def type(self):             # <<<<<<<<<<<<<<
 *     return GRPC_OP_RECV_STATUS_ON_CLIENT
 * 
 */

  /* function exit code */
  __pyx_L1_error:;
  __Pyx_XDECREF(__pyx_t_1);
  __Pyx_AddTraceback("grpc._cython.cygrpc.ReceiveStatusOnClientOperation.type", __pyx_clineno, __pyx_lineno, __pyx_filename);
  __pyx_r = NULL;
  __pyx_L0:;
  __Pyx_XGIVEREF(__pyx_r);
  __Pyx_RefNannyFinishContext();
  return __pyx_r;
}

/* "src/python/grpcio/grpc/_cython/_cygrpc/operation.pyx.pxi":192
 *     return GRPC_OP_RECV_STATUS_ON_CLIENT
 * 
 *   cdef void c(self) except *:             # <<<<<<<<<<<<<<
 *     self.c_op.type = GRPC_OP_RECV_STATUS_ON_CLIENT
 *     self.c_op.flags = self._flags
 */

static void __pyx_f_4grpc_7_cython_6cygrpc_30ReceiveStatusOnClientOperation_c(struct __pyx_obj_4grpc_7_cython_6cygrpc_ReceiveStatusOnClientOperation *__pyx_v_self) {
  __Pyx_RefNannyDeclarations
  int __pyx_t_1;
  __Pyx_RefNannySetupContext("c", 0);

  /* "src/python/grpcio/grpc/_cython/_cygrpc/operation.pyx.pxi":193
 * 
 *   cdef void c(self) except *:
 *     self.c_op.type = GRPC_OP_RECV_STATUS_ON_CLIENT             # <<<<<<<<<<<<<<
 *     self.c_op.flags = self._flags
 *     grpc_metadata_array_init(&self._c_trailing_metadata)
 */
  __pyx_v_self->__pyx_base.c_op.op = GRPC_OP_RECV_STATUS_ON_CLIENT;

  /* "src/python/grpcio/grpc/_cython/_cygrpc/operation.pyx.pxi":194
 *   cdef void c(self) except *:
 *     self.c_op.type = GRPC_OP_RECV_STATUS_ON_CLIENT
 *     self.c_op.flags = self._flags             # <<<<<<<<<<<<<<
 *     grpc_metadata_array_init(&self._c_trailing_metadata)
 *     self.c_op.data.receive_status_on_client.trailing_metadata = (
 */
  __pyx_t_1 = __pyx_v_self->_flags;
  __pyx_v_self->__pyx_base.c_op.flags = __pyx_t_1;

  /* "src/python/grpcio/grpc/_cython/_cygrpc/operation.pyx.pxi":195
 *     self.c_op.type = GRPC_OP_RECV_STATUS_ON_CLIENT
 *     self.c_op.flags = self._flags
 *     grpc_metadata_array_init(&self._c_trailing_metadata)             # <<<<<<<<<<<<<<
 *     self.c_op.data.receive_status_on_client.trailing_metadata = (
 *         &self._c_trailing_metadata)
 */
  grpc_metadata_array_init((&__pyx_v_self->_c_trailing_metadata));

  /* "src/python/grpcio/grpc/_cython/_cygrpc/operation.pyx.pxi":196
 *     self.c_op.flags = self._flags
 *     grpc_metadata_array_init(&self._c_trailing_metadata)
 *     self.c_op.data.receive_status_on_client.trailing_metadata = (             # <<<<<<<<<<<<<<
 *         &self._c_trailing_metadata)
 *     self.c_op.data.receive_status_on_client.status = (
 */
  __pyx_v_self->__pyx_base.c_op.data.recv_status_on_client.trailing_metadata = (&__pyx_v_self->_c_trailing_metadata);

  /* "src/python/grpcio/grpc/_cython/_cygrpc/operation.pyx.pxi":198
 *     self.c_op.data.receive_status_on_client.trailing_metadata = (
 *         &self._c_trailing_metadata)
 *     self.c_op.data.receive_status_on_client.status = (             # <<<<<<<<<<<<<<
 *         &self._c_code)
 *     self.c_op.data.receive_status_on_client.status_details = (
 */
  __pyx_v_self->__pyx_base.c_op.data.recv_status_on_client.status = (&__pyx_v_self->_c_code);

  /* "src/python/grpcio/grpc/_cython/_cygrpc/operation.pyx.pxi":200
 *     self.c_op.data.receive_status_on_client.status = (
 *         &self._c_code)
 *     self.c_op.data.receive_status_on_client.status_details = (             # <<<<<<<<<<<<<<
 *         &self._c_details)
 *     self.c_op.data.receive_status_on_client.error_string = (
 */
  __pyx_v_self->__pyx_base.c_op.data.recv_status_on_client.status_details = (&__pyx_v_self->_c_details);

  /* "src/python/grpcio/grpc/_cython/_cygrpc/operation.pyx.pxi":202
 *     self.c_op.data.receive_status_on_client.status_details = (
 *         &self._c_details)
 *     self.c_op.data.receive_status_on_client.error_string = (             # <<<<<<<<<<<<<<
 *         &self._c_error_string)
 * 
 */
  __pyx_v_self->__pyx_base.c_op.data.recv_status_on_client.error_string = (&__pyx_v_self->_c_error_string);

  /* "src/python/grpcio/grpc/_cython/_cygrpc/operation.pyx.pxi":192
 *     return GRPC_OP_RECV_STATUS_ON_CLIENT
 * 
 *   cdef void c(self) except *:             # <<<<<<<<<<<<<<
 *     self.c_op.type = GRPC_OP_RECV_STATUS_ON_CLIENT
 *     self.c_op.flags = self._flags
 */

  /* function exit code */
  __Pyx_RefNannyFinishContext();
}

/* "src/python/grpcio/grpc/_cython/_cygrpc/operation.pyx.pxi":205
 *         &self._c_error_string)
 * 
 *   cdef void un_c(self) except *:             # <<<<<<<<<<<<<<
 *     self._trailing_metadata = _metadata(&self._c_trailing_metadata)
 *     grpc_metadata_array_destroy(&self._c_trailing_metadata)
 */

static void __pyx_f_4grpc_7_cython_6cygrpc_30ReceiveStatusOnClientOperation_un_c(struct __pyx_obj_4grpc_7_cython_6cygrpc_ReceiveStatusOnClientOperation *__pyx_v_self) {
  __Pyx_RefNannyDeclarations
  PyObject *__pyx_t_1 = NULL;
  PyObject *__pyx_t_2 = NULL;
  int __pyx_t_3;
  __Pyx_RefNannySetupContext("un_c", 0);

  /* "src/python/grpcio/grpc/_cython/_cygrpc/operation.pyx.pxi":206
 * 
 *   cdef void un_c(self) except *:
 *     self._trailing_metadata = _metadata(&self._c_trailing_metadata)             # <<<<<<<<<<<<<<
 *     grpc_metadata_array_destroy(&self._c_trailing_metadata)
 *     self._code = self._c_code
 */
  __pyx_t_1 = __pyx_f_4grpc_7_cython_6cygrpc__metadata((&__pyx_v_self->_c_trailing_metadata)); if (unlikely(!__pyx_t_1)) __PYX_ERR(14, 206, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_1);
  __Pyx_GIVEREF(__pyx_t_1);
  __Pyx_GOTREF(__pyx_v_self->_trailing_metadata);
  __Pyx_DECREF(__pyx_v_self->_trailing_metadata);
  __pyx_v_self->_trailing_metadata = ((PyObject*)__pyx_t_1);
  __pyx_t_1 = 0;

  /* "src/python/grpcio/grpc/_cython/_cygrpc/operation.pyx.pxi":207
 *   cdef void un_c(self) except *:
 *     self._trailing_metadata = _metadata(&self._c_trailing_metadata)
 *     grpc_metadata_array_destroy(&self._c_trailing_metadata)             # <<<<<<<<<<<<<<
 *     self._code = self._c_code
 *     self._details = _decode(_slice_bytes(self._c_details))
 */
  grpc_metadata_array_destroy((&__pyx_v_self->_c_trailing_metadata));

  /* "src/python/grpcio/grpc/_cython/_cygrpc/operation.pyx.pxi":208
 *     self._trailing_metadata = _metadata(&self._c_trailing_metadata)
 *     grpc_metadata_array_destroy(&self._c_trailing_metadata)
 *     self._code = self._c_code             # <<<<<<<<<<<<<<
 *     self._details = _decode(_slice_bytes(self._c_details))
 *     grpc_slice_unref(self._c_details)
 */
  __pyx_t_1 = __Pyx_PyInt_From_grpc_status_code(__pyx_v_self->_c_code); if (unlikely(!__pyx_t_1)) __PYX_ERR(14, 208, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_1);
  __Pyx_GIVEREF(__pyx_t_1);
  __Pyx_GOTREF(__pyx_v_self->_code);
  __Pyx_DECREF(__pyx_v_self->_code);
  __pyx_v_self->_code = __pyx_t_1;
  __pyx_t_1 = 0;

  /* "src/python/grpcio/grpc/_cython/_cygrpc/operation.pyx.pxi":209
 *     grpc_metadata_array_destroy(&self._c_trailing_metadata)
 *     self._code = self._c_code
 *     self._details = _decode(_slice_bytes(self._c_details))             # <<<<<<<<<<<<<<
 *     grpc_slice_unref(self._c_details)
 *     if self._c_error_string != NULL:
 */
  __pyx_t_1 = __pyx_f_4grpc_7_cython_6cygrpc__slice_bytes(__pyx_v_self->_c_details); if (unlikely(!__pyx_t_1)) __PYX_ERR(14, 209, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_1);
  __pyx_t_2 = __pyx_f_4grpc_7_cython_6cygrpc__decode(((PyObject*)__pyx_t_1)); if (unlikely(!__pyx_t_2)) __PYX_ERR(14, 209, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_2);
  __Pyx_DECREF(__pyx_t_1); __pyx_t_1 = 0;
  __Pyx_GIVEREF(__pyx_t_2);
  __Pyx_GOTREF(__pyx_v_self->_details);
  __Pyx_DECREF(__pyx_v_self->_details);
  __pyx_v_self->_details = ((PyObject*)__pyx_t_2);
  __pyx_t_2 = 0;

  /* "src/python/grpcio/grpc/_cython/_cygrpc/operation.pyx.pxi":210
 *     self._code = self._c_code
 *     self._details = _decode(_slice_bytes(self._c_details))
 *     grpc_slice_unref(self._c_details)             # <<<<<<<<<<<<<<
 *     if self._c_error_string != NULL:
 *       self._error_string = _decode(self._c_error_string)
 */
  grpc_slice_unref(__pyx_v_self->_c_details);

  /* "src/python/grpcio/grpc/_cython/_cygrpc/operation.pyx.pxi":211
 *     self._details = _decode(_slice_bytes(self._c_details))
 *     grpc_slice_unref(self._c_details)
 *     if self._c_error_string != NULL:             # <<<<<<<<<<<<<<
 *       self._error_string = _decode(self._c_error_string)
 *       gpr_free(<void*>self._c_error_string)
 */
  __pyx_t_3 = ((__pyx_v_self->_c_error_string != NULL) != 0);
  if (__pyx_t_3) {

    /* "src/python/grpcio/grpc/_cython/_cygrpc/operation.pyx.pxi":212
 *     grpc_slice_unref(self._c_details)
 *     if self._c_error_string != NULL:
 *       self._error_string = _decode(self._c_error_string)             # <<<<<<<<<<<<<<
 *       gpr_free(<void*>self._c_error_string)
 *     else:
 */
    __pyx_t_2 = __Pyx_PyBytes_FromString(__pyx_v_self->_c_error_string); if (unlikely(!__pyx_t_2)) __PYX_ERR(14, 212, __pyx_L1_error)
    __Pyx_GOTREF(__pyx_t_2);
    __pyx_t_1 = __pyx_f_4grpc_7_cython_6cygrpc__decode(((PyObject*)__pyx_t_2)); if (unlikely(!__pyx_t_1)) __PYX_ERR(14, 212, __pyx_L1_error)
    __Pyx_GOTREF(__pyx_t_1);
    __Pyx_DECREF(__pyx_t_2); __pyx_t_2 = 0;
    __Pyx_GIVEREF(__pyx_t_1);
    __Pyx_GOTREF(__pyx_v_self->_error_string);
    __Pyx_DECREF(__pyx_v_self->_error_string);
    __pyx_v_self->_error_string = ((PyObject*)__pyx_t_1);
    __pyx_t_1 = 0;

    /* "src/python/grpcio/grpc/_cython/_cygrpc/operation.pyx.pxi":213
 *     if self._c_error_string != NULL:
 *       self._error_string = _decode(self._c_error_string)
 *       gpr_free(<void*>self._c_error_string)             # <<<<<<<<<<<<<<
 *     else:
 *       self._error_string = ""
 */
    gpr_free(((void *)__pyx_v_self->_c_error_string));

    /* "src/python/grpcio/grpc/_cython/_cygrpc/operation.pyx.pxi":211
 *     self._details = _decode(_slice_bytes(self._c_details))
 *     grpc_slice_unref(self._c_details)
 *     if self._c_error_string != NULL:             # <<<<<<<<<<<<<<
 *       self._error_string = _decode(self._c_error_string)
 *       gpr_free(<void*>self._c_error_string)
 */
    goto __pyx_L3;
  }

  /* "src/python/grpcio/grpc/_cython/_cygrpc/operation.pyx.pxi":215
 *       gpr_free(<void*>self._c_error_string)
 *     else:
 *       self._error_string = ""             # <<<<<<<<<<<<<<
 * 
 *   def trailing_metadata(self):
 */
  /*else*/ {
    __Pyx_INCREF(__pyx_kp_s__2);
    __Pyx_GIVEREF(__pyx_kp_s__2);
    __Pyx_GOTREF(__pyx_v_self->_error_string);
    __Pyx_DECREF(__pyx_v_self->_error_string);
    __pyx_v_self->_error_string = __pyx_kp_s__2;
  }
  __pyx_L3:;

  /* "src/python/grpcio/grpc/_cython/_cygrpc/operation.pyx.pxi":205
 *         &self._c_error_string)
 * 
 *   cdef void un_c(self) except *:             # <<<<<<<<<<<<<<
 *     self._trailing_metadata = _metadata(&self._c_trailing_metadata)
 *     grpc_metadata_array_destroy(&self._c_trailing_metadata)
 */

  /* function exit code */
  goto __pyx_L0;
  __pyx_L1_error:;
  __Pyx_XDECREF(__pyx_t_1);
  __Pyx_XDECREF(__pyx_t_2);
  __Pyx_AddTraceback("grpc._cython.cygrpc.ReceiveStatusOnClientOperation.un_c", __pyx_clineno, __pyx_lineno, __pyx_filename);
  __pyx_L0:;
  __Pyx_RefNannyFinishContext();
}

/* "src/python/grpcio/grpc/_cython/_cygrpc/operation.pyx.pxi":217
 *       self._error_string = ""
 * 
 *   def trailing_metadata(self):             # <<<<<<<<<<<<<<
 *     return self._trailing_metadata
 * 
 */

/* Python wrapper */
static PyObject *__pyx_pw_4grpc_7_cython_6cygrpc_30ReceiveStatusOnClientOperation_5trailing_metadata(PyObject *__pyx_v_self, CYTHON_UNUSED PyObject *unused); /*proto*/
static PyObject *__pyx_pw_4grpc_7_cython_6cygrpc_30ReceiveStatusOnClientOperation_5trailing_metadata(PyObject *__pyx_v_self, CYTHON_UNUSED PyObject *unused) {
  PyObject *__pyx_r = 0;
  __Pyx_RefNannyDeclarations
  __Pyx_RefNannySetupContext("trailing_metadata (wrapper)", 0);
  __pyx_r = __pyx_pf_4grpc_7_cython_6cygrpc_30ReceiveStatusOnClientOperation_4trailing_metadata(((struct __pyx_obj_4grpc_7_cython_6cygrpc_ReceiveStatusOnClientOperation *)__pyx_v_self));

  /* function exit code */
  __Pyx_RefNannyFinishContext();
  return __pyx_r;
}

static PyObject *__pyx_pf_4grpc_7_cython_6cygrpc_30ReceiveStatusOnClientOperation_4trailing_metadata(struct __pyx_obj_4grpc_7_cython_6cygrpc_ReceiveStatusOnClientOperation *__pyx_v_self) {
  PyObject *__pyx_r = NULL;
  __Pyx_RefNannyDeclarations
  __Pyx_RefNannySetupContext("trailing_metadata", 0);

  /* "src/python/grpcio/grpc/_cython/_cygrpc/operation.pyx.pxi":218
 * 
 *   def trailing_metadata(self):
 *     return self._trailing_metadata             # <<<<<<<<<<<<<<
 * 
 *   def code(self):
 */
  __Pyx_XDECREF(__pyx_r);
  __Pyx_INCREF(__pyx_v_self->_trailing_metadata);
  __pyx_r = __pyx_v_self->_trailing_metadata;
  goto __pyx_L0;

  /* "src/python/grpcio/grpc/_cython/_cygrpc/operation.pyx.pxi":217
 *       self._error_string = ""
 * 
 *   def trailing_metadata(self):             # <<<<<<<<<<<<<<
 *     return self._trailing_metadata
 * 
 */

  /* function exit code */
  __pyx_L0:;
  __Pyx_XGIVEREF(__pyx_r);
  __Pyx_RefNannyFinishContext();
  return __pyx_r;
}

/* "src/python/grpcio/grpc/_cython/_cygrpc/operation.pyx.pxi":220
 *     return self._trailing_metadata
 * 
 *   def code(self):             # <<<<<<<<<<<<<<
 *     return self._code
 * 
 */

/* Python wrapper */
static PyObject *__pyx_pw_4grpc_7_cython_6cygrpc_30ReceiveStatusOnClientOperation_7code(PyObject *__pyx_v_self, CYTHON_UNUSED PyObject *unused); /*proto*/
static PyObject *__pyx_pw_4grpc_7_cython_6cygrpc_30ReceiveStatusOnClientOperation_7code(PyObject *__pyx_v_self, CYTHON_UNUSED PyObject *unused) {
  PyObject *__pyx_r = 0;
  __Pyx_RefNannyDeclarations
  __Pyx_RefNannySetupContext("code (wrapper)", 0);
  __pyx_r = __pyx_pf_4grpc_7_cython_6cygrpc_30ReceiveStatusOnClientOperation_6code(((struct __pyx_obj_4grpc_7_cython_6cygrpc_ReceiveStatusOnClientOperation *)__pyx_v_self));

  /* function exit code */
  __Pyx_RefNannyFinishContext();
  return __pyx_r;
}

static PyObject *__pyx_pf_4grpc_7_cython_6cygrpc_30ReceiveStatusOnClientOperation_6code(struct __pyx_obj_4grpc_7_cython_6cygrpc_ReceiveStatusOnClientOperation *__pyx_v_self) {
  PyObject *__pyx_r = NULL;
  __Pyx_RefNannyDeclarations
  __Pyx_RefNannySetupContext("code", 0);

  /* "src/python/grpcio/grpc/_cython/_cygrpc/operation.pyx.pxi":221
 * 
 *   def code(self):
 *     return self._code             # <<<<<<<<<<<<<<
 * 
 *   def details(self):
 */
  __Pyx_XDECREF(__pyx_r);
  __Pyx_INCREF(__pyx_v_self->_code);
  __pyx_r = __pyx_v_self->_code;
  goto __pyx_L0;

  /* "src/python/grpcio/grpc/_cython/_cygrpc/operation.pyx.pxi":220
 *     return self._trailing_metadata
 * 
 *   def code(self):             # <<<<<<<<<<<<<<
 *     return self._code
 * 
 */

  /* function exit code */
  __pyx_L0:;
  __Pyx_XGIVEREF(__pyx_r);
  __Pyx_RefNannyFinishContext();
  return __pyx_r;
}

/* "src/python/grpcio/grpc/_cython/_cygrpc/operation.pyx.pxi":223
 *     return self._code
 * 
 *   def details(self):             # <<<<<<<<<<<<<<
 *     return self._details
 * 
 */

/* Python wrapper */
static PyObject *__pyx_pw_4grpc_7_cython_6cygrpc_30ReceiveStatusOnClientOperation_9details(PyObject *__pyx_v_self, CYTHON_UNUSED PyObject *unused); /*proto*/
static PyObject *__pyx_pw_4grpc_7_cython_6cygrpc_30ReceiveStatusOnClientOperation_9details(PyObject *__pyx_v_self, CYTHON_UNUSED PyObject *unused) {
  PyObject *__pyx_r = 0;
  __Pyx_RefNannyDeclarations
  __Pyx_RefNannySetupContext("details (wrapper)", 0);
  __pyx_r = __pyx_pf_4grpc_7_cython_6cygrpc_30ReceiveStatusOnClientOperation_8details(((struct __pyx_obj_4grpc_7_cython_6cygrpc_ReceiveStatusOnClientOperation *)__pyx_v_self));

  /* function exit code */
  __Pyx_RefNannyFinishContext();
  return __pyx_r;
}

static PyObject *__pyx_pf_4grpc_7_cython_6cygrpc_30ReceiveStatusOnClientOperation_8details(struct __pyx_obj_4grpc_7_cython_6cygrpc_ReceiveStatusOnClientOperation *__pyx_v_self) {
  PyObject *__pyx_r = NULL;
  __Pyx_RefNannyDeclarations
  __Pyx_RefNannySetupContext("details", 0);

  /* "src/python/grpcio/grpc/_cython/_cygrpc/operation.pyx.pxi":224
 * 
 *   def details(self):
 *     return self._details             # <<<<<<<<<<<<<<
 * 
 *   def error_string(self):
 */
  __Pyx_XDECREF(__pyx_r);
  __Pyx_INCREF(__pyx_v_self->_details);
  __pyx_r = __pyx_v_self->_details;
  goto __pyx_L0;

  /* "src/python/grpcio/grpc/_cython/_cygrpc/operation.pyx.pxi":223
 *     return self._code
 * 
 *   def details(self):             # <<<<<<<<<<<<<<
 *     return self._details
 * 
 */

  /* function exit code */
  __pyx_L0:;
  __Pyx_XGIVEREF(__pyx_r);
  __Pyx_RefNannyFinishContext();
  return __pyx_r;
}

/* "src/python/grpcio/grpc/_cython/_cygrpc/operation.pyx.pxi":226
 *     return self._details
 * 
 *   def error_string(self):             # <<<<<<<<<<<<<<
 *     return self._error_string
 * 
 */

/* Python wrapper */
static PyObject *__pyx_pw_4grpc_7_cython_6cygrpc_30ReceiveStatusOnClientOperation_11error_string(PyObject *__pyx_v_self, CYTHON_UNUSED PyObject *unused); /*proto*/
static PyObject *__pyx_pw_4grpc_7_cython_6cygrpc_30ReceiveStatusOnClientOperation_11error_string(PyObject *__pyx_v_self, CYTHON_UNUSED PyObject *unused) {
  PyObject *__pyx_r = 0;
  __Pyx_RefNannyDeclarations
  __Pyx_RefNannySetupContext("error_string (wrapper)", 0);
  __pyx_r = __pyx_pf_4grpc_7_cython_6cygrpc_30ReceiveStatusOnClientOperation_10error_string(((struct __pyx_obj_4grpc_7_cython_6cygrpc_ReceiveStatusOnClientOperation *)__pyx_v_self));

  /* function exit code */
  __Pyx_RefNannyFinishContext();
  return __pyx_r;
}

static PyObject *__pyx_pf_4grpc_7_cython_6cygrpc_30ReceiveStatusOnClientOperation_10error_string(struct __pyx_obj_4grpc_7_cython_6cygrpc_ReceiveStatusOnClientOperation *__pyx_v_self) {
  PyObject *__pyx_r = NULL;
  __Pyx_RefNannyDeclarations
  __Pyx_RefNannySetupContext("error_string", 0);

  /* "src/python/grpcio/grpc/_cython/_cygrpc/operation.pyx.pxi":227
 * 
 *   def error_string(self):
 *     return self._error_string             # <<<<<<<<<<<<<<
 * 
 * 
 */
  __Pyx_XDECREF(__pyx_r);
  __Pyx_INCREF(__pyx_v_self->_error_string);
  __pyx_r = __pyx_v_self->_error_string;
  goto __pyx_L0;

  /* "src/python/grpcio/grpc/_cython/_cygrpc/operation.pyx.pxi":226
 *     return self._details
 * 
 *   def error_string(self):             # <<<<<<<<<<<<<<
 *     return self._error_string
 * 
 */

  /* function exit code */
  __pyx_L0:;
  __Pyx_XGIVEREF(__pyx_r);
  __Pyx_RefNannyFinishContext();
  return __pyx_r;
}

/* "src/python/grpcio/grpc/_cython/_cygrpc/operation.pxd.pxi":90
 * cdef class ReceiveStatusOnClientOperation(Operation):
 * 
 *   cdef readonly int _flags             # <<<<<<<<<<<<<<
 *   cdef grpc_metadata_array _c_trailing_metadata
 *   cdef grpc_status_code _c_code
 */

/* Python wrapper */
static PyObject *__pyx_pw_4grpc_7_cython_6cygrpc_30ReceiveStatusOnClientOperation_6_flags_1__get__(PyObject *__pyx_v_self); /*proto*/
static PyObject *__pyx_pw_4grpc_7_cython_6cygrpc_30ReceiveStatusOnClientOperation_6_flags_1__get__(PyObject *__pyx_v_self) {
  PyObject *__pyx_r = 0;
  __Pyx_RefNannyDeclarations
  __Pyx_RefNannySetupContext("__get__ (wrapper)", 0);
  __pyx_r = __pyx_pf_4grpc_7_cython_6cygrpc_30ReceiveStatusOnClientOperation_6_flags___get__(((struct __pyx_obj_4grpc_7_cython_6cygrpc_ReceiveStatusOnClientOperation *)__pyx_v_self));

  /* function exit code */
  __Pyx_RefNannyFinishContext();
  return __pyx_r;
}

static PyObject *__pyx_pf_4grpc_7_cython_6cygrpc_30ReceiveStatusOnClientOperation_6_flags___get__(struct __pyx_obj_4grpc_7_cython_6cygrpc_ReceiveStatusOnClientOperation *__pyx_v_self) {
  PyObject *__pyx_r = NULL;
  __Pyx_RefNannyDeclarations
  PyObject *__pyx_t_1 = NULL;
  __Pyx_RefNannySetupContext("__get__", 0);
  __Pyx_XDECREF(__pyx_r);
  __pyx_t_1 = __Pyx_PyInt_From_int(__pyx_v_self->_flags); if (unlikely(!__pyx_t_1)) __PYX_ERR(15, 90, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_1);
  __pyx_r = __pyx_t_1;
  __pyx_t_1 = 0;
  goto __pyx_L0;

  /* function exit code */
  __pyx_L1_error:;
  __Pyx_XDECREF(__pyx_t_1);
  __Pyx_AddTraceback("grpc._cython.cygrpc.ReceiveStatusOnClientOperation._flags.__get__", __pyx_clineno, __pyx_lineno, __pyx_filename);
  __pyx_r = NULL;
  __pyx_L0:;
  __Pyx_XGIVEREF(__pyx_r);
  __Pyx_RefNannyFinishContext();
  return __pyx_r;
}

/* "(tree fragment)":1
 * def __reduce_cython__(self):             # <<<<<<<<<<<<<<
 *     raise TypeError("no default __reduce__ due to non-trivial __cinit__")
 * def __setstate_cython__(self, __pyx_state):
 */

/* Python wrapper */
static PyObject *__pyx_pw_4grpc_7_cython_6cygrpc_30ReceiveStatusOnClientOperation_13__reduce_cython__(PyObject *__pyx_v_self, CYTHON_UNUSED PyObject *unused); /*proto*/
static PyObject *__pyx_pw_4grpc_7_cython_6cygrpc_30ReceiveStatusOnClientOperation_13__reduce_cython__(PyObject *__pyx_v_self, CYTHON_UNUSED PyObject *unused) {
  PyObject *__pyx_r = 0;
  __Pyx_RefNannyDeclarations
  __Pyx_RefNannySetupContext("__reduce_cython__ (wrapper)", 0);
  __pyx_r = __pyx_pf_4grpc_7_cython_6cygrpc_30ReceiveStatusOnClientOperation_12__reduce_cython__(((struct __pyx_obj_4grpc_7_cython_6cygrpc_ReceiveStatusOnClientOperation *)__pyx_v_self));

  /* function exit code */
  __Pyx_RefNannyFinishContext();
  return __pyx_r;
}

static PyObject *__pyx_pf_4grpc_7_cython_6cygrpc_30ReceiveStatusOnClientOperation_12__reduce_cython__(CYTHON_UNUSED struct __pyx_obj_4grpc_7_cython_6cygrpc_ReceiveStatusOnClientOperation *__pyx_v_self) {
  PyObject *__pyx_r = NULL;
  __Pyx_RefNannyDeclarations
  PyObject *__pyx_t_1 = NULL;
  __Pyx_RefNannySetupContext("__reduce_cython__", 0);

  /* "(tree fragment)":2
 * def __reduce_cython__(self):
 *     raise TypeError("no default __reduce__ due to non-trivial __cinit__")             # <<<<<<<<<<<<<<
 * def __setstate_cython__(self, __pyx_state):
 *     raise TypeError("no default __reduce__ due to non-trivial __cinit__")
 */
  __pyx_t_1 = __Pyx_PyObject_Call(__pyx_builtin_TypeError, __pyx_tuple__98, NULL); if (unlikely(!__pyx_t_1)) __PYX_ERR(7, 2, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_1);
  __Pyx_Raise(__pyx_t_1, 0, 0, 0);
  __Pyx_DECREF(__pyx_t_1); __pyx_t_1 = 0;
  __PYX_ERR(7, 2, __pyx_L1_error)

  /* "(tree fragment)":1
 * def __reduce_cython__(self):             # <<<<<<<<<<<<<<
 *     raise TypeError("no default __reduce__ due to non-trivial __cinit__")
 * def __setstate_cython__(self, __pyx_state):
 */

  /* function exit code */
  __pyx_L1_error:;
  __Pyx_XDECREF(__pyx_t_1);
  __Pyx_AddTraceback("grpc._cython.cygrpc.ReceiveStatusOnClientOperation.__reduce_cython__", __pyx_clineno, __pyx_lineno, __pyx_filename);
  __pyx_r = NULL;
  __Pyx_XGIVEREF(__pyx_r);
  __Pyx_RefNannyFinishContext();
  return __pyx_r;
}

/* "(tree fragment)":3
 * def __reduce_cython__(self):
 *     raise TypeError("no default __reduce__ due to non-trivial __cinit__")
 * def __setstate_cython__(self, __pyx_state):             # <<<<<<<<<<<<<<
 *     raise TypeError("no default __reduce__ due to non-trivial __cinit__")
 */

/* Python wrapper */
static PyObject *__pyx_pw_4grpc_7_cython_6cygrpc_30ReceiveStatusOnClientOperation_15__setstate_cython__(PyObject *__pyx_v_self, PyObject *__pyx_v___pyx_state); /*proto*/
static PyObject *__pyx_pw_4grpc_7_cython_6cygrpc_30ReceiveStatusOnClientOperation_15__setstate_cython__(PyObject *__pyx_v_self, PyObject *__pyx_v___pyx_state) {
  PyObject *__pyx_r = 0;
  __Pyx_RefNannyDeclarations
  __Pyx_RefNannySetupContext("__setstate_cython__ (wrapper)", 0);
  __pyx_r = __pyx_pf_4grpc_7_cython_6cygrpc_30ReceiveStatusOnClientOperation_14__setstate_cython__(((struct __pyx_obj_4grpc_7_cython_6cygrpc_ReceiveStatusOnClientOperation *)__pyx_v_self), ((PyObject *)__pyx_v___pyx_state));

  /* function exit code */
  __Pyx_RefNannyFinishContext();
  return __pyx_r;
}

static PyObject *__pyx_pf_4grpc_7_cython_6cygrpc_30ReceiveStatusOnClientOperation_14__setstate_cython__(CYTHON_UNUSED struct __pyx_obj_4grpc_7_cython_6cygrpc_ReceiveStatusOnClientOperation *__pyx_v_self, CYTHON_UNUSED PyObject *__pyx_v___pyx_state) {
  PyObject *__pyx_r = NULL;
  __Pyx_RefNannyDeclarations
  PyObject *__pyx_t_1 = NULL;
  __Pyx_RefNannySetupContext("__setstate_cython__", 0);

  /* "(tree fragment)":4
 *     raise TypeError("no default __reduce__ due to non-trivial __cinit__")
 * def __setstate_cython__(self, __pyx_state):
 *     raise TypeError("no default __reduce__ due to non-trivial __cinit__")             # <<<<<<<<<<<<<<
 */
  __pyx_t_1 = __Pyx_PyObject_Call(__pyx_builtin_TypeError, __pyx_tuple__99, NULL); if (unlikely(!__pyx_t_1)) __PYX_ERR(7, 4, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_1);
  __Pyx_Raise(__pyx_t_1, 0, 0, 0);
  __Pyx_DECREF(__pyx_t_1); __pyx_t_1 = 0;
  __PYX_ERR(7, 4, __pyx_L1_error)

  /* "(tree fragment)":3
 * def __reduce_cython__(self):
 *     raise TypeError("no default __reduce__ due to non-trivial __cinit__")
 * def __setstate_cython__(self, __pyx_state):             # <<<<<<<<<<<<<<
 *     raise TypeError("no default __reduce__ due to non-trivial __cinit__")
 */

  /* function exit code */
  __pyx_L1_error:;
  __Pyx_XDECREF(__pyx_t_1);
  __Pyx_AddTraceback("grpc._cython.cygrpc.ReceiveStatusOnClientOperation.__setstate_cython__", __pyx_clineno, __pyx_lineno, __pyx_filename);
  __pyx_r = NULL;
  __Pyx_XGIVEREF(__pyx_r);
  __Pyx_RefNannyFinishContext();
  return __pyx_r;
}

/* "src/python/grpcio/grpc/_cython/_cygrpc/operation.pyx.pxi":232
 * cdef class ReceiveCloseOnServerOperation(Operation):
 * 
 *   def __cinit__(self, flags):             # <<<<<<<<<<<<<<
 *     self._flags = flags
 * 
 */

/* Python wrapper */
static int __pyx_pw_4grpc_7_cython_6cygrpc_29ReceiveCloseOnServerOperation_1__cinit__(PyObject *__pyx_v_self, PyObject *__pyx_args, PyObject *__pyx_kwds); /*proto*/
static int __pyx_pw_4grpc_7_cython_6cygrpc_29ReceiveCloseOnServerOperation_1__cinit__(PyObject *__pyx_v_self, PyObject *__pyx_args, PyObject *__pyx_kwds) {
  PyObject *__pyx_v_flags = 0;
  int __pyx_r;
  __Pyx_RefNannyDeclarations
  __Pyx_RefNannySetupContext("__cinit__ (wrapper)", 0);
  {
    static PyObject **__pyx_pyargnames[] = {&__pyx_n_s_flags,0};
    PyObject* values[1] = {0};
    if (unlikely(__pyx_kwds)) {
      Py_ssize_t kw_args;
      const Py_ssize_t pos_args = PyTuple_GET_SIZE(__pyx_args);
      switch (pos_args) {
        case  1: values[0] = PyTuple_GET_ITEM(__pyx_args, 0);
        CYTHON_FALLTHROUGH;
        case  0: break;
        default: goto __pyx_L5_argtuple_error;
      }
      kw_args = PyDict_Size(__pyx_kwds);
      switch (pos_args) {
        case  0:
        if (likely((values[0] = __Pyx_PyDict_GetItemStr(__pyx_kwds, __pyx_n_s_flags)) != 0)) kw_args--;
        else goto __pyx_L5_argtuple_error;
      }
      if (unlikely(kw_args > 0)) {
        if (unlikely(__Pyx_ParseOptionalKeywords(__pyx_kwds, __pyx_pyargnames, 0, values, pos_args, "__cinit__") < 0)) __PYX_ERR(14, 232, __pyx_L3_error)
      }
    } else if (PyTuple_GET_SIZE(__pyx_args) != 1) {
      goto __pyx_L5_argtuple_error;
    } else {
      values[0] = PyTuple_GET_ITEM(__pyx_args, 0);
    }
    __pyx_v_flags = values[0];
  }
  goto __pyx_L4_argument_unpacking_done;
  __pyx_L5_argtuple_error:;
  __Pyx_RaiseArgtupleInvalid("__cinit__", 1, 1, 1, PyTuple_GET_SIZE(__pyx_args)); __PYX_ERR(14, 232, __pyx_L3_error)
  __pyx_L3_error:;
  __Pyx_AddTraceback("grpc._cython.cygrpc.ReceiveCloseOnServerOperation.__cinit__", __pyx_clineno, __pyx_lineno, __pyx_filename);
  __Pyx_RefNannyFinishContext();
  return -1;
  __pyx_L4_argument_unpacking_done:;
  __pyx_r = __pyx_pf_4grpc_7_cython_6cygrpc_29ReceiveCloseOnServerOperation___cinit__(((struct __pyx_obj_4grpc_7_cython_6cygrpc_ReceiveCloseOnServerOperation *)__pyx_v_self), __pyx_v_flags);

  /* function exit code */
  __Pyx_RefNannyFinishContext();
  return __pyx_r;
}

static int __pyx_pf_4grpc_7_cython_6cygrpc_29ReceiveCloseOnServerOperation___cinit__(struct __pyx_obj_4grpc_7_cython_6cygrpc_ReceiveCloseOnServerOperation *__pyx_v_self, PyObject *__pyx_v_flags) {
  int __pyx_r;
  __Pyx_RefNannyDeclarations
  int __pyx_t_1;
  __Pyx_RefNannySetupContext("__cinit__", 0);

  /* "src/python/grpcio/grpc/_cython/_cygrpc/operation.pyx.pxi":233
 * 
 *   def __cinit__(self, flags):
 *     self._flags = flags             # <<<<<<<<<<<<<<
 * 
 *   def type(self):
 */
  __pyx_t_1 = __Pyx_PyInt_As_int(__pyx_v_flags); if (unlikely((__pyx_t_1 == (int)-1) && PyErr_Occurred())) __PYX_ERR(14, 233, __pyx_L1_error)
  __pyx_v_self->_flags = __pyx_t_1;

  /* "src/python/grpcio/grpc/_cython/_cygrpc/operation.pyx.pxi":232
 * cdef class ReceiveCloseOnServerOperation(Operation):
 * 
 *   def __cinit__(self, flags):             # <<<<<<<<<<<<<<
 *     self._flags = flags
 * 
 */

  /* function exit code */
  __pyx_r = 0;
  goto __pyx_L0;
  __pyx_L1_error:;
  __Pyx_AddTraceback("grpc._cython.cygrpc.ReceiveCloseOnServerOperation.__cinit__", __pyx_clineno, __pyx_lineno, __pyx_filename);
  __pyx_r = -1;
  __pyx_L0:;
  __Pyx_RefNannyFinishContext();
  return __pyx_r;
}

/* "src/python/grpcio/grpc/_cython/_cygrpc/operation.pyx.pxi":235
 *     self._flags = flags
 * 
 *   def type(self):             # <<<<<<<<<<<<<<
 *     return GRPC_OP_RECV_CLOSE_ON_SERVER
 * 
 */

/* Python wrapper */
static PyObject *__pyx_pw_4grpc_7_cython_6cygrpc_29ReceiveCloseOnServerOperation_3type(PyObject *__pyx_v_self, CYTHON_UNUSED PyObject *unused); /*proto*/
static PyObject *__pyx_pw_4grpc_7_cython_6cygrpc_29ReceiveCloseOnServerOperation_3type(PyObject *__pyx_v_self, CYTHON_UNUSED PyObject *unused) {
  PyObject *__pyx_r = 0;
  __Pyx_RefNannyDeclarations
  __Pyx_RefNannySetupContext("type (wrapper)", 0);
  __pyx_r = __pyx_pf_4grpc_7_cython_6cygrpc_29ReceiveCloseOnServerOperation_2type(((struct __pyx_obj_4grpc_7_cython_6cygrpc_ReceiveCloseOnServerOperation *)__pyx_v_self));

  /* function exit code */
  __Pyx_RefNannyFinishContext();
  return __pyx_r;
}

static PyObject *__pyx_pf_4grpc_7_cython_6cygrpc_29ReceiveCloseOnServerOperation_2type(CYTHON_UNUSED struct __pyx_obj_4grpc_7_cython_6cygrpc_ReceiveCloseOnServerOperation *__pyx_v_self) {
  PyObject *__pyx_r = NULL;
  __Pyx_RefNannyDeclarations
  PyObject *__pyx_t_1 = NULL;
  __Pyx_RefNannySetupContext("type", 0);

  /* "src/python/grpcio/grpc/_cython/_cygrpc/operation.pyx.pxi":236
 * 
 *   def type(self):
 *     return GRPC_OP_RECV_CLOSE_ON_SERVER             # <<<<<<<<<<<<<<
 * 
 *   cdef void c(self) except *:
 */
  __Pyx_XDECREF(__pyx_r);
  __pyx_t_1 = __Pyx_PyInt_From_grpc_op_type(GRPC_OP_RECV_CLOSE_ON_SERVER); if (unlikely(!__pyx_t_1)) __PYX_ERR(14, 236, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_1);
  __pyx_r = __pyx_t_1;
  __pyx_t_1 = 0;
  goto __pyx_L0;

  /* "src/python/grpcio/grpc/_cython/_cygrpc/operation.pyx.pxi":235
 *     self._flags = flags
 * 
 *   def type(self):             # <<<<<<<<<<<<<<
 *     return GRPC_OP_RECV_CLOSE_ON_SERVER
 * 
 */

  /* function exit code */
  __pyx_L1_error:;
  __Pyx_XDECREF(__pyx_t_1);
  __Pyx_AddTraceback("grpc._cython.cygrpc.ReceiveCloseOnServerOperation.type", __pyx_clineno, __pyx_lineno, __pyx_filename);
  __pyx_r = NULL;
  __pyx_L0:;
  __Pyx_XGIVEREF(__pyx_r);
  __Pyx_RefNannyFinishContext();
  return __pyx_r;
}

/* "src/python/grpcio/grpc/_cython/_cygrpc/operation.pyx.pxi":238
 *     return GRPC_OP_RECV_CLOSE_ON_SERVER
 * 
 *   cdef void c(self) except *:             # <<<<<<<<<<<<<<
 *     self.c_op.type = GRPC_OP_RECV_CLOSE_ON_SERVER
 *     self.c_op.flags = self._flags
 */

static void __pyx_f_4grpc_7_cython_6cygrpc_29ReceiveCloseOnServerOperation_c(struct __pyx_obj_4grpc_7_cython_6cygrpc_ReceiveCloseOnServerOperation *__pyx_v_self) {
  __Pyx_RefNannyDeclarations
  int __pyx_t_1;
  __Pyx_RefNannySetupContext("c", 0);

  /* "src/python/grpcio/grpc/_cython/_cygrpc/operation.pyx.pxi":239
 * 
 *   cdef void c(self) except *:
 *     self.c_op.type = GRPC_OP_RECV_CLOSE_ON_SERVER             # <<<<<<<<<<<<<<
 *     self.c_op.flags = self._flags
 *     self.c_op.data.receive_close_on_server.cancelled = &self._c_cancelled
 */
  __pyx_v_self->__pyx_base.c_op.op = GRPC_OP_RECV_CLOSE_ON_SERVER;

  /* "src/python/grpcio/grpc/_cython/_cygrpc/operation.pyx.pxi":240
 *   cdef void c(self) except *:
 *     self.c_op.type = GRPC_OP_RECV_CLOSE_ON_SERVER
 *     self.c_op.flags = self._flags             # <<<<<<<<<<<<<<
 *     self.c_op.data.receive_close_on_server.cancelled = &self._c_cancelled
 * 
 */
  __pyx_t_1 = __pyx_v_self->_flags;
  __pyx_v_self->__pyx_base.c_op.flags = __pyx_t_1;

  /* "src/python/grpcio/grpc/_cython/_cygrpc/operation.pyx.pxi":241
 *     self.c_op.type = GRPC_OP_RECV_CLOSE_ON_SERVER
 *     self.c_op.flags = self._flags
 *     self.c_op.data.receive_close_on_server.cancelled = &self._c_cancelled             # <<<<<<<<<<<<<<
 * 
 *   cdef void un_c(self) except *:
 */
  __pyx_v_self->__pyx_base.c_op.data.recv_close_on_server.cancelled = (&__pyx_v_self->_c_cancelled);

  /* "src/python/grpcio/grpc/_cython/_cygrpc/operation.pyx.pxi":238
 *     return GRPC_OP_RECV_CLOSE_ON_SERVER
 * 
 *   cdef void c(self) except *:             # <<<<<<<<<<<<<<
 *     self.c_op.type = GRPC_OP_RECV_CLOSE_ON_SERVER
 *     self.c_op.flags = self._flags
 */

  /* function exit code */
  __Pyx_RefNannyFinishContext();
}

/* "src/python/grpcio/grpc/_cython/_cygrpc/operation.pyx.pxi":243
 *     self.c_op.data.receive_close_on_server.cancelled = &self._c_cancelled
 * 
 *   cdef void un_c(self) except *:             # <<<<<<<<<<<<<<
 *     self._cancelled = bool(self._c_cancelled)
 * 
 */

static void __pyx_f_4grpc_7_cython_6cygrpc_29ReceiveCloseOnServerOperation_un_c(struct __pyx_obj_4grpc_7_cython_6cygrpc_ReceiveCloseOnServerOperation *__pyx_v_self) {
  __Pyx_RefNannyDeclarations
  PyObject *__pyx_t_1 = NULL;
  int __pyx_t_2;
  __Pyx_RefNannySetupContext("un_c", 0);

  /* "src/python/grpcio/grpc/_cython/_cygrpc/operation.pyx.pxi":244
 * 
 *   cdef void un_c(self) except *:
 *     self._cancelled = bool(self._c_cancelled)             # <<<<<<<<<<<<<<
 * 
 *   def cancelled(self):
 */
  __pyx_t_1 = __Pyx_PyInt_From_int(__pyx_v_self->_c_cancelled); if (unlikely(!__pyx_t_1)) __PYX_ERR(14, 244, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_1);
  __pyx_t_2 = __Pyx_PyObject_IsTrue(__pyx_t_1); if (unlikely(__pyx_t_2 < 0)) __PYX_ERR(14, 244, __pyx_L1_error)
  __Pyx_DECREF(__pyx_t_1); __pyx_t_1 = 0;
  __pyx_t_1 = __Pyx_PyBool_FromLong((!(!__pyx_t_2))); if (unlikely(!__pyx_t_1)) __PYX_ERR(14, 244, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_1);
  __Pyx_GIVEREF(__pyx_t_1);
  __Pyx_GOTREF(__pyx_v_self->_cancelled);
  __Pyx_DECREF(__pyx_v_self->_cancelled);
  __pyx_v_self->_cancelled = __pyx_t_1;
  __pyx_t_1 = 0;

  /* "src/python/grpcio/grpc/_cython/_cygrpc/operation.pyx.pxi":243
 *     self.c_op.data.receive_close_on_server.cancelled = &self._c_cancelled
 * 
 *   cdef void un_c(self) except *:             # <<<<<<<<<<<<<<
 *     self._cancelled = bool(self._c_cancelled)
 * 
 */

  /* function exit code */
  goto __pyx_L0;
  __pyx_L1_error:;
  __Pyx_XDECREF(__pyx_t_1);
  __Pyx_AddTraceback("grpc._cython.cygrpc.ReceiveCloseOnServerOperation.un_c", __pyx_clineno, __pyx_lineno, __pyx_filename);
  __pyx_L0:;
  __Pyx_RefNannyFinishContext();
}

/* "src/python/grpcio/grpc/_cython/_cygrpc/operation.pyx.pxi":246
 *     self._cancelled = bool(self._c_cancelled)
 * 
 *   def cancelled(self):             # <<<<<<<<<<<<<<
 *     return self._cancelled
 */

/* Python wrapper */
static PyObject *__pyx_pw_4grpc_7_cython_6cygrpc_29ReceiveCloseOnServerOperation_5cancelled(PyObject *__pyx_v_self, CYTHON_UNUSED PyObject *unused); /*proto*/
static PyObject *__pyx_pw_4grpc_7_cython_6cygrpc_29ReceiveCloseOnServerOperation_5cancelled(PyObject *__pyx_v_self, CYTHON_UNUSED PyObject *unused) {
  PyObject *__pyx_r = 0;
  __Pyx_RefNannyDeclarations
  __Pyx_RefNannySetupContext("cancelled (wrapper)", 0);
  __pyx_r = __pyx_pf_4grpc_7_cython_6cygrpc_29ReceiveCloseOnServerOperation_4cancelled(((struct __pyx_obj_4grpc_7_cython_6cygrpc_ReceiveCloseOnServerOperation *)__pyx_v_self));

  /* function exit code */
  __Pyx_RefNannyFinishContext();
  return __pyx_r;
}

static PyObject *__pyx_pf_4grpc_7_cython_6cygrpc_29ReceiveCloseOnServerOperation_4cancelled(struct __pyx_obj_4grpc_7_cython_6cygrpc_ReceiveCloseOnServerOperation *__pyx_v_self) {
  PyObject *__pyx_r = NULL;
  __Pyx_RefNannyDeclarations
  __Pyx_RefNannySetupContext("cancelled", 0);

  /* "src/python/grpcio/grpc/_cython/_cygrpc/operation.pyx.pxi":247
 * 
 *   def cancelled(self):
 *     return self._cancelled             # <<<<<<<<<<<<<<
 */
  __Pyx_XDECREF(__pyx_r);
  __Pyx_INCREF(__pyx_v_self->_cancelled);
  __pyx_r = __pyx_v_self->_cancelled;
  goto __pyx_L0;

  /* "src/python/grpcio/grpc/_cython/_cygrpc/operation.pyx.pxi":246
 *     self._cancelled = bool(self._c_cancelled)
 * 
 *   def cancelled(self):             # <<<<<<<<<<<<<<
 *     return self._cancelled
 */

  /* function exit code */
  __pyx_L0:;
  __Pyx_XGIVEREF(__pyx_r);
  __Pyx_RefNannyFinishContext();
  return __pyx_r;
}

/* "src/python/grpcio/grpc/_cython/_cygrpc/operation.pxd.pxi":106
 * cdef class ReceiveCloseOnServerOperation(Operation):
 * 
 *   cdef readonly int _flags             # <<<<<<<<<<<<<<
 *   cdef object _cancelled
 *   cdef int _c_cancelled
 */

/* Python wrapper */
static PyObject *__pyx_pw_4grpc_7_cython_6cygrpc_29ReceiveCloseOnServerOperation_6_flags_1__get__(PyObject *__pyx_v_self); /*proto*/
static PyObject *__pyx_pw_4grpc_7_cython_6cygrpc_29ReceiveCloseOnServerOperation_6_flags_1__get__(PyObject *__pyx_v_self) {
  PyObject *__pyx_r = 0;
  __Pyx_RefNannyDeclarations
  __Pyx_RefNannySetupContext("__get__ (wrapper)", 0);
  __pyx_r = __pyx_pf_4grpc_7_cython_6cygrpc_29ReceiveCloseOnServerOperation_6_flags___get__(((struct __pyx_obj_4grpc_7_cython_6cygrpc_ReceiveCloseOnServerOperation *)__pyx_v_self));

  /* function exit code */
  __Pyx_RefNannyFinishContext();
  return __pyx_r;
}

static PyObject *__pyx_pf_4grpc_7_cython_6cygrpc_29ReceiveCloseOnServerOperation_6_flags___get__(struct __pyx_obj_4grpc_7_cython_6cygrpc_ReceiveCloseOnServerOperation *__pyx_v_self) {
  PyObject *__pyx_r = NULL;
  __Pyx_RefNannyDeclarations
  PyObject *__pyx_t_1 = NULL;
  __Pyx_RefNannySetupContext("__get__", 0);
  __Pyx_XDECREF(__pyx_r);
  __pyx_t_1 = __Pyx_PyInt_From_int(__pyx_v_self->_flags); if (unlikely(!__pyx_t_1)) __PYX_ERR(15, 106, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_1);
  __pyx_r = __pyx_t_1;
  __pyx_t_1 = 0;
  goto __pyx_L0;

  /* function exit code */
  __pyx_L1_error:;
  __Pyx_XDECREF(__pyx_t_1);
  __Pyx_AddTraceback("grpc._cython.cygrpc.ReceiveCloseOnServerOperation._flags.__get__", __pyx_clineno, __pyx_lineno, __pyx_filename);
  __pyx_r = NULL;
  __pyx_L0:;
  __Pyx_XGIVEREF(__pyx_r);
  __Pyx_RefNannyFinishContext();
  return __pyx_r;
}

/* "(tree fragment)":1
 * def __reduce_cython__(self):             # <<<<<<<<<<<<<<
 *     raise TypeError("no default __reduce__ due to non-trivial __cinit__")
 * def __setstate_cython__(self, __pyx_state):
 */

/* Python wrapper */
static PyObject *__pyx_pw_4grpc_7_cython_6cygrpc_29ReceiveCloseOnServerOperation_7__reduce_cython__(PyObject *__pyx_v_self, CYTHON_UNUSED PyObject *unused); /*proto*/
static PyObject *__pyx_pw_4grpc_7_cython_6cygrpc_29ReceiveCloseOnServerOperation_7__reduce_cython__(PyObject *__pyx_v_self, CYTHON_UNUSED PyObject *unused) {
  PyObject *__pyx_r = 0;
  __Pyx_RefNannyDeclarations
  __Pyx_RefNannySetupContext("__reduce_cython__ (wrapper)", 0);
  __pyx_r = __pyx_pf_4grpc_7_cython_6cygrpc_29ReceiveCloseOnServerOperation_6__reduce_cython__(((struct __pyx_obj_4grpc_7_cython_6cygrpc_ReceiveCloseOnServerOperation *)__pyx_v_self));

  /* function exit code */
  __Pyx_RefNannyFinishContext();
  return __pyx_r;
}

static PyObject *__pyx_pf_4grpc_7_cython_6cygrpc_29ReceiveCloseOnServerOperation_6__reduce_cython__(CYTHON_UNUSED struct __pyx_obj_4grpc_7_cython_6cygrpc_ReceiveCloseOnServerOperation *__pyx_v_self) {
  PyObject *__pyx_r = NULL;
  __Pyx_RefNannyDeclarations
  PyObject *__pyx_t_1 = NULL;
  __Pyx_RefNannySetupContext("__reduce_cython__", 0);

  /* "(tree fragment)":2
 * def __reduce_cython__(self):
 *     raise TypeError("no default __reduce__ due to non-trivial __cinit__")             # <<<<<<<<<<<<<<
 * def __setstate_cython__(self, __pyx_state):
 *     raise TypeError("no default __reduce__ due to non-trivial __cinit__")
 */
  __pyx_t_1 = __Pyx_PyObject_Call(__pyx_builtin_TypeError, __pyx_tuple__100, NULL); if (unlikely(!__pyx_t_1)) __PYX_ERR(7, 2, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_1);
  __Pyx_Raise(__pyx_t_1, 0, 0, 0);
  __Pyx_DECREF(__pyx_t_1); __pyx_t_1 = 0;
  __PYX_ERR(7, 2, __pyx_L1_error)

  /* "(tree fragment)":1
 * def __reduce_cython__(self):             # <<<<<<<<<<<<<<
 *     raise TypeError("no default __reduce__ due to non-trivial __cinit__")
 * def __setstate_cython__(self, __pyx_state):
 */

  /* function exit code */
  __pyx_L1_error:;
  __Pyx_XDECREF(__pyx_t_1);
  __Pyx_AddTraceback("grpc._cython.cygrpc.ReceiveCloseOnServerOperation.__reduce_cython__", __pyx_clineno, __pyx_lineno, __pyx_filename);
  __pyx_r = NULL;
  __Pyx_XGIVEREF(__pyx_r);
  __Pyx_RefNannyFinishContext();
  return __pyx_r;
}

/* "(tree fragment)":3
 * def __reduce_cython__(self):
 *     raise TypeError("no default __reduce__ due to non-trivial __cinit__")
 * def __setstate_cython__(self, __pyx_state):             # <<<<<<<<<<<<<<
 *     raise TypeError("no default __reduce__ due to non-trivial __cinit__")
 */

/* Python wrapper */
static PyObject *__pyx_pw_4grpc_7_cython_6cygrpc_29ReceiveCloseOnServerOperation_9__setstate_cython__(PyObject *__pyx_v_self, PyObject *__pyx_v___pyx_state); /*proto*/
static PyObject *__pyx_pw_4grpc_7_cython_6cygrpc_29ReceiveCloseOnServerOperation_9__setstate_cython__(PyObject *__pyx_v_self, PyObject *__pyx_v___pyx_state) {
  PyObject *__pyx_r = 0;
  __Pyx_RefNannyDeclarations
  __Pyx_RefNannySetupContext("__setstate_cython__ (wrapper)", 0);
  __pyx_r = __pyx_pf_4grpc_7_cython_6cygrpc_29ReceiveCloseOnServerOperation_8__setstate_cython__(((struct __pyx_obj_4grpc_7_cython_6cygrpc_ReceiveCloseOnServerOperation *)__pyx_v_self), ((PyObject *)__pyx_v___pyx_state));

  /* function exit code */
  __Pyx_RefNannyFinishContext();
  return __pyx_r;
}

static PyObject *__pyx_pf_4grpc_7_cython_6cygrpc_29ReceiveCloseOnServerOperation_8__setstate_cython__(CYTHON_UNUSED struct __pyx_obj_4grpc_7_cython_6cygrpc_ReceiveCloseOnServerOperation *__pyx_v_self, CYTHON_UNUSED PyObject *__pyx_v___pyx_state) {
  PyObject *__pyx_r = NULL;
  __Pyx_RefNannyDeclarations
  PyObject *__pyx_t_1 = NULL;
  __Pyx_RefNannySetupContext("__setstate_cython__", 0);

  /* "(tree fragment)":4
 *     raise TypeError("no default __reduce__ due to non-trivial __cinit__")
 * def __setstate_cython__(self, __pyx_state):
 *     raise TypeError("no default __reduce__ due to non-trivial __cinit__")             # <<<<<<<<<<<<<<
 */
  __pyx_t_1 = __Pyx_PyObject_Call(__pyx_builtin_TypeError, __pyx_tuple__101, NULL); if (unlikely(!__pyx_t_1)) __PYX_ERR(7, 4, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_1);
  __Pyx_Raise(__pyx_t_1, 0, 0, 0);
  __Pyx_DECREF(__pyx_t_1); __pyx_t_1 = 0;
  __PYX_ERR(7, 4, __pyx_L1_error)

  /* "(tree fragment)":3
 * def __reduce_cython__(self):
 *     raise TypeError("no default __reduce__ due to non-trivial __cinit__")
 * def __setstate_cython__(self, __pyx_state):             # <<<<<<<<<<<<<<
 *     raise TypeError("no default __reduce__ due to non-trivial __cinit__")
 */

  /* function exit code */
  __pyx_L1_error:;
  __Pyx_XDECREF(__pyx_t_1);
  __Pyx_AddTraceback("grpc._cython.cygrpc.ReceiveCloseOnServerOperation.__setstate_cython__", __pyx_clineno, __pyx_lineno, __pyx_filename);
  __pyx_r = NULL;
  __Pyx_XGIVEREF(__pyx_r);
  __Pyx_RefNannyFinishContext();
  return __pyx_r;
}

/* "src/python/grpcio/grpc/_cython/_cygrpc/records.pyx.pxi":16
 * 
 * 
 * cdef bytes _slice_bytes(grpc_slice slice):             # <<<<<<<<<<<<<<
 *   cdef void *start = grpc_slice_start_ptr(slice)
 *   cdef size_t length = grpc_slice_length(slice)
 */

static PyObject *__pyx_f_4grpc_7_cython_6cygrpc__slice_bytes(grpc_slice __pyx_v_slice) {
  void *__pyx_v_start;
  size_t __pyx_v_length;
  PyObject *__pyx_r = NULL;
  __Pyx_RefNannyDeclarations
  PyObject *__pyx_t_1 = NULL;
  __Pyx_RefNannySetupContext("_slice_bytes", 0);

  /* "src/python/grpcio/grpc/_cython/_cygrpc/records.pyx.pxi":17
 * 
 * cdef bytes _slice_bytes(grpc_slice slice):
 *   cdef void *start = grpc_slice_start_ptr(slice)             # <<<<<<<<<<<<<<
 *   cdef size_t length = grpc_slice_length(slice)
 *   return (<const char *>start)[:length]
 */
  __pyx_v_start = GRPC_SLICE_START_PTR(__pyx_v_slice);

  /* "src/python/grpcio/grpc/_cython/_cygrpc/records.pyx.pxi":18
 * cdef bytes _slice_bytes(grpc_slice slice):
 *   cdef void *start = grpc_slice_start_ptr(slice)
 *   cdef size_t length = grpc_slice_length(slice)             # <<<<<<<<<<<<<<
 *   return (<const char *>start)[:length]
 * 
 */
  __pyx_v_length = GRPC_SLICE_LENGTH(__pyx_v_slice);

  /* "src/python/grpcio/grpc/_cython/_cygrpc/records.pyx.pxi":19
 *   cdef void *start = grpc_slice_start_ptr(slice)
 *   cdef size_t length = grpc_slice_length(slice)
 *   return (<const char *>start)[:length]             # <<<<<<<<<<<<<<
 * 
 * cdef grpc_slice _copy_slice(grpc_slice slice) nogil:
 */
  __Pyx_XDECREF(__pyx_r);
  __pyx_t_1 = __Pyx_PyBytes_FromStringAndSize(((char const *)__pyx_v_start) + 0, __pyx_v_length - 0); if (unlikely(!__pyx_t_1)) __PYX_ERR(16, 19, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_1);
  __pyx_r = ((PyObject*)__pyx_t_1);
  __pyx_t_1 = 0;
  goto __pyx_L0;

  /* "src/python/grpcio/grpc/_cython/_cygrpc/records.pyx.pxi":16
 * 
 * 
 * cdef bytes _slice_bytes(grpc_slice slice):             # <<<<<<<<<<<<<<
 *   cdef void *start = grpc_slice_start_ptr(slice)
 *   cdef size_t length = grpc_slice_length(slice)
 */

  /* function exit code */
  __pyx_L1_error:;
  __Pyx_XDECREF(__pyx_t_1);
  __Pyx_AddTraceback("grpc._cython.cygrpc._slice_bytes", __pyx_clineno, __pyx_lineno, __pyx_filename);
  __pyx_r = 0;
  __pyx_L0:;
  __Pyx_XGIVEREF(__pyx_r);
  __Pyx_RefNannyFinishContext();
  return __pyx_r;
}

/* "src/python/grpcio/grpc/_cython/_cygrpc/records.pyx.pxi":21
 *   return (<const char *>start)[:length]
 * 
 * cdef grpc_slice _copy_slice(grpc_slice slice) nogil:             # <<<<<<<<<<<<<<
 *   cdef void *start = grpc_slice_start_ptr(slice)
 *   cdef size_t length = grpc_slice_length(slice)
 */

static grpc_slice __pyx_f_4grpc_7_cython_6cygrpc__copy_slice(grpc_slice __pyx_v_slice) {
  void *__pyx_v_start;
  size_t __pyx_v_length;
  grpc_slice __pyx_r;

  /* "src/python/grpcio/grpc/_cython/_cygrpc/records.pyx.pxi":22
 * 
 * cdef grpc_slice _copy_slice(grpc_slice slice) nogil:
 *   cdef void *start = grpc_slice_start_ptr(slice)             # <<<<<<<<<<<<<<
 *   cdef size_t length = grpc_slice_length(slice)
 *   return grpc_slice_from_copied_buffer(<const char *>start, length)
 */
  __pyx_v_start = GRPC_SLICE_START_PTR(__pyx_v_slice);

  /* "src/python/grpcio/grpc/_cython/_cygrpc/records.pyx.pxi":23
 * cdef grpc_slice _copy_slice(grpc_slice slice) nogil:
 *   cdef void *start = grpc_slice_start_ptr(slice)
 *   cdef size_t length = grpc_slice_length(slice)             # <<<<<<<<<<<<<<
 *   return grpc_slice_from_copied_buffer(<const char *>start, length)
 * 
 */
  __pyx_v_length = GRPC_SLICE_LENGTH(__pyx_v_slice);

  /* "src/python/grpcio/grpc/_cython/_cygrpc/records.pyx.pxi":24
 *   cdef void *start = grpc_slice_start_ptr(slice)
 *   cdef size_t length = grpc_slice_length(slice)
 *   return grpc_slice_from_copied_buffer(<const char *>start, length)             # <<<<<<<<<<<<<<
 * 
 * cdef grpc_slice _slice_from_bytes(bytes value) nogil:
 */
  __pyx_r = grpc_slice_from_copied_buffer(((char const *)__pyx_v_start), __pyx_v_length);
  goto __pyx_L0;

  /* "src/python/grpcio/grpc/_cython/_cygrpc/records.pyx.pxi":21
 *   return (<const char *>start)[:length]
 * 
 * cdef grpc_slice _copy_slice(grpc_slice slice) nogil:             # <<<<<<<<<<<<<<
 *   cdef void *start = grpc_slice_start_ptr(slice)
 *   cdef size_t length = grpc_slice_length(slice)
 */

  /* function exit code */
  __pyx_L0:;
  return __pyx_r;
}

/* "src/python/grpcio/grpc/_cython/_cygrpc/records.pyx.pxi":26
 *   return grpc_slice_from_copied_buffer(<const char *>start, length)
 * 
 * cdef grpc_slice _slice_from_bytes(bytes value) nogil:             # <<<<<<<<<<<<<<
 *   cdef const char *value_ptr
 *   cdef size_t length
 */

static grpc_slice __pyx_f_4grpc_7_cython_6cygrpc__slice_from_bytes(PyObject *__pyx_v_value) {
  char const *__pyx_v_value_ptr;
  size_t __pyx_v_length;
  grpc_slice __pyx_r;
  __Pyx_RefNannyDeclarations
  char const *__pyx_t_1;
  Py_ssize_t __pyx_t_2;
  #ifdef WITH_THREAD
  PyGILState_STATE __pyx_gilstate_save = __Pyx_PyGILState_Ensure();
  #endif
  __Pyx_RefNannySetupContext("_slice_from_bytes", 0);
  #ifdef WITH_THREAD
  __Pyx_PyGILState_Release(__pyx_gilstate_save);
  #endif

  /* "src/python/grpcio/grpc/_cython/_cygrpc/records.pyx.pxi":27
 * 
 * cdef grpc_slice _slice_from_bytes(bytes value) nogil:
 *   cdef const char *value_ptr             # <<<<<<<<<<<<<<
 *   cdef size_t length
 *   with gil:
 */
  /*try:*/ {

    /* "src/python/grpcio/grpc/_cython/_cygrpc/records.pyx.pxi":29
 *   cdef const char *value_ptr
 *   cdef size_t length
 *   with gil:             # <<<<<<<<<<<<<<
 *     value_ptr = <const char *>value
 *     length = len(value)
 */
    {
        #ifdef WITH_THREAD
        PyGILState_STATE __pyx_gilstate_save = __Pyx_PyGILState_Ensure();
        #endif
        /*try:*/ {

          /* "src/python/grpcio/grpc/_cython/_cygrpc/records.pyx.pxi":30
 *   cdef size_t length
 *   with gil:
 *     value_ptr = <const char *>value             # <<<<<<<<<<<<<<
 *     length = len(value)
 *   return grpc_slice_from_copied_buffer(value_ptr, length)
 */
          if (unlikely(__pyx_v_value == Py_None)) {
            PyErr_SetString(PyExc_TypeError, "expected bytes, NoneType found");
            __PYX_ERR(16, 30, __pyx_L7_error)
          }
          __pyx_t_1 = __Pyx_PyBytes_AsString(__pyx_v_value); if (unlikely((!__pyx_t_1) && PyErr_Occurred())) __PYX_ERR(16, 30, __pyx_L7_error)
          __pyx_v_value_ptr = ((char const *)__pyx_t_1);

          /* "src/python/grpcio/grpc/_cython/_cygrpc/records.pyx.pxi":31
 *   with gil:
 *     value_ptr = <const char *>value
 *     length = len(value)             # <<<<<<<<<<<<<<
 *   return grpc_slice_from_copied_buffer(value_ptr, length)
 * 
 */
          if (unlikely(__pyx_v_value == Py_None)) {
            PyErr_SetString(PyExc_TypeError, "object of type 'NoneType' has no len()");
            __PYX_ERR(16, 31, __pyx_L7_error)
          }
          __pyx_t_2 = PyBytes_GET_SIZE(__pyx_v_value); if (unlikely(__pyx_t_2 == ((Py_ssize_t)-1))) __PYX_ERR(16, 31, __pyx_L7_error)
          __pyx_v_length = __pyx_t_2;
        }

        /* "src/python/grpcio/grpc/_cython/_cygrpc/records.pyx.pxi":29
 *   cdef const char *value_ptr
 *   cdef size_t length
 *   with gil:             # <<<<<<<<<<<<<<
 *     value_ptr = <const char *>value
 *     length = len(value)
 */
        /*finally:*/ {
          /*normal exit:*/{
            #ifdef WITH_THREAD
            __Pyx_PyGILState_Release(__pyx_gilstate_save);
            #endif
            goto __pyx_L8;
          }
          __pyx_L7_error: {
            #ifdef WITH_THREAD
            __Pyx_PyGILState_Release(__pyx_gilstate_save);
            #endif
            goto __pyx_L4_error;
          }
          __pyx_L8:;
        }
    }

    /* "src/python/grpcio/grpc/_cython/_cygrpc/records.pyx.pxi":32
 *     value_ptr = <const char *>value
 *     length = len(value)
 *   return grpc_slice_from_copied_buffer(value_ptr, length)             # <<<<<<<<<<<<<<
 * 
 * 
 */
    __pyx_r = grpc_slice_from_copied_buffer(__pyx_v_value_ptr, __pyx_v_length);
    goto __pyx_L3_return;
  }

  /* "src/python/grpcio/grpc/_cython/_cygrpc/records.pyx.pxi":27
 * 
 * cdef grpc_slice _slice_from_bytes(bytes value) nogil:
 *   cdef const char *value_ptr             # <<<<<<<<<<<<<<
 *   cdef size_t length
 *   with gil:
 */
  /*finally:*/ {
    __pyx_L3_return: {
      #ifdef WITH_THREAD
      __pyx_gilstate_save = __Pyx_PyGILState_Ensure();
      #endif
      goto __pyx_L0;
    }
    __pyx_L4_error: {
      #ifdef WITH_THREAD
      __pyx_gilstate_save = __Pyx_PyGILState_Ensure();
      #endif
      goto __pyx_L1_error;
    }
  }

  /* "src/python/grpcio/grpc/_cython/_cygrpc/records.pyx.pxi":26
 *   return grpc_slice_from_copied_buffer(<const char *>start, length)
 * 
 * cdef grpc_slice _slice_from_bytes(bytes value) nogil:             # <<<<<<<<<<<<<<
 *   cdef const char *value_ptr
 *   cdef size_t length
 */

  /* function exit code */
  __Pyx_pretend_to_initialize(&__pyx_r);
  goto __pyx_L0;
  __pyx_L1_error:;
  __Pyx_WriteUnraisable("grpc._cython.cygrpc._slice_from_bytes", __pyx_clineno, __pyx_lineno, __pyx_filename, 1, 1);
  __Pyx_pretend_to_initialize(&__pyx_r);
  __pyx_L0:;
  #ifdef WITH_THREAD
  __Pyx_PyGILState_Release(__pyx_gilstate_save);
  #endif
  return __pyx_r;
}

/* "src/python/grpcio/grpc/_cython/_cygrpc/records.pyx.pxi":132
 * cdef class CallDetails:
 * 
 *   def __cinit__(self):             # <<<<<<<<<<<<<<
 *     fork_handlers_and_grpc_init()
 *     with nogil:
 */

/* Python wrapper */
static int __pyx_pw_4grpc_7_cython_6cygrpc_11CallDetails_1__cinit__(PyObject *__pyx_v_self, PyObject *__pyx_args, PyObject *__pyx_kwds); /*proto*/
static int __pyx_pw_4grpc_7_cython_6cygrpc_11CallDetails_1__cinit__(PyObject *__pyx_v_self, PyObject *__pyx_args, PyObject *__pyx_kwds) {
  int __pyx_r;
  __Pyx_RefNannyDeclarations
  __Pyx_RefNannySetupContext("__cinit__ (wrapper)", 0);
  if (unlikely(PyTuple_GET_SIZE(__pyx_args) > 0)) {
    __Pyx_RaiseArgtupleInvalid("__cinit__", 1, 0, 0, PyTuple_GET_SIZE(__pyx_args)); return -1;}
  if (unlikely(__pyx_kwds) && unlikely(PyDict_Size(__pyx_kwds) > 0) && unlikely(!__Pyx_CheckKeywordStrings(__pyx_kwds, "__cinit__", 0))) return -1;
  __pyx_r = __pyx_pf_4grpc_7_cython_6cygrpc_11CallDetails___cinit__(((struct __pyx_obj_4grpc_7_cython_6cygrpc_CallDetails *)__pyx_v_self));

  /* function exit code */
  __Pyx_RefNannyFinishContext();
  return __pyx_r;
}

static int __pyx_pf_4grpc_7_cython_6cygrpc_11CallDetails___cinit__(struct __pyx_obj_4grpc_7_cython_6cygrpc_CallDetails *__pyx_v_self) {
  int __pyx_r;
  __Pyx_RefNannyDeclarations
  PyObject *__pyx_t_1 = NULL;
  PyObject *__pyx_t_2 = NULL;
  PyObject *__pyx_t_3 = NULL;
  __Pyx_RefNannySetupContext("__cinit__", 0);

  /* "src/python/grpcio/grpc/_cython/_cygrpc/records.pyx.pxi":133
 * 
 *   def __cinit__(self):
 *     fork_handlers_and_grpc_init()             # <<<<<<<<<<<<<<
 *     with nogil:
 *       grpc_call_details_init(&self.c_details)
 */
  __pyx_t_2 = __Pyx_GetModuleGlobalName(__pyx_n_s_fork_handlers_and_grpc_init); if (unlikely(!__pyx_t_2)) __PYX_ERR(16, 133, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_2);
  __pyx_t_3 = NULL;
  if (CYTHON_UNPACK_METHODS && unlikely(PyMethod_Check(__pyx_t_2))) {
    __pyx_t_3 = PyMethod_GET_SELF(__pyx_t_2);
    if (likely(__pyx_t_3)) {
      PyObject* function = PyMethod_GET_FUNCTION(__pyx_t_2);
      __Pyx_INCREF(__pyx_t_3);
      __Pyx_INCREF(function);
      __Pyx_DECREF_SET(__pyx_t_2, function);
    }
  }
  if (__pyx_t_3) {
    __pyx_t_1 = __Pyx_PyObject_CallOneArg(__pyx_t_2, __pyx_t_3); if (unlikely(!__pyx_t_1)) __PYX_ERR(16, 133, __pyx_L1_error)
    __Pyx_DECREF(__pyx_t_3); __pyx_t_3 = 0;
  } else {
    __pyx_t_1 = __Pyx_PyObject_CallNoArg(__pyx_t_2); if (unlikely(!__pyx_t_1)) __PYX_ERR(16, 133, __pyx_L1_error)
  }
  __Pyx_GOTREF(__pyx_t_1);
  __Pyx_DECREF(__pyx_t_2); __pyx_t_2 = 0;
  __Pyx_DECREF(__pyx_t_1); __pyx_t_1 = 0;

  /* "src/python/grpcio/grpc/_cython/_cygrpc/records.pyx.pxi":134
 *   def __cinit__(self):
 *     fork_handlers_and_grpc_init()
 *     with nogil:             # <<<<<<<<<<<<<<
 *       grpc_call_details_init(&self.c_details)
 * 
 */
  {
      #ifdef WITH_THREAD
      PyThreadState *_save;
      Py_UNBLOCK_THREADS
      __Pyx_FastGIL_Remember();
      #endif
      /*try:*/ {

        /* "src/python/grpcio/grpc/_cython/_cygrpc/records.pyx.pxi":135
 *     fork_handlers_and_grpc_init()
 *     with nogil:
 *       grpc_call_details_init(&self.c_details)             # <<<<<<<<<<<<<<
 * 
 *   def __dealloc__(self):
 */
        grpc_call_details_init((&__pyx_v_self->c_details));
      }

      /* "src/python/grpcio/grpc/_cython/_cygrpc/records.pyx.pxi":134
 *   def __cinit__(self):
 *     fork_handlers_and_grpc_init()
 *     with nogil:             # <<<<<<<<<<<<<<
 *       grpc_call_details_init(&self.c_details)
 * 
 */
      /*finally:*/ {
        /*normal exit:*/{
          #ifdef WITH_THREAD
          __Pyx_FastGIL_Forget();
          Py_BLOCK_THREADS
          #endif
          goto __pyx_L5;
        }
        __pyx_L5:;
      }
  }

  /* "src/python/grpcio/grpc/_cython/_cygrpc/records.pyx.pxi":132
 * cdef class CallDetails:
 * 
 *   def __cinit__(self):             # <<<<<<<<<<<<<<
 *     fork_handlers_and_grpc_init()
 *     with nogil:
 */

  /* function exit code */
  __pyx_r = 0;
  goto __pyx_L0;
  __pyx_L1_error:;
  __Pyx_XDECREF(__pyx_t_1);
  __Pyx_XDECREF(__pyx_t_2);
  __Pyx_XDECREF(__pyx_t_3);
  __Pyx_AddTraceback("grpc._cython.cygrpc.CallDetails.__cinit__", __pyx_clineno, __pyx_lineno, __pyx_filename);
  __pyx_r = -1;
  __pyx_L0:;
  __Pyx_RefNannyFinishContext();
  return __pyx_r;
}

/* "src/python/grpcio/grpc/_cython/_cygrpc/records.pyx.pxi":137
 *       grpc_call_details_init(&self.c_details)
 * 
 *   def __dealloc__(self):             # <<<<<<<<<<<<<<
 *     with nogil:
 *       grpc_call_details_destroy(&self.c_details)
 */

/* Python wrapper */
static void __pyx_pw_4grpc_7_cython_6cygrpc_11CallDetails_3__dealloc__(PyObject *__pyx_v_self); /*proto*/
static void __pyx_pw_4grpc_7_cython_6cygrpc_11CallDetails_3__dealloc__(PyObject *__pyx_v_self) {
  __Pyx_RefNannyDeclarations
  __Pyx_RefNannySetupContext("__dealloc__ (wrapper)", 0);
  __pyx_pf_4grpc_7_cython_6cygrpc_11CallDetails_2__dealloc__(((struct __pyx_obj_4grpc_7_cython_6cygrpc_CallDetails *)__pyx_v_self));

  /* function exit code */
  __Pyx_RefNannyFinishContext();
}

static void __pyx_pf_4grpc_7_cython_6cygrpc_11CallDetails_2__dealloc__(struct __pyx_obj_4grpc_7_cython_6cygrpc_CallDetails *__pyx_v_self) {
  __Pyx_RefNannyDeclarations
  __Pyx_RefNannySetupContext("__dealloc__", 0);

  /* "src/python/grpcio/grpc/_cython/_cygrpc/records.pyx.pxi":138
 * 
 *   def __dealloc__(self):
 *     with nogil:             # <<<<<<<<<<<<<<
 *       grpc_call_details_destroy(&self.c_details)
 *     grpc_shutdown_blocking()
 */
  {
      #ifdef WITH_THREAD
      PyThreadState *_save;
      Py_UNBLOCK_THREADS
      __Pyx_FastGIL_Remember();
      #endif
      /*try:*/ {

        /* "src/python/grpcio/grpc/_cython/_cygrpc/records.pyx.pxi":139
 *   def __dealloc__(self):
 *     with nogil:
 *       grpc_call_details_destroy(&self.c_details)             # <<<<<<<<<<<<<<
 *     grpc_shutdown_blocking()
 * 
 */
        grpc_call_details_destroy((&__pyx_v_self->c_details));
      }

      /* "src/python/grpcio/grpc/_cython/_cygrpc/records.pyx.pxi":138
 * 
 *   def __dealloc__(self):
 *     with nogil:             # <<<<<<<<<<<<<<
 *       grpc_call_details_destroy(&self.c_details)
 *     grpc_shutdown_blocking()
 */
      /*finally:*/ {
        /*normal exit:*/{
          #ifdef WITH_THREAD
          __Pyx_FastGIL_Forget();
          Py_BLOCK_THREADS
          #endif
          goto __pyx_L5;
        }
        __pyx_L5:;
      }
  }

  /* "src/python/grpcio/grpc/_cython/_cygrpc/records.pyx.pxi":140
 *     with nogil:
 *       grpc_call_details_destroy(&self.c_details)
 *     grpc_shutdown_blocking()             # <<<<<<<<<<<<<<
 * 
 *   @property
 */
  grpc_shutdown_blocking();

  /* "src/python/grpcio/grpc/_cython/_cygrpc/records.pyx.pxi":137
 *       grpc_call_details_init(&self.c_details)
 * 
 *   def __dealloc__(self):             # <<<<<<<<<<<<<<
 *     with nogil:
 *       grpc_call_details_destroy(&self.c_details)
 */

  /* function exit code */
  __Pyx_RefNannyFinishContext();
}

/* "src/python/grpcio/grpc/_cython/_cygrpc/records.pyx.pxi":143
 * 
 *   @property
 *   def method(self):             # <<<<<<<<<<<<<<
 *     return _slice_bytes(self.c_details.method)
 * 
 */

/* Python wrapper */
static PyObject *__pyx_pw_4grpc_7_cython_6cygrpc_11CallDetails_6method_1__get__(PyObject *__pyx_v_self); /*proto*/
static PyObject *__pyx_pw_4grpc_7_cython_6cygrpc_11CallDetails_6method_1__get__(PyObject *__pyx_v_self) {
  PyObject *__pyx_r = 0;
  __Pyx_RefNannyDeclarations
  __Pyx_RefNannySetupContext("__get__ (wrapper)", 0);
  __pyx_r = __pyx_pf_4grpc_7_cython_6cygrpc_11CallDetails_6method___get__(((struct __pyx_obj_4grpc_7_cython_6cygrpc_CallDetails *)__pyx_v_self));

  /* function exit code */
  __Pyx_RefNannyFinishContext();
  return __pyx_r;
}

static PyObject *__pyx_pf_4grpc_7_cython_6cygrpc_11CallDetails_6method___get__(struct __pyx_obj_4grpc_7_cython_6cygrpc_CallDetails *__pyx_v_self) {
  PyObject *__pyx_r = NULL;
  __Pyx_RefNannyDeclarations
  PyObject *__pyx_t_1 = NULL;
  __Pyx_RefNannySetupContext("__get__", 0);

  /* "src/python/grpcio/grpc/_cython/_cygrpc/records.pyx.pxi":144
 *   @property
 *   def method(self):
 *     return _slice_bytes(self.c_details.method)             # <<<<<<<<<<<<<<
 * 
 *   @property
 */
  __Pyx_XDECREF(__pyx_r);
  __pyx_t_1 = __pyx_f_4grpc_7_cython_6cygrpc__slice_bytes(__pyx_v_self->c_details.method); if (unlikely(!__pyx_t_1)) __PYX_ERR(16, 144, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_1);
  __pyx_r = __pyx_t_1;
  __pyx_t_1 = 0;
  goto __pyx_L0;

  /* "src/python/grpcio/grpc/_cython/_cygrpc/records.pyx.pxi":143
 * 
 *   @property
 *   def method(self):             # <<<<<<<<<<<<<<
 *     return _slice_bytes(self.c_details.method)
 * 
 */

  /* function exit code */
  __pyx_L1_error:;
  __Pyx_XDECREF(__pyx_t_1);
  __Pyx_AddTraceback("grpc._cython.cygrpc.CallDetails.method.__get__", __pyx_clineno, __pyx_lineno, __pyx_filename);
  __pyx_r = NULL;
  __pyx_L0:;
  __Pyx_XGIVEREF(__pyx_r);
  __Pyx_RefNannyFinishContext();
  return __pyx_r;
}

/* "src/python/grpcio/grpc/_cython/_cygrpc/records.pyx.pxi":147
 * 
 *   @property
 *   def host(self):             # <<<<<<<<<<<<<<
 *     return _slice_bytes(self.c_details.host)
 * 
 */

/* Python wrapper */
static PyObject *__pyx_pw_4grpc_7_cython_6cygrpc_11CallDetails_4host_1__get__(PyObject *__pyx_v_self); /*proto*/
static PyObject *__pyx_pw_4grpc_7_cython_6cygrpc_11CallDetails_4host_1__get__(PyObject *__pyx_v_self) {
  PyObject *__pyx_r = 0;
  __Pyx_RefNannyDeclarations
  __Pyx_RefNannySetupContext("__get__ (wrapper)", 0);
  __pyx_r = __pyx_pf_4grpc_7_cython_6cygrpc_11CallDetails_4host___get__(((struct __pyx_obj_4grpc_7_cython_6cygrpc_CallDetails *)__pyx_v_self));

  /* function exit code */
  __Pyx_RefNannyFinishContext();
  return __pyx_r;
}

static PyObject *__pyx_pf_4grpc_7_cython_6cygrpc_11CallDetails_4host___get__(struct __pyx_obj_4grpc_7_cython_6cygrpc_CallDetails *__pyx_v_self) {
  PyObject *__pyx_r = NULL;
  __Pyx_RefNannyDeclarations
  PyObject *__pyx_t_1 = NULL;
  __Pyx_RefNannySetupContext("__get__", 0);

  /* "src/python/grpcio/grpc/_cython/_cygrpc/records.pyx.pxi":148
 *   @property
 *   def host(self):
 *     return _slice_bytes(self.c_details.host)             # <<<<<<<<<<<<<<
 * 
 *   @property
 */
  __Pyx_XDECREF(__pyx_r);
  __pyx_t_1 = __pyx_f_4grpc_7_cython_6cygrpc__slice_bytes(__pyx_v_self->c_details.host); if (unlikely(!__pyx_t_1)) __PYX_ERR(16, 148, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_1);
  __pyx_r = __pyx_t_1;
  __pyx_t_1 = 0;
  goto __pyx_L0;

  /* "src/python/grpcio/grpc/_cython/_cygrpc/records.pyx.pxi":147
 * 
 *   @property
 *   def host(self):             # <<<<<<<<<<<<<<
 *     return _slice_bytes(self.c_details.host)
 * 
 */

  /* function exit code */
  __pyx_L1_error:;
  __Pyx_XDECREF(__pyx_t_1);
  __Pyx_AddTraceback("grpc._cython.cygrpc.CallDetails.host.__get__", __pyx_clineno, __pyx_lineno, __pyx_filename);
  __pyx_r = NULL;
  __pyx_L0:;
  __Pyx_XGIVEREF(__pyx_r);
  __Pyx_RefNannyFinishContext();
  return __pyx_r;
}

/* "src/python/grpcio/grpc/_cython/_cygrpc/records.pyx.pxi":151
 * 
 *   @property
 *   def deadline(self):             # <<<<<<<<<<<<<<
 *     return _time_from_timespec(self.c_details.deadline)
 * 
 */

/* Python wrapper */
static PyObject *__pyx_pw_4grpc_7_cython_6cygrpc_11CallDetails_8deadline_1__get__(PyObject *__pyx_v_self); /*proto*/
static PyObject *__pyx_pw_4grpc_7_cython_6cygrpc_11CallDetails_8deadline_1__get__(PyObject *__pyx_v_self) {
  PyObject *__pyx_r = 0;
  __Pyx_RefNannyDeclarations
  __Pyx_RefNannySetupContext("__get__ (wrapper)", 0);
  __pyx_r = __pyx_pf_4grpc_7_cython_6cygrpc_11CallDetails_8deadline___get__(((struct __pyx_obj_4grpc_7_cython_6cygrpc_CallDetails *)__pyx_v_self));

  /* function exit code */
  __Pyx_RefNannyFinishContext();
  return __pyx_r;
}

static PyObject *__pyx_pf_4grpc_7_cython_6cygrpc_11CallDetails_8deadline___get__(struct __pyx_obj_4grpc_7_cython_6cygrpc_CallDetails *__pyx_v_self) {
  PyObject *__pyx_r = NULL;
  __Pyx_RefNannyDeclarations
  double __pyx_t_1;
  PyObject *__pyx_t_2 = NULL;
  __Pyx_RefNannySetupContext("__get__", 0);

  /* "src/python/grpcio/grpc/_cython/_cygrpc/records.pyx.pxi":152
 *   @property
 *   def deadline(self):
 *     return _time_from_timespec(self.c_details.deadline)             # <<<<<<<<<<<<<<
 * 
 * 
 */
  __Pyx_XDECREF(__pyx_r);
  __pyx_t_1 = __pyx_f_4grpc_7_cython_6cygrpc__time_from_timespec(__pyx_v_self->c_details.deadline); if (unlikely(__pyx_t_1 == ((double)-1) && PyErr_Occurred())) __PYX_ERR(16, 152, __pyx_L1_error)
  __pyx_t_2 = PyFloat_FromDouble(__pyx_t_1); if (unlikely(!__pyx_t_2)) __PYX_ERR(16, 152, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_2);
  __pyx_r = __pyx_t_2;
  __pyx_t_2 = 0;
  goto __pyx_L0;

  /* "src/python/grpcio/grpc/_cython/_cygrpc/records.pyx.pxi":151
 * 
 *   @property
 *   def deadline(self):             # <<<<<<<<<<<<<<
 *     return _time_from_timespec(self.c_details.deadline)
 * 
 */

  /* function exit code */
  __pyx_L1_error:;
  __Pyx_XDECREF(__pyx_t_2);
  __Pyx_AddTraceback("grpc._cython.cygrpc.CallDetails.deadline.__get__", __pyx_clineno, __pyx_lineno, __pyx_filename);
  __pyx_r = NULL;
  __pyx_L0:;
  __Pyx_XGIVEREF(__pyx_r);
  __Pyx_RefNannyFinishContext();
  return __pyx_r;
}

/* "(tree fragment)":1
 * def __reduce_cython__(self):             # <<<<<<<<<<<<<<
 *     raise TypeError("no default __reduce__ due to non-trivial __cinit__")
 * def __setstate_cython__(self, __pyx_state):
 */

/* Python wrapper */
static PyObject *__pyx_pw_4grpc_7_cython_6cygrpc_11CallDetails_5__reduce_cython__(PyObject *__pyx_v_self, CYTHON_UNUSED PyObject *unused); /*proto*/
static PyObject *__pyx_pw_4grpc_7_cython_6cygrpc_11CallDetails_5__reduce_cython__(PyObject *__pyx_v_self, CYTHON_UNUSED PyObject *unused) {
  PyObject *__pyx_r = 0;
  __Pyx_RefNannyDeclarations
  __Pyx_RefNannySetupContext("__reduce_cython__ (wrapper)", 0);
  __pyx_r = __pyx_pf_4grpc_7_cython_6cygrpc_11CallDetails_4__reduce_cython__(((struct __pyx_obj_4grpc_7_cython_6cygrpc_CallDetails *)__pyx_v_self));

  /* function exit code */
  __Pyx_RefNannyFinishContext();
  return __pyx_r;
}

static PyObject *__pyx_pf_4grpc_7_cython_6cygrpc_11CallDetails_4__reduce_cython__(CYTHON_UNUSED struct __pyx_obj_4grpc_7_cython_6cygrpc_CallDetails *__pyx_v_self) {
  PyObject *__pyx_r = NULL;
  __Pyx_RefNannyDeclarations
  PyObject *__pyx_t_1 = NULL;
  __Pyx_RefNannySetupContext("__reduce_cython__", 0);

  /* "(tree fragment)":2
 * def __reduce_cython__(self):
 *     raise TypeError("no default __reduce__ due to non-trivial __cinit__")             # <<<<<<<<<<<<<<
 * def __setstate_cython__(self, __pyx_state):
 *     raise TypeError("no default __reduce__ due to non-trivial __cinit__")
 */
  __pyx_t_1 = __Pyx_PyObject_Call(__pyx_builtin_TypeError, __pyx_tuple__102, NULL); if (unlikely(!__pyx_t_1)) __PYX_ERR(7, 2, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_1);
  __Pyx_Raise(__pyx_t_1, 0, 0, 0);
  __Pyx_DECREF(__pyx_t_1); __pyx_t_1 = 0;
  __PYX_ERR(7, 2, __pyx_L1_error)

  /* "(tree fragment)":1
 * def __reduce_cython__(self):             # <<<<<<<<<<<<<<
 *     raise TypeError("no default __reduce__ due to non-trivial __cinit__")
 * def __setstate_cython__(self, __pyx_state):
 */

  /* function exit code */
  __pyx_L1_error:;
  __Pyx_XDECREF(__pyx_t_1);
  __Pyx_AddTraceback("grpc._cython.cygrpc.CallDetails.__reduce_cython__", __pyx_clineno, __pyx_lineno, __pyx_filename);
  __pyx_r = NULL;
  __Pyx_XGIVEREF(__pyx_r);
  __Pyx_RefNannyFinishContext();
  return __pyx_r;
}

/* "(tree fragment)":3
 * def __reduce_cython__(self):
 *     raise TypeError("no default __reduce__ due to non-trivial __cinit__")
 * def __setstate_cython__(self, __pyx_state):             # <<<<<<<<<<<<<<
 *     raise TypeError("no default __reduce__ due to non-trivial __cinit__")
 */

/* Python wrapper */
static PyObject *__pyx_pw_4grpc_7_cython_6cygrpc_11CallDetails_7__setstate_cython__(PyObject *__pyx_v_self, PyObject *__pyx_v___pyx_state); /*proto*/
static PyObject *__pyx_pw_4grpc_7_cython_6cygrpc_11CallDetails_7__setstate_cython__(PyObject *__pyx_v_self, PyObject *__pyx_v___pyx_state) {
  PyObject *__pyx_r = 0;
  __Pyx_RefNannyDeclarations
  __Pyx_RefNannySetupContext("__setstate_cython__ (wrapper)", 0);
  __pyx_r = __pyx_pf_4grpc_7_cython_6cygrpc_11CallDetails_6__setstate_cython__(((struct __pyx_obj_4grpc_7_cython_6cygrpc_CallDetails *)__pyx_v_self), ((PyObject *)__pyx_v___pyx_state));

  /* function exit code */
  __Pyx_RefNannyFinishContext();
  return __pyx_r;
}

static PyObject *__pyx_pf_4grpc_7_cython_6cygrpc_11CallDetails_6__setstate_cython__(CYTHON_UNUSED struct __pyx_obj_4grpc_7_cython_6cygrpc_CallDetails *__pyx_v_self, CYTHON_UNUSED PyObject *__pyx_v___pyx_state) {
  PyObject *__pyx_r = NULL;
  __Pyx_RefNannyDeclarations
  PyObject *__pyx_t_1 = NULL;
  __Pyx_RefNannySetupContext("__setstate_cython__", 0);

  /* "(tree fragment)":4
 *     raise TypeError("no default __reduce__ due to non-trivial __cinit__")
 * def __setstate_cython__(self, __pyx_state):
 *     raise TypeError("no default __reduce__ due to non-trivial __cinit__")             # <<<<<<<<<<<<<<
 */
  __pyx_t_1 = __Pyx_PyObject_Call(__pyx_builtin_TypeError, __pyx_tuple__103, NULL); if (unlikely(!__pyx_t_1)) __PYX_ERR(7, 4, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_1);
  __Pyx_Raise(__pyx_t_1, 0, 0, 0);
  __Pyx_DECREF(__pyx_t_1); __pyx_t_1 = 0;
  __PYX_ERR(7, 4, __pyx_L1_error)

  /* "(tree fragment)":3
 * def __reduce_cython__(self):
 *     raise TypeError("no default __reduce__ due to non-trivial __cinit__")
 * def __setstate_cython__(self, __pyx_state):             # <<<<<<<<<<<<<<
 *     raise TypeError("no default __reduce__ due to non-trivial __cinit__")
 */

  /* function exit code */
  __pyx_L1_error:;
  __Pyx_XDECREF(__pyx_t_1);
  __Pyx_AddTraceback("grpc._cython.cygrpc.CallDetails.__setstate_cython__", __pyx_clineno, __pyx_lineno, __pyx_filename);
  __pyx_r = NULL;
  __Pyx_XGIVEREF(__pyx_r);
  __Pyx_RefNannyFinishContext();
  return __pyx_r;
}

/* "src/python/grpcio/grpc/_cython/_cygrpc/records.pyx.pxi":157
 * cdef class SslPemKeyCertPair:
 * 
 *   def __cinit__(self, bytes private_key, bytes certificate_chain):             # <<<<<<<<<<<<<<
 *     self.private_key = private_key
 *     self.certificate_chain = certificate_chain
 */

/* Python wrapper */
static int __pyx_pw_4grpc_7_cython_6cygrpc_17SslPemKeyCertPair_1__cinit__(PyObject *__pyx_v_self, PyObject *__pyx_args, PyObject *__pyx_kwds); /*proto*/
static int __pyx_pw_4grpc_7_cython_6cygrpc_17SslPemKeyCertPair_1__cinit__(PyObject *__pyx_v_self, PyObject *__pyx_args, PyObject *__pyx_kwds) {
  PyObject *__pyx_v_private_key = 0;
  PyObject *__pyx_v_certificate_chain = 0;
  int __pyx_r;
  __Pyx_RefNannyDeclarations
  __Pyx_RefNannySetupContext("__cinit__ (wrapper)", 0);
  {
    static PyObject **__pyx_pyargnames[] = {&__pyx_n_s_private_key,&__pyx_n_s_certificate_chain,0};
    PyObject* values[2] = {0,0};
    if (unlikely(__pyx_kwds)) {
      Py_ssize_t kw_args;
      const Py_ssize_t pos_args = PyTuple_GET_SIZE(__pyx_args);
      switch (pos_args) {
        case  2: values[1] = PyTuple_GET_ITEM(__pyx_args, 1);
        CYTHON_FALLTHROUGH;
        case  1: values[0] = PyTuple_GET_ITEM(__pyx_args, 0);
        CYTHON_FALLTHROUGH;
        case  0: break;
        default: goto __pyx_L5_argtuple_error;
      }
      kw_args = PyDict_Size(__pyx_kwds);
      switch (pos_args) {
        case  0:
        if (likely((values[0] = __Pyx_PyDict_GetItemStr(__pyx_kwds, __pyx_n_s_private_key)) != 0)) kw_args--;
        else goto __pyx_L5_argtuple_error;
        CYTHON_FALLTHROUGH;
        case  1:
        if (likely((values[1] = __Pyx_PyDict_GetItemStr(__pyx_kwds, __pyx_n_s_certificate_chain)) != 0)) kw_args--;
        else {
          __Pyx_RaiseArgtupleInvalid("__cinit__", 1, 2, 2, 1); __PYX_ERR(16, 157, __pyx_L3_error)
        }
      }
      if (unlikely(kw_args > 0)) {
        if (unlikely(__Pyx_ParseOptionalKeywords(__pyx_kwds, __pyx_pyargnames, 0, values, pos_args, "__cinit__") < 0)) __PYX_ERR(16, 157, __pyx_L3_error)
      }
    } else if (PyTuple_GET_SIZE(__pyx_args) != 2) {
      goto __pyx_L5_argtuple_error;
    } else {
      values[0] = PyTuple_GET_ITEM(__pyx_args, 0);
      values[1] = PyTuple_GET_ITEM(__pyx_args, 1);
    }
    __pyx_v_private_key = ((PyObject*)values[0]);
    __pyx_v_certificate_chain = ((PyObject*)values[1]);
  }
  goto __pyx_L4_argument_unpacking_done;
  __pyx_L5_argtuple_error:;
  __Pyx_RaiseArgtupleInvalid("__cinit__", 1, 2, 2, PyTuple_GET_SIZE(__pyx_args)); __PYX_ERR(16, 157, __pyx_L3_error)
  __pyx_L3_error:;
  __Pyx_AddTraceback("grpc._cython.cygrpc.SslPemKeyCertPair.__cinit__", __pyx_clineno, __pyx_lineno, __pyx_filename);
  __Pyx_RefNannyFinishContext();
  return -1;
  __pyx_L4_argument_unpacking_done:;
  if (unlikely(!__Pyx_ArgTypeTest(((PyObject *)__pyx_v_private_key), (&PyBytes_Type), 1, "private_key", 1))) __PYX_ERR(16, 157, __pyx_L1_error)
  if (unlikely(!__Pyx_ArgTypeTest(((PyObject *)__pyx_v_certificate_chain), (&PyBytes_Type), 1, "certificate_chain", 1))) __PYX_ERR(16, 157, __pyx_L1_error)
  __pyx_r = __pyx_pf_4grpc_7_cython_6cygrpc_17SslPemKeyCertPair___cinit__(((struct __pyx_obj_4grpc_7_cython_6cygrpc_SslPemKeyCertPair *)__pyx_v_self), __pyx_v_private_key, __pyx_v_certificate_chain);

  /* function exit code */
  goto __pyx_L0;
  __pyx_L1_error:;
  __pyx_r = -1;
  __pyx_L0:;
  __Pyx_RefNannyFinishContext();
  return __pyx_r;
}

static int __pyx_pf_4grpc_7_cython_6cygrpc_17SslPemKeyCertPair___cinit__(struct __pyx_obj_4grpc_7_cython_6cygrpc_SslPemKeyCertPair *__pyx_v_self, PyObject *__pyx_v_private_key, PyObject *__pyx_v_certificate_chain) {
  int __pyx_r;
  __Pyx_RefNannyDeclarations
  char const *__pyx_t_1;
  char const *__pyx_t_2;
  __Pyx_RefNannySetupContext("__cinit__", 0);

  /* "src/python/grpcio/grpc/_cython/_cygrpc/records.pyx.pxi":158
 * 
 *   def __cinit__(self, bytes private_key, bytes certificate_chain):
 *     self.private_key = private_key             # <<<<<<<<<<<<<<
 *     self.certificate_chain = certificate_chain
 *     self.c_pair.private_key = self.private_key
 */
  __Pyx_INCREF(__pyx_v_private_key);
  __Pyx_GIVEREF(__pyx_v_private_key);
  __Pyx_GOTREF(__pyx_v_self->private_key);
  __Pyx_DECREF(__pyx_v_self->private_key);
  __pyx_v_self->private_key = __pyx_v_private_key;

  /* "src/python/grpcio/grpc/_cython/_cygrpc/records.pyx.pxi":159
 *   def __cinit__(self, bytes private_key, bytes certificate_chain):
 *     self.private_key = private_key
 *     self.certificate_chain = certificate_chain             # <<<<<<<<<<<<<<
 *     self.c_pair.private_key = self.private_key
 *     self.c_pair.certificate_chain = self.certificate_chain
 */
  __Pyx_INCREF(__pyx_v_certificate_chain);
  __Pyx_GIVEREF(__pyx_v_certificate_chain);
  __Pyx_GOTREF(__pyx_v_self->certificate_chain);
  __Pyx_DECREF(__pyx_v_self->certificate_chain);
  __pyx_v_self->certificate_chain = __pyx_v_certificate_chain;

  /* "src/python/grpcio/grpc/_cython/_cygrpc/records.pyx.pxi":160
 *     self.private_key = private_key
 *     self.certificate_chain = certificate_chain
 *     self.c_pair.private_key = self.private_key             # <<<<<<<<<<<<<<
 *     self.c_pair.certificate_chain = self.certificate_chain
 * 
 */
  __pyx_t_1 = __Pyx_PyObject_AsString(__pyx_v_self->private_key); if (unlikely((!__pyx_t_1) && PyErr_Occurred())) __PYX_ERR(16, 160, __pyx_L1_error)
  __pyx_v_self->c_pair.private_key = __pyx_t_1;

  /* "src/python/grpcio/grpc/_cython/_cygrpc/records.pyx.pxi":161
 *     self.certificate_chain = certificate_chain
 *     self.c_pair.private_key = self.private_key
 *     self.c_pair.certificate_chain = self.certificate_chain             # <<<<<<<<<<<<<<
 * 
 * 
 */
  __pyx_t_2 = __Pyx_PyObject_AsString(__pyx_v_self->certificate_chain); if (unlikely((!__pyx_t_2) && PyErr_Occurred())) __PYX_ERR(16, 161, __pyx_L1_error)
  __pyx_v_self->c_pair.cert_chain = __pyx_t_2;

  /* "src/python/grpcio/grpc/_cython/_cygrpc/records.pyx.pxi":157
 * cdef class SslPemKeyCertPair:
 * 
 *   def __cinit__(self, bytes private_key, bytes certificate_chain):             # <<<<<<<<<<<<<<
 *     self.private_key = private_key
 *     self.certificate_chain = certificate_chain
 */

  /* function exit code */
  __pyx_r = 0;
  goto __pyx_L0;
  __pyx_L1_error:;
  __Pyx_AddTraceback("grpc._cython.cygrpc.SslPemKeyCertPair.__cinit__", __pyx_clineno, __pyx_lineno, __pyx_filename);
  __pyx_r = -1;
  __pyx_L0:;
  __Pyx_RefNannyFinishContext();
  return __pyx_r;
}

/* "src/python/grpcio/grpc/_cython/_cygrpc/records.pxd.pxi":29
 * 
 *   cdef grpc_ssl_pem_key_cert_pair c_pair
 *   cdef readonly object private_key, certificate_chain             # <<<<<<<<<<<<<<
 * 
 * 
 */

/* Python wrapper */
static PyObject *__pyx_pw_4grpc_7_cython_6cygrpc_17SslPemKeyCertPair_11private_key_1__get__(PyObject *__pyx_v_self); /*proto*/
static PyObject *__pyx_pw_4grpc_7_cython_6cygrpc_17SslPemKeyCertPair_11private_key_1__get__(PyObject *__pyx_v_self) {
  PyObject *__pyx_r = 0;
  __Pyx_RefNannyDeclarations
  __Pyx_RefNannySetupContext("__get__ (wrapper)", 0);
  __pyx_r = __pyx_pf_4grpc_7_cython_6cygrpc_17SslPemKeyCertPair_11private_key___get__(((struct __pyx_obj_4grpc_7_cython_6cygrpc_SslPemKeyCertPair *)__pyx_v_self));

  /* function exit code */
  __Pyx_RefNannyFinishContext();
  return __pyx_r;
}

static PyObject *__pyx_pf_4grpc_7_cython_6cygrpc_17SslPemKeyCertPair_11private_key___get__(struct __pyx_obj_4grpc_7_cython_6cygrpc_SslPemKeyCertPair *__pyx_v_self) {
  PyObject *__pyx_r = NULL;
  __Pyx_RefNannyDeclarations
  __Pyx_RefNannySetupContext("__get__", 0);
  __Pyx_XDECREF(__pyx_r);
  __Pyx_INCREF(__pyx_v_self->private_key);
  __pyx_r = __pyx_v_self->private_key;
  goto __pyx_L0;

  /* function exit code */
  __pyx_L0:;
  __Pyx_XGIVEREF(__pyx_r);
  __Pyx_RefNannyFinishContext();
  return __pyx_r;
}

/* Python wrapper */
static PyObject *__pyx_pw_4grpc_7_cython_6cygrpc_17SslPemKeyCertPair_17certificate_chain_1__get__(PyObject *__pyx_v_self); /*proto*/
static PyObject *__pyx_pw_4grpc_7_cython_6cygrpc_17SslPemKeyCertPair_17certificate_chain_1__get__(PyObject *__pyx_v_self) {
  PyObject *__pyx_r = 0;
  __Pyx_RefNannyDeclarations
  __Pyx_RefNannySetupContext("__get__ (wrapper)", 0);
  __pyx_r = __pyx_pf_4grpc_7_cython_6cygrpc_17SslPemKeyCertPair_17certificate_chain___get__(((struct __pyx_obj_4grpc_7_cython_6cygrpc_SslPemKeyCertPair *)__pyx_v_self));

  /* function exit code */
  __Pyx_RefNannyFinishContext();
  return __pyx_r;
}

static PyObject *__pyx_pf_4grpc_7_cython_6cygrpc_17SslPemKeyCertPair_17certificate_chain___get__(struct __pyx_obj_4grpc_7_cython_6cygrpc_SslPemKeyCertPair *__pyx_v_self) {
  PyObject *__pyx_r = NULL;
  __Pyx_RefNannyDeclarations
  __Pyx_RefNannySetupContext("__get__", 0);
  __Pyx_XDECREF(__pyx_r);
  __Pyx_INCREF(__pyx_v_self->certificate_chain);
  __pyx_r = __pyx_v_self->certificate_chain;
  goto __pyx_L0;

  /* function exit code */
  __pyx_L0:;
  __Pyx_XGIVEREF(__pyx_r);
  __Pyx_RefNannyFinishContext();
  return __pyx_r;
}

/* "(tree fragment)":1
 * def __reduce_cython__(self):             # <<<<<<<<<<<<<<
 *     raise TypeError("no default __reduce__ due to non-trivial __cinit__")
 * def __setstate_cython__(self, __pyx_state):
 */

/* Python wrapper */
static PyObject *__pyx_pw_4grpc_7_cython_6cygrpc_17SslPemKeyCertPair_3__reduce_cython__(PyObject *__pyx_v_self, CYTHON_UNUSED PyObject *unused); /*proto*/
static PyObject *__pyx_pw_4grpc_7_cython_6cygrpc_17SslPemKeyCertPair_3__reduce_cython__(PyObject *__pyx_v_self, CYTHON_UNUSED PyObject *unused) {
  PyObject *__pyx_r = 0;
  __Pyx_RefNannyDeclarations
  __Pyx_RefNannySetupContext("__reduce_cython__ (wrapper)", 0);
  __pyx_r = __pyx_pf_4grpc_7_cython_6cygrpc_17SslPemKeyCertPair_2__reduce_cython__(((struct __pyx_obj_4grpc_7_cython_6cygrpc_SslPemKeyCertPair *)__pyx_v_self));

  /* function exit code */
  __Pyx_RefNannyFinishContext();
  return __pyx_r;
}

static PyObject *__pyx_pf_4grpc_7_cython_6cygrpc_17SslPemKeyCertPair_2__reduce_cython__(CYTHON_UNUSED struct __pyx_obj_4grpc_7_cython_6cygrpc_SslPemKeyCertPair *__pyx_v_self) {
  PyObject *__pyx_r = NULL;
  __Pyx_RefNannyDeclarations
  PyObject *__pyx_t_1 = NULL;
  __Pyx_RefNannySetupContext("__reduce_cython__", 0);

  /* "(tree fragment)":2
 * def __reduce_cython__(self):
 *     raise TypeError("no default __reduce__ due to non-trivial __cinit__")             # <<<<<<<<<<<<<<
 * def __setstate_cython__(self, __pyx_state):
 *     raise TypeError("no default __reduce__ due to non-trivial __cinit__")
 */
  __pyx_t_1 = __Pyx_PyObject_Call(__pyx_builtin_TypeError, __pyx_tuple__104, NULL); if (unlikely(!__pyx_t_1)) __PYX_ERR(7, 2, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_1);
  __Pyx_Raise(__pyx_t_1, 0, 0, 0);
  __Pyx_DECREF(__pyx_t_1); __pyx_t_1 = 0;
  __PYX_ERR(7, 2, __pyx_L1_error)

  /* "(tree fragment)":1
 * def __reduce_cython__(self):             # <<<<<<<<<<<<<<
 *     raise TypeError("no default __reduce__ due to non-trivial __cinit__")
 * def __setstate_cython__(self, __pyx_state):
 */

  /* function exit code */
  __pyx_L1_error:;
  __Pyx_XDECREF(__pyx_t_1);
  __Pyx_AddTraceback("grpc._cython.cygrpc.SslPemKeyCertPair.__reduce_cython__", __pyx_clineno, __pyx_lineno, __pyx_filename);
  __pyx_r = NULL;
  __Pyx_XGIVEREF(__pyx_r);
  __Pyx_RefNannyFinishContext();
  return __pyx_r;
}

/* "(tree fragment)":3
 * def __reduce_cython__(self):
 *     raise TypeError("no default __reduce__ due to non-trivial __cinit__")
 * def __setstate_cython__(self, __pyx_state):             # <<<<<<<<<<<<<<
 *     raise TypeError("no default __reduce__ due to non-trivial __cinit__")
 */

/* Python wrapper */
static PyObject *__pyx_pw_4grpc_7_cython_6cygrpc_17SslPemKeyCertPair_5__setstate_cython__(PyObject *__pyx_v_self, PyObject *__pyx_v___pyx_state); /*proto*/
static PyObject *__pyx_pw_4grpc_7_cython_6cygrpc_17SslPemKeyCertPair_5__setstate_cython__(PyObject *__pyx_v_self, PyObject *__pyx_v___pyx_state) {
  PyObject *__pyx_r = 0;
  __Pyx_RefNannyDeclarations
  __Pyx_RefNannySetupContext("__setstate_cython__ (wrapper)", 0);
  __pyx_r = __pyx_pf_4grpc_7_cython_6cygrpc_17SslPemKeyCertPair_4__setstate_cython__(((struct __pyx_obj_4grpc_7_cython_6cygrpc_SslPemKeyCertPair *)__pyx_v_self), ((PyObject *)__pyx_v___pyx_state));

  /* function exit code */
  __Pyx_RefNannyFinishContext();
  return __pyx_r;
}

static PyObject *__pyx_pf_4grpc_7_cython_6cygrpc_17SslPemKeyCertPair_4__setstate_cython__(CYTHON_UNUSED struct __pyx_obj_4grpc_7_cython_6cygrpc_SslPemKeyCertPair *__pyx_v_self, CYTHON_UNUSED PyObject *__pyx_v___pyx_state) {
  PyObject *__pyx_r = NULL;
  __Pyx_RefNannyDeclarations
  PyObject *__pyx_t_1 = NULL;
  __Pyx_RefNannySetupContext("__setstate_cython__", 0);

  /* "(tree fragment)":4
 *     raise TypeError("no default __reduce__ due to non-trivial __cinit__")
 * def __setstate_cython__(self, __pyx_state):
 *     raise TypeError("no default __reduce__ due to non-trivial __cinit__")             # <<<<<<<<<<<<<<
 */
  __pyx_t_1 = __Pyx_PyObject_Call(__pyx_builtin_TypeError, __pyx_tuple__105, NULL); if (unlikely(!__pyx_t_1)) __PYX_ERR(7, 4, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_1);
  __Pyx_Raise(__pyx_t_1, 0, 0, 0);
  __Pyx_DECREF(__pyx_t_1); __pyx_t_1 = 0;
  __PYX_ERR(7, 4, __pyx_L1_error)

  /* "(tree fragment)":3
 * def __reduce_cython__(self):
 *     raise TypeError("no default __reduce__ due to non-trivial __cinit__")
 * def __setstate_cython__(self, __pyx_state):             # <<<<<<<<<<<<<<
 *     raise TypeError("no default __reduce__ due to non-trivial __cinit__")
 */

  /* function exit code */
  __pyx_L1_error:;
  __Pyx_XDECREF(__pyx_t_1);
  __Pyx_AddTraceback("grpc._cython.cygrpc.SslPemKeyCertPair.__setstate_cython__", __pyx_clineno, __pyx_lineno, __pyx_filename);
  __pyx_r = NULL;
  __Pyx_XGIVEREF(__pyx_r);
  __Pyx_RefNannyFinishContext();
  return __pyx_r;
}

/* "src/python/grpcio/grpc/_cython/_cygrpc/records.pyx.pxi":166
 * cdef class CompressionOptions:
 * 
 *   def __cinit__(self):             # <<<<<<<<<<<<<<
 *     with nogil:
 *       grpc_compression_options_init(&self.c_options)
 */

/* Python wrapper */
static int __pyx_pw_4grpc_7_cython_6cygrpc_18CompressionOptions_1__cinit__(PyObject *__pyx_v_self, PyObject *__pyx_args, PyObject *__pyx_kwds); /*proto*/
static int __pyx_pw_4grpc_7_cython_6cygrpc_18CompressionOptions_1__cinit__(PyObject *__pyx_v_self, PyObject *__pyx_args, PyObject *__pyx_kwds) {
  int __pyx_r;
  __Pyx_RefNannyDeclarations
  __Pyx_RefNannySetupContext("__cinit__ (wrapper)", 0);
  if (unlikely(PyTuple_GET_SIZE(__pyx_args) > 0)) {
    __Pyx_RaiseArgtupleInvalid("__cinit__", 1, 0, 0, PyTuple_GET_SIZE(__pyx_args)); return -1;}
  if (unlikely(__pyx_kwds) && unlikely(PyDict_Size(__pyx_kwds) > 0) && unlikely(!__Pyx_CheckKeywordStrings(__pyx_kwds, "__cinit__", 0))) return -1;
  __pyx_r = __pyx_pf_4grpc_7_cython_6cygrpc_18CompressionOptions___cinit__(((struct __pyx_obj_4grpc_7_cython_6cygrpc_CompressionOptions *)__pyx_v_self));

  /* function exit code */
  __Pyx_RefNannyFinishContext();
  return __pyx_r;
}

static int __pyx_pf_4grpc_7_cython_6cygrpc_18CompressionOptions___cinit__(struct __pyx_obj_4grpc_7_cython_6cygrpc_CompressionOptions *__pyx_v_self) {
  int __pyx_r;
  __Pyx_RefNannyDeclarations
  __Pyx_RefNannySetupContext("__cinit__", 0);

  /* "src/python/grpcio/grpc/_cython/_cygrpc/records.pyx.pxi":167
 * 
 *   def __cinit__(self):
 *     with nogil:             # <<<<<<<<<<<<<<
 *       grpc_compression_options_init(&self.c_options)
 * 
 */
  {
      #ifdef WITH_THREAD
      PyThreadState *_save;
      Py_UNBLOCK_THREADS
      __Pyx_FastGIL_Remember();
      #endif
      /*try:*/ {

        /* "src/python/grpcio/grpc/_cython/_cygrpc/records.pyx.pxi":168
 *   def __cinit__(self):
 *     with nogil:
 *       grpc_compression_options_init(&self.c_options)             # <<<<<<<<<<<<<<
 * 
 *   def enable_algorithm(self, grpc_compression_algorithm algorithm):
 */
        grpc_compression_options_init((&__pyx_v_self->c_options));
      }

      /* "src/python/grpcio/grpc/_cython/_cygrpc/records.pyx.pxi":167
 * 
 *   def __cinit__(self):
 *     with nogil:             # <<<<<<<<<<<<<<
 *       grpc_compression_options_init(&self.c_options)
 * 
 */
      /*finally:*/ {
        /*normal exit:*/{
          #ifdef WITH_THREAD
          __Pyx_FastGIL_Forget();
          Py_BLOCK_THREADS
          #endif
          goto __pyx_L5;
        }
        __pyx_L5:;
      }
  }

  /* "src/python/grpcio/grpc/_cython/_cygrpc/records.pyx.pxi":166
 * cdef class CompressionOptions:
 * 
 *   def __cinit__(self):             # <<<<<<<<<<<<<<
 *     with nogil:
 *       grpc_compression_options_init(&self.c_options)
 */

  /* function exit code */
  __pyx_r = 0;
  __Pyx_RefNannyFinishContext();
  return __pyx_r;
}

/* "src/python/grpcio/grpc/_cython/_cygrpc/records.pyx.pxi":170
 *       grpc_compression_options_init(&self.c_options)
 * 
 *   def enable_algorithm(self, grpc_compression_algorithm algorithm):             # <<<<<<<<<<<<<<
 *     with nogil:
 *       grpc_compression_options_enable_algorithm(&self.c_options, algorithm)
 */

/* Python wrapper */
static PyObject *__pyx_pw_4grpc_7_cython_6cygrpc_18CompressionOptions_3enable_algorithm(PyObject *__pyx_v_self, PyObject *__pyx_arg_algorithm); /*proto*/
static PyObject *__pyx_pw_4grpc_7_cython_6cygrpc_18CompressionOptions_3enable_algorithm(PyObject *__pyx_v_self, PyObject *__pyx_arg_algorithm) {
  grpc_compression_algorithm __pyx_v_algorithm;
  PyObject *__pyx_r = 0;
  __Pyx_RefNannyDeclarations
  __Pyx_RefNannySetupContext("enable_algorithm (wrapper)", 0);
  assert(__pyx_arg_algorithm); {
    __pyx_v_algorithm = ((grpc_compression_algorithm)__Pyx_PyInt_As_grpc_compression_algorithm(__pyx_arg_algorithm)); if (unlikely(PyErr_Occurred())) __PYX_ERR(16, 170, __pyx_L3_error)
  }
  goto __pyx_L4_argument_unpacking_done;
  __pyx_L3_error:;
  __Pyx_AddTraceback("grpc._cython.cygrpc.CompressionOptions.enable_algorithm", __pyx_clineno, __pyx_lineno, __pyx_filename);
  __Pyx_RefNannyFinishContext();
  return NULL;
  __pyx_L4_argument_unpacking_done:;
  __pyx_r = __pyx_pf_4grpc_7_cython_6cygrpc_18CompressionOptions_2enable_algorithm(((struct __pyx_obj_4grpc_7_cython_6cygrpc_CompressionOptions *)__pyx_v_self), ((grpc_compression_algorithm)__pyx_v_algorithm));

  /* function exit code */
  __Pyx_RefNannyFinishContext();
  return __pyx_r;
}

static PyObject *__pyx_pf_4grpc_7_cython_6cygrpc_18CompressionOptions_2enable_algorithm(struct __pyx_obj_4grpc_7_cython_6cygrpc_CompressionOptions *__pyx_v_self, grpc_compression_algorithm __pyx_v_algorithm) {
  PyObject *__pyx_r = NULL;
  __Pyx_RefNannyDeclarations
  __Pyx_RefNannySetupContext("enable_algorithm", 0);

  /* "src/python/grpcio/grpc/_cython/_cygrpc/records.pyx.pxi":171
 * 
 *   def enable_algorithm(self, grpc_compression_algorithm algorithm):
 *     with nogil:             # <<<<<<<<<<<<<<
 *       grpc_compression_options_enable_algorithm(&self.c_options, algorithm)
 * 
 */
  {
      #ifdef WITH_THREAD
      PyThreadState *_save;
      Py_UNBLOCK_THREADS
      __Pyx_FastGIL_Remember();
      #endif
      /*try:*/ {

        /* "src/python/grpcio/grpc/_cython/_cygrpc/records.pyx.pxi":172
 *   def enable_algorithm(self, grpc_compression_algorithm algorithm):
 *     with nogil:
 *       grpc_compression_options_enable_algorithm(&self.c_options, algorithm)             # <<<<<<<<<<<<<<
 * 
 *   def disable_algorithm(self, grpc_compression_algorithm algorithm):
 */
        grpc_compression_options_enable_algorithm((&__pyx_v_self->c_options), __pyx_v_algorithm);
      }

      /* "src/python/grpcio/grpc/_cython/_cygrpc/records.pyx.pxi":171
 * 
 *   def enable_algorithm(self, grpc_compression_algorithm algorithm):
 *     with nogil:             # <<<<<<<<<<<<<<
 *       grpc_compression_options_enable_algorithm(&self.c_options, algorithm)
 * 
 */
      /*finally:*/ {
        /*normal exit:*/{
          #ifdef WITH_THREAD
          __Pyx_FastGIL_Forget();
          Py_BLOCK_THREADS
          #endif
          goto __pyx_L5;
        }
        __pyx_L5:;
      }
  }

  /* "src/python/grpcio/grpc/_cython/_cygrpc/records.pyx.pxi":170
 *       grpc_compression_options_init(&self.c_options)
 * 
 *   def enable_algorithm(self, grpc_compression_algorithm algorithm):             # <<<<<<<<<<<<<<
 *     with nogil:
 *       grpc_compression_options_enable_algorithm(&self.c_options, algorithm)
 */

  /* function exit code */
  __pyx_r = Py_None; __Pyx_INCREF(Py_None);
  __Pyx_XGIVEREF(__pyx_r);
  __Pyx_RefNannyFinishContext();
  return __pyx_r;
}

/* "src/python/grpcio/grpc/_cython/_cygrpc/records.pyx.pxi":174
 *       grpc_compression_options_enable_algorithm(&self.c_options, algorithm)
 * 
 *   def disable_algorithm(self, grpc_compression_algorithm algorithm):             # <<<<<<<<<<<<<<
 *     with nogil:
 *       grpc_compression_options_disable_algorithm(&self.c_options, algorithm)
 */

/* Python wrapper */
static PyObject *__pyx_pw_4grpc_7_cython_6cygrpc_18CompressionOptions_5disable_algorithm(PyObject *__pyx_v_self, PyObject *__pyx_arg_algorithm); /*proto*/
static PyObject *__pyx_pw_4grpc_7_cython_6cygrpc_18CompressionOptions_5disable_algorithm(PyObject *__pyx_v_self, PyObject *__pyx_arg_algorithm) {
  grpc_compression_algorithm __pyx_v_algorithm;
  PyObject *__pyx_r = 0;
  __Pyx_RefNannyDeclarations
  __Pyx_RefNannySetupContext("disable_algorithm (wrapper)", 0);
  assert(__pyx_arg_algorithm); {
    __pyx_v_algorithm = ((grpc_compression_algorithm)__Pyx_PyInt_As_grpc_compression_algorithm(__pyx_arg_algorithm)); if (unlikely(PyErr_Occurred())) __PYX_ERR(16, 174, __pyx_L3_error)
  }
  goto __pyx_L4_argument_unpacking_done;
  __pyx_L3_error:;
  __Pyx_AddTraceback("grpc._cython.cygrpc.CompressionOptions.disable_algorithm", __pyx_clineno, __pyx_lineno, __pyx_filename);
  __Pyx_RefNannyFinishContext();
  return NULL;
  __pyx_L4_argument_unpacking_done:;
  __pyx_r = __pyx_pf_4grpc_7_cython_6cygrpc_18CompressionOptions_4disable_algorithm(((struct __pyx_obj_4grpc_7_cython_6cygrpc_CompressionOptions *)__pyx_v_self), ((grpc_compression_algorithm)__pyx_v_algorithm));

  /* function exit code */
  __Pyx_RefNannyFinishContext();
  return __pyx_r;
}

static PyObject *__pyx_pf_4grpc_7_cython_6cygrpc_18CompressionOptions_4disable_algorithm(struct __pyx_obj_4grpc_7_cython_6cygrpc_CompressionOptions *__pyx_v_self, grpc_compression_algorithm __pyx_v_algorithm) {
  PyObject *__pyx_r = NULL;
  __Pyx_RefNannyDeclarations
  __Pyx_RefNannySetupContext("disable_algorithm", 0);

  /* "src/python/grpcio/grpc/_cython/_cygrpc/records.pyx.pxi":175
 * 
 *   def disable_algorithm(self, grpc_compression_algorithm algorithm):
 *     with nogil:             # <<<<<<<<<<<<<<
 *       grpc_compression_options_disable_algorithm(&self.c_options, algorithm)
 * 
 */
  {
      #ifdef WITH_THREAD
      PyThreadState *_save;
      Py_UNBLOCK_THREADS
      __Pyx_FastGIL_Remember();
      #endif
      /*try:*/ {

        /* "src/python/grpcio/grpc/_cython/_cygrpc/records.pyx.pxi":176
 *   def disable_algorithm(self, grpc_compression_algorithm algorithm):
 *     with nogil:
 *       grpc_compression_options_disable_algorithm(&self.c_options, algorithm)             # <<<<<<<<<<<<<<
 * 
 *   def is_algorithm_enabled(self, grpc_compression_algorithm algorithm):
 */
        grpc_compression_options_disable_algorithm((&__pyx_v_self->c_options), __pyx_v_algorithm);
      }

      /* "src/python/grpcio/grpc/_cython/_cygrpc/records.pyx.pxi":175
 * 
 *   def disable_algorithm(self, grpc_compression_algorithm algorithm):
 *     with nogil:             # <<<<<<<<<<<<<<
 *       grpc_compression_options_disable_algorithm(&self.c_options, algorithm)
 * 
 */
      /*finally:*/ {
        /*normal exit:*/{
          #ifdef WITH_THREAD
          __Pyx_FastGIL_Forget();
          Py_BLOCK_THREADS
          #endif
          goto __pyx_L5;
        }
        __pyx_L5:;
      }
  }

  /* "src/python/grpcio/grpc/_cython/_cygrpc/records.pyx.pxi":174
 *       grpc_compression_options_enable_algorithm(&self.c_options, algorithm)
 * 
 *   def disable_algorithm(self, grpc_compression_algorithm algorithm):             # <<<<<<<<<<<<<<
 *     with nogil:
 *       grpc_compression_options_disable_algorithm(&self.c_options, algorithm)
 */

  /* function exit code */
  __pyx_r = Py_None; __Pyx_INCREF(Py_None);
  __Pyx_XGIVEREF(__pyx_r);
  __Pyx_RefNannyFinishContext();
  return __pyx_r;
}

/* "src/python/grpcio/grpc/_cython/_cygrpc/records.pyx.pxi":178
 *       grpc_compression_options_disable_algorithm(&self.c_options, algorithm)
 * 
 *   def is_algorithm_enabled(self, grpc_compression_algorithm algorithm):             # <<<<<<<<<<<<<<
 *     cdef int result
 *     with nogil:
 */

/* Python wrapper */
static PyObject *__pyx_pw_4grpc_7_cython_6cygrpc_18CompressionOptions_7is_algorithm_enabled(PyObject *__pyx_v_self, PyObject *__pyx_arg_algorithm); /*proto*/
static PyObject *__pyx_pw_4grpc_7_cython_6cygrpc_18CompressionOptions_7is_algorithm_enabled(PyObject *__pyx_v_self, PyObject *__pyx_arg_algorithm) {
  grpc_compression_algorithm __pyx_v_algorithm;
  PyObject *__pyx_r = 0;
  __Pyx_RefNannyDeclarations
  __Pyx_RefNannySetupContext("is_algorithm_enabled (wrapper)", 0);
  assert(__pyx_arg_algorithm); {
    __pyx_v_algorithm = ((grpc_compression_algorithm)__Pyx_PyInt_As_grpc_compression_algorithm(__pyx_arg_algorithm)); if (unlikely(PyErr_Occurred())) __PYX_ERR(16, 178, __pyx_L3_error)
  }
  goto __pyx_L4_argument_unpacking_done;
  __pyx_L3_error:;
  __Pyx_AddTraceback("grpc._cython.cygrpc.CompressionOptions.is_algorithm_enabled", __pyx_clineno, __pyx_lineno, __pyx_filename);
  __Pyx_RefNannyFinishContext();
  return NULL;
  __pyx_L4_argument_unpacking_done:;
  __pyx_r = __pyx_pf_4grpc_7_cython_6cygrpc_18CompressionOptions_6is_algorithm_enabled(((struct __pyx_obj_4grpc_7_cython_6cygrpc_CompressionOptions *)__pyx_v_self), ((grpc_compression_algorithm)__pyx_v_algorithm));

  /* function exit code */
  __Pyx_RefNannyFinishContext();
  return __pyx_r;
}

static PyObject *__pyx_pf_4grpc_7_cython_6cygrpc_18CompressionOptions_6is_algorithm_enabled(struct __pyx_obj_4grpc_7_cython_6cygrpc_CompressionOptions *__pyx_v_self, grpc_compression_algorithm __pyx_v_algorithm) {
  int __pyx_v_result;
  PyObject *__pyx_r = NULL;
  __Pyx_RefNannyDeclarations
  PyObject *__pyx_t_1 = NULL;
  __Pyx_RefNannySetupContext("is_algorithm_enabled", 0);

  /* "src/python/grpcio/grpc/_cython/_cygrpc/records.pyx.pxi":180
 *   def is_algorithm_enabled(self, grpc_compression_algorithm algorithm):
 *     cdef int result
 *     with nogil:             # <<<<<<<<<<<<<<
 *       result = grpc_compression_options_is_algorithm_enabled(
 *           &self.c_options, algorithm)
 */
  {
      #ifdef WITH_THREAD
      PyThreadState *_save;
      Py_UNBLOCK_THREADS
      __Pyx_FastGIL_Remember();
      #endif
      /*try:*/ {

        /* "src/python/grpcio/grpc/_cython/_cygrpc/records.pyx.pxi":181
 *     cdef int result
 *     with nogil:
 *       result = grpc_compression_options_is_algorithm_enabled(             # <<<<<<<<<<<<<<
 *           &self.c_options, algorithm)
 *     return result
 */
        __pyx_v_result = grpc_compression_options_is_algorithm_enabled((&__pyx_v_self->c_options), __pyx_v_algorithm);
      }

      /* "src/python/grpcio/grpc/_cython/_cygrpc/records.pyx.pxi":180
 *   def is_algorithm_enabled(self, grpc_compression_algorithm algorithm):
 *     cdef int result
 *     with nogil:             # <<<<<<<<<<<<<<
 *       result = grpc_compression_options_is_algorithm_enabled(
 *           &self.c_options, algorithm)
 */
      /*finally:*/ {
        /*normal exit:*/{
          #ifdef WITH_THREAD
          __Pyx_FastGIL_Forget();
          Py_BLOCK_THREADS
          #endif
          goto __pyx_L5;
        }
        __pyx_L5:;
      }
  }

  /* "src/python/grpcio/grpc/_cython/_cygrpc/records.pyx.pxi":183
 *       result = grpc_compression_options_is_algorithm_enabled(
 *           &self.c_options, algorithm)
 *     return result             # <<<<<<<<<<<<<<
 * 
 *   def to_channel_arg(self):
 */
  __Pyx_XDECREF(__pyx_r);
  __pyx_t_1 = __Pyx_PyInt_From_int(__pyx_v_result); if (unlikely(!__pyx_t_1)) __PYX_ERR(16, 183, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_1);
  __pyx_r = __pyx_t_1;
  __pyx_t_1 = 0;
  goto __pyx_L0;

  /* "src/python/grpcio/grpc/_cython/_cygrpc/records.pyx.pxi":178
 *       grpc_compression_options_disable_algorithm(&self.c_options, algorithm)
 * 
 *   def is_algorithm_enabled(self, grpc_compression_algorithm algorithm):             # <<<<<<<<<<<<<<
 *     cdef int result
 *     with nogil:
 */

  /* function exit code */
  __pyx_L1_error:;
  __Pyx_XDECREF(__pyx_t_1);
  __Pyx_AddTraceback("grpc._cython.cygrpc.CompressionOptions.is_algorithm_enabled", __pyx_clineno, __pyx_lineno, __pyx_filename);
  __pyx_r = NULL;
  __pyx_L0:;
  __Pyx_XGIVEREF(__pyx_r);
  __Pyx_RefNannyFinishContext();
  return __pyx_r;
}

/* "src/python/grpcio/grpc/_cython/_cygrpc/records.pyx.pxi":185
 *     return result
 * 
 *   def to_channel_arg(self):             # <<<<<<<<<<<<<<
 *     return (
 *         GRPC_COMPRESSION_CHANNEL_ENABLED_ALGORITHMS_BITSET,
 */

/* Python wrapper */
static PyObject *__pyx_pw_4grpc_7_cython_6cygrpc_18CompressionOptions_9to_channel_arg(PyObject *__pyx_v_self, CYTHON_UNUSED PyObject *unused); /*proto*/
static PyObject *__pyx_pw_4grpc_7_cython_6cygrpc_18CompressionOptions_9to_channel_arg(PyObject *__pyx_v_self, CYTHON_UNUSED PyObject *unused) {
  PyObject *__pyx_r = 0;
  __Pyx_RefNannyDeclarations
  __Pyx_RefNannySetupContext("to_channel_arg (wrapper)", 0);
  __pyx_r = __pyx_pf_4grpc_7_cython_6cygrpc_18CompressionOptions_8to_channel_arg(((struct __pyx_obj_4grpc_7_cython_6cygrpc_CompressionOptions *)__pyx_v_self));

  /* function exit code */
  __Pyx_RefNannyFinishContext();
  return __pyx_r;
}

static PyObject *__pyx_pf_4grpc_7_cython_6cygrpc_18CompressionOptions_8to_channel_arg(struct __pyx_obj_4grpc_7_cython_6cygrpc_CompressionOptions *__pyx_v_self) {
  PyObject *__pyx_r = NULL;
  __Pyx_RefNannyDeclarations
  PyObject *__pyx_t_1 = NULL;
  PyObject *__pyx_t_2 = NULL;
  PyObject *__pyx_t_3 = NULL;
  __Pyx_RefNannySetupContext("to_channel_arg", 0);

  /* "src/python/grpcio/grpc/_cython/_cygrpc/records.pyx.pxi":186
 * 
 *   def to_channel_arg(self):
 *     return (             # <<<<<<<<<<<<<<
 *         GRPC_COMPRESSION_CHANNEL_ENABLED_ALGORITHMS_BITSET,
 *         self.c_options.enabled_algorithms_bitset,
 */
  __Pyx_XDECREF(__pyx_r);

  /* "src/python/grpcio/grpc/_cython/_cygrpc/records.pyx.pxi":187
 *   def to_channel_arg(self):
 *     return (
 *         GRPC_COMPRESSION_CHANNEL_ENABLED_ALGORITHMS_BITSET,             # <<<<<<<<<<<<<<
 *         self.c_options.enabled_algorithms_bitset,
 *     )
 */
  __pyx_t_1 = __Pyx_PyBytes_FromString(GRPC_COMPRESSION_CHANNEL_ENABLED_ALGORITHMS_BITSET); if (unlikely(!__pyx_t_1)) __PYX_ERR(16, 187, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_1);

  /* "src/python/grpcio/grpc/_cython/_cygrpc/records.pyx.pxi":188
 *     return (
 *         GRPC_COMPRESSION_CHANNEL_ENABLED_ALGORITHMS_BITSET,
 *         self.c_options.enabled_algorithms_bitset,             # <<<<<<<<<<<<<<
 *     )
 * 
 */
  __pyx_t_2 = __Pyx_PyInt_From_unsigned_int(__pyx_v_self->c_options.enabled_algorithms_bitset); if (unlikely(!__pyx_t_2)) __PYX_ERR(16, 188, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_2);

  /* "src/python/grpcio/grpc/_cython/_cygrpc/records.pyx.pxi":187
 *   def to_channel_arg(self):
 *     return (
 *         GRPC_COMPRESSION_CHANNEL_ENABLED_ALGORITHMS_BITSET,             # <<<<<<<<<<<<<<
 *         self.c_options.enabled_algorithms_bitset,
 *     )
 */
  __pyx_t_3 = PyTuple_New(2); if (unlikely(!__pyx_t_3)) __PYX_ERR(16, 187, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_3);
  __Pyx_GIVEREF(__pyx_t_1);
  PyTuple_SET_ITEM(__pyx_t_3, 0, __pyx_t_1);
  __Pyx_GIVEREF(__pyx_t_2);
  PyTuple_SET_ITEM(__pyx_t_3, 1, __pyx_t_2);
  __pyx_t_1 = 0;
  __pyx_t_2 = 0;
  __pyx_r = __pyx_t_3;
  __pyx_t_3 = 0;
  goto __pyx_L0;

  /* "src/python/grpcio/grpc/_cython/_cygrpc/records.pyx.pxi":185
 *     return result
 * 
 *   def to_channel_arg(self):             # <<<<<<<<<<<<<<
 *     return (
 *         GRPC_COMPRESSION_CHANNEL_ENABLED_ALGORITHMS_BITSET,
 */

  /* function exit code */
  __pyx_L1_error:;
  __Pyx_XDECREF(__pyx_t_1);
  __Pyx_XDECREF(__pyx_t_2);
  __Pyx_XDECREF(__pyx_t_3);
  __Pyx_AddTraceback("grpc._cython.cygrpc.CompressionOptions.to_channel_arg", __pyx_clineno, __pyx_lineno, __pyx_filename);
  __pyx_r = NULL;
  __pyx_L0:;
  __Pyx_XGIVEREF(__pyx_r);
  __Pyx_RefNannyFinishContext();
  return __pyx_r;
}

/* "(tree fragment)":1
 * def __reduce_cython__(self):             # <<<<<<<<<<<<<<
 *     raise TypeError("no default __reduce__ due to non-trivial __cinit__")
 * def __setstate_cython__(self, __pyx_state):
 */

/* Python wrapper */
static PyObject *__pyx_pw_4grpc_7_cython_6cygrpc_18CompressionOptions_11__reduce_cython__(PyObject *__pyx_v_self, CYTHON_UNUSED PyObject *unused); /*proto*/
static PyObject *__pyx_pw_4grpc_7_cython_6cygrpc_18CompressionOptions_11__reduce_cython__(PyObject *__pyx_v_self, CYTHON_UNUSED PyObject *unused) {
  PyObject *__pyx_r = 0;
  __Pyx_RefNannyDeclarations
  __Pyx_RefNannySetupContext("__reduce_cython__ (wrapper)", 0);
  __pyx_r = __pyx_pf_4grpc_7_cython_6cygrpc_18CompressionOptions_10__reduce_cython__(((struct __pyx_obj_4grpc_7_cython_6cygrpc_CompressionOptions *)__pyx_v_self));

  /* function exit code */
  __Pyx_RefNannyFinishContext();
  return __pyx_r;
}

static PyObject *__pyx_pf_4grpc_7_cython_6cygrpc_18CompressionOptions_10__reduce_cython__(CYTHON_UNUSED struct __pyx_obj_4grpc_7_cython_6cygrpc_CompressionOptions *__pyx_v_self) {
  PyObject *__pyx_r = NULL;
  __Pyx_RefNannyDeclarations
  PyObject *__pyx_t_1 = NULL;
  __Pyx_RefNannySetupContext("__reduce_cython__", 0);

  /* "(tree fragment)":2
 * def __reduce_cython__(self):
 *     raise TypeError("no default __reduce__ due to non-trivial __cinit__")             # <<<<<<<<<<<<<<
 * def __setstate_cython__(self, __pyx_state):
 *     raise TypeError("no default __reduce__ due to non-trivial __cinit__")
 */
  __pyx_t_1 = __Pyx_PyObject_Call(__pyx_builtin_TypeError, __pyx_tuple__106, NULL); if (unlikely(!__pyx_t_1)) __PYX_ERR(7, 2, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_1);
  __Pyx_Raise(__pyx_t_1, 0, 0, 0);
  __Pyx_DECREF(__pyx_t_1); __pyx_t_1 = 0;
  __PYX_ERR(7, 2, __pyx_L1_error)

  /* "(tree fragment)":1
 * def __reduce_cython__(self):             # <<<<<<<<<<<<<<
 *     raise TypeError("no default __reduce__ due to non-trivial __cinit__")
 * def __setstate_cython__(self, __pyx_state):
 */

  /* function exit code */
  __pyx_L1_error:;
  __Pyx_XDECREF(__pyx_t_1);
  __Pyx_AddTraceback("grpc._cython.cygrpc.CompressionOptions.__reduce_cython__", __pyx_clineno, __pyx_lineno, __pyx_filename);
  __pyx_r = NULL;
  __Pyx_XGIVEREF(__pyx_r);
  __Pyx_RefNannyFinishContext();
  return __pyx_r;
}

/* "(tree fragment)":3
 * def __reduce_cython__(self):
 *     raise TypeError("no default __reduce__ due to non-trivial __cinit__")
 * def __setstate_cython__(self, __pyx_state):             # <<<<<<<<<<<<<<
 *     raise TypeError("no default __reduce__ due to non-trivial __cinit__")
 */

/* Python wrapper */
static PyObject *__pyx_pw_4grpc_7_cython_6cygrpc_18CompressionOptions_13__setstate_cython__(PyObject *__pyx_v_self, PyObject *__pyx_v___pyx_state); /*proto*/
static PyObject *__pyx_pw_4grpc_7_cython_6cygrpc_18CompressionOptions_13__setstate_cython__(PyObject *__pyx_v_self, PyObject *__pyx_v___pyx_state) {
  PyObject *__pyx_r = 0;
  __Pyx_RefNannyDeclarations
  __Pyx_RefNannySetupContext("__setstate_cython__ (wrapper)", 0);
  __pyx_r = __pyx_pf_4grpc_7_cython_6cygrpc_18CompressionOptions_12__setstate_cython__(((struct __pyx_obj_4grpc_7_cython_6cygrpc_CompressionOptions *)__pyx_v_self), ((PyObject *)__pyx_v___pyx_state));

  /* function exit code */
  __Pyx_RefNannyFinishContext();
  return __pyx_r;
}

static PyObject *__pyx_pf_4grpc_7_cython_6cygrpc_18CompressionOptions_12__setstate_cython__(CYTHON_UNUSED struct __pyx_obj_4grpc_7_cython_6cygrpc_CompressionOptions *__pyx_v_self, CYTHON_UNUSED PyObject *__pyx_v___pyx_state) {
  PyObject *__pyx_r = NULL;
  __Pyx_RefNannyDeclarations
  PyObject *__pyx_t_1 = NULL;
  __Pyx_RefNannySetupContext("__setstate_cython__", 0);

  /* "(tree fragment)":4
 *     raise TypeError("no default __reduce__ due to non-trivial __cinit__")
 * def __setstate_cython__(self, __pyx_state):
 *     raise TypeError("no default __reduce__ due to non-trivial __cinit__")             # <<<<<<<<<<<<<<
 */
  __pyx_t_1 = __Pyx_PyObject_Call(__pyx_builtin_TypeError, __pyx_tuple__107, NULL); if (unlikely(!__pyx_t_1)) __PYX_ERR(7, 4, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_1);
  __Pyx_Raise(__pyx_t_1, 0, 0, 0);
  __Pyx_DECREF(__pyx_t_1); __pyx_t_1 = 0;
  __PYX_ERR(7, 4, __pyx_L1_error)

  /* "(tree fragment)":3
 * def __reduce_cython__(self):
 *     raise TypeError("no default __reduce__ due to non-trivial __cinit__")
 * def __setstate_cython__(self, __pyx_state):             # <<<<<<<<<<<<<<
 *     raise TypeError("no default __reduce__ due to non-trivial __cinit__")
 */

  /* function exit code */
  __pyx_L1_error:;
  __Pyx_XDECREF(__pyx_t_1);
  __Pyx_AddTraceback("grpc._cython.cygrpc.CompressionOptions.__setstate_cython__", __pyx_clineno, __pyx_lineno, __pyx_filename);
  __pyx_r = NULL;
  __Pyx_XGIVEREF(__pyx_r);
  __Pyx_RefNannyFinishContext();
  return __pyx_r;
}

/* "src/python/grpcio/grpc/_cython/_cygrpc/records.pyx.pxi":192
 * 
 * 
 * def compression_algorithm_name(grpc_compression_algorithm algorithm):             # <<<<<<<<<<<<<<
 *   cdef const char* name
 *   with nogil:
 */

/* Python wrapper */
static PyObject *__pyx_pw_4grpc_7_cython_6cygrpc_27compression_algorithm_name(PyObject *__pyx_self, PyObject *__pyx_arg_algorithm); /*proto*/
static PyMethodDef __pyx_mdef_4grpc_7_cython_6cygrpc_27compression_algorithm_name = {"compression_algorithm_name", (PyCFunction)__pyx_pw_4grpc_7_cython_6cygrpc_27compression_algorithm_name, METH_O, 0};
static PyObject *__pyx_pw_4grpc_7_cython_6cygrpc_27compression_algorithm_name(PyObject *__pyx_self, PyObject *__pyx_arg_algorithm) {
  grpc_compression_algorithm __pyx_v_algorithm;
  PyObject *__pyx_r = 0;
  __Pyx_RefNannyDeclarations
  __Pyx_RefNannySetupContext("compression_algorithm_name (wrapper)", 0);
  assert(__pyx_arg_algorithm); {
    __pyx_v_algorithm = ((grpc_compression_algorithm)__Pyx_PyInt_As_grpc_compression_algorithm(__pyx_arg_algorithm)); if (unlikely(PyErr_Occurred())) __PYX_ERR(16, 192, __pyx_L3_error)
  }
  goto __pyx_L4_argument_unpacking_done;
  __pyx_L3_error:;
  __Pyx_AddTraceback("grpc._cython.cygrpc.compression_algorithm_name", __pyx_clineno, __pyx_lineno, __pyx_filename);
  __Pyx_RefNannyFinishContext();
  return NULL;
  __pyx_L4_argument_unpacking_done:;
  __pyx_r = __pyx_pf_4grpc_7_cython_6cygrpc_26compression_algorithm_name(__pyx_self, ((grpc_compression_algorithm)__pyx_v_algorithm));

  /* function exit code */
  __Pyx_RefNannyFinishContext();
  return __pyx_r;
}

static PyObject *__pyx_pf_4grpc_7_cython_6cygrpc_26compression_algorithm_name(CYTHON_UNUSED PyObject *__pyx_self, grpc_compression_algorithm __pyx_v_algorithm) {
  char const *__pyx_v_name;
  PyObject *__pyx_r = NULL;
  __Pyx_RefNannyDeclarations
  PyObject *__pyx_t_1 = NULL;
  __Pyx_RefNannySetupContext("compression_algorithm_name", 0);

  /* "src/python/grpcio/grpc/_cython/_cygrpc/records.pyx.pxi":194
 * def compression_algorithm_name(grpc_compression_algorithm algorithm):
 *   cdef const char* name
 *   with nogil:             # <<<<<<<<<<<<<<
 *     grpc_compression_algorithm_name(algorithm, &name)
 *   # Let Cython do the right thing with string casting
 */
  {
      #ifdef WITH_THREAD
      PyThreadState *_save;
      Py_UNBLOCK_THREADS
      __Pyx_FastGIL_Remember();
      #endif
      /*try:*/ {

        /* "src/python/grpcio/grpc/_cython/_cygrpc/records.pyx.pxi":195
 *   cdef const char* name
 *   with nogil:
 *     grpc_compression_algorithm_name(algorithm, &name)             # <<<<<<<<<<<<<<
 *   # Let Cython do the right thing with string casting
 *   return name
 */
        (void)(grpc_compression_algorithm_name(__pyx_v_algorithm, (&__pyx_v_name)));
      }

      /* "src/python/grpcio/grpc/_cython/_cygrpc/records.pyx.pxi":194
 * def compression_algorithm_name(grpc_compression_algorithm algorithm):
 *   cdef const char* name
 *   with nogil:             # <<<<<<<<<<<<<<
 *     grpc_compression_algorithm_name(algorithm, &name)
 *   # Let Cython do the right thing with string casting
 */
      /*finally:*/ {
        /*normal exit:*/{
          #ifdef WITH_THREAD
          __Pyx_FastGIL_Forget();
          Py_BLOCK_THREADS
          #endif
          goto __pyx_L5;
        }
        __pyx_L5:;
      }
  }

  /* "src/python/grpcio/grpc/_cython/_cygrpc/records.pyx.pxi":197
 *     grpc_compression_algorithm_name(algorithm, &name)
 *   # Let Cython do the right thing with string casting
 *   return name             # <<<<<<<<<<<<<<
 */
  __Pyx_XDECREF(__pyx_r);
  __pyx_t_1 = __Pyx_PyBytes_FromString(__pyx_v_name); if (unlikely(!__pyx_t_1)) __PYX_ERR(16, 197, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_1);
  __pyx_r = __pyx_t_1;
  __pyx_t_1 = 0;
  goto __pyx_L0;

  /* "src/python/grpcio/grpc/_cython/_cygrpc/records.pyx.pxi":192
 * 
 * 
 * def compression_algorithm_name(grpc_compression_algorithm algorithm):             # <<<<<<<<<<<<<<
 *   cdef const char* name
 *   with nogil:
 */

  /* function exit code */
  __pyx_L1_error:;
  __Pyx_XDECREF(__pyx_t_1);
  __Pyx_AddTraceback("grpc._cython.cygrpc.compression_algorithm_name", __pyx_clineno, __pyx_lineno, __pyx_filename);
  __pyx_r = NULL;
  __pyx_L0:;
  __Pyx_XGIVEREF(__pyx_r);
  __Pyx_RefNannyFinishContext();
  return __pyx_r;
}

/* "src/python/grpcio/grpc/_cython/_cygrpc/security.pyx.pxi":20
 * 
 * 
 * cdef grpc_ssl_roots_override_result ssl_roots_override_callback(             # <<<<<<<<<<<<<<
 *     char **pem_root_certs) nogil:
 *   with gil:
 */

static grpc_ssl_roots_override_result __pyx_f_4grpc_7_cython_6cygrpc_ssl_roots_override_callback(char **__pyx_v_pem_root_certs) {
  PyObject *__pyx_v_pkg = NULL;
  PyObject *__pyx_v_temporary_pem_root_certs = NULL;
  grpc_ssl_roots_override_result __pyx_r;
  __Pyx_RefNannyDeclarations
  PyObject *__pyx_t_1 = NULL;
  PyObject *__pyx_t_2 = NULL;
  int __pyx_t_3;
  Py_ssize_t __pyx_t_4;
  PyObject *__pyx_t_5 = NULL;
  int __pyx_t_6;
  PyObject *__pyx_t_7 = NULL;
  char *__pyx_t_8;
  #ifdef WITH_THREAD
  PyGILState_STATE __pyx_gilstate_save;
  #endif
  __Pyx_RefNannySetupContext("ssl_roots_override_callback", 1);

  /* "src/python/grpcio/grpc/_cython/_cygrpc/security.pyx.pxi":22
 * cdef grpc_ssl_roots_override_result ssl_roots_override_callback(
 *     char **pem_root_certs) nogil:
 *   with gil:             # <<<<<<<<<<<<<<
 *     pkg = __name__
 *     if pkg.endswith('.cygrpc'):
 */
  /*try:*/ {
    {
        #ifdef WITH_THREAD
        PyGILState_STATE __pyx_gilstate_save = __Pyx_PyGILState_Ensure();
        #endif
        /*try:*/ {

          /* "src/python/grpcio/grpc/_cython/_cygrpc/security.pyx.pxi":23
 *     char **pem_root_certs) nogil:
 *   with gil:
 *     pkg = __name__             # <<<<<<<<<<<<<<
 *     if pkg.endswith('.cygrpc'):
 *       pkg = pkg[:-len('.cygrpc')]
 */
          __pyx_t_1 = __Pyx_GetModuleGlobalName(__pyx_n_s_name_2); if (unlikely(!__pyx_t_1)) __PYX_ERR(17, 23, __pyx_L7_error)
          __Pyx_GOTREF(__pyx_t_1);
          __pyx_v_pkg = __pyx_t_1;
          __pyx_t_1 = 0;

          /* "src/python/grpcio/grpc/_cython/_cygrpc/security.pyx.pxi":24
 *   with gil:
 *     pkg = __name__
 *     if pkg.endswith('.cygrpc'):             # <<<<<<<<<<<<<<
 *       pkg = pkg[:-len('.cygrpc')]
 *     temporary_pem_root_certs = pkgutil.get_data(pkg, '_credentials/roots.pem')
 */
          __pyx_t_1 = __Pyx_PyObject_GetAttrStr(__pyx_v_pkg, __pyx_n_s_endswith); if (unlikely(!__pyx_t_1)) __PYX_ERR(17, 24, __pyx_L7_error)
          __Pyx_GOTREF(__pyx_t_1);
          __pyx_t_2 = __Pyx_PyObject_Call(__pyx_t_1, __pyx_tuple__108, NULL); if (unlikely(!__pyx_t_2)) __PYX_ERR(17, 24, __pyx_L7_error)
          __Pyx_GOTREF(__pyx_t_2);
          __Pyx_DECREF(__pyx_t_1); __pyx_t_1 = 0;
          __pyx_t_3 = __Pyx_PyObject_IsTrue(__pyx_t_2); if (unlikely(__pyx_t_3 < 0)) __PYX_ERR(17, 24, __pyx_L7_error)
          __Pyx_DECREF(__pyx_t_2); __pyx_t_2 = 0;
          if (__pyx_t_3) {

            /* "src/python/grpcio/grpc/_cython/_cygrpc/security.pyx.pxi":25
 *     pkg = __name__
 *     if pkg.endswith('.cygrpc'):
 *       pkg = pkg[:-len('.cygrpc')]             # <<<<<<<<<<<<<<
 *     temporary_pem_root_certs = pkgutil.get_data(pkg, '_credentials/roots.pem')
 *     pem_root_certs[0] = <char *>gpr_malloc(len(temporary_pem_root_certs) + 1)
 */
            __pyx_t_4 = PyObject_Length(__pyx_kp_s_cygrpc); if (unlikely(__pyx_t_4 == ((Py_ssize_t)-1))) __PYX_ERR(17, 25, __pyx_L7_error)
            __pyx_t_2 = __Pyx_PyObject_GetSlice(__pyx_v_pkg, 0, (-__pyx_t_4), NULL, NULL, NULL, 0, 1, 1); if (unlikely(!__pyx_t_2)) __PYX_ERR(17, 25, __pyx_L7_error)
            __Pyx_GOTREF(__pyx_t_2);
            __Pyx_DECREF_SET(__pyx_v_pkg, __pyx_t_2);
            __pyx_t_2 = 0;

            /* "src/python/grpcio/grpc/_cython/_cygrpc/security.pyx.pxi":24
 *   with gil:
 *     pkg = __name__
 *     if pkg.endswith('.cygrpc'):             # <<<<<<<<<<<<<<
 *       pkg = pkg[:-len('.cygrpc')]
 *     temporary_pem_root_certs = pkgutil.get_data(pkg, '_credentials/roots.pem')
 */
          }

          /* "src/python/grpcio/grpc/_cython/_cygrpc/security.pyx.pxi":26
 *     if pkg.endswith('.cygrpc'):
 *       pkg = pkg[:-len('.cygrpc')]
 *     temporary_pem_root_certs = pkgutil.get_data(pkg, '_credentials/roots.pem')             # <<<<<<<<<<<<<<
 *     pem_root_certs[0] = <char *>gpr_malloc(len(temporary_pem_root_certs) + 1)
 *     memcpy(
 */
          __pyx_t_1 = __Pyx_GetModuleGlobalName(__pyx_n_s_pkgutil); if (unlikely(!__pyx_t_1)) __PYX_ERR(17, 26, __pyx_L7_error)
          __Pyx_GOTREF(__pyx_t_1);
          __pyx_t_5 = __Pyx_PyObject_GetAttrStr(__pyx_t_1, __pyx_n_s_get_data); if (unlikely(!__pyx_t_5)) __PYX_ERR(17, 26, __pyx_L7_error)
          __Pyx_GOTREF(__pyx_t_5);
          __Pyx_DECREF(__pyx_t_1); __pyx_t_1 = 0;
          __pyx_t_1 = NULL;
          __pyx_t_6 = 0;
          if (CYTHON_UNPACK_METHODS && unlikely(PyMethod_Check(__pyx_t_5))) {
            __pyx_t_1 = PyMethod_GET_SELF(__pyx_t_5);
            if (likely(__pyx_t_1)) {
              PyObject* function = PyMethod_GET_FUNCTION(__pyx_t_5);
              __Pyx_INCREF(__pyx_t_1);
              __Pyx_INCREF(function);
              __Pyx_DECREF_SET(__pyx_t_5, function);
              __pyx_t_6 = 1;
            }
          }
          #if CYTHON_FAST_PYCALL
          if (PyFunction_Check(__pyx_t_5)) {
            PyObject *__pyx_temp[3] = {__pyx_t_1, __pyx_v_pkg, __pyx_kp_s_credentials_roots_pem};
            __pyx_t_2 = __Pyx_PyFunction_FastCall(__pyx_t_5, __pyx_temp+1-__pyx_t_6, 2+__pyx_t_6); if (unlikely(!__pyx_t_2)) __PYX_ERR(17, 26, __pyx_L7_error)
            __Pyx_XDECREF(__pyx_t_1); __pyx_t_1 = 0;
            __Pyx_GOTREF(__pyx_t_2);
          } else
          #endif
          #if CYTHON_FAST_PYCCALL
          if (__Pyx_PyFastCFunction_Check(__pyx_t_5)) {
            PyObject *__pyx_temp[3] = {__pyx_t_1, __pyx_v_pkg, __pyx_kp_s_credentials_roots_pem};
            __pyx_t_2 = __Pyx_PyCFunction_FastCall(__pyx_t_5, __pyx_temp+1-__pyx_t_6, 2+__pyx_t_6); if (unlikely(!__pyx_t_2)) __PYX_ERR(17, 26, __pyx_L7_error)
            __Pyx_XDECREF(__pyx_t_1); __pyx_t_1 = 0;
            __Pyx_GOTREF(__pyx_t_2);
          } else
          #endif
          {
            __pyx_t_7 = PyTuple_New(2+__pyx_t_6); if (unlikely(!__pyx_t_7)) __PYX_ERR(17, 26, __pyx_L7_error)
            __Pyx_GOTREF(__pyx_t_7);
            if (__pyx_t_1) {
              __Pyx_GIVEREF(__pyx_t_1); PyTuple_SET_ITEM(__pyx_t_7, 0, __pyx_t_1); __pyx_t_1 = NULL;
            }
            __Pyx_INCREF(__pyx_v_pkg);
            __Pyx_GIVEREF(__pyx_v_pkg);
            PyTuple_SET_ITEM(__pyx_t_7, 0+__pyx_t_6, __pyx_v_pkg);
            __Pyx_INCREF(__pyx_kp_s_credentials_roots_pem);
            __Pyx_GIVEREF(__pyx_kp_s_credentials_roots_pem);
            PyTuple_SET_ITEM(__pyx_t_7, 1+__pyx_t_6, __pyx_kp_s_credentials_roots_pem);
            __pyx_t_2 = __Pyx_PyObject_Call(__pyx_t_5, __pyx_t_7, NULL); if (unlikely(!__pyx_t_2)) __PYX_ERR(17, 26, __pyx_L7_error)
            __Pyx_GOTREF(__pyx_t_2);
            __Pyx_DECREF(__pyx_t_7); __pyx_t_7 = 0;
          }
          __Pyx_DECREF(__pyx_t_5); __pyx_t_5 = 0;
          __pyx_v_temporary_pem_root_certs = __pyx_t_2;
          __pyx_t_2 = 0;

          /* "src/python/grpcio/grpc/_cython/_cygrpc/security.pyx.pxi":27
 *       pkg = pkg[:-len('.cygrpc')]
 *     temporary_pem_root_certs = pkgutil.get_data(pkg, '_credentials/roots.pem')
 *     pem_root_certs[0] = <char *>gpr_malloc(len(temporary_pem_root_certs) + 1)             # <<<<<<<<<<<<<<
 *     memcpy(
 *         pem_root_certs[0], <char *>temporary_pem_root_certs,
 */
          __pyx_t_4 = PyObject_Length(__pyx_v_temporary_pem_root_certs); if (unlikely(__pyx_t_4 == ((Py_ssize_t)-1))) __PYX_ERR(17, 27, __pyx_L7_error)
          (__pyx_v_pem_root_certs[0]) = ((char *)gpr_malloc((__pyx_t_4 + 1)));

          /* "src/python/grpcio/grpc/_cython/_cygrpc/security.pyx.pxi":29
 *     pem_root_certs[0] = <char *>gpr_malloc(len(temporary_pem_root_certs) + 1)
 *     memcpy(
 *         pem_root_certs[0], <char *>temporary_pem_root_certs,             # <<<<<<<<<<<<<<
 *         len(temporary_pem_root_certs))
 *     pem_root_certs[0][len(temporary_pem_root_certs)] = '\0'
 */
          __pyx_t_8 = __Pyx_PyObject_AsWritableString(__pyx_v_temporary_pem_root_certs); if (unlikely((!__pyx_t_8) && PyErr_Occurred())) __PYX_ERR(17, 29, __pyx_L7_error)

          /* "src/python/grpcio/grpc/_cython/_cygrpc/security.pyx.pxi":30
 *     memcpy(
 *         pem_root_certs[0], <char *>temporary_pem_root_certs,
 *         len(temporary_pem_root_certs))             # <<<<<<<<<<<<<<
 *     pem_root_certs[0][len(temporary_pem_root_certs)] = '\0'
 * 
 */
          __pyx_t_4 = PyObject_Length(__pyx_v_temporary_pem_root_certs); if (unlikely(__pyx_t_4 == ((Py_ssize_t)-1))) __PYX_ERR(17, 30, __pyx_L7_error)

          /* "src/python/grpcio/grpc/_cython/_cygrpc/security.pyx.pxi":28
 *     temporary_pem_root_certs = pkgutil.get_data(pkg, '_credentials/roots.pem')
 *     pem_root_certs[0] = <char *>gpr_malloc(len(temporary_pem_root_certs) + 1)
 *     memcpy(             # <<<<<<<<<<<<<<
 *         pem_root_certs[0], <char *>temporary_pem_root_certs,
 *         len(temporary_pem_root_certs))
 */
          (void)(memcpy((__pyx_v_pem_root_certs[0]), ((char *)__pyx_t_8), __pyx_t_4));

          /* "src/python/grpcio/grpc/_cython/_cygrpc/security.pyx.pxi":31
 *         pem_root_certs[0], <char *>temporary_pem_root_certs,
 *         len(temporary_pem_root_certs))
 *     pem_root_certs[0][len(temporary_pem_root_certs)] = '\0'             # <<<<<<<<<<<<<<
 * 
 *   return GRPC_SSL_ROOTS_OVERRIDE_OK
 */
          __pyx_t_4 = PyObject_Length(__pyx_v_temporary_pem_root_certs); if (unlikely(__pyx_t_4 == ((Py_ssize_t)-1))) __PYX_ERR(17, 31, __pyx_L7_error)
          ((__pyx_v_pem_root_certs[0])[__pyx_t_4]) = '\x00';
        }

        /* "src/python/grpcio/grpc/_cython/_cygrpc/security.pyx.pxi":22
 * cdef grpc_ssl_roots_override_result ssl_roots_override_callback(
 *     char **pem_root_certs) nogil:
 *   with gil:             # <<<<<<<<<<<<<<
 *     pkg = __name__
 *     if pkg.endswith('.cygrpc'):
 */
        /*finally:*/ {
          /*normal exit:*/{
            #ifdef WITH_THREAD
            __Pyx_PyGILState_Release(__pyx_gilstate_save);
            #endif
            goto __pyx_L8;
          }
          __pyx_L7_error: {
            #ifdef WITH_THREAD
            __Pyx_PyGILState_Release(__pyx_gilstate_save);
            #endif
            goto __pyx_L4_error;
          }
          __pyx_L8:;
        }
    }

    /* "src/python/grpcio/grpc/_cython/_cygrpc/security.pyx.pxi":33
 *     pem_root_certs[0][len(temporary_pem_root_certs)] = '\0'
 * 
 *   return GRPC_SSL_ROOTS_OVERRIDE_OK             # <<<<<<<<<<<<<<
 * 
 * 
 */
    __pyx_r = GRPC_SSL_ROOTS_OVERRIDE_OK;
    goto __pyx_L3_return;
  }

  /* "src/python/grpcio/grpc/_cython/_cygrpc/security.pyx.pxi":22
 * cdef grpc_ssl_roots_override_result ssl_roots_override_callback(
 *     char **pem_root_certs) nogil:
 *   with gil:             # <<<<<<<<<<<<<<
 *     pkg = __name__
 *     if pkg.endswith('.cygrpc'):
 */
  /*finally:*/ {
    __pyx_L3_return: {
      #ifdef WITH_THREAD
      __pyx_gilstate_save = __Pyx_PyGILState_Ensure();
      #endif
      goto __pyx_L0;
    }
    __pyx_L4_error: {
      #ifdef WITH_THREAD
      __pyx_gilstate_save = __Pyx_PyGILState_Ensure();
      #endif
      goto __pyx_L1_error;
    }
  }

  /* "src/python/grpcio/grpc/_cython/_cygrpc/security.pyx.pxi":20
 * 
 * 
 * cdef grpc_ssl_roots_override_result ssl_roots_override_callback(             # <<<<<<<<<<<<<<
 *     char **pem_root_certs) nogil:
 *   with gil:
 */

  /* function exit code */
  __pyx_r = (grpc_ssl_roots_override_result) 0;
  goto __pyx_L0;
  __pyx_L1_error:;
  __Pyx_XDECREF(__pyx_t_1);
  __Pyx_XDECREF(__pyx_t_2);
  __Pyx_XDECREF(__pyx_t_5);
  __Pyx_XDECREF(__pyx_t_7);
  __Pyx_WriteUnraisable("grpc._cython.cygrpc.ssl_roots_override_callback", __pyx_clineno, __pyx_lineno, __pyx_filename, 1, 1);
  __pyx_r = (grpc_ssl_roots_override_result) 0;
  __pyx_L0:;
  __Pyx_XDECREF(__pyx_v_pkg);
  __Pyx_XDECREF(__pyx_v_temporary_pem_root_certs);
  #ifdef WITH_THREAD
  __Pyx_PyGILState_Release(__pyx_gilstate_save);
  #endif
  return __pyx_r;
}

/* "src/python/grpcio/grpc/_cython/_cygrpc/security.pyx.pxi":36
 * 
 * 
 * def peer_identities(Call call):             # <<<<<<<<<<<<<<
 *   cdef grpc_auth_context* auth_context
 *   cdef grpc_auth_property_iterator properties
 */

/* Python wrapper */
static PyObject *__pyx_pw_4grpc_7_cython_6cygrpc_29peer_identities(PyObject *__pyx_self, PyObject *__pyx_v_call); /*proto*/
static PyMethodDef __pyx_mdef_4grpc_7_cython_6cygrpc_29peer_identities = {"peer_identities", (PyCFunction)__pyx_pw_4grpc_7_cython_6cygrpc_29peer_identities, METH_O, 0};
static PyObject *__pyx_pw_4grpc_7_cython_6cygrpc_29peer_identities(PyObject *__pyx_self, PyObject *__pyx_v_call) {
  PyObject *__pyx_r = 0;
  __Pyx_RefNannyDeclarations
  __Pyx_RefNannySetupContext("peer_identities (wrapper)", 0);
  if (unlikely(!__Pyx_ArgTypeTest(((PyObject *)__pyx_v_call), __pyx_ptype_4grpc_7_cython_6cygrpc_Call, 1, "call", 0))) __PYX_ERR(17, 36, __pyx_L1_error)
  __pyx_r = __pyx_pf_4grpc_7_cython_6cygrpc_28peer_identities(__pyx_self, ((struct __pyx_obj_4grpc_7_cython_6cygrpc_Call *)__pyx_v_call));

  /* function exit code */
  goto __pyx_L0;
  __pyx_L1_error:;
  __pyx_r = NULL;
  __pyx_L0:;
  __Pyx_RefNannyFinishContext();
  return __pyx_r;
}

static PyObject *__pyx_pf_4grpc_7_cython_6cygrpc_28peer_identities(CYTHON_UNUSED PyObject *__pyx_self, struct __pyx_obj_4grpc_7_cython_6cygrpc_Call *__pyx_v_call) {
  grpc_auth_context *__pyx_v_auth_context;
  grpc_auth_property_iterator __pyx_v_properties;
  grpc_auth_property const *__pyx_v_property;
  PyObject *__pyx_v_identities = NULL;
  PyObject *__pyx_r = NULL;
  __Pyx_RefNannyDeclarations
  int __pyx_t_1;
  PyObject *__pyx_t_2 = NULL;
  int __pyx_t_3;
  __Pyx_RefNannySetupContext("peer_identities", 0);

  /* "src/python/grpcio/grpc/_cython/_cygrpc/security.pyx.pxi":41
 *   cdef const grpc_auth_property* property
 * 
 *   auth_context = grpc_call_auth_context(call.c_call)             # <<<<<<<<<<<<<<
 *   if auth_context == NULL:
 *     return None
 */
  __pyx_v_auth_context = grpc_call_auth_context(__pyx_v_call->c_call);

  /* "src/python/grpcio/grpc/_cython/_cygrpc/security.pyx.pxi":42
 * 
 *   auth_context = grpc_call_auth_context(call.c_call)
 *   if auth_context == NULL:             # <<<<<<<<<<<<<<
 *     return None
 *   properties = grpc_auth_context_peer_identity(auth_context)
 */
  __pyx_t_1 = ((__pyx_v_auth_context == NULL) != 0);
  if (__pyx_t_1) {

    /* "src/python/grpcio/grpc/_cython/_cygrpc/security.pyx.pxi":43
 *   auth_context = grpc_call_auth_context(call.c_call)
 *   if auth_context == NULL:
 *     return None             # <<<<<<<<<<<<<<
 *   properties = grpc_auth_context_peer_identity(auth_context)
 *   identities = []
 */
    __Pyx_XDECREF(__pyx_r);
    __pyx_r = Py_None; __Pyx_INCREF(Py_None);
    goto __pyx_L0;

    /* "src/python/grpcio/grpc/_cython/_cygrpc/security.pyx.pxi":42
 * 
 *   auth_context = grpc_call_auth_context(call.c_call)
 *   if auth_context == NULL:             # <<<<<<<<<<<<<<
 *     return None
 *   properties = grpc_auth_context_peer_identity(auth_context)
 */
  }

  /* "src/python/grpcio/grpc/_cython/_cygrpc/security.pyx.pxi":44
 *   if auth_context == NULL:
 *     return None
 *   properties = grpc_auth_context_peer_identity(auth_context)             # <<<<<<<<<<<<<<
 *   identities = []
 *   while True:
 */
  __pyx_v_properties = grpc_auth_context_peer_identity(__pyx_v_auth_context);

  /* "src/python/grpcio/grpc/_cython/_cygrpc/security.pyx.pxi":45
 *     return None
 *   properties = grpc_auth_context_peer_identity(auth_context)
 *   identities = []             # <<<<<<<<<<<<<<
 *   while True:
 *     property = grpc_auth_property_iterator_next(&properties)
 */
  __pyx_t_2 = PyList_New(0); if (unlikely(!__pyx_t_2)) __PYX_ERR(17, 45, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_2);
  __pyx_v_identities = ((PyObject*)__pyx_t_2);
  __pyx_t_2 = 0;

  /* "src/python/grpcio/grpc/_cython/_cygrpc/security.pyx.pxi":46
 *   properties = grpc_auth_context_peer_identity(auth_context)
 *   identities = []
 *   while True:             # <<<<<<<<<<<<<<
 *     property = grpc_auth_property_iterator_next(&properties)
 *     if property == NULL:
 */
  while (1) {

    /* "src/python/grpcio/grpc/_cython/_cygrpc/security.pyx.pxi":47
 *   identities = []
 *   while True:
 *     property = grpc_auth_property_iterator_next(&properties)             # <<<<<<<<<<<<<<
 *     if property == NULL:
 *       break
 */
    __pyx_v_property = grpc_auth_property_iterator_next((&__pyx_v_properties));

    /* "src/python/grpcio/grpc/_cython/_cygrpc/security.pyx.pxi":48
 *   while True:
 *     property = grpc_auth_property_iterator_next(&properties)
 *     if property == NULL:             # <<<<<<<<<<<<<<
 *       break
 *     if property.value != NULL:
 */
    __pyx_t_1 = ((__pyx_v_property == NULL) != 0);
    if (__pyx_t_1) {

      /* "src/python/grpcio/grpc/_cython/_cygrpc/security.pyx.pxi":49
 *     property = grpc_auth_property_iterator_next(&properties)
 *     if property == NULL:
 *       break             # <<<<<<<<<<<<<<
 *     if property.value != NULL:
 *       identities.append(<bytes>(property.value))
 */
      goto __pyx_L5_break;

      /* "src/python/grpcio/grpc/_cython/_cygrpc/security.pyx.pxi":48
 *   while True:
 *     property = grpc_auth_property_iterator_next(&properties)
 *     if property == NULL:             # <<<<<<<<<<<<<<
 *       break
 *     if property.value != NULL:
 */
    }

    /* "src/python/grpcio/grpc/_cython/_cygrpc/security.pyx.pxi":50
 *     if property == NULL:
 *       break
 *     if property.value != NULL:             # <<<<<<<<<<<<<<
 *       identities.append(<bytes>(property.value))
 *   grpc_auth_context_release(auth_context)
 */
    __pyx_t_1 = ((__pyx_v_property->value != NULL) != 0);
    if (__pyx_t_1) {

      /* "src/python/grpcio/grpc/_cython/_cygrpc/security.pyx.pxi":51
 *       break
 *     if property.value != NULL:
 *       identities.append(<bytes>(property.value))             # <<<<<<<<<<<<<<
 *   grpc_auth_context_release(auth_context)
 *   return identities if identities else None
 */
      __pyx_t_2 = __Pyx_PyBytes_FromString(__pyx_v_property->value); if (unlikely(!__pyx_t_2)) __PYX_ERR(17, 51, __pyx_L1_error)
      __Pyx_GOTREF(__pyx_t_2);
      __pyx_t_3 = __Pyx_PyList_Append(__pyx_v_identities, __pyx_t_2); if (unlikely(__pyx_t_3 == ((int)-1))) __PYX_ERR(17, 51, __pyx_L1_error)
      __Pyx_DECREF(__pyx_t_2); __pyx_t_2 = 0;

      /* "src/python/grpcio/grpc/_cython/_cygrpc/security.pyx.pxi":50
 *     if property == NULL:
 *       break
 *     if property.value != NULL:             # <<<<<<<<<<<<<<
 *       identities.append(<bytes>(property.value))
 *   grpc_auth_context_release(auth_context)
 */
    }
  }
  __pyx_L5_break:;

  /* "src/python/grpcio/grpc/_cython/_cygrpc/security.pyx.pxi":52
 *     if property.value != NULL:
 *       identities.append(<bytes>(property.value))
 *   grpc_auth_context_release(auth_context)             # <<<<<<<<<<<<<<
 *   return identities if identities else None
 * 
 */
  grpc_auth_context_release(__pyx_v_auth_context);

  /* "src/python/grpcio/grpc/_cython/_cygrpc/security.pyx.pxi":53
 *       identities.append(<bytes>(property.value))
 *   grpc_auth_context_release(auth_context)
 *   return identities if identities else None             # <<<<<<<<<<<<<<
 * 
 * def peer_identity_key(Call call):
 */
  __Pyx_XDECREF(__pyx_r);
  __pyx_t_1 = (PyList_GET_SIZE(__pyx_v_identities) != 0);
  if (__pyx_t_1) {
    __Pyx_INCREF(__pyx_v_identities);
    __pyx_t_2 = __pyx_v_identities;
  } else {
    __Pyx_INCREF(Py_None);
    __pyx_t_2 = Py_None;
  }
  __pyx_r = __pyx_t_2;
  __pyx_t_2 = 0;
  goto __pyx_L0;

  /* "src/python/grpcio/grpc/_cython/_cygrpc/security.pyx.pxi":36
 * 
 * 
 * def peer_identities(Call call):             # <<<<<<<<<<<<<<
 *   cdef grpc_auth_context* auth_context
 *   cdef grpc_auth_property_iterator properties
 */

  /* function exit code */
  __pyx_L1_error:;
  __Pyx_XDECREF(__pyx_t_2);
  __Pyx_AddTraceback("grpc._cython.cygrpc.peer_identities", __pyx_clineno, __pyx_lineno, __pyx_filename);
  __pyx_r = NULL;
  __pyx_L0:;
  __Pyx_XDECREF(__pyx_v_identities);
  __Pyx_XGIVEREF(__pyx_r);
  __Pyx_RefNannyFinishContext();
  return __pyx_r;
}

/* "src/python/grpcio/grpc/_cython/_cygrpc/security.pyx.pxi":55
 *   return identities if identities else None
 * 
 * def peer_identity_key(Call call):             # <<<<<<<<<<<<<<
 *   cdef grpc_auth_context* auth_context
 *   cdef const char* c_key
 */

/* Python wrapper */
static PyObject *__pyx_pw_4grpc_7_cython_6cygrpc_31peer_identity_key(PyObject *__pyx_self, PyObject *__pyx_v_call); /*proto*/
static PyMethodDef __pyx_mdef_4grpc_7_cython_6cygrpc_31peer_identity_key = {"peer_identity_key", (PyCFunction)__pyx_pw_4grpc_7_cython_6cygrpc_31peer_identity_key, METH_O, 0};
static PyObject *__pyx_pw_4grpc_7_cython_6cygrpc_31peer_identity_key(PyObject *__pyx_self, PyObject *__pyx_v_call) {
  PyObject *__pyx_r = 0;
  __Pyx_RefNannyDeclarations
  __Pyx_RefNannySetupContext("peer_identity_key (wrapper)", 0);
  if (unlikely(!__Pyx_ArgTypeTest(((PyObject *)__pyx_v_call), __pyx_ptype_4grpc_7_cython_6cygrpc_Call, 1, "call", 0))) __PYX_ERR(17, 55, __pyx_L1_error)
  __pyx_r = __pyx_pf_4grpc_7_cython_6cygrpc_30peer_identity_key(__pyx_self, ((struct __pyx_obj_4grpc_7_cython_6cygrpc_Call *)__pyx_v_call));

  /* function exit code */
  goto __pyx_L0;
  __pyx_L1_error:;
  __pyx_r = NULL;
  __pyx_L0:;
  __Pyx_RefNannyFinishContext();
  return __pyx_r;
}

static PyObject *__pyx_pf_4grpc_7_cython_6cygrpc_30peer_identity_key(CYTHON_UNUSED PyObject *__pyx_self, struct __pyx_obj_4grpc_7_cython_6cygrpc_Call *__pyx_v_call) {
  grpc_auth_context *__pyx_v_auth_context;
  char const *__pyx_v_c_key;
  PyObject *__pyx_v_key = NULL;
  PyObject *__pyx_r = NULL;
  __Pyx_RefNannyDeclarations
  int __pyx_t_1;
  PyObject *__pyx_t_2 = NULL;
  PyObject *__pyx_t_3 = NULL;
  __Pyx_RefNannySetupContext("peer_identity_key", 0);

  /* "src/python/grpcio/grpc/_cython/_cygrpc/security.pyx.pxi":58
 *   cdef grpc_auth_context* auth_context
 *   cdef const char* c_key
 *   auth_context = grpc_call_auth_context(call.c_call)             # <<<<<<<<<<<<<<
 *   if auth_context == NULL:
 *     return None
 */
  __pyx_v_auth_context = grpc_call_auth_context(__pyx_v_call->c_call);

  /* "src/python/grpcio/grpc/_cython/_cygrpc/security.pyx.pxi":59
 *   cdef const char* c_key
 *   auth_context = grpc_call_auth_context(call.c_call)
 *   if auth_context == NULL:             # <<<<<<<<<<<<<<
 *     return None
 *   c_key = grpc_auth_context_peer_identity_property_name(auth_context)
 */
  __pyx_t_1 = ((__pyx_v_auth_context == NULL) != 0);
  if (__pyx_t_1) {

    /* "src/python/grpcio/grpc/_cython/_cygrpc/security.pyx.pxi":60
 *   auth_context = grpc_call_auth_context(call.c_call)
 *   if auth_context == NULL:
 *     return None             # <<<<<<<<<<<<<<
 *   c_key = grpc_auth_context_peer_identity_property_name(auth_context)
 *   if c_key == NULL:
 */
    __Pyx_XDECREF(__pyx_r);
    __pyx_r = Py_None; __Pyx_INCREF(Py_None);
    goto __pyx_L0;

    /* "src/python/grpcio/grpc/_cython/_cygrpc/security.pyx.pxi":59
 *   cdef const char* c_key
 *   auth_context = grpc_call_auth_context(call.c_call)
 *   if auth_context == NULL:             # <<<<<<<<<<<<<<
 *     return None
 *   c_key = grpc_auth_context_peer_identity_property_name(auth_context)
 */
  }

  /* "src/python/grpcio/grpc/_cython/_cygrpc/security.pyx.pxi":61
 *   if auth_context == NULL:
 *     return None
 *   c_key = grpc_auth_context_peer_identity_property_name(auth_context)             # <<<<<<<<<<<<<<
 *   if c_key == NULL:
 *     key = None
 */
  __pyx_v_c_key = grpc_auth_context_peer_identity_property_name(__pyx_v_auth_context);

  /* "src/python/grpcio/grpc/_cython/_cygrpc/security.pyx.pxi":62
 *     return None
 *   c_key = grpc_auth_context_peer_identity_property_name(auth_context)
 *   if c_key == NULL:             # <<<<<<<<<<<<<<
 *     key = None
 *   else:
 */
  __pyx_t_1 = ((__pyx_v_c_key == NULL) != 0);
  if (__pyx_t_1) {

    /* "src/python/grpcio/grpc/_cython/_cygrpc/security.pyx.pxi":63
 *   c_key = grpc_auth_context_peer_identity_property_name(auth_context)
 *   if c_key == NULL:
 *     key = None             # <<<<<<<<<<<<<<
 *   else:
 *     key = <bytes> grpc_auth_context_peer_identity_property_name(auth_context)
 */
    __Pyx_INCREF(Py_None);
    __pyx_v_key = ((PyObject*)Py_None);

    /* "src/python/grpcio/grpc/_cython/_cygrpc/security.pyx.pxi":62
 *     return None
 *   c_key = grpc_auth_context_peer_identity_property_name(auth_context)
 *   if c_key == NULL:             # <<<<<<<<<<<<<<
 *     key = None
 *   else:
 */
    goto __pyx_L4;
  }

  /* "src/python/grpcio/grpc/_cython/_cygrpc/security.pyx.pxi":65
 *     key = None
 *   else:
 *     key = <bytes> grpc_auth_context_peer_identity_property_name(auth_context)             # <<<<<<<<<<<<<<
 *   grpc_auth_context_release(auth_context)
 *   return key
 */
  /*else*/ {
    __pyx_t_2 = __Pyx_PyBytes_FromString(grpc_auth_context_peer_identity_property_name(__pyx_v_auth_context)); if (unlikely(!__pyx_t_2)) __PYX_ERR(17, 65, __pyx_L1_error)
    __Pyx_GOTREF(__pyx_t_2);
    __pyx_t_3 = __pyx_t_2;
    __Pyx_INCREF(__pyx_t_3);
    __Pyx_DECREF(__pyx_t_2); __pyx_t_2 = 0;
    __pyx_v_key = ((PyObject*)__pyx_t_3);
    __pyx_t_3 = 0;
  }
  __pyx_L4:;

  /* "src/python/grpcio/grpc/_cython/_cygrpc/security.pyx.pxi":66
 *   else:
 *     key = <bytes> grpc_auth_context_peer_identity_property_name(auth_context)
 *   grpc_auth_context_release(auth_context)             # <<<<<<<<<<<<<<
 *   return key
 * 
 */
  grpc_auth_context_release(__pyx_v_auth_context);

  /* "src/python/grpcio/grpc/_cython/_cygrpc/security.pyx.pxi":67
 *     key = <bytes> grpc_auth_context_peer_identity_property_name(auth_context)
 *   grpc_auth_context_release(auth_context)
 *   return key             # <<<<<<<<<<<<<<
 * 
 * def auth_context(Call call):
 */
  __Pyx_XDECREF(__pyx_r);
  __Pyx_INCREF(__pyx_v_key);
  __pyx_r = __pyx_v_key;
  goto __pyx_L0;

  /* "src/python/grpcio/grpc/_cython/_cygrpc/security.pyx.pxi":55
 *   return identities if identities else None
 * 
 * def peer_identity_key(Call call):             # <<<<<<<<<<<<<<
 *   cdef grpc_auth_context* auth_context
 *   cdef const char* c_key
 */

  /* function exit code */
  __pyx_L1_error:;
  __Pyx_XDECREF(__pyx_t_2);
  __Pyx_XDECREF(__pyx_t_3);
  __Pyx_AddTraceback("grpc._cython.cygrpc.peer_identity_key", __pyx_clineno, __pyx_lineno, __pyx_filename);
  __pyx_r = NULL;
  __pyx_L0:;
  __Pyx_XDECREF(__pyx_v_key);
  __Pyx_XGIVEREF(__pyx_r);
  __Pyx_RefNannyFinishContext();
  return __pyx_r;
}

/* "src/python/grpcio/grpc/_cython/_cygrpc/security.pyx.pxi":69
 *   return key
 * 
 * def auth_context(Call call):             # <<<<<<<<<<<<<<
 *   cdef grpc_auth_context* auth_context
 *   cdef grpc_auth_property_iterator properties
 */

/* Python wrapper */
static PyObject *__pyx_pw_4grpc_7_cython_6cygrpc_33auth_context(PyObject *__pyx_self, PyObject *__pyx_v_call); /*proto*/
static PyMethodDef __pyx_mdef_4grpc_7_cython_6cygrpc_33auth_context = {"auth_context", (PyCFunction)__pyx_pw_4grpc_7_cython_6cygrpc_33auth_context, METH_O, 0};
static PyObject *__pyx_pw_4grpc_7_cython_6cygrpc_33auth_context(PyObject *__pyx_self, PyObject *__pyx_v_call) {
  PyObject *__pyx_r = 0;
  __Pyx_RefNannyDeclarations
  __Pyx_RefNannySetupContext("auth_context (wrapper)", 0);
  if (unlikely(!__Pyx_ArgTypeTest(((PyObject *)__pyx_v_call), __pyx_ptype_4grpc_7_cython_6cygrpc_Call, 1, "call", 0))) __PYX_ERR(17, 69, __pyx_L1_error)
  __pyx_r = __pyx_pf_4grpc_7_cython_6cygrpc_32auth_context(__pyx_self, ((struct __pyx_obj_4grpc_7_cython_6cygrpc_Call *)__pyx_v_call));

  /* function exit code */
  goto __pyx_L0;
  __pyx_L1_error:;
  __pyx_r = NULL;
  __pyx_L0:;
  __Pyx_RefNannyFinishContext();
  return __pyx_r;
}

static PyObject *__pyx_pf_4grpc_7_cython_6cygrpc_32auth_context(CYTHON_UNUSED PyObject *__pyx_self, struct __pyx_obj_4grpc_7_cython_6cygrpc_Call *__pyx_v_call) {
  grpc_auth_context *__pyx_v_auth_context;
  grpc_auth_property_iterator __pyx_v_properties;
  grpc_auth_property const *__pyx_v_property;
  PyObject *__pyx_v_py_auth_context = NULL;
  PyObject *__pyx_v_key = NULL;
  PyObject *__pyx_r = NULL;
  __Pyx_RefNannyDeclarations
  int __pyx_t_1;
  PyObject *__pyx_t_2 = NULL;
  int __pyx_t_3;
  PyObject *__pyx_t_4 = NULL;
  int __pyx_t_5;
  __Pyx_RefNannySetupContext("auth_context", 0);

  /* "src/python/grpcio/grpc/_cython/_cygrpc/security.pyx.pxi":74
 *   cdef const grpc_auth_property* property
 * 
 *   auth_context = grpc_call_auth_context(call.c_call)             # <<<<<<<<<<<<<<
 *   if auth_context == NULL:
 *     return {}
 */
  __pyx_v_auth_context = grpc_call_auth_context(__pyx_v_call->c_call);

  /* "src/python/grpcio/grpc/_cython/_cygrpc/security.pyx.pxi":75
 * 
 *   auth_context = grpc_call_auth_context(call.c_call)
 *   if auth_context == NULL:             # <<<<<<<<<<<<<<
 *     return {}
 *   properties = grpc_auth_context_property_iterator(auth_context)
 */
  __pyx_t_1 = ((__pyx_v_auth_context == NULL) != 0);
  if (__pyx_t_1) {

    /* "src/python/grpcio/grpc/_cython/_cygrpc/security.pyx.pxi":76
 *   auth_context = grpc_call_auth_context(call.c_call)
 *   if auth_context == NULL:
 *     return {}             # <<<<<<<<<<<<<<
 *   properties = grpc_auth_context_property_iterator(auth_context)
 *   py_auth_context = {}
 */
    __Pyx_XDECREF(__pyx_r);
    __pyx_t_2 = __Pyx_PyDict_NewPresized(0); if (unlikely(!__pyx_t_2)) __PYX_ERR(17, 76, __pyx_L1_error)
    __Pyx_GOTREF(__pyx_t_2);
    __pyx_r = __pyx_t_2;
    __pyx_t_2 = 0;
    goto __pyx_L0;

    /* "src/python/grpcio/grpc/_cython/_cygrpc/security.pyx.pxi":75
 * 
 *   auth_context = grpc_call_auth_context(call.c_call)
 *   if auth_context == NULL:             # <<<<<<<<<<<<<<
 *     return {}
 *   properties = grpc_auth_context_property_iterator(auth_context)
 */
  }

  /* "src/python/grpcio/grpc/_cython/_cygrpc/security.pyx.pxi":77
 *   if auth_context == NULL:
 *     return {}
 *   properties = grpc_auth_context_property_iterator(auth_context)             # <<<<<<<<<<<<<<
 *   py_auth_context = {}
 *   while True:
 */
  __pyx_v_properties = grpc_auth_context_property_iterator(__pyx_v_auth_context);

  /* "src/python/grpcio/grpc/_cython/_cygrpc/security.pyx.pxi":78
 *     return {}
 *   properties = grpc_auth_context_property_iterator(auth_context)
 *   py_auth_context = {}             # <<<<<<<<<<<<<<
 *   while True:
 *     property = grpc_auth_property_iterator_next(&properties)
 */
  __pyx_t_2 = __Pyx_PyDict_NewPresized(0); if (unlikely(!__pyx_t_2)) __PYX_ERR(17, 78, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_2);
  __pyx_v_py_auth_context = ((PyObject*)__pyx_t_2);
  __pyx_t_2 = 0;

  /* "src/python/grpcio/grpc/_cython/_cygrpc/security.pyx.pxi":79
 *   properties = grpc_auth_context_property_iterator(auth_context)
 *   py_auth_context = {}
 *   while True:             # <<<<<<<<<<<<<<
 *     property = grpc_auth_property_iterator_next(&properties)
 *     if property == NULL:
 */
  while (1) {

    /* "src/python/grpcio/grpc/_cython/_cygrpc/security.pyx.pxi":80
 *   py_auth_context = {}
 *   while True:
 *     property = grpc_auth_property_iterator_next(&properties)             # <<<<<<<<<<<<<<
 *     if property == NULL:
 *       break
 */
    __pyx_v_property = grpc_auth_property_iterator_next((&__pyx_v_properties));

    /* "src/python/grpcio/grpc/_cython/_cygrpc/security.pyx.pxi":81
 *   while True:
 *     property = grpc_auth_property_iterator_next(&properties)
 *     if property == NULL:             # <<<<<<<<<<<<<<
 *       break
 *     if property.name != NULL and property.value != NULL:
 */
    __pyx_t_1 = ((__pyx_v_property == NULL) != 0);
    if (__pyx_t_1) {

      /* "src/python/grpcio/grpc/_cython/_cygrpc/security.pyx.pxi":82
 *     property = grpc_auth_property_iterator_next(&properties)
 *     if property == NULL:
 *       break             # <<<<<<<<<<<<<<
 *     if property.name != NULL and property.value != NULL:
 *       key = <bytes> property.name
 */
      goto __pyx_L5_break;

      /* "src/python/grpcio/grpc/_cython/_cygrpc/security.pyx.pxi":81
 *   while True:
 *     property = grpc_auth_property_iterator_next(&properties)
 *     if property == NULL:             # <<<<<<<<<<<<<<
 *       break
 *     if property.name != NULL and property.value != NULL:
 */
    }

    /* "src/python/grpcio/grpc/_cython/_cygrpc/security.pyx.pxi":83
 *     if property == NULL:
 *       break
 *     if property.name != NULL and property.value != NULL:             # <<<<<<<<<<<<<<
 *       key = <bytes> property.name
 *       if key in py_auth_context:
 */
    __pyx_t_3 = ((__pyx_v_property->name != NULL) != 0);
    if (__pyx_t_3) {
    } else {
      __pyx_t_1 = __pyx_t_3;
      goto __pyx_L8_bool_binop_done;
    }
    __pyx_t_3 = ((__pyx_v_property->value != NULL) != 0);
    __pyx_t_1 = __pyx_t_3;
    __pyx_L8_bool_binop_done:;
    if (__pyx_t_1) {

      /* "src/python/grpcio/grpc/_cython/_cygrpc/security.pyx.pxi":84
 *       break
 *     if property.name != NULL and property.value != NULL:
 *       key = <bytes> property.name             # <<<<<<<<<<<<<<
 *       if key in py_auth_context:
 *         py_auth_context[key].append(<bytes>(property.value))
 */
      __pyx_t_2 = __Pyx_PyBytes_FromString(__pyx_v_property->name); if (unlikely(!__pyx_t_2)) __PYX_ERR(17, 84, __pyx_L1_error)
      __Pyx_GOTREF(__pyx_t_2);
      __pyx_t_4 = __pyx_t_2;
      __Pyx_INCREF(__pyx_t_4);
      __Pyx_DECREF(__pyx_t_2); __pyx_t_2 = 0;
      __Pyx_XDECREF_SET(__pyx_v_key, ((PyObject*)__pyx_t_4));
      __pyx_t_4 = 0;

      /* "src/python/grpcio/grpc/_cython/_cygrpc/security.pyx.pxi":85
 *     if property.name != NULL and property.value != NULL:
 *       key = <bytes> property.name
 *       if key in py_auth_context:             # <<<<<<<<<<<<<<
 *         py_auth_context[key].append(<bytes>(property.value))
 *       else:
 */
      __pyx_t_1 = (__Pyx_PyDict_ContainsTF(__pyx_v_key, __pyx_v_py_auth_context, Py_EQ)); if (unlikely(__pyx_t_1 < 0)) __PYX_ERR(17, 85, __pyx_L1_error)
      __pyx_t_3 = (__pyx_t_1 != 0);
      if (__pyx_t_3) {

        /* "src/python/grpcio/grpc/_cython/_cygrpc/security.pyx.pxi":86
 *       key = <bytes> property.name
 *       if key in py_auth_context:
 *         py_auth_context[key].append(<bytes>(property.value))             # <<<<<<<<<<<<<<
 *       else:
 *         py_auth_context[key] = [<bytes> property.value]
 */
        __pyx_t_4 = __Pyx_PyDict_GetItem(__pyx_v_py_auth_context, __pyx_v_key); if (unlikely(!__pyx_t_4)) __PYX_ERR(17, 86, __pyx_L1_error)
        __Pyx_GOTREF(__pyx_t_4);
        __pyx_t_2 = __Pyx_PyBytes_FromString(__pyx_v_property->value); if (unlikely(!__pyx_t_2)) __PYX_ERR(17, 86, __pyx_L1_error)
        __Pyx_GOTREF(__pyx_t_2);
        __pyx_t_5 = __Pyx_PyObject_Append(__pyx_t_4, __pyx_t_2); if (unlikely(__pyx_t_5 == ((int)-1))) __PYX_ERR(17, 86, __pyx_L1_error)
        __Pyx_DECREF(__pyx_t_4); __pyx_t_4 = 0;
        __Pyx_DECREF(__pyx_t_2); __pyx_t_2 = 0;

        /* "src/python/grpcio/grpc/_cython/_cygrpc/security.pyx.pxi":85
 *     if property.name != NULL and property.value != NULL:
 *       key = <bytes> property.name
 *       if key in py_auth_context:             # <<<<<<<<<<<<<<
 *         py_auth_context[key].append(<bytes>(property.value))
 *       else:
 */
        goto __pyx_L10;
      }

      /* "src/python/grpcio/grpc/_cython/_cygrpc/security.pyx.pxi":88
 *         py_auth_context[key].append(<bytes>(property.value))
 *       else:
 *         py_auth_context[key] = [<bytes> property.value]             # <<<<<<<<<<<<<<
 *   grpc_auth_context_release(auth_context)
 *   return py_auth_context
 */
      /*else*/ {
        __pyx_t_2 = __Pyx_PyBytes_FromString(__pyx_v_property->value); if (unlikely(!__pyx_t_2)) __PYX_ERR(17, 88, __pyx_L1_error)
        __Pyx_GOTREF(__pyx_t_2);
        __pyx_t_4 = PyList_New(1); if (unlikely(!__pyx_t_4)) __PYX_ERR(17, 88, __pyx_L1_error)
        __Pyx_GOTREF(__pyx_t_4);
        __Pyx_INCREF(((PyObject*)__pyx_t_2));
        __Pyx_GIVEREF(__pyx_t_2);
        PyList_SET_ITEM(__pyx_t_4, 0, __pyx_t_2);
        __Pyx_DECREF(__pyx_t_2); __pyx_t_2 = 0;
        if (unlikely(PyDict_SetItem(__pyx_v_py_auth_context, __pyx_v_key, __pyx_t_4) < 0)) __PYX_ERR(17, 88, __pyx_L1_error)
        __Pyx_DECREF(__pyx_t_4); __pyx_t_4 = 0;
      }
      __pyx_L10:;

      /* "src/python/grpcio/grpc/_cython/_cygrpc/security.pyx.pxi":83
 *     if property == NULL:
 *       break
 *     if property.name != NULL and property.value != NULL:             # <<<<<<<<<<<<<<
 *       key = <bytes> property.name
 *       if key in py_auth_context:
 */
    }
  }
  __pyx_L5_break:;

  /* "src/python/grpcio/grpc/_cython/_cygrpc/security.pyx.pxi":89
 *       else:
 *         py_auth_context[key] = [<bytes> property.value]
 *   grpc_auth_context_release(auth_context)             # <<<<<<<<<<<<<<
 *   return py_auth_context
 * 
 */
  grpc_auth_context_release(__pyx_v_auth_context);

  /* "src/python/grpcio/grpc/_cython/_cygrpc/security.pyx.pxi":90
 *         py_auth_context[key] = [<bytes> property.value]
 *   grpc_auth_context_release(auth_context)
 *   return py_auth_context             # <<<<<<<<<<<<<<
 * 
 */
  __Pyx_XDECREF(__pyx_r);
  __Pyx_INCREF(__pyx_v_py_auth_context);
  __pyx_r = __pyx_v_py_auth_context;
  goto __pyx_L0;

  /* "src/python/grpcio/grpc/_cython/_cygrpc/security.pyx.pxi":69
 *   return key
 * 
 * def auth_context(Call call):             # <<<<<<<<<<<<<<
 *   cdef grpc_auth_context* auth_context
 *   cdef grpc_auth_property_iterator properties
 */

  /* function exit code */
  __pyx_L1_error:;
  __Pyx_XDECREF(__pyx_t_2);
  __Pyx_XDECREF(__pyx_t_4);
  __Pyx_AddTraceback("grpc._cython.cygrpc.auth_context", __pyx_clineno, __pyx_lineno, __pyx_filename);
  __pyx_r = NULL;
  __pyx_L0:;
  __Pyx_XDECREF(__pyx_v_py_auth_context);
  __Pyx_XDECREF(__pyx_v_key);
  __Pyx_XGIVEREF(__pyx_r);
  __Pyx_RefNannyFinishContext();
  return __pyx_r;
}

/* "src/python/grpcio/grpc/_cython/_cygrpc/server.pyx.pxi":26
 * cdef class Server:
 * 
 *   def __cinit__(self, object arguments):             # <<<<<<<<<<<<<<
 *     fork_handlers_and_grpc_init()
 *     self.references = []
 */

/* Python wrapper */
static int __pyx_pw_4grpc_7_cython_6cygrpc_6Server_1__cinit__(PyObject *__pyx_v_self, PyObject *__pyx_args, PyObject *__pyx_kwds); /*proto*/
static int __pyx_pw_4grpc_7_cython_6cygrpc_6Server_1__cinit__(PyObject *__pyx_v_self, PyObject *__pyx_args, PyObject *__pyx_kwds) {
  PyObject *__pyx_v_arguments = 0;
  int __pyx_r;
  __Pyx_RefNannyDeclarations
  __Pyx_RefNannySetupContext("__cinit__ (wrapper)", 0);
  {
    static PyObject **__pyx_pyargnames[] = {&__pyx_n_s_arguments,0};
    PyObject* values[1] = {0};
    if (unlikely(__pyx_kwds)) {
      Py_ssize_t kw_args;
      const Py_ssize_t pos_args = PyTuple_GET_SIZE(__pyx_args);
      switch (pos_args) {
        case  1: values[0] = PyTuple_GET_ITEM(__pyx_args, 0);
        CYTHON_FALLTHROUGH;
        case  0: break;
        default: goto __pyx_L5_argtuple_error;
      }
      kw_args = PyDict_Size(__pyx_kwds);
      switch (pos_args) {
        case  0:
        if (likely((values[0] = __Pyx_PyDict_GetItemStr(__pyx_kwds, __pyx_n_s_arguments)) != 0)) kw_args--;
        else goto __pyx_L5_argtuple_error;
      }
      if (unlikely(kw_args > 0)) {
        if (unlikely(__Pyx_ParseOptionalKeywords(__pyx_kwds, __pyx_pyargnames, 0, values, pos_args, "__cinit__") < 0)) __PYX_ERR(5, 26, __pyx_L3_error)
      }
    } else if (PyTuple_GET_SIZE(__pyx_args) != 1) {
      goto __pyx_L5_argtuple_error;
    } else {
      values[0] = PyTuple_GET_ITEM(__pyx_args, 0);
    }
    __pyx_v_arguments = values[0];
  }
  goto __pyx_L4_argument_unpacking_done;
  __pyx_L5_argtuple_error:;
  __Pyx_RaiseArgtupleInvalid("__cinit__", 1, 1, 1, PyTuple_GET_SIZE(__pyx_args)); __PYX_ERR(5, 26, __pyx_L3_error)
  __pyx_L3_error:;
  __Pyx_AddTraceback("grpc._cython.cygrpc.Server.__cinit__", __pyx_clineno, __pyx_lineno, __pyx_filename);
  __Pyx_RefNannyFinishContext();
  return -1;
  __pyx_L4_argument_unpacking_done:;
  __pyx_r = __pyx_pf_4grpc_7_cython_6cygrpc_6Server___cinit__(((struct __pyx_obj_4grpc_7_cython_6cygrpc_Server *)__pyx_v_self), __pyx_v_arguments);

  /* function exit code */
  __Pyx_RefNannyFinishContext();
  return __pyx_r;
}

static int __pyx_pf_4grpc_7_cython_6cygrpc_6Server___cinit__(struct __pyx_obj_4grpc_7_cython_6cygrpc_Server *__pyx_v_self, PyObject *__pyx_v_arguments) {
  struct __pyx_obj_4grpc_7_cython_6cygrpc__ChannelArgs *__pyx_v_channel_args = 0;
  int __pyx_r;
  __Pyx_RefNannyDeclarations
  PyObject *__pyx_t_1 = NULL;
  PyObject *__pyx_t_2 = NULL;
  PyObject *__pyx_t_3 = NULL;
  grpc_channel_args *__pyx_t_4;
  int __pyx_t_5;
  __Pyx_RefNannySetupContext("__cinit__", 0);

  /* "src/python/grpcio/grpc/_cython/_cygrpc/server.pyx.pxi":27
 * 
 *   def __cinit__(self, object arguments):
 *     fork_handlers_and_grpc_init()             # <<<<<<<<<<<<<<
 *     self.references = []
 *     self.registered_completion_queues = []
 */
  __pyx_t_2 = __Pyx_GetModuleGlobalName(__pyx_n_s_fork_handlers_and_grpc_init); if (unlikely(!__pyx_t_2)) __PYX_ERR(5, 27, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_2);
  __pyx_t_3 = NULL;
  if (CYTHON_UNPACK_METHODS && unlikely(PyMethod_Check(__pyx_t_2))) {
    __pyx_t_3 = PyMethod_GET_SELF(__pyx_t_2);
    if (likely(__pyx_t_3)) {
      PyObject* function = PyMethod_GET_FUNCTION(__pyx_t_2);
      __Pyx_INCREF(__pyx_t_3);
      __Pyx_INCREF(function);
      __Pyx_DECREF_SET(__pyx_t_2, function);
    }
  }
  if (__pyx_t_3) {
    __pyx_t_1 = __Pyx_PyObject_CallOneArg(__pyx_t_2, __pyx_t_3); if (unlikely(!__pyx_t_1)) __PYX_ERR(5, 27, __pyx_L1_error)
    __Pyx_DECREF(__pyx_t_3); __pyx_t_3 = 0;
  } else {
    __pyx_t_1 = __Pyx_PyObject_CallNoArg(__pyx_t_2); if (unlikely(!__pyx_t_1)) __PYX_ERR(5, 27, __pyx_L1_error)
  }
  __Pyx_GOTREF(__pyx_t_1);
  __Pyx_DECREF(__pyx_t_2); __pyx_t_2 = 0;
  __Pyx_DECREF(__pyx_t_1); __pyx_t_1 = 0;

  /* "src/python/grpcio/grpc/_cython/_cygrpc/server.pyx.pxi":28
 *   def __cinit__(self, object arguments):
 *     fork_handlers_and_grpc_init()
 *     self.references = []             # <<<<<<<<<<<<<<
 *     self.registered_completion_queues = []
 *     self.is_started = False
 */
  __pyx_t_1 = PyList_New(0); if (unlikely(!__pyx_t_1)) __PYX_ERR(5, 28, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_1);
  __Pyx_GIVEREF(__pyx_t_1);
  __Pyx_GOTREF(__pyx_v_self->references);
  __Pyx_DECREF(__pyx_v_self->references);
  __pyx_v_self->references = ((PyObject*)__pyx_t_1);
  __pyx_t_1 = 0;

  /* "src/python/grpcio/grpc/_cython/_cygrpc/server.pyx.pxi":29
 *     fork_handlers_and_grpc_init()
 *     self.references = []
 *     self.registered_completion_queues = []             # <<<<<<<<<<<<<<
 *     self.is_started = False
 *     self.is_shutting_down = False
 */
  __pyx_t_1 = PyList_New(0); if (unlikely(!__pyx_t_1)) __PYX_ERR(5, 29, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_1);
  __Pyx_GIVEREF(__pyx_t_1);
  __Pyx_GOTREF(__pyx_v_self->registered_completion_queues);
  __Pyx_DECREF(__pyx_v_self->registered_completion_queues);
  __pyx_v_self->registered_completion_queues = ((PyObject*)__pyx_t_1);
  __pyx_t_1 = 0;

  /* "src/python/grpcio/grpc/_cython/_cygrpc/server.pyx.pxi":30
 *     self.references = []
 *     self.registered_completion_queues = []
 *     self.is_started = False             # <<<<<<<<<<<<<<
 *     self.is_shutting_down = False
 *     self.is_shutdown = False
 */
  __pyx_v_self->is_started = 0;

  /* "src/python/grpcio/grpc/_cython/_cygrpc/server.pyx.pxi":31
 *     self.registered_completion_queues = []
 *     self.is_started = False
 *     self.is_shutting_down = False             # <<<<<<<<<<<<<<
 *     self.is_shutdown = False
 *     self.c_server = NULL
 */
  __pyx_v_self->is_shutting_down = 0;

  /* "src/python/grpcio/grpc/_cython/_cygrpc/server.pyx.pxi":32
 *     self.is_started = False
 *     self.is_shutting_down = False
 *     self.is_shutdown = False             # <<<<<<<<<<<<<<
 *     self.c_server = NULL
 *     self._vtable = _VTable()
 */
  __pyx_v_self->is_shutdown = 0;

  /* "src/python/grpcio/grpc/_cython/_cygrpc/server.pyx.pxi":33
 *     self.is_shutting_down = False
 *     self.is_shutdown = False
 *     self.c_server = NULL             # <<<<<<<<<<<<<<
 *     self._vtable = _VTable()
 *     cdef _ChannelArgs channel_args = _ChannelArgs(arguments, self._vtable)
 */
  __pyx_v_self->c_server = NULL;

  /* "src/python/grpcio/grpc/_cython/_cygrpc/server.pyx.pxi":34
 *     self.is_shutdown = False
 *     self.c_server = NULL
 *     self._vtable = _VTable()             # <<<<<<<<<<<<<<
 *     cdef _ChannelArgs channel_args = _ChannelArgs(arguments, self._vtable)
 *     self.c_server = grpc_server_create(channel_args.c_args(), NULL)
 */
  __pyx_t_1 = __Pyx_PyObject_CallNoArg(((PyObject *)__pyx_ptype_4grpc_7_cython_6cygrpc__VTable)); if (unlikely(!__pyx_t_1)) __PYX_ERR(5, 34, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_1);
  __Pyx_GIVEREF(__pyx_t_1);
  __Pyx_GOTREF(__pyx_v_self->_vtable);
  __Pyx_DECREF(((PyObject *)__pyx_v_self->_vtable));
  __pyx_v_self->_vtable = ((struct __pyx_obj_4grpc_7_cython_6cygrpc__VTable *)__pyx_t_1);
  __pyx_t_1 = 0;

  /* "src/python/grpcio/grpc/_cython/_cygrpc/server.pyx.pxi":35
 *     self.c_server = NULL
 *     self._vtable = _VTable()
 *     cdef _ChannelArgs channel_args = _ChannelArgs(arguments, self._vtable)             # <<<<<<<<<<<<<<
 *     self.c_server = grpc_server_create(channel_args.c_args(), NULL)
 *     self.references.append(arguments)
 */
  __pyx_t_1 = PyTuple_New(2); if (unlikely(!__pyx_t_1)) __PYX_ERR(5, 35, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_1);
  __Pyx_INCREF(__pyx_v_arguments);
  __Pyx_GIVEREF(__pyx_v_arguments);
  PyTuple_SET_ITEM(__pyx_t_1, 0, __pyx_v_arguments);
  __Pyx_INCREF(((PyObject *)__pyx_v_self->_vtable));
  __Pyx_GIVEREF(((PyObject *)__pyx_v_self->_vtable));
  PyTuple_SET_ITEM(__pyx_t_1, 1, ((PyObject *)__pyx_v_self->_vtable));
  __pyx_t_2 = __Pyx_PyObject_Call(((PyObject *)__pyx_ptype_4grpc_7_cython_6cygrpc__ChannelArgs), __pyx_t_1, NULL); if (unlikely(!__pyx_t_2)) __PYX_ERR(5, 35, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_2);
  __Pyx_DECREF(__pyx_t_1); __pyx_t_1 = 0;
  __pyx_v_channel_args = ((struct __pyx_obj_4grpc_7_cython_6cygrpc__ChannelArgs *)__pyx_t_2);
  __pyx_t_2 = 0;

  /* "src/python/grpcio/grpc/_cython/_cygrpc/server.pyx.pxi":36
 *     self._vtable = _VTable()
 *     cdef _ChannelArgs channel_args = _ChannelArgs(arguments, self._vtable)
 *     self.c_server = grpc_server_create(channel_args.c_args(), NULL)             # <<<<<<<<<<<<<<
 *     self.references.append(arguments)
 * 
 */
  __pyx_t_4 = ((struct __pyx_vtabstruct_4grpc_7_cython_6cygrpc__ChannelArgs *)__pyx_v_channel_args->__pyx_vtab)->c_args(__pyx_v_channel_args); if (unlikely(PyErr_Occurred())) __PYX_ERR(5, 36, __pyx_L1_error)
  __pyx_v_self->c_server = grpc_server_create(__pyx_t_4, NULL);

  /* "src/python/grpcio/grpc/_cython/_cygrpc/server.pyx.pxi":37
 *     cdef _ChannelArgs channel_args = _ChannelArgs(arguments, self._vtable)
 *     self.c_server = grpc_server_create(channel_args.c_args(), NULL)
 *     self.references.append(arguments)             # <<<<<<<<<<<<<<
 * 
 *   def request_call(
 */
  if (unlikely(__pyx_v_self->references == Py_None)) {
    PyErr_Format(PyExc_AttributeError, "'NoneType' object has no attribute '%.30s'", "append");
    __PYX_ERR(5, 37, __pyx_L1_error)
  }
  __pyx_t_5 = __Pyx_PyList_Append(__pyx_v_self->references, __pyx_v_arguments); if (unlikely(__pyx_t_5 == ((int)-1))) __PYX_ERR(5, 37, __pyx_L1_error)

  /* "src/python/grpcio/grpc/_cython/_cygrpc/server.pyx.pxi":26
 * cdef class Server:
 * 
 *   def __cinit__(self, object arguments):             # <<<<<<<<<<<<<<
 *     fork_handlers_and_grpc_init()
 *     self.references = []
 */

  /* function exit code */
  __pyx_r = 0;
  goto __pyx_L0;
  __pyx_L1_error:;
  __Pyx_XDECREF(__pyx_t_1);
  __Pyx_XDECREF(__pyx_t_2);
  __Pyx_XDECREF(__pyx_t_3);
  __Pyx_AddTraceback("grpc._cython.cygrpc.Server.__cinit__", __pyx_clineno, __pyx_lineno, __pyx_filename);
  __pyx_r = -1;
  __pyx_L0:;
  __Pyx_XDECREF((PyObject *)__pyx_v_channel_args);
  __Pyx_RefNannyFinishContext();
  return __pyx_r;
}

/* "src/python/grpcio/grpc/_cython/_cygrpc/server.pyx.pxi":39
 *     self.references.append(arguments)
 * 
 *   def request_call(             # <<<<<<<<<<<<<<
 *       self, CompletionQueue call_queue not None,
 *       CompletionQueue server_queue not None, tag):
 */

/* Python wrapper */
static PyObject *__pyx_pw_4grpc_7_cython_6cygrpc_6Server_3request_call(PyObject *__pyx_v_self, PyObject *__pyx_args, PyObject *__pyx_kwds); /*proto*/
static PyObject *__pyx_pw_4grpc_7_cython_6cygrpc_6Server_3request_call(PyObject *__pyx_v_self, PyObject *__pyx_args, PyObject *__pyx_kwds) {
  struct __pyx_obj_4grpc_7_cython_6cygrpc_CompletionQueue *__pyx_v_call_queue = 0;
  struct __pyx_obj_4grpc_7_cython_6cygrpc_CompletionQueue *__pyx_v_server_queue = 0;
  PyObject *__pyx_v_tag = 0;
  PyObject *__pyx_r = 0;
  __Pyx_RefNannyDeclarations
  __Pyx_RefNannySetupContext("request_call (wrapper)", 0);
  {
    static PyObject **__pyx_pyargnames[] = {&__pyx_n_s_call_queue,&__pyx_n_s_server_queue,&__pyx_n_s_tag,0};
    PyObject* values[3] = {0,0,0};
    if (unlikely(__pyx_kwds)) {
      Py_ssize_t kw_args;
      const Py_ssize_t pos_args = PyTuple_GET_SIZE(__pyx_args);
      switch (pos_args) {
        case  3: values[2] = PyTuple_GET_ITEM(__pyx_args, 2);
        CYTHON_FALLTHROUGH;
        case  2: values[1] = PyTuple_GET_ITEM(__pyx_args, 1);
        CYTHON_FALLTHROUGH;
        case  1: values[0] = PyTuple_GET_ITEM(__pyx_args, 0);
        CYTHON_FALLTHROUGH;
        case  0: break;
        default: goto __pyx_L5_argtuple_error;
      }
      kw_args = PyDict_Size(__pyx_kwds);
      switch (pos_args) {
        case  0:
        if (likely((values[0] = __Pyx_PyDict_GetItemStr(__pyx_kwds, __pyx_n_s_call_queue)) != 0)) kw_args--;
        else goto __pyx_L5_argtuple_error;
        CYTHON_FALLTHROUGH;
        case  1:
        if (likely((values[1] = __Pyx_PyDict_GetItemStr(__pyx_kwds, __pyx_n_s_server_queue)) != 0)) kw_args--;
        else {
          __Pyx_RaiseArgtupleInvalid("request_call", 1, 3, 3, 1); __PYX_ERR(5, 39, __pyx_L3_error)
        }
        CYTHON_FALLTHROUGH;
        case  2:
        if (likely((values[2] = __Pyx_PyDict_GetItemStr(__pyx_kwds, __pyx_n_s_tag)) != 0)) kw_args--;
        else {
          __Pyx_RaiseArgtupleInvalid("request_call", 1, 3, 3, 2); __PYX_ERR(5, 39, __pyx_L3_error)
        }
      }
      if (unlikely(kw_args > 0)) {
        if (unlikely(__Pyx_ParseOptionalKeywords(__pyx_kwds, __pyx_pyargnames, 0, values, pos_args, "request_call") < 0)) __PYX_ERR(5, 39, __pyx_L3_error)
      }
    } else if (PyTuple_GET_SIZE(__pyx_args) != 3) {
      goto __pyx_L5_argtuple_error;
    } else {
      values[0] = PyTuple_GET_ITEM(__pyx_args, 0);
      values[1] = PyTuple_GET_ITEM(__pyx_args, 1);
      values[2] = PyTuple_GET_ITEM(__pyx_args, 2);
    }
    __pyx_v_call_queue = ((struct __pyx_obj_4grpc_7_cython_6cygrpc_CompletionQueue *)values[0]);
    __pyx_v_server_queue = ((struct __pyx_obj_4grpc_7_cython_6cygrpc_CompletionQueue *)values[1]);
    __pyx_v_tag = values[2];
  }
  goto __pyx_L4_argument_unpacking_done;
  __pyx_L5_argtuple_error:;
  __Pyx_RaiseArgtupleInvalid("request_call", 1, 3, 3, PyTuple_GET_SIZE(__pyx_args)); __PYX_ERR(5, 39, __pyx_L3_error)
  __pyx_L3_error:;
  __Pyx_AddTraceback("grpc._cython.cygrpc.Server.request_call", __pyx_clineno, __pyx_lineno, __pyx_filename);
  __Pyx_RefNannyFinishContext();
  return NULL;
  __pyx_L4_argument_unpacking_done:;
  if (unlikely(!__Pyx_ArgTypeTest(((PyObject *)__pyx_v_call_queue), __pyx_ptype_4grpc_7_cython_6cygrpc_CompletionQueue, 0, "call_queue", 0))) __PYX_ERR(5, 40, __pyx_L1_error)
  if (unlikely(!__Pyx_ArgTypeTest(((PyObject *)__pyx_v_server_queue), __pyx_ptype_4grpc_7_cython_6cygrpc_CompletionQueue, 0, "server_queue", 0))) __PYX_ERR(5, 41, __pyx_L1_error)
  __pyx_r = __pyx_pf_4grpc_7_cython_6cygrpc_6Server_2request_call(((struct __pyx_obj_4grpc_7_cython_6cygrpc_Server *)__pyx_v_self), __pyx_v_call_queue, __pyx_v_server_queue, __pyx_v_tag);

  /* function exit code */
  goto __pyx_L0;
  __pyx_L1_error:;
  __pyx_r = NULL;
  __pyx_L0:;
  __Pyx_RefNannyFinishContext();
  return __pyx_r;
}

static PyObject *__pyx_pf_4grpc_7_cython_6cygrpc_6Server_2request_call(struct __pyx_obj_4grpc_7_cython_6cygrpc_Server *__pyx_v_self, struct __pyx_obj_4grpc_7_cython_6cygrpc_CompletionQueue *__pyx_v_call_queue, struct __pyx_obj_4grpc_7_cython_6cygrpc_CompletionQueue *__pyx_v_server_queue, PyObject *__pyx_v_tag) {
  struct __pyx_obj_4grpc_7_cython_6cygrpc__RequestCallTag *__pyx_v_request_call_tag = 0;
  PyObject *__pyx_r = NULL;
  __Pyx_RefNannyDeclarations
  int __pyx_t_1;
  int __pyx_t_2;
  PyObject *__pyx_t_3 = NULL;
  PyObject *__pyx_t_4 = NULL;
  __Pyx_RefNannySetupContext("request_call", 0);

  /* "src/python/grpcio/grpc/_cython/_cygrpc/server.pyx.pxi":42
 *       self, CompletionQueue call_queue not None,
 *       CompletionQueue server_queue not None, tag):
 *     if not self.is_started or self.is_shutting_down:             # <<<<<<<<<<<<<<
 *       raise ValueError("server must be started and not shutting down")
 *     if server_queue not in self.registered_completion_queues:
 */
  __pyx_t_2 = ((!(__pyx_v_self->is_started != 0)) != 0);
  if (!__pyx_t_2) {
  } else {
    __pyx_t_1 = __pyx_t_2;
    goto __pyx_L4_bool_binop_done;
  }
  __pyx_t_2 = (__pyx_v_self->is_shutting_down != 0);
  __pyx_t_1 = __pyx_t_2;
  __pyx_L4_bool_binop_done:;
  if (unlikely(__pyx_t_1)) {

    /* "src/python/grpcio/grpc/_cython/_cygrpc/server.pyx.pxi":43
 *       CompletionQueue server_queue not None, tag):
 *     if not self.is_started or self.is_shutting_down:
 *       raise ValueError("server must be started and not shutting down")             # <<<<<<<<<<<<<<
 *     if server_queue not in self.registered_completion_queues:
 *       raise ValueError("server_queue must be a registered completion queue")
 */
    __pyx_t_3 = __Pyx_PyObject_Call(__pyx_builtin_ValueError, __pyx_tuple__109, NULL); if (unlikely(!__pyx_t_3)) __PYX_ERR(5, 43, __pyx_L1_error)
    __Pyx_GOTREF(__pyx_t_3);
    __Pyx_Raise(__pyx_t_3, 0, 0, 0);
    __Pyx_DECREF(__pyx_t_3); __pyx_t_3 = 0;
    __PYX_ERR(5, 43, __pyx_L1_error)

    /* "src/python/grpcio/grpc/_cython/_cygrpc/server.pyx.pxi":42
 *       self, CompletionQueue call_queue not None,
 *       CompletionQueue server_queue not None, tag):
 *     if not self.is_started or self.is_shutting_down:             # <<<<<<<<<<<<<<
 *       raise ValueError("server must be started and not shutting down")
 *     if server_queue not in self.registered_completion_queues:
 */
  }

  /* "src/python/grpcio/grpc/_cython/_cygrpc/server.pyx.pxi":44
 *     if not self.is_started or self.is_shutting_down:
 *       raise ValueError("server must be started and not shutting down")
 *     if server_queue not in self.registered_completion_queues:             # <<<<<<<<<<<<<<
 *       raise ValueError("server_queue must be a registered completion queue")
 *     cdef _RequestCallTag request_call_tag = _RequestCallTag(tag, self._vtable)
 */
  __pyx_t_1 = (__Pyx_PySequence_ContainsTF(((PyObject *)__pyx_v_server_queue), __pyx_v_self->registered_completion_queues, Py_NE)); if (unlikely(__pyx_t_1 < 0)) __PYX_ERR(5, 44, __pyx_L1_error)
  __pyx_t_2 = (__pyx_t_1 != 0);
  if (unlikely(__pyx_t_2)) {

    /* "src/python/grpcio/grpc/_cython/_cygrpc/server.pyx.pxi":45
 *       raise ValueError("server must be started and not shutting down")
 *     if server_queue not in self.registered_completion_queues:
 *       raise ValueError("server_queue must be a registered completion queue")             # <<<<<<<<<<<<<<
 *     cdef _RequestCallTag request_call_tag = _RequestCallTag(tag, self._vtable)
 *     request_call_tag.prepare()
 */
    __pyx_t_3 = __Pyx_PyObject_Call(__pyx_builtin_ValueError, __pyx_tuple__110, NULL); if (unlikely(!__pyx_t_3)) __PYX_ERR(5, 45, __pyx_L1_error)
    __Pyx_GOTREF(__pyx_t_3);
    __Pyx_Raise(__pyx_t_3, 0, 0, 0);
    __Pyx_DECREF(__pyx_t_3); __pyx_t_3 = 0;
    __PYX_ERR(5, 45, __pyx_L1_error)

    /* "src/python/grpcio/grpc/_cython/_cygrpc/server.pyx.pxi":44
 *     if not self.is_started or self.is_shutting_down:
 *       raise ValueError("server must be started and not shutting down")
 *     if server_queue not in self.registered_completion_queues:             # <<<<<<<<<<<<<<
 *       raise ValueError("server_queue must be a registered completion queue")
 *     cdef _RequestCallTag request_call_tag = _RequestCallTag(tag, self._vtable)
 */
  }

  /* "src/python/grpcio/grpc/_cython/_cygrpc/server.pyx.pxi":46
 *     if server_queue not in self.registered_completion_queues:
 *       raise ValueError("server_queue must be a registered completion queue")
 *     cdef _RequestCallTag request_call_tag = _RequestCallTag(tag, self._vtable)             # <<<<<<<<<<<<<<
 *     request_call_tag.prepare()
 *     cpython.Py_INCREF(request_call_tag)
 */
  __pyx_t_3 = PyTuple_New(2); if (unlikely(!__pyx_t_3)) __PYX_ERR(5, 46, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_3);
  __Pyx_INCREF(__pyx_v_tag);
  __Pyx_GIVEREF(__pyx_v_tag);
  PyTuple_SET_ITEM(__pyx_t_3, 0, __pyx_v_tag);
  __Pyx_INCREF(((PyObject *)__pyx_v_self->_vtable));
  __Pyx_GIVEREF(((PyObject *)__pyx_v_self->_vtable));
  PyTuple_SET_ITEM(__pyx_t_3, 1, ((PyObject *)__pyx_v_self->_vtable));
  __pyx_t_4 = __Pyx_PyObject_Call(((PyObject *)__pyx_ptype_4grpc_7_cython_6cygrpc__RequestCallTag), __pyx_t_3, NULL); if (unlikely(!__pyx_t_4)) __PYX_ERR(5, 46, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_4);
  __Pyx_DECREF(__pyx_t_3); __pyx_t_3 = 0;
  __pyx_v_request_call_tag = ((struct __pyx_obj_4grpc_7_cython_6cygrpc__RequestCallTag *)__pyx_t_4);
  __pyx_t_4 = 0;

  /* "src/python/grpcio/grpc/_cython/_cygrpc/server.pyx.pxi":47
 *       raise ValueError("server_queue must be a registered completion queue")
 *     cdef _RequestCallTag request_call_tag = _RequestCallTag(tag, self._vtable)
 *     request_call_tag.prepare()             # <<<<<<<<<<<<<<
 *     cpython.Py_INCREF(request_call_tag)
 *     return grpc_server_request_call(
 */
  ((struct __pyx_vtabstruct_4grpc_7_cython_6cygrpc__RequestCallTag *)__pyx_v_request_call_tag->__pyx_base.__pyx_vtab)->prepare(__pyx_v_request_call_tag); if (unlikely(PyErr_Occurred())) __PYX_ERR(5, 47, __pyx_L1_error)

  /* "src/python/grpcio/grpc/_cython/_cygrpc/server.pyx.pxi":48
 *     cdef _RequestCallTag request_call_tag = _RequestCallTag(tag, self._vtable)
 *     request_call_tag.prepare()
 *     cpython.Py_INCREF(request_call_tag)             # <<<<<<<<<<<<<<
 *     return grpc_server_request_call(
 *         self.c_server, &request_call_tag.call.c_call,
 */
  Py_INCREF(((PyObject *)__pyx_v_request_call_tag));

  /* "src/python/grpcio/grpc/_cython/_cygrpc/server.pyx.pxi":49
 *     request_call_tag.prepare()
 *     cpython.Py_INCREF(request_call_tag)
 *     return grpc_server_request_call(             # <<<<<<<<<<<<<<
 *         self.c_server, &request_call_tag.call.c_call,
 *         &request_call_tag.call_details.c_details,
 */
  __Pyx_XDECREF(__pyx_r);

  /* "src/python/grpcio/grpc/_cython/_cygrpc/server.pyx.pxi":54
 *         &request_call_tag.c_invocation_metadata,
 *         call_queue.c_completion_queue, server_queue.c_completion_queue,
 *         <cpython.PyObject *>request_call_tag)             # <<<<<<<<<<<<<<
 * 
 *   def register_completion_queue(
 */
  __pyx_t_4 = __Pyx_PyInt_From_grpc_call_error(grpc_server_request_call(__pyx_v_self->c_server, (&__pyx_v_request_call_tag->call->c_call), (&__pyx_v_request_call_tag->call_details->c_details), (&__pyx_v_request_call_tag->c_invocation_metadata), __pyx_v_call_queue->c_completion_queue, __pyx_v_server_queue->c_completion_queue, ((PyObject *)__pyx_v_request_call_tag))); if (unlikely(!__pyx_t_4)) __PYX_ERR(5, 49, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_4);
  __pyx_r = __pyx_t_4;
  __pyx_t_4 = 0;
  goto __pyx_L0;

  /* "src/python/grpcio/grpc/_cython/_cygrpc/server.pyx.pxi":39
 *     self.references.append(arguments)
 * 
 *   def request_call(             # <<<<<<<<<<<<<<
 *       self, CompletionQueue call_queue not None,
 *       CompletionQueue server_queue not None, tag):
 */

  /* function exit code */
  __pyx_L1_error:;
  __Pyx_XDECREF(__pyx_t_3);
  __Pyx_XDECREF(__pyx_t_4);
  __Pyx_AddTraceback("grpc._cython.cygrpc.Server.request_call", __pyx_clineno, __pyx_lineno, __pyx_filename);
  __pyx_r = NULL;
  __pyx_L0:;
  __Pyx_XDECREF((PyObject *)__pyx_v_request_call_tag);
  __Pyx_XGIVEREF(__pyx_r);
  __Pyx_RefNannyFinishContext();
  return __pyx_r;
}

/* "src/python/grpcio/grpc/_cython/_cygrpc/server.pyx.pxi":56
 *         <cpython.PyObject *>request_call_tag)
 * 
 *   def register_completion_queue(             # <<<<<<<<<<<<<<
 *       self, CompletionQueue queue not None):
 *     if self.is_started:
 */

/* Python wrapper */
static PyObject *__pyx_pw_4grpc_7_cython_6cygrpc_6Server_5register_completion_queue(PyObject *__pyx_v_self, PyObject *__pyx_v_queue); /*proto*/
static PyObject *__pyx_pw_4grpc_7_cython_6cygrpc_6Server_5register_completion_queue(PyObject *__pyx_v_self, PyObject *__pyx_v_queue) {
  PyObject *__pyx_r = 0;
  __Pyx_RefNannyDeclarations
  __Pyx_RefNannySetupContext("register_completion_queue (wrapper)", 0);
  if (unlikely(!__Pyx_ArgTypeTest(((PyObject *)__pyx_v_queue), __pyx_ptype_4grpc_7_cython_6cygrpc_CompletionQueue, 0, "queue", 0))) __PYX_ERR(5, 57, __pyx_L1_error)
  __pyx_r = __pyx_pf_4grpc_7_cython_6cygrpc_6Server_4register_completion_queue(((struct __pyx_obj_4grpc_7_cython_6cygrpc_Server *)__pyx_v_self), ((struct __pyx_obj_4grpc_7_cython_6cygrpc_CompletionQueue *)__pyx_v_queue));

  /* function exit code */
  goto __pyx_L0;
  __pyx_L1_error:;
  __pyx_r = NULL;
  __pyx_L0:;
  __Pyx_RefNannyFinishContext();
  return __pyx_r;
}

static PyObject *__pyx_pf_4grpc_7_cython_6cygrpc_6Server_4register_completion_queue(struct __pyx_obj_4grpc_7_cython_6cygrpc_Server *__pyx_v_self, struct __pyx_obj_4grpc_7_cython_6cygrpc_CompletionQueue *__pyx_v_queue) {
  PyObject *__pyx_r = NULL;
  __Pyx_RefNannyDeclarations
  int __pyx_t_1;
  PyObject *__pyx_t_2 = NULL;
  int __pyx_t_3;
  __Pyx_RefNannySetupContext("register_completion_queue", 0);

  /* "src/python/grpcio/grpc/_cython/_cygrpc/server.pyx.pxi":58
 *   def register_completion_queue(
 *       self, CompletionQueue queue not None):
 *     if self.is_started:             # <<<<<<<<<<<<<<
 *       raise ValueError("cannot register completion queues after start")
 *     with nogil:
 */
  __pyx_t_1 = (__pyx_v_self->is_started != 0);
  if (unlikely(__pyx_t_1)) {

    /* "src/python/grpcio/grpc/_cython/_cygrpc/server.pyx.pxi":59
 *       self, CompletionQueue queue not None):
 *     if self.is_started:
 *       raise ValueError("cannot register completion queues after start")             # <<<<<<<<<<<<<<
 *     with nogil:
 *       grpc_server_register_completion_queue(
 */
    __pyx_t_2 = __Pyx_PyObject_Call(__pyx_builtin_ValueError, __pyx_tuple__111, NULL); if (unlikely(!__pyx_t_2)) __PYX_ERR(5, 59, __pyx_L1_error)
    __Pyx_GOTREF(__pyx_t_2);
    __Pyx_Raise(__pyx_t_2, 0, 0, 0);
    __Pyx_DECREF(__pyx_t_2); __pyx_t_2 = 0;
    __PYX_ERR(5, 59, __pyx_L1_error)

    /* "src/python/grpcio/grpc/_cython/_cygrpc/server.pyx.pxi":58
 *   def register_completion_queue(
 *       self, CompletionQueue queue not None):
 *     if self.is_started:             # <<<<<<<<<<<<<<
 *       raise ValueError("cannot register completion queues after start")
 *     with nogil:
 */
  }

  /* "src/python/grpcio/grpc/_cython/_cygrpc/server.pyx.pxi":60
 *     if self.is_started:
 *       raise ValueError("cannot register completion queues after start")
 *     with nogil:             # <<<<<<<<<<<<<<
 *       grpc_server_register_completion_queue(
 *           self.c_server, queue.c_completion_queue, NULL)
 */
  {
      #ifdef WITH_THREAD
      PyThreadState *_save;
      Py_UNBLOCK_THREADS
      __Pyx_FastGIL_Remember();
      #endif
      /*try:*/ {

        /* "src/python/grpcio/grpc/_cython/_cygrpc/server.pyx.pxi":61
 *       raise ValueError("cannot register completion queues after start")
 *     with nogil:
 *       grpc_server_register_completion_queue(             # <<<<<<<<<<<<<<
 *           self.c_server, queue.c_completion_queue, NULL)
 *     self.registered_completion_queues.append(queue)
 */
        grpc_server_register_completion_queue(__pyx_v_self->c_server, __pyx_v_queue->c_completion_queue, NULL);
      }

      /* "src/python/grpcio/grpc/_cython/_cygrpc/server.pyx.pxi":60
 *     if self.is_started:
 *       raise ValueError("cannot register completion queues after start")
 *     with nogil:             # <<<<<<<<<<<<<<
 *       grpc_server_register_completion_queue(
 *           self.c_server, queue.c_completion_queue, NULL)
 */
      /*finally:*/ {
        /*normal exit:*/{
          #ifdef WITH_THREAD
          __Pyx_FastGIL_Forget();
          Py_BLOCK_THREADS
          #endif
          goto __pyx_L6;
        }
        __pyx_L6:;
      }
  }

  /* "src/python/grpcio/grpc/_cython/_cygrpc/server.pyx.pxi":63
 *       grpc_server_register_completion_queue(
 *           self.c_server, queue.c_completion_queue, NULL)
 *     self.registered_completion_queues.append(queue)             # <<<<<<<<<<<<<<
 * 
 *   def start(self):
 */
  if (unlikely(__pyx_v_self->registered_completion_queues == Py_None)) {
    PyErr_Format(PyExc_AttributeError, "'NoneType' object has no attribute '%.30s'", "append");
    __PYX_ERR(5, 63, __pyx_L1_error)
  }
  __pyx_t_3 = __Pyx_PyList_Append(__pyx_v_self->registered_completion_queues, ((PyObject *)__pyx_v_queue)); if (unlikely(__pyx_t_3 == ((int)-1))) __PYX_ERR(5, 63, __pyx_L1_error)

  /* "src/python/grpcio/grpc/_cython/_cygrpc/server.pyx.pxi":56
 *         <cpython.PyObject *>request_call_tag)
 * 
 *   def register_completion_queue(             # <<<<<<<<<<<<<<
 *       self, CompletionQueue queue not None):
 *     if self.is_started:
 */

  /* function exit code */
  __pyx_r = Py_None; __Pyx_INCREF(Py_None);
  goto __pyx_L0;
  __pyx_L1_error:;
  __Pyx_XDECREF(__pyx_t_2);
  __Pyx_AddTraceback("grpc._cython.cygrpc.Server.register_completion_queue", __pyx_clineno, __pyx_lineno, __pyx_filename);
  __pyx_r = NULL;
  __pyx_L0:;
  __Pyx_XGIVEREF(__pyx_r);
  __Pyx_RefNannyFinishContext();
  return __pyx_r;
}

/* "src/python/grpcio/grpc/_cython/_cygrpc/server.pyx.pxi":65
 *     self.registered_completion_queues.append(queue)
 * 
 *   def start(self):             # <<<<<<<<<<<<<<
 *     if self.is_started:
 *       raise ValueError("the server has already started")
 */

/* Python wrapper */
static PyObject *__pyx_pw_4grpc_7_cython_6cygrpc_6Server_7start(PyObject *__pyx_v_self, CYTHON_UNUSED PyObject *unused); /*proto*/
static PyObject *__pyx_pw_4grpc_7_cython_6cygrpc_6Server_7start(PyObject *__pyx_v_self, CYTHON_UNUSED PyObject *unused) {
  PyObject *__pyx_r = 0;
  __Pyx_RefNannyDeclarations
  __Pyx_RefNannySetupContext("start (wrapper)", 0);
  __pyx_r = __pyx_pf_4grpc_7_cython_6cygrpc_6Server_6start(((struct __pyx_obj_4grpc_7_cython_6cygrpc_Server *)__pyx_v_self));

  /* function exit code */
  __Pyx_RefNannyFinishContext();
  return __pyx_r;
}

static PyObject *__pyx_pf_4grpc_7_cython_6cygrpc_6Server_6start(struct __pyx_obj_4grpc_7_cython_6cygrpc_Server *__pyx_v_self) {
  PyObject *__pyx_r = NULL;
  __Pyx_RefNannyDeclarations
  int __pyx_t_1;
  PyObject *__pyx_t_2 = NULL;
  PyObject *__pyx_t_3 = NULL;
  PyObject *__pyx_t_4 = NULL;
  PyObject *__pyx_t_5 = NULL;
  PyObject *__pyx_t_6 = NULL;
  __Pyx_RefNannySetupContext("start", 0);

  /* "src/python/grpcio/grpc/_cython/_cygrpc/server.pyx.pxi":66
 * 
 *   def start(self):
 *     if self.is_started:             # <<<<<<<<<<<<<<
 *       raise ValueError("the server has already started")
 *     self.backup_shutdown_queue = CompletionQueue(shutdown_cq=True)
 */
  __pyx_t_1 = (__pyx_v_self->is_started != 0);
  if (unlikely(__pyx_t_1)) {

    /* "src/python/grpcio/grpc/_cython/_cygrpc/server.pyx.pxi":67
 *   def start(self):
 *     if self.is_started:
 *       raise ValueError("the server has already started")             # <<<<<<<<<<<<<<
 *     self.backup_shutdown_queue = CompletionQueue(shutdown_cq=True)
 *     self.register_completion_queue(self.backup_shutdown_queue)
 */
    __pyx_t_2 = __Pyx_PyObject_Call(__pyx_builtin_ValueError, __pyx_tuple__112, NULL); if (unlikely(!__pyx_t_2)) __PYX_ERR(5, 67, __pyx_L1_error)
    __Pyx_GOTREF(__pyx_t_2);
    __Pyx_Raise(__pyx_t_2, 0, 0, 0);
    __Pyx_DECREF(__pyx_t_2); __pyx_t_2 = 0;
    __PYX_ERR(5, 67, __pyx_L1_error)

    /* "src/python/grpcio/grpc/_cython/_cygrpc/server.pyx.pxi":66
 * 
 *   def start(self):
 *     if self.is_started:             # <<<<<<<<<<<<<<
 *       raise ValueError("the server has already started")
 *     self.backup_shutdown_queue = CompletionQueue(shutdown_cq=True)
 */
  }

  /* "src/python/grpcio/grpc/_cython/_cygrpc/server.pyx.pxi":68
 *     if self.is_started:
 *       raise ValueError("the server has already started")
 *     self.backup_shutdown_queue = CompletionQueue(shutdown_cq=True)             # <<<<<<<<<<<<<<
 *     self.register_completion_queue(self.backup_shutdown_queue)
 *     self.is_started = True
 */
  __pyx_t_2 = __Pyx_PyDict_NewPresized(1); if (unlikely(!__pyx_t_2)) __PYX_ERR(5, 68, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_2);
  if (PyDict_SetItem(__pyx_t_2, __pyx_n_s_shutdown_cq, Py_True) < 0) __PYX_ERR(5, 68, __pyx_L1_error)
  __pyx_t_3 = __Pyx_PyObject_Call(((PyObject *)__pyx_ptype_4grpc_7_cython_6cygrpc_CompletionQueue), __pyx_empty_tuple, __pyx_t_2); if (unlikely(!__pyx_t_3)) __PYX_ERR(5, 68, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_3);
  __Pyx_DECREF(__pyx_t_2); __pyx_t_2 = 0;
  __Pyx_GIVEREF(__pyx_t_3);
  __Pyx_GOTREF(__pyx_v_self->backup_shutdown_queue);
  __Pyx_DECREF(((PyObject *)__pyx_v_self->backup_shutdown_queue));
  __pyx_v_self->backup_shutdown_queue = ((struct __pyx_obj_4grpc_7_cython_6cygrpc_CompletionQueue *)__pyx_t_3);
  __pyx_t_3 = 0;

  /* "src/python/grpcio/grpc/_cython/_cygrpc/server.pyx.pxi":69
 *       raise ValueError("the server has already started")
 *     self.backup_shutdown_queue = CompletionQueue(shutdown_cq=True)
 *     self.register_completion_queue(self.backup_shutdown_queue)             # <<<<<<<<<<<<<<
 *     self.is_started = True
 *     with nogil:
 */
  __pyx_t_2 = __Pyx_PyObject_GetAttrStr(((PyObject *)__pyx_v_self), __pyx_n_s_register_completion_queue); if (unlikely(!__pyx_t_2)) __PYX_ERR(5, 69, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_2);
  __pyx_t_4 = NULL;
  if (CYTHON_UNPACK_METHODS && likely(PyMethod_Check(__pyx_t_2))) {
    __pyx_t_4 = PyMethod_GET_SELF(__pyx_t_2);
    if (likely(__pyx_t_4)) {
      PyObject* function = PyMethod_GET_FUNCTION(__pyx_t_2);
      __Pyx_INCREF(__pyx_t_4);
      __Pyx_INCREF(function);
      __Pyx_DECREF_SET(__pyx_t_2, function);
    }
  }
  if (!__pyx_t_4) {
    __pyx_t_3 = __Pyx_PyObject_CallOneArg(__pyx_t_2, ((PyObject *)__pyx_v_self->backup_shutdown_queue)); if (unlikely(!__pyx_t_3)) __PYX_ERR(5, 69, __pyx_L1_error)
    __Pyx_GOTREF(__pyx_t_3);
  } else {
    #if CYTHON_FAST_PYCALL
    if (PyFunction_Check(__pyx_t_2)) {
      PyObject *__pyx_temp[2] = {__pyx_t_4, ((PyObject *)__pyx_v_self->backup_shutdown_queue)};
      __pyx_t_3 = __Pyx_PyFunction_FastCall(__pyx_t_2, __pyx_temp+1-1, 1+1); if (unlikely(!__pyx_t_3)) __PYX_ERR(5, 69, __pyx_L1_error)
      __Pyx_XDECREF(__pyx_t_4); __pyx_t_4 = 0;
      __Pyx_GOTREF(__pyx_t_3);
    } else
    #endif
    #if CYTHON_FAST_PYCCALL
    if (__Pyx_PyFastCFunction_Check(__pyx_t_2)) {
      PyObject *__pyx_temp[2] = {__pyx_t_4, ((PyObject *)__pyx_v_self->backup_shutdown_queue)};
      __pyx_t_3 = __Pyx_PyCFunction_FastCall(__pyx_t_2, __pyx_temp+1-1, 1+1); if (unlikely(!__pyx_t_3)) __PYX_ERR(5, 69, __pyx_L1_error)
      __Pyx_XDECREF(__pyx_t_4); __pyx_t_4 = 0;
      __Pyx_GOTREF(__pyx_t_3);
    } else
    #endif
    {
      __pyx_t_5 = PyTuple_New(1+1); if (unlikely(!__pyx_t_5)) __PYX_ERR(5, 69, __pyx_L1_error)
      __Pyx_GOTREF(__pyx_t_5);
      __Pyx_GIVEREF(__pyx_t_4); PyTuple_SET_ITEM(__pyx_t_5, 0, __pyx_t_4); __pyx_t_4 = NULL;
      __Pyx_INCREF(((PyObject *)__pyx_v_self->backup_shutdown_queue));
      __Pyx_GIVEREF(((PyObject *)__pyx_v_self->backup_shutdown_queue));
      PyTuple_SET_ITEM(__pyx_t_5, 0+1, ((PyObject *)__pyx_v_self->backup_shutdown_queue));
      __pyx_t_3 = __Pyx_PyObject_Call(__pyx_t_2, __pyx_t_5, NULL); if (unlikely(!__pyx_t_3)) __PYX_ERR(5, 69, __pyx_L1_error)
      __Pyx_GOTREF(__pyx_t_3);
      __Pyx_DECREF(__pyx_t_5); __pyx_t_5 = 0;
    }
  }
  __Pyx_DECREF(__pyx_t_2); __pyx_t_2 = 0;
  __Pyx_DECREF(__pyx_t_3); __pyx_t_3 = 0;

  /* "src/python/grpcio/grpc/_cython/_cygrpc/server.pyx.pxi":70
 *     self.backup_shutdown_queue = CompletionQueue(shutdown_cq=True)
 *     self.register_completion_queue(self.backup_shutdown_queue)
 *     self.is_started = True             # <<<<<<<<<<<<<<
 *     with nogil:
 *       grpc_server_start(self.c_server)
 */
  __pyx_v_self->is_started = 1;

  /* "src/python/grpcio/grpc/_cython/_cygrpc/server.pyx.pxi":71
 *     self.register_completion_queue(self.backup_shutdown_queue)
 *     self.is_started = True
 *     with nogil:             # <<<<<<<<<<<<<<
 *       grpc_server_start(self.c_server)
 *     # Ensure the core has gotten a chance to do the start-up work
 */
  {
      #ifdef WITH_THREAD
      PyThreadState *_save;
      Py_UNBLOCK_THREADS
      __Pyx_FastGIL_Remember();
      #endif
      /*try:*/ {

        /* "src/python/grpcio/grpc/_cython/_cygrpc/server.pyx.pxi":72
 *     self.is_started = True
 *     with nogil:
 *       grpc_server_start(self.c_server)             # <<<<<<<<<<<<<<
 *     # Ensure the core has gotten a chance to do the start-up work
 *     self.backup_shutdown_queue.poll(deadline=time.time())
 */
        grpc_server_start(__pyx_v_self->c_server);
      }

      /* "src/python/grpcio/grpc/_cython/_cygrpc/server.pyx.pxi":71
 *     self.register_completion_queue(self.backup_shutdown_queue)
 *     self.is_started = True
 *     with nogil:             # <<<<<<<<<<<<<<
 *       grpc_server_start(self.c_server)
 *     # Ensure the core has gotten a chance to do the start-up work
 */
      /*finally:*/ {
        /*normal exit:*/{
          #ifdef WITH_THREAD
          __Pyx_FastGIL_Forget();
          Py_BLOCK_THREADS
          #endif
          goto __pyx_L6;
        }
        __pyx_L6:;
      }
  }

  /* "src/python/grpcio/grpc/_cython/_cygrpc/server.pyx.pxi":74
 *       grpc_server_start(self.c_server)
 *     # Ensure the core has gotten a chance to do the start-up work
 *     self.backup_shutdown_queue.poll(deadline=time.time())             # <<<<<<<<<<<<<<
 * 
 *   def add_http2_port(self, bytes address,
 */
  __pyx_t_3 = __Pyx_PyObject_GetAttrStr(((PyObject *)__pyx_v_self->backup_shutdown_queue), __pyx_n_s_poll); if (unlikely(!__pyx_t_3)) __PYX_ERR(5, 74, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_3);
  __pyx_t_2 = __Pyx_PyDict_NewPresized(1); if (unlikely(!__pyx_t_2)) __PYX_ERR(5, 74, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_2);
  __pyx_t_4 = __Pyx_GetModuleGlobalName(__pyx_n_s_time); if (unlikely(!__pyx_t_4)) __PYX_ERR(5, 74, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_4);
  __pyx_t_6 = __Pyx_PyObject_GetAttrStr(__pyx_t_4, __pyx_n_s_time); if (unlikely(!__pyx_t_6)) __PYX_ERR(5, 74, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_6);
  __Pyx_DECREF(__pyx_t_4); __pyx_t_4 = 0;
  __pyx_t_4 = NULL;
  if (CYTHON_UNPACK_METHODS && unlikely(PyMethod_Check(__pyx_t_6))) {
    __pyx_t_4 = PyMethod_GET_SELF(__pyx_t_6);
    if (likely(__pyx_t_4)) {
      PyObject* function = PyMethod_GET_FUNCTION(__pyx_t_6);
      __Pyx_INCREF(__pyx_t_4);
      __Pyx_INCREF(function);
      __Pyx_DECREF_SET(__pyx_t_6, function);
    }
  }
  if (__pyx_t_4) {
    __pyx_t_5 = __Pyx_PyObject_CallOneArg(__pyx_t_6, __pyx_t_4); if (unlikely(!__pyx_t_5)) __PYX_ERR(5, 74, __pyx_L1_error)
    __Pyx_DECREF(__pyx_t_4); __pyx_t_4 = 0;
  } else {
    __pyx_t_5 = __Pyx_PyObject_CallNoArg(__pyx_t_6); if (unlikely(!__pyx_t_5)) __PYX_ERR(5, 74, __pyx_L1_error)
  }
  __Pyx_GOTREF(__pyx_t_5);
  __Pyx_DECREF(__pyx_t_6); __pyx_t_6 = 0;
  if (PyDict_SetItem(__pyx_t_2, __pyx_n_s_deadline, __pyx_t_5) < 0) __PYX_ERR(5, 74, __pyx_L1_error)
  __Pyx_DECREF(__pyx_t_5); __pyx_t_5 = 0;
  __pyx_t_5 = __Pyx_PyObject_Call(__pyx_t_3, __pyx_empty_tuple, __pyx_t_2); if (unlikely(!__pyx_t_5)) __PYX_ERR(5, 74, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_5);
  __Pyx_DECREF(__pyx_t_3); __pyx_t_3 = 0;
  __Pyx_DECREF(__pyx_t_2); __pyx_t_2 = 0;
  __Pyx_DECREF(__pyx_t_5); __pyx_t_5 = 0;

  /* "src/python/grpcio/grpc/_cython/_cygrpc/server.pyx.pxi":65
 *     self.registered_completion_queues.append(queue)
 * 
 *   def start(self):             # <<<<<<<<<<<<<<
 *     if self.is_started:
 *       raise ValueError("the server has already started")
 */

  /* function exit code */
  __pyx_r = Py_None; __Pyx_INCREF(Py_None);
  goto __pyx_L0;
  __pyx_L1_error:;
  __Pyx_XDECREF(__pyx_t_2);
  __Pyx_XDECREF(__pyx_t_3);
  __Pyx_XDECREF(__pyx_t_4);
  __Pyx_XDECREF(__pyx_t_5);
  __Pyx_XDECREF(__pyx_t_6);
  __Pyx_AddTraceback("grpc._cython.cygrpc.Server.start", __pyx_clineno, __pyx_lineno, __pyx_filename);
  __pyx_r = NULL;
  __pyx_L0:;
  __Pyx_XGIVEREF(__pyx_r);
  __Pyx_RefNannyFinishContext();
  return __pyx_r;
}

/* "src/python/grpcio/grpc/_cython/_cygrpc/server.pyx.pxi":76
 *     self.backup_shutdown_queue.poll(deadline=time.time())
 * 
 *   def add_http2_port(self, bytes address,             # <<<<<<<<<<<<<<
 *                      ServerCredentials server_credentials=None):
 *     address = str_to_bytes(address)
 */

/* Python wrapper */
static PyObject *__pyx_pw_4grpc_7_cython_6cygrpc_6Server_9add_http2_port(PyObject *__pyx_v_self, PyObject *__pyx_args, PyObject *__pyx_kwds); /*proto*/
static PyObject *__pyx_pw_4grpc_7_cython_6cygrpc_6Server_9add_http2_port(PyObject *__pyx_v_self, PyObject *__pyx_args, PyObject *__pyx_kwds) {
  PyObject *__pyx_v_address = 0;
  struct __pyx_obj_4grpc_7_cython_6cygrpc_ServerCredentials *__pyx_v_server_credentials = 0;
  PyObject *__pyx_r = 0;
  __Pyx_RefNannyDeclarations
  __Pyx_RefNannySetupContext("add_http2_port (wrapper)", 0);
  {
    static PyObject **__pyx_pyargnames[] = {&__pyx_n_s_address,&__pyx_n_s_server_credentials,0};
    PyObject* values[2] = {0,0};

    /* "src/python/grpcio/grpc/_cython/_cygrpc/server.pyx.pxi":77
 * 
 *   def add_http2_port(self, bytes address,
 *                      ServerCredentials server_credentials=None):             # <<<<<<<<<<<<<<
 *     address = str_to_bytes(address)
 *     self.references.append(address)
 */
    values[1] = (PyObject *)((struct __pyx_obj_4grpc_7_cython_6cygrpc_ServerCredentials *)Py_None);
    if (unlikely(__pyx_kwds)) {
      Py_ssize_t kw_args;
      const Py_ssize_t pos_args = PyTuple_GET_SIZE(__pyx_args);
      switch (pos_args) {
        case  2: values[1] = PyTuple_GET_ITEM(__pyx_args, 1);
        CYTHON_FALLTHROUGH;
        case  1: values[0] = PyTuple_GET_ITEM(__pyx_args, 0);
        CYTHON_FALLTHROUGH;
        case  0: break;
        default: goto __pyx_L5_argtuple_error;
      }
      kw_args = PyDict_Size(__pyx_kwds);
      switch (pos_args) {
        case  0:
        if (likely((values[0] = __Pyx_PyDict_GetItemStr(__pyx_kwds, __pyx_n_s_address)) != 0)) kw_args--;
        else goto __pyx_L5_argtuple_error;
        CYTHON_FALLTHROUGH;
        case  1:
        if (kw_args > 0) {
          PyObject* value = __Pyx_PyDict_GetItemStr(__pyx_kwds, __pyx_n_s_server_credentials);
          if (value) { values[1] = value; kw_args--; }
        }
      }
      if (unlikely(kw_args > 0)) {
        if (unlikely(__Pyx_ParseOptionalKeywords(__pyx_kwds, __pyx_pyargnames, 0, values, pos_args, "add_http2_port") < 0)) __PYX_ERR(5, 76, __pyx_L3_error)
      }
    } else {
      switch (PyTuple_GET_SIZE(__pyx_args)) {
        case  2: values[1] = PyTuple_GET_ITEM(__pyx_args, 1);
        CYTHON_FALLTHROUGH;
        case  1: values[0] = PyTuple_GET_ITEM(__pyx_args, 0);
        break;
        default: goto __pyx_L5_argtuple_error;
      }
    }
    __pyx_v_address = ((PyObject*)values[0]);
    __pyx_v_server_credentials = ((struct __pyx_obj_4grpc_7_cython_6cygrpc_ServerCredentials *)values[1]);
  }
  goto __pyx_L4_argument_unpacking_done;
  __pyx_L5_argtuple_error:;
  __Pyx_RaiseArgtupleInvalid("add_http2_port", 0, 1, 2, PyTuple_GET_SIZE(__pyx_args)); __PYX_ERR(5, 76, __pyx_L3_error)
  __pyx_L3_error:;
  __Pyx_AddTraceback("grpc._cython.cygrpc.Server.add_http2_port", __pyx_clineno, __pyx_lineno, __pyx_filename);
  __Pyx_RefNannyFinishContext();
  return NULL;
  __pyx_L4_argument_unpacking_done:;
  if (unlikely(!__Pyx_ArgTypeTest(((PyObject *)__pyx_v_address), (&PyBytes_Type), 1, "address", 1))) __PYX_ERR(5, 76, __pyx_L1_error)
  if (unlikely(!__Pyx_ArgTypeTest(((PyObject *)__pyx_v_server_credentials), __pyx_ptype_4grpc_7_cython_6cygrpc_ServerCredentials, 1, "server_credentials", 0))) __PYX_ERR(5, 77, __pyx_L1_error)
  __pyx_r = __pyx_pf_4grpc_7_cython_6cygrpc_6Server_8add_http2_port(((struct __pyx_obj_4grpc_7_cython_6cygrpc_Server *)__pyx_v_self), __pyx_v_address, __pyx_v_server_credentials);

  /* "src/python/grpcio/grpc/_cython/_cygrpc/server.pyx.pxi":76
 *     self.backup_shutdown_queue.poll(deadline=time.time())
 * 
 *   def add_http2_port(self, bytes address,             # <<<<<<<<<<<<<<
 *                      ServerCredentials server_credentials=None):
 *     address = str_to_bytes(address)
 */

  /* function exit code */
  goto __pyx_L0;
  __pyx_L1_error:;
  __pyx_r = NULL;
  __pyx_L0:;
  __Pyx_RefNannyFinishContext();
  return __pyx_r;
}

static PyObject *__pyx_pf_4grpc_7_cython_6cygrpc_6Server_8add_http2_port(struct __pyx_obj_4grpc_7_cython_6cygrpc_Server *__pyx_v_self, PyObject *__pyx_v_address, struct __pyx_obj_4grpc_7_cython_6cygrpc_ServerCredentials *__pyx_v_server_credentials) {
  int __pyx_v_result;
  char *__pyx_v_address_c_string;
  PyObject *__pyx_r = NULL;
  __Pyx_RefNannyDeclarations
  PyObject *__pyx_t_1 = NULL;
  int __pyx_t_2;
  char *__pyx_t_3;
  int __pyx_t_4;
  int __pyx_t_5;
  __Pyx_RefNannySetupContext("add_http2_port", 0);
  __Pyx_INCREF(__pyx_v_address);

  /* "src/python/grpcio/grpc/_cython/_cygrpc/server.pyx.pxi":78
 *   def add_http2_port(self, bytes address,
 *                      ServerCredentials server_credentials=None):
 *     address = str_to_bytes(address)             # <<<<<<<<<<<<<<
 *     self.references.append(address)
 *     cdef int result
 */
  __pyx_t_1 = __pyx_f_4grpc_7_cython_6cygrpc_str_to_bytes(__pyx_v_address); if (unlikely(!__pyx_t_1)) __PYX_ERR(5, 78, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_1);
  __Pyx_DECREF_SET(__pyx_v_address, ((PyObject*)__pyx_t_1));
  __pyx_t_1 = 0;

  /* "src/python/grpcio/grpc/_cython/_cygrpc/server.pyx.pxi":79
 *                      ServerCredentials server_credentials=None):
 *     address = str_to_bytes(address)
 *     self.references.append(address)             # <<<<<<<<<<<<<<
 *     cdef int result
 *     cdef char *address_c_string = address
 */
  if (unlikely(__pyx_v_self->references == Py_None)) {
    PyErr_Format(PyExc_AttributeError, "'NoneType' object has no attribute '%.30s'", "append");
    __PYX_ERR(5, 79, __pyx_L1_error)
  }
  __pyx_t_2 = __Pyx_PyList_Append(__pyx_v_self->references, __pyx_v_address); if (unlikely(__pyx_t_2 == ((int)-1))) __PYX_ERR(5, 79, __pyx_L1_error)

  /* "src/python/grpcio/grpc/_cython/_cygrpc/server.pyx.pxi":81
 *     self.references.append(address)
 *     cdef int result
 *     cdef char *address_c_string = address             # <<<<<<<<<<<<<<
 *     if server_credentials is not None:
 *       self.references.append(server_credentials)
 */
  if (unlikely(__pyx_v_address == Py_None)) {
    PyErr_SetString(PyExc_TypeError, "expected bytes, NoneType found");
    __PYX_ERR(5, 81, __pyx_L1_error)
  }
  __pyx_t_3 = __Pyx_PyBytes_AsWritableString(__pyx_v_address); if (unlikely((!__pyx_t_3) && PyErr_Occurred())) __PYX_ERR(5, 81, __pyx_L1_error)
  __pyx_v_address_c_string = __pyx_t_3;

  /* "src/python/grpcio/grpc/_cython/_cygrpc/server.pyx.pxi":82
 *     cdef int result
 *     cdef char *address_c_string = address
 *     if server_credentials is not None:             # <<<<<<<<<<<<<<
 *       self.references.append(server_credentials)
 *       with nogil:
 */
  __pyx_t_4 = (((PyObject *)__pyx_v_server_credentials) != Py_None);
  __pyx_t_5 = (__pyx_t_4 != 0);
  if (__pyx_t_5) {

    /* "src/python/grpcio/grpc/_cython/_cygrpc/server.pyx.pxi":83
 *     cdef char *address_c_string = address
 *     if server_credentials is not None:
 *       self.references.append(server_credentials)             # <<<<<<<<<<<<<<
 *       with nogil:
 *         result = grpc_server_add_secure_http2_port(
 */
    if (unlikely(__pyx_v_self->references == Py_None)) {
      PyErr_Format(PyExc_AttributeError, "'NoneType' object has no attribute '%.30s'", "append");
      __PYX_ERR(5, 83, __pyx_L1_error)
    }
    __pyx_t_2 = __Pyx_PyList_Append(__pyx_v_self->references, ((PyObject *)__pyx_v_server_credentials)); if (unlikely(__pyx_t_2 == ((int)-1))) __PYX_ERR(5, 83, __pyx_L1_error)

    /* "src/python/grpcio/grpc/_cython/_cygrpc/server.pyx.pxi":84
 *     if server_credentials is not None:
 *       self.references.append(server_credentials)
 *       with nogil:             # <<<<<<<<<<<<<<
 *         result = grpc_server_add_secure_http2_port(
 *             self.c_server, address_c_string, server_credentials.c_credentials)
 */
    {
        #ifdef WITH_THREAD
        PyThreadState *_save;
        Py_UNBLOCK_THREADS
        __Pyx_FastGIL_Remember();
        #endif
        /*try:*/ {

          /* "src/python/grpcio/grpc/_cython/_cygrpc/server.pyx.pxi":85
 *       self.references.append(server_credentials)
 *       with nogil:
 *         result = grpc_server_add_secure_http2_port(             # <<<<<<<<<<<<<<
 *             self.c_server, address_c_string, server_credentials.c_credentials)
 *     else:
 */
          __pyx_v_result = grpc_server_add_secure_http2_port(__pyx_v_self->c_server, __pyx_v_address_c_string, __pyx_v_server_credentials->c_credentials);
        }

        /* "src/python/grpcio/grpc/_cython/_cygrpc/server.pyx.pxi":84
 *     if server_credentials is not None:
 *       self.references.append(server_credentials)
 *       with nogil:             # <<<<<<<<<<<<<<
 *         result = grpc_server_add_secure_http2_port(
 *             self.c_server, address_c_string, server_credentials.c_credentials)
 */
        /*finally:*/ {
          /*normal exit:*/{
            #ifdef WITH_THREAD
            __Pyx_FastGIL_Forget();
            Py_BLOCK_THREADS
            #endif
            goto __pyx_L6;
          }
          __pyx_L6:;
        }
    }

    /* "src/python/grpcio/grpc/_cython/_cygrpc/server.pyx.pxi":82
 *     cdef int result
 *     cdef char *address_c_string = address
 *     if server_credentials is not None:             # <<<<<<<<<<<<<<
 *       self.references.append(server_credentials)
 *       with nogil:
 */
    goto __pyx_L3;
  }

  /* "src/python/grpcio/grpc/_cython/_cygrpc/server.pyx.pxi":88
 *             self.c_server, address_c_string, server_credentials.c_credentials)
 *     else:
 *       with nogil:             # <<<<<<<<<<<<<<
 *         result = grpc_server_add_insecure_http2_port(self.c_server,
 *                                                      address_c_string)
 */
  /*else*/ {
    {
        #ifdef WITH_THREAD
        PyThreadState *_save;
        Py_UNBLOCK_THREADS
        __Pyx_FastGIL_Remember();
        #endif
        /*try:*/ {

          /* "src/python/grpcio/grpc/_cython/_cygrpc/server.pyx.pxi":89
 *     else:
 *       with nogil:
 *         result = grpc_server_add_insecure_http2_port(self.c_server,             # <<<<<<<<<<<<<<
 *                                                      address_c_string)
 *     return result
 */
          __pyx_v_result = grpc_server_add_insecure_http2_port(__pyx_v_self->c_server, __pyx_v_address_c_string);
        }

        /* "src/python/grpcio/grpc/_cython/_cygrpc/server.pyx.pxi":88
 *             self.c_server, address_c_string, server_credentials.c_credentials)
 *     else:
 *       with nogil:             # <<<<<<<<<<<<<<
 *         result = grpc_server_add_insecure_http2_port(self.c_server,
 *                                                      address_c_string)
 */
        /*finally:*/ {
          /*normal exit:*/{
            #ifdef WITH_THREAD
            __Pyx_FastGIL_Forget();
            Py_BLOCK_THREADS
            #endif
            goto __pyx_L9;
          }
          __pyx_L9:;
        }
    }
  }
  __pyx_L3:;

  /* "src/python/grpcio/grpc/_cython/_cygrpc/server.pyx.pxi":91
 *         result = grpc_server_add_insecure_http2_port(self.c_server,
 *                                                      address_c_string)
 *     return result             # <<<<<<<<<<<<<<
 * 
 *   cdef _c_shutdown(self, CompletionQueue queue, tag):
 */
  __Pyx_XDECREF(__pyx_r);
  __pyx_t_1 = __Pyx_PyInt_From_int(__pyx_v_result); if (unlikely(!__pyx_t_1)) __PYX_ERR(5, 91, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_1);
  __pyx_r = __pyx_t_1;
  __pyx_t_1 = 0;
  goto __pyx_L0;

  /* "src/python/grpcio/grpc/_cython/_cygrpc/server.pyx.pxi":76
 *     self.backup_shutdown_queue.poll(deadline=time.time())
 * 
 *   def add_http2_port(self, bytes address,             # <<<<<<<<<<<<<<
 *                      ServerCredentials server_credentials=None):
 *     address = str_to_bytes(address)
 */

  /* function exit code */
  __pyx_L1_error:;
  __Pyx_XDECREF(__pyx_t_1);
  __Pyx_AddTraceback("grpc._cython.cygrpc.Server.add_http2_port", __pyx_clineno, __pyx_lineno, __pyx_filename);
  __pyx_r = NULL;
  __pyx_L0:;
  __Pyx_XDECREF(__pyx_v_address);
  __Pyx_XGIVEREF(__pyx_r);
  __Pyx_RefNannyFinishContext();
  return __pyx_r;
}

/* "src/python/grpcio/grpc/_cython/_cygrpc/server.pyx.pxi":93
 *     return result
 * 
 *   cdef _c_shutdown(self, CompletionQueue queue, tag):             # <<<<<<<<<<<<<<
 *     self.is_shutting_down = True
 *     cdef _ServerShutdownTag server_shutdown_tag = _ServerShutdownTag(tag, self)
 */

static PyObject *__pyx_f_4grpc_7_cython_6cygrpc_6Server__c_shutdown(struct __pyx_obj_4grpc_7_cython_6cygrpc_Server *__pyx_v_self, struct __pyx_obj_4grpc_7_cython_6cygrpc_CompletionQueue *__pyx_v_queue, PyObject *__pyx_v_tag) {
  struct __pyx_obj_4grpc_7_cython_6cygrpc__ServerShutdownTag *__pyx_v_server_shutdown_tag = 0;
  PyObject *__pyx_r = NULL;
  __Pyx_RefNannyDeclarations
  PyObject *__pyx_t_1 = NULL;
  PyObject *__pyx_t_2 = NULL;
  __Pyx_RefNannySetupContext("_c_shutdown", 0);

  /* "src/python/grpcio/grpc/_cython/_cygrpc/server.pyx.pxi":94
 * 
 *   cdef _c_shutdown(self, CompletionQueue queue, tag):
 *     self.is_shutting_down = True             # <<<<<<<<<<<<<<
 *     cdef _ServerShutdownTag server_shutdown_tag = _ServerShutdownTag(tag, self)
 *     cpython.Py_INCREF(server_shutdown_tag)
 */
  __pyx_v_self->is_shutting_down = 1;

  /* "src/python/grpcio/grpc/_cython/_cygrpc/server.pyx.pxi":95
 *   cdef _c_shutdown(self, CompletionQueue queue, tag):
 *     self.is_shutting_down = True
 *     cdef _ServerShutdownTag server_shutdown_tag = _ServerShutdownTag(tag, self)             # <<<<<<<<<<<<<<
 *     cpython.Py_INCREF(server_shutdown_tag)
 *     with nogil:
 */
  __pyx_t_1 = PyTuple_New(2); if (unlikely(!__pyx_t_1)) __PYX_ERR(5, 95, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_1);
  __Pyx_INCREF(__pyx_v_tag);
  __Pyx_GIVEREF(__pyx_v_tag);
  PyTuple_SET_ITEM(__pyx_t_1, 0, __pyx_v_tag);
  __Pyx_INCREF(((PyObject *)__pyx_v_self));
  __Pyx_GIVEREF(((PyObject *)__pyx_v_self));
  PyTuple_SET_ITEM(__pyx_t_1, 1, ((PyObject *)__pyx_v_self));
  __pyx_t_2 = __Pyx_PyObject_Call(((PyObject *)__pyx_ptype_4grpc_7_cython_6cygrpc__ServerShutdownTag), __pyx_t_1, NULL); if (unlikely(!__pyx_t_2)) __PYX_ERR(5, 95, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_2);
  __Pyx_DECREF(__pyx_t_1); __pyx_t_1 = 0;
  __pyx_v_server_shutdown_tag = ((struct __pyx_obj_4grpc_7_cython_6cygrpc__ServerShutdownTag *)__pyx_t_2);
  __pyx_t_2 = 0;

  /* "src/python/grpcio/grpc/_cython/_cygrpc/server.pyx.pxi":96
 *     self.is_shutting_down = True
 *     cdef _ServerShutdownTag server_shutdown_tag = _ServerShutdownTag(tag, self)
 *     cpython.Py_INCREF(server_shutdown_tag)             # <<<<<<<<<<<<<<
 *     with nogil:
 *       grpc_server_shutdown_and_notify(
 */
  Py_INCREF(((PyObject *)__pyx_v_server_shutdown_tag));

  /* "src/python/grpcio/grpc/_cython/_cygrpc/server.pyx.pxi":97
 *     cdef _ServerShutdownTag server_shutdown_tag = _ServerShutdownTag(tag, self)
 *     cpython.Py_INCREF(server_shutdown_tag)
 *     with nogil:             # <<<<<<<<<<<<<<
 *       grpc_server_shutdown_and_notify(
 *           self.c_server, queue.c_completion_queue,
 */
  {
      #ifdef WITH_THREAD
      PyThreadState *_save;
      Py_UNBLOCK_THREADS
      __Pyx_FastGIL_Remember();
      #endif
      /*try:*/ {

        /* "src/python/grpcio/grpc/_cython/_cygrpc/server.pyx.pxi":98
 *     cpython.Py_INCREF(server_shutdown_tag)
 *     with nogil:
 *       grpc_server_shutdown_and_notify(             # <<<<<<<<<<<<<<
 *           self.c_server, queue.c_completion_queue,
 *           <cpython.PyObject *>server_shutdown_tag)
 */
        grpc_server_shutdown_and_notify(__pyx_v_self->c_server, __pyx_v_queue->c_completion_queue, ((PyObject *)__pyx_v_server_shutdown_tag));
      }

      /* "src/python/grpcio/grpc/_cython/_cygrpc/server.pyx.pxi":97
 *     cdef _ServerShutdownTag server_shutdown_tag = _ServerShutdownTag(tag, self)
 *     cpython.Py_INCREF(server_shutdown_tag)
 *     with nogil:             # <<<<<<<<<<<<<<
 *       grpc_server_shutdown_and_notify(
 *           self.c_server, queue.c_completion_queue,
 */
      /*finally:*/ {
        /*normal exit:*/{
          #ifdef WITH_THREAD
          __Pyx_FastGIL_Forget();
          Py_BLOCK_THREADS
          #endif
          goto __pyx_L5;
        }
        __pyx_L5:;
      }
  }

  /* "src/python/grpcio/grpc/_cython/_cygrpc/server.pyx.pxi":93
 *     return result
 * 
 *   cdef _c_shutdown(self, CompletionQueue queue, tag):             # <<<<<<<<<<<<<<
 *     self.is_shutting_down = True
 *     cdef _ServerShutdownTag server_shutdown_tag = _ServerShutdownTag(tag, self)
 */

  /* function exit code */
  __pyx_r = Py_None; __Pyx_INCREF(Py_None);
  goto __pyx_L0;
  __pyx_L1_error:;
  __Pyx_XDECREF(__pyx_t_1);
  __Pyx_XDECREF(__pyx_t_2);
  __Pyx_AddTraceback("grpc._cython.cygrpc.Server._c_shutdown", __pyx_clineno, __pyx_lineno, __pyx_filename);
  __pyx_r = 0;
  __pyx_L0:;
  __Pyx_XDECREF((PyObject *)__pyx_v_server_shutdown_tag);
  __Pyx_XGIVEREF(__pyx_r);
  __Pyx_RefNannyFinishContext();
  return __pyx_r;
}

/* "src/python/grpcio/grpc/_cython/_cygrpc/server.pyx.pxi":102
 *           <cpython.PyObject *>server_shutdown_tag)
 * 
 *   def shutdown(self, CompletionQueue queue not None, tag):             # <<<<<<<<<<<<<<
 *     if queue.is_shutting_down:
 *       raise ValueError("queue must be live")
 */

/* Python wrapper */
static PyObject *__pyx_pw_4grpc_7_cython_6cygrpc_6Server_11shutdown(PyObject *__pyx_v_self, PyObject *__pyx_args, PyObject *__pyx_kwds); /*proto*/
static PyObject *__pyx_pw_4grpc_7_cython_6cygrpc_6Server_11shutdown(PyObject *__pyx_v_self, PyObject *__pyx_args, PyObject *__pyx_kwds) {
  struct __pyx_obj_4grpc_7_cython_6cygrpc_CompletionQueue *__pyx_v_queue = 0;
  PyObject *__pyx_v_tag = 0;
  PyObject *__pyx_r = 0;
  __Pyx_RefNannyDeclarations
  __Pyx_RefNannySetupContext("shutdown (wrapper)", 0);
  {
    static PyObject **__pyx_pyargnames[] = {&__pyx_n_s_queue,&__pyx_n_s_tag,0};
    PyObject* values[2] = {0,0};
    if (unlikely(__pyx_kwds)) {
      Py_ssize_t kw_args;
      const Py_ssize_t pos_args = PyTuple_GET_SIZE(__pyx_args);
      switch (pos_args) {
        case  2: values[1] = PyTuple_GET_ITEM(__pyx_args, 1);
        CYTHON_FALLTHROUGH;
        case  1: values[0] = PyTuple_GET_ITEM(__pyx_args, 0);
        CYTHON_FALLTHROUGH;
        case  0: break;
        default: goto __pyx_L5_argtuple_error;
      }
      kw_args = PyDict_Size(__pyx_kwds);
      switch (pos_args) {
        case  0:
        if (likely((values[0] = __Pyx_PyDict_GetItemStr(__pyx_kwds, __pyx_n_s_queue)) != 0)) kw_args--;
        else goto __pyx_L5_argtuple_error;
        CYTHON_FALLTHROUGH;
        case  1:
        if (likely((values[1] = __Pyx_PyDict_GetItemStr(__pyx_kwds, __pyx_n_s_tag)) != 0)) kw_args--;
        else {
          __Pyx_RaiseArgtupleInvalid("shutdown", 1, 2, 2, 1); __PYX_ERR(5, 102, __pyx_L3_error)
        }
      }
      if (unlikely(kw_args > 0)) {
        if (unlikely(__Pyx_ParseOptionalKeywords(__pyx_kwds, __pyx_pyargnames, 0, values, pos_args, "shutdown") < 0)) __PYX_ERR(5, 102, __pyx_L3_error)
      }
    } else if (PyTuple_GET_SIZE(__pyx_args) != 2) {
      goto __pyx_L5_argtuple_error;
    } else {
      values[0] = PyTuple_GET_ITEM(__pyx_args, 0);
      values[1] = PyTuple_GET_ITEM(__pyx_args, 1);
    }
    __pyx_v_queue = ((struct __pyx_obj_4grpc_7_cython_6cygrpc_CompletionQueue *)values[0]);
    __pyx_v_tag = values[1];
  }
  goto __pyx_L4_argument_unpacking_done;
  __pyx_L5_argtuple_error:;
  __Pyx_RaiseArgtupleInvalid("shutdown", 1, 2, 2, PyTuple_GET_SIZE(__pyx_args)); __PYX_ERR(5, 102, __pyx_L3_error)
  __pyx_L3_error:;
  __Pyx_AddTraceback("grpc._cython.cygrpc.Server.shutdown", __pyx_clineno, __pyx_lineno, __pyx_filename);
  __Pyx_RefNannyFinishContext();
  return NULL;
  __pyx_L4_argument_unpacking_done:;
  if (unlikely(!__Pyx_ArgTypeTest(((PyObject *)__pyx_v_queue), __pyx_ptype_4grpc_7_cython_6cygrpc_CompletionQueue, 0, "queue", 0))) __PYX_ERR(5, 102, __pyx_L1_error)
  __pyx_r = __pyx_pf_4grpc_7_cython_6cygrpc_6Server_10shutdown(((struct __pyx_obj_4grpc_7_cython_6cygrpc_Server *)__pyx_v_self), __pyx_v_queue, __pyx_v_tag);

  /* function exit code */
  goto __pyx_L0;
  __pyx_L1_error:;
  __pyx_r = NULL;
  __pyx_L0:;
  __Pyx_RefNannyFinishContext();
  return __pyx_r;
}

static PyObject *__pyx_pf_4grpc_7_cython_6cygrpc_6Server_10shutdown(struct __pyx_obj_4grpc_7_cython_6cygrpc_Server *__pyx_v_self, struct __pyx_obj_4grpc_7_cython_6cygrpc_CompletionQueue *__pyx_v_queue, PyObject *__pyx_v_tag) {
  PyObject *__pyx_r = NULL;
  __Pyx_RefNannyDeclarations
  int __pyx_t_1;
  PyObject *__pyx_t_2 = NULL;
  int __pyx_t_3;
  __Pyx_RefNannySetupContext("shutdown", 0);

  /* "src/python/grpcio/grpc/_cython/_cygrpc/server.pyx.pxi":103
 * 
 *   def shutdown(self, CompletionQueue queue not None, tag):
 *     if queue.is_shutting_down:             # <<<<<<<<<<<<<<
 *       raise ValueError("queue must be live")
 *     elif not self.is_started:
 */
  __pyx_t_1 = (__pyx_v_queue->is_shutting_down != 0);
  if (unlikely(__pyx_t_1)) {

    /* "src/python/grpcio/grpc/_cython/_cygrpc/server.pyx.pxi":104
 *   def shutdown(self, CompletionQueue queue not None, tag):
 *     if queue.is_shutting_down:
 *       raise ValueError("queue must be live")             # <<<<<<<<<<<<<<
 *     elif not self.is_started:
 *       raise ValueError("the server hasn't started yet")
 */
    __pyx_t_2 = __Pyx_PyObject_Call(__pyx_builtin_ValueError, __pyx_tuple__113, NULL); if (unlikely(!__pyx_t_2)) __PYX_ERR(5, 104, __pyx_L1_error)
    __Pyx_GOTREF(__pyx_t_2);
    __Pyx_Raise(__pyx_t_2, 0, 0, 0);
    __Pyx_DECREF(__pyx_t_2); __pyx_t_2 = 0;
    __PYX_ERR(5, 104, __pyx_L1_error)

    /* "src/python/grpcio/grpc/_cython/_cygrpc/server.pyx.pxi":103
 * 
 *   def shutdown(self, CompletionQueue queue not None, tag):
 *     if queue.is_shutting_down:             # <<<<<<<<<<<<<<
 *       raise ValueError("queue must be live")
 *     elif not self.is_started:
 */
  }

  /* "src/python/grpcio/grpc/_cython/_cygrpc/server.pyx.pxi":105
 *     if queue.is_shutting_down:
 *       raise ValueError("queue must be live")
 *     elif not self.is_started:             # <<<<<<<<<<<<<<
 *       raise ValueError("the server hasn't started yet")
 *     elif self.is_shutting_down:
 */
  __pyx_t_1 = ((!(__pyx_v_self->is_started != 0)) != 0);
  if (unlikely(__pyx_t_1)) {

    /* "src/python/grpcio/grpc/_cython/_cygrpc/server.pyx.pxi":106
 *       raise ValueError("queue must be live")
 *     elif not self.is_started:
 *       raise ValueError("the server hasn't started yet")             # <<<<<<<<<<<<<<
 *     elif self.is_shutting_down:
 *       return
 */
    __pyx_t_2 = __Pyx_PyObject_Call(__pyx_builtin_ValueError, __pyx_tuple__114, NULL); if (unlikely(!__pyx_t_2)) __PYX_ERR(5, 106, __pyx_L1_error)
    __Pyx_GOTREF(__pyx_t_2);
    __Pyx_Raise(__pyx_t_2, 0, 0, 0);
    __Pyx_DECREF(__pyx_t_2); __pyx_t_2 = 0;
    __PYX_ERR(5, 106, __pyx_L1_error)

    /* "src/python/grpcio/grpc/_cython/_cygrpc/server.pyx.pxi":105
 *     if queue.is_shutting_down:
 *       raise ValueError("queue must be live")
 *     elif not self.is_started:             # <<<<<<<<<<<<<<
 *       raise ValueError("the server hasn't started yet")
 *     elif self.is_shutting_down:
 */
  }

  /* "src/python/grpcio/grpc/_cython/_cygrpc/server.pyx.pxi":107
 *     elif not self.is_started:
 *       raise ValueError("the server hasn't started yet")
 *     elif self.is_shutting_down:             # <<<<<<<<<<<<<<
 *       return
 *     elif queue not in self.registered_completion_queues:
 */
  __pyx_t_1 = (__pyx_v_self->is_shutting_down != 0);
  if (__pyx_t_1) {

    /* "src/python/grpcio/grpc/_cython/_cygrpc/server.pyx.pxi":108
 *       raise ValueError("the server hasn't started yet")
 *     elif self.is_shutting_down:
 *       return             # <<<<<<<<<<<<<<
 *     elif queue not in self.registered_completion_queues:
 *       raise ValueError("expected registered completion queue")
 */
    __Pyx_XDECREF(__pyx_r);
    __pyx_r = Py_None; __Pyx_INCREF(Py_None);
    goto __pyx_L0;

    /* "src/python/grpcio/grpc/_cython/_cygrpc/server.pyx.pxi":107
 *     elif not self.is_started:
 *       raise ValueError("the server hasn't started yet")
 *     elif self.is_shutting_down:             # <<<<<<<<<<<<<<
 *       return
 *     elif queue not in self.registered_completion_queues:
 */
  }

  /* "src/python/grpcio/grpc/_cython/_cygrpc/server.pyx.pxi":109
 *     elif self.is_shutting_down:
 *       return
 *     elif queue not in self.registered_completion_queues:             # <<<<<<<<<<<<<<
 *       raise ValueError("expected registered completion queue")
 *     else:
 */
  __pyx_t_1 = (__Pyx_PySequence_ContainsTF(((PyObject *)__pyx_v_queue), __pyx_v_self->registered_completion_queues, Py_NE)); if (unlikely(__pyx_t_1 < 0)) __PYX_ERR(5, 109, __pyx_L1_error)
  __pyx_t_3 = (__pyx_t_1 != 0);
  if (unlikely(__pyx_t_3)) {

    /* "src/python/grpcio/grpc/_cython/_cygrpc/server.pyx.pxi":110
 *       return
 *     elif queue not in self.registered_completion_queues:
 *       raise ValueError("expected registered completion queue")             # <<<<<<<<<<<<<<
 *     else:
 *       self._c_shutdown(queue, tag)
 */
    __pyx_t_2 = __Pyx_PyObject_Call(__pyx_builtin_ValueError, __pyx_tuple__115, NULL); if (unlikely(!__pyx_t_2)) __PYX_ERR(5, 110, __pyx_L1_error)
    __Pyx_GOTREF(__pyx_t_2);
    __Pyx_Raise(__pyx_t_2, 0, 0, 0);
    __Pyx_DECREF(__pyx_t_2); __pyx_t_2 = 0;
    __PYX_ERR(5, 110, __pyx_L1_error)

    /* "src/python/grpcio/grpc/_cython/_cygrpc/server.pyx.pxi":109
 *     elif self.is_shutting_down:
 *       return
 *     elif queue not in self.registered_completion_queues:             # <<<<<<<<<<<<<<
 *       raise ValueError("expected registered completion queue")
 *     else:
 */
  }

  /* "src/python/grpcio/grpc/_cython/_cygrpc/server.pyx.pxi":112
 *       raise ValueError("expected registered completion queue")
 *     else:
 *       self._c_shutdown(queue, tag)             # <<<<<<<<<<<<<<
 * 
 *   cdef notify_shutdown_complete(self):
 */
  /*else*/ {
    __pyx_t_2 = ((struct __pyx_vtabstruct_4grpc_7_cython_6cygrpc_Server *)__pyx_v_self->__pyx_vtab)->_c_shutdown(__pyx_v_self, __pyx_v_queue, __pyx_v_tag); if (unlikely(!__pyx_t_2)) __PYX_ERR(5, 112, __pyx_L1_error)
    __Pyx_GOTREF(__pyx_t_2);
    __Pyx_DECREF(__pyx_t_2); __pyx_t_2 = 0;
  }

  /* "src/python/grpcio/grpc/_cython/_cygrpc/server.pyx.pxi":102
 *           <cpython.PyObject *>server_shutdown_tag)
 * 
 *   def shutdown(self, CompletionQueue queue not None, tag):             # <<<<<<<<<<<<<<
 *     if queue.is_shutting_down:
 *       raise ValueError("queue must be live")
 */

  /* function exit code */
  __pyx_r = Py_None; __Pyx_INCREF(Py_None);
  goto __pyx_L0;
  __pyx_L1_error:;
  __Pyx_XDECREF(__pyx_t_2);
  __Pyx_AddTraceback("grpc._cython.cygrpc.Server.shutdown", __pyx_clineno, __pyx_lineno, __pyx_filename);
  __pyx_r = NULL;
  __pyx_L0:;
  __Pyx_XGIVEREF(__pyx_r);
  __Pyx_RefNannyFinishContext();
  return __pyx_r;
}

/* "src/python/grpcio/grpc/_cython/_cygrpc/server.pyx.pxi":114
 *       self._c_shutdown(queue, tag)
 * 
 *   cdef notify_shutdown_complete(self):             # <<<<<<<<<<<<<<
 *     # called only after our server shutdown tag has emerged from a completion
 *     # queue.
 */

static PyObject *__pyx_f_4grpc_7_cython_6cygrpc_6Server_notify_shutdown_complete(struct __pyx_obj_4grpc_7_cython_6cygrpc_Server *__pyx_v_self) {
  PyObject *__pyx_r = NULL;
  __Pyx_RefNannyDeclarations
  __Pyx_RefNannySetupContext("notify_shutdown_complete", 0);

  /* "src/python/grpcio/grpc/_cython/_cygrpc/server.pyx.pxi":117
 *     # called only after our server shutdown tag has emerged from a completion
 *     # queue.
 *     self.is_shutdown = True             # <<<<<<<<<<<<<<
 * 
 *   def cancel_all_calls(self):
 */
  __pyx_v_self->is_shutdown = 1;

  /* "src/python/grpcio/grpc/_cython/_cygrpc/server.pyx.pxi":114
 *       self._c_shutdown(queue, tag)
 * 
 *   cdef notify_shutdown_complete(self):             # <<<<<<<<<<<<<<
 *     # called only after our server shutdown tag has emerged from a completion
 *     # queue.
 */

  /* function exit code */
  __pyx_r = Py_None; __Pyx_INCREF(Py_None);
  __Pyx_XGIVEREF(__pyx_r);
  __Pyx_RefNannyFinishContext();
  return __pyx_r;
}

/* "src/python/grpcio/grpc/_cython/_cygrpc/server.pyx.pxi":119
 *     self.is_shutdown = True
 * 
 *   def cancel_all_calls(self):             # <<<<<<<<<<<<<<
 *     if not self.is_shutting_down:
 *       raise RuntimeError("the server must be shutting down to cancel all calls")
 */

/* Python wrapper */
static PyObject *__pyx_pw_4grpc_7_cython_6cygrpc_6Server_13cancel_all_calls(PyObject *__pyx_v_self, CYTHON_UNUSED PyObject *unused); /*proto*/
static PyObject *__pyx_pw_4grpc_7_cython_6cygrpc_6Server_13cancel_all_calls(PyObject *__pyx_v_self, CYTHON_UNUSED PyObject *unused) {
  PyObject *__pyx_r = 0;
  __Pyx_RefNannyDeclarations
  __Pyx_RefNannySetupContext("cancel_all_calls (wrapper)", 0);
  __pyx_r = __pyx_pf_4grpc_7_cython_6cygrpc_6Server_12cancel_all_calls(((struct __pyx_obj_4grpc_7_cython_6cygrpc_Server *)__pyx_v_self));

  /* function exit code */
  __Pyx_RefNannyFinishContext();
  return __pyx_r;
}

static PyObject *__pyx_pf_4grpc_7_cython_6cygrpc_6Server_12cancel_all_calls(struct __pyx_obj_4grpc_7_cython_6cygrpc_Server *__pyx_v_self) {
  PyObject *__pyx_r = NULL;
  __Pyx_RefNannyDeclarations
  int __pyx_t_1;
  PyObject *__pyx_t_2 = NULL;
  __Pyx_RefNannySetupContext("cancel_all_calls", 0);

  /* "src/python/grpcio/grpc/_cython/_cygrpc/server.pyx.pxi":120
 * 
 *   def cancel_all_calls(self):
 *     if not self.is_shutting_down:             # <<<<<<<<<<<<<<
 *       raise RuntimeError("the server must be shutting down to cancel all calls")
 *     elif self.is_shutdown:
 */
  __pyx_t_1 = ((!(__pyx_v_self->is_shutting_down != 0)) != 0);
  if (unlikely(__pyx_t_1)) {

    /* "src/python/grpcio/grpc/_cython/_cygrpc/server.pyx.pxi":121
 *   def cancel_all_calls(self):
 *     if not self.is_shutting_down:
 *       raise RuntimeError("the server must be shutting down to cancel all calls")             # <<<<<<<<<<<<<<
 *     elif self.is_shutdown:
 *       return
 */
    __pyx_t_2 = __Pyx_PyObject_Call(__pyx_builtin_RuntimeError, __pyx_tuple__116, NULL); if (unlikely(!__pyx_t_2)) __PYX_ERR(5, 121, __pyx_L1_error)
    __Pyx_GOTREF(__pyx_t_2);
    __Pyx_Raise(__pyx_t_2, 0, 0, 0);
    __Pyx_DECREF(__pyx_t_2); __pyx_t_2 = 0;
    __PYX_ERR(5, 121, __pyx_L1_error)

    /* "src/python/grpcio/grpc/_cython/_cygrpc/server.pyx.pxi":120
 * 
 *   def cancel_all_calls(self):
 *     if not self.is_shutting_down:             # <<<<<<<<<<<<<<
 *       raise RuntimeError("the server must be shutting down to cancel all calls")
 *     elif self.is_shutdown:
 */
  }

  /* "src/python/grpcio/grpc/_cython/_cygrpc/server.pyx.pxi":122
 *     if not self.is_shutting_down:
 *       raise RuntimeError("the server must be shutting down to cancel all calls")
 *     elif self.is_shutdown:             # <<<<<<<<<<<<<<
 *       return
 *     else:
 */
  __pyx_t_1 = (__pyx_v_self->is_shutdown != 0);
  if (__pyx_t_1) {

    /* "src/python/grpcio/grpc/_cython/_cygrpc/server.pyx.pxi":123
 *       raise RuntimeError("the server must be shutting down to cancel all calls")
 *     elif self.is_shutdown:
 *       return             # <<<<<<<<<<<<<<
 *     else:
 *       with nogil:
 */
    __Pyx_XDECREF(__pyx_r);
    __pyx_r = Py_None; __Pyx_INCREF(Py_None);
    goto __pyx_L0;

    /* "src/python/grpcio/grpc/_cython/_cygrpc/server.pyx.pxi":122
 *     if not self.is_shutting_down:
 *       raise RuntimeError("the server must be shutting down to cancel all calls")
 *     elif self.is_shutdown:             # <<<<<<<<<<<<<<
 *       return
 *     else:
 */
  }

  /* "src/python/grpcio/grpc/_cython/_cygrpc/server.pyx.pxi":125
 *       return
 *     else:
 *       with nogil:             # <<<<<<<<<<<<<<
 *         grpc_server_cancel_all_calls(self.c_server)
 * 
 */
  /*else*/ {
    {
        #ifdef WITH_THREAD
        PyThreadState *_save;
        Py_UNBLOCK_THREADS
        __Pyx_FastGIL_Remember();
        #endif
        /*try:*/ {

          /* "src/python/grpcio/grpc/_cython/_cygrpc/server.pyx.pxi":126
 *     else:
 *       with nogil:
 *         grpc_server_cancel_all_calls(self.c_server)             # <<<<<<<<<<<<<<
 * 
 *   # TODO(https://github.com/grpc/grpc/issues/17515) Determine what, if any,
 */
          grpc_server_cancel_all_calls(__pyx_v_self->c_server);
        }

        /* "src/python/grpcio/grpc/_cython/_cygrpc/server.pyx.pxi":125
 *       return
 *     else:
 *       with nogil:             # <<<<<<<<<<<<<<
 *         grpc_server_cancel_all_calls(self.c_server)
 * 
 */
        /*finally:*/ {
          /*normal exit:*/{
            #ifdef WITH_THREAD
            __Pyx_FastGIL_Forget();
            Py_BLOCK_THREADS
            #endif
            goto __pyx_L6;
          }
          __pyx_L6:;
        }
    }
  }

  /* "src/python/grpcio/grpc/_cython/_cygrpc/server.pyx.pxi":119
 *     self.is_shutdown = True
 * 
 *   def cancel_all_calls(self):             # <<<<<<<<<<<<<<
 *     if not self.is_shutting_down:
 *       raise RuntimeError("the server must be shutting down to cancel all calls")
 */

  /* function exit code */
  __pyx_r = Py_None; __Pyx_INCREF(Py_None);
  goto __pyx_L0;
  __pyx_L1_error:;
  __Pyx_XDECREF(__pyx_t_2);
  __Pyx_AddTraceback("grpc._cython.cygrpc.Server.cancel_all_calls", __pyx_clineno, __pyx_lineno, __pyx_filename);
  __pyx_r = NULL;
  __pyx_L0:;
  __Pyx_XGIVEREF(__pyx_r);
  __Pyx_RefNannyFinishContext();
  return __pyx_r;
}

/* "src/python/grpcio/grpc/_cython/_cygrpc/server.pyx.pxi":131
 *   # portion of this is safe to call from __dealloc__, and potentially remove
 *   # backup_shutdown_queue.
 *   def destroy(self):             # <<<<<<<<<<<<<<
 *     if self.c_server != NULL:
 *       if not self.is_started:
 */

/* Python wrapper */
static PyObject *__pyx_pw_4grpc_7_cython_6cygrpc_6Server_15destroy(PyObject *__pyx_v_self, CYTHON_UNUSED PyObject *unused); /*proto*/
static PyObject *__pyx_pw_4grpc_7_cython_6cygrpc_6Server_15destroy(PyObject *__pyx_v_self, CYTHON_UNUSED PyObject *unused) {
  PyObject *__pyx_r = 0;
  __Pyx_RefNannyDeclarations
  __Pyx_RefNannySetupContext("destroy (wrapper)", 0);
  __pyx_r = __pyx_pf_4grpc_7_cython_6cygrpc_6Server_14destroy(((struct __pyx_obj_4grpc_7_cython_6cygrpc_Server *)__pyx_v_self));

  /* function exit code */
  __Pyx_RefNannyFinishContext();
  return __pyx_r;
}

static PyObject *__pyx_pf_4grpc_7_cython_6cygrpc_6Server_14destroy(struct __pyx_obj_4grpc_7_cython_6cygrpc_Server *__pyx_v_self) {
  PyObject *__pyx_r = NULL;
  __Pyx_RefNannyDeclarations
  int __pyx_t_1;
  PyObject *__pyx_t_2 = NULL;
  PyObject *__pyx_t_3 = NULL;
  PyObject *__pyx_t_4 = NULL;
  __Pyx_RefNannySetupContext("destroy", 0);

  /* "src/python/grpcio/grpc/_cython/_cygrpc/server.pyx.pxi":132
 *   # backup_shutdown_queue.
 *   def destroy(self):
 *     if self.c_server != NULL:             # <<<<<<<<<<<<<<
 *       if not self.is_started:
 *         pass
 */
  __pyx_t_1 = ((__pyx_v_self->c_server != NULL) != 0);
  if (__pyx_t_1) {

    /* "src/python/grpcio/grpc/_cython/_cygrpc/server.pyx.pxi":133
 *   def destroy(self):
 *     if self.c_server != NULL:
 *       if not self.is_started:             # <<<<<<<<<<<<<<
 *         pass
 *       elif self.is_shutdown:
 */
    __pyx_t_1 = ((!(__pyx_v_self->is_started != 0)) != 0);
    if (__pyx_t_1) {
      goto __pyx_L4;
    }

    /* "src/python/grpcio/grpc/_cython/_cygrpc/server.pyx.pxi":135
 *       if not self.is_started:
 *         pass
 *       elif self.is_shutdown:             # <<<<<<<<<<<<<<
 *         pass
 *       elif not self.is_shutting_down:
 */
    __pyx_t_1 = (__pyx_v_self->is_shutdown != 0);
    if (__pyx_t_1) {
      goto __pyx_L4;
    }

    /* "src/python/grpcio/grpc/_cython/_cygrpc/server.pyx.pxi":137
 *       elif self.is_shutdown:
 *         pass
 *       elif not self.is_shutting_down:             # <<<<<<<<<<<<<<
 *         # the user didn't call shutdown - use our backup queue
 *         self._c_shutdown(self.backup_shutdown_queue, None)
 */
    __pyx_t_1 = ((!(__pyx_v_self->is_shutting_down != 0)) != 0);
    if (__pyx_t_1) {

      /* "src/python/grpcio/grpc/_cython/_cygrpc/server.pyx.pxi":139
 *       elif not self.is_shutting_down:
 *         # the user didn't call shutdown - use our backup queue
 *         self._c_shutdown(self.backup_shutdown_queue, None)             # <<<<<<<<<<<<<<
 *         # and now we wait
 *         while not self.is_shutdown:
 */
      __pyx_t_2 = ((PyObject *)__pyx_v_self->backup_shutdown_queue);
      __Pyx_INCREF(__pyx_t_2);
      __pyx_t_3 = ((struct __pyx_vtabstruct_4grpc_7_cython_6cygrpc_Server *)__pyx_v_self->__pyx_vtab)->_c_shutdown(__pyx_v_self, ((struct __pyx_obj_4grpc_7_cython_6cygrpc_CompletionQueue *)__pyx_t_2), Py_None); if (unlikely(!__pyx_t_3)) __PYX_ERR(5, 139, __pyx_L1_error)
      __Pyx_GOTREF(__pyx_t_3);
      __Pyx_DECREF(__pyx_t_2); __pyx_t_2 = 0;
      __Pyx_DECREF(__pyx_t_3); __pyx_t_3 = 0;

      /* "src/python/grpcio/grpc/_cython/_cygrpc/server.pyx.pxi":141
 *         self._c_shutdown(self.backup_shutdown_queue, None)
 *         # and now we wait
 *         while not self.is_shutdown:             # <<<<<<<<<<<<<<
 *           self.backup_shutdown_queue.poll()
 *       else:
 */
      while (1) {
        __pyx_t_1 = ((!(__pyx_v_self->is_shutdown != 0)) != 0);
        if (!__pyx_t_1) break;

        /* "src/python/grpcio/grpc/_cython/_cygrpc/server.pyx.pxi":142
 *         # and now we wait
 *         while not self.is_shutdown:
 *           self.backup_shutdown_queue.poll()             # <<<<<<<<<<<<<<
 *       else:
 *         # We're in the process of shutting down, but have not shutdown; can't do
 */
        __pyx_t_2 = __Pyx_PyObject_GetAttrStr(((PyObject *)__pyx_v_self->backup_shutdown_queue), __pyx_n_s_poll); if (unlikely(!__pyx_t_2)) __PYX_ERR(5, 142, __pyx_L1_error)
        __Pyx_GOTREF(__pyx_t_2);
        __pyx_t_4 = NULL;
        if (CYTHON_UNPACK_METHODS && likely(PyMethod_Check(__pyx_t_2))) {
          __pyx_t_4 = PyMethod_GET_SELF(__pyx_t_2);
          if (likely(__pyx_t_4)) {
            PyObject* function = PyMethod_GET_FUNCTION(__pyx_t_2);
            __Pyx_INCREF(__pyx_t_4);
            __Pyx_INCREF(function);
            __Pyx_DECREF_SET(__pyx_t_2, function);
          }
        }
        if (__pyx_t_4) {
          __pyx_t_3 = __Pyx_PyObject_CallOneArg(__pyx_t_2, __pyx_t_4); if (unlikely(!__pyx_t_3)) __PYX_ERR(5, 142, __pyx_L1_error)
          __Pyx_DECREF(__pyx_t_4); __pyx_t_4 = 0;
        } else {
          __pyx_t_3 = __Pyx_PyObject_CallNoArg(__pyx_t_2); if (unlikely(!__pyx_t_3)) __PYX_ERR(5, 142, __pyx_L1_error)
        }
        __Pyx_GOTREF(__pyx_t_3);
        __Pyx_DECREF(__pyx_t_2); __pyx_t_2 = 0;
        __Pyx_DECREF(__pyx_t_3); __pyx_t_3 = 0;
      }

      /* "src/python/grpcio/grpc/_cython/_cygrpc/server.pyx.pxi":137
 *       elif self.is_shutdown:
 *         pass
 *       elif not self.is_shutting_down:             # <<<<<<<<<<<<<<
 *         # the user didn't call shutdown - use our backup queue
 *         self._c_shutdown(self.backup_shutdown_queue, None)
 */
      goto __pyx_L4;
    }

    /* "src/python/grpcio/grpc/_cython/_cygrpc/server.pyx.pxi":146
 *         # We're in the process of shutting down, but have not shutdown; can't do
 *         # much but repeatedly release the GIL and wait
 *         while not self.is_shutdown:             # <<<<<<<<<<<<<<
 *           time.sleep(0)
 *       grpc_server_destroy(self.c_server)
 */
    /*else*/ {
      while (1) {
        __pyx_t_1 = ((!(__pyx_v_self->is_shutdown != 0)) != 0);
        if (!__pyx_t_1) break;

        /* "src/python/grpcio/grpc/_cython/_cygrpc/server.pyx.pxi":147
 *         # much but repeatedly release the GIL and wait
 *         while not self.is_shutdown:
 *           time.sleep(0)             # <<<<<<<<<<<<<<
 *       grpc_server_destroy(self.c_server)
 *       self.c_server = NULL
 */
        __pyx_t_3 = __Pyx_GetModuleGlobalName(__pyx_n_s_time); if (unlikely(!__pyx_t_3)) __PYX_ERR(5, 147, __pyx_L1_error)
        __Pyx_GOTREF(__pyx_t_3);
        __pyx_t_2 = __Pyx_PyObject_GetAttrStr(__pyx_t_3, __pyx_n_s_sleep); if (unlikely(!__pyx_t_2)) __PYX_ERR(5, 147, __pyx_L1_error)
        __Pyx_GOTREF(__pyx_t_2);
        __Pyx_DECREF(__pyx_t_3); __pyx_t_3 = 0;
        __pyx_t_3 = __Pyx_PyObject_Call(__pyx_t_2, __pyx_tuple__117, NULL); if (unlikely(!__pyx_t_3)) __PYX_ERR(5, 147, __pyx_L1_error)
        __Pyx_GOTREF(__pyx_t_3);
        __Pyx_DECREF(__pyx_t_2); __pyx_t_2 = 0;
        __Pyx_DECREF(__pyx_t_3); __pyx_t_3 = 0;
      }
    }
    __pyx_L4:;

    /* "src/python/grpcio/grpc/_cython/_cygrpc/server.pyx.pxi":148
 *         while not self.is_shutdown:
 *           time.sleep(0)
 *       grpc_server_destroy(self.c_server)             # <<<<<<<<<<<<<<
 *       self.c_server = NULL
 * 
 */
    grpc_server_destroy(__pyx_v_self->c_server);

    /* "src/python/grpcio/grpc/_cython/_cygrpc/server.pyx.pxi":149
 *           time.sleep(0)
 *       grpc_server_destroy(self.c_server)
 *       self.c_server = NULL             # <<<<<<<<<<<<<<
 * 
 *   def __dealloc__(self):
 */
    __pyx_v_self->c_server = NULL;

    /* "src/python/grpcio/grpc/_cython/_cygrpc/server.pyx.pxi":132
 *   # backup_shutdown_queue.
 *   def destroy(self):
 *     if self.c_server != NULL:             # <<<<<<<<<<<<<<
 *       if not self.is_started:
 *         pass
 */
  }

  /* "src/python/grpcio/grpc/_cython/_cygrpc/server.pyx.pxi":131
 *   # portion of this is safe to call from __dealloc__, and potentially remove
 *   # backup_shutdown_queue.
 *   def destroy(self):             # <<<<<<<<<<<<<<
 *     if self.c_server != NULL:
 *       if not self.is_started:
 */

  /* function exit code */
  __pyx_r = Py_None; __Pyx_INCREF(Py_None);
  goto __pyx_L0;
  __pyx_L1_error:;
  __Pyx_XDECREF(__pyx_t_2);
  __Pyx_XDECREF(__pyx_t_3);
  __Pyx_XDECREF(__pyx_t_4);
  __Pyx_AddTraceback("grpc._cython.cygrpc.Server.destroy", __pyx_clineno, __pyx_lineno, __pyx_filename);
  __pyx_r = NULL;
  __pyx_L0:;
  __Pyx_XGIVEREF(__pyx_r);
  __Pyx_RefNannyFinishContext();
  return __pyx_r;
}

/* "src/python/grpcio/grpc/_cython/_cygrpc/server.pyx.pxi":151
 *       self.c_server = NULL
 * 
 *   def __dealloc__(self):             # <<<<<<<<<<<<<<
 *     if self.c_server == NULL:
 *       grpc_shutdown_blocking()
 */

/* Python wrapper */
static void __pyx_pw_4grpc_7_cython_6cygrpc_6Server_17__dealloc__(PyObject *__pyx_v_self); /*proto*/
static void __pyx_pw_4grpc_7_cython_6cygrpc_6Server_17__dealloc__(PyObject *__pyx_v_self) {
  __Pyx_RefNannyDeclarations
  __Pyx_RefNannySetupContext("__dealloc__ (wrapper)", 0);
  __pyx_pf_4grpc_7_cython_6cygrpc_6Server_16__dealloc__(((struct __pyx_obj_4grpc_7_cython_6cygrpc_Server *)__pyx_v_self));

  /* function exit code */
  __Pyx_RefNannyFinishContext();
}

static void __pyx_pf_4grpc_7_cython_6cygrpc_6Server_16__dealloc__(struct __pyx_obj_4grpc_7_cython_6cygrpc_Server *__pyx_v_self) {
  __Pyx_RefNannyDeclarations
  int __pyx_t_1;
  __Pyx_RefNannySetupContext("__dealloc__", 0);

  /* "src/python/grpcio/grpc/_cython/_cygrpc/server.pyx.pxi":152
 * 
 *   def __dealloc__(self):
 *     if self.c_server == NULL:             # <<<<<<<<<<<<<<
 *       grpc_shutdown_blocking()
 */
  __pyx_t_1 = ((__pyx_v_self->c_server == NULL) != 0);
  if (__pyx_t_1) {

    /* "src/python/grpcio/grpc/_cython/_cygrpc/server.pyx.pxi":153
 *   def __dealloc__(self):
 *     if self.c_server == NULL:
 *       grpc_shutdown_blocking()             # <<<<<<<<<<<<<<
 */
    grpc_shutdown_blocking();

    /* "src/python/grpcio/grpc/_cython/_cygrpc/server.pyx.pxi":152
 * 
 *   def __dealloc__(self):
 *     if self.c_server == NULL:             # <<<<<<<<<<<<<<
 *       grpc_shutdown_blocking()
 */
  }

  /* "src/python/grpcio/grpc/_cython/_cygrpc/server.pyx.pxi":151
 *       self.c_server = NULL
 * 
 *   def __dealloc__(self):             # <<<<<<<<<<<<<<
 *     if self.c_server == NULL:
 *       grpc_shutdown_blocking()
 */

  /* function exit code */
  __Pyx_RefNannyFinishContext();
}

/* "(tree fragment)":1
 * def __reduce_cython__(self):             # <<<<<<<<<<<<<<
 *     raise TypeError("no default __reduce__ due to non-trivial __cinit__")
 * def __setstate_cython__(self, __pyx_state):
 */

/* Python wrapper */
static PyObject *__pyx_pw_4grpc_7_cython_6cygrpc_6Server_19__reduce_cython__(PyObject *__pyx_v_self, CYTHON_UNUSED PyObject *unused); /*proto*/
static PyObject *__pyx_pw_4grpc_7_cython_6cygrpc_6Server_19__reduce_cython__(PyObject *__pyx_v_self, CYTHON_UNUSED PyObject *unused) {
  PyObject *__pyx_r = 0;
  __Pyx_RefNannyDeclarations
  __Pyx_RefNannySetupContext("__reduce_cython__ (wrapper)", 0);
  __pyx_r = __pyx_pf_4grpc_7_cython_6cygrpc_6Server_18__reduce_cython__(((struct __pyx_obj_4grpc_7_cython_6cygrpc_Server *)__pyx_v_self));

  /* function exit code */
  __Pyx_RefNannyFinishContext();
  return __pyx_r;
}

static PyObject *__pyx_pf_4grpc_7_cython_6cygrpc_6Server_18__reduce_cython__(CYTHON_UNUSED struct __pyx_obj_4grpc_7_cython_6cygrpc_Server *__pyx_v_self) {
  PyObject *__pyx_r = NULL;
  __Pyx_RefNannyDeclarations
  PyObject *__pyx_t_1 = NULL;
  __Pyx_RefNannySetupContext("__reduce_cython__", 0);

  /* "(tree fragment)":2
 * def __reduce_cython__(self):
 *     raise TypeError("no default __reduce__ due to non-trivial __cinit__")             # <<<<<<<<<<<<<<
 * def __setstate_cython__(self, __pyx_state):
 *     raise TypeError("no default __reduce__ due to non-trivial __cinit__")
 */
  __pyx_t_1 = __Pyx_PyObject_Call(__pyx_builtin_TypeError, __pyx_tuple__118, NULL); if (unlikely(!__pyx_t_1)) __PYX_ERR(7, 2, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_1);
  __Pyx_Raise(__pyx_t_1, 0, 0, 0);
  __Pyx_DECREF(__pyx_t_1); __pyx_t_1 = 0;
  __PYX_ERR(7, 2, __pyx_L1_error)

  /* "(tree fragment)":1
 * def __reduce_cython__(self):             # <<<<<<<<<<<<<<
 *     raise TypeError("no default __reduce__ due to non-trivial __cinit__")
 * def __setstate_cython__(self, __pyx_state):
 */

  /* function exit code */
  __pyx_L1_error:;
  __Pyx_XDECREF(__pyx_t_1);
  __Pyx_AddTraceback("grpc._cython.cygrpc.Server.__reduce_cython__", __pyx_clineno, __pyx_lineno, __pyx_filename);
  __pyx_r = NULL;
  __Pyx_XGIVEREF(__pyx_r);
  __Pyx_RefNannyFinishContext();
  return __pyx_r;
}

/* "(tree fragment)":3
 * def __reduce_cython__(self):
 *     raise TypeError("no default __reduce__ due to non-trivial __cinit__")
 * def __setstate_cython__(self, __pyx_state):             # <<<<<<<<<<<<<<
 *     raise TypeError("no default __reduce__ due to non-trivial __cinit__")
 */

/* Python wrapper */
static PyObject *__pyx_pw_4grpc_7_cython_6cygrpc_6Server_21__setstate_cython__(PyObject *__pyx_v_self, PyObject *__pyx_v___pyx_state); /*proto*/
static PyObject *__pyx_pw_4grpc_7_cython_6cygrpc_6Server_21__setstate_cython__(PyObject *__pyx_v_self, PyObject *__pyx_v___pyx_state) {
  PyObject *__pyx_r = 0;
  __Pyx_RefNannyDeclarations
  __Pyx_RefNannySetupContext("__setstate_cython__ (wrapper)", 0);
  __pyx_r = __pyx_pf_4grpc_7_cython_6cygrpc_6Server_20__setstate_cython__(((struct __pyx_obj_4grpc_7_cython_6cygrpc_Server *)__pyx_v_self), ((PyObject *)__pyx_v___pyx_state));

  /* function exit code */
  __Pyx_RefNannyFinishContext();
  return __pyx_r;
}

static PyObject *__pyx_pf_4grpc_7_cython_6cygrpc_6Server_20__setstate_cython__(CYTHON_UNUSED struct __pyx_obj_4grpc_7_cython_6cygrpc_Server *__pyx_v_self, CYTHON_UNUSED PyObject *__pyx_v___pyx_state) {
  PyObject *__pyx_r = NULL;
  __Pyx_RefNannyDeclarations
  PyObject *__pyx_t_1 = NULL;
  __Pyx_RefNannySetupContext("__setstate_cython__", 0);

  /* "(tree fragment)":4
 *     raise TypeError("no default __reduce__ due to non-trivial __cinit__")
 * def __setstate_cython__(self, __pyx_state):
 *     raise TypeError("no default __reduce__ due to non-trivial __cinit__")             # <<<<<<<<<<<<<<
 */
  __pyx_t_1 = __Pyx_PyObject_Call(__pyx_builtin_TypeError, __pyx_tuple__119, NULL); if (unlikely(!__pyx_t_1)) __PYX_ERR(7, 4, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_1);
  __Pyx_Raise(__pyx_t_1, 0, 0, 0);
  __Pyx_DECREF(__pyx_t_1); __pyx_t_1 = 0;
  __PYX_ERR(7, 4, __pyx_L1_error)

  /* "(tree fragment)":3
 * def __reduce_cython__(self):
 *     raise TypeError("no default __reduce__ due to non-trivial __cinit__")
 * def __setstate_cython__(self, __pyx_state):             # <<<<<<<<<<<<<<
 *     raise TypeError("no default __reduce__ due to non-trivial __cinit__")
 */

  /* function exit code */
  __pyx_L1_error:;
  __Pyx_XDECREF(__pyx_t_1);
  __Pyx_AddTraceback("grpc._cython.cygrpc.Server.__setstate_cython__", __pyx_clineno, __pyx_lineno, __pyx_filename);
  __pyx_r = NULL;
  __Pyx_XGIVEREF(__pyx_r);
  __Pyx_RefNannyFinishContext();
  return __pyx_r;
}

/* "src/python/grpcio/grpc/_cython/_cygrpc/tag.pyx.pxi":18
 * cdef class _Tag:
 * 
 *   cdef object event(self, grpc_event c_event):             # <<<<<<<<<<<<<<
 *     raise NotImplementedError()
 * 
 */

static PyObject *__pyx_f_4grpc_7_cython_6cygrpc_4_Tag_event(CYTHON_UNUSED struct __pyx_obj_4grpc_7_cython_6cygrpc__Tag *__pyx_v_self, CYTHON_UNUSED grpc_event __pyx_v_c_event) {
  PyObject *__pyx_r = NULL;
  __Pyx_RefNannyDeclarations
  PyObject *__pyx_t_1 = NULL;
  __Pyx_RefNannySetupContext("event", 0);

  /* "src/python/grpcio/grpc/_cython/_cygrpc/tag.pyx.pxi":19
 * 
 *   cdef object event(self, grpc_event c_event):
 *     raise NotImplementedError()             # <<<<<<<<<<<<<<
 * 
 * 
 */
  __pyx_t_1 = __Pyx_PyObject_CallNoArg(__pyx_builtin_NotImplementedError); if (unlikely(!__pyx_t_1)) __PYX_ERR(18, 19, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_1);
  __Pyx_Raise(__pyx_t_1, 0, 0, 0);
  __Pyx_DECREF(__pyx_t_1); __pyx_t_1 = 0;
  __PYX_ERR(18, 19, __pyx_L1_error)

  /* "src/python/grpcio/grpc/_cython/_cygrpc/tag.pyx.pxi":18
 * cdef class _Tag:
 * 
 *   cdef object event(self, grpc_event c_event):             # <<<<<<<<<<<<<<
 *     raise NotImplementedError()
 * 
 */

  /* function exit code */
  __pyx_L1_error:;
  __Pyx_XDECREF(__pyx_t_1);
  __Pyx_AddTraceback("grpc._cython.cygrpc._Tag.event", __pyx_clineno, __pyx_lineno, __pyx_filename);
  __pyx_r = 0;
  __Pyx_XGIVEREF(__pyx_r);
  __Pyx_RefNannyFinishContext();
  return __pyx_r;
}

/* "(tree fragment)":1
 * def __reduce_cython__(self):             # <<<<<<<<<<<<<<
 *     cdef bint use_setstate
 *     state = ()
 */

/* Python wrapper */
static PyObject *__pyx_pw_4grpc_7_cython_6cygrpc_4_Tag_1__reduce_cython__(PyObject *__pyx_v_self, CYTHON_UNUSED PyObject *unused); /*proto*/
static PyObject *__pyx_pw_4grpc_7_cython_6cygrpc_4_Tag_1__reduce_cython__(PyObject *__pyx_v_self, CYTHON_UNUSED PyObject *unused) {
  PyObject *__pyx_r = 0;
  __Pyx_RefNannyDeclarations
  __Pyx_RefNannySetupContext("__reduce_cython__ (wrapper)", 0);
  __pyx_r = __pyx_pf_4grpc_7_cython_6cygrpc_4_Tag___reduce_cython__(((struct __pyx_obj_4grpc_7_cython_6cygrpc__Tag *)__pyx_v_self));

  /* function exit code */
  __Pyx_RefNannyFinishContext();
  return __pyx_r;
}

static PyObject *__pyx_pf_4grpc_7_cython_6cygrpc_4_Tag___reduce_cython__(struct __pyx_obj_4grpc_7_cython_6cygrpc__Tag *__pyx_v_self) {
  int __pyx_v_use_setstate;
  PyObject *__pyx_v_state = NULL;
  PyObject *__pyx_v__dict = NULL;
  PyObject *__pyx_r = NULL;
  __Pyx_RefNannyDeclarations
  PyObject *__pyx_t_1 = NULL;
  int __pyx_t_2;
  int __pyx_t_3;
  PyObject *__pyx_t_4 = NULL;
  PyObject *__pyx_t_5 = NULL;
  __Pyx_RefNannySetupContext("__reduce_cython__", 0);

  /* "(tree fragment)":3
 * def __reduce_cython__(self):
 *     cdef bint use_setstate
 *     state = ()             # <<<<<<<<<<<<<<
 *     _dict = getattr(self, '__dict__', None)
 *     if _dict is not None:
 */
  __Pyx_INCREF(__pyx_empty_tuple);
  __pyx_v_state = __pyx_empty_tuple;

  /* "(tree fragment)":4
 *     cdef bint use_setstate
 *     state = ()
 *     _dict = getattr(self, '__dict__', None)             # <<<<<<<<<<<<<<
 *     if _dict is not None:
 *         state += (_dict,)
 */
  __pyx_t_1 = __Pyx_GetAttr3(((PyObject *)__pyx_v_self), __pyx_n_s_dict, Py_None); if (unlikely(!__pyx_t_1)) __PYX_ERR(7, 4, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_1);
  __pyx_v__dict = __pyx_t_1;
  __pyx_t_1 = 0;

  /* "(tree fragment)":5
 *     state = ()
 *     _dict = getattr(self, '__dict__', None)
 *     if _dict is not None:             # <<<<<<<<<<<<<<
 *         state += (_dict,)
 *         use_setstate = True
 */
  __pyx_t_2 = (__pyx_v__dict != Py_None);
  __pyx_t_3 = (__pyx_t_2 != 0);
  if (__pyx_t_3) {

    /* "(tree fragment)":6
 *     _dict = getattr(self, '__dict__', None)
 *     if _dict is not None:
 *         state += (_dict,)             # <<<<<<<<<<<<<<
 *         use_setstate = True
 *     else:
 */
    __pyx_t_1 = PyTuple_New(1); if (unlikely(!__pyx_t_1)) __PYX_ERR(7, 6, __pyx_L1_error)
    __Pyx_GOTREF(__pyx_t_1);
    __Pyx_INCREF(__pyx_v__dict);
    __Pyx_GIVEREF(__pyx_v__dict);
    PyTuple_SET_ITEM(__pyx_t_1, 0, __pyx_v__dict);
    __pyx_t_4 = PyNumber_InPlaceAdd(__pyx_v_state, __pyx_t_1); if (unlikely(!__pyx_t_4)) __PYX_ERR(7, 6, __pyx_L1_error)
    __Pyx_GOTREF(__pyx_t_4);
    __Pyx_DECREF(__pyx_t_1); __pyx_t_1 = 0;
    __Pyx_DECREF_SET(__pyx_v_state, ((PyObject*)__pyx_t_4));
    __pyx_t_4 = 0;

    /* "(tree fragment)":7
 *     if _dict is not None:
 *         state += (_dict,)
 *         use_setstate = True             # <<<<<<<<<<<<<<
 *     else:
 *         use_setstate = False
 */
    __pyx_v_use_setstate = 1;

    /* "(tree fragment)":5
 *     state = ()
 *     _dict = getattr(self, '__dict__', None)
 *     if _dict is not None:             # <<<<<<<<<<<<<<
 *         state += (_dict,)
 *         use_setstate = True
 */
    goto __pyx_L3;
  }

  /* "(tree fragment)":9
 *         use_setstate = True
 *     else:
 *         use_setstate = False             # <<<<<<<<<<<<<<
 *     if use_setstate:
 *         return __pyx_unpickle__Tag, (type(self), 0xd41d8cd, None), state
 */
  /*else*/ {
    __pyx_v_use_setstate = 0;
  }
  __pyx_L3:;

  /* "(tree fragment)":10
 *     else:
 *         use_setstate = False
 *     if use_setstate:             # <<<<<<<<<<<<<<
 *         return __pyx_unpickle__Tag, (type(self), 0xd41d8cd, None), state
 *     else:
 */
  __pyx_t_3 = (__pyx_v_use_setstate != 0);
  if (__pyx_t_3) {

    /* "(tree fragment)":11
 *         use_setstate = False
 *     if use_setstate:
 *         return __pyx_unpickle__Tag, (type(self), 0xd41d8cd, None), state             # <<<<<<<<<<<<<<
 *     else:
 *         return __pyx_unpickle__Tag, (type(self), 0xd41d8cd, state)
 */
    __Pyx_XDECREF(__pyx_r);
    __pyx_t_4 = __Pyx_GetModuleGlobalName(__pyx_n_s_pyx_unpickle__Tag); if (unlikely(!__pyx_t_4)) __PYX_ERR(7, 11, __pyx_L1_error)
    __Pyx_GOTREF(__pyx_t_4);
    __pyx_t_1 = PyTuple_New(3); if (unlikely(!__pyx_t_1)) __PYX_ERR(7, 11, __pyx_L1_error)
    __Pyx_GOTREF(__pyx_t_1);
    __Pyx_INCREF(((PyObject *)Py_TYPE(((PyObject *)__pyx_v_self))));
    __Pyx_GIVEREF(((PyObject *)Py_TYPE(((PyObject *)__pyx_v_self))));
    PyTuple_SET_ITEM(__pyx_t_1, 0, ((PyObject *)Py_TYPE(((PyObject *)__pyx_v_self))));
    __Pyx_INCREF(__pyx_int_222419149);
    __Pyx_GIVEREF(__pyx_int_222419149);
    PyTuple_SET_ITEM(__pyx_t_1, 1, __pyx_int_222419149);
    __Pyx_INCREF(Py_None);
    __Pyx_GIVEREF(Py_None);
    PyTuple_SET_ITEM(__pyx_t_1, 2, Py_None);
    __pyx_t_5 = PyTuple_New(3); if (unlikely(!__pyx_t_5)) __PYX_ERR(7, 11, __pyx_L1_error)
    __Pyx_GOTREF(__pyx_t_5);
    __Pyx_GIVEREF(__pyx_t_4);
    PyTuple_SET_ITEM(__pyx_t_5, 0, __pyx_t_4);
    __Pyx_GIVEREF(__pyx_t_1);
    PyTuple_SET_ITEM(__pyx_t_5, 1, __pyx_t_1);
    __Pyx_INCREF(__pyx_v_state);
    __Pyx_GIVEREF(__pyx_v_state);
    PyTuple_SET_ITEM(__pyx_t_5, 2, __pyx_v_state);
    __pyx_t_4 = 0;
    __pyx_t_1 = 0;
    __pyx_r = __pyx_t_5;
    __pyx_t_5 = 0;
    goto __pyx_L0;

    /* "(tree fragment)":10
 *     else:
 *         use_setstate = False
 *     if use_setstate:             # <<<<<<<<<<<<<<
 *         return __pyx_unpickle__Tag, (type(self), 0xd41d8cd, None), state
 *     else:
 */
  }

  /* "(tree fragment)":13
 *         return __pyx_unpickle__Tag, (type(self), 0xd41d8cd, None), state
 *     else:
 *         return __pyx_unpickle__Tag, (type(self), 0xd41d8cd, state)             # <<<<<<<<<<<<<<
 * def __setstate_cython__(self, __pyx_state):
 *     __pyx_unpickle__Tag__set_state(self, __pyx_state)
 */
  /*else*/ {
    __Pyx_XDECREF(__pyx_r);
    __pyx_t_5 = __Pyx_GetModuleGlobalName(__pyx_n_s_pyx_unpickle__Tag); if (unlikely(!__pyx_t_5)) __PYX_ERR(7, 13, __pyx_L1_error)
    __Pyx_GOTREF(__pyx_t_5);
    __pyx_t_1 = PyTuple_New(3); if (unlikely(!__pyx_t_1)) __PYX_ERR(7, 13, __pyx_L1_error)
    __Pyx_GOTREF(__pyx_t_1);
    __Pyx_INCREF(((PyObject *)Py_TYPE(((PyObject *)__pyx_v_self))));
    __Pyx_GIVEREF(((PyObject *)Py_TYPE(((PyObject *)__pyx_v_self))));
    PyTuple_SET_ITEM(__pyx_t_1, 0, ((PyObject *)Py_TYPE(((PyObject *)__pyx_v_self))));
    __Pyx_INCREF(__pyx_int_222419149);
    __Pyx_GIVEREF(__pyx_int_222419149);
    PyTuple_SET_ITEM(__pyx_t_1, 1, __pyx_int_222419149);
    __Pyx_INCREF(__pyx_v_state);
    __Pyx_GIVEREF(__pyx_v_state);
    PyTuple_SET_ITEM(__pyx_t_1, 2, __pyx_v_state);
    __pyx_t_4 = PyTuple_New(2); if (unlikely(!__pyx_t_4)) __PYX_ERR(7, 13, __pyx_L1_error)
    __Pyx_GOTREF(__pyx_t_4);
    __Pyx_GIVEREF(__pyx_t_5);
    PyTuple_SET_ITEM(__pyx_t_4, 0, __pyx_t_5);
    __Pyx_GIVEREF(__pyx_t_1);
    PyTuple_SET_ITEM(__pyx_t_4, 1, __pyx_t_1);
    __pyx_t_5 = 0;
    __pyx_t_1 = 0;
    __pyx_r = __pyx_t_4;
    __pyx_t_4 = 0;
    goto __pyx_L0;
  }

  /* "(tree fragment)":1
 * def __reduce_cython__(self):             # <<<<<<<<<<<<<<
 *     cdef bint use_setstate
 *     state = ()
 */

  /* function exit code */
  __pyx_L1_error:;
  __Pyx_XDECREF(__pyx_t_1);
  __Pyx_XDECREF(__pyx_t_4);
  __Pyx_XDECREF(__pyx_t_5);
  __Pyx_AddTraceback("grpc._cython.cygrpc._Tag.__reduce_cython__", __pyx_clineno, __pyx_lineno, __pyx_filename);
  __pyx_r = NULL;
  __pyx_L0:;
  __Pyx_XDECREF(__pyx_v_state);
  __Pyx_XDECREF(__pyx_v__dict);
  __Pyx_XGIVEREF(__pyx_r);
  __Pyx_RefNannyFinishContext();
  return __pyx_r;
}

/* "(tree fragment)":14
 *     else:
 *         return __pyx_unpickle__Tag, (type(self), 0xd41d8cd, state)
 * def __setstate_cython__(self, __pyx_state):             # <<<<<<<<<<<<<<
 *     __pyx_unpickle__Tag__set_state(self, __pyx_state)
 */

/* Python wrapper */
static PyObject *__pyx_pw_4grpc_7_cython_6cygrpc_4_Tag_3__setstate_cython__(PyObject *__pyx_v_self, PyObject *__pyx_v___pyx_state); /*proto*/
static PyObject *__pyx_pw_4grpc_7_cython_6cygrpc_4_Tag_3__setstate_cython__(PyObject *__pyx_v_self, PyObject *__pyx_v___pyx_state) {
  PyObject *__pyx_r = 0;
  __Pyx_RefNannyDeclarations
  __Pyx_RefNannySetupContext("__setstate_cython__ (wrapper)", 0);
  __pyx_r = __pyx_pf_4grpc_7_cython_6cygrpc_4_Tag_2__setstate_cython__(((struct __pyx_obj_4grpc_7_cython_6cygrpc__Tag *)__pyx_v_self), ((PyObject *)__pyx_v___pyx_state));

  /* function exit code */
  __Pyx_RefNannyFinishContext();
  return __pyx_r;
}

static PyObject *__pyx_pf_4grpc_7_cython_6cygrpc_4_Tag_2__setstate_cython__(struct __pyx_obj_4grpc_7_cython_6cygrpc__Tag *__pyx_v_self, PyObject *__pyx_v___pyx_state) {
  PyObject *__pyx_r = NULL;
  __Pyx_RefNannyDeclarations
  PyObject *__pyx_t_1 = NULL;
  __Pyx_RefNannySetupContext("__setstate_cython__", 0);

  /* "(tree fragment)":15
 *         return __pyx_unpickle__Tag, (type(self), 0xd41d8cd, state)
 * def __setstate_cython__(self, __pyx_state):
 *     __pyx_unpickle__Tag__set_state(self, __pyx_state)             # <<<<<<<<<<<<<<
 */
  if (!(likely(PyTuple_CheckExact(__pyx_v___pyx_state))||((__pyx_v___pyx_state) == Py_None)||(PyErr_Format(PyExc_TypeError, "Expected %.16s, got %.200s", "tuple", Py_TYPE(__pyx_v___pyx_state)->tp_name), 0))) __PYX_ERR(7, 15, __pyx_L1_error)
  __pyx_t_1 = __pyx_f_4grpc_7_cython_6cygrpc___pyx_unpickle__Tag__set_state(__pyx_v_self, ((PyObject*)__pyx_v___pyx_state)); if (unlikely(!__pyx_t_1)) __PYX_ERR(7, 15, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_1);
  __Pyx_DECREF(__pyx_t_1); __pyx_t_1 = 0;

  /* "(tree fragment)":14
 *     else:
 *         return __pyx_unpickle__Tag, (type(self), 0xd41d8cd, state)
 * def __setstate_cython__(self, __pyx_state):             # <<<<<<<<<<<<<<
 *     __pyx_unpickle__Tag__set_state(self, __pyx_state)
 */

  /* function exit code */
  __pyx_r = Py_None; __Pyx_INCREF(Py_None);
  goto __pyx_L0;
  __pyx_L1_error:;
  __Pyx_XDECREF(__pyx_t_1);
  __Pyx_AddTraceback("grpc._cython.cygrpc._Tag.__setstate_cython__", __pyx_clineno, __pyx_lineno, __pyx_filename);
  __pyx_r = NULL;
  __pyx_L0:;
  __Pyx_XGIVEREF(__pyx_r);
  __Pyx_RefNannyFinishContext();
  return __pyx_r;
}

/* "src/python/grpcio/grpc/_cython/_cygrpc/tag.pyx.pxi":24
 * cdef class _ConnectivityTag(_Tag):
 * 
 *   def __cinit__(self, user_tag):             # <<<<<<<<<<<<<<
 *     self._user_tag = user_tag
 * 
 */

/* Python wrapper */
static int __pyx_pw_4grpc_7_cython_6cygrpc_16_ConnectivityTag_1__cinit__(PyObject *__pyx_v_self, PyObject *__pyx_args, PyObject *__pyx_kwds); /*proto*/
static int __pyx_pw_4grpc_7_cython_6cygrpc_16_ConnectivityTag_1__cinit__(PyObject *__pyx_v_self, PyObject *__pyx_args, PyObject *__pyx_kwds) {
  PyObject *__pyx_v_user_tag = 0;
  int __pyx_r;
  __Pyx_RefNannyDeclarations
  __Pyx_RefNannySetupContext("__cinit__ (wrapper)", 0);
  {
    static PyObject **__pyx_pyargnames[] = {&__pyx_n_s_user_tag,0};
    PyObject* values[1] = {0};
    if (unlikely(__pyx_kwds)) {
      Py_ssize_t kw_args;
      const Py_ssize_t pos_args = PyTuple_GET_SIZE(__pyx_args);
      switch (pos_args) {
        case  1: values[0] = PyTuple_GET_ITEM(__pyx_args, 0);
        CYTHON_FALLTHROUGH;
        case  0: break;
        default: goto __pyx_L5_argtuple_error;
      }
      kw_args = PyDict_Size(__pyx_kwds);
      switch (pos_args) {
        case  0:
        if (likely((values[0] = __Pyx_PyDict_GetItemStr(__pyx_kwds, __pyx_n_s_user_tag)) != 0)) kw_args--;
        else goto __pyx_L5_argtuple_error;
      }
      if (unlikely(kw_args > 0)) {
        if (unlikely(__Pyx_ParseOptionalKeywords(__pyx_kwds, __pyx_pyargnames, 0, values, pos_args, "__cinit__") < 0)) __PYX_ERR(18, 24, __pyx_L3_error)
      }
    } else if (PyTuple_GET_SIZE(__pyx_args) != 1) {
      goto __pyx_L5_argtuple_error;
    } else {
      values[0] = PyTuple_GET_ITEM(__pyx_args, 0);
    }
    __pyx_v_user_tag = values[0];
  }
  goto __pyx_L4_argument_unpacking_done;
  __pyx_L5_argtuple_error:;
  __Pyx_RaiseArgtupleInvalid("__cinit__", 1, 1, 1, PyTuple_GET_SIZE(__pyx_args)); __PYX_ERR(18, 24, __pyx_L3_error)
  __pyx_L3_error:;
  __Pyx_AddTraceback("grpc._cython.cygrpc._ConnectivityTag.__cinit__", __pyx_clineno, __pyx_lineno, __pyx_filename);
  __Pyx_RefNannyFinishContext();
  return -1;
  __pyx_L4_argument_unpacking_done:;
  __pyx_r = __pyx_pf_4grpc_7_cython_6cygrpc_16_ConnectivityTag___cinit__(((struct __pyx_obj_4grpc_7_cython_6cygrpc__ConnectivityTag *)__pyx_v_self), __pyx_v_user_tag);

  /* function exit code */
  __Pyx_RefNannyFinishContext();
  return __pyx_r;
}

static int __pyx_pf_4grpc_7_cython_6cygrpc_16_ConnectivityTag___cinit__(struct __pyx_obj_4grpc_7_cython_6cygrpc__ConnectivityTag *__pyx_v_self, PyObject *__pyx_v_user_tag) {
  int __pyx_r;
  __Pyx_RefNannyDeclarations
  __Pyx_RefNannySetupContext("__cinit__", 0);

  /* "src/python/grpcio/grpc/_cython/_cygrpc/tag.pyx.pxi":25
 * 
 *   def __cinit__(self, user_tag):
 *     self._user_tag = user_tag             # <<<<<<<<<<<<<<
 * 
 *   cdef ConnectivityEvent event(self, grpc_event c_event):
 */
  __Pyx_INCREF(__pyx_v_user_tag);
  __Pyx_GIVEREF(__pyx_v_user_tag);
  __Pyx_GOTREF(__pyx_v_self->_user_tag);
  __Pyx_DECREF(__pyx_v_self->_user_tag);
  __pyx_v_self->_user_tag = __pyx_v_user_tag;

  /* "src/python/grpcio/grpc/_cython/_cygrpc/tag.pyx.pxi":24
 * cdef class _ConnectivityTag(_Tag):
 * 
 *   def __cinit__(self, user_tag):             # <<<<<<<<<<<<<<
 *     self._user_tag = user_tag
 * 
 */

  /* function exit code */
  __pyx_r = 0;
  __Pyx_RefNannyFinishContext();
  return __pyx_r;
}

/* "src/python/grpcio/grpc/_cython/_cygrpc/tag.pyx.pxi":27
 *     self._user_tag = user_tag
 * 
 *   cdef ConnectivityEvent event(self, grpc_event c_event):             # <<<<<<<<<<<<<<
 *     return ConnectivityEvent(c_event.type, c_event.success, self._user_tag)
 * 
 */

struct __pyx_obj_4grpc_7_cython_6cygrpc_ConnectivityEvent *__pyx_f_4grpc_7_cython_6cygrpc_16_ConnectivityTag_event(struct __pyx_obj_4grpc_7_cython_6cygrpc__ConnectivityTag *__pyx_v_self, grpc_event __pyx_v_c_event) {
  struct __pyx_obj_4grpc_7_cython_6cygrpc_ConnectivityEvent *__pyx_r = NULL;
  __Pyx_RefNannyDeclarations
  PyObject *__pyx_t_1 = NULL;
  PyObject *__pyx_t_2 = NULL;
  PyObject *__pyx_t_3 = NULL;
  __Pyx_RefNannySetupContext("event", 0);

  /* "src/python/grpcio/grpc/_cython/_cygrpc/tag.pyx.pxi":28
 * 
 *   cdef ConnectivityEvent event(self, grpc_event c_event):
 *     return ConnectivityEvent(c_event.type, c_event.success, self._user_tag)             # <<<<<<<<<<<<<<
 * 
 * 
 */
  __Pyx_XDECREF(((PyObject *)__pyx_r));
  __pyx_t_1 = __Pyx_PyInt_From_grpc_completion_type(__pyx_v_c_event.type); if (unlikely(!__pyx_t_1)) __PYX_ERR(18, 28, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_1);
  __pyx_t_2 = __Pyx_PyInt_From_int(__pyx_v_c_event.success); if (unlikely(!__pyx_t_2)) __PYX_ERR(18, 28, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_2);
  __pyx_t_3 = PyTuple_New(3); if (unlikely(!__pyx_t_3)) __PYX_ERR(18, 28, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_3);
  __Pyx_GIVEREF(__pyx_t_1);
  PyTuple_SET_ITEM(__pyx_t_3, 0, __pyx_t_1);
  __Pyx_GIVEREF(__pyx_t_2);
  PyTuple_SET_ITEM(__pyx_t_3, 1, __pyx_t_2);
  __Pyx_INCREF(__pyx_v_self->_user_tag);
  __Pyx_GIVEREF(__pyx_v_self->_user_tag);
  PyTuple_SET_ITEM(__pyx_t_3, 2, __pyx_v_self->_user_tag);
  __pyx_t_1 = 0;
  __pyx_t_2 = 0;
  __pyx_t_2 = __Pyx_PyObject_Call(((PyObject *)__pyx_ptype_4grpc_7_cython_6cygrpc_ConnectivityEvent), __pyx_t_3, NULL); if (unlikely(!__pyx_t_2)) __PYX_ERR(18, 28, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_2);
  __Pyx_DECREF(__pyx_t_3); __pyx_t_3 = 0;
  __pyx_r = ((struct __pyx_obj_4grpc_7_cython_6cygrpc_ConnectivityEvent *)__pyx_t_2);
  __pyx_t_2 = 0;
  goto __pyx_L0;

  /* "src/python/grpcio/grpc/_cython/_cygrpc/tag.pyx.pxi":27
 *     self._user_tag = user_tag
 * 
 *   cdef ConnectivityEvent event(self, grpc_event c_event):             # <<<<<<<<<<<<<<
 *     return ConnectivityEvent(c_event.type, c_event.success, self._user_tag)
 * 
 */

  /* function exit code */
  __pyx_L1_error:;
  __Pyx_XDECREF(__pyx_t_1);
  __Pyx_XDECREF(__pyx_t_2);
  __Pyx_XDECREF(__pyx_t_3);
  __Pyx_AddTraceback("grpc._cython.cygrpc._ConnectivityTag.event", __pyx_clineno, __pyx_lineno, __pyx_filename);
  __pyx_r = 0;
  __pyx_L0:;
  __Pyx_XGIVEREF((PyObject *)__pyx_r);
  __Pyx_RefNannyFinishContext();
  return __pyx_r;
}

struct __pyx_obj_4grpc_7_cython_6cygrpc_ConnectivityEvent *__pyx_f_4grpc_7_cython_6cygrpc_16_ConnectivityTag_event__pyx_wrap_1(struct __pyx_obj_4grpc_7_cython_6cygrpc__ConnectivityTag *__pyx_v_self, grpc_event __pyx_v_c_event) {
  return __pyx_f_4grpc_7_cython_6cygrpc_16_ConnectivityTag_event(__pyx_v_self, __pyx_v_c_event);
}

/* "src/python/grpcio/grpc/_cython/_cygrpc/tag.pxd.pxi":23
 * cdef class _ConnectivityTag(_Tag):
 * 
 *   cdef readonly object _user_tag             # <<<<<<<<<<<<<<
 * 
 *   cdef ConnectivityEvent event(self, grpc_event c_event)
 */

/* Python wrapper */
static PyObject *__pyx_pw_4grpc_7_cython_6cygrpc_16_ConnectivityTag_9_user_tag_1__get__(PyObject *__pyx_v_self); /*proto*/
static PyObject *__pyx_pw_4grpc_7_cython_6cygrpc_16_ConnectivityTag_9_user_tag_1__get__(PyObject *__pyx_v_self) {
  PyObject *__pyx_r = 0;
  __Pyx_RefNannyDeclarations
  __Pyx_RefNannySetupContext("__get__ (wrapper)", 0);
  __pyx_r = __pyx_pf_4grpc_7_cython_6cygrpc_16_ConnectivityTag_9_user_tag___get__(((struct __pyx_obj_4grpc_7_cython_6cygrpc__ConnectivityTag *)__pyx_v_self));

  /* function exit code */
  __Pyx_RefNannyFinishContext();
  return __pyx_r;
}

static PyObject *__pyx_pf_4grpc_7_cython_6cygrpc_16_ConnectivityTag_9_user_tag___get__(struct __pyx_obj_4grpc_7_cython_6cygrpc__ConnectivityTag *__pyx_v_self) {
  PyObject *__pyx_r = NULL;
  __Pyx_RefNannyDeclarations
  __Pyx_RefNannySetupContext("__get__", 0);
  __Pyx_XDECREF(__pyx_r);
  __Pyx_INCREF(__pyx_v_self->_user_tag);
  __pyx_r = __pyx_v_self->_user_tag;
  goto __pyx_L0;

  /* function exit code */
  __pyx_L0:;
  __Pyx_XGIVEREF(__pyx_r);
  __Pyx_RefNannyFinishContext();
  return __pyx_r;
}

/* "(tree fragment)":1
 * def __reduce_cython__(self):             # <<<<<<<<<<<<<<
 *     raise TypeError("no default __reduce__ due to non-trivial __cinit__")
 * def __setstate_cython__(self, __pyx_state):
 */

/* Python wrapper */
static PyObject *__pyx_pw_4grpc_7_cython_6cygrpc_16_ConnectivityTag_3__reduce_cython__(PyObject *__pyx_v_self, CYTHON_UNUSED PyObject *unused); /*proto*/
static PyObject *__pyx_pw_4grpc_7_cython_6cygrpc_16_ConnectivityTag_3__reduce_cython__(PyObject *__pyx_v_self, CYTHON_UNUSED PyObject *unused) {
  PyObject *__pyx_r = 0;
  __Pyx_RefNannyDeclarations
  __Pyx_RefNannySetupContext("__reduce_cython__ (wrapper)", 0);
  __pyx_r = __pyx_pf_4grpc_7_cython_6cygrpc_16_ConnectivityTag_2__reduce_cython__(((struct __pyx_obj_4grpc_7_cython_6cygrpc__ConnectivityTag *)__pyx_v_self));

  /* function exit code */
  __Pyx_RefNannyFinishContext();
  return __pyx_r;
}

static PyObject *__pyx_pf_4grpc_7_cython_6cygrpc_16_ConnectivityTag_2__reduce_cython__(CYTHON_UNUSED struct __pyx_obj_4grpc_7_cython_6cygrpc__ConnectivityTag *__pyx_v_self) {
  PyObject *__pyx_r = NULL;
  __Pyx_RefNannyDeclarations
  PyObject *__pyx_t_1 = NULL;
  __Pyx_RefNannySetupContext("__reduce_cython__", 0);

  /* "(tree fragment)":2
 * def __reduce_cython__(self):
 *     raise TypeError("no default __reduce__ due to non-trivial __cinit__")             # <<<<<<<<<<<<<<
 * def __setstate_cython__(self, __pyx_state):
 *     raise TypeError("no default __reduce__ due to non-trivial __cinit__")
 */
  __pyx_t_1 = __Pyx_PyObject_Call(__pyx_builtin_TypeError, __pyx_tuple__120, NULL); if (unlikely(!__pyx_t_1)) __PYX_ERR(7, 2, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_1);
  __Pyx_Raise(__pyx_t_1, 0, 0, 0);
  __Pyx_DECREF(__pyx_t_1); __pyx_t_1 = 0;
  __PYX_ERR(7, 2, __pyx_L1_error)

  /* "(tree fragment)":1
 * def __reduce_cython__(self):             # <<<<<<<<<<<<<<
 *     raise TypeError("no default __reduce__ due to non-trivial __cinit__")
 * def __setstate_cython__(self, __pyx_state):
 */

  /* function exit code */
  __pyx_L1_error:;
  __Pyx_XDECREF(__pyx_t_1);
  __Pyx_AddTraceback("grpc._cython.cygrpc._ConnectivityTag.__reduce_cython__", __pyx_clineno, __pyx_lineno, __pyx_filename);
  __pyx_r = NULL;
  __Pyx_XGIVEREF(__pyx_r);
  __Pyx_RefNannyFinishContext();
  return __pyx_r;
}

/* "(tree fragment)":3
 * def __reduce_cython__(self):
 *     raise TypeError("no default __reduce__ due to non-trivial __cinit__")
 * def __setstate_cython__(self, __pyx_state):             # <<<<<<<<<<<<<<
 *     raise TypeError("no default __reduce__ due to non-trivial __cinit__")
 */

/* Python wrapper */
static PyObject *__pyx_pw_4grpc_7_cython_6cygrpc_16_ConnectivityTag_5__setstate_cython__(PyObject *__pyx_v_self, PyObject *__pyx_v___pyx_state); /*proto*/
static PyObject *__pyx_pw_4grpc_7_cython_6cygrpc_16_ConnectivityTag_5__setstate_cython__(PyObject *__pyx_v_self, PyObject *__pyx_v___pyx_state) {
  PyObject *__pyx_r = 0;
  __Pyx_RefNannyDeclarations
  __Pyx_RefNannySetupContext("__setstate_cython__ (wrapper)", 0);
  __pyx_r = __pyx_pf_4grpc_7_cython_6cygrpc_16_ConnectivityTag_4__setstate_cython__(((struct __pyx_obj_4grpc_7_cython_6cygrpc__ConnectivityTag *)__pyx_v_self), ((PyObject *)__pyx_v___pyx_state));

  /* function exit code */
  __Pyx_RefNannyFinishContext();
  return __pyx_r;
}

static PyObject *__pyx_pf_4grpc_7_cython_6cygrpc_16_ConnectivityTag_4__setstate_cython__(CYTHON_UNUSED struct __pyx_obj_4grpc_7_cython_6cygrpc__ConnectivityTag *__pyx_v_self, CYTHON_UNUSED PyObject *__pyx_v___pyx_state) {
  PyObject *__pyx_r = NULL;
  __Pyx_RefNannyDeclarations
  PyObject *__pyx_t_1 = NULL;
  __Pyx_RefNannySetupContext("__setstate_cython__", 0);

  /* "(tree fragment)":4
 *     raise TypeError("no default __reduce__ due to non-trivial __cinit__")
 * def __setstate_cython__(self, __pyx_state):
 *     raise TypeError("no default __reduce__ due to non-trivial __cinit__")             # <<<<<<<<<<<<<<
 */
  __pyx_t_1 = __Pyx_PyObject_Call(__pyx_builtin_TypeError, __pyx_tuple__121, NULL); if (unlikely(!__pyx_t_1)) __PYX_ERR(7, 4, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_1);
  __Pyx_Raise(__pyx_t_1, 0, 0, 0);
  __Pyx_DECREF(__pyx_t_1); __pyx_t_1 = 0;
  __PYX_ERR(7, 4, __pyx_L1_error)

  /* "(tree fragment)":3
 * def __reduce_cython__(self):
 *     raise TypeError("no default __reduce__ due to non-trivial __cinit__")
 * def __setstate_cython__(self, __pyx_state):             # <<<<<<<<<<<<<<
 *     raise TypeError("no default __reduce__ due to non-trivial __cinit__")
 */

  /* function exit code */
  __pyx_L1_error:;
  __Pyx_XDECREF(__pyx_t_1);
  __Pyx_AddTraceback("grpc._cython.cygrpc._ConnectivityTag.__setstate_cython__", __pyx_clineno, __pyx_lineno, __pyx_filename);
  __pyx_r = NULL;
  __Pyx_XGIVEREF(__pyx_r);
  __Pyx_RefNannyFinishContext();
  return __pyx_r;
}

/* "src/python/grpcio/grpc/_cython/_cygrpc/tag.pyx.pxi":33
 * cdef class _RequestCallTag(_Tag):
 * 
 *   def __cinit__(self, user_tag, _VTable vtable not None):             # <<<<<<<<<<<<<<
 *     self._user_tag = user_tag
 *     self.call = None
 */

/* Python wrapper */
static int __pyx_pw_4grpc_7_cython_6cygrpc_15_RequestCallTag_1__cinit__(PyObject *__pyx_v_self, PyObject *__pyx_args, PyObject *__pyx_kwds); /*proto*/
static int __pyx_pw_4grpc_7_cython_6cygrpc_15_RequestCallTag_1__cinit__(PyObject *__pyx_v_self, PyObject *__pyx_args, PyObject *__pyx_kwds) {
  PyObject *__pyx_v_user_tag = 0;
  struct __pyx_obj_4grpc_7_cython_6cygrpc__VTable *__pyx_v_vtable = 0;
  int __pyx_r;
  __Pyx_RefNannyDeclarations
  __Pyx_RefNannySetupContext("__cinit__ (wrapper)", 0);
  {
    static PyObject **__pyx_pyargnames[] = {&__pyx_n_s_user_tag,&__pyx_n_s_vtable,0};
    PyObject* values[2] = {0,0};
    if (unlikely(__pyx_kwds)) {
      Py_ssize_t kw_args;
      const Py_ssize_t pos_args = PyTuple_GET_SIZE(__pyx_args);
      switch (pos_args) {
        case  2: values[1] = PyTuple_GET_ITEM(__pyx_args, 1);
        CYTHON_FALLTHROUGH;
        case  1: values[0] = PyTuple_GET_ITEM(__pyx_args, 0);
        CYTHON_FALLTHROUGH;
        case  0: break;
        default: goto __pyx_L5_argtuple_error;
      }
      kw_args = PyDict_Size(__pyx_kwds);
      switch (pos_args) {
        case  0:
        if (likely((values[0] = __Pyx_PyDict_GetItemStr(__pyx_kwds, __pyx_n_s_user_tag)) != 0)) kw_args--;
        else goto __pyx_L5_argtuple_error;
        CYTHON_FALLTHROUGH;
        case  1:
        if (likely((values[1] = __Pyx_PyDict_GetItemStr(__pyx_kwds, __pyx_n_s_vtable)) != 0)) kw_args--;
        else {
          __Pyx_RaiseArgtupleInvalid("__cinit__", 1, 2, 2, 1); __PYX_ERR(18, 33, __pyx_L3_error)
        }
      }
      if (unlikely(kw_args > 0)) {
        if (unlikely(__Pyx_ParseOptionalKeywords(__pyx_kwds, __pyx_pyargnames, 0, values, pos_args, "__cinit__") < 0)) __PYX_ERR(18, 33, __pyx_L3_error)
      }
    } else if (PyTuple_GET_SIZE(__pyx_args) != 2) {
      goto __pyx_L5_argtuple_error;
    } else {
      values[0] = PyTuple_GET_ITEM(__pyx_args, 0);
      values[1] = PyTuple_GET_ITEM(__pyx_args, 1);
    }
    __pyx_v_user_tag = values[0];
    __pyx_v_vtable = ((struct __pyx_obj_4grpc_7_cython_6cygrpc__VTable *)values[1]);
  }
  goto __pyx_L4_argument_unpacking_done;
  __pyx_L5_argtuple_error:;
  __Pyx_RaiseArgtupleInvalid("__cinit__", 1, 2, 2, PyTuple_GET_SIZE(__pyx_args)); __PYX_ERR(18, 33, __pyx_L3_error)
  __pyx_L3_error:;
  __Pyx_AddTraceback("grpc._cython.cygrpc._RequestCallTag.__cinit__", __pyx_clineno, __pyx_lineno, __pyx_filename);
  __Pyx_RefNannyFinishContext();
  return -1;
  __pyx_L4_argument_unpacking_done:;
  if (unlikely(!__Pyx_ArgTypeTest(((PyObject *)__pyx_v_vtable), __pyx_ptype_4grpc_7_cython_6cygrpc__VTable, 0, "vtable", 0))) __PYX_ERR(18, 33, __pyx_L1_error)
  __pyx_r = __pyx_pf_4grpc_7_cython_6cygrpc_15_RequestCallTag___cinit__(((struct __pyx_obj_4grpc_7_cython_6cygrpc__RequestCallTag *)__pyx_v_self), __pyx_v_user_tag, __pyx_v_vtable);

  /* function exit code */
  goto __pyx_L0;
  __pyx_L1_error:;
  __pyx_r = -1;
  __pyx_L0:;
  __Pyx_RefNannyFinishContext();
  return __pyx_r;
}

static int __pyx_pf_4grpc_7_cython_6cygrpc_15_RequestCallTag___cinit__(struct __pyx_obj_4grpc_7_cython_6cygrpc__RequestCallTag *__pyx_v_self, PyObject *__pyx_v_user_tag, struct __pyx_obj_4grpc_7_cython_6cygrpc__VTable *__pyx_v_vtable) {
  int __pyx_r;
  __Pyx_RefNannyDeclarations
  __Pyx_RefNannySetupContext("__cinit__", 0);

  /* "src/python/grpcio/grpc/_cython/_cygrpc/tag.pyx.pxi":34
 * 
 *   def __cinit__(self, user_tag, _VTable vtable not None):
 *     self._user_tag = user_tag             # <<<<<<<<<<<<<<
 *     self.call = None
 *     self.call_details = None
 */
  __Pyx_INCREF(__pyx_v_user_tag);
  __Pyx_GIVEREF(__pyx_v_user_tag);
  __Pyx_GOTREF(__pyx_v_self->_user_tag);
  __Pyx_DECREF(__pyx_v_self->_user_tag);
  __pyx_v_self->_user_tag = __pyx_v_user_tag;

  /* "src/python/grpcio/grpc/_cython/_cygrpc/tag.pyx.pxi":35
 *   def __cinit__(self, user_tag, _VTable vtable not None):
 *     self._user_tag = user_tag
 *     self.call = None             # <<<<<<<<<<<<<<
 *     self.call_details = None
 *     self._vtable = vtable
 */
  __Pyx_INCREF(Py_None);
  __Pyx_GIVEREF(Py_None);
  __Pyx_GOTREF(__pyx_v_self->call);
  __Pyx_DECREF(((PyObject *)__pyx_v_self->call));
  __pyx_v_self->call = ((struct __pyx_obj_4grpc_7_cython_6cygrpc_Call *)Py_None);

  /* "src/python/grpcio/grpc/_cython/_cygrpc/tag.pyx.pxi":36
 *     self._user_tag = user_tag
 *     self.call = None
 *     self.call_details = None             # <<<<<<<<<<<<<<
 *     self._vtable = vtable
 * 
 */
  __Pyx_INCREF(Py_None);
  __Pyx_GIVEREF(Py_None);
  __Pyx_GOTREF(__pyx_v_self->call_details);
  __Pyx_DECREF(((PyObject *)__pyx_v_self->call_details));
  __pyx_v_self->call_details = ((struct __pyx_obj_4grpc_7_cython_6cygrpc_CallDetails *)Py_None);

  /* "src/python/grpcio/grpc/_cython/_cygrpc/tag.pyx.pxi":37
 *     self.call = None
 *     self.call_details = None
 *     self._vtable = vtable             # <<<<<<<<<<<<<<
 * 
 *   cdef void prepare(self) except *:
 */
  __Pyx_INCREF(((PyObject *)__pyx_v_vtable));
  __Pyx_GIVEREF(((PyObject *)__pyx_v_vtable));
  __Pyx_GOTREF(__pyx_v_self->_vtable);
  __Pyx_DECREF(((PyObject *)__pyx_v_self->_vtable));
  __pyx_v_self->_vtable = __pyx_v_vtable;

  /* "src/python/grpcio/grpc/_cython/_cygrpc/tag.pyx.pxi":33
 * cdef class _RequestCallTag(_Tag):
 * 
 *   def __cinit__(self, user_tag, _VTable vtable not None):             # <<<<<<<<<<<<<<
 *     self._user_tag = user_tag
 *     self.call = None
 */

  /* function exit code */
  __pyx_r = 0;
  __Pyx_RefNannyFinishContext();
  return __pyx_r;
}

/* "src/python/grpcio/grpc/_cython/_cygrpc/tag.pyx.pxi":39
 *     self._vtable = vtable
 * 
 *   cdef void prepare(self) except *:             # <<<<<<<<<<<<<<
 *     self.call = Call(self._vtable)
 *     self.call_details = CallDetails()
 */

static void __pyx_f_4grpc_7_cython_6cygrpc_15_RequestCallTag_prepare(struct __pyx_obj_4grpc_7_cython_6cygrpc__RequestCallTag *__pyx_v_self) {
  __Pyx_RefNannyDeclarations
  PyObject *__pyx_t_1 = NULL;
  __Pyx_RefNannySetupContext("prepare", 0);

  /* "src/python/grpcio/grpc/_cython/_cygrpc/tag.pyx.pxi":40
 * 
 *   cdef void prepare(self) except *:
 *     self.call = Call(self._vtable)             # <<<<<<<<<<<<<<
 *     self.call_details = CallDetails()
 *     grpc_metadata_array_init(&self.c_invocation_metadata)
 */
  __pyx_t_1 = __Pyx_PyObject_CallOneArg(((PyObject *)__pyx_ptype_4grpc_7_cython_6cygrpc_Call), ((PyObject *)__pyx_v_self->_vtable)); if (unlikely(!__pyx_t_1)) __PYX_ERR(18, 40, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_1);
  __Pyx_GIVEREF(__pyx_t_1);
  __Pyx_GOTREF(__pyx_v_self->call);
  __Pyx_DECREF(((PyObject *)__pyx_v_self->call));
  __pyx_v_self->call = ((struct __pyx_obj_4grpc_7_cython_6cygrpc_Call *)__pyx_t_1);
  __pyx_t_1 = 0;

  /* "src/python/grpcio/grpc/_cython/_cygrpc/tag.pyx.pxi":41
 *   cdef void prepare(self) except *:
 *     self.call = Call(self._vtable)
 *     self.call_details = CallDetails()             # <<<<<<<<<<<<<<
 *     grpc_metadata_array_init(&self.c_invocation_metadata)
 * 
 */
  __pyx_t_1 = __Pyx_PyObject_CallNoArg(((PyObject *)__pyx_ptype_4grpc_7_cython_6cygrpc_CallDetails)); if (unlikely(!__pyx_t_1)) __PYX_ERR(18, 41, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_1);
  __Pyx_GIVEREF(__pyx_t_1);
  __Pyx_GOTREF(__pyx_v_self->call_details);
  __Pyx_DECREF(((PyObject *)__pyx_v_self->call_details));
  __pyx_v_self->call_details = ((struct __pyx_obj_4grpc_7_cython_6cygrpc_CallDetails *)__pyx_t_1);
  __pyx_t_1 = 0;

  /* "src/python/grpcio/grpc/_cython/_cygrpc/tag.pyx.pxi":42
 *     self.call = Call(self._vtable)
 *     self.call_details = CallDetails()
 *     grpc_metadata_array_init(&self.c_invocation_metadata)             # <<<<<<<<<<<<<<
 * 
 *   cdef RequestCallEvent event(self, grpc_event c_event):
 */
  grpc_metadata_array_init((&__pyx_v_self->c_invocation_metadata));

  /* "src/python/grpcio/grpc/_cython/_cygrpc/tag.pyx.pxi":39
 *     self._vtable = vtable
 * 
 *   cdef void prepare(self) except *:             # <<<<<<<<<<<<<<
 *     self.call = Call(self._vtable)
 *     self.call_details = CallDetails()
 */

  /* function exit code */
  goto __pyx_L0;
  __pyx_L1_error:;
  __Pyx_XDECREF(__pyx_t_1);
  __Pyx_AddTraceback("grpc._cython.cygrpc._RequestCallTag.prepare", __pyx_clineno, __pyx_lineno, __pyx_filename);
  __pyx_L0:;
  __Pyx_RefNannyFinishContext();
}

/* "src/python/grpcio/grpc/_cython/_cygrpc/tag.pyx.pxi":44
 *     grpc_metadata_array_init(&self.c_invocation_metadata)
 * 
 *   cdef RequestCallEvent event(self, grpc_event c_event):             # <<<<<<<<<<<<<<
 *     cdef tuple invocation_metadata = _metadata(&self.c_invocation_metadata)
 *     grpc_metadata_array_destroy(&self.c_invocation_metadata)
 */

struct __pyx_obj_4grpc_7_cython_6cygrpc_RequestCallEvent *__pyx_f_4grpc_7_cython_6cygrpc_15_RequestCallTag_event(struct __pyx_obj_4grpc_7_cython_6cygrpc__RequestCallTag *__pyx_v_self, grpc_event __pyx_v_c_event) {
  PyObject *__pyx_v_invocation_metadata = 0;
  struct __pyx_obj_4grpc_7_cython_6cygrpc_RequestCallEvent *__pyx_r = NULL;
  __Pyx_RefNannyDeclarations
  PyObject *__pyx_t_1 = NULL;
  PyObject *__pyx_t_2 = NULL;
  PyObject *__pyx_t_3 = NULL;
  __Pyx_RefNannySetupContext("event", 0);

  /* "src/python/grpcio/grpc/_cython/_cygrpc/tag.pyx.pxi":45
 * 
 *   cdef RequestCallEvent event(self, grpc_event c_event):
 *     cdef tuple invocation_metadata = _metadata(&self.c_invocation_metadata)             # <<<<<<<<<<<<<<
 *     grpc_metadata_array_destroy(&self.c_invocation_metadata)
 *     return RequestCallEvent(
 */
  __pyx_t_1 = __pyx_f_4grpc_7_cython_6cygrpc__metadata((&__pyx_v_self->c_invocation_metadata)); if (unlikely(!__pyx_t_1)) __PYX_ERR(18, 45, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_1);
  __pyx_v_invocation_metadata = ((PyObject*)__pyx_t_1);
  __pyx_t_1 = 0;

  /* "src/python/grpcio/grpc/_cython/_cygrpc/tag.pyx.pxi":46
 *   cdef RequestCallEvent event(self, grpc_event c_event):
 *     cdef tuple invocation_metadata = _metadata(&self.c_invocation_metadata)
 *     grpc_metadata_array_destroy(&self.c_invocation_metadata)             # <<<<<<<<<<<<<<
 *     return RequestCallEvent(
 *         c_event.type, c_event.success, self._user_tag, self.call,
 */
  grpc_metadata_array_destroy((&__pyx_v_self->c_invocation_metadata));

  /* "src/python/grpcio/grpc/_cython/_cygrpc/tag.pyx.pxi":47
 *     cdef tuple invocation_metadata = _metadata(&self.c_invocation_metadata)
 *     grpc_metadata_array_destroy(&self.c_invocation_metadata)
 *     return RequestCallEvent(             # <<<<<<<<<<<<<<
 *         c_event.type, c_event.success, self._user_tag, self.call,
 *         self.call_details, invocation_metadata)
 */
  __Pyx_XDECREF(((PyObject *)__pyx_r));

  /* "src/python/grpcio/grpc/_cython/_cygrpc/tag.pyx.pxi":48
 *     grpc_metadata_array_destroy(&self.c_invocation_metadata)
 *     return RequestCallEvent(
 *         c_event.type, c_event.success, self._user_tag, self.call,             # <<<<<<<<<<<<<<
 *         self.call_details, invocation_metadata)
 * 
 */
  __pyx_t_1 = __Pyx_PyInt_From_grpc_completion_type(__pyx_v_c_event.type); if (unlikely(!__pyx_t_1)) __PYX_ERR(18, 48, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_1);
  __pyx_t_2 = __Pyx_PyInt_From_int(__pyx_v_c_event.success); if (unlikely(!__pyx_t_2)) __PYX_ERR(18, 48, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_2);

  /* "src/python/grpcio/grpc/_cython/_cygrpc/tag.pyx.pxi":47
 *     cdef tuple invocation_metadata = _metadata(&self.c_invocation_metadata)
 *     grpc_metadata_array_destroy(&self.c_invocation_metadata)
 *     return RequestCallEvent(             # <<<<<<<<<<<<<<
 *         c_event.type, c_event.success, self._user_tag, self.call,
 *         self.call_details, invocation_metadata)
 */
  __pyx_t_3 = PyTuple_New(6); if (unlikely(!__pyx_t_3)) __PYX_ERR(18, 47, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_3);
  __Pyx_GIVEREF(__pyx_t_1);
  PyTuple_SET_ITEM(__pyx_t_3, 0, __pyx_t_1);
  __Pyx_GIVEREF(__pyx_t_2);
  PyTuple_SET_ITEM(__pyx_t_3, 1, __pyx_t_2);
  __Pyx_INCREF(__pyx_v_self->_user_tag);
  __Pyx_GIVEREF(__pyx_v_self->_user_tag);
  PyTuple_SET_ITEM(__pyx_t_3, 2, __pyx_v_self->_user_tag);
  __Pyx_INCREF(((PyObject *)__pyx_v_self->call));
  __Pyx_GIVEREF(((PyObject *)__pyx_v_self->call));
  PyTuple_SET_ITEM(__pyx_t_3, 3, ((PyObject *)__pyx_v_self->call));
  __Pyx_INCREF(((PyObject *)__pyx_v_self->call_details));
  __Pyx_GIVEREF(((PyObject *)__pyx_v_self->call_details));
  PyTuple_SET_ITEM(__pyx_t_3, 4, ((PyObject *)__pyx_v_self->call_details));
  __Pyx_INCREF(__pyx_v_invocation_metadata);
  __Pyx_GIVEREF(__pyx_v_invocation_metadata);
  PyTuple_SET_ITEM(__pyx_t_3, 5, __pyx_v_invocation_metadata);
  __pyx_t_1 = 0;
  __pyx_t_2 = 0;
  __pyx_t_2 = __Pyx_PyObject_Call(((PyObject *)__pyx_ptype_4grpc_7_cython_6cygrpc_RequestCallEvent), __pyx_t_3, NULL); if (unlikely(!__pyx_t_2)) __PYX_ERR(18, 47, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_2);
  __Pyx_DECREF(__pyx_t_3); __pyx_t_3 = 0;
  __pyx_r = ((struct __pyx_obj_4grpc_7_cython_6cygrpc_RequestCallEvent *)__pyx_t_2);
  __pyx_t_2 = 0;
  goto __pyx_L0;

  /* "src/python/grpcio/grpc/_cython/_cygrpc/tag.pyx.pxi":44
 *     grpc_metadata_array_init(&self.c_invocation_metadata)
 * 
 *   cdef RequestCallEvent event(self, grpc_event c_event):             # <<<<<<<<<<<<<<
 *     cdef tuple invocation_metadata = _metadata(&self.c_invocation_metadata)
 *     grpc_metadata_array_destroy(&self.c_invocation_metadata)
 */

  /* function exit code */
  __pyx_L1_error:;
  __Pyx_XDECREF(__pyx_t_1);
  __Pyx_XDECREF(__pyx_t_2);
  __Pyx_XDECREF(__pyx_t_3);
  __Pyx_AddTraceback("grpc._cython.cygrpc._RequestCallTag.event", __pyx_clineno, __pyx_lineno, __pyx_filename);
  __pyx_r = 0;
  __pyx_L0:;
  __Pyx_XDECREF(__pyx_v_invocation_metadata);
  __Pyx_XGIVEREF((PyObject *)__pyx_r);
  __Pyx_RefNannyFinishContext();
  return __pyx_r;
}

struct __pyx_obj_4grpc_7_cython_6cygrpc_RequestCallEvent *__pyx_f_4grpc_7_cython_6cygrpc_15_RequestCallTag_event__pyx_wrap_1(struct __pyx_obj_4grpc_7_cython_6cygrpc__RequestCallTag *__pyx_v_self, grpc_event __pyx_v_c_event) {
  return __pyx_f_4grpc_7_cython_6cygrpc_15_RequestCallTag_event(__pyx_v_self, __pyx_v_c_event);
}

/* "src/python/grpcio/grpc/_cython/_cygrpc/tag.pxd.pxi":30
 * cdef class _RequestCallTag(_Tag):
 * 
 *   cdef readonly object _user_tag             # <<<<<<<<<<<<<<
 *   cdef Call call
 *   cdef _VTable _vtable
 */

/* Python wrapper */
static PyObject *__pyx_pw_4grpc_7_cython_6cygrpc_15_RequestCallTag_9_user_tag_1__get__(PyObject *__pyx_v_self); /*proto*/
static PyObject *__pyx_pw_4grpc_7_cython_6cygrpc_15_RequestCallTag_9_user_tag_1__get__(PyObject *__pyx_v_self) {
  PyObject *__pyx_r = 0;
  __Pyx_RefNannyDeclarations
  __Pyx_RefNannySetupContext("__get__ (wrapper)", 0);
  __pyx_r = __pyx_pf_4grpc_7_cython_6cygrpc_15_RequestCallTag_9_user_tag___get__(((struct __pyx_obj_4grpc_7_cython_6cygrpc__RequestCallTag *)__pyx_v_self));

  /* function exit code */
  __Pyx_RefNannyFinishContext();
  return __pyx_r;
}

static PyObject *__pyx_pf_4grpc_7_cython_6cygrpc_15_RequestCallTag_9_user_tag___get__(struct __pyx_obj_4grpc_7_cython_6cygrpc__RequestCallTag *__pyx_v_self) {
  PyObject *__pyx_r = NULL;
  __Pyx_RefNannyDeclarations
  __Pyx_RefNannySetupContext("__get__", 0);
  __Pyx_XDECREF(__pyx_r);
  __Pyx_INCREF(__pyx_v_self->_user_tag);
  __pyx_r = __pyx_v_self->_user_tag;
  goto __pyx_L0;

  /* function exit code */
  __pyx_L0:;
  __Pyx_XGIVEREF(__pyx_r);
  __Pyx_RefNannyFinishContext();
  return __pyx_r;
}

/* "(tree fragment)":1
 * def __reduce_cython__(self):             # <<<<<<<<<<<<<<
 *     raise TypeError("no default __reduce__ due to non-trivial __cinit__")
 * def __setstate_cython__(self, __pyx_state):
 */

/* Python wrapper */
static PyObject *__pyx_pw_4grpc_7_cython_6cygrpc_15_RequestCallTag_3__reduce_cython__(PyObject *__pyx_v_self, CYTHON_UNUSED PyObject *unused); /*proto*/
static PyObject *__pyx_pw_4grpc_7_cython_6cygrpc_15_RequestCallTag_3__reduce_cython__(PyObject *__pyx_v_self, CYTHON_UNUSED PyObject *unused) {
  PyObject *__pyx_r = 0;
  __Pyx_RefNannyDeclarations
  __Pyx_RefNannySetupContext("__reduce_cython__ (wrapper)", 0);
  __pyx_r = __pyx_pf_4grpc_7_cython_6cygrpc_15_RequestCallTag_2__reduce_cython__(((struct __pyx_obj_4grpc_7_cython_6cygrpc__RequestCallTag *)__pyx_v_self));

  /* function exit code */
  __Pyx_RefNannyFinishContext();
  return __pyx_r;
}

static PyObject *__pyx_pf_4grpc_7_cython_6cygrpc_15_RequestCallTag_2__reduce_cython__(CYTHON_UNUSED struct __pyx_obj_4grpc_7_cython_6cygrpc__RequestCallTag *__pyx_v_self) {
  PyObject *__pyx_r = NULL;
  __Pyx_RefNannyDeclarations
  PyObject *__pyx_t_1 = NULL;
  __Pyx_RefNannySetupContext("__reduce_cython__", 0);

  /* "(tree fragment)":2
 * def __reduce_cython__(self):
 *     raise TypeError("no default __reduce__ due to non-trivial __cinit__")             # <<<<<<<<<<<<<<
 * def __setstate_cython__(self, __pyx_state):
 *     raise TypeError("no default __reduce__ due to non-trivial __cinit__")
 */
  __pyx_t_1 = __Pyx_PyObject_Call(__pyx_builtin_TypeError, __pyx_tuple__122, NULL); if (unlikely(!__pyx_t_1)) __PYX_ERR(7, 2, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_1);
  __Pyx_Raise(__pyx_t_1, 0, 0, 0);
  __Pyx_DECREF(__pyx_t_1); __pyx_t_1 = 0;
  __PYX_ERR(7, 2, __pyx_L1_error)

  /* "(tree fragment)":1
 * def __reduce_cython__(self):             # <<<<<<<<<<<<<<
 *     raise TypeError("no default __reduce__ due to non-trivial __cinit__")
 * def __setstate_cython__(self, __pyx_state):
 */

  /* function exit code */
  __pyx_L1_error:;
  __Pyx_XDECREF(__pyx_t_1);
  __Pyx_AddTraceback("grpc._cython.cygrpc._RequestCallTag.__reduce_cython__", __pyx_clineno, __pyx_lineno, __pyx_filename);
  __pyx_r = NULL;
  __Pyx_XGIVEREF(__pyx_r);
  __Pyx_RefNannyFinishContext();
  return __pyx_r;
}

/* "(tree fragment)":3
 * def __reduce_cython__(self):
 *     raise TypeError("no default __reduce__ due to non-trivial __cinit__")
 * def __setstate_cython__(self, __pyx_state):             # <<<<<<<<<<<<<<
 *     raise TypeError("no default __reduce__ due to non-trivial __cinit__")
 */

/* Python wrapper */
static PyObject *__pyx_pw_4grpc_7_cython_6cygrpc_15_RequestCallTag_5__setstate_cython__(PyObject *__pyx_v_self, PyObject *__pyx_v___pyx_state); /*proto*/
static PyObject *__pyx_pw_4grpc_7_cython_6cygrpc_15_RequestCallTag_5__setstate_cython__(PyObject *__pyx_v_self, PyObject *__pyx_v___pyx_state) {
  PyObject *__pyx_r = 0;
  __Pyx_RefNannyDeclarations
  __Pyx_RefNannySetupContext("__setstate_cython__ (wrapper)", 0);
  __pyx_r = __pyx_pf_4grpc_7_cython_6cygrpc_15_RequestCallTag_4__setstate_cython__(((struct __pyx_obj_4grpc_7_cython_6cygrpc__RequestCallTag *)__pyx_v_self), ((PyObject *)__pyx_v___pyx_state));

  /* function exit code */
  __Pyx_RefNannyFinishContext();
  return __pyx_r;
}

static PyObject *__pyx_pf_4grpc_7_cython_6cygrpc_15_RequestCallTag_4__setstate_cython__(CYTHON_UNUSED struct __pyx_obj_4grpc_7_cython_6cygrpc__RequestCallTag *__pyx_v_self, CYTHON_UNUSED PyObject *__pyx_v___pyx_state) {
  PyObject *__pyx_r = NULL;
  __Pyx_RefNannyDeclarations
  PyObject *__pyx_t_1 = NULL;
  __Pyx_RefNannySetupContext("__setstate_cython__", 0);

  /* "(tree fragment)":4
 *     raise TypeError("no default __reduce__ due to non-trivial __cinit__")
 * def __setstate_cython__(self, __pyx_state):
 *     raise TypeError("no default __reduce__ due to non-trivial __cinit__")             # <<<<<<<<<<<<<<
 */
  __pyx_t_1 = __Pyx_PyObject_Call(__pyx_builtin_TypeError, __pyx_tuple__123, NULL); if (unlikely(!__pyx_t_1)) __PYX_ERR(7, 4, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_1);
  __Pyx_Raise(__pyx_t_1, 0, 0, 0);
  __Pyx_DECREF(__pyx_t_1); __pyx_t_1 = 0;
  __PYX_ERR(7, 4, __pyx_L1_error)

  /* "(tree fragment)":3
 * def __reduce_cython__(self):
 *     raise TypeError("no default __reduce__ due to non-trivial __cinit__")
 * def __setstate_cython__(self, __pyx_state):             # <<<<<<<<<<<<<<
 *     raise TypeError("no default __reduce__ due to non-trivial __cinit__")
 */

  /* function exit code */
  __pyx_L1_error:;
  __Pyx_XDECREF(__pyx_t_1);
  __Pyx_AddTraceback("grpc._cython.cygrpc._RequestCallTag.__setstate_cython__", __pyx_clineno, __pyx_lineno, __pyx_filename);
  __pyx_r = NULL;
  __Pyx_XGIVEREF(__pyx_r);
  __Pyx_RefNannyFinishContext();
  return __pyx_r;
}

/* "src/python/grpcio/grpc/_cython/_cygrpc/tag.pyx.pxi":54
 * cdef class _BatchOperationTag:
 * 
 *   def __cinit__(self, user_tag, operations, call):             # <<<<<<<<<<<<<<
 *     self._user_tag = user_tag
 *     self._operations = operations
 */

/* Python wrapper */
static int __pyx_pw_4grpc_7_cython_6cygrpc_18_BatchOperationTag_1__cinit__(PyObject *__pyx_v_self, PyObject *__pyx_args, PyObject *__pyx_kwds); /*proto*/
static int __pyx_pw_4grpc_7_cython_6cygrpc_18_BatchOperationTag_1__cinit__(PyObject *__pyx_v_self, PyObject *__pyx_args, PyObject *__pyx_kwds) {
  PyObject *__pyx_v_user_tag = 0;
  PyObject *__pyx_v_operations = 0;
  PyObject *__pyx_v_call = 0;
  int __pyx_r;
  __Pyx_RefNannyDeclarations
  __Pyx_RefNannySetupContext("__cinit__ (wrapper)", 0);
  {
    static PyObject **__pyx_pyargnames[] = {&__pyx_n_s_user_tag,&__pyx_n_s_operations,&__pyx_n_s_call,0};
    PyObject* values[3] = {0,0,0};
    if (unlikely(__pyx_kwds)) {
      Py_ssize_t kw_args;
      const Py_ssize_t pos_args = PyTuple_GET_SIZE(__pyx_args);
      switch (pos_args) {
        case  3: values[2] = PyTuple_GET_ITEM(__pyx_args, 2);
        CYTHON_FALLTHROUGH;
        case  2: values[1] = PyTuple_GET_ITEM(__pyx_args, 1);
        CYTHON_FALLTHROUGH;
        case  1: values[0] = PyTuple_GET_ITEM(__pyx_args, 0);
        CYTHON_FALLTHROUGH;
        case  0: break;
        default: goto __pyx_L5_argtuple_error;
      }
      kw_args = PyDict_Size(__pyx_kwds);
      switch (pos_args) {
        case  0:
        if (likely((values[0] = __Pyx_PyDict_GetItemStr(__pyx_kwds, __pyx_n_s_user_tag)) != 0)) kw_args--;
        else goto __pyx_L5_argtuple_error;
        CYTHON_FALLTHROUGH;
        case  1:
        if (likely((values[1] = __Pyx_PyDict_GetItemStr(__pyx_kwds, __pyx_n_s_operations)) != 0)) kw_args--;
        else {
          __Pyx_RaiseArgtupleInvalid("__cinit__", 1, 3, 3, 1); __PYX_ERR(18, 54, __pyx_L3_error)
        }
        CYTHON_FALLTHROUGH;
        case  2:
        if (likely((values[2] = __Pyx_PyDict_GetItemStr(__pyx_kwds, __pyx_n_s_call)) != 0)) kw_args--;
        else {
          __Pyx_RaiseArgtupleInvalid("__cinit__", 1, 3, 3, 2); __PYX_ERR(18, 54, __pyx_L3_error)
        }
      }
      if (unlikely(kw_args > 0)) {
        if (unlikely(__Pyx_ParseOptionalKeywords(__pyx_kwds, __pyx_pyargnames, 0, values, pos_args, "__cinit__") < 0)) __PYX_ERR(18, 54, __pyx_L3_error)
      }
    } else if (PyTuple_GET_SIZE(__pyx_args) != 3) {
      goto __pyx_L5_argtuple_error;
    } else {
      values[0] = PyTuple_GET_ITEM(__pyx_args, 0);
      values[1] = PyTuple_GET_ITEM(__pyx_args, 1);
      values[2] = PyTuple_GET_ITEM(__pyx_args, 2);
    }
    __pyx_v_user_tag = values[0];
    __pyx_v_operations = values[1];
    __pyx_v_call = values[2];
  }
  goto __pyx_L4_argument_unpacking_done;
  __pyx_L5_argtuple_error:;
  __Pyx_RaiseArgtupleInvalid("__cinit__", 1, 3, 3, PyTuple_GET_SIZE(__pyx_args)); __PYX_ERR(18, 54, __pyx_L3_error)
  __pyx_L3_error:;
  __Pyx_AddTraceback("grpc._cython.cygrpc._BatchOperationTag.__cinit__", __pyx_clineno, __pyx_lineno, __pyx_filename);
  __Pyx_RefNannyFinishContext();
  return -1;
  __pyx_L4_argument_unpacking_done:;
  __pyx_r = __pyx_pf_4grpc_7_cython_6cygrpc_18_BatchOperationTag___cinit__(((struct __pyx_obj_4grpc_7_cython_6cygrpc__BatchOperationTag *)__pyx_v_self), __pyx_v_user_tag, __pyx_v_operations, __pyx_v_call);

  /* function exit code */
  __Pyx_RefNannyFinishContext();
  return __pyx_r;
}

static int __pyx_pf_4grpc_7_cython_6cygrpc_18_BatchOperationTag___cinit__(struct __pyx_obj_4grpc_7_cython_6cygrpc__BatchOperationTag *__pyx_v_self, PyObject *__pyx_v_user_tag, PyObject *__pyx_v_operations, PyObject *__pyx_v_call) {
  int __pyx_r;
  __Pyx_RefNannyDeclarations
  __Pyx_RefNannySetupContext("__cinit__", 0);

  /* "src/python/grpcio/grpc/_cython/_cygrpc/tag.pyx.pxi":55
 * 
 *   def __cinit__(self, user_tag, operations, call):
 *     self._user_tag = user_tag             # <<<<<<<<<<<<<<
 *     self._operations = operations
 *     self._retained_call = call
 */
  __Pyx_INCREF(__pyx_v_user_tag);
  __Pyx_GIVEREF(__pyx_v_user_tag);
  __Pyx_GOTREF(__pyx_v_self->_user_tag);
  __Pyx_DECREF(__pyx_v_self->_user_tag);
  __pyx_v_self->_user_tag = __pyx_v_user_tag;

  /* "src/python/grpcio/grpc/_cython/_cygrpc/tag.pyx.pxi":56
 *   def __cinit__(self, user_tag, operations, call):
 *     self._user_tag = user_tag
 *     self._operations = operations             # <<<<<<<<<<<<<<
 *     self._retained_call = call
 * 
 */
  __Pyx_INCREF(__pyx_v_operations);
  __Pyx_GIVEREF(__pyx_v_operations);
  __Pyx_GOTREF(__pyx_v_self->_operations);
  __Pyx_DECREF(__pyx_v_self->_operations);
  __pyx_v_self->_operations = __pyx_v_operations;

  /* "src/python/grpcio/grpc/_cython/_cygrpc/tag.pyx.pxi":57
 *     self._user_tag = user_tag
 *     self._operations = operations
 *     self._retained_call = call             # <<<<<<<<<<<<<<
 * 
 *   cdef void prepare(self) except *:
 */
  __Pyx_INCREF(__pyx_v_call);
  __Pyx_GIVEREF(__pyx_v_call);
  __Pyx_GOTREF(__pyx_v_self->_retained_call);
  __Pyx_DECREF(__pyx_v_self->_retained_call);
  __pyx_v_self->_retained_call = __pyx_v_call;

  /* "src/python/grpcio/grpc/_cython/_cygrpc/tag.pyx.pxi":54
 * cdef class _BatchOperationTag:
 * 
 *   def __cinit__(self, user_tag, operations, call):             # <<<<<<<<<<<<<<
 *     self._user_tag = user_tag
 *     self._operations = operations
 */

  /* function exit code */
  __pyx_r = 0;
  __Pyx_RefNannyFinishContext();
  return __pyx_r;
}

/* "src/python/grpcio/grpc/_cython/_cygrpc/tag.pyx.pxi":59
 *     self._retained_call = call
 * 
 *   cdef void prepare(self) except *:             # <<<<<<<<<<<<<<
 *     cdef Operation operation
 *     self.c_nops = 0 if self._operations is None else len(self._operations)
 */

static void __pyx_f_4grpc_7_cython_6cygrpc_18_BatchOperationTag_prepare(struct __pyx_obj_4grpc_7_cython_6cygrpc__BatchOperationTag *__pyx_v_self) {
  struct __pyx_obj_4grpc_7_cython_6cygrpc_Operation *__pyx_v_operation = 0;
  PyObject *__pyx_v_index = NULL;
  __Pyx_RefNannyDeclarations
  size_t __pyx_t_1;
  int __pyx_t_2;
  PyObject *__pyx_t_3 = NULL;
  Py_ssize_t __pyx_t_4;
  PyObject *__pyx_t_5 = NULL;
  PyObject *(*__pyx_t_6)(PyObject *);
  PyObject *__pyx_t_7 = NULL;
  grpc_op __pyx_t_8;
  Py_ssize_t __pyx_t_9;
  __Pyx_RefNannySetupContext("prepare", 0);

  /* "src/python/grpcio/grpc/_cython/_cygrpc/tag.pyx.pxi":61
 *   cdef void prepare(self) except *:
 *     cdef Operation operation
 *     self.c_nops = 0 if self._operations is None else len(self._operations)             # <<<<<<<<<<<<<<
 *     if 0 < self.c_nops:
 *       self.c_ops = <grpc_op *>gpr_malloc(sizeof(grpc_op) * self.c_nops)
 */
  __pyx_t_2 = (__pyx_v_self->_operations == Py_None);
  if ((__pyx_t_2 != 0)) {
    __pyx_t_1 = 0;
  } else {
    __pyx_t_3 = __pyx_v_self->_operations;
    __Pyx_INCREF(__pyx_t_3);
    __pyx_t_4 = PyObject_Length(__pyx_t_3); if (unlikely(__pyx_t_4 == ((Py_ssize_t)-1))) __PYX_ERR(18, 61, __pyx_L1_error)
    __Pyx_DECREF(__pyx_t_3); __pyx_t_3 = 0;
    __pyx_t_1 = __pyx_t_4;
  }
  __pyx_v_self->c_nops = __pyx_t_1;

  /* "src/python/grpcio/grpc/_cython/_cygrpc/tag.pyx.pxi":62
 *     cdef Operation operation
 *     self.c_nops = 0 if self._operations is None else len(self._operations)
 *     if 0 < self.c_nops:             # <<<<<<<<<<<<<<
 *       self.c_ops = <grpc_op *>gpr_malloc(sizeof(grpc_op) * self.c_nops)
 *       for index, operation in enumerate(self._operations):
 */
  __pyx_t_2 = ((0 < __pyx_v_self->c_nops) != 0);
  if (__pyx_t_2) {

    /* "src/python/grpcio/grpc/_cython/_cygrpc/tag.pyx.pxi":63
 *     self.c_nops = 0 if self._operations is None else len(self._operations)
 *     if 0 < self.c_nops:
 *       self.c_ops = <grpc_op *>gpr_malloc(sizeof(grpc_op) * self.c_nops)             # <<<<<<<<<<<<<<
 *       for index, operation in enumerate(self._operations):
 *         operation.c()
 */
    __pyx_v_self->c_ops = ((grpc_op *)gpr_malloc(((sizeof(grpc_op)) * __pyx_v_self->c_nops)));

    /* "src/python/grpcio/grpc/_cython/_cygrpc/tag.pyx.pxi":64
 *     if 0 < self.c_nops:
 *       self.c_ops = <grpc_op *>gpr_malloc(sizeof(grpc_op) * self.c_nops)
 *       for index, operation in enumerate(self._operations):             # <<<<<<<<<<<<<<
 *         operation.c()
 *         self.c_ops[index] = operation.c_op
 */
    __Pyx_INCREF(__pyx_int_0);
    __pyx_t_3 = __pyx_int_0;
    if (likely(PyList_CheckExact(__pyx_v_self->_operations)) || PyTuple_CheckExact(__pyx_v_self->_operations)) {
      __pyx_t_5 = __pyx_v_self->_operations; __Pyx_INCREF(__pyx_t_5); __pyx_t_4 = 0;
      __pyx_t_6 = NULL;
    } else {
      __pyx_t_4 = -1; __pyx_t_5 = PyObject_GetIter(__pyx_v_self->_operations); if (unlikely(!__pyx_t_5)) __PYX_ERR(18, 64, __pyx_L1_error)
      __Pyx_GOTREF(__pyx_t_5);
      __pyx_t_6 = Py_TYPE(__pyx_t_5)->tp_iternext; if (unlikely(!__pyx_t_6)) __PYX_ERR(18, 64, __pyx_L1_error)
    }
    for (;;) {
      if (likely(!__pyx_t_6)) {
        if (likely(PyList_CheckExact(__pyx_t_5))) {
          if (__pyx_t_4 >= PyList_GET_SIZE(__pyx_t_5)) break;
          #if CYTHON_ASSUME_SAFE_MACROS && !CYTHON_AVOID_BORROWED_REFS
          __pyx_t_7 = PyList_GET_ITEM(__pyx_t_5, __pyx_t_4); __Pyx_INCREF(__pyx_t_7); __pyx_t_4++; if (unlikely(0 < 0)) __PYX_ERR(18, 64, __pyx_L1_error)
          #else
          __pyx_t_7 = PySequence_ITEM(__pyx_t_5, __pyx_t_4); __pyx_t_4++; if (unlikely(!__pyx_t_7)) __PYX_ERR(18, 64, __pyx_L1_error)
          __Pyx_GOTREF(__pyx_t_7);
          #endif
        } else {
          if (__pyx_t_4 >= PyTuple_GET_SIZE(__pyx_t_5)) break;
          #if CYTHON_ASSUME_SAFE_MACROS && !CYTHON_AVOID_BORROWED_REFS
          __pyx_t_7 = PyTuple_GET_ITEM(__pyx_t_5, __pyx_t_4); __Pyx_INCREF(__pyx_t_7); __pyx_t_4++; if (unlikely(0 < 0)) __PYX_ERR(18, 64, __pyx_L1_error)
          #else
          __pyx_t_7 = PySequence_ITEM(__pyx_t_5, __pyx_t_4); __pyx_t_4++; if (unlikely(!__pyx_t_7)) __PYX_ERR(18, 64, __pyx_L1_error)
          __Pyx_GOTREF(__pyx_t_7);
          #endif
        }
      } else {
        __pyx_t_7 = __pyx_t_6(__pyx_t_5);
        if (unlikely(!__pyx_t_7)) {
          PyObject* exc_type = PyErr_Occurred();
          if (exc_type) {
            if (likely(__Pyx_PyErr_GivenExceptionMatches(exc_type, PyExc_StopIteration))) PyErr_Clear();
            else __PYX_ERR(18, 64, __pyx_L1_error)
          }
          break;
        }
        __Pyx_GOTREF(__pyx_t_7);
      }
      if (!(likely(((__pyx_t_7) == Py_None) || likely(__Pyx_TypeTest(__pyx_t_7, __pyx_ptype_4grpc_7_cython_6cygrpc_Operation))))) __PYX_ERR(18, 64, __pyx_L1_error)
      __Pyx_XDECREF_SET(__pyx_v_operation, ((struct __pyx_obj_4grpc_7_cython_6cygrpc_Operation *)__pyx_t_7));
      __pyx_t_7 = 0;
      __Pyx_INCREF(__pyx_t_3);
      __Pyx_XDECREF_SET(__pyx_v_index, __pyx_t_3);
      __pyx_t_7 = __Pyx_PyInt_AddObjC(__pyx_t_3, __pyx_int_1, 1, 0); if (unlikely(!__pyx_t_7)) __PYX_ERR(18, 64, __pyx_L1_error)
      __Pyx_GOTREF(__pyx_t_7);
      __Pyx_DECREF(__pyx_t_3);
      __pyx_t_3 = __pyx_t_7;
      __pyx_t_7 = 0;

      /* "src/python/grpcio/grpc/_cython/_cygrpc/tag.pyx.pxi":65
 *       self.c_ops = <grpc_op *>gpr_malloc(sizeof(grpc_op) * self.c_nops)
 *       for index, operation in enumerate(self._operations):
 *         operation.c()             # <<<<<<<<<<<<<<
 *         self.c_ops[index] = operation.c_op
 * 
 */
      ((struct __pyx_vtabstruct_4grpc_7_cython_6cygrpc_Operation *)__pyx_v_operation->__pyx_vtab)->c(__pyx_v_operation); if (unlikely(PyErr_Occurred())) __PYX_ERR(18, 65, __pyx_L1_error)

      /* "src/python/grpcio/grpc/_cython/_cygrpc/tag.pyx.pxi":66
 *       for index, operation in enumerate(self._operations):
 *         operation.c()
 *         self.c_ops[index] = operation.c_op             # <<<<<<<<<<<<<<
 * 
 *   cdef BatchOperationEvent event(self, grpc_event c_event):
 */
      __pyx_t_8 = __pyx_v_operation->c_op;
      __pyx_t_9 = __Pyx_PyIndex_AsSsize_t(__pyx_v_index); if (unlikely((__pyx_t_9 == (Py_ssize_t)-1) && PyErr_Occurred())) __PYX_ERR(18, 66, __pyx_L1_error)
      (__pyx_v_self->c_ops[__pyx_t_9]) = __pyx_t_8;

      /* "src/python/grpcio/grpc/_cython/_cygrpc/tag.pyx.pxi":64
 *     if 0 < self.c_nops:
 *       self.c_ops = <grpc_op *>gpr_malloc(sizeof(grpc_op) * self.c_nops)
 *       for index, operation in enumerate(self._operations):             # <<<<<<<<<<<<<<
 *         operation.c()
 *         self.c_ops[index] = operation.c_op
 */
    }
    __Pyx_DECREF(__pyx_t_5); __pyx_t_5 = 0;
    __Pyx_DECREF(__pyx_t_3); __pyx_t_3 = 0;

    /* "src/python/grpcio/grpc/_cython/_cygrpc/tag.pyx.pxi":62
 *     cdef Operation operation
 *     self.c_nops = 0 if self._operations is None else len(self._operations)
 *     if 0 < self.c_nops:             # <<<<<<<<<<<<<<
 *       self.c_ops = <grpc_op *>gpr_malloc(sizeof(grpc_op) * self.c_nops)
 *       for index, operation in enumerate(self._operations):
 */
  }

  /* "src/python/grpcio/grpc/_cython/_cygrpc/tag.pyx.pxi":59
 *     self._retained_call = call
 * 
 *   cdef void prepare(self) except *:             # <<<<<<<<<<<<<<
 *     cdef Operation operation
 *     self.c_nops = 0 if self._operations is None else len(self._operations)
 */

  /* function exit code */
  goto __pyx_L0;
  __pyx_L1_error:;
  __Pyx_XDECREF(__pyx_t_3);
  __Pyx_XDECREF(__pyx_t_5);
  __Pyx_XDECREF(__pyx_t_7);
  __Pyx_AddTraceback("grpc._cython.cygrpc._BatchOperationTag.prepare", __pyx_clineno, __pyx_lineno, __pyx_filename);
  __pyx_L0:;
  __Pyx_XDECREF((PyObject *)__pyx_v_operation);
  __Pyx_XDECREF(__pyx_v_index);
  __Pyx_RefNannyFinishContext();
}

/* "src/python/grpcio/grpc/_cython/_cygrpc/tag.pyx.pxi":68
 *         self.c_ops[index] = operation.c_op
 * 
 *   cdef BatchOperationEvent event(self, grpc_event c_event):             # <<<<<<<<<<<<<<
 *     cdef Operation operation
 *     if 0 < self.c_nops:
 */

struct __pyx_obj_4grpc_7_cython_6cygrpc_BatchOperationEvent *__pyx_f_4grpc_7_cython_6cygrpc_18_BatchOperationTag_event(struct __pyx_obj_4grpc_7_cython_6cygrpc__BatchOperationTag *__pyx_v_self, grpc_event __pyx_v_c_event) {
  struct __pyx_obj_4grpc_7_cython_6cygrpc_Operation *__pyx_v_operation = 0;
  struct __pyx_obj_4grpc_7_cython_6cygrpc_BatchOperationEvent *__pyx_r = NULL;
  __Pyx_RefNannyDeclarations
  int __pyx_t_1;
  PyObject *__pyx_t_2 = NULL;
  Py_ssize_t __pyx_t_3;
  PyObject *(*__pyx_t_4)(PyObject *);
  PyObject *__pyx_t_5 = NULL;
  PyObject *__pyx_t_6 = NULL;
  __Pyx_RefNannySetupContext("event", 0);

  /* "src/python/grpcio/grpc/_cython/_cygrpc/tag.pyx.pxi":70
 *   cdef BatchOperationEvent event(self, grpc_event c_event):
 *     cdef Operation operation
 *     if 0 < self.c_nops:             # <<<<<<<<<<<<<<
 *       for operation in self._operations:
 *         operation.un_c()
 */
  __pyx_t_1 = ((0 < __pyx_v_self->c_nops) != 0);
  if (__pyx_t_1) {

    /* "src/python/grpcio/grpc/_cython/_cygrpc/tag.pyx.pxi":71
 *     cdef Operation operation
 *     if 0 < self.c_nops:
 *       for operation in self._operations:             # <<<<<<<<<<<<<<
 *         operation.un_c()
 *       gpr_free(self.c_ops)
 */
    if (likely(PyList_CheckExact(__pyx_v_self->_operations)) || PyTuple_CheckExact(__pyx_v_self->_operations)) {
      __pyx_t_2 = __pyx_v_self->_operations; __Pyx_INCREF(__pyx_t_2); __pyx_t_3 = 0;
      __pyx_t_4 = NULL;
    } else {
      __pyx_t_3 = -1; __pyx_t_2 = PyObject_GetIter(__pyx_v_self->_operations); if (unlikely(!__pyx_t_2)) __PYX_ERR(18, 71, __pyx_L1_error)
      __Pyx_GOTREF(__pyx_t_2);
      __pyx_t_4 = Py_TYPE(__pyx_t_2)->tp_iternext; if (unlikely(!__pyx_t_4)) __PYX_ERR(18, 71, __pyx_L1_error)
    }
    for (;;) {
      if (likely(!__pyx_t_4)) {
        if (likely(PyList_CheckExact(__pyx_t_2))) {
          if (__pyx_t_3 >= PyList_GET_SIZE(__pyx_t_2)) break;
          #if CYTHON_ASSUME_SAFE_MACROS && !CYTHON_AVOID_BORROWED_REFS
          __pyx_t_5 = PyList_GET_ITEM(__pyx_t_2, __pyx_t_3); __Pyx_INCREF(__pyx_t_5); __pyx_t_3++; if (unlikely(0 < 0)) __PYX_ERR(18, 71, __pyx_L1_error)
          #else
          __pyx_t_5 = PySequence_ITEM(__pyx_t_2, __pyx_t_3); __pyx_t_3++; if (unlikely(!__pyx_t_5)) __PYX_ERR(18, 71, __pyx_L1_error)
          __Pyx_GOTREF(__pyx_t_5);
          #endif
        } else {
          if (__pyx_t_3 >= PyTuple_GET_SIZE(__pyx_t_2)) break;
          #if CYTHON_ASSUME_SAFE_MACROS && !CYTHON_AVOID_BORROWED_REFS
          __pyx_t_5 = PyTuple_GET_ITEM(__pyx_t_2, __pyx_t_3); __Pyx_INCREF(__pyx_t_5); __pyx_t_3++; if (unlikely(0 < 0)) __PYX_ERR(18, 71, __pyx_L1_error)
          #else
          __pyx_t_5 = PySequence_ITEM(__pyx_t_2, __pyx_t_3); __pyx_t_3++; if (unlikely(!__pyx_t_5)) __PYX_ERR(18, 71, __pyx_L1_error)
          __Pyx_GOTREF(__pyx_t_5);
          #endif
        }
      } else {
        __pyx_t_5 = __pyx_t_4(__pyx_t_2);
        if (unlikely(!__pyx_t_5)) {
          PyObject* exc_type = PyErr_Occurred();
          if (exc_type) {
            if (likely(__Pyx_PyErr_GivenExceptionMatches(exc_type, PyExc_StopIteration))) PyErr_Clear();
            else __PYX_ERR(18, 71, __pyx_L1_error)
          }
          break;
        }
        __Pyx_GOTREF(__pyx_t_5);
      }
      if (!(likely(((__pyx_t_5) == Py_None) || likely(__Pyx_TypeTest(__pyx_t_5, __pyx_ptype_4grpc_7_cython_6cygrpc_Operation))))) __PYX_ERR(18, 71, __pyx_L1_error)
      __Pyx_XDECREF_SET(__pyx_v_operation, ((struct __pyx_obj_4grpc_7_cython_6cygrpc_Operation *)__pyx_t_5));
      __pyx_t_5 = 0;

      /* "src/python/grpcio/grpc/_cython/_cygrpc/tag.pyx.pxi":72
 *     if 0 < self.c_nops:
 *       for operation in self._operations:
 *         operation.un_c()             # <<<<<<<<<<<<<<
 *       gpr_free(self.c_ops)
 *       return BatchOperationEvent(
 */
      ((struct __pyx_vtabstruct_4grpc_7_cython_6cygrpc_Operation *)__pyx_v_operation->__pyx_vtab)->un_c(__pyx_v_operation); if (unlikely(PyErr_Occurred())) __PYX_ERR(18, 72, __pyx_L1_error)

      /* "src/python/grpcio/grpc/_cython/_cygrpc/tag.pyx.pxi":71
 *     cdef Operation operation
 *     if 0 < self.c_nops:
 *       for operation in self._operations:             # <<<<<<<<<<<<<<
 *         operation.un_c()
 *       gpr_free(self.c_ops)
 */
    }
    __Pyx_DECREF(__pyx_t_2); __pyx_t_2 = 0;

    /* "src/python/grpcio/grpc/_cython/_cygrpc/tag.pyx.pxi":73
 *       for operation in self._operations:
 *         operation.un_c()
 *       gpr_free(self.c_ops)             # <<<<<<<<<<<<<<
 *       return BatchOperationEvent(
 *           c_event.type, c_event.success, self._user_tag, self._operations)
 */
    gpr_free(__pyx_v_self->c_ops);

    /* "src/python/grpcio/grpc/_cython/_cygrpc/tag.pyx.pxi":74
 *         operation.un_c()
 *       gpr_free(self.c_ops)
 *       return BatchOperationEvent(             # <<<<<<<<<<<<<<
 *           c_event.type, c_event.success, self._user_tag, self._operations)
 *     else:
 */
    __Pyx_XDECREF(((PyObject *)__pyx_r));

    /* "src/python/grpcio/grpc/_cython/_cygrpc/tag.pyx.pxi":75
 *       gpr_free(self.c_ops)
 *       return BatchOperationEvent(
 *           c_event.type, c_event.success, self._user_tag, self._operations)             # <<<<<<<<<<<<<<
 *     else:
 *       return BatchOperationEvent(
 */
    __pyx_t_2 = __Pyx_PyInt_From_grpc_completion_type(__pyx_v_c_event.type); if (unlikely(!__pyx_t_2)) __PYX_ERR(18, 75, __pyx_L1_error)
    __Pyx_GOTREF(__pyx_t_2);
    __pyx_t_5 = __Pyx_PyInt_From_int(__pyx_v_c_event.success); if (unlikely(!__pyx_t_5)) __PYX_ERR(18, 75, __pyx_L1_error)
    __Pyx_GOTREF(__pyx_t_5);

    /* "src/python/grpcio/grpc/_cython/_cygrpc/tag.pyx.pxi":74
 *         operation.un_c()
 *       gpr_free(self.c_ops)
 *       return BatchOperationEvent(             # <<<<<<<<<<<<<<
 *           c_event.type, c_event.success, self._user_tag, self._operations)
 *     else:
 */
    __pyx_t_6 = PyTuple_New(4); if (unlikely(!__pyx_t_6)) __PYX_ERR(18, 74, __pyx_L1_error)
    __Pyx_GOTREF(__pyx_t_6);
    __Pyx_GIVEREF(__pyx_t_2);
    PyTuple_SET_ITEM(__pyx_t_6, 0, __pyx_t_2);
    __Pyx_GIVEREF(__pyx_t_5);
    PyTuple_SET_ITEM(__pyx_t_6, 1, __pyx_t_5);
    __Pyx_INCREF(__pyx_v_self->_user_tag);
    __Pyx_GIVEREF(__pyx_v_self->_user_tag);
    PyTuple_SET_ITEM(__pyx_t_6, 2, __pyx_v_self->_user_tag);
    __Pyx_INCREF(__pyx_v_self->_operations);
    __Pyx_GIVEREF(__pyx_v_self->_operations);
    PyTuple_SET_ITEM(__pyx_t_6, 3, __pyx_v_self->_operations);
    __pyx_t_2 = 0;
    __pyx_t_5 = 0;
    __pyx_t_5 = __Pyx_PyObject_Call(((PyObject *)__pyx_ptype_4grpc_7_cython_6cygrpc_BatchOperationEvent), __pyx_t_6, NULL); if (unlikely(!__pyx_t_5)) __PYX_ERR(18, 74, __pyx_L1_error)
    __Pyx_GOTREF(__pyx_t_5);
    __Pyx_DECREF(__pyx_t_6); __pyx_t_6 = 0;
    __pyx_r = ((struct __pyx_obj_4grpc_7_cython_6cygrpc_BatchOperationEvent *)__pyx_t_5);
    __pyx_t_5 = 0;
    goto __pyx_L0;

    /* "src/python/grpcio/grpc/_cython/_cygrpc/tag.pyx.pxi":70
 *   cdef BatchOperationEvent event(self, grpc_event c_event):
 *     cdef Operation operation
 *     if 0 < self.c_nops:             # <<<<<<<<<<<<<<
 *       for operation in self._operations:
 *         operation.un_c()
 */
  }

  /* "src/python/grpcio/grpc/_cython/_cygrpc/tag.pyx.pxi":77
 *           c_event.type, c_event.success, self._user_tag, self._operations)
 *     else:
 *       return BatchOperationEvent(             # <<<<<<<<<<<<<<
 *           c_event.type, c_event.success, self._user_tag, ())
 * 
 */
  /*else*/ {
    __Pyx_XDECREF(((PyObject *)__pyx_r));

    /* "src/python/grpcio/grpc/_cython/_cygrpc/tag.pyx.pxi":78
 *     else:
 *       return BatchOperationEvent(
 *           c_event.type, c_event.success, self._user_tag, ())             # <<<<<<<<<<<<<<
 * 
 * 
 */
    __pyx_t_5 = __Pyx_PyInt_From_grpc_completion_type(__pyx_v_c_event.type); if (unlikely(!__pyx_t_5)) __PYX_ERR(18, 78, __pyx_L1_error)
    __Pyx_GOTREF(__pyx_t_5);
    __pyx_t_6 = __Pyx_PyInt_From_int(__pyx_v_c_event.success); if (unlikely(!__pyx_t_6)) __PYX_ERR(18, 78, __pyx_L1_error)
    __Pyx_GOTREF(__pyx_t_6);

    /* "src/python/grpcio/grpc/_cython/_cygrpc/tag.pyx.pxi":77
 *           c_event.type, c_event.success, self._user_tag, self._operations)
 *     else:
 *       return BatchOperationEvent(             # <<<<<<<<<<<<<<
 *           c_event.type, c_event.success, self._user_tag, ())
 * 
 */
    __pyx_t_2 = PyTuple_New(4); if (unlikely(!__pyx_t_2)) __PYX_ERR(18, 77, __pyx_L1_error)
    __Pyx_GOTREF(__pyx_t_2);
    __Pyx_GIVEREF(__pyx_t_5);
    PyTuple_SET_ITEM(__pyx_t_2, 0, __pyx_t_5);
    __Pyx_GIVEREF(__pyx_t_6);
    PyTuple_SET_ITEM(__pyx_t_2, 1, __pyx_t_6);
    __Pyx_INCREF(__pyx_v_self->_user_tag);
    __Pyx_GIVEREF(__pyx_v_self->_user_tag);
    PyTuple_SET_ITEM(__pyx_t_2, 2, __pyx_v_self->_user_tag);
    __Pyx_INCREF(__pyx_empty_tuple);
    __Pyx_GIVEREF(__pyx_empty_tuple);
    PyTuple_SET_ITEM(__pyx_t_2, 3, __pyx_empty_tuple);
    __pyx_t_5 = 0;
    __pyx_t_6 = 0;
    __pyx_t_6 = __Pyx_PyObject_Call(((PyObject *)__pyx_ptype_4grpc_7_cython_6cygrpc_BatchOperationEvent), __pyx_t_2, NULL); if (unlikely(!__pyx_t_6)) __PYX_ERR(18, 77, __pyx_L1_error)
    __Pyx_GOTREF(__pyx_t_6);
    __Pyx_DECREF(__pyx_t_2); __pyx_t_2 = 0;
    __pyx_r = ((struct __pyx_obj_4grpc_7_cython_6cygrpc_BatchOperationEvent *)__pyx_t_6);
    __pyx_t_6 = 0;
    goto __pyx_L0;
  }

  /* "src/python/grpcio/grpc/_cython/_cygrpc/tag.pyx.pxi":68
 *         self.c_ops[index] = operation.c_op
 * 
 *   cdef BatchOperationEvent event(self, grpc_event c_event):             # <<<<<<<<<<<<<<
 *     cdef Operation operation
 *     if 0 < self.c_nops:
 */

  /* function exit code */
  __pyx_L1_error:;
  __Pyx_XDECREF(__pyx_t_2);
  __Pyx_XDECREF(__pyx_t_5);
  __Pyx_XDECREF(__pyx_t_6);
  __Pyx_AddTraceback("grpc._cython.cygrpc._BatchOperationTag.event", __pyx_clineno, __pyx_lineno, __pyx_filename);
  __pyx_r = 0;
  __pyx_L0:;
  __Pyx_XDECREF((PyObject *)__pyx_v_operation);
  __Pyx_XGIVEREF((PyObject *)__pyx_r);
  __Pyx_RefNannyFinishContext();
  return __pyx_r;
}

struct __pyx_obj_4grpc_7_cython_6cygrpc_BatchOperationEvent *__pyx_f_4grpc_7_cython_6cygrpc_18_BatchOperationTag_event__pyx_wrap_1(struct __pyx_obj_4grpc_7_cython_6cygrpc__BatchOperationTag *__pyx_v_self, grpc_event __pyx_v_c_event) {
  return __pyx_f_4grpc_7_cython_6cygrpc_18_BatchOperationTag_event(__pyx_v_self, __pyx_v_c_event);
}

/* "src/python/grpcio/grpc/_cython/_cygrpc/tag.pxd.pxi":43
 * 
 *   cdef object _user_tag
 *   cdef readonly object _operations             # <<<<<<<<<<<<<<
 *   cdef readonly object _retained_call
 *   cdef grpc_op *c_ops
 */

/* Python wrapper */
static PyObject *__pyx_pw_4grpc_7_cython_6cygrpc_18_BatchOperationTag_11_operations_1__get__(PyObject *__pyx_v_self); /*proto*/
static PyObject *__pyx_pw_4grpc_7_cython_6cygrpc_18_BatchOperationTag_11_operations_1__get__(PyObject *__pyx_v_self) {
  PyObject *__pyx_r = 0;
  __Pyx_RefNannyDeclarations
  __Pyx_RefNannySetupContext("__get__ (wrapper)", 0);
  __pyx_r = __pyx_pf_4grpc_7_cython_6cygrpc_18_BatchOperationTag_11_operations___get__(((struct __pyx_obj_4grpc_7_cython_6cygrpc__BatchOperationTag *)__pyx_v_self));

  /* function exit code */
  __Pyx_RefNannyFinishContext();
  return __pyx_r;
}

static PyObject *__pyx_pf_4grpc_7_cython_6cygrpc_18_BatchOperationTag_11_operations___get__(struct __pyx_obj_4grpc_7_cython_6cygrpc__BatchOperationTag *__pyx_v_self) {
  PyObject *__pyx_r = NULL;
  __Pyx_RefNannyDeclarations
  __Pyx_RefNannySetupContext("__get__", 0);
  __Pyx_XDECREF(__pyx_r);
  __Pyx_INCREF(__pyx_v_self->_operations);
  __pyx_r = __pyx_v_self->_operations;
  goto __pyx_L0;

  /* function exit code */
  __pyx_L0:;
  __Pyx_XGIVEREF(__pyx_r);
  __Pyx_RefNannyFinishContext();
  return __pyx_r;
}

/* "src/python/grpcio/grpc/_cython/_cygrpc/tag.pxd.pxi":44
 *   cdef object _user_tag
 *   cdef readonly object _operations
 *   cdef readonly object _retained_call             # <<<<<<<<<<<<<<
 *   cdef grpc_op *c_ops
 *   cdef size_t c_nops
 */

/* Python wrapper */
static PyObject *__pyx_pw_4grpc_7_cython_6cygrpc_18_BatchOperationTag_14_retained_call_1__get__(PyObject *__pyx_v_self); /*proto*/
static PyObject *__pyx_pw_4grpc_7_cython_6cygrpc_18_BatchOperationTag_14_retained_call_1__get__(PyObject *__pyx_v_self) {
  PyObject *__pyx_r = 0;
  __Pyx_RefNannyDeclarations
  __Pyx_RefNannySetupContext("__get__ (wrapper)", 0);
  __pyx_r = __pyx_pf_4grpc_7_cython_6cygrpc_18_BatchOperationTag_14_retained_call___get__(((struct __pyx_obj_4grpc_7_cython_6cygrpc__BatchOperationTag *)__pyx_v_self));

  /* function exit code */
  __Pyx_RefNannyFinishContext();
  return __pyx_r;
}

static PyObject *__pyx_pf_4grpc_7_cython_6cygrpc_18_BatchOperationTag_14_retained_call___get__(struct __pyx_obj_4grpc_7_cython_6cygrpc__BatchOperationTag *__pyx_v_self) {
  PyObject *__pyx_r = NULL;
  __Pyx_RefNannyDeclarations
  __Pyx_RefNannySetupContext("__get__", 0);
  __Pyx_XDECREF(__pyx_r);
  __Pyx_INCREF(__pyx_v_self->_retained_call);
  __pyx_r = __pyx_v_self->_retained_call;
  goto __pyx_L0;

  /* function exit code */
  __pyx_L0:;
  __Pyx_XGIVEREF(__pyx_r);
  __Pyx_RefNannyFinishContext();
  return __pyx_r;
}

/* "(tree fragment)":1
 * def __reduce_cython__(self):             # <<<<<<<<<<<<<<
 *     raise TypeError("no default __reduce__ due to non-trivial __cinit__")
 * def __setstate_cython__(self, __pyx_state):
 */

/* Python wrapper */
static PyObject *__pyx_pw_4grpc_7_cython_6cygrpc_18_BatchOperationTag_3__reduce_cython__(PyObject *__pyx_v_self, CYTHON_UNUSED PyObject *unused); /*proto*/
static PyObject *__pyx_pw_4grpc_7_cython_6cygrpc_18_BatchOperationTag_3__reduce_cython__(PyObject *__pyx_v_self, CYTHON_UNUSED PyObject *unused) {
  PyObject *__pyx_r = 0;
  __Pyx_RefNannyDeclarations
  __Pyx_RefNannySetupContext("__reduce_cython__ (wrapper)", 0);
  __pyx_r = __pyx_pf_4grpc_7_cython_6cygrpc_18_BatchOperationTag_2__reduce_cython__(((struct __pyx_obj_4grpc_7_cython_6cygrpc__BatchOperationTag *)__pyx_v_self));

  /* function exit code */
  __Pyx_RefNannyFinishContext();
  return __pyx_r;
}

static PyObject *__pyx_pf_4grpc_7_cython_6cygrpc_18_BatchOperationTag_2__reduce_cython__(CYTHON_UNUSED struct __pyx_obj_4grpc_7_cython_6cygrpc__BatchOperationTag *__pyx_v_self) {
  PyObject *__pyx_r = NULL;
  __Pyx_RefNannyDeclarations
  PyObject *__pyx_t_1 = NULL;
  __Pyx_RefNannySetupContext("__reduce_cython__", 0);

  /* "(tree fragment)":2
 * def __reduce_cython__(self):
 *     raise TypeError("no default __reduce__ due to non-trivial __cinit__")             # <<<<<<<<<<<<<<
 * def __setstate_cython__(self, __pyx_state):
 *     raise TypeError("no default __reduce__ due to non-trivial __cinit__")
 */
  __pyx_t_1 = __Pyx_PyObject_Call(__pyx_builtin_TypeError, __pyx_tuple__124, NULL); if (unlikely(!__pyx_t_1)) __PYX_ERR(7, 2, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_1);
  __Pyx_Raise(__pyx_t_1, 0, 0, 0);
  __Pyx_DECREF(__pyx_t_1); __pyx_t_1 = 0;
  __PYX_ERR(7, 2, __pyx_L1_error)

  /* "(tree fragment)":1
 * def __reduce_cython__(self):             # <<<<<<<<<<<<<<
 *     raise TypeError("no default __reduce__ due to non-trivial __cinit__")
 * def __setstate_cython__(self, __pyx_state):
 */

  /* function exit code */
  __pyx_L1_error:;
  __Pyx_XDECREF(__pyx_t_1);
  __Pyx_AddTraceback("grpc._cython.cygrpc._BatchOperationTag.__reduce_cython__", __pyx_clineno, __pyx_lineno, __pyx_filename);
  __pyx_r = NULL;
  __Pyx_XGIVEREF(__pyx_r);
  __Pyx_RefNannyFinishContext();
  return __pyx_r;
}

/* "(tree fragment)":3
 * def __reduce_cython__(self):
 *     raise TypeError("no default __reduce__ due to non-trivial __cinit__")
 * def __setstate_cython__(self, __pyx_state):             # <<<<<<<<<<<<<<
 *     raise TypeError("no default __reduce__ due to non-trivial __cinit__")
 */

/* Python wrapper */
static PyObject *__pyx_pw_4grpc_7_cython_6cygrpc_18_BatchOperationTag_5__setstate_cython__(PyObject *__pyx_v_self, PyObject *__pyx_v___pyx_state); /*proto*/
static PyObject *__pyx_pw_4grpc_7_cython_6cygrpc_18_BatchOperationTag_5__setstate_cython__(PyObject *__pyx_v_self, PyObject *__pyx_v___pyx_state) {
  PyObject *__pyx_r = 0;
  __Pyx_RefNannyDeclarations
  __Pyx_RefNannySetupContext("__setstate_cython__ (wrapper)", 0);
  __pyx_r = __pyx_pf_4grpc_7_cython_6cygrpc_18_BatchOperationTag_4__setstate_cython__(((struct __pyx_obj_4grpc_7_cython_6cygrpc__BatchOperationTag *)__pyx_v_self), ((PyObject *)__pyx_v___pyx_state));

  /* function exit code */
  __Pyx_RefNannyFinishContext();
  return __pyx_r;
}

static PyObject *__pyx_pf_4grpc_7_cython_6cygrpc_18_BatchOperationTag_4__setstate_cython__(CYTHON_UNUSED struct __pyx_obj_4grpc_7_cython_6cygrpc__BatchOperationTag *__pyx_v_self, CYTHON_UNUSED PyObject *__pyx_v___pyx_state) {
  PyObject *__pyx_r = NULL;
  __Pyx_RefNannyDeclarations
  PyObject *__pyx_t_1 = NULL;
  __Pyx_RefNannySetupContext("__setstate_cython__", 0);

  /* "(tree fragment)":4
 *     raise TypeError("no default __reduce__ due to non-trivial __cinit__")
 * def __setstate_cython__(self, __pyx_state):
 *     raise TypeError("no default __reduce__ due to non-trivial __cinit__")             # <<<<<<<<<<<<<<
 */
  __pyx_t_1 = __Pyx_PyObject_Call(__pyx_builtin_TypeError, __pyx_tuple__125, NULL); if (unlikely(!__pyx_t_1)) __PYX_ERR(7, 4, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_1);
  __Pyx_Raise(__pyx_t_1, 0, 0, 0);
  __Pyx_DECREF(__pyx_t_1); __pyx_t_1 = 0;
  __PYX_ERR(7, 4, __pyx_L1_error)

  /* "(tree fragment)":3
 * def __reduce_cython__(self):
 *     raise TypeError("no default __reduce__ due to non-trivial __cinit__")
 * def __setstate_cython__(self, __pyx_state):             # <<<<<<<<<<<<<<
 *     raise TypeError("no default __reduce__ due to non-trivial __cinit__")
 */

  /* function exit code */
  __pyx_L1_error:;
  __Pyx_XDECREF(__pyx_t_1);
  __Pyx_AddTraceback("grpc._cython.cygrpc._BatchOperationTag.__setstate_cython__", __pyx_clineno, __pyx_lineno, __pyx_filename);
  __pyx_r = NULL;
  __Pyx_XGIVEREF(__pyx_r);
  __Pyx_RefNannyFinishContext();
  return __pyx_r;
}

/* "src/python/grpcio/grpc/_cython/_cygrpc/tag.pyx.pxi":83
 * cdef class _ServerShutdownTag(_Tag):
 * 
 *   def __cinit__(self, user_tag, shutting_down_server):             # <<<<<<<<<<<<<<
 *     self._user_tag = user_tag
 *     self._shutting_down_server = shutting_down_server
 */

/* Python wrapper */
static int __pyx_pw_4grpc_7_cython_6cygrpc_18_ServerShutdownTag_1__cinit__(PyObject *__pyx_v_self, PyObject *__pyx_args, PyObject *__pyx_kwds); /*proto*/
static int __pyx_pw_4grpc_7_cython_6cygrpc_18_ServerShutdownTag_1__cinit__(PyObject *__pyx_v_self, PyObject *__pyx_args, PyObject *__pyx_kwds) {
  PyObject *__pyx_v_user_tag = 0;
  PyObject *__pyx_v_shutting_down_server = 0;
  int __pyx_r;
  __Pyx_RefNannyDeclarations
  __Pyx_RefNannySetupContext("__cinit__ (wrapper)", 0);
  {
    static PyObject **__pyx_pyargnames[] = {&__pyx_n_s_user_tag,&__pyx_n_s_shutting_down_server,0};
    PyObject* values[2] = {0,0};
    if (unlikely(__pyx_kwds)) {
      Py_ssize_t kw_args;
      const Py_ssize_t pos_args = PyTuple_GET_SIZE(__pyx_args);
      switch (pos_args) {
        case  2: values[1] = PyTuple_GET_ITEM(__pyx_args, 1);
        CYTHON_FALLTHROUGH;
        case  1: values[0] = PyTuple_GET_ITEM(__pyx_args, 0);
        CYTHON_FALLTHROUGH;
        case  0: break;
        default: goto __pyx_L5_argtuple_error;
      }
      kw_args = PyDict_Size(__pyx_kwds);
      switch (pos_args) {
        case  0:
        if (likely((values[0] = __Pyx_PyDict_GetItemStr(__pyx_kwds, __pyx_n_s_user_tag)) != 0)) kw_args--;
        else goto __pyx_L5_argtuple_error;
        CYTHON_FALLTHROUGH;
        case  1:
        if (likely((values[1] = __Pyx_PyDict_GetItemStr(__pyx_kwds, __pyx_n_s_shutting_down_server)) != 0)) kw_args--;
        else {
          __Pyx_RaiseArgtupleInvalid("__cinit__", 1, 2, 2, 1); __PYX_ERR(18, 83, __pyx_L3_error)
        }
      }
      if (unlikely(kw_args > 0)) {
        if (unlikely(__Pyx_ParseOptionalKeywords(__pyx_kwds, __pyx_pyargnames, 0, values, pos_args, "__cinit__") < 0)) __PYX_ERR(18, 83, __pyx_L3_error)
      }
    } else if (PyTuple_GET_SIZE(__pyx_args) != 2) {
      goto __pyx_L5_argtuple_error;
    } else {
      values[0] = PyTuple_GET_ITEM(__pyx_args, 0);
      values[1] = PyTuple_GET_ITEM(__pyx_args, 1);
    }
    __pyx_v_user_tag = values[0];
    __pyx_v_shutting_down_server = values[1];
  }
  goto __pyx_L4_argument_unpacking_done;
  __pyx_L5_argtuple_error:;
  __Pyx_RaiseArgtupleInvalid("__cinit__", 1, 2, 2, PyTuple_GET_SIZE(__pyx_args)); __PYX_ERR(18, 83, __pyx_L3_error)
  __pyx_L3_error:;
  __Pyx_AddTraceback("grpc._cython.cygrpc._ServerShutdownTag.__cinit__", __pyx_clineno, __pyx_lineno, __pyx_filename);
  __Pyx_RefNannyFinishContext();
  return -1;
  __pyx_L4_argument_unpacking_done:;
  __pyx_r = __pyx_pf_4grpc_7_cython_6cygrpc_18_ServerShutdownTag___cinit__(((struct __pyx_obj_4grpc_7_cython_6cygrpc__ServerShutdownTag *)__pyx_v_self), __pyx_v_user_tag, __pyx_v_shutting_down_server);

  /* function exit code */
  __Pyx_RefNannyFinishContext();
  return __pyx_r;
}

static int __pyx_pf_4grpc_7_cython_6cygrpc_18_ServerShutdownTag___cinit__(struct __pyx_obj_4grpc_7_cython_6cygrpc__ServerShutdownTag *__pyx_v_self, PyObject *__pyx_v_user_tag, PyObject *__pyx_v_shutting_down_server) {
  int __pyx_r;
  __Pyx_RefNannyDeclarations
  PyObject *__pyx_t_1 = NULL;
  __Pyx_RefNannySetupContext("__cinit__", 0);

  /* "src/python/grpcio/grpc/_cython/_cygrpc/tag.pyx.pxi":84
 * 
 *   def __cinit__(self, user_tag, shutting_down_server):
 *     self._user_tag = user_tag             # <<<<<<<<<<<<<<
 *     self._shutting_down_server = shutting_down_server
 * 
 */
  __Pyx_INCREF(__pyx_v_user_tag);
  __Pyx_GIVEREF(__pyx_v_user_tag);
  __Pyx_GOTREF(__pyx_v_self->_user_tag);
  __Pyx_DECREF(__pyx_v_self->_user_tag);
  __pyx_v_self->_user_tag = __pyx_v_user_tag;

  /* "src/python/grpcio/grpc/_cython/_cygrpc/tag.pyx.pxi":85
 *   def __cinit__(self, user_tag, shutting_down_server):
 *     self._user_tag = user_tag
 *     self._shutting_down_server = shutting_down_server             # <<<<<<<<<<<<<<
 * 
 *   cdef ServerShutdownEvent event(self, grpc_event c_event):
 */
  if (!(likely(((__pyx_v_shutting_down_server) == Py_None) || likely(__Pyx_TypeTest(__pyx_v_shutting_down_server, __pyx_ptype_4grpc_7_cython_6cygrpc_Server))))) __PYX_ERR(18, 85, __pyx_L1_error)
  __pyx_t_1 = __pyx_v_shutting_down_server;
  __Pyx_INCREF(__pyx_t_1);
  __Pyx_GIVEREF(__pyx_t_1);
  __Pyx_GOTREF(__pyx_v_self->_shutting_down_server);
  __Pyx_DECREF(((PyObject *)__pyx_v_self->_shutting_down_server));
  __pyx_v_self->_shutting_down_server = ((struct __pyx_obj_4grpc_7_cython_6cygrpc_Server *)__pyx_t_1);
  __pyx_t_1 = 0;

  /* "src/python/grpcio/grpc/_cython/_cygrpc/tag.pyx.pxi":83
 * cdef class _ServerShutdownTag(_Tag):
 * 
 *   def __cinit__(self, user_tag, shutting_down_server):             # <<<<<<<<<<<<<<
 *     self._user_tag = user_tag
 *     self._shutting_down_server = shutting_down_server
 */

  /* function exit code */
  __pyx_r = 0;
  goto __pyx_L0;
  __pyx_L1_error:;
  __Pyx_XDECREF(__pyx_t_1);
  __Pyx_AddTraceback("grpc._cython.cygrpc._ServerShutdownTag.__cinit__", __pyx_clineno, __pyx_lineno, __pyx_filename);
  __pyx_r = -1;
  __pyx_L0:;
  __Pyx_RefNannyFinishContext();
  return __pyx_r;
}

/* "src/python/grpcio/grpc/_cython/_cygrpc/tag.pyx.pxi":87
 *     self._shutting_down_server = shutting_down_server
 * 
 *   cdef ServerShutdownEvent event(self, grpc_event c_event):             # <<<<<<<<<<<<<<
 *     self._shutting_down_server.notify_shutdown_complete()
 *     return ServerShutdownEvent(c_event.type, c_event.success, self._user_tag)
 */

struct __pyx_obj_4grpc_7_cython_6cygrpc_ServerShutdownEvent *__pyx_f_4grpc_7_cython_6cygrpc_18_ServerShutdownTag_event(struct __pyx_obj_4grpc_7_cython_6cygrpc__ServerShutdownTag *__pyx_v_self, grpc_event __pyx_v_c_event) {
  struct __pyx_obj_4grpc_7_cython_6cygrpc_ServerShutdownEvent *__pyx_r = NULL;
  __Pyx_RefNannyDeclarations
  PyObject *__pyx_t_1 = NULL;
  PyObject *__pyx_t_2 = NULL;
  PyObject *__pyx_t_3 = NULL;
  __Pyx_RefNannySetupContext("event", 0);

  /* "src/python/grpcio/grpc/_cython/_cygrpc/tag.pyx.pxi":88
 * 
 *   cdef ServerShutdownEvent event(self, grpc_event c_event):
 *     self._shutting_down_server.notify_shutdown_complete()             # <<<<<<<<<<<<<<
 *     return ServerShutdownEvent(c_event.type, c_event.success, self._user_tag)
 */
  __pyx_t_1 = ((struct __pyx_vtabstruct_4grpc_7_cython_6cygrpc_Server *)__pyx_v_self->_shutting_down_server->__pyx_vtab)->notify_shutdown_complete(__pyx_v_self->_shutting_down_server); if (unlikely(!__pyx_t_1)) __PYX_ERR(18, 88, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_1);
  __Pyx_DECREF(__pyx_t_1); __pyx_t_1 = 0;

  /* "src/python/grpcio/grpc/_cython/_cygrpc/tag.pyx.pxi":89
 *   cdef ServerShutdownEvent event(self, grpc_event c_event):
 *     self._shutting_down_server.notify_shutdown_complete()
 *     return ServerShutdownEvent(c_event.type, c_event.success, self._user_tag)             # <<<<<<<<<<<<<<
 */
  __Pyx_XDECREF(((PyObject *)__pyx_r));
  __pyx_t_1 = __Pyx_PyInt_From_grpc_completion_type(__pyx_v_c_event.type); if (unlikely(!__pyx_t_1)) __PYX_ERR(18, 89, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_1);
  __pyx_t_2 = __Pyx_PyInt_From_int(__pyx_v_c_event.success); if (unlikely(!__pyx_t_2)) __PYX_ERR(18, 89, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_2);
  __pyx_t_3 = PyTuple_New(3); if (unlikely(!__pyx_t_3)) __PYX_ERR(18, 89, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_3);
  __Pyx_GIVEREF(__pyx_t_1);
  PyTuple_SET_ITEM(__pyx_t_3, 0, __pyx_t_1);
  __Pyx_GIVEREF(__pyx_t_2);
  PyTuple_SET_ITEM(__pyx_t_3, 1, __pyx_t_2);
  __Pyx_INCREF(__pyx_v_self->_user_tag);
  __Pyx_GIVEREF(__pyx_v_self->_user_tag);
  PyTuple_SET_ITEM(__pyx_t_3, 2, __pyx_v_self->_user_tag);
  __pyx_t_1 = 0;
  __pyx_t_2 = 0;
  __pyx_t_2 = __Pyx_PyObject_Call(((PyObject *)__pyx_ptype_4grpc_7_cython_6cygrpc_ServerShutdownEvent), __pyx_t_3, NULL); if (unlikely(!__pyx_t_2)) __PYX_ERR(18, 89, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_2);
  __Pyx_DECREF(__pyx_t_3); __pyx_t_3 = 0;
  __pyx_r = ((struct __pyx_obj_4grpc_7_cython_6cygrpc_ServerShutdownEvent *)__pyx_t_2);
  __pyx_t_2 = 0;
  goto __pyx_L0;

  /* "src/python/grpcio/grpc/_cython/_cygrpc/tag.pyx.pxi":87
 *     self._shutting_down_server = shutting_down_server
 * 
 *   cdef ServerShutdownEvent event(self, grpc_event c_event):             # <<<<<<<<<<<<<<
 *     self._shutting_down_server.notify_shutdown_complete()
 *     return ServerShutdownEvent(c_event.type, c_event.success, self._user_tag)
 */

  /* function exit code */
  __pyx_L1_error:;
  __Pyx_XDECREF(__pyx_t_1);
  __Pyx_XDECREF(__pyx_t_2);
  __Pyx_XDECREF(__pyx_t_3);
  __Pyx_AddTraceback("grpc._cython.cygrpc._ServerShutdownTag.event", __pyx_clineno, __pyx_lineno, __pyx_filename);
  __pyx_r = 0;
  __pyx_L0:;
  __Pyx_XGIVEREF((PyObject *)__pyx_r);
  __Pyx_RefNannyFinishContext();
  return __pyx_r;
}

struct __pyx_obj_4grpc_7_cython_6cygrpc_ServerShutdownEvent *__pyx_f_4grpc_7_cython_6cygrpc_18_ServerShutdownTag_event__pyx_wrap_1(struct __pyx_obj_4grpc_7_cython_6cygrpc__ServerShutdownTag *__pyx_v_self, grpc_event __pyx_v_c_event) {
  return __pyx_f_4grpc_7_cython_6cygrpc_18_ServerShutdownTag_event(__pyx_v_self, __pyx_v_c_event);
}

/* "src/python/grpcio/grpc/_cython/_cygrpc/tag.pxd.pxi":54
 * cdef class _ServerShutdownTag(_Tag):
 * 
 *   cdef readonly object _user_tag             # <<<<<<<<<<<<<<
 *   # This allows CompletionQueue to notify the Python Server object that the
 *   # underlying GRPC core server has shutdown
 */

/* Python wrapper */
static PyObject *__pyx_pw_4grpc_7_cython_6cygrpc_18_ServerShutdownTag_9_user_tag_1__get__(PyObject *__pyx_v_self); /*proto*/
static PyObject *__pyx_pw_4grpc_7_cython_6cygrpc_18_ServerShutdownTag_9_user_tag_1__get__(PyObject *__pyx_v_self) {
  PyObject *__pyx_r = 0;
  __Pyx_RefNannyDeclarations
  __Pyx_RefNannySetupContext("__get__ (wrapper)", 0);
  __pyx_r = __pyx_pf_4grpc_7_cython_6cygrpc_18_ServerShutdownTag_9_user_tag___get__(((struct __pyx_obj_4grpc_7_cython_6cygrpc__ServerShutdownTag *)__pyx_v_self));

  /* function exit code */
  __Pyx_RefNannyFinishContext();
  return __pyx_r;
}

static PyObject *__pyx_pf_4grpc_7_cython_6cygrpc_18_ServerShutdownTag_9_user_tag___get__(struct __pyx_obj_4grpc_7_cython_6cygrpc__ServerShutdownTag *__pyx_v_self) {
  PyObject *__pyx_r = NULL;
  __Pyx_RefNannyDeclarations
  __Pyx_RefNannySetupContext("__get__", 0);
  __Pyx_XDECREF(__pyx_r);
  __Pyx_INCREF(__pyx_v_self->_user_tag);
  __pyx_r = __pyx_v_self->_user_tag;
  goto __pyx_L0;

  /* function exit code */
  __pyx_L0:;
  __Pyx_XGIVEREF(__pyx_r);
  __Pyx_RefNannyFinishContext();
  return __pyx_r;
}

/* "src/python/grpcio/grpc/_cython/_cygrpc/tag.pxd.pxi":57
 *   # This allows CompletionQueue to notify the Python Server object that the
 *   # underlying GRPC core server has shutdown
 *   cdef readonly Server _shutting_down_server             # <<<<<<<<<<<<<<
 * 
 *   cdef ServerShutdownEvent event(self, grpc_event c_event)
 */

/* Python wrapper */
static PyObject *__pyx_pw_4grpc_7_cython_6cygrpc_18_ServerShutdownTag_21_shutting_down_server_1__get__(PyObject *__pyx_v_self); /*proto*/
static PyObject *__pyx_pw_4grpc_7_cython_6cygrpc_18_ServerShutdownTag_21_shutting_down_server_1__get__(PyObject *__pyx_v_self) {
  PyObject *__pyx_r = 0;
  __Pyx_RefNannyDeclarations
  __Pyx_RefNannySetupContext("__get__ (wrapper)", 0);
  __pyx_r = __pyx_pf_4grpc_7_cython_6cygrpc_18_ServerShutdownTag_21_shutting_down_server___get__(((struct __pyx_obj_4grpc_7_cython_6cygrpc__ServerShutdownTag *)__pyx_v_self));

  /* function exit code */
  __Pyx_RefNannyFinishContext();
  return __pyx_r;
}

static PyObject *__pyx_pf_4grpc_7_cython_6cygrpc_18_ServerShutdownTag_21_shutting_down_server___get__(struct __pyx_obj_4grpc_7_cython_6cygrpc__ServerShutdownTag *__pyx_v_self) {
  PyObject *__pyx_r = NULL;
  __Pyx_RefNannyDeclarations
  __Pyx_RefNannySetupContext("__get__", 0);
  __Pyx_XDECREF(__pyx_r);
  __Pyx_INCREF(((PyObject *)__pyx_v_self->_shutting_down_server));
  __pyx_r = ((PyObject *)__pyx_v_self->_shutting_down_server);
  goto __pyx_L0;

  /* function exit code */
  __pyx_L0:;
  __Pyx_XGIVEREF(__pyx_r);
  __Pyx_RefNannyFinishContext();
  return __pyx_r;
}

/* "(tree fragment)":1
 * def __reduce_cython__(self):             # <<<<<<<<<<<<<<
 *     raise TypeError("no default __reduce__ due to non-trivial __cinit__")
 * def __setstate_cython__(self, __pyx_state):
 */

/* Python wrapper */
static PyObject *__pyx_pw_4grpc_7_cython_6cygrpc_18_ServerShutdownTag_3__reduce_cython__(PyObject *__pyx_v_self, CYTHON_UNUSED PyObject *unused); /*proto*/
static PyObject *__pyx_pw_4grpc_7_cython_6cygrpc_18_ServerShutdownTag_3__reduce_cython__(PyObject *__pyx_v_self, CYTHON_UNUSED PyObject *unused) {
  PyObject *__pyx_r = 0;
  __Pyx_RefNannyDeclarations
  __Pyx_RefNannySetupContext("__reduce_cython__ (wrapper)", 0);
  __pyx_r = __pyx_pf_4grpc_7_cython_6cygrpc_18_ServerShutdownTag_2__reduce_cython__(((struct __pyx_obj_4grpc_7_cython_6cygrpc__ServerShutdownTag *)__pyx_v_self));

  /* function exit code */
  __Pyx_RefNannyFinishContext();
  return __pyx_r;
}

static PyObject *__pyx_pf_4grpc_7_cython_6cygrpc_18_ServerShutdownTag_2__reduce_cython__(CYTHON_UNUSED struct __pyx_obj_4grpc_7_cython_6cygrpc__ServerShutdownTag *__pyx_v_self) {
  PyObject *__pyx_r = NULL;
  __Pyx_RefNannyDeclarations
  PyObject *__pyx_t_1 = NULL;
  __Pyx_RefNannySetupContext("__reduce_cython__", 0);

  /* "(tree fragment)":2
 * def __reduce_cython__(self):
 *     raise TypeError("no default __reduce__ due to non-trivial __cinit__")             # <<<<<<<<<<<<<<
 * def __setstate_cython__(self, __pyx_state):
 *     raise TypeError("no default __reduce__ due to non-trivial __cinit__")
 */
  __pyx_t_1 = __Pyx_PyObject_Call(__pyx_builtin_TypeError, __pyx_tuple__126, NULL); if (unlikely(!__pyx_t_1)) __PYX_ERR(7, 2, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_1);
  __Pyx_Raise(__pyx_t_1, 0, 0, 0);
  __Pyx_DECREF(__pyx_t_1); __pyx_t_1 = 0;
  __PYX_ERR(7, 2, __pyx_L1_error)

  /* "(tree fragment)":1
 * def __reduce_cython__(self):             # <<<<<<<<<<<<<<
 *     raise TypeError("no default __reduce__ due to non-trivial __cinit__")
 * def __setstate_cython__(self, __pyx_state):
 */

  /* function exit code */
  __pyx_L1_error:;
  __Pyx_XDECREF(__pyx_t_1);
  __Pyx_AddTraceback("grpc._cython.cygrpc._ServerShutdownTag.__reduce_cython__", __pyx_clineno, __pyx_lineno, __pyx_filename);
  __pyx_r = NULL;
  __Pyx_XGIVEREF(__pyx_r);
  __Pyx_RefNannyFinishContext();
  return __pyx_r;
}

/* "(tree fragment)":3
 * def __reduce_cython__(self):
 *     raise TypeError("no default __reduce__ due to non-trivial __cinit__")
 * def __setstate_cython__(self, __pyx_state):             # <<<<<<<<<<<<<<
 *     raise TypeError("no default __reduce__ due to non-trivial __cinit__")
 */

/* Python wrapper */
static PyObject *__pyx_pw_4grpc_7_cython_6cygrpc_18_ServerShutdownTag_5__setstate_cython__(PyObject *__pyx_v_self, PyObject *__pyx_v___pyx_state); /*proto*/
static PyObject *__pyx_pw_4grpc_7_cython_6cygrpc_18_ServerShutdownTag_5__setstate_cython__(PyObject *__pyx_v_self, PyObject *__pyx_v___pyx_state) {
  PyObject *__pyx_r = 0;
  __Pyx_RefNannyDeclarations
  __Pyx_RefNannySetupContext("__setstate_cython__ (wrapper)", 0);
  __pyx_r = __pyx_pf_4grpc_7_cython_6cygrpc_18_ServerShutdownTag_4__setstate_cython__(((struct __pyx_obj_4grpc_7_cython_6cygrpc__ServerShutdownTag *)__pyx_v_self), ((PyObject *)__pyx_v___pyx_state));

  /* function exit code */
  __Pyx_RefNannyFinishContext();
  return __pyx_r;
}

static PyObject *__pyx_pf_4grpc_7_cython_6cygrpc_18_ServerShutdownTag_4__setstate_cython__(CYTHON_UNUSED struct __pyx_obj_4grpc_7_cython_6cygrpc__ServerShutdownTag *__pyx_v_self, CYTHON_UNUSED PyObject *__pyx_v___pyx_state) {
  PyObject *__pyx_r = NULL;
  __Pyx_RefNannyDeclarations
  PyObject *__pyx_t_1 = NULL;
  __Pyx_RefNannySetupContext("__setstate_cython__", 0);

  /* "(tree fragment)":4
 *     raise TypeError("no default __reduce__ due to non-trivial __cinit__")
 * def __setstate_cython__(self, __pyx_state):
 *     raise TypeError("no default __reduce__ due to non-trivial __cinit__")             # <<<<<<<<<<<<<<
 */
  __pyx_t_1 = __Pyx_PyObject_Call(__pyx_builtin_TypeError, __pyx_tuple__127, NULL); if (unlikely(!__pyx_t_1)) __PYX_ERR(7, 4, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_1);
  __Pyx_Raise(__pyx_t_1, 0, 0, 0);
  __Pyx_DECREF(__pyx_t_1); __pyx_t_1 = 0;
  __PYX_ERR(7, 4, __pyx_L1_error)

  /* "(tree fragment)":3
 * def __reduce_cython__(self):
 *     raise TypeError("no default __reduce__ due to non-trivial __cinit__")
 * def __setstate_cython__(self, __pyx_state):             # <<<<<<<<<<<<<<
 *     raise TypeError("no default __reduce__ due to non-trivial __cinit__")
 */

  /* function exit code */
  __pyx_L1_error:;
  __Pyx_XDECREF(__pyx_t_1);
  __Pyx_AddTraceback("grpc._cython.cygrpc._ServerShutdownTag.__setstate_cython__", __pyx_clineno, __pyx_lineno, __pyx_filename);
  __pyx_r = NULL;
  __Pyx_XGIVEREF(__pyx_r);
  __Pyx_RefNannyFinishContext();
  return __pyx_r;
}

/* "src/python/grpcio/grpc/_cython/_cygrpc/time.pyx.pxi":16
 * 
 * 
 * cdef gpr_timespec _timespec_from_time(object time) except *:             # <<<<<<<<<<<<<<
 *   if time is None:
 *     return gpr_inf_future(GPR_CLOCK_REALTIME)
 */

static gpr_timespec __pyx_f_4grpc_7_cython_6cygrpc__timespec_from_time(PyObject *__pyx_v_time) {
  gpr_timespec __pyx_r;
  __Pyx_RefNannyDeclarations
  int __pyx_t_1;
  int __pyx_t_2;
  double __pyx_t_3;
  __Pyx_RefNannySetupContext("_timespec_from_time", 0);

  /* "src/python/grpcio/grpc/_cython/_cygrpc/time.pyx.pxi":17
 * 
 * cdef gpr_timespec _timespec_from_time(object time) except *:
 *   if time is None:             # <<<<<<<<<<<<<<
 *     return gpr_inf_future(GPR_CLOCK_REALTIME)
 *   else:
 */
  __pyx_t_1 = (__pyx_v_time == Py_None);
  __pyx_t_2 = (__pyx_t_1 != 0);
  if (__pyx_t_2) {

    /* "src/python/grpcio/grpc/_cython/_cygrpc/time.pyx.pxi":18
 * cdef gpr_timespec _timespec_from_time(object time) except *:
 *   if time is None:
 *     return gpr_inf_future(GPR_CLOCK_REALTIME)             # <<<<<<<<<<<<<<
 *   else:
 *     return gpr_time_from_nanos(
 */
    __pyx_r = gpr_inf_future(GPR_CLOCK_REALTIME);
    goto __pyx_L0;

    /* "src/python/grpcio/grpc/_cython/_cygrpc/time.pyx.pxi":17
 * 
 * cdef gpr_timespec _timespec_from_time(object time) except *:
 *   if time is None:             # <<<<<<<<<<<<<<
 *     return gpr_inf_future(GPR_CLOCK_REALTIME)
 *   else:
 */
  }

  /* "src/python/grpcio/grpc/_cython/_cygrpc/time.pyx.pxi":20
 *     return gpr_inf_future(GPR_CLOCK_REALTIME)
 *   else:
 *     return gpr_time_from_nanos(             # <<<<<<<<<<<<<<
 *       <int64_t>(<double>time * GPR_NS_PER_SEC),
 *       GPR_CLOCK_REALTIME,
 */
  /*else*/ {

    /* "src/python/grpcio/grpc/_cython/_cygrpc/time.pyx.pxi":21
 *   else:
 *     return gpr_time_from_nanos(
 *       <int64_t>(<double>time * GPR_NS_PER_SEC),             # <<<<<<<<<<<<<<
 *       GPR_CLOCK_REALTIME,
 *     )
 */
    __pyx_t_3 = __pyx_PyFloat_AsDouble(__pyx_v_time); if (unlikely((__pyx_t_3 == (double)-1) && PyErr_Occurred())) __PYX_ERR(19, 21, __pyx_L1_error)

    /* "src/python/grpcio/grpc/_cython/_cygrpc/time.pyx.pxi":20
 *     return gpr_inf_future(GPR_CLOCK_REALTIME)
 *   else:
 *     return gpr_time_from_nanos(             # <<<<<<<<<<<<<<
 *       <int64_t>(<double>time * GPR_NS_PER_SEC),
 *       GPR_CLOCK_REALTIME,
 */
    __pyx_r = gpr_time_from_nanos(((__pyx_t_4grpc_7_cython_6cygrpc_int64_t)(((double)__pyx_t_3) * GPR_NS_PER_SEC)), GPR_CLOCK_REALTIME);
    goto __pyx_L0;
  }

  /* "src/python/grpcio/grpc/_cython/_cygrpc/time.pyx.pxi":16
 * 
 * 
 * cdef gpr_timespec _timespec_from_time(object time) except *:             # <<<<<<<<<<<<<<
 *   if time is None:
 *     return gpr_inf_future(GPR_CLOCK_REALTIME)
 */

  /* function exit code */
  __pyx_L1_error:;
  __Pyx_AddTraceback("grpc._cython.cygrpc._timespec_from_time", __pyx_clineno, __pyx_lineno, __pyx_filename);
  __Pyx_pretend_to_initialize(&__pyx_r);
  __pyx_L0:;
  __Pyx_RefNannyFinishContext();
  return __pyx_r;
}

/* "src/python/grpcio/grpc/_cython/_cygrpc/time.pyx.pxi":26
 * 
 * 
 * cdef double _time_from_timespec(gpr_timespec timespec) except *:             # <<<<<<<<<<<<<<
 *   cdef gpr_timespec real_timespec = gpr_convert_clock_type(
 *       timespec, GPR_CLOCK_REALTIME)
 */

static double __pyx_f_4grpc_7_cython_6cygrpc__time_from_timespec(gpr_timespec __pyx_v_timespec) {
  gpr_timespec __pyx_v_real_timespec;
  double __pyx_r;
  __Pyx_RefNannyDeclarations
  double __pyx_t_1;
  __Pyx_RefNannySetupContext("_time_from_timespec", 0);

  /* "src/python/grpcio/grpc/_cython/_cygrpc/time.pyx.pxi":27
 * 
 * cdef double _time_from_timespec(gpr_timespec timespec) except *:
 *   cdef gpr_timespec real_timespec = gpr_convert_clock_type(             # <<<<<<<<<<<<<<
 *       timespec, GPR_CLOCK_REALTIME)
 *   return gpr_timespec_to_micros(real_timespec) / GPR_US_PER_SEC
 */
  __pyx_v_real_timespec = gpr_convert_clock_type(__pyx_v_timespec, GPR_CLOCK_REALTIME);

  /* "src/python/grpcio/grpc/_cython/_cygrpc/time.pyx.pxi":29
 *   cdef gpr_timespec real_timespec = gpr_convert_clock_type(
 *       timespec, GPR_CLOCK_REALTIME)
 *   return gpr_timespec_to_micros(real_timespec) / GPR_US_PER_SEC             # <<<<<<<<<<<<<<
 */
  __pyx_t_1 = gpr_timespec_to_micros(__pyx_v_real_timespec);
  if (unlikely(GPR_US_PER_SEC == 0)) {
    PyErr_SetString(PyExc_ZeroDivisionError, "float division");
    __PYX_ERR(19, 29, __pyx_L1_error)
  }
  __pyx_r = (__pyx_t_1 / GPR_US_PER_SEC);
  goto __pyx_L0;

  /* "src/python/grpcio/grpc/_cython/_cygrpc/time.pyx.pxi":26
 * 
 * 
 * cdef double _time_from_timespec(gpr_timespec timespec) except *:             # <<<<<<<<<<<<<<
 *   cdef gpr_timespec real_timespec = gpr_convert_clock_type(
 *       timespec, GPR_CLOCK_REALTIME)
 */

  /* function exit code */
  __pyx_L1_error:;
  __Pyx_AddTraceback("grpc._cython.cygrpc._time_from_timespec", __pyx_clineno, __pyx_lineno, __pyx_filename);
  __pyx_r = -1;
  __pyx_L0:;
  __Pyx_RefNannyFinishContext();
  return __pyx_r;
}

/* "src/python/grpcio/grpc/_cython/_cygrpc/vtable.pyx.pxi":16
 * 
 * # TODO(https://github.com/grpc/grpc/issues/15662): Reform this.
 * cdef void* _copy_pointer(void* pointer):             # <<<<<<<<<<<<<<
 *   return pointer
 * 
 */

static void *__pyx_f_4grpc_7_cython_6cygrpc__copy_pointer(void *__pyx_v_pointer) {
  void *__pyx_r;
  __Pyx_RefNannyDeclarations
  __Pyx_RefNannySetupContext("_copy_pointer", 0);

  /* "src/python/grpcio/grpc/_cython/_cygrpc/vtable.pyx.pxi":17
 * # TODO(https://github.com/grpc/grpc/issues/15662): Reform this.
 * cdef void* _copy_pointer(void* pointer):
 *   return pointer             # <<<<<<<<<<<<<<
 * 
 * 
 */
  __pyx_r = __pyx_v_pointer;
  goto __pyx_L0;

  /* "src/python/grpcio/grpc/_cython/_cygrpc/vtable.pyx.pxi":16
 * 
 * # TODO(https://github.com/grpc/grpc/issues/15662): Reform this.
 * cdef void* _copy_pointer(void* pointer):             # <<<<<<<<<<<<<<
 *   return pointer
 * 
 */

  /* function exit code */
  __pyx_L0:;
  __Pyx_RefNannyFinishContext();
  return __pyx_r;
}

/* "src/python/grpcio/grpc/_cython/_cygrpc/vtable.pyx.pxi":21
 * 
 * # TODO(https://github.com/grpc/grpc/issues/15662): Reform this.
 * cdef void _destroy_pointer(void* pointer):             # <<<<<<<<<<<<<<
 *   pass
 * 
 */

static void __pyx_f_4grpc_7_cython_6cygrpc__destroy_pointer(CYTHON_UNUSED void *__pyx_v_pointer) {
  __Pyx_RefNannyDeclarations
  __Pyx_RefNannySetupContext("_destroy_pointer", 0);

  /* function exit code */
  __Pyx_RefNannyFinishContext();
}

/* "src/python/grpcio/grpc/_cython/_cygrpc/vtable.pyx.pxi":25
 * 
 * 
 * cdef int _compare_pointer(void* first_pointer, void* second_pointer):             # <<<<<<<<<<<<<<
 *   if first_pointer < second_pointer:
 *     return -1
 */

static int __pyx_f_4grpc_7_cython_6cygrpc__compare_pointer(void *__pyx_v_first_pointer, void *__pyx_v_second_pointer) {
  int __pyx_r;
  __Pyx_RefNannyDeclarations
  int __pyx_t_1;
  __Pyx_RefNannySetupContext("_compare_pointer", 0);

  /* "src/python/grpcio/grpc/_cython/_cygrpc/vtable.pyx.pxi":26
 * 
 * cdef int _compare_pointer(void* first_pointer, void* second_pointer):
 *   if first_pointer < second_pointer:             # <<<<<<<<<<<<<<
 *     return -1
 *   elif first_pointer > second_pointer:
 */
  __pyx_t_1 = ((__pyx_v_first_pointer < __pyx_v_second_pointer) != 0);
  if (__pyx_t_1) {

    /* "src/python/grpcio/grpc/_cython/_cygrpc/vtable.pyx.pxi":27
 * cdef int _compare_pointer(void* first_pointer, void* second_pointer):
 *   if first_pointer < second_pointer:
 *     return -1             # <<<<<<<<<<<<<<
 *   elif first_pointer > second_pointer:
 *     return 1
 */
    __pyx_r = -1;
    goto __pyx_L0;

    /* "src/python/grpcio/grpc/_cython/_cygrpc/vtable.pyx.pxi":26
 * 
 * cdef int _compare_pointer(void* first_pointer, void* second_pointer):
 *   if first_pointer < second_pointer:             # <<<<<<<<<<<<<<
 *     return -1
 *   elif first_pointer > second_pointer:
 */
  }

  /* "src/python/grpcio/grpc/_cython/_cygrpc/vtable.pyx.pxi":28
 *   if first_pointer < second_pointer:
 *     return -1
 *   elif first_pointer > second_pointer:             # <<<<<<<<<<<<<<
 *     return 1
 *   else:
 */
  __pyx_t_1 = ((__pyx_v_first_pointer > __pyx_v_second_pointer) != 0);
  if (__pyx_t_1) {

    /* "src/python/grpcio/grpc/_cython/_cygrpc/vtable.pyx.pxi":29
 *     return -1
 *   elif first_pointer > second_pointer:
 *     return 1             # <<<<<<<<<<<<<<
 *   else:
 *     return 0
 */
    __pyx_r = 1;
    goto __pyx_L0;

    /* "src/python/grpcio/grpc/_cython/_cygrpc/vtable.pyx.pxi":28
 *   if first_pointer < second_pointer:
 *     return -1
 *   elif first_pointer > second_pointer:             # <<<<<<<<<<<<<<
 *     return 1
 *   else:
 */
  }

  /* "src/python/grpcio/grpc/_cython/_cygrpc/vtable.pyx.pxi":31
 *     return 1
 *   else:
 *     return 0             # <<<<<<<<<<<<<<
 * 
 * 
 */
  /*else*/ {
    __pyx_r = 0;
    goto __pyx_L0;
  }

  /* "src/python/grpcio/grpc/_cython/_cygrpc/vtable.pyx.pxi":25
 * 
 * 
 * cdef int _compare_pointer(void* first_pointer, void* second_pointer):             # <<<<<<<<<<<<<<
 *   if first_pointer < second_pointer:
 *     return -1
 */

  /* function exit code */
  __pyx_L0:;
  __Pyx_RefNannyFinishContext();
  return __pyx_r;
}

/* "src/python/grpcio/grpc/_cython/_cygrpc/vtable.pyx.pxi":35
 * 
 * cdef class _VTable:
 *   def __cinit__(self):             # <<<<<<<<<<<<<<
 *     self.c_vtable.copy = &_copy_pointer
 *     self.c_vtable.destroy = &_destroy_pointer
 */

/* Python wrapper */
static int __pyx_pw_4grpc_7_cython_6cygrpc_7_VTable_1__cinit__(PyObject *__pyx_v_self, PyObject *__pyx_args, PyObject *__pyx_kwds); /*proto*/
static int __pyx_pw_4grpc_7_cython_6cygrpc_7_VTable_1__cinit__(PyObject *__pyx_v_self, PyObject *__pyx_args, PyObject *__pyx_kwds) {
  int __pyx_r;
  __Pyx_RefNannyDeclarations
  __Pyx_RefNannySetupContext("__cinit__ (wrapper)", 0);
  if (unlikely(PyTuple_GET_SIZE(__pyx_args) > 0)) {
    __Pyx_RaiseArgtupleInvalid("__cinit__", 1, 0, 0, PyTuple_GET_SIZE(__pyx_args)); return -1;}
  if (unlikely(__pyx_kwds) && unlikely(PyDict_Size(__pyx_kwds) > 0) && unlikely(!__Pyx_CheckKeywordStrings(__pyx_kwds, "__cinit__", 0))) return -1;
  __pyx_r = __pyx_pf_4grpc_7_cython_6cygrpc_7_VTable___cinit__(((struct __pyx_obj_4grpc_7_cython_6cygrpc__VTable *)__pyx_v_self));

  /* function exit code */
  __Pyx_RefNannyFinishContext();
  return __pyx_r;
}

static int __pyx_pf_4grpc_7_cython_6cygrpc_7_VTable___cinit__(struct __pyx_obj_4grpc_7_cython_6cygrpc__VTable *__pyx_v_self) {
  int __pyx_r;
  __Pyx_RefNannyDeclarations
  __Pyx_RefNannySetupContext("__cinit__", 0);

  /* "src/python/grpcio/grpc/_cython/_cygrpc/vtable.pyx.pxi":36
 * cdef class _VTable:
 *   def __cinit__(self):
 *     self.c_vtable.copy = &_copy_pointer             # <<<<<<<<<<<<<<
 *     self.c_vtable.destroy = &_destroy_pointer
 *     self.c_vtable.cmp = &_compare_pointer
 */
  __pyx_v_self->c_vtable.copy = (&__pyx_f_4grpc_7_cython_6cygrpc__copy_pointer);

  /* "src/python/grpcio/grpc/_cython/_cygrpc/vtable.pyx.pxi":37
 *   def __cinit__(self):
 *     self.c_vtable.copy = &_copy_pointer
 *     self.c_vtable.destroy = &_destroy_pointer             # <<<<<<<<<<<<<<
 *     self.c_vtable.cmp = &_compare_pointer
 * 
 */
  __pyx_v_self->c_vtable.destroy = (&__pyx_f_4grpc_7_cython_6cygrpc__destroy_pointer);

  /* "src/python/grpcio/grpc/_cython/_cygrpc/vtable.pyx.pxi":38
 *     self.c_vtable.copy = &_copy_pointer
 *     self.c_vtable.destroy = &_destroy_pointer
 *     self.c_vtable.cmp = &_compare_pointer             # <<<<<<<<<<<<<<
 * 
 */
  __pyx_v_self->c_vtable.cmp = (&__pyx_f_4grpc_7_cython_6cygrpc__compare_pointer);

  /* "src/python/grpcio/grpc/_cython/_cygrpc/vtable.pyx.pxi":35
 * 
 * cdef class _VTable:
 *   def __cinit__(self):             # <<<<<<<<<<<<<<
 *     self.c_vtable.copy = &_copy_pointer
 *     self.c_vtable.destroy = &_destroy_pointer
 */

  /* function exit code */
  __pyx_r = 0;
  __Pyx_RefNannyFinishContext();
  return __pyx_r;
}

/* "(tree fragment)":1
 * def __reduce_cython__(self):             # <<<<<<<<<<<<<<
 *     raise TypeError("no default __reduce__ due to non-trivial __cinit__")
 * def __setstate_cython__(self, __pyx_state):
 */

/* Python wrapper */
static PyObject *__pyx_pw_4grpc_7_cython_6cygrpc_7_VTable_3__reduce_cython__(PyObject *__pyx_v_self, CYTHON_UNUSED PyObject *unused); /*proto*/
static PyObject *__pyx_pw_4grpc_7_cython_6cygrpc_7_VTable_3__reduce_cython__(PyObject *__pyx_v_self, CYTHON_UNUSED PyObject *unused) {
  PyObject *__pyx_r = 0;
  __Pyx_RefNannyDeclarations
  __Pyx_RefNannySetupContext("__reduce_cython__ (wrapper)", 0);
  __pyx_r = __pyx_pf_4grpc_7_cython_6cygrpc_7_VTable_2__reduce_cython__(((struct __pyx_obj_4grpc_7_cython_6cygrpc__VTable *)__pyx_v_self));

  /* function exit code */
  __Pyx_RefNannyFinishContext();
  return __pyx_r;
}

static PyObject *__pyx_pf_4grpc_7_cython_6cygrpc_7_VTable_2__reduce_cython__(CYTHON_UNUSED struct __pyx_obj_4grpc_7_cython_6cygrpc__VTable *__pyx_v_self) {
  PyObject *__pyx_r = NULL;
  __Pyx_RefNannyDeclarations
  PyObject *__pyx_t_1 = NULL;
  __Pyx_RefNannySetupContext("__reduce_cython__", 0);

  /* "(tree fragment)":2
 * def __reduce_cython__(self):
 *     raise TypeError("no default __reduce__ due to non-trivial __cinit__")             # <<<<<<<<<<<<<<
 * def __setstate_cython__(self, __pyx_state):
 *     raise TypeError("no default __reduce__ due to non-trivial __cinit__")
 */
  __pyx_t_1 = __Pyx_PyObject_Call(__pyx_builtin_TypeError, __pyx_tuple__128, NULL); if (unlikely(!__pyx_t_1)) __PYX_ERR(7, 2, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_1);
  __Pyx_Raise(__pyx_t_1, 0, 0, 0);
  __Pyx_DECREF(__pyx_t_1); __pyx_t_1 = 0;
  __PYX_ERR(7, 2, __pyx_L1_error)

  /* "(tree fragment)":1
 * def __reduce_cython__(self):             # <<<<<<<<<<<<<<
 *     raise TypeError("no default __reduce__ due to non-trivial __cinit__")
 * def __setstate_cython__(self, __pyx_state):
 */

  /* function exit code */
  __pyx_L1_error:;
  __Pyx_XDECREF(__pyx_t_1);
  __Pyx_AddTraceback("grpc._cython.cygrpc._VTable.__reduce_cython__", __pyx_clineno, __pyx_lineno, __pyx_filename);
  __pyx_r = NULL;
  __Pyx_XGIVEREF(__pyx_r);
  __Pyx_RefNannyFinishContext();
  return __pyx_r;
}

/* "(tree fragment)":3
 * def __reduce_cython__(self):
 *     raise TypeError("no default __reduce__ due to non-trivial __cinit__")
 * def __setstate_cython__(self, __pyx_state):             # <<<<<<<<<<<<<<
 *     raise TypeError("no default __reduce__ due to non-trivial __cinit__")
 */

/* Python wrapper */
static PyObject *__pyx_pw_4grpc_7_cython_6cygrpc_7_VTable_5__setstate_cython__(PyObject *__pyx_v_self, PyObject *__pyx_v___pyx_state); /*proto*/
static PyObject *__pyx_pw_4grpc_7_cython_6cygrpc_7_VTable_5__setstate_cython__(PyObject *__pyx_v_self, PyObject *__pyx_v___pyx_state) {
  PyObject *__pyx_r = 0;
  __Pyx_RefNannyDeclarations
  __Pyx_RefNannySetupContext("__setstate_cython__ (wrapper)", 0);
  __pyx_r = __pyx_pf_4grpc_7_cython_6cygrpc_7_VTable_4__setstate_cython__(((struct __pyx_obj_4grpc_7_cython_6cygrpc__VTable *)__pyx_v_self), ((PyObject *)__pyx_v___pyx_state));

  /* function exit code */
  __Pyx_RefNannyFinishContext();
  return __pyx_r;
}

static PyObject *__pyx_pf_4grpc_7_cython_6cygrpc_7_VTable_4__setstate_cython__(CYTHON_UNUSED struct __pyx_obj_4grpc_7_cython_6cygrpc__VTable *__pyx_v_self, CYTHON_UNUSED PyObject *__pyx_v___pyx_state) {
  PyObject *__pyx_r = NULL;
  __Pyx_RefNannyDeclarations
  PyObject *__pyx_t_1 = NULL;
  __Pyx_RefNannySetupContext("__setstate_cython__", 0);

  /* "(tree fragment)":4
 *     raise TypeError("no default __reduce__ due to non-trivial __cinit__")
 * def __setstate_cython__(self, __pyx_state):
 *     raise TypeError("no default __reduce__ due to non-trivial __cinit__")             # <<<<<<<<<<<<<<
 */
  __pyx_t_1 = __Pyx_PyObject_Call(__pyx_builtin_TypeError, __pyx_tuple__129, NULL); if (unlikely(!__pyx_t_1)) __PYX_ERR(7, 4, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_1);
  __Pyx_Raise(__pyx_t_1, 0, 0, 0);
  __Pyx_DECREF(__pyx_t_1); __pyx_t_1 = 0;
  __PYX_ERR(7, 4, __pyx_L1_error)

  /* "(tree fragment)":3
 * def __reduce_cython__(self):
 *     raise TypeError("no default __reduce__ due to non-trivial __cinit__")
 * def __setstate_cython__(self, __pyx_state):             # <<<<<<<<<<<<<<
 *     raise TypeError("no default __reduce__ due to non-trivial __cinit__")
 */

  /* function exit code */
  __pyx_L1_error:;
  __Pyx_XDECREF(__pyx_t_1);
  __Pyx_AddTraceback("grpc._cython.cygrpc._VTable.__setstate_cython__", __pyx_clineno, __pyx_lineno, __pyx_filename);
  __pyx_r = NULL;
  __Pyx_XGIVEREF(__pyx_r);
  __Pyx_RefNannyFinishContext();
  return __pyx_r;
}

/* "src/python/grpcio/grpc/_cython/_cygrpc/_hooks.pyx.pxi":16
 * 
 * 
 * cdef object _custom_op_on_c_call(int op, grpc_call *call):             # <<<<<<<<<<<<<<
 *   raise NotImplementedError("No custom hooks are implemented")
 * 
 */

static PyObject *__pyx_f_4grpc_7_cython_6cygrpc__custom_op_on_c_call(CYTHON_UNUSED int __pyx_v_op, CYTHON_UNUSED grpc_call *__pyx_v_call) {
  PyObject *__pyx_r = NULL;
  __Pyx_RefNannyDeclarations
  PyObject *__pyx_t_1 = NULL;
  __Pyx_RefNannySetupContext("_custom_op_on_c_call", 0);

  /* "src/python/grpcio/grpc/_cython/_cygrpc/_hooks.pyx.pxi":17
 * 
 * cdef object _custom_op_on_c_call(int op, grpc_call *call):
 *   raise NotImplementedError("No custom hooks are implemented")             # <<<<<<<<<<<<<<
 * 
 * def install_context_from_request_call_event(RequestCallEvent event):
 */
  __pyx_t_1 = __Pyx_PyObject_Call(__pyx_builtin_NotImplementedError, __pyx_tuple__130, NULL); if (unlikely(!__pyx_t_1)) __PYX_ERR(20, 17, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_1);
  __Pyx_Raise(__pyx_t_1, 0, 0, 0);
  __Pyx_DECREF(__pyx_t_1); __pyx_t_1 = 0;
  __PYX_ERR(20, 17, __pyx_L1_error)

  /* "src/python/grpcio/grpc/_cython/_cygrpc/_hooks.pyx.pxi":16
 * 
 * 
 * cdef object _custom_op_on_c_call(int op, grpc_call *call):             # <<<<<<<<<<<<<<
 *   raise NotImplementedError("No custom hooks are implemented")
 * 
 */

  /* function exit code */
  __pyx_L1_error:;
  __Pyx_XDECREF(__pyx_t_1);
  __Pyx_AddTraceback("grpc._cython.cygrpc._custom_op_on_c_call", __pyx_clineno, __pyx_lineno, __pyx_filename);
  __pyx_r = 0;
  __Pyx_XGIVEREF(__pyx_r);
  __Pyx_RefNannyFinishContext();
  return __pyx_r;
}

/* "src/python/grpcio/grpc/_cython/_cygrpc/_hooks.pyx.pxi":19
 *   raise NotImplementedError("No custom hooks are implemented")
 * 
 * def install_context_from_request_call_event(RequestCallEvent event):             # <<<<<<<<<<<<<<
 *   pass
 * 
 */

/* Python wrapper */
static PyObject *__pyx_pw_4grpc_7_cython_6cygrpc_35install_context_from_request_call_event(PyObject *__pyx_self, PyObject *__pyx_v_event); /*proto*/
static PyMethodDef __pyx_mdef_4grpc_7_cython_6cygrpc_35install_context_from_request_call_event = {"install_context_from_request_call_event", (PyCFunction)__pyx_pw_4grpc_7_cython_6cygrpc_35install_context_from_request_call_event, METH_O, 0};
static PyObject *__pyx_pw_4grpc_7_cython_6cygrpc_35install_context_from_request_call_event(PyObject *__pyx_self, PyObject *__pyx_v_event) {
  PyObject *__pyx_r = 0;
  __Pyx_RefNannyDeclarations
  __Pyx_RefNannySetupContext("install_context_from_request_call_event (wrapper)", 0);
  if (unlikely(!__Pyx_ArgTypeTest(((PyObject *)__pyx_v_event), __pyx_ptype_4grpc_7_cython_6cygrpc_RequestCallEvent, 1, "event", 0))) __PYX_ERR(20, 19, __pyx_L1_error)
  __pyx_r = __pyx_pf_4grpc_7_cython_6cygrpc_34install_context_from_request_call_event(__pyx_self, ((struct __pyx_obj_4grpc_7_cython_6cygrpc_RequestCallEvent *)__pyx_v_event));

  /* function exit code */
  goto __pyx_L0;
  __pyx_L1_error:;
  __pyx_r = NULL;
  __pyx_L0:;
  __Pyx_RefNannyFinishContext();
  return __pyx_r;
}

static PyObject *__pyx_pf_4grpc_7_cython_6cygrpc_34install_context_from_request_call_event(CYTHON_UNUSED PyObject *__pyx_self, CYTHON_UNUSED struct __pyx_obj_4grpc_7_cython_6cygrpc_RequestCallEvent *__pyx_v_event) {
  PyObject *__pyx_r = NULL;
  __Pyx_RefNannyDeclarations
  __Pyx_RefNannySetupContext("install_context_from_request_call_event", 0);

  /* function exit code */
  __pyx_r = Py_None; __Pyx_INCREF(Py_None);
  __Pyx_XGIVEREF(__pyx_r);
  __Pyx_RefNannyFinishContext();
  return __pyx_r;
}

/* "src/python/grpcio/grpc/_cython/_cygrpc/_hooks.pyx.pxi":22
 *   pass
 * 
 * def uninstall_context():             # <<<<<<<<<<<<<<
 *   pass
 * 
 */

/* Python wrapper */
static PyObject *__pyx_pw_4grpc_7_cython_6cygrpc_37uninstall_context(PyObject *__pyx_self, CYTHON_UNUSED PyObject *unused); /*proto*/
static PyMethodDef __pyx_mdef_4grpc_7_cython_6cygrpc_37uninstall_context = {"uninstall_context", (PyCFunction)__pyx_pw_4grpc_7_cython_6cygrpc_37uninstall_context, METH_NOARGS, 0};
static PyObject *__pyx_pw_4grpc_7_cython_6cygrpc_37uninstall_context(PyObject *__pyx_self, CYTHON_UNUSED PyObject *unused) {
  PyObject *__pyx_r = 0;
  __Pyx_RefNannyDeclarations
  __Pyx_RefNannySetupContext("uninstall_context (wrapper)", 0);
  __pyx_r = __pyx_pf_4grpc_7_cython_6cygrpc_36uninstall_context(__pyx_self);

  /* function exit code */
  __Pyx_RefNannyFinishContext();
  return __pyx_r;
}

static PyObject *__pyx_pf_4grpc_7_cython_6cygrpc_36uninstall_context(CYTHON_UNUSED PyObject *__pyx_self) {
  PyObject *__pyx_r = NULL;
  __Pyx_RefNannyDeclarations
  __Pyx_RefNannySetupContext("uninstall_context", 0);

  /* function exit code */
  __pyx_r = Py_None; __Pyx_INCREF(Py_None);
  __Pyx_XGIVEREF(__pyx_r);
  __Pyx_RefNannyFinishContext();
  return __pyx_r;
}

/* "src/python/grpcio/grpc/_cython/_cygrpc/_hooks.pyx.pxi":25
 *   pass
 * 
 * def build_census_context():             # <<<<<<<<<<<<<<
 *   pass
 * 
 */

/* Python wrapper */
static PyObject *__pyx_pw_4grpc_7_cython_6cygrpc_39build_census_context(PyObject *__pyx_self, CYTHON_UNUSED PyObject *unused); /*proto*/
static PyMethodDef __pyx_mdef_4grpc_7_cython_6cygrpc_39build_census_context = {"build_census_context", (PyCFunction)__pyx_pw_4grpc_7_cython_6cygrpc_39build_census_context, METH_NOARGS, 0};
static PyObject *__pyx_pw_4grpc_7_cython_6cygrpc_39build_census_context(PyObject *__pyx_self, CYTHON_UNUSED PyObject *unused) {
  PyObject *__pyx_r = 0;
  __Pyx_RefNannyDeclarations
  __Pyx_RefNannySetupContext("build_census_context (wrapper)", 0);
  __pyx_r = __pyx_pf_4grpc_7_cython_6cygrpc_38build_census_context(__pyx_self);

  /* function exit code */
  __Pyx_RefNannyFinishContext();
  return __pyx_r;
}

static PyObject *__pyx_pf_4grpc_7_cython_6cygrpc_38build_census_context(CYTHON_UNUSED PyObject *__pyx_self) {
  PyObject *__pyx_r = NULL;
  __Pyx_RefNannyDeclarations
  __Pyx_RefNannySetupContext("build_census_context", 0);

  /* function exit code */
  __pyx_r = Py_None; __Pyx_INCREF(Py_None);
  __Pyx_XGIVEREF(__pyx_r);
  __Pyx_RefNannyFinishContext();
  return __pyx_r;
}

/* "(tree fragment)":1
 * def __reduce_cython__(self):             # <<<<<<<<<<<<<<
 *     cdef bint use_setstate
 *     state = ()
 */

/* Python wrapper */
static PyObject *__pyx_pw_4grpc_7_cython_6cygrpc_13CensusContext_1__reduce_cython__(PyObject *__pyx_v_self, CYTHON_UNUSED PyObject *unused); /*proto*/
static PyObject *__pyx_pw_4grpc_7_cython_6cygrpc_13CensusContext_1__reduce_cython__(PyObject *__pyx_v_self, CYTHON_UNUSED PyObject *unused) {
  PyObject *__pyx_r = 0;
  __Pyx_RefNannyDeclarations
  __Pyx_RefNannySetupContext("__reduce_cython__ (wrapper)", 0);
  __pyx_r = __pyx_pf_4grpc_7_cython_6cygrpc_13CensusContext___reduce_cython__(((struct __pyx_obj_4grpc_7_cython_6cygrpc_CensusContext *)__pyx_v_self));

  /* function exit code */
  __Pyx_RefNannyFinishContext();
  return __pyx_r;
}

static PyObject *__pyx_pf_4grpc_7_cython_6cygrpc_13CensusContext___reduce_cython__(struct __pyx_obj_4grpc_7_cython_6cygrpc_CensusContext *__pyx_v_self) {
  int __pyx_v_use_setstate;
  PyObject *__pyx_v_state = NULL;
  PyObject *__pyx_v__dict = NULL;
  PyObject *__pyx_r = NULL;
  __Pyx_RefNannyDeclarations
  PyObject *__pyx_t_1 = NULL;
  int __pyx_t_2;
  int __pyx_t_3;
  PyObject *__pyx_t_4 = NULL;
  PyObject *__pyx_t_5 = NULL;
  __Pyx_RefNannySetupContext("__reduce_cython__", 0);

  /* "(tree fragment)":3
 * def __reduce_cython__(self):
 *     cdef bint use_setstate
 *     state = ()             # <<<<<<<<<<<<<<
 *     _dict = getattr(self, '__dict__', None)
 *     if _dict is not None:
 */
  __Pyx_INCREF(__pyx_empty_tuple);
  __pyx_v_state = __pyx_empty_tuple;

  /* "(tree fragment)":4
 *     cdef bint use_setstate
 *     state = ()
 *     _dict = getattr(self, '__dict__', None)             # <<<<<<<<<<<<<<
 *     if _dict is not None:
 *         state += (_dict,)
 */
  __pyx_t_1 = __Pyx_GetAttr3(((PyObject *)__pyx_v_self), __pyx_n_s_dict, Py_None); if (unlikely(!__pyx_t_1)) __PYX_ERR(7, 4, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_1);
  __pyx_v__dict = __pyx_t_1;
  __pyx_t_1 = 0;

  /* "(tree fragment)":5
 *     state = ()
 *     _dict = getattr(self, '__dict__', None)
 *     if _dict is not None:             # <<<<<<<<<<<<<<
 *         state += (_dict,)
 *         use_setstate = True
 */
  __pyx_t_2 = (__pyx_v__dict != Py_None);
  __pyx_t_3 = (__pyx_t_2 != 0);
  if (__pyx_t_3) {

    /* "(tree fragment)":6
 *     _dict = getattr(self, '__dict__', None)
 *     if _dict is not None:
 *         state += (_dict,)             # <<<<<<<<<<<<<<
 *         use_setstate = True
 *     else:
 */
    __pyx_t_1 = PyTuple_New(1); if (unlikely(!__pyx_t_1)) __PYX_ERR(7, 6, __pyx_L1_error)
    __Pyx_GOTREF(__pyx_t_1);
    __Pyx_INCREF(__pyx_v__dict);
    __Pyx_GIVEREF(__pyx_v__dict);
    PyTuple_SET_ITEM(__pyx_t_1, 0, __pyx_v__dict);
    __pyx_t_4 = PyNumber_InPlaceAdd(__pyx_v_state, __pyx_t_1); if (unlikely(!__pyx_t_4)) __PYX_ERR(7, 6, __pyx_L1_error)
    __Pyx_GOTREF(__pyx_t_4);
    __Pyx_DECREF(__pyx_t_1); __pyx_t_1 = 0;
    __Pyx_DECREF_SET(__pyx_v_state, ((PyObject*)__pyx_t_4));
    __pyx_t_4 = 0;

    /* "(tree fragment)":7
 *     if _dict is not None:
 *         state += (_dict,)
 *         use_setstate = True             # <<<<<<<<<<<<<<
 *     else:
 *         use_setstate = False
 */
    __pyx_v_use_setstate = 1;

    /* "(tree fragment)":5
 *     state = ()
 *     _dict = getattr(self, '__dict__', None)
 *     if _dict is not None:             # <<<<<<<<<<<<<<
 *         state += (_dict,)
 *         use_setstate = True
 */
    goto __pyx_L3;
  }

  /* "(tree fragment)":9
 *         use_setstate = True
 *     else:
 *         use_setstate = False             # <<<<<<<<<<<<<<
 *     if use_setstate:
 *         return __pyx_unpickle_CensusContext, (type(self), 0xd41d8cd, None), state
 */
  /*else*/ {
    __pyx_v_use_setstate = 0;
  }
  __pyx_L3:;

  /* "(tree fragment)":10
 *     else:
 *         use_setstate = False
 *     if use_setstate:             # <<<<<<<<<<<<<<
 *         return __pyx_unpickle_CensusContext, (type(self), 0xd41d8cd, None), state
 *     else:
 */
  __pyx_t_3 = (__pyx_v_use_setstate != 0);
  if (__pyx_t_3) {

    /* "(tree fragment)":11
 *         use_setstate = False
 *     if use_setstate:
 *         return __pyx_unpickle_CensusContext, (type(self), 0xd41d8cd, None), state             # <<<<<<<<<<<<<<
 *     else:
 *         return __pyx_unpickle_CensusContext, (type(self), 0xd41d8cd, state)
 */
    __Pyx_XDECREF(__pyx_r);
    __pyx_t_4 = __Pyx_GetModuleGlobalName(__pyx_n_s_pyx_unpickle_CensusContext); if (unlikely(!__pyx_t_4)) __PYX_ERR(7, 11, __pyx_L1_error)
    __Pyx_GOTREF(__pyx_t_4);
    __pyx_t_1 = PyTuple_New(3); if (unlikely(!__pyx_t_1)) __PYX_ERR(7, 11, __pyx_L1_error)
    __Pyx_GOTREF(__pyx_t_1);
    __Pyx_INCREF(((PyObject *)Py_TYPE(((PyObject *)__pyx_v_self))));
    __Pyx_GIVEREF(((PyObject *)Py_TYPE(((PyObject *)__pyx_v_self))));
    PyTuple_SET_ITEM(__pyx_t_1, 0, ((PyObject *)Py_TYPE(((PyObject *)__pyx_v_self))));
    __Pyx_INCREF(__pyx_int_222419149);
    __Pyx_GIVEREF(__pyx_int_222419149);
    PyTuple_SET_ITEM(__pyx_t_1, 1, __pyx_int_222419149);
    __Pyx_INCREF(Py_None);
    __Pyx_GIVEREF(Py_None);
    PyTuple_SET_ITEM(__pyx_t_1, 2, Py_None);
    __pyx_t_5 = PyTuple_New(3); if (unlikely(!__pyx_t_5)) __PYX_ERR(7, 11, __pyx_L1_error)
    __Pyx_GOTREF(__pyx_t_5);
    __Pyx_GIVEREF(__pyx_t_4);
    PyTuple_SET_ITEM(__pyx_t_5, 0, __pyx_t_4);
    __Pyx_GIVEREF(__pyx_t_1);
    PyTuple_SET_ITEM(__pyx_t_5, 1, __pyx_t_1);
    __Pyx_INCREF(__pyx_v_state);
    __Pyx_GIVEREF(__pyx_v_state);
    PyTuple_SET_ITEM(__pyx_t_5, 2, __pyx_v_state);
    __pyx_t_4 = 0;
    __pyx_t_1 = 0;
    __pyx_r = __pyx_t_5;
    __pyx_t_5 = 0;
    goto __pyx_L0;

    /* "(tree fragment)":10
 *     else:
 *         use_setstate = False
 *     if use_setstate:             # <<<<<<<<<<<<<<
 *         return __pyx_unpickle_CensusContext, (type(self), 0xd41d8cd, None), state
 *     else:
 */
  }

  /* "(tree fragment)":13
 *         return __pyx_unpickle_CensusContext, (type(self), 0xd41d8cd, None), state
 *     else:
 *         return __pyx_unpickle_CensusContext, (type(self), 0xd41d8cd, state)             # <<<<<<<<<<<<<<
 * def __setstate_cython__(self, __pyx_state):
 *     __pyx_unpickle_CensusContext__set_state(self, __pyx_state)
 */
  /*else*/ {
    __Pyx_XDECREF(__pyx_r);
    __pyx_t_5 = __Pyx_GetModuleGlobalName(__pyx_n_s_pyx_unpickle_CensusContext); if (unlikely(!__pyx_t_5)) __PYX_ERR(7, 13, __pyx_L1_error)
    __Pyx_GOTREF(__pyx_t_5);
    __pyx_t_1 = PyTuple_New(3); if (unlikely(!__pyx_t_1)) __PYX_ERR(7, 13, __pyx_L1_error)
    __Pyx_GOTREF(__pyx_t_1);
    __Pyx_INCREF(((PyObject *)Py_TYPE(((PyObject *)__pyx_v_self))));
    __Pyx_GIVEREF(((PyObject *)Py_TYPE(((PyObject *)__pyx_v_self))));
    PyTuple_SET_ITEM(__pyx_t_1, 0, ((PyObject *)Py_TYPE(((PyObject *)__pyx_v_self))));
    __Pyx_INCREF(__pyx_int_222419149);
    __Pyx_GIVEREF(__pyx_int_222419149);
    PyTuple_SET_ITEM(__pyx_t_1, 1, __pyx_int_222419149);
    __Pyx_INCREF(__pyx_v_state);
    __Pyx_GIVEREF(__pyx_v_state);
    PyTuple_SET_ITEM(__pyx_t_1, 2, __pyx_v_state);
    __pyx_t_4 = PyTuple_New(2); if (unlikely(!__pyx_t_4)) __PYX_ERR(7, 13, __pyx_L1_error)
    __Pyx_GOTREF(__pyx_t_4);
    __Pyx_GIVEREF(__pyx_t_5);
    PyTuple_SET_ITEM(__pyx_t_4, 0, __pyx_t_5);
    __Pyx_GIVEREF(__pyx_t_1);
    PyTuple_SET_ITEM(__pyx_t_4, 1, __pyx_t_1);
    __pyx_t_5 = 0;
    __pyx_t_1 = 0;
    __pyx_r = __pyx_t_4;
    __pyx_t_4 = 0;
    goto __pyx_L0;
  }

  /* "(tree fragment)":1
 * def __reduce_cython__(self):             # <<<<<<<<<<<<<<
 *     cdef bint use_setstate
 *     state = ()
 */

  /* function exit code */
  __pyx_L1_error:;
  __Pyx_XDECREF(__pyx_t_1);
  __Pyx_XDECREF(__pyx_t_4);
  __Pyx_XDECREF(__pyx_t_5);
  __Pyx_AddTraceback("grpc._cython.cygrpc.CensusContext.__reduce_cython__", __pyx_clineno, __pyx_lineno, __pyx_filename);
  __pyx_r = NULL;
  __pyx_L0:;
  __Pyx_XDECREF(__pyx_v_state);
  __Pyx_XDECREF(__pyx_v__dict);
  __Pyx_XGIVEREF(__pyx_r);
  __Pyx_RefNannyFinishContext();
  return __pyx_r;
}

/* "(tree fragment)":14
 *     else:
 *         return __pyx_unpickle_CensusContext, (type(self), 0xd41d8cd, state)
 * def __setstate_cython__(self, __pyx_state):             # <<<<<<<<<<<<<<
 *     __pyx_unpickle_CensusContext__set_state(self, __pyx_state)
 */

/* Python wrapper */
static PyObject *__pyx_pw_4grpc_7_cython_6cygrpc_13CensusContext_3__setstate_cython__(PyObject *__pyx_v_self, PyObject *__pyx_v___pyx_state); /*proto*/
static PyObject *__pyx_pw_4grpc_7_cython_6cygrpc_13CensusContext_3__setstate_cython__(PyObject *__pyx_v_self, PyObject *__pyx_v___pyx_state) {
  PyObject *__pyx_r = 0;
  __Pyx_RefNannyDeclarations
  __Pyx_RefNannySetupContext("__setstate_cython__ (wrapper)", 0);
  __pyx_r = __pyx_pf_4grpc_7_cython_6cygrpc_13CensusContext_2__setstate_cython__(((struct __pyx_obj_4grpc_7_cython_6cygrpc_CensusContext *)__pyx_v_self), ((PyObject *)__pyx_v___pyx_state));

  /* function exit code */
  __Pyx_RefNannyFinishContext();
  return __pyx_r;
}

static PyObject *__pyx_pf_4grpc_7_cython_6cygrpc_13CensusContext_2__setstate_cython__(struct __pyx_obj_4grpc_7_cython_6cygrpc_CensusContext *__pyx_v_self, PyObject *__pyx_v___pyx_state) {
  PyObject *__pyx_r = NULL;
  __Pyx_RefNannyDeclarations
  PyObject *__pyx_t_1 = NULL;
  __Pyx_RefNannySetupContext("__setstate_cython__", 0);

  /* "(tree fragment)":15
 *         return __pyx_unpickle_CensusContext, (type(self), 0xd41d8cd, state)
 * def __setstate_cython__(self, __pyx_state):
 *     __pyx_unpickle_CensusContext__set_state(self, __pyx_state)             # <<<<<<<<<<<<<<
 */
  if (!(likely(PyTuple_CheckExact(__pyx_v___pyx_state))||((__pyx_v___pyx_state) == Py_None)||(PyErr_Format(PyExc_TypeError, "Expected %.16s, got %.200s", "tuple", Py_TYPE(__pyx_v___pyx_state)->tp_name), 0))) __PYX_ERR(7, 15, __pyx_L1_error)
  __pyx_t_1 = __pyx_f_4grpc_7_cython_6cygrpc___pyx_unpickle_CensusContext__set_state(__pyx_v_self, ((PyObject*)__pyx_v___pyx_state)); if (unlikely(!__pyx_t_1)) __PYX_ERR(7, 15, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_1);
  __Pyx_DECREF(__pyx_t_1); __pyx_t_1 = 0;

  /* "(tree fragment)":14
 *     else:
 *         return __pyx_unpickle_CensusContext, (type(self), 0xd41d8cd, state)
 * def __setstate_cython__(self, __pyx_state):             # <<<<<<<<<<<<<<
 *     __pyx_unpickle_CensusContext__set_state(self, __pyx_state)
 */

  /* function exit code */
  __pyx_r = Py_None; __Pyx_INCREF(Py_None);
  goto __pyx_L0;
  __pyx_L1_error:;
  __Pyx_XDECREF(__pyx_t_1);
  __Pyx_AddTraceback("grpc._cython.cygrpc.CensusContext.__setstate_cython__", __pyx_clineno, __pyx_lineno, __pyx_filename);
  __pyx_r = NULL;
  __pyx_L0:;
  __Pyx_XGIVEREF(__pyx_r);
  __Pyx_RefNannyFinishContext();
  return __pyx_r;
}

/* "src/python/grpcio/grpc/_cython/_cygrpc/_hooks.pyx.pxi":31
 *   pass
 * 
 * def set_census_context_on_call(_CallState call_state, CensusContext census_ctx):             # <<<<<<<<<<<<<<
 *   pass
 * 
 */

/* Python wrapper */
static PyObject *__pyx_pw_4grpc_7_cython_6cygrpc_41set_census_context_on_call(PyObject *__pyx_self, PyObject *__pyx_args, PyObject *__pyx_kwds); /*proto*/
static PyMethodDef __pyx_mdef_4grpc_7_cython_6cygrpc_41set_census_context_on_call = {"set_census_context_on_call", (PyCFunction)__pyx_pw_4grpc_7_cython_6cygrpc_41set_census_context_on_call, METH_VARARGS|METH_KEYWORDS, 0};
static PyObject *__pyx_pw_4grpc_7_cython_6cygrpc_41set_census_context_on_call(PyObject *__pyx_self, PyObject *__pyx_args, PyObject *__pyx_kwds) {
  CYTHON_UNUSED struct __pyx_obj_4grpc_7_cython_6cygrpc__CallState *__pyx_v_call_state = 0;
  CYTHON_UNUSED struct __pyx_obj_4grpc_7_cython_6cygrpc_CensusContext *__pyx_v_census_ctx = 0;
  PyObject *__pyx_r = 0;
  __Pyx_RefNannyDeclarations
  __Pyx_RefNannySetupContext("set_census_context_on_call (wrapper)", 0);
  {
    static PyObject **__pyx_pyargnames[] = {&__pyx_n_s_call_state,&__pyx_n_s_census_ctx,0};
    PyObject* values[2] = {0,0};
    if (unlikely(__pyx_kwds)) {
      Py_ssize_t kw_args;
      const Py_ssize_t pos_args = PyTuple_GET_SIZE(__pyx_args);
      switch (pos_args) {
        case  2: values[1] = PyTuple_GET_ITEM(__pyx_args, 1);
        CYTHON_FALLTHROUGH;
        case  1: values[0] = PyTuple_GET_ITEM(__pyx_args, 0);
        CYTHON_FALLTHROUGH;
        case  0: break;
        default: goto __pyx_L5_argtuple_error;
      }
      kw_args = PyDict_Size(__pyx_kwds);
      switch (pos_args) {
        case  0:
        if (likely((values[0] = __Pyx_PyDict_GetItemStr(__pyx_kwds, __pyx_n_s_call_state)) != 0)) kw_args--;
        else goto __pyx_L5_argtuple_error;
        CYTHON_FALLTHROUGH;
        case  1:
        if (likely((values[1] = __Pyx_PyDict_GetItemStr(__pyx_kwds, __pyx_n_s_census_ctx)) != 0)) kw_args--;
        else {
          __Pyx_RaiseArgtupleInvalid("set_census_context_on_call", 1, 2, 2, 1); __PYX_ERR(20, 31, __pyx_L3_error)
        }
      }
      if (unlikely(kw_args > 0)) {
        if (unlikely(__Pyx_ParseOptionalKeywords(__pyx_kwds, __pyx_pyargnames, 0, values, pos_args, "set_census_context_on_call") < 0)) __PYX_ERR(20, 31, __pyx_L3_error)
      }
    } else if (PyTuple_GET_SIZE(__pyx_args) != 2) {
      goto __pyx_L5_argtuple_error;
    } else {
      values[0] = PyTuple_GET_ITEM(__pyx_args, 0);
      values[1] = PyTuple_GET_ITEM(__pyx_args, 1);
    }
    __pyx_v_call_state = ((struct __pyx_obj_4grpc_7_cython_6cygrpc__CallState *)values[0]);
    __pyx_v_census_ctx = ((struct __pyx_obj_4grpc_7_cython_6cygrpc_CensusContext *)values[1]);
  }
  goto __pyx_L4_argument_unpacking_done;
  __pyx_L5_argtuple_error:;
  __Pyx_RaiseArgtupleInvalid("set_census_context_on_call", 1, 2, 2, PyTuple_GET_SIZE(__pyx_args)); __PYX_ERR(20, 31, __pyx_L3_error)
  __pyx_L3_error:;
  __Pyx_AddTraceback("grpc._cython.cygrpc.set_census_context_on_call", __pyx_clineno, __pyx_lineno, __pyx_filename);
  __Pyx_RefNannyFinishContext();
  return NULL;
  __pyx_L4_argument_unpacking_done:;
  if (unlikely(!__Pyx_ArgTypeTest(((PyObject *)__pyx_v_call_state), __pyx_ptype_4grpc_7_cython_6cygrpc__CallState, 1, "call_state", 0))) __PYX_ERR(20, 31, __pyx_L1_error)
  if (unlikely(!__Pyx_ArgTypeTest(((PyObject *)__pyx_v_census_ctx), __pyx_ptype_4grpc_7_cython_6cygrpc_CensusContext, 1, "census_ctx", 0))) __PYX_ERR(20, 31, __pyx_L1_error)
  __pyx_r = __pyx_pf_4grpc_7_cython_6cygrpc_40set_census_context_on_call(__pyx_self, __pyx_v_call_state, __pyx_v_census_ctx);

  /* function exit code */
  goto __pyx_L0;
  __pyx_L1_error:;
  __pyx_r = NULL;
  __pyx_L0:;
  __Pyx_RefNannyFinishContext();
  return __pyx_r;
}

static PyObject *__pyx_pf_4grpc_7_cython_6cygrpc_40set_census_context_on_call(CYTHON_UNUSED PyObject *__pyx_self, CYTHON_UNUSED struct __pyx_obj_4grpc_7_cython_6cygrpc__CallState *__pyx_v_call_state, CYTHON_UNUSED struct __pyx_obj_4grpc_7_cython_6cygrpc_CensusContext *__pyx_v_census_ctx) {
  PyObject *__pyx_r = NULL;
  __Pyx_RefNannyDeclarations
  __Pyx_RefNannySetupContext("set_census_context_on_call", 0);

  /* function exit code */
  __pyx_r = Py_None; __Pyx_INCREF(Py_None);
  __Pyx_XGIVEREF(__pyx_r);
  __Pyx_RefNannyFinishContext();
  return __pyx_r;
}

/* "src/python/grpcio/grpc/_cython/_cygrpc/_hooks.pyx.pxi":34
 *   pass
 * 
 * def get_deadline_from_context():             # <<<<<<<<<<<<<<
 *   return None
 */

/* Python wrapper */
static PyObject *__pyx_pw_4grpc_7_cython_6cygrpc_43get_deadline_from_context(PyObject *__pyx_self, CYTHON_UNUSED PyObject *unused); /*proto*/
static PyMethodDef __pyx_mdef_4grpc_7_cython_6cygrpc_43get_deadline_from_context = {"get_deadline_from_context", (PyCFunction)__pyx_pw_4grpc_7_cython_6cygrpc_43get_deadline_from_context, METH_NOARGS, 0};
static PyObject *__pyx_pw_4grpc_7_cython_6cygrpc_43get_deadline_from_context(PyObject *__pyx_self, CYTHON_UNUSED PyObject *unused) {
  PyObject *__pyx_r = 0;
  __Pyx_RefNannyDeclarations
  __Pyx_RefNannySetupContext("get_deadline_from_context (wrapper)", 0);
  __pyx_r = __pyx_pf_4grpc_7_cython_6cygrpc_42get_deadline_from_context(__pyx_self);

  /* function exit code */
  __Pyx_RefNannyFinishContext();
  return __pyx_r;
}

static PyObject *__pyx_pf_4grpc_7_cython_6cygrpc_42get_deadline_from_context(CYTHON_UNUSED PyObject *__pyx_self) {
  PyObject *__pyx_r = NULL;
  __Pyx_RefNannyDeclarations
  __Pyx_RefNannySetupContext("get_deadline_from_context", 0);

  /* "src/python/grpcio/grpc/_cython/_cygrpc/_hooks.pyx.pxi":35
 * 
 * def get_deadline_from_context():
 *   return None             # <<<<<<<<<<<<<<
 */
  __Pyx_XDECREF(__pyx_r);
  __pyx_r = Py_None; __Pyx_INCREF(Py_None);
  goto __pyx_L0;

  /* "src/python/grpcio/grpc/_cython/_cygrpc/_hooks.pyx.pxi":34
 *   pass
 * 
 * def get_deadline_from_context():             # <<<<<<<<<<<<<<
 *   return None
 */

  /* function exit code */
  __pyx_L0:;
  __Pyx_XGIVEREF(__pyx_r);
  __Pyx_RefNannyFinishContext();
  return __pyx_r;
}

/* "src/python/grpcio/grpc/_cython/_cygrpc/grpc_gevent.pyx.pxi":27
 * g_pool = None
 * 
 * cdef grpc_error* grpc_error_none():             # <<<<<<<<<<<<<<
 *   return <grpc_error*>0
 * 
 */

static struct grpc_error *__pyx_f_4grpc_7_cython_6cygrpc_grpc_error_none(void) {
  struct grpc_error *__pyx_r;
  __Pyx_RefNannyDeclarations
  __Pyx_RefNannySetupContext("grpc_error_none", 0);

  /* "src/python/grpcio/grpc/_cython/_cygrpc/grpc_gevent.pyx.pxi":28
 * 
 * cdef grpc_error* grpc_error_none():
 *   return <grpc_error*>0             # <<<<<<<<<<<<<<
 * 
 * cdef grpc_error* socket_error(str syscall, str err):
 */
  __pyx_r = ((struct grpc_error *)0);
  goto __pyx_L0;

  /* "src/python/grpcio/grpc/_cython/_cygrpc/grpc_gevent.pyx.pxi":27
 * g_pool = None
 * 
 * cdef grpc_error* grpc_error_none():             # <<<<<<<<<<<<<<
 *   return <grpc_error*>0
 * 
 */

  /* function exit code */
  __pyx_L0:;
  __Pyx_RefNannyFinishContext();
  return __pyx_r;
}

/* "src/python/grpcio/grpc/_cython/_cygrpc/grpc_gevent.pyx.pxi":30
 *   return <grpc_error*>0
 * 
 * cdef grpc_error* socket_error(str syscall, str err):             # <<<<<<<<<<<<<<
 *   error_str = "{} failed: {}".format(syscall, err)
 *   error_bytes = str_to_bytes(error_str)
 */

static struct grpc_error *__pyx_f_4grpc_7_cython_6cygrpc_socket_error(PyObject *__pyx_v_syscall, PyObject *__pyx_v_err) {
  PyObject *__pyx_v_error_str = NULL;
  PyObject *__pyx_v_error_bytes = NULL;
  struct grpc_error *__pyx_r;
  __Pyx_RefNannyDeclarations
  PyObject *__pyx_t_1 = NULL;
  PyObject *__pyx_t_2 = NULL;
  PyObject *__pyx_t_3 = NULL;
  int __pyx_t_4;
  PyObject *__pyx_t_5 = NULL;
  char *__pyx_t_6;
  __Pyx_RefNannySetupContext("socket_error", 0);

  /* "src/python/grpcio/grpc/_cython/_cygrpc/grpc_gevent.pyx.pxi":31
 * 
 * cdef grpc_error* socket_error(str syscall, str err):
 *   error_str = "{} failed: {}".format(syscall, err)             # <<<<<<<<<<<<<<
 *   error_bytes = str_to_bytes(error_str)
 *   return grpc_socket_error(error_bytes)
 */
  __pyx_t_2 = __Pyx_PyObject_GetAttrStr(__pyx_kp_s_failed, __pyx_n_s_format); if (unlikely(!__pyx_t_2)) __PYX_ERR(6, 31, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_2);
  __pyx_t_3 = NULL;
  __pyx_t_4 = 0;
  if (CYTHON_UNPACK_METHODS && likely(PyMethod_Check(__pyx_t_2))) {
    __pyx_t_3 = PyMethod_GET_SELF(__pyx_t_2);
    if (likely(__pyx_t_3)) {
      PyObject* function = PyMethod_GET_FUNCTION(__pyx_t_2);
      __Pyx_INCREF(__pyx_t_3);
      __Pyx_INCREF(function);
      __Pyx_DECREF_SET(__pyx_t_2, function);
      __pyx_t_4 = 1;
    }
  }
  #if CYTHON_FAST_PYCALL
  if (PyFunction_Check(__pyx_t_2)) {
    PyObject *__pyx_temp[3] = {__pyx_t_3, __pyx_v_syscall, __pyx_v_err};
    __pyx_t_1 = __Pyx_PyFunction_FastCall(__pyx_t_2, __pyx_temp+1-__pyx_t_4, 2+__pyx_t_4); if (unlikely(!__pyx_t_1)) __PYX_ERR(6, 31, __pyx_L1_error)
    __Pyx_XDECREF(__pyx_t_3); __pyx_t_3 = 0;
    __Pyx_GOTREF(__pyx_t_1);
  } else
  #endif
  #if CYTHON_FAST_PYCCALL
  if (__Pyx_PyFastCFunction_Check(__pyx_t_2)) {
    PyObject *__pyx_temp[3] = {__pyx_t_3, __pyx_v_syscall, __pyx_v_err};
    __pyx_t_1 = __Pyx_PyCFunction_FastCall(__pyx_t_2, __pyx_temp+1-__pyx_t_4, 2+__pyx_t_4); if (unlikely(!__pyx_t_1)) __PYX_ERR(6, 31, __pyx_L1_error)
    __Pyx_XDECREF(__pyx_t_3); __pyx_t_3 = 0;
    __Pyx_GOTREF(__pyx_t_1);
  } else
  #endif
  {
    __pyx_t_5 = PyTuple_New(2+__pyx_t_4); if (unlikely(!__pyx_t_5)) __PYX_ERR(6, 31, __pyx_L1_error)
    __Pyx_GOTREF(__pyx_t_5);
    if (__pyx_t_3) {
      __Pyx_GIVEREF(__pyx_t_3); PyTuple_SET_ITEM(__pyx_t_5, 0, __pyx_t_3); __pyx_t_3 = NULL;
    }
    __Pyx_INCREF(__pyx_v_syscall);
    __Pyx_GIVEREF(__pyx_v_syscall);
    PyTuple_SET_ITEM(__pyx_t_5, 0+__pyx_t_4, __pyx_v_syscall);
    __Pyx_INCREF(__pyx_v_err);
    __Pyx_GIVEREF(__pyx_v_err);
    PyTuple_SET_ITEM(__pyx_t_5, 1+__pyx_t_4, __pyx_v_err);
    __pyx_t_1 = __Pyx_PyObject_Call(__pyx_t_2, __pyx_t_5, NULL); if (unlikely(!__pyx_t_1)) __PYX_ERR(6, 31, __pyx_L1_error)
    __Pyx_GOTREF(__pyx_t_1);
    __Pyx_DECREF(__pyx_t_5); __pyx_t_5 = 0;
  }
  __Pyx_DECREF(__pyx_t_2); __pyx_t_2 = 0;
  __pyx_v_error_str = __pyx_t_1;
  __pyx_t_1 = 0;

  /* "src/python/grpcio/grpc/_cython/_cygrpc/grpc_gevent.pyx.pxi":32
 * cdef grpc_error* socket_error(str syscall, str err):
 *   error_str = "{} failed: {}".format(syscall, err)
 *   error_bytes = str_to_bytes(error_str)             # <<<<<<<<<<<<<<
 *   return grpc_socket_error(error_bytes)
 * 
 */
  __pyx_t_1 = __pyx_f_4grpc_7_cython_6cygrpc_str_to_bytes(__pyx_v_error_str); if (unlikely(!__pyx_t_1)) __PYX_ERR(6, 32, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_1);
  __pyx_v_error_bytes = ((PyObject*)__pyx_t_1);
  __pyx_t_1 = 0;

  /* "src/python/grpcio/grpc/_cython/_cygrpc/grpc_gevent.pyx.pxi":33
 *   error_str = "{} failed: {}".format(syscall, err)
 *   error_bytes = str_to_bytes(error_str)
 *   return grpc_socket_error(error_bytes)             # <<<<<<<<<<<<<<
 * 
 * cdef resolved_addr_to_tuple(grpc_resolved_address* address):
 */
  if (unlikely(__pyx_v_error_bytes == Py_None)) {
    PyErr_SetString(PyExc_TypeError, "expected bytes, NoneType found");
    __PYX_ERR(6, 33, __pyx_L1_error)
  }
  __pyx_t_6 = __Pyx_PyBytes_AsWritableString(__pyx_v_error_bytes); if (unlikely((!__pyx_t_6) && PyErr_Occurred())) __PYX_ERR(6, 33, __pyx_L1_error)
  __pyx_r = grpc_socket_error(__pyx_t_6);
  goto __pyx_L0;

  /* "src/python/grpcio/grpc/_cython/_cygrpc/grpc_gevent.pyx.pxi":30
 *   return <grpc_error*>0
 * 
 * cdef grpc_error* socket_error(str syscall, str err):             # <<<<<<<<<<<<<<
 *   error_str = "{} failed: {}".format(syscall, err)
 *   error_bytes = str_to_bytes(error_str)
 */

  /* function exit code */
  __pyx_L1_error:;
  __Pyx_XDECREF(__pyx_t_1);
  __Pyx_XDECREF(__pyx_t_2);
  __Pyx_XDECREF(__pyx_t_3);
  __Pyx_XDECREF(__pyx_t_5);
  __Pyx_WriteUnraisable("grpc._cython.cygrpc.socket_error", __pyx_clineno, __pyx_lineno, __pyx_filename, 1, 0);
  __pyx_r = 0;
  __pyx_L0:;
  __Pyx_XDECREF(__pyx_v_error_str);
  __Pyx_XDECREF(__pyx_v_error_bytes);
  __Pyx_RefNannyFinishContext();
  return __pyx_r;
}

/* "src/python/grpcio/grpc/_cython/_cygrpc/grpc_gevent.pyx.pxi":35
 *   return grpc_socket_error(error_bytes)
 * 
 * cdef resolved_addr_to_tuple(grpc_resolved_address* address):             # <<<<<<<<<<<<<<
 *   cdef char* res_str
 *   port = grpc_sockaddr_get_port(address)
 */

static PyObject *__pyx_f_4grpc_7_cython_6cygrpc_resolved_addr_to_tuple(grpc_resolved_address *__pyx_v_address) {
  char *__pyx_v_res_str;
  PyObject *__pyx_v_port = NULL;
  int __pyx_v_str_len;
  PyObject *__pyx_v_byte_str = NULL;
  PyObject *__pyx_r = NULL;
  __Pyx_RefNannyDeclarations
  PyObject *__pyx_t_1 = NULL;
  PyObject *__pyx_t_2 = NULL;
  PyObject *__pyx_t_3 = NULL;
  PyObject *__pyx_t_4 = NULL;
  PyObject *__pyx_t_5 = NULL;
  int __pyx_t_6;
  Py_ssize_t __pyx_t_7;
  __Pyx_RefNannySetupContext("resolved_addr_to_tuple", 0);

  /* "src/python/grpcio/grpc/_cython/_cygrpc/grpc_gevent.pyx.pxi":37
 * cdef resolved_addr_to_tuple(grpc_resolved_address* address):
 *   cdef char* res_str
 *   port = grpc_sockaddr_get_port(address)             # <<<<<<<<<<<<<<
 *   str_len = grpc_sockaddr_to_string(&res_str, address, 0)
 *   byte_str = _decode(<bytes>res_str[:str_len])
 */
  __pyx_t_1 = __Pyx_PyInt_From_int(grpc_sockaddr_get_port(__pyx_v_address)); if (unlikely(!__pyx_t_1)) __PYX_ERR(6, 37, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_1);
  __pyx_v_port = __pyx_t_1;
  __pyx_t_1 = 0;

  /* "src/python/grpcio/grpc/_cython/_cygrpc/grpc_gevent.pyx.pxi":38
 *   cdef char* res_str
 *   port = grpc_sockaddr_get_port(address)
 *   str_len = grpc_sockaddr_to_string(&res_str, address, 0)             # <<<<<<<<<<<<<<
 *   byte_str = _decode(<bytes>res_str[:str_len])
 *   if byte_str.endswith(':' + str(port)):
 */
  __pyx_v_str_len = grpc_sockaddr_to_string((&__pyx_v_res_str), __pyx_v_address, 0);

  /* "src/python/grpcio/grpc/_cython/_cygrpc/grpc_gevent.pyx.pxi":39
 *   port = grpc_sockaddr_get_port(address)
 *   str_len = grpc_sockaddr_to_string(&res_str, address, 0)
 *   byte_str = _decode(<bytes>res_str[:str_len])             # <<<<<<<<<<<<<<
 *   if byte_str.endswith(':' + str(port)):
 *     byte_str = byte_str[:(0 - len(str(port)) - 1)]
 */
  __pyx_t_1 = __Pyx_PyBytes_FromStringAndSize(__pyx_v_res_str + 0, __pyx_v_str_len - 0); if (unlikely(!__pyx_t_1)) __PYX_ERR(6, 39, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_1);
  __pyx_t_2 = __pyx_f_4grpc_7_cython_6cygrpc__decode(((PyObject*)__pyx_t_1)); if (unlikely(!__pyx_t_2)) __PYX_ERR(6, 39, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_2);
  __Pyx_DECREF(__pyx_t_1); __pyx_t_1 = 0;
  __pyx_v_byte_str = __pyx_t_2;
  __pyx_t_2 = 0;

  /* "src/python/grpcio/grpc/_cython/_cygrpc/grpc_gevent.pyx.pxi":40
 *   str_len = grpc_sockaddr_to_string(&res_str, address, 0)
 *   byte_str = _decode(<bytes>res_str[:str_len])
 *   if byte_str.endswith(':' + str(port)):             # <<<<<<<<<<<<<<
 *     byte_str = byte_str[:(0 - len(str(port)) - 1)]
 *   byte_str = byte_str.lstrip('[')
 */
  __pyx_t_1 = __Pyx_PyObject_GetAttrStr(__pyx_v_byte_str, __pyx_n_s_endswith); if (unlikely(!__pyx_t_1)) __PYX_ERR(6, 40, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_1);
  __pyx_t_3 = __Pyx_PyObject_CallOneArg(((PyObject *)(&PyString_Type)), __pyx_v_port); if (unlikely(!__pyx_t_3)) __PYX_ERR(6, 40, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_3);
  __pyx_t_4 = PyNumber_Add(__pyx_kp_s__131, __pyx_t_3); if (unlikely(!__pyx_t_4)) __PYX_ERR(6, 40, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_4);
  __Pyx_DECREF(__pyx_t_3); __pyx_t_3 = 0;
  __pyx_t_3 = NULL;
  if (CYTHON_UNPACK_METHODS && likely(PyMethod_Check(__pyx_t_1))) {
    __pyx_t_3 = PyMethod_GET_SELF(__pyx_t_1);
    if (likely(__pyx_t_3)) {
      PyObject* function = PyMethod_GET_FUNCTION(__pyx_t_1);
      __Pyx_INCREF(__pyx_t_3);
      __Pyx_INCREF(function);
      __Pyx_DECREF_SET(__pyx_t_1, function);
    }
  }
  if (!__pyx_t_3) {
    __pyx_t_2 = __Pyx_PyObject_CallOneArg(__pyx_t_1, __pyx_t_4); if (unlikely(!__pyx_t_2)) __PYX_ERR(6, 40, __pyx_L1_error)
    __Pyx_DECREF(__pyx_t_4); __pyx_t_4 = 0;
    __Pyx_GOTREF(__pyx_t_2);
  } else {
    #if CYTHON_FAST_PYCALL
    if (PyFunction_Check(__pyx_t_1)) {
      PyObject *__pyx_temp[2] = {__pyx_t_3, __pyx_t_4};
      __pyx_t_2 = __Pyx_PyFunction_FastCall(__pyx_t_1, __pyx_temp+1-1, 1+1); if (unlikely(!__pyx_t_2)) __PYX_ERR(6, 40, __pyx_L1_error)
      __Pyx_XDECREF(__pyx_t_3); __pyx_t_3 = 0;
      __Pyx_GOTREF(__pyx_t_2);
      __Pyx_DECREF(__pyx_t_4); __pyx_t_4 = 0;
    } else
    #endif
    #if CYTHON_FAST_PYCCALL
    if (__Pyx_PyFastCFunction_Check(__pyx_t_1)) {
      PyObject *__pyx_temp[2] = {__pyx_t_3, __pyx_t_4};
      __pyx_t_2 = __Pyx_PyCFunction_FastCall(__pyx_t_1, __pyx_temp+1-1, 1+1); if (unlikely(!__pyx_t_2)) __PYX_ERR(6, 40, __pyx_L1_error)
      __Pyx_XDECREF(__pyx_t_3); __pyx_t_3 = 0;
      __Pyx_GOTREF(__pyx_t_2);
      __Pyx_DECREF(__pyx_t_4); __pyx_t_4 = 0;
    } else
    #endif
    {
      __pyx_t_5 = PyTuple_New(1+1); if (unlikely(!__pyx_t_5)) __PYX_ERR(6, 40, __pyx_L1_error)
      __Pyx_GOTREF(__pyx_t_5);
      __Pyx_GIVEREF(__pyx_t_3); PyTuple_SET_ITEM(__pyx_t_5, 0, __pyx_t_3); __pyx_t_3 = NULL;
      __Pyx_GIVEREF(__pyx_t_4);
      PyTuple_SET_ITEM(__pyx_t_5, 0+1, __pyx_t_4);
      __pyx_t_4 = 0;
      __pyx_t_2 = __Pyx_PyObject_Call(__pyx_t_1, __pyx_t_5, NULL); if (unlikely(!__pyx_t_2)) __PYX_ERR(6, 40, __pyx_L1_error)
      __Pyx_GOTREF(__pyx_t_2);
      __Pyx_DECREF(__pyx_t_5); __pyx_t_5 = 0;
    }
  }
  __Pyx_DECREF(__pyx_t_1); __pyx_t_1 = 0;
  __pyx_t_6 = __Pyx_PyObject_IsTrue(__pyx_t_2); if (unlikely(__pyx_t_6 < 0)) __PYX_ERR(6, 40, __pyx_L1_error)
  __Pyx_DECREF(__pyx_t_2); __pyx_t_2 = 0;
  if (__pyx_t_6) {

    /* "src/python/grpcio/grpc/_cython/_cygrpc/grpc_gevent.pyx.pxi":41
 *   byte_str = _decode(<bytes>res_str[:str_len])
 *   if byte_str.endswith(':' + str(port)):
 *     byte_str = byte_str[:(0 - len(str(port)) - 1)]             # <<<<<<<<<<<<<<
 *   byte_str = byte_str.lstrip('[')
 *   byte_str = byte_str.rstrip(']')
 */
    __pyx_t_2 = __Pyx_PyObject_CallOneArg(((PyObject *)(&PyString_Type)), __pyx_v_port); if (unlikely(!__pyx_t_2)) __PYX_ERR(6, 41, __pyx_L1_error)
    __Pyx_GOTREF(__pyx_t_2);
    __pyx_t_7 = PyObject_Length(__pyx_t_2); if (unlikely(__pyx_t_7 == ((Py_ssize_t)-1))) __PYX_ERR(6, 41, __pyx_L1_error)
    __Pyx_DECREF(__pyx_t_2); __pyx_t_2 = 0;
    __pyx_t_2 = __Pyx_PyObject_GetSlice(__pyx_v_byte_str, 0, ((0 - __pyx_t_7) - 1), NULL, NULL, NULL, 0, 1, 1); if (unlikely(!__pyx_t_2)) __PYX_ERR(6, 41, __pyx_L1_error)
    __Pyx_GOTREF(__pyx_t_2);
    __Pyx_DECREF_SET(__pyx_v_byte_str, __pyx_t_2);
    __pyx_t_2 = 0;

    /* "src/python/grpcio/grpc/_cython/_cygrpc/grpc_gevent.pyx.pxi":40
 *   str_len = grpc_sockaddr_to_string(&res_str, address, 0)
 *   byte_str = _decode(<bytes>res_str[:str_len])
 *   if byte_str.endswith(':' + str(port)):             # <<<<<<<<<<<<<<
 *     byte_str = byte_str[:(0 - len(str(port)) - 1)]
 *   byte_str = byte_str.lstrip('[')
 */
  }

  /* "src/python/grpcio/grpc/_cython/_cygrpc/grpc_gevent.pyx.pxi":42
 *   if byte_str.endswith(':' + str(port)):
 *     byte_str = byte_str[:(0 - len(str(port)) - 1)]
 *   byte_str = byte_str.lstrip('[')             # <<<<<<<<<<<<<<
 *   byte_str = byte_str.rstrip(']')
 *   byte_str = '{}'.format(byte_str)
 */
  __pyx_t_2 = __Pyx_PyObject_GetAttrStr(__pyx_v_byte_str, __pyx_n_s_lstrip); if (unlikely(!__pyx_t_2)) __PYX_ERR(6, 42, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_2);
  __pyx_t_1 = __Pyx_PyObject_Call(__pyx_t_2, __pyx_tuple__133, NULL); if (unlikely(!__pyx_t_1)) __PYX_ERR(6, 42, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_1);
  __Pyx_DECREF(__pyx_t_2); __pyx_t_2 = 0;
  __Pyx_DECREF_SET(__pyx_v_byte_str, __pyx_t_1);
  __pyx_t_1 = 0;

  /* "src/python/grpcio/grpc/_cython/_cygrpc/grpc_gevent.pyx.pxi":43
 *     byte_str = byte_str[:(0 - len(str(port)) - 1)]
 *   byte_str = byte_str.lstrip('[')
 *   byte_str = byte_str.rstrip(']')             # <<<<<<<<<<<<<<
 *   byte_str = '{}'.format(byte_str)
 *   return byte_str, port
 */
  __pyx_t_1 = __Pyx_PyObject_GetAttrStr(__pyx_v_byte_str, __pyx_n_s_rstrip); if (unlikely(!__pyx_t_1)) __PYX_ERR(6, 43, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_1);
  __pyx_t_2 = __Pyx_PyObject_Call(__pyx_t_1, __pyx_tuple__135, NULL); if (unlikely(!__pyx_t_2)) __PYX_ERR(6, 43, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_2);
  __Pyx_DECREF(__pyx_t_1); __pyx_t_1 = 0;
  __Pyx_DECREF_SET(__pyx_v_byte_str, __pyx_t_2);
  __pyx_t_2 = 0;

  /* "src/python/grpcio/grpc/_cython/_cygrpc/grpc_gevent.pyx.pxi":44
 *   byte_str = byte_str.lstrip('[')
 *   byte_str = byte_str.rstrip(']')
 *   byte_str = '{}'.format(byte_str)             # <<<<<<<<<<<<<<
 *   return byte_str, port
 * 
 */
  __pyx_t_1 = __Pyx_PyObject_GetAttrStr(__pyx_kp_s__136, __pyx_n_s_format); if (unlikely(!__pyx_t_1)) __PYX_ERR(6, 44, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_1);
  __pyx_t_5 = NULL;
  if (CYTHON_UNPACK_METHODS && likely(PyMethod_Check(__pyx_t_1))) {
    __pyx_t_5 = PyMethod_GET_SELF(__pyx_t_1);
    if (likely(__pyx_t_5)) {
      PyObject* function = PyMethod_GET_FUNCTION(__pyx_t_1);
      __Pyx_INCREF(__pyx_t_5);
      __Pyx_INCREF(function);
      __Pyx_DECREF_SET(__pyx_t_1, function);
    }
  }
  if (!__pyx_t_5) {
    __pyx_t_2 = __Pyx_PyObject_CallOneArg(__pyx_t_1, __pyx_v_byte_str); if (unlikely(!__pyx_t_2)) __PYX_ERR(6, 44, __pyx_L1_error)
    __Pyx_GOTREF(__pyx_t_2);
  } else {
    #if CYTHON_FAST_PYCALL
    if (PyFunction_Check(__pyx_t_1)) {
      PyObject *__pyx_temp[2] = {__pyx_t_5, __pyx_v_byte_str};
      __pyx_t_2 = __Pyx_PyFunction_FastCall(__pyx_t_1, __pyx_temp+1-1, 1+1); if (unlikely(!__pyx_t_2)) __PYX_ERR(6, 44, __pyx_L1_error)
      __Pyx_XDECREF(__pyx_t_5); __pyx_t_5 = 0;
      __Pyx_GOTREF(__pyx_t_2);
    } else
    #endif
    #if CYTHON_FAST_PYCCALL
    if (__Pyx_PyFastCFunction_Check(__pyx_t_1)) {
      PyObject *__pyx_temp[2] = {__pyx_t_5, __pyx_v_byte_str};
      __pyx_t_2 = __Pyx_PyCFunction_FastCall(__pyx_t_1, __pyx_temp+1-1, 1+1); if (unlikely(!__pyx_t_2)) __PYX_ERR(6, 44, __pyx_L1_error)
      __Pyx_XDECREF(__pyx_t_5); __pyx_t_5 = 0;
      __Pyx_GOTREF(__pyx_t_2);
    } else
    #endif
    {
      __pyx_t_4 = PyTuple_New(1+1); if (unlikely(!__pyx_t_4)) __PYX_ERR(6, 44, __pyx_L1_error)
      __Pyx_GOTREF(__pyx_t_4);
      __Pyx_GIVEREF(__pyx_t_5); PyTuple_SET_ITEM(__pyx_t_4, 0, __pyx_t_5); __pyx_t_5 = NULL;
      __Pyx_INCREF(__pyx_v_byte_str);
      __Pyx_GIVEREF(__pyx_v_byte_str);
      PyTuple_SET_ITEM(__pyx_t_4, 0+1, __pyx_v_byte_str);
      __pyx_t_2 = __Pyx_PyObject_Call(__pyx_t_1, __pyx_t_4, NULL); if (unlikely(!__pyx_t_2)) __PYX_ERR(6, 44, __pyx_L1_error)
      __Pyx_GOTREF(__pyx_t_2);
      __Pyx_DECREF(__pyx_t_4); __pyx_t_4 = 0;
    }
  }
  __Pyx_DECREF(__pyx_t_1); __pyx_t_1 = 0;
  __Pyx_DECREF_SET(__pyx_v_byte_str, __pyx_t_2);
  __pyx_t_2 = 0;

  /* "src/python/grpcio/grpc/_cython/_cygrpc/grpc_gevent.pyx.pxi":45
 *   byte_str = byte_str.rstrip(']')
 *   byte_str = '{}'.format(byte_str)
 *   return byte_str, port             # <<<<<<<<<<<<<<
 * 
 * cdef sockaddr_to_tuple(const grpc_sockaddr* address, size_t length):
 */
  __Pyx_XDECREF(__pyx_r);
  __pyx_t_2 = PyTuple_New(2); if (unlikely(!__pyx_t_2)) __PYX_ERR(6, 45, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_2);
  __Pyx_INCREF(__pyx_v_byte_str);
  __Pyx_GIVEREF(__pyx_v_byte_str);
  PyTuple_SET_ITEM(__pyx_t_2, 0, __pyx_v_byte_str);
  __Pyx_INCREF(__pyx_v_port);
  __Pyx_GIVEREF(__pyx_v_port);
  PyTuple_SET_ITEM(__pyx_t_2, 1, __pyx_v_port);
  __pyx_r = __pyx_t_2;
  __pyx_t_2 = 0;
  goto __pyx_L0;

  /* "src/python/grpcio/grpc/_cython/_cygrpc/grpc_gevent.pyx.pxi":35
 *   return grpc_socket_error(error_bytes)
 * 
 * cdef resolved_addr_to_tuple(grpc_resolved_address* address):             # <<<<<<<<<<<<<<
 *   cdef char* res_str
 *   port = grpc_sockaddr_get_port(address)
 */

  /* function exit code */
  __pyx_L1_error:;
  __Pyx_XDECREF(__pyx_t_1);
  __Pyx_XDECREF(__pyx_t_2);
  __Pyx_XDECREF(__pyx_t_3);
  __Pyx_XDECREF(__pyx_t_4);
  __Pyx_XDECREF(__pyx_t_5);
  __Pyx_AddTraceback("grpc._cython.cygrpc.resolved_addr_to_tuple", __pyx_clineno, __pyx_lineno, __pyx_filename);
  __pyx_r = 0;
  __pyx_L0:;
  __Pyx_XDECREF(__pyx_v_port);
  __Pyx_XDECREF(__pyx_v_byte_str);
  __Pyx_XGIVEREF(__pyx_r);
  __Pyx_RefNannyFinishContext();
  return __pyx_r;
}

/* "src/python/grpcio/grpc/_cython/_cygrpc/grpc_gevent.pyx.pxi":47
 *   return byte_str, port
 * 
 * cdef sockaddr_to_tuple(const grpc_sockaddr* address, size_t length):             # <<<<<<<<<<<<<<
 *   cdef grpc_resolved_address c_addr
 *   string.memcpy(<void*>c_addr.addr, <void*> address, length)
 */

static PyObject *__pyx_f_4grpc_7_cython_6cygrpc_sockaddr_to_tuple(grpc_sockaddr const *__pyx_v_address, size_t __pyx_v_length) {
  grpc_resolved_address __pyx_v_c_addr;
  PyObject *__pyx_r = NULL;
  __Pyx_RefNannyDeclarations
  PyObject *__pyx_t_1 = NULL;
  __Pyx_RefNannySetupContext("sockaddr_to_tuple", 0);

  /* "src/python/grpcio/grpc/_cython/_cygrpc/grpc_gevent.pyx.pxi":49
 * cdef sockaddr_to_tuple(const grpc_sockaddr* address, size_t length):
 *   cdef grpc_resolved_address c_addr
 *   string.memcpy(<void*>c_addr.addr, <void*> address, length)             # <<<<<<<<<<<<<<
 *   c_addr.len = length
 *   return resolved_addr_to_tuple(&c_addr)
 */
  (void)(memcpy(((void *)__pyx_v_c_addr.addr), ((void *)__pyx_v_address), __pyx_v_length));

  /* "src/python/grpcio/grpc/_cython/_cygrpc/grpc_gevent.pyx.pxi":50
 *   cdef grpc_resolved_address c_addr
 *   string.memcpy(<void*>c_addr.addr, <void*> address, length)
 *   c_addr.len = length             # <<<<<<<<<<<<<<
 *   return resolved_addr_to_tuple(&c_addr)
 * 
 */
  __pyx_v_c_addr.len = __pyx_v_length;

  /* "src/python/grpcio/grpc/_cython/_cygrpc/grpc_gevent.pyx.pxi":51
 *   string.memcpy(<void*>c_addr.addr, <void*> address, length)
 *   c_addr.len = length
 *   return resolved_addr_to_tuple(&c_addr)             # <<<<<<<<<<<<<<
 * 
 * cdef sockaddr_is_ipv4(const grpc_sockaddr* address, size_t length):
 */
  __Pyx_XDECREF(__pyx_r);
  __pyx_t_1 = __pyx_f_4grpc_7_cython_6cygrpc_resolved_addr_to_tuple((&__pyx_v_c_addr)); if (unlikely(!__pyx_t_1)) __PYX_ERR(6, 51, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_1);
  __pyx_r = __pyx_t_1;
  __pyx_t_1 = 0;
  goto __pyx_L0;

  /* "src/python/grpcio/grpc/_cython/_cygrpc/grpc_gevent.pyx.pxi":47
 *   return byte_str, port
 * 
 * cdef sockaddr_to_tuple(const grpc_sockaddr* address, size_t length):             # <<<<<<<<<<<<<<
 *   cdef grpc_resolved_address c_addr
 *   string.memcpy(<void*>c_addr.addr, <void*> address, length)
 */

  /* function exit code */
  __pyx_L1_error:;
  __Pyx_XDECREF(__pyx_t_1);
  __Pyx_AddTraceback("grpc._cython.cygrpc.sockaddr_to_tuple", __pyx_clineno, __pyx_lineno, __pyx_filename);
  __pyx_r = 0;
  __pyx_L0:;
  __Pyx_XGIVEREF(__pyx_r);
  __Pyx_RefNannyFinishContext();
  return __pyx_r;
}

/* "src/python/grpcio/grpc/_cython/_cygrpc/grpc_gevent.pyx.pxi":53
 *   return resolved_addr_to_tuple(&c_addr)
 * 
 * cdef sockaddr_is_ipv4(const grpc_sockaddr* address, size_t length):             # <<<<<<<<<<<<<<
 *   cdef grpc_resolved_address c_addr
 *   string.memcpy(<void*>c_addr.addr, <void*> address, length)
 */

static PyObject *__pyx_f_4grpc_7_cython_6cygrpc_sockaddr_is_ipv4(grpc_sockaddr const *__pyx_v_address, size_t __pyx_v_length) {
  grpc_resolved_address __pyx_v_c_addr;
  PyObject *__pyx_r = NULL;
  __Pyx_RefNannyDeclarations
  PyObject *__pyx_t_1 = NULL;
  PyObject *__pyx_t_2 = NULL;
  __Pyx_RefNannySetupContext("sockaddr_is_ipv4", 0);

  /* "src/python/grpcio/grpc/_cython/_cygrpc/grpc_gevent.pyx.pxi":55
 * cdef sockaddr_is_ipv4(const grpc_sockaddr* address, size_t length):
 *   cdef grpc_resolved_address c_addr
 *   string.memcpy(<void*>c_addr.addr, <void*> address, length)             # <<<<<<<<<<<<<<
 *   c_addr.len = length
 *   return grpc_sockaddr_get_uri_scheme(&c_addr) == b'ipv4'
 */
  (void)(memcpy(((void *)__pyx_v_c_addr.addr), ((void *)__pyx_v_address), __pyx_v_length));

  /* "src/python/grpcio/grpc/_cython/_cygrpc/grpc_gevent.pyx.pxi":56
 *   cdef grpc_resolved_address c_addr
 *   string.memcpy(<void*>c_addr.addr, <void*> address, length)
 *   c_addr.len = length             # <<<<<<<<<<<<<<
 *   return grpc_sockaddr_get_uri_scheme(&c_addr) == b'ipv4'
 * 
 */
  __pyx_v_c_addr.len = __pyx_v_length;

  /* "src/python/grpcio/grpc/_cython/_cygrpc/grpc_gevent.pyx.pxi":57
 *   string.memcpy(<void*>c_addr.addr, <void*> address, length)
 *   c_addr.len = length
 *   return grpc_sockaddr_get_uri_scheme(&c_addr) == b'ipv4'             # <<<<<<<<<<<<<<
 * 
 * cdef grpc_resolved_addresses* tuples_to_resolvaddr(tups) except *:
 */
  __Pyx_XDECREF(__pyx_r);
  __pyx_t_1 = __Pyx_PyBytes_FromString(grpc_sockaddr_get_uri_scheme((&__pyx_v_c_addr))); if (unlikely(!__pyx_t_1)) __PYX_ERR(6, 57, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_1);
  __pyx_t_2 = PyObject_RichCompare(__pyx_t_1, __pyx_n_b_ipv4, Py_EQ); __Pyx_XGOTREF(__pyx_t_2); if (unlikely(!__pyx_t_2)) __PYX_ERR(6, 57, __pyx_L1_error)
  __Pyx_DECREF(__pyx_t_1); __pyx_t_1 = 0;
  __pyx_r = __pyx_t_2;
  __pyx_t_2 = 0;
  goto __pyx_L0;

  /* "src/python/grpcio/grpc/_cython/_cygrpc/grpc_gevent.pyx.pxi":53
 *   return resolved_addr_to_tuple(&c_addr)
 * 
 * cdef sockaddr_is_ipv4(const grpc_sockaddr* address, size_t length):             # <<<<<<<<<<<<<<
 *   cdef grpc_resolved_address c_addr
 *   string.memcpy(<void*>c_addr.addr, <void*> address, length)
 */

  /* function exit code */
  __pyx_L1_error:;
  __Pyx_XDECREF(__pyx_t_1);
  __Pyx_XDECREF(__pyx_t_2);
  __Pyx_AddTraceback("grpc._cython.cygrpc.sockaddr_is_ipv4", __pyx_clineno, __pyx_lineno, __pyx_filename);
  __pyx_r = 0;
  __pyx_L0:;
  __Pyx_XGIVEREF(__pyx_r);
  __Pyx_RefNannyFinishContext();
  return __pyx_r;
}
static PyObject *__pyx_gb_4grpc_7_cython_6cygrpc_20tuples_to_resolvaddr_2generator1(__pyx_CoroutineObject *__pyx_generator, CYTHON_UNUSED PyThreadState *__pyx_tstate, PyObject *__pyx_sent_value); /* proto */

/* "src/python/grpcio/grpc/_cython/_cygrpc/grpc_gevent.pyx.pxi":61
 * cdef grpc_resolved_addresses* tuples_to_resolvaddr(tups) except *:
 *   cdef grpc_resolved_addresses* addresses
 *   tups_set = set((tup[4][0], tup[4][1]) for tup in tups)             # <<<<<<<<<<<<<<
 *   addresses = <grpc_resolved_addresses*> malloc(sizeof(grpc_resolved_addresses))
 *   addresses.naddrs = len(tups_set)
 */

static PyObject *__pyx_pf_4grpc_7_cython_6cygrpc_20tuples_to_resolvaddr_genexpr(PyObject *__pyx_self) {
  struct __pyx_obj_4grpc_7_cython_6cygrpc___pyx_scope_struct_8_genexpr *__pyx_cur_scope;
  PyObject *__pyx_r = NULL;
  __Pyx_RefNannyDeclarations
  __Pyx_RefNannySetupContext("genexpr", 0);
  __pyx_cur_scope = (struct __pyx_obj_4grpc_7_cython_6cygrpc___pyx_scope_struct_8_genexpr *)__pyx_tp_new_4grpc_7_cython_6cygrpc___pyx_scope_struct_8_genexpr(__pyx_ptype_4grpc_7_cython_6cygrpc___pyx_scope_struct_8_genexpr, __pyx_empty_tuple, NULL);
  if (unlikely(!__pyx_cur_scope)) {
    __pyx_cur_scope = ((struct __pyx_obj_4grpc_7_cython_6cygrpc___pyx_scope_struct_8_genexpr *)Py_None);
    __Pyx_INCREF(Py_None);
    __PYX_ERR(6, 61, __pyx_L1_error)
  } else {
    __Pyx_GOTREF(__pyx_cur_scope);
  }
  __pyx_cur_scope->__pyx_outer_scope = (struct __pyx_obj_4grpc_7_cython_6cygrpc___pyx_scope_struct_7___pyx_f_4grpc_7_cython_6cygrpc_tuples_to_resolvaddr *) __pyx_self;
  __Pyx_INCREF(((PyObject *)__pyx_cur_scope->__pyx_outer_scope));
  __Pyx_GIVEREF(__pyx_cur_scope->__pyx_outer_scope);
  {
    __pyx_CoroutineObject *gen = __Pyx_Generator_New((__pyx_coroutine_body_t) __pyx_gb_4grpc_7_cython_6cygrpc_20tuples_to_resolvaddr_2generator1, NULL, (PyObject *) __pyx_cur_scope, __pyx_n_s_genexpr, __pyx_n_s_tuples_to_resolvaddr_locals_gene, __pyx_n_s_grpc__cython_cygrpc); if (unlikely(!gen)) __PYX_ERR(6, 61, __pyx_L1_error)
    __Pyx_DECREF(__pyx_cur_scope);
    __Pyx_RefNannyFinishContext();
    return (PyObject *) gen;
  }

  /* function exit code */
  __pyx_L1_error:;
  __Pyx_AddTraceback("grpc._cython.cygrpc.tuples_to_resolvaddr.genexpr", __pyx_clineno, __pyx_lineno, __pyx_filename);
  __pyx_r = NULL;
  __Pyx_DECREF(((PyObject *)__pyx_cur_scope));
  __Pyx_XGIVEREF(__pyx_r);
  __Pyx_RefNannyFinishContext();
  return __pyx_r;
}

static PyObject *__pyx_gb_4grpc_7_cython_6cygrpc_20tuples_to_resolvaddr_2generator1(__pyx_CoroutineObject *__pyx_generator, CYTHON_UNUSED PyThreadState *__pyx_tstate, PyObject *__pyx_sent_value) /* generator body */
{
  struct __pyx_obj_4grpc_7_cython_6cygrpc___pyx_scope_struct_8_genexpr *__pyx_cur_scope = ((struct __pyx_obj_4grpc_7_cython_6cygrpc___pyx_scope_struct_8_genexpr *)__pyx_generator->closure);
  PyObject *__pyx_r = NULL;
  PyObject *__pyx_t_1 = NULL;
  Py_ssize_t __pyx_t_2;
  PyObject *(*__pyx_t_3)(PyObject *);
  PyObject *__pyx_t_4 = NULL;
  PyObject *__pyx_t_5 = NULL;
  PyObject *__pyx_t_6 = NULL;
  __Pyx_RefNannyDeclarations
  __Pyx_RefNannySetupContext("genexpr", 0);
  switch (__pyx_generator->resume_label) {
    case 0: goto __pyx_L3_first_run;
    default: /* CPython raises the right error here */
    __Pyx_RefNannyFinishContext();
    return NULL;
  }
  __pyx_L3_first_run:;
  if (unlikely(!__pyx_sent_value)) __PYX_ERR(6, 61, __pyx_L1_error)
  __pyx_r = PySet_New(NULL); if (unlikely(!__pyx_r)) __PYX_ERR(6, 61, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_r);
  if (unlikely(!__pyx_cur_scope->__pyx_outer_scope->__pyx_v_tups)) { __Pyx_RaiseClosureNameError("tups"); __PYX_ERR(6, 61, __pyx_L1_error) }
  if (likely(PyList_CheckExact(__pyx_cur_scope->__pyx_outer_scope->__pyx_v_tups)) || PyTuple_CheckExact(__pyx_cur_scope->__pyx_outer_scope->__pyx_v_tups)) {
    __pyx_t_1 = __pyx_cur_scope->__pyx_outer_scope->__pyx_v_tups; __Pyx_INCREF(__pyx_t_1); __pyx_t_2 = 0;
    __pyx_t_3 = NULL;
  } else {
    __pyx_t_2 = -1; __pyx_t_1 = PyObject_GetIter(__pyx_cur_scope->__pyx_outer_scope->__pyx_v_tups); if (unlikely(!__pyx_t_1)) __PYX_ERR(6, 61, __pyx_L1_error)
    __Pyx_GOTREF(__pyx_t_1);
    __pyx_t_3 = Py_TYPE(__pyx_t_1)->tp_iternext; if (unlikely(!__pyx_t_3)) __PYX_ERR(6, 61, __pyx_L1_error)
  }
  for (;;) {
    if (likely(!__pyx_t_3)) {
      if (likely(PyList_CheckExact(__pyx_t_1))) {
        if (__pyx_t_2 >= PyList_GET_SIZE(__pyx_t_1)) break;
        #if CYTHON_ASSUME_SAFE_MACROS && !CYTHON_AVOID_BORROWED_REFS
        __pyx_t_4 = PyList_GET_ITEM(__pyx_t_1, __pyx_t_2); __Pyx_INCREF(__pyx_t_4); __pyx_t_2++; if (unlikely(0 < 0)) __PYX_ERR(6, 61, __pyx_L1_error)
        #else
        __pyx_t_4 = PySequence_ITEM(__pyx_t_1, __pyx_t_2); __pyx_t_2++; if (unlikely(!__pyx_t_4)) __PYX_ERR(6, 61, __pyx_L1_error)
        __Pyx_GOTREF(__pyx_t_4);
        #endif
      } else {
        if (__pyx_t_2 >= PyTuple_GET_SIZE(__pyx_t_1)) break;
        #if CYTHON_ASSUME_SAFE_MACROS && !CYTHON_AVOID_BORROWED_REFS
        __pyx_t_4 = PyTuple_GET_ITEM(__pyx_t_1, __pyx_t_2); __Pyx_INCREF(__pyx_t_4); __pyx_t_2++; if (unlikely(0 < 0)) __PYX_ERR(6, 61, __pyx_L1_error)
        #else
        __pyx_t_4 = PySequence_ITEM(__pyx_t_1, __pyx_t_2); __pyx_t_2++; if (unlikely(!__pyx_t_4)) __PYX_ERR(6, 61, __pyx_L1_error)
        __Pyx_GOTREF(__pyx_t_4);
        #endif
      }
    } else {
      __pyx_t_4 = __pyx_t_3(__pyx_t_1);
      if (unlikely(!__pyx_t_4)) {
        PyObject* exc_type = PyErr_Occurred();
        if (exc_type) {
          if (likely(__Pyx_PyErr_GivenExceptionMatches(exc_type, PyExc_StopIteration))) PyErr_Clear();
          else __PYX_ERR(6, 61, __pyx_L1_error)
        }
        break;
      }
      __Pyx_GOTREF(__pyx_t_4);
    }
    __Pyx_XGOTREF(__pyx_cur_scope->__pyx_v_tup);
    __Pyx_XDECREF_SET(__pyx_cur_scope->__pyx_v_tup, __pyx_t_4);
    __Pyx_GIVEREF(__pyx_t_4);
    __pyx_t_4 = 0;
    __pyx_t_4 = __Pyx_GetItemInt(__pyx_cur_scope->__pyx_v_tup, 4, long, 1, __Pyx_PyInt_From_long, 0, 0, 1); if (unlikely(!__pyx_t_4)) __PYX_ERR(6, 61, __pyx_L1_error)
    __Pyx_GOTREF(__pyx_t_4);
    __pyx_t_5 = __Pyx_GetItemInt(__pyx_t_4, 0, long, 1, __Pyx_PyInt_From_long, 0, 0, 1); if (unlikely(!__pyx_t_5)) __PYX_ERR(6, 61, __pyx_L1_error)
    __Pyx_GOTREF(__pyx_t_5);
    __Pyx_DECREF(__pyx_t_4); __pyx_t_4 = 0;
    __pyx_t_4 = __Pyx_GetItemInt(__pyx_cur_scope->__pyx_v_tup, 4, long, 1, __Pyx_PyInt_From_long, 0, 0, 1); if (unlikely(!__pyx_t_4)) __PYX_ERR(6, 61, __pyx_L1_error)
    __Pyx_GOTREF(__pyx_t_4);
    __pyx_t_6 = __Pyx_GetItemInt(__pyx_t_4, 1, long, 1, __Pyx_PyInt_From_long, 0, 0, 1); if (unlikely(!__pyx_t_6)) __PYX_ERR(6, 61, __pyx_L1_error)
    __Pyx_GOTREF(__pyx_t_6);
    __Pyx_DECREF(__pyx_t_4); __pyx_t_4 = 0;
    __pyx_t_4 = PyTuple_New(2); if (unlikely(!__pyx_t_4)) __PYX_ERR(6, 61, __pyx_L1_error)
    __Pyx_GOTREF(__pyx_t_4);
    __Pyx_GIVEREF(__pyx_t_5);
    PyTuple_SET_ITEM(__pyx_t_4, 0, __pyx_t_5);
    __Pyx_GIVEREF(__pyx_t_6);
    PyTuple_SET_ITEM(__pyx_t_4, 1, __pyx_t_6);
    __pyx_t_5 = 0;
    __pyx_t_6 = 0;
    if (unlikely(PySet_Add(__pyx_r, (PyObject*)__pyx_t_4))) __PYX_ERR(6, 61, __pyx_L1_error)
    __Pyx_DECREF(__pyx_t_4); __pyx_t_4 = 0;
  }
  __Pyx_DECREF(__pyx_t_1); __pyx_t_1 = 0;
  CYTHON_MAYBE_UNUSED_VAR(__pyx_cur_scope);

  /* function exit code */
  goto __pyx_L0;
  __pyx_L1_error:;
  __Pyx_XDECREF(__pyx_r); __pyx_r = 0;
  __Pyx_XDECREF(__pyx_t_1);
  __Pyx_XDECREF(__pyx_t_4);
  __Pyx_XDECREF(__pyx_t_5);
  __Pyx_XDECREF(__pyx_t_6);
  __Pyx_AddTraceback("genexpr", __pyx_clineno, __pyx_lineno, __pyx_filename);
  __pyx_L0:;
  __Pyx_XGIVEREF(__pyx_r);
  __Pyx_Coroutine_ResetAndClearException(__pyx_generator);
  __pyx_generator->resume_label = -1;
  __Pyx_Coroutine_clear((PyObject*)__pyx_generator);
  __Pyx_RefNannyFinishContext();
  return __pyx_r;
}

/* "src/python/grpcio/grpc/_cython/_cygrpc/grpc_gevent.pyx.pxi":59
 *   return grpc_sockaddr_get_uri_scheme(&c_addr) == b'ipv4'
 * 
 * cdef grpc_resolved_addresses* tuples_to_resolvaddr(tups) except *:             # <<<<<<<<<<<<<<
 *   cdef grpc_resolved_addresses* addresses
 *   tups_set = set((tup[4][0], tup[4][1]) for tup in tups)
 */

static grpc_resolved_addresses *__pyx_f_4grpc_7_cython_6cygrpc_tuples_to_resolvaddr(PyObject *__pyx_v_tups) {
  struct __pyx_obj_4grpc_7_cython_6cygrpc___pyx_scope_struct_7___pyx_f_4grpc_7_cython_6cygrpc_tuples_to_resolvaddr *__pyx_cur_scope;
  grpc_resolved_addresses *__pyx_v_addresses;
  PyObject *__pyx_v_tups_set = NULL;
  PyObject *__pyx_v_i = NULL;
  PyObject *__pyx_v_tup = NULL;
  PyObject *__pyx_v_hostname = NULL;
  grpc_resolved_addresses *__pyx_r;
  __Pyx_RefNannyDeclarations
  PyObject *__pyx_t_1 = NULL;
  PyObject *__pyx_t_2 = NULL;
  Py_ssize_t __pyx_t_3;
  Py_ssize_t __pyx_t_4;
  int __pyx_t_5;
  PyObject *__pyx_t_6 = NULL;
  int __pyx_t_7;
  Py_ssize_t __pyx_t_8;
  char *__pyx_t_9;
  __Pyx_RefNannySetupContext("tuples_to_resolvaddr", 0);
  __pyx_cur_scope = (struct __pyx_obj_4grpc_7_cython_6cygrpc___pyx_scope_struct_7___pyx_f_4grpc_7_cython_6cygrpc_tuples_to_resolvaddr *)__pyx_tp_new_4grpc_7_cython_6cygrpc___pyx_scope_struct_7___pyx_f_4grpc_7_cython_6cygrpc_tuples_to_resolvaddr(__pyx_ptype_4grpc_7_cython_6cygrpc___pyx_scope_struct_7___pyx_f_4grpc_7_cython_6cygrpc_tuples_to_resolvaddr, __pyx_empty_tuple, NULL);
  if (unlikely(!__pyx_cur_scope)) {
    __pyx_cur_scope = ((struct __pyx_obj_4grpc_7_cython_6cygrpc___pyx_scope_struct_7___pyx_f_4grpc_7_cython_6cygrpc_tuples_to_resolvaddr *)Py_None);
    __Pyx_INCREF(Py_None);
    __PYX_ERR(6, 59, __pyx_L1_error)
  } else {
    __Pyx_GOTREF(__pyx_cur_scope);
  }
  __pyx_cur_scope->__pyx_v_tups = __pyx_v_tups;
  __Pyx_INCREF(__pyx_cur_scope->__pyx_v_tups);
  __Pyx_GIVEREF(__pyx_cur_scope->__pyx_v_tups);

  /* "src/python/grpcio/grpc/_cython/_cygrpc/grpc_gevent.pyx.pxi":61
 * cdef grpc_resolved_addresses* tuples_to_resolvaddr(tups) except *:
 *   cdef grpc_resolved_addresses* addresses
 *   tups_set = set((tup[4][0], tup[4][1]) for tup in tups)             # <<<<<<<<<<<<<<
 *   addresses = <grpc_resolved_addresses*> malloc(sizeof(grpc_resolved_addresses))
 *   addresses.naddrs = len(tups_set)
 */
  __pyx_t_1 = __pyx_pf_4grpc_7_cython_6cygrpc_20tuples_to_resolvaddr_genexpr(((PyObject*)__pyx_cur_scope)); if (unlikely(!__pyx_t_1)) __PYX_ERR(6, 61, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_1);
  __pyx_t_2 = __Pyx_Generator_Next(__pyx_t_1); if (unlikely(!__pyx_t_2)) __PYX_ERR(6, 61, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_2);
  __Pyx_DECREF(__pyx_t_1); __pyx_t_1 = 0;
  __pyx_v_tups_set = ((PyObject*)__pyx_t_2);
  __pyx_t_2 = 0;

  /* "src/python/grpcio/grpc/_cython/_cygrpc/grpc_gevent.pyx.pxi":62
 *   cdef grpc_resolved_addresses* addresses
 *   tups_set = set((tup[4][0], tup[4][1]) for tup in tups)
 *   addresses = <grpc_resolved_addresses*> malloc(sizeof(grpc_resolved_addresses))             # <<<<<<<<<<<<<<
 *   addresses.naddrs = len(tups_set)
 *   addresses.addrs = <grpc_resolved_address*> malloc(sizeof(grpc_resolved_address) * len(tups_set))
 */
  __pyx_v_addresses = ((grpc_resolved_addresses *)malloc((sizeof(grpc_resolved_addresses))));

  /* "src/python/grpcio/grpc/_cython/_cygrpc/grpc_gevent.pyx.pxi":63
 *   tups_set = set((tup[4][0], tup[4][1]) for tup in tups)
 *   addresses = <grpc_resolved_addresses*> malloc(sizeof(grpc_resolved_addresses))
 *   addresses.naddrs = len(tups_set)             # <<<<<<<<<<<<<<
 *   addresses.addrs = <grpc_resolved_address*> malloc(sizeof(grpc_resolved_address) * len(tups_set))
 *   i = 0
 */
  if (unlikely(__pyx_v_tups_set == Py_None)) {
    PyErr_SetString(PyExc_TypeError, "object of type 'NoneType' has no len()");
    __PYX_ERR(6, 63, __pyx_L1_error)
  }
  __pyx_t_3 = PySet_GET_SIZE(__pyx_v_tups_set); if (unlikely(__pyx_t_3 == ((Py_ssize_t)-1))) __PYX_ERR(6, 63, __pyx_L1_error)
  __pyx_v_addresses->naddrs = __pyx_t_3;

  /* "src/python/grpcio/grpc/_cython/_cygrpc/grpc_gevent.pyx.pxi":64
 *   addresses = <grpc_resolved_addresses*> malloc(sizeof(grpc_resolved_addresses))
 *   addresses.naddrs = len(tups_set)
 *   addresses.addrs = <grpc_resolved_address*> malloc(sizeof(grpc_resolved_address) * len(tups_set))             # <<<<<<<<<<<<<<
 *   i = 0
 *   for tup in set(tups_set):
 */
  if (unlikely(__pyx_v_tups_set == Py_None)) {
    PyErr_SetString(PyExc_TypeError, "object of type 'NoneType' has no len()");
    __PYX_ERR(6, 64, __pyx_L1_error)
  }
  __pyx_t_3 = PySet_GET_SIZE(__pyx_v_tups_set); if (unlikely(__pyx_t_3 == ((Py_ssize_t)-1))) __PYX_ERR(6, 64, __pyx_L1_error)
  __pyx_v_addresses->addrs = ((grpc_resolved_address *)malloc(((sizeof(grpc_resolved_address)) * __pyx_t_3)));

  /* "src/python/grpcio/grpc/_cython/_cygrpc/grpc_gevent.pyx.pxi":65
 *   addresses.naddrs = len(tups_set)
 *   addresses.addrs = <grpc_resolved_address*> malloc(sizeof(grpc_resolved_address) * len(tups_set))
 *   i = 0             # <<<<<<<<<<<<<<
 *   for tup in set(tups_set):
 *     hostname = str_to_bytes(tup[0])
 */
  __Pyx_INCREF(__pyx_int_0);
  __pyx_v_i = __pyx_int_0;

  /* "src/python/grpcio/grpc/_cython/_cygrpc/grpc_gevent.pyx.pxi":66
 *   addresses.addrs = <grpc_resolved_address*> malloc(sizeof(grpc_resolved_address) * len(tups_set))
 *   i = 0
 *   for tup in set(tups_set):             # <<<<<<<<<<<<<<
 *     hostname = str_to_bytes(tup[0])
 *     grpc_string_to_sockaddr(&addresses.addrs[i], hostname, tup[1])
 */
  __pyx_t_3 = 0;
  __pyx_t_1 = PySet_New(__pyx_v_tups_set); if (unlikely(!__pyx_t_1)) __PYX_ERR(6, 66, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_1);
  __pyx_t_6 = __Pyx_set_iterator(__pyx_t_1, 1, (&__pyx_t_4), (&__pyx_t_5)); if (unlikely(!__pyx_t_6)) __PYX_ERR(6, 66, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_6);
  __Pyx_DECREF(__pyx_t_1); __pyx_t_1 = 0;
  __Pyx_XDECREF(__pyx_t_2);
  __pyx_t_2 = __pyx_t_6;
  __pyx_t_6 = 0;
  while (1) {
    __pyx_t_7 = __Pyx_set_iter_next(__pyx_t_2, __pyx_t_4, &__pyx_t_3, &__pyx_t_6, __pyx_t_5);
    if (unlikely(__pyx_t_7 == 0)) break;
    if (unlikely(__pyx_t_7 == -1)) __PYX_ERR(6, 66, __pyx_L1_error)
    __Pyx_GOTREF(__pyx_t_6);
    __Pyx_XDECREF_SET(__pyx_v_tup, __pyx_t_6);
    __pyx_t_6 = 0;

    /* "src/python/grpcio/grpc/_cython/_cygrpc/grpc_gevent.pyx.pxi":67
 *   i = 0
 *   for tup in set(tups_set):
 *     hostname = str_to_bytes(tup[0])             # <<<<<<<<<<<<<<
 *     grpc_string_to_sockaddr(&addresses.addrs[i], hostname, tup[1])
 *     i += 1
 */
    __pyx_t_6 = __Pyx_GetItemInt(__pyx_v_tup, 0, long, 1, __Pyx_PyInt_From_long, 0, 0, 1); if (unlikely(!__pyx_t_6)) __PYX_ERR(6, 67, __pyx_L1_error)
    __Pyx_GOTREF(__pyx_t_6);
    __pyx_t_1 = __pyx_f_4grpc_7_cython_6cygrpc_str_to_bytes(__pyx_t_6); if (unlikely(!__pyx_t_1)) __PYX_ERR(6, 67, __pyx_L1_error)
    __Pyx_GOTREF(__pyx_t_1);
    __Pyx_DECREF(__pyx_t_6); __pyx_t_6 = 0;
    __Pyx_XDECREF_SET(__pyx_v_hostname, ((PyObject*)__pyx_t_1));
    __pyx_t_1 = 0;

    /* "src/python/grpcio/grpc/_cython/_cygrpc/grpc_gevent.pyx.pxi":68
 *   for tup in set(tups_set):
 *     hostname = str_to_bytes(tup[0])
 *     grpc_string_to_sockaddr(&addresses.addrs[i], hostname, tup[1])             # <<<<<<<<<<<<<<
 *     i += 1
 *   return addresses
 */
    __pyx_t_8 = __Pyx_PyIndex_AsSsize_t(__pyx_v_i); if (unlikely((__pyx_t_8 == (Py_ssize_t)-1) && PyErr_Occurred())) __PYX_ERR(6, 68, __pyx_L1_error)
    if (unlikely(__pyx_v_hostname == Py_None)) {
      PyErr_SetString(PyExc_TypeError, "expected bytes, NoneType found");
      __PYX_ERR(6, 68, __pyx_L1_error)
    }
    __pyx_t_9 = __Pyx_PyBytes_AsWritableString(__pyx_v_hostname); if (unlikely((!__pyx_t_9) && PyErr_Occurred())) __PYX_ERR(6, 68, __pyx_L1_error)
    __pyx_t_1 = __Pyx_GetItemInt(__pyx_v_tup, 1, long, 1, __Pyx_PyInt_From_long, 0, 0, 1); if (unlikely(!__pyx_t_1)) __PYX_ERR(6, 68, __pyx_L1_error)
    __Pyx_GOTREF(__pyx_t_1);
    __pyx_t_7 = __Pyx_PyInt_As_int(__pyx_t_1); if (unlikely((__pyx_t_7 == (int)-1) && PyErr_Occurred())) __PYX_ERR(6, 68, __pyx_L1_error)
    __Pyx_DECREF(__pyx_t_1); __pyx_t_1 = 0;
    grpc_string_to_sockaddr((&(__pyx_v_addresses->addrs[__pyx_t_8])), __pyx_t_9, __pyx_t_7);

    /* "src/python/grpcio/grpc/_cython/_cygrpc/grpc_gevent.pyx.pxi":69
 *     hostname = str_to_bytes(tup[0])
 *     grpc_string_to_sockaddr(&addresses.addrs[i], hostname, tup[1])
 *     i += 1             # <<<<<<<<<<<<<<
 *   return addresses
 * 
 */
    __pyx_t_1 = __Pyx_PyInt_AddObjC(__pyx_v_i, __pyx_int_1, 1, 1); if (unlikely(!__pyx_t_1)) __PYX_ERR(6, 69, __pyx_L1_error)
    __Pyx_GOTREF(__pyx_t_1);
    __Pyx_DECREF_SET(__pyx_v_i, __pyx_t_1);
    __pyx_t_1 = 0;
  }
  __Pyx_DECREF(__pyx_t_2); __pyx_t_2 = 0;

  /* "src/python/grpcio/grpc/_cython/_cygrpc/grpc_gevent.pyx.pxi":70
 *     grpc_string_to_sockaddr(&addresses.addrs[i], hostname, tup[1])
 *     i += 1
 *   return addresses             # <<<<<<<<<<<<<<
 * 
 * def _spawn_greenlet(*args):
 */
  __pyx_r = __pyx_v_addresses;
  goto __pyx_L0;

  /* "src/python/grpcio/grpc/_cython/_cygrpc/grpc_gevent.pyx.pxi":59
 *   return grpc_sockaddr_get_uri_scheme(&c_addr) == b'ipv4'
 * 
 * cdef grpc_resolved_addresses* tuples_to_resolvaddr(tups) except *:             # <<<<<<<<<<<<<<
 *   cdef grpc_resolved_addresses* addresses
 *   tups_set = set((tup[4][0], tup[4][1]) for tup in tups)
 */

  /* function exit code */
  __pyx_L1_error:;
  __Pyx_XDECREF(__pyx_t_1);
  __Pyx_XDECREF(__pyx_t_2);
  __Pyx_XDECREF(__pyx_t_6);
  __Pyx_AddTraceback("grpc._cython.cygrpc.tuples_to_resolvaddr", __pyx_clineno, __pyx_lineno, __pyx_filename);
  __pyx_r = 0;
  __pyx_L0:;
  __Pyx_XDECREF(__pyx_v_tups_set);
  __Pyx_XDECREF(__pyx_v_i);
  __Pyx_XDECREF(__pyx_v_tup);
  __Pyx_XDECREF(__pyx_v_hostname);
  __Pyx_DECREF(((PyObject *)__pyx_cur_scope));
  __Pyx_RefNannyFinishContext();
  return __pyx_r;
}

/* "src/python/grpcio/grpc/_cython/_cygrpc/grpc_gevent.pyx.pxi":72
 *   return addresses
 * 
 * def _spawn_greenlet(*args):             # <<<<<<<<<<<<<<
 *   greenlet = g_pool.spawn(*args)
 * 
 */

/* Python wrapper */
static PyObject *__pyx_pw_4grpc_7_cython_6cygrpc_45_spawn_greenlet(PyObject *__pyx_self, PyObject *__pyx_args, PyObject *__pyx_kwds); /*proto*/
static PyMethodDef __pyx_mdef_4grpc_7_cython_6cygrpc_45_spawn_greenlet = {"_spawn_greenlet", (PyCFunction)__pyx_pw_4grpc_7_cython_6cygrpc_45_spawn_greenlet, METH_VARARGS|METH_KEYWORDS, 0};
static PyObject *__pyx_pw_4grpc_7_cython_6cygrpc_45_spawn_greenlet(PyObject *__pyx_self, PyObject *__pyx_args, PyObject *__pyx_kwds) {
  PyObject *__pyx_v_args = 0;
  PyObject *__pyx_r = 0;
  __Pyx_RefNannyDeclarations
  __Pyx_RefNannySetupContext("_spawn_greenlet (wrapper)", 0);
  if (unlikely(__pyx_kwds) && unlikely(PyDict_Size(__pyx_kwds) > 0) && unlikely(!__Pyx_CheckKeywordStrings(__pyx_kwds, "_spawn_greenlet", 0))) return NULL;
  __Pyx_INCREF(__pyx_args);
  __pyx_v_args = __pyx_args;
  __pyx_r = __pyx_pf_4grpc_7_cython_6cygrpc_44_spawn_greenlet(__pyx_self, __pyx_v_args);

  /* function exit code */
  __Pyx_XDECREF(__pyx_v_args);
  __Pyx_RefNannyFinishContext();
  return __pyx_r;
}

static PyObject *__pyx_pf_4grpc_7_cython_6cygrpc_44_spawn_greenlet(CYTHON_UNUSED PyObject *__pyx_self, PyObject *__pyx_v_args) {
  CYTHON_UNUSED PyObject *__pyx_v_greenlet = NULL;
  PyObject *__pyx_r = NULL;
  __Pyx_RefNannyDeclarations
  PyObject *__pyx_t_1 = NULL;
  PyObject *__pyx_t_2 = NULL;
  __Pyx_RefNannySetupContext("_spawn_greenlet", 0);

  /* "src/python/grpcio/grpc/_cython/_cygrpc/grpc_gevent.pyx.pxi":73
 * 
 * def _spawn_greenlet(*args):
 *   greenlet = g_pool.spawn(*args)             # <<<<<<<<<<<<<<
 * 
 * ###############################
 */
  __pyx_t_1 = __Pyx_GetModuleGlobalName(__pyx_n_s_g_pool); if (unlikely(!__pyx_t_1)) __PYX_ERR(6, 73, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_1);
  __pyx_t_2 = __Pyx_PyObject_GetAttrStr(__pyx_t_1, __pyx_n_s_spawn); if (unlikely(!__pyx_t_2)) __PYX_ERR(6, 73, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_2);
  __Pyx_DECREF(__pyx_t_1); __pyx_t_1 = 0;
  __pyx_t_1 = __Pyx_PyObject_Call(__pyx_t_2, __pyx_v_args, NULL); if (unlikely(!__pyx_t_1)) __PYX_ERR(6, 73, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_1);
  __Pyx_DECREF(__pyx_t_2); __pyx_t_2 = 0;
  __pyx_v_greenlet = __pyx_t_1;
  __pyx_t_1 = 0;

  /* "src/python/grpcio/grpc/_cython/_cygrpc/grpc_gevent.pyx.pxi":72
 *   return addresses
 * 
 * def _spawn_greenlet(*args):             # <<<<<<<<<<<<<<
 *   greenlet = g_pool.spawn(*args)
 * 
 */

  /* function exit code */
  __pyx_r = Py_None; __Pyx_INCREF(Py_None);
  goto __pyx_L0;
  __pyx_L1_error:;
  __Pyx_XDECREF(__pyx_t_1);
  __Pyx_XDECREF(__pyx_t_2);
  __Pyx_AddTraceback("grpc._cython.cygrpc._spawn_greenlet", __pyx_clineno, __pyx_lineno, __pyx_filename);
  __pyx_r = NULL;
  __pyx_L0:;
  __Pyx_XDECREF(__pyx_v_greenlet);
  __Pyx_XGIVEREF(__pyx_r);
  __Pyx_RefNannyFinishContext();
  return __pyx_r;
}

/* "src/python/grpcio/grpc/_cython/_cygrpc/grpc_gevent.pyx.pxi":80
 * 
 * cdef class SocketWrapper:
 *   def __cinit__(self):             # <<<<<<<<<<<<<<
 *     self.sockopts = []
 *     self.socket = None
 */

/* Python wrapper */
static int __pyx_pw_4grpc_7_cython_6cygrpc_13SocketWrapper_1__cinit__(PyObject *__pyx_v_self, PyObject *__pyx_args, PyObject *__pyx_kwds); /*proto*/
static int __pyx_pw_4grpc_7_cython_6cygrpc_13SocketWrapper_1__cinit__(PyObject *__pyx_v_self, PyObject *__pyx_args, PyObject *__pyx_kwds) {
  int __pyx_r;
  __Pyx_RefNannyDeclarations
  __Pyx_RefNannySetupContext("__cinit__ (wrapper)", 0);
  if (unlikely(PyTuple_GET_SIZE(__pyx_args) > 0)) {
    __Pyx_RaiseArgtupleInvalid("__cinit__", 1, 0, 0, PyTuple_GET_SIZE(__pyx_args)); return -1;}
  if (unlikely(__pyx_kwds) && unlikely(PyDict_Size(__pyx_kwds) > 0) && unlikely(!__Pyx_CheckKeywordStrings(__pyx_kwds, "__cinit__", 0))) return -1;
  __pyx_r = __pyx_pf_4grpc_7_cython_6cygrpc_13SocketWrapper___cinit__(((struct __pyx_obj_4grpc_7_cython_6cygrpc_SocketWrapper *)__pyx_v_self));

  /* function exit code */
  __Pyx_RefNannyFinishContext();
  return __pyx_r;
}

static int __pyx_pf_4grpc_7_cython_6cygrpc_13SocketWrapper___cinit__(struct __pyx_obj_4grpc_7_cython_6cygrpc_SocketWrapper *__pyx_v_self) {
  int __pyx_r;
  __Pyx_RefNannyDeclarations
  PyObject *__pyx_t_1 = NULL;
  __Pyx_RefNannySetupContext("__cinit__", 0);

  /* "src/python/grpcio/grpc/_cython/_cygrpc/grpc_gevent.pyx.pxi":81
 * cdef class SocketWrapper:
 *   def __cinit__(self):
 *     self.sockopts = []             # <<<<<<<<<<<<<<
 *     self.socket = None
 *     self.c_socket = NULL
 */
  __pyx_t_1 = PyList_New(0); if (unlikely(!__pyx_t_1)) __PYX_ERR(6, 81, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_1);
  __Pyx_GIVEREF(__pyx_t_1);
  __Pyx_GOTREF(__pyx_v_self->sockopts);
  __Pyx_DECREF(__pyx_v_self->sockopts);
  __pyx_v_self->sockopts = __pyx_t_1;
  __pyx_t_1 = 0;

  /* "src/python/grpcio/grpc/_cython/_cygrpc/grpc_gevent.pyx.pxi":82
 *   def __cinit__(self):
 *     self.sockopts = []
 *     self.socket = None             # <<<<<<<<<<<<<<
 *     self.c_socket = NULL
 *     self.c_buffer = NULL
 */
  __Pyx_INCREF(Py_None);
  __Pyx_GIVEREF(Py_None);
  __Pyx_GOTREF(__pyx_v_self->socket);
  __Pyx_DECREF(__pyx_v_self->socket);
  __pyx_v_self->socket = Py_None;

  /* "src/python/grpcio/grpc/_cython/_cygrpc/grpc_gevent.pyx.pxi":83
 *     self.sockopts = []
 *     self.socket = None
 *     self.c_socket = NULL             # <<<<<<<<<<<<<<
 *     self.c_buffer = NULL
 *     self.len = 0
 */
  __pyx_v_self->c_socket = NULL;

  /* "src/python/grpcio/grpc/_cython/_cygrpc/grpc_gevent.pyx.pxi":84
 *     self.socket = None
 *     self.c_socket = NULL
 *     self.c_buffer = NULL             # <<<<<<<<<<<<<<
 *     self.len = 0
 * 
 */
  __pyx_v_self->c_buffer = NULL;

  /* "src/python/grpcio/grpc/_cython/_cygrpc/grpc_gevent.pyx.pxi":85
 *     self.c_socket = NULL
 *     self.c_buffer = NULL
 *     self.len = 0             # <<<<<<<<<<<<<<
 * 
 * cdef grpc_error* socket_init(grpc_custom_socket* socket, int domain) except * with gil:
 */
  __pyx_v_self->len = 0;

  /* "src/python/grpcio/grpc/_cython/_cygrpc/grpc_gevent.pyx.pxi":80
 * 
 * cdef class SocketWrapper:
 *   def __cinit__(self):             # <<<<<<<<<<<<<<
 *     self.sockopts = []
 *     self.socket = None
 */

  /* function exit code */
  __pyx_r = 0;
  goto __pyx_L0;
  __pyx_L1_error:;
  __Pyx_XDECREF(__pyx_t_1);
  __Pyx_AddTraceback("grpc._cython.cygrpc.SocketWrapper.__cinit__", __pyx_clineno, __pyx_lineno, __pyx_filename);
  __pyx_r = -1;
  __pyx_L0:;
  __Pyx_RefNannyFinishContext();
  return __pyx_r;
}

/* "(tree fragment)":1
 * def __reduce_cython__(self):             # <<<<<<<<<<<<<<
 *     raise TypeError("no default __reduce__ due to non-trivial __cinit__")
 * def __setstate_cython__(self, __pyx_state):
 */

/* Python wrapper */
static PyObject *__pyx_pw_4grpc_7_cython_6cygrpc_13SocketWrapper_3__reduce_cython__(PyObject *__pyx_v_self, CYTHON_UNUSED PyObject *unused); /*proto*/
static PyObject *__pyx_pw_4grpc_7_cython_6cygrpc_13SocketWrapper_3__reduce_cython__(PyObject *__pyx_v_self, CYTHON_UNUSED PyObject *unused) {
  PyObject *__pyx_r = 0;
  __Pyx_RefNannyDeclarations
  __Pyx_RefNannySetupContext("__reduce_cython__ (wrapper)", 0);
  __pyx_r = __pyx_pf_4grpc_7_cython_6cygrpc_13SocketWrapper_2__reduce_cython__(((struct __pyx_obj_4grpc_7_cython_6cygrpc_SocketWrapper *)__pyx_v_self));

  /* function exit code */
  __Pyx_RefNannyFinishContext();
  return __pyx_r;
}

static PyObject *__pyx_pf_4grpc_7_cython_6cygrpc_13SocketWrapper_2__reduce_cython__(CYTHON_UNUSED struct __pyx_obj_4grpc_7_cython_6cygrpc_SocketWrapper *__pyx_v_self) {
  PyObject *__pyx_r = NULL;
  __Pyx_RefNannyDeclarations
  PyObject *__pyx_t_1 = NULL;
  __Pyx_RefNannySetupContext("__reduce_cython__", 0);

  /* "(tree fragment)":2
 * def __reduce_cython__(self):
 *     raise TypeError("no default __reduce__ due to non-trivial __cinit__")             # <<<<<<<<<<<<<<
 * def __setstate_cython__(self, __pyx_state):
 *     raise TypeError("no default __reduce__ due to non-trivial __cinit__")
 */
  __pyx_t_1 = __Pyx_PyObject_Call(__pyx_builtin_TypeError, __pyx_tuple__137, NULL); if (unlikely(!__pyx_t_1)) __PYX_ERR(7, 2, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_1);
  __Pyx_Raise(__pyx_t_1, 0, 0, 0);
  __Pyx_DECREF(__pyx_t_1); __pyx_t_1 = 0;
  __PYX_ERR(7, 2, __pyx_L1_error)

  /* "(tree fragment)":1
 * def __reduce_cython__(self):             # <<<<<<<<<<<<<<
 *     raise TypeError("no default __reduce__ due to non-trivial __cinit__")
 * def __setstate_cython__(self, __pyx_state):
 */

  /* function exit code */
  __pyx_L1_error:;
  __Pyx_XDECREF(__pyx_t_1);
  __Pyx_AddTraceback("grpc._cython.cygrpc.SocketWrapper.__reduce_cython__", __pyx_clineno, __pyx_lineno, __pyx_filename);
  __pyx_r = NULL;
  __Pyx_XGIVEREF(__pyx_r);
  __Pyx_RefNannyFinishContext();
  return __pyx_r;
}

/* "(tree fragment)":3
 * def __reduce_cython__(self):
 *     raise TypeError("no default __reduce__ due to non-trivial __cinit__")
 * def __setstate_cython__(self, __pyx_state):             # <<<<<<<<<<<<<<
 *     raise TypeError("no default __reduce__ due to non-trivial __cinit__")
 */

/* Python wrapper */
static PyObject *__pyx_pw_4grpc_7_cython_6cygrpc_13SocketWrapper_5__setstate_cython__(PyObject *__pyx_v_self, PyObject *__pyx_v___pyx_state); /*proto*/
static PyObject *__pyx_pw_4grpc_7_cython_6cygrpc_13SocketWrapper_5__setstate_cython__(PyObject *__pyx_v_self, PyObject *__pyx_v___pyx_state) {
  PyObject *__pyx_r = 0;
  __Pyx_RefNannyDeclarations
  __Pyx_RefNannySetupContext("__setstate_cython__ (wrapper)", 0);
  __pyx_r = __pyx_pf_4grpc_7_cython_6cygrpc_13SocketWrapper_4__setstate_cython__(((struct __pyx_obj_4grpc_7_cython_6cygrpc_SocketWrapper *)__pyx_v_self), ((PyObject *)__pyx_v___pyx_state));

  /* function exit code */
  __Pyx_RefNannyFinishContext();
  return __pyx_r;
}

static PyObject *__pyx_pf_4grpc_7_cython_6cygrpc_13SocketWrapper_4__setstate_cython__(CYTHON_UNUSED struct __pyx_obj_4grpc_7_cython_6cygrpc_SocketWrapper *__pyx_v_self, CYTHON_UNUSED PyObject *__pyx_v___pyx_state) {
  PyObject *__pyx_r = NULL;
  __Pyx_RefNannyDeclarations
  PyObject *__pyx_t_1 = NULL;
  __Pyx_RefNannySetupContext("__setstate_cython__", 0);

  /* "(tree fragment)":4
 *     raise TypeError("no default __reduce__ due to non-trivial __cinit__")
 * def __setstate_cython__(self, __pyx_state):
 *     raise TypeError("no default __reduce__ due to non-trivial __cinit__")             # <<<<<<<<<<<<<<
 */
  __pyx_t_1 = __Pyx_PyObject_Call(__pyx_builtin_TypeError, __pyx_tuple__138, NULL); if (unlikely(!__pyx_t_1)) __PYX_ERR(7, 4, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_1);
  __Pyx_Raise(__pyx_t_1, 0, 0, 0);
  __Pyx_DECREF(__pyx_t_1); __pyx_t_1 = 0;
  __PYX_ERR(7, 4, __pyx_L1_error)

  /* "(tree fragment)":3
 * def __reduce_cython__(self):
 *     raise TypeError("no default __reduce__ due to non-trivial __cinit__")
 * def __setstate_cython__(self, __pyx_state):             # <<<<<<<<<<<<<<
 *     raise TypeError("no default __reduce__ due to non-trivial __cinit__")
 */

  /* function exit code */
  __pyx_L1_error:;
  __Pyx_XDECREF(__pyx_t_1);
  __Pyx_AddTraceback("grpc._cython.cygrpc.SocketWrapper.__setstate_cython__", __pyx_clineno, __pyx_lineno, __pyx_filename);
  __pyx_r = NULL;
  __Pyx_XGIVEREF(__pyx_r);
  __Pyx_RefNannyFinishContext();
  return __pyx_r;
}

/* "src/python/grpcio/grpc/_cython/_cygrpc/grpc_gevent.pyx.pxi":87
 *     self.len = 0
 * 
 * cdef grpc_error* socket_init(grpc_custom_socket* socket, int domain) except * with gil:             # <<<<<<<<<<<<<<
 *   sw = SocketWrapper()
 *   sw.c_socket = socket
 */

static struct grpc_error *__pyx_f_4grpc_7_cython_6cygrpc_socket_init(struct grpc_custom_socket *__pyx_v_socket, CYTHON_UNUSED int __pyx_v_domain) {
  struct __pyx_obj_4grpc_7_cython_6cygrpc_SocketWrapper *__pyx_v_sw = NULL;
  struct grpc_error *__pyx_r;
  __Pyx_RefNannyDeclarations
  PyObject *__pyx_t_1 = NULL;
  #ifdef WITH_THREAD
  PyGILState_STATE __pyx_gilstate_save = __Pyx_PyGILState_Ensure();
  #endif
  __Pyx_RefNannySetupContext("socket_init", 0);

  /* "src/python/grpcio/grpc/_cython/_cygrpc/grpc_gevent.pyx.pxi":88
 * 
 * cdef grpc_error* socket_init(grpc_custom_socket* socket, int domain) except * with gil:
 *   sw = SocketWrapper()             # <<<<<<<<<<<<<<
 *   sw.c_socket = socket
 *   sw.sockopts = []
 */
  __pyx_t_1 = __Pyx_PyObject_CallNoArg(((PyObject *)__pyx_ptype_4grpc_7_cython_6cygrpc_SocketWrapper)); if (unlikely(!__pyx_t_1)) __PYX_ERR(6, 88, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_1);
  __pyx_v_sw = ((struct __pyx_obj_4grpc_7_cython_6cygrpc_SocketWrapper *)__pyx_t_1);
  __pyx_t_1 = 0;

  /* "src/python/grpcio/grpc/_cython/_cygrpc/grpc_gevent.pyx.pxi":89
 * cdef grpc_error* socket_init(grpc_custom_socket* socket, int domain) except * with gil:
 *   sw = SocketWrapper()
 *   sw.c_socket = socket             # <<<<<<<<<<<<<<
 *   sw.sockopts = []
 *   cpython.Py_INCREF(sw)
 */
  __pyx_v_sw->c_socket = __pyx_v_socket;

  /* "src/python/grpcio/grpc/_cython/_cygrpc/grpc_gevent.pyx.pxi":90
 *   sw = SocketWrapper()
 *   sw.c_socket = socket
 *   sw.sockopts = []             # <<<<<<<<<<<<<<
 *   cpython.Py_INCREF(sw)
 *   # Python doesn't support AF_UNSPEC sockets, so we defer creation until
 */
  __pyx_t_1 = PyList_New(0); if (unlikely(!__pyx_t_1)) __PYX_ERR(6, 90, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_1);
  __Pyx_GIVEREF(__pyx_t_1);
  __Pyx_GOTREF(__pyx_v_sw->sockopts);
  __Pyx_DECREF(__pyx_v_sw->sockopts);
  __pyx_v_sw->sockopts = __pyx_t_1;
  __pyx_t_1 = 0;

  /* "src/python/grpcio/grpc/_cython/_cygrpc/grpc_gevent.pyx.pxi":91
 *   sw.c_socket = socket
 *   sw.sockopts = []
 *   cpython.Py_INCREF(sw)             # <<<<<<<<<<<<<<
 *   # Python doesn't support AF_UNSPEC sockets, so we defer creation until
 *   # bind/connect when we know what type of socket we need
 */
  Py_INCREF(((PyObject *)__pyx_v_sw));

  /* "src/python/grpcio/grpc/_cython/_cygrpc/grpc_gevent.pyx.pxi":94
 *   # Python doesn't support AF_UNSPEC sockets, so we defer creation until
 *   # bind/connect when we know what type of socket we need
 *   sw.socket = None             # <<<<<<<<<<<<<<
 *   sw.closed = False
 *   sw.accepting_socket = NULL
 */
  __Pyx_INCREF(Py_None);
  __Pyx_GIVEREF(Py_None);
  __Pyx_GOTREF(__pyx_v_sw->socket);
  __Pyx_DECREF(__pyx_v_sw->socket);
  __pyx_v_sw->socket = Py_None;

  /* "src/python/grpcio/grpc/_cython/_cygrpc/grpc_gevent.pyx.pxi":95
 *   # bind/connect when we know what type of socket we need
 *   sw.socket = None
 *   sw.closed = False             # <<<<<<<<<<<<<<
 *   sw.accepting_socket = NULL
 *   socket.impl = <void*>sw
 */
  __Pyx_INCREF(Py_False);
  __Pyx_GIVEREF(Py_False);
  __Pyx_GOTREF(__pyx_v_sw->closed);
  __Pyx_DECREF(__pyx_v_sw->closed);
  __pyx_v_sw->closed = Py_False;

  /* "src/python/grpcio/grpc/_cython/_cygrpc/grpc_gevent.pyx.pxi":96
 *   sw.socket = None
 *   sw.closed = False
 *   sw.accepting_socket = NULL             # <<<<<<<<<<<<<<
 *   socket.impl = <void*>sw
 *   return grpc_error_none()
 */
  __pyx_v_sw->accepting_socket = NULL;

  /* "src/python/grpcio/grpc/_cython/_cygrpc/grpc_gevent.pyx.pxi":97
 *   sw.closed = False
 *   sw.accepting_socket = NULL
 *   socket.impl = <void*>sw             # <<<<<<<<<<<<<<
 *   return grpc_error_none()
 * 
 */
  __pyx_v_socket->impl = ((void *)__pyx_v_sw);

  /* "src/python/grpcio/grpc/_cython/_cygrpc/grpc_gevent.pyx.pxi":98
 *   sw.accepting_socket = NULL
 *   socket.impl = <void*>sw
 *   return grpc_error_none()             # <<<<<<<<<<<<<<
 * 
 * cdef socket_connect_async_cython(SocketWrapper socket_wrapper, addr_tuple):
 */
  __pyx_r = __pyx_f_4grpc_7_cython_6cygrpc_grpc_error_none();
  goto __pyx_L0;

  /* "src/python/grpcio/grpc/_cython/_cygrpc/grpc_gevent.pyx.pxi":87
 *     self.len = 0
 * 
 * cdef grpc_error* socket_init(grpc_custom_socket* socket, int domain) except * with gil:             # <<<<<<<<<<<<<<
 *   sw = SocketWrapper()
 *   sw.c_socket = socket
 */

  /* function exit code */
  __pyx_L1_error:;
  __Pyx_XDECREF(__pyx_t_1);
  __Pyx_AddTraceback("grpc._cython.cygrpc.socket_init", __pyx_clineno, __pyx_lineno, __pyx_filename);
  __pyx_r = 0;
  __pyx_L0:;
  __Pyx_XDECREF((PyObject *)__pyx_v_sw);
  __Pyx_RefNannyFinishContext();
  #ifdef WITH_THREAD
  __Pyx_PyGILState_Release(__pyx_gilstate_save);
  #endif
  return __pyx_r;
}

/* "src/python/grpcio/grpc/_cython/_cygrpc/grpc_gevent.pyx.pxi":100
 *   return grpc_error_none()
 * 
 * cdef socket_connect_async_cython(SocketWrapper socket_wrapper, addr_tuple):             # <<<<<<<<<<<<<<
 *   try:
 *     socket_wrapper.socket.connect(addr_tuple)
 */

static PyObject *__pyx_f_4grpc_7_cython_6cygrpc_socket_connect_async_cython(struct __pyx_obj_4grpc_7_cython_6cygrpc_SocketWrapper *__pyx_v_socket_wrapper, PyObject *__pyx_v_addr_tuple) {
  PyObject *__pyx_v_io_error = NULL;
  PyObject *__pyx_r = NULL;
  __Pyx_RefNannyDeclarations
  PyObject *__pyx_t_1 = NULL;
  PyObject *__pyx_t_2 = NULL;
  PyObject *__pyx_t_3 = NULL;
  PyObject *__pyx_t_4 = NULL;
  PyObject *__pyx_t_5 = NULL;
  PyObject *__pyx_t_6 = NULL;
  PyObject *__pyx_t_7 = NULL;
  int __pyx_t_8;
  __Pyx_RefNannySetupContext("socket_connect_async_cython", 0);

  /* "src/python/grpcio/grpc/_cython/_cygrpc/grpc_gevent.pyx.pxi":101
 * 
 * cdef socket_connect_async_cython(SocketWrapper socket_wrapper, addr_tuple):
 *   try:             # <<<<<<<<<<<<<<
 *     socket_wrapper.socket.connect(addr_tuple)
 *     socket_wrapper.connect_cb(<grpc_custom_socket*>socket_wrapper.c_socket,
 */
  {
    __Pyx_PyThreadState_declare
    __Pyx_PyThreadState_assign
    __Pyx_ExceptionSave(&__pyx_t_1, &__pyx_t_2, &__pyx_t_3);
    __Pyx_XGOTREF(__pyx_t_1);
    __Pyx_XGOTREF(__pyx_t_2);
    __Pyx_XGOTREF(__pyx_t_3);
    /*try:*/ {

      /* "src/python/grpcio/grpc/_cython/_cygrpc/grpc_gevent.pyx.pxi":102
 * cdef socket_connect_async_cython(SocketWrapper socket_wrapper, addr_tuple):
 *   try:
 *     socket_wrapper.socket.connect(addr_tuple)             # <<<<<<<<<<<<<<
 *     socket_wrapper.connect_cb(<grpc_custom_socket*>socket_wrapper.c_socket,
 *                               grpc_error_none())
 */
      __pyx_t_5 = __Pyx_PyObject_GetAttrStr(__pyx_v_socket_wrapper->socket, __pyx_n_s_connect); if (unlikely(!__pyx_t_5)) __PYX_ERR(6, 102, __pyx_L3_error)
      __Pyx_GOTREF(__pyx_t_5);
      __pyx_t_6 = NULL;
      if (CYTHON_UNPACK_METHODS && likely(PyMethod_Check(__pyx_t_5))) {
        __pyx_t_6 = PyMethod_GET_SELF(__pyx_t_5);
        if (likely(__pyx_t_6)) {
          PyObject* function = PyMethod_GET_FUNCTION(__pyx_t_5);
          __Pyx_INCREF(__pyx_t_6);
          __Pyx_INCREF(function);
          __Pyx_DECREF_SET(__pyx_t_5, function);
        }
      }
      if (!__pyx_t_6) {
        __pyx_t_4 = __Pyx_PyObject_CallOneArg(__pyx_t_5, __pyx_v_addr_tuple); if (unlikely(!__pyx_t_4)) __PYX_ERR(6, 102, __pyx_L3_error)
        __Pyx_GOTREF(__pyx_t_4);
      } else {
        #if CYTHON_FAST_PYCALL
        if (PyFunction_Check(__pyx_t_5)) {
          PyObject *__pyx_temp[2] = {__pyx_t_6, __pyx_v_addr_tuple};
          __pyx_t_4 = __Pyx_PyFunction_FastCall(__pyx_t_5, __pyx_temp+1-1, 1+1); if (unlikely(!__pyx_t_4)) __PYX_ERR(6, 102, __pyx_L3_error)
          __Pyx_XDECREF(__pyx_t_6); __pyx_t_6 = 0;
          __Pyx_GOTREF(__pyx_t_4);
        } else
        #endif
        #if CYTHON_FAST_PYCCALL
        if (__Pyx_PyFastCFunction_Check(__pyx_t_5)) {
          PyObject *__pyx_temp[2] = {__pyx_t_6, __pyx_v_addr_tuple};
          __pyx_t_4 = __Pyx_PyCFunction_FastCall(__pyx_t_5, __pyx_temp+1-1, 1+1); if (unlikely(!__pyx_t_4)) __PYX_ERR(6, 102, __pyx_L3_error)
          __Pyx_XDECREF(__pyx_t_6); __pyx_t_6 = 0;
          __Pyx_GOTREF(__pyx_t_4);
        } else
        #endif
        {
          __pyx_t_7 = PyTuple_New(1+1); if (unlikely(!__pyx_t_7)) __PYX_ERR(6, 102, __pyx_L3_error)
          __Pyx_GOTREF(__pyx_t_7);
          __Pyx_GIVEREF(__pyx_t_6); PyTuple_SET_ITEM(__pyx_t_7, 0, __pyx_t_6); __pyx_t_6 = NULL;
          __Pyx_INCREF(__pyx_v_addr_tuple);
          __Pyx_GIVEREF(__pyx_v_addr_tuple);
          PyTuple_SET_ITEM(__pyx_t_7, 0+1, __pyx_v_addr_tuple);
          __pyx_t_4 = __Pyx_PyObject_Call(__pyx_t_5, __pyx_t_7, NULL); if (unlikely(!__pyx_t_4)) __PYX_ERR(6, 102, __pyx_L3_error)
          __Pyx_GOTREF(__pyx_t_4);
          __Pyx_DECREF(__pyx_t_7); __pyx_t_7 = 0;
        }
      }
      __Pyx_DECREF(__pyx_t_5); __pyx_t_5 = 0;
      __Pyx_DECREF(__pyx_t_4); __pyx_t_4 = 0;

      /* "src/python/grpcio/grpc/_cython/_cygrpc/grpc_gevent.pyx.pxi":103
 *   try:
 *     socket_wrapper.socket.connect(addr_tuple)
 *     socket_wrapper.connect_cb(<grpc_custom_socket*>socket_wrapper.c_socket,             # <<<<<<<<<<<<<<
 *                               grpc_error_none())
 *   except IOError as io_error:
 */
      __pyx_v_socket_wrapper->connect_cb(((struct grpc_custom_socket *)__pyx_v_socket_wrapper->c_socket), __pyx_f_4grpc_7_cython_6cygrpc_grpc_error_none());

      /* "src/python/grpcio/grpc/_cython/_cygrpc/grpc_gevent.pyx.pxi":101
 * 
 * cdef socket_connect_async_cython(SocketWrapper socket_wrapper, addr_tuple):
 *   try:             # <<<<<<<<<<<<<<
 *     socket_wrapper.socket.connect(addr_tuple)
 *     socket_wrapper.connect_cb(<grpc_custom_socket*>socket_wrapper.c_socket,
 */
    }
    __Pyx_XDECREF(__pyx_t_1); __pyx_t_1 = 0;
    __Pyx_XDECREF(__pyx_t_2); __pyx_t_2 = 0;
    __Pyx_XDECREF(__pyx_t_3); __pyx_t_3 = 0;
    goto __pyx_L8_try_end;
    __pyx_L3_error:;
    __Pyx_XDECREF(__pyx_t_6); __pyx_t_6 = 0;
    __Pyx_XDECREF(__pyx_t_7); __pyx_t_7 = 0;
    __Pyx_XDECREF(__pyx_t_5); __pyx_t_5 = 0;
    __Pyx_XDECREF(__pyx_t_4); __pyx_t_4 = 0;

    /* "src/python/grpcio/grpc/_cython/_cygrpc/grpc_gevent.pyx.pxi":105
 *     socket_wrapper.connect_cb(<grpc_custom_socket*>socket_wrapper.c_socket,
 *                               grpc_error_none())
 *   except IOError as io_error:             # <<<<<<<<<<<<<<
 *     socket_wrapper.connect_cb(<grpc_custom_socket*>socket_wrapper.c_socket,
 *                               socket_error("connect", str(io_error)))
 */
    __pyx_t_8 = __Pyx_PyErr_ExceptionMatches(__pyx_builtin_IOError);
    if (__pyx_t_8) {
      __Pyx_AddTraceback("grpc._cython.cygrpc.socket_connect_async_cython", __pyx_clineno, __pyx_lineno, __pyx_filename);
      if (__Pyx_GetException(&__pyx_t_4, &__pyx_t_5, &__pyx_t_7) < 0) __PYX_ERR(6, 105, __pyx_L5_except_error)
      __Pyx_GOTREF(__pyx_t_4);
      __Pyx_GOTREF(__pyx_t_5);
      __Pyx_GOTREF(__pyx_t_7);
      __Pyx_INCREF(__pyx_t_5);
      __pyx_v_io_error = __pyx_t_5;

      /* "src/python/grpcio/grpc/_cython/_cygrpc/grpc_gevent.pyx.pxi":107
 *   except IOError as io_error:
 *     socket_wrapper.connect_cb(<grpc_custom_socket*>socket_wrapper.c_socket,
 *                               socket_error("connect", str(io_error)))             # <<<<<<<<<<<<<<
 *   g_event.set()
 * 
 */
      __pyx_t_6 = __Pyx_PyObject_CallOneArg(((PyObject *)(&PyString_Type)), __pyx_v_io_error); if (unlikely(!__pyx_t_6)) __PYX_ERR(6, 107, __pyx_L5_except_error)
      __Pyx_GOTREF(__pyx_t_6);
      if (!(likely(PyString_CheckExact(__pyx_t_6))||((__pyx_t_6) == Py_None)||(PyErr_Format(PyExc_TypeError, "Expected %.16s, got %.200s", "str", Py_TYPE(__pyx_t_6)->tp_name), 0))) __PYX_ERR(6, 107, __pyx_L5_except_error)

      /* "src/python/grpcio/grpc/_cython/_cygrpc/grpc_gevent.pyx.pxi":106
 *                               grpc_error_none())
 *   except IOError as io_error:
 *     socket_wrapper.connect_cb(<grpc_custom_socket*>socket_wrapper.c_socket,             # <<<<<<<<<<<<<<
 *                               socket_error("connect", str(io_error)))
 *   g_event.set()
 */
      __pyx_v_socket_wrapper->connect_cb(((struct grpc_custom_socket *)__pyx_v_socket_wrapper->c_socket), __pyx_f_4grpc_7_cython_6cygrpc_socket_error(__pyx_n_s_connect, ((PyObject*)__pyx_t_6)));
      __Pyx_DECREF(__pyx_t_6); __pyx_t_6 = 0;
      __Pyx_DECREF(__pyx_t_4); __pyx_t_4 = 0;
      __Pyx_DECREF(__pyx_t_5); __pyx_t_5 = 0;
      __Pyx_DECREF(__pyx_t_7); __pyx_t_7 = 0;
      goto __pyx_L4_exception_handled;
    }
    goto __pyx_L5_except_error;
    __pyx_L5_except_error:;

    /* "src/python/grpcio/grpc/_cython/_cygrpc/grpc_gevent.pyx.pxi":101
 * 
 * cdef socket_connect_async_cython(SocketWrapper socket_wrapper, addr_tuple):
 *   try:             # <<<<<<<<<<<<<<
 *     socket_wrapper.socket.connect(addr_tuple)
 *     socket_wrapper.connect_cb(<grpc_custom_socket*>socket_wrapper.c_socket,
 */
    __Pyx_XGIVEREF(__pyx_t_1);
    __Pyx_XGIVEREF(__pyx_t_2);
    __Pyx_XGIVEREF(__pyx_t_3);
    __Pyx_ExceptionReset(__pyx_t_1, __pyx_t_2, __pyx_t_3);
    goto __pyx_L1_error;
    __pyx_L4_exception_handled:;
    __Pyx_XGIVEREF(__pyx_t_1);
    __Pyx_XGIVEREF(__pyx_t_2);
    __Pyx_XGIVEREF(__pyx_t_3);
    __Pyx_ExceptionReset(__pyx_t_1, __pyx_t_2, __pyx_t_3);
    __pyx_L8_try_end:;
  }

  /* "src/python/grpcio/grpc/_cython/_cygrpc/grpc_gevent.pyx.pxi":108
 *     socket_wrapper.connect_cb(<grpc_custom_socket*>socket_wrapper.c_socket,
 *                               socket_error("connect", str(io_error)))
 *   g_event.set()             # <<<<<<<<<<<<<<
 * 
 * def socket_connect_async(socket_wrapper, addr_tuple):
 */
  __pyx_t_5 = __Pyx_GetModuleGlobalName(__pyx_n_s_g_event); if (unlikely(!__pyx_t_5)) __PYX_ERR(6, 108, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_5);
  __pyx_t_4 = __Pyx_PyObject_GetAttrStr(__pyx_t_5, __pyx_n_s_set); if (unlikely(!__pyx_t_4)) __PYX_ERR(6, 108, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_4);
  __Pyx_DECREF(__pyx_t_5); __pyx_t_5 = 0;
  __pyx_t_5 = NULL;
  if (CYTHON_UNPACK_METHODS && unlikely(PyMethod_Check(__pyx_t_4))) {
    __pyx_t_5 = PyMethod_GET_SELF(__pyx_t_4);
    if (likely(__pyx_t_5)) {
      PyObject* function = PyMethod_GET_FUNCTION(__pyx_t_4);
      __Pyx_INCREF(__pyx_t_5);
      __Pyx_INCREF(function);
      __Pyx_DECREF_SET(__pyx_t_4, function);
    }
  }
  if (__pyx_t_5) {
    __pyx_t_7 = __Pyx_PyObject_CallOneArg(__pyx_t_4, __pyx_t_5); if (unlikely(!__pyx_t_7)) __PYX_ERR(6, 108, __pyx_L1_error)
    __Pyx_DECREF(__pyx_t_5); __pyx_t_5 = 0;
  } else {
    __pyx_t_7 = __Pyx_PyObject_CallNoArg(__pyx_t_4); if (unlikely(!__pyx_t_7)) __PYX_ERR(6, 108, __pyx_L1_error)
  }
  __Pyx_GOTREF(__pyx_t_7);
  __Pyx_DECREF(__pyx_t_4); __pyx_t_4 = 0;
  __Pyx_DECREF(__pyx_t_7); __pyx_t_7 = 0;

  /* "src/python/grpcio/grpc/_cython/_cygrpc/grpc_gevent.pyx.pxi":100
 *   return grpc_error_none()
 * 
 * cdef socket_connect_async_cython(SocketWrapper socket_wrapper, addr_tuple):             # <<<<<<<<<<<<<<
 *   try:
 *     socket_wrapper.socket.connect(addr_tuple)
 */

  /* function exit code */
  __pyx_r = Py_None; __Pyx_INCREF(Py_None);
  goto __pyx_L0;
  __pyx_L1_error:;
  __Pyx_XDECREF(__pyx_t_4);
  __Pyx_XDECREF(__pyx_t_5);
  __Pyx_XDECREF(__pyx_t_6);
  __Pyx_XDECREF(__pyx_t_7);
  __Pyx_AddTraceback("grpc._cython.cygrpc.socket_connect_async_cython", __pyx_clineno, __pyx_lineno, __pyx_filename);
  __pyx_r = 0;
  __pyx_L0:;
  __Pyx_XDECREF(__pyx_v_io_error);
  __Pyx_XGIVEREF(__pyx_r);
  __Pyx_RefNannyFinishContext();
  return __pyx_r;
}

/* "src/python/grpcio/grpc/_cython/_cygrpc/grpc_gevent.pyx.pxi":110
 *   g_event.set()
 * 
 * def socket_connect_async(socket_wrapper, addr_tuple):             # <<<<<<<<<<<<<<
 *   socket_connect_async_cython(socket_wrapper, addr_tuple)
 * 
 */

/* Python wrapper */
static PyObject *__pyx_pw_4grpc_7_cython_6cygrpc_47socket_connect_async(PyObject *__pyx_self, PyObject *__pyx_args, PyObject *__pyx_kwds); /*proto*/
static PyMethodDef __pyx_mdef_4grpc_7_cython_6cygrpc_47socket_connect_async = {"socket_connect_async", (PyCFunction)__pyx_pw_4grpc_7_cython_6cygrpc_47socket_connect_async, METH_VARARGS|METH_KEYWORDS, 0};
static PyObject *__pyx_pw_4grpc_7_cython_6cygrpc_47socket_connect_async(PyObject *__pyx_self, PyObject *__pyx_args, PyObject *__pyx_kwds) {
  PyObject *__pyx_v_socket_wrapper = 0;
  PyObject *__pyx_v_addr_tuple = 0;
  PyObject *__pyx_r = 0;
  __Pyx_RefNannyDeclarations
  __Pyx_RefNannySetupContext("socket_connect_async (wrapper)", 0);
  {
    static PyObject **__pyx_pyargnames[] = {&__pyx_n_s_socket_wrapper,&__pyx_n_s_addr_tuple,0};
    PyObject* values[2] = {0,0};
    if (unlikely(__pyx_kwds)) {
      Py_ssize_t kw_args;
      const Py_ssize_t pos_args = PyTuple_GET_SIZE(__pyx_args);
      switch (pos_args) {
        case  2: values[1] = PyTuple_GET_ITEM(__pyx_args, 1);
        CYTHON_FALLTHROUGH;
        case  1: values[0] = PyTuple_GET_ITEM(__pyx_args, 0);
        CYTHON_FALLTHROUGH;
        case  0: break;
        default: goto __pyx_L5_argtuple_error;
      }
      kw_args = PyDict_Size(__pyx_kwds);
      switch (pos_args) {
        case  0:
        if (likely((values[0] = __Pyx_PyDict_GetItemStr(__pyx_kwds, __pyx_n_s_socket_wrapper)) != 0)) kw_args--;
        else goto __pyx_L5_argtuple_error;
        CYTHON_FALLTHROUGH;
        case  1:
        if (likely((values[1] = __Pyx_PyDict_GetItemStr(__pyx_kwds, __pyx_n_s_addr_tuple)) != 0)) kw_args--;
        else {
          __Pyx_RaiseArgtupleInvalid("socket_connect_async", 1, 2, 2, 1); __PYX_ERR(6, 110, __pyx_L3_error)
        }
      }
      if (unlikely(kw_args > 0)) {
        if (unlikely(__Pyx_ParseOptionalKeywords(__pyx_kwds, __pyx_pyargnames, 0, values, pos_args, "socket_connect_async") < 0)) __PYX_ERR(6, 110, __pyx_L3_error)
      }
    } else if (PyTuple_GET_SIZE(__pyx_args) != 2) {
      goto __pyx_L5_argtuple_error;
    } else {
      values[0] = PyTuple_GET_ITEM(__pyx_args, 0);
      values[1] = PyTuple_GET_ITEM(__pyx_args, 1);
    }
    __pyx_v_socket_wrapper = values[0];
    __pyx_v_addr_tuple = values[1];
  }
  goto __pyx_L4_argument_unpacking_done;
  __pyx_L5_argtuple_error:;
  __Pyx_RaiseArgtupleInvalid("socket_connect_async", 1, 2, 2, PyTuple_GET_SIZE(__pyx_args)); __PYX_ERR(6, 110, __pyx_L3_error)
  __pyx_L3_error:;
  __Pyx_AddTraceback("grpc._cython.cygrpc.socket_connect_async", __pyx_clineno, __pyx_lineno, __pyx_filename);
  __Pyx_RefNannyFinishContext();
  return NULL;
  __pyx_L4_argument_unpacking_done:;
  __pyx_r = __pyx_pf_4grpc_7_cython_6cygrpc_46socket_connect_async(__pyx_self, __pyx_v_socket_wrapper, __pyx_v_addr_tuple);

  /* function exit code */
  __Pyx_RefNannyFinishContext();
  return __pyx_r;
}

static PyObject *__pyx_pf_4grpc_7_cython_6cygrpc_46socket_connect_async(CYTHON_UNUSED PyObject *__pyx_self, PyObject *__pyx_v_socket_wrapper, PyObject *__pyx_v_addr_tuple) {
  PyObject *__pyx_r = NULL;
  __Pyx_RefNannyDeclarations
  PyObject *__pyx_t_1 = NULL;
  __Pyx_RefNannySetupContext("socket_connect_async", 0);

  /* "src/python/grpcio/grpc/_cython/_cygrpc/grpc_gevent.pyx.pxi":111
 * 
 * def socket_connect_async(socket_wrapper, addr_tuple):
 *   socket_connect_async_cython(socket_wrapper, addr_tuple)             # <<<<<<<<<<<<<<
 * 
 * cdef void socket_connect(grpc_custom_socket* socket, const grpc_sockaddr* addr,
 */
  if (!(likely(((__pyx_v_socket_wrapper) == Py_None) || likely(__Pyx_TypeTest(__pyx_v_socket_wrapper, __pyx_ptype_4grpc_7_cython_6cygrpc_SocketWrapper))))) __PYX_ERR(6, 111, __pyx_L1_error)
  __pyx_t_1 = __pyx_f_4grpc_7_cython_6cygrpc_socket_connect_async_cython(((struct __pyx_obj_4grpc_7_cython_6cygrpc_SocketWrapper *)__pyx_v_socket_wrapper), __pyx_v_addr_tuple); if (unlikely(!__pyx_t_1)) __PYX_ERR(6, 111, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_1);
  __Pyx_DECREF(__pyx_t_1); __pyx_t_1 = 0;

  /* "src/python/grpcio/grpc/_cython/_cygrpc/grpc_gevent.pyx.pxi":110
 *   g_event.set()
 * 
 * def socket_connect_async(socket_wrapper, addr_tuple):             # <<<<<<<<<<<<<<
 *   socket_connect_async_cython(socket_wrapper, addr_tuple)
 * 
 */

  /* function exit code */
  __pyx_r = Py_None; __Pyx_INCREF(Py_None);
  goto __pyx_L0;
  __pyx_L1_error:;
  __Pyx_XDECREF(__pyx_t_1);
  __Pyx_AddTraceback("grpc._cython.cygrpc.socket_connect_async", __pyx_clineno, __pyx_lineno, __pyx_filename);
  __pyx_r = NULL;
  __pyx_L0:;
  __Pyx_XGIVEREF(__pyx_r);
  __Pyx_RefNannyFinishContext();
  return __pyx_r;
}

/* "src/python/grpcio/grpc/_cython/_cygrpc/grpc_gevent.pyx.pxi":113
 *   socket_connect_async_cython(socket_wrapper, addr_tuple)
 * 
 * cdef void socket_connect(grpc_custom_socket* socket, const grpc_sockaddr* addr,             # <<<<<<<<<<<<<<
 *                          size_t addr_len,
 *                          grpc_custom_connect_callback cb) except * with gil:
 */

static void __pyx_f_4grpc_7_cython_6cygrpc_socket_connect(struct grpc_custom_socket *__pyx_v_socket, grpc_sockaddr const *__pyx_v_addr, size_t __pyx_v_addr_len, grpc_custom_connect_callback __pyx_v_cb) {
  PyObject *__pyx_v_py_socket = NULL;
  struct __pyx_obj_4grpc_7_cython_6cygrpc_SocketWrapper *__pyx_v_socket_wrapper = NULL;
  PyObject *__pyx_v_addr_tuple = NULL;
  __Pyx_RefNannyDeclarations
  PyObject *__pyx_t_1 = NULL;
  int __pyx_t_2;
  PyObject *__pyx_t_3 = NULL;
  PyObject *__pyx_t_4 = NULL;
  PyObject *__pyx_t_5 = NULL;
  PyObject *__pyx_t_6 = NULL;
  int __pyx_t_7;
  #ifdef WITH_THREAD
  PyGILState_STATE __pyx_gilstate_save = __Pyx_PyGILState_Ensure();
  #endif
  __Pyx_RefNannySetupContext("socket_connect", 0);

  /* "src/python/grpcio/grpc/_cython/_cygrpc/grpc_gevent.pyx.pxi":116
 *                          size_t addr_len,
 *                          grpc_custom_connect_callback cb) except * with gil:
 *   py_socket = None             # <<<<<<<<<<<<<<
 *   socket_wrapper = <SocketWrapper>socket.impl
 *   socket_wrapper.connect_cb = cb
 */
  __Pyx_INCREF(Py_None);
  __pyx_v_py_socket = Py_None;

  /* "src/python/grpcio/grpc/_cython/_cygrpc/grpc_gevent.pyx.pxi":117
 *                          grpc_custom_connect_callback cb) except * with gil:
 *   py_socket = None
 *   socket_wrapper = <SocketWrapper>socket.impl             # <<<<<<<<<<<<<<
 *   socket_wrapper.connect_cb = cb
 *   addr_tuple = sockaddr_to_tuple(addr, addr_len)
 */
  __pyx_t_1 = ((PyObject *)__pyx_v_socket->impl);
  __Pyx_INCREF(__pyx_t_1);
  __pyx_v_socket_wrapper = ((struct __pyx_obj_4grpc_7_cython_6cygrpc_SocketWrapper *)__pyx_t_1);
  __pyx_t_1 = 0;

  /* "src/python/grpcio/grpc/_cython/_cygrpc/grpc_gevent.pyx.pxi":118
 *   py_socket = None
 *   socket_wrapper = <SocketWrapper>socket.impl
 *   socket_wrapper.connect_cb = cb             # <<<<<<<<<<<<<<
 *   addr_tuple = sockaddr_to_tuple(addr, addr_len)
 *   if sockaddr_is_ipv4(addr, addr_len):
 */
  __pyx_v_socket_wrapper->connect_cb = __pyx_v_cb;

  /* "src/python/grpcio/grpc/_cython/_cygrpc/grpc_gevent.pyx.pxi":119
 *   socket_wrapper = <SocketWrapper>socket.impl
 *   socket_wrapper.connect_cb = cb
 *   addr_tuple = sockaddr_to_tuple(addr, addr_len)             # <<<<<<<<<<<<<<
 *   if sockaddr_is_ipv4(addr, addr_len):
 *       py_socket = gevent_socket.socket(gevent_socket.AF_INET)
 */
  __pyx_t_1 = __pyx_f_4grpc_7_cython_6cygrpc_sockaddr_to_tuple(__pyx_v_addr, __pyx_v_addr_len); if (unlikely(!__pyx_t_1)) __PYX_ERR(6, 119, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_1);
  __pyx_v_addr_tuple = __pyx_t_1;
  __pyx_t_1 = 0;

  /* "src/python/grpcio/grpc/_cython/_cygrpc/grpc_gevent.pyx.pxi":120
 *   socket_wrapper.connect_cb = cb
 *   addr_tuple = sockaddr_to_tuple(addr, addr_len)
 *   if sockaddr_is_ipv4(addr, addr_len):             # <<<<<<<<<<<<<<
 *       py_socket = gevent_socket.socket(gevent_socket.AF_INET)
 *   else:
 */
  __pyx_t_1 = __pyx_f_4grpc_7_cython_6cygrpc_sockaddr_is_ipv4(__pyx_v_addr, __pyx_v_addr_len); if (unlikely(!__pyx_t_1)) __PYX_ERR(6, 120, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_1);
  __pyx_t_2 = __Pyx_PyObject_IsTrue(__pyx_t_1); if (unlikely(__pyx_t_2 < 0)) __PYX_ERR(6, 120, __pyx_L1_error)
  __Pyx_DECREF(__pyx_t_1); __pyx_t_1 = 0;
  if (__pyx_t_2) {

    /* "src/python/grpcio/grpc/_cython/_cygrpc/grpc_gevent.pyx.pxi":121
 *   addr_tuple = sockaddr_to_tuple(addr, addr_len)
 *   if sockaddr_is_ipv4(addr, addr_len):
 *       py_socket = gevent_socket.socket(gevent_socket.AF_INET)             # <<<<<<<<<<<<<<
 *   else:
 *       py_socket = gevent_socket.socket(gevent_socket.AF_INET6)
 */
    __pyx_t_3 = __Pyx_GetModuleGlobalName(__pyx_n_s_gevent_socket); if (unlikely(!__pyx_t_3)) __PYX_ERR(6, 121, __pyx_L1_error)
    __Pyx_GOTREF(__pyx_t_3);
    __pyx_t_4 = __Pyx_PyObject_GetAttrStr(__pyx_t_3, __pyx_n_s_socket); if (unlikely(!__pyx_t_4)) __PYX_ERR(6, 121, __pyx_L1_error)
    __Pyx_GOTREF(__pyx_t_4);
    __Pyx_DECREF(__pyx_t_3); __pyx_t_3 = 0;
    __pyx_t_3 = __Pyx_GetModuleGlobalName(__pyx_n_s_gevent_socket); if (unlikely(!__pyx_t_3)) __PYX_ERR(6, 121, __pyx_L1_error)
    __Pyx_GOTREF(__pyx_t_3);
    __pyx_t_5 = __Pyx_PyObject_GetAttrStr(__pyx_t_3, __pyx_n_s_AF_INET); if (unlikely(!__pyx_t_5)) __PYX_ERR(6, 121, __pyx_L1_error)
    __Pyx_GOTREF(__pyx_t_5);
    __Pyx_DECREF(__pyx_t_3); __pyx_t_3 = 0;
    __pyx_t_3 = NULL;
    if (CYTHON_UNPACK_METHODS && unlikely(PyMethod_Check(__pyx_t_4))) {
      __pyx_t_3 = PyMethod_GET_SELF(__pyx_t_4);
      if (likely(__pyx_t_3)) {
        PyObject* function = PyMethod_GET_FUNCTION(__pyx_t_4);
        __Pyx_INCREF(__pyx_t_3);
        __Pyx_INCREF(function);
        __Pyx_DECREF_SET(__pyx_t_4, function);
      }
    }
    if (!__pyx_t_3) {
      __pyx_t_1 = __Pyx_PyObject_CallOneArg(__pyx_t_4, __pyx_t_5); if (unlikely(!__pyx_t_1)) __PYX_ERR(6, 121, __pyx_L1_error)
      __Pyx_DECREF(__pyx_t_5); __pyx_t_5 = 0;
      __Pyx_GOTREF(__pyx_t_1);
    } else {
      #if CYTHON_FAST_PYCALL
      if (PyFunction_Check(__pyx_t_4)) {
        PyObject *__pyx_temp[2] = {__pyx_t_3, __pyx_t_5};
        __pyx_t_1 = __Pyx_PyFunction_FastCall(__pyx_t_4, __pyx_temp+1-1, 1+1); if (unlikely(!__pyx_t_1)) __PYX_ERR(6, 121, __pyx_L1_error)
        __Pyx_XDECREF(__pyx_t_3); __pyx_t_3 = 0;
        __Pyx_GOTREF(__pyx_t_1);
        __Pyx_DECREF(__pyx_t_5); __pyx_t_5 = 0;
      } else
      #endif
      #if CYTHON_FAST_PYCCALL
      if (__Pyx_PyFastCFunction_Check(__pyx_t_4)) {
        PyObject *__pyx_temp[2] = {__pyx_t_3, __pyx_t_5};
        __pyx_t_1 = __Pyx_PyCFunction_FastCall(__pyx_t_4, __pyx_temp+1-1, 1+1); if (unlikely(!__pyx_t_1)) __PYX_ERR(6, 121, __pyx_L1_error)
        __Pyx_XDECREF(__pyx_t_3); __pyx_t_3 = 0;
        __Pyx_GOTREF(__pyx_t_1);
        __Pyx_DECREF(__pyx_t_5); __pyx_t_5 = 0;
      } else
      #endif
      {
        __pyx_t_6 = PyTuple_New(1+1); if (unlikely(!__pyx_t_6)) __PYX_ERR(6, 121, __pyx_L1_error)
        __Pyx_GOTREF(__pyx_t_6);
        __Pyx_GIVEREF(__pyx_t_3); PyTuple_SET_ITEM(__pyx_t_6, 0, __pyx_t_3); __pyx_t_3 = NULL;
        __Pyx_GIVEREF(__pyx_t_5);
        PyTuple_SET_ITEM(__pyx_t_6, 0+1, __pyx_t_5);
        __pyx_t_5 = 0;
        __pyx_t_1 = __Pyx_PyObject_Call(__pyx_t_4, __pyx_t_6, NULL); if (unlikely(!__pyx_t_1)) __PYX_ERR(6, 121, __pyx_L1_error)
        __Pyx_GOTREF(__pyx_t_1);
        __Pyx_DECREF(__pyx_t_6); __pyx_t_6 = 0;
      }
    }
    __Pyx_DECREF(__pyx_t_4); __pyx_t_4 = 0;
    __Pyx_DECREF_SET(__pyx_v_py_socket, __pyx_t_1);
    __pyx_t_1 = 0;

    /* "src/python/grpcio/grpc/_cython/_cygrpc/grpc_gevent.pyx.pxi":120
 *   socket_wrapper.connect_cb = cb
 *   addr_tuple = sockaddr_to_tuple(addr, addr_len)
 *   if sockaddr_is_ipv4(addr, addr_len):             # <<<<<<<<<<<<<<
 *       py_socket = gevent_socket.socket(gevent_socket.AF_INET)
 *   else:
 */
    goto __pyx_L3;
  }

  /* "src/python/grpcio/grpc/_cython/_cygrpc/grpc_gevent.pyx.pxi":123
 *       py_socket = gevent_socket.socket(gevent_socket.AF_INET)
 *   else:
 *       py_socket = gevent_socket.socket(gevent_socket.AF_INET6)             # <<<<<<<<<<<<<<
 *   applysockopts(py_socket)
 *   socket_wrapper.socket = py_socket
 */
  /*else*/ {
    __pyx_t_4 = __Pyx_GetModuleGlobalName(__pyx_n_s_gevent_socket); if (unlikely(!__pyx_t_4)) __PYX_ERR(6, 123, __pyx_L1_error)
    __Pyx_GOTREF(__pyx_t_4);
    __pyx_t_6 = __Pyx_PyObject_GetAttrStr(__pyx_t_4, __pyx_n_s_socket); if (unlikely(!__pyx_t_6)) __PYX_ERR(6, 123, __pyx_L1_error)
    __Pyx_GOTREF(__pyx_t_6);
    __Pyx_DECREF(__pyx_t_4); __pyx_t_4 = 0;
    __pyx_t_4 = __Pyx_GetModuleGlobalName(__pyx_n_s_gevent_socket); if (unlikely(!__pyx_t_4)) __PYX_ERR(6, 123, __pyx_L1_error)
    __Pyx_GOTREF(__pyx_t_4);
    __pyx_t_5 = __Pyx_PyObject_GetAttrStr(__pyx_t_4, __pyx_n_s_AF_INET6); if (unlikely(!__pyx_t_5)) __PYX_ERR(6, 123, __pyx_L1_error)
    __Pyx_GOTREF(__pyx_t_5);
    __Pyx_DECREF(__pyx_t_4); __pyx_t_4 = 0;
    __pyx_t_4 = NULL;
    if (CYTHON_UNPACK_METHODS && unlikely(PyMethod_Check(__pyx_t_6))) {
      __pyx_t_4 = PyMethod_GET_SELF(__pyx_t_6);
      if (likely(__pyx_t_4)) {
        PyObject* function = PyMethod_GET_FUNCTION(__pyx_t_6);
        __Pyx_INCREF(__pyx_t_4);
        __Pyx_INCREF(function);
        __Pyx_DECREF_SET(__pyx_t_6, function);
      }
    }
    if (!__pyx_t_4) {
      __pyx_t_1 = __Pyx_PyObject_CallOneArg(__pyx_t_6, __pyx_t_5); if (unlikely(!__pyx_t_1)) __PYX_ERR(6, 123, __pyx_L1_error)
      __Pyx_DECREF(__pyx_t_5); __pyx_t_5 = 0;
      __Pyx_GOTREF(__pyx_t_1);
    } else {
      #if CYTHON_FAST_PYCALL
      if (PyFunction_Check(__pyx_t_6)) {
        PyObject *__pyx_temp[2] = {__pyx_t_4, __pyx_t_5};
        __pyx_t_1 = __Pyx_PyFunction_FastCall(__pyx_t_6, __pyx_temp+1-1, 1+1); if (unlikely(!__pyx_t_1)) __PYX_ERR(6, 123, __pyx_L1_error)
        __Pyx_XDECREF(__pyx_t_4); __pyx_t_4 = 0;
        __Pyx_GOTREF(__pyx_t_1);
        __Pyx_DECREF(__pyx_t_5); __pyx_t_5 = 0;
      } else
      #endif
      #if CYTHON_FAST_PYCCALL
      if (__Pyx_PyFastCFunction_Check(__pyx_t_6)) {
        PyObject *__pyx_temp[2] = {__pyx_t_4, __pyx_t_5};
        __pyx_t_1 = __Pyx_PyCFunction_FastCall(__pyx_t_6, __pyx_temp+1-1, 1+1); if (unlikely(!__pyx_t_1)) __PYX_ERR(6, 123, __pyx_L1_error)
        __Pyx_XDECREF(__pyx_t_4); __pyx_t_4 = 0;
        __Pyx_GOTREF(__pyx_t_1);
        __Pyx_DECREF(__pyx_t_5); __pyx_t_5 = 0;
      } else
      #endif
      {
        __pyx_t_3 = PyTuple_New(1+1); if (unlikely(!__pyx_t_3)) __PYX_ERR(6, 123, __pyx_L1_error)
        __Pyx_GOTREF(__pyx_t_3);
        __Pyx_GIVEREF(__pyx_t_4); PyTuple_SET_ITEM(__pyx_t_3, 0, __pyx_t_4); __pyx_t_4 = NULL;
        __Pyx_GIVEREF(__pyx_t_5);
        PyTuple_SET_ITEM(__pyx_t_3, 0+1, __pyx_t_5);
        __pyx_t_5 = 0;
        __pyx_t_1 = __Pyx_PyObject_Call(__pyx_t_6, __pyx_t_3, NULL); if (unlikely(!__pyx_t_1)) __PYX_ERR(6, 123, __pyx_L1_error)
        __Pyx_GOTREF(__pyx_t_1);
        __Pyx_DECREF(__pyx_t_3); __pyx_t_3 = 0;
      }
    }
    __Pyx_DECREF(__pyx_t_6); __pyx_t_6 = 0;
    __Pyx_DECREF_SET(__pyx_v_py_socket, __pyx_t_1);
    __pyx_t_1 = 0;
  }
  __pyx_L3:;

  /* "src/python/grpcio/grpc/_cython/_cygrpc/grpc_gevent.pyx.pxi":124
 *   else:
 *       py_socket = gevent_socket.socket(gevent_socket.AF_INET6)
 *   applysockopts(py_socket)             # <<<<<<<<<<<<<<
 *   socket_wrapper.socket = py_socket
 *   _spawn_greenlet(socket_connect_async, socket_wrapper, addr_tuple)
 */
  __pyx_t_6 = __Pyx_GetModuleGlobalName(__pyx_n_s_applysockopts); if (unlikely(!__pyx_t_6)) __PYX_ERR(6, 124, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_6);
  __pyx_t_3 = NULL;
  if (CYTHON_UNPACK_METHODS && unlikely(PyMethod_Check(__pyx_t_6))) {
    __pyx_t_3 = PyMethod_GET_SELF(__pyx_t_6);
    if (likely(__pyx_t_3)) {
      PyObject* function = PyMethod_GET_FUNCTION(__pyx_t_6);
      __Pyx_INCREF(__pyx_t_3);
      __Pyx_INCREF(function);
      __Pyx_DECREF_SET(__pyx_t_6, function);
    }
  }
  if (!__pyx_t_3) {
    __pyx_t_1 = __Pyx_PyObject_CallOneArg(__pyx_t_6, __pyx_v_py_socket); if (unlikely(!__pyx_t_1)) __PYX_ERR(6, 124, __pyx_L1_error)
    __Pyx_GOTREF(__pyx_t_1);
  } else {
    #if CYTHON_FAST_PYCALL
    if (PyFunction_Check(__pyx_t_6)) {
      PyObject *__pyx_temp[2] = {__pyx_t_3, __pyx_v_py_socket};
      __pyx_t_1 = __Pyx_PyFunction_FastCall(__pyx_t_6, __pyx_temp+1-1, 1+1); if (unlikely(!__pyx_t_1)) __PYX_ERR(6, 124, __pyx_L1_error)
      __Pyx_XDECREF(__pyx_t_3); __pyx_t_3 = 0;
      __Pyx_GOTREF(__pyx_t_1);
    } else
    #endif
    #if CYTHON_FAST_PYCCALL
    if (__Pyx_PyFastCFunction_Check(__pyx_t_6)) {
      PyObject *__pyx_temp[2] = {__pyx_t_3, __pyx_v_py_socket};
      __pyx_t_1 = __Pyx_PyCFunction_FastCall(__pyx_t_6, __pyx_temp+1-1, 1+1); if (unlikely(!__pyx_t_1)) __PYX_ERR(6, 124, __pyx_L1_error)
      __Pyx_XDECREF(__pyx_t_3); __pyx_t_3 = 0;
      __Pyx_GOTREF(__pyx_t_1);
    } else
    #endif
    {
      __pyx_t_5 = PyTuple_New(1+1); if (unlikely(!__pyx_t_5)) __PYX_ERR(6, 124, __pyx_L1_error)
      __Pyx_GOTREF(__pyx_t_5);
      __Pyx_GIVEREF(__pyx_t_3); PyTuple_SET_ITEM(__pyx_t_5, 0, __pyx_t_3); __pyx_t_3 = NULL;
      __Pyx_INCREF(__pyx_v_py_socket);
      __Pyx_GIVEREF(__pyx_v_py_socket);
      PyTuple_SET_ITEM(__pyx_t_5, 0+1, __pyx_v_py_socket);
      __pyx_t_1 = __Pyx_PyObject_Call(__pyx_t_6, __pyx_t_5, NULL); if (unlikely(!__pyx_t_1)) __PYX_ERR(6, 124, __pyx_L1_error)
      __Pyx_GOTREF(__pyx_t_1);
      __Pyx_DECREF(__pyx_t_5); __pyx_t_5 = 0;
    }
  }
  __Pyx_DECREF(__pyx_t_6); __pyx_t_6 = 0;
  __Pyx_DECREF(__pyx_t_1); __pyx_t_1 = 0;

  /* "src/python/grpcio/grpc/_cython/_cygrpc/grpc_gevent.pyx.pxi":125
 *       py_socket = gevent_socket.socket(gevent_socket.AF_INET6)
 *   applysockopts(py_socket)
 *   socket_wrapper.socket = py_socket             # <<<<<<<<<<<<<<
 *   _spawn_greenlet(socket_connect_async, socket_wrapper, addr_tuple)
 * 
 */
  __Pyx_INCREF(__pyx_v_py_socket);
  __Pyx_GIVEREF(__pyx_v_py_socket);
  __Pyx_GOTREF(__pyx_v_socket_wrapper->socket);
  __Pyx_DECREF(__pyx_v_socket_wrapper->socket);
  __pyx_v_socket_wrapper->socket = __pyx_v_py_socket;

  /* "src/python/grpcio/grpc/_cython/_cygrpc/grpc_gevent.pyx.pxi":126
 *   applysockopts(py_socket)
 *   socket_wrapper.socket = py_socket
 *   _spawn_greenlet(socket_connect_async, socket_wrapper, addr_tuple)             # <<<<<<<<<<<<<<
 * 
 * cdef void socket_destroy(grpc_custom_socket* socket) except * with gil:
 */
  __pyx_t_6 = __Pyx_GetModuleGlobalName(__pyx_n_s_spawn_greenlet); if (unlikely(!__pyx_t_6)) __PYX_ERR(6, 126, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_6);
  __pyx_t_5 = __Pyx_GetModuleGlobalName(__pyx_n_s_socket_connect_async); if (unlikely(!__pyx_t_5)) __PYX_ERR(6, 126, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_5);
  __pyx_t_3 = NULL;
  __pyx_t_7 = 0;
  if (CYTHON_UNPACK_METHODS && unlikely(PyMethod_Check(__pyx_t_6))) {
    __pyx_t_3 = PyMethod_GET_SELF(__pyx_t_6);
    if (likely(__pyx_t_3)) {
      PyObject* function = PyMethod_GET_FUNCTION(__pyx_t_6);
      __Pyx_INCREF(__pyx_t_3);
      __Pyx_INCREF(function);
      __Pyx_DECREF_SET(__pyx_t_6, function);
      __pyx_t_7 = 1;
    }
  }
  #if CYTHON_FAST_PYCALL
  if (PyFunction_Check(__pyx_t_6)) {
    PyObject *__pyx_temp[4] = {__pyx_t_3, __pyx_t_5, ((PyObject *)__pyx_v_socket_wrapper), __pyx_v_addr_tuple};
    __pyx_t_1 = __Pyx_PyFunction_FastCall(__pyx_t_6, __pyx_temp+1-__pyx_t_7, 3+__pyx_t_7); if (unlikely(!__pyx_t_1)) __PYX_ERR(6, 126, __pyx_L1_error)
    __Pyx_XDECREF(__pyx_t_3); __pyx_t_3 = 0;
    __Pyx_GOTREF(__pyx_t_1);
    __Pyx_DECREF(__pyx_t_5); __pyx_t_5 = 0;
  } else
  #endif
  #if CYTHON_FAST_PYCCALL
  if (__Pyx_PyFastCFunction_Check(__pyx_t_6)) {
    PyObject *__pyx_temp[4] = {__pyx_t_3, __pyx_t_5, ((PyObject *)__pyx_v_socket_wrapper), __pyx_v_addr_tuple};
    __pyx_t_1 = __Pyx_PyCFunction_FastCall(__pyx_t_6, __pyx_temp+1-__pyx_t_7, 3+__pyx_t_7); if (unlikely(!__pyx_t_1)) __PYX_ERR(6, 126, __pyx_L1_error)
    __Pyx_XDECREF(__pyx_t_3); __pyx_t_3 = 0;
    __Pyx_GOTREF(__pyx_t_1);
    __Pyx_DECREF(__pyx_t_5); __pyx_t_5 = 0;
  } else
  #endif
  {
    __pyx_t_4 = PyTuple_New(3+__pyx_t_7); if (unlikely(!__pyx_t_4)) __PYX_ERR(6, 126, __pyx_L1_error)
    __Pyx_GOTREF(__pyx_t_4);
    if (__pyx_t_3) {
      __Pyx_GIVEREF(__pyx_t_3); PyTuple_SET_ITEM(__pyx_t_4, 0, __pyx_t_3); __pyx_t_3 = NULL;
    }
    __Pyx_GIVEREF(__pyx_t_5);
    PyTuple_SET_ITEM(__pyx_t_4, 0+__pyx_t_7, __pyx_t_5);
    __Pyx_INCREF(((PyObject *)__pyx_v_socket_wrapper));
    __Pyx_GIVEREF(((PyObject *)__pyx_v_socket_wrapper));
    PyTuple_SET_ITEM(__pyx_t_4, 1+__pyx_t_7, ((PyObject *)__pyx_v_socket_wrapper));
    __Pyx_INCREF(__pyx_v_addr_tuple);
    __Pyx_GIVEREF(__pyx_v_addr_tuple);
    PyTuple_SET_ITEM(__pyx_t_4, 2+__pyx_t_7, __pyx_v_addr_tuple);
    __pyx_t_5 = 0;
    __pyx_t_1 = __Pyx_PyObject_Call(__pyx_t_6, __pyx_t_4, NULL); if (unlikely(!__pyx_t_1)) __PYX_ERR(6, 126, __pyx_L1_error)
    __Pyx_GOTREF(__pyx_t_1);
    __Pyx_DECREF(__pyx_t_4); __pyx_t_4 = 0;
  }
  __Pyx_DECREF(__pyx_t_6); __pyx_t_6 = 0;
  __Pyx_DECREF(__pyx_t_1); __pyx_t_1 = 0;

  /* "src/python/grpcio/grpc/_cython/_cygrpc/grpc_gevent.pyx.pxi":113
 *   socket_connect_async_cython(socket_wrapper, addr_tuple)
 * 
 * cdef void socket_connect(grpc_custom_socket* socket, const grpc_sockaddr* addr,             # <<<<<<<<<<<<<<
 *                          size_t addr_len,
 *                          grpc_custom_connect_callback cb) except * with gil:
 */

  /* function exit code */
  goto __pyx_L0;
  __pyx_L1_error:;
  __Pyx_XDECREF(__pyx_t_1);
  __Pyx_XDECREF(__pyx_t_3);
  __Pyx_XDECREF(__pyx_t_4);
  __Pyx_XDECREF(__pyx_t_5);
  __Pyx_XDECREF(__pyx_t_6);
  __Pyx_AddTraceback("grpc._cython.cygrpc.socket_connect", __pyx_clineno, __pyx_lineno, __pyx_filename);
  __pyx_L0:;
  __Pyx_XDECREF(__pyx_v_py_socket);
  __Pyx_XDECREF((PyObject *)__pyx_v_socket_wrapper);
  __Pyx_XDECREF(__pyx_v_addr_tuple);
  __Pyx_RefNannyFinishContext();
  #ifdef WITH_THREAD
  __Pyx_PyGILState_Release(__pyx_gilstate_save);
  #endif
}

/* "src/python/grpcio/grpc/_cython/_cygrpc/grpc_gevent.pyx.pxi":128
 *   _spawn_greenlet(socket_connect_async, socket_wrapper, addr_tuple)
 * 
 * cdef void socket_destroy(grpc_custom_socket* socket) except * with gil:             # <<<<<<<<<<<<<<
 *   cpython.Py_DECREF(<SocketWrapper>socket.impl)
 * 
 */

static void __pyx_f_4grpc_7_cython_6cygrpc_socket_destroy(struct grpc_custom_socket *__pyx_v_socket) {
  __Pyx_RefNannyDeclarations
  PyObject *__pyx_t_1 = NULL;
  #ifdef WITH_THREAD
  PyGILState_STATE __pyx_gilstate_save = __Pyx_PyGILState_Ensure();
  #endif
  __Pyx_RefNannySetupContext("socket_destroy", 0);

  /* "src/python/grpcio/grpc/_cython/_cygrpc/grpc_gevent.pyx.pxi":129
 * 
 * cdef void socket_destroy(grpc_custom_socket* socket) except * with gil:
 *   cpython.Py_DECREF(<SocketWrapper>socket.impl)             # <<<<<<<<<<<<<<
 * 
 * cdef void socket_shutdown(grpc_custom_socket* socket) except * with gil:
 */
  __pyx_t_1 = ((PyObject *)__pyx_v_socket->impl);
  __Pyx_INCREF(__pyx_t_1);
  Py_DECREF(__pyx_t_1);
  __Pyx_DECREF(__pyx_t_1); __pyx_t_1 = 0;

  /* "src/python/grpcio/grpc/_cython/_cygrpc/grpc_gevent.pyx.pxi":128
 *   _spawn_greenlet(socket_connect_async, socket_wrapper, addr_tuple)
 * 
 * cdef void socket_destroy(grpc_custom_socket* socket) except * with gil:             # <<<<<<<<<<<<<<
 *   cpython.Py_DECREF(<SocketWrapper>socket.impl)
 * 
 */

  /* function exit code */
  __Pyx_RefNannyFinishContext();
  #ifdef WITH_THREAD
  __Pyx_PyGILState_Release(__pyx_gilstate_save);
  #endif
}

/* "src/python/grpcio/grpc/_cython/_cygrpc/grpc_gevent.pyx.pxi":131
 *   cpython.Py_DECREF(<SocketWrapper>socket.impl)
 * 
 * cdef void socket_shutdown(grpc_custom_socket* socket) except * with gil:             # <<<<<<<<<<<<<<
 *   try:
 *     (<SocketWrapper>socket.impl).socket.shutdown(gevent_socket.SHUT_RDWR)
 */

static void __pyx_f_4grpc_7_cython_6cygrpc_socket_shutdown(struct grpc_custom_socket *__pyx_v_socket) {
  PyObject *__pyx_v_io_error = NULL;
  __Pyx_RefNannyDeclarations
  PyObject *__pyx_t_1 = NULL;
  PyObject *__pyx_t_2 = NULL;
  PyObject *__pyx_t_3 = NULL;
  PyObject *__pyx_t_4 = NULL;
  PyObject *__pyx_t_5 = NULL;
  PyObject *__pyx_t_6 = NULL;
  PyObject *__pyx_t_7 = NULL;
  PyObject *__pyx_t_8 = NULL;
  int __pyx_t_9;
  PyObject *__pyx_t_10 = NULL;
  int __pyx_t_11;
  #ifdef WITH_THREAD
  PyGILState_STATE __pyx_gilstate_save = __Pyx_PyGILState_Ensure();
  #endif
  __Pyx_RefNannySetupContext("socket_shutdown", 0);

  /* "src/python/grpcio/grpc/_cython/_cygrpc/grpc_gevent.pyx.pxi":132
 * 
 * cdef void socket_shutdown(grpc_custom_socket* socket) except * with gil:
 *   try:             # <<<<<<<<<<<<<<
 *     (<SocketWrapper>socket.impl).socket.shutdown(gevent_socket.SHUT_RDWR)
 *   except IOError as io_error:
 */
  {
    __Pyx_PyThreadState_declare
    __Pyx_PyThreadState_assign
    __Pyx_ExceptionSave(&__pyx_t_1, &__pyx_t_2, &__pyx_t_3);
    __Pyx_XGOTREF(__pyx_t_1);
    __Pyx_XGOTREF(__pyx_t_2);
    __Pyx_XGOTREF(__pyx_t_3);
    /*try:*/ {

      /* "src/python/grpcio/grpc/_cython/_cygrpc/grpc_gevent.pyx.pxi":133
 * cdef void socket_shutdown(grpc_custom_socket* socket) except * with gil:
 *   try:
 *     (<SocketWrapper>socket.impl).socket.shutdown(gevent_socket.SHUT_RDWR)             # <<<<<<<<<<<<<<
 *   except IOError as io_error:
 *     if io_error.errno != errno.ENOTCONN:
 */
      __pyx_t_5 = __Pyx_PyObject_GetAttrStr(((struct __pyx_obj_4grpc_7_cython_6cygrpc_SocketWrapper *)__pyx_v_socket->impl)->socket, __pyx_n_s_shutdown); if (unlikely(!__pyx_t_5)) __PYX_ERR(6, 133, __pyx_L3_error)
      __Pyx_GOTREF(__pyx_t_5);
      __pyx_t_6 = __Pyx_GetModuleGlobalName(__pyx_n_s_gevent_socket); if (unlikely(!__pyx_t_6)) __PYX_ERR(6, 133, __pyx_L3_error)
      __Pyx_GOTREF(__pyx_t_6);
      __pyx_t_7 = __Pyx_PyObject_GetAttrStr(__pyx_t_6, __pyx_n_s_SHUT_RDWR); if (unlikely(!__pyx_t_7)) __PYX_ERR(6, 133, __pyx_L3_error)
      __Pyx_GOTREF(__pyx_t_7);
      __Pyx_DECREF(__pyx_t_6); __pyx_t_6 = 0;
      __pyx_t_6 = NULL;
      if (CYTHON_UNPACK_METHODS && likely(PyMethod_Check(__pyx_t_5))) {
        __pyx_t_6 = PyMethod_GET_SELF(__pyx_t_5);
        if (likely(__pyx_t_6)) {
          PyObject* function = PyMethod_GET_FUNCTION(__pyx_t_5);
          __Pyx_INCREF(__pyx_t_6);
          __Pyx_INCREF(function);
          __Pyx_DECREF_SET(__pyx_t_5, function);
        }
      }
      if (!__pyx_t_6) {
        __pyx_t_4 = __Pyx_PyObject_CallOneArg(__pyx_t_5, __pyx_t_7); if (unlikely(!__pyx_t_4)) __PYX_ERR(6, 133, __pyx_L3_error)
        __Pyx_DECREF(__pyx_t_7); __pyx_t_7 = 0;
        __Pyx_GOTREF(__pyx_t_4);
      } else {
        #if CYTHON_FAST_PYCALL
        if (PyFunction_Check(__pyx_t_5)) {
          PyObject *__pyx_temp[2] = {__pyx_t_6, __pyx_t_7};
          __pyx_t_4 = __Pyx_PyFunction_FastCall(__pyx_t_5, __pyx_temp+1-1, 1+1); if (unlikely(!__pyx_t_4)) __PYX_ERR(6, 133, __pyx_L3_error)
          __Pyx_XDECREF(__pyx_t_6); __pyx_t_6 = 0;
          __Pyx_GOTREF(__pyx_t_4);
          __Pyx_DECREF(__pyx_t_7); __pyx_t_7 = 0;
        } else
        #endif
        #if CYTHON_FAST_PYCCALL
        if (__Pyx_PyFastCFunction_Check(__pyx_t_5)) {
          PyObject *__pyx_temp[2] = {__pyx_t_6, __pyx_t_7};
          __pyx_t_4 = __Pyx_PyCFunction_FastCall(__pyx_t_5, __pyx_temp+1-1, 1+1); if (unlikely(!__pyx_t_4)) __PYX_ERR(6, 133, __pyx_L3_error)
          __Pyx_XDECREF(__pyx_t_6); __pyx_t_6 = 0;
          __Pyx_GOTREF(__pyx_t_4);
          __Pyx_DECREF(__pyx_t_7); __pyx_t_7 = 0;
        } else
        #endif
        {
          __pyx_t_8 = PyTuple_New(1+1); if (unlikely(!__pyx_t_8)) __PYX_ERR(6, 133, __pyx_L3_error)
          __Pyx_GOTREF(__pyx_t_8);
          __Pyx_GIVEREF(__pyx_t_6); PyTuple_SET_ITEM(__pyx_t_8, 0, __pyx_t_6); __pyx_t_6 = NULL;
          __Pyx_GIVEREF(__pyx_t_7);
          PyTuple_SET_ITEM(__pyx_t_8, 0+1, __pyx_t_7);
          __pyx_t_7 = 0;
          __pyx_t_4 = __Pyx_PyObject_Call(__pyx_t_5, __pyx_t_8, NULL); if (unlikely(!__pyx_t_4)) __PYX_ERR(6, 133, __pyx_L3_error)
          __Pyx_GOTREF(__pyx_t_4);
          __Pyx_DECREF(__pyx_t_8); __pyx_t_8 = 0;
        }
      }
      __Pyx_DECREF(__pyx_t_5); __pyx_t_5 = 0;
      __Pyx_DECREF(__pyx_t_4); __pyx_t_4 = 0;

      /* "src/python/grpcio/grpc/_cython/_cygrpc/grpc_gevent.pyx.pxi":132
 * 
 * cdef void socket_shutdown(grpc_custom_socket* socket) except * with gil:
 *   try:             # <<<<<<<<<<<<<<
 *     (<SocketWrapper>socket.impl).socket.shutdown(gevent_socket.SHUT_RDWR)
 *   except IOError as io_error:
 */
    }
    __Pyx_XDECREF(__pyx_t_1); __pyx_t_1 = 0;
    __Pyx_XDECREF(__pyx_t_2); __pyx_t_2 = 0;
    __Pyx_XDECREF(__pyx_t_3); __pyx_t_3 = 0;
    goto __pyx_L8_try_end;
    __pyx_L3_error:;
    __Pyx_XDECREF(__pyx_t_6); __pyx_t_6 = 0;
    __Pyx_XDECREF(__pyx_t_7); __pyx_t_7 = 0;
    __Pyx_XDECREF(__pyx_t_8); __pyx_t_8 = 0;
    __Pyx_XDECREF(__pyx_t_5); __pyx_t_5 = 0;
    __Pyx_XDECREF(__pyx_t_4); __pyx_t_4 = 0;

    /* "src/python/grpcio/grpc/_cython/_cygrpc/grpc_gevent.pyx.pxi":134
 *   try:
 *     (<SocketWrapper>socket.impl).socket.shutdown(gevent_socket.SHUT_RDWR)
 *   except IOError as io_error:             # <<<<<<<<<<<<<<
 *     if io_error.errno != errno.ENOTCONN:
 *       raise io_error
 */
    __pyx_t_9 = __Pyx_PyErr_ExceptionMatches(__pyx_builtin_IOError);
    if (__pyx_t_9) {
      __Pyx_AddTraceback("grpc._cython.cygrpc.socket_shutdown", __pyx_clineno, __pyx_lineno, __pyx_filename);
      if (__Pyx_GetException(&__pyx_t_4, &__pyx_t_5, &__pyx_t_8) < 0) __PYX_ERR(6, 134, __pyx_L5_except_error)
      __Pyx_GOTREF(__pyx_t_4);
      __Pyx_GOTREF(__pyx_t_5);
      __Pyx_GOTREF(__pyx_t_8);
      __Pyx_INCREF(__pyx_t_5);
      __pyx_v_io_error = __pyx_t_5;

      /* "src/python/grpcio/grpc/_cython/_cygrpc/grpc_gevent.pyx.pxi":135
 *     (<SocketWrapper>socket.impl).socket.shutdown(gevent_socket.SHUT_RDWR)
 *   except IOError as io_error:
 *     if io_error.errno != errno.ENOTCONN:             # <<<<<<<<<<<<<<
 *       raise io_error
 * 
 */
      __pyx_t_7 = __Pyx_PyObject_GetAttrStr(__pyx_v_io_error, __pyx_n_s_errno); if (unlikely(!__pyx_t_7)) __PYX_ERR(6, 135, __pyx_L5_except_error)
      __Pyx_GOTREF(__pyx_t_7);
      __pyx_t_6 = __Pyx_GetModuleGlobalName(__pyx_n_s_errno); if (unlikely(!__pyx_t_6)) __PYX_ERR(6, 135, __pyx_L5_except_error)
      __Pyx_GOTREF(__pyx_t_6);
      __pyx_t_10 = __Pyx_PyObject_GetAttrStr(__pyx_t_6, __pyx_n_s_ENOTCONN); if (unlikely(!__pyx_t_10)) __PYX_ERR(6, 135, __pyx_L5_except_error)
      __Pyx_GOTREF(__pyx_t_10);
      __Pyx_DECREF(__pyx_t_6); __pyx_t_6 = 0;
      __pyx_t_6 = PyObject_RichCompare(__pyx_t_7, __pyx_t_10, Py_NE); __Pyx_XGOTREF(__pyx_t_6); if (unlikely(!__pyx_t_6)) __PYX_ERR(6, 135, __pyx_L5_except_error)
      __Pyx_DECREF(__pyx_t_7); __pyx_t_7 = 0;
      __Pyx_DECREF(__pyx_t_10); __pyx_t_10 = 0;
      __pyx_t_11 = __Pyx_PyObject_IsTrue(__pyx_t_6); if (unlikely(__pyx_t_11 < 0)) __PYX_ERR(6, 135, __pyx_L5_except_error)
      __Pyx_DECREF(__pyx_t_6); __pyx_t_6 = 0;
      if (unlikely(__pyx_t_11)) {

        /* "src/python/grpcio/grpc/_cython/_cygrpc/grpc_gevent.pyx.pxi":136
 *   except IOError as io_error:
 *     if io_error.errno != errno.ENOTCONN:
 *       raise io_error             # <<<<<<<<<<<<<<
 * 
 * cdef void socket_close(grpc_custom_socket* socket,
 */
        __Pyx_Raise(__pyx_v_io_error, 0, 0, 0);
        __PYX_ERR(6, 136, __pyx_L5_except_error)

        /* "src/python/grpcio/grpc/_cython/_cygrpc/grpc_gevent.pyx.pxi":135
 *     (<SocketWrapper>socket.impl).socket.shutdown(gevent_socket.SHUT_RDWR)
 *   except IOError as io_error:
 *     if io_error.errno != errno.ENOTCONN:             # <<<<<<<<<<<<<<
 *       raise io_error
 * 
 */
      }
      __Pyx_DECREF(__pyx_t_4); __pyx_t_4 = 0;
      __Pyx_DECREF(__pyx_t_5); __pyx_t_5 = 0;
      __Pyx_DECREF(__pyx_t_8); __pyx_t_8 = 0;
      goto __pyx_L4_exception_handled;
    }
    goto __pyx_L5_except_error;
    __pyx_L5_except_error:;

    /* "src/python/grpcio/grpc/_cython/_cygrpc/grpc_gevent.pyx.pxi":132
 * 
 * cdef void socket_shutdown(grpc_custom_socket* socket) except * with gil:
 *   try:             # <<<<<<<<<<<<<<
 *     (<SocketWrapper>socket.impl).socket.shutdown(gevent_socket.SHUT_RDWR)
 *   except IOError as io_error:
 */
    __Pyx_XGIVEREF(__pyx_t_1);
    __Pyx_XGIVEREF(__pyx_t_2);
    __Pyx_XGIVEREF(__pyx_t_3);
    __Pyx_ExceptionReset(__pyx_t_1, __pyx_t_2, __pyx_t_3);
    goto __pyx_L1_error;
    __pyx_L4_exception_handled:;
    __Pyx_XGIVEREF(__pyx_t_1);
    __Pyx_XGIVEREF(__pyx_t_2);
    __Pyx_XGIVEREF(__pyx_t_3);
    __Pyx_ExceptionReset(__pyx_t_1, __pyx_t_2, __pyx_t_3);
    __pyx_L8_try_end:;
  }

  /* "src/python/grpcio/grpc/_cython/_cygrpc/grpc_gevent.pyx.pxi":131
 *   cpython.Py_DECREF(<SocketWrapper>socket.impl)
 * 
 * cdef void socket_shutdown(grpc_custom_socket* socket) except * with gil:             # <<<<<<<<<<<<<<
 *   try:
 *     (<SocketWrapper>socket.impl).socket.shutdown(gevent_socket.SHUT_RDWR)
 */

  /* function exit code */
  goto __pyx_L0;
  __pyx_L1_error:;
  __Pyx_XDECREF(__pyx_t_4);
  __Pyx_XDECREF(__pyx_t_5);
  __Pyx_XDECREF(__pyx_t_6);
  __Pyx_XDECREF(__pyx_t_7);
  __Pyx_XDECREF(__pyx_t_8);
  __Pyx_XDECREF(__pyx_t_10);
  __Pyx_AddTraceback("grpc._cython.cygrpc.socket_shutdown", __pyx_clineno, __pyx_lineno, __pyx_filename);
  __pyx_L0:;
  __Pyx_XDECREF(__pyx_v_io_error);
  __Pyx_RefNannyFinishContext();
  #ifdef WITH_THREAD
  __Pyx_PyGILState_Release(__pyx_gilstate_save);
  #endif
}

/* "src/python/grpcio/grpc/_cython/_cygrpc/grpc_gevent.pyx.pxi":138
 *       raise io_error
 * 
 * cdef void socket_close(grpc_custom_socket* socket,             # <<<<<<<<<<<<<<
 *                        grpc_custom_close_callback cb) except * with gil:
 *   socket_wrapper = (<SocketWrapper>socket.impl)
 */

static void __pyx_f_4grpc_7_cython_6cygrpc_socket_close(struct grpc_custom_socket *__pyx_v_socket, grpc_custom_close_callback __pyx_v_cb) {
  struct __pyx_obj_4grpc_7_cython_6cygrpc_SocketWrapper *__pyx_v_socket_wrapper = NULL;
  __Pyx_RefNannyDeclarations
  PyObject *__pyx_t_1 = NULL;
  int __pyx_t_2;
  int __pyx_t_3;
  PyObject *__pyx_t_4 = NULL;
  PyObject *__pyx_t_5 = NULL;
  #ifdef WITH_THREAD
  PyGILState_STATE __pyx_gilstate_save = __Pyx_PyGILState_Ensure();
  #endif
  __Pyx_RefNannySetupContext("socket_close", 0);

  /* "src/python/grpcio/grpc/_cython/_cygrpc/grpc_gevent.pyx.pxi":140
 * cdef void socket_close(grpc_custom_socket* socket,
 *                        grpc_custom_close_callback cb) except * with gil:
 *   socket_wrapper = (<SocketWrapper>socket.impl)             # <<<<<<<<<<<<<<
 *   if socket_wrapper.socket is not None:
 *     socket_wrapper.socket.close()
 */
  __pyx_t_1 = ((PyObject *)__pyx_v_socket->impl);
  __Pyx_INCREF(__pyx_t_1);
  __pyx_v_socket_wrapper = ((struct __pyx_obj_4grpc_7_cython_6cygrpc_SocketWrapper *)__pyx_t_1);
  __pyx_t_1 = 0;

  /* "src/python/grpcio/grpc/_cython/_cygrpc/grpc_gevent.pyx.pxi":141
 *                        grpc_custom_close_callback cb) except * with gil:
 *   socket_wrapper = (<SocketWrapper>socket.impl)
 *   if socket_wrapper.socket is not None:             # <<<<<<<<<<<<<<
 *     socket_wrapper.socket.close()
 *     socket_wrapper.closed = True
 */
  __pyx_t_2 = (__pyx_v_socket_wrapper->socket != Py_None);
  __pyx_t_3 = (__pyx_t_2 != 0);
  if (__pyx_t_3) {

    /* "src/python/grpcio/grpc/_cython/_cygrpc/grpc_gevent.pyx.pxi":142
 *   socket_wrapper = (<SocketWrapper>socket.impl)
 *   if socket_wrapper.socket is not None:
 *     socket_wrapper.socket.close()             # <<<<<<<<<<<<<<
 *     socket_wrapper.closed = True
 *     socket_wrapper.close_cb = cb
 */
    __pyx_t_4 = __Pyx_PyObject_GetAttrStr(__pyx_v_socket_wrapper->socket, __pyx_n_s_close); if (unlikely(!__pyx_t_4)) __PYX_ERR(6, 142, __pyx_L1_error)
    __Pyx_GOTREF(__pyx_t_4);
    __pyx_t_5 = NULL;
    if (CYTHON_UNPACK_METHODS && likely(PyMethod_Check(__pyx_t_4))) {
      __pyx_t_5 = PyMethod_GET_SELF(__pyx_t_4);
      if (likely(__pyx_t_5)) {
        PyObject* function = PyMethod_GET_FUNCTION(__pyx_t_4);
        __Pyx_INCREF(__pyx_t_5);
        __Pyx_INCREF(function);
        __Pyx_DECREF_SET(__pyx_t_4, function);
      }
    }
    if (__pyx_t_5) {
      __pyx_t_1 = __Pyx_PyObject_CallOneArg(__pyx_t_4, __pyx_t_5); if (unlikely(!__pyx_t_1)) __PYX_ERR(6, 142, __pyx_L1_error)
      __Pyx_DECREF(__pyx_t_5); __pyx_t_5 = 0;
    } else {
      __pyx_t_1 = __Pyx_PyObject_CallNoArg(__pyx_t_4); if (unlikely(!__pyx_t_1)) __PYX_ERR(6, 142, __pyx_L1_error)
    }
    __Pyx_GOTREF(__pyx_t_1);
    __Pyx_DECREF(__pyx_t_4); __pyx_t_4 = 0;
    __Pyx_DECREF(__pyx_t_1); __pyx_t_1 = 0;

    /* "src/python/grpcio/grpc/_cython/_cygrpc/grpc_gevent.pyx.pxi":143
 *   if socket_wrapper.socket is not None:
 *     socket_wrapper.socket.close()
 *     socket_wrapper.closed = True             # <<<<<<<<<<<<<<
 *     socket_wrapper.close_cb = cb
 *     # Delay the close callback until the accept() call has picked it up
 */
    __Pyx_INCREF(Py_True);
    __Pyx_GIVEREF(Py_True);
    __Pyx_GOTREF(__pyx_v_socket_wrapper->closed);
    __Pyx_DECREF(__pyx_v_socket_wrapper->closed);
    __pyx_v_socket_wrapper->closed = Py_True;

    /* "src/python/grpcio/grpc/_cython/_cygrpc/grpc_gevent.pyx.pxi":144
 *     socket_wrapper.socket.close()
 *     socket_wrapper.closed = True
 *     socket_wrapper.close_cb = cb             # <<<<<<<<<<<<<<
 *     # Delay the close callback until the accept() call has picked it up
 *     if socket_wrapper.accepting_socket != NULL:
 */
    __pyx_v_socket_wrapper->close_cb = __pyx_v_cb;

    /* "src/python/grpcio/grpc/_cython/_cygrpc/grpc_gevent.pyx.pxi":146
 *     socket_wrapper.close_cb = cb
 *     # Delay the close callback until the accept() call has picked it up
 *     if socket_wrapper.accepting_socket != NULL:             # <<<<<<<<<<<<<<
 *       return
 *   socket_wrapper.close_cb(socket)
 */
    __pyx_t_3 = ((__pyx_v_socket_wrapper->accepting_socket != NULL) != 0);
    if (__pyx_t_3) {

      /* "src/python/grpcio/grpc/_cython/_cygrpc/grpc_gevent.pyx.pxi":147
 *     # Delay the close callback until the accept() call has picked it up
 *     if socket_wrapper.accepting_socket != NULL:
 *       return             # <<<<<<<<<<<<<<
 *   socket_wrapper.close_cb(socket)
 * 
 */
      goto __pyx_L0;

      /* "src/python/grpcio/grpc/_cython/_cygrpc/grpc_gevent.pyx.pxi":146
 *     socket_wrapper.close_cb = cb
 *     # Delay the close callback until the accept() call has picked it up
 *     if socket_wrapper.accepting_socket != NULL:             # <<<<<<<<<<<<<<
 *       return
 *   socket_wrapper.close_cb(socket)
 */
    }

    /* "src/python/grpcio/grpc/_cython/_cygrpc/grpc_gevent.pyx.pxi":141
 *                        grpc_custom_close_callback cb) except * with gil:
 *   socket_wrapper = (<SocketWrapper>socket.impl)
 *   if socket_wrapper.socket is not None:             # <<<<<<<<<<<<<<
 *     socket_wrapper.socket.close()
 *     socket_wrapper.closed = True
 */
  }

  /* "src/python/grpcio/grpc/_cython/_cygrpc/grpc_gevent.pyx.pxi":148
 *     if socket_wrapper.accepting_socket != NULL:
 *       return
 *   socket_wrapper.close_cb(socket)             # <<<<<<<<<<<<<<
 * 
 * def socket_sendmsg(socket, write_bytes):
 */
  __pyx_v_socket_wrapper->close_cb(__pyx_v_socket);

  /* "src/python/grpcio/grpc/_cython/_cygrpc/grpc_gevent.pyx.pxi":138
 *       raise io_error
 * 
 * cdef void socket_close(grpc_custom_socket* socket,             # <<<<<<<<<<<<<<
 *                        grpc_custom_close_callback cb) except * with gil:
 *   socket_wrapper = (<SocketWrapper>socket.impl)
 */

  /* function exit code */
  goto __pyx_L0;
  __pyx_L1_error:;
  __Pyx_XDECREF(__pyx_t_1);
  __Pyx_XDECREF(__pyx_t_4);
  __Pyx_XDECREF(__pyx_t_5);
  __Pyx_AddTraceback("grpc._cython.cygrpc.socket_close", __pyx_clineno, __pyx_lineno, __pyx_filename);
  __pyx_L0:;
  __Pyx_XDECREF((PyObject *)__pyx_v_socket_wrapper);
  __Pyx_RefNannyFinishContext();
  #ifdef WITH_THREAD
  __Pyx_PyGILState_Release(__pyx_gilstate_save);
  #endif
}

/* "src/python/grpcio/grpc/_cython/_cygrpc/grpc_gevent.pyx.pxi":150
 *   socket_wrapper.close_cb(socket)
 * 
 * def socket_sendmsg(socket, write_bytes):             # <<<<<<<<<<<<<<
 *   try:
 *     return socket.sendmsg(write_bytes)
 */

/* Python wrapper */
static PyObject *__pyx_pw_4grpc_7_cython_6cygrpc_49socket_sendmsg(PyObject *__pyx_self, PyObject *__pyx_args, PyObject *__pyx_kwds); /*proto*/
static PyMethodDef __pyx_mdef_4grpc_7_cython_6cygrpc_49socket_sendmsg = {"socket_sendmsg", (PyCFunction)__pyx_pw_4grpc_7_cython_6cygrpc_49socket_sendmsg, METH_VARARGS|METH_KEYWORDS, 0};
static PyObject *__pyx_pw_4grpc_7_cython_6cygrpc_49socket_sendmsg(PyObject *__pyx_self, PyObject *__pyx_args, PyObject *__pyx_kwds) {
  PyObject *__pyx_v_socket = 0;
  PyObject *__pyx_v_write_bytes = 0;
  PyObject *__pyx_r = 0;
  __Pyx_RefNannyDeclarations
  __Pyx_RefNannySetupContext("socket_sendmsg (wrapper)", 0);
  {
    static PyObject **__pyx_pyargnames[] = {&__pyx_n_s_socket,&__pyx_n_s_write_bytes,0};
    PyObject* values[2] = {0,0};
    if (unlikely(__pyx_kwds)) {
      Py_ssize_t kw_args;
      const Py_ssize_t pos_args = PyTuple_GET_SIZE(__pyx_args);
      switch (pos_args) {
        case  2: values[1] = PyTuple_GET_ITEM(__pyx_args, 1);
        CYTHON_FALLTHROUGH;
        case  1: values[0] = PyTuple_GET_ITEM(__pyx_args, 0);
        CYTHON_FALLTHROUGH;
        case  0: break;
        default: goto __pyx_L5_argtuple_error;
      }
      kw_args = PyDict_Size(__pyx_kwds);
      switch (pos_args) {
        case  0:
        if (likely((values[0] = __Pyx_PyDict_GetItemStr(__pyx_kwds, __pyx_n_s_socket)) != 0)) kw_args--;
        else goto __pyx_L5_argtuple_error;
        CYTHON_FALLTHROUGH;
        case  1:
        if (likely((values[1] = __Pyx_PyDict_GetItemStr(__pyx_kwds, __pyx_n_s_write_bytes)) != 0)) kw_args--;
        else {
          __Pyx_RaiseArgtupleInvalid("socket_sendmsg", 1, 2, 2, 1); __PYX_ERR(6, 150, __pyx_L3_error)
        }
      }
      if (unlikely(kw_args > 0)) {
        if (unlikely(__Pyx_ParseOptionalKeywords(__pyx_kwds, __pyx_pyargnames, 0, values, pos_args, "socket_sendmsg") < 0)) __PYX_ERR(6, 150, __pyx_L3_error)
      }
    } else if (PyTuple_GET_SIZE(__pyx_args) != 2) {
      goto __pyx_L5_argtuple_error;
    } else {
      values[0] = PyTuple_GET_ITEM(__pyx_args, 0);
      values[1] = PyTuple_GET_ITEM(__pyx_args, 1);
    }
    __pyx_v_socket = values[0];
    __pyx_v_write_bytes = values[1];
  }
  goto __pyx_L4_argument_unpacking_done;
  __pyx_L5_argtuple_error:;
  __Pyx_RaiseArgtupleInvalid("socket_sendmsg", 1, 2, 2, PyTuple_GET_SIZE(__pyx_args)); __PYX_ERR(6, 150, __pyx_L3_error)
  __pyx_L3_error:;
  __Pyx_AddTraceback("grpc._cython.cygrpc.socket_sendmsg", __pyx_clineno, __pyx_lineno, __pyx_filename);
  __Pyx_RefNannyFinishContext();
  return NULL;
  __pyx_L4_argument_unpacking_done:;
  __pyx_r = __pyx_pf_4grpc_7_cython_6cygrpc_48socket_sendmsg(__pyx_self, __pyx_v_socket, __pyx_v_write_bytes);

  /* function exit code */
  __Pyx_RefNannyFinishContext();
  return __pyx_r;
}

static PyObject *__pyx_pf_4grpc_7_cython_6cygrpc_48socket_sendmsg(CYTHON_UNUSED PyObject *__pyx_self, PyObject *__pyx_v_socket, PyObject *__pyx_v_write_bytes) {
  PyObject *__pyx_r = NULL;
  __Pyx_RefNannyDeclarations
  PyObject *__pyx_t_1 = NULL;
  PyObject *__pyx_t_2 = NULL;
  PyObject *__pyx_t_3 = NULL;
  PyObject *__pyx_t_4 = NULL;
  PyObject *__pyx_t_5 = NULL;
  PyObject *__pyx_t_6 = NULL;
  PyObject *__pyx_t_7 = NULL;
  int __pyx_t_8;
  PyObject *__pyx_t_9 = NULL;
  PyObject *__pyx_t_10 = NULL;
  PyObject *__pyx_t_11 = NULL;
  PyObject *__pyx_t_12 = NULL;
  __Pyx_RefNannySetupContext("socket_sendmsg", 0);

  /* "src/python/grpcio/grpc/_cython/_cygrpc/grpc_gevent.pyx.pxi":151
 * 
 * def socket_sendmsg(socket, write_bytes):
 *   try:             # <<<<<<<<<<<<<<
 *     return socket.sendmsg(write_bytes)
 *   except AttributeError:
 */
  {
    __Pyx_PyThreadState_declare
    __Pyx_PyThreadState_assign
    __Pyx_ExceptionSave(&__pyx_t_1, &__pyx_t_2, &__pyx_t_3);
    __Pyx_XGOTREF(__pyx_t_1);
    __Pyx_XGOTREF(__pyx_t_2);
    __Pyx_XGOTREF(__pyx_t_3);
    /*try:*/ {

      /* "src/python/grpcio/grpc/_cython/_cygrpc/grpc_gevent.pyx.pxi":152
 * def socket_sendmsg(socket, write_bytes):
 *   try:
 *     return socket.sendmsg(write_bytes)             # <<<<<<<<<<<<<<
 *   except AttributeError:
 *     # sendmsg not available on all Pythons/Platforms
 */
      __Pyx_XDECREF(__pyx_r);
      __pyx_t_5 = __Pyx_PyObject_GetAttrStr(__pyx_v_socket, __pyx_n_s_sendmsg); if (unlikely(!__pyx_t_5)) __PYX_ERR(6, 152, __pyx_L3_error)
      __Pyx_GOTREF(__pyx_t_5);
      __pyx_t_6 = NULL;
      if (CYTHON_UNPACK_METHODS && likely(PyMethod_Check(__pyx_t_5))) {
        __pyx_t_6 = PyMethod_GET_SELF(__pyx_t_5);
        if (likely(__pyx_t_6)) {
          PyObject* function = PyMethod_GET_FUNCTION(__pyx_t_5);
          __Pyx_INCREF(__pyx_t_6);
          __Pyx_INCREF(function);
          __Pyx_DECREF_SET(__pyx_t_5, function);
        }
      }
      if (!__pyx_t_6) {
        __pyx_t_4 = __Pyx_PyObject_CallOneArg(__pyx_t_5, __pyx_v_write_bytes); if (unlikely(!__pyx_t_4)) __PYX_ERR(6, 152, __pyx_L3_error)
        __Pyx_GOTREF(__pyx_t_4);
      } else {
        #if CYTHON_FAST_PYCALL
        if (PyFunction_Check(__pyx_t_5)) {
          PyObject *__pyx_temp[2] = {__pyx_t_6, __pyx_v_write_bytes};
          __pyx_t_4 = __Pyx_PyFunction_FastCall(__pyx_t_5, __pyx_temp+1-1, 1+1); if (unlikely(!__pyx_t_4)) __PYX_ERR(6, 152, __pyx_L3_error)
          __Pyx_XDECREF(__pyx_t_6); __pyx_t_6 = 0;
          __Pyx_GOTREF(__pyx_t_4);
        } else
        #endif
        #if CYTHON_FAST_PYCCALL
        if (__Pyx_PyFastCFunction_Check(__pyx_t_5)) {
          PyObject *__pyx_temp[2] = {__pyx_t_6, __pyx_v_write_bytes};
          __pyx_t_4 = __Pyx_PyCFunction_FastCall(__pyx_t_5, __pyx_temp+1-1, 1+1); if (unlikely(!__pyx_t_4)) __PYX_ERR(6, 152, __pyx_L3_error)
          __Pyx_XDECREF(__pyx_t_6); __pyx_t_6 = 0;
          __Pyx_GOTREF(__pyx_t_4);
        } else
        #endif
        {
          __pyx_t_7 = PyTuple_New(1+1); if (unlikely(!__pyx_t_7)) __PYX_ERR(6, 152, __pyx_L3_error)
          __Pyx_GOTREF(__pyx_t_7);
          __Pyx_GIVEREF(__pyx_t_6); PyTuple_SET_ITEM(__pyx_t_7, 0, __pyx_t_6); __pyx_t_6 = NULL;
          __Pyx_INCREF(__pyx_v_write_bytes);
          __Pyx_GIVEREF(__pyx_v_write_bytes);
          PyTuple_SET_ITEM(__pyx_t_7, 0+1, __pyx_v_write_bytes);
          __pyx_t_4 = __Pyx_PyObject_Call(__pyx_t_5, __pyx_t_7, NULL); if (unlikely(!__pyx_t_4)) __PYX_ERR(6, 152, __pyx_L3_error)
          __Pyx_GOTREF(__pyx_t_4);
          __Pyx_DECREF(__pyx_t_7); __pyx_t_7 = 0;
        }
      }
      __Pyx_DECREF(__pyx_t_5); __pyx_t_5 = 0;
      __pyx_r = __pyx_t_4;
      __pyx_t_4 = 0;
      goto __pyx_L7_try_return;

      /* "src/python/grpcio/grpc/_cython/_cygrpc/grpc_gevent.pyx.pxi":151
 * 
 * def socket_sendmsg(socket, write_bytes):
 *   try:             # <<<<<<<<<<<<<<
 *     return socket.sendmsg(write_bytes)
 *   except AttributeError:
 */
    }
    __pyx_L3_error:;
    __Pyx_XDECREF(__pyx_t_6); __pyx_t_6 = 0;
    __Pyx_XDECREF(__pyx_t_7); __pyx_t_7 = 0;
    __Pyx_XDECREF(__pyx_t_5); __pyx_t_5 = 0;
    __Pyx_XDECREF(__pyx_t_4); __pyx_t_4 = 0;

    /* "src/python/grpcio/grpc/_cython/_cygrpc/grpc_gevent.pyx.pxi":153
 *   try:
 *     return socket.sendmsg(write_bytes)
 *   except AttributeError:             # <<<<<<<<<<<<<<
 *     # sendmsg not available on all Pythons/Platforms
 *     return socket.send(b''.join(write_bytes))
 */
    __pyx_t_8 = __Pyx_PyErr_ExceptionMatches(__pyx_builtin_AttributeError);
    if (__pyx_t_8) {
      __Pyx_AddTraceback("grpc._cython.cygrpc.socket_sendmsg", __pyx_clineno, __pyx_lineno, __pyx_filename);
      if (__Pyx_GetException(&__pyx_t_4, &__pyx_t_5, &__pyx_t_7) < 0) __PYX_ERR(6, 153, __pyx_L5_except_error)
      __Pyx_GOTREF(__pyx_t_4);
      __Pyx_GOTREF(__pyx_t_5);
      __Pyx_GOTREF(__pyx_t_7);

      /* "src/python/grpcio/grpc/_cython/_cygrpc/grpc_gevent.pyx.pxi":155
 *   except AttributeError:
 *     # sendmsg not available on all Pythons/Platforms
 *     return socket.send(b''.join(write_bytes))             # <<<<<<<<<<<<<<
 * 
 * cdef socket_write_async_cython(SocketWrapper socket_wrapper, write_bytes):
 */
      __Pyx_XDECREF(__pyx_r);
      __pyx_t_9 = __Pyx_PyObject_GetAttrStr(__pyx_v_socket, __pyx_n_s_send); if (unlikely(!__pyx_t_9)) __PYX_ERR(6, 155, __pyx_L5_except_error)
      __Pyx_GOTREF(__pyx_t_9);
      __pyx_t_10 = __Pyx_PyBytes_Join(__pyx_kp_b__2, __pyx_v_write_bytes); if (unlikely(!__pyx_t_10)) __PYX_ERR(6, 155, __pyx_L5_except_error)
      __Pyx_GOTREF(__pyx_t_10);
      __pyx_t_11 = NULL;
      if (CYTHON_UNPACK_METHODS && likely(PyMethod_Check(__pyx_t_9))) {
        __pyx_t_11 = PyMethod_GET_SELF(__pyx_t_9);
        if (likely(__pyx_t_11)) {
          PyObject* function = PyMethod_GET_FUNCTION(__pyx_t_9);
          __Pyx_INCREF(__pyx_t_11);
          __Pyx_INCREF(function);
          __Pyx_DECREF_SET(__pyx_t_9, function);
        }
      }
      if (!__pyx_t_11) {
        __pyx_t_6 = __Pyx_PyObject_CallOneArg(__pyx_t_9, __pyx_t_10); if (unlikely(!__pyx_t_6)) __PYX_ERR(6, 155, __pyx_L5_except_error)
        __Pyx_DECREF(__pyx_t_10); __pyx_t_10 = 0;
        __Pyx_GOTREF(__pyx_t_6);
      } else {
        #if CYTHON_FAST_PYCALL
        if (PyFunction_Check(__pyx_t_9)) {
          PyObject *__pyx_temp[2] = {__pyx_t_11, __pyx_t_10};
          __pyx_t_6 = __Pyx_PyFunction_FastCall(__pyx_t_9, __pyx_temp+1-1, 1+1); if (unlikely(!__pyx_t_6)) __PYX_ERR(6, 155, __pyx_L5_except_error)
          __Pyx_XDECREF(__pyx_t_11); __pyx_t_11 = 0;
          __Pyx_GOTREF(__pyx_t_6);
          __Pyx_DECREF(__pyx_t_10); __pyx_t_10 = 0;
        } else
        #endif
        #if CYTHON_FAST_PYCCALL
        if (__Pyx_PyFastCFunction_Check(__pyx_t_9)) {
          PyObject *__pyx_temp[2] = {__pyx_t_11, __pyx_t_10};
          __pyx_t_6 = __Pyx_PyCFunction_FastCall(__pyx_t_9, __pyx_temp+1-1, 1+1); if (unlikely(!__pyx_t_6)) __PYX_ERR(6, 155, __pyx_L5_except_error)
          __Pyx_XDECREF(__pyx_t_11); __pyx_t_11 = 0;
          __Pyx_GOTREF(__pyx_t_6);
          __Pyx_DECREF(__pyx_t_10); __pyx_t_10 = 0;
        } else
        #endif
        {
          __pyx_t_12 = PyTuple_New(1+1); if (unlikely(!__pyx_t_12)) __PYX_ERR(6, 155, __pyx_L5_except_error)
          __Pyx_GOTREF(__pyx_t_12);
          __Pyx_GIVEREF(__pyx_t_11); PyTuple_SET_ITEM(__pyx_t_12, 0, __pyx_t_11); __pyx_t_11 = NULL;
          __Pyx_GIVEREF(__pyx_t_10);
          PyTuple_SET_ITEM(__pyx_t_12, 0+1, __pyx_t_10);
          __pyx_t_10 = 0;
          __pyx_t_6 = __Pyx_PyObject_Call(__pyx_t_9, __pyx_t_12, NULL); if (unlikely(!__pyx_t_6)) __PYX_ERR(6, 155, __pyx_L5_except_error)
          __Pyx_GOTREF(__pyx_t_6);
          __Pyx_DECREF(__pyx_t_12); __pyx_t_12 = 0;
        }
      }
      __Pyx_DECREF(__pyx_t_9); __pyx_t_9 = 0;
      __pyx_r = __pyx_t_6;
      __pyx_t_6 = 0;
      __Pyx_DECREF(__pyx_t_4); __pyx_t_4 = 0;
      __Pyx_DECREF(__pyx_t_5); __pyx_t_5 = 0;
      __Pyx_DECREF(__pyx_t_7); __pyx_t_7 = 0;
      goto __pyx_L6_except_return;
    }
    goto __pyx_L5_except_error;
    __pyx_L5_except_error:;

    /* "src/python/grpcio/grpc/_cython/_cygrpc/grpc_gevent.pyx.pxi":151
 * 
 * def socket_sendmsg(socket, write_bytes):
 *   try:             # <<<<<<<<<<<<<<
 *     return socket.sendmsg(write_bytes)
 *   except AttributeError:
 */
    __Pyx_XGIVEREF(__pyx_t_1);
    __Pyx_XGIVEREF(__pyx_t_2);
    __Pyx_XGIVEREF(__pyx_t_3);
    __Pyx_ExceptionReset(__pyx_t_1, __pyx_t_2, __pyx_t_3);
    goto __pyx_L1_error;
    __pyx_L7_try_return:;
    __Pyx_XGIVEREF(__pyx_t_1);
    __Pyx_XGIVEREF(__pyx_t_2);
    __Pyx_XGIVEREF(__pyx_t_3);
    __Pyx_ExceptionReset(__pyx_t_1, __pyx_t_2, __pyx_t_3);
    goto __pyx_L0;
    __pyx_L6_except_return:;
    __Pyx_XGIVEREF(__pyx_t_1);
    __Pyx_XGIVEREF(__pyx_t_2);
    __Pyx_XGIVEREF(__pyx_t_3);
    __Pyx_ExceptionReset(__pyx_t_1, __pyx_t_2, __pyx_t_3);
    goto __pyx_L0;
  }

  /* "src/python/grpcio/grpc/_cython/_cygrpc/grpc_gevent.pyx.pxi":150
 *   socket_wrapper.close_cb(socket)
 * 
 * def socket_sendmsg(socket, write_bytes):             # <<<<<<<<<<<<<<
 *   try:
 *     return socket.sendmsg(write_bytes)
 */

  /* function exit code */
  __pyx_L1_error:;
  __Pyx_XDECREF(__pyx_t_4);
  __Pyx_XDECREF(__pyx_t_5);
  __Pyx_XDECREF(__pyx_t_6);
  __Pyx_XDECREF(__pyx_t_7);
  __Pyx_XDECREF(__pyx_t_9);
  __Pyx_XDECREF(__pyx_t_10);
  __Pyx_XDECREF(__pyx_t_11);
  __Pyx_XDECREF(__pyx_t_12);
  __Pyx_AddTraceback("grpc._cython.cygrpc.socket_sendmsg", __pyx_clineno, __pyx_lineno, __pyx_filename);
  __pyx_r = NULL;
  __pyx_L0:;
  __Pyx_XGIVEREF(__pyx_r);
  __Pyx_RefNannyFinishContext();
  return __pyx_r;
}

/* "src/python/grpcio/grpc/_cython/_cygrpc/grpc_gevent.pyx.pxi":157
 *     return socket.send(b''.join(write_bytes))
 * 
 * cdef socket_write_async_cython(SocketWrapper socket_wrapper, write_bytes):             # <<<<<<<<<<<<<<
 *   try:
 *     while write_bytes:
 */

static PyObject *__pyx_f_4grpc_7_cython_6cygrpc_socket_write_async_cython(struct __pyx_obj_4grpc_7_cython_6cygrpc_SocketWrapper *__pyx_v_socket_wrapper, PyObject *__pyx_v_write_bytes) {
  PyObject *__pyx_v_sent_byte_count = NULL;
  PyObject *__pyx_v_io_error = NULL;
  PyObject *__pyx_r = NULL;
  __Pyx_RefNannyDeclarations
  PyObject *__pyx_t_1 = NULL;
  PyObject *__pyx_t_2 = NULL;
  PyObject *__pyx_t_3 = NULL;
  int __pyx_t_4;
  PyObject *__pyx_t_5 = NULL;
  PyObject *__pyx_t_6 = NULL;
  PyObject *__pyx_t_7 = NULL;
  int __pyx_t_8;
  PyObject *__pyx_t_9 = NULL;
  Py_ssize_t __pyx_t_10;
  __Pyx_RefNannySetupContext("socket_write_async_cython", 0);
  __Pyx_INCREF(__pyx_v_write_bytes);

  /* "src/python/grpcio/grpc/_cython/_cygrpc/grpc_gevent.pyx.pxi":158
 * 
 * cdef socket_write_async_cython(SocketWrapper socket_wrapper, write_bytes):
 *   try:             # <<<<<<<<<<<<<<
 *     while write_bytes:
 *       sent_byte_count = socket_sendmsg(socket_wrapper.socket, write_bytes)
 */
  {
    __Pyx_PyThreadState_declare
    __Pyx_PyThreadState_assign
    __Pyx_ExceptionSave(&__pyx_t_1, &__pyx_t_2, &__pyx_t_3);
    __Pyx_XGOTREF(__pyx_t_1);
    __Pyx_XGOTREF(__pyx_t_2);
    __Pyx_XGOTREF(__pyx_t_3);
    /*try:*/ {

      /* "src/python/grpcio/grpc/_cython/_cygrpc/grpc_gevent.pyx.pxi":159
 * cdef socket_write_async_cython(SocketWrapper socket_wrapper, write_bytes):
 *   try:
 *     while write_bytes:             # <<<<<<<<<<<<<<
 *       sent_byte_count = socket_sendmsg(socket_wrapper.socket, write_bytes)
 *       while sent_byte_count > 0:
 */
      while (1) {
        __pyx_t_4 = __Pyx_PyObject_IsTrue(__pyx_v_write_bytes); if (unlikely(__pyx_t_4 < 0)) __PYX_ERR(6, 159, __pyx_L3_error)
        if (!__pyx_t_4) break;

        /* "src/python/grpcio/grpc/_cython/_cygrpc/grpc_gevent.pyx.pxi":160
 *   try:
 *     while write_bytes:
 *       sent_byte_count = socket_sendmsg(socket_wrapper.socket, write_bytes)             # <<<<<<<<<<<<<<
 *       while sent_byte_count > 0:
 *         if sent_byte_count < len(write_bytes[0]):
 */
        __pyx_t_6 = __Pyx_GetModuleGlobalName(__pyx_n_s_socket_sendmsg); if (unlikely(!__pyx_t_6)) __PYX_ERR(6, 160, __pyx_L3_error)
        __Pyx_GOTREF(__pyx_t_6);
        __pyx_t_7 = NULL;
        __pyx_t_8 = 0;
        if (CYTHON_UNPACK_METHODS && unlikely(PyMethod_Check(__pyx_t_6))) {
          __pyx_t_7 = PyMethod_GET_SELF(__pyx_t_6);
          if (likely(__pyx_t_7)) {
            PyObject* function = PyMethod_GET_FUNCTION(__pyx_t_6);
            __Pyx_INCREF(__pyx_t_7);
            __Pyx_INCREF(function);
            __Pyx_DECREF_SET(__pyx_t_6, function);
            __pyx_t_8 = 1;
          }
        }
        #if CYTHON_FAST_PYCALL
        if (PyFunction_Check(__pyx_t_6)) {
          PyObject *__pyx_temp[3] = {__pyx_t_7, __pyx_v_socket_wrapper->socket, __pyx_v_write_bytes};
          __pyx_t_5 = __Pyx_PyFunction_FastCall(__pyx_t_6, __pyx_temp+1-__pyx_t_8, 2+__pyx_t_8); if (unlikely(!__pyx_t_5)) __PYX_ERR(6, 160, __pyx_L3_error)
          __Pyx_XDECREF(__pyx_t_7); __pyx_t_7 = 0;
          __Pyx_GOTREF(__pyx_t_5);
        } else
        #endif
        #if CYTHON_FAST_PYCCALL
        if (__Pyx_PyFastCFunction_Check(__pyx_t_6)) {
          PyObject *__pyx_temp[3] = {__pyx_t_7, __pyx_v_socket_wrapper->socket, __pyx_v_write_bytes};
          __pyx_t_5 = __Pyx_PyCFunction_FastCall(__pyx_t_6, __pyx_temp+1-__pyx_t_8, 2+__pyx_t_8); if (unlikely(!__pyx_t_5)) __PYX_ERR(6, 160, __pyx_L3_error)
          __Pyx_XDECREF(__pyx_t_7); __pyx_t_7 = 0;
          __Pyx_GOTREF(__pyx_t_5);
        } else
        #endif
        {
          __pyx_t_9 = PyTuple_New(2+__pyx_t_8); if (unlikely(!__pyx_t_9)) __PYX_ERR(6, 160, __pyx_L3_error)
          __Pyx_GOTREF(__pyx_t_9);
          if (__pyx_t_7) {
            __Pyx_GIVEREF(__pyx_t_7); PyTuple_SET_ITEM(__pyx_t_9, 0, __pyx_t_7); __pyx_t_7 = NULL;
          }
          __Pyx_INCREF(__pyx_v_socket_wrapper->socket);
          __Pyx_GIVEREF(__pyx_v_socket_wrapper->socket);
          PyTuple_SET_ITEM(__pyx_t_9, 0+__pyx_t_8, __pyx_v_socket_wrapper->socket);
          __Pyx_INCREF(__pyx_v_write_bytes);
          __Pyx_GIVEREF(__pyx_v_write_bytes);
          PyTuple_SET_ITEM(__pyx_t_9, 1+__pyx_t_8, __pyx_v_write_bytes);
          __pyx_t_5 = __Pyx_PyObject_Call(__pyx_t_6, __pyx_t_9, NULL); if (unlikely(!__pyx_t_5)) __PYX_ERR(6, 160, __pyx_L3_error)
          __Pyx_GOTREF(__pyx_t_5);
          __Pyx_DECREF(__pyx_t_9); __pyx_t_9 = 0;
        }
        __Pyx_DECREF(__pyx_t_6); __pyx_t_6 = 0;
        __Pyx_XDECREF_SET(__pyx_v_sent_byte_count, __pyx_t_5);
        __pyx_t_5 = 0;

        /* "src/python/grpcio/grpc/_cython/_cygrpc/grpc_gevent.pyx.pxi":161
 *     while write_bytes:
 *       sent_byte_count = socket_sendmsg(socket_wrapper.socket, write_bytes)
 *       while sent_byte_count > 0:             # <<<<<<<<<<<<<<
 *         if sent_byte_count < len(write_bytes[0]):
 *           write_bytes[0] = write_bytes[0][sent_byte_count:]
 */
        while (1) {
          __pyx_t_5 = PyObject_RichCompare(__pyx_v_sent_byte_count, __pyx_int_0, Py_GT); __Pyx_XGOTREF(__pyx_t_5); if (unlikely(!__pyx_t_5)) __PYX_ERR(6, 161, __pyx_L3_error)
          __pyx_t_4 = __Pyx_PyObject_IsTrue(__pyx_t_5); if (unlikely(__pyx_t_4 < 0)) __PYX_ERR(6, 161, __pyx_L3_error)
          __Pyx_DECREF(__pyx_t_5); __pyx_t_5 = 0;
          if (!__pyx_t_4) break;

          /* "src/python/grpcio/grpc/_cython/_cygrpc/grpc_gevent.pyx.pxi":162
 *       sent_byte_count = socket_sendmsg(socket_wrapper.socket, write_bytes)
 *       while sent_byte_count > 0:
 *         if sent_byte_count < len(write_bytes[0]):             # <<<<<<<<<<<<<<
 *           write_bytes[0] = write_bytes[0][sent_byte_count:]
 *           sent_byte_count = 0
 */
          __pyx_t_5 = __Pyx_GetItemInt(__pyx_v_write_bytes, 0, long, 1, __Pyx_PyInt_From_long, 0, 0, 1); if (unlikely(!__pyx_t_5)) __PYX_ERR(6, 162, __pyx_L3_error)
          __Pyx_GOTREF(__pyx_t_5);
          __pyx_t_10 = PyObject_Length(__pyx_t_5); if (unlikely(__pyx_t_10 == ((Py_ssize_t)-1))) __PYX_ERR(6, 162, __pyx_L3_error)
          __Pyx_DECREF(__pyx_t_5); __pyx_t_5 = 0;
          __pyx_t_5 = PyInt_FromSsize_t(__pyx_t_10); if (unlikely(!__pyx_t_5)) __PYX_ERR(6, 162, __pyx_L3_error)
          __Pyx_GOTREF(__pyx_t_5);
          __pyx_t_6 = PyObject_RichCompare(__pyx_v_sent_byte_count, __pyx_t_5, Py_LT); __Pyx_XGOTREF(__pyx_t_6); if (unlikely(!__pyx_t_6)) __PYX_ERR(6, 162, __pyx_L3_error)
          __Pyx_DECREF(__pyx_t_5); __pyx_t_5 = 0;
          __pyx_t_4 = __Pyx_PyObject_IsTrue(__pyx_t_6); if (unlikely(__pyx_t_4 < 0)) __PYX_ERR(6, 162, __pyx_L3_error)
          __Pyx_DECREF(__pyx_t_6); __pyx_t_6 = 0;
          if (__pyx_t_4) {

            /* "src/python/grpcio/grpc/_cython/_cygrpc/grpc_gevent.pyx.pxi":163
 *       while sent_byte_count > 0:
 *         if sent_byte_count < len(write_bytes[0]):
 *           write_bytes[0] = write_bytes[0][sent_byte_count:]             # <<<<<<<<<<<<<<
 *           sent_byte_count = 0
 *         else:
 */
            __pyx_t_6 = __Pyx_GetItemInt(__pyx_v_write_bytes, 0, long, 1, __Pyx_PyInt_From_long, 0, 0, 1); if (unlikely(!__pyx_t_6)) __PYX_ERR(6, 163, __pyx_L3_error)
            __Pyx_GOTREF(__pyx_t_6);
            __pyx_t_5 = __Pyx_PyObject_GetSlice(__pyx_t_6, 0, 0, &__pyx_v_sent_byte_count, NULL, NULL, 0, 0, 1); if (unlikely(!__pyx_t_5)) __PYX_ERR(6, 163, __pyx_L3_error)
            __Pyx_GOTREF(__pyx_t_5);
            __Pyx_DECREF(__pyx_t_6); __pyx_t_6 = 0;
            if (unlikely(__Pyx_SetItemInt(__pyx_v_write_bytes, 0, __pyx_t_5, long, 1, __Pyx_PyInt_From_long, 0, 0, 1) < 0)) __PYX_ERR(6, 163, __pyx_L3_error)
            __Pyx_DECREF(__pyx_t_5); __pyx_t_5 = 0;

            /* "src/python/grpcio/grpc/_cython/_cygrpc/grpc_gevent.pyx.pxi":164
 *         if sent_byte_count < len(write_bytes[0]):
 *           write_bytes[0] = write_bytes[0][sent_byte_count:]
 *           sent_byte_count = 0             # <<<<<<<<<<<<<<
 *         else:
 *           sent_byte_count -= len(write_bytes[0])
 */
            __Pyx_INCREF(__pyx_int_0);
            __Pyx_DECREF_SET(__pyx_v_sent_byte_count, __pyx_int_0);

            /* "src/python/grpcio/grpc/_cython/_cygrpc/grpc_gevent.pyx.pxi":162
 *       sent_byte_count = socket_sendmsg(socket_wrapper.socket, write_bytes)
 *       while sent_byte_count > 0:
 *         if sent_byte_count < len(write_bytes[0]):             # <<<<<<<<<<<<<<
 *           write_bytes[0] = write_bytes[0][sent_byte_count:]
 *           sent_byte_count = 0
 */
            goto __pyx_L13;
          }

          /* "src/python/grpcio/grpc/_cython/_cygrpc/grpc_gevent.pyx.pxi":166
 *           sent_byte_count = 0
 *         else:
 *           sent_byte_count -= len(write_bytes[0])             # <<<<<<<<<<<<<<
 *           write_bytes = write_bytes[1:]
 *     socket_wrapper.write_cb(<grpc_custom_socket*>socket_wrapper.c_socket,
 */
          /*else*/ {
            __pyx_t_5 = __Pyx_GetItemInt(__pyx_v_write_bytes, 0, long, 1, __Pyx_PyInt_From_long, 0, 0, 1); if (unlikely(!__pyx_t_5)) __PYX_ERR(6, 166, __pyx_L3_error)
            __Pyx_GOTREF(__pyx_t_5);
            __pyx_t_10 = PyObject_Length(__pyx_t_5); if (unlikely(__pyx_t_10 == ((Py_ssize_t)-1))) __PYX_ERR(6, 166, __pyx_L3_error)
            __Pyx_DECREF(__pyx_t_5); __pyx_t_5 = 0;
            __pyx_t_5 = PyInt_FromSsize_t(__pyx_t_10); if (unlikely(!__pyx_t_5)) __PYX_ERR(6, 166, __pyx_L3_error)
            __Pyx_GOTREF(__pyx_t_5);
            __pyx_t_6 = PyNumber_InPlaceSubtract(__pyx_v_sent_byte_count, __pyx_t_5); if (unlikely(!__pyx_t_6)) __PYX_ERR(6, 166, __pyx_L3_error)
            __Pyx_GOTREF(__pyx_t_6);
            __Pyx_DECREF(__pyx_t_5); __pyx_t_5 = 0;
            __Pyx_DECREF_SET(__pyx_v_sent_byte_count, __pyx_t_6);
            __pyx_t_6 = 0;

            /* "src/python/grpcio/grpc/_cython/_cygrpc/grpc_gevent.pyx.pxi":167
 *         else:
 *           sent_byte_count -= len(write_bytes[0])
 *           write_bytes = write_bytes[1:]             # <<<<<<<<<<<<<<
 *     socket_wrapper.write_cb(<grpc_custom_socket*>socket_wrapper.c_socket,
 *                             grpc_error_none())
 */
            __pyx_t_6 = __Pyx_PyObject_GetSlice(__pyx_v_write_bytes, 1, 0, NULL, NULL, &__pyx_slice__139, 1, 0, 1); if (unlikely(!__pyx_t_6)) __PYX_ERR(6, 167, __pyx_L3_error)
            __Pyx_GOTREF(__pyx_t_6);
            __Pyx_DECREF_SET(__pyx_v_write_bytes, __pyx_t_6);
            __pyx_t_6 = 0;
          }
          __pyx_L13:;
        }
      }

      /* "src/python/grpcio/grpc/_cython/_cygrpc/grpc_gevent.pyx.pxi":168
 *           sent_byte_count -= len(write_bytes[0])
 *           write_bytes = write_bytes[1:]
 *     socket_wrapper.write_cb(<grpc_custom_socket*>socket_wrapper.c_socket,             # <<<<<<<<<<<<<<
 *                             grpc_error_none())
 *   except IOError as io_error:
 */
      __pyx_v_socket_wrapper->write_cb(((struct grpc_custom_socket *)__pyx_v_socket_wrapper->c_socket), __pyx_f_4grpc_7_cython_6cygrpc_grpc_error_none());

      /* "src/python/grpcio/grpc/_cython/_cygrpc/grpc_gevent.pyx.pxi":158
 * 
 * cdef socket_write_async_cython(SocketWrapper socket_wrapper, write_bytes):
 *   try:             # <<<<<<<<<<<<<<
 *     while write_bytes:
 *       sent_byte_count = socket_sendmsg(socket_wrapper.socket, write_bytes)
 */
    }
    __Pyx_XDECREF(__pyx_t_1); __pyx_t_1 = 0;
    __Pyx_XDECREF(__pyx_t_2); __pyx_t_2 = 0;
    __Pyx_XDECREF(__pyx_t_3); __pyx_t_3 = 0;
    goto __pyx_L8_try_end;
    __pyx_L3_error:;
    __Pyx_XDECREF(__pyx_t_7); __pyx_t_7 = 0;
    __Pyx_XDECREF(__pyx_t_9); __pyx_t_9 = 0;
    __Pyx_XDECREF(__pyx_t_5); __pyx_t_5 = 0;
    __Pyx_XDECREF(__pyx_t_6); __pyx_t_6 = 0;

    /* "src/python/grpcio/grpc/_cython/_cygrpc/grpc_gevent.pyx.pxi":170
 *     socket_wrapper.write_cb(<grpc_custom_socket*>socket_wrapper.c_socket,
 *                             grpc_error_none())
 *   except IOError as io_error:             # <<<<<<<<<<<<<<
 *     socket_wrapper.write_cb(<grpc_custom_socket*>socket_wrapper.c_socket,
 *                             socket_error("send", str(io_error)))
 */
    __pyx_t_8 = __Pyx_PyErr_ExceptionMatches(__pyx_builtin_IOError);
    if (__pyx_t_8) {
      __Pyx_AddTraceback("grpc._cython.cygrpc.socket_write_async_cython", __pyx_clineno, __pyx_lineno, __pyx_filename);
      if (__Pyx_GetException(&__pyx_t_6, &__pyx_t_5, &__pyx_t_9) < 0) __PYX_ERR(6, 170, __pyx_L5_except_error)
      __Pyx_GOTREF(__pyx_t_6);
      __Pyx_GOTREF(__pyx_t_5);
      __Pyx_GOTREF(__pyx_t_9);
      __Pyx_INCREF(__pyx_t_5);
      __pyx_v_io_error = __pyx_t_5;

      /* "src/python/grpcio/grpc/_cython/_cygrpc/grpc_gevent.pyx.pxi":172
 *   except IOError as io_error:
 *     socket_wrapper.write_cb(<grpc_custom_socket*>socket_wrapper.c_socket,
 *                             socket_error("send", str(io_error)))             # <<<<<<<<<<<<<<
 *   g_event.set()
 * 
 */
      __pyx_t_7 = __Pyx_PyObject_CallOneArg(((PyObject *)(&PyString_Type)), __pyx_v_io_error); if (unlikely(!__pyx_t_7)) __PYX_ERR(6, 172, __pyx_L5_except_error)
      __Pyx_GOTREF(__pyx_t_7);
      if (!(likely(PyString_CheckExact(__pyx_t_7))||((__pyx_t_7) == Py_None)||(PyErr_Format(PyExc_TypeError, "Expected %.16s, got %.200s", "str", Py_TYPE(__pyx_t_7)->tp_name), 0))) __PYX_ERR(6, 172, __pyx_L5_except_error)

      /* "src/python/grpcio/grpc/_cython/_cygrpc/grpc_gevent.pyx.pxi":171
 *                             grpc_error_none())
 *   except IOError as io_error:
 *     socket_wrapper.write_cb(<grpc_custom_socket*>socket_wrapper.c_socket,             # <<<<<<<<<<<<<<
 *                             socket_error("send", str(io_error)))
 *   g_event.set()
 */
      __pyx_v_socket_wrapper->write_cb(((struct grpc_custom_socket *)__pyx_v_socket_wrapper->c_socket), __pyx_f_4grpc_7_cython_6cygrpc_socket_error(__pyx_n_s_send, ((PyObject*)__pyx_t_7)));
      __Pyx_DECREF(__pyx_t_7); __pyx_t_7 = 0;
      __Pyx_DECREF(__pyx_t_6); __pyx_t_6 = 0;
      __Pyx_DECREF(__pyx_t_5); __pyx_t_5 = 0;
      __Pyx_DECREF(__pyx_t_9); __pyx_t_9 = 0;
      goto __pyx_L4_exception_handled;
    }
    goto __pyx_L5_except_error;
    __pyx_L5_except_error:;

    /* "src/python/grpcio/grpc/_cython/_cygrpc/grpc_gevent.pyx.pxi":158
 * 
 * cdef socket_write_async_cython(SocketWrapper socket_wrapper, write_bytes):
 *   try:             # <<<<<<<<<<<<<<
 *     while write_bytes:
 *       sent_byte_count = socket_sendmsg(socket_wrapper.socket, write_bytes)
 */
    __Pyx_XGIVEREF(__pyx_t_1);
    __Pyx_XGIVEREF(__pyx_t_2);
    __Pyx_XGIVEREF(__pyx_t_3);
    __Pyx_ExceptionReset(__pyx_t_1, __pyx_t_2, __pyx_t_3);
    goto __pyx_L1_error;
    __pyx_L4_exception_handled:;
    __Pyx_XGIVEREF(__pyx_t_1);
    __Pyx_XGIVEREF(__pyx_t_2);
    __Pyx_XGIVEREF(__pyx_t_3);
    __Pyx_ExceptionReset(__pyx_t_1, __pyx_t_2, __pyx_t_3);
    __pyx_L8_try_end:;
  }

  /* "src/python/grpcio/grpc/_cython/_cygrpc/grpc_gevent.pyx.pxi":173
 *     socket_wrapper.write_cb(<grpc_custom_socket*>socket_wrapper.c_socket,
 *                             socket_error("send", str(io_error)))
 *   g_event.set()             # <<<<<<<<<<<<<<
 * 
 * def socket_write_async(socket_wrapper, write_bytes):
 */
  __pyx_t_5 = __Pyx_GetModuleGlobalName(__pyx_n_s_g_event); if (unlikely(!__pyx_t_5)) __PYX_ERR(6, 173, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_5);
  __pyx_t_6 = __Pyx_PyObject_GetAttrStr(__pyx_t_5, __pyx_n_s_set); if (unlikely(!__pyx_t_6)) __PYX_ERR(6, 173, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_6);
  __Pyx_DECREF(__pyx_t_5); __pyx_t_5 = 0;
  __pyx_t_5 = NULL;
  if (CYTHON_UNPACK_METHODS && unlikely(PyMethod_Check(__pyx_t_6))) {
    __pyx_t_5 = PyMethod_GET_SELF(__pyx_t_6);
    if (likely(__pyx_t_5)) {
      PyObject* function = PyMethod_GET_FUNCTION(__pyx_t_6);
      __Pyx_INCREF(__pyx_t_5);
      __Pyx_INCREF(function);
      __Pyx_DECREF_SET(__pyx_t_6, function);
    }
  }
  if (__pyx_t_5) {
    __pyx_t_9 = __Pyx_PyObject_CallOneArg(__pyx_t_6, __pyx_t_5); if (unlikely(!__pyx_t_9)) __PYX_ERR(6, 173, __pyx_L1_error)
    __Pyx_DECREF(__pyx_t_5); __pyx_t_5 = 0;
  } else {
    __pyx_t_9 = __Pyx_PyObject_CallNoArg(__pyx_t_6); if (unlikely(!__pyx_t_9)) __PYX_ERR(6, 173, __pyx_L1_error)
  }
  __Pyx_GOTREF(__pyx_t_9);
  __Pyx_DECREF(__pyx_t_6); __pyx_t_6 = 0;
  __Pyx_DECREF(__pyx_t_9); __pyx_t_9 = 0;

  /* "src/python/grpcio/grpc/_cython/_cygrpc/grpc_gevent.pyx.pxi":157
 *     return socket.send(b''.join(write_bytes))
 * 
 * cdef socket_write_async_cython(SocketWrapper socket_wrapper, write_bytes):             # <<<<<<<<<<<<<<
 *   try:
 *     while write_bytes:
 */

  /* function exit code */
  __pyx_r = Py_None; __Pyx_INCREF(Py_None);
  goto __pyx_L0;
  __pyx_L1_error:;
  __Pyx_XDECREF(__pyx_t_5);
  __Pyx_XDECREF(__pyx_t_6);
  __Pyx_XDECREF(__pyx_t_7);
  __Pyx_XDECREF(__pyx_t_9);
  __Pyx_AddTraceback("grpc._cython.cygrpc.socket_write_async_cython", __pyx_clineno, __pyx_lineno, __pyx_filename);
  __pyx_r = 0;
  __pyx_L0:;
  __Pyx_XDECREF(__pyx_v_sent_byte_count);
  __Pyx_XDECREF(__pyx_v_io_error);
  __Pyx_XDECREF(__pyx_v_write_bytes);
  __Pyx_XGIVEREF(__pyx_r);
  __Pyx_RefNannyFinishContext();
  return __pyx_r;
}

/* "src/python/grpcio/grpc/_cython/_cygrpc/grpc_gevent.pyx.pxi":175
 *   g_event.set()
 * 
 * def socket_write_async(socket_wrapper, write_bytes):             # <<<<<<<<<<<<<<
 *   socket_write_async_cython(socket_wrapper, write_bytes)
 * 
 */

/* Python wrapper */
static PyObject *__pyx_pw_4grpc_7_cython_6cygrpc_51socket_write_async(PyObject *__pyx_self, PyObject *__pyx_args, PyObject *__pyx_kwds); /*proto*/
static PyMethodDef __pyx_mdef_4grpc_7_cython_6cygrpc_51socket_write_async = {"socket_write_async", (PyCFunction)__pyx_pw_4grpc_7_cython_6cygrpc_51socket_write_async, METH_VARARGS|METH_KEYWORDS, 0};
static PyObject *__pyx_pw_4grpc_7_cython_6cygrpc_51socket_write_async(PyObject *__pyx_self, PyObject *__pyx_args, PyObject *__pyx_kwds) {
  PyObject *__pyx_v_socket_wrapper = 0;
  PyObject *__pyx_v_write_bytes = 0;
  PyObject *__pyx_r = 0;
  __Pyx_RefNannyDeclarations
  __Pyx_RefNannySetupContext("socket_write_async (wrapper)", 0);
  {
    static PyObject **__pyx_pyargnames[] = {&__pyx_n_s_socket_wrapper,&__pyx_n_s_write_bytes,0};
    PyObject* values[2] = {0,0};
    if (unlikely(__pyx_kwds)) {
      Py_ssize_t kw_args;
      const Py_ssize_t pos_args = PyTuple_GET_SIZE(__pyx_args);
      switch (pos_args) {
        case  2: values[1] = PyTuple_GET_ITEM(__pyx_args, 1);
        CYTHON_FALLTHROUGH;
        case  1: values[0] = PyTuple_GET_ITEM(__pyx_args, 0);
        CYTHON_FALLTHROUGH;
        case  0: break;
        default: goto __pyx_L5_argtuple_error;
      }
      kw_args = PyDict_Size(__pyx_kwds);
      switch (pos_args) {
        case  0:
        if (likely((values[0] = __Pyx_PyDict_GetItemStr(__pyx_kwds, __pyx_n_s_socket_wrapper)) != 0)) kw_args--;
        else goto __pyx_L5_argtuple_error;
        CYTHON_FALLTHROUGH;
        case  1:
        if (likely((values[1] = __Pyx_PyDict_GetItemStr(__pyx_kwds, __pyx_n_s_write_bytes)) != 0)) kw_args--;
        else {
          __Pyx_RaiseArgtupleInvalid("socket_write_async", 1, 2, 2, 1); __PYX_ERR(6, 175, __pyx_L3_error)
        }
      }
      if (unlikely(kw_args > 0)) {
        if (unlikely(__Pyx_ParseOptionalKeywords(__pyx_kwds, __pyx_pyargnames, 0, values, pos_args, "socket_write_async") < 0)) __PYX_ERR(6, 175, __pyx_L3_error)
      }
    } else if (PyTuple_GET_SIZE(__pyx_args) != 2) {
      goto __pyx_L5_argtuple_error;
    } else {
      values[0] = PyTuple_GET_ITEM(__pyx_args, 0);
      values[1] = PyTuple_GET_ITEM(__pyx_args, 1);
    }
    __pyx_v_socket_wrapper = values[0];
    __pyx_v_write_bytes = values[1];
  }
  goto __pyx_L4_argument_unpacking_done;
  __pyx_L5_argtuple_error:;
  __Pyx_RaiseArgtupleInvalid("socket_write_async", 1, 2, 2, PyTuple_GET_SIZE(__pyx_args)); __PYX_ERR(6, 175, __pyx_L3_error)
  __pyx_L3_error:;
  __Pyx_AddTraceback("grpc._cython.cygrpc.socket_write_async", __pyx_clineno, __pyx_lineno, __pyx_filename);
  __Pyx_RefNannyFinishContext();
  return NULL;
  __pyx_L4_argument_unpacking_done:;
  __pyx_r = __pyx_pf_4grpc_7_cython_6cygrpc_50socket_write_async(__pyx_self, __pyx_v_socket_wrapper, __pyx_v_write_bytes);

  /* function exit code */
  __Pyx_RefNannyFinishContext();
  return __pyx_r;
}

static PyObject *__pyx_pf_4grpc_7_cython_6cygrpc_50socket_write_async(CYTHON_UNUSED PyObject *__pyx_self, PyObject *__pyx_v_socket_wrapper, PyObject *__pyx_v_write_bytes) {
  PyObject *__pyx_r = NULL;
  __Pyx_RefNannyDeclarations
  PyObject *__pyx_t_1 = NULL;
  __Pyx_RefNannySetupContext("socket_write_async", 0);

  /* "src/python/grpcio/grpc/_cython/_cygrpc/grpc_gevent.pyx.pxi":176
 * 
 * def socket_write_async(socket_wrapper, write_bytes):
 *   socket_write_async_cython(socket_wrapper, write_bytes)             # <<<<<<<<<<<<<<
 * 
 * cdef void socket_write(grpc_custom_socket* socket, grpc_slice_buffer* buffer,
 */
  if (!(likely(((__pyx_v_socket_wrapper) == Py_None) || likely(__Pyx_TypeTest(__pyx_v_socket_wrapper, __pyx_ptype_4grpc_7_cython_6cygrpc_SocketWrapper))))) __PYX_ERR(6, 176, __pyx_L1_error)
  __pyx_t_1 = __pyx_f_4grpc_7_cython_6cygrpc_socket_write_async_cython(((struct __pyx_obj_4grpc_7_cython_6cygrpc_SocketWrapper *)__pyx_v_socket_wrapper), __pyx_v_write_bytes); if (unlikely(!__pyx_t_1)) __PYX_ERR(6, 176, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_1);
  __Pyx_DECREF(__pyx_t_1); __pyx_t_1 = 0;

  /* "src/python/grpcio/grpc/_cython/_cygrpc/grpc_gevent.pyx.pxi":175
 *   g_event.set()
 * 
 * def socket_write_async(socket_wrapper, write_bytes):             # <<<<<<<<<<<<<<
 *   socket_write_async_cython(socket_wrapper, write_bytes)
 * 
 */

  /* function exit code */
  __pyx_r = Py_None; __Pyx_INCREF(Py_None);
  goto __pyx_L0;
  __pyx_L1_error:;
  __Pyx_XDECREF(__pyx_t_1);
  __Pyx_AddTraceback("grpc._cython.cygrpc.socket_write_async", __pyx_clineno, __pyx_lineno, __pyx_filename);
  __pyx_r = NULL;
  __pyx_L0:;
  __Pyx_XGIVEREF(__pyx_r);
  __Pyx_RefNannyFinishContext();
  return __pyx_r;
}

/* "src/python/grpcio/grpc/_cython/_cygrpc/grpc_gevent.pyx.pxi":178
 *   socket_write_async_cython(socket_wrapper, write_bytes)
 * 
 * cdef void socket_write(grpc_custom_socket* socket, grpc_slice_buffer* buffer,             # <<<<<<<<<<<<<<
 *                        grpc_custom_write_callback cb) except * with gil:
 *   cdef char* start
 */

static void __pyx_f_4grpc_7_cython_6cygrpc_socket_write(struct grpc_custom_socket *__pyx_v_socket, struct grpc_slice_buffer *__pyx_v_buffer, grpc_custom_write_callback __pyx_v_cb) {
  char *__pyx_v_start;
  struct __pyx_obj_4grpc_7_cython_6cygrpc_SocketWrapper *__pyx_v_sw = NULL;
  PyObject *__pyx_v_write_bytes = NULL;
  int __pyx_v_i;
  int __pyx_v_length;
  __Pyx_RefNannyDeclarations
  PyObject *__pyx_t_1 = NULL;
  int __pyx_t_2;
  int __pyx_t_3;
  int __pyx_t_4;
  int __pyx_t_5;
  PyObject *__pyx_t_6 = NULL;
  PyObject *__pyx_t_7 = NULL;
  PyObject *__pyx_t_8 = NULL;
  PyObject *__pyx_t_9 = NULL;
  #ifdef WITH_THREAD
  PyGILState_STATE __pyx_gilstate_save = __Pyx_PyGILState_Ensure();
  #endif
  __Pyx_RefNannySetupContext("socket_write", 0);

  /* "src/python/grpcio/grpc/_cython/_cygrpc/grpc_gevent.pyx.pxi":181
 *                        grpc_custom_write_callback cb) except * with gil:
 *   cdef char* start
 *   sw = <SocketWrapper>socket.impl             # <<<<<<<<<<<<<<
 *   sw.write_cb = cb
 *   write_bytes = []
 */
  __pyx_t_1 = ((PyObject *)__pyx_v_socket->impl);
  __Pyx_INCREF(__pyx_t_1);
  __pyx_v_sw = ((struct __pyx_obj_4grpc_7_cython_6cygrpc_SocketWrapper *)__pyx_t_1);
  __pyx_t_1 = 0;

  /* "src/python/grpcio/grpc/_cython/_cygrpc/grpc_gevent.pyx.pxi":182
 *   cdef char* start
 *   sw = <SocketWrapper>socket.impl
 *   sw.write_cb = cb             # <<<<<<<<<<<<<<
 *   write_bytes = []
 *   for i in range(buffer.count):
 */
  __pyx_v_sw->write_cb = __pyx_v_cb;

  /* "src/python/grpcio/grpc/_cython/_cygrpc/grpc_gevent.pyx.pxi":183
 *   sw = <SocketWrapper>socket.impl
 *   sw.write_cb = cb
 *   write_bytes = []             # <<<<<<<<<<<<<<
 *   for i in range(buffer.count):
 *     start = grpc_slice_buffer_start(buffer, i)
 */
  __pyx_t_1 = PyList_New(0); if (unlikely(!__pyx_t_1)) __PYX_ERR(6, 183, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_1);
  __pyx_v_write_bytes = ((PyObject*)__pyx_t_1);
  __pyx_t_1 = 0;

  /* "src/python/grpcio/grpc/_cython/_cygrpc/grpc_gevent.pyx.pxi":184
 *   sw.write_cb = cb
 *   write_bytes = []
 *   for i in range(buffer.count):             # <<<<<<<<<<<<<<
 *     start = grpc_slice_buffer_start(buffer, i)
 *     length = grpc_slice_buffer_length(buffer, i)
 */
  __pyx_t_2 = __pyx_v_buffer->count;
  __pyx_t_3 = __pyx_t_2;
  for (__pyx_t_4 = 0; __pyx_t_4 < __pyx_t_3; __pyx_t_4+=1) {
    __pyx_v_i = __pyx_t_4;

    /* "src/python/grpcio/grpc/_cython/_cygrpc/grpc_gevent.pyx.pxi":185
 *   write_bytes = []
 *   for i in range(buffer.count):
 *     start = grpc_slice_buffer_start(buffer, i)             # <<<<<<<<<<<<<<
 *     length = grpc_slice_buffer_length(buffer, i)
 *     write_bytes.append(<bytes>start[:length])
 */
    __pyx_v_start = grpc_slice_buffer_start(__pyx_v_buffer, __pyx_v_i);

    /* "src/python/grpcio/grpc/_cython/_cygrpc/grpc_gevent.pyx.pxi":186
 *   for i in range(buffer.count):
 *     start = grpc_slice_buffer_start(buffer, i)
 *     length = grpc_slice_buffer_length(buffer, i)             # <<<<<<<<<<<<<<
 *     write_bytes.append(<bytes>start[:length])
 *   _spawn_greenlet(socket_write_async, <SocketWrapper>socket.impl, write_bytes)
 */
    __pyx_v_length = grpc_slice_buffer_length(__pyx_v_buffer, __pyx_v_i);

    /* "src/python/grpcio/grpc/_cython/_cygrpc/grpc_gevent.pyx.pxi":187
 *     start = grpc_slice_buffer_start(buffer, i)
 *     length = grpc_slice_buffer_length(buffer, i)
 *     write_bytes.append(<bytes>start[:length])             # <<<<<<<<<<<<<<
 *   _spawn_greenlet(socket_write_async, <SocketWrapper>socket.impl, write_bytes)
 * 
 */
    __pyx_t_1 = __Pyx_PyBytes_FromStringAndSize(__pyx_v_start + 0, __pyx_v_length - 0); if (unlikely(!__pyx_t_1)) __PYX_ERR(6, 187, __pyx_L1_error)
    __Pyx_GOTREF(__pyx_t_1);
    __pyx_t_5 = __Pyx_PyList_Append(__pyx_v_write_bytes, __pyx_t_1); if (unlikely(__pyx_t_5 == ((int)-1))) __PYX_ERR(6, 187, __pyx_L1_error)
    __Pyx_DECREF(__pyx_t_1); __pyx_t_1 = 0;
  }

  /* "src/python/grpcio/grpc/_cython/_cygrpc/grpc_gevent.pyx.pxi":188
 *     length = grpc_slice_buffer_length(buffer, i)
 *     write_bytes.append(<bytes>start[:length])
 *   _spawn_greenlet(socket_write_async, <SocketWrapper>socket.impl, write_bytes)             # <<<<<<<<<<<<<<
 * 
 * cdef socket_read_async_cython(SocketWrapper socket_wrapper):
 */
  __pyx_t_6 = __Pyx_GetModuleGlobalName(__pyx_n_s_spawn_greenlet); if (unlikely(!__pyx_t_6)) __PYX_ERR(6, 188, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_6);
  __pyx_t_7 = __Pyx_GetModuleGlobalName(__pyx_n_s_socket_write_async); if (unlikely(!__pyx_t_7)) __PYX_ERR(6, 188, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_7);
  __pyx_t_8 = NULL;
  __pyx_t_2 = 0;
  if (CYTHON_UNPACK_METHODS && unlikely(PyMethod_Check(__pyx_t_6))) {
    __pyx_t_8 = PyMethod_GET_SELF(__pyx_t_6);
    if (likely(__pyx_t_8)) {
      PyObject* function = PyMethod_GET_FUNCTION(__pyx_t_6);
      __Pyx_INCREF(__pyx_t_8);
      __Pyx_INCREF(function);
      __Pyx_DECREF_SET(__pyx_t_6, function);
      __pyx_t_2 = 1;
    }
  }
  #if CYTHON_FAST_PYCALL
  if (PyFunction_Check(__pyx_t_6)) {
    PyObject *__pyx_temp[4] = {__pyx_t_8, __pyx_t_7, ((PyObject *)__pyx_v_socket->impl), __pyx_v_write_bytes};
    __pyx_t_1 = __Pyx_PyFunction_FastCall(__pyx_t_6, __pyx_temp+1-__pyx_t_2, 3+__pyx_t_2); if (unlikely(!__pyx_t_1)) __PYX_ERR(6, 188, __pyx_L1_error)
    __Pyx_XDECREF(__pyx_t_8); __pyx_t_8 = 0;
    __Pyx_GOTREF(__pyx_t_1);
    __Pyx_DECREF(__pyx_t_7); __pyx_t_7 = 0;
  } else
  #endif
  #if CYTHON_FAST_PYCCALL
  if (__Pyx_PyFastCFunction_Check(__pyx_t_6)) {
    PyObject *__pyx_temp[4] = {__pyx_t_8, __pyx_t_7, ((PyObject *)__pyx_v_socket->impl), __pyx_v_write_bytes};
    __pyx_t_1 = __Pyx_PyCFunction_FastCall(__pyx_t_6, __pyx_temp+1-__pyx_t_2, 3+__pyx_t_2); if (unlikely(!__pyx_t_1)) __PYX_ERR(6, 188, __pyx_L1_error)
    __Pyx_XDECREF(__pyx_t_8); __pyx_t_8 = 0;
    __Pyx_GOTREF(__pyx_t_1);
    __Pyx_DECREF(__pyx_t_7); __pyx_t_7 = 0;
  } else
  #endif
  {
    __pyx_t_9 = PyTuple_New(3+__pyx_t_2); if (unlikely(!__pyx_t_9)) __PYX_ERR(6, 188, __pyx_L1_error)
    __Pyx_GOTREF(__pyx_t_9);
    if (__pyx_t_8) {
      __Pyx_GIVEREF(__pyx_t_8); PyTuple_SET_ITEM(__pyx_t_9, 0, __pyx_t_8); __pyx_t_8 = NULL;
    }
    __Pyx_GIVEREF(__pyx_t_7);
    PyTuple_SET_ITEM(__pyx_t_9, 0+__pyx_t_2, __pyx_t_7);
    __Pyx_INCREF(((PyObject *)((struct __pyx_obj_4grpc_7_cython_6cygrpc_SocketWrapper *)__pyx_v_socket->impl)));
    __Pyx_GIVEREF(((PyObject *)__pyx_v_socket->impl));
    PyTuple_SET_ITEM(__pyx_t_9, 1+__pyx_t_2, ((PyObject *)__pyx_v_socket->impl));
    __Pyx_INCREF(__pyx_v_write_bytes);
    __Pyx_GIVEREF(__pyx_v_write_bytes);
    PyTuple_SET_ITEM(__pyx_t_9, 2+__pyx_t_2, __pyx_v_write_bytes);
    __pyx_t_7 = 0;
    __pyx_t_1 = __Pyx_PyObject_Call(__pyx_t_6, __pyx_t_9, NULL); if (unlikely(!__pyx_t_1)) __PYX_ERR(6, 188, __pyx_L1_error)
    __Pyx_GOTREF(__pyx_t_1);
    __Pyx_DECREF(__pyx_t_9); __pyx_t_9 = 0;
  }
  __Pyx_DECREF(__pyx_t_6); __pyx_t_6 = 0;
  __Pyx_DECREF(__pyx_t_1); __pyx_t_1 = 0;

  /* "src/python/grpcio/grpc/_cython/_cygrpc/grpc_gevent.pyx.pxi":178
 *   socket_write_async_cython(socket_wrapper, write_bytes)
 * 
 * cdef void socket_write(grpc_custom_socket* socket, grpc_slice_buffer* buffer,             # <<<<<<<<<<<<<<
 *                        grpc_custom_write_callback cb) except * with gil:
 *   cdef char* start
 */

  /* function exit code */
  goto __pyx_L0;
  __pyx_L1_error:;
  __Pyx_XDECREF(__pyx_t_1);
  __Pyx_XDECREF(__pyx_t_6);
  __Pyx_XDECREF(__pyx_t_7);
  __Pyx_XDECREF(__pyx_t_8);
  __Pyx_XDECREF(__pyx_t_9);
  __Pyx_AddTraceback("grpc._cython.cygrpc.socket_write", __pyx_clineno, __pyx_lineno, __pyx_filename);
  __pyx_L0:;
  __Pyx_XDECREF((PyObject *)__pyx_v_sw);
  __Pyx_XDECREF(__pyx_v_write_bytes);
  __Pyx_RefNannyFinishContext();
  #ifdef WITH_THREAD
  __Pyx_PyGILState_Release(__pyx_gilstate_save);
  #endif
}

/* "src/python/grpcio/grpc/_cython/_cygrpc/grpc_gevent.pyx.pxi":190
 *   _spawn_greenlet(socket_write_async, <SocketWrapper>socket.impl, write_bytes)
 * 
 * cdef socket_read_async_cython(SocketWrapper socket_wrapper):             # <<<<<<<<<<<<<<
 *   cdef char* buff_char_arr
 *   try:
 */

static PyObject *__pyx_f_4grpc_7_cython_6cygrpc_socket_read_async_cython(struct __pyx_obj_4grpc_7_cython_6cygrpc_SocketWrapper *__pyx_v_socket_wrapper) {
  char *__pyx_v_buff_char_arr;
  PyObject *__pyx_v_buff_str = NULL;
  PyObject *__pyx_v_io_error = NULL;
  PyObject *__pyx_r = NULL;
  __Pyx_RefNannyDeclarations
  PyObject *__pyx_t_1 = NULL;
  PyObject *__pyx_t_2 = NULL;
  PyObject *__pyx_t_3 = NULL;
  PyObject *__pyx_t_4 = NULL;
  PyObject *__pyx_t_5 = NULL;
  PyObject *__pyx_t_6 = NULL;
  PyObject *__pyx_t_7 = NULL;
  PyObject *__pyx_t_8 = NULL;
  char *__pyx_t_9;
  Py_ssize_t __pyx_t_10;
  int __pyx_t_11;
  __Pyx_RefNannySetupContext("socket_read_async_cython", 0);

  /* "src/python/grpcio/grpc/_cython/_cygrpc/grpc_gevent.pyx.pxi":192
 * cdef socket_read_async_cython(SocketWrapper socket_wrapper):
 *   cdef char* buff_char_arr
 *   try:             # <<<<<<<<<<<<<<
 *     buff_str = socket_wrapper.socket.recv(socket_wrapper.len)
 *     buff_char_arr = buff_str
 */
  {
    __Pyx_PyThreadState_declare
    __Pyx_PyThreadState_assign
    __Pyx_ExceptionSave(&__pyx_t_1, &__pyx_t_2, &__pyx_t_3);
    __Pyx_XGOTREF(__pyx_t_1);
    __Pyx_XGOTREF(__pyx_t_2);
    __Pyx_XGOTREF(__pyx_t_3);
    /*try:*/ {

      /* "src/python/grpcio/grpc/_cython/_cygrpc/grpc_gevent.pyx.pxi":193
 *   cdef char* buff_char_arr
 *   try:
 *     buff_str = socket_wrapper.socket.recv(socket_wrapper.len)             # <<<<<<<<<<<<<<
 *     buff_char_arr = buff_str
 *     string.memcpy(<void*>socket_wrapper.c_buffer, buff_char_arr, len(buff_str))
 */
      __pyx_t_5 = __Pyx_PyObject_GetAttrStr(__pyx_v_socket_wrapper->socket, __pyx_n_s_recv); if (unlikely(!__pyx_t_5)) __PYX_ERR(6, 193, __pyx_L3_error)
      __Pyx_GOTREF(__pyx_t_5);
      __pyx_t_6 = __Pyx_PyInt_FromSize_t(__pyx_v_socket_wrapper->len); if (unlikely(!__pyx_t_6)) __PYX_ERR(6, 193, __pyx_L3_error)
      __Pyx_GOTREF(__pyx_t_6);
      __pyx_t_7 = NULL;
      if (CYTHON_UNPACK_METHODS && likely(PyMethod_Check(__pyx_t_5))) {
        __pyx_t_7 = PyMethod_GET_SELF(__pyx_t_5);
        if (likely(__pyx_t_7)) {
          PyObject* function = PyMethod_GET_FUNCTION(__pyx_t_5);
          __Pyx_INCREF(__pyx_t_7);
          __Pyx_INCREF(function);
          __Pyx_DECREF_SET(__pyx_t_5, function);
        }
      }
      if (!__pyx_t_7) {
        __pyx_t_4 = __Pyx_PyObject_CallOneArg(__pyx_t_5, __pyx_t_6); if (unlikely(!__pyx_t_4)) __PYX_ERR(6, 193, __pyx_L3_error)
        __Pyx_DECREF(__pyx_t_6); __pyx_t_6 = 0;
        __Pyx_GOTREF(__pyx_t_4);
      } else {
        #if CYTHON_FAST_PYCALL
        if (PyFunction_Check(__pyx_t_5)) {
          PyObject *__pyx_temp[2] = {__pyx_t_7, __pyx_t_6};
          __pyx_t_4 = __Pyx_PyFunction_FastCall(__pyx_t_5, __pyx_temp+1-1, 1+1); if (unlikely(!__pyx_t_4)) __PYX_ERR(6, 193, __pyx_L3_error)
          __Pyx_XDECREF(__pyx_t_7); __pyx_t_7 = 0;
          __Pyx_GOTREF(__pyx_t_4);
          __Pyx_DECREF(__pyx_t_6); __pyx_t_6 = 0;
        } else
        #endif
        #if CYTHON_FAST_PYCCALL
        if (__Pyx_PyFastCFunction_Check(__pyx_t_5)) {
          PyObject *__pyx_temp[2] = {__pyx_t_7, __pyx_t_6};
          __pyx_t_4 = __Pyx_PyCFunction_FastCall(__pyx_t_5, __pyx_temp+1-1, 1+1); if (unlikely(!__pyx_t_4)) __PYX_ERR(6, 193, __pyx_L3_error)
          __Pyx_XDECREF(__pyx_t_7); __pyx_t_7 = 0;
          __Pyx_GOTREF(__pyx_t_4);
          __Pyx_DECREF(__pyx_t_6); __pyx_t_6 = 0;
        } else
        #endif
        {
          __pyx_t_8 = PyTuple_New(1+1); if (unlikely(!__pyx_t_8)) __PYX_ERR(6, 193, __pyx_L3_error)
          __Pyx_GOTREF(__pyx_t_8);
          __Pyx_GIVEREF(__pyx_t_7); PyTuple_SET_ITEM(__pyx_t_8, 0, __pyx_t_7); __pyx_t_7 = NULL;
          __Pyx_GIVEREF(__pyx_t_6);
          PyTuple_SET_ITEM(__pyx_t_8, 0+1, __pyx_t_6);
          __pyx_t_6 = 0;
          __pyx_t_4 = __Pyx_PyObject_Call(__pyx_t_5, __pyx_t_8, NULL); if (unlikely(!__pyx_t_4)) __PYX_ERR(6, 193, __pyx_L3_error)
          __Pyx_GOTREF(__pyx_t_4);
          __Pyx_DECREF(__pyx_t_8); __pyx_t_8 = 0;
        }
      }
      __Pyx_DECREF(__pyx_t_5); __pyx_t_5 = 0;
      __pyx_v_buff_str = __pyx_t_4;
      __pyx_t_4 = 0;

      /* "src/python/grpcio/grpc/_cython/_cygrpc/grpc_gevent.pyx.pxi":194
 *   try:
 *     buff_str = socket_wrapper.socket.recv(socket_wrapper.len)
 *     buff_char_arr = buff_str             # <<<<<<<<<<<<<<
 *     string.memcpy(<void*>socket_wrapper.c_buffer, buff_char_arr, len(buff_str))
 *     socket_wrapper.read_cb(<grpc_custom_socket*>socket_wrapper.c_socket,
 */
      __pyx_t_9 = __Pyx_PyObject_AsWritableString(__pyx_v_buff_str); if (unlikely((!__pyx_t_9) && PyErr_Occurred())) __PYX_ERR(6, 194, __pyx_L3_error)
      __pyx_v_buff_char_arr = __pyx_t_9;

      /* "src/python/grpcio/grpc/_cython/_cygrpc/grpc_gevent.pyx.pxi":195
 *     buff_str = socket_wrapper.socket.recv(socket_wrapper.len)
 *     buff_char_arr = buff_str
 *     string.memcpy(<void*>socket_wrapper.c_buffer, buff_char_arr, len(buff_str))             # <<<<<<<<<<<<<<
 *     socket_wrapper.read_cb(<grpc_custom_socket*>socket_wrapper.c_socket,
 *                            len(buff_str), grpc_error_none())
 */
      __pyx_t_10 = PyObject_Length(__pyx_v_buff_str); if (unlikely(__pyx_t_10 == ((Py_ssize_t)-1))) __PYX_ERR(6, 195, __pyx_L3_error)
      (void)(memcpy(((void *)__pyx_v_socket_wrapper->c_buffer), __pyx_v_buff_char_arr, __pyx_t_10));

      /* "src/python/grpcio/grpc/_cython/_cygrpc/grpc_gevent.pyx.pxi":197
 *     string.memcpy(<void*>socket_wrapper.c_buffer, buff_char_arr, len(buff_str))
 *     socket_wrapper.read_cb(<grpc_custom_socket*>socket_wrapper.c_socket,
 *                            len(buff_str), grpc_error_none())             # <<<<<<<<<<<<<<
 *   except IOError as io_error:
 *     socket_wrapper.read_cb(<grpc_custom_socket*>socket_wrapper.c_socket,
 */
      __pyx_t_10 = PyObject_Length(__pyx_v_buff_str); if (unlikely(__pyx_t_10 == ((Py_ssize_t)-1))) __PYX_ERR(6, 197, __pyx_L3_error)

      /* "src/python/grpcio/grpc/_cython/_cygrpc/grpc_gevent.pyx.pxi":196
 *     buff_char_arr = buff_str
 *     string.memcpy(<void*>socket_wrapper.c_buffer, buff_char_arr, len(buff_str))
 *     socket_wrapper.read_cb(<grpc_custom_socket*>socket_wrapper.c_socket,             # <<<<<<<<<<<<<<
 *                            len(buff_str), grpc_error_none())
 *   except IOError as io_error:
 */
      __pyx_v_socket_wrapper->read_cb(((struct grpc_custom_socket *)__pyx_v_socket_wrapper->c_socket), __pyx_t_10, __pyx_f_4grpc_7_cython_6cygrpc_grpc_error_none());

      /* "src/python/grpcio/grpc/_cython/_cygrpc/grpc_gevent.pyx.pxi":192
 * cdef socket_read_async_cython(SocketWrapper socket_wrapper):
 *   cdef char* buff_char_arr
 *   try:             # <<<<<<<<<<<<<<
 *     buff_str = socket_wrapper.socket.recv(socket_wrapper.len)
 *     buff_char_arr = buff_str
 */
    }
    __Pyx_XDECREF(__pyx_t_1); __pyx_t_1 = 0;
    __Pyx_XDECREF(__pyx_t_2); __pyx_t_2 = 0;
    __Pyx_XDECREF(__pyx_t_3); __pyx_t_3 = 0;
    goto __pyx_L8_try_end;
    __pyx_L3_error:;
    __Pyx_XDECREF(__pyx_t_7); __pyx_t_7 = 0;
    __Pyx_XDECREF(__pyx_t_6); __pyx_t_6 = 0;
    __Pyx_XDECREF(__pyx_t_8); __pyx_t_8 = 0;
    __Pyx_XDECREF(__pyx_t_5); __pyx_t_5 = 0;
    __Pyx_XDECREF(__pyx_t_4); __pyx_t_4 = 0;

    /* "src/python/grpcio/grpc/_cython/_cygrpc/grpc_gevent.pyx.pxi":198
 *     socket_wrapper.read_cb(<grpc_custom_socket*>socket_wrapper.c_socket,
 *                            len(buff_str), grpc_error_none())
 *   except IOError as io_error:             # <<<<<<<<<<<<<<
 *     socket_wrapper.read_cb(<grpc_custom_socket*>socket_wrapper.c_socket,
 *                            -1, socket_error("recv", str(io_error)))
 */
    __pyx_t_11 = __Pyx_PyErr_ExceptionMatches(__pyx_builtin_IOError);
    if (__pyx_t_11) {
      __Pyx_AddTraceback("grpc._cython.cygrpc.socket_read_async_cython", __pyx_clineno, __pyx_lineno, __pyx_filename);
      if (__Pyx_GetException(&__pyx_t_4, &__pyx_t_5, &__pyx_t_8) < 0) __PYX_ERR(6, 198, __pyx_L5_except_error)
      __Pyx_GOTREF(__pyx_t_4);
      __Pyx_GOTREF(__pyx_t_5);
      __Pyx_GOTREF(__pyx_t_8);
      __Pyx_INCREF(__pyx_t_5);
      __pyx_v_io_error = __pyx_t_5;

      /* "src/python/grpcio/grpc/_cython/_cygrpc/grpc_gevent.pyx.pxi":200
 *   except IOError as io_error:
 *     socket_wrapper.read_cb(<grpc_custom_socket*>socket_wrapper.c_socket,
 *                            -1, socket_error("recv", str(io_error)))             # <<<<<<<<<<<<<<
 *   g_event.set()
 * 
 */
      __pyx_t_6 = __Pyx_PyObject_CallOneArg(((PyObject *)(&PyString_Type)), __pyx_v_io_error); if (unlikely(!__pyx_t_6)) __PYX_ERR(6, 200, __pyx_L5_except_error)
      __Pyx_GOTREF(__pyx_t_6);
      if (!(likely(PyString_CheckExact(__pyx_t_6))||((__pyx_t_6) == Py_None)||(PyErr_Format(PyExc_TypeError, "Expected %.16s, got %.200s", "str", Py_TYPE(__pyx_t_6)->tp_name), 0))) __PYX_ERR(6, 200, __pyx_L5_except_error)

      /* "src/python/grpcio/grpc/_cython/_cygrpc/grpc_gevent.pyx.pxi":199
 *                            len(buff_str), grpc_error_none())
 *   except IOError as io_error:
 *     socket_wrapper.read_cb(<grpc_custom_socket*>socket_wrapper.c_socket,             # <<<<<<<<<<<<<<
 *                            -1, socket_error("recv", str(io_error)))
 *   g_event.set()
 */
      __pyx_v_socket_wrapper->read_cb(((struct grpc_custom_socket *)__pyx_v_socket_wrapper->c_socket), -1L, __pyx_f_4grpc_7_cython_6cygrpc_socket_error(__pyx_n_s_recv, ((PyObject*)__pyx_t_6)));
      __Pyx_DECREF(__pyx_t_6); __pyx_t_6 = 0;
      __Pyx_DECREF(__pyx_t_4); __pyx_t_4 = 0;
      __Pyx_DECREF(__pyx_t_5); __pyx_t_5 = 0;
      __Pyx_DECREF(__pyx_t_8); __pyx_t_8 = 0;
      goto __pyx_L4_exception_handled;
    }
    goto __pyx_L5_except_error;
    __pyx_L5_except_error:;

    /* "src/python/grpcio/grpc/_cython/_cygrpc/grpc_gevent.pyx.pxi":192
 * cdef socket_read_async_cython(SocketWrapper socket_wrapper):
 *   cdef char* buff_char_arr
 *   try:             # <<<<<<<<<<<<<<
 *     buff_str = socket_wrapper.socket.recv(socket_wrapper.len)
 *     buff_char_arr = buff_str
 */
    __Pyx_XGIVEREF(__pyx_t_1);
    __Pyx_XGIVEREF(__pyx_t_2);
    __Pyx_XGIVEREF(__pyx_t_3);
    __Pyx_ExceptionReset(__pyx_t_1, __pyx_t_2, __pyx_t_3);
    goto __pyx_L1_error;
    __pyx_L4_exception_handled:;
    __Pyx_XGIVEREF(__pyx_t_1);
    __Pyx_XGIVEREF(__pyx_t_2);
    __Pyx_XGIVEREF(__pyx_t_3);
    __Pyx_ExceptionReset(__pyx_t_1, __pyx_t_2, __pyx_t_3);
    __pyx_L8_try_end:;
  }

  /* "src/python/grpcio/grpc/_cython/_cygrpc/grpc_gevent.pyx.pxi":201
 *     socket_wrapper.read_cb(<grpc_custom_socket*>socket_wrapper.c_socket,
 *                            -1, socket_error("recv", str(io_error)))
 *   g_event.set()             # <<<<<<<<<<<<<<
 * 
 * def socket_read_async(socket_wrapper):
 */
  __pyx_t_5 = __Pyx_GetModuleGlobalName(__pyx_n_s_g_event); if (unlikely(!__pyx_t_5)) __PYX_ERR(6, 201, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_5);
  __pyx_t_4 = __Pyx_PyObject_GetAttrStr(__pyx_t_5, __pyx_n_s_set); if (unlikely(!__pyx_t_4)) __PYX_ERR(6, 201, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_4);
  __Pyx_DECREF(__pyx_t_5); __pyx_t_5 = 0;
  __pyx_t_5 = NULL;
  if (CYTHON_UNPACK_METHODS && unlikely(PyMethod_Check(__pyx_t_4))) {
    __pyx_t_5 = PyMethod_GET_SELF(__pyx_t_4);
    if (likely(__pyx_t_5)) {
      PyObject* function = PyMethod_GET_FUNCTION(__pyx_t_4);
      __Pyx_INCREF(__pyx_t_5);
      __Pyx_INCREF(function);
      __Pyx_DECREF_SET(__pyx_t_4, function);
    }
  }
  if (__pyx_t_5) {
    __pyx_t_8 = __Pyx_PyObject_CallOneArg(__pyx_t_4, __pyx_t_5); if (unlikely(!__pyx_t_8)) __PYX_ERR(6, 201, __pyx_L1_error)
    __Pyx_DECREF(__pyx_t_5); __pyx_t_5 = 0;
  } else {
    __pyx_t_8 = __Pyx_PyObject_CallNoArg(__pyx_t_4); if (unlikely(!__pyx_t_8)) __PYX_ERR(6, 201, __pyx_L1_error)
  }
  __Pyx_GOTREF(__pyx_t_8);
  __Pyx_DECREF(__pyx_t_4); __pyx_t_4 = 0;
  __Pyx_DECREF(__pyx_t_8); __pyx_t_8 = 0;

  /* "src/python/grpcio/grpc/_cython/_cygrpc/grpc_gevent.pyx.pxi":190
 *   _spawn_greenlet(socket_write_async, <SocketWrapper>socket.impl, write_bytes)
 * 
 * cdef socket_read_async_cython(SocketWrapper socket_wrapper):             # <<<<<<<<<<<<<<
 *   cdef char* buff_char_arr
 *   try:
 */

  /* function exit code */
  __pyx_r = Py_None; __Pyx_INCREF(Py_None);
  goto __pyx_L0;
  __pyx_L1_error:;
  __Pyx_XDECREF(__pyx_t_4);
  __Pyx_XDECREF(__pyx_t_5);
  __Pyx_XDECREF(__pyx_t_6);
  __Pyx_XDECREF(__pyx_t_7);
  __Pyx_XDECREF(__pyx_t_8);
  __Pyx_AddTraceback("grpc._cython.cygrpc.socket_read_async_cython", __pyx_clineno, __pyx_lineno, __pyx_filename);
  __pyx_r = 0;
  __pyx_L0:;
  __Pyx_XDECREF(__pyx_v_buff_str);
  __Pyx_XDECREF(__pyx_v_io_error);
  __Pyx_XGIVEREF(__pyx_r);
  __Pyx_RefNannyFinishContext();
  return __pyx_r;
}

/* "src/python/grpcio/grpc/_cython/_cygrpc/grpc_gevent.pyx.pxi":203
 *   g_event.set()
 * 
 * def socket_read_async(socket_wrapper):             # <<<<<<<<<<<<<<
 *   socket_read_async_cython(socket_wrapper)
 * 
 */

/* Python wrapper */
static PyObject *__pyx_pw_4grpc_7_cython_6cygrpc_53socket_read_async(PyObject *__pyx_self, PyObject *__pyx_v_socket_wrapper); /*proto*/
static PyMethodDef __pyx_mdef_4grpc_7_cython_6cygrpc_53socket_read_async = {"socket_read_async", (PyCFunction)__pyx_pw_4grpc_7_cython_6cygrpc_53socket_read_async, METH_O, 0};
static PyObject *__pyx_pw_4grpc_7_cython_6cygrpc_53socket_read_async(PyObject *__pyx_self, PyObject *__pyx_v_socket_wrapper) {
  PyObject *__pyx_r = 0;
  __Pyx_RefNannyDeclarations
  __Pyx_RefNannySetupContext("socket_read_async (wrapper)", 0);
  __pyx_r = __pyx_pf_4grpc_7_cython_6cygrpc_52socket_read_async(__pyx_self, ((PyObject *)__pyx_v_socket_wrapper));

  /* function exit code */
  __Pyx_RefNannyFinishContext();
  return __pyx_r;
}

static PyObject *__pyx_pf_4grpc_7_cython_6cygrpc_52socket_read_async(CYTHON_UNUSED PyObject *__pyx_self, PyObject *__pyx_v_socket_wrapper) {
  PyObject *__pyx_r = NULL;
  __Pyx_RefNannyDeclarations
  PyObject *__pyx_t_1 = NULL;
  __Pyx_RefNannySetupContext("socket_read_async", 0);

  /* "src/python/grpcio/grpc/_cython/_cygrpc/grpc_gevent.pyx.pxi":204
 * 
 * def socket_read_async(socket_wrapper):
 *   socket_read_async_cython(socket_wrapper)             # <<<<<<<<<<<<<<
 * 
 * cdef void socket_read(grpc_custom_socket* socket, char* buffer,
 */
  if (!(likely(((__pyx_v_socket_wrapper) == Py_None) || likely(__Pyx_TypeTest(__pyx_v_socket_wrapper, __pyx_ptype_4grpc_7_cython_6cygrpc_SocketWrapper))))) __PYX_ERR(6, 204, __pyx_L1_error)
  __pyx_t_1 = __pyx_f_4grpc_7_cython_6cygrpc_socket_read_async_cython(((struct __pyx_obj_4grpc_7_cython_6cygrpc_SocketWrapper *)__pyx_v_socket_wrapper)); if (unlikely(!__pyx_t_1)) __PYX_ERR(6, 204, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_1);
  __Pyx_DECREF(__pyx_t_1); __pyx_t_1 = 0;

  /* "src/python/grpcio/grpc/_cython/_cygrpc/grpc_gevent.pyx.pxi":203
 *   g_event.set()
 * 
 * def socket_read_async(socket_wrapper):             # <<<<<<<<<<<<<<
 *   socket_read_async_cython(socket_wrapper)
 * 
 */

  /* function exit code */
  __pyx_r = Py_None; __Pyx_INCREF(Py_None);
  goto __pyx_L0;
  __pyx_L1_error:;
  __Pyx_XDECREF(__pyx_t_1);
  __Pyx_AddTraceback("grpc._cython.cygrpc.socket_read_async", __pyx_clineno, __pyx_lineno, __pyx_filename);
  __pyx_r = NULL;
  __pyx_L0:;
  __Pyx_XGIVEREF(__pyx_r);
  __Pyx_RefNannyFinishContext();
  return __pyx_r;
}

/* "src/python/grpcio/grpc/_cython/_cygrpc/grpc_gevent.pyx.pxi":206
 *   socket_read_async_cython(socket_wrapper)
 * 
 * cdef void socket_read(grpc_custom_socket* socket, char* buffer,             # <<<<<<<<<<<<<<
 *                       size_t length, grpc_custom_read_callback cb) except * with gil:
 *   sw = <SocketWrapper>socket.impl
 */

static void __pyx_f_4grpc_7_cython_6cygrpc_socket_read(struct grpc_custom_socket *__pyx_v_socket, char *__pyx_v_buffer, size_t __pyx_v_length, grpc_custom_read_callback __pyx_v_cb) {
  struct __pyx_obj_4grpc_7_cython_6cygrpc_SocketWrapper *__pyx_v_sw = NULL;
  __Pyx_RefNannyDeclarations
  PyObject *__pyx_t_1 = NULL;
  PyObject *__pyx_t_2 = NULL;
  PyObject *__pyx_t_3 = NULL;
  PyObject *__pyx_t_4 = NULL;
  int __pyx_t_5;
  PyObject *__pyx_t_6 = NULL;
  #ifdef WITH_THREAD
  PyGILState_STATE __pyx_gilstate_save = __Pyx_PyGILState_Ensure();
  #endif
  __Pyx_RefNannySetupContext("socket_read", 0);

  /* "src/python/grpcio/grpc/_cython/_cygrpc/grpc_gevent.pyx.pxi":208
 * cdef void socket_read(grpc_custom_socket* socket, char* buffer,
 *                       size_t length, grpc_custom_read_callback cb) except * with gil:
 *   sw = <SocketWrapper>socket.impl             # <<<<<<<<<<<<<<
 *   sw.read_cb = cb
 *   sw.c_buffer = buffer
 */
  __pyx_t_1 = ((PyObject *)__pyx_v_socket->impl);
  __Pyx_INCREF(__pyx_t_1);
  __pyx_v_sw = ((struct __pyx_obj_4grpc_7_cython_6cygrpc_SocketWrapper *)__pyx_t_1);
  __pyx_t_1 = 0;

  /* "src/python/grpcio/grpc/_cython/_cygrpc/grpc_gevent.pyx.pxi":209
 *                       size_t length, grpc_custom_read_callback cb) except * with gil:
 *   sw = <SocketWrapper>socket.impl
 *   sw.read_cb = cb             # <<<<<<<<<<<<<<
 *   sw.c_buffer = buffer
 *   sw.len = length
 */
  __pyx_v_sw->read_cb = __pyx_v_cb;

  /* "src/python/grpcio/grpc/_cython/_cygrpc/grpc_gevent.pyx.pxi":210
 *   sw = <SocketWrapper>socket.impl
 *   sw.read_cb = cb
 *   sw.c_buffer = buffer             # <<<<<<<<<<<<<<
 *   sw.len = length
 *   _spawn_greenlet(socket_read_async, sw)
 */
  __pyx_v_sw->c_buffer = __pyx_v_buffer;

  /* "src/python/grpcio/grpc/_cython/_cygrpc/grpc_gevent.pyx.pxi":211
 *   sw.read_cb = cb
 *   sw.c_buffer = buffer
 *   sw.len = length             # <<<<<<<<<<<<<<
 *   _spawn_greenlet(socket_read_async, sw)
 * 
 */
  __pyx_v_sw->len = __pyx_v_length;

  /* "src/python/grpcio/grpc/_cython/_cygrpc/grpc_gevent.pyx.pxi":212
 *   sw.c_buffer = buffer
 *   sw.len = length
 *   _spawn_greenlet(socket_read_async, sw)             # <<<<<<<<<<<<<<
 * 
 * cdef grpc_error* socket_getpeername(grpc_custom_socket* socket,
 */
  __pyx_t_2 = __Pyx_GetModuleGlobalName(__pyx_n_s_spawn_greenlet); if (unlikely(!__pyx_t_2)) __PYX_ERR(6, 212, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_2);
  __pyx_t_3 = __Pyx_GetModuleGlobalName(__pyx_n_s_socket_read_async); if (unlikely(!__pyx_t_3)) __PYX_ERR(6, 212, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_3);
  __pyx_t_4 = NULL;
  __pyx_t_5 = 0;
  if (CYTHON_UNPACK_METHODS && unlikely(PyMethod_Check(__pyx_t_2))) {
    __pyx_t_4 = PyMethod_GET_SELF(__pyx_t_2);
    if (likely(__pyx_t_4)) {
      PyObject* function = PyMethod_GET_FUNCTION(__pyx_t_2);
      __Pyx_INCREF(__pyx_t_4);
      __Pyx_INCREF(function);
      __Pyx_DECREF_SET(__pyx_t_2, function);
      __pyx_t_5 = 1;
    }
  }
  #if CYTHON_FAST_PYCALL
  if (PyFunction_Check(__pyx_t_2)) {
    PyObject *__pyx_temp[3] = {__pyx_t_4, __pyx_t_3, ((PyObject *)__pyx_v_sw)};
    __pyx_t_1 = __Pyx_PyFunction_FastCall(__pyx_t_2, __pyx_temp+1-__pyx_t_5, 2+__pyx_t_5); if (unlikely(!__pyx_t_1)) __PYX_ERR(6, 212, __pyx_L1_error)
    __Pyx_XDECREF(__pyx_t_4); __pyx_t_4 = 0;
    __Pyx_GOTREF(__pyx_t_1);
    __Pyx_DECREF(__pyx_t_3); __pyx_t_3 = 0;
  } else
  #endif
  #if CYTHON_FAST_PYCCALL
  if (__Pyx_PyFastCFunction_Check(__pyx_t_2)) {
    PyObject *__pyx_temp[3] = {__pyx_t_4, __pyx_t_3, ((PyObject *)__pyx_v_sw)};
    __pyx_t_1 = __Pyx_PyCFunction_FastCall(__pyx_t_2, __pyx_temp+1-__pyx_t_5, 2+__pyx_t_5); if (unlikely(!__pyx_t_1)) __PYX_ERR(6, 212, __pyx_L1_error)
    __Pyx_XDECREF(__pyx_t_4); __pyx_t_4 = 0;
    __Pyx_GOTREF(__pyx_t_1);
    __Pyx_DECREF(__pyx_t_3); __pyx_t_3 = 0;
  } else
  #endif
  {
    __pyx_t_6 = PyTuple_New(2+__pyx_t_5); if (unlikely(!__pyx_t_6)) __PYX_ERR(6, 212, __pyx_L1_error)
    __Pyx_GOTREF(__pyx_t_6);
    if (__pyx_t_4) {
      __Pyx_GIVEREF(__pyx_t_4); PyTuple_SET_ITEM(__pyx_t_6, 0, __pyx_t_4); __pyx_t_4 = NULL;
    }
    __Pyx_GIVEREF(__pyx_t_3);
    PyTuple_SET_ITEM(__pyx_t_6, 0+__pyx_t_5, __pyx_t_3);
    __Pyx_INCREF(((PyObject *)__pyx_v_sw));
    __Pyx_GIVEREF(((PyObject *)__pyx_v_sw));
    PyTuple_SET_ITEM(__pyx_t_6, 1+__pyx_t_5, ((PyObject *)__pyx_v_sw));
    __pyx_t_3 = 0;
    __pyx_t_1 = __Pyx_PyObject_Call(__pyx_t_2, __pyx_t_6, NULL); if (unlikely(!__pyx_t_1)) __PYX_ERR(6, 212, __pyx_L1_error)
    __Pyx_GOTREF(__pyx_t_1);
    __Pyx_DECREF(__pyx_t_6); __pyx_t_6 = 0;
  }
  __Pyx_DECREF(__pyx_t_2); __pyx_t_2 = 0;
  __Pyx_DECREF(__pyx_t_1); __pyx_t_1 = 0;

  /* "src/python/grpcio/grpc/_cython/_cygrpc/grpc_gevent.pyx.pxi":206
 *   socket_read_async_cython(socket_wrapper)
 * 
 * cdef void socket_read(grpc_custom_socket* socket, char* buffer,             # <<<<<<<<<<<<<<
 *                       size_t length, grpc_custom_read_callback cb) except * with gil:
 *   sw = <SocketWrapper>socket.impl
 */

  /* function exit code */
  goto __pyx_L0;
  __pyx_L1_error:;
  __Pyx_XDECREF(__pyx_t_1);
  __Pyx_XDECREF(__pyx_t_2);
  __Pyx_XDECREF(__pyx_t_3);
  __Pyx_XDECREF(__pyx_t_4);
  __Pyx_XDECREF(__pyx_t_6);
  __Pyx_AddTraceback("grpc._cython.cygrpc.socket_read", __pyx_clineno, __pyx_lineno, __pyx_filename);
  __pyx_L0:;
  __Pyx_XDECREF((PyObject *)__pyx_v_sw);
  __Pyx_RefNannyFinishContext();
  #ifdef WITH_THREAD
  __Pyx_PyGILState_Release(__pyx_gilstate_save);
  #endif
}

/* "src/python/grpcio/grpc/_cython/_cygrpc/grpc_gevent.pyx.pxi":214
 *   _spawn_greenlet(socket_read_async, sw)
 * 
 * cdef grpc_error* socket_getpeername(grpc_custom_socket* socket,             # <<<<<<<<<<<<<<
 *                                     const grpc_sockaddr* addr,
 *                                     int* length) except * with gil:
 */

static struct grpc_error *__pyx_f_4grpc_7_cython_6cygrpc_socket_getpeername(struct grpc_custom_socket *__pyx_v_socket, grpc_sockaddr const *__pyx_v_addr, int *__pyx_v_length) {
  PyObject *__pyx_v_peer = NULL;
  grpc_resolved_address __pyx_v_c_addr;
  PyObject *__pyx_v_hostname = NULL;
  struct grpc_error *__pyx_r;
  __Pyx_RefNannyDeclarations
  PyObject *__pyx_t_1 = NULL;
  PyObject *__pyx_t_2 = NULL;
  PyObject *__pyx_t_3 = NULL;
  char *__pyx_t_4;
  int __pyx_t_5;
  size_t __pyx_t_6;
  #ifdef WITH_THREAD
  PyGILState_STATE __pyx_gilstate_save = __Pyx_PyGILState_Ensure();
  #endif
  __Pyx_RefNannySetupContext("socket_getpeername", 0);

  /* "src/python/grpcio/grpc/_cython/_cygrpc/grpc_gevent.pyx.pxi":218
 *                                     int* length) except * with gil:
 *   cdef char* src_buf
 *   peer = (<SocketWrapper>socket.impl).socket.getpeername()             # <<<<<<<<<<<<<<
 * 
 *   cdef grpc_resolved_address c_addr
 */
  __pyx_t_2 = __Pyx_PyObject_GetAttrStr(((struct __pyx_obj_4grpc_7_cython_6cygrpc_SocketWrapper *)__pyx_v_socket->impl)->socket, __pyx_n_s_getpeername); if (unlikely(!__pyx_t_2)) __PYX_ERR(6, 218, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_2);
  __pyx_t_3 = NULL;
  if (CYTHON_UNPACK_METHODS && likely(PyMethod_Check(__pyx_t_2))) {
    __pyx_t_3 = PyMethod_GET_SELF(__pyx_t_2);
    if (likely(__pyx_t_3)) {
      PyObject* function = PyMethod_GET_FUNCTION(__pyx_t_2);
      __Pyx_INCREF(__pyx_t_3);
      __Pyx_INCREF(function);
      __Pyx_DECREF_SET(__pyx_t_2, function);
    }
  }
  if (__pyx_t_3) {
    __pyx_t_1 = __Pyx_PyObject_CallOneArg(__pyx_t_2, __pyx_t_3); if (unlikely(!__pyx_t_1)) __PYX_ERR(6, 218, __pyx_L1_error)
    __Pyx_DECREF(__pyx_t_3); __pyx_t_3 = 0;
  } else {
    __pyx_t_1 = __Pyx_PyObject_CallNoArg(__pyx_t_2); if (unlikely(!__pyx_t_1)) __PYX_ERR(6, 218, __pyx_L1_error)
  }
  __Pyx_GOTREF(__pyx_t_1);
  __Pyx_DECREF(__pyx_t_2); __pyx_t_2 = 0;
  __pyx_v_peer = __pyx_t_1;
  __pyx_t_1 = 0;

  /* "src/python/grpcio/grpc/_cython/_cygrpc/grpc_gevent.pyx.pxi":221
 * 
 *   cdef grpc_resolved_address c_addr
 *   hostname = str_to_bytes(peer[0])             # <<<<<<<<<<<<<<
 *   grpc_string_to_sockaddr(&c_addr, hostname, peer[1])
 *   string.memcpy(<void*>addr, <void*>c_addr.addr, c_addr.len)
 */
  __pyx_t_1 = __Pyx_GetItemInt(__pyx_v_peer, 0, long, 1, __Pyx_PyInt_From_long, 0, 0, 1); if (unlikely(!__pyx_t_1)) __PYX_ERR(6, 221, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_1);
  __pyx_t_2 = __pyx_f_4grpc_7_cython_6cygrpc_str_to_bytes(__pyx_t_1); if (unlikely(!__pyx_t_2)) __PYX_ERR(6, 221, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_2);
  __Pyx_DECREF(__pyx_t_1); __pyx_t_1 = 0;
  __pyx_v_hostname = ((PyObject*)__pyx_t_2);
  __pyx_t_2 = 0;

  /* "src/python/grpcio/grpc/_cython/_cygrpc/grpc_gevent.pyx.pxi":222
 *   cdef grpc_resolved_address c_addr
 *   hostname = str_to_bytes(peer[0])
 *   grpc_string_to_sockaddr(&c_addr, hostname, peer[1])             # <<<<<<<<<<<<<<
 *   string.memcpy(<void*>addr, <void*>c_addr.addr, c_addr.len)
 *   length[0] = c_addr.len
 */
  if (unlikely(__pyx_v_hostname == Py_None)) {
    PyErr_SetString(PyExc_TypeError, "expected bytes, NoneType found");
    __PYX_ERR(6, 222, __pyx_L1_error)
  }
  __pyx_t_4 = __Pyx_PyBytes_AsWritableString(__pyx_v_hostname); if (unlikely((!__pyx_t_4) && PyErr_Occurred())) __PYX_ERR(6, 222, __pyx_L1_error)
  __pyx_t_2 = __Pyx_GetItemInt(__pyx_v_peer, 1, long, 1, __Pyx_PyInt_From_long, 0, 0, 1); if (unlikely(!__pyx_t_2)) __PYX_ERR(6, 222, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_2);
  __pyx_t_5 = __Pyx_PyInt_As_int(__pyx_t_2); if (unlikely((__pyx_t_5 == (int)-1) && PyErr_Occurred())) __PYX_ERR(6, 222, __pyx_L1_error)
  __Pyx_DECREF(__pyx_t_2); __pyx_t_2 = 0;
  grpc_string_to_sockaddr((&__pyx_v_c_addr), __pyx_t_4, __pyx_t_5);

  /* "src/python/grpcio/grpc/_cython/_cygrpc/grpc_gevent.pyx.pxi":223
 *   hostname = str_to_bytes(peer[0])
 *   grpc_string_to_sockaddr(&c_addr, hostname, peer[1])
 *   string.memcpy(<void*>addr, <void*>c_addr.addr, c_addr.len)             # <<<<<<<<<<<<<<
 *   length[0] = c_addr.len
 *   return grpc_error_none()
 */
  (void)(memcpy(((void *)__pyx_v_addr), ((void *)__pyx_v_c_addr.addr), __pyx_v_c_addr.len));

  /* "src/python/grpcio/grpc/_cython/_cygrpc/grpc_gevent.pyx.pxi":224
 *   grpc_string_to_sockaddr(&c_addr, hostname, peer[1])
 *   string.memcpy(<void*>addr, <void*>c_addr.addr, c_addr.len)
 *   length[0] = c_addr.len             # <<<<<<<<<<<<<<
 *   return grpc_error_none()
 * 
 */
  __pyx_t_6 = __pyx_v_c_addr.len;
  (__pyx_v_length[0]) = __pyx_t_6;

  /* "src/python/grpcio/grpc/_cython/_cygrpc/grpc_gevent.pyx.pxi":225
 *   string.memcpy(<void*>addr, <void*>c_addr.addr, c_addr.len)
 *   length[0] = c_addr.len
 *   return grpc_error_none()             # <<<<<<<<<<<<<<
 * 
 * cdef grpc_error* socket_getsockname(grpc_custom_socket* socket,
 */
  __pyx_r = __pyx_f_4grpc_7_cython_6cygrpc_grpc_error_none();
  goto __pyx_L0;

  /* "src/python/grpcio/grpc/_cython/_cygrpc/grpc_gevent.pyx.pxi":214
 *   _spawn_greenlet(socket_read_async, sw)
 * 
 * cdef grpc_error* socket_getpeername(grpc_custom_socket* socket,             # <<<<<<<<<<<<<<
 *                                     const grpc_sockaddr* addr,
 *                                     int* length) except * with gil:
 */

  /* function exit code */
  __pyx_L1_error:;
  __Pyx_XDECREF(__pyx_t_1);
  __Pyx_XDECREF(__pyx_t_2);
  __Pyx_XDECREF(__pyx_t_3);
  __Pyx_AddTraceback("grpc._cython.cygrpc.socket_getpeername", __pyx_clineno, __pyx_lineno, __pyx_filename);
  __pyx_r = 0;
  __pyx_L0:;
  __Pyx_XDECREF(__pyx_v_peer);
  __Pyx_XDECREF(__pyx_v_hostname);
  __Pyx_RefNannyFinishContext();
  #ifdef WITH_THREAD
  __Pyx_PyGILState_Release(__pyx_gilstate_save);
  #endif
  return __pyx_r;
}

/* "src/python/grpcio/grpc/_cython/_cygrpc/grpc_gevent.pyx.pxi":227
 *   return grpc_error_none()
 * 
 * cdef grpc_error* socket_getsockname(grpc_custom_socket* socket,             # <<<<<<<<<<<<<<
 *                                     const grpc_sockaddr* addr,
 *                                     int* length) except * with gil:
 */

static struct grpc_error *__pyx_f_4grpc_7_cython_6cygrpc_socket_getsockname(struct grpc_custom_socket *__pyx_v_socket, grpc_sockaddr const *__pyx_v_addr, int *__pyx_v_length) {
  grpc_resolved_address __pyx_v_c_addr;
  PyObject *__pyx_v_peer = NULL;
  PyObject *__pyx_v_hostname = NULL;
  struct grpc_error *__pyx_r;
  __Pyx_RefNannyDeclarations
  int __pyx_t_1;
  int __pyx_t_2;
  PyObject *__pyx_t_3 = NULL;
  PyObject *__pyx_t_4 = NULL;
  PyObject *__pyx_t_5 = NULL;
  char *__pyx_t_6;
  int __pyx_t_7;
  size_t __pyx_t_8;
  #ifdef WITH_THREAD
  PyGILState_STATE __pyx_gilstate_save = __Pyx_PyGILState_Ensure();
  #endif
  __Pyx_RefNannySetupContext("socket_getsockname", 0);

  /* "src/python/grpcio/grpc/_cython/_cygrpc/grpc_gevent.pyx.pxi":232
 *   cdef char* src_buf
 *   cdef grpc_resolved_address c_addr
 *   if (<SocketWrapper>socket.impl).socket is None:             # <<<<<<<<<<<<<<
 *     peer = ('0.0.0.0', 0)
 *   else:
 */
  __pyx_t_1 = (((struct __pyx_obj_4grpc_7_cython_6cygrpc_SocketWrapper *)__pyx_v_socket->impl)->socket == Py_None);
  __pyx_t_2 = (__pyx_t_1 != 0);
  if (__pyx_t_2) {

    /* "src/python/grpcio/grpc/_cython/_cygrpc/grpc_gevent.pyx.pxi":233
 *   cdef grpc_resolved_address c_addr
 *   if (<SocketWrapper>socket.impl).socket is None:
 *     peer = ('0.0.0.0', 0)             # <<<<<<<<<<<<<<
 *   else:
 *     peer = (<SocketWrapper>socket.impl).socket.getsockname()
 */
    __Pyx_INCREF(__pyx_tuple__140);
    __pyx_v_peer = __pyx_tuple__140;

    /* "src/python/grpcio/grpc/_cython/_cygrpc/grpc_gevent.pyx.pxi":232
 *   cdef char* src_buf
 *   cdef grpc_resolved_address c_addr
 *   if (<SocketWrapper>socket.impl).socket is None:             # <<<<<<<<<<<<<<
 *     peer = ('0.0.0.0', 0)
 *   else:
 */
    goto __pyx_L3;
  }

  /* "src/python/grpcio/grpc/_cython/_cygrpc/grpc_gevent.pyx.pxi":235
 *     peer = ('0.0.0.0', 0)
 *   else:
 *     peer = (<SocketWrapper>socket.impl).socket.getsockname()             # <<<<<<<<<<<<<<
 *   hostname = str_to_bytes(peer[0])
 *   grpc_string_to_sockaddr(&c_addr, hostname, peer[1])
 */
  /*else*/ {
    __pyx_t_4 = __Pyx_PyObject_GetAttrStr(((struct __pyx_obj_4grpc_7_cython_6cygrpc_SocketWrapper *)__pyx_v_socket->impl)->socket, __pyx_n_s_getsockname); if (unlikely(!__pyx_t_4)) __PYX_ERR(6, 235, __pyx_L1_error)
    __Pyx_GOTREF(__pyx_t_4);
    __pyx_t_5 = NULL;
    if (CYTHON_UNPACK_METHODS && likely(PyMethod_Check(__pyx_t_4))) {
      __pyx_t_5 = PyMethod_GET_SELF(__pyx_t_4);
      if (likely(__pyx_t_5)) {
        PyObject* function = PyMethod_GET_FUNCTION(__pyx_t_4);
        __Pyx_INCREF(__pyx_t_5);
        __Pyx_INCREF(function);
        __Pyx_DECREF_SET(__pyx_t_4, function);
      }
    }
    if (__pyx_t_5) {
      __pyx_t_3 = __Pyx_PyObject_CallOneArg(__pyx_t_4, __pyx_t_5); if (unlikely(!__pyx_t_3)) __PYX_ERR(6, 235, __pyx_L1_error)
      __Pyx_DECREF(__pyx_t_5); __pyx_t_5 = 0;
    } else {
      __pyx_t_3 = __Pyx_PyObject_CallNoArg(__pyx_t_4); if (unlikely(!__pyx_t_3)) __PYX_ERR(6, 235, __pyx_L1_error)
    }
    __Pyx_GOTREF(__pyx_t_3);
    __Pyx_DECREF(__pyx_t_4); __pyx_t_4 = 0;
    __pyx_v_peer = __pyx_t_3;
    __pyx_t_3 = 0;
  }
  __pyx_L3:;

  /* "src/python/grpcio/grpc/_cython/_cygrpc/grpc_gevent.pyx.pxi":236
 *   else:
 *     peer = (<SocketWrapper>socket.impl).socket.getsockname()
 *   hostname = str_to_bytes(peer[0])             # <<<<<<<<<<<<<<
 *   grpc_string_to_sockaddr(&c_addr, hostname, peer[1])
 *   string.memcpy(<void*>addr, <void*>c_addr.addr, c_addr.len)
 */
  __pyx_t_3 = __Pyx_GetItemInt(__pyx_v_peer, 0, long, 1, __Pyx_PyInt_From_long, 0, 0, 1); if (unlikely(!__pyx_t_3)) __PYX_ERR(6, 236, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_3);
  __pyx_t_4 = __pyx_f_4grpc_7_cython_6cygrpc_str_to_bytes(__pyx_t_3); if (unlikely(!__pyx_t_4)) __PYX_ERR(6, 236, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_4);
  __Pyx_DECREF(__pyx_t_3); __pyx_t_3 = 0;
  __pyx_v_hostname = ((PyObject*)__pyx_t_4);
  __pyx_t_4 = 0;

  /* "src/python/grpcio/grpc/_cython/_cygrpc/grpc_gevent.pyx.pxi":237
 *     peer = (<SocketWrapper>socket.impl).socket.getsockname()
 *   hostname = str_to_bytes(peer[0])
 *   grpc_string_to_sockaddr(&c_addr, hostname, peer[1])             # <<<<<<<<<<<<<<
 *   string.memcpy(<void*>addr, <void*>c_addr.addr, c_addr.len)
 *   length[0] = c_addr.len
 */
  if (unlikely(__pyx_v_hostname == Py_None)) {
    PyErr_SetString(PyExc_TypeError, "expected bytes, NoneType found");
    __PYX_ERR(6, 237, __pyx_L1_error)
  }
  __pyx_t_6 = __Pyx_PyBytes_AsWritableString(__pyx_v_hostname); if (unlikely((!__pyx_t_6) && PyErr_Occurred())) __PYX_ERR(6, 237, __pyx_L1_error)
  __pyx_t_4 = __Pyx_GetItemInt(__pyx_v_peer, 1, long, 1, __Pyx_PyInt_From_long, 0, 0, 1); if (unlikely(!__pyx_t_4)) __PYX_ERR(6, 237, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_4);
  __pyx_t_7 = __Pyx_PyInt_As_int(__pyx_t_4); if (unlikely((__pyx_t_7 == (int)-1) && PyErr_Occurred())) __PYX_ERR(6, 237, __pyx_L1_error)
  __Pyx_DECREF(__pyx_t_4); __pyx_t_4 = 0;
  grpc_string_to_sockaddr((&__pyx_v_c_addr), __pyx_t_6, __pyx_t_7);

  /* "src/python/grpcio/grpc/_cython/_cygrpc/grpc_gevent.pyx.pxi":238
 *   hostname = str_to_bytes(peer[0])
 *   grpc_string_to_sockaddr(&c_addr, hostname, peer[1])
 *   string.memcpy(<void*>addr, <void*>c_addr.addr, c_addr.len)             # <<<<<<<<<<<<<<
 *   length[0] = c_addr.len
 *   return grpc_error_none()
 */
  (void)(memcpy(((void *)__pyx_v_addr), ((void *)__pyx_v_c_addr.addr), __pyx_v_c_addr.len));

  /* "src/python/grpcio/grpc/_cython/_cygrpc/grpc_gevent.pyx.pxi":239
 *   grpc_string_to_sockaddr(&c_addr, hostname, peer[1])
 *   string.memcpy(<void*>addr, <void*>c_addr.addr, c_addr.len)
 *   length[0] = c_addr.len             # <<<<<<<<<<<<<<
 *   return grpc_error_none()
 * 
 */
  __pyx_t_8 = __pyx_v_c_addr.len;
  (__pyx_v_length[0]) = __pyx_t_8;

  /* "src/python/grpcio/grpc/_cython/_cygrpc/grpc_gevent.pyx.pxi":240
 *   string.memcpy(<void*>addr, <void*>c_addr.addr, c_addr.len)
 *   length[0] = c_addr.len
 *   return grpc_error_none()             # <<<<<<<<<<<<<<
 * 
 * def applysockopts(s):
 */
  __pyx_r = __pyx_f_4grpc_7_cython_6cygrpc_grpc_error_none();
  goto __pyx_L0;

  /* "src/python/grpcio/grpc/_cython/_cygrpc/grpc_gevent.pyx.pxi":227
 *   return grpc_error_none()
 * 
 * cdef grpc_error* socket_getsockname(grpc_custom_socket* socket,             # <<<<<<<<<<<<<<
 *                                     const grpc_sockaddr* addr,
 *                                     int* length) except * with gil:
 */

  /* function exit code */
  __pyx_L1_error:;
  __Pyx_XDECREF(__pyx_t_3);
  __Pyx_XDECREF(__pyx_t_4);
  __Pyx_XDECREF(__pyx_t_5);
  __Pyx_AddTraceback("grpc._cython.cygrpc.socket_getsockname", __pyx_clineno, __pyx_lineno, __pyx_filename);
  __pyx_r = 0;
  __pyx_L0:;
  __Pyx_XDECREF(__pyx_v_peer);
  __Pyx_XDECREF(__pyx_v_hostname);
  __Pyx_RefNannyFinishContext();
  #ifdef WITH_THREAD
  __Pyx_PyGILState_Release(__pyx_gilstate_save);
  #endif
  return __pyx_r;
}

/* "src/python/grpcio/grpc/_cython/_cygrpc/grpc_gevent.pyx.pxi":242
 *   return grpc_error_none()
 * 
 * def applysockopts(s):             # <<<<<<<<<<<<<<
 *   s.setsockopt(gevent_socket.SOL_SOCKET, gevent_socket.SO_REUSEADDR, 1)
 *   s.setsockopt(gevent_socket.IPPROTO_TCP, gevent_socket.TCP_NODELAY, True)
 */

/* Python wrapper */
static PyObject *__pyx_pw_4grpc_7_cython_6cygrpc_55applysockopts(PyObject *__pyx_self, PyObject *__pyx_v_s); /*proto*/
static PyMethodDef __pyx_mdef_4grpc_7_cython_6cygrpc_55applysockopts = {"applysockopts", (PyCFunction)__pyx_pw_4grpc_7_cython_6cygrpc_55applysockopts, METH_O, 0};
static PyObject *__pyx_pw_4grpc_7_cython_6cygrpc_55applysockopts(PyObject *__pyx_self, PyObject *__pyx_v_s) {
  PyObject *__pyx_r = 0;
  __Pyx_RefNannyDeclarations
  __Pyx_RefNannySetupContext("applysockopts (wrapper)", 0);
  __pyx_r = __pyx_pf_4grpc_7_cython_6cygrpc_54applysockopts(__pyx_self, ((PyObject *)__pyx_v_s));

  /* function exit code */
  __Pyx_RefNannyFinishContext();
  return __pyx_r;
}

static PyObject *__pyx_pf_4grpc_7_cython_6cygrpc_54applysockopts(CYTHON_UNUSED PyObject *__pyx_self, PyObject *__pyx_v_s) {
  PyObject *__pyx_r = NULL;
  __Pyx_RefNannyDeclarations
  PyObject *__pyx_t_1 = NULL;
  PyObject *__pyx_t_2 = NULL;
  PyObject *__pyx_t_3 = NULL;
  PyObject *__pyx_t_4 = NULL;
  PyObject *__pyx_t_5 = NULL;
  int __pyx_t_6;
  PyObject *__pyx_t_7 = NULL;
  __Pyx_RefNannySetupContext("applysockopts", 0);

  /* "src/python/grpcio/grpc/_cython/_cygrpc/grpc_gevent.pyx.pxi":243
 * 
 * def applysockopts(s):
 *   s.setsockopt(gevent_socket.SOL_SOCKET, gevent_socket.SO_REUSEADDR, 1)             # <<<<<<<<<<<<<<
 *   s.setsockopt(gevent_socket.IPPROTO_TCP, gevent_socket.TCP_NODELAY, True)
 * 
 */
  __pyx_t_2 = __Pyx_PyObject_GetAttrStr(__pyx_v_s, __pyx_n_s_setsockopt); if (unlikely(!__pyx_t_2)) __PYX_ERR(6, 243, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_2);
  __pyx_t_3 = __Pyx_GetModuleGlobalName(__pyx_n_s_gevent_socket); if (unlikely(!__pyx_t_3)) __PYX_ERR(6, 243, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_3);
  __pyx_t_4 = __Pyx_PyObject_GetAttrStr(__pyx_t_3, __pyx_n_s_SOL_SOCKET); if (unlikely(!__pyx_t_4)) __PYX_ERR(6, 243, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_4);
  __Pyx_DECREF(__pyx_t_3); __pyx_t_3 = 0;
  __pyx_t_3 = __Pyx_GetModuleGlobalName(__pyx_n_s_gevent_socket); if (unlikely(!__pyx_t_3)) __PYX_ERR(6, 243, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_3);
  __pyx_t_5 = __Pyx_PyObject_GetAttrStr(__pyx_t_3, __pyx_n_s_SO_REUSEADDR); if (unlikely(!__pyx_t_5)) __PYX_ERR(6, 243, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_5);
  __Pyx_DECREF(__pyx_t_3); __pyx_t_3 = 0;
  __pyx_t_3 = NULL;
  __pyx_t_6 = 0;
  if (CYTHON_UNPACK_METHODS && likely(PyMethod_Check(__pyx_t_2))) {
    __pyx_t_3 = PyMethod_GET_SELF(__pyx_t_2);
    if (likely(__pyx_t_3)) {
      PyObject* function = PyMethod_GET_FUNCTION(__pyx_t_2);
      __Pyx_INCREF(__pyx_t_3);
      __Pyx_INCREF(function);
      __Pyx_DECREF_SET(__pyx_t_2, function);
      __pyx_t_6 = 1;
    }
  }
  #if CYTHON_FAST_PYCALL
  if (PyFunction_Check(__pyx_t_2)) {
    PyObject *__pyx_temp[4] = {__pyx_t_3, __pyx_t_4, __pyx_t_5, __pyx_int_1};
    __pyx_t_1 = __Pyx_PyFunction_FastCall(__pyx_t_2, __pyx_temp+1-__pyx_t_6, 3+__pyx_t_6); if (unlikely(!__pyx_t_1)) __PYX_ERR(6, 243, __pyx_L1_error)
    __Pyx_XDECREF(__pyx_t_3); __pyx_t_3 = 0;
    __Pyx_GOTREF(__pyx_t_1);
    __Pyx_DECREF(__pyx_t_4); __pyx_t_4 = 0;
    __Pyx_DECREF(__pyx_t_5); __pyx_t_5 = 0;
  } else
  #endif
  #if CYTHON_FAST_PYCCALL
  if (__Pyx_PyFastCFunction_Check(__pyx_t_2)) {
    PyObject *__pyx_temp[4] = {__pyx_t_3, __pyx_t_4, __pyx_t_5, __pyx_int_1};
    __pyx_t_1 = __Pyx_PyCFunction_FastCall(__pyx_t_2, __pyx_temp+1-__pyx_t_6, 3+__pyx_t_6); if (unlikely(!__pyx_t_1)) __PYX_ERR(6, 243, __pyx_L1_error)
    __Pyx_XDECREF(__pyx_t_3); __pyx_t_3 = 0;
    __Pyx_GOTREF(__pyx_t_1);
    __Pyx_DECREF(__pyx_t_4); __pyx_t_4 = 0;
    __Pyx_DECREF(__pyx_t_5); __pyx_t_5 = 0;
  } else
  #endif
  {
    __pyx_t_7 = PyTuple_New(3+__pyx_t_6); if (unlikely(!__pyx_t_7)) __PYX_ERR(6, 243, __pyx_L1_error)
    __Pyx_GOTREF(__pyx_t_7);
    if (__pyx_t_3) {
      __Pyx_GIVEREF(__pyx_t_3); PyTuple_SET_ITEM(__pyx_t_7, 0, __pyx_t_3); __pyx_t_3 = NULL;
    }
    __Pyx_GIVEREF(__pyx_t_4);
    PyTuple_SET_ITEM(__pyx_t_7, 0+__pyx_t_6, __pyx_t_4);
    __Pyx_GIVEREF(__pyx_t_5);
    PyTuple_SET_ITEM(__pyx_t_7, 1+__pyx_t_6, __pyx_t_5);
    __Pyx_INCREF(__pyx_int_1);
    __Pyx_GIVEREF(__pyx_int_1);
    PyTuple_SET_ITEM(__pyx_t_7, 2+__pyx_t_6, __pyx_int_1);
    __pyx_t_4 = 0;
    __pyx_t_5 = 0;
    __pyx_t_1 = __Pyx_PyObject_Call(__pyx_t_2, __pyx_t_7, NULL); if (unlikely(!__pyx_t_1)) __PYX_ERR(6, 243, __pyx_L1_error)
    __Pyx_GOTREF(__pyx_t_1);
    __Pyx_DECREF(__pyx_t_7); __pyx_t_7 = 0;
  }
  __Pyx_DECREF(__pyx_t_2); __pyx_t_2 = 0;
  __Pyx_DECREF(__pyx_t_1); __pyx_t_1 = 0;

  /* "src/python/grpcio/grpc/_cython/_cygrpc/grpc_gevent.pyx.pxi":244
 * def applysockopts(s):
 *   s.setsockopt(gevent_socket.SOL_SOCKET, gevent_socket.SO_REUSEADDR, 1)
 *   s.setsockopt(gevent_socket.IPPROTO_TCP, gevent_socket.TCP_NODELAY, True)             # <<<<<<<<<<<<<<
 * 
 * cdef grpc_error* socket_bind(grpc_custom_socket* socket,
 */
  __pyx_t_2 = __Pyx_PyObject_GetAttrStr(__pyx_v_s, __pyx_n_s_setsockopt); if (unlikely(!__pyx_t_2)) __PYX_ERR(6, 244, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_2);
  __pyx_t_7 = __Pyx_GetModuleGlobalName(__pyx_n_s_gevent_socket); if (unlikely(!__pyx_t_7)) __PYX_ERR(6, 244, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_7);
  __pyx_t_5 = __Pyx_PyObject_GetAttrStr(__pyx_t_7, __pyx_n_s_IPPROTO_TCP); if (unlikely(!__pyx_t_5)) __PYX_ERR(6, 244, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_5);
  __Pyx_DECREF(__pyx_t_7); __pyx_t_7 = 0;
  __pyx_t_7 = __Pyx_GetModuleGlobalName(__pyx_n_s_gevent_socket); if (unlikely(!__pyx_t_7)) __PYX_ERR(6, 244, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_7);
  __pyx_t_4 = __Pyx_PyObject_GetAttrStr(__pyx_t_7, __pyx_n_s_TCP_NODELAY); if (unlikely(!__pyx_t_4)) __PYX_ERR(6, 244, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_4);
  __Pyx_DECREF(__pyx_t_7); __pyx_t_7 = 0;
  __pyx_t_7 = NULL;
  __pyx_t_6 = 0;
  if (CYTHON_UNPACK_METHODS && likely(PyMethod_Check(__pyx_t_2))) {
    __pyx_t_7 = PyMethod_GET_SELF(__pyx_t_2);
    if (likely(__pyx_t_7)) {
      PyObject* function = PyMethod_GET_FUNCTION(__pyx_t_2);
      __Pyx_INCREF(__pyx_t_7);
      __Pyx_INCREF(function);
      __Pyx_DECREF_SET(__pyx_t_2, function);
      __pyx_t_6 = 1;
    }
  }
  #if CYTHON_FAST_PYCALL
  if (PyFunction_Check(__pyx_t_2)) {
    PyObject *__pyx_temp[4] = {__pyx_t_7, __pyx_t_5, __pyx_t_4, Py_True};
    __pyx_t_1 = __Pyx_PyFunction_FastCall(__pyx_t_2, __pyx_temp+1-__pyx_t_6, 3+__pyx_t_6); if (unlikely(!__pyx_t_1)) __PYX_ERR(6, 244, __pyx_L1_error)
    __Pyx_XDECREF(__pyx_t_7); __pyx_t_7 = 0;
    __Pyx_GOTREF(__pyx_t_1);
    __Pyx_DECREF(__pyx_t_5); __pyx_t_5 = 0;
    __Pyx_DECREF(__pyx_t_4); __pyx_t_4 = 0;
  } else
  #endif
  #if CYTHON_FAST_PYCCALL
  if (__Pyx_PyFastCFunction_Check(__pyx_t_2)) {
    PyObject *__pyx_temp[4] = {__pyx_t_7, __pyx_t_5, __pyx_t_4, Py_True};
    __pyx_t_1 = __Pyx_PyCFunction_FastCall(__pyx_t_2, __pyx_temp+1-__pyx_t_6, 3+__pyx_t_6); if (unlikely(!__pyx_t_1)) __PYX_ERR(6, 244, __pyx_L1_error)
    __Pyx_XDECREF(__pyx_t_7); __pyx_t_7 = 0;
    __Pyx_GOTREF(__pyx_t_1);
    __Pyx_DECREF(__pyx_t_5); __pyx_t_5 = 0;
    __Pyx_DECREF(__pyx_t_4); __pyx_t_4 = 0;
  } else
  #endif
  {
    __pyx_t_3 = PyTuple_New(3+__pyx_t_6); if (unlikely(!__pyx_t_3)) __PYX_ERR(6, 244, __pyx_L1_error)
    __Pyx_GOTREF(__pyx_t_3);
    if (__pyx_t_7) {
      __Pyx_GIVEREF(__pyx_t_7); PyTuple_SET_ITEM(__pyx_t_3, 0, __pyx_t_7); __pyx_t_7 = NULL;
    }
    __Pyx_GIVEREF(__pyx_t_5);
    PyTuple_SET_ITEM(__pyx_t_3, 0+__pyx_t_6, __pyx_t_5);
    __Pyx_GIVEREF(__pyx_t_4);
    PyTuple_SET_ITEM(__pyx_t_3, 1+__pyx_t_6, __pyx_t_4);
    __Pyx_INCREF(Py_True);
    __Pyx_GIVEREF(Py_True);
    PyTuple_SET_ITEM(__pyx_t_3, 2+__pyx_t_6, Py_True);
    __pyx_t_5 = 0;
    __pyx_t_4 = 0;
    __pyx_t_1 = __Pyx_PyObject_Call(__pyx_t_2, __pyx_t_3, NULL); if (unlikely(!__pyx_t_1)) __PYX_ERR(6, 244, __pyx_L1_error)
    __Pyx_GOTREF(__pyx_t_1);
    __Pyx_DECREF(__pyx_t_3); __pyx_t_3 = 0;
  }
  __Pyx_DECREF(__pyx_t_2); __pyx_t_2 = 0;
  __Pyx_DECREF(__pyx_t_1); __pyx_t_1 = 0;

  /* "src/python/grpcio/grpc/_cython/_cygrpc/grpc_gevent.pyx.pxi":242
 *   return grpc_error_none()
 * 
 * def applysockopts(s):             # <<<<<<<<<<<<<<
 *   s.setsockopt(gevent_socket.SOL_SOCKET, gevent_socket.SO_REUSEADDR, 1)
 *   s.setsockopt(gevent_socket.IPPROTO_TCP, gevent_socket.TCP_NODELAY, True)
 */

  /* function exit code */
  __pyx_r = Py_None; __Pyx_INCREF(Py_None);
  goto __pyx_L0;
  __pyx_L1_error:;
  __Pyx_XDECREF(__pyx_t_1);
  __Pyx_XDECREF(__pyx_t_2);
  __Pyx_XDECREF(__pyx_t_3);
  __Pyx_XDECREF(__pyx_t_4);
  __Pyx_XDECREF(__pyx_t_5);
  __Pyx_XDECREF(__pyx_t_7);
  __Pyx_AddTraceback("grpc._cython.cygrpc.applysockopts", __pyx_clineno, __pyx_lineno, __pyx_filename);
  __pyx_r = NULL;
  __pyx_L0:;
  __Pyx_XGIVEREF(__pyx_r);
  __Pyx_RefNannyFinishContext();
  return __pyx_r;
}

/* "src/python/grpcio/grpc/_cython/_cygrpc/grpc_gevent.pyx.pxi":246
 *   s.setsockopt(gevent_socket.IPPROTO_TCP, gevent_socket.TCP_NODELAY, True)
 * 
 * cdef grpc_error* socket_bind(grpc_custom_socket* socket,             # <<<<<<<<<<<<<<
 *                              const grpc_sockaddr* addr,
 *                              size_t len, int flags) except * with gil:
 */

static struct grpc_error *__pyx_f_4grpc_7_cython_6cygrpc_socket_bind(struct grpc_custom_socket *__pyx_v_socket, grpc_sockaddr const *__pyx_v_addr, size_t __pyx_v_len, CYTHON_UNUSED int __pyx_v_flags) {
  PyObject *__pyx_v_addr_tuple = NULL;
  PyObject *__pyx_v_py_socket = NULL;
  CYTHON_UNUSED PyObject *__pyx_v_e = NULL;
  PyObject *__pyx_v_io_error = NULL;
  struct grpc_error *__pyx_r;
  __Pyx_RefNannyDeclarations
  PyObject *__pyx_t_1 = NULL;
  PyObject *__pyx_t_2 = NULL;
  PyObject *__pyx_t_3 = NULL;
  PyObject *__pyx_t_4 = NULL;
  PyObject *__pyx_t_5 = NULL;
  PyObject *__pyx_t_6 = NULL;
  PyObject *__pyx_t_7 = NULL;
  PyObject *__pyx_t_8 = NULL;
  PyObject *__pyx_t_9 = NULL;
  PyObject *__pyx_t_10 = NULL;
  PyObject *__pyx_t_11 = NULL;
  int __pyx_t_12;
  PyObject *__pyx_t_13 = NULL;
  PyObject *__pyx_t_14 = NULL;
  PyObject *__pyx_t_15 = NULL;
  #ifdef WITH_THREAD
  PyGILState_STATE __pyx_gilstate_save = __Pyx_PyGILState_Ensure();
  #endif
  __Pyx_RefNannySetupContext("socket_bind", 0);

  /* "src/python/grpcio/grpc/_cython/_cygrpc/grpc_gevent.pyx.pxi":249
 *                              const grpc_sockaddr* addr,
 *                              size_t len, int flags) except * with gil:
 *   addr_tuple = sockaddr_to_tuple(addr, len)             # <<<<<<<<<<<<<<
 *   try:
 *     try:
 */
  __pyx_t_1 = __pyx_f_4grpc_7_cython_6cygrpc_sockaddr_to_tuple(__pyx_v_addr, __pyx_v_len); if (unlikely(!__pyx_t_1)) __PYX_ERR(6, 249, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_1);
  __pyx_v_addr_tuple = __pyx_t_1;
  __pyx_t_1 = 0;

  /* "src/python/grpcio/grpc/_cython/_cygrpc/grpc_gevent.pyx.pxi":250
 *                              size_t len, int flags) except * with gil:
 *   addr_tuple = sockaddr_to_tuple(addr, len)
 *   try:             # <<<<<<<<<<<<<<
 *     try:
 *       py_socket = gevent_socket.socket(gevent_socket.AF_INET)
 */
  {
    __Pyx_PyThreadState_declare
    __Pyx_PyThreadState_assign
    __Pyx_ExceptionSave(&__pyx_t_2, &__pyx_t_3, &__pyx_t_4);
    __Pyx_XGOTREF(__pyx_t_2);
    __Pyx_XGOTREF(__pyx_t_3);
    __Pyx_XGOTREF(__pyx_t_4);
    /*try:*/ {

      /* "src/python/grpcio/grpc/_cython/_cygrpc/grpc_gevent.pyx.pxi":251
 *   addr_tuple = sockaddr_to_tuple(addr, len)
 *   try:
 *     try:             # <<<<<<<<<<<<<<
 *       py_socket = gevent_socket.socket(gevent_socket.AF_INET)
 *       applysockopts(py_socket)
 */
      {
        __Pyx_PyThreadState_declare
        __Pyx_PyThreadState_assign
        __Pyx_ExceptionSave(&__pyx_t_5, &__pyx_t_6, &__pyx_t_7);
        __Pyx_XGOTREF(__pyx_t_5);
        __Pyx_XGOTREF(__pyx_t_6);
        __Pyx_XGOTREF(__pyx_t_7);
        /*try:*/ {

          /* "src/python/grpcio/grpc/_cython/_cygrpc/grpc_gevent.pyx.pxi":252
 *   try:
 *     try:
 *       py_socket = gevent_socket.socket(gevent_socket.AF_INET)             # <<<<<<<<<<<<<<
 *       applysockopts(py_socket)
 *       py_socket.bind(addr_tuple)
 */
          __pyx_t_8 = __Pyx_GetModuleGlobalName(__pyx_n_s_gevent_socket); if (unlikely(!__pyx_t_8)) __PYX_ERR(6, 252, __pyx_L9_error)
          __Pyx_GOTREF(__pyx_t_8);
          __pyx_t_9 = __Pyx_PyObject_GetAttrStr(__pyx_t_8, __pyx_n_s_socket); if (unlikely(!__pyx_t_9)) __PYX_ERR(6, 252, __pyx_L9_error)
          __Pyx_GOTREF(__pyx_t_9);
          __Pyx_DECREF(__pyx_t_8); __pyx_t_8 = 0;
          __pyx_t_8 = __Pyx_GetModuleGlobalName(__pyx_n_s_gevent_socket); if (unlikely(!__pyx_t_8)) __PYX_ERR(6, 252, __pyx_L9_error)
          __Pyx_GOTREF(__pyx_t_8);
          __pyx_t_10 = __Pyx_PyObject_GetAttrStr(__pyx_t_8, __pyx_n_s_AF_INET); if (unlikely(!__pyx_t_10)) __PYX_ERR(6, 252, __pyx_L9_error)
          __Pyx_GOTREF(__pyx_t_10);
          __Pyx_DECREF(__pyx_t_8); __pyx_t_8 = 0;
          __pyx_t_8 = NULL;
          if (CYTHON_UNPACK_METHODS && unlikely(PyMethod_Check(__pyx_t_9))) {
            __pyx_t_8 = PyMethod_GET_SELF(__pyx_t_9);
            if (likely(__pyx_t_8)) {
              PyObject* function = PyMethod_GET_FUNCTION(__pyx_t_9);
              __Pyx_INCREF(__pyx_t_8);
              __Pyx_INCREF(function);
              __Pyx_DECREF_SET(__pyx_t_9, function);
            }
          }
          if (!__pyx_t_8) {
            __pyx_t_1 = __Pyx_PyObject_CallOneArg(__pyx_t_9, __pyx_t_10); if (unlikely(!__pyx_t_1)) __PYX_ERR(6, 252, __pyx_L9_error)
            __Pyx_DECREF(__pyx_t_10); __pyx_t_10 = 0;
            __Pyx_GOTREF(__pyx_t_1);
          } else {
            #if CYTHON_FAST_PYCALL
            if (PyFunction_Check(__pyx_t_9)) {
              PyObject *__pyx_temp[2] = {__pyx_t_8, __pyx_t_10};
              __pyx_t_1 = __Pyx_PyFunction_FastCall(__pyx_t_9, __pyx_temp+1-1, 1+1); if (unlikely(!__pyx_t_1)) __PYX_ERR(6, 252, __pyx_L9_error)
              __Pyx_XDECREF(__pyx_t_8); __pyx_t_8 = 0;
              __Pyx_GOTREF(__pyx_t_1);
              __Pyx_DECREF(__pyx_t_10); __pyx_t_10 = 0;
            } else
            #endif
            #if CYTHON_FAST_PYCCALL
            if (__Pyx_PyFastCFunction_Check(__pyx_t_9)) {
              PyObject *__pyx_temp[2] = {__pyx_t_8, __pyx_t_10};
              __pyx_t_1 = __Pyx_PyCFunction_FastCall(__pyx_t_9, __pyx_temp+1-1, 1+1); if (unlikely(!__pyx_t_1)) __PYX_ERR(6, 252, __pyx_L9_error)
              __Pyx_XDECREF(__pyx_t_8); __pyx_t_8 = 0;
              __Pyx_GOTREF(__pyx_t_1);
              __Pyx_DECREF(__pyx_t_10); __pyx_t_10 = 0;
            } else
            #endif
            {
              __pyx_t_11 = PyTuple_New(1+1); if (unlikely(!__pyx_t_11)) __PYX_ERR(6, 252, __pyx_L9_error)
              __Pyx_GOTREF(__pyx_t_11);
              __Pyx_GIVEREF(__pyx_t_8); PyTuple_SET_ITEM(__pyx_t_11, 0, __pyx_t_8); __pyx_t_8 = NULL;
              __Pyx_GIVEREF(__pyx_t_10);
              PyTuple_SET_ITEM(__pyx_t_11, 0+1, __pyx_t_10);
              __pyx_t_10 = 0;
              __pyx_t_1 = __Pyx_PyObject_Call(__pyx_t_9, __pyx_t_11, NULL); if (unlikely(!__pyx_t_1)) __PYX_ERR(6, 252, __pyx_L9_error)
              __Pyx_GOTREF(__pyx_t_1);
              __Pyx_DECREF(__pyx_t_11); __pyx_t_11 = 0;
            }
          }
          __Pyx_DECREF(__pyx_t_9); __pyx_t_9 = 0;
          __pyx_v_py_socket = __pyx_t_1;
          __pyx_t_1 = 0;

          /* "src/python/grpcio/grpc/_cython/_cygrpc/grpc_gevent.pyx.pxi":253
 *     try:
 *       py_socket = gevent_socket.socket(gevent_socket.AF_INET)
 *       applysockopts(py_socket)             # <<<<<<<<<<<<<<
 *       py_socket.bind(addr_tuple)
 *     except gevent_socket.gaierror as e:
 */
          __pyx_t_9 = __Pyx_GetModuleGlobalName(__pyx_n_s_applysockopts); if (unlikely(!__pyx_t_9)) __PYX_ERR(6, 253, __pyx_L9_error)
          __Pyx_GOTREF(__pyx_t_9);
          __pyx_t_11 = NULL;
          if (CYTHON_UNPACK_METHODS && unlikely(PyMethod_Check(__pyx_t_9))) {
            __pyx_t_11 = PyMethod_GET_SELF(__pyx_t_9);
            if (likely(__pyx_t_11)) {
              PyObject* function = PyMethod_GET_FUNCTION(__pyx_t_9);
              __Pyx_INCREF(__pyx_t_11);
              __Pyx_INCREF(function);
              __Pyx_DECREF_SET(__pyx_t_9, function);
            }
          }
          if (!__pyx_t_11) {
            __pyx_t_1 = __Pyx_PyObject_CallOneArg(__pyx_t_9, __pyx_v_py_socket); if (unlikely(!__pyx_t_1)) __PYX_ERR(6, 253, __pyx_L9_error)
            __Pyx_GOTREF(__pyx_t_1);
          } else {
            #if CYTHON_FAST_PYCALL
            if (PyFunction_Check(__pyx_t_9)) {
              PyObject *__pyx_temp[2] = {__pyx_t_11, __pyx_v_py_socket};
              __pyx_t_1 = __Pyx_PyFunction_FastCall(__pyx_t_9, __pyx_temp+1-1, 1+1); if (unlikely(!__pyx_t_1)) __PYX_ERR(6, 253, __pyx_L9_error)
              __Pyx_XDECREF(__pyx_t_11); __pyx_t_11 = 0;
              __Pyx_GOTREF(__pyx_t_1);
            } else
            #endif
            #if CYTHON_FAST_PYCCALL
            if (__Pyx_PyFastCFunction_Check(__pyx_t_9)) {
              PyObject *__pyx_temp[2] = {__pyx_t_11, __pyx_v_py_socket};
              __pyx_t_1 = __Pyx_PyCFunction_FastCall(__pyx_t_9, __pyx_temp+1-1, 1+1); if (unlikely(!__pyx_t_1)) __PYX_ERR(6, 253, __pyx_L9_error)
              __Pyx_XDECREF(__pyx_t_11); __pyx_t_11 = 0;
              __Pyx_GOTREF(__pyx_t_1);
            } else
            #endif
            {
              __pyx_t_10 = PyTuple_New(1+1); if (unlikely(!__pyx_t_10)) __PYX_ERR(6, 253, __pyx_L9_error)
              __Pyx_GOTREF(__pyx_t_10);
              __Pyx_GIVEREF(__pyx_t_11); PyTuple_SET_ITEM(__pyx_t_10, 0, __pyx_t_11); __pyx_t_11 = NULL;
              __Pyx_INCREF(__pyx_v_py_socket);
              __Pyx_GIVEREF(__pyx_v_py_socket);
              PyTuple_SET_ITEM(__pyx_t_10, 0+1, __pyx_v_py_socket);
              __pyx_t_1 = __Pyx_PyObject_Call(__pyx_t_9, __pyx_t_10, NULL); if (unlikely(!__pyx_t_1)) __PYX_ERR(6, 253, __pyx_L9_error)
              __Pyx_GOTREF(__pyx_t_1);
              __Pyx_DECREF(__pyx_t_10); __pyx_t_10 = 0;
            }
          }
          __Pyx_DECREF(__pyx_t_9); __pyx_t_9 = 0;
          __Pyx_DECREF(__pyx_t_1); __pyx_t_1 = 0;

          /* "src/python/grpcio/grpc/_cython/_cygrpc/grpc_gevent.pyx.pxi":254
 *       py_socket = gevent_socket.socket(gevent_socket.AF_INET)
 *       applysockopts(py_socket)
 *       py_socket.bind(addr_tuple)             # <<<<<<<<<<<<<<
 *     except gevent_socket.gaierror as e:
 *       py_socket = gevent_socket.socket(gevent_socket.AF_INET6)
 */
          __pyx_t_9 = __Pyx_PyObject_GetAttrStr(__pyx_v_py_socket, __pyx_n_s_bind); if (unlikely(!__pyx_t_9)) __PYX_ERR(6, 254, __pyx_L9_error)
          __Pyx_GOTREF(__pyx_t_9);
          __pyx_t_10 = NULL;
          if (CYTHON_UNPACK_METHODS && likely(PyMethod_Check(__pyx_t_9))) {
            __pyx_t_10 = PyMethod_GET_SELF(__pyx_t_9);
            if (likely(__pyx_t_10)) {
              PyObject* function = PyMethod_GET_FUNCTION(__pyx_t_9);
              __Pyx_INCREF(__pyx_t_10);
              __Pyx_INCREF(function);
              __Pyx_DECREF_SET(__pyx_t_9, function);
            }
          }
          if (!__pyx_t_10) {
            __pyx_t_1 = __Pyx_PyObject_CallOneArg(__pyx_t_9, __pyx_v_addr_tuple); if (unlikely(!__pyx_t_1)) __PYX_ERR(6, 254, __pyx_L9_error)
            __Pyx_GOTREF(__pyx_t_1);
          } else {
            #if CYTHON_FAST_PYCALL
            if (PyFunction_Check(__pyx_t_9)) {
              PyObject *__pyx_temp[2] = {__pyx_t_10, __pyx_v_addr_tuple};
              __pyx_t_1 = __Pyx_PyFunction_FastCall(__pyx_t_9, __pyx_temp+1-1, 1+1); if (unlikely(!__pyx_t_1)) __PYX_ERR(6, 254, __pyx_L9_error)
              __Pyx_XDECREF(__pyx_t_10); __pyx_t_10 = 0;
              __Pyx_GOTREF(__pyx_t_1);
            } else
            #endif
            #if CYTHON_FAST_PYCCALL
            if (__Pyx_PyFastCFunction_Check(__pyx_t_9)) {
              PyObject *__pyx_temp[2] = {__pyx_t_10, __pyx_v_addr_tuple};
              __pyx_t_1 = __Pyx_PyCFunction_FastCall(__pyx_t_9, __pyx_temp+1-1, 1+1); if (unlikely(!__pyx_t_1)) __PYX_ERR(6, 254, __pyx_L9_error)
              __Pyx_XDECREF(__pyx_t_10); __pyx_t_10 = 0;
              __Pyx_GOTREF(__pyx_t_1);
            } else
            #endif
            {
              __pyx_t_11 = PyTuple_New(1+1); if (unlikely(!__pyx_t_11)) __PYX_ERR(6, 254, __pyx_L9_error)
              __Pyx_GOTREF(__pyx_t_11);
              __Pyx_GIVEREF(__pyx_t_10); PyTuple_SET_ITEM(__pyx_t_11, 0, __pyx_t_10); __pyx_t_10 = NULL;
              __Pyx_INCREF(__pyx_v_addr_tuple);
              __Pyx_GIVEREF(__pyx_v_addr_tuple);
              PyTuple_SET_ITEM(__pyx_t_11, 0+1, __pyx_v_addr_tuple);
              __pyx_t_1 = __Pyx_PyObject_Call(__pyx_t_9, __pyx_t_11, NULL); if (unlikely(!__pyx_t_1)) __PYX_ERR(6, 254, __pyx_L9_error)
              __Pyx_GOTREF(__pyx_t_1);
              __Pyx_DECREF(__pyx_t_11); __pyx_t_11 = 0;
            }
          }
          __Pyx_DECREF(__pyx_t_9); __pyx_t_9 = 0;
          __Pyx_DECREF(__pyx_t_1); __pyx_t_1 = 0;

          /* "src/python/grpcio/grpc/_cython/_cygrpc/grpc_gevent.pyx.pxi":251
 *   addr_tuple = sockaddr_to_tuple(addr, len)
 *   try:
 *     try:             # <<<<<<<<<<<<<<
 *       py_socket = gevent_socket.socket(gevent_socket.AF_INET)
 *       applysockopts(py_socket)
 */
        }
        __Pyx_XDECREF(__pyx_t_5); __pyx_t_5 = 0;
        __Pyx_XDECREF(__pyx_t_6); __pyx_t_6 = 0;
        __Pyx_XDECREF(__pyx_t_7); __pyx_t_7 = 0;
        goto __pyx_L14_try_end;
        __pyx_L9_error:;
        __Pyx_XDECREF(__pyx_t_8); __pyx_t_8 = 0;
        __Pyx_XDECREF(__pyx_t_10); __pyx_t_10 = 0;
        __Pyx_XDECREF(__pyx_t_11); __pyx_t_11 = 0;
        __Pyx_XDECREF(__pyx_t_9); __pyx_t_9 = 0;
        __Pyx_XDECREF(__pyx_t_1); __pyx_t_1 = 0;

        /* "src/python/grpcio/grpc/_cython/_cygrpc/grpc_gevent.pyx.pxi":255
 *       applysockopts(py_socket)
 *       py_socket.bind(addr_tuple)
 *     except gevent_socket.gaierror as e:             # <<<<<<<<<<<<<<
 *       py_socket = gevent_socket.socket(gevent_socket.AF_INET6)
 *       applysockopts(py_socket)
 */
        __Pyx_ErrFetch(&__pyx_t_1, &__pyx_t_9, &__pyx_t_11);
        __pyx_t_10 = __Pyx_GetModuleGlobalName(__pyx_n_s_gevent_socket); if (unlikely(!__pyx_t_10)) __PYX_ERR(6, 255, __pyx_L11_except_error)
        __Pyx_GOTREF(__pyx_t_10);
        __pyx_t_8 = __Pyx_PyObject_GetAttrStr(__pyx_t_10, __pyx_n_s_gaierror); if (unlikely(!__pyx_t_8)) __PYX_ERR(6, 255, __pyx_L11_except_error)
        __Pyx_GOTREF(__pyx_t_8);
        __Pyx_DECREF(__pyx_t_10); __pyx_t_10 = 0;
        __pyx_t_12 = __Pyx_PyErr_GivenExceptionMatches(__pyx_t_1, __pyx_t_8);
        __Pyx_DECREF(__pyx_t_8); __pyx_t_8 = 0;
        __Pyx_ErrRestore(__pyx_t_1, __pyx_t_9, __pyx_t_11);
        __pyx_t_1 = 0; __pyx_t_9 = 0; __pyx_t_11 = 0;
        if (__pyx_t_12) {
          __Pyx_AddTraceback("grpc._cython.cygrpc.socket_bind", __pyx_clineno, __pyx_lineno, __pyx_filename);
          if (__Pyx_GetException(&__pyx_t_11, &__pyx_t_9, &__pyx_t_1) < 0) __PYX_ERR(6, 255, __pyx_L11_except_error)
          __Pyx_GOTREF(__pyx_t_11);
          __Pyx_GOTREF(__pyx_t_9);
          __Pyx_GOTREF(__pyx_t_1);
          __Pyx_INCREF(__pyx_t_9);
          __pyx_v_e = __pyx_t_9;

          /* "src/python/grpcio/grpc/_cython/_cygrpc/grpc_gevent.pyx.pxi":256
 *       py_socket.bind(addr_tuple)
 *     except gevent_socket.gaierror as e:
 *       py_socket = gevent_socket.socket(gevent_socket.AF_INET6)             # <<<<<<<<<<<<<<
 *       applysockopts(py_socket)
 *       py_socket.bind(addr_tuple)
 */
          __pyx_t_10 = __Pyx_GetModuleGlobalName(__pyx_n_s_gevent_socket); if (unlikely(!__pyx_t_10)) __PYX_ERR(6, 256, __pyx_L11_except_error)
          __Pyx_GOTREF(__pyx_t_10);
          __pyx_t_13 = __Pyx_PyObject_GetAttrStr(__pyx_t_10, __pyx_n_s_socket); if (unlikely(!__pyx_t_13)) __PYX_ERR(6, 256, __pyx_L11_except_error)
          __Pyx_GOTREF(__pyx_t_13);
          __Pyx_DECREF(__pyx_t_10); __pyx_t_10 = 0;
          __pyx_t_10 = __Pyx_GetModuleGlobalName(__pyx_n_s_gevent_socket); if (unlikely(!__pyx_t_10)) __PYX_ERR(6, 256, __pyx_L11_except_error)
          __Pyx_GOTREF(__pyx_t_10);
          __pyx_t_14 = __Pyx_PyObject_GetAttrStr(__pyx_t_10, __pyx_n_s_AF_INET6); if (unlikely(!__pyx_t_14)) __PYX_ERR(6, 256, __pyx_L11_except_error)
          __Pyx_GOTREF(__pyx_t_14);
          __Pyx_DECREF(__pyx_t_10); __pyx_t_10 = 0;
          __pyx_t_10 = NULL;
          if (CYTHON_UNPACK_METHODS && unlikely(PyMethod_Check(__pyx_t_13))) {
            __pyx_t_10 = PyMethod_GET_SELF(__pyx_t_13);
            if (likely(__pyx_t_10)) {
              PyObject* function = PyMethod_GET_FUNCTION(__pyx_t_13);
              __Pyx_INCREF(__pyx_t_10);
              __Pyx_INCREF(function);
              __Pyx_DECREF_SET(__pyx_t_13, function);
            }
          }
          if (!__pyx_t_10) {
            __pyx_t_8 = __Pyx_PyObject_CallOneArg(__pyx_t_13, __pyx_t_14); if (unlikely(!__pyx_t_8)) __PYX_ERR(6, 256, __pyx_L11_except_error)
            __Pyx_DECREF(__pyx_t_14); __pyx_t_14 = 0;
            __Pyx_GOTREF(__pyx_t_8);
          } else {
            #if CYTHON_FAST_PYCALL
            if (PyFunction_Check(__pyx_t_13)) {
              PyObject *__pyx_temp[2] = {__pyx_t_10, __pyx_t_14};
              __pyx_t_8 = __Pyx_PyFunction_FastCall(__pyx_t_13, __pyx_temp+1-1, 1+1); if (unlikely(!__pyx_t_8)) __PYX_ERR(6, 256, __pyx_L11_except_error)
              __Pyx_XDECREF(__pyx_t_10); __pyx_t_10 = 0;
              __Pyx_GOTREF(__pyx_t_8);
              __Pyx_DECREF(__pyx_t_14); __pyx_t_14 = 0;
            } else
            #endif
            #if CYTHON_FAST_PYCCALL
            if (__Pyx_PyFastCFunction_Check(__pyx_t_13)) {
              PyObject *__pyx_temp[2] = {__pyx_t_10, __pyx_t_14};
              __pyx_t_8 = __Pyx_PyCFunction_FastCall(__pyx_t_13, __pyx_temp+1-1, 1+1); if (unlikely(!__pyx_t_8)) __PYX_ERR(6, 256, __pyx_L11_except_error)
              __Pyx_XDECREF(__pyx_t_10); __pyx_t_10 = 0;
              __Pyx_GOTREF(__pyx_t_8);
              __Pyx_DECREF(__pyx_t_14); __pyx_t_14 = 0;
            } else
            #endif
            {
              __pyx_t_15 = PyTuple_New(1+1); if (unlikely(!__pyx_t_15)) __PYX_ERR(6, 256, __pyx_L11_except_error)
              __Pyx_GOTREF(__pyx_t_15);
              __Pyx_GIVEREF(__pyx_t_10); PyTuple_SET_ITEM(__pyx_t_15, 0, __pyx_t_10); __pyx_t_10 = NULL;
              __Pyx_GIVEREF(__pyx_t_14);
              PyTuple_SET_ITEM(__pyx_t_15, 0+1, __pyx_t_14);
              __pyx_t_14 = 0;
              __pyx_t_8 = __Pyx_PyObject_Call(__pyx_t_13, __pyx_t_15, NULL); if (unlikely(!__pyx_t_8)) __PYX_ERR(6, 256, __pyx_L11_except_error)
              __Pyx_GOTREF(__pyx_t_8);
              __Pyx_DECREF(__pyx_t_15); __pyx_t_15 = 0;
            }
          }
          __Pyx_DECREF(__pyx_t_13); __pyx_t_13 = 0;
          __Pyx_XDECREF_SET(__pyx_v_py_socket, __pyx_t_8);
          __pyx_t_8 = 0;

          /* "src/python/grpcio/grpc/_cython/_cygrpc/grpc_gevent.pyx.pxi":257
 *     except gevent_socket.gaierror as e:
 *       py_socket = gevent_socket.socket(gevent_socket.AF_INET6)
 *       applysockopts(py_socket)             # <<<<<<<<<<<<<<
 *       py_socket.bind(addr_tuple)
 *     (<SocketWrapper>socket.impl).socket = py_socket
 */
          __pyx_t_13 = __Pyx_GetModuleGlobalName(__pyx_n_s_applysockopts); if (unlikely(!__pyx_t_13)) __PYX_ERR(6, 257, __pyx_L11_except_error)
          __Pyx_GOTREF(__pyx_t_13);
          __pyx_t_15 = NULL;
          if (CYTHON_UNPACK_METHODS && unlikely(PyMethod_Check(__pyx_t_13))) {
            __pyx_t_15 = PyMethod_GET_SELF(__pyx_t_13);
            if (likely(__pyx_t_15)) {
              PyObject* function = PyMethod_GET_FUNCTION(__pyx_t_13);
              __Pyx_INCREF(__pyx_t_15);
              __Pyx_INCREF(function);
              __Pyx_DECREF_SET(__pyx_t_13, function);
            }
          }
          if (!__pyx_t_15) {
            __pyx_t_8 = __Pyx_PyObject_CallOneArg(__pyx_t_13, __pyx_v_py_socket); if (unlikely(!__pyx_t_8)) __PYX_ERR(6, 257, __pyx_L11_except_error)
            __Pyx_GOTREF(__pyx_t_8);
          } else {
            #if CYTHON_FAST_PYCALL
            if (PyFunction_Check(__pyx_t_13)) {
              PyObject *__pyx_temp[2] = {__pyx_t_15, __pyx_v_py_socket};
              __pyx_t_8 = __Pyx_PyFunction_FastCall(__pyx_t_13, __pyx_temp+1-1, 1+1); if (unlikely(!__pyx_t_8)) __PYX_ERR(6, 257, __pyx_L11_except_error)
              __Pyx_XDECREF(__pyx_t_15); __pyx_t_15 = 0;
              __Pyx_GOTREF(__pyx_t_8);
            } else
            #endif
            #if CYTHON_FAST_PYCCALL
            if (__Pyx_PyFastCFunction_Check(__pyx_t_13)) {
              PyObject *__pyx_temp[2] = {__pyx_t_15, __pyx_v_py_socket};
              __pyx_t_8 = __Pyx_PyCFunction_FastCall(__pyx_t_13, __pyx_temp+1-1, 1+1); if (unlikely(!__pyx_t_8)) __PYX_ERR(6, 257, __pyx_L11_except_error)
              __Pyx_XDECREF(__pyx_t_15); __pyx_t_15 = 0;
              __Pyx_GOTREF(__pyx_t_8);
            } else
            #endif
            {
              __pyx_t_14 = PyTuple_New(1+1); if (unlikely(!__pyx_t_14)) __PYX_ERR(6, 257, __pyx_L11_except_error)
              __Pyx_GOTREF(__pyx_t_14);
              __Pyx_GIVEREF(__pyx_t_15); PyTuple_SET_ITEM(__pyx_t_14, 0, __pyx_t_15); __pyx_t_15 = NULL;
              __Pyx_INCREF(__pyx_v_py_socket);
              __Pyx_GIVEREF(__pyx_v_py_socket);
              PyTuple_SET_ITEM(__pyx_t_14, 0+1, __pyx_v_py_socket);
              __pyx_t_8 = __Pyx_PyObject_Call(__pyx_t_13, __pyx_t_14, NULL); if (unlikely(!__pyx_t_8)) __PYX_ERR(6, 257, __pyx_L11_except_error)
              __Pyx_GOTREF(__pyx_t_8);
              __Pyx_DECREF(__pyx_t_14); __pyx_t_14 = 0;
            }
          }
          __Pyx_DECREF(__pyx_t_13); __pyx_t_13 = 0;
          __Pyx_DECREF(__pyx_t_8); __pyx_t_8 = 0;

          /* "src/python/grpcio/grpc/_cython/_cygrpc/grpc_gevent.pyx.pxi":258
 *       py_socket = gevent_socket.socket(gevent_socket.AF_INET6)
 *       applysockopts(py_socket)
 *       py_socket.bind(addr_tuple)             # <<<<<<<<<<<<<<
 *     (<SocketWrapper>socket.impl).socket = py_socket
 *   except IOError as io_error:
 */
          __pyx_t_13 = __Pyx_PyObject_GetAttrStr(__pyx_v_py_socket, __pyx_n_s_bind); if (unlikely(!__pyx_t_13)) __PYX_ERR(6, 258, __pyx_L11_except_error)
          __Pyx_GOTREF(__pyx_t_13);
          __pyx_t_14 = NULL;
          if (CYTHON_UNPACK_METHODS && likely(PyMethod_Check(__pyx_t_13))) {
            __pyx_t_14 = PyMethod_GET_SELF(__pyx_t_13);
            if (likely(__pyx_t_14)) {
              PyObject* function = PyMethod_GET_FUNCTION(__pyx_t_13);
              __Pyx_INCREF(__pyx_t_14);
              __Pyx_INCREF(function);
              __Pyx_DECREF_SET(__pyx_t_13, function);
            }
          }
          if (!__pyx_t_14) {
            __pyx_t_8 = __Pyx_PyObject_CallOneArg(__pyx_t_13, __pyx_v_addr_tuple); if (unlikely(!__pyx_t_8)) __PYX_ERR(6, 258, __pyx_L11_except_error)
            __Pyx_GOTREF(__pyx_t_8);
          } else {
            #if CYTHON_FAST_PYCALL
            if (PyFunction_Check(__pyx_t_13)) {
              PyObject *__pyx_temp[2] = {__pyx_t_14, __pyx_v_addr_tuple};
              __pyx_t_8 = __Pyx_PyFunction_FastCall(__pyx_t_13, __pyx_temp+1-1, 1+1); if (unlikely(!__pyx_t_8)) __PYX_ERR(6, 258, __pyx_L11_except_error)
              __Pyx_XDECREF(__pyx_t_14); __pyx_t_14 = 0;
              __Pyx_GOTREF(__pyx_t_8);
            } else
            #endif
            #if CYTHON_FAST_PYCCALL
            if (__Pyx_PyFastCFunction_Check(__pyx_t_13)) {
              PyObject *__pyx_temp[2] = {__pyx_t_14, __pyx_v_addr_tuple};
              __pyx_t_8 = __Pyx_PyCFunction_FastCall(__pyx_t_13, __pyx_temp+1-1, 1+1); if (unlikely(!__pyx_t_8)) __PYX_ERR(6, 258, __pyx_L11_except_error)
              __Pyx_XDECREF(__pyx_t_14); __pyx_t_14 = 0;
              __Pyx_GOTREF(__pyx_t_8);
            } else
            #endif
            {
              __pyx_t_15 = PyTuple_New(1+1); if (unlikely(!__pyx_t_15)) __PYX_ERR(6, 258, __pyx_L11_except_error)
              __Pyx_GOTREF(__pyx_t_15);
              __Pyx_GIVEREF(__pyx_t_14); PyTuple_SET_ITEM(__pyx_t_15, 0, __pyx_t_14); __pyx_t_14 = NULL;
              __Pyx_INCREF(__pyx_v_addr_tuple);
              __Pyx_GIVEREF(__pyx_v_addr_tuple);
              PyTuple_SET_ITEM(__pyx_t_15, 0+1, __pyx_v_addr_tuple);
              __pyx_t_8 = __Pyx_PyObject_Call(__pyx_t_13, __pyx_t_15, NULL); if (unlikely(!__pyx_t_8)) __PYX_ERR(6, 258, __pyx_L11_except_error)
              __Pyx_GOTREF(__pyx_t_8);
              __Pyx_DECREF(__pyx_t_15); __pyx_t_15 = 0;
            }
          }
          __Pyx_DECREF(__pyx_t_13); __pyx_t_13 = 0;
          __Pyx_DECREF(__pyx_t_8); __pyx_t_8 = 0;
          __Pyx_DECREF(__pyx_t_11); __pyx_t_11 = 0;
          __Pyx_DECREF(__pyx_t_9); __pyx_t_9 = 0;
          __Pyx_DECREF(__pyx_t_1); __pyx_t_1 = 0;
          goto __pyx_L10_exception_handled;
        }
        goto __pyx_L11_except_error;
        __pyx_L11_except_error:;

        /* "src/python/grpcio/grpc/_cython/_cygrpc/grpc_gevent.pyx.pxi":251
 *   addr_tuple = sockaddr_to_tuple(addr, len)
 *   try:
 *     try:             # <<<<<<<<<<<<<<
 *       py_socket = gevent_socket.socket(gevent_socket.AF_INET)
 *       applysockopts(py_socket)
 */
        __Pyx_XGIVEREF(__pyx_t_5);
        __Pyx_XGIVEREF(__pyx_t_6);
        __Pyx_XGIVEREF(__pyx_t_7);
        __Pyx_ExceptionReset(__pyx_t_5, __pyx_t_6, __pyx_t_7);
        goto __pyx_L3_error;
        __pyx_L10_exception_handled:;
        __Pyx_XGIVEREF(__pyx_t_5);
        __Pyx_XGIVEREF(__pyx_t_6);
        __Pyx_XGIVEREF(__pyx_t_7);
        __Pyx_ExceptionReset(__pyx_t_5, __pyx_t_6, __pyx_t_7);
        __pyx_L14_try_end:;
      }

      /* "src/python/grpcio/grpc/_cython/_cygrpc/grpc_gevent.pyx.pxi":259
 *       applysockopts(py_socket)
 *       py_socket.bind(addr_tuple)
 *     (<SocketWrapper>socket.impl).socket = py_socket             # <<<<<<<<<<<<<<
 *   except IOError as io_error:
 *     return socket_error("bind", str(io_error))
 */
      __Pyx_INCREF(__pyx_v_py_socket);
      __Pyx_GIVEREF(__pyx_v_py_socket);
      __Pyx_GOTREF(((struct __pyx_obj_4grpc_7_cython_6cygrpc_SocketWrapper *)__pyx_v_socket->impl)->socket);
      __Pyx_DECREF(((struct __pyx_obj_4grpc_7_cython_6cygrpc_SocketWrapper *)__pyx_v_socket->impl)->socket);
      ((struct __pyx_obj_4grpc_7_cython_6cygrpc_SocketWrapper *)__pyx_v_socket->impl)->socket = __pyx_v_py_socket;

      /* "src/python/grpcio/grpc/_cython/_cygrpc/grpc_gevent.pyx.pxi":250
 *                              size_t len, int flags) except * with gil:
 *   addr_tuple = sockaddr_to_tuple(addr, len)
 *   try:             # <<<<<<<<<<<<<<
 *     try:
 *       py_socket = gevent_socket.socket(gevent_socket.AF_INET)
 */
    }

    /* "src/python/grpcio/grpc/_cython/_cygrpc/grpc_gevent.pyx.pxi":263
 *     return socket_error("bind", str(io_error))
 *   else:
 *     return grpc_error_none()             # <<<<<<<<<<<<<<
 * 
 * cdef grpc_error* socket_listen(grpc_custom_socket* socket) except * with gil:
 */
    /*else:*/ {
      __pyx_r = __pyx_f_4grpc_7_cython_6cygrpc_grpc_error_none();
      goto __pyx_L6_except_return;
    }
    __pyx_L3_error:;
    __Pyx_XDECREF(__pyx_t_10); __pyx_t_10 = 0;
    __Pyx_XDECREF(__pyx_t_14); __pyx_t_14 = 0;
    __Pyx_XDECREF(__pyx_t_15); __pyx_t_15 = 0;
    __Pyx_XDECREF(__pyx_t_13); __pyx_t_13 = 0;
    __Pyx_XDECREF(__pyx_t_8); __pyx_t_8 = 0;
    __Pyx_XDECREF(__pyx_t_11); __pyx_t_11 = 0;
    __Pyx_XDECREF(__pyx_t_9); __pyx_t_9 = 0;
    __Pyx_XDECREF(__pyx_t_1); __pyx_t_1 = 0;

    /* "src/python/grpcio/grpc/_cython/_cygrpc/grpc_gevent.pyx.pxi":260
 *       py_socket.bind(addr_tuple)
 *     (<SocketWrapper>socket.impl).socket = py_socket
 *   except IOError as io_error:             # <<<<<<<<<<<<<<
 *     return socket_error("bind", str(io_error))
 *   else:
 */
    __pyx_t_12 = __Pyx_PyErr_ExceptionMatches(__pyx_builtin_IOError);
    if (__pyx_t_12) {
      __Pyx_AddTraceback("grpc._cython.cygrpc.socket_bind", __pyx_clineno, __pyx_lineno, __pyx_filename);
      if (__Pyx_GetException(&__pyx_t_1, &__pyx_t_9, &__pyx_t_11) < 0) __PYX_ERR(6, 260, __pyx_L5_except_error)
      __Pyx_GOTREF(__pyx_t_1);
      __Pyx_GOTREF(__pyx_t_9);
      __Pyx_GOTREF(__pyx_t_11);
      __Pyx_INCREF(__pyx_t_9);
      __pyx_v_io_error = __pyx_t_9;

      /* "src/python/grpcio/grpc/_cython/_cygrpc/grpc_gevent.pyx.pxi":261
 *     (<SocketWrapper>socket.impl).socket = py_socket
 *   except IOError as io_error:
 *     return socket_error("bind", str(io_error))             # <<<<<<<<<<<<<<
 *   else:
 *     return grpc_error_none()
 */
      __pyx_t_8 = __Pyx_PyObject_CallOneArg(((PyObject *)(&PyString_Type)), __pyx_v_io_error); if (unlikely(!__pyx_t_8)) __PYX_ERR(6, 261, __pyx_L5_except_error)
      __Pyx_GOTREF(__pyx_t_8);
      if (!(likely(PyString_CheckExact(__pyx_t_8))||((__pyx_t_8) == Py_None)||(PyErr_Format(PyExc_TypeError, "Expected %.16s, got %.200s", "str", Py_TYPE(__pyx_t_8)->tp_name), 0))) __PYX_ERR(6, 261, __pyx_L5_except_error)
      __pyx_r = __pyx_f_4grpc_7_cython_6cygrpc_socket_error(__pyx_n_s_bind, ((PyObject*)__pyx_t_8));
      __Pyx_DECREF(__pyx_t_8); __pyx_t_8 = 0;
      __Pyx_DECREF(__pyx_t_1); __pyx_t_1 = 0;
      __Pyx_DECREF(__pyx_t_9); __pyx_t_9 = 0;
      __Pyx_DECREF(__pyx_t_11); __pyx_t_11 = 0;
      goto __pyx_L6_except_return;
    }
    goto __pyx_L5_except_error;
    __pyx_L5_except_error:;

    /* "src/python/grpcio/grpc/_cython/_cygrpc/grpc_gevent.pyx.pxi":250
 *                              size_t len, int flags) except * with gil:
 *   addr_tuple = sockaddr_to_tuple(addr, len)
 *   try:             # <<<<<<<<<<<<<<
 *     try:
 *       py_socket = gevent_socket.socket(gevent_socket.AF_INET)
 */
    __Pyx_XGIVEREF(__pyx_t_2);
    __Pyx_XGIVEREF(__pyx_t_3);
    __Pyx_XGIVEREF(__pyx_t_4);
    __Pyx_ExceptionReset(__pyx_t_2, __pyx_t_3, __pyx_t_4);
    goto __pyx_L1_error;
    __pyx_L6_except_return:;
    __Pyx_XGIVEREF(__pyx_t_2);
    __Pyx_XGIVEREF(__pyx_t_3);
    __Pyx_XGIVEREF(__pyx_t_4);
    __Pyx_ExceptionReset(__pyx_t_2, __pyx_t_3, __pyx_t_4);
    goto __pyx_L0;
  }

  /* "src/python/grpcio/grpc/_cython/_cygrpc/grpc_gevent.pyx.pxi":246
 *   s.setsockopt(gevent_socket.IPPROTO_TCP, gevent_socket.TCP_NODELAY, True)
 * 
 * cdef grpc_error* socket_bind(grpc_custom_socket* socket,             # <<<<<<<<<<<<<<
 *                              const grpc_sockaddr* addr,
 *                              size_t len, int flags) except * with gil:
 */

  /* function exit code */
  __pyx_L1_error:;
  __Pyx_XDECREF(__pyx_t_1);
  __Pyx_XDECREF(__pyx_t_8);
  __Pyx_XDECREF(__pyx_t_9);
  __Pyx_XDECREF(__pyx_t_10);
  __Pyx_XDECREF(__pyx_t_11);
  __Pyx_XDECREF(__pyx_t_13);
  __Pyx_XDECREF(__pyx_t_14);
  __Pyx_XDECREF(__pyx_t_15);
  __Pyx_AddTraceback("grpc._cython.cygrpc.socket_bind", __pyx_clineno, __pyx_lineno, __pyx_filename);
  __pyx_r = 0;
  __pyx_L0:;
  __Pyx_XDECREF(__pyx_v_addr_tuple);
  __Pyx_XDECREF(__pyx_v_py_socket);
  __Pyx_XDECREF(__pyx_v_e);
  __Pyx_XDECREF(__pyx_v_io_error);
  __Pyx_RefNannyFinishContext();
  #ifdef WITH_THREAD
  __Pyx_PyGILState_Release(__pyx_gilstate_save);
  #endif
  return __pyx_r;
}

/* "src/python/grpcio/grpc/_cython/_cygrpc/grpc_gevent.pyx.pxi":265
 *     return grpc_error_none()
 * 
 * cdef grpc_error* socket_listen(grpc_custom_socket* socket) except * with gil:             # <<<<<<<<<<<<<<
 *   (<SocketWrapper>socket.impl).socket.listen(50)
 *   return grpc_error_none()
 */

static struct grpc_error *__pyx_f_4grpc_7_cython_6cygrpc_socket_listen(struct grpc_custom_socket *__pyx_v_socket) {
  struct grpc_error *__pyx_r;
  __Pyx_RefNannyDeclarations
  PyObject *__pyx_t_1 = NULL;
  PyObject *__pyx_t_2 = NULL;
  #ifdef WITH_THREAD
  PyGILState_STATE __pyx_gilstate_save = __Pyx_PyGILState_Ensure();
  #endif
  __Pyx_RefNannySetupContext("socket_listen", 0);

  /* "src/python/grpcio/grpc/_cython/_cygrpc/grpc_gevent.pyx.pxi":266
 * 
 * cdef grpc_error* socket_listen(grpc_custom_socket* socket) except * with gil:
 *   (<SocketWrapper>socket.impl).socket.listen(50)             # <<<<<<<<<<<<<<
 *   return grpc_error_none()
 * 
 */
  __pyx_t_1 = __Pyx_PyObject_GetAttrStr(((struct __pyx_obj_4grpc_7_cython_6cygrpc_SocketWrapper *)__pyx_v_socket->impl)->socket, __pyx_n_s_listen); if (unlikely(!__pyx_t_1)) __PYX_ERR(6, 266, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_1);
  __pyx_t_2 = __Pyx_PyObject_Call(__pyx_t_1, __pyx_tuple__141, NULL); if (unlikely(!__pyx_t_2)) __PYX_ERR(6, 266, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_2);
  __Pyx_DECREF(__pyx_t_1); __pyx_t_1 = 0;
  __Pyx_DECREF(__pyx_t_2); __pyx_t_2 = 0;

  /* "src/python/grpcio/grpc/_cython/_cygrpc/grpc_gevent.pyx.pxi":267
 * cdef grpc_error* socket_listen(grpc_custom_socket* socket) except * with gil:
 *   (<SocketWrapper>socket.impl).socket.listen(50)
 *   return grpc_error_none()             # <<<<<<<<<<<<<<
 * 
 * cdef void accept_callback_cython(SocketWrapper s) except *:
 */
  __pyx_r = __pyx_f_4grpc_7_cython_6cygrpc_grpc_error_none();
  goto __pyx_L0;

  /* "src/python/grpcio/grpc/_cython/_cygrpc/grpc_gevent.pyx.pxi":265
 *     return grpc_error_none()
 * 
 * cdef grpc_error* socket_listen(grpc_custom_socket* socket) except * with gil:             # <<<<<<<<<<<<<<
 *   (<SocketWrapper>socket.impl).socket.listen(50)
 *   return grpc_error_none()
 */

  /* function exit code */
  __pyx_L1_error:;
  __Pyx_XDECREF(__pyx_t_1);
  __Pyx_XDECREF(__pyx_t_2);
  __Pyx_AddTraceback("grpc._cython.cygrpc.socket_listen", __pyx_clineno, __pyx_lineno, __pyx_filename);
  __pyx_r = 0;
  __pyx_L0:;
  __Pyx_RefNannyFinishContext();
  #ifdef WITH_THREAD
  __Pyx_PyGILState_Release(__pyx_gilstate_save);
  #endif
  return __pyx_r;
}

/* "src/python/grpcio/grpc/_cython/_cygrpc/grpc_gevent.pyx.pxi":269
 *   return grpc_error_none()
 * 
 * cdef void accept_callback_cython(SocketWrapper s) except *:             # <<<<<<<<<<<<<<
 *    try:
 *      conn, address = s.socket.accept()
 */

static void __pyx_f_4grpc_7_cython_6cygrpc_accept_callback_cython(struct __pyx_obj_4grpc_7_cython_6cygrpc_SocketWrapper *__pyx_v_s) {
  PyObject *__pyx_v_conn = NULL;
  CYTHON_UNUSED PyObject *__pyx_v_address = NULL;
  struct __pyx_obj_4grpc_7_cython_6cygrpc_SocketWrapper *__pyx_v_sw = NULL;
  PyObject *__pyx_v_io_error = NULL;
  __Pyx_RefNannyDeclarations
  PyObject *__pyx_t_1 = NULL;
  PyObject *__pyx_t_2 = NULL;
  PyObject *__pyx_t_3 = NULL;
  PyObject *__pyx_t_4 = NULL;
  PyObject *__pyx_t_5 = NULL;
  PyObject *__pyx_t_6 = NULL;
  PyObject *__pyx_t_7 = NULL;
  PyObject *(*__pyx_t_8)(PyObject *);
  struct grpc_custom_socket *__pyx_t_9;
  int __pyx_t_10;
  int __pyx_t_11;
  __Pyx_RefNannySetupContext("accept_callback_cython", 0);

  /* "src/python/grpcio/grpc/_cython/_cygrpc/grpc_gevent.pyx.pxi":270
 * 
 * cdef void accept_callback_cython(SocketWrapper s) except *:
 *    try:             # <<<<<<<<<<<<<<
 *      conn, address = s.socket.accept()
 *      sw = SocketWrapper()
 */
  {
    __Pyx_PyThreadState_declare
    __Pyx_PyThreadState_assign
    __Pyx_ExceptionSave(&__pyx_t_1, &__pyx_t_2, &__pyx_t_3);
    __Pyx_XGOTREF(__pyx_t_1);
    __Pyx_XGOTREF(__pyx_t_2);
    __Pyx_XGOTREF(__pyx_t_3);
    /*try:*/ {

      /* "src/python/grpcio/grpc/_cython/_cygrpc/grpc_gevent.pyx.pxi":271
 * cdef void accept_callback_cython(SocketWrapper s) except *:
 *    try:
 *      conn, address = s.socket.accept()             # <<<<<<<<<<<<<<
 *      sw = SocketWrapper()
 *      sw.closed = False
 */
      __pyx_t_5 = __Pyx_PyObject_GetAttrStr(__pyx_v_s->socket, __pyx_n_s_accept); if (unlikely(!__pyx_t_5)) __PYX_ERR(6, 271, __pyx_L3_error)
      __Pyx_GOTREF(__pyx_t_5);
      __pyx_t_6 = NULL;
      if (CYTHON_UNPACK_METHODS && likely(PyMethod_Check(__pyx_t_5))) {
        __pyx_t_6 = PyMethod_GET_SELF(__pyx_t_5);
        if (likely(__pyx_t_6)) {
          PyObject* function = PyMethod_GET_FUNCTION(__pyx_t_5);
          __Pyx_INCREF(__pyx_t_6);
          __Pyx_INCREF(function);
          __Pyx_DECREF_SET(__pyx_t_5, function);
        }
      }
      if (__pyx_t_6) {
        __pyx_t_4 = __Pyx_PyObject_CallOneArg(__pyx_t_5, __pyx_t_6); if (unlikely(!__pyx_t_4)) __PYX_ERR(6, 271, __pyx_L3_error)
        __Pyx_DECREF(__pyx_t_6); __pyx_t_6 = 0;
      } else {
        __pyx_t_4 = __Pyx_PyObject_CallNoArg(__pyx_t_5); if (unlikely(!__pyx_t_4)) __PYX_ERR(6, 271, __pyx_L3_error)
      }
      __Pyx_GOTREF(__pyx_t_4);
      __Pyx_DECREF(__pyx_t_5); __pyx_t_5 = 0;
      if ((likely(PyTuple_CheckExact(__pyx_t_4))) || (PyList_CheckExact(__pyx_t_4))) {
        PyObject* sequence = __pyx_t_4;
        Py_ssize_t size = __Pyx_PySequence_SIZE(sequence);
        if (unlikely(size != 2)) {
          if (size > 2) __Pyx_RaiseTooManyValuesError(2);
          else if (size >= 0) __Pyx_RaiseNeedMoreValuesError(size);
          __PYX_ERR(6, 271, __pyx_L3_error)
        }
        #if CYTHON_ASSUME_SAFE_MACROS && !CYTHON_AVOID_BORROWED_REFS
        if (likely(PyTuple_CheckExact(sequence))) {
          __pyx_t_5 = PyTuple_GET_ITEM(sequence, 0); 
          __pyx_t_6 = PyTuple_GET_ITEM(sequence, 1); 
        } else {
          __pyx_t_5 = PyList_GET_ITEM(sequence, 0); 
          __pyx_t_6 = PyList_GET_ITEM(sequence, 1); 
        }
        __Pyx_INCREF(__pyx_t_5);
        __Pyx_INCREF(__pyx_t_6);
        #else
        __pyx_t_5 = PySequence_ITEM(sequence, 0); if (unlikely(!__pyx_t_5)) __PYX_ERR(6, 271, __pyx_L3_error)
        __Pyx_GOTREF(__pyx_t_5);
        __pyx_t_6 = PySequence_ITEM(sequence, 1); if (unlikely(!__pyx_t_6)) __PYX_ERR(6, 271, __pyx_L3_error)
        __Pyx_GOTREF(__pyx_t_6);
        #endif
        __Pyx_DECREF(__pyx_t_4); __pyx_t_4 = 0;
      } else {
        Py_ssize_t index = -1;
        __pyx_t_7 = PyObject_GetIter(__pyx_t_4); if (unlikely(!__pyx_t_7)) __PYX_ERR(6, 271, __pyx_L3_error)
        __Pyx_GOTREF(__pyx_t_7);
        __Pyx_DECREF(__pyx_t_4); __pyx_t_4 = 0;
        __pyx_t_8 = Py_TYPE(__pyx_t_7)->tp_iternext;
        index = 0; __pyx_t_5 = __pyx_t_8(__pyx_t_7); if (unlikely(!__pyx_t_5)) goto __pyx_L9_unpacking_failed;
        __Pyx_GOTREF(__pyx_t_5);
        index = 1; __pyx_t_6 = __pyx_t_8(__pyx_t_7); if (unlikely(!__pyx_t_6)) goto __pyx_L9_unpacking_failed;
        __Pyx_GOTREF(__pyx_t_6);
        if (__Pyx_IternextUnpackEndCheck(__pyx_t_8(__pyx_t_7), 2) < 0) __PYX_ERR(6, 271, __pyx_L3_error)
        __pyx_t_8 = NULL;
        __Pyx_DECREF(__pyx_t_7); __pyx_t_7 = 0;
        goto __pyx_L10_unpacking_done;
        __pyx_L9_unpacking_failed:;
        __Pyx_DECREF(__pyx_t_7); __pyx_t_7 = 0;
        __pyx_t_8 = NULL;
        if (__Pyx_IterFinish() == 0) __Pyx_RaiseNeedMoreValuesError(index);
        __PYX_ERR(6, 271, __pyx_L3_error)
        __pyx_L10_unpacking_done:;
      }
      __pyx_v_conn = __pyx_t_5;
      __pyx_t_5 = 0;
      __pyx_v_address = __pyx_t_6;
      __pyx_t_6 = 0;

      /* "src/python/grpcio/grpc/_cython/_cygrpc/grpc_gevent.pyx.pxi":272
 *    try:
 *      conn, address = s.socket.accept()
 *      sw = SocketWrapper()             # <<<<<<<<<<<<<<
 *      sw.closed = False
 *      sw.c_socket = s.accepting_socket
 */
      __pyx_t_4 = __Pyx_PyObject_CallNoArg(((PyObject *)__pyx_ptype_4grpc_7_cython_6cygrpc_SocketWrapper)); if (unlikely(!__pyx_t_4)) __PYX_ERR(6, 272, __pyx_L3_error)
      __Pyx_GOTREF(__pyx_t_4);
      __pyx_v_sw = ((struct __pyx_obj_4grpc_7_cython_6cygrpc_SocketWrapper *)__pyx_t_4);
      __pyx_t_4 = 0;

      /* "src/python/grpcio/grpc/_cython/_cygrpc/grpc_gevent.pyx.pxi":273
 *      conn, address = s.socket.accept()
 *      sw = SocketWrapper()
 *      sw.closed = False             # <<<<<<<<<<<<<<
 *      sw.c_socket = s.accepting_socket
 *      sw.sockopts = []
 */
      __Pyx_INCREF(Py_False);
      __Pyx_GIVEREF(Py_False);
      __Pyx_GOTREF(__pyx_v_sw->closed);
      __Pyx_DECREF(__pyx_v_sw->closed);
      __pyx_v_sw->closed = Py_False;

      /* "src/python/grpcio/grpc/_cython/_cygrpc/grpc_gevent.pyx.pxi":274
 *      sw = SocketWrapper()
 *      sw.closed = False
 *      sw.c_socket = s.accepting_socket             # <<<<<<<<<<<<<<
 *      sw.sockopts = []
 *      sw.socket = conn
 */
      __pyx_t_9 = __pyx_v_s->accepting_socket;
      __pyx_v_sw->c_socket = __pyx_t_9;

      /* "src/python/grpcio/grpc/_cython/_cygrpc/grpc_gevent.pyx.pxi":275
 *      sw.closed = False
 *      sw.c_socket = s.accepting_socket
 *      sw.sockopts = []             # <<<<<<<<<<<<<<
 *      sw.socket = conn
 *      sw.c_socket.impl = <void*>sw
 */
      __pyx_t_4 = PyList_New(0); if (unlikely(!__pyx_t_4)) __PYX_ERR(6, 275, __pyx_L3_error)
      __Pyx_GOTREF(__pyx_t_4);
      __Pyx_GIVEREF(__pyx_t_4);
      __Pyx_GOTREF(__pyx_v_sw->sockopts);
      __Pyx_DECREF(__pyx_v_sw->sockopts);
      __pyx_v_sw->sockopts = __pyx_t_4;
      __pyx_t_4 = 0;

      /* "src/python/grpcio/grpc/_cython/_cygrpc/grpc_gevent.pyx.pxi":276
 *      sw.c_socket = s.accepting_socket
 *      sw.sockopts = []
 *      sw.socket = conn             # <<<<<<<<<<<<<<
 *      sw.c_socket.impl = <void*>sw
 *      sw.accepting_socket = NULL
 */
      __Pyx_INCREF(__pyx_v_conn);
      __Pyx_GIVEREF(__pyx_v_conn);
      __Pyx_GOTREF(__pyx_v_sw->socket);
      __Pyx_DECREF(__pyx_v_sw->socket);
      __pyx_v_sw->socket = __pyx_v_conn;

      /* "src/python/grpcio/grpc/_cython/_cygrpc/grpc_gevent.pyx.pxi":277
 *      sw.sockopts = []
 *      sw.socket = conn
 *      sw.c_socket.impl = <void*>sw             # <<<<<<<<<<<<<<
 *      sw.accepting_socket = NULL
 *      cpython.Py_INCREF(sw)
 */
      __pyx_v_sw->c_socket->impl = ((void *)__pyx_v_sw);

      /* "src/python/grpcio/grpc/_cython/_cygrpc/grpc_gevent.pyx.pxi":278
 *      sw.socket = conn
 *      sw.c_socket.impl = <void*>sw
 *      sw.accepting_socket = NULL             # <<<<<<<<<<<<<<
 *      cpython.Py_INCREF(sw)
 *      s.accepting_socket = NULL
 */
      __pyx_v_sw->accepting_socket = NULL;

      /* "src/python/grpcio/grpc/_cython/_cygrpc/grpc_gevent.pyx.pxi":279
 *      sw.c_socket.impl = <void*>sw
 *      sw.accepting_socket = NULL
 *      cpython.Py_INCREF(sw)             # <<<<<<<<<<<<<<
 *      s.accepting_socket = NULL
 *      s.accept_cb(<grpc_custom_socket*>s.c_socket, sw.c_socket, grpc_error_none())
 */
      Py_INCREF(((PyObject *)__pyx_v_sw));

      /* "src/python/grpcio/grpc/_cython/_cygrpc/grpc_gevent.pyx.pxi":280
 *      sw.accepting_socket = NULL
 *      cpython.Py_INCREF(sw)
 *      s.accepting_socket = NULL             # <<<<<<<<<<<<<<
 *      s.accept_cb(<grpc_custom_socket*>s.c_socket, sw.c_socket, grpc_error_none())
 *    except IOError as io_error:
 */
      __pyx_v_s->accepting_socket = NULL;

      /* "src/python/grpcio/grpc/_cython/_cygrpc/grpc_gevent.pyx.pxi":281
 *      cpython.Py_INCREF(sw)
 *      s.accepting_socket = NULL
 *      s.accept_cb(<grpc_custom_socket*>s.c_socket, sw.c_socket, grpc_error_none())             # <<<<<<<<<<<<<<
 *    except IOError as io_error:
 *       #TODO actual error
 */
      __pyx_v_s->accept_cb(((struct grpc_custom_socket *)__pyx_v_s->c_socket), __pyx_v_sw->c_socket, __pyx_f_4grpc_7_cython_6cygrpc_grpc_error_none());

      /* "src/python/grpcio/grpc/_cython/_cygrpc/grpc_gevent.pyx.pxi":270
 * 
 * cdef void accept_callback_cython(SocketWrapper s) except *:
 *    try:             # <<<<<<<<<<<<<<
 *      conn, address = s.socket.accept()
 *      sw = SocketWrapper()
 */
    }
    __Pyx_XDECREF(__pyx_t_1); __pyx_t_1 = 0;
    __Pyx_XDECREF(__pyx_t_2); __pyx_t_2 = 0;
    __Pyx_XDECREF(__pyx_t_3); __pyx_t_3 = 0;
    goto __pyx_L8_try_end;
    __pyx_L3_error:;
    __Pyx_XDECREF(__pyx_t_7); __pyx_t_7 = 0;
    __Pyx_XDECREF(__pyx_t_5); __pyx_t_5 = 0;
    __Pyx_XDECREF(__pyx_t_6); __pyx_t_6 = 0;
    __Pyx_XDECREF(__pyx_t_4); __pyx_t_4 = 0;

    /* "src/python/grpcio/grpc/_cython/_cygrpc/grpc_gevent.pyx.pxi":282
 *      s.accepting_socket = NULL
 *      s.accept_cb(<grpc_custom_socket*>s.c_socket, sw.c_socket, grpc_error_none())
 *    except IOError as io_error:             # <<<<<<<<<<<<<<
 *       #TODO actual error
 *       s.accepting_socket = NULL
 */
    __pyx_t_10 = __Pyx_PyErr_ExceptionMatches(__pyx_builtin_IOError);
    if (__pyx_t_10) {
      __Pyx_AddTraceback("grpc._cython.cygrpc.accept_callback_cython", __pyx_clineno, __pyx_lineno, __pyx_filename);
      if (__Pyx_GetException(&__pyx_t_4, &__pyx_t_6, &__pyx_t_5) < 0) __PYX_ERR(6, 282, __pyx_L5_except_error)
      __Pyx_GOTREF(__pyx_t_4);
      __Pyx_GOTREF(__pyx_t_6);
      __Pyx_GOTREF(__pyx_t_5);
      __Pyx_INCREF(__pyx_t_6);
      __pyx_v_io_error = __pyx_t_6;

      /* "src/python/grpcio/grpc/_cython/_cygrpc/grpc_gevent.pyx.pxi":284
 *    except IOError as io_error:
 *       #TODO actual error
 *       s.accepting_socket = NULL             # <<<<<<<<<<<<<<
 *       s.accept_cb(<grpc_custom_socket*>s.c_socket, s.accepting_socket,
 *                   socket_error("accept", str(io_error)))
 */
      __pyx_v_s->accepting_socket = NULL;

      /* "src/python/grpcio/grpc/_cython/_cygrpc/grpc_gevent.pyx.pxi":286
 *       s.accepting_socket = NULL
 *       s.accept_cb(<grpc_custom_socket*>s.c_socket, s.accepting_socket,
 *                   socket_error("accept", str(io_error)))             # <<<<<<<<<<<<<<
 *       if s.closed:
 *         s.close_cb(<grpc_custom_socket*>s.c_socket)
 */
      __pyx_t_7 = __Pyx_PyObject_CallOneArg(((PyObject *)(&PyString_Type)), __pyx_v_io_error); if (unlikely(!__pyx_t_7)) __PYX_ERR(6, 286, __pyx_L5_except_error)
      __Pyx_GOTREF(__pyx_t_7);
      if (!(likely(PyString_CheckExact(__pyx_t_7))||((__pyx_t_7) == Py_None)||(PyErr_Format(PyExc_TypeError, "Expected %.16s, got %.200s", "str", Py_TYPE(__pyx_t_7)->tp_name), 0))) __PYX_ERR(6, 286, __pyx_L5_except_error)

      /* "src/python/grpcio/grpc/_cython/_cygrpc/grpc_gevent.pyx.pxi":285
 *       #TODO actual error
 *       s.accepting_socket = NULL
 *       s.accept_cb(<grpc_custom_socket*>s.c_socket, s.accepting_socket,             # <<<<<<<<<<<<<<
 *                   socket_error("accept", str(io_error)))
 *       if s.closed:
 */
      __pyx_v_s->accept_cb(((struct grpc_custom_socket *)__pyx_v_s->c_socket), __pyx_v_s->accepting_socket, __pyx_f_4grpc_7_cython_6cygrpc_socket_error(__pyx_n_s_accept, ((PyObject*)__pyx_t_7)));
      __Pyx_DECREF(__pyx_t_7); __pyx_t_7 = 0;

      /* "src/python/grpcio/grpc/_cython/_cygrpc/grpc_gevent.pyx.pxi":287
 *       s.accept_cb(<grpc_custom_socket*>s.c_socket, s.accepting_socket,
 *                   socket_error("accept", str(io_error)))
 *       if s.closed:             # <<<<<<<<<<<<<<
 *         s.close_cb(<grpc_custom_socket*>s.c_socket)
 *    g_event.set()
 */
      __pyx_t_11 = __Pyx_PyObject_IsTrue(__pyx_v_s->closed); if (unlikely(__pyx_t_11 < 0)) __PYX_ERR(6, 287, __pyx_L5_except_error)
      if (__pyx_t_11) {

        /* "src/python/grpcio/grpc/_cython/_cygrpc/grpc_gevent.pyx.pxi":288
 *                   socket_error("accept", str(io_error)))
 *       if s.closed:
 *         s.close_cb(<grpc_custom_socket*>s.c_socket)             # <<<<<<<<<<<<<<
 *    g_event.set()
 * 
 */
        __pyx_v_s->close_cb(((struct grpc_custom_socket *)__pyx_v_s->c_socket));

        /* "src/python/grpcio/grpc/_cython/_cygrpc/grpc_gevent.pyx.pxi":287
 *       s.accept_cb(<grpc_custom_socket*>s.c_socket, s.accepting_socket,
 *                   socket_error("accept", str(io_error)))
 *       if s.closed:             # <<<<<<<<<<<<<<
 *         s.close_cb(<grpc_custom_socket*>s.c_socket)
 *    g_event.set()
 */
      }
      __Pyx_DECREF(__pyx_t_4); __pyx_t_4 = 0;
      __Pyx_DECREF(__pyx_t_6); __pyx_t_6 = 0;
      __Pyx_DECREF(__pyx_t_5); __pyx_t_5 = 0;
      goto __pyx_L4_exception_handled;
    }
    goto __pyx_L5_except_error;
    __pyx_L5_except_error:;

    /* "src/python/grpcio/grpc/_cython/_cygrpc/grpc_gevent.pyx.pxi":270
 * 
 * cdef void accept_callback_cython(SocketWrapper s) except *:
 *    try:             # <<<<<<<<<<<<<<
 *      conn, address = s.socket.accept()
 *      sw = SocketWrapper()
 */
    __Pyx_XGIVEREF(__pyx_t_1);
    __Pyx_XGIVEREF(__pyx_t_2);
    __Pyx_XGIVEREF(__pyx_t_3);
    __Pyx_ExceptionReset(__pyx_t_1, __pyx_t_2, __pyx_t_3);
    goto __pyx_L1_error;
    __pyx_L4_exception_handled:;
    __Pyx_XGIVEREF(__pyx_t_1);
    __Pyx_XGIVEREF(__pyx_t_2);
    __Pyx_XGIVEREF(__pyx_t_3);
    __Pyx_ExceptionReset(__pyx_t_1, __pyx_t_2, __pyx_t_3);
    __pyx_L8_try_end:;
  }

  /* "src/python/grpcio/grpc/_cython/_cygrpc/grpc_gevent.pyx.pxi":289
 *       if s.closed:
 *         s.close_cb(<grpc_custom_socket*>s.c_socket)
 *    g_event.set()             # <<<<<<<<<<<<<<
 * 
 * def socket_accept_async(s):
 */
  __pyx_t_6 = __Pyx_GetModuleGlobalName(__pyx_n_s_g_event); if (unlikely(!__pyx_t_6)) __PYX_ERR(6, 289, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_6);
  __pyx_t_4 = __Pyx_PyObject_GetAttrStr(__pyx_t_6, __pyx_n_s_set); if (unlikely(!__pyx_t_4)) __PYX_ERR(6, 289, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_4);
  __Pyx_DECREF(__pyx_t_6); __pyx_t_6 = 0;
  __pyx_t_6 = NULL;
  if (CYTHON_UNPACK_METHODS && unlikely(PyMethod_Check(__pyx_t_4))) {
    __pyx_t_6 = PyMethod_GET_SELF(__pyx_t_4);
    if (likely(__pyx_t_6)) {
      PyObject* function = PyMethod_GET_FUNCTION(__pyx_t_4);
      __Pyx_INCREF(__pyx_t_6);
      __Pyx_INCREF(function);
      __Pyx_DECREF_SET(__pyx_t_4, function);
    }
  }
  if (__pyx_t_6) {
    __pyx_t_5 = __Pyx_PyObject_CallOneArg(__pyx_t_4, __pyx_t_6); if (unlikely(!__pyx_t_5)) __PYX_ERR(6, 289, __pyx_L1_error)
    __Pyx_DECREF(__pyx_t_6); __pyx_t_6 = 0;
  } else {
    __pyx_t_5 = __Pyx_PyObject_CallNoArg(__pyx_t_4); if (unlikely(!__pyx_t_5)) __PYX_ERR(6, 289, __pyx_L1_error)
  }
  __Pyx_GOTREF(__pyx_t_5);
  __Pyx_DECREF(__pyx_t_4); __pyx_t_4 = 0;
  __Pyx_DECREF(__pyx_t_5); __pyx_t_5 = 0;

  /* "src/python/grpcio/grpc/_cython/_cygrpc/grpc_gevent.pyx.pxi":269
 *   return grpc_error_none()
 * 
 * cdef void accept_callback_cython(SocketWrapper s) except *:             # <<<<<<<<<<<<<<
 *    try:
 *      conn, address = s.socket.accept()
 */

  /* function exit code */
  goto __pyx_L0;
  __pyx_L1_error:;
  __Pyx_XDECREF(__pyx_t_4);
  __Pyx_XDECREF(__pyx_t_5);
  __Pyx_XDECREF(__pyx_t_6);
  __Pyx_XDECREF(__pyx_t_7);
  __Pyx_AddTraceback("grpc._cython.cygrpc.accept_callback_cython", __pyx_clineno, __pyx_lineno, __pyx_filename);
  __pyx_L0:;
  __Pyx_XDECREF(__pyx_v_conn);
  __Pyx_XDECREF(__pyx_v_address);
  __Pyx_XDECREF((PyObject *)__pyx_v_sw);
  __Pyx_XDECREF(__pyx_v_io_error);
  __Pyx_RefNannyFinishContext();
}

/* "src/python/grpcio/grpc/_cython/_cygrpc/grpc_gevent.pyx.pxi":291
 *    g_event.set()
 * 
 * def socket_accept_async(s):             # <<<<<<<<<<<<<<
 *   accept_callback_cython(s)
 * 
 */

/* Python wrapper */
static PyObject *__pyx_pw_4grpc_7_cython_6cygrpc_57socket_accept_async(PyObject *__pyx_self, PyObject *__pyx_v_s); /*proto*/
static PyMethodDef __pyx_mdef_4grpc_7_cython_6cygrpc_57socket_accept_async = {"socket_accept_async", (PyCFunction)__pyx_pw_4grpc_7_cython_6cygrpc_57socket_accept_async, METH_O, 0};
static PyObject *__pyx_pw_4grpc_7_cython_6cygrpc_57socket_accept_async(PyObject *__pyx_self, PyObject *__pyx_v_s) {
  PyObject *__pyx_r = 0;
  __Pyx_RefNannyDeclarations
  __Pyx_RefNannySetupContext("socket_accept_async (wrapper)", 0);
  __pyx_r = __pyx_pf_4grpc_7_cython_6cygrpc_56socket_accept_async(__pyx_self, ((PyObject *)__pyx_v_s));

  /* function exit code */
  __Pyx_RefNannyFinishContext();
  return __pyx_r;
}

static PyObject *__pyx_pf_4grpc_7_cython_6cygrpc_56socket_accept_async(CYTHON_UNUSED PyObject *__pyx_self, PyObject *__pyx_v_s) {
  PyObject *__pyx_r = NULL;
  __Pyx_RefNannyDeclarations
  __Pyx_RefNannySetupContext("socket_accept_async", 0);

  /* "src/python/grpcio/grpc/_cython/_cygrpc/grpc_gevent.pyx.pxi":292
 * 
 * def socket_accept_async(s):
 *   accept_callback_cython(s)             # <<<<<<<<<<<<<<
 * 
 * cdef void socket_accept(grpc_custom_socket* socket, grpc_custom_socket* client,
 */
  if (!(likely(((__pyx_v_s) == Py_None) || likely(__Pyx_TypeTest(__pyx_v_s, __pyx_ptype_4grpc_7_cython_6cygrpc_SocketWrapper))))) __PYX_ERR(6, 292, __pyx_L1_error)
  __pyx_f_4grpc_7_cython_6cygrpc_accept_callback_cython(((struct __pyx_obj_4grpc_7_cython_6cygrpc_SocketWrapper *)__pyx_v_s)); if (unlikely(PyErr_Occurred())) __PYX_ERR(6, 292, __pyx_L1_error)

  /* "src/python/grpcio/grpc/_cython/_cygrpc/grpc_gevent.pyx.pxi":291
 *    g_event.set()
 * 
 * def socket_accept_async(s):             # <<<<<<<<<<<<<<
 *   accept_callback_cython(s)
 * 
 */

  /* function exit code */
  __pyx_r = Py_None; __Pyx_INCREF(Py_None);
  goto __pyx_L0;
  __pyx_L1_error:;
  __Pyx_AddTraceback("grpc._cython.cygrpc.socket_accept_async", __pyx_clineno, __pyx_lineno, __pyx_filename);
  __pyx_r = NULL;
  __pyx_L0:;
  __Pyx_XGIVEREF(__pyx_r);
  __Pyx_RefNannyFinishContext();
  return __pyx_r;
}

/* "src/python/grpcio/grpc/_cython/_cygrpc/grpc_gevent.pyx.pxi":294
 *   accept_callback_cython(s)
 * 
 * cdef void socket_accept(grpc_custom_socket* socket, grpc_custom_socket* client,             # <<<<<<<<<<<<<<
 *                         grpc_custom_accept_callback cb) except * with gil:
 *   sw = <SocketWrapper>socket.impl
 */

static void __pyx_f_4grpc_7_cython_6cygrpc_socket_accept(struct grpc_custom_socket *__pyx_v_socket, struct grpc_custom_socket *__pyx_v_client, grpc_custom_accept_callback __pyx_v_cb) {
  struct __pyx_obj_4grpc_7_cython_6cygrpc_SocketWrapper *__pyx_v_sw = NULL;
  __Pyx_RefNannyDeclarations
  PyObject *__pyx_t_1 = NULL;
  PyObject *__pyx_t_2 = NULL;
  PyObject *__pyx_t_3 = NULL;
  PyObject *__pyx_t_4 = NULL;
  int __pyx_t_5;
  PyObject *__pyx_t_6 = NULL;
  #ifdef WITH_THREAD
  PyGILState_STATE __pyx_gilstate_save = __Pyx_PyGILState_Ensure();
  #endif
  __Pyx_RefNannySetupContext("socket_accept", 0);

  /* "src/python/grpcio/grpc/_cython/_cygrpc/grpc_gevent.pyx.pxi":296
 * cdef void socket_accept(grpc_custom_socket* socket, grpc_custom_socket* client,
 *                         grpc_custom_accept_callback cb) except * with gil:
 *   sw = <SocketWrapper>socket.impl             # <<<<<<<<<<<<<<
 *   sw.accepting_socket = client
 *   sw.accept_cb = cb
 */
  __pyx_t_1 = ((PyObject *)__pyx_v_socket->impl);
  __Pyx_INCREF(__pyx_t_1);
  __pyx_v_sw = ((struct __pyx_obj_4grpc_7_cython_6cygrpc_SocketWrapper *)__pyx_t_1);
  __pyx_t_1 = 0;

  /* "src/python/grpcio/grpc/_cython/_cygrpc/grpc_gevent.pyx.pxi":297
 *                         grpc_custom_accept_callback cb) except * with gil:
 *   sw = <SocketWrapper>socket.impl
 *   sw.accepting_socket = client             # <<<<<<<<<<<<<<
 *   sw.accept_cb = cb
 *   _spawn_greenlet(socket_accept_async, sw)
 */
  __pyx_v_sw->accepting_socket = __pyx_v_client;

  /* "src/python/grpcio/grpc/_cython/_cygrpc/grpc_gevent.pyx.pxi":298
 *   sw = <SocketWrapper>socket.impl
 *   sw.accepting_socket = client
 *   sw.accept_cb = cb             # <<<<<<<<<<<<<<
 *   _spawn_greenlet(socket_accept_async, sw)
 * 
 */
  __pyx_v_sw->accept_cb = __pyx_v_cb;

  /* "src/python/grpcio/grpc/_cython/_cygrpc/grpc_gevent.pyx.pxi":299
 *   sw.accepting_socket = client
 *   sw.accept_cb = cb
 *   _spawn_greenlet(socket_accept_async, sw)             # <<<<<<<<<<<<<<
 * 
 * #####################################
 */
  __pyx_t_2 = __Pyx_GetModuleGlobalName(__pyx_n_s_spawn_greenlet); if (unlikely(!__pyx_t_2)) __PYX_ERR(6, 299, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_2);
  __pyx_t_3 = __Pyx_GetModuleGlobalName(__pyx_n_s_socket_accept_async); if (unlikely(!__pyx_t_3)) __PYX_ERR(6, 299, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_3);
  __pyx_t_4 = NULL;
  __pyx_t_5 = 0;
  if (CYTHON_UNPACK_METHODS && unlikely(PyMethod_Check(__pyx_t_2))) {
    __pyx_t_4 = PyMethod_GET_SELF(__pyx_t_2);
    if (likely(__pyx_t_4)) {
      PyObject* function = PyMethod_GET_FUNCTION(__pyx_t_2);
      __Pyx_INCREF(__pyx_t_4);
      __Pyx_INCREF(function);
      __Pyx_DECREF_SET(__pyx_t_2, function);
      __pyx_t_5 = 1;
    }
  }
  #if CYTHON_FAST_PYCALL
  if (PyFunction_Check(__pyx_t_2)) {
    PyObject *__pyx_temp[3] = {__pyx_t_4, __pyx_t_3, ((PyObject *)__pyx_v_sw)};
    __pyx_t_1 = __Pyx_PyFunction_FastCall(__pyx_t_2, __pyx_temp+1-__pyx_t_5, 2+__pyx_t_5); if (unlikely(!__pyx_t_1)) __PYX_ERR(6, 299, __pyx_L1_error)
    __Pyx_XDECREF(__pyx_t_4); __pyx_t_4 = 0;
    __Pyx_GOTREF(__pyx_t_1);
    __Pyx_DECREF(__pyx_t_3); __pyx_t_3 = 0;
  } else
  #endif
  #if CYTHON_FAST_PYCCALL
  if (__Pyx_PyFastCFunction_Check(__pyx_t_2)) {
    PyObject *__pyx_temp[3] = {__pyx_t_4, __pyx_t_3, ((PyObject *)__pyx_v_sw)};
    __pyx_t_1 = __Pyx_PyCFunction_FastCall(__pyx_t_2, __pyx_temp+1-__pyx_t_5, 2+__pyx_t_5); if (unlikely(!__pyx_t_1)) __PYX_ERR(6, 299, __pyx_L1_error)
    __Pyx_XDECREF(__pyx_t_4); __pyx_t_4 = 0;
    __Pyx_GOTREF(__pyx_t_1);
    __Pyx_DECREF(__pyx_t_3); __pyx_t_3 = 0;
  } else
  #endif
  {
    __pyx_t_6 = PyTuple_New(2+__pyx_t_5); if (unlikely(!__pyx_t_6)) __PYX_ERR(6, 299, __pyx_L1_error)
    __Pyx_GOTREF(__pyx_t_6);
    if (__pyx_t_4) {
      __Pyx_GIVEREF(__pyx_t_4); PyTuple_SET_ITEM(__pyx_t_6, 0, __pyx_t_4); __pyx_t_4 = NULL;
    }
    __Pyx_GIVEREF(__pyx_t_3);
    PyTuple_SET_ITEM(__pyx_t_6, 0+__pyx_t_5, __pyx_t_3);
    __Pyx_INCREF(((PyObject *)__pyx_v_sw));
    __Pyx_GIVEREF(((PyObject *)__pyx_v_sw));
    PyTuple_SET_ITEM(__pyx_t_6, 1+__pyx_t_5, ((PyObject *)__pyx_v_sw));
    __pyx_t_3 = 0;
    __pyx_t_1 = __Pyx_PyObject_Call(__pyx_t_2, __pyx_t_6, NULL); if (unlikely(!__pyx_t_1)) __PYX_ERR(6, 299, __pyx_L1_error)
    __Pyx_GOTREF(__pyx_t_1);
    __Pyx_DECREF(__pyx_t_6); __pyx_t_6 = 0;
  }
  __Pyx_DECREF(__pyx_t_2); __pyx_t_2 = 0;
  __Pyx_DECREF(__pyx_t_1); __pyx_t_1 = 0;

  /* "src/python/grpcio/grpc/_cython/_cygrpc/grpc_gevent.pyx.pxi":294
 *   accept_callback_cython(s)
 * 
 * cdef void socket_accept(grpc_custom_socket* socket, grpc_custom_socket* client,             # <<<<<<<<<<<<<<
 *                         grpc_custom_accept_callback cb) except * with gil:
 *   sw = <SocketWrapper>socket.impl
 */

  /* function exit code */
  goto __pyx_L0;
  __pyx_L1_error:;
  __Pyx_XDECREF(__pyx_t_1);
  __Pyx_XDECREF(__pyx_t_2);
  __Pyx_XDECREF(__pyx_t_3);
  __Pyx_XDECREF(__pyx_t_4);
  __Pyx_XDECREF(__pyx_t_6);
  __Pyx_AddTraceback("grpc._cython.cygrpc.socket_accept", __pyx_clineno, __pyx_lineno, __pyx_filename);
  __pyx_L0:;
  __Pyx_XDECREF((PyObject *)__pyx_v_sw);
  __Pyx_RefNannyFinishContext();
  #ifdef WITH_THREAD
  __Pyx_PyGILState_Release(__pyx_gilstate_save);
  #endif
}

/* "src/python/grpcio/grpc/_cython/_cygrpc/grpc_gevent.pyx.pxi":306
 * 
 * cdef class ResolveWrapper:
 *   def __cinit__(self):             # <<<<<<<<<<<<<<
 *     self.c_resolver = NULL
 *     self.c_host = NULL
 */

/* Python wrapper */
static int __pyx_pw_4grpc_7_cython_6cygrpc_14ResolveWrapper_1__cinit__(PyObject *__pyx_v_self, PyObject *__pyx_args, PyObject *__pyx_kwds); /*proto*/
static int __pyx_pw_4grpc_7_cython_6cygrpc_14ResolveWrapper_1__cinit__(PyObject *__pyx_v_self, PyObject *__pyx_args, PyObject *__pyx_kwds) {
  int __pyx_r;
  __Pyx_RefNannyDeclarations
  __Pyx_RefNannySetupContext("__cinit__ (wrapper)", 0);
  if (unlikely(PyTuple_GET_SIZE(__pyx_args) > 0)) {
    __Pyx_RaiseArgtupleInvalid("__cinit__", 1, 0, 0, PyTuple_GET_SIZE(__pyx_args)); return -1;}
  if (unlikely(__pyx_kwds) && unlikely(PyDict_Size(__pyx_kwds) > 0) && unlikely(!__Pyx_CheckKeywordStrings(__pyx_kwds, "__cinit__", 0))) return -1;
  __pyx_r = __pyx_pf_4grpc_7_cython_6cygrpc_14ResolveWrapper___cinit__(((struct __pyx_obj_4grpc_7_cython_6cygrpc_ResolveWrapper *)__pyx_v_self));

  /* function exit code */
  __Pyx_RefNannyFinishContext();
  return __pyx_r;
}

static int __pyx_pf_4grpc_7_cython_6cygrpc_14ResolveWrapper___cinit__(struct __pyx_obj_4grpc_7_cython_6cygrpc_ResolveWrapper *__pyx_v_self) {
  int __pyx_r;
  __Pyx_RefNannyDeclarations
  __Pyx_RefNannySetupContext("__cinit__", 0);

  /* "src/python/grpcio/grpc/_cython/_cygrpc/grpc_gevent.pyx.pxi":307
 * cdef class ResolveWrapper:
 *   def __cinit__(self):
 *     self.c_resolver = NULL             # <<<<<<<<<<<<<<
 *     self.c_host = NULL
 *     self.c_port = NULL
 */
  __pyx_v_self->c_resolver = NULL;

  /* "src/python/grpcio/grpc/_cython/_cygrpc/grpc_gevent.pyx.pxi":308
 *   def __cinit__(self):
 *     self.c_resolver = NULL
 *     self.c_host = NULL             # <<<<<<<<<<<<<<
 *     self.c_port = NULL
 * 
 */
  __pyx_v_self->c_host = NULL;

  /* "src/python/grpcio/grpc/_cython/_cygrpc/grpc_gevent.pyx.pxi":309
 *     self.c_resolver = NULL
 *     self.c_host = NULL
 *     self.c_port = NULL             # <<<<<<<<<<<<<<
 * 
 * cdef socket_resolve_async_cython(ResolveWrapper resolve_wrapper):
 */
  __pyx_v_self->c_port = NULL;

  /* "src/python/grpcio/grpc/_cython/_cygrpc/grpc_gevent.pyx.pxi":306
 * 
 * cdef class ResolveWrapper:
 *   def __cinit__(self):             # <<<<<<<<<<<<<<
 *     self.c_resolver = NULL
 *     self.c_host = NULL
 */

  /* function exit code */
  __pyx_r = 0;
  __Pyx_RefNannyFinishContext();
  return __pyx_r;
}

/* "(tree fragment)":1
 * def __reduce_cython__(self):             # <<<<<<<<<<<<<<
 *     raise TypeError("no default __reduce__ due to non-trivial __cinit__")
 * def __setstate_cython__(self, __pyx_state):
 */

/* Python wrapper */
static PyObject *__pyx_pw_4grpc_7_cython_6cygrpc_14ResolveWrapper_3__reduce_cython__(PyObject *__pyx_v_self, CYTHON_UNUSED PyObject *unused); /*proto*/
static PyObject *__pyx_pw_4grpc_7_cython_6cygrpc_14ResolveWrapper_3__reduce_cython__(PyObject *__pyx_v_self, CYTHON_UNUSED PyObject *unused) {
  PyObject *__pyx_r = 0;
  __Pyx_RefNannyDeclarations
  __Pyx_RefNannySetupContext("__reduce_cython__ (wrapper)", 0);
  __pyx_r = __pyx_pf_4grpc_7_cython_6cygrpc_14ResolveWrapper_2__reduce_cython__(((struct __pyx_obj_4grpc_7_cython_6cygrpc_ResolveWrapper *)__pyx_v_self));

  /* function exit code */
  __Pyx_RefNannyFinishContext();
  return __pyx_r;
}

static PyObject *__pyx_pf_4grpc_7_cython_6cygrpc_14ResolveWrapper_2__reduce_cython__(CYTHON_UNUSED struct __pyx_obj_4grpc_7_cython_6cygrpc_ResolveWrapper *__pyx_v_self) {
  PyObject *__pyx_r = NULL;
  __Pyx_RefNannyDeclarations
  PyObject *__pyx_t_1 = NULL;
  __Pyx_RefNannySetupContext("__reduce_cython__", 0);

  /* "(tree fragment)":2
 * def __reduce_cython__(self):
 *     raise TypeError("no default __reduce__ due to non-trivial __cinit__")             # <<<<<<<<<<<<<<
 * def __setstate_cython__(self, __pyx_state):
 *     raise TypeError("no default __reduce__ due to non-trivial __cinit__")
 */
  __pyx_t_1 = __Pyx_PyObject_Call(__pyx_builtin_TypeError, __pyx_tuple__142, NULL); if (unlikely(!__pyx_t_1)) __PYX_ERR(7, 2, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_1);
  __Pyx_Raise(__pyx_t_1, 0, 0, 0);
  __Pyx_DECREF(__pyx_t_1); __pyx_t_1 = 0;
  __PYX_ERR(7, 2, __pyx_L1_error)

  /* "(tree fragment)":1
 * def __reduce_cython__(self):             # <<<<<<<<<<<<<<
 *     raise TypeError("no default __reduce__ due to non-trivial __cinit__")
 * def __setstate_cython__(self, __pyx_state):
 */

  /* function exit code */
  __pyx_L1_error:;
  __Pyx_XDECREF(__pyx_t_1);
  __Pyx_AddTraceback("grpc._cython.cygrpc.ResolveWrapper.__reduce_cython__", __pyx_clineno, __pyx_lineno, __pyx_filename);
  __pyx_r = NULL;
  __Pyx_XGIVEREF(__pyx_r);
  __Pyx_RefNannyFinishContext();
  return __pyx_r;
}

/* "(tree fragment)":3
 * def __reduce_cython__(self):
 *     raise TypeError("no default __reduce__ due to non-trivial __cinit__")
 * def __setstate_cython__(self, __pyx_state):             # <<<<<<<<<<<<<<
 *     raise TypeError("no default __reduce__ due to non-trivial __cinit__")
 */

/* Python wrapper */
static PyObject *__pyx_pw_4grpc_7_cython_6cygrpc_14ResolveWrapper_5__setstate_cython__(PyObject *__pyx_v_self, PyObject *__pyx_v___pyx_state); /*proto*/
static PyObject *__pyx_pw_4grpc_7_cython_6cygrpc_14ResolveWrapper_5__setstate_cython__(PyObject *__pyx_v_self, PyObject *__pyx_v___pyx_state) {
  PyObject *__pyx_r = 0;
  __Pyx_RefNannyDeclarations
  __Pyx_RefNannySetupContext("__setstate_cython__ (wrapper)", 0);
  __pyx_r = __pyx_pf_4grpc_7_cython_6cygrpc_14ResolveWrapper_4__setstate_cython__(((struct __pyx_obj_4grpc_7_cython_6cygrpc_ResolveWrapper *)__pyx_v_self), ((PyObject *)__pyx_v___pyx_state));

  /* function exit code */
  __Pyx_RefNannyFinishContext();
  return __pyx_r;
}

static PyObject *__pyx_pf_4grpc_7_cython_6cygrpc_14ResolveWrapper_4__setstate_cython__(CYTHON_UNUSED struct __pyx_obj_4grpc_7_cython_6cygrpc_ResolveWrapper *__pyx_v_self, CYTHON_UNUSED PyObject *__pyx_v___pyx_state) {
  PyObject *__pyx_r = NULL;
  __Pyx_RefNannyDeclarations
  PyObject *__pyx_t_1 = NULL;
  __Pyx_RefNannySetupContext("__setstate_cython__", 0);

  /* "(tree fragment)":4
 *     raise TypeError("no default __reduce__ due to non-trivial __cinit__")
 * def __setstate_cython__(self, __pyx_state):
 *     raise TypeError("no default __reduce__ due to non-trivial __cinit__")             # <<<<<<<<<<<<<<
 */
  __pyx_t_1 = __Pyx_PyObject_Call(__pyx_builtin_TypeError, __pyx_tuple__143, NULL); if (unlikely(!__pyx_t_1)) __PYX_ERR(7, 4, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_1);
  __Pyx_Raise(__pyx_t_1, 0, 0, 0);
  __Pyx_DECREF(__pyx_t_1); __pyx_t_1 = 0;
  __PYX_ERR(7, 4, __pyx_L1_error)

  /* "(tree fragment)":3
 * def __reduce_cython__(self):
 *     raise TypeError("no default __reduce__ due to non-trivial __cinit__")
 * def __setstate_cython__(self, __pyx_state):             # <<<<<<<<<<<<<<
 *     raise TypeError("no default __reduce__ due to non-trivial __cinit__")
 */

  /* function exit code */
  __pyx_L1_error:;
  __Pyx_XDECREF(__pyx_t_1);
  __Pyx_AddTraceback("grpc._cython.cygrpc.ResolveWrapper.__setstate_cython__", __pyx_clineno, __pyx_lineno, __pyx_filename);
  __pyx_r = NULL;
  __Pyx_XGIVEREF(__pyx_r);
  __Pyx_RefNannyFinishContext();
  return __pyx_r;
}

/* "src/python/grpcio/grpc/_cython/_cygrpc/grpc_gevent.pyx.pxi":311
 *     self.c_port = NULL
 * 
 * cdef socket_resolve_async_cython(ResolveWrapper resolve_wrapper):             # <<<<<<<<<<<<<<
 *   try:
 *     res = gevent_socket.getaddrinfo(resolve_wrapper.c_host, resolve_wrapper.c_port)
 */

static PyObject *__pyx_f_4grpc_7_cython_6cygrpc_socket_resolve_async_cython(struct __pyx_obj_4grpc_7_cython_6cygrpc_ResolveWrapper *__pyx_v_resolve_wrapper) {
  PyObject *__pyx_v_res = NULL;
  PyObject *__pyx_v_io_error = NULL;
  PyObject *__pyx_r = NULL;
  __Pyx_RefNannyDeclarations
  PyObject *__pyx_t_1 = NULL;
  PyObject *__pyx_t_2 = NULL;
  PyObject *__pyx_t_3 = NULL;
  PyObject *__pyx_t_4 = NULL;
  PyObject *__pyx_t_5 = NULL;
  PyObject *__pyx_t_6 = NULL;
  PyObject *__pyx_t_7 = NULL;
  PyObject *__pyx_t_8 = NULL;
  int __pyx_t_9;
  PyObject *__pyx_t_10 = NULL;
  grpc_resolved_addresses *__pyx_t_11;
  __Pyx_RefNannySetupContext("socket_resolve_async_cython", 0);

  /* "src/python/grpcio/grpc/_cython/_cygrpc/grpc_gevent.pyx.pxi":312
 * 
 * cdef socket_resolve_async_cython(ResolveWrapper resolve_wrapper):
 *   try:             # <<<<<<<<<<<<<<
 *     res = gevent_socket.getaddrinfo(resolve_wrapper.c_host, resolve_wrapper.c_port)
 *     grpc_custom_resolve_callback(<grpc_custom_resolver*>resolve_wrapper.c_resolver,
 */
  {
    __Pyx_PyThreadState_declare
    __Pyx_PyThreadState_assign
    __Pyx_ExceptionSave(&__pyx_t_1, &__pyx_t_2, &__pyx_t_3);
    __Pyx_XGOTREF(__pyx_t_1);
    __Pyx_XGOTREF(__pyx_t_2);
    __Pyx_XGOTREF(__pyx_t_3);
    /*try:*/ {

      /* "src/python/grpcio/grpc/_cython/_cygrpc/grpc_gevent.pyx.pxi":313
 * cdef socket_resolve_async_cython(ResolveWrapper resolve_wrapper):
 *   try:
 *     res = gevent_socket.getaddrinfo(resolve_wrapper.c_host, resolve_wrapper.c_port)             # <<<<<<<<<<<<<<
 *     grpc_custom_resolve_callback(<grpc_custom_resolver*>resolve_wrapper.c_resolver,
 *                                  tuples_to_resolvaddr(res), grpc_error_none())
 */
      __pyx_t_5 = __Pyx_GetModuleGlobalName(__pyx_n_s_gevent_socket); if (unlikely(!__pyx_t_5)) __PYX_ERR(6, 313, __pyx_L3_error)
      __Pyx_GOTREF(__pyx_t_5);
      __pyx_t_6 = __Pyx_PyObject_GetAttrStr(__pyx_t_5, __pyx_n_s_getaddrinfo); if (unlikely(!__pyx_t_6)) __PYX_ERR(6, 313, __pyx_L3_error)
      __Pyx_GOTREF(__pyx_t_6);
      __Pyx_DECREF(__pyx_t_5); __pyx_t_5 = 0;
      __pyx_t_5 = __Pyx_PyBytes_FromString(__pyx_v_resolve_wrapper->c_host); if (unlikely(!__pyx_t_5)) __PYX_ERR(6, 313, __pyx_L3_error)
      __Pyx_GOTREF(__pyx_t_5);
      __pyx_t_7 = __Pyx_PyBytes_FromString(__pyx_v_resolve_wrapper->c_port); if (unlikely(!__pyx_t_7)) __PYX_ERR(6, 313, __pyx_L3_error)
      __Pyx_GOTREF(__pyx_t_7);
      __pyx_t_8 = NULL;
      __pyx_t_9 = 0;
      if (CYTHON_UNPACK_METHODS && unlikely(PyMethod_Check(__pyx_t_6))) {
        __pyx_t_8 = PyMethod_GET_SELF(__pyx_t_6);
        if (likely(__pyx_t_8)) {
          PyObject* function = PyMethod_GET_FUNCTION(__pyx_t_6);
          __Pyx_INCREF(__pyx_t_8);
          __Pyx_INCREF(function);
          __Pyx_DECREF_SET(__pyx_t_6, function);
          __pyx_t_9 = 1;
        }
      }
      #if CYTHON_FAST_PYCALL
      if (PyFunction_Check(__pyx_t_6)) {
        PyObject *__pyx_temp[3] = {__pyx_t_8, __pyx_t_5, __pyx_t_7};
        __pyx_t_4 = __Pyx_PyFunction_FastCall(__pyx_t_6, __pyx_temp+1-__pyx_t_9, 2+__pyx_t_9); if (unlikely(!__pyx_t_4)) __PYX_ERR(6, 313, __pyx_L3_error)
        __Pyx_XDECREF(__pyx_t_8); __pyx_t_8 = 0;
        __Pyx_GOTREF(__pyx_t_4);
        __Pyx_DECREF(__pyx_t_5); __pyx_t_5 = 0;
        __Pyx_DECREF(__pyx_t_7); __pyx_t_7 = 0;
      } else
      #endif
      #if CYTHON_FAST_PYCCALL
      if (__Pyx_PyFastCFunction_Check(__pyx_t_6)) {
        PyObject *__pyx_temp[3] = {__pyx_t_8, __pyx_t_5, __pyx_t_7};
        __pyx_t_4 = __Pyx_PyCFunction_FastCall(__pyx_t_6, __pyx_temp+1-__pyx_t_9, 2+__pyx_t_9); if (unlikely(!__pyx_t_4)) __PYX_ERR(6, 313, __pyx_L3_error)
        __Pyx_XDECREF(__pyx_t_8); __pyx_t_8 = 0;
        __Pyx_GOTREF(__pyx_t_4);
        __Pyx_DECREF(__pyx_t_5); __pyx_t_5 = 0;
        __Pyx_DECREF(__pyx_t_7); __pyx_t_7 = 0;
      } else
      #endif
      {
        __pyx_t_10 = PyTuple_New(2+__pyx_t_9); if (unlikely(!__pyx_t_10)) __PYX_ERR(6, 313, __pyx_L3_error)
        __Pyx_GOTREF(__pyx_t_10);
        if (__pyx_t_8) {
          __Pyx_GIVEREF(__pyx_t_8); PyTuple_SET_ITEM(__pyx_t_10, 0, __pyx_t_8); __pyx_t_8 = NULL;
        }
        __Pyx_GIVEREF(__pyx_t_5);
        PyTuple_SET_ITEM(__pyx_t_10, 0+__pyx_t_9, __pyx_t_5);
        __Pyx_GIVEREF(__pyx_t_7);
        PyTuple_SET_ITEM(__pyx_t_10, 1+__pyx_t_9, __pyx_t_7);
        __pyx_t_5 = 0;
        __pyx_t_7 = 0;
        __pyx_t_4 = __Pyx_PyObject_Call(__pyx_t_6, __pyx_t_10, NULL); if (unlikely(!__pyx_t_4)) __PYX_ERR(6, 313, __pyx_L3_error)
        __Pyx_GOTREF(__pyx_t_4);
        __Pyx_DECREF(__pyx_t_10); __pyx_t_10 = 0;
      }
      __Pyx_DECREF(__pyx_t_6); __pyx_t_6 = 0;
      __pyx_v_res = __pyx_t_4;
      __pyx_t_4 = 0;

      /* "src/python/grpcio/grpc/_cython/_cygrpc/grpc_gevent.pyx.pxi":315
 *     res = gevent_socket.getaddrinfo(resolve_wrapper.c_host, resolve_wrapper.c_port)
 *     grpc_custom_resolve_callback(<grpc_custom_resolver*>resolve_wrapper.c_resolver,
 *                                  tuples_to_resolvaddr(res), grpc_error_none())             # <<<<<<<<<<<<<<
 *   except IOError as io_error:
 *     grpc_custom_resolve_callback(<grpc_custom_resolver*>resolve_wrapper.c_resolver,
 */
      __pyx_t_11 = __pyx_f_4grpc_7_cython_6cygrpc_tuples_to_resolvaddr(__pyx_v_res); if (unlikely(PyErr_Occurred())) __PYX_ERR(6, 315, __pyx_L3_error)

      /* "src/python/grpcio/grpc/_cython/_cygrpc/grpc_gevent.pyx.pxi":314
 *   try:
 *     res = gevent_socket.getaddrinfo(resolve_wrapper.c_host, resolve_wrapper.c_port)
 *     grpc_custom_resolve_callback(<grpc_custom_resolver*>resolve_wrapper.c_resolver,             # <<<<<<<<<<<<<<
 *                                  tuples_to_resolvaddr(res), grpc_error_none())
 *   except IOError as io_error:
 */
      grpc_custom_resolve_callback(((struct grpc_custom_resolver *)__pyx_v_resolve_wrapper->c_resolver), __pyx_t_11, __pyx_f_4grpc_7_cython_6cygrpc_grpc_error_none());

      /* "src/python/grpcio/grpc/_cython/_cygrpc/grpc_gevent.pyx.pxi":312
 * 
 * cdef socket_resolve_async_cython(ResolveWrapper resolve_wrapper):
 *   try:             # <<<<<<<<<<<<<<
 *     res = gevent_socket.getaddrinfo(resolve_wrapper.c_host, resolve_wrapper.c_port)
 *     grpc_custom_resolve_callback(<grpc_custom_resolver*>resolve_wrapper.c_resolver,
 */
    }
    __Pyx_XDECREF(__pyx_t_1); __pyx_t_1 = 0;
    __Pyx_XDECREF(__pyx_t_2); __pyx_t_2 = 0;
    __Pyx_XDECREF(__pyx_t_3); __pyx_t_3 = 0;
    goto __pyx_L8_try_end;
    __pyx_L3_error:;
    __Pyx_XDECREF(__pyx_t_8); __pyx_t_8 = 0;
    __Pyx_XDECREF(__pyx_t_5); __pyx_t_5 = 0;
    __Pyx_XDECREF(__pyx_t_7); __pyx_t_7 = 0;
    __Pyx_XDECREF(__pyx_t_10); __pyx_t_10 = 0;
    __Pyx_XDECREF(__pyx_t_6); __pyx_t_6 = 0;
    __Pyx_XDECREF(__pyx_t_4); __pyx_t_4 = 0;

    /* "src/python/grpcio/grpc/_cython/_cygrpc/grpc_gevent.pyx.pxi":316
 *     grpc_custom_resolve_callback(<grpc_custom_resolver*>resolve_wrapper.c_resolver,
 *                                  tuples_to_resolvaddr(res), grpc_error_none())
 *   except IOError as io_error:             # <<<<<<<<<<<<<<
 *     grpc_custom_resolve_callback(<grpc_custom_resolver*>resolve_wrapper.c_resolver,
 *                                  <grpc_resolved_addresses*>0,
 */
    __pyx_t_9 = __Pyx_PyErr_ExceptionMatches(__pyx_builtin_IOError);
    if (__pyx_t_9) {
      __Pyx_AddTraceback("grpc._cython.cygrpc.socket_resolve_async_cython", __pyx_clineno, __pyx_lineno, __pyx_filename);
      if (__Pyx_GetException(&__pyx_t_4, &__pyx_t_6, &__pyx_t_10) < 0) __PYX_ERR(6, 316, __pyx_L5_except_error)
      __Pyx_GOTREF(__pyx_t_4);
      __Pyx_GOTREF(__pyx_t_6);
      __Pyx_GOTREF(__pyx_t_10);
      __Pyx_INCREF(__pyx_t_6);
      __pyx_v_io_error = __pyx_t_6;

      /* "src/python/grpcio/grpc/_cython/_cygrpc/grpc_gevent.pyx.pxi":319
 *     grpc_custom_resolve_callback(<grpc_custom_resolver*>resolve_wrapper.c_resolver,
 *                                  <grpc_resolved_addresses*>0,
 *                                  socket_error("getaddrinfo", str(io_error)))             # <<<<<<<<<<<<<<
 *   g_event.set()
 * 
 */
      __pyx_t_7 = __Pyx_PyObject_CallOneArg(((PyObject *)(&PyString_Type)), __pyx_v_io_error); if (unlikely(!__pyx_t_7)) __PYX_ERR(6, 319, __pyx_L5_except_error)
      __Pyx_GOTREF(__pyx_t_7);
      if (!(likely(PyString_CheckExact(__pyx_t_7))||((__pyx_t_7) == Py_None)||(PyErr_Format(PyExc_TypeError, "Expected %.16s, got %.200s", "str", Py_TYPE(__pyx_t_7)->tp_name), 0))) __PYX_ERR(6, 319, __pyx_L5_except_error)

      /* "src/python/grpcio/grpc/_cython/_cygrpc/grpc_gevent.pyx.pxi":317
 *                                  tuples_to_resolvaddr(res), grpc_error_none())
 *   except IOError as io_error:
 *     grpc_custom_resolve_callback(<grpc_custom_resolver*>resolve_wrapper.c_resolver,             # <<<<<<<<<<<<<<
 *                                  <grpc_resolved_addresses*>0,
 *                                  socket_error("getaddrinfo", str(io_error)))
 */
      grpc_custom_resolve_callback(((struct grpc_custom_resolver *)__pyx_v_resolve_wrapper->c_resolver), ((grpc_resolved_addresses *)0), __pyx_f_4grpc_7_cython_6cygrpc_socket_error(__pyx_n_s_getaddrinfo, ((PyObject*)__pyx_t_7)));
      __Pyx_DECREF(__pyx_t_7); __pyx_t_7 = 0;
      __Pyx_DECREF(__pyx_t_4); __pyx_t_4 = 0;
      __Pyx_DECREF(__pyx_t_6); __pyx_t_6 = 0;
      __Pyx_DECREF(__pyx_t_10); __pyx_t_10 = 0;
      goto __pyx_L4_exception_handled;
    }
    goto __pyx_L5_except_error;
    __pyx_L5_except_error:;

    /* "src/python/grpcio/grpc/_cython/_cygrpc/grpc_gevent.pyx.pxi":312
 * 
 * cdef socket_resolve_async_cython(ResolveWrapper resolve_wrapper):
 *   try:             # <<<<<<<<<<<<<<
 *     res = gevent_socket.getaddrinfo(resolve_wrapper.c_host, resolve_wrapper.c_port)
 *     grpc_custom_resolve_callback(<grpc_custom_resolver*>resolve_wrapper.c_resolver,
 */
    __Pyx_XGIVEREF(__pyx_t_1);
    __Pyx_XGIVEREF(__pyx_t_2);
    __Pyx_XGIVEREF(__pyx_t_3);
    __Pyx_ExceptionReset(__pyx_t_1, __pyx_t_2, __pyx_t_3);
    goto __pyx_L1_error;
    __pyx_L4_exception_handled:;
    __Pyx_XGIVEREF(__pyx_t_1);
    __Pyx_XGIVEREF(__pyx_t_2);
    __Pyx_XGIVEREF(__pyx_t_3);
    __Pyx_ExceptionReset(__pyx_t_1, __pyx_t_2, __pyx_t_3);
    __pyx_L8_try_end:;
  }

  /* "src/python/grpcio/grpc/_cython/_cygrpc/grpc_gevent.pyx.pxi":320
 *                                  <grpc_resolved_addresses*>0,
 *                                  socket_error("getaddrinfo", str(io_error)))
 *   g_event.set()             # <<<<<<<<<<<<<<
 * 
 * def socket_resolve_async_python(resolve_wrapper):
 */
  __pyx_t_6 = __Pyx_GetModuleGlobalName(__pyx_n_s_g_event); if (unlikely(!__pyx_t_6)) __PYX_ERR(6, 320, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_6);
  __pyx_t_4 = __Pyx_PyObject_GetAttrStr(__pyx_t_6, __pyx_n_s_set); if (unlikely(!__pyx_t_4)) __PYX_ERR(6, 320, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_4);
  __Pyx_DECREF(__pyx_t_6); __pyx_t_6 = 0;
  __pyx_t_6 = NULL;
  if (CYTHON_UNPACK_METHODS && unlikely(PyMethod_Check(__pyx_t_4))) {
    __pyx_t_6 = PyMethod_GET_SELF(__pyx_t_4);
    if (likely(__pyx_t_6)) {
      PyObject* function = PyMethod_GET_FUNCTION(__pyx_t_4);
      __Pyx_INCREF(__pyx_t_6);
      __Pyx_INCREF(function);
      __Pyx_DECREF_SET(__pyx_t_4, function);
    }
  }
  if (__pyx_t_6) {
    __pyx_t_10 = __Pyx_PyObject_CallOneArg(__pyx_t_4, __pyx_t_6); if (unlikely(!__pyx_t_10)) __PYX_ERR(6, 320, __pyx_L1_error)
    __Pyx_DECREF(__pyx_t_6); __pyx_t_6 = 0;
  } else {
    __pyx_t_10 = __Pyx_PyObject_CallNoArg(__pyx_t_4); if (unlikely(!__pyx_t_10)) __PYX_ERR(6, 320, __pyx_L1_error)
  }
  __Pyx_GOTREF(__pyx_t_10);
  __Pyx_DECREF(__pyx_t_4); __pyx_t_4 = 0;
  __Pyx_DECREF(__pyx_t_10); __pyx_t_10 = 0;

  /* "src/python/grpcio/grpc/_cython/_cygrpc/grpc_gevent.pyx.pxi":311
 *     self.c_port = NULL
 * 
 * cdef socket_resolve_async_cython(ResolveWrapper resolve_wrapper):             # <<<<<<<<<<<<<<
 *   try:
 *     res = gevent_socket.getaddrinfo(resolve_wrapper.c_host, resolve_wrapper.c_port)
 */

  /* function exit code */
  __pyx_r = Py_None; __Pyx_INCREF(Py_None);
  goto __pyx_L0;
  __pyx_L1_error:;
  __Pyx_XDECREF(__pyx_t_4);
  __Pyx_XDECREF(__pyx_t_5);
  __Pyx_XDECREF(__pyx_t_6);
  __Pyx_XDECREF(__pyx_t_7);
  __Pyx_XDECREF(__pyx_t_8);
  __Pyx_XDECREF(__pyx_t_10);
  __Pyx_AddTraceback("grpc._cython.cygrpc.socket_resolve_async_cython", __pyx_clineno, __pyx_lineno, __pyx_filename);
  __pyx_r = 0;
  __pyx_L0:;
  __Pyx_XDECREF(__pyx_v_res);
  __Pyx_XDECREF(__pyx_v_io_error);
  __Pyx_XGIVEREF(__pyx_r);
  __Pyx_RefNannyFinishContext();
  return __pyx_r;
}

/* "src/python/grpcio/grpc/_cython/_cygrpc/grpc_gevent.pyx.pxi":322
 *   g_event.set()
 * 
 * def socket_resolve_async_python(resolve_wrapper):             # <<<<<<<<<<<<<<
 *   socket_resolve_async_cython(resolve_wrapper)
 * 
 */

/* Python wrapper */
static PyObject *__pyx_pw_4grpc_7_cython_6cygrpc_59socket_resolve_async_python(PyObject *__pyx_self, PyObject *__pyx_v_resolve_wrapper); /*proto*/
static PyMethodDef __pyx_mdef_4grpc_7_cython_6cygrpc_59socket_resolve_async_python = {"socket_resolve_async_python", (PyCFunction)__pyx_pw_4grpc_7_cython_6cygrpc_59socket_resolve_async_python, METH_O, 0};
static PyObject *__pyx_pw_4grpc_7_cython_6cygrpc_59socket_resolve_async_python(PyObject *__pyx_self, PyObject *__pyx_v_resolve_wrapper) {
  PyObject *__pyx_r = 0;
  __Pyx_RefNannyDeclarations
  __Pyx_RefNannySetupContext("socket_resolve_async_python (wrapper)", 0);
  __pyx_r = __pyx_pf_4grpc_7_cython_6cygrpc_58socket_resolve_async_python(__pyx_self, ((PyObject *)__pyx_v_resolve_wrapper));

  /* function exit code */
  __Pyx_RefNannyFinishContext();
  return __pyx_r;
}

static PyObject *__pyx_pf_4grpc_7_cython_6cygrpc_58socket_resolve_async_python(CYTHON_UNUSED PyObject *__pyx_self, PyObject *__pyx_v_resolve_wrapper) {
  PyObject *__pyx_r = NULL;
  __Pyx_RefNannyDeclarations
  PyObject *__pyx_t_1 = NULL;
  __Pyx_RefNannySetupContext("socket_resolve_async_python", 0);

  /* "src/python/grpcio/grpc/_cython/_cygrpc/grpc_gevent.pyx.pxi":323
 * 
 * def socket_resolve_async_python(resolve_wrapper):
 *   socket_resolve_async_cython(resolve_wrapper)             # <<<<<<<<<<<<<<
 * 
 * cdef void socket_resolve_async(grpc_custom_resolver* r, char* host, char* port) except * with gil:
 */
  if (!(likely(((__pyx_v_resolve_wrapper) == Py_None) || likely(__Pyx_TypeTest(__pyx_v_resolve_wrapper, __pyx_ptype_4grpc_7_cython_6cygrpc_ResolveWrapper))))) __PYX_ERR(6, 323, __pyx_L1_error)
  __pyx_t_1 = __pyx_f_4grpc_7_cython_6cygrpc_socket_resolve_async_cython(((struct __pyx_obj_4grpc_7_cython_6cygrpc_ResolveWrapper *)__pyx_v_resolve_wrapper)); if (unlikely(!__pyx_t_1)) __PYX_ERR(6, 323, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_1);
  __Pyx_DECREF(__pyx_t_1); __pyx_t_1 = 0;

  /* "src/python/grpcio/grpc/_cython/_cygrpc/grpc_gevent.pyx.pxi":322
 *   g_event.set()
 * 
 * def socket_resolve_async_python(resolve_wrapper):             # <<<<<<<<<<<<<<
 *   socket_resolve_async_cython(resolve_wrapper)
 * 
 */

  /* function exit code */
  __pyx_r = Py_None; __Pyx_INCREF(Py_None);
  goto __pyx_L0;
  __pyx_L1_error:;
  __Pyx_XDECREF(__pyx_t_1);
  __Pyx_AddTraceback("grpc._cython.cygrpc.socket_resolve_async_python", __pyx_clineno, __pyx_lineno, __pyx_filename);
  __pyx_r = NULL;
  __pyx_L0:;
  __Pyx_XGIVEREF(__pyx_r);
  __Pyx_RefNannyFinishContext();
  return __pyx_r;
}

/* "src/python/grpcio/grpc/_cython/_cygrpc/grpc_gevent.pyx.pxi":325
 *   socket_resolve_async_cython(resolve_wrapper)
 * 
 * cdef void socket_resolve_async(grpc_custom_resolver* r, char* host, char* port) except * with gil:             # <<<<<<<<<<<<<<
 *   rw = ResolveWrapper()
 *   rw.c_resolver = r
 */

static void __pyx_f_4grpc_7_cython_6cygrpc_socket_resolve_async(struct grpc_custom_resolver *__pyx_v_r, char *__pyx_v_host, char *__pyx_v_port) {
  struct __pyx_obj_4grpc_7_cython_6cygrpc_ResolveWrapper *__pyx_v_rw = NULL;
  __Pyx_RefNannyDeclarations
  PyObject *__pyx_t_1 = NULL;
  PyObject *__pyx_t_2 = NULL;
  PyObject *__pyx_t_3 = NULL;
  PyObject *__pyx_t_4 = NULL;
  int __pyx_t_5;
  PyObject *__pyx_t_6 = NULL;
  #ifdef WITH_THREAD
  PyGILState_STATE __pyx_gilstate_save = __Pyx_PyGILState_Ensure();
  #endif
  __Pyx_RefNannySetupContext("socket_resolve_async", 0);

  /* "src/python/grpcio/grpc/_cython/_cygrpc/grpc_gevent.pyx.pxi":326
 * 
 * cdef void socket_resolve_async(grpc_custom_resolver* r, char* host, char* port) except * with gil:
 *   rw = ResolveWrapper()             # <<<<<<<<<<<<<<
 *   rw.c_resolver = r
 *   rw.c_host = host
 */
  __pyx_t_1 = __Pyx_PyObject_CallNoArg(((PyObject *)__pyx_ptype_4grpc_7_cython_6cygrpc_ResolveWrapper)); if (unlikely(!__pyx_t_1)) __PYX_ERR(6, 326, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_1);
  __pyx_v_rw = ((struct __pyx_obj_4grpc_7_cython_6cygrpc_ResolveWrapper *)__pyx_t_1);
  __pyx_t_1 = 0;

  /* "src/python/grpcio/grpc/_cython/_cygrpc/grpc_gevent.pyx.pxi":327
 * cdef void socket_resolve_async(grpc_custom_resolver* r, char* host, char* port) except * with gil:
 *   rw = ResolveWrapper()
 *   rw.c_resolver = r             # <<<<<<<<<<<<<<
 *   rw.c_host = host
 *   rw.c_port = port
 */
  __pyx_v_rw->c_resolver = __pyx_v_r;

  /* "src/python/grpcio/grpc/_cython/_cygrpc/grpc_gevent.pyx.pxi":328
 *   rw = ResolveWrapper()
 *   rw.c_resolver = r
 *   rw.c_host = host             # <<<<<<<<<<<<<<
 *   rw.c_port = port
 *   _spawn_greenlet(socket_resolve_async_python, rw)
 */
  __pyx_v_rw->c_host = __pyx_v_host;

  /* "src/python/grpcio/grpc/_cython/_cygrpc/grpc_gevent.pyx.pxi":329
 *   rw.c_resolver = r
 *   rw.c_host = host
 *   rw.c_port = port             # <<<<<<<<<<<<<<
 *   _spawn_greenlet(socket_resolve_async_python, rw)
 * 
 */
  __pyx_v_rw->c_port = __pyx_v_port;

  /* "src/python/grpcio/grpc/_cython/_cygrpc/grpc_gevent.pyx.pxi":330
 *   rw.c_host = host
 *   rw.c_port = port
 *   _spawn_greenlet(socket_resolve_async_python, rw)             # <<<<<<<<<<<<<<
 * 
 * cdef grpc_error* socket_resolve(char* host, char* port,
 */
  __pyx_t_2 = __Pyx_GetModuleGlobalName(__pyx_n_s_spawn_greenlet); if (unlikely(!__pyx_t_2)) __PYX_ERR(6, 330, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_2);
  __pyx_t_3 = __Pyx_GetModuleGlobalName(__pyx_n_s_socket_resolve_async_python); if (unlikely(!__pyx_t_3)) __PYX_ERR(6, 330, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_3);
  __pyx_t_4 = NULL;
  __pyx_t_5 = 0;
  if (CYTHON_UNPACK_METHODS && unlikely(PyMethod_Check(__pyx_t_2))) {
    __pyx_t_4 = PyMethod_GET_SELF(__pyx_t_2);
    if (likely(__pyx_t_4)) {
      PyObject* function = PyMethod_GET_FUNCTION(__pyx_t_2);
      __Pyx_INCREF(__pyx_t_4);
      __Pyx_INCREF(function);
      __Pyx_DECREF_SET(__pyx_t_2, function);
      __pyx_t_5 = 1;
    }
  }
  #if CYTHON_FAST_PYCALL
  if (PyFunction_Check(__pyx_t_2)) {
    PyObject *__pyx_temp[3] = {__pyx_t_4, __pyx_t_3, ((PyObject *)__pyx_v_rw)};
    __pyx_t_1 = __Pyx_PyFunction_FastCall(__pyx_t_2, __pyx_temp+1-__pyx_t_5, 2+__pyx_t_5); if (unlikely(!__pyx_t_1)) __PYX_ERR(6, 330, __pyx_L1_error)
    __Pyx_XDECREF(__pyx_t_4); __pyx_t_4 = 0;
    __Pyx_GOTREF(__pyx_t_1);
    __Pyx_DECREF(__pyx_t_3); __pyx_t_3 = 0;
  } else
  #endif
  #if CYTHON_FAST_PYCCALL
  if (__Pyx_PyFastCFunction_Check(__pyx_t_2)) {
    PyObject *__pyx_temp[3] = {__pyx_t_4, __pyx_t_3, ((PyObject *)__pyx_v_rw)};
    __pyx_t_1 = __Pyx_PyCFunction_FastCall(__pyx_t_2, __pyx_temp+1-__pyx_t_5, 2+__pyx_t_5); if (unlikely(!__pyx_t_1)) __PYX_ERR(6, 330, __pyx_L1_error)
    __Pyx_XDECREF(__pyx_t_4); __pyx_t_4 = 0;
    __Pyx_GOTREF(__pyx_t_1);
    __Pyx_DECREF(__pyx_t_3); __pyx_t_3 = 0;
  } else
  #endif
  {
    __pyx_t_6 = PyTuple_New(2+__pyx_t_5); if (unlikely(!__pyx_t_6)) __PYX_ERR(6, 330, __pyx_L1_error)
    __Pyx_GOTREF(__pyx_t_6);
    if (__pyx_t_4) {
      __Pyx_GIVEREF(__pyx_t_4); PyTuple_SET_ITEM(__pyx_t_6, 0, __pyx_t_4); __pyx_t_4 = NULL;
    }
    __Pyx_GIVEREF(__pyx_t_3);
    PyTuple_SET_ITEM(__pyx_t_6, 0+__pyx_t_5, __pyx_t_3);
    __Pyx_INCREF(((PyObject *)__pyx_v_rw));
    __Pyx_GIVEREF(((PyObject *)__pyx_v_rw));
    PyTuple_SET_ITEM(__pyx_t_6, 1+__pyx_t_5, ((PyObject *)__pyx_v_rw));
    __pyx_t_3 = 0;
    __pyx_t_1 = __Pyx_PyObject_Call(__pyx_t_2, __pyx_t_6, NULL); if (unlikely(!__pyx_t_1)) __PYX_ERR(6, 330, __pyx_L1_error)
    __Pyx_GOTREF(__pyx_t_1);
    __Pyx_DECREF(__pyx_t_6); __pyx_t_6 = 0;
  }
  __Pyx_DECREF(__pyx_t_2); __pyx_t_2 = 0;
  __Pyx_DECREF(__pyx_t_1); __pyx_t_1 = 0;

  /* "src/python/grpcio/grpc/_cython/_cygrpc/grpc_gevent.pyx.pxi":325
 *   socket_resolve_async_cython(resolve_wrapper)
 * 
 * cdef void socket_resolve_async(grpc_custom_resolver* r, char* host, char* port) except * with gil:             # <<<<<<<<<<<<<<
 *   rw = ResolveWrapper()
 *   rw.c_resolver = r
 */

  /* function exit code */
  goto __pyx_L0;
  __pyx_L1_error:;
  __Pyx_XDECREF(__pyx_t_1);
  __Pyx_XDECREF(__pyx_t_2);
  __Pyx_XDECREF(__pyx_t_3);
  __Pyx_XDECREF(__pyx_t_4);
  __Pyx_XDECREF(__pyx_t_6);
  __Pyx_AddTraceback("grpc._cython.cygrpc.socket_resolve_async", __pyx_clineno, __pyx_lineno, __pyx_filename);
  __pyx_L0:;
  __Pyx_XDECREF((PyObject *)__pyx_v_rw);
  __Pyx_RefNannyFinishContext();
  #ifdef WITH_THREAD
  __Pyx_PyGILState_Release(__pyx_gilstate_save);
  #endif
}

/* "src/python/grpcio/grpc/_cython/_cygrpc/grpc_gevent.pyx.pxi":332
 *   _spawn_greenlet(socket_resolve_async_python, rw)
 * 
 * cdef grpc_error* socket_resolve(char* host, char* port,             # <<<<<<<<<<<<<<
 *                                 grpc_resolved_addresses** res) except * with gil:
 *     try:
 */

static struct grpc_error *__pyx_f_4grpc_7_cython_6cygrpc_socket_resolve(char *__pyx_v_host, char *__pyx_v_port, grpc_resolved_addresses **__pyx_v_res) {
  PyObject *__pyx_v_result = NULL;
  PyObject *__pyx_v_io_error = NULL;
  struct grpc_error *__pyx_r;
  __Pyx_RefNannyDeclarations
  PyObject *__pyx_t_1 = NULL;
  PyObject *__pyx_t_2 = NULL;
  PyObject *__pyx_t_3 = NULL;
  PyObject *__pyx_t_4 = NULL;
  PyObject *__pyx_t_5 = NULL;
  PyObject *__pyx_t_6 = NULL;
  PyObject *__pyx_t_7 = NULL;
  PyObject *__pyx_t_8 = NULL;
  int __pyx_t_9;
  PyObject *__pyx_t_10 = NULL;
  grpc_resolved_addresses *__pyx_t_11;
  #ifdef WITH_THREAD
  PyGILState_STATE __pyx_gilstate_save = __Pyx_PyGILState_Ensure();
  #endif
  __Pyx_RefNannySetupContext("socket_resolve", 0);

  /* "src/python/grpcio/grpc/_cython/_cygrpc/grpc_gevent.pyx.pxi":334
 * cdef grpc_error* socket_resolve(char* host, char* port,
 *                                 grpc_resolved_addresses** res) except * with gil:
 *     try:             # <<<<<<<<<<<<<<
 *       result = gevent_socket.getaddrinfo(host, port)
 *       res[0] = tuples_to_resolvaddr(result)
 */
  {
    __Pyx_PyThreadState_declare
    __Pyx_PyThreadState_assign
    __Pyx_ExceptionSave(&__pyx_t_1, &__pyx_t_2, &__pyx_t_3);
    __Pyx_XGOTREF(__pyx_t_1);
    __Pyx_XGOTREF(__pyx_t_2);
    __Pyx_XGOTREF(__pyx_t_3);
    /*try:*/ {

      /* "src/python/grpcio/grpc/_cython/_cygrpc/grpc_gevent.pyx.pxi":335
 *                                 grpc_resolved_addresses** res) except * with gil:
 *     try:
 *       result = gevent_socket.getaddrinfo(host, port)             # <<<<<<<<<<<<<<
 *       res[0] = tuples_to_resolvaddr(result)
 *       return grpc_error_none()
 */
      __pyx_t_5 = __Pyx_GetModuleGlobalName(__pyx_n_s_gevent_socket); if (unlikely(!__pyx_t_5)) __PYX_ERR(6, 335, __pyx_L3_error)
      __Pyx_GOTREF(__pyx_t_5);
      __pyx_t_6 = __Pyx_PyObject_GetAttrStr(__pyx_t_5, __pyx_n_s_getaddrinfo); if (unlikely(!__pyx_t_6)) __PYX_ERR(6, 335, __pyx_L3_error)
      __Pyx_GOTREF(__pyx_t_6);
      __Pyx_DECREF(__pyx_t_5); __pyx_t_5 = 0;
      __pyx_t_5 = __Pyx_PyBytes_FromString(__pyx_v_host); if (unlikely(!__pyx_t_5)) __PYX_ERR(6, 335, __pyx_L3_error)
      __Pyx_GOTREF(__pyx_t_5);
      __pyx_t_7 = __Pyx_PyBytes_FromString(__pyx_v_port); if (unlikely(!__pyx_t_7)) __PYX_ERR(6, 335, __pyx_L3_error)
      __Pyx_GOTREF(__pyx_t_7);
      __pyx_t_8 = NULL;
      __pyx_t_9 = 0;
      if (CYTHON_UNPACK_METHODS && unlikely(PyMethod_Check(__pyx_t_6))) {
        __pyx_t_8 = PyMethod_GET_SELF(__pyx_t_6);
        if (likely(__pyx_t_8)) {
          PyObject* function = PyMethod_GET_FUNCTION(__pyx_t_6);
          __Pyx_INCREF(__pyx_t_8);
          __Pyx_INCREF(function);
          __Pyx_DECREF_SET(__pyx_t_6, function);
          __pyx_t_9 = 1;
        }
      }
      #if CYTHON_FAST_PYCALL
      if (PyFunction_Check(__pyx_t_6)) {
        PyObject *__pyx_temp[3] = {__pyx_t_8, __pyx_t_5, __pyx_t_7};
        __pyx_t_4 = __Pyx_PyFunction_FastCall(__pyx_t_6, __pyx_temp+1-__pyx_t_9, 2+__pyx_t_9); if (unlikely(!__pyx_t_4)) __PYX_ERR(6, 335, __pyx_L3_error)
        __Pyx_XDECREF(__pyx_t_8); __pyx_t_8 = 0;
        __Pyx_GOTREF(__pyx_t_4);
        __Pyx_DECREF(__pyx_t_5); __pyx_t_5 = 0;
        __Pyx_DECREF(__pyx_t_7); __pyx_t_7 = 0;
      } else
      #endif
      #if CYTHON_FAST_PYCCALL
      if (__Pyx_PyFastCFunction_Check(__pyx_t_6)) {
        PyObject *__pyx_temp[3] = {__pyx_t_8, __pyx_t_5, __pyx_t_7};
        __pyx_t_4 = __Pyx_PyCFunction_FastCall(__pyx_t_6, __pyx_temp+1-__pyx_t_9, 2+__pyx_t_9); if (unlikely(!__pyx_t_4)) __PYX_ERR(6, 335, __pyx_L3_error)
        __Pyx_XDECREF(__pyx_t_8); __pyx_t_8 = 0;
        __Pyx_GOTREF(__pyx_t_4);
        __Pyx_DECREF(__pyx_t_5); __pyx_t_5 = 0;
        __Pyx_DECREF(__pyx_t_7); __pyx_t_7 = 0;
      } else
      #endif
      {
        __pyx_t_10 = PyTuple_New(2+__pyx_t_9); if (unlikely(!__pyx_t_10)) __PYX_ERR(6, 335, __pyx_L3_error)
        __Pyx_GOTREF(__pyx_t_10);
        if (__pyx_t_8) {
          __Pyx_GIVEREF(__pyx_t_8); PyTuple_SET_ITEM(__pyx_t_10, 0, __pyx_t_8); __pyx_t_8 = NULL;
        }
        __Pyx_GIVEREF(__pyx_t_5);
        PyTuple_SET_ITEM(__pyx_t_10, 0+__pyx_t_9, __pyx_t_5);
        __Pyx_GIVEREF(__pyx_t_7);
        PyTuple_SET_ITEM(__pyx_t_10, 1+__pyx_t_9, __pyx_t_7);
        __pyx_t_5 = 0;
        __pyx_t_7 = 0;
        __pyx_t_4 = __Pyx_PyObject_Call(__pyx_t_6, __pyx_t_10, NULL); if (unlikely(!__pyx_t_4)) __PYX_ERR(6, 335, __pyx_L3_error)
        __Pyx_GOTREF(__pyx_t_4);
        __Pyx_DECREF(__pyx_t_10); __pyx_t_10 = 0;
      }
      __Pyx_DECREF(__pyx_t_6); __pyx_t_6 = 0;
      __pyx_v_result = __pyx_t_4;
      __pyx_t_4 = 0;

      /* "src/python/grpcio/grpc/_cython/_cygrpc/grpc_gevent.pyx.pxi":336
 *     try:
 *       result = gevent_socket.getaddrinfo(host, port)
 *       res[0] = tuples_to_resolvaddr(result)             # <<<<<<<<<<<<<<
 *       return grpc_error_none()
 *     except IOError as io_error:
 */
      __pyx_t_11 = __pyx_f_4grpc_7_cython_6cygrpc_tuples_to_resolvaddr(__pyx_v_result); if (unlikely(PyErr_Occurred())) __PYX_ERR(6, 336, __pyx_L3_error)
      (__pyx_v_res[0]) = __pyx_t_11;

      /* "src/python/grpcio/grpc/_cython/_cygrpc/grpc_gevent.pyx.pxi":337
 *       result = gevent_socket.getaddrinfo(host, port)
 *       res[0] = tuples_to_resolvaddr(result)
 *       return grpc_error_none()             # <<<<<<<<<<<<<<
 *     except IOError as io_error:
 *       return socket_error("getaddrinfo", str(io_error))
 */
      __pyx_r = __pyx_f_4grpc_7_cython_6cygrpc_grpc_error_none();
      goto __pyx_L7_try_return;

      /* "src/python/grpcio/grpc/_cython/_cygrpc/grpc_gevent.pyx.pxi":334
 * cdef grpc_error* socket_resolve(char* host, char* port,
 *                                 grpc_resolved_addresses** res) except * with gil:
 *     try:             # <<<<<<<<<<<<<<
 *       result = gevent_socket.getaddrinfo(host, port)
 *       res[0] = tuples_to_resolvaddr(result)
 */
    }
    __pyx_L3_error:;
    __Pyx_XDECREF(__pyx_t_8); __pyx_t_8 = 0;
    __Pyx_XDECREF(__pyx_t_5); __pyx_t_5 = 0;
    __Pyx_XDECREF(__pyx_t_7); __pyx_t_7 = 0;
    __Pyx_XDECREF(__pyx_t_10); __pyx_t_10 = 0;
    __Pyx_XDECREF(__pyx_t_6); __pyx_t_6 = 0;
    __Pyx_XDECREF(__pyx_t_4); __pyx_t_4 = 0;

    /* "src/python/grpcio/grpc/_cython/_cygrpc/grpc_gevent.pyx.pxi":338
 *       res[0] = tuples_to_resolvaddr(result)
 *       return grpc_error_none()
 *     except IOError as io_error:             # <<<<<<<<<<<<<<
 *       return socket_error("getaddrinfo", str(io_error))
 * 
 */
    __pyx_t_9 = __Pyx_PyErr_ExceptionMatches(__pyx_builtin_IOError);
    if (__pyx_t_9) {
      __Pyx_AddTraceback("grpc._cython.cygrpc.socket_resolve", __pyx_clineno, __pyx_lineno, __pyx_filename);
      if (__Pyx_GetException(&__pyx_t_4, &__pyx_t_6, &__pyx_t_10) < 0) __PYX_ERR(6, 338, __pyx_L5_except_error)
      __Pyx_GOTREF(__pyx_t_4);
      __Pyx_GOTREF(__pyx_t_6);
      __Pyx_GOTREF(__pyx_t_10);
      __Pyx_INCREF(__pyx_t_6);
      __pyx_v_io_error = __pyx_t_6;

      /* "src/python/grpcio/grpc/_cython/_cygrpc/grpc_gevent.pyx.pxi":339
 *       return grpc_error_none()
 *     except IOError as io_error:
 *       return socket_error("getaddrinfo", str(io_error))             # <<<<<<<<<<<<<<
 * 
 * ###############################
 */
      __pyx_t_7 = __Pyx_PyObject_CallOneArg(((PyObject *)(&PyString_Type)), __pyx_v_io_error); if (unlikely(!__pyx_t_7)) __PYX_ERR(6, 339, __pyx_L5_except_error)
      __Pyx_GOTREF(__pyx_t_7);
      if (!(likely(PyString_CheckExact(__pyx_t_7))||((__pyx_t_7) == Py_None)||(PyErr_Format(PyExc_TypeError, "Expected %.16s, got %.200s", "str", Py_TYPE(__pyx_t_7)->tp_name), 0))) __PYX_ERR(6, 339, __pyx_L5_except_error)
      __pyx_r = __pyx_f_4grpc_7_cython_6cygrpc_socket_error(__pyx_n_s_getaddrinfo, ((PyObject*)__pyx_t_7));
      __Pyx_DECREF(__pyx_t_7); __pyx_t_7 = 0;
      __Pyx_DECREF(__pyx_t_4); __pyx_t_4 = 0;
      __Pyx_DECREF(__pyx_t_6); __pyx_t_6 = 0;
      __Pyx_DECREF(__pyx_t_10); __pyx_t_10 = 0;
      goto __pyx_L6_except_return;
    }
    goto __pyx_L5_except_error;
    __pyx_L5_except_error:;

    /* "src/python/grpcio/grpc/_cython/_cygrpc/grpc_gevent.pyx.pxi":334
 * cdef grpc_error* socket_resolve(char* host, char* port,
 *                                 grpc_resolved_addresses** res) except * with gil:
 *     try:             # <<<<<<<<<<<<<<
 *       result = gevent_socket.getaddrinfo(host, port)
 *       res[0] = tuples_to_resolvaddr(result)
 */
    __Pyx_XGIVEREF(__pyx_t_1);
    __Pyx_XGIVEREF(__pyx_t_2);
    __Pyx_XGIVEREF(__pyx_t_3);
    __Pyx_ExceptionReset(__pyx_t_1, __pyx_t_2, __pyx_t_3);
    goto __pyx_L1_error;
    __pyx_L7_try_return:;
    __Pyx_XGIVEREF(__pyx_t_1);
    __Pyx_XGIVEREF(__pyx_t_2);
    __Pyx_XGIVEREF(__pyx_t_3);
    __Pyx_ExceptionReset(__pyx_t_1, __pyx_t_2, __pyx_t_3);
    goto __pyx_L0;
    __pyx_L6_except_return:;
    __Pyx_XGIVEREF(__pyx_t_1);
    __Pyx_XGIVEREF(__pyx_t_2);
    __Pyx_XGIVEREF(__pyx_t_3);
    __Pyx_ExceptionReset(__pyx_t_1, __pyx_t_2, __pyx_t_3);
    goto __pyx_L0;
  }

  /* "src/python/grpcio/grpc/_cython/_cygrpc/grpc_gevent.pyx.pxi":332
 *   _spawn_greenlet(socket_resolve_async_python, rw)
 * 
 * cdef grpc_error* socket_resolve(char* host, char* port,             # <<<<<<<<<<<<<<
 *                                 grpc_resolved_addresses** res) except * with gil:
 *     try:
 */

  /* function exit code */
  __pyx_L1_error:;
  __Pyx_XDECREF(__pyx_t_4);
  __Pyx_XDECREF(__pyx_t_5);
  __Pyx_XDECREF(__pyx_t_6);
  __Pyx_XDECREF(__pyx_t_7);
  __Pyx_XDECREF(__pyx_t_8);
  __Pyx_XDECREF(__pyx_t_10);
  __Pyx_AddTraceback("grpc._cython.cygrpc.socket_resolve", __pyx_clineno, __pyx_lineno, __pyx_filename);
  __pyx_r = 0;
  __pyx_L0:;
  __Pyx_XDECREF(__pyx_v_result);
  __Pyx_XDECREF(__pyx_v_io_error);
  __Pyx_RefNannyFinishContext();
  #ifdef WITH_THREAD
  __Pyx_PyGILState_Release(__pyx_gilstate_save);
  #endif
  return __pyx_r;
}

/* "src/python/grpcio/grpc/_cython/_cygrpc/grpc_gevent.pyx.pxi":346
 * 
 * cdef class TimerWrapper:
 *   def __cinit__(self, deadline):             # <<<<<<<<<<<<<<
 *     self.timer = gevent_hub.get_hub().loop.timer(deadline)
 *     self.event = None
 */

/* Python wrapper */
static int __pyx_pw_4grpc_7_cython_6cygrpc_12TimerWrapper_1__cinit__(PyObject *__pyx_v_self, PyObject *__pyx_args, PyObject *__pyx_kwds); /*proto*/
static int __pyx_pw_4grpc_7_cython_6cygrpc_12TimerWrapper_1__cinit__(PyObject *__pyx_v_self, PyObject *__pyx_args, PyObject *__pyx_kwds) {
  PyObject *__pyx_v_deadline = 0;
  int __pyx_r;
  __Pyx_RefNannyDeclarations
  __Pyx_RefNannySetupContext("__cinit__ (wrapper)", 0);
  {
    static PyObject **__pyx_pyargnames[] = {&__pyx_n_s_deadline,0};
    PyObject* values[1] = {0};
    if (unlikely(__pyx_kwds)) {
      Py_ssize_t kw_args;
      const Py_ssize_t pos_args = PyTuple_GET_SIZE(__pyx_args);
      switch (pos_args) {
        case  1: values[0] = PyTuple_GET_ITEM(__pyx_args, 0);
        CYTHON_FALLTHROUGH;
        case  0: break;
        default: goto __pyx_L5_argtuple_error;
      }
      kw_args = PyDict_Size(__pyx_kwds);
      switch (pos_args) {
        case  0:
        if (likely((values[0] = __Pyx_PyDict_GetItemStr(__pyx_kwds, __pyx_n_s_deadline)) != 0)) kw_args--;
        else goto __pyx_L5_argtuple_error;
      }
      if (unlikely(kw_args > 0)) {
        if (unlikely(__Pyx_ParseOptionalKeywords(__pyx_kwds, __pyx_pyargnames, 0, values, pos_args, "__cinit__") < 0)) __PYX_ERR(6, 346, __pyx_L3_error)
      }
    } else if (PyTuple_GET_SIZE(__pyx_args) != 1) {
      goto __pyx_L5_argtuple_error;
    } else {
      values[0] = PyTuple_GET_ITEM(__pyx_args, 0);
    }
    __pyx_v_deadline = values[0];
  }
  goto __pyx_L4_argument_unpacking_done;
  __pyx_L5_argtuple_error:;
  __Pyx_RaiseArgtupleInvalid("__cinit__", 1, 1, 1, PyTuple_GET_SIZE(__pyx_args)); __PYX_ERR(6, 346, __pyx_L3_error)
  __pyx_L3_error:;
  __Pyx_AddTraceback("grpc._cython.cygrpc.TimerWrapper.__cinit__", __pyx_clineno, __pyx_lineno, __pyx_filename);
  __Pyx_RefNannyFinishContext();
  return -1;
  __pyx_L4_argument_unpacking_done:;
  __pyx_r = __pyx_pf_4grpc_7_cython_6cygrpc_12TimerWrapper___cinit__(((struct __pyx_obj_4grpc_7_cython_6cygrpc_TimerWrapper *)__pyx_v_self), __pyx_v_deadline);

  /* function exit code */
  __Pyx_RefNannyFinishContext();
  return __pyx_r;
}

static int __pyx_pf_4grpc_7_cython_6cygrpc_12TimerWrapper___cinit__(struct __pyx_obj_4grpc_7_cython_6cygrpc_TimerWrapper *__pyx_v_self, PyObject *__pyx_v_deadline) {
  int __pyx_r;
  __Pyx_RefNannyDeclarations
  PyObject *__pyx_t_1 = NULL;
  PyObject *__pyx_t_2 = NULL;
  PyObject *__pyx_t_3 = NULL;
  PyObject *__pyx_t_4 = NULL;
  __Pyx_RefNannySetupContext("__cinit__", 0);

  /* "src/python/grpcio/grpc/_cython/_cygrpc/grpc_gevent.pyx.pxi":347
 * cdef class TimerWrapper:
 *   def __cinit__(self, deadline):
 *     self.timer = gevent_hub.get_hub().loop.timer(deadline)             # <<<<<<<<<<<<<<
 *     self.event = None
 * 
 */
  __pyx_t_3 = __Pyx_GetModuleGlobalName(__pyx_n_s_gevent_hub); if (unlikely(!__pyx_t_3)) __PYX_ERR(6, 347, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_3);
  __pyx_t_4 = __Pyx_PyObject_GetAttrStr(__pyx_t_3, __pyx_n_s_get_hub); if (unlikely(!__pyx_t_4)) __PYX_ERR(6, 347, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_4);
  __Pyx_DECREF(__pyx_t_3); __pyx_t_3 = 0;
  __pyx_t_3 = NULL;
  if (CYTHON_UNPACK_METHODS && unlikely(PyMethod_Check(__pyx_t_4))) {
    __pyx_t_3 = PyMethod_GET_SELF(__pyx_t_4);
    if (likely(__pyx_t_3)) {
      PyObject* function = PyMethod_GET_FUNCTION(__pyx_t_4);
      __Pyx_INCREF(__pyx_t_3);
      __Pyx_INCREF(function);
      __Pyx_DECREF_SET(__pyx_t_4, function);
    }
  }
  if (__pyx_t_3) {
    __pyx_t_2 = __Pyx_PyObject_CallOneArg(__pyx_t_4, __pyx_t_3); if (unlikely(!__pyx_t_2)) __PYX_ERR(6, 347, __pyx_L1_error)
    __Pyx_DECREF(__pyx_t_3); __pyx_t_3 = 0;
  } else {
    __pyx_t_2 = __Pyx_PyObject_CallNoArg(__pyx_t_4); if (unlikely(!__pyx_t_2)) __PYX_ERR(6, 347, __pyx_L1_error)
  }
  __Pyx_GOTREF(__pyx_t_2);
  __Pyx_DECREF(__pyx_t_4); __pyx_t_4 = 0;
  __pyx_t_4 = __Pyx_PyObject_GetAttrStr(__pyx_t_2, __pyx_n_s_loop); if (unlikely(!__pyx_t_4)) __PYX_ERR(6, 347, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_4);
  __Pyx_DECREF(__pyx_t_2); __pyx_t_2 = 0;
  __pyx_t_2 = __Pyx_PyObject_GetAttrStr(__pyx_t_4, __pyx_n_s_timer); if (unlikely(!__pyx_t_2)) __PYX_ERR(6, 347, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_2);
  __Pyx_DECREF(__pyx_t_4); __pyx_t_4 = 0;
  __pyx_t_4 = NULL;
  if (CYTHON_UNPACK_METHODS && likely(PyMethod_Check(__pyx_t_2))) {
    __pyx_t_4 = PyMethod_GET_SELF(__pyx_t_2);
    if (likely(__pyx_t_4)) {
      PyObject* function = PyMethod_GET_FUNCTION(__pyx_t_2);
      __Pyx_INCREF(__pyx_t_4);
      __Pyx_INCREF(function);
      __Pyx_DECREF_SET(__pyx_t_2, function);
    }
  }
  if (!__pyx_t_4) {
    __pyx_t_1 = __Pyx_PyObject_CallOneArg(__pyx_t_2, __pyx_v_deadline); if (unlikely(!__pyx_t_1)) __PYX_ERR(6, 347, __pyx_L1_error)
    __Pyx_GOTREF(__pyx_t_1);
  } else {
    #if CYTHON_FAST_PYCALL
    if (PyFunction_Check(__pyx_t_2)) {
      PyObject *__pyx_temp[2] = {__pyx_t_4, __pyx_v_deadline};
      __pyx_t_1 = __Pyx_PyFunction_FastCall(__pyx_t_2, __pyx_temp+1-1, 1+1); if (unlikely(!__pyx_t_1)) __PYX_ERR(6, 347, __pyx_L1_error)
      __Pyx_XDECREF(__pyx_t_4); __pyx_t_4 = 0;
      __Pyx_GOTREF(__pyx_t_1);
    } else
    #endif
    #if CYTHON_FAST_PYCCALL
    if (__Pyx_PyFastCFunction_Check(__pyx_t_2)) {
      PyObject *__pyx_temp[2] = {__pyx_t_4, __pyx_v_deadline};
      __pyx_t_1 = __Pyx_PyCFunction_FastCall(__pyx_t_2, __pyx_temp+1-1, 1+1); if (unlikely(!__pyx_t_1)) __PYX_ERR(6, 347, __pyx_L1_error)
      __Pyx_XDECREF(__pyx_t_4); __pyx_t_4 = 0;
      __Pyx_GOTREF(__pyx_t_1);
    } else
    #endif
    {
      __pyx_t_3 = PyTuple_New(1+1); if (unlikely(!__pyx_t_3)) __PYX_ERR(6, 347, __pyx_L1_error)
      __Pyx_GOTREF(__pyx_t_3);
      __Pyx_GIVEREF(__pyx_t_4); PyTuple_SET_ITEM(__pyx_t_3, 0, __pyx_t_4); __pyx_t_4 = NULL;
      __Pyx_INCREF(__pyx_v_deadline);
      __Pyx_GIVEREF(__pyx_v_deadline);
      PyTuple_SET_ITEM(__pyx_t_3, 0+1, __pyx_v_deadline);
      __pyx_t_1 = __Pyx_PyObject_Call(__pyx_t_2, __pyx_t_3, NULL); if (unlikely(!__pyx_t_1)) __PYX_ERR(6, 347, __pyx_L1_error)
      __Pyx_GOTREF(__pyx_t_1);
      __Pyx_DECREF(__pyx_t_3); __pyx_t_3 = 0;
    }
  }
  __Pyx_DECREF(__pyx_t_2); __pyx_t_2 = 0;
  __Pyx_GIVEREF(__pyx_t_1);
  __Pyx_GOTREF(__pyx_v_self->timer);
  __Pyx_DECREF(__pyx_v_self->timer);
  __pyx_v_self->timer = __pyx_t_1;
  __pyx_t_1 = 0;

  /* "src/python/grpcio/grpc/_cython/_cygrpc/grpc_gevent.pyx.pxi":348
 *   def __cinit__(self, deadline):
 *     self.timer = gevent_hub.get_hub().loop.timer(deadline)
 *     self.event = None             # <<<<<<<<<<<<<<
 * 
 *   def start(self):
 */
  __Pyx_INCREF(Py_None);
  __Pyx_GIVEREF(Py_None);
  __Pyx_GOTREF(__pyx_v_self->event);
  __Pyx_DECREF(__pyx_v_self->event);
  __pyx_v_self->event = Py_None;

  /* "src/python/grpcio/grpc/_cython/_cygrpc/grpc_gevent.pyx.pxi":346
 * 
 * cdef class TimerWrapper:
 *   def __cinit__(self, deadline):             # <<<<<<<<<<<<<<
 *     self.timer = gevent_hub.get_hub().loop.timer(deadline)
 *     self.event = None
 */

  /* function exit code */
  __pyx_r = 0;
  goto __pyx_L0;
  __pyx_L1_error:;
  __Pyx_XDECREF(__pyx_t_1);
  __Pyx_XDECREF(__pyx_t_2);
  __Pyx_XDECREF(__pyx_t_3);
  __Pyx_XDECREF(__pyx_t_4);
  __Pyx_AddTraceback("grpc._cython.cygrpc.TimerWrapper.__cinit__", __pyx_clineno, __pyx_lineno, __pyx_filename);
  __pyx_r = -1;
  __pyx_L0:;
  __Pyx_RefNannyFinishContext();
  return __pyx_r;
}

/* "src/python/grpcio/grpc/_cython/_cygrpc/grpc_gevent.pyx.pxi":350
 *     self.event = None
 * 
 *   def start(self):             # <<<<<<<<<<<<<<
 *     self.event = gevent_event.Event()
 *     self.timer.start(self.on_finish)
 */

/* Python wrapper */
static PyObject *__pyx_pw_4grpc_7_cython_6cygrpc_12TimerWrapper_3start(PyObject *__pyx_v_self, CYTHON_UNUSED PyObject *unused); /*proto*/
static PyObject *__pyx_pw_4grpc_7_cython_6cygrpc_12TimerWrapper_3start(PyObject *__pyx_v_self, CYTHON_UNUSED PyObject *unused) {
  PyObject *__pyx_r = 0;
  __Pyx_RefNannyDeclarations
  __Pyx_RefNannySetupContext("start (wrapper)", 0);
  __pyx_r = __pyx_pf_4grpc_7_cython_6cygrpc_12TimerWrapper_2start(((struct __pyx_obj_4grpc_7_cython_6cygrpc_TimerWrapper *)__pyx_v_self));

  /* function exit code */
  __Pyx_RefNannyFinishContext();
  return __pyx_r;
}

static PyObject *__pyx_pf_4grpc_7_cython_6cygrpc_12TimerWrapper_2start(struct __pyx_obj_4grpc_7_cython_6cygrpc_TimerWrapper *__pyx_v_self) {
  PyObject *__pyx_r = NULL;
  __Pyx_RefNannyDeclarations
  PyObject *__pyx_t_1 = NULL;
  PyObject *__pyx_t_2 = NULL;
  PyObject *__pyx_t_3 = NULL;
  PyObject *__pyx_t_4 = NULL;
  PyObject *__pyx_t_5 = NULL;
  __Pyx_RefNannySetupContext("start", 0);

  /* "src/python/grpcio/grpc/_cython/_cygrpc/grpc_gevent.pyx.pxi":351
 * 
 *   def start(self):
 *     self.event = gevent_event.Event()             # <<<<<<<<<<<<<<
 *     self.timer.start(self.on_finish)
 * 
 */
  __pyx_t_2 = __Pyx_GetModuleGlobalName(__pyx_n_s_gevent_event); if (unlikely(!__pyx_t_2)) __PYX_ERR(6, 351, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_2);
  __pyx_t_3 = __Pyx_PyObject_GetAttrStr(__pyx_t_2, __pyx_n_s_Event); if (unlikely(!__pyx_t_3)) __PYX_ERR(6, 351, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_3);
  __Pyx_DECREF(__pyx_t_2); __pyx_t_2 = 0;
  __pyx_t_2 = NULL;
  if (CYTHON_UNPACK_METHODS && unlikely(PyMethod_Check(__pyx_t_3))) {
    __pyx_t_2 = PyMethod_GET_SELF(__pyx_t_3);
    if (likely(__pyx_t_2)) {
      PyObject* function = PyMethod_GET_FUNCTION(__pyx_t_3);
      __Pyx_INCREF(__pyx_t_2);
      __Pyx_INCREF(function);
      __Pyx_DECREF_SET(__pyx_t_3, function);
    }
  }
  if (__pyx_t_2) {
    __pyx_t_1 = __Pyx_PyObject_CallOneArg(__pyx_t_3, __pyx_t_2); if (unlikely(!__pyx_t_1)) __PYX_ERR(6, 351, __pyx_L1_error)
    __Pyx_DECREF(__pyx_t_2); __pyx_t_2 = 0;
  } else {
    __pyx_t_1 = __Pyx_PyObject_CallNoArg(__pyx_t_3); if (unlikely(!__pyx_t_1)) __PYX_ERR(6, 351, __pyx_L1_error)
  }
  __Pyx_GOTREF(__pyx_t_1);
  __Pyx_DECREF(__pyx_t_3); __pyx_t_3 = 0;
  __Pyx_GIVEREF(__pyx_t_1);
  __Pyx_GOTREF(__pyx_v_self->event);
  __Pyx_DECREF(__pyx_v_self->event);
  __pyx_v_self->event = __pyx_t_1;
  __pyx_t_1 = 0;

  /* "src/python/grpcio/grpc/_cython/_cygrpc/grpc_gevent.pyx.pxi":352
 *   def start(self):
 *     self.event = gevent_event.Event()
 *     self.timer.start(self.on_finish)             # <<<<<<<<<<<<<<
 * 
 *   def on_finish(self):
 */
  __pyx_t_3 = __Pyx_PyObject_GetAttrStr(__pyx_v_self->timer, __pyx_n_s_start); if (unlikely(!__pyx_t_3)) __PYX_ERR(6, 352, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_3);
  __pyx_t_2 = __Pyx_PyObject_GetAttrStr(((PyObject *)__pyx_v_self), __pyx_n_s_on_finish); if (unlikely(!__pyx_t_2)) __PYX_ERR(6, 352, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_2);
  __pyx_t_4 = NULL;
  if (CYTHON_UNPACK_METHODS && likely(PyMethod_Check(__pyx_t_3))) {
    __pyx_t_4 = PyMethod_GET_SELF(__pyx_t_3);
    if (likely(__pyx_t_4)) {
      PyObject* function = PyMethod_GET_FUNCTION(__pyx_t_3);
      __Pyx_INCREF(__pyx_t_4);
      __Pyx_INCREF(function);
      __Pyx_DECREF_SET(__pyx_t_3, function);
    }
  }
  if (!__pyx_t_4) {
    __pyx_t_1 = __Pyx_PyObject_CallOneArg(__pyx_t_3, __pyx_t_2); if (unlikely(!__pyx_t_1)) __PYX_ERR(6, 352, __pyx_L1_error)
    __Pyx_DECREF(__pyx_t_2); __pyx_t_2 = 0;
    __Pyx_GOTREF(__pyx_t_1);
  } else {
    #if CYTHON_FAST_PYCALL
    if (PyFunction_Check(__pyx_t_3)) {
      PyObject *__pyx_temp[2] = {__pyx_t_4, __pyx_t_2};
      __pyx_t_1 = __Pyx_PyFunction_FastCall(__pyx_t_3, __pyx_temp+1-1, 1+1); if (unlikely(!__pyx_t_1)) __PYX_ERR(6, 352, __pyx_L1_error)
      __Pyx_XDECREF(__pyx_t_4); __pyx_t_4 = 0;
      __Pyx_GOTREF(__pyx_t_1);
      __Pyx_DECREF(__pyx_t_2); __pyx_t_2 = 0;
    } else
    #endif
    #if CYTHON_FAST_PYCCALL
    if (__Pyx_PyFastCFunction_Check(__pyx_t_3)) {
      PyObject *__pyx_temp[2] = {__pyx_t_4, __pyx_t_2};
      __pyx_t_1 = __Pyx_PyCFunction_FastCall(__pyx_t_3, __pyx_temp+1-1, 1+1); if (unlikely(!__pyx_t_1)) __PYX_ERR(6, 352, __pyx_L1_error)
      __Pyx_XDECREF(__pyx_t_4); __pyx_t_4 = 0;
      __Pyx_GOTREF(__pyx_t_1);
      __Pyx_DECREF(__pyx_t_2); __pyx_t_2 = 0;
    } else
    #endif
    {
      __pyx_t_5 = PyTuple_New(1+1); if (unlikely(!__pyx_t_5)) __PYX_ERR(6, 352, __pyx_L1_error)
      __Pyx_GOTREF(__pyx_t_5);
      __Pyx_GIVEREF(__pyx_t_4); PyTuple_SET_ITEM(__pyx_t_5, 0, __pyx_t_4); __pyx_t_4 = NULL;
      __Pyx_GIVEREF(__pyx_t_2);
      PyTuple_SET_ITEM(__pyx_t_5, 0+1, __pyx_t_2);
      __pyx_t_2 = 0;
      __pyx_t_1 = __Pyx_PyObject_Call(__pyx_t_3, __pyx_t_5, NULL); if (unlikely(!__pyx_t_1)) __PYX_ERR(6, 352, __pyx_L1_error)
      __Pyx_GOTREF(__pyx_t_1);
      __Pyx_DECREF(__pyx_t_5); __pyx_t_5 = 0;
    }
  }
  __Pyx_DECREF(__pyx_t_3); __pyx_t_3 = 0;
  __Pyx_DECREF(__pyx_t_1); __pyx_t_1 = 0;

  /* "src/python/grpcio/grpc/_cython/_cygrpc/grpc_gevent.pyx.pxi":350
 *     self.event = None
 * 
 *   def start(self):             # <<<<<<<<<<<<<<
 *     self.event = gevent_event.Event()
 *     self.timer.start(self.on_finish)
 */

  /* function exit code */
  __pyx_r = Py_None; __Pyx_INCREF(Py_None);
  goto __pyx_L0;
  __pyx_L1_error:;
  __Pyx_XDECREF(__pyx_t_1);
  __Pyx_XDECREF(__pyx_t_2);
  __Pyx_XDECREF(__pyx_t_3);
  __Pyx_XDECREF(__pyx_t_4);
  __Pyx_XDECREF(__pyx_t_5);
  __Pyx_AddTraceback("grpc._cython.cygrpc.TimerWrapper.start", __pyx_clineno, __pyx_lineno, __pyx_filename);
  __pyx_r = NULL;
  __pyx_L0:;
  __Pyx_XGIVEREF(__pyx_r);
  __Pyx_RefNannyFinishContext();
  return __pyx_r;
}

/* "src/python/grpcio/grpc/_cython/_cygrpc/grpc_gevent.pyx.pxi":354
 *     self.timer.start(self.on_finish)
 * 
 *   def on_finish(self):             # <<<<<<<<<<<<<<
 *     grpc_custom_timer_callback(self.c_timer, grpc_error_none())
 *     self.timer.stop()
 */

/* Python wrapper */
static PyObject *__pyx_pw_4grpc_7_cython_6cygrpc_12TimerWrapper_5on_finish(PyObject *__pyx_v_self, CYTHON_UNUSED PyObject *unused); /*proto*/
static PyObject *__pyx_pw_4grpc_7_cython_6cygrpc_12TimerWrapper_5on_finish(PyObject *__pyx_v_self, CYTHON_UNUSED PyObject *unused) {
  PyObject *__pyx_r = 0;
  __Pyx_RefNannyDeclarations
  __Pyx_RefNannySetupContext("on_finish (wrapper)", 0);
  __pyx_r = __pyx_pf_4grpc_7_cython_6cygrpc_12TimerWrapper_4on_finish(((struct __pyx_obj_4grpc_7_cython_6cygrpc_TimerWrapper *)__pyx_v_self));

  /* function exit code */
  __Pyx_RefNannyFinishContext();
  return __pyx_r;
}

static PyObject *__pyx_pf_4grpc_7_cython_6cygrpc_12TimerWrapper_4on_finish(struct __pyx_obj_4grpc_7_cython_6cygrpc_TimerWrapper *__pyx_v_self) {
  PyObject *__pyx_r = NULL;
  __Pyx_RefNannyDeclarations
  PyObject *__pyx_t_1 = NULL;
  PyObject *__pyx_t_2 = NULL;
  PyObject *__pyx_t_3 = NULL;
  __Pyx_RefNannySetupContext("on_finish", 0);

  /* "src/python/grpcio/grpc/_cython/_cygrpc/grpc_gevent.pyx.pxi":355
 * 
 *   def on_finish(self):
 *     grpc_custom_timer_callback(self.c_timer, grpc_error_none())             # <<<<<<<<<<<<<<
 *     self.timer.stop()
 *     g_event.set()
 */
  grpc_custom_timer_callback(__pyx_v_self->c_timer, __pyx_f_4grpc_7_cython_6cygrpc_grpc_error_none());

  /* "src/python/grpcio/grpc/_cython/_cygrpc/grpc_gevent.pyx.pxi":356
 *   def on_finish(self):
 *     grpc_custom_timer_callback(self.c_timer, grpc_error_none())
 *     self.timer.stop()             # <<<<<<<<<<<<<<
 *     g_event.set()
 * 
 */
  __pyx_t_2 = __Pyx_PyObject_GetAttrStr(__pyx_v_self->timer, __pyx_n_s_stop); if (unlikely(!__pyx_t_2)) __PYX_ERR(6, 356, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_2);
  __pyx_t_3 = NULL;
  if (CYTHON_UNPACK_METHODS && likely(PyMethod_Check(__pyx_t_2))) {
    __pyx_t_3 = PyMethod_GET_SELF(__pyx_t_2);
    if (likely(__pyx_t_3)) {
      PyObject* function = PyMethod_GET_FUNCTION(__pyx_t_2);
      __Pyx_INCREF(__pyx_t_3);
      __Pyx_INCREF(function);
      __Pyx_DECREF_SET(__pyx_t_2, function);
    }
  }
  if (__pyx_t_3) {
    __pyx_t_1 = __Pyx_PyObject_CallOneArg(__pyx_t_2, __pyx_t_3); if (unlikely(!__pyx_t_1)) __PYX_ERR(6, 356, __pyx_L1_error)
    __Pyx_DECREF(__pyx_t_3); __pyx_t_3 = 0;
  } else {
    __pyx_t_1 = __Pyx_PyObject_CallNoArg(__pyx_t_2); if (unlikely(!__pyx_t_1)) __PYX_ERR(6, 356, __pyx_L1_error)
  }
  __Pyx_GOTREF(__pyx_t_1);
  __Pyx_DECREF(__pyx_t_2); __pyx_t_2 = 0;
  __Pyx_DECREF(__pyx_t_1); __pyx_t_1 = 0;

  /* "src/python/grpcio/grpc/_cython/_cygrpc/grpc_gevent.pyx.pxi":357
 *     grpc_custom_timer_callback(self.c_timer, grpc_error_none())
 *     self.timer.stop()
 *     g_event.set()             # <<<<<<<<<<<<<<
 * 
 *   def stop(self):
 */
  __pyx_t_2 = __Pyx_GetModuleGlobalName(__pyx_n_s_g_event); if (unlikely(!__pyx_t_2)) __PYX_ERR(6, 357, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_2);
  __pyx_t_3 = __Pyx_PyObject_GetAttrStr(__pyx_t_2, __pyx_n_s_set); if (unlikely(!__pyx_t_3)) __PYX_ERR(6, 357, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_3);
  __Pyx_DECREF(__pyx_t_2); __pyx_t_2 = 0;
  __pyx_t_2 = NULL;
  if (CYTHON_UNPACK_METHODS && unlikely(PyMethod_Check(__pyx_t_3))) {
    __pyx_t_2 = PyMethod_GET_SELF(__pyx_t_3);
    if (likely(__pyx_t_2)) {
      PyObject* function = PyMethod_GET_FUNCTION(__pyx_t_3);
      __Pyx_INCREF(__pyx_t_2);
      __Pyx_INCREF(function);
      __Pyx_DECREF_SET(__pyx_t_3, function);
    }
  }
  if (__pyx_t_2) {
    __pyx_t_1 = __Pyx_PyObject_CallOneArg(__pyx_t_3, __pyx_t_2); if (unlikely(!__pyx_t_1)) __PYX_ERR(6, 357, __pyx_L1_error)
    __Pyx_DECREF(__pyx_t_2); __pyx_t_2 = 0;
  } else {
    __pyx_t_1 = __Pyx_PyObject_CallNoArg(__pyx_t_3); if (unlikely(!__pyx_t_1)) __PYX_ERR(6, 357, __pyx_L1_error)
  }
  __Pyx_GOTREF(__pyx_t_1);
  __Pyx_DECREF(__pyx_t_3); __pyx_t_3 = 0;
  __Pyx_DECREF(__pyx_t_1); __pyx_t_1 = 0;

  /* "src/python/grpcio/grpc/_cython/_cygrpc/grpc_gevent.pyx.pxi":354
 *     self.timer.start(self.on_finish)
 * 
 *   def on_finish(self):             # <<<<<<<<<<<<<<
 *     grpc_custom_timer_callback(self.c_timer, grpc_error_none())
 *     self.timer.stop()
 */

  /* function exit code */
  __pyx_r = Py_None; __Pyx_INCREF(Py_None);
  goto __pyx_L0;
  __pyx_L1_error:;
  __Pyx_XDECREF(__pyx_t_1);
  __Pyx_XDECREF(__pyx_t_2);
  __Pyx_XDECREF(__pyx_t_3);
  __Pyx_AddTraceback("grpc._cython.cygrpc.TimerWrapper.on_finish", __pyx_clineno, __pyx_lineno, __pyx_filename);
  __pyx_r = NULL;
  __pyx_L0:;
  __Pyx_XGIVEREF(__pyx_r);
  __Pyx_RefNannyFinishContext();
  return __pyx_r;
}

/* "src/python/grpcio/grpc/_cython/_cygrpc/grpc_gevent.pyx.pxi":359
 *     g_event.set()
 * 
 *   def stop(self):             # <<<<<<<<<<<<<<
 *     self.event.set()
 *     self.timer.stop()
 */

/* Python wrapper */
static PyObject *__pyx_pw_4grpc_7_cython_6cygrpc_12TimerWrapper_7stop(PyObject *__pyx_v_self, CYTHON_UNUSED PyObject *unused); /*proto*/
static PyObject *__pyx_pw_4grpc_7_cython_6cygrpc_12TimerWrapper_7stop(PyObject *__pyx_v_self, CYTHON_UNUSED PyObject *unused) {
  PyObject *__pyx_r = 0;
  __Pyx_RefNannyDeclarations
  __Pyx_RefNannySetupContext("stop (wrapper)", 0);
  __pyx_r = __pyx_pf_4grpc_7_cython_6cygrpc_12TimerWrapper_6stop(((struct __pyx_obj_4grpc_7_cython_6cygrpc_TimerWrapper *)__pyx_v_self));

  /* function exit code */
  __Pyx_RefNannyFinishContext();
  return __pyx_r;
}

static PyObject *__pyx_pf_4grpc_7_cython_6cygrpc_12TimerWrapper_6stop(struct __pyx_obj_4grpc_7_cython_6cygrpc_TimerWrapper *__pyx_v_self) {
  PyObject *__pyx_r = NULL;
  __Pyx_RefNannyDeclarations
  PyObject *__pyx_t_1 = NULL;
  PyObject *__pyx_t_2 = NULL;
  PyObject *__pyx_t_3 = NULL;
  __Pyx_RefNannySetupContext("stop", 0);

  /* "src/python/grpcio/grpc/_cython/_cygrpc/grpc_gevent.pyx.pxi":360
 * 
 *   def stop(self):
 *     self.event.set()             # <<<<<<<<<<<<<<
 *     self.timer.stop()
 * 
 */
  __pyx_t_2 = __Pyx_PyObject_GetAttrStr(__pyx_v_self->event, __pyx_n_s_set); if (unlikely(!__pyx_t_2)) __PYX_ERR(6, 360, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_2);
  __pyx_t_3 = NULL;
  if (CYTHON_UNPACK_METHODS && likely(PyMethod_Check(__pyx_t_2))) {
    __pyx_t_3 = PyMethod_GET_SELF(__pyx_t_2);
    if (likely(__pyx_t_3)) {
      PyObject* function = PyMethod_GET_FUNCTION(__pyx_t_2);
      __Pyx_INCREF(__pyx_t_3);
      __Pyx_INCREF(function);
      __Pyx_DECREF_SET(__pyx_t_2, function);
    }
  }
  if (__pyx_t_3) {
    __pyx_t_1 = __Pyx_PyObject_CallOneArg(__pyx_t_2, __pyx_t_3); if (unlikely(!__pyx_t_1)) __PYX_ERR(6, 360, __pyx_L1_error)
    __Pyx_DECREF(__pyx_t_3); __pyx_t_3 = 0;
  } else {
    __pyx_t_1 = __Pyx_PyObject_CallNoArg(__pyx_t_2); if (unlikely(!__pyx_t_1)) __PYX_ERR(6, 360, __pyx_L1_error)
  }
  __Pyx_GOTREF(__pyx_t_1);
  __Pyx_DECREF(__pyx_t_2); __pyx_t_2 = 0;
  __Pyx_DECREF(__pyx_t_1); __pyx_t_1 = 0;

  /* "src/python/grpcio/grpc/_cython/_cygrpc/grpc_gevent.pyx.pxi":361
 *   def stop(self):
 *     self.event.set()
 *     self.timer.stop()             # <<<<<<<<<<<<<<
 * 
 * cdef void timer_start(grpc_custom_timer* t) except * with gil:
 */
  __pyx_t_2 = __Pyx_PyObject_GetAttrStr(__pyx_v_self->timer, __pyx_n_s_stop); if (unlikely(!__pyx_t_2)) __PYX_ERR(6, 361, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_2);
  __pyx_t_3 = NULL;
  if (CYTHON_UNPACK_METHODS && likely(PyMethod_Check(__pyx_t_2))) {
    __pyx_t_3 = PyMethod_GET_SELF(__pyx_t_2);
    if (likely(__pyx_t_3)) {
      PyObject* function = PyMethod_GET_FUNCTION(__pyx_t_2);
      __Pyx_INCREF(__pyx_t_3);
      __Pyx_INCREF(function);
      __Pyx_DECREF_SET(__pyx_t_2, function);
    }
  }
  if (__pyx_t_3) {
    __pyx_t_1 = __Pyx_PyObject_CallOneArg(__pyx_t_2, __pyx_t_3); if (unlikely(!__pyx_t_1)) __PYX_ERR(6, 361, __pyx_L1_error)
    __Pyx_DECREF(__pyx_t_3); __pyx_t_3 = 0;
  } else {
    __pyx_t_1 = __Pyx_PyObject_CallNoArg(__pyx_t_2); if (unlikely(!__pyx_t_1)) __PYX_ERR(6, 361, __pyx_L1_error)
  }
  __Pyx_GOTREF(__pyx_t_1);
  __Pyx_DECREF(__pyx_t_2); __pyx_t_2 = 0;
  __Pyx_DECREF(__pyx_t_1); __pyx_t_1 = 0;

  /* "src/python/grpcio/grpc/_cython/_cygrpc/grpc_gevent.pyx.pxi":359
 *     g_event.set()
 * 
 *   def stop(self):             # <<<<<<<<<<<<<<
 *     self.event.set()
 *     self.timer.stop()
 */

  /* function exit code */
  __pyx_r = Py_None; __Pyx_INCREF(Py_None);
  goto __pyx_L0;
  __pyx_L1_error:;
  __Pyx_XDECREF(__pyx_t_1);
  __Pyx_XDECREF(__pyx_t_2);
  __Pyx_XDECREF(__pyx_t_3);
  __Pyx_AddTraceback("grpc._cython.cygrpc.TimerWrapper.stop", __pyx_clineno, __pyx_lineno, __pyx_filename);
  __pyx_r = NULL;
  __pyx_L0:;
  __Pyx_XGIVEREF(__pyx_r);
  __Pyx_RefNannyFinishContext();
  return __pyx_r;
}

/* "(tree fragment)":1
 * def __reduce_cython__(self):             # <<<<<<<<<<<<<<
 *     raise TypeError("no default __reduce__ due to non-trivial __cinit__")
 * def __setstate_cython__(self, __pyx_state):
 */

/* Python wrapper */
static PyObject *__pyx_pw_4grpc_7_cython_6cygrpc_12TimerWrapper_9__reduce_cython__(PyObject *__pyx_v_self, CYTHON_UNUSED PyObject *unused); /*proto*/
static PyObject *__pyx_pw_4grpc_7_cython_6cygrpc_12TimerWrapper_9__reduce_cython__(PyObject *__pyx_v_self, CYTHON_UNUSED PyObject *unused) {
  PyObject *__pyx_r = 0;
  __Pyx_RefNannyDeclarations
  __Pyx_RefNannySetupContext("__reduce_cython__ (wrapper)", 0);
  __pyx_r = __pyx_pf_4grpc_7_cython_6cygrpc_12TimerWrapper_8__reduce_cython__(((struct __pyx_obj_4grpc_7_cython_6cygrpc_TimerWrapper *)__pyx_v_self));

  /* function exit code */
  __Pyx_RefNannyFinishContext();
  return __pyx_r;
}

static PyObject *__pyx_pf_4grpc_7_cython_6cygrpc_12TimerWrapper_8__reduce_cython__(CYTHON_UNUSED struct __pyx_obj_4grpc_7_cython_6cygrpc_TimerWrapper *__pyx_v_self) {
  PyObject *__pyx_r = NULL;
  __Pyx_RefNannyDeclarations
  PyObject *__pyx_t_1 = NULL;
  __Pyx_RefNannySetupContext("__reduce_cython__", 0);

  /* "(tree fragment)":2
 * def __reduce_cython__(self):
 *     raise TypeError("no default __reduce__ due to non-trivial __cinit__")             # <<<<<<<<<<<<<<
 * def __setstate_cython__(self, __pyx_state):
 *     raise TypeError("no default __reduce__ due to non-trivial __cinit__")
 */
  __pyx_t_1 = __Pyx_PyObject_Call(__pyx_builtin_TypeError, __pyx_tuple__144, NULL); if (unlikely(!__pyx_t_1)) __PYX_ERR(7, 2, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_1);
  __Pyx_Raise(__pyx_t_1, 0, 0, 0);
  __Pyx_DECREF(__pyx_t_1); __pyx_t_1 = 0;
  __PYX_ERR(7, 2, __pyx_L1_error)

  /* "(tree fragment)":1
 * def __reduce_cython__(self):             # <<<<<<<<<<<<<<
 *     raise TypeError("no default __reduce__ due to non-trivial __cinit__")
 * def __setstate_cython__(self, __pyx_state):
 */

  /* function exit code */
  __pyx_L1_error:;
  __Pyx_XDECREF(__pyx_t_1);
  __Pyx_AddTraceback("grpc._cython.cygrpc.TimerWrapper.__reduce_cython__", __pyx_clineno, __pyx_lineno, __pyx_filename);
  __pyx_r = NULL;
  __Pyx_XGIVEREF(__pyx_r);
  __Pyx_RefNannyFinishContext();
  return __pyx_r;
}

/* "(tree fragment)":3
 * def __reduce_cython__(self):
 *     raise TypeError("no default __reduce__ due to non-trivial __cinit__")
 * def __setstate_cython__(self, __pyx_state):             # <<<<<<<<<<<<<<
 *     raise TypeError("no default __reduce__ due to non-trivial __cinit__")
 */

/* Python wrapper */
static PyObject *__pyx_pw_4grpc_7_cython_6cygrpc_12TimerWrapper_11__setstate_cython__(PyObject *__pyx_v_self, PyObject *__pyx_v___pyx_state); /*proto*/
static PyObject *__pyx_pw_4grpc_7_cython_6cygrpc_12TimerWrapper_11__setstate_cython__(PyObject *__pyx_v_self, PyObject *__pyx_v___pyx_state) {
  PyObject *__pyx_r = 0;
  __Pyx_RefNannyDeclarations
  __Pyx_RefNannySetupContext("__setstate_cython__ (wrapper)", 0);
  __pyx_r = __pyx_pf_4grpc_7_cython_6cygrpc_12TimerWrapper_10__setstate_cython__(((struct __pyx_obj_4grpc_7_cython_6cygrpc_TimerWrapper *)__pyx_v_self), ((PyObject *)__pyx_v___pyx_state));

  /* function exit code */
  __Pyx_RefNannyFinishContext();
  return __pyx_r;
}

static PyObject *__pyx_pf_4grpc_7_cython_6cygrpc_12TimerWrapper_10__setstate_cython__(CYTHON_UNUSED struct __pyx_obj_4grpc_7_cython_6cygrpc_TimerWrapper *__pyx_v_self, CYTHON_UNUSED PyObject *__pyx_v___pyx_state) {
  PyObject *__pyx_r = NULL;
  __Pyx_RefNannyDeclarations
  PyObject *__pyx_t_1 = NULL;
  __Pyx_RefNannySetupContext("__setstate_cython__", 0);

  /* "(tree fragment)":4
 *     raise TypeError("no default __reduce__ due to non-trivial __cinit__")
 * def __setstate_cython__(self, __pyx_state):
 *     raise TypeError("no default __reduce__ due to non-trivial __cinit__")             # <<<<<<<<<<<<<<
 */
  __pyx_t_1 = __Pyx_PyObject_Call(__pyx_builtin_TypeError, __pyx_tuple__145, NULL); if (unlikely(!__pyx_t_1)) __PYX_ERR(7, 4, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_1);
  __Pyx_Raise(__pyx_t_1, 0, 0, 0);
  __Pyx_DECREF(__pyx_t_1); __pyx_t_1 = 0;
  __PYX_ERR(7, 4, __pyx_L1_error)

  /* "(tree fragment)":3
 * def __reduce_cython__(self):
 *     raise TypeError("no default __reduce__ due to non-trivial __cinit__")
 * def __setstate_cython__(self, __pyx_state):             # <<<<<<<<<<<<<<
 *     raise TypeError("no default __reduce__ due to non-trivial __cinit__")
 */

  /* function exit code */
  __pyx_L1_error:;
  __Pyx_XDECREF(__pyx_t_1);
  __Pyx_AddTraceback("grpc._cython.cygrpc.TimerWrapper.__setstate_cython__", __pyx_clineno, __pyx_lineno, __pyx_filename);
  __pyx_r = NULL;
  __Pyx_XGIVEREF(__pyx_r);
  __Pyx_RefNannyFinishContext();
  return __pyx_r;
}

/* "src/python/grpcio/grpc/_cython/_cygrpc/grpc_gevent.pyx.pxi":363
 *     self.timer.stop()
 * 
 * cdef void timer_start(grpc_custom_timer* t) except * with gil:             # <<<<<<<<<<<<<<
 *   timer = TimerWrapper(t.timeout_ms / 1000.0)
 *   timer.c_timer = t
 */

static void __pyx_f_4grpc_7_cython_6cygrpc_timer_start(struct grpc_custom_timer *__pyx_v_t) {
  struct __pyx_obj_4grpc_7_cython_6cygrpc_TimerWrapper *__pyx_v_timer = NULL;
  __Pyx_RefNannyDeclarations
  PyObject *__pyx_t_1 = NULL;
  PyObject *__pyx_t_2 = NULL;
  PyObject *__pyx_t_3 = NULL;
  #ifdef WITH_THREAD
  PyGILState_STATE __pyx_gilstate_save = __Pyx_PyGILState_Ensure();
  #endif
  __Pyx_RefNannySetupContext("timer_start", 0);

  /* "src/python/grpcio/grpc/_cython/_cygrpc/grpc_gevent.pyx.pxi":364
 * 
 * cdef void timer_start(grpc_custom_timer* t) except * with gil:
 *   timer = TimerWrapper(t.timeout_ms / 1000.0)             # <<<<<<<<<<<<<<
 *   timer.c_timer = t
 *   t.timer = <void*>timer
 */
  __pyx_t_1 = PyFloat_FromDouble((__pyx_v_t->timeout_ms / 1000.0)); if (unlikely(!__pyx_t_1)) __PYX_ERR(6, 364, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_1);
  __pyx_t_2 = __Pyx_PyObject_CallOneArg(((PyObject *)__pyx_ptype_4grpc_7_cython_6cygrpc_TimerWrapper), __pyx_t_1); if (unlikely(!__pyx_t_2)) __PYX_ERR(6, 364, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_2);
  __Pyx_DECREF(__pyx_t_1); __pyx_t_1 = 0;
  __pyx_v_timer = ((struct __pyx_obj_4grpc_7_cython_6cygrpc_TimerWrapper *)__pyx_t_2);
  __pyx_t_2 = 0;

  /* "src/python/grpcio/grpc/_cython/_cygrpc/grpc_gevent.pyx.pxi":365
 * cdef void timer_start(grpc_custom_timer* t) except * with gil:
 *   timer = TimerWrapper(t.timeout_ms / 1000.0)
 *   timer.c_timer = t             # <<<<<<<<<<<<<<
 *   t.timer = <void*>timer
 *   timer.start()
 */
  __pyx_v_timer->c_timer = __pyx_v_t;

  /* "src/python/grpcio/grpc/_cython/_cygrpc/grpc_gevent.pyx.pxi":366
 *   timer = TimerWrapper(t.timeout_ms / 1000.0)
 *   timer.c_timer = t
 *   t.timer = <void*>timer             # <<<<<<<<<<<<<<
 *   timer.start()
 * 
 */
  __pyx_v_t->timer = ((void *)__pyx_v_timer);

  /* "src/python/grpcio/grpc/_cython/_cygrpc/grpc_gevent.pyx.pxi":367
 *   timer.c_timer = t
 *   t.timer = <void*>timer
 *   timer.start()             # <<<<<<<<<<<<<<
 * 
 * cdef void timer_stop(grpc_custom_timer* t) except * with gil:
 */
  __pyx_t_1 = __Pyx_PyObject_GetAttrStr(((PyObject *)__pyx_v_timer), __pyx_n_s_start); if (unlikely(!__pyx_t_1)) __PYX_ERR(6, 367, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_1);
  __pyx_t_3 = NULL;
  if (CYTHON_UNPACK_METHODS && likely(PyMethod_Check(__pyx_t_1))) {
    __pyx_t_3 = PyMethod_GET_SELF(__pyx_t_1);
    if (likely(__pyx_t_3)) {
      PyObject* function = PyMethod_GET_FUNCTION(__pyx_t_1);
      __Pyx_INCREF(__pyx_t_3);
      __Pyx_INCREF(function);
      __Pyx_DECREF_SET(__pyx_t_1, function);
    }
  }
  if (__pyx_t_3) {
    __pyx_t_2 = __Pyx_PyObject_CallOneArg(__pyx_t_1, __pyx_t_3); if (unlikely(!__pyx_t_2)) __PYX_ERR(6, 367, __pyx_L1_error)
    __Pyx_DECREF(__pyx_t_3); __pyx_t_3 = 0;
  } else {
    __pyx_t_2 = __Pyx_PyObject_CallNoArg(__pyx_t_1); if (unlikely(!__pyx_t_2)) __PYX_ERR(6, 367, __pyx_L1_error)
  }
  __Pyx_GOTREF(__pyx_t_2);
  __Pyx_DECREF(__pyx_t_1); __pyx_t_1 = 0;
  __Pyx_DECREF(__pyx_t_2); __pyx_t_2 = 0;

  /* "src/python/grpcio/grpc/_cython/_cygrpc/grpc_gevent.pyx.pxi":363
 *     self.timer.stop()
 * 
 * cdef void timer_start(grpc_custom_timer* t) except * with gil:             # <<<<<<<<<<<<<<
 *   timer = TimerWrapper(t.timeout_ms / 1000.0)
 *   timer.c_timer = t
 */

  /* function exit code */
  goto __pyx_L0;
  __pyx_L1_error:;
  __Pyx_XDECREF(__pyx_t_1);
  __Pyx_XDECREF(__pyx_t_2);
  __Pyx_XDECREF(__pyx_t_3);
  __Pyx_AddTraceback("grpc._cython.cygrpc.timer_start", __pyx_clineno, __pyx_lineno, __pyx_filename);
  __pyx_L0:;
  __Pyx_XDECREF((PyObject *)__pyx_v_timer);
  __Pyx_RefNannyFinishContext();
  #ifdef WITH_THREAD
  __Pyx_PyGILState_Release(__pyx_gilstate_save);
  #endif
}

/* "src/python/grpcio/grpc/_cython/_cygrpc/grpc_gevent.pyx.pxi":369
 *   timer.start()
 * 
 * cdef void timer_stop(grpc_custom_timer* t) except * with gil:             # <<<<<<<<<<<<<<
 *   time_wrapper = <object>t.timer
 *   time_wrapper.stop()
 */

static void __pyx_f_4grpc_7_cython_6cygrpc_timer_stop(struct grpc_custom_timer *__pyx_v_t) {
  PyObject *__pyx_v_time_wrapper = NULL;
  __Pyx_RefNannyDeclarations
  PyObject *__pyx_t_1 = NULL;
  PyObject *__pyx_t_2 = NULL;
  PyObject *__pyx_t_3 = NULL;
  #ifdef WITH_THREAD
  PyGILState_STATE __pyx_gilstate_save = __Pyx_PyGILState_Ensure();
  #endif
  __Pyx_RefNannySetupContext("timer_stop", 0);

  /* "src/python/grpcio/grpc/_cython/_cygrpc/grpc_gevent.pyx.pxi":370
 * 
 * cdef void timer_stop(grpc_custom_timer* t) except * with gil:
 *   time_wrapper = <object>t.timer             # <<<<<<<<<<<<<<
 *   time_wrapper.stop()
 * 
 */
  __pyx_t_1 = ((PyObject *)__pyx_v_t->timer);
  __Pyx_INCREF(__pyx_t_1);
  __pyx_v_time_wrapper = __pyx_t_1;
  __pyx_t_1 = 0;

  /* "src/python/grpcio/grpc/_cython/_cygrpc/grpc_gevent.pyx.pxi":371
 * cdef void timer_stop(grpc_custom_timer* t) except * with gil:
 *   time_wrapper = <object>t.timer
 *   time_wrapper.stop()             # <<<<<<<<<<<<<<
 * 
 * ###############################
 */
  __pyx_t_2 = __Pyx_PyObject_GetAttrStr(__pyx_v_time_wrapper, __pyx_n_s_stop); if (unlikely(!__pyx_t_2)) __PYX_ERR(6, 371, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_2);
  __pyx_t_3 = NULL;
  if (CYTHON_UNPACK_METHODS && likely(PyMethod_Check(__pyx_t_2))) {
    __pyx_t_3 = PyMethod_GET_SELF(__pyx_t_2);
    if (likely(__pyx_t_3)) {
      PyObject* function = PyMethod_GET_FUNCTION(__pyx_t_2);
      __Pyx_INCREF(__pyx_t_3);
      __Pyx_INCREF(function);
      __Pyx_DECREF_SET(__pyx_t_2, function);
    }
  }
  if (__pyx_t_3) {
    __pyx_t_1 = __Pyx_PyObject_CallOneArg(__pyx_t_2, __pyx_t_3); if (unlikely(!__pyx_t_1)) __PYX_ERR(6, 371, __pyx_L1_error)
    __Pyx_DECREF(__pyx_t_3); __pyx_t_3 = 0;
  } else {
    __pyx_t_1 = __Pyx_PyObject_CallNoArg(__pyx_t_2); if (unlikely(!__pyx_t_1)) __PYX_ERR(6, 371, __pyx_L1_error)
  }
  __Pyx_GOTREF(__pyx_t_1);
  __Pyx_DECREF(__pyx_t_2); __pyx_t_2 = 0;
  __Pyx_DECREF(__pyx_t_1); __pyx_t_1 = 0;

  /* "src/python/grpcio/grpc/_cython/_cygrpc/grpc_gevent.pyx.pxi":369
 *   timer.start()
 * 
 * cdef void timer_stop(grpc_custom_timer* t) except * with gil:             # <<<<<<<<<<<<<<
 *   time_wrapper = <object>t.timer
 *   time_wrapper.stop()
 */

  /* function exit code */
  goto __pyx_L0;
  __pyx_L1_error:;
  __Pyx_XDECREF(__pyx_t_1);
  __Pyx_XDECREF(__pyx_t_2);
  __Pyx_XDECREF(__pyx_t_3);
  __Pyx_AddTraceback("grpc._cython.cygrpc.timer_stop", __pyx_clineno, __pyx_lineno, __pyx_filename);
  __pyx_L0:;
  __Pyx_XDECREF(__pyx_v_time_wrapper);
  __Pyx_RefNannyFinishContext();
  #ifdef WITH_THREAD
  __Pyx_PyGILState_Release(__pyx_gilstate_save);
  #endif
}

/* "src/python/grpcio/grpc/_cython/_cygrpc/grpc_gevent.pyx.pxi":377
 * ###############################
 * 
 * cdef void init_loop() except * with gil:             # <<<<<<<<<<<<<<
 *   pass
 * 
 */

static void __pyx_f_4grpc_7_cython_6cygrpc_init_loop(void) {
  __Pyx_RefNannyDeclarations
  #ifdef WITH_THREAD
  PyGILState_STATE __pyx_gilstate_save = __Pyx_PyGILState_Ensure();
  #endif
  __Pyx_RefNannySetupContext("init_loop", 0);

  /* function exit code */
  __Pyx_RefNannyFinishContext();
  #ifdef WITH_THREAD
  __Pyx_PyGILState_Release(__pyx_gilstate_save);
  #endif
}

/* "src/python/grpcio/grpc/_cython/_cygrpc/grpc_gevent.pyx.pxi":380
 *   pass
 * 
 * cdef void destroy_loop() except * with gil:             # <<<<<<<<<<<<<<
 *   g_pool.join()
 * 
 */

static void __pyx_f_4grpc_7_cython_6cygrpc_destroy_loop(void) {
  __Pyx_RefNannyDeclarations
  PyObject *__pyx_t_1 = NULL;
  PyObject *__pyx_t_2 = NULL;
  PyObject *__pyx_t_3 = NULL;
  #ifdef WITH_THREAD
  PyGILState_STATE __pyx_gilstate_save = __Pyx_PyGILState_Ensure();
  #endif
  __Pyx_RefNannySetupContext("destroy_loop", 0);

  /* "src/python/grpcio/grpc/_cython/_cygrpc/grpc_gevent.pyx.pxi":381
 * 
 * cdef void destroy_loop() except * with gil:
 *   g_pool.join()             # <<<<<<<<<<<<<<
 * 
 * cdef void kick_loop() except * with gil:
 */
  __pyx_t_2 = __Pyx_GetModuleGlobalName(__pyx_n_s_g_pool); if (unlikely(!__pyx_t_2)) __PYX_ERR(6, 381, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_2);
  __pyx_t_3 = __Pyx_PyObject_GetAttrStr(__pyx_t_2, __pyx_n_s_join); if (unlikely(!__pyx_t_3)) __PYX_ERR(6, 381, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_3);
  __Pyx_DECREF(__pyx_t_2); __pyx_t_2 = 0;
  __pyx_t_2 = NULL;
  if (CYTHON_UNPACK_METHODS && unlikely(PyMethod_Check(__pyx_t_3))) {
    __pyx_t_2 = PyMethod_GET_SELF(__pyx_t_3);
    if (likely(__pyx_t_2)) {
      PyObject* function = PyMethod_GET_FUNCTION(__pyx_t_3);
      __Pyx_INCREF(__pyx_t_2);
      __Pyx_INCREF(function);
      __Pyx_DECREF_SET(__pyx_t_3, function);
    }
  }
  if (__pyx_t_2) {
    __pyx_t_1 = __Pyx_PyObject_CallOneArg(__pyx_t_3, __pyx_t_2); if (unlikely(!__pyx_t_1)) __PYX_ERR(6, 381, __pyx_L1_error)
    __Pyx_DECREF(__pyx_t_2); __pyx_t_2 = 0;
  } else {
    __pyx_t_1 = __Pyx_PyObject_CallNoArg(__pyx_t_3); if (unlikely(!__pyx_t_1)) __PYX_ERR(6, 381, __pyx_L1_error)
  }
  __Pyx_GOTREF(__pyx_t_1);
  __Pyx_DECREF(__pyx_t_3); __pyx_t_3 = 0;
  __Pyx_DECREF(__pyx_t_1); __pyx_t_1 = 0;

  /* "src/python/grpcio/grpc/_cython/_cygrpc/grpc_gevent.pyx.pxi":380
 *   pass
 * 
 * cdef void destroy_loop() except * with gil:             # <<<<<<<<<<<<<<
 *   g_pool.join()
 * 
 */

  /* function exit code */
  goto __pyx_L0;
  __pyx_L1_error:;
  __Pyx_XDECREF(__pyx_t_1);
  __Pyx_XDECREF(__pyx_t_2);
  __Pyx_XDECREF(__pyx_t_3);
  __Pyx_AddTraceback("grpc._cython.cygrpc.destroy_loop", __pyx_clineno, __pyx_lineno, __pyx_filename);
  __pyx_L0:;
  __Pyx_RefNannyFinishContext();
  #ifdef WITH_THREAD
  __Pyx_PyGILState_Release(__pyx_gilstate_save);
  #endif
}

/* "src/python/grpcio/grpc/_cython/_cygrpc/grpc_gevent.pyx.pxi":383
 *   g_pool.join()
 * 
 * cdef void kick_loop() except * with gil:             # <<<<<<<<<<<<<<
 *   g_event.set()
 * 
 */

static void __pyx_f_4grpc_7_cython_6cygrpc_kick_loop(void) {
  __Pyx_RefNannyDeclarations
  PyObject *__pyx_t_1 = NULL;
  PyObject *__pyx_t_2 = NULL;
  PyObject *__pyx_t_3 = NULL;
  #ifdef WITH_THREAD
  PyGILState_STATE __pyx_gilstate_save = __Pyx_PyGILState_Ensure();
  #endif
  __Pyx_RefNannySetupContext("kick_loop", 0);

  /* "src/python/grpcio/grpc/_cython/_cygrpc/grpc_gevent.pyx.pxi":384
 * 
 * cdef void kick_loop() except * with gil:
 *   g_event.set()             # <<<<<<<<<<<<<<
 * 
 * cdef void run_loop(size_t timeout_ms) except * with gil:
 */
  __pyx_t_2 = __Pyx_GetModuleGlobalName(__pyx_n_s_g_event); if (unlikely(!__pyx_t_2)) __PYX_ERR(6, 384, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_2);
  __pyx_t_3 = __Pyx_PyObject_GetAttrStr(__pyx_t_2, __pyx_n_s_set); if (unlikely(!__pyx_t_3)) __PYX_ERR(6, 384, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_3);
  __Pyx_DECREF(__pyx_t_2); __pyx_t_2 = 0;
  __pyx_t_2 = NULL;
  if (CYTHON_UNPACK_METHODS && unlikely(PyMethod_Check(__pyx_t_3))) {
    __pyx_t_2 = PyMethod_GET_SELF(__pyx_t_3);
    if (likely(__pyx_t_2)) {
      PyObject* function = PyMethod_GET_FUNCTION(__pyx_t_3);
      __Pyx_INCREF(__pyx_t_2);
      __Pyx_INCREF(function);
      __Pyx_DECREF_SET(__pyx_t_3, function);
    }
  }
  if (__pyx_t_2) {
    __pyx_t_1 = __Pyx_PyObject_CallOneArg(__pyx_t_3, __pyx_t_2); if (unlikely(!__pyx_t_1)) __PYX_ERR(6, 384, __pyx_L1_error)
    __Pyx_DECREF(__pyx_t_2); __pyx_t_2 = 0;
  } else {
    __pyx_t_1 = __Pyx_PyObject_CallNoArg(__pyx_t_3); if (unlikely(!__pyx_t_1)) __PYX_ERR(6, 384, __pyx_L1_error)
  }
  __Pyx_GOTREF(__pyx_t_1);
  __Pyx_DECREF(__pyx_t_3); __pyx_t_3 = 0;
  __Pyx_DECREF(__pyx_t_1); __pyx_t_1 = 0;

  /* "src/python/grpcio/grpc/_cython/_cygrpc/grpc_gevent.pyx.pxi":383
 *   g_pool.join()
 * 
 * cdef void kick_loop() except * with gil:             # <<<<<<<<<<<<<<
 *   g_event.set()
 * 
 */

  /* function exit code */
  goto __pyx_L0;
  __pyx_L1_error:;
  __Pyx_XDECREF(__pyx_t_1);
  __Pyx_XDECREF(__pyx_t_2);
  __Pyx_XDECREF(__pyx_t_3);
  __Pyx_AddTraceback("grpc._cython.cygrpc.kick_loop", __pyx_clineno, __pyx_lineno, __pyx_filename);
  __pyx_L0:;
  __Pyx_RefNannyFinishContext();
  #ifdef WITH_THREAD
  __Pyx_PyGILState_Release(__pyx_gilstate_save);
  #endif
}

/* "src/python/grpcio/grpc/_cython/_cygrpc/grpc_gevent.pyx.pxi":386
 *   g_event.set()
 * 
 * cdef void run_loop(size_t timeout_ms) except * with gil:             # <<<<<<<<<<<<<<
 *     timeout = timeout_ms / 1000.0
 *     if timeout_ms > 0:
 */

static void __pyx_f_4grpc_7_cython_6cygrpc_run_loop(size_t __pyx_v_timeout_ms) {
  double __pyx_v_timeout;
  __Pyx_RefNannyDeclarations
  int __pyx_t_1;
  PyObject *__pyx_t_2 = NULL;
  PyObject *__pyx_t_3 = NULL;
  PyObject *__pyx_t_4 = NULL;
  PyObject *__pyx_t_5 = NULL;
  PyObject *__pyx_t_6 = NULL;
  #ifdef WITH_THREAD
  PyGILState_STATE __pyx_gilstate_save = __Pyx_PyGILState_Ensure();
  #endif
  __Pyx_RefNannySetupContext("run_loop", 0);

  /* "src/python/grpcio/grpc/_cython/_cygrpc/grpc_gevent.pyx.pxi":387
 * 
 * cdef void run_loop(size_t timeout_ms) except * with gil:
 *     timeout = timeout_ms / 1000.0             # <<<<<<<<<<<<<<
 *     if timeout_ms > 0:
 *       g_event.wait(timeout)
 */
  __pyx_v_timeout = (__pyx_v_timeout_ms / 1000.0);

  /* "src/python/grpcio/grpc/_cython/_cygrpc/grpc_gevent.pyx.pxi":388
 * cdef void run_loop(size_t timeout_ms) except * with gil:
 *     timeout = timeout_ms / 1000.0
 *     if timeout_ms > 0:             # <<<<<<<<<<<<<<
 *       g_event.wait(timeout)
 *       g_event.clear()
 */
  __pyx_t_1 = ((__pyx_v_timeout_ms > 0) != 0);
  if (__pyx_t_1) {

    /* "src/python/grpcio/grpc/_cython/_cygrpc/grpc_gevent.pyx.pxi":389
 *     timeout = timeout_ms / 1000.0
 *     if timeout_ms > 0:
 *       g_event.wait(timeout)             # <<<<<<<<<<<<<<
 *       g_event.clear()
 * 
 */
    __pyx_t_3 = __Pyx_GetModuleGlobalName(__pyx_n_s_g_event); if (unlikely(!__pyx_t_3)) __PYX_ERR(6, 389, __pyx_L1_error)
    __Pyx_GOTREF(__pyx_t_3);
    __pyx_t_4 = __Pyx_PyObject_GetAttrStr(__pyx_t_3, __pyx_n_s_wait); if (unlikely(!__pyx_t_4)) __PYX_ERR(6, 389, __pyx_L1_error)
    __Pyx_GOTREF(__pyx_t_4);
    __Pyx_DECREF(__pyx_t_3); __pyx_t_3 = 0;
    __pyx_t_3 = PyFloat_FromDouble(__pyx_v_timeout); if (unlikely(!__pyx_t_3)) __PYX_ERR(6, 389, __pyx_L1_error)
    __Pyx_GOTREF(__pyx_t_3);
    __pyx_t_5 = NULL;
    if (CYTHON_UNPACK_METHODS && unlikely(PyMethod_Check(__pyx_t_4))) {
      __pyx_t_5 = PyMethod_GET_SELF(__pyx_t_4);
      if (likely(__pyx_t_5)) {
        PyObject* function = PyMethod_GET_FUNCTION(__pyx_t_4);
        __Pyx_INCREF(__pyx_t_5);
        __Pyx_INCREF(function);
        __Pyx_DECREF_SET(__pyx_t_4, function);
      }
    }
    if (!__pyx_t_5) {
      __pyx_t_2 = __Pyx_PyObject_CallOneArg(__pyx_t_4, __pyx_t_3); if (unlikely(!__pyx_t_2)) __PYX_ERR(6, 389, __pyx_L1_error)
      __Pyx_DECREF(__pyx_t_3); __pyx_t_3 = 0;
      __Pyx_GOTREF(__pyx_t_2);
    } else {
      #if CYTHON_FAST_PYCALL
      if (PyFunction_Check(__pyx_t_4)) {
        PyObject *__pyx_temp[2] = {__pyx_t_5, __pyx_t_3};
        __pyx_t_2 = __Pyx_PyFunction_FastCall(__pyx_t_4, __pyx_temp+1-1, 1+1); if (unlikely(!__pyx_t_2)) __PYX_ERR(6, 389, __pyx_L1_error)
        __Pyx_XDECREF(__pyx_t_5); __pyx_t_5 = 0;
        __Pyx_GOTREF(__pyx_t_2);
        __Pyx_DECREF(__pyx_t_3); __pyx_t_3 = 0;
      } else
      #endif
      #if CYTHON_FAST_PYCCALL
      if (__Pyx_PyFastCFunction_Check(__pyx_t_4)) {
        PyObject *__pyx_temp[2] = {__pyx_t_5, __pyx_t_3};
        __pyx_t_2 = __Pyx_PyCFunction_FastCall(__pyx_t_4, __pyx_temp+1-1, 1+1); if (unlikely(!__pyx_t_2)) __PYX_ERR(6, 389, __pyx_L1_error)
        __Pyx_XDECREF(__pyx_t_5); __pyx_t_5 = 0;
        __Pyx_GOTREF(__pyx_t_2);
        __Pyx_DECREF(__pyx_t_3); __pyx_t_3 = 0;
      } else
      #endif
      {
        __pyx_t_6 = PyTuple_New(1+1); if (unlikely(!__pyx_t_6)) __PYX_ERR(6, 389, __pyx_L1_error)
        __Pyx_GOTREF(__pyx_t_6);
        __Pyx_GIVEREF(__pyx_t_5); PyTuple_SET_ITEM(__pyx_t_6, 0, __pyx_t_5); __pyx_t_5 = NULL;
        __Pyx_GIVEREF(__pyx_t_3);
        PyTuple_SET_ITEM(__pyx_t_6, 0+1, __pyx_t_3);
        __pyx_t_3 = 0;
        __pyx_t_2 = __Pyx_PyObject_Call(__pyx_t_4, __pyx_t_6, NULL); if (unlikely(!__pyx_t_2)) __PYX_ERR(6, 389, __pyx_L1_error)
        __Pyx_GOTREF(__pyx_t_2);
        __Pyx_DECREF(__pyx_t_6); __pyx_t_6 = 0;
      }
    }
    __Pyx_DECREF(__pyx_t_4); __pyx_t_4 = 0;
    __Pyx_DECREF(__pyx_t_2); __pyx_t_2 = 0;

    /* "src/python/grpcio/grpc/_cython/_cygrpc/grpc_gevent.pyx.pxi":390
 *     if timeout_ms > 0:
 *       g_event.wait(timeout)
 *       g_event.clear()             # <<<<<<<<<<<<<<
 * 
 * ###############################
 */
    __pyx_t_4 = __Pyx_GetModuleGlobalName(__pyx_n_s_g_event); if (unlikely(!__pyx_t_4)) __PYX_ERR(6, 390, __pyx_L1_error)
    __Pyx_GOTREF(__pyx_t_4);
    __pyx_t_6 = __Pyx_PyObject_GetAttrStr(__pyx_t_4, __pyx_n_s_clear); if (unlikely(!__pyx_t_6)) __PYX_ERR(6, 390, __pyx_L1_error)
    __Pyx_GOTREF(__pyx_t_6);
    __Pyx_DECREF(__pyx_t_4); __pyx_t_4 = 0;
    __pyx_t_4 = NULL;
    if (CYTHON_UNPACK_METHODS && unlikely(PyMethod_Check(__pyx_t_6))) {
      __pyx_t_4 = PyMethod_GET_SELF(__pyx_t_6);
      if (likely(__pyx_t_4)) {
        PyObject* function = PyMethod_GET_FUNCTION(__pyx_t_6);
        __Pyx_INCREF(__pyx_t_4);
        __Pyx_INCREF(function);
        __Pyx_DECREF_SET(__pyx_t_6, function);
      }
    }
    if (__pyx_t_4) {
      __pyx_t_2 = __Pyx_PyObject_CallOneArg(__pyx_t_6, __pyx_t_4); if (unlikely(!__pyx_t_2)) __PYX_ERR(6, 390, __pyx_L1_error)
      __Pyx_DECREF(__pyx_t_4); __pyx_t_4 = 0;
    } else {
      __pyx_t_2 = __Pyx_PyObject_CallNoArg(__pyx_t_6); if (unlikely(!__pyx_t_2)) __PYX_ERR(6, 390, __pyx_L1_error)
    }
    __Pyx_GOTREF(__pyx_t_2);
    __Pyx_DECREF(__pyx_t_6); __pyx_t_6 = 0;
    __Pyx_DECREF(__pyx_t_2); __pyx_t_2 = 0;

    /* "src/python/grpcio/grpc/_cython/_cygrpc/grpc_gevent.pyx.pxi":388
 * cdef void run_loop(size_t timeout_ms) except * with gil:
 *     timeout = timeout_ms / 1000.0
 *     if timeout_ms > 0:             # <<<<<<<<<<<<<<
 *       g_event.wait(timeout)
 *       g_event.clear()
 */
  }

  /* "src/python/grpcio/grpc/_cython/_cygrpc/grpc_gevent.pyx.pxi":386
 *   g_event.set()
 * 
 * cdef void run_loop(size_t timeout_ms) except * with gil:             # <<<<<<<<<<<<<<
 *     timeout = timeout_ms / 1000.0
 *     if timeout_ms > 0:
 */

  /* function exit code */
  goto __pyx_L0;
  __pyx_L1_error:;
  __Pyx_XDECREF(__pyx_t_2);
  __Pyx_XDECREF(__pyx_t_3);
  __Pyx_XDECREF(__pyx_t_4);
  __Pyx_XDECREF(__pyx_t_5);
  __Pyx_XDECREF(__pyx_t_6);
  __Pyx_AddTraceback("grpc._cython.cygrpc.run_loop", __pyx_clineno, __pyx_lineno, __pyx_filename);
  __pyx_L0:;
  __Pyx_RefNannyFinishContext();
  #ifdef WITH_THREAD
  __Pyx_PyGILState_Release(__pyx_gilstate_save);
  #endif
}

/* "src/python/grpcio/grpc/_cython/_cygrpc/grpc_gevent.pyx.pxi":401
 * cdef grpc_custom_poller_vtable gevent_pollset_vtable
 * 
 * def init_grpc_gevent():             # <<<<<<<<<<<<<<
 *   # Lazily import gevent
 *   global gevent_socket
 */

/* Python wrapper */
static PyObject *__pyx_pw_4grpc_7_cython_6cygrpc_61init_grpc_gevent(PyObject *__pyx_self, CYTHON_UNUSED PyObject *unused); /*proto*/
static PyMethodDef __pyx_mdef_4grpc_7_cython_6cygrpc_61init_grpc_gevent = {"init_grpc_gevent", (PyCFunction)__pyx_pw_4grpc_7_cython_6cygrpc_61init_grpc_gevent, METH_NOARGS, 0};
static PyObject *__pyx_pw_4grpc_7_cython_6cygrpc_61init_grpc_gevent(PyObject *__pyx_self, CYTHON_UNUSED PyObject *unused) {
  PyObject *__pyx_r = 0;
  __Pyx_RefNannyDeclarations
  __Pyx_RefNannySetupContext("init_grpc_gevent (wrapper)", 0);
  __pyx_r = __pyx_pf_4grpc_7_cython_6cygrpc_60init_grpc_gevent(__pyx_self);

  /* function exit code */
  __Pyx_RefNannyFinishContext();
  return __pyx_r;
}

/* "src/python/grpcio/grpc/_cython/_cygrpc/grpc_gevent.pyx.pxi":422
 *   g_pool = gevent.pool.Group()
 * 
 *   def cb_func(cb, args):             # <<<<<<<<<<<<<<
 *     _spawn_greenlet(cb, *args)
 *   set_async_callback_func(cb_func)
 */

/* Python wrapper */
static PyObject *__pyx_pw_4grpc_7_cython_6cygrpc_16init_grpc_gevent_1cb_func(PyObject *__pyx_self, PyObject *__pyx_args, PyObject *__pyx_kwds); /*proto*/
static PyMethodDef __pyx_mdef_4grpc_7_cython_6cygrpc_16init_grpc_gevent_1cb_func = {"cb_func", (PyCFunction)__pyx_pw_4grpc_7_cython_6cygrpc_16init_grpc_gevent_1cb_func, METH_VARARGS|METH_KEYWORDS, 0};
static PyObject *__pyx_pw_4grpc_7_cython_6cygrpc_16init_grpc_gevent_1cb_func(PyObject *__pyx_self, PyObject *__pyx_args, PyObject *__pyx_kwds) {
  PyObject *__pyx_v_cb = 0;
  PyObject *__pyx_v_args = 0;
  PyObject *__pyx_r = 0;
  __Pyx_RefNannyDeclarations
  __Pyx_RefNannySetupContext("cb_func (wrapper)", 0);
  {
    static PyObject **__pyx_pyargnames[] = {&__pyx_n_s_cb,&__pyx_n_s_args,0};
    PyObject* values[2] = {0,0};
    if (unlikely(__pyx_kwds)) {
      Py_ssize_t kw_args;
      const Py_ssize_t pos_args = PyTuple_GET_SIZE(__pyx_args);
      switch (pos_args) {
        case  2: values[1] = PyTuple_GET_ITEM(__pyx_args, 1);
        CYTHON_FALLTHROUGH;
        case  1: values[0] = PyTuple_GET_ITEM(__pyx_args, 0);
        CYTHON_FALLTHROUGH;
        case  0: break;
        default: goto __pyx_L5_argtuple_error;
      }
      kw_args = PyDict_Size(__pyx_kwds);
      switch (pos_args) {
        case  0:
        if (likely((values[0] = __Pyx_PyDict_GetItemStr(__pyx_kwds, __pyx_n_s_cb)) != 0)) kw_args--;
        else goto __pyx_L5_argtuple_error;
        CYTHON_FALLTHROUGH;
        case  1:
        if (likely((values[1] = __Pyx_PyDict_GetItemStr(__pyx_kwds, __pyx_n_s_args)) != 0)) kw_args--;
        else {
          __Pyx_RaiseArgtupleInvalid("cb_func", 1, 2, 2, 1); __PYX_ERR(6, 422, __pyx_L3_error)
        }
      }
      if (unlikely(kw_args > 0)) {
        if (unlikely(__Pyx_ParseOptionalKeywords(__pyx_kwds, __pyx_pyargnames, 0, values, pos_args, "cb_func") < 0)) __PYX_ERR(6, 422, __pyx_L3_error)
      }
    } else if (PyTuple_GET_SIZE(__pyx_args) != 2) {
      goto __pyx_L5_argtuple_error;
    } else {
      values[0] = PyTuple_GET_ITEM(__pyx_args, 0);
      values[1] = PyTuple_GET_ITEM(__pyx_args, 1);
    }
    __pyx_v_cb = values[0];
    __pyx_v_args = values[1];
  }
  goto __pyx_L4_argument_unpacking_done;
  __pyx_L5_argtuple_error:;
  __Pyx_RaiseArgtupleInvalid("cb_func", 1, 2, 2, PyTuple_GET_SIZE(__pyx_args)); __PYX_ERR(6, 422, __pyx_L3_error)
  __pyx_L3_error:;
  __Pyx_AddTraceback("grpc._cython.cygrpc.init_grpc_gevent.cb_func", __pyx_clineno, __pyx_lineno, __pyx_filename);
  __Pyx_RefNannyFinishContext();
  return NULL;
  __pyx_L4_argument_unpacking_done:;
  __pyx_r = __pyx_pf_4grpc_7_cython_6cygrpc_16init_grpc_gevent_cb_func(__pyx_self, __pyx_v_cb, __pyx_v_args);

  /* function exit code */
  __Pyx_RefNannyFinishContext();
  return __pyx_r;
}

static PyObject *__pyx_pf_4grpc_7_cython_6cygrpc_16init_grpc_gevent_cb_func(CYTHON_UNUSED PyObject *__pyx_self, PyObject *__pyx_v_cb, PyObject *__pyx_v_args) {
  PyObject *__pyx_r = NULL;
  __Pyx_RefNannyDeclarations
  PyObject *__pyx_t_1 = NULL;
  PyObject *__pyx_t_2 = NULL;
  PyObject *__pyx_t_3 = NULL;
  PyObject *__pyx_t_4 = NULL;
  __Pyx_RefNannySetupContext("cb_func", 0);

  /* "src/python/grpcio/grpc/_cython/_cygrpc/grpc_gevent.pyx.pxi":423
 * 
 *   def cb_func(cb, args):
 *     _spawn_greenlet(cb, *args)             # <<<<<<<<<<<<<<
 *   set_async_callback_func(cb_func)
 * 
 */
  __pyx_t_1 = __Pyx_GetModuleGlobalName(__pyx_n_s_spawn_greenlet); if (unlikely(!__pyx_t_1)) __PYX_ERR(6, 423, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_1);
  __pyx_t_2 = PyTuple_New(1); if (unlikely(!__pyx_t_2)) __PYX_ERR(6, 423, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_2);
  __Pyx_INCREF(__pyx_v_cb);
  __Pyx_GIVEREF(__pyx_v_cb);
  PyTuple_SET_ITEM(__pyx_t_2, 0, __pyx_v_cb);
  __pyx_t_3 = __Pyx_PySequence_Tuple(__pyx_v_args); if (unlikely(!__pyx_t_3)) __PYX_ERR(6, 423, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_3);
  __pyx_t_4 = PyNumber_Add(__pyx_t_2, __pyx_t_3); if (unlikely(!__pyx_t_4)) __PYX_ERR(6, 423, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_4);
  __Pyx_DECREF(__pyx_t_2); __pyx_t_2 = 0;
  __Pyx_DECREF(__pyx_t_3); __pyx_t_3 = 0;
  __pyx_t_3 = __Pyx_PyObject_Call(__pyx_t_1, __pyx_t_4, NULL); if (unlikely(!__pyx_t_3)) __PYX_ERR(6, 423, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_3);
  __Pyx_DECREF(__pyx_t_1); __pyx_t_1 = 0;
  __Pyx_DECREF(__pyx_t_4); __pyx_t_4 = 0;
  __Pyx_DECREF(__pyx_t_3); __pyx_t_3 = 0;

  /* "src/python/grpcio/grpc/_cython/_cygrpc/grpc_gevent.pyx.pxi":422
 *   g_pool = gevent.pool.Group()
 * 
 *   def cb_func(cb, args):             # <<<<<<<<<<<<<<
 *     _spawn_greenlet(cb, *args)
 *   set_async_callback_func(cb_func)
 */

  /* function exit code */
  __pyx_r = Py_None; __Pyx_INCREF(Py_None);
  goto __pyx_L0;
  __pyx_L1_error:;
  __Pyx_XDECREF(__pyx_t_1);
  __Pyx_XDECREF(__pyx_t_2);
  __Pyx_XDECREF(__pyx_t_3);
  __Pyx_XDECREF(__pyx_t_4);
  __Pyx_AddTraceback("grpc._cython.cygrpc.init_grpc_gevent.cb_func", __pyx_clineno, __pyx_lineno, __pyx_filename);
  __pyx_r = NULL;
  __pyx_L0:;
  __Pyx_XGIVEREF(__pyx_r);
  __Pyx_RefNannyFinishContext();
  return __pyx_r;
}

/* "src/python/grpcio/grpc/_cython/_cygrpc/grpc_gevent.pyx.pxi":401
 * cdef grpc_custom_poller_vtable gevent_pollset_vtable
 * 
 * def init_grpc_gevent():             # <<<<<<<<<<<<<<
 *   # Lazily import gevent
 *   global gevent_socket
 */

static PyObject *__pyx_pf_4grpc_7_cython_6cygrpc_60init_grpc_gevent(CYTHON_UNUSED PyObject *__pyx_self) {
  PyObject *__pyx_v_gevent = NULL;
  PyObject *__pyx_v_cb_func = 0;
  PyObject *__pyx_r = NULL;
  __Pyx_RefNannyDeclarations
  PyObject *__pyx_t_1 = NULL;
  PyObject *__pyx_t_2 = NULL;
  PyObject *__pyx_t_3 = NULL;
  PyObject *__pyx_t_4 = NULL;
  __Pyx_RefNannySetupContext("init_grpc_gevent", 0);

  /* "src/python/grpcio/grpc/_cython/_cygrpc/grpc_gevent.pyx.pxi":409
 *   global g_event
 *   global g_pool
 *   import gevent             # <<<<<<<<<<<<<<
 *   gevent_g = gevent
 *   import gevent.socket
 */
  __pyx_t_1 = __Pyx_Import(__pyx_n_s_gevent, 0, -1); if (unlikely(!__pyx_t_1)) __PYX_ERR(6, 409, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_1);
  __pyx_v_gevent = __pyx_t_1;
  __pyx_t_1 = 0;

  /* "src/python/grpcio/grpc/_cython/_cygrpc/grpc_gevent.pyx.pxi":410
 *   global g_pool
 *   import gevent
 *   gevent_g = gevent             # <<<<<<<<<<<<<<
 *   import gevent.socket
 *   gevent_socket = gevent.socket
 */
  if (PyDict_SetItem(__pyx_d, __pyx_n_s_gevent_g, __pyx_v_gevent) < 0) __PYX_ERR(6, 410, __pyx_L1_error)

  /* "src/python/grpcio/grpc/_cython/_cygrpc/grpc_gevent.pyx.pxi":411
 *   import gevent
 *   gevent_g = gevent
 *   import gevent.socket             # <<<<<<<<<<<<<<
 *   gevent_socket = gevent.socket
 *   import gevent.hub
 */
  __pyx_t_1 = __Pyx_Import(__pyx_n_s_gevent_socket_2, 0, -1); if (unlikely(!__pyx_t_1)) __PYX_ERR(6, 411, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_1);
  __Pyx_DECREF_SET(__pyx_v_gevent, __pyx_t_1);
  __pyx_t_1 = 0;

  /* "src/python/grpcio/grpc/_cython/_cygrpc/grpc_gevent.pyx.pxi":412
 *   gevent_g = gevent
 *   import gevent.socket
 *   gevent_socket = gevent.socket             # <<<<<<<<<<<<<<
 *   import gevent.hub
 *   gevent_hub = gevent.hub
 */
  __pyx_t_1 = __Pyx_PyObject_GetAttrStr(__pyx_v_gevent, __pyx_n_s_socket); if (unlikely(!__pyx_t_1)) __PYX_ERR(6, 412, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_1);
  if (PyDict_SetItem(__pyx_d, __pyx_n_s_gevent_socket, __pyx_t_1) < 0) __PYX_ERR(6, 412, __pyx_L1_error)
  __Pyx_DECREF(__pyx_t_1); __pyx_t_1 = 0;

  /* "src/python/grpcio/grpc/_cython/_cygrpc/grpc_gevent.pyx.pxi":413
 *   import gevent.socket
 *   gevent_socket = gevent.socket
 *   import gevent.hub             # <<<<<<<<<<<<<<
 *   gevent_hub = gevent.hub
 *   import gevent.event
 */
  __pyx_t_1 = __Pyx_Import(__pyx_n_s_gevent_hub_2, 0, -1); if (unlikely(!__pyx_t_1)) __PYX_ERR(6, 413, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_1);
  __Pyx_DECREF_SET(__pyx_v_gevent, __pyx_t_1);
  __pyx_t_1 = 0;

  /* "src/python/grpcio/grpc/_cython/_cygrpc/grpc_gevent.pyx.pxi":414
 *   gevent_socket = gevent.socket
 *   import gevent.hub
 *   gevent_hub = gevent.hub             # <<<<<<<<<<<<<<
 *   import gevent.event
 *   gevent_event = gevent.event
 */
  __pyx_t_1 = __Pyx_PyObject_GetAttrStr(__pyx_v_gevent, __pyx_n_s_hub); if (unlikely(!__pyx_t_1)) __PYX_ERR(6, 414, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_1);
  if (PyDict_SetItem(__pyx_d, __pyx_n_s_gevent_hub, __pyx_t_1) < 0) __PYX_ERR(6, 414, __pyx_L1_error)
  __Pyx_DECREF(__pyx_t_1); __pyx_t_1 = 0;

  /* "src/python/grpcio/grpc/_cython/_cygrpc/grpc_gevent.pyx.pxi":415
 *   import gevent.hub
 *   gevent_hub = gevent.hub
 *   import gevent.event             # <<<<<<<<<<<<<<
 *   gevent_event = gevent.event
 *   import gevent.pool
 */
  __pyx_t_1 = __Pyx_Import(__pyx_n_s_gevent_event_2, 0, -1); if (unlikely(!__pyx_t_1)) __PYX_ERR(6, 415, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_1);
  __Pyx_DECREF_SET(__pyx_v_gevent, __pyx_t_1);
  __pyx_t_1 = 0;

  /* "src/python/grpcio/grpc/_cython/_cygrpc/grpc_gevent.pyx.pxi":416
 *   gevent_hub = gevent.hub
 *   import gevent.event
 *   gevent_event = gevent.event             # <<<<<<<<<<<<<<
 *   import gevent.pool
 * 
 */
  __pyx_t_1 = __Pyx_PyObject_GetAttrStr(__pyx_v_gevent, __pyx_n_s_event); if (unlikely(!__pyx_t_1)) __PYX_ERR(6, 416, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_1);
  if (PyDict_SetItem(__pyx_d, __pyx_n_s_gevent_event, __pyx_t_1) < 0) __PYX_ERR(6, 416, __pyx_L1_error)
  __Pyx_DECREF(__pyx_t_1); __pyx_t_1 = 0;

  /* "src/python/grpcio/grpc/_cython/_cygrpc/grpc_gevent.pyx.pxi":417
 *   import gevent.event
 *   gevent_event = gevent.event
 *   import gevent.pool             # <<<<<<<<<<<<<<
 * 
 *   g_event = gevent.event.Event()
 */
  __pyx_t_1 = __Pyx_Import(__pyx_n_s_gevent_pool, 0, -1); if (unlikely(!__pyx_t_1)) __PYX_ERR(6, 417, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_1);
  __Pyx_DECREF_SET(__pyx_v_gevent, __pyx_t_1);
  __pyx_t_1 = 0;

  /* "src/python/grpcio/grpc/_cython/_cygrpc/grpc_gevent.pyx.pxi":419
 *   import gevent.pool
 * 
 *   g_event = gevent.event.Event()             # <<<<<<<<<<<<<<
 *   g_pool = gevent.pool.Group()
 * 
 */
  __pyx_t_2 = __Pyx_PyObject_GetAttrStr(__pyx_v_gevent, __pyx_n_s_event); if (unlikely(!__pyx_t_2)) __PYX_ERR(6, 419, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_2);
  __pyx_t_3 = __Pyx_PyObject_GetAttrStr(__pyx_t_2, __pyx_n_s_Event); if (unlikely(!__pyx_t_3)) __PYX_ERR(6, 419, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_3);
  __Pyx_DECREF(__pyx_t_2); __pyx_t_2 = 0;
  __pyx_t_2 = NULL;
  if (CYTHON_UNPACK_METHODS && likely(PyMethod_Check(__pyx_t_3))) {
    __pyx_t_2 = PyMethod_GET_SELF(__pyx_t_3);
    if (likely(__pyx_t_2)) {
      PyObject* function = PyMethod_GET_FUNCTION(__pyx_t_3);
      __Pyx_INCREF(__pyx_t_2);
      __Pyx_INCREF(function);
      __Pyx_DECREF_SET(__pyx_t_3, function);
    }
  }
  if (__pyx_t_2) {
    __pyx_t_1 = __Pyx_PyObject_CallOneArg(__pyx_t_3, __pyx_t_2); if (unlikely(!__pyx_t_1)) __PYX_ERR(6, 419, __pyx_L1_error)
    __Pyx_DECREF(__pyx_t_2); __pyx_t_2 = 0;
  } else {
    __pyx_t_1 = __Pyx_PyObject_CallNoArg(__pyx_t_3); if (unlikely(!__pyx_t_1)) __PYX_ERR(6, 419, __pyx_L1_error)
  }
  __Pyx_GOTREF(__pyx_t_1);
  __Pyx_DECREF(__pyx_t_3); __pyx_t_3 = 0;
  if (PyDict_SetItem(__pyx_d, __pyx_n_s_g_event, __pyx_t_1) < 0) __PYX_ERR(6, 419, __pyx_L1_error)
  __Pyx_DECREF(__pyx_t_1); __pyx_t_1 = 0;

  /* "src/python/grpcio/grpc/_cython/_cygrpc/grpc_gevent.pyx.pxi":420
 * 
 *   g_event = gevent.event.Event()
 *   g_pool = gevent.pool.Group()             # <<<<<<<<<<<<<<
 * 
 *   def cb_func(cb, args):
 */
  __pyx_t_3 = __Pyx_PyObject_GetAttrStr(__pyx_v_gevent, __pyx_n_s_pool); if (unlikely(!__pyx_t_3)) __PYX_ERR(6, 420, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_3);
  __pyx_t_2 = __Pyx_PyObject_GetAttrStr(__pyx_t_3, __pyx_n_s_Group); if (unlikely(!__pyx_t_2)) __PYX_ERR(6, 420, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_2);
  __Pyx_DECREF(__pyx_t_3); __pyx_t_3 = 0;
  __pyx_t_3 = NULL;
  if (CYTHON_UNPACK_METHODS && likely(PyMethod_Check(__pyx_t_2))) {
    __pyx_t_3 = PyMethod_GET_SELF(__pyx_t_2);
    if (likely(__pyx_t_3)) {
      PyObject* function = PyMethod_GET_FUNCTION(__pyx_t_2);
      __Pyx_INCREF(__pyx_t_3);
      __Pyx_INCREF(function);
      __Pyx_DECREF_SET(__pyx_t_2, function);
    }
  }
  if (__pyx_t_3) {
    __pyx_t_1 = __Pyx_PyObject_CallOneArg(__pyx_t_2, __pyx_t_3); if (unlikely(!__pyx_t_1)) __PYX_ERR(6, 420, __pyx_L1_error)
    __Pyx_DECREF(__pyx_t_3); __pyx_t_3 = 0;
  } else {
    __pyx_t_1 = __Pyx_PyObject_CallNoArg(__pyx_t_2); if (unlikely(!__pyx_t_1)) __PYX_ERR(6, 420, __pyx_L1_error)
  }
  __Pyx_GOTREF(__pyx_t_1);
  __Pyx_DECREF(__pyx_t_2); __pyx_t_2 = 0;
  if (PyDict_SetItem(__pyx_d, __pyx_n_s_g_pool, __pyx_t_1) < 0) __PYX_ERR(6, 420, __pyx_L1_error)
  __Pyx_DECREF(__pyx_t_1); __pyx_t_1 = 0;

  /* "src/python/grpcio/grpc/_cython/_cygrpc/grpc_gevent.pyx.pxi":422
 *   g_pool = gevent.pool.Group()
 * 
 *   def cb_func(cb, args):             # <<<<<<<<<<<<<<
 *     _spawn_greenlet(cb, *args)
 *   set_async_callback_func(cb_func)
 */
  __pyx_t_1 = __Pyx_CyFunction_NewEx(&__pyx_mdef_4grpc_7_cython_6cygrpc_16init_grpc_gevent_1cb_func, 0, __pyx_n_s_init_grpc_gevent_locals_cb_func, NULL, __pyx_n_s_grpc__cython_cygrpc, __pyx_d, ((PyObject *)__pyx_codeobj__147)); if (unlikely(!__pyx_t_1)) __PYX_ERR(6, 422, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_1);
  __pyx_v_cb_func = __pyx_t_1;
  __pyx_t_1 = 0;

  /* "src/python/grpcio/grpc/_cython/_cygrpc/grpc_gevent.pyx.pxi":424
 *   def cb_func(cb, args):
 *     _spawn_greenlet(cb, *args)
 *   set_async_callback_func(cb_func)             # <<<<<<<<<<<<<<
 * 
 *   gevent_resolver_vtable.resolve = socket_resolve
 */
  __pyx_t_2 = __Pyx_GetModuleGlobalName(__pyx_n_s_set_async_callback_func); if (unlikely(!__pyx_t_2)) __PYX_ERR(6, 424, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_2);
  __pyx_t_3 = NULL;
  if (CYTHON_UNPACK_METHODS && unlikely(PyMethod_Check(__pyx_t_2))) {
    __pyx_t_3 = PyMethod_GET_SELF(__pyx_t_2);
    if (likely(__pyx_t_3)) {
      PyObject* function = PyMethod_GET_FUNCTION(__pyx_t_2);
      __Pyx_INCREF(__pyx_t_3);
      __Pyx_INCREF(function);
      __Pyx_DECREF_SET(__pyx_t_2, function);
    }
  }
  if (!__pyx_t_3) {
    __pyx_t_1 = __Pyx_PyObject_CallOneArg(__pyx_t_2, __pyx_v_cb_func); if (unlikely(!__pyx_t_1)) __PYX_ERR(6, 424, __pyx_L1_error)
    __Pyx_GOTREF(__pyx_t_1);
  } else {
    #if CYTHON_FAST_PYCALL
    if (PyFunction_Check(__pyx_t_2)) {
      PyObject *__pyx_temp[2] = {__pyx_t_3, __pyx_v_cb_func};
      __pyx_t_1 = __Pyx_PyFunction_FastCall(__pyx_t_2, __pyx_temp+1-1, 1+1); if (unlikely(!__pyx_t_1)) __PYX_ERR(6, 424, __pyx_L1_error)
      __Pyx_XDECREF(__pyx_t_3); __pyx_t_3 = 0;
      __Pyx_GOTREF(__pyx_t_1);
    } else
    #endif
    #if CYTHON_FAST_PYCCALL
    if (__Pyx_PyFastCFunction_Check(__pyx_t_2)) {
      PyObject *__pyx_temp[2] = {__pyx_t_3, __pyx_v_cb_func};
      __pyx_t_1 = __Pyx_PyCFunction_FastCall(__pyx_t_2, __pyx_temp+1-1, 1+1); if (unlikely(!__pyx_t_1)) __PYX_ERR(6, 424, __pyx_L1_error)
      __Pyx_XDECREF(__pyx_t_3); __pyx_t_3 = 0;
      __Pyx_GOTREF(__pyx_t_1);
    } else
    #endif
    {
      __pyx_t_4 = PyTuple_New(1+1); if (unlikely(!__pyx_t_4)) __PYX_ERR(6, 424, __pyx_L1_error)
      __Pyx_GOTREF(__pyx_t_4);
      __Pyx_GIVEREF(__pyx_t_3); PyTuple_SET_ITEM(__pyx_t_4, 0, __pyx_t_3); __pyx_t_3 = NULL;
      __Pyx_INCREF(__pyx_v_cb_func);
      __Pyx_GIVEREF(__pyx_v_cb_func);
      PyTuple_SET_ITEM(__pyx_t_4, 0+1, __pyx_v_cb_func);
      __pyx_t_1 = __Pyx_PyObject_Call(__pyx_t_2, __pyx_t_4, NULL); if (unlikely(!__pyx_t_1)) __PYX_ERR(6, 424, __pyx_L1_error)
      __Pyx_GOTREF(__pyx_t_1);
      __Pyx_DECREF(__pyx_t_4); __pyx_t_4 = 0;
    }
  }
  __Pyx_DECREF(__pyx_t_2); __pyx_t_2 = 0;
  __Pyx_DECREF(__pyx_t_1); __pyx_t_1 = 0;

  /* "src/python/grpcio/grpc/_cython/_cygrpc/grpc_gevent.pyx.pxi":426
 *   set_async_callback_func(cb_func)
 * 
 *   gevent_resolver_vtable.resolve = socket_resolve             # <<<<<<<<<<<<<<
 *   gevent_resolver_vtable.resolve_async = socket_resolve_async
 * 
 */
  __pyx_v_4grpc_7_cython_6cygrpc_gevent_resolver_vtable.resolve = __pyx_f_4grpc_7_cython_6cygrpc_socket_resolve;

  /* "src/python/grpcio/grpc/_cython/_cygrpc/grpc_gevent.pyx.pxi":427
 * 
 *   gevent_resolver_vtable.resolve = socket_resolve
 *   gevent_resolver_vtable.resolve_async = socket_resolve_async             # <<<<<<<<<<<<<<
 * 
 *   gevent_socket_vtable.init = socket_init
 */
  __pyx_v_4grpc_7_cython_6cygrpc_gevent_resolver_vtable.resolve_async = __pyx_f_4grpc_7_cython_6cygrpc_socket_resolve_async;

  /* "src/python/grpcio/grpc/_cython/_cygrpc/grpc_gevent.pyx.pxi":429
 *   gevent_resolver_vtable.resolve_async = socket_resolve_async
 * 
 *   gevent_socket_vtable.init = socket_init             # <<<<<<<<<<<<<<
 *   gevent_socket_vtable.connect = socket_connect
 *   gevent_socket_vtable.destroy = socket_destroy
 */
  __pyx_v_4grpc_7_cython_6cygrpc_gevent_socket_vtable.init = __pyx_f_4grpc_7_cython_6cygrpc_socket_init;

  /* "src/python/grpcio/grpc/_cython/_cygrpc/grpc_gevent.pyx.pxi":430
 * 
 *   gevent_socket_vtable.init = socket_init
 *   gevent_socket_vtable.connect = socket_connect             # <<<<<<<<<<<<<<
 *   gevent_socket_vtable.destroy = socket_destroy
 *   gevent_socket_vtable.shutdown = socket_shutdown
 */
  __pyx_v_4grpc_7_cython_6cygrpc_gevent_socket_vtable.connect = __pyx_f_4grpc_7_cython_6cygrpc_socket_connect;

  /* "src/python/grpcio/grpc/_cython/_cygrpc/grpc_gevent.pyx.pxi":431
 *   gevent_socket_vtable.init = socket_init
 *   gevent_socket_vtable.connect = socket_connect
 *   gevent_socket_vtable.destroy = socket_destroy             # <<<<<<<<<<<<<<
 *   gevent_socket_vtable.shutdown = socket_shutdown
 *   gevent_socket_vtable.close = socket_close
 */
  __pyx_v_4grpc_7_cython_6cygrpc_gevent_socket_vtable.destroy = __pyx_f_4grpc_7_cython_6cygrpc_socket_destroy;

  /* "src/python/grpcio/grpc/_cython/_cygrpc/grpc_gevent.pyx.pxi":432
 *   gevent_socket_vtable.connect = socket_connect
 *   gevent_socket_vtable.destroy = socket_destroy
 *   gevent_socket_vtable.shutdown = socket_shutdown             # <<<<<<<<<<<<<<
 *   gevent_socket_vtable.close = socket_close
 *   gevent_socket_vtable.write = socket_write
 */
  __pyx_v_4grpc_7_cython_6cygrpc_gevent_socket_vtable.shutdown = __pyx_f_4grpc_7_cython_6cygrpc_socket_shutdown;

  /* "src/python/grpcio/grpc/_cython/_cygrpc/grpc_gevent.pyx.pxi":433
 *   gevent_socket_vtable.destroy = socket_destroy
 *   gevent_socket_vtable.shutdown = socket_shutdown
 *   gevent_socket_vtable.close = socket_close             # <<<<<<<<<<<<<<
 *   gevent_socket_vtable.write = socket_write
 *   gevent_socket_vtable.read = socket_read
 */
  __pyx_v_4grpc_7_cython_6cygrpc_gevent_socket_vtable.close = __pyx_f_4grpc_7_cython_6cygrpc_socket_close;

  /* "src/python/grpcio/grpc/_cython/_cygrpc/grpc_gevent.pyx.pxi":434
 *   gevent_socket_vtable.shutdown = socket_shutdown
 *   gevent_socket_vtable.close = socket_close
 *   gevent_socket_vtable.write = socket_write             # <<<<<<<<<<<<<<
 *   gevent_socket_vtable.read = socket_read
 *   gevent_socket_vtable.getpeername = socket_getpeername
 */
  __pyx_v_4grpc_7_cython_6cygrpc_gevent_socket_vtable.write = __pyx_f_4grpc_7_cython_6cygrpc_socket_write;

  /* "src/python/grpcio/grpc/_cython/_cygrpc/grpc_gevent.pyx.pxi":435
 *   gevent_socket_vtable.close = socket_close
 *   gevent_socket_vtable.write = socket_write
 *   gevent_socket_vtable.read = socket_read             # <<<<<<<<<<<<<<
 *   gevent_socket_vtable.getpeername = socket_getpeername
 *   gevent_socket_vtable.getsockname = socket_getsockname
 */
  __pyx_v_4grpc_7_cython_6cygrpc_gevent_socket_vtable.read = __pyx_f_4grpc_7_cython_6cygrpc_socket_read;

  /* "src/python/grpcio/grpc/_cython/_cygrpc/grpc_gevent.pyx.pxi":436
 *   gevent_socket_vtable.write = socket_write
 *   gevent_socket_vtable.read = socket_read
 *   gevent_socket_vtable.getpeername = socket_getpeername             # <<<<<<<<<<<<<<
 *   gevent_socket_vtable.getsockname = socket_getsockname
 *   gevent_socket_vtable.bind = socket_bind
 */
  __pyx_v_4grpc_7_cython_6cygrpc_gevent_socket_vtable.getpeername = __pyx_f_4grpc_7_cython_6cygrpc_socket_getpeername;

  /* "src/python/grpcio/grpc/_cython/_cygrpc/grpc_gevent.pyx.pxi":437
 *   gevent_socket_vtable.read = socket_read
 *   gevent_socket_vtable.getpeername = socket_getpeername
 *   gevent_socket_vtable.getsockname = socket_getsockname             # <<<<<<<<<<<<<<
 *   gevent_socket_vtable.bind = socket_bind
 *   gevent_socket_vtable.listen = socket_listen
 */
  __pyx_v_4grpc_7_cython_6cygrpc_gevent_socket_vtable.getsockname = __pyx_f_4grpc_7_cython_6cygrpc_socket_getsockname;

  /* "src/python/grpcio/grpc/_cython/_cygrpc/grpc_gevent.pyx.pxi":438
 *   gevent_socket_vtable.getpeername = socket_getpeername
 *   gevent_socket_vtable.getsockname = socket_getsockname
 *   gevent_socket_vtable.bind = socket_bind             # <<<<<<<<<<<<<<
 *   gevent_socket_vtable.listen = socket_listen
 *   gevent_socket_vtable.accept = socket_accept
 */
  __pyx_v_4grpc_7_cython_6cygrpc_gevent_socket_vtable.bind = __pyx_f_4grpc_7_cython_6cygrpc_socket_bind;

  /* "src/python/grpcio/grpc/_cython/_cygrpc/grpc_gevent.pyx.pxi":439
 *   gevent_socket_vtable.getsockname = socket_getsockname
 *   gevent_socket_vtable.bind = socket_bind
 *   gevent_socket_vtable.listen = socket_listen             # <<<<<<<<<<<<<<
 *   gevent_socket_vtable.accept = socket_accept
 * 
 */
  __pyx_v_4grpc_7_cython_6cygrpc_gevent_socket_vtable.listen = __pyx_f_4grpc_7_cython_6cygrpc_socket_listen;

  /* "src/python/grpcio/grpc/_cython/_cygrpc/grpc_gevent.pyx.pxi":440
 *   gevent_socket_vtable.bind = socket_bind
 *   gevent_socket_vtable.listen = socket_listen
 *   gevent_socket_vtable.accept = socket_accept             # <<<<<<<<<<<<<<
 * 
 *   gevent_timer_vtable.start = timer_start
 */
  __pyx_v_4grpc_7_cython_6cygrpc_gevent_socket_vtable.accept = __pyx_f_4grpc_7_cython_6cygrpc_socket_accept;

  /* "src/python/grpcio/grpc/_cython/_cygrpc/grpc_gevent.pyx.pxi":442
 *   gevent_socket_vtable.accept = socket_accept
 * 
 *   gevent_timer_vtable.start = timer_start             # <<<<<<<<<<<<<<
 *   gevent_timer_vtable.stop = timer_stop
 * 
 */
  __pyx_v_4grpc_7_cython_6cygrpc_gevent_timer_vtable.start = __pyx_f_4grpc_7_cython_6cygrpc_timer_start;

  /* "src/python/grpcio/grpc/_cython/_cygrpc/grpc_gevent.pyx.pxi":443
 * 
 *   gevent_timer_vtable.start = timer_start
 *   gevent_timer_vtable.stop = timer_stop             # <<<<<<<<<<<<<<
 * 
 *   gevent_pollset_vtable.init = init_loop
 */
  __pyx_v_4grpc_7_cython_6cygrpc_gevent_timer_vtable.stop = __pyx_f_4grpc_7_cython_6cygrpc_timer_stop;

  /* "src/python/grpcio/grpc/_cython/_cygrpc/grpc_gevent.pyx.pxi":445
 *   gevent_timer_vtable.stop = timer_stop
 * 
 *   gevent_pollset_vtable.init = init_loop             # <<<<<<<<<<<<<<
 *   gevent_pollset_vtable.poll = run_loop
 *   gevent_pollset_vtable.kick = kick_loop
 */
  __pyx_v_4grpc_7_cython_6cygrpc_gevent_pollset_vtable.init = __pyx_f_4grpc_7_cython_6cygrpc_init_loop;

  /* "src/python/grpcio/grpc/_cython/_cygrpc/grpc_gevent.pyx.pxi":446
 * 
 *   gevent_pollset_vtable.init = init_loop
 *   gevent_pollset_vtable.poll = run_loop             # <<<<<<<<<<<<<<
 *   gevent_pollset_vtable.kick = kick_loop
 *   gevent_pollset_vtable.shutdown = destroy_loop
 */
  __pyx_v_4grpc_7_cython_6cygrpc_gevent_pollset_vtable.poll = __pyx_f_4grpc_7_cython_6cygrpc_run_loop;

  /* "src/python/grpcio/grpc/_cython/_cygrpc/grpc_gevent.pyx.pxi":447
 *   gevent_pollset_vtable.init = init_loop
 *   gevent_pollset_vtable.poll = run_loop
 *   gevent_pollset_vtable.kick = kick_loop             # <<<<<<<<<<<<<<
 *   gevent_pollset_vtable.shutdown = destroy_loop
 * 
 */
  __pyx_v_4grpc_7_cython_6cygrpc_gevent_pollset_vtable.kick = __pyx_f_4grpc_7_cython_6cygrpc_kick_loop;

  /* "src/python/grpcio/grpc/_cython/_cygrpc/grpc_gevent.pyx.pxi":448
 *   gevent_pollset_vtable.poll = run_loop
 *   gevent_pollset_vtable.kick = kick_loop
 *   gevent_pollset_vtable.shutdown = destroy_loop             # <<<<<<<<<<<<<<
 * 
 *   grpc_custom_iomgr_init(&gevent_socket_vtable,
 */
  __pyx_v_4grpc_7_cython_6cygrpc_gevent_pollset_vtable.shutdown = __pyx_f_4grpc_7_cython_6cygrpc_destroy_loop;

  /* "src/python/grpcio/grpc/_cython/_cygrpc/grpc_gevent.pyx.pxi":450
 *   gevent_pollset_vtable.shutdown = destroy_loop
 * 
 *   grpc_custom_iomgr_init(&gevent_socket_vtable,             # <<<<<<<<<<<<<<
 *                          &gevent_resolver_vtable,
 *                          &gevent_timer_vtable,
 */
  grpc_custom_iomgr_init((&__pyx_v_4grpc_7_cython_6cygrpc_gevent_socket_vtable), (&__pyx_v_4grpc_7_cython_6cygrpc_gevent_resolver_vtable), (&__pyx_v_4grpc_7_cython_6cygrpc_gevent_timer_vtable), (&__pyx_v_4grpc_7_cython_6cygrpc_gevent_pollset_vtable));

  /* "src/python/grpcio/grpc/_cython/_cygrpc/grpc_gevent.pyx.pxi":401
 * cdef grpc_custom_poller_vtable gevent_pollset_vtable
 * 
 * def init_grpc_gevent():             # <<<<<<<<<<<<<<
 *   # Lazily import gevent
 *   global gevent_socket
 */

  /* function exit code */
  __pyx_r = Py_None; __Pyx_INCREF(Py_None);
  goto __pyx_L0;
  __pyx_L1_error:;
  __Pyx_XDECREF(__pyx_t_1);
  __Pyx_XDECREF(__pyx_t_2);
  __Pyx_XDECREF(__pyx_t_3);
  __Pyx_XDECREF(__pyx_t_4);
  __Pyx_AddTraceback("grpc._cython.cygrpc.init_grpc_gevent", __pyx_clineno, __pyx_lineno, __pyx_filename);
  __pyx_r = NULL;
  __pyx_L0:;
  __Pyx_XDECREF(__pyx_v_gevent);
  __Pyx_XDECREF(__pyx_v_cb_func);
  __Pyx_XGIVEREF(__pyx_r);
  __Pyx_RefNannyFinishContext();
  return __pyx_r;
}

/* "src/python/grpcio/grpc/_cython/_cygrpc/fork_posix.pyx.pxi":41
 * _fork_handler_failed = False
 * 
 * cdef void __prefork() nogil:             # <<<<<<<<<<<<<<
 *     with gil:
 *         global _fork_handler_failed
 */

static void __pyx_f_4grpc_7_cython_6cygrpc___prefork(void) {
  __Pyx_RefNannyDeclarations
  PyObject *__pyx_t_1 = NULL;
  PyObject *__pyx_t_2 = NULL;
  PyObject *__pyx_t_3 = NULL;
  PyObject *__pyx_t_4 = NULL;
  PyObject *__pyx_t_5 = NULL;
  PyObject *__pyx_t_6 = NULL;
  PyObject *__pyx_t_7 = NULL;
  PyObject *__pyx_t_8 = NULL;
  PyObject *__pyx_t_9 = NULL;
  int __pyx_t_10;
  int __pyx_t_11;
  PyObject *__pyx_t_12 = NULL;
  #ifdef WITH_THREAD
  PyGILState_STATE __pyx_gilstate_save;
  #endif
  __Pyx_RefNannySetupContext("__prefork", 1);

  /* "src/python/grpcio/grpc/_cython/_cygrpc/fork_posix.pyx.pxi":42
 * 
 * cdef void __prefork() nogil:
 *     with gil:             # <<<<<<<<<<<<<<
 *         global _fork_handler_failed
 *         _fork_handler_failed = False
 */
  /*try:*/ {
    {
        #ifdef WITH_THREAD
        PyGILState_STATE __pyx_gilstate_save = __Pyx_PyGILState_Ensure();
        #endif
        /*try:*/ {

          /* "src/python/grpcio/grpc/_cython/_cygrpc/fork_posix.pyx.pxi":44
 *     with gil:
 *         global _fork_handler_failed
 *         _fork_handler_failed = False             # <<<<<<<<<<<<<<
 *         with _fork_state.fork_in_progress_condition:
 *             _fork_state.fork_in_progress = True
 */
          if (PyDict_SetItem(__pyx_d, __pyx_n_s_fork_handler_failed, Py_False) < 0) __PYX_ERR(0, 44, __pyx_L7_error)

          /* "src/python/grpcio/grpc/_cython/_cygrpc/fork_posix.pyx.pxi":45
 *         global _fork_handler_failed
 *         _fork_handler_failed = False
 *         with _fork_state.fork_in_progress_condition:             # <<<<<<<<<<<<<<
 *             _fork_state.fork_in_progress = True
 *         if not _fork_state.active_thread_count.await_zero_threads(
 */
          /*with:*/ {
            __pyx_t_1 = __Pyx_GetModuleGlobalName(__pyx_n_s_fork_state); if (unlikely(!__pyx_t_1)) __PYX_ERR(0, 45, __pyx_L7_error)
            __Pyx_GOTREF(__pyx_t_1);
            __pyx_t_2 = __Pyx_PyObject_GetAttrStr(__pyx_t_1, __pyx_n_s_fork_in_progress_condition); if (unlikely(!__pyx_t_2)) __PYX_ERR(0, 45, __pyx_L7_error)
            __Pyx_GOTREF(__pyx_t_2);
            __Pyx_DECREF(__pyx_t_1); __pyx_t_1 = 0;
            __pyx_t_3 = __Pyx_PyObject_LookupSpecial(__pyx_t_2, __pyx_n_s_exit); if (unlikely(!__pyx_t_3)) __PYX_ERR(0, 45, __pyx_L7_error)
            __Pyx_GOTREF(__pyx_t_3);
            __pyx_t_4 = __Pyx_PyObject_LookupSpecial(__pyx_t_2, __pyx_n_s_enter); if (unlikely(!__pyx_t_4)) __PYX_ERR(0, 45, __pyx_L9_error)
            __Pyx_GOTREF(__pyx_t_4);
            __pyx_t_5 = NULL;
            if (CYTHON_UNPACK_METHODS && likely(PyMethod_Check(__pyx_t_4))) {
              __pyx_t_5 = PyMethod_GET_SELF(__pyx_t_4);
              if (likely(__pyx_t_5)) {
                PyObject* function = PyMethod_GET_FUNCTION(__pyx_t_4);
                __Pyx_INCREF(__pyx_t_5);
                __Pyx_INCREF(function);
                __Pyx_DECREF_SET(__pyx_t_4, function);
              }
            }
            if (__pyx_t_5) {
              __pyx_t_1 = __Pyx_PyObject_CallOneArg(__pyx_t_4, __pyx_t_5); if (unlikely(!__pyx_t_1)) __PYX_ERR(0, 45, __pyx_L9_error)
              __Pyx_DECREF(__pyx_t_5); __pyx_t_5 = 0;
            } else {
              __pyx_t_1 = __Pyx_PyObject_CallNoArg(__pyx_t_4); if (unlikely(!__pyx_t_1)) __PYX_ERR(0, 45, __pyx_L9_error)
            }
            __Pyx_GOTREF(__pyx_t_1);
            __Pyx_DECREF(__pyx_t_4); __pyx_t_4 = 0;
            __Pyx_DECREF(__pyx_t_1); __pyx_t_1 = 0;
            __Pyx_DECREF(__pyx_t_2); __pyx_t_2 = 0;
            /*try:*/ {
              {
                __Pyx_PyThreadState_declare
                __Pyx_PyThreadState_assign
                __Pyx_ExceptionSave(&__pyx_t_6, &__pyx_t_7, &__pyx_t_8);
                __Pyx_XGOTREF(__pyx_t_6);
                __Pyx_XGOTREF(__pyx_t_7);
                __Pyx_XGOTREF(__pyx_t_8);
                /*try:*/ {

                  /* "src/python/grpcio/grpc/_cython/_cygrpc/fork_posix.pyx.pxi":46
 *         _fork_handler_failed = False
 *         with _fork_state.fork_in_progress_condition:
 *             _fork_state.fork_in_progress = True             # <<<<<<<<<<<<<<
 *         if not _fork_state.active_thread_count.await_zero_threads(
 *                 _AWAIT_THREADS_TIMEOUT_SECONDS):
 */
                  __pyx_t_2 = __Pyx_GetModuleGlobalName(__pyx_n_s_fork_state); if (unlikely(!__pyx_t_2)) __PYX_ERR(0, 46, __pyx_L13_error)
                  __Pyx_GOTREF(__pyx_t_2);
                  if (__Pyx_PyObject_SetAttrStr(__pyx_t_2, __pyx_n_s_fork_in_progress, Py_True) < 0) __PYX_ERR(0, 46, __pyx_L13_error)
                  __Pyx_DECREF(__pyx_t_2); __pyx_t_2 = 0;

                  /* "src/python/grpcio/grpc/_cython/_cygrpc/fork_posix.pyx.pxi":45
 *         global _fork_handler_failed
 *         _fork_handler_failed = False
 *         with _fork_state.fork_in_progress_condition:             # <<<<<<<<<<<<<<
 *             _fork_state.fork_in_progress = True
 *         if not _fork_state.active_thread_count.await_zero_threads(
 */
                }
                __Pyx_XDECREF(__pyx_t_6); __pyx_t_6 = 0;
                __Pyx_XDECREF(__pyx_t_7); __pyx_t_7 = 0;
                __Pyx_XDECREF(__pyx_t_8); __pyx_t_8 = 0;
                goto __pyx_L18_try_end;
                __pyx_L13_error:;
                __Pyx_XDECREF(__pyx_t_5); __pyx_t_5 = 0;
                __Pyx_XDECREF(__pyx_t_4); __pyx_t_4 = 0;
                __Pyx_XDECREF(__pyx_t_1); __pyx_t_1 = 0;
                __Pyx_XDECREF(__pyx_t_2); __pyx_t_2 = 0;
                /*except:*/ {
                  __Pyx_AddTraceback("grpc._cython.cygrpc.__prefork", __pyx_clineno, __pyx_lineno, __pyx_filename);
                  if (__Pyx_GetException(&__pyx_t_2, &__pyx_t_1, &__pyx_t_4) < 0) __PYX_ERR(0, 45, __pyx_L15_except_error)
                  __Pyx_GOTREF(__pyx_t_2);
                  __Pyx_GOTREF(__pyx_t_1);
                  __Pyx_GOTREF(__pyx_t_4);
                  __pyx_t_5 = PyTuple_Pack(3, __pyx_t_2, __pyx_t_1, __pyx_t_4); if (unlikely(!__pyx_t_5)) __PYX_ERR(0, 45, __pyx_L15_except_error)
                  __Pyx_GOTREF(__pyx_t_5);
                  __pyx_t_9 = __Pyx_PyObject_Call(__pyx_t_3, __pyx_t_5, NULL);
                  __Pyx_DECREF(__pyx_t_3); __pyx_t_3 = 0;
                  __Pyx_DECREF(__pyx_t_5); __pyx_t_5 = 0;
                  if (unlikely(!__pyx_t_9)) __PYX_ERR(0, 45, __pyx_L15_except_error)
                  __Pyx_GOTREF(__pyx_t_9);
                  __pyx_t_10 = __Pyx_PyObject_IsTrue(__pyx_t_9);
                  __Pyx_DECREF(__pyx_t_9); __pyx_t_9 = 0;
                  if (__pyx_t_10 < 0) __PYX_ERR(0, 45, __pyx_L15_except_error)
                  __pyx_t_11 = ((!(__pyx_t_10 != 0)) != 0);
                  if (__pyx_t_11) {
                    __Pyx_GIVEREF(__pyx_t_2);
                    __Pyx_GIVEREF(__pyx_t_1);
                    __Pyx_XGIVEREF(__pyx_t_4);
                    __Pyx_ErrRestoreWithState(__pyx_t_2, __pyx_t_1, __pyx_t_4);
                    __pyx_t_2 = 0; __pyx_t_1 = 0; __pyx_t_4 = 0; 
                    __PYX_ERR(0, 45, __pyx_L15_except_error)
                  }
                  __Pyx_DECREF(__pyx_t_2); __pyx_t_2 = 0;
                  __Pyx_DECREF(__pyx_t_1); __pyx_t_1 = 0;
                  __Pyx_DECREF(__pyx_t_4); __pyx_t_4 = 0;
                  goto __pyx_L14_exception_handled;
                }
                __pyx_L15_except_error:;
                __Pyx_XGIVEREF(__pyx_t_6);
                __Pyx_XGIVEREF(__pyx_t_7);
                __Pyx_XGIVEREF(__pyx_t_8);
                __Pyx_ExceptionReset(__pyx_t_6, __pyx_t_7, __pyx_t_8);
                goto __pyx_L7_error;
                __pyx_L14_exception_handled:;
                __Pyx_XGIVEREF(__pyx_t_6);
                __Pyx_XGIVEREF(__pyx_t_7);
                __Pyx_XGIVEREF(__pyx_t_8);
                __Pyx_ExceptionReset(__pyx_t_6, __pyx_t_7, __pyx_t_8);
                __pyx_L18_try_end:;
              }
            }
            /*finally:*/ {
              /*normal exit:*/{
                if (__pyx_t_3) {
                  __pyx_t_8 = __Pyx_PyObject_Call(__pyx_t_3, __pyx_tuple__148, NULL);
                  __Pyx_DECREF(__pyx_t_3); __pyx_t_3 = 0;
                  if (unlikely(!__pyx_t_8)) __PYX_ERR(0, 45, __pyx_L7_error)
                  __Pyx_GOTREF(__pyx_t_8);
                  __Pyx_DECREF(__pyx_t_8); __pyx_t_8 = 0;
                }
                goto __pyx_L12;
              }
              __pyx_L12:;
            }
            goto __pyx_L22;
            __pyx_L9_error:;
            __Pyx_DECREF(__pyx_t_3); __pyx_t_3 = 0;
            goto __pyx_L7_error;
            __pyx_L22:;
          }

          /* "src/python/grpcio/grpc/_cython/_cygrpc/fork_posix.pyx.pxi":47
 *         with _fork_state.fork_in_progress_condition:
 *             _fork_state.fork_in_progress = True
 *         if not _fork_state.active_thread_count.await_zero_threads(             # <<<<<<<<<<<<<<
 *                 _AWAIT_THREADS_TIMEOUT_SECONDS):
 *             _LOGGER.error(
 */
          __pyx_t_1 = __Pyx_GetModuleGlobalName(__pyx_n_s_fork_state); if (unlikely(!__pyx_t_1)) __PYX_ERR(0, 47, __pyx_L7_error)
          __Pyx_GOTREF(__pyx_t_1);
          __pyx_t_2 = __Pyx_PyObject_GetAttrStr(__pyx_t_1, __pyx_n_s_active_thread_count); if (unlikely(!__pyx_t_2)) __PYX_ERR(0, 47, __pyx_L7_error)
          __Pyx_GOTREF(__pyx_t_2);
          __Pyx_DECREF(__pyx_t_1); __pyx_t_1 = 0;
          __pyx_t_1 = __Pyx_PyObject_GetAttrStr(__pyx_t_2, __pyx_n_s_await_zero_threads); if (unlikely(!__pyx_t_1)) __PYX_ERR(0, 47, __pyx_L7_error)
          __Pyx_GOTREF(__pyx_t_1);
          __Pyx_DECREF(__pyx_t_2); __pyx_t_2 = 0;

          /* "src/python/grpcio/grpc/_cython/_cygrpc/fork_posix.pyx.pxi":48
 *             _fork_state.fork_in_progress = True
 *         if not _fork_state.active_thread_count.await_zero_threads(
 *                 _AWAIT_THREADS_TIMEOUT_SECONDS):             # <<<<<<<<<<<<<<
 *             _LOGGER.error(
 *                 'Failed to shutdown gRPC Python threads prior to fork. '
 */
          __pyx_t_2 = __Pyx_GetModuleGlobalName(__pyx_n_s_AWAIT_THREADS_TIMEOUT_SECONDS); if (unlikely(!__pyx_t_2)) __PYX_ERR(0, 48, __pyx_L7_error)
          __Pyx_GOTREF(__pyx_t_2);
          __pyx_t_5 = NULL;
          if (CYTHON_UNPACK_METHODS && likely(PyMethod_Check(__pyx_t_1))) {
            __pyx_t_5 = PyMethod_GET_SELF(__pyx_t_1);
            if (likely(__pyx_t_5)) {
              PyObject* function = PyMethod_GET_FUNCTION(__pyx_t_1);
              __Pyx_INCREF(__pyx_t_5);
              __Pyx_INCREF(function);
              __Pyx_DECREF_SET(__pyx_t_1, function);
            }
          }
          if (!__pyx_t_5) {
            __pyx_t_4 = __Pyx_PyObject_CallOneArg(__pyx_t_1, __pyx_t_2); if (unlikely(!__pyx_t_4)) __PYX_ERR(0, 47, __pyx_L7_error)
            __Pyx_DECREF(__pyx_t_2); __pyx_t_2 = 0;
            __Pyx_GOTREF(__pyx_t_4);
          } else {
            #if CYTHON_FAST_PYCALL
            if (PyFunction_Check(__pyx_t_1)) {
              PyObject *__pyx_temp[2] = {__pyx_t_5, __pyx_t_2};
              __pyx_t_4 = __Pyx_PyFunction_FastCall(__pyx_t_1, __pyx_temp+1-1, 1+1); if (unlikely(!__pyx_t_4)) __PYX_ERR(0, 47, __pyx_L7_error)
              __Pyx_XDECREF(__pyx_t_5); __pyx_t_5 = 0;
              __Pyx_GOTREF(__pyx_t_4);
              __Pyx_DECREF(__pyx_t_2); __pyx_t_2 = 0;
            } else
            #endif
            #if CYTHON_FAST_PYCCALL
            if (__Pyx_PyFastCFunction_Check(__pyx_t_1)) {
              PyObject *__pyx_temp[2] = {__pyx_t_5, __pyx_t_2};
              __pyx_t_4 = __Pyx_PyCFunction_FastCall(__pyx_t_1, __pyx_temp+1-1, 1+1); if (unlikely(!__pyx_t_4)) __PYX_ERR(0, 47, __pyx_L7_error)
              __Pyx_XDECREF(__pyx_t_5); __pyx_t_5 = 0;
              __Pyx_GOTREF(__pyx_t_4);
              __Pyx_DECREF(__pyx_t_2); __pyx_t_2 = 0;
            } else
            #endif
            {
              __pyx_t_12 = PyTuple_New(1+1); if (unlikely(!__pyx_t_12)) __PYX_ERR(0, 47, __pyx_L7_error)
              __Pyx_GOTREF(__pyx_t_12);
              __Pyx_GIVEREF(__pyx_t_5); PyTuple_SET_ITEM(__pyx_t_12, 0, __pyx_t_5); __pyx_t_5 = NULL;
              __Pyx_GIVEREF(__pyx_t_2);
              PyTuple_SET_ITEM(__pyx_t_12, 0+1, __pyx_t_2);
              __pyx_t_2 = 0;
              __pyx_t_4 = __Pyx_PyObject_Call(__pyx_t_1, __pyx_t_12, NULL); if (unlikely(!__pyx_t_4)) __PYX_ERR(0, 47, __pyx_L7_error)
              __Pyx_GOTREF(__pyx_t_4);
              __Pyx_DECREF(__pyx_t_12); __pyx_t_12 = 0;
            }
          }
          __Pyx_DECREF(__pyx_t_1); __pyx_t_1 = 0;

          /* "src/python/grpcio/grpc/_cython/_cygrpc/fork_posix.pyx.pxi":47
 *         with _fork_state.fork_in_progress_condition:
 *             _fork_state.fork_in_progress = True
 *         if not _fork_state.active_thread_count.await_zero_threads(             # <<<<<<<<<<<<<<
 *                 _AWAIT_THREADS_TIMEOUT_SECONDS):
 *             _LOGGER.error(
 */
          __pyx_t_11 = __Pyx_PyObject_IsTrue(__pyx_t_4); if (unlikely(__pyx_t_11 < 0)) __PYX_ERR(0, 47, __pyx_L7_error)
          __Pyx_DECREF(__pyx_t_4); __pyx_t_4 = 0;
          __pyx_t_10 = ((!__pyx_t_11) != 0);
          if (__pyx_t_10) {

            /* "src/python/grpcio/grpc/_cython/_cygrpc/fork_posix.pyx.pxi":49
 *         if not _fork_state.active_thread_count.await_zero_threads(
 *                 _AWAIT_THREADS_TIMEOUT_SECONDS):
 *             _LOGGER.error(             # <<<<<<<<<<<<<<
 *                 'Failed to shutdown gRPC Python threads prior to fork. '
 *                 'Behavior after fork will be undefined.')
 */
            __pyx_t_4 = __Pyx_GetModuleGlobalName(__pyx_n_s_LOGGER); if (unlikely(!__pyx_t_4)) __PYX_ERR(0, 49, __pyx_L7_error)
            __Pyx_GOTREF(__pyx_t_4);
            __pyx_t_1 = __Pyx_PyObject_GetAttrStr(__pyx_t_4, __pyx_n_s_error); if (unlikely(!__pyx_t_1)) __PYX_ERR(0, 49, __pyx_L7_error)
            __Pyx_GOTREF(__pyx_t_1);
            __Pyx_DECREF(__pyx_t_4); __pyx_t_4 = 0;
            __pyx_t_4 = __Pyx_PyObject_Call(__pyx_t_1, __pyx_tuple__149, NULL); if (unlikely(!__pyx_t_4)) __PYX_ERR(0, 49, __pyx_L7_error)
            __Pyx_GOTREF(__pyx_t_4);
            __Pyx_DECREF(__pyx_t_1); __pyx_t_1 = 0;
            __Pyx_DECREF(__pyx_t_4); __pyx_t_4 = 0;

            /* "src/python/grpcio/grpc/_cython/_cygrpc/fork_posix.pyx.pxi":52
 *                 'Failed to shutdown gRPC Python threads prior to fork. '
 *                 'Behavior after fork will be undefined.')
 *             _fork_handler_failed = True             # <<<<<<<<<<<<<<
 * 
 * 
 */
            if (PyDict_SetItem(__pyx_d, __pyx_n_s_fork_handler_failed, Py_True) < 0) __PYX_ERR(0, 52, __pyx_L7_error)

            /* "src/python/grpcio/grpc/_cython/_cygrpc/fork_posix.pyx.pxi":47
 *         with _fork_state.fork_in_progress_condition:
 *             _fork_state.fork_in_progress = True
 *         if not _fork_state.active_thread_count.await_zero_threads(             # <<<<<<<<<<<<<<
 *                 _AWAIT_THREADS_TIMEOUT_SECONDS):
 *             _LOGGER.error(
 */
          }
        }

        /* "src/python/grpcio/grpc/_cython/_cygrpc/fork_posix.pyx.pxi":42
 * 
 * cdef void __prefork() nogil:
 *     with gil:             # <<<<<<<<<<<<<<
 *         global _fork_handler_failed
 *         _fork_handler_failed = False
 */
        /*finally:*/ {
          /*normal exit:*/{
            #ifdef WITH_THREAD
            __Pyx_PyGILState_Release(__pyx_gilstate_save);
            #endif
            goto __pyx_L8;
          }
          __pyx_L7_error: {
            #ifdef WITH_THREAD
            __Pyx_PyGILState_Release(__pyx_gilstate_save);
            #endif
            goto __pyx_L4_error;
          }
          __pyx_L8:;
        }
    }
  }
  /*finally:*/ {
    /*normal exit:*/{
      #ifdef WITH_THREAD
      __pyx_gilstate_save = __Pyx_PyGILState_Ensure();
      #endif
      goto __pyx_L5;
    }
    __pyx_L4_error: {
      #ifdef WITH_THREAD
      __pyx_gilstate_save = __Pyx_PyGILState_Ensure();
      #endif
      goto __pyx_L1_error;
    }
    __pyx_L5:;
  }

  /* "src/python/grpcio/grpc/_cython/_cygrpc/fork_posix.pyx.pxi":41
 * _fork_handler_failed = False
 * 
 * cdef void __prefork() nogil:             # <<<<<<<<<<<<<<
 *     with gil:
 *         global _fork_handler_failed
 */

  /* function exit code */
  goto __pyx_L0;
  __pyx_L1_error:;
  __Pyx_XDECREF(__pyx_t_1);
  __Pyx_XDECREF(__pyx_t_2);
  __Pyx_XDECREF(__pyx_t_4);
  __Pyx_XDECREF(__pyx_t_5);
  __Pyx_XDECREF(__pyx_t_12);
  __Pyx_WriteUnraisable("grpc._cython.cygrpc.__prefork", __pyx_clineno, __pyx_lineno, __pyx_filename, 1, 1);
  __pyx_L0:;
  #ifdef WITH_THREAD
  __Pyx_PyGILState_Release(__pyx_gilstate_save);
  #endif
}

/* "src/python/grpcio/grpc/_cython/_cygrpc/fork_posix.pyx.pxi":55
 * 
 * 
 * cdef void __postfork_parent() nogil:             # <<<<<<<<<<<<<<
 *     with gil:
 *         with _fork_state.fork_in_progress_condition:
 */

static void __pyx_f_4grpc_7_cython_6cygrpc___postfork_parent(void) {
  __Pyx_RefNannyDeclarations
  PyObject *__pyx_t_1 = NULL;
  PyObject *__pyx_t_2 = NULL;
  PyObject *__pyx_t_3 = NULL;
  PyObject *__pyx_t_4 = NULL;
  PyObject *__pyx_t_5 = NULL;
  PyObject *__pyx_t_6 = NULL;
  PyObject *__pyx_t_7 = NULL;
  PyObject *__pyx_t_8 = NULL;
  PyObject *__pyx_t_9 = NULL;
  int __pyx_t_10;
  int __pyx_t_11;
  #ifdef WITH_THREAD
  PyGILState_STATE __pyx_gilstate_save;
  #endif
  __Pyx_RefNannySetupContext("__postfork_parent", 1);

  /* "src/python/grpcio/grpc/_cython/_cygrpc/fork_posix.pyx.pxi":56
 * 
 * cdef void __postfork_parent() nogil:
 *     with gil:             # <<<<<<<<<<<<<<
 *         with _fork_state.fork_in_progress_condition:
 *             _fork_state.fork_in_progress = False
 */
  /*try:*/ {
    {
        #ifdef WITH_THREAD
        PyGILState_STATE __pyx_gilstate_save = __Pyx_PyGILState_Ensure();
        #endif
        /*try:*/ {

          /* "src/python/grpcio/grpc/_cython/_cygrpc/fork_posix.pyx.pxi":57
 * cdef void __postfork_parent() nogil:
 *     with gil:
 *         with _fork_state.fork_in_progress_condition:             # <<<<<<<<<<<<<<
 *             _fork_state.fork_in_progress = False
 *             _fork_state.fork_in_progress_condition.notify_all()
 */
          /*with:*/ {
            __pyx_t_1 = __Pyx_GetModuleGlobalName(__pyx_n_s_fork_state); if (unlikely(!__pyx_t_1)) __PYX_ERR(0, 57, __pyx_L7_error)
            __Pyx_GOTREF(__pyx_t_1);
            __pyx_t_2 = __Pyx_PyObject_GetAttrStr(__pyx_t_1, __pyx_n_s_fork_in_progress_condition); if (unlikely(!__pyx_t_2)) __PYX_ERR(0, 57, __pyx_L7_error)
            __Pyx_GOTREF(__pyx_t_2);
            __Pyx_DECREF(__pyx_t_1); __pyx_t_1 = 0;
            __pyx_t_3 = __Pyx_PyObject_LookupSpecial(__pyx_t_2, __pyx_n_s_exit); if (unlikely(!__pyx_t_3)) __PYX_ERR(0, 57, __pyx_L7_error)
            __Pyx_GOTREF(__pyx_t_3);
            __pyx_t_4 = __Pyx_PyObject_LookupSpecial(__pyx_t_2, __pyx_n_s_enter); if (unlikely(!__pyx_t_4)) __PYX_ERR(0, 57, __pyx_L9_error)
            __Pyx_GOTREF(__pyx_t_4);
            __pyx_t_5 = NULL;
            if (CYTHON_UNPACK_METHODS && likely(PyMethod_Check(__pyx_t_4))) {
              __pyx_t_5 = PyMethod_GET_SELF(__pyx_t_4);
              if (likely(__pyx_t_5)) {
                PyObject* function = PyMethod_GET_FUNCTION(__pyx_t_4);
                __Pyx_INCREF(__pyx_t_5);
                __Pyx_INCREF(function);
                __Pyx_DECREF_SET(__pyx_t_4, function);
              }
            }
            if (__pyx_t_5) {
              __pyx_t_1 = __Pyx_PyObject_CallOneArg(__pyx_t_4, __pyx_t_5); if (unlikely(!__pyx_t_1)) __PYX_ERR(0, 57, __pyx_L9_error)
              __Pyx_DECREF(__pyx_t_5); __pyx_t_5 = 0;
            } else {
              __pyx_t_1 = __Pyx_PyObject_CallNoArg(__pyx_t_4); if (unlikely(!__pyx_t_1)) __PYX_ERR(0, 57, __pyx_L9_error)
            }
            __Pyx_GOTREF(__pyx_t_1);
            __Pyx_DECREF(__pyx_t_4); __pyx_t_4 = 0;
            __Pyx_DECREF(__pyx_t_1); __pyx_t_1 = 0;
            __Pyx_DECREF(__pyx_t_2); __pyx_t_2 = 0;
            /*try:*/ {
              {
                __Pyx_PyThreadState_declare
                __Pyx_PyThreadState_assign
                __Pyx_ExceptionSave(&__pyx_t_6, &__pyx_t_7, &__pyx_t_8);
                __Pyx_XGOTREF(__pyx_t_6);
                __Pyx_XGOTREF(__pyx_t_7);
                __Pyx_XGOTREF(__pyx_t_8);
                /*try:*/ {

                  /* "src/python/grpcio/grpc/_cython/_cygrpc/fork_posix.pyx.pxi":58
 *     with gil:
 *         with _fork_state.fork_in_progress_condition:
 *             _fork_state.fork_in_progress = False             # <<<<<<<<<<<<<<
 *             _fork_state.fork_in_progress_condition.notify_all()
 * 
 */
                  __pyx_t_2 = __Pyx_GetModuleGlobalName(__pyx_n_s_fork_state); if (unlikely(!__pyx_t_2)) __PYX_ERR(0, 58, __pyx_L13_error)
                  __Pyx_GOTREF(__pyx_t_2);
                  if (__Pyx_PyObject_SetAttrStr(__pyx_t_2, __pyx_n_s_fork_in_progress, Py_False) < 0) __PYX_ERR(0, 58, __pyx_L13_error)
                  __Pyx_DECREF(__pyx_t_2); __pyx_t_2 = 0;

                  /* "src/python/grpcio/grpc/_cython/_cygrpc/fork_posix.pyx.pxi":59
 *         with _fork_state.fork_in_progress_condition:
 *             _fork_state.fork_in_progress = False
 *             _fork_state.fork_in_progress_condition.notify_all()             # <<<<<<<<<<<<<<
 * 
 * 
 */
                  __pyx_t_1 = __Pyx_GetModuleGlobalName(__pyx_n_s_fork_state); if (unlikely(!__pyx_t_1)) __PYX_ERR(0, 59, __pyx_L13_error)
                  __Pyx_GOTREF(__pyx_t_1);
                  __pyx_t_4 = __Pyx_PyObject_GetAttrStr(__pyx_t_1, __pyx_n_s_fork_in_progress_condition); if (unlikely(!__pyx_t_4)) __PYX_ERR(0, 59, __pyx_L13_error)
                  __Pyx_GOTREF(__pyx_t_4);
                  __Pyx_DECREF(__pyx_t_1); __pyx_t_1 = 0;
                  __pyx_t_1 = __Pyx_PyObject_GetAttrStr(__pyx_t_4, __pyx_n_s_notify_all); if (unlikely(!__pyx_t_1)) __PYX_ERR(0, 59, __pyx_L13_error)
                  __Pyx_GOTREF(__pyx_t_1);
                  __Pyx_DECREF(__pyx_t_4); __pyx_t_4 = 0;
                  __pyx_t_4 = NULL;
                  if (CYTHON_UNPACK_METHODS && likely(PyMethod_Check(__pyx_t_1))) {
                    __pyx_t_4 = PyMethod_GET_SELF(__pyx_t_1);
                    if (likely(__pyx_t_4)) {
                      PyObject* function = PyMethod_GET_FUNCTION(__pyx_t_1);
                      __Pyx_INCREF(__pyx_t_4);
                      __Pyx_INCREF(function);
                      __Pyx_DECREF_SET(__pyx_t_1, function);
                    }
                  }
                  if (__pyx_t_4) {
                    __pyx_t_2 = __Pyx_PyObject_CallOneArg(__pyx_t_1, __pyx_t_4); if (unlikely(!__pyx_t_2)) __PYX_ERR(0, 59, __pyx_L13_error)
                    __Pyx_DECREF(__pyx_t_4); __pyx_t_4 = 0;
                  } else {
                    __pyx_t_2 = __Pyx_PyObject_CallNoArg(__pyx_t_1); if (unlikely(!__pyx_t_2)) __PYX_ERR(0, 59, __pyx_L13_error)
                  }
                  __Pyx_GOTREF(__pyx_t_2);
                  __Pyx_DECREF(__pyx_t_1); __pyx_t_1 = 0;
                  __Pyx_DECREF(__pyx_t_2); __pyx_t_2 = 0;

                  /* "src/python/grpcio/grpc/_cython/_cygrpc/fork_posix.pyx.pxi":57
 * cdef void __postfork_parent() nogil:
 *     with gil:
 *         with _fork_state.fork_in_progress_condition:             # <<<<<<<<<<<<<<
 *             _fork_state.fork_in_progress = False
 *             _fork_state.fork_in_progress_condition.notify_all()
 */
                }
                __Pyx_XDECREF(__pyx_t_6); __pyx_t_6 = 0;
                __Pyx_XDECREF(__pyx_t_7); __pyx_t_7 = 0;
                __Pyx_XDECREF(__pyx_t_8); __pyx_t_8 = 0;
                goto __pyx_L18_try_end;
                __pyx_L13_error:;
                __Pyx_XDECREF(__pyx_t_5); __pyx_t_5 = 0;
                __Pyx_XDECREF(__pyx_t_4); __pyx_t_4 = 0;
                __Pyx_XDECREF(__pyx_t_1); __pyx_t_1 = 0;
                __Pyx_XDECREF(__pyx_t_2); __pyx_t_2 = 0;
                /*except:*/ {
                  __Pyx_AddTraceback("grpc._cython.cygrpc.__postfork_parent", __pyx_clineno, __pyx_lineno, __pyx_filename);
                  if (__Pyx_GetException(&__pyx_t_2, &__pyx_t_1, &__pyx_t_4) < 0) __PYX_ERR(0, 57, __pyx_L15_except_error)
                  __Pyx_GOTREF(__pyx_t_2);
                  __Pyx_GOTREF(__pyx_t_1);
                  __Pyx_GOTREF(__pyx_t_4);
                  __pyx_t_5 = PyTuple_Pack(3, __pyx_t_2, __pyx_t_1, __pyx_t_4); if (unlikely(!__pyx_t_5)) __PYX_ERR(0, 57, __pyx_L15_except_error)
                  __Pyx_GOTREF(__pyx_t_5);
                  __pyx_t_9 = __Pyx_PyObject_Call(__pyx_t_3, __pyx_t_5, NULL);
                  __Pyx_DECREF(__pyx_t_3); __pyx_t_3 = 0;
                  __Pyx_DECREF(__pyx_t_5); __pyx_t_5 = 0;
                  if (unlikely(!__pyx_t_9)) __PYX_ERR(0, 57, __pyx_L15_except_error)
                  __Pyx_GOTREF(__pyx_t_9);
                  __pyx_t_10 = __Pyx_PyObject_IsTrue(__pyx_t_9);
                  __Pyx_DECREF(__pyx_t_9); __pyx_t_9 = 0;
                  if (__pyx_t_10 < 0) __PYX_ERR(0, 57, __pyx_L15_except_error)
                  __pyx_t_11 = ((!(__pyx_t_10 != 0)) != 0);
                  if (__pyx_t_11) {
                    __Pyx_GIVEREF(__pyx_t_2);
                    __Pyx_GIVEREF(__pyx_t_1);
                    __Pyx_XGIVEREF(__pyx_t_4);
                    __Pyx_ErrRestoreWithState(__pyx_t_2, __pyx_t_1, __pyx_t_4);
                    __pyx_t_2 = 0; __pyx_t_1 = 0; __pyx_t_4 = 0; 
                    __PYX_ERR(0, 57, __pyx_L15_except_error)
                  }
                  __Pyx_DECREF(__pyx_t_2); __pyx_t_2 = 0;
                  __Pyx_DECREF(__pyx_t_1); __pyx_t_1 = 0;
                  __Pyx_DECREF(__pyx_t_4); __pyx_t_4 = 0;
                  goto __pyx_L14_exception_handled;
                }
                __pyx_L15_except_error:;
                __Pyx_XGIVEREF(__pyx_t_6);
                __Pyx_XGIVEREF(__pyx_t_7);
                __Pyx_XGIVEREF(__pyx_t_8);
                __Pyx_ExceptionReset(__pyx_t_6, __pyx_t_7, __pyx_t_8);
                goto __pyx_L7_error;
                __pyx_L14_exception_handled:;
                __Pyx_XGIVEREF(__pyx_t_6);
                __Pyx_XGIVEREF(__pyx_t_7);
                __Pyx_XGIVEREF(__pyx_t_8);
                __Pyx_ExceptionReset(__pyx_t_6, __pyx_t_7, __pyx_t_8);
                __pyx_L18_try_end:;
              }
            }
            /*finally:*/ {
              /*normal exit:*/{
                if (__pyx_t_3) {
                  __pyx_t_8 = __Pyx_PyObject_Call(__pyx_t_3, __pyx_tuple__150, NULL);
                  __Pyx_DECREF(__pyx_t_3); __pyx_t_3 = 0;
                  if (unlikely(!__pyx_t_8)) __PYX_ERR(0, 57, __pyx_L7_error)
                  __Pyx_GOTREF(__pyx_t_8);
                  __Pyx_DECREF(__pyx_t_8); __pyx_t_8 = 0;
                }
                goto __pyx_L12;
              }
              __pyx_L12:;
            }
            goto __pyx_L22;
            __pyx_L9_error:;
            __Pyx_DECREF(__pyx_t_3); __pyx_t_3 = 0;
            goto __pyx_L7_error;
            __pyx_L22:;
          }
        }

        /* "src/python/grpcio/grpc/_cython/_cygrpc/fork_posix.pyx.pxi":56
 * 
 * cdef void __postfork_parent() nogil:
 *     with gil:             # <<<<<<<<<<<<<<
 *         with _fork_state.fork_in_progress_condition:
 *             _fork_state.fork_in_progress = False
 */
        /*finally:*/ {
          /*normal exit:*/{
            #ifdef WITH_THREAD
            __Pyx_PyGILState_Release(__pyx_gilstate_save);
            #endif
            goto __pyx_L8;
          }
          __pyx_L7_error: {
            #ifdef WITH_THREAD
            __Pyx_PyGILState_Release(__pyx_gilstate_save);
            #endif
            goto __pyx_L4_error;
          }
          __pyx_L8:;
        }
    }
  }
  /*finally:*/ {
    /*normal exit:*/{
      #ifdef WITH_THREAD
      __pyx_gilstate_save = __Pyx_PyGILState_Ensure();
      #endif
      goto __pyx_L5;
    }
    __pyx_L4_error: {
      #ifdef WITH_THREAD
      __pyx_gilstate_save = __Pyx_PyGILState_Ensure();
      #endif
      goto __pyx_L1_error;
    }
    __pyx_L5:;
  }

  /* "src/python/grpcio/grpc/_cython/_cygrpc/fork_posix.pyx.pxi":55
 * 
 * 
 * cdef void __postfork_parent() nogil:             # <<<<<<<<<<<<<<
 *     with gil:
 *         with _fork_state.fork_in_progress_condition:
 */

  /* function exit code */
  goto __pyx_L0;
  __pyx_L1_error:;
  __Pyx_XDECREF(__pyx_t_1);
  __Pyx_XDECREF(__pyx_t_2);
  __Pyx_XDECREF(__pyx_t_4);
  __Pyx_XDECREF(__pyx_t_5);
  __Pyx_WriteUnraisable("grpc._cython.cygrpc.__postfork_parent", __pyx_clineno, __pyx_lineno, __pyx_filename, 1, 1);
  __pyx_L0:;
  #ifdef WITH_THREAD
  __Pyx_PyGILState_Release(__pyx_gilstate_save);
  #endif
}

/* "src/python/grpcio/grpc/_cython/_cygrpc/fork_posix.pyx.pxi":62
 * 
 * 
 * cdef void __postfork_child() nogil:             # <<<<<<<<<<<<<<
 *     with gil:
 *         try:
 */

static void __pyx_f_4grpc_7_cython_6cygrpc___postfork_child(void) {
  PyObject *__pyx_v_state_to_reset = NULL;
  PyObject *__pyx_v_channel = NULL;
  __Pyx_RefNannyDeclarations
  PyObject *__pyx_t_1 = NULL;
  PyObject *__pyx_t_2 = NULL;
  PyObject *__pyx_t_3 = NULL;
  PyObject *__pyx_t_4 = NULL;
  int __pyx_t_5;
  PyObject *__pyx_t_6 = NULL;
  PyObject *__pyx_t_7 = NULL;
  Py_ssize_t __pyx_t_8;
  PyObject *(*__pyx_t_9)(PyObject *);
  PyObject *__pyx_t_10 = NULL;
  PyObject *__pyx_t_11 = NULL;
  PyObject *__pyx_t_12 = NULL;
  PyObject *__pyx_t_13 = NULL;
  PyObject *__pyx_t_14 = NULL;
  PyObject *__pyx_t_15 = NULL;
  int __pyx_t_16;
  PyObject *__pyx_t_17 = NULL;
  PyObject *__pyx_t_18 = NULL;
  PyObject *__pyx_t_19 = NULL;
  PyObject *__pyx_t_20 = NULL;
  #ifdef WITH_THREAD
  PyGILState_STATE __pyx_gilstate_save;
  #endif
  __Pyx_RefNannySetupContext("__postfork_child", 1);

  /* "src/python/grpcio/grpc/_cython/_cygrpc/fork_posix.pyx.pxi":63
 * 
 * cdef void __postfork_child() nogil:
 *     with gil:             # <<<<<<<<<<<<<<
 *         try:
 *             if _fork_handler_failed:
 */
  /*try:*/ {
    {
        #ifdef WITH_THREAD
        PyGILState_STATE __pyx_gilstate_save = __Pyx_PyGILState_Ensure();
        #endif
        /*try:*/ {

          /* "src/python/grpcio/grpc/_cython/_cygrpc/fork_posix.pyx.pxi":64
 * cdef void __postfork_child() nogil:
 *     with gil:
 *         try:             # <<<<<<<<<<<<<<
 *             if _fork_handler_failed:
 *                 return
 */
          {
            __Pyx_PyThreadState_declare
            __Pyx_PyThreadState_assign
            __Pyx_ExceptionSave(&__pyx_t_1, &__pyx_t_2, &__pyx_t_3);
            __Pyx_XGOTREF(__pyx_t_1);
            __Pyx_XGOTREF(__pyx_t_2);
            __Pyx_XGOTREF(__pyx_t_3);
            /*try:*/ {

              /* "src/python/grpcio/grpc/_cython/_cygrpc/fork_posix.pyx.pxi":65
 *     with gil:
 *         try:
 *             if _fork_handler_failed:             # <<<<<<<<<<<<<<
 *                 return
 *             # Thread could be holding the fork_in_progress_condition inside of
 */
              __pyx_t_4 = __Pyx_GetModuleGlobalName(__pyx_n_s_fork_handler_failed); if (unlikely(!__pyx_t_4)) __PYX_ERR(0, 65, __pyx_L9_error)
              __Pyx_GOTREF(__pyx_t_4);
              __pyx_t_5 = __Pyx_PyObject_IsTrue(__pyx_t_4); if (unlikely(__pyx_t_5 < 0)) __PYX_ERR(0, 65, __pyx_L9_error)
              __Pyx_DECREF(__pyx_t_4); __pyx_t_4 = 0;
              if (__pyx_t_5) {

                /* "src/python/grpcio/grpc/_cython/_cygrpc/fork_posix.pyx.pxi":66
 *         try:
 *             if _fork_handler_failed:
 *                 return             # <<<<<<<<<<<<<<
 *             # Thread could be holding the fork_in_progress_condition inside of
 *             # block_if_fork_in_progress() when fork occurs. Reset the lock here.
 */
                goto __pyx_L13_try_return;

                /* "src/python/grpcio/grpc/_cython/_cygrpc/fork_posix.pyx.pxi":65
 *     with gil:
 *         try:
 *             if _fork_handler_failed:             # <<<<<<<<<<<<<<
 *                 return
 *             # Thread could be holding the fork_in_progress_condition inside of
 */
              }

              /* "src/python/grpcio/grpc/_cython/_cygrpc/fork_posix.pyx.pxi":69
 *             # Thread could be holding the fork_in_progress_condition inside of
 *             # block_if_fork_in_progress() when fork occurs. Reset the lock here.
 *             _fork_state.fork_in_progress_condition = threading.Condition()             # <<<<<<<<<<<<<<
 *             # A thread in return_from_user_request_generator() may hold this lock
 *             # when fork occurs.
 */
              __pyx_t_6 = __Pyx_GetModuleGlobalName(__pyx_n_s_threading); if (unlikely(!__pyx_t_6)) __PYX_ERR(0, 69, __pyx_L9_error)
              __Pyx_GOTREF(__pyx_t_6);
              __pyx_t_7 = __Pyx_PyObject_GetAttrStr(__pyx_t_6, __pyx_n_s_Condition); if (unlikely(!__pyx_t_7)) __PYX_ERR(0, 69, __pyx_L9_error)
              __Pyx_GOTREF(__pyx_t_7);
              __Pyx_DECREF(__pyx_t_6); __pyx_t_6 = 0;
              __pyx_t_6 = NULL;
              if (CYTHON_UNPACK_METHODS && unlikely(PyMethod_Check(__pyx_t_7))) {
                __pyx_t_6 = PyMethod_GET_SELF(__pyx_t_7);
                if (likely(__pyx_t_6)) {
                  PyObject* function = PyMethod_GET_FUNCTION(__pyx_t_7);
                  __Pyx_INCREF(__pyx_t_6);
                  __Pyx_INCREF(function);
                  __Pyx_DECREF_SET(__pyx_t_7, function);
                }
              }
              if (__pyx_t_6) {
                __pyx_t_4 = __Pyx_PyObject_CallOneArg(__pyx_t_7, __pyx_t_6); if (unlikely(!__pyx_t_4)) __PYX_ERR(0, 69, __pyx_L9_error)
                __Pyx_DECREF(__pyx_t_6); __pyx_t_6 = 0;
              } else {
                __pyx_t_4 = __Pyx_PyObject_CallNoArg(__pyx_t_7); if (unlikely(!__pyx_t_4)) __PYX_ERR(0, 69, __pyx_L9_error)
              }
              __Pyx_GOTREF(__pyx_t_4);
              __Pyx_DECREF(__pyx_t_7); __pyx_t_7 = 0;
              __pyx_t_7 = __Pyx_GetModuleGlobalName(__pyx_n_s_fork_state); if (unlikely(!__pyx_t_7)) __PYX_ERR(0, 69, __pyx_L9_error)
              __Pyx_GOTREF(__pyx_t_7);
              if (__Pyx_PyObject_SetAttrStr(__pyx_t_7, __pyx_n_s_fork_in_progress_condition, __pyx_t_4) < 0) __PYX_ERR(0, 69, __pyx_L9_error)
              __Pyx_DECREF(__pyx_t_4); __pyx_t_4 = 0;
              __Pyx_DECREF(__pyx_t_7); __pyx_t_7 = 0;

              /* "src/python/grpcio/grpc/_cython/_cygrpc/fork_posix.pyx.pxi":72
 *             # A thread in return_from_user_request_generator() may hold this lock
 *             # when fork occurs.
 *             _fork_state.active_thread_count = _ActiveThreadCount()             # <<<<<<<<<<<<<<
 *             for state_to_reset in _fork_state.postfork_states_to_reset:
 *                 state_to_reset.reset_postfork_child()
 */
              __pyx_t_4 = __Pyx_GetModuleGlobalName(__pyx_n_s_ActiveThreadCount); if (unlikely(!__pyx_t_4)) __PYX_ERR(0, 72, __pyx_L9_error)
              __Pyx_GOTREF(__pyx_t_4);
              __pyx_t_6 = NULL;
              if (CYTHON_UNPACK_METHODS && unlikely(PyMethod_Check(__pyx_t_4))) {
                __pyx_t_6 = PyMethod_GET_SELF(__pyx_t_4);
                if (likely(__pyx_t_6)) {
                  PyObject* function = PyMethod_GET_FUNCTION(__pyx_t_4);
                  __Pyx_INCREF(__pyx_t_6);
                  __Pyx_INCREF(function);
                  __Pyx_DECREF_SET(__pyx_t_4, function);
                }
              }
              if (__pyx_t_6) {
                __pyx_t_7 = __Pyx_PyObject_CallOneArg(__pyx_t_4, __pyx_t_6); if (unlikely(!__pyx_t_7)) __PYX_ERR(0, 72, __pyx_L9_error)
                __Pyx_DECREF(__pyx_t_6); __pyx_t_6 = 0;
              } else {
                __pyx_t_7 = __Pyx_PyObject_CallNoArg(__pyx_t_4); if (unlikely(!__pyx_t_7)) __PYX_ERR(0, 72, __pyx_L9_error)
              }
              __Pyx_GOTREF(__pyx_t_7);
              __Pyx_DECREF(__pyx_t_4); __pyx_t_4 = 0;
              __pyx_t_4 = __Pyx_GetModuleGlobalName(__pyx_n_s_fork_state); if (unlikely(!__pyx_t_4)) __PYX_ERR(0, 72, __pyx_L9_error)
              __Pyx_GOTREF(__pyx_t_4);
              if (__Pyx_PyObject_SetAttrStr(__pyx_t_4, __pyx_n_s_active_thread_count, __pyx_t_7) < 0) __PYX_ERR(0, 72, __pyx_L9_error)
              __Pyx_DECREF(__pyx_t_7); __pyx_t_7 = 0;
              __Pyx_DECREF(__pyx_t_4); __pyx_t_4 = 0;

              /* "src/python/grpcio/grpc/_cython/_cygrpc/fork_posix.pyx.pxi":73
 *             # when fork occurs.
 *             _fork_state.active_thread_count = _ActiveThreadCount()
 *             for state_to_reset in _fork_state.postfork_states_to_reset:             # <<<<<<<<<<<<<<
 *                 state_to_reset.reset_postfork_child()
 *             _fork_state.postfork_states_to_reset = []
 */
              __pyx_t_4 = __Pyx_GetModuleGlobalName(__pyx_n_s_fork_state); if (unlikely(!__pyx_t_4)) __PYX_ERR(0, 73, __pyx_L9_error)
              __Pyx_GOTREF(__pyx_t_4);
              __pyx_t_7 = __Pyx_PyObject_GetAttrStr(__pyx_t_4, __pyx_n_s_postfork_states_to_reset); if (unlikely(!__pyx_t_7)) __PYX_ERR(0, 73, __pyx_L9_error)
              __Pyx_GOTREF(__pyx_t_7);
              __Pyx_DECREF(__pyx_t_4); __pyx_t_4 = 0;
              if (likely(PyList_CheckExact(__pyx_t_7)) || PyTuple_CheckExact(__pyx_t_7)) {
                __pyx_t_4 = __pyx_t_7; __Pyx_INCREF(__pyx_t_4); __pyx_t_8 = 0;
                __pyx_t_9 = NULL;
              } else {
                __pyx_t_8 = -1; __pyx_t_4 = PyObject_GetIter(__pyx_t_7); if (unlikely(!__pyx_t_4)) __PYX_ERR(0, 73, __pyx_L9_error)
                __Pyx_GOTREF(__pyx_t_4);
                __pyx_t_9 = Py_TYPE(__pyx_t_4)->tp_iternext; if (unlikely(!__pyx_t_9)) __PYX_ERR(0, 73, __pyx_L9_error)
              }
              __Pyx_DECREF(__pyx_t_7); __pyx_t_7 = 0;
              for (;;) {
                if (likely(!__pyx_t_9)) {
                  if (likely(PyList_CheckExact(__pyx_t_4))) {
                    if (__pyx_t_8 >= PyList_GET_SIZE(__pyx_t_4)) break;
                    #if CYTHON_ASSUME_SAFE_MACROS && !CYTHON_AVOID_BORROWED_REFS
                    __pyx_t_7 = PyList_GET_ITEM(__pyx_t_4, __pyx_t_8); __Pyx_INCREF(__pyx_t_7); __pyx_t_8++; if (unlikely(0 < 0)) __PYX_ERR(0, 73, __pyx_L9_error)
                    #else
                    __pyx_t_7 = PySequence_ITEM(__pyx_t_4, __pyx_t_8); __pyx_t_8++; if (unlikely(!__pyx_t_7)) __PYX_ERR(0, 73, __pyx_L9_error)
                    __Pyx_GOTREF(__pyx_t_7);
                    #endif
                  } else {
                    if (__pyx_t_8 >= PyTuple_GET_SIZE(__pyx_t_4)) break;
                    #if CYTHON_ASSUME_SAFE_MACROS && !CYTHON_AVOID_BORROWED_REFS
                    __pyx_t_7 = PyTuple_GET_ITEM(__pyx_t_4, __pyx_t_8); __Pyx_INCREF(__pyx_t_7); __pyx_t_8++; if (unlikely(0 < 0)) __PYX_ERR(0, 73, __pyx_L9_error)
                    #else
                    __pyx_t_7 = PySequence_ITEM(__pyx_t_4, __pyx_t_8); __pyx_t_8++; if (unlikely(!__pyx_t_7)) __PYX_ERR(0, 73, __pyx_L9_error)
                    __Pyx_GOTREF(__pyx_t_7);
                    #endif
                  }
                } else {
                  __pyx_t_7 = __pyx_t_9(__pyx_t_4);
                  if (unlikely(!__pyx_t_7)) {
                    PyObject* exc_type = PyErr_Occurred();
                    if (exc_type) {
                      if (likely(__Pyx_PyErr_GivenExceptionMatches(exc_type, PyExc_StopIteration))) PyErr_Clear();
                      else __PYX_ERR(0, 73, __pyx_L9_error)
                    }
                    break;
                  }
                  __Pyx_GOTREF(__pyx_t_7);
                }
                __Pyx_XDECREF_SET(__pyx_v_state_to_reset, __pyx_t_7);
                __pyx_t_7 = 0;

                /* "src/python/grpcio/grpc/_cython/_cygrpc/fork_posix.pyx.pxi":74
 *             _fork_state.active_thread_count = _ActiveThreadCount()
 *             for state_to_reset in _fork_state.postfork_states_to_reset:
 *                 state_to_reset.reset_postfork_child()             # <<<<<<<<<<<<<<
 *             _fork_state.postfork_states_to_reset = []
 *             _fork_state.fork_epoch += 1
 */
                __pyx_t_6 = __Pyx_PyObject_GetAttrStr(__pyx_v_state_to_reset, __pyx_n_s_reset_postfork_child); if (unlikely(!__pyx_t_6)) __PYX_ERR(0, 74, __pyx_L9_error)
                __Pyx_GOTREF(__pyx_t_6);
                __pyx_t_10 = NULL;
                if (CYTHON_UNPACK_METHODS && likely(PyMethod_Check(__pyx_t_6))) {
                  __pyx_t_10 = PyMethod_GET_SELF(__pyx_t_6);
                  if (likely(__pyx_t_10)) {
                    PyObject* function = PyMethod_GET_FUNCTION(__pyx_t_6);
                    __Pyx_INCREF(__pyx_t_10);
                    __Pyx_INCREF(function);
                    __Pyx_DECREF_SET(__pyx_t_6, function);
                  }
                }
                if (__pyx_t_10) {
                  __pyx_t_7 = __Pyx_PyObject_CallOneArg(__pyx_t_6, __pyx_t_10); if (unlikely(!__pyx_t_7)) __PYX_ERR(0, 74, __pyx_L9_error)
                  __Pyx_DECREF(__pyx_t_10); __pyx_t_10 = 0;
                } else {
                  __pyx_t_7 = __Pyx_PyObject_CallNoArg(__pyx_t_6); if (unlikely(!__pyx_t_7)) __PYX_ERR(0, 74, __pyx_L9_error)
                }
                __Pyx_GOTREF(__pyx_t_7);
                __Pyx_DECREF(__pyx_t_6); __pyx_t_6 = 0;
                __Pyx_DECREF(__pyx_t_7); __pyx_t_7 = 0;

                /* "src/python/grpcio/grpc/_cython/_cygrpc/fork_posix.pyx.pxi":73
 *             # when fork occurs.
 *             _fork_state.active_thread_count = _ActiveThreadCount()
 *             for state_to_reset in _fork_state.postfork_states_to_reset:             # <<<<<<<<<<<<<<
 *                 state_to_reset.reset_postfork_child()
 *             _fork_state.postfork_states_to_reset = []
 */
              }
              __Pyx_DECREF(__pyx_t_4); __pyx_t_4 = 0;

              /* "src/python/grpcio/grpc/_cython/_cygrpc/fork_posix.pyx.pxi":75
 *             for state_to_reset in _fork_state.postfork_states_to_reset:
 *                 state_to_reset.reset_postfork_child()
 *             _fork_state.postfork_states_to_reset = []             # <<<<<<<<<<<<<<
 *             _fork_state.fork_epoch += 1
 *             for channel in _fork_state.channels:
 */
              __pyx_t_4 = PyList_New(0); if (unlikely(!__pyx_t_4)) __PYX_ERR(0, 75, __pyx_L9_error)
              __Pyx_GOTREF(__pyx_t_4);
              __pyx_t_7 = __Pyx_GetModuleGlobalName(__pyx_n_s_fork_state); if (unlikely(!__pyx_t_7)) __PYX_ERR(0, 75, __pyx_L9_error)
              __Pyx_GOTREF(__pyx_t_7);
              if (__Pyx_PyObject_SetAttrStr(__pyx_t_7, __pyx_n_s_postfork_states_to_reset, __pyx_t_4) < 0) __PYX_ERR(0, 75, __pyx_L9_error)
              __Pyx_DECREF(__pyx_t_4); __pyx_t_4 = 0;
              __Pyx_DECREF(__pyx_t_7); __pyx_t_7 = 0;

              /* "src/python/grpcio/grpc/_cython/_cygrpc/fork_posix.pyx.pxi":76
 *                 state_to_reset.reset_postfork_child()
 *             _fork_state.postfork_states_to_reset = []
 *             _fork_state.fork_epoch += 1             # <<<<<<<<<<<<<<
 *             for channel in _fork_state.channels:
 *                 channel._close_on_fork()
 */
              __pyx_t_7 = __Pyx_GetModuleGlobalName(__pyx_n_s_fork_state); if (unlikely(!__pyx_t_7)) __PYX_ERR(0, 76, __pyx_L9_error)
              __Pyx_GOTREF(__pyx_t_7);
              __pyx_t_4 = __Pyx_PyObject_GetAttrStr(__pyx_t_7, __pyx_n_s_fork_epoch); if (unlikely(!__pyx_t_4)) __PYX_ERR(0, 76, __pyx_L9_error)
              __Pyx_GOTREF(__pyx_t_4);
              __Pyx_DECREF(__pyx_t_7); __pyx_t_7 = 0;
              __pyx_t_7 = __Pyx_PyInt_AddObjC(__pyx_t_4, __pyx_int_1, 1, 1); if (unlikely(!__pyx_t_7)) __PYX_ERR(0, 76, __pyx_L9_error)
              __Pyx_GOTREF(__pyx_t_7);
              __Pyx_DECREF(__pyx_t_4); __pyx_t_4 = 0;
              __pyx_t_4 = __Pyx_GetModuleGlobalName(__pyx_n_s_fork_state); if (unlikely(!__pyx_t_4)) __PYX_ERR(0, 76, __pyx_L9_error)
              __Pyx_GOTREF(__pyx_t_4);
              if (__Pyx_PyObject_SetAttrStr(__pyx_t_4, __pyx_n_s_fork_epoch, __pyx_t_7) < 0) __PYX_ERR(0, 76, __pyx_L9_error)
              __Pyx_DECREF(__pyx_t_7); __pyx_t_7 = 0;
              __Pyx_DECREF(__pyx_t_4); __pyx_t_4 = 0;

              /* "src/python/grpcio/grpc/_cython/_cygrpc/fork_posix.pyx.pxi":77
 *             _fork_state.postfork_states_to_reset = []
 *             _fork_state.fork_epoch += 1
 *             for channel in _fork_state.channels:             # <<<<<<<<<<<<<<
 *                 channel._close_on_fork()
 *             with _fork_state.fork_in_progress_condition:
 */
              __pyx_t_4 = __Pyx_GetModuleGlobalName(__pyx_n_s_fork_state); if (unlikely(!__pyx_t_4)) __PYX_ERR(0, 77, __pyx_L9_error)
              __Pyx_GOTREF(__pyx_t_4);
              __pyx_t_7 = __Pyx_PyObject_GetAttrStr(__pyx_t_4, __pyx_n_s_channels); if (unlikely(!__pyx_t_7)) __PYX_ERR(0, 77, __pyx_L9_error)
              __Pyx_GOTREF(__pyx_t_7);
              __Pyx_DECREF(__pyx_t_4); __pyx_t_4 = 0;
              if (likely(PyList_CheckExact(__pyx_t_7)) || PyTuple_CheckExact(__pyx_t_7)) {
                __pyx_t_4 = __pyx_t_7; __Pyx_INCREF(__pyx_t_4); __pyx_t_8 = 0;
                __pyx_t_9 = NULL;
              } else {
                __pyx_t_8 = -1; __pyx_t_4 = PyObject_GetIter(__pyx_t_7); if (unlikely(!__pyx_t_4)) __PYX_ERR(0, 77, __pyx_L9_error)
                __Pyx_GOTREF(__pyx_t_4);
                __pyx_t_9 = Py_TYPE(__pyx_t_4)->tp_iternext; if (unlikely(!__pyx_t_9)) __PYX_ERR(0, 77, __pyx_L9_error)
              }
              __Pyx_DECREF(__pyx_t_7); __pyx_t_7 = 0;
              for (;;) {
                if (likely(!__pyx_t_9)) {
                  if (likely(PyList_CheckExact(__pyx_t_4))) {
                    if (__pyx_t_8 >= PyList_GET_SIZE(__pyx_t_4)) break;
                    #if CYTHON_ASSUME_SAFE_MACROS && !CYTHON_AVOID_BORROWED_REFS
                    __pyx_t_7 = PyList_GET_ITEM(__pyx_t_4, __pyx_t_8); __Pyx_INCREF(__pyx_t_7); __pyx_t_8++; if (unlikely(0 < 0)) __PYX_ERR(0, 77, __pyx_L9_error)
                    #else
                    __pyx_t_7 = PySequence_ITEM(__pyx_t_4, __pyx_t_8); __pyx_t_8++; if (unlikely(!__pyx_t_7)) __PYX_ERR(0, 77, __pyx_L9_error)
                    __Pyx_GOTREF(__pyx_t_7);
                    #endif
                  } else {
                    if (__pyx_t_8 >= PyTuple_GET_SIZE(__pyx_t_4)) break;
                    #if CYTHON_ASSUME_SAFE_MACROS && !CYTHON_AVOID_BORROWED_REFS
                    __pyx_t_7 = PyTuple_GET_ITEM(__pyx_t_4, __pyx_t_8); __Pyx_INCREF(__pyx_t_7); __pyx_t_8++; if (unlikely(0 < 0)) __PYX_ERR(0, 77, __pyx_L9_error)
                    #else
                    __pyx_t_7 = PySequence_ITEM(__pyx_t_4, __pyx_t_8); __pyx_t_8++; if (unlikely(!__pyx_t_7)) __PYX_ERR(0, 77, __pyx_L9_error)
                    __Pyx_GOTREF(__pyx_t_7);
                    #endif
                  }
                } else {
                  __pyx_t_7 = __pyx_t_9(__pyx_t_4);
                  if (unlikely(!__pyx_t_7)) {
                    PyObject* exc_type = PyErr_Occurred();
                    if (exc_type) {
                      if (likely(__Pyx_PyErr_GivenExceptionMatches(exc_type, PyExc_StopIteration))) PyErr_Clear();
                      else __PYX_ERR(0, 77, __pyx_L9_error)
                    }
                    break;
                  }
                  __Pyx_GOTREF(__pyx_t_7);
                }
                __Pyx_XDECREF_SET(__pyx_v_channel, __pyx_t_7);
                __pyx_t_7 = 0;

                /* "src/python/grpcio/grpc/_cython/_cygrpc/fork_posix.pyx.pxi":78
 *             _fork_state.fork_epoch += 1
 *             for channel in _fork_state.channels:
 *                 channel._close_on_fork()             # <<<<<<<<<<<<<<
 *             with _fork_state.fork_in_progress_condition:
 *                 _fork_state.fork_in_progress = False
 */
                __pyx_t_6 = __Pyx_PyObject_GetAttrStr(__pyx_v_channel, __pyx_n_s_close_on_fork); if (unlikely(!__pyx_t_6)) __PYX_ERR(0, 78, __pyx_L9_error)
                __Pyx_GOTREF(__pyx_t_6);
                __pyx_t_10 = NULL;
                if (CYTHON_UNPACK_METHODS && likely(PyMethod_Check(__pyx_t_6))) {
                  __pyx_t_10 = PyMethod_GET_SELF(__pyx_t_6);
                  if (likely(__pyx_t_10)) {
                    PyObject* function = PyMethod_GET_FUNCTION(__pyx_t_6);
                    __Pyx_INCREF(__pyx_t_10);
                    __Pyx_INCREF(function);
                    __Pyx_DECREF_SET(__pyx_t_6, function);
                  }
                }
                if (__pyx_t_10) {
                  __pyx_t_7 = __Pyx_PyObject_CallOneArg(__pyx_t_6, __pyx_t_10); if (unlikely(!__pyx_t_7)) __PYX_ERR(0, 78, __pyx_L9_error)
                  __Pyx_DECREF(__pyx_t_10); __pyx_t_10 = 0;
                } else {
                  __pyx_t_7 = __Pyx_PyObject_CallNoArg(__pyx_t_6); if (unlikely(!__pyx_t_7)) __PYX_ERR(0, 78, __pyx_L9_error)
                }
                __Pyx_GOTREF(__pyx_t_7);
                __Pyx_DECREF(__pyx_t_6); __pyx_t_6 = 0;
                __Pyx_DECREF(__pyx_t_7); __pyx_t_7 = 0;

                /* "src/python/grpcio/grpc/_cython/_cygrpc/fork_posix.pyx.pxi":77
 *             _fork_state.postfork_states_to_reset = []
 *             _fork_state.fork_epoch += 1
 *             for channel in _fork_state.channels:             # <<<<<<<<<<<<<<
 *                 channel._close_on_fork()
 *             with _fork_state.fork_in_progress_condition:
 */
              }
              __Pyx_DECREF(__pyx_t_4); __pyx_t_4 = 0;

              /* "src/python/grpcio/grpc/_cython/_cygrpc/fork_posix.pyx.pxi":79
 *             for channel in _fork_state.channels:
 *                 channel._close_on_fork()
 *             with _fork_state.fork_in_progress_condition:             # <<<<<<<<<<<<<<
 *                 _fork_state.fork_in_progress = False
 *         except:
 */
              /*with:*/ {
                __pyx_t_4 = __Pyx_GetModuleGlobalName(__pyx_n_s_fork_state); if (unlikely(!__pyx_t_4)) __PYX_ERR(0, 79, __pyx_L9_error)
                __Pyx_GOTREF(__pyx_t_4);
                __pyx_t_7 = __Pyx_PyObject_GetAttrStr(__pyx_t_4, __pyx_n_s_fork_in_progress_condition); if (unlikely(!__pyx_t_7)) __PYX_ERR(0, 79, __pyx_L9_error)
                __Pyx_GOTREF(__pyx_t_7);
                __Pyx_DECREF(__pyx_t_4); __pyx_t_4 = 0;
                __pyx_t_11 = __Pyx_PyObject_LookupSpecial(__pyx_t_7, __pyx_n_s_exit); if (unlikely(!__pyx_t_11)) __PYX_ERR(0, 79, __pyx_L9_error)
                __Pyx_GOTREF(__pyx_t_11);
                __pyx_t_6 = __Pyx_PyObject_LookupSpecial(__pyx_t_7, __pyx_n_s_enter); if (unlikely(!__pyx_t_6)) __PYX_ERR(0, 79, __pyx_L20_error)
                __Pyx_GOTREF(__pyx_t_6);
                __pyx_t_10 = NULL;
                if (CYTHON_UNPACK_METHODS && likely(PyMethod_Check(__pyx_t_6))) {
                  __pyx_t_10 = PyMethod_GET_SELF(__pyx_t_6);
                  if (likely(__pyx_t_10)) {
                    PyObject* function = PyMethod_GET_FUNCTION(__pyx_t_6);
                    __Pyx_INCREF(__pyx_t_10);
                    __Pyx_INCREF(function);
                    __Pyx_DECREF_SET(__pyx_t_6, function);
                  }
                }
                if (__pyx_t_10) {
                  __pyx_t_4 = __Pyx_PyObject_CallOneArg(__pyx_t_6, __pyx_t_10); if (unlikely(!__pyx_t_4)) __PYX_ERR(0, 79, __pyx_L20_error)
                  __Pyx_DECREF(__pyx_t_10); __pyx_t_10 = 0;
                } else {
                  __pyx_t_4 = __Pyx_PyObject_CallNoArg(__pyx_t_6); if (unlikely(!__pyx_t_4)) __PYX_ERR(0, 79, __pyx_L20_error)
                }
                __Pyx_GOTREF(__pyx_t_4);
                __Pyx_DECREF(__pyx_t_6); __pyx_t_6 = 0;
                __Pyx_DECREF(__pyx_t_4); __pyx_t_4 = 0;
                __Pyx_DECREF(__pyx_t_7); __pyx_t_7 = 0;
                /*try:*/ {
                  {
                    __Pyx_PyThreadState_declare
                    __Pyx_PyThreadState_assign
                    __Pyx_ExceptionSave(&__pyx_t_12, &__pyx_t_13, &__pyx_t_14);
                    __Pyx_XGOTREF(__pyx_t_12);
                    __Pyx_XGOTREF(__pyx_t_13);
                    __Pyx_XGOTREF(__pyx_t_14);
                    /*try:*/ {

                      /* "src/python/grpcio/grpc/_cython/_cygrpc/fork_posix.pyx.pxi":80
 *                 channel._close_on_fork()
 *             with _fork_state.fork_in_progress_condition:
 *                 _fork_state.fork_in_progress = False             # <<<<<<<<<<<<<<
 *         except:
 *             _LOGGER.error('Exiting child due to raised exception')
 */
                      __pyx_t_7 = __Pyx_GetModuleGlobalName(__pyx_n_s_fork_state); if (unlikely(!__pyx_t_7)) __PYX_ERR(0, 80, __pyx_L24_error)
                      __Pyx_GOTREF(__pyx_t_7);
                      if (__Pyx_PyObject_SetAttrStr(__pyx_t_7, __pyx_n_s_fork_in_progress, Py_False) < 0) __PYX_ERR(0, 80, __pyx_L24_error)
                      __Pyx_DECREF(__pyx_t_7); __pyx_t_7 = 0;

                      /* "src/python/grpcio/grpc/_cython/_cygrpc/fork_posix.pyx.pxi":79
 *             for channel in _fork_state.channels:
 *                 channel._close_on_fork()
 *             with _fork_state.fork_in_progress_condition:             # <<<<<<<<<<<<<<
 *                 _fork_state.fork_in_progress = False
 *         except:
 */
                    }
                    __Pyx_XDECREF(__pyx_t_12); __pyx_t_12 = 0;
                    __Pyx_XDECREF(__pyx_t_13); __pyx_t_13 = 0;
                    __Pyx_XDECREF(__pyx_t_14); __pyx_t_14 = 0;
                    goto __pyx_L29_try_end;
                    __pyx_L24_error:;
                    __Pyx_XDECREF(__pyx_t_10); __pyx_t_10 = 0;
                    __Pyx_XDECREF(__pyx_t_6); __pyx_t_6 = 0;
                    __Pyx_XDECREF(__pyx_t_4); __pyx_t_4 = 0;
                    __Pyx_XDECREF(__pyx_t_7); __pyx_t_7 = 0;
                    /*except:*/ {
                      __Pyx_AddTraceback("grpc._cython.cygrpc.__postfork_child", __pyx_clineno, __pyx_lineno, __pyx_filename);
                      if (__Pyx_GetException(&__pyx_t_7, &__pyx_t_4, &__pyx_t_6) < 0) __PYX_ERR(0, 79, __pyx_L26_except_error)
                      __Pyx_GOTREF(__pyx_t_7);
                      __Pyx_GOTREF(__pyx_t_4);
                      __Pyx_GOTREF(__pyx_t_6);
                      __pyx_t_10 = PyTuple_Pack(3, __pyx_t_7, __pyx_t_4, __pyx_t_6); if (unlikely(!__pyx_t_10)) __PYX_ERR(0, 79, __pyx_L26_except_error)
                      __Pyx_GOTREF(__pyx_t_10);
                      __pyx_t_15 = __Pyx_PyObject_Call(__pyx_t_11, __pyx_t_10, NULL);
                      __Pyx_DECREF(__pyx_t_11); __pyx_t_11 = 0;
                      __Pyx_DECREF(__pyx_t_10); __pyx_t_10 = 0;
                      if (unlikely(!__pyx_t_15)) __PYX_ERR(0, 79, __pyx_L26_except_error)
                      __Pyx_GOTREF(__pyx_t_15);
                      __pyx_t_5 = __Pyx_PyObject_IsTrue(__pyx_t_15);
                      __Pyx_DECREF(__pyx_t_15); __pyx_t_15 = 0;
                      if (__pyx_t_5 < 0) __PYX_ERR(0, 79, __pyx_L26_except_error)
                      __pyx_t_16 = ((!(__pyx_t_5 != 0)) != 0);
                      if (__pyx_t_16) {
                        __Pyx_GIVEREF(__pyx_t_7);
                        __Pyx_GIVEREF(__pyx_t_4);
                        __Pyx_XGIVEREF(__pyx_t_6);
                        __Pyx_ErrRestoreWithState(__pyx_t_7, __pyx_t_4, __pyx_t_6);
                        __pyx_t_7 = 0; __pyx_t_4 = 0; __pyx_t_6 = 0; 
                        __PYX_ERR(0, 79, __pyx_L26_except_error)
                      }
                      __Pyx_DECREF(__pyx_t_7); __pyx_t_7 = 0;
                      __Pyx_DECREF(__pyx_t_4); __pyx_t_4 = 0;
                      __Pyx_DECREF(__pyx_t_6); __pyx_t_6 = 0;
                      goto __pyx_L25_exception_handled;
                    }
                    __pyx_L26_except_error:;
                    __Pyx_XGIVEREF(__pyx_t_12);
                    __Pyx_XGIVEREF(__pyx_t_13);
                    __Pyx_XGIVEREF(__pyx_t_14);
                    __Pyx_ExceptionReset(__pyx_t_12, __pyx_t_13, __pyx_t_14);
                    goto __pyx_L9_error;
                    __pyx_L25_exception_handled:;
                    __Pyx_XGIVEREF(__pyx_t_12);
                    __Pyx_XGIVEREF(__pyx_t_13);
                    __Pyx_XGIVEREF(__pyx_t_14);
                    __Pyx_ExceptionReset(__pyx_t_12, __pyx_t_13, __pyx_t_14);
                    __pyx_L29_try_end:;
                  }
                }
                /*finally:*/ {
                  /*normal exit:*/{
                    if (__pyx_t_11) {
                      __pyx_t_14 = __Pyx_PyObject_Call(__pyx_t_11, __pyx_tuple__151, NULL);
                      __Pyx_DECREF(__pyx_t_11); __pyx_t_11 = 0;
                      if (unlikely(!__pyx_t_14)) __PYX_ERR(0, 79, __pyx_L9_error)
                      __Pyx_GOTREF(__pyx_t_14);
                      __Pyx_DECREF(__pyx_t_14); __pyx_t_14 = 0;
                    }
                    goto __pyx_L23;
                  }
                  __pyx_L23:;
                }
                goto __pyx_L33;
                __pyx_L20_error:;
                __Pyx_DECREF(__pyx_t_11); __pyx_t_11 = 0;
                goto __pyx_L9_error;
                __pyx_L33:;
              }

              /* "src/python/grpcio/grpc/_cython/_cygrpc/fork_posix.pyx.pxi":64
 * cdef void __postfork_child() nogil:
 *     with gil:
 *         try:             # <<<<<<<<<<<<<<
 *             if _fork_handler_failed:
 *                 return
 */
            }
            __Pyx_XDECREF(__pyx_t_1); __pyx_t_1 = 0;
            __Pyx_XDECREF(__pyx_t_2); __pyx_t_2 = 0;
            __Pyx_XDECREF(__pyx_t_3); __pyx_t_3 = 0;
            goto __pyx_L14_try_end;
            __pyx_L9_error:;
            __Pyx_XDECREF(__pyx_t_10); __pyx_t_10 = 0;
            __Pyx_XDECREF(__pyx_t_7); __pyx_t_7 = 0;
            __Pyx_XDECREF(__pyx_t_4); __pyx_t_4 = 0;
            __Pyx_XDECREF(__pyx_t_6); __pyx_t_6 = 0;

            /* "src/python/grpcio/grpc/_cython/_cygrpc/fork_posix.pyx.pxi":81
 *             with _fork_state.fork_in_progress_condition:
 *                 _fork_state.fork_in_progress = False
 *         except:             # <<<<<<<<<<<<<<
 *             _LOGGER.error('Exiting child due to raised exception')
 *             _LOGGER.error(sys.exc_info()[0])
 */
            /*except:*/ {
              __Pyx_AddTraceback("grpc._cython.cygrpc.__postfork_child", __pyx_clineno, __pyx_lineno, __pyx_filename);
              if (__Pyx_GetException(&__pyx_t_6, &__pyx_t_4, &__pyx_t_7) < 0) __PYX_ERR(0, 81, __pyx_L11_except_error)
              __Pyx_GOTREF(__pyx_t_6);
              __Pyx_GOTREF(__pyx_t_4);
              __Pyx_GOTREF(__pyx_t_7);

              /* "src/python/grpcio/grpc/_cython/_cygrpc/fork_posix.pyx.pxi":82
 *                 _fork_state.fork_in_progress = False
 *         except:
 *             _LOGGER.error('Exiting child due to raised exception')             # <<<<<<<<<<<<<<
 *             _LOGGER.error(sys.exc_info()[0])
 *             os._exit(os.EX_USAGE)
 */
              __pyx_t_10 = __Pyx_GetModuleGlobalName(__pyx_n_s_LOGGER); if (unlikely(!__pyx_t_10)) __PYX_ERR(0, 82, __pyx_L11_except_error)
              __Pyx_GOTREF(__pyx_t_10);
              __pyx_t_17 = __Pyx_PyObject_GetAttrStr(__pyx_t_10, __pyx_n_s_error); if (unlikely(!__pyx_t_17)) __PYX_ERR(0, 82, __pyx_L11_except_error)
              __Pyx_GOTREF(__pyx_t_17);
              __Pyx_DECREF(__pyx_t_10); __pyx_t_10 = 0;
              __pyx_t_10 = __Pyx_PyObject_Call(__pyx_t_17, __pyx_tuple__152, NULL); if (unlikely(!__pyx_t_10)) __PYX_ERR(0, 82, __pyx_L11_except_error)
              __Pyx_GOTREF(__pyx_t_10);
              __Pyx_DECREF(__pyx_t_17); __pyx_t_17 = 0;
              __Pyx_DECREF(__pyx_t_10); __pyx_t_10 = 0;

              /* "src/python/grpcio/grpc/_cython/_cygrpc/fork_posix.pyx.pxi":83
 *         except:
 *             _LOGGER.error('Exiting child due to raised exception')
 *             _LOGGER.error(sys.exc_info()[0])             # <<<<<<<<<<<<<<
 *             os._exit(os.EX_USAGE)
 * 
 */
              __pyx_t_17 = __Pyx_GetModuleGlobalName(__pyx_n_s_LOGGER); if (unlikely(!__pyx_t_17)) __PYX_ERR(0, 83, __pyx_L11_except_error)
              __Pyx_GOTREF(__pyx_t_17);
              __pyx_t_18 = __Pyx_PyObject_GetAttrStr(__pyx_t_17, __pyx_n_s_error); if (unlikely(!__pyx_t_18)) __PYX_ERR(0, 83, __pyx_L11_except_error)
              __Pyx_GOTREF(__pyx_t_18);
              __Pyx_DECREF(__pyx_t_17); __pyx_t_17 = 0;
              __pyx_t_19 = __Pyx_GetModuleGlobalName(__pyx_n_s_sys); if (unlikely(!__pyx_t_19)) __PYX_ERR(0, 83, __pyx_L11_except_error)
              __Pyx_GOTREF(__pyx_t_19);
              __pyx_t_20 = __Pyx_PyObject_GetAttrStr(__pyx_t_19, __pyx_n_s_exc_info); if (unlikely(!__pyx_t_20)) __PYX_ERR(0, 83, __pyx_L11_except_error)
              __Pyx_GOTREF(__pyx_t_20);
              __Pyx_DECREF(__pyx_t_19); __pyx_t_19 = 0;
              __pyx_t_19 = NULL;
              if (CYTHON_UNPACK_METHODS && unlikely(PyMethod_Check(__pyx_t_20))) {
                __pyx_t_19 = PyMethod_GET_SELF(__pyx_t_20);
                if (likely(__pyx_t_19)) {
                  PyObject* function = PyMethod_GET_FUNCTION(__pyx_t_20);
                  __Pyx_INCREF(__pyx_t_19);
                  __Pyx_INCREF(function);
                  __Pyx_DECREF_SET(__pyx_t_20, function);
                }
              }
              if (__pyx_t_19) {
                __pyx_t_17 = __Pyx_PyObject_CallOneArg(__pyx_t_20, __pyx_t_19); if (unlikely(!__pyx_t_17)) __PYX_ERR(0, 83, __pyx_L11_except_error)
                __Pyx_DECREF(__pyx_t_19); __pyx_t_19 = 0;
              } else {
                __pyx_t_17 = __Pyx_PyObject_CallNoArg(__pyx_t_20); if (unlikely(!__pyx_t_17)) __PYX_ERR(0, 83, __pyx_L11_except_error)
              }
              __Pyx_GOTREF(__pyx_t_17);
              __Pyx_DECREF(__pyx_t_20); __pyx_t_20 = 0;
              __pyx_t_20 = __Pyx_GetItemInt(__pyx_t_17, 0, long, 1, __Pyx_PyInt_From_long, 0, 0, 1); if (unlikely(!__pyx_t_20)) __PYX_ERR(0, 83, __pyx_L11_except_error)
              __Pyx_GOTREF(__pyx_t_20);
              __Pyx_DECREF(__pyx_t_17); __pyx_t_17 = 0;
              __pyx_t_17 = NULL;
              if (CYTHON_UNPACK_METHODS && unlikely(PyMethod_Check(__pyx_t_18))) {
                __pyx_t_17 = PyMethod_GET_SELF(__pyx_t_18);
                if (likely(__pyx_t_17)) {
                  PyObject* function = PyMethod_GET_FUNCTION(__pyx_t_18);
                  __Pyx_INCREF(__pyx_t_17);
                  __Pyx_INCREF(function);
                  __Pyx_DECREF_SET(__pyx_t_18, function);
                }
              }
              if (!__pyx_t_17) {
                __pyx_t_10 = __Pyx_PyObject_CallOneArg(__pyx_t_18, __pyx_t_20); if (unlikely(!__pyx_t_10)) __PYX_ERR(0, 83, __pyx_L11_except_error)
                __Pyx_DECREF(__pyx_t_20); __pyx_t_20 = 0;
                __Pyx_GOTREF(__pyx_t_10);
              } else {
                #if CYTHON_FAST_PYCALL
                if (PyFunction_Check(__pyx_t_18)) {
                  PyObject *__pyx_temp[2] = {__pyx_t_17, __pyx_t_20};
                  __pyx_t_10 = __Pyx_PyFunction_FastCall(__pyx_t_18, __pyx_temp+1-1, 1+1); if (unlikely(!__pyx_t_10)) __PYX_ERR(0, 83, __pyx_L11_except_error)
                  __Pyx_XDECREF(__pyx_t_17); __pyx_t_17 = 0;
                  __Pyx_GOTREF(__pyx_t_10);
                  __Pyx_DECREF(__pyx_t_20); __pyx_t_20 = 0;
                } else
                #endif
                #if CYTHON_FAST_PYCCALL
                if (__Pyx_PyFastCFunction_Check(__pyx_t_18)) {
                  PyObject *__pyx_temp[2] = {__pyx_t_17, __pyx_t_20};
                  __pyx_t_10 = __Pyx_PyCFunction_FastCall(__pyx_t_18, __pyx_temp+1-1, 1+1); if (unlikely(!__pyx_t_10)) __PYX_ERR(0, 83, __pyx_L11_except_error)
                  __Pyx_XDECREF(__pyx_t_17); __pyx_t_17 = 0;
                  __Pyx_GOTREF(__pyx_t_10);
                  __Pyx_DECREF(__pyx_t_20); __pyx_t_20 = 0;
                } else
                #endif
                {
                  __pyx_t_19 = PyTuple_New(1+1); if (unlikely(!__pyx_t_19)) __PYX_ERR(0, 83, __pyx_L11_except_error)
                  __Pyx_GOTREF(__pyx_t_19);
                  __Pyx_GIVEREF(__pyx_t_17); PyTuple_SET_ITEM(__pyx_t_19, 0, __pyx_t_17); __pyx_t_17 = NULL;
                  __Pyx_GIVEREF(__pyx_t_20);
                  PyTuple_SET_ITEM(__pyx_t_19, 0+1, __pyx_t_20);
                  __pyx_t_20 = 0;
                  __pyx_t_10 = __Pyx_PyObject_Call(__pyx_t_18, __pyx_t_19, NULL); if (unlikely(!__pyx_t_10)) __PYX_ERR(0, 83, __pyx_L11_except_error)
                  __Pyx_GOTREF(__pyx_t_10);
                  __Pyx_DECREF(__pyx_t_19); __pyx_t_19 = 0;
                }
              }
              __Pyx_DECREF(__pyx_t_18); __pyx_t_18 = 0;
              __Pyx_DECREF(__pyx_t_10); __pyx_t_10 = 0;

              /* "src/python/grpcio/grpc/_cython/_cygrpc/fork_posix.pyx.pxi":84
 *             _LOGGER.error('Exiting child due to raised exception')
 *             _LOGGER.error(sys.exc_info()[0])
 *             os._exit(os.EX_USAGE)             # <<<<<<<<<<<<<<
 * 
 *     if grpc_is_initialized() > 0:
 */
              __pyx_t_18 = __Pyx_GetModuleGlobalName(__pyx_n_s_os); if (unlikely(!__pyx_t_18)) __PYX_ERR(0, 84, __pyx_L11_except_error)
              __Pyx_GOTREF(__pyx_t_18);
              __pyx_t_19 = __Pyx_PyObject_GetAttrStr(__pyx_t_18, __pyx_n_s_exit_2); if (unlikely(!__pyx_t_19)) __PYX_ERR(0, 84, __pyx_L11_except_error)
              __Pyx_GOTREF(__pyx_t_19);
              __Pyx_DECREF(__pyx_t_18); __pyx_t_18 = 0;
              __pyx_t_18 = __Pyx_GetModuleGlobalName(__pyx_n_s_os); if (unlikely(!__pyx_t_18)) __PYX_ERR(0, 84, __pyx_L11_except_error)
              __Pyx_GOTREF(__pyx_t_18);
              __pyx_t_20 = __Pyx_PyObject_GetAttrStr(__pyx_t_18, __pyx_n_s_EX_USAGE); if (unlikely(!__pyx_t_20)) __PYX_ERR(0, 84, __pyx_L11_except_error)
              __Pyx_GOTREF(__pyx_t_20);
              __Pyx_DECREF(__pyx_t_18); __pyx_t_18 = 0;
              __pyx_t_18 = NULL;
              if (CYTHON_UNPACK_METHODS && unlikely(PyMethod_Check(__pyx_t_19))) {
                __pyx_t_18 = PyMethod_GET_SELF(__pyx_t_19);
                if (likely(__pyx_t_18)) {
                  PyObject* function = PyMethod_GET_FUNCTION(__pyx_t_19);
                  __Pyx_INCREF(__pyx_t_18);
                  __Pyx_INCREF(function);
                  __Pyx_DECREF_SET(__pyx_t_19, function);
                }
              }
              if (!__pyx_t_18) {
                __pyx_t_10 = __Pyx_PyObject_CallOneArg(__pyx_t_19, __pyx_t_20); if (unlikely(!__pyx_t_10)) __PYX_ERR(0, 84, __pyx_L11_except_error)
                __Pyx_DECREF(__pyx_t_20); __pyx_t_20 = 0;
                __Pyx_GOTREF(__pyx_t_10);
              } else {
                #if CYTHON_FAST_PYCALL
                if (PyFunction_Check(__pyx_t_19)) {
                  PyObject *__pyx_temp[2] = {__pyx_t_18, __pyx_t_20};
                  __pyx_t_10 = __Pyx_PyFunction_FastCall(__pyx_t_19, __pyx_temp+1-1, 1+1); if (unlikely(!__pyx_t_10)) __PYX_ERR(0, 84, __pyx_L11_except_error)
                  __Pyx_XDECREF(__pyx_t_18); __pyx_t_18 = 0;
                  __Pyx_GOTREF(__pyx_t_10);
                  __Pyx_DECREF(__pyx_t_20); __pyx_t_20 = 0;
                } else
                #endif
                #if CYTHON_FAST_PYCCALL
                if (__Pyx_PyFastCFunction_Check(__pyx_t_19)) {
                  PyObject *__pyx_temp[2] = {__pyx_t_18, __pyx_t_20};
                  __pyx_t_10 = __Pyx_PyCFunction_FastCall(__pyx_t_19, __pyx_temp+1-1, 1+1); if (unlikely(!__pyx_t_10)) __PYX_ERR(0, 84, __pyx_L11_except_error)
                  __Pyx_XDECREF(__pyx_t_18); __pyx_t_18 = 0;
                  __Pyx_GOTREF(__pyx_t_10);
                  __Pyx_DECREF(__pyx_t_20); __pyx_t_20 = 0;
                } else
                #endif
                {
                  __pyx_t_17 = PyTuple_New(1+1); if (unlikely(!__pyx_t_17)) __PYX_ERR(0, 84, __pyx_L11_except_error)
                  __Pyx_GOTREF(__pyx_t_17);
                  __Pyx_GIVEREF(__pyx_t_18); PyTuple_SET_ITEM(__pyx_t_17, 0, __pyx_t_18); __pyx_t_18 = NULL;
                  __Pyx_GIVEREF(__pyx_t_20);
                  PyTuple_SET_ITEM(__pyx_t_17, 0+1, __pyx_t_20);
                  __pyx_t_20 = 0;
                  __pyx_t_10 = __Pyx_PyObject_Call(__pyx_t_19, __pyx_t_17, NULL); if (unlikely(!__pyx_t_10)) __PYX_ERR(0, 84, __pyx_L11_except_error)
                  __Pyx_GOTREF(__pyx_t_10);
                  __Pyx_DECREF(__pyx_t_17); __pyx_t_17 = 0;
                }
              }
              __Pyx_DECREF(__pyx_t_19); __pyx_t_19 = 0;
              __Pyx_DECREF(__pyx_t_10); __pyx_t_10 = 0;
              __Pyx_DECREF(__pyx_t_6); __pyx_t_6 = 0;
              __Pyx_DECREF(__pyx_t_4); __pyx_t_4 = 0;
              __Pyx_DECREF(__pyx_t_7); __pyx_t_7 = 0;
              goto __pyx_L10_exception_handled;
            }
            __pyx_L11_except_error:;

            /* "src/python/grpcio/grpc/_cython/_cygrpc/fork_posix.pyx.pxi":64
 * cdef void __postfork_child() nogil:
 *     with gil:
 *         try:             # <<<<<<<<<<<<<<
 *             if _fork_handler_failed:
 *                 return
 */
            __Pyx_XGIVEREF(__pyx_t_1);
            __Pyx_XGIVEREF(__pyx_t_2);
            __Pyx_XGIVEREF(__pyx_t_3);
            __Pyx_ExceptionReset(__pyx_t_1, __pyx_t_2, __pyx_t_3);
            goto __pyx_L7_error;
            __pyx_L13_try_return:;
            __Pyx_XGIVEREF(__pyx_t_1);
            __Pyx_XGIVEREF(__pyx_t_2);
            __Pyx_XGIVEREF(__pyx_t_3);
            __Pyx_ExceptionReset(__pyx_t_1, __pyx_t_2, __pyx_t_3);
            goto __pyx_L6_return;
            __pyx_L10_exception_handled:;
            __Pyx_XGIVEREF(__pyx_t_1);
            __Pyx_XGIVEREF(__pyx_t_2);
            __Pyx_XGIVEREF(__pyx_t_3);
            __Pyx_ExceptionReset(__pyx_t_1, __pyx_t_2, __pyx_t_3);
            __pyx_L14_try_end:;
          }
        }

        /* "src/python/grpcio/grpc/_cython/_cygrpc/fork_posix.pyx.pxi":63
 * 
 * cdef void __postfork_child() nogil:
 *     with gil:             # <<<<<<<<<<<<<<
 *         try:
 *             if _fork_handler_failed:
 */
        /*finally:*/ {
          /*normal exit:*/{
            #ifdef WITH_THREAD
            __Pyx_PyGILState_Release(__pyx_gilstate_save);
            #endif
            goto __pyx_L8;
          }
          __pyx_L6_return: {
            #ifdef WITH_THREAD
            __Pyx_PyGILState_Release(__pyx_gilstate_save);
            #endif
            goto __pyx_L3_return;
          }
          __pyx_L7_error: {
            #ifdef WITH_THREAD
            __Pyx_PyGILState_Release(__pyx_gilstate_save);
            #endif
            goto __pyx_L4_error;
          }
          __pyx_L8:;
        }
    }

    /* "src/python/grpcio/grpc/_cython/_cygrpc/fork_posix.pyx.pxi":86
 *             os._exit(os.EX_USAGE)
 * 
 *     if grpc_is_initialized() > 0:             # <<<<<<<<<<<<<<
 *         with gil:
 *             _LOGGER.error('Failed to shutdown gRPC Core after fork()')
 */
    __pyx_t_16 = ((grpc_is_initialized() > 0) != 0);
    if (__pyx_t_16) {

      /* "src/python/grpcio/grpc/_cython/_cygrpc/fork_posix.pyx.pxi":87
 * 
 *     if grpc_is_initialized() > 0:
 *         with gil:             # <<<<<<<<<<<<<<
 *             _LOGGER.error('Failed to shutdown gRPC Core after fork()')
 *             os._exit(os.EX_USAGE)
 */
      {
          #ifdef WITH_THREAD
          PyGILState_STATE __pyx_gilstate_save = __Pyx_PyGILState_Ensure();
          #endif
          /*try:*/ {

            /* "src/python/grpcio/grpc/_cython/_cygrpc/fork_posix.pyx.pxi":88
 *     if grpc_is_initialized() > 0:
 *         with gil:
 *             _LOGGER.error('Failed to shutdown gRPC Core after fork()')             # <<<<<<<<<<<<<<
 *             os._exit(os.EX_USAGE)
 * 
 */
            __pyx_t_7 = __Pyx_GetModuleGlobalName(__pyx_n_s_LOGGER); if (unlikely(!__pyx_t_7)) __PYX_ERR(0, 88, __pyx_L38_error)
            __Pyx_GOTREF(__pyx_t_7);
            __pyx_t_4 = __Pyx_PyObject_GetAttrStr(__pyx_t_7, __pyx_n_s_error); if (unlikely(!__pyx_t_4)) __PYX_ERR(0, 88, __pyx_L38_error)
            __Pyx_GOTREF(__pyx_t_4);
            __Pyx_DECREF(__pyx_t_7); __pyx_t_7 = 0;
            __pyx_t_7 = __Pyx_PyObject_Call(__pyx_t_4, __pyx_tuple__153, NULL); if (unlikely(!__pyx_t_7)) __PYX_ERR(0, 88, __pyx_L38_error)
            __Pyx_GOTREF(__pyx_t_7);
            __Pyx_DECREF(__pyx_t_4); __pyx_t_4 = 0;
            __Pyx_DECREF(__pyx_t_7); __pyx_t_7 = 0;

            /* "src/python/grpcio/grpc/_cython/_cygrpc/fork_posix.pyx.pxi":89
 *         with gil:
 *             _LOGGER.error('Failed to shutdown gRPC Core after fork()')
 *             os._exit(os.EX_USAGE)             # <<<<<<<<<<<<<<
 * 
 * 
 */
            __pyx_t_4 = __Pyx_GetModuleGlobalName(__pyx_n_s_os); if (unlikely(!__pyx_t_4)) __PYX_ERR(0, 89, __pyx_L38_error)
            __Pyx_GOTREF(__pyx_t_4);
            __pyx_t_6 = __Pyx_PyObject_GetAttrStr(__pyx_t_4, __pyx_n_s_exit_2); if (unlikely(!__pyx_t_6)) __PYX_ERR(0, 89, __pyx_L38_error)
            __Pyx_GOTREF(__pyx_t_6);
            __Pyx_DECREF(__pyx_t_4); __pyx_t_4 = 0;
            __pyx_t_4 = __Pyx_GetModuleGlobalName(__pyx_n_s_os); if (unlikely(!__pyx_t_4)) __PYX_ERR(0, 89, __pyx_L38_error)
            __Pyx_GOTREF(__pyx_t_4);
            __pyx_t_10 = __Pyx_PyObject_GetAttrStr(__pyx_t_4, __pyx_n_s_EX_USAGE); if (unlikely(!__pyx_t_10)) __PYX_ERR(0, 89, __pyx_L38_error)
            __Pyx_GOTREF(__pyx_t_10);
            __Pyx_DECREF(__pyx_t_4); __pyx_t_4 = 0;
            __pyx_t_4 = NULL;
            if (CYTHON_UNPACK_METHODS && unlikely(PyMethod_Check(__pyx_t_6))) {
              __pyx_t_4 = PyMethod_GET_SELF(__pyx_t_6);
              if (likely(__pyx_t_4)) {
                PyObject* function = PyMethod_GET_FUNCTION(__pyx_t_6);
                __Pyx_INCREF(__pyx_t_4);
                __Pyx_INCREF(function);
                __Pyx_DECREF_SET(__pyx_t_6, function);
              }
            }
            if (!__pyx_t_4) {
              __pyx_t_7 = __Pyx_PyObject_CallOneArg(__pyx_t_6, __pyx_t_10); if (unlikely(!__pyx_t_7)) __PYX_ERR(0, 89, __pyx_L38_error)
              __Pyx_DECREF(__pyx_t_10); __pyx_t_10 = 0;
              __Pyx_GOTREF(__pyx_t_7);
            } else {
              #if CYTHON_FAST_PYCALL
              if (PyFunction_Check(__pyx_t_6)) {
                PyObject *__pyx_temp[2] = {__pyx_t_4, __pyx_t_10};
                __pyx_t_7 = __Pyx_PyFunction_FastCall(__pyx_t_6, __pyx_temp+1-1, 1+1); if (unlikely(!__pyx_t_7)) __PYX_ERR(0, 89, __pyx_L38_error)
                __Pyx_XDECREF(__pyx_t_4); __pyx_t_4 = 0;
                __Pyx_GOTREF(__pyx_t_7);
                __Pyx_DECREF(__pyx_t_10); __pyx_t_10 = 0;
              } else
              #endif
              #if CYTHON_FAST_PYCCALL
              if (__Pyx_PyFastCFunction_Check(__pyx_t_6)) {
                PyObject *__pyx_temp[2] = {__pyx_t_4, __pyx_t_10};
                __pyx_t_7 = __Pyx_PyCFunction_FastCall(__pyx_t_6, __pyx_temp+1-1, 1+1); if (unlikely(!__pyx_t_7)) __PYX_ERR(0, 89, __pyx_L38_error)
                __Pyx_XDECREF(__pyx_t_4); __pyx_t_4 = 0;
                __Pyx_GOTREF(__pyx_t_7);
                __Pyx_DECREF(__pyx_t_10); __pyx_t_10 = 0;
              } else
              #endif
              {
                __pyx_t_19 = PyTuple_New(1+1); if (unlikely(!__pyx_t_19)) __PYX_ERR(0, 89, __pyx_L38_error)
                __Pyx_GOTREF(__pyx_t_19);
                __Pyx_GIVEREF(__pyx_t_4); PyTuple_SET_ITEM(__pyx_t_19, 0, __pyx_t_4); __pyx_t_4 = NULL;
                __Pyx_GIVEREF(__pyx_t_10);
                PyTuple_SET_ITEM(__pyx_t_19, 0+1, __pyx_t_10);
                __pyx_t_10 = 0;
                __pyx_t_7 = __Pyx_PyObject_Call(__pyx_t_6, __pyx_t_19, NULL); if (unlikely(!__pyx_t_7)) __PYX_ERR(0, 89, __pyx_L38_error)
                __Pyx_GOTREF(__pyx_t_7);
                __Pyx_DECREF(__pyx_t_19); __pyx_t_19 = 0;
              }
            }
            __Pyx_DECREF(__pyx_t_6); __pyx_t_6 = 0;
            __Pyx_DECREF(__pyx_t_7); __pyx_t_7 = 0;
          }

          /* "src/python/grpcio/grpc/_cython/_cygrpc/fork_posix.pyx.pxi":87
 * 
 *     if grpc_is_initialized() > 0:
 *         with gil:             # <<<<<<<<<<<<<<
 *             _LOGGER.error('Failed to shutdown gRPC Core after fork()')
 *             os._exit(os.EX_USAGE)
 */
          /*finally:*/ {
            /*normal exit:*/{
              #ifdef WITH_THREAD
              __Pyx_PyGILState_Release(__pyx_gilstate_save);
              #endif
              goto __pyx_L39;
            }
            __pyx_L38_error: {
              #ifdef WITH_THREAD
              __Pyx_PyGILState_Release(__pyx_gilstate_save);
              #endif
              goto __pyx_L4_error;
            }
            __pyx_L39:;
          }
      }

      /* "src/python/grpcio/grpc/_cython/_cygrpc/fork_posix.pyx.pxi":86
 *             os._exit(os.EX_USAGE)
 * 
 *     if grpc_is_initialized() > 0:             # <<<<<<<<<<<<<<
 *         with gil:
 *             _LOGGER.error('Failed to shutdown gRPC Core after fork()')
 */
    }
  }

  /* "src/python/grpcio/grpc/_cython/_cygrpc/fork_posix.pyx.pxi":63
 * 
 * cdef void __postfork_child() nogil:
 *     with gil:             # <<<<<<<<<<<<<<
 *         try:
 *             if _fork_handler_failed:
 */
  /*finally:*/ {
    /*normal exit:*/{
      #ifdef WITH_THREAD
      __pyx_gilstate_save = __Pyx_PyGILState_Ensure();
      #endif
      goto __pyx_L5;
    }
    __pyx_L3_return: {
      #ifdef WITH_THREAD
      __pyx_gilstate_save = __Pyx_PyGILState_Ensure();
      #endif
      goto __pyx_L0;
    }
    __pyx_L4_error: {
      #ifdef WITH_THREAD
      __pyx_gilstate_save = __Pyx_PyGILState_Ensure();
      #endif
      goto __pyx_L1_error;
    }
    __pyx_L5:;
  }

  /* "src/python/grpcio/grpc/_cython/_cygrpc/fork_posix.pyx.pxi":62
 * 
 * 
 * cdef void __postfork_child() nogil:             # <<<<<<<<<<<<<<
 *     with gil:
 *         try:
 */

  /* function exit code */
  goto __pyx_L0;
  __pyx_L1_error:;
  __Pyx_XDECREF(__pyx_t_4);
  __Pyx_XDECREF(__pyx_t_6);
  __Pyx_XDECREF(__pyx_t_7);
  __Pyx_XDECREF(__pyx_t_10);
  __Pyx_XDECREF(__pyx_t_17);
  __Pyx_XDECREF(__pyx_t_18);
  __Pyx_XDECREF(__pyx_t_19);
  __Pyx_XDECREF(__pyx_t_20);
  __Pyx_WriteUnraisable("grpc._cython.cygrpc.__postfork_child", __pyx_clineno, __pyx_lineno, __pyx_filename, 1, 1);
  __pyx_L0:;
  __Pyx_XDECREF(__pyx_v_state_to_reset);
  __Pyx_XDECREF(__pyx_v_channel);
  #ifdef WITH_THREAD
  __Pyx_PyGILState_Release(__pyx_gilstate_save);
  #endif
}

/* "src/python/grpcio/grpc/_cython/_cygrpc/fork_posix.pyx.pxi":92
 * 
 * 
 * def fork_handlers_and_grpc_init():             # <<<<<<<<<<<<<<
 *     grpc_init()
 *     if _GRPC_ENABLE_FORK_SUPPORT:
 */

/* Python wrapper */
static PyObject *__pyx_pw_4grpc_7_cython_6cygrpc_63fork_handlers_and_grpc_init(PyObject *__pyx_self, CYTHON_UNUSED PyObject *unused); /*proto*/
static PyMethodDef __pyx_mdef_4grpc_7_cython_6cygrpc_63fork_handlers_and_grpc_init = {"fork_handlers_and_grpc_init", (PyCFunction)__pyx_pw_4grpc_7_cython_6cygrpc_63fork_handlers_and_grpc_init, METH_NOARGS, 0};
static PyObject *__pyx_pw_4grpc_7_cython_6cygrpc_63fork_handlers_and_grpc_init(PyObject *__pyx_self, CYTHON_UNUSED PyObject *unused) {
  PyObject *__pyx_r = 0;
  __Pyx_RefNannyDeclarations
  __Pyx_RefNannySetupContext("fork_handlers_and_grpc_init (wrapper)", 0);
  __pyx_r = __pyx_pf_4grpc_7_cython_6cygrpc_62fork_handlers_and_grpc_init(__pyx_self);

  /* function exit code */
  __Pyx_RefNannyFinishContext();
  return __pyx_r;
}

static PyObject *__pyx_pf_4grpc_7_cython_6cygrpc_62fork_handlers_and_grpc_init(CYTHON_UNUSED PyObject *__pyx_self) {
  PyObject *__pyx_r = NULL;
  __Pyx_RefNannyDeclarations
  PyObject *__pyx_t_1 = NULL;
  int __pyx_t_2;
  PyObject *__pyx_t_3 = NULL;
  PyObject *__pyx_t_4 = NULL;
  PyObject *__pyx_t_5 = NULL;
  PyObject *__pyx_t_6 = NULL;
  PyObject *__pyx_t_7 = NULL;
  PyObject *__pyx_t_8 = NULL;
  PyObject *__pyx_t_9 = NULL;
  int __pyx_t_10;
  PyObject *__pyx_t_11 = NULL;
  __Pyx_RefNannySetupContext("fork_handlers_and_grpc_init", 0);

  /* "src/python/grpcio/grpc/_cython/_cygrpc/fork_posix.pyx.pxi":93
 * 
 * def fork_handlers_and_grpc_init():
 *     grpc_init()             # <<<<<<<<<<<<<<
 *     if _GRPC_ENABLE_FORK_SUPPORT:
 *         with _fork_state.fork_handler_registered_lock:
 */
  grpc_init();

  /* "src/python/grpcio/grpc/_cython/_cygrpc/fork_posix.pyx.pxi":94
 * def fork_handlers_and_grpc_init():
 *     grpc_init()
 *     if _GRPC_ENABLE_FORK_SUPPORT:             # <<<<<<<<<<<<<<
 *         with _fork_state.fork_handler_registered_lock:
 *             if not _fork_state.fork_handler_registered:
 */
  __pyx_t_1 = __Pyx_GetModuleGlobalName(__pyx_n_s_GRPC_ENABLE_FORK_SUPPORT); if (unlikely(!__pyx_t_1)) __PYX_ERR(0, 94, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_1);
  __pyx_t_2 = __Pyx_PyObject_IsTrue(__pyx_t_1); if (unlikely(__pyx_t_2 < 0)) __PYX_ERR(0, 94, __pyx_L1_error)
  __Pyx_DECREF(__pyx_t_1); __pyx_t_1 = 0;
  if (__pyx_t_2) {

    /* "src/python/grpcio/grpc/_cython/_cygrpc/fork_posix.pyx.pxi":95
 *     grpc_init()
 *     if _GRPC_ENABLE_FORK_SUPPORT:
 *         with _fork_state.fork_handler_registered_lock:             # <<<<<<<<<<<<<<
 *             if not _fork_state.fork_handler_registered:
 *                 pthread_atfork(&__prefork, &__postfork_parent, &__postfork_child)
 */
    /*with:*/ {
      __pyx_t_1 = __Pyx_GetModuleGlobalName(__pyx_n_s_fork_state); if (unlikely(!__pyx_t_1)) __PYX_ERR(0, 95, __pyx_L1_error)
      __Pyx_GOTREF(__pyx_t_1);
      __pyx_t_3 = __Pyx_PyObject_GetAttrStr(__pyx_t_1, __pyx_n_s_fork_handler_registered_lock); if (unlikely(!__pyx_t_3)) __PYX_ERR(0, 95, __pyx_L1_error)
      __Pyx_GOTREF(__pyx_t_3);
      __Pyx_DECREF(__pyx_t_1); __pyx_t_1 = 0;
      __pyx_t_4 = __Pyx_PyObject_LookupSpecial(__pyx_t_3, __pyx_n_s_exit); if (unlikely(!__pyx_t_4)) __PYX_ERR(0, 95, __pyx_L1_error)
      __Pyx_GOTREF(__pyx_t_4);
      __pyx_t_5 = __Pyx_PyObject_LookupSpecial(__pyx_t_3, __pyx_n_s_enter); if (unlikely(!__pyx_t_5)) __PYX_ERR(0, 95, __pyx_L4_error)
      __Pyx_GOTREF(__pyx_t_5);
      __pyx_t_6 = NULL;
      if (CYTHON_UNPACK_METHODS && likely(PyMethod_Check(__pyx_t_5))) {
        __pyx_t_6 = PyMethod_GET_SELF(__pyx_t_5);
        if (likely(__pyx_t_6)) {
          PyObject* function = PyMethod_GET_FUNCTION(__pyx_t_5);
          __Pyx_INCREF(__pyx_t_6);
          __Pyx_INCREF(function);
          __Pyx_DECREF_SET(__pyx_t_5, function);
        }
      }
      if (__pyx_t_6) {
        __pyx_t_1 = __Pyx_PyObject_CallOneArg(__pyx_t_5, __pyx_t_6); if (unlikely(!__pyx_t_1)) __PYX_ERR(0, 95, __pyx_L4_error)
        __Pyx_DECREF(__pyx_t_6); __pyx_t_6 = 0;
      } else {
        __pyx_t_1 = __Pyx_PyObject_CallNoArg(__pyx_t_5); if (unlikely(!__pyx_t_1)) __PYX_ERR(0, 95, __pyx_L4_error)
      }
      __Pyx_GOTREF(__pyx_t_1);
      __Pyx_DECREF(__pyx_t_5); __pyx_t_5 = 0;
      __Pyx_DECREF(__pyx_t_1); __pyx_t_1 = 0;
      __Pyx_DECREF(__pyx_t_3); __pyx_t_3 = 0;
      /*try:*/ {
        {
          __Pyx_PyThreadState_declare
          __Pyx_PyThreadState_assign
          __Pyx_ExceptionSave(&__pyx_t_7, &__pyx_t_8, &__pyx_t_9);
          __Pyx_XGOTREF(__pyx_t_7);
          __Pyx_XGOTREF(__pyx_t_8);
          __Pyx_XGOTREF(__pyx_t_9);
          /*try:*/ {

            /* "src/python/grpcio/grpc/_cython/_cygrpc/fork_posix.pyx.pxi":96
 *     if _GRPC_ENABLE_FORK_SUPPORT:
 *         with _fork_state.fork_handler_registered_lock:
 *             if not _fork_state.fork_handler_registered:             # <<<<<<<<<<<<<<
 *                 pthread_atfork(&__prefork, &__postfork_parent, &__postfork_child)
 *                 _fork_state.fork_handler_registered = True
 */
            __pyx_t_3 = __Pyx_GetModuleGlobalName(__pyx_n_s_fork_state); if (unlikely(!__pyx_t_3)) __PYX_ERR(0, 96, __pyx_L8_error)
            __Pyx_GOTREF(__pyx_t_3);
            __pyx_t_1 = __Pyx_PyObject_GetAttrStr(__pyx_t_3, __pyx_n_s_fork_handler_registered); if (unlikely(!__pyx_t_1)) __PYX_ERR(0, 96, __pyx_L8_error)
            __Pyx_GOTREF(__pyx_t_1);
            __Pyx_DECREF(__pyx_t_3); __pyx_t_3 = 0;
            __pyx_t_2 = __Pyx_PyObject_IsTrue(__pyx_t_1); if (unlikely(__pyx_t_2 < 0)) __PYX_ERR(0, 96, __pyx_L8_error)
            __Pyx_DECREF(__pyx_t_1); __pyx_t_1 = 0;
            __pyx_t_10 = ((!__pyx_t_2) != 0);
            if (__pyx_t_10) {

              /* "src/python/grpcio/grpc/_cython/_cygrpc/fork_posix.pyx.pxi":97
 *         with _fork_state.fork_handler_registered_lock:
 *             if not _fork_state.fork_handler_registered:
 *                 pthread_atfork(&__prefork, &__postfork_parent, &__postfork_child)             # <<<<<<<<<<<<<<
 *                 _fork_state.fork_handler_registered = True
 * 
 */
              (void)(pthread_atfork((&__pyx_f_4grpc_7_cython_6cygrpc___prefork), (&__pyx_f_4grpc_7_cython_6cygrpc___postfork_parent), (&__pyx_f_4grpc_7_cython_6cygrpc___postfork_child)));

              /* "src/python/grpcio/grpc/_cython/_cygrpc/fork_posix.pyx.pxi":98
 *             if not _fork_state.fork_handler_registered:
 *                 pthread_atfork(&__prefork, &__postfork_parent, &__postfork_child)
 *                 _fork_state.fork_handler_registered = True             # <<<<<<<<<<<<<<
 * 
 * 
 */
              __pyx_t_1 = __Pyx_GetModuleGlobalName(__pyx_n_s_fork_state); if (unlikely(!__pyx_t_1)) __PYX_ERR(0, 98, __pyx_L8_error)
              __Pyx_GOTREF(__pyx_t_1);
              if (__Pyx_PyObject_SetAttrStr(__pyx_t_1, __pyx_n_s_fork_handler_registered, Py_True) < 0) __PYX_ERR(0, 98, __pyx_L8_error)
              __Pyx_DECREF(__pyx_t_1); __pyx_t_1 = 0;

              /* "src/python/grpcio/grpc/_cython/_cygrpc/fork_posix.pyx.pxi":96
 *     if _GRPC_ENABLE_FORK_SUPPORT:
 *         with _fork_state.fork_handler_registered_lock:
 *             if not _fork_state.fork_handler_registered:             # <<<<<<<<<<<<<<
 *                 pthread_atfork(&__prefork, &__postfork_parent, &__postfork_child)
 *                 _fork_state.fork_handler_registered = True
 */
            }

            /* "src/python/grpcio/grpc/_cython/_cygrpc/fork_posix.pyx.pxi":95
 *     grpc_init()
 *     if _GRPC_ENABLE_FORK_SUPPORT:
 *         with _fork_state.fork_handler_registered_lock:             # <<<<<<<<<<<<<<
 *             if not _fork_state.fork_handler_registered:
 *                 pthread_atfork(&__prefork, &__postfork_parent, &__postfork_child)
 */
          }
          __Pyx_XDECREF(__pyx_t_7); __pyx_t_7 = 0;
          __Pyx_XDECREF(__pyx_t_8); __pyx_t_8 = 0;
          __Pyx_XDECREF(__pyx_t_9); __pyx_t_9 = 0;
          goto __pyx_L13_try_end;
          __pyx_L8_error:;
          __Pyx_XDECREF(__pyx_t_6); __pyx_t_6 = 0;
          __Pyx_XDECREF(__pyx_t_5); __pyx_t_5 = 0;
          __Pyx_XDECREF(__pyx_t_3); __pyx_t_3 = 0;
          __Pyx_XDECREF(__pyx_t_1); __pyx_t_1 = 0;
          /*except:*/ {
            __Pyx_AddTraceback("grpc._cython.cygrpc.fork_handlers_and_grpc_init", __pyx_clineno, __pyx_lineno, __pyx_filename);
            if (__Pyx_GetException(&__pyx_t_1, &__pyx_t_3, &__pyx_t_5) < 0) __PYX_ERR(0, 95, __pyx_L10_except_error)
            __Pyx_GOTREF(__pyx_t_1);
            __Pyx_GOTREF(__pyx_t_3);
            __Pyx_GOTREF(__pyx_t_5);
            __pyx_t_6 = PyTuple_Pack(3, __pyx_t_1, __pyx_t_3, __pyx_t_5); if (unlikely(!__pyx_t_6)) __PYX_ERR(0, 95, __pyx_L10_except_error)
            __Pyx_GOTREF(__pyx_t_6);
            __pyx_t_11 = __Pyx_PyObject_Call(__pyx_t_4, __pyx_t_6, NULL);
            __Pyx_DECREF(__pyx_t_4); __pyx_t_4 = 0;
            __Pyx_DECREF(__pyx_t_6); __pyx_t_6 = 0;
            if (unlikely(!__pyx_t_11)) __PYX_ERR(0, 95, __pyx_L10_except_error)
            __Pyx_GOTREF(__pyx_t_11);
            __pyx_t_10 = __Pyx_PyObject_IsTrue(__pyx_t_11);
            __Pyx_DECREF(__pyx_t_11); __pyx_t_11 = 0;
            if (__pyx_t_10 < 0) __PYX_ERR(0, 95, __pyx_L10_except_error)
            __pyx_t_2 = ((!(__pyx_t_10 != 0)) != 0);
            if (__pyx_t_2) {
              __Pyx_GIVEREF(__pyx_t_1);
              __Pyx_GIVEREF(__pyx_t_3);
              __Pyx_XGIVEREF(__pyx_t_5);
              __Pyx_ErrRestoreWithState(__pyx_t_1, __pyx_t_3, __pyx_t_5);
              __pyx_t_1 = 0; __pyx_t_3 = 0; __pyx_t_5 = 0; 
              __PYX_ERR(0, 95, __pyx_L10_except_error)
            }
            __Pyx_DECREF(__pyx_t_1); __pyx_t_1 = 0;
            __Pyx_DECREF(__pyx_t_3); __pyx_t_3 = 0;
            __Pyx_DECREF(__pyx_t_5); __pyx_t_5 = 0;
            goto __pyx_L9_exception_handled;
          }
          __pyx_L10_except_error:;
          __Pyx_XGIVEREF(__pyx_t_7);
          __Pyx_XGIVEREF(__pyx_t_8);
          __Pyx_XGIVEREF(__pyx_t_9);
          __Pyx_ExceptionReset(__pyx_t_7, __pyx_t_8, __pyx_t_9);
          goto __pyx_L1_error;
          __pyx_L9_exception_handled:;
          __Pyx_XGIVEREF(__pyx_t_7);
          __Pyx_XGIVEREF(__pyx_t_8);
          __Pyx_XGIVEREF(__pyx_t_9);
          __Pyx_ExceptionReset(__pyx_t_7, __pyx_t_8, __pyx_t_9);
          __pyx_L13_try_end:;
        }
      }
      /*finally:*/ {
        /*normal exit:*/{
          if (__pyx_t_4) {
            __pyx_t_9 = __Pyx_PyObject_Call(__pyx_t_4, __pyx_tuple__154, NULL);
            __Pyx_DECREF(__pyx_t_4); __pyx_t_4 = 0;
            if (unlikely(!__pyx_t_9)) __PYX_ERR(0, 95, __pyx_L1_error)
            __Pyx_GOTREF(__pyx_t_9);
            __Pyx_DECREF(__pyx_t_9); __pyx_t_9 = 0;
          }
          goto __pyx_L7;
        }
        __pyx_L7:;
      }
      goto __pyx_L18;
      __pyx_L4_error:;
      __Pyx_DECREF(__pyx_t_4); __pyx_t_4 = 0;
      goto __pyx_L1_error;
      __pyx_L18:;
    }

    /* "src/python/grpcio/grpc/_cython/_cygrpc/fork_posix.pyx.pxi":94
 * def fork_handlers_and_grpc_init():
 *     grpc_init()
 *     if _GRPC_ENABLE_FORK_SUPPORT:             # <<<<<<<<<<<<<<
 *         with _fork_state.fork_handler_registered_lock:
 *             if not _fork_state.fork_handler_registered:
 */
  }

  /* "src/python/grpcio/grpc/_cython/_cygrpc/fork_posix.pyx.pxi":92
 * 
 * 
 * def fork_handlers_and_grpc_init():             # <<<<<<<<<<<<<<
 *     grpc_init()
 *     if _GRPC_ENABLE_FORK_SUPPORT:
 */

  /* function exit code */
  __pyx_r = Py_None; __Pyx_INCREF(Py_None);
  goto __pyx_L0;
  __pyx_L1_error:;
  __Pyx_XDECREF(__pyx_t_1);
  __Pyx_XDECREF(__pyx_t_3);
  __Pyx_XDECREF(__pyx_t_5);
  __Pyx_XDECREF(__pyx_t_6);
  __Pyx_AddTraceback("grpc._cython.cygrpc.fork_handlers_and_grpc_init", __pyx_clineno, __pyx_lineno, __pyx_filename);
  __pyx_r = NULL;
  __pyx_L0:;
  __Pyx_XGIVEREF(__pyx_r);
  __Pyx_RefNannyFinishContext();
  return __pyx_r;
}

/* "src/python/grpcio/grpc/_cython/_cygrpc/fork_posix.pyx.pxi":102
 * 
 * class ForkManagedThread(object):
 *     def __init__(self, target, args=()):             # <<<<<<<<<<<<<<
 *         if _GRPC_ENABLE_FORK_SUPPORT:
 *             def managed_target(*args):
 */

/* Python wrapper */
static PyObject *__pyx_pw_4grpc_7_cython_6cygrpc_17ForkManagedThread_1__init__(PyObject *__pyx_self, PyObject *__pyx_args, PyObject *__pyx_kwds); /*proto*/
static PyMethodDef __pyx_mdef_4grpc_7_cython_6cygrpc_17ForkManagedThread_1__init__ = {"__init__", (PyCFunction)__pyx_pw_4grpc_7_cython_6cygrpc_17ForkManagedThread_1__init__, METH_VARARGS|METH_KEYWORDS, 0};
static PyObject *__pyx_pw_4grpc_7_cython_6cygrpc_17ForkManagedThread_1__init__(PyObject *__pyx_self, PyObject *__pyx_args, PyObject *__pyx_kwds) {
  PyObject *__pyx_v_self = 0;
  PyObject *__pyx_v_target = 0;
  PyObject *__pyx_v_args = 0;
  PyObject *__pyx_r = 0;
  __Pyx_RefNannyDeclarations
  __Pyx_RefNannySetupContext("__init__ (wrapper)", 0);
  {
    static PyObject **__pyx_pyargnames[] = {&__pyx_n_s_self,&__pyx_n_s_target,&__pyx_n_s_args,0};
    PyObject* values[3] = {0,0,0};
    values[2] = ((PyObject *)((PyObject*)__pyx_empty_tuple));
    if (unlikely(__pyx_kwds)) {
      Py_ssize_t kw_args;
      const Py_ssize_t pos_args = PyTuple_GET_SIZE(__pyx_args);
      switch (pos_args) {
        case  3: values[2] = PyTuple_GET_ITEM(__pyx_args, 2);
        CYTHON_FALLTHROUGH;
        case  2: values[1] = PyTuple_GET_ITEM(__pyx_args, 1);
        CYTHON_FALLTHROUGH;
        case  1: values[0] = PyTuple_GET_ITEM(__pyx_args, 0);
        CYTHON_FALLTHROUGH;
        case  0: break;
        default: goto __pyx_L5_argtuple_error;
      }
      kw_args = PyDict_Size(__pyx_kwds);
      switch (pos_args) {
        case  0:
        if (likely((values[0] = __Pyx_PyDict_GetItemStr(__pyx_kwds, __pyx_n_s_self)) != 0)) kw_args--;
        else goto __pyx_L5_argtuple_error;
        CYTHON_FALLTHROUGH;
        case  1:
        if (likely((values[1] = __Pyx_PyDict_GetItemStr(__pyx_kwds, __pyx_n_s_target)) != 0)) kw_args--;
        else {
          __Pyx_RaiseArgtupleInvalid("__init__", 0, 2, 3, 1); __PYX_ERR(0, 102, __pyx_L3_error)
        }
        CYTHON_FALLTHROUGH;
        case  2:
        if (kw_args > 0) {
          PyObject* value = __Pyx_PyDict_GetItemStr(__pyx_kwds, __pyx_n_s_args);
          if (value) { values[2] = value; kw_args--; }
        }
      }
      if (unlikely(kw_args > 0)) {
        if (unlikely(__Pyx_ParseOptionalKeywords(__pyx_kwds, __pyx_pyargnames, 0, values, pos_args, "__init__") < 0)) __PYX_ERR(0, 102, __pyx_L3_error)
      }
    } else {
      switch (PyTuple_GET_SIZE(__pyx_args)) {
        case  3: values[2] = PyTuple_GET_ITEM(__pyx_args, 2);
        CYTHON_FALLTHROUGH;
        case  2: values[1] = PyTuple_GET_ITEM(__pyx_args, 1);
        values[0] = PyTuple_GET_ITEM(__pyx_args, 0);
        break;
        default: goto __pyx_L5_argtuple_error;
      }
    }
    __pyx_v_self = values[0];
    __pyx_v_target = values[1];
    __pyx_v_args = values[2];
  }
  goto __pyx_L4_argument_unpacking_done;
  __pyx_L5_argtuple_error:;
  __Pyx_RaiseArgtupleInvalid("__init__", 0, 2, 3, PyTuple_GET_SIZE(__pyx_args)); __PYX_ERR(0, 102, __pyx_L3_error)
  __pyx_L3_error:;
  __Pyx_AddTraceback("grpc._cython.cygrpc.ForkManagedThread.__init__", __pyx_clineno, __pyx_lineno, __pyx_filename);
  __Pyx_RefNannyFinishContext();
  return NULL;
  __pyx_L4_argument_unpacking_done:;
  __pyx_r = __pyx_pf_4grpc_7_cython_6cygrpc_17ForkManagedThread___init__(__pyx_self, __pyx_v_self, __pyx_v_target, __pyx_v_args);

  /* function exit code */
  __Pyx_RefNannyFinishContext();
  return __pyx_r;
}

/* "src/python/grpcio/grpc/_cython/_cygrpc/fork_posix.pyx.pxi":104
 *     def __init__(self, target, args=()):
 *         if _GRPC_ENABLE_FORK_SUPPORT:
 *             def managed_target(*args):             # <<<<<<<<<<<<<<
 *                 try:
 *                     target(*args)
 */

/* Python wrapper */
static PyObject *__pyx_pw_4grpc_7_cython_6cygrpc_17ForkManagedThread_8__init___1managed_target(PyObject *__pyx_self, PyObject *__pyx_args, PyObject *__pyx_kwds); /*proto*/
static PyMethodDef __pyx_mdef_4grpc_7_cython_6cygrpc_17ForkManagedThread_8__init___1managed_target = {"managed_target", (PyCFunction)__pyx_pw_4grpc_7_cython_6cygrpc_17ForkManagedThread_8__init___1managed_target, METH_VARARGS|METH_KEYWORDS, 0};
static PyObject *__pyx_pw_4grpc_7_cython_6cygrpc_17ForkManagedThread_8__init___1managed_target(PyObject *__pyx_self, PyObject *__pyx_args, PyObject *__pyx_kwds) {
  PyObject *__pyx_v_args = 0;
  PyObject *__pyx_r = 0;
  __Pyx_RefNannyDeclarations
  __Pyx_RefNannySetupContext("managed_target (wrapper)", 0);
  if (unlikely(__pyx_kwds) && unlikely(PyDict_Size(__pyx_kwds) > 0) && unlikely(!__Pyx_CheckKeywordStrings(__pyx_kwds, "managed_target", 0))) return NULL;
  __Pyx_INCREF(__pyx_args);
  __pyx_v_args = __pyx_args;
  __pyx_r = __pyx_pf_4grpc_7_cython_6cygrpc_17ForkManagedThread_8__init___managed_target(__pyx_self, __pyx_v_args);

  /* function exit code */
  __Pyx_XDECREF(__pyx_v_args);
  __Pyx_RefNannyFinishContext();
  return __pyx_r;
}

static PyObject *__pyx_pf_4grpc_7_cython_6cygrpc_17ForkManagedThread_8__init___managed_target(PyObject *__pyx_self, PyObject *__pyx_v_args) {
  struct __pyx_obj_4grpc_7_cython_6cygrpc___pyx_scope_struct_9___init__ *__pyx_cur_scope;
  struct __pyx_obj_4grpc_7_cython_6cygrpc___pyx_scope_struct_9___init__ *__pyx_outer_scope;
  PyObject *__pyx_r = NULL;
  __Pyx_RefNannyDeclarations
  PyObject *__pyx_t_1 = NULL;
  PyObject *__pyx_t_2 = NULL;
  PyObject *__pyx_t_3 = NULL;
  int __pyx_t_4;
  int __pyx_t_5;
  char const *__pyx_t_6;
  PyObject *__pyx_t_7 = NULL;
  PyObject *__pyx_t_8 = NULL;
  PyObject *__pyx_t_9 = NULL;
  PyObject *__pyx_t_10 = NULL;
  PyObject *__pyx_t_11 = NULL;
  PyObject *__pyx_t_12 = NULL;
  __Pyx_RefNannySetupContext("managed_target", 0);
  __pyx_outer_scope = (struct __pyx_obj_4grpc_7_cython_6cygrpc___pyx_scope_struct_9___init__ *) __Pyx_CyFunction_GetClosure(__pyx_self);
  __pyx_cur_scope = __pyx_outer_scope;

  /* "src/python/grpcio/grpc/_cython/_cygrpc/fork_posix.pyx.pxi":105
 *         if _GRPC_ENABLE_FORK_SUPPORT:
 *             def managed_target(*args):
 *                 try:             # <<<<<<<<<<<<<<
 *                     target(*args)
 *                 finally:
 */
  /*try:*/ {

    /* "src/python/grpcio/grpc/_cython/_cygrpc/fork_posix.pyx.pxi":106
 *             def managed_target(*args):
 *                 try:
 *                     target(*args)             # <<<<<<<<<<<<<<
 *                 finally:
 *                     _fork_state.active_thread_count.decrement()
 */
    if (unlikely(!__pyx_cur_scope->__pyx_v_target)) { __Pyx_RaiseClosureNameError("target"); __PYX_ERR(0, 106, __pyx_L4_error) }
    __pyx_t_1 = __Pyx_PyObject_Call(__pyx_cur_scope->__pyx_v_target, __pyx_v_args, NULL); if (unlikely(!__pyx_t_1)) __PYX_ERR(0, 106, __pyx_L4_error)
    __Pyx_GOTREF(__pyx_t_1);
    __Pyx_DECREF(__pyx_t_1); __pyx_t_1 = 0;
  }

  /* "src/python/grpcio/grpc/_cython/_cygrpc/fork_posix.pyx.pxi":108
 *                     target(*args)
 *                 finally:
 *                     _fork_state.active_thread_count.decrement()             # <<<<<<<<<<<<<<
 *             self._thread = threading.Thread(target=managed_target, args=args)
 *         else:
 */
  /*finally:*/ {
    /*normal exit:*/{
      __pyx_t_2 = __Pyx_GetModuleGlobalName(__pyx_n_s_fork_state); if (unlikely(!__pyx_t_2)) __PYX_ERR(0, 108, __pyx_L1_error)
      __Pyx_GOTREF(__pyx_t_2);
      __pyx_t_3 = __Pyx_PyObject_GetAttrStr(__pyx_t_2, __pyx_n_s_active_thread_count); if (unlikely(!__pyx_t_3)) __PYX_ERR(0, 108, __pyx_L1_error)
      __Pyx_GOTREF(__pyx_t_3);
      __Pyx_DECREF(__pyx_t_2); __pyx_t_2 = 0;
      __pyx_t_2 = __Pyx_PyObject_GetAttrStr(__pyx_t_3, __pyx_n_s_decrement); if (unlikely(!__pyx_t_2)) __PYX_ERR(0, 108, __pyx_L1_error)
      __Pyx_GOTREF(__pyx_t_2);
      __Pyx_DECREF(__pyx_t_3); __pyx_t_3 = 0;
      __pyx_t_3 = NULL;
      if (CYTHON_UNPACK_METHODS && likely(PyMethod_Check(__pyx_t_2))) {
        __pyx_t_3 = PyMethod_GET_SELF(__pyx_t_2);
        if (likely(__pyx_t_3)) {
          PyObject* function = PyMethod_GET_FUNCTION(__pyx_t_2);
          __Pyx_INCREF(__pyx_t_3);
          __Pyx_INCREF(function);
          __Pyx_DECREF_SET(__pyx_t_2, function);
        }
      }
      if (__pyx_t_3) {
        __pyx_t_1 = __Pyx_PyObject_CallOneArg(__pyx_t_2, __pyx_t_3); if (unlikely(!__pyx_t_1)) __PYX_ERR(0, 108, __pyx_L1_error)
        __Pyx_DECREF(__pyx_t_3); __pyx_t_3 = 0;
      } else {
        __pyx_t_1 = __Pyx_PyObject_CallNoArg(__pyx_t_2); if (unlikely(!__pyx_t_1)) __PYX_ERR(0, 108, __pyx_L1_error)
      }
      __Pyx_GOTREF(__pyx_t_1);
      __Pyx_DECREF(__pyx_t_2); __pyx_t_2 = 0;
      __Pyx_DECREF(__pyx_t_1); __pyx_t_1 = 0;
      goto __pyx_L5;
    }
    __pyx_L4_error:;
    /*exception exit:*/{
      __Pyx_PyThreadState_declare
      __Pyx_PyThreadState_assign
      __pyx_t_7 = 0; __pyx_t_8 = 0; __pyx_t_9 = 0; __pyx_t_10 = 0; __pyx_t_11 = 0; __pyx_t_12 = 0;
      __Pyx_XDECREF(__pyx_t_1); __pyx_t_1 = 0;
      if (PY_MAJOR_VERSION >= 3) __Pyx_ExceptionSwap(&__pyx_t_10, &__pyx_t_11, &__pyx_t_12);
      if ((PY_MAJOR_VERSION < 3) || unlikely(__Pyx_GetException(&__pyx_t_7, &__pyx_t_8, &__pyx_t_9) < 0)) __Pyx_ErrFetch(&__pyx_t_7, &__pyx_t_8, &__pyx_t_9);
      __Pyx_XGOTREF(__pyx_t_7);
      __Pyx_XGOTREF(__pyx_t_8);
      __Pyx_XGOTREF(__pyx_t_9);
      __Pyx_XGOTREF(__pyx_t_10);
      __Pyx_XGOTREF(__pyx_t_11);
      __Pyx_XGOTREF(__pyx_t_12);
      __pyx_t_4 = __pyx_lineno; __pyx_t_5 = __pyx_clineno; __pyx_t_6 = __pyx_filename;
      {
        __pyx_t_2 = __Pyx_GetModuleGlobalName(__pyx_n_s_fork_state); if (unlikely(!__pyx_t_2)) __PYX_ERR(0, 108, __pyx_L7_error)
        __Pyx_GOTREF(__pyx_t_2);
        __pyx_t_3 = __Pyx_PyObject_GetAttrStr(__pyx_t_2, __pyx_n_s_active_thread_count); if (unlikely(!__pyx_t_3)) __PYX_ERR(0, 108, __pyx_L7_error)
        __Pyx_GOTREF(__pyx_t_3);
        __Pyx_DECREF(__pyx_t_2); __pyx_t_2 = 0;
        __pyx_t_2 = __Pyx_PyObject_GetAttrStr(__pyx_t_3, __pyx_n_s_decrement); if (unlikely(!__pyx_t_2)) __PYX_ERR(0, 108, __pyx_L7_error)
        __Pyx_GOTREF(__pyx_t_2);
        __Pyx_DECREF(__pyx_t_3); __pyx_t_3 = 0;
        __pyx_t_3 = NULL;
        if (CYTHON_UNPACK_METHODS && likely(PyMethod_Check(__pyx_t_2))) {
          __pyx_t_3 = PyMethod_GET_SELF(__pyx_t_2);
          if (likely(__pyx_t_3)) {
            PyObject* function = PyMethod_GET_FUNCTION(__pyx_t_2);
            __Pyx_INCREF(__pyx_t_3);
            __Pyx_INCREF(function);
            __Pyx_DECREF_SET(__pyx_t_2, function);
          }
        }
        if (__pyx_t_3) {
          __pyx_t_1 = __Pyx_PyObject_CallOneArg(__pyx_t_2, __pyx_t_3); if (unlikely(!__pyx_t_1)) __PYX_ERR(0, 108, __pyx_L7_error)
          __Pyx_DECREF(__pyx_t_3); __pyx_t_3 = 0;
        } else {
          __pyx_t_1 = __Pyx_PyObject_CallNoArg(__pyx_t_2); if (unlikely(!__pyx_t_1)) __PYX_ERR(0, 108, __pyx_L7_error)
        }
        __Pyx_GOTREF(__pyx_t_1);
        __Pyx_DECREF(__pyx_t_2); __pyx_t_2 = 0;
        __Pyx_DECREF(__pyx_t_1); __pyx_t_1 = 0;
      }
      if (PY_MAJOR_VERSION >= 3) {
        __Pyx_XGIVEREF(__pyx_t_10);
        __Pyx_XGIVEREF(__pyx_t_11);
        __Pyx_XGIVEREF(__pyx_t_12);
        __Pyx_ExceptionReset(__pyx_t_10, __pyx_t_11, __pyx_t_12);
      }
      __Pyx_XGIVEREF(__pyx_t_7);
      __Pyx_XGIVEREF(__pyx_t_8);
      __Pyx_XGIVEREF(__pyx_t_9);
      __Pyx_ErrRestore(__pyx_t_7, __pyx_t_8, __pyx_t_9);
      __pyx_t_7 = 0; __pyx_t_8 = 0; __pyx_t_9 = 0; __pyx_t_10 = 0; __pyx_t_11 = 0; __pyx_t_12 = 0;
      __pyx_lineno = __pyx_t_4; __pyx_clineno = __pyx_t_5; __pyx_filename = __pyx_t_6;
      goto __pyx_L1_error;
      __pyx_L7_error:;
      if (PY_MAJOR_VERSION >= 3) {
        __Pyx_XGIVEREF(__pyx_t_10);
        __Pyx_XGIVEREF(__pyx_t_11);
        __Pyx_XGIVEREF(__pyx_t_12);
        __Pyx_ExceptionReset(__pyx_t_10, __pyx_t_11, __pyx_t_12);
      }
      __Pyx_XDECREF(__pyx_t_7); __pyx_t_7 = 0;
      __Pyx_XDECREF(__pyx_t_8); __pyx_t_8 = 0;
      __Pyx_XDECREF(__pyx_t_9); __pyx_t_9 = 0;
      __pyx_t_10 = 0; __pyx_t_11 = 0; __pyx_t_12 = 0;
      goto __pyx_L1_error;
    }
    __pyx_L5:;
  }

  /* "src/python/grpcio/grpc/_cython/_cygrpc/fork_posix.pyx.pxi":104
 *     def __init__(self, target, args=()):
 *         if _GRPC_ENABLE_FORK_SUPPORT:
 *             def managed_target(*args):             # <<<<<<<<<<<<<<
 *                 try:
 *                     target(*args)
 */

  /* function exit code */
  __pyx_r = Py_None; __Pyx_INCREF(Py_None);
  goto __pyx_L0;
  __pyx_L1_error:;
  __Pyx_XDECREF(__pyx_t_1);
  __Pyx_XDECREF(__pyx_t_2);
  __Pyx_XDECREF(__pyx_t_3);
  __Pyx_AddTraceback("grpc._cython.cygrpc.ForkManagedThread.__init__.managed_target", __pyx_clineno, __pyx_lineno, __pyx_filename);
  __pyx_r = NULL;
  __pyx_L0:;
  __Pyx_XGIVEREF(__pyx_r);
  __Pyx_RefNannyFinishContext();
  return __pyx_r;
}

/* "src/python/grpcio/grpc/_cython/_cygrpc/fork_posix.pyx.pxi":102
 * 
 * class ForkManagedThread(object):
 *     def __init__(self, target, args=()):             # <<<<<<<<<<<<<<
 *         if _GRPC_ENABLE_FORK_SUPPORT:
 *             def managed_target(*args):
 */

static PyObject *__pyx_pf_4grpc_7_cython_6cygrpc_17ForkManagedThread___init__(CYTHON_UNUSED PyObject *__pyx_self, PyObject *__pyx_v_self, PyObject *__pyx_v_target, PyObject *__pyx_v_args) {
  struct __pyx_obj_4grpc_7_cython_6cygrpc___pyx_scope_struct_9___init__ *__pyx_cur_scope;
  PyObject *__pyx_v_managed_target = 0;
  PyObject *__pyx_r = NULL;
  __Pyx_RefNannyDeclarations
  PyObject *__pyx_t_1 = NULL;
  int __pyx_t_2;
  PyObject *__pyx_t_3 = NULL;
  PyObject *__pyx_t_4 = NULL;
  __Pyx_RefNannySetupContext("__init__", 0);
  __pyx_cur_scope = (struct __pyx_obj_4grpc_7_cython_6cygrpc___pyx_scope_struct_9___init__ *)__pyx_tp_new_4grpc_7_cython_6cygrpc___pyx_scope_struct_9___init__(__pyx_ptype_4grpc_7_cython_6cygrpc___pyx_scope_struct_9___init__, __pyx_empty_tuple, NULL);
  if (unlikely(!__pyx_cur_scope)) {
    __pyx_cur_scope = ((struct __pyx_obj_4grpc_7_cython_6cygrpc___pyx_scope_struct_9___init__ *)Py_None);
    __Pyx_INCREF(Py_None);
    __PYX_ERR(0, 102, __pyx_L1_error)
  } else {
    __Pyx_GOTREF(__pyx_cur_scope);
  }
  __pyx_cur_scope->__pyx_v_target = __pyx_v_target;
  __Pyx_INCREF(__pyx_cur_scope->__pyx_v_target);
  __Pyx_GIVEREF(__pyx_cur_scope->__pyx_v_target);

  /* "src/python/grpcio/grpc/_cython/_cygrpc/fork_posix.pyx.pxi":103
 * class ForkManagedThread(object):
 *     def __init__(self, target, args=()):
 *         if _GRPC_ENABLE_FORK_SUPPORT:             # <<<<<<<<<<<<<<
 *             def managed_target(*args):
 *                 try:
 */
  __pyx_t_1 = __Pyx_GetModuleGlobalName(__pyx_n_s_GRPC_ENABLE_FORK_SUPPORT); if (unlikely(!__pyx_t_1)) __PYX_ERR(0, 103, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_1);
  __pyx_t_2 = __Pyx_PyObject_IsTrue(__pyx_t_1); if (unlikely(__pyx_t_2 < 0)) __PYX_ERR(0, 103, __pyx_L1_error)
  __Pyx_DECREF(__pyx_t_1); __pyx_t_1 = 0;
  if (__pyx_t_2) {

    /* "src/python/grpcio/grpc/_cython/_cygrpc/fork_posix.pyx.pxi":104
 *     def __init__(self, target, args=()):
 *         if _GRPC_ENABLE_FORK_SUPPORT:
 *             def managed_target(*args):             # <<<<<<<<<<<<<<
 *                 try:
 *                     target(*args)
 */
    __pyx_t_1 = __Pyx_CyFunction_NewEx(&__pyx_mdef_4grpc_7_cython_6cygrpc_17ForkManagedThread_8__init___1managed_target, 0, __pyx_n_s_ForkManagedThread___init___local, ((PyObject*)__pyx_cur_scope), __pyx_n_s_grpc__cython_cygrpc, __pyx_d, ((PyObject *)__pyx_codeobj__156)); if (unlikely(!__pyx_t_1)) __PYX_ERR(0, 104, __pyx_L1_error)
    __Pyx_GOTREF(__pyx_t_1);
    __pyx_v_managed_target = __pyx_t_1;
    __pyx_t_1 = 0;

    /* "src/python/grpcio/grpc/_cython/_cygrpc/fork_posix.pyx.pxi":109
 *                 finally:
 *                     _fork_state.active_thread_count.decrement()
 *             self._thread = threading.Thread(target=managed_target, args=args)             # <<<<<<<<<<<<<<
 *         else:
 *             self._thread = threading.Thread(target=target, args=args)
 */
    __pyx_t_1 = __Pyx_GetModuleGlobalName(__pyx_n_s_threading); if (unlikely(!__pyx_t_1)) __PYX_ERR(0, 109, __pyx_L1_error)
    __Pyx_GOTREF(__pyx_t_1);
    __pyx_t_3 = __Pyx_PyObject_GetAttrStr(__pyx_t_1, __pyx_n_s_Thread); if (unlikely(!__pyx_t_3)) __PYX_ERR(0, 109, __pyx_L1_error)
    __Pyx_GOTREF(__pyx_t_3);
    __Pyx_DECREF(__pyx_t_1); __pyx_t_1 = 0;
    __pyx_t_1 = __Pyx_PyDict_NewPresized(2); if (unlikely(!__pyx_t_1)) __PYX_ERR(0, 109, __pyx_L1_error)
    __Pyx_GOTREF(__pyx_t_1);
    if (PyDict_SetItem(__pyx_t_1, __pyx_n_s_target, __pyx_v_managed_target) < 0) __PYX_ERR(0, 109, __pyx_L1_error)
    if (PyDict_SetItem(__pyx_t_1, __pyx_n_s_args, __pyx_v_args) < 0) __PYX_ERR(0, 109, __pyx_L1_error)
    __pyx_t_4 = __Pyx_PyObject_Call(__pyx_t_3, __pyx_empty_tuple, __pyx_t_1); if (unlikely(!__pyx_t_4)) __PYX_ERR(0, 109, __pyx_L1_error)
    __Pyx_GOTREF(__pyx_t_4);
    __Pyx_DECREF(__pyx_t_3); __pyx_t_3 = 0;
    __Pyx_DECREF(__pyx_t_1); __pyx_t_1 = 0;
    if (__Pyx_PyObject_SetAttrStr(__pyx_v_self, __pyx_n_s_thread, __pyx_t_4) < 0) __PYX_ERR(0, 109, __pyx_L1_error)
    __Pyx_DECREF(__pyx_t_4); __pyx_t_4 = 0;

    /* "src/python/grpcio/grpc/_cython/_cygrpc/fork_posix.pyx.pxi":103
 * class ForkManagedThread(object):
 *     def __init__(self, target, args=()):
 *         if _GRPC_ENABLE_FORK_SUPPORT:             # <<<<<<<<<<<<<<
 *             def managed_target(*args):
 *                 try:
 */
    goto __pyx_L3;
  }

  /* "src/python/grpcio/grpc/_cython/_cygrpc/fork_posix.pyx.pxi":111
 *             self._thread = threading.Thread(target=managed_target, args=args)
 *         else:
 *             self._thread = threading.Thread(target=target, args=args)             # <<<<<<<<<<<<<<
 * 
 *     def setDaemon(self, daemonic):
 */
  /*else*/ {
    __pyx_t_4 = __Pyx_GetModuleGlobalName(__pyx_n_s_threading); if (unlikely(!__pyx_t_4)) __PYX_ERR(0, 111, __pyx_L1_error)
    __Pyx_GOTREF(__pyx_t_4);
    __pyx_t_1 = __Pyx_PyObject_GetAttrStr(__pyx_t_4, __pyx_n_s_Thread); if (unlikely(!__pyx_t_1)) __PYX_ERR(0, 111, __pyx_L1_error)
    __Pyx_GOTREF(__pyx_t_1);
    __Pyx_DECREF(__pyx_t_4); __pyx_t_4 = 0;
    __pyx_t_4 = __Pyx_PyDict_NewPresized(2); if (unlikely(!__pyx_t_4)) __PYX_ERR(0, 111, __pyx_L1_error)
    __Pyx_GOTREF(__pyx_t_4);
    if (PyDict_SetItem(__pyx_t_4, __pyx_n_s_target, __pyx_cur_scope->__pyx_v_target) < 0) __PYX_ERR(0, 111, __pyx_L1_error)
    if (PyDict_SetItem(__pyx_t_4, __pyx_n_s_args, __pyx_v_args) < 0) __PYX_ERR(0, 111, __pyx_L1_error)
    __pyx_t_3 = __Pyx_PyObject_Call(__pyx_t_1, __pyx_empty_tuple, __pyx_t_4); if (unlikely(!__pyx_t_3)) __PYX_ERR(0, 111, __pyx_L1_error)
    __Pyx_GOTREF(__pyx_t_3);
    __Pyx_DECREF(__pyx_t_1); __pyx_t_1 = 0;
    __Pyx_DECREF(__pyx_t_4); __pyx_t_4 = 0;
    if (__Pyx_PyObject_SetAttrStr(__pyx_v_self, __pyx_n_s_thread, __pyx_t_3) < 0) __PYX_ERR(0, 111, __pyx_L1_error)
    __Pyx_DECREF(__pyx_t_3); __pyx_t_3 = 0;
  }
  __pyx_L3:;

  /* "src/python/grpcio/grpc/_cython/_cygrpc/fork_posix.pyx.pxi":102
 * 
 * class ForkManagedThread(object):
 *     def __init__(self, target, args=()):             # <<<<<<<<<<<<<<
 *         if _GRPC_ENABLE_FORK_SUPPORT:
 *             def managed_target(*args):
 */

  /* function exit code */
  __pyx_r = Py_None; __Pyx_INCREF(Py_None);
  goto __pyx_L0;
  __pyx_L1_error:;
  __Pyx_XDECREF(__pyx_t_1);
  __Pyx_XDECREF(__pyx_t_3);
  __Pyx_XDECREF(__pyx_t_4);
  __Pyx_AddTraceback("grpc._cython.cygrpc.ForkManagedThread.__init__", __pyx_clineno, __pyx_lineno, __pyx_filename);
  __pyx_r = NULL;
  __pyx_L0:;
  __Pyx_XDECREF(__pyx_v_managed_target);
  __Pyx_DECREF(((PyObject *)__pyx_cur_scope));
  __Pyx_XGIVEREF(__pyx_r);
  __Pyx_RefNannyFinishContext();
  return __pyx_r;
}

/* "src/python/grpcio/grpc/_cython/_cygrpc/fork_posix.pyx.pxi":113
 *             self._thread = threading.Thread(target=target, args=args)
 * 
 *     def setDaemon(self, daemonic):             # <<<<<<<<<<<<<<
 *         self._thread.daemon = daemonic
 * 
 */

/* Python wrapper */
static PyObject *__pyx_pw_4grpc_7_cython_6cygrpc_17ForkManagedThread_3setDaemon(PyObject *__pyx_self, PyObject *__pyx_args, PyObject *__pyx_kwds); /*proto*/
static PyMethodDef __pyx_mdef_4grpc_7_cython_6cygrpc_17ForkManagedThread_3setDaemon = {"setDaemon", (PyCFunction)__pyx_pw_4grpc_7_cython_6cygrpc_17ForkManagedThread_3setDaemon, METH_VARARGS|METH_KEYWORDS, 0};
static PyObject *__pyx_pw_4grpc_7_cython_6cygrpc_17ForkManagedThread_3setDaemon(PyObject *__pyx_self, PyObject *__pyx_args, PyObject *__pyx_kwds) {
  PyObject *__pyx_v_self = 0;
  PyObject *__pyx_v_daemonic = 0;
  PyObject *__pyx_r = 0;
  __Pyx_RefNannyDeclarations
  __Pyx_RefNannySetupContext("setDaemon (wrapper)", 0);
  {
    static PyObject **__pyx_pyargnames[] = {&__pyx_n_s_self,&__pyx_n_s_daemonic,0};
    PyObject* values[2] = {0,0};
    if (unlikely(__pyx_kwds)) {
      Py_ssize_t kw_args;
      const Py_ssize_t pos_args = PyTuple_GET_SIZE(__pyx_args);
      switch (pos_args) {
        case  2: values[1] = PyTuple_GET_ITEM(__pyx_args, 1);
        CYTHON_FALLTHROUGH;
        case  1: values[0] = PyTuple_GET_ITEM(__pyx_args, 0);
        CYTHON_FALLTHROUGH;
        case  0: break;
        default: goto __pyx_L5_argtuple_error;
      }
      kw_args = PyDict_Size(__pyx_kwds);
      switch (pos_args) {
        case  0:
        if (likely((values[0] = __Pyx_PyDict_GetItemStr(__pyx_kwds, __pyx_n_s_self)) != 0)) kw_args--;
        else goto __pyx_L5_argtuple_error;
        CYTHON_FALLTHROUGH;
        case  1:
        if (likely((values[1] = __Pyx_PyDict_GetItemStr(__pyx_kwds, __pyx_n_s_daemonic)) != 0)) kw_args--;
        else {
          __Pyx_RaiseArgtupleInvalid("setDaemon", 1, 2, 2, 1); __PYX_ERR(0, 113, __pyx_L3_error)
        }
      }
      if (unlikely(kw_args > 0)) {
        if (unlikely(__Pyx_ParseOptionalKeywords(__pyx_kwds, __pyx_pyargnames, 0, values, pos_args, "setDaemon") < 0)) __PYX_ERR(0, 113, __pyx_L3_error)
      }
    } else if (PyTuple_GET_SIZE(__pyx_args) != 2) {
      goto __pyx_L5_argtuple_error;
    } else {
      values[0] = PyTuple_GET_ITEM(__pyx_args, 0);
      values[1] = PyTuple_GET_ITEM(__pyx_args, 1);
    }
    __pyx_v_self = values[0];
    __pyx_v_daemonic = values[1];
  }
  goto __pyx_L4_argument_unpacking_done;
  __pyx_L5_argtuple_error:;
  __Pyx_RaiseArgtupleInvalid("setDaemon", 1, 2, 2, PyTuple_GET_SIZE(__pyx_args)); __PYX_ERR(0, 113, __pyx_L3_error)
  __pyx_L3_error:;
  __Pyx_AddTraceback("grpc._cython.cygrpc.ForkManagedThread.setDaemon", __pyx_clineno, __pyx_lineno, __pyx_filename);
  __Pyx_RefNannyFinishContext();
  return NULL;
  __pyx_L4_argument_unpacking_done:;
  __pyx_r = __pyx_pf_4grpc_7_cython_6cygrpc_17ForkManagedThread_2setDaemon(__pyx_self, __pyx_v_self, __pyx_v_daemonic);

  /* function exit code */
  __Pyx_RefNannyFinishContext();
  return __pyx_r;
}

static PyObject *__pyx_pf_4grpc_7_cython_6cygrpc_17ForkManagedThread_2setDaemon(CYTHON_UNUSED PyObject *__pyx_self, PyObject *__pyx_v_self, PyObject *__pyx_v_daemonic) {
  PyObject *__pyx_r = NULL;
  __Pyx_RefNannyDeclarations
  PyObject *__pyx_t_1 = NULL;
  __Pyx_RefNannySetupContext("setDaemon", 0);

  /* "src/python/grpcio/grpc/_cython/_cygrpc/fork_posix.pyx.pxi":114
 * 
 *     def setDaemon(self, daemonic):
 *         self._thread.daemon = daemonic             # <<<<<<<<<<<<<<
 * 
 *     def start(self):
 */
  __pyx_t_1 = __Pyx_PyObject_GetAttrStr(__pyx_v_self, __pyx_n_s_thread); if (unlikely(!__pyx_t_1)) __PYX_ERR(0, 114, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_1);
  if (__Pyx_PyObject_SetAttrStr(__pyx_t_1, __pyx_n_s_daemon, __pyx_v_daemonic) < 0) __PYX_ERR(0, 114, __pyx_L1_error)
  __Pyx_DECREF(__pyx_t_1); __pyx_t_1 = 0;

  /* "src/python/grpcio/grpc/_cython/_cygrpc/fork_posix.pyx.pxi":113
 *             self._thread = threading.Thread(target=target, args=args)
 * 
 *     def setDaemon(self, daemonic):             # <<<<<<<<<<<<<<
 *         self._thread.daemon = daemonic
 * 
 */

  /* function exit code */
  __pyx_r = Py_None; __Pyx_INCREF(Py_None);
  goto __pyx_L0;
  __pyx_L1_error:;
  __Pyx_XDECREF(__pyx_t_1);
  __Pyx_AddTraceback("grpc._cython.cygrpc.ForkManagedThread.setDaemon", __pyx_clineno, __pyx_lineno, __pyx_filename);
  __pyx_r = NULL;
  __pyx_L0:;
  __Pyx_XGIVEREF(__pyx_r);
  __Pyx_RefNannyFinishContext();
  return __pyx_r;
}

/* "src/python/grpcio/grpc/_cython/_cygrpc/fork_posix.pyx.pxi":116
 *         self._thread.daemon = daemonic
 * 
 *     def start(self):             # <<<<<<<<<<<<<<
 *         if _GRPC_ENABLE_FORK_SUPPORT:
 *             _fork_state.active_thread_count.increment()
 */

/* Python wrapper */
static PyObject *__pyx_pw_4grpc_7_cython_6cygrpc_17ForkManagedThread_5start(PyObject *__pyx_self, PyObject *__pyx_v_self); /*proto*/
static PyMethodDef __pyx_mdef_4grpc_7_cython_6cygrpc_17ForkManagedThread_5start = {"start", (PyCFunction)__pyx_pw_4grpc_7_cython_6cygrpc_17ForkManagedThread_5start, METH_O, 0};
static PyObject *__pyx_pw_4grpc_7_cython_6cygrpc_17ForkManagedThread_5start(PyObject *__pyx_self, PyObject *__pyx_v_self) {
  PyObject *__pyx_r = 0;
  __Pyx_RefNannyDeclarations
  __Pyx_RefNannySetupContext("start (wrapper)", 0);
  __pyx_r = __pyx_pf_4grpc_7_cython_6cygrpc_17ForkManagedThread_4start(__pyx_self, ((PyObject *)__pyx_v_self));

  /* function exit code */
  __Pyx_RefNannyFinishContext();
  return __pyx_r;
}

static PyObject *__pyx_pf_4grpc_7_cython_6cygrpc_17ForkManagedThread_4start(CYTHON_UNUSED PyObject *__pyx_self, PyObject *__pyx_v_self) {
  PyObject *__pyx_r = NULL;
  __Pyx_RefNannyDeclarations
  PyObject *__pyx_t_1 = NULL;
  int __pyx_t_2;
  PyObject *__pyx_t_3 = NULL;
  PyObject *__pyx_t_4 = NULL;
  __Pyx_RefNannySetupContext("start", 0);

  /* "src/python/grpcio/grpc/_cython/_cygrpc/fork_posix.pyx.pxi":117
 * 
 *     def start(self):
 *         if _GRPC_ENABLE_FORK_SUPPORT:             # <<<<<<<<<<<<<<
 *             _fork_state.active_thread_count.increment()
 *         self._thread.start()
 */
  __pyx_t_1 = __Pyx_GetModuleGlobalName(__pyx_n_s_GRPC_ENABLE_FORK_SUPPORT); if (unlikely(!__pyx_t_1)) __PYX_ERR(0, 117, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_1);
  __pyx_t_2 = __Pyx_PyObject_IsTrue(__pyx_t_1); if (unlikely(__pyx_t_2 < 0)) __PYX_ERR(0, 117, __pyx_L1_error)
  __Pyx_DECREF(__pyx_t_1); __pyx_t_1 = 0;
  if (__pyx_t_2) {

    /* "src/python/grpcio/grpc/_cython/_cygrpc/fork_posix.pyx.pxi":118
 *     def start(self):
 *         if _GRPC_ENABLE_FORK_SUPPORT:
 *             _fork_state.active_thread_count.increment()             # <<<<<<<<<<<<<<
 *         self._thread.start()
 * 
 */
    __pyx_t_3 = __Pyx_GetModuleGlobalName(__pyx_n_s_fork_state); if (unlikely(!__pyx_t_3)) __PYX_ERR(0, 118, __pyx_L1_error)
    __Pyx_GOTREF(__pyx_t_3);
    __pyx_t_4 = __Pyx_PyObject_GetAttrStr(__pyx_t_3, __pyx_n_s_active_thread_count); if (unlikely(!__pyx_t_4)) __PYX_ERR(0, 118, __pyx_L1_error)
    __Pyx_GOTREF(__pyx_t_4);
    __Pyx_DECREF(__pyx_t_3); __pyx_t_3 = 0;
    __pyx_t_3 = __Pyx_PyObject_GetAttrStr(__pyx_t_4, __pyx_n_s_increment); if (unlikely(!__pyx_t_3)) __PYX_ERR(0, 118, __pyx_L1_error)
    __Pyx_GOTREF(__pyx_t_3);
    __Pyx_DECREF(__pyx_t_4); __pyx_t_4 = 0;
    __pyx_t_4 = NULL;
    if (CYTHON_UNPACK_METHODS && likely(PyMethod_Check(__pyx_t_3))) {
      __pyx_t_4 = PyMethod_GET_SELF(__pyx_t_3);
      if (likely(__pyx_t_4)) {
        PyObject* function = PyMethod_GET_FUNCTION(__pyx_t_3);
        __Pyx_INCREF(__pyx_t_4);
        __Pyx_INCREF(function);
        __Pyx_DECREF_SET(__pyx_t_3, function);
      }
    }
    if (__pyx_t_4) {
      __pyx_t_1 = __Pyx_PyObject_CallOneArg(__pyx_t_3, __pyx_t_4); if (unlikely(!__pyx_t_1)) __PYX_ERR(0, 118, __pyx_L1_error)
      __Pyx_DECREF(__pyx_t_4); __pyx_t_4 = 0;
    } else {
      __pyx_t_1 = __Pyx_PyObject_CallNoArg(__pyx_t_3); if (unlikely(!__pyx_t_1)) __PYX_ERR(0, 118, __pyx_L1_error)
    }
    __Pyx_GOTREF(__pyx_t_1);
    __Pyx_DECREF(__pyx_t_3); __pyx_t_3 = 0;
    __Pyx_DECREF(__pyx_t_1); __pyx_t_1 = 0;

    /* "src/python/grpcio/grpc/_cython/_cygrpc/fork_posix.pyx.pxi":117
 * 
 *     def start(self):
 *         if _GRPC_ENABLE_FORK_SUPPORT:             # <<<<<<<<<<<<<<
 *             _fork_state.active_thread_count.increment()
 *         self._thread.start()
 */
  }

  /* "src/python/grpcio/grpc/_cython/_cygrpc/fork_posix.pyx.pxi":119
 *         if _GRPC_ENABLE_FORK_SUPPORT:
 *             _fork_state.active_thread_count.increment()
 *         self._thread.start()             # <<<<<<<<<<<<<<
 * 
 *     def join(self):
 */
  __pyx_t_3 = __Pyx_PyObject_GetAttrStr(__pyx_v_self, __pyx_n_s_thread); if (unlikely(!__pyx_t_3)) __PYX_ERR(0, 119, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_3);
  __pyx_t_4 = __Pyx_PyObject_GetAttrStr(__pyx_t_3, __pyx_n_s_start); if (unlikely(!__pyx_t_4)) __PYX_ERR(0, 119, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_4);
  __Pyx_DECREF(__pyx_t_3); __pyx_t_3 = 0;
  __pyx_t_3 = NULL;
  if (CYTHON_UNPACK_METHODS && likely(PyMethod_Check(__pyx_t_4))) {
    __pyx_t_3 = PyMethod_GET_SELF(__pyx_t_4);
    if (likely(__pyx_t_3)) {
      PyObject* function = PyMethod_GET_FUNCTION(__pyx_t_4);
      __Pyx_INCREF(__pyx_t_3);
      __Pyx_INCREF(function);
      __Pyx_DECREF_SET(__pyx_t_4, function);
    }
  }
  if (__pyx_t_3) {
    __pyx_t_1 = __Pyx_PyObject_CallOneArg(__pyx_t_4, __pyx_t_3); if (unlikely(!__pyx_t_1)) __PYX_ERR(0, 119, __pyx_L1_error)
    __Pyx_DECREF(__pyx_t_3); __pyx_t_3 = 0;
  } else {
    __pyx_t_1 = __Pyx_PyObject_CallNoArg(__pyx_t_4); if (unlikely(!__pyx_t_1)) __PYX_ERR(0, 119, __pyx_L1_error)
  }
  __Pyx_GOTREF(__pyx_t_1);
  __Pyx_DECREF(__pyx_t_4); __pyx_t_4 = 0;
  __Pyx_DECREF(__pyx_t_1); __pyx_t_1 = 0;

  /* "src/python/grpcio/grpc/_cython/_cygrpc/fork_posix.pyx.pxi":116
 *         self._thread.daemon = daemonic
 * 
 *     def start(self):             # <<<<<<<<<<<<<<
 *         if _GRPC_ENABLE_FORK_SUPPORT:
 *             _fork_state.active_thread_count.increment()
 */

  /* function exit code */
  __pyx_r = Py_None; __Pyx_INCREF(Py_None);
  goto __pyx_L0;
  __pyx_L1_error:;
  __Pyx_XDECREF(__pyx_t_1);
  __Pyx_XDECREF(__pyx_t_3);
  __Pyx_XDECREF(__pyx_t_4);
  __Pyx_AddTraceback("grpc._cython.cygrpc.ForkManagedThread.start", __pyx_clineno, __pyx_lineno, __pyx_filename);
  __pyx_r = NULL;
  __pyx_L0:;
  __Pyx_XGIVEREF(__pyx_r);
  __Pyx_RefNannyFinishContext();
  return __pyx_r;
}

/* "src/python/grpcio/grpc/_cython/_cygrpc/fork_posix.pyx.pxi":121
 *         self._thread.start()
 * 
 *     def join(self):             # <<<<<<<<<<<<<<
 *         self._thread.join()
 * 
 */

/* Python wrapper */
static PyObject *__pyx_pw_4grpc_7_cython_6cygrpc_17ForkManagedThread_7join(PyObject *__pyx_self, PyObject *__pyx_v_self); /*proto*/
static PyMethodDef __pyx_mdef_4grpc_7_cython_6cygrpc_17ForkManagedThread_7join = {"join", (PyCFunction)__pyx_pw_4grpc_7_cython_6cygrpc_17ForkManagedThread_7join, METH_O, 0};
static PyObject *__pyx_pw_4grpc_7_cython_6cygrpc_17ForkManagedThread_7join(PyObject *__pyx_self, PyObject *__pyx_v_self) {
  PyObject *__pyx_r = 0;
  __Pyx_RefNannyDeclarations
  __Pyx_RefNannySetupContext("join (wrapper)", 0);
  __pyx_r = __pyx_pf_4grpc_7_cython_6cygrpc_17ForkManagedThread_6join(__pyx_self, ((PyObject *)__pyx_v_self));

  /* function exit code */
  __Pyx_RefNannyFinishContext();
  return __pyx_r;
}

static PyObject *__pyx_pf_4grpc_7_cython_6cygrpc_17ForkManagedThread_6join(CYTHON_UNUSED PyObject *__pyx_self, PyObject *__pyx_v_self) {
  PyObject *__pyx_r = NULL;
  __Pyx_RefNannyDeclarations
  PyObject *__pyx_t_1 = NULL;
  PyObject *__pyx_t_2 = NULL;
  PyObject *__pyx_t_3 = NULL;
  __Pyx_RefNannySetupContext("join", 0);

  /* "src/python/grpcio/grpc/_cython/_cygrpc/fork_posix.pyx.pxi":122
 * 
 *     def join(self):
 *         self._thread.join()             # <<<<<<<<<<<<<<
 * 
 * 
 */
  __pyx_t_2 = __Pyx_PyObject_GetAttrStr(__pyx_v_self, __pyx_n_s_thread); if (unlikely(!__pyx_t_2)) __PYX_ERR(0, 122, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_2);
  __pyx_t_3 = __Pyx_PyObject_GetAttrStr(__pyx_t_2, __pyx_n_s_join); if (unlikely(!__pyx_t_3)) __PYX_ERR(0, 122, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_3);
  __Pyx_DECREF(__pyx_t_2); __pyx_t_2 = 0;
  __pyx_t_2 = NULL;
  if (CYTHON_UNPACK_METHODS && likely(PyMethod_Check(__pyx_t_3))) {
    __pyx_t_2 = PyMethod_GET_SELF(__pyx_t_3);
    if (likely(__pyx_t_2)) {
      PyObject* function = PyMethod_GET_FUNCTION(__pyx_t_3);
      __Pyx_INCREF(__pyx_t_2);
      __Pyx_INCREF(function);
      __Pyx_DECREF_SET(__pyx_t_3, function);
    }
  }
  if (__pyx_t_2) {
    __pyx_t_1 = __Pyx_PyObject_CallOneArg(__pyx_t_3, __pyx_t_2); if (unlikely(!__pyx_t_1)) __PYX_ERR(0, 122, __pyx_L1_error)
    __Pyx_DECREF(__pyx_t_2); __pyx_t_2 = 0;
  } else {
    __pyx_t_1 = __Pyx_PyObject_CallNoArg(__pyx_t_3); if (unlikely(!__pyx_t_1)) __PYX_ERR(0, 122, __pyx_L1_error)
  }
  __Pyx_GOTREF(__pyx_t_1);
  __Pyx_DECREF(__pyx_t_3); __pyx_t_3 = 0;
  __Pyx_DECREF(__pyx_t_1); __pyx_t_1 = 0;

  /* "src/python/grpcio/grpc/_cython/_cygrpc/fork_posix.pyx.pxi":121
 *         self._thread.start()
 * 
 *     def join(self):             # <<<<<<<<<<<<<<
 *         self._thread.join()
 * 
 */

  /* function exit code */
  __pyx_r = Py_None; __Pyx_INCREF(Py_None);
  goto __pyx_L0;
  __pyx_L1_error:;
  __Pyx_XDECREF(__pyx_t_1);
  __Pyx_XDECREF(__pyx_t_2);
  __Pyx_XDECREF(__pyx_t_3);
  __Pyx_AddTraceback("grpc._cython.cygrpc.ForkManagedThread.join", __pyx_clineno, __pyx_lineno, __pyx_filename);
  __pyx_r = NULL;
  __pyx_L0:;
  __Pyx_XGIVEREF(__pyx_r);
  __Pyx_RefNannyFinishContext();
  return __pyx_r;
}

/* "src/python/grpcio/grpc/_cython/_cygrpc/fork_posix.pyx.pxi":125
 * 
 * 
 * def block_if_fork_in_progress(postfork_state_to_reset=None):             # <<<<<<<<<<<<<<
 *     if _GRPC_ENABLE_FORK_SUPPORT:
 *         with _fork_state.fork_in_progress_condition:
 */

/* Python wrapper */
static PyObject *__pyx_pw_4grpc_7_cython_6cygrpc_65block_if_fork_in_progress(PyObject *__pyx_self, PyObject *__pyx_args, PyObject *__pyx_kwds); /*proto*/
static PyMethodDef __pyx_mdef_4grpc_7_cython_6cygrpc_65block_if_fork_in_progress = {"block_if_fork_in_progress", (PyCFunction)__pyx_pw_4grpc_7_cython_6cygrpc_65block_if_fork_in_progress, METH_VARARGS|METH_KEYWORDS, 0};
static PyObject *__pyx_pw_4grpc_7_cython_6cygrpc_65block_if_fork_in_progress(PyObject *__pyx_self, PyObject *__pyx_args, PyObject *__pyx_kwds) {
  PyObject *__pyx_v_postfork_state_to_reset = 0;
  PyObject *__pyx_r = 0;
  __Pyx_RefNannyDeclarations
  __Pyx_RefNannySetupContext("block_if_fork_in_progress (wrapper)", 0);
  {
    static PyObject **__pyx_pyargnames[] = {&__pyx_n_s_postfork_state_to_reset,0};
    PyObject* values[1] = {0};
    values[0] = ((PyObject *)Py_None);
    if (unlikely(__pyx_kwds)) {
      Py_ssize_t kw_args;
      const Py_ssize_t pos_args = PyTuple_GET_SIZE(__pyx_args);
      switch (pos_args) {
        case  1: values[0] = PyTuple_GET_ITEM(__pyx_args, 0);
        CYTHON_FALLTHROUGH;
        case  0: break;
        default: goto __pyx_L5_argtuple_error;
      }
      kw_args = PyDict_Size(__pyx_kwds);
      switch (pos_args) {
        case  0:
        if (kw_args > 0) {
          PyObject* value = __Pyx_PyDict_GetItemStr(__pyx_kwds, __pyx_n_s_postfork_state_to_reset);
          if (value) { values[0] = value; kw_args--; }
        }
      }
      if (unlikely(kw_args > 0)) {
        if (unlikely(__Pyx_ParseOptionalKeywords(__pyx_kwds, __pyx_pyargnames, 0, values, pos_args, "block_if_fork_in_progress") < 0)) __PYX_ERR(0, 125, __pyx_L3_error)
      }
    } else {
      switch (PyTuple_GET_SIZE(__pyx_args)) {
        case  1: values[0] = PyTuple_GET_ITEM(__pyx_args, 0);
        CYTHON_FALLTHROUGH;
        case  0: break;
        default: goto __pyx_L5_argtuple_error;
      }
    }
    __pyx_v_postfork_state_to_reset = values[0];
  }
  goto __pyx_L4_argument_unpacking_done;
  __pyx_L5_argtuple_error:;
  __Pyx_RaiseArgtupleInvalid("block_if_fork_in_progress", 0, 0, 1, PyTuple_GET_SIZE(__pyx_args)); __PYX_ERR(0, 125, __pyx_L3_error)
  __pyx_L3_error:;
  __Pyx_AddTraceback("grpc._cython.cygrpc.block_if_fork_in_progress", __pyx_clineno, __pyx_lineno, __pyx_filename);
  __Pyx_RefNannyFinishContext();
  return NULL;
  __pyx_L4_argument_unpacking_done:;
  __pyx_r = __pyx_pf_4grpc_7_cython_6cygrpc_64block_if_fork_in_progress(__pyx_self, __pyx_v_postfork_state_to_reset);

  /* function exit code */
  __Pyx_RefNannyFinishContext();
  return __pyx_r;
}

static PyObject *__pyx_pf_4grpc_7_cython_6cygrpc_64block_if_fork_in_progress(CYTHON_UNUSED PyObject *__pyx_self, PyObject *__pyx_v_postfork_state_to_reset) {
  PyObject *__pyx_r = NULL;
  __Pyx_RefNannyDeclarations
  PyObject *__pyx_t_1 = NULL;
  int __pyx_t_2;
  PyObject *__pyx_t_3 = NULL;
  PyObject *__pyx_t_4 = NULL;
  PyObject *__pyx_t_5 = NULL;
  PyObject *__pyx_t_6 = NULL;
  PyObject *__pyx_t_7 = NULL;
  PyObject *__pyx_t_8 = NULL;
  PyObject *__pyx_t_9 = NULL;
  int __pyx_t_10;
  int __pyx_t_11;
  PyObject *__pyx_t_12 = NULL;
  __Pyx_RefNannySetupContext("block_if_fork_in_progress", 0);

  /* "src/python/grpcio/grpc/_cython/_cygrpc/fork_posix.pyx.pxi":126
 * 
 * def block_if_fork_in_progress(postfork_state_to_reset=None):
 *     if _GRPC_ENABLE_FORK_SUPPORT:             # <<<<<<<<<<<<<<
 *         with _fork_state.fork_in_progress_condition:
 *             if not _fork_state.fork_in_progress:
 */
  __pyx_t_1 = __Pyx_GetModuleGlobalName(__pyx_n_s_GRPC_ENABLE_FORK_SUPPORT); if (unlikely(!__pyx_t_1)) __PYX_ERR(0, 126, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_1);
  __pyx_t_2 = __Pyx_PyObject_IsTrue(__pyx_t_1); if (unlikely(__pyx_t_2 < 0)) __PYX_ERR(0, 126, __pyx_L1_error)
  __Pyx_DECREF(__pyx_t_1); __pyx_t_1 = 0;
  if (__pyx_t_2) {

    /* "src/python/grpcio/grpc/_cython/_cygrpc/fork_posix.pyx.pxi":127
 * def block_if_fork_in_progress(postfork_state_to_reset=None):
 *     if _GRPC_ENABLE_FORK_SUPPORT:
 *         with _fork_state.fork_in_progress_condition:             # <<<<<<<<<<<<<<
 *             if not _fork_state.fork_in_progress:
 *                 return
 */
    /*with:*/ {
      __pyx_t_1 = __Pyx_GetModuleGlobalName(__pyx_n_s_fork_state); if (unlikely(!__pyx_t_1)) __PYX_ERR(0, 127, __pyx_L1_error)
      __Pyx_GOTREF(__pyx_t_1);
      __pyx_t_3 = __Pyx_PyObject_GetAttrStr(__pyx_t_1, __pyx_n_s_fork_in_progress_condition); if (unlikely(!__pyx_t_3)) __PYX_ERR(0, 127, __pyx_L1_error)
      __Pyx_GOTREF(__pyx_t_3);
      __Pyx_DECREF(__pyx_t_1); __pyx_t_1 = 0;
      __pyx_t_4 = __Pyx_PyObject_LookupSpecial(__pyx_t_3, __pyx_n_s_exit); if (unlikely(!__pyx_t_4)) __PYX_ERR(0, 127, __pyx_L1_error)
      __Pyx_GOTREF(__pyx_t_4);
      __pyx_t_5 = __Pyx_PyObject_LookupSpecial(__pyx_t_3, __pyx_n_s_enter); if (unlikely(!__pyx_t_5)) __PYX_ERR(0, 127, __pyx_L4_error)
      __Pyx_GOTREF(__pyx_t_5);
      __pyx_t_6 = NULL;
      if (CYTHON_UNPACK_METHODS && likely(PyMethod_Check(__pyx_t_5))) {
        __pyx_t_6 = PyMethod_GET_SELF(__pyx_t_5);
        if (likely(__pyx_t_6)) {
          PyObject* function = PyMethod_GET_FUNCTION(__pyx_t_5);
          __Pyx_INCREF(__pyx_t_6);
          __Pyx_INCREF(function);
          __Pyx_DECREF_SET(__pyx_t_5, function);
        }
      }
      if (__pyx_t_6) {
        __pyx_t_1 = __Pyx_PyObject_CallOneArg(__pyx_t_5, __pyx_t_6); if (unlikely(!__pyx_t_1)) __PYX_ERR(0, 127, __pyx_L4_error)
        __Pyx_DECREF(__pyx_t_6); __pyx_t_6 = 0;
      } else {
        __pyx_t_1 = __Pyx_PyObject_CallNoArg(__pyx_t_5); if (unlikely(!__pyx_t_1)) __PYX_ERR(0, 127, __pyx_L4_error)
      }
      __Pyx_GOTREF(__pyx_t_1);
      __Pyx_DECREF(__pyx_t_5); __pyx_t_5 = 0;
      __Pyx_DECREF(__pyx_t_1); __pyx_t_1 = 0;
      __Pyx_DECREF(__pyx_t_3); __pyx_t_3 = 0;
      /*try:*/ {
        {
          __Pyx_PyThreadState_declare
          __Pyx_PyThreadState_assign
          __Pyx_ExceptionSave(&__pyx_t_7, &__pyx_t_8, &__pyx_t_9);
          __Pyx_XGOTREF(__pyx_t_7);
          __Pyx_XGOTREF(__pyx_t_8);
          __Pyx_XGOTREF(__pyx_t_9);
          /*try:*/ {

            /* "src/python/grpcio/grpc/_cython/_cygrpc/fork_posix.pyx.pxi":128
 *     if _GRPC_ENABLE_FORK_SUPPORT:
 *         with _fork_state.fork_in_progress_condition:
 *             if not _fork_state.fork_in_progress:             # <<<<<<<<<<<<<<
 *                 return
 *             if postfork_state_to_reset is not None:
 */
            __pyx_t_3 = __Pyx_GetModuleGlobalName(__pyx_n_s_fork_state); if (unlikely(!__pyx_t_3)) __PYX_ERR(0, 128, __pyx_L8_error)
            __Pyx_GOTREF(__pyx_t_3);
            __pyx_t_1 = __Pyx_PyObject_GetAttrStr(__pyx_t_3, __pyx_n_s_fork_in_progress); if (unlikely(!__pyx_t_1)) __PYX_ERR(0, 128, __pyx_L8_error)
            __Pyx_GOTREF(__pyx_t_1);
            __Pyx_DECREF(__pyx_t_3); __pyx_t_3 = 0;
            __pyx_t_2 = __Pyx_PyObject_IsTrue(__pyx_t_1); if (unlikely(__pyx_t_2 < 0)) __PYX_ERR(0, 128, __pyx_L8_error)
            __Pyx_DECREF(__pyx_t_1); __pyx_t_1 = 0;
            __pyx_t_10 = ((!__pyx_t_2) != 0);
            if (__pyx_t_10) {

              /* "src/python/grpcio/grpc/_cython/_cygrpc/fork_posix.pyx.pxi":129
 *         with _fork_state.fork_in_progress_condition:
 *             if not _fork_state.fork_in_progress:
 *                 return             # <<<<<<<<<<<<<<
 *             if postfork_state_to_reset is not None:
 *                 _fork_state.postfork_states_to_reset.append(postfork_state_to_reset)
 */
              __Pyx_XDECREF(__pyx_r);
              __pyx_r = Py_None; __Pyx_INCREF(Py_None);
              goto __pyx_L12_try_return;

              /* "src/python/grpcio/grpc/_cython/_cygrpc/fork_posix.pyx.pxi":128
 *     if _GRPC_ENABLE_FORK_SUPPORT:
 *         with _fork_state.fork_in_progress_condition:
 *             if not _fork_state.fork_in_progress:             # <<<<<<<<<<<<<<
 *                 return
 *             if postfork_state_to_reset is not None:
 */
            }

            /* "src/python/grpcio/grpc/_cython/_cygrpc/fork_posix.pyx.pxi":130
 *             if not _fork_state.fork_in_progress:
 *                 return
 *             if postfork_state_to_reset is not None:             # <<<<<<<<<<<<<<
 *                 _fork_state.postfork_states_to_reset.append(postfork_state_to_reset)
 *             _fork_state.active_thread_count.decrement()
 */
            __pyx_t_10 = (__pyx_v_postfork_state_to_reset != Py_None);
            __pyx_t_2 = (__pyx_t_10 != 0);
            if (__pyx_t_2) {

              /* "src/python/grpcio/grpc/_cython/_cygrpc/fork_posix.pyx.pxi":131
 *                 return
 *             if postfork_state_to_reset is not None:
 *                 _fork_state.postfork_states_to_reset.append(postfork_state_to_reset)             # <<<<<<<<<<<<<<
 *             _fork_state.active_thread_count.decrement()
 *             _fork_state.fork_in_progress_condition.wait()
 */
              __pyx_t_1 = __Pyx_GetModuleGlobalName(__pyx_n_s_fork_state); if (unlikely(!__pyx_t_1)) __PYX_ERR(0, 131, __pyx_L8_error)
              __Pyx_GOTREF(__pyx_t_1);
              __pyx_t_3 = __Pyx_PyObject_GetAttrStr(__pyx_t_1, __pyx_n_s_postfork_states_to_reset); if (unlikely(!__pyx_t_3)) __PYX_ERR(0, 131, __pyx_L8_error)
              __Pyx_GOTREF(__pyx_t_3);
              __Pyx_DECREF(__pyx_t_1); __pyx_t_1 = 0;
              __pyx_t_11 = __Pyx_PyObject_Append(__pyx_t_3, __pyx_v_postfork_state_to_reset); if (unlikely(__pyx_t_11 == ((int)-1))) __PYX_ERR(0, 131, __pyx_L8_error)
              __Pyx_DECREF(__pyx_t_3); __pyx_t_3 = 0;

              /* "src/python/grpcio/grpc/_cython/_cygrpc/fork_posix.pyx.pxi":130
 *             if not _fork_state.fork_in_progress:
 *                 return
 *             if postfork_state_to_reset is not None:             # <<<<<<<<<<<<<<
 *                 _fork_state.postfork_states_to_reset.append(postfork_state_to_reset)
 *             _fork_state.active_thread_count.decrement()
 */
            }

            /* "src/python/grpcio/grpc/_cython/_cygrpc/fork_posix.pyx.pxi":132
 *             if postfork_state_to_reset is not None:
 *                 _fork_state.postfork_states_to_reset.append(postfork_state_to_reset)
 *             _fork_state.active_thread_count.decrement()             # <<<<<<<<<<<<<<
 *             _fork_state.fork_in_progress_condition.wait()
 *             _fork_state.active_thread_count.increment()
 */
            __pyx_t_1 = __Pyx_GetModuleGlobalName(__pyx_n_s_fork_state); if (unlikely(!__pyx_t_1)) __PYX_ERR(0, 132, __pyx_L8_error)
            __Pyx_GOTREF(__pyx_t_1);
            __pyx_t_5 = __Pyx_PyObject_GetAttrStr(__pyx_t_1, __pyx_n_s_active_thread_count); if (unlikely(!__pyx_t_5)) __PYX_ERR(0, 132, __pyx_L8_error)
            __Pyx_GOTREF(__pyx_t_5);
            __Pyx_DECREF(__pyx_t_1); __pyx_t_1 = 0;
            __pyx_t_1 = __Pyx_PyObject_GetAttrStr(__pyx_t_5, __pyx_n_s_decrement); if (unlikely(!__pyx_t_1)) __PYX_ERR(0, 132, __pyx_L8_error)
            __Pyx_GOTREF(__pyx_t_1);
            __Pyx_DECREF(__pyx_t_5); __pyx_t_5 = 0;
            __pyx_t_5 = NULL;
            if (CYTHON_UNPACK_METHODS && likely(PyMethod_Check(__pyx_t_1))) {
              __pyx_t_5 = PyMethod_GET_SELF(__pyx_t_1);
              if (likely(__pyx_t_5)) {
                PyObject* function = PyMethod_GET_FUNCTION(__pyx_t_1);
                __Pyx_INCREF(__pyx_t_5);
                __Pyx_INCREF(function);
                __Pyx_DECREF_SET(__pyx_t_1, function);
              }
            }
            if (__pyx_t_5) {
              __pyx_t_3 = __Pyx_PyObject_CallOneArg(__pyx_t_1, __pyx_t_5); if (unlikely(!__pyx_t_3)) __PYX_ERR(0, 132, __pyx_L8_error)
              __Pyx_DECREF(__pyx_t_5); __pyx_t_5 = 0;
            } else {
              __pyx_t_3 = __Pyx_PyObject_CallNoArg(__pyx_t_1); if (unlikely(!__pyx_t_3)) __PYX_ERR(0, 132, __pyx_L8_error)
            }
            __Pyx_GOTREF(__pyx_t_3);
            __Pyx_DECREF(__pyx_t_1); __pyx_t_1 = 0;
            __Pyx_DECREF(__pyx_t_3); __pyx_t_3 = 0;

            /* "src/python/grpcio/grpc/_cython/_cygrpc/fork_posix.pyx.pxi":133
 *                 _fork_state.postfork_states_to_reset.append(postfork_state_to_reset)
 *             _fork_state.active_thread_count.decrement()
 *             _fork_state.fork_in_progress_condition.wait()             # <<<<<<<<<<<<<<
 *             _fork_state.active_thread_count.increment()
 * 
 */
            __pyx_t_1 = __Pyx_GetModuleGlobalName(__pyx_n_s_fork_state); if (unlikely(!__pyx_t_1)) __PYX_ERR(0, 133, __pyx_L8_error)
            __Pyx_GOTREF(__pyx_t_1);
            __pyx_t_5 = __Pyx_PyObject_GetAttrStr(__pyx_t_1, __pyx_n_s_fork_in_progress_condition); if (unlikely(!__pyx_t_5)) __PYX_ERR(0, 133, __pyx_L8_error)
            __Pyx_GOTREF(__pyx_t_5);
            __Pyx_DECREF(__pyx_t_1); __pyx_t_1 = 0;
            __pyx_t_1 = __Pyx_PyObject_GetAttrStr(__pyx_t_5, __pyx_n_s_wait); if (unlikely(!__pyx_t_1)) __PYX_ERR(0, 133, __pyx_L8_error)
            __Pyx_GOTREF(__pyx_t_1);
            __Pyx_DECREF(__pyx_t_5); __pyx_t_5 = 0;
            __pyx_t_5 = NULL;
            if (CYTHON_UNPACK_METHODS && likely(PyMethod_Check(__pyx_t_1))) {
              __pyx_t_5 = PyMethod_GET_SELF(__pyx_t_1);
              if (likely(__pyx_t_5)) {
                PyObject* function = PyMethod_GET_FUNCTION(__pyx_t_1);
                __Pyx_INCREF(__pyx_t_5);
                __Pyx_INCREF(function);
                __Pyx_DECREF_SET(__pyx_t_1, function);
              }
            }
            if (__pyx_t_5) {
              __pyx_t_3 = __Pyx_PyObject_CallOneArg(__pyx_t_1, __pyx_t_5); if (unlikely(!__pyx_t_3)) __PYX_ERR(0, 133, __pyx_L8_error)
              __Pyx_DECREF(__pyx_t_5); __pyx_t_5 = 0;
            } else {
              __pyx_t_3 = __Pyx_PyObject_CallNoArg(__pyx_t_1); if (unlikely(!__pyx_t_3)) __PYX_ERR(0, 133, __pyx_L8_error)
            }
            __Pyx_GOTREF(__pyx_t_3);
            __Pyx_DECREF(__pyx_t_1); __pyx_t_1 = 0;
            __Pyx_DECREF(__pyx_t_3); __pyx_t_3 = 0;

            /* "src/python/grpcio/grpc/_cython/_cygrpc/fork_posix.pyx.pxi":134
 *             _fork_state.active_thread_count.decrement()
 *             _fork_state.fork_in_progress_condition.wait()
 *             _fork_state.active_thread_count.increment()             # <<<<<<<<<<<<<<
 * 
 * 
 */
            __pyx_t_1 = __Pyx_GetModuleGlobalName(__pyx_n_s_fork_state); if (unlikely(!__pyx_t_1)) __PYX_ERR(0, 134, __pyx_L8_error)
            __Pyx_GOTREF(__pyx_t_1);
            __pyx_t_5 = __Pyx_PyObject_GetAttrStr(__pyx_t_1, __pyx_n_s_active_thread_count); if (unlikely(!__pyx_t_5)) __PYX_ERR(0, 134, __pyx_L8_error)
            __Pyx_GOTREF(__pyx_t_5);
            __Pyx_DECREF(__pyx_t_1); __pyx_t_1 = 0;
            __pyx_t_1 = __Pyx_PyObject_GetAttrStr(__pyx_t_5, __pyx_n_s_increment); if (unlikely(!__pyx_t_1)) __PYX_ERR(0, 134, __pyx_L8_error)
            __Pyx_GOTREF(__pyx_t_1);
            __Pyx_DECREF(__pyx_t_5); __pyx_t_5 = 0;
            __pyx_t_5 = NULL;
            if (CYTHON_UNPACK_METHODS && likely(PyMethod_Check(__pyx_t_1))) {
              __pyx_t_5 = PyMethod_GET_SELF(__pyx_t_1);
              if (likely(__pyx_t_5)) {
                PyObject* function = PyMethod_GET_FUNCTION(__pyx_t_1);
                __Pyx_INCREF(__pyx_t_5);
                __Pyx_INCREF(function);
                __Pyx_DECREF_SET(__pyx_t_1, function);
              }
            }
            if (__pyx_t_5) {
              __pyx_t_3 = __Pyx_PyObject_CallOneArg(__pyx_t_1, __pyx_t_5); if (unlikely(!__pyx_t_3)) __PYX_ERR(0, 134, __pyx_L8_error)
              __Pyx_DECREF(__pyx_t_5); __pyx_t_5 = 0;
            } else {
              __pyx_t_3 = __Pyx_PyObject_CallNoArg(__pyx_t_1); if (unlikely(!__pyx_t_3)) __PYX_ERR(0, 134, __pyx_L8_error)
            }
            __Pyx_GOTREF(__pyx_t_3);
            __Pyx_DECREF(__pyx_t_1); __pyx_t_1 = 0;
            __Pyx_DECREF(__pyx_t_3); __pyx_t_3 = 0;

            /* "src/python/grpcio/grpc/_cython/_cygrpc/fork_posix.pyx.pxi":127
 * def block_if_fork_in_progress(postfork_state_to_reset=None):
 *     if _GRPC_ENABLE_FORK_SUPPORT:
 *         with _fork_state.fork_in_progress_condition:             # <<<<<<<<<<<<<<
 *             if not _fork_state.fork_in_progress:
 *                 return
 */
          }
          __Pyx_XDECREF(__pyx_t_7); __pyx_t_7 = 0;
          __Pyx_XDECREF(__pyx_t_8); __pyx_t_8 = 0;
          __Pyx_XDECREF(__pyx_t_9); __pyx_t_9 = 0;
          goto __pyx_L13_try_end;
          __pyx_L8_error:;
          __Pyx_XDECREF(__pyx_t_6); __pyx_t_6 = 0;
          __Pyx_XDECREF(__pyx_t_5); __pyx_t_5 = 0;
          __Pyx_XDECREF(__pyx_t_1); __pyx_t_1 = 0;
          __Pyx_XDECREF(__pyx_t_3); __pyx_t_3 = 0;
          /*except:*/ {
            __Pyx_AddTraceback("grpc._cython.cygrpc.block_if_fork_in_progress", __pyx_clineno, __pyx_lineno, __pyx_filename);
            if (__Pyx_GetException(&__pyx_t_3, &__pyx_t_1, &__pyx_t_5) < 0) __PYX_ERR(0, 127, __pyx_L10_except_error)
            __Pyx_GOTREF(__pyx_t_3);
            __Pyx_GOTREF(__pyx_t_1);
            __Pyx_GOTREF(__pyx_t_5);
            __pyx_t_6 = PyTuple_Pack(3, __pyx_t_3, __pyx_t_1, __pyx_t_5); if (unlikely(!__pyx_t_6)) __PYX_ERR(0, 127, __pyx_L10_except_error)
            __Pyx_GOTREF(__pyx_t_6);
            __pyx_t_12 = __Pyx_PyObject_Call(__pyx_t_4, __pyx_t_6, NULL);
            __Pyx_DECREF(__pyx_t_4); __pyx_t_4 = 0;
            __Pyx_DECREF(__pyx_t_6); __pyx_t_6 = 0;
            if (unlikely(!__pyx_t_12)) __PYX_ERR(0, 127, __pyx_L10_except_error)
            __Pyx_GOTREF(__pyx_t_12);
            __pyx_t_2 = __Pyx_PyObject_IsTrue(__pyx_t_12);
            __Pyx_DECREF(__pyx_t_12); __pyx_t_12 = 0;
            if (__pyx_t_2 < 0) __PYX_ERR(0, 127, __pyx_L10_except_error)
            __pyx_t_10 = ((!(__pyx_t_2 != 0)) != 0);
            if (__pyx_t_10) {
              __Pyx_GIVEREF(__pyx_t_3);
              __Pyx_GIVEREF(__pyx_t_1);
              __Pyx_XGIVEREF(__pyx_t_5);
              __Pyx_ErrRestoreWithState(__pyx_t_3, __pyx_t_1, __pyx_t_5);
              __pyx_t_3 = 0; __pyx_t_1 = 0; __pyx_t_5 = 0; 
              __PYX_ERR(0, 127, __pyx_L10_except_error)
            }
            __Pyx_DECREF(__pyx_t_3); __pyx_t_3 = 0;
            __Pyx_DECREF(__pyx_t_1); __pyx_t_1 = 0;
            __Pyx_DECREF(__pyx_t_5); __pyx_t_5 = 0;
            goto __pyx_L9_exception_handled;
          }
          __pyx_L10_except_error:;
          __Pyx_XGIVEREF(__pyx_t_7);
          __Pyx_XGIVEREF(__pyx_t_8);
          __Pyx_XGIVEREF(__pyx_t_9);
          __Pyx_ExceptionReset(__pyx_t_7, __pyx_t_8, __pyx_t_9);
          goto __pyx_L1_error;
          __pyx_L12_try_return:;
          __Pyx_XGIVEREF(__pyx_t_7);
          __Pyx_XGIVEREF(__pyx_t_8);
          __Pyx_XGIVEREF(__pyx_t_9);
          __Pyx_ExceptionReset(__pyx_t_7, __pyx_t_8, __pyx_t_9);
          goto __pyx_L5_return;
          __pyx_L9_exception_handled:;
          __Pyx_XGIVEREF(__pyx_t_7);
          __Pyx_XGIVEREF(__pyx_t_8);
          __Pyx_XGIVEREF(__pyx_t_9);
          __Pyx_ExceptionReset(__pyx_t_7, __pyx_t_8, __pyx_t_9);
          __pyx_L13_try_end:;
        }
      }
      /*finally:*/ {
        /*normal exit:*/{
          if (__pyx_t_4) {
            __pyx_t_9 = __Pyx_PyObject_Call(__pyx_t_4, __pyx_tuple__157, NULL);
            __Pyx_DECREF(__pyx_t_4); __pyx_t_4 = 0;
            if (unlikely(!__pyx_t_9)) __PYX_ERR(0, 127, __pyx_L1_error)
            __Pyx_GOTREF(__pyx_t_9);
            __Pyx_DECREF(__pyx_t_9); __pyx_t_9 = 0;
          }
          goto __pyx_L7;
        }
        __pyx_L5_return: {
          __pyx_t_9 = __pyx_r;
          __pyx_r = 0;
          if (__pyx_t_4) {
            __pyx_t_8 = __Pyx_PyObject_Call(__pyx_t_4, __pyx_tuple__158, NULL);
            __Pyx_DECREF(__pyx_t_4); __pyx_t_4 = 0;
            if (unlikely(!__pyx_t_8)) __PYX_ERR(0, 127, __pyx_L1_error)
            __Pyx_GOTREF(__pyx_t_8);
            __Pyx_DECREF(__pyx_t_8); __pyx_t_8 = 0;
          }
          __pyx_r = __pyx_t_9;
          __pyx_t_9 = 0;
          goto __pyx_L0;
        }
        __pyx_L7:;
      }
      goto __pyx_L19;
      __pyx_L4_error:;
      __Pyx_DECREF(__pyx_t_4); __pyx_t_4 = 0;
      goto __pyx_L1_error;
      __pyx_L19:;
    }

    /* "src/python/grpcio/grpc/_cython/_cygrpc/fork_posix.pyx.pxi":126
 * 
 * def block_if_fork_in_progress(postfork_state_to_reset=None):
 *     if _GRPC_ENABLE_FORK_SUPPORT:             # <<<<<<<<<<<<<<
 *         with _fork_state.fork_in_progress_condition:
 *             if not _fork_state.fork_in_progress:
 */
  }

  /* "src/python/grpcio/grpc/_cython/_cygrpc/fork_posix.pyx.pxi":125
 * 
 * 
 * def block_if_fork_in_progress(postfork_state_to_reset=None):             # <<<<<<<<<<<<<<
 *     if _GRPC_ENABLE_FORK_SUPPORT:
 *         with _fork_state.fork_in_progress_condition:
 */

  /* function exit code */
  __pyx_r = Py_None; __Pyx_INCREF(Py_None);
  goto __pyx_L0;
  __pyx_L1_error:;
  __Pyx_XDECREF(__pyx_t_1);
  __Pyx_XDECREF(__pyx_t_3);
  __Pyx_XDECREF(__pyx_t_5);
  __Pyx_XDECREF(__pyx_t_6);
  __Pyx_AddTraceback("grpc._cython.cygrpc.block_if_fork_in_progress", __pyx_clineno, __pyx_lineno, __pyx_filename);
  __pyx_r = NULL;
  __pyx_L0:;
  __Pyx_XGIVEREF(__pyx_r);
  __Pyx_RefNannyFinishContext();
  return __pyx_r;
}

/* "src/python/grpcio/grpc/_cython/_cygrpc/fork_posix.pyx.pxi":137
 * 
 * 
 * def enter_user_request_generator():             # <<<<<<<<<<<<<<
 *     if _GRPC_ENABLE_FORK_SUPPORT:
 *         _fork_state.active_thread_count.decrement()
 */

/* Python wrapper */
static PyObject *__pyx_pw_4grpc_7_cython_6cygrpc_67enter_user_request_generator(PyObject *__pyx_self, CYTHON_UNUSED PyObject *unused); /*proto*/
static PyMethodDef __pyx_mdef_4grpc_7_cython_6cygrpc_67enter_user_request_generator = {"enter_user_request_generator", (PyCFunction)__pyx_pw_4grpc_7_cython_6cygrpc_67enter_user_request_generator, METH_NOARGS, 0};
static PyObject *__pyx_pw_4grpc_7_cython_6cygrpc_67enter_user_request_generator(PyObject *__pyx_self, CYTHON_UNUSED PyObject *unused) {
  PyObject *__pyx_r = 0;
  __Pyx_RefNannyDeclarations
  __Pyx_RefNannySetupContext("enter_user_request_generator (wrapper)", 0);
  __pyx_r = __pyx_pf_4grpc_7_cython_6cygrpc_66enter_user_request_generator(__pyx_self);

  /* function exit code */
  __Pyx_RefNannyFinishContext();
  return __pyx_r;
}

static PyObject *__pyx_pf_4grpc_7_cython_6cygrpc_66enter_user_request_generator(CYTHON_UNUSED PyObject *__pyx_self) {
  PyObject *__pyx_r = NULL;
  __Pyx_RefNannyDeclarations
  PyObject *__pyx_t_1 = NULL;
  int __pyx_t_2;
  PyObject *__pyx_t_3 = NULL;
  PyObject *__pyx_t_4 = NULL;
  __Pyx_RefNannySetupContext("enter_user_request_generator", 0);

  /* "src/python/grpcio/grpc/_cython/_cygrpc/fork_posix.pyx.pxi":138
 * 
 * def enter_user_request_generator():
 *     if _GRPC_ENABLE_FORK_SUPPORT:             # <<<<<<<<<<<<<<
 *         _fork_state.active_thread_count.decrement()
 * 
 */
  __pyx_t_1 = __Pyx_GetModuleGlobalName(__pyx_n_s_GRPC_ENABLE_FORK_SUPPORT); if (unlikely(!__pyx_t_1)) __PYX_ERR(0, 138, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_1);
  __pyx_t_2 = __Pyx_PyObject_IsTrue(__pyx_t_1); if (unlikely(__pyx_t_2 < 0)) __PYX_ERR(0, 138, __pyx_L1_error)
  __Pyx_DECREF(__pyx_t_1); __pyx_t_1 = 0;
  if (__pyx_t_2) {

    /* "src/python/grpcio/grpc/_cython/_cygrpc/fork_posix.pyx.pxi":139
 * def enter_user_request_generator():
 *     if _GRPC_ENABLE_FORK_SUPPORT:
 *         _fork_state.active_thread_count.decrement()             # <<<<<<<<<<<<<<
 * 
 * 
 */
    __pyx_t_3 = __Pyx_GetModuleGlobalName(__pyx_n_s_fork_state); if (unlikely(!__pyx_t_3)) __PYX_ERR(0, 139, __pyx_L1_error)
    __Pyx_GOTREF(__pyx_t_3);
    __pyx_t_4 = __Pyx_PyObject_GetAttrStr(__pyx_t_3, __pyx_n_s_active_thread_count); if (unlikely(!__pyx_t_4)) __PYX_ERR(0, 139, __pyx_L1_error)
    __Pyx_GOTREF(__pyx_t_4);
    __Pyx_DECREF(__pyx_t_3); __pyx_t_3 = 0;
    __pyx_t_3 = __Pyx_PyObject_GetAttrStr(__pyx_t_4, __pyx_n_s_decrement); if (unlikely(!__pyx_t_3)) __PYX_ERR(0, 139, __pyx_L1_error)
    __Pyx_GOTREF(__pyx_t_3);
    __Pyx_DECREF(__pyx_t_4); __pyx_t_4 = 0;
    __pyx_t_4 = NULL;
    if (CYTHON_UNPACK_METHODS && likely(PyMethod_Check(__pyx_t_3))) {
      __pyx_t_4 = PyMethod_GET_SELF(__pyx_t_3);
      if (likely(__pyx_t_4)) {
        PyObject* function = PyMethod_GET_FUNCTION(__pyx_t_3);
        __Pyx_INCREF(__pyx_t_4);
        __Pyx_INCREF(function);
        __Pyx_DECREF_SET(__pyx_t_3, function);
      }
    }
    if (__pyx_t_4) {
      __pyx_t_1 = __Pyx_PyObject_CallOneArg(__pyx_t_3, __pyx_t_4); if (unlikely(!__pyx_t_1)) __PYX_ERR(0, 139, __pyx_L1_error)
      __Pyx_DECREF(__pyx_t_4); __pyx_t_4 = 0;
    } else {
      __pyx_t_1 = __Pyx_PyObject_CallNoArg(__pyx_t_3); if (unlikely(!__pyx_t_1)) __PYX_ERR(0, 139, __pyx_L1_error)
    }
    __Pyx_GOTREF(__pyx_t_1);
    __Pyx_DECREF(__pyx_t_3); __pyx_t_3 = 0;
    __Pyx_DECREF(__pyx_t_1); __pyx_t_1 = 0;

    /* "src/python/grpcio/grpc/_cython/_cygrpc/fork_posix.pyx.pxi":138
 * 
 * def enter_user_request_generator():
 *     if _GRPC_ENABLE_FORK_SUPPORT:             # <<<<<<<<<<<<<<
 *         _fork_state.active_thread_count.decrement()
 * 
 */
  }

  /* "src/python/grpcio/grpc/_cython/_cygrpc/fork_posix.pyx.pxi":137
 * 
 * 
 * def enter_user_request_generator():             # <<<<<<<<<<<<<<
 *     if _GRPC_ENABLE_FORK_SUPPORT:
 *         _fork_state.active_thread_count.decrement()
 */

  /* function exit code */
  __pyx_r = Py_None; __Pyx_INCREF(Py_None);
  goto __pyx_L0;
  __pyx_L1_error:;
  __Pyx_XDECREF(__pyx_t_1);
  __Pyx_XDECREF(__pyx_t_3);
  __Pyx_XDECREF(__pyx_t_4);
  __Pyx_AddTraceback("grpc._cython.cygrpc.enter_user_request_generator", __pyx_clineno, __pyx_lineno, __pyx_filename);
  __pyx_r = NULL;
  __pyx_L0:;
  __Pyx_XGIVEREF(__pyx_r);
  __Pyx_RefNannyFinishContext();
  return __pyx_r;
}

/* "src/python/grpcio/grpc/_cython/_cygrpc/fork_posix.pyx.pxi":142
 * 
 * 
 * def return_from_user_request_generator():             # <<<<<<<<<<<<<<
 *     if _GRPC_ENABLE_FORK_SUPPORT:
 *         _fork_state.active_thread_count.increment()
 */

/* Python wrapper */
static PyObject *__pyx_pw_4grpc_7_cython_6cygrpc_69return_from_user_request_generator(PyObject *__pyx_self, CYTHON_UNUSED PyObject *unused); /*proto*/
static PyMethodDef __pyx_mdef_4grpc_7_cython_6cygrpc_69return_from_user_request_generator = {"return_from_user_request_generator", (PyCFunction)__pyx_pw_4grpc_7_cython_6cygrpc_69return_from_user_request_generator, METH_NOARGS, 0};
static PyObject *__pyx_pw_4grpc_7_cython_6cygrpc_69return_from_user_request_generator(PyObject *__pyx_self, CYTHON_UNUSED PyObject *unused) {
  PyObject *__pyx_r = 0;
  __Pyx_RefNannyDeclarations
  __Pyx_RefNannySetupContext("return_from_user_request_generator (wrapper)", 0);
  __pyx_r = __pyx_pf_4grpc_7_cython_6cygrpc_68return_from_user_request_generator(__pyx_self);

  /* function exit code */
  __Pyx_RefNannyFinishContext();
  return __pyx_r;
}

static PyObject *__pyx_pf_4grpc_7_cython_6cygrpc_68return_from_user_request_generator(CYTHON_UNUSED PyObject *__pyx_self) {
  PyObject *__pyx_r = NULL;
  __Pyx_RefNannyDeclarations
  PyObject *__pyx_t_1 = NULL;
  int __pyx_t_2;
  PyObject *__pyx_t_3 = NULL;
  PyObject *__pyx_t_4 = NULL;
  __Pyx_RefNannySetupContext("return_from_user_request_generator", 0);

  /* "src/python/grpcio/grpc/_cython/_cygrpc/fork_posix.pyx.pxi":143
 * 
 * def return_from_user_request_generator():
 *     if _GRPC_ENABLE_FORK_SUPPORT:             # <<<<<<<<<<<<<<
 *         _fork_state.active_thread_count.increment()
 *         block_if_fork_in_progress()
 */
  __pyx_t_1 = __Pyx_GetModuleGlobalName(__pyx_n_s_GRPC_ENABLE_FORK_SUPPORT); if (unlikely(!__pyx_t_1)) __PYX_ERR(0, 143, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_1);
  __pyx_t_2 = __Pyx_PyObject_IsTrue(__pyx_t_1); if (unlikely(__pyx_t_2 < 0)) __PYX_ERR(0, 143, __pyx_L1_error)
  __Pyx_DECREF(__pyx_t_1); __pyx_t_1 = 0;
  if (__pyx_t_2) {

    /* "src/python/grpcio/grpc/_cython/_cygrpc/fork_posix.pyx.pxi":144
 * def return_from_user_request_generator():
 *     if _GRPC_ENABLE_FORK_SUPPORT:
 *         _fork_state.active_thread_count.increment()             # <<<<<<<<<<<<<<
 *         block_if_fork_in_progress()
 * 
 */
    __pyx_t_3 = __Pyx_GetModuleGlobalName(__pyx_n_s_fork_state); if (unlikely(!__pyx_t_3)) __PYX_ERR(0, 144, __pyx_L1_error)
    __Pyx_GOTREF(__pyx_t_3);
    __pyx_t_4 = __Pyx_PyObject_GetAttrStr(__pyx_t_3, __pyx_n_s_active_thread_count); if (unlikely(!__pyx_t_4)) __PYX_ERR(0, 144, __pyx_L1_error)
    __Pyx_GOTREF(__pyx_t_4);
    __Pyx_DECREF(__pyx_t_3); __pyx_t_3 = 0;
    __pyx_t_3 = __Pyx_PyObject_GetAttrStr(__pyx_t_4, __pyx_n_s_increment); if (unlikely(!__pyx_t_3)) __PYX_ERR(0, 144, __pyx_L1_error)
    __Pyx_GOTREF(__pyx_t_3);
    __Pyx_DECREF(__pyx_t_4); __pyx_t_4 = 0;
    __pyx_t_4 = NULL;
    if (CYTHON_UNPACK_METHODS && likely(PyMethod_Check(__pyx_t_3))) {
      __pyx_t_4 = PyMethod_GET_SELF(__pyx_t_3);
      if (likely(__pyx_t_4)) {
        PyObject* function = PyMethod_GET_FUNCTION(__pyx_t_3);
        __Pyx_INCREF(__pyx_t_4);
        __Pyx_INCREF(function);
        __Pyx_DECREF_SET(__pyx_t_3, function);
      }
    }
    if (__pyx_t_4) {
      __pyx_t_1 = __Pyx_PyObject_CallOneArg(__pyx_t_3, __pyx_t_4); if (unlikely(!__pyx_t_1)) __PYX_ERR(0, 144, __pyx_L1_error)
      __Pyx_DECREF(__pyx_t_4); __pyx_t_4 = 0;
    } else {
      __pyx_t_1 = __Pyx_PyObject_CallNoArg(__pyx_t_3); if (unlikely(!__pyx_t_1)) __PYX_ERR(0, 144, __pyx_L1_error)
    }
    __Pyx_GOTREF(__pyx_t_1);
    __Pyx_DECREF(__pyx_t_3); __pyx_t_3 = 0;
    __Pyx_DECREF(__pyx_t_1); __pyx_t_1 = 0;

    /* "src/python/grpcio/grpc/_cython/_cygrpc/fork_posix.pyx.pxi":145
 *     if _GRPC_ENABLE_FORK_SUPPORT:
 *         _fork_state.active_thread_count.increment()
 *         block_if_fork_in_progress()             # <<<<<<<<<<<<<<
 * 
 * 
 */
    __pyx_t_3 = __Pyx_GetModuleGlobalName(__pyx_n_s_block_if_fork_in_progress); if (unlikely(!__pyx_t_3)) __PYX_ERR(0, 145, __pyx_L1_error)
    __Pyx_GOTREF(__pyx_t_3);
    __pyx_t_4 = NULL;
    if (CYTHON_UNPACK_METHODS && unlikely(PyMethod_Check(__pyx_t_3))) {
      __pyx_t_4 = PyMethod_GET_SELF(__pyx_t_3);
      if (likely(__pyx_t_4)) {
        PyObject* function = PyMethod_GET_FUNCTION(__pyx_t_3);
        __Pyx_INCREF(__pyx_t_4);
        __Pyx_INCREF(function);
        __Pyx_DECREF_SET(__pyx_t_3, function);
      }
    }
    if (__pyx_t_4) {
      __pyx_t_1 = __Pyx_PyObject_CallOneArg(__pyx_t_3, __pyx_t_4); if (unlikely(!__pyx_t_1)) __PYX_ERR(0, 145, __pyx_L1_error)
      __Pyx_DECREF(__pyx_t_4); __pyx_t_4 = 0;
    } else {
      __pyx_t_1 = __Pyx_PyObject_CallNoArg(__pyx_t_3); if (unlikely(!__pyx_t_1)) __PYX_ERR(0, 145, __pyx_L1_error)
    }
    __Pyx_GOTREF(__pyx_t_1);
    __Pyx_DECREF(__pyx_t_3); __pyx_t_3 = 0;
    __Pyx_DECREF(__pyx_t_1); __pyx_t_1 = 0;

    /* "src/python/grpcio/grpc/_cython/_cygrpc/fork_posix.pyx.pxi":143
 * 
 * def return_from_user_request_generator():
 *     if _GRPC_ENABLE_FORK_SUPPORT:             # <<<<<<<<<<<<<<
 *         _fork_state.active_thread_count.increment()
 *         block_if_fork_in_progress()
 */
  }

  /* "src/python/grpcio/grpc/_cython/_cygrpc/fork_posix.pyx.pxi":142
 * 
 * 
 * def return_from_user_request_generator():             # <<<<<<<<<<<<<<
 *     if _GRPC_ENABLE_FORK_SUPPORT:
 *         _fork_state.active_thread_count.increment()
 */

  /* function exit code */
  __pyx_r = Py_None; __Pyx_INCREF(Py_None);
  goto __pyx_L0;
  __pyx_L1_error:;
  __Pyx_XDECREF(__pyx_t_1);
  __Pyx_XDECREF(__pyx_t_3);
  __Pyx_XDECREF(__pyx_t_4);
  __Pyx_AddTraceback("grpc._cython.cygrpc.return_from_user_request_generator", __pyx_clineno, __pyx_lineno, __pyx_filename);
  __pyx_r = NULL;
  __pyx_L0:;
  __Pyx_XGIVEREF(__pyx_r);
  __Pyx_RefNannyFinishContext();
  return __pyx_r;
}

/* "src/python/grpcio/grpc/_cython/_cygrpc/fork_posix.pyx.pxi":148
 * 
 * 
 * def get_fork_epoch():             # <<<<<<<<<<<<<<
 *     return _fork_state.fork_epoch
 * 
 */

/* Python wrapper */
static PyObject *__pyx_pw_4grpc_7_cython_6cygrpc_71get_fork_epoch(PyObject *__pyx_self, CYTHON_UNUSED PyObject *unused); /*proto*/
static PyMethodDef __pyx_mdef_4grpc_7_cython_6cygrpc_71get_fork_epoch = {"get_fork_epoch", (PyCFunction)__pyx_pw_4grpc_7_cython_6cygrpc_71get_fork_epoch, METH_NOARGS, 0};
static PyObject *__pyx_pw_4grpc_7_cython_6cygrpc_71get_fork_epoch(PyObject *__pyx_self, CYTHON_UNUSED PyObject *unused) {
  PyObject *__pyx_r = 0;
  __Pyx_RefNannyDeclarations
  __Pyx_RefNannySetupContext("get_fork_epoch (wrapper)", 0);
  __pyx_r = __pyx_pf_4grpc_7_cython_6cygrpc_70get_fork_epoch(__pyx_self);

  /* function exit code */
  __Pyx_RefNannyFinishContext();
  return __pyx_r;
}

static PyObject *__pyx_pf_4grpc_7_cython_6cygrpc_70get_fork_epoch(CYTHON_UNUSED PyObject *__pyx_self) {
  PyObject *__pyx_r = NULL;
  __Pyx_RefNannyDeclarations
  PyObject *__pyx_t_1 = NULL;
  PyObject *__pyx_t_2 = NULL;
  __Pyx_RefNannySetupContext("get_fork_epoch", 0);

  /* "src/python/grpcio/grpc/_cython/_cygrpc/fork_posix.pyx.pxi":149
 * 
 * def get_fork_epoch():
 *     return _fork_state.fork_epoch             # <<<<<<<<<<<<<<
 * 
 * 
 */
  __Pyx_XDECREF(__pyx_r);
  __pyx_t_1 = __Pyx_GetModuleGlobalName(__pyx_n_s_fork_state); if (unlikely(!__pyx_t_1)) __PYX_ERR(0, 149, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_1);
  __pyx_t_2 = __Pyx_PyObject_GetAttrStr(__pyx_t_1, __pyx_n_s_fork_epoch); if (unlikely(!__pyx_t_2)) __PYX_ERR(0, 149, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_2);
  __Pyx_DECREF(__pyx_t_1); __pyx_t_1 = 0;
  __pyx_r = __pyx_t_2;
  __pyx_t_2 = 0;
  goto __pyx_L0;

  /* "src/python/grpcio/grpc/_cython/_cygrpc/fork_posix.pyx.pxi":148
 * 
 * 
 * def get_fork_epoch():             # <<<<<<<<<<<<<<
 *     return _fork_state.fork_epoch
 * 
 */

  /* function exit code */
  __pyx_L1_error:;
  __Pyx_XDECREF(__pyx_t_1);
  __Pyx_XDECREF(__pyx_t_2);
  __Pyx_AddTraceback("grpc._cython.cygrpc.get_fork_epoch", __pyx_clineno, __pyx_lineno, __pyx_filename);
  __pyx_r = NULL;
  __pyx_L0:;
  __Pyx_XGIVEREF(__pyx_r);
  __Pyx_RefNannyFinishContext();
  return __pyx_r;
}

/* "src/python/grpcio/grpc/_cython/_cygrpc/fork_posix.pyx.pxi":152
 * 
 * 
 * def is_fork_support_enabled():             # <<<<<<<<<<<<<<
 *     return _GRPC_ENABLE_FORK_SUPPORT
 * 
 */

/* Python wrapper */
static PyObject *__pyx_pw_4grpc_7_cython_6cygrpc_73is_fork_support_enabled(PyObject *__pyx_self, CYTHON_UNUSED PyObject *unused); /*proto*/
static PyMethodDef __pyx_mdef_4grpc_7_cython_6cygrpc_73is_fork_support_enabled = {"is_fork_support_enabled", (PyCFunction)__pyx_pw_4grpc_7_cython_6cygrpc_73is_fork_support_enabled, METH_NOARGS, 0};
static PyObject *__pyx_pw_4grpc_7_cython_6cygrpc_73is_fork_support_enabled(PyObject *__pyx_self, CYTHON_UNUSED PyObject *unused) {
  PyObject *__pyx_r = 0;
  __Pyx_RefNannyDeclarations
  __Pyx_RefNannySetupContext("is_fork_support_enabled (wrapper)", 0);
  __pyx_r = __pyx_pf_4grpc_7_cython_6cygrpc_72is_fork_support_enabled(__pyx_self);

  /* function exit code */
  __Pyx_RefNannyFinishContext();
  return __pyx_r;
}

static PyObject *__pyx_pf_4grpc_7_cython_6cygrpc_72is_fork_support_enabled(CYTHON_UNUSED PyObject *__pyx_self) {
  PyObject *__pyx_r = NULL;
  __Pyx_RefNannyDeclarations
  PyObject *__pyx_t_1 = NULL;
  __Pyx_RefNannySetupContext("is_fork_support_enabled", 0);

  /* "src/python/grpcio/grpc/_cython/_cygrpc/fork_posix.pyx.pxi":153
 * 
 * def is_fork_support_enabled():
 *     return _GRPC_ENABLE_FORK_SUPPORT             # <<<<<<<<<<<<<<
 * 
 * 
 */
  __Pyx_XDECREF(__pyx_r);
  __pyx_t_1 = __Pyx_GetModuleGlobalName(__pyx_n_s_GRPC_ENABLE_FORK_SUPPORT); if (unlikely(!__pyx_t_1)) __PYX_ERR(0, 153, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_1);
  __pyx_r = __pyx_t_1;
  __pyx_t_1 = 0;
  goto __pyx_L0;

  /* "src/python/grpcio/grpc/_cython/_cygrpc/fork_posix.pyx.pxi":152
 * 
 * 
 * def is_fork_support_enabled():             # <<<<<<<<<<<<<<
 *     return _GRPC_ENABLE_FORK_SUPPORT
 * 
 */

  /* function exit code */
  __pyx_L1_error:;
  __Pyx_XDECREF(__pyx_t_1);
  __Pyx_AddTraceback("grpc._cython.cygrpc.is_fork_support_enabled", __pyx_clineno, __pyx_lineno, __pyx_filename);
  __pyx_r = NULL;
  __pyx_L0:;
  __Pyx_XGIVEREF(__pyx_r);
  __Pyx_RefNannyFinishContext();
  return __pyx_r;
}

/* "src/python/grpcio/grpc/_cython/_cygrpc/fork_posix.pyx.pxi":156
 * 
 * 
 * def fork_register_channel(channel):             # <<<<<<<<<<<<<<
 *     if _GRPC_ENABLE_FORK_SUPPORT:
 *         _fork_state.channels.add(channel)
 */

/* Python wrapper */
static PyObject *__pyx_pw_4grpc_7_cython_6cygrpc_75fork_register_channel(PyObject *__pyx_self, PyObject *__pyx_v_channel); /*proto*/
static PyMethodDef __pyx_mdef_4grpc_7_cython_6cygrpc_75fork_register_channel = {"fork_register_channel", (PyCFunction)__pyx_pw_4grpc_7_cython_6cygrpc_75fork_register_channel, METH_O, 0};
static PyObject *__pyx_pw_4grpc_7_cython_6cygrpc_75fork_register_channel(PyObject *__pyx_self, PyObject *__pyx_v_channel) {
  PyObject *__pyx_r = 0;
  __Pyx_RefNannyDeclarations
  __Pyx_RefNannySetupContext("fork_register_channel (wrapper)", 0);
  __pyx_r = __pyx_pf_4grpc_7_cython_6cygrpc_74fork_register_channel(__pyx_self, ((PyObject *)__pyx_v_channel));

  /* function exit code */
  __Pyx_RefNannyFinishContext();
  return __pyx_r;
}

static PyObject *__pyx_pf_4grpc_7_cython_6cygrpc_74fork_register_channel(CYTHON_UNUSED PyObject *__pyx_self, PyObject *__pyx_v_channel) {
  PyObject *__pyx_r = NULL;
  __Pyx_RefNannyDeclarations
  PyObject *__pyx_t_1 = NULL;
  int __pyx_t_2;
  PyObject *__pyx_t_3 = NULL;
  PyObject *__pyx_t_4 = NULL;
  PyObject *__pyx_t_5 = NULL;
  __Pyx_RefNannySetupContext("fork_register_channel", 0);

  /* "src/python/grpcio/grpc/_cython/_cygrpc/fork_posix.pyx.pxi":157
 * 
 * def fork_register_channel(channel):
 *     if _GRPC_ENABLE_FORK_SUPPORT:             # <<<<<<<<<<<<<<
 *         _fork_state.channels.add(channel)
 * 
 */
  __pyx_t_1 = __Pyx_GetModuleGlobalName(__pyx_n_s_GRPC_ENABLE_FORK_SUPPORT); if (unlikely(!__pyx_t_1)) __PYX_ERR(0, 157, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_1);
  __pyx_t_2 = __Pyx_PyObject_IsTrue(__pyx_t_1); if (unlikely(__pyx_t_2 < 0)) __PYX_ERR(0, 157, __pyx_L1_error)
  __Pyx_DECREF(__pyx_t_1); __pyx_t_1 = 0;
  if (__pyx_t_2) {

    /* "src/python/grpcio/grpc/_cython/_cygrpc/fork_posix.pyx.pxi":158
 * def fork_register_channel(channel):
 *     if _GRPC_ENABLE_FORK_SUPPORT:
 *         _fork_state.channels.add(channel)             # <<<<<<<<<<<<<<
 * 
 * 
 */
    __pyx_t_3 = __Pyx_GetModuleGlobalName(__pyx_n_s_fork_state); if (unlikely(!__pyx_t_3)) __PYX_ERR(0, 158, __pyx_L1_error)
    __Pyx_GOTREF(__pyx_t_3);
    __pyx_t_4 = __Pyx_PyObject_GetAttrStr(__pyx_t_3, __pyx_n_s_channels); if (unlikely(!__pyx_t_4)) __PYX_ERR(0, 158, __pyx_L1_error)
    __Pyx_GOTREF(__pyx_t_4);
    __Pyx_DECREF(__pyx_t_3); __pyx_t_3 = 0;
    __pyx_t_3 = __Pyx_PyObject_GetAttrStr(__pyx_t_4, __pyx_n_s_add); if (unlikely(!__pyx_t_3)) __PYX_ERR(0, 158, __pyx_L1_error)
    __Pyx_GOTREF(__pyx_t_3);
    __Pyx_DECREF(__pyx_t_4); __pyx_t_4 = 0;
    __pyx_t_4 = NULL;
    if (CYTHON_UNPACK_METHODS && likely(PyMethod_Check(__pyx_t_3))) {
      __pyx_t_4 = PyMethod_GET_SELF(__pyx_t_3);
      if (likely(__pyx_t_4)) {
        PyObject* function = PyMethod_GET_FUNCTION(__pyx_t_3);
        __Pyx_INCREF(__pyx_t_4);
        __Pyx_INCREF(function);
        __Pyx_DECREF_SET(__pyx_t_3, function);
      }
    }
    if (!__pyx_t_4) {
      __pyx_t_1 = __Pyx_PyObject_CallOneArg(__pyx_t_3, __pyx_v_channel); if (unlikely(!__pyx_t_1)) __PYX_ERR(0, 158, __pyx_L1_error)
      __Pyx_GOTREF(__pyx_t_1);
    } else {
      #if CYTHON_FAST_PYCALL
      if (PyFunction_Check(__pyx_t_3)) {
        PyObject *__pyx_temp[2] = {__pyx_t_4, __pyx_v_channel};
        __pyx_t_1 = __Pyx_PyFunction_FastCall(__pyx_t_3, __pyx_temp+1-1, 1+1); if (unlikely(!__pyx_t_1)) __PYX_ERR(0, 158, __pyx_L1_error)
        __Pyx_XDECREF(__pyx_t_4); __pyx_t_4 = 0;
        __Pyx_GOTREF(__pyx_t_1);
      } else
      #endif
      #if CYTHON_FAST_PYCCALL
      if (__Pyx_PyFastCFunction_Check(__pyx_t_3)) {
        PyObject *__pyx_temp[2] = {__pyx_t_4, __pyx_v_channel};
        __pyx_t_1 = __Pyx_PyCFunction_FastCall(__pyx_t_3, __pyx_temp+1-1, 1+1); if (unlikely(!__pyx_t_1)) __PYX_ERR(0, 158, __pyx_L1_error)
        __Pyx_XDECREF(__pyx_t_4); __pyx_t_4 = 0;
        __Pyx_GOTREF(__pyx_t_1);
      } else
      #endif
      {
        __pyx_t_5 = PyTuple_New(1+1); if (unlikely(!__pyx_t_5)) __PYX_ERR(0, 158, __pyx_L1_error)
        __Pyx_GOTREF(__pyx_t_5);
        __Pyx_GIVEREF(__pyx_t_4); PyTuple_SET_ITEM(__pyx_t_5, 0, __pyx_t_4); __pyx_t_4 = NULL;
        __Pyx_INCREF(__pyx_v_channel);
        __Pyx_GIVEREF(__pyx_v_channel);
        PyTuple_SET_ITEM(__pyx_t_5, 0+1, __pyx_v_channel);
        __pyx_t_1 = __Pyx_PyObject_Call(__pyx_t_3, __pyx_t_5, NULL); if (unlikely(!__pyx_t_1)) __PYX_ERR(0, 158, __pyx_L1_error)
        __Pyx_GOTREF(__pyx_t_1);
        __Pyx_DECREF(__pyx_t_5); __pyx_t_5 = 0;
      }
    }
    __Pyx_DECREF(__pyx_t_3); __pyx_t_3 = 0;
    __Pyx_DECREF(__pyx_t_1); __pyx_t_1 = 0;

    /* "src/python/grpcio/grpc/_cython/_cygrpc/fork_posix.pyx.pxi":157
 * 
 * def fork_register_channel(channel):
 *     if _GRPC_ENABLE_FORK_SUPPORT:             # <<<<<<<<<<<<<<
 *         _fork_state.channels.add(channel)
 * 
 */
  }

  /* "src/python/grpcio/grpc/_cython/_cygrpc/fork_posix.pyx.pxi":156
 * 
 * 
 * def fork_register_channel(channel):             # <<<<<<<<<<<<<<
 *     if _GRPC_ENABLE_FORK_SUPPORT:
 *         _fork_state.channels.add(channel)
 */

  /* function exit code */
  __pyx_r = Py_None; __Pyx_INCREF(Py_None);
  goto __pyx_L0;
  __pyx_L1_error:;
  __Pyx_XDECREF(__pyx_t_1);
  __Pyx_XDECREF(__pyx_t_3);
  __Pyx_XDECREF(__pyx_t_4);
  __Pyx_XDECREF(__pyx_t_5);
  __Pyx_AddTraceback("grpc._cython.cygrpc.fork_register_channel", __pyx_clineno, __pyx_lineno, __pyx_filename);
  __pyx_r = NULL;
  __pyx_L0:;
  __Pyx_XGIVEREF(__pyx_r);
  __Pyx_RefNannyFinishContext();
  return __pyx_r;
}

/* "src/python/grpcio/grpc/_cython/_cygrpc/fork_posix.pyx.pxi":161
 * 
 * 
 * def fork_unregister_channel(channel):             # <<<<<<<<<<<<<<
 *     if _GRPC_ENABLE_FORK_SUPPORT:
 *         _fork_state.channels.discard(channel)
 */

/* Python wrapper */
static PyObject *__pyx_pw_4grpc_7_cython_6cygrpc_77fork_unregister_channel(PyObject *__pyx_self, PyObject *__pyx_v_channel); /*proto*/
static PyMethodDef __pyx_mdef_4grpc_7_cython_6cygrpc_77fork_unregister_channel = {"fork_unregister_channel", (PyCFunction)__pyx_pw_4grpc_7_cython_6cygrpc_77fork_unregister_channel, METH_O, 0};
static PyObject *__pyx_pw_4grpc_7_cython_6cygrpc_77fork_unregister_channel(PyObject *__pyx_self, PyObject *__pyx_v_channel) {
  PyObject *__pyx_r = 0;
  __Pyx_RefNannyDeclarations
  __Pyx_RefNannySetupContext("fork_unregister_channel (wrapper)", 0);
  __pyx_r = __pyx_pf_4grpc_7_cython_6cygrpc_76fork_unregister_channel(__pyx_self, ((PyObject *)__pyx_v_channel));

  /* function exit code */
  __Pyx_RefNannyFinishContext();
  return __pyx_r;
}

static PyObject *__pyx_pf_4grpc_7_cython_6cygrpc_76fork_unregister_channel(CYTHON_UNUSED PyObject *__pyx_self, PyObject *__pyx_v_channel) {
  PyObject *__pyx_r = NULL;
  __Pyx_RefNannyDeclarations
  PyObject *__pyx_t_1 = NULL;
  int __pyx_t_2;
  PyObject *__pyx_t_3 = NULL;
  PyObject *__pyx_t_4 = NULL;
  PyObject *__pyx_t_5 = NULL;
  __Pyx_RefNannySetupContext("fork_unregister_channel", 0);

  /* "src/python/grpcio/grpc/_cython/_cygrpc/fork_posix.pyx.pxi":162
 * 
 * def fork_unregister_channel(channel):
 *     if _GRPC_ENABLE_FORK_SUPPORT:             # <<<<<<<<<<<<<<
 *         _fork_state.channels.discard(channel)
 * 
 */
  __pyx_t_1 = __Pyx_GetModuleGlobalName(__pyx_n_s_GRPC_ENABLE_FORK_SUPPORT); if (unlikely(!__pyx_t_1)) __PYX_ERR(0, 162, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_1);
  __pyx_t_2 = __Pyx_PyObject_IsTrue(__pyx_t_1); if (unlikely(__pyx_t_2 < 0)) __PYX_ERR(0, 162, __pyx_L1_error)
  __Pyx_DECREF(__pyx_t_1); __pyx_t_1 = 0;
  if (__pyx_t_2) {

    /* "src/python/grpcio/grpc/_cython/_cygrpc/fork_posix.pyx.pxi":163
 * def fork_unregister_channel(channel):
 *     if _GRPC_ENABLE_FORK_SUPPORT:
 *         _fork_state.channels.discard(channel)             # <<<<<<<<<<<<<<
 * 
 * 
 */
    __pyx_t_3 = __Pyx_GetModuleGlobalName(__pyx_n_s_fork_state); if (unlikely(!__pyx_t_3)) __PYX_ERR(0, 163, __pyx_L1_error)
    __Pyx_GOTREF(__pyx_t_3);
    __pyx_t_4 = __Pyx_PyObject_GetAttrStr(__pyx_t_3, __pyx_n_s_channels); if (unlikely(!__pyx_t_4)) __PYX_ERR(0, 163, __pyx_L1_error)
    __Pyx_GOTREF(__pyx_t_4);
    __Pyx_DECREF(__pyx_t_3); __pyx_t_3 = 0;
    __pyx_t_3 = __Pyx_PyObject_GetAttrStr(__pyx_t_4, __pyx_n_s_discard); if (unlikely(!__pyx_t_3)) __PYX_ERR(0, 163, __pyx_L1_error)
    __Pyx_GOTREF(__pyx_t_3);
    __Pyx_DECREF(__pyx_t_4); __pyx_t_4 = 0;
    __pyx_t_4 = NULL;
    if (CYTHON_UNPACK_METHODS && likely(PyMethod_Check(__pyx_t_3))) {
      __pyx_t_4 = PyMethod_GET_SELF(__pyx_t_3);
      if (likely(__pyx_t_4)) {
        PyObject* function = PyMethod_GET_FUNCTION(__pyx_t_3);
        __Pyx_INCREF(__pyx_t_4);
        __Pyx_INCREF(function);
        __Pyx_DECREF_SET(__pyx_t_3, function);
      }
    }
    if (!__pyx_t_4) {
      __pyx_t_1 = __Pyx_PyObject_CallOneArg(__pyx_t_3, __pyx_v_channel); if (unlikely(!__pyx_t_1)) __PYX_ERR(0, 163, __pyx_L1_error)
      __Pyx_GOTREF(__pyx_t_1);
    } else {
      #if CYTHON_FAST_PYCALL
      if (PyFunction_Check(__pyx_t_3)) {
        PyObject *__pyx_temp[2] = {__pyx_t_4, __pyx_v_channel};
        __pyx_t_1 = __Pyx_PyFunction_FastCall(__pyx_t_3, __pyx_temp+1-1, 1+1); if (unlikely(!__pyx_t_1)) __PYX_ERR(0, 163, __pyx_L1_error)
        __Pyx_XDECREF(__pyx_t_4); __pyx_t_4 = 0;
        __Pyx_GOTREF(__pyx_t_1);
      } else
      #endif
      #if CYTHON_FAST_PYCCALL
      if (__Pyx_PyFastCFunction_Check(__pyx_t_3)) {
        PyObject *__pyx_temp[2] = {__pyx_t_4, __pyx_v_channel};
        __pyx_t_1 = __Pyx_PyCFunction_FastCall(__pyx_t_3, __pyx_temp+1-1, 1+1); if (unlikely(!__pyx_t_1)) __PYX_ERR(0, 163, __pyx_L1_error)
        __Pyx_XDECREF(__pyx_t_4); __pyx_t_4 = 0;
        __Pyx_GOTREF(__pyx_t_1);
      } else
      #endif
      {
        __pyx_t_5 = PyTuple_New(1+1); if (unlikely(!__pyx_t_5)) __PYX_ERR(0, 163, __pyx_L1_error)
        __Pyx_GOTREF(__pyx_t_5);
        __Pyx_GIVEREF(__pyx_t_4); PyTuple_SET_ITEM(__pyx_t_5, 0, __pyx_t_4); __pyx_t_4 = NULL;
        __Pyx_INCREF(__pyx_v_channel);
        __Pyx_GIVEREF(__pyx_v_channel);
        PyTuple_SET_ITEM(__pyx_t_5, 0+1, __pyx_v_channel);
        __pyx_t_1 = __Pyx_PyObject_Call(__pyx_t_3, __pyx_t_5, NULL); if (unlikely(!__pyx_t_1)) __PYX_ERR(0, 163, __pyx_L1_error)
        __Pyx_GOTREF(__pyx_t_1);
        __Pyx_DECREF(__pyx_t_5); __pyx_t_5 = 0;
      }
    }
    __Pyx_DECREF(__pyx_t_3); __pyx_t_3 = 0;
    __Pyx_DECREF(__pyx_t_1); __pyx_t_1 = 0;

    /* "src/python/grpcio/grpc/_cython/_cygrpc/fork_posix.pyx.pxi":162
 * 
 * def fork_unregister_channel(channel):
 *     if _GRPC_ENABLE_FORK_SUPPORT:             # <<<<<<<<<<<<<<
 *         _fork_state.channels.discard(channel)
 * 
 */
  }

  /* "src/python/grpcio/grpc/_cython/_cygrpc/fork_posix.pyx.pxi":161
 * 
 * 
 * def fork_unregister_channel(channel):             # <<<<<<<<<<<<<<
 *     if _GRPC_ENABLE_FORK_SUPPORT:
 *         _fork_state.channels.discard(channel)
 */

  /* function exit code */
  __pyx_r = Py_None; __Pyx_INCREF(Py_None);
  goto __pyx_L0;
  __pyx_L1_error:;
  __Pyx_XDECREF(__pyx_t_1);
  __Pyx_XDECREF(__pyx_t_3);
  __Pyx_XDECREF(__pyx_t_4);
  __Pyx_XDECREF(__pyx_t_5);
  __Pyx_AddTraceback("grpc._cython.cygrpc.fork_unregister_channel", __pyx_clineno, __pyx_lineno, __pyx_filename);
  __pyx_r = NULL;
  __pyx_L0:;
  __Pyx_XGIVEREF(__pyx_r);
  __Pyx_RefNannyFinishContext();
  return __pyx_r;
}

/* "src/python/grpcio/grpc/_cython/_cygrpc/fork_posix.pyx.pxi":167
 * 
 * class _ActiveThreadCount(object):
 *     def __init__(self):             # <<<<<<<<<<<<<<
 *         self._num_active_threads = 0
 *         self._condition = threading.Condition()
 */

/* Python wrapper */
static PyObject *__pyx_pw_4grpc_7_cython_6cygrpc_18_ActiveThreadCount_1__init__(PyObject *__pyx_self, PyObject *__pyx_v_self); /*proto*/
static PyMethodDef __pyx_mdef_4grpc_7_cython_6cygrpc_18_ActiveThreadCount_1__init__ = {"__init__", (PyCFunction)__pyx_pw_4grpc_7_cython_6cygrpc_18_ActiveThreadCount_1__init__, METH_O, 0};
static PyObject *__pyx_pw_4grpc_7_cython_6cygrpc_18_ActiveThreadCount_1__init__(PyObject *__pyx_self, PyObject *__pyx_v_self) {
  PyObject *__pyx_r = 0;
  __Pyx_RefNannyDeclarations
  __Pyx_RefNannySetupContext("__init__ (wrapper)", 0);
  __pyx_r = __pyx_pf_4grpc_7_cython_6cygrpc_18_ActiveThreadCount___init__(__pyx_self, ((PyObject *)__pyx_v_self));

  /* function exit code */
  __Pyx_RefNannyFinishContext();
  return __pyx_r;
}

static PyObject *__pyx_pf_4grpc_7_cython_6cygrpc_18_ActiveThreadCount___init__(CYTHON_UNUSED PyObject *__pyx_self, PyObject *__pyx_v_self) {
  PyObject *__pyx_r = NULL;
  __Pyx_RefNannyDeclarations
  PyObject *__pyx_t_1 = NULL;
  PyObject *__pyx_t_2 = NULL;
  PyObject *__pyx_t_3 = NULL;
  __Pyx_RefNannySetupContext("__init__", 0);

  /* "src/python/grpcio/grpc/_cython/_cygrpc/fork_posix.pyx.pxi":168
 * class _ActiveThreadCount(object):
 *     def __init__(self):
 *         self._num_active_threads = 0             # <<<<<<<<<<<<<<
 *         self._condition = threading.Condition()
 * 
 */
  if (__Pyx_PyObject_SetAttrStr(__pyx_v_self, __pyx_n_s_num_active_threads, __pyx_int_0) < 0) __PYX_ERR(0, 168, __pyx_L1_error)

  /* "src/python/grpcio/grpc/_cython/_cygrpc/fork_posix.pyx.pxi":169
 *     def __init__(self):
 *         self._num_active_threads = 0
 *         self._condition = threading.Condition()             # <<<<<<<<<<<<<<
 * 
 *     def increment(self):
 */
  __pyx_t_2 = __Pyx_GetModuleGlobalName(__pyx_n_s_threading); if (unlikely(!__pyx_t_2)) __PYX_ERR(0, 169, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_2);
  __pyx_t_3 = __Pyx_PyObject_GetAttrStr(__pyx_t_2, __pyx_n_s_Condition); if (unlikely(!__pyx_t_3)) __PYX_ERR(0, 169, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_3);
  __Pyx_DECREF(__pyx_t_2); __pyx_t_2 = 0;
  __pyx_t_2 = NULL;
  if (CYTHON_UNPACK_METHODS && unlikely(PyMethod_Check(__pyx_t_3))) {
    __pyx_t_2 = PyMethod_GET_SELF(__pyx_t_3);
    if (likely(__pyx_t_2)) {
      PyObject* function = PyMethod_GET_FUNCTION(__pyx_t_3);
      __Pyx_INCREF(__pyx_t_2);
      __Pyx_INCREF(function);
      __Pyx_DECREF_SET(__pyx_t_3, function);
    }
  }
  if (__pyx_t_2) {
    __pyx_t_1 = __Pyx_PyObject_CallOneArg(__pyx_t_3, __pyx_t_2); if (unlikely(!__pyx_t_1)) __PYX_ERR(0, 169, __pyx_L1_error)
    __Pyx_DECREF(__pyx_t_2); __pyx_t_2 = 0;
  } else {
    __pyx_t_1 = __Pyx_PyObject_CallNoArg(__pyx_t_3); if (unlikely(!__pyx_t_1)) __PYX_ERR(0, 169, __pyx_L1_error)
  }
  __Pyx_GOTREF(__pyx_t_1);
  __Pyx_DECREF(__pyx_t_3); __pyx_t_3 = 0;
  if (__Pyx_PyObject_SetAttrStr(__pyx_v_self, __pyx_n_s_condition, __pyx_t_1) < 0) __PYX_ERR(0, 169, __pyx_L1_error)
  __Pyx_DECREF(__pyx_t_1); __pyx_t_1 = 0;

  /* "src/python/grpcio/grpc/_cython/_cygrpc/fork_posix.pyx.pxi":167
 * 
 * class _ActiveThreadCount(object):
 *     def __init__(self):             # <<<<<<<<<<<<<<
 *         self._num_active_threads = 0
 *         self._condition = threading.Condition()
 */

  /* function exit code */
  __pyx_r = Py_None; __Pyx_INCREF(Py_None);
  goto __pyx_L0;
  __pyx_L1_error:;
  __Pyx_XDECREF(__pyx_t_1);
  __Pyx_XDECREF(__pyx_t_2);
  __Pyx_XDECREF(__pyx_t_3);
  __Pyx_AddTraceback("grpc._cython.cygrpc._ActiveThreadCount.__init__", __pyx_clineno, __pyx_lineno, __pyx_filename);
  __pyx_r = NULL;
  __pyx_L0:;
  __Pyx_XGIVEREF(__pyx_r);
  __Pyx_RefNannyFinishContext();
  return __pyx_r;
}

/* "src/python/grpcio/grpc/_cython/_cygrpc/fork_posix.pyx.pxi":171
 *         self._condition = threading.Condition()
 * 
 *     def increment(self):             # <<<<<<<<<<<<<<
 *         with self._condition:
 *             self._num_active_threads += 1
 */

/* Python wrapper */
static PyObject *__pyx_pw_4grpc_7_cython_6cygrpc_18_ActiveThreadCount_3increment(PyObject *__pyx_self, PyObject *__pyx_v_self); /*proto*/
static PyMethodDef __pyx_mdef_4grpc_7_cython_6cygrpc_18_ActiveThreadCount_3increment = {"increment", (PyCFunction)__pyx_pw_4grpc_7_cython_6cygrpc_18_ActiveThreadCount_3increment, METH_O, 0};
static PyObject *__pyx_pw_4grpc_7_cython_6cygrpc_18_ActiveThreadCount_3increment(PyObject *__pyx_self, PyObject *__pyx_v_self) {
  PyObject *__pyx_r = 0;
  __Pyx_RefNannyDeclarations
  __Pyx_RefNannySetupContext("increment (wrapper)", 0);
  __pyx_r = __pyx_pf_4grpc_7_cython_6cygrpc_18_ActiveThreadCount_2increment(__pyx_self, ((PyObject *)__pyx_v_self));

  /* function exit code */
  __Pyx_RefNannyFinishContext();
  return __pyx_r;
}

static PyObject *__pyx_pf_4grpc_7_cython_6cygrpc_18_ActiveThreadCount_2increment(CYTHON_UNUSED PyObject *__pyx_self, PyObject *__pyx_v_self) {
  PyObject *__pyx_r = NULL;
  __Pyx_RefNannyDeclarations
  PyObject *__pyx_t_1 = NULL;
  PyObject *__pyx_t_2 = NULL;
  PyObject *__pyx_t_3 = NULL;
  PyObject *__pyx_t_4 = NULL;
  PyObject *__pyx_t_5 = NULL;
  PyObject *__pyx_t_6 = NULL;
  PyObject *__pyx_t_7 = NULL;
  PyObject *__pyx_t_8 = NULL;
  PyObject *__pyx_t_9 = NULL;
  int __pyx_t_10;
  int __pyx_t_11;
  __Pyx_RefNannySetupContext("increment", 0);

  /* "src/python/grpcio/grpc/_cython/_cygrpc/fork_posix.pyx.pxi":172
 * 
 *     def increment(self):
 *         with self._condition:             # <<<<<<<<<<<<<<
 *             self._num_active_threads += 1
 * 
 */
  /*with:*/ {
    __pyx_t_1 = __Pyx_PyObject_GetAttrStr(__pyx_v_self, __pyx_n_s_condition); if (unlikely(!__pyx_t_1)) __PYX_ERR(0, 172, __pyx_L1_error)
    __Pyx_GOTREF(__pyx_t_1);
    __pyx_t_2 = __Pyx_PyObject_LookupSpecial(__pyx_t_1, __pyx_n_s_exit); if (unlikely(!__pyx_t_2)) __PYX_ERR(0, 172, __pyx_L1_error)
    __Pyx_GOTREF(__pyx_t_2);
    __pyx_t_4 = __Pyx_PyObject_LookupSpecial(__pyx_t_1, __pyx_n_s_enter); if (unlikely(!__pyx_t_4)) __PYX_ERR(0, 172, __pyx_L3_error)
    __Pyx_GOTREF(__pyx_t_4);
    __pyx_t_5 = NULL;
    if (CYTHON_UNPACK_METHODS && likely(PyMethod_Check(__pyx_t_4))) {
      __pyx_t_5 = PyMethod_GET_SELF(__pyx_t_4);
      if (likely(__pyx_t_5)) {
        PyObject* function = PyMethod_GET_FUNCTION(__pyx_t_4);
        __Pyx_INCREF(__pyx_t_5);
        __Pyx_INCREF(function);
        __Pyx_DECREF_SET(__pyx_t_4, function);
      }
    }
    if (__pyx_t_5) {
      __pyx_t_3 = __Pyx_PyObject_CallOneArg(__pyx_t_4, __pyx_t_5); if (unlikely(!__pyx_t_3)) __PYX_ERR(0, 172, __pyx_L3_error)
      __Pyx_DECREF(__pyx_t_5); __pyx_t_5 = 0;
    } else {
      __pyx_t_3 = __Pyx_PyObject_CallNoArg(__pyx_t_4); if (unlikely(!__pyx_t_3)) __PYX_ERR(0, 172, __pyx_L3_error)
    }
    __Pyx_GOTREF(__pyx_t_3);
    __Pyx_DECREF(__pyx_t_4); __pyx_t_4 = 0;
    __Pyx_DECREF(__pyx_t_3); __pyx_t_3 = 0;
    __Pyx_DECREF(__pyx_t_1); __pyx_t_1 = 0;
    /*try:*/ {
      {
        __Pyx_PyThreadState_declare
        __Pyx_PyThreadState_assign
        __Pyx_ExceptionSave(&__pyx_t_6, &__pyx_t_7, &__pyx_t_8);
        __Pyx_XGOTREF(__pyx_t_6);
        __Pyx_XGOTREF(__pyx_t_7);
        __Pyx_XGOTREF(__pyx_t_8);
        /*try:*/ {

          /* "src/python/grpcio/grpc/_cython/_cygrpc/fork_posix.pyx.pxi":173
 *     def increment(self):
 *         with self._condition:
 *             self._num_active_threads += 1             # <<<<<<<<<<<<<<
 * 
 *     def decrement(self):
 */
          __pyx_t_1 = __Pyx_PyObject_GetAttrStr(__pyx_v_self, __pyx_n_s_num_active_threads); if (unlikely(!__pyx_t_1)) __PYX_ERR(0, 173, __pyx_L7_error)
          __Pyx_GOTREF(__pyx_t_1);
          __pyx_t_3 = __Pyx_PyInt_AddObjC(__pyx_t_1, __pyx_int_1, 1, 1); if (unlikely(!__pyx_t_3)) __PYX_ERR(0, 173, __pyx_L7_error)
          __Pyx_GOTREF(__pyx_t_3);
          __Pyx_DECREF(__pyx_t_1); __pyx_t_1 = 0;
          if (__Pyx_PyObject_SetAttrStr(__pyx_v_self, __pyx_n_s_num_active_threads, __pyx_t_3) < 0) __PYX_ERR(0, 173, __pyx_L7_error)
          __Pyx_DECREF(__pyx_t_3); __pyx_t_3 = 0;

          /* "src/python/grpcio/grpc/_cython/_cygrpc/fork_posix.pyx.pxi":172
 * 
 *     def increment(self):
 *         with self._condition:             # <<<<<<<<<<<<<<
 *             self._num_active_threads += 1
 * 
 */
        }
        __Pyx_XDECREF(__pyx_t_6); __pyx_t_6 = 0;
        __Pyx_XDECREF(__pyx_t_7); __pyx_t_7 = 0;
        __Pyx_XDECREF(__pyx_t_8); __pyx_t_8 = 0;
        goto __pyx_L12_try_end;
        __pyx_L7_error:;
        __Pyx_XDECREF(__pyx_t_5); __pyx_t_5 = 0;
        __Pyx_XDECREF(__pyx_t_4); __pyx_t_4 = 0;
        __Pyx_XDECREF(__pyx_t_1); __pyx_t_1 = 0;
        __Pyx_XDECREF(__pyx_t_3); __pyx_t_3 = 0;
        /*except:*/ {
          __Pyx_AddTraceback("grpc._cython.cygrpc._ActiveThreadCount.increment", __pyx_clineno, __pyx_lineno, __pyx_filename);
          if (__Pyx_GetException(&__pyx_t_3, &__pyx_t_1, &__pyx_t_4) < 0) __PYX_ERR(0, 172, __pyx_L9_except_error)
          __Pyx_GOTREF(__pyx_t_3);
          __Pyx_GOTREF(__pyx_t_1);
          __Pyx_GOTREF(__pyx_t_4);
          __pyx_t_5 = PyTuple_Pack(3, __pyx_t_3, __pyx_t_1, __pyx_t_4); if (unlikely(!__pyx_t_5)) __PYX_ERR(0, 172, __pyx_L9_except_error)
          __Pyx_GOTREF(__pyx_t_5);
          __pyx_t_9 = __Pyx_PyObject_Call(__pyx_t_2, __pyx_t_5, NULL);
          __Pyx_DECREF(__pyx_t_2); __pyx_t_2 = 0;
          __Pyx_DECREF(__pyx_t_5); __pyx_t_5 = 0;
          if (unlikely(!__pyx_t_9)) __PYX_ERR(0, 172, __pyx_L9_except_error)
          __Pyx_GOTREF(__pyx_t_9);
          __pyx_t_10 = __Pyx_PyObject_IsTrue(__pyx_t_9);
          __Pyx_DECREF(__pyx_t_9); __pyx_t_9 = 0;
          if (__pyx_t_10 < 0) __PYX_ERR(0, 172, __pyx_L9_except_error)
          __pyx_t_11 = ((!(__pyx_t_10 != 0)) != 0);
          if (__pyx_t_11) {
            __Pyx_GIVEREF(__pyx_t_3);
            __Pyx_GIVEREF(__pyx_t_1);
            __Pyx_XGIVEREF(__pyx_t_4);
            __Pyx_ErrRestoreWithState(__pyx_t_3, __pyx_t_1, __pyx_t_4);
            __pyx_t_3 = 0; __pyx_t_1 = 0; __pyx_t_4 = 0; 
            __PYX_ERR(0, 172, __pyx_L9_except_error)
          }
          __Pyx_DECREF(__pyx_t_3); __pyx_t_3 = 0;
          __Pyx_DECREF(__pyx_t_1); __pyx_t_1 = 0;
          __Pyx_DECREF(__pyx_t_4); __pyx_t_4 = 0;
          goto __pyx_L8_exception_handled;
        }
        __pyx_L9_except_error:;
        __Pyx_XGIVEREF(__pyx_t_6);
        __Pyx_XGIVEREF(__pyx_t_7);
        __Pyx_XGIVEREF(__pyx_t_8);
        __Pyx_ExceptionReset(__pyx_t_6, __pyx_t_7, __pyx_t_8);
        goto __pyx_L1_error;
        __pyx_L8_exception_handled:;
        __Pyx_XGIVEREF(__pyx_t_6);
        __Pyx_XGIVEREF(__pyx_t_7);
        __Pyx_XGIVEREF(__pyx_t_8);
        __Pyx_ExceptionReset(__pyx_t_6, __pyx_t_7, __pyx_t_8);
        __pyx_L12_try_end:;
      }
    }
    /*finally:*/ {
      /*normal exit:*/{
        if (__pyx_t_2) {
          __pyx_t_8 = __Pyx_PyObject_Call(__pyx_t_2, __pyx_tuple__159, NULL);
          __Pyx_DECREF(__pyx_t_2); __pyx_t_2 = 0;
          if (unlikely(!__pyx_t_8)) __PYX_ERR(0, 172, __pyx_L1_error)
          __Pyx_GOTREF(__pyx_t_8);
          __Pyx_DECREF(__pyx_t_8); __pyx_t_8 = 0;
        }
        goto __pyx_L6;
      }
      __pyx_L6:;
    }
    goto __pyx_L16;
    __pyx_L3_error:;
    __Pyx_DECREF(__pyx_t_2); __pyx_t_2 = 0;
    goto __pyx_L1_error;
    __pyx_L16:;
  }

  /* "src/python/grpcio/grpc/_cython/_cygrpc/fork_posix.pyx.pxi":171
 *         self._condition = threading.Condition()
 * 
 *     def increment(self):             # <<<<<<<<<<<<<<
 *         with self._condition:
 *             self._num_active_threads += 1
 */

  /* function exit code */
  __pyx_r = Py_None; __Pyx_INCREF(Py_None);
  goto __pyx_L0;
  __pyx_L1_error:;
  __Pyx_XDECREF(__pyx_t_1);
  __Pyx_XDECREF(__pyx_t_3);
  __Pyx_XDECREF(__pyx_t_4);
  __Pyx_XDECREF(__pyx_t_5);
  __Pyx_AddTraceback("grpc._cython.cygrpc._ActiveThreadCount.increment", __pyx_clineno, __pyx_lineno, __pyx_filename);
  __pyx_r = NULL;
  __pyx_L0:;
  __Pyx_XGIVEREF(__pyx_r);
  __Pyx_RefNannyFinishContext();
  return __pyx_r;
}

/* "src/python/grpcio/grpc/_cython/_cygrpc/fork_posix.pyx.pxi":175
 *             self._num_active_threads += 1
 * 
 *     def decrement(self):             # <<<<<<<<<<<<<<
 *         with self._condition:
 *             self._num_active_threads -= 1
 */

/* Python wrapper */
static PyObject *__pyx_pw_4grpc_7_cython_6cygrpc_18_ActiveThreadCount_5decrement(PyObject *__pyx_self, PyObject *__pyx_v_self); /*proto*/
static PyMethodDef __pyx_mdef_4grpc_7_cython_6cygrpc_18_ActiveThreadCount_5decrement = {"decrement", (PyCFunction)__pyx_pw_4grpc_7_cython_6cygrpc_18_ActiveThreadCount_5decrement, METH_O, 0};
static PyObject *__pyx_pw_4grpc_7_cython_6cygrpc_18_ActiveThreadCount_5decrement(PyObject *__pyx_self, PyObject *__pyx_v_self) {
  PyObject *__pyx_r = 0;
  __Pyx_RefNannyDeclarations
  __Pyx_RefNannySetupContext("decrement (wrapper)", 0);
  __pyx_r = __pyx_pf_4grpc_7_cython_6cygrpc_18_ActiveThreadCount_4decrement(__pyx_self, ((PyObject *)__pyx_v_self));

  /* function exit code */
  __Pyx_RefNannyFinishContext();
  return __pyx_r;
}

static PyObject *__pyx_pf_4grpc_7_cython_6cygrpc_18_ActiveThreadCount_4decrement(CYTHON_UNUSED PyObject *__pyx_self, PyObject *__pyx_v_self) {
  PyObject *__pyx_r = NULL;
  __Pyx_RefNannyDeclarations
  PyObject *__pyx_t_1 = NULL;
  PyObject *__pyx_t_2 = NULL;
  PyObject *__pyx_t_3 = NULL;
  PyObject *__pyx_t_4 = NULL;
  PyObject *__pyx_t_5 = NULL;
  PyObject *__pyx_t_6 = NULL;
  PyObject *__pyx_t_7 = NULL;
  PyObject *__pyx_t_8 = NULL;
  int __pyx_t_9;
  PyObject *__pyx_t_10 = NULL;
  int __pyx_t_11;
  __Pyx_RefNannySetupContext("decrement", 0);

  /* "src/python/grpcio/grpc/_cython/_cygrpc/fork_posix.pyx.pxi":176
 * 
 *     def decrement(self):
 *         with self._condition:             # <<<<<<<<<<<<<<
 *             self._num_active_threads -= 1
 *             if self._num_active_threads == 0:
 */
  /*with:*/ {
    __pyx_t_1 = __Pyx_PyObject_GetAttrStr(__pyx_v_self, __pyx_n_s_condition); if (unlikely(!__pyx_t_1)) __PYX_ERR(0, 176, __pyx_L1_error)
    __Pyx_GOTREF(__pyx_t_1);
    __pyx_t_2 = __Pyx_PyObject_LookupSpecial(__pyx_t_1, __pyx_n_s_exit); if (unlikely(!__pyx_t_2)) __PYX_ERR(0, 176, __pyx_L1_error)
    __Pyx_GOTREF(__pyx_t_2);
    __pyx_t_4 = __Pyx_PyObject_LookupSpecial(__pyx_t_1, __pyx_n_s_enter); if (unlikely(!__pyx_t_4)) __PYX_ERR(0, 176, __pyx_L3_error)
    __Pyx_GOTREF(__pyx_t_4);
    __pyx_t_5 = NULL;
    if (CYTHON_UNPACK_METHODS && likely(PyMethod_Check(__pyx_t_4))) {
      __pyx_t_5 = PyMethod_GET_SELF(__pyx_t_4);
      if (likely(__pyx_t_5)) {
        PyObject* function = PyMethod_GET_FUNCTION(__pyx_t_4);
        __Pyx_INCREF(__pyx_t_5);
        __Pyx_INCREF(function);
        __Pyx_DECREF_SET(__pyx_t_4, function);
      }
    }
    if (__pyx_t_5) {
      __pyx_t_3 = __Pyx_PyObject_CallOneArg(__pyx_t_4, __pyx_t_5); if (unlikely(!__pyx_t_3)) __PYX_ERR(0, 176, __pyx_L3_error)
      __Pyx_DECREF(__pyx_t_5); __pyx_t_5 = 0;
    } else {
      __pyx_t_3 = __Pyx_PyObject_CallNoArg(__pyx_t_4); if (unlikely(!__pyx_t_3)) __PYX_ERR(0, 176, __pyx_L3_error)
    }
    __Pyx_GOTREF(__pyx_t_3);
    __Pyx_DECREF(__pyx_t_4); __pyx_t_4 = 0;
    __Pyx_DECREF(__pyx_t_3); __pyx_t_3 = 0;
    __Pyx_DECREF(__pyx_t_1); __pyx_t_1 = 0;
    /*try:*/ {
      {
        __Pyx_PyThreadState_declare
        __Pyx_PyThreadState_assign
        __Pyx_ExceptionSave(&__pyx_t_6, &__pyx_t_7, &__pyx_t_8);
        __Pyx_XGOTREF(__pyx_t_6);
        __Pyx_XGOTREF(__pyx_t_7);
        __Pyx_XGOTREF(__pyx_t_8);
        /*try:*/ {

          /* "src/python/grpcio/grpc/_cython/_cygrpc/fork_posix.pyx.pxi":177
 *     def decrement(self):
 *         with self._condition:
 *             self._num_active_threads -= 1             # <<<<<<<<<<<<<<
 *             if self._num_active_threads == 0:
 *                 self._condition.notify_all()
 */
          __pyx_t_1 = __Pyx_PyObject_GetAttrStr(__pyx_v_self, __pyx_n_s_num_active_threads); if (unlikely(!__pyx_t_1)) __PYX_ERR(0, 177, __pyx_L7_error)
          __Pyx_GOTREF(__pyx_t_1);
          __pyx_t_3 = __Pyx_PyInt_SubtractObjC(__pyx_t_1, __pyx_int_1, 1, 1); if (unlikely(!__pyx_t_3)) __PYX_ERR(0, 177, __pyx_L7_error)
          __Pyx_GOTREF(__pyx_t_3);
          __Pyx_DECREF(__pyx_t_1); __pyx_t_1 = 0;
          if (__Pyx_PyObject_SetAttrStr(__pyx_v_self, __pyx_n_s_num_active_threads, __pyx_t_3) < 0) __PYX_ERR(0, 177, __pyx_L7_error)
          __Pyx_DECREF(__pyx_t_3); __pyx_t_3 = 0;

          /* "src/python/grpcio/grpc/_cython/_cygrpc/fork_posix.pyx.pxi":178
 *         with self._condition:
 *             self._num_active_threads -= 1
 *             if self._num_active_threads == 0:             # <<<<<<<<<<<<<<
 *                 self._condition.notify_all()
 * 
 */
          __pyx_t_3 = __Pyx_PyObject_GetAttrStr(__pyx_v_self, __pyx_n_s_num_active_threads); if (unlikely(!__pyx_t_3)) __PYX_ERR(0, 178, __pyx_L7_error)
          __Pyx_GOTREF(__pyx_t_3);
          __pyx_t_1 = __Pyx_PyInt_EqObjC(__pyx_t_3, __pyx_int_0, 0, 0); if (unlikely(!__pyx_t_1)) __PYX_ERR(0, 178, __pyx_L7_error)
          __Pyx_GOTREF(__pyx_t_1);
          __Pyx_DECREF(__pyx_t_3); __pyx_t_3 = 0;
          __pyx_t_9 = __Pyx_PyObject_IsTrue(__pyx_t_1); if (unlikely(__pyx_t_9 < 0)) __PYX_ERR(0, 178, __pyx_L7_error)
          __Pyx_DECREF(__pyx_t_1); __pyx_t_1 = 0;
          if (__pyx_t_9) {

            /* "src/python/grpcio/grpc/_cython/_cygrpc/fork_posix.pyx.pxi":179
 *             self._num_active_threads -= 1
 *             if self._num_active_threads == 0:
 *                 self._condition.notify_all()             # <<<<<<<<<<<<<<
 * 
 *     def await_zero_threads(self, timeout_secs):
 */
            __pyx_t_3 = __Pyx_PyObject_GetAttrStr(__pyx_v_self, __pyx_n_s_condition); if (unlikely(!__pyx_t_3)) __PYX_ERR(0, 179, __pyx_L7_error)
            __Pyx_GOTREF(__pyx_t_3);
            __pyx_t_4 = __Pyx_PyObject_GetAttrStr(__pyx_t_3, __pyx_n_s_notify_all); if (unlikely(!__pyx_t_4)) __PYX_ERR(0, 179, __pyx_L7_error)
            __Pyx_GOTREF(__pyx_t_4);
            __Pyx_DECREF(__pyx_t_3); __pyx_t_3 = 0;
            __pyx_t_3 = NULL;
            if (CYTHON_UNPACK_METHODS && likely(PyMethod_Check(__pyx_t_4))) {
              __pyx_t_3 = PyMethod_GET_SELF(__pyx_t_4);
              if (likely(__pyx_t_3)) {
                PyObject* function = PyMethod_GET_FUNCTION(__pyx_t_4);
                __Pyx_INCREF(__pyx_t_3);
                __Pyx_INCREF(function);
                __Pyx_DECREF_SET(__pyx_t_4, function);
              }
            }
            if (__pyx_t_3) {
              __pyx_t_1 = __Pyx_PyObject_CallOneArg(__pyx_t_4, __pyx_t_3); if (unlikely(!__pyx_t_1)) __PYX_ERR(0, 179, __pyx_L7_error)
              __Pyx_DECREF(__pyx_t_3); __pyx_t_3 = 0;
            } else {
              __pyx_t_1 = __Pyx_PyObject_CallNoArg(__pyx_t_4); if (unlikely(!__pyx_t_1)) __PYX_ERR(0, 179, __pyx_L7_error)
            }
            __Pyx_GOTREF(__pyx_t_1);
            __Pyx_DECREF(__pyx_t_4); __pyx_t_4 = 0;
            __Pyx_DECREF(__pyx_t_1); __pyx_t_1 = 0;

            /* "src/python/grpcio/grpc/_cython/_cygrpc/fork_posix.pyx.pxi":178
 *         with self._condition:
 *             self._num_active_threads -= 1
 *             if self._num_active_threads == 0:             # <<<<<<<<<<<<<<
 *                 self._condition.notify_all()
 * 
 */
          }

          /* "src/python/grpcio/grpc/_cython/_cygrpc/fork_posix.pyx.pxi":176
 * 
 *     def decrement(self):
 *         with self._condition:             # <<<<<<<<<<<<<<
 *             self._num_active_threads -= 1
 *             if self._num_active_threads == 0:
 */
        }
        __Pyx_XDECREF(__pyx_t_6); __pyx_t_6 = 0;
        __Pyx_XDECREF(__pyx_t_7); __pyx_t_7 = 0;
        __Pyx_XDECREF(__pyx_t_8); __pyx_t_8 = 0;
        goto __pyx_L12_try_end;
        __pyx_L7_error:;
        __Pyx_XDECREF(__pyx_t_5); __pyx_t_5 = 0;
        __Pyx_XDECREF(__pyx_t_3); __pyx_t_3 = 0;
        __Pyx_XDECREF(__pyx_t_4); __pyx_t_4 = 0;
        __Pyx_XDECREF(__pyx_t_1); __pyx_t_1 = 0;
        /*except:*/ {
          __Pyx_AddTraceback("grpc._cython.cygrpc._ActiveThreadCount.decrement", __pyx_clineno, __pyx_lineno, __pyx_filename);
          if (__Pyx_GetException(&__pyx_t_1, &__pyx_t_4, &__pyx_t_3) < 0) __PYX_ERR(0, 176, __pyx_L9_except_error)
          __Pyx_GOTREF(__pyx_t_1);
          __Pyx_GOTREF(__pyx_t_4);
          __Pyx_GOTREF(__pyx_t_3);
          __pyx_t_5 = PyTuple_Pack(3, __pyx_t_1, __pyx_t_4, __pyx_t_3); if (unlikely(!__pyx_t_5)) __PYX_ERR(0, 176, __pyx_L9_except_error)
          __Pyx_GOTREF(__pyx_t_5);
          __pyx_t_10 = __Pyx_PyObject_Call(__pyx_t_2, __pyx_t_5, NULL);
          __Pyx_DECREF(__pyx_t_2); __pyx_t_2 = 0;
          __Pyx_DECREF(__pyx_t_5); __pyx_t_5 = 0;
          if (unlikely(!__pyx_t_10)) __PYX_ERR(0, 176, __pyx_L9_except_error)
          __Pyx_GOTREF(__pyx_t_10);
          __pyx_t_9 = __Pyx_PyObject_IsTrue(__pyx_t_10);
          __Pyx_DECREF(__pyx_t_10); __pyx_t_10 = 0;
          if (__pyx_t_9 < 0) __PYX_ERR(0, 176, __pyx_L9_except_error)
          __pyx_t_11 = ((!(__pyx_t_9 != 0)) != 0);
          if (__pyx_t_11) {
            __Pyx_GIVEREF(__pyx_t_1);
            __Pyx_GIVEREF(__pyx_t_4);
            __Pyx_XGIVEREF(__pyx_t_3);
            __Pyx_ErrRestoreWithState(__pyx_t_1, __pyx_t_4, __pyx_t_3);
            __pyx_t_1 = 0; __pyx_t_4 = 0; __pyx_t_3 = 0; 
            __PYX_ERR(0, 176, __pyx_L9_except_error)
          }
          __Pyx_DECREF(__pyx_t_1); __pyx_t_1 = 0;
          __Pyx_DECREF(__pyx_t_4); __pyx_t_4 = 0;
          __Pyx_DECREF(__pyx_t_3); __pyx_t_3 = 0;
          goto __pyx_L8_exception_handled;
        }
        __pyx_L9_except_error:;
        __Pyx_XGIVEREF(__pyx_t_6);
        __Pyx_XGIVEREF(__pyx_t_7);
        __Pyx_XGIVEREF(__pyx_t_8);
        __Pyx_ExceptionReset(__pyx_t_6, __pyx_t_7, __pyx_t_8);
        goto __pyx_L1_error;
        __pyx_L8_exception_handled:;
        __Pyx_XGIVEREF(__pyx_t_6);
        __Pyx_XGIVEREF(__pyx_t_7);
        __Pyx_XGIVEREF(__pyx_t_8);
        __Pyx_ExceptionReset(__pyx_t_6, __pyx_t_7, __pyx_t_8);
        __pyx_L12_try_end:;
      }
    }
    /*finally:*/ {
      /*normal exit:*/{
        if (__pyx_t_2) {
          __pyx_t_8 = __Pyx_PyObject_Call(__pyx_t_2, __pyx_tuple__160, NULL);
          __Pyx_DECREF(__pyx_t_2); __pyx_t_2 = 0;
          if (unlikely(!__pyx_t_8)) __PYX_ERR(0, 176, __pyx_L1_error)
          __Pyx_GOTREF(__pyx_t_8);
          __Pyx_DECREF(__pyx_t_8); __pyx_t_8 = 0;
        }
        goto __pyx_L6;
      }
      __pyx_L6:;
    }
    goto __pyx_L17;
    __pyx_L3_error:;
    __Pyx_DECREF(__pyx_t_2); __pyx_t_2 = 0;
    goto __pyx_L1_error;
    __pyx_L17:;
  }

  /* "src/python/grpcio/grpc/_cython/_cygrpc/fork_posix.pyx.pxi":175
 *             self._num_active_threads += 1
 * 
 *     def decrement(self):             # <<<<<<<<<<<<<<
 *         with self._condition:
 *             self._num_active_threads -= 1
 */

  /* function exit code */
  __pyx_r = Py_None; __Pyx_INCREF(Py_None);
  goto __pyx_L0;
  __pyx_L1_error:;
  __Pyx_XDECREF(__pyx_t_1);
  __Pyx_XDECREF(__pyx_t_3);
  __Pyx_XDECREF(__pyx_t_4);
  __Pyx_XDECREF(__pyx_t_5);
  __Pyx_AddTraceback("grpc._cython.cygrpc._ActiveThreadCount.decrement", __pyx_clineno, __pyx_lineno, __pyx_filename);
  __pyx_r = NULL;
  __pyx_L0:;
  __Pyx_XGIVEREF(__pyx_r);
  __Pyx_RefNannyFinishContext();
  return __pyx_r;
}

/* "src/python/grpcio/grpc/_cython/_cygrpc/fork_posix.pyx.pxi":181
 *                 self._condition.notify_all()
 * 
 *     def await_zero_threads(self, timeout_secs):             # <<<<<<<<<<<<<<
 *         end_time = time.time() + timeout_secs
 *         wait_time = timeout_secs
 */

/* Python wrapper */
static PyObject *__pyx_pw_4grpc_7_cython_6cygrpc_18_ActiveThreadCount_7await_zero_threads(PyObject *__pyx_self, PyObject *__pyx_args, PyObject *__pyx_kwds); /*proto*/
static PyMethodDef __pyx_mdef_4grpc_7_cython_6cygrpc_18_ActiveThreadCount_7await_zero_threads = {"await_zero_threads", (PyCFunction)__pyx_pw_4grpc_7_cython_6cygrpc_18_ActiveThreadCount_7await_zero_threads, METH_VARARGS|METH_KEYWORDS, 0};
static PyObject *__pyx_pw_4grpc_7_cython_6cygrpc_18_ActiveThreadCount_7await_zero_threads(PyObject *__pyx_self, PyObject *__pyx_args, PyObject *__pyx_kwds) {
  PyObject *__pyx_v_self = 0;
  PyObject *__pyx_v_timeout_secs = 0;
  PyObject *__pyx_r = 0;
  __Pyx_RefNannyDeclarations
  __Pyx_RefNannySetupContext("await_zero_threads (wrapper)", 0);
  {
    static PyObject **__pyx_pyargnames[] = {&__pyx_n_s_self,&__pyx_n_s_timeout_secs,0};
    PyObject* values[2] = {0,0};
    if (unlikely(__pyx_kwds)) {
      Py_ssize_t kw_args;
      const Py_ssize_t pos_args = PyTuple_GET_SIZE(__pyx_args);
      switch (pos_args) {
        case  2: values[1] = PyTuple_GET_ITEM(__pyx_args, 1);
        CYTHON_FALLTHROUGH;
        case  1: values[0] = PyTuple_GET_ITEM(__pyx_args, 0);
        CYTHON_FALLTHROUGH;
        case  0: break;
        default: goto __pyx_L5_argtuple_error;
      }
      kw_args = PyDict_Size(__pyx_kwds);
      switch (pos_args) {
        case  0:
        if (likely((values[0] = __Pyx_PyDict_GetItemStr(__pyx_kwds, __pyx_n_s_self)) != 0)) kw_args--;
        else goto __pyx_L5_argtuple_error;
        CYTHON_FALLTHROUGH;
        case  1:
        if (likely((values[1] = __Pyx_PyDict_GetItemStr(__pyx_kwds, __pyx_n_s_timeout_secs)) != 0)) kw_args--;
        else {
          __Pyx_RaiseArgtupleInvalid("await_zero_threads", 1, 2, 2, 1); __PYX_ERR(0, 181, __pyx_L3_error)
        }
      }
      if (unlikely(kw_args > 0)) {
        if (unlikely(__Pyx_ParseOptionalKeywords(__pyx_kwds, __pyx_pyargnames, 0, values, pos_args, "await_zero_threads") < 0)) __PYX_ERR(0, 181, __pyx_L3_error)
      }
    } else if (PyTuple_GET_SIZE(__pyx_args) != 2) {
      goto __pyx_L5_argtuple_error;
    } else {
      values[0] = PyTuple_GET_ITEM(__pyx_args, 0);
      values[1] = PyTuple_GET_ITEM(__pyx_args, 1);
    }
    __pyx_v_self = values[0];
    __pyx_v_timeout_secs = values[1];
  }
  goto __pyx_L4_argument_unpacking_done;
  __pyx_L5_argtuple_error:;
  __Pyx_RaiseArgtupleInvalid("await_zero_threads", 1, 2, 2, PyTuple_GET_SIZE(__pyx_args)); __PYX_ERR(0, 181, __pyx_L3_error)
  __pyx_L3_error:;
  __Pyx_AddTraceback("grpc._cython.cygrpc._ActiveThreadCount.await_zero_threads", __pyx_clineno, __pyx_lineno, __pyx_filename);
  __Pyx_RefNannyFinishContext();
  return NULL;
  __pyx_L4_argument_unpacking_done:;
  __pyx_r = __pyx_pf_4grpc_7_cython_6cygrpc_18_ActiveThreadCount_6await_zero_threads(__pyx_self, __pyx_v_self, __pyx_v_timeout_secs);

  /* function exit code */
  __Pyx_RefNannyFinishContext();
  return __pyx_r;
}

static PyObject *__pyx_pf_4grpc_7_cython_6cygrpc_18_ActiveThreadCount_6await_zero_threads(CYTHON_UNUSED PyObject *__pyx_self, PyObject *__pyx_v_self, PyObject *__pyx_v_timeout_secs) {
  PyObject *__pyx_v_end_time = NULL;
  PyObject *__pyx_v_wait_time = NULL;
  PyObject *__pyx_r = NULL;
  __Pyx_RefNannyDeclarations
  PyObject *__pyx_t_1 = NULL;
  PyObject *__pyx_t_2 = NULL;
  PyObject *__pyx_t_3 = NULL;
  PyObject *__pyx_t_4 = NULL;
  PyObject *__pyx_t_5 = NULL;
  PyObject *__pyx_t_6 = NULL;
  PyObject *__pyx_t_7 = NULL;
  PyObject *__pyx_t_8 = NULL;
  int __pyx_t_9;
  PyObject *__pyx_t_10 = NULL;
  int __pyx_t_11;
  __Pyx_RefNannySetupContext("await_zero_threads", 0);

  /* "src/python/grpcio/grpc/_cython/_cygrpc/fork_posix.pyx.pxi":182
 * 
 *     def await_zero_threads(self, timeout_secs):
 *         end_time = time.time() + timeout_secs             # <<<<<<<<<<<<<<
 *         wait_time = timeout_secs
 *         with self._condition:
 */
  __pyx_t_2 = __Pyx_GetModuleGlobalName(__pyx_n_s_time); if (unlikely(!__pyx_t_2)) __PYX_ERR(0, 182, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_2);
  __pyx_t_3 = __Pyx_PyObject_GetAttrStr(__pyx_t_2, __pyx_n_s_time); if (unlikely(!__pyx_t_3)) __PYX_ERR(0, 182, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_3);
  __Pyx_DECREF(__pyx_t_2); __pyx_t_2 = 0;
  __pyx_t_2 = NULL;
  if (CYTHON_UNPACK_METHODS && unlikely(PyMethod_Check(__pyx_t_3))) {
    __pyx_t_2 = PyMethod_GET_SELF(__pyx_t_3);
    if (likely(__pyx_t_2)) {
      PyObject* function = PyMethod_GET_FUNCTION(__pyx_t_3);
      __Pyx_INCREF(__pyx_t_2);
      __Pyx_INCREF(function);
      __Pyx_DECREF_SET(__pyx_t_3, function);
    }
  }
  if (__pyx_t_2) {
    __pyx_t_1 = __Pyx_PyObject_CallOneArg(__pyx_t_3, __pyx_t_2); if (unlikely(!__pyx_t_1)) __PYX_ERR(0, 182, __pyx_L1_error)
    __Pyx_DECREF(__pyx_t_2); __pyx_t_2 = 0;
  } else {
    __pyx_t_1 = __Pyx_PyObject_CallNoArg(__pyx_t_3); if (unlikely(!__pyx_t_1)) __PYX_ERR(0, 182, __pyx_L1_error)
  }
  __Pyx_GOTREF(__pyx_t_1);
  __Pyx_DECREF(__pyx_t_3); __pyx_t_3 = 0;
  __pyx_t_3 = PyNumber_Add(__pyx_t_1, __pyx_v_timeout_secs); if (unlikely(!__pyx_t_3)) __PYX_ERR(0, 182, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_3);
  __Pyx_DECREF(__pyx_t_1); __pyx_t_1 = 0;
  __pyx_v_end_time = __pyx_t_3;
  __pyx_t_3 = 0;

  /* "src/python/grpcio/grpc/_cython/_cygrpc/fork_posix.pyx.pxi":183
 *     def await_zero_threads(self, timeout_secs):
 *         end_time = time.time() + timeout_secs
 *         wait_time = timeout_secs             # <<<<<<<<<<<<<<
 *         with self._condition:
 *             while True:
 */
  __Pyx_INCREF(__pyx_v_timeout_secs);
  __pyx_v_wait_time = __pyx_v_timeout_secs;

  /* "src/python/grpcio/grpc/_cython/_cygrpc/fork_posix.pyx.pxi":184
 *         end_time = time.time() + timeout_secs
 *         wait_time = timeout_secs
 *         with self._condition:             # <<<<<<<<<<<<<<
 *             while True:
 *                 if self._num_active_threads > 0:
 */
  /*with:*/ {
    __pyx_t_3 = __Pyx_PyObject_GetAttrStr(__pyx_v_self, __pyx_n_s_condition); if (unlikely(!__pyx_t_3)) __PYX_ERR(0, 184, __pyx_L1_error)
    __Pyx_GOTREF(__pyx_t_3);
    __pyx_t_4 = __Pyx_PyObject_LookupSpecial(__pyx_t_3, __pyx_n_s_exit); if (unlikely(!__pyx_t_4)) __PYX_ERR(0, 184, __pyx_L1_error)
    __Pyx_GOTREF(__pyx_t_4);
    __pyx_t_2 = __Pyx_PyObject_LookupSpecial(__pyx_t_3, __pyx_n_s_enter); if (unlikely(!__pyx_t_2)) __PYX_ERR(0, 184, __pyx_L3_error)
    __Pyx_GOTREF(__pyx_t_2);
    __pyx_t_5 = NULL;
    if (CYTHON_UNPACK_METHODS && likely(PyMethod_Check(__pyx_t_2))) {
      __pyx_t_5 = PyMethod_GET_SELF(__pyx_t_2);
      if (likely(__pyx_t_5)) {
        PyObject* function = PyMethod_GET_FUNCTION(__pyx_t_2);
        __Pyx_INCREF(__pyx_t_5);
        __Pyx_INCREF(function);
        __Pyx_DECREF_SET(__pyx_t_2, function);
      }
    }
    if (__pyx_t_5) {
      __pyx_t_1 = __Pyx_PyObject_CallOneArg(__pyx_t_2, __pyx_t_5); if (unlikely(!__pyx_t_1)) __PYX_ERR(0, 184, __pyx_L3_error)
      __Pyx_DECREF(__pyx_t_5); __pyx_t_5 = 0;
    } else {
      __pyx_t_1 = __Pyx_PyObject_CallNoArg(__pyx_t_2); if (unlikely(!__pyx_t_1)) __PYX_ERR(0, 184, __pyx_L3_error)
    }
    __Pyx_GOTREF(__pyx_t_1);
    __Pyx_DECREF(__pyx_t_2); __pyx_t_2 = 0;
    __Pyx_DECREF(__pyx_t_1); __pyx_t_1 = 0;
    __Pyx_DECREF(__pyx_t_3); __pyx_t_3 = 0;
    /*try:*/ {
      {
        __Pyx_PyThreadState_declare
        __Pyx_PyThreadState_assign
        __Pyx_ExceptionSave(&__pyx_t_6, &__pyx_t_7, &__pyx_t_8);
        __Pyx_XGOTREF(__pyx_t_6);
        __Pyx_XGOTREF(__pyx_t_7);
        __Pyx_XGOTREF(__pyx_t_8);
        /*try:*/ {

          /* "src/python/grpcio/grpc/_cython/_cygrpc/fork_posix.pyx.pxi":185
 *         wait_time = timeout_secs
 *         with self._condition:
 *             while True:             # <<<<<<<<<<<<<<
 *                 if self._num_active_threads > 0:
 *                     self._condition.wait(wait_time)
 */
          while (1) {

            /* "src/python/grpcio/grpc/_cython/_cygrpc/fork_posix.pyx.pxi":186
 *         with self._condition:
 *             while True:
 *                 if self._num_active_threads > 0:             # <<<<<<<<<<<<<<
 *                     self._condition.wait(wait_time)
 *                 if self._num_active_threads == 0:
 */
            __pyx_t_3 = __Pyx_PyObject_GetAttrStr(__pyx_v_self, __pyx_n_s_num_active_threads); if (unlikely(!__pyx_t_3)) __PYX_ERR(0, 186, __pyx_L7_error)
            __Pyx_GOTREF(__pyx_t_3);
            __pyx_t_1 = PyObject_RichCompare(__pyx_t_3, __pyx_int_0, Py_GT); __Pyx_XGOTREF(__pyx_t_1); if (unlikely(!__pyx_t_1)) __PYX_ERR(0, 186, __pyx_L7_error)
            __Pyx_DECREF(__pyx_t_3); __pyx_t_3 = 0;
            __pyx_t_9 = __Pyx_PyObject_IsTrue(__pyx_t_1); if (unlikely(__pyx_t_9 < 0)) __PYX_ERR(0, 186, __pyx_L7_error)
            __Pyx_DECREF(__pyx_t_1); __pyx_t_1 = 0;
            if (__pyx_t_9) {

              /* "src/python/grpcio/grpc/_cython/_cygrpc/fork_posix.pyx.pxi":187
 *             while True:
 *                 if self._num_active_threads > 0:
 *                     self._condition.wait(wait_time)             # <<<<<<<<<<<<<<
 *                 if self._num_active_threads == 0:
 *                     return True
 */
              __pyx_t_3 = __Pyx_PyObject_GetAttrStr(__pyx_v_self, __pyx_n_s_condition); if (unlikely(!__pyx_t_3)) __PYX_ERR(0, 187, __pyx_L7_error)
              __Pyx_GOTREF(__pyx_t_3);
              __pyx_t_2 = __Pyx_PyObject_GetAttrStr(__pyx_t_3, __pyx_n_s_wait); if (unlikely(!__pyx_t_2)) __PYX_ERR(0, 187, __pyx_L7_error)
              __Pyx_GOTREF(__pyx_t_2);
              __Pyx_DECREF(__pyx_t_3); __pyx_t_3 = 0;
              __pyx_t_3 = NULL;
              if (CYTHON_UNPACK_METHODS && likely(PyMethod_Check(__pyx_t_2))) {
                __pyx_t_3 = PyMethod_GET_SELF(__pyx_t_2);
                if (likely(__pyx_t_3)) {
                  PyObject* function = PyMethod_GET_FUNCTION(__pyx_t_2);
                  __Pyx_INCREF(__pyx_t_3);
                  __Pyx_INCREF(function);
                  __Pyx_DECREF_SET(__pyx_t_2, function);
                }
              }
              if (!__pyx_t_3) {
                __pyx_t_1 = __Pyx_PyObject_CallOneArg(__pyx_t_2, __pyx_v_wait_time); if (unlikely(!__pyx_t_1)) __PYX_ERR(0, 187, __pyx_L7_error)
                __Pyx_GOTREF(__pyx_t_1);
              } else {
                #if CYTHON_FAST_PYCALL
                if (PyFunction_Check(__pyx_t_2)) {
                  PyObject *__pyx_temp[2] = {__pyx_t_3, __pyx_v_wait_time};
                  __pyx_t_1 = __Pyx_PyFunction_FastCall(__pyx_t_2, __pyx_temp+1-1, 1+1); if (unlikely(!__pyx_t_1)) __PYX_ERR(0, 187, __pyx_L7_error)
                  __Pyx_XDECREF(__pyx_t_3); __pyx_t_3 = 0;
                  __Pyx_GOTREF(__pyx_t_1);
                } else
                #endif
                #if CYTHON_FAST_PYCCALL
                if (__Pyx_PyFastCFunction_Check(__pyx_t_2)) {
                  PyObject *__pyx_temp[2] = {__pyx_t_3, __pyx_v_wait_time};
                  __pyx_t_1 = __Pyx_PyCFunction_FastCall(__pyx_t_2, __pyx_temp+1-1, 1+1); if (unlikely(!__pyx_t_1)) __PYX_ERR(0, 187, __pyx_L7_error)
                  __Pyx_XDECREF(__pyx_t_3); __pyx_t_3 = 0;
                  __Pyx_GOTREF(__pyx_t_1);
                } else
                #endif
                {
                  __pyx_t_5 = PyTuple_New(1+1); if (unlikely(!__pyx_t_5)) __PYX_ERR(0, 187, __pyx_L7_error)
                  __Pyx_GOTREF(__pyx_t_5);
                  __Pyx_GIVEREF(__pyx_t_3); PyTuple_SET_ITEM(__pyx_t_5, 0, __pyx_t_3); __pyx_t_3 = NULL;
                  __Pyx_INCREF(__pyx_v_wait_time);
                  __Pyx_GIVEREF(__pyx_v_wait_time);
                  PyTuple_SET_ITEM(__pyx_t_5, 0+1, __pyx_v_wait_time);
                  __pyx_t_1 = __Pyx_PyObject_Call(__pyx_t_2, __pyx_t_5, NULL); if (unlikely(!__pyx_t_1)) __PYX_ERR(0, 187, __pyx_L7_error)
                  __Pyx_GOTREF(__pyx_t_1);
                  __Pyx_DECREF(__pyx_t_5); __pyx_t_5 = 0;
                }
              }
              __Pyx_DECREF(__pyx_t_2); __pyx_t_2 = 0;
              __Pyx_DECREF(__pyx_t_1); __pyx_t_1 = 0;

              /* "src/python/grpcio/grpc/_cython/_cygrpc/fork_posix.pyx.pxi":186
 *         with self._condition:
 *             while True:
 *                 if self._num_active_threads > 0:             # <<<<<<<<<<<<<<
 *                     self._condition.wait(wait_time)
 *                 if self._num_active_threads == 0:
 */
            }

            /* "src/python/grpcio/grpc/_cython/_cygrpc/fork_posix.pyx.pxi":188
 *                 if self._num_active_threads > 0:
 *                     self._condition.wait(wait_time)
 *                 if self._num_active_threads == 0:             # <<<<<<<<<<<<<<
 *                     return True
 *                 # Thread count may have increased before this re-obtains the
 */
            __pyx_t_1 = __Pyx_PyObject_GetAttrStr(__pyx_v_self, __pyx_n_s_num_active_threads); if (unlikely(!__pyx_t_1)) __PYX_ERR(0, 188, __pyx_L7_error)
            __Pyx_GOTREF(__pyx_t_1);
            __pyx_t_2 = __Pyx_PyInt_EqObjC(__pyx_t_1, __pyx_int_0, 0, 0); if (unlikely(!__pyx_t_2)) __PYX_ERR(0, 188, __pyx_L7_error)
            __Pyx_GOTREF(__pyx_t_2);
            __Pyx_DECREF(__pyx_t_1); __pyx_t_1 = 0;
            __pyx_t_9 = __Pyx_PyObject_IsTrue(__pyx_t_2); if (unlikely(__pyx_t_9 < 0)) __PYX_ERR(0, 188, __pyx_L7_error)
            __Pyx_DECREF(__pyx_t_2); __pyx_t_2 = 0;
            if (__pyx_t_9) {

              /* "src/python/grpcio/grpc/_cython/_cygrpc/fork_posix.pyx.pxi":189
 *                     self._condition.wait(wait_time)
 *                 if self._num_active_threads == 0:
 *                     return True             # <<<<<<<<<<<<<<
 *                 # Thread count may have increased before this re-obtains the
 *                 # lock after a notify(). Wait again until timeout_secs has
 */
              __Pyx_XDECREF(__pyx_r);
              __Pyx_INCREF(Py_True);
              __pyx_r = Py_True;
              goto __pyx_L11_try_return;

              /* "src/python/grpcio/grpc/_cython/_cygrpc/fork_posix.pyx.pxi":188
 *                 if self._num_active_threads > 0:
 *                     self._condition.wait(wait_time)
 *                 if self._num_active_threads == 0:             # <<<<<<<<<<<<<<
 *                     return True
 *                 # Thread count may have increased before this re-obtains the
 */
            }

            /* "src/python/grpcio/grpc/_cython/_cygrpc/fork_posix.pyx.pxi":193
 *                 # lock after a notify(). Wait again until timeout_secs has
 *                 # elapsed.
 *                 wait_time = end_time - time.time()             # <<<<<<<<<<<<<<
 *                 if wait_time <= 0:
 *                     return False
 */
            __pyx_t_1 = __Pyx_GetModuleGlobalName(__pyx_n_s_time); if (unlikely(!__pyx_t_1)) __PYX_ERR(0, 193, __pyx_L7_error)
            __Pyx_GOTREF(__pyx_t_1);
            __pyx_t_5 = __Pyx_PyObject_GetAttrStr(__pyx_t_1, __pyx_n_s_time); if (unlikely(!__pyx_t_5)) __PYX_ERR(0, 193, __pyx_L7_error)
            __Pyx_GOTREF(__pyx_t_5);
            __Pyx_DECREF(__pyx_t_1); __pyx_t_1 = 0;
            __pyx_t_1 = NULL;
            if (CYTHON_UNPACK_METHODS && unlikely(PyMethod_Check(__pyx_t_5))) {
              __pyx_t_1 = PyMethod_GET_SELF(__pyx_t_5);
              if (likely(__pyx_t_1)) {
                PyObject* function = PyMethod_GET_FUNCTION(__pyx_t_5);
                __Pyx_INCREF(__pyx_t_1);
                __Pyx_INCREF(function);
                __Pyx_DECREF_SET(__pyx_t_5, function);
              }
            }
            if (__pyx_t_1) {
              __pyx_t_2 = __Pyx_PyObject_CallOneArg(__pyx_t_5, __pyx_t_1); if (unlikely(!__pyx_t_2)) __PYX_ERR(0, 193, __pyx_L7_error)
              __Pyx_DECREF(__pyx_t_1); __pyx_t_1 = 0;
            } else {
              __pyx_t_2 = __Pyx_PyObject_CallNoArg(__pyx_t_5); if (unlikely(!__pyx_t_2)) __PYX_ERR(0, 193, __pyx_L7_error)
            }
            __Pyx_GOTREF(__pyx_t_2);
            __Pyx_DECREF(__pyx_t_5); __pyx_t_5 = 0;
            __pyx_t_5 = PyNumber_Subtract(__pyx_v_end_time, __pyx_t_2); if (unlikely(!__pyx_t_5)) __PYX_ERR(0, 193, __pyx_L7_error)
            __Pyx_GOTREF(__pyx_t_5);
            __Pyx_DECREF(__pyx_t_2); __pyx_t_2 = 0;
            __Pyx_DECREF_SET(__pyx_v_wait_time, __pyx_t_5);
            __pyx_t_5 = 0;

            /* "src/python/grpcio/grpc/_cython/_cygrpc/fork_posix.pyx.pxi":194
 *                 # elapsed.
 *                 wait_time = end_time - time.time()
 *                 if wait_time <= 0:             # <<<<<<<<<<<<<<
 *                     return False
 * 
 */
            __pyx_t_5 = PyObject_RichCompare(__pyx_v_wait_time, __pyx_int_0, Py_LE); __Pyx_XGOTREF(__pyx_t_5); if (unlikely(!__pyx_t_5)) __PYX_ERR(0, 194, __pyx_L7_error)
            __pyx_t_9 = __Pyx_PyObject_IsTrue(__pyx_t_5); if (unlikely(__pyx_t_9 < 0)) __PYX_ERR(0, 194, __pyx_L7_error)
            __Pyx_DECREF(__pyx_t_5); __pyx_t_5 = 0;
            if (__pyx_t_9) {

              /* "src/python/grpcio/grpc/_cython/_cygrpc/fork_posix.pyx.pxi":195
 *                 wait_time = end_time - time.time()
 *                 if wait_time <= 0:
 *                     return False             # <<<<<<<<<<<<<<
 * 
 * 
 */
              __Pyx_XDECREF(__pyx_r);
              __Pyx_INCREF(Py_False);
              __pyx_r = Py_False;
              goto __pyx_L11_try_return;

              /* "src/python/grpcio/grpc/_cython/_cygrpc/fork_posix.pyx.pxi":194
 *                 # elapsed.
 *                 wait_time = end_time - time.time()
 *                 if wait_time <= 0:             # <<<<<<<<<<<<<<
 *                     return False
 * 
 */
            }
          }

          /* "src/python/grpcio/grpc/_cython/_cygrpc/fork_posix.pyx.pxi":184
 *         end_time = time.time() + timeout_secs
 *         wait_time = timeout_secs
 *         with self._condition:             # <<<<<<<<<<<<<<
 *             while True:
 *                 if self._num_active_threads > 0:
 */
        }
        __Pyx_XDECREF(__pyx_t_6); __pyx_t_6 = 0;
        __Pyx_XDECREF(__pyx_t_7); __pyx_t_7 = 0;
        __Pyx_XDECREF(__pyx_t_8); __pyx_t_8 = 0;
        goto __pyx_L12_try_end;
        __pyx_L7_error:;
        __Pyx_XDECREF(__pyx_t_3); __pyx_t_3 = 0;
        __Pyx_XDECREF(__pyx_t_1); __pyx_t_1 = 0;
        __Pyx_XDECREF(__pyx_t_2); __pyx_t_2 = 0;
        __Pyx_XDECREF(__pyx_t_5); __pyx_t_5 = 0;
        /*except:*/ {
          __Pyx_AddTraceback("grpc._cython.cygrpc._ActiveThreadCount.await_zero_threads", __pyx_clineno, __pyx_lineno, __pyx_filename);
          if (__Pyx_GetException(&__pyx_t_5, &__pyx_t_2, &__pyx_t_1) < 0) __PYX_ERR(0, 184, __pyx_L9_except_error)
          __Pyx_GOTREF(__pyx_t_5);
          __Pyx_GOTREF(__pyx_t_2);
          __Pyx_GOTREF(__pyx_t_1);
          __pyx_t_3 = PyTuple_Pack(3, __pyx_t_5, __pyx_t_2, __pyx_t_1); if (unlikely(!__pyx_t_3)) __PYX_ERR(0, 184, __pyx_L9_except_error)
          __Pyx_GOTREF(__pyx_t_3);
          __pyx_t_10 = __Pyx_PyObject_Call(__pyx_t_4, __pyx_t_3, NULL);
          __Pyx_DECREF(__pyx_t_4); __pyx_t_4 = 0;
          __Pyx_DECREF(__pyx_t_3); __pyx_t_3 = 0;
          if (unlikely(!__pyx_t_10)) __PYX_ERR(0, 184, __pyx_L9_except_error)
          __Pyx_GOTREF(__pyx_t_10);
          __pyx_t_9 = __Pyx_PyObject_IsTrue(__pyx_t_10);
          __Pyx_DECREF(__pyx_t_10); __pyx_t_10 = 0;
          if (__pyx_t_9 < 0) __PYX_ERR(0, 184, __pyx_L9_except_error)
          __pyx_t_11 = ((!(__pyx_t_9 != 0)) != 0);
          if (__pyx_t_11) {
            __Pyx_GIVEREF(__pyx_t_5);
            __Pyx_GIVEREF(__pyx_t_2);
            __Pyx_XGIVEREF(__pyx_t_1);
            __Pyx_ErrRestoreWithState(__pyx_t_5, __pyx_t_2, __pyx_t_1);
            __pyx_t_5 = 0; __pyx_t_2 = 0; __pyx_t_1 = 0; 
            __PYX_ERR(0, 184, __pyx_L9_except_error)
          }
          __Pyx_DECREF(__pyx_t_5); __pyx_t_5 = 0;
          __Pyx_DECREF(__pyx_t_2); __pyx_t_2 = 0;
          __Pyx_DECREF(__pyx_t_1); __pyx_t_1 = 0;
          goto __pyx_L8_exception_handled;
        }
        __pyx_L9_except_error:;
        __Pyx_XGIVEREF(__pyx_t_6);
        __Pyx_XGIVEREF(__pyx_t_7);
        __Pyx_XGIVEREF(__pyx_t_8);
        __Pyx_ExceptionReset(__pyx_t_6, __pyx_t_7, __pyx_t_8);
        goto __pyx_L1_error;
        __pyx_L11_try_return:;
        __Pyx_XGIVEREF(__pyx_t_6);
        __Pyx_XGIVEREF(__pyx_t_7);
        __Pyx_XGIVEREF(__pyx_t_8);
        __Pyx_ExceptionReset(__pyx_t_6, __pyx_t_7, __pyx_t_8);
        goto __pyx_L4_return;
        __pyx_L8_exception_handled:;
        __Pyx_XGIVEREF(__pyx_t_6);
        __Pyx_XGIVEREF(__pyx_t_7);
        __Pyx_XGIVEREF(__pyx_t_8);
        __Pyx_ExceptionReset(__pyx_t_6, __pyx_t_7, __pyx_t_8);
        __pyx_L12_try_end:;
      }
    }
    /*finally:*/ {
      /*normal exit:*/{
        if (__pyx_t_4) {
          __pyx_t_8 = __Pyx_PyObject_Call(__pyx_t_4, __pyx_tuple__161, NULL);
          __Pyx_DECREF(__pyx_t_4); __pyx_t_4 = 0;
          if (unlikely(!__pyx_t_8)) __PYX_ERR(0, 184, __pyx_L1_error)
          __Pyx_GOTREF(__pyx_t_8);
          __Pyx_DECREF(__pyx_t_8); __pyx_t_8 = 0;
        }
        goto __pyx_L6;
      }
      __pyx_L4_return: {
        __pyx_t_8 = __pyx_r;
        __pyx_r = 0;
        if (__pyx_t_4) {
          __pyx_t_7 = __Pyx_PyObject_Call(__pyx_t_4, __pyx_tuple__162, NULL);
          __Pyx_DECREF(__pyx_t_4); __pyx_t_4 = 0;
          if (unlikely(!__pyx_t_7)) __PYX_ERR(0, 184, __pyx_L1_error)
          __Pyx_GOTREF(__pyx_t_7);
          __Pyx_DECREF(__pyx_t_7); __pyx_t_7 = 0;
        }
        __pyx_r = __pyx_t_8;
        __pyx_t_8 = 0;
        goto __pyx_L0;
      }
      __pyx_L6:;
    }
    goto __pyx_L21;
    __pyx_L3_error:;
    __Pyx_DECREF(__pyx_t_4); __pyx_t_4 = 0;
    goto __pyx_L1_error;
    __pyx_L21:;
  }

  /* "src/python/grpcio/grpc/_cython/_cygrpc/fork_posix.pyx.pxi":181
 *                 self._condition.notify_all()
 * 
 *     def await_zero_threads(self, timeout_secs):             # <<<<<<<<<<<<<<
 *         end_time = time.time() + timeout_secs
 *         wait_time = timeout_secs
 */

  /* function exit code */
  __pyx_r = Py_None; __Pyx_INCREF(Py_None);
  goto __pyx_L0;
  __pyx_L1_error:;
  __Pyx_XDECREF(__pyx_t_1);
  __Pyx_XDECREF(__pyx_t_2);
  __Pyx_XDECREF(__pyx_t_3);
  __Pyx_XDECREF(__pyx_t_5);
  __Pyx_AddTraceback("grpc._cython.cygrpc._ActiveThreadCount.await_zero_threads", __pyx_clineno, __pyx_lineno, __pyx_filename);
  __pyx_r = NULL;
  __pyx_L0:;
  __Pyx_XDECREF(__pyx_v_end_time);
  __Pyx_XDECREF(__pyx_v_wait_time);
  __Pyx_XGIVEREF(__pyx_r);
  __Pyx_RefNannyFinishContext();
  return __pyx_r;
}

/* "src/python/grpcio/grpc/_cython/_cygrpc/fork_posix.pyx.pxi":199
 * 
 * class _ForkState(object):
 *     def __init__(self):             # <<<<<<<<<<<<<<
 *         self.fork_in_progress_condition = threading.Condition()
 *         self.fork_in_progress = False
 */

/* Python wrapper */
static PyObject *__pyx_pw_4grpc_7_cython_6cygrpc_10_ForkState_1__init__(PyObject *__pyx_self, PyObject *__pyx_v_self); /*proto*/
static PyMethodDef __pyx_mdef_4grpc_7_cython_6cygrpc_10_ForkState_1__init__ = {"__init__", (PyCFunction)__pyx_pw_4grpc_7_cython_6cygrpc_10_ForkState_1__init__, METH_O, 0};
static PyObject *__pyx_pw_4grpc_7_cython_6cygrpc_10_ForkState_1__init__(PyObject *__pyx_self, PyObject *__pyx_v_self) {
  PyObject *__pyx_r = 0;
  __Pyx_RefNannyDeclarations
  __Pyx_RefNannySetupContext("__init__ (wrapper)", 0);
  __pyx_r = __pyx_pf_4grpc_7_cython_6cygrpc_10_ForkState___init__(__pyx_self, ((PyObject *)__pyx_v_self));

  /* function exit code */
  __Pyx_RefNannyFinishContext();
  return __pyx_r;
}

static PyObject *__pyx_pf_4grpc_7_cython_6cygrpc_10_ForkState___init__(CYTHON_UNUSED PyObject *__pyx_self, PyObject *__pyx_v_self) {
  PyObject *__pyx_r = NULL;
  __Pyx_RefNannyDeclarations
  PyObject *__pyx_t_1 = NULL;
  PyObject *__pyx_t_2 = NULL;
  PyObject *__pyx_t_3 = NULL;
  __Pyx_RefNannySetupContext("__init__", 0);

  /* "src/python/grpcio/grpc/_cython/_cygrpc/fork_posix.pyx.pxi":200
 * class _ForkState(object):
 *     def __init__(self):
 *         self.fork_in_progress_condition = threading.Condition()             # <<<<<<<<<<<<<<
 *         self.fork_in_progress = False
 *         self.postfork_states_to_reset = []
 */
  __pyx_t_2 = __Pyx_GetModuleGlobalName(__pyx_n_s_threading); if (unlikely(!__pyx_t_2)) __PYX_ERR(0, 200, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_2);
  __pyx_t_3 = __Pyx_PyObject_GetAttrStr(__pyx_t_2, __pyx_n_s_Condition); if (unlikely(!__pyx_t_3)) __PYX_ERR(0, 200, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_3);
  __Pyx_DECREF(__pyx_t_2); __pyx_t_2 = 0;
  __pyx_t_2 = NULL;
  if (CYTHON_UNPACK_METHODS && unlikely(PyMethod_Check(__pyx_t_3))) {
    __pyx_t_2 = PyMethod_GET_SELF(__pyx_t_3);
    if (likely(__pyx_t_2)) {
      PyObject* function = PyMethod_GET_FUNCTION(__pyx_t_3);
      __Pyx_INCREF(__pyx_t_2);
      __Pyx_INCREF(function);
      __Pyx_DECREF_SET(__pyx_t_3, function);
    }
  }
  if (__pyx_t_2) {
    __pyx_t_1 = __Pyx_PyObject_CallOneArg(__pyx_t_3, __pyx_t_2); if (unlikely(!__pyx_t_1)) __PYX_ERR(0, 200, __pyx_L1_error)
    __Pyx_DECREF(__pyx_t_2); __pyx_t_2 = 0;
  } else {
    __pyx_t_1 = __Pyx_PyObject_CallNoArg(__pyx_t_3); if (unlikely(!__pyx_t_1)) __PYX_ERR(0, 200, __pyx_L1_error)
  }
  __Pyx_GOTREF(__pyx_t_1);
  __Pyx_DECREF(__pyx_t_3); __pyx_t_3 = 0;
  if (__Pyx_PyObject_SetAttrStr(__pyx_v_self, __pyx_n_s_fork_in_progress_condition, __pyx_t_1) < 0) __PYX_ERR(0, 200, __pyx_L1_error)
  __Pyx_DECREF(__pyx_t_1); __pyx_t_1 = 0;

  /* "src/python/grpcio/grpc/_cython/_cygrpc/fork_posix.pyx.pxi":201
 *     def __init__(self):
 *         self.fork_in_progress_condition = threading.Condition()
 *         self.fork_in_progress = False             # <<<<<<<<<<<<<<
 *         self.postfork_states_to_reset = []
 *         self.fork_handler_registered_lock = threading.Lock()
 */
  if (__Pyx_PyObject_SetAttrStr(__pyx_v_self, __pyx_n_s_fork_in_progress, Py_False) < 0) __PYX_ERR(0, 201, __pyx_L1_error)

  /* "src/python/grpcio/grpc/_cython/_cygrpc/fork_posix.pyx.pxi":202
 *         self.fork_in_progress_condition = threading.Condition()
 *         self.fork_in_progress = False
 *         self.postfork_states_to_reset = []             # <<<<<<<<<<<<<<
 *         self.fork_handler_registered_lock = threading.Lock()
 *         self.fork_handler_registered = False
 */
  __pyx_t_1 = PyList_New(0); if (unlikely(!__pyx_t_1)) __PYX_ERR(0, 202, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_1);
  if (__Pyx_PyObject_SetAttrStr(__pyx_v_self, __pyx_n_s_postfork_states_to_reset, __pyx_t_1) < 0) __PYX_ERR(0, 202, __pyx_L1_error)
  __Pyx_DECREF(__pyx_t_1); __pyx_t_1 = 0;

  /* "src/python/grpcio/grpc/_cython/_cygrpc/fork_posix.pyx.pxi":203
 *         self.fork_in_progress = False
 *         self.postfork_states_to_reset = []
 *         self.fork_handler_registered_lock = threading.Lock()             # <<<<<<<<<<<<<<
 *         self.fork_handler_registered = False
 *         self.active_thread_count = _ActiveThreadCount()
 */
  __pyx_t_3 = __Pyx_GetModuleGlobalName(__pyx_n_s_threading); if (unlikely(!__pyx_t_3)) __PYX_ERR(0, 203, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_3);
  __pyx_t_2 = __Pyx_PyObject_GetAttrStr(__pyx_t_3, __pyx_n_s_Lock); if (unlikely(!__pyx_t_2)) __PYX_ERR(0, 203, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_2);
  __Pyx_DECREF(__pyx_t_3); __pyx_t_3 = 0;
  __pyx_t_3 = NULL;
  if (CYTHON_UNPACK_METHODS && unlikely(PyMethod_Check(__pyx_t_2))) {
    __pyx_t_3 = PyMethod_GET_SELF(__pyx_t_2);
    if (likely(__pyx_t_3)) {
      PyObject* function = PyMethod_GET_FUNCTION(__pyx_t_2);
      __Pyx_INCREF(__pyx_t_3);
      __Pyx_INCREF(function);
      __Pyx_DECREF_SET(__pyx_t_2, function);
    }
  }
  if (__pyx_t_3) {
    __pyx_t_1 = __Pyx_PyObject_CallOneArg(__pyx_t_2, __pyx_t_3); if (unlikely(!__pyx_t_1)) __PYX_ERR(0, 203, __pyx_L1_error)
    __Pyx_DECREF(__pyx_t_3); __pyx_t_3 = 0;
  } else {
    __pyx_t_1 = __Pyx_PyObject_CallNoArg(__pyx_t_2); if (unlikely(!__pyx_t_1)) __PYX_ERR(0, 203, __pyx_L1_error)
  }
  __Pyx_GOTREF(__pyx_t_1);
  __Pyx_DECREF(__pyx_t_2); __pyx_t_2 = 0;
  if (__Pyx_PyObject_SetAttrStr(__pyx_v_self, __pyx_n_s_fork_handler_registered_lock, __pyx_t_1) < 0) __PYX_ERR(0, 203, __pyx_L1_error)
  __Pyx_DECREF(__pyx_t_1); __pyx_t_1 = 0;

  /* "src/python/grpcio/grpc/_cython/_cygrpc/fork_posix.pyx.pxi":204
 *         self.postfork_states_to_reset = []
 *         self.fork_handler_registered_lock = threading.Lock()
 *         self.fork_handler_registered = False             # <<<<<<<<<<<<<<
 *         self.active_thread_count = _ActiveThreadCount()
 *         self.fork_epoch = 0
 */
  if (__Pyx_PyObject_SetAttrStr(__pyx_v_self, __pyx_n_s_fork_handler_registered, Py_False) < 0) __PYX_ERR(0, 204, __pyx_L1_error)

  /* "src/python/grpcio/grpc/_cython/_cygrpc/fork_posix.pyx.pxi":205
 *         self.fork_handler_registered_lock = threading.Lock()
 *         self.fork_handler_registered = False
 *         self.active_thread_count = _ActiveThreadCount()             # <<<<<<<<<<<<<<
 *         self.fork_epoch = 0
 *         self.channels = set()
 */
  __pyx_t_2 = __Pyx_GetModuleGlobalName(__pyx_n_s_ActiveThreadCount); if (unlikely(!__pyx_t_2)) __PYX_ERR(0, 205, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_2);
  __pyx_t_3 = NULL;
  if (CYTHON_UNPACK_METHODS && unlikely(PyMethod_Check(__pyx_t_2))) {
    __pyx_t_3 = PyMethod_GET_SELF(__pyx_t_2);
    if (likely(__pyx_t_3)) {
      PyObject* function = PyMethod_GET_FUNCTION(__pyx_t_2);
      __Pyx_INCREF(__pyx_t_3);
      __Pyx_INCREF(function);
      __Pyx_DECREF_SET(__pyx_t_2, function);
    }
  }
  if (__pyx_t_3) {
    __pyx_t_1 = __Pyx_PyObject_CallOneArg(__pyx_t_2, __pyx_t_3); if (unlikely(!__pyx_t_1)) __PYX_ERR(0, 205, __pyx_L1_error)
    __Pyx_DECREF(__pyx_t_3); __pyx_t_3 = 0;
  } else {
    __pyx_t_1 = __Pyx_PyObject_CallNoArg(__pyx_t_2); if (unlikely(!__pyx_t_1)) __PYX_ERR(0, 205, __pyx_L1_error)
  }
  __Pyx_GOTREF(__pyx_t_1);
  __Pyx_DECREF(__pyx_t_2); __pyx_t_2 = 0;
  if (__Pyx_PyObject_SetAttrStr(__pyx_v_self, __pyx_n_s_active_thread_count, __pyx_t_1) < 0) __PYX_ERR(0, 205, __pyx_L1_error)
  __Pyx_DECREF(__pyx_t_1); __pyx_t_1 = 0;

  /* "src/python/grpcio/grpc/_cython/_cygrpc/fork_posix.pyx.pxi":206
 *         self.fork_handler_registered = False
 *         self.active_thread_count = _ActiveThreadCount()
 *         self.fork_epoch = 0             # <<<<<<<<<<<<<<
 *         self.channels = set()
 * 
 */
  if (__Pyx_PyObject_SetAttrStr(__pyx_v_self, __pyx_n_s_fork_epoch, __pyx_int_0) < 0) __PYX_ERR(0, 206, __pyx_L1_error)

  /* "src/python/grpcio/grpc/_cython/_cygrpc/fork_posix.pyx.pxi":207
 *         self.active_thread_count = _ActiveThreadCount()
 *         self.fork_epoch = 0
 *         self.channels = set()             # <<<<<<<<<<<<<<
 * 
 * 
 */
  __pyx_t_1 = PySet_New(0); if (unlikely(!__pyx_t_1)) __PYX_ERR(0, 207, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_1);
  if (__Pyx_PyObject_SetAttrStr(__pyx_v_self, __pyx_n_s_channels, __pyx_t_1) < 0) __PYX_ERR(0, 207, __pyx_L1_error)
  __Pyx_DECREF(__pyx_t_1); __pyx_t_1 = 0;

  /* "src/python/grpcio/grpc/_cython/_cygrpc/fork_posix.pyx.pxi":199
 * 
 * class _ForkState(object):
 *     def __init__(self):             # <<<<<<<<<<<<<<
 *         self.fork_in_progress_condition = threading.Condition()
 *         self.fork_in_progress = False
 */

  /* function exit code */
  __pyx_r = Py_None; __Pyx_INCREF(Py_None);
  goto __pyx_L0;
  __pyx_L1_error:;
  __Pyx_XDECREF(__pyx_t_1);
  __Pyx_XDECREF(__pyx_t_2);
  __Pyx_XDECREF(__pyx_t_3);
  __Pyx_AddTraceback("grpc._cython.cygrpc._ForkState.__init__", __pyx_clineno, __pyx_lineno, __pyx_filename);
  __pyx_r = NULL;
  __pyx_L0:;
  __Pyx_XGIVEREF(__pyx_r);
  __Pyx_RefNannyFinishContext();
  return __pyx_r;
}

/* "grpc/_cython/cygrpc.pyx":56
 *   int PyEval_InitThreads()
 * 
 * cdef _initialize():             # <<<<<<<<<<<<<<
 *   # We have Python callbacks called by c-core threads, this ensures the GIL
 *   # is initialized.
 */

static PyObject *__pyx_f_4grpc_7_cython_6cygrpc__initialize(void) {
  PyObject *__pyx_r = NULL;
  __Pyx_RefNannyDeclarations
  __Pyx_RefNannySetupContext("_initialize", 0);

  /* "grpc/_cython/cygrpc.pyx":59
 *   # We have Python callbacks called by c-core threads, this ensures the GIL
 *   # is initialized.
 *   PyEval_InitThreads()             # <<<<<<<<<<<<<<
 *   grpc_set_ssl_roots_override_callback(
 *           <grpc_ssl_roots_override_callback>ssl_roots_override_callback)
 */
  (void)(PyEval_InitThreads());

  /* "grpc/_cython/cygrpc.pyx":60
 *   # is initialized.
 *   PyEval_InitThreads()
 *   grpc_set_ssl_roots_override_callback(             # <<<<<<<<<<<<<<
 *           <grpc_ssl_roots_override_callback>ssl_roots_override_callback)
 * 
 */
  grpc_set_ssl_roots_override_callback(((grpc_ssl_roots_override_callback)__pyx_f_4grpc_7_cython_6cygrpc_ssl_roots_override_callback));

  /* "grpc/_cython/cygrpc.pyx":56
 *   int PyEval_InitThreads()
 * 
 * cdef _initialize():             # <<<<<<<<<<<<<<
 *   # We have Python callbacks called by c-core threads, this ensures the GIL
 *   # is initialized.
 */

  /* function exit code */
  __pyx_r = Py_None; __Pyx_INCREF(Py_None);
  __Pyx_XGIVEREF(__pyx_r);
  __Pyx_RefNannyFinishContext();
  return __pyx_r;
}

/* "(tree fragment)":1
 * def __pyx_unpickle_ChannelCredentials(__pyx_type, long __pyx_checksum, __pyx_state):             # <<<<<<<<<<<<<<
 *     if __pyx_checksum != 0xd41d8cd:
 *         from pickle import PickleError as __pyx_PickleError
 */

/* Python wrapper */
static PyObject *__pyx_pw_4grpc_7_cython_6cygrpc_79__pyx_unpickle_ChannelCredentials(PyObject *__pyx_self, PyObject *__pyx_args, PyObject *__pyx_kwds); /*proto*/
static PyMethodDef __pyx_mdef_4grpc_7_cython_6cygrpc_79__pyx_unpickle_ChannelCredentials = {"__pyx_unpickle_ChannelCredentials", (PyCFunction)__pyx_pw_4grpc_7_cython_6cygrpc_79__pyx_unpickle_ChannelCredentials, METH_VARARGS|METH_KEYWORDS, 0};
static PyObject *__pyx_pw_4grpc_7_cython_6cygrpc_79__pyx_unpickle_ChannelCredentials(PyObject *__pyx_self, PyObject *__pyx_args, PyObject *__pyx_kwds) {
  PyObject *__pyx_v___pyx_type = 0;
  long __pyx_v___pyx_checksum;
  PyObject *__pyx_v___pyx_state = 0;
  PyObject *__pyx_r = 0;
  __Pyx_RefNannyDeclarations
  __Pyx_RefNannySetupContext("__pyx_unpickle_ChannelCredentials (wrapper)", 0);
  {
    static PyObject **__pyx_pyargnames[] = {&__pyx_n_s_pyx_type,&__pyx_n_s_pyx_checksum,&__pyx_n_s_pyx_state,0};
    PyObject* values[3] = {0,0,0};
    if (unlikely(__pyx_kwds)) {
      Py_ssize_t kw_args;
      const Py_ssize_t pos_args = PyTuple_GET_SIZE(__pyx_args);
      switch (pos_args) {
        case  3: values[2] = PyTuple_GET_ITEM(__pyx_args, 2);
        CYTHON_FALLTHROUGH;
        case  2: values[1] = PyTuple_GET_ITEM(__pyx_args, 1);
        CYTHON_FALLTHROUGH;
        case  1: values[0] = PyTuple_GET_ITEM(__pyx_args, 0);
        CYTHON_FALLTHROUGH;
        case  0: break;
        default: goto __pyx_L5_argtuple_error;
      }
      kw_args = PyDict_Size(__pyx_kwds);
      switch (pos_args) {
        case  0:
        if (likely((values[0] = __Pyx_PyDict_GetItemStr(__pyx_kwds, __pyx_n_s_pyx_type)) != 0)) kw_args--;
        else goto __pyx_L5_argtuple_error;
        CYTHON_FALLTHROUGH;
        case  1:
        if (likely((values[1] = __Pyx_PyDict_GetItemStr(__pyx_kwds, __pyx_n_s_pyx_checksum)) != 0)) kw_args--;
        else {
          __Pyx_RaiseArgtupleInvalid("__pyx_unpickle_ChannelCredentials", 1, 3, 3, 1); __PYX_ERR(7, 1, __pyx_L3_error)
        }
        CYTHON_FALLTHROUGH;
        case  2:
        if (likely((values[2] = __Pyx_PyDict_GetItemStr(__pyx_kwds, __pyx_n_s_pyx_state)) != 0)) kw_args--;
        else {
          __Pyx_RaiseArgtupleInvalid("__pyx_unpickle_ChannelCredentials", 1, 3, 3, 2); __PYX_ERR(7, 1, __pyx_L3_error)
        }
      }
      if (unlikely(kw_args > 0)) {
        if (unlikely(__Pyx_ParseOptionalKeywords(__pyx_kwds, __pyx_pyargnames, 0, values, pos_args, "__pyx_unpickle_ChannelCredentials") < 0)) __PYX_ERR(7, 1, __pyx_L3_error)
      }
    } else if (PyTuple_GET_SIZE(__pyx_args) != 3) {
      goto __pyx_L5_argtuple_error;
    } else {
      values[0] = PyTuple_GET_ITEM(__pyx_args, 0);
      values[1] = PyTuple_GET_ITEM(__pyx_args, 1);
      values[2] = PyTuple_GET_ITEM(__pyx_args, 2);
    }
    __pyx_v___pyx_type = values[0];
    __pyx_v___pyx_checksum = __Pyx_PyInt_As_long(values[1]); if (unlikely((__pyx_v___pyx_checksum == (long)-1) && PyErr_Occurred())) __PYX_ERR(7, 1, __pyx_L3_error)
    __pyx_v___pyx_state = values[2];
  }
  goto __pyx_L4_argument_unpacking_done;
  __pyx_L5_argtuple_error:;
  __Pyx_RaiseArgtupleInvalid("__pyx_unpickle_ChannelCredentials", 1, 3, 3, PyTuple_GET_SIZE(__pyx_args)); __PYX_ERR(7, 1, __pyx_L3_error)
  __pyx_L3_error:;
  __Pyx_AddTraceback("grpc._cython.cygrpc.__pyx_unpickle_ChannelCredentials", __pyx_clineno, __pyx_lineno, __pyx_filename);
  __Pyx_RefNannyFinishContext();
  return NULL;
  __pyx_L4_argument_unpacking_done:;
  __pyx_r = __pyx_pf_4grpc_7_cython_6cygrpc_78__pyx_unpickle_ChannelCredentials(__pyx_self, __pyx_v___pyx_type, __pyx_v___pyx_checksum, __pyx_v___pyx_state);

  /* function exit code */
  __Pyx_RefNannyFinishContext();
  return __pyx_r;
}

static PyObject *__pyx_pf_4grpc_7_cython_6cygrpc_78__pyx_unpickle_ChannelCredentials(CYTHON_UNUSED PyObject *__pyx_self, PyObject *__pyx_v___pyx_type, long __pyx_v___pyx_checksum, PyObject *__pyx_v___pyx_state) {
  PyObject *__pyx_v___pyx_PickleError = NULL;
  PyObject *__pyx_v___pyx_result = NULL;
  PyObject *__pyx_r = NULL;
  __Pyx_RefNannyDeclarations
  int __pyx_t_1;
  PyObject *__pyx_t_2 = NULL;
  PyObject *__pyx_t_3 = NULL;
  PyObject *__pyx_t_4 = NULL;
  PyObject *__pyx_t_5 = NULL;
  PyObject *__pyx_t_6 = NULL;
  int __pyx_t_7;
  __Pyx_RefNannySetupContext("__pyx_unpickle_ChannelCredentials", 0);

  /* "(tree fragment)":2
 * def __pyx_unpickle_ChannelCredentials(__pyx_type, long __pyx_checksum, __pyx_state):
 *     if __pyx_checksum != 0xd41d8cd:             # <<<<<<<<<<<<<<
 *         from pickle import PickleError as __pyx_PickleError
 *         raise __pyx_PickleError("Incompatible checksums (%s vs 0xd41d8cd = ())" % __pyx_checksum)
 */
  __pyx_t_1 = ((__pyx_v___pyx_checksum != 0xd41d8cd) != 0);
  if (__pyx_t_1) {

    /* "(tree fragment)":3
 * def __pyx_unpickle_ChannelCredentials(__pyx_type, long __pyx_checksum, __pyx_state):
 *     if __pyx_checksum != 0xd41d8cd:
 *         from pickle import PickleError as __pyx_PickleError             # <<<<<<<<<<<<<<
 *         raise __pyx_PickleError("Incompatible checksums (%s vs 0xd41d8cd = ())" % __pyx_checksum)
 *     __pyx_result = ChannelCredentials.__new__(__pyx_type)
 */
    __pyx_t_2 = PyList_New(1); if (unlikely(!__pyx_t_2)) __PYX_ERR(7, 3, __pyx_L1_error)
    __Pyx_GOTREF(__pyx_t_2);
    __Pyx_INCREF(__pyx_n_s_PickleError);
    __Pyx_GIVEREF(__pyx_n_s_PickleError);
    PyList_SET_ITEM(__pyx_t_2, 0, __pyx_n_s_PickleError);
    __pyx_t_3 = __Pyx_Import(__pyx_n_s_pickle, __pyx_t_2, -1); if (unlikely(!__pyx_t_3)) __PYX_ERR(7, 3, __pyx_L1_error)
    __Pyx_GOTREF(__pyx_t_3);
    __Pyx_DECREF(__pyx_t_2); __pyx_t_2 = 0;
    __pyx_t_2 = __Pyx_ImportFrom(__pyx_t_3, __pyx_n_s_PickleError); if (unlikely(!__pyx_t_2)) __PYX_ERR(7, 3, __pyx_L1_error)
    __Pyx_GOTREF(__pyx_t_2);
    __Pyx_INCREF(__pyx_t_2);
    __pyx_v___pyx_PickleError = __pyx_t_2;
    __Pyx_DECREF(__pyx_t_2); __pyx_t_2 = 0;
    __Pyx_DECREF(__pyx_t_3); __pyx_t_3 = 0;

    /* "(tree fragment)":4
 *     if __pyx_checksum != 0xd41d8cd:
 *         from pickle import PickleError as __pyx_PickleError
 *         raise __pyx_PickleError("Incompatible checksums (%s vs 0xd41d8cd = ())" % __pyx_checksum)             # <<<<<<<<<<<<<<
 *     __pyx_result = ChannelCredentials.__new__(__pyx_type)
 *     if __pyx_state is not None:
 */
    __pyx_t_2 = __Pyx_PyInt_From_long(__pyx_v___pyx_checksum); if (unlikely(!__pyx_t_2)) __PYX_ERR(7, 4, __pyx_L1_error)
    __Pyx_GOTREF(__pyx_t_2);
    __pyx_t_4 = __Pyx_PyString_Format(__pyx_kp_s_Incompatible_checksums_s_vs_0xd4, __pyx_t_2); if (unlikely(!__pyx_t_4)) __PYX_ERR(7, 4, __pyx_L1_error)
    __Pyx_GOTREF(__pyx_t_4);
    __Pyx_DECREF(__pyx_t_2); __pyx_t_2 = 0;
    __Pyx_INCREF(__pyx_v___pyx_PickleError);
    __pyx_t_2 = __pyx_v___pyx_PickleError; __pyx_t_5 = NULL;
    if (CYTHON_UNPACK_METHODS && unlikely(PyMethod_Check(__pyx_t_2))) {
      __pyx_t_5 = PyMethod_GET_SELF(__pyx_t_2);
      if (likely(__pyx_t_5)) {
        PyObject* function = PyMethod_GET_FUNCTION(__pyx_t_2);
        __Pyx_INCREF(__pyx_t_5);
        __Pyx_INCREF(function);
        __Pyx_DECREF_SET(__pyx_t_2, function);
      }
    }
    if (!__pyx_t_5) {
      __pyx_t_3 = __Pyx_PyObject_CallOneArg(__pyx_t_2, __pyx_t_4); if (unlikely(!__pyx_t_3)) __PYX_ERR(7, 4, __pyx_L1_error)
      __Pyx_DECREF(__pyx_t_4); __pyx_t_4 = 0;
      __Pyx_GOTREF(__pyx_t_3);
    } else {
      #if CYTHON_FAST_PYCALL
      if (PyFunction_Check(__pyx_t_2)) {
        PyObject *__pyx_temp[2] = {__pyx_t_5, __pyx_t_4};
        __pyx_t_3 = __Pyx_PyFunction_FastCall(__pyx_t_2, __pyx_temp+1-1, 1+1); if (unlikely(!__pyx_t_3)) __PYX_ERR(7, 4, __pyx_L1_error)
        __Pyx_XDECREF(__pyx_t_5); __pyx_t_5 = 0;
        __Pyx_GOTREF(__pyx_t_3);
        __Pyx_DECREF(__pyx_t_4); __pyx_t_4 = 0;
      } else
      #endif
      #if CYTHON_FAST_PYCCALL
      if (__Pyx_PyFastCFunction_Check(__pyx_t_2)) {
        PyObject *__pyx_temp[2] = {__pyx_t_5, __pyx_t_4};
        __pyx_t_3 = __Pyx_PyCFunction_FastCall(__pyx_t_2, __pyx_temp+1-1, 1+1); if (unlikely(!__pyx_t_3)) __PYX_ERR(7, 4, __pyx_L1_error)
        __Pyx_XDECREF(__pyx_t_5); __pyx_t_5 = 0;
        __Pyx_GOTREF(__pyx_t_3);
        __Pyx_DECREF(__pyx_t_4); __pyx_t_4 = 0;
      } else
      #endif
      {
        __pyx_t_6 = PyTuple_New(1+1); if (unlikely(!__pyx_t_6)) __PYX_ERR(7, 4, __pyx_L1_error)
        __Pyx_GOTREF(__pyx_t_6);
        __Pyx_GIVEREF(__pyx_t_5); PyTuple_SET_ITEM(__pyx_t_6, 0, __pyx_t_5); __pyx_t_5 = NULL;
        __Pyx_GIVEREF(__pyx_t_4);
        PyTuple_SET_ITEM(__pyx_t_6, 0+1, __pyx_t_4);
        __pyx_t_4 = 0;
        __pyx_t_3 = __Pyx_PyObject_Call(__pyx_t_2, __pyx_t_6, NULL); if (unlikely(!__pyx_t_3)) __PYX_ERR(7, 4, __pyx_L1_error)
        __Pyx_GOTREF(__pyx_t_3);
        __Pyx_DECREF(__pyx_t_6); __pyx_t_6 = 0;
      }
    }
    __Pyx_DECREF(__pyx_t_2); __pyx_t_2 = 0;
    __Pyx_Raise(__pyx_t_3, 0, 0, 0);
    __Pyx_DECREF(__pyx_t_3); __pyx_t_3 = 0;
    __PYX_ERR(7, 4, __pyx_L1_error)

    /* "(tree fragment)":2
 * def __pyx_unpickle_ChannelCredentials(__pyx_type, long __pyx_checksum, __pyx_state):
 *     if __pyx_checksum != 0xd41d8cd:             # <<<<<<<<<<<<<<
 *         from pickle import PickleError as __pyx_PickleError
 *         raise __pyx_PickleError("Incompatible checksums (%s vs 0xd41d8cd = ())" % __pyx_checksum)
 */
  }

  /* "(tree fragment)":5
 *         from pickle import PickleError as __pyx_PickleError
 *         raise __pyx_PickleError("Incompatible checksums (%s vs 0xd41d8cd = ())" % __pyx_checksum)
 *     __pyx_result = ChannelCredentials.__new__(__pyx_type)             # <<<<<<<<<<<<<<
 *     if __pyx_state is not None:
 *         __pyx_unpickle_ChannelCredentials__set_state(<ChannelCredentials> __pyx_result, __pyx_state)
 */
  __pyx_t_2 = __Pyx_PyObject_GetAttrStr(((PyObject *)__pyx_ptype_4grpc_7_cython_6cygrpc_ChannelCredentials), __pyx_n_s_new); if (unlikely(!__pyx_t_2)) __PYX_ERR(7, 5, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_2);
  __pyx_t_6 = NULL;
  if (CYTHON_UNPACK_METHODS && likely(PyMethod_Check(__pyx_t_2))) {
    __pyx_t_6 = PyMethod_GET_SELF(__pyx_t_2);
    if (likely(__pyx_t_6)) {
      PyObject* function = PyMethod_GET_FUNCTION(__pyx_t_2);
      __Pyx_INCREF(__pyx_t_6);
      __Pyx_INCREF(function);
      __Pyx_DECREF_SET(__pyx_t_2, function);
    }
  }
  if (!__pyx_t_6) {
    __pyx_t_3 = __Pyx_PyObject_CallOneArg(__pyx_t_2, __pyx_v___pyx_type); if (unlikely(!__pyx_t_3)) __PYX_ERR(7, 5, __pyx_L1_error)
    __Pyx_GOTREF(__pyx_t_3);
  } else {
    #if CYTHON_FAST_PYCALL
    if (PyFunction_Check(__pyx_t_2)) {
      PyObject *__pyx_temp[2] = {__pyx_t_6, __pyx_v___pyx_type};
      __pyx_t_3 = __Pyx_PyFunction_FastCall(__pyx_t_2, __pyx_temp+1-1, 1+1); if (unlikely(!__pyx_t_3)) __PYX_ERR(7, 5, __pyx_L1_error)
      __Pyx_XDECREF(__pyx_t_6); __pyx_t_6 = 0;
      __Pyx_GOTREF(__pyx_t_3);
    } else
    #endif
    #if CYTHON_FAST_PYCCALL
    if (__Pyx_PyFastCFunction_Check(__pyx_t_2)) {
      PyObject *__pyx_temp[2] = {__pyx_t_6, __pyx_v___pyx_type};
      __pyx_t_3 = __Pyx_PyCFunction_FastCall(__pyx_t_2, __pyx_temp+1-1, 1+1); if (unlikely(!__pyx_t_3)) __PYX_ERR(7, 5, __pyx_L1_error)
      __Pyx_XDECREF(__pyx_t_6); __pyx_t_6 = 0;
      __Pyx_GOTREF(__pyx_t_3);
    } else
    #endif
    {
      __pyx_t_4 = PyTuple_New(1+1); if (unlikely(!__pyx_t_4)) __PYX_ERR(7, 5, __pyx_L1_error)
      __Pyx_GOTREF(__pyx_t_4);
      __Pyx_GIVEREF(__pyx_t_6); PyTuple_SET_ITEM(__pyx_t_4, 0, __pyx_t_6); __pyx_t_6 = NULL;
      __Pyx_INCREF(__pyx_v___pyx_type);
      __Pyx_GIVEREF(__pyx_v___pyx_type);
      PyTuple_SET_ITEM(__pyx_t_4, 0+1, __pyx_v___pyx_type);
      __pyx_t_3 = __Pyx_PyObject_Call(__pyx_t_2, __pyx_t_4, NULL); if (unlikely(!__pyx_t_3)) __PYX_ERR(7, 5, __pyx_L1_error)
      __Pyx_GOTREF(__pyx_t_3);
      __Pyx_DECREF(__pyx_t_4); __pyx_t_4 = 0;
    }
  }
  __Pyx_DECREF(__pyx_t_2); __pyx_t_2 = 0;
  __pyx_v___pyx_result = __pyx_t_3;
  __pyx_t_3 = 0;

  /* "(tree fragment)":6
 *         raise __pyx_PickleError("Incompatible checksums (%s vs 0xd41d8cd = ())" % __pyx_checksum)
 *     __pyx_result = ChannelCredentials.__new__(__pyx_type)
 *     if __pyx_state is not None:             # <<<<<<<<<<<<<<
 *         __pyx_unpickle_ChannelCredentials__set_state(<ChannelCredentials> __pyx_result, __pyx_state)
 *     return __pyx_result
 */
  __pyx_t_1 = (__pyx_v___pyx_state != Py_None);
  __pyx_t_7 = (__pyx_t_1 != 0);
  if (__pyx_t_7) {

    /* "(tree fragment)":7
 *     __pyx_result = ChannelCredentials.__new__(__pyx_type)
 *     if __pyx_state is not None:
 *         __pyx_unpickle_ChannelCredentials__set_state(<ChannelCredentials> __pyx_result, __pyx_state)             # <<<<<<<<<<<<<<
 *     return __pyx_result
 * cdef __pyx_unpickle_ChannelCredentials__set_state(ChannelCredentials __pyx_result, tuple __pyx_state):
 */
    if (!(likely(PyTuple_CheckExact(__pyx_v___pyx_state))||((__pyx_v___pyx_state) == Py_None)||(PyErr_Format(PyExc_TypeError, "Expected %.16s, got %.200s", "tuple", Py_TYPE(__pyx_v___pyx_state)->tp_name), 0))) __PYX_ERR(7, 7, __pyx_L1_error)
    __pyx_t_3 = __pyx_f_4grpc_7_cython_6cygrpc___pyx_unpickle_ChannelCredentials__set_state(((struct __pyx_obj_4grpc_7_cython_6cygrpc_ChannelCredentials *)__pyx_v___pyx_result), ((PyObject*)__pyx_v___pyx_state)); if (unlikely(!__pyx_t_3)) __PYX_ERR(7, 7, __pyx_L1_error)
    __Pyx_GOTREF(__pyx_t_3);
    __Pyx_DECREF(__pyx_t_3); __pyx_t_3 = 0;

    /* "(tree fragment)":6
 *         raise __pyx_PickleError("Incompatible checksums (%s vs 0xd41d8cd = ())" % __pyx_checksum)
 *     __pyx_result = ChannelCredentials.__new__(__pyx_type)
 *     if __pyx_state is not None:             # <<<<<<<<<<<<<<
 *         __pyx_unpickle_ChannelCredentials__set_state(<ChannelCredentials> __pyx_result, __pyx_state)
 *     return __pyx_result
 */
  }

  /* "(tree fragment)":8
 *     if __pyx_state is not None:
 *         __pyx_unpickle_ChannelCredentials__set_state(<ChannelCredentials> __pyx_result, __pyx_state)
 *     return __pyx_result             # <<<<<<<<<<<<<<
 * cdef __pyx_unpickle_ChannelCredentials__set_state(ChannelCredentials __pyx_result, tuple __pyx_state):
 *     if len(__pyx_state) > 0 and hasattr(__pyx_result, '__dict__'):
 */
  __Pyx_XDECREF(__pyx_r);
  __Pyx_INCREF(__pyx_v___pyx_result);
  __pyx_r = __pyx_v___pyx_result;
  goto __pyx_L0;

  /* "(tree fragment)":1
 * def __pyx_unpickle_ChannelCredentials(__pyx_type, long __pyx_checksum, __pyx_state):             # <<<<<<<<<<<<<<
 *     if __pyx_checksum != 0xd41d8cd:
 *         from pickle import PickleError as __pyx_PickleError
 */

  /* function exit code */
  __pyx_L1_error:;
  __Pyx_XDECREF(__pyx_t_2);
  __Pyx_XDECREF(__pyx_t_3);
  __Pyx_XDECREF(__pyx_t_4);
  __Pyx_XDECREF(__pyx_t_5);
  __Pyx_XDECREF(__pyx_t_6);
  __Pyx_AddTraceback("grpc._cython.cygrpc.__pyx_unpickle_ChannelCredentials", __pyx_clineno, __pyx_lineno, __pyx_filename);
  __pyx_r = NULL;
  __pyx_L0:;
  __Pyx_XDECREF(__pyx_v___pyx_PickleError);
  __Pyx_XDECREF(__pyx_v___pyx_result);
  __Pyx_XGIVEREF(__pyx_r);
  __Pyx_RefNannyFinishContext();
  return __pyx_r;
}

/* "(tree fragment)":9
 *         __pyx_unpickle_ChannelCredentials__set_state(<ChannelCredentials> __pyx_result, __pyx_state)
 *     return __pyx_result
 * cdef __pyx_unpickle_ChannelCredentials__set_state(ChannelCredentials __pyx_result, tuple __pyx_state):             # <<<<<<<<<<<<<<
 *     if len(__pyx_state) > 0 and hasattr(__pyx_result, '__dict__'):
 *         __pyx_result.__dict__.update(__pyx_state[0])
 */

static PyObject *__pyx_f_4grpc_7_cython_6cygrpc___pyx_unpickle_ChannelCredentials__set_state(struct __pyx_obj_4grpc_7_cython_6cygrpc_ChannelCredentials *__pyx_v___pyx_result, PyObject *__pyx_v___pyx_state) {
  PyObject *__pyx_r = NULL;
  __Pyx_RefNannyDeclarations
  int __pyx_t_1;
  Py_ssize_t __pyx_t_2;
  int __pyx_t_3;
  int __pyx_t_4;
  PyObject *__pyx_t_5 = NULL;
  PyObject *__pyx_t_6 = NULL;
  PyObject *__pyx_t_7 = NULL;
  PyObject *__pyx_t_8 = NULL;
  PyObject *__pyx_t_9 = NULL;
  __Pyx_RefNannySetupContext("__pyx_unpickle_ChannelCredentials__set_state", 0);

  /* "(tree fragment)":10
 *     return __pyx_result
 * cdef __pyx_unpickle_ChannelCredentials__set_state(ChannelCredentials __pyx_result, tuple __pyx_state):
 *     if len(__pyx_state) > 0 and hasattr(__pyx_result, '__dict__'):             # <<<<<<<<<<<<<<
 *         __pyx_result.__dict__.update(__pyx_state[0])
 */
  if (unlikely(__pyx_v___pyx_state == Py_None)) {
    PyErr_SetString(PyExc_TypeError, "object of type 'NoneType' has no len()");
    __PYX_ERR(7, 10, __pyx_L1_error)
  }
  __pyx_t_2 = PyTuple_GET_SIZE(__pyx_v___pyx_state); if (unlikely(__pyx_t_2 == ((Py_ssize_t)-1))) __PYX_ERR(7, 10, __pyx_L1_error)
  __pyx_t_3 = ((__pyx_t_2 > 0) != 0);
  if (__pyx_t_3) {
  } else {
    __pyx_t_1 = __pyx_t_3;
    goto __pyx_L4_bool_binop_done;
  }
  __pyx_t_3 = __Pyx_HasAttr(((PyObject *)__pyx_v___pyx_result), __pyx_n_s_dict); if (unlikely(__pyx_t_3 == ((int)-1))) __PYX_ERR(7, 10, __pyx_L1_error)
  __pyx_t_4 = (__pyx_t_3 != 0);
  __pyx_t_1 = __pyx_t_4;
  __pyx_L4_bool_binop_done:;
  if (__pyx_t_1) {

    /* "(tree fragment)":11
 * cdef __pyx_unpickle_ChannelCredentials__set_state(ChannelCredentials __pyx_result, tuple __pyx_state):
 *     if len(__pyx_state) > 0 and hasattr(__pyx_result, '__dict__'):
 *         __pyx_result.__dict__.update(__pyx_state[0])             # <<<<<<<<<<<<<<
 */
    __pyx_t_6 = __Pyx_PyObject_GetAttrStr(((PyObject *)__pyx_v___pyx_result), __pyx_n_s_dict); if (unlikely(!__pyx_t_6)) __PYX_ERR(7, 11, __pyx_L1_error)
    __Pyx_GOTREF(__pyx_t_6);
    __pyx_t_7 = __Pyx_PyObject_GetAttrStr(__pyx_t_6, __pyx_n_s_update); if (unlikely(!__pyx_t_7)) __PYX_ERR(7, 11, __pyx_L1_error)
    __Pyx_GOTREF(__pyx_t_7);
    __Pyx_DECREF(__pyx_t_6); __pyx_t_6 = 0;
    if (unlikely(__pyx_v___pyx_state == Py_None)) {
      PyErr_SetString(PyExc_TypeError, "'NoneType' object is not subscriptable");
      __PYX_ERR(7, 11, __pyx_L1_error)
    }
    __pyx_t_6 = __Pyx_GetItemInt_Tuple(__pyx_v___pyx_state, 0, long, 1, __Pyx_PyInt_From_long, 0, 0, 1); if (unlikely(!__pyx_t_6)) __PYX_ERR(7, 11, __pyx_L1_error)
    __Pyx_GOTREF(__pyx_t_6);
    __pyx_t_8 = NULL;
    if (CYTHON_UNPACK_METHODS && likely(PyMethod_Check(__pyx_t_7))) {
      __pyx_t_8 = PyMethod_GET_SELF(__pyx_t_7);
      if (likely(__pyx_t_8)) {
        PyObject* function = PyMethod_GET_FUNCTION(__pyx_t_7);
        __Pyx_INCREF(__pyx_t_8);
        __Pyx_INCREF(function);
        __Pyx_DECREF_SET(__pyx_t_7, function);
      }
    }
    if (!__pyx_t_8) {
      __pyx_t_5 = __Pyx_PyObject_CallOneArg(__pyx_t_7, __pyx_t_6); if (unlikely(!__pyx_t_5)) __PYX_ERR(7, 11, __pyx_L1_error)
      __Pyx_DECREF(__pyx_t_6); __pyx_t_6 = 0;
      __Pyx_GOTREF(__pyx_t_5);
    } else {
      #if CYTHON_FAST_PYCALL
      if (PyFunction_Check(__pyx_t_7)) {
        PyObject *__pyx_temp[2] = {__pyx_t_8, __pyx_t_6};
        __pyx_t_5 = __Pyx_PyFunction_FastCall(__pyx_t_7, __pyx_temp+1-1, 1+1); if (unlikely(!__pyx_t_5)) __PYX_ERR(7, 11, __pyx_L1_error)
        __Pyx_XDECREF(__pyx_t_8); __pyx_t_8 = 0;
        __Pyx_GOTREF(__pyx_t_5);
        __Pyx_DECREF(__pyx_t_6); __pyx_t_6 = 0;
      } else
      #endif
      #if CYTHON_FAST_PYCCALL
      if (__Pyx_PyFastCFunction_Check(__pyx_t_7)) {
        PyObject *__pyx_temp[2] = {__pyx_t_8, __pyx_t_6};
        __pyx_t_5 = __Pyx_PyCFunction_FastCall(__pyx_t_7, __pyx_temp+1-1, 1+1); if (unlikely(!__pyx_t_5)) __PYX_ERR(7, 11, __pyx_L1_error)
        __Pyx_XDECREF(__pyx_t_8); __pyx_t_8 = 0;
        __Pyx_GOTREF(__pyx_t_5);
        __Pyx_DECREF(__pyx_t_6); __pyx_t_6 = 0;
      } else
      #endif
      {
        __pyx_t_9 = PyTuple_New(1+1); if (unlikely(!__pyx_t_9)) __PYX_ERR(7, 11, __pyx_L1_error)
        __Pyx_GOTREF(__pyx_t_9);
        __Pyx_GIVEREF(__pyx_t_8); PyTuple_SET_ITEM(__pyx_t_9, 0, __pyx_t_8); __pyx_t_8 = NULL;
        __Pyx_GIVEREF(__pyx_t_6);
        PyTuple_SET_ITEM(__pyx_t_9, 0+1, __pyx_t_6);
        __pyx_t_6 = 0;
        __pyx_t_5 = __Pyx_PyObject_Call(__pyx_t_7, __pyx_t_9, NULL); if (unlikely(!__pyx_t_5)) __PYX_ERR(7, 11, __pyx_L1_error)
        __Pyx_GOTREF(__pyx_t_5);
        __Pyx_DECREF(__pyx_t_9); __pyx_t_9 = 0;
      }
    }
    __Pyx_DECREF(__pyx_t_7); __pyx_t_7 = 0;
    __Pyx_DECREF(__pyx_t_5); __pyx_t_5 = 0;

    /* "(tree fragment)":10
 *     return __pyx_result
 * cdef __pyx_unpickle_ChannelCredentials__set_state(ChannelCredentials __pyx_result, tuple __pyx_state):
 *     if len(__pyx_state) > 0 and hasattr(__pyx_result, '__dict__'):             # <<<<<<<<<<<<<<
 *         __pyx_result.__dict__.update(__pyx_state[0])
 */
  }

  /* "(tree fragment)":9
 *         __pyx_unpickle_ChannelCredentials__set_state(<ChannelCredentials> __pyx_result, __pyx_state)
 *     return __pyx_result
 * cdef __pyx_unpickle_ChannelCredentials__set_state(ChannelCredentials __pyx_result, tuple __pyx_state):             # <<<<<<<<<<<<<<
 *     if len(__pyx_state) > 0 and hasattr(__pyx_result, '__dict__'):
 *         __pyx_result.__dict__.update(__pyx_state[0])
 */

  /* function exit code */
  __pyx_r = Py_None; __Pyx_INCREF(Py_None);
  goto __pyx_L0;
  __pyx_L1_error:;
  __Pyx_XDECREF(__pyx_t_5);
  __Pyx_XDECREF(__pyx_t_6);
  __Pyx_XDECREF(__pyx_t_7);
  __Pyx_XDECREF(__pyx_t_8);
  __Pyx_XDECREF(__pyx_t_9);
  __Pyx_AddTraceback("grpc._cython.cygrpc.__pyx_unpickle_ChannelCredentials__set_state", __pyx_clineno, __pyx_lineno, __pyx_filename);
  __pyx_r = 0;
  __pyx_L0:;
  __Pyx_XGIVEREF(__pyx_r);
  __Pyx_RefNannyFinishContext();
  return __pyx_r;
}

/* "(tree fragment)":1
 * def __pyx_unpickle__Tag(__pyx_type, long __pyx_checksum, __pyx_state):             # <<<<<<<<<<<<<<
 *     if __pyx_checksum != 0xd41d8cd:
 *         from pickle import PickleError as __pyx_PickleError
 */

/* Python wrapper */
static PyObject *__pyx_pw_4grpc_7_cython_6cygrpc_81__pyx_unpickle__Tag(PyObject *__pyx_self, PyObject *__pyx_args, PyObject *__pyx_kwds); /*proto*/
static PyMethodDef __pyx_mdef_4grpc_7_cython_6cygrpc_81__pyx_unpickle__Tag = {"__pyx_unpickle__Tag", (PyCFunction)__pyx_pw_4grpc_7_cython_6cygrpc_81__pyx_unpickle__Tag, METH_VARARGS|METH_KEYWORDS, 0};
static PyObject *__pyx_pw_4grpc_7_cython_6cygrpc_81__pyx_unpickle__Tag(PyObject *__pyx_self, PyObject *__pyx_args, PyObject *__pyx_kwds) {
  PyObject *__pyx_v___pyx_type = 0;
  long __pyx_v___pyx_checksum;
  PyObject *__pyx_v___pyx_state = 0;
  PyObject *__pyx_r = 0;
  __Pyx_RefNannyDeclarations
  __Pyx_RefNannySetupContext("__pyx_unpickle__Tag (wrapper)", 0);
  {
    static PyObject **__pyx_pyargnames[] = {&__pyx_n_s_pyx_type,&__pyx_n_s_pyx_checksum,&__pyx_n_s_pyx_state,0};
    PyObject* values[3] = {0,0,0};
    if (unlikely(__pyx_kwds)) {
      Py_ssize_t kw_args;
      const Py_ssize_t pos_args = PyTuple_GET_SIZE(__pyx_args);
      switch (pos_args) {
        case  3: values[2] = PyTuple_GET_ITEM(__pyx_args, 2);
        CYTHON_FALLTHROUGH;
        case  2: values[1] = PyTuple_GET_ITEM(__pyx_args, 1);
        CYTHON_FALLTHROUGH;
        case  1: values[0] = PyTuple_GET_ITEM(__pyx_args, 0);
        CYTHON_FALLTHROUGH;
        case  0: break;
        default: goto __pyx_L5_argtuple_error;
      }
      kw_args = PyDict_Size(__pyx_kwds);
      switch (pos_args) {
        case  0:
        if (likely((values[0] = __Pyx_PyDict_GetItemStr(__pyx_kwds, __pyx_n_s_pyx_type)) != 0)) kw_args--;
        else goto __pyx_L5_argtuple_error;
        CYTHON_FALLTHROUGH;
        case  1:
        if (likely((values[1] = __Pyx_PyDict_GetItemStr(__pyx_kwds, __pyx_n_s_pyx_checksum)) != 0)) kw_args--;
        else {
          __Pyx_RaiseArgtupleInvalid("__pyx_unpickle__Tag", 1, 3, 3, 1); __PYX_ERR(7, 1, __pyx_L3_error)
        }
        CYTHON_FALLTHROUGH;
        case  2:
        if (likely((values[2] = __Pyx_PyDict_GetItemStr(__pyx_kwds, __pyx_n_s_pyx_state)) != 0)) kw_args--;
        else {
          __Pyx_RaiseArgtupleInvalid("__pyx_unpickle__Tag", 1, 3, 3, 2); __PYX_ERR(7, 1, __pyx_L3_error)
        }
      }
      if (unlikely(kw_args > 0)) {
        if (unlikely(__Pyx_ParseOptionalKeywords(__pyx_kwds, __pyx_pyargnames, 0, values, pos_args, "__pyx_unpickle__Tag") < 0)) __PYX_ERR(7, 1, __pyx_L3_error)
      }
    } else if (PyTuple_GET_SIZE(__pyx_args) != 3) {
      goto __pyx_L5_argtuple_error;
    } else {
      values[0] = PyTuple_GET_ITEM(__pyx_args, 0);
      values[1] = PyTuple_GET_ITEM(__pyx_args, 1);
      values[2] = PyTuple_GET_ITEM(__pyx_args, 2);
    }
    __pyx_v___pyx_type = values[0];
    __pyx_v___pyx_checksum = __Pyx_PyInt_As_long(values[1]); if (unlikely((__pyx_v___pyx_checksum == (long)-1) && PyErr_Occurred())) __PYX_ERR(7, 1, __pyx_L3_error)
    __pyx_v___pyx_state = values[2];
  }
  goto __pyx_L4_argument_unpacking_done;
  __pyx_L5_argtuple_error:;
  __Pyx_RaiseArgtupleInvalid("__pyx_unpickle__Tag", 1, 3, 3, PyTuple_GET_SIZE(__pyx_args)); __PYX_ERR(7, 1, __pyx_L3_error)
  __pyx_L3_error:;
  __Pyx_AddTraceback("grpc._cython.cygrpc.__pyx_unpickle__Tag", __pyx_clineno, __pyx_lineno, __pyx_filename);
  __Pyx_RefNannyFinishContext();
  return NULL;
  __pyx_L4_argument_unpacking_done:;
  __pyx_r = __pyx_pf_4grpc_7_cython_6cygrpc_80__pyx_unpickle__Tag(__pyx_self, __pyx_v___pyx_type, __pyx_v___pyx_checksum, __pyx_v___pyx_state);

  /* function exit code */
  __Pyx_RefNannyFinishContext();
  return __pyx_r;
}

static PyObject *__pyx_pf_4grpc_7_cython_6cygrpc_80__pyx_unpickle__Tag(CYTHON_UNUSED PyObject *__pyx_self, PyObject *__pyx_v___pyx_type, long __pyx_v___pyx_checksum, PyObject *__pyx_v___pyx_state) {
  PyObject *__pyx_v___pyx_PickleError = NULL;
  PyObject *__pyx_v___pyx_result = NULL;
  PyObject *__pyx_r = NULL;
  __Pyx_RefNannyDeclarations
  int __pyx_t_1;
  PyObject *__pyx_t_2 = NULL;
  PyObject *__pyx_t_3 = NULL;
  PyObject *__pyx_t_4 = NULL;
  PyObject *__pyx_t_5 = NULL;
  PyObject *__pyx_t_6 = NULL;
  int __pyx_t_7;
  __Pyx_RefNannySetupContext("__pyx_unpickle__Tag", 0);

  /* "(tree fragment)":2
 * def __pyx_unpickle__Tag(__pyx_type, long __pyx_checksum, __pyx_state):
 *     if __pyx_checksum != 0xd41d8cd:             # <<<<<<<<<<<<<<
 *         from pickle import PickleError as __pyx_PickleError
 *         raise __pyx_PickleError("Incompatible checksums (%s vs 0xd41d8cd = ())" % __pyx_checksum)
 */
  __pyx_t_1 = ((__pyx_v___pyx_checksum != 0xd41d8cd) != 0);
  if (__pyx_t_1) {

    /* "(tree fragment)":3
 * def __pyx_unpickle__Tag(__pyx_type, long __pyx_checksum, __pyx_state):
 *     if __pyx_checksum != 0xd41d8cd:
 *         from pickle import PickleError as __pyx_PickleError             # <<<<<<<<<<<<<<
 *         raise __pyx_PickleError("Incompatible checksums (%s vs 0xd41d8cd = ())" % __pyx_checksum)
 *     __pyx_result = _Tag.__new__(__pyx_type)
 */
    __pyx_t_2 = PyList_New(1); if (unlikely(!__pyx_t_2)) __PYX_ERR(7, 3, __pyx_L1_error)
    __Pyx_GOTREF(__pyx_t_2);
    __Pyx_INCREF(__pyx_n_s_PickleError);
    __Pyx_GIVEREF(__pyx_n_s_PickleError);
    PyList_SET_ITEM(__pyx_t_2, 0, __pyx_n_s_PickleError);
    __pyx_t_3 = __Pyx_Import(__pyx_n_s_pickle, __pyx_t_2, -1); if (unlikely(!__pyx_t_3)) __PYX_ERR(7, 3, __pyx_L1_error)
    __Pyx_GOTREF(__pyx_t_3);
    __Pyx_DECREF(__pyx_t_2); __pyx_t_2 = 0;
    __pyx_t_2 = __Pyx_ImportFrom(__pyx_t_3, __pyx_n_s_PickleError); if (unlikely(!__pyx_t_2)) __PYX_ERR(7, 3, __pyx_L1_error)
    __Pyx_GOTREF(__pyx_t_2);
    __Pyx_INCREF(__pyx_t_2);
    __pyx_v___pyx_PickleError = __pyx_t_2;
    __Pyx_DECREF(__pyx_t_2); __pyx_t_2 = 0;
    __Pyx_DECREF(__pyx_t_3); __pyx_t_3 = 0;

    /* "(tree fragment)":4
 *     if __pyx_checksum != 0xd41d8cd:
 *         from pickle import PickleError as __pyx_PickleError
 *         raise __pyx_PickleError("Incompatible checksums (%s vs 0xd41d8cd = ())" % __pyx_checksum)             # <<<<<<<<<<<<<<
 *     __pyx_result = _Tag.__new__(__pyx_type)
 *     if __pyx_state is not None:
 */
    __pyx_t_2 = __Pyx_PyInt_From_long(__pyx_v___pyx_checksum); if (unlikely(!__pyx_t_2)) __PYX_ERR(7, 4, __pyx_L1_error)
    __Pyx_GOTREF(__pyx_t_2);
    __pyx_t_4 = __Pyx_PyString_Format(__pyx_kp_s_Incompatible_checksums_s_vs_0xd4, __pyx_t_2); if (unlikely(!__pyx_t_4)) __PYX_ERR(7, 4, __pyx_L1_error)
    __Pyx_GOTREF(__pyx_t_4);
    __Pyx_DECREF(__pyx_t_2); __pyx_t_2 = 0;
    __Pyx_INCREF(__pyx_v___pyx_PickleError);
    __pyx_t_2 = __pyx_v___pyx_PickleError; __pyx_t_5 = NULL;
    if (CYTHON_UNPACK_METHODS && unlikely(PyMethod_Check(__pyx_t_2))) {
      __pyx_t_5 = PyMethod_GET_SELF(__pyx_t_2);
      if (likely(__pyx_t_5)) {
        PyObject* function = PyMethod_GET_FUNCTION(__pyx_t_2);
        __Pyx_INCREF(__pyx_t_5);
        __Pyx_INCREF(function);
        __Pyx_DECREF_SET(__pyx_t_2, function);
      }
    }
    if (!__pyx_t_5) {
      __pyx_t_3 = __Pyx_PyObject_CallOneArg(__pyx_t_2, __pyx_t_4); if (unlikely(!__pyx_t_3)) __PYX_ERR(7, 4, __pyx_L1_error)
      __Pyx_DECREF(__pyx_t_4); __pyx_t_4 = 0;
      __Pyx_GOTREF(__pyx_t_3);
    } else {
      #if CYTHON_FAST_PYCALL
      if (PyFunction_Check(__pyx_t_2)) {
        PyObject *__pyx_temp[2] = {__pyx_t_5, __pyx_t_4};
        __pyx_t_3 = __Pyx_PyFunction_FastCall(__pyx_t_2, __pyx_temp+1-1, 1+1); if (unlikely(!__pyx_t_3)) __PYX_ERR(7, 4, __pyx_L1_error)
        __Pyx_XDECREF(__pyx_t_5); __pyx_t_5 = 0;
        __Pyx_GOTREF(__pyx_t_3);
        __Pyx_DECREF(__pyx_t_4); __pyx_t_4 = 0;
      } else
      #endif
      #if CYTHON_FAST_PYCCALL
      if (__Pyx_PyFastCFunction_Check(__pyx_t_2)) {
        PyObject *__pyx_temp[2] = {__pyx_t_5, __pyx_t_4};
        __pyx_t_3 = __Pyx_PyCFunction_FastCall(__pyx_t_2, __pyx_temp+1-1, 1+1); if (unlikely(!__pyx_t_3)) __PYX_ERR(7, 4, __pyx_L1_error)
        __Pyx_XDECREF(__pyx_t_5); __pyx_t_5 = 0;
        __Pyx_GOTREF(__pyx_t_3);
        __Pyx_DECREF(__pyx_t_4); __pyx_t_4 = 0;
      } else
      #endif
      {
        __pyx_t_6 = PyTuple_New(1+1); if (unlikely(!__pyx_t_6)) __PYX_ERR(7, 4, __pyx_L1_error)
        __Pyx_GOTREF(__pyx_t_6);
        __Pyx_GIVEREF(__pyx_t_5); PyTuple_SET_ITEM(__pyx_t_6, 0, __pyx_t_5); __pyx_t_5 = NULL;
        __Pyx_GIVEREF(__pyx_t_4);
        PyTuple_SET_ITEM(__pyx_t_6, 0+1, __pyx_t_4);
        __pyx_t_4 = 0;
        __pyx_t_3 = __Pyx_PyObject_Call(__pyx_t_2, __pyx_t_6, NULL); if (unlikely(!__pyx_t_3)) __PYX_ERR(7, 4, __pyx_L1_error)
        __Pyx_GOTREF(__pyx_t_3);
        __Pyx_DECREF(__pyx_t_6); __pyx_t_6 = 0;
      }
    }
    __Pyx_DECREF(__pyx_t_2); __pyx_t_2 = 0;
    __Pyx_Raise(__pyx_t_3, 0, 0, 0);
    __Pyx_DECREF(__pyx_t_3); __pyx_t_3 = 0;
    __PYX_ERR(7, 4, __pyx_L1_error)

    /* "(tree fragment)":2
 * def __pyx_unpickle__Tag(__pyx_type, long __pyx_checksum, __pyx_state):
 *     if __pyx_checksum != 0xd41d8cd:             # <<<<<<<<<<<<<<
 *         from pickle import PickleError as __pyx_PickleError
 *         raise __pyx_PickleError("Incompatible checksums (%s vs 0xd41d8cd = ())" % __pyx_checksum)
 */
  }

  /* "(tree fragment)":5
 *         from pickle import PickleError as __pyx_PickleError
 *         raise __pyx_PickleError("Incompatible checksums (%s vs 0xd41d8cd = ())" % __pyx_checksum)
 *     __pyx_result = _Tag.__new__(__pyx_type)             # <<<<<<<<<<<<<<
 *     if __pyx_state is not None:
 *         __pyx_unpickle__Tag__set_state(<_Tag> __pyx_result, __pyx_state)
 */
  __pyx_t_2 = __Pyx_PyObject_GetAttrStr(((PyObject *)__pyx_ptype_4grpc_7_cython_6cygrpc__Tag), __pyx_n_s_new); if (unlikely(!__pyx_t_2)) __PYX_ERR(7, 5, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_2);
  __pyx_t_6 = NULL;
  if (CYTHON_UNPACK_METHODS && likely(PyMethod_Check(__pyx_t_2))) {
    __pyx_t_6 = PyMethod_GET_SELF(__pyx_t_2);
    if (likely(__pyx_t_6)) {
      PyObject* function = PyMethod_GET_FUNCTION(__pyx_t_2);
      __Pyx_INCREF(__pyx_t_6);
      __Pyx_INCREF(function);
      __Pyx_DECREF_SET(__pyx_t_2, function);
    }
  }
  if (!__pyx_t_6) {
    __pyx_t_3 = __Pyx_PyObject_CallOneArg(__pyx_t_2, __pyx_v___pyx_type); if (unlikely(!__pyx_t_3)) __PYX_ERR(7, 5, __pyx_L1_error)
    __Pyx_GOTREF(__pyx_t_3);
  } else {
    #if CYTHON_FAST_PYCALL
    if (PyFunction_Check(__pyx_t_2)) {
      PyObject *__pyx_temp[2] = {__pyx_t_6, __pyx_v___pyx_type};
      __pyx_t_3 = __Pyx_PyFunction_FastCall(__pyx_t_2, __pyx_temp+1-1, 1+1); if (unlikely(!__pyx_t_3)) __PYX_ERR(7, 5, __pyx_L1_error)
      __Pyx_XDECREF(__pyx_t_6); __pyx_t_6 = 0;
      __Pyx_GOTREF(__pyx_t_3);
    } else
    #endif
    #if CYTHON_FAST_PYCCALL
    if (__Pyx_PyFastCFunction_Check(__pyx_t_2)) {
      PyObject *__pyx_temp[2] = {__pyx_t_6, __pyx_v___pyx_type};
      __pyx_t_3 = __Pyx_PyCFunction_FastCall(__pyx_t_2, __pyx_temp+1-1, 1+1); if (unlikely(!__pyx_t_3)) __PYX_ERR(7, 5, __pyx_L1_error)
      __Pyx_XDECREF(__pyx_t_6); __pyx_t_6 = 0;
      __Pyx_GOTREF(__pyx_t_3);
    } else
    #endif
    {
      __pyx_t_4 = PyTuple_New(1+1); if (unlikely(!__pyx_t_4)) __PYX_ERR(7, 5, __pyx_L1_error)
      __Pyx_GOTREF(__pyx_t_4);
      __Pyx_GIVEREF(__pyx_t_6); PyTuple_SET_ITEM(__pyx_t_4, 0, __pyx_t_6); __pyx_t_6 = NULL;
      __Pyx_INCREF(__pyx_v___pyx_type);
      __Pyx_GIVEREF(__pyx_v___pyx_type);
      PyTuple_SET_ITEM(__pyx_t_4, 0+1, __pyx_v___pyx_type);
      __pyx_t_3 = __Pyx_PyObject_Call(__pyx_t_2, __pyx_t_4, NULL); if (unlikely(!__pyx_t_3)) __PYX_ERR(7, 5, __pyx_L1_error)
      __Pyx_GOTREF(__pyx_t_3);
      __Pyx_DECREF(__pyx_t_4); __pyx_t_4 = 0;
    }
  }
  __Pyx_DECREF(__pyx_t_2); __pyx_t_2 = 0;
  __pyx_v___pyx_result = __pyx_t_3;
  __pyx_t_3 = 0;

  /* "(tree fragment)":6
 *         raise __pyx_PickleError("Incompatible checksums (%s vs 0xd41d8cd = ())" % __pyx_checksum)
 *     __pyx_result = _Tag.__new__(__pyx_type)
 *     if __pyx_state is not None:             # <<<<<<<<<<<<<<
 *         __pyx_unpickle__Tag__set_state(<_Tag> __pyx_result, __pyx_state)
 *     return __pyx_result
 */
  __pyx_t_1 = (__pyx_v___pyx_state != Py_None);
  __pyx_t_7 = (__pyx_t_1 != 0);
  if (__pyx_t_7) {

    /* "(tree fragment)":7
 *     __pyx_result = _Tag.__new__(__pyx_type)
 *     if __pyx_state is not None:
 *         __pyx_unpickle__Tag__set_state(<_Tag> __pyx_result, __pyx_state)             # <<<<<<<<<<<<<<
 *     return __pyx_result
 * cdef __pyx_unpickle__Tag__set_state(_Tag __pyx_result, tuple __pyx_state):
 */
    if (!(likely(PyTuple_CheckExact(__pyx_v___pyx_state))||((__pyx_v___pyx_state) == Py_None)||(PyErr_Format(PyExc_TypeError, "Expected %.16s, got %.200s", "tuple", Py_TYPE(__pyx_v___pyx_state)->tp_name), 0))) __PYX_ERR(7, 7, __pyx_L1_error)
    __pyx_t_3 = __pyx_f_4grpc_7_cython_6cygrpc___pyx_unpickle__Tag__set_state(((struct __pyx_obj_4grpc_7_cython_6cygrpc__Tag *)__pyx_v___pyx_result), ((PyObject*)__pyx_v___pyx_state)); if (unlikely(!__pyx_t_3)) __PYX_ERR(7, 7, __pyx_L1_error)
    __Pyx_GOTREF(__pyx_t_3);
    __Pyx_DECREF(__pyx_t_3); __pyx_t_3 = 0;

    /* "(tree fragment)":6
 *         raise __pyx_PickleError("Incompatible checksums (%s vs 0xd41d8cd = ())" % __pyx_checksum)
 *     __pyx_result = _Tag.__new__(__pyx_type)
 *     if __pyx_state is not None:             # <<<<<<<<<<<<<<
 *         __pyx_unpickle__Tag__set_state(<_Tag> __pyx_result, __pyx_state)
 *     return __pyx_result
 */
  }

  /* "(tree fragment)":8
 *     if __pyx_state is not None:
 *         __pyx_unpickle__Tag__set_state(<_Tag> __pyx_result, __pyx_state)
 *     return __pyx_result             # <<<<<<<<<<<<<<
 * cdef __pyx_unpickle__Tag__set_state(_Tag __pyx_result, tuple __pyx_state):
 *     if len(__pyx_state) > 0 and hasattr(__pyx_result, '__dict__'):
 */
  __Pyx_XDECREF(__pyx_r);
  __Pyx_INCREF(__pyx_v___pyx_result);
  __pyx_r = __pyx_v___pyx_result;
  goto __pyx_L0;

  /* "(tree fragment)":1
 * def __pyx_unpickle__Tag(__pyx_type, long __pyx_checksum, __pyx_state):             # <<<<<<<<<<<<<<
 *     if __pyx_checksum != 0xd41d8cd:
 *         from pickle import PickleError as __pyx_PickleError
 */

  /* function exit code */
  __pyx_L1_error:;
  __Pyx_XDECREF(__pyx_t_2);
  __Pyx_XDECREF(__pyx_t_3);
  __Pyx_XDECREF(__pyx_t_4);
  __Pyx_XDECREF(__pyx_t_5);
  __Pyx_XDECREF(__pyx_t_6);
  __Pyx_AddTraceback("grpc._cython.cygrpc.__pyx_unpickle__Tag", __pyx_clineno, __pyx_lineno, __pyx_filename);
  __pyx_r = NULL;
  __pyx_L0:;
  __Pyx_XDECREF(__pyx_v___pyx_PickleError);
  __Pyx_XDECREF(__pyx_v___pyx_result);
  __Pyx_XGIVEREF(__pyx_r);
  __Pyx_RefNannyFinishContext();
  return __pyx_r;
}

/* "(tree fragment)":9
 *         __pyx_unpickle__Tag__set_state(<_Tag> __pyx_result, __pyx_state)
 *     return __pyx_result
 * cdef __pyx_unpickle__Tag__set_state(_Tag __pyx_result, tuple __pyx_state):             # <<<<<<<<<<<<<<
 *     if len(__pyx_state) > 0 and hasattr(__pyx_result, '__dict__'):
 *         __pyx_result.__dict__.update(__pyx_state[0])
 */

static PyObject *__pyx_f_4grpc_7_cython_6cygrpc___pyx_unpickle__Tag__set_state(struct __pyx_obj_4grpc_7_cython_6cygrpc__Tag *__pyx_v___pyx_result, PyObject *__pyx_v___pyx_state) {
  PyObject *__pyx_r = NULL;
  __Pyx_RefNannyDeclarations
  int __pyx_t_1;
  Py_ssize_t __pyx_t_2;
  int __pyx_t_3;
  int __pyx_t_4;
  PyObject *__pyx_t_5 = NULL;
  PyObject *__pyx_t_6 = NULL;
  PyObject *__pyx_t_7 = NULL;
  PyObject *__pyx_t_8 = NULL;
  PyObject *__pyx_t_9 = NULL;
  __Pyx_RefNannySetupContext("__pyx_unpickle__Tag__set_state", 0);

  /* "(tree fragment)":10
 *     return __pyx_result
 * cdef __pyx_unpickle__Tag__set_state(_Tag __pyx_result, tuple __pyx_state):
 *     if len(__pyx_state) > 0 and hasattr(__pyx_result, '__dict__'):             # <<<<<<<<<<<<<<
 *         __pyx_result.__dict__.update(__pyx_state[0])
 */
  if (unlikely(__pyx_v___pyx_state == Py_None)) {
    PyErr_SetString(PyExc_TypeError, "object of type 'NoneType' has no len()");
    __PYX_ERR(7, 10, __pyx_L1_error)
  }
  __pyx_t_2 = PyTuple_GET_SIZE(__pyx_v___pyx_state); if (unlikely(__pyx_t_2 == ((Py_ssize_t)-1))) __PYX_ERR(7, 10, __pyx_L1_error)
  __pyx_t_3 = ((__pyx_t_2 > 0) != 0);
  if (__pyx_t_3) {
  } else {
    __pyx_t_1 = __pyx_t_3;
    goto __pyx_L4_bool_binop_done;
  }
  __pyx_t_3 = __Pyx_HasAttr(((PyObject *)__pyx_v___pyx_result), __pyx_n_s_dict); if (unlikely(__pyx_t_3 == ((int)-1))) __PYX_ERR(7, 10, __pyx_L1_error)
  __pyx_t_4 = (__pyx_t_3 != 0);
  __pyx_t_1 = __pyx_t_4;
  __pyx_L4_bool_binop_done:;
  if (__pyx_t_1) {

    /* "(tree fragment)":11
 * cdef __pyx_unpickle__Tag__set_state(_Tag __pyx_result, tuple __pyx_state):
 *     if len(__pyx_state) > 0 and hasattr(__pyx_result, '__dict__'):
 *         __pyx_result.__dict__.update(__pyx_state[0])             # <<<<<<<<<<<<<<
 */
    __pyx_t_6 = __Pyx_PyObject_GetAttrStr(((PyObject *)__pyx_v___pyx_result), __pyx_n_s_dict); if (unlikely(!__pyx_t_6)) __PYX_ERR(7, 11, __pyx_L1_error)
    __Pyx_GOTREF(__pyx_t_6);
    __pyx_t_7 = __Pyx_PyObject_GetAttrStr(__pyx_t_6, __pyx_n_s_update); if (unlikely(!__pyx_t_7)) __PYX_ERR(7, 11, __pyx_L1_error)
    __Pyx_GOTREF(__pyx_t_7);
    __Pyx_DECREF(__pyx_t_6); __pyx_t_6 = 0;
    if (unlikely(__pyx_v___pyx_state == Py_None)) {
      PyErr_SetString(PyExc_TypeError, "'NoneType' object is not subscriptable");
      __PYX_ERR(7, 11, __pyx_L1_error)
    }
    __pyx_t_6 = __Pyx_GetItemInt_Tuple(__pyx_v___pyx_state, 0, long, 1, __Pyx_PyInt_From_long, 0, 0, 1); if (unlikely(!__pyx_t_6)) __PYX_ERR(7, 11, __pyx_L1_error)
    __Pyx_GOTREF(__pyx_t_6);
    __pyx_t_8 = NULL;
    if (CYTHON_UNPACK_METHODS && likely(PyMethod_Check(__pyx_t_7))) {
      __pyx_t_8 = PyMethod_GET_SELF(__pyx_t_7);
      if (likely(__pyx_t_8)) {
        PyObject* function = PyMethod_GET_FUNCTION(__pyx_t_7);
        __Pyx_INCREF(__pyx_t_8);
        __Pyx_INCREF(function);
        __Pyx_DECREF_SET(__pyx_t_7, function);
      }
    }
    if (!__pyx_t_8) {
      __pyx_t_5 = __Pyx_PyObject_CallOneArg(__pyx_t_7, __pyx_t_6); if (unlikely(!__pyx_t_5)) __PYX_ERR(7, 11, __pyx_L1_error)
      __Pyx_DECREF(__pyx_t_6); __pyx_t_6 = 0;
      __Pyx_GOTREF(__pyx_t_5);
    } else {
      #if CYTHON_FAST_PYCALL
      if (PyFunction_Check(__pyx_t_7)) {
        PyObject *__pyx_temp[2] = {__pyx_t_8, __pyx_t_6};
        __pyx_t_5 = __Pyx_PyFunction_FastCall(__pyx_t_7, __pyx_temp+1-1, 1+1); if (unlikely(!__pyx_t_5)) __PYX_ERR(7, 11, __pyx_L1_error)
        __Pyx_XDECREF(__pyx_t_8); __pyx_t_8 = 0;
        __Pyx_GOTREF(__pyx_t_5);
        __Pyx_DECREF(__pyx_t_6); __pyx_t_6 = 0;
      } else
      #endif
      #if CYTHON_FAST_PYCCALL
      if (__Pyx_PyFastCFunction_Check(__pyx_t_7)) {
        PyObject *__pyx_temp[2] = {__pyx_t_8, __pyx_t_6};
        __pyx_t_5 = __Pyx_PyCFunction_FastCall(__pyx_t_7, __pyx_temp+1-1, 1+1); if (unlikely(!__pyx_t_5)) __PYX_ERR(7, 11, __pyx_L1_error)
        __Pyx_XDECREF(__pyx_t_8); __pyx_t_8 = 0;
        __Pyx_GOTREF(__pyx_t_5);
        __Pyx_DECREF(__pyx_t_6); __pyx_t_6 = 0;
      } else
      #endif
      {
        __pyx_t_9 = PyTuple_New(1+1); if (unlikely(!__pyx_t_9)) __PYX_ERR(7, 11, __pyx_L1_error)
        __Pyx_GOTREF(__pyx_t_9);
        __Pyx_GIVEREF(__pyx_t_8); PyTuple_SET_ITEM(__pyx_t_9, 0, __pyx_t_8); __pyx_t_8 = NULL;
        __Pyx_GIVEREF(__pyx_t_6);
        PyTuple_SET_ITEM(__pyx_t_9, 0+1, __pyx_t_6);
        __pyx_t_6 = 0;
        __pyx_t_5 = __Pyx_PyObject_Call(__pyx_t_7, __pyx_t_9, NULL); if (unlikely(!__pyx_t_5)) __PYX_ERR(7, 11, __pyx_L1_error)
        __Pyx_GOTREF(__pyx_t_5);
        __Pyx_DECREF(__pyx_t_9); __pyx_t_9 = 0;
      }
    }
    __Pyx_DECREF(__pyx_t_7); __pyx_t_7 = 0;
    __Pyx_DECREF(__pyx_t_5); __pyx_t_5 = 0;

    /* "(tree fragment)":10
 *     return __pyx_result
 * cdef __pyx_unpickle__Tag__set_state(_Tag __pyx_result, tuple __pyx_state):
 *     if len(__pyx_state) > 0 and hasattr(__pyx_result, '__dict__'):             # <<<<<<<<<<<<<<
 *         __pyx_result.__dict__.update(__pyx_state[0])
 */
  }

  /* "(tree fragment)":9
 *         __pyx_unpickle__Tag__set_state(<_Tag> __pyx_result, __pyx_state)
 *     return __pyx_result
 * cdef __pyx_unpickle__Tag__set_state(_Tag __pyx_result, tuple __pyx_state):             # <<<<<<<<<<<<<<
 *     if len(__pyx_state) > 0 and hasattr(__pyx_result, '__dict__'):
 *         __pyx_result.__dict__.update(__pyx_state[0])
 */

  /* function exit code */
  __pyx_r = Py_None; __Pyx_INCREF(Py_None);
  goto __pyx_L0;
  __pyx_L1_error:;
  __Pyx_XDECREF(__pyx_t_5);
  __Pyx_XDECREF(__pyx_t_6);
  __Pyx_XDECREF(__pyx_t_7);
  __Pyx_XDECREF(__pyx_t_8);
  __Pyx_XDECREF(__pyx_t_9);
  __Pyx_AddTraceback("grpc._cython.cygrpc.__pyx_unpickle__Tag__set_state", __pyx_clineno, __pyx_lineno, __pyx_filename);
  __pyx_r = 0;
  __pyx_L0:;
  __Pyx_XGIVEREF(__pyx_r);
  __Pyx_RefNannyFinishContext();
  return __pyx_r;
}

/* "(tree fragment)":1
 * def __pyx_unpickle_CensusContext(__pyx_type, long __pyx_checksum, __pyx_state):             # <<<<<<<<<<<<<<
 *     if __pyx_checksum != 0xd41d8cd:
 *         from pickle import PickleError as __pyx_PickleError
 */

/* Python wrapper */
static PyObject *__pyx_pw_4grpc_7_cython_6cygrpc_83__pyx_unpickle_CensusContext(PyObject *__pyx_self, PyObject *__pyx_args, PyObject *__pyx_kwds); /*proto*/
static PyMethodDef __pyx_mdef_4grpc_7_cython_6cygrpc_83__pyx_unpickle_CensusContext = {"__pyx_unpickle_CensusContext", (PyCFunction)__pyx_pw_4grpc_7_cython_6cygrpc_83__pyx_unpickle_CensusContext, METH_VARARGS|METH_KEYWORDS, 0};
static PyObject *__pyx_pw_4grpc_7_cython_6cygrpc_83__pyx_unpickle_CensusContext(PyObject *__pyx_self, PyObject *__pyx_args, PyObject *__pyx_kwds) {
  PyObject *__pyx_v___pyx_type = 0;
  long __pyx_v___pyx_checksum;
  PyObject *__pyx_v___pyx_state = 0;
  PyObject *__pyx_r = 0;
  __Pyx_RefNannyDeclarations
  __Pyx_RefNannySetupContext("__pyx_unpickle_CensusContext (wrapper)", 0);
  {
    static PyObject **__pyx_pyargnames[] = {&__pyx_n_s_pyx_type,&__pyx_n_s_pyx_checksum,&__pyx_n_s_pyx_state,0};
    PyObject* values[3] = {0,0,0};
    if (unlikely(__pyx_kwds)) {
      Py_ssize_t kw_args;
      const Py_ssize_t pos_args = PyTuple_GET_SIZE(__pyx_args);
      switch (pos_args) {
        case  3: values[2] = PyTuple_GET_ITEM(__pyx_args, 2);
        CYTHON_FALLTHROUGH;
        case  2: values[1] = PyTuple_GET_ITEM(__pyx_args, 1);
        CYTHON_FALLTHROUGH;
        case  1: values[0] = PyTuple_GET_ITEM(__pyx_args, 0);
        CYTHON_FALLTHROUGH;
        case  0: break;
        default: goto __pyx_L5_argtuple_error;
      }
      kw_args = PyDict_Size(__pyx_kwds);
      switch (pos_args) {
        case  0:
        if (likely((values[0] = __Pyx_PyDict_GetItemStr(__pyx_kwds, __pyx_n_s_pyx_type)) != 0)) kw_args--;
        else goto __pyx_L5_argtuple_error;
        CYTHON_FALLTHROUGH;
        case  1:
        if (likely((values[1] = __Pyx_PyDict_GetItemStr(__pyx_kwds, __pyx_n_s_pyx_checksum)) != 0)) kw_args--;
        else {
          __Pyx_RaiseArgtupleInvalid("__pyx_unpickle_CensusContext", 1, 3, 3, 1); __PYX_ERR(7, 1, __pyx_L3_error)
        }
        CYTHON_FALLTHROUGH;
        case  2:
        if (likely((values[2] = __Pyx_PyDict_GetItemStr(__pyx_kwds, __pyx_n_s_pyx_state)) != 0)) kw_args--;
        else {
          __Pyx_RaiseArgtupleInvalid("__pyx_unpickle_CensusContext", 1, 3, 3, 2); __PYX_ERR(7, 1, __pyx_L3_error)
        }
      }
      if (unlikely(kw_args > 0)) {
        if (unlikely(__Pyx_ParseOptionalKeywords(__pyx_kwds, __pyx_pyargnames, 0, values, pos_args, "__pyx_unpickle_CensusContext") < 0)) __PYX_ERR(7, 1, __pyx_L3_error)
      }
    } else if (PyTuple_GET_SIZE(__pyx_args) != 3) {
      goto __pyx_L5_argtuple_error;
    } else {
      values[0] = PyTuple_GET_ITEM(__pyx_args, 0);
      values[1] = PyTuple_GET_ITEM(__pyx_args, 1);
      values[2] = PyTuple_GET_ITEM(__pyx_args, 2);
    }
    __pyx_v___pyx_type = values[0];
    __pyx_v___pyx_checksum = __Pyx_PyInt_As_long(values[1]); if (unlikely((__pyx_v___pyx_checksum == (long)-1) && PyErr_Occurred())) __PYX_ERR(7, 1, __pyx_L3_error)
    __pyx_v___pyx_state = values[2];
  }
  goto __pyx_L4_argument_unpacking_done;
  __pyx_L5_argtuple_error:;
  __Pyx_RaiseArgtupleInvalid("__pyx_unpickle_CensusContext", 1, 3, 3, PyTuple_GET_SIZE(__pyx_args)); __PYX_ERR(7, 1, __pyx_L3_error)
  __pyx_L3_error:;
  __Pyx_AddTraceback("grpc._cython.cygrpc.__pyx_unpickle_CensusContext", __pyx_clineno, __pyx_lineno, __pyx_filename);
  __Pyx_RefNannyFinishContext();
  return NULL;
  __pyx_L4_argument_unpacking_done:;
  __pyx_r = __pyx_pf_4grpc_7_cython_6cygrpc_82__pyx_unpickle_CensusContext(__pyx_self, __pyx_v___pyx_type, __pyx_v___pyx_checksum, __pyx_v___pyx_state);

  /* function exit code */
  __Pyx_RefNannyFinishContext();
  return __pyx_r;
}

static PyObject *__pyx_pf_4grpc_7_cython_6cygrpc_82__pyx_unpickle_CensusContext(CYTHON_UNUSED PyObject *__pyx_self, PyObject *__pyx_v___pyx_type, long __pyx_v___pyx_checksum, PyObject *__pyx_v___pyx_state) {
  PyObject *__pyx_v___pyx_PickleError = NULL;
  PyObject *__pyx_v___pyx_result = NULL;
  PyObject *__pyx_r = NULL;
  __Pyx_RefNannyDeclarations
  int __pyx_t_1;
  PyObject *__pyx_t_2 = NULL;
  PyObject *__pyx_t_3 = NULL;
  PyObject *__pyx_t_4 = NULL;
  PyObject *__pyx_t_5 = NULL;
  PyObject *__pyx_t_6 = NULL;
  int __pyx_t_7;
  __Pyx_RefNannySetupContext("__pyx_unpickle_CensusContext", 0);

  /* "(tree fragment)":2
 * def __pyx_unpickle_CensusContext(__pyx_type, long __pyx_checksum, __pyx_state):
 *     if __pyx_checksum != 0xd41d8cd:             # <<<<<<<<<<<<<<
 *         from pickle import PickleError as __pyx_PickleError
 *         raise __pyx_PickleError("Incompatible checksums (%s vs 0xd41d8cd = ())" % __pyx_checksum)
 */
  __pyx_t_1 = ((__pyx_v___pyx_checksum != 0xd41d8cd) != 0);
  if (__pyx_t_1) {

    /* "(tree fragment)":3
 * def __pyx_unpickle_CensusContext(__pyx_type, long __pyx_checksum, __pyx_state):
 *     if __pyx_checksum != 0xd41d8cd:
 *         from pickle import PickleError as __pyx_PickleError             # <<<<<<<<<<<<<<
 *         raise __pyx_PickleError("Incompatible checksums (%s vs 0xd41d8cd = ())" % __pyx_checksum)
 *     __pyx_result = CensusContext.__new__(__pyx_type)
 */
    __pyx_t_2 = PyList_New(1); if (unlikely(!__pyx_t_2)) __PYX_ERR(7, 3, __pyx_L1_error)
    __Pyx_GOTREF(__pyx_t_2);
    __Pyx_INCREF(__pyx_n_s_PickleError);
    __Pyx_GIVEREF(__pyx_n_s_PickleError);
    PyList_SET_ITEM(__pyx_t_2, 0, __pyx_n_s_PickleError);
    __pyx_t_3 = __Pyx_Import(__pyx_n_s_pickle, __pyx_t_2, -1); if (unlikely(!__pyx_t_3)) __PYX_ERR(7, 3, __pyx_L1_error)
    __Pyx_GOTREF(__pyx_t_3);
    __Pyx_DECREF(__pyx_t_2); __pyx_t_2 = 0;
    __pyx_t_2 = __Pyx_ImportFrom(__pyx_t_3, __pyx_n_s_PickleError); if (unlikely(!__pyx_t_2)) __PYX_ERR(7, 3, __pyx_L1_error)
    __Pyx_GOTREF(__pyx_t_2);
    __Pyx_INCREF(__pyx_t_2);
    __pyx_v___pyx_PickleError = __pyx_t_2;
    __Pyx_DECREF(__pyx_t_2); __pyx_t_2 = 0;
    __Pyx_DECREF(__pyx_t_3); __pyx_t_3 = 0;

    /* "(tree fragment)":4
 *     if __pyx_checksum != 0xd41d8cd:
 *         from pickle import PickleError as __pyx_PickleError
 *         raise __pyx_PickleError("Incompatible checksums (%s vs 0xd41d8cd = ())" % __pyx_checksum)             # <<<<<<<<<<<<<<
 *     __pyx_result = CensusContext.__new__(__pyx_type)
 *     if __pyx_state is not None:
 */
    __pyx_t_2 = __Pyx_PyInt_From_long(__pyx_v___pyx_checksum); if (unlikely(!__pyx_t_2)) __PYX_ERR(7, 4, __pyx_L1_error)
    __Pyx_GOTREF(__pyx_t_2);
    __pyx_t_4 = __Pyx_PyString_Format(__pyx_kp_s_Incompatible_checksums_s_vs_0xd4, __pyx_t_2); if (unlikely(!__pyx_t_4)) __PYX_ERR(7, 4, __pyx_L1_error)
    __Pyx_GOTREF(__pyx_t_4);
    __Pyx_DECREF(__pyx_t_2); __pyx_t_2 = 0;
    __Pyx_INCREF(__pyx_v___pyx_PickleError);
    __pyx_t_2 = __pyx_v___pyx_PickleError; __pyx_t_5 = NULL;
    if (CYTHON_UNPACK_METHODS && unlikely(PyMethod_Check(__pyx_t_2))) {
      __pyx_t_5 = PyMethod_GET_SELF(__pyx_t_2);
      if (likely(__pyx_t_5)) {
        PyObject* function = PyMethod_GET_FUNCTION(__pyx_t_2);
        __Pyx_INCREF(__pyx_t_5);
        __Pyx_INCREF(function);
        __Pyx_DECREF_SET(__pyx_t_2, function);
      }
    }
    if (!__pyx_t_5) {
      __pyx_t_3 = __Pyx_PyObject_CallOneArg(__pyx_t_2, __pyx_t_4); if (unlikely(!__pyx_t_3)) __PYX_ERR(7, 4, __pyx_L1_error)
      __Pyx_DECREF(__pyx_t_4); __pyx_t_4 = 0;
      __Pyx_GOTREF(__pyx_t_3);
    } else {
      #if CYTHON_FAST_PYCALL
      if (PyFunction_Check(__pyx_t_2)) {
        PyObject *__pyx_temp[2] = {__pyx_t_5, __pyx_t_4};
        __pyx_t_3 = __Pyx_PyFunction_FastCall(__pyx_t_2, __pyx_temp+1-1, 1+1); if (unlikely(!__pyx_t_3)) __PYX_ERR(7, 4, __pyx_L1_error)
        __Pyx_XDECREF(__pyx_t_5); __pyx_t_5 = 0;
        __Pyx_GOTREF(__pyx_t_3);
        __Pyx_DECREF(__pyx_t_4); __pyx_t_4 = 0;
      } else
      #endif
      #if CYTHON_FAST_PYCCALL
      if (__Pyx_PyFastCFunction_Check(__pyx_t_2)) {
        PyObject *__pyx_temp[2] = {__pyx_t_5, __pyx_t_4};
        __pyx_t_3 = __Pyx_PyCFunction_FastCall(__pyx_t_2, __pyx_temp+1-1, 1+1); if (unlikely(!__pyx_t_3)) __PYX_ERR(7, 4, __pyx_L1_error)
        __Pyx_XDECREF(__pyx_t_5); __pyx_t_5 = 0;
        __Pyx_GOTREF(__pyx_t_3);
        __Pyx_DECREF(__pyx_t_4); __pyx_t_4 = 0;
      } else
      #endif
      {
        __pyx_t_6 = PyTuple_New(1+1); if (unlikely(!__pyx_t_6)) __PYX_ERR(7, 4, __pyx_L1_error)
        __Pyx_GOTREF(__pyx_t_6);
        __Pyx_GIVEREF(__pyx_t_5); PyTuple_SET_ITEM(__pyx_t_6, 0, __pyx_t_5); __pyx_t_5 = NULL;
        __Pyx_GIVEREF(__pyx_t_4);
        PyTuple_SET_ITEM(__pyx_t_6, 0+1, __pyx_t_4);
        __pyx_t_4 = 0;
        __pyx_t_3 = __Pyx_PyObject_Call(__pyx_t_2, __pyx_t_6, NULL); if (unlikely(!__pyx_t_3)) __PYX_ERR(7, 4, __pyx_L1_error)
        __Pyx_GOTREF(__pyx_t_3);
        __Pyx_DECREF(__pyx_t_6); __pyx_t_6 = 0;
      }
    }
    __Pyx_DECREF(__pyx_t_2); __pyx_t_2 = 0;
    __Pyx_Raise(__pyx_t_3, 0, 0, 0);
    __Pyx_DECREF(__pyx_t_3); __pyx_t_3 = 0;
    __PYX_ERR(7, 4, __pyx_L1_error)

    /* "(tree fragment)":2
 * def __pyx_unpickle_CensusContext(__pyx_type, long __pyx_checksum, __pyx_state):
 *     if __pyx_checksum != 0xd41d8cd:             # <<<<<<<<<<<<<<
 *         from pickle import PickleError as __pyx_PickleError
 *         raise __pyx_PickleError("Incompatible checksums (%s vs 0xd41d8cd = ())" % __pyx_checksum)
 */
  }

  /* "(tree fragment)":5
 *         from pickle import PickleError as __pyx_PickleError
 *         raise __pyx_PickleError("Incompatible checksums (%s vs 0xd41d8cd = ())" % __pyx_checksum)
 *     __pyx_result = CensusContext.__new__(__pyx_type)             # <<<<<<<<<<<<<<
 *     if __pyx_state is not None:
 *         __pyx_unpickle_CensusContext__set_state(<CensusContext> __pyx_result, __pyx_state)
 */
  __pyx_t_2 = __Pyx_PyObject_GetAttrStr(((PyObject *)__pyx_ptype_4grpc_7_cython_6cygrpc_CensusContext), __pyx_n_s_new); if (unlikely(!__pyx_t_2)) __PYX_ERR(7, 5, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_2);
  __pyx_t_6 = NULL;
  if (CYTHON_UNPACK_METHODS && likely(PyMethod_Check(__pyx_t_2))) {
    __pyx_t_6 = PyMethod_GET_SELF(__pyx_t_2);
    if (likely(__pyx_t_6)) {
      PyObject* function = PyMethod_GET_FUNCTION(__pyx_t_2);
      __Pyx_INCREF(__pyx_t_6);
      __Pyx_INCREF(function);
      __Pyx_DECREF_SET(__pyx_t_2, function);
    }
  }
  if (!__pyx_t_6) {
    __pyx_t_3 = __Pyx_PyObject_CallOneArg(__pyx_t_2, __pyx_v___pyx_type); if (unlikely(!__pyx_t_3)) __PYX_ERR(7, 5, __pyx_L1_error)
    __Pyx_GOTREF(__pyx_t_3);
  } else {
    #if CYTHON_FAST_PYCALL
    if (PyFunction_Check(__pyx_t_2)) {
      PyObject *__pyx_temp[2] = {__pyx_t_6, __pyx_v___pyx_type};
      __pyx_t_3 = __Pyx_PyFunction_FastCall(__pyx_t_2, __pyx_temp+1-1, 1+1); if (unlikely(!__pyx_t_3)) __PYX_ERR(7, 5, __pyx_L1_error)
      __Pyx_XDECREF(__pyx_t_6); __pyx_t_6 = 0;
      __Pyx_GOTREF(__pyx_t_3);
    } else
    #endif
    #if CYTHON_FAST_PYCCALL
    if (__Pyx_PyFastCFunction_Check(__pyx_t_2)) {
      PyObject *__pyx_temp[2] = {__pyx_t_6, __pyx_v___pyx_type};
      __pyx_t_3 = __Pyx_PyCFunction_FastCall(__pyx_t_2, __pyx_temp+1-1, 1+1); if (unlikely(!__pyx_t_3)) __PYX_ERR(7, 5, __pyx_L1_error)
      __Pyx_XDECREF(__pyx_t_6); __pyx_t_6 = 0;
      __Pyx_GOTREF(__pyx_t_3);
    } else
    #endif
    {
      __pyx_t_4 = PyTuple_New(1+1); if (unlikely(!__pyx_t_4)) __PYX_ERR(7, 5, __pyx_L1_error)
      __Pyx_GOTREF(__pyx_t_4);
      __Pyx_GIVEREF(__pyx_t_6); PyTuple_SET_ITEM(__pyx_t_4, 0, __pyx_t_6); __pyx_t_6 = NULL;
      __Pyx_INCREF(__pyx_v___pyx_type);
      __Pyx_GIVEREF(__pyx_v___pyx_type);
      PyTuple_SET_ITEM(__pyx_t_4, 0+1, __pyx_v___pyx_type);
      __pyx_t_3 = __Pyx_PyObject_Call(__pyx_t_2, __pyx_t_4, NULL); if (unlikely(!__pyx_t_3)) __PYX_ERR(7, 5, __pyx_L1_error)
      __Pyx_GOTREF(__pyx_t_3);
      __Pyx_DECREF(__pyx_t_4); __pyx_t_4 = 0;
    }
  }
  __Pyx_DECREF(__pyx_t_2); __pyx_t_2 = 0;
  __pyx_v___pyx_result = __pyx_t_3;
  __pyx_t_3 = 0;

  /* "(tree fragment)":6
 *         raise __pyx_PickleError("Incompatible checksums (%s vs 0xd41d8cd = ())" % __pyx_checksum)
 *     __pyx_result = CensusContext.__new__(__pyx_type)
 *     if __pyx_state is not None:             # <<<<<<<<<<<<<<
 *         __pyx_unpickle_CensusContext__set_state(<CensusContext> __pyx_result, __pyx_state)
 *     return __pyx_result
 */
  __pyx_t_1 = (__pyx_v___pyx_state != Py_None);
  __pyx_t_7 = (__pyx_t_1 != 0);
  if (__pyx_t_7) {

    /* "(tree fragment)":7
 *     __pyx_result = CensusContext.__new__(__pyx_type)
 *     if __pyx_state is not None:
 *         __pyx_unpickle_CensusContext__set_state(<CensusContext> __pyx_result, __pyx_state)             # <<<<<<<<<<<<<<
 *     return __pyx_result
 * cdef __pyx_unpickle_CensusContext__set_state(CensusContext __pyx_result, tuple __pyx_state):
 */
    if (!(likely(PyTuple_CheckExact(__pyx_v___pyx_state))||((__pyx_v___pyx_state) == Py_None)||(PyErr_Format(PyExc_TypeError, "Expected %.16s, got %.200s", "tuple", Py_TYPE(__pyx_v___pyx_state)->tp_name), 0))) __PYX_ERR(7, 7, __pyx_L1_error)
    __pyx_t_3 = __pyx_f_4grpc_7_cython_6cygrpc___pyx_unpickle_CensusContext__set_state(((struct __pyx_obj_4grpc_7_cython_6cygrpc_CensusContext *)__pyx_v___pyx_result), ((PyObject*)__pyx_v___pyx_state)); if (unlikely(!__pyx_t_3)) __PYX_ERR(7, 7, __pyx_L1_error)
    __Pyx_GOTREF(__pyx_t_3);
    __Pyx_DECREF(__pyx_t_3); __pyx_t_3 = 0;

    /* "(tree fragment)":6
 *         raise __pyx_PickleError("Incompatible checksums (%s vs 0xd41d8cd = ())" % __pyx_checksum)
 *     __pyx_result = CensusContext.__new__(__pyx_type)
 *     if __pyx_state is not None:             # <<<<<<<<<<<<<<
 *         __pyx_unpickle_CensusContext__set_state(<CensusContext> __pyx_result, __pyx_state)
 *     return __pyx_result
 */
  }

  /* "(tree fragment)":8
 *     if __pyx_state is not None:
 *         __pyx_unpickle_CensusContext__set_state(<CensusContext> __pyx_result, __pyx_state)
 *     return __pyx_result             # <<<<<<<<<<<<<<
 * cdef __pyx_unpickle_CensusContext__set_state(CensusContext __pyx_result, tuple __pyx_state):
 *     if len(__pyx_state) > 0 and hasattr(__pyx_result, '__dict__'):
 */
  __Pyx_XDECREF(__pyx_r);
  __Pyx_INCREF(__pyx_v___pyx_result);
  __pyx_r = __pyx_v___pyx_result;
  goto __pyx_L0;

  /* "(tree fragment)":1
 * def __pyx_unpickle_CensusContext(__pyx_type, long __pyx_checksum, __pyx_state):             # <<<<<<<<<<<<<<
 *     if __pyx_checksum != 0xd41d8cd:
 *         from pickle import PickleError as __pyx_PickleError
 */

  /* function exit code */
  __pyx_L1_error:;
  __Pyx_XDECREF(__pyx_t_2);
  __Pyx_XDECREF(__pyx_t_3);
  __Pyx_XDECREF(__pyx_t_4);
  __Pyx_XDECREF(__pyx_t_5);
  __Pyx_XDECREF(__pyx_t_6);
  __Pyx_AddTraceback("grpc._cython.cygrpc.__pyx_unpickle_CensusContext", __pyx_clineno, __pyx_lineno, __pyx_filename);
  __pyx_r = NULL;
  __pyx_L0:;
  __Pyx_XDECREF(__pyx_v___pyx_PickleError);
  __Pyx_XDECREF(__pyx_v___pyx_result);
  __Pyx_XGIVEREF(__pyx_r);
  __Pyx_RefNannyFinishContext();
  return __pyx_r;
}

/* "(tree fragment)":9
 *         __pyx_unpickle_CensusContext__set_state(<CensusContext> __pyx_result, __pyx_state)
 *     return __pyx_result
 * cdef __pyx_unpickle_CensusContext__set_state(CensusContext __pyx_result, tuple __pyx_state):             # <<<<<<<<<<<<<<
 *     if len(__pyx_state) > 0 and hasattr(__pyx_result, '__dict__'):
 *         __pyx_result.__dict__.update(__pyx_state[0])
 */

static PyObject *__pyx_f_4grpc_7_cython_6cygrpc___pyx_unpickle_CensusContext__set_state(struct __pyx_obj_4grpc_7_cython_6cygrpc_CensusContext *__pyx_v___pyx_result, PyObject *__pyx_v___pyx_state) {
  PyObject *__pyx_r = NULL;
  __Pyx_RefNannyDeclarations
  int __pyx_t_1;
  Py_ssize_t __pyx_t_2;
  int __pyx_t_3;
  int __pyx_t_4;
  PyObject *__pyx_t_5 = NULL;
  PyObject *__pyx_t_6 = NULL;
  PyObject *__pyx_t_7 = NULL;
  PyObject *__pyx_t_8 = NULL;
  PyObject *__pyx_t_9 = NULL;
  __Pyx_RefNannySetupContext("__pyx_unpickle_CensusContext__set_state", 0);

  /* "(tree fragment)":10
 *     return __pyx_result
 * cdef __pyx_unpickle_CensusContext__set_state(CensusContext __pyx_result, tuple __pyx_state):
 *     if len(__pyx_state) > 0 and hasattr(__pyx_result, '__dict__'):             # <<<<<<<<<<<<<<
 *         __pyx_result.__dict__.update(__pyx_state[0])
 */
  if (unlikely(__pyx_v___pyx_state == Py_None)) {
    PyErr_SetString(PyExc_TypeError, "object of type 'NoneType' has no len()");
    __PYX_ERR(7, 10, __pyx_L1_error)
  }
  __pyx_t_2 = PyTuple_GET_SIZE(__pyx_v___pyx_state); if (unlikely(__pyx_t_2 == ((Py_ssize_t)-1))) __PYX_ERR(7, 10, __pyx_L1_error)
  __pyx_t_3 = ((__pyx_t_2 > 0) != 0);
  if (__pyx_t_3) {
  } else {
    __pyx_t_1 = __pyx_t_3;
    goto __pyx_L4_bool_binop_done;
  }
  __pyx_t_3 = __Pyx_HasAttr(((PyObject *)__pyx_v___pyx_result), __pyx_n_s_dict); if (unlikely(__pyx_t_3 == ((int)-1))) __PYX_ERR(7, 10, __pyx_L1_error)
  __pyx_t_4 = (__pyx_t_3 != 0);
  __pyx_t_1 = __pyx_t_4;
  __pyx_L4_bool_binop_done:;
  if (__pyx_t_1) {

    /* "(tree fragment)":11
 * cdef __pyx_unpickle_CensusContext__set_state(CensusContext __pyx_result, tuple __pyx_state):
 *     if len(__pyx_state) > 0 and hasattr(__pyx_result, '__dict__'):
 *         __pyx_result.__dict__.update(__pyx_state[0])             # <<<<<<<<<<<<<<
 */
    __pyx_t_6 = __Pyx_PyObject_GetAttrStr(((PyObject *)__pyx_v___pyx_result), __pyx_n_s_dict); if (unlikely(!__pyx_t_6)) __PYX_ERR(7, 11, __pyx_L1_error)
    __Pyx_GOTREF(__pyx_t_6);
    __pyx_t_7 = __Pyx_PyObject_GetAttrStr(__pyx_t_6, __pyx_n_s_update); if (unlikely(!__pyx_t_7)) __PYX_ERR(7, 11, __pyx_L1_error)
    __Pyx_GOTREF(__pyx_t_7);
    __Pyx_DECREF(__pyx_t_6); __pyx_t_6 = 0;
    if (unlikely(__pyx_v___pyx_state == Py_None)) {
      PyErr_SetString(PyExc_TypeError, "'NoneType' object is not subscriptable");
      __PYX_ERR(7, 11, __pyx_L1_error)
    }
    __pyx_t_6 = __Pyx_GetItemInt_Tuple(__pyx_v___pyx_state, 0, long, 1, __Pyx_PyInt_From_long, 0, 0, 1); if (unlikely(!__pyx_t_6)) __PYX_ERR(7, 11, __pyx_L1_error)
    __Pyx_GOTREF(__pyx_t_6);
    __pyx_t_8 = NULL;
    if (CYTHON_UNPACK_METHODS && likely(PyMethod_Check(__pyx_t_7))) {
      __pyx_t_8 = PyMethod_GET_SELF(__pyx_t_7);
      if (likely(__pyx_t_8)) {
        PyObject* function = PyMethod_GET_FUNCTION(__pyx_t_7);
        __Pyx_INCREF(__pyx_t_8);
        __Pyx_INCREF(function);
        __Pyx_DECREF_SET(__pyx_t_7, function);
      }
    }
    if (!__pyx_t_8) {
      __pyx_t_5 = __Pyx_PyObject_CallOneArg(__pyx_t_7, __pyx_t_6); if (unlikely(!__pyx_t_5)) __PYX_ERR(7, 11, __pyx_L1_error)
      __Pyx_DECREF(__pyx_t_6); __pyx_t_6 = 0;
      __Pyx_GOTREF(__pyx_t_5);
    } else {
      #if CYTHON_FAST_PYCALL
      if (PyFunction_Check(__pyx_t_7)) {
        PyObject *__pyx_temp[2] = {__pyx_t_8, __pyx_t_6};
        __pyx_t_5 = __Pyx_PyFunction_FastCall(__pyx_t_7, __pyx_temp+1-1, 1+1); if (unlikely(!__pyx_t_5)) __PYX_ERR(7, 11, __pyx_L1_error)
        __Pyx_XDECREF(__pyx_t_8); __pyx_t_8 = 0;
        __Pyx_GOTREF(__pyx_t_5);
        __Pyx_DECREF(__pyx_t_6); __pyx_t_6 = 0;
      } else
      #endif
      #if CYTHON_FAST_PYCCALL
      if (__Pyx_PyFastCFunction_Check(__pyx_t_7)) {
        PyObject *__pyx_temp[2] = {__pyx_t_8, __pyx_t_6};
        __pyx_t_5 = __Pyx_PyCFunction_FastCall(__pyx_t_7, __pyx_temp+1-1, 1+1); if (unlikely(!__pyx_t_5)) __PYX_ERR(7, 11, __pyx_L1_error)
        __Pyx_XDECREF(__pyx_t_8); __pyx_t_8 = 0;
        __Pyx_GOTREF(__pyx_t_5);
        __Pyx_DECREF(__pyx_t_6); __pyx_t_6 = 0;
      } else
      #endif
      {
        __pyx_t_9 = PyTuple_New(1+1); if (unlikely(!__pyx_t_9)) __PYX_ERR(7, 11, __pyx_L1_error)
        __Pyx_GOTREF(__pyx_t_9);
        __Pyx_GIVEREF(__pyx_t_8); PyTuple_SET_ITEM(__pyx_t_9, 0, __pyx_t_8); __pyx_t_8 = NULL;
        __Pyx_GIVEREF(__pyx_t_6);
        PyTuple_SET_ITEM(__pyx_t_9, 0+1, __pyx_t_6);
        __pyx_t_6 = 0;
        __pyx_t_5 = __Pyx_PyObject_Call(__pyx_t_7, __pyx_t_9, NULL); if (unlikely(!__pyx_t_5)) __PYX_ERR(7, 11, __pyx_L1_error)
        __Pyx_GOTREF(__pyx_t_5);
        __Pyx_DECREF(__pyx_t_9); __pyx_t_9 = 0;
      }
    }
    __Pyx_DECREF(__pyx_t_7); __pyx_t_7 = 0;
    __Pyx_DECREF(__pyx_t_5); __pyx_t_5 = 0;

    /* "(tree fragment)":10
 *     return __pyx_result
 * cdef __pyx_unpickle_CensusContext__set_state(CensusContext __pyx_result, tuple __pyx_state):
 *     if len(__pyx_state) > 0 and hasattr(__pyx_result, '__dict__'):             # <<<<<<<<<<<<<<
 *         __pyx_result.__dict__.update(__pyx_state[0])
 */
  }

  /* "(tree fragment)":9
 *         __pyx_unpickle_CensusContext__set_state(<CensusContext> __pyx_result, __pyx_state)
 *     return __pyx_result
 * cdef __pyx_unpickle_CensusContext__set_state(CensusContext __pyx_result, tuple __pyx_state):             # <<<<<<<<<<<<<<
 *     if len(__pyx_state) > 0 and hasattr(__pyx_result, '__dict__'):
 *         __pyx_result.__dict__.update(__pyx_state[0])
 */

  /* function exit code */
  __pyx_r = Py_None; __Pyx_INCREF(Py_None);
  goto __pyx_L0;
  __pyx_L1_error:;
  __Pyx_XDECREF(__pyx_t_5);
  __Pyx_XDECREF(__pyx_t_6);
  __Pyx_XDECREF(__pyx_t_7);
  __Pyx_XDECREF(__pyx_t_8);
  __Pyx_XDECREF(__pyx_t_9);
  __Pyx_AddTraceback("grpc._cython.cygrpc.__pyx_unpickle_CensusContext__set_state", __pyx_clineno, __pyx_lineno, __pyx_filename);
  __pyx_r = 0;
  __pyx_L0:;
  __Pyx_XGIVEREF(__pyx_r);
  __Pyx_RefNannyFinishContext();
  return __pyx_r;
}
static struct __pyx_vtabstruct_4grpc_7_cython_6cygrpc__ChannelArg __pyx_vtable_4grpc_7_cython_6cygrpc__ChannelArg;

static PyObject *__pyx_tp_new_4grpc_7_cython_6cygrpc__ChannelArg(PyTypeObject *t, CYTHON_UNUSED PyObject *a, CYTHON_UNUSED PyObject *k) {
  struct __pyx_obj_4grpc_7_cython_6cygrpc__ChannelArg *p;
  PyObject *o;
  if (likely((t->tp_flags & Py_TPFLAGS_IS_ABSTRACT) == 0)) {
    o = (*t->tp_alloc)(t, 0);
  } else {
    o = (PyObject *) PyBaseObject_Type.tp_new(t, __pyx_empty_tuple, 0);
  }
  if (unlikely(!o)) return 0;
  p = ((struct __pyx_obj_4grpc_7_cython_6cygrpc__ChannelArg *)o);
  p->__pyx_vtab = __pyx_vtabptr_4grpc_7_cython_6cygrpc__ChannelArg;
  return o;
}

static void __pyx_tp_dealloc_4grpc_7_cython_6cygrpc__ChannelArg(PyObject *o) {
  #if CYTHON_USE_TP_FINALIZE
  if (unlikely(PyType_HasFeature(Py_TYPE(o), Py_TPFLAGS_HAVE_FINALIZE) && Py_TYPE(o)->tp_finalize) && (!PyType_IS_GC(Py_TYPE(o)) || !_PyGC_FINALIZED(o))) {
    if (PyObject_CallFinalizerFromDealloc(o)) return;
  }
  #endif
  (*Py_TYPE(o)->tp_free)(o);
}

static PyMethodDef __pyx_methods_4grpc_7_cython_6cygrpc__ChannelArg[] = {
  {"__reduce_cython__", (PyCFunction)__pyx_pw_4grpc_7_cython_6cygrpc_11_ChannelArg_1__reduce_cython__, METH_NOARGS, 0},
  {"__setstate_cython__", (PyCFunction)__pyx_pw_4grpc_7_cython_6cygrpc_11_ChannelArg_3__setstate_cython__, METH_O, 0},
  {0, 0, 0, 0}
};

static PyTypeObject __pyx_type_4grpc_7_cython_6cygrpc__ChannelArg = {
  PyVarObject_HEAD_INIT(0, 0)
  "grpc._cython.cygrpc._ChannelArg", /*tp_name*/
  sizeof(struct __pyx_obj_4grpc_7_cython_6cygrpc__ChannelArg), /*tp_basicsize*/
  0, /*tp_itemsize*/
  __pyx_tp_dealloc_4grpc_7_cython_6cygrpc__ChannelArg, /*tp_dealloc*/
  0, /*tp_print*/
  0, /*tp_getattr*/
  0, /*tp_setattr*/
  #if PY_MAJOR_VERSION < 3
  0, /*tp_compare*/
  #endif
  #if PY_MAJOR_VERSION >= 3
  0, /*tp_as_async*/
  #endif
  0, /*tp_repr*/
  0, /*tp_as_number*/
  0, /*tp_as_sequence*/
  0, /*tp_as_mapping*/
  0, /*tp_hash*/
  0, /*tp_call*/
  0, /*tp_str*/
  0, /*tp_getattro*/
  0, /*tp_setattro*/
  0, /*tp_as_buffer*/
  Py_TPFLAGS_DEFAULT|Py_TPFLAGS_HAVE_VERSION_TAG|Py_TPFLAGS_CHECKTYPES|Py_TPFLAGS_HAVE_NEWBUFFER|Py_TPFLAGS_BASETYPE, /*tp_flags*/
  0, /*tp_doc*/
  0, /*tp_traverse*/
  0, /*tp_clear*/
  0, /*tp_richcompare*/
  0, /*tp_weaklistoffset*/
  0, /*tp_iter*/
  0, /*tp_iternext*/
  __pyx_methods_4grpc_7_cython_6cygrpc__ChannelArg, /*tp_methods*/
  0, /*tp_members*/
  0, /*tp_getset*/
  0, /*tp_base*/
  0, /*tp_dict*/
  0, /*tp_descr_get*/
  0, /*tp_descr_set*/
  0, /*tp_dictoffset*/
  0, /*tp_init*/
  0, /*tp_alloc*/
  __pyx_tp_new_4grpc_7_cython_6cygrpc__ChannelArg, /*tp_new*/
  0, /*tp_free*/
  0, /*tp_is_gc*/
  0, /*tp_bases*/
  0, /*tp_mro*/
  0, /*tp_cache*/
  0, /*tp_subclasses*/
  0, /*tp_weaklist*/
  0, /*tp_del*/
  0, /*tp_version_tag*/
  #if PY_VERSION_HEX >= 0x030400a1
  0, /*tp_finalize*/
  #endif
};
static struct __pyx_vtabstruct_4grpc_7_cython_6cygrpc__ChannelArgs __pyx_vtable_4grpc_7_cython_6cygrpc__ChannelArgs;

static PyObject *__pyx_tp_new_4grpc_7_cython_6cygrpc__ChannelArgs(PyTypeObject *t, PyObject *a, PyObject *k) {
  struct __pyx_obj_4grpc_7_cython_6cygrpc__ChannelArgs *p;
  PyObject *o;
  if (likely((t->tp_flags & Py_TPFLAGS_IS_ABSTRACT) == 0)) {
    o = (*t->tp_alloc)(t, 0);
  } else {
    o = (PyObject *) PyBaseObject_Type.tp_new(t, __pyx_empty_tuple, 0);
  }
  if (unlikely(!o)) return 0;
  p = ((struct __pyx_obj_4grpc_7_cython_6cygrpc__ChannelArgs *)o);
  p->__pyx_vtab = __pyx_vtabptr_4grpc_7_cython_6cygrpc__ChannelArgs;
  p->_arguments = ((PyObject*)Py_None); Py_INCREF(Py_None);
  p->_channel_args = ((PyObject*)Py_None); Py_INCREF(Py_None);
  p->_references = ((PyObject*)Py_None); Py_INCREF(Py_None);
  if (unlikely(__pyx_pw_4grpc_7_cython_6cygrpc_12_ChannelArgs_1__cinit__(o, a, k) < 0)) goto bad;
  return o;
  bad:
  Py_DECREF(o); o = 0;
  return NULL;
}

static void __pyx_tp_dealloc_4grpc_7_cython_6cygrpc__ChannelArgs(PyObject *o) {
  struct __pyx_obj_4grpc_7_cython_6cygrpc__ChannelArgs *p = (struct __pyx_obj_4grpc_7_cython_6cygrpc__ChannelArgs *)o;
  #if CYTHON_USE_TP_FINALIZE
  if (unlikely(PyType_HasFeature(Py_TYPE(o), Py_TPFLAGS_HAVE_FINALIZE) && Py_TYPE(o)->tp_finalize) && !_PyGC_FINALIZED(o)) {
    if (PyObject_CallFinalizerFromDealloc(o)) return;
  }
  #endif
  PyObject_GC_UnTrack(o);
  {
    PyObject *etype, *eval, *etb;
    PyErr_Fetch(&etype, &eval, &etb);
    ++Py_REFCNT(o);
    __pyx_pw_4grpc_7_cython_6cygrpc_12_ChannelArgs_3__dealloc__(o);
    --Py_REFCNT(o);
    PyErr_Restore(etype, eval, etb);
  }
  Py_CLEAR(p->_arguments);
  Py_CLEAR(p->_channel_args);
  Py_CLEAR(p->_references);
  (*Py_TYPE(o)->tp_free)(o);
}

static int __pyx_tp_traverse_4grpc_7_cython_6cygrpc__ChannelArgs(PyObject *o, visitproc v, void *a) {
  int e;
  struct __pyx_obj_4grpc_7_cython_6cygrpc__ChannelArgs *p = (struct __pyx_obj_4grpc_7_cython_6cygrpc__ChannelArgs *)o;
  if (p->_arguments) {
    e = (*v)(p->_arguments, a); if (e) return e;
  }
  if (p->_channel_args) {
    e = (*v)(p->_channel_args, a); if (e) return e;
  }
  if (p->_references) {
    e = (*v)(p->_references, a); if (e) return e;
  }
  return 0;
}

static int __pyx_tp_clear_4grpc_7_cython_6cygrpc__ChannelArgs(PyObject *o) {
  PyObject* tmp;
  struct __pyx_obj_4grpc_7_cython_6cygrpc__ChannelArgs *p = (struct __pyx_obj_4grpc_7_cython_6cygrpc__ChannelArgs *)o;
  tmp = ((PyObject*)p->_arguments);
  p->_arguments = ((PyObject*)Py_None); Py_INCREF(Py_None);
  Py_XDECREF(tmp);
  tmp = ((PyObject*)p->_channel_args);
  p->_channel_args = ((PyObject*)Py_None); Py_INCREF(Py_None);
  Py_XDECREF(tmp);
  tmp = ((PyObject*)p->_references);
  p->_references = ((PyObject*)Py_None); Py_INCREF(Py_None);
  Py_XDECREF(tmp);
  return 0;
}

static PyObject *__pyx_getprop_4grpc_7_cython_6cygrpc_12_ChannelArgs__arguments(PyObject *o, CYTHON_UNUSED void *x) {
  return __pyx_pw_4grpc_7_cython_6cygrpc_12_ChannelArgs_10_arguments_1__get__(o);
}

static PyObject *__pyx_getprop_4grpc_7_cython_6cygrpc_12_ChannelArgs__references(PyObject *o, CYTHON_UNUSED void *x) {
  return __pyx_pw_4grpc_7_cython_6cygrpc_12_ChannelArgs_11_references_1__get__(o);
}

static PyMethodDef __pyx_methods_4grpc_7_cython_6cygrpc__ChannelArgs[] = {
  {"__reduce_cython__", (PyCFunction)__pyx_pw_4grpc_7_cython_6cygrpc_12_ChannelArgs_5__reduce_cython__, METH_NOARGS, 0},
  {"__setstate_cython__", (PyCFunction)__pyx_pw_4grpc_7_cython_6cygrpc_12_ChannelArgs_7__setstate_cython__, METH_O, 0},
  {0, 0, 0, 0}
};

static struct PyGetSetDef __pyx_getsets_4grpc_7_cython_6cygrpc__ChannelArgs[] = {
  {(char *)"_arguments", __pyx_getprop_4grpc_7_cython_6cygrpc_12_ChannelArgs__arguments, 0, (char *)0, 0},
  {(char *)"_references", __pyx_getprop_4grpc_7_cython_6cygrpc_12_ChannelArgs__references, 0, (char *)0, 0},
  {0, 0, 0, 0, 0}
};

static PyTypeObject __pyx_type_4grpc_7_cython_6cygrpc__ChannelArgs = {
  PyVarObject_HEAD_INIT(0, 0)
  "grpc._cython.cygrpc._ChannelArgs", /*tp_name*/
  sizeof(struct __pyx_obj_4grpc_7_cython_6cygrpc__ChannelArgs), /*tp_basicsize*/
  0, /*tp_itemsize*/
  __pyx_tp_dealloc_4grpc_7_cython_6cygrpc__ChannelArgs, /*tp_dealloc*/
  0, /*tp_print*/
  0, /*tp_getattr*/
  0, /*tp_setattr*/
  #if PY_MAJOR_VERSION < 3
  0, /*tp_compare*/
  #endif
  #if PY_MAJOR_VERSION >= 3
  0, /*tp_as_async*/
  #endif
  0, /*tp_repr*/
  0, /*tp_as_number*/
  0, /*tp_as_sequence*/
  0, /*tp_as_mapping*/
  0, /*tp_hash*/
  0, /*tp_call*/
  0, /*tp_str*/
  0, /*tp_getattro*/
  0, /*tp_setattro*/
  0, /*tp_as_buffer*/
  Py_TPFLAGS_DEFAULT|Py_TPFLAGS_HAVE_VERSION_TAG|Py_TPFLAGS_CHECKTYPES|Py_TPFLAGS_HAVE_NEWBUFFER|Py_TPFLAGS_BASETYPE|Py_TPFLAGS_HAVE_GC, /*tp_flags*/
  0, /*tp_doc*/
  __pyx_tp_traverse_4grpc_7_cython_6cygrpc__ChannelArgs, /*tp_traverse*/
  __pyx_tp_clear_4grpc_7_cython_6cygrpc__ChannelArgs, /*tp_clear*/
  0, /*tp_richcompare*/
  0, /*tp_weaklistoffset*/
  0, /*tp_iter*/
  0, /*tp_iternext*/
  __pyx_methods_4grpc_7_cython_6cygrpc__ChannelArgs, /*tp_methods*/
  0, /*tp_members*/
  __pyx_getsets_4grpc_7_cython_6cygrpc__ChannelArgs, /*tp_getset*/
  0, /*tp_base*/
  0, /*tp_dict*/
  0, /*tp_descr_get*/
  0, /*tp_descr_set*/
  0, /*tp_dictoffset*/
  0, /*tp_init*/
  0, /*tp_alloc*/
  __pyx_tp_new_4grpc_7_cython_6cygrpc__ChannelArgs, /*tp_new*/
  0, /*tp_free*/
  0, /*tp_is_gc*/
  0, /*tp_bases*/
  0, /*tp_mro*/
  0, /*tp_cache*/
  0, /*tp_subclasses*/
  0, /*tp_weaklist*/
  0, /*tp_del*/
  0, /*tp_version_tag*/
  #if PY_VERSION_HEX >= 0x030400a1
  0, /*tp_finalize*/
  #endif
};

static PyObject *__pyx_tp_new_4grpc_7_cython_6cygrpc_Call(PyTypeObject *t, PyObject *a, PyObject *k) {
  struct __pyx_obj_4grpc_7_cython_6cygrpc_Call *p;
  PyObject *o;
  if (likely((t->tp_flags & Py_TPFLAGS_IS_ABSTRACT) == 0)) {
    o = (*t->tp_alloc)(t, 0);
  } else {
    o = (PyObject *) PyBaseObject_Type.tp_new(t, __pyx_empty_tuple, 0);
  }
  if (unlikely(!o)) return 0;
  p = ((struct __pyx_obj_4grpc_7_cython_6cygrpc_Call *)o);
  p->references = ((PyObject*)Py_None); Py_INCREF(Py_None);
  if (unlikely(__pyx_pw_4grpc_7_cython_6cygrpc_4Call_1__cinit__(o, a, k) < 0)) goto bad;
  return o;
  bad:
  Py_DECREF(o); o = 0;
  return NULL;
}

static void __pyx_tp_dealloc_4grpc_7_cython_6cygrpc_Call(PyObject *o) {
  struct __pyx_obj_4grpc_7_cython_6cygrpc_Call *p = (struct __pyx_obj_4grpc_7_cython_6cygrpc_Call *)o;
  #if CYTHON_USE_TP_FINALIZE
  if (unlikely(PyType_HasFeature(Py_TYPE(o), Py_TPFLAGS_HAVE_FINALIZE) && Py_TYPE(o)->tp_finalize) && !_PyGC_FINALIZED(o)) {
    if (PyObject_CallFinalizerFromDealloc(o)) return;
  }
  #endif
  PyObject_GC_UnTrack(o);
  {
    PyObject *etype, *eval, *etb;
    PyErr_Fetch(&etype, &eval, &etb);
    ++Py_REFCNT(o);
    __pyx_pw_4grpc_7_cython_6cygrpc_4Call_15__dealloc__(o);
    --Py_REFCNT(o);
    PyErr_Restore(etype, eval, etb);
  }
  Py_CLEAR(p->references);
  (*Py_TYPE(o)->tp_free)(o);
}

static int __pyx_tp_traverse_4grpc_7_cython_6cygrpc_Call(PyObject *o, visitproc v, void *a) {
  int e;
  struct __pyx_obj_4grpc_7_cython_6cygrpc_Call *p = (struct __pyx_obj_4grpc_7_cython_6cygrpc_Call *)o;
  if (p->references) {
    e = (*v)(p->references, a); if (e) return e;
  }
  return 0;
}

static int __pyx_tp_clear_4grpc_7_cython_6cygrpc_Call(PyObject *o) {
  PyObject* tmp;
  struct __pyx_obj_4grpc_7_cython_6cygrpc_Call *p = (struct __pyx_obj_4grpc_7_cython_6cygrpc_Call *)o;
  tmp = ((PyObject*)p->references);
  p->references = ((PyObject*)Py_None); Py_INCREF(Py_None);
  Py_XDECREF(tmp);
  return 0;
}

static PyObject *__pyx_getprop_4grpc_7_cython_6cygrpc_4Call_is_valid(PyObject *o, CYTHON_UNUSED void *x) {
  return __pyx_pw_4grpc_7_cython_6cygrpc_4Call_8is_valid_1__get__(o);
}

static PyMethodDef __pyx_methods_4grpc_7_cython_6cygrpc_Call[] = {
  {"_start_batch", (PyCFunction)__pyx_pw_4grpc_7_cython_6cygrpc_4Call_3_start_batch, METH_VARARGS|METH_KEYWORDS, 0},
  {"start_client_batch", (PyCFunction)__pyx_pw_4grpc_7_cython_6cygrpc_4Call_5start_client_batch, METH_VARARGS|METH_KEYWORDS, 0},
  {"start_server_batch", (PyCFunction)__pyx_pw_4grpc_7_cython_6cygrpc_4Call_7start_server_batch, METH_VARARGS|METH_KEYWORDS, 0},
  {"cancel", (PyCFunction)__pyx_pw_4grpc_7_cython_6cygrpc_4Call_9cancel, METH_VARARGS|METH_KEYWORDS, 0},
  {"set_credentials", (PyCFunction)__pyx_pw_4grpc_7_cython_6cygrpc_4Call_11set_credentials, METH_O, 0},
  {"peer", (PyCFunction)__pyx_pw_4grpc_7_cython_6cygrpc_4Call_13peer, METH_NOARGS, 0},
  {"_custom_op_on_c_call", (PyCFunction)__pyx_pw_4grpc_7_cython_6cygrpc_4Call_17_custom_op_on_c_call, METH_O, 0},
  {"__reduce_cython__", (PyCFunction)__pyx_pw_4grpc_7_cython_6cygrpc_4Call_19__reduce_cython__, METH_NOARGS, 0},
  {"__setstate_cython__", (PyCFunction)__pyx_pw_4grpc_7_cython_6cygrpc_4Call_21__setstate_cython__, METH_O, 0},
  {0, 0, 0, 0}
};

static struct PyGetSetDef __pyx_getsets_4grpc_7_cython_6cygrpc_Call[] = {
  {(char *)"is_valid", __pyx_getprop_4grpc_7_cython_6cygrpc_4Call_is_valid, 0, (char *)0, 0},
  {0, 0, 0, 0, 0}
};

static PyTypeObject __pyx_type_4grpc_7_cython_6cygrpc_Call = {
  PyVarObject_HEAD_INIT(0, 0)
  "grpc._cython.cygrpc.Call", /*tp_name*/
  sizeof(struct __pyx_obj_4grpc_7_cython_6cygrpc_Call), /*tp_basicsize*/
  0, /*tp_itemsize*/
  __pyx_tp_dealloc_4grpc_7_cython_6cygrpc_Call, /*tp_dealloc*/
  0, /*tp_print*/
  0, /*tp_getattr*/
  0, /*tp_setattr*/
  #if PY_MAJOR_VERSION < 3
  0, /*tp_compare*/
  #endif
  #if PY_MAJOR_VERSION >= 3
  0, /*tp_as_async*/
  #endif
  0, /*tp_repr*/
  0, /*tp_as_number*/
  0, /*tp_as_sequence*/
  0, /*tp_as_mapping*/
  0, /*tp_hash*/
  0, /*tp_call*/
  0, /*tp_str*/
  0, /*tp_getattro*/
  0, /*tp_setattro*/
  0, /*tp_as_buffer*/
  Py_TPFLAGS_DEFAULT|Py_TPFLAGS_HAVE_VERSION_TAG|Py_TPFLAGS_CHECKTYPES|Py_TPFLAGS_HAVE_NEWBUFFER|Py_TPFLAGS_BASETYPE|Py_TPFLAGS_HAVE_GC, /*tp_flags*/
  0, /*tp_doc*/
  __pyx_tp_traverse_4grpc_7_cython_6cygrpc_Call, /*tp_traverse*/
  __pyx_tp_clear_4grpc_7_cython_6cygrpc_Call, /*tp_clear*/
  0, /*tp_richcompare*/
  0, /*tp_weaklistoffset*/
  0, /*tp_iter*/
  0, /*tp_iternext*/
  __pyx_methods_4grpc_7_cython_6cygrpc_Call, /*tp_methods*/
  0, /*tp_members*/
  __pyx_getsets_4grpc_7_cython_6cygrpc_Call, /*tp_getset*/
  0, /*tp_base*/
  0, /*tp_dict*/
  0, /*tp_descr_get*/
  0, /*tp_descr_set*/
  0, /*tp_dictoffset*/
  0, /*tp_init*/
  0, /*tp_alloc*/
  __pyx_tp_new_4grpc_7_cython_6cygrpc_Call, /*tp_new*/
  0, /*tp_free*/
  0, /*tp_is_gc*/
  0, /*tp_bases*/
  0, /*tp_mro*/
  0, /*tp_cache*/
  0, /*tp_subclasses*/
  0, /*tp_weaklist*/
  0, /*tp_del*/
  0, /*tp_version_tag*/
  #if PY_VERSION_HEX >= 0x030400a1
  0, /*tp_finalize*/
  #endif
};

static PyObject *__pyx_tp_new_4grpc_7_cython_6cygrpc__CallState(PyTypeObject *t, CYTHON_UNUSED PyObject *a, CYTHON_UNUSED PyObject *k) {
  struct __pyx_obj_4grpc_7_cython_6cygrpc__CallState *p;
  PyObject *o;
  if (likely((t->tp_flags & Py_TPFLAGS_IS_ABSTRACT) == 0)) {
    o = (*t->tp_alloc)(t, 0);
  } else {
    o = (PyObject *) PyBaseObject_Type.tp_new(t, __pyx_empty_tuple, 0);
  }
  if (unlikely(!o)) return 0;
  p = ((struct __pyx_obj_4grpc_7_cython_6cygrpc__CallState *)o);
  p->due = ((PyObject*)Py_None); Py_INCREF(Py_None);
  if (unlikely(__pyx_pw_4grpc_7_cython_6cygrpc_10_CallState_1__cinit__(o, __pyx_empty_tuple, NULL) < 0)) goto bad;
  return o;
  bad:
  Py_DECREF(o); o = 0;
  return NULL;
}

static void __pyx_tp_dealloc_4grpc_7_cython_6cygrpc__CallState(PyObject *o) {
  struct __pyx_obj_4grpc_7_cython_6cygrpc__CallState *p = (struct __pyx_obj_4grpc_7_cython_6cygrpc__CallState *)o;
  #if CYTHON_USE_TP_FINALIZE
  if (unlikely(PyType_HasFeature(Py_TYPE(o), Py_TPFLAGS_HAVE_FINALIZE) && Py_TYPE(o)->tp_finalize) && !_PyGC_FINALIZED(o)) {
    if (PyObject_CallFinalizerFromDealloc(o)) return;
  }
  #endif
  PyObject_GC_UnTrack(o);
  Py_CLEAR(p->due);
  (*Py_TYPE(o)->tp_free)(o);
}

static int __pyx_tp_traverse_4grpc_7_cython_6cygrpc__CallState(PyObject *o, visitproc v, void *a) {
  int e;
  struct __pyx_obj_4grpc_7_cython_6cygrpc__CallState *p = (struct __pyx_obj_4grpc_7_cython_6cygrpc__CallState *)o;
  if (p->due) {
    e = (*v)(p->due, a); if (e) return e;
  }
  return 0;
}

static int __pyx_tp_clear_4grpc_7_cython_6cygrpc__CallState(PyObject *o) {
  PyObject* tmp;
  struct __pyx_obj_4grpc_7_cython_6cygrpc__CallState *p = (struct __pyx_obj_4grpc_7_cython_6cygrpc__CallState *)o;
  tmp = ((PyObject*)p->due);
  p->due = ((PyObject*)Py_None); Py_INCREF(Py_None);
  Py_XDECREF(tmp);
  return 0;
}

static PyMethodDef __pyx_methods_4grpc_7_cython_6cygrpc__CallState[] = {
  {"__reduce_cython__", (PyCFunction)__pyx_pw_4grpc_7_cython_6cygrpc_10_CallState_3__reduce_cython__, METH_NOARGS, 0},
  {"__setstate_cython__", (PyCFunction)__pyx_pw_4grpc_7_cython_6cygrpc_10_CallState_5__setstate_cython__, METH_O, 0},
  {0, 0, 0, 0}
};

static PyTypeObject __pyx_type_4grpc_7_cython_6cygrpc__CallState = {
  PyVarObject_HEAD_INIT(0, 0)
  "grpc._cython.cygrpc._CallState", /*tp_name*/
  sizeof(struct __pyx_obj_4grpc_7_cython_6cygrpc__CallState), /*tp_basicsize*/
  0, /*tp_itemsize*/
  __pyx_tp_dealloc_4grpc_7_cython_6cygrpc__CallState, /*tp_dealloc*/
  0, /*tp_print*/
  0, /*tp_getattr*/
  0, /*tp_setattr*/
  #if PY_MAJOR_VERSION < 3
  0, /*tp_compare*/
  #endif
  #if PY_MAJOR_VERSION >= 3
  0, /*tp_as_async*/
  #endif
  0, /*tp_repr*/
  0, /*tp_as_number*/
  0, /*tp_as_sequence*/
  0, /*tp_as_mapping*/
  0, /*tp_hash*/
  0, /*tp_call*/
  0, /*tp_str*/
  0, /*tp_getattro*/
  0, /*tp_setattro*/
  0, /*tp_as_buffer*/
  Py_TPFLAGS_DEFAULT|Py_TPFLAGS_HAVE_VERSION_TAG|Py_TPFLAGS_CHECKTYPES|Py_TPFLAGS_HAVE_NEWBUFFER|Py_TPFLAGS_BASETYPE|Py_TPFLAGS_HAVE_GC, /*tp_flags*/
  0, /*tp_doc*/
  __pyx_tp_traverse_4grpc_7_cython_6cygrpc__CallState, /*tp_traverse*/
  __pyx_tp_clear_4grpc_7_cython_6cygrpc__CallState, /*tp_clear*/
  0, /*tp_richcompare*/
  0, /*tp_weaklistoffset*/
  0, /*tp_iter*/
  0, /*tp_iternext*/
  __pyx_methods_4grpc_7_cython_6cygrpc__CallState, /*tp_methods*/
  0, /*tp_members*/
  0, /*tp_getset*/
  0, /*tp_base*/
  0, /*tp_dict*/
  0, /*tp_descr_get*/
  0, /*tp_descr_set*/
  0, /*tp_dictoffset*/
  0, /*tp_init*/
  0, /*tp_alloc*/
  __pyx_tp_new_4grpc_7_cython_6cygrpc__CallState, /*tp_new*/
  0, /*tp_free*/
  0, /*tp_is_gc*/
  0, /*tp_bases*/
  0, /*tp_mro*/
  0, /*tp_cache*/
  0, /*tp_subclasses*/
  0, /*tp_weaklist*/
  0, /*tp_del*/
  0, /*tp_version_tag*/
  #if PY_VERSION_HEX >= 0x030400a1
  0, /*tp_finalize*/
  #endif
};

static PyObject *__pyx_tp_new_4grpc_7_cython_6cygrpc__ChannelState(PyTypeObject *t, CYTHON_UNUSED PyObject *a, CYTHON_UNUSED PyObject *k) {
  struct __pyx_obj_4grpc_7_cython_6cygrpc__ChannelState *p;
  PyObject *o;
  if (likely((t->tp_flags & Py_TPFLAGS_IS_ABSTRACT) == 0)) {
    o = (*t->tp_alloc)(t, 0);
  } else {
    o = (PyObject *) PyBaseObject_Type.tp_new(t, __pyx_empty_tuple, 0);
  }
  if (unlikely(!o)) return 0;
  p = ((struct __pyx_obj_4grpc_7_cython_6cygrpc__ChannelState *)o);
  p->condition = Py_None; Py_INCREF(Py_None);
  p->open = Py_None; Py_INCREF(Py_None);
  p->closed_reason = Py_None; Py_INCREF(Py_None);
  p->integrated_call_states = ((PyObject*)Py_None); Py_INCREF(Py_None);
  p->segregated_call_states = ((PyObject*)Py_None); Py_INCREF(Py_None);
  p->connectivity_due = ((PyObject*)Py_None); Py_INCREF(Py_None);
  if (unlikely(__pyx_pw_4grpc_7_cython_6cygrpc_13_ChannelState_1__cinit__(o, __pyx_empty_tuple, NULL) < 0)) goto bad;
  return o;
  bad:
  Py_DECREF(o); o = 0;
  return NULL;
}

static void __pyx_tp_dealloc_4grpc_7_cython_6cygrpc__ChannelState(PyObject *o) {
  struct __pyx_obj_4grpc_7_cython_6cygrpc__ChannelState *p = (struct __pyx_obj_4grpc_7_cython_6cygrpc__ChannelState *)o;
  #if CYTHON_USE_TP_FINALIZE
  if (unlikely(PyType_HasFeature(Py_TYPE(o), Py_TPFLAGS_HAVE_FINALIZE) && Py_TYPE(o)->tp_finalize) && !_PyGC_FINALIZED(o)) {
    if (PyObject_CallFinalizerFromDealloc(o)) return;
  }
  #endif
  PyObject_GC_UnTrack(o);
  Py_CLEAR(p->condition);
  Py_CLEAR(p->open);
  Py_CLEAR(p->closed_reason);
  Py_CLEAR(p->integrated_call_states);
  Py_CLEAR(p->segregated_call_states);
  Py_CLEAR(p->connectivity_due);
  (*Py_TYPE(o)->tp_free)(o);
}

static int __pyx_tp_traverse_4grpc_7_cython_6cygrpc__ChannelState(PyObject *o, visitproc v, void *a) {
  int e;
  struct __pyx_obj_4grpc_7_cython_6cygrpc__ChannelState *p = (struct __pyx_obj_4grpc_7_cython_6cygrpc__ChannelState *)o;
  if (p->condition) {
    e = (*v)(p->condition, a); if (e) return e;
  }
  if (p->open) {
    e = (*v)(p->open, a); if (e) return e;
  }
  if (p->closed_reason) {
    e = (*v)(p->closed_reason, a); if (e) return e;
  }
  if (p->integrated_call_states) {
    e = (*v)(p->integrated_call_states, a); if (e) return e;
  }
  if (p->segregated_call_states) {
    e = (*v)(p->segregated_call_states, a); if (e) return e;
  }
  if (p->connectivity_due) {
    e = (*v)(p->connectivity_due, a); if (e) return e;
  }
  return 0;
}

static int __pyx_tp_clear_4grpc_7_cython_6cygrpc__ChannelState(PyObject *o) {
  PyObject* tmp;
  struct __pyx_obj_4grpc_7_cython_6cygrpc__ChannelState *p = (struct __pyx_obj_4grpc_7_cython_6cygrpc__ChannelState *)o;
  tmp = ((PyObject*)p->condition);
  p->condition = Py_None; Py_INCREF(Py_None);
  Py_XDECREF(tmp);
  tmp = ((PyObject*)p->open);
  p->open = Py_None; Py_INCREF(Py_None);
  Py_XDECREF(tmp);
  tmp = ((PyObject*)p->closed_reason);
  p->closed_reason = Py_None; Py_INCREF(Py_None);
  Py_XDECREF(tmp);
  tmp = ((PyObject*)p->integrated_call_states);
  p->integrated_call_states = ((PyObject*)Py_None); Py_INCREF(Py_None);
  Py_XDECREF(tmp);
  tmp = ((PyObject*)p->segregated_call_states);
  p->segregated_call_states = ((PyObject*)Py_None); Py_INCREF(Py_None);
  Py_XDECREF(tmp);
  tmp = ((PyObject*)p->connectivity_due);
  p->connectivity_due = ((PyObject*)Py_None); Py_INCREF(Py_None);
  Py_XDECREF(tmp);
  return 0;
}

static PyMethodDef __pyx_methods_4grpc_7_cython_6cygrpc__ChannelState[] = {
  {"__reduce_cython__", (PyCFunction)__pyx_pw_4grpc_7_cython_6cygrpc_13_ChannelState_3__reduce_cython__, METH_NOARGS, 0},
  {"__setstate_cython__", (PyCFunction)__pyx_pw_4grpc_7_cython_6cygrpc_13_ChannelState_5__setstate_cython__, METH_O, 0},
  {0, 0, 0, 0}
};

static PyTypeObject __pyx_type_4grpc_7_cython_6cygrpc__ChannelState = {
  PyVarObject_HEAD_INIT(0, 0)
  "grpc._cython.cygrpc._ChannelState", /*tp_name*/
  sizeof(struct __pyx_obj_4grpc_7_cython_6cygrpc__ChannelState), /*tp_basicsize*/
  0, /*tp_itemsize*/
  __pyx_tp_dealloc_4grpc_7_cython_6cygrpc__ChannelState, /*tp_dealloc*/
  0, /*tp_print*/
  0, /*tp_getattr*/
  0, /*tp_setattr*/
  #if PY_MAJOR_VERSION < 3
  0, /*tp_compare*/
  #endif
  #if PY_MAJOR_VERSION >= 3
  0, /*tp_as_async*/
  #endif
  0, /*tp_repr*/
  0, /*tp_as_number*/
  0, /*tp_as_sequence*/
  0, /*tp_as_mapping*/
  0, /*tp_hash*/
  0, /*tp_call*/
  0, /*tp_str*/
  0, /*tp_getattro*/
  0, /*tp_setattro*/
  0, /*tp_as_buffer*/
  Py_TPFLAGS_DEFAULT|Py_TPFLAGS_HAVE_VERSION_TAG|Py_TPFLAGS_CHECKTYPES|Py_TPFLAGS_HAVE_NEWBUFFER|Py_TPFLAGS_BASETYPE|Py_TPFLAGS_HAVE_GC, /*tp_flags*/
  0, /*tp_doc*/
  __pyx_tp_traverse_4grpc_7_cython_6cygrpc__ChannelState, /*tp_traverse*/
  __pyx_tp_clear_4grpc_7_cython_6cygrpc__ChannelState, /*tp_clear*/
  0, /*tp_richcompare*/
  0, /*tp_weaklistoffset*/
  0, /*tp_iter*/
  0, /*tp_iternext*/
  __pyx_methods_4grpc_7_cython_6cygrpc__ChannelState, /*tp_methods*/
  0, /*tp_members*/
  0, /*tp_getset*/
  0, /*tp_base*/
  0, /*tp_dict*/
  0, /*tp_descr_get*/
  0, /*tp_descr_set*/
  0, /*tp_dictoffset*/
  0, /*tp_init*/
  0, /*tp_alloc*/
  __pyx_tp_new_4grpc_7_cython_6cygrpc__ChannelState, /*tp_new*/
  0, /*tp_free*/
  0, /*tp_is_gc*/
  0, /*tp_bases*/
  0, /*tp_mro*/
  0, /*tp_cache*/
  0, /*tp_subclasses*/
  0, /*tp_weaklist*/
  0, /*tp_del*/
  0, /*tp_version_tag*/
  #if PY_VERSION_HEX >= 0x030400a1
  0, /*tp_finalize*/
  #endif
};

static PyObject *__pyx_tp_new_4grpc_7_cython_6cygrpc_IntegratedCall(PyTypeObject *t, PyObject *a, PyObject *k) {
  struct __pyx_obj_4grpc_7_cython_6cygrpc_IntegratedCall *p;
  PyObject *o;
  if (likely((t->tp_flags & Py_TPFLAGS_IS_ABSTRACT) == 0)) {
    o = (*t->tp_alloc)(t, 0);
  } else {
    o = (PyObject *) PyBaseObject_Type.tp_new(t, __pyx_empty_tuple, 0);
  }
  if (unlikely(!o)) return 0;
  p = ((struct __pyx_obj_4grpc_7_cython_6cygrpc_IntegratedCall *)o);
  p->_channel_state = ((struct __pyx_obj_4grpc_7_cython_6cygrpc__ChannelState *)Py_None); Py_INCREF(Py_None);
  p->_call_state = ((struct __pyx_obj_4grpc_7_cython_6cygrpc__CallState *)Py_None); Py_INCREF(Py_None);
  if (unlikely(__pyx_pw_4grpc_7_cython_6cygrpc_14IntegratedCall_1__cinit__(o, a, k) < 0)) goto bad;
  return o;
  bad:
  Py_DECREF(o); o = 0;
  return NULL;
}

static void __pyx_tp_dealloc_4grpc_7_cython_6cygrpc_IntegratedCall(PyObject *o) {
  struct __pyx_obj_4grpc_7_cython_6cygrpc_IntegratedCall *p = (struct __pyx_obj_4grpc_7_cython_6cygrpc_IntegratedCall *)o;
  #if CYTHON_USE_TP_FINALIZE
  if (unlikely(PyType_HasFeature(Py_TYPE(o), Py_TPFLAGS_HAVE_FINALIZE) && Py_TYPE(o)->tp_finalize) && !_PyGC_FINALIZED(o)) {
    if (PyObject_CallFinalizerFromDealloc(o)) return;
  }
  #endif
  PyObject_GC_UnTrack(o);
  Py_CLEAR(p->_channel_state);
  Py_CLEAR(p->_call_state);
  (*Py_TYPE(o)->tp_free)(o);
}

static int __pyx_tp_traverse_4grpc_7_cython_6cygrpc_IntegratedCall(PyObject *o, visitproc v, void *a) {
  int e;
  struct __pyx_obj_4grpc_7_cython_6cygrpc_IntegratedCall *p = (struct __pyx_obj_4grpc_7_cython_6cygrpc_IntegratedCall *)o;
  if (p->_channel_state) {
    e = (*v)(((PyObject *)p->_channel_state), a); if (e) return e;
  }
  if (p->_call_state) {
    e = (*v)(((PyObject *)p->_call_state), a); if (e) return e;
  }
  return 0;
}

static int __pyx_tp_clear_4grpc_7_cython_6cygrpc_IntegratedCall(PyObject *o) {
  PyObject* tmp;
  struct __pyx_obj_4grpc_7_cython_6cygrpc_IntegratedCall *p = (struct __pyx_obj_4grpc_7_cython_6cygrpc_IntegratedCall *)o;
  tmp = ((PyObject*)p->_channel_state);
  p->_channel_state = ((struct __pyx_obj_4grpc_7_cython_6cygrpc__ChannelState *)Py_None); Py_INCREF(Py_None);
  Py_XDECREF(tmp);
  tmp = ((PyObject*)p->_call_state);
  p->_call_state = ((struct __pyx_obj_4grpc_7_cython_6cygrpc__CallState *)Py_None); Py_INCREF(Py_None);
  Py_XDECREF(tmp);
  return 0;
}

static PyMethodDef __pyx_methods_4grpc_7_cython_6cygrpc_IntegratedCall[] = {
  {"operate", (PyCFunction)__pyx_pw_4grpc_7_cython_6cygrpc_14IntegratedCall_3operate, METH_VARARGS|METH_KEYWORDS, 0},
  {"cancel", (PyCFunction)__pyx_pw_4grpc_7_cython_6cygrpc_14IntegratedCall_5cancel, METH_VARARGS|METH_KEYWORDS, 0},
  {"__reduce_cython__", (PyCFunction)__pyx_pw_4grpc_7_cython_6cygrpc_14IntegratedCall_7__reduce_cython__, METH_NOARGS, 0},
  {"__setstate_cython__", (PyCFunction)__pyx_pw_4grpc_7_cython_6cygrpc_14IntegratedCall_9__setstate_cython__, METH_O, 0},
  {0, 0, 0, 0}
};

static PyTypeObject __pyx_type_4grpc_7_cython_6cygrpc_IntegratedCall = {
  PyVarObject_HEAD_INIT(0, 0)
  "grpc._cython.cygrpc.IntegratedCall", /*tp_name*/
  sizeof(struct __pyx_obj_4grpc_7_cython_6cygrpc_IntegratedCall), /*tp_basicsize*/
  0, /*tp_itemsize*/
  __pyx_tp_dealloc_4grpc_7_cython_6cygrpc_IntegratedCall, /*tp_dealloc*/
  0, /*tp_print*/
  0, /*tp_getattr*/
  0, /*tp_setattr*/
  #if PY_MAJOR_VERSION < 3
  0, /*tp_compare*/
  #endif
  #if PY_MAJOR_VERSION >= 3
  0, /*tp_as_async*/
  #endif
  0, /*tp_repr*/
  0, /*tp_as_number*/
  0, /*tp_as_sequence*/
  0, /*tp_as_mapping*/
  0, /*tp_hash*/
  0, /*tp_call*/
  0, /*tp_str*/
  0, /*tp_getattro*/
  0, /*tp_setattro*/
  0, /*tp_as_buffer*/
  Py_TPFLAGS_DEFAULT|Py_TPFLAGS_HAVE_VERSION_TAG|Py_TPFLAGS_CHECKTYPES|Py_TPFLAGS_HAVE_NEWBUFFER|Py_TPFLAGS_BASETYPE|Py_TPFLAGS_HAVE_GC, /*tp_flags*/
  0, /*tp_doc*/
  __pyx_tp_traverse_4grpc_7_cython_6cygrpc_IntegratedCall, /*tp_traverse*/
  __pyx_tp_clear_4grpc_7_cython_6cygrpc_IntegratedCall, /*tp_clear*/
  0, /*tp_richcompare*/
  0, /*tp_weaklistoffset*/
  0, /*tp_iter*/
  0, /*tp_iternext*/
  __pyx_methods_4grpc_7_cython_6cygrpc_IntegratedCall, /*tp_methods*/
  0, /*tp_members*/
  0, /*tp_getset*/
  0, /*tp_base*/
  0, /*tp_dict*/
  0, /*tp_descr_get*/
  0, /*tp_descr_set*/
  0, /*tp_dictoffset*/
  0, /*tp_init*/
  0, /*tp_alloc*/
  __pyx_tp_new_4grpc_7_cython_6cygrpc_IntegratedCall, /*tp_new*/
  0, /*tp_free*/
  0, /*tp_is_gc*/
  0, /*tp_bases*/
  0, /*tp_mro*/
  0, /*tp_cache*/
  0, /*tp_subclasses*/
  0, /*tp_weaklist*/
  0, /*tp_del*/
  0, /*tp_version_tag*/
  #if PY_VERSION_HEX >= 0x030400a1
  0, /*tp_finalize*/
  #endif
};

static PyObject *__pyx_tp_new_4grpc_7_cython_6cygrpc_SegregatedCall(PyTypeObject *t, PyObject *a, PyObject *k) {
  struct __pyx_obj_4grpc_7_cython_6cygrpc_SegregatedCall *p;
  PyObject *o;
  if (likely((t->tp_flags & Py_TPFLAGS_IS_ABSTRACT) == 0)) {
    o = (*t->tp_alloc)(t, 0);
  } else {
    o = (PyObject *) PyBaseObject_Type.tp_new(t, __pyx_empty_tuple, 0);
  }
  if (unlikely(!o)) return 0;
  p = ((struct __pyx_obj_4grpc_7_cython_6cygrpc_SegregatedCall *)o);
  p->_channel_state = ((struct __pyx_obj_4grpc_7_cython_6cygrpc__ChannelState *)Py_None); Py_INCREF(Py_None);
  p->_call_state = ((struct __pyx_obj_4grpc_7_cython_6cygrpc__CallState *)Py_None); Py_INCREF(Py_None);
  if (unlikely(__pyx_pw_4grpc_7_cython_6cygrpc_14SegregatedCall_1__cinit__(o, a, k) < 0)) goto bad;
  return o;
  bad:
  Py_DECREF(o); o = 0;
  return NULL;
}

static void __pyx_tp_dealloc_4grpc_7_cython_6cygrpc_SegregatedCall(PyObject *o) {
  struct __pyx_obj_4grpc_7_cython_6cygrpc_SegregatedCall *p = (struct __pyx_obj_4grpc_7_cython_6cygrpc_SegregatedCall *)o;
  #if CYTHON_USE_TP_FINALIZE
  if (unlikely(PyType_HasFeature(Py_TYPE(o), Py_TPFLAGS_HAVE_FINALIZE) && Py_TYPE(o)->tp_finalize) && !_PyGC_FINALIZED(o)) {
    if (PyObject_CallFinalizerFromDealloc(o)) return;
  }
  #endif
  PyObject_GC_UnTrack(o);
  Py_CLEAR(p->_channel_state);
  Py_CLEAR(p->_call_state);
  (*Py_TYPE(o)->tp_free)(o);
}

static int __pyx_tp_traverse_4grpc_7_cython_6cygrpc_SegregatedCall(PyObject *o, visitproc v, void *a) {
  int e;
  struct __pyx_obj_4grpc_7_cython_6cygrpc_SegregatedCall *p = (struct __pyx_obj_4grpc_7_cython_6cygrpc_SegregatedCall *)o;
  if (p->_channel_state) {
    e = (*v)(((PyObject *)p->_channel_state), a); if (e) return e;
  }
  if (p->_call_state) {
    e = (*v)(((PyObject *)p->_call_state), a); if (e) return e;
  }
  return 0;
}

static int __pyx_tp_clear_4grpc_7_cython_6cygrpc_SegregatedCall(PyObject *o) {
  PyObject* tmp;
  struct __pyx_obj_4grpc_7_cython_6cygrpc_SegregatedCall *p = (struct __pyx_obj_4grpc_7_cython_6cygrpc_SegregatedCall *)o;
  tmp = ((PyObject*)p->_channel_state);
  p->_channel_state = ((struct __pyx_obj_4grpc_7_cython_6cygrpc__ChannelState *)Py_None); Py_INCREF(Py_None);
  Py_XDECREF(tmp);
  tmp = ((PyObject*)p->_call_state);
  p->_call_state = ((struct __pyx_obj_4grpc_7_cython_6cygrpc__CallState *)Py_None); Py_INCREF(Py_None);
  Py_XDECREF(tmp);
  return 0;
}

static PyMethodDef __pyx_methods_4grpc_7_cython_6cygrpc_SegregatedCall[] = {
  {"operate", (PyCFunction)__pyx_pw_4grpc_7_cython_6cygrpc_14SegregatedCall_3operate, METH_VARARGS|METH_KEYWORDS, 0},
  {"cancel", (PyCFunction)__pyx_pw_4grpc_7_cython_6cygrpc_14SegregatedCall_5cancel, METH_VARARGS|METH_KEYWORDS, 0},
  {"next_event", (PyCFunction)__pyx_pw_4grpc_7_cython_6cygrpc_14SegregatedCall_7next_event, METH_NOARGS, 0},
  {"__reduce_cython__", (PyCFunction)__pyx_pw_4grpc_7_cython_6cygrpc_14SegregatedCall_9__reduce_cython__, METH_NOARGS, 0},
  {"__setstate_cython__", (PyCFunction)__pyx_pw_4grpc_7_cython_6cygrpc_14SegregatedCall_11__setstate_cython__, METH_O, 0},
  {0, 0, 0, 0}
};

static PyTypeObject __pyx_type_4grpc_7_cython_6cygrpc_SegregatedCall = {
  PyVarObject_HEAD_INIT(0, 0)
  "grpc._cython.cygrpc.SegregatedCall", /*tp_name*/
  sizeof(struct __pyx_obj_4grpc_7_cython_6cygrpc_SegregatedCall), /*tp_basicsize*/
  0, /*tp_itemsize*/
  __pyx_tp_dealloc_4grpc_7_cython_6cygrpc_SegregatedCall, /*tp_dealloc*/
  0, /*tp_print*/
  0, /*tp_getattr*/
  0, /*tp_setattr*/
  #if PY_MAJOR_VERSION < 3
  0, /*tp_compare*/
  #endif
  #if PY_MAJOR_VERSION >= 3
  0, /*tp_as_async*/
  #endif
  0, /*tp_repr*/
  0, /*tp_as_number*/
  0, /*tp_as_sequence*/
  0, /*tp_as_mapping*/
  0, /*tp_hash*/
  0, /*tp_call*/
  0, /*tp_str*/
  0, /*tp_getattro*/
  0, /*tp_setattro*/
  0, /*tp_as_buffer*/
  Py_TPFLAGS_DEFAULT|Py_TPFLAGS_HAVE_VERSION_TAG|Py_TPFLAGS_CHECKTYPES|Py_TPFLAGS_HAVE_NEWBUFFER|Py_TPFLAGS_BASETYPE|Py_TPFLAGS_HAVE_GC, /*tp_flags*/
  0, /*tp_doc*/
  __pyx_tp_traverse_4grpc_7_cython_6cygrpc_SegregatedCall, /*tp_traverse*/
  __pyx_tp_clear_4grpc_7_cython_6cygrpc_SegregatedCall, /*tp_clear*/
  0, /*tp_richcompare*/
  0, /*tp_weaklistoffset*/
  0, /*tp_iter*/
  0, /*tp_iternext*/
  __pyx_methods_4grpc_7_cython_6cygrpc_SegregatedCall, /*tp_methods*/
  0, /*tp_members*/
  0, /*tp_getset*/
  0, /*tp_base*/
  0, /*tp_dict*/
  0, /*tp_descr_get*/
  0, /*tp_descr_set*/
  0, /*tp_dictoffset*/
  0, /*tp_init*/
  0, /*tp_alloc*/
  __pyx_tp_new_4grpc_7_cython_6cygrpc_SegregatedCall, /*tp_new*/
  0, /*tp_free*/
  0, /*tp_is_gc*/
  0, /*tp_bases*/
  0, /*tp_mro*/
  0, /*tp_cache*/
  0, /*tp_subclasses*/
  0, /*tp_weaklist*/
  0, /*tp_del*/
  0, /*tp_version_tag*/
  #if PY_VERSION_HEX >= 0x030400a1
  0, /*tp_finalize*/
  #endif
};

static PyObject *__pyx_tp_new_4grpc_7_cython_6cygrpc_Channel(PyTypeObject *t, PyObject *a, PyObject *k) {
  struct __pyx_obj_4grpc_7_cython_6cygrpc_Channel *p;
  PyObject *o;
  if (likely((t->tp_flags & Py_TPFLAGS_IS_ABSTRACT) == 0)) {
    o = (*t->tp_alloc)(t, 0);
  } else {
    o = (PyObject *) PyBaseObject_Type.tp_new(t, __pyx_empty_tuple, 0);
  }
  if (unlikely(!o)) return 0;
  p = ((struct __pyx_obj_4grpc_7_cython_6cygrpc_Channel *)o);
  p->_state = ((struct __pyx_obj_4grpc_7_cython_6cygrpc__ChannelState *)Py_None); Py_INCREF(Py_None);
  p->_vtable = ((struct __pyx_obj_4grpc_7_cython_6cygrpc__VTable *)Py_None); Py_INCREF(Py_None);
  p->_arguments = ((PyObject*)Py_None); Py_INCREF(Py_None);
  if (unlikely(__pyx_pw_4grpc_7_cython_6cygrpc_7Channel_1__cinit__(o, a, k) < 0)) goto bad;
  return o;
  bad:
  Py_DECREF(o); o = 0;
  return NULL;
}

static void __pyx_tp_dealloc_4grpc_7_cython_6cygrpc_Channel(PyObject *o) {
  struct __pyx_obj_4grpc_7_cython_6cygrpc_Channel *p = (struct __pyx_obj_4grpc_7_cython_6cygrpc_Channel *)o;
  #if CYTHON_USE_TP_FINALIZE
  if (unlikely(PyType_HasFeature(Py_TYPE(o), Py_TPFLAGS_HAVE_FINALIZE) && Py_TYPE(o)->tp_finalize) && !_PyGC_FINALIZED(o)) {
    if (PyObject_CallFinalizerFromDealloc(o)) return;
  }
  #endif
  PyObject_GC_UnTrack(o);
  Py_CLEAR(p->_state);
  Py_CLEAR(p->_vtable);
  Py_CLEAR(p->_arguments);
  (*Py_TYPE(o)->tp_free)(o);
}

static int __pyx_tp_traverse_4grpc_7_cython_6cygrpc_Channel(PyObject *o, visitproc v, void *a) {
  int e;
  struct __pyx_obj_4grpc_7_cython_6cygrpc_Channel *p = (struct __pyx_obj_4grpc_7_cython_6cygrpc_Channel *)o;
  if (p->_state) {
    e = (*v)(((PyObject *)p->_state), a); if (e) return e;
  }
  if (p->_vtable) {
    e = (*v)(((PyObject *)p->_vtable), a); if (e) return e;
  }
  if (p->_arguments) {
    e = (*v)(p->_arguments, a); if (e) return e;
  }
  return 0;
}

static int __pyx_tp_clear_4grpc_7_cython_6cygrpc_Channel(PyObject *o) {
  PyObject* tmp;
  struct __pyx_obj_4grpc_7_cython_6cygrpc_Channel *p = (struct __pyx_obj_4grpc_7_cython_6cygrpc_Channel *)o;
  tmp = ((PyObject*)p->_state);
  p->_state = ((struct __pyx_obj_4grpc_7_cython_6cygrpc__ChannelState *)Py_None); Py_INCREF(Py_None);
  Py_XDECREF(tmp);
  tmp = ((PyObject*)p->_vtable);
  p->_vtable = ((struct __pyx_obj_4grpc_7_cython_6cygrpc__VTable *)Py_None); Py_INCREF(Py_None);
  Py_XDECREF(tmp);
  tmp = ((PyObject*)p->_arguments);
  p->_arguments = ((PyObject*)Py_None); Py_INCREF(Py_None);
  Py_XDECREF(tmp);
  return 0;
}

static PyMethodDef __pyx_methods_4grpc_7_cython_6cygrpc_Channel[] = {
  {"target", (PyCFunction)__pyx_pw_4grpc_7_cython_6cygrpc_7Channel_3target, METH_NOARGS, 0},
  {"integrated_call", (PyCFunction)__pyx_pw_4grpc_7_cython_6cygrpc_7Channel_5integrated_call, METH_VARARGS|METH_KEYWORDS, 0},
  {"next_call_event", (PyCFunction)__pyx_pw_4grpc_7_cython_6cygrpc_7Channel_7next_call_event, METH_NOARGS, 0},
  {"segregated_call", (PyCFunction)__pyx_pw_4grpc_7_cython_6cygrpc_7Channel_9segregated_call, METH_VARARGS|METH_KEYWORDS, 0},
  {"check_connectivity_state", (PyCFunction)__pyx_pw_4grpc_7_cython_6cygrpc_7Channel_11check_connectivity_state, METH_O, 0},
  {"watch_connectivity_state", (PyCFunction)__pyx_pw_4grpc_7_cython_6cygrpc_7Channel_13watch_connectivity_state, METH_VARARGS|METH_KEYWORDS, 0},
  {"close", (PyCFunction)__pyx_pw_4grpc_7_cython_6cygrpc_7Channel_15close, METH_VARARGS|METH_KEYWORDS, 0},
  {"close_on_fork", (PyCFunction)__pyx_pw_4grpc_7_cython_6cygrpc_7Channel_17close_on_fork, METH_VARARGS|METH_KEYWORDS, 0},
  {"__reduce_cython__", (PyCFunction)__pyx_pw_4grpc_7_cython_6cygrpc_7Channel_19__reduce_cython__, METH_NOARGS, 0},
  {"__setstate_cython__", (PyCFunction)__pyx_pw_4grpc_7_cython_6cygrpc_7Channel_21__setstate_cython__, METH_O, 0},
  {0, 0, 0, 0}
};

static PyTypeObject __pyx_type_4grpc_7_cython_6cygrpc_Channel = {
  PyVarObject_HEAD_INIT(0, 0)
  "grpc._cython.cygrpc.Channel", /*tp_name*/
  sizeof(struct __pyx_obj_4grpc_7_cython_6cygrpc_Channel), /*tp_basicsize*/
  0, /*tp_itemsize*/
  __pyx_tp_dealloc_4grpc_7_cython_6cygrpc_Channel, /*tp_dealloc*/
  0, /*tp_print*/
  0, /*tp_getattr*/
  0, /*tp_setattr*/
  #if PY_MAJOR_VERSION < 3
  0, /*tp_compare*/
  #endif
  #if PY_MAJOR_VERSION >= 3
  0, /*tp_as_async*/
  #endif
  0, /*tp_repr*/
  0, /*tp_as_number*/
  0, /*tp_as_sequence*/
  0, /*tp_as_mapping*/
  0, /*tp_hash*/
  0, /*tp_call*/
  0, /*tp_str*/
  0, /*tp_getattro*/
  0, /*tp_setattro*/
  0, /*tp_as_buffer*/
  Py_TPFLAGS_DEFAULT|Py_TPFLAGS_HAVE_VERSION_TAG|Py_TPFLAGS_CHECKTYPES|Py_TPFLAGS_HAVE_NEWBUFFER|Py_TPFLAGS_BASETYPE|Py_TPFLAGS_HAVE_GC, /*tp_flags*/
  0, /*tp_doc*/
  __pyx_tp_traverse_4grpc_7_cython_6cygrpc_Channel, /*tp_traverse*/
  __pyx_tp_clear_4grpc_7_cython_6cygrpc_Channel, /*tp_clear*/
  0, /*tp_richcompare*/
  0, /*tp_weaklistoffset*/
  0, /*tp_iter*/
  0, /*tp_iternext*/
  __pyx_methods_4grpc_7_cython_6cygrpc_Channel, /*tp_methods*/
  0, /*tp_members*/
  0, /*tp_getset*/
  0, /*tp_base*/
  0, /*tp_dict*/
  0, /*tp_descr_get*/
  0, /*tp_descr_set*/
  0, /*tp_dictoffset*/
  0, /*tp_init*/
  0, /*tp_alloc*/
  __pyx_tp_new_4grpc_7_cython_6cygrpc_Channel, /*tp_new*/
  0, /*tp_free*/
  0, /*tp_is_gc*/
  0, /*tp_bases*/
  0, /*tp_mro*/
  0, /*tp_cache*/
  0, /*tp_subclasses*/
  0, /*tp_weaklist*/
  0, /*tp_del*/
  0, /*tp_version_tag*/
  #if PY_VERSION_HEX >= 0x030400a1
  0, /*tp_finalize*/
  #endif
};
static struct __pyx_vtabstruct_4grpc_7_cython_6cygrpc_CallCredentials __pyx_vtable_4grpc_7_cython_6cygrpc_CallCredentials;

static PyObject *__pyx_tp_new_4grpc_7_cython_6cygrpc_CallCredentials(PyTypeObject *t, CYTHON_UNUSED PyObject *a, CYTHON_UNUSED PyObject *k) {
  struct __pyx_obj_4grpc_7_cython_6cygrpc_CallCredentials *p;
  PyObject *o;
  if (likely((t->tp_flags & Py_TPFLAGS_IS_ABSTRACT) == 0)) {
    o = (*t->tp_alloc)(t, 0);
  } else {
    o = (PyObject *) PyBaseObject_Type.tp_new(t, __pyx_empty_tuple, 0);
  }
  if (unlikely(!o)) return 0;
  p = ((struct __pyx_obj_4grpc_7_cython_6cygrpc_CallCredentials *)o);
  p->__pyx_vtab = __pyx_vtabptr_4grpc_7_cython_6cygrpc_CallCredentials;
  return o;
}

static void __pyx_tp_dealloc_4grpc_7_cython_6cygrpc_CallCredentials(PyObject *o) {
  #if CYTHON_USE_TP_FINALIZE
  if (unlikely(PyType_HasFeature(Py_TYPE(o), Py_TPFLAGS_HAVE_FINALIZE) && Py_TYPE(o)->tp_finalize) && (!PyType_IS_GC(Py_TYPE(o)) || !_PyGC_FINALIZED(o))) {
    if (PyObject_CallFinalizerFromDealloc(o)) return;
  }
  #endif
  (*Py_TYPE(o)->tp_free)(o);
}

static PyMethodDef __pyx_methods_4grpc_7_cython_6cygrpc_CallCredentials[] = {
  {"__reduce_cython__", (PyCFunction)__pyx_pw_4grpc_7_cython_6cygrpc_15CallCredentials_1__reduce_cython__, METH_NOARGS, 0},
  {"__setstate_cython__", (PyCFunction)__pyx_pw_4grpc_7_cython_6cygrpc_15CallCredentials_3__setstate_cython__, METH_O, 0},
  {0, 0, 0, 0}
};

static PyTypeObject __pyx_type_4grpc_7_cython_6cygrpc_CallCredentials = {
  PyVarObject_HEAD_INIT(0, 0)
  "grpc._cython.cygrpc.CallCredentials", /*tp_name*/
  sizeof(struct __pyx_obj_4grpc_7_cython_6cygrpc_CallCredentials), /*tp_basicsize*/
  0, /*tp_itemsize*/
  __pyx_tp_dealloc_4grpc_7_cython_6cygrpc_CallCredentials, /*tp_dealloc*/
  0, /*tp_print*/
  0, /*tp_getattr*/
  0, /*tp_setattr*/
  #if PY_MAJOR_VERSION < 3
  0, /*tp_compare*/
  #endif
  #if PY_MAJOR_VERSION >= 3
  0, /*tp_as_async*/
  #endif
  0, /*tp_repr*/
  0, /*tp_as_number*/
  0, /*tp_as_sequence*/
  0, /*tp_as_mapping*/
  0, /*tp_hash*/
  0, /*tp_call*/
  0, /*tp_str*/
  0, /*tp_getattro*/
  0, /*tp_setattro*/
  0, /*tp_as_buffer*/
  Py_TPFLAGS_DEFAULT|Py_TPFLAGS_HAVE_VERSION_TAG|Py_TPFLAGS_CHECKTYPES|Py_TPFLAGS_HAVE_NEWBUFFER|Py_TPFLAGS_BASETYPE, /*tp_flags*/
  0, /*tp_doc*/
  0, /*tp_traverse*/
  0, /*tp_clear*/
  0, /*tp_richcompare*/
  0, /*tp_weaklistoffset*/
  0, /*tp_iter*/
  0, /*tp_iternext*/
  __pyx_methods_4grpc_7_cython_6cygrpc_CallCredentials, /*tp_methods*/
  0, /*tp_members*/
  0, /*tp_getset*/
  0, /*tp_base*/
  0, /*tp_dict*/
  0, /*tp_descr_get*/
  0, /*tp_descr_set*/
  0, /*tp_dictoffset*/
  0, /*tp_init*/
  0, /*tp_alloc*/
  __pyx_tp_new_4grpc_7_cython_6cygrpc_CallCredentials, /*tp_new*/
  0, /*tp_free*/
  0, /*tp_is_gc*/
  0, /*tp_bases*/
  0, /*tp_mro*/
  0, /*tp_cache*/
  0, /*tp_subclasses*/
  0, /*tp_weaklist*/
  0, /*tp_del*/
  0, /*tp_version_tag*/
  #if PY_VERSION_HEX >= 0x030400a1
  0, /*tp_finalize*/
  #endif
};
static struct __pyx_vtabstruct_4grpc_7_cython_6cygrpc_MetadataPluginCallCredentials __pyx_vtable_4grpc_7_cython_6cygrpc_MetadataPluginCallCredentials;

static PyObject *__pyx_tp_new_4grpc_7_cython_6cygrpc_MetadataPluginCallCredentials(PyTypeObject *t, PyObject *a, PyObject *k) {
  struct __pyx_obj_4grpc_7_cython_6cygrpc_MetadataPluginCallCredentials *p;
  PyObject *o = __pyx_tp_new_4grpc_7_cython_6cygrpc_CallCredentials(t, a, k);
  if (unlikely(!o)) return 0;
  p = ((struct __pyx_obj_4grpc_7_cython_6cygrpc_MetadataPluginCallCredentials *)o);
  p->__pyx_base.__pyx_vtab = (struct __pyx_vtabstruct_4grpc_7_cython_6cygrpc_CallCredentials*)__pyx_vtabptr_4grpc_7_cython_6cygrpc_MetadataPluginCallCredentials;
  p->_metadata_plugin = Py_None; Py_INCREF(Py_None);
  p->_name = ((PyObject*)Py_None); Py_INCREF(Py_None);
  if (unlikely(__pyx_pw_4grpc_7_cython_6cygrpc_29MetadataPluginCallCredentials_1__cinit__(o, a, k) < 0)) goto bad;
  return o;
  bad:
  Py_DECREF(o); o = 0;
  return NULL;
}

static void __pyx_tp_dealloc_4grpc_7_cython_6cygrpc_MetadataPluginCallCredentials(PyObject *o) {
  struct __pyx_obj_4grpc_7_cython_6cygrpc_MetadataPluginCallCredentials *p = (struct __pyx_obj_4grpc_7_cython_6cygrpc_MetadataPluginCallCredentials *)o;
  #if CYTHON_USE_TP_FINALIZE
  if (unlikely(PyType_HasFeature(Py_TYPE(o), Py_TPFLAGS_HAVE_FINALIZE) && Py_TYPE(o)->tp_finalize) && !_PyGC_FINALIZED(o)) {
    if (PyObject_CallFinalizerFromDealloc(o)) return;
  }
  #endif
  PyObject_GC_UnTrack(o);
  Py_CLEAR(p->_metadata_plugin);
  Py_CLEAR(p->_name);
  #if CYTHON_USE_TYPE_SLOTS
  if (PyType_IS_GC(Py_TYPE(o)->tp_base))
  #endif
  PyObject_GC_Track(o);
  __pyx_tp_dealloc_4grpc_7_cython_6cygrpc_CallCredentials(o);
}

static int __pyx_tp_traverse_4grpc_7_cython_6cygrpc_MetadataPluginCallCredentials(PyObject *o, visitproc v, void *a) {
  int e;
  struct __pyx_obj_4grpc_7_cython_6cygrpc_MetadataPluginCallCredentials *p = (struct __pyx_obj_4grpc_7_cython_6cygrpc_MetadataPluginCallCredentials *)o;
  e = ((likely(__pyx_ptype_4grpc_7_cython_6cygrpc_CallCredentials)) ? ((__pyx_ptype_4grpc_7_cython_6cygrpc_CallCredentials->tp_traverse) ? __pyx_ptype_4grpc_7_cython_6cygrpc_CallCredentials->tp_traverse(o, v, a) : 0) : __Pyx_call_next_tp_traverse(o, v, a, __pyx_tp_traverse_4grpc_7_cython_6cygrpc_MetadataPluginCallCredentials)); if (e) return e;
  if (p->_metadata_plugin) {
    e = (*v)(p->_metadata_plugin, a); if (e) return e;
  }
  return 0;
}

static int __pyx_tp_clear_4grpc_7_cython_6cygrpc_MetadataPluginCallCredentials(PyObject *o) {
  PyObject* tmp;
  struct __pyx_obj_4grpc_7_cython_6cygrpc_MetadataPluginCallCredentials *p = (struct __pyx_obj_4grpc_7_cython_6cygrpc_MetadataPluginCallCredentials *)o;
  if (likely(__pyx_ptype_4grpc_7_cython_6cygrpc_CallCredentials)) { if (__pyx_ptype_4grpc_7_cython_6cygrpc_CallCredentials->tp_clear) __pyx_ptype_4grpc_7_cython_6cygrpc_CallCredentials->tp_clear(o); } else __Pyx_call_next_tp_clear(o, __pyx_tp_clear_4grpc_7_cython_6cygrpc_MetadataPluginCallCredentials);
  tmp = ((PyObject*)p->_metadata_plugin);
  p->_metadata_plugin = Py_None; Py_INCREF(Py_None);
  Py_XDECREF(tmp);
  return 0;
}

static PyObject *__pyx_getprop_4grpc_7_cython_6cygrpc_29MetadataPluginCallCredentials__metadata_plugin(PyObject *o, CYTHON_UNUSED void *x) {
  return __pyx_pw_4grpc_7_cython_6cygrpc_29MetadataPluginCallCredentials_16_metadata_plugin_1__get__(o);
}

static PyObject *__pyx_getprop_4grpc_7_cython_6cygrpc_29MetadataPluginCallCredentials__name(PyObject *o, CYTHON_UNUSED void *x) {
  return __pyx_pw_4grpc_7_cython_6cygrpc_29MetadataPluginCallCredentials_5_name_1__get__(o);
}

static PyMethodDef __pyx_methods_4grpc_7_cython_6cygrpc_MetadataPluginCallCredentials[] = {
  {"__reduce_cython__", (PyCFunction)__pyx_pw_4grpc_7_cython_6cygrpc_29MetadataPluginCallCredentials_3__reduce_cython__, METH_NOARGS, 0},
  {"__setstate_cython__", (PyCFunction)__pyx_pw_4grpc_7_cython_6cygrpc_29MetadataPluginCallCredentials_5__setstate_cython__, METH_O, 0},
  {0, 0, 0, 0}
};

static struct PyGetSetDef __pyx_getsets_4grpc_7_cython_6cygrpc_MetadataPluginCallCredentials[] = {
  {(char *)"_metadata_plugin", __pyx_getprop_4grpc_7_cython_6cygrpc_29MetadataPluginCallCredentials__metadata_plugin, 0, (char *)0, 0},
  {(char *)"_name", __pyx_getprop_4grpc_7_cython_6cygrpc_29MetadataPluginCallCredentials__name, 0, (char *)0, 0},
  {0, 0, 0, 0, 0}
};

static PyTypeObject __pyx_type_4grpc_7_cython_6cygrpc_MetadataPluginCallCredentials = {
  PyVarObject_HEAD_INIT(0, 0)
  "grpc._cython.cygrpc.MetadataPluginCallCredentials", /*tp_name*/
  sizeof(struct __pyx_obj_4grpc_7_cython_6cygrpc_MetadataPluginCallCredentials), /*tp_basicsize*/
  0, /*tp_itemsize*/
  __pyx_tp_dealloc_4grpc_7_cython_6cygrpc_MetadataPluginCallCredentials, /*tp_dealloc*/
  0, /*tp_print*/
  0, /*tp_getattr*/
  0, /*tp_setattr*/
  #if PY_MAJOR_VERSION < 3
  0, /*tp_compare*/
  #endif
  #if PY_MAJOR_VERSION >= 3
  0, /*tp_as_async*/
  #endif
  0, /*tp_repr*/
  0, /*tp_as_number*/
  0, /*tp_as_sequence*/
  0, /*tp_as_mapping*/
  0, /*tp_hash*/
  0, /*tp_call*/
  0, /*tp_str*/
  0, /*tp_getattro*/
  0, /*tp_setattro*/
  0, /*tp_as_buffer*/
  Py_TPFLAGS_DEFAULT|Py_TPFLAGS_HAVE_VERSION_TAG|Py_TPFLAGS_CHECKTYPES|Py_TPFLAGS_HAVE_NEWBUFFER|Py_TPFLAGS_BASETYPE|Py_TPFLAGS_HAVE_GC, /*tp_flags*/
  0, /*tp_doc*/
  __pyx_tp_traverse_4grpc_7_cython_6cygrpc_MetadataPluginCallCredentials, /*tp_traverse*/
  __pyx_tp_clear_4grpc_7_cython_6cygrpc_MetadataPluginCallCredentials, /*tp_clear*/
  0, /*tp_richcompare*/
  0, /*tp_weaklistoffset*/
  0, /*tp_iter*/
  0, /*tp_iternext*/
  __pyx_methods_4grpc_7_cython_6cygrpc_MetadataPluginCallCredentials, /*tp_methods*/
  0, /*tp_members*/
  __pyx_getsets_4grpc_7_cython_6cygrpc_MetadataPluginCallCredentials, /*tp_getset*/
  0, /*tp_base*/
  0, /*tp_dict*/
  0, /*tp_descr_get*/
  0, /*tp_descr_set*/
  0, /*tp_dictoffset*/
  0, /*tp_init*/
  0, /*tp_alloc*/
  __pyx_tp_new_4grpc_7_cython_6cygrpc_MetadataPluginCallCredentials, /*tp_new*/
  0, /*tp_free*/
  0, /*tp_is_gc*/
  0, /*tp_bases*/
  0, /*tp_mro*/
  0, /*tp_cache*/
  0, /*tp_subclasses*/
  0, /*tp_weaklist*/
  0, /*tp_del*/
  0, /*tp_version_tag*/
  #if PY_VERSION_HEX >= 0x030400a1
  0, /*tp_finalize*/
  #endif
};
static struct __pyx_vtabstruct_4grpc_7_cython_6cygrpc_CompositeCallCredentials __pyx_vtable_4grpc_7_cython_6cygrpc_CompositeCallCredentials;

static PyObject *__pyx_tp_new_4grpc_7_cython_6cygrpc_CompositeCallCredentials(PyTypeObject *t, PyObject *a, PyObject *k) {
  struct __pyx_obj_4grpc_7_cython_6cygrpc_CompositeCallCredentials *p;
  PyObject *o = __pyx_tp_new_4grpc_7_cython_6cygrpc_CallCredentials(t, a, k);
  if (unlikely(!o)) return 0;
  p = ((struct __pyx_obj_4grpc_7_cython_6cygrpc_CompositeCallCredentials *)o);
  p->__pyx_base.__pyx_vtab = (struct __pyx_vtabstruct_4grpc_7_cython_6cygrpc_CallCredentials*)__pyx_vtabptr_4grpc_7_cython_6cygrpc_CompositeCallCredentials;
  p->_call_credentialses = ((PyObject*)Py_None); Py_INCREF(Py_None);
  if (unlikely(__pyx_pw_4grpc_7_cython_6cygrpc_24CompositeCallCredentials_1__cinit__(o, a, k) < 0)) goto bad;
  return o;
  bad:
  Py_DECREF(o); o = 0;
  return NULL;
}

static void __pyx_tp_dealloc_4grpc_7_cython_6cygrpc_CompositeCallCredentials(PyObject *o) {
  struct __pyx_obj_4grpc_7_cython_6cygrpc_CompositeCallCredentials *p = (struct __pyx_obj_4grpc_7_cython_6cygrpc_CompositeCallCredentials *)o;
  #if CYTHON_USE_TP_FINALIZE
  if (unlikely(PyType_HasFeature(Py_TYPE(o), Py_TPFLAGS_HAVE_FINALIZE) && Py_TYPE(o)->tp_finalize) && !_PyGC_FINALIZED(o)) {
    if (PyObject_CallFinalizerFromDealloc(o)) return;
  }
  #endif
  PyObject_GC_UnTrack(o);
  Py_CLEAR(p->_call_credentialses);
  #if CYTHON_USE_TYPE_SLOTS
  if (PyType_IS_GC(Py_TYPE(o)->tp_base))
  #endif
  PyObject_GC_Track(o);
  __pyx_tp_dealloc_4grpc_7_cython_6cygrpc_CallCredentials(o);
}

static int __pyx_tp_traverse_4grpc_7_cython_6cygrpc_CompositeCallCredentials(PyObject *o, visitproc v, void *a) {
  int e;
  struct __pyx_obj_4grpc_7_cython_6cygrpc_CompositeCallCredentials *p = (struct __pyx_obj_4grpc_7_cython_6cygrpc_CompositeCallCredentials *)o;
  e = ((likely(__pyx_ptype_4grpc_7_cython_6cygrpc_CallCredentials)) ? ((__pyx_ptype_4grpc_7_cython_6cygrpc_CallCredentials->tp_traverse) ? __pyx_ptype_4grpc_7_cython_6cygrpc_CallCredentials->tp_traverse(o, v, a) : 0) : __Pyx_call_next_tp_traverse(o, v, a, __pyx_tp_traverse_4grpc_7_cython_6cygrpc_CompositeCallCredentials)); if (e) return e;
  if (p->_call_credentialses) {
    e = (*v)(p->_call_credentialses, a); if (e) return e;
  }
  return 0;
}

static int __pyx_tp_clear_4grpc_7_cython_6cygrpc_CompositeCallCredentials(PyObject *o) {
  PyObject* tmp;
  struct __pyx_obj_4grpc_7_cython_6cygrpc_CompositeCallCredentials *p = (struct __pyx_obj_4grpc_7_cython_6cygrpc_CompositeCallCredentials *)o;
  if (likely(__pyx_ptype_4grpc_7_cython_6cygrpc_CallCredentials)) { if (__pyx_ptype_4grpc_7_cython_6cygrpc_CallCredentials->tp_clear) __pyx_ptype_4grpc_7_cython_6cygrpc_CallCredentials->tp_clear(o); } else __Pyx_call_next_tp_clear(o, __pyx_tp_clear_4grpc_7_cython_6cygrpc_CompositeCallCredentials);
  tmp = ((PyObject*)p->_call_credentialses);
  p->_call_credentialses = ((PyObject*)Py_None); Py_INCREF(Py_None);
  Py_XDECREF(tmp);
  return 0;
}

static PyObject *__pyx_getprop_4grpc_7_cython_6cygrpc_24CompositeCallCredentials__call_credentialses(PyObject *o, CYTHON_UNUSED void *x) {
  return __pyx_pw_4grpc_7_cython_6cygrpc_24CompositeCallCredentials_19_call_credentialses_1__get__(o);
}

static PyMethodDef __pyx_methods_4grpc_7_cython_6cygrpc_CompositeCallCredentials[] = {
  {"__reduce_cython__", (PyCFunction)__pyx_pw_4grpc_7_cython_6cygrpc_24CompositeCallCredentials_3__reduce_cython__, METH_NOARGS, 0},
  {"__setstate_cython__", (PyCFunction)__pyx_pw_4grpc_7_cython_6cygrpc_24CompositeCallCredentials_5__setstate_cython__, METH_O, 0},
  {0, 0, 0, 0}
};

static struct PyGetSetDef __pyx_getsets_4grpc_7_cython_6cygrpc_CompositeCallCredentials[] = {
  {(char *)"_call_credentialses", __pyx_getprop_4grpc_7_cython_6cygrpc_24CompositeCallCredentials__call_credentialses, 0, (char *)0, 0},
  {0, 0, 0, 0, 0}
};

static PyTypeObject __pyx_type_4grpc_7_cython_6cygrpc_CompositeCallCredentials = {
  PyVarObject_HEAD_INIT(0, 0)
  "grpc._cython.cygrpc.CompositeCallCredentials", /*tp_name*/
  sizeof(struct __pyx_obj_4grpc_7_cython_6cygrpc_CompositeCallCredentials), /*tp_basicsize*/
  0, /*tp_itemsize*/
  __pyx_tp_dealloc_4grpc_7_cython_6cygrpc_CompositeCallCredentials, /*tp_dealloc*/
  0, /*tp_print*/
  0, /*tp_getattr*/
  0, /*tp_setattr*/
  #if PY_MAJOR_VERSION < 3
  0, /*tp_compare*/
  #endif
  #if PY_MAJOR_VERSION >= 3
  0, /*tp_as_async*/
  #endif
  0, /*tp_repr*/
  0, /*tp_as_number*/
  0, /*tp_as_sequence*/
  0, /*tp_as_mapping*/
  0, /*tp_hash*/
  0, /*tp_call*/
  0, /*tp_str*/
  0, /*tp_getattro*/
  0, /*tp_setattro*/
  0, /*tp_as_buffer*/
  Py_TPFLAGS_DEFAULT|Py_TPFLAGS_HAVE_VERSION_TAG|Py_TPFLAGS_CHECKTYPES|Py_TPFLAGS_HAVE_NEWBUFFER|Py_TPFLAGS_BASETYPE|Py_TPFLAGS_HAVE_GC, /*tp_flags*/
  0, /*tp_doc*/
  __pyx_tp_traverse_4grpc_7_cython_6cygrpc_CompositeCallCredentials, /*tp_traverse*/
  __pyx_tp_clear_4grpc_7_cython_6cygrpc_CompositeCallCredentials, /*tp_clear*/
  0, /*tp_richcompare*/
  0, /*tp_weaklistoffset*/
  0, /*tp_iter*/
  0, /*tp_iternext*/
  __pyx_methods_4grpc_7_cython_6cygrpc_CompositeCallCredentials, /*tp_methods*/
  0, /*tp_members*/
  __pyx_getsets_4grpc_7_cython_6cygrpc_CompositeCallCredentials, /*tp_getset*/
  0, /*tp_base*/
  0, /*tp_dict*/
  0, /*tp_descr_get*/
  0, /*tp_descr_set*/
  0, /*tp_dictoffset*/
  0, /*tp_init*/
  0, /*tp_alloc*/
  __pyx_tp_new_4grpc_7_cython_6cygrpc_CompositeCallCredentials, /*tp_new*/
  0, /*tp_free*/
  0, /*tp_is_gc*/
  0, /*tp_bases*/
  0, /*tp_mro*/
  0, /*tp_cache*/
  0, /*tp_subclasses*/
  0, /*tp_weaklist*/
  0, /*tp_del*/
  0, /*tp_version_tag*/
  #if PY_VERSION_HEX >= 0x030400a1
  0, /*tp_finalize*/
  #endif
};
static struct __pyx_vtabstruct_4grpc_7_cython_6cygrpc_ChannelCredentials __pyx_vtable_4grpc_7_cython_6cygrpc_ChannelCredentials;

static PyObject *__pyx_tp_new_4grpc_7_cython_6cygrpc_ChannelCredentials(PyTypeObject *t, CYTHON_UNUSED PyObject *a, CYTHON_UNUSED PyObject *k) {
  struct __pyx_obj_4grpc_7_cython_6cygrpc_ChannelCredentials *p;
  PyObject *o;
  if (likely((t->tp_flags & Py_TPFLAGS_IS_ABSTRACT) == 0)) {
    o = (*t->tp_alloc)(t, 0);
  } else {
    o = (PyObject *) PyBaseObject_Type.tp_new(t, __pyx_empty_tuple, 0);
  }
  if (unlikely(!o)) return 0;
  p = ((struct __pyx_obj_4grpc_7_cython_6cygrpc_ChannelCredentials *)o);
  p->__pyx_vtab = __pyx_vtabptr_4grpc_7_cython_6cygrpc_ChannelCredentials;
  return o;
}

static void __pyx_tp_dealloc_4grpc_7_cython_6cygrpc_ChannelCredentials(PyObject *o) {
  #if CYTHON_USE_TP_FINALIZE
  if (unlikely(PyType_HasFeature(Py_TYPE(o), Py_TPFLAGS_HAVE_FINALIZE) && Py_TYPE(o)->tp_finalize) && (!PyType_IS_GC(Py_TYPE(o)) || !_PyGC_FINALIZED(o))) {
    if (PyObject_CallFinalizerFromDealloc(o)) return;
  }
  #endif
  (*Py_TYPE(o)->tp_free)(o);
}

static PyMethodDef __pyx_methods_4grpc_7_cython_6cygrpc_ChannelCredentials[] = {
  {"__reduce_cython__", (PyCFunction)__pyx_pw_4grpc_7_cython_6cygrpc_18ChannelCredentials_1__reduce_cython__, METH_NOARGS, 0},
  {"__setstate_cython__", (PyCFunction)__pyx_pw_4grpc_7_cython_6cygrpc_18ChannelCredentials_3__setstate_cython__, METH_O, 0},
  {0, 0, 0, 0}
};

static PyTypeObject __pyx_type_4grpc_7_cython_6cygrpc_ChannelCredentials = {
  PyVarObject_HEAD_INIT(0, 0)
  "grpc._cython.cygrpc.ChannelCredentials", /*tp_name*/
  sizeof(struct __pyx_obj_4grpc_7_cython_6cygrpc_ChannelCredentials), /*tp_basicsize*/
  0, /*tp_itemsize*/
  __pyx_tp_dealloc_4grpc_7_cython_6cygrpc_ChannelCredentials, /*tp_dealloc*/
  0, /*tp_print*/
  0, /*tp_getattr*/
  0, /*tp_setattr*/
  #if PY_MAJOR_VERSION < 3
  0, /*tp_compare*/
  #endif
  #if PY_MAJOR_VERSION >= 3
  0, /*tp_as_async*/
  #endif
  0, /*tp_repr*/
  0, /*tp_as_number*/
  0, /*tp_as_sequence*/
  0, /*tp_as_mapping*/
  0, /*tp_hash*/
  0, /*tp_call*/
  0, /*tp_str*/
  0, /*tp_getattro*/
  0, /*tp_setattro*/
  0, /*tp_as_buffer*/
  Py_TPFLAGS_DEFAULT|Py_TPFLAGS_HAVE_VERSION_TAG|Py_TPFLAGS_CHECKTYPES|Py_TPFLAGS_HAVE_NEWBUFFER|Py_TPFLAGS_BASETYPE, /*tp_flags*/
  0, /*tp_doc*/
  0, /*tp_traverse*/
  0, /*tp_clear*/
  0, /*tp_richcompare*/
  0, /*tp_weaklistoffset*/
  0, /*tp_iter*/
  0, /*tp_iternext*/
  __pyx_methods_4grpc_7_cython_6cygrpc_ChannelCredentials, /*tp_methods*/
  0, /*tp_members*/
  0, /*tp_getset*/
  0, /*tp_base*/
  0, /*tp_dict*/
  0, /*tp_descr_get*/
  0, /*tp_descr_set*/
  0, /*tp_dictoffset*/
  0, /*tp_init*/
  0, /*tp_alloc*/
  __pyx_tp_new_4grpc_7_cython_6cygrpc_ChannelCredentials, /*tp_new*/
  0, /*tp_free*/
  0, /*tp_is_gc*/
  0, /*tp_bases*/
  0, /*tp_mro*/
  0, /*tp_cache*/
  0, /*tp_subclasses*/
  0, /*tp_weaklist*/
  0, /*tp_del*/
  0, /*tp_version_tag*/
  #if PY_VERSION_HEX >= 0x030400a1
  0, /*tp_finalize*/
  #endif
};

static PyObject *__pyx_tp_new_4grpc_7_cython_6cygrpc_SSLSessionCacheLRU(PyTypeObject *t, PyObject *a, PyObject *k) {
  PyObject *o;
  if (likely((t->tp_flags & Py_TPFLAGS_IS_ABSTRACT) == 0)) {
    o = (*t->tp_alloc)(t, 0);
  } else {
    o = (PyObject *) PyBaseObject_Type.tp_new(t, __pyx_empty_tuple, 0);
  }
  if (unlikely(!o)) return 0;
  if (unlikely(__pyx_pw_4grpc_7_cython_6cygrpc_18SSLSessionCacheLRU_1__cinit__(o, a, k) < 0)) goto bad;
  return o;
  bad:
  Py_DECREF(o); o = 0;
  return NULL;
}

static void __pyx_tp_dealloc_4grpc_7_cython_6cygrpc_SSLSessionCacheLRU(PyObject *o) {
  #if CYTHON_USE_TP_FINALIZE
  if (unlikely(PyType_HasFeature(Py_TYPE(o), Py_TPFLAGS_HAVE_FINALIZE) && Py_TYPE(o)->tp_finalize) && (!PyType_IS_GC(Py_TYPE(o)) || !_PyGC_FINALIZED(o))) {
    if (PyObject_CallFinalizerFromDealloc(o)) return;
  }
  #endif
  {
    PyObject *etype, *eval, *etb;
    PyErr_Fetch(&etype, &eval, &etb);
    ++Py_REFCNT(o);
    __pyx_pw_4grpc_7_cython_6cygrpc_18SSLSessionCacheLRU_5__dealloc__(o);
    --Py_REFCNT(o);
    PyErr_Restore(etype, eval, etb);
  }
  (*Py_TYPE(o)->tp_free)(o);
}

static PyMethodDef __pyx_methods_4grpc_7_cython_6cygrpc_SSLSessionCacheLRU[] = {
  {"__reduce_cython__", (PyCFunction)__pyx_pw_4grpc_7_cython_6cygrpc_18SSLSessionCacheLRU_7__reduce_cython__, METH_NOARGS, 0},
  {"__setstate_cython__", (PyCFunction)__pyx_pw_4grpc_7_cython_6cygrpc_18SSLSessionCacheLRU_9__setstate_cython__, METH_O, 0},
  {0, 0, 0, 0}
};

static PyNumberMethods __pyx_tp_as_number_SSLSessionCacheLRU = {
  0, /*nb_add*/
  0, /*nb_subtract*/
  0, /*nb_multiply*/
  #if PY_MAJOR_VERSION < 3 || (CYTHON_COMPILING_IN_PYPY && PY_VERSION_HEX < 0x03050000)
  0, /*nb_divide*/
  #endif
  0, /*nb_remainder*/
  0, /*nb_divmod*/
  0, /*nb_power*/
  0, /*nb_negative*/
  0, /*nb_positive*/
  0, /*nb_absolute*/
  0, /*nb_nonzero*/
  0, /*nb_invert*/
  0, /*nb_lshift*/
  0, /*nb_rshift*/
  0, /*nb_and*/
  0, /*nb_xor*/
  0, /*nb_or*/
  #if PY_MAJOR_VERSION < 3 || (CYTHON_COMPILING_IN_PYPY && PY_VERSION_HEX < 0x03050000)
  0, /*nb_coerce*/
  #endif
  __pyx_pw_4grpc_7_cython_6cygrpc_18SSLSessionCacheLRU_3__int__, /*nb_int*/
  #if PY_MAJOR_VERSION < 3
  __pyx_pw_4grpc_7_cython_6cygrpc_18SSLSessionCacheLRU_3__int__, /*nb_long*/
  #else
  0, /*reserved*/
  #endif
  0, /*nb_float*/
  #if PY_MAJOR_VERSION < 3 || (CYTHON_COMPILING_IN_PYPY && PY_VERSION_HEX < 0x03050000)
  0, /*nb_oct*/
  #endif
  #if PY_MAJOR_VERSION < 3 || (CYTHON_COMPILING_IN_PYPY && PY_VERSION_HEX < 0x03050000)
  0, /*nb_hex*/
  #endif
  0, /*nb_inplace_add*/
  0, /*nb_inplace_subtract*/
  0, /*nb_inplace_multiply*/
  #if PY_MAJOR_VERSION < 3 || (CYTHON_COMPILING_IN_PYPY && PY_VERSION_HEX < 0x03050000)
  0, /*nb_inplace_divide*/
  #endif
  0, /*nb_inplace_remainder*/
  0, /*nb_inplace_power*/
  0, /*nb_inplace_lshift*/
  0, /*nb_inplace_rshift*/
  0, /*nb_inplace_and*/
  0, /*nb_inplace_xor*/
  0, /*nb_inplace_or*/
  0, /*nb_floor_divide*/
  0, /*nb_true_divide*/
  0, /*nb_inplace_floor_divide*/
  0, /*nb_inplace_true_divide*/
  0, /*nb_index*/
  #if PY_VERSION_HEX >= 0x03050000
  0, /*nb_matrix_multiply*/
  #endif
  #if PY_VERSION_HEX >= 0x03050000
  0, /*nb_inplace_matrix_multiply*/
  #endif
};

static PyTypeObject __pyx_type_4grpc_7_cython_6cygrpc_SSLSessionCacheLRU = {
  PyVarObject_HEAD_INIT(0, 0)
  "grpc._cython.cygrpc.SSLSessionCacheLRU", /*tp_name*/
  sizeof(struct __pyx_obj_4grpc_7_cython_6cygrpc_SSLSessionCacheLRU), /*tp_basicsize*/
  0, /*tp_itemsize*/
  __pyx_tp_dealloc_4grpc_7_cython_6cygrpc_SSLSessionCacheLRU, /*tp_dealloc*/
  0, /*tp_print*/
  0, /*tp_getattr*/
  0, /*tp_setattr*/
  #if PY_MAJOR_VERSION < 3
  0, /*tp_compare*/
  #endif
  #if PY_MAJOR_VERSION >= 3
  0, /*tp_as_async*/
  #endif
  0, /*tp_repr*/
  &__pyx_tp_as_number_SSLSessionCacheLRU, /*tp_as_number*/
  0, /*tp_as_sequence*/
  0, /*tp_as_mapping*/
  0, /*tp_hash*/
  0, /*tp_call*/
  0, /*tp_str*/
  0, /*tp_getattro*/
  0, /*tp_setattro*/
  0, /*tp_as_buffer*/
  Py_TPFLAGS_DEFAULT|Py_TPFLAGS_HAVE_VERSION_TAG|Py_TPFLAGS_CHECKTYPES|Py_TPFLAGS_HAVE_NEWBUFFER|Py_TPFLAGS_BASETYPE, /*tp_flags*/
  0, /*tp_doc*/
  0, /*tp_traverse*/
  0, /*tp_clear*/
  0, /*tp_richcompare*/
  0, /*tp_weaklistoffset*/
  0, /*tp_iter*/
  0, /*tp_iternext*/
  __pyx_methods_4grpc_7_cython_6cygrpc_SSLSessionCacheLRU, /*tp_methods*/
  0, /*tp_members*/
  0, /*tp_getset*/
  0, /*tp_base*/
  0, /*tp_dict*/
  0, /*tp_descr_get*/
  0, /*tp_descr_set*/
  0, /*tp_dictoffset*/
  0, /*tp_init*/
  0, /*tp_alloc*/
  __pyx_tp_new_4grpc_7_cython_6cygrpc_SSLSessionCacheLRU, /*tp_new*/
  0, /*tp_free*/
  0, /*tp_is_gc*/
  0, /*tp_bases*/
  0, /*tp_mro*/
  0, /*tp_cache*/
  0, /*tp_subclasses*/
  0, /*tp_weaklist*/
  0, /*tp_del*/
  0, /*tp_version_tag*/
  #if PY_VERSION_HEX >= 0x030400a1
  0, /*tp_finalize*/
  #endif
};
static struct __pyx_vtabstruct_4grpc_7_cython_6cygrpc_SSLChannelCredentials __pyx_vtable_4grpc_7_cython_6cygrpc_SSLChannelCredentials;

static PyObject *__pyx_tp_new_4grpc_7_cython_6cygrpc_SSLChannelCredentials(PyTypeObject *t, PyObject *a, PyObject *k) {
  struct __pyx_obj_4grpc_7_cython_6cygrpc_SSLChannelCredentials *p;
  PyObject *o = __pyx_tp_new_4grpc_7_cython_6cygrpc_ChannelCredentials(t, a, k);
  if (unlikely(!o)) return 0;
  p = ((struct __pyx_obj_4grpc_7_cython_6cygrpc_SSLChannelCredentials *)o);
  p->__pyx_base.__pyx_vtab = (struct __pyx_vtabstruct_4grpc_7_cython_6cygrpc_ChannelCredentials*)__pyx_vtabptr_4grpc_7_cython_6cygrpc_SSLChannelCredentials;
  p->_pem_root_certificates = Py_None; Py_INCREF(Py_None);
  p->_private_key = Py_None; Py_INCREF(Py_None);
  p->_certificate_chain = Py_None; Py_INCREF(Py_None);
  if (unlikely(__pyx_pw_4grpc_7_cython_6cygrpc_21SSLChannelCredentials_1__cinit__(o, a, k) < 0)) goto bad;
  return o;
  bad:
  Py_DECREF(o); o = 0;
  return NULL;
}

static void __pyx_tp_dealloc_4grpc_7_cython_6cygrpc_SSLChannelCredentials(PyObject *o) {
  struct __pyx_obj_4grpc_7_cython_6cygrpc_SSLChannelCredentials *p = (struct __pyx_obj_4grpc_7_cython_6cygrpc_SSLChannelCredentials *)o;
  #if CYTHON_USE_TP_FINALIZE
  if (unlikely(PyType_HasFeature(Py_TYPE(o), Py_TPFLAGS_HAVE_FINALIZE) && Py_TYPE(o)->tp_finalize) && !_PyGC_FINALIZED(o)) {
    if (PyObject_CallFinalizerFromDealloc(o)) return;
  }
  #endif
  PyObject_GC_UnTrack(o);
  Py_CLEAR(p->_pem_root_certificates);
  Py_CLEAR(p->_private_key);
  Py_CLEAR(p->_certificate_chain);
  #if CYTHON_USE_TYPE_SLOTS
  if (PyType_IS_GC(Py_TYPE(o)->tp_base))
  #endif
  PyObject_GC_Track(o);
  __pyx_tp_dealloc_4grpc_7_cython_6cygrpc_ChannelCredentials(o);
}

static int __pyx_tp_traverse_4grpc_7_cython_6cygrpc_SSLChannelCredentials(PyObject *o, visitproc v, void *a) {
  int e;
  struct __pyx_obj_4grpc_7_cython_6cygrpc_SSLChannelCredentials *p = (struct __pyx_obj_4grpc_7_cython_6cygrpc_SSLChannelCredentials *)o;
  e = ((likely(__pyx_ptype_4grpc_7_cython_6cygrpc_ChannelCredentials)) ? ((__pyx_ptype_4grpc_7_cython_6cygrpc_ChannelCredentials->tp_traverse) ? __pyx_ptype_4grpc_7_cython_6cygrpc_ChannelCredentials->tp_traverse(o, v, a) : 0) : __Pyx_call_next_tp_traverse(o, v, a, __pyx_tp_traverse_4grpc_7_cython_6cygrpc_SSLChannelCredentials)); if (e) return e;
  if (p->_pem_root_certificates) {
    e = (*v)(p->_pem_root_certificates, a); if (e) return e;
  }
  if (p->_private_key) {
    e = (*v)(p->_private_key, a); if (e) return e;
  }
  if (p->_certificate_chain) {
    e = (*v)(p->_certificate_chain, a); if (e) return e;
  }
  return 0;
}

static int __pyx_tp_clear_4grpc_7_cython_6cygrpc_SSLChannelCredentials(PyObject *o) {
  PyObject* tmp;
  struct __pyx_obj_4grpc_7_cython_6cygrpc_SSLChannelCredentials *p = (struct __pyx_obj_4grpc_7_cython_6cygrpc_SSLChannelCredentials *)o;
  if (likely(__pyx_ptype_4grpc_7_cython_6cygrpc_ChannelCredentials)) { if (__pyx_ptype_4grpc_7_cython_6cygrpc_ChannelCredentials->tp_clear) __pyx_ptype_4grpc_7_cython_6cygrpc_ChannelCredentials->tp_clear(o); } else __Pyx_call_next_tp_clear(o, __pyx_tp_clear_4grpc_7_cython_6cygrpc_SSLChannelCredentials);
  tmp = ((PyObject*)p->_pem_root_certificates);
  p->_pem_root_certificates = Py_None; Py_INCREF(Py_None);
  Py_XDECREF(tmp);
  tmp = ((PyObject*)p->_private_key);
  p->_private_key = Py_None; Py_INCREF(Py_None);
  Py_XDECREF(tmp);
  tmp = ((PyObject*)p->_certificate_chain);
  p->_certificate_chain = Py_None; Py_INCREF(Py_None);
  Py_XDECREF(tmp);
  return 0;
}

static PyObject *__pyx_getprop_4grpc_7_cython_6cygrpc_21SSLChannelCredentials__pem_root_certificates(PyObject *o, CYTHON_UNUSED void *x) {
  return __pyx_pw_4grpc_7_cython_6cygrpc_21SSLChannelCredentials_22_pem_root_certificates_1__get__(o);
}

static PyObject *__pyx_getprop_4grpc_7_cython_6cygrpc_21SSLChannelCredentials__private_key(PyObject *o, CYTHON_UNUSED void *x) {
  return __pyx_pw_4grpc_7_cython_6cygrpc_21SSLChannelCredentials_12_private_key_1__get__(o);
}

static PyObject *__pyx_getprop_4grpc_7_cython_6cygrpc_21SSLChannelCredentials__certificate_chain(PyObject *o, CYTHON_UNUSED void *x) {
  return __pyx_pw_4grpc_7_cython_6cygrpc_21SSLChannelCredentials_18_certificate_chain_1__get__(o);
}

static PyMethodDef __pyx_methods_4grpc_7_cython_6cygrpc_SSLChannelCredentials[] = {
  {"__reduce_cython__", (PyCFunction)__pyx_pw_4grpc_7_cython_6cygrpc_21SSLChannelCredentials_3__reduce_cython__, METH_NOARGS, 0},
  {"__setstate_cython__", (PyCFunction)__pyx_pw_4grpc_7_cython_6cygrpc_21SSLChannelCredentials_5__setstate_cython__, METH_O, 0},
  {0, 0, 0, 0}
};

static struct PyGetSetDef __pyx_getsets_4grpc_7_cython_6cygrpc_SSLChannelCredentials[] = {
  {(char *)"_pem_root_certificates", __pyx_getprop_4grpc_7_cython_6cygrpc_21SSLChannelCredentials__pem_root_certificates, 0, (char *)0, 0},
  {(char *)"_private_key", __pyx_getprop_4grpc_7_cython_6cygrpc_21SSLChannelCredentials__private_key, 0, (char *)0, 0},
  {(char *)"_certificate_chain", __pyx_getprop_4grpc_7_cython_6cygrpc_21SSLChannelCredentials__certificate_chain, 0, (char *)0, 0},
  {0, 0, 0, 0, 0}
};

static PyTypeObject __pyx_type_4grpc_7_cython_6cygrpc_SSLChannelCredentials = {
  PyVarObject_HEAD_INIT(0, 0)
  "grpc._cython.cygrpc.SSLChannelCredentials", /*tp_name*/
  sizeof(struct __pyx_obj_4grpc_7_cython_6cygrpc_SSLChannelCredentials), /*tp_basicsize*/
  0, /*tp_itemsize*/
  __pyx_tp_dealloc_4grpc_7_cython_6cygrpc_SSLChannelCredentials, /*tp_dealloc*/
  0, /*tp_print*/
  0, /*tp_getattr*/
  0, /*tp_setattr*/
  #if PY_MAJOR_VERSION < 3
  0, /*tp_compare*/
  #endif
  #if PY_MAJOR_VERSION >= 3
  0, /*tp_as_async*/
  #endif
  0, /*tp_repr*/
  0, /*tp_as_number*/
  0, /*tp_as_sequence*/
  0, /*tp_as_mapping*/
  0, /*tp_hash*/
  0, /*tp_call*/
  0, /*tp_str*/
  0, /*tp_getattro*/
  0, /*tp_setattro*/
  0, /*tp_as_buffer*/
  Py_TPFLAGS_DEFAULT|Py_TPFLAGS_HAVE_VERSION_TAG|Py_TPFLAGS_CHECKTYPES|Py_TPFLAGS_HAVE_NEWBUFFER|Py_TPFLAGS_BASETYPE|Py_TPFLAGS_HAVE_GC, /*tp_flags*/
  0, /*tp_doc*/
  __pyx_tp_traverse_4grpc_7_cython_6cygrpc_SSLChannelCredentials, /*tp_traverse*/
  __pyx_tp_clear_4grpc_7_cython_6cygrpc_SSLChannelCredentials, /*tp_clear*/
  0, /*tp_richcompare*/
  0, /*tp_weaklistoffset*/
  0, /*tp_iter*/
  0, /*tp_iternext*/
  __pyx_methods_4grpc_7_cython_6cygrpc_SSLChannelCredentials, /*tp_methods*/
  0, /*tp_members*/
  __pyx_getsets_4grpc_7_cython_6cygrpc_SSLChannelCredentials, /*tp_getset*/
  0, /*tp_base*/
  0, /*tp_dict*/
  0, /*tp_descr_get*/
  0, /*tp_descr_set*/
  0, /*tp_dictoffset*/
  0, /*tp_init*/
  0, /*tp_alloc*/
  __pyx_tp_new_4grpc_7_cython_6cygrpc_SSLChannelCredentials, /*tp_new*/
  0, /*tp_free*/
  0, /*tp_is_gc*/
  0, /*tp_bases*/
  0, /*tp_mro*/
  0, /*tp_cache*/
  0, /*tp_subclasses*/
  0, /*tp_weaklist*/
  0, /*tp_del*/
  0, /*tp_version_tag*/
  #if PY_VERSION_HEX >= 0x030400a1
  0, /*tp_finalize*/
  #endif
};
static struct __pyx_vtabstruct_4grpc_7_cython_6cygrpc_CompositeChannelCredentials __pyx_vtable_4grpc_7_cython_6cygrpc_CompositeChannelCredentials;

static PyObject *__pyx_tp_new_4grpc_7_cython_6cygrpc_CompositeChannelCredentials(PyTypeObject *t, PyObject *a, PyObject *k) {
  struct __pyx_obj_4grpc_7_cython_6cygrpc_CompositeChannelCredentials *p;
  PyObject *o = __pyx_tp_new_4grpc_7_cython_6cygrpc_ChannelCredentials(t, a, k);
  if (unlikely(!o)) return 0;
  p = ((struct __pyx_obj_4grpc_7_cython_6cygrpc_CompositeChannelCredentials *)o);
  p->__pyx_base.__pyx_vtab = (struct __pyx_vtabstruct_4grpc_7_cython_6cygrpc_ChannelCredentials*)__pyx_vtabptr_4grpc_7_cython_6cygrpc_CompositeChannelCredentials;
  p->_call_credentialses = ((PyObject*)Py_None); Py_INCREF(Py_None);
  p->_channel_credentials = ((struct __pyx_obj_4grpc_7_cython_6cygrpc_ChannelCredentials *)Py_None); Py_INCREF(Py_None);
  if (unlikely(__pyx_pw_4grpc_7_cython_6cygrpc_27CompositeChannelCredentials_1__cinit__(o, a, k) < 0)) goto bad;
  return o;
  bad:
  Py_DECREF(o); o = 0;
  return NULL;
}

static void __pyx_tp_dealloc_4grpc_7_cython_6cygrpc_CompositeChannelCredentials(PyObject *o) {
  struct __pyx_obj_4grpc_7_cython_6cygrpc_CompositeChannelCredentials *p = (struct __pyx_obj_4grpc_7_cython_6cygrpc_CompositeChannelCredentials *)o;
  #if CYTHON_USE_TP_FINALIZE
  if (unlikely(PyType_HasFeature(Py_TYPE(o), Py_TPFLAGS_HAVE_FINALIZE) && Py_TYPE(o)->tp_finalize) && !_PyGC_FINALIZED(o)) {
    if (PyObject_CallFinalizerFromDealloc(o)) return;
  }
  #endif
  PyObject_GC_UnTrack(o);
  Py_CLEAR(p->_call_credentialses);
  Py_CLEAR(p->_channel_credentials);
  #if CYTHON_USE_TYPE_SLOTS
  if (PyType_IS_GC(Py_TYPE(o)->tp_base))
  #endif
  PyObject_GC_Track(o);
  __pyx_tp_dealloc_4grpc_7_cython_6cygrpc_ChannelCredentials(o);
}

static int __pyx_tp_traverse_4grpc_7_cython_6cygrpc_CompositeChannelCredentials(PyObject *o, visitproc v, void *a) {
  int e;
  struct __pyx_obj_4grpc_7_cython_6cygrpc_CompositeChannelCredentials *p = (struct __pyx_obj_4grpc_7_cython_6cygrpc_CompositeChannelCredentials *)o;
  e = ((likely(__pyx_ptype_4grpc_7_cython_6cygrpc_ChannelCredentials)) ? ((__pyx_ptype_4grpc_7_cython_6cygrpc_ChannelCredentials->tp_traverse) ? __pyx_ptype_4grpc_7_cython_6cygrpc_ChannelCredentials->tp_traverse(o, v, a) : 0) : __Pyx_call_next_tp_traverse(o, v, a, __pyx_tp_traverse_4grpc_7_cython_6cygrpc_CompositeChannelCredentials)); if (e) return e;
  if (p->_call_credentialses) {
    e = (*v)(p->_call_credentialses, a); if (e) return e;
  }
  if (p->_channel_credentials) {
    e = (*v)(((PyObject *)p->_channel_credentials), a); if (e) return e;
  }
  return 0;
}

static int __pyx_tp_clear_4grpc_7_cython_6cygrpc_CompositeChannelCredentials(PyObject *o) {
  PyObject* tmp;
  struct __pyx_obj_4grpc_7_cython_6cygrpc_CompositeChannelCredentials *p = (struct __pyx_obj_4grpc_7_cython_6cygrpc_CompositeChannelCredentials *)o;
  if (likely(__pyx_ptype_4grpc_7_cython_6cygrpc_ChannelCredentials)) { if (__pyx_ptype_4grpc_7_cython_6cygrpc_ChannelCredentials->tp_clear) __pyx_ptype_4grpc_7_cython_6cygrpc_ChannelCredentials->tp_clear(o); } else __Pyx_call_next_tp_clear(o, __pyx_tp_clear_4grpc_7_cython_6cygrpc_CompositeChannelCredentials);
  tmp = ((PyObject*)p->_call_credentialses);
  p->_call_credentialses = ((PyObject*)Py_None); Py_INCREF(Py_None);
  Py_XDECREF(tmp);
  tmp = ((PyObject*)p->_channel_credentials);
  p->_channel_credentials = ((struct __pyx_obj_4grpc_7_cython_6cygrpc_ChannelCredentials *)Py_None); Py_INCREF(Py_None);
  Py_XDECREF(tmp);
  return 0;
}

static PyObject *__pyx_getprop_4grpc_7_cython_6cygrpc_27CompositeChannelCredentials__call_credentialses(PyObject *o, CYTHON_UNUSED void *x) {
  return __pyx_pw_4grpc_7_cython_6cygrpc_27CompositeChannelCredentials_19_call_credentialses_1__get__(o);
}

static PyObject *__pyx_getprop_4grpc_7_cython_6cygrpc_27CompositeChannelCredentials__channel_credentials(PyObject *o, CYTHON_UNUSED void *x) {
  return __pyx_pw_4grpc_7_cython_6cygrpc_27CompositeChannelCredentials_20_channel_credentials_1__get__(o);
}

static PyMethodDef __pyx_methods_4grpc_7_cython_6cygrpc_CompositeChannelCredentials[] = {
  {"__reduce_cython__", (PyCFunction)__pyx_pw_4grpc_7_cython_6cygrpc_27CompositeChannelCredentials_3__reduce_cython__, METH_NOARGS, 0},
  {"__setstate_cython__", (PyCFunction)__pyx_pw_4grpc_7_cython_6cygrpc_27CompositeChannelCredentials_5__setstate_cython__, METH_O, 0},
  {0, 0, 0, 0}
};

static struct PyGetSetDef __pyx_getsets_4grpc_7_cython_6cygrpc_CompositeChannelCredentials[] = {
  {(char *)"_call_credentialses", __pyx_getprop_4grpc_7_cython_6cygrpc_27CompositeChannelCredentials__call_credentialses, 0, (char *)0, 0},
  {(char *)"_channel_credentials", __pyx_getprop_4grpc_7_cython_6cygrpc_27CompositeChannelCredentials__channel_credentials, 0, (char *)0, 0},
  {0, 0, 0, 0, 0}
};

static PyTypeObject __pyx_type_4grpc_7_cython_6cygrpc_CompositeChannelCredentials = {
  PyVarObject_HEAD_INIT(0, 0)
  "grpc._cython.cygrpc.CompositeChannelCredentials", /*tp_name*/
  sizeof(struct __pyx_obj_4grpc_7_cython_6cygrpc_CompositeChannelCredentials), /*tp_basicsize*/
  0, /*tp_itemsize*/
  __pyx_tp_dealloc_4grpc_7_cython_6cygrpc_CompositeChannelCredentials, /*tp_dealloc*/
  0, /*tp_print*/
  0, /*tp_getattr*/
  0, /*tp_setattr*/
  #if PY_MAJOR_VERSION < 3
  0, /*tp_compare*/
  #endif
  #if PY_MAJOR_VERSION >= 3
  0, /*tp_as_async*/
  #endif
  0, /*tp_repr*/
  0, /*tp_as_number*/
  0, /*tp_as_sequence*/
  0, /*tp_as_mapping*/
  0, /*tp_hash*/
  0, /*tp_call*/
  0, /*tp_str*/
  0, /*tp_getattro*/
  0, /*tp_setattro*/
  0, /*tp_as_buffer*/
  Py_TPFLAGS_DEFAULT|Py_TPFLAGS_HAVE_VERSION_TAG|Py_TPFLAGS_CHECKTYPES|Py_TPFLAGS_HAVE_NEWBUFFER|Py_TPFLAGS_BASETYPE|Py_TPFLAGS_HAVE_GC, /*tp_flags*/
  0, /*tp_doc*/
  __pyx_tp_traverse_4grpc_7_cython_6cygrpc_CompositeChannelCredentials, /*tp_traverse*/
  __pyx_tp_clear_4grpc_7_cython_6cygrpc_CompositeChannelCredentials, /*tp_clear*/
  0, /*tp_richcompare*/
  0, /*tp_weaklistoffset*/
  0, /*tp_iter*/
  0, /*tp_iternext*/
  __pyx_methods_4grpc_7_cython_6cygrpc_CompositeChannelCredentials, /*tp_methods*/
  0, /*tp_members*/
  __pyx_getsets_4grpc_7_cython_6cygrpc_CompositeChannelCredentials, /*tp_getset*/
  0, /*tp_base*/
  0, /*tp_dict*/
  0, /*tp_descr_get*/
  0, /*tp_descr_set*/
  0, /*tp_dictoffset*/
  0, /*tp_init*/
  0, /*tp_alloc*/
  __pyx_tp_new_4grpc_7_cython_6cygrpc_CompositeChannelCredentials, /*tp_new*/
  0, /*tp_free*/
  0, /*tp_is_gc*/
  0, /*tp_bases*/
  0, /*tp_mro*/
  0, /*tp_cache*/
  0, /*tp_subclasses*/
  0, /*tp_weaklist*/
  0, /*tp_del*/
  0, /*tp_version_tag*/
  #if PY_VERSION_HEX >= 0x030400a1
  0, /*tp_finalize*/
  #endif
};

static PyObject *__pyx_tp_new_4grpc_7_cython_6cygrpc_ServerCertificateConfig(PyTypeObject *t, CYTHON_UNUSED PyObject *a, CYTHON_UNUSED PyObject *k) {
  struct __pyx_obj_4grpc_7_cython_6cygrpc_ServerCertificateConfig *p;
  PyObject *o;
  if (likely((t->tp_flags & Py_TPFLAGS_IS_ABSTRACT) == 0)) {
    o = (*t->tp_alloc)(t, 0);
  } else {
    o = (PyObject *) PyBaseObject_Type.tp_new(t, __pyx_empty_tuple, 0);
  }
  if (unlikely(!o)) return 0;
  p = ((struct __pyx_obj_4grpc_7_cython_6cygrpc_ServerCertificateConfig *)o);
  p->references = ((PyObject*)Py_None); Py_INCREF(Py_None);
  if (unlikely(__pyx_pw_4grpc_7_cython_6cygrpc_23ServerCertificateConfig_1__cinit__(o, __pyx_empty_tuple, NULL) < 0)) goto bad;
  return o;
  bad:
  Py_DECREF(o); o = 0;
  return NULL;
}

static void __pyx_tp_dealloc_4grpc_7_cython_6cygrpc_ServerCertificateConfig(PyObject *o) {
  struct __pyx_obj_4grpc_7_cython_6cygrpc_ServerCertificateConfig *p = (struct __pyx_obj_4grpc_7_cython_6cygrpc_ServerCertificateConfig *)o;
  #if CYTHON_USE_TP_FINALIZE
  if (unlikely(PyType_HasFeature(Py_TYPE(o), Py_TPFLAGS_HAVE_FINALIZE) && Py_TYPE(o)->tp_finalize) && !_PyGC_FINALIZED(o)) {
    if (PyObject_CallFinalizerFromDealloc(o)) return;
  }
  #endif
  PyObject_GC_UnTrack(o);
  {
    PyObject *etype, *eval, *etb;
    PyErr_Fetch(&etype, &eval, &etb);
    ++Py_REFCNT(o);
    __pyx_pw_4grpc_7_cython_6cygrpc_23ServerCertificateConfig_3__dealloc__(o);
    --Py_REFCNT(o);
    PyErr_Restore(etype, eval, etb);
  }
  Py_CLEAR(p->references);
  (*Py_TYPE(o)->tp_free)(o);
}

static int __pyx_tp_traverse_4grpc_7_cython_6cygrpc_ServerCertificateConfig(PyObject *o, visitproc v, void *a) {
  int e;
  struct __pyx_obj_4grpc_7_cython_6cygrpc_ServerCertificateConfig *p = (struct __pyx_obj_4grpc_7_cython_6cygrpc_ServerCertificateConfig *)o;
  if (p->references) {
    e = (*v)(p->references, a); if (e) return e;
  }
  return 0;
}

static int __pyx_tp_clear_4grpc_7_cython_6cygrpc_ServerCertificateConfig(PyObject *o) {
  PyObject* tmp;
  struct __pyx_obj_4grpc_7_cython_6cygrpc_ServerCertificateConfig *p = (struct __pyx_obj_4grpc_7_cython_6cygrpc_ServerCertificateConfig *)o;
  tmp = ((PyObject*)p->references);
  p->references = ((PyObject*)Py_None); Py_INCREF(Py_None);
  Py_XDECREF(tmp);
  return 0;
}

static PyMethodDef __pyx_methods_4grpc_7_cython_6cygrpc_ServerCertificateConfig[] = {
  {"__reduce_cython__", (PyCFunction)__pyx_pw_4grpc_7_cython_6cygrpc_23ServerCertificateConfig_5__reduce_cython__, METH_NOARGS, 0},
  {"__setstate_cython__", (PyCFunction)__pyx_pw_4grpc_7_cython_6cygrpc_23ServerCertificateConfig_7__setstate_cython__, METH_O, 0},
  {0, 0, 0, 0}
};

static PyTypeObject __pyx_type_4grpc_7_cython_6cygrpc_ServerCertificateConfig = {
  PyVarObject_HEAD_INIT(0, 0)
  "grpc._cython.cygrpc.ServerCertificateConfig", /*tp_name*/
  sizeof(struct __pyx_obj_4grpc_7_cython_6cygrpc_ServerCertificateConfig), /*tp_basicsize*/
  0, /*tp_itemsize*/
  __pyx_tp_dealloc_4grpc_7_cython_6cygrpc_ServerCertificateConfig, /*tp_dealloc*/
  0, /*tp_print*/
  0, /*tp_getattr*/
  0, /*tp_setattr*/
  #if PY_MAJOR_VERSION < 3
  0, /*tp_compare*/
  #endif
  #if PY_MAJOR_VERSION >= 3
  0, /*tp_as_async*/
  #endif
  0, /*tp_repr*/
  0, /*tp_as_number*/
  0, /*tp_as_sequence*/
  0, /*tp_as_mapping*/
  0, /*tp_hash*/
  0, /*tp_call*/
  0, /*tp_str*/
  0, /*tp_getattro*/
  0, /*tp_setattro*/
  0, /*tp_as_buffer*/
  Py_TPFLAGS_DEFAULT|Py_TPFLAGS_HAVE_VERSION_TAG|Py_TPFLAGS_CHECKTYPES|Py_TPFLAGS_HAVE_NEWBUFFER|Py_TPFLAGS_BASETYPE|Py_TPFLAGS_HAVE_GC, /*tp_flags*/
  0, /*tp_doc*/
  __pyx_tp_traverse_4grpc_7_cython_6cygrpc_ServerCertificateConfig, /*tp_traverse*/
  __pyx_tp_clear_4grpc_7_cython_6cygrpc_ServerCertificateConfig, /*tp_clear*/
  0, /*tp_richcompare*/
  0, /*tp_weaklistoffset*/
  0, /*tp_iter*/
  0, /*tp_iternext*/
  __pyx_methods_4grpc_7_cython_6cygrpc_ServerCertificateConfig, /*tp_methods*/
  0, /*tp_members*/
  0, /*tp_getset*/
  0, /*tp_base*/
  0, /*tp_dict*/
  0, /*tp_descr_get*/
  0, /*tp_descr_set*/
  0, /*tp_dictoffset*/
  0, /*tp_init*/
  0, /*tp_alloc*/
  __pyx_tp_new_4grpc_7_cython_6cygrpc_ServerCertificateConfig, /*tp_new*/
  0, /*tp_free*/
  0, /*tp_is_gc*/
  0, /*tp_bases*/
  0, /*tp_mro*/
  0, /*tp_cache*/
  0, /*tp_subclasses*/
  0, /*tp_weaklist*/
  0, /*tp_del*/
  0, /*tp_version_tag*/
  #if PY_VERSION_HEX >= 0x030400a1
  0, /*tp_finalize*/
  #endif
};

static PyObject *__pyx_tp_new_4grpc_7_cython_6cygrpc_ServerCredentials(PyTypeObject *t, CYTHON_UNUSED PyObject *a, CYTHON_UNUSED PyObject *k) {
  struct __pyx_obj_4grpc_7_cython_6cygrpc_ServerCredentials *p;
  PyObject *o;
  if (likely((t->tp_flags & Py_TPFLAGS_IS_ABSTRACT) == 0)) {
    o = (*t->tp_alloc)(t, 0);
  } else {
    o = (PyObject *) PyBaseObject_Type.tp_new(t, __pyx_empty_tuple, 0);
  }
  if (unlikely(!o)) return 0;
  p = ((struct __pyx_obj_4grpc_7_cython_6cygrpc_ServerCredentials *)o);
  p->references = ((PyObject*)Py_None); Py_INCREF(Py_None);
  p->initial_cert_config = Py_None; Py_INCREF(Py_None);
  p->cert_config_fetcher = Py_None; Py_INCREF(Py_None);
  if (unlikely(__pyx_pw_4grpc_7_cython_6cygrpc_17ServerCredentials_1__cinit__(o, __pyx_empty_tuple, NULL) < 0)) goto bad;
  return o;
  bad:
  Py_DECREF(o); o = 0;
  return NULL;
}

static void __pyx_tp_dealloc_4grpc_7_cython_6cygrpc_ServerCredentials(PyObject *o) {
  struct __pyx_obj_4grpc_7_cython_6cygrpc_ServerCredentials *p = (struct __pyx_obj_4grpc_7_cython_6cygrpc_ServerCredentials *)o;
  #if CYTHON_USE_TP_FINALIZE
  if (unlikely(PyType_HasFeature(Py_TYPE(o), Py_TPFLAGS_HAVE_FINALIZE) && Py_TYPE(o)->tp_finalize) && !_PyGC_FINALIZED(o)) {
    if (PyObject_CallFinalizerFromDealloc(o)) return;
  }
  #endif
  PyObject_GC_UnTrack(o);
  {
    PyObject *etype, *eval, *etb;
    PyErr_Fetch(&etype, &eval, &etb);
    ++Py_REFCNT(o);
    __pyx_pw_4grpc_7_cython_6cygrpc_17ServerCredentials_3__dealloc__(o);
    --Py_REFCNT(o);
    PyErr_Restore(etype, eval, etb);
  }
  Py_CLEAR(p->references);
  Py_CLEAR(p->initial_cert_config);
  Py_CLEAR(p->cert_config_fetcher);
  (*Py_TYPE(o)->tp_free)(o);
}

static int __pyx_tp_traverse_4grpc_7_cython_6cygrpc_ServerCredentials(PyObject *o, visitproc v, void *a) {
  int e;
  struct __pyx_obj_4grpc_7_cython_6cygrpc_ServerCredentials *p = (struct __pyx_obj_4grpc_7_cython_6cygrpc_ServerCredentials *)o;
  if (p->references) {
    e = (*v)(p->references, a); if (e) return e;
  }
  if (p->initial_cert_config) {
    e = (*v)(p->initial_cert_config, a); if (e) return e;
  }
  if (p->cert_config_fetcher) {
    e = (*v)(p->cert_config_fetcher, a); if (e) return e;
  }
  return 0;
}

static int __pyx_tp_clear_4grpc_7_cython_6cygrpc_ServerCredentials(PyObject *o) {
  PyObject* tmp;
  struct __pyx_obj_4grpc_7_cython_6cygrpc_ServerCredentials *p = (struct __pyx_obj_4grpc_7_cython_6cygrpc_ServerCredentials *)o;
  tmp = ((PyObject*)p->references);
  p->references = ((PyObject*)Py_None); Py_INCREF(Py_None);
  Py_XDECREF(tmp);
  tmp = ((PyObject*)p->initial_cert_config);
  p->initial_cert_config = Py_None; Py_INCREF(Py_None);
  Py_XDECREF(tmp);
  tmp = ((PyObject*)p->cert_config_fetcher);
  p->cert_config_fetcher = Py_None; Py_INCREF(Py_None);
  Py_XDECREF(tmp);
  return 0;
}

static PyMethodDef __pyx_methods_4grpc_7_cython_6cygrpc_ServerCredentials[] = {
  {"__reduce_cython__", (PyCFunction)__pyx_pw_4grpc_7_cython_6cygrpc_17ServerCredentials_5__reduce_cython__, METH_NOARGS, 0},
  {"__setstate_cython__", (PyCFunction)__pyx_pw_4grpc_7_cython_6cygrpc_17ServerCredentials_7__setstate_cython__, METH_O, 0},
  {0, 0, 0, 0}
};

static PyTypeObject __pyx_type_4grpc_7_cython_6cygrpc_ServerCredentials = {
  PyVarObject_HEAD_INIT(0, 0)
  "grpc._cython.cygrpc.ServerCredentials", /*tp_name*/
  sizeof(struct __pyx_obj_4grpc_7_cython_6cygrpc_ServerCredentials), /*tp_basicsize*/
  0, /*tp_itemsize*/
  __pyx_tp_dealloc_4grpc_7_cython_6cygrpc_ServerCredentials, /*tp_dealloc*/
  0, /*tp_print*/
  0, /*tp_getattr*/
  0, /*tp_setattr*/
  #if PY_MAJOR_VERSION < 3
  0, /*tp_compare*/
  #endif
  #if PY_MAJOR_VERSION >= 3
  0, /*tp_as_async*/
  #endif
  0, /*tp_repr*/
  0, /*tp_as_number*/
  0, /*tp_as_sequence*/
  0, /*tp_as_mapping*/
  0, /*tp_hash*/
  0, /*tp_call*/
  0, /*tp_str*/
  0, /*tp_getattro*/
  0, /*tp_setattro*/
  0, /*tp_as_buffer*/
  Py_TPFLAGS_DEFAULT|Py_TPFLAGS_HAVE_VERSION_TAG|Py_TPFLAGS_CHECKTYPES|Py_TPFLAGS_HAVE_NEWBUFFER|Py_TPFLAGS_BASETYPE|Py_TPFLAGS_HAVE_GC, /*tp_flags*/
  0, /*tp_doc*/
  __pyx_tp_traverse_4grpc_7_cython_6cygrpc_ServerCredentials, /*tp_traverse*/
  __pyx_tp_clear_4grpc_7_cython_6cygrpc_ServerCredentials, /*tp_clear*/
  0, /*tp_richcompare*/
  0, /*tp_weaklistoffset*/
  0, /*tp_iter*/
  0, /*tp_iternext*/
  __pyx_methods_4grpc_7_cython_6cygrpc_ServerCredentials, /*tp_methods*/
  0, /*tp_members*/
  0, /*tp_getset*/
  0, /*tp_base*/
  0, /*tp_dict*/
  0, /*tp_descr_get*/
  0, /*tp_descr_set*/
  0, /*tp_dictoffset*/
  0, /*tp_init*/
  0, /*tp_alloc*/
  __pyx_tp_new_4grpc_7_cython_6cygrpc_ServerCredentials, /*tp_new*/
  0, /*tp_free*/
  0, /*tp_is_gc*/
  0, /*tp_bases*/
  0, /*tp_mro*/
  0, /*tp_cache*/
  0, /*tp_subclasses*/
  0, /*tp_weaklist*/
  0, /*tp_del*/
  0, /*tp_version_tag*/
  #if PY_VERSION_HEX >= 0x030400a1
  0, /*tp_finalize*/
  #endif
};
static struct __pyx_vtabstruct_4grpc_7_cython_6cygrpc_CompletionQueue __pyx_vtable_4grpc_7_cython_6cygrpc_CompletionQueue;

static PyObject *__pyx_tp_new_4grpc_7_cython_6cygrpc_CompletionQueue(PyTypeObject *t, PyObject *a, PyObject *k) {
  struct __pyx_obj_4grpc_7_cython_6cygrpc_CompletionQueue *p;
  PyObject *o;
  if (likely((t->tp_flags & Py_TPFLAGS_IS_ABSTRACT) == 0)) {
    o = (*t->tp_alloc)(t, 0);
  } else {
    o = (PyObject *) PyBaseObject_Type.tp_new(t, __pyx_empty_tuple, 0);
  }
  if (unlikely(!o)) return 0;
  p = ((struct __pyx_obj_4grpc_7_cython_6cygrpc_CompletionQueue *)o);
  p->__pyx_vtab = __pyx_vtabptr_4grpc_7_cython_6cygrpc_CompletionQueue;
  if (unlikely(__pyx_pw_4grpc_7_cython_6cygrpc_15CompletionQueue_1__cinit__(o, a, k) < 0)) goto bad;
  return o;
  bad:
  Py_DECREF(o); o = 0;
  return NULL;
}

static void __pyx_tp_dealloc_4grpc_7_cython_6cygrpc_CompletionQueue(PyObject *o) {
  #if CYTHON_USE_TP_FINALIZE
  if (unlikely(PyType_HasFeature(Py_TYPE(o), Py_TPFLAGS_HAVE_FINALIZE) && Py_TYPE(o)->tp_finalize) && (!PyType_IS_GC(Py_TYPE(o)) || !_PyGC_FINALIZED(o))) {
    if (PyObject_CallFinalizerFromDealloc(o)) return;
  }
  #endif
  {
    PyObject *etype, *eval, *etb;
    PyErr_Fetch(&etype, &eval, &etb);
    ++Py_REFCNT(o);
    __pyx_pw_4grpc_7_cython_6cygrpc_15CompletionQueue_9__dealloc__(o);
    --Py_REFCNT(o);
    PyErr_Restore(etype, eval, etb);
  }
  (*Py_TYPE(o)->tp_free)(o);
}

static PyMethodDef __pyx_methods_4grpc_7_cython_6cygrpc_CompletionQueue[] = {
  {"poll", (PyCFunction)__pyx_pw_4grpc_7_cython_6cygrpc_15CompletionQueue_3poll, METH_VARARGS|METH_KEYWORDS, 0},
  {"shutdown", (PyCFunction)__pyx_pw_4grpc_7_cython_6cygrpc_15CompletionQueue_5shutdown, METH_NOARGS, 0},
  {"clear", (PyCFunction)__pyx_pw_4grpc_7_cython_6cygrpc_15CompletionQueue_7clear, METH_NOARGS, 0},
  {"__reduce_cython__", (PyCFunction)__pyx_pw_4grpc_7_cython_6cygrpc_15CompletionQueue_11__reduce_cython__, METH_NOARGS, 0},
  {"__setstate_cython__", (PyCFunction)__pyx_pw_4grpc_7_cython_6cygrpc_15CompletionQueue_13__setstate_cython__, METH_O, 0},
  {0, 0, 0, 0}
};

static PyTypeObject __pyx_type_4grpc_7_cython_6cygrpc_CompletionQueue = {
  PyVarObject_HEAD_INIT(0, 0)
  "grpc._cython.cygrpc.CompletionQueue", /*tp_name*/
  sizeof(struct __pyx_obj_4grpc_7_cython_6cygrpc_CompletionQueue), /*tp_basicsize*/
  0, /*tp_itemsize*/
  __pyx_tp_dealloc_4grpc_7_cython_6cygrpc_CompletionQueue, /*tp_dealloc*/
  0, /*tp_print*/
  0, /*tp_getattr*/
  0, /*tp_setattr*/
  #if PY_MAJOR_VERSION < 3
  0, /*tp_compare*/
  #endif
  #if PY_MAJOR_VERSION >= 3
  0, /*tp_as_async*/
  #endif
  0, /*tp_repr*/
  0, /*tp_as_number*/
  0, /*tp_as_sequence*/
  0, /*tp_as_mapping*/
  0, /*tp_hash*/
  0, /*tp_call*/
  0, /*tp_str*/
  0, /*tp_getattro*/
  0, /*tp_setattro*/
  0, /*tp_as_buffer*/
  Py_TPFLAGS_DEFAULT|Py_TPFLAGS_HAVE_VERSION_TAG|Py_TPFLAGS_CHECKTYPES|Py_TPFLAGS_HAVE_NEWBUFFER|Py_TPFLAGS_BASETYPE, /*tp_flags*/
  0, /*tp_doc*/
  0, /*tp_traverse*/
  0, /*tp_clear*/
  0, /*tp_richcompare*/
  0, /*tp_weaklistoffset*/
  0, /*tp_iter*/
  0, /*tp_iternext*/
  __pyx_methods_4grpc_7_cython_6cygrpc_CompletionQueue, /*tp_methods*/
  0, /*tp_members*/
  0, /*tp_getset*/
  0, /*tp_base*/
  0, /*tp_dict*/
  0, /*tp_descr_get*/
  0, /*tp_descr_set*/
  0, /*tp_dictoffset*/
  0, /*tp_init*/
  0, /*tp_alloc*/
  __pyx_tp_new_4grpc_7_cython_6cygrpc_CompletionQueue, /*tp_new*/
  0, /*tp_free*/
  0, /*tp_is_gc*/
  0, /*tp_bases*/
  0, /*tp_mro*/
  0, /*tp_cache*/
  0, /*tp_subclasses*/
  0, /*tp_weaklist*/
  0, /*tp_del*/
  0, /*tp_version_tag*/
  #if PY_VERSION_HEX >= 0x030400a1
  0, /*tp_finalize*/
  #endif
};

static PyObject *__pyx_tp_new_4grpc_7_cython_6cygrpc_ConnectivityEvent(PyTypeObject *t, PyObject *a, PyObject *k) {
  struct __pyx_obj_4grpc_7_cython_6cygrpc_ConnectivityEvent *p;
  PyObject *o;
  if (likely((t->tp_flags & Py_TPFLAGS_IS_ABSTRACT) == 0)) {
    o = (*t->tp_alloc)(t, 0);
  } else {
    o = (PyObject *) PyBaseObject_Type.tp_new(t, __pyx_empty_tuple, 0);
  }
  if (unlikely(!o)) return 0;
  p = ((struct __pyx_obj_4grpc_7_cython_6cygrpc_ConnectivityEvent *)o);
  p->tag = Py_None; Py_INCREF(Py_None);
  if (unlikely(__pyx_pw_4grpc_7_cython_6cygrpc_17ConnectivityEvent_1__cinit__(o, a, k) < 0)) goto bad;
  return o;
  bad:
  Py_DECREF(o); o = 0;
  return NULL;
}

static void __pyx_tp_dealloc_4grpc_7_cython_6cygrpc_ConnectivityEvent(PyObject *o) {
  struct __pyx_obj_4grpc_7_cython_6cygrpc_ConnectivityEvent *p = (struct __pyx_obj_4grpc_7_cython_6cygrpc_ConnectivityEvent *)o;
  #if CYTHON_USE_TP_FINALIZE
  if (unlikely(PyType_HasFeature(Py_TYPE(o), Py_TPFLAGS_HAVE_FINALIZE) && Py_TYPE(o)->tp_finalize) && !_PyGC_FINALIZED(o)) {
    if (PyObject_CallFinalizerFromDealloc(o)) return;
  }
  #endif
  PyObject_GC_UnTrack(o);
  Py_CLEAR(p->tag);
  (*Py_TYPE(o)->tp_free)(o);
}

static int __pyx_tp_traverse_4grpc_7_cython_6cygrpc_ConnectivityEvent(PyObject *o, visitproc v, void *a) {
  int e;
  struct __pyx_obj_4grpc_7_cython_6cygrpc_ConnectivityEvent *p = (struct __pyx_obj_4grpc_7_cython_6cygrpc_ConnectivityEvent *)o;
  if (p->tag) {
    e = (*v)(p->tag, a); if (e) return e;
  }
  return 0;
}

static int __pyx_tp_clear_4grpc_7_cython_6cygrpc_ConnectivityEvent(PyObject *o) {
  PyObject* tmp;
  struct __pyx_obj_4grpc_7_cython_6cygrpc_ConnectivityEvent *p = (struct __pyx_obj_4grpc_7_cython_6cygrpc_ConnectivityEvent *)o;
  tmp = ((PyObject*)p->tag);
  p->tag = Py_None; Py_INCREF(Py_None);
  Py_XDECREF(tmp);
  return 0;
}

static PyObject *__pyx_getprop_4grpc_7_cython_6cygrpc_17ConnectivityEvent_completion_type(PyObject *o, CYTHON_UNUSED void *x) {
  return __pyx_pw_4grpc_7_cython_6cygrpc_17ConnectivityEvent_15completion_type_1__get__(o);
}

static PyObject *__pyx_getprop_4grpc_7_cython_6cygrpc_17ConnectivityEvent_success(PyObject *o, CYTHON_UNUSED void *x) {
  return __pyx_pw_4grpc_7_cython_6cygrpc_17ConnectivityEvent_7success_1__get__(o);
}

static PyObject *__pyx_getprop_4grpc_7_cython_6cygrpc_17ConnectivityEvent_tag(PyObject *o, CYTHON_UNUSED void *x) {
  return __pyx_pw_4grpc_7_cython_6cygrpc_17ConnectivityEvent_3tag_1__get__(o);
}

static PyMethodDef __pyx_methods_4grpc_7_cython_6cygrpc_ConnectivityEvent[] = {
  {"__reduce_cython__", (PyCFunction)__pyx_pw_4grpc_7_cython_6cygrpc_17ConnectivityEvent_3__reduce_cython__, METH_NOARGS, 0},
  {"__setstate_cython__", (PyCFunction)__pyx_pw_4grpc_7_cython_6cygrpc_17ConnectivityEvent_5__setstate_cython__, METH_O, 0},
  {0, 0, 0, 0}
};

static struct PyGetSetDef __pyx_getsets_4grpc_7_cython_6cygrpc_ConnectivityEvent[] = {
  {(char *)"completion_type", __pyx_getprop_4grpc_7_cython_6cygrpc_17ConnectivityEvent_completion_type, 0, (char *)0, 0},
  {(char *)"success", __pyx_getprop_4grpc_7_cython_6cygrpc_17ConnectivityEvent_success, 0, (char *)0, 0},
  {(char *)"tag", __pyx_getprop_4grpc_7_cython_6cygrpc_17ConnectivityEvent_tag, 0, (char *)0, 0},
  {0, 0, 0, 0, 0}
};

static PyTypeObject __pyx_type_4grpc_7_cython_6cygrpc_ConnectivityEvent = {
  PyVarObject_HEAD_INIT(0, 0)
  "grpc._cython.cygrpc.ConnectivityEvent", /*tp_name*/
  sizeof(struct __pyx_obj_4grpc_7_cython_6cygrpc_ConnectivityEvent), /*tp_basicsize*/
  0, /*tp_itemsize*/
  __pyx_tp_dealloc_4grpc_7_cython_6cygrpc_ConnectivityEvent, /*tp_dealloc*/
  0, /*tp_print*/
  0, /*tp_getattr*/
  0, /*tp_setattr*/
  #if PY_MAJOR_VERSION < 3
  0, /*tp_compare*/
  #endif
  #if PY_MAJOR_VERSION >= 3
  0, /*tp_as_async*/
  #endif
  0, /*tp_repr*/
  0, /*tp_as_number*/
  0, /*tp_as_sequence*/
  0, /*tp_as_mapping*/
  0, /*tp_hash*/
  0, /*tp_call*/
  0, /*tp_str*/
  0, /*tp_getattro*/
  0, /*tp_setattro*/
  0, /*tp_as_buffer*/
  Py_TPFLAGS_DEFAULT|Py_TPFLAGS_HAVE_VERSION_TAG|Py_TPFLAGS_CHECKTYPES|Py_TPFLAGS_HAVE_NEWBUFFER|Py_TPFLAGS_BASETYPE|Py_TPFLAGS_HAVE_GC, /*tp_flags*/
  0, /*tp_doc*/
  __pyx_tp_traverse_4grpc_7_cython_6cygrpc_ConnectivityEvent, /*tp_traverse*/
  __pyx_tp_clear_4grpc_7_cython_6cygrpc_ConnectivityEvent, /*tp_clear*/
  0, /*tp_richcompare*/
  0, /*tp_weaklistoffset*/
  0, /*tp_iter*/
  0, /*tp_iternext*/
  __pyx_methods_4grpc_7_cython_6cygrpc_ConnectivityEvent, /*tp_methods*/
  0, /*tp_members*/
  __pyx_getsets_4grpc_7_cython_6cygrpc_ConnectivityEvent, /*tp_getset*/
  0, /*tp_base*/
  0, /*tp_dict*/
  0, /*tp_descr_get*/
  0, /*tp_descr_set*/
  0, /*tp_dictoffset*/
  0, /*tp_init*/
  0, /*tp_alloc*/
  __pyx_tp_new_4grpc_7_cython_6cygrpc_ConnectivityEvent, /*tp_new*/
  0, /*tp_free*/
  0, /*tp_is_gc*/
  0, /*tp_bases*/
  0, /*tp_mro*/
  0, /*tp_cache*/
  0, /*tp_subclasses*/
  0, /*tp_weaklist*/
  0, /*tp_del*/
  0, /*tp_version_tag*/
  #if PY_VERSION_HEX >= 0x030400a1
  0, /*tp_finalize*/
  #endif
};

static PyObject *__pyx_tp_new_4grpc_7_cython_6cygrpc_RequestCallEvent(PyTypeObject *t, PyObject *a, PyObject *k) {
  struct __pyx_obj_4grpc_7_cython_6cygrpc_RequestCallEvent *p;
  PyObject *o;
  if (likely((t->tp_flags & Py_TPFLAGS_IS_ABSTRACT) == 0)) {
    o = (*t->tp_alloc)(t, 0);
  } else {
    o = (PyObject *) PyBaseObject_Type.tp_new(t, __pyx_empty_tuple, 0);
  }
  if (unlikely(!o)) return 0;
  p = ((struct __pyx_obj_4grpc_7_cython_6cygrpc_RequestCallEvent *)o);
  p->tag = Py_None; Py_INCREF(Py_None);
  p->call = ((struct __pyx_obj_4grpc_7_cython_6cygrpc_Call *)Py_None); Py_INCREF(Py_None);
  p->call_details = ((struct __pyx_obj_4grpc_7_cython_6cygrpc_CallDetails *)Py_None); Py_INCREF(Py_None);
  p->invocation_metadata = ((PyObject*)Py_None); Py_INCREF(Py_None);
  if (unlikely(__pyx_pw_4grpc_7_cython_6cygrpc_16RequestCallEvent_1__cinit__(o, a, k) < 0)) goto bad;
  return o;
  bad:
  Py_DECREF(o); o = 0;
  return NULL;
}

static void __pyx_tp_dealloc_4grpc_7_cython_6cygrpc_RequestCallEvent(PyObject *o) {
  struct __pyx_obj_4grpc_7_cython_6cygrpc_RequestCallEvent *p = (struct __pyx_obj_4grpc_7_cython_6cygrpc_RequestCallEvent *)o;
  #if CYTHON_USE_TP_FINALIZE
  if (unlikely(PyType_HasFeature(Py_TYPE(o), Py_TPFLAGS_HAVE_FINALIZE) && Py_TYPE(o)->tp_finalize) && !_PyGC_FINALIZED(o)) {
    if (PyObject_CallFinalizerFromDealloc(o)) return;
  }
  #endif
  PyObject_GC_UnTrack(o);
  Py_CLEAR(p->tag);
  Py_CLEAR(p->call);
  Py_CLEAR(p->call_details);
  Py_CLEAR(p->invocation_metadata);
  (*Py_TYPE(o)->tp_free)(o);
}

static int __pyx_tp_traverse_4grpc_7_cython_6cygrpc_RequestCallEvent(PyObject *o, visitproc v, void *a) {
  int e;
  struct __pyx_obj_4grpc_7_cython_6cygrpc_RequestCallEvent *p = (struct __pyx_obj_4grpc_7_cython_6cygrpc_RequestCallEvent *)o;
  if (p->tag) {
    e = (*v)(p->tag, a); if (e) return e;
  }
  if (p->call) {
    e = (*v)(((PyObject *)p->call), a); if (e) return e;
  }
  if (p->call_details) {
    e = (*v)(((PyObject *)p->call_details), a); if (e) return e;
  }
  if (p->invocation_metadata) {
    e = (*v)(p->invocation_metadata, a); if (e) return e;
  }
  return 0;
}

static int __pyx_tp_clear_4grpc_7_cython_6cygrpc_RequestCallEvent(PyObject *o) {
  PyObject* tmp;
  struct __pyx_obj_4grpc_7_cython_6cygrpc_RequestCallEvent *p = (struct __pyx_obj_4grpc_7_cython_6cygrpc_RequestCallEvent *)o;
  tmp = ((PyObject*)p->tag);
  p->tag = Py_None; Py_INCREF(Py_None);
  Py_XDECREF(tmp);
  tmp = ((PyObject*)p->call);
  p->call = ((struct __pyx_obj_4grpc_7_cython_6cygrpc_Call *)Py_None); Py_INCREF(Py_None);
  Py_XDECREF(tmp);
  tmp = ((PyObject*)p->call_details);
  p->call_details = ((struct __pyx_obj_4grpc_7_cython_6cygrpc_CallDetails *)Py_None); Py_INCREF(Py_None);
  Py_XDECREF(tmp);
  tmp = ((PyObject*)p->invocation_metadata);
  p->invocation_metadata = ((PyObject*)Py_None); Py_INCREF(Py_None);
  Py_XDECREF(tmp);
  return 0;
}

static PyObject *__pyx_getprop_4grpc_7_cython_6cygrpc_16RequestCallEvent_completion_type(PyObject *o, CYTHON_UNUSED void *x) {
  return __pyx_pw_4grpc_7_cython_6cygrpc_16RequestCallEvent_15completion_type_1__get__(o);
}

static PyObject *__pyx_getprop_4grpc_7_cython_6cygrpc_16RequestCallEvent_success(PyObject *o, CYTHON_UNUSED void *x) {
  return __pyx_pw_4grpc_7_cython_6cygrpc_16RequestCallEvent_7success_1__get__(o);
}

static PyObject *__pyx_getprop_4grpc_7_cython_6cygrpc_16RequestCallEvent_tag(PyObject *o, CYTHON_UNUSED void *x) {
  return __pyx_pw_4grpc_7_cython_6cygrpc_16RequestCallEvent_3tag_1__get__(o);
}

static PyObject *__pyx_getprop_4grpc_7_cython_6cygrpc_16RequestCallEvent_call(PyObject *o, CYTHON_UNUSED void *x) {
  return __pyx_pw_4grpc_7_cython_6cygrpc_16RequestCallEvent_4call_1__get__(o);
}

static PyObject *__pyx_getprop_4grpc_7_cython_6cygrpc_16RequestCallEvent_call_details(PyObject *o, CYTHON_UNUSED void *x) {
  return __pyx_pw_4grpc_7_cython_6cygrpc_16RequestCallEvent_12call_details_1__get__(o);
}

static PyObject *__pyx_getprop_4grpc_7_cython_6cygrpc_16RequestCallEvent_invocation_metadata(PyObject *o, CYTHON_UNUSED void *x) {
  return __pyx_pw_4grpc_7_cython_6cygrpc_16RequestCallEvent_19invocation_metadata_1__get__(o);
}

static PyMethodDef __pyx_methods_4grpc_7_cython_6cygrpc_RequestCallEvent[] = {
  {"__reduce_cython__", (PyCFunction)__pyx_pw_4grpc_7_cython_6cygrpc_16RequestCallEvent_3__reduce_cython__, METH_NOARGS, 0},
  {"__setstate_cython__", (PyCFunction)__pyx_pw_4grpc_7_cython_6cygrpc_16RequestCallEvent_5__setstate_cython__, METH_O, 0},
  {0, 0, 0, 0}
};

static struct PyGetSetDef __pyx_getsets_4grpc_7_cython_6cygrpc_RequestCallEvent[] = {
  {(char *)"completion_type", __pyx_getprop_4grpc_7_cython_6cygrpc_16RequestCallEvent_completion_type, 0, (char *)0, 0},
  {(char *)"success", __pyx_getprop_4grpc_7_cython_6cygrpc_16RequestCallEvent_success, 0, (char *)0, 0},
  {(char *)"tag", __pyx_getprop_4grpc_7_cython_6cygrpc_16RequestCallEvent_tag, 0, (char *)0, 0},
  {(char *)"call", __pyx_getprop_4grpc_7_cython_6cygrpc_16RequestCallEvent_call, 0, (char *)0, 0},
  {(char *)"call_details", __pyx_getprop_4grpc_7_cython_6cygrpc_16RequestCallEvent_call_details, 0, (char *)0, 0},
  {(char *)"invocation_metadata", __pyx_getprop_4grpc_7_cython_6cygrpc_16RequestCallEvent_invocation_metadata, 0, (char *)0, 0},
  {0, 0, 0, 0, 0}
};

static PyTypeObject __pyx_type_4grpc_7_cython_6cygrpc_RequestCallEvent = {
  PyVarObject_HEAD_INIT(0, 0)
  "grpc._cython.cygrpc.RequestCallEvent", /*tp_name*/
  sizeof(struct __pyx_obj_4grpc_7_cython_6cygrpc_RequestCallEvent), /*tp_basicsize*/
  0, /*tp_itemsize*/
  __pyx_tp_dealloc_4grpc_7_cython_6cygrpc_RequestCallEvent, /*tp_dealloc*/
  0, /*tp_print*/
  0, /*tp_getattr*/
  0, /*tp_setattr*/
  #if PY_MAJOR_VERSION < 3
  0, /*tp_compare*/
  #endif
  #if PY_MAJOR_VERSION >= 3
  0, /*tp_as_async*/
  #endif
  0, /*tp_repr*/
  0, /*tp_as_number*/
  0, /*tp_as_sequence*/
  0, /*tp_as_mapping*/
  0, /*tp_hash*/
  0, /*tp_call*/
  0, /*tp_str*/
  0, /*tp_getattro*/
  0, /*tp_setattro*/
  0, /*tp_as_buffer*/
  Py_TPFLAGS_DEFAULT|Py_TPFLAGS_HAVE_VERSION_TAG|Py_TPFLAGS_CHECKTYPES|Py_TPFLAGS_HAVE_NEWBUFFER|Py_TPFLAGS_BASETYPE|Py_TPFLAGS_HAVE_GC, /*tp_flags*/
  0, /*tp_doc*/
  __pyx_tp_traverse_4grpc_7_cython_6cygrpc_RequestCallEvent, /*tp_traverse*/
  __pyx_tp_clear_4grpc_7_cython_6cygrpc_RequestCallEvent, /*tp_clear*/
  0, /*tp_richcompare*/
  0, /*tp_weaklistoffset*/
  0, /*tp_iter*/
  0, /*tp_iternext*/
  __pyx_methods_4grpc_7_cython_6cygrpc_RequestCallEvent, /*tp_methods*/
  0, /*tp_members*/
  __pyx_getsets_4grpc_7_cython_6cygrpc_RequestCallEvent, /*tp_getset*/
  0, /*tp_base*/
  0, /*tp_dict*/
  0, /*tp_descr_get*/
  0, /*tp_descr_set*/
  0, /*tp_dictoffset*/
  0, /*tp_init*/
  0, /*tp_alloc*/
  __pyx_tp_new_4grpc_7_cython_6cygrpc_RequestCallEvent, /*tp_new*/
  0, /*tp_free*/
  0, /*tp_is_gc*/
  0, /*tp_bases*/
  0, /*tp_mro*/
  0, /*tp_cache*/
  0, /*tp_subclasses*/
  0, /*tp_weaklist*/
  0, /*tp_del*/
  0, /*tp_version_tag*/
  #if PY_VERSION_HEX >= 0x030400a1
  0, /*tp_finalize*/
  #endif
};

static PyObject *__pyx_tp_new_4grpc_7_cython_6cygrpc_BatchOperationEvent(PyTypeObject *t, PyObject *a, PyObject *k) {
  struct __pyx_obj_4grpc_7_cython_6cygrpc_BatchOperationEvent *p;
  PyObject *o;
  if (likely((t->tp_flags & Py_TPFLAGS_IS_ABSTRACT) == 0)) {
    o = (*t->tp_alloc)(t, 0);
  } else {
    o = (PyObject *) PyBaseObject_Type.tp_new(t, __pyx_empty_tuple, 0);
  }
  if (unlikely(!o)) return 0;
  p = ((struct __pyx_obj_4grpc_7_cython_6cygrpc_BatchOperationEvent *)o);
  p->tag = Py_None; Py_INCREF(Py_None);
  p->batch_operations = Py_None; Py_INCREF(Py_None);
  if (unlikely(__pyx_pw_4grpc_7_cython_6cygrpc_19BatchOperationEvent_1__cinit__(o, a, k) < 0)) goto bad;
  return o;
  bad:
  Py_DECREF(o); o = 0;
  return NULL;
}

static void __pyx_tp_dealloc_4grpc_7_cython_6cygrpc_BatchOperationEvent(PyObject *o) {
  struct __pyx_obj_4grpc_7_cython_6cygrpc_BatchOperationEvent *p = (struct __pyx_obj_4grpc_7_cython_6cygrpc_BatchOperationEvent *)o;
  #if CYTHON_USE_TP_FINALIZE
  if (unlikely(PyType_HasFeature(Py_TYPE(o), Py_TPFLAGS_HAVE_FINALIZE) && Py_TYPE(o)->tp_finalize) && !_PyGC_FINALIZED(o)) {
    if (PyObject_CallFinalizerFromDealloc(o)) return;
  }
  #endif
  PyObject_GC_UnTrack(o);
  Py_CLEAR(p->tag);
  Py_CLEAR(p->batch_operations);
  (*Py_TYPE(o)->tp_free)(o);
}

static int __pyx_tp_traverse_4grpc_7_cython_6cygrpc_BatchOperationEvent(PyObject *o, visitproc v, void *a) {
  int e;
  struct __pyx_obj_4grpc_7_cython_6cygrpc_BatchOperationEvent *p = (struct __pyx_obj_4grpc_7_cython_6cygrpc_BatchOperationEvent *)o;
  if (p->tag) {
    e = (*v)(p->tag, a); if (e) return e;
  }
  if (p->batch_operations) {
    e = (*v)(p->batch_operations, a); if (e) return e;
  }
  return 0;
}

static int __pyx_tp_clear_4grpc_7_cython_6cygrpc_BatchOperationEvent(PyObject *o) {
  PyObject* tmp;
  struct __pyx_obj_4grpc_7_cython_6cygrpc_BatchOperationEvent *p = (struct __pyx_obj_4grpc_7_cython_6cygrpc_BatchOperationEvent *)o;
  tmp = ((PyObject*)p->tag);
  p->tag = Py_None; Py_INCREF(Py_None);
  Py_XDECREF(tmp);
  tmp = ((PyObject*)p->batch_operations);
  p->batch_operations = Py_None; Py_INCREF(Py_None);
  Py_XDECREF(tmp);
  return 0;
}

static PyObject *__pyx_getprop_4grpc_7_cython_6cygrpc_19BatchOperationEvent_completion_type(PyObject *o, CYTHON_UNUSED void *x) {
  return __pyx_pw_4grpc_7_cython_6cygrpc_19BatchOperationEvent_15completion_type_1__get__(o);
}

static PyObject *__pyx_getprop_4grpc_7_cython_6cygrpc_19BatchOperationEvent_success(PyObject *o, CYTHON_UNUSED void *x) {
  return __pyx_pw_4grpc_7_cython_6cygrpc_19BatchOperationEvent_7success_1__get__(o);
}

static PyObject *__pyx_getprop_4grpc_7_cython_6cygrpc_19BatchOperationEvent_tag(PyObject *o, CYTHON_UNUSED void *x) {
  return __pyx_pw_4grpc_7_cython_6cygrpc_19BatchOperationEvent_3tag_1__get__(o);
}

static PyObject *__pyx_getprop_4grpc_7_cython_6cygrpc_19BatchOperationEvent_batch_operations(PyObject *o, CYTHON_UNUSED void *x) {
  return __pyx_pw_4grpc_7_cython_6cygrpc_19BatchOperationEvent_16batch_operations_1__get__(o);
}

static PyMethodDef __pyx_methods_4grpc_7_cython_6cygrpc_BatchOperationEvent[] = {
  {"__reduce_cython__", (PyCFunction)__pyx_pw_4grpc_7_cython_6cygrpc_19BatchOperationEvent_3__reduce_cython__, METH_NOARGS, 0},
  {"__setstate_cython__", (PyCFunction)__pyx_pw_4grpc_7_cython_6cygrpc_19BatchOperationEvent_5__setstate_cython__, METH_O, 0},
  {0, 0, 0, 0}
};

static struct PyGetSetDef __pyx_getsets_4grpc_7_cython_6cygrpc_BatchOperationEvent[] = {
  {(char *)"completion_type", __pyx_getprop_4grpc_7_cython_6cygrpc_19BatchOperationEvent_completion_type, 0, (char *)0, 0},
  {(char *)"success", __pyx_getprop_4grpc_7_cython_6cygrpc_19BatchOperationEvent_success, 0, (char *)0, 0},
  {(char *)"tag", __pyx_getprop_4grpc_7_cython_6cygrpc_19BatchOperationEvent_tag, 0, (char *)0, 0},
  {(char *)"batch_operations", __pyx_getprop_4grpc_7_cython_6cygrpc_19BatchOperationEvent_batch_operations, 0, (char *)0, 0},
  {0, 0, 0, 0, 0}
};

static PyTypeObject __pyx_type_4grpc_7_cython_6cygrpc_BatchOperationEvent = {
  PyVarObject_HEAD_INIT(0, 0)
  "grpc._cython.cygrpc.BatchOperationEvent", /*tp_name*/
  sizeof(struct __pyx_obj_4grpc_7_cython_6cygrpc_BatchOperationEvent), /*tp_basicsize*/
  0, /*tp_itemsize*/
  __pyx_tp_dealloc_4grpc_7_cython_6cygrpc_BatchOperationEvent, /*tp_dealloc*/
  0, /*tp_print*/
  0, /*tp_getattr*/
  0, /*tp_setattr*/
  #if PY_MAJOR_VERSION < 3
  0, /*tp_compare*/
  #endif
  #if PY_MAJOR_VERSION >= 3
  0, /*tp_as_async*/
  #endif
  0, /*tp_repr*/
  0, /*tp_as_number*/
  0, /*tp_as_sequence*/
  0, /*tp_as_mapping*/
  0, /*tp_hash*/
  0, /*tp_call*/
  0, /*tp_str*/
  0, /*tp_getattro*/
  0, /*tp_setattro*/
  0, /*tp_as_buffer*/
  Py_TPFLAGS_DEFAULT|Py_TPFLAGS_HAVE_VERSION_TAG|Py_TPFLAGS_CHECKTYPES|Py_TPFLAGS_HAVE_NEWBUFFER|Py_TPFLAGS_BASETYPE|Py_TPFLAGS_HAVE_GC, /*tp_flags*/
  0, /*tp_doc*/
  __pyx_tp_traverse_4grpc_7_cython_6cygrpc_BatchOperationEvent, /*tp_traverse*/
  __pyx_tp_clear_4grpc_7_cython_6cygrpc_BatchOperationEvent, /*tp_clear*/
  0, /*tp_richcompare*/
  0, /*tp_weaklistoffset*/
  0, /*tp_iter*/
  0, /*tp_iternext*/
  __pyx_methods_4grpc_7_cython_6cygrpc_BatchOperationEvent, /*tp_methods*/
  0, /*tp_members*/
  __pyx_getsets_4grpc_7_cython_6cygrpc_BatchOperationEvent, /*tp_getset*/
  0, /*tp_base*/
  0, /*tp_dict*/
  0, /*tp_descr_get*/
  0, /*tp_descr_set*/
  0, /*tp_dictoffset*/
  0, /*tp_init*/
  0, /*tp_alloc*/
  __pyx_tp_new_4grpc_7_cython_6cygrpc_BatchOperationEvent, /*tp_new*/
  0, /*tp_free*/
  0, /*tp_is_gc*/
  0, /*tp_bases*/
  0, /*tp_mro*/
  0, /*tp_cache*/
  0, /*tp_subclasses*/
  0, /*tp_weaklist*/
  0, /*tp_del*/
  0, /*tp_version_tag*/
  #if PY_VERSION_HEX >= 0x030400a1
  0, /*tp_finalize*/
  #endif
};

static PyObject *__pyx_tp_new_4grpc_7_cython_6cygrpc_ServerShutdownEvent(PyTypeObject *t, PyObject *a, PyObject *k) {
  struct __pyx_obj_4grpc_7_cython_6cygrpc_ServerShutdownEvent *p;
  PyObject *o;
  if (likely((t->tp_flags & Py_TPFLAGS_IS_ABSTRACT) == 0)) {
    o = (*t->tp_alloc)(t, 0);
  } else {
    o = (PyObject *) PyBaseObject_Type.tp_new(t, __pyx_empty_tuple, 0);
  }
  if (unlikely(!o)) return 0;
  p = ((struct __pyx_obj_4grpc_7_cython_6cygrpc_ServerShutdownEvent *)o);
  p->tag = Py_None; Py_INCREF(Py_None);
  if (unlikely(__pyx_pw_4grpc_7_cython_6cygrpc_19ServerShutdownEvent_1__cinit__(o, a, k) < 0)) goto bad;
  return o;
  bad:
  Py_DECREF(o); o = 0;
  return NULL;
}

static void __pyx_tp_dealloc_4grpc_7_cython_6cygrpc_ServerShutdownEvent(PyObject *o) {
  struct __pyx_obj_4grpc_7_cython_6cygrpc_ServerShutdownEvent *p = (struct __pyx_obj_4grpc_7_cython_6cygrpc_ServerShutdownEvent *)o;
  #if CYTHON_USE_TP_FINALIZE
  if (unlikely(PyType_HasFeature(Py_TYPE(o), Py_TPFLAGS_HAVE_FINALIZE) && Py_TYPE(o)->tp_finalize) && !_PyGC_FINALIZED(o)) {
    if (PyObject_CallFinalizerFromDealloc(o)) return;
  }
  #endif
  PyObject_GC_UnTrack(o);
  Py_CLEAR(p->tag);
  (*Py_TYPE(o)->tp_free)(o);
}

static int __pyx_tp_traverse_4grpc_7_cython_6cygrpc_ServerShutdownEvent(PyObject *o, visitproc v, void *a) {
  int e;
  struct __pyx_obj_4grpc_7_cython_6cygrpc_ServerShutdownEvent *p = (struct __pyx_obj_4grpc_7_cython_6cygrpc_ServerShutdownEvent *)o;
  if (p->tag) {
    e = (*v)(p->tag, a); if (e) return e;
  }
  return 0;
}

static int __pyx_tp_clear_4grpc_7_cython_6cygrpc_ServerShutdownEvent(PyObject *o) {
  PyObject* tmp;
  struct __pyx_obj_4grpc_7_cython_6cygrpc_ServerShutdownEvent *p = (struct __pyx_obj_4grpc_7_cython_6cygrpc_ServerShutdownEvent *)o;
  tmp = ((PyObject*)p->tag);
  p->tag = Py_None; Py_INCREF(Py_None);
  Py_XDECREF(tmp);
  return 0;
}

static PyObject *__pyx_getprop_4grpc_7_cython_6cygrpc_19ServerShutdownEvent_completion_type(PyObject *o, CYTHON_UNUSED void *x) {
  return __pyx_pw_4grpc_7_cython_6cygrpc_19ServerShutdownEvent_15completion_type_1__get__(o);
}

static PyObject *__pyx_getprop_4grpc_7_cython_6cygrpc_19ServerShutdownEvent_success(PyObject *o, CYTHON_UNUSED void *x) {
  return __pyx_pw_4grpc_7_cython_6cygrpc_19ServerShutdownEvent_7success_1__get__(o);
}

static PyObject *__pyx_getprop_4grpc_7_cython_6cygrpc_19ServerShutdownEvent_tag(PyObject *o, CYTHON_UNUSED void *x) {
  return __pyx_pw_4grpc_7_cython_6cygrpc_19ServerShutdownEvent_3tag_1__get__(o);
}

static PyMethodDef __pyx_methods_4grpc_7_cython_6cygrpc_ServerShutdownEvent[] = {
  {"__reduce_cython__", (PyCFunction)__pyx_pw_4grpc_7_cython_6cygrpc_19ServerShutdownEvent_3__reduce_cython__, METH_NOARGS, 0},
  {"__setstate_cython__", (PyCFunction)__pyx_pw_4grpc_7_cython_6cygrpc_19ServerShutdownEvent_5__setstate_cython__, METH_O, 0},
  {0, 0, 0, 0}
};

static struct PyGetSetDef __pyx_getsets_4grpc_7_cython_6cygrpc_ServerShutdownEvent[] = {
  {(char *)"completion_type", __pyx_getprop_4grpc_7_cython_6cygrpc_19ServerShutdownEvent_completion_type, 0, (char *)0, 0},
  {(char *)"success", __pyx_getprop_4grpc_7_cython_6cygrpc_19ServerShutdownEvent_success, 0, (char *)0, 0},
  {(char *)"tag", __pyx_getprop_4grpc_7_cython_6cygrpc_19ServerShutdownEvent_tag, 0, (char *)0, 0},
  {0, 0, 0, 0, 0}
};

static PyTypeObject __pyx_type_4grpc_7_cython_6cygrpc_ServerShutdownEvent = {
  PyVarObject_HEAD_INIT(0, 0)
  "grpc._cython.cygrpc.ServerShutdownEvent", /*tp_name*/
  sizeof(struct __pyx_obj_4grpc_7_cython_6cygrpc_ServerShutdownEvent), /*tp_basicsize*/
  0, /*tp_itemsize*/
  __pyx_tp_dealloc_4grpc_7_cython_6cygrpc_ServerShutdownEvent, /*tp_dealloc*/
  0, /*tp_print*/
  0, /*tp_getattr*/
  0, /*tp_setattr*/
  #if PY_MAJOR_VERSION < 3
  0, /*tp_compare*/
  #endif
  #if PY_MAJOR_VERSION >= 3
  0, /*tp_as_async*/
  #endif
  0, /*tp_repr*/
  0, /*tp_as_number*/
  0, /*tp_as_sequence*/
  0, /*tp_as_mapping*/
  0, /*tp_hash*/
  0, /*tp_call*/
  0, /*tp_str*/
  0, /*tp_getattro*/
  0, /*tp_setattro*/
  0, /*tp_as_buffer*/
  Py_TPFLAGS_DEFAULT|Py_TPFLAGS_HAVE_VERSION_TAG|Py_TPFLAGS_CHECKTYPES|Py_TPFLAGS_HAVE_NEWBUFFER|Py_TPFLAGS_BASETYPE|Py_TPFLAGS_HAVE_GC, /*tp_flags*/
  0, /*tp_doc*/
  __pyx_tp_traverse_4grpc_7_cython_6cygrpc_ServerShutdownEvent, /*tp_traverse*/
  __pyx_tp_clear_4grpc_7_cython_6cygrpc_ServerShutdownEvent, /*tp_clear*/
  0, /*tp_richcompare*/
  0, /*tp_weaklistoffset*/
  0, /*tp_iter*/
  0, /*tp_iternext*/
  __pyx_methods_4grpc_7_cython_6cygrpc_ServerShutdownEvent, /*tp_methods*/
  0, /*tp_members*/
  __pyx_getsets_4grpc_7_cython_6cygrpc_ServerShutdownEvent, /*tp_getset*/
  0, /*tp_base*/
  0, /*tp_dict*/
  0, /*tp_descr_get*/
  0, /*tp_descr_set*/
  0, /*tp_dictoffset*/
  0, /*tp_init*/
  0, /*tp_alloc*/
  __pyx_tp_new_4grpc_7_cython_6cygrpc_ServerShutdownEvent, /*tp_new*/
  0, /*tp_free*/
  0, /*tp_is_gc*/
  0, /*tp_bases*/
  0, /*tp_mro*/
  0, /*tp_cache*/
  0, /*tp_subclasses*/
  0, /*tp_weaklist*/
  0, /*tp_del*/
  0, /*tp_version_tag*/
  #if PY_VERSION_HEX >= 0x030400a1
  0, /*tp_finalize*/
  #endif
};
static struct __pyx_vtabstruct_4grpc_7_cython_6cygrpc_Operation __pyx_vtable_4grpc_7_cython_6cygrpc_Operation;

static PyObject *__pyx_tp_new_4grpc_7_cython_6cygrpc_Operation(PyTypeObject *t, CYTHON_UNUSED PyObject *a, CYTHON_UNUSED PyObject *k) {
  struct __pyx_obj_4grpc_7_cython_6cygrpc_Operation *p;
  PyObject *o;
  if (likely((t->tp_flags & Py_TPFLAGS_IS_ABSTRACT) == 0)) {
    o = (*t->tp_alloc)(t, 0);
  } else {
    o = (PyObject *) PyBaseObject_Type.tp_new(t, __pyx_empty_tuple, 0);
  }
  if (unlikely(!o)) return 0;
  p = ((struct __pyx_obj_4grpc_7_cython_6cygrpc_Operation *)o);
  p->__pyx_vtab = __pyx_vtabptr_4grpc_7_cython_6cygrpc_Operation;
  return o;
}

static void __pyx_tp_dealloc_4grpc_7_cython_6cygrpc_Operation(PyObject *o) {
  #if CYTHON_USE_TP_FINALIZE
  if (unlikely(PyType_HasFeature(Py_TYPE(o), Py_TPFLAGS_HAVE_FINALIZE) && Py_TYPE(o)->tp_finalize) && (!PyType_IS_GC(Py_TYPE(o)) || !_PyGC_FINALIZED(o))) {
    if (PyObject_CallFinalizerFromDealloc(o)) return;
  }
  #endif
  (*Py_TYPE(o)->tp_free)(o);
}

static PyMethodDef __pyx_methods_4grpc_7_cython_6cygrpc_Operation[] = {
  {"__reduce_cython__", (PyCFunction)__pyx_pw_4grpc_7_cython_6cygrpc_9Operation_1__reduce_cython__, METH_NOARGS, 0},
  {"__setstate_cython__", (PyCFunction)__pyx_pw_4grpc_7_cython_6cygrpc_9Operation_3__setstate_cython__, METH_O, 0},
  {0, 0, 0, 0}
};

static PyTypeObject __pyx_type_4grpc_7_cython_6cygrpc_Operation = {
  PyVarObject_HEAD_INIT(0, 0)
  "grpc._cython.cygrpc.Operation", /*tp_name*/
  sizeof(struct __pyx_obj_4grpc_7_cython_6cygrpc_Operation), /*tp_basicsize*/
  0, /*tp_itemsize*/
  __pyx_tp_dealloc_4grpc_7_cython_6cygrpc_Operation, /*tp_dealloc*/
  0, /*tp_print*/
  0, /*tp_getattr*/
  0, /*tp_setattr*/
  #if PY_MAJOR_VERSION < 3
  0, /*tp_compare*/
  #endif
  #if PY_MAJOR_VERSION >= 3
  0, /*tp_as_async*/
  #endif
  0, /*tp_repr*/
  0, /*tp_as_number*/
  0, /*tp_as_sequence*/
  0, /*tp_as_mapping*/
  0, /*tp_hash*/
  0, /*tp_call*/
  0, /*tp_str*/
  0, /*tp_getattro*/
  0, /*tp_setattro*/
  0, /*tp_as_buffer*/
  Py_TPFLAGS_DEFAULT|Py_TPFLAGS_HAVE_VERSION_TAG|Py_TPFLAGS_CHECKTYPES|Py_TPFLAGS_HAVE_NEWBUFFER|Py_TPFLAGS_BASETYPE, /*tp_flags*/
  0, /*tp_doc*/
  0, /*tp_traverse*/
  0, /*tp_clear*/
  0, /*tp_richcompare*/
  0, /*tp_weaklistoffset*/
  0, /*tp_iter*/
  0, /*tp_iternext*/
  __pyx_methods_4grpc_7_cython_6cygrpc_Operation, /*tp_methods*/
  0, /*tp_members*/
  0, /*tp_getset*/
  0, /*tp_base*/
  0, /*tp_dict*/
  0, /*tp_descr_get*/
  0, /*tp_descr_set*/
  0, /*tp_dictoffset*/
  0, /*tp_init*/
  0, /*tp_alloc*/
  __pyx_tp_new_4grpc_7_cython_6cygrpc_Operation, /*tp_new*/
  0, /*tp_free*/
  0, /*tp_is_gc*/
  0, /*tp_bases*/
  0, /*tp_mro*/
  0, /*tp_cache*/
  0, /*tp_subclasses*/
  0, /*tp_weaklist*/
  0, /*tp_del*/
  0, /*tp_version_tag*/
  #if PY_VERSION_HEX >= 0x030400a1
  0, /*tp_finalize*/
  #endif
};
static struct __pyx_vtabstruct_4grpc_7_cython_6cygrpc_SendInitialMetadataOperation __pyx_vtable_4grpc_7_cython_6cygrpc_SendInitialMetadataOperation;

static PyObject *__pyx_tp_new_4grpc_7_cython_6cygrpc_SendInitialMetadataOperation(PyTypeObject *t, PyObject *a, PyObject *k) {
  struct __pyx_obj_4grpc_7_cython_6cygrpc_SendInitialMetadataOperation *p;
  PyObject *o = __pyx_tp_new_4grpc_7_cython_6cygrpc_Operation(t, a, k);
  if (unlikely(!o)) return 0;
  p = ((struct __pyx_obj_4grpc_7_cython_6cygrpc_SendInitialMetadataOperation *)o);
  p->__pyx_base.__pyx_vtab = (struct __pyx_vtabstruct_4grpc_7_cython_6cygrpc_Operation*)__pyx_vtabptr_4grpc_7_cython_6cygrpc_SendInitialMetadataOperation;
  p->_initial_metadata = Py_None; Py_INCREF(Py_None);
  if (unlikely(__pyx_pw_4grpc_7_cython_6cygrpc_28SendInitialMetadataOperation_1__cinit__(o, a, k) < 0)) goto bad;
  return o;
  bad:
  Py_DECREF(o); o = 0;
  return NULL;
}

static void __pyx_tp_dealloc_4grpc_7_cython_6cygrpc_SendInitialMetadataOperation(PyObject *o) {
  struct __pyx_obj_4grpc_7_cython_6cygrpc_SendInitialMetadataOperation *p = (struct __pyx_obj_4grpc_7_cython_6cygrpc_SendInitialMetadataOperation *)o;
  #if CYTHON_USE_TP_FINALIZE
  if (unlikely(PyType_HasFeature(Py_TYPE(o), Py_TPFLAGS_HAVE_FINALIZE) && Py_TYPE(o)->tp_finalize) && !_PyGC_FINALIZED(o)) {
    if (PyObject_CallFinalizerFromDealloc(o)) return;
  }
  #endif
  PyObject_GC_UnTrack(o);
  Py_CLEAR(p->_initial_metadata);
  #if CYTHON_USE_TYPE_SLOTS
  if (PyType_IS_GC(Py_TYPE(o)->tp_base))
  #endif
  PyObject_GC_Track(o);
  __pyx_tp_dealloc_4grpc_7_cython_6cygrpc_Operation(o);
}

static int __pyx_tp_traverse_4grpc_7_cython_6cygrpc_SendInitialMetadataOperation(PyObject *o, visitproc v, void *a) {
  int e;
  struct __pyx_obj_4grpc_7_cython_6cygrpc_SendInitialMetadataOperation *p = (struct __pyx_obj_4grpc_7_cython_6cygrpc_SendInitialMetadataOperation *)o;
  e = ((likely(__pyx_ptype_4grpc_7_cython_6cygrpc_Operation)) ? ((__pyx_ptype_4grpc_7_cython_6cygrpc_Operation->tp_traverse) ? __pyx_ptype_4grpc_7_cython_6cygrpc_Operation->tp_traverse(o, v, a) : 0) : __Pyx_call_next_tp_traverse(o, v, a, __pyx_tp_traverse_4grpc_7_cython_6cygrpc_SendInitialMetadataOperation)); if (e) return e;
  if (p->_initial_metadata) {
    e = (*v)(p->_initial_metadata, a); if (e) return e;
  }
  return 0;
}

static int __pyx_tp_clear_4grpc_7_cython_6cygrpc_SendInitialMetadataOperation(PyObject *o) {
  PyObject* tmp;
  struct __pyx_obj_4grpc_7_cython_6cygrpc_SendInitialMetadataOperation *p = (struct __pyx_obj_4grpc_7_cython_6cygrpc_SendInitialMetadataOperation *)o;
  if (likely(__pyx_ptype_4grpc_7_cython_6cygrpc_Operation)) { if (__pyx_ptype_4grpc_7_cython_6cygrpc_Operation->tp_clear) __pyx_ptype_4grpc_7_cython_6cygrpc_Operation->tp_clear(o); } else __Pyx_call_next_tp_clear(o, __pyx_tp_clear_4grpc_7_cython_6cygrpc_SendInitialMetadataOperation);
  tmp = ((PyObject*)p->_initial_metadata);
  p->_initial_metadata = Py_None; Py_INCREF(Py_None);
  Py_XDECREF(tmp);
  return 0;
}

static PyObject *__pyx_getprop_4grpc_7_cython_6cygrpc_28SendInitialMetadataOperation__initial_metadata(PyObject *o, CYTHON_UNUSED void *x) {
  return __pyx_pw_4grpc_7_cython_6cygrpc_28SendInitialMetadataOperation_17_initial_metadata_1__get__(o);
}

static PyObject *__pyx_getprop_4grpc_7_cython_6cygrpc_28SendInitialMetadataOperation__flags(PyObject *o, CYTHON_UNUSED void *x) {
  return __pyx_pw_4grpc_7_cython_6cygrpc_28SendInitialMetadataOperation_6_flags_1__get__(o);
}

static PyMethodDef __pyx_methods_4grpc_7_cython_6cygrpc_SendInitialMetadataOperation[] = {
  {"type", (PyCFunction)__pyx_pw_4grpc_7_cython_6cygrpc_28SendInitialMetadataOperation_3type, METH_NOARGS, 0},
  {"__reduce_cython__", (PyCFunction)__pyx_pw_4grpc_7_cython_6cygrpc_28SendInitialMetadataOperation_5__reduce_cython__, METH_NOARGS, 0},
  {"__setstate_cython__", (PyCFunction)__pyx_pw_4grpc_7_cython_6cygrpc_28SendInitialMetadataOperation_7__setstate_cython__, METH_O, 0},
  {0, 0, 0, 0}
};

static struct PyGetSetDef __pyx_getsets_4grpc_7_cython_6cygrpc_SendInitialMetadataOperation[] = {
  {(char *)"_initial_metadata", __pyx_getprop_4grpc_7_cython_6cygrpc_28SendInitialMetadataOperation__initial_metadata, 0, (char *)0, 0},
  {(char *)"_flags", __pyx_getprop_4grpc_7_cython_6cygrpc_28SendInitialMetadataOperation__flags, 0, (char *)0, 0},
  {0, 0, 0, 0, 0}
};

static PyTypeObject __pyx_type_4grpc_7_cython_6cygrpc_SendInitialMetadataOperation = {
  PyVarObject_HEAD_INIT(0, 0)
  "grpc._cython.cygrpc.SendInitialMetadataOperation", /*tp_name*/
  sizeof(struct __pyx_obj_4grpc_7_cython_6cygrpc_SendInitialMetadataOperation), /*tp_basicsize*/
  0, /*tp_itemsize*/
  __pyx_tp_dealloc_4grpc_7_cython_6cygrpc_SendInitialMetadataOperation, /*tp_dealloc*/
  0, /*tp_print*/
  0, /*tp_getattr*/
  0, /*tp_setattr*/
  #if PY_MAJOR_VERSION < 3
  0, /*tp_compare*/
  #endif
  #if PY_MAJOR_VERSION >= 3
  0, /*tp_as_async*/
  #endif
  0, /*tp_repr*/
  0, /*tp_as_number*/
  0, /*tp_as_sequence*/
  0, /*tp_as_mapping*/
  0, /*tp_hash*/
  0, /*tp_call*/
  0, /*tp_str*/
  0, /*tp_getattro*/
  0, /*tp_setattro*/
  0, /*tp_as_buffer*/
  Py_TPFLAGS_DEFAULT|Py_TPFLAGS_HAVE_VERSION_TAG|Py_TPFLAGS_CHECKTYPES|Py_TPFLAGS_HAVE_NEWBUFFER|Py_TPFLAGS_BASETYPE|Py_TPFLAGS_HAVE_GC, /*tp_flags*/
  0, /*tp_doc*/
  __pyx_tp_traverse_4grpc_7_cython_6cygrpc_SendInitialMetadataOperation, /*tp_traverse*/
  __pyx_tp_clear_4grpc_7_cython_6cygrpc_SendInitialMetadataOperation, /*tp_clear*/
  0, /*tp_richcompare*/
  0, /*tp_weaklistoffset*/
  0, /*tp_iter*/
  0, /*tp_iternext*/
  __pyx_methods_4grpc_7_cython_6cygrpc_SendInitialMetadataOperation, /*tp_methods*/
  0, /*tp_members*/
  __pyx_getsets_4grpc_7_cython_6cygrpc_SendInitialMetadataOperation, /*tp_getset*/
  0, /*tp_base*/
  0, /*tp_dict*/
  0, /*tp_descr_get*/
  0, /*tp_descr_set*/
  0, /*tp_dictoffset*/
  0, /*tp_init*/
  0, /*tp_alloc*/
  __pyx_tp_new_4grpc_7_cython_6cygrpc_SendInitialMetadataOperation, /*tp_new*/
  0, /*tp_free*/
  0, /*tp_is_gc*/
  0, /*tp_bases*/
  0, /*tp_mro*/
  0, /*tp_cache*/
  0, /*tp_subclasses*/
  0, /*tp_weaklist*/
  0, /*tp_del*/
  0, /*tp_version_tag*/
  #if PY_VERSION_HEX >= 0x030400a1
  0, /*tp_finalize*/
  #endif
};
static struct __pyx_vtabstruct_4grpc_7_cython_6cygrpc_SendMessageOperation __pyx_vtable_4grpc_7_cython_6cygrpc_SendMessageOperation;

static PyObject *__pyx_tp_new_4grpc_7_cython_6cygrpc_SendMessageOperation(PyTypeObject *t, PyObject *a, PyObject *k) {
  struct __pyx_obj_4grpc_7_cython_6cygrpc_SendMessageOperation *p;
  PyObject *o = __pyx_tp_new_4grpc_7_cython_6cygrpc_Operation(t, a, k);
  if (unlikely(!o)) return 0;
  p = ((struct __pyx_obj_4grpc_7_cython_6cygrpc_SendMessageOperation *)o);
  p->__pyx_base.__pyx_vtab = (struct __pyx_vtabstruct_4grpc_7_cython_6cygrpc_Operation*)__pyx_vtabptr_4grpc_7_cython_6cygrpc_SendMessageOperation;
  p->_message = ((PyObject*)Py_None); Py_INCREF(Py_None);
  if (unlikely(__pyx_pw_4grpc_7_cython_6cygrpc_20SendMessageOperation_1__cinit__(o, a, k) < 0)) goto bad;
  return o;
  bad:
  Py_DECREF(o); o = 0;
  return NULL;
}

static void __pyx_tp_dealloc_4grpc_7_cython_6cygrpc_SendMessageOperation(PyObject *o) {
  struct __pyx_obj_4grpc_7_cython_6cygrpc_SendMessageOperation *p = (struct __pyx_obj_4grpc_7_cython_6cygrpc_SendMessageOperation *)o;
  #if CYTHON_USE_TP_FINALIZE
  if (unlikely(PyType_HasFeature(Py_TYPE(o), Py_TPFLAGS_HAVE_FINALIZE) && Py_TYPE(o)->tp_finalize) && (!PyType_IS_GC(Py_TYPE(o)) || !_PyGC_FINALIZED(o))) {
    if (PyObject_CallFinalizerFromDealloc(o)) return;
  }
  #endif
  Py_CLEAR(p->_message);
  __pyx_tp_dealloc_4grpc_7_cython_6cygrpc_Operation(o);
}

static PyObject *__pyx_getprop_4grpc_7_cython_6cygrpc_20SendMessageOperation__message(PyObject *o, CYTHON_UNUSED void *x) {
  return __pyx_pw_4grpc_7_cython_6cygrpc_20SendMessageOperation_8_message_1__get__(o);
}

static PyObject *__pyx_getprop_4grpc_7_cython_6cygrpc_20SendMessageOperation__flags(PyObject *o, CYTHON_UNUSED void *x) {
  return __pyx_pw_4grpc_7_cython_6cygrpc_20SendMessageOperation_6_flags_1__get__(o);
}

static PyMethodDef __pyx_methods_4grpc_7_cython_6cygrpc_SendMessageOperation[] = {
  {"type", (PyCFunction)__pyx_pw_4grpc_7_cython_6cygrpc_20SendMessageOperation_3type, METH_NOARGS, 0},
  {"__reduce_cython__", (PyCFunction)__pyx_pw_4grpc_7_cython_6cygrpc_20SendMessageOperation_5__reduce_cython__, METH_NOARGS, 0},
  {"__setstate_cython__", (PyCFunction)__pyx_pw_4grpc_7_cython_6cygrpc_20SendMessageOperation_7__setstate_cython__, METH_O, 0},
  {0, 0, 0, 0}
};

static struct PyGetSetDef __pyx_getsets_4grpc_7_cython_6cygrpc_SendMessageOperation[] = {
  {(char *)"_message", __pyx_getprop_4grpc_7_cython_6cygrpc_20SendMessageOperation__message, 0, (char *)0, 0},
  {(char *)"_flags", __pyx_getprop_4grpc_7_cython_6cygrpc_20SendMessageOperation__flags, 0, (char *)0, 0},
  {0, 0, 0, 0, 0}
};

static PyTypeObject __pyx_type_4grpc_7_cython_6cygrpc_SendMessageOperation = {
  PyVarObject_HEAD_INIT(0, 0)
  "grpc._cython.cygrpc.SendMessageOperation", /*tp_name*/
  sizeof(struct __pyx_obj_4grpc_7_cython_6cygrpc_SendMessageOperation), /*tp_basicsize*/
  0, /*tp_itemsize*/
  __pyx_tp_dealloc_4grpc_7_cython_6cygrpc_SendMessageOperation, /*tp_dealloc*/
  0, /*tp_print*/
  0, /*tp_getattr*/
  0, /*tp_setattr*/
  #if PY_MAJOR_VERSION < 3
  0, /*tp_compare*/
  #endif
  #if PY_MAJOR_VERSION >= 3
  0, /*tp_as_async*/
  #endif
  0, /*tp_repr*/
  0, /*tp_as_number*/
  0, /*tp_as_sequence*/
  0, /*tp_as_mapping*/
  0, /*tp_hash*/
  0, /*tp_call*/
  0, /*tp_str*/
  0, /*tp_getattro*/
  0, /*tp_setattro*/
  0, /*tp_as_buffer*/
  Py_TPFLAGS_DEFAULT|Py_TPFLAGS_HAVE_VERSION_TAG|Py_TPFLAGS_CHECKTYPES|Py_TPFLAGS_HAVE_NEWBUFFER|Py_TPFLAGS_BASETYPE, /*tp_flags*/
  0, /*tp_doc*/
  0, /*tp_traverse*/
  0, /*tp_clear*/
  0, /*tp_richcompare*/
  0, /*tp_weaklistoffset*/
  0, /*tp_iter*/
  0, /*tp_iternext*/
  __pyx_methods_4grpc_7_cython_6cygrpc_SendMessageOperation, /*tp_methods*/
  0, /*tp_members*/
  __pyx_getsets_4grpc_7_cython_6cygrpc_SendMessageOperation, /*tp_getset*/
  0, /*tp_base*/
  0, /*tp_dict*/
  0, /*tp_descr_get*/
  0, /*tp_descr_set*/
  0, /*tp_dictoffset*/
  0, /*tp_init*/
  0, /*tp_alloc*/
  __pyx_tp_new_4grpc_7_cython_6cygrpc_SendMessageOperation, /*tp_new*/
  0, /*tp_free*/
  0, /*tp_is_gc*/
  0, /*tp_bases*/
  0, /*tp_mro*/
  0, /*tp_cache*/
  0, /*tp_subclasses*/
  0, /*tp_weaklist*/
  0, /*tp_del*/
  0, /*tp_version_tag*/
  #if PY_VERSION_HEX >= 0x030400a1
  0, /*tp_finalize*/
  #endif
};
static struct __pyx_vtabstruct_4grpc_7_cython_6cygrpc_SendCloseFromClientOperation __pyx_vtable_4grpc_7_cython_6cygrpc_SendCloseFromClientOperation;

static PyObject *__pyx_tp_new_4grpc_7_cython_6cygrpc_SendCloseFromClientOperation(PyTypeObject *t, PyObject *a, PyObject *k) {
  struct __pyx_obj_4grpc_7_cython_6cygrpc_SendCloseFromClientOperation *p;
  PyObject *o = __pyx_tp_new_4grpc_7_cython_6cygrpc_Operation(t, a, k);
  if (unlikely(!o)) return 0;
  p = ((struct __pyx_obj_4grpc_7_cython_6cygrpc_SendCloseFromClientOperation *)o);
  p->__pyx_base.__pyx_vtab = (struct __pyx_vtabstruct_4grpc_7_cython_6cygrpc_Operation*)__pyx_vtabptr_4grpc_7_cython_6cygrpc_SendCloseFromClientOperation;
  if (unlikely(__pyx_pw_4grpc_7_cython_6cygrpc_28SendCloseFromClientOperation_1__cinit__(o, a, k) < 0)) goto bad;
  return o;
  bad:
  Py_DECREF(o); o = 0;
  return NULL;
}

static PyObject *__pyx_getprop_4grpc_7_cython_6cygrpc_28SendCloseFromClientOperation__flags(PyObject *o, CYTHON_UNUSED void *x) {
  return __pyx_pw_4grpc_7_cython_6cygrpc_28SendCloseFromClientOperation_6_flags_1__get__(o);
}

static PyMethodDef __pyx_methods_4grpc_7_cython_6cygrpc_SendCloseFromClientOperation[] = {
  {"type", (PyCFunction)__pyx_pw_4grpc_7_cython_6cygrpc_28SendCloseFromClientOperation_3type, METH_NOARGS, 0},
  {"__reduce_cython__", (PyCFunction)__pyx_pw_4grpc_7_cython_6cygrpc_28SendCloseFromClientOperation_5__reduce_cython__, METH_NOARGS, 0},
  {"__setstate_cython__", (PyCFunction)__pyx_pw_4grpc_7_cython_6cygrpc_28SendCloseFromClientOperation_7__setstate_cython__, METH_O, 0},
  {0, 0, 0, 0}
};

static struct PyGetSetDef __pyx_getsets_4grpc_7_cython_6cygrpc_SendCloseFromClientOperation[] = {
  {(char *)"_flags", __pyx_getprop_4grpc_7_cython_6cygrpc_28SendCloseFromClientOperation__flags, 0, (char *)0, 0},
  {0, 0, 0, 0, 0}
};

static PyTypeObject __pyx_type_4grpc_7_cython_6cygrpc_SendCloseFromClientOperation = {
  PyVarObject_HEAD_INIT(0, 0)
  "grpc._cython.cygrpc.SendCloseFromClientOperation", /*tp_name*/
  sizeof(struct __pyx_obj_4grpc_7_cython_6cygrpc_SendCloseFromClientOperation), /*tp_basicsize*/
  0, /*tp_itemsize*/
  __pyx_tp_dealloc_4grpc_7_cython_6cygrpc_Operation, /*tp_dealloc*/
  0, /*tp_print*/
  0, /*tp_getattr*/
  0, /*tp_setattr*/
  #if PY_MAJOR_VERSION < 3
  0, /*tp_compare*/
  #endif
  #if PY_MAJOR_VERSION >= 3
  0, /*tp_as_async*/
  #endif
  0, /*tp_repr*/
  0, /*tp_as_number*/
  0, /*tp_as_sequence*/
  0, /*tp_as_mapping*/
  0, /*tp_hash*/
  0, /*tp_call*/
  0, /*tp_str*/
  0, /*tp_getattro*/
  0, /*tp_setattro*/
  0, /*tp_as_buffer*/
  Py_TPFLAGS_DEFAULT|Py_TPFLAGS_HAVE_VERSION_TAG|Py_TPFLAGS_CHECKTYPES|Py_TPFLAGS_HAVE_NEWBUFFER|Py_TPFLAGS_BASETYPE, /*tp_flags*/
  0, /*tp_doc*/
  0, /*tp_traverse*/
  0, /*tp_clear*/
  0, /*tp_richcompare*/
  0, /*tp_weaklistoffset*/
  0, /*tp_iter*/
  0, /*tp_iternext*/
  __pyx_methods_4grpc_7_cython_6cygrpc_SendCloseFromClientOperation, /*tp_methods*/
  0, /*tp_members*/
  __pyx_getsets_4grpc_7_cython_6cygrpc_SendCloseFromClientOperation, /*tp_getset*/
  0, /*tp_base*/
  0, /*tp_dict*/
  0, /*tp_descr_get*/
  0, /*tp_descr_set*/
  0, /*tp_dictoffset*/
  0, /*tp_init*/
  0, /*tp_alloc*/
  __pyx_tp_new_4grpc_7_cython_6cygrpc_SendCloseFromClientOperation, /*tp_new*/
  0, /*tp_free*/
  0, /*tp_is_gc*/
  0, /*tp_bases*/
  0, /*tp_mro*/
  0, /*tp_cache*/
  0, /*tp_subclasses*/
  0, /*tp_weaklist*/
  0, /*tp_del*/
  0, /*tp_version_tag*/
  #if PY_VERSION_HEX >= 0x030400a1
  0, /*tp_finalize*/
  #endif
};
static struct __pyx_vtabstruct_4grpc_7_cython_6cygrpc_SendStatusFromServerOperation __pyx_vtable_4grpc_7_cython_6cygrpc_SendStatusFromServerOperation;

static PyObject *__pyx_tp_new_4grpc_7_cython_6cygrpc_SendStatusFromServerOperation(PyTypeObject *t, PyObject *a, PyObject *k) {
  struct __pyx_obj_4grpc_7_cython_6cygrpc_SendStatusFromServerOperation *p;
  PyObject *o = __pyx_tp_new_4grpc_7_cython_6cygrpc_Operation(t, a, k);
  if (unlikely(!o)) return 0;
  p = ((struct __pyx_obj_4grpc_7_cython_6cygrpc_SendStatusFromServerOperation *)o);
  p->__pyx_base.__pyx_vtab = (struct __pyx_vtabstruct_4grpc_7_cython_6cygrpc_Operation*)__pyx_vtabptr_4grpc_7_cython_6cygrpc_SendStatusFromServerOperation;
  p->_trailing_metadata = Py_None; Py_INCREF(Py_None);
  p->_code = Py_None; Py_INCREF(Py_None);
  p->_details = Py_None; Py_INCREF(Py_None);
  if (unlikely(__pyx_pw_4grpc_7_cython_6cygrpc_29SendStatusFromServerOperation_1__cinit__(o, a, k) < 0)) goto bad;
  return o;
  bad:
  Py_DECREF(o); o = 0;
  return NULL;
}

static void __pyx_tp_dealloc_4grpc_7_cython_6cygrpc_SendStatusFromServerOperation(PyObject *o) {
  struct __pyx_obj_4grpc_7_cython_6cygrpc_SendStatusFromServerOperation *p = (struct __pyx_obj_4grpc_7_cython_6cygrpc_SendStatusFromServerOperation *)o;
  #if CYTHON_USE_TP_FINALIZE
  if (unlikely(PyType_HasFeature(Py_TYPE(o), Py_TPFLAGS_HAVE_FINALIZE) && Py_TYPE(o)->tp_finalize) && !_PyGC_FINALIZED(o)) {
    if (PyObject_CallFinalizerFromDealloc(o)) return;
  }
  #endif
  PyObject_GC_UnTrack(o);
  Py_CLEAR(p->_trailing_metadata);
  Py_CLEAR(p->_code);
  Py_CLEAR(p->_details);
  #if CYTHON_USE_TYPE_SLOTS
  if (PyType_IS_GC(Py_TYPE(o)->tp_base))
  #endif
  PyObject_GC_Track(o);
  __pyx_tp_dealloc_4grpc_7_cython_6cygrpc_Operation(o);
}

static int __pyx_tp_traverse_4grpc_7_cython_6cygrpc_SendStatusFromServerOperation(PyObject *o, visitproc v, void *a) {
  int e;
  struct __pyx_obj_4grpc_7_cython_6cygrpc_SendStatusFromServerOperation *p = (struct __pyx_obj_4grpc_7_cython_6cygrpc_SendStatusFromServerOperation *)o;
  e = ((likely(__pyx_ptype_4grpc_7_cython_6cygrpc_Operation)) ? ((__pyx_ptype_4grpc_7_cython_6cygrpc_Operation->tp_traverse) ? __pyx_ptype_4grpc_7_cython_6cygrpc_Operation->tp_traverse(o, v, a) : 0) : __Pyx_call_next_tp_traverse(o, v, a, __pyx_tp_traverse_4grpc_7_cython_6cygrpc_SendStatusFromServerOperation)); if (e) return e;
  if (p->_trailing_metadata) {
    e = (*v)(p->_trailing_metadata, a); if (e) return e;
  }
  if (p->_code) {
    e = (*v)(p->_code, a); if (e) return e;
  }
  if (p->_details) {
    e = (*v)(p->_details, a); if (e) return e;
  }
  return 0;
}

static int __pyx_tp_clear_4grpc_7_cython_6cygrpc_SendStatusFromServerOperation(PyObject *o) {
  PyObject* tmp;
  struct __pyx_obj_4grpc_7_cython_6cygrpc_SendStatusFromServerOperation *p = (struct __pyx_obj_4grpc_7_cython_6cygrpc_SendStatusFromServerOperation *)o;
  if (likely(__pyx_ptype_4grpc_7_cython_6cygrpc_Operation)) { if (__pyx_ptype_4grpc_7_cython_6cygrpc_Operation->tp_clear) __pyx_ptype_4grpc_7_cython_6cygrpc_Operation->tp_clear(o); } else __Pyx_call_next_tp_clear(o, __pyx_tp_clear_4grpc_7_cython_6cygrpc_SendStatusFromServerOperation);
  tmp = ((PyObject*)p->_trailing_metadata);
  p->_trailing_metadata = Py_None; Py_INCREF(Py_None);
  Py_XDECREF(tmp);
  tmp = ((PyObject*)p->_code);
  p->_code = Py_None; Py_INCREF(Py_None);
  Py_XDECREF(tmp);
  tmp = ((PyObject*)p->_details);
  p->_details = Py_None; Py_INCREF(Py_None);
  Py_XDECREF(tmp);
  return 0;
}

static PyObject *__pyx_getprop_4grpc_7_cython_6cygrpc_29SendStatusFromServerOperation__trailing_metadata(PyObject *o, CYTHON_UNUSED void *x) {
  return __pyx_pw_4grpc_7_cython_6cygrpc_29SendStatusFromServerOperation_18_trailing_metadata_1__get__(o);
}

static PyObject *__pyx_getprop_4grpc_7_cython_6cygrpc_29SendStatusFromServerOperation__code(PyObject *o, CYTHON_UNUSED void *x) {
  return __pyx_pw_4grpc_7_cython_6cygrpc_29SendStatusFromServerOperation_5_code_1__get__(o);
}

static PyObject *__pyx_getprop_4grpc_7_cython_6cygrpc_29SendStatusFromServerOperation__details(PyObject *o, CYTHON_UNUSED void *x) {
  return __pyx_pw_4grpc_7_cython_6cygrpc_29SendStatusFromServerOperation_8_details_1__get__(o);
}

static PyObject *__pyx_getprop_4grpc_7_cython_6cygrpc_29SendStatusFromServerOperation__flags(PyObject *o, CYTHON_UNUSED void *x) {
  return __pyx_pw_4grpc_7_cython_6cygrpc_29SendStatusFromServerOperation_6_flags_1__get__(o);
}

static PyMethodDef __pyx_methods_4grpc_7_cython_6cygrpc_SendStatusFromServerOperation[] = {
  {"type", (PyCFunction)__pyx_pw_4grpc_7_cython_6cygrpc_29SendStatusFromServerOperation_3type, METH_NOARGS, 0},
  {"__reduce_cython__", (PyCFunction)__pyx_pw_4grpc_7_cython_6cygrpc_29SendStatusFromServerOperation_5__reduce_cython__, METH_NOARGS, 0},
  {"__setstate_cython__", (PyCFunction)__pyx_pw_4grpc_7_cython_6cygrpc_29SendStatusFromServerOperation_7__setstate_cython__, METH_O, 0},
  {0, 0, 0, 0}
};

static struct PyGetSetDef __pyx_getsets_4grpc_7_cython_6cygrpc_SendStatusFromServerOperation[] = {
  {(char *)"_trailing_metadata", __pyx_getprop_4grpc_7_cython_6cygrpc_29SendStatusFromServerOperation__trailing_metadata, 0, (char *)0, 0},
  {(char *)"_code", __pyx_getprop_4grpc_7_cython_6cygrpc_29SendStatusFromServerOperation__code, 0, (char *)0, 0},
  {(char *)"_details", __pyx_getprop_4grpc_7_cython_6cygrpc_29SendStatusFromServerOperation__details, 0, (char *)0, 0},
  {(char *)"_flags", __pyx_getprop_4grpc_7_cython_6cygrpc_29SendStatusFromServerOperation__flags, 0, (char *)0, 0},
  {0, 0, 0, 0, 0}
};

static PyTypeObject __pyx_type_4grpc_7_cython_6cygrpc_SendStatusFromServerOperation = {
  PyVarObject_HEAD_INIT(0, 0)
  "grpc._cython.cygrpc.SendStatusFromServerOperation", /*tp_name*/
  sizeof(struct __pyx_obj_4grpc_7_cython_6cygrpc_SendStatusFromServerOperation), /*tp_basicsize*/
  0, /*tp_itemsize*/
  __pyx_tp_dealloc_4grpc_7_cython_6cygrpc_SendStatusFromServerOperation, /*tp_dealloc*/
  0, /*tp_print*/
  0, /*tp_getattr*/
  0, /*tp_setattr*/
  #if PY_MAJOR_VERSION < 3
  0, /*tp_compare*/
  #endif
  #if PY_MAJOR_VERSION >= 3
  0, /*tp_as_async*/
  #endif
  0, /*tp_repr*/
  0, /*tp_as_number*/
  0, /*tp_as_sequence*/
  0, /*tp_as_mapping*/
  0, /*tp_hash*/
  0, /*tp_call*/
  0, /*tp_str*/
  0, /*tp_getattro*/
  0, /*tp_setattro*/
  0, /*tp_as_buffer*/
  Py_TPFLAGS_DEFAULT|Py_TPFLAGS_HAVE_VERSION_TAG|Py_TPFLAGS_CHECKTYPES|Py_TPFLAGS_HAVE_NEWBUFFER|Py_TPFLAGS_BASETYPE|Py_TPFLAGS_HAVE_GC, /*tp_flags*/
  0, /*tp_doc*/
  __pyx_tp_traverse_4grpc_7_cython_6cygrpc_SendStatusFromServerOperation, /*tp_traverse*/
  __pyx_tp_clear_4grpc_7_cython_6cygrpc_SendStatusFromServerOperation, /*tp_clear*/
  0, /*tp_richcompare*/
  0, /*tp_weaklistoffset*/
  0, /*tp_iter*/
  0, /*tp_iternext*/
  __pyx_methods_4grpc_7_cython_6cygrpc_SendStatusFromServerOperation, /*tp_methods*/
  0, /*tp_members*/
  __pyx_getsets_4grpc_7_cython_6cygrpc_SendStatusFromServerOperation, /*tp_getset*/
  0, /*tp_base*/
  0, /*tp_dict*/
  0, /*tp_descr_get*/
  0, /*tp_descr_set*/
  0, /*tp_dictoffset*/
  0, /*tp_init*/
  0, /*tp_alloc*/
  __pyx_tp_new_4grpc_7_cython_6cygrpc_SendStatusFromServerOperation, /*tp_new*/
  0, /*tp_free*/
  0, /*tp_is_gc*/
  0, /*tp_bases*/
  0, /*tp_mro*/
  0, /*tp_cache*/
  0, /*tp_subclasses*/
  0, /*tp_weaklist*/
  0, /*tp_del*/
  0, /*tp_version_tag*/
  #if PY_VERSION_HEX >= 0x030400a1
  0, /*tp_finalize*/
  #endif
};
static struct __pyx_vtabstruct_4grpc_7_cython_6cygrpc_ReceiveInitialMetadataOperation __pyx_vtable_4grpc_7_cython_6cygrpc_ReceiveInitialMetadataOperation;

static PyObject *__pyx_tp_new_4grpc_7_cython_6cygrpc_ReceiveInitialMetadataOperation(PyTypeObject *t, PyObject *a, PyObject *k) {
  struct __pyx_obj_4grpc_7_cython_6cygrpc_ReceiveInitialMetadataOperation *p;
  PyObject *o = __pyx_tp_new_4grpc_7_cython_6cygrpc_Operation(t, a, k);
  if (unlikely(!o)) return 0;
  p = ((struct __pyx_obj_4grpc_7_cython_6cygrpc_ReceiveInitialMetadataOperation *)o);
  p->__pyx_base.__pyx_vtab = (struct __pyx_vtabstruct_4grpc_7_cython_6cygrpc_Operation*)__pyx_vtabptr_4grpc_7_cython_6cygrpc_ReceiveInitialMetadataOperation;
  p->_initial_metadata = ((PyObject*)Py_None); Py_INCREF(Py_None);
  if (unlikely(__pyx_pw_4grpc_7_cython_6cygrpc_31ReceiveInitialMetadataOperation_1__cinit__(o, a, k) < 0)) goto bad;
  return o;
  bad:
  Py_DECREF(o); o = 0;
  return NULL;
}

static void __pyx_tp_dealloc_4grpc_7_cython_6cygrpc_ReceiveInitialMetadataOperation(PyObject *o) {
  struct __pyx_obj_4grpc_7_cython_6cygrpc_ReceiveInitialMetadataOperation *p = (struct __pyx_obj_4grpc_7_cython_6cygrpc_ReceiveInitialMetadataOperation *)o;
  #if CYTHON_USE_TP_FINALIZE
  if (unlikely(PyType_HasFeature(Py_TYPE(o), Py_TPFLAGS_HAVE_FINALIZE) && Py_TYPE(o)->tp_finalize) && !_PyGC_FINALIZED(o)) {
    if (PyObject_CallFinalizerFromDealloc(o)) return;
  }
  #endif
  PyObject_GC_UnTrack(o);
  Py_CLEAR(p->_initial_metadata);
  #if CYTHON_USE_TYPE_SLOTS
  if (PyType_IS_GC(Py_TYPE(o)->tp_base))
  #endif
  PyObject_GC_Track(o);
  __pyx_tp_dealloc_4grpc_7_cython_6cygrpc_Operation(o);
}

static int __pyx_tp_traverse_4grpc_7_cython_6cygrpc_ReceiveInitialMetadataOperation(PyObject *o, visitproc v, void *a) {
  int e;
  struct __pyx_obj_4grpc_7_cython_6cygrpc_ReceiveInitialMetadataOperation *p = (struct __pyx_obj_4grpc_7_cython_6cygrpc_ReceiveInitialMetadataOperation *)o;
  e = ((likely(__pyx_ptype_4grpc_7_cython_6cygrpc_Operation)) ? ((__pyx_ptype_4grpc_7_cython_6cygrpc_Operation->tp_traverse) ? __pyx_ptype_4grpc_7_cython_6cygrpc_Operation->tp_traverse(o, v, a) : 0) : __Pyx_call_next_tp_traverse(o, v, a, __pyx_tp_traverse_4grpc_7_cython_6cygrpc_ReceiveInitialMetadataOperation)); if (e) return e;
  if (p->_initial_metadata) {
    e = (*v)(p->_initial_metadata, a); if (e) return e;
  }
  return 0;
}

static int __pyx_tp_clear_4grpc_7_cython_6cygrpc_ReceiveInitialMetadataOperation(PyObject *o) {
  PyObject* tmp;
  struct __pyx_obj_4grpc_7_cython_6cygrpc_ReceiveInitialMetadataOperation *p = (struct __pyx_obj_4grpc_7_cython_6cygrpc_ReceiveInitialMetadataOperation *)o;
  if (likely(__pyx_ptype_4grpc_7_cython_6cygrpc_Operation)) { if (__pyx_ptype_4grpc_7_cython_6cygrpc_Operation->tp_clear) __pyx_ptype_4grpc_7_cython_6cygrpc_Operation->tp_clear(o); } else __Pyx_call_next_tp_clear(o, __pyx_tp_clear_4grpc_7_cython_6cygrpc_ReceiveInitialMetadataOperation);
  tmp = ((PyObject*)p->_initial_metadata);
  p->_initial_metadata = ((PyObject*)Py_None); Py_INCREF(Py_None);
  Py_XDECREF(tmp);
  return 0;
}

static PyObject *__pyx_getprop_4grpc_7_cython_6cygrpc_31ReceiveInitialMetadataOperation__flags(PyObject *o, CYTHON_UNUSED void *x) {
  return __pyx_pw_4grpc_7_cython_6cygrpc_31ReceiveInitialMetadataOperation_6_flags_1__get__(o);
}

static PyMethodDef __pyx_methods_4grpc_7_cython_6cygrpc_ReceiveInitialMetadataOperation[] = {
  {"type", (PyCFunction)__pyx_pw_4grpc_7_cython_6cygrpc_31ReceiveInitialMetadataOperation_3type, METH_NOARGS, 0},
  {"initial_metadata", (PyCFunction)__pyx_pw_4grpc_7_cython_6cygrpc_31ReceiveInitialMetadataOperation_5initial_metadata, METH_NOARGS, 0},
  {"__reduce_cython__", (PyCFunction)__pyx_pw_4grpc_7_cython_6cygrpc_31ReceiveInitialMetadataOperation_7__reduce_cython__, METH_NOARGS, 0},
  {"__setstate_cython__", (PyCFunction)__pyx_pw_4grpc_7_cython_6cygrpc_31ReceiveInitialMetadataOperation_9__setstate_cython__, METH_O, 0},
  {0, 0, 0, 0}
};

static struct PyGetSetDef __pyx_getsets_4grpc_7_cython_6cygrpc_ReceiveInitialMetadataOperation[] = {
  {(char *)"_flags", __pyx_getprop_4grpc_7_cython_6cygrpc_31ReceiveInitialMetadataOperation__flags, 0, (char *)0, 0},
  {0, 0, 0, 0, 0}
};

static PyTypeObject __pyx_type_4grpc_7_cython_6cygrpc_ReceiveInitialMetadataOperation = {
  PyVarObject_HEAD_INIT(0, 0)
  "grpc._cython.cygrpc.ReceiveInitialMetadataOperation", /*tp_name*/
  sizeof(struct __pyx_obj_4grpc_7_cython_6cygrpc_ReceiveInitialMetadataOperation), /*tp_basicsize*/
  0, /*tp_itemsize*/
  __pyx_tp_dealloc_4grpc_7_cython_6cygrpc_ReceiveInitialMetadataOperation, /*tp_dealloc*/
  0, /*tp_print*/
  0, /*tp_getattr*/
  0, /*tp_setattr*/
  #if PY_MAJOR_VERSION < 3
  0, /*tp_compare*/
  #endif
  #if PY_MAJOR_VERSION >= 3
  0, /*tp_as_async*/
  #endif
  0, /*tp_repr*/
  0, /*tp_as_number*/
  0, /*tp_as_sequence*/
  0, /*tp_as_mapping*/
  0, /*tp_hash*/
  0, /*tp_call*/
  0, /*tp_str*/
  0, /*tp_getattro*/
  0, /*tp_setattro*/
  0, /*tp_as_buffer*/
  Py_TPFLAGS_DEFAULT|Py_TPFLAGS_HAVE_VERSION_TAG|Py_TPFLAGS_CHECKTYPES|Py_TPFLAGS_HAVE_NEWBUFFER|Py_TPFLAGS_BASETYPE|Py_TPFLAGS_HAVE_GC, /*tp_flags*/
  0, /*tp_doc*/
  __pyx_tp_traverse_4grpc_7_cython_6cygrpc_ReceiveInitialMetadataOperation, /*tp_traverse*/
  __pyx_tp_clear_4grpc_7_cython_6cygrpc_ReceiveInitialMetadataOperation, /*tp_clear*/
  0, /*tp_richcompare*/
  0, /*tp_weaklistoffset*/
  0, /*tp_iter*/
  0, /*tp_iternext*/
  __pyx_methods_4grpc_7_cython_6cygrpc_ReceiveInitialMetadataOperation, /*tp_methods*/
  0, /*tp_members*/
  __pyx_getsets_4grpc_7_cython_6cygrpc_ReceiveInitialMetadataOperation, /*tp_getset*/
  0, /*tp_base*/
  0, /*tp_dict*/
  0, /*tp_descr_get*/
  0, /*tp_descr_set*/
  0, /*tp_dictoffset*/
  0, /*tp_init*/
  0, /*tp_alloc*/
  __pyx_tp_new_4grpc_7_cython_6cygrpc_ReceiveInitialMetadataOperation, /*tp_new*/
  0, /*tp_free*/
  0, /*tp_is_gc*/
  0, /*tp_bases*/
  0, /*tp_mro*/
  0, /*tp_cache*/
  0, /*tp_subclasses*/
  0, /*tp_weaklist*/
  0, /*tp_del*/
  0, /*tp_version_tag*/
  #if PY_VERSION_HEX >= 0x030400a1
  0, /*tp_finalize*/
  #endif
};
static struct __pyx_vtabstruct_4grpc_7_cython_6cygrpc_ReceiveMessageOperation __pyx_vtable_4grpc_7_cython_6cygrpc_ReceiveMessageOperation;

static PyObject *__pyx_tp_new_4grpc_7_cython_6cygrpc_ReceiveMessageOperation(PyTypeObject *t, PyObject *a, PyObject *k) {
  struct __pyx_obj_4grpc_7_cython_6cygrpc_ReceiveMessageOperation *p;
  PyObject *o = __pyx_tp_new_4grpc_7_cython_6cygrpc_Operation(t, a, k);
  if (unlikely(!o)) return 0;
  p = ((struct __pyx_obj_4grpc_7_cython_6cygrpc_ReceiveMessageOperation *)o);
  p->__pyx_base.__pyx_vtab = (struct __pyx_vtabstruct_4grpc_7_cython_6cygrpc_Operation*)__pyx_vtabptr_4grpc_7_cython_6cygrpc_ReceiveMessageOperation;
  p->_message = ((PyObject*)Py_None); Py_INCREF(Py_None);
  if (unlikely(__pyx_pw_4grpc_7_cython_6cygrpc_23ReceiveMessageOperation_1__cinit__(o, a, k) < 0)) goto bad;
  return o;
  bad:
  Py_DECREF(o); o = 0;
  return NULL;
}

static void __pyx_tp_dealloc_4grpc_7_cython_6cygrpc_ReceiveMessageOperation(PyObject *o) {
  struct __pyx_obj_4grpc_7_cython_6cygrpc_ReceiveMessageOperation *p = (struct __pyx_obj_4grpc_7_cython_6cygrpc_ReceiveMessageOperation *)o;
  #if CYTHON_USE_TP_FINALIZE
  if (unlikely(PyType_HasFeature(Py_TYPE(o), Py_TPFLAGS_HAVE_FINALIZE) && Py_TYPE(o)->tp_finalize) && (!PyType_IS_GC(Py_TYPE(o)) || !_PyGC_FINALIZED(o))) {
    if (PyObject_CallFinalizerFromDealloc(o)) return;
  }
  #endif
  Py_CLEAR(p->_message);
  __pyx_tp_dealloc_4grpc_7_cython_6cygrpc_Operation(o);
}

static PyObject *__pyx_getprop_4grpc_7_cython_6cygrpc_23ReceiveMessageOperation__flags(PyObject *o, CYTHON_UNUSED void *x) {
  return __pyx_pw_4grpc_7_cython_6cygrpc_23ReceiveMessageOperation_6_flags_1__get__(o);
}

static PyMethodDef __pyx_methods_4grpc_7_cython_6cygrpc_ReceiveMessageOperation[] = {
  {"type", (PyCFunction)__pyx_pw_4grpc_7_cython_6cygrpc_23ReceiveMessageOperation_3type, METH_NOARGS, 0},
  {"message", (PyCFunction)__pyx_pw_4grpc_7_cython_6cygrpc_23ReceiveMessageOperation_5message, METH_NOARGS, 0},
  {"__reduce_cython__", (PyCFunction)__pyx_pw_4grpc_7_cython_6cygrpc_23ReceiveMessageOperation_7__reduce_cython__, METH_NOARGS, 0},
  {"__setstate_cython__", (PyCFunction)__pyx_pw_4grpc_7_cython_6cygrpc_23ReceiveMessageOperation_9__setstate_cython__, METH_O, 0},
  {0, 0, 0, 0}
};

static struct PyGetSetDef __pyx_getsets_4grpc_7_cython_6cygrpc_ReceiveMessageOperation[] = {
  {(char *)"_flags", __pyx_getprop_4grpc_7_cython_6cygrpc_23ReceiveMessageOperation__flags, 0, (char *)0, 0},
  {0, 0, 0, 0, 0}
};

static PyTypeObject __pyx_type_4grpc_7_cython_6cygrpc_ReceiveMessageOperation = {
  PyVarObject_HEAD_INIT(0, 0)
  "grpc._cython.cygrpc.ReceiveMessageOperation", /*tp_name*/
  sizeof(struct __pyx_obj_4grpc_7_cython_6cygrpc_ReceiveMessageOperation), /*tp_basicsize*/
  0, /*tp_itemsize*/
  __pyx_tp_dealloc_4grpc_7_cython_6cygrpc_ReceiveMessageOperation, /*tp_dealloc*/
  0, /*tp_print*/
  0, /*tp_getattr*/
  0, /*tp_setattr*/
  #if PY_MAJOR_VERSION < 3
  0, /*tp_compare*/
  #endif
  #if PY_MAJOR_VERSION >= 3
  0, /*tp_as_async*/
  #endif
  0, /*tp_repr*/
  0, /*tp_as_number*/
  0, /*tp_as_sequence*/
  0, /*tp_as_mapping*/
  0, /*tp_hash*/
  0, /*tp_call*/
  0, /*tp_str*/
  0, /*tp_getattro*/
  0, /*tp_setattro*/
  0, /*tp_as_buffer*/
  Py_TPFLAGS_DEFAULT|Py_TPFLAGS_HAVE_VERSION_TAG|Py_TPFLAGS_CHECKTYPES|Py_TPFLAGS_HAVE_NEWBUFFER|Py_TPFLAGS_BASETYPE, /*tp_flags*/
  0, /*tp_doc*/
  0, /*tp_traverse*/
  0, /*tp_clear*/
  0, /*tp_richcompare*/
  0, /*tp_weaklistoffset*/
  0, /*tp_iter*/
  0, /*tp_iternext*/
  __pyx_methods_4grpc_7_cython_6cygrpc_ReceiveMessageOperation, /*tp_methods*/
  0, /*tp_members*/
  __pyx_getsets_4grpc_7_cython_6cygrpc_ReceiveMessageOperation, /*tp_getset*/
  0, /*tp_base*/
  0, /*tp_dict*/
  0, /*tp_descr_get*/
  0, /*tp_descr_set*/
  0, /*tp_dictoffset*/
  0, /*tp_init*/
  0, /*tp_alloc*/
  __pyx_tp_new_4grpc_7_cython_6cygrpc_ReceiveMessageOperation, /*tp_new*/
  0, /*tp_free*/
  0, /*tp_is_gc*/
  0, /*tp_bases*/
  0, /*tp_mro*/
  0, /*tp_cache*/
  0, /*tp_subclasses*/
  0, /*tp_weaklist*/
  0, /*tp_del*/
  0, /*tp_version_tag*/
  #if PY_VERSION_HEX >= 0x030400a1
  0, /*tp_finalize*/
  #endif
};
static struct __pyx_vtabstruct_4grpc_7_cython_6cygrpc_ReceiveStatusOnClientOperation __pyx_vtable_4grpc_7_cython_6cygrpc_ReceiveStatusOnClientOperation;

static PyObject *__pyx_tp_new_4grpc_7_cython_6cygrpc_ReceiveStatusOnClientOperation(PyTypeObject *t, PyObject *a, PyObject *k) {
  struct __pyx_obj_4grpc_7_cython_6cygrpc_ReceiveStatusOnClientOperation *p;
  PyObject *o = __pyx_tp_new_4grpc_7_cython_6cygrpc_Operation(t, a, k);
  if (unlikely(!o)) return 0;
  p = ((struct __pyx_obj_4grpc_7_cython_6cygrpc_ReceiveStatusOnClientOperation *)o);
  p->__pyx_base.__pyx_vtab = (struct __pyx_vtabstruct_4grpc_7_cython_6cygrpc_Operation*)__pyx_vtabptr_4grpc_7_cython_6cygrpc_ReceiveStatusOnClientOperation;
  p->_trailing_metadata = ((PyObject*)Py_None); Py_INCREF(Py_None);
  p->_code = Py_None; Py_INCREF(Py_None);
  p->_details = ((PyObject*)Py_None); Py_INCREF(Py_None);
  p->_error_string = ((PyObject*)Py_None); Py_INCREF(Py_None);
  if (unlikely(__pyx_pw_4grpc_7_cython_6cygrpc_30ReceiveStatusOnClientOperation_1__cinit__(o, a, k) < 0)) goto bad;
  return o;
  bad:
  Py_DECREF(o); o = 0;
  return NULL;
}

static void __pyx_tp_dealloc_4grpc_7_cython_6cygrpc_ReceiveStatusOnClientOperation(PyObject *o) {
  struct __pyx_obj_4grpc_7_cython_6cygrpc_ReceiveStatusOnClientOperation *p = (struct __pyx_obj_4grpc_7_cython_6cygrpc_ReceiveStatusOnClientOperation *)o;
  #if CYTHON_USE_TP_FINALIZE
  if (unlikely(PyType_HasFeature(Py_TYPE(o), Py_TPFLAGS_HAVE_FINALIZE) && Py_TYPE(o)->tp_finalize) && !_PyGC_FINALIZED(o)) {
    if (PyObject_CallFinalizerFromDealloc(o)) return;
  }
  #endif
  PyObject_GC_UnTrack(o);
  Py_CLEAR(p->_trailing_metadata);
  Py_CLEAR(p->_code);
  Py_CLEAR(p->_details);
  Py_CLEAR(p->_error_string);
  #if CYTHON_USE_TYPE_SLOTS
  if (PyType_IS_GC(Py_TYPE(o)->tp_base))
  #endif
  PyObject_GC_Track(o);
  __pyx_tp_dealloc_4grpc_7_cython_6cygrpc_Operation(o);
}

static int __pyx_tp_traverse_4grpc_7_cython_6cygrpc_ReceiveStatusOnClientOperation(PyObject *o, visitproc v, void *a) {
  int e;
  struct __pyx_obj_4grpc_7_cython_6cygrpc_ReceiveStatusOnClientOperation *p = (struct __pyx_obj_4grpc_7_cython_6cygrpc_ReceiveStatusOnClientOperation *)o;
  e = ((likely(__pyx_ptype_4grpc_7_cython_6cygrpc_Operation)) ? ((__pyx_ptype_4grpc_7_cython_6cygrpc_Operation->tp_traverse) ? __pyx_ptype_4grpc_7_cython_6cygrpc_Operation->tp_traverse(o, v, a) : 0) : __Pyx_call_next_tp_traverse(o, v, a, __pyx_tp_traverse_4grpc_7_cython_6cygrpc_ReceiveStatusOnClientOperation)); if (e) return e;
  if (p->_trailing_metadata) {
    e = (*v)(p->_trailing_metadata, a); if (e) return e;
  }
  if (p->_code) {
    e = (*v)(p->_code, a); if (e) return e;
  }
  return 0;
}

static int __pyx_tp_clear_4grpc_7_cython_6cygrpc_ReceiveStatusOnClientOperation(PyObject *o) {
  PyObject* tmp;
  struct __pyx_obj_4grpc_7_cython_6cygrpc_ReceiveStatusOnClientOperation *p = (struct __pyx_obj_4grpc_7_cython_6cygrpc_ReceiveStatusOnClientOperation *)o;
  if (likely(__pyx_ptype_4grpc_7_cython_6cygrpc_Operation)) { if (__pyx_ptype_4grpc_7_cython_6cygrpc_Operation->tp_clear) __pyx_ptype_4grpc_7_cython_6cygrpc_Operation->tp_clear(o); } else __Pyx_call_next_tp_clear(o, __pyx_tp_clear_4grpc_7_cython_6cygrpc_ReceiveStatusOnClientOperation);
  tmp = ((PyObject*)p->_trailing_metadata);
  p->_trailing_metadata = ((PyObject*)Py_None); Py_INCREF(Py_None);
  Py_XDECREF(tmp);
  tmp = ((PyObject*)p->_code);
  p->_code = Py_None; Py_INCREF(Py_None);
  Py_XDECREF(tmp);
  return 0;
}

static PyObject *__pyx_getprop_4grpc_7_cython_6cygrpc_30ReceiveStatusOnClientOperation__flags(PyObject *o, CYTHON_UNUSED void *x) {
  return __pyx_pw_4grpc_7_cython_6cygrpc_30ReceiveStatusOnClientOperation_6_flags_1__get__(o);
}

static PyMethodDef __pyx_methods_4grpc_7_cython_6cygrpc_ReceiveStatusOnClientOperation[] = {
  {"type", (PyCFunction)__pyx_pw_4grpc_7_cython_6cygrpc_30ReceiveStatusOnClientOperation_3type, METH_NOARGS, 0},
  {"trailing_metadata", (PyCFunction)__pyx_pw_4grpc_7_cython_6cygrpc_30ReceiveStatusOnClientOperation_5trailing_metadata, METH_NOARGS, 0},
  {"code", (PyCFunction)__pyx_pw_4grpc_7_cython_6cygrpc_30ReceiveStatusOnClientOperation_7code, METH_NOARGS, 0},
  {"details", (PyCFunction)__pyx_pw_4grpc_7_cython_6cygrpc_30ReceiveStatusOnClientOperation_9details, METH_NOARGS, 0},
  {"error_string", (PyCFunction)__pyx_pw_4grpc_7_cython_6cygrpc_30ReceiveStatusOnClientOperation_11error_string, METH_NOARGS, 0},
  {"__reduce_cython__", (PyCFunction)__pyx_pw_4grpc_7_cython_6cygrpc_30ReceiveStatusOnClientOperation_13__reduce_cython__, METH_NOARGS, 0},
  {"__setstate_cython__", (PyCFunction)__pyx_pw_4grpc_7_cython_6cygrpc_30ReceiveStatusOnClientOperation_15__setstate_cython__, METH_O, 0},
  {0, 0, 0, 0}
};

static struct PyGetSetDef __pyx_getsets_4grpc_7_cython_6cygrpc_ReceiveStatusOnClientOperation[] = {
  {(char *)"_flags", __pyx_getprop_4grpc_7_cython_6cygrpc_30ReceiveStatusOnClientOperation__flags, 0, (char *)0, 0},
  {0, 0, 0, 0, 0}
};

static PyTypeObject __pyx_type_4grpc_7_cython_6cygrpc_ReceiveStatusOnClientOperation = {
  PyVarObject_HEAD_INIT(0, 0)
  "grpc._cython.cygrpc.ReceiveStatusOnClientOperation", /*tp_name*/
  sizeof(struct __pyx_obj_4grpc_7_cython_6cygrpc_ReceiveStatusOnClientOperation), /*tp_basicsize*/
  0, /*tp_itemsize*/
  __pyx_tp_dealloc_4grpc_7_cython_6cygrpc_ReceiveStatusOnClientOperation, /*tp_dealloc*/
  0, /*tp_print*/
  0, /*tp_getattr*/
  0, /*tp_setattr*/
  #if PY_MAJOR_VERSION < 3
  0, /*tp_compare*/
  #endif
  #if PY_MAJOR_VERSION >= 3
  0, /*tp_as_async*/
  #endif
  0, /*tp_repr*/
  0, /*tp_as_number*/
  0, /*tp_as_sequence*/
  0, /*tp_as_mapping*/
  0, /*tp_hash*/
  0, /*tp_call*/
  0, /*tp_str*/
  0, /*tp_getattro*/
  0, /*tp_setattro*/
  0, /*tp_as_buffer*/
  Py_TPFLAGS_DEFAULT|Py_TPFLAGS_HAVE_VERSION_TAG|Py_TPFLAGS_CHECKTYPES|Py_TPFLAGS_HAVE_NEWBUFFER|Py_TPFLAGS_BASETYPE|Py_TPFLAGS_HAVE_GC, /*tp_flags*/
  0, /*tp_doc*/
  __pyx_tp_traverse_4grpc_7_cython_6cygrpc_ReceiveStatusOnClientOperation, /*tp_traverse*/
  __pyx_tp_clear_4grpc_7_cython_6cygrpc_ReceiveStatusOnClientOperation, /*tp_clear*/
  0, /*tp_richcompare*/
  0, /*tp_weaklistoffset*/
  0, /*tp_iter*/
  0, /*tp_iternext*/
  __pyx_methods_4grpc_7_cython_6cygrpc_ReceiveStatusOnClientOperation, /*tp_methods*/
  0, /*tp_members*/
  __pyx_getsets_4grpc_7_cython_6cygrpc_ReceiveStatusOnClientOperation, /*tp_getset*/
  0, /*tp_base*/
  0, /*tp_dict*/
  0, /*tp_descr_get*/
  0, /*tp_descr_set*/
  0, /*tp_dictoffset*/
  0, /*tp_init*/
  0, /*tp_alloc*/
  __pyx_tp_new_4grpc_7_cython_6cygrpc_ReceiveStatusOnClientOperation, /*tp_new*/
  0, /*tp_free*/
  0, /*tp_is_gc*/
  0, /*tp_bases*/
  0, /*tp_mro*/
  0, /*tp_cache*/
  0, /*tp_subclasses*/
  0, /*tp_weaklist*/
  0, /*tp_del*/
  0, /*tp_version_tag*/
  #if PY_VERSION_HEX >= 0x030400a1
  0, /*tp_finalize*/
  #endif
};
static struct __pyx_vtabstruct_4grpc_7_cython_6cygrpc_ReceiveCloseOnServerOperation __pyx_vtable_4grpc_7_cython_6cygrpc_ReceiveCloseOnServerOperation;

static PyObject *__pyx_tp_new_4grpc_7_cython_6cygrpc_ReceiveCloseOnServerOperation(PyTypeObject *t, PyObject *a, PyObject *k) {
  struct __pyx_obj_4grpc_7_cython_6cygrpc_ReceiveCloseOnServerOperation *p;
  PyObject *o = __pyx_tp_new_4grpc_7_cython_6cygrpc_Operation(t, a, k);
  if (unlikely(!o)) return 0;
  p = ((struct __pyx_obj_4grpc_7_cython_6cygrpc_ReceiveCloseOnServerOperation *)o);
  p->__pyx_base.__pyx_vtab = (struct __pyx_vtabstruct_4grpc_7_cython_6cygrpc_Operation*)__pyx_vtabptr_4grpc_7_cython_6cygrpc_ReceiveCloseOnServerOperation;
  p->_cancelled = Py_None; Py_INCREF(Py_None);
  if (unlikely(__pyx_pw_4grpc_7_cython_6cygrpc_29ReceiveCloseOnServerOperation_1__cinit__(o, a, k) < 0)) goto bad;
  return o;
  bad:
  Py_DECREF(o); o = 0;
  return NULL;
}

static void __pyx_tp_dealloc_4grpc_7_cython_6cygrpc_ReceiveCloseOnServerOperation(PyObject *o) {
  struct __pyx_obj_4grpc_7_cython_6cygrpc_ReceiveCloseOnServerOperation *p = (struct __pyx_obj_4grpc_7_cython_6cygrpc_ReceiveCloseOnServerOperation *)o;
  #if CYTHON_USE_TP_FINALIZE
  if (unlikely(PyType_HasFeature(Py_TYPE(o), Py_TPFLAGS_HAVE_FINALIZE) && Py_TYPE(o)->tp_finalize) && !_PyGC_FINALIZED(o)) {
    if (PyObject_CallFinalizerFromDealloc(o)) return;
  }
  #endif
  PyObject_GC_UnTrack(o);
  Py_CLEAR(p->_cancelled);
  #if CYTHON_USE_TYPE_SLOTS
  if (PyType_IS_GC(Py_TYPE(o)->tp_base))
  #endif
  PyObject_GC_Track(o);
  __pyx_tp_dealloc_4grpc_7_cython_6cygrpc_Operation(o);
}

static int __pyx_tp_traverse_4grpc_7_cython_6cygrpc_ReceiveCloseOnServerOperation(PyObject *o, visitproc v, void *a) {
  int e;
  struct __pyx_obj_4grpc_7_cython_6cygrpc_ReceiveCloseOnServerOperation *p = (struct __pyx_obj_4grpc_7_cython_6cygrpc_ReceiveCloseOnServerOperation *)o;
  e = ((likely(__pyx_ptype_4grpc_7_cython_6cygrpc_Operation)) ? ((__pyx_ptype_4grpc_7_cython_6cygrpc_Operation->tp_traverse) ? __pyx_ptype_4grpc_7_cython_6cygrpc_Operation->tp_traverse(o, v, a) : 0) : __Pyx_call_next_tp_traverse(o, v, a, __pyx_tp_traverse_4grpc_7_cython_6cygrpc_ReceiveCloseOnServerOperation)); if (e) return e;
  if (p->_cancelled) {
    e = (*v)(p->_cancelled, a); if (e) return e;
  }
  return 0;
}

static int __pyx_tp_clear_4grpc_7_cython_6cygrpc_ReceiveCloseOnServerOperation(PyObject *o) {
  PyObject* tmp;
  struct __pyx_obj_4grpc_7_cython_6cygrpc_ReceiveCloseOnServerOperation *p = (struct __pyx_obj_4grpc_7_cython_6cygrpc_ReceiveCloseOnServerOperation *)o;
  if (likely(__pyx_ptype_4grpc_7_cython_6cygrpc_Operation)) { if (__pyx_ptype_4grpc_7_cython_6cygrpc_Operation->tp_clear) __pyx_ptype_4grpc_7_cython_6cygrpc_Operation->tp_clear(o); } else __Pyx_call_next_tp_clear(o, __pyx_tp_clear_4grpc_7_cython_6cygrpc_ReceiveCloseOnServerOperation);
  tmp = ((PyObject*)p->_cancelled);
  p->_cancelled = Py_None; Py_INCREF(Py_None);
  Py_XDECREF(tmp);
  return 0;
}

static PyObject *__pyx_getprop_4grpc_7_cython_6cygrpc_29ReceiveCloseOnServerOperation__flags(PyObject *o, CYTHON_UNUSED void *x) {
  return __pyx_pw_4grpc_7_cython_6cygrpc_29ReceiveCloseOnServerOperation_6_flags_1__get__(o);
}

static PyMethodDef __pyx_methods_4grpc_7_cython_6cygrpc_ReceiveCloseOnServerOperation[] = {
  {"type", (PyCFunction)__pyx_pw_4grpc_7_cython_6cygrpc_29ReceiveCloseOnServerOperation_3type, METH_NOARGS, 0},
  {"cancelled", (PyCFunction)__pyx_pw_4grpc_7_cython_6cygrpc_29ReceiveCloseOnServerOperation_5cancelled, METH_NOARGS, 0},
  {"__reduce_cython__", (PyCFunction)__pyx_pw_4grpc_7_cython_6cygrpc_29ReceiveCloseOnServerOperation_7__reduce_cython__, METH_NOARGS, 0},
  {"__setstate_cython__", (PyCFunction)__pyx_pw_4grpc_7_cython_6cygrpc_29ReceiveCloseOnServerOperation_9__setstate_cython__, METH_O, 0},
  {0, 0, 0, 0}
};

static struct PyGetSetDef __pyx_getsets_4grpc_7_cython_6cygrpc_ReceiveCloseOnServerOperation[] = {
  {(char *)"_flags", __pyx_getprop_4grpc_7_cython_6cygrpc_29ReceiveCloseOnServerOperation__flags, 0, (char *)0, 0},
  {0, 0, 0, 0, 0}
};

static PyTypeObject __pyx_type_4grpc_7_cython_6cygrpc_ReceiveCloseOnServerOperation = {
  PyVarObject_HEAD_INIT(0, 0)
  "grpc._cython.cygrpc.ReceiveCloseOnServerOperation", /*tp_name*/
  sizeof(struct __pyx_obj_4grpc_7_cython_6cygrpc_ReceiveCloseOnServerOperation), /*tp_basicsize*/
  0, /*tp_itemsize*/
  __pyx_tp_dealloc_4grpc_7_cython_6cygrpc_ReceiveCloseOnServerOperation, /*tp_dealloc*/
  0, /*tp_print*/
  0, /*tp_getattr*/
  0, /*tp_setattr*/
  #if PY_MAJOR_VERSION < 3
  0, /*tp_compare*/
  #endif
  #if PY_MAJOR_VERSION >= 3
  0, /*tp_as_async*/
  #endif
  0, /*tp_repr*/
  0, /*tp_as_number*/
  0, /*tp_as_sequence*/
  0, /*tp_as_mapping*/
  0, /*tp_hash*/
  0, /*tp_call*/
  0, /*tp_str*/
  0, /*tp_getattro*/
  0, /*tp_setattro*/
  0, /*tp_as_buffer*/
  Py_TPFLAGS_DEFAULT|Py_TPFLAGS_HAVE_VERSION_TAG|Py_TPFLAGS_CHECKTYPES|Py_TPFLAGS_HAVE_NEWBUFFER|Py_TPFLAGS_BASETYPE|Py_TPFLAGS_HAVE_GC, /*tp_flags*/
  0, /*tp_doc*/
  __pyx_tp_traverse_4grpc_7_cython_6cygrpc_ReceiveCloseOnServerOperation, /*tp_traverse*/
  __pyx_tp_clear_4grpc_7_cython_6cygrpc_ReceiveCloseOnServerOperation, /*tp_clear*/
  0, /*tp_richcompare*/
  0, /*tp_weaklistoffset*/
  0, /*tp_iter*/
  0, /*tp_iternext*/
  __pyx_methods_4grpc_7_cython_6cygrpc_ReceiveCloseOnServerOperation, /*tp_methods*/
  0, /*tp_members*/
  __pyx_getsets_4grpc_7_cython_6cygrpc_ReceiveCloseOnServerOperation, /*tp_getset*/
  0, /*tp_base*/
  0, /*tp_dict*/
  0, /*tp_descr_get*/
  0, /*tp_descr_set*/
  0, /*tp_dictoffset*/
  0, /*tp_init*/
  0, /*tp_alloc*/
  __pyx_tp_new_4grpc_7_cython_6cygrpc_ReceiveCloseOnServerOperation, /*tp_new*/
  0, /*tp_free*/
  0, /*tp_is_gc*/
  0, /*tp_bases*/
  0, /*tp_mro*/
  0, /*tp_cache*/
  0, /*tp_subclasses*/
  0, /*tp_weaklist*/
  0, /*tp_del*/
  0, /*tp_version_tag*/
  #if PY_VERSION_HEX >= 0x030400a1
  0, /*tp_finalize*/
  #endif
};

static PyObject *__pyx_tp_new_4grpc_7_cython_6cygrpc_CallDetails(PyTypeObject *t, CYTHON_UNUSED PyObject *a, CYTHON_UNUSED PyObject *k) {
  PyObject *o;
  if (likely((t->tp_flags & Py_TPFLAGS_IS_ABSTRACT) == 0)) {
    o = (*t->tp_alloc)(t, 0);
  } else {
    o = (PyObject *) PyBaseObject_Type.tp_new(t, __pyx_empty_tuple, 0);
  }
  if (unlikely(!o)) return 0;
  if (unlikely(__pyx_pw_4grpc_7_cython_6cygrpc_11CallDetails_1__cinit__(o, __pyx_empty_tuple, NULL) < 0)) goto bad;
  return o;
  bad:
  Py_DECREF(o); o = 0;
  return NULL;
}

static void __pyx_tp_dealloc_4grpc_7_cython_6cygrpc_CallDetails(PyObject *o) {
  #if CYTHON_USE_TP_FINALIZE
  if (unlikely(PyType_HasFeature(Py_TYPE(o), Py_TPFLAGS_HAVE_FINALIZE) && Py_TYPE(o)->tp_finalize) && (!PyType_IS_GC(Py_TYPE(o)) || !_PyGC_FINALIZED(o))) {
    if (PyObject_CallFinalizerFromDealloc(o)) return;
  }
  #endif
  {
    PyObject *etype, *eval, *etb;
    PyErr_Fetch(&etype, &eval, &etb);
    ++Py_REFCNT(o);
    __pyx_pw_4grpc_7_cython_6cygrpc_11CallDetails_3__dealloc__(o);
    --Py_REFCNT(o);
    PyErr_Restore(etype, eval, etb);
  }
  (*Py_TYPE(o)->tp_free)(o);
}

static PyObject *__pyx_getprop_4grpc_7_cython_6cygrpc_11CallDetails_method(PyObject *o, CYTHON_UNUSED void *x) {
  return __pyx_pw_4grpc_7_cython_6cygrpc_11CallDetails_6method_1__get__(o);
}

static PyObject *__pyx_getprop_4grpc_7_cython_6cygrpc_11CallDetails_host(PyObject *o, CYTHON_UNUSED void *x) {
  return __pyx_pw_4grpc_7_cython_6cygrpc_11CallDetails_4host_1__get__(o);
}

static PyObject *__pyx_getprop_4grpc_7_cython_6cygrpc_11CallDetails_deadline(PyObject *o, CYTHON_UNUSED void *x) {
  return __pyx_pw_4grpc_7_cython_6cygrpc_11CallDetails_8deadline_1__get__(o);
}

static PyMethodDef __pyx_methods_4grpc_7_cython_6cygrpc_CallDetails[] = {
  {"__reduce_cython__", (PyCFunction)__pyx_pw_4grpc_7_cython_6cygrpc_11CallDetails_5__reduce_cython__, METH_NOARGS, 0},
  {"__setstate_cython__", (PyCFunction)__pyx_pw_4grpc_7_cython_6cygrpc_11CallDetails_7__setstate_cython__, METH_O, 0},
  {0, 0, 0, 0}
};

static struct PyGetSetDef __pyx_getsets_4grpc_7_cython_6cygrpc_CallDetails[] = {
  {(char *)"method", __pyx_getprop_4grpc_7_cython_6cygrpc_11CallDetails_method, 0, (char *)0, 0},
  {(char *)"host", __pyx_getprop_4grpc_7_cython_6cygrpc_11CallDetails_host, 0, (char *)0, 0},
  {(char *)"deadline", __pyx_getprop_4grpc_7_cython_6cygrpc_11CallDetails_deadline, 0, (char *)0, 0},
  {0, 0, 0, 0, 0}
};

static PyTypeObject __pyx_type_4grpc_7_cython_6cygrpc_CallDetails = {
  PyVarObject_HEAD_INIT(0, 0)
  "grpc._cython.cygrpc.CallDetails", /*tp_name*/
  sizeof(struct __pyx_obj_4grpc_7_cython_6cygrpc_CallDetails), /*tp_basicsize*/
  0, /*tp_itemsize*/
  __pyx_tp_dealloc_4grpc_7_cython_6cygrpc_CallDetails, /*tp_dealloc*/
  0, /*tp_print*/
  0, /*tp_getattr*/
  0, /*tp_setattr*/
  #if PY_MAJOR_VERSION < 3
  0, /*tp_compare*/
  #endif
  #if PY_MAJOR_VERSION >= 3
  0, /*tp_as_async*/
  #endif
  0, /*tp_repr*/
  0, /*tp_as_number*/
  0, /*tp_as_sequence*/
  0, /*tp_as_mapping*/
  0, /*tp_hash*/
  0, /*tp_call*/
  0, /*tp_str*/
  0, /*tp_getattro*/
  0, /*tp_setattro*/
  0, /*tp_as_buffer*/
  Py_TPFLAGS_DEFAULT|Py_TPFLAGS_HAVE_VERSION_TAG|Py_TPFLAGS_CHECKTYPES|Py_TPFLAGS_HAVE_NEWBUFFER|Py_TPFLAGS_BASETYPE, /*tp_flags*/
  0, /*tp_doc*/
  0, /*tp_traverse*/
  0, /*tp_clear*/
  0, /*tp_richcompare*/
  0, /*tp_weaklistoffset*/
  0, /*tp_iter*/
  0, /*tp_iternext*/
  __pyx_methods_4grpc_7_cython_6cygrpc_CallDetails, /*tp_methods*/
  0, /*tp_members*/
  __pyx_getsets_4grpc_7_cython_6cygrpc_CallDetails, /*tp_getset*/
  0, /*tp_base*/
  0, /*tp_dict*/
  0, /*tp_descr_get*/
  0, /*tp_descr_set*/
  0, /*tp_dictoffset*/
  0, /*tp_init*/
  0, /*tp_alloc*/
  __pyx_tp_new_4grpc_7_cython_6cygrpc_CallDetails, /*tp_new*/
  0, /*tp_free*/
  0, /*tp_is_gc*/
  0, /*tp_bases*/
  0, /*tp_mro*/
  0, /*tp_cache*/
  0, /*tp_subclasses*/
  0, /*tp_weaklist*/
  0, /*tp_del*/
  0, /*tp_version_tag*/
  #if PY_VERSION_HEX >= 0x030400a1
  0, /*tp_finalize*/
  #endif
};

static PyObject *__pyx_tp_new_4grpc_7_cython_6cygrpc_SslPemKeyCertPair(PyTypeObject *t, PyObject *a, PyObject *k) {
  struct __pyx_obj_4grpc_7_cython_6cygrpc_SslPemKeyCertPair *p;
  PyObject *o;
  if (likely((t->tp_flags & Py_TPFLAGS_IS_ABSTRACT) == 0)) {
    o = (*t->tp_alloc)(t, 0);
  } else {
    o = (PyObject *) PyBaseObject_Type.tp_new(t, __pyx_empty_tuple, 0);
  }
  if (unlikely(!o)) return 0;
  p = ((struct __pyx_obj_4grpc_7_cython_6cygrpc_SslPemKeyCertPair *)o);
  p->private_key = Py_None; Py_INCREF(Py_None);
  p->certificate_chain = Py_None; Py_INCREF(Py_None);
  if (unlikely(__pyx_pw_4grpc_7_cython_6cygrpc_17SslPemKeyCertPair_1__cinit__(o, a, k) < 0)) goto bad;
  return o;
  bad:
  Py_DECREF(o); o = 0;
  return NULL;
}

static void __pyx_tp_dealloc_4grpc_7_cython_6cygrpc_SslPemKeyCertPair(PyObject *o) {
  struct __pyx_obj_4grpc_7_cython_6cygrpc_SslPemKeyCertPair *p = (struct __pyx_obj_4grpc_7_cython_6cygrpc_SslPemKeyCertPair *)o;
  #if CYTHON_USE_TP_FINALIZE
  if (unlikely(PyType_HasFeature(Py_TYPE(o), Py_TPFLAGS_HAVE_FINALIZE) && Py_TYPE(o)->tp_finalize) && !_PyGC_FINALIZED(o)) {
    if (PyObject_CallFinalizerFromDealloc(o)) return;
  }
  #endif
  PyObject_GC_UnTrack(o);
  Py_CLEAR(p->private_key);
  Py_CLEAR(p->certificate_chain);
  (*Py_TYPE(o)->tp_free)(o);
}

static int __pyx_tp_traverse_4grpc_7_cython_6cygrpc_SslPemKeyCertPair(PyObject *o, visitproc v, void *a) {
  int e;
  struct __pyx_obj_4grpc_7_cython_6cygrpc_SslPemKeyCertPair *p = (struct __pyx_obj_4grpc_7_cython_6cygrpc_SslPemKeyCertPair *)o;
  if (p->private_key) {
    e = (*v)(p->private_key, a); if (e) return e;
  }
  if (p->certificate_chain) {
    e = (*v)(p->certificate_chain, a); if (e) return e;
  }
  return 0;
}

static int __pyx_tp_clear_4grpc_7_cython_6cygrpc_SslPemKeyCertPair(PyObject *o) {
  PyObject* tmp;
  struct __pyx_obj_4grpc_7_cython_6cygrpc_SslPemKeyCertPair *p = (struct __pyx_obj_4grpc_7_cython_6cygrpc_SslPemKeyCertPair *)o;
  tmp = ((PyObject*)p->private_key);
  p->private_key = Py_None; Py_INCREF(Py_None);
  Py_XDECREF(tmp);
  tmp = ((PyObject*)p->certificate_chain);
  p->certificate_chain = Py_None; Py_INCREF(Py_None);
  Py_XDECREF(tmp);
  return 0;
}

static PyObject *__pyx_getprop_4grpc_7_cython_6cygrpc_17SslPemKeyCertPair_private_key(PyObject *o, CYTHON_UNUSED void *x) {
  return __pyx_pw_4grpc_7_cython_6cygrpc_17SslPemKeyCertPair_11private_key_1__get__(o);
}

static PyObject *__pyx_getprop_4grpc_7_cython_6cygrpc_17SslPemKeyCertPair_certificate_chain(PyObject *o, CYTHON_UNUSED void *x) {
  return __pyx_pw_4grpc_7_cython_6cygrpc_17SslPemKeyCertPair_17certificate_chain_1__get__(o);
}

static PyMethodDef __pyx_methods_4grpc_7_cython_6cygrpc_SslPemKeyCertPair[] = {
  {"__reduce_cython__", (PyCFunction)__pyx_pw_4grpc_7_cython_6cygrpc_17SslPemKeyCertPair_3__reduce_cython__, METH_NOARGS, 0},
  {"__setstate_cython__", (PyCFunction)__pyx_pw_4grpc_7_cython_6cygrpc_17SslPemKeyCertPair_5__setstate_cython__, METH_O, 0},
  {0, 0, 0, 0}
};

static struct PyGetSetDef __pyx_getsets_4grpc_7_cython_6cygrpc_SslPemKeyCertPair[] = {
  {(char *)"private_key", __pyx_getprop_4grpc_7_cython_6cygrpc_17SslPemKeyCertPair_private_key, 0, (char *)0, 0},
  {(char *)"certificate_chain", __pyx_getprop_4grpc_7_cython_6cygrpc_17SslPemKeyCertPair_certificate_chain, 0, (char *)0, 0},
  {0, 0, 0, 0, 0}
};

static PyTypeObject __pyx_type_4grpc_7_cython_6cygrpc_SslPemKeyCertPair = {
  PyVarObject_HEAD_INIT(0, 0)
  "grpc._cython.cygrpc.SslPemKeyCertPair", /*tp_name*/
  sizeof(struct __pyx_obj_4grpc_7_cython_6cygrpc_SslPemKeyCertPair), /*tp_basicsize*/
  0, /*tp_itemsize*/
  __pyx_tp_dealloc_4grpc_7_cython_6cygrpc_SslPemKeyCertPair, /*tp_dealloc*/
  0, /*tp_print*/
  0, /*tp_getattr*/
  0, /*tp_setattr*/
  #if PY_MAJOR_VERSION < 3
  0, /*tp_compare*/
  #endif
  #if PY_MAJOR_VERSION >= 3
  0, /*tp_as_async*/
  #endif
  0, /*tp_repr*/
  0, /*tp_as_number*/
  0, /*tp_as_sequence*/
  0, /*tp_as_mapping*/
  0, /*tp_hash*/
  0, /*tp_call*/
  0, /*tp_str*/
  0, /*tp_getattro*/
  0, /*tp_setattro*/
  0, /*tp_as_buffer*/
  Py_TPFLAGS_DEFAULT|Py_TPFLAGS_HAVE_VERSION_TAG|Py_TPFLAGS_CHECKTYPES|Py_TPFLAGS_HAVE_NEWBUFFER|Py_TPFLAGS_BASETYPE|Py_TPFLAGS_HAVE_GC, /*tp_flags*/
  0, /*tp_doc*/
  __pyx_tp_traverse_4grpc_7_cython_6cygrpc_SslPemKeyCertPair, /*tp_traverse*/
  __pyx_tp_clear_4grpc_7_cython_6cygrpc_SslPemKeyCertPair, /*tp_clear*/
  0, /*tp_richcompare*/
  0, /*tp_weaklistoffset*/
  0, /*tp_iter*/
  0, /*tp_iternext*/
  __pyx_methods_4grpc_7_cython_6cygrpc_SslPemKeyCertPair, /*tp_methods*/
  0, /*tp_members*/
  __pyx_getsets_4grpc_7_cython_6cygrpc_SslPemKeyCertPair, /*tp_getset*/
  0, /*tp_base*/
  0, /*tp_dict*/
  0, /*tp_descr_get*/
  0, /*tp_descr_set*/
  0, /*tp_dictoffset*/
  0, /*tp_init*/
  0, /*tp_alloc*/
  __pyx_tp_new_4grpc_7_cython_6cygrpc_SslPemKeyCertPair, /*tp_new*/
  0, /*tp_free*/
  0, /*tp_is_gc*/
  0, /*tp_bases*/
  0, /*tp_mro*/
  0, /*tp_cache*/
  0, /*tp_subclasses*/
  0, /*tp_weaklist*/
  0, /*tp_del*/
  0, /*tp_version_tag*/
  #if PY_VERSION_HEX >= 0x030400a1
  0, /*tp_finalize*/
  #endif
};

static PyObject *__pyx_tp_new_4grpc_7_cython_6cygrpc_CompressionOptions(PyTypeObject *t, CYTHON_UNUSED PyObject *a, CYTHON_UNUSED PyObject *k) {
  PyObject *o;
  if (likely((t->tp_flags & Py_TPFLAGS_IS_ABSTRACT) == 0)) {
    o = (*t->tp_alloc)(t, 0);
  } else {
    o = (PyObject *) PyBaseObject_Type.tp_new(t, __pyx_empty_tuple, 0);
  }
  if (unlikely(!o)) return 0;
  if (unlikely(__pyx_pw_4grpc_7_cython_6cygrpc_18CompressionOptions_1__cinit__(o, __pyx_empty_tuple, NULL) < 0)) goto bad;
  return o;
  bad:
  Py_DECREF(o); o = 0;
  return NULL;
}

static void __pyx_tp_dealloc_4grpc_7_cython_6cygrpc_CompressionOptions(PyObject *o) {
  #if CYTHON_USE_TP_FINALIZE
  if (unlikely(PyType_HasFeature(Py_TYPE(o), Py_TPFLAGS_HAVE_FINALIZE) && Py_TYPE(o)->tp_finalize) && (!PyType_IS_GC(Py_TYPE(o)) || !_PyGC_FINALIZED(o))) {
    if (PyObject_CallFinalizerFromDealloc(o)) return;
  }
  #endif
  (*Py_TYPE(o)->tp_free)(o);
}

static PyMethodDef __pyx_methods_4grpc_7_cython_6cygrpc_CompressionOptions[] = {
  {"enable_algorithm", (PyCFunction)__pyx_pw_4grpc_7_cython_6cygrpc_18CompressionOptions_3enable_algorithm, METH_O, 0},
  {"disable_algorithm", (PyCFunction)__pyx_pw_4grpc_7_cython_6cygrpc_18CompressionOptions_5disable_algorithm, METH_O, 0},
  {"is_algorithm_enabled", (PyCFunction)__pyx_pw_4grpc_7_cython_6cygrpc_18CompressionOptions_7is_algorithm_enabled, METH_O, 0},
  {"to_channel_arg", (PyCFunction)__pyx_pw_4grpc_7_cython_6cygrpc_18CompressionOptions_9to_channel_arg, METH_NOARGS, 0},
  {"__reduce_cython__", (PyCFunction)__pyx_pw_4grpc_7_cython_6cygrpc_18CompressionOptions_11__reduce_cython__, METH_NOARGS, 0},
  {"__setstate_cython__", (PyCFunction)__pyx_pw_4grpc_7_cython_6cygrpc_18CompressionOptions_13__setstate_cython__, METH_O, 0},
  {0, 0, 0, 0}
};

static PyTypeObject __pyx_type_4grpc_7_cython_6cygrpc_CompressionOptions = {
  PyVarObject_HEAD_INIT(0, 0)
  "grpc._cython.cygrpc.CompressionOptions", /*tp_name*/
  sizeof(struct __pyx_obj_4grpc_7_cython_6cygrpc_CompressionOptions), /*tp_basicsize*/
  0, /*tp_itemsize*/
  __pyx_tp_dealloc_4grpc_7_cython_6cygrpc_CompressionOptions, /*tp_dealloc*/
  0, /*tp_print*/
  0, /*tp_getattr*/
  0, /*tp_setattr*/
  #if PY_MAJOR_VERSION < 3
  0, /*tp_compare*/
  #endif
  #if PY_MAJOR_VERSION >= 3
  0, /*tp_as_async*/
  #endif
  0, /*tp_repr*/
  0, /*tp_as_number*/
  0, /*tp_as_sequence*/
  0, /*tp_as_mapping*/
  0, /*tp_hash*/
  0, /*tp_call*/
  0, /*tp_str*/
  0, /*tp_getattro*/
  0, /*tp_setattro*/
  0, /*tp_as_buffer*/
  Py_TPFLAGS_DEFAULT|Py_TPFLAGS_HAVE_VERSION_TAG|Py_TPFLAGS_CHECKTYPES|Py_TPFLAGS_HAVE_NEWBUFFER|Py_TPFLAGS_BASETYPE, /*tp_flags*/
  0, /*tp_doc*/
  0, /*tp_traverse*/
  0, /*tp_clear*/
  0, /*tp_richcompare*/
  0, /*tp_weaklistoffset*/
  0, /*tp_iter*/
  0, /*tp_iternext*/
  __pyx_methods_4grpc_7_cython_6cygrpc_CompressionOptions, /*tp_methods*/
  0, /*tp_members*/
  0, /*tp_getset*/
  0, /*tp_base*/
  0, /*tp_dict*/
  0, /*tp_descr_get*/
  0, /*tp_descr_set*/
  0, /*tp_dictoffset*/
  0, /*tp_init*/
  0, /*tp_alloc*/
  __pyx_tp_new_4grpc_7_cython_6cygrpc_CompressionOptions, /*tp_new*/
  0, /*tp_free*/
  0, /*tp_is_gc*/
  0, /*tp_bases*/
  0, /*tp_mro*/
  0, /*tp_cache*/
  0, /*tp_subclasses*/
  0, /*tp_weaklist*/
  0, /*tp_del*/
  0, /*tp_version_tag*/
  #if PY_VERSION_HEX >= 0x030400a1
  0, /*tp_finalize*/
  #endif
};
static struct __pyx_vtabstruct_4grpc_7_cython_6cygrpc_Server __pyx_vtable_4grpc_7_cython_6cygrpc_Server;

static PyObject *__pyx_tp_new_4grpc_7_cython_6cygrpc_Server(PyTypeObject *t, PyObject *a, PyObject *k) {
  struct __pyx_obj_4grpc_7_cython_6cygrpc_Server *p;
  PyObject *o;
  if (likely((t->tp_flags & Py_TPFLAGS_IS_ABSTRACT) == 0)) {
    o = (*t->tp_alloc)(t, 0);
  } else {
    o = (PyObject *) PyBaseObject_Type.tp_new(t, __pyx_empty_tuple, 0);
  }
  if (unlikely(!o)) return 0;
  p = ((struct __pyx_obj_4grpc_7_cython_6cygrpc_Server *)o);
  p->__pyx_vtab = __pyx_vtabptr_4grpc_7_cython_6cygrpc_Server;
  p->_vtable = ((struct __pyx_obj_4grpc_7_cython_6cygrpc__VTable *)Py_None); Py_INCREF(Py_None);
  p->backup_shutdown_queue = ((struct __pyx_obj_4grpc_7_cython_6cygrpc_CompletionQueue *)Py_None); Py_INCREF(Py_None);
  p->references = ((PyObject*)Py_None); Py_INCREF(Py_None);
  p->registered_completion_queues = ((PyObject*)Py_None); Py_INCREF(Py_None);
  if (unlikely(__pyx_pw_4grpc_7_cython_6cygrpc_6Server_1__cinit__(o, a, k) < 0)) goto bad;
  return o;
  bad:
  Py_DECREF(o); o = 0;
  return NULL;
}

static void __pyx_tp_dealloc_4grpc_7_cython_6cygrpc_Server(PyObject *o) {
  struct __pyx_obj_4grpc_7_cython_6cygrpc_Server *p = (struct __pyx_obj_4grpc_7_cython_6cygrpc_Server *)o;
  #if CYTHON_USE_TP_FINALIZE
  if (unlikely(PyType_HasFeature(Py_TYPE(o), Py_TPFLAGS_HAVE_FINALIZE) && Py_TYPE(o)->tp_finalize) && !_PyGC_FINALIZED(o)) {
    if (PyObject_CallFinalizerFromDealloc(o)) return;
  }
  #endif
  PyObject_GC_UnTrack(o);
  {
    PyObject *etype, *eval, *etb;
    PyErr_Fetch(&etype, &eval, &etb);
    ++Py_REFCNT(o);
    __pyx_pw_4grpc_7_cython_6cygrpc_6Server_17__dealloc__(o);
    --Py_REFCNT(o);
    PyErr_Restore(etype, eval, etb);
  }
  Py_CLEAR(p->_vtable);
  Py_CLEAR(p->backup_shutdown_queue);
  Py_CLEAR(p->references);
  Py_CLEAR(p->registered_completion_queues);
  (*Py_TYPE(o)->tp_free)(o);
}

static int __pyx_tp_traverse_4grpc_7_cython_6cygrpc_Server(PyObject *o, visitproc v, void *a) {
  int e;
  struct __pyx_obj_4grpc_7_cython_6cygrpc_Server *p = (struct __pyx_obj_4grpc_7_cython_6cygrpc_Server *)o;
  if (p->_vtable) {
    e = (*v)(((PyObject *)p->_vtable), a); if (e) return e;
  }
  if (p->backup_shutdown_queue) {
    e = (*v)(((PyObject *)p->backup_shutdown_queue), a); if (e) return e;
  }
  if (p->references) {
    e = (*v)(p->references, a); if (e) return e;
  }
  if (p->registered_completion_queues) {
    e = (*v)(p->registered_completion_queues, a); if (e) return e;
  }
  return 0;
}

static int __pyx_tp_clear_4grpc_7_cython_6cygrpc_Server(PyObject *o) {
  PyObject* tmp;
  struct __pyx_obj_4grpc_7_cython_6cygrpc_Server *p = (struct __pyx_obj_4grpc_7_cython_6cygrpc_Server *)o;
  tmp = ((PyObject*)p->_vtable);
  p->_vtable = ((struct __pyx_obj_4grpc_7_cython_6cygrpc__VTable *)Py_None); Py_INCREF(Py_None);
  Py_XDECREF(tmp);
  tmp = ((PyObject*)p->backup_shutdown_queue);
  p->backup_shutdown_queue = ((struct __pyx_obj_4grpc_7_cython_6cygrpc_CompletionQueue *)Py_None); Py_INCREF(Py_None);
  Py_XDECREF(tmp);
  tmp = ((PyObject*)p->references);
  p->references = ((PyObject*)Py_None); Py_INCREF(Py_None);
  Py_XDECREF(tmp);
  tmp = ((PyObject*)p->registered_completion_queues);
  p->registered_completion_queues = ((PyObject*)Py_None); Py_INCREF(Py_None);
  Py_XDECREF(tmp);
  return 0;
}

static PyMethodDef __pyx_methods_4grpc_7_cython_6cygrpc_Server[] = {
  {"request_call", (PyCFunction)__pyx_pw_4grpc_7_cython_6cygrpc_6Server_3request_call, METH_VARARGS|METH_KEYWORDS, 0},
  {"register_completion_queue", (PyCFunction)__pyx_pw_4grpc_7_cython_6cygrpc_6Server_5register_completion_queue, METH_O, 0},
  {"start", (PyCFunction)__pyx_pw_4grpc_7_cython_6cygrpc_6Server_7start, METH_NOARGS, 0},
  {"add_http2_port", (PyCFunction)__pyx_pw_4grpc_7_cython_6cygrpc_6Server_9add_http2_port, METH_VARARGS|METH_KEYWORDS, 0},
  {"shutdown", (PyCFunction)__pyx_pw_4grpc_7_cython_6cygrpc_6Server_11shutdown, METH_VARARGS|METH_KEYWORDS, 0},
  {"cancel_all_calls", (PyCFunction)__pyx_pw_4grpc_7_cython_6cygrpc_6Server_13cancel_all_calls, METH_NOARGS, 0},
  {"destroy", (PyCFunction)__pyx_pw_4grpc_7_cython_6cygrpc_6Server_15destroy, METH_NOARGS, 0},
  {"__reduce_cython__", (PyCFunction)__pyx_pw_4grpc_7_cython_6cygrpc_6Server_19__reduce_cython__, METH_NOARGS, 0},
  {"__setstate_cython__", (PyCFunction)__pyx_pw_4grpc_7_cython_6cygrpc_6Server_21__setstate_cython__, METH_O, 0},
  {0, 0, 0, 0}
};

static PyTypeObject __pyx_type_4grpc_7_cython_6cygrpc_Server = {
  PyVarObject_HEAD_INIT(0, 0)
  "grpc._cython.cygrpc.Server", /*tp_name*/
  sizeof(struct __pyx_obj_4grpc_7_cython_6cygrpc_Server), /*tp_basicsize*/
  0, /*tp_itemsize*/
  __pyx_tp_dealloc_4grpc_7_cython_6cygrpc_Server, /*tp_dealloc*/
  0, /*tp_print*/
  0, /*tp_getattr*/
  0, /*tp_setattr*/
  #if PY_MAJOR_VERSION < 3
  0, /*tp_compare*/
  #endif
  #if PY_MAJOR_VERSION >= 3
  0, /*tp_as_async*/
  #endif
  0, /*tp_repr*/
  0, /*tp_as_number*/
  0, /*tp_as_sequence*/
  0, /*tp_as_mapping*/
  0, /*tp_hash*/
  0, /*tp_call*/
  0, /*tp_str*/
  0, /*tp_getattro*/
  0, /*tp_setattro*/
  0, /*tp_as_buffer*/
  Py_TPFLAGS_DEFAULT|Py_TPFLAGS_HAVE_VERSION_TAG|Py_TPFLAGS_CHECKTYPES|Py_TPFLAGS_HAVE_NEWBUFFER|Py_TPFLAGS_BASETYPE|Py_TPFLAGS_HAVE_GC, /*tp_flags*/
  0, /*tp_doc*/
  __pyx_tp_traverse_4grpc_7_cython_6cygrpc_Server, /*tp_traverse*/
  __pyx_tp_clear_4grpc_7_cython_6cygrpc_Server, /*tp_clear*/
  0, /*tp_richcompare*/
  0, /*tp_weaklistoffset*/
  0, /*tp_iter*/
  0, /*tp_iternext*/
  __pyx_methods_4grpc_7_cython_6cygrpc_Server, /*tp_methods*/
  0, /*tp_members*/
  0, /*tp_getset*/
  0, /*tp_base*/
  0, /*tp_dict*/
  0, /*tp_descr_get*/
  0, /*tp_descr_set*/
  0, /*tp_dictoffset*/
  0, /*tp_init*/
  0, /*tp_alloc*/
  __pyx_tp_new_4grpc_7_cython_6cygrpc_Server, /*tp_new*/
  0, /*tp_free*/
  0, /*tp_is_gc*/
  0, /*tp_bases*/
  0, /*tp_mro*/
  0, /*tp_cache*/
  0, /*tp_subclasses*/
  0, /*tp_weaklist*/
  0, /*tp_del*/
  0, /*tp_version_tag*/
  #if PY_VERSION_HEX >= 0x030400a1
  0, /*tp_finalize*/
  #endif
};
static struct __pyx_vtabstruct_4grpc_7_cython_6cygrpc__Tag __pyx_vtable_4grpc_7_cython_6cygrpc__Tag;

static PyObject *__pyx_tp_new_4grpc_7_cython_6cygrpc__Tag(PyTypeObject *t, CYTHON_UNUSED PyObject *a, CYTHON_UNUSED PyObject *k) {
  struct __pyx_obj_4grpc_7_cython_6cygrpc__Tag *p;
  PyObject *o;
  if (likely((t->tp_flags & Py_TPFLAGS_IS_ABSTRACT) == 0)) {
    o = (*t->tp_alloc)(t, 0);
  } else {
    o = (PyObject *) PyBaseObject_Type.tp_new(t, __pyx_empty_tuple, 0);
  }
  if (unlikely(!o)) return 0;
  p = ((struct __pyx_obj_4grpc_7_cython_6cygrpc__Tag *)o);
  p->__pyx_vtab = __pyx_vtabptr_4grpc_7_cython_6cygrpc__Tag;
  return o;
}

static void __pyx_tp_dealloc_4grpc_7_cython_6cygrpc__Tag(PyObject *o) {
  #if CYTHON_USE_TP_FINALIZE
  if (unlikely(PyType_HasFeature(Py_TYPE(o), Py_TPFLAGS_HAVE_FINALIZE) && Py_TYPE(o)->tp_finalize) && (!PyType_IS_GC(Py_TYPE(o)) || !_PyGC_FINALIZED(o))) {
    if (PyObject_CallFinalizerFromDealloc(o)) return;
  }
  #endif
  (*Py_TYPE(o)->tp_free)(o);
}

static PyMethodDef __pyx_methods_4grpc_7_cython_6cygrpc__Tag[] = {
  {"__reduce_cython__", (PyCFunction)__pyx_pw_4grpc_7_cython_6cygrpc_4_Tag_1__reduce_cython__, METH_NOARGS, 0},
  {"__setstate_cython__", (PyCFunction)__pyx_pw_4grpc_7_cython_6cygrpc_4_Tag_3__setstate_cython__, METH_O, 0},
  {0, 0, 0, 0}
};

static PyTypeObject __pyx_type_4grpc_7_cython_6cygrpc__Tag = {
  PyVarObject_HEAD_INIT(0, 0)
  "grpc._cython.cygrpc._Tag", /*tp_name*/
  sizeof(struct __pyx_obj_4grpc_7_cython_6cygrpc__Tag), /*tp_basicsize*/
  0, /*tp_itemsize*/
  __pyx_tp_dealloc_4grpc_7_cython_6cygrpc__Tag, /*tp_dealloc*/
  0, /*tp_print*/
  0, /*tp_getattr*/
  0, /*tp_setattr*/
  #if PY_MAJOR_VERSION < 3
  0, /*tp_compare*/
  #endif
  #if PY_MAJOR_VERSION >= 3
  0, /*tp_as_async*/
  #endif
  0, /*tp_repr*/
  0, /*tp_as_number*/
  0, /*tp_as_sequence*/
  0, /*tp_as_mapping*/
  0, /*tp_hash*/
  0, /*tp_call*/
  0, /*tp_str*/
  0, /*tp_getattro*/
  0, /*tp_setattro*/
  0, /*tp_as_buffer*/
  Py_TPFLAGS_DEFAULT|Py_TPFLAGS_HAVE_VERSION_TAG|Py_TPFLAGS_CHECKTYPES|Py_TPFLAGS_HAVE_NEWBUFFER|Py_TPFLAGS_BASETYPE, /*tp_flags*/
  0, /*tp_doc*/
  0, /*tp_traverse*/
  0, /*tp_clear*/
  0, /*tp_richcompare*/
  0, /*tp_weaklistoffset*/
  0, /*tp_iter*/
  0, /*tp_iternext*/
  __pyx_methods_4grpc_7_cython_6cygrpc__Tag, /*tp_methods*/
  0, /*tp_members*/
  0, /*tp_getset*/
  0, /*tp_base*/
  0, /*tp_dict*/
  0, /*tp_descr_get*/
  0, /*tp_descr_set*/
  0, /*tp_dictoffset*/
  0, /*tp_init*/
  0, /*tp_alloc*/
  __pyx_tp_new_4grpc_7_cython_6cygrpc__Tag, /*tp_new*/
  0, /*tp_free*/
  0, /*tp_is_gc*/
  0, /*tp_bases*/
  0, /*tp_mro*/
  0, /*tp_cache*/
  0, /*tp_subclasses*/
  0, /*tp_weaklist*/
  0, /*tp_del*/
  0, /*tp_version_tag*/
  #if PY_VERSION_HEX >= 0x030400a1
  0, /*tp_finalize*/
  #endif
};
static struct __pyx_vtabstruct_4grpc_7_cython_6cygrpc__ConnectivityTag __pyx_vtable_4grpc_7_cython_6cygrpc__ConnectivityTag;

static PyObject *__pyx_tp_new_4grpc_7_cython_6cygrpc__ConnectivityTag(PyTypeObject *t, PyObject *a, PyObject *k) {
  struct __pyx_obj_4grpc_7_cython_6cygrpc__ConnectivityTag *p;
  PyObject *o = __pyx_tp_new_4grpc_7_cython_6cygrpc__Tag(t, a, k);
  if (unlikely(!o)) return 0;
  p = ((struct __pyx_obj_4grpc_7_cython_6cygrpc__ConnectivityTag *)o);
  p->__pyx_base.__pyx_vtab = (struct __pyx_vtabstruct_4grpc_7_cython_6cygrpc__Tag*)__pyx_vtabptr_4grpc_7_cython_6cygrpc__ConnectivityTag;
  p->_user_tag = Py_None; Py_INCREF(Py_None);
  if (unlikely(__pyx_pw_4grpc_7_cython_6cygrpc_16_ConnectivityTag_1__cinit__(o, a, k) < 0)) goto bad;
  return o;
  bad:
  Py_DECREF(o); o = 0;
  return NULL;
}

static void __pyx_tp_dealloc_4grpc_7_cython_6cygrpc__ConnectivityTag(PyObject *o) {
  struct __pyx_obj_4grpc_7_cython_6cygrpc__ConnectivityTag *p = (struct __pyx_obj_4grpc_7_cython_6cygrpc__ConnectivityTag *)o;
  #if CYTHON_USE_TP_FINALIZE
  if (unlikely(PyType_HasFeature(Py_TYPE(o), Py_TPFLAGS_HAVE_FINALIZE) && Py_TYPE(o)->tp_finalize) && !_PyGC_FINALIZED(o)) {
    if (PyObject_CallFinalizerFromDealloc(o)) return;
  }
  #endif
  PyObject_GC_UnTrack(o);
  Py_CLEAR(p->_user_tag);
  #if CYTHON_USE_TYPE_SLOTS
  if (PyType_IS_GC(Py_TYPE(o)->tp_base))
  #endif
  PyObject_GC_Track(o);
  __pyx_tp_dealloc_4grpc_7_cython_6cygrpc__Tag(o);
}

static int __pyx_tp_traverse_4grpc_7_cython_6cygrpc__ConnectivityTag(PyObject *o, visitproc v, void *a) {
  int e;
  struct __pyx_obj_4grpc_7_cython_6cygrpc__ConnectivityTag *p = (struct __pyx_obj_4grpc_7_cython_6cygrpc__ConnectivityTag *)o;
  e = ((likely(__pyx_ptype_4grpc_7_cython_6cygrpc__Tag)) ? ((__pyx_ptype_4grpc_7_cython_6cygrpc__Tag->tp_traverse) ? __pyx_ptype_4grpc_7_cython_6cygrpc__Tag->tp_traverse(o, v, a) : 0) : __Pyx_call_next_tp_traverse(o, v, a, __pyx_tp_traverse_4grpc_7_cython_6cygrpc__ConnectivityTag)); if (e) return e;
  if (p->_user_tag) {
    e = (*v)(p->_user_tag, a); if (e) return e;
  }
  return 0;
}

static int __pyx_tp_clear_4grpc_7_cython_6cygrpc__ConnectivityTag(PyObject *o) {
  PyObject* tmp;
  struct __pyx_obj_4grpc_7_cython_6cygrpc__ConnectivityTag *p = (struct __pyx_obj_4grpc_7_cython_6cygrpc__ConnectivityTag *)o;
  if (likely(__pyx_ptype_4grpc_7_cython_6cygrpc__Tag)) { if (__pyx_ptype_4grpc_7_cython_6cygrpc__Tag->tp_clear) __pyx_ptype_4grpc_7_cython_6cygrpc__Tag->tp_clear(o); } else __Pyx_call_next_tp_clear(o, __pyx_tp_clear_4grpc_7_cython_6cygrpc__ConnectivityTag);
  tmp = ((PyObject*)p->_user_tag);
  p->_user_tag = Py_None; Py_INCREF(Py_None);
  Py_XDECREF(tmp);
  return 0;
}

static PyObject *__pyx_getprop_4grpc_7_cython_6cygrpc_16_ConnectivityTag__user_tag(PyObject *o, CYTHON_UNUSED void *x) {
  return __pyx_pw_4grpc_7_cython_6cygrpc_16_ConnectivityTag_9_user_tag_1__get__(o);
}

static PyMethodDef __pyx_methods_4grpc_7_cython_6cygrpc__ConnectivityTag[] = {
  {"__reduce_cython__", (PyCFunction)__pyx_pw_4grpc_7_cython_6cygrpc_16_ConnectivityTag_3__reduce_cython__, METH_NOARGS, 0},
  {"__setstate_cython__", (PyCFunction)__pyx_pw_4grpc_7_cython_6cygrpc_16_ConnectivityTag_5__setstate_cython__, METH_O, 0},
  {0, 0, 0, 0}
};

static struct PyGetSetDef __pyx_getsets_4grpc_7_cython_6cygrpc__ConnectivityTag[] = {
  {(char *)"_user_tag", __pyx_getprop_4grpc_7_cython_6cygrpc_16_ConnectivityTag__user_tag, 0, (char *)0, 0},
  {0, 0, 0, 0, 0}
};

static PyTypeObject __pyx_type_4grpc_7_cython_6cygrpc__ConnectivityTag = {
  PyVarObject_HEAD_INIT(0, 0)
  "grpc._cython.cygrpc._ConnectivityTag", /*tp_name*/
  sizeof(struct __pyx_obj_4grpc_7_cython_6cygrpc__ConnectivityTag), /*tp_basicsize*/
  0, /*tp_itemsize*/
  __pyx_tp_dealloc_4grpc_7_cython_6cygrpc__ConnectivityTag, /*tp_dealloc*/
  0, /*tp_print*/
  0, /*tp_getattr*/
  0, /*tp_setattr*/
  #if PY_MAJOR_VERSION < 3
  0, /*tp_compare*/
  #endif
  #if PY_MAJOR_VERSION >= 3
  0, /*tp_as_async*/
  #endif
  0, /*tp_repr*/
  0, /*tp_as_number*/
  0, /*tp_as_sequence*/
  0, /*tp_as_mapping*/
  0, /*tp_hash*/
  0, /*tp_call*/
  0, /*tp_str*/
  0, /*tp_getattro*/
  0, /*tp_setattro*/
  0, /*tp_as_buffer*/
  Py_TPFLAGS_DEFAULT|Py_TPFLAGS_HAVE_VERSION_TAG|Py_TPFLAGS_CHECKTYPES|Py_TPFLAGS_HAVE_NEWBUFFER|Py_TPFLAGS_BASETYPE|Py_TPFLAGS_HAVE_GC, /*tp_flags*/
  0, /*tp_doc*/
  __pyx_tp_traverse_4grpc_7_cython_6cygrpc__ConnectivityTag, /*tp_traverse*/
  __pyx_tp_clear_4grpc_7_cython_6cygrpc__ConnectivityTag, /*tp_clear*/
  0, /*tp_richcompare*/
  0, /*tp_weaklistoffset*/
  0, /*tp_iter*/
  0, /*tp_iternext*/
  __pyx_methods_4grpc_7_cython_6cygrpc__ConnectivityTag, /*tp_methods*/
  0, /*tp_members*/
  __pyx_getsets_4grpc_7_cython_6cygrpc__ConnectivityTag, /*tp_getset*/
  0, /*tp_base*/
  0, /*tp_dict*/
  0, /*tp_descr_get*/
  0, /*tp_descr_set*/
  0, /*tp_dictoffset*/
  0, /*tp_init*/
  0, /*tp_alloc*/
  __pyx_tp_new_4grpc_7_cython_6cygrpc__ConnectivityTag, /*tp_new*/
  0, /*tp_free*/
  0, /*tp_is_gc*/
  0, /*tp_bases*/
  0, /*tp_mro*/
  0, /*tp_cache*/
  0, /*tp_subclasses*/
  0, /*tp_weaklist*/
  0, /*tp_del*/
  0, /*tp_version_tag*/
  #if PY_VERSION_HEX >= 0x030400a1
  0, /*tp_finalize*/
  #endif
};
static struct __pyx_vtabstruct_4grpc_7_cython_6cygrpc__RequestCallTag __pyx_vtable_4grpc_7_cython_6cygrpc__RequestCallTag;

static PyObject *__pyx_tp_new_4grpc_7_cython_6cygrpc__RequestCallTag(PyTypeObject *t, PyObject *a, PyObject *k) {
  struct __pyx_obj_4grpc_7_cython_6cygrpc__RequestCallTag *p;
  PyObject *o = __pyx_tp_new_4grpc_7_cython_6cygrpc__Tag(t, a, k);
  if (unlikely(!o)) return 0;
  p = ((struct __pyx_obj_4grpc_7_cython_6cygrpc__RequestCallTag *)o);
  p->__pyx_base.__pyx_vtab = (struct __pyx_vtabstruct_4grpc_7_cython_6cygrpc__Tag*)__pyx_vtabptr_4grpc_7_cython_6cygrpc__RequestCallTag;
  p->_user_tag = Py_None; Py_INCREF(Py_None);
  p->call = ((struct __pyx_obj_4grpc_7_cython_6cygrpc_Call *)Py_None); Py_INCREF(Py_None);
  p->_vtable = ((struct __pyx_obj_4grpc_7_cython_6cygrpc__VTable *)Py_None); Py_INCREF(Py_None);
  p->call_details = ((struct __pyx_obj_4grpc_7_cython_6cygrpc_CallDetails *)Py_None); Py_INCREF(Py_None);
  if (unlikely(__pyx_pw_4grpc_7_cython_6cygrpc_15_RequestCallTag_1__cinit__(o, a, k) < 0)) goto bad;
  return o;
  bad:
  Py_DECREF(o); o = 0;
  return NULL;
}

static void __pyx_tp_dealloc_4grpc_7_cython_6cygrpc__RequestCallTag(PyObject *o) {
  struct __pyx_obj_4grpc_7_cython_6cygrpc__RequestCallTag *p = (struct __pyx_obj_4grpc_7_cython_6cygrpc__RequestCallTag *)o;
  #if CYTHON_USE_TP_FINALIZE
  if (unlikely(PyType_HasFeature(Py_TYPE(o), Py_TPFLAGS_HAVE_FINALIZE) && Py_TYPE(o)->tp_finalize) && !_PyGC_FINALIZED(o)) {
    if (PyObject_CallFinalizerFromDealloc(o)) return;
  }
  #endif
  PyObject_GC_UnTrack(o);
  Py_CLEAR(p->_user_tag);
  Py_CLEAR(p->call);
  Py_CLEAR(p->_vtable);
  Py_CLEAR(p->call_details);
  #if CYTHON_USE_TYPE_SLOTS
  if (PyType_IS_GC(Py_TYPE(o)->tp_base))
  #endif
  PyObject_GC_Track(o);
  __pyx_tp_dealloc_4grpc_7_cython_6cygrpc__Tag(o);
}

static int __pyx_tp_traverse_4grpc_7_cython_6cygrpc__RequestCallTag(PyObject *o, visitproc v, void *a) {
  int e;
  struct __pyx_obj_4grpc_7_cython_6cygrpc__RequestCallTag *p = (struct __pyx_obj_4grpc_7_cython_6cygrpc__RequestCallTag *)o;
  e = ((likely(__pyx_ptype_4grpc_7_cython_6cygrpc__Tag)) ? ((__pyx_ptype_4grpc_7_cython_6cygrpc__Tag->tp_traverse) ? __pyx_ptype_4grpc_7_cython_6cygrpc__Tag->tp_traverse(o, v, a) : 0) : __Pyx_call_next_tp_traverse(o, v, a, __pyx_tp_traverse_4grpc_7_cython_6cygrpc__RequestCallTag)); if (e) return e;
  if (p->_user_tag) {
    e = (*v)(p->_user_tag, a); if (e) return e;
  }
  if (p->call) {
    e = (*v)(((PyObject *)p->call), a); if (e) return e;
  }
  if (p->_vtable) {
    e = (*v)(((PyObject *)p->_vtable), a); if (e) return e;
  }
  if (p->call_details) {
    e = (*v)(((PyObject *)p->call_details), a); if (e) return e;
  }
  return 0;
}

static int __pyx_tp_clear_4grpc_7_cython_6cygrpc__RequestCallTag(PyObject *o) {
  PyObject* tmp;
  struct __pyx_obj_4grpc_7_cython_6cygrpc__RequestCallTag *p = (struct __pyx_obj_4grpc_7_cython_6cygrpc__RequestCallTag *)o;
  if (likely(__pyx_ptype_4grpc_7_cython_6cygrpc__Tag)) { if (__pyx_ptype_4grpc_7_cython_6cygrpc__Tag->tp_clear) __pyx_ptype_4grpc_7_cython_6cygrpc__Tag->tp_clear(o); } else __Pyx_call_next_tp_clear(o, __pyx_tp_clear_4grpc_7_cython_6cygrpc__RequestCallTag);
  tmp = ((PyObject*)p->_user_tag);
  p->_user_tag = Py_None; Py_INCREF(Py_None);
  Py_XDECREF(tmp);
  tmp = ((PyObject*)p->call);
  p->call = ((struct __pyx_obj_4grpc_7_cython_6cygrpc_Call *)Py_None); Py_INCREF(Py_None);
  Py_XDECREF(tmp);
  tmp = ((PyObject*)p->_vtable);
  p->_vtable = ((struct __pyx_obj_4grpc_7_cython_6cygrpc__VTable *)Py_None); Py_INCREF(Py_None);
  Py_XDECREF(tmp);
  tmp = ((PyObject*)p->call_details);
  p->call_details = ((struct __pyx_obj_4grpc_7_cython_6cygrpc_CallDetails *)Py_None); Py_INCREF(Py_None);
  Py_XDECREF(tmp);
  return 0;
}

static PyObject *__pyx_getprop_4grpc_7_cython_6cygrpc_15_RequestCallTag__user_tag(PyObject *o, CYTHON_UNUSED void *x) {
  return __pyx_pw_4grpc_7_cython_6cygrpc_15_RequestCallTag_9_user_tag_1__get__(o);
}

static PyMethodDef __pyx_methods_4grpc_7_cython_6cygrpc__RequestCallTag[] = {
  {"__reduce_cython__", (PyCFunction)__pyx_pw_4grpc_7_cython_6cygrpc_15_RequestCallTag_3__reduce_cython__, METH_NOARGS, 0},
  {"__setstate_cython__", (PyCFunction)__pyx_pw_4grpc_7_cython_6cygrpc_15_RequestCallTag_5__setstate_cython__, METH_O, 0},
  {0, 0, 0, 0}
};

static struct PyGetSetDef __pyx_getsets_4grpc_7_cython_6cygrpc__RequestCallTag[] = {
  {(char *)"_user_tag", __pyx_getprop_4grpc_7_cython_6cygrpc_15_RequestCallTag__user_tag, 0, (char *)0, 0},
  {0, 0, 0, 0, 0}
};

static PyTypeObject __pyx_type_4grpc_7_cython_6cygrpc__RequestCallTag = {
  PyVarObject_HEAD_INIT(0, 0)
  "grpc._cython.cygrpc._RequestCallTag", /*tp_name*/
  sizeof(struct __pyx_obj_4grpc_7_cython_6cygrpc__RequestCallTag), /*tp_basicsize*/
  0, /*tp_itemsize*/
  __pyx_tp_dealloc_4grpc_7_cython_6cygrpc__RequestCallTag, /*tp_dealloc*/
  0, /*tp_print*/
  0, /*tp_getattr*/
  0, /*tp_setattr*/
  #if PY_MAJOR_VERSION < 3
  0, /*tp_compare*/
  #endif
  #if PY_MAJOR_VERSION >= 3
  0, /*tp_as_async*/
  #endif
  0, /*tp_repr*/
  0, /*tp_as_number*/
  0, /*tp_as_sequence*/
  0, /*tp_as_mapping*/
  0, /*tp_hash*/
  0, /*tp_call*/
  0, /*tp_str*/
  0, /*tp_getattro*/
  0, /*tp_setattro*/
  0, /*tp_as_buffer*/
  Py_TPFLAGS_DEFAULT|Py_TPFLAGS_HAVE_VERSION_TAG|Py_TPFLAGS_CHECKTYPES|Py_TPFLAGS_HAVE_NEWBUFFER|Py_TPFLAGS_BASETYPE|Py_TPFLAGS_HAVE_GC, /*tp_flags*/
  0, /*tp_doc*/
  __pyx_tp_traverse_4grpc_7_cython_6cygrpc__RequestCallTag, /*tp_traverse*/
  __pyx_tp_clear_4grpc_7_cython_6cygrpc__RequestCallTag, /*tp_clear*/
  0, /*tp_richcompare*/
  0, /*tp_weaklistoffset*/
  0, /*tp_iter*/
  0, /*tp_iternext*/
  __pyx_methods_4grpc_7_cython_6cygrpc__RequestCallTag, /*tp_methods*/
  0, /*tp_members*/
  __pyx_getsets_4grpc_7_cython_6cygrpc__RequestCallTag, /*tp_getset*/
  0, /*tp_base*/
  0, /*tp_dict*/
  0, /*tp_descr_get*/
  0, /*tp_descr_set*/
  0, /*tp_dictoffset*/
  0, /*tp_init*/
  0, /*tp_alloc*/
  __pyx_tp_new_4grpc_7_cython_6cygrpc__RequestCallTag, /*tp_new*/
  0, /*tp_free*/
  0, /*tp_is_gc*/
  0, /*tp_bases*/
  0, /*tp_mro*/
  0, /*tp_cache*/
  0, /*tp_subclasses*/
  0, /*tp_weaklist*/
  0, /*tp_del*/
  0, /*tp_version_tag*/
  #if PY_VERSION_HEX >= 0x030400a1
  0, /*tp_finalize*/
  #endif
};
static struct __pyx_vtabstruct_4grpc_7_cython_6cygrpc__BatchOperationTag __pyx_vtable_4grpc_7_cython_6cygrpc__BatchOperationTag;

static PyObject *__pyx_tp_new_4grpc_7_cython_6cygrpc__BatchOperationTag(PyTypeObject *t, PyObject *a, PyObject *k) {
  struct __pyx_obj_4grpc_7_cython_6cygrpc__BatchOperationTag *p;
  PyObject *o = __pyx_tp_new_4grpc_7_cython_6cygrpc__Tag(t, a, k);
  if (unlikely(!o)) return 0;
  p = ((struct __pyx_obj_4grpc_7_cython_6cygrpc__BatchOperationTag *)o);
  p->__pyx_base.__pyx_vtab = (struct __pyx_vtabstruct_4grpc_7_cython_6cygrpc__Tag*)__pyx_vtabptr_4grpc_7_cython_6cygrpc__BatchOperationTag;
  p->_user_tag = Py_None; Py_INCREF(Py_None);
  p->_operations = Py_None; Py_INCREF(Py_None);
  p->_retained_call = Py_None; Py_INCREF(Py_None);
  if (unlikely(__pyx_pw_4grpc_7_cython_6cygrpc_18_BatchOperationTag_1__cinit__(o, a, k) < 0)) goto bad;
  return o;
  bad:
  Py_DECREF(o); o = 0;
  return NULL;
}

static void __pyx_tp_dealloc_4grpc_7_cython_6cygrpc__BatchOperationTag(PyObject *o) {
  struct __pyx_obj_4grpc_7_cython_6cygrpc__BatchOperationTag *p = (struct __pyx_obj_4grpc_7_cython_6cygrpc__BatchOperationTag *)o;
  #if CYTHON_USE_TP_FINALIZE
  if (unlikely(PyType_HasFeature(Py_TYPE(o), Py_TPFLAGS_HAVE_FINALIZE) && Py_TYPE(o)->tp_finalize) && !_PyGC_FINALIZED(o)) {
    if (PyObject_CallFinalizerFromDealloc(o)) return;
  }
  #endif
  PyObject_GC_UnTrack(o);
  Py_CLEAR(p->_user_tag);
  Py_CLEAR(p->_operations);
  Py_CLEAR(p->_retained_call);
  #if CYTHON_USE_TYPE_SLOTS
  if (PyType_IS_GC(Py_TYPE(o)->tp_base))
  #endif
  PyObject_GC_Track(o);
  __pyx_tp_dealloc_4grpc_7_cython_6cygrpc__Tag(o);
}

static int __pyx_tp_traverse_4grpc_7_cython_6cygrpc__BatchOperationTag(PyObject *o, visitproc v, void *a) {
  int e;
  struct __pyx_obj_4grpc_7_cython_6cygrpc__BatchOperationTag *p = (struct __pyx_obj_4grpc_7_cython_6cygrpc__BatchOperationTag *)o;
  e = ((likely(__pyx_ptype_4grpc_7_cython_6cygrpc__Tag)) ? ((__pyx_ptype_4grpc_7_cython_6cygrpc__Tag->tp_traverse) ? __pyx_ptype_4grpc_7_cython_6cygrpc__Tag->tp_traverse(o, v, a) : 0) : __Pyx_call_next_tp_traverse(o, v, a, __pyx_tp_traverse_4grpc_7_cython_6cygrpc__BatchOperationTag)); if (e) return e;
  if (p->_user_tag) {
    e = (*v)(p->_user_tag, a); if (e) return e;
  }
  if (p->_operations) {
    e = (*v)(p->_operations, a); if (e) return e;
  }
  if (p->_retained_call) {
    e = (*v)(p->_retained_call, a); if (e) return e;
  }
  return 0;
}

static int __pyx_tp_clear_4grpc_7_cython_6cygrpc__BatchOperationTag(PyObject *o) {
  PyObject* tmp;
  struct __pyx_obj_4grpc_7_cython_6cygrpc__BatchOperationTag *p = (struct __pyx_obj_4grpc_7_cython_6cygrpc__BatchOperationTag *)o;
  if (likely(__pyx_ptype_4grpc_7_cython_6cygrpc__Tag)) { if (__pyx_ptype_4grpc_7_cython_6cygrpc__Tag->tp_clear) __pyx_ptype_4grpc_7_cython_6cygrpc__Tag->tp_clear(o); } else __Pyx_call_next_tp_clear(o, __pyx_tp_clear_4grpc_7_cython_6cygrpc__BatchOperationTag);
  tmp = ((PyObject*)p->_user_tag);
  p->_user_tag = Py_None; Py_INCREF(Py_None);
  Py_XDECREF(tmp);
  tmp = ((PyObject*)p->_operations);
  p->_operations = Py_None; Py_INCREF(Py_None);
  Py_XDECREF(tmp);
  tmp = ((PyObject*)p->_retained_call);
  p->_retained_call = Py_None; Py_INCREF(Py_None);
  Py_XDECREF(tmp);
  return 0;
}

static PyObject *__pyx_getprop_4grpc_7_cython_6cygrpc_18_BatchOperationTag__operations(PyObject *o, CYTHON_UNUSED void *x) {
  return __pyx_pw_4grpc_7_cython_6cygrpc_18_BatchOperationTag_11_operations_1__get__(o);
}

static PyObject *__pyx_getprop_4grpc_7_cython_6cygrpc_18_BatchOperationTag__retained_call(PyObject *o, CYTHON_UNUSED void *x) {
  return __pyx_pw_4grpc_7_cython_6cygrpc_18_BatchOperationTag_14_retained_call_1__get__(o);
}

static PyMethodDef __pyx_methods_4grpc_7_cython_6cygrpc__BatchOperationTag[] = {
  {"__reduce_cython__", (PyCFunction)__pyx_pw_4grpc_7_cython_6cygrpc_18_BatchOperationTag_3__reduce_cython__, METH_NOARGS, 0},
  {"__setstate_cython__", (PyCFunction)__pyx_pw_4grpc_7_cython_6cygrpc_18_BatchOperationTag_5__setstate_cython__, METH_O, 0},
  {0, 0, 0, 0}
};

static struct PyGetSetDef __pyx_getsets_4grpc_7_cython_6cygrpc__BatchOperationTag[] = {
  {(char *)"_operations", __pyx_getprop_4grpc_7_cython_6cygrpc_18_BatchOperationTag__operations, 0, (char *)0, 0},
  {(char *)"_retained_call", __pyx_getprop_4grpc_7_cython_6cygrpc_18_BatchOperationTag__retained_call, 0, (char *)0, 0},
  {0, 0, 0, 0, 0}
};

static PyTypeObject __pyx_type_4grpc_7_cython_6cygrpc__BatchOperationTag = {
  PyVarObject_HEAD_INIT(0, 0)
  "grpc._cython.cygrpc._BatchOperationTag", /*tp_name*/
  sizeof(struct __pyx_obj_4grpc_7_cython_6cygrpc__BatchOperationTag), /*tp_basicsize*/
  0, /*tp_itemsize*/
  __pyx_tp_dealloc_4grpc_7_cython_6cygrpc__BatchOperationTag, /*tp_dealloc*/
  0, /*tp_print*/
  0, /*tp_getattr*/
  0, /*tp_setattr*/
  #if PY_MAJOR_VERSION < 3
  0, /*tp_compare*/
  #endif
  #if PY_MAJOR_VERSION >= 3
  0, /*tp_as_async*/
  #endif
  0, /*tp_repr*/
  0, /*tp_as_number*/
  0, /*tp_as_sequence*/
  0, /*tp_as_mapping*/
  0, /*tp_hash*/
  0, /*tp_call*/
  0, /*tp_str*/
  0, /*tp_getattro*/
  0, /*tp_setattro*/
  0, /*tp_as_buffer*/
  Py_TPFLAGS_DEFAULT|Py_TPFLAGS_HAVE_VERSION_TAG|Py_TPFLAGS_CHECKTYPES|Py_TPFLAGS_HAVE_NEWBUFFER|Py_TPFLAGS_BASETYPE|Py_TPFLAGS_HAVE_GC, /*tp_flags*/
  0, /*tp_doc*/
  __pyx_tp_traverse_4grpc_7_cython_6cygrpc__BatchOperationTag, /*tp_traverse*/
  __pyx_tp_clear_4grpc_7_cython_6cygrpc__BatchOperationTag, /*tp_clear*/
  0, /*tp_richcompare*/
  0, /*tp_weaklistoffset*/
  0, /*tp_iter*/
  0, /*tp_iternext*/
  __pyx_methods_4grpc_7_cython_6cygrpc__BatchOperationTag, /*tp_methods*/
  0, /*tp_members*/
  __pyx_getsets_4grpc_7_cython_6cygrpc__BatchOperationTag, /*tp_getset*/
  0, /*tp_base*/
  0, /*tp_dict*/
  0, /*tp_descr_get*/
  0, /*tp_descr_set*/
  0, /*tp_dictoffset*/
  0, /*tp_init*/
  0, /*tp_alloc*/
  __pyx_tp_new_4grpc_7_cython_6cygrpc__BatchOperationTag, /*tp_new*/
  0, /*tp_free*/
  0, /*tp_is_gc*/
  0, /*tp_bases*/
  0, /*tp_mro*/
  0, /*tp_cache*/
  0, /*tp_subclasses*/
  0, /*tp_weaklist*/
  0, /*tp_del*/
  0, /*tp_version_tag*/
  #if PY_VERSION_HEX >= 0x030400a1
  0, /*tp_finalize*/
  #endif
};
static struct __pyx_vtabstruct_4grpc_7_cython_6cygrpc__ServerShutdownTag __pyx_vtable_4grpc_7_cython_6cygrpc__ServerShutdownTag;

static PyObject *__pyx_tp_new_4grpc_7_cython_6cygrpc__ServerShutdownTag(PyTypeObject *t, PyObject *a, PyObject *k) {
  struct __pyx_obj_4grpc_7_cython_6cygrpc__ServerShutdownTag *p;
  PyObject *o = __pyx_tp_new_4grpc_7_cython_6cygrpc__Tag(t, a, k);
  if (unlikely(!o)) return 0;
  p = ((struct __pyx_obj_4grpc_7_cython_6cygrpc__ServerShutdownTag *)o);
  p->__pyx_base.__pyx_vtab = (struct __pyx_vtabstruct_4grpc_7_cython_6cygrpc__Tag*)__pyx_vtabptr_4grpc_7_cython_6cygrpc__ServerShutdownTag;
  p->_user_tag = Py_None; Py_INCREF(Py_None);
  p->_shutting_down_server = ((struct __pyx_obj_4grpc_7_cython_6cygrpc_Server *)Py_None); Py_INCREF(Py_None);
  if (unlikely(__pyx_pw_4grpc_7_cython_6cygrpc_18_ServerShutdownTag_1__cinit__(o, a, k) < 0)) goto bad;
  return o;
  bad:
  Py_DECREF(o); o = 0;
  return NULL;
}

static void __pyx_tp_dealloc_4grpc_7_cython_6cygrpc__ServerShutdownTag(PyObject *o) {
  struct __pyx_obj_4grpc_7_cython_6cygrpc__ServerShutdownTag *p = (struct __pyx_obj_4grpc_7_cython_6cygrpc__ServerShutdownTag *)o;
  #if CYTHON_USE_TP_FINALIZE
  if (unlikely(PyType_HasFeature(Py_TYPE(o), Py_TPFLAGS_HAVE_FINALIZE) && Py_TYPE(o)->tp_finalize) && !_PyGC_FINALIZED(o)) {
    if (PyObject_CallFinalizerFromDealloc(o)) return;
  }
  #endif
  PyObject_GC_UnTrack(o);
  Py_CLEAR(p->_user_tag);
  Py_CLEAR(p->_shutting_down_server);
  #if CYTHON_USE_TYPE_SLOTS
  if (PyType_IS_GC(Py_TYPE(o)->tp_base))
  #endif
  PyObject_GC_Track(o);
  __pyx_tp_dealloc_4grpc_7_cython_6cygrpc__Tag(o);
}

static int __pyx_tp_traverse_4grpc_7_cython_6cygrpc__ServerShutdownTag(PyObject *o, visitproc v, void *a) {
  int e;
  struct __pyx_obj_4grpc_7_cython_6cygrpc__ServerShutdownTag *p = (struct __pyx_obj_4grpc_7_cython_6cygrpc__ServerShutdownTag *)o;
  e = ((likely(__pyx_ptype_4grpc_7_cython_6cygrpc__Tag)) ? ((__pyx_ptype_4grpc_7_cython_6cygrpc__Tag->tp_traverse) ? __pyx_ptype_4grpc_7_cython_6cygrpc__Tag->tp_traverse(o, v, a) : 0) : __Pyx_call_next_tp_traverse(o, v, a, __pyx_tp_traverse_4grpc_7_cython_6cygrpc__ServerShutdownTag)); if (e) return e;
  if (p->_user_tag) {
    e = (*v)(p->_user_tag, a); if (e) return e;
  }
  if (p->_shutting_down_server) {
    e = (*v)(((PyObject *)p->_shutting_down_server), a); if (e) return e;
  }
  return 0;
}

static int __pyx_tp_clear_4grpc_7_cython_6cygrpc__ServerShutdownTag(PyObject *o) {
  PyObject* tmp;
  struct __pyx_obj_4grpc_7_cython_6cygrpc__ServerShutdownTag *p = (struct __pyx_obj_4grpc_7_cython_6cygrpc__ServerShutdownTag *)o;
  if (likely(__pyx_ptype_4grpc_7_cython_6cygrpc__Tag)) { if (__pyx_ptype_4grpc_7_cython_6cygrpc__Tag->tp_clear) __pyx_ptype_4grpc_7_cython_6cygrpc__Tag->tp_clear(o); } else __Pyx_call_next_tp_clear(o, __pyx_tp_clear_4grpc_7_cython_6cygrpc__ServerShutdownTag);
  tmp = ((PyObject*)p->_user_tag);
  p->_user_tag = Py_None; Py_INCREF(Py_None);
  Py_XDECREF(tmp);
  tmp = ((PyObject*)p->_shutting_down_server);
  p->_shutting_down_server = ((struct __pyx_obj_4grpc_7_cython_6cygrpc_Server *)Py_None); Py_INCREF(Py_None);
  Py_XDECREF(tmp);
  return 0;
}

static PyObject *__pyx_getprop_4grpc_7_cython_6cygrpc_18_ServerShutdownTag__user_tag(PyObject *o, CYTHON_UNUSED void *x) {
  return __pyx_pw_4grpc_7_cython_6cygrpc_18_ServerShutdownTag_9_user_tag_1__get__(o);
}

static PyObject *__pyx_getprop_4grpc_7_cython_6cygrpc_18_ServerShutdownTag__shutting_down_server(PyObject *o, CYTHON_UNUSED void *x) {
  return __pyx_pw_4grpc_7_cython_6cygrpc_18_ServerShutdownTag_21_shutting_down_server_1__get__(o);
}

static PyMethodDef __pyx_methods_4grpc_7_cython_6cygrpc__ServerShutdownTag[] = {
  {"__reduce_cython__", (PyCFunction)__pyx_pw_4grpc_7_cython_6cygrpc_18_ServerShutdownTag_3__reduce_cython__, METH_NOARGS, 0},
  {"__setstate_cython__", (PyCFunction)__pyx_pw_4grpc_7_cython_6cygrpc_18_ServerShutdownTag_5__setstate_cython__, METH_O, 0},
  {0, 0, 0, 0}
};

static struct PyGetSetDef __pyx_getsets_4grpc_7_cython_6cygrpc__ServerShutdownTag[] = {
  {(char *)"_user_tag", __pyx_getprop_4grpc_7_cython_6cygrpc_18_ServerShutdownTag__user_tag, 0, (char *)0, 0},
  {(char *)"_shutting_down_server", __pyx_getprop_4grpc_7_cython_6cygrpc_18_ServerShutdownTag__shutting_down_server, 0, (char *)0, 0},
  {0, 0, 0, 0, 0}
};

static PyTypeObject __pyx_type_4grpc_7_cython_6cygrpc__ServerShutdownTag = {
  PyVarObject_HEAD_INIT(0, 0)
  "grpc._cython.cygrpc._ServerShutdownTag", /*tp_name*/
  sizeof(struct __pyx_obj_4grpc_7_cython_6cygrpc__ServerShutdownTag), /*tp_basicsize*/
  0, /*tp_itemsize*/
  __pyx_tp_dealloc_4grpc_7_cython_6cygrpc__ServerShutdownTag, /*tp_dealloc*/
  0, /*tp_print*/
  0, /*tp_getattr*/
  0, /*tp_setattr*/
  #if PY_MAJOR_VERSION < 3
  0, /*tp_compare*/
  #endif
  #if PY_MAJOR_VERSION >= 3
  0, /*tp_as_async*/
  #endif
  0, /*tp_repr*/
  0, /*tp_as_number*/
  0, /*tp_as_sequence*/
  0, /*tp_as_mapping*/
  0, /*tp_hash*/
  0, /*tp_call*/
  0, /*tp_str*/
  0, /*tp_getattro*/
  0, /*tp_setattro*/
  0, /*tp_as_buffer*/
  Py_TPFLAGS_DEFAULT|Py_TPFLAGS_HAVE_VERSION_TAG|Py_TPFLAGS_CHECKTYPES|Py_TPFLAGS_HAVE_NEWBUFFER|Py_TPFLAGS_BASETYPE|Py_TPFLAGS_HAVE_GC, /*tp_flags*/
  0, /*tp_doc*/
  __pyx_tp_traverse_4grpc_7_cython_6cygrpc__ServerShutdownTag, /*tp_traverse*/
  __pyx_tp_clear_4grpc_7_cython_6cygrpc__ServerShutdownTag, /*tp_clear*/
  0, /*tp_richcompare*/
  0, /*tp_weaklistoffset*/
  0, /*tp_iter*/
  0, /*tp_iternext*/
  __pyx_methods_4grpc_7_cython_6cygrpc__ServerShutdownTag, /*tp_methods*/
  0, /*tp_members*/
  __pyx_getsets_4grpc_7_cython_6cygrpc__ServerShutdownTag, /*tp_getset*/
  0, /*tp_base*/
  0, /*tp_dict*/
  0, /*tp_descr_get*/
  0, /*tp_descr_set*/
  0, /*tp_dictoffset*/
  0, /*tp_init*/
  0, /*tp_alloc*/
  __pyx_tp_new_4grpc_7_cython_6cygrpc__ServerShutdownTag, /*tp_new*/
  0, /*tp_free*/
  0, /*tp_is_gc*/
  0, /*tp_bases*/
  0, /*tp_mro*/
  0, /*tp_cache*/
  0, /*tp_subclasses*/
  0, /*tp_weaklist*/
  0, /*tp_del*/
  0, /*tp_version_tag*/
  #if PY_VERSION_HEX >= 0x030400a1
  0, /*tp_finalize*/
  #endif
};

static PyObject *__pyx_tp_new_4grpc_7_cython_6cygrpc__VTable(PyTypeObject *t, CYTHON_UNUSED PyObject *a, CYTHON_UNUSED PyObject *k) {
  PyObject *o;
  if (likely((t->tp_flags & Py_TPFLAGS_IS_ABSTRACT) == 0)) {
    o = (*t->tp_alloc)(t, 0);
  } else {
    o = (PyObject *) PyBaseObject_Type.tp_new(t, __pyx_empty_tuple, 0);
  }
  if (unlikely(!o)) return 0;
  if (unlikely(__pyx_pw_4grpc_7_cython_6cygrpc_7_VTable_1__cinit__(o, __pyx_empty_tuple, NULL) < 0)) goto bad;
  return o;
  bad:
  Py_DECREF(o); o = 0;
  return NULL;
}

static void __pyx_tp_dealloc_4grpc_7_cython_6cygrpc__VTable(PyObject *o) {
  #if CYTHON_USE_TP_FINALIZE
  if (unlikely(PyType_HasFeature(Py_TYPE(o), Py_TPFLAGS_HAVE_FINALIZE) && Py_TYPE(o)->tp_finalize) && (!PyType_IS_GC(Py_TYPE(o)) || !_PyGC_FINALIZED(o))) {
    if (PyObject_CallFinalizerFromDealloc(o)) return;
  }
  #endif
  (*Py_TYPE(o)->tp_free)(o);
}

static PyMethodDef __pyx_methods_4grpc_7_cython_6cygrpc__VTable[] = {
  {"__reduce_cython__", (PyCFunction)__pyx_pw_4grpc_7_cython_6cygrpc_7_VTable_3__reduce_cython__, METH_NOARGS, 0},
  {"__setstate_cython__", (PyCFunction)__pyx_pw_4grpc_7_cython_6cygrpc_7_VTable_5__setstate_cython__, METH_O, 0},
  {0, 0, 0, 0}
};

static PyTypeObject __pyx_type_4grpc_7_cython_6cygrpc__VTable = {
  PyVarObject_HEAD_INIT(0, 0)
  "grpc._cython.cygrpc._VTable", /*tp_name*/
  sizeof(struct __pyx_obj_4grpc_7_cython_6cygrpc__VTable), /*tp_basicsize*/
  0, /*tp_itemsize*/
  __pyx_tp_dealloc_4grpc_7_cython_6cygrpc__VTable, /*tp_dealloc*/
  0, /*tp_print*/
  0, /*tp_getattr*/
  0, /*tp_setattr*/
  #if PY_MAJOR_VERSION < 3
  0, /*tp_compare*/
  #endif
  #if PY_MAJOR_VERSION >= 3
  0, /*tp_as_async*/
  #endif
  0, /*tp_repr*/
  0, /*tp_as_number*/
  0, /*tp_as_sequence*/
  0, /*tp_as_mapping*/
  0, /*tp_hash*/
  0, /*tp_call*/
  0, /*tp_str*/
  0, /*tp_getattro*/
  0, /*tp_setattro*/
  0, /*tp_as_buffer*/
  Py_TPFLAGS_DEFAULT|Py_TPFLAGS_HAVE_VERSION_TAG|Py_TPFLAGS_CHECKTYPES|Py_TPFLAGS_HAVE_NEWBUFFER|Py_TPFLAGS_BASETYPE, /*tp_flags*/
  0, /*tp_doc*/
  0, /*tp_traverse*/
  0, /*tp_clear*/
  0, /*tp_richcompare*/
  0, /*tp_weaklistoffset*/
  0, /*tp_iter*/
  0, /*tp_iternext*/
  __pyx_methods_4grpc_7_cython_6cygrpc__VTable, /*tp_methods*/
  0, /*tp_members*/
  0, /*tp_getset*/
  0, /*tp_base*/
  0, /*tp_dict*/
  0, /*tp_descr_get*/
  0, /*tp_descr_set*/
  0, /*tp_dictoffset*/
  0, /*tp_init*/
  0, /*tp_alloc*/
  __pyx_tp_new_4grpc_7_cython_6cygrpc__VTable, /*tp_new*/
  0, /*tp_free*/
  0, /*tp_is_gc*/
  0, /*tp_bases*/
  0, /*tp_mro*/
  0, /*tp_cache*/
  0, /*tp_subclasses*/
  0, /*tp_weaklist*/
  0, /*tp_del*/
  0, /*tp_version_tag*/
  #if PY_VERSION_HEX >= 0x030400a1
  0, /*tp_finalize*/
  #endif
};

static PyObject *__pyx_tp_new_4grpc_7_cython_6cygrpc_TimerWrapper(PyTypeObject *t, PyObject *a, PyObject *k) {
  struct __pyx_obj_4grpc_7_cython_6cygrpc_TimerWrapper *p;
  PyObject *o;
  if (likely((t->tp_flags & Py_TPFLAGS_IS_ABSTRACT) == 0)) {
    o = (*t->tp_alloc)(t, 0);
  } else {
    o = (PyObject *) PyBaseObject_Type.tp_new(t, __pyx_empty_tuple, 0);
  }
  if (unlikely(!o)) return 0;
  p = ((struct __pyx_obj_4grpc_7_cython_6cygrpc_TimerWrapper *)o);
  p->timer = Py_None; Py_INCREF(Py_None);
  p->event = Py_None; Py_INCREF(Py_None);
  if (unlikely(__pyx_pw_4grpc_7_cython_6cygrpc_12TimerWrapper_1__cinit__(o, a, k) < 0)) goto bad;
  return o;
  bad:
  Py_DECREF(o); o = 0;
  return NULL;
}

static void __pyx_tp_dealloc_4grpc_7_cython_6cygrpc_TimerWrapper(PyObject *o) {
  struct __pyx_obj_4grpc_7_cython_6cygrpc_TimerWrapper *p = (struct __pyx_obj_4grpc_7_cython_6cygrpc_TimerWrapper *)o;
  #if CYTHON_USE_TP_FINALIZE
  if (unlikely(PyType_HasFeature(Py_TYPE(o), Py_TPFLAGS_HAVE_FINALIZE) && Py_TYPE(o)->tp_finalize) && !_PyGC_FINALIZED(o)) {
    if (PyObject_CallFinalizerFromDealloc(o)) return;
  }
  #endif
  PyObject_GC_UnTrack(o);
  Py_CLEAR(p->timer);
  Py_CLEAR(p->event);
  (*Py_TYPE(o)->tp_free)(o);
}

static int __pyx_tp_traverse_4grpc_7_cython_6cygrpc_TimerWrapper(PyObject *o, visitproc v, void *a) {
  int e;
  struct __pyx_obj_4grpc_7_cython_6cygrpc_TimerWrapper *p = (struct __pyx_obj_4grpc_7_cython_6cygrpc_TimerWrapper *)o;
  if (p->timer) {
    e = (*v)(p->timer, a); if (e) return e;
  }
  if (p->event) {
    e = (*v)(p->event, a); if (e) return e;
  }
  return 0;
}

static int __pyx_tp_clear_4grpc_7_cython_6cygrpc_TimerWrapper(PyObject *o) {
  PyObject* tmp;
  struct __pyx_obj_4grpc_7_cython_6cygrpc_TimerWrapper *p = (struct __pyx_obj_4grpc_7_cython_6cygrpc_TimerWrapper *)o;
  tmp = ((PyObject*)p->timer);
  p->timer = Py_None; Py_INCREF(Py_None);
  Py_XDECREF(tmp);
  tmp = ((PyObject*)p->event);
  p->event = Py_None; Py_INCREF(Py_None);
  Py_XDECREF(tmp);
  return 0;
}

static PyMethodDef __pyx_methods_4grpc_7_cython_6cygrpc_TimerWrapper[] = {
  {"start", (PyCFunction)__pyx_pw_4grpc_7_cython_6cygrpc_12TimerWrapper_3start, METH_NOARGS, 0},
  {"on_finish", (PyCFunction)__pyx_pw_4grpc_7_cython_6cygrpc_12TimerWrapper_5on_finish, METH_NOARGS, 0},
  {"stop", (PyCFunction)__pyx_pw_4grpc_7_cython_6cygrpc_12TimerWrapper_7stop, METH_NOARGS, 0},
  {"__reduce_cython__", (PyCFunction)__pyx_pw_4grpc_7_cython_6cygrpc_12TimerWrapper_9__reduce_cython__, METH_NOARGS, 0},
  {"__setstate_cython__", (PyCFunction)__pyx_pw_4grpc_7_cython_6cygrpc_12TimerWrapper_11__setstate_cython__, METH_O, 0},
  {0, 0, 0, 0}
};

static PyTypeObject __pyx_type_4grpc_7_cython_6cygrpc_TimerWrapper = {
  PyVarObject_HEAD_INIT(0, 0)
  "grpc._cython.cygrpc.TimerWrapper", /*tp_name*/
  sizeof(struct __pyx_obj_4grpc_7_cython_6cygrpc_TimerWrapper), /*tp_basicsize*/
  0, /*tp_itemsize*/
  __pyx_tp_dealloc_4grpc_7_cython_6cygrpc_TimerWrapper, /*tp_dealloc*/
  0, /*tp_print*/
  0, /*tp_getattr*/
  0, /*tp_setattr*/
  #if PY_MAJOR_VERSION < 3
  0, /*tp_compare*/
  #endif
  #if PY_MAJOR_VERSION >= 3
  0, /*tp_as_async*/
  #endif
  0, /*tp_repr*/
  0, /*tp_as_number*/
  0, /*tp_as_sequence*/
  0, /*tp_as_mapping*/
  0, /*tp_hash*/
  0, /*tp_call*/
  0, /*tp_str*/
  0, /*tp_getattro*/
  0, /*tp_setattro*/
  0, /*tp_as_buffer*/
  Py_TPFLAGS_DEFAULT|Py_TPFLAGS_HAVE_VERSION_TAG|Py_TPFLAGS_CHECKTYPES|Py_TPFLAGS_HAVE_NEWBUFFER|Py_TPFLAGS_BASETYPE|Py_TPFLAGS_HAVE_GC, /*tp_flags*/
  0, /*tp_doc*/
  __pyx_tp_traverse_4grpc_7_cython_6cygrpc_TimerWrapper, /*tp_traverse*/
  __pyx_tp_clear_4grpc_7_cython_6cygrpc_TimerWrapper, /*tp_clear*/
  0, /*tp_richcompare*/
  0, /*tp_weaklistoffset*/
  0, /*tp_iter*/
  0, /*tp_iternext*/
  __pyx_methods_4grpc_7_cython_6cygrpc_TimerWrapper, /*tp_methods*/
  0, /*tp_members*/
  0, /*tp_getset*/
  0, /*tp_base*/
  0, /*tp_dict*/
  0, /*tp_descr_get*/
  0, /*tp_descr_set*/
  0, /*tp_dictoffset*/
  0, /*tp_init*/
  0, /*tp_alloc*/
  __pyx_tp_new_4grpc_7_cython_6cygrpc_TimerWrapper, /*tp_new*/
  0, /*tp_free*/
  0, /*tp_is_gc*/
  0, /*tp_bases*/
  0, /*tp_mro*/
  0, /*tp_cache*/
  0, /*tp_subclasses*/
  0, /*tp_weaklist*/
  0, /*tp_del*/
  0, /*tp_version_tag*/
  #if PY_VERSION_HEX >= 0x030400a1
  0, /*tp_finalize*/
  #endif
};

static PyObject *__pyx_tp_new_4grpc_7_cython_6cygrpc_SocketWrapper(PyTypeObject *t, CYTHON_UNUSED PyObject *a, CYTHON_UNUSED PyObject *k) {
  struct __pyx_obj_4grpc_7_cython_6cygrpc_SocketWrapper *p;
  PyObject *o;
  if (likely((t->tp_flags & Py_TPFLAGS_IS_ABSTRACT) == 0)) {
    o = (*t->tp_alloc)(t, 0);
  } else {
    o = (PyObject *) PyBaseObject_Type.tp_new(t, __pyx_empty_tuple, 0);
  }
  if (unlikely(!o)) return 0;
  p = ((struct __pyx_obj_4grpc_7_cython_6cygrpc_SocketWrapper *)o);
  p->sockopts = Py_None; Py_INCREF(Py_None);
  p->socket = Py_None; Py_INCREF(Py_None);
  p->closed = Py_None; Py_INCREF(Py_None);
  if (unlikely(__pyx_pw_4grpc_7_cython_6cygrpc_13SocketWrapper_1__cinit__(o, __pyx_empty_tuple, NULL) < 0)) goto bad;
  return o;
  bad:
  Py_DECREF(o); o = 0;
  return NULL;
}

static void __pyx_tp_dealloc_4grpc_7_cython_6cygrpc_SocketWrapper(PyObject *o) {
  struct __pyx_obj_4grpc_7_cython_6cygrpc_SocketWrapper *p = (struct __pyx_obj_4grpc_7_cython_6cygrpc_SocketWrapper *)o;
  #if CYTHON_USE_TP_FINALIZE
  if (unlikely(PyType_HasFeature(Py_TYPE(o), Py_TPFLAGS_HAVE_FINALIZE) && Py_TYPE(o)->tp_finalize) && !_PyGC_FINALIZED(o)) {
    if (PyObject_CallFinalizerFromDealloc(o)) return;
  }
  #endif
  PyObject_GC_UnTrack(o);
  Py_CLEAR(p->sockopts);
  Py_CLEAR(p->socket);
  Py_CLEAR(p->closed);
  (*Py_TYPE(o)->tp_free)(o);
}

static int __pyx_tp_traverse_4grpc_7_cython_6cygrpc_SocketWrapper(PyObject *o, visitproc v, void *a) {
  int e;
  struct __pyx_obj_4grpc_7_cython_6cygrpc_SocketWrapper *p = (struct __pyx_obj_4grpc_7_cython_6cygrpc_SocketWrapper *)o;
  if (p->sockopts) {
    e = (*v)(p->sockopts, a); if (e) return e;
  }
  if (p->socket) {
    e = (*v)(p->socket, a); if (e) return e;
  }
  if (p->closed) {
    e = (*v)(p->closed, a); if (e) return e;
  }
  return 0;
}

static int __pyx_tp_clear_4grpc_7_cython_6cygrpc_SocketWrapper(PyObject *o) {
  PyObject* tmp;
  struct __pyx_obj_4grpc_7_cython_6cygrpc_SocketWrapper *p = (struct __pyx_obj_4grpc_7_cython_6cygrpc_SocketWrapper *)o;
  tmp = ((PyObject*)p->sockopts);
  p->sockopts = Py_None; Py_INCREF(Py_None);
  Py_XDECREF(tmp);
  tmp = ((PyObject*)p->socket);
  p->socket = Py_None; Py_INCREF(Py_None);
  Py_XDECREF(tmp);
  tmp = ((PyObject*)p->closed);
  p->closed = Py_None; Py_INCREF(Py_None);
  Py_XDECREF(tmp);
  return 0;
}

static PyMethodDef __pyx_methods_4grpc_7_cython_6cygrpc_SocketWrapper[] = {
  {"__reduce_cython__", (PyCFunction)__pyx_pw_4grpc_7_cython_6cygrpc_13SocketWrapper_3__reduce_cython__, METH_NOARGS, 0},
  {"__setstate_cython__", (PyCFunction)__pyx_pw_4grpc_7_cython_6cygrpc_13SocketWrapper_5__setstate_cython__, METH_O, 0},
  {0, 0, 0, 0}
};

static PyTypeObject __pyx_type_4grpc_7_cython_6cygrpc_SocketWrapper = {
  PyVarObject_HEAD_INIT(0, 0)
  "grpc._cython.cygrpc.SocketWrapper", /*tp_name*/
  sizeof(struct __pyx_obj_4grpc_7_cython_6cygrpc_SocketWrapper), /*tp_basicsize*/
  0, /*tp_itemsize*/
  __pyx_tp_dealloc_4grpc_7_cython_6cygrpc_SocketWrapper, /*tp_dealloc*/
  0, /*tp_print*/
  0, /*tp_getattr*/
  0, /*tp_setattr*/
  #if PY_MAJOR_VERSION < 3
  0, /*tp_compare*/
  #endif
  #if PY_MAJOR_VERSION >= 3
  0, /*tp_as_async*/
  #endif
  0, /*tp_repr*/
  0, /*tp_as_number*/
  0, /*tp_as_sequence*/
  0, /*tp_as_mapping*/
  0, /*tp_hash*/
  0, /*tp_call*/
  0, /*tp_str*/
  0, /*tp_getattro*/
  0, /*tp_setattro*/
  0, /*tp_as_buffer*/
  Py_TPFLAGS_DEFAULT|Py_TPFLAGS_HAVE_VERSION_TAG|Py_TPFLAGS_CHECKTYPES|Py_TPFLAGS_HAVE_NEWBUFFER|Py_TPFLAGS_BASETYPE|Py_TPFLAGS_HAVE_GC, /*tp_flags*/
  0, /*tp_doc*/
  __pyx_tp_traverse_4grpc_7_cython_6cygrpc_SocketWrapper, /*tp_traverse*/
  __pyx_tp_clear_4grpc_7_cython_6cygrpc_SocketWrapper, /*tp_clear*/
  0, /*tp_richcompare*/
  0, /*tp_weaklistoffset*/
  0, /*tp_iter*/
  0, /*tp_iternext*/
  __pyx_methods_4grpc_7_cython_6cygrpc_SocketWrapper, /*tp_methods*/
  0, /*tp_members*/
  0, /*tp_getset*/
  0, /*tp_base*/
  0, /*tp_dict*/
  0, /*tp_descr_get*/
  0, /*tp_descr_set*/
  0, /*tp_dictoffset*/
  0, /*tp_init*/
  0, /*tp_alloc*/
  __pyx_tp_new_4grpc_7_cython_6cygrpc_SocketWrapper, /*tp_new*/
  0, /*tp_free*/
  0, /*tp_is_gc*/
  0, /*tp_bases*/
  0, /*tp_mro*/
  0, /*tp_cache*/
  0, /*tp_subclasses*/
  0, /*tp_weaklist*/
  0, /*tp_del*/
  0, /*tp_version_tag*/
  #if PY_VERSION_HEX >= 0x030400a1
  0, /*tp_finalize*/
  #endif
};

static PyObject *__pyx_tp_new_4grpc_7_cython_6cygrpc_ResolveWrapper(PyTypeObject *t, CYTHON_UNUSED PyObject *a, CYTHON_UNUSED PyObject *k) {
  PyObject *o;
  if (likely((t->tp_flags & Py_TPFLAGS_IS_ABSTRACT) == 0)) {
    o = (*t->tp_alloc)(t, 0);
  } else {
    o = (PyObject *) PyBaseObject_Type.tp_new(t, __pyx_empty_tuple, 0);
  }
  if (unlikely(!o)) return 0;
  if (unlikely(__pyx_pw_4grpc_7_cython_6cygrpc_14ResolveWrapper_1__cinit__(o, __pyx_empty_tuple, NULL) < 0)) goto bad;
  return o;
  bad:
  Py_DECREF(o); o = 0;
  return NULL;
}

static void __pyx_tp_dealloc_4grpc_7_cython_6cygrpc_ResolveWrapper(PyObject *o) {
  #if CYTHON_USE_TP_FINALIZE
  if (unlikely(PyType_HasFeature(Py_TYPE(o), Py_TPFLAGS_HAVE_FINALIZE) && Py_TYPE(o)->tp_finalize) && (!PyType_IS_GC(Py_TYPE(o)) || !_PyGC_FINALIZED(o))) {
    if (PyObject_CallFinalizerFromDealloc(o)) return;
  }
  #endif
  (*Py_TYPE(o)->tp_free)(o);
}

static PyMethodDef __pyx_methods_4grpc_7_cython_6cygrpc_ResolveWrapper[] = {
  {"__reduce_cython__", (PyCFunction)__pyx_pw_4grpc_7_cython_6cygrpc_14ResolveWrapper_3__reduce_cython__, METH_NOARGS, 0},
  {"__setstate_cython__", (PyCFunction)__pyx_pw_4grpc_7_cython_6cygrpc_14ResolveWrapper_5__setstate_cython__, METH_O, 0},
  {0, 0, 0, 0}
};

static PyTypeObject __pyx_type_4grpc_7_cython_6cygrpc_ResolveWrapper = {
  PyVarObject_HEAD_INIT(0, 0)
  "grpc._cython.cygrpc.ResolveWrapper", /*tp_name*/
  sizeof(struct __pyx_obj_4grpc_7_cython_6cygrpc_ResolveWrapper), /*tp_basicsize*/
  0, /*tp_itemsize*/
  __pyx_tp_dealloc_4grpc_7_cython_6cygrpc_ResolveWrapper, /*tp_dealloc*/
  0, /*tp_print*/
  0, /*tp_getattr*/
  0, /*tp_setattr*/
  #if PY_MAJOR_VERSION < 3
  0, /*tp_compare*/
  #endif
  #if PY_MAJOR_VERSION >= 3
  0, /*tp_as_async*/
  #endif
  0, /*tp_repr*/
  0, /*tp_as_number*/
  0, /*tp_as_sequence*/
  0, /*tp_as_mapping*/
  0, /*tp_hash*/
  0, /*tp_call*/
  0, /*tp_str*/
  0, /*tp_getattro*/
  0, /*tp_setattro*/
  0, /*tp_as_buffer*/
  Py_TPFLAGS_DEFAULT|Py_TPFLAGS_HAVE_VERSION_TAG|Py_TPFLAGS_CHECKTYPES|Py_TPFLAGS_HAVE_NEWBUFFER|Py_TPFLAGS_BASETYPE, /*tp_flags*/
  0, /*tp_doc*/
  0, /*tp_traverse*/
  0, /*tp_clear*/
  0, /*tp_richcompare*/
  0, /*tp_weaklistoffset*/
  0, /*tp_iter*/
  0, /*tp_iternext*/
  __pyx_methods_4grpc_7_cython_6cygrpc_ResolveWrapper, /*tp_methods*/
  0, /*tp_members*/
  0, /*tp_getset*/
  0, /*tp_base*/
  0, /*tp_dict*/
  0, /*tp_descr_get*/
  0, /*tp_descr_set*/
  0, /*tp_dictoffset*/
  0, /*tp_init*/
  0, /*tp_alloc*/
  __pyx_tp_new_4grpc_7_cython_6cygrpc_ResolveWrapper, /*tp_new*/
  0, /*tp_free*/
  0, /*tp_is_gc*/
  0, /*tp_bases*/
  0, /*tp_mro*/
  0, /*tp_cache*/
  0, /*tp_subclasses*/
  0, /*tp_weaklist*/
  0, /*tp_del*/
  0, /*tp_version_tag*/
  #if PY_VERSION_HEX >= 0x030400a1
  0, /*tp_finalize*/
  #endif
};

static PyObject *__pyx_tp_new_4grpc_7_cython_6cygrpc__GrpcArgWrapper(PyTypeObject *t, CYTHON_UNUSED PyObject *a, CYTHON_UNUSED PyObject *k) {
  PyObject *o;
  if (likely((t->tp_flags & Py_TPFLAGS_IS_ABSTRACT) == 0)) {
    o = (*t->tp_alloc)(t, 0);
  } else {
    o = (PyObject *) PyBaseObject_Type.tp_new(t, __pyx_empty_tuple, 0);
  }
  if (unlikely(!o)) return 0;
  return o;
}

static void __pyx_tp_dealloc_4grpc_7_cython_6cygrpc__GrpcArgWrapper(PyObject *o) {
  #if CYTHON_USE_TP_FINALIZE
  if (unlikely(PyType_HasFeature(Py_TYPE(o), Py_TPFLAGS_HAVE_FINALIZE) && Py_TYPE(o)->tp_finalize) && (!PyType_IS_GC(Py_TYPE(o)) || !_PyGC_FINALIZED(o))) {
    if (PyObject_CallFinalizerFromDealloc(o)) return;
  }
  #endif
  (*Py_TYPE(o)->tp_free)(o);
}

static PyMethodDef __pyx_methods_4grpc_7_cython_6cygrpc__GrpcArgWrapper[] = {
  {"__reduce_cython__", (PyCFunction)__pyx_pw_4grpc_7_cython_6cygrpc_15_GrpcArgWrapper_1__reduce_cython__, METH_NOARGS, 0},
  {"__setstate_cython__", (PyCFunction)__pyx_pw_4grpc_7_cython_6cygrpc_15_GrpcArgWrapper_3__setstate_cython__, METH_O, 0},
  {0, 0, 0, 0}
};

static PyTypeObject __pyx_type_4grpc_7_cython_6cygrpc__GrpcArgWrapper = {
  PyVarObject_HEAD_INIT(0, 0)
  "grpc._cython.cygrpc._GrpcArgWrapper", /*tp_name*/
  sizeof(struct __pyx_obj_4grpc_7_cython_6cygrpc__GrpcArgWrapper), /*tp_basicsize*/
  0, /*tp_itemsize*/
  __pyx_tp_dealloc_4grpc_7_cython_6cygrpc__GrpcArgWrapper, /*tp_dealloc*/
  0, /*tp_print*/
  0, /*tp_getattr*/
  0, /*tp_setattr*/
  #if PY_MAJOR_VERSION < 3
  0, /*tp_compare*/
  #endif
  #if PY_MAJOR_VERSION >= 3
  0, /*tp_as_async*/
  #endif
  0, /*tp_repr*/
  0, /*tp_as_number*/
  0, /*tp_as_sequence*/
  0, /*tp_as_mapping*/
  0, /*tp_hash*/
  0, /*tp_call*/
  0, /*tp_str*/
  0, /*tp_getattro*/
  0, /*tp_setattro*/
  0, /*tp_as_buffer*/
  Py_TPFLAGS_DEFAULT|Py_TPFLAGS_HAVE_VERSION_TAG|Py_TPFLAGS_CHECKTYPES|Py_TPFLAGS_HAVE_NEWBUFFER|Py_TPFLAGS_BASETYPE, /*tp_flags*/
  0, /*tp_doc*/
  0, /*tp_traverse*/
  0, /*tp_clear*/
  0, /*tp_richcompare*/
  0, /*tp_weaklistoffset*/
  0, /*tp_iter*/
  0, /*tp_iternext*/
  __pyx_methods_4grpc_7_cython_6cygrpc__GrpcArgWrapper, /*tp_methods*/
  0, /*tp_members*/
  0, /*tp_getset*/
  0, /*tp_base*/
  0, /*tp_dict*/
  0, /*tp_descr_get*/
  0, /*tp_descr_set*/
  0, /*tp_dictoffset*/
  0, /*tp_init*/
  0, /*tp_alloc*/
  __pyx_tp_new_4grpc_7_cython_6cygrpc__GrpcArgWrapper, /*tp_new*/
  0, /*tp_free*/
  0, /*tp_is_gc*/
  0, /*tp_bases*/
  0, /*tp_mro*/
  0, /*tp_cache*/
  0, /*tp_subclasses*/
  0, /*tp_weaklist*/
  0, /*tp_del*/
  0, /*tp_version_tag*/
  #if PY_VERSION_HEX >= 0x030400a1
  0, /*tp_finalize*/
  #endif
};

static PyObject *__pyx_tp_new_4grpc_7_cython_6cygrpc_CensusContext(PyTypeObject *t, CYTHON_UNUSED PyObject *a, CYTHON_UNUSED PyObject *k) {
  PyObject *o;
  if (likely((t->tp_flags & Py_TPFLAGS_IS_ABSTRACT) == 0)) {
    o = (*t->tp_alloc)(t, 0);
  } else {
    o = (PyObject *) PyBaseObject_Type.tp_new(t, __pyx_empty_tuple, 0);
  }
  if (unlikely(!o)) return 0;
  return o;
}

static void __pyx_tp_dealloc_4grpc_7_cython_6cygrpc_CensusContext(PyObject *o) {
  #if CYTHON_USE_TP_FINALIZE
  if (unlikely(PyType_HasFeature(Py_TYPE(o), Py_TPFLAGS_HAVE_FINALIZE) && Py_TYPE(o)->tp_finalize) && (!PyType_IS_GC(Py_TYPE(o)) || !_PyGC_FINALIZED(o))) {
    if (PyObject_CallFinalizerFromDealloc(o)) return;
  }
  #endif
  (*Py_TYPE(o)->tp_free)(o);
}

static PyMethodDef __pyx_methods_4grpc_7_cython_6cygrpc_CensusContext[] = {
  {"__reduce_cython__", (PyCFunction)__pyx_pw_4grpc_7_cython_6cygrpc_13CensusContext_1__reduce_cython__, METH_NOARGS, 0},
  {"__setstate_cython__", (PyCFunction)__pyx_pw_4grpc_7_cython_6cygrpc_13CensusContext_3__setstate_cython__, METH_O, 0},
  {0, 0, 0, 0}
};

static PyTypeObject __pyx_type_4grpc_7_cython_6cygrpc_CensusContext = {
  PyVarObject_HEAD_INIT(0, 0)
  "grpc._cython.cygrpc.CensusContext", /*tp_name*/
  sizeof(struct __pyx_obj_4grpc_7_cython_6cygrpc_CensusContext), /*tp_basicsize*/
  0, /*tp_itemsize*/
  __pyx_tp_dealloc_4grpc_7_cython_6cygrpc_CensusContext, /*tp_dealloc*/
  0, /*tp_print*/
  0, /*tp_getattr*/
  0, /*tp_setattr*/
  #if PY_MAJOR_VERSION < 3
  0, /*tp_compare*/
  #endif
  #if PY_MAJOR_VERSION >= 3
  0, /*tp_as_async*/
  #endif
  0, /*tp_repr*/
  0, /*tp_as_number*/
  0, /*tp_as_sequence*/
  0, /*tp_as_mapping*/
  0, /*tp_hash*/
  0, /*tp_call*/
  0, /*tp_str*/
  0, /*tp_getattro*/
  0, /*tp_setattro*/
  0, /*tp_as_buffer*/
  Py_TPFLAGS_DEFAULT|Py_TPFLAGS_HAVE_VERSION_TAG|Py_TPFLAGS_CHECKTYPES|Py_TPFLAGS_HAVE_NEWBUFFER|Py_TPFLAGS_BASETYPE, /*tp_flags*/
  0, /*tp_doc*/
  0, /*tp_traverse*/
  0, /*tp_clear*/
  0, /*tp_richcompare*/
  0, /*tp_weaklistoffset*/
  0, /*tp_iter*/
  0, /*tp_iternext*/
  __pyx_methods_4grpc_7_cython_6cygrpc_CensusContext, /*tp_methods*/
  0, /*tp_members*/
  0, /*tp_getset*/
  0, /*tp_base*/
  0, /*tp_dict*/
  0, /*tp_descr_get*/
  0, /*tp_descr_set*/
  0, /*tp_dictoffset*/
  0, /*tp_init*/
  0, /*tp_alloc*/
  __pyx_tp_new_4grpc_7_cython_6cygrpc_CensusContext, /*tp_new*/
  0, /*tp_free*/
  0, /*tp_is_gc*/
  0, /*tp_bases*/
  0, /*tp_mro*/
  0, /*tp_cache*/
  0, /*tp_subclasses*/
  0, /*tp_weaklist*/
  0, /*tp_del*/
  0, /*tp_version_tag*/
  #if PY_VERSION_HEX >= 0x030400a1
  0, /*tp_finalize*/
  #endif
};

static struct __pyx_obj_4grpc_7_cython_6cygrpc___pyx_scope_struct____pyx_f_4grpc_7_cython_6cygrpc__integrated_call *__pyx_freelist_4grpc_7_cython_6cygrpc___pyx_scope_struct____pyx_f_4grpc_7_cython_6cygrpc__integrated_call[8];
static int __pyx_freecount_4grpc_7_cython_6cygrpc___pyx_scope_struct____pyx_f_4grpc_7_cython_6cygrpc__integrated_call = 0;

static PyObject *__pyx_tp_new_4grpc_7_cython_6cygrpc___pyx_scope_struct____pyx_f_4grpc_7_cython_6cygrpc__integrated_call(PyTypeObject *t, CYTHON_UNUSED PyObject *a, CYTHON_UNUSED PyObject *k) {
  PyObject *o;
  if (CYTHON_COMPILING_IN_CPYTHON && likely((__pyx_freecount_4grpc_7_cython_6cygrpc___pyx_scope_struct____pyx_f_4grpc_7_cython_6cygrpc__integrated_call > 0) & (t->tp_basicsize == sizeof(struct __pyx_obj_4grpc_7_cython_6cygrpc___pyx_scope_struct____pyx_f_4grpc_7_cython_6cygrpc__integrated_call)))) {
    o = (PyObject*)__pyx_freelist_4grpc_7_cython_6cygrpc___pyx_scope_struct____pyx_f_4grpc_7_cython_6cygrpc__integrated_call[--__pyx_freecount_4grpc_7_cython_6cygrpc___pyx_scope_struct____pyx_f_4grpc_7_cython_6cygrpc__integrated_call];
    memset(o, 0, sizeof(struct __pyx_obj_4grpc_7_cython_6cygrpc___pyx_scope_struct____pyx_f_4grpc_7_cython_6cygrpc__integrated_call));
    (void) PyObject_INIT(o, t);
    PyObject_GC_Track(o);
  } else {
    o = (*t->tp_alloc)(t, 0);
    if (unlikely(!o)) return 0;
  }
  return o;
}

static void __pyx_tp_dealloc_4grpc_7_cython_6cygrpc___pyx_scope_struct____pyx_f_4grpc_7_cython_6cygrpc__integrated_call(PyObject *o) {
  struct __pyx_obj_4grpc_7_cython_6cygrpc___pyx_scope_struct____pyx_f_4grpc_7_cython_6cygrpc__integrated_call *p = (struct __pyx_obj_4grpc_7_cython_6cygrpc___pyx_scope_struct____pyx_f_4grpc_7_cython_6cygrpc__integrated_call *)o;
  PyObject_GC_UnTrack(o);
  Py_CLEAR(p->__pyx_v_call_state);
  Py_CLEAR(p->__pyx_v_state);
  if (CYTHON_COMPILING_IN_CPYTHON && ((__pyx_freecount_4grpc_7_cython_6cygrpc___pyx_scope_struct____pyx_f_4grpc_7_cython_6cygrpc__integrated_call < 8) & (Py_TYPE(o)->tp_basicsize == sizeof(struct __pyx_obj_4grpc_7_cython_6cygrpc___pyx_scope_struct____pyx_f_4grpc_7_cython_6cygrpc__integrated_call)))) {
    __pyx_freelist_4grpc_7_cython_6cygrpc___pyx_scope_struct____pyx_f_4grpc_7_cython_6cygrpc__integrated_call[__pyx_freecount_4grpc_7_cython_6cygrpc___pyx_scope_struct____pyx_f_4grpc_7_cython_6cygrpc__integrated_call++] = ((struct __pyx_obj_4grpc_7_cython_6cygrpc___pyx_scope_struct____pyx_f_4grpc_7_cython_6cygrpc__integrated_call *)o);
  } else {
    (*Py_TYPE(o)->tp_free)(o);
  }
}

static int __pyx_tp_traverse_4grpc_7_cython_6cygrpc___pyx_scope_struct____pyx_f_4grpc_7_cython_6cygrpc__integrated_call(PyObject *o, visitproc v, void *a) {
  int e;
  struct __pyx_obj_4grpc_7_cython_6cygrpc___pyx_scope_struct____pyx_f_4grpc_7_cython_6cygrpc__integrated_call *p = (struct __pyx_obj_4grpc_7_cython_6cygrpc___pyx_scope_struct____pyx_f_4grpc_7_cython_6cygrpc__integrated_call *)o;
  if (p->__pyx_v_call_state) {
    e = (*v)(((PyObject *)p->__pyx_v_call_state), a); if (e) return e;
  }
  if (p->__pyx_v_state) {
    e = (*v)(((PyObject *)p->__pyx_v_state), a); if (e) return e;
  }
  return 0;
}

static int __pyx_tp_clear_4grpc_7_cython_6cygrpc___pyx_scope_struct____pyx_f_4grpc_7_cython_6cygrpc__integrated_call(PyObject *o) {
  PyObject* tmp;
  struct __pyx_obj_4grpc_7_cython_6cygrpc___pyx_scope_struct____pyx_f_4grpc_7_cython_6cygrpc__integrated_call *p = (struct __pyx_obj_4grpc_7_cython_6cygrpc___pyx_scope_struct____pyx_f_4grpc_7_cython_6cygrpc__integrated_call *)o;
  tmp = ((PyObject*)p->__pyx_v_call_state);
  p->__pyx_v_call_state = ((struct __pyx_obj_4grpc_7_cython_6cygrpc__CallState *)Py_None); Py_INCREF(Py_None);
  Py_XDECREF(tmp);
  tmp = ((PyObject*)p->__pyx_v_state);
  p->__pyx_v_state = ((struct __pyx_obj_4grpc_7_cython_6cygrpc__ChannelState *)Py_None); Py_INCREF(Py_None);
  Py_XDECREF(tmp);
  return 0;
}

static PyTypeObject __pyx_type_4grpc_7_cython_6cygrpc___pyx_scope_struct____pyx_f_4grpc_7_cython_6cygrpc__integrated_call = {
  PyVarObject_HEAD_INIT(0, 0)
  "grpc._cython.cygrpc.__pyx_scope_struct____pyx_f_4grpc_7_cython_6cygrpc__integrated_call", /*tp_name*/
  sizeof(struct __pyx_obj_4grpc_7_cython_6cygrpc___pyx_scope_struct____pyx_f_4grpc_7_cython_6cygrpc__integrated_call), /*tp_basicsize*/
  0, /*tp_itemsize*/
  __pyx_tp_dealloc_4grpc_7_cython_6cygrpc___pyx_scope_struct____pyx_f_4grpc_7_cython_6cygrpc__integrated_call, /*tp_dealloc*/
  0, /*tp_print*/
  0, /*tp_getattr*/
  0, /*tp_setattr*/
  #if PY_MAJOR_VERSION < 3
  0, /*tp_compare*/
  #endif
  #if PY_MAJOR_VERSION >= 3
  0, /*tp_as_async*/
  #endif
  0, /*tp_repr*/
  0, /*tp_as_number*/
  0, /*tp_as_sequence*/
  0, /*tp_as_mapping*/
  0, /*tp_hash*/
  0, /*tp_call*/
  0, /*tp_str*/
  0, /*tp_getattro*/
  0, /*tp_setattro*/
  0, /*tp_as_buffer*/
  Py_TPFLAGS_DEFAULT|Py_TPFLAGS_HAVE_VERSION_TAG|Py_TPFLAGS_CHECKTYPES|Py_TPFLAGS_HAVE_NEWBUFFER|Py_TPFLAGS_HAVE_GC, /*tp_flags*/
  0, /*tp_doc*/
  __pyx_tp_traverse_4grpc_7_cython_6cygrpc___pyx_scope_struct____pyx_f_4grpc_7_cython_6cygrpc__integrated_call, /*tp_traverse*/
  __pyx_tp_clear_4grpc_7_cython_6cygrpc___pyx_scope_struct____pyx_f_4grpc_7_cython_6cygrpc__integrated_call, /*tp_clear*/
  0, /*tp_richcompare*/
  0, /*tp_weaklistoffset*/
  0, /*tp_iter*/
  0, /*tp_iternext*/
  0, /*tp_methods*/
  0, /*tp_members*/
  0, /*tp_getset*/
  0, /*tp_base*/
  0, /*tp_dict*/
  0, /*tp_descr_get*/
  0, /*tp_descr_set*/
  0, /*tp_dictoffset*/
  0, /*tp_init*/
  0, /*tp_alloc*/
  __pyx_tp_new_4grpc_7_cython_6cygrpc___pyx_scope_struct____pyx_f_4grpc_7_cython_6cygrpc__integrated_call, /*tp_new*/
  0, /*tp_free*/
  0, /*tp_is_gc*/
  0, /*tp_bases*/
  0, /*tp_mro*/
  0, /*tp_cache*/
  0, /*tp_subclasses*/
  0, /*tp_weaklist*/
  0, /*tp_del*/
  0, /*tp_version_tag*/
  #if PY_VERSION_HEX >= 0x030400a1
  0, /*tp_finalize*/
  #endif
};

static struct __pyx_obj_4grpc_7_cython_6cygrpc___pyx_scope_struct_1_next_event *__pyx_freelist_4grpc_7_cython_6cygrpc___pyx_scope_struct_1_next_event[8];
static int __pyx_freecount_4grpc_7_cython_6cygrpc___pyx_scope_struct_1_next_event = 0;

static PyObject *__pyx_tp_new_4grpc_7_cython_6cygrpc___pyx_scope_struct_1_next_event(PyTypeObject *t, CYTHON_UNUSED PyObject *a, CYTHON_UNUSED PyObject *k) {
  PyObject *o;
  if (CYTHON_COMPILING_IN_CPYTHON && likely((__pyx_freecount_4grpc_7_cython_6cygrpc___pyx_scope_struct_1_next_event > 0) & (t->tp_basicsize == sizeof(struct __pyx_obj_4grpc_7_cython_6cygrpc___pyx_scope_struct_1_next_event)))) {
    o = (PyObject*)__pyx_freelist_4grpc_7_cython_6cygrpc___pyx_scope_struct_1_next_event[--__pyx_freecount_4grpc_7_cython_6cygrpc___pyx_scope_struct_1_next_event];
    memset(o, 0, sizeof(struct __pyx_obj_4grpc_7_cython_6cygrpc___pyx_scope_struct_1_next_event));
    (void) PyObject_INIT(o, t);
    PyObject_GC_Track(o);
  } else {
    o = (*t->tp_alloc)(t, 0);
    if (unlikely(!o)) return 0;
  }
  return o;
}

static void __pyx_tp_dealloc_4grpc_7_cython_6cygrpc___pyx_scope_struct_1_next_event(PyObject *o) {
  struct __pyx_obj_4grpc_7_cython_6cygrpc___pyx_scope_struct_1_next_event *p = (struct __pyx_obj_4grpc_7_cython_6cygrpc___pyx_scope_struct_1_next_event *)o;
  PyObject_GC_UnTrack(o);
  Py_CLEAR(p->__pyx_v_self);
  if (CYTHON_COMPILING_IN_CPYTHON && ((__pyx_freecount_4grpc_7_cython_6cygrpc___pyx_scope_struct_1_next_event < 8) & (Py_TYPE(o)->tp_basicsize == sizeof(struct __pyx_obj_4grpc_7_cython_6cygrpc___pyx_scope_struct_1_next_event)))) {
    __pyx_freelist_4grpc_7_cython_6cygrpc___pyx_scope_struct_1_next_event[__pyx_freecount_4grpc_7_cython_6cygrpc___pyx_scope_struct_1_next_event++] = ((struct __pyx_obj_4grpc_7_cython_6cygrpc___pyx_scope_struct_1_next_event *)o);
  } else {
    (*Py_TYPE(o)->tp_free)(o);
  }
}

static int __pyx_tp_traverse_4grpc_7_cython_6cygrpc___pyx_scope_struct_1_next_event(PyObject *o, visitproc v, void *a) {
  int e;
  struct __pyx_obj_4grpc_7_cython_6cygrpc___pyx_scope_struct_1_next_event *p = (struct __pyx_obj_4grpc_7_cython_6cygrpc___pyx_scope_struct_1_next_event *)o;
  if (p->__pyx_v_self) {
    e = (*v)(((PyObject *)p->__pyx_v_self), a); if (e) return e;
  }
  return 0;
}

static int __pyx_tp_clear_4grpc_7_cython_6cygrpc___pyx_scope_struct_1_next_event(PyObject *o) {
  PyObject* tmp;
  struct __pyx_obj_4grpc_7_cython_6cygrpc___pyx_scope_struct_1_next_event *p = (struct __pyx_obj_4grpc_7_cython_6cygrpc___pyx_scope_struct_1_next_event *)o;
  tmp = ((PyObject*)p->__pyx_v_self);
  p->__pyx_v_self = ((struct __pyx_obj_4grpc_7_cython_6cygrpc_SegregatedCall *)Py_None); Py_INCREF(Py_None);
  Py_XDECREF(tmp);
  return 0;
}

static PyTypeObject __pyx_type_4grpc_7_cython_6cygrpc___pyx_scope_struct_1_next_event = {
  PyVarObject_HEAD_INIT(0, 0)
  "grpc._cython.cygrpc.__pyx_scope_struct_1_next_event", /*tp_name*/
  sizeof(struct __pyx_obj_4grpc_7_cython_6cygrpc___pyx_scope_struct_1_next_event), /*tp_basicsize*/
  0, /*tp_itemsize*/
  __pyx_tp_dealloc_4grpc_7_cython_6cygrpc___pyx_scope_struct_1_next_event, /*tp_dealloc*/
  0, /*tp_print*/
  0, /*tp_getattr*/
  0, /*tp_setattr*/
  #if PY_MAJOR_VERSION < 3
  0, /*tp_compare*/
  #endif
  #if PY_MAJOR_VERSION >= 3
  0, /*tp_as_async*/
  #endif
  0, /*tp_repr*/
  0, /*tp_as_number*/
  0, /*tp_as_sequence*/
  0, /*tp_as_mapping*/
  0, /*tp_hash*/
  0, /*tp_call*/
  0, /*tp_str*/
  0, /*tp_getattro*/
  0, /*tp_setattro*/
  0, /*tp_as_buffer*/
  Py_TPFLAGS_DEFAULT|Py_TPFLAGS_HAVE_VERSION_TAG|Py_TPFLAGS_CHECKTYPES|Py_TPFLAGS_HAVE_NEWBUFFER|Py_TPFLAGS_HAVE_GC, /*tp_flags*/
  0, /*tp_doc*/
  __pyx_tp_traverse_4grpc_7_cython_6cygrpc___pyx_scope_struct_1_next_event, /*tp_traverse*/
  __pyx_tp_clear_4grpc_7_cython_6cygrpc___pyx_scope_struct_1_next_event, /*tp_clear*/
  0, /*tp_richcompare*/
  0, /*tp_weaklistoffset*/
  0, /*tp_iter*/
  0, /*tp_iternext*/
  0, /*tp_methods*/
  0, /*tp_members*/
  0, /*tp_getset*/
  0, /*tp_base*/
  0, /*tp_dict*/
  0, /*tp_descr_get*/
  0, /*tp_descr_set*/
  0, /*tp_dictoffset*/
  0, /*tp_init*/
  0, /*tp_alloc*/
  __pyx_tp_new_4grpc_7_cython_6cygrpc___pyx_scope_struct_1_next_event, /*tp_new*/
  0, /*tp_free*/
  0, /*tp_is_gc*/
  0, /*tp_bases*/
  0, /*tp_mro*/
  0, /*tp_cache*/
  0, /*tp_subclasses*/
  0, /*tp_weaklist*/
  0, /*tp_del*/
  0, /*tp_version_tag*/
  #if PY_VERSION_HEX >= 0x030400a1
  0, /*tp_finalize*/
  #endif
};

static struct __pyx_obj_4grpc_7_cython_6cygrpc___pyx_scope_struct_2___pyx_f_4grpc_7_cython_6cygrpc__segregated_call *__pyx_freelist_4grpc_7_cython_6cygrpc___pyx_scope_struct_2___pyx_f_4grpc_7_cython_6cygrpc__segregated_call[8];
static int __pyx_freecount_4grpc_7_cython_6cygrpc___pyx_scope_struct_2___pyx_f_4grpc_7_cython_6cygrpc__segregated_call = 0;

static PyObject *__pyx_tp_new_4grpc_7_cython_6cygrpc___pyx_scope_struct_2___pyx_f_4grpc_7_cython_6cygrpc__segregated_call(PyTypeObject *t, CYTHON_UNUSED PyObject *a, CYTHON_UNUSED PyObject *k) {
  PyObject *o;
  if (CYTHON_COMPILING_IN_CPYTHON && likely((__pyx_freecount_4grpc_7_cython_6cygrpc___pyx_scope_struct_2___pyx_f_4grpc_7_cython_6cygrpc__segregated_call > 0) & (t->tp_basicsize == sizeof(struct __pyx_obj_4grpc_7_cython_6cygrpc___pyx_scope_struct_2___pyx_f_4grpc_7_cython_6cygrpc__segregated_call)))) {
    o = (PyObject*)__pyx_freelist_4grpc_7_cython_6cygrpc___pyx_scope_struct_2___pyx_f_4grpc_7_cython_6cygrpc__segregated_call[--__pyx_freecount_4grpc_7_cython_6cygrpc___pyx_scope_struct_2___pyx_f_4grpc_7_cython_6cygrpc__segregated_call];
    memset(o, 0, sizeof(struct __pyx_obj_4grpc_7_cython_6cygrpc___pyx_scope_struct_2___pyx_f_4grpc_7_cython_6cygrpc__segregated_call));
    (void) PyObject_INIT(o, t);
    PyObject_GC_Track(o);
  } else {
    o = (*t->tp_alloc)(t, 0);
    if (unlikely(!o)) return 0;
  }
  return o;
}

static void __pyx_tp_dealloc_4grpc_7_cython_6cygrpc___pyx_scope_struct_2___pyx_f_4grpc_7_cython_6cygrpc__segregated_call(PyObject *o) {
  struct __pyx_obj_4grpc_7_cython_6cygrpc___pyx_scope_struct_2___pyx_f_4grpc_7_cython_6cygrpc__segregated_call *p = (struct __pyx_obj_4grpc_7_cython_6cygrpc___pyx_scope_struct_2___pyx_f_4grpc_7_cython_6cygrpc__segregated_call *)o;
  PyObject_GC_UnTrack(o);
  Py_CLEAR(p->__pyx_v_call_state);
  Py_CLEAR(p->__pyx_v_state);
  if (CYTHON_COMPILING_IN_CPYTHON && ((__pyx_freecount_4grpc_7_cython_6cygrpc___pyx_scope_struct_2___pyx_f_4grpc_7_cython_6cygrpc__segregated_call < 8) & (Py_TYPE(o)->tp_basicsize == sizeof(struct __pyx_obj_4grpc_7_cython_6cygrpc___pyx_scope_struct_2___pyx_f_4grpc_7_cython_6cygrpc__segregated_call)))) {
    __pyx_freelist_4grpc_7_cython_6cygrpc___pyx_scope_struct_2___pyx_f_4grpc_7_cython_6cygrpc__segregated_call[__pyx_freecount_4grpc_7_cython_6cygrpc___pyx_scope_struct_2___pyx_f_4grpc_7_cython_6cygrpc__segregated_call++] = ((struct __pyx_obj_4grpc_7_cython_6cygrpc___pyx_scope_struct_2___pyx_f_4grpc_7_cython_6cygrpc__segregated_call *)o);
  } else {
    (*Py_TYPE(o)->tp_free)(o);
  }
}

static int __pyx_tp_traverse_4grpc_7_cython_6cygrpc___pyx_scope_struct_2___pyx_f_4grpc_7_cython_6cygrpc__segregated_call(PyObject *o, visitproc v, void *a) {
  int e;
  struct __pyx_obj_4grpc_7_cython_6cygrpc___pyx_scope_struct_2___pyx_f_4grpc_7_cython_6cygrpc__segregated_call *p = (struct __pyx_obj_4grpc_7_cython_6cygrpc___pyx_scope_struct_2___pyx_f_4grpc_7_cython_6cygrpc__segregated_call *)o;
  if (p->__pyx_v_call_state) {
    e = (*v)(((PyObject *)p->__pyx_v_call_state), a); if (e) return e;
  }
  if (p->__pyx_v_state) {
    e = (*v)(((PyObject *)p->__pyx_v_state), a); if (e) return e;
  }
  return 0;
}

static int __pyx_tp_clear_4grpc_7_cython_6cygrpc___pyx_scope_struct_2___pyx_f_4grpc_7_cython_6cygrpc__segregated_call(PyObject *o) {
  PyObject* tmp;
  struct __pyx_obj_4grpc_7_cython_6cygrpc___pyx_scope_struct_2___pyx_f_4grpc_7_cython_6cygrpc__segregated_call *p = (struct __pyx_obj_4grpc_7_cython_6cygrpc___pyx_scope_struct_2___pyx_f_4grpc_7_cython_6cygrpc__segregated_call *)o;
  tmp = ((PyObject*)p->__pyx_v_call_state);
  p->__pyx_v_call_state = ((struct __pyx_obj_4grpc_7_cython_6cygrpc__CallState *)Py_None); Py_INCREF(Py_None);
  Py_XDECREF(tmp);
  tmp = ((PyObject*)p->__pyx_v_state);
  p->__pyx_v_state = ((struct __pyx_obj_4grpc_7_cython_6cygrpc__ChannelState *)Py_None); Py_INCREF(Py_None);
  Py_XDECREF(tmp);
  return 0;
}

static PyTypeObject __pyx_type_4grpc_7_cython_6cygrpc___pyx_scope_struct_2___pyx_f_4grpc_7_cython_6cygrpc__segregated_call = {
  PyVarObject_HEAD_INIT(0, 0)
  "grpc._cython.cygrpc.__pyx_scope_struct_2___pyx_f_4grpc_7_cython_6cygrpc__segregated_call", /*tp_name*/
  sizeof(struct __pyx_obj_4grpc_7_cython_6cygrpc___pyx_scope_struct_2___pyx_f_4grpc_7_cython_6cygrpc__segregated_call), /*tp_basicsize*/
  0, /*tp_itemsize*/
  __pyx_tp_dealloc_4grpc_7_cython_6cygrpc___pyx_scope_struct_2___pyx_f_4grpc_7_cython_6cygrpc__segregated_call, /*tp_dealloc*/
  0, /*tp_print*/
  0, /*tp_getattr*/
  0, /*tp_setattr*/
  #if PY_MAJOR_VERSION < 3
  0, /*tp_compare*/
  #endif
  #if PY_MAJOR_VERSION >= 3
  0, /*tp_as_async*/
  #endif
  0, /*tp_repr*/
  0, /*tp_as_number*/
  0, /*tp_as_sequence*/
  0, /*tp_as_mapping*/
  0, /*tp_hash*/
  0, /*tp_call*/
  0, /*tp_str*/
  0, /*tp_getattro*/
  0, /*tp_setattro*/
  0, /*tp_as_buffer*/
  Py_TPFLAGS_DEFAULT|Py_TPFLAGS_HAVE_VERSION_TAG|Py_TPFLAGS_CHECKTYPES|Py_TPFLAGS_HAVE_NEWBUFFER|Py_TPFLAGS_HAVE_GC, /*tp_flags*/
  0, /*tp_doc*/
  __pyx_tp_traverse_4grpc_7_cython_6cygrpc___pyx_scope_struct_2___pyx_f_4grpc_7_cython_6cygrpc__segregated_call, /*tp_traverse*/
  __pyx_tp_clear_4grpc_7_cython_6cygrpc___pyx_scope_struct_2___pyx_f_4grpc_7_cython_6cygrpc__segregated_call, /*tp_clear*/
  0, /*tp_richcompare*/
  0, /*tp_weaklistoffset*/
  0, /*tp_iter*/
  0, /*tp_iternext*/
  0, /*tp_methods*/
  0, /*tp_members*/
  0, /*tp_getset*/
  0, /*tp_base*/
  0, /*tp_dict*/
  0, /*tp_descr_get*/
  0, /*tp_descr_set*/
  0, /*tp_dictoffset*/
  0, /*tp_init*/
  0, /*tp_alloc*/
  __pyx_tp_new_4grpc_7_cython_6cygrpc___pyx_scope_struct_2___pyx_f_4grpc_7_cython_6cygrpc__segregated_call, /*tp_new*/
  0, /*tp_free*/
  0, /*tp_is_gc*/
  0, /*tp_bases*/
  0, /*tp_mro*/
  0, /*tp_cache*/
  0, /*tp_subclasses*/
  0, /*tp_weaklist*/
  0, /*tp_del*/
  0, /*tp_version_tag*/
  #if PY_VERSION_HEX >= 0x030400a1
  0, /*tp_finalize*/
  #endif
};

static struct __pyx_obj_4grpc_7_cython_6cygrpc___pyx_scope_struct_3_next_call_event *__pyx_freelist_4grpc_7_cython_6cygrpc___pyx_scope_struct_3_next_call_event[8];
static int __pyx_freecount_4grpc_7_cython_6cygrpc___pyx_scope_struct_3_next_call_event = 0;

static PyObject *__pyx_tp_new_4grpc_7_cython_6cygrpc___pyx_scope_struct_3_next_call_event(PyTypeObject *t, CYTHON_UNUSED PyObject *a, CYTHON_UNUSED PyObject *k) {
  PyObject *o;
  if (CYTHON_COMPILING_IN_CPYTHON && likely((__pyx_freecount_4grpc_7_cython_6cygrpc___pyx_scope_struct_3_next_call_event > 0) & (t->tp_basicsize == sizeof(struct __pyx_obj_4grpc_7_cython_6cygrpc___pyx_scope_struct_3_next_call_event)))) {
    o = (PyObject*)__pyx_freelist_4grpc_7_cython_6cygrpc___pyx_scope_struct_3_next_call_event[--__pyx_freecount_4grpc_7_cython_6cygrpc___pyx_scope_struct_3_next_call_event];
    memset(o, 0, sizeof(struct __pyx_obj_4grpc_7_cython_6cygrpc___pyx_scope_struct_3_next_call_event));
    (void) PyObject_INIT(o, t);
    PyObject_GC_Track(o);
  } else {
    o = (*t->tp_alloc)(t, 0);
    if (unlikely(!o)) return 0;
  }
  return o;
}

static void __pyx_tp_dealloc_4grpc_7_cython_6cygrpc___pyx_scope_struct_3_next_call_event(PyObject *o) {
  struct __pyx_obj_4grpc_7_cython_6cygrpc___pyx_scope_struct_3_next_call_event *p = (struct __pyx_obj_4grpc_7_cython_6cygrpc___pyx_scope_struct_3_next_call_event *)o;
  PyObject_GC_UnTrack(o);
  Py_CLEAR(p->__pyx_v_self);
  if (CYTHON_COMPILING_IN_CPYTHON && ((__pyx_freecount_4grpc_7_cython_6cygrpc___pyx_scope_struct_3_next_call_event < 8) & (Py_TYPE(o)->tp_basicsize == sizeof(struct __pyx_obj_4grpc_7_cython_6cygrpc___pyx_scope_struct_3_next_call_event)))) {
    __pyx_freelist_4grpc_7_cython_6cygrpc___pyx_scope_struct_3_next_call_event[__pyx_freecount_4grpc_7_cython_6cygrpc___pyx_scope_struct_3_next_call_event++] = ((struct __pyx_obj_4grpc_7_cython_6cygrpc___pyx_scope_struct_3_next_call_event *)o);
  } else {
    (*Py_TYPE(o)->tp_free)(o);
  }
}

static int __pyx_tp_traverse_4grpc_7_cython_6cygrpc___pyx_scope_struct_3_next_call_event(PyObject *o, visitproc v, void *a) {
  int e;
  struct __pyx_obj_4grpc_7_cython_6cygrpc___pyx_scope_struct_3_next_call_event *p = (struct __pyx_obj_4grpc_7_cython_6cygrpc___pyx_scope_struct_3_next_call_event *)o;
  if (p->__pyx_v_self) {
    e = (*v)(((PyObject *)p->__pyx_v_self), a); if (e) return e;
  }
  return 0;
}

static int __pyx_tp_clear_4grpc_7_cython_6cygrpc___pyx_scope_struct_3_next_call_event(PyObject *o) {
  PyObject* tmp;
  struct __pyx_obj_4grpc_7_cython_6cygrpc___pyx_scope_struct_3_next_call_event *p = (struct __pyx_obj_4grpc_7_cython_6cygrpc___pyx_scope_struct_3_next_call_event *)o;
  tmp = ((PyObject*)p->__pyx_v_self);
  p->__pyx_v_self = ((struct __pyx_obj_4grpc_7_cython_6cygrpc_Channel *)Py_None); Py_INCREF(Py_None);
  Py_XDECREF(tmp);
  return 0;
}

static PyTypeObject __pyx_type_4grpc_7_cython_6cygrpc___pyx_scope_struct_3_next_call_event = {
  PyVarObject_HEAD_INIT(0, 0)
  "grpc._cython.cygrpc.__pyx_scope_struct_3_next_call_event", /*tp_name*/
  sizeof(struct __pyx_obj_4grpc_7_cython_6cygrpc___pyx_scope_struct_3_next_call_event), /*tp_basicsize*/
  0, /*tp_itemsize*/
  __pyx_tp_dealloc_4grpc_7_cython_6cygrpc___pyx_scope_struct_3_next_call_event, /*tp_dealloc*/
  0, /*tp_print*/
  0, /*tp_getattr*/
  0, /*tp_setattr*/
  #if PY_MAJOR_VERSION < 3
  0, /*tp_compare*/
  #endif
  #if PY_MAJOR_VERSION >= 3
  0, /*tp_as_async*/
  #endif
  0, /*tp_repr*/
  0, /*tp_as_number*/
  0, /*tp_as_sequence*/
  0, /*tp_as_mapping*/
  0, /*tp_hash*/
  0, /*tp_call*/
  0, /*tp_str*/
  0, /*tp_getattro*/
  0, /*tp_setattro*/
  0, /*tp_as_buffer*/
  Py_TPFLAGS_DEFAULT|Py_TPFLAGS_HAVE_VERSION_TAG|Py_TPFLAGS_CHECKTYPES|Py_TPFLAGS_HAVE_NEWBUFFER|Py_TPFLAGS_HAVE_GC, /*tp_flags*/
  0, /*tp_doc*/
  __pyx_tp_traverse_4grpc_7_cython_6cygrpc___pyx_scope_struct_3_next_call_event, /*tp_traverse*/
  __pyx_tp_clear_4grpc_7_cython_6cygrpc___pyx_scope_struct_3_next_call_event, /*tp_clear*/
  0, /*tp_richcompare*/
  0, /*tp_weaklistoffset*/
  0, /*tp_iter*/
  0, /*tp_iternext*/
  0, /*tp_methods*/
  0, /*tp_members*/
  0, /*tp_getset*/
  0, /*tp_base*/
  0, /*tp_dict*/
  0, /*tp_descr_get*/
  0, /*tp_descr_set*/
  0, /*tp_dictoffset*/
  0, /*tp_init*/
  0, /*tp_alloc*/
  __pyx_tp_new_4grpc_7_cython_6cygrpc___pyx_scope_struct_3_next_call_event, /*tp_new*/
  0, /*tp_free*/
  0, /*tp_is_gc*/
  0, /*tp_bases*/
  0, /*tp_mro*/
  0, /*tp_cache*/
  0, /*tp_subclasses*/
  0, /*tp_weaklist*/
  0, /*tp_del*/
  0, /*tp_version_tag*/
  #if PY_VERSION_HEX >= 0x030400a1
  0, /*tp_finalize*/
  #endif
};

static struct __pyx_obj_4grpc_7_cython_6cygrpc___pyx_scope_struct_4___pyx_f_4grpc_7_cython_6cygrpc__get_metadata *__pyx_freelist_4grpc_7_cython_6cygrpc___pyx_scope_struct_4___pyx_f_4grpc_7_cython_6cygrpc__get_metadata[8];
static int __pyx_freecount_4grpc_7_cython_6cygrpc___pyx_scope_struct_4___pyx_f_4grpc_7_cython_6cygrpc__get_metadata = 0;

static PyObject *__pyx_tp_new_4grpc_7_cython_6cygrpc___pyx_scope_struct_4___pyx_f_4grpc_7_cython_6cygrpc__get_metadata(PyTypeObject *t, CYTHON_UNUSED PyObject *a, CYTHON_UNUSED PyObject *k) {
  PyObject *o;
  if (CYTHON_COMPILING_IN_CPYTHON && likely((__pyx_freecount_4grpc_7_cython_6cygrpc___pyx_scope_struct_4___pyx_f_4grpc_7_cython_6cygrpc__get_metadata > 0) & (t->tp_basicsize == sizeof(struct __pyx_obj_4grpc_7_cython_6cygrpc___pyx_scope_struct_4___pyx_f_4grpc_7_cython_6cygrpc__get_metadata)))) {
    o = (PyObject*)__pyx_freelist_4grpc_7_cython_6cygrpc___pyx_scope_struct_4___pyx_f_4grpc_7_cython_6cygrpc__get_metadata[--__pyx_freecount_4grpc_7_cython_6cygrpc___pyx_scope_struct_4___pyx_f_4grpc_7_cython_6cygrpc__get_metadata];
    memset(o, 0, sizeof(struct __pyx_obj_4grpc_7_cython_6cygrpc___pyx_scope_struct_4___pyx_f_4grpc_7_cython_6cygrpc__get_metadata));
    (void) PyObject_INIT(o, t);
  } else {
    o = (*t->tp_alloc)(t, 0);
    if (unlikely(!o)) return 0;
  }
  return o;
}

static void __pyx_tp_dealloc_4grpc_7_cython_6cygrpc___pyx_scope_struct_4___pyx_f_4grpc_7_cython_6cygrpc__get_metadata(PyObject *o) {
  if (CYTHON_COMPILING_IN_CPYTHON && ((__pyx_freecount_4grpc_7_cython_6cygrpc___pyx_scope_struct_4___pyx_f_4grpc_7_cython_6cygrpc__get_metadata < 8) & (Py_TYPE(o)->tp_basicsize == sizeof(struct __pyx_obj_4grpc_7_cython_6cygrpc___pyx_scope_struct_4___pyx_f_4grpc_7_cython_6cygrpc__get_metadata)))) {
    __pyx_freelist_4grpc_7_cython_6cygrpc___pyx_scope_struct_4___pyx_f_4grpc_7_cython_6cygrpc__get_metadata[__pyx_freecount_4grpc_7_cython_6cygrpc___pyx_scope_struct_4___pyx_f_4grpc_7_cython_6cygrpc__get_metadata++] = ((struct __pyx_obj_4grpc_7_cython_6cygrpc___pyx_scope_struct_4___pyx_f_4grpc_7_cython_6cygrpc__get_metadata *)o);
  } else {
    (*Py_TYPE(o)->tp_free)(o);
  }
}

static PyTypeObject __pyx_type_4grpc_7_cython_6cygrpc___pyx_scope_struct_4___pyx_f_4grpc_7_cython_6cygrpc__get_metadata = {
  PyVarObject_HEAD_INIT(0, 0)
  "grpc._cython.cygrpc.__pyx_scope_struct_4___pyx_f_4grpc_7_cython_6cygrpc__get_metadata", /*tp_name*/
  sizeof(struct __pyx_obj_4grpc_7_cython_6cygrpc___pyx_scope_struct_4___pyx_f_4grpc_7_cython_6cygrpc__get_metadata), /*tp_basicsize*/
  0, /*tp_itemsize*/
  __pyx_tp_dealloc_4grpc_7_cython_6cygrpc___pyx_scope_struct_4___pyx_f_4grpc_7_cython_6cygrpc__get_metadata, /*tp_dealloc*/
  0, /*tp_print*/
  0, /*tp_getattr*/
  0, /*tp_setattr*/
  #if PY_MAJOR_VERSION < 3
  0, /*tp_compare*/
  #endif
  #if PY_MAJOR_VERSION >= 3
  0, /*tp_as_async*/
  #endif
  0, /*tp_repr*/
  0, /*tp_as_number*/
  0, /*tp_as_sequence*/
  0, /*tp_as_mapping*/
  0, /*tp_hash*/
  0, /*tp_call*/
  0, /*tp_str*/
  0, /*tp_getattro*/
  0, /*tp_setattro*/
  0, /*tp_as_buffer*/
  Py_TPFLAGS_DEFAULT|Py_TPFLAGS_HAVE_VERSION_TAG|Py_TPFLAGS_CHECKTYPES|Py_TPFLAGS_HAVE_NEWBUFFER, /*tp_flags*/
  0, /*tp_doc*/
  0, /*tp_traverse*/
  0, /*tp_clear*/
  0, /*tp_richcompare*/
  0, /*tp_weaklistoffset*/
  0, /*tp_iter*/
  0, /*tp_iternext*/
  0, /*tp_methods*/
  0, /*tp_members*/
  0, /*tp_getset*/
  0, /*tp_base*/
  0, /*tp_dict*/
  0, /*tp_descr_get*/
  0, /*tp_descr_set*/
  0, /*tp_dictoffset*/
  0, /*tp_init*/
  0, /*tp_alloc*/
  __pyx_tp_new_4grpc_7_cython_6cygrpc___pyx_scope_struct_4___pyx_f_4grpc_7_cython_6cygrpc__get_metadata, /*tp_new*/
  0, /*tp_free*/
  0, /*tp_is_gc*/
  0, /*tp_bases*/
  0, /*tp_mro*/
  0, /*tp_cache*/
  0, /*tp_subclasses*/
  0, /*tp_weaklist*/
  0, /*tp_del*/
  0, /*tp_version_tag*/
  #if PY_VERSION_HEX >= 0x030400a1
  0, /*tp_finalize*/
  #endif
};

static struct __pyx_obj_4grpc_7_cython_6cygrpc___pyx_scope_struct_5___pyx_f_4grpc_7_cython_6cygrpc__metadata *__pyx_freelist_4grpc_7_cython_6cygrpc___pyx_scope_struct_5___pyx_f_4grpc_7_cython_6cygrpc__metadata[8];
static int __pyx_freecount_4grpc_7_cython_6cygrpc___pyx_scope_struct_5___pyx_f_4grpc_7_cython_6cygrpc__metadata = 0;

static PyObject *__pyx_tp_new_4grpc_7_cython_6cygrpc___pyx_scope_struct_5___pyx_f_4grpc_7_cython_6cygrpc__metadata(PyTypeObject *t, CYTHON_UNUSED PyObject *a, CYTHON_UNUSED PyObject *k) {
  PyObject *o;
  if (CYTHON_COMPILING_IN_CPYTHON && likely((__pyx_freecount_4grpc_7_cython_6cygrpc___pyx_scope_struct_5___pyx_f_4grpc_7_cython_6cygrpc__metadata > 0) & (t->tp_basicsize == sizeof(struct __pyx_obj_4grpc_7_cython_6cygrpc___pyx_scope_struct_5___pyx_f_4grpc_7_cython_6cygrpc__metadata)))) {
    o = (PyObject*)__pyx_freelist_4grpc_7_cython_6cygrpc___pyx_scope_struct_5___pyx_f_4grpc_7_cython_6cygrpc__metadata[--__pyx_freecount_4grpc_7_cython_6cygrpc___pyx_scope_struct_5___pyx_f_4grpc_7_cython_6cygrpc__metadata];
    memset(o, 0, sizeof(struct __pyx_obj_4grpc_7_cython_6cygrpc___pyx_scope_struct_5___pyx_f_4grpc_7_cython_6cygrpc__metadata));
    (void) PyObject_INIT(o, t);
  } else {
    o = (*t->tp_alloc)(t, 0);
    if (unlikely(!o)) return 0;
  }
  return o;
}

static void __pyx_tp_dealloc_4grpc_7_cython_6cygrpc___pyx_scope_struct_5___pyx_f_4grpc_7_cython_6cygrpc__metadata(PyObject *o) {
  if (CYTHON_COMPILING_IN_CPYTHON && ((__pyx_freecount_4grpc_7_cython_6cygrpc___pyx_scope_struct_5___pyx_f_4grpc_7_cython_6cygrpc__metadata < 8) & (Py_TYPE(o)->tp_basicsize == sizeof(struct __pyx_obj_4grpc_7_cython_6cygrpc___pyx_scope_struct_5___pyx_f_4grpc_7_cython_6cygrpc__metadata)))) {
    __pyx_freelist_4grpc_7_cython_6cygrpc___pyx_scope_struct_5___pyx_f_4grpc_7_cython_6cygrpc__metadata[__pyx_freecount_4grpc_7_cython_6cygrpc___pyx_scope_struct_5___pyx_f_4grpc_7_cython_6cygrpc__metadata++] = ((struct __pyx_obj_4grpc_7_cython_6cygrpc___pyx_scope_struct_5___pyx_f_4grpc_7_cython_6cygrpc__metadata *)o);
  } else {
    (*Py_TYPE(o)->tp_free)(o);
  }
}

static PyTypeObject __pyx_type_4grpc_7_cython_6cygrpc___pyx_scope_struct_5___pyx_f_4grpc_7_cython_6cygrpc__metadata = {
  PyVarObject_HEAD_INIT(0, 0)
  "grpc._cython.cygrpc.__pyx_scope_struct_5___pyx_f_4grpc_7_cython_6cygrpc__metadata", /*tp_name*/
  sizeof(struct __pyx_obj_4grpc_7_cython_6cygrpc___pyx_scope_struct_5___pyx_f_4grpc_7_cython_6cygrpc__metadata), /*tp_basicsize*/
  0, /*tp_itemsize*/
  __pyx_tp_dealloc_4grpc_7_cython_6cygrpc___pyx_scope_struct_5___pyx_f_4grpc_7_cython_6cygrpc__metadata, /*tp_dealloc*/
  0, /*tp_print*/
  0, /*tp_getattr*/
  0, /*tp_setattr*/
  #if PY_MAJOR_VERSION < 3
  0, /*tp_compare*/
  #endif
  #if PY_MAJOR_VERSION >= 3
  0, /*tp_as_async*/
  #endif
  0, /*tp_repr*/
  0, /*tp_as_number*/
  0, /*tp_as_sequence*/
  0, /*tp_as_mapping*/
  0, /*tp_hash*/
  0, /*tp_call*/
  0, /*tp_str*/
  0, /*tp_getattro*/
  0, /*tp_setattro*/
  0, /*tp_as_buffer*/
  Py_TPFLAGS_DEFAULT|Py_TPFLAGS_HAVE_VERSION_TAG|Py_TPFLAGS_CHECKTYPES|Py_TPFLAGS_HAVE_NEWBUFFER, /*tp_flags*/
  0, /*tp_doc*/
  0, /*tp_traverse*/
  0, /*tp_clear*/
  0, /*tp_richcompare*/
  0, /*tp_weaklistoffset*/
  0, /*tp_iter*/
  0, /*tp_iternext*/
  0, /*tp_methods*/
  0, /*tp_members*/
  0, /*tp_getset*/
  0, /*tp_base*/
  0, /*tp_dict*/
  0, /*tp_descr_get*/
  0, /*tp_descr_set*/
  0, /*tp_dictoffset*/
  0, /*tp_init*/
  0, /*tp_alloc*/
  __pyx_tp_new_4grpc_7_cython_6cygrpc___pyx_scope_struct_5___pyx_f_4grpc_7_cython_6cygrpc__metadata, /*tp_new*/
  0, /*tp_free*/
  0, /*tp_is_gc*/
  0, /*tp_bases*/
  0, /*tp_mro*/
  0, /*tp_cache*/
  0, /*tp_subclasses*/
  0, /*tp_weaklist*/
  0, /*tp_del*/
  0, /*tp_version_tag*/
  #if PY_VERSION_HEX >= 0x030400a1
  0, /*tp_finalize*/
  #endif
};

static struct __pyx_obj_4grpc_7_cython_6cygrpc___pyx_scope_struct_6_genexpr *__pyx_freelist_4grpc_7_cython_6cygrpc___pyx_scope_struct_6_genexpr[8];
static int __pyx_freecount_4grpc_7_cython_6cygrpc___pyx_scope_struct_6_genexpr = 0;

static PyObject *__pyx_tp_new_4grpc_7_cython_6cygrpc___pyx_scope_struct_6_genexpr(PyTypeObject *t, CYTHON_UNUSED PyObject *a, CYTHON_UNUSED PyObject *k) {
  PyObject *o;
  if (CYTHON_COMPILING_IN_CPYTHON && likely((__pyx_freecount_4grpc_7_cython_6cygrpc___pyx_scope_struct_6_genexpr > 0) & (t->tp_basicsize == sizeof(struct __pyx_obj_4grpc_7_cython_6cygrpc___pyx_scope_struct_6_genexpr)))) {
    o = (PyObject*)__pyx_freelist_4grpc_7_cython_6cygrpc___pyx_scope_struct_6_genexpr[--__pyx_freecount_4grpc_7_cython_6cygrpc___pyx_scope_struct_6_genexpr];
    memset(o, 0, sizeof(struct __pyx_obj_4grpc_7_cython_6cygrpc___pyx_scope_struct_6_genexpr));
    (void) PyObject_INIT(o, t);
    PyObject_GC_Track(o);
  } else {
    o = (*t->tp_alloc)(t, 0);
    if (unlikely(!o)) return 0;
  }
  return o;
}

static void __pyx_tp_dealloc_4grpc_7_cython_6cygrpc___pyx_scope_struct_6_genexpr(PyObject *o) {
  struct __pyx_obj_4grpc_7_cython_6cygrpc___pyx_scope_struct_6_genexpr *p = (struct __pyx_obj_4grpc_7_cython_6cygrpc___pyx_scope_struct_6_genexpr *)o;
  PyObject_GC_UnTrack(o);
  Py_CLEAR(p->__pyx_outer_scope);
  if (CYTHON_COMPILING_IN_CPYTHON && ((__pyx_freecount_4grpc_7_cython_6cygrpc___pyx_scope_struct_6_genexpr < 8) & (Py_TYPE(o)->tp_basicsize == sizeof(struct __pyx_obj_4grpc_7_cython_6cygrpc___pyx_scope_struct_6_genexpr)))) {
    __pyx_freelist_4grpc_7_cython_6cygrpc___pyx_scope_struct_6_genexpr[__pyx_freecount_4grpc_7_cython_6cygrpc___pyx_scope_struct_6_genexpr++] = ((struct __pyx_obj_4grpc_7_cython_6cygrpc___pyx_scope_struct_6_genexpr *)o);
  } else {
    (*Py_TYPE(o)->tp_free)(o);
  }
}

static int __pyx_tp_traverse_4grpc_7_cython_6cygrpc___pyx_scope_struct_6_genexpr(PyObject *o, visitproc v, void *a) {
  int e;
  struct __pyx_obj_4grpc_7_cython_6cygrpc___pyx_scope_struct_6_genexpr *p = (struct __pyx_obj_4grpc_7_cython_6cygrpc___pyx_scope_struct_6_genexpr *)o;
  if (p->__pyx_outer_scope) {
    e = (*v)(((PyObject *)p->__pyx_outer_scope), a); if (e) return e;
  }
  return 0;
}

static PyTypeObject __pyx_type_4grpc_7_cython_6cygrpc___pyx_scope_struct_6_genexpr = {
  PyVarObject_HEAD_INIT(0, 0)
  "grpc._cython.cygrpc.__pyx_scope_struct_6_genexpr", /*tp_name*/
  sizeof(struct __pyx_obj_4grpc_7_cython_6cygrpc___pyx_scope_struct_6_genexpr), /*tp_basicsize*/
  0, /*tp_itemsize*/
  __pyx_tp_dealloc_4grpc_7_cython_6cygrpc___pyx_scope_struct_6_genexpr, /*tp_dealloc*/
  0, /*tp_print*/
  0, /*tp_getattr*/
  0, /*tp_setattr*/
  #if PY_MAJOR_VERSION < 3
  0, /*tp_compare*/
  #endif
  #if PY_MAJOR_VERSION >= 3
  0, /*tp_as_async*/
  #endif
  0, /*tp_repr*/
  0, /*tp_as_number*/
  0, /*tp_as_sequence*/
  0, /*tp_as_mapping*/
  0, /*tp_hash*/
  0, /*tp_call*/
  0, /*tp_str*/
  0, /*tp_getattro*/
  0, /*tp_setattro*/
  0, /*tp_as_buffer*/
  Py_TPFLAGS_DEFAULT|Py_TPFLAGS_HAVE_VERSION_TAG|Py_TPFLAGS_CHECKTYPES|Py_TPFLAGS_HAVE_NEWBUFFER|Py_TPFLAGS_HAVE_GC, /*tp_flags*/
  0, /*tp_doc*/
  __pyx_tp_traverse_4grpc_7_cython_6cygrpc___pyx_scope_struct_6_genexpr, /*tp_traverse*/
  0, /*tp_clear*/
  0, /*tp_richcompare*/
  0, /*tp_weaklistoffset*/
  0, /*tp_iter*/
  0, /*tp_iternext*/
  0, /*tp_methods*/
  0, /*tp_members*/
  0, /*tp_getset*/
  0, /*tp_base*/
  0, /*tp_dict*/
  0, /*tp_descr_get*/
  0, /*tp_descr_set*/
  0, /*tp_dictoffset*/
  0, /*tp_init*/
  0, /*tp_alloc*/
  __pyx_tp_new_4grpc_7_cython_6cygrpc___pyx_scope_struct_6_genexpr, /*tp_new*/
  0, /*tp_free*/
  0, /*tp_is_gc*/
  0, /*tp_bases*/
  0, /*tp_mro*/
  0, /*tp_cache*/
  0, /*tp_subclasses*/
  0, /*tp_weaklist*/
  0, /*tp_del*/
  0, /*tp_version_tag*/
  #if PY_VERSION_HEX >= 0x030400a1
  0, /*tp_finalize*/
  #endif
};

static struct __pyx_obj_4grpc_7_cython_6cygrpc___pyx_scope_struct_7___pyx_f_4grpc_7_cython_6cygrpc_tuples_to_resolvaddr *__pyx_freelist_4grpc_7_cython_6cygrpc___pyx_scope_struct_7___pyx_f_4grpc_7_cython_6cygrpc_tuples_to_resolvaddr[8];
static int __pyx_freecount_4grpc_7_cython_6cygrpc___pyx_scope_struct_7___pyx_f_4grpc_7_cython_6cygrpc_tuples_to_resolvaddr = 0;

static PyObject *__pyx_tp_new_4grpc_7_cython_6cygrpc___pyx_scope_struct_7___pyx_f_4grpc_7_cython_6cygrpc_tuples_to_resolvaddr(PyTypeObject *t, CYTHON_UNUSED PyObject *a, CYTHON_UNUSED PyObject *k) {
  PyObject *o;
  if (CYTHON_COMPILING_IN_CPYTHON && likely((__pyx_freecount_4grpc_7_cython_6cygrpc___pyx_scope_struct_7___pyx_f_4grpc_7_cython_6cygrpc_tuples_to_resolvaddr > 0) & (t->tp_basicsize == sizeof(struct __pyx_obj_4grpc_7_cython_6cygrpc___pyx_scope_struct_7___pyx_f_4grpc_7_cython_6cygrpc_tuples_to_resolvaddr)))) {
    o = (PyObject*)__pyx_freelist_4grpc_7_cython_6cygrpc___pyx_scope_struct_7___pyx_f_4grpc_7_cython_6cygrpc_tuples_to_resolvaddr[--__pyx_freecount_4grpc_7_cython_6cygrpc___pyx_scope_struct_7___pyx_f_4grpc_7_cython_6cygrpc_tuples_to_resolvaddr];
    memset(o, 0, sizeof(struct __pyx_obj_4grpc_7_cython_6cygrpc___pyx_scope_struct_7___pyx_f_4grpc_7_cython_6cygrpc_tuples_to_resolvaddr));
    (void) PyObject_INIT(o, t);
    PyObject_GC_Track(o);
  } else {
    o = (*t->tp_alloc)(t, 0);
    if (unlikely(!o)) return 0;
  }
  return o;
}

static void __pyx_tp_dealloc_4grpc_7_cython_6cygrpc___pyx_scope_struct_7___pyx_f_4grpc_7_cython_6cygrpc_tuples_to_resolvaddr(PyObject *o) {
  struct __pyx_obj_4grpc_7_cython_6cygrpc___pyx_scope_struct_7___pyx_f_4grpc_7_cython_6cygrpc_tuples_to_resolvaddr *p = (struct __pyx_obj_4grpc_7_cython_6cygrpc___pyx_scope_struct_7___pyx_f_4grpc_7_cython_6cygrpc_tuples_to_resolvaddr *)o;
  PyObject_GC_UnTrack(o);
  Py_CLEAR(p->__pyx_v_tups);
  if (CYTHON_COMPILING_IN_CPYTHON && ((__pyx_freecount_4grpc_7_cython_6cygrpc___pyx_scope_struct_7___pyx_f_4grpc_7_cython_6cygrpc_tuples_to_resolvaddr < 8) & (Py_TYPE(o)->tp_basicsize == sizeof(struct __pyx_obj_4grpc_7_cython_6cygrpc___pyx_scope_struct_7___pyx_f_4grpc_7_cython_6cygrpc_tuples_to_resolvaddr)))) {
    __pyx_freelist_4grpc_7_cython_6cygrpc___pyx_scope_struct_7___pyx_f_4grpc_7_cython_6cygrpc_tuples_to_resolvaddr[__pyx_freecount_4grpc_7_cython_6cygrpc___pyx_scope_struct_7___pyx_f_4grpc_7_cython_6cygrpc_tuples_to_resolvaddr++] = ((struct __pyx_obj_4grpc_7_cython_6cygrpc___pyx_scope_struct_7___pyx_f_4grpc_7_cython_6cygrpc_tuples_to_resolvaddr *)o);
  } else {
    (*Py_TYPE(o)->tp_free)(o);
  }
}

static int __pyx_tp_traverse_4grpc_7_cython_6cygrpc___pyx_scope_struct_7___pyx_f_4grpc_7_cython_6cygrpc_tuples_to_resolvaddr(PyObject *o, visitproc v, void *a) {
  int e;
  struct __pyx_obj_4grpc_7_cython_6cygrpc___pyx_scope_struct_7___pyx_f_4grpc_7_cython_6cygrpc_tuples_to_resolvaddr *p = (struct __pyx_obj_4grpc_7_cython_6cygrpc___pyx_scope_struct_7___pyx_f_4grpc_7_cython_6cygrpc_tuples_to_resolvaddr *)o;
  if (p->__pyx_v_tups) {
    e = (*v)(p->__pyx_v_tups, a); if (e) return e;
  }
  return 0;
}

static int __pyx_tp_clear_4grpc_7_cython_6cygrpc___pyx_scope_struct_7___pyx_f_4grpc_7_cython_6cygrpc_tuples_to_resolvaddr(PyObject *o) {
  PyObject* tmp;
  struct __pyx_obj_4grpc_7_cython_6cygrpc___pyx_scope_struct_7___pyx_f_4grpc_7_cython_6cygrpc_tuples_to_resolvaddr *p = (struct __pyx_obj_4grpc_7_cython_6cygrpc___pyx_scope_struct_7___pyx_f_4grpc_7_cython_6cygrpc_tuples_to_resolvaddr *)o;
  tmp = ((PyObject*)p->__pyx_v_tups);
  p->__pyx_v_tups = Py_None; Py_INCREF(Py_None);
  Py_XDECREF(tmp);
  return 0;
}

static PyTypeObject __pyx_type_4grpc_7_cython_6cygrpc___pyx_scope_struct_7___pyx_f_4grpc_7_cython_6cygrpc_tuples_to_resolvaddr = {
  PyVarObject_HEAD_INIT(0, 0)
  "grpc._cython.cygrpc.__pyx_scope_struct_7___pyx_f_4grpc_7_cython_6cygrpc_tuples_to_resolvaddr", /*tp_name*/
  sizeof(struct __pyx_obj_4grpc_7_cython_6cygrpc___pyx_scope_struct_7___pyx_f_4grpc_7_cython_6cygrpc_tuples_to_resolvaddr), /*tp_basicsize*/
  0, /*tp_itemsize*/
  __pyx_tp_dealloc_4grpc_7_cython_6cygrpc___pyx_scope_struct_7___pyx_f_4grpc_7_cython_6cygrpc_tuples_to_resolvaddr, /*tp_dealloc*/
  0, /*tp_print*/
  0, /*tp_getattr*/
  0, /*tp_setattr*/
  #if PY_MAJOR_VERSION < 3
  0, /*tp_compare*/
  #endif
  #if PY_MAJOR_VERSION >= 3
  0, /*tp_as_async*/
  #endif
  0, /*tp_repr*/
  0, /*tp_as_number*/
  0, /*tp_as_sequence*/
  0, /*tp_as_mapping*/
  0, /*tp_hash*/
  0, /*tp_call*/
  0, /*tp_str*/
  0, /*tp_getattro*/
  0, /*tp_setattro*/
  0, /*tp_as_buffer*/
  Py_TPFLAGS_DEFAULT|Py_TPFLAGS_HAVE_VERSION_TAG|Py_TPFLAGS_CHECKTYPES|Py_TPFLAGS_HAVE_NEWBUFFER|Py_TPFLAGS_HAVE_GC, /*tp_flags*/
  0, /*tp_doc*/
  __pyx_tp_traverse_4grpc_7_cython_6cygrpc___pyx_scope_struct_7___pyx_f_4grpc_7_cython_6cygrpc_tuples_to_resolvaddr, /*tp_traverse*/
  __pyx_tp_clear_4grpc_7_cython_6cygrpc___pyx_scope_struct_7___pyx_f_4grpc_7_cython_6cygrpc_tuples_to_resolvaddr, /*tp_clear*/
  0, /*tp_richcompare*/
  0, /*tp_weaklistoffset*/
  0, /*tp_iter*/
  0, /*tp_iternext*/
  0, /*tp_methods*/
  0, /*tp_members*/
  0, /*tp_getset*/
  0, /*tp_base*/
  0, /*tp_dict*/
  0, /*tp_descr_get*/
  0, /*tp_descr_set*/
  0, /*tp_dictoffset*/
  0, /*tp_init*/
  0, /*tp_alloc*/
  __pyx_tp_new_4grpc_7_cython_6cygrpc___pyx_scope_struct_7___pyx_f_4grpc_7_cython_6cygrpc_tuples_to_resolvaddr, /*tp_new*/
  0, /*tp_free*/
  0, /*tp_is_gc*/
  0, /*tp_bases*/
  0, /*tp_mro*/
  0, /*tp_cache*/
  0, /*tp_subclasses*/
  0, /*tp_weaklist*/
  0, /*tp_del*/
  0, /*tp_version_tag*/
  #if PY_VERSION_HEX >= 0x030400a1
  0, /*tp_finalize*/
  #endif
};

static struct __pyx_obj_4grpc_7_cython_6cygrpc___pyx_scope_struct_8_genexpr *__pyx_freelist_4grpc_7_cython_6cygrpc___pyx_scope_struct_8_genexpr[8];
static int __pyx_freecount_4grpc_7_cython_6cygrpc___pyx_scope_struct_8_genexpr = 0;

static PyObject *__pyx_tp_new_4grpc_7_cython_6cygrpc___pyx_scope_struct_8_genexpr(PyTypeObject *t, CYTHON_UNUSED PyObject *a, CYTHON_UNUSED PyObject *k) {
  PyObject *o;
  if (CYTHON_COMPILING_IN_CPYTHON && likely((__pyx_freecount_4grpc_7_cython_6cygrpc___pyx_scope_struct_8_genexpr > 0) & (t->tp_basicsize == sizeof(struct __pyx_obj_4grpc_7_cython_6cygrpc___pyx_scope_struct_8_genexpr)))) {
    o = (PyObject*)__pyx_freelist_4grpc_7_cython_6cygrpc___pyx_scope_struct_8_genexpr[--__pyx_freecount_4grpc_7_cython_6cygrpc___pyx_scope_struct_8_genexpr];
    memset(o, 0, sizeof(struct __pyx_obj_4grpc_7_cython_6cygrpc___pyx_scope_struct_8_genexpr));
    (void) PyObject_INIT(o, t);
    PyObject_GC_Track(o);
  } else {
    o = (*t->tp_alloc)(t, 0);
    if (unlikely(!o)) return 0;
  }
  return o;
}

static void __pyx_tp_dealloc_4grpc_7_cython_6cygrpc___pyx_scope_struct_8_genexpr(PyObject *o) {
  struct __pyx_obj_4grpc_7_cython_6cygrpc___pyx_scope_struct_8_genexpr *p = (struct __pyx_obj_4grpc_7_cython_6cygrpc___pyx_scope_struct_8_genexpr *)o;
  PyObject_GC_UnTrack(o);
  Py_CLEAR(p->__pyx_outer_scope);
  Py_CLEAR(p->__pyx_v_tup);
  if (CYTHON_COMPILING_IN_CPYTHON && ((__pyx_freecount_4grpc_7_cython_6cygrpc___pyx_scope_struct_8_genexpr < 8) & (Py_TYPE(o)->tp_basicsize == sizeof(struct __pyx_obj_4grpc_7_cython_6cygrpc___pyx_scope_struct_8_genexpr)))) {
    __pyx_freelist_4grpc_7_cython_6cygrpc___pyx_scope_struct_8_genexpr[__pyx_freecount_4grpc_7_cython_6cygrpc___pyx_scope_struct_8_genexpr++] = ((struct __pyx_obj_4grpc_7_cython_6cygrpc___pyx_scope_struct_8_genexpr *)o);
  } else {
    (*Py_TYPE(o)->tp_free)(o);
  }
}

static int __pyx_tp_traverse_4grpc_7_cython_6cygrpc___pyx_scope_struct_8_genexpr(PyObject *o, visitproc v, void *a) {
  int e;
  struct __pyx_obj_4grpc_7_cython_6cygrpc___pyx_scope_struct_8_genexpr *p = (struct __pyx_obj_4grpc_7_cython_6cygrpc___pyx_scope_struct_8_genexpr *)o;
  if (p->__pyx_outer_scope) {
    e = (*v)(((PyObject *)p->__pyx_outer_scope), a); if (e) return e;
  }
  if (p->__pyx_v_tup) {
    e = (*v)(p->__pyx_v_tup, a); if (e) return e;
  }
  return 0;
}

static PyTypeObject __pyx_type_4grpc_7_cython_6cygrpc___pyx_scope_struct_8_genexpr = {
  PyVarObject_HEAD_INIT(0, 0)
  "grpc._cython.cygrpc.__pyx_scope_struct_8_genexpr", /*tp_name*/
  sizeof(struct __pyx_obj_4grpc_7_cython_6cygrpc___pyx_scope_struct_8_genexpr), /*tp_basicsize*/
  0, /*tp_itemsize*/
  __pyx_tp_dealloc_4grpc_7_cython_6cygrpc___pyx_scope_struct_8_genexpr, /*tp_dealloc*/
  0, /*tp_print*/
  0, /*tp_getattr*/
  0, /*tp_setattr*/
  #if PY_MAJOR_VERSION < 3
  0, /*tp_compare*/
  #endif
  #if PY_MAJOR_VERSION >= 3
  0, /*tp_as_async*/
  #endif
  0, /*tp_repr*/
  0, /*tp_as_number*/
  0, /*tp_as_sequence*/
  0, /*tp_as_mapping*/
  0, /*tp_hash*/
  0, /*tp_call*/
  0, /*tp_str*/
  0, /*tp_getattro*/
  0, /*tp_setattro*/
  0, /*tp_as_buffer*/
  Py_TPFLAGS_DEFAULT|Py_TPFLAGS_HAVE_VERSION_TAG|Py_TPFLAGS_CHECKTYPES|Py_TPFLAGS_HAVE_NEWBUFFER|Py_TPFLAGS_HAVE_GC, /*tp_flags*/
  0, /*tp_doc*/
  __pyx_tp_traverse_4grpc_7_cython_6cygrpc___pyx_scope_struct_8_genexpr, /*tp_traverse*/
  0, /*tp_clear*/
  0, /*tp_richcompare*/
  0, /*tp_weaklistoffset*/
  0, /*tp_iter*/
  0, /*tp_iternext*/
  0, /*tp_methods*/
  0, /*tp_members*/
  0, /*tp_getset*/
  0, /*tp_base*/
  0, /*tp_dict*/
  0, /*tp_descr_get*/
  0, /*tp_descr_set*/
  0, /*tp_dictoffset*/
  0, /*tp_init*/
  0, /*tp_alloc*/
  __pyx_tp_new_4grpc_7_cython_6cygrpc___pyx_scope_struct_8_genexpr, /*tp_new*/
  0, /*tp_free*/
  0, /*tp_is_gc*/
  0, /*tp_bases*/
  0, /*tp_mro*/
  0, /*tp_cache*/
  0, /*tp_subclasses*/
  0, /*tp_weaklist*/
  0, /*tp_del*/
  0, /*tp_version_tag*/
  #if PY_VERSION_HEX >= 0x030400a1
  0, /*tp_finalize*/
  #endif
};

static struct __pyx_obj_4grpc_7_cython_6cygrpc___pyx_scope_struct_9___init__ *__pyx_freelist_4grpc_7_cython_6cygrpc___pyx_scope_struct_9___init__[8];
static int __pyx_freecount_4grpc_7_cython_6cygrpc___pyx_scope_struct_9___init__ = 0;

static PyObject *__pyx_tp_new_4grpc_7_cython_6cygrpc___pyx_scope_struct_9___init__(PyTypeObject *t, CYTHON_UNUSED PyObject *a, CYTHON_UNUSED PyObject *k) {
  PyObject *o;
  if (CYTHON_COMPILING_IN_CPYTHON && likely((__pyx_freecount_4grpc_7_cython_6cygrpc___pyx_scope_struct_9___init__ > 0) & (t->tp_basicsize == sizeof(struct __pyx_obj_4grpc_7_cython_6cygrpc___pyx_scope_struct_9___init__)))) {
    o = (PyObject*)__pyx_freelist_4grpc_7_cython_6cygrpc___pyx_scope_struct_9___init__[--__pyx_freecount_4grpc_7_cython_6cygrpc___pyx_scope_struct_9___init__];
    memset(o, 0, sizeof(struct __pyx_obj_4grpc_7_cython_6cygrpc___pyx_scope_struct_9___init__));
    (void) PyObject_INIT(o, t);
    PyObject_GC_Track(o);
  } else {
    o = (*t->tp_alloc)(t, 0);
    if (unlikely(!o)) return 0;
  }
  return o;
}

static void __pyx_tp_dealloc_4grpc_7_cython_6cygrpc___pyx_scope_struct_9___init__(PyObject *o) {
  struct __pyx_obj_4grpc_7_cython_6cygrpc___pyx_scope_struct_9___init__ *p = (struct __pyx_obj_4grpc_7_cython_6cygrpc___pyx_scope_struct_9___init__ *)o;
  PyObject_GC_UnTrack(o);
  Py_CLEAR(p->__pyx_v_target);
  if (CYTHON_COMPILING_IN_CPYTHON && ((__pyx_freecount_4grpc_7_cython_6cygrpc___pyx_scope_struct_9___init__ < 8) & (Py_TYPE(o)->tp_basicsize == sizeof(struct __pyx_obj_4grpc_7_cython_6cygrpc___pyx_scope_struct_9___init__)))) {
    __pyx_freelist_4grpc_7_cython_6cygrpc___pyx_scope_struct_9___init__[__pyx_freecount_4grpc_7_cython_6cygrpc___pyx_scope_struct_9___init__++] = ((struct __pyx_obj_4grpc_7_cython_6cygrpc___pyx_scope_struct_9___init__ *)o);
  } else {
    (*Py_TYPE(o)->tp_free)(o);
  }
}

static int __pyx_tp_traverse_4grpc_7_cython_6cygrpc___pyx_scope_struct_9___init__(PyObject *o, visitproc v, void *a) {
  int e;
  struct __pyx_obj_4grpc_7_cython_6cygrpc___pyx_scope_struct_9___init__ *p = (struct __pyx_obj_4grpc_7_cython_6cygrpc___pyx_scope_struct_9___init__ *)o;
  if (p->__pyx_v_target) {
    e = (*v)(p->__pyx_v_target, a); if (e) return e;
  }
  return 0;
}

static int __pyx_tp_clear_4grpc_7_cython_6cygrpc___pyx_scope_struct_9___init__(PyObject *o) {
  PyObject* tmp;
  struct __pyx_obj_4grpc_7_cython_6cygrpc___pyx_scope_struct_9___init__ *p = (struct __pyx_obj_4grpc_7_cython_6cygrpc___pyx_scope_struct_9___init__ *)o;
  tmp = ((PyObject*)p->__pyx_v_target);
  p->__pyx_v_target = Py_None; Py_INCREF(Py_None);
  Py_XDECREF(tmp);
  return 0;
}

static PyTypeObject __pyx_type_4grpc_7_cython_6cygrpc___pyx_scope_struct_9___init__ = {
  PyVarObject_HEAD_INIT(0, 0)
  "grpc._cython.cygrpc.__pyx_scope_struct_9___init__", /*tp_name*/
  sizeof(struct __pyx_obj_4grpc_7_cython_6cygrpc___pyx_scope_struct_9___init__), /*tp_basicsize*/
  0, /*tp_itemsize*/
  __pyx_tp_dealloc_4grpc_7_cython_6cygrpc___pyx_scope_struct_9___init__, /*tp_dealloc*/
  0, /*tp_print*/
  0, /*tp_getattr*/
  0, /*tp_setattr*/
  #if PY_MAJOR_VERSION < 3
  0, /*tp_compare*/
  #endif
  #if PY_MAJOR_VERSION >= 3
  0, /*tp_as_async*/
  #endif
  0, /*tp_repr*/
  0, /*tp_as_number*/
  0, /*tp_as_sequence*/
  0, /*tp_as_mapping*/
  0, /*tp_hash*/
  0, /*tp_call*/
  0, /*tp_str*/
  0, /*tp_getattro*/
  0, /*tp_setattro*/
  0, /*tp_as_buffer*/
  Py_TPFLAGS_DEFAULT|Py_TPFLAGS_HAVE_VERSION_TAG|Py_TPFLAGS_CHECKTYPES|Py_TPFLAGS_HAVE_NEWBUFFER|Py_TPFLAGS_HAVE_GC, /*tp_flags*/
  0, /*tp_doc*/
  __pyx_tp_traverse_4grpc_7_cython_6cygrpc___pyx_scope_struct_9___init__, /*tp_traverse*/
  __pyx_tp_clear_4grpc_7_cython_6cygrpc___pyx_scope_struct_9___init__, /*tp_clear*/
  0, /*tp_richcompare*/
  0, /*tp_weaklistoffset*/
  0, /*tp_iter*/
  0, /*tp_iternext*/
  0, /*tp_methods*/
  0, /*tp_members*/
  0, /*tp_getset*/
  0, /*tp_base*/
  0, /*tp_dict*/
  0, /*tp_descr_get*/
  0, /*tp_descr_set*/
  0, /*tp_dictoffset*/
  0, /*tp_init*/
  0, /*tp_alloc*/
  __pyx_tp_new_4grpc_7_cython_6cygrpc___pyx_scope_struct_9___init__, /*tp_new*/
  0, /*tp_free*/
  0, /*tp_is_gc*/
  0, /*tp_bases*/
  0, /*tp_mro*/
  0, /*tp_cache*/
  0, /*tp_subclasses*/
  0, /*tp_weaklist*/
  0, /*tp_del*/
  0, /*tp_version_tag*/
  #if PY_VERSION_HEX >= 0x030400a1
  0, /*tp_finalize*/
  #endif
};

static PyMethodDef __pyx_methods[] = {
  {0, 0, 0, 0}
};

#if PY_MAJOR_VERSION >= 3
#if CYTHON_PEP489_MULTI_PHASE_INIT
static PyObject* __pyx_pymod_create(PyObject *spec, PyModuleDef *def); /*proto*/
static int __pyx_pymod_exec_cygrpc(PyObject* module); /*proto*/
static PyModuleDef_Slot __pyx_moduledef_slots[] = {
  {Py_mod_create, (void*)__pyx_pymod_create},
  {Py_mod_exec, (void*)__pyx_pymod_exec_cygrpc},
  {0, NULL}
};
#endif

static struct PyModuleDef __pyx_moduledef = {
    PyModuleDef_HEAD_INIT,
    "cygrpc",
    0, /* m_doc */
  #if CYTHON_PEP489_MULTI_PHASE_INIT
    0, /* m_size */
  #else
    -1, /* m_size */
  #endif
    __pyx_methods /* m_methods */,
  #if CYTHON_PEP489_MULTI_PHASE_INIT
    __pyx_moduledef_slots, /* m_slots */
  #else
    NULL, /* m_reload */
  #endif
    NULL, /* m_traverse */
    NULL, /* m_clear */
    NULL /* m_free */
};
#endif

static __Pyx_StringTabEntry __pyx_string_tab[] = {
  {&__pyx_kp_s_0, __pyx_k_0, sizeof(__pyx_k_0), 0, 0, 1, 0},
  {&__pyx_kp_s_0_0_0_0, __pyx_k_0_0_0_0, sizeof(__pyx_k_0_0_0_0), 0, 0, 1, 0},
  {&__pyx_kp_s_1, __pyx_k_1, sizeof(__pyx_k_1), 0, 0, 1, 0},
  {&__pyx_n_s_AF_INET, __pyx_k_AF_INET, sizeof(__pyx_k_AF_INET), 0, 0, 1, 1},
  {&__pyx_n_s_AF_INET6, __pyx_k_AF_INET6, sizeof(__pyx_k_AF_INET6), 0, 0, 1, 1},
  {&__pyx_n_s_AWAIT_THREADS_TIMEOUT_SECONDS, __pyx_k_AWAIT_THREADS_TIMEOUT_SECONDS, sizeof(__pyx_k_AWAIT_THREADS_TIMEOUT_SECONDS), 0, 0, 1, 1},
  {&__pyx_n_s_ActiveThreadCount, __pyx_k_ActiveThreadCount, sizeof(__pyx_k_ActiveThreadCount), 0, 0, 1, 1},
  {&__pyx_n_s_ActiveThreadCount___init, __pyx_k_ActiveThreadCount___init, sizeof(__pyx_k_ActiveThreadCount___init), 0, 0, 1, 1},
  {&__pyx_n_s_ActiveThreadCount_await_zero_th, __pyx_k_ActiveThreadCount_await_zero_th, sizeof(__pyx_k_ActiveThreadCount_await_zero_th), 0, 0, 1, 1},
  {&__pyx_n_s_ActiveThreadCount_decrement, __pyx_k_ActiveThreadCount_decrement, sizeof(__pyx_k_ActiveThreadCount_decrement), 0, 0, 1, 1},
  {&__pyx_n_s_ActiveThreadCount_increment, __pyx_k_ActiveThreadCount_increment, sizeof(__pyx_k_ActiveThreadCount_increment), 0, 0, 1, 1},
  {&__pyx_n_s_AttributeError, __pyx_k_AttributeError, sizeof(__pyx_k_AttributeError), 0, 0, 1, 1},
  {&__pyx_n_s_CallError, __pyx_k_CallError, sizeof(__pyx_k_CallError), 0, 0, 1, 1},
  {&__pyx_kp_s_Cannot_invoke_RPC_on_closed_chan, __pyx_k_Cannot_invoke_RPC_on_closed_chan, sizeof(__pyx_k_Cannot_invoke_RPC_on_closed_chan), 0, 0, 1, 0},
  {&__pyx_kp_s_Cannot_invoke_RPC_s, __pyx_k_Cannot_invoke_RPC_s, sizeof(__pyx_k_Cannot_invoke_RPC_s), 0, 0, 1, 0},
  {&__pyx_n_s_ChannelArgKey, __pyx_k_ChannelArgKey, sizeof(__pyx_k_ChannelArgKey), 0, 0, 1, 1},
  {&__pyx_n_s_CompletionType, __pyx_k_CompletionType, sizeof(__pyx_k_CompletionType), 0, 0, 1, 1},
  {&__pyx_n_s_CompressionAlgorithm, __pyx_k_CompressionAlgorithm, sizeof(__pyx_k_CompressionAlgorithm), 0, 0, 1, 1},
  {&__pyx_n_s_CompressionLevel, __pyx_k_CompressionLevel, sizeof(__pyx_k_CompressionLevel), 0, 0, 1, 1},
  {&__pyx_n_s_Condition, __pyx_k_Condition, sizeof(__pyx_k_Condition), 0, 0, 1, 1},
  {&__pyx_n_s_ConnectivityState, __pyx_k_ConnectivityState, sizeof(__pyx_k_ConnectivityState), 0, 0, 1, 1},
  {&__pyx_n_s_ENOTCONN, __pyx_k_ENOTCONN, sizeof(__pyx_k_ENOTCONN), 0, 0, 1, 1},
  {&__pyx_n_s_EX_USAGE, __pyx_k_EX_USAGE, sizeof(__pyx_k_EX_USAGE), 0, 0, 1, 1},
  {&__pyx_kp_s_Error_fetching_certificate_confi, __pyx_k_Error_fetching_certificate_confi, sizeof(__pyx_k_Error_fetching_certificate_confi), 0, 0, 1, 0},
  {&__pyx_kp_s_Error_fetching_certificate_confi_2, __pyx_k_Error_fetching_certificate_confi_2, sizeof(__pyx_k_Error_fetching_certificate_confi_2), 0, 0, 1, 0},
  {&__pyx_n_s_Event, __pyx_k_Event, sizeof(__pyx_k_Event), 0, 0, 1, 1},
  {&__pyx_kp_s_Exiting_child_due_to_raised_exce, __pyx_k_Exiting_child_due_to_raised_exce, sizeof(__pyx_k_Exiting_child_due_to_raised_exce), 0, 0, 1, 0},
  {&__pyx_kp_s_Expected_bytes_str_or_unicode_no, __pyx_k_Expected_bytes_str_or_unicode_no, sizeof(__pyx_k_Expected_bytes_str_or_unicode_no), 0, 0, 1, 0},
  {&__pyx_kp_s_Expected_int_bytes_or_behavior_g, __pyx_k_Expected_int_bytes_or_behavior_g, sizeof(__pyx_k_Expected_int_bytes_or_behavior_g), 0, 0, 1, 0},
  {&__pyx_kp_s_Expected_str_not, __pyx_k_Expected_str_not, sizeof(__pyx_k_Expected_str_not), 0, 0, 1, 0},
  {&__pyx_kp_s_Failed_to_get_server_sockets_ple, __pyx_k_Failed_to_get_server_sockets_ple, sizeof(__pyx_k_Failed_to_get_server_sockets_ple), 0, 0, 1, 0},
  {&__pyx_kp_s_Failed_to_get_servers_please_ens, __pyx_k_Failed_to_get_servers_please_ens, sizeof(__pyx_k_Failed_to_get_servers_please_ens), 0, 0, 1, 0},
  {&__pyx_kp_s_Failed_to_get_the_channel_please, __pyx_k_Failed_to_get_the_channel_please, sizeof(__pyx_k_Failed_to_get_the_channel_please), 0, 0, 1, 0},
  {&__pyx_kp_s_Failed_to_get_the_server_please, __pyx_k_Failed_to_get_the_server_please, sizeof(__pyx_k_Failed_to_get_the_server_please), 0, 0, 1, 0},
  {&__pyx_kp_s_Failed_to_get_the_socket_please, __pyx_k_Failed_to_get_the_socket_please, sizeof(__pyx_k_Failed_to_get_the_socket_please), 0, 0, 1, 0},
  {&__pyx_kp_s_Failed_to_get_the_subchannel_ple, __pyx_k_Failed_to_get_the_subchannel_ple, sizeof(__pyx_k_Failed_to_get_the_subchannel_ple), 0, 0, 1, 0},
  {&__pyx_kp_s_Failed_to_get_top_channels_pleas, __pyx_k_Failed_to_get_top_channels_pleas, sizeof(__pyx_k_Failed_to_get_top_channels_pleas), 0, 0, 1, 0},
  {&__pyx_kp_s_Failed_to_shutdown_gRPC_Core_aft, __pyx_k_Failed_to_shutdown_gRPC_Core_aft, sizeof(__pyx_k_Failed_to_shutdown_gRPC_Core_aft), 0, 0, 1, 0},
  {&__pyx_kp_s_Failed_to_shutdown_gRPC_Python_t, __pyx_k_Failed_to_shutdown_gRPC_Python_t, sizeof(__pyx_k_Failed_to_shutdown_gRPC_Python_t), 0, 0, 1, 0},
  {&__pyx_n_s_ForkManagedThread, __pyx_k_ForkManagedThread, sizeof(__pyx_k_ForkManagedThread), 0, 0, 1, 1},
  {&__pyx_n_s_ForkManagedThread___init, __pyx_k_ForkManagedThread___init, sizeof(__pyx_k_ForkManagedThread___init), 0, 0, 1, 1},
  {&__pyx_n_s_ForkManagedThread___init___local, __pyx_k_ForkManagedThread___init___local, sizeof(__pyx_k_ForkManagedThread___init___local), 0, 0, 1, 1},
  {&__pyx_n_s_ForkManagedThread_join, __pyx_k_ForkManagedThread_join, sizeof(__pyx_k_ForkManagedThread_join), 0, 0, 1, 1},
  {&__pyx_n_s_ForkManagedThread_setDaemon, __pyx_k_ForkManagedThread_setDaemon, sizeof(__pyx_k_ForkManagedThread_setDaemon), 0, 0, 1, 1},
  {&__pyx_n_s_ForkManagedThread_start, __pyx_k_ForkManagedThread_start, sizeof(__pyx_k_ForkManagedThread_start), 0, 0, 1, 1},
  {&__pyx_n_s_ForkState, __pyx_k_ForkState, sizeof(__pyx_k_ForkState), 0, 0, 1, 1},
  {&__pyx_n_s_ForkState___init, __pyx_k_ForkState___init, sizeof(__pyx_k_ForkState___init), 0, 0, 1, 1},
  {&__pyx_n_s_GRPC_COMPRESSION_CHANNEL_DEFAULT, __pyx_k_GRPC_COMPRESSION_CHANNEL_DEFAULT, sizeof(__pyx_k_GRPC_COMPRESSION_CHANNEL_DEFAULT), 0, 0, 1, 1},
  {&__pyx_n_s_GRPC_COMPRESSION_REQUEST_ALGORIT, __pyx_k_GRPC_COMPRESSION_REQUEST_ALGORIT, sizeof(__pyx_k_GRPC_COMPRESSION_REQUEST_ALGORIT), 0, 0, 1, 1},
  {&__pyx_n_s_GRPC_ENABLE_FORK_SUPPORT, __pyx_k_GRPC_ENABLE_FORK_SUPPORT, sizeof(__pyx_k_GRPC_ENABLE_FORK_SUPPORT), 0, 0, 1, 1},
  {&__pyx_n_s_GRPC_ENABLE_FORK_SUPPORT_2, __pyx_k_GRPC_ENABLE_FORK_SUPPORT_2, sizeof(__pyx_k_GRPC_ENABLE_FORK_SUPPORT_2), 0, 0, 1, 1},
  {&__pyx_n_s_GRPC_PROPAGATE_CANCELLATION, __pyx_k_GRPC_PROPAGATE_CANCELLATION, sizeof(__pyx_k_GRPC_PROPAGATE_CANCELLATION), 0, 0, 1, 1},
  {&__pyx_n_s_GRPC_PROPAGATE_CENSUS_STATS_CONT, __pyx_k_GRPC_PROPAGATE_CENSUS_STATS_CONT, sizeof(__pyx_k_GRPC_PROPAGATE_CENSUS_STATS_CONT), 0, 0, 1, 1},
  {&__pyx_n_s_GRPC_PROPAGATE_CENSUS_TRACING_CO, __pyx_k_GRPC_PROPAGATE_CENSUS_TRACING_CO, sizeof(__pyx_k_GRPC_PROPAGATE_CENSUS_TRACING_CO), 0, 0, 1, 1},
  {&__pyx_n_s_GRPC_PROPAGATE_DEADLINE, __pyx_k_GRPC_PROPAGATE_DEADLINE, sizeof(__pyx_k_GRPC_PROPAGATE_DEADLINE), 0, 0, 1, 1},
  {&__pyx_n_s_GRPC_PROPAGATE_DEFAULTS, __pyx_k_GRPC_PROPAGATE_DEFAULTS, sizeof(__pyx_k_GRPC_PROPAGATE_DEFAULTS), 0, 0, 1, 1},
  {&__pyx_n_s_Group, __pyx_k_Group, sizeof(__pyx_k_Group), 0, 0, 1, 1},
  {&__pyx_n_s_INTERNAL_CALL_ERROR_MESSAGE_FOR, __pyx_k_INTERNAL_CALL_ERROR_MESSAGE_FOR, sizeof(__pyx_k_INTERNAL_CALL_ERROR_MESSAGE_FOR), 0, 0, 1, 1},
  {&__pyx_n_s_IOError, __pyx_k_IOError, sizeof(__pyx_k_IOError), 0, 0, 1, 1},
  {&__pyx_n_s_IPPROTO_TCP, __pyx_k_IPPROTO_TCP, sizeof(__pyx_k_IPPROTO_TCP), 0, 0, 1, 1},
  {&__pyx_kp_s_Incompatible_checksums_s_vs_0xd4, __pyx_k_Incompatible_checksums_s_vs_0xd4, sizeof(__pyx_k_Incompatible_checksums_s_vs_0xd4), 0, 0, 1, 0},
  {&__pyx_n_s_InitialMetadataFlags, __pyx_k_InitialMetadataFlags, sizeof(__pyx_k_InitialMetadataFlags), 0, 0, 1, 1},
  {&__pyx_kp_s_Internal_gRPC_call_error_d, __pyx_k_Internal_gRPC_call_error_d, sizeof(__pyx_k_Internal_gRPC_call_error_d), 0, 0, 1, 0},
  {&__pyx_kp_s_Invalid_encoding_on_s, __pyx_k_Invalid_encoding_on_s, sizeof(__pyx_k_Invalid_encoding_on_s), 0, 0, 1, 0},
  {&__pyx_n_s_LOGGER, __pyx_k_LOGGER, sizeof(__pyx_k_LOGGER), 0, 0, 1, 1},
  {&__pyx_n_s_Lock, __pyx_k_Lock, sizeof(__pyx_k_Lock), 0, 0, 1, 1},
  {&__pyx_n_s_Metadatum, __pyx_k_Metadatum, sizeof(__pyx_k_Metadatum), 0, 0, 1, 1},
  {&__pyx_kp_s_No_custom_hooks_are_implemented, __pyx_k_No_custom_hooks_are_implemented, sizeof(__pyx_k_No_custom_hooks_are_implemented), 0, 0, 1, 0},
  {&__pyx_n_s_NotImplementedError, __pyx_k_NotImplementedError, sizeof(__pyx_k_NotImplementedError), 0, 0, 1, 1},
  {&__pyx_n_s_OperationType, __pyx_k_OperationType, sizeof(__pyx_k_OperationType), 0, 0, 1, 1},
  {&__pyx_n_s_PickleError, __pyx_k_PickleError, sizeof(__pyx_k_PickleError), 0, 0, 1, 1},
  {&__pyx_kp_s_Please_report_to_https_github_co, __pyx_k_Please_report_to_https_github_co, sizeof(__pyx_k_Please_report_to_https_github_co), 0, 0, 1, 0},
  {&__pyx_n_s_PropagationConstants, __pyx_k_PropagationConstants, sizeof(__pyx_k_PropagationConstants), 0, 0, 1, 1},
  {&__pyx_n_s_RuntimeError, __pyx_k_RuntimeError, sizeof(__pyx_k_RuntimeError), 0, 0, 1, 1},
  {&__pyx_n_s_SHUT_RDWR, __pyx_k_SHUT_RDWR, sizeof(__pyx_k_SHUT_RDWR), 0, 0, 1, 1},
  {&__pyx_n_s_SOL_SOCKET, __pyx_k_SOL_SOCKET, sizeof(__pyx_k_SOL_SOCKET), 0, 0, 1, 1},
  {&__pyx_n_s_SO_REUSEADDR, __pyx_k_SO_REUSEADDR, sizeof(__pyx_k_SO_REUSEADDR), 0, 0, 1, 1},
  {&__pyx_n_s_ServerCertificateConfiguration, __pyx_k_ServerCertificateConfiguration, sizeof(__pyx_k_ServerCertificateConfiguration), 0, 0, 1, 1},
  {&__pyx_n_s_StatusCode, __pyx_k_StatusCode, sizeof(__pyx_k_StatusCode), 0, 0, 1, 1},
  {&__pyx_n_s_TCP_NODELAY, __pyx_k_TCP_NODELAY, sizeof(__pyx_k_TCP_NODELAY), 0, 0, 1, 1},
  {&__pyx_n_s_TRUE, __pyx_k_TRUE, sizeof(__pyx_k_TRUE), 0, 0, 1, 1},
  {&__pyx_n_s_TRUE_VALUES, __pyx_k_TRUE_VALUES, sizeof(__pyx_k_TRUE_VALUES), 0, 0, 1, 1},
  {&__pyx_n_s_Thread, __pyx_k_Thread, sizeof(__pyx_k_Thread), 0, 0, 1, 1},
  {&__pyx_n_s_True, __pyx_k_True, sizeof(__pyx_k_True), 0, 0, 1, 1},
  {&__pyx_n_s_TypeError, __pyx_k_TypeError, sizeof(__pyx_k_TypeError), 0, 0, 1, 1},
  {&__pyx_n_s_UnicodeDecodeError, __pyx_k_UnicodeDecodeError, sizeof(__pyx_k_UnicodeDecodeError), 0, 0, 1, 1},
  {&__pyx_n_s_ValueError, __pyx_k_ValueError, sizeof(__pyx_k_ValueError), 0, 0, 1, 1},
  {&__pyx_n_s_WriteFlag, __pyx_k_WriteFlag, sizeof(__pyx_k_WriteFlag), 0, 0, 1, 1},
  {&__pyx_n_s_YES, __pyx_k_YES, sizeof(__pyx_k_YES), 0, 0, 1, 1},
  {&__pyx_n_s_Yes, __pyx_k_Yes, sizeof(__pyx_k_Yes), 0, 0, 1, 1},
  {&__pyx_kp_s__131, __pyx_k__131, sizeof(__pyx_k__131), 0, 0, 1, 0},
  {&__pyx_kp_s__132, __pyx_k__132, sizeof(__pyx_k__132), 0, 0, 1, 0},
  {&__pyx_kp_s__134, __pyx_k__134, sizeof(__pyx_k__134), 0, 0, 1, 0},
  {&__pyx_kp_s__136, __pyx_k__136, sizeof(__pyx_k__136), 0, 0, 1, 0},
  {&__pyx_kp_b__2, __pyx_k__2, sizeof(__pyx_k__2), 0, 0, 0, 0},
  {&__pyx_kp_s__2, __pyx_k__2, sizeof(__pyx_k__2), 0, 0, 1, 0},
  {&__pyx_n_s_aborted, __pyx_k_aborted, sizeof(__pyx_k_aborted), 0, 0, 1, 1},
  {&__pyx_n_s_accept, __pyx_k_accept, sizeof(__pyx_k_accept), 0, 0, 1, 1},
  {&__pyx_n_s_active_thread_count, __pyx_k_active_thread_count, sizeof(__pyx_k_active_thread_count), 0, 0, 1, 1},
  {&__pyx_n_s_add, __pyx_k_add, sizeof(__pyx_k_add), 0, 0, 1, 1},
  {&__pyx_n_s_addr_tuple, __pyx_k_addr_tuple, sizeof(__pyx_k_addr_tuple), 0, 0, 1, 1},
  {&__pyx_n_s_address, __pyx_k_address, sizeof(__pyx_k_address), 0, 0, 1, 1},
  {&__pyx_n_s_algorithm, __pyx_k_algorithm, sizeof(__pyx_k_algorithm), 0, 0, 1, 1},
  {&__pyx_n_s_already_accepted, __pyx_k_already_accepted, sizeof(__pyx_k_already_accepted), 0, 0, 1, 1},
  {&__pyx_n_s_already_exists, __pyx_k_already_exists, sizeof(__pyx_k_already_exists), 0, 0, 1, 1},
  {&__pyx_n_s_already_finished, __pyx_k_already_finished, sizeof(__pyx_k_already_finished), 0, 0, 1, 1},
  {&__pyx_n_s_already_invoked, __pyx_k_already_invoked, sizeof(__pyx_k_already_invoked), 0, 0, 1, 1},
  {&__pyx_n_s_append, __pyx_k_append, sizeof(__pyx_k_append), 0, 0, 1, 1},
  {&__pyx_n_s_applysockopts, __pyx_k_applysockopts, sizeof(__pyx_k_applysockopts), 0, 0, 1, 1},
  {&__pyx_n_s_args, __pyx_k_args, sizeof(__pyx_k_args), 0, 0, 1, 1},
  {&__pyx_n_s_arguments, __pyx_k_arguments, sizeof(__pyx_k_arguments), 0, 0, 1, 1},
  {&__pyx_n_s_ascii, __pyx_k_ascii, sizeof(__pyx_k_ascii), 0, 0, 1, 1},
  {&__pyx_n_s_async_callback_func, __pyx_k_async_callback_func, sizeof(__pyx_k_async_callback_func), 0, 0, 1, 1},
  {&__pyx_n_s_auth_context, __pyx_k_auth_context, sizeof(__pyx_k_auth_context), 0, 0, 1, 1},
  {&__pyx_n_s_await_zero_threads, __pyx_k_await_zero_threads, sizeof(__pyx_k_await_zero_threads), 0, 0, 1, 1},
  {&__pyx_n_s_batch_operations, __pyx_k_batch_operations, sizeof(__pyx_k_batch_operations), 0, 0, 1, 1},
  {&__pyx_kp_b_bin, __pyx_k_bin, sizeof(__pyx_k_bin), 0, 0, 0, 0},
  {&__pyx_n_s_bind, __pyx_k_bind, sizeof(__pyx_k_bind), 0, 0, 1, 1},
  {&__pyx_n_s_block_if_fork_in_progress, __pyx_k_block_if_fork_in_progress, sizeof(__pyx_k_block_if_fork_in_progress), 0, 0, 1, 1},
  {&__pyx_n_s_buffer_hint, __pyx_k_buffer_hint, sizeof(__pyx_k_buffer_hint), 0, 0, 1, 1},
  {&__pyx_n_s_build_census_context, __pyx_k_build_census_context, sizeof(__pyx_k_build_census_context), 0, 0, 1, 1},
  {&__pyx_n_s_c_cert_config, __pyx_k_c_cert_config, sizeof(__pyx_k_c_cert_config), 0, 0, 1, 1},
  {&__pyx_n_s_c_key, __pyx_k_c_key, sizeof(__pyx_k_c_key), 0, 0, 1, 1},
  {&__pyx_n_s_c_options, __pyx_k_c_options, sizeof(__pyx_k_c_options), 0, 0, 1, 1},
  {&__pyx_n_s_c_pem_root_certs, __pyx_k_c_pem_root_certs, sizeof(__pyx_k_c_pem_root_certs), 0, 0, 1, 1},
  {&__pyx_n_s_c_returned_str, __pyx_k_c_returned_str, sizeof(__pyx_k_c_returned_str), 0, 0, 1, 1},
  {&__pyx_n_s_call, __pyx_k_call, sizeof(__pyx_k_call), 0, 0, 1, 1},
  {&__pyx_n_s_call_credentialses, __pyx_k_call_credentialses, sizeof(__pyx_k_call_credentialses), 0, 0, 1, 1},
  {&__pyx_n_s_call_details, __pyx_k_call_details, sizeof(__pyx_k_call_details), 0, 0, 1, 1},
  {&__pyx_n_s_call_queue, __pyx_k_call_queue, sizeof(__pyx_k_call_queue), 0, 0, 1, 1},
  {&__pyx_n_s_call_state, __pyx_k_call_state, sizeof(__pyx_k_call_state), 0, 0, 1, 1},
  {&__pyx_n_s_callback, __pyx_k_callback, sizeof(__pyx_k_callback), 0, 0, 1, 1},
  {&__pyx_n_s_callback_func, __pyx_k_callback_func, sizeof(__pyx_k_callback_func), 0, 0, 1, 1},
  {&__pyx_n_s_cancelled, __pyx_k_cancelled, sizeof(__pyx_k_cancelled), 0, 0, 1, 1},
  {&__pyx_kp_s_cannot_register_completion_queue, __pyx_k_cannot_register_completion_queue, sizeof(__pyx_k_cannot_register_completion_queue), 0, 0, 1, 0},
  {&__pyx_n_s_capacity, __pyx_k_capacity, sizeof(__pyx_k_capacity), 0, 0, 1, 1},
  {&__pyx_n_s_cb, __pyx_k_cb, sizeof(__pyx_k_cb), 0, 0, 1, 1},
  {&__pyx_n_s_cb_func, __pyx_k_cb_func, sizeof(__pyx_k_cb_func), 0, 0, 1, 1},
  {&__pyx_n_s_census_ctx, __pyx_k_census_ctx, sizeof(__pyx_k_census_ctx), 0, 0, 1, 1},
  {&__pyx_n_s_cert_config, __pyx_k_cert_config, sizeof(__pyx_k_cert_config), 0, 0, 1, 1},
  {&__pyx_n_s_cert_config_fetcher, __pyx_k_cert_config_fetcher, sizeof(__pyx_k_cert_config_fetcher), 0, 0, 1, 1},
  {&__pyx_kp_s_cert_config_fetcher_must_be_call, __pyx_k_cert_config_fetcher_must_be_call, sizeof(__pyx_k_cert_config_fetcher_must_be_call), 0, 0, 1, 0},
  {&__pyx_n_s_certificate_chain, __pyx_k_certificate_chain, sizeof(__pyx_k_certificate_chain), 0, 0, 1, 1},
  {&__pyx_n_s_certificate_configuration, __pyx_k_certificate_configuration, sizeof(__pyx_k_certificate_configuration), 0, 0, 1, 1},
  {&__pyx_n_s_channel, __pyx_k_channel, sizeof(__pyx_k_channel), 0, 0, 1, 1},
  {&__pyx_n_s_channel_credentials, __pyx_k_channel_credentials, sizeof(__pyx_k_channel_credentials), 0, 0, 1, 1},
  {&__pyx_n_s_channel_id, __pyx_k_channel_id, sizeof(__pyx_k_channel_id), 0, 0, 1, 1},
  {&__pyx_n_s_channel_state, __pyx_k_channel_state, sizeof(__pyx_k_channel_state), 0, 0, 1, 1},
  {&__pyx_n_s_channels, __pyx_k_channels, sizeof(__pyx_k_channels), 0, 0, 1, 1},
  {&__pyx_n_s_channelz_get_channel, __pyx_k_channelz_get_channel, sizeof(__pyx_k_channelz_get_channel), 0, 0, 1, 1},
  {&__pyx_n_s_channelz_get_server, __pyx_k_channelz_get_server, sizeof(__pyx_k_channelz_get_server), 0, 0, 1, 1},
  {&__pyx_n_s_channelz_get_server_sockets, __pyx_k_channelz_get_server_sockets, sizeof(__pyx_k_channelz_get_server_sockets), 0, 0, 1, 1},
  {&__pyx_n_s_channelz_get_servers, __pyx_k_channelz_get_servers, sizeof(__pyx_k_channelz_get_servers), 0, 0, 1, 1},
  {&__pyx_n_s_channelz_get_socket, __pyx_k_channelz_get_socket, sizeof(__pyx_k_channelz_get_socket), 0, 0, 1, 1},
  {&__pyx_n_s_channelz_get_subchannel, __pyx_k_channelz_get_subchannel, sizeof(__pyx_k_channelz_get_subchannel), 0, 0, 1, 1},
  {&__pyx_n_s_channelz_get_top_channels, __pyx_k_channelz_get_top_channels, sizeof(__pyx_k_channelz_get_top_channels), 0, 0, 1, 1},
  {&__pyx_n_s_clear, __pyx_k_clear, sizeof(__pyx_k_clear), 0, 0, 1, 1},
  {&__pyx_n_s_cline_in_traceback, __pyx_k_cline_in_traceback, sizeof(__pyx_k_cline_in_traceback), 0, 0, 1, 1},
  {&__pyx_n_s_close, __pyx_k_close, sizeof(__pyx_k_close), 0, 0, 1, 1},
  {&__pyx_n_s_close_on_fork, __pyx_k_close_on_fork, sizeof(__pyx_k_close_on_fork), 0, 0, 1, 1},
  {&__pyx_n_s_code, __pyx_k_code, sizeof(__pyx_k_code), 0, 0, 1, 1},
  {&__pyx_n_s_collections, __pyx_k_collections, sizeof(__pyx_k_collections), 0, 0, 1, 1},
  {&__pyx_n_s_completion_type, __pyx_k_completion_type, sizeof(__pyx_k_completion_type), 0, 0, 1, 1},
  {&__pyx_n_s_compression_algorithm_name, __pyx_k_compression_algorithm_name, sizeof(__pyx_k_compression_algorithm_name), 0, 0, 1, 1},
  {&__pyx_n_s_condition, __pyx_k_condition, sizeof(__pyx_k_condition), 0, 0, 1, 1},
  {&__pyx_n_s_connect, __pyx_k_connect, sizeof(__pyx_k_connect), 0, 0, 1, 1},
  {&__pyx_n_s_connecting, __pyx_k_connecting, sizeof(__pyx_k_connecting), 0, 0, 1, 1},
  {&__pyx_n_s_context, __pyx_k_context, sizeof(__pyx_k_context), 0, 0, 1, 1},
  {&__pyx_n_s_credentials, __pyx_k_credentials, sizeof(__pyx_k_credentials), 0, 0, 1, 1},
  {&__pyx_kp_s_credentials_roots_pem, __pyx_k_credentials_roots_pem, sizeof(__pyx_k_credentials_roots_pem), 0, 0, 1, 0},
  {&__pyx_kp_s_cygrpc, __pyx_k_cygrpc, sizeof(__pyx_k_cygrpc), 0, 0, 1, 0},
  {&__pyx_n_s_daemon, __pyx_k_daemon, sizeof(__pyx_k_daemon), 0, 0, 1, 1},
  {&__pyx_n_s_daemonic, __pyx_k_daemonic, sizeof(__pyx_k_daemonic), 0, 0, 1, 1},
  {&__pyx_n_s_data_loss, __pyx_k_data_loss, sizeof(__pyx_k_data_loss), 0, 0, 1, 1},
  {&__pyx_n_s_deadline, __pyx_k_deadline, sizeof(__pyx_k_deadline), 0, 0, 1, 1},
  {&__pyx_n_s_deadline_exceeded, __pyx_k_deadline_exceeded, sizeof(__pyx_k_deadline_exceeded), 0, 0, 1, 1},
  {&__pyx_n_s_decrement, __pyx_k_decrement, sizeof(__pyx_k_decrement), 0, 0, 1, 1},
  {&__pyx_n_s_default_authority, __pyx_k_default_authority, sizeof(__pyx_k_default_authority), 0, 0, 1, 1},
  {&__pyx_n_s_deflate, __pyx_k_deflate, sizeof(__pyx_k_deflate), 0, 0, 1, 1},
  {&__pyx_n_s_details, __pyx_k_details, sizeof(__pyx_k_details), 0, 0, 1, 1},
  {&__pyx_n_s_dict, __pyx_k_dict, sizeof(__pyx_k_dict), 0, 0, 1, 1},
  {&__pyx_n_s_discard, __pyx_k_discard, sizeof(__pyx_k_discard), 0, 0, 1, 1},
  {&__pyx_n_s_doc, __pyx_k_doc, sizeof(__pyx_k_doc), 0, 0, 1, 1},
  {&__pyx_n_s_enable_census, __pyx_k_enable_census, sizeof(__pyx_k_enable_census), 0, 0, 1, 1},
  {&__pyx_n_s_encode, __pyx_k_encode, sizeof(__pyx_k_encode), 0, 0, 1, 1},
  {&__pyx_n_s_end_time, __pyx_k_end_time, sizeof(__pyx_k_end_time), 0, 0, 1, 1},
  {&__pyx_n_s_endswith, __pyx_k_endswith, sizeof(__pyx_k_endswith), 0, 0, 1, 1},
  {&__pyx_n_s_enter, __pyx_k_enter, sizeof(__pyx_k_enter), 0, 0, 1, 1},
  {&__pyx_n_s_enter_user_request_generator, __pyx_k_enter_user_request_generator, sizeof(__pyx_k_enter_user_request_generator), 0, 0, 1, 1},
  {&__pyx_n_s_enumerate, __pyx_k_enumerate, sizeof(__pyx_k_enumerate), 0, 0, 1, 1},
  {&__pyx_n_s_environ, __pyx_k_environ, sizeof(__pyx_k_environ), 0, 0, 1, 1},
  {&__pyx_n_s_errno, __pyx_k_errno, sizeof(__pyx_k_errno), 0, 0, 1, 1},
  {&__pyx_n_s_error, __pyx_k_error, sizeof(__pyx_k_error), 0, 0, 1, 1},
  {&__pyx_n_s_error_code, __pyx_k_error_code, sizeof(__pyx_k_error_code), 0, 0, 1, 1},
  {&__pyx_n_s_error_details, __pyx_k_error_details, sizeof(__pyx_k_error_details), 0, 0, 1, 1},
  {&__pyx_n_s_event, __pyx_k_event, sizeof(__pyx_k_event), 0, 0, 1, 1},
  {&__pyx_n_s_exc_info, __pyx_k_exc_info, sizeof(__pyx_k_exc_info), 0, 0, 1, 1},
  {&__pyx_n_s_exception, __pyx_k_exception, sizeof(__pyx_k_exception), 0, 0, 1, 1},
  {&__pyx_n_s_exit, __pyx_k_exit, sizeof(__pyx_k_exit), 0, 0, 1, 1},
  {&__pyx_n_s_exit_2, __pyx_k_exit_2, sizeof(__pyx_k_exit_2), 0, 0, 1, 1},
  {&__pyx_kp_s_expected_certificate_to_be_bytes, __pyx_k_expected_certificate_to_be_bytes, sizeof(__pyx_k_expected_certificate_to_be_bytes), 0, 0, 1, 0},
  {&__pyx_kp_s_expected_pem_key_cert_pairs_to_b, __pyx_k_expected_pem_key_cert_pairs_to_b, sizeof(__pyx_k_expected_pem_key_cert_pairs_to_b), 0, 0, 1, 0},
  {&__pyx_kp_s_expected_registered_completion_q, __pyx_k_expected_registered_completion_q, sizeof(__pyx_k_expected_registered_completion_q), 0, 0, 1, 0},
  {&__pyx_kp_s_failed, __pyx_k_failed, sizeof(__pyx_k_failed), 0, 0, 1, 0},
  {&__pyx_n_s_failed_precondition, __pyx_k_failed_precondition, sizeof(__pyx_k_failed_precondition), 0, 0, 1, 1},
  {&__pyx_n_s_flags, __pyx_k_flags, sizeof(__pyx_k_flags), 0, 0, 1, 1},
  {&__pyx_n_s_force_client_auth, __pyx_k_force_client_auth, sizeof(__pyx_k_force_client_auth), 0, 0, 1, 1},
  {&__pyx_n_s_fork_epoch, __pyx_k_fork_epoch, sizeof(__pyx_k_fork_epoch), 0, 0, 1, 1},
  {&__pyx_n_s_fork_handler_failed, __pyx_k_fork_handler_failed, sizeof(__pyx_k_fork_handler_failed), 0, 0, 1, 1},
  {&__pyx_n_s_fork_handler_registered, __pyx_k_fork_handler_registered, sizeof(__pyx_k_fork_handler_registered), 0, 0, 1, 1},
  {&__pyx_n_s_fork_handler_registered_lock, __pyx_k_fork_handler_registered_lock, sizeof(__pyx_k_fork_handler_registered_lock), 0, 0, 1, 1},
  {&__pyx_n_s_fork_handlers_and_grpc_init, __pyx_k_fork_handlers_and_grpc_init, sizeof(__pyx_k_fork_handlers_and_grpc_init), 0, 0, 1, 1},
  {&__pyx_n_s_fork_in_progress, __pyx_k_fork_in_progress, sizeof(__pyx_k_fork_in_progress), 0, 0, 1, 1},
  {&__pyx_n_s_fork_in_progress_condition, __pyx_k_fork_in_progress_condition, sizeof(__pyx_k_fork_in_progress_condition), 0, 0, 1, 1},
  {&__pyx_n_s_fork_register_channel, __pyx_k_fork_register_channel, sizeof(__pyx_k_fork_register_channel), 0, 0, 1, 1},
  {&__pyx_n_s_fork_state, __pyx_k_fork_state, sizeof(__pyx_k_fork_state), 0, 0, 1, 1},
  {&__pyx_n_s_fork_unregister_channel, __pyx_k_fork_unregister_channel, sizeof(__pyx_k_fork_unregister_channel), 0, 0, 1, 1},
  {&__pyx_n_s_format, __pyx_k_format, sizeof(__pyx_k_format), 0, 0, 1, 1},
  {&__pyx_n_s_g_event, __pyx_k_g_event, sizeof(__pyx_k_g_event), 0, 0, 1, 1},
  {&__pyx_n_s_g_pool, __pyx_k_g_pool, sizeof(__pyx_k_g_pool), 0, 0, 1, 1},
  {&__pyx_n_s_gaierror, __pyx_k_gaierror, sizeof(__pyx_k_gaierror), 0, 0, 1, 1},
  {&__pyx_n_s_genexpr, __pyx_k_genexpr, sizeof(__pyx_k_genexpr), 0, 0, 1, 1},
  {&__pyx_n_s_get, __pyx_k_get, sizeof(__pyx_k_get), 0, 0, 1, 1},
  {&__pyx_n_s_getLogger, __pyx_k_getLogger, sizeof(__pyx_k_getLogger), 0, 0, 1, 1},
  {&__pyx_n_s_get_data, __pyx_k_get_data, sizeof(__pyx_k_get_data), 0, 0, 1, 1},
  {&__pyx_n_s_get_deadline_from_context, __pyx_k_get_deadline_from_context, sizeof(__pyx_k_get_deadline_from_context), 0, 0, 1, 1},
  {&__pyx_n_s_get_fork_epoch, __pyx_k_get_fork_epoch, sizeof(__pyx_k_get_fork_epoch), 0, 0, 1, 1},
  {&__pyx_n_s_get_hub, __pyx_k_get_hub, sizeof(__pyx_k_get_hub), 0, 0, 1, 1},
  {&__pyx_n_s_get_metadata_locals_callback, __pyx_k_get_metadata_locals_callback, sizeof(__pyx_k_get_metadata_locals_callback), 0, 0, 1, 1},
  {&__pyx_n_s_getaddrinfo, __pyx_k_getaddrinfo, sizeof(__pyx_k_getaddrinfo), 0, 0, 1, 1},
  {&__pyx_n_s_getpeername, __pyx_k_getpeername, sizeof(__pyx_k_getpeername), 0, 0, 1, 1},
  {&__pyx_n_s_getsockname, __pyx_k_getsockname, sizeof(__pyx_k_getsockname), 0, 0, 1, 1},
  {&__pyx_n_s_getstate, __pyx_k_getstate, sizeof(__pyx_k_getstate), 0, 0, 1, 1},
  {&__pyx_n_s_gevent, __pyx_k_gevent, sizeof(__pyx_k_gevent), 0, 0, 1, 1},
  {&__pyx_n_s_gevent_event, __pyx_k_gevent_event, sizeof(__pyx_k_gevent_event), 0, 0, 1, 1},
  {&__pyx_n_s_gevent_event_2, __pyx_k_gevent_event_2, sizeof(__pyx_k_gevent_event_2), 0, 0, 1, 1},
  {&__pyx_n_s_gevent_g, __pyx_k_gevent_g, sizeof(__pyx_k_gevent_g), 0, 0, 1, 1},
  {&__pyx_n_s_gevent_hub, __pyx_k_gevent_hub, sizeof(__pyx_k_gevent_hub), 0, 0, 1, 1},
  {&__pyx_n_s_gevent_hub_2, __pyx_k_gevent_hub_2, sizeof(__pyx_k_gevent_hub_2), 0, 0, 1, 1},
  {&__pyx_n_s_gevent_pool, __pyx_k_gevent_pool, sizeof(__pyx_k_gevent_pool), 0, 0, 1, 1},
  {&__pyx_n_s_gevent_socket, __pyx_k_gevent_socket, sizeof(__pyx_k_gevent_socket), 0, 0, 1, 1},
  {&__pyx_n_s_gevent_socket_2, __pyx_k_gevent_socket_2, sizeof(__pyx_k_gevent_socket_2), 0, 0, 1, 1},
  {&__pyx_n_s_greenlet, __pyx_k_greenlet, sizeof(__pyx_k_greenlet), 0, 0, 1, 1},
  {&__pyx_n_s_grpc, __pyx_k_grpc, sizeof(__pyx_k_grpc), 0, 0, 1, 1},
  {&__pyx_n_s_grpc__cython_cygrpc, __pyx_k_grpc__cython_cygrpc, sizeof(__pyx_k_grpc__cython_cygrpc), 0, 0, 1, 1},
  {&__pyx_kp_s_grpc_python__cygrpc__GrpcArgWrap, __pyx_k_grpc_python__cygrpc__GrpcArgWrap, sizeof(__pyx_k_grpc_python__cygrpc__GrpcArgWrap), 0, 0, 1, 0},
  {&__pyx_n_s_gzip, __pyx_k_gzip, sizeof(__pyx_k_gzip), 0, 0, 1, 1},
  {&__pyx_n_s_high, __pyx_k_high, sizeof(__pyx_k_high), 0, 0, 1, 1},
  {&__pyx_n_s_host, __pyx_k_host, sizeof(__pyx_k_host), 0, 0, 1, 1},
  {&__pyx_n_s_http2_initial_sequence_number, __pyx_k_http2_initial_sequence_number, sizeof(__pyx_k_http2_initial_sequence_number), 0, 0, 1, 1},
  {&__pyx_n_s_hub, __pyx_k_hub, sizeof(__pyx_k_hub), 0, 0, 1, 1},
  {&__pyx_n_s_identities, __pyx_k_identities, sizeof(__pyx_k_identities), 0, 0, 1, 1},
  {&__pyx_n_s_idle, __pyx_k_idle, sizeof(__pyx_k_idle), 0, 0, 1, 1},
  {&__pyx_kp_s_if_error_code_is_specified_so_mu, __pyx_k_if_error_code_is_specified_so_mu, sizeof(__pyx_k_if_error_code_is_specified_so_mu), 0, 0, 1, 0},
  {&__pyx_n_s_import, __pyx_k_import, sizeof(__pyx_k_import), 0, 0, 1, 1},
  {&__pyx_n_s_increment, __pyx_k_increment, sizeof(__pyx_k_increment), 0, 0, 1, 1},
  {&__pyx_n_s_init, __pyx_k_init, sizeof(__pyx_k_init), 0, 0, 1, 1},
  {&__pyx_n_s_init_grpc_gevent, __pyx_k_init_grpc_gevent, sizeof(__pyx_k_init_grpc_gevent), 0, 0, 1, 1},
  {&__pyx_n_s_init_grpc_gevent_locals_cb_func, __pyx_k_init_grpc_gevent_locals_cb_func, sizeof(__pyx_k_init_grpc_gevent_locals_cb_func), 0, 0, 1, 1},
  {&__pyx_n_s_initial_cert_config, __pyx_k_initial_cert_config, sizeof(__pyx_k_initial_cert_config), 0, 0, 1, 1},
  {&__pyx_kp_s_initial_cert_config_must_be_a_gr, __pyx_k_initial_cert_config_must_be_a_gr, sizeof(__pyx_k_initial_cert_config_must_be_a_gr), 0, 0, 1, 0},
  {&__pyx_n_s_initial_metadata, __pyx_k_initial_metadata, sizeof(__pyx_k_initial_metadata), 0, 0, 1, 1},
  {&__pyx_n_s_install_context_from_request_cal, __pyx_k_install_context_from_request_cal, sizeof(__pyx_k_install_context_from_request_cal), 0, 0, 1, 1},
  {&__pyx_n_s_int, __pyx_k_int, sizeof(__pyx_k_int), 0, 0, 1, 1},
  {&__pyx_n_s_integrated_call_locals_on_succe, __pyx_k_integrated_call_locals_on_succe, sizeof(__pyx_k_integrated_call_locals_on_succe), 0, 0, 1, 1},
  {&__pyx_n_s_internal, __pyx_k_internal, sizeof(__pyx_k_internal), 0, 0, 1, 1},
  {&__pyx_kp_s_internal_error_user_data_must_be, __pyx_k_internal_error_user_data_must_be, sizeof(__pyx_k_internal_error_user_data_must_be), 0, 0, 1, 0},
  {&__pyx_n_s_invalid_argument, __pyx_k_invalid_argument, sizeof(__pyx_k_invalid_argument), 0, 0, 1, 1},
  {&__pyx_kp_s_invalid_call_object_cannot_be_us, __pyx_k_invalid_call_object_cannot_be_us, sizeof(__pyx_k_invalid_call_object_cannot_be_us), 0, 0, 1, 0},
  {&__pyx_n_s_invalid_flags, __pyx_k_invalid_flags, sizeof(__pyx_k_invalid_flags), 0, 0, 1, 1},
  {&__pyx_n_s_invalid_metadata, __pyx_k_invalid_metadata, sizeof(__pyx_k_invalid_metadata), 0, 0, 1, 1},
  {&__pyx_n_s_invocation_metadata, __pyx_k_invocation_metadata, sizeof(__pyx_k_invocation_metadata), 0, 0, 1, 1},
  {&__pyx_n_b_ipv4, __pyx_k_ipv4, sizeof(__pyx_k_ipv4), 0, 0, 0, 1},
  {&__pyx_n_s_is_fork_support_enabled, __pyx_k_is_fork_support_enabled, sizeof(__pyx_k_is_fork_support_enabled), 0, 0, 1, 1},
  {&__pyx_n_s_is_valid, __pyx_k_is_valid, sizeof(__pyx_k_is_valid), 0, 0, 1, 1},
  {&__pyx_n_s_join, __pyx_k_join, sizeof(__pyx_k_join), 0, 0, 1, 1},
  {&__pyx_n_s_key, __pyx_k_key, sizeof(__pyx_k_key), 0, 0, 1, 1},
  {&__pyx_n_s_last_observed_state, __pyx_k_last_observed_state, sizeof(__pyx_k_last_observed_state), 0, 0, 1, 1},
  {&__pyx_n_s_listen, __pyx_k_listen, sizeof(__pyx_k_listen), 0, 0, 1, 1},
  {&__pyx_n_s_logging, __pyx_k_logging, sizeof(__pyx_k_logging), 0, 0, 1, 1},
  {&__pyx_n_s_loop, __pyx_k_loop, sizeof(__pyx_k_loop), 0, 0, 1, 1},
  {&__pyx_n_s_low, __pyx_k_low, sizeof(__pyx_k_low), 0, 0, 1, 1},
  {&__pyx_n_s_lower, __pyx_k_lower, sizeof(__pyx_k_lower), 0, 0, 1, 1},
  {&__pyx_n_s_lstrip, __pyx_k_lstrip, sizeof(__pyx_k_lstrip), 0, 0, 1, 1},
  {&__pyx_n_s_main, __pyx_k_main, sizeof(__pyx_k_main), 0, 0, 1, 1},
  {&__pyx_n_s_managed_target, __pyx_k_managed_target, sizeof(__pyx_k_managed_target), 0, 0, 1, 1},
  {&__pyx_n_s_max_concurrent_streams, __pyx_k_max_concurrent_streams, sizeof(__pyx_k_max_concurrent_streams), 0, 0, 1, 1},
  {&__pyx_n_s_max_receive_message_length, __pyx_k_max_receive_message_length, sizeof(__pyx_k_max_receive_message_length), 0, 0, 1, 1},
  {&__pyx_n_s_max_results, __pyx_k_max_results, sizeof(__pyx_k_max_results), 0, 0, 1, 1},
  {&__pyx_n_s_max_send_message_length, __pyx_k_max_send_message_length, sizeof(__pyx_k_max_send_message_length), 0, 0, 1, 1},
  {&__pyx_n_s_medium, __pyx_k_medium, sizeof(__pyx_k_medium), 0, 0, 1, 1},
  {&__pyx_n_s_message, __pyx_k_message, sizeof(__pyx_k_message), 0, 0, 1, 1},
  {&__pyx_n_s_metaclass, __pyx_k_metaclass, sizeof(__pyx_k_metaclass), 0, 0, 1, 1},
  {&__pyx_n_s_metadata, __pyx_k_metadata, sizeof(__pyx_k_metadata), 0, 0, 1, 1},
  {&__pyx_n_s_metadata_locals_genexpr, __pyx_k_metadata_locals_genexpr, sizeof(__pyx_k_metadata_locals_genexpr), 0, 0, 1, 1},
  {&__pyx_n_s_metadata_plugin, __pyx_k_metadata_plugin, sizeof(__pyx_k_metadata_plugin), 0, 0, 1, 1},
  {&__pyx_kp_s_metadata_was_invalid_s, __pyx_k_metadata_was_invalid_s, sizeof(__pyx_k_metadata_was_invalid_s), 0, 0, 1, 0},
  {&__pyx_n_s_method, __pyx_k_method, sizeof(__pyx_k_method), 0, 0, 1, 1},
  {&__pyx_n_s_module, __pyx_k_module, sizeof(__pyx_k_module), 0, 0, 1, 1},
  {&__pyx_n_s_name, __pyx_k_name, sizeof(__pyx_k_name), 0, 0, 1, 1},
  {&__pyx_n_s_name_2, __pyx_k_name_2, sizeof(__pyx_k_name_2), 0, 0, 1, 1},
  {&__pyx_n_s_namedtuple, __pyx_k_namedtuple, sizeof(__pyx_k_namedtuple), 0, 0, 1, 1},
  {&__pyx_n_s_new, __pyx_k_new, sizeof(__pyx_k_new), 0, 0, 1, 1},
  {&__pyx_n_s_next_call_event, __pyx_k_next_call_event, sizeof(__pyx_k_next_call_event), 0, 0, 1, 1},
  {&__pyx_n_s_next_call_event_locals_on_succes, __pyx_k_next_call_event_locals_on_succes, sizeof(__pyx_k_next_call_event_locals_on_succes), 0, 0, 1, 1},
  {&__pyx_n_s_next_event_locals_on_success, __pyx_k_next_event_locals_on_success, sizeof(__pyx_k_next_event_locals_on_success), 0, 0, 1, 1},
  {&__pyx_n_s_no_compress, __pyx_k_no_compress, sizeof(__pyx_k_no_compress), 0, 0, 1, 1},
  {&__pyx_kp_s_no_default___reduce___due_to_non, __pyx_k_no_default___reduce___due_to_non, sizeof(__pyx_k_no_default___reduce___due_to_non), 0, 0, 1, 0},
  {&__pyx_n_s_none, __pyx_k_none, sizeof(__pyx_k_none), 0, 0, 1, 1},
  {&__pyx_n_s_not_found, __pyx_k_not_found, sizeof(__pyx_k_not_found), 0, 0, 1, 1},
  {&__pyx_n_s_not_invoked, __pyx_k_not_invoked, sizeof(__pyx_k_not_invoked), 0, 0, 1, 1},
  {&__pyx_n_s_not_on_client, __pyx_k_not_on_client, sizeof(__pyx_k_not_on_client), 0, 0, 1, 1},
  {&__pyx_n_s_not_on_server, __pyx_k_not_on_server, sizeof(__pyx_k_not_on_server), 0, 0, 1, 1},
  {&__pyx_n_s_notify_all, __pyx_k_notify_all, sizeof(__pyx_k_notify_all), 0, 0, 1, 1},
  {&__pyx_n_s_num_active_threads, __pyx_k_num_active_threads, sizeof(__pyx_k_num_active_threads), 0, 0, 1, 1},
  {&__pyx_n_s_object, __pyx_k_object, sizeof(__pyx_k_object), 0, 0, 1, 1},
  {&__pyx_n_s_ok, __pyx_k_ok, sizeof(__pyx_k_ok), 0, 0, 1, 1},
  {&__pyx_n_s_on_finish, __pyx_k_on_finish, sizeof(__pyx_k_on_finish), 0, 0, 1, 1},
  {&__pyx_n_s_on_success, __pyx_k_on_success, sizeof(__pyx_k_on_success), 0, 0, 1, 1},
  {&__pyx_n_s_operation_complete, __pyx_k_operation_complete, sizeof(__pyx_k_operation_complete), 0, 0, 1, 1},
  {&__pyx_n_s_operations, __pyx_k_operations, sizeof(__pyx_k_operations), 0, 0, 1, 1},
  {&__pyx_n_s_operationses_and_tags, __pyx_k_operationses_and_tags, sizeof(__pyx_k_operationses_and_tags), 0, 0, 1, 1},
  {&__pyx_n_s_os, __pyx_k_os, sizeof(__pyx_k_os), 0, 0, 1, 1},
  {&__pyx_n_s_os_path, __pyx_k_os_path, sizeof(__pyx_k_os_path), 0, 0, 1, 1},
  {&__pyx_n_s_out_of_range, __pyx_k_out_of_range, sizeof(__pyx_k_out_of_range), 0, 0, 1, 1},
  {&__pyx_n_s_peer_identities, __pyx_k_peer_identities, sizeof(__pyx_k_peer_identities), 0, 0, 1, 1},
  {&__pyx_n_s_peer_identity_key, __pyx_k_peer_identity_key, sizeof(__pyx_k_peer_identity_key), 0, 0, 1, 1},
  {&__pyx_n_s_pem_key_cert_pairs, __pyx_k_pem_key_cert_pairs, sizeof(__pyx_k_pem_key_cert_pairs), 0, 0, 1, 1},
  {&__pyx_n_s_pem_root_certificates, __pyx_k_pem_root_certificates, sizeof(__pyx_k_pem_root_certificates), 0, 0, 1, 1},
  {&__pyx_n_s_pem_root_certs, __pyx_k_pem_root_certs, sizeof(__pyx_k_pem_root_certs), 0, 0, 1, 1},
  {&__pyx_n_s_permission_denied, __pyx_k_permission_denied, sizeof(__pyx_k_permission_denied), 0, 0, 1, 1},
  {&__pyx_n_s_pickle, __pyx_k_pickle, sizeof(__pyx_k_pickle), 0, 0, 1, 1},
  {&__pyx_n_s_pkgutil, __pyx_k_pkgutil, sizeof(__pyx_k_pkgutil), 0, 0, 1, 1},
  {&__pyx_n_s_poll, __pyx_k_poll, sizeof(__pyx_k_poll), 0, 0, 1, 1},
  {&__pyx_n_s_pool, __pyx_k_pool, sizeof(__pyx_k_pool), 0, 0, 1, 1},
  {&__pyx_n_s_pop, __pyx_k_pop, sizeof(__pyx_k_pop), 0, 0, 1, 1},
  {&__pyx_n_s_postfork_state_to_reset, __pyx_k_postfork_state_to_reset, sizeof(__pyx_k_postfork_state_to_reset), 0, 0, 1, 1},
  {&__pyx_n_s_postfork_states_to_reset, __pyx_k_postfork_states_to_reset, sizeof(__pyx_k_postfork_states_to_reset), 0, 0, 1, 1},
  {&__pyx_n_s_prepare, __pyx_k_prepare, sizeof(__pyx_k_prepare), 0, 0, 1, 1},
  {&__pyx_n_s_primary_user_agent_string, __pyx_k_primary_user_agent_string, sizeof(__pyx_k_primary_user_agent_string), 0, 0, 1, 1},
  {&__pyx_n_s_private_key, __pyx_k_private_key, sizeof(__pyx_k_private_key), 0, 0, 1, 1},
  {&__pyx_n_s_properties, __pyx_k_properties, sizeof(__pyx_k_properties), 0, 0, 1, 1},
  {&__pyx_n_s_property, __pyx_k_property, sizeof(__pyx_k_property), 0, 0, 1, 1},
  {&__pyx_n_s_py_auth_context, __pyx_k_py_auth_context, sizeof(__pyx_k_py_auth_context), 0, 0, 1, 1},
  {&__pyx_n_s_pyx_PickleError, __pyx_k_pyx_PickleError, sizeof(__pyx_k_pyx_PickleError), 0, 0, 1, 1},
  {&__pyx_n_s_pyx_checksum, __pyx_k_pyx_checksum, sizeof(__pyx_k_pyx_checksum), 0, 0, 1, 1},
  {&__pyx_n_s_pyx_result, __pyx_k_pyx_result, sizeof(__pyx_k_pyx_result), 0, 0, 1, 1},
  {&__pyx_n_s_pyx_state, __pyx_k_pyx_state, sizeof(__pyx_k_pyx_state), 0, 0, 1, 1},
  {&__pyx_n_s_pyx_type, __pyx_k_pyx_type, sizeof(__pyx_k_pyx_type), 0, 0, 1, 1},
  {&__pyx_n_s_pyx_unpickle_CensusContext, __pyx_k_pyx_unpickle_CensusContext, sizeof(__pyx_k_pyx_unpickle_CensusContext), 0, 0, 1, 1},
  {&__pyx_n_s_pyx_unpickle_ChannelCredential, __pyx_k_pyx_unpickle_ChannelCredential, sizeof(__pyx_k_pyx_unpickle_ChannelCredential), 0, 0, 1, 1},
  {&__pyx_n_s_pyx_unpickle__Tag, __pyx_k_pyx_unpickle__Tag, sizeof(__pyx_k_pyx_unpickle__Tag), 0, 0, 1, 1},
  {&__pyx_n_s_pyx_vtable, __pyx_k_pyx_vtable, sizeof(__pyx_k_pyx_vtable), 0, 0, 1, 1},
  {&__pyx_n_s_qualname, __pyx_k_qualname, sizeof(__pyx_k_qualname), 0, 0, 1, 1},
  {&__pyx_n_s_queue, __pyx_k_queue, sizeof(__pyx_k_queue), 0, 0, 1, 1},
  {&__pyx_kp_s_queue_must_be_live, __pyx_k_queue_must_be_live, sizeof(__pyx_k_queue_must_be_live), 0, 0, 1, 0},
  {&__pyx_kp_s_queue_must_be_shutting_down_to_b, __pyx_k_queue_must_be_shutting_down_to_b, sizeof(__pyx_k_queue_must_be_shutting_down_to_b), 0, 0, 1, 0},
  {&__pyx_n_s_queue_shutdown, __pyx_k_queue_shutdown, sizeof(__pyx_k_queue_shutdown), 0, 0, 1, 1},
  {&__pyx_n_s_queue_timeout, __pyx_k_queue_timeout, sizeof(__pyx_k_queue_timeout), 0, 0, 1, 1},
  {&__pyx_n_s_range, __pyx_k_range, sizeof(__pyx_k_range), 0, 0, 1, 1},
  {&__pyx_n_s_ready, __pyx_k_ready, sizeof(__pyx_k_ready), 0, 0, 1, 1},
  {&__pyx_n_s_receive_close_on_server, __pyx_k_receive_close_on_server, sizeof(__pyx_k_receive_close_on_server), 0, 0, 1, 1},
  {&__pyx_n_s_receive_initial_metadata, __pyx_k_receive_initial_metadata, sizeof(__pyx_k_receive_initial_metadata), 0, 0, 1, 1},
  {&__pyx_n_s_receive_message, __pyx_k_receive_message, sizeof(__pyx_k_receive_message), 0, 0, 1, 1},
  {&__pyx_n_s_receive_status_on_client, __pyx_k_receive_status_on_client, sizeof(__pyx_k_receive_status_on_client), 0, 0, 1, 1},
  {&__pyx_n_s_recv, __pyx_k_recv, sizeof(__pyx_k_recv), 0, 0, 1, 1},
  {&__pyx_n_s_reduce, __pyx_k_reduce, sizeof(__pyx_k_reduce), 0, 0, 1, 1},
  {&__pyx_n_s_reduce_cython, __pyx_k_reduce_cython, sizeof(__pyx_k_reduce_cython), 0, 0, 1, 1},
  {&__pyx_n_s_reduce_ex, __pyx_k_reduce_ex, sizeof(__pyx_k_reduce_ex), 0, 0, 1, 1},
  {&__pyx_n_s_register_completion_queue, __pyx_k_register_completion_queue, sizeof(__pyx_k_register_completion_queue), 0, 0, 1, 1},
  {&__pyx_n_s_reset_postfork_child, __pyx_k_reset_postfork_child, sizeof(__pyx_k_reset_postfork_child), 0, 0, 1, 1},
  {&__pyx_n_s_resolve_wrapper, __pyx_k_resolve_wrapper, sizeof(__pyx_k_resolve_wrapper), 0, 0, 1, 1},
  {&__pyx_n_s_resource_exhausted, __pyx_k_resource_exhausted, sizeof(__pyx_k_resource_exhausted), 0, 0, 1, 1},
  {&__pyx_n_s_retain_self, __pyx_k_retain_self, sizeof(__pyx_k_retain_self), 0, 0, 1, 1},
  {&__pyx_n_s_return_from_user_request_generat, __pyx_k_return_from_user_request_generat, sizeof(__pyx_k_return_from_user_request_generat), 0, 0, 1, 1},
  {&__pyx_n_s_rstrip, __pyx_k_rstrip, sizeof(__pyx_k_rstrip), 0, 0, 1, 1},
  {&__pyx_n_s_s, __pyx_k_s, sizeof(__pyx_k_s), 0, 0, 1, 1},
  {&__pyx_n_s_secondary_user_agent_string, __pyx_k_secondary_user_agent_string, sizeof(__pyx_k_secondary_user_agent_string), 0, 0, 1, 1},
  {&__pyx_n_s_segregated_call_locals_on_succe, __pyx_k_segregated_call_locals_on_succe, sizeof(__pyx_k_segregated_call_locals_on_succe), 0, 0, 1, 1},
  {&__pyx_n_s_self, __pyx_k_self, sizeof(__pyx_k_self), 0, 0, 1, 1},
  {&__pyx_kp_s_self_arg_cannot_be_converted_to, __pyx_k_self_arg_cannot_be_converted_to, sizeof(__pyx_k_self_arg_cannot_be_converted_to), 0, 0, 1, 0},
  {&__pyx_kp_s_self_c_argument_cannot_be_conver, __pyx_k_self_c_argument_cannot_be_conver, sizeof(__pyx_k_self_c_argument_cannot_be_conver), 0, 0, 1, 0},
  {&__pyx_kp_s_self_c_credentials_cannot_be_con, __pyx_k_self_c_credentials_cannot_be_con, sizeof(__pyx_k_self_c_credentials_cannot_be_con), 0, 0, 1, 0},
  {&__pyx_kp_s_self_c_op_cannot_be_converted_to, __pyx_k_self_c_op_cannot_be_converted_to, sizeof(__pyx_k_self_c_op_cannot_be_converted_to), 0, 0, 1, 0},
  {&__pyx_n_s_send, __pyx_k_send, sizeof(__pyx_k_send), 0, 0, 1, 1},
  {&__pyx_n_s_send_close_from_client, __pyx_k_send_close_from_client, sizeof(__pyx_k_send_close_from_client), 0, 0, 1, 1},
  {&__pyx_n_s_send_initial_metadata, __pyx_k_send_initial_metadata, sizeof(__pyx_k_send_initial_metadata), 0, 0, 1, 1},
  {&__pyx_n_s_send_message, __pyx_k_send_message, sizeof(__pyx_k_send_message), 0, 0, 1, 1},
  {&__pyx_n_s_send_status_from_server, __pyx_k_send_status_from_server, sizeof(__pyx_k_send_status_from_server), 0, 0, 1, 1},
  {&__pyx_n_s_sendmsg, __pyx_k_sendmsg, sizeof(__pyx_k_sendmsg), 0, 0, 1, 1},
  {&__pyx_n_s_server_certificate_config_ssl, __pyx_k_server_certificate_config_ssl, sizeof(__pyx_k_server_certificate_config_ssl), 0, 0, 1, 1},
  {&__pyx_n_s_server_credentials, __pyx_k_server_credentials, sizeof(__pyx_k_server_credentials), 0, 0, 1, 1},
  {&__pyx_n_s_server_credentials_ssl, __pyx_k_server_credentials_ssl, sizeof(__pyx_k_server_credentials_ssl), 0, 0, 1, 1},
  {&__pyx_n_s_server_credentials_ssl_dynamic_c, __pyx_k_server_credentials_ssl_dynamic_c, sizeof(__pyx_k_server_credentials_ssl_dynamic_c), 0, 0, 1, 1},
  {&__pyx_n_s_server_id, __pyx_k_server_id, sizeof(__pyx_k_server_id), 0, 0, 1, 1},
  {&__pyx_kp_s_server_must_be_started_and_not_s, __pyx_k_server_must_be_started_and_not_s, sizeof(__pyx_k_server_must_be_started_and_not_s), 0, 0, 1, 0},
  {&__pyx_n_s_server_queue, __pyx_k_server_queue, sizeof(__pyx_k_server_queue), 0, 0, 1, 1},
  {&__pyx_kp_s_server_queue_must_be_a_registere, __pyx_k_server_queue_must_be_a_registere, sizeof(__pyx_k_server_queue_must_be_a_registere), 0, 0, 1, 0},
  {&__pyx_n_s_set, __pyx_k_set, sizeof(__pyx_k_set), 0, 0, 1, 1},
  {&__pyx_n_s_setDaemon, __pyx_k_setDaemon, sizeof(__pyx_k_setDaemon), 0, 0, 1, 1},
  {&__pyx_n_s_set_async_callback_func, __pyx_k_set_async_callback_func, sizeof(__pyx_k_set_async_callback_func), 0, 0, 1, 1},
  {&__pyx_n_s_set_census_context_on_call, __pyx_k_set_census_context_on_call, sizeof(__pyx_k_set_census_context_on_call), 0, 0, 1, 1},
  {&__pyx_n_s_setsockopt, __pyx_k_setsockopt, sizeof(__pyx_k_setsockopt), 0, 0, 1, 1},
  {&__pyx_n_s_setstate, __pyx_k_setstate, sizeof(__pyx_k_setstate), 0, 0, 1, 1},
  {&__pyx_n_s_setstate_cython, __pyx_k_setstate_cython, sizeof(__pyx_k_setstate_cython), 0, 0, 1, 1},
  {&__pyx_n_s_shutdown, __pyx_k_shutdown, sizeof(__pyx_k_shutdown), 0, 0, 1, 1},
  {&__pyx_n_s_shutdown_cq, __pyx_k_shutdown_cq, sizeof(__pyx_k_shutdown_cq), 0, 0, 1, 1},
  {&__pyx_n_s_shutting_down_server, __pyx_k_shutting_down_server, sizeof(__pyx_k_shutting_down_server), 0, 0, 1, 1},
  {&__pyx_n_s_sleep, __pyx_k_sleep, sizeof(__pyx_k_sleep), 0, 0, 1, 1},
  {&__pyx_n_s_socket, __pyx_k_socket, sizeof(__pyx_k_socket), 0, 0, 1, 1},
  {&__pyx_n_s_socket_accept_async, __pyx_k_socket_accept_async, sizeof(__pyx_k_socket_accept_async), 0, 0, 1, 1},
  {&__pyx_n_s_socket_connect_async, __pyx_k_socket_connect_async, sizeof(__pyx_k_socket_connect_async), 0, 0, 1, 1},
  {&__pyx_n_s_socket_id, __pyx_k_socket_id, sizeof(__pyx_k_socket_id), 0, 0, 1, 1},
  {&__pyx_n_s_socket_read_async, __pyx_k_socket_read_async, sizeof(__pyx_k_socket_read_async), 0, 0, 1, 1},
  {&__pyx_n_s_socket_resolve_async_python, __pyx_k_socket_resolve_async_python, sizeof(__pyx_k_socket_resolve_async_python), 0, 0, 1, 1},
  {&__pyx_n_s_socket_sendmsg, __pyx_k_socket_sendmsg, sizeof(__pyx_k_socket_sendmsg), 0, 0, 1, 1},
  {&__pyx_n_s_socket_wrapper, __pyx_k_socket_wrapper, sizeof(__pyx_k_socket_wrapper), 0, 0, 1, 1},
  {&__pyx_n_s_socket_write_async, __pyx_k_socket_write_async, sizeof(__pyx_k_socket_write_async), 0, 0, 1, 1},
  {&__pyx_n_s_spawn, __pyx_k_spawn, sizeof(__pyx_k_spawn), 0, 0, 1, 1},
  {&__pyx_n_s_spawn_callback_async, __pyx_k_spawn_callback_async, sizeof(__pyx_k_spawn_callback_async), 0, 0, 1, 1},
  {&__pyx_n_s_spawn_callback_in_thread, __pyx_k_spawn_callback_in_thread, sizeof(__pyx_k_spawn_callback_in_thread), 0, 0, 1, 1},
  {&__pyx_n_s_spawn_greenlet, __pyx_k_spawn_greenlet, sizeof(__pyx_k_spawn_greenlet), 0, 0, 1, 1},
  {&__pyx_kp_s_src_python_grpcio_grpc__cython, __pyx_k_src_python_grpcio_grpc__cython, sizeof(__pyx_k_src_python_grpcio_grpc__cython), 0, 0, 1, 0},
  {&__pyx_kp_s_src_python_grpcio_grpc__cython_2, __pyx_k_src_python_grpcio_grpc__cython_2, sizeof(__pyx_k_src_python_grpcio_grpc__cython_2), 0, 0, 1, 0},
  {&__pyx_kp_s_src_python_grpcio_grpc__cython_3, __pyx_k_src_python_grpcio_grpc__cython_3, sizeof(__pyx_k_src_python_grpcio_grpc__cython_3), 0, 0, 1, 0},
  {&__pyx_kp_s_src_python_grpcio_grpc__cython_4, __pyx_k_src_python_grpcio_grpc__cython_4, sizeof(__pyx_k_src_python_grpcio_grpc__cython_4), 0, 0, 1, 0},
  {&__pyx_kp_s_src_python_grpcio_grpc__cython_5, __pyx_k_src_python_grpcio_grpc__cython_5, sizeof(__pyx_k_src_python_grpcio_grpc__cython_5), 0, 0, 1, 0},
  {&__pyx_kp_s_src_python_grpcio_grpc__cython_6, __pyx_k_src_python_grpcio_grpc__cython_6, sizeof(__pyx_k_src_python_grpcio_grpc__cython_6), 0, 0, 1, 0},
  {&__pyx_kp_s_src_python_grpcio_grpc__cython_7, __pyx_k_src_python_grpcio_grpc__cython_7, sizeof(__pyx_k_src_python_grpcio_grpc__cython_7), 0, 0, 1, 0},
  {&__pyx_kp_s_src_python_grpcio_grpc__cython_8, __pyx_k_src_python_grpcio_grpc__cython_8, sizeof(__pyx_k_src_python_grpcio_grpc__cython_8), 0, 0, 1, 0},
  {&__pyx_n_s_ssl_session_cache, __pyx_k_ssl_session_cache, sizeof(__pyx_k_ssl_session_cache), 0, 0, 1, 1},
  {&__pyx_n_s_ssl_target_name_override, __pyx_k_ssl_target_name_override, sizeof(__pyx_k_ssl_target_name_override), 0, 0, 1, 1},
  {&__pyx_n_s_start, __pyx_k_start, sizeof(__pyx_k_start), 0, 0, 1, 1},
  {&__pyx_n_s_start_batch, __pyx_k_start_batch, sizeof(__pyx_k_start_batch), 0, 0, 1, 1},
  {&__pyx_n_s_start_channel_id, __pyx_k_start_channel_id, sizeof(__pyx_k_start_channel_id), 0, 0, 1, 1},
  {&__pyx_n_s_start_server_id, __pyx_k_start_server_id, sizeof(__pyx_k_start_server_id), 0, 0, 1, 1},
  {&__pyx_n_s_start_socket_id, __pyx_k_start_socket_id, sizeof(__pyx_k_start_socket_id), 0, 0, 1, 1},
  {&__pyx_n_s_started_tag, __pyx_k_started_tag, sizeof(__pyx_k_started_tag), 0, 0, 1, 1},
  {&__pyx_n_s_started_tags, __pyx_k_started_tags, sizeof(__pyx_k_started_tags), 0, 0, 1, 1},
  {&__pyx_n_s_status, __pyx_k_status, sizeof(__pyx_k_status), 0, 0, 1, 1},
  {&__pyx_n_s_stop, __pyx_k_stop, sizeof(__pyx_k_stop), 0, 0, 1, 1},
  {&__pyx_kp_s_stringsource, __pyx_k_stringsource, sizeof(__pyx_k_stringsource), 0, 0, 1, 0},
  {&__pyx_n_s_subchannel_id, __pyx_k_subchannel_id, sizeof(__pyx_k_subchannel_id), 0, 0, 1, 1},
  {&__pyx_n_s_success, __pyx_k_success, sizeof(__pyx_k_success), 0, 0, 1, 1},
  {&__pyx_n_s_sys, __pyx_k_sys, sizeof(__pyx_k_sys), 0, 0, 1, 1},
  {&__pyx_n_s_tag, __pyx_k_tag, sizeof(__pyx_k_tag), 0, 0, 1, 1},
  {&__pyx_n_s_target, __pyx_k_target, sizeof(__pyx_k_target), 0, 0, 1, 1},
  {&__pyx_n_s_test, __pyx_k_test, sizeof(__pyx_k_test), 0, 0, 1, 1},
  {&__pyx_kp_s_the_server_has_already_started, __pyx_k_the_server_has_already_started, sizeof(__pyx_k_the_server_has_already_started), 0, 0, 1, 0},
  {&__pyx_kp_s_the_server_hasn_t_started_yet, __pyx_k_the_server_hasn_t_started_yet, sizeof(__pyx_k_the_server_hasn_t_started_yet), 0, 0, 1, 0},
  {&__pyx_kp_s_the_server_must_be_shutting_down, __pyx_k_the_server_must_be_shutting_down, sizeof(__pyx_k_the_server_must_be_shutting_down), 0, 0, 1, 0},
  {&__pyx_n_s_thread, __pyx_k_thread, sizeof(__pyx_k_thread), 0, 0, 1, 1},
  {&__pyx_n_s_threading, __pyx_k_threading, sizeof(__pyx_k_threading), 0, 0, 1, 1},
  {&__pyx_n_s_throw, __pyx_k_throw, sizeof(__pyx_k_throw), 0, 0, 1, 1},
  {&__pyx_n_s_time, __pyx_k_time, sizeof(__pyx_k_time), 0, 0, 1, 1},
  {&__pyx_n_s_timeout_secs, __pyx_k_timeout_secs, sizeof(__pyx_k_timeout_secs), 0, 0, 1, 1},
  {&__pyx_n_s_timer, __pyx_k_timer, sizeof(__pyx_k_timer), 0, 0, 1, 1},
  {&__pyx_n_s_too_many_operations, __pyx_k_too_many_operations, sizeof(__pyx_k_too_many_operations), 0, 0, 1, 1},
  {&__pyx_n_s_trailing_metadata, __pyx_k_trailing_metadata, sizeof(__pyx_k_trailing_metadata), 0, 0, 1, 1},
  {&__pyx_n_s_transient_failure, __pyx_k_transient_failure, sizeof(__pyx_k_transient_failure), 0, 0, 1, 1},
  {&__pyx_n_s_true, __pyx_k_true, sizeof(__pyx_k_true), 0, 0, 1, 1},
  {&__pyx_n_s_tuples_to_resolvaddr_locals_gene, __pyx_k_tuples_to_resolvaddr_locals_gene, sizeof(__pyx_k_tuples_to_resolvaddr_locals_gene), 0, 0, 1, 1},
  {&__pyx_n_s_type, __pyx_k_type, sizeof(__pyx_k_type), 0, 0, 1, 1},
  {&__pyx_n_s_unauthenticated, __pyx_k_unauthenticated, sizeof(__pyx_k_unauthenticated), 0, 0, 1, 1},
  {&__pyx_n_s_unavailable, __pyx_k_unavailable, sizeof(__pyx_k_unavailable), 0, 0, 1, 1},
  {&__pyx_n_s_unimplemented, __pyx_k_unimplemented, sizeof(__pyx_k_unimplemented), 0, 0, 1, 1},
  {&__pyx_n_s_uninstall_context, __pyx_k_uninstall_context, sizeof(__pyx_k_uninstall_context), 0, 0, 1, 1},
  {&__pyx_n_s_unknown, __pyx_k_unknown, sizeof(__pyx_k_unknown), 0, 0, 1, 1},
  {&__pyx_n_s_update, __pyx_k_update, sizeof(__pyx_k_update), 0, 0, 1, 1},
  {&__pyx_n_s_used_mask, __pyx_k_used_mask, sizeof(__pyx_k_used_mask), 0, 0, 1, 1},
  {&__pyx_n_s_user_tag, __pyx_k_user_tag, sizeof(__pyx_k_user_tag), 0, 0, 1, 1},
  {&__pyx_n_s_value, __pyx_k_value, sizeof(__pyx_k_value), 0, 0, 1, 1},
  {&__pyx_n_s_values, __pyx_k_values, sizeof(__pyx_k_values), 0, 0, 1, 1},
  {&__pyx_n_s_vtable, __pyx_k_vtable, sizeof(__pyx_k_vtable), 0, 0, 1, 1},
  {&__pyx_n_s_wait, __pyx_k_wait, sizeof(__pyx_k_wait), 0, 0, 1, 1},
  {&__pyx_n_s_wait_for_ready, __pyx_k_wait_for_ready, sizeof(__pyx_k_wait_for_ready), 0, 0, 1, 1},
  {&__pyx_n_s_wait_for_ready_explicitly_set, __pyx_k_wait_for_ready_explicitly_set, sizeof(__pyx_k_wait_for_ready_explicitly_set), 0, 0, 1, 1},
  {&__pyx_n_s_wait_time, __pyx_k_wait_time, sizeof(__pyx_k_wait_time), 0, 0, 1, 1},
  {&__pyx_n_s_write_bytes, __pyx_k_write_bytes, sizeof(__pyx_k_write_bytes), 0, 0, 1, 1},
  {&__pyx_n_s_yes, __pyx_k_yes, sizeof(__pyx_k_yes), 0, 0, 1, 1},
  {0, 0, 0, 0, 0, 0, 0}
};
static int __Pyx_InitCachedBuiltins(void) {
  __pyx_builtin_object = __Pyx_GetBuiltinName(__pyx_n_s_object); if (!__pyx_builtin_object) __PYX_ERR(0, 101, __pyx_L1_error)
  __pyx_builtin_TypeError = __Pyx_GetBuiltinName(__pyx_n_s_TypeError); if (!__pyx_builtin_TypeError) __PYX_ERR(1, 27, __pyx_L1_error)
  __pyx_builtin_UnicodeDecodeError = __Pyx_GetBuiltinName(__pyx_n_s_UnicodeDecodeError); if (!__pyx_builtin_UnicodeDecodeError) __PYX_ERR(1, 52, __pyx_L1_error)
  __pyx_builtin_enumerate = __Pyx_GetBuiltinName(__pyx_n_s_enumerate); if (!__pyx_builtin_enumerate) __PYX_ERR(2, 76, __pyx_L1_error)
  __pyx_builtin_ValueError = __Pyx_GetBuiltinName(__pyx_n_s_ValueError); if (!__pyx_builtin_ValueError) __PYX_ERR(3, 28, __pyx_L1_error)
  __pyx_builtin_NotImplementedError = __Pyx_GetBuiltinName(__pyx_n_s_NotImplementedError); if (!__pyx_builtin_NotImplementedError) __PYX_ERR(4, 37, __pyx_L1_error)
  __pyx_builtin_range = __Pyx_GetBuiltinName(__pyx_n_s_range); if (!__pyx_builtin_range) __PYX_ERR(4, 228, __pyx_L1_error)
  __pyx_builtin_RuntimeError = __Pyx_GetBuiltinName(__pyx_n_s_RuntimeError); if (!__pyx_builtin_RuntimeError) __PYX_ERR(5, 121, __pyx_L1_error)
  __pyx_builtin_IOError = __Pyx_GetBuiltinName(__pyx_n_s_IOError); if (!__pyx_builtin_IOError) __PYX_ERR(6, 105, __pyx_L1_error)
  __pyx_builtin_AttributeError = __Pyx_GetBuiltinName(__pyx_n_s_AttributeError); if (!__pyx_builtin_AttributeError) __PYX_ERR(6, 153, __pyx_L1_error)
  return 0;
  __pyx_L1_error:;
  return -1;
}

static int __Pyx_InitCachedConstants(void) {
  __Pyx_RefNannyDeclarations
  __Pyx_RefNannySetupContext("__Pyx_InitCachedConstants", 0);

  /* "src/python/grpcio/grpc/_cython/_cygrpc/grpc_string.pyx.pxi":25
 *     return s
 *   elif isinstance(s, unicode):
 *     return s.encode('ascii')             # <<<<<<<<<<<<<<
 *   else:
 *     raise TypeError('Expected bytes, str, or unicode, not {}'.format(type(s)))
 */
  __pyx_tuple_ = PyTuple_Pack(1, __pyx_n_s_ascii); if (unlikely(!__pyx_tuple_)) __PYX_ERR(1, 25, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_tuple_);
  __Pyx_GIVEREF(__pyx_tuple_);

  /* "src/python/grpcio/grpc/_cython/_cygrpc/grpc_string.pyx.pxi":41
 *     return <bytes>string_or_none
 *   elif isinstance(string_or_none, (unicode,)):
 *     return string_or_none.encode('ascii')             # <<<<<<<<<<<<<<
 *   else:
 *     raise TypeError('Expected str, not {}'.format(type(string_or_none)))
 */
  __pyx_tuple__3 = PyTuple_Pack(1, __pyx_n_s_ascii); if (unlikely(!__pyx_tuple__3)) __PYX_ERR(1, 41, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_tuple__3);
  __Pyx_GIVEREF(__pyx_tuple__3);

  /* "(tree fragment)":2
 * def __reduce_cython__(self):
 *     raise TypeError("self.arg cannot be converted to a Python object for pickling")             # <<<<<<<<<<<<<<
 * def __setstate_cython__(self, __pyx_state):
 *     raise TypeError("self.arg cannot be converted to a Python object for pickling")
 */
  __pyx_tuple__4 = PyTuple_Pack(1, __pyx_kp_s_self_arg_cannot_be_converted_to); if (unlikely(!__pyx_tuple__4)) __PYX_ERR(7, 2, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_tuple__4);
  __Pyx_GIVEREF(__pyx_tuple__4);

  /* "(tree fragment)":4
 *     raise TypeError("self.arg cannot be converted to a Python object for pickling")
 * def __setstate_cython__(self, __pyx_state):
 *     raise TypeError("self.arg cannot be converted to a Python object for pickling")             # <<<<<<<<<<<<<<
 */
  __pyx_tuple__5 = PyTuple_Pack(1, __pyx_kp_s_self_arg_cannot_be_converted_to); if (unlikely(!__pyx_tuple__5)) __PYX_ERR(7, 4, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_tuple__5);
  __Pyx_GIVEREF(__pyx_tuple__5);

  /* "(tree fragment)":2
 * def __reduce_cython__(self):
 *     raise TypeError("self.c_argument cannot be converted to a Python object for pickling")             # <<<<<<<<<<<<<<
 * def __setstate_cython__(self, __pyx_state):
 *     raise TypeError("self.c_argument cannot be converted to a Python object for pickling")
 */
  __pyx_tuple__6 = PyTuple_Pack(1, __pyx_kp_s_self_c_argument_cannot_be_conver); if (unlikely(!__pyx_tuple__6)) __PYX_ERR(7, 2, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_tuple__6);
  __Pyx_GIVEREF(__pyx_tuple__6);

  /* "(tree fragment)":4
 *     raise TypeError("self.c_argument cannot be converted to a Python object for pickling")
 * def __setstate_cython__(self, __pyx_state):
 *     raise TypeError("self.c_argument cannot be converted to a Python object for pickling")             # <<<<<<<<<<<<<<
 */
  __pyx_tuple__7 = PyTuple_Pack(1, __pyx_kp_s_self_c_argument_cannot_be_conver); if (unlikely(!__pyx_tuple__7)) __PYX_ERR(7, 4, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_tuple__7);
  __Pyx_GIVEREF(__pyx_tuple__7);

  /* "(tree fragment)":2
 * def __reduce_cython__(self):
 *     raise TypeError("no default __reduce__ due to non-trivial __cinit__")             # <<<<<<<<<<<<<<
 * def __setstate_cython__(self, __pyx_state):
 *     raise TypeError("no default __reduce__ due to non-trivial __cinit__")
 */
  __pyx_tuple__8 = PyTuple_Pack(1, __pyx_kp_s_no_default___reduce___due_to_non); if (unlikely(!__pyx_tuple__8)) __PYX_ERR(7, 2, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_tuple__8);
  __Pyx_GIVEREF(__pyx_tuple__8);

  /* "(tree fragment)":4
 *     raise TypeError("no default __reduce__ due to non-trivial __cinit__")
 * def __setstate_cython__(self, __pyx_state):
 *     raise TypeError("no default __reduce__ due to non-trivial __cinit__")             # <<<<<<<<<<<<<<
 */
  __pyx_tuple__9 = PyTuple_Pack(1, __pyx_kp_s_no_default___reduce___due_to_non); if (unlikely(!__pyx_tuple__9)) __PYX_ERR(7, 4, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_tuple__9);
  __Pyx_GIVEREF(__pyx_tuple__9);

  /* "src/python/grpcio/grpc/_cython/_cygrpc/call.pyx.pxi":28
 *   def _start_batch(self, operations, tag, retain_self):
 *     if not self.is_valid:
 *       raise ValueError("invalid call object cannot be used from Python")             # <<<<<<<<<<<<<<
 *     cdef _BatchOperationTag batch_operation_tag = _BatchOperationTag(
 *         tag, operations, self if retain_self else None)
 */
  __pyx_tuple__10 = PyTuple_Pack(1, __pyx_kp_s_invalid_call_object_cannot_be_us); if (unlikely(!__pyx_tuple__10)) __PYX_ERR(3, 28, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_tuple__10);
  __Pyx_GIVEREF(__pyx_tuple__10);

  /* "src/python/grpcio/grpc/_cython/_cygrpc/call.pyx.pxi":53
 *     details = str_to_bytes(details)
 *     if not self.is_valid:
 *       raise ValueError("invalid call object cannot be used from Python")             # <<<<<<<<<<<<<<
 *     if (details is None) != (error_code == GRPC_STATUS__DO_NOT_USE):
 *       raise ValueError("if error_code is specified, so must details "
 */
  __pyx_tuple__12 = PyTuple_Pack(1, __pyx_kp_s_invalid_call_object_cannot_be_us); if (unlikely(!__pyx_tuple__12)) __PYX_ERR(3, 53, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_tuple__12);
  __Pyx_GIVEREF(__pyx_tuple__12);

  /* "src/python/grpcio/grpc/_cython/_cygrpc/call.pyx.pxi":55
 *       raise ValueError("invalid call object cannot be used from Python")
 *     if (details is None) != (error_code == GRPC_STATUS__DO_NOT_USE):
 *       raise ValueError("if error_code is specified, so must details "             # <<<<<<<<<<<<<<
 *                        "(and vice-versa)")
 *     cdef grpc_call_error result
 */
  __pyx_tuple__13 = PyTuple_Pack(1, __pyx_kp_s_if_error_code_is_specified_so_mu); if (unlikely(!__pyx_tuple__13)) __PYX_ERR(3, 55, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_tuple__13);
  __Pyx_GIVEREF(__pyx_tuple__13);

  /* "(tree fragment)":2
 * def __reduce_cython__(self):
 *     raise TypeError("no default __reduce__ due to non-trivial __cinit__")             # <<<<<<<<<<<<<<
 * def __setstate_cython__(self, __pyx_state):
 *     raise TypeError("no default __reduce__ due to non-trivial __cinit__")
 */
  __pyx_tuple__14 = PyTuple_Pack(1, __pyx_kp_s_no_default___reduce___due_to_non); if (unlikely(!__pyx_tuple__14)) __PYX_ERR(7, 2, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_tuple__14);
  __Pyx_GIVEREF(__pyx_tuple__14);

  /* "(tree fragment)":4
 *     raise TypeError("no default __reduce__ due to non-trivial __cinit__")
 * def __setstate_cython__(self, __pyx_state):
 *     raise TypeError("no default __reduce__ due to non-trivial __cinit__")             # <<<<<<<<<<<<<<
 */
  __pyx_tuple__15 = PyTuple_Pack(1, __pyx_kp_s_no_default___reduce___due_to_non); if (unlikely(!__pyx_tuple__15)) __PYX_ERR(7, 4, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_tuple__15);
  __Pyx_GIVEREF(__pyx_tuple__15);

  /* "(tree fragment)":2
 * def __reduce_cython__(self):
 *     raise TypeError("no default __reduce__ due to non-trivial __cinit__")             # <<<<<<<<<<<<<<
 * def __setstate_cython__(self, __pyx_state):
 *     raise TypeError("no default __reduce__ due to non-trivial __cinit__")
 */
  __pyx_tuple__16 = PyTuple_Pack(1, __pyx_kp_s_no_default___reduce___due_to_non); if (unlikely(!__pyx_tuple__16)) __PYX_ERR(7, 2, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_tuple__16);
  __Pyx_GIVEREF(__pyx_tuple__16);

  /* "(tree fragment)":4
 *     raise TypeError("no default __reduce__ due to non-trivial __cinit__")
 * def __setstate_cython__(self, __pyx_state):
 *     raise TypeError("no default __reduce__ due to non-trivial __cinit__")             # <<<<<<<<<<<<<<
 */
  __pyx_tuple__17 = PyTuple_Pack(1, __pyx_kp_s_no_default___reduce___due_to_non); if (unlikely(!__pyx_tuple__17)) __PYX_ERR(7, 4, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_tuple__17);
  __Pyx_GIVEREF(__pyx_tuple__17);

  /* "(tree fragment)":2
 * def __reduce_cython__(self):
 *     raise TypeError("no default __reduce__ due to non-trivial __cinit__")             # <<<<<<<<<<<<<<
 * def __setstate_cython__(self, __pyx_state):
 *     raise TypeError("no default __reduce__ due to non-trivial __cinit__")
 */
  __pyx_tuple__18 = PyTuple_Pack(1, __pyx_kp_s_no_default___reduce___due_to_non); if (unlikely(!__pyx_tuple__18)) __PYX_ERR(7, 2, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_tuple__18);
  __Pyx_GIVEREF(__pyx_tuple__18);

  /* "(tree fragment)":4
 *     raise TypeError("no default __reduce__ due to non-trivial __cinit__")
 * def __setstate_cython__(self, __pyx_state):
 *     raise TypeError("no default __reduce__ due to non-trivial __cinit__")             # <<<<<<<<<<<<<<
 */
  __pyx_tuple__19 = PyTuple_Pack(1, __pyx_kp_s_no_default___reduce___due_to_non); if (unlikely(!__pyx_tuple__19)) __PYX_ERR(7, 4, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_tuple__19);
  __Pyx_GIVEREF(__pyx_tuple__19);

  /* "src/python/grpcio/grpc/_cython/_cygrpc/channel.pyx.pxi":106
 *   cdef grpc_call_error c_call_error
 *   cdef _BatchOperationTag tag
 *   with channel_state.condition:             # <<<<<<<<<<<<<<
 *     if call_state.due:
 *       c_call_error, tag = _operate(call_state.c_call, operations, user_tag)
 */
  __pyx_tuple__20 = PyTuple_Pack(3, Py_None, Py_None, Py_None); if (unlikely(!__pyx_tuple__20)) __PYX_ERR(8, 106, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_tuple__20);
  __Pyx_GIVEREF(__pyx_tuple__20);
  __pyx_tuple__21 = PyTuple_Pack(3, Py_None, Py_None, Py_None); if (unlikely(!__pyx_tuple__21)) __PYX_ERR(8, 106, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_tuple__21);
  __Pyx_GIVEREF(__pyx_tuple__21);

  /* "src/python/grpcio/grpc/_cython/_cygrpc/channel.pyx.pxi":124
 *   cdef grpc_call_error c_call_error
 *   cdef _BatchOperationTag tag
 *   with channel_state.condition:             # <<<<<<<<<<<<<<
 *     if call_state.due:
 *       c_call_error, tag = _operate(call_state.c_call, operations, user_tag)
 */
  __pyx_tuple__22 = PyTuple_Pack(3, Py_None, Py_None, Py_None); if (unlikely(!__pyx_tuple__22)) __PYX_ERR(8, 124, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_tuple__22);
  __Pyx_GIVEREF(__pyx_tuple__22);
  __pyx_tuple__23 = PyTuple_Pack(3, Py_None, Py_None, Py_None); if (unlikely(!__pyx_tuple__23)) __PYX_ERR(8, 124, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_tuple__23);
  __Pyx_GIVEREF(__pyx_tuple__23);

  /* "src/python/grpcio/grpc/_cython/_cygrpc/channel.pyx.pxi":140
 *     str details):
 *   cdef grpc_call_error c_call_error
 *   with channel_state.condition:             # <<<<<<<<<<<<<<
 *     if call_state.due:
 *       c_call_error = grpc_call_cancel_with_status(
 */
  __pyx_tuple__24 = PyTuple_Pack(3, Py_None, Py_None, Py_None); if (unlikely(!__pyx_tuple__24)) __PYX_ERR(8, 140, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_tuple__24);
  __Pyx_GIVEREF(__pyx_tuple__24);

  /* "src/python/grpcio/grpc/_cython/_cygrpc/channel.pyx.pxi":151
 *     on_success, deadline):
 *   tag, event = _latent_event(c_completion_queue, deadline)
 *   with channel_state.condition:             # <<<<<<<<<<<<<<
 *     on_success(tag)
 *     channel_state.condition.notify_all()
 */
  __pyx_tuple__25 = PyTuple_Pack(3, Py_None, Py_None, Py_None); if (unlikely(!__pyx_tuple__25)) __PYX_ERR(8, 151, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_tuple__25);
  __Pyx_GIVEREF(__pyx_tuple__25);

  /* "src/python/grpcio/grpc/_cython/_cygrpc/channel.pyx.pxi":198
 *   cdef tuple error_and_wrapper_tag
 *   cdef _BatchOperationTag wrapper_tag
 *   with channel_state.condition:             # <<<<<<<<<<<<<<
 *     if channel_state.open:
 *       method_slice = _slice_from_bytes(method)
 */
  __pyx_tuple__26 = PyTuple_Pack(3, Py_None, Py_None, Py_None); if (unlikely(!__pyx_tuple__26)) __PYX_ERR(8, 198, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_tuple__26);
  __Pyx_GIVEREF(__pyx_tuple__26);

  /* "(tree fragment)":2
 * def __reduce_cython__(self):
 *     raise TypeError("no default __reduce__ due to non-trivial __cinit__")             # <<<<<<<<<<<<<<
 * def __setstate_cython__(self, __pyx_state):
 *     raise TypeError("no default __reduce__ due to non-trivial __cinit__")
 */
  __pyx_tuple__27 = PyTuple_Pack(1, __pyx_kp_s_no_default___reduce___due_to_non); if (unlikely(!__pyx_tuple__27)) __PYX_ERR(7, 2, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_tuple__27);
  __Pyx_GIVEREF(__pyx_tuple__27);

  /* "(tree fragment)":4
 *     raise TypeError("no default __reduce__ due to non-trivial __cinit__")
 * def __setstate_cython__(self, __pyx_state):
 *     raise TypeError("no default __reduce__ due to non-trivial __cinit__")             # <<<<<<<<<<<<<<
 */
  __pyx_tuple__28 = PyTuple_Pack(1, __pyx_kp_s_no_default___reduce___due_to_non); if (unlikely(!__pyx_tuple__28)) __PYX_ERR(7, 4, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_tuple__28);
  __Pyx_GIVEREF(__pyx_tuple__28);

  /* "src/python/grpcio/grpc/_cython/_cygrpc/channel.pyx.pxi":268
 *   call_state = _CallState()
 * 
 *   def on_success(started_tags):             # <<<<<<<<<<<<<<
 *     for started_tag in started_tags:
 *       state.integrated_call_states[started_tag] = call_state
 */
  __pyx_tuple__29 = PyTuple_Pack(2, __pyx_n_s_started_tags, __pyx_n_s_started_tag); if (unlikely(!__pyx_tuple__29)) __PYX_ERR(8, 268, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_tuple__29);
  __Pyx_GIVEREF(__pyx_tuple__29);
  __pyx_codeobj__30 = (PyObject*)__Pyx_PyCode_New(1, 0, 2, 0, CO_OPTIMIZED|CO_NEWLOCALS, __pyx_empty_bytes, __pyx_empty_tuple, __pyx_empty_tuple, __pyx_tuple__29, __pyx_empty_tuple, __pyx_empty_tuple, __pyx_kp_s_src_python_grpcio_grpc__cython, __pyx_n_s_on_success, 268, __pyx_empty_bytes); if (unlikely(!__pyx_codeobj__30)) __PYX_ERR(8, 268, __pyx_L1_error)

  /* "src/python/grpcio/grpc/_cython/_cygrpc/channel.pyx.pxi":307
 * 
 *   def next_event(self):
 *     def on_success(tag):             # <<<<<<<<<<<<<<
 *       _process_segregated_call_tag(
 *         self._channel_state, self._call_state, self._c_completion_queue, tag)
 */
  __pyx_tuple__31 = PyTuple_Pack(1, __pyx_n_s_tag); if (unlikely(!__pyx_tuple__31)) __PYX_ERR(8, 307, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_tuple__31);
  __Pyx_GIVEREF(__pyx_tuple__31);
  __pyx_codeobj__32 = (PyObject*)__Pyx_PyCode_New(1, 0, 1, 0, CO_OPTIMIZED|CO_NEWLOCALS, __pyx_empty_bytes, __pyx_empty_tuple, __pyx_empty_tuple, __pyx_tuple__31, __pyx_empty_tuple, __pyx_empty_tuple, __pyx_kp_s_src_python_grpcio_grpc__cython, __pyx_n_s_on_success, 307, __pyx_empty_bytes); if (unlikely(!__pyx_codeobj__32)) __PYX_ERR(8, 307, __pyx_L1_error)

  /* "(tree fragment)":2
 * def __reduce_cython__(self):
 *     raise TypeError("no default __reduce__ due to non-trivial __cinit__")             # <<<<<<<<<<<<<<
 * def __setstate_cython__(self, __pyx_state):
 *     raise TypeError("no default __reduce__ due to non-trivial __cinit__")
 */
  __pyx_tuple__33 = PyTuple_Pack(1, __pyx_kp_s_no_default___reduce___due_to_non); if (unlikely(!__pyx_tuple__33)) __PYX_ERR(7, 2, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_tuple__33);
  __Pyx_GIVEREF(__pyx_tuple__33);

  /* "(tree fragment)":4
 *     raise TypeError("no default __reduce__ due to non-trivial __cinit__")
 * def __setstate_cython__(self, __pyx_state):
 *     raise TypeError("no default __reduce__ due to non-trivial __cinit__")             # <<<<<<<<<<<<<<
 */
  __pyx_tuple__34 = PyTuple_Pack(1, __pyx_kp_s_no_default___reduce___due_to_non); if (unlikely(!__pyx_tuple__34)) __PYX_ERR(7, 4, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_tuple__34);
  __Pyx_GIVEREF(__pyx_tuple__34);

  /* "src/python/grpcio/grpc/_cython/_cygrpc/channel.pyx.pxi":322
 *   cdef grpc_completion_queue *c_completion_queue
 * 
 *   def on_success(started_tags):             # <<<<<<<<<<<<<<
 *     state.segregated_call_states.add(call_state)
 * 
 */
  __pyx_tuple__35 = PyTuple_Pack(1, __pyx_n_s_started_tags); if (unlikely(!__pyx_tuple__35)) __PYX_ERR(8, 322, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_tuple__35);
  __Pyx_GIVEREF(__pyx_tuple__35);
  __pyx_codeobj__36 = (PyObject*)__Pyx_PyCode_New(1, 0, 1, 0, CO_OPTIMIZED|CO_NEWLOCALS, __pyx_empty_bytes, __pyx_empty_tuple, __pyx_empty_tuple, __pyx_tuple__35, __pyx_empty_tuple, __pyx_empty_tuple, __pyx_kp_s_src_python_grpcio_grpc__cython, __pyx_n_s_on_success, 322, __pyx_empty_bytes); if (unlikely(!__pyx_codeobj__36)) __PYX_ERR(8, 322, __pyx_L1_error)

  /* "src/python/grpcio/grpc/_cython/_cygrpc/channel.pyx.pxi":329
 *       c_completion_queue = (grpc_completion_queue_create_for_next(NULL))
 *     else:
 *       raise ValueError('Cannot invoke RPC on closed channel!')             # <<<<<<<<<<<<<<
 * 
 *   try:
 */
  __pyx_tuple__37 = PyTuple_Pack(1, __pyx_kp_s_Cannot_invoke_RPC_on_closed_chan); if (unlikely(!__pyx_tuple__37)) __PYX_ERR(8, 329, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_tuple__37);
  __Pyx_GIVEREF(__pyx_tuple__37);

  /* "src/python/grpcio/grpc/_cython/_cygrpc/channel.pyx.pxi":325
 *     state.segregated_call_states.add(call_state)
 * 
 *   with state.condition:             # <<<<<<<<<<<<<<
 *     if state.open:
 *       c_completion_queue = (grpc_completion_queue_create_for_next(NULL))
 */
  __pyx_tuple__38 = PyTuple_Pack(3, Py_None, Py_None, Py_None); if (unlikely(!__pyx_tuple__38)) __PYX_ERR(8, 325, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_tuple__38);
  __Pyx_GIVEREF(__pyx_tuple__38);

  /* "src/python/grpcio/grpc/_cython/_cygrpc/channel.pyx.pxi":349
 *     object deadline):
 *   cdef _ConnectivityTag tag = _ConnectivityTag(object())
 *   with state.condition:             # <<<<<<<<<<<<<<
 *     if state.open:
 *       cpython.Py_INCREF(tag)
 */
  __pyx_tuple__39 = PyTuple_Pack(3, Py_None, Py_None, Py_None); if (unlikely(!__pyx_tuple__39)) __PYX_ERR(8, 349, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_tuple__39);
  __Pyx_GIVEREF(__pyx_tuple__39);

  /* "src/python/grpcio/grpc/_cython/_cygrpc/channel.pyx.pxi":360
 *   completed_tag, event = _latent_event(
 *       state.c_connectivity_completion_queue, None)
 *   with state.condition:             # <<<<<<<<<<<<<<
 *     state.connectivity_due.remove(completed_tag)
 *     state.condition.notify_all()
 */
  __pyx_tuple__40 = PyTuple_Pack(3, Py_None, Py_None, Py_None); if (unlikely(!__pyx_tuple__40)) __PYX_ERR(8, 360, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_tuple__40);
  __Pyx_GIVEREF(__pyx_tuple__40);

  /* "src/python/grpcio/grpc/_cython/_cygrpc/channel.pyx.pxi":371
 *   cdef _CallState call_state
 *   encoded_details = _encode(details)
 *   with state.condition:             # <<<<<<<<<<<<<<
 *     if state.open:
 *       state.open = False
 */
  __pyx_tuple__41 = PyTuple_Pack(3, Py_None, Py_None, Py_None); if (unlikely(!__pyx_tuple__41)) __PYX_ERR(8, 371, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_tuple__41);
  __Pyx_GIVEREF(__pyx_tuple__41);

  /* "src/python/grpcio/grpc/_cython/_cygrpc/channel.pyx.pxi":442
 *   def target(self):
 *     cdef char *c_target
 *     with self._state.condition:             # <<<<<<<<<<<<<<
 *       c_target = grpc_channel_get_target(self._state.c_channel)
 *       target = <bytes>c_target
 */
  __pyx_tuple__42 = PyTuple_Pack(3, Py_None, Py_None, Py_None); if (unlikely(!__pyx_tuple__42)) __PYX_ERR(8, 442, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_tuple__42);
  __Pyx_GIVEREF(__pyx_tuple__42);
  __pyx_tuple__43 = PyTuple_Pack(3, Py_None, Py_None, Py_None); if (unlikely(!__pyx_tuple__43)) __PYX_ERR(8, 442, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_tuple__43);
  __Pyx_GIVEREF(__pyx_tuple__43);

  /* "src/python/grpcio/grpc/_cython/_cygrpc/channel.pyx.pxi":457
 * 
 *   def next_call_event(self):
 *     def on_success(tag):             # <<<<<<<<<<<<<<
 *       if tag is not None:
 *         _process_integrated_call_tag(self._state, tag)
 */
  __pyx_tuple__44 = PyTuple_Pack(1, __pyx_n_s_tag); if (unlikely(!__pyx_tuple__44)) __PYX_ERR(8, 457, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_tuple__44);
  __Pyx_GIVEREF(__pyx_tuple__44);
  __pyx_codeobj__45 = (PyObject*)__Pyx_PyCode_New(1, 0, 1, 0, CO_OPTIMIZED|CO_NEWLOCALS, __pyx_empty_bytes, __pyx_empty_tuple, __pyx_empty_tuple, __pyx_tuple__44, __pyx_empty_tuple, __pyx_empty_tuple, __pyx_kp_s_src_python_grpcio_grpc__cython, __pyx_n_s_on_success, 457, __pyx_empty_bytes); if (unlikely(!__pyx_codeobj__45)) __PYX_ERR(8, 457, __pyx_L1_error)

  /* "src/python/grpcio/grpc/_cython/_cygrpc/channel.pyx.pxi":476
 * 
 *   def check_connectivity_state(self, bint try_to_connect):
 *     with self._state.condition:             # <<<<<<<<<<<<<<
 *       if self._state.open:
 *         return grpc_channel_check_connectivity_state(
 */
  __pyx_tuple__46 = PyTuple_Pack(3, Py_None, Py_None, Py_None); if (unlikely(!__pyx_tuple__46)) __PYX_ERR(8, 476, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_tuple__46);
  __Pyx_GIVEREF(__pyx_tuple__46);
  __pyx_tuple__47 = PyTuple_Pack(3, Py_None, Py_None, Py_None); if (unlikely(!__pyx_tuple__47)) __PYX_ERR(8, 476, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_tuple__47);
  __Pyx_GIVEREF(__pyx_tuple__47);

  /* "(tree fragment)":2
 * def __reduce_cython__(self):
 *     raise TypeError("no default __reduce__ due to non-trivial __cinit__")             # <<<<<<<<<<<<<<
 * def __setstate_cython__(self, __pyx_state):
 *     raise TypeError("no default __reduce__ due to non-trivial __cinit__")
 */
  __pyx_tuple__48 = PyTuple_Pack(1, __pyx_kp_s_no_default___reduce___due_to_non); if (unlikely(!__pyx_tuple__48)) __PYX_ERR(7, 2, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_tuple__48);
  __Pyx_GIVEREF(__pyx_tuple__48);

  /* "(tree fragment)":4
 *     raise TypeError("no default __reduce__ due to non-trivial __cinit__")
 * def __setstate_cython__(self, __pyx_state):
 *     raise TypeError("no default __reduce__ due to non-trivial __cinit__")             # <<<<<<<<<<<<<<
 */
  __pyx_tuple__49 = PyTuple_Pack(1, __pyx_kp_s_no_default___reduce___due_to_non); if (unlikely(!__pyx_tuple__49)) __PYX_ERR(7, 4, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_tuple__49);
  __Pyx_GIVEREF(__pyx_tuple__49);

  /* "(tree fragment)":2
 * def __reduce_cython__(self):
 *     raise TypeError("self.c_credentials cannot be converted to a Python object for pickling")             # <<<<<<<<<<<<<<
 * def __setstate_cython__(self, __pyx_state):
 *     raise TypeError("self.c_credentials cannot be converted to a Python object for pickling")
 */
  __pyx_tuple__50 = PyTuple_Pack(1, __pyx_kp_s_self_c_credentials_cannot_be_con); if (unlikely(!__pyx_tuple__50)) __PYX_ERR(7, 2, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_tuple__50);
  __Pyx_GIVEREF(__pyx_tuple__50);

  /* "(tree fragment)":4
 *     raise TypeError("self.c_credentials cannot be converted to a Python object for pickling")
 * def __setstate_cython__(self, __pyx_state):
 *     raise TypeError("self.c_credentials cannot be converted to a Python object for pickling")             # <<<<<<<<<<<<<<
 */
  __pyx_tuple__51 = PyTuple_Pack(1, __pyx_kp_s_self_c_credentials_cannot_be_con); if (unlikely(!__pyx_tuple__51)) __PYX_ERR(7, 4, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_tuple__51);
  __Pyx_GIVEREF(__pyx_tuple__51);

  /* "src/python/grpcio/grpc/_cython/_cygrpc/credentials.pyx.pxi":48
 *   cdef size_t metadata_count
 *   cdef grpc_metadata *c_metadata
 *   def callback(metadata, grpc_status_code status, bytes error_details):             # <<<<<<<<<<<<<<
 *     if status == StatusCode.ok:
 *       _store_c_metadata(metadata, &c_metadata, &metadata_count)
 */
  __pyx_tuple__52 = PyTuple_Pack(3, __pyx_n_s_metadata, __pyx_n_s_status, __pyx_n_s_error_details); if (unlikely(!__pyx_tuple__52)) __PYX_ERR(4, 48, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_tuple__52);
  __Pyx_GIVEREF(__pyx_tuple__52);
  __pyx_codeobj__53 = (PyObject*)__Pyx_PyCode_New(3, 0, 3, 0, CO_OPTIMIZED|CO_NEWLOCALS, __pyx_empty_bytes, __pyx_empty_tuple, __pyx_empty_tuple, __pyx_tuple__52, __pyx_empty_tuple, __pyx_empty_tuple, __pyx_kp_s_src_python_grpcio_grpc__cython_2, __pyx_n_s_callback, 48, __pyx_empty_bytes); if (unlikely(!__pyx_codeobj__53)) __PYX_ERR(4, 48, __pyx_L1_error)

  /* "(tree fragment)":2
 * def __reduce_cython__(self):
 *     raise TypeError("no default __reduce__ due to non-trivial __cinit__")             # <<<<<<<<<<<<<<
 * def __setstate_cython__(self, __pyx_state):
 *     raise TypeError("no default __reduce__ due to non-trivial __cinit__")
 */
  __pyx_tuple__54 = PyTuple_Pack(1, __pyx_kp_s_no_default___reduce___due_to_non); if (unlikely(!__pyx_tuple__54)) __PYX_ERR(7, 2, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_tuple__54);
  __Pyx_GIVEREF(__pyx_tuple__54);

  /* "(tree fragment)":4
 *     raise TypeError("no default __reduce__ due to non-trivial __cinit__")
 * def __setstate_cython__(self, __pyx_state):
 *     raise TypeError("no default __reduce__ due to non-trivial __cinit__")             # <<<<<<<<<<<<<<
 */
  __pyx_tuple__55 = PyTuple_Pack(1, __pyx_kp_s_no_default___reduce___due_to_non); if (unlikely(!__pyx_tuple__55)) __PYX_ERR(7, 4, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_tuple__55);
  __Pyx_GIVEREF(__pyx_tuple__55);

  /* "(tree fragment)":2
 * def __reduce_cython__(self):
 *     raise TypeError("no default __reduce__ due to non-trivial __cinit__")             # <<<<<<<<<<<<<<
 * def __setstate_cython__(self, __pyx_state):
 *     raise TypeError("no default __reduce__ due to non-trivial __cinit__")
 */
  __pyx_tuple__56 = PyTuple_Pack(1, __pyx_kp_s_no_default___reduce___due_to_non); if (unlikely(!__pyx_tuple__56)) __PYX_ERR(7, 2, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_tuple__56);
  __Pyx_GIVEREF(__pyx_tuple__56);

  /* "(tree fragment)":4
 *     raise TypeError("no default __reduce__ due to non-trivial __cinit__")
 * def __setstate_cython__(self, __pyx_state):
 *     raise TypeError("no default __reduce__ due to non-trivial __cinit__")             # <<<<<<<<<<<<<<
 */
  __pyx_tuple__57 = PyTuple_Pack(1, __pyx_kp_s_no_default___reduce___due_to_non); if (unlikely(!__pyx_tuple__57)) __PYX_ERR(7, 4, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_tuple__57);
  __Pyx_GIVEREF(__pyx_tuple__57);

  /* "(tree fragment)":2
 * def __reduce_cython__(self):
 *     raise TypeError("no default __reduce__ due to non-trivial __cinit__")             # <<<<<<<<<<<<<<
 * def __setstate_cython__(self, __pyx_state):
 *     raise TypeError("no default __reduce__ due to non-trivial __cinit__")
 */
  __pyx_tuple__58 = PyTuple_Pack(1, __pyx_kp_s_no_default___reduce___due_to_non); if (unlikely(!__pyx_tuple__58)) __PYX_ERR(7, 2, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_tuple__58);
  __Pyx_GIVEREF(__pyx_tuple__58);

  /* "(tree fragment)":4
 *     raise TypeError("no default __reduce__ due to non-trivial __cinit__")
 * def __setstate_cython__(self, __pyx_state):
 *     raise TypeError("no default __reduce__ due to non-trivial __cinit__")             # <<<<<<<<<<<<<<
 */
  __pyx_tuple__59 = PyTuple_Pack(1, __pyx_kp_s_no_default___reduce___due_to_non); if (unlikely(!__pyx_tuple__59)) __PYX_ERR(7, 4, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_tuple__59);
  __Pyx_GIVEREF(__pyx_tuple__59);

  /* "(tree fragment)":2
 * def __reduce_cython__(self):
 *     raise TypeError("no default __reduce__ due to non-trivial __cinit__")             # <<<<<<<<<<<<<<
 * def __setstate_cython__(self, __pyx_state):
 *     raise TypeError("no default __reduce__ due to non-trivial __cinit__")
 */
  __pyx_tuple__60 = PyTuple_Pack(1, __pyx_kp_s_no_default___reduce___due_to_non); if (unlikely(!__pyx_tuple__60)) __PYX_ERR(7, 2, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_tuple__60);
  __Pyx_GIVEREF(__pyx_tuple__60);

  /* "(tree fragment)":4
 *     raise TypeError("no default __reduce__ due to non-trivial __cinit__")
 * def __setstate_cython__(self, __pyx_state):
 *     raise TypeError("no default __reduce__ due to non-trivial __cinit__")             # <<<<<<<<<<<<<<
 */
  __pyx_tuple__61 = PyTuple_Pack(1, __pyx_kp_s_no_default___reduce___due_to_non); if (unlikely(!__pyx_tuple__61)) __PYX_ERR(7, 4, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_tuple__61);
  __Pyx_GIVEREF(__pyx_tuple__61);

  /* "(tree fragment)":2
 * def __reduce_cython__(self):
 *     raise TypeError("no default __reduce__ due to non-trivial __cinit__")             # <<<<<<<<<<<<<<
 * def __setstate_cython__(self, __pyx_state):
 *     raise TypeError("no default __reduce__ due to non-trivial __cinit__")
 */
  __pyx_tuple__62 = PyTuple_Pack(1, __pyx_kp_s_no_default___reduce___due_to_non); if (unlikely(!__pyx_tuple__62)) __PYX_ERR(7, 2, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_tuple__62);
  __Pyx_GIVEREF(__pyx_tuple__62);

  /* "(tree fragment)":4
 *     raise TypeError("no default __reduce__ due to non-trivial __cinit__")
 * def __setstate_cython__(self, __pyx_state):
 *     raise TypeError("no default __reduce__ due to non-trivial __cinit__")             # <<<<<<<<<<<<<<
 */
  __pyx_tuple__63 = PyTuple_Pack(1, __pyx_kp_s_no_default___reduce___due_to_non); if (unlikely(!__pyx_tuple__63)) __PYX_ERR(7, 4, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_tuple__63);
  __Pyx_GIVEREF(__pyx_tuple__63);

  /* "(tree fragment)":2
 * def __reduce_cython__(self):
 *     raise TypeError("no default __reduce__ due to non-trivial __cinit__")             # <<<<<<<<<<<<<<
 * def __setstate_cython__(self, __pyx_state):
 *     raise TypeError("no default __reduce__ due to non-trivial __cinit__")
 */
  __pyx_tuple__64 = PyTuple_Pack(1, __pyx_kp_s_no_default___reduce___due_to_non); if (unlikely(!__pyx_tuple__64)) __PYX_ERR(7, 2, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_tuple__64);
  __Pyx_GIVEREF(__pyx_tuple__64);

  /* "(tree fragment)":4
 *     raise TypeError("no default __reduce__ due to non-trivial __cinit__")
 * def __setstate_cython__(self, __pyx_state):
 *     raise TypeError("no default __reduce__ due to non-trivial __cinit__")             # <<<<<<<<<<<<<<
 */
  __pyx_tuple__65 = PyTuple_Pack(1, __pyx_kp_s_no_default___reduce___due_to_non); if (unlikely(!__pyx_tuple__65)) __PYX_ERR(7, 4, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_tuple__65);
  __Pyx_GIVEREF(__pyx_tuple__65);

  /* "(tree fragment)":2
 * def __reduce_cython__(self):
 *     raise TypeError("no default __reduce__ due to non-trivial __cinit__")             # <<<<<<<<<<<<<<
 * def __setstate_cython__(self, __pyx_state):
 *     raise TypeError("no default __reduce__ due to non-trivial __cinit__")
 */
  __pyx_tuple__66 = PyTuple_Pack(1, __pyx_kp_s_no_default___reduce___due_to_non); if (unlikely(!__pyx_tuple__66)) __PYX_ERR(7, 2, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_tuple__66);
  __Pyx_GIVEREF(__pyx_tuple__66);

  /* "(tree fragment)":4
 *     raise TypeError("no default __reduce__ due to non-trivial __cinit__")
 * def __setstate_cython__(self, __pyx_state):
 *     raise TypeError("no default __reduce__ due to non-trivial __cinit__")             # <<<<<<<<<<<<<<
 */
  __pyx_tuple__67 = PyTuple_Pack(1, __pyx_kp_s_no_default___reduce___due_to_non); if (unlikely(!__pyx_tuple__67)) __PYX_ERR(7, 4, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_tuple__67);
  __Pyx_GIVEREF(__pyx_tuple__67);

  /* "src/python/grpcio/grpc/_cython/_cygrpc/credentials.pyx.pxi":220
 *   for pair in pem_key_cert_pairs:
 *     if not isinstance(pair, SslPemKeyCertPair):
 *       raise TypeError("expected pem_key_cert_pairs to be sequence of "             # <<<<<<<<<<<<<<
 *                       "SslPemKeyCertPair")
 *   cdef size_t c_ssl_pem_key_cert_pairs_count = len(pem_key_cert_pairs)
 */
  __pyx_tuple__68 = PyTuple_Pack(1, __pyx_kp_s_expected_pem_key_cert_pairs_to_b); if (unlikely(!__pyx_tuple__68)) __PYX_ERR(4, 220, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_tuple__68);
  __Pyx_GIVEREF(__pyx_tuple__68);

  /* "src/python/grpcio/grpc/_cython/_cygrpc/credentials.pyx.pxi":276
 *                                                bint force_client_auth):
 *   if not isinstance(initial_cert_config, grpc.ServerCertificateConfiguration):
 *     raise TypeError(             # <<<<<<<<<<<<<<
 *         'initial_cert_config must be a grpc.ServerCertificateConfiguration')
 *   if not callable(cert_config_fetcher):
 */
  __pyx_tuple__69 = PyTuple_Pack(1, __pyx_kp_s_initial_cert_config_must_be_a_gr); if (unlikely(!__pyx_tuple__69)) __PYX_ERR(4, 276, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_tuple__69);
  __Pyx_GIVEREF(__pyx_tuple__69);

  /* "src/python/grpcio/grpc/_cython/_cygrpc/credentials.pyx.pxi":279
 *         'initial_cert_config must be a grpc.ServerCertificateConfiguration')
 *   if not callable(cert_config_fetcher):
 *     raise TypeError('cert_config_fetcher must be callable')             # <<<<<<<<<<<<<<
 *   cdef ServerCredentials credentials = ServerCredentials()
 *   credentials.initial_cert_config = initial_cert_config
 */
  __pyx_tuple__70 = PyTuple_Pack(1, __pyx_kp_s_cert_config_fetcher_must_be_call); if (unlikely(!__pyx_tuple__70)) __PYX_ERR(4, 279, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_tuple__70);
  __Pyx_GIVEREF(__pyx_tuple__70);

  /* "src/python/grpcio/grpc/_cython/_cygrpc/credentials.pyx.pxi":299
 *   cdef ServerCertificateConfig cert_config = None
 *   if not user_data:
 *     raise ValueError('internal error: user_data must be specified')             # <<<<<<<<<<<<<<
 *   credentials = <ServerCredentials>user_data
 *   if not credentials.initial_cert_config_fetched:
 */
  __pyx_tuple__71 = PyTuple_Pack(1, __pyx_kp_s_internal_error_user_data_must_be); if (unlikely(!__pyx_tuple__71)) __PYX_ERR(4, 299, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_tuple__71);
  __Pyx_GIVEREF(__pyx_tuple__71);

  /* "src/python/grpcio/grpc/_cython/_cygrpc/credentials.pyx.pxi":310
 *       cert_config_wrapper = user_cb()
 *     except Exception:
 *       _LOGGER.exception('Error fetching certificate config')             # <<<<<<<<<<<<<<
 *       return GRPC_SSL_CERTIFICATE_CONFIG_RELOAD_FAIL
 *     if cert_config_wrapper is None:
 */
  __pyx_tuple__72 = PyTuple_Pack(1, __pyx_kp_s_Error_fetching_certificate_confi); if (unlikely(!__pyx_tuple__72)) __PYX_ERR(4, 310, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_tuple__72);
  __Pyx_GIVEREF(__pyx_tuple__72);

  /* "src/python/grpcio/grpc/_cython/_cygrpc/completion_queue.pyx.pxi":105
 *   def clear(self):
 *     if not self.is_shutting_down:
 *       raise ValueError('queue must be shutting down to be cleared')             # <<<<<<<<<<<<<<
 *     while self.poll().type != GRPC_QUEUE_SHUTDOWN:
 *       pass
 */
  __pyx_tuple__73 = PyTuple_Pack(1, __pyx_kp_s_queue_must_be_shutting_down_to_b); if (unlikely(!__pyx_tuple__73)) __PYX_ERR(10, 105, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_tuple__73);
  __Pyx_GIVEREF(__pyx_tuple__73);

  /* "(tree fragment)":2
 * def __reduce_cython__(self):
 *     raise TypeError("no default __reduce__ due to non-trivial __cinit__")             # <<<<<<<<<<<<<<
 * def __setstate_cython__(self, __pyx_state):
 *     raise TypeError("no default __reduce__ due to non-trivial __cinit__")
 */
  __pyx_tuple__74 = PyTuple_Pack(1, __pyx_kp_s_no_default___reduce___due_to_non); if (unlikely(!__pyx_tuple__74)) __PYX_ERR(7, 2, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_tuple__74);
  __Pyx_GIVEREF(__pyx_tuple__74);

  /* "(tree fragment)":4
 *     raise TypeError("no default __reduce__ due to non-trivial __cinit__")
 * def __setstate_cython__(self, __pyx_state):
 *     raise TypeError("no default __reduce__ due to non-trivial __cinit__")             # <<<<<<<<<<<<<<
 */
  __pyx_tuple__75 = PyTuple_Pack(1, __pyx_kp_s_no_default___reduce___due_to_non); if (unlikely(!__pyx_tuple__75)) __PYX_ERR(7, 4, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_tuple__75);
  __Pyx_GIVEREF(__pyx_tuple__75);

  /* "(tree fragment)":2
 * def __reduce_cython__(self):
 *     raise TypeError("no default __reduce__ due to non-trivial __cinit__")             # <<<<<<<<<<<<<<
 * def __setstate_cython__(self, __pyx_state):
 *     raise TypeError("no default __reduce__ due to non-trivial __cinit__")
 */
  __pyx_tuple__76 = PyTuple_Pack(1, __pyx_kp_s_no_default___reduce___due_to_non); if (unlikely(!__pyx_tuple__76)) __PYX_ERR(7, 2, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_tuple__76);
  __Pyx_GIVEREF(__pyx_tuple__76);

  /* "(tree fragment)":4
 *     raise TypeError("no default __reduce__ due to non-trivial __cinit__")
 * def __setstate_cython__(self, __pyx_state):
 *     raise TypeError("no default __reduce__ due to non-trivial __cinit__")             # <<<<<<<<<<<<<<
 */
  __pyx_tuple__77 = PyTuple_Pack(1, __pyx_kp_s_no_default___reduce___due_to_non); if (unlikely(!__pyx_tuple__77)) __PYX_ERR(7, 4, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_tuple__77);
  __Pyx_GIVEREF(__pyx_tuple__77);

  /* "(tree fragment)":2
 * def __reduce_cython__(self):
 *     raise TypeError("no default __reduce__ due to non-trivial __cinit__")             # <<<<<<<<<<<<<<
 * def __setstate_cython__(self, __pyx_state):
 *     raise TypeError("no default __reduce__ due to non-trivial __cinit__")
 */
  __pyx_tuple__78 = PyTuple_Pack(1, __pyx_kp_s_no_default___reduce___due_to_non); if (unlikely(!__pyx_tuple__78)) __PYX_ERR(7, 2, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_tuple__78);
  __Pyx_GIVEREF(__pyx_tuple__78);

  /* "(tree fragment)":4
 *     raise TypeError("no default __reduce__ due to non-trivial __cinit__")
 * def __setstate_cython__(self, __pyx_state):
 *     raise TypeError("no default __reduce__ due to non-trivial __cinit__")             # <<<<<<<<<<<<<<
 */
  __pyx_tuple__79 = PyTuple_Pack(1, __pyx_kp_s_no_default___reduce___due_to_non); if (unlikely(!__pyx_tuple__79)) __PYX_ERR(7, 4, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_tuple__79);
  __Pyx_GIVEREF(__pyx_tuple__79);

  /* "(tree fragment)":2
 * def __reduce_cython__(self):
 *     raise TypeError("no default __reduce__ due to non-trivial __cinit__")             # <<<<<<<<<<<<<<
 * def __setstate_cython__(self, __pyx_state):
 *     raise TypeError("no default __reduce__ due to non-trivial __cinit__")
 */
  __pyx_tuple__80 = PyTuple_Pack(1, __pyx_kp_s_no_default___reduce___due_to_non); if (unlikely(!__pyx_tuple__80)) __PYX_ERR(7, 2, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_tuple__80);
  __Pyx_GIVEREF(__pyx_tuple__80);

  /* "(tree fragment)":4
 *     raise TypeError("no default __reduce__ due to non-trivial __cinit__")
 * def __setstate_cython__(self, __pyx_state):
 *     raise TypeError("no default __reduce__ due to non-trivial __cinit__")             # <<<<<<<<<<<<<<
 */
  __pyx_tuple__81 = PyTuple_Pack(1, __pyx_kp_s_no_default___reduce___due_to_non); if (unlikely(!__pyx_tuple__81)) __PYX_ERR(7, 4, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_tuple__81);
  __Pyx_GIVEREF(__pyx_tuple__81);

  /* "(tree fragment)":2
 * def __reduce_cython__(self):
 *     raise TypeError("no default __reduce__ due to non-trivial __cinit__")             # <<<<<<<<<<<<<<
 * def __setstate_cython__(self, __pyx_state):
 *     raise TypeError("no default __reduce__ due to non-trivial __cinit__")
 */
  __pyx_tuple__82 = PyTuple_Pack(1, __pyx_kp_s_no_default___reduce___due_to_non); if (unlikely(!__pyx_tuple__82)) __PYX_ERR(7, 2, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_tuple__82);
  __Pyx_GIVEREF(__pyx_tuple__82);

  /* "(tree fragment)":4
 *     raise TypeError("no default __reduce__ due to non-trivial __cinit__")
 * def __setstate_cython__(self, __pyx_state):
 *     raise TypeError("no default __reduce__ due to non-trivial __cinit__")             # <<<<<<<<<<<<<<
 */
  __pyx_tuple__83 = PyTuple_Pack(1, __pyx_kp_s_no_default___reduce___due_to_non); if (unlikely(!__pyx_tuple__83)) __PYX_ERR(7, 4, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_tuple__83);
  __Pyx_GIVEREF(__pyx_tuple__83);

  /* "(tree fragment)":2
 * def __reduce_cython__(self):
 *     raise TypeError("self.c_op cannot be converted to a Python object for pickling")             # <<<<<<<<<<<<<<
 * def __setstate_cython__(self, __pyx_state):
 *     raise TypeError("self.c_op cannot be converted to a Python object for pickling")
 */
  __pyx_tuple__84 = PyTuple_Pack(1, __pyx_kp_s_self_c_op_cannot_be_converted_to); if (unlikely(!__pyx_tuple__84)) __PYX_ERR(7, 2, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_tuple__84);
  __Pyx_GIVEREF(__pyx_tuple__84);

  /* "(tree fragment)":4
 *     raise TypeError("self.c_op cannot be converted to a Python object for pickling")
 * def __setstate_cython__(self, __pyx_state):
 *     raise TypeError("self.c_op cannot be converted to a Python object for pickling")             # <<<<<<<<<<<<<<
 */
  __pyx_tuple__85 = PyTuple_Pack(1, __pyx_kp_s_self_c_op_cannot_be_converted_to); if (unlikely(!__pyx_tuple__85)) __PYX_ERR(7, 4, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_tuple__85);
  __Pyx_GIVEREF(__pyx_tuple__85);

  /* "(tree fragment)":2
 * def __reduce_cython__(self):
 *     raise TypeError("no default __reduce__ due to non-trivial __cinit__")             # <<<<<<<<<<<<<<
 * def __setstate_cython__(self, __pyx_state):
 *     raise TypeError("no default __reduce__ due to non-trivial __cinit__")
 */
  __pyx_tuple__86 = PyTuple_Pack(1, __pyx_kp_s_no_default___reduce___due_to_non); if (unlikely(!__pyx_tuple__86)) __PYX_ERR(7, 2, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_tuple__86);
  __Pyx_GIVEREF(__pyx_tuple__86);

  /* "(tree fragment)":4
 *     raise TypeError("no default __reduce__ due to non-trivial __cinit__")
 * def __setstate_cython__(self, __pyx_state):
 *     raise TypeError("no default __reduce__ due to non-trivial __cinit__")             # <<<<<<<<<<<<<<
 */
  __pyx_tuple__87 = PyTuple_Pack(1, __pyx_kp_s_no_default___reduce___due_to_non); if (unlikely(!__pyx_tuple__87)) __PYX_ERR(7, 4, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_tuple__87);
  __Pyx_GIVEREF(__pyx_tuple__87);

  /* "(tree fragment)":2
 * def __reduce_cython__(self):
 *     raise TypeError("no default __reduce__ due to non-trivial __cinit__")             # <<<<<<<<<<<<<<
 * def __setstate_cython__(self, __pyx_state):
 *     raise TypeError("no default __reduce__ due to non-trivial __cinit__")
 */
  __pyx_tuple__88 = PyTuple_Pack(1, __pyx_kp_s_no_default___reduce___due_to_non); if (unlikely(!__pyx_tuple__88)) __PYX_ERR(7, 2, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_tuple__88);
  __Pyx_GIVEREF(__pyx_tuple__88);

  /* "(tree fragment)":4
 *     raise TypeError("no default __reduce__ due to non-trivial __cinit__")
 * def __setstate_cython__(self, __pyx_state):
 *     raise TypeError("no default __reduce__ due to non-trivial __cinit__")             # <<<<<<<<<<<<<<
 */
  __pyx_tuple__89 = PyTuple_Pack(1, __pyx_kp_s_no_default___reduce___due_to_non); if (unlikely(!__pyx_tuple__89)) __PYX_ERR(7, 4, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_tuple__89);
  __Pyx_GIVEREF(__pyx_tuple__89);

  /* "(tree fragment)":2
 * def __reduce_cython__(self):
 *     raise TypeError("no default __reduce__ due to non-trivial __cinit__")             # <<<<<<<<<<<<<<
 * def __setstate_cython__(self, __pyx_state):
 *     raise TypeError("no default __reduce__ due to non-trivial __cinit__")
 */
  __pyx_tuple__90 = PyTuple_Pack(1, __pyx_kp_s_no_default___reduce___due_to_non); if (unlikely(!__pyx_tuple__90)) __PYX_ERR(7, 2, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_tuple__90);
  __Pyx_GIVEREF(__pyx_tuple__90);

  /* "(tree fragment)":4
 *     raise TypeError("no default __reduce__ due to non-trivial __cinit__")
 * def __setstate_cython__(self, __pyx_state):
 *     raise TypeError("no default __reduce__ due to non-trivial __cinit__")             # <<<<<<<<<<<<<<
 */
  __pyx_tuple__91 = PyTuple_Pack(1, __pyx_kp_s_no_default___reduce___due_to_non); if (unlikely(!__pyx_tuple__91)) __PYX_ERR(7, 4, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_tuple__91);
  __Pyx_GIVEREF(__pyx_tuple__91);

  /* "(tree fragment)":2
 * def __reduce_cython__(self):
 *     raise TypeError("no default __reduce__ due to non-trivial __cinit__")             # <<<<<<<<<<<<<<
 * def __setstate_cython__(self, __pyx_state):
 *     raise TypeError("no default __reduce__ due to non-trivial __cinit__")
 */
  __pyx_tuple__92 = PyTuple_Pack(1, __pyx_kp_s_no_default___reduce___due_to_non); if (unlikely(!__pyx_tuple__92)) __PYX_ERR(7, 2, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_tuple__92);
  __Pyx_GIVEREF(__pyx_tuple__92);

  /* "(tree fragment)":4
 *     raise TypeError("no default __reduce__ due to non-trivial __cinit__")
 * def __setstate_cython__(self, __pyx_state):
 *     raise TypeError("no default __reduce__ due to non-trivial __cinit__")             # <<<<<<<<<<<<<<
 */
  __pyx_tuple__93 = PyTuple_Pack(1, __pyx_kp_s_no_default___reduce___due_to_non); if (unlikely(!__pyx_tuple__93)) __PYX_ERR(7, 4, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_tuple__93);
  __Pyx_GIVEREF(__pyx_tuple__93);

  /* "(tree fragment)":2
 * def __reduce_cython__(self):
 *     raise TypeError("no default __reduce__ due to non-trivial __cinit__")             # <<<<<<<<<<<<<<
 * def __setstate_cython__(self, __pyx_state):
 *     raise TypeError("no default __reduce__ due to non-trivial __cinit__")
 */
  __pyx_tuple__94 = PyTuple_Pack(1, __pyx_kp_s_no_default___reduce___due_to_non); if (unlikely(!__pyx_tuple__94)) __PYX_ERR(7, 2, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_tuple__94);
  __Pyx_GIVEREF(__pyx_tuple__94);

  /* "(tree fragment)":4
 *     raise TypeError("no default __reduce__ due to non-trivial __cinit__")
 * def __setstate_cython__(self, __pyx_state):
 *     raise TypeError("no default __reduce__ due to non-trivial __cinit__")             # <<<<<<<<<<<<<<
 */
  __pyx_tuple__95 = PyTuple_Pack(1, __pyx_kp_s_no_default___reduce___due_to_non); if (unlikely(!__pyx_tuple__95)) __PYX_ERR(7, 4, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_tuple__95);
  __Pyx_GIVEREF(__pyx_tuple__95);

  /* "(tree fragment)":2
 * def __reduce_cython__(self):
 *     raise TypeError("no default __reduce__ due to non-trivial __cinit__")             # <<<<<<<<<<<<<<
 * def __setstate_cython__(self, __pyx_state):
 *     raise TypeError("no default __reduce__ due to non-trivial __cinit__")
 */
  __pyx_tuple__96 = PyTuple_Pack(1, __pyx_kp_s_no_default___reduce___due_to_non); if (unlikely(!__pyx_tuple__96)) __PYX_ERR(7, 2, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_tuple__96);
  __Pyx_GIVEREF(__pyx_tuple__96);

  /* "(tree fragment)":4
 *     raise TypeError("no default __reduce__ due to non-trivial __cinit__")
 * def __setstate_cython__(self, __pyx_state):
 *     raise TypeError("no default __reduce__ due to non-trivial __cinit__")             # <<<<<<<<<<<<<<
 */
  __pyx_tuple__97 = PyTuple_Pack(1, __pyx_kp_s_no_default___reduce___due_to_non); if (unlikely(!__pyx_tuple__97)) __PYX_ERR(7, 4, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_tuple__97);
  __Pyx_GIVEREF(__pyx_tuple__97);

  /* "(tree fragment)":2
 * def __reduce_cython__(self):
 *     raise TypeError("no default __reduce__ due to non-trivial __cinit__")             # <<<<<<<<<<<<<<
 * def __setstate_cython__(self, __pyx_state):
 *     raise TypeError("no default __reduce__ due to non-trivial __cinit__")
 */
  __pyx_tuple__98 = PyTuple_Pack(1, __pyx_kp_s_no_default___reduce___due_to_non); if (unlikely(!__pyx_tuple__98)) __PYX_ERR(7, 2, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_tuple__98);
  __Pyx_GIVEREF(__pyx_tuple__98);

  /* "(tree fragment)":4
 *     raise TypeError("no default __reduce__ due to non-trivial __cinit__")
 * def __setstate_cython__(self, __pyx_state):
 *     raise TypeError("no default __reduce__ due to non-trivial __cinit__")             # <<<<<<<<<<<<<<
 */
  __pyx_tuple__99 = PyTuple_Pack(1, __pyx_kp_s_no_default___reduce___due_to_non); if (unlikely(!__pyx_tuple__99)) __PYX_ERR(7, 4, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_tuple__99);
  __Pyx_GIVEREF(__pyx_tuple__99);

  /* "(tree fragment)":2
 * def __reduce_cython__(self):
 *     raise TypeError("no default __reduce__ due to non-trivial __cinit__")             # <<<<<<<<<<<<<<
 * def __setstate_cython__(self, __pyx_state):
 *     raise TypeError("no default __reduce__ due to non-trivial __cinit__")
 */
  __pyx_tuple__100 = PyTuple_Pack(1, __pyx_kp_s_no_default___reduce___due_to_non); if (unlikely(!__pyx_tuple__100)) __PYX_ERR(7, 2, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_tuple__100);
  __Pyx_GIVEREF(__pyx_tuple__100);

  /* "(tree fragment)":4
 *     raise TypeError("no default __reduce__ due to non-trivial __cinit__")
 * def __setstate_cython__(self, __pyx_state):
 *     raise TypeError("no default __reduce__ due to non-trivial __cinit__")             # <<<<<<<<<<<<<<
 */
  __pyx_tuple__101 = PyTuple_Pack(1, __pyx_kp_s_no_default___reduce___due_to_non); if (unlikely(!__pyx_tuple__101)) __PYX_ERR(7, 4, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_tuple__101);
  __Pyx_GIVEREF(__pyx_tuple__101);

  /* "(tree fragment)":2
 * def __reduce_cython__(self):
 *     raise TypeError("no default __reduce__ due to non-trivial __cinit__")             # <<<<<<<<<<<<<<
 * def __setstate_cython__(self, __pyx_state):
 *     raise TypeError("no default __reduce__ due to non-trivial __cinit__")
 */
  __pyx_tuple__102 = PyTuple_Pack(1, __pyx_kp_s_no_default___reduce___due_to_non); if (unlikely(!__pyx_tuple__102)) __PYX_ERR(7, 2, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_tuple__102);
  __Pyx_GIVEREF(__pyx_tuple__102);

  /* "(tree fragment)":4
 *     raise TypeError("no default __reduce__ due to non-trivial __cinit__")
 * def __setstate_cython__(self, __pyx_state):
 *     raise TypeError("no default __reduce__ due to non-trivial __cinit__")             # <<<<<<<<<<<<<<
 */
  __pyx_tuple__103 = PyTuple_Pack(1, __pyx_kp_s_no_default___reduce___due_to_non); if (unlikely(!__pyx_tuple__103)) __PYX_ERR(7, 4, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_tuple__103);
  __Pyx_GIVEREF(__pyx_tuple__103);

  /* "(tree fragment)":2
 * def __reduce_cython__(self):
 *     raise TypeError("no default __reduce__ due to non-trivial __cinit__")             # <<<<<<<<<<<<<<
 * def __setstate_cython__(self, __pyx_state):
 *     raise TypeError("no default __reduce__ due to non-trivial __cinit__")
 */
  __pyx_tuple__104 = PyTuple_Pack(1, __pyx_kp_s_no_default___reduce___due_to_non); if (unlikely(!__pyx_tuple__104)) __PYX_ERR(7, 2, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_tuple__104);
  __Pyx_GIVEREF(__pyx_tuple__104);

  /* "(tree fragment)":4
 *     raise TypeError("no default __reduce__ due to non-trivial __cinit__")
 * def __setstate_cython__(self, __pyx_state):
 *     raise TypeError("no default __reduce__ due to non-trivial __cinit__")             # <<<<<<<<<<<<<<
 */
  __pyx_tuple__105 = PyTuple_Pack(1, __pyx_kp_s_no_default___reduce___due_to_non); if (unlikely(!__pyx_tuple__105)) __PYX_ERR(7, 4, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_tuple__105);
  __Pyx_GIVEREF(__pyx_tuple__105);

  /* "(tree fragment)":2
 * def __reduce_cython__(self):
 *     raise TypeError("no default __reduce__ due to non-trivial __cinit__")             # <<<<<<<<<<<<<<
 * def __setstate_cython__(self, __pyx_state):
 *     raise TypeError("no default __reduce__ due to non-trivial __cinit__")
 */
  __pyx_tuple__106 = PyTuple_Pack(1, __pyx_kp_s_no_default___reduce___due_to_non); if (unlikely(!__pyx_tuple__106)) __PYX_ERR(7, 2, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_tuple__106);
  __Pyx_GIVEREF(__pyx_tuple__106);

  /* "(tree fragment)":4
 *     raise TypeError("no default __reduce__ due to non-trivial __cinit__")
 * def __setstate_cython__(self, __pyx_state):
 *     raise TypeError("no default __reduce__ due to non-trivial __cinit__")             # <<<<<<<<<<<<<<
 */
  __pyx_tuple__107 = PyTuple_Pack(1, __pyx_kp_s_no_default___reduce___due_to_non); if (unlikely(!__pyx_tuple__107)) __PYX_ERR(7, 4, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_tuple__107);
  __Pyx_GIVEREF(__pyx_tuple__107);

  /* "src/python/grpcio/grpc/_cython/_cygrpc/security.pyx.pxi":24
 *   with gil:
 *     pkg = __name__
 *     if pkg.endswith('.cygrpc'):             # <<<<<<<<<<<<<<
 *       pkg = pkg[:-len('.cygrpc')]
 *     temporary_pem_root_certs = pkgutil.get_data(pkg, '_credentials/roots.pem')
 */
  __pyx_tuple__108 = PyTuple_Pack(1, __pyx_kp_s_cygrpc); if (unlikely(!__pyx_tuple__108)) __PYX_ERR(17, 24, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_tuple__108);
  __Pyx_GIVEREF(__pyx_tuple__108);

  /* "src/python/grpcio/grpc/_cython/_cygrpc/server.pyx.pxi":43
 *       CompletionQueue server_queue not None, tag):
 *     if not self.is_started or self.is_shutting_down:
 *       raise ValueError("server must be started and not shutting down")             # <<<<<<<<<<<<<<
 *     if server_queue not in self.registered_completion_queues:
 *       raise ValueError("server_queue must be a registered completion queue")
 */
  __pyx_tuple__109 = PyTuple_Pack(1, __pyx_kp_s_server_must_be_started_and_not_s); if (unlikely(!__pyx_tuple__109)) __PYX_ERR(5, 43, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_tuple__109);
  __Pyx_GIVEREF(__pyx_tuple__109);

  /* "src/python/grpcio/grpc/_cython/_cygrpc/server.pyx.pxi":45
 *       raise ValueError("server must be started and not shutting down")
 *     if server_queue not in self.registered_completion_queues:
 *       raise ValueError("server_queue must be a registered completion queue")             # <<<<<<<<<<<<<<
 *     cdef _RequestCallTag request_call_tag = _RequestCallTag(tag, self._vtable)
 *     request_call_tag.prepare()
 */
  __pyx_tuple__110 = PyTuple_Pack(1, __pyx_kp_s_server_queue_must_be_a_registere); if (unlikely(!__pyx_tuple__110)) __PYX_ERR(5, 45, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_tuple__110);
  __Pyx_GIVEREF(__pyx_tuple__110);

  /* "src/python/grpcio/grpc/_cython/_cygrpc/server.pyx.pxi":59
 *       self, CompletionQueue queue not None):
 *     if self.is_started:
 *       raise ValueError("cannot register completion queues after start")             # <<<<<<<<<<<<<<
 *     with nogil:
 *       grpc_server_register_completion_queue(
 */
  __pyx_tuple__111 = PyTuple_Pack(1, __pyx_kp_s_cannot_register_completion_queue); if (unlikely(!__pyx_tuple__111)) __PYX_ERR(5, 59, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_tuple__111);
  __Pyx_GIVEREF(__pyx_tuple__111);

  /* "src/python/grpcio/grpc/_cython/_cygrpc/server.pyx.pxi":67
 *   def start(self):
 *     if self.is_started:
 *       raise ValueError("the server has already started")             # <<<<<<<<<<<<<<
 *     self.backup_shutdown_queue = CompletionQueue(shutdown_cq=True)
 *     self.register_completion_queue(self.backup_shutdown_queue)
 */
  __pyx_tuple__112 = PyTuple_Pack(1, __pyx_kp_s_the_server_has_already_started); if (unlikely(!__pyx_tuple__112)) __PYX_ERR(5, 67, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_tuple__112);
  __Pyx_GIVEREF(__pyx_tuple__112);

  /* "src/python/grpcio/grpc/_cython/_cygrpc/server.pyx.pxi":104
 *   def shutdown(self, CompletionQueue queue not None, tag):
 *     if queue.is_shutting_down:
 *       raise ValueError("queue must be live")             # <<<<<<<<<<<<<<
 *     elif not self.is_started:
 *       raise ValueError("the server hasn't started yet")
 */
  __pyx_tuple__113 = PyTuple_Pack(1, __pyx_kp_s_queue_must_be_live); if (unlikely(!__pyx_tuple__113)) __PYX_ERR(5, 104, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_tuple__113);
  __Pyx_GIVEREF(__pyx_tuple__113);

  /* "src/python/grpcio/grpc/_cython/_cygrpc/server.pyx.pxi":106
 *       raise ValueError("queue must be live")
 *     elif not self.is_started:
 *       raise ValueError("the server hasn't started yet")             # <<<<<<<<<<<<<<
 *     elif self.is_shutting_down:
 *       return
 */
  __pyx_tuple__114 = PyTuple_Pack(1, __pyx_kp_s_the_server_hasn_t_started_yet); if (unlikely(!__pyx_tuple__114)) __PYX_ERR(5, 106, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_tuple__114);
  __Pyx_GIVEREF(__pyx_tuple__114);

  /* "src/python/grpcio/grpc/_cython/_cygrpc/server.pyx.pxi":110
 *       return
 *     elif queue not in self.registered_completion_queues:
 *       raise ValueError("expected registered completion queue")             # <<<<<<<<<<<<<<
 *     else:
 *       self._c_shutdown(queue, tag)
 */
  __pyx_tuple__115 = PyTuple_Pack(1, __pyx_kp_s_expected_registered_completion_q); if (unlikely(!__pyx_tuple__115)) __PYX_ERR(5, 110, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_tuple__115);
  __Pyx_GIVEREF(__pyx_tuple__115);

  /* "src/python/grpcio/grpc/_cython/_cygrpc/server.pyx.pxi":121
 *   def cancel_all_calls(self):
 *     if not self.is_shutting_down:
 *       raise RuntimeError("the server must be shutting down to cancel all calls")             # <<<<<<<<<<<<<<
 *     elif self.is_shutdown:
 *       return
 */
  __pyx_tuple__116 = PyTuple_Pack(1, __pyx_kp_s_the_server_must_be_shutting_down); if (unlikely(!__pyx_tuple__116)) __PYX_ERR(5, 121, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_tuple__116);
  __Pyx_GIVEREF(__pyx_tuple__116);

  /* "src/python/grpcio/grpc/_cython/_cygrpc/server.pyx.pxi":147
 *         # much but repeatedly release the GIL and wait
 *         while not self.is_shutdown:
 *           time.sleep(0)             # <<<<<<<<<<<<<<
 *       grpc_server_destroy(self.c_server)
 *       self.c_server = NULL
 */
  __pyx_tuple__117 = PyTuple_Pack(1, __pyx_int_0); if (unlikely(!__pyx_tuple__117)) __PYX_ERR(5, 147, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_tuple__117);
  __Pyx_GIVEREF(__pyx_tuple__117);

  /* "(tree fragment)":2
 * def __reduce_cython__(self):
 *     raise TypeError("no default __reduce__ due to non-trivial __cinit__")             # <<<<<<<<<<<<<<
 * def __setstate_cython__(self, __pyx_state):
 *     raise TypeError("no default __reduce__ due to non-trivial __cinit__")
 */
  __pyx_tuple__118 = PyTuple_Pack(1, __pyx_kp_s_no_default___reduce___due_to_non); if (unlikely(!__pyx_tuple__118)) __PYX_ERR(7, 2, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_tuple__118);
  __Pyx_GIVEREF(__pyx_tuple__118);

  /* "(tree fragment)":4
 *     raise TypeError("no default __reduce__ due to non-trivial __cinit__")
 * def __setstate_cython__(self, __pyx_state):
 *     raise TypeError("no default __reduce__ due to non-trivial __cinit__")             # <<<<<<<<<<<<<<
 */
  __pyx_tuple__119 = PyTuple_Pack(1, __pyx_kp_s_no_default___reduce___due_to_non); if (unlikely(!__pyx_tuple__119)) __PYX_ERR(7, 4, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_tuple__119);
  __Pyx_GIVEREF(__pyx_tuple__119);

  /* "(tree fragment)":2
 * def __reduce_cython__(self):
 *     raise TypeError("no default __reduce__ due to non-trivial __cinit__")             # <<<<<<<<<<<<<<
 * def __setstate_cython__(self, __pyx_state):
 *     raise TypeError("no default __reduce__ due to non-trivial __cinit__")
 */
  __pyx_tuple__120 = PyTuple_Pack(1, __pyx_kp_s_no_default___reduce___due_to_non); if (unlikely(!__pyx_tuple__120)) __PYX_ERR(7, 2, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_tuple__120);
  __Pyx_GIVEREF(__pyx_tuple__120);

  /* "(tree fragment)":4
 *     raise TypeError("no default __reduce__ due to non-trivial __cinit__")
 * def __setstate_cython__(self, __pyx_state):
 *     raise TypeError("no default __reduce__ due to non-trivial __cinit__")             # <<<<<<<<<<<<<<
 */
  __pyx_tuple__121 = PyTuple_Pack(1, __pyx_kp_s_no_default___reduce___due_to_non); if (unlikely(!__pyx_tuple__121)) __PYX_ERR(7, 4, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_tuple__121);
  __Pyx_GIVEREF(__pyx_tuple__121);

  /* "(tree fragment)":2
 * def __reduce_cython__(self):
 *     raise TypeError("no default __reduce__ due to non-trivial __cinit__")             # <<<<<<<<<<<<<<
 * def __setstate_cython__(self, __pyx_state):
 *     raise TypeError("no default __reduce__ due to non-trivial __cinit__")
 */
  __pyx_tuple__122 = PyTuple_Pack(1, __pyx_kp_s_no_default___reduce___due_to_non); if (unlikely(!__pyx_tuple__122)) __PYX_ERR(7, 2, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_tuple__122);
  __Pyx_GIVEREF(__pyx_tuple__122);

  /* "(tree fragment)":4
 *     raise TypeError("no default __reduce__ due to non-trivial __cinit__")
 * def __setstate_cython__(self, __pyx_state):
 *     raise TypeError("no default __reduce__ due to non-trivial __cinit__")             # <<<<<<<<<<<<<<
 */
  __pyx_tuple__123 = PyTuple_Pack(1, __pyx_kp_s_no_default___reduce___due_to_non); if (unlikely(!__pyx_tuple__123)) __PYX_ERR(7, 4, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_tuple__123);
  __Pyx_GIVEREF(__pyx_tuple__123);

  /* "(tree fragment)":2
 * def __reduce_cython__(self):
 *     raise TypeError("no default __reduce__ due to non-trivial __cinit__")             # <<<<<<<<<<<<<<
 * def __setstate_cython__(self, __pyx_state):
 *     raise TypeError("no default __reduce__ due to non-trivial __cinit__")
 */
  __pyx_tuple__124 = PyTuple_Pack(1, __pyx_kp_s_no_default___reduce___due_to_non); if (unlikely(!__pyx_tuple__124)) __PYX_ERR(7, 2, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_tuple__124);
  __Pyx_GIVEREF(__pyx_tuple__124);

  /* "(tree fragment)":4
 *     raise TypeError("no default __reduce__ due to non-trivial __cinit__")
 * def __setstate_cython__(self, __pyx_state):
 *     raise TypeError("no default __reduce__ due to non-trivial __cinit__")             # <<<<<<<<<<<<<<
 */
  __pyx_tuple__125 = PyTuple_Pack(1, __pyx_kp_s_no_default___reduce___due_to_non); if (unlikely(!__pyx_tuple__125)) __PYX_ERR(7, 4, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_tuple__125);
  __Pyx_GIVEREF(__pyx_tuple__125);

  /* "(tree fragment)":2
 * def __reduce_cython__(self):
 *     raise TypeError("no default __reduce__ due to non-trivial __cinit__")             # <<<<<<<<<<<<<<
 * def __setstate_cython__(self, __pyx_state):
 *     raise TypeError("no default __reduce__ due to non-trivial __cinit__")
 */
  __pyx_tuple__126 = PyTuple_Pack(1, __pyx_kp_s_no_default___reduce___due_to_non); if (unlikely(!__pyx_tuple__126)) __PYX_ERR(7, 2, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_tuple__126);
  __Pyx_GIVEREF(__pyx_tuple__126);

  /* "(tree fragment)":4
 *     raise TypeError("no default __reduce__ due to non-trivial __cinit__")
 * def __setstate_cython__(self, __pyx_state):
 *     raise TypeError("no default __reduce__ due to non-trivial __cinit__")             # <<<<<<<<<<<<<<
 */
  __pyx_tuple__127 = PyTuple_Pack(1, __pyx_kp_s_no_default___reduce___due_to_non); if (unlikely(!__pyx_tuple__127)) __PYX_ERR(7, 4, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_tuple__127);
  __Pyx_GIVEREF(__pyx_tuple__127);

  /* "(tree fragment)":2
 * def __reduce_cython__(self):
 *     raise TypeError("no default __reduce__ due to non-trivial __cinit__")             # <<<<<<<<<<<<<<
 * def __setstate_cython__(self, __pyx_state):
 *     raise TypeError("no default __reduce__ due to non-trivial __cinit__")
 */
  __pyx_tuple__128 = PyTuple_Pack(1, __pyx_kp_s_no_default___reduce___due_to_non); if (unlikely(!__pyx_tuple__128)) __PYX_ERR(7, 2, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_tuple__128);
  __Pyx_GIVEREF(__pyx_tuple__128);

  /* "(tree fragment)":4
 *     raise TypeError("no default __reduce__ due to non-trivial __cinit__")
 * def __setstate_cython__(self, __pyx_state):
 *     raise TypeError("no default __reduce__ due to non-trivial __cinit__")             # <<<<<<<<<<<<<<
 */
  __pyx_tuple__129 = PyTuple_Pack(1, __pyx_kp_s_no_default___reduce___due_to_non); if (unlikely(!__pyx_tuple__129)) __PYX_ERR(7, 4, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_tuple__129);
  __Pyx_GIVEREF(__pyx_tuple__129);

  /* "src/python/grpcio/grpc/_cython/_cygrpc/_hooks.pyx.pxi":17
 * 
 * cdef object _custom_op_on_c_call(int op, grpc_call *call):
 *   raise NotImplementedError("No custom hooks are implemented")             # <<<<<<<<<<<<<<
 * 
 * def install_context_from_request_call_event(RequestCallEvent event):
 */
  __pyx_tuple__130 = PyTuple_Pack(1, __pyx_kp_s_No_custom_hooks_are_implemented); if (unlikely(!__pyx_tuple__130)) __PYX_ERR(20, 17, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_tuple__130);
  __Pyx_GIVEREF(__pyx_tuple__130);

  /* "src/python/grpcio/grpc/_cython/_cygrpc/grpc_gevent.pyx.pxi":42
 *   if byte_str.endswith(':' + str(port)):
 *     byte_str = byte_str[:(0 - len(str(port)) - 1)]
 *   byte_str = byte_str.lstrip('[')             # <<<<<<<<<<<<<<
 *   byte_str = byte_str.rstrip(']')
 *   byte_str = '{}'.format(byte_str)
 */
  __pyx_tuple__133 = PyTuple_Pack(1, __pyx_kp_s__132); if (unlikely(!__pyx_tuple__133)) __PYX_ERR(6, 42, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_tuple__133);
  __Pyx_GIVEREF(__pyx_tuple__133);

  /* "src/python/grpcio/grpc/_cython/_cygrpc/grpc_gevent.pyx.pxi":43
 *     byte_str = byte_str[:(0 - len(str(port)) - 1)]
 *   byte_str = byte_str.lstrip('[')
 *   byte_str = byte_str.rstrip(']')             # <<<<<<<<<<<<<<
 *   byte_str = '{}'.format(byte_str)
 *   return byte_str, port
 */
  __pyx_tuple__135 = PyTuple_Pack(1, __pyx_kp_s__134); if (unlikely(!__pyx_tuple__135)) __PYX_ERR(6, 43, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_tuple__135);
  __Pyx_GIVEREF(__pyx_tuple__135);

  /* "(tree fragment)":2
 * def __reduce_cython__(self):
 *     raise TypeError("no default __reduce__ due to non-trivial __cinit__")             # <<<<<<<<<<<<<<
 * def __setstate_cython__(self, __pyx_state):
 *     raise TypeError("no default __reduce__ due to non-trivial __cinit__")
 */
  __pyx_tuple__137 = PyTuple_Pack(1, __pyx_kp_s_no_default___reduce___due_to_non); if (unlikely(!__pyx_tuple__137)) __PYX_ERR(7, 2, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_tuple__137);
  __Pyx_GIVEREF(__pyx_tuple__137);

  /* "(tree fragment)":4
 *     raise TypeError("no default __reduce__ due to non-trivial __cinit__")
 * def __setstate_cython__(self, __pyx_state):
 *     raise TypeError("no default __reduce__ due to non-trivial __cinit__")             # <<<<<<<<<<<<<<
 */
  __pyx_tuple__138 = PyTuple_Pack(1, __pyx_kp_s_no_default___reduce___due_to_non); if (unlikely(!__pyx_tuple__138)) __PYX_ERR(7, 4, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_tuple__138);
  __Pyx_GIVEREF(__pyx_tuple__138);

  /* "src/python/grpcio/grpc/_cython/_cygrpc/grpc_gevent.pyx.pxi":167
 *         else:
 *           sent_byte_count -= len(write_bytes[0])
 *           write_bytes = write_bytes[1:]             # <<<<<<<<<<<<<<
 *     socket_wrapper.write_cb(<grpc_custom_socket*>socket_wrapper.c_socket,
 *                             grpc_error_none())
 */
  __pyx_slice__139 = PySlice_New(__pyx_int_1, Py_None, Py_None); if (unlikely(!__pyx_slice__139)) __PYX_ERR(6, 167, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_slice__139);
  __Pyx_GIVEREF(__pyx_slice__139);

  /* "src/python/grpcio/grpc/_cython/_cygrpc/grpc_gevent.pyx.pxi":233
 *   cdef grpc_resolved_address c_addr
 *   if (<SocketWrapper>socket.impl).socket is None:
 *     peer = ('0.0.0.0', 0)             # <<<<<<<<<<<<<<
 *   else:
 *     peer = (<SocketWrapper>socket.impl).socket.getsockname()
 */
  __pyx_tuple__140 = PyTuple_Pack(2, __pyx_kp_s_0_0_0_0, __pyx_int_0); if (unlikely(!__pyx_tuple__140)) __PYX_ERR(6, 233, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_tuple__140);
  __Pyx_GIVEREF(__pyx_tuple__140);

  /* "src/python/grpcio/grpc/_cython/_cygrpc/grpc_gevent.pyx.pxi":266
 * 
 * cdef grpc_error* socket_listen(grpc_custom_socket* socket) except * with gil:
 *   (<SocketWrapper>socket.impl).socket.listen(50)             # <<<<<<<<<<<<<<
 *   return grpc_error_none()
 * 
 */
  __pyx_tuple__141 = PyTuple_Pack(1, __pyx_int_50); if (unlikely(!__pyx_tuple__141)) __PYX_ERR(6, 266, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_tuple__141);
  __Pyx_GIVEREF(__pyx_tuple__141);

  /* "(tree fragment)":2
 * def __reduce_cython__(self):
 *     raise TypeError("no default __reduce__ due to non-trivial __cinit__")             # <<<<<<<<<<<<<<
 * def __setstate_cython__(self, __pyx_state):
 *     raise TypeError("no default __reduce__ due to non-trivial __cinit__")
 */
  __pyx_tuple__142 = PyTuple_Pack(1, __pyx_kp_s_no_default___reduce___due_to_non); if (unlikely(!__pyx_tuple__142)) __PYX_ERR(7, 2, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_tuple__142);
  __Pyx_GIVEREF(__pyx_tuple__142);

  /* "(tree fragment)":4
 *     raise TypeError("no default __reduce__ due to non-trivial __cinit__")
 * def __setstate_cython__(self, __pyx_state):
 *     raise TypeError("no default __reduce__ due to non-trivial __cinit__")             # <<<<<<<<<<<<<<
 */
  __pyx_tuple__143 = PyTuple_Pack(1, __pyx_kp_s_no_default___reduce___due_to_non); if (unlikely(!__pyx_tuple__143)) __PYX_ERR(7, 4, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_tuple__143);
  __Pyx_GIVEREF(__pyx_tuple__143);

  /* "(tree fragment)":2
 * def __reduce_cython__(self):
 *     raise TypeError("no default __reduce__ due to non-trivial __cinit__")             # <<<<<<<<<<<<<<
 * def __setstate_cython__(self, __pyx_state):
 *     raise TypeError("no default __reduce__ due to non-trivial __cinit__")
 */
  __pyx_tuple__144 = PyTuple_Pack(1, __pyx_kp_s_no_default___reduce___due_to_non); if (unlikely(!__pyx_tuple__144)) __PYX_ERR(7, 2, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_tuple__144);
  __Pyx_GIVEREF(__pyx_tuple__144);

  /* "(tree fragment)":4
 *     raise TypeError("no default __reduce__ due to non-trivial __cinit__")
 * def __setstate_cython__(self, __pyx_state):
 *     raise TypeError("no default __reduce__ due to non-trivial __cinit__")             # <<<<<<<<<<<<<<
 */
  __pyx_tuple__145 = PyTuple_Pack(1, __pyx_kp_s_no_default___reduce___due_to_non); if (unlikely(!__pyx_tuple__145)) __PYX_ERR(7, 4, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_tuple__145);
  __Pyx_GIVEREF(__pyx_tuple__145);

  /* "src/python/grpcio/grpc/_cython/_cygrpc/grpc_gevent.pyx.pxi":422
 *   g_pool = gevent.pool.Group()
 * 
 *   def cb_func(cb, args):             # <<<<<<<<<<<<<<
 *     _spawn_greenlet(cb, *args)
 *   set_async_callback_func(cb_func)
 */
  __pyx_tuple__146 = PyTuple_Pack(2, __pyx_n_s_cb, __pyx_n_s_args); if (unlikely(!__pyx_tuple__146)) __PYX_ERR(6, 422, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_tuple__146);
  __Pyx_GIVEREF(__pyx_tuple__146);
  __pyx_codeobj__147 = (PyObject*)__Pyx_PyCode_New(2, 0, 2, 0, CO_OPTIMIZED|CO_NEWLOCALS, __pyx_empty_bytes, __pyx_empty_tuple, __pyx_empty_tuple, __pyx_tuple__146, __pyx_empty_tuple, __pyx_empty_tuple, __pyx_kp_s_src_python_grpcio_grpc__cython_3, __pyx_n_s_cb_func, 422, __pyx_empty_bytes); if (unlikely(!__pyx_codeobj__147)) __PYX_ERR(6, 422, __pyx_L1_error)

  /* "src/python/grpcio/grpc/_cython/_cygrpc/fork_posix.pyx.pxi":45
 *         global _fork_handler_failed
 *         _fork_handler_failed = False
 *         with _fork_state.fork_in_progress_condition:             # <<<<<<<<<<<<<<
 *             _fork_state.fork_in_progress = True
 *         if not _fork_state.active_thread_count.await_zero_threads(
 */
  __pyx_tuple__148 = PyTuple_Pack(3, Py_None, Py_None, Py_None); if (unlikely(!__pyx_tuple__148)) __PYX_ERR(0, 45, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_tuple__148);
  __Pyx_GIVEREF(__pyx_tuple__148);

  /* "src/python/grpcio/grpc/_cython/_cygrpc/fork_posix.pyx.pxi":49
 *         if not _fork_state.active_thread_count.await_zero_threads(
 *                 _AWAIT_THREADS_TIMEOUT_SECONDS):
 *             _LOGGER.error(             # <<<<<<<<<<<<<<
 *                 'Failed to shutdown gRPC Python threads prior to fork. '
 *                 'Behavior after fork will be undefined.')
 */
  __pyx_tuple__149 = PyTuple_Pack(1, __pyx_kp_s_Failed_to_shutdown_gRPC_Python_t); if (unlikely(!__pyx_tuple__149)) __PYX_ERR(0, 49, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_tuple__149);
  __Pyx_GIVEREF(__pyx_tuple__149);

  /* "src/python/grpcio/grpc/_cython/_cygrpc/fork_posix.pyx.pxi":57
 * cdef void __postfork_parent() nogil:
 *     with gil:
 *         with _fork_state.fork_in_progress_condition:             # <<<<<<<<<<<<<<
 *             _fork_state.fork_in_progress = False
 *             _fork_state.fork_in_progress_condition.notify_all()
 */
  __pyx_tuple__150 = PyTuple_Pack(3, Py_None, Py_None, Py_None); if (unlikely(!__pyx_tuple__150)) __PYX_ERR(0, 57, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_tuple__150);
  __Pyx_GIVEREF(__pyx_tuple__150);

  /* "src/python/grpcio/grpc/_cython/_cygrpc/fork_posix.pyx.pxi":79
 *             for channel in _fork_state.channels:
 *                 channel._close_on_fork()
 *             with _fork_state.fork_in_progress_condition:             # <<<<<<<<<<<<<<
 *                 _fork_state.fork_in_progress = False
 *         except:
 */
  __pyx_tuple__151 = PyTuple_Pack(3, Py_None, Py_None, Py_None); if (unlikely(!__pyx_tuple__151)) __PYX_ERR(0, 79, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_tuple__151);
  __Pyx_GIVEREF(__pyx_tuple__151);

  /* "src/python/grpcio/grpc/_cython/_cygrpc/fork_posix.pyx.pxi":82
 *                 _fork_state.fork_in_progress = False
 *         except:
 *             _LOGGER.error('Exiting child due to raised exception')             # <<<<<<<<<<<<<<
 *             _LOGGER.error(sys.exc_info()[0])
 *             os._exit(os.EX_USAGE)
 */
  __pyx_tuple__152 = PyTuple_Pack(1, __pyx_kp_s_Exiting_child_due_to_raised_exce); if (unlikely(!__pyx_tuple__152)) __PYX_ERR(0, 82, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_tuple__152);
  __Pyx_GIVEREF(__pyx_tuple__152);

  /* "src/python/grpcio/grpc/_cython/_cygrpc/fork_posix.pyx.pxi":88
 *     if grpc_is_initialized() > 0:
 *         with gil:
 *             _LOGGER.error('Failed to shutdown gRPC Core after fork()')             # <<<<<<<<<<<<<<
 *             os._exit(os.EX_USAGE)
 * 
 */
  __pyx_tuple__153 = PyTuple_Pack(1, __pyx_kp_s_Failed_to_shutdown_gRPC_Core_aft); if (unlikely(!__pyx_tuple__153)) __PYX_ERR(0, 88, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_tuple__153);
  __Pyx_GIVEREF(__pyx_tuple__153);

  /* "src/python/grpcio/grpc/_cython/_cygrpc/fork_posix.pyx.pxi":95
 *     grpc_init()
 *     if _GRPC_ENABLE_FORK_SUPPORT:
 *         with _fork_state.fork_handler_registered_lock:             # <<<<<<<<<<<<<<
 *             if not _fork_state.fork_handler_registered:
 *                 pthread_atfork(&__prefork, &__postfork_parent, &__postfork_child)
 */
  __pyx_tuple__154 = PyTuple_Pack(3, Py_None, Py_None, Py_None); if (unlikely(!__pyx_tuple__154)) __PYX_ERR(0, 95, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_tuple__154);
  __Pyx_GIVEREF(__pyx_tuple__154);

  /* "src/python/grpcio/grpc/_cython/_cygrpc/fork_posix.pyx.pxi":104
 *     def __init__(self, target, args=()):
 *         if _GRPC_ENABLE_FORK_SUPPORT:
 *             def managed_target(*args):             # <<<<<<<<<<<<<<
 *                 try:
 *                     target(*args)
 */
  __pyx_tuple__155 = PyTuple_Pack(1, __pyx_n_s_args); if (unlikely(!__pyx_tuple__155)) __PYX_ERR(0, 104, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_tuple__155);
  __Pyx_GIVEREF(__pyx_tuple__155);
  __pyx_codeobj__156 = (PyObject*)__Pyx_PyCode_New(0, 0, 1, 0, CO_OPTIMIZED|CO_NEWLOCALS|CO_VARARGS, __pyx_empty_bytes, __pyx_empty_tuple, __pyx_empty_tuple, __pyx_tuple__155, __pyx_empty_tuple, __pyx_empty_tuple, __pyx_kp_s_src_python_grpcio_grpc__cython_4, __pyx_n_s_managed_target, 104, __pyx_empty_bytes); if (unlikely(!__pyx_codeobj__156)) __PYX_ERR(0, 104, __pyx_L1_error)

  /* "src/python/grpcio/grpc/_cython/_cygrpc/fork_posix.pyx.pxi":127
 * def block_if_fork_in_progress(postfork_state_to_reset=None):
 *     if _GRPC_ENABLE_FORK_SUPPORT:
 *         with _fork_state.fork_in_progress_condition:             # <<<<<<<<<<<<<<
 *             if not _fork_state.fork_in_progress:
 *                 return
 */
  __pyx_tuple__157 = PyTuple_Pack(3, Py_None, Py_None, Py_None); if (unlikely(!__pyx_tuple__157)) __PYX_ERR(0, 127, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_tuple__157);
  __Pyx_GIVEREF(__pyx_tuple__157);
  __pyx_tuple__158 = PyTuple_Pack(3, Py_None, Py_None, Py_None); if (unlikely(!__pyx_tuple__158)) __PYX_ERR(0, 127, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_tuple__158);
  __Pyx_GIVEREF(__pyx_tuple__158);

  /* "src/python/grpcio/grpc/_cython/_cygrpc/fork_posix.pyx.pxi":172
 * 
 *     def increment(self):
 *         with self._condition:             # <<<<<<<<<<<<<<
 *             self._num_active_threads += 1
 * 
 */
  __pyx_tuple__159 = PyTuple_Pack(3, Py_None, Py_None, Py_None); if (unlikely(!__pyx_tuple__159)) __PYX_ERR(0, 172, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_tuple__159);
  __Pyx_GIVEREF(__pyx_tuple__159);

  /* "src/python/grpcio/grpc/_cython/_cygrpc/fork_posix.pyx.pxi":176
 * 
 *     def decrement(self):
 *         with self._condition:             # <<<<<<<<<<<<<<
 *             self._num_active_threads -= 1
 *             if self._num_active_threads == 0:
 */
  __pyx_tuple__160 = PyTuple_Pack(3, Py_None, Py_None, Py_None); if (unlikely(!__pyx_tuple__160)) __PYX_ERR(0, 176, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_tuple__160);
  __Pyx_GIVEREF(__pyx_tuple__160);

  /* "src/python/grpcio/grpc/_cython/_cygrpc/fork_posix.pyx.pxi":184
 *         end_time = time.time() + timeout_secs
 *         wait_time = timeout_secs
 *         with self._condition:             # <<<<<<<<<<<<<<
 *             while True:
 *                 if self._num_active_threads > 0:
 */
  __pyx_tuple__161 = PyTuple_Pack(3, Py_None, Py_None, Py_None); if (unlikely(!__pyx_tuple__161)) __PYX_ERR(0, 184, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_tuple__161);
  __Pyx_GIVEREF(__pyx_tuple__161);
  __pyx_tuple__162 = PyTuple_Pack(3, Py_None, Py_None, Py_None); if (unlikely(!__pyx_tuple__162)) __PYX_ERR(0, 184, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_tuple__162);
  __Pyx_GIVEREF(__pyx_tuple__162);

  /* "src/python/grpcio/grpc/_cython/_cygrpc/channelz.pyx.pxi":16
 * 
 * 
 * def channelz_get_top_channels(start_channel_id):             # <<<<<<<<<<<<<<
 *     cdef char *c_returned_str = grpc_channelz_get_top_channels(
 *         start_channel_id,
 */
  __pyx_tuple__163 = PyTuple_Pack(2, __pyx_n_s_start_channel_id, __pyx_n_s_c_returned_str); if (unlikely(!__pyx_tuple__163)) __PYX_ERR(9, 16, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_tuple__163);
  __Pyx_GIVEREF(__pyx_tuple__163);
  __pyx_codeobj__164 = (PyObject*)__Pyx_PyCode_New(1, 0, 2, 0, CO_OPTIMIZED|CO_NEWLOCALS, __pyx_empty_bytes, __pyx_empty_tuple, __pyx_empty_tuple, __pyx_tuple__163, __pyx_empty_tuple, __pyx_empty_tuple, __pyx_kp_s_src_python_grpcio_grpc__cython_5, __pyx_n_s_channelz_get_top_channels, 16, __pyx_empty_bytes); if (unlikely(!__pyx_codeobj__164)) __PYX_ERR(9, 16, __pyx_L1_error)

  /* "src/python/grpcio/grpc/_cython/_cygrpc/channelz.pyx.pxi":25
 *     return c_returned_str
 * 
 * def channelz_get_servers(start_server_id):             # <<<<<<<<<<<<<<
 *     cdef char *c_returned_str = grpc_channelz_get_servers(start_server_id)
 *     if c_returned_str == NULL:
 */
  __pyx_tuple__165 = PyTuple_Pack(2, __pyx_n_s_start_server_id, __pyx_n_s_c_returned_str); if (unlikely(!__pyx_tuple__165)) __PYX_ERR(9, 25, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_tuple__165);
  __Pyx_GIVEREF(__pyx_tuple__165);
  __pyx_codeobj__166 = (PyObject*)__Pyx_PyCode_New(1, 0, 2, 0, CO_OPTIMIZED|CO_NEWLOCALS, __pyx_empty_bytes, __pyx_empty_tuple, __pyx_empty_tuple, __pyx_tuple__165, __pyx_empty_tuple, __pyx_empty_tuple, __pyx_kp_s_src_python_grpcio_grpc__cython_5, __pyx_n_s_channelz_get_servers, 25, __pyx_empty_bytes); if (unlikely(!__pyx_codeobj__166)) __PYX_ERR(9, 25, __pyx_L1_error)

  /* "src/python/grpcio/grpc/_cython/_cygrpc/channelz.pyx.pxi":32
 *     return c_returned_str
 * 
 * def channelz_get_server(server_id):             # <<<<<<<<<<<<<<
 *     cdef char *c_returned_str = grpc_channelz_get_server(server_id)
 *     if c_returned_str == NULL:
 */
  __pyx_tuple__167 = PyTuple_Pack(2, __pyx_n_s_server_id, __pyx_n_s_c_returned_str); if (unlikely(!__pyx_tuple__167)) __PYX_ERR(9, 32, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_tuple__167);
  __Pyx_GIVEREF(__pyx_tuple__167);
  __pyx_codeobj__168 = (PyObject*)__Pyx_PyCode_New(1, 0, 2, 0, CO_OPTIMIZED|CO_NEWLOCALS, __pyx_empty_bytes, __pyx_empty_tuple, __pyx_empty_tuple, __pyx_tuple__167, __pyx_empty_tuple, __pyx_empty_tuple, __pyx_kp_s_src_python_grpcio_grpc__cython_5, __pyx_n_s_channelz_get_server, 32, __pyx_empty_bytes); if (unlikely(!__pyx_codeobj__168)) __PYX_ERR(9, 32, __pyx_L1_error)

  /* "src/python/grpcio/grpc/_cython/_cygrpc/channelz.pyx.pxi":39
 *     return c_returned_str
 * 
 * def channelz_get_server_sockets(server_id, start_socket_id, max_results):             # <<<<<<<<<<<<<<
 *     cdef char *c_returned_str = grpc_channelz_get_server_sockets(
 *         server_id,
 */
  __pyx_tuple__169 = PyTuple_Pack(4, __pyx_n_s_server_id, __pyx_n_s_start_socket_id, __pyx_n_s_max_results, __pyx_n_s_c_returned_str); if (unlikely(!__pyx_tuple__169)) __PYX_ERR(9, 39, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_tuple__169);
  __Pyx_GIVEREF(__pyx_tuple__169);
  __pyx_codeobj__170 = (PyObject*)__Pyx_PyCode_New(3, 0, 4, 0, CO_OPTIMIZED|CO_NEWLOCALS, __pyx_empty_bytes, __pyx_empty_tuple, __pyx_empty_tuple, __pyx_tuple__169, __pyx_empty_tuple, __pyx_empty_tuple, __pyx_kp_s_src_python_grpcio_grpc__cython_5, __pyx_n_s_channelz_get_server_sockets, 39, __pyx_empty_bytes); if (unlikely(!__pyx_codeobj__170)) __PYX_ERR(9, 39, __pyx_L1_error)

  /* "src/python/grpcio/grpc/_cython/_cygrpc/channelz.pyx.pxi":52
 *     return c_returned_str
 * 
 * def channelz_get_channel(channel_id):             # <<<<<<<<<<<<<<
 *     cdef char *c_returned_str = grpc_channelz_get_channel(channel_id)
 *     if c_returned_str == NULL:
 */
  __pyx_tuple__171 = PyTuple_Pack(2, __pyx_n_s_channel_id, __pyx_n_s_c_returned_str); if (unlikely(!__pyx_tuple__171)) __PYX_ERR(9, 52, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_tuple__171);
  __Pyx_GIVEREF(__pyx_tuple__171);
  __pyx_codeobj__172 = (PyObject*)__Pyx_PyCode_New(1, 0, 2, 0, CO_OPTIMIZED|CO_NEWLOCALS, __pyx_empty_bytes, __pyx_empty_tuple, __pyx_empty_tuple, __pyx_tuple__171, __pyx_empty_tuple, __pyx_empty_tuple, __pyx_kp_s_src_python_grpcio_grpc__cython_5, __pyx_n_s_channelz_get_channel, 52, __pyx_empty_bytes); if (unlikely(!__pyx_codeobj__172)) __PYX_ERR(9, 52, __pyx_L1_error)

  /* "src/python/grpcio/grpc/_cython/_cygrpc/channelz.pyx.pxi":59
 *     return c_returned_str
 * 
 * def channelz_get_subchannel(subchannel_id):             # <<<<<<<<<<<<<<
 *     cdef char *c_returned_str = grpc_channelz_get_subchannel(subchannel_id)
 *     if c_returned_str == NULL:
 */
  __pyx_tuple__173 = PyTuple_Pack(2, __pyx_n_s_subchannel_id, __pyx_n_s_c_returned_str); if (unlikely(!__pyx_tuple__173)) __PYX_ERR(9, 59, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_tuple__173);
  __Pyx_GIVEREF(__pyx_tuple__173);
  __pyx_codeobj__174 = (PyObject*)__Pyx_PyCode_New(1, 0, 2, 0, CO_OPTIMIZED|CO_NEWLOCALS, __pyx_empty_bytes, __pyx_empty_tuple, __pyx_empty_tuple, __pyx_tuple__173, __pyx_empty_tuple, __pyx_empty_tuple, __pyx_kp_s_src_python_grpcio_grpc__cython_5, __pyx_n_s_channelz_get_subchannel, 59, __pyx_empty_bytes); if (unlikely(!__pyx_codeobj__174)) __PYX_ERR(9, 59, __pyx_L1_error)

  /* "src/python/grpcio/grpc/_cython/_cygrpc/channelz.pyx.pxi":66
 *     return c_returned_str
 * 
 * def channelz_get_socket(socket_id):             # <<<<<<<<<<<<<<
 *     cdef char *c_returned_str = grpc_channelz_get_socket(socket_id)
 *     if c_returned_str == NULL:
 */
  __pyx_tuple__175 = PyTuple_Pack(2, __pyx_n_s_socket_id, __pyx_n_s_c_returned_str); if (unlikely(!__pyx_tuple__175)) __PYX_ERR(9, 66, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_tuple__175);
  __Pyx_GIVEREF(__pyx_tuple__175);
  __pyx_codeobj__176 = (PyObject*)__Pyx_PyCode_New(1, 0, 2, 0, CO_OPTIMIZED|CO_NEWLOCALS, __pyx_empty_bytes, __pyx_empty_tuple, __pyx_empty_tuple, __pyx_tuple__175, __pyx_empty_tuple, __pyx_empty_tuple, __pyx_kp_s_src_python_grpcio_grpc__cython_5, __pyx_n_s_channelz_get_socket, 66, __pyx_empty_bytes); if (unlikely(!__pyx_codeobj__176)) __PYX_ERR(9, 66, __pyx_L1_error)

  /* "src/python/grpcio/grpc/_cython/_cygrpc/credentials.pyx.pxi":21
 * 
 * 
 * def _spawn_callback_in_thread(cb_func, args):             # <<<<<<<<<<<<<<
 *   ForkManagedThread(target=cb_func, args=args).start()
 * 
 */
  __pyx_tuple__177 = PyTuple_Pack(2, __pyx_n_s_cb_func, __pyx_n_s_args); if (unlikely(!__pyx_tuple__177)) __PYX_ERR(4, 21, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_tuple__177);
  __Pyx_GIVEREF(__pyx_tuple__177);
  __pyx_codeobj__178 = (PyObject*)__Pyx_PyCode_New(2, 0, 2, 0, CO_OPTIMIZED|CO_NEWLOCALS, __pyx_empty_bytes, __pyx_empty_tuple, __pyx_empty_tuple, __pyx_tuple__177, __pyx_empty_tuple, __pyx_empty_tuple, __pyx_kp_s_src_python_grpcio_grpc__cython_2, __pyx_n_s_spawn_callback_in_thread, 21, __pyx_empty_bytes); if (unlikely(!__pyx_codeobj__178)) __PYX_ERR(4, 21, __pyx_L1_error)

  /* "src/python/grpcio/grpc/_cython/_cygrpc/credentials.pyx.pxi":26
 * async_callback_func = _spawn_callback_in_thread
 * 
 * def set_async_callback_func(callback_func):             # <<<<<<<<<<<<<<
 *   global async_callback_func
 *   async_callback_func = callback_func
 */
  __pyx_tuple__179 = PyTuple_Pack(1, __pyx_n_s_callback_func); if (unlikely(!__pyx_tuple__179)) __PYX_ERR(4, 26, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_tuple__179);
  __Pyx_GIVEREF(__pyx_tuple__179);
  __pyx_codeobj__180 = (PyObject*)__Pyx_PyCode_New(1, 0, 1, 0, CO_OPTIMIZED|CO_NEWLOCALS, __pyx_empty_bytes, __pyx_empty_tuple, __pyx_empty_tuple, __pyx_tuple__179, __pyx_empty_tuple, __pyx_empty_tuple, __pyx_kp_s_src_python_grpcio_grpc__cython_2, __pyx_n_s_set_async_callback_func, 26, __pyx_empty_bytes); if (unlikely(!__pyx_codeobj__180)) __PYX_ERR(4, 26, __pyx_L1_error)

  /* "src/python/grpcio/grpc/_cython/_cygrpc/credentials.pyx.pxi":30
 *   async_callback_func = callback_func
 * 
 * def _spawn_callback_async(callback, args):             # <<<<<<<<<<<<<<
 *   async_callback_func(callback, args)
 * 
 */
  __pyx_tuple__181 = PyTuple_Pack(2, __pyx_n_s_callback, __pyx_n_s_args); if (unlikely(!__pyx_tuple__181)) __PYX_ERR(4, 30, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_tuple__181);
  __Pyx_GIVEREF(__pyx_tuple__181);
  __pyx_codeobj__182 = (PyObject*)__Pyx_PyCode_New(2, 0, 2, 0, CO_OPTIMIZED|CO_NEWLOCALS, __pyx_empty_bytes, __pyx_empty_tuple, __pyx_empty_tuple, __pyx_tuple__181, __pyx_empty_tuple, __pyx_empty_tuple, __pyx_kp_s_src_python_grpcio_grpc__cython_2, __pyx_n_s_spawn_callback_async, 30, __pyx_empty_bytes); if (unlikely(!__pyx_codeobj__182)) __PYX_ERR(4, 30, __pyx_L1_error)

  /* "src/python/grpcio/grpc/_cython/_cygrpc/credentials.pyx.pxi":233
 *   return c_ssl_pem_key_cert_pairs
 * 
 * def server_credentials_ssl(pem_root_certs, pem_key_cert_pairs,             # <<<<<<<<<<<<<<
 *                            bint force_client_auth):
 *   pem_root_certs = str_to_bytes(pem_root_certs)
 */
  __pyx_tuple__183 = PyTuple_Pack(7, __pyx_n_s_pem_root_certs, __pyx_n_s_pem_key_cert_pairs, __pyx_n_s_force_client_auth, __pyx_n_s_credentials, __pyx_n_s_c_pem_root_certs, __pyx_n_s_c_cert_config, __pyx_n_s_c_options); if (unlikely(!__pyx_tuple__183)) __PYX_ERR(4, 233, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_tuple__183);
  __Pyx_GIVEREF(__pyx_tuple__183);
  __pyx_codeobj__184 = (PyObject*)__Pyx_PyCode_New(3, 0, 7, 0, CO_OPTIMIZED|CO_NEWLOCALS, __pyx_empty_bytes, __pyx_empty_tuple, __pyx_empty_tuple, __pyx_tuple__183, __pyx_empty_tuple, __pyx_empty_tuple, __pyx_kp_s_src_python_grpcio_grpc__cython_2, __pyx_n_s_server_credentials_ssl, 233, __pyx_empty_bytes); if (unlikely(!__pyx_codeobj__184)) __PYX_ERR(4, 233, __pyx_L1_error)

  /* "src/python/grpcio/grpc/_cython/_cygrpc/credentials.pyx.pxi":258
 *   return credentials
 * 
 * def server_certificate_config_ssl(pem_root_certs, pem_key_cert_pairs):             # <<<<<<<<<<<<<<
 *   pem_root_certs = str_to_bytes(pem_root_certs)
 *   pem_key_cert_pairs = list(pem_key_cert_pairs)
 */
  __pyx_tuple__185 = PyTuple_Pack(3, __pyx_n_s_pem_root_certs, __pyx_n_s_pem_key_cert_pairs, __pyx_n_s_cert_config); if (unlikely(!__pyx_tuple__185)) __PYX_ERR(4, 258, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_tuple__185);
  __Pyx_GIVEREF(__pyx_tuple__185);
  __pyx_codeobj__186 = (PyObject*)__Pyx_PyCode_New(2, 0, 3, 0, CO_OPTIMIZED|CO_NEWLOCALS, __pyx_empty_bytes, __pyx_empty_tuple, __pyx_empty_tuple, __pyx_tuple__185, __pyx_empty_tuple, __pyx_empty_tuple, __pyx_kp_s_src_python_grpcio_grpc__cython_2, __pyx_n_s_server_certificate_config_ssl, 258, __pyx_empty_bytes); if (unlikely(!__pyx_codeobj__186)) __PYX_ERR(4, 258, __pyx_L1_error)

  /* "src/python/grpcio/grpc/_cython/_cygrpc/credentials.pyx.pxi":272
 *   return cert_config
 * 
 * def server_credentials_ssl_dynamic_cert_config(initial_cert_config,             # <<<<<<<<<<<<<<
 *                                                cert_config_fetcher,
 *                                                bint force_client_auth):
 */
  __pyx_tuple__187 = PyTuple_Pack(5, __pyx_n_s_initial_cert_config, __pyx_n_s_cert_config_fetcher, __pyx_n_s_force_client_auth, __pyx_n_s_credentials, __pyx_n_s_c_options); if (unlikely(!__pyx_tuple__187)) __PYX_ERR(4, 272, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_tuple__187);
  __Pyx_GIVEREF(__pyx_tuple__187);
  __pyx_codeobj__188 = (PyObject*)__Pyx_PyCode_New(3, 0, 5, 0, CO_OPTIMIZED|CO_NEWLOCALS, __pyx_empty_bytes, __pyx_empty_tuple, __pyx_empty_tuple, __pyx_tuple__187, __pyx_empty_tuple, __pyx_empty_tuple, __pyx_kp_s_src_python_grpcio_grpc__cython_2, __pyx_n_s_server_credentials_ssl_dynamic_c, 272, __pyx_empty_bytes); if (unlikely(!__pyx_codeobj__188)) __PYX_ERR(4, 272, __pyx_L1_error)

  /* "src/python/grpcio/grpc/_cython/_cygrpc/metadata.pyx.pxi":24
 * 
 * 
 * _Metadatum = collections.namedtuple('_Metadatum', ('key', 'value',))             # <<<<<<<<<<<<<<
 * 
 * 
 */
  __pyx_tuple__189 = PyTuple_Pack(2, __pyx_n_s_key, __pyx_n_s_value); if (unlikely(!__pyx_tuple__189)) __PYX_ERR(13, 24, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_tuple__189);
  __Pyx_GIVEREF(__pyx_tuple__189);
  __pyx_tuple__190 = PyTuple_Pack(2, __pyx_n_s_Metadatum, __pyx_tuple__189); if (unlikely(!__pyx_tuple__190)) __PYX_ERR(13, 24, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_tuple__190);
  __Pyx_GIVEREF(__pyx_tuple__190);

  /* "src/python/grpcio/grpc/_cython/_cygrpc/records.pyx.pxi":192
 * 
 * 
 * def compression_algorithm_name(grpc_compression_algorithm algorithm):             # <<<<<<<<<<<<<<
 *   cdef const char* name
 *   with nogil:
 */
  __pyx_tuple__191 = PyTuple_Pack(3, __pyx_n_s_algorithm, __pyx_n_s_algorithm, __pyx_n_s_name); if (unlikely(!__pyx_tuple__191)) __PYX_ERR(16, 192, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_tuple__191);
  __Pyx_GIVEREF(__pyx_tuple__191);
  __pyx_codeobj__192 = (PyObject*)__Pyx_PyCode_New(1, 0, 3, 0, CO_OPTIMIZED|CO_NEWLOCALS, __pyx_empty_bytes, __pyx_empty_tuple, __pyx_empty_tuple, __pyx_tuple__191, __pyx_empty_tuple, __pyx_empty_tuple, __pyx_kp_s_src_python_grpcio_grpc__cython_6, __pyx_n_s_compression_algorithm_name, 192, __pyx_empty_bytes); if (unlikely(!__pyx_codeobj__192)) __PYX_ERR(16, 192, __pyx_L1_error)

  /* "src/python/grpcio/grpc/_cython/_cygrpc/security.pyx.pxi":36
 * 
 * 
 * def peer_identities(Call call):             # <<<<<<<<<<<<<<
 *   cdef grpc_auth_context* auth_context
 *   cdef grpc_auth_property_iterator properties
 */
  __pyx_tuple__193 = PyTuple_Pack(5, __pyx_n_s_call, __pyx_n_s_auth_context, __pyx_n_s_properties, __pyx_n_s_property, __pyx_n_s_identities); if (unlikely(!__pyx_tuple__193)) __PYX_ERR(17, 36, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_tuple__193);
  __Pyx_GIVEREF(__pyx_tuple__193);
  __pyx_codeobj__194 = (PyObject*)__Pyx_PyCode_New(1, 0, 5, 0, CO_OPTIMIZED|CO_NEWLOCALS, __pyx_empty_bytes, __pyx_empty_tuple, __pyx_empty_tuple, __pyx_tuple__193, __pyx_empty_tuple, __pyx_empty_tuple, __pyx_kp_s_src_python_grpcio_grpc__cython_7, __pyx_n_s_peer_identities, 36, __pyx_empty_bytes); if (unlikely(!__pyx_codeobj__194)) __PYX_ERR(17, 36, __pyx_L1_error)

  /* "src/python/grpcio/grpc/_cython/_cygrpc/security.pyx.pxi":55
 *   return identities if identities else None
 * 
 * def peer_identity_key(Call call):             # <<<<<<<<<<<<<<
 *   cdef grpc_auth_context* auth_context
 *   cdef const char* c_key
 */
  __pyx_tuple__195 = PyTuple_Pack(4, __pyx_n_s_call, __pyx_n_s_auth_context, __pyx_n_s_c_key, __pyx_n_s_key); if (unlikely(!__pyx_tuple__195)) __PYX_ERR(17, 55, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_tuple__195);
  __Pyx_GIVEREF(__pyx_tuple__195);
  __pyx_codeobj__196 = (PyObject*)__Pyx_PyCode_New(1, 0, 4, 0, CO_OPTIMIZED|CO_NEWLOCALS, __pyx_empty_bytes, __pyx_empty_tuple, __pyx_empty_tuple, __pyx_tuple__195, __pyx_empty_tuple, __pyx_empty_tuple, __pyx_kp_s_src_python_grpcio_grpc__cython_7, __pyx_n_s_peer_identity_key, 55, __pyx_empty_bytes); if (unlikely(!__pyx_codeobj__196)) __PYX_ERR(17, 55, __pyx_L1_error)

  /* "src/python/grpcio/grpc/_cython/_cygrpc/security.pyx.pxi":69
 *   return key
 * 
 * def auth_context(Call call):             # <<<<<<<<<<<<<<
 *   cdef grpc_auth_context* auth_context
 *   cdef grpc_auth_property_iterator properties
 */
  __pyx_tuple__197 = PyTuple_Pack(6, __pyx_n_s_call, __pyx_n_s_auth_context, __pyx_n_s_properties, __pyx_n_s_property, __pyx_n_s_py_auth_context, __pyx_n_s_key); if (unlikely(!__pyx_tuple__197)) __PYX_ERR(17, 69, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_tuple__197);
  __Pyx_GIVEREF(__pyx_tuple__197);
  __pyx_codeobj__198 = (PyObject*)__Pyx_PyCode_New(1, 0, 6, 0, CO_OPTIMIZED|CO_NEWLOCALS, __pyx_empty_bytes, __pyx_empty_tuple, __pyx_empty_tuple, __pyx_tuple__197, __pyx_empty_tuple, __pyx_empty_tuple, __pyx_kp_s_src_python_grpcio_grpc__cython_7, __pyx_n_s_auth_context, 69, __pyx_empty_bytes); if (unlikely(!__pyx_codeobj__198)) __PYX_ERR(17, 69, __pyx_L1_error)

  /* "src/python/grpcio/grpc/_cython/_cygrpc/_hooks.pyx.pxi":19
 *   raise NotImplementedError("No custom hooks are implemented")
 * 
 * def install_context_from_request_call_event(RequestCallEvent event):             # <<<<<<<<<<<<<<
 *   pass
 * 
 */
  __pyx_tuple__199 = PyTuple_Pack(1, __pyx_n_s_event); if (unlikely(!__pyx_tuple__199)) __PYX_ERR(20, 19, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_tuple__199);
  __Pyx_GIVEREF(__pyx_tuple__199);
  __pyx_codeobj__200 = (PyObject*)__Pyx_PyCode_New(1, 0, 1, 0, CO_OPTIMIZED|CO_NEWLOCALS, __pyx_empty_bytes, __pyx_empty_tuple, __pyx_empty_tuple, __pyx_tuple__199, __pyx_empty_tuple, __pyx_empty_tuple, __pyx_kp_s_src_python_grpcio_grpc__cython_8, __pyx_n_s_install_context_from_request_cal, 19, __pyx_empty_bytes); if (unlikely(!__pyx_codeobj__200)) __PYX_ERR(20, 19, __pyx_L1_error)

  /* "src/python/grpcio/grpc/_cython/_cygrpc/_hooks.pyx.pxi":22
 *   pass
 * 
 * def uninstall_context():             # <<<<<<<<<<<<<<
 *   pass
 * 
 */
  __pyx_codeobj__201 = (PyObject*)__Pyx_PyCode_New(0, 0, 0, 0, CO_OPTIMIZED|CO_NEWLOCALS, __pyx_empty_bytes, __pyx_empty_tuple, __pyx_empty_tuple, __pyx_empty_tuple, __pyx_empty_tuple, __pyx_empty_tuple, __pyx_kp_s_src_python_grpcio_grpc__cython_8, __pyx_n_s_uninstall_context, 22, __pyx_empty_bytes); if (unlikely(!__pyx_codeobj__201)) __PYX_ERR(20, 22, __pyx_L1_error)

  /* "src/python/grpcio/grpc/_cython/_cygrpc/_hooks.pyx.pxi":25
 *   pass
 * 
 * def build_census_context():             # <<<<<<<<<<<<<<
 *   pass
 * 
 */
  __pyx_codeobj__202 = (PyObject*)__Pyx_PyCode_New(0, 0, 0, 0, CO_OPTIMIZED|CO_NEWLOCALS, __pyx_empty_bytes, __pyx_empty_tuple, __pyx_empty_tuple, __pyx_empty_tuple, __pyx_empty_tuple, __pyx_empty_tuple, __pyx_kp_s_src_python_grpcio_grpc__cython_8, __pyx_n_s_build_census_context, 25, __pyx_empty_bytes); if (unlikely(!__pyx_codeobj__202)) __PYX_ERR(20, 25, __pyx_L1_error)

  /* "src/python/grpcio/grpc/_cython/_cygrpc/_hooks.pyx.pxi":31
 *   pass
 * 
 * def set_census_context_on_call(_CallState call_state, CensusContext census_ctx):             # <<<<<<<<<<<<<<
 *   pass
 * 
 */
  __pyx_tuple__203 = PyTuple_Pack(2, __pyx_n_s_call_state, __pyx_n_s_census_ctx); if (unlikely(!__pyx_tuple__203)) __PYX_ERR(20, 31, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_tuple__203);
  __Pyx_GIVEREF(__pyx_tuple__203);
  __pyx_codeobj__204 = (PyObject*)__Pyx_PyCode_New(2, 0, 2, 0, CO_OPTIMIZED|CO_NEWLOCALS, __pyx_empty_bytes, __pyx_empty_tuple, __pyx_empty_tuple, __pyx_tuple__203, __pyx_empty_tuple, __pyx_empty_tuple, __pyx_kp_s_src_python_grpcio_grpc__cython_8, __pyx_n_s_set_census_context_on_call, 31, __pyx_empty_bytes); if (unlikely(!__pyx_codeobj__204)) __PYX_ERR(20, 31, __pyx_L1_error)

  /* "src/python/grpcio/grpc/_cython/_cygrpc/_hooks.pyx.pxi":34
 *   pass
 * 
 * def get_deadline_from_context():             # <<<<<<<<<<<<<<
 *   return None
 */
  __pyx_codeobj__205 = (PyObject*)__Pyx_PyCode_New(0, 0, 0, 0, CO_OPTIMIZED|CO_NEWLOCALS, __pyx_empty_bytes, __pyx_empty_tuple, __pyx_empty_tuple, __pyx_empty_tuple, __pyx_empty_tuple, __pyx_empty_tuple, __pyx_kp_s_src_python_grpcio_grpc__cython_8, __pyx_n_s_get_deadline_from_context, 34, __pyx_empty_bytes); if (unlikely(!__pyx_codeobj__205)) __PYX_ERR(20, 34, __pyx_L1_error)

  /* "src/python/grpcio/grpc/_cython/_cygrpc/grpc_gevent.pyx.pxi":72
 *   return addresses
 * 
 * def _spawn_greenlet(*args):             # <<<<<<<<<<<<<<
 *   greenlet = g_pool.spawn(*args)
 * 
 */
  __pyx_tuple__206 = PyTuple_Pack(2, __pyx_n_s_args, __pyx_n_s_greenlet); if (unlikely(!__pyx_tuple__206)) __PYX_ERR(6, 72, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_tuple__206);
  __Pyx_GIVEREF(__pyx_tuple__206);
  __pyx_codeobj__207 = (PyObject*)__Pyx_PyCode_New(0, 0, 2, 0, CO_OPTIMIZED|CO_NEWLOCALS|CO_VARARGS, __pyx_empty_bytes, __pyx_empty_tuple, __pyx_empty_tuple, __pyx_tuple__206, __pyx_empty_tuple, __pyx_empty_tuple, __pyx_kp_s_src_python_grpcio_grpc__cython_3, __pyx_n_s_spawn_greenlet, 72, __pyx_empty_bytes); if (unlikely(!__pyx_codeobj__207)) __PYX_ERR(6, 72, __pyx_L1_error)

  /* "src/python/grpcio/grpc/_cython/_cygrpc/grpc_gevent.pyx.pxi":110
 *   g_event.set()
 * 
 * def socket_connect_async(socket_wrapper, addr_tuple):             # <<<<<<<<<<<<<<
 *   socket_connect_async_cython(socket_wrapper, addr_tuple)
 * 
 */
  __pyx_tuple__208 = PyTuple_Pack(2, __pyx_n_s_socket_wrapper, __pyx_n_s_addr_tuple); if (unlikely(!__pyx_tuple__208)) __PYX_ERR(6, 110, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_tuple__208);
  __Pyx_GIVEREF(__pyx_tuple__208);
  __pyx_codeobj__209 = (PyObject*)__Pyx_PyCode_New(2, 0, 2, 0, CO_OPTIMIZED|CO_NEWLOCALS, __pyx_empty_bytes, __pyx_empty_tuple, __pyx_empty_tuple, __pyx_tuple__208, __pyx_empty_tuple, __pyx_empty_tuple, __pyx_kp_s_src_python_grpcio_grpc__cython_3, __pyx_n_s_socket_connect_async, 110, __pyx_empty_bytes); if (unlikely(!__pyx_codeobj__209)) __PYX_ERR(6, 110, __pyx_L1_error)

  /* "src/python/grpcio/grpc/_cython/_cygrpc/grpc_gevent.pyx.pxi":150
 *   socket_wrapper.close_cb(socket)
 * 
 * def socket_sendmsg(socket, write_bytes):             # <<<<<<<<<<<<<<
 *   try:
 *     return socket.sendmsg(write_bytes)
 */
  __pyx_tuple__210 = PyTuple_Pack(2, __pyx_n_s_socket, __pyx_n_s_write_bytes); if (unlikely(!__pyx_tuple__210)) __PYX_ERR(6, 150, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_tuple__210);
  __Pyx_GIVEREF(__pyx_tuple__210);
  __pyx_codeobj__211 = (PyObject*)__Pyx_PyCode_New(2, 0, 2, 0, CO_OPTIMIZED|CO_NEWLOCALS, __pyx_empty_bytes, __pyx_empty_tuple, __pyx_empty_tuple, __pyx_tuple__210, __pyx_empty_tuple, __pyx_empty_tuple, __pyx_kp_s_src_python_grpcio_grpc__cython_3, __pyx_n_s_socket_sendmsg, 150, __pyx_empty_bytes); if (unlikely(!__pyx_codeobj__211)) __PYX_ERR(6, 150, __pyx_L1_error)

  /* "src/python/grpcio/grpc/_cython/_cygrpc/grpc_gevent.pyx.pxi":175
 *   g_event.set()
 * 
 * def socket_write_async(socket_wrapper, write_bytes):             # <<<<<<<<<<<<<<
 *   socket_write_async_cython(socket_wrapper, write_bytes)
 * 
 */
  __pyx_tuple__212 = PyTuple_Pack(2, __pyx_n_s_socket_wrapper, __pyx_n_s_write_bytes); if (unlikely(!__pyx_tuple__212)) __PYX_ERR(6, 175, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_tuple__212);
  __Pyx_GIVEREF(__pyx_tuple__212);
  __pyx_codeobj__213 = (PyObject*)__Pyx_PyCode_New(2, 0, 2, 0, CO_OPTIMIZED|CO_NEWLOCALS, __pyx_empty_bytes, __pyx_empty_tuple, __pyx_empty_tuple, __pyx_tuple__212, __pyx_empty_tuple, __pyx_empty_tuple, __pyx_kp_s_src_python_grpcio_grpc__cython_3, __pyx_n_s_socket_write_async, 175, __pyx_empty_bytes); if (unlikely(!__pyx_codeobj__213)) __PYX_ERR(6, 175, __pyx_L1_error)

  /* "src/python/grpcio/grpc/_cython/_cygrpc/grpc_gevent.pyx.pxi":203
 *   g_event.set()
 * 
 * def socket_read_async(socket_wrapper):             # <<<<<<<<<<<<<<
 *   socket_read_async_cython(socket_wrapper)
 * 
 */
  __pyx_tuple__214 = PyTuple_Pack(1, __pyx_n_s_socket_wrapper); if (unlikely(!__pyx_tuple__214)) __PYX_ERR(6, 203, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_tuple__214);
  __Pyx_GIVEREF(__pyx_tuple__214);
  __pyx_codeobj__215 = (PyObject*)__Pyx_PyCode_New(1, 0, 1, 0, CO_OPTIMIZED|CO_NEWLOCALS, __pyx_empty_bytes, __pyx_empty_tuple, __pyx_empty_tuple, __pyx_tuple__214, __pyx_empty_tuple, __pyx_empty_tuple, __pyx_kp_s_src_python_grpcio_grpc__cython_3, __pyx_n_s_socket_read_async, 203, __pyx_empty_bytes); if (unlikely(!__pyx_codeobj__215)) __PYX_ERR(6, 203, __pyx_L1_error)

  /* "src/python/grpcio/grpc/_cython/_cygrpc/grpc_gevent.pyx.pxi":242
 *   return grpc_error_none()
 * 
 * def applysockopts(s):             # <<<<<<<<<<<<<<
 *   s.setsockopt(gevent_socket.SOL_SOCKET, gevent_socket.SO_REUSEADDR, 1)
 *   s.setsockopt(gevent_socket.IPPROTO_TCP, gevent_socket.TCP_NODELAY, True)
 */
  __pyx_tuple__216 = PyTuple_Pack(1, __pyx_n_s_s); if (unlikely(!__pyx_tuple__216)) __PYX_ERR(6, 242, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_tuple__216);
  __Pyx_GIVEREF(__pyx_tuple__216);
  __pyx_codeobj__217 = (PyObject*)__Pyx_PyCode_New(1, 0, 1, 0, CO_OPTIMIZED|CO_NEWLOCALS, __pyx_empty_bytes, __pyx_empty_tuple, __pyx_empty_tuple, __pyx_tuple__216, __pyx_empty_tuple, __pyx_empty_tuple, __pyx_kp_s_src_python_grpcio_grpc__cython_3, __pyx_n_s_applysockopts, 242, __pyx_empty_bytes); if (unlikely(!__pyx_codeobj__217)) __PYX_ERR(6, 242, __pyx_L1_error)

  /* "src/python/grpcio/grpc/_cython/_cygrpc/grpc_gevent.pyx.pxi":291
 *    g_event.set()
 * 
 * def socket_accept_async(s):             # <<<<<<<<<<<<<<
 *   accept_callback_cython(s)
 * 
 */
  __pyx_tuple__218 = PyTuple_Pack(1, __pyx_n_s_s); if (unlikely(!__pyx_tuple__218)) __PYX_ERR(6, 291, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_tuple__218);
  __Pyx_GIVEREF(__pyx_tuple__218);
  __pyx_codeobj__219 = (PyObject*)__Pyx_PyCode_New(1, 0, 1, 0, CO_OPTIMIZED|CO_NEWLOCALS, __pyx_empty_bytes, __pyx_empty_tuple, __pyx_empty_tuple, __pyx_tuple__218, __pyx_empty_tuple, __pyx_empty_tuple, __pyx_kp_s_src_python_grpcio_grpc__cython_3, __pyx_n_s_socket_accept_async, 291, __pyx_empty_bytes); if (unlikely(!__pyx_codeobj__219)) __PYX_ERR(6, 291, __pyx_L1_error)

  /* "src/python/grpcio/grpc/_cython/_cygrpc/grpc_gevent.pyx.pxi":322
 *   g_event.set()
 * 
 * def socket_resolve_async_python(resolve_wrapper):             # <<<<<<<<<<<<<<
 *   socket_resolve_async_cython(resolve_wrapper)
 * 
 */
  __pyx_tuple__220 = PyTuple_Pack(1, __pyx_n_s_resolve_wrapper); if (unlikely(!__pyx_tuple__220)) __PYX_ERR(6, 322, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_tuple__220);
  __Pyx_GIVEREF(__pyx_tuple__220);
  __pyx_codeobj__221 = (PyObject*)__Pyx_PyCode_New(1, 0, 1, 0, CO_OPTIMIZED|CO_NEWLOCALS, __pyx_empty_bytes, __pyx_empty_tuple, __pyx_empty_tuple, __pyx_tuple__220, __pyx_empty_tuple, __pyx_empty_tuple, __pyx_kp_s_src_python_grpcio_grpc__cython_3, __pyx_n_s_socket_resolve_async_python, 322, __pyx_empty_bytes); if (unlikely(!__pyx_codeobj__221)) __PYX_ERR(6, 322, __pyx_L1_error)

  /* "src/python/grpcio/grpc/_cython/_cygrpc/grpc_gevent.pyx.pxi":401
 * cdef grpc_custom_poller_vtable gevent_pollset_vtable
 * 
 * def init_grpc_gevent():             # <<<<<<<<<<<<<<
 *   # Lazily import gevent
 *   global gevent_socket
 */
  __pyx_tuple__222 = PyTuple_Pack(3, __pyx_n_s_gevent, __pyx_n_s_cb_func, __pyx_n_s_cb_func); if (unlikely(!__pyx_tuple__222)) __PYX_ERR(6, 401, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_tuple__222);
  __Pyx_GIVEREF(__pyx_tuple__222);
  __pyx_codeobj__223 = (PyObject*)__Pyx_PyCode_New(0, 0, 3, 0, CO_OPTIMIZED|CO_NEWLOCALS, __pyx_empty_bytes, __pyx_empty_tuple, __pyx_empty_tuple, __pyx_tuple__222, __pyx_empty_tuple, __pyx_empty_tuple, __pyx_kp_s_src_python_grpcio_grpc__cython_3, __pyx_n_s_init_grpc_gevent, 401, __pyx_empty_bytes); if (unlikely(!__pyx_codeobj__223)) __PYX_ERR(6, 401, __pyx_L1_error)

  /* "src/python/grpcio/grpc/_cython/_cygrpc/fork_posix.pyx.pxi":36
 * # This flag is not supported on Windows.
 * _GRPC_ENABLE_FORK_SUPPORT = (
 *     os.environ.get('GRPC_ENABLE_FORK_SUPPORT', '0')             # <<<<<<<<<<<<<<
 *         .lower() in _TRUE_VALUES)
 * 
 */
  __pyx_tuple__224 = PyTuple_Pack(2, __pyx_n_s_GRPC_ENABLE_FORK_SUPPORT_2, __pyx_kp_s_0); if (unlikely(!__pyx_tuple__224)) __PYX_ERR(0, 36, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_tuple__224);
  __Pyx_GIVEREF(__pyx_tuple__224);

  /* "src/python/grpcio/grpc/_cython/_cygrpc/fork_posix.pyx.pxi":92
 * 
 * 
 * def fork_handlers_and_grpc_init():             # <<<<<<<<<<<<<<
 *     grpc_init()
 *     if _GRPC_ENABLE_FORK_SUPPORT:
 */
  __pyx_codeobj__225 = (PyObject*)__Pyx_PyCode_New(0, 0, 0, 0, CO_OPTIMIZED|CO_NEWLOCALS, __pyx_empty_bytes, __pyx_empty_tuple, __pyx_empty_tuple, __pyx_empty_tuple, __pyx_empty_tuple, __pyx_empty_tuple, __pyx_kp_s_src_python_grpcio_grpc__cython_4, __pyx_n_s_fork_handlers_and_grpc_init, 92, __pyx_empty_bytes); if (unlikely(!__pyx_codeobj__225)) __PYX_ERR(0, 92, __pyx_L1_error)

  /* "src/python/grpcio/grpc/_cython/_cygrpc/fork_posix.pyx.pxi":101
 * 
 * 
 * class ForkManagedThread(object):             # <<<<<<<<<<<<<<
 *     def __init__(self, target, args=()):
 *         if _GRPC_ENABLE_FORK_SUPPORT:
 */
  __pyx_tuple__226 = PyTuple_Pack(1, __pyx_builtin_object); if (unlikely(!__pyx_tuple__226)) __PYX_ERR(0, 101, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_tuple__226);
  __Pyx_GIVEREF(__pyx_tuple__226);

  /* "src/python/grpcio/grpc/_cython/_cygrpc/fork_posix.pyx.pxi":102
 * 
 * class ForkManagedThread(object):
 *     def __init__(self, target, args=()):             # <<<<<<<<<<<<<<
 *         if _GRPC_ENABLE_FORK_SUPPORT:
 *             def managed_target(*args):
 */
  __pyx_tuple__227 = PyTuple_Pack(5, __pyx_n_s_self, __pyx_n_s_target, __pyx_n_s_args, __pyx_n_s_managed_target, __pyx_n_s_managed_target); if (unlikely(!__pyx_tuple__227)) __PYX_ERR(0, 102, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_tuple__227);
  __Pyx_GIVEREF(__pyx_tuple__227);
  __pyx_codeobj__228 = (PyObject*)__Pyx_PyCode_New(3, 0, 5, 0, CO_OPTIMIZED|CO_NEWLOCALS, __pyx_empty_bytes, __pyx_empty_tuple, __pyx_empty_tuple, __pyx_tuple__227, __pyx_empty_tuple, __pyx_empty_tuple, __pyx_kp_s_src_python_grpcio_grpc__cython_4, __pyx_n_s_init, 102, __pyx_empty_bytes); if (unlikely(!__pyx_codeobj__228)) __PYX_ERR(0, 102, __pyx_L1_error)
  __pyx_tuple__229 = PyTuple_Pack(1, ((PyObject*)__pyx_empty_tuple)); if (unlikely(!__pyx_tuple__229)) __PYX_ERR(0, 102, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_tuple__229);
  __Pyx_GIVEREF(__pyx_tuple__229);

  /* "src/python/grpcio/grpc/_cython/_cygrpc/fork_posix.pyx.pxi":113
 *             self._thread = threading.Thread(target=target, args=args)
 * 
 *     def setDaemon(self, daemonic):             # <<<<<<<<<<<<<<
 *         self._thread.daemon = daemonic
 * 
 */
  __pyx_tuple__230 = PyTuple_Pack(2, __pyx_n_s_self, __pyx_n_s_daemonic); if (unlikely(!__pyx_tuple__230)) __PYX_ERR(0, 113, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_tuple__230);
  __Pyx_GIVEREF(__pyx_tuple__230);
  __pyx_codeobj__231 = (PyObject*)__Pyx_PyCode_New(2, 0, 2, 0, CO_OPTIMIZED|CO_NEWLOCALS, __pyx_empty_bytes, __pyx_empty_tuple, __pyx_empty_tuple, __pyx_tuple__230, __pyx_empty_tuple, __pyx_empty_tuple, __pyx_kp_s_src_python_grpcio_grpc__cython_4, __pyx_n_s_setDaemon, 113, __pyx_empty_bytes); if (unlikely(!__pyx_codeobj__231)) __PYX_ERR(0, 113, __pyx_L1_error)

  /* "src/python/grpcio/grpc/_cython/_cygrpc/fork_posix.pyx.pxi":116
 *         self._thread.daemon = daemonic
 * 
 *     def start(self):             # <<<<<<<<<<<<<<
 *         if _GRPC_ENABLE_FORK_SUPPORT:
 *             _fork_state.active_thread_count.increment()
 */
  __pyx_tuple__232 = PyTuple_Pack(1, __pyx_n_s_self); if (unlikely(!__pyx_tuple__232)) __PYX_ERR(0, 116, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_tuple__232);
  __Pyx_GIVEREF(__pyx_tuple__232);
  __pyx_codeobj__233 = (PyObject*)__Pyx_PyCode_New(1, 0, 1, 0, CO_OPTIMIZED|CO_NEWLOCALS, __pyx_empty_bytes, __pyx_empty_tuple, __pyx_empty_tuple, __pyx_tuple__232, __pyx_empty_tuple, __pyx_empty_tuple, __pyx_kp_s_src_python_grpcio_grpc__cython_4, __pyx_n_s_start, 116, __pyx_empty_bytes); if (unlikely(!__pyx_codeobj__233)) __PYX_ERR(0, 116, __pyx_L1_error)

  /* "src/python/grpcio/grpc/_cython/_cygrpc/fork_posix.pyx.pxi":121
 *         self._thread.start()
 * 
 *     def join(self):             # <<<<<<<<<<<<<<
 *         self._thread.join()
 * 
 */
  __pyx_tuple__234 = PyTuple_Pack(1, __pyx_n_s_self); if (unlikely(!__pyx_tuple__234)) __PYX_ERR(0, 121, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_tuple__234);
  __Pyx_GIVEREF(__pyx_tuple__234);
  __pyx_codeobj__235 = (PyObject*)__Pyx_PyCode_New(1, 0, 1, 0, CO_OPTIMIZED|CO_NEWLOCALS, __pyx_empty_bytes, __pyx_empty_tuple, __pyx_empty_tuple, __pyx_tuple__234, __pyx_empty_tuple, __pyx_empty_tuple, __pyx_kp_s_src_python_grpcio_grpc__cython_4, __pyx_n_s_join, 121, __pyx_empty_bytes); if (unlikely(!__pyx_codeobj__235)) __PYX_ERR(0, 121, __pyx_L1_error)

  /* "src/python/grpcio/grpc/_cython/_cygrpc/fork_posix.pyx.pxi":125
 * 
 * 
 * def block_if_fork_in_progress(postfork_state_to_reset=None):             # <<<<<<<<<<<<<<
 *     if _GRPC_ENABLE_FORK_SUPPORT:
 *         with _fork_state.fork_in_progress_condition:
 */
  __pyx_tuple__236 = PyTuple_Pack(1, __pyx_n_s_postfork_state_to_reset); if (unlikely(!__pyx_tuple__236)) __PYX_ERR(0, 125, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_tuple__236);
  __Pyx_GIVEREF(__pyx_tuple__236);
  __pyx_codeobj__237 = (PyObject*)__Pyx_PyCode_New(1, 0, 1, 0, CO_OPTIMIZED|CO_NEWLOCALS, __pyx_empty_bytes, __pyx_empty_tuple, __pyx_empty_tuple, __pyx_tuple__236, __pyx_empty_tuple, __pyx_empty_tuple, __pyx_kp_s_src_python_grpcio_grpc__cython_4, __pyx_n_s_block_if_fork_in_progress, 125, __pyx_empty_bytes); if (unlikely(!__pyx_codeobj__237)) __PYX_ERR(0, 125, __pyx_L1_error)

  /* "src/python/grpcio/grpc/_cython/_cygrpc/fork_posix.pyx.pxi":137
 * 
 * 
 * def enter_user_request_generator():             # <<<<<<<<<<<<<<
 *     if _GRPC_ENABLE_FORK_SUPPORT:
 *         _fork_state.active_thread_count.decrement()
 */
  __pyx_codeobj__238 = (PyObject*)__Pyx_PyCode_New(0, 0, 0, 0, CO_OPTIMIZED|CO_NEWLOCALS, __pyx_empty_bytes, __pyx_empty_tuple, __pyx_empty_tuple, __pyx_empty_tuple, __pyx_empty_tuple, __pyx_empty_tuple, __pyx_kp_s_src_python_grpcio_grpc__cython_4, __pyx_n_s_enter_user_request_generator, 137, __pyx_empty_bytes); if (unlikely(!__pyx_codeobj__238)) __PYX_ERR(0, 137, __pyx_L1_error)

  /* "src/python/grpcio/grpc/_cython/_cygrpc/fork_posix.pyx.pxi":142
 * 
 * 
 * def return_from_user_request_generator():             # <<<<<<<<<<<<<<
 *     if _GRPC_ENABLE_FORK_SUPPORT:
 *         _fork_state.active_thread_count.increment()
 */
  __pyx_codeobj__239 = (PyObject*)__Pyx_PyCode_New(0, 0, 0, 0, CO_OPTIMIZED|CO_NEWLOCALS, __pyx_empty_bytes, __pyx_empty_tuple, __pyx_empty_tuple, __pyx_empty_tuple, __pyx_empty_tuple, __pyx_empty_tuple, __pyx_kp_s_src_python_grpcio_grpc__cython_4, __pyx_n_s_return_from_user_request_generat, 142, __pyx_empty_bytes); if (unlikely(!__pyx_codeobj__239)) __PYX_ERR(0, 142, __pyx_L1_error)

  /* "src/python/grpcio/grpc/_cython/_cygrpc/fork_posix.pyx.pxi":148
 * 
 * 
 * def get_fork_epoch():             # <<<<<<<<<<<<<<
 *     return _fork_state.fork_epoch
 * 
 */
  __pyx_codeobj__240 = (PyObject*)__Pyx_PyCode_New(0, 0, 0, 0, CO_OPTIMIZED|CO_NEWLOCALS, __pyx_empty_bytes, __pyx_empty_tuple, __pyx_empty_tuple, __pyx_empty_tuple, __pyx_empty_tuple, __pyx_empty_tuple, __pyx_kp_s_src_python_grpcio_grpc__cython_4, __pyx_n_s_get_fork_epoch, 148, __pyx_empty_bytes); if (unlikely(!__pyx_codeobj__240)) __PYX_ERR(0, 148, __pyx_L1_error)

  /* "src/python/grpcio/grpc/_cython/_cygrpc/fork_posix.pyx.pxi":152
 * 
 * 
 * def is_fork_support_enabled():             # <<<<<<<<<<<<<<
 *     return _GRPC_ENABLE_FORK_SUPPORT
 * 
 */
  __pyx_codeobj__241 = (PyObject*)__Pyx_PyCode_New(0, 0, 0, 0, CO_OPTIMIZED|CO_NEWLOCALS, __pyx_empty_bytes, __pyx_empty_tuple, __pyx_empty_tuple, __pyx_empty_tuple, __pyx_empty_tuple, __pyx_empty_tuple, __pyx_kp_s_src_python_grpcio_grpc__cython_4, __pyx_n_s_is_fork_support_enabled, 152, __pyx_empty_bytes); if (unlikely(!__pyx_codeobj__241)) __PYX_ERR(0, 152, __pyx_L1_error)

  /* "src/python/grpcio/grpc/_cython/_cygrpc/fork_posix.pyx.pxi":156
 * 
 * 
 * def fork_register_channel(channel):             # <<<<<<<<<<<<<<
 *     if _GRPC_ENABLE_FORK_SUPPORT:
 *         _fork_state.channels.add(channel)
 */
  __pyx_tuple__242 = PyTuple_Pack(1, __pyx_n_s_channel); if (unlikely(!__pyx_tuple__242)) __PYX_ERR(0, 156, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_tuple__242);
  __Pyx_GIVEREF(__pyx_tuple__242);
  __pyx_codeobj__243 = (PyObject*)__Pyx_PyCode_New(1, 0, 1, 0, CO_OPTIMIZED|CO_NEWLOCALS, __pyx_empty_bytes, __pyx_empty_tuple, __pyx_empty_tuple, __pyx_tuple__242, __pyx_empty_tuple, __pyx_empty_tuple, __pyx_kp_s_src_python_grpcio_grpc__cython_4, __pyx_n_s_fork_register_channel, 156, __pyx_empty_bytes); if (unlikely(!__pyx_codeobj__243)) __PYX_ERR(0, 156, __pyx_L1_error)

  /* "src/python/grpcio/grpc/_cython/_cygrpc/fork_posix.pyx.pxi":161
 * 
 * 
 * def fork_unregister_channel(channel):             # <<<<<<<<<<<<<<
 *     if _GRPC_ENABLE_FORK_SUPPORT:
 *         _fork_state.channels.discard(channel)
 */
  __pyx_tuple__244 = PyTuple_Pack(1, __pyx_n_s_channel); if (unlikely(!__pyx_tuple__244)) __PYX_ERR(0, 161, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_tuple__244);
  __Pyx_GIVEREF(__pyx_tuple__244);
  __pyx_codeobj__245 = (PyObject*)__Pyx_PyCode_New(1, 0, 1, 0, CO_OPTIMIZED|CO_NEWLOCALS, __pyx_empty_bytes, __pyx_empty_tuple, __pyx_empty_tuple, __pyx_tuple__244, __pyx_empty_tuple, __pyx_empty_tuple, __pyx_kp_s_src_python_grpcio_grpc__cython_4, __pyx_n_s_fork_unregister_channel, 161, __pyx_empty_bytes); if (unlikely(!__pyx_codeobj__245)) __PYX_ERR(0, 161, __pyx_L1_error)

  /* "src/python/grpcio/grpc/_cython/_cygrpc/fork_posix.pyx.pxi":166
 * 
 * 
 * class _ActiveThreadCount(object):             # <<<<<<<<<<<<<<
 *     def __init__(self):
 *         self._num_active_threads = 0
 */
  __pyx_tuple__246 = PyTuple_Pack(1, __pyx_builtin_object); if (unlikely(!__pyx_tuple__246)) __PYX_ERR(0, 166, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_tuple__246);
  __Pyx_GIVEREF(__pyx_tuple__246);

  /* "src/python/grpcio/grpc/_cython/_cygrpc/fork_posix.pyx.pxi":167
 * 
 * class _ActiveThreadCount(object):
 *     def __init__(self):             # <<<<<<<<<<<<<<
 *         self._num_active_threads = 0
 *         self._condition = threading.Condition()
 */
  __pyx_tuple__247 = PyTuple_Pack(1, __pyx_n_s_self); if (unlikely(!__pyx_tuple__247)) __PYX_ERR(0, 167, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_tuple__247);
  __Pyx_GIVEREF(__pyx_tuple__247);
  __pyx_codeobj__248 = (PyObject*)__Pyx_PyCode_New(1, 0, 1, 0, CO_OPTIMIZED|CO_NEWLOCALS, __pyx_empty_bytes, __pyx_empty_tuple, __pyx_empty_tuple, __pyx_tuple__247, __pyx_empty_tuple, __pyx_empty_tuple, __pyx_kp_s_src_python_grpcio_grpc__cython_4, __pyx_n_s_init, 167, __pyx_empty_bytes); if (unlikely(!__pyx_codeobj__248)) __PYX_ERR(0, 167, __pyx_L1_error)

  /* "src/python/grpcio/grpc/_cython/_cygrpc/fork_posix.pyx.pxi":171
 *         self._condition = threading.Condition()
 * 
 *     def increment(self):             # <<<<<<<<<<<<<<
 *         with self._condition:
 *             self._num_active_threads += 1
 */
  __pyx_tuple__249 = PyTuple_Pack(1, __pyx_n_s_self); if (unlikely(!__pyx_tuple__249)) __PYX_ERR(0, 171, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_tuple__249);
  __Pyx_GIVEREF(__pyx_tuple__249);
  __pyx_codeobj__250 = (PyObject*)__Pyx_PyCode_New(1, 0, 1, 0, CO_OPTIMIZED|CO_NEWLOCALS, __pyx_empty_bytes, __pyx_empty_tuple, __pyx_empty_tuple, __pyx_tuple__249, __pyx_empty_tuple, __pyx_empty_tuple, __pyx_kp_s_src_python_grpcio_grpc__cython_4, __pyx_n_s_increment, 171, __pyx_empty_bytes); if (unlikely(!__pyx_codeobj__250)) __PYX_ERR(0, 171, __pyx_L1_error)

  /* "src/python/grpcio/grpc/_cython/_cygrpc/fork_posix.pyx.pxi":175
 *             self._num_active_threads += 1
 * 
 *     def decrement(self):             # <<<<<<<<<<<<<<
 *         with self._condition:
 *             self._num_active_threads -= 1
 */
  __pyx_tuple__251 = PyTuple_Pack(1, __pyx_n_s_self); if (unlikely(!__pyx_tuple__251)) __PYX_ERR(0, 175, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_tuple__251);
  __Pyx_GIVEREF(__pyx_tuple__251);
  __pyx_codeobj__252 = (PyObject*)__Pyx_PyCode_New(1, 0, 1, 0, CO_OPTIMIZED|CO_NEWLOCALS, __pyx_empty_bytes, __pyx_empty_tuple, __pyx_empty_tuple, __pyx_tuple__251, __pyx_empty_tuple, __pyx_empty_tuple, __pyx_kp_s_src_python_grpcio_grpc__cython_4, __pyx_n_s_decrement, 175, __pyx_empty_bytes); if (unlikely(!__pyx_codeobj__252)) __PYX_ERR(0, 175, __pyx_L1_error)

  /* "src/python/grpcio/grpc/_cython/_cygrpc/fork_posix.pyx.pxi":181
 *                 self._condition.notify_all()
 * 
 *     def await_zero_threads(self, timeout_secs):             # <<<<<<<<<<<<<<
 *         end_time = time.time() + timeout_secs
 *         wait_time = timeout_secs
 */
  __pyx_tuple__253 = PyTuple_Pack(4, __pyx_n_s_self, __pyx_n_s_timeout_secs, __pyx_n_s_end_time, __pyx_n_s_wait_time); if (unlikely(!__pyx_tuple__253)) __PYX_ERR(0, 181, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_tuple__253);
  __Pyx_GIVEREF(__pyx_tuple__253);
  __pyx_codeobj__254 = (PyObject*)__Pyx_PyCode_New(2, 0, 4, 0, CO_OPTIMIZED|CO_NEWLOCALS, __pyx_empty_bytes, __pyx_empty_tuple, __pyx_empty_tuple, __pyx_tuple__253, __pyx_empty_tuple, __pyx_empty_tuple, __pyx_kp_s_src_python_grpcio_grpc__cython_4, __pyx_n_s_await_zero_threads, 181, __pyx_empty_bytes); if (unlikely(!__pyx_codeobj__254)) __PYX_ERR(0, 181, __pyx_L1_error)

  /* "src/python/grpcio/grpc/_cython/_cygrpc/fork_posix.pyx.pxi":198
 * 
 * 
 * class _ForkState(object):             # <<<<<<<<<<<<<<
 *     def __init__(self):
 *         self.fork_in_progress_condition = threading.Condition()
 */
  __pyx_tuple__255 = PyTuple_Pack(1, __pyx_builtin_object); if (unlikely(!__pyx_tuple__255)) __PYX_ERR(0, 198, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_tuple__255);
  __Pyx_GIVEREF(__pyx_tuple__255);

  /* "src/python/grpcio/grpc/_cython/_cygrpc/fork_posix.pyx.pxi":199
 * 
 * class _ForkState(object):
 *     def __init__(self):             # <<<<<<<<<<<<<<
 *         self.fork_in_progress_condition = threading.Condition()
 *         self.fork_in_progress = False
 */
  __pyx_tuple__256 = PyTuple_Pack(1, __pyx_n_s_self); if (unlikely(!__pyx_tuple__256)) __PYX_ERR(0, 199, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_tuple__256);
  __Pyx_GIVEREF(__pyx_tuple__256);
  __pyx_codeobj__257 = (PyObject*)__Pyx_PyCode_New(1, 0, 1, 0, CO_OPTIMIZED|CO_NEWLOCALS, __pyx_empty_bytes, __pyx_empty_tuple, __pyx_empty_tuple, __pyx_tuple__256, __pyx_empty_tuple, __pyx_empty_tuple, __pyx_kp_s_src_python_grpcio_grpc__cython_4, __pyx_n_s_init, 199, __pyx_empty_bytes); if (unlikely(!__pyx_codeobj__257)) __PYX_ERR(0, 199, __pyx_L1_error)

  /* "(tree fragment)":1
 * def __pyx_unpickle_ChannelCredentials(__pyx_type, long __pyx_checksum, __pyx_state):             # <<<<<<<<<<<<<<
 *     if __pyx_checksum != 0xd41d8cd:
 *         from pickle import PickleError as __pyx_PickleError
 */
  __pyx_tuple__258 = PyTuple_Pack(5, __pyx_n_s_pyx_type, __pyx_n_s_pyx_checksum, __pyx_n_s_pyx_state, __pyx_n_s_pyx_PickleError, __pyx_n_s_pyx_result); if (unlikely(!__pyx_tuple__258)) __PYX_ERR(7, 1, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_tuple__258);
  __Pyx_GIVEREF(__pyx_tuple__258);
  __pyx_codeobj__259 = (PyObject*)__Pyx_PyCode_New(3, 0, 5, 0, CO_OPTIMIZED|CO_NEWLOCALS, __pyx_empty_bytes, __pyx_empty_tuple, __pyx_empty_tuple, __pyx_tuple__258, __pyx_empty_tuple, __pyx_empty_tuple, __pyx_kp_s_stringsource, __pyx_n_s_pyx_unpickle_ChannelCredential, 1, __pyx_empty_bytes); if (unlikely(!__pyx_codeobj__259)) __PYX_ERR(7, 1, __pyx_L1_error)
  __pyx_tuple__260 = PyTuple_Pack(5, __pyx_n_s_pyx_type, __pyx_n_s_pyx_checksum, __pyx_n_s_pyx_state, __pyx_n_s_pyx_PickleError, __pyx_n_s_pyx_result); if (unlikely(!__pyx_tuple__260)) __PYX_ERR(7, 1, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_tuple__260);
  __Pyx_GIVEREF(__pyx_tuple__260);
  __pyx_codeobj__261 = (PyObject*)__Pyx_PyCode_New(3, 0, 5, 0, CO_OPTIMIZED|CO_NEWLOCALS, __pyx_empty_bytes, __pyx_empty_tuple, __pyx_empty_tuple, __pyx_tuple__260, __pyx_empty_tuple, __pyx_empty_tuple, __pyx_kp_s_stringsource, __pyx_n_s_pyx_unpickle__Tag, 1, __pyx_empty_bytes); if (unlikely(!__pyx_codeobj__261)) __PYX_ERR(7, 1, __pyx_L1_error)
  __pyx_tuple__262 = PyTuple_Pack(5, __pyx_n_s_pyx_type, __pyx_n_s_pyx_checksum, __pyx_n_s_pyx_state, __pyx_n_s_pyx_PickleError, __pyx_n_s_pyx_result); if (unlikely(!__pyx_tuple__262)) __PYX_ERR(7, 1, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_tuple__262);
  __Pyx_GIVEREF(__pyx_tuple__262);
  __pyx_codeobj__263 = (PyObject*)__Pyx_PyCode_New(3, 0, 5, 0, CO_OPTIMIZED|CO_NEWLOCALS, __pyx_empty_bytes, __pyx_empty_tuple, __pyx_empty_tuple, __pyx_tuple__262, __pyx_empty_tuple, __pyx_empty_tuple, __pyx_kp_s_stringsource, __pyx_n_s_pyx_unpickle_CensusContext, 1, __pyx_empty_bytes); if (unlikely(!__pyx_codeobj__263)) __PYX_ERR(7, 1, __pyx_L1_error)
  __Pyx_RefNannyFinishContext();
  return 0;
  __pyx_L1_error:;
  __Pyx_RefNannyFinishContext();
  return -1;
}

static int __Pyx_InitGlobals(void) {
  __pyx_umethod_PyDict_Type_pop.type = (PyObject*)&PyDict_Type;
  __pyx_umethod_PyDict_Type_values.type = (PyObject*)&PyDict_Type;
  if (__Pyx_InitStrings(__pyx_string_tab) < 0) __PYX_ERR(21, 1, __pyx_L1_error);
  __pyx_float_1_0 = PyFloat_FromDouble(1.0); if (unlikely(!__pyx_float_1_0)) __PYX_ERR(21, 1, __pyx_L1_error)
  __pyx_int_0 = PyInt_FromLong(0); if (unlikely(!__pyx_int_0)) __PYX_ERR(21, 1, __pyx_L1_error)
  __pyx_int_1 = PyInt_FromLong(1); if (unlikely(!__pyx_int_1)) __PYX_ERR(21, 1, __pyx_L1_error)
  __pyx_int_5 = PyInt_FromLong(5); if (unlikely(!__pyx_int_5)) __PYX_ERR(21, 1, __pyx_L1_error)
  __pyx_int_50 = PyInt_FromLong(50); if (unlikely(!__pyx_int_50)) __PYX_ERR(21, 1, __pyx_L1_error)
  __pyx_int_222419149 = PyInt_FromLong(222419149L); if (unlikely(!__pyx_int_222419149)) __PYX_ERR(21, 1, __pyx_L1_error)
  return 0;
  __pyx_L1_error:;
  return -1;
}

static int __Pyx_modinit_global_init_code(void); /*proto*/
static int __Pyx_modinit_variable_export_code(void); /*proto*/
static int __Pyx_modinit_function_export_code(void); /*proto*/
static int __Pyx_modinit_type_init_code(void); /*proto*/
static int __Pyx_modinit_type_import_code(void); /*proto*/
static int __Pyx_modinit_variable_import_code(void); /*proto*/
static int __Pyx_modinit_function_import_code(void); /*proto*/

static int __Pyx_modinit_global_init_code(void) {
  __Pyx_RefNannyDeclarations
  __Pyx_RefNannySetupContext("__Pyx_modinit_global_init_code", 0);
  /*--- Global init code ---*/
  __Pyx_RefNannyFinishContext();
  return 0;
}

static int __Pyx_modinit_variable_export_code(void) {
  __Pyx_RefNannyDeclarations
  __Pyx_RefNannySetupContext("__Pyx_modinit_variable_export_code", 0);
  /*--- Variable export code ---*/
  __Pyx_RefNannyFinishContext();
  return 0;
}

static int __Pyx_modinit_function_export_code(void) {
  __Pyx_RefNannyDeclarations
  __Pyx_RefNannySetupContext("__Pyx_modinit_function_export_code", 0);
  /*--- Function export code ---*/
  if (__Pyx_ExportFunction("_wrap_grpc_arg", (void (*)(void))__pyx_f_4grpc_7_cython_6cygrpc__wrap_grpc_arg, "PyObject *(grpc_arg)") < 0) __PYX_ERR(21, 1, __pyx_L1_error)
  if (__Pyx_ExportFunction("_unwrap_grpc_arg", (void (*)(void))__pyx_f_4grpc_7_cython_6cygrpc__unwrap_grpc_arg, "grpc_arg (PyObject *)") < 0) __PYX_ERR(21, 1, __pyx_L1_error)
  if (__Pyx_ExportFunction("_check_call_error_no_metadata", (void (*)(void))__pyx_f_4grpc_7_cython_6cygrpc__check_call_error_no_metadata, "PyObject *(PyObject *)") < 0) __PYX_ERR(21, 1, __pyx_L1_error)
  if (__Pyx_ExportFunction("_check_and_raise_call_error_no_metadata", (void (*)(void))__pyx_f_4grpc_7_cython_6cygrpc__check_and_raise_call_error_no_metadata, "PyObject *(PyObject *)") < 0) __PYX_ERR(21, 1, __pyx_L1_error)
  if (__Pyx_ExportFunction("_check_call_error", (void (*)(void))__pyx_f_4grpc_7_cython_6cygrpc__check_call_error, "PyObject *(PyObject *, PyObject *)") < 0) __PYX_ERR(21, 1, __pyx_L1_error)
  if (__Pyx_ExportFunction("_get_metadata", (void (*)(void))__pyx_f_4grpc_7_cython_6cygrpc__get_metadata, "int (void *, grpc_auth_metadata_context, grpc_credentials_plugin_metadata_cb, void *, grpc_metadata *, size_t *, grpc_status_code *, char const **)") < 0) __PYX_ERR(21, 1, __pyx_L1_error)
  if (__Pyx_ExportFunction("_destroy", (void (*)(void))__pyx_f_4grpc_7_cython_6cygrpc__destroy, "void (void *)") < 0) __PYX_ERR(21, 1, __pyx_L1_error)
  if (__Pyx_ExportFunction("_composition", (void (*)(void))__pyx_f_4grpc_7_cython_6cygrpc__composition, "grpc_call_credentials *(PyObject *)") < 0) __PYX_ERR(21, 1, __pyx_L1_error)
  if (__Pyx_ExportFunction("_next", (void (*)(void))__pyx_f_4grpc_7_cython_6cygrpc__next, "grpc_event (grpc_completion_queue *, PyObject *)") < 0) __PYX_ERR(21, 1, __pyx_L1_error)
  if (__Pyx_ExportFunction("_interpret_event", (void (*)(void))__pyx_f_4grpc_7_cython_6cygrpc__interpret_event, "PyObject *(grpc_event)") < 0) __PYX_ERR(21, 1, __pyx_L1_error)
  if (__Pyx_ExportFunction("_store_c_metadata", (void (*)(void))__pyx_f_4grpc_7_cython_6cygrpc__store_c_metadata, "void (PyObject *, grpc_metadata **, size_t *)") < 0) __PYX_ERR(21, 1, __pyx_L1_error)
  if (__Pyx_ExportFunction("_release_c_metadata", (void (*)(void))__pyx_f_4grpc_7_cython_6cygrpc__release_c_metadata, "void (grpc_metadata *, int)") < 0) __PYX_ERR(21, 1, __pyx_L1_error)
  if (__Pyx_ExportFunction("_metadatum", (void (*)(void))__pyx_f_4grpc_7_cython_6cygrpc__metadatum, "PyObject *(grpc_slice, grpc_slice)") < 0) __PYX_ERR(21, 1, __pyx_L1_error)
  if (__Pyx_ExportFunction("_metadata", (void (*)(void))__pyx_f_4grpc_7_cython_6cygrpc__metadata, "PyObject *(grpc_metadata_array *)") < 0) __PYX_ERR(21, 1, __pyx_L1_error)
  if (__Pyx_ExportFunction("_slice_bytes", (void (*)(void))__pyx_f_4grpc_7_cython_6cygrpc__slice_bytes, "PyObject *(grpc_slice)") < 0) __PYX_ERR(21, 1, __pyx_L1_error)
  if (__Pyx_ExportFunction("_copy_slice", (void (*)(void))__pyx_f_4grpc_7_cython_6cygrpc__copy_slice, "grpc_slice (grpc_slice)") < 0) __PYX_ERR(21, 1, __pyx_L1_error)
  if (__Pyx_ExportFunction("_slice_from_bytes", (void (*)(void))__pyx_f_4grpc_7_cython_6cygrpc__slice_from_bytes, "grpc_slice (PyObject *)") < 0) __PYX_ERR(21, 1, __pyx_L1_error)
  if (__Pyx_ExportFunction("ssl_roots_override_callback", (void (*)(void))__pyx_f_4grpc_7_cython_6cygrpc_ssl_roots_override_callback, "grpc_ssl_roots_override_result (char **)") < 0) __PYX_ERR(21, 1, __pyx_L1_error)
  if (__Pyx_ExportFunction("_timespec_from_time", (void (*)(void))__pyx_f_4grpc_7_cython_6cygrpc__timespec_from_time, "gpr_timespec (PyObject *)") < 0) __PYX_ERR(21, 1, __pyx_L1_error)
  if (__Pyx_ExportFunction("_time_from_timespec", (void (*)(void))__pyx_f_4grpc_7_cython_6cygrpc__time_from_timespec, "double (gpr_timespec)") < 0) __PYX_ERR(21, 1, __pyx_L1_error)
  if (__Pyx_ExportFunction("_copy_pointer", (void (*)(void))__pyx_f_4grpc_7_cython_6cygrpc__copy_pointer, "void *(void *)") < 0) __PYX_ERR(21, 1, __pyx_L1_error)
  if (__Pyx_ExportFunction("_destroy_pointer", (void (*)(void))__pyx_f_4grpc_7_cython_6cygrpc__destroy_pointer, "void (void *)") < 0) __PYX_ERR(21, 1, __pyx_L1_error)
  if (__Pyx_ExportFunction("_compare_pointer", (void (*)(void))__pyx_f_4grpc_7_cython_6cygrpc__compare_pointer, "int (void *, void *)") < 0) __PYX_ERR(21, 1, __pyx_L1_error)
  if (__Pyx_ExportFunction("_custom_op_on_c_call", (void (*)(void))__pyx_f_4grpc_7_cython_6cygrpc__custom_op_on_c_call, "PyObject *(int, grpc_call *)") < 0) __PYX_ERR(21, 1, __pyx_L1_error)
  if (__Pyx_ExportFunction("__prefork", (void (*)(void))__pyx_f_4grpc_7_cython_6cygrpc___prefork, "void (void)") < 0) __PYX_ERR(21, 1, __pyx_L1_error)
  if (__Pyx_ExportFunction("__postfork_parent", (void (*)(void))__pyx_f_4grpc_7_cython_6cygrpc___postfork_parent, "void (void)") < 0) __PYX_ERR(21, 1, __pyx_L1_error)
  if (__Pyx_ExportFunction("__postfork_child", (void (*)(void))__pyx_f_4grpc_7_cython_6cygrpc___postfork_child, "void (void)") < 0) __PYX_ERR(21, 1, __pyx_L1_error)
  __Pyx_RefNannyFinishContext();
  return 0;
  __pyx_L1_error:;
  __Pyx_RefNannyFinishContext();
  return -1;
}

static int __Pyx_modinit_type_init_code(void) {
  __Pyx_RefNannyDeclarations
  __Pyx_RefNannySetupContext("__Pyx_modinit_type_init_code", 0);
  /*--- Type init code ---*/
  __pyx_vtabptr_4grpc_7_cython_6cygrpc__ChannelArg = &__pyx_vtable_4grpc_7_cython_6cygrpc__ChannelArg;
  __pyx_vtable_4grpc_7_cython_6cygrpc__ChannelArg.c = (void (*)(struct __pyx_obj_4grpc_7_cython_6cygrpc__ChannelArg *, PyObject *, struct __pyx_obj_4grpc_7_cython_6cygrpc__VTable *, PyObject *))__pyx_f_4grpc_7_cython_6cygrpc_11_ChannelArg_c;
  if (PyType_Ready(&__pyx_type_4grpc_7_cython_6cygrpc__ChannelArg) < 0) __PYX_ERR(2, 34, __pyx_L1_error)
  __pyx_type_4grpc_7_cython_6cygrpc__ChannelArg.tp_print = 0;
  if ((CYTHON_USE_TYPE_SLOTS && CYTHON_USE_PYTYPE_LOOKUP) && likely(!__pyx_type_4grpc_7_cython_6cygrpc__ChannelArg.tp_dictoffset && __pyx_type_4grpc_7_cython_6cygrpc__ChannelArg.tp_getattro == PyObject_GenericGetAttr)) {
    __pyx_type_4grpc_7_cython_6cygrpc__ChannelArg.tp_getattro = __Pyx_PyObject_GenericGetAttr;
  }
  if (__Pyx_SetVtable(__pyx_type_4grpc_7_cython_6cygrpc__ChannelArg.tp_dict, __pyx_vtabptr_4grpc_7_cython_6cygrpc__ChannelArg) < 0) __PYX_ERR(2, 34, __pyx_L1_error)
  if (PyObject_SetAttrString(__pyx_m, "_ChannelArg", (PyObject *)&__pyx_type_4grpc_7_cython_6cygrpc__ChannelArg) < 0) __PYX_ERR(2, 34, __pyx_L1_error)
  if (__Pyx_setup_reduce((PyObject*)&__pyx_type_4grpc_7_cython_6cygrpc__ChannelArg) < 0) __PYX_ERR(2, 34, __pyx_L1_error)
  __pyx_ptype_4grpc_7_cython_6cygrpc__ChannelArg = &__pyx_type_4grpc_7_cython_6cygrpc__ChannelArg;
  __pyx_vtabptr_4grpc_7_cython_6cygrpc__ChannelArgs = &__pyx_vtable_4grpc_7_cython_6cygrpc__ChannelArgs;
  __pyx_vtable_4grpc_7_cython_6cygrpc__ChannelArgs.c_args = (grpc_channel_args *(*)(struct __pyx_obj_4grpc_7_cython_6cygrpc__ChannelArgs *))__pyx_f_4grpc_7_cython_6cygrpc_12_ChannelArgs_c_args;
  if (PyType_Ready(&__pyx_type_4grpc_7_cython_6cygrpc__ChannelArgs) < 0) __PYX_ERR(2, 66, __pyx_L1_error)
  __pyx_type_4grpc_7_cython_6cygrpc__ChannelArgs.tp_print = 0;
  if ((CYTHON_USE_TYPE_SLOTS && CYTHON_USE_PYTYPE_LOOKUP) && likely(!__pyx_type_4grpc_7_cython_6cygrpc__ChannelArgs.tp_dictoffset && __pyx_type_4grpc_7_cython_6cygrpc__ChannelArgs.tp_getattro == PyObject_GenericGetAttr)) {
    __pyx_type_4grpc_7_cython_6cygrpc__ChannelArgs.tp_getattro = __Pyx_PyObject_GenericGetAttr;
  }
  if (__Pyx_SetVtable(__pyx_type_4grpc_7_cython_6cygrpc__ChannelArgs.tp_dict, __pyx_vtabptr_4grpc_7_cython_6cygrpc__ChannelArgs) < 0) __PYX_ERR(2, 66, __pyx_L1_error)
  if (PyObject_SetAttrString(__pyx_m, "_ChannelArgs", (PyObject *)&__pyx_type_4grpc_7_cython_6cygrpc__ChannelArgs) < 0) __PYX_ERR(2, 66, __pyx_L1_error)
  if (__Pyx_setup_reduce((PyObject*)&__pyx_type_4grpc_7_cython_6cygrpc__ChannelArgs) < 0) __PYX_ERR(2, 66, __pyx_L1_error)
  __pyx_ptype_4grpc_7_cython_6cygrpc__ChannelArgs = &__pyx_type_4grpc_7_cython_6cygrpc__ChannelArgs;
  if (PyType_Ready(&__pyx_type_4grpc_7_cython_6cygrpc_Call) < 0) __PYX_ERR(3, 18, __pyx_L1_error)
  __pyx_type_4grpc_7_cython_6cygrpc_Call.tp_print = 0;
  if ((CYTHON_USE_TYPE_SLOTS && CYTHON_USE_PYTYPE_LOOKUP) && likely(!__pyx_type_4grpc_7_cython_6cygrpc_Call.tp_dictoffset && __pyx_type_4grpc_7_cython_6cygrpc_Call.tp_getattro == PyObject_GenericGetAttr)) {
    __pyx_type_4grpc_7_cython_6cygrpc_Call.tp_getattro = __Pyx_PyObject_GenericGetAttr;
  }
  if (PyObject_SetAttrString(__pyx_m, "Call", (PyObject *)&__pyx_type_4grpc_7_cython_6cygrpc_Call) < 0) __PYX_ERR(3, 18, __pyx_L1_error)
  if (__Pyx_setup_reduce((PyObject*)&__pyx_type_4grpc_7_cython_6cygrpc_Call) < 0) __PYX_ERR(3, 18, __pyx_L1_error)
  __pyx_ptype_4grpc_7_cython_6cygrpc_Call = &__pyx_type_4grpc_7_cython_6cygrpc_Call;
  if (PyType_Ready(&__pyx_type_4grpc_7_cython_6cygrpc__CallState) < 0) __PYX_ERR(8, 73, __pyx_L1_error)
  __pyx_type_4grpc_7_cython_6cygrpc__CallState.tp_print = 0;
  if ((CYTHON_USE_TYPE_SLOTS && CYTHON_USE_PYTYPE_LOOKUP) && likely(!__pyx_type_4grpc_7_cython_6cygrpc__CallState.tp_dictoffset && __pyx_type_4grpc_7_cython_6cygrpc__CallState.tp_getattro == PyObject_GenericGetAttr)) {
    __pyx_type_4grpc_7_cython_6cygrpc__CallState.tp_getattro = __Pyx_PyObject_GenericGetAttr;
  }
  if (PyObject_SetAttrString(__pyx_m, "_CallState", (PyObject *)&__pyx_type_4grpc_7_cython_6cygrpc__CallState) < 0) __PYX_ERR(8, 73, __pyx_L1_error)
  if (__Pyx_setup_reduce((PyObject*)&__pyx_type_4grpc_7_cython_6cygrpc__CallState) < 0) __PYX_ERR(8, 73, __pyx_L1_error)
  __pyx_ptype_4grpc_7_cython_6cygrpc__CallState = &__pyx_type_4grpc_7_cython_6cygrpc__CallState;
  if (PyType_Ready(&__pyx_type_4grpc_7_cython_6cygrpc__ChannelState) < 0) __PYX_ERR(8, 79, __pyx_L1_error)
  __pyx_type_4grpc_7_cython_6cygrpc__ChannelState.tp_print = 0;
  if ((CYTHON_USE_TYPE_SLOTS && CYTHON_USE_PYTYPE_LOOKUP) && likely(!__pyx_type_4grpc_7_cython_6cygrpc__ChannelState.tp_dictoffset && __pyx_type_4grpc_7_cython_6cygrpc__ChannelState.tp_getattro == PyObject_GenericGetAttr)) {
    __pyx_type_4grpc_7_cython_6cygrpc__ChannelState.tp_getattro = __Pyx_PyObject_GenericGetAttr;
  }
  if (PyObject_SetAttrString(__pyx_m, "_ChannelState", (PyObject *)&__pyx_type_4grpc_7_cython_6cygrpc__ChannelState) < 0) __PYX_ERR(8, 79, __pyx_L1_error)
  if (__Pyx_setup_reduce((PyObject*)&__pyx_type_4grpc_7_cython_6cygrpc__ChannelState) < 0) __PYX_ERR(8, 79, __pyx_L1_error)
  __pyx_ptype_4grpc_7_cython_6cygrpc__ChannelState = &__pyx_type_4grpc_7_cython_6cygrpc__ChannelState;
  if (PyType_Ready(&__pyx_type_4grpc_7_cython_6cygrpc_IntegratedCall) < 0) __PYX_ERR(8, 248, __pyx_L1_error)
  __pyx_type_4grpc_7_cython_6cygrpc_IntegratedCall.tp_print = 0;
  if ((CYTHON_USE_TYPE_SLOTS && CYTHON_USE_PYTYPE_LOOKUP) && likely(!__pyx_type_4grpc_7_cython_6cygrpc_IntegratedCall.tp_dictoffset && __pyx_type_4grpc_7_cython_6cygrpc_IntegratedCall.tp_getattro == PyObject_GenericGetAttr)) {
    __pyx_type_4grpc_7_cython_6cygrpc_IntegratedCall.tp_getattro = __Pyx_PyObject_GenericGetAttr;
  }
  if (PyObject_SetAttrString(__pyx_m, "IntegratedCall", (PyObject *)&__pyx_type_4grpc_7_cython_6cygrpc_IntegratedCall) < 0) __PYX_ERR(8, 248, __pyx_L1_error)
  if (__Pyx_setup_reduce((PyObject*)&__pyx_type_4grpc_7_cython_6cygrpc_IntegratedCall) < 0) __PYX_ERR(8, 248, __pyx_L1_error)
  __pyx_ptype_4grpc_7_cython_6cygrpc_IntegratedCall = &__pyx_type_4grpc_7_cython_6cygrpc_IntegratedCall;
  if (PyType_Ready(&__pyx_type_4grpc_7_cython_6cygrpc_SegregatedCall) < 0) __PYX_ERR(8, 293, __pyx_L1_error)
  __pyx_type_4grpc_7_cython_6cygrpc_SegregatedCall.tp_print = 0;
  if ((CYTHON_USE_TYPE_SLOTS && CYTHON_USE_PYTYPE_LOOKUP) && likely(!__pyx_type_4grpc_7_cython_6cygrpc_SegregatedCall.tp_dictoffset && __pyx_type_4grpc_7_cython_6cygrpc_SegregatedCall.tp_getattro == PyObject_GenericGetAttr)) {
    __pyx_type_4grpc_7_cython_6cygrpc_SegregatedCall.tp_getattro = __Pyx_PyObject_GenericGetAttr;
  }
  if (PyObject_SetAttrString(__pyx_m, "SegregatedCall", (PyObject *)&__pyx_type_4grpc_7_cython_6cygrpc_SegregatedCall) < 0) __PYX_ERR(8, 293, __pyx_L1_error)
  if (__Pyx_setup_reduce((PyObject*)&__pyx_type_4grpc_7_cython_6cygrpc_SegregatedCall) < 0) __PYX_ERR(8, 293, __pyx_L1_error)
  __pyx_ptype_4grpc_7_cython_6cygrpc_SegregatedCall = &__pyx_type_4grpc_7_cython_6cygrpc_SegregatedCall;
  if (PyType_Ready(&__pyx_type_4grpc_7_cython_6cygrpc_Channel) < 0) __PYX_ERR(8, 415, __pyx_L1_error)
  __pyx_type_4grpc_7_cython_6cygrpc_Channel.tp_print = 0;
  if ((CYTHON_USE_TYPE_SLOTS && CYTHON_USE_PYTYPE_LOOKUP) && likely(!__pyx_type_4grpc_7_cython_6cygrpc_Channel.tp_dictoffset && __pyx_type_4grpc_7_cython_6cygrpc_Channel.tp_getattro == PyObject_GenericGetAttr)) {
    __pyx_type_4grpc_7_cython_6cygrpc_Channel.tp_getattro = __Pyx_PyObject_GenericGetAttr;
  }
  if (PyObject_SetAttrString(__pyx_m, "Channel", (PyObject *)&__pyx_type_4grpc_7_cython_6cygrpc_Channel) < 0) __PYX_ERR(8, 415, __pyx_L1_error)
  if (__Pyx_setup_reduce((PyObject*)&__pyx_type_4grpc_7_cython_6cygrpc_Channel) < 0) __PYX_ERR(8, 415, __pyx_L1_error)
  __pyx_ptype_4grpc_7_cython_6cygrpc_Channel = &__pyx_type_4grpc_7_cython_6cygrpc_Channel;
  __pyx_vtabptr_4grpc_7_cython_6cygrpc_CallCredentials = &__pyx_vtable_4grpc_7_cython_6cygrpc_CallCredentials;
  __pyx_vtable_4grpc_7_cython_6cygrpc_CallCredentials.c = (grpc_call_credentials *(*)(struct __pyx_obj_4grpc_7_cython_6cygrpc_CallCredentials *))__pyx_f_4grpc_7_cython_6cygrpc_15CallCredentials_c;
  if (PyType_Ready(&__pyx_type_4grpc_7_cython_6cygrpc_CallCredentials) < 0) __PYX_ERR(4, 34, __pyx_L1_error)
  __pyx_type_4grpc_7_cython_6cygrpc_CallCredentials.tp_print = 0;
  if ((CYTHON_USE_TYPE_SLOTS && CYTHON_USE_PYTYPE_LOOKUP) && likely(!__pyx_type_4grpc_7_cython_6cygrpc_CallCredentials.tp_dictoffset && __pyx_type_4grpc_7_cython_6cygrpc_CallCredentials.tp_getattro == PyObject_GenericGetAttr)) {
    __pyx_type_4grpc_7_cython_6cygrpc_CallCredentials.tp_getattro = __Pyx_PyObject_GenericGetAttr;
  }
  if (__Pyx_SetVtable(__pyx_type_4grpc_7_cython_6cygrpc_CallCredentials.tp_dict, __pyx_vtabptr_4grpc_7_cython_6cygrpc_CallCredentials) < 0) __PYX_ERR(4, 34, __pyx_L1_error)
  if (PyObject_SetAttrString(__pyx_m, "CallCredentials", (PyObject *)&__pyx_type_4grpc_7_cython_6cygrpc_CallCredentials) < 0) __PYX_ERR(4, 34, __pyx_L1_error)
  if (__Pyx_setup_reduce((PyObject*)&__pyx_type_4grpc_7_cython_6cygrpc_CallCredentials) < 0) __PYX_ERR(4, 34, __pyx_L1_error)
  __pyx_ptype_4grpc_7_cython_6cygrpc_CallCredentials = &__pyx_type_4grpc_7_cython_6cygrpc_CallCredentials;
  __pyx_vtabptr_4grpc_7_cython_6cygrpc_MetadataPluginCallCredentials = &__pyx_vtable_4grpc_7_cython_6cygrpc_MetadataPluginCallCredentials;
  __pyx_vtable_4grpc_7_cython_6cygrpc_MetadataPluginCallCredentials.__pyx_base = *__pyx_vtabptr_4grpc_7_cython_6cygrpc_CallCredentials;
  __pyx_vtable_4grpc_7_cython_6cygrpc_MetadataPluginCallCredentials.__pyx_base.c = (grpc_call_credentials *(*)(struct __pyx_obj_4grpc_7_cython_6cygrpc_CallCredentials *))__pyx_f_4grpc_7_cython_6cygrpc_29MetadataPluginCallCredentials_c;
  __pyx_type_4grpc_7_cython_6cygrpc_MetadataPluginCallCredentials.tp_base = __pyx_ptype_4grpc_7_cython_6cygrpc_CallCredentials;
  if (PyType_Ready(&__pyx_type_4grpc_7_cython_6cygrpc_MetadataPluginCallCredentials) < 0) __PYX_ERR(4, 65, __pyx_L1_error)
  __pyx_type_4grpc_7_cython_6cygrpc_MetadataPluginCallCredentials.tp_print = 0;
  if ((CYTHON_USE_TYPE_SLOTS && CYTHON_USE_PYTYPE_LOOKUP) && likely(!__pyx_type_4grpc_7_cython_6cygrpc_MetadataPluginCallCredentials.tp_dictoffset && __pyx_type_4grpc_7_cython_6cygrpc_MetadataPluginCallCredentials.tp_getattro == PyObject_GenericGetAttr)) {
    __pyx_type_4grpc_7_cython_6cygrpc_MetadataPluginCallCredentials.tp_getattro = __Pyx_PyObject_GenericGetAttr;
  }
  if (__Pyx_SetVtable(__pyx_type_4grpc_7_cython_6cygrpc_MetadataPluginCallCredentials.tp_dict, __pyx_vtabptr_4grpc_7_cython_6cygrpc_MetadataPluginCallCredentials) < 0) __PYX_ERR(4, 65, __pyx_L1_error)
  if (PyObject_SetAttrString(__pyx_m, "MetadataPluginCallCredentials", (PyObject *)&__pyx_type_4grpc_7_cython_6cygrpc_MetadataPluginCallCredentials) < 0) __PYX_ERR(4, 65, __pyx_L1_error)
  if (__Pyx_setup_reduce((PyObject*)&__pyx_type_4grpc_7_cython_6cygrpc_MetadataPluginCallCredentials) < 0) __PYX_ERR(4, 65, __pyx_L1_error)
  __pyx_ptype_4grpc_7_cython_6cygrpc_MetadataPluginCallCredentials = &__pyx_type_4grpc_7_cython_6cygrpc_MetadataPluginCallCredentials;
  __pyx_vtabptr_4grpc_7_cython_6cygrpc_CompositeCallCredentials = &__pyx_vtable_4grpc_7_cython_6cygrpc_CompositeCallCredentials;
  __pyx_vtable_4grpc_7_cython_6cygrpc_CompositeCallCredentials.__pyx_base = *__pyx_vtabptr_4grpc_7_cython_6cygrpc_CallCredentials;
  __pyx_vtable_4grpc_7_cython_6cygrpc_CompositeCallCredentials.__pyx_base.c = (grpc_call_credentials *(*)(struct __pyx_obj_4grpc_7_cython_6cygrpc_CallCredentials *))__pyx_f_4grpc_7_cython_6cygrpc_24CompositeCallCredentials_c;
  __pyx_type_4grpc_7_cython_6cygrpc_CompositeCallCredentials.tp_base = __pyx_ptype_4grpc_7_cython_6cygrpc_CallCredentials;
  if (PyType_Ready(&__pyx_type_4grpc_7_cython_6cygrpc_CompositeCallCredentials) < 0) __PYX_ERR(4, 99, __pyx_L1_error)
  __pyx_type_4grpc_7_cython_6cygrpc_CompositeCallCredentials.tp_print = 0;
  if ((CYTHON_USE_TYPE_SLOTS && CYTHON_USE_PYTYPE_LOOKUP) && likely(!__pyx_type_4grpc_7_cython_6cygrpc_CompositeCallCredentials.tp_dictoffset && __pyx_type_4grpc_7_cython_6cygrpc_CompositeCallCredentials.tp_getattro == PyObject_GenericGetAttr)) {
    __pyx_type_4grpc_7_cython_6cygrpc_CompositeCallCredentials.tp_getattro = __Pyx_PyObject_GenericGetAttr;
  }
  if (__Pyx_SetVtable(__pyx_type_4grpc_7_cython_6cygrpc_CompositeCallCredentials.tp_dict, __pyx_vtabptr_4grpc_7_cython_6cygrpc_CompositeCallCredentials) < 0) __PYX_ERR(4, 99, __pyx_L1_error)
  if (PyObject_SetAttrString(__pyx_m, "CompositeCallCredentials", (PyObject *)&__pyx_type_4grpc_7_cython_6cygrpc_CompositeCallCredentials) < 0) __PYX_ERR(4, 99, __pyx_L1_error)
  if (__Pyx_setup_reduce((PyObject*)&__pyx_type_4grpc_7_cython_6cygrpc_CompositeCallCredentials) < 0) __PYX_ERR(4, 99, __pyx_L1_error)
  __pyx_ptype_4grpc_7_cython_6cygrpc_CompositeCallCredentials = &__pyx_type_4grpc_7_cython_6cygrpc_CompositeCallCredentials;
  __pyx_vtabptr_4grpc_7_cython_6cygrpc_ChannelCredentials = &__pyx_vtable_4grpc_7_cython_6cygrpc_ChannelCredentials;
  __pyx_vtable_4grpc_7_cython_6cygrpc_ChannelCredentials.c = (grpc_channel_credentials *(*)(struct __pyx_obj_4grpc_7_cython_6cygrpc_ChannelCredentials *))__pyx_f_4grpc_7_cython_6cygrpc_18ChannelCredentials_c;
  if (PyType_Ready(&__pyx_type_4grpc_7_cython_6cygrpc_ChannelCredentials) < 0) __PYX_ERR(4, 108, __pyx_L1_error)
  __pyx_type_4grpc_7_cython_6cygrpc_ChannelCredentials.tp_print = 0;
  if ((CYTHON_USE_TYPE_SLOTS && CYTHON_USE_PYTYPE_LOOKUP) && likely(!__pyx_type_4grpc_7_cython_6cygrpc_ChannelCredentials.tp_dictoffset && __pyx_type_4grpc_7_cython_6cygrpc_ChannelCredentials.tp_getattro == PyObject_GenericGetAttr)) {
    __pyx_type_4grpc_7_cython_6cygrpc_ChannelCredentials.tp_getattro = __Pyx_PyObject_GenericGetAttr;
  }
  if (__Pyx_SetVtable(__pyx_type_4grpc_7_cython_6cygrpc_ChannelCredentials.tp_dict, __pyx_vtabptr_4grpc_7_cython_6cygrpc_ChannelCredentials) < 0) __PYX_ERR(4, 108, __pyx_L1_error)
  if (PyObject_SetAttrString(__pyx_m, "ChannelCredentials", (PyObject *)&__pyx_type_4grpc_7_cython_6cygrpc_ChannelCredentials) < 0) __PYX_ERR(4, 108, __pyx_L1_error)
  if (__Pyx_setup_reduce((PyObject*)&__pyx_type_4grpc_7_cython_6cygrpc_ChannelCredentials) < 0) __PYX_ERR(4, 108, __pyx_L1_error)
  __pyx_ptype_4grpc_7_cython_6cygrpc_ChannelCredentials = &__pyx_type_4grpc_7_cython_6cygrpc_ChannelCredentials;
  if (PyType_Ready(&__pyx_type_4grpc_7_cython_6cygrpc_SSLSessionCacheLRU) < 0) __PYX_ERR(4, 114, __pyx_L1_error)
  __pyx_type_4grpc_7_cython_6cygrpc_SSLSessionCacheLRU.tp_print = 0;
  if ((CYTHON_USE_TYPE_SLOTS && CYTHON_USE_PYTYPE_LOOKUP) && likely(!__pyx_type_4grpc_7_cython_6cygrpc_SSLSessionCacheLRU.tp_dictoffset && __pyx_type_4grpc_7_cython_6cygrpc_SSLSessionCacheLRU.tp_getattro == PyObject_GenericGetAttr)) {
    __pyx_type_4grpc_7_cython_6cygrpc_SSLSessionCacheLRU.tp_getattro = __Pyx_PyObject_GenericGetAttr;
  }
  if (PyObject_SetAttrString(__pyx_m, "SSLSessionCacheLRU", (PyObject *)&__pyx_type_4grpc_7_cython_6cygrpc_SSLSessionCacheLRU) < 0) __PYX_ERR(4, 114, __pyx_L1_error)
  if (__Pyx_setup_reduce((PyObject*)&__pyx_type_4grpc_7_cython_6cygrpc_SSLSessionCacheLRU) < 0) __PYX_ERR(4, 114, __pyx_L1_error)
  __pyx_ptype_4grpc_7_cython_6cygrpc_SSLSessionCacheLRU = &__pyx_type_4grpc_7_cython_6cygrpc_SSLSessionCacheLRU;
  __pyx_vtabptr_4grpc_7_cython_6cygrpc_SSLChannelCredentials = &__pyx_vtable_4grpc_7_cython_6cygrpc_SSLChannelCredentials;
  __pyx_vtable_4grpc_7_cython_6cygrpc_SSLChannelCredentials.__pyx_base = *__pyx_vtabptr_4grpc_7_cython_6cygrpc_ChannelCredentials;
  __pyx_vtable_4grpc_7_cython_6cygrpc_SSLChannelCredentials.__pyx_base.c = (grpc_channel_credentials *(*)(struct __pyx_obj_4grpc_7_cython_6cygrpc_ChannelCredentials *))__pyx_f_4grpc_7_cython_6cygrpc_21SSLChannelCredentials_c;
  __pyx_type_4grpc_7_cython_6cygrpc_SSLChannelCredentials.tp_base = __pyx_ptype_4grpc_7_cython_6cygrpc_ChannelCredentials;
  if (PyType_Ready(&__pyx_type_4grpc_7_cython_6cygrpc_SSLChannelCredentials) < 0) __PYX_ERR(4, 129, __pyx_L1_error)
  __pyx_type_4grpc_7_cython_6cygrpc_SSLChannelCredentials.tp_print = 0;
  if ((CYTHON_USE_TYPE_SLOTS && CYTHON_USE_PYTYPE_LOOKUP) && likely(!__pyx_type_4grpc_7_cython_6cygrpc_SSLChannelCredentials.tp_dictoffset && __pyx_type_4grpc_7_cython_6cygrpc_SSLChannelCredentials.tp_getattro == PyObject_GenericGetAttr)) {
    __pyx_type_4grpc_7_cython_6cygrpc_SSLChannelCredentials.tp_getattro = __Pyx_PyObject_GenericGetAttr;
  }
  if (__Pyx_SetVtable(__pyx_type_4grpc_7_cython_6cygrpc_SSLChannelCredentials.tp_dict, __pyx_vtabptr_4grpc_7_cython_6cygrpc_SSLChannelCredentials) < 0) __PYX_ERR(4, 129, __pyx_L1_error)
  if (PyObject_SetAttrString(__pyx_m, "SSLChannelCredentials", (PyObject *)&__pyx_type_4grpc_7_cython_6cygrpc_SSLChannelCredentials) < 0) __PYX_ERR(4, 129, __pyx_L1_error)
  if (__Pyx_setup_reduce((PyObject*)&__pyx_type_4grpc_7_cython_6cygrpc_SSLChannelCredentials) < 0) __PYX_ERR(4, 129, __pyx_L1_error)
  __pyx_ptype_4grpc_7_cython_6cygrpc_SSLChannelCredentials = &__pyx_type_4grpc_7_cython_6cygrpc_SSLChannelCredentials;
  __pyx_vtabptr_4grpc_7_cython_6cygrpc_CompositeChannelCredentials = &__pyx_vtable_4grpc_7_cython_6cygrpc_CompositeChannelCredentials;
  __pyx_vtable_4grpc_7_cython_6cygrpc_CompositeChannelCredentials.__pyx_base = *__pyx_vtabptr_4grpc_7_cython_6cygrpc_ChannelCredentials;
  __pyx_vtable_4grpc_7_cython_6cygrpc_CompositeChannelCredentials.__pyx_base.c = (grpc_channel_credentials *(*)(struct __pyx_obj_4grpc_7_cython_6cygrpc_ChannelCredentials *))__pyx_f_4grpc_7_cython_6cygrpc_27CompositeChannelCredentials_c;
  __pyx_type_4grpc_7_cython_6cygrpc_CompositeChannelCredentials.tp_base = __pyx_ptype_4grpc_7_cython_6cygrpc_ChannelCredentials;
  if (PyType_Ready(&__pyx_type_4grpc_7_cython_6cygrpc_CompositeChannelCredentials) < 0) __PYX_ERR(4, 161, __pyx_L1_error)
  __pyx_type_4grpc_7_cython_6cygrpc_CompositeChannelCredentials.tp_print = 0;
  if ((CYTHON_USE_TYPE_SLOTS && CYTHON_USE_PYTYPE_LOOKUP) && likely(!__pyx_type_4grpc_7_cython_6cygrpc_CompositeChannelCredentials.tp_dictoffset && __pyx_type_4grpc_7_cython_6cygrpc_CompositeChannelCredentials.tp_getattro == PyObject_GenericGetAttr)) {
    __pyx_type_4grpc_7_cython_6cygrpc_CompositeChannelCredentials.tp_getattro = __Pyx_PyObject_GenericGetAttr;
  }
  if (__Pyx_SetVtable(__pyx_type_4grpc_7_cython_6cygrpc_CompositeChannelCredentials.tp_dict, __pyx_vtabptr_4grpc_7_cython_6cygrpc_CompositeChannelCredentials) < 0) __PYX_ERR(4, 161, __pyx_L1_error)
  if (PyObject_SetAttrString(__pyx_m, "CompositeChannelCredentials", (PyObject *)&__pyx_type_4grpc_7_cython_6cygrpc_CompositeChannelCredentials) < 0) __PYX_ERR(4, 161, __pyx_L1_error)
  if (__Pyx_setup_reduce((PyObject*)&__pyx_type_4grpc_7_cython_6cygrpc_CompositeChannelCredentials) < 0) __PYX_ERR(4, 161, __pyx_L1_error)
  __pyx_ptype_4grpc_7_cython_6cygrpc_CompositeChannelCredentials = &__pyx_type_4grpc_7_cython_6cygrpc_CompositeChannelCredentials;
  if (PyType_Ready(&__pyx_type_4grpc_7_cython_6cygrpc_ServerCertificateConfig) < 0) __PYX_ERR(4, 180, __pyx_L1_error)
  __pyx_type_4grpc_7_cython_6cygrpc_ServerCertificateConfig.tp_print = 0;
  if ((CYTHON_USE_TYPE_SLOTS && CYTHON_USE_PYTYPE_LOOKUP) && likely(!__pyx_type_4grpc_7_cython_6cygrpc_ServerCertificateConfig.tp_dictoffset && __pyx_type_4grpc_7_cython_6cygrpc_ServerCertificateConfig.tp_getattro == PyObject_GenericGetAttr)) {
    __pyx_type_4grpc_7_cython_6cygrpc_ServerCertificateConfig.tp_getattro = __Pyx_PyObject_GenericGetAttr;
  }
  if (PyObject_SetAttrString(__pyx_m, "ServerCertificateConfig", (PyObject *)&__pyx_type_4grpc_7_cython_6cygrpc_ServerCertificateConfig) < 0) __PYX_ERR(4, 180, __pyx_L1_error)
  if (__Pyx_setup_reduce((PyObject*)&__pyx_type_4grpc_7_cython_6cygrpc_ServerCertificateConfig) < 0) __PYX_ERR(4, 180, __pyx_L1_error)
  __pyx_ptype_4grpc_7_cython_6cygrpc_ServerCertificateConfig = &__pyx_type_4grpc_7_cython_6cygrpc_ServerCertificateConfig;
  if (PyType_Ready(&__pyx_type_4grpc_7_cython_6cygrpc_ServerCredentials) < 0) __PYX_ERR(4, 195, __pyx_L1_error)
  __pyx_type_4grpc_7_cython_6cygrpc_ServerCredentials.tp_print = 0;
  if ((CYTHON_USE_TYPE_SLOTS && CYTHON_USE_PYTYPE_LOOKUP) && likely(!__pyx_type_4grpc_7_cython_6cygrpc_ServerCredentials.tp_dictoffset && __pyx_type_4grpc_7_cython_6cygrpc_ServerCredentials.tp_getattro == PyObject_GenericGetAttr)) {
    __pyx_type_4grpc_7_cython_6cygrpc_ServerCredentials.tp_getattro = __Pyx_PyObject_GenericGetAttr;
  }
  if (PyObject_SetAttrString(__pyx_m, "ServerCredentials", (PyObject *)&__pyx_type_4grpc_7_cython_6cygrpc_ServerCredentials) < 0) __PYX_ERR(4, 195, __pyx_L1_error)
  if (__Pyx_setup_reduce((PyObject*)&__pyx_type_4grpc_7_cython_6cygrpc_ServerCredentials) < 0) __PYX_ERR(4, 195, __pyx_L1_error)
  __pyx_ptype_4grpc_7_cython_6cygrpc_ServerCredentials = &__pyx_type_4grpc_7_cython_6cygrpc_ServerCredentials;
  __pyx_vtabptr_4grpc_7_cython_6cygrpc_CompletionQueue = &__pyx_vtable_4grpc_7_cython_6cygrpc_CompletionQueue;
  __pyx_vtable_4grpc_7_cython_6cygrpc_CompletionQueue._interpret_event = (PyObject *(*)(struct __pyx_obj_4grpc_7_cython_6cygrpc_CompletionQueue *, grpc_event))__pyx_f_4grpc_7_cython_6cygrpc_15CompletionQueue__interpret_event;
  if (PyType_Ready(&__pyx_type_4grpc_7_cython_6cygrpc_CompletionQueue) < 0) __PYX_ERR(10, 71, __pyx_L1_error)
  __pyx_type_4grpc_7_cython_6cygrpc_CompletionQueue.tp_print = 0;
  if ((CYTHON_USE_TYPE_SLOTS && CYTHON_USE_PYTYPE_LOOKUP) && likely(!__pyx_type_4grpc_7_cython_6cygrpc_CompletionQueue.tp_dictoffset && __pyx_type_4grpc_7_cython_6cygrpc_CompletionQueue.tp_getattro == PyObject_GenericGetAttr)) {
    __pyx_type_4grpc_7_cython_6cygrpc_CompletionQueue.tp_getattro = __Pyx_PyObject_GenericGetAttr;
  }
  if (__Pyx_SetVtable(__pyx_type_4grpc_7_cython_6cygrpc_CompletionQueue.tp_dict, __pyx_vtabptr_4grpc_7_cython_6cygrpc_CompletionQueue) < 0) __PYX_ERR(10, 71, __pyx_L1_error)
  if (PyObject_SetAttrString(__pyx_m, "CompletionQueue", (PyObject *)&__pyx_type_4grpc_7_cython_6cygrpc_CompletionQueue) < 0) __PYX_ERR(10, 71, __pyx_L1_error)
  if (__Pyx_setup_reduce((PyObject*)&__pyx_type_4grpc_7_cython_6cygrpc_CompletionQueue) < 0) __PYX_ERR(10, 71, __pyx_L1_error)
  __pyx_ptype_4grpc_7_cython_6cygrpc_CompletionQueue = &__pyx_type_4grpc_7_cython_6cygrpc_CompletionQueue;
  if (PyType_Ready(&__pyx_type_4grpc_7_cython_6cygrpc_ConnectivityEvent) < 0) __PYX_ERR(11, 16, __pyx_L1_error)
  __pyx_type_4grpc_7_cython_6cygrpc_ConnectivityEvent.tp_print = 0;
  if ((CYTHON_USE_TYPE_SLOTS && CYTHON_USE_PYTYPE_LOOKUP) && likely(!__pyx_type_4grpc_7_cython_6cygrpc_ConnectivityEvent.tp_dictoffset && __pyx_type_4grpc_7_cython_6cygrpc_ConnectivityEvent.tp_getattro == PyObject_GenericGetAttr)) {
    __pyx_type_4grpc_7_cython_6cygrpc_ConnectivityEvent.tp_getattro = __Pyx_PyObject_GenericGetAttr;
  }
  if (PyObject_SetAttrString(__pyx_m, "ConnectivityEvent", (PyObject *)&__pyx_type_4grpc_7_cython_6cygrpc_ConnectivityEvent) < 0) __PYX_ERR(11, 16, __pyx_L1_error)
  if (__Pyx_setup_reduce((PyObject*)&__pyx_type_4grpc_7_cython_6cygrpc_ConnectivityEvent) < 0) __PYX_ERR(11, 16, __pyx_L1_error)
  __pyx_ptype_4grpc_7_cython_6cygrpc_ConnectivityEvent = &__pyx_type_4grpc_7_cython_6cygrpc_ConnectivityEvent;
  if (PyType_Ready(&__pyx_type_4grpc_7_cython_6cygrpc_RequestCallEvent) < 0) __PYX_ERR(11, 25, __pyx_L1_error)
  __pyx_type_4grpc_7_cython_6cygrpc_RequestCallEvent.tp_print = 0;
  if ((CYTHON_USE_TYPE_SLOTS && CYTHON_USE_PYTYPE_LOOKUP) && likely(!__pyx_type_4grpc_7_cython_6cygrpc_RequestCallEvent.tp_dictoffset && __pyx_type_4grpc_7_cython_6cygrpc_RequestCallEvent.tp_getattro == PyObject_GenericGetAttr)) {
    __pyx_type_4grpc_7_cython_6cygrpc_RequestCallEvent.tp_getattro = __Pyx_PyObject_GenericGetAttr;
  }
  if (PyObject_SetAttrString(__pyx_m, "RequestCallEvent", (PyObject *)&__pyx_type_4grpc_7_cython_6cygrpc_RequestCallEvent) < 0) __PYX_ERR(11, 25, __pyx_L1_error)
  if (__Pyx_setup_reduce((PyObject*)&__pyx_type_4grpc_7_cython_6cygrpc_RequestCallEvent) < 0) __PYX_ERR(11, 25, __pyx_L1_error)
  __pyx_ptype_4grpc_7_cython_6cygrpc_RequestCallEvent = &__pyx_type_4grpc_7_cython_6cygrpc_RequestCallEvent;
  if (PyType_Ready(&__pyx_type_4grpc_7_cython_6cygrpc_BatchOperationEvent) < 0) __PYX_ERR(11, 38, __pyx_L1_error)
  __pyx_type_4grpc_7_cython_6cygrpc_BatchOperationEvent.tp_print = 0;
  if ((CYTHON_USE_TYPE_SLOTS && CYTHON_USE_PYTYPE_LOOKUP) && likely(!__pyx_type_4grpc_7_cython_6cygrpc_BatchOperationEvent.tp_dictoffset && __pyx_type_4grpc_7_cython_6cygrpc_BatchOperationEvent.tp_getattro == PyObject_GenericGetAttr)) {
    __pyx_type_4grpc_7_cython_6cygrpc_BatchOperationEvent.tp_getattro = __Pyx_PyObject_GenericGetAttr;
  }
  if (PyObject_SetAttrString(__pyx_m, "BatchOperationEvent", (PyObject *)&__pyx_type_4grpc_7_cython_6cygrpc_BatchOperationEvent) < 0) __PYX_ERR(11, 38, __pyx_L1_error)
  if (__Pyx_setup_reduce((PyObject*)&__pyx_type_4grpc_7_cython_6cygrpc_BatchOperationEvent) < 0) __PYX_ERR(11, 38, __pyx_L1_error)
  __pyx_ptype_4grpc_7_cython_6cygrpc_BatchOperationEvent = &__pyx_type_4grpc_7_cython_6cygrpc_BatchOperationEvent;
  if (PyType_Ready(&__pyx_type_4grpc_7_cython_6cygrpc_ServerShutdownEvent) < 0) __PYX_ERR(11, 49, __pyx_L1_error)
  __pyx_type_4grpc_7_cython_6cygrpc_ServerShutdownEvent.tp_print = 0;
  if ((CYTHON_USE_TYPE_SLOTS && CYTHON_USE_PYTYPE_LOOKUP) && likely(!__pyx_type_4grpc_7_cython_6cygrpc_ServerShutdownEvent.tp_dictoffset && __pyx_type_4grpc_7_cython_6cygrpc_ServerShutdownEvent.tp_getattro == PyObject_GenericGetAttr)) {
    __pyx_type_4grpc_7_cython_6cygrpc_ServerShutdownEvent.tp_getattro = __Pyx_PyObject_GenericGetAttr;
  }
  if (PyObject_SetAttrString(__pyx_m, "ServerShutdownEvent", (PyObject *)&__pyx_type_4grpc_7_cython_6cygrpc_ServerShutdownEvent) < 0) __PYX_ERR(11, 49, __pyx_L1_error)
  if (__Pyx_setup_reduce((PyObject*)&__pyx_type_4grpc_7_cython_6cygrpc_ServerShutdownEvent) < 0) __PYX_ERR(11, 49, __pyx_L1_error)
  __pyx_ptype_4grpc_7_cython_6cygrpc_ServerShutdownEvent = &__pyx_type_4grpc_7_cython_6cygrpc_ServerShutdownEvent;
  __pyx_vtabptr_4grpc_7_cython_6cygrpc_Operation = &__pyx_vtable_4grpc_7_cython_6cygrpc_Operation;
  __pyx_vtable_4grpc_7_cython_6cygrpc_Operation.c = (void (*)(struct __pyx_obj_4grpc_7_cython_6cygrpc_Operation *))__pyx_f_4grpc_7_cython_6cygrpc_9Operation_c;
  __pyx_vtable_4grpc_7_cython_6cygrpc_Operation.un_c = (void (*)(struct __pyx_obj_4grpc_7_cython_6cygrpc_Operation *))__pyx_f_4grpc_7_cython_6cygrpc_9Operation_un_c;
  if (PyType_Ready(&__pyx_type_4grpc_7_cython_6cygrpc_Operation) < 0) __PYX_ERR(14, 16, __pyx_L1_error)
  __pyx_type_4grpc_7_cython_6cygrpc_Operation.tp_print = 0;
  if ((CYTHON_USE_TYPE_SLOTS && CYTHON_USE_PYTYPE_LOOKUP) && likely(!__pyx_type_4grpc_7_cython_6cygrpc_Operation.tp_dictoffset && __pyx_type_4grpc_7_cython_6cygrpc_Operation.tp_getattro == PyObject_GenericGetAttr)) {
    __pyx_type_4grpc_7_cython_6cygrpc_Operation.tp_getattro = __Pyx_PyObject_GenericGetAttr;
  }
  if (__Pyx_SetVtable(__pyx_type_4grpc_7_cython_6cygrpc_Operation.tp_dict, __pyx_vtabptr_4grpc_7_cython_6cygrpc_Operation) < 0) __PYX_ERR(14, 16, __pyx_L1_error)
  if (PyObject_SetAttrString(__pyx_m, "Operation", (PyObject *)&__pyx_type_4grpc_7_cython_6cygrpc_Operation) < 0) __PYX_ERR(14, 16, __pyx_L1_error)
  if (__Pyx_setup_reduce((PyObject*)&__pyx_type_4grpc_7_cython_6cygrpc_Operation) < 0) __PYX_ERR(14, 16, __pyx_L1_error)
  __pyx_ptype_4grpc_7_cython_6cygrpc_Operation = &__pyx_type_4grpc_7_cython_6cygrpc_Operation;
  __pyx_vtabptr_4grpc_7_cython_6cygrpc_SendInitialMetadataOperation = &__pyx_vtable_4grpc_7_cython_6cygrpc_SendInitialMetadataOperation;
  __pyx_vtable_4grpc_7_cython_6cygrpc_SendInitialMetadataOperation.__pyx_base = *__pyx_vtabptr_4grpc_7_cython_6cygrpc_Operation;
  __pyx_vtable_4grpc_7_cython_6cygrpc_SendInitialMetadataOperation.__pyx_base.c = (void (*)(struct __pyx_obj_4grpc_7_cython_6cygrpc_Operation *))__pyx_f_4grpc_7_cython_6cygrpc_28SendInitialMetadataOperation_c;
  __pyx_vtable_4grpc_7_cython_6cygrpc_SendInitialMetadataOperation.__pyx_base.un_c = (void (*)(struct __pyx_obj_4grpc_7_cython_6cygrpc_Operation *))__pyx_f_4grpc_7_cython_6cygrpc_28SendInitialMetadataOperation_un_c;
  __pyx_type_4grpc_7_cython_6cygrpc_SendInitialMetadataOperation.tp_base = __pyx_ptype_4grpc_7_cython_6cygrpc_Operation;
  if (PyType_Ready(&__pyx_type_4grpc_7_cython_6cygrpc_SendInitialMetadataOperation) < 0) __PYX_ERR(14, 25, __pyx_L1_error)
  __pyx_type_4grpc_7_cython_6cygrpc_SendInitialMetadataOperation.tp_print = 0;
  if ((CYTHON_USE_TYPE_SLOTS && CYTHON_USE_PYTYPE_LOOKUP) && likely(!__pyx_type_4grpc_7_cython_6cygrpc_SendInitialMetadataOperation.tp_dictoffset && __pyx_type_4grpc_7_cython_6cygrpc_SendInitialMetadataOperation.tp_getattro == PyObject_GenericGetAttr)) {
    __pyx_type_4grpc_7_cython_6cygrpc_SendInitialMetadataOperation.tp_getattro = __Pyx_PyObject_GenericGetAttr;
  }
  if (__Pyx_SetVtable(__pyx_type_4grpc_7_cython_6cygrpc_SendInitialMetadataOperation.tp_dict, __pyx_vtabptr_4grpc_7_cython_6cygrpc_SendInitialMetadataOperation) < 0) __PYX_ERR(14, 25, __pyx_L1_error)
  if (PyObject_SetAttrString(__pyx_m, "SendInitialMetadataOperation", (PyObject *)&__pyx_type_4grpc_7_cython_6cygrpc_SendInitialMetadataOperation) < 0) __PYX_ERR(14, 25, __pyx_L1_error)
  if (__Pyx_setup_reduce((PyObject*)&__pyx_type_4grpc_7_cython_6cygrpc_SendInitialMetadataOperation) < 0) __PYX_ERR(14, 25, __pyx_L1_error)
  __pyx_ptype_4grpc_7_cython_6cygrpc_SendInitialMetadataOperation = &__pyx_type_4grpc_7_cython_6cygrpc_SendInitialMetadataOperation;
  __pyx_vtabptr_4grpc_7_cython_6cygrpc_SendMessageOperation = &__pyx_vtable_4grpc_7_cython_6cygrpc_SendMessageOperation;
  __pyx_vtable_4grpc_7_cython_6cygrpc_SendMessageOperation.__pyx_base = *__pyx_vtabptr_4grpc_7_cython_6cygrpc_Operation;
  __pyx_vtable_4grpc_7_cython_6cygrpc_SendMessageOperation.__pyx_base.c = (void (*)(struct __pyx_obj_4grpc_7_cython_6cygrpc_Operation *))__pyx_f_4grpc_7_cython_6cygrpc_20SendMessageOperation_c;
  __pyx_vtable_4grpc_7_cython_6cygrpc_SendMessageOperation.__pyx_base.un_c = (void (*)(struct __pyx_obj_4grpc_7_cython_6cygrpc_Operation *))__pyx_f_4grpc_7_cython_6cygrpc_20SendMessageOperation_un_c;
  __pyx_type_4grpc_7_cython_6cygrpc_SendMessageOperation.tp_base = __pyx_ptype_4grpc_7_cython_6cygrpc_Operation;
  if (PyType_Ready(&__pyx_type_4grpc_7_cython_6cygrpc_SendMessageOperation) < 0) __PYX_ERR(14, 49, __pyx_L1_error)
  __pyx_type_4grpc_7_cython_6cygrpc_SendMessageOperation.tp_print = 0;
  if ((CYTHON_USE_TYPE_SLOTS && CYTHON_USE_PYTYPE_LOOKUP) && likely(!__pyx_type_4grpc_7_cython_6cygrpc_SendMessageOperation.tp_dictoffset && __pyx_type_4grpc_7_cython_6cygrpc_SendMessageOperation.tp_getattro == PyObject_GenericGetAttr)) {
    __pyx_type_4grpc_7_cython_6cygrpc_SendMessageOperation.tp_getattro = __Pyx_PyObject_GenericGetAttr;
  }
  if (__Pyx_SetVtable(__pyx_type_4grpc_7_cython_6cygrpc_SendMessageOperation.tp_dict, __pyx_vtabptr_4grpc_7_cython_6cygrpc_SendMessageOperation) < 0) __PYX_ERR(14, 49, __pyx_L1_error)
  if (PyObject_SetAttrString(__pyx_m, "SendMessageOperation", (PyObject *)&__pyx_type_4grpc_7_cython_6cygrpc_SendMessageOperation) < 0) __PYX_ERR(14, 49, __pyx_L1_error)
  if (__Pyx_setup_reduce((PyObject*)&__pyx_type_4grpc_7_cython_6cygrpc_SendMessageOperation) < 0) __PYX_ERR(14, 49, __pyx_L1_error)
  __pyx_ptype_4grpc_7_cython_6cygrpc_SendMessageOperation = &__pyx_type_4grpc_7_cython_6cygrpc_SendMessageOperation;
  __pyx_vtabptr_4grpc_7_cython_6cygrpc_SendCloseFromClientOperation = &__pyx_vtable_4grpc_7_cython_6cygrpc_SendCloseFromClientOperation;
  __pyx_vtable_4grpc_7_cython_6cygrpc_SendCloseFromClientOperation.__pyx_base = *__pyx_vtabptr_4grpc_7_cython_6cygrpc_Operation;
  __pyx_vtable_4grpc_7_cython_6cygrpc_SendCloseFromClientOperation.__pyx_base.c = (void (*)(struct __pyx_obj_4grpc_7_cython_6cygrpc_Operation *))__pyx_f_4grpc_7_cython_6cygrpc_28SendCloseFromClientOperation_c;
  __pyx_vtable_4grpc_7_cython_6cygrpc_SendCloseFromClientOperation.__pyx_base.un_c = (void (*)(struct __pyx_obj_4grpc_7_cython_6cygrpc_Operation *))__pyx_f_4grpc_7_cython_6cygrpc_28SendCloseFromClientOperation_un_c;
  __pyx_type_4grpc_7_cython_6cygrpc_SendCloseFromClientOperation.tp_base = __pyx_ptype_4grpc_7_cython_6cygrpc_Operation;
  if (PyType_Ready(&__pyx_type_4grpc_7_cython_6cygrpc_SendCloseFromClientOperation) < 0) __PYX_ERR(14, 72, __pyx_L1_error)
  __pyx_type_4grpc_7_cython_6cygrpc_SendCloseFromClientOperation.tp_print = 0;
  if ((CYTHON_USE_TYPE_SLOTS && CYTHON_USE_PYTYPE_LOOKUP) && likely(!__pyx_type_4grpc_7_cython_6cygrpc_SendCloseFromClientOperation.tp_dictoffset && __pyx_type_4grpc_7_cython_6cygrpc_SendCloseFromClientOperation.tp_getattro == PyObject_GenericGetAttr)) {
    __pyx_type_4grpc_7_cython_6cygrpc_SendCloseFromClientOperation.tp_getattro = __Pyx_PyObject_GenericGetAttr;
  }
  if (__Pyx_SetVtable(__pyx_type_4grpc_7_cython_6cygrpc_SendCloseFromClientOperation.tp_dict, __pyx_vtabptr_4grpc_7_cython_6cygrpc_SendCloseFromClientOperation) < 0) __PYX_ERR(14, 72, __pyx_L1_error)
  if (PyObject_SetAttrString(__pyx_m, "SendCloseFromClientOperation", (PyObject *)&__pyx_type_4grpc_7_cython_6cygrpc_SendCloseFromClientOperation) < 0) __PYX_ERR(14, 72, __pyx_L1_error)
  if (__Pyx_setup_reduce((PyObject*)&__pyx_type_4grpc_7_cython_6cygrpc_SendCloseFromClientOperation) < 0) __PYX_ERR(14, 72, __pyx_L1_error)
  __pyx_ptype_4grpc_7_cython_6cygrpc_SendCloseFromClientOperation = &__pyx_type_4grpc_7_cython_6cygrpc_SendCloseFromClientOperation;
  __pyx_vtabptr_4grpc_7_cython_6cygrpc_SendStatusFromServerOperation = &__pyx_vtable_4grpc_7_cython_6cygrpc_SendStatusFromServerOperation;
  __pyx_vtable_4grpc_7_cython_6cygrpc_SendStatusFromServerOperation.__pyx_base = *__pyx_vtabptr_4grpc_7_cython_6cygrpc_Operation;
  __pyx_vtable_4grpc_7_cython_6cygrpc_SendStatusFromServerOperation.__pyx_base.c = (void (*)(struct __pyx_obj_4grpc_7_cython_6cygrpc_Operation *))__pyx_f_4grpc_7_cython_6cygrpc_29SendStatusFromServerOperation_c;
  __pyx_vtable_4grpc_7_cython_6cygrpc_SendStatusFromServerOperation.__pyx_base.un_c = (void (*)(struct __pyx_obj_4grpc_7_cython_6cygrpc_Operation *))__pyx_f_4grpc_7_cython_6cygrpc_29SendStatusFromServerOperation_un_c;
  __pyx_type_4grpc_7_cython_6cygrpc_SendStatusFromServerOperation.tp_base = __pyx_ptype_4grpc_7_cython_6cygrpc_Operation;
  if (PyType_Ready(&__pyx_type_4grpc_7_cython_6cygrpc_SendStatusFromServerOperation) < 0) __PYX_ERR(14, 88, __pyx_L1_error)
  __pyx_type_4grpc_7_cython_6cygrpc_SendStatusFromServerOperation.tp_print = 0;
  if ((CYTHON_USE_TYPE_SLOTS && CYTHON_USE_PYTYPE_LOOKUP) && likely(!__pyx_type_4grpc_7_cython_6cygrpc_SendStatusFromServerOperation.tp_dictoffset && __pyx_type_4grpc_7_cython_6cygrpc_SendStatusFromServerOperation.tp_getattro == PyObject_GenericGetAttr)) {
    __pyx_type_4grpc_7_cython_6cygrpc_SendStatusFromServerOperation.tp_getattro = __Pyx_PyObject_GenericGetAttr;
  }
  if (__Pyx_SetVtable(__pyx_type_4grpc_7_cython_6cygrpc_SendStatusFromServerOperation.tp_dict, __pyx_vtabptr_4grpc_7_cython_6cygrpc_SendStatusFromServerOperation) < 0) __PYX_ERR(14, 88, __pyx_L1_error)
  if (PyObject_SetAttrString(__pyx_m, "SendStatusFromServerOperation", (PyObject *)&__pyx_type_4grpc_7_cython_6cygrpc_SendStatusFromServerOperation) < 0) __PYX_ERR(14, 88, __pyx_L1_error)
  if (__Pyx_setup_reduce((PyObject*)&__pyx_type_4grpc_7_cython_6cygrpc_SendStatusFromServerOperation) < 0) __PYX_ERR(14, 88, __pyx_L1_error)
  __pyx_ptype_4grpc_7_cython_6cygrpc_SendStatusFromServerOperation = &__pyx_type_4grpc_7_cython_6cygrpc_SendStatusFromServerOperation;
  __pyx_vtabptr_4grpc_7_cython_6cygrpc_ReceiveInitialMetadataOperation = &__pyx_vtable_4grpc_7_cython_6cygrpc_ReceiveInitialMetadataOperation;
  __pyx_vtable_4grpc_7_cython_6cygrpc_ReceiveInitialMetadataOperation.__pyx_base = *__pyx_vtabptr_4grpc_7_cython_6cygrpc_Operation;
  __pyx_vtable_4grpc_7_cython_6cygrpc_ReceiveInitialMetadataOperation.__pyx_base.c = (void (*)(struct __pyx_obj_4grpc_7_cython_6cygrpc_Operation *))__pyx_f_4grpc_7_cython_6cygrpc_31ReceiveInitialMetadataOperation_c;
  __pyx_vtable_4grpc_7_cython_6cygrpc_ReceiveInitialMetadataOperation.__pyx_base.un_c = (void (*)(struct __pyx_obj_4grpc_7_cython_6cygrpc_Operation *))__pyx_f_4grpc_7_cython_6cygrpc_31ReceiveInitialMetadataOperation_un_c;
  __pyx_type_4grpc_7_cython_6cygrpc_ReceiveInitialMetadataOperation.tp_base = __pyx_ptype_4grpc_7_cython_6cygrpc_Operation;
  if (PyType_Ready(&__pyx_type_4grpc_7_cython_6cygrpc_ReceiveInitialMetadataOperation) < 0) __PYX_ERR(14, 119, __pyx_L1_error)
  __pyx_type_4grpc_7_cython_6cygrpc_ReceiveInitialMetadataOperation.tp_print = 0;
  if ((CYTHON_USE_TYPE_SLOTS && CYTHON_USE_PYTYPE_LOOKUP) && likely(!__pyx_type_4grpc_7_cython_6cygrpc_ReceiveInitialMetadataOperation.tp_dictoffset && __pyx_type_4grpc_7_cython_6cygrpc_ReceiveInitialMetadataOperation.tp_getattro == PyObject_GenericGetAttr)) {
    __pyx_type_4grpc_7_cython_6cygrpc_ReceiveInitialMetadataOperation.tp_getattro = __Pyx_PyObject_GenericGetAttr;
  }
  if (__Pyx_SetVtable(__pyx_type_4grpc_7_cython_6cygrpc_ReceiveInitialMetadataOperation.tp_dict, __pyx_vtabptr_4grpc_7_cython_6cygrpc_ReceiveInitialMetadataOperation) < 0) __PYX_ERR(14, 119, __pyx_L1_error)
  if (PyObject_SetAttrString(__pyx_m, "ReceiveInitialMetadataOperation", (PyObject *)&__pyx_type_4grpc_7_cython_6cygrpc_ReceiveInitialMetadataOperation) < 0) __PYX_ERR(14, 119, __pyx_L1_error)
  if (__Pyx_setup_reduce((PyObject*)&__pyx_type_4grpc_7_cython_6cygrpc_ReceiveInitialMetadataOperation) < 0) __PYX_ERR(14, 119, __pyx_L1_error)
  __pyx_ptype_4grpc_7_cython_6cygrpc_ReceiveInitialMetadataOperation = &__pyx_type_4grpc_7_cython_6cygrpc_ReceiveInitialMetadataOperation;
  __pyx_vtabptr_4grpc_7_cython_6cygrpc_ReceiveMessageOperation = &__pyx_vtable_4grpc_7_cython_6cygrpc_ReceiveMessageOperation;
  __pyx_vtable_4grpc_7_cython_6cygrpc_ReceiveMessageOperation.__pyx_base = *__pyx_vtabptr_4grpc_7_cython_6cygrpc_Operation;
  __pyx_vtable_4grpc_7_cython_6cygrpc_ReceiveMessageOperation.__pyx_base.c = (void (*)(struct __pyx_obj_4grpc_7_cython_6cygrpc_Operation *))__pyx_f_4grpc_7_cython_6cygrpc_23ReceiveMessageOperation_c;
  __pyx_vtable_4grpc_7_cython_6cygrpc_ReceiveMessageOperation.__pyx_base.un_c = (void (*)(struct __pyx_obj_4grpc_7_cython_6cygrpc_Operation *))__pyx_f_4grpc_7_cython_6cygrpc_23ReceiveMessageOperation_un_c;
  __pyx_type_4grpc_7_cython_6cygrpc_ReceiveMessageOperation.tp_base = __pyx_ptype_4grpc_7_cython_6cygrpc_Operation;
  if (PyType_Ready(&__pyx_type_4grpc_7_cython_6cygrpc_ReceiveMessageOperation) < 0) __PYX_ERR(14, 142, __pyx_L1_error)
  __pyx_type_4grpc_7_cython_6cygrpc_ReceiveMessageOperation.tp_print = 0;
  if ((CYTHON_USE_TYPE_SLOTS && CYTHON_USE_PYTYPE_LOOKUP) && likely(!__pyx_type_4grpc_7_cython_6cygrpc_ReceiveMessageOperation.tp_dictoffset && __pyx_type_4grpc_7_cython_6cygrpc_ReceiveMessageOperation.tp_getattro == PyObject_GenericGetAttr)) {
    __pyx_type_4grpc_7_cython_6cygrpc_ReceiveMessageOperation.tp_getattro = __Pyx_PyObject_GenericGetAttr;
  }
  if (__Pyx_SetVtable(__pyx_type_4grpc_7_cython_6cygrpc_ReceiveMessageOperation.tp_dict, __pyx_vtabptr_4grpc_7_cython_6cygrpc_ReceiveMessageOperation) < 0) __PYX_ERR(14, 142, __pyx_L1_error)
  if (PyObject_SetAttrString(__pyx_m, "ReceiveMessageOperation", (PyObject *)&__pyx_type_4grpc_7_cython_6cygrpc_ReceiveMessageOperation) < 0) __PYX_ERR(14, 142, __pyx_L1_error)
  if (__Pyx_setup_reduce((PyObject*)&__pyx_type_4grpc_7_cython_6cygrpc_ReceiveMessageOperation) < 0) __PYX_ERR(14, 142, __pyx_L1_error)
  __pyx_ptype_4grpc_7_cython_6cygrpc_ReceiveMessageOperation = &__pyx_type_4grpc_7_cython_6cygrpc_ReceiveMessageOperation;
  __pyx_vtabptr_4grpc_7_cython_6cygrpc_ReceiveStatusOnClientOperation = &__pyx_vtable_4grpc_7_cython_6cygrpc_ReceiveStatusOnClientOperation;
  __pyx_vtable_4grpc_7_cython_6cygrpc_ReceiveStatusOnClientOperation.__pyx_base = *__pyx_vtabptr_4grpc_7_cython_6cygrpc_Operation;
  __pyx_vtable_4grpc_7_cython_6cygrpc_ReceiveStatusOnClientOperation.__pyx_base.c = (void (*)(struct __pyx_obj_4grpc_7_cython_6cygrpc_Operation *))__pyx_f_4grpc_7_cython_6cygrpc_30ReceiveStatusOnClientOperation_c;
  __pyx_vtable_4grpc_7_cython_6cygrpc_ReceiveStatusOnClientOperation.__pyx_base.un_c = (void (*)(struct __pyx_obj_4grpc_7_cython_6cygrpc_Operation *))__pyx_f_4grpc_7_cython_6cygrpc_30ReceiveStatusOnClientOperation_un_c;
  __pyx_type_4grpc_7_cython_6cygrpc_ReceiveStatusOnClientOperation.tp_base = __pyx_ptype_4grpc_7_cython_6cygrpc_Operation;
  if (PyType_Ready(&__pyx_type_4grpc_7_cython_6cygrpc_ReceiveStatusOnClientOperation) < 0) __PYX_ERR(14, 184, __pyx_L1_error)
  __pyx_type_4grpc_7_cython_6cygrpc_ReceiveStatusOnClientOperation.tp_print = 0;
  if ((CYTHON_USE_TYPE_SLOTS && CYTHON_USE_PYTYPE_LOOKUP) && likely(!__pyx_type_4grpc_7_cython_6cygrpc_ReceiveStatusOnClientOperation.tp_dictoffset && __pyx_type_4grpc_7_cython_6cygrpc_ReceiveStatusOnClientOperation.tp_getattro == PyObject_GenericGetAttr)) {
    __pyx_type_4grpc_7_cython_6cygrpc_ReceiveStatusOnClientOperation.tp_getattro = __Pyx_PyObject_GenericGetAttr;
  }
  if (__Pyx_SetVtable(__pyx_type_4grpc_7_cython_6cygrpc_ReceiveStatusOnClientOperation.tp_dict, __pyx_vtabptr_4grpc_7_cython_6cygrpc_ReceiveStatusOnClientOperation) < 0) __PYX_ERR(14, 184, __pyx_L1_error)
  if (PyObject_SetAttrString(__pyx_m, "ReceiveStatusOnClientOperation", (PyObject *)&__pyx_type_4grpc_7_cython_6cygrpc_ReceiveStatusOnClientOperation) < 0) __PYX_ERR(14, 184, __pyx_L1_error)
  if (__Pyx_setup_reduce((PyObject*)&__pyx_type_4grpc_7_cython_6cygrpc_ReceiveStatusOnClientOperation) < 0) __PYX_ERR(14, 184, __pyx_L1_error)
  __pyx_ptype_4grpc_7_cython_6cygrpc_ReceiveStatusOnClientOperation = &__pyx_type_4grpc_7_cython_6cygrpc_ReceiveStatusOnClientOperation;
  __pyx_vtabptr_4grpc_7_cython_6cygrpc_ReceiveCloseOnServerOperation = &__pyx_vtable_4grpc_7_cython_6cygrpc_ReceiveCloseOnServerOperation;
  __pyx_vtable_4grpc_7_cython_6cygrpc_ReceiveCloseOnServerOperation.__pyx_base = *__pyx_vtabptr_4grpc_7_cython_6cygrpc_Operation;
  __pyx_vtable_4grpc_7_cython_6cygrpc_ReceiveCloseOnServerOperation.__pyx_base.c = (void (*)(struct __pyx_obj_4grpc_7_cython_6cygrpc_Operation *))__pyx_f_4grpc_7_cython_6cygrpc_29ReceiveCloseOnServerOperation_c;
  __pyx_vtable_4grpc_7_cython_6cygrpc_ReceiveCloseOnServerOperation.__pyx_base.un_c = (void (*)(struct __pyx_obj_4grpc_7_cython_6cygrpc_Operation *))__pyx_f_4grpc_7_cython_6cygrpc_29ReceiveCloseOnServerOperation_un_c;
  __pyx_type_4grpc_7_cython_6cygrpc_ReceiveCloseOnServerOperation.tp_base = __pyx_ptype_4grpc_7_cython_6cygrpc_Operation;
  if (PyType_Ready(&__pyx_type_4grpc_7_cython_6cygrpc_ReceiveCloseOnServerOperation) < 0) __PYX_ERR(14, 230, __pyx_L1_error)
  __pyx_type_4grpc_7_cython_6cygrpc_ReceiveCloseOnServerOperation.tp_print = 0;
  if ((CYTHON_USE_TYPE_SLOTS && CYTHON_USE_PYTYPE_LOOKUP) && likely(!__pyx_type_4grpc_7_cython_6cygrpc_ReceiveCloseOnServerOperation.tp_dictoffset && __pyx_type_4grpc_7_cython_6cygrpc_ReceiveCloseOnServerOperation.tp_getattro == PyObject_GenericGetAttr)) {
    __pyx_type_4grpc_7_cython_6cygrpc_ReceiveCloseOnServerOperation.tp_getattro = __Pyx_PyObject_GenericGetAttr;
  }
  if (__Pyx_SetVtable(__pyx_type_4grpc_7_cython_6cygrpc_ReceiveCloseOnServerOperation.tp_dict, __pyx_vtabptr_4grpc_7_cython_6cygrpc_ReceiveCloseOnServerOperation) < 0) __PYX_ERR(14, 230, __pyx_L1_error)
  if (PyObject_SetAttrString(__pyx_m, "ReceiveCloseOnServerOperation", (PyObject *)&__pyx_type_4grpc_7_cython_6cygrpc_ReceiveCloseOnServerOperation) < 0) __PYX_ERR(14, 230, __pyx_L1_error)
  if (__Pyx_setup_reduce((PyObject*)&__pyx_type_4grpc_7_cython_6cygrpc_ReceiveCloseOnServerOperation) < 0) __PYX_ERR(14, 230, __pyx_L1_error)
  __pyx_ptype_4grpc_7_cython_6cygrpc_ReceiveCloseOnServerOperation = &__pyx_type_4grpc_7_cython_6cygrpc_ReceiveCloseOnServerOperation;
  if (PyType_Ready(&__pyx_type_4grpc_7_cython_6cygrpc_CallDetails) < 0) __PYX_ERR(16, 130, __pyx_L1_error)
  __pyx_type_4grpc_7_cython_6cygrpc_CallDetails.tp_print = 0;
  if ((CYTHON_USE_TYPE_SLOTS && CYTHON_USE_PYTYPE_LOOKUP) && likely(!__pyx_type_4grpc_7_cython_6cygrpc_CallDetails.tp_dictoffset && __pyx_type_4grpc_7_cython_6cygrpc_CallDetails.tp_getattro == PyObject_GenericGetAttr)) {
    __pyx_type_4grpc_7_cython_6cygrpc_CallDetails.tp_getattro = __Pyx_PyObject_GenericGetAttr;
  }
  if (PyObject_SetAttrString(__pyx_m, "CallDetails", (PyObject *)&__pyx_type_4grpc_7_cython_6cygrpc_CallDetails) < 0) __PYX_ERR(16, 130, __pyx_L1_error)
  if (__Pyx_setup_reduce((PyObject*)&__pyx_type_4grpc_7_cython_6cygrpc_CallDetails) < 0) __PYX_ERR(16, 130, __pyx_L1_error)
  __pyx_ptype_4grpc_7_cython_6cygrpc_CallDetails = &__pyx_type_4grpc_7_cython_6cygrpc_CallDetails;
  if (PyType_Ready(&__pyx_type_4grpc_7_cython_6cygrpc_SslPemKeyCertPair) < 0) __PYX_ERR(16, 155, __pyx_L1_error)
  __pyx_type_4grpc_7_cython_6cygrpc_SslPemKeyCertPair.tp_print = 0;
  if ((CYTHON_USE_TYPE_SLOTS && CYTHON_USE_PYTYPE_LOOKUP) && likely(!__pyx_type_4grpc_7_cython_6cygrpc_SslPemKeyCertPair.tp_dictoffset && __pyx_type_4grpc_7_cython_6cygrpc_SslPemKeyCertPair.tp_getattro == PyObject_GenericGetAttr)) {
    __pyx_type_4grpc_7_cython_6cygrpc_SslPemKeyCertPair.tp_getattro = __Pyx_PyObject_GenericGetAttr;
  }
  if (PyObject_SetAttrString(__pyx_m, "SslPemKeyCertPair", (PyObject *)&__pyx_type_4grpc_7_cython_6cygrpc_SslPemKeyCertPair) < 0) __PYX_ERR(16, 155, __pyx_L1_error)
  if (__Pyx_setup_reduce((PyObject*)&__pyx_type_4grpc_7_cython_6cygrpc_SslPemKeyCertPair) < 0) __PYX_ERR(16, 155, __pyx_L1_error)
  __pyx_ptype_4grpc_7_cython_6cygrpc_SslPemKeyCertPair = &__pyx_type_4grpc_7_cython_6cygrpc_SslPemKeyCertPair;
  if (PyType_Ready(&__pyx_type_4grpc_7_cython_6cygrpc_CompressionOptions) < 0) __PYX_ERR(16, 164, __pyx_L1_error)
  __pyx_type_4grpc_7_cython_6cygrpc_CompressionOptions.tp_print = 0;
  if ((CYTHON_USE_TYPE_SLOTS && CYTHON_USE_PYTYPE_LOOKUP) && likely(!__pyx_type_4grpc_7_cython_6cygrpc_CompressionOptions.tp_dictoffset && __pyx_type_4grpc_7_cython_6cygrpc_CompressionOptions.tp_getattro == PyObject_GenericGetAttr)) {
    __pyx_type_4grpc_7_cython_6cygrpc_CompressionOptions.tp_getattro = __Pyx_PyObject_GenericGetAttr;
  }
  if (PyObject_SetAttrString(__pyx_m, "CompressionOptions", (PyObject *)&__pyx_type_4grpc_7_cython_6cygrpc_CompressionOptions) < 0) __PYX_ERR(16, 164, __pyx_L1_error)
  if (__Pyx_setup_reduce((PyObject*)&__pyx_type_4grpc_7_cython_6cygrpc_CompressionOptions) < 0) __PYX_ERR(16, 164, __pyx_L1_error)
  __pyx_ptype_4grpc_7_cython_6cygrpc_CompressionOptions = &__pyx_type_4grpc_7_cython_6cygrpc_CompressionOptions;
  __pyx_vtabptr_4grpc_7_cython_6cygrpc_Server = &__pyx_vtable_4grpc_7_cython_6cygrpc_Server;
  __pyx_vtable_4grpc_7_cython_6cygrpc_Server._c_shutdown = (PyObject *(*)(struct __pyx_obj_4grpc_7_cython_6cygrpc_Server *, struct __pyx_obj_4grpc_7_cython_6cygrpc_CompletionQueue *, PyObject *))__pyx_f_4grpc_7_cython_6cygrpc_6Server__c_shutdown;
  __pyx_vtable_4grpc_7_cython_6cygrpc_Server.notify_shutdown_complete = (PyObject *(*)(struct __pyx_obj_4grpc_7_cython_6cygrpc_Server *))__pyx_f_4grpc_7_cython_6cygrpc_6Server_notify_shutdown_complete;
  if (PyType_Ready(&__pyx_type_4grpc_7_cython_6cygrpc_Server) < 0) __PYX_ERR(5, 24, __pyx_L1_error)
  __pyx_type_4grpc_7_cython_6cygrpc_Server.tp_print = 0;
  if ((CYTHON_USE_TYPE_SLOTS && CYTHON_USE_PYTYPE_LOOKUP) && likely(!__pyx_type_4grpc_7_cython_6cygrpc_Server.tp_dictoffset && __pyx_type_4grpc_7_cython_6cygrpc_Server.tp_getattro == PyObject_GenericGetAttr)) {
    __pyx_type_4grpc_7_cython_6cygrpc_Server.tp_getattro = __Pyx_PyObject_GenericGetAttr;
  }
  if (__Pyx_SetVtable(__pyx_type_4grpc_7_cython_6cygrpc_Server.tp_dict, __pyx_vtabptr_4grpc_7_cython_6cygrpc_Server) < 0) __PYX_ERR(5, 24, __pyx_L1_error)
  if (PyObject_SetAttrString(__pyx_m, "Server", (PyObject *)&__pyx_type_4grpc_7_cython_6cygrpc_Server) < 0) __PYX_ERR(5, 24, __pyx_L1_error)
  if (__Pyx_setup_reduce((PyObject*)&__pyx_type_4grpc_7_cython_6cygrpc_Server) < 0) __PYX_ERR(5, 24, __pyx_L1_error)
  __pyx_ptype_4grpc_7_cython_6cygrpc_Server = &__pyx_type_4grpc_7_cython_6cygrpc_Server;
  __pyx_vtabptr_4grpc_7_cython_6cygrpc__Tag = &__pyx_vtable_4grpc_7_cython_6cygrpc__Tag;
  __pyx_vtable_4grpc_7_cython_6cygrpc__Tag.event = (PyObject *(*)(struct __pyx_obj_4grpc_7_cython_6cygrpc__Tag *, grpc_event))__pyx_f_4grpc_7_cython_6cygrpc_4_Tag_event;
  if (PyType_Ready(&__pyx_type_4grpc_7_cython_6cygrpc__Tag) < 0) __PYX_ERR(18, 16, __pyx_L1_error)
  __pyx_type_4grpc_7_cython_6cygrpc__Tag.tp_print = 0;
  if ((CYTHON_USE_TYPE_SLOTS && CYTHON_USE_PYTYPE_LOOKUP) && likely(!__pyx_type_4grpc_7_cython_6cygrpc__Tag.tp_dictoffset && __pyx_type_4grpc_7_cython_6cygrpc__Tag.tp_getattro == PyObject_GenericGetAttr)) {
    __pyx_type_4grpc_7_cython_6cygrpc__Tag.tp_getattro = __Pyx_PyObject_GenericGetAttr;
  }
  if (__Pyx_SetVtable(__pyx_type_4grpc_7_cython_6cygrpc__Tag.tp_dict, __pyx_vtabptr_4grpc_7_cython_6cygrpc__Tag) < 0) __PYX_ERR(18, 16, __pyx_L1_error)
  if (PyObject_SetAttrString(__pyx_m, "_Tag", (PyObject *)&__pyx_type_4grpc_7_cython_6cygrpc__Tag) < 0) __PYX_ERR(18, 16, __pyx_L1_error)
  if (__Pyx_setup_reduce((PyObject*)&__pyx_type_4grpc_7_cython_6cygrpc__Tag) < 0) __PYX_ERR(18, 16, __pyx_L1_error)
  __pyx_ptype_4grpc_7_cython_6cygrpc__Tag = &__pyx_type_4grpc_7_cython_6cygrpc__Tag;
  __pyx_vtabptr_4grpc_7_cython_6cygrpc__ConnectivityTag = &__pyx_vtable_4grpc_7_cython_6cygrpc__ConnectivityTag;
  __pyx_vtable_4grpc_7_cython_6cygrpc__ConnectivityTag.__pyx_base = *__pyx_vtabptr_4grpc_7_cython_6cygrpc__Tag;
  __pyx_vtable_4grpc_7_cython_6cygrpc__ConnectivityTag.__pyx_base.event = (PyObject *(*)(struct __pyx_obj_4grpc_7_cython_6cygrpc__Tag *, grpc_event))__pyx_f_4grpc_7_cython_6cygrpc_16_ConnectivityTag_event__pyx_wrap_1;
  __pyx_vtable_4grpc_7_cython_6cygrpc__ConnectivityTag.event = (struct __pyx_obj_4grpc_7_cython_6cygrpc_ConnectivityEvent *(*)(struct __pyx_obj_4grpc_7_cython_6cygrpc__ConnectivityTag *, grpc_event))__pyx_f_4grpc_7_cython_6cygrpc_16_ConnectivityTag_event;
  __pyx_type_4grpc_7_cython_6cygrpc__ConnectivityTag.tp_base = __pyx_ptype_4grpc_7_cython_6cygrpc__Tag;
  if (PyType_Ready(&__pyx_type_4grpc_7_cython_6cygrpc__ConnectivityTag) < 0) __PYX_ERR(18, 22, __pyx_L1_error)
  __pyx_type_4grpc_7_cython_6cygrpc__ConnectivityTag.tp_print = 0;
  if ((CYTHON_USE_TYPE_SLOTS && CYTHON_USE_PYTYPE_LOOKUP) && likely(!__pyx_type_4grpc_7_cython_6cygrpc__ConnectivityTag.tp_dictoffset && __pyx_type_4grpc_7_cython_6cygrpc__ConnectivityTag.tp_getattro == PyObject_GenericGetAttr)) {
    __pyx_type_4grpc_7_cython_6cygrpc__ConnectivityTag.tp_getattro = __Pyx_PyObject_GenericGetAttr;
  }
  if (__Pyx_SetVtable(__pyx_type_4grpc_7_cython_6cygrpc__ConnectivityTag.tp_dict, __pyx_vtabptr_4grpc_7_cython_6cygrpc__ConnectivityTag) < 0) __PYX_ERR(18, 22, __pyx_L1_error)
  if (PyObject_SetAttrString(__pyx_m, "_ConnectivityTag", (PyObject *)&__pyx_type_4grpc_7_cython_6cygrpc__ConnectivityTag) < 0) __PYX_ERR(18, 22, __pyx_L1_error)
  if (__Pyx_setup_reduce((PyObject*)&__pyx_type_4grpc_7_cython_6cygrpc__ConnectivityTag) < 0) __PYX_ERR(18, 22, __pyx_L1_error)
  __pyx_ptype_4grpc_7_cython_6cygrpc__ConnectivityTag = &__pyx_type_4grpc_7_cython_6cygrpc__ConnectivityTag;
  __pyx_vtabptr_4grpc_7_cython_6cygrpc__RequestCallTag = &__pyx_vtable_4grpc_7_cython_6cygrpc__RequestCallTag;
  __pyx_vtable_4grpc_7_cython_6cygrpc__RequestCallTag.__pyx_base = *__pyx_vtabptr_4grpc_7_cython_6cygrpc__Tag;
  __pyx_vtable_4grpc_7_cython_6cygrpc__RequestCallTag.__pyx_base.event = (PyObject *(*)(struct __pyx_obj_4grpc_7_cython_6cygrpc__Tag *, grpc_event))__pyx_f_4grpc_7_cython_6cygrpc_15_RequestCallTag_event__pyx_wrap_1;
  __pyx_vtable_4grpc_7_cython_6cygrpc__RequestCallTag.prepare = (void (*)(struct __pyx_obj_4grpc_7_cython_6cygrpc__RequestCallTag *))__pyx_f_4grpc_7_cython_6cygrpc_15_RequestCallTag_prepare;
  __pyx_vtable_4grpc_7_cython_6cygrpc__RequestCallTag.event = (struct __pyx_obj_4grpc_7_cython_6cygrpc_RequestCallEvent *(*)(struct __pyx_obj_4grpc_7_cython_6cygrpc__RequestCallTag *, grpc_event))__pyx_f_4grpc_7_cython_6cygrpc_15_RequestCallTag_event;
  __pyx_type_4grpc_7_cython_6cygrpc__RequestCallTag.tp_base = __pyx_ptype_4grpc_7_cython_6cygrpc__Tag;
  if (PyType_Ready(&__pyx_type_4grpc_7_cython_6cygrpc__RequestCallTag) < 0) __PYX_ERR(18, 31, __pyx_L1_error)
  __pyx_type_4grpc_7_cython_6cygrpc__RequestCallTag.tp_print = 0;
  if ((CYTHON_USE_TYPE_SLOTS && CYTHON_USE_PYTYPE_LOOKUP) && likely(!__pyx_type_4grpc_7_cython_6cygrpc__RequestCallTag.tp_dictoffset && __pyx_type_4grpc_7_cython_6cygrpc__RequestCallTag.tp_getattro == PyObject_GenericGetAttr)) {
    __pyx_type_4grpc_7_cython_6cygrpc__RequestCallTag.tp_getattro = __Pyx_PyObject_GenericGetAttr;
  }
  if (__Pyx_SetVtable(__pyx_type_4grpc_7_cython_6cygrpc__RequestCallTag.tp_dict, __pyx_vtabptr_4grpc_7_cython_6cygrpc__RequestCallTag) < 0) __PYX_ERR(18, 31, __pyx_L1_error)
  if (PyObject_SetAttrString(__pyx_m, "_RequestCallTag", (PyObject *)&__pyx_type_4grpc_7_cython_6cygrpc__RequestCallTag) < 0) __PYX_ERR(18, 31, __pyx_L1_error)
  if (__Pyx_setup_reduce((PyObject*)&__pyx_type_4grpc_7_cython_6cygrpc__RequestCallTag) < 0) __PYX_ERR(18, 31, __pyx_L1_error)
  __pyx_ptype_4grpc_7_cython_6cygrpc__RequestCallTag = &__pyx_type_4grpc_7_cython_6cygrpc__RequestCallTag;
  __pyx_vtabptr_4grpc_7_cython_6cygrpc__BatchOperationTag = &__pyx_vtable_4grpc_7_cython_6cygrpc__BatchOperationTag;
  __pyx_vtable_4grpc_7_cython_6cygrpc__BatchOperationTag.__pyx_base = *__pyx_vtabptr_4grpc_7_cython_6cygrpc__Tag;
  __pyx_vtable_4grpc_7_cython_6cygrpc__BatchOperationTag.__pyx_base.event = (PyObject *(*)(struct __pyx_obj_4grpc_7_cython_6cygrpc__Tag *, grpc_event))__pyx_f_4grpc_7_cython_6cygrpc_18_BatchOperationTag_event__pyx_wrap_1;
  __pyx_vtable_4grpc_7_cython_6cygrpc__BatchOperationTag.prepare = (void (*)(struct __pyx_obj_4grpc_7_cython_6cygrpc__BatchOperationTag *))__pyx_f_4grpc_7_cython_6cygrpc_18_BatchOperationTag_prepare;
  __pyx_vtable_4grpc_7_cython_6cygrpc__BatchOperationTag.event = (struct __pyx_obj_4grpc_7_cython_6cygrpc_BatchOperationEvent *(*)(struct __pyx_obj_4grpc_7_cython_6cygrpc__BatchOperationTag *, grpc_event))__pyx_f_4grpc_7_cython_6cygrpc_18_BatchOperationTag_event;
  __pyx_type_4grpc_7_cython_6cygrpc__BatchOperationTag.tp_base = __pyx_ptype_4grpc_7_cython_6cygrpc__Tag;
  if (PyType_Ready(&__pyx_type_4grpc_7_cython_6cygrpc__BatchOperationTag) < 0) __PYX_ERR(18, 52, __pyx_L1_error)
  __pyx_type_4grpc_7_cython_6cygrpc__BatchOperationTag.tp_print = 0;
  if ((CYTHON_USE_TYPE_SLOTS && CYTHON_USE_PYTYPE_LOOKUP) && likely(!__pyx_type_4grpc_7_cython_6cygrpc__BatchOperationTag.tp_dictoffset && __pyx_type_4grpc_7_cython_6cygrpc__BatchOperationTag.tp_getattro == PyObject_GenericGetAttr)) {
    __pyx_type_4grpc_7_cython_6cygrpc__BatchOperationTag.tp_getattro = __Pyx_PyObject_GenericGetAttr;
  }
  if (__Pyx_SetVtable(__pyx_type_4grpc_7_cython_6cygrpc__BatchOperationTag.tp_dict, __pyx_vtabptr_4grpc_7_cython_6cygrpc__BatchOperationTag) < 0) __PYX_ERR(18, 52, __pyx_L1_error)
  if (PyObject_SetAttrString(__pyx_m, "_BatchOperationTag", (PyObject *)&__pyx_type_4grpc_7_cython_6cygrpc__BatchOperationTag) < 0) __PYX_ERR(18, 52, __pyx_L1_error)
  if (__Pyx_setup_reduce((PyObject*)&__pyx_type_4grpc_7_cython_6cygrpc__BatchOperationTag) < 0) __PYX_ERR(18, 52, __pyx_L1_error)
  __pyx_ptype_4grpc_7_cython_6cygrpc__BatchOperationTag = &__pyx_type_4grpc_7_cython_6cygrpc__BatchOperationTag;
  __pyx_vtabptr_4grpc_7_cython_6cygrpc__ServerShutdownTag = &__pyx_vtable_4grpc_7_cython_6cygrpc__ServerShutdownTag;
  __pyx_vtable_4grpc_7_cython_6cygrpc__ServerShutdownTag.__pyx_base = *__pyx_vtabptr_4grpc_7_cython_6cygrpc__Tag;
  __pyx_vtable_4grpc_7_cython_6cygrpc__ServerShutdownTag.__pyx_base.event = (PyObject *(*)(struct __pyx_obj_4grpc_7_cython_6cygrpc__Tag *, grpc_event))__pyx_f_4grpc_7_cython_6cygrpc_18_ServerShutdownTag_event__pyx_wrap_1;
  __pyx_vtable_4grpc_7_cython_6cygrpc__ServerShutdownTag.event = (struct __pyx_obj_4grpc_7_cython_6cygrpc_ServerShutdownEvent *(*)(struct __pyx_obj_4grpc_7_cython_6cygrpc__ServerShutdownTag *, grpc_event))__pyx_f_4grpc_7_cython_6cygrpc_18_ServerShutdownTag_event;
  __pyx_type_4grpc_7_cython_6cygrpc__ServerShutdownTag.tp_base = __pyx_ptype_4grpc_7_cython_6cygrpc__Tag;
  if (PyType_Ready(&__pyx_type_4grpc_7_cython_6cygrpc__ServerShutdownTag) < 0) __PYX_ERR(18, 81, __pyx_L1_error)
  __pyx_type_4grpc_7_cython_6cygrpc__ServerShutdownTag.tp_print = 0;
  if ((CYTHON_USE_TYPE_SLOTS && CYTHON_USE_PYTYPE_LOOKUP) && likely(!__pyx_type_4grpc_7_cython_6cygrpc__ServerShutdownTag.tp_dictoffset && __pyx_type_4grpc_7_cython_6cygrpc__ServerShutdownTag.tp_getattro == PyObject_GenericGetAttr)) {
    __pyx_type_4grpc_7_cython_6cygrpc__ServerShutdownTag.tp_getattro = __Pyx_PyObject_GenericGetAttr;
  }
  if (__Pyx_SetVtable(__pyx_type_4grpc_7_cython_6cygrpc__ServerShutdownTag.tp_dict, __pyx_vtabptr_4grpc_7_cython_6cygrpc__ServerShutdownTag) < 0) __PYX_ERR(18, 81, __pyx_L1_error)
  if (PyObject_SetAttrString(__pyx_m, "_ServerShutdownTag", (PyObject *)&__pyx_type_4grpc_7_cython_6cygrpc__ServerShutdownTag) < 0) __PYX_ERR(18, 81, __pyx_L1_error)
  if (__Pyx_setup_reduce((PyObject*)&__pyx_type_4grpc_7_cython_6cygrpc__ServerShutdownTag) < 0) __PYX_ERR(18, 81, __pyx_L1_error)
  __pyx_ptype_4grpc_7_cython_6cygrpc__ServerShutdownTag = &__pyx_type_4grpc_7_cython_6cygrpc__ServerShutdownTag;
  if (PyType_Ready(&__pyx_type_4grpc_7_cython_6cygrpc__VTable) < 0) __PYX_ERR(22, 34, __pyx_L1_error)
  __pyx_type_4grpc_7_cython_6cygrpc__VTable.tp_print = 0;
  if ((CYTHON_USE_TYPE_SLOTS && CYTHON_USE_PYTYPE_LOOKUP) && likely(!__pyx_type_4grpc_7_cython_6cygrpc__VTable.tp_dictoffset && __pyx_type_4grpc_7_cython_6cygrpc__VTable.tp_getattro == PyObject_GenericGetAttr)) {
    __pyx_type_4grpc_7_cython_6cygrpc__VTable.tp_getattro = __Pyx_PyObject_GenericGetAttr;
  }
  if (PyObject_SetAttrString(__pyx_m, "_VTable", (PyObject *)&__pyx_type_4grpc_7_cython_6cygrpc__VTable) < 0) __PYX_ERR(22, 34, __pyx_L1_error)
  if (__Pyx_setup_reduce((PyObject*)&__pyx_type_4grpc_7_cython_6cygrpc__VTable) < 0) __PYX_ERR(22, 34, __pyx_L1_error)
  __pyx_ptype_4grpc_7_cython_6cygrpc__VTable = &__pyx_type_4grpc_7_cython_6cygrpc__VTable;
  if (PyType_Ready(&__pyx_type_4grpc_7_cython_6cygrpc_TimerWrapper) < 0) __PYX_ERR(6, 345, __pyx_L1_error)
  __pyx_type_4grpc_7_cython_6cygrpc_TimerWrapper.tp_print = 0;
  if ((CYTHON_USE_TYPE_SLOTS && CYTHON_USE_PYTYPE_LOOKUP) && likely(!__pyx_type_4grpc_7_cython_6cygrpc_TimerWrapper.tp_dictoffset && __pyx_type_4grpc_7_cython_6cygrpc_TimerWrapper.tp_getattro == PyObject_GenericGetAttr)) {
    __pyx_type_4grpc_7_cython_6cygrpc_TimerWrapper.tp_getattro = __Pyx_PyObject_GenericGetAttr;
  }
  if (PyObject_SetAttrString(__pyx_m, "TimerWrapper", (PyObject *)&__pyx_type_4grpc_7_cython_6cygrpc_TimerWrapper) < 0) __PYX_ERR(6, 345, __pyx_L1_error)
  if (__Pyx_setup_reduce((PyObject*)&__pyx_type_4grpc_7_cython_6cygrpc_TimerWrapper) < 0) __PYX_ERR(6, 345, __pyx_L1_error)
  __pyx_ptype_4grpc_7_cython_6cygrpc_TimerWrapper = &__pyx_type_4grpc_7_cython_6cygrpc_TimerWrapper;
  if (PyType_Ready(&__pyx_type_4grpc_7_cython_6cygrpc_SocketWrapper) < 0) __PYX_ERR(6, 79, __pyx_L1_error)
  __pyx_type_4grpc_7_cython_6cygrpc_SocketWrapper.tp_print = 0;
  if ((CYTHON_USE_TYPE_SLOTS && CYTHON_USE_PYTYPE_LOOKUP) && likely(!__pyx_type_4grpc_7_cython_6cygrpc_SocketWrapper.tp_dictoffset && __pyx_type_4grpc_7_cython_6cygrpc_SocketWrapper.tp_getattro == PyObject_GenericGetAttr)) {
    __pyx_type_4grpc_7_cython_6cygrpc_SocketWrapper.tp_getattro = __Pyx_PyObject_GenericGetAttr;
  }
  if (PyObject_SetAttrString(__pyx_m, "SocketWrapper", (PyObject *)&__pyx_type_4grpc_7_cython_6cygrpc_SocketWrapper) < 0) __PYX_ERR(6, 79, __pyx_L1_error)
  if (__Pyx_setup_reduce((PyObject*)&__pyx_type_4grpc_7_cython_6cygrpc_SocketWrapper) < 0) __PYX_ERR(6, 79, __pyx_L1_error)
  __pyx_ptype_4grpc_7_cython_6cygrpc_SocketWrapper = &__pyx_type_4grpc_7_cython_6cygrpc_SocketWrapper;
  if (PyType_Ready(&__pyx_type_4grpc_7_cython_6cygrpc_ResolveWrapper) < 0) __PYX_ERR(6, 305, __pyx_L1_error)
  __pyx_type_4grpc_7_cython_6cygrpc_ResolveWrapper.tp_print = 0;
  if ((CYTHON_USE_TYPE_SLOTS && CYTHON_USE_PYTYPE_LOOKUP) && likely(!__pyx_type_4grpc_7_cython_6cygrpc_ResolveWrapper.tp_dictoffset && __pyx_type_4grpc_7_cython_6cygrpc_ResolveWrapper.tp_getattro == PyObject_GenericGetAttr)) {
    __pyx_type_4grpc_7_cython_6cygrpc_ResolveWrapper.tp_getattro = __Pyx_PyObject_GenericGetAttr;
  }
  if (PyObject_SetAttrString(__pyx_m, "ResolveWrapper", (PyObject *)&__pyx_type_4grpc_7_cython_6cygrpc_ResolveWrapper) < 0) __PYX_ERR(6, 305, __pyx_L1_error)
  if (__Pyx_setup_reduce((PyObject*)&__pyx_type_4grpc_7_cython_6cygrpc_ResolveWrapper) < 0) __PYX_ERR(6, 305, __pyx_L1_error)
  __pyx_ptype_4grpc_7_cython_6cygrpc_ResolveWrapper = &__pyx_type_4grpc_7_cython_6cygrpc_ResolveWrapper;
  if (PyType_Ready(&__pyx_type_4grpc_7_cython_6cygrpc__GrpcArgWrapper) < 0) __PYX_ERR(2, 18, __pyx_L1_error)
  __pyx_type_4grpc_7_cython_6cygrpc__GrpcArgWrapper.tp_print = 0;
  if ((CYTHON_USE_TYPE_SLOTS && CYTHON_USE_PYTYPE_LOOKUP) && likely(!__pyx_type_4grpc_7_cython_6cygrpc__GrpcArgWrapper.tp_dictoffset && __pyx_type_4grpc_7_cython_6cygrpc__GrpcArgWrapper.tp_getattro == PyObject_GenericGetAttr)) {
    __pyx_type_4grpc_7_cython_6cygrpc__GrpcArgWrapper.tp_getattro = __Pyx_PyObject_GenericGetAttr;
  }
  if (PyObject_SetAttrString(__pyx_m, "_GrpcArgWrapper", (PyObject *)&__pyx_type_4grpc_7_cython_6cygrpc__GrpcArgWrapper) < 0) __PYX_ERR(2, 18, __pyx_L1_error)
  if (__Pyx_setup_reduce((PyObject*)&__pyx_type_4grpc_7_cython_6cygrpc__GrpcArgWrapper) < 0) __PYX_ERR(2, 18, __pyx_L1_error)
  __pyx_ptype_4grpc_7_cython_6cygrpc__GrpcArgWrapper = &__pyx_type_4grpc_7_cython_6cygrpc__GrpcArgWrapper;
  if (PyType_Ready(&__pyx_type_4grpc_7_cython_6cygrpc_CensusContext) < 0) __PYX_ERR(20, 28, __pyx_L1_error)
  __pyx_type_4grpc_7_cython_6cygrpc_CensusContext.tp_print = 0;
  if ((CYTHON_USE_TYPE_SLOTS && CYTHON_USE_PYTYPE_LOOKUP) && likely(!__pyx_type_4grpc_7_cython_6cygrpc_CensusContext.tp_dictoffset && __pyx_type_4grpc_7_cython_6cygrpc_CensusContext.tp_getattro == PyObject_GenericGetAttr)) {
    __pyx_type_4grpc_7_cython_6cygrpc_CensusContext.tp_getattro = __Pyx_PyObject_GenericGetAttr;
  }
  if (PyObject_SetAttrString(__pyx_m, "CensusContext", (PyObject *)&__pyx_type_4grpc_7_cython_6cygrpc_CensusContext) < 0) __PYX_ERR(20, 28, __pyx_L1_error)
  if (__Pyx_setup_reduce((PyObject*)&__pyx_type_4grpc_7_cython_6cygrpc_CensusContext) < 0) __PYX_ERR(20, 28, __pyx_L1_error)
  __pyx_ptype_4grpc_7_cython_6cygrpc_CensusContext = &__pyx_type_4grpc_7_cython_6cygrpc_CensusContext;
  if (PyType_Ready(&__pyx_type_4grpc_7_cython_6cygrpc___pyx_scope_struct____pyx_f_4grpc_7_cython_6cygrpc__integrated_call) < 0) __PYX_ERR(8, 262, __pyx_L1_error)
  __pyx_type_4grpc_7_cython_6cygrpc___pyx_scope_struct____pyx_f_4grpc_7_cython_6cygrpc__integrated_call.tp_print = 0;
  if ((CYTHON_USE_TYPE_SLOTS && CYTHON_USE_PYTYPE_LOOKUP) && likely(!__pyx_type_4grpc_7_cython_6cygrpc___pyx_scope_struct____pyx_f_4grpc_7_cython_6cygrpc__integrated_call.tp_dictoffset && __pyx_type_4grpc_7_cython_6cygrpc___pyx_scope_struct____pyx_f_4grpc_7_cython_6cygrpc__integrated_call.tp_getattro == PyObject_GenericGetAttr)) {
    __pyx_type_4grpc_7_cython_6cygrpc___pyx_scope_struct____pyx_f_4grpc_7_cython_6cygrpc__integrated_call.tp_getattro = __Pyx_PyObject_GenericGetAttrNoDict;
  }
  __pyx_ptype_4grpc_7_cython_6cygrpc___pyx_scope_struct____pyx_f_4grpc_7_cython_6cygrpc__integrated_call = &__pyx_type_4grpc_7_cython_6cygrpc___pyx_scope_struct____pyx_f_4grpc_7_cython_6cygrpc__integrated_call;
  if (PyType_Ready(&__pyx_type_4grpc_7_cython_6cygrpc___pyx_scope_struct_1_next_event) < 0) __PYX_ERR(8, 306, __pyx_L1_error)
  __pyx_type_4grpc_7_cython_6cygrpc___pyx_scope_struct_1_next_event.tp_print = 0;
  if ((CYTHON_USE_TYPE_SLOTS && CYTHON_USE_PYTYPE_LOOKUP) && likely(!__pyx_type_4grpc_7_cython_6cygrpc___pyx_scope_struct_1_next_event.tp_dictoffset && __pyx_type_4grpc_7_cython_6cygrpc___pyx_scope_struct_1_next_event.tp_getattro == PyObject_GenericGetAttr)) {
    __pyx_type_4grpc_7_cython_6cygrpc___pyx_scope_struct_1_next_event.tp_getattro = __Pyx_PyObject_GenericGetAttrNoDict;
  }
  __pyx_ptype_4grpc_7_cython_6cygrpc___pyx_scope_struct_1_next_event = &__pyx_type_4grpc_7_cython_6cygrpc___pyx_scope_struct_1_next_event;
  if (PyType_Ready(&__pyx_type_4grpc_7_cython_6cygrpc___pyx_scope_struct_2___pyx_f_4grpc_7_cython_6cygrpc__segregated_call) < 0) __PYX_ERR(8, 314, __pyx_L1_error)
  __pyx_type_4grpc_7_cython_6cygrpc___pyx_scope_struct_2___pyx_f_4grpc_7_cython_6cygrpc__segregated_call.tp_print = 0;
  if ((CYTHON_USE_TYPE_SLOTS && CYTHON_USE_PYTYPE_LOOKUP) && likely(!__pyx_type_4grpc_7_cython_6cygrpc___pyx_scope_struct_2___pyx_f_4grpc_7_cython_6cygrpc__segregated_call.tp_dictoffset && __pyx_type_4grpc_7_cython_6cygrpc___pyx_scope_struct_2___pyx_f_4grpc_7_cython_6cygrpc__segregated_call.tp_getattro == PyObject_GenericGetAttr)) {
    __pyx_type_4grpc_7_cython_6cygrpc___pyx_scope_struct_2___pyx_f_4grpc_7_cython_6cygrpc__segregated_call.tp_getattro = __Pyx_PyObject_GenericGetAttrNoDict;
  }
  __pyx_ptype_4grpc_7_cython_6cygrpc___pyx_scope_struct_2___pyx_f_4grpc_7_cython_6cygrpc__segregated_call = &__pyx_type_4grpc_7_cython_6cygrpc___pyx_scope_struct_2___pyx_f_4grpc_7_cython_6cygrpc__segregated_call;
  if (PyType_Ready(&__pyx_type_4grpc_7_cython_6cygrpc___pyx_scope_struct_3_next_call_event) < 0) __PYX_ERR(8, 456, __pyx_L1_error)
  __pyx_type_4grpc_7_cython_6cygrpc___pyx_scope_struct_3_next_call_event.tp_print = 0;
  if ((CYTHON_USE_TYPE_SLOTS && CYTHON_USE_PYTYPE_LOOKUP) && likely(!__pyx_type_4grpc_7_cython_6cygrpc___pyx_scope_struct_3_next_call_event.tp_dictoffset && __pyx_type_4grpc_7_cython_6cygrpc___pyx_scope_struct_3_next_call_event.tp_getattro == PyObject_GenericGetAttr)) {
    __pyx_type_4grpc_7_cython_6cygrpc___pyx_scope_struct_3_next_call_event.tp_getattro = __Pyx_PyObject_GenericGetAttrNoDict;
  }
  __pyx_ptype_4grpc_7_cython_6cygrpc___pyx_scope_struct_3_next_call_event = &__pyx_type_4grpc_7_cython_6cygrpc___pyx_scope_struct_3_next_call_event;
  if (PyType_Ready(&__pyx_type_4grpc_7_cython_6cygrpc___pyx_scope_struct_4___pyx_f_4grpc_7_cython_6cygrpc__get_metadata) < 0) __PYX_ERR(4, 40, __pyx_L1_error)
  __pyx_type_4grpc_7_cython_6cygrpc___pyx_scope_struct_4___pyx_f_4grpc_7_cython_6cygrpc__get_metadata.tp_print = 0;
  if ((CYTHON_USE_TYPE_SLOTS && CYTHON_USE_PYTYPE_LOOKUP) && likely(!__pyx_type_4grpc_7_cython_6cygrpc___pyx_scope_struct_4___pyx_f_4grpc_7_cython_6cygrpc__get_metadata.tp_dictoffset && __pyx_type_4grpc_7_cython_6cygrpc___pyx_scope_struct_4___pyx_f_4grpc_7_cython_6cygrpc__get_metadata.tp_getattro == PyObject_GenericGetAttr)) {
    __pyx_type_4grpc_7_cython_6cygrpc___pyx_scope_struct_4___pyx_f_4grpc_7_cython_6cygrpc__get_metadata.tp_getattro = __Pyx_PyObject_GenericGetAttrNoDict;
  }
  __pyx_ptype_4grpc_7_cython_6cygrpc___pyx_scope_struct_4___pyx_f_4grpc_7_cython_6cygrpc__get_metadata = &__pyx_type_4grpc_7_cython_6cygrpc___pyx_scope_struct_4___pyx_f_4grpc_7_cython_6cygrpc__get_metadata;
  if (PyType_Ready(&__pyx_type_4grpc_7_cython_6cygrpc___pyx_scope_struct_5___pyx_f_4grpc_7_cython_6cygrpc__metadata) < 0) __PYX_ERR(13, 63, __pyx_L1_error)
  __pyx_type_4grpc_7_cython_6cygrpc___pyx_scope_struct_5___pyx_f_4grpc_7_cython_6cygrpc__metadata.tp_print = 0;
  if ((CYTHON_USE_TYPE_SLOTS && CYTHON_USE_PYTYPE_LOOKUP) && likely(!__pyx_type_4grpc_7_cython_6cygrpc___pyx_scope_struct_5___pyx_f_4grpc_7_cython_6cygrpc__metadata.tp_dictoffset && __pyx_type_4grpc_7_cython_6cygrpc___pyx_scope_struct_5___pyx_f_4grpc_7_cython_6cygrpc__metadata.tp_getattro == PyObject_GenericGetAttr)) {
    __pyx_type_4grpc_7_cython_6cygrpc___pyx_scope_struct_5___pyx_f_4grpc_7_cython_6cygrpc__metadata.tp_getattro = __Pyx_PyObject_GenericGetAttrNoDict;
  }
  __pyx_ptype_4grpc_7_cython_6cygrpc___pyx_scope_struct_5___pyx_f_4grpc_7_cython_6cygrpc__metadata = &__pyx_type_4grpc_7_cython_6cygrpc___pyx_scope_struct_5___pyx_f_4grpc_7_cython_6cygrpc__metadata;
  if (PyType_Ready(&__pyx_type_4grpc_7_cython_6cygrpc___pyx_scope_struct_6_genexpr) < 0) __PYX_ERR(13, 65, __pyx_L1_error)
  __pyx_type_4grpc_7_cython_6cygrpc___pyx_scope_struct_6_genexpr.tp_print = 0;
  if ((CYTHON_USE_TYPE_SLOTS && CYTHON_USE_PYTYPE_LOOKUP) && likely(!__pyx_type_4grpc_7_cython_6cygrpc___pyx_scope_struct_6_genexpr.tp_dictoffset && __pyx_type_4grpc_7_cython_6cygrpc___pyx_scope_struct_6_genexpr.tp_getattro == PyObject_GenericGetAttr)) {
    __pyx_type_4grpc_7_cython_6cygrpc___pyx_scope_struct_6_genexpr.tp_getattro = __Pyx_PyObject_GenericGetAttrNoDict;
  }
  __pyx_ptype_4grpc_7_cython_6cygrpc___pyx_scope_struct_6_genexpr = &__pyx_type_4grpc_7_cython_6cygrpc___pyx_scope_struct_6_genexpr;
  if (PyType_Ready(&__pyx_type_4grpc_7_cython_6cygrpc___pyx_scope_struct_7___pyx_f_4grpc_7_cython_6cygrpc_tuples_to_resolvaddr) < 0) __PYX_ERR(6, 59, __pyx_L1_error)
  __pyx_type_4grpc_7_cython_6cygrpc___pyx_scope_struct_7___pyx_f_4grpc_7_cython_6cygrpc_tuples_to_resolvaddr.tp_print = 0;
  if ((CYTHON_USE_TYPE_SLOTS && CYTHON_USE_PYTYPE_LOOKUP) && likely(!__pyx_type_4grpc_7_cython_6cygrpc___pyx_scope_struct_7___pyx_f_4grpc_7_cython_6cygrpc_tuples_to_resolvaddr.tp_dictoffset && __pyx_type_4grpc_7_cython_6cygrpc___pyx_scope_struct_7___pyx_f_4grpc_7_cython_6cygrpc_tuples_to_resolvaddr.tp_getattro == PyObject_GenericGetAttr)) {
    __pyx_type_4grpc_7_cython_6cygrpc___pyx_scope_struct_7___pyx_f_4grpc_7_cython_6cygrpc_tuples_to_resolvaddr.tp_getattro = __Pyx_PyObject_GenericGetAttrNoDict;
  }
  __pyx_ptype_4grpc_7_cython_6cygrpc___pyx_scope_struct_7___pyx_f_4grpc_7_cython_6cygrpc_tuples_to_resolvaddr = &__pyx_type_4grpc_7_cython_6cygrpc___pyx_scope_struct_7___pyx_f_4grpc_7_cython_6cygrpc_tuples_to_resolvaddr;
  if (PyType_Ready(&__pyx_type_4grpc_7_cython_6cygrpc___pyx_scope_struct_8_genexpr) < 0) __PYX_ERR(6, 61, __pyx_L1_error)
  __pyx_type_4grpc_7_cython_6cygrpc___pyx_scope_struct_8_genexpr.tp_print = 0;
  if ((CYTHON_USE_TYPE_SLOTS && CYTHON_USE_PYTYPE_LOOKUP) && likely(!__pyx_type_4grpc_7_cython_6cygrpc___pyx_scope_struct_8_genexpr.tp_dictoffset && __pyx_type_4grpc_7_cython_6cygrpc___pyx_scope_struct_8_genexpr.tp_getattro == PyObject_GenericGetAttr)) {
    __pyx_type_4grpc_7_cython_6cygrpc___pyx_scope_struct_8_genexpr.tp_getattro = __Pyx_PyObject_GenericGetAttrNoDict;
  }
  __pyx_ptype_4grpc_7_cython_6cygrpc___pyx_scope_struct_8_genexpr = &__pyx_type_4grpc_7_cython_6cygrpc___pyx_scope_struct_8_genexpr;
  if (PyType_Ready(&__pyx_type_4grpc_7_cython_6cygrpc___pyx_scope_struct_9___init__) < 0) __PYX_ERR(0, 102, __pyx_L1_error)
  __pyx_type_4grpc_7_cython_6cygrpc___pyx_scope_struct_9___init__.tp_print = 0;
  if ((CYTHON_USE_TYPE_SLOTS && CYTHON_USE_PYTYPE_LOOKUP) && likely(!__pyx_type_4grpc_7_cython_6cygrpc___pyx_scope_struct_9___init__.tp_dictoffset && __pyx_type_4grpc_7_cython_6cygrpc___pyx_scope_struct_9___init__.tp_getattro == PyObject_GenericGetAttr)) {
    __pyx_type_4grpc_7_cython_6cygrpc___pyx_scope_struct_9___init__.tp_getattro = __Pyx_PyObject_GenericGetAttrNoDict;
  }
  __pyx_ptype_4grpc_7_cython_6cygrpc___pyx_scope_struct_9___init__ = &__pyx_type_4grpc_7_cython_6cygrpc___pyx_scope_struct_9___init__;
  __Pyx_RefNannyFinishContext();
  return 0;
  __pyx_L1_error:;
  __Pyx_RefNannyFinishContext();
  return -1;
}

static int __Pyx_modinit_type_import_code(void) {
  __Pyx_RefNannyDeclarations
  __Pyx_RefNannySetupContext("__Pyx_modinit_type_import_code", 0);
  /*--- Type import code ---*/
  __pyx_ptype_7cpython_4type_type = __Pyx_ImportType(__Pyx_BUILTIN_MODULE_NAME, "type", 
  #if defined(PYPY_VERSION_NUM) && PYPY_VERSION_NUM < 0x050B0000
  sizeof(PyTypeObject),
  #else
  sizeof(PyHeapTypeObject),
  #endif
  0); if (unlikely(!__pyx_ptype_7cpython_4type_type)) __PYX_ERR(23, 9, __pyx_L1_error)
  __pyx_ptype_7cpython_4bool_bool = __Pyx_ImportType(__Pyx_BUILTIN_MODULE_NAME, "bool", sizeof(PyBoolObject), 0); if (unlikely(!__pyx_ptype_7cpython_4bool_bool)) __PYX_ERR(24, 8, __pyx_L1_error)
  __pyx_ptype_7cpython_7complex_complex = __Pyx_ImportType(__Pyx_BUILTIN_MODULE_NAME, "complex", sizeof(PyComplexObject), 0); if (unlikely(!__pyx_ptype_7cpython_7complex_complex)) __PYX_ERR(25, 15, __pyx_L1_error)
  __Pyx_RefNannyFinishContext();
  return 0;
  __pyx_L1_error:;
  __Pyx_RefNannyFinishContext();
  return -1;
}

static int __Pyx_modinit_variable_import_code(void) {
  __Pyx_RefNannyDeclarations
  __Pyx_RefNannySetupContext("__Pyx_modinit_variable_import_code", 0);
  /*--- Variable import code ---*/
  __Pyx_RefNannyFinishContext();
  return 0;
}

static int __Pyx_modinit_function_import_code(void) {
  __Pyx_RefNannyDeclarations
  __Pyx_RefNannySetupContext("__Pyx_modinit_function_import_code", 0);
  /*--- Function import code ---*/
  __Pyx_RefNannyFinishContext();
  return 0;
}


#if PY_MAJOR_VERSION < 3
#ifdef CYTHON_NO_PYINIT_EXPORT
#define __Pyx_PyMODINIT_FUNC void
#else
#define __Pyx_PyMODINIT_FUNC PyMODINIT_FUNC
#endif
#else
#ifdef CYTHON_NO_PYINIT_EXPORT
#define __Pyx_PyMODINIT_FUNC PyObject *
#else
#define __Pyx_PyMODINIT_FUNC PyMODINIT_FUNC
#endif
#endif
#ifndef CYTHON_SMALL_CODE
#if defined(__clang__)
    #define CYTHON_SMALL_CODE
#elif defined(__GNUC__) && (!(defined(__cplusplus)) || (__GNUC__ > 4 || (__GNUC__ == 4 && __GNUC_MINOR__ > 4)))
    #define CYTHON_SMALL_CODE __attribute__((optimize("Os")))
#else
    #define CYTHON_SMALL_CODE
#endif
#endif


#if PY_MAJOR_VERSION < 3
__Pyx_PyMODINIT_FUNC initcygrpc(void) CYTHON_SMALL_CODE; /*proto*/
__Pyx_PyMODINIT_FUNC initcygrpc(void)
#else
__Pyx_PyMODINIT_FUNC PyInit_cygrpc(void) CYTHON_SMALL_CODE; /*proto*/
__Pyx_PyMODINIT_FUNC PyInit_cygrpc(void)
#if CYTHON_PEP489_MULTI_PHASE_INIT
{
  return PyModuleDef_Init(&__pyx_moduledef);
}
static int __Pyx_copy_spec_to_module(PyObject *spec, PyObject *moddict, const char* from_name, const char* to_name) {
    PyObject *value = PyObject_GetAttrString(spec, from_name);
    int result = 0;
    if (likely(value)) {
        result = PyDict_SetItemString(moddict, to_name, value);
        Py_DECREF(value);
    } else if (PyErr_ExceptionMatches(PyExc_AttributeError)) {
        PyErr_Clear();
    } else {
        result = -1;
    }
    return result;
}
static PyObject* __pyx_pymod_create(PyObject *spec, CYTHON_UNUSED PyModuleDef *def) {
    PyObject *module = NULL, *moddict, *modname;
    if (__pyx_m)
        return __Pyx_NewRef(__pyx_m);
    modname = PyObject_GetAttrString(spec, "name");
    if (unlikely(!modname)) goto bad;
    module = PyModule_NewObject(modname);
    Py_DECREF(modname);
    if (unlikely(!module)) goto bad;
    moddict = PyModule_GetDict(module);
    if (unlikely(!moddict)) goto bad;
    if (unlikely(__Pyx_copy_spec_to_module(spec, moddict, "loader", "__loader__") < 0)) goto bad;
    if (unlikely(__Pyx_copy_spec_to_module(spec, moddict, "origin", "__file__") < 0)) goto bad;
    if (unlikely(__Pyx_copy_spec_to_module(spec, moddict, "parent", "__package__") < 0)) goto bad;
    if (unlikely(__Pyx_copy_spec_to_module(spec, moddict, "submodule_search_locations", "__path__") < 0)) goto bad;
    return module;
bad:
    Py_XDECREF(module);
    return NULL;
}


static int __pyx_pymod_exec_cygrpc(PyObject *__pyx_pyinit_module)
#endif
#endif
{
  PyObject *__pyx_t_1 = NULL;
  PyObject *__pyx_t_2 = NULL;
  PyObject *__pyx_t_3 = NULL;
  int __pyx_t_4;
  __Pyx_RefNannyDeclarations
  #if CYTHON_PEP489_MULTI_PHASE_INIT
  if (__pyx_m && __pyx_m == __pyx_pyinit_module) return 0;
  #elif PY_MAJOR_VERSION >= 3
  if (__pyx_m) return __Pyx_NewRef(__pyx_m);
  #endif
  #if CYTHON_REFNANNY
__Pyx_RefNanny = __Pyx_RefNannyImportAPI("refnanny");
if (!__Pyx_RefNanny) {
  PyErr_Clear();
  __Pyx_RefNanny = __Pyx_RefNannyImportAPI("Cython.Runtime.refnanny");
  if (!__Pyx_RefNanny)
      Py_FatalError("failed to import 'refnanny' module");
}
#endif
  __Pyx_RefNannySetupContext("__Pyx_PyMODINIT_FUNC PyInit_cygrpc(void)", 0);
  if (__Pyx_check_binary_version() < 0) __PYX_ERR(21, 1, __pyx_L1_error)
  __pyx_empty_tuple = PyTuple_New(0); if (unlikely(!__pyx_empty_tuple)) __PYX_ERR(21, 1, __pyx_L1_error)
  __pyx_empty_bytes = PyBytes_FromStringAndSize("", 0); if (unlikely(!__pyx_empty_bytes)) __PYX_ERR(21, 1, __pyx_L1_error)
  __pyx_empty_unicode = PyUnicode_FromStringAndSize("", 0); if (unlikely(!__pyx_empty_unicode)) __PYX_ERR(21, 1, __pyx_L1_error)
  #ifdef __Pyx_CyFunction_USED
  if (__pyx_CyFunction_init() < 0) __PYX_ERR(21, 1, __pyx_L1_error)
  #endif
  #ifdef __Pyx_FusedFunction_USED
  if (__pyx_FusedFunction_init() < 0) __PYX_ERR(21, 1, __pyx_L1_error)
  #endif
  #ifdef __Pyx_Coroutine_USED
  if (__pyx_Coroutine_init() < 0) __PYX_ERR(21, 1, __pyx_L1_error)
  #endif
  #ifdef __Pyx_Generator_USED
  if (__pyx_Generator_init() < 0) __PYX_ERR(21, 1, __pyx_L1_error)
  #endif
  #ifdef __Pyx_AsyncGen_USED
  if (__pyx_AsyncGen_init() < 0) __PYX_ERR(21, 1, __pyx_L1_error)
  #endif
  #ifdef __Pyx_StopAsyncIteration_USED
  if (__pyx_StopAsyncIteration_init() < 0) __PYX_ERR(21, 1, __pyx_L1_error)
  #endif
  /*--- Library function declarations ---*/
  /*--- Threads initialization code ---*/
  #if defined(__PYX_FORCE_INIT_THREADS) && __PYX_FORCE_INIT_THREADS
  #ifdef WITH_THREAD /* Python build with threading support? */
  PyEval_InitThreads();
  #endif
  #endif
  /*--- Module creation code ---*/
  #if CYTHON_PEP489_MULTI_PHASE_INIT
  __pyx_m = __pyx_pyinit_module;
  Py_INCREF(__pyx_m);
  #else
  #if PY_MAJOR_VERSION < 3
  __pyx_m = Py_InitModule4("cygrpc", __pyx_methods, 0, 0, PYTHON_API_VERSION); Py_XINCREF(__pyx_m);
  #else
  __pyx_m = PyModule_Create(&__pyx_moduledef);
  #endif
  if (unlikely(!__pyx_m)) __PYX_ERR(21, 1, __pyx_L1_error)
  #endif
  __pyx_d = PyModule_GetDict(__pyx_m); if (unlikely(!__pyx_d)) __PYX_ERR(21, 1, __pyx_L1_error)
  Py_INCREF(__pyx_d);
  __pyx_b = PyImport_AddModule(__Pyx_BUILTIN_MODULE_NAME); if (unlikely(!__pyx_b)) __PYX_ERR(21, 1, __pyx_L1_error)
  __pyx_cython_runtime = PyImport_AddModule((char *) "cython_runtime"); if (unlikely(!__pyx_cython_runtime)) __PYX_ERR(21, 1, __pyx_L1_error)
  #if CYTHON_COMPILING_IN_PYPY
  Py_INCREF(__pyx_b);
  #endif
  if (PyObject_SetAttrString(__pyx_m, "__builtins__", __pyx_b) < 0) __PYX_ERR(21, 1, __pyx_L1_error);
  /*--- Initialize various global constants etc. ---*/
  if (__Pyx_InitGlobals() < 0) __PYX_ERR(21, 1, __pyx_L1_error)
  #if PY_MAJOR_VERSION < 3 && (__PYX_DEFAULT_STRING_ENCODING_IS_ASCII || __PYX_DEFAULT_STRING_ENCODING_IS_DEFAULT)
  if (__Pyx_init_sys_getdefaultencoding_params() < 0) __PYX_ERR(21, 1, __pyx_L1_error)
  #endif
  if (__pyx_module_is_main_grpc___cython__cygrpc) {
    if (PyObject_SetAttrString(__pyx_m, "__name__", __pyx_n_s_main) < 0) __PYX_ERR(21, 1, __pyx_L1_error)
  }
  #if PY_MAJOR_VERSION >= 3
  {
    PyObject *modules = PyImport_GetModuleDict(); if (unlikely(!modules)) __PYX_ERR(21, 1, __pyx_L1_error)
    if (!PyDict_GetItemString(modules, "grpc._cython.cygrpc")) {
      if (unlikely(PyDict_SetItemString(modules, "grpc._cython.cygrpc", __pyx_m) < 0)) __PYX_ERR(21, 1, __pyx_L1_error)
    }
  }
  #endif
  /*--- Builtin init code ---*/
  if (__Pyx_InitCachedBuiltins() < 0) __PYX_ERR(21, 1, __pyx_L1_error)
  /*--- Constants init code ---*/
  if (__Pyx_InitCachedConstants() < 0) __PYX_ERR(21, 1, __pyx_L1_error)
  /*--- Global type/function init code ---*/
  (void)__Pyx_modinit_global_init_code();
  (void)__Pyx_modinit_variable_export_code();
  if (unlikely(__Pyx_modinit_function_export_code() != 0)) goto __pyx_L1_error;
  if (unlikely(__Pyx_modinit_type_init_code() != 0)) goto __pyx_L1_error;
  if (unlikely(__Pyx_modinit_type_import_code() != 0)) goto __pyx_L1_error;
  (void)__Pyx_modinit_variable_import_code();
  (void)__Pyx_modinit_function_import_code();
  /*--- Execution code ---*/
  #if defined(__Pyx_Generator_USED) || defined(__Pyx_Coroutine_USED)
  if (__Pyx_patch_abc() < 0) __PYX_ERR(21, 1, __pyx_L1_error)
  #endif

  /* "grpc/_cython/cygrpc.pyx":18
 * cimport cpython
 * 
 * import os.path             # <<<<<<<<<<<<<<
 * import sys
 * 
 */
  __pyx_t_1 = __Pyx_Import(__pyx_n_s_os_path, 0, -1); if (unlikely(!__pyx_t_1)) __PYX_ERR(21, 18, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_1);
  if (PyDict_SetItem(__pyx_d, __pyx_n_s_os, __pyx_t_1) < 0) __PYX_ERR(21, 18, __pyx_L1_error)
  __Pyx_DECREF(__pyx_t_1); __pyx_t_1 = 0;

  /* "grpc/_cython/cygrpc.pyx":19
 * 
 * import os.path
 * import sys             # <<<<<<<<<<<<<<
 * 
 * # TODO(atash): figure out why the coverage tool gets confused about the Cython
 */
  __pyx_t_1 = __Pyx_Import(__pyx_n_s_sys, 0, -1); if (unlikely(!__pyx_t_1)) __PYX_ERR(21, 19, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_1);
  if (PyDict_SetItem(__pyx_d, __pyx_n_s_sys, __pyx_t_1) < 0) __PYX_ERR(21, 19, __pyx_L1_error)
  __Pyx_DECREF(__pyx_t_1); __pyx_t_1 = 0;

  /* "src/python/grpcio/grpc/_cython/_cygrpc/grpc_string.pyx.pxi":15
 * # limitations under the License.
 * 
 * import logging             # <<<<<<<<<<<<<<
 * 
 * _LOGGER = logging.getLogger(__name__)
 */
  __pyx_t_1 = __Pyx_Import(__pyx_n_s_logging, 0, -1); if (unlikely(!__pyx_t_1)) __PYX_ERR(1, 15, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_1);
  if (PyDict_SetItem(__pyx_d, __pyx_n_s_logging, __pyx_t_1) < 0) __PYX_ERR(1, 15, __pyx_L1_error)
  __Pyx_DECREF(__pyx_t_1); __pyx_t_1 = 0;

  /* "src/python/grpcio/grpc/_cython/_cygrpc/grpc_string.pyx.pxi":17
 * import logging
 * 
 * _LOGGER = logging.getLogger(__name__)             # <<<<<<<<<<<<<<
 * 
 * # This function will ascii encode unicode string inputs if neccesary.
 */
  __pyx_t_1 = __Pyx_GetModuleGlobalName(__pyx_n_s_logging); if (unlikely(!__pyx_t_1)) __PYX_ERR(1, 17, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_1);
  __pyx_t_2 = __Pyx_PyObject_GetAttrStr(__pyx_t_1, __pyx_n_s_getLogger); if (unlikely(!__pyx_t_2)) __PYX_ERR(1, 17, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_2);
  __Pyx_DECREF(__pyx_t_1); __pyx_t_1 = 0;
  __pyx_t_1 = __Pyx_GetModuleGlobalName(__pyx_n_s_name_2); if (unlikely(!__pyx_t_1)) __PYX_ERR(1, 17, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_1);
  __pyx_t_3 = __Pyx_PyObject_CallOneArg(__pyx_t_2, __pyx_t_1); if (unlikely(!__pyx_t_3)) __PYX_ERR(1, 17, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_3);
  __Pyx_DECREF(__pyx_t_2); __pyx_t_2 = 0;
  __Pyx_DECREF(__pyx_t_1); __pyx_t_1 = 0;
  if (PyDict_SetItem(__pyx_d, __pyx_n_s_LOGGER, __pyx_t_3) < 0) __PYX_ERR(1, 17, __pyx_L1_error)
  __Pyx_DECREF(__pyx_t_3); __pyx_t_3 = 0;

  /* "src/python/grpcio/grpc/_cython/_cygrpc/call.pyx.pxi":49
 * 
 *   def cancel(
 *       self, grpc_status_code error_code=GRPC_STATUS__DO_NOT_USE,             # <<<<<<<<<<<<<<
 *       details=None):
 *     details = str_to_bytes(details)
 */
  __pyx_k__11 = GRPC_STATUS__DO_NOT_USE;

  /* "src/python/grpcio/grpc/_cython/_cygrpc/channel.pyx.pxi":17
 * cimport cpython
 * 
 * import threading             # <<<<<<<<<<<<<<
 * import time
 * 
 */
  __pyx_t_3 = __Pyx_Import(__pyx_n_s_threading, 0, -1); if (unlikely(!__pyx_t_3)) __PYX_ERR(8, 17, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_3);
  if (PyDict_SetItem(__pyx_d, __pyx_n_s_threading, __pyx_t_3) < 0) __PYX_ERR(8, 17, __pyx_L1_error)
  __Pyx_DECREF(__pyx_t_3); __pyx_t_3 = 0;

  /* "src/python/grpcio/grpc/_cython/_cygrpc/channel.pyx.pxi":18
 * 
 * import threading
 * import time             # <<<<<<<<<<<<<<
 * 
 * _INTERNAL_CALL_ERROR_MESSAGE_FORMAT = (
 */
  __pyx_t_3 = __Pyx_Import(__pyx_n_s_time, 0, -1); if (unlikely(!__pyx_t_3)) __PYX_ERR(8, 18, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_3);
  if (PyDict_SetItem(__pyx_d, __pyx_n_s_time, __pyx_t_3) < 0) __PYX_ERR(8, 18, __pyx_L1_error)
  __Pyx_DECREF(__pyx_t_3); __pyx_t_3 = 0;

  /* "src/python/grpcio/grpc/_cython/_cygrpc/channel.pyx.pxi":21
 * 
 * _INTERNAL_CALL_ERROR_MESSAGE_FORMAT = (
 *     'Internal gRPC call error %d. ' +             # <<<<<<<<<<<<<<
 *     'Please report to https://github.com/grpc/grpc/issues')
 * 
 */
  __pyx_t_3 = PyNumber_Add(__pyx_kp_s_Internal_gRPC_call_error_d, __pyx_kp_s_Please_report_to_https_github_co); if (unlikely(!__pyx_t_3)) __PYX_ERR(8, 21, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_3);
  if (PyDict_SetItem(__pyx_d, __pyx_n_s_INTERNAL_CALL_ERROR_MESSAGE_FOR, __pyx_t_3) < 0) __PYX_ERR(8, 20, __pyx_L1_error)
  __Pyx_DECREF(__pyx_t_3); __pyx_t_3 = 0;

  /* "src/python/grpcio/grpc/_cython/_cygrpc/channelz.pyx.pxi":16
 * 
 * 
 * def channelz_get_top_channels(start_channel_id):             # <<<<<<<<<<<<<<
 *     cdef char *c_returned_str = grpc_channelz_get_top_channels(
 *         start_channel_id,
 */
  __pyx_t_3 = PyCFunction_NewEx(&__pyx_mdef_4grpc_7_cython_6cygrpc_1channelz_get_top_channels, NULL, __pyx_n_s_grpc__cython_cygrpc); if (unlikely(!__pyx_t_3)) __PYX_ERR(9, 16, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_3);
  if (PyDict_SetItem(__pyx_d, __pyx_n_s_channelz_get_top_channels, __pyx_t_3) < 0) __PYX_ERR(9, 16, __pyx_L1_error)
  __Pyx_DECREF(__pyx_t_3); __pyx_t_3 = 0;

  /* "src/python/grpcio/grpc/_cython/_cygrpc/channelz.pyx.pxi":25
 *     return c_returned_str
 * 
 * def channelz_get_servers(start_server_id):             # <<<<<<<<<<<<<<
 *     cdef char *c_returned_str = grpc_channelz_get_servers(start_server_id)
 *     if c_returned_str == NULL:
 */
  __pyx_t_3 = PyCFunction_NewEx(&__pyx_mdef_4grpc_7_cython_6cygrpc_3channelz_get_servers, NULL, __pyx_n_s_grpc__cython_cygrpc); if (unlikely(!__pyx_t_3)) __PYX_ERR(9, 25, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_3);
  if (PyDict_SetItem(__pyx_d, __pyx_n_s_channelz_get_servers, __pyx_t_3) < 0) __PYX_ERR(9, 25, __pyx_L1_error)
  __Pyx_DECREF(__pyx_t_3); __pyx_t_3 = 0;

  /* "src/python/grpcio/grpc/_cython/_cygrpc/channelz.pyx.pxi":32
 *     return c_returned_str
 * 
 * def channelz_get_server(server_id):             # <<<<<<<<<<<<<<
 *     cdef char *c_returned_str = grpc_channelz_get_server(server_id)
 *     if c_returned_str == NULL:
 */
  __pyx_t_3 = PyCFunction_NewEx(&__pyx_mdef_4grpc_7_cython_6cygrpc_5channelz_get_server, NULL, __pyx_n_s_grpc__cython_cygrpc); if (unlikely(!__pyx_t_3)) __PYX_ERR(9, 32, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_3);
  if (PyDict_SetItem(__pyx_d, __pyx_n_s_channelz_get_server, __pyx_t_3) < 0) __PYX_ERR(9, 32, __pyx_L1_error)
  __Pyx_DECREF(__pyx_t_3); __pyx_t_3 = 0;

  /* "src/python/grpcio/grpc/_cython/_cygrpc/channelz.pyx.pxi":39
 *     return c_returned_str
 * 
 * def channelz_get_server_sockets(server_id, start_socket_id, max_results):             # <<<<<<<<<<<<<<
 *     cdef char *c_returned_str = grpc_channelz_get_server_sockets(
 *         server_id,
 */
  __pyx_t_3 = PyCFunction_NewEx(&__pyx_mdef_4grpc_7_cython_6cygrpc_7channelz_get_server_sockets, NULL, __pyx_n_s_grpc__cython_cygrpc); if (unlikely(!__pyx_t_3)) __PYX_ERR(9, 39, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_3);
  if (PyDict_SetItem(__pyx_d, __pyx_n_s_channelz_get_server_sockets, __pyx_t_3) < 0) __PYX_ERR(9, 39, __pyx_L1_error)
  __Pyx_DECREF(__pyx_t_3); __pyx_t_3 = 0;

  /* "src/python/grpcio/grpc/_cython/_cygrpc/channelz.pyx.pxi":52
 *     return c_returned_str
 * 
 * def channelz_get_channel(channel_id):             # <<<<<<<<<<<<<<
 *     cdef char *c_returned_str = grpc_channelz_get_channel(channel_id)
 *     if c_returned_str == NULL:
 */
  __pyx_t_3 = PyCFunction_NewEx(&__pyx_mdef_4grpc_7_cython_6cygrpc_9channelz_get_channel, NULL, __pyx_n_s_grpc__cython_cygrpc); if (unlikely(!__pyx_t_3)) __PYX_ERR(9, 52, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_3);
  if (PyDict_SetItem(__pyx_d, __pyx_n_s_channelz_get_channel, __pyx_t_3) < 0) __PYX_ERR(9, 52, __pyx_L1_error)
  __Pyx_DECREF(__pyx_t_3); __pyx_t_3 = 0;

  /* "src/python/grpcio/grpc/_cython/_cygrpc/channelz.pyx.pxi":59
 *     return c_returned_str
 * 
 * def channelz_get_subchannel(subchannel_id):             # <<<<<<<<<<<<<<
 *     cdef char *c_returned_str = grpc_channelz_get_subchannel(subchannel_id)
 *     if c_returned_str == NULL:
 */
  __pyx_t_3 = PyCFunction_NewEx(&__pyx_mdef_4grpc_7_cython_6cygrpc_11channelz_get_subchannel, NULL, __pyx_n_s_grpc__cython_cygrpc); if (unlikely(!__pyx_t_3)) __PYX_ERR(9, 59, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_3);
  if (PyDict_SetItem(__pyx_d, __pyx_n_s_channelz_get_subchannel, __pyx_t_3) < 0) __PYX_ERR(9, 59, __pyx_L1_error)
  __Pyx_DECREF(__pyx_t_3); __pyx_t_3 = 0;

  /* "src/python/grpcio/grpc/_cython/_cygrpc/channelz.pyx.pxi":66
 *     return c_returned_str
 * 
 * def channelz_get_socket(socket_id):             # <<<<<<<<<<<<<<
 *     cdef char *c_returned_str = grpc_channelz_get_socket(socket_id)
 *     if c_returned_str == NULL:
 */
  __pyx_t_3 = PyCFunction_NewEx(&__pyx_mdef_4grpc_7_cython_6cygrpc_13channelz_get_socket, NULL, __pyx_n_s_grpc__cython_cygrpc); if (unlikely(!__pyx_t_3)) __PYX_ERR(9, 66, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_3);
  if (PyDict_SetItem(__pyx_d, __pyx_n_s_channelz_get_socket, __pyx_t_3) < 0) __PYX_ERR(9, 66, __pyx_L1_error)
  __Pyx_DECREF(__pyx_t_3); __pyx_t_3 = 0;

  /* "src/python/grpcio/grpc/_cython/_cygrpc/credentials.pyx.pxi":17
 * cimport cpython
 * 
 * import grpc             # <<<<<<<<<<<<<<
 * import threading
 * 
 */
  __pyx_t_3 = __Pyx_Import(__pyx_n_s_grpc, 0, -1); if (unlikely(!__pyx_t_3)) __PYX_ERR(4, 17, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_3);
  if (PyDict_SetItem(__pyx_d, __pyx_n_s_grpc, __pyx_t_3) < 0) __PYX_ERR(4, 17, __pyx_L1_error)
  __Pyx_DECREF(__pyx_t_3); __pyx_t_3 = 0;

  /* "src/python/grpcio/grpc/_cython/_cygrpc/credentials.pyx.pxi":18
 * 
 * import grpc
 * import threading             # <<<<<<<<<<<<<<
 * 
 * 
 */
  __pyx_t_3 = __Pyx_Import(__pyx_n_s_threading, 0, -1); if (unlikely(!__pyx_t_3)) __PYX_ERR(4, 18, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_3);
  if (PyDict_SetItem(__pyx_d, __pyx_n_s_threading, __pyx_t_3) < 0) __PYX_ERR(4, 18, __pyx_L1_error)
  __Pyx_DECREF(__pyx_t_3); __pyx_t_3 = 0;

  /* "src/python/grpcio/grpc/_cython/_cygrpc/credentials.pyx.pxi":21
 * 
 * 
 * def _spawn_callback_in_thread(cb_func, args):             # <<<<<<<<<<<<<<
 *   ForkManagedThread(target=cb_func, args=args).start()
 * 
 */
  __pyx_t_3 = PyCFunction_NewEx(&__pyx_mdef_4grpc_7_cython_6cygrpc_15_spawn_callback_in_thread, NULL, __pyx_n_s_grpc__cython_cygrpc); if (unlikely(!__pyx_t_3)) __PYX_ERR(4, 21, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_3);
  if (PyDict_SetItem(__pyx_d, __pyx_n_s_spawn_callback_in_thread, __pyx_t_3) < 0) __PYX_ERR(4, 21, __pyx_L1_error)
  __Pyx_DECREF(__pyx_t_3); __pyx_t_3 = 0;

  /* "src/python/grpcio/grpc/_cython/_cygrpc/credentials.pyx.pxi":24
 *   ForkManagedThread(target=cb_func, args=args).start()
 * 
 * async_callback_func = _spawn_callback_in_thread             # <<<<<<<<<<<<<<
 * 
 * def set_async_callback_func(callback_func):
 */
  __pyx_t_3 = __Pyx_GetModuleGlobalName(__pyx_n_s_spawn_callback_in_thread); if (unlikely(!__pyx_t_3)) __PYX_ERR(4, 24, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_3);
  if (PyDict_SetItem(__pyx_d, __pyx_n_s_async_callback_func, __pyx_t_3) < 0) __PYX_ERR(4, 24, __pyx_L1_error)
  __Pyx_DECREF(__pyx_t_3); __pyx_t_3 = 0;

  /* "src/python/grpcio/grpc/_cython/_cygrpc/credentials.pyx.pxi":26
 * async_callback_func = _spawn_callback_in_thread
 * 
 * def set_async_callback_func(callback_func):             # <<<<<<<<<<<<<<
 *   global async_callback_func
 *   async_callback_func = callback_func
 */
  __pyx_t_3 = PyCFunction_NewEx(&__pyx_mdef_4grpc_7_cython_6cygrpc_17set_async_callback_func, NULL, __pyx_n_s_grpc__cython_cygrpc); if (unlikely(!__pyx_t_3)) __PYX_ERR(4, 26, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_3);
  if (PyDict_SetItem(__pyx_d, __pyx_n_s_set_async_callback_func, __pyx_t_3) < 0) __PYX_ERR(4, 26, __pyx_L1_error)
  __Pyx_DECREF(__pyx_t_3); __pyx_t_3 = 0;

  /* "src/python/grpcio/grpc/_cython/_cygrpc/credentials.pyx.pxi":30
 *   async_callback_func = callback_func
 * 
 * def _spawn_callback_async(callback, args):             # <<<<<<<<<<<<<<
 *   async_callback_func(callback, args)
 * 
 */
  __pyx_t_3 = PyCFunction_NewEx(&__pyx_mdef_4grpc_7_cython_6cygrpc_19_spawn_callback_async, NULL, __pyx_n_s_grpc__cython_cygrpc); if (unlikely(!__pyx_t_3)) __PYX_ERR(4, 30, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_3);
  if (PyDict_SetItem(__pyx_d, __pyx_n_s_spawn_callback_async, __pyx_t_3) < 0) __PYX_ERR(4, 30, __pyx_L1_error)
  __Pyx_DECREF(__pyx_t_3); __pyx_t_3 = 0;

  /* "src/python/grpcio/grpc/_cython/_cygrpc/credentials.pyx.pxi":233
 *   return c_ssl_pem_key_cert_pairs
 * 
 * def server_credentials_ssl(pem_root_certs, pem_key_cert_pairs,             # <<<<<<<<<<<<<<
 *                            bint force_client_auth):
 *   pem_root_certs = str_to_bytes(pem_root_certs)
 */
  __pyx_t_3 = PyCFunction_NewEx(&__pyx_mdef_4grpc_7_cython_6cygrpc_21server_credentials_ssl, NULL, __pyx_n_s_grpc__cython_cygrpc); if (unlikely(!__pyx_t_3)) __PYX_ERR(4, 233, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_3);
  if (PyDict_SetItem(__pyx_d, __pyx_n_s_server_credentials_ssl, __pyx_t_3) < 0) __PYX_ERR(4, 233, __pyx_L1_error)
  __Pyx_DECREF(__pyx_t_3); __pyx_t_3 = 0;

  /* "src/python/grpcio/grpc/_cython/_cygrpc/credentials.pyx.pxi":258
 *   return credentials
 * 
 * def server_certificate_config_ssl(pem_root_certs, pem_key_cert_pairs):             # <<<<<<<<<<<<<<
 *   pem_root_certs = str_to_bytes(pem_root_certs)
 *   pem_key_cert_pairs = list(pem_key_cert_pairs)
 */
  __pyx_t_3 = PyCFunction_NewEx(&__pyx_mdef_4grpc_7_cython_6cygrpc_23server_certificate_config_ssl, NULL, __pyx_n_s_grpc__cython_cygrpc); if (unlikely(!__pyx_t_3)) __PYX_ERR(4, 258, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_3);
  if (PyDict_SetItem(__pyx_d, __pyx_n_s_server_certificate_config_ssl, __pyx_t_3) < 0) __PYX_ERR(4, 258, __pyx_L1_error)
  __Pyx_DECREF(__pyx_t_3); __pyx_t_3 = 0;

  /* "src/python/grpcio/grpc/_cython/_cygrpc/credentials.pyx.pxi":272
 *   return cert_config
 * 
 * def server_credentials_ssl_dynamic_cert_config(initial_cert_config,             # <<<<<<<<<<<<<<
 *                                                cert_config_fetcher,
 *                                                bint force_client_auth):
 */
  __pyx_t_3 = PyCFunction_NewEx(&__pyx_mdef_4grpc_7_cython_6cygrpc_25server_credentials_ssl_dynamic_cert_config, NULL, __pyx_n_s_grpc__cython_cygrpc); if (unlikely(!__pyx_t_3)) __PYX_ERR(4, 272, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_3);
  if (PyDict_SetItem(__pyx_d, __pyx_n_s_server_credentials_ssl_dynamic_c, __pyx_t_3) < 0) __PYX_ERR(4, 272, __pyx_L1_error)
  __Pyx_DECREF(__pyx_t_3); __pyx_t_3 = 0;

  /* "src/python/grpcio/grpc/_cython/_cygrpc/completion_queue.pyx.pxi":17
 * cimport cpython
 * 
 * import threading             # <<<<<<<<<<<<<<
 * import time
 * 
 */
  __pyx_t_3 = __Pyx_Import(__pyx_n_s_threading, 0, -1); if (unlikely(!__pyx_t_3)) __PYX_ERR(10, 17, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_3);
  if (PyDict_SetItem(__pyx_d, __pyx_n_s_threading, __pyx_t_3) < 0) __PYX_ERR(10, 17, __pyx_L1_error)
  __Pyx_DECREF(__pyx_t_3); __pyx_t_3 = 0;

  /* "src/python/grpcio/grpc/_cython/_cygrpc/completion_queue.pyx.pxi":18
 * 
 * import threading
 * import time             # <<<<<<<<<<<<<<
 * 
 * cdef int _INTERRUPT_CHECK_PERIOD_MS = 200
 */
  __pyx_t_3 = __Pyx_Import(__pyx_n_s_time, 0, -1); if (unlikely(!__pyx_t_3)) __PYX_ERR(10, 18, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_3);
  if (PyDict_SetItem(__pyx_d, __pyx_n_s_time, __pyx_t_3) < 0) __PYX_ERR(10, 18, __pyx_L1_error)
  __Pyx_DECREF(__pyx_t_3); __pyx_t_3 = 0;

  /* "src/python/grpcio/grpc/_cython/_cygrpc/completion_queue.pyx.pxi":20
 * import time
 * 
 * cdef int _INTERRUPT_CHECK_PERIOD_MS = 200             # <<<<<<<<<<<<<<
 * 
 * 
 */
  __pyx_v_4grpc_7_cython_6cygrpc__INTERRUPT_CHECK_PERIOD_MS = 0xC8;

  /* "src/python/grpcio/grpc/_cython/_cygrpc/metadata.pyx.pxi":15
 * # limitations under the License.
 * 
 * import collections             # <<<<<<<<<<<<<<
 * 
 * 
 */
  __pyx_t_3 = __Pyx_Import(__pyx_n_s_collections, 0, -1); if (unlikely(!__pyx_t_3)) __PYX_ERR(13, 15, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_3);
  if (PyDict_SetItem(__pyx_d, __pyx_n_s_collections, __pyx_t_3) < 0) __PYX_ERR(13, 15, __pyx_L1_error)
  __Pyx_DECREF(__pyx_t_3); __pyx_t_3 = 0;

  /* "src/python/grpcio/grpc/_cython/_cygrpc/metadata.pyx.pxi":18
 * 
 * 
 * class InitialMetadataFlags:             # <<<<<<<<<<<<<<
 *   used_mask = GRPC_INITIAL_METADATA_USED_MASK
 *   wait_for_ready = GRPC_INITIAL_METADATA_WAIT_FOR_READY
 */
  __pyx_t_3 = __Pyx_Py3MetaclassPrepare((PyObject *) NULL, __pyx_empty_tuple, __pyx_n_s_InitialMetadataFlags, __pyx_n_s_InitialMetadataFlags, (PyObject *) NULL, __pyx_n_s_grpc__cython_cygrpc, (PyObject *) NULL); if (unlikely(!__pyx_t_3)) __PYX_ERR(13, 18, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_3);

  /* "src/python/grpcio/grpc/_cython/_cygrpc/metadata.pyx.pxi":19
 * 
 * class InitialMetadataFlags:
 *   used_mask = GRPC_INITIAL_METADATA_USED_MASK             # <<<<<<<<<<<<<<
 *   wait_for_ready = GRPC_INITIAL_METADATA_WAIT_FOR_READY
 *   wait_for_ready_explicitly_set = GRPC_INITIAL_METADATA_WAIT_FOR_READY_EXPLICITLY_SET
 */
  __pyx_t_1 = __Pyx_PyInt_From_int(GRPC_INITIAL_METADATA_USED_MASK); if (unlikely(!__pyx_t_1)) __PYX_ERR(13, 19, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_1);
  if (__Pyx_SetNameInClass(__pyx_t_3, __pyx_n_s_used_mask, __pyx_t_1) < 0) __PYX_ERR(13, 19, __pyx_L1_error)
  __Pyx_DECREF(__pyx_t_1); __pyx_t_1 = 0;

  /* "src/python/grpcio/grpc/_cython/_cygrpc/metadata.pyx.pxi":20
 * class InitialMetadataFlags:
 *   used_mask = GRPC_INITIAL_METADATA_USED_MASK
 *   wait_for_ready = GRPC_INITIAL_METADATA_WAIT_FOR_READY             # <<<<<<<<<<<<<<
 *   wait_for_ready_explicitly_set = GRPC_INITIAL_METADATA_WAIT_FOR_READY_EXPLICITLY_SET
 * 
 */
  __pyx_t_1 = __Pyx_PyInt_From_int(GRPC_INITIAL_METADATA_WAIT_FOR_READY); if (unlikely(!__pyx_t_1)) __PYX_ERR(13, 20, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_1);
  if (__Pyx_SetNameInClass(__pyx_t_3, __pyx_n_s_wait_for_ready, __pyx_t_1) < 0) __PYX_ERR(13, 20, __pyx_L1_error)
  __Pyx_DECREF(__pyx_t_1); __pyx_t_1 = 0;

  /* "src/python/grpcio/grpc/_cython/_cygrpc/metadata.pyx.pxi":21
 *   used_mask = GRPC_INITIAL_METADATA_USED_MASK
 *   wait_for_ready = GRPC_INITIAL_METADATA_WAIT_FOR_READY
 *   wait_for_ready_explicitly_set = GRPC_INITIAL_METADATA_WAIT_FOR_READY_EXPLICITLY_SET             # <<<<<<<<<<<<<<
 * 
 * 
 */
  __pyx_t_1 = __Pyx_PyInt_From_int(GRPC_INITIAL_METADATA_WAIT_FOR_READY_EXPLICITLY_SET); if (unlikely(!__pyx_t_1)) __PYX_ERR(13, 21, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_1);
  if (__Pyx_SetNameInClass(__pyx_t_3, __pyx_n_s_wait_for_ready_explicitly_set, __pyx_t_1) < 0) __PYX_ERR(13, 21, __pyx_L1_error)
  __Pyx_DECREF(__pyx_t_1); __pyx_t_1 = 0;

  /* "src/python/grpcio/grpc/_cython/_cygrpc/metadata.pyx.pxi":18
 * 
 * 
 * class InitialMetadataFlags:             # <<<<<<<<<<<<<<
 *   used_mask = GRPC_INITIAL_METADATA_USED_MASK
 *   wait_for_ready = GRPC_INITIAL_METADATA_WAIT_FOR_READY
 */
  __pyx_t_1 = __Pyx_Py3ClassCreate(((PyObject*)&__Pyx_DefaultClassType), __pyx_n_s_InitialMetadataFlags, __pyx_empty_tuple, __pyx_t_3, NULL, 0, 1); if (unlikely(!__pyx_t_1)) __PYX_ERR(13, 18, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_1);
  if (PyDict_SetItem(__pyx_d, __pyx_n_s_InitialMetadataFlags, __pyx_t_1) < 0) __PYX_ERR(13, 18, __pyx_L1_error)
  __Pyx_DECREF(__pyx_t_1); __pyx_t_1 = 0;
  __Pyx_DECREF(__pyx_t_3); __pyx_t_3 = 0;

  /* "src/python/grpcio/grpc/_cython/_cygrpc/metadata.pyx.pxi":24
 * 
 * 
 * _Metadatum = collections.namedtuple('_Metadatum', ('key', 'value',))             # <<<<<<<<<<<<<<
 * 
 * 
 */
  __pyx_t_3 = __Pyx_GetModuleGlobalName(__pyx_n_s_collections); if (unlikely(!__pyx_t_3)) __PYX_ERR(13, 24, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_3);
  __pyx_t_1 = __Pyx_PyObject_GetAttrStr(__pyx_t_3, __pyx_n_s_namedtuple); if (unlikely(!__pyx_t_1)) __PYX_ERR(13, 24, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_1);
  __Pyx_DECREF(__pyx_t_3); __pyx_t_3 = 0;
  __pyx_t_3 = __Pyx_PyObject_Call(__pyx_t_1, __pyx_tuple__190, NULL); if (unlikely(!__pyx_t_3)) __PYX_ERR(13, 24, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_3);
  __Pyx_DECREF(__pyx_t_1); __pyx_t_1 = 0;
  if (PyDict_SetItem(__pyx_d, __pyx_n_s_Metadatum, __pyx_t_3) < 0) __PYX_ERR(13, 24, __pyx_L1_error)
  __Pyx_DECREF(__pyx_t_3); __pyx_t_3 = 0;

  /* "src/python/grpcio/grpc/_cython/_cygrpc/propagation_bits.pyx.pxi":15
 * # limitations under the License.
 * 
 * class PropagationConstants:             # <<<<<<<<<<<<<<
 *     GRPC_PROPAGATE_DEADLINE = _GRPC_PROPAGATE_DEADLINE
 *     GRPC_PROPAGATE_CENSUS_STATS_CONTEXT = _GRPC_PROPAGATE_CENSUS_STATS_CONTEXT
 */
  __pyx_t_3 = __Pyx_Py3MetaclassPrepare((PyObject *) NULL, __pyx_empty_tuple, __pyx_n_s_PropagationConstants, __pyx_n_s_PropagationConstants, (PyObject *) NULL, __pyx_n_s_grpc__cython_cygrpc, (PyObject *) NULL); if (unlikely(!__pyx_t_3)) __PYX_ERR(26, 15, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_3);

  /* "src/python/grpcio/grpc/_cython/_cygrpc/propagation_bits.pyx.pxi":16
 * 
 * class PropagationConstants:
 *     GRPC_PROPAGATE_DEADLINE = _GRPC_PROPAGATE_DEADLINE             # <<<<<<<<<<<<<<
 *     GRPC_PROPAGATE_CENSUS_STATS_CONTEXT = _GRPC_PROPAGATE_CENSUS_STATS_CONTEXT
 *     GRPC_PROPAGATE_CENSUS_TRACING_CONTEXT = _GRPC_PROPAGATE_CENSUS_TRACING_CONTEXT
 */
  __pyx_t_1 = __Pyx_PyInt_From_int(GRPC_PROPAGATE_DEADLINE); if (unlikely(!__pyx_t_1)) __PYX_ERR(26, 16, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_1);
  if (__Pyx_SetNameInClass(__pyx_t_3, __pyx_n_s_GRPC_PROPAGATE_DEADLINE, __pyx_t_1) < 0) __PYX_ERR(26, 16, __pyx_L1_error)
  __Pyx_DECREF(__pyx_t_1); __pyx_t_1 = 0;

  /* "src/python/grpcio/grpc/_cython/_cygrpc/propagation_bits.pyx.pxi":17
 * class PropagationConstants:
 *     GRPC_PROPAGATE_DEADLINE = _GRPC_PROPAGATE_DEADLINE
 *     GRPC_PROPAGATE_CENSUS_STATS_CONTEXT = _GRPC_PROPAGATE_CENSUS_STATS_CONTEXT             # <<<<<<<<<<<<<<
 *     GRPC_PROPAGATE_CENSUS_TRACING_CONTEXT = _GRPC_PROPAGATE_CENSUS_TRACING_CONTEXT
 *     GRPC_PROPAGATE_CANCELLATION = _GRPC_PROPAGATE_CANCELLATION
 */
  __pyx_t_1 = __Pyx_PyInt_From_int(GRPC_PROPAGATE_CENSUS_STATS_CONTEXT); if (unlikely(!__pyx_t_1)) __PYX_ERR(26, 17, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_1);
  if (__Pyx_SetNameInClass(__pyx_t_3, __pyx_n_s_GRPC_PROPAGATE_CENSUS_STATS_CONT, __pyx_t_1) < 0) __PYX_ERR(26, 17, __pyx_L1_error)
  __Pyx_DECREF(__pyx_t_1); __pyx_t_1 = 0;

  /* "src/python/grpcio/grpc/_cython/_cygrpc/propagation_bits.pyx.pxi":18
 *     GRPC_PROPAGATE_DEADLINE = _GRPC_PROPAGATE_DEADLINE
 *     GRPC_PROPAGATE_CENSUS_STATS_CONTEXT = _GRPC_PROPAGATE_CENSUS_STATS_CONTEXT
 *     GRPC_PROPAGATE_CENSUS_TRACING_CONTEXT = _GRPC_PROPAGATE_CENSUS_TRACING_CONTEXT             # <<<<<<<<<<<<<<
 *     GRPC_PROPAGATE_CANCELLATION = _GRPC_PROPAGATE_CANCELLATION
 *     GRPC_PROPAGATE_DEFAULTS = _GRPC_PROPAGATE_DEFAULTS
 */
  __pyx_t_1 = __Pyx_PyInt_From_int(GRPC_PROPAGATE_CENSUS_TRACING_CONTEXT); if (unlikely(!__pyx_t_1)) __PYX_ERR(26, 18, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_1);
  if (__Pyx_SetNameInClass(__pyx_t_3, __pyx_n_s_GRPC_PROPAGATE_CENSUS_TRACING_CO, __pyx_t_1) < 0) __PYX_ERR(26, 18, __pyx_L1_error)
  __Pyx_DECREF(__pyx_t_1); __pyx_t_1 = 0;

  /* "src/python/grpcio/grpc/_cython/_cygrpc/propagation_bits.pyx.pxi":19
 *     GRPC_PROPAGATE_CENSUS_STATS_CONTEXT = _GRPC_PROPAGATE_CENSUS_STATS_CONTEXT
 *     GRPC_PROPAGATE_CENSUS_TRACING_CONTEXT = _GRPC_PROPAGATE_CENSUS_TRACING_CONTEXT
 *     GRPC_PROPAGATE_CANCELLATION = _GRPC_PROPAGATE_CANCELLATION             # <<<<<<<<<<<<<<
 *     GRPC_PROPAGATE_DEFAULTS = _GRPC_PROPAGATE_DEFAULTS
 */
  __pyx_t_1 = __Pyx_PyInt_From_int(GRPC_PROPAGATE_CANCELLATION); if (unlikely(!__pyx_t_1)) __PYX_ERR(26, 19, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_1);
  if (__Pyx_SetNameInClass(__pyx_t_3, __pyx_n_s_GRPC_PROPAGATE_CANCELLATION, __pyx_t_1) < 0) __PYX_ERR(26, 19, __pyx_L1_error)
  __Pyx_DECREF(__pyx_t_1); __pyx_t_1 = 0;

  /* "src/python/grpcio/grpc/_cython/_cygrpc/propagation_bits.pyx.pxi":20
 *     GRPC_PROPAGATE_CENSUS_TRACING_CONTEXT = _GRPC_PROPAGATE_CENSUS_TRACING_CONTEXT
 *     GRPC_PROPAGATE_CANCELLATION = _GRPC_PROPAGATE_CANCELLATION
 *     GRPC_PROPAGATE_DEFAULTS = _GRPC_PROPAGATE_DEFAULTS             # <<<<<<<<<<<<<<
 */
  __pyx_t_1 = __Pyx_PyInt_From_int(GRPC_PROPAGATE_DEFAULTS); if (unlikely(!__pyx_t_1)) __PYX_ERR(26, 20, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_1);
  if (__Pyx_SetNameInClass(__pyx_t_3, __pyx_n_s_GRPC_PROPAGATE_DEFAULTS, __pyx_t_1) < 0) __PYX_ERR(26, 20, __pyx_L1_error)
  __Pyx_DECREF(__pyx_t_1); __pyx_t_1 = 0;

  /* "src/python/grpcio/grpc/_cython/_cygrpc/propagation_bits.pyx.pxi":15
 * # limitations under the License.
 * 
 * class PropagationConstants:             # <<<<<<<<<<<<<<
 *     GRPC_PROPAGATE_DEADLINE = _GRPC_PROPAGATE_DEADLINE
 *     GRPC_PROPAGATE_CENSUS_STATS_CONTEXT = _GRPC_PROPAGATE_CENSUS_STATS_CONTEXT
 */
  __pyx_t_1 = __Pyx_Py3ClassCreate(((PyObject*)&__Pyx_DefaultClassType), __pyx_n_s_PropagationConstants, __pyx_empty_tuple, __pyx_t_3, NULL, 0, 1); if (unlikely(!__pyx_t_1)) __PYX_ERR(26, 15, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_1);
  if (PyDict_SetItem(__pyx_d, __pyx_n_s_PropagationConstants, __pyx_t_1) < 0) __PYX_ERR(26, 15, __pyx_L1_error)
  __Pyx_DECREF(__pyx_t_1); __pyx_t_1 = 0;
  __Pyx_DECREF(__pyx_t_3); __pyx_t_3 = 0;

  /* "src/python/grpcio/grpc/_cython/_cygrpc/records.pyx.pxi":35
 * 
 * 
 * class ConnectivityState:             # <<<<<<<<<<<<<<
 *   idle = GRPC_CHANNEL_IDLE
 *   connecting = GRPC_CHANNEL_CONNECTING
 */
  __pyx_t_3 = __Pyx_Py3MetaclassPrepare((PyObject *) NULL, __pyx_empty_tuple, __pyx_n_s_ConnectivityState, __pyx_n_s_ConnectivityState, (PyObject *) NULL, __pyx_n_s_grpc__cython_cygrpc, (PyObject *) NULL); if (unlikely(!__pyx_t_3)) __PYX_ERR(16, 35, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_3);

  /* "src/python/grpcio/grpc/_cython/_cygrpc/records.pyx.pxi":36
 * 
 * class ConnectivityState:
 *   idle = GRPC_CHANNEL_IDLE             # <<<<<<<<<<<<<<
 *   connecting = GRPC_CHANNEL_CONNECTING
 *   ready = GRPC_CHANNEL_READY
 */
  __pyx_t_1 = __Pyx_PyInt_From_grpc_connectivity_state(GRPC_CHANNEL_IDLE); if (unlikely(!__pyx_t_1)) __PYX_ERR(16, 36, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_1);
  if (__Pyx_SetNameInClass(__pyx_t_3, __pyx_n_s_idle, __pyx_t_1) < 0) __PYX_ERR(16, 36, __pyx_L1_error)
  __Pyx_DECREF(__pyx_t_1); __pyx_t_1 = 0;

  /* "src/python/grpcio/grpc/_cython/_cygrpc/records.pyx.pxi":37
 * class ConnectivityState:
 *   idle = GRPC_CHANNEL_IDLE
 *   connecting = GRPC_CHANNEL_CONNECTING             # <<<<<<<<<<<<<<
 *   ready = GRPC_CHANNEL_READY
 *   transient_failure = GRPC_CHANNEL_TRANSIENT_FAILURE
 */
  __pyx_t_1 = __Pyx_PyInt_From_grpc_connectivity_state(GRPC_CHANNEL_CONNECTING); if (unlikely(!__pyx_t_1)) __PYX_ERR(16, 37, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_1);
  if (__Pyx_SetNameInClass(__pyx_t_3, __pyx_n_s_connecting, __pyx_t_1) < 0) __PYX_ERR(16, 37, __pyx_L1_error)
  __Pyx_DECREF(__pyx_t_1); __pyx_t_1 = 0;

  /* "src/python/grpcio/grpc/_cython/_cygrpc/records.pyx.pxi":38
 *   idle = GRPC_CHANNEL_IDLE
 *   connecting = GRPC_CHANNEL_CONNECTING
 *   ready = GRPC_CHANNEL_READY             # <<<<<<<<<<<<<<
 *   transient_failure = GRPC_CHANNEL_TRANSIENT_FAILURE
 *   shutdown = GRPC_CHANNEL_SHUTDOWN
 */
  __pyx_t_1 = __Pyx_PyInt_From_grpc_connectivity_state(GRPC_CHANNEL_READY); if (unlikely(!__pyx_t_1)) __PYX_ERR(16, 38, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_1);
  if (__Pyx_SetNameInClass(__pyx_t_3, __pyx_n_s_ready, __pyx_t_1) < 0) __PYX_ERR(16, 38, __pyx_L1_error)
  __Pyx_DECREF(__pyx_t_1); __pyx_t_1 = 0;

  /* "src/python/grpcio/grpc/_cython/_cygrpc/records.pyx.pxi":39
 *   connecting = GRPC_CHANNEL_CONNECTING
 *   ready = GRPC_CHANNEL_READY
 *   transient_failure = GRPC_CHANNEL_TRANSIENT_FAILURE             # <<<<<<<<<<<<<<
 *   shutdown = GRPC_CHANNEL_SHUTDOWN
 * 
 */
  __pyx_t_1 = __Pyx_PyInt_From_grpc_connectivity_state(GRPC_CHANNEL_TRANSIENT_FAILURE); if (unlikely(!__pyx_t_1)) __PYX_ERR(16, 39, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_1);
  if (__Pyx_SetNameInClass(__pyx_t_3, __pyx_n_s_transient_failure, __pyx_t_1) < 0) __PYX_ERR(16, 39, __pyx_L1_error)
  __Pyx_DECREF(__pyx_t_1); __pyx_t_1 = 0;

  /* "src/python/grpcio/grpc/_cython/_cygrpc/records.pyx.pxi":40
 *   ready = GRPC_CHANNEL_READY
 *   transient_failure = GRPC_CHANNEL_TRANSIENT_FAILURE
 *   shutdown = GRPC_CHANNEL_SHUTDOWN             # <<<<<<<<<<<<<<
 * 
 * 
 */
  __pyx_t_1 = __Pyx_PyInt_From_grpc_connectivity_state(GRPC_CHANNEL_SHUTDOWN); if (unlikely(!__pyx_t_1)) __PYX_ERR(16, 40, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_1);
  if (__Pyx_SetNameInClass(__pyx_t_3, __pyx_n_s_shutdown, __pyx_t_1) < 0) __PYX_ERR(16, 40, __pyx_L1_error)
  __Pyx_DECREF(__pyx_t_1); __pyx_t_1 = 0;

  /* "src/python/grpcio/grpc/_cython/_cygrpc/records.pyx.pxi":35
 * 
 * 
 * class ConnectivityState:             # <<<<<<<<<<<<<<
 *   idle = GRPC_CHANNEL_IDLE
 *   connecting = GRPC_CHANNEL_CONNECTING
 */
  __pyx_t_1 = __Pyx_Py3ClassCreate(((PyObject*)&__Pyx_DefaultClassType), __pyx_n_s_ConnectivityState, __pyx_empty_tuple, __pyx_t_3, NULL, 0, 1); if (unlikely(!__pyx_t_1)) __PYX_ERR(16, 35, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_1);
  if (PyDict_SetItem(__pyx_d, __pyx_n_s_ConnectivityState, __pyx_t_1) < 0) __PYX_ERR(16, 35, __pyx_L1_error)
  __Pyx_DECREF(__pyx_t_1); __pyx_t_1 = 0;
  __Pyx_DECREF(__pyx_t_3); __pyx_t_3 = 0;

  /* "src/python/grpcio/grpc/_cython/_cygrpc/records.pyx.pxi":43
 * 
 * 
 * class ChannelArgKey:             # <<<<<<<<<<<<<<
 *   enable_census = GRPC_ARG_ENABLE_CENSUS
 *   max_concurrent_streams = GRPC_ARG_MAX_CONCURRENT_STREAMS
 */
  __pyx_t_3 = __Pyx_Py3MetaclassPrepare((PyObject *) NULL, __pyx_empty_tuple, __pyx_n_s_ChannelArgKey, __pyx_n_s_ChannelArgKey, (PyObject *) NULL, __pyx_n_s_grpc__cython_cygrpc, (PyObject *) NULL); if (unlikely(!__pyx_t_3)) __PYX_ERR(16, 43, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_3);

  /* "src/python/grpcio/grpc/_cython/_cygrpc/records.pyx.pxi":44
 * 
 * class ChannelArgKey:
 *   enable_census = GRPC_ARG_ENABLE_CENSUS             # <<<<<<<<<<<<<<
 *   max_concurrent_streams = GRPC_ARG_MAX_CONCURRENT_STREAMS
 *   max_receive_message_length = GRPC_ARG_MAX_RECEIVE_MESSAGE_LENGTH
 */
  __pyx_t_1 = __Pyx_PyBytes_FromString(GRPC_ARG_ENABLE_CENSUS); if (unlikely(!__pyx_t_1)) __PYX_ERR(16, 44, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_1);
  if (__Pyx_SetNameInClass(__pyx_t_3, __pyx_n_s_enable_census, __pyx_t_1) < 0) __PYX_ERR(16, 44, __pyx_L1_error)
  __Pyx_DECREF(__pyx_t_1); __pyx_t_1 = 0;

  /* "src/python/grpcio/grpc/_cython/_cygrpc/records.pyx.pxi":45
 * class ChannelArgKey:
 *   enable_census = GRPC_ARG_ENABLE_CENSUS
 *   max_concurrent_streams = GRPC_ARG_MAX_CONCURRENT_STREAMS             # <<<<<<<<<<<<<<
 *   max_receive_message_length = GRPC_ARG_MAX_RECEIVE_MESSAGE_LENGTH
 *   max_send_message_length = GRPC_ARG_MAX_SEND_MESSAGE_LENGTH
 */
  __pyx_t_1 = __Pyx_PyBytes_FromString(GRPC_ARG_MAX_CONCURRENT_STREAMS); if (unlikely(!__pyx_t_1)) __PYX_ERR(16, 45, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_1);
  if (__Pyx_SetNameInClass(__pyx_t_3, __pyx_n_s_max_concurrent_streams, __pyx_t_1) < 0) __PYX_ERR(16, 45, __pyx_L1_error)
  __Pyx_DECREF(__pyx_t_1); __pyx_t_1 = 0;

  /* "src/python/grpcio/grpc/_cython/_cygrpc/records.pyx.pxi":46
 *   enable_census = GRPC_ARG_ENABLE_CENSUS
 *   max_concurrent_streams = GRPC_ARG_MAX_CONCURRENT_STREAMS
 *   max_receive_message_length = GRPC_ARG_MAX_RECEIVE_MESSAGE_LENGTH             # <<<<<<<<<<<<<<
 *   max_send_message_length = GRPC_ARG_MAX_SEND_MESSAGE_LENGTH
 *   http2_initial_sequence_number = GRPC_ARG_HTTP2_INITIAL_SEQUENCE_NUMBER
 */
  __pyx_t_1 = __Pyx_PyBytes_FromString(GRPC_ARG_MAX_RECEIVE_MESSAGE_LENGTH); if (unlikely(!__pyx_t_1)) __PYX_ERR(16, 46, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_1);
  if (__Pyx_SetNameInClass(__pyx_t_3, __pyx_n_s_max_receive_message_length, __pyx_t_1) < 0) __PYX_ERR(16, 46, __pyx_L1_error)
  __Pyx_DECREF(__pyx_t_1); __pyx_t_1 = 0;

  /* "src/python/grpcio/grpc/_cython/_cygrpc/records.pyx.pxi":47
 *   max_concurrent_streams = GRPC_ARG_MAX_CONCURRENT_STREAMS
 *   max_receive_message_length = GRPC_ARG_MAX_RECEIVE_MESSAGE_LENGTH
 *   max_send_message_length = GRPC_ARG_MAX_SEND_MESSAGE_LENGTH             # <<<<<<<<<<<<<<
 *   http2_initial_sequence_number = GRPC_ARG_HTTP2_INITIAL_SEQUENCE_NUMBER
 *   default_authority = GRPC_ARG_DEFAULT_AUTHORITY
 */
  __pyx_t_1 = __Pyx_PyBytes_FromString(GRPC_ARG_MAX_SEND_MESSAGE_LENGTH); if (unlikely(!__pyx_t_1)) __PYX_ERR(16, 47, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_1);
  if (__Pyx_SetNameInClass(__pyx_t_3, __pyx_n_s_max_send_message_length, __pyx_t_1) < 0) __PYX_ERR(16, 47, __pyx_L1_error)
  __Pyx_DECREF(__pyx_t_1); __pyx_t_1 = 0;

  /* "src/python/grpcio/grpc/_cython/_cygrpc/records.pyx.pxi":48
 *   max_receive_message_length = GRPC_ARG_MAX_RECEIVE_MESSAGE_LENGTH
 *   max_send_message_length = GRPC_ARG_MAX_SEND_MESSAGE_LENGTH
 *   http2_initial_sequence_number = GRPC_ARG_HTTP2_INITIAL_SEQUENCE_NUMBER             # <<<<<<<<<<<<<<
 *   default_authority = GRPC_ARG_DEFAULT_AUTHORITY
 *   primary_user_agent_string = GRPC_ARG_PRIMARY_USER_AGENT_STRING
 */
  __pyx_t_1 = __Pyx_PyBytes_FromString(GRPC_ARG_HTTP2_INITIAL_SEQUENCE_NUMBER); if (unlikely(!__pyx_t_1)) __PYX_ERR(16, 48, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_1);
  if (__Pyx_SetNameInClass(__pyx_t_3, __pyx_n_s_http2_initial_sequence_number, __pyx_t_1) < 0) __PYX_ERR(16, 48, __pyx_L1_error)
  __Pyx_DECREF(__pyx_t_1); __pyx_t_1 = 0;

  /* "src/python/grpcio/grpc/_cython/_cygrpc/records.pyx.pxi":49
 *   max_send_message_length = GRPC_ARG_MAX_SEND_MESSAGE_LENGTH
 *   http2_initial_sequence_number = GRPC_ARG_HTTP2_INITIAL_SEQUENCE_NUMBER
 *   default_authority = GRPC_ARG_DEFAULT_AUTHORITY             # <<<<<<<<<<<<<<
 *   primary_user_agent_string = GRPC_ARG_PRIMARY_USER_AGENT_STRING
 *   secondary_user_agent_string = GRPC_ARG_SECONDARY_USER_AGENT_STRING
 */
  __pyx_t_1 = __Pyx_PyBytes_FromString(GRPC_ARG_DEFAULT_AUTHORITY); if (unlikely(!__pyx_t_1)) __PYX_ERR(16, 49, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_1);
  if (__Pyx_SetNameInClass(__pyx_t_3, __pyx_n_s_default_authority, __pyx_t_1) < 0) __PYX_ERR(16, 49, __pyx_L1_error)
  __Pyx_DECREF(__pyx_t_1); __pyx_t_1 = 0;

  /* "src/python/grpcio/grpc/_cython/_cygrpc/records.pyx.pxi":50
 *   http2_initial_sequence_number = GRPC_ARG_HTTP2_INITIAL_SEQUENCE_NUMBER
 *   default_authority = GRPC_ARG_DEFAULT_AUTHORITY
 *   primary_user_agent_string = GRPC_ARG_PRIMARY_USER_AGENT_STRING             # <<<<<<<<<<<<<<
 *   secondary_user_agent_string = GRPC_ARG_SECONDARY_USER_AGENT_STRING
 *   ssl_session_cache = GRPC_SSL_SESSION_CACHE_ARG
 */
  __pyx_t_1 = __Pyx_PyBytes_FromString(GRPC_ARG_PRIMARY_USER_AGENT_STRING); if (unlikely(!__pyx_t_1)) __PYX_ERR(16, 50, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_1);
  if (__Pyx_SetNameInClass(__pyx_t_3, __pyx_n_s_primary_user_agent_string, __pyx_t_1) < 0) __PYX_ERR(16, 50, __pyx_L1_error)
  __Pyx_DECREF(__pyx_t_1); __pyx_t_1 = 0;

  /* "src/python/grpcio/grpc/_cython/_cygrpc/records.pyx.pxi":51
 *   default_authority = GRPC_ARG_DEFAULT_AUTHORITY
 *   primary_user_agent_string = GRPC_ARG_PRIMARY_USER_AGENT_STRING
 *   secondary_user_agent_string = GRPC_ARG_SECONDARY_USER_AGENT_STRING             # <<<<<<<<<<<<<<
 *   ssl_session_cache = GRPC_SSL_SESSION_CACHE_ARG
 *   ssl_target_name_override = GRPC_SSL_TARGET_NAME_OVERRIDE_ARG
 */
  __pyx_t_1 = __Pyx_PyBytes_FromString(GRPC_ARG_SECONDARY_USER_AGENT_STRING); if (unlikely(!__pyx_t_1)) __PYX_ERR(16, 51, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_1);
  if (__Pyx_SetNameInClass(__pyx_t_3, __pyx_n_s_secondary_user_agent_string, __pyx_t_1) < 0) __PYX_ERR(16, 51, __pyx_L1_error)
  __Pyx_DECREF(__pyx_t_1); __pyx_t_1 = 0;

  /* "src/python/grpcio/grpc/_cython/_cygrpc/records.pyx.pxi":52
 *   primary_user_agent_string = GRPC_ARG_PRIMARY_USER_AGENT_STRING
 *   secondary_user_agent_string = GRPC_ARG_SECONDARY_USER_AGENT_STRING
 *   ssl_session_cache = GRPC_SSL_SESSION_CACHE_ARG             # <<<<<<<<<<<<<<
 *   ssl_target_name_override = GRPC_SSL_TARGET_NAME_OVERRIDE_ARG
 * 
 */
  __pyx_t_1 = __Pyx_PyBytes_FromString(GRPC_SSL_SESSION_CACHE_ARG); if (unlikely(!__pyx_t_1)) __PYX_ERR(16, 52, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_1);
  if (__Pyx_SetNameInClass(__pyx_t_3, __pyx_n_s_ssl_session_cache, __pyx_t_1) < 0) __PYX_ERR(16, 52, __pyx_L1_error)
  __Pyx_DECREF(__pyx_t_1); __pyx_t_1 = 0;

  /* "src/python/grpcio/grpc/_cython/_cygrpc/records.pyx.pxi":53
 *   secondary_user_agent_string = GRPC_ARG_SECONDARY_USER_AGENT_STRING
 *   ssl_session_cache = GRPC_SSL_SESSION_CACHE_ARG
 *   ssl_target_name_override = GRPC_SSL_TARGET_NAME_OVERRIDE_ARG             # <<<<<<<<<<<<<<
 * 
 * 
 */
  __pyx_t_1 = __Pyx_PyBytes_FromString(GRPC_SSL_TARGET_NAME_OVERRIDE_ARG); if (unlikely(!__pyx_t_1)) __PYX_ERR(16, 53, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_1);
  if (__Pyx_SetNameInClass(__pyx_t_3, __pyx_n_s_ssl_target_name_override, __pyx_t_1) < 0) __PYX_ERR(16, 53, __pyx_L1_error)
  __Pyx_DECREF(__pyx_t_1); __pyx_t_1 = 0;

  /* "src/python/grpcio/grpc/_cython/_cygrpc/records.pyx.pxi":43
 * 
 * 
 * class ChannelArgKey:             # <<<<<<<<<<<<<<
 *   enable_census = GRPC_ARG_ENABLE_CENSUS
 *   max_concurrent_streams = GRPC_ARG_MAX_CONCURRENT_STREAMS
 */
  __pyx_t_1 = __Pyx_Py3ClassCreate(((PyObject*)&__Pyx_DefaultClassType), __pyx_n_s_ChannelArgKey, __pyx_empty_tuple, __pyx_t_3, NULL, 0, 1); if (unlikely(!__pyx_t_1)) __PYX_ERR(16, 43, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_1);
  if (PyDict_SetItem(__pyx_d, __pyx_n_s_ChannelArgKey, __pyx_t_1) < 0) __PYX_ERR(16, 43, __pyx_L1_error)
  __Pyx_DECREF(__pyx_t_1); __pyx_t_1 = 0;
  __Pyx_DECREF(__pyx_t_3); __pyx_t_3 = 0;

  /* "src/python/grpcio/grpc/_cython/_cygrpc/records.pyx.pxi":56
 * 
 * 
 * class WriteFlag:             # <<<<<<<<<<<<<<
 *   buffer_hint = GRPC_WRITE_BUFFER_HINT
 *   no_compress = GRPC_WRITE_NO_COMPRESS
 */
  __pyx_t_3 = __Pyx_Py3MetaclassPrepare((PyObject *) NULL, __pyx_empty_tuple, __pyx_n_s_WriteFlag, __pyx_n_s_WriteFlag, (PyObject *) NULL, __pyx_n_s_grpc__cython_cygrpc, (PyObject *) NULL); if (unlikely(!__pyx_t_3)) __PYX_ERR(16, 56, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_3);

  /* "src/python/grpcio/grpc/_cython/_cygrpc/records.pyx.pxi":57
 * 
 * class WriteFlag:
 *   buffer_hint = GRPC_WRITE_BUFFER_HINT             # <<<<<<<<<<<<<<
 *   no_compress = GRPC_WRITE_NO_COMPRESS
 * 
 */
  __pyx_t_1 = __Pyx_PyInt_From_int(GRPC_WRITE_BUFFER_HINT); if (unlikely(!__pyx_t_1)) __PYX_ERR(16, 57, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_1);
  if (__Pyx_SetNameInClass(__pyx_t_3, __pyx_n_s_buffer_hint, __pyx_t_1) < 0) __PYX_ERR(16, 57, __pyx_L1_error)
  __Pyx_DECREF(__pyx_t_1); __pyx_t_1 = 0;

  /* "src/python/grpcio/grpc/_cython/_cygrpc/records.pyx.pxi":58
 * class WriteFlag:
 *   buffer_hint = GRPC_WRITE_BUFFER_HINT
 *   no_compress = GRPC_WRITE_NO_COMPRESS             # <<<<<<<<<<<<<<
 * 
 * 
 */
  __pyx_t_1 = __Pyx_PyInt_From_int(GRPC_WRITE_NO_COMPRESS); if (unlikely(!__pyx_t_1)) __PYX_ERR(16, 58, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_1);
  if (__Pyx_SetNameInClass(__pyx_t_3, __pyx_n_s_no_compress, __pyx_t_1) < 0) __PYX_ERR(16, 58, __pyx_L1_error)
  __Pyx_DECREF(__pyx_t_1); __pyx_t_1 = 0;

  /* "src/python/grpcio/grpc/_cython/_cygrpc/records.pyx.pxi":56
 * 
 * 
 * class WriteFlag:             # <<<<<<<<<<<<<<
 *   buffer_hint = GRPC_WRITE_BUFFER_HINT
 *   no_compress = GRPC_WRITE_NO_COMPRESS
 */
  __pyx_t_1 = __Pyx_Py3ClassCreate(((PyObject*)&__Pyx_DefaultClassType), __pyx_n_s_WriteFlag, __pyx_empty_tuple, __pyx_t_3, NULL, 0, 1); if (unlikely(!__pyx_t_1)) __PYX_ERR(16, 56, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_1);
  if (PyDict_SetItem(__pyx_d, __pyx_n_s_WriteFlag, __pyx_t_1) < 0) __PYX_ERR(16, 56, __pyx_L1_error)
  __Pyx_DECREF(__pyx_t_1); __pyx_t_1 = 0;
  __Pyx_DECREF(__pyx_t_3); __pyx_t_3 = 0;

  /* "src/python/grpcio/grpc/_cython/_cygrpc/records.pyx.pxi":61
 * 
 * 
 * class StatusCode:             # <<<<<<<<<<<<<<
 *   ok = GRPC_STATUS_OK
 *   cancelled = GRPC_STATUS_CANCELLED
 */
  __pyx_t_3 = __Pyx_Py3MetaclassPrepare((PyObject *) NULL, __pyx_empty_tuple, __pyx_n_s_StatusCode, __pyx_n_s_StatusCode, (PyObject *) NULL, __pyx_n_s_grpc__cython_cygrpc, (PyObject *) NULL); if (unlikely(!__pyx_t_3)) __PYX_ERR(16, 61, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_3);

  /* "src/python/grpcio/grpc/_cython/_cygrpc/records.pyx.pxi":62
 * 
 * class StatusCode:
 *   ok = GRPC_STATUS_OK             # <<<<<<<<<<<<<<
 *   cancelled = GRPC_STATUS_CANCELLED
 *   unknown = GRPC_STATUS_UNKNOWN
 */
  __pyx_t_1 = __Pyx_PyInt_From_grpc_status_code(GRPC_STATUS_OK); if (unlikely(!__pyx_t_1)) __PYX_ERR(16, 62, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_1);
  if (__Pyx_SetNameInClass(__pyx_t_3, __pyx_n_s_ok, __pyx_t_1) < 0) __PYX_ERR(16, 62, __pyx_L1_error)
  __Pyx_DECREF(__pyx_t_1); __pyx_t_1 = 0;

  /* "src/python/grpcio/grpc/_cython/_cygrpc/records.pyx.pxi":63
 * class StatusCode:
 *   ok = GRPC_STATUS_OK
 *   cancelled = GRPC_STATUS_CANCELLED             # <<<<<<<<<<<<<<
 *   unknown = GRPC_STATUS_UNKNOWN
 *   invalid_argument = GRPC_STATUS_INVALID_ARGUMENT
 */
  __pyx_t_1 = __Pyx_PyInt_From_grpc_status_code(GRPC_STATUS_CANCELLED); if (unlikely(!__pyx_t_1)) __PYX_ERR(16, 63, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_1);
  if (__Pyx_SetNameInClass(__pyx_t_3, __pyx_n_s_cancelled, __pyx_t_1) < 0) __PYX_ERR(16, 63, __pyx_L1_error)
  __Pyx_DECREF(__pyx_t_1); __pyx_t_1 = 0;

  /* "src/python/grpcio/grpc/_cython/_cygrpc/records.pyx.pxi":64
 *   ok = GRPC_STATUS_OK
 *   cancelled = GRPC_STATUS_CANCELLED
 *   unknown = GRPC_STATUS_UNKNOWN             # <<<<<<<<<<<<<<
 *   invalid_argument = GRPC_STATUS_INVALID_ARGUMENT
 *   deadline_exceeded = GRPC_STATUS_DEADLINE_EXCEEDED
 */
  __pyx_t_1 = __Pyx_PyInt_From_grpc_status_code(GRPC_STATUS_UNKNOWN); if (unlikely(!__pyx_t_1)) __PYX_ERR(16, 64, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_1);
  if (__Pyx_SetNameInClass(__pyx_t_3, __pyx_n_s_unknown, __pyx_t_1) < 0) __PYX_ERR(16, 64, __pyx_L1_error)
  __Pyx_DECREF(__pyx_t_1); __pyx_t_1 = 0;

  /* "src/python/grpcio/grpc/_cython/_cygrpc/records.pyx.pxi":65
 *   cancelled = GRPC_STATUS_CANCELLED
 *   unknown = GRPC_STATUS_UNKNOWN
 *   invalid_argument = GRPC_STATUS_INVALID_ARGUMENT             # <<<<<<<<<<<<<<
 *   deadline_exceeded = GRPC_STATUS_DEADLINE_EXCEEDED
 *   not_found = GRPC_STATUS_NOT_FOUND
 */
  __pyx_t_1 = __Pyx_PyInt_From_grpc_status_code(GRPC_STATUS_INVALID_ARGUMENT); if (unlikely(!__pyx_t_1)) __PYX_ERR(16, 65, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_1);
  if (__Pyx_SetNameInClass(__pyx_t_3, __pyx_n_s_invalid_argument, __pyx_t_1) < 0) __PYX_ERR(16, 65, __pyx_L1_error)
  __Pyx_DECREF(__pyx_t_1); __pyx_t_1 = 0;

  /* "src/python/grpcio/grpc/_cython/_cygrpc/records.pyx.pxi":66
 *   unknown = GRPC_STATUS_UNKNOWN
 *   invalid_argument = GRPC_STATUS_INVALID_ARGUMENT
 *   deadline_exceeded = GRPC_STATUS_DEADLINE_EXCEEDED             # <<<<<<<<<<<<<<
 *   not_found = GRPC_STATUS_NOT_FOUND
 *   already_exists = GRPC_STATUS_ALREADY_EXISTS
 */
  __pyx_t_1 = __Pyx_PyInt_From_grpc_status_code(GRPC_STATUS_DEADLINE_EXCEEDED); if (unlikely(!__pyx_t_1)) __PYX_ERR(16, 66, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_1);
  if (__Pyx_SetNameInClass(__pyx_t_3, __pyx_n_s_deadline_exceeded, __pyx_t_1) < 0) __PYX_ERR(16, 66, __pyx_L1_error)
  __Pyx_DECREF(__pyx_t_1); __pyx_t_1 = 0;

  /* "src/python/grpcio/grpc/_cython/_cygrpc/records.pyx.pxi":67
 *   invalid_argument = GRPC_STATUS_INVALID_ARGUMENT
 *   deadline_exceeded = GRPC_STATUS_DEADLINE_EXCEEDED
 *   not_found = GRPC_STATUS_NOT_FOUND             # <<<<<<<<<<<<<<
 *   already_exists = GRPC_STATUS_ALREADY_EXISTS
 *   permission_denied = GRPC_STATUS_PERMISSION_DENIED
 */
  __pyx_t_1 = __Pyx_PyInt_From_grpc_status_code(GRPC_STATUS_NOT_FOUND); if (unlikely(!__pyx_t_1)) __PYX_ERR(16, 67, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_1);
  if (__Pyx_SetNameInClass(__pyx_t_3, __pyx_n_s_not_found, __pyx_t_1) < 0) __PYX_ERR(16, 67, __pyx_L1_error)
  __Pyx_DECREF(__pyx_t_1); __pyx_t_1 = 0;

  /* "src/python/grpcio/grpc/_cython/_cygrpc/records.pyx.pxi":68
 *   deadline_exceeded = GRPC_STATUS_DEADLINE_EXCEEDED
 *   not_found = GRPC_STATUS_NOT_FOUND
 *   already_exists = GRPC_STATUS_ALREADY_EXISTS             # <<<<<<<<<<<<<<
 *   permission_denied = GRPC_STATUS_PERMISSION_DENIED
 *   unauthenticated = GRPC_STATUS_UNAUTHENTICATED
 */
  __pyx_t_1 = __Pyx_PyInt_From_grpc_status_code(GRPC_STATUS_ALREADY_EXISTS); if (unlikely(!__pyx_t_1)) __PYX_ERR(16, 68, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_1);
  if (__Pyx_SetNameInClass(__pyx_t_3, __pyx_n_s_already_exists, __pyx_t_1) < 0) __PYX_ERR(16, 68, __pyx_L1_error)
  __Pyx_DECREF(__pyx_t_1); __pyx_t_1 = 0;

  /* "src/python/grpcio/grpc/_cython/_cygrpc/records.pyx.pxi":69
 *   not_found = GRPC_STATUS_NOT_FOUND
 *   already_exists = GRPC_STATUS_ALREADY_EXISTS
 *   permission_denied = GRPC_STATUS_PERMISSION_DENIED             # <<<<<<<<<<<<<<
 *   unauthenticated = GRPC_STATUS_UNAUTHENTICATED
 *   resource_exhausted = GRPC_STATUS_RESOURCE_EXHAUSTED
 */
  __pyx_t_1 = __Pyx_PyInt_From_grpc_status_code(GRPC_STATUS_PERMISSION_DENIED); if (unlikely(!__pyx_t_1)) __PYX_ERR(16, 69, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_1);
  if (__Pyx_SetNameInClass(__pyx_t_3, __pyx_n_s_permission_denied, __pyx_t_1) < 0) __PYX_ERR(16, 69, __pyx_L1_error)
  __Pyx_DECREF(__pyx_t_1); __pyx_t_1 = 0;

  /* "src/python/grpcio/grpc/_cython/_cygrpc/records.pyx.pxi":70
 *   already_exists = GRPC_STATUS_ALREADY_EXISTS
 *   permission_denied = GRPC_STATUS_PERMISSION_DENIED
 *   unauthenticated = GRPC_STATUS_UNAUTHENTICATED             # <<<<<<<<<<<<<<
 *   resource_exhausted = GRPC_STATUS_RESOURCE_EXHAUSTED
 *   failed_precondition = GRPC_STATUS_FAILED_PRECONDITION
 */
  __pyx_t_1 = __Pyx_PyInt_From_grpc_status_code(GRPC_STATUS_UNAUTHENTICATED); if (unlikely(!__pyx_t_1)) __PYX_ERR(16, 70, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_1);
  if (__Pyx_SetNameInClass(__pyx_t_3, __pyx_n_s_unauthenticated, __pyx_t_1) < 0) __PYX_ERR(16, 70, __pyx_L1_error)
  __Pyx_DECREF(__pyx_t_1); __pyx_t_1 = 0;

  /* "src/python/grpcio/grpc/_cython/_cygrpc/records.pyx.pxi":71
 *   permission_denied = GRPC_STATUS_PERMISSION_DENIED
 *   unauthenticated = GRPC_STATUS_UNAUTHENTICATED
 *   resource_exhausted = GRPC_STATUS_RESOURCE_EXHAUSTED             # <<<<<<<<<<<<<<
 *   failed_precondition = GRPC_STATUS_FAILED_PRECONDITION
 *   aborted = GRPC_STATUS_ABORTED
 */
  __pyx_t_1 = __Pyx_PyInt_From_grpc_status_code(GRPC_STATUS_RESOURCE_EXHAUSTED); if (unlikely(!__pyx_t_1)) __PYX_ERR(16, 71, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_1);
  if (__Pyx_SetNameInClass(__pyx_t_3, __pyx_n_s_resource_exhausted, __pyx_t_1) < 0) __PYX_ERR(16, 71, __pyx_L1_error)
  __Pyx_DECREF(__pyx_t_1); __pyx_t_1 = 0;

  /* "src/python/grpcio/grpc/_cython/_cygrpc/records.pyx.pxi":72
 *   unauthenticated = GRPC_STATUS_UNAUTHENTICATED
 *   resource_exhausted = GRPC_STATUS_RESOURCE_EXHAUSTED
 *   failed_precondition = GRPC_STATUS_FAILED_PRECONDITION             # <<<<<<<<<<<<<<
 *   aborted = GRPC_STATUS_ABORTED
 *   out_of_range = GRPC_STATUS_OUT_OF_RANGE
 */
  __pyx_t_1 = __Pyx_PyInt_From_grpc_status_code(GRPC_STATUS_FAILED_PRECONDITION); if (unlikely(!__pyx_t_1)) __PYX_ERR(16, 72, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_1);
  if (__Pyx_SetNameInClass(__pyx_t_3, __pyx_n_s_failed_precondition, __pyx_t_1) < 0) __PYX_ERR(16, 72, __pyx_L1_error)
  __Pyx_DECREF(__pyx_t_1); __pyx_t_1 = 0;

  /* "src/python/grpcio/grpc/_cython/_cygrpc/records.pyx.pxi":73
 *   resource_exhausted = GRPC_STATUS_RESOURCE_EXHAUSTED
 *   failed_precondition = GRPC_STATUS_FAILED_PRECONDITION
 *   aborted = GRPC_STATUS_ABORTED             # <<<<<<<<<<<<<<
 *   out_of_range = GRPC_STATUS_OUT_OF_RANGE
 *   unimplemented = GRPC_STATUS_UNIMPLEMENTED
 */
  __pyx_t_1 = __Pyx_PyInt_From_grpc_status_code(GRPC_STATUS_ABORTED); if (unlikely(!__pyx_t_1)) __PYX_ERR(16, 73, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_1);
  if (__Pyx_SetNameInClass(__pyx_t_3, __pyx_n_s_aborted, __pyx_t_1) < 0) __PYX_ERR(16, 73, __pyx_L1_error)
  __Pyx_DECREF(__pyx_t_1); __pyx_t_1 = 0;

  /* "src/python/grpcio/grpc/_cython/_cygrpc/records.pyx.pxi":74
 *   failed_precondition = GRPC_STATUS_FAILED_PRECONDITION
 *   aborted = GRPC_STATUS_ABORTED
 *   out_of_range = GRPC_STATUS_OUT_OF_RANGE             # <<<<<<<<<<<<<<
 *   unimplemented = GRPC_STATUS_UNIMPLEMENTED
 *   internal = GRPC_STATUS_INTERNAL
 */
  __pyx_t_1 = __Pyx_PyInt_From_grpc_status_code(GRPC_STATUS_OUT_OF_RANGE); if (unlikely(!__pyx_t_1)) __PYX_ERR(16, 74, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_1);
  if (__Pyx_SetNameInClass(__pyx_t_3, __pyx_n_s_out_of_range, __pyx_t_1) < 0) __PYX_ERR(16, 74, __pyx_L1_error)
  __Pyx_DECREF(__pyx_t_1); __pyx_t_1 = 0;

  /* "src/python/grpcio/grpc/_cython/_cygrpc/records.pyx.pxi":75
 *   aborted = GRPC_STATUS_ABORTED
 *   out_of_range = GRPC_STATUS_OUT_OF_RANGE
 *   unimplemented = GRPC_STATUS_UNIMPLEMENTED             # <<<<<<<<<<<<<<
 *   internal = GRPC_STATUS_INTERNAL
 *   unavailable = GRPC_STATUS_UNAVAILABLE
 */
  __pyx_t_1 = __Pyx_PyInt_From_grpc_status_code(GRPC_STATUS_UNIMPLEMENTED); if (unlikely(!__pyx_t_1)) __PYX_ERR(16, 75, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_1);
  if (__Pyx_SetNameInClass(__pyx_t_3, __pyx_n_s_unimplemented, __pyx_t_1) < 0) __PYX_ERR(16, 75, __pyx_L1_error)
  __Pyx_DECREF(__pyx_t_1); __pyx_t_1 = 0;

  /* "src/python/grpcio/grpc/_cython/_cygrpc/records.pyx.pxi":76
 *   out_of_range = GRPC_STATUS_OUT_OF_RANGE
 *   unimplemented = GRPC_STATUS_UNIMPLEMENTED
 *   internal = GRPC_STATUS_INTERNAL             # <<<<<<<<<<<<<<
 *   unavailable = GRPC_STATUS_UNAVAILABLE
 *   data_loss = GRPC_STATUS_DATA_LOSS
 */
  __pyx_t_1 = __Pyx_PyInt_From_grpc_status_code(GRPC_STATUS_INTERNAL); if (unlikely(!__pyx_t_1)) __PYX_ERR(16, 76, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_1);
  if (__Pyx_SetNameInClass(__pyx_t_3, __pyx_n_s_internal, __pyx_t_1) < 0) __PYX_ERR(16, 76, __pyx_L1_error)
  __Pyx_DECREF(__pyx_t_1); __pyx_t_1 = 0;

  /* "src/python/grpcio/grpc/_cython/_cygrpc/records.pyx.pxi":77
 *   unimplemented = GRPC_STATUS_UNIMPLEMENTED
 *   internal = GRPC_STATUS_INTERNAL
 *   unavailable = GRPC_STATUS_UNAVAILABLE             # <<<<<<<<<<<<<<
 *   data_loss = GRPC_STATUS_DATA_LOSS
 * 
 */
  __pyx_t_1 = __Pyx_PyInt_From_grpc_status_code(GRPC_STATUS_UNAVAILABLE); if (unlikely(!__pyx_t_1)) __PYX_ERR(16, 77, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_1);
  if (__Pyx_SetNameInClass(__pyx_t_3, __pyx_n_s_unavailable, __pyx_t_1) < 0) __PYX_ERR(16, 77, __pyx_L1_error)
  __Pyx_DECREF(__pyx_t_1); __pyx_t_1 = 0;

  /* "src/python/grpcio/grpc/_cython/_cygrpc/records.pyx.pxi":78
 *   internal = GRPC_STATUS_INTERNAL
 *   unavailable = GRPC_STATUS_UNAVAILABLE
 *   data_loss = GRPC_STATUS_DATA_LOSS             # <<<<<<<<<<<<<<
 * 
 * 
 */
  __pyx_t_1 = __Pyx_PyInt_From_grpc_status_code(GRPC_STATUS_DATA_LOSS); if (unlikely(!__pyx_t_1)) __PYX_ERR(16, 78, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_1);
  if (__Pyx_SetNameInClass(__pyx_t_3, __pyx_n_s_data_loss, __pyx_t_1) < 0) __PYX_ERR(16, 78, __pyx_L1_error)
  __Pyx_DECREF(__pyx_t_1); __pyx_t_1 = 0;

  /* "src/python/grpcio/grpc/_cython/_cygrpc/records.pyx.pxi":61
 * 
 * 
 * class StatusCode:             # <<<<<<<<<<<<<<
 *   ok = GRPC_STATUS_OK
 *   cancelled = GRPC_STATUS_CANCELLED
 */
  __pyx_t_1 = __Pyx_Py3ClassCreate(((PyObject*)&__Pyx_DefaultClassType), __pyx_n_s_StatusCode, __pyx_empty_tuple, __pyx_t_3, NULL, 0, 1); if (unlikely(!__pyx_t_1)) __PYX_ERR(16, 61, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_1);
  if (PyDict_SetItem(__pyx_d, __pyx_n_s_StatusCode, __pyx_t_1) < 0) __PYX_ERR(16, 61, __pyx_L1_error)
  __Pyx_DECREF(__pyx_t_1); __pyx_t_1 = 0;
  __Pyx_DECREF(__pyx_t_3); __pyx_t_3 = 0;

  /* "src/python/grpcio/grpc/_cython/_cygrpc/records.pyx.pxi":81
 * 
 * 
 * class CallError:             # <<<<<<<<<<<<<<
 *   ok = GRPC_CALL_OK
 *   error = GRPC_CALL_ERROR
 */
  __pyx_t_3 = __Pyx_Py3MetaclassPrepare((PyObject *) NULL, __pyx_empty_tuple, __pyx_n_s_CallError, __pyx_n_s_CallError, (PyObject *) NULL, __pyx_n_s_grpc__cython_cygrpc, (PyObject *) NULL); if (unlikely(!__pyx_t_3)) __PYX_ERR(16, 81, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_3);

  /* "src/python/grpcio/grpc/_cython/_cygrpc/records.pyx.pxi":82
 * 
 * class CallError:
 *   ok = GRPC_CALL_OK             # <<<<<<<<<<<<<<
 *   error = GRPC_CALL_ERROR
 *   not_on_server = GRPC_CALL_ERROR_NOT_ON_SERVER
 */
  __pyx_t_1 = __Pyx_PyInt_From_grpc_call_error(GRPC_CALL_OK); if (unlikely(!__pyx_t_1)) __PYX_ERR(16, 82, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_1);
  if (__Pyx_SetNameInClass(__pyx_t_3, __pyx_n_s_ok, __pyx_t_1) < 0) __PYX_ERR(16, 82, __pyx_L1_error)
  __Pyx_DECREF(__pyx_t_1); __pyx_t_1 = 0;

  /* "src/python/grpcio/grpc/_cython/_cygrpc/records.pyx.pxi":83
 * class CallError:
 *   ok = GRPC_CALL_OK
 *   error = GRPC_CALL_ERROR             # <<<<<<<<<<<<<<
 *   not_on_server = GRPC_CALL_ERROR_NOT_ON_SERVER
 *   not_on_client = GRPC_CALL_ERROR_NOT_ON_CLIENT
 */
  __pyx_t_1 = __Pyx_PyInt_From_grpc_call_error(GRPC_CALL_ERROR); if (unlikely(!__pyx_t_1)) __PYX_ERR(16, 83, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_1);
  if (__Pyx_SetNameInClass(__pyx_t_3, __pyx_n_s_error, __pyx_t_1) < 0) __PYX_ERR(16, 83, __pyx_L1_error)
  __Pyx_DECREF(__pyx_t_1); __pyx_t_1 = 0;

  /* "src/python/grpcio/grpc/_cython/_cygrpc/records.pyx.pxi":84
 *   ok = GRPC_CALL_OK
 *   error = GRPC_CALL_ERROR
 *   not_on_server = GRPC_CALL_ERROR_NOT_ON_SERVER             # <<<<<<<<<<<<<<
 *   not_on_client = GRPC_CALL_ERROR_NOT_ON_CLIENT
 *   already_accepted = GRPC_CALL_ERROR_ALREADY_ACCEPTED
 */
  __pyx_t_1 = __Pyx_PyInt_From_grpc_call_error(GRPC_CALL_ERROR_NOT_ON_SERVER); if (unlikely(!__pyx_t_1)) __PYX_ERR(16, 84, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_1);
  if (__Pyx_SetNameInClass(__pyx_t_3, __pyx_n_s_not_on_server, __pyx_t_1) < 0) __PYX_ERR(16, 84, __pyx_L1_error)
  __Pyx_DECREF(__pyx_t_1); __pyx_t_1 = 0;

  /* "src/python/grpcio/grpc/_cython/_cygrpc/records.pyx.pxi":85
 *   error = GRPC_CALL_ERROR
 *   not_on_server = GRPC_CALL_ERROR_NOT_ON_SERVER
 *   not_on_client = GRPC_CALL_ERROR_NOT_ON_CLIENT             # <<<<<<<<<<<<<<
 *   already_accepted = GRPC_CALL_ERROR_ALREADY_ACCEPTED
 *   already_invoked = GRPC_CALL_ERROR_ALREADY_INVOKED
 */
  __pyx_t_1 = __Pyx_PyInt_From_grpc_call_error(GRPC_CALL_ERROR_NOT_ON_CLIENT); if (unlikely(!__pyx_t_1)) __PYX_ERR(16, 85, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_1);
  if (__Pyx_SetNameInClass(__pyx_t_3, __pyx_n_s_not_on_client, __pyx_t_1) < 0) __PYX_ERR(16, 85, __pyx_L1_error)
  __Pyx_DECREF(__pyx_t_1); __pyx_t_1 = 0;

  /* "src/python/grpcio/grpc/_cython/_cygrpc/records.pyx.pxi":86
 *   not_on_server = GRPC_CALL_ERROR_NOT_ON_SERVER
 *   not_on_client = GRPC_CALL_ERROR_NOT_ON_CLIENT
 *   already_accepted = GRPC_CALL_ERROR_ALREADY_ACCEPTED             # <<<<<<<<<<<<<<
 *   already_invoked = GRPC_CALL_ERROR_ALREADY_INVOKED
 *   not_invoked = GRPC_CALL_ERROR_NOT_INVOKED
 */
  __pyx_t_1 = __Pyx_PyInt_From_grpc_call_error(GRPC_CALL_ERROR_ALREADY_ACCEPTED); if (unlikely(!__pyx_t_1)) __PYX_ERR(16, 86, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_1);
  if (__Pyx_SetNameInClass(__pyx_t_3, __pyx_n_s_already_accepted, __pyx_t_1) < 0) __PYX_ERR(16, 86, __pyx_L1_error)
  __Pyx_DECREF(__pyx_t_1); __pyx_t_1 = 0;

  /* "src/python/grpcio/grpc/_cython/_cygrpc/records.pyx.pxi":87
 *   not_on_client = GRPC_CALL_ERROR_NOT_ON_CLIENT
 *   already_accepted = GRPC_CALL_ERROR_ALREADY_ACCEPTED
 *   already_invoked = GRPC_CALL_ERROR_ALREADY_INVOKED             # <<<<<<<<<<<<<<
 *   not_invoked = GRPC_CALL_ERROR_NOT_INVOKED
 *   already_finished = GRPC_CALL_ERROR_ALREADY_FINISHED
 */
  __pyx_t_1 = __Pyx_PyInt_From_grpc_call_error(GRPC_CALL_ERROR_ALREADY_INVOKED); if (unlikely(!__pyx_t_1)) __PYX_ERR(16, 87, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_1);
  if (__Pyx_SetNameInClass(__pyx_t_3, __pyx_n_s_already_invoked, __pyx_t_1) < 0) __PYX_ERR(16, 87, __pyx_L1_error)
  __Pyx_DECREF(__pyx_t_1); __pyx_t_1 = 0;

  /* "src/python/grpcio/grpc/_cython/_cygrpc/records.pyx.pxi":88
 *   already_accepted = GRPC_CALL_ERROR_ALREADY_ACCEPTED
 *   already_invoked = GRPC_CALL_ERROR_ALREADY_INVOKED
 *   not_invoked = GRPC_CALL_ERROR_NOT_INVOKED             # <<<<<<<<<<<<<<
 *   already_finished = GRPC_CALL_ERROR_ALREADY_FINISHED
 *   too_many_operations = GRPC_CALL_ERROR_TOO_MANY_OPERATIONS
 */
  __pyx_t_1 = __Pyx_PyInt_From_grpc_call_error(GRPC_CALL_ERROR_NOT_INVOKED); if (unlikely(!__pyx_t_1)) __PYX_ERR(16, 88, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_1);
  if (__Pyx_SetNameInClass(__pyx_t_3, __pyx_n_s_not_invoked, __pyx_t_1) < 0) __PYX_ERR(16, 88, __pyx_L1_error)
  __Pyx_DECREF(__pyx_t_1); __pyx_t_1 = 0;

  /* "src/python/grpcio/grpc/_cython/_cygrpc/records.pyx.pxi":89
 *   already_invoked = GRPC_CALL_ERROR_ALREADY_INVOKED
 *   not_invoked = GRPC_CALL_ERROR_NOT_INVOKED
 *   already_finished = GRPC_CALL_ERROR_ALREADY_FINISHED             # <<<<<<<<<<<<<<
 *   too_many_operations = GRPC_CALL_ERROR_TOO_MANY_OPERATIONS
 *   invalid_flags = GRPC_CALL_ERROR_INVALID_FLAGS
 */
  __pyx_t_1 = __Pyx_PyInt_From_grpc_call_error(GRPC_CALL_ERROR_ALREADY_FINISHED); if (unlikely(!__pyx_t_1)) __PYX_ERR(16, 89, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_1);
  if (__Pyx_SetNameInClass(__pyx_t_3, __pyx_n_s_already_finished, __pyx_t_1) < 0) __PYX_ERR(16, 89, __pyx_L1_error)
  __Pyx_DECREF(__pyx_t_1); __pyx_t_1 = 0;

  /* "src/python/grpcio/grpc/_cython/_cygrpc/records.pyx.pxi":90
 *   not_invoked = GRPC_CALL_ERROR_NOT_INVOKED
 *   already_finished = GRPC_CALL_ERROR_ALREADY_FINISHED
 *   too_many_operations = GRPC_CALL_ERROR_TOO_MANY_OPERATIONS             # <<<<<<<<<<<<<<
 *   invalid_flags = GRPC_CALL_ERROR_INVALID_FLAGS
 *   invalid_metadata = GRPC_CALL_ERROR_INVALID_METADATA
 */
  __pyx_t_1 = __Pyx_PyInt_From_grpc_call_error(GRPC_CALL_ERROR_TOO_MANY_OPERATIONS); if (unlikely(!__pyx_t_1)) __PYX_ERR(16, 90, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_1);
  if (__Pyx_SetNameInClass(__pyx_t_3, __pyx_n_s_too_many_operations, __pyx_t_1) < 0) __PYX_ERR(16, 90, __pyx_L1_error)
  __Pyx_DECREF(__pyx_t_1); __pyx_t_1 = 0;

  /* "src/python/grpcio/grpc/_cython/_cygrpc/records.pyx.pxi":91
 *   already_finished = GRPC_CALL_ERROR_ALREADY_FINISHED
 *   too_many_operations = GRPC_CALL_ERROR_TOO_MANY_OPERATIONS
 *   invalid_flags = GRPC_CALL_ERROR_INVALID_FLAGS             # <<<<<<<<<<<<<<
 *   invalid_metadata = GRPC_CALL_ERROR_INVALID_METADATA
 * 
 */
  __pyx_t_1 = __Pyx_PyInt_From_grpc_call_error(GRPC_CALL_ERROR_INVALID_FLAGS); if (unlikely(!__pyx_t_1)) __PYX_ERR(16, 91, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_1);
  if (__Pyx_SetNameInClass(__pyx_t_3, __pyx_n_s_invalid_flags, __pyx_t_1) < 0) __PYX_ERR(16, 91, __pyx_L1_error)
  __Pyx_DECREF(__pyx_t_1); __pyx_t_1 = 0;

  /* "src/python/grpcio/grpc/_cython/_cygrpc/records.pyx.pxi":92
 *   too_many_operations = GRPC_CALL_ERROR_TOO_MANY_OPERATIONS
 *   invalid_flags = GRPC_CALL_ERROR_INVALID_FLAGS
 *   invalid_metadata = GRPC_CALL_ERROR_INVALID_METADATA             # <<<<<<<<<<<<<<
 * 
 * 
 */
  __pyx_t_1 = __Pyx_PyInt_From_grpc_call_error(GRPC_CALL_ERROR_INVALID_METADATA); if (unlikely(!__pyx_t_1)) __PYX_ERR(16, 92, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_1);
  if (__Pyx_SetNameInClass(__pyx_t_3, __pyx_n_s_invalid_metadata, __pyx_t_1) < 0) __PYX_ERR(16, 92, __pyx_L1_error)
  __Pyx_DECREF(__pyx_t_1); __pyx_t_1 = 0;

  /* "src/python/grpcio/grpc/_cython/_cygrpc/records.pyx.pxi":81
 * 
 * 
 * class CallError:             # <<<<<<<<<<<<<<
 *   ok = GRPC_CALL_OK
 *   error = GRPC_CALL_ERROR
 */
  __pyx_t_1 = __Pyx_Py3ClassCreate(((PyObject*)&__Pyx_DefaultClassType), __pyx_n_s_CallError, __pyx_empty_tuple, __pyx_t_3, NULL, 0, 1); if (unlikely(!__pyx_t_1)) __PYX_ERR(16, 81, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_1);
  if (PyDict_SetItem(__pyx_d, __pyx_n_s_CallError, __pyx_t_1) < 0) __PYX_ERR(16, 81, __pyx_L1_error)
  __Pyx_DECREF(__pyx_t_1); __pyx_t_1 = 0;
  __Pyx_DECREF(__pyx_t_3); __pyx_t_3 = 0;

  /* "src/python/grpcio/grpc/_cython/_cygrpc/records.pyx.pxi":95
 * 
 * 
 * class CompletionType:             # <<<<<<<<<<<<<<
 *   queue_shutdown = GRPC_QUEUE_SHUTDOWN
 *   queue_timeout = GRPC_QUEUE_TIMEOUT
 */
  __pyx_t_3 = __Pyx_Py3MetaclassPrepare((PyObject *) NULL, __pyx_empty_tuple, __pyx_n_s_CompletionType, __pyx_n_s_CompletionType, (PyObject *) NULL, __pyx_n_s_grpc__cython_cygrpc, (PyObject *) NULL); if (unlikely(!__pyx_t_3)) __PYX_ERR(16, 95, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_3);

  /* "src/python/grpcio/grpc/_cython/_cygrpc/records.pyx.pxi":96
 * 
 * class CompletionType:
 *   queue_shutdown = GRPC_QUEUE_SHUTDOWN             # <<<<<<<<<<<<<<
 *   queue_timeout = GRPC_QUEUE_TIMEOUT
 *   operation_complete = GRPC_OP_COMPLETE
 */
  __pyx_t_1 = __Pyx_PyInt_From_grpc_completion_type(GRPC_QUEUE_SHUTDOWN); if (unlikely(!__pyx_t_1)) __PYX_ERR(16, 96, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_1);
  if (__Pyx_SetNameInClass(__pyx_t_3, __pyx_n_s_queue_shutdown, __pyx_t_1) < 0) __PYX_ERR(16, 96, __pyx_L1_error)
  __Pyx_DECREF(__pyx_t_1); __pyx_t_1 = 0;

  /* "src/python/grpcio/grpc/_cython/_cygrpc/records.pyx.pxi":97
 * class CompletionType:
 *   queue_shutdown = GRPC_QUEUE_SHUTDOWN
 *   queue_timeout = GRPC_QUEUE_TIMEOUT             # <<<<<<<<<<<<<<
 *   operation_complete = GRPC_OP_COMPLETE
 * 
 */
  __pyx_t_1 = __Pyx_PyInt_From_grpc_completion_type(GRPC_QUEUE_TIMEOUT); if (unlikely(!__pyx_t_1)) __PYX_ERR(16, 97, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_1);
  if (__Pyx_SetNameInClass(__pyx_t_3, __pyx_n_s_queue_timeout, __pyx_t_1) < 0) __PYX_ERR(16, 97, __pyx_L1_error)
  __Pyx_DECREF(__pyx_t_1); __pyx_t_1 = 0;

  /* "src/python/grpcio/grpc/_cython/_cygrpc/records.pyx.pxi":98
 *   queue_shutdown = GRPC_QUEUE_SHUTDOWN
 *   queue_timeout = GRPC_QUEUE_TIMEOUT
 *   operation_complete = GRPC_OP_COMPLETE             # <<<<<<<<<<<<<<
 * 
 * 
 */
  __pyx_t_1 = __Pyx_PyInt_From_grpc_completion_type(GRPC_OP_COMPLETE); if (unlikely(!__pyx_t_1)) __PYX_ERR(16, 98, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_1);
  if (__Pyx_SetNameInClass(__pyx_t_3, __pyx_n_s_operation_complete, __pyx_t_1) < 0) __PYX_ERR(16, 98, __pyx_L1_error)
  __Pyx_DECREF(__pyx_t_1); __pyx_t_1 = 0;

  /* "src/python/grpcio/grpc/_cython/_cygrpc/records.pyx.pxi":95
 * 
 * 
 * class CompletionType:             # <<<<<<<<<<<<<<
 *   queue_shutdown = GRPC_QUEUE_SHUTDOWN
 *   queue_timeout = GRPC_QUEUE_TIMEOUT
 */
  __pyx_t_1 = __Pyx_Py3ClassCreate(((PyObject*)&__Pyx_DefaultClassType), __pyx_n_s_CompletionType, __pyx_empty_tuple, __pyx_t_3, NULL, 0, 1); if (unlikely(!__pyx_t_1)) __PYX_ERR(16, 95, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_1);
  if (PyDict_SetItem(__pyx_d, __pyx_n_s_CompletionType, __pyx_t_1) < 0) __PYX_ERR(16, 95, __pyx_L1_error)
  __Pyx_DECREF(__pyx_t_1); __pyx_t_1 = 0;
  __Pyx_DECREF(__pyx_t_3); __pyx_t_3 = 0;

  /* "src/python/grpcio/grpc/_cython/_cygrpc/records.pyx.pxi":101
 * 
 * 
 * class OperationType:             # <<<<<<<<<<<<<<
 *   send_initial_metadata = GRPC_OP_SEND_INITIAL_METADATA
 *   send_message = GRPC_OP_SEND_MESSAGE
 */
  __pyx_t_3 = __Pyx_Py3MetaclassPrepare((PyObject *) NULL, __pyx_empty_tuple, __pyx_n_s_OperationType, __pyx_n_s_OperationType, (PyObject *) NULL, __pyx_n_s_grpc__cython_cygrpc, (PyObject *) NULL); if (unlikely(!__pyx_t_3)) __PYX_ERR(16, 101, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_3);

  /* "src/python/grpcio/grpc/_cython/_cygrpc/records.pyx.pxi":102
 * 
 * class OperationType:
 *   send_initial_metadata = GRPC_OP_SEND_INITIAL_METADATA             # <<<<<<<<<<<<<<
 *   send_message = GRPC_OP_SEND_MESSAGE
 *   send_close_from_client = GRPC_OP_SEND_CLOSE_FROM_CLIENT
 */
  __pyx_t_1 = __Pyx_PyInt_From_grpc_op_type(GRPC_OP_SEND_INITIAL_METADATA); if (unlikely(!__pyx_t_1)) __PYX_ERR(16, 102, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_1);
  if (__Pyx_SetNameInClass(__pyx_t_3, __pyx_n_s_send_initial_metadata, __pyx_t_1) < 0) __PYX_ERR(16, 102, __pyx_L1_error)
  __Pyx_DECREF(__pyx_t_1); __pyx_t_1 = 0;

  /* "src/python/grpcio/grpc/_cython/_cygrpc/records.pyx.pxi":103
 * class OperationType:
 *   send_initial_metadata = GRPC_OP_SEND_INITIAL_METADATA
 *   send_message = GRPC_OP_SEND_MESSAGE             # <<<<<<<<<<<<<<
 *   send_close_from_client = GRPC_OP_SEND_CLOSE_FROM_CLIENT
 *   send_status_from_server = GRPC_OP_SEND_STATUS_FROM_SERVER
 */
  __pyx_t_1 = __Pyx_PyInt_From_grpc_op_type(GRPC_OP_SEND_MESSAGE); if (unlikely(!__pyx_t_1)) __PYX_ERR(16, 103, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_1);
  if (__Pyx_SetNameInClass(__pyx_t_3, __pyx_n_s_send_message, __pyx_t_1) < 0) __PYX_ERR(16, 103, __pyx_L1_error)
  __Pyx_DECREF(__pyx_t_1); __pyx_t_1 = 0;

  /* "src/python/grpcio/grpc/_cython/_cygrpc/records.pyx.pxi":104
 *   send_initial_metadata = GRPC_OP_SEND_INITIAL_METADATA
 *   send_message = GRPC_OP_SEND_MESSAGE
 *   send_close_from_client = GRPC_OP_SEND_CLOSE_FROM_CLIENT             # <<<<<<<<<<<<<<
 *   send_status_from_server = GRPC_OP_SEND_STATUS_FROM_SERVER
 *   receive_initial_metadata = GRPC_OP_RECV_INITIAL_METADATA
 */
  __pyx_t_1 = __Pyx_PyInt_From_grpc_op_type(GRPC_OP_SEND_CLOSE_FROM_CLIENT); if (unlikely(!__pyx_t_1)) __PYX_ERR(16, 104, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_1);
  if (__Pyx_SetNameInClass(__pyx_t_3, __pyx_n_s_send_close_from_client, __pyx_t_1) < 0) __PYX_ERR(16, 104, __pyx_L1_error)
  __Pyx_DECREF(__pyx_t_1); __pyx_t_1 = 0;

  /* "src/python/grpcio/grpc/_cython/_cygrpc/records.pyx.pxi":105
 *   send_message = GRPC_OP_SEND_MESSAGE
 *   send_close_from_client = GRPC_OP_SEND_CLOSE_FROM_CLIENT
 *   send_status_from_server = GRPC_OP_SEND_STATUS_FROM_SERVER             # <<<<<<<<<<<<<<
 *   receive_initial_metadata = GRPC_OP_RECV_INITIAL_METADATA
 *   receive_message = GRPC_OP_RECV_MESSAGE
 */
  __pyx_t_1 = __Pyx_PyInt_From_grpc_op_type(GRPC_OP_SEND_STATUS_FROM_SERVER); if (unlikely(!__pyx_t_1)) __PYX_ERR(16, 105, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_1);
  if (__Pyx_SetNameInClass(__pyx_t_3, __pyx_n_s_send_status_from_server, __pyx_t_1) < 0) __PYX_ERR(16, 105, __pyx_L1_error)
  __Pyx_DECREF(__pyx_t_1); __pyx_t_1 = 0;

  /* "src/python/grpcio/grpc/_cython/_cygrpc/records.pyx.pxi":106
 *   send_close_from_client = GRPC_OP_SEND_CLOSE_FROM_CLIENT
 *   send_status_from_server = GRPC_OP_SEND_STATUS_FROM_SERVER
 *   receive_initial_metadata = GRPC_OP_RECV_INITIAL_METADATA             # <<<<<<<<<<<<<<
 *   receive_message = GRPC_OP_RECV_MESSAGE
 *   receive_status_on_client = GRPC_OP_RECV_STATUS_ON_CLIENT
 */
  __pyx_t_1 = __Pyx_PyInt_From_grpc_op_type(GRPC_OP_RECV_INITIAL_METADATA); if (unlikely(!__pyx_t_1)) __PYX_ERR(16, 106, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_1);
  if (__Pyx_SetNameInClass(__pyx_t_3, __pyx_n_s_receive_initial_metadata, __pyx_t_1) < 0) __PYX_ERR(16, 106, __pyx_L1_error)
  __Pyx_DECREF(__pyx_t_1); __pyx_t_1 = 0;

  /* "src/python/grpcio/grpc/_cython/_cygrpc/records.pyx.pxi":107
 *   send_status_from_server = GRPC_OP_SEND_STATUS_FROM_SERVER
 *   receive_initial_metadata = GRPC_OP_RECV_INITIAL_METADATA
 *   receive_message = GRPC_OP_RECV_MESSAGE             # <<<<<<<<<<<<<<
 *   receive_status_on_client = GRPC_OP_RECV_STATUS_ON_CLIENT
 *   receive_close_on_server = GRPC_OP_RECV_CLOSE_ON_SERVER
 */
  __pyx_t_1 = __Pyx_PyInt_From_grpc_op_type(GRPC_OP_RECV_MESSAGE); if (unlikely(!__pyx_t_1)) __PYX_ERR(16, 107, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_1);
  if (__Pyx_SetNameInClass(__pyx_t_3, __pyx_n_s_receive_message, __pyx_t_1) < 0) __PYX_ERR(16, 107, __pyx_L1_error)
  __Pyx_DECREF(__pyx_t_1); __pyx_t_1 = 0;

  /* "src/python/grpcio/grpc/_cython/_cygrpc/records.pyx.pxi":108
 *   receive_initial_metadata = GRPC_OP_RECV_INITIAL_METADATA
 *   receive_message = GRPC_OP_RECV_MESSAGE
 *   receive_status_on_client = GRPC_OP_RECV_STATUS_ON_CLIENT             # <<<<<<<<<<<<<<
 *   receive_close_on_server = GRPC_OP_RECV_CLOSE_ON_SERVER
 * 
 */
  __pyx_t_1 = __Pyx_PyInt_From_grpc_op_type(GRPC_OP_RECV_STATUS_ON_CLIENT); if (unlikely(!__pyx_t_1)) __PYX_ERR(16, 108, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_1);
  if (__Pyx_SetNameInClass(__pyx_t_3, __pyx_n_s_receive_status_on_client, __pyx_t_1) < 0) __PYX_ERR(16, 108, __pyx_L1_error)
  __Pyx_DECREF(__pyx_t_1); __pyx_t_1 = 0;

  /* "src/python/grpcio/grpc/_cython/_cygrpc/records.pyx.pxi":109
 *   receive_message = GRPC_OP_RECV_MESSAGE
 *   receive_status_on_client = GRPC_OP_RECV_STATUS_ON_CLIENT
 *   receive_close_on_server = GRPC_OP_RECV_CLOSE_ON_SERVER             # <<<<<<<<<<<<<<
 * 
 * GRPC_COMPRESSION_CHANNEL_DEFAULT_ALGORITHM= (
 */
  __pyx_t_1 = __Pyx_PyInt_From_grpc_op_type(GRPC_OP_RECV_CLOSE_ON_SERVER); if (unlikely(!__pyx_t_1)) __PYX_ERR(16, 109, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_1);
  if (__Pyx_SetNameInClass(__pyx_t_3, __pyx_n_s_receive_close_on_server, __pyx_t_1) < 0) __PYX_ERR(16, 109, __pyx_L1_error)
  __Pyx_DECREF(__pyx_t_1); __pyx_t_1 = 0;

  /* "src/python/grpcio/grpc/_cython/_cygrpc/records.pyx.pxi":101
 * 
 * 
 * class OperationType:             # <<<<<<<<<<<<<<
 *   send_initial_metadata = GRPC_OP_SEND_INITIAL_METADATA
 *   send_message = GRPC_OP_SEND_MESSAGE
 */
  __pyx_t_1 = __Pyx_Py3ClassCreate(((PyObject*)&__Pyx_DefaultClassType), __pyx_n_s_OperationType, __pyx_empty_tuple, __pyx_t_3, NULL, 0, 1); if (unlikely(!__pyx_t_1)) __PYX_ERR(16, 101, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_1);
  if (PyDict_SetItem(__pyx_d, __pyx_n_s_OperationType, __pyx_t_1) < 0) __PYX_ERR(16, 101, __pyx_L1_error)
  __Pyx_DECREF(__pyx_t_1); __pyx_t_1 = 0;
  __Pyx_DECREF(__pyx_t_3); __pyx_t_3 = 0;

  /* "src/python/grpcio/grpc/_cython/_cygrpc/records.pyx.pxi":112
 * 
 * GRPC_COMPRESSION_CHANNEL_DEFAULT_ALGORITHM= (
 *   _GRPC_COMPRESSION_CHANNEL_DEFAULT_ALGORITHM)             # <<<<<<<<<<<<<<
 * 
 * GRPC_COMPRESSION_REQUEST_ALGORITHM_MD_KEY = (
 */
  __pyx_t_3 = __Pyx_PyBytes_FromString(GRPC_COMPRESSION_CHANNEL_DEFAULT_ALGORITHM); if (unlikely(!__pyx_t_3)) __PYX_ERR(16, 112, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_3);
  if (PyDict_SetItem(__pyx_d, __pyx_n_s_GRPC_COMPRESSION_CHANNEL_DEFAULT, __pyx_t_3) < 0) __PYX_ERR(16, 111, __pyx_L1_error)
  __Pyx_DECREF(__pyx_t_3); __pyx_t_3 = 0;

  /* "src/python/grpcio/grpc/_cython/_cygrpc/records.pyx.pxi":115
 * 
 * GRPC_COMPRESSION_REQUEST_ALGORITHM_MD_KEY = (
 *   _GRPC_COMPRESSION_REQUEST_ALGORITHM_MD_KEY)             # <<<<<<<<<<<<<<
 * 
 * class CompressionAlgorithm:
 */
  __pyx_t_3 = __Pyx_PyBytes_FromString(GRPC_COMPRESSION_REQUEST_ALGORITHM_MD_KEY); if (unlikely(!__pyx_t_3)) __PYX_ERR(16, 115, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_3);
  if (PyDict_SetItem(__pyx_d, __pyx_n_s_GRPC_COMPRESSION_REQUEST_ALGORIT, __pyx_t_3) < 0) __PYX_ERR(16, 114, __pyx_L1_error)
  __Pyx_DECREF(__pyx_t_3); __pyx_t_3 = 0;

  /* "src/python/grpcio/grpc/_cython/_cygrpc/records.pyx.pxi":117
 *   _GRPC_COMPRESSION_REQUEST_ALGORITHM_MD_KEY)
 * 
 * class CompressionAlgorithm:             # <<<<<<<<<<<<<<
 *   none = GRPC_COMPRESS_NONE
 *   deflate = GRPC_COMPRESS_DEFLATE
 */
  __pyx_t_3 = __Pyx_Py3MetaclassPrepare((PyObject *) NULL, __pyx_empty_tuple, __pyx_n_s_CompressionAlgorithm, __pyx_n_s_CompressionAlgorithm, (PyObject *) NULL, __pyx_n_s_grpc__cython_cygrpc, (PyObject *) NULL); if (unlikely(!__pyx_t_3)) __PYX_ERR(16, 117, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_3);

  /* "src/python/grpcio/grpc/_cython/_cygrpc/records.pyx.pxi":118
 * 
 * class CompressionAlgorithm:
 *   none = GRPC_COMPRESS_NONE             # <<<<<<<<<<<<<<
 *   deflate = GRPC_COMPRESS_DEFLATE
 *   gzip = GRPC_COMPRESS_GZIP
 */
  __pyx_t_1 = __Pyx_PyInt_From_grpc_compression_algorithm(GRPC_COMPRESS_NONE); if (unlikely(!__pyx_t_1)) __PYX_ERR(16, 118, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_1);
  if (__Pyx_SetNameInClass(__pyx_t_3, __pyx_n_s_none, __pyx_t_1) < 0) __PYX_ERR(16, 118, __pyx_L1_error)
  __Pyx_DECREF(__pyx_t_1); __pyx_t_1 = 0;

  /* "src/python/grpcio/grpc/_cython/_cygrpc/records.pyx.pxi":119
 * class CompressionAlgorithm:
 *   none = GRPC_COMPRESS_NONE
 *   deflate = GRPC_COMPRESS_DEFLATE             # <<<<<<<<<<<<<<
 *   gzip = GRPC_COMPRESS_GZIP
 * 
 */
  __pyx_t_1 = __Pyx_PyInt_From_grpc_compression_algorithm(GRPC_COMPRESS_DEFLATE); if (unlikely(!__pyx_t_1)) __PYX_ERR(16, 119, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_1);
  if (__Pyx_SetNameInClass(__pyx_t_3, __pyx_n_s_deflate, __pyx_t_1) < 0) __PYX_ERR(16, 119, __pyx_L1_error)
  __Pyx_DECREF(__pyx_t_1); __pyx_t_1 = 0;

  /* "src/python/grpcio/grpc/_cython/_cygrpc/records.pyx.pxi":120
 *   none = GRPC_COMPRESS_NONE
 *   deflate = GRPC_COMPRESS_DEFLATE
 *   gzip = GRPC_COMPRESS_GZIP             # <<<<<<<<<<<<<<
 * 
 * 
 */
  __pyx_t_1 = __Pyx_PyInt_From_grpc_compression_algorithm(GRPC_COMPRESS_GZIP); if (unlikely(!__pyx_t_1)) __PYX_ERR(16, 120, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_1);
  if (__Pyx_SetNameInClass(__pyx_t_3, __pyx_n_s_gzip, __pyx_t_1) < 0) __PYX_ERR(16, 120, __pyx_L1_error)
  __Pyx_DECREF(__pyx_t_1); __pyx_t_1 = 0;

  /* "src/python/grpcio/grpc/_cython/_cygrpc/records.pyx.pxi":117
 *   _GRPC_COMPRESSION_REQUEST_ALGORITHM_MD_KEY)
 * 
 * class CompressionAlgorithm:             # <<<<<<<<<<<<<<
 *   none = GRPC_COMPRESS_NONE
 *   deflate = GRPC_COMPRESS_DEFLATE
 */
  __pyx_t_1 = __Pyx_Py3ClassCreate(((PyObject*)&__Pyx_DefaultClassType), __pyx_n_s_CompressionAlgorithm, __pyx_empty_tuple, __pyx_t_3, NULL, 0, 1); if (unlikely(!__pyx_t_1)) __PYX_ERR(16, 117, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_1);
  if (PyDict_SetItem(__pyx_d, __pyx_n_s_CompressionAlgorithm, __pyx_t_1) < 0) __PYX_ERR(16, 117, __pyx_L1_error)
  __Pyx_DECREF(__pyx_t_1); __pyx_t_1 = 0;
  __Pyx_DECREF(__pyx_t_3); __pyx_t_3 = 0;

  /* "src/python/grpcio/grpc/_cython/_cygrpc/records.pyx.pxi":123
 * 
 * 
 * class CompressionLevel:             # <<<<<<<<<<<<<<
 *   none = GRPC_COMPRESS_LEVEL_NONE
 *   low = GRPC_COMPRESS_LEVEL_LOW
 */
  __pyx_t_3 = __Pyx_Py3MetaclassPrepare((PyObject *) NULL, __pyx_empty_tuple, __pyx_n_s_CompressionLevel, __pyx_n_s_CompressionLevel, (PyObject *) NULL, __pyx_n_s_grpc__cython_cygrpc, (PyObject *) NULL); if (unlikely(!__pyx_t_3)) __PYX_ERR(16, 123, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_3);

  /* "src/python/grpcio/grpc/_cython/_cygrpc/records.pyx.pxi":124
 * 
 * class CompressionLevel:
 *   none = GRPC_COMPRESS_LEVEL_NONE             # <<<<<<<<<<<<<<
 *   low = GRPC_COMPRESS_LEVEL_LOW
 *   medium = GRPC_COMPRESS_LEVEL_MED
 */
  __pyx_t_1 = __Pyx_PyInt_From_grpc_compression_level(GRPC_COMPRESS_LEVEL_NONE); if (unlikely(!__pyx_t_1)) __PYX_ERR(16, 124, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_1);
  if (__Pyx_SetNameInClass(__pyx_t_3, __pyx_n_s_none, __pyx_t_1) < 0) __PYX_ERR(16, 124, __pyx_L1_error)
  __Pyx_DECREF(__pyx_t_1); __pyx_t_1 = 0;

  /* "src/python/grpcio/grpc/_cython/_cygrpc/records.pyx.pxi":125
 * class CompressionLevel:
 *   none = GRPC_COMPRESS_LEVEL_NONE
 *   low = GRPC_COMPRESS_LEVEL_LOW             # <<<<<<<<<<<<<<
 *   medium = GRPC_COMPRESS_LEVEL_MED
 *   high = GRPC_COMPRESS_LEVEL_HIGH
 */
  __pyx_t_1 = __Pyx_PyInt_From_grpc_compression_level(GRPC_COMPRESS_LEVEL_LOW); if (unlikely(!__pyx_t_1)) __PYX_ERR(16, 125, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_1);
  if (__Pyx_SetNameInClass(__pyx_t_3, __pyx_n_s_low, __pyx_t_1) < 0) __PYX_ERR(16, 125, __pyx_L1_error)
  __Pyx_DECREF(__pyx_t_1); __pyx_t_1 = 0;

  /* "src/python/grpcio/grpc/_cython/_cygrpc/records.pyx.pxi":126
 *   none = GRPC_COMPRESS_LEVEL_NONE
 *   low = GRPC_COMPRESS_LEVEL_LOW
 *   medium = GRPC_COMPRESS_LEVEL_MED             # <<<<<<<<<<<<<<
 *   high = GRPC_COMPRESS_LEVEL_HIGH
 * 
 */
  __pyx_t_1 = __Pyx_PyInt_From_grpc_compression_level(GRPC_COMPRESS_LEVEL_MED); if (unlikely(!__pyx_t_1)) __PYX_ERR(16, 126, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_1);
  if (__Pyx_SetNameInClass(__pyx_t_3, __pyx_n_s_medium, __pyx_t_1) < 0) __PYX_ERR(16, 126, __pyx_L1_error)
  __Pyx_DECREF(__pyx_t_1); __pyx_t_1 = 0;

  /* "src/python/grpcio/grpc/_cython/_cygrpc/records.pyx.pxi":127
 *   low = GRPC_COMPRESS_LEVEL_LOW
 *   medium = GRPC_COMPRESS_LEVEL_MED
 *   high = GRPC_COMPRESS_LEVEL_HIGH             # <<<<<<<<<<<<<<
 * 
 * 
 */
  __pyx_t_1 = __Pyx_PyInt_From_grpc_compression_level(GRPC_COMPRESS_LEVEL_HIGH); if (unlikely(!__pyx_t_1)) __PYX_ERR(16, 127, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_1);
  if (__Pyx_SetNameInClass(__pyx_t_3, __pyx_n_s_high, __pyx_t_1) < 0) __PYX_ERR(16, 127, __pyx_L1_error)
  __Pyx_DECREF(__pyx_t_1); __pyx_t_1 = 0;

  /* "src/python/grpcio/grpc/_cython/_cygrpc/records.pyx.pxi":123
 * 
 * 
 * class CompressionLevel:             # <<<<<<<<<<<<<<
 *   none = GRPC_COMPRESS_LEVEL_NONE
 *   low = GRPC_COMPRESS_LEVEL_LOW
 */
  __pyx_t_1 = __Pyx_Py3ClassCreate(((PyObject*)&__Pyx_DefaultClassType), __pyx_n_s_CompressionLevel, __pyx_empty_tuple, __pyx_t_3, NULL, 0, 1); if (unlikely(!__pyx_t_1)) __PYX_ERR(16, 123, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_1);
  if (PyDict_SetItem(__pyx_d, __pyx_n_s_CompressionLevel, __pyx_t_1) < 0) __PYX_ERR(16, 123, __pyx_L1_error)
  __Pyx_DECREF(__pyx_t_1); __pyx_t_1 = 0;
  __Pyx_DECREF(__pyx_t_3); __pyx_t_3 = 0;

  /* "src/python/grpcio/grpc/_cython/_cygrpc/records.pyx.pxi":192
 * 
 * 
 * def compression_algorithm_name(grpc_compression_algorithm algorithm):             # <<<<<<<<<<<<<<
 *   cdef const char* name
 *   with nogil:
 */
  __pyx_t_3 = PyCFunction_NewEx(&__pyx_mdef_4grpc_7_cython_6cygrpc_27compression_algorithm_name, NULL, __pyx_n_s_grpc__cython_cygrpc); if (unlikely(!__pyx_t_3)) __PYX_ERR(16, 192, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_3);
  if (PyDict_SetItem(__pyx_d, __pyx_n_s_compression_algorithm_name, __pyx_t_3) < 0) __PYX_ERR(16, 192, __pyx_L1_error)
  __Pyx_DECREF(__pyx_t_3); __pyx_t_3 = 0;

  /* "src/python/grpcio/grpc/_cython/_cygrpc/security.pyx.pxi":17
 * from libc.string cimport memcpy
 * 
 * import pkgutil             # <<<<<<<<<<<<<<
 * 
 * 
 */
  __pyx_t_3 = __Pyx_Import(__pyx_n_s_pkgutil, 0, -1); if (unlikely(!__pyx_t_3)) __PYX_ERR(17, 17, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_3);
  if (PyDict_SetItem(__pyx_d, __pyx_n_s_pkgutil, __pyx_t_3) < 0) __PYX_ERR(17, 17, __pyx_L1_error)
  __Pyx_DECREF(__pyx_t_3); __pyx_t_3 = 0;

  /* "src/python/grpcio/grpc/_cython/_cygrpc/security.pyx.pxi":36
 * 
 * 
 * def peer_identities(Call call):             # <<<<<<<<<<<<<<
 *   cdef grpc_auth_context* auth_context
 *   cdef grpc_auth_property_iterator properties
 */
  __pyx_t_3 = PyCFunction_NewEx(&__pyx_mdef_4grpc_7_cython_6cygrpc_29peer_identities, NULL, __pyx_n_s_grpc__cython_cygrpc); if (unlikely(!__pyx_t_3)) __PYX_ERR(17, 36, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_3);
  if (PyDict_SetItem(__pyx_d, __pyx_n_s_peer_identities, __pyx_t_3) < 0) __PYX_ERR(17, 36, __pyx_L1_error)
  __Pyx_DECREF(__pyx_t_3); __pyx_t_3 = 0;

  /* "src/python/grpcio/grpc/_cython/_cygrpc/security.pyx.pxi":55
 *   return identities if identities else None
 * 
 * def peer_identity_key(Call call):             # <<<<<<<<<<<<<<
 *   cdef grpc_auth_context* auth_context
 *   cdef const char* c_key
 */
  __pyx_t_3 = PyCFunction_NewEx(&__pyx_mdef_4grpc_7_cython_6cygrpc_31peer_identity_key, NULL, __pyx_n_s_grpc__cython_cygrpc); if (unlikely(!__pyx_t_3)) __PYX_ERR(17, 55, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_3);
  if (PyDict_SetItem(__pyx_d, __pyx_n_s_peer_identity_key, __pyx_t_3) < 0) __PYX_ERR(17, 55, __pyx_L1_error)
  __Pyx_DECREF(__pyx_t_3); __pyx_t_3 = 0;

  /* "src/python/grpcio/grpc/_cython/_cygrpc/security.pyx.pxi":69
 *   return key
 * 
 * def auth_context(Call call):             # <<<<<<<<<<<<<<
 *   cdef grpc_auth_context* auth_context
 *   cdef grpc_auth_property_iterator properties
 */
  __pyx_t_3 = PyCFunction_NewEx(&__pyx_mdef_4grpc_7_cython_6cygrpc_33auth_context, NULL, __pyx_n_s_grpc__cython_cygrpc); if (unlikely(!__pyx_t_3)) __PYX_ERR(17, 69, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_3);
  if (PyDict_SetItem(__pyx_d, __pyx_n_s_auth_context, __pyx_t_3) < 0) __PYX_ERR(17, 69, __pyx_L1_error)
  __Pyx_DECREF(__pyx_t_3); __pyx_t_3 = 0;

  /* "src/python/grpcio/grpc/_cython/_cygrpc/server.pyx.pxi":17
 * cimport cpython
 * 
 * import logging             # <<<<<<<<<<<<<<
 * import time
 * import grpc
 */
  __pyx_t_3 = __Pyx_Import(__pyx_n_s_logging, 0, -1); if (unlikely(!__pyx_t_3)) __PYX_ERR(5, 17, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_3);
  if (PyDict_SetItem(__pyx_d, __pyx_n_s_logging, __pyx_t_3) < 0) __PYX_ERR(5, 17, __pyx_L1_error)
  __Pyx_DECREF(__pyx_t_3); __pyx_t_3 = 0;

  /* "src/python/grpcio/grpc/_cython/_cygrpc/server.pyx.pxi":18
 * 
 * import logging
 * import time             # <<<<<<<<<<<<<<
 * import grpc
 * 
 */
  __pyx_t_3 = __Pyx_Import(__pyx_n_s_time, 0, -1); if (unlikely(!__pyx_t_3)) __PYX_ERR(5, 18, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_3);
  if (PyDict_SetItem(__pyx_d, __pyx_n_s_time, __pyx_t_3) < 0) __PYX_ERR(5, 18, __pyx_L1_error)
  __Pyx_DECREF(__pyx_t_3); __pyx_t_3 = 0;

  /* "src/python/grpcio/grpc/_cython/_cygrpc/server.pyx.pxi":19
 * import logging
 * import time
 * import grpc             # <<<<<<<<<<<<<<
 * 
 * _LOGGER = logging.getLogger(__name__)
 */
  __pyx_t_3 = __Pyx_Import(__pyx_n_s_grpc, 0, -1); if (unlikely(!__pyx_t_3)) __PYX_ERR(5, 19, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_3);
  if (PyDict_SetItem(__pyx_d, __pyx_n_s_grpc, __pyx_t_3) < 0) __PYX_ERR(5, 19, __pyx_L1_error)
  __Pyx_DECREF(__pyx_t_3); __pyx_t_3 = 0;

  /* "src/python/grpcio/grpc/_cython/_cygrpc/server.pyx.pxi":21
 * import grpc
 * 
 * _LOGGER = logging.getLogger(__name__)             # <<<<<<<<<<<<<<
 * 
 * 
 */
  __pyx_t_3 = __Pyx_GetModuleGlobalName(__pyx_n_s_logging); if (unlikely(!__pyx_t_3)) __PYX_ERR(5, 21, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_3);
  __pyx_t_1 = __Pyx_PyObject_GetAttrStr(__pyx_t_3, __pyx_n_s_getLogger); if (unlikely(!__pyx_t_1)) __PYX_ERR(5, 21, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_1);
  __Pyx_DECREF(__pyx_t_3); __pyx_t_3 = 0;
  __pyx_t_3 = __Pyx_GetModuleGlobalName(__pyx_n_s_name_2); if (unlikely(!__pyx_t_3)) __PYX_ERR(5, 21, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_3);
  __pyx_t_2 = __Pyx_PyObject_CallOneArg(__pyx_t_1, __pyx_t_3); if (unlikely(!__pyx_t_2)) __PYX_ERR(5, 21, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_2);
  __Pyx_DECREF(__pyx_t_1); __pyx_t_1 = 0;
  __Pyx_DECREF(__pyx_t_3); __pyx_t_3 = 0;
  if (PyDict_SetItem(__pyx_d, __pyx_n_s_LOGGER, __pyx_t_2) < 0) __PYX_ERR(5, 21, __pyx_L1_error)
  __Pyx_DECREF(__pyx_t_2); __pyx_t_2 = 0;

  /* "src/python/grpcio/grpc/_cython/_cygrpc/_hooks.pyx.pxi":19
 *   raise NotImplementedError("No custom hooks are implemented")
 * 
 * def install_context_from_request_call_event(RequestCallEvent event):             # <<<<<<<<<<<<<<
 *   pass
 * 
 */
  __pyx_t_2 = PyCFunction_NewEx(&__pyx_mdef_4grpc_7_cython_6cygrpc_35install_context_from_request_call_event, NULL, __pyx_n_s_grpc__cython_cygrpc); if (unlikely(!__pyx_t_2)) __PYX_ERR(20, 19, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_2);
  if (PyDict_SetItem(__pyx_d, __pyx_n_s_install_context_from_request_cal, __pyx_t_2) < 0) __PYX_ERR(20, 19, __pyx_L1_error)
  __Pyx_DECREF(__pyx_t_2); __pyx_t_2 = 0;

  /* "src/python/grpcio/grpc/_cython/_cygrpc/_hooks.pyx.pxi":22
 *   pass
 * 
 * def uninstall_context():             # <<<<<<<<<<<<<<
 *   pass
 * 
 */
  __pyx_t_2 = PyCFunction_NewEx(&__pyx_mdef_4grpc_7_cython_6cygrpc_37uninstall_context, NULL, __pyx_n_s_grpc__cython_cygrpc); if (unlikely(!__pyx_t_2)) __PYX_ERR(20, 22, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_2);
  if (PyDict_SetItem(__pyx_d, __pyx_n_s_uninstall_context, __pyx_t_2) < 0) __PYX_ERR(20, 22, __pyx_L1_error)
  __Pyx_DECREF(__pyx_t_2); __pyx_t_2 = 0;

  /* "src/python/grpcio/grpc/_cython/_cygrpc/_hooks.pyx.pxi":25
 *   pass
 * 
 * def build_census_context():             # <<<<<<<<<<<<<<
 *   pass
 * 
 */
  __pyx_t_2 = PyCFunction_NewEx(&__pyx_mdef_4grpc_7_cython_6cygrpc_39build_census_context, NULL, __pyx_n_s_grpc__cython_cygrpc); if (unlikely(!__pyx_t_2)) __PYX_ERR(20, 25, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_2);
  if (PyDict_SetItem(__pyx_d, __pyx_n_s_build_census_context, __pyx_t_2) < 0) __PYX_ERR(20, 25, __pyx_L1_error)
  __Pyx_DECREF(__pyx_t_2); __pyx_t_2 = 0;

  /* "src/python/grpcio/grpc/_cython/_cygrpc/_hooks.pyx.pxi":31
 *   pass
 * 
 * def set_census_context_on_call(_CallState call_state, CensusContext census_ctx):             # <<<<<<<<<<<<<<
 *   pass
 * 
 */
  __pyx_t_2 = PyCFunction_NewEx(&__pyx_mdef_4grpc_7_cython_6cygrpc_41set_census_context_on_call, NULL, __pyx_n_s_grpc__cython_cygrpc); if (unlikely(!__pyx_t_2)) __PYX_ERR(20, 31, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_2);
  if (PyDict_SetItem(__pyx_d, __pyx_n_s_set_census_context_on_call, __pyx_t_2) < 0) __PYX_ERR(20, 31, __pyx_L1_error)
  __Pyx_DECREF(__pyx_t_2); __pyx_t_2 = 0;

  /* "src/python/grpcio/grpc/_cython/_cygrpc/_hooks.pyx.pxi":34
 *   pass
 * 
 * def get_deadline_from_context():             # <<<<<<<<<<<<<<
 *   return None
 */
  __pyx_t_2 = PyCFunction_NewEx(&__pyx_mdef_4grpc_7_cython_6cygrpc_43get_deadline_from_context, NULL, __pyx_n_s_grpc__cython_cygrpc); if (unlikely(!__pyx_t_2)) __PYX_ERR(20, 34, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_2);
  if (PyDict_SetItem(__pyx_d, __pyx_n_s_get_deadline_from_context, __pyx_t_2) < 0) __PYX_ERR(20, 34, __pyx_L1_error)
  __Pyx_DECREF(__pyx_t_2); __pyx_t_2 = 0;

  /* "src/python/grpcio/grpc/_cython/_cygrpc/grpc_gevent.pyx.pxi":19
 * from libc cimport string
 * from libc.stdlib cimport malloc, free
 * import errno             # <<<<<<<<<<<<<<
 * gevent_g = None
 * gevent_socket = None
 */
  __pyx_t_2 = __Pyx_Import(__pyx_n_s_errno, 0, -1); if (unlikely(!__pyx_t_2)) __PYX_ERR(6, 19, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_2);
  if (PyDict_SetItem(__pyx_d, __pyx_n_s_errno, __pyx_t_2) < 0) __PYX_ERR(6, 19, __pyx_L1_error)
  __Pyx_DECREF(__pyx_t_2); __pyx_t_2 = 0;

  /* "src/python/grpcio/grpc/_cython/_cygrpc/grpc_gevent.pyx.pxi":20
 * from libc.stdlib cimport malloc, free
 * import errno
 * gevent_g = None             # <<<<<<<<<<<<<<
 * gevent_socket = None
 * gevent_hub = None
 */
  if (PyDict_SetItem(__pyx_d, __pyx_n_s_gevent_g, Py_None) < 0) __PYX_ERR(6, 20, __pyx_L1_error)

  /* "src/python/grpcio/grpc/_cython/_cygrpc/grpc_gevent.pyx.pxi":21
 * import errno
 * gevent_g = None
 * gevent_socket = None             # <<<<<<<<<<<<<<
 * gevent_hub = None
 * gevent_event = None
 */
  if (PyDict_SetItem(__pyx_d, __pyx_n_s_gevent_socket, Py_None) < 0) __PYX_ERR(6, 21, __pyx_L1_error)

  /* "src/python/grpcio/grpc/_cython/_cygrpc/grpc_gevent.pyx.pxi":22
 * gevent_g = None
 * gevent_socket = None
 * gevent_hub = None             # <<<<<<<<<<<<<<
 * gevent_event = None
 * g_event = None
 */
  if (PyDict_SetItem(__pyx_d, __pyx_n_s_gevent_hub, Py_None) < 0) __PYX_ERR(6, 22, __pyx_L1_error)

  /* "src/python/grpcio/grpc/_cython/_cygrpc/grpc_gevent.pyx.pxi":23
 * gevent_socket = None
 * gevent_hub = None
 * gevent_event = None             # <<<<<<<<<<<<<<
 * g_event = None
 * g_pool = None
 */
  if (PyDict_SetItem(__pyx_d, __pyx_n_s_gevent_event, Py_None) < 0) __PYX_ERR(6, 23, __pyx_L1_error)

  /* "src/python/grpcio/grpc/_cython/_cygrpc/grpc_gevent.pyx.pxi":24
 * gevent_hub = None
 * gevent_event = None
 * g_event = None             # <<<<<<<<<<<<<<
 * g_pool = None
 * 
 */
  if (PyDict_SetItem(__pyx_d, __pyx_n_s_g_event, Py_None) < 0) __PYX_ERR(6, 24, __pyx_L1_error)

  /* "src/python/grpcio/grpc/_cython/_cygrpc/grpc_gevent.pyx.pxi":25
 * gevent_event = None
 * g_event = None
 * g_pool = None             # <<<<<<<<<<<<<<
 * 
 * cdef grpc_error* grpc_error_none():
 */
  if (PyDict_SetItem(__pyx_d, __pyx_n_s_g_pool, Py_None) < 0) __PYX_ERR(6, 25, __pyx_L1_error)

  /* "src/python/grpcio/grpc/_cython/_cygrpc/grpc_gevent.pyx.pxi":72
 *   return addresses
 * 
 * def _spawn_greenlet(*args):             # <<<<<<<<<<<<<<
 *   greenlet = g_pool.spawn(*args)
 * 
 */
  __pyx_t_2 = PyCFunction_NewEx(&__pyx_mdef_4grpc_7_cython_6cygrpc_45_spawn_greenlet, NULL, __pyx_n_s_grpc__cython_cygrpc); if (unlikely(!__pyx_t_2)) __PYX_ERR(6, 72, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_2);
  if (PyDict_SetItem(__pyx_d, __pyx_n_s_spawn_greenlet, __pyx_t_2) < 0) __PYX_ERR(6, 72, __pyx_L1_error)
  __Pyx_DECREF(__pyx_t_2); __pyx_t_2 = 0;

  /* "src/python/grpcio/grpc/_cython/_cygrpc/grpc_gevent.pyx.pxi":110
 *   g_event.set()
 * 
 * def socket_connect_async(socket_wrapper, addr_tuple):             # <<<<<<<<<<<<<<
 *   socket_connect_async_cython(socket_wrapper, addr_tuple)
 * 
 */
  __pyx_t_2 = PyCFunction_NewEx(&__pyx_mdef_4grpc_7_cython_6cygrpc_47socket_connect_async, NULL, __pyx_n_s_grpc__cython_cygrpc); if (unlikely(!__pyx_t_2)) __PYX_ERR(6, 110, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_2);
  if (PyDict_SetItem(__pyx_d, __pyx_n_s_socket_connect_async, __pyx_t_2) < 0) __PYX_ERR(6, 110, __pyx_L1_error)
  __Pyx_DECREF(__pyx_t_2); __pyx_t_2 = 0;

  /* "src/python/grpcio/grpc/_cython/_cygrpc/grpc_gevent.pyx.pxi":150
 *   socket_wrapper.close_cb(socket)
 * 
 * def socket_sendmsg(socket, write_bytes):             # <<<<<<<<<<<<<<
 *   try:
 *     return socket.sendmsg(write_bytes)
 */
  __pyx_t_2 = PyCFunction_NewEx(&__pyx_mdef_4grpc_7_cython_6cygrpc_49socket_sendmsg, NULL, __pyx_n_s_grpc__cython_cygrpc); if (unlikely(!__pyx_t_2)) __PYX_ERR(6, 150, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_2);
  if (PyDict_SetItem(__pyx_d, __pyx_n_s_socket_sendmsg, __pyx_t_2) < 0) __PYX_ERR(6, 150, __pyx_L1_error)
  __Pyx_DECREF(__pyx_t_2); __pyx_t_2 = 0;

  /* "src/python/grpcio/grpc/_cython/_cygrpc/grpc_gevent.pyx.pxi":175
 *   g_event.set()
 * 
 * def socket_write_async(socket_wrapper, write_bytes):             # <<<<<<<<<<<<<<
 *   socket_write_async_cython(socket_wrapper, write_bytes)
 * 
 */
  __pyx_t_2 = PyCFunction_NewEx(&__pyx_mdef_4grpc_7_cython_6cygrpc_51socket_write_async, NULL, __pyx_n_s_grpc__cython_cygrpc); if (unlikely(!__pyx_t_2)) __PYX_ERR(6, 175, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_2);
  if (PyDict_SetItem(__pyx_d, __pyx_n_s_socket_write_async, __pyx_t_2) < 0) __PYX_ERR(6, 175, __pyx_L1_error)
  __Pyx_DECREF(__pyx_t_2); __pyx_t_2 = 0;

  /* "src/python/grpcio/grpc/_cython/_cygrpc/grpc_gevent.pyx.pxi":203
 *   g_event.set()
 * 
 * def socket_read_async(socket_wrapper):             # <<<<<<<<<<<<<<
 *   socket_read_async_cython(socket_wrapper)
 * 
 */
  __pyx_t_2 = PyCFunction_NewEx(&__pyx_mdef_4grpc_7_cython_6cygrpc_53socket_read_async, NULL, __pyx_n_s_grpc__cython_cygrpc); if (unlikely(!__pyx_t_2)) __PYX_ERR(6, 203, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_2);
  if (PyDict_SetItem(__pyx_d, __pyx_n_s_socket_read_async, __pyx_t_2) < 0) __PYX_ERR(6, 203, __pyx_L1_error)
  __Pyx_DECREF(__pyx_t_2); __pyx_t_2 = 0;

  /* "src/python/grpcio/grpc/_cython/_cygrpc/grpc_gevent.pyx.pxi":242
 *   return grpc_error_none()
 * 
 * def applysockopts(s):             # <<<<<<<<<<<<<<
 *   s.setsockopt(gevent_socket.SOL_SOCKET, gevent_socket.SO_REUSEADDR, 1)
 *   s.setsockopt(gevent_socket.IPPROTO_TCP, gevent_socket.TCP_NODELAY, True)
 */
  __pyx_t_2 = PyCFunction_NewEx(&__pyx_mdef_4grpc_7_cython_6cygrpc_55applysockopts, NULL, __pyx_n_s_grpc__cython_cygrpc); if (unlikely(!__pyx_t_2)) __PYX_ERR(6, 242, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_2);
  if (PyDict_SetItem(__pyx_d, __pyx_n_s_applysockopts, __pyx_t_2) < 0) __PYX_ERR(6, 242, __pyx_L1_error)
  __Pyx_DECREF(__pyx_t_2); __pyx_t_2 = 0;

  /* "src/python/grpcio/grpc/_cython/_cygrpc/grpc_gevent.pyx.pxi":291
 *    g_event.set()
 * 
 * def socket_accept_async(s):             # <<<<<<<<<<<<<<
 *   accept_callback_cython(s)
 * 
 */
  __pyx_t_2 = PyCFunction_NewEx(&__pyx_mdef_4grpc_7_cython_6cygrpc_57socket_accept_async, NULL, __pyx_n_s_grpc__cython_cygrpc); if (unlikely(!__pyx_t_2)) __PYX_ERR(6, 291, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_2);
  if (PyDict_SetItem(__pyx_d, __pyx_n_s_socket_accept_async, __pyx_t_2) < 0) __PYX_ERR(6, 291, __pyx_L1_error)
  __Pyx_DECREF(__pyx_t_2); __pyx_t_2 = 0;

  /* "src/python/grpcio/grpc/_cython/_cygrpc/grpc_gevent.pyx.pxi":322
 *   g_event.set()
 * 
 * def socket_resolve_async_python(resolve_wrapper):             # <<<<<<<<<<<<<<
 *   socket_resolve_async_cython(resolve_wrapper)
 * 
 */
  __pyx_t_2 = PyCFunction_NewEx(&__pyx_mdef_4grpc_7_cython_6cygrpc_59socket_resolve_async_python, NULL, __pyx_n_s_grpc__cython_cygrpc); if (unlikely(!__pyx_t_2)) __PYX_ERR(6, 322, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_2);
  if (PyDict_SetItem(__pyx_d, __pyx_n_s_socket_resolve_async_python, __pyx_t_2) < 0) __PYX_ERR(6, 322, __pyx_L1_error)
  __Pyx_DECREF(__pyx_t_2); __pyx_t_2 = 0;

  /* "src/python/grpcio/grpc/_cython/_cygrpc/grpc_gevent.pyx.pxi":401
 * cdef grpc_custom_poller_vtable gevent_pollset_vtable
 * 
 * def init_grpc_gevent():             # <<<<<<<<<<<<<<
 *   # Lazily import gevent
 *   global gevent_socket
 */
  __pyx_t_2 = PyCFunction_NewEx(&__pyx_mdef_4grpc_7_cython_6cygrpc_61init_grpc_gevent, NULL, __pyx_n_s_grpc__cython_cygrpc); if (unlikely(!__pyx_t_2)) __PYX_ERR(6, 401, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_2);
  if (PyDict_SetItem(__pyx_d, __pyx_n_s_init_grpc_gevent, __pyx_t_2) < 0) __PYX_ERR(6, 401, __pyx_L1_error)
  __Pyx_DECREF(__pyx_t_2); __pyx_t_2 = 0;

  /* "src/python/grpcio/grpc/_cython/_cygrpc/fork_posix.pyx.pxi":15
 * # limitations under the License.
 * 
 * import logging             # <<<<<<<<<<<<<<
 * import os
 * import threading
 */
  __pyx_t_2 = __Pyx_Import(__pyx_n_s_logging, 0, -1); if (unlikely(!__pyx_t_2)) __PYX_ERR(0, 15, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_2);
  if (PyDict_SetItem(__pyx_d, __pyx_n_s_logging, __pyx_t_2) < 0) __PYX_ERR(0, 15, __pyx_L1_error)
  __Pyx_DECREF(__pyx_t_2); __pyx_t_2 = 0;

  /* "src/python/grpcio/grpc/_cython/_cygrpc/fork_posix.pyx.pxi":16
 * 
 * import logging
 * import os             # <<<<<<<<<<<<<<
 * import threading
 * 
 */
  __pyx_t_2 = __Pyx_Import(__pyx_n_s_os, 0, -1); if (unlikely(!__pyx_t_2)) __PYX_ERR(0, 16, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_2);
  if (PyDict_SetItem(__pyx_d, __pyx_n_s_os, __pyx_t_2) < 0) __PYX_ERR(0, 16, __pyx_L1_error)
  __Pyx_DECREF(__pyx_t_2); __pyx_t_2 = 0;

  /* "src/python/grpcio/grpc/_cython/_cygrpc/fork_posix.pyx.pxi":17
 * import logging
 * import os
 * import threading             # <<<<<<<<<<<<<<
 * 
 * _LOGGER = logging.getLogger(__name__)
 */
  __pyx_t_2 = __Pyx_Import(__pyx_n_s_threading, 0, -1); if (unlikely(!__pyx_t_2)) __PYX_ERR(0, 17, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_2);
  if (PyDict_SetItem(__pyx_d, __pyx_n_s_threading, __pyx_t_2) < 0) __PYX_ERR(0, 17, __pyx_L1_error)
  __Pyx_DECREF(__pyx_t_2); __pyx_t_2 = 0;

  /* "src/python/grpcio/grpc/_cython/_cygrpc/fork_posix.pyx.pxi":19
 * import threading
 * 
 * _LOGGER = logging.getLogger(__name__)             # <<<<<<<<<<<<<<
 * 
 * _AWAIT_THREADS_TIMEOUT_SECONDS = 5
 */
  __pyx_t_2 = __Pyx_GetModuleGlobalName(__pyx_n_s_logging); if (unlikely(!__pyx_t_2)) __PYX_ERR(0, 19, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_2);
  __pyx_t_3 = __Pyx_PyObject_GetAttrStr(__pyx_t_2, __pyx_n_s_getLogger); if (unlikely(!__pyx_t_3)) __PYX_ERR(0, 19, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_3);
  __Pyx_DECREF(__pyx_t_2); __pyx_t_2 = 0;
  __pyx_t_2 = __Pyx_GetModuleGlobalName(__pyx_n_s_name_2); if (unlikely(!__pyx_t_2)) __PYX_ERR(0, 19, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_2);
  __pyx_t_1 = __Pyx_PyObject_CallOneArg(__pyx_t_3, __pyx_t_2); if (unlikely(!__pyx_t_1)) __PYX_ERR(0, 19, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_1);
  __Pyx_DECREF(__pyx_t_3); __pyx_t_3 = 0;
  __Pyx_DECREF(__pyx_t_2); __pyx_t_2 = 0;
  if (PyDict_SetItem(__pyx_d, __pyx_n_s_LOGGER, __pyx_t_1) < 0) __PYX_ERR(0, 19, __pyx_L1_error)
  __Pyx_DECREF(__pyx_t_1); __pyx_t_1 = 0;

  /* "src/python/grpcio/grpc/_cython/_cygrpc/fork_posix.pyx.pxi":21
 * _LOGGER = logging.getLogger(__name__)
 * 
 * _AWAIT_THREADS_TIMEOUT_SECONDS = 5             # <<<<<<<<<<<<<<
 * 
 * _TRUE_VALUES = ['yes',  'Yes',  'YES', 'true', 'True', 'TRUE', '1']
 */
  if (PyDict_SetItem(__pyx_d, __pyx_n_s_AWAIT_THREADS_TIMEOUT_SECONDS, __pyx_int_5) < 0) __PYX_ERR(0, 21, __pyx_L1_error)

  /* "src/python/grpcio/grpc/_cython/_cygrpc/fork_posix.pyx.pxi":23
 * _AWAIT_THREADS_TIMEOUT_SECONDS = 5
 * 
 * _TRUE_VALUES = ['yes',  'Yes',  'YES', 'true', 'True', 'TRUE', '1']             # <<<<<<<<<<<<<<
 * 
 * # This flag enables experimental support within gRPC Python for applications
 */
  __pyx_t_1 = PyList_New(7); if (unlikely(!__pyx_t_1)) __PYX_ERR(0, 23, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_1);
  __Pyx_INCREF(__pyx_n_s_yes);
  __Pyx_GIVEREF(__pyx_n_s_yes);
  PyList_SET_ITEM(__pyx_t_1, 0, __pyx_n_s_yes);
  __Pyx_INCREF(__pyx_n_s_Yes);
  __Pyx_GIVEREF(__pyx_n_s_Yes);
  PyList_SET_ITEM(__pyx_t_1, 1, __pyx_n_s_Yes);
  __Pyx_INCREF(__pyx_n_s_YES);
  __Pyx_GIVEREF(__pyx_n_s_YES);
  PyList_SET_ITEM(__pyx_t_1, 2, __pyx_n_s_YES);
  __Pyx_INCREF(__pyx_n_s_true);
  __Pyx_GIVEREF(__pyx_n_s_true);
  PyList_SET_ITEM(__pyx_t_1, 3, __pyx_n_s_true);
  __Pyx_INCREF(__pyx_n_s_True);
  __Pyx_GIVEREF(__pyx_n_s_True);
  PyList_SET_ITEM(__pyx_t_1, 4, __pyx_n_s_True);
  __Pyx_INCREF(__pyx_n_s_TRUE);
  __Pyx_GIVEREF(__pyx_n_s_TRUE);
  PyList_SET_ITEM(__pyx_t_1, 5, __pyx_n_s_TRUE);
  __Pyx_INCREF(__pyx_kp_s_1);
  __Pyx_GIVEREF(__pyx_kp_s_1);
  PyList_SET_ITEM(__pyx_t_1, 6, __pyx_kp_s_1);
  if (PyDict_SetItem(__pyx_d, __pyx_n_s_TRUE_VALUES, __pyx_t_1) < 0) __PYX_ERR(0, 23, __pyx_L1_error)
  __Pyx_DECREF(__pyx_t_1); __pyx_t_1 = 0;

  /* "src/python/grpcio/grpc/_cython/_cygrpc/fork_posix.pyx.pxi":36
 * # This flag is not supported on Windows.
 * _GRPC_ENABLE_FORK_SUPPORT = (
 *     os.environ.get('GRPC_ENABLE_FORK_SUPPORT', '0')             # <<<<<<<<<<<<<<
 *         .lower() in _TRUE_VALUES)
 * 
 */
  __pyx_t_1 = __Pyx_GetModuleGlobalName(__pyx_n_s_os); if (unlikely(!__pyx_t_1)) __PYX_ERR(0, 36, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_1);
  __pyx_t_2 = __Pyx_PyObject_GetAttrStr(__pyx_t_1, __pyx_n_s_environ); if (unlikely(!__pyx_t_2)) __PYX_ERR(0, 36, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_2);
  __Pyx_DECREF(__pyx_t_1); __pyx_t_1 = 0;
  __pyx_t_1 = __Pyx_PyObject_GetAttrStr(__pyx_t_2, __pyx_n_s_get); if (unlikely(!__pyx_t_1)) __PYX_ERR(0, 36, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_1);
  __Pyx_DECREF(__pyx_t_2); __pyx_t_2 = 0;
  __pyx_t_2 = __Pyx_PyObject_Call(__pyx_t_1, __pyx_tuple__224, NULL); if (unlikely(!__pyx_t_2)) __PYX_ERR(0, 36, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_2);
  __Pyx_DECREF(__pyx_t_1); __pyx_t_1 = 0;

  /* "src/python/grpcio/grpc/_cython/_cygrpc/fork_posix.pyx.pxi":37
 * _GRPC_ENABLE_FORK_SUPPORT = (
 *     os.environ.get('GRPC_ENABLE_FORK_SUPPORT', '0')
 *         .lower() in _TRUE_VALUES)             # <<<<<<<<<<<<<<
 * 
 * _fork_handler_failed = False
 */
  __pyx_t_1 = __Pyx_PyObject_GetAttrStr(__pyx_t_2, __pyx_n_s_lower); if (unlikely(!__pyx_t_1)) __PYX_ERR(0, 37, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_1);
  __Pyx_DECREF(__pyx_t_2); __pyx_t_2 = 0;
  __pyx_t_2 = __Pyx_PyObject_CallNoArg(__pyx_t_1); if (unlikely(!__pyx_t_2)) __PYX_ERR(0, 37, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_2);
  __Pyx_DECREF(__pyx_t_1); __pyx_t_1 = 0;
  __pyx_t_1 = __Pyx_GetModuleGlobalName(__pyx_n_s_TRUE_VALUES); if (unlikely(!__pyx_t_1)) __PYX_ERR(0, 37, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_1);
  __pyx_t_4 = (__Pyx_PySequence_ContainsTF(__pyx_t_2, __pyx_t_1, Py_EQ)); if (unlikely(__pyx_t_4 < 0)) __PYX_ERR(0, 37, __pyx_L1_error)
  __Pyx_DECREF(__pyx_t_2); __pyx_t_2 = 0;
  __Pyx_DECREF(__pyx_t_1); __pyx_t_1 = 0;
  __pyx_t_1 = __Pyx_PyBool_FromLong(__pyx_t_4); if (unlikely(!__pyx_t_1)) __PYX_ERR(0, 37, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_1);
  if (PyDict_SetItem(__pyx_d, __pyx_n_s_GRPC_ENABLE_FORK_SUPPORT, __pyx_t_1) < 0) __PYX_ERR(0, 35, __pyx_L1_error)
  __Pyx_DECREF(__pyx_t_1); __pyx_t_1 = 0;

  /* "src/python/grpcio/grpc/_cython/_cygrpc/fork_posix.pyx.pxi":39
 *         .lower() in _TRUE_VALUES)
 * 
 * _fork_handler_failed = False             # <<<<<<<<<<<<<<
 * 
 * cdef void __prefork() nogil:
 */
  if (PyDict_SetItem(__pyx_d, __pyx_n_s_fork_handler_failed, Py_False) < 0) __PYX_ERR(0, 39, __pyx_L1_error)

  /* "src/python/grpcio/grpc/_cython/_cygrpc/fork_posix.pyx.pxi":92
 * 
 * 
 * def fork_handlers_and_grpc_init():             # <<<<<<<<<<<<<<
 *     grpc_init()
 *     if _GRPC_ENABLE_FORK_SUPPORT:
 */
  __pyx_t_1 = PyCFunction_NewEx(&__pyx_mdef_4grpc_7_cython_6cygrpc_63fork_handlers_and_grpc_init, NULL, __pyx_n_s_grpc__cython_cygrpc); if (unlikely(!__pyx_t_1)) __PYX_ERR(0, 92, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_1);
  if (PyDict_SetItem(__pyx_d, __pyx_n_s_fork_handlers_and_grpc_init, __pyx_t_1) < 0) __PYX_ERR(0, 92, __pyx_L1_error)
  __Pyx_DECREF(__pyx_t_1); __pyx_t_1 = 0;

  /* "src/python/grpcio/grpc/_cython/_cygrpc/fork_posix.pyx.pxi":101
 * 
 * 
 * class ForkManagedThread(object):             # <<<<<<<<<<<<<<
 *     def __init__(self, target, args=()):
 *         if _GRPC_ENABLE_FORK_SUPPORT:
 */
  __pyx_t_1 = __Pyx_CalculateMetaclass(NULL, __pyx_tuple__226); if (unlikely(!__pyx_t_1)) __PYX_ERR(0, 101, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_1);
  __pyx_t_2 = __Pyx_Py3MetaclassPrepare(__pyx_t_1, __pyx_tuple__226, __pyx_n_s_ForkManagedThread, __pyx_n_s_ForkManagedThread, (PyObject *) NULL, __pyx_n_s_grpc__cython_cygrpc, (PyObject *) NULL); if (unlikely(!__pyx_t_2)) __PYX_ERR(0, 101, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_2);

  /* "src/python/grpcio/grpc/_cython/_cygrpc/fork_posix.pyx.pxi":102
 * 
 * class ForkManagedThread(object):
 *     def __init__(self, target, args=()):             # <<<<<<<<<<<<<<
 *         if _GRPC_ENABLE_FORK_SUPPORT:
 *             def managed_target(*args):
 */
  __pyx_t_3 = __Pyx_CyFunction_NewEx(&__pyx_mdef_4grpc_7_cython_6cygrpc_17ForkManagedThread_1__init__, 0, __pyx_n_s_ForkManagedThread___init, NULL, __pyx_n_s_grpc__cython_cygrpc, __pyx_d, ((PyObject *)__pyx_codeobj__228)); if (unlikely(!__pyx_t_3)) __PYX_ERR(0, 102, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_3);
  __Pyx_CyFunction_SetDefaultsTuple(__pyx_t_3, __pyx_tuple__229);
  if (__Pyx_SetNameInClass(__pyx_t_2, __pyx_n_s_init, __pyx_t_3) < 0) __PYX_ERR(0, 102, __pyx_L1_error)
  __Pyx_DECREF(__pyx_t_3); __pyx_t_3 = 0;

  /* "src/python/grpcio/grpc/_cython/_cygrpc/fork_posix.pyx.pxi":113
 *             self._thread = threading.Thread(target=target, args=args)
 * 
 *     def setDaemon(self, daemonic):             # <<<<<<<<<<<<<<
 *         self._thread.daemon = daemonic
 * 
 */
  __pyx_t_3 = __Pyx_CyFunction_NewEx(&__pyx_mdef_4grpc_7_cython_6cygrpc_17ForkManagedThread_3setDaemon, 0, __pyx_n_s_ForkManagedThread_setDaemon, NULL, __pyx_n_s_grpc__cython_cygrpc, __pyx_d, ((PyObject *)__pyx_codeobj__231)); if (unlikely(!__pyx_t_3)) __PYX_ERR(0, 113, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_3);
  if (__Pyx_SetNameInClass(__pyx_t_2, __pyx_n_s_setDaemon, __pyx_t_3) < 0) __PYX_ERR(0, 113, __pyx_L1_error)
  __Pyx_DECREF(__pyx_t_3); __pyx_t_3 = 0;

  /* "src/python/grpcio/grpc/_cython/_cygrpc/fork_posix.pyx.pxi":116
 *         self._thread.daemon = daemonic
 * 
 *     def start(self):             # <<<<<<<<<<<<<<
 *         if _GRPC_ENABLE_FORK_SUPPORT:
 *             _fork_state.active_thread_count.increment()
 */
  __pyx_t_3 = __Pyx_CyFunction_NewEx(&__pyx_mdef_4grpc_7_cython_6cygrpc_17ForkManagedThread_5start, 0, __pyx_n_s_ForkManagedThread_start, NULL, __pyx_n_s_grpc__cython_cygrpc, __pyx_d, ((PyObject *)__pyx_codeobj__233)); if (unlikely(!__pyx_t_3)) __PYX_ERR(0, 116, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_3);
  if (__Pyx_SetNameInClass(__pyx_t_2, __pyx_n_s_start, __pyx_t_3) < 0) __PYX_ERR(0, 116, __pyx_L1_error)
  __Pyx_DECREF(__pyx_t_3); __pyx_t_3 = 0;

  /* "src/python/grpcio/grpc/_cython/_cygrpc/fork_posix.pyx.pxi":121
 *         self._thread.start()
 * 
 *     def join(self):             # <<<<<<<<<<<<<<
 *         self._thread.join()
 * 
 */
  __pyx_t_3 = __Pyx_CyFunction_NewEx(&__pyx_mdef_4grpc_7_cython_6cygrpc_17ForkManagedThread_7join, 0, __pyx_n_s_ForkManagedThread_join, NULL, __pyx_n_s_grpc__cython_cygrpc, __pyx_d, ((PyObject *)__pyx_codeobj__235)); if (unlikely(!__pyx_t_3)) __PYX_ERR(0, 121, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_3);
  if (__Pyx_SetNameInClass(__pyx_t_2, __pyx_n_s_join, __pyx_t_3) < 0) __PYX_ERR(0, 121, __pyx_L1_error)
  __Pyx_DECREF(__pyx_t_3); __pyx_t_3 = 0;

  /* "src/python/grpcio/grpc/_cython/_cygrpc/fork_posix.pyx.pxi":101
 * 
 * 
 * class ForkManagedThread(object):             # <<<<<<<<<<<<<<
 *     def __init__(self, target, args=()):
 *         if _GRPC_ENABLE_FORK_SUPPORT:
 */
  __pyx_t_3 = __Pyx_Py3ClassCreate(__pyx_t_1, __pyx_n_s_ForkManagedThread, __pyx_tuple__226, __pyx_t_2, NULL, 0, 1); if (unlikely(!__pyx_t_3)) __PYX_ERR(0, 101, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_3);
  if (PyDict_SetItem(__pyx_d, __pyx_n_s_ForkManagedThread, __pyx_t_3) < 0) __PYX_ERR(0, 101, __pyx_L1_error)
  __Pyx_DECREF(__pyx_t_3); __pyx_t_3 = 0;
  __Pyx_DECREF(__pyx_t_2); __pyx_t_2 = 0;
  __Pyx_DECREF(__pyx_t_1); __pyx_t_1 = 0;

  /* "src/python/grpcio/grpc/_cython/_cygrpc/fork_posix.pyx.pxi":125
 * 
 * 
 * def block_if_fork_in_progress(postfork_state_to_reset=None):             # <<<<<<<<<<<<<<
 *     if _GRPC_ENABLE_FORK_SUPPORT:
 *         with _fork_state.fork_in_progress_condition:
 */
  __pyx_t_1 = PyCFunction_NewEx(&__pyx_mdef_4grpc_7_cython_6cygrpc_65block_if_fork_in_progress, NULL, __pyx_n_s_grpc__cython_cygrpc); if (unlikely(!__pyx_t_1)) __PYX_ERR(0, 125, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_1);
  if (PyDict_SetItem(__pyx_d, __pyx_n_s_block_if_fork_in_progress, __pyx_t_1) < 0) __PYX_ERR(0, 125, __pyx_L1_error)
  __Pyx_DECREF(__pyx_t_1); __pyx_t_1 = 0;

  /* "src/python/grpcio/grpc/_cython/_cygrpc/fork_posix.pyx.pxi":137
 * 
 * 
 * def enter_user_request_generator():             # <<<<<<<<<<<<<<
 *     if _GRPC_ENABLE_FORK_SUPPORT:
 *         _fork_state.active_thread_count.decrement()
 */
  __pyx_t_1 = PyCFunction_NewEx(&__pyx_mdef_4grpc_7_cython_6cygrpc_67enter_user_request_generator, NULL, __pyx_n_s_grpc__cython_cygrpc); if (unlikely(!__pyx_t_1)) __PYX_ERR(0, 137, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_1);
  if (PyDict_SetItem(__pyx_d, __pyx_n_s_enter_user_request_generator, __pyx_t_1) < 0) __PYX_ERR(0, 137, __pyx_L1_error)
  __Pyx_DECREF(__pyx_t_1); __pyx_t_1 = 0;

  /* "src/python/grpcio/grpc/_cython/_cygrpc/fork_posix.pyx.pxi":142
 * 
 * 
 * def return_from_user_request_generator():             # <<<<<<<<<<<<<<
 *     if _GRPC_ENABLE_FORK_SUPPORT:
 *         _fork_state.active_thread_count.increment()
 */
  __pyx_t_1 = PyCFunction_NewEx(&__pyx_mdef_4grpc_7_cython_6cygrpc_69return_from_user_request_generator, NULL, __pyx_n_s_grpc__cython_cygrpc); if (unlikely(!__pyx_t_1)) __PYX_ERR(0, 142, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_1);
  if (PyDict_SetItem(__pyx_d, __pyx_n_s_return_from_user_request_generat, __pyx_t_1) < 0) __PYX_ERR(0, 142, __pyx_L1_error)
  __Pyx_DECREF(__pyx_t_1); __pyx_t_1 = 0;

  /* "src/python/grpcio/grpc/_cython/_cygrpc/fork_posix.pyx.pxi":148
 * 
 * 
 * def get_fork_epoch():             # <<<<<<<<<<<<<<
 *     return _fork_state.fork_epoch
 * 
 */
  __pyx_t_1 = PyCFunction_NewEx(&__pyx_mdef_4grpc_7_cython_6cygrpc_71get_fork_epoch, NULL, __pyx_n_s_grpc__cython_cygrpc); if (unlikely(!__pyx_t_1)) __PYX_ERR(0, 148, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_1);
  if (PyDict_SetItem(__pyx_d, __pyx_n_s_get_fork_epoch, __pyx_t_1) < 0) __PYX_ERR(0, 148, __pyx_L1_error)
  __Pyx_DECREF(__pyx_t_1); __pyx_t_1 = 0;

  /* "src/python/grpcio/grpc/_cython/_cygrpc/fork_posix.pyx.pxi":152
 * 
 * 
 * def is_fork_support_enabled():             # <<<<<<<<<<<<<<
 *     return _GRPC_ENABLE_FORK_SUPPORT
 * 
 */
  __pyx_t_1 = PyCFunction_NewEx(&__pyx_mdef_4grpc_7_cython_6cygrpc_73is_fork_support_enabled, NULL, __pyx_n_s_grpc__cython_cygrpc); if (unlikely(!__pyx_t_1)) __PYX_ERR(0, 152, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_1);
  if (PyDict_SetItem(__pyx_d, __pyx_n_s_is_fork_support_enabled, __pyx_t_1) < 0) __PYX_ERR(0, 152, __pyx_L1_error)
  __Pyx_DECREF(__pyx_t_1); __pyx_t_1 = 0;

  /* "src/python/grpcio/grpc/_cython/_cygrpc/fork_posix.pyx.pxi":156
 * 
 * 
 * def fork_register_channel(channel):             # <<<<<<<<<<<<<<
 *     if _GRPC_ENABLE_FORK_SUPPORT:
 *         _fork_state.channels.add(channel)
 */
  __pyx_t_1 = PyCFunction_NewEx(&__pyx_mdef_4grpc_7_cython_6cygrpc_75fork_register_channel, NULL, __pyx_n_s_grpc__cython_cygrpc); if (unlikely(!__pyx_t_1)) __PYX_ERR(0, 156, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_1);
  if (PyDict_SetItem(__pyx_d, __pyx_n_s_fork_register_channel, __pyx_t_1) < 0) __PYX_ERR(0, 156, __pyx_L1_error)
  __Pyx_DECREF(__pyx_t_1); __pyx_t_1 = 0;

  /* "src/python/grpcio/grpc/_cython/_cygrpc/fork_posix.pyx.pxi":161
 * 
 * 
 * def fork_unregister_channel(channel):             # <<<<<<<<<<<<<<
 *     if _GRPC_ENABLE_FORK_SUPPORT:
 *         _fork_state.channels.discard(channel)
 */
  __pyx_t_1 = PyCFunction_NewEx(&__pyx_mdef_4grpc_7_cython_6cygrpc_77fork_unregister_channel, NULL, __pyx_n_s_grpc__cython_cygrpc); if (unlikely(!__pyx_t_1)) __PYX_ERR(0, 161, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_1);
  if (PyDict_SetItem(__pyx_d, __pyx_n_s_fork_unregister_channel, __pyx_t_1) < 0) __PYX_ERR(0, 161, __pyx_L1_error)
  __Pyx_DECREF(__pyx_t_1); __pyx_t_1 = 0;

  /* "src/python/grpcio/grpc/_cython/_cygrpc/fork_posix.pyx.pxi":166
 * 
 * 
 * class _ActiveThreadCount(object):             # <<<<<<<<<<<<<<
 *     def __init__(self):
 *         self._num_active_threads = 0
 */
  __pyx_t_1 = __Pyx_CalculateMetaclass(NULL, __pyx_tuple__246); if (unlikely(!__pyx_t_1)) __PYX_ERR(0, 166, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_1);
  __pyx_t_2 = __Pyx_Py3MetaclassPrepare(__pyx_t_1, __pyx_tuple__246, __pyx_n_s_ActiveThreadCount, __pyx_n_s_ActiveThreadCount, (PyObject *) NULL, __pyx_n_s_grpc__cython_cygrpc, (PyObject *) NULL); if (unlikely(!__pyx_t_2)) __PYX_ERR(0, 166, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_2);

  /* "src/python/grpcio/grpc/_cython/_cygrpc/fork_posix.pyx.pxi":167
 * 
 * class _ActiveThreadCount(object):
 *     def __init__(self):             # <<<<<<<<<<<<<<
 *         self._num_active_threads = 0
 *         self._condition = threading.Condition()
 */
  __pyx_t_3 = __Pyx_CyFunction_NewEx(&__pyx_mdef_4grpc_7_cython_6cygrpc_18_ActiveThreadCount_1__init__, 0, __pyx_n_s_ActiveThreadCount___init, NULL, __pyx_n_s_grpc__cython_cygrpc, __pyx_d, ((PyObject *)__pyx_codeobj__248)); if (unlikely(!__pyx_t_3)) __PYX_ERR(0, 167, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_3);
  if (__Pyx_SetNameInClass(__pyx_t_2, __pyx_n_s_init, __pyx_t_3) < 0) __PYX_ERR(0, 167, __pyx_L1_error)
  __Pyx_DECREF(__pyx_t_3); __pyx_t_3 = 0;

  /* "src/python/grpcio/grpc/_cython/_cygrpc/fork_posix.pyx.pxi":171
 *         self._condition = threading.Condition()
 * 
 *     def increment(self):             # <<<<<<<<<<<<<<
 *         with self._condition:
 *             self._num_active_threads += 1
 */
  __pyx_t_3 = __Pyx_CyFunction_NewEx(&__pyx_mdef_4grpc_7_cython_6cygrpc_18_ActiveThreadCount_3increment, 0, __pyx_n_s_ActiveThreadCount_increment, NULL, __pyx_n_s_grpc__cython_cygrpc, __pyx_d, ((PyObject *)__pyx_codeobj__250)); if (unlikely(!__pyx_t_3)) __PYX_ERR(0, 171, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_3);
  if (__Pyx_SetNameInClass(__pyx_t_2, __pyx_n_s_increment, __pyx_t_3) < 0) __PYX_ERR(0, 171, __pyx_L1_error)
  __Pyx_DECREF(__pyx_t_3); __pyx_t_3 = 0;

  /* "src/python/grpcio/grpc/_cython/_cygrpc/fork_posix.pyx.pxi":175
 *             self._num_active_threads += 1
 * 
 *     def decrement(self):             # <<<<<<<<<<<<<<
 *         with self._condition:
 *             self._num_active_threads -= 1
 */
  __pyx_t_3 = __Pyx_CyFunction_NewEx(&__pyx_mdef_4grpc_7_cython_6cygrpc_18_ActiveThreadCount_5decrement, 0, __pyx_n_s_ActiveThreadCount_decrement, NULL, __pyx_n_s_grpc__cython_cygrpc, __pyx_d, ((PyObject *)__pyx_codeobj__252)); if (unlikely(!__pyx_t_3)) __PYX_ERR(0, 175, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_3);
  if (__Pyx_SetNameInClass(__pyx_t_2, __pyx_n_s_decrement, __pyx_t_3) < 0) __PYX_ERR(0, 175, __pyx_L1_error)
  __Pyx_DECREF(__pyx_t_3); __pyx_t_3 = 0;

  /* "src/python/grpcio/grpc/_cython/_cygrpc/fork_posix.pyx.pxi":181
 *                 self._condition.notify_all()
 * 
 *     def await_zero_threads(self, timeout_secs):             # <<<<<<<<<<<<<<
 *         end_time = time.time() + timeout_secs
 *         wait_time = timeout_secs
 */
  __pyx_t_3 = __Pyx_CyFunction_NewEx(&__pyx_mdef_4grpc_7_cython_6cygrpc_18_ActiveThreadCount_7await_zero_threads, 0, __pyx_n_s_ActiveThreadCount_await_zero_th, NULL, __pyx_n_s_grpc__cython_cygrpc, __pyx_d, ((PyObject *)__pyx_codeobj__254)); if (unlikely(!__pyx_t_3)) __PYX_ERR(0, 181, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_3);
  if (__Pyx_SetNameInClass(__pyx_t_2, __pyx_n_s_await_zero_threads, __pyx_t_3) < 0) __PYX_ERR(0, 181, __pyx_L1_error)
  __Pyx_DECREF(__pyx_t_3); __pyx_t_3 = 0;

  /* "src/python/grpcio/grpc/_cython/_cygrpc/fork_posix.pyx.pxi":166
 * 
 * 
 * class _ActiveThreadCount(object):             # <<<<<<<<<<<<<<
 *     def __init__(self):
 *         self._num_active_threads = 0
 */
  __pyx_t_3 = __Pyx_Py3ClassCreate(__pyx_t_1, __pyx_n_s_ActiveThreadCount, __pyx_tuple__246, __pyx_t_2, NULL, 0, 1); if (unlikely(!__pyx_t_3)) __PYX_ERR(0, 166, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_3);
  if (PyDict_SetItem(__pyx_d, __pyx_n_s_ActiveThreadCount, __pyx_t_3) < 0) __PYX_ERR(0, 166, __pyx_L1_error)
  __Pyx_DECREF(__pyx_t_3); __pyx_t_3 = 0;
  __Pyx_DECREF(__pyx_t_2); __pyx_t_2 = 0;
  __Pyx_DECREF(__pyx_t_1); __pyx_t_1 = 0;

  /* "src/python/grpcio/grpc/_cython/_cygrpc/fork_posix.pyx.pxi":198
 * 
 * 
 * class _ForkState(object):             # <<<<<<<<<<<<<<
 *     def __init__(self):
 *         self.fork_in_progress_condition = threading.Condition()
 */
  __pyx_t_1 = __Pyx_CalculateMetaclass(NULL, __pyx_tuple__255); if (unlikely(!__pyx_t_1)) __PYX_ERR(0, 198, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_1);
  __pyx_t_2 = __Pyx_Py3MetaclassPrepare(__pyx_t_1, __pyx_tuple__255, __pyx_n_s_ForkState, __pyx_n_s_ForkState, (PyObject *) NULL, __pyx_n_s_grpc__cython_cygrpc, (PyObject *) NULL); if (unlikely(!__pyx_t_2)) __PYX_ERR(0, 198, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_2);

  /* "src/python/grpcio/grpc/_cython/_cygrpc/fork_posix.pyx.pxi":199
 * 
 * class _ForkState(object):
 *     def __init__(self):             # <<<<<<<<<<<<<<
 *         self.fork_in_progress_condition = threading.Condition()
 *         self.fork_in_progress = False
 */
  __pyx_t_3 = __Pyx_CyFunction_NewEx(&__pyx_mdef_4grpc_7_cython_6cygrpc_10_ForkState_1__init__, 0, __pyx_n_s_ForkState___init, NULL, __pyx_n_s_grpc__cython_cygrpc, __pyx_d, ((PyObject *)__pyx_codeobj__257)); if (unlikely(!__pyx_t_3)) __PYX_ERR(0, 199, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_3);
  if (__Pyx_SetNameInClass(__pyx_t_2, __pyx_n_s_init, __pyx_t_3) < 0) __PYX_ERR(0, 199, __pyx_L1_error)
  __Pyx_DECREF(__pyx_t_3); __pyx_t_3 = 0;

  /* "src/python/grpcio/grpc/_cython/_cygrpc/fork_posix.pyx.pxi":198
 * 
 * 
 * class _ForkState(object):             # <<<<<<<<<<<<<<
 *     def __init__(self):
 *         self.fork_in_progress_condition = threading.Condition()
 */
  __pyx_t_3 = __Pyx_Py3ClassCreate(__pyx_t_1, __pyx_n_s_ForkState, __pyx_tuple__255, __pyx_t_2, NULL, 0, 1); if (unlikely(!__pyx_t_3)) __PYX_ERR(0, 198, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_3);
  if (PyDict_SetItem(__pyx_d, __pyx_n_s_ForkState, __pyx_t_3) < 0) __PYX_ERR(0, 198, __pyx_L1_error)
  __Pyx_DECREF(__pyx_t_3); __pyx_t_3 = 0;
  __Pyx_DECREF(__pyx_t_2); __pyx_t_2 = 0;
  __Pyx_DECREF(__pyx_t_1); __pyx_t_1 = 0;

  /* "src/python/grpcio/grpc/_cython/_cygrpc/fork_posix.pyx.pxi":210
 * 
 * 
 * _fork_state = _ForkState()             # <<<<<<<<<<<<<<
 */
  __pyx_t_1 = __Pyx_GetModuleGlobalName(__pyx_n_s_ForkState); if (unlikely(!__pyx_t_1)) __PYX_ERR(0, 210, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_1);
  __pyx_t_2 = __Pyx_PyObject_CallNoArg(__pyx_t_1); if (unlikely(!__pyx_t_2)) __PYX_ERR(0, 210, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_2);
  __Pyx_DECREF(__pyx_t_1); __pyx_t_1 = 0;
  if (PyDict_SetItem(__pyx_d, __pyx_n_s_fork_state, __pyx_t_2) < 0) __PYX_ERR(0, 210, __pyx_L1_error)
  __Pyx_DECREF(__pyx_t_2); __pyx_t_2 = 0;

  /* "grpc/_cython/cygrpc.pyx":64
 * 
 * 
 * _initialize()             # <<<<<<<<<<<<<<
 */
  __pyx_t_2 = __pyx_f_4grpc_7_cython_6cygrpc__initialize(); if (unlikely(!__pyx_t_2)) __PYX_ERR(21, 64, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_2);
  __Pyx_DECREF(__pyx_t_2); __pyx_t_2 = 0;

  /* "(tree fragment)":1
 * def __pyx_unpickle_ChannelCredentials(__pyx_type, long __pyx_checksum, __pyx_state):             # <<<<<<<<<<<<<<
 *     if __pyx_checksum != 0xd41d8cd:
 *         from pickle import PickleError as __pyx_PickleError
 */
  __pyx_t_2 = PyCFunction_NewEx(&__pyx_mdef_4grpc_7_cython_6cygrpc_79__pyx_unpickle_ChannelCredentials, NULL, __pyx_n_s_grpc__cython_cygrpc); if (unlikely(!__pyx_t_2)) __PYX_ERR(7, 1, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_2);
  if (PyDict_SetItem(__pyx_d, __pyx_n_s_pyx_unpickle_ChannelCredential, __pyx_t_2) < 0) __PYX_ERR(7, 1, __pyx_L1_error)
  __Pyx_DECREF(__pyx_t_2); __pyx_t_2 = 0;

  /* "(tree fragment)":9
 *         __pyx_unpickle_ChannelCredentials__set_state(<ChannelCredentials> __pyx_result, __pyx_state)
 *     return __pyx_result
 * cdef __pyx_unpickle_ChannelCredentials__set_state(ChannelCredentials __pyx_result, tuple __pyx_state):             # <<<<<<<<<<<<<<
 *     if len(__pyx_state) > 0 and hasattr(__pyx_result, '__dict__'):
 *         __pyx_result.__dict__.update(__pyx_state[0])
 */
  __pyx_t_2 = PyCFunction_NewEx(&__pyx_mdef_4grpc_7_cython_6cygrpc_81__pyx_unpickle__Tag, NULL, __pyx_n_s_grpc__cython_cygrpc); if (unlikely(!__pyx_t_2)) __PYX_ERR(7, 1, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_2);
  if (PyDict_SetItem(__pyx_d, __pyx_n_s_pyx_unpickle__Tag, __pyx_t_2) < 0) __PYX_ERR(7, 1, __pyx_L1_error)
  __Pyx_DECREF(__pyx_t_2); __pyx_t_2 = 0;

  /* "(tree fragment)":1
 * def __pyx_unpickle_CensusContext(__pyx_type, long __pyx_checksum, __pyx_state):             # <<<<<<<<<<<<<<
 *     if __pyx_checksum != 0xd41d8cd:
 *         from pickle import PickleError as __pyx_PickleError
 */
  __pyx_t_2 = PyCFunction_NewEx(&__pyx_mdef_4grpc_7_cython_6cygrpc_83__pyx_unpickle_CensusContext, NULL, __pyx_n_s_grpc__cython_cygrpc); if (unlikely(!__pyx_t_2)) __PYX_ERR(7, 1, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_2);
  if (PyDict_SetItem(__pyx_d, __pyx_n_s_pyx_unpickle_CensusContext, __pyx_t_2) < 0) __PYX_ERR(7, 1, __pyx_L1_error)
  __Pyx_DECREF(__pyx_t_2); __pyx_t_2 = 0;

  /* "grpc/_cython/cygrpc.pyx":1
 * # Copyright 2015 gRPC authors.             # <<<<<<<<<<<<<<
 * #
 * # Licensed under the Apache License, Version 2.0 (the "License");
 */
  __pyx_t_2 = __Pyx_PyDict_NewPresized(0); if (unlikely(!__pyx_t_2)) __PYX_ERR(21, 1, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_2);
  if (PyDict_SetItem(__pyx_d, __pyx_n_s_test, __pyx_t_2) < 0) __PYX_ERR(21, 1, __pyx_L1_error)
  __Pyx_DECREF(__pyx_t_2); __pyx_t_2 = 0;

  /*--- Wrapped vars code ---*/

  goto __pyx_L0;
  __pyx_L1_error:;
  __Pyx_XDECREF(__pyx_t_1);
  __Pyx_XDECREF(__pyx_t_2);
  __Pyx_XDECREF(__pyx_t_3);
  if (__pyx_m) {
    if (__pyx_d) {
      __Pyx_AddTraceback("init grpc._cython.cygrpc", 0, __pyx_lineno, __pyx_filename);
    }
    Py_DECREF(__pyx_m); __pyx_m = 0;
  } else if (!PyErr_Occurred()) {
    PyErr_SetString(PyExc_ImportError, "init grpc._cython.cygrpc");
  }
  __pyx_L0:;
  __Pyx_RefNannyFinishContext();
  #if CYTHON_PEP489_MULTI_PHASE_INIT
  return (__pyx_m != NULL) ? 0 : -1;
  #elif PY_MAJOR_VERSION >= 3
  return __pyx_m;
  #else
  return;
  #endif
}

/* --- Runtime support code --- */
/* Refnanny */
#if CYTHON_REFNANNY
static __Pyx_RefNannyAPIStruct *__Pyx_RefNannyImportAPI(const char *modname) {
    PyObject *m = NULL, *p = NULL;
    void *r = NULL;
    m = PyImport_ImportModule((char *)modname);
    if (!m) goto end;
    p = PyObject_GetAttrString(m, (char *)"RefNannyAPI");
    if (!p) goto end;
    r = PyLong_AsVoidPtr(p);
end:
    Py_XDECREF(p);
    Py_XDECREF(m);
    return (__Pyx_RefNannyAPIStruct *)r;
}
#endif

/* PyObjectGetAttrStr */
#if CYTHON_USE_TYPE_SLOTS
static CYTHON_INLINE PyObject* __Pyx_PyObject_GetAttrStr(PyObject* obj, PyObject* attr_name) {
    PyTypeObject* tp = Py_TYPE(obj);
    if (likely(tp->tp_getattro))
        return tp->tp_getattro(obj, attr_name);
#if PY_MAJOR_VERSION < 3
    if (likely(tp->tp_getattr))
        return tp->tp_getattr(obj, PyString_AS_STRING(attr_name));
#endif
    return PyObject_GetAttr(obj, attr_name);
}
#endif

/* GetBuiltinName */
static PyObject *__Pyx_GetBuiltinName(PyObject *name) {
    PyObject* result = __Pyx_PyObject_GetAttrStr(__pyx_b, name);
    if (unlikely(!result)) {
        PyErr_Format(PyExc_NameError,
#if PY_MAJOR_VERSION >= 3
            "name '%U' is not defined", name);
#else
            "name '%.200s' is not defined", PyString_AS_STRING(name));
#endif
    }
    return result;
}

/* PyObjectCall */
#if CYTHON_COMPILING_IN_CPYTHON
static CYTHON_INLINE PyObject* __Pyx_PyObject_Call(PyObject *func, PyObject *arg, PyObject *kw) {
    PyObject *result;
    ternaryfunc call = func->ob_type->tp_call;
    if (unlikely(!call))
        return PyObject_Call(func, arg, kw);
    if (unlikely(Py_EnterRecursiveCall((char*)" while calling a Python object")))
        return NULL;
    result = (*call)(func, arg, kw);
    Py_LeaveRecursiveCall();
    if (unlikely(!result) && unlikely(!PyErr_Occurred())) {
        PyErr_SetString(
            PyExc_SystemError,
            "NULL result without error in PyObject_Call");
    }
    return result;
}
#endif

/* PyCFunctionFastCall */
#if CYTHON_FAST_PYCCALL
static CYTHON_INLINE PyObject * __Pyx_PyCFunction_FastCall(PyObject *func_obj, PyObject **args, Py_ssize_t nargs) {
    PyCFunctionObject *func = (PyCFunctionObject*)func_obj;
    PyCFunction meth = PyCFunction_GET_FUNCTION(func);
    PyObject *self = PyCFunction_GET_SELF(func);
    int flags = PyCFunction_GET_FLAGS(func);
    assert(PyCFunction_Check(func));
    assert(METH_FASTCALL == (flags & ~(METH_CLASS | METH_STATIC | METH_COEXIST | METH_KEYWORDS)));
    assert(nargs >= 0);
    assert(nargs == 0 || args != NULL);
    /* _PyCFunction_FastCallDict() must not be called with an exception set,
       because it may clear it (directly or indirectly) and so the
       caller loses its exception */
    assert(!PyErr_Occurred());
    if ((PY_VERSION_HEX < 0x030700A0) || unlikely(flags & METH_KEYWORDS)) {
        return (*((__Pyx_PyCFunctionFastWithKeywords)meth)) (self, args, nargs, NULL);
    } else {
        return (*((__Pyx_PyCFunctionFast)meth)) (self, args, nargs);
    }
}
#endif

/* PyFunctionFastCall */
#if CYTHON_FAST_PYCALL
#include "frameobject.h"
static PyObject* __Pyx_PyFunction_FastCallNoKw(PyCodeObject *co, PyObject **args, Py_ssize_t na,
                                               PyObject *globals) {
    PyFrameObject *f;
    PyThreadState *tstate = __Pyx_PyThreadState_Current;
    PyObject **fastlocals;
    Py_ssize_t i;
    PyObject *result;
    assert(globals != NULL);
    /* XXX Perhaps we should create a specialized
       PyFrame_New() that doesn't take locals, but does
       take builtins without sanity checking them.
       */
    assert(tstate != NULL);
    f = PyFrame_New(tstate, co, globals, NULL);
    if (f == NULL) {
        return NULL;
    }
    fastlocals = f->f_localsplus;
    for (i = 0; i < na; i++) {
        Py_INCREF(*args);
        fastlocals[i] = *args++;
    }
    result = PyEval_EvalFrameEx(f,0);
    ++tstate->recursion_depth;
    Py_DECREF(f);
    --tstate->recursion_depth;
    return result;
}
#if 1 || PY_VERSION_HEX < 0x030600B1
static PyObject *__Pyx_PyFunction_FastCallDict(PyObject *func, PyObject **args, int nargs, PyObject *kwargs) {
    PyCodeObject *co = (PyCodeObject *)PyFunction_GET_CODE(func);
    PyObject *globals = PyFunction_GET_GLOBALS(func);
    PyObject *argdefs = PyFunction_GET_DEFAULTS(func);
    PyObject *closure;
#if PY_MAJOR_VERSION >= 3
    PyObject *kwdefs;
#endif
    PyObject *kwtuple, **k;
    PyObject **d;
    Py_ssize_t nd;
    Py_ssize_t nk;
    PyObject *result;
    assert(kwargs == NULL || PyDict_Check(kwargs));
    nk = kwargs ? PyDict_Size(kwargs) : 0;
    if (Py_EnterRecursiveCall((char*)" while calling a Python object")) {
        return NULL;
    }
    if (
#if PY_MAJOR_VERSION >= 3
            co->co_kwonlyargcount == 0 &&
#endif
            likely(kwargs == NULL || nk == 0) &&
            co->co_flags == (CO_OPTIMIZED | CO_NEWLOCALS | CO_NOFREE)) {
        if (argdefs == NULL && co->co_argcount == nargs) {
            result = __Pyx_PyFunction_FastCallNoKw(co, args, nargs, globals);
            goto done;
        }
        else if (nargs == 0 && argdefs != NULL
                 && co->co_argcount == Py_SIZE(argdefs)) {
            /* function called with no arguments, but all parameters have
               a default value: use default values as arguments .*/
            args = &PyTuple_GET_ITEM(argdefs, 0);
            result =__Pyx_PyFunction_FastCallNoKw(co, args, Py_SIZE(argdefs), globals);
            goto done;
        }
    }
    if (kwargs != NULL) {
        Py_ssize_t pos, i;
        kwtuple = PyTuple_New(2 * nk);
        if (kwtuple == NULL) {
            result = NULL;
            goto done;
        }
        k = &PyTuple_GET_ITEM(kwtuple, 0);
        pos = i = 0;
        while (PyDict_Next(kwargs, &pos, &k[i], &k[i+1])) {
            Py_INCREF(k[i]);
            Py_INCREF(k[i+1]);
            i += 2;
        }
        nk = i / 2;
    }
    else {
        kwtuple = NULL;
        k = NULL;
    }
    closure = PyFunction_GET_CLOSURE(func);
#if PY_MAJOR_VERSION >= 3
    kwdefs = PyFunction_GET_KW_DEFAULTS(func);
#endif
    if (argdefs != NULL) {
        d = &PyTuple_GET_ITEM(argdefs, 0);
        nd = Py_SIZE(argdefs);
    }
    else {
        d = NULL;
        nd = 0;
    }
#if PY_MAJOR_VERSION >= 3
    result = PyEval_EvalCodeEx((PyObject*)co, globals, (PyObject *)NULL,
                               args, nargs,
                               k, (int)nk,
                               d, (int)nd, kwdefs, closure);
#else
    result = PyEval_EvalCodeEx(co, globals, (PyObject *)NULL,
                               args, nargs,
                               k, (int)nk,
                               d, (int)nd, closure);
#endif
    Py_XDECREF(kwtuple);
done:
    Py_LeaveRecursiveCall();
    return result;
}
#endif
#endif

/* PyObjectCallMethO */
#if CYTHON_COMPILING_IN_CPYTHON
static CYTHON_INLINE PyObject* __Pyx_PyObject_CallMethO(PyObject *func, PyObject *arg) {
    PyObject *self, *result;
    PyCFunction cfunc;
    cfunc = PyCFunction_GET_FUNCTION(func);
    self = PyCFunction_GET_SELF(func);
    if (unlikely(Py_EnterRecursiveCall((char*)" while calling a Python object")))
        return NULL;
    result = cfunc(self, arg);
    Py_LeaveRecursiveCall();
    if (unlikely(!result) && unlikely(!PyErr_Occurred())) {
        PyErr_SetString(
            PyExc_SystemError,
            "NULL result without error in PyObject_Call");
    }
    return result;
}
#endif

/* PyObjectCallOneArg */
#if CYTHON_COMPILING_IN_CPYTHON
static PyObject* __Pyx__PyObject_CallOneArg(PyObject *func, PyObject *arg) {
    PyObject *result;
    PyObject *args = PyTuple_New(1);
    if (unlikely(!args)) return NULL;
    Py_INCREF(arg);
    PyTuple_SET_ITEM(args, 0, arg);
    result = __Pyx_PyObject_Call(func, args, NULL);
    Py_DECREF(args);
    return result;
}
static CYTHON_INLINE PyObject* __Pyx_PyObject_CallOneArg(PyObject *func, PyObject *arg) {
#if CYTHON_FAST_PYCALL
    if (PyFunction_Check(func)) {
        return __Pyx_PyFunction_FastCall(func, &arg, 1);
    }
#endif
    if (likely(PyCFunction_Check(func))) {
        if (likely(PyCFunction_GET_FLAGS(func) & METH_O)) {
            return __Pyx_PyObject_CallMethO(func, arg);
#if CYTHON_FAST_PYCCALL
        } else if (PyCFunction_GET_FLAGS(func) & METH_FASTCALL) {
            return __Pyx_PyCFunction_FastCall(func, &arg, 1);
#endif
        }
    }
    return __Pyx__PyObject_CallOneArg(func, arg);
}
#else
static CYTHON_INLINE PyObject* __Pyx_PyObject_CallOneArg(PyObject *func, PyObject *arg) {
    PyObject *result;
    PyObject *args = PyTuple_Pack(1, arg);
    if (unlikely(!args)) return NULL;
    result = __Pyx_PyObject_Call(func, args, NULL);
    Py_DECREF(args);
    return result;
}
#endif

/* PyErrFetchRestore */
#if CYTHON_FAST_THREAD_STATE
static CYTHON_INLINE void __Pyx_ErrRestoreInState(PyThreadState *tstate, PyObject *type, PyObject *value, PyObject *tb) {
    PyObject *tmp_type, *tmp_value, *tmp_tb;
    tmp_type = tstate->curexc_type;
    tmp_value = tstate->curexc_value;
    tmp_tb = tstate->curexc_traceback;
    tstate->curexc_type = type;
    tstate->curexc_value = value;
    tstate->curexc_traceback = tb;
    Py_XDECREF(tmp_type);
    Py_XDECREF(tmp_value);
    Py_XDECREF(tmp_tb);
}
static CYTHON_INLINE void __Pyx_ErrFetchInState(PyThreadState *tstate, PyObject **type, PyObject **value, PyObject **tb) {
    *type = tstate->curexc_type;
    *value = tstate->curexc_value;
    *tb = tstate->curexc_traceback;
    tstate->curexc_type = 0;
    tstate->curexc_value = 0;
    tstate->curexc_traceback = 0;
}
#endif

/* RaiseException */
#if PY_MAJOR_VERSION < 3
static void __Pyx_Raise(PyObject *type, PyObject *value, PyObject *tb,
                        CYTHON_UNUSED PyObject *cause) {
    __Pyx_PyThreadState_declare
    Py_XINCREF(type);
    if (!value || value == Py_None)
        value = NULL;
    else
        Py_INCREF(value);
    if (!tb || tb == Py_None)
        tb = NULL;
    else {
        Py_INCREF(tb);
        if (!PyTraceBack_Check(tb)) {
            PyErr_SetString(PyExc_TypeError,
                "raise: arg 3 must be a traceback or None");
            goto raise_error;
        }
    }
    if (PyType_Check(type)) {
#if CYTHON_COMPILING_IN_PYPY
        if (!value) {
            Py_INCREF(Py_None);
            value = Py_None;
        }
#endif
        PyErr_NormalizeException(&type, &value, &tb);
    } else {
        if (value) {
            PyErr_SetString(PyExc_TypeError,
                "instance exception may not have a separate value");
            goto raise_error;
        }
        value = type;
        type = (PyObject*) Py_TYPE(type);
        Py_INCREF(type);
        if (!PyType_IsSubtype((PyTypeObject *)type, (PyTypeObject *)PyExc_BaseException)) {
            PyErr_SetString(PyExc_TypeError,
                "raise: exception class must be a subclass of BaseException");
            goto raise_error;
        }
    }
    __Pyx_PyThreadState_assign
    __Pyx_ErrRestore(type, value, tb);
    return;
raise_error:
    Py_XDECREF(value);
    Py_XDECREF(type);
    Py_XDECREF(tb);
    return;
}
#else
static void __Pyx_Raise(PyObject *type, PyObject *value, PyObject *tb, PyObject *cause) {
    PyObject* owned_instance = NULL;
    if (tb == Py_None) {
        tb = 0;
    } else if (tb && !PyTraceBack_Check(tb)) {
        PyErr_SetString(PyExc_TypeError,
            "raise: arg 3 must be a traceback or None");
        goto bad;
    }
    if (value == Py_None)
        value = 0;
    if (PyExceptionInstance_Check(type)) {
        if (value) {
            PyErr_SetString(PyExc_TypeError,
                "instance exception may not have a separate value");
            goto bad;
        }
        value = type;
        type = (PyObject*) Py_TYPE(value);
    } else if (PyExceptionClass_Check(type)) {
        PyObject *instance_class = NULL;
        if (value && PyExceptionInstance_Check(value)) {
            instance_class = (PyObject*) Py_TYPE(value);
            if (instance_class != type) {
                int is_subclass = PyObject_IsSubclass(instance_class, type);
                if (!is_subclass) {
                    instance_class = NULL;
                } else if (unlikely(is_subclass == -1)) {
                    goto bad;
                } else {
                    type = instance_class;
                }
            }
        }
        if (!instance_class) {
            PyObject *args;
            if (!value)
                args = PyTuple_New(0);
            else if (PyTuple_Check(value)) {
                Py_INCREF(value);
                args = value;
            } else
                args = PyTuple_Pack(1, value);
            if (!args)
                goto bad;
            owned_instance = PyObject_Call(type, args, NULL);
            Py_DECREF(args);
            if (!owned_instance)
                goto bad;
            value = owned_instance;
            if (!PyExceptionInstance_Check(value)) {
                PyErr_Format(PyExc_TypeError,
                             "calling %R should have returned an instance of "
                             "BaseException, not %R",
                             type, Py_TYPE(value));
                goto bad;
            }
        }
    } else {
        PyErr_SetString(PyExc_TypeError,
            "raise: exception class must be a subclass of BaseException");
        goto bad;
    }
    if (cause) {
        PyObject *fixed_cause;
        if (cause == Py_None) {
            fixed_cause = NULL;
        } else if (PyExceptionClass_Check(cause)) {
            fixed_cause = PyObject_CallObject(cause, NULL);
            if (fixed_cause == NULL)
                goto bad;
        } else if (PyExceptionInstance_Check(cause)) {
            fixed_cause = cause;
            Py_INCREF(fixed_cause);
        } else {
            PyErr_SetString(PyExc_TypeError,
                            "exception causes must derive from "
                            "BaseException");
            goto bad;
        }
        PyException_SetCause(value, fixed_cause);
    }
    PyErr_SetObject(type, value);
    if (tb) {
#if CYTHON_COMPILING_IN_PYPY
        PyObject *tmp_type, *tmp_value, *tmp_tb;
        PyErr_Fetch(&tmp_type, &tmp_value, &tmp_tb);
        Py_INCREF(tb);
        PyErr_Restore(tmp_type, tmp_value, tb);
        Py_XDECREF(tmp_tb);
#else
        PyThreadState *tstate = __Pyx_PyThreadState_Current;
        PyObject* tmp_tb = tstate->curexc_traceback;
        if (tb != tmp_tb) {
            Py_INCREF(tb);
            tstate->curexc_traceback = tb;
            Py_XDECREF(tmp_tb);
        }
#endif
    }
bad:
    Py_XDECREF(owned_instance);
    return;
}
#endif

/* decode_c_bytes */
static CYTHON_INLINE PyObject* __Pyx_decode_c_bytes(
         const char* cstring, Py_ssize_t length, Py_ssize_t start, Py_ssize_t stop,
         const char* encoding, const char* errors,
         PyObject* (*decode_func)(const char *s, Py_ssize_t size, const char *errors)) {
    if (unlikely((start < 0) | (stop < 0))) {
        if (start < 0) {
            start += length;
            if (start < 0)
                start = 0;
        }
        if (stop < 0)
            stop += length;
    }
    if (stop > length)
        stop = length;
    length = stop - start;
    if (unlikely(length <= 0))
        return PyUnicode_FromUnicode(NULL, 0);
    cstring += start;
    if (decode_func) {
        return decode_func(cstring, length, errors);
    } else {
        return PyUnicode_Decode(cstring, length, encoding, errors);
    }
}

/* SaveResetException */
#if CYTHON_FAST_THREAD_STATE
static CYTHON_INLINE void __Pyx__ExceptionSave(PyThreadState *tstate, PyObject **type, PyObject **value, PyObject **tb) {
    #if PY_VERSION_HEX >= 0x030700A3
    *type = tstate->exc_state.exc_type;
    *value = tstate->exc_state.exc_value;
    *tb = tstate->exc_state.exc_traceback;
    #else
    *type = tstate->exc_type;
    *value = tstate->exc_value;
    *tb = tstate->exc_traceback;
    #endif
    Py_XINCREF(*type);
    Py_XINCREF(*value);
    Py_XINCREF(*tb);
}
static CYTHON_INLINE void __Pyx__ExceptionReset(PyThreadState *tstate, PyObject *type, PyObject *value, PyObject *tb) {
    PyObject *tmp_type, *tmp_value, *tmp_tb;
    #if PY_VERSION_HEX >= 0x030700A3
    tmp_type = tstate->exc_state.exc_type;
    tmp_value = tstate->exc_state.exc_value;
    tmp_tb = tstate->exc_state.exc_traceback;
    tstate->exc_state.exc_type = type;
    tstate->exc_state.exc_value = value;
    tstate->exc_state.exc_traceback = tb;
    #else
    tmp_type = tstate->exc_type;
    tmp_value = tstate->exc_value;
    tmp_tb = tstate->exc_traceback;
    tstate->exc_type = type;
    tstate->exc_value = value;
    tstate->exc_traceback = tb;
    #endif
    Py_XDECREF(tmp_type);
    Py_XDECREF(tmp_value);
    Py_XDECREF(tmp_tb);
}
#endif

/* PyErrExceptionMatches */
#if CYTHON_FAST_THREAD_STATE
static int __Pyx_PyErr_ExceptionMatchesTuple(PyObject *exc_type, PyObject *tuple) {
    Py_ssize_t i, n;
    n = PyTuple_GET_SIZE(tuple);
#if PY_MAJOR_VERSION >= 3
    for (i=0; i<n; i++) {
        if (exc_type == PyTuple_GET_ITEM(tuple, i)) return 1;
    }
#endif
    for (i=0; i<n; i++) {
        if (__Pyx_PyErr_GivenExceptionMatches(exc_type, PyTuple_GET_ITEM(tuple, i))) return 1;
    }
    return 0;
}
static CYTHON_INLINE int __Pyx_PyErr_ExceptionMatchesInState(PyThreadState* tstate, PyObject* err) {
    PyObject *exc_type = tstate->curexc_type;
    if (exc_type == err) return 1;
    if (unlikely(!exc_type)) return 0;
    if (unlikely(PyTuple_Check(err)))
        return __Pyx_PyErr_ExceptionMatchesTuple(exc_type, err);
    return __Pyx_PyErr_GivenExceptionMatches(exc_type, err);
}
#endif

/* GetException */
#if CYTHON_FAST_THREAD_STATE
static int __Pyx__GetException(PyThreadState *tstate, PyObject **type, PyObject **value, PyObject **tb) {
#else
static int __Pyx_GetException(PyObject **type, PyObject **value, PyObject **tb) {
#endif
    PyObject *local_type, *local_value, *local_tb;
#if CYTHON_FAST_THREAD_STATE
    PyObject *tmp_type, *tmp_value, *tmp_tb;
    local_type = tstate->curexc_type;
    local_value = tstate->curexc_value;
    local_tb = tstate->curexc_traceback;
    tstate->curexc_type = 0;
    tstate->curexc_value = 0;
    tstate->curexc_traceback = 0;
#else
    PyErr_Fetch(&local_type, &local_value, &local_tb);
#endif
    PyErr_NormalizeException(&local_type, &local_value, &local_tb);
#if CYTHON_FAST_THREAD_STATE
    if (unlikely(tstate->curexc_type))
#else
    if (unlikely(PyErr_Occurred()))
#endif
        goto bad;
    #if PY_MAJOR_VERSION >= 3
    if (local_tb) {
        if (unlikely(PyException_SetTraceback(local_value, local_tb) < 0))
            goto bad;
    }
    #endif
    Py_XINCREF(local_tb);
    Py_XINCREF(local_type);
    Py_XINCREF(local_value);
    *type = local_type;
    *value = local_value;
    *tb = local_tb;
#if CYTHON_FAST_THREAD_STATE
    #if PY_VERSION_HEX >= 0x030700A3
    tmp_type = tstate->exc_state.exc_type;
    tmp_value = tstate->exc_state.exc_value;
    tmp_tb = tstate->exc_state.exc_traceback;
    tstate->exc_state.exc_type = local_type;
    tstate->exc_state.exc_value = local_value;
    tstate->exc_state.exc_traceback = local_tb;
    #else
    tmp_type = tstate->exc_type;
    tmp_value = tstate->exc_value;
    tmp_tb = tstate->exc_traceback;
    tstate->exc_type = local_type;
    tstate->exc_value = local_value;
    tstate->exc_traceback = local_tb;
    #endif
    Py_XDECREF(tmp_type);
    Py_XDECREF(tmp_value);
    Py_XDECREF(tmp_tb);
#else
    PyErr_SetExcInfo(local_type, local_value, local_tb);
#endif
    return 0;
bad:
    *type = 0;
    *value = 0;
    *tb = 0;
    Py_XDECREF(local_type);
    Py_XDECREF(local_value);
    Py_XDECREF(local_tb);
    return -1;
}

/* GetModuleGlobalName */
  static CYTHON_INLINE PyObject *__Pyx_GetModuleGlobalName(PyObject *name) {
    PyObject *result;
#if !CYTHON_AVOID_BORROWED_REFS
#if CYTHON_COMPILING_IN_CPYTHON && PY_VERSION_HEX >= 0x030500A1
    result = _PyDict_GetItem_KnownHash(__pyx_d, name, ((PyASCIIObject *) name)->hash);
    if (likely(result)) {
        Py_INCREF(result);
    } else if (unlikely(PyErr_Occurred())) {
        result = NULL;
    } else {
#else
    result = PyDict_GetItem(__pyx_d, name);
    if (likely(result)) {
        Py_INCREF(result);
    } else {
#endif
#else
    result = PyObject_GetItem(__pyx_d, name);
    if (!result) {
        PyErr_Clear();
#endif
        result = __Pyx_GetBuiltinName(name);
    }
    return result;
}

/* PyObjectCallNoArg */
      #if CYTHON_COMPILING_IN_CPYTHON
static CYTHON_INLINE PyObject* __Pyx_PyObject_CallNoArg(PyObject *func) {
#if CYTHON_FAST_PYCALL
    if (PyFunction_Check(func)) {
        return __Pyx_PyFunction_FastCall(func, NULL, 0);
    }
#endif
#ifdef __Pyx_CyFunction_USED
    if (likely(PyCFunction_Check(func) || __Pyx_TypeCheck(func, __pyx_CyFunctionType))) {
#else
    if (likely(PyCFunction_Check(func))) {
#endif
        if (likely(PyCFunction_GET_FLAGS(func) & METH_NOARGS)) {
            return __Pyx_PyObject_CallMethO(func, NULL);
        }
    }
    return __Pyx_PyObject_Call(func, __pyx_empty_tuple, NULL);
}
#endif

/* GetItemInt */
        static PyObject *__Pyx_GetItemInt_Generic(PyObject *o, PyObject* j) {
    PyObject *r;
    if (!j) return NULL;
    r = PyObject_GetItem(o, j);
    Py_DECREF(j);
    return r;
}
static CYTHON_INLINE PyObject *__Pyx_GetItemInt_List_Fast(PyObject *o, Py_ssize_t i,
                                                              CYTHON_NCP_UNUSED int wraparound,
                                                              CYTHON_NCP_UNUSED int boundscheck) {
#if CYTHON_ASSUME_SAFE_MACROS && !CYTHON_AVOID_BORROWED_REFS
    Py_ssize_t wrapped_i = i;
    if (wraparound & unlikely(i < 0)) {
        wrapped_i += PyList_GET_SIZE(o);
    }
    if ((!boundscheck) || likely((0 <= wrapped_i) & (wrapped_i < PyList_GET_SIZE(o)))) {
        PyObject *r = PyList_GET_ITEM(o, wrapped_i);
        Py_INCREF(r);
        return r;
    }
    return __Pyx_GetItemInt_Generic(o, PyInt_FromSsize_t(i));
#else
    return PySequence_GetItem(o, i);
#endif
}
static CYTHON_INLINE PyObject *__Pyx_GetItemInt_Tuple_Fast(PyObject *o, Py_ssize_t i,
                                                              CYTHON_NCP_UNUSED int wraparound,
                                                              CYTHON_NCP_UNUSED int boundscheck) {
#if CYTHON_ASSUME_SAFE_MACROS && !CYTHON_AVOID_BORROWED_REFS
    Py_ssize_t wrapped_i = i;
    if (wraparound & unlikely(i < 0)) {
        wrapped_i += PyTuple_GET_SIZE(o);
    }
    if ((!boundscheck) || likely((0 <= wrapped_i) & (wrapped_i < PyTuple_GET_SIZE(o)))) {
        PyObject *r = PyTuple_GET_ITEM(o, wrapped_i);
        Py_INCREF(r);
        return r;
    }
    return __Pyx_GetItemInt_Generic(o, PyInt_FromSsize_t(i));
#else
    return PySequence_GetItem(o, i);
#endif
}
static CYTHON_INLINE PyObject *__Pyx_GetItemInt_Fast(PyObject *o, Py_ssize_t i, int is_list,
                                                     CYTHON_NCP_UNUSED int wraparound,
                                                     CYTHON_NCP_UNUSED int boundscheck) {
#if CYTHON_ASSUME_SAFE_MACROS && !CYTHON_AVOID_BORROWED_REFS && CYTHON_USE_TYPE_SLOTS
    if (is_list || PyList_CheckExact(o)) {
        Py_ssize_t n = ((!wraparound) | likely(i >= 0)) ? i : i + PyList_GET_SIZE(o);
        if ((!boundscheck) || (likely((n >= 0) & (n < PyList_GET_SIZE(o))))) {
            PyObject *r = PyList_GET_ITEM(o, n);
            Py_INCREF(r);
            return r;
        }
    }
    else if (PyTuple_CheckExact(o)) {
        Py_ssize_t n = ((!wraparound) | likely(i >= 0)) ? i : i + PyTuple_GET_SIZE(o);
        if ((!boundscheck) || likely((n >= 0) & (n < PyTuple_GET_SIZE(o)))) {
            PyObject *r = PyTuple_GET_ITEM(o, n);
            Py_INCREF(r);
            return r;
        }
    } else {
        PySequenceMethods *m = Py_TYPE(o)->tp_as_sequence;
        if (likely(m && m->sq_item)) {
            if (wraparound && unlikely(i < 0) && likely(m->sq_length)) {
                Py_ssize_t l = m->sq_length(o);
                if (likely(l >= 0)) {
                    i += l;
                } else {
                    if (!PyErr_ExceptionMatches(PyExc_OverflowError))
                        return NULL;
                    PyErr_Clear();
                }
            }
            return m->sq_item(o, i);
        }
    }
#else
    if (is_list || PySequence_Check(o)) {
        return PySequence_GetItem(o, i);
    }
#endif
    return __Pyx_GetItemInt_Generic(o, PyInt_FromSsize_t(i));
}

/* ExtTypeTest */
        static CYTHON_INLINE int __Pyx_TypeTest(PyObject *obj, PyTypeObject *type) {
    if (unlikely(!type)) {
        PyErr_SetString(PyExc_SystemError, "Missing type object");
        return 0;
    }
    if (likely(__Pyx_TypeCheck(obj, type)))
        return 1;
    PyErr_Format(PyExc_TypeError, "Cannot convert %.200s to %.200s",
                 Py_TYPE(obj)->tp_name, type->tp_name);
    return 0;
}

/* WriteUnraisableException */
        static void __Pyx_WriteUnraisable(const char *name, CYTHON_UNUSED int clineno,
                                  CYTHON_UNUSED int lineno, CYTHON_UNUSED const char *filename,
                                  int full_traceback, CYTHON_UNUSED int nogil) {
    PyObject *old_exc, *old_val, *old_tb;
    PyObject *ctx;
    __Pyx_PyThreadState_declare
#ifdef WITH_THREAD
    PyGILState_STATE state;
    if (nogil)
        state = PyGILState_Ensure();
#ifdef _MSC_VER
    else state = (PyGILState_STATE)-1;
#endif
#endif
    __Pyx_PyThreadState_assign
    __Pyx_ErrFetch(&old_exc, &old_val, &old_tb);
    if (full_traceback) {
        Py_XINCREF(old_exc);
        Py_XINCREF(old_val);
        Py_XINCREF(old_tb);
        __Pyx_ErrRestore(old_exc, old_val, old_tb);
        PyErr_PrintEx(1);
    }
    #if PY_MAJOR_VERSION < 3
    ctx = PyString_FromString(name);
    #else
    ctx = PyUnicode_FromString(name);
    #endif
    __Pyx_ErrRestore(old_exc, old_val, old_tb);
    if (!ctx) {
        PyErr_WriteUnraisable(Py_None);
    } else {
        PyErr_WriteUnraisable(ctx);
        Py_DECREF(ctx);
    }
#ifdef WITH_THREAD
    if (nogil)
        PyGILState_Release(state);
#endif
}

/* RaiseTooManyValuesToUnpack */
        static CYTHON_INLINE void __Pyx_RaiseTooManyValuesError(Py_ssize_t expected) {
    PyErr_Format(PyExc_ValueError,
                 "too many values to unpack (expected %" CYTHON_FORMAT_SSIZE_T "d)", expected);
}

/* RaiseNeedMoreValuesToUnpack */
        static CYTHON_INLINE void __Pyx_RaiseNeedMoreValuesError(Py_ssize_t index) {
    PyErr_Format(PyExc_ValueError,
                 "need more than %" CYTHON_FORMAT_SSIZE_T "d value%.1s to unpack",
                 index, (index == 1) ? "" : "s");
}

/* IterFinish */
        static CYTHON_INLINE int __Pyx_IterFinish(void) {
#if CYTHON_FAST_THREAD_STATE
    PyThreadState *tstate = __Pyx_PyThreadState_Current;
    PyObject* exc_type = tstate->curexc_type;
    if (unlikely(exc_type)) {
        if (likely(__Pyx_PyErr_GivenExceptionMatches(exc_type, PyExc_StopIteration))) {
            PyObject *exc_value, *exc_tb;
            exc_value = tstate->curexc_value;
            exc_tb = tstate->curexc_traceback;
            tstate->curexc_type = 0;
            tstate->curexc_value = 0;
            tstate->curexc_traceback = 0;
            Py_DECREF(exc_type);
            Py_XDECREF(exc_value);
            Py_XDECREF(exc_tb);
            return 0;
        } else {
            return -1;
        }
    }
    return 0;
#else
    if (unlikely(PyErr_Occurred())) {
        if (likely(PyErr_ExceptionMatches(PyExc_StopIteration))) {
            PyErr_Clear();
            return 0;
        } else {
            return -1;
        }
    }
    return 0;
#endif
}

/* UnpackItemEndCheck */
        static int __Pyx_IternextUnpackEndCheck(PyObject *retval, Py_ssize_t expected) {
    if (unlikely(retval)) {
        Py_DECREF(retval);
        __Pyx_RaiseTooManyValuesError(expected);
        return -1;
    } else {
        return __Pyx_IterFinish();
    }
    return 0;
}

/* PyObjectCallMethod1 */
        static PyObject* __Pyx__PyObject_CallMethod1(PyObject* method, PyObject* arg) {
    PyObject *result = NULL;
#if CYTHON_UNPACK_METHODS
    if (likely(PyMethod_Check(method))) {
        PyObject *self = PyMethod_GET_SELF(method);
        if (likely(self)) {
            PyObject *args;
            PyObject *function = PyMethod_GET_FUNCTION(method);
            #if CYTHON_FAST_PYCALL
            if (PyFunction_Check(function)) {
                PyObject *args[2] = {self, arg};
                result = __Pyx_PyFunction_FastCall(function, args, 2);
                goto done;
            }
            #endif
            #if CYTHON_FAST_PYCCALL
            if (__Pyx_PyFastCFunction_Check(function)) {
                PyObject *args[2] = {self, arg};
                result = __Pyx_PyCFunction_FastCall(function, args, 2);
                goto done;
            }
            #endif
            args = PyTuple_New(2);
            if (unlikely(!args)) goto done;
            Py_INCREF(self);
            PyTuple_SET_ITEM(args, 0, self);
            Py_INCREF(arg);
            PyTuple_SET_ITEM(args, 1, arg);
            Py_INCREF(function);
            result = __Pyx_PyObject_Call(function, args, NULL);
            Py_DECREF(args);
            Py_DECREF(function);
            return result;
        }
    }
#endif
    result = __Pyx_PyObject_CallOneArg(method, arg);
    goto done;
done:
    return result;
}
static PyObject* __Pyx_PyObject_CallMethod1(PyObject* obj, PyObject* method_name, PyObject* arg) {
    PyObject *method, *result;
    method = __Pyx_PyObject_GetAttrStr(obj, method_name);
    if (unlikely(!method)) return NULL;
    result = __Pyx__PyObject_CallMethod1(method, arg);
    Py_DECREF(method);
    return result;
}

/* append */
        static CYTHON_INLINE int __Pyx_PyObject_Append(PyObject* L, PyObject* x) {
    if (likely(PyList_CheckExact(L))) {
        if (unlikely(__Pyx_PyList_Append(L, x) < 0)) return -1;
    } else {
        PyObject* retval = __Pyx_PyObject_CallMethod1(L, __pyx_n_s_append, x);
        if (unlikely(!retval))
            return -1;
        Py_DECREF(retval);
    }
    return 0;
}

/* GetAttr */
        static CYTHON_INLINE PyObject *__Pyx_GetAttr(PyObject *o, PyObject *n) {
#if CYTHON_USE_TYPE_SLOTS
#if PY_MAJOR_VERSION >= 3
    if (likely(PyUnicode_Check(n)))
#else
    if (likely(PyString_Check(n)))
#endif
        return __Pyx_PyObject_GetAttrStr(o, n);
#endif
    return PyObject_GetAttr(o, n);
}

/* HasAttr */
        static CYTHON_INLINE int __Pyx_HasAttr(PyObject *o, PyObject *n) {
    PyObject *r;
    if (unlikely(!__Pyx_PyBaseString_Check(n))) {
        PyErr_SetString(PyExc_TypeError,
                        "hasattr(): attribute name must be string");
        return -1;
    }
    r = __Pyx_GetAttr(o, n);
    if (unlikely(!r)) {
        PyErr_Clear();
        return 0;
    } else {
        Py_DECREF(r);
        return 1;
    }
}

/* RaiseArgTupleInvalid */
        static void __Pyx_RaiseArgtupleInvalid(
    const char* func_name,
    int exact,
    Py_ssize_t num_min,
    Py_ssize_t num_max,
    Py_ssize_t num_found)
{
    Py_ssize_t num_expected;
    const char *more_or_less;
    if (num_found < num_min) {
        num_expected = num_min;
        more_or_less = "at least";
    } else {
        num_expected = num_max;
        more_or_less = "at most";
    }
    if (exact) {
        more_or_less = "exactly";
    }
    PyErr_Format(PyExc_TypeError,
                 "%.200s() takes %.8s %" CYTHON_FORMAT_SSIZE_T "d positional argument%.1s (%" CYTHON_FORMAT_SSIZE_T "d given)",
                 func_name, more_or_less, num_expected,
                 (num_expected == 1) ? "" : "s", num_found);
}

/* RaiseDoubleKeywords */
        static void __Pyx_RaiseDoubleKeywordsError(
    const char* func_name,
    PyObject* kw_name)
{
    PyErr_Format(PyExc_TypeError,
        #if PY_MAJOR_VERSION >= 3
        "%s() got multiple values for keyword argument '%U'", func_name, kw_name);
        #else
        "%s() got multiple values for keyword argument '%s'", func_name,
        PyString_AsString(kw_name));
        #endif
}

/* ParseKeywords */
        static int __Pyx_ParseOptionalKeywords(
    PyObject *kwds,
    PyObject **argnames[],
    PyObject *kwds2,
    PyObject *values[],
    Py_ssize_t num_pos_args,
    const char* function_name)
{
    PyObject *key = 0, *value = 0;
    Py_ssize_t pos = 0;
    PyObject*** name;
    PyObject*** first_kw_arg = argnames + num_pos_args;
    while (PyDict_Next(kwds, &pos, &key, &value)) {
        name = first_kw_arg;
        while (*name && (**name != key)) name++;
        if (*name) {
            values[name-argnames] = value;
            continue;
        }
        name = first_kw_arg;
        #if PY_MAJOR_VERSION < 3
        if (likely(PyString_CheckExact(key)) || likely(PyString_Check(key))) {
            while (*name) {
                if ((CYTHON_COMPILING_IN_PYPY || PyString_GET_SIZE(**name) == PyString_GET_SIZE(key))
                        && _PyString_Eq(**name, key)) {
                    values[name-argnames] = value;
                    break;
                }
                name++;
            }
            if (*name) continue;
            else {
                PyObject*** argname = argnames;
                while (argname != first_kw_arg) {
                    if ((**argname == key) || (
                            (CYTHON_COMPILING_IN_PYPY || PyString_GET_SIZE(**argname) == PyString_GET_SIZE(key))
                             && _PyString_Eq(**argname, key))) {
                        goto arg_passed_twice;
                    }
                    argname++;
                }
            }
        } else
        #endif
        if (likely(PyUnicode_Check(key))) {
            while (*name) {
                int cmp = (**name == key) ? 0 :
                #if !CYTHON_COMPILING_IN_PYPY && PY_MAJOR_VERSION >= 3
                    (PyUnicode_GET_SIZE(**name) != PyUnicode_GET_SIZE(key)) ? 1 :
                #endif
                    PyUnicode_Compare(**name, key);
                if (cmp < 0 && unlikely(PyErr_Occurred())) goto bad;
                if (cmp == 0) {
                    values[name-argnames] = value;
                    break;
                }
                name++;
            }
            if (*name) continue;
            else {
                PyObject*** argname = argnames;
                while (argname != first_kw_arg) {
                    int cmp = (**argname == key) ? 0 :
                    #if !CYTHON_COMPILING_IN_PYPY && PY_MAJOR_VERSION >= 3
                        (PyUnicode_GET_SIZE(**argname) != PyUnicode_GET_SIZE(key)) ? 1 :
                    #endif
                        PyUnicode_Compare(**argname, key);
                    if (cmp < 0 && unlikely(PyErr_Occurred())) goto bad;
                    if (cmp == 0) goto arg_passed_twice;
                    argname++;
                }
            }
        } else
            goto invalid_keyword_type;
        if (kwds2) {
            if (unlikely(PyDict_SetItem(kwds2, key, value))) goto bad;
        } else {
            goto invalid_keyword;
        }
    }
    return 0;
arg_passed_twice:
    __Pyx_RaiseDoubleKeywordsError(function_name, key);
    goto bad;
invalid_keyword_type:
    PyErr_Format(PyExc_TypeError,
        "%.200s() keywords must be strings", function_name);
    goto bad;
invalid_keyword:
    PyErr_Format(PyExc_TypeError,
    #if PY_MAJOR_VERSION < 3
        "%.200s() got an unexpected keyword argument '%.200s'",
        function_name, PyString_AsString(key));
    #else
        "%s() got an unexpected keyword argument '%U'",
        function_name, key);
    #endif
bad:
    return -1;
}

/* ArgTypeTest */
        static int __Pyx__ArgTypeTest(PyObject *obj, PyTypeObject *type, const char *name, int exact)
{
    if (unlikely(!type)) {
        PyErr_SetString(PyExc_SystemError, "Missing type object");
        return 0;
    }
    else if (exact) {
        #if PY_MAJOR_VERSION == 2
        if ((type == &PyBaseString_Type) && likely(__Pyx_PyBaseString_CheckExact(obj))) return 1;
        #endif
    }
    else {
        if (likely(__Pyx_TypeCheck(obj, type))) return 1;
    }
    PyErr_Format(PyExc_TypeError,
        "Argument '%.200s' has incorrect type (expected %.200s, got %.200s)",
        name, type->tp_name, Py_TYPE(obj)->tp_name);
    return 0;
}

/* PyIntBinop */
        #if !CYTHON_COMPILING_IN_PYPY
static PyObject* __Pyx_PyInt_AddObjC(PyObject *op1, PyObject *op2, CYTHON_UNUSED long intval, CYTHON_UNUSED int inplace) {
    #if PY_MAJOR_VERSION < 3
    if (likely(PyInt_CheckExact(op1))) {
        const long b = intval;
        long x;
        long a = PyInt_AS_LONG(op1);
            x = (long)((unsigned long)a + b);
            if (likely((x^a) >= 0 || (x^b) >= 0))
                return PyInt_FromLong(x);
            return PyLong_Type.tp_as_number->nb_add(op1, op2);
    }
    #endif
    #if CYTHON_USE_PYLONG_INTERNALS
    if (likely(PyLong_CheckExact(op1))) {
        const long b = intval;
        long a, x;
#ifdef HAVE_LONG_LONG
        const PY_LONG_LONG llb = intval;
        PY_LONG_LONG lla, llx;
#endif
        const digit* digits = ((PyLongObject*)op1)->ob_digit;
        const Py_ssize_t size = Py_SIZE(op1);
        if (likely(__Pyx_sst_abs(size) <= 1)) {
            a = likely(size) ? digits[0] : 0;
            if (size == -1) a = -a;
        } else {
            switch (size) {
                case -2:
                    if (8 * sizeof(long) - 1 > 2 * PyLong_SHIFT) {
                        a = -(long) (((((unsigned long)digits[1]) << PyLong_SHIFT) | (unsigned long)digits[0]));
                        break;
#ifdef HAVE_LONG_LONG
                    } else if (8 * sizeof(PY_LONG_LONG) - 1 > 2 * PyLong_SHIFT) {
                        lla = -(PY_LONG_LONG) (((((unsigned PY_LONG_LONG)digits[1]) << PyLong_SHIFT) | (unsigned PY_LONG_LONG)digits[0]));
                        goto long_long;
#endif
                    }
                    CYTHON_FALLTHROUGH;
                case 2:
                    if (8 * sizeof(long) - 1 > 2 * PyLong_SHIFT) {
                        a = (long) (((((unsigned long)digits[1]) << PyLong_SHIFT) | (unsigned long)digits[0]));
                        break;
#ifdef HAVE_LONG_LONG
                    } else if (8 * sizeof(PY_LONG_LONG) - 1 > 2 * PyLong_SHIFT) {
                        lla = (PY_LONG_LONG) (((((unsigned PY_LONG_LONG)digits[1]) << PyLong_SHIFT) | (unsigned PY_LONG_LONG)digits[0]));
                        goto long_long;
#endif
                    }
                    CYTHON_FALLTHROUGH;
                case -3:
                    if (8 * sizeof(long) - 1 > 3 * PyLong_SHIFT) {
                        a = -(long) (((((((unsigned long)digits[2]) << PyLong_SHIFT) | (unsigned long)digits[1]) << PyLong_SHIFT) | (unsigned long)digits[0]));
                        break;
#ifdef HAVE_LONG_LONG
                    } else if (8 * sizeof(PY_LONG_LONG) - 1 > 3 * PyLong_SHIFT) {
                        lla = -(PY_LONG_LONG) (((((((unsigned PY_LONG_LONG)digits[2]) << PyLong_SHIFT) | (unsigned PY_LONG_LONG)digits[1]) << PyLong_SHIFT) | (unsigned PY_LONG_LONG)digits[0]));
                        goto long_long;
#endif
                    }
                    CYTHON_FALLTHROUGH;
                case 3:
                    if (8 * sizeof(long) - 1 > 3 * PyLong_SHIFT) {
                        a = (long) (((((((unsigned long)digits[2]) << PyLong_SHIFT) | (unsigned long)digits[1]) << PyLong_SHIFT) | (unsigned long)digits[0]));
                        break;
#ifdef HAVE_LONG_LONG
                    } else if (8 * sizeof(PY_LONG_LONG) - 1 > 3 * PyLong_SHIFT) {
                        lla = (PY_LONG_LONG) (((((((unsigned PY_LONG_LONG)digits[2]) << PyLong_SHIFT) | (unsigned PY_LONG_LONG)digits[1]) << PyLong_SHIFT) | (unsigned PY_LONG_LONG)digits[0]));
                        goto long_long;
#endif
                    }
                    CYTHON_FALLTHROUGH;
                case -4:
                    if (8 * sizeof(long) - 1 > 4 * PyLong_SHIFT) {
                        a = -(long) (((((((((unsigned long)digits[3]) << PyLong_SHIFT) | (unsigned long)digits[2]) << PyLong_SHIFT) | (unsigned long)digits[1]) << PyLong_SHIFT) | (unsigned long)digits[0]));
                        break;
#ifdef HAVE_LONG_LONG
                    } else if (8 * sizeof(PY_LONG_LONG) - 1 > 4 * PyLong_SHIFT) {
                        lla = -(PY_LONG_LONG) (((((((((unsigned PY_LONG_LONG)digits[3]) << PyLong_SHIFT) | (unsigned PY_LONG_LONG)digits[2]) << PyLong_SHIFT) | (unsigned PY_LONG_LONG)digits[1]) << PyLong_SHIFT) | (unsigned PY_LONG_LONG)digits[0]));
                        goto long_long;
#endif
                    }
                    CYTHON_FALLTHROUGH;
                case 4:
                    if (8 * sizeof(long) - 1 > 4 * PyLong_SHIFT) {
                        a = (long) (((((((((unsigned long)digits[3]) << PyLong_SHIFT) | (unsigned long)digits[2]) << PyLong_SHIFT) | (unsigned long)digits[1]) << PyLong_SHIFT) | (unsigned long)digits[0]));
                        break;
#ifdef HAVE_LONG_LONG
                    } else if (8 * sizeof(PY_LONG_LONG) - 1 > 4 * PyLong_SHIFT) {
                        lla = (PY_LONG_LONG) (((((((((unsigned PY_LONG_LONG)digits[3]) << PyLong_SHIFT) | (unsigned PY_LONG_LONG)digits[2]) << PyLong_SHIFT) | (unsigned PY_LONG_LONG)digits[1]) << PyLong_SHIFT) | (unsigned PY_LONG_LONG)digits[0]));
                        goto long_long;
#endif
                    }
                    CYTHON_FALLTHROUGH;
                default: return PyLong_Type.tp_as_number->nb_add(op1, op2);
            }
        }
                x = a + b;
            return PyLong_FromLong(x);
#ifdef HAVE_LONG_LONG
        long_long:
                llx = lla + llb;
            return PyLong_FromLongLong(llx);
#endif
        
        
    }
    #endif
    if (PyFloat_CheckExact(op1)) {
        const long b = intval;
        double a = PyFloat_AS_DOUBLE(op1);
            double result;
            PyFPE_START_PROTECT("add", return NULL)
            result = ((double)a) + (double)b;
            PyFPE_END_PROTECT(result)
            return PyFloat_FromDouble(result);
    }
    return (inplace ? PyNumber_InPlaceAdd : PyNumber_Add)(op1, op2);
}
#endif

/* KeywordStringCheck */
        static int __Pyx_CheckKeywordStrings(
    PyObject *kwdict,
    const char* function_name,
    int kw_allowed)
{
    PyObject* key = 0;
    Py_ssize_t pos = 0;
#if CYTHON_COMPILING_IN_PYPY
    if (!kw_allowed && PyDict_Next(kwdict, &pos, &key, 0))
        goto invalid_keyword;
    return 1;
#else
    while (PyDict_Next(kwdict, &pos, &key, 0)) {
        #if PY_MAJOR_VERSION < 3
        if (unlikely(!PyString_Check(key)))
        #endif
            if (unlikely(!PyUnicode_Check(key)))
                goto invalid_keyword_type;
    }
    if ((!kw_allowed) && unlikely(key))
        goto invalid_keyword;
    return 1;
invalid_keyword_type:
    PyErr_Format(PyExc_TypeError,
        "%.200s() keywords must be strings", function_name);
    return 0;
#endif
invalid_keyword:
    PyErr_Format(PyExc_TypeError,
    #if PY_MAJOR_VERSION < 3
        "%.200s() got an unexpected keyword argument '%.200s'",
        function_name, PyString_AsString(key));
    #else
        "%s() got an unexpected keyword argument '%U'",
        function_name, key);
    #endif
    return 0;
}

/* RaiseNoneIterError */
        static CYTHON_INLINE void __Pyx_RaiseNoneNotIterableError(void) {
    PyErr_SetString(PyExc_TypeError, "'NoneType' object is not iterable");
}

/* UnpackUnboundCMethod */
        static int __Pyx_TryUnpackUnboundCMethod(__Pyx_CachedCFunction* target) {
    PyObject *method;
    method = __Pyx_PyObject_GetAttrStr(target->type, *target->method_name);
    if (unlikely(!method))
        return -1;
    target->method = method;
#if CYTHON_COMPILING_IN_CPYTHON
    #if PY_MAJOR_VERSION >= 3
    if (likely(__Pyx_TypeCheck(method, &PyMethodDescr_Type)))
    #endif
    {
        PyMethodDescrObject *descr = (PyMethodDescrObject*) method;
        target->func = descr->d_method->ml_meth;
        target->flag = descr->d_method->ml_flags & ~(METH_CLASS | METH_STATIC | METH_COEXIST);
    }
#endif
    return 0;
}

/* CallUnboundCMethod2 */
        #if CYTHON_COMPILING_IN_CPYTHON && PY_VERSION_HEX >= 0x030600B1
static CYTHON_INLINE PyObject *__Pyx_CallUnboundCMethod2(__Pyx_CachedCFunction *cfunc, PyObject *self, PyObject *arg1, PyObject *arg2) {
    if (likely(cfunc->func)) {
        PyObject *args[2] = {arg1, arg2};
        if (cfunc->flag == METH_FASTCALL) {
            #if PY_VERSION_HEX >= 0x030700A0
            return (*(__Pyx_PyCFunctionFast)cfunc->func)(self, args, 2);
            #else
            return (*(__Pyx_PyCFunctionFastWithKeywords)cfunc->func)(self, args, 2, NULL);
            #endif
        }
        #if PY_VERSION_HEX >= 0x030700A0
        if (cfunc->flag == (METH_FASTCALL | METH_KEYWORDS))
            return (*(__Pyx_PyCFunctionFastWithKeywords)cfunc->func)(self, args, 2, NULL);
        #endif
    }
    return __Pyx__CallUnboundCMethod2(cfunc, self, arg1, arg2);
}
#endif
static PyObject* __Pyx__CallUnboundCMethod2(__Pyx_CachedCFunction* cfunc, PyObject* self, PyObject* arg1, PyObject* arg2){
    PyObject *args, *result = NULL;
    if (unlikely(!cfunc->func && !cfunc->method) && unlikely(__Pyx_TryUnpackUnboundCMethod(cfunc) < 0)) return NULL;
#if CYTHON_COMPILING_IN_CPYTHON
    if (cfunc->func && (cfunc->flag & METH_VARARGS)) {
        args = PyTuple_New(2);
        if (unlikely(!args)) goto bad;
        Py_INCREF(arg1);
        PyTuple_SET_ITEM(args, 0, arg1);
        Py_INCREF(arg2);
        PyTuple_SET_ITEM(args, 1, arg2);
        if (cfunc->flag & METH_KEYWORDS)
            result = (*(PyCFunctionWithKeywords)cfunc->func)(self, args, NULL);
        else
            result = (*cfunc->func)(self, args);
    } else {
        args = PyTuple_New(3);
        if (unlikely(!args)) goto bad;
        Py_INCREF(self);
        PyTuple_SET_ITEM(args, 0, self);
        Py_INCREF(arg1);
        PyTuple_SET_ITEM(args, 1, arg1);
        Py_INCREF(arg2);
        PyTuple_SET_ITEM(args, 2, arg2);
        result = __Pyx_PyObject_Call(cfunc->method, args, NULL);
    }
#else
    args = PyTuple_Pack(3, self, arg1, arg2);
    if (unlikely(!args)) goto bad;
    result = __Pyx_PyObject_Call(cfunc->method, args, NULL);
#endif
bad:
    Py_XDECREF(args);
    return result;
}

/* CallUnboundCMethod1 */
        #if CYTHON_COMPILING_IN_CPYTHON
static CYTHON_INLINE PyObject* __Pyx_CallUnboundCMethod1(__Pyx_CachedCFunction* cfunc, PyObject* self, PyObject* arg) {
    if (likely(cfunc->func)) {
        int flag = cfunc->flag;
        if (flag == METH_O) {
            return (*(cfunc->func))(self, arg);
        } else if (PY_VERSION_HEX >= 0x030600B1 && flag == METH_FASTCALL) {
            if (PY_VERSION_HEX >= 0x030700A0) {
                return (*(__Pyx_PyCFunctionFast)cfunc->func)(self, &arg, 1);
            } else {
                return (*(__Pyx_PyCFunctionFastWithKeywords)cfunc->func)(self, &arg, 1, NULL);
            }
        } else if (PY_VERSION_HEX >= 0x030700A0 && flag == (METH_FASTCALL | METH_KEYWORDS)) {
            return (*(__Pyx_PyCFunctionFastWithKeywords)cfunc->func)(self, &arg, 1, NULL);
        }
    }
    return __Pyx__CallUnboundCMethod1(cfunc, self, arg);
}
#endif
static PyObject* __Pyx__CallUnboundCMethod1(__Pyx_CachedCFunction* cfunc, PyObject* self, PyObject* arg){
    PyObject *args, *result = NULL;
    if (unlikely(!cfunc->func && !cfunc->method) && unlikely(__Pyx_TryUnpackUnboundCMethod(cfunc) < 0)) return NULL;
#if CYTHON_COMPILING_IN_CPYTHON
    if (cfunc->func && (cfunc->flag & METH_VARARGS)) {
        args = PyTuple_New(1);
        if (unlikely(!args)) goto bad;
        Py_INCREF(arg);
        PyTuple_SET_ITEM(args, 0, arg);
        if (cfunc->flag & METH_KEYWORDS)
            result = (*(PyCFunctionWithKeywords)cfunc->func)(self, args, NULL);
        else
            result = (*cfunc->func)(self, args);
    } else {
        args = PyTuple_New(2);
        if (unlikely(!args)) goto bad;
        Py_INCREF(self);
        PyTuple_SET_ITEM(args, 0, self);
        Py_INCREF(arg);
        PyTuple_SET_ITEM(args, 1, arg);
        result = __Pyx_PyObject_Call(cfunc->method, args, NULL);
    }
#else
    args = PyTuple_Pack(2, self, arg);
    if (unlikely(!args)) goto bad;
    result = __Pyx_PyObject_Call(cfunc->method, args, NULL);
#endif
bad:
    Py_XDECREF(args);
    return result;
}

/* py_dict_pop */
        static CYTHON_INLINE PyObject *__Pyx_PyDict_Pop(PyObject *d, PyObject *key, PyObject *default_value) {
#if CYTHON_COMPILING_IN_CPYTHON && PY_VERSION_HEX > 0x030600B3
    if ((1)) {
        return _PyDict_Pop(d, key, default_value);
    } else
#endif
    if (default_value) {
        return __Pyx_CallUnboundCMethod2(&__pyx_umethod_PyDict_Type_pop, d, key, default_value);
    } else {
        return __Pyx_CallUnboundCMethod1(&__pyx_umethod_PyDict_Type_pop, d, key);
    }
}

/* pyfrozenset_new */
        static CYTHON_INLINE PyObject* __Pyx_PyFrozenSet_New(PyObject* it) {
    if (it) {
        PyObject* result;
#if CYTHON_COMPILING_IN_PYPY
        PyObject* args;
        args = PyTuple_Pack(1, it);
        if (unlikely(!args))
            return NULL;
        result = PyObject_Call((PyObject*)&PyFrozenSet_Type, args, NULL);
        Py_DECREF(args);
        return result;
#else
        if (PyFrozenSet_CheckExact(it)) {
            Py_INCREF(it);
            return it;
        }
        result = PyFrozenSet_New(it);
        if (unlikely(!result))
            return NULL;
        if (likely(PySet_GET_SIZE(result)))
            return result;
        Py_DECREF(result);
#endif
    }
#if CYTHON_USE_TYPE_SLOTS
    return PyFrozenSet_Type.tp_new(&PyFrozenSet_Type, __pyx_empty_tuple, NULL);
#else
    return PyObject_Call((PyObject*)&PyFrozenSet_Type, __pyx_empty_tuple, NULL);
#endif
}

/* py_set_discard_unhashable */
        static int __Pyx_PySet_DiscardUnhashable(PyObject *set, PyObject *key) {
    PyObject *tmpkey;
    int rv;
    if (likely(!PySet_Check(key) || !PyErr_ExceptionMatches(PyExc_TypeError)))
        return -1;
    PyErr_Clear();
    tmpkey = __Pyx_PyFrozenSet_New(key);
    if (tmpkey == NULL)
        return -1;
    rv = PySet_Discard(set, tmpkey);
    Py_DECREF(tmpkey);
    return rv;
}

/* py_set_remove */
        static int __Pyx_PySet_RemoveNotFound(PyObject *set, PyObject *key, int found) {
    if (unlikely(found < 0)) {
        found = __Pyx_PySet_DiscardUnhashable(set, key);
    }
    if (likely(found == 0)) {
        PyObject *tup;
        tup = PyTuple_Pack(1, key);
        if (!tup)
            return -1;
        PyErr_SetObject(PyExc_KeyError, tup);
        Py_DECREF(tup);
        return -1;
    }
    return found;
}
static CYTHON_INLINE int __Pyx_PySet_Remove(PyObject *set, PyObject *key) {
    int found = PySet_Discard(set, key);
    if (unlikely(found != 1)) {
        return __Pyx_PySet_RemoveNotFound(set, key, found);
    }
    return 0;
}

/* None */
        static CYTHON_INLINE void __Pyx_RaiseClosureNameError(const char *varname) {
    PyErr_Format(PyExc_NameError, "free variable '%s' referenced before assignment in enclosing scope", varname);
}

/* FetchCommonType */
        static PyTypeObject* __Pyx_FetchCommonType(PyTypeObject* type) {
    PyObject* fake_module;
    PyTypeObject* cached_type = NULL;
    fake_module = PyImport_AddModule((char*) "_cython_" CYTHON_ABI);
    if (!fake_module) return NULL;
    Py_INCREF(fake_module);
    cached_type = (PyTypeObject*) PyObject_GetAttrString(fake_module, type->tp_name);
    if (cached_type) {
        if (!PyType_Check((PyObject*)cached_type)) {
            PyErr_Format(PyExc_TypeError,
                "Shared Cython type %.200s is not a type object",
                type->tp_name);
            goto bad;
        }
        if (cached_type->tp_basicsize != type->tp_basicsize) {
            PyErr_Format(PyExc_TypeError,
                "Shared Cython type %.200s has the wrong size, try recompiling",
                type->tp_name);
            goto bad;
        }
    } else {
        if (!PyErr_ExceptionMatches(PyExc_AttributeError)) goto bad;
        PyErr_Clear();
        if (PyType_Ready(type) < 0) goto bad;
        if (PyObject_SetAttrString(fake_module, type->tp_name, (PyObject*) type) < 0)
            goto bad;
        Py_INCREF(type);
        cached_type = type;
    }
done:
    Py_DECREF(fake_module);
    return cached_type;
bad:
    Py_XDECREF(cached_type);
    cached_type = NULL;
    goto done;
}

/* CythonFunction */
        #include <structmember.h>
static PyObject *
__Pyx_CyFunction_get_doc(__pyx_CyFunctionObject *op, CYTHON_UNUSED void *closure)
{
    if (unlikely(op->func_doc == NULL)) {
        if (op->func.m_ml->ml_doc) {
#if PY_MAJOR_VERSION >= 3
            op->func_doc = PyUnicode_FromString(op->func.m_ml->ml_doc);
#else
            op->func_doc = PyString_FromString(op->func.m_ml->ml_doc);
#endif
            if (unlikely(op->func_doc == NULL))
                return NULL;
        } else {
            Py_INCREF(Py_None);
            return Py_None;
        }
    }
    Py_INCREF(op->func_doc);
    return op->func_doc;
}
static int
__Pyx_CyFunction_set_doc(__pyx_CyFunctionObject *op, PyObject *value)
{
    PyObject *tmp = op->func_doc;
    if (value == NULL) {
        value = Py_None;
    }
    Py_INCREF(value);
    op->func_doc = value;
    Py_XDECREF(tmp);
    return 0;
}
static PyObject *
__Pyx_CyFunction_get_name(__pyx_CyFunctionObject *op)
{
    if (unlikely(op->func_name == NULL)) {
#if PY_MAJOR_VERSION >= 3
        op->func_name = PyUnicode_InternFromString(op->func.m_ml->ml_name);
#else
        op->func_name = PyString_InternFromString(op->func.m_ml->ml_name);
#endif
        if (unlikely(op->func_name == NULL))
            return NULL;
    }
    Py_INCREF(op->func_name);
    return op->func_name;
}
static int
__Pyx_CyFunction_set_name(__pyx_CyFunctionObject *op, PyObject *value)
{
    PyObject *tmp;
#if PY_MAJOR_VERSION >= 3
    if (unlikely(value == NULL || !PyUnicode_Check(value))) {
#else
    if (unlikely(value == NULL || !PyString_Check(value))) {
#endif
        PyErr_SetString(PyExc_TypeError,
                        "__name__ must be set to a string object");
        return -1;
    }
    tmp = op->func_name;
    Py_INCREF(value);
    op->func_name = value;
    Py_XDECREF(tmp);
    return 0;
}
static PyObject *
__Pyx_CyFunction_get_qualname(__pyx_CyFunctionObject *op)
{
    Py_INCREF(op->func_qualname);
    return op->func_qualname;
}
static int
__Pyx_CyFunction_set_qualname(__pyx_CyFunctionObject *op, PyObject *value)
{
    PyObject *tmp;
#if PY_MAJOR_VERSION >= 3
    if (unlikely(value == NULL || !PyUnicode_Check(value))) {
#else
    if (unlikely(value == NULL || !PyString_Check(value))) {
#endif
        PyErr_SetString(PyExc_TypeError,
                        "__qualname__ must be set to a string object");
        return -1;
    }
    tmp = op->func_qualname;
    Py_INCREF(value);
    op->func_qualname = value;
    Py_XDECREF(tmp);
    return 0;
}
static PyObject *
__Pyx_CyFunction_get_self(__pyx_CyFunctionObject *m, CYTHON_UNUSED void *closure)
{
    PyObject *self;
    self = m->func_closure;
    if (self == NULL)
        self = Py_None;
    Py_INCREF(self);
    return self;
}
static PyObject *
__Pyx_CyFunction_get_dict(__pyx_CyFunctionObject *op)
{
    if (unlikely(op->func_dict == NULL)) {
        op->func_dict = PyDict_New();
        if (unlikely(op->func_dict == NULL))
            return NULL;
    }
    Py_INCREF(op->func_dict);
    return op->func_dict;
}
static int
__Pyx_CyFunction_set_dict(__pyx_CyFunctionObject *op, PyObject *value)
{
    PyObject *tmp;
    if (unlikely(value == NULL)) {
        PyErr_SetString(PyExc_TypeError,
               "function's dictionary may not be deleted");
        return -1;
    }
    if (unlikely(!PyDict_Check(value))) {
        PyErr_SetString(PyExc_TypeError,
               "setting function's dictionary to a non-dict");
        return -1;
    }
    tmp = op->func_dict;
    Py_INCREF(value);
    op->func_dict = value;
    Py_XDECREF(tmp);
    return 0;
}
static PyObject *
__Pyx_CyFunction_get_globals(__pyx_CyFunctionObject *op)
{
    Py_INCREF(op->func_globals);
    return op->func_globals;
}
static PyObject *
__Pyx_CyFunction_get_closure(CYTHON_UNUSED __pyx_CyFunctionObject *op)
{
    Py_INCREF(Py_None);
    return Py_None;
}
static PyObject *
__Pyx_CyFunction_get_code(__pyx_CyFunctionObject *op)
{
    PyObject* result = (op->func_code) ? op->func_code : Py_None;
    Py_INCREF(result);
    return result;
}
static int
__Pyx_CyFunction_init_defaults(__pyx_CyFunctionObject *op) {
    int result = 0;
    PyObject *res = op->defaults_getter((PyObject *) op);
    if (unlikely(!res))
        return -1;
    #if CYTHON_ASSUME_SAFE_MACROS && !CYTHON_AVOID_BORROWED_REFS
    op->defaults_tuple = PyTuple_GET_ITEM(res, 0);
    Py_INCREF(op->defaults_tuple);
    op->defaults_kwdict = PyTuple_GET_ITEM(res, 1);
    Py_INCREF(op->defaults_kwdict);
    #else
    op->defaults_tuple = PySequence_ITEM(res, 0);
    if (unlikely(!op->defaults_tuple)) result = -1;
    else {
        op->defaults_kwdict = PySequence_ITEM(res, 1);
        if (unlikely(!op->defaults_kwdict)) result = -1;
    }
    #endif
    Py_DECREF(res);
    return result;
}
static int
__Pyx_CyFunction_set_defaults(__pyx_CyFunctionObject *op, PyObject* value) {
    PyObject* tmp;
    if (!value) {
        value = Py_None;
    } else if (value != Py_None && !PyTuple_Check(value)) {
        PyErr_SetString(PyExc_TypeError,
                        "__defaults__ must be set to a tuple object");
        return -1;
    }
    Py_INCREF(value);
    tmp = op->defaults_tuple;
    op->defaults_tuple = value;
    Py_XDECREF(tmp);
    return 0;
}
static PyObject *
__Pyx_CyFunction_get_defaults(__pyx_CyFunctionObject *op) {
    PyObject* result = op->defaults_tuple;
    if (unlikely(!result)) {
        if (op->defaults_getter) {
            if (__Pyx_CyFunction_init_defaults(op) < 0) return NULL;
            result = op->defaults_tuple;
        } else {
            result = Py_None;
        }
    }
    Py_INCREF(result);
    return result;
}
static int
__Pyx_CyFunction_set_kwdefaults(__pyx_CyFunctionObject *op, PyObject* value) {
    PyObject* tmp;
    if (!value) {
        value = Py_None;
    } else if (value != Py_None && !PyDict_Check(value)) {
        PyErr_SetString(PyExc_TypeError,
                        "__kwdefaults__ must be set to a dict object");
        return -1;
    }
    Py_INCREF(value);
    tmp = op->defaults_kwdict;
    op->defaults_kwdict = value;
    Py_XDECREF(tmp);
    return 0;
}
static PyObject *
__Pyx_CyFunction_get_kwdefaults(__pyx_CyFunctionObject *op) {
    PyObject* result = op->defaults_kwdict;
    if (unlikely(!result)) {
        if (op->defaults_getter) {
            if (__Pyx_CyFunction_init_defaults(op) < 0) return NULL;
            result = op->defaults_kwdict;
        } else {
            result = Py_None;
        }
    }
    Py_INCREF(result);
    return result;
}
static int
__Pyx_CyFunction_set_annotations(__pyx_CyFunctionObject *op, PyObject* value) {
    PyObject* tmp;
    if (!value || value == Py_None) {
        value = NULL;
    } else if (!PyDict_Check(value)) {
        PyErr_SetString(PyExc_TypeError,
                        "__annotations__ must be set to a dict object");
        return -1;
    }
    Py_XINCREF(value);
    tmp = op->func_annotations;
    op->func_annotations = value;
    Py_XDECREF(tmp);
    return 0;
}
static PyObject *
__Pyx_CyFunction_get_annotations(__pyx_CyFunctionObject *op) {
    PyObject* result = op->func_annotations;
    if (unlikely(!result)) {
        result = PyDict_New();
        if (unlikely(!result)) return NULL;
        op->func_annotations = result;
    }
    Py_INCREF(result);
    return result;
}
static PyGetSetDef __pyx_CyFunction_getsets[] = {
    {(char *) "func_doc", (getter)__Pyx_CyFunction_get_doc, (setter)__Pyx_CyFunction_set_doc, 0, 0},
    {(char *) "__doc__",  (getter)__Pyx_CyFunction_get_doc, (setter)__Pyx_CyFunction_set_doc, 0, 0},
    {(char *) "func_name", (getter)__Pyx_CyFunction_get_name, (setter)__Pyx_CyFunction_set_name, 0, 0},
    {(char *) "__name__", (getter)__Pyx_CyFunction_get_name, (setter)__Pyx_CyFunction_set_name, 0, 0},
    {(char *) "__qualname__", (getter)__Pyx_CyFunction_get_qualname, (setter)__Pyx_CyFunction_set_qualname, 0, 0},
    {(char *) "__self__", (getter)__Pyx_CyFunction_get_self, 0, 0, 0},
    {(char *) "func_dict", (getter)__Pyx_CyFunction_get_dict, (setter)__Pyx_CyFunction_set_dict, 0, 0},
    {(char *) "__dict__", (getter)__Pyx_CyFunction_get_dict, (setter)__Pyx_CyFunction_set_dict, 0, 0},
    {(char *) "func_globals", (getter)__Pyx_CyFunction_get_globals, 0, 0, 0},
    {(char *) "__globals__", (getter)__Pyx_CyFunction_get_globals, 0, 0, 0},
    {(char *) "func_closure", (getter)__Pyx_CyFunction_get_closure, 0, 0, 0},
    {(char *) "__closure__", (getter)__Pyx_CyFunction_get_closure, 0, 0, 0},
    {(char *) "func_code", (getter)__Pyx_CyFunction_get_code, 0, 0, 0},
    {(char *) "__code__", (getter)__Pyx_CyFunction_get_code, 0, 0, 0},
    {(char *) "func_defaults", (getter)__Pyx_CyFunction_get_defaults, (setter)__Pyx_CyFunction_set_defaults, 0, 0},
    {(char *) "__defaults__", (getter)__Pyx_CyFunction_get_defaults, (setter)__Pyx_CyFunction_set_defaults, 0, 0},
    {(char *) "__kwdefaults__", (getter)__Pyx_CyFunction_get_kwdefaults, (setter)__Pyx_CyFunction_set_kwdefaults, 0, 0},
    {(char *) "__annotations__", (getter)__Pyx_CyFunction_get_annotations, (setter)__Pyx_CyFunction_set_annotations, 0, 0},
    {0, 0, 0, 0, 0}
};
static PyMemberDef __pyx_CyFunction_members[] = {
    {(char *) "__module__", T_OBJECT, offsetof(PyCFunctionObject, m_module), PY_WRITE_RESTRICTED, 0},
    {0, 0, 0,  0, 0}
};
static PyObject *
__Pyx_CyFunction_reduce(__pyx_CyFunctionObject *m, CYTHON_UNUSED PyObject *args)
{
#if PY_MAJOR_VERSION >= 3
    return PyUnicode_FromString(m->func.m_ml->ml_name);
#else
    return PyString_FromString(m->func.m_ml->ml_name);
#endif
}
static PyMethodDef __pyx_CyFunction_methods[] = {
    {"__reduce__", (PyCFunction)__Pyx_CyFunction_reduce, METH_VARARGS, 0},
    {0, 0, 0, 0}
};
#if PY_VERSION_HEX < 0x030500A0
#define __Pyx_CyFunction_weakreflist(cyfunc) ((cyfunc)->func_weakreflist)
#else
#define __Pyx_CyFunction_weakreflist(cyfunc) ((cyfunc)->func.m_weakreflist)
#endif
static PyObject *__Pyx_CyFunction_New(PyTypeObject *type, PyMethodDef *ml, int flags, PyObject* qualname,
                                      PyObject *closure, PyObject *module, PyObject* globals, PyObject* code) {
    __pyx_CyFunctionObject *op = PyObject_GC_New(__pyx_CyFunctionObject, type);
    if (op == NULL)
        return NULL;
    op->flags = flags;
    __Pyx_CyFunction_weakreflist(op) = NULL;
    op->func.m_ml = ml;
    op->func.m_self = (PyObject *) op;
    Py_XINCREF(closure);
    op->func_closure = closure;
    Py_XINCREF(module);
    op->func.m_module = module;
    op->func_dict = NULL;
    op->func_name = NULL;
    Py_INCREF(qualname);
    op->func_qualname = qualname;
    op->func_doc = NULL;
    op->func_classobj = NULL;
    op->func_globals = globals;
    Py_INCREF(op->func_globals);
    Py_XINCREF(code);
    op->func_code = code;
    op->defaults_pyobjects = 0;
    op->defaults = NULL;
    op->defaults_tuple = NULL;
    op->defaults_kwdict = NULL;
    op->defaults_getter = NULL;
    op->func_annotations = NULL;
    PyObject_GC_Track(op);
    return (PyObject *) op;
}
static int
__Pyx_CyFunction_clear(__pyx_CyFunctionObject *m)
{
    Py_CLEAR(m->func_closure);
    Py_CLEAR(m->func.m_module);
    Py_CLEAR(m->func_dict);
    Py_CLEAR(m->func_name);
    Py_CLEAR(m->func_qualname);
    Py_CLEAR(m->func_doc);
    Py_CLEAR(m->func_globals);
    Py_CLEAR(m->func_code);
    Py_CLEAR(m->func_classobj);
    Py_CLEAR(m->defaults_tuple);
    Py_CLEAR(m->defaults_kwdict);
    Py_CLEAR(m->func_annotations);
    if (m->defaults) {
        PyObject **pydefaults = __Pyx_CyFunction_Defaults(PyObject *, m);
        int i;
        for (i = 0; i < m->defaults_pyobjects; i++)
            Py_XDECREF(pydefaults[i]);
        PyObject_Free(m->defaults);
        m->defaults = NULL;
    }
    return 0;
}
static void __Pyx__CyFunction_dealloc(__pyx_CyFunctionObject *m)
{
    if (__Pyx_CyFunction_weakreflist(m) != NULL)
        PyObject_ClearWeakRefs((PyObject *) m);
    __Pyx_CyFunction_clear(m);
    PyObject_GC_Del(m);
}
static void __Pyx_CyFunction_dealloc(__pyx_CyFunctionObject *m)
{
    PyObject_GC_UnTrack(m);
    __Pyx__CyFunction_dealloc(m);
}
static int __Pyx_CyFunction_traverse(__pyx_CyFunctionObject *m, visitproc visit, void *arg)
{
    Py_VISIT(m->func_closure);
    Py_VISIT(m->func.m_module);
    Py_VISIT(m->func_dict);
    Py_VISIT(m->func_name);
    Py_VISIT(m->func_qualname);
    Py_VISIT(m->func_doc);
    Py_VISIT(m->func_globals);
    Py_VISIT(m->func_code);
    Py_VISIT(m->func_classobj);
    Py_VISIT(m->defaults_tuple);
    Py_VISIT(m->defaults_kwdict);
    if (m->defaults) {
        PyObject **pydefaults = __Pyx_CyFunction_Defaults(PyObject *, m);
        int i;
        for (i = 0; i < m->defaults_pyobjects; i++)
            Py_VISIT(pydefaults[i]);
    }
    return 0;
}
static PyObject *__Pyx_CyFunction_descr_get(PyObject *func, PyObject *obj, PyObject *type)
{
    __pyx_CyFunctionObject *m = (__pyx_CyFunctionObject *) func;
    if (m->flags & __Pyx_CYFUNCTION_STATICMETHOD) {
        Py_INCREF(func);
        return func;
    }
    if (m->flags & __Pyx_CYFUNCTION_CLASSMETHOD) {
        if (type == NULL)
            type = (PyObject *)(Py_TYPE(obj));
        return __Pyx_PyMethod_New(func, type, (PyObject *)(Py_TYPE(type)));
    }
    if (obj == Py_None)
        obj = NULL;
    return __Pyx_PyMethod_New(func, obj, type);
}
static PyObject*
__Pyx_CyFunction_repr(__pyx_CyFunctionObject *op)
{
#if PY_MAJOR_VERSION >= 3
    return PyUnicode_FromFormat("<cyfunction %U at %p>",
                                op->func_qualname, (void *)op);
#else
    return PyString_FromFormat("<cyfunction %s at %p>",
                               PyString_AsString(op->func_qualname), (void *)op);
#endif
}
static PyObject * __Pyx_CyFunction_CallMethod(PyObject *func, PyObject *self, PyObject *arg, PyObject *kw) {
    PyCFunctionObject* f = (PyCFunctionObject*)func;
    PyCFunction meth = f->m_ml->ml_meth;
    Py_ssize_t size;
    switch (f->m_ml->ml_flags & (METH_VARARGS | METH_KEYWORDS | METH_NOARGS | METH_O)) {
    case METH_VARARGS:
        if (likely(kw == NULL || PyDict_Size(kw) == 0))
            return (*meth)(self, arg);
        break;
    case METH_VARARGS | METH_KEYWORDS:
        return (*(PyCFunctionWithKeywords)meth)(self, arg, kw);
    case METH_NOARGS:
        if (likely(kw == NULL || PyDict_Size(kw) == 0)) {
            size = PyTuple_GET_SIZE(arg);
            if (likely(size == 0))
                return (*meth)(self, NULL);
            PyErr_Format(PyExc_TypeError,
                "%.200s() takes no arguments (%" CYTHON_FORMAT_SSIZE_T "d given)",
                f->m_ml->ml_name, size);
            return NULL;
        }
        break;
    case METH_O:
        if (likely(kw == NULL || PyDict_Size(kw) == 0)) {
            size = PyTuple_GET_SIZE(arg);
            if (likely(size == 1)) {
                PyObject *result, *arg0;
                #if CYTHON_ASSUME_SAFE_MACROS && !CYTHON_AVOID_BORROWED_REFS
                arg0 = PyTuple_GET_ITEM(arg, 0);
                #else
                arg0 = PySequence_ITEM(arg, 0); if (unlikely(!arg0)) return NULL;
                #endif
                result = (*meth)(self, arg0);
                #if !(CYTHON_ASSUME_SAFE_MACROS && !CYTHON_AVOID_BORROWED_REFS)
                Py_DECREF(arg0);
                #endif
                return result;
            }
            PyErr_Format(PyExc_TypeError,
                "%.200s() takes exactly one argument (%" CYTHON_FORMAT_SSIZE_T "d given)",
                f->m_ml->ml_name, size);
            return NULL;
        }
        break;
    default:
        PyErr_SetString(PyExc_SystemError, "Bad call flags in "
                        "__Pyx_CyFunction_Call. METH_OLDARGS is no "
                        "longer supported!");
        return NULL;
    }
    PyErr_Format(PyExc_TypeError, "%.200s() takes no keyword arguments",
                 f->m_ml->ml_name);
    return NULL;
}
static CYTHON_INLINE PyObject *__Pyx_CyFunction_Call(PyObject *func, PyObject *arg, PyObject *kw) {
    return __Pyx_CyFunction_CallMethod(func, ((PyCFunctionObject*)func)->m_self, arg, kw);
}
static PyObject *__Pyx_CyFunction_CallAsMethod(PyObject *func, PyObject *args, PyObject *kw) {
    PyObject *result;
    __pyx_CyFunctionObject *cyfunc = (__pyx_CyFunctionObject *) func;
    if ((cyfunc->flags & __Pyx_CYFUNCTION_CCLASS) && !(cyfunc->flags & __Pyx_CYFUNCTION_STATICMETHOD)) {
        Py_ssize_t argc;
        PyObject *new_args;
        PyObject *self;
        argc = PyTuple_GET_SIZE(args);
        new_args = PyTuple_GetSlice(args, 1, argc);
        if (unlikely(!new_args))
            return NULL;
        self = PyTuple_GetItem(args, 0);
        if (unlikely(!self)) {
            Py_DECREF(new_args);
            return NULL;
        }
        result = __Pyx_CyFunction_CallMethod(func, self, new_args, kw);
        Py_DECREF(new_args);
    } else {
        result = __Pyx_CyFunction_Call(func, args, kw);
    }
    return result;
}
static PyTypeObject __pyx_CyFunctionType_type = {
    PyVarObject_HEAD_INIT(0, 0)
    "cython_function_or_method",
    sizeof(__pyx_CyFunctionObject),
    0,
    (destructor) __Pyx_CyFunction_dealloc,
    0,
    0,
    0,
#if PY_MAJOR_VERSION < 3
    0,
#else
    0,
#endif
    (reprfunc) __Pyx_CyFunction_repr,
    0,
    0,
    0,
    0,
    __Pyx_CyFunction_CallAsMethod,
    0,
    0,
    0,
    0,
    Py_TPFLAGS_DEFAULT | Py_TPFLAGS_HAVE_GC,
    0,
    (traverseproc) __Pyx_CyFunction_traverse,
    (inquiry) __Pyx_CyFunction_clear,
    0,
#if PY_VERSION_HEX < 0x030500A0
    offsetof(__pyx_CyFunctionObject, func_weakreflist),
#else
    offsetof(PyCFunctionObject, m_weakreflist),
#endif
    0,
    0,
    __pyx_CyFunction_methods,
    __pyx_CyFunction_members,
    __pyx_CyFunction_getsets,
    0,
    0,
    __Pyx_CyFunction_descr_get,
    0,
    offsetof(__pyx_CyFunctionObject, func_dict),
    0,
    0,
    0,
    0,
    0,
    0,
    0,
    0,
    0,
    0,
    0,
    0,
#if PY_VERSION_HEX >= 0x030400a1
    0,
#endif
};
static int __pyx_CyFunction_init(void) {
    __pyx_CyFunctionType = __Pyx_FetchCommonType(&__pyx_CyFunctionType_type);
    if (unlikely(__pyx_CyFunctionType == NULL)) {
        return -1;
    }
    return 0;
}
static CYTHON_INLINE void *__Pyx_CyFunction_InitDefaults(PyObject *func, size_t size, int pyobjects) {
    __pyx_CyFunctionObject *m = (__pyx_CyFunctionObject *) func;
    m->defaults = PyObject_Malloc(size);
    if (unlikely(!m->defaults))
        return PyErr_NoMemory();
    memset(m->defaults, 0, size);
    m->defaults_pyobjects = pyobjects;
    return m->defaults;
}
static CYTHON_INLINE void __Pyx_CyFunction_SetDefaultsTuple(PyObject *func, PyObject *tuple) {
    __pyx_CyFunctionObject *m = (__pyx_CyFunctionObject *) func;
    m->defaults_tuple = tuple;
    Py_INCREF(tuple);
}
static CYTHON_INLINE void __Pyx_CyFunction_SetDefaultsKwDict(PyObject *func, PyObject *dict) {
    __pyx_CyFunctionObject *m = (__pyx_CyFunctionObject *) func;
    m->defaults_kwdict = dict;
    Py_INCREF(dict);
}
static CYTHON_INLINE void __Pyx_CyFunction_SetAnnotationsDict(PyObject *func, PyObject *dict) {
    __pyx_CyFunctionObject *m = (__pyx_CyFunctionObject *) func;
    m->func_annotations = dict;
    Py_INCREF(dict);
}

/* set_iter */
            static CYTHON_INLINE PyObject* __Pyx_set_iterator(PyObject* iterable, int is_set,
                                                  Py_ssize_t* p_orig_length, int* p_source_is_set) {
#if CYTHON_COMPILING_IN_CPYTHON
    is_set = is_set || likely(PySet_CheckExact(iterable) || PyFrozenSet_CheckExact(iterable));
    *p_source_is_set = is_set;
    if (likely(is_set)) {
        *p_orig_length = PySet_Size(iterable);
        Py_INCREF(iterable);
        return iterable;
    }
#else
    (void)is_set;
    *p_source_is_set = 0;
#endif
    *p_orig_length = 0;
    return PyObject_GetIter(iterable);
}
static CYTHON_INLINE int __Pyx_set_iter_next(
        PyObject* iter_obj, Py_ssize_t orig_length,
        Py_ssize_t* ppos, PyObject **value,
        int source_is_set) {
    if (!CYTHON_COMPILING_IN_CPYTHON || unlikely(!source_is_set)) {
        *value = PyIter_Next(iter_obj);
        if (unlikely(!*value)) {
            return __Pyx_IterFinish();
        }
        (void)orig_length;
        (void)ppos;
        return 1;
    }
#if CYTHON_COMPILING_IN_CPYTHON
    if (unlikely(PySet_GET_SIZE(iter_obj) != orig_length)) {
        PyErr_SetString(
            PyExc_RuntimeError,
            "set changed size during iteration");
        return -1;
    }
    {
        Py_hash_t hash;
        int ret = _PySet_NextEntry(iter_obj, ppos, value, &hash);
        assert (ret != -1);
        if (likely(ret)) {
            Py_INCREF(*value);
            return 1;
        }
    }
#endif
    return 0;
}

/* CallUnboundCMethod0 */
            static PyObject* __Pyx__CallUnboundCMethod0(__Pyx_CachedCFunction* cfunc, PyObject* self) {
    PyObject *args, *result = NULL;
    if (unlikely(!cfunc->method) && unlikely(__Pyx_TryUnpackUnboundCMethod(cfunc) < 0)) return NULL;
#if CYTHON_ASSUME_SAFE_MACROS
    args = PyTuple_New(1);
    if (unlikely(!args)) goto bad;
    Py_INCREF(self);
    PyTuple_SET_ITEM(args, 0, self);
#else
    args = PyTuple_Pack(1, self);
    if (unlikely(!args)) goto bad;
#endif
    result = __Pyx_PyObject_Call(cfunc->method, args, NULL);
    Py_DECREF(args);
bad:
    return result;
}

/* py_dict_values */
            static CYTHON_INLINE PyObject* __Pyx_PyDict_Values(PyObject* d) {
    if (PY_MAJOR_VERSION >= 3)
        return __Pyx_CallUnboundCMethod0(&__pyx_umethod_PyDict_Type_values, d);
    else
        return PyDict_Values(d);
}

/* PyFloatBinop */
            #if !CYTHON_COMPILING_IN_PYPY
static PyObject* __Pyx_PyFloat_AddObjC(PyObject *op1, PyObject *op2, double floatval, CYTHON_UNUSED int inplace) {
    const double b = floatval;
    double a, result;
    if (likely(PyFloat_CheckExact(op1))) {
        a = PyFloat_AS_DOUBLE(op1);
    } else
    #if PY_MAJOR_VERSION < 3
    if (likely(PyInt_CheckExact(op1))) {
        a = (double) PyInt_AS_LONG(op1);
    } else
    #endif
    if (likely(PyLong_CheckExact(op1))) {
        #if CYTHON_USE_PYLONG_INTERNALS
        const digit* digits = ((PyLongObject*)op1)->ob_digit;
        const Py_ssize_t size = Py_SIZE(op1);
        switch (size) {
            case  0: a = 0.0; break;
            case -1: a = -(double) digits[0]; break;
            case  1: a = (double) digits[0]; break;
            case -2:
            case 2:
                if (8 * sizeof(unsigned long) > 2 * PyLong_SHIFT && ((8 * sizeof(unsigned long) < 53) || (1 * PyLong_SHIFT < 53))) {
                    a = (double) (((((unsigned long)digits[1]) << PyLong_SHIFT) | (unsigned long)digits[0]));
                    if ((8 * sizeof(unsigned long) < 53) || (2 * PyLong_SHIFT < 53) || (a < (double) ((PY_LONG_LONG)1 << 53))) {
                        if (size == -2)
                            a = -a;
                        break;
                    }
                }
                CYTHON_FALLTHROUGH;
            case -3:
            case 3:
                if (8 * sizeof(unsigned long) > 3 * PyLong_SHIFT && ((8 * sizeof(unsigned long) < 53) || (2 * PyLong_SHIFT < 53))) {
                    a = (double) (((((((unsigned long)digits[2]) << PyLong_SHIFT) | (unsigned long)digits[1]) << PyLong_SHIFT) | (unsigned long)digits[0]));
                    if ((8 * sizeof(unsigned long) < 53) || (3 * PyLong_SHIFT < 53) || (a < (double) ((PY_LONG_LONG)1 << 53))) {
                        if (size == -3)
                            a = -a;
                        break;
                    }
                }
                CYTHON_FALLTHROUGH;
            case -4:
            case 4:
                if (8 * sizeof(unsigned long) > 4 * PyLong_SHIFT && ((8 * sizeof(unsigned long) < 53) || (3 * PyLong_SHIFT < 53))) {
                    a = (double) (((((((((unsigned long)digits[3]) << PyLong_SHIFT) | (unsigned long)digits[2]) << PyLong_SHIFT) | (unsigned long)digits[1]) << PyLong_SHIFT) | (unsigned long)digits[0]));
                    if ((8 * sizeof(unsigned long) < 53) || (4 * PyLong_SHIFT < 53) || (a < (double) ((PY_LONG_LONG)1 << 53))) {
                        if (size == -4)
                            a = -a;
                        break;
                    }
                }
                CYTHON_FALLTHROUGH;
            default:
        #else
        {
        #endif
            a = PyLong_AsDouble(op1);
            if (unlikely(a == -1.0 && PyErr_Occurred())) return NULL;
        }
    } else {
        return (inplace ? PyNumber_InPlaceAdd : PyNumber_Add)(op1, op2);
    }
        PyFPE_START_PROTECT("add", return NULL)
        result = a + b;
        PyFPE_END_PROTECT(result)
        return PyFloat_FromDouble(result);
}
#endif

/* IterNext */
              static PyObject *__Pyx_PyIter_Next2Default(PyObject* defval) {
    PyObject* exc_type;
    __Pyx_PyThreadState_declare
    __Pyx_PyThreadState_assign
    exc_type = __Pyx_PyErr_Occurred();
    if (unlikely(exc_type)) {
        if (!defval || unlikely(!__Pyx_PyErr_GivenExceptionMatches(exc_type, PyExc_StopIteration)))
            return NULL;
        __Pyx_PyErr_Clear();
        Py_INCREF(defval);
        return defval;
    }
    if (defval) {
        Py_INCREF(defval);
        return defval;
    }
    __Pyx_PyErr_SetNone(PyExc_StopIteration);
    return NULL;
}
static void __Pyx_PyIter_Next_ErrorNoIterator(PyObject *iterator) {
    PyErr_Format(PyExc_TypeError,
        "%.200s object is not an iterator", Py_TYPE(iterator)->tp_name);
}
static CYTHON_INLINE PyObject *__Pyx_PyIter_Next2(PyObject* iterator, PyObject* defval) {
    PyObject* next;
    iternextfunc iternext = Py_TYPE(iterator)->tp_iternext;
    if (likely(iternext)) {
#if CYTHON_USE_TYPE_SLOTS
        next = iternext(iterator);
        if (likely(next))
            return next;
        #if PY_VERSION_HEX >= 0x02070000
        if (unlikely(iternext == &_PyObject_NextNotImplemented))
            return NULL;
        #endif
#else
        next = PyIter_Next(iterator);
        if (likely(next))
            return next;
#endif
    } else if (CYTHON_USE_TYPE_SLOTS || unlikely(!PyIter_Check(iterator))) {
        __Pyx_PyIter_Next_ErrorNoIterator(iterator);
        return NULL;
    }
#if !CYTHON_USE_TYPE_SLOTS
    else {
        next = PyIter_Next(iterator);
        if (likely(next))
            return next;
    }
#endif
    return __Pyx_PyIter_Next2Default(defval);
}

/* GetAttr3 */
              static PyObject *__Pyx_GetAttr3Default(PyObject *d) {
    __Pyx_PyThreadState_declare
    __Pyx_PyThreadState_assign
    if (unlikely(!__Pyx_PyErr_ExceptionMatches(PyExc_AttributeError)))
        return NULL;
    __Pyx_PyErr_Clear();
    Py_INCREF(d);
    return d;
}
static CYTHON_INLINE PyObject *__Pyx_GetAttr3(PyObject *o, PyObject *n, PyObject *d) {
    PyObject *r = __Pyx_GetAttr(o, n);
    return (likely(r)) ? r : __Pyx_GetAttr3Default(d);
}

/* PyIntBinop */
              #if !CYTHON_COMPILING_IN_PYPY
static PyObject* __Pyx_PyInt_EqObjC(PyObject *op1, PyObject *op2, CYTHON_UNUSED long intval, CYTHON_UNUSED int inplace) {
    if (op1 == op2) {
        Py_RETURN_TRUE;
    }
    #if PY_MAJOR_VERSION < 3
    if (likely(PyInt_CheckExact(op1))) {
        const long b = intval;
        long a = PyInt_AS_LONG(op1);
        if (a == b) {
            Py_RETURN_TRUE;
        } else {
            Py_RETURN_FALSE;
        }
    }
    #endif
    #if CYTHON_USE_PYLONG_INTERNALS
    if (likely(PyLong_CheckExact(op1))) {
        const long b = intval;
        long a;
        const digit* digits = ((PyLongObject*)op1)->ob_digit;
        const Py_ssize_t size = Py_SIZE(op1);
        if (likely(__Pyx_sst_abs(size) <= 1)) {
            a = likely(size) ? digits[0] : 0;
            if (size == -1) a = -a;
        } else {
            switch (size) {
                case -2:
                    if (8 * sizeof(long) - 1 > 2 * PyLong_SHIFT) {
                        a = -(long) (((((unsigned long)digits[1]) << PyLong_SHIFT) | (unsigned long)digits[0]));
                        break;
                    }
                    CYTHON_FALLTHROUGH;
                case 2:
                    if (8 * sizeof(long) - 1 > 2 * PyLong_SHIFT) {
                        a = (long) (((((unsigned long)digits[1]) << PyLong_SHIFT) | (unsigned long)digits[0]));
                        break;
                    }
                    CYTHON_FALLTHROUGH;
                case -3:
                    if (8 * sizeof(long) - 1 > 3 * PyLong_SHIFT) {
                        a = -(long) (((((((unsigned long)digits[2]) << PyLong_SHIFT) | (unsigned long)digits[1]) << PyLong_SHIFT) | (unsigned long)digits[0]));
                        break;
                    }
                    CYTHON_FALLTHROUGH;
                case 3:
                    if (8 * sizeof(long) - 1 > 3 * PyLong_SHIFT) {
                        a = (long) (((((((unsigned long)digits[2]) << PyLong_SHIFT) | (unsigned long)digits[1]) << PyLong_SHIFT) | (unsigned long)digits[0]));
                        break;
                    }
                    CYTHON_FALLTHROUGH;
                case -4:
                    if (8 * sizeof(long) - 1 > 4 * PyLong_SHIFT) {
                        a = -(long) (((((((((unsigned long)digits[3]) << PyLong_SHIFT) | (unsigned long)digits[2]) << PyLong_SHIFT) | (unsigned long)digits[1]) << PyLong_SHIFT) | (unsigned long)digits[0]));
                        break;
                    }
                    CYTHON_FALLTHROUGH;
                case 4:
                    if (8 * sizeof(long) - 1 > 4 * PyLong_SHIFT) {
                        a = (long) (((((((((unsigned long)digits[3]) << PyLong_SHIFT) | (unsigned long)digits[2]) << PyLong_SHIFT) | (unsigned long)digits[1]) << PyLong_SHIFT) | (unsigned long)digits[0]));
                        break;
                    }
                    CYTHON_FALLTHROUGH;
                #if PyLong_SHIFT < 30 && PyLong_SHIFT != 15
                default: return PyLong_Type.tp_richcompare(op1, op2, Py_EQ);
                #else
                default: Py_RETURN_FALSE;
                #endif
            }
        }
            if (a == b) {
                Py_RETURN_TRUE;
            } else {
                Py_RETURN_FALSE;
            }
    }
    #endif
    if (PyFloat_CheckExact(op1)) {
        const long b = intval;
        double a = PyFloat_AS_DOUBLE(op1);
            if ((double)a == (double)b) {
                Py_RETURN_TRUE;
            } else {
                Py_RETURN_FALSE;
            }
    }
    return PyObject_RichCompare(op1, op2, Py_EQ);
}
#endif

/* BytesEquals */
              static CYTHON_INLINE int __Pyx_PyBytes_Equals(PyObject* s1, PyObject* s2, int equals) {
#if CYTHON_COMPILING_IN_PYPY
    return PyObject_RichCompareBool(s1, s2, equals);
#else
    if (s1 == s2) {
        return (equals == Py_EQ);
    } else if (PyBytes_CheckExact(s1) & PyBytes_CheckExact(s2)) {
        const char *ps1, *ps2;
        Py_ssize_t length = PyBytes_GET_SIZE(s1);
        if (length != PyBytes_GET_SIZE(s2))
            return (equals == Py_NE);
        ps1 = PyBytes_AS_STRING(s1);
        ps2 = PyBytes_AS_STRING(s2);
        if (ps1[0] != ps2[0]) {
            return (equals == Py_NE);
        } else if (length == 1) {
            return (equals == Py_EQ);
        } else {
            int result;
#if CYTHON_USE_UNICODE_INTERNALS
            Py_hash_t hash1, hash2;
            hash1 = ((PyBytesObject*)s1)->ob_shash;
            hash2 = ((PyBytesObject*)s2)->ob_shash;
            if (hash1 != hash2 && hash1 != -1 && hash2 != -1) {
                return (equals == Py_NE);
            }
#endif
            result = memcmp(ps1, ps2, (size_t)length);
            return (equals == Py_EQ) ? (result == 0) : (result != 0);
        }
    } else if ((s1 == Py_None) & PyBytes_CheckExact(s2)) {
        return (equals == Py_NE);
    } else if ((s2 == Py_None) & PyBytes_CheckExact(s1)) {
        return (equals == Py_NE);
    } else {
        int result;
        PyObject* py_result = PyObject_RichCompare(s1, s2, equals);
        if (!py_result)
            return -1;
        result = __Pyx_PyObject_IsTrue(py_result);
        Py_DECREF(py_result);
        return result;
    }
#endif
}

/* SliceObject */
              static CYTHON_INLINE PyObject* __Pyx_PyObject_GetSlice(PyObject* obj,
        Py_ssize_t cstart, Py_ssize_t cstop,
        PyObject** _py_start, PyObject** _py_stop, PyObject** _py_slice,
        int has_cstart, int has_cstop, CYTHON_UNUSED int wraparound) {
#if CYTHON_USE_TYPE_SLOTS
    PyMappingMethods* mp;
#if PY_MAJOR_VERSION < 3
    PySequenceMethods* ms = Py_TYPE(obj)->tp_as_sequence;
    if (likely(ms && ms->sq_slice)) {
        if (!has_cstart) {
            if (_py_start && (*_py_start != Py_None)) {
                cstart = __Pyx_PyIndex_AsSsize_t(*_py_start);
                if ((cstart == (Py_ssize_t)-1) && PyErr_Occurred()) goto bad;
            } else
                cstart = 0;
        }
        if (!has_cstop) {
            if (_py_stop && (*_py_stop != Py_None)) {
                cstop = __Pyx_PyIndex_AsSsize_t(*_py_stop);
                if ((cstop == (Py_ssize_t)-1) && PyErr_Occurred()) goto bad;
            } else
                cstop = PY_SSIZE_T_MAX;
        }
        if (wraparound && unlikely((cstart < 0) | (cstop < 0)) && likely(ms->sq_length)) {
            Py_ssize_t l = ms->sq_length(obj);
            if (likely(l >= 0)) {
                if (cstop < 0) {
                    cstop += l;
                    if (cstop < 0) cstop = 0;
                }
                if (cstart < 0) {
                    cstart += l;
                    if (cstart < 0) cstart = 0;
                }
            } else {
                if (!PyErr_ExceptionMatches(PyExc_OverflowError))
                    goto bad;
                PyErr_Clear();
            }
        }
        return ms->sq_slice(obj, cstart, cstop);
    }
#endif
    mp = Py_TYPE(obj)->tp_as_mapping;
    if (likely(mp && mp->mp_subscript))
#endif
    {
        PyObject* result;
        PyObject *py_slice, *py_start, *py_stop;
        if (_py_slice) {
            py_slice = *_py_slice;
        } else {
            PyObject* owned_start = NULL;
            PyObject* owned_stop = NULL;
            if (_py_start) {
                py_start = *_py_start;
            } else {
                if (has_cstart) {
                    owned_start = py_start = PyInt_FromSsize_t(cstart);
                    if (unlikely(!py_start)) goto bad;
                } else
                    py_start = Py_None;
            }
            if (_py_stop) {
                py_stop = *_py_stop;
            } else {
                if (has_cstop) {
                    owned_stop = py_stop = PyInt_FromSsize_t(cstop);
                    if (unlikely(!py_stop)) {
                        Py_XDECREF(owned_start);
                        goto bad;
                    }
                } else
                    py_stop = Py_None;
            }
            py_slice = PySlice_New(py_start, py_stop, Py_None);
            Py_XDECREF(owned_start);
            Py_XDECREF(owned_stop);
            if (unlikely(!py_slice)) goto bad;
        }
#if CYTHON_USE_TYPE_SLOTS
        result = mp->mp_subscript(obj, py_slice);
#else
        result = PyObject_GetItem(obj, py_slice);
#endif
        if (!_py_slice) {
            Py_DECREF(py_slice);
        }
        return result;
    }
    PyErr_Format(PyExc_TypeError,
        "'%.200s' object is unsliceable", Py_TYPE(obj)->tp_name);
bad:
    return NULL;
}

/* DictGetItem */
              #if PY_MAJOR_VERSION >= 3 && !CYTHON_COMPILING_IN_PYPY
static PyObject *__Pyx_PyDict_GetItem(PyObject *d, PyObject* key) {
    PyObject *value;
    value = PyDict_GetItemWithError(d, key);
    if (unlikely(!value)) {
        if (!PyErr_Occurred()) {
            PyObject* args = PyTuple_Pack(1, key);
            if (likely(args))
                PyErr_SetObject(PyExc_KeyError, args);
            Py_XDECREF(args);
        }
        return NULL;
    }
    Py_INCREF(value);
    return value;
}
#endif

/* StringJoin */
              #if !CYTHON_COMPILING_IN_CPYTHON
static CYTHON_INLINE PyObject* __Pyx_PyBytes_Join(PyObject* sep, PyObject* values) {
    return PyObject_CallMethodObjArgs(sep, __pyx_n_s_join, values, NULL);
}
#endif

/* SetItemInt */
              static int __Pyx_SetItemInt_Generic(PyObject *o, PyObject *j, PyObject *v) {
    int r;
    if (!j) return -1;
    r = PyObject_SetItem(o, j, v);
    Py_DECREF(j);
    return r;
}
static CYTHON_INLINE int __Pyx_SetItemInt_Fast(PyObject *o, Py_ssize_t i, PyObject *v, int is_list,
                                               CYTHON_NCP_UNUSED int wraparound, CYTHON_NCP_UNUSED int boundscheck) {
#if CYTHON_ASSUME_SAFE_MACROS && !CYTHON_AVOID_BORROWED_REFS && CYTHON_USE_TYPE_SLOTS
    if (is_list || PyList_CheckExact(o)) {
        Py_ssize_t n = (!wraparound) ? i : ((likely(i >= 0)) ? i : i + PyList_GET_SIZE(o));
        if ((!boundscheck) || likely((n >= 0) & (n < PyList_GET_SIZE(o)))) {
            PyObject* old = PyList_GET_ITEM(o, n);
            Py_INCREF(v);
            PyList_SET_ITEM(o, n, v);
            Py_DECREF(old);
            return 1;
        }
    } else {
        PySequenceMethods *m = Py_TYPE(o)->tp_as_sequence;
        if (likely(m && m->sq_ass_item)) {
            if (wraparound && unlikely(i < 0) && likely(m->sq_length)) {
                Py_ssize_t l = m->sq_length(o);
                if (likely(l >= 0)) {
                    i += l;
                } else {
                    if (!PyErr_ExceptionMatches(PyExc_OverflowError))
                        return -1;
                    PyErr_Clear();
                }
            }
            return m->sq_ass_item(o, i, v);
        }
    }
#else
#if CYTHON_COMPILING_IN_PYPY
    if (is_list || (PySequence_Check(o) && !PyDict_Check(o))) {
#else
    if (is_list || PySequence_Check(o)) {
#endif
        return PySequence_SetItem(o, i, v);
    }
#endif
    return __Pyx_SetItemInt_Generic(o, PyInt_FromSsize_t(i), v);
}

/* FastTypeChecks */
                #if CYTHON_COMPILING_IN_CPYTHON
static int __Pyx_InBases(PyTypeObject *a, PyTypeObject *b) {
    while (a) {
        a = a->tp_base;
        if (a == b)
            return 1;
    }
    return b == &PyBaseObject_Type;
}
static CYTHON_INLINE int __Pyx_IsSubtype(PyTypeObject *a, PyTypeObject *b) {
    PyObject *mro;
    if (a == b) return 1;
    mro = a->tp_mro;
    if (likely(mro)) {
        Py_ssize_t i, n;
        n = PyTuple_GET_SIZE(mro);
        for (i = 0; i < n; i++) {
            if (PyTuple_GET_ITEM(mro, i) == (PyObject *)b)
                return 1;
        }
        return 0;
    }
    return __Pyx_InBases(a, b);
}
#if PY_MAJOR_VERSION == 2
static int __Pyx_inner_PyErr_GivenExceptionMatches2(PyObject *err, PyObject* exc_type1, PyObject* exc_type2) {
    PyObject *exception, *value, *tb;
    int res;
    __Pyx_PyThreadState_declare
    __Pyx_PyThreadState_assign
    __Pyx_ErrFetch(&exception, &value, &tb);
    res = exc_type1 ? PyObject_IsSubclass(err, exc_type1) : 0;
    if (unlikely(res == -1)) {
        PyErr_WriteUnraisable(err);
        res = 0;
    }
    if (!res) {
        res = PyObject_IsSubclass(err, exc_type2);
        if (unlikely(res == -1)) {
            PyErr_WriteUnraisable(err);
            res = 0;
        }
    }
    __Pyx_ErrRestore(exception, value, tb);
    return res;
}
#else
static CYTHON_INLINE int __Pyx_inner_PyErr_GivenExceptionMatches2(PyObject *err, PyObject* exc_type1, PyObject *exc_type2) {
    int res = exc_type1 ? __Pyx_IsSubtype((PyTypeObject*)err, (PyTypeObject*)exc_type1) : 0;
    if (!res) {
        res = __Pyx_IsSubtype((PyTypeObject*)err, (PyTypeObject*)exc_type2);
    }
    return res;
}
#endif
static int __Pyx_PyErr_GivenExceptionMatchesTuple(PyObject *exc_type, PyObject *tuple) {
    Py_ssize_t i, n;
    assert(PyExceptionClass_Check(exc_type));
    n = PyTuple_GET_SIZE(tuple);
#if PY_MAJOR_VERSION >= 3
    for (i=0; i<n; i++) {
        if (exc_type == PyTuple_GET_ITEM(tuple, i)) return 1;
    }
#endif
    for (i=0; i<n; i++) {
        PyObject *t = PyTuple_GET_ITEM(tuple, i);
        #if PY_MAJOR_VERSION < 3
        if (likely(exc_type == t)) return 1;
        #endif
        if (likely(PyExceptionClass_Check(t))) {
            if (__Pyx_inner_PyErr_GivenExceptionMatches2(exc_type, NULL, t)) return 1;
        } else {
        }
    }
    return 0;
}
static CYTHON_INLINE int __Pyx_PyErr_GivenExceptionMatches(PyObject *err, PyObject* exc_type) {
    if (likely(err == exc_type)) return 1;
    if (likely(PyExceptionClass_Check(err))) {
        if (likely(PyExceptionClass_Check(exc_type))) {
            return __Pyx_inner_PyErr_GivenExceptionMatches2(err, NULL, exc_type);
        } else if (likely(PyTuple_Check(exc_type))) {
            return __Pyx_PyErr_GivenExceptionMatchesTuple(err, exc_type);
        } else {
        }
    }
    return PyErr_GivenExceptionMatches(err, exc_type);
}
static CYTHON_INLINE int __Pyx_PyErr_GivenExceptionMatches2(PyObject *err, PyObject *exc_type1, PyObject *exc_type2) {
    assert(PyExceptionClass_Check(exc_type1));
    assert(PyExceptionClass_Check(exc_type2));
    if (likely(err == exc_type1 || err == exc_type2)) return 1;
    if (likely(PyExceptionClass_Check(err))) {
        return __Pyx_inner_PyErr_GivenExceptionMatches2(err, exc_type1, exc_type2);
    }
    return (PyErr_GivenExceptionMatches(err, exc_type1) || PyErr_GivenExceptionMatches(err, exc_type2));
}
#endif

/* Import */
                static PyObject *__Pyx_Import(PyObject *name, PyObject *from_list, int level) {
    PyObject *empty_list = 0;
    PyObject *module = 0;
    PyObject *global_dict = 0;
    PyObject *empty_dict = 0;
    PyObject *list;
    #if PY_MAJOR_VERSION < 3
    PyObject *py_import;
    py_import = __Pyx_PyObject_GetAttrStr(__pyx_b, __pyx_n_s_import);
    if (!py_import)
        goto bad;
    #endif
    if (from_list)
        list = from_list;
    else {
        empty_list = PyList_New(0);
        if (!empty_list)
            goto bad;
        list = empty_list;
    }
    global_dict = PyModule_GetDict(__pyx_m);
    if (!global_dict)
        goto bad;
    empty_dict = PyDict_New();
    if (!empty_dict)
        goto bad;
    {
        #if PY_MAJOR_VERSION >= 3
        if (level == -1) {
            if (strchr(__Pyx_MODULE_NAME, '.')) {
                module = PyImport_ImportModuleLevelObject(
                    name, global_dict, empty_dict, list, 1);
                if (!module) {
                    if (!PyErr_ExceptionMatches(PyExc_ImportError))
                        goto bad;
                    PyErr_Clear();
                }
            }
            level = 0;
        }
        #endif
        if (!module) {
            #if PY_MAJOR_VERSION < 3
            PyObject *py_level = PyInt_FromLong(level);
            if (!py_level)
                goto bad;
            module = PyObject_CallFunctionObjArgs(py_import,
                name, global_dict, empty_dict, list, py_level, NULL);
            Py_DECREF(py_level);
            #else
            module = PyImport_ImportModuleLevelObject(
                name, global_dict, empty_dict, list, level);
            #endif
        }
    }
bad:
    #if PY_MAJOR_VERSION < 3
    Py_XDECREF(py_import);
    #endif
    Py_XDECREF(empty_list);
    Py_XDECREF(empty_dict);
    return module;
}

/* PyObjectSetAttrStr */
                #if CYTHON_USE_TYPE_SLOTS
static CYTHON_INLINE int __Pyx_PyObject_SetAttrStr(PyObject* obj, PyObject* attr_name, PyObject* value) {
    PyTypeObject* tp = Py_TYPE(obj);
    if (likely(tp->tp_setattro))
        return tp->tp_setattro(obj, attr_name, value);
#if PY_MAJOR_VERSION < 3
    if (likely(tp->tp_setattr))
        return tp->tp_setattr(obj, PyString_AS_STRING(attr_name), value);
#endif
    return PyObject_SetAttr(obj, attr_name, value);
}
#endif

/* SwapException */
                #if CYTHON_FAST_THREAD_STATE
static CYTHON_INLINE void __Pyx__ExceptionSwap(PyThreadState *tstate, PyObject **type, PyObject **value, PyObject **tb) {
    PyObject *tmp_type, *tmp_value, *tmp_tb;
    #if PY_VERSION_HEX >= 0x030700A3
    tmp_type = tstate->exc_state.exc_type;
    tmp_value = tstate->exc_state.exc_value;
    tmp_tb = tstate->exc_state.exc_traceback;
    tstate->exc_state.exc_type = *type;
    tstate->exc_state.exc_value = *value;
    tstate->exc_state.exc_traceback = *tb;
    #else
    tmp_type = tstate->exc_type;
    tmp_value = tstate->exc_value;
    tmp_tb = tstate->exc_traceback;
    tstate->exc_type = *type;
    tstate->exc_value = *value;
    tstate->exc_traceback = *tb;
    #endif
    *type = tmp_type;
    *value = tmp_value;
    *tb = tmp_tb;
}
#else
static CYTHON_INLINE void __Pyx_ExceptionSwap(PyObject **type, PyObject **value, PyObject **tb) {
    PyObject *tmp_type, *tmp_value, *tmp_tb;
    PyErr_GetExcInfo(&tmp_type, &tmp_value, &tmp_tb);
    PyErr_SetExcInfo(*type, *value, *tb);
    *type = tmp_type;
    *value = tmp_value;
    *tb = tmp_tb;
}
#endif

/* PyIntBinop */
                #if !CYTHON_COMPILING_IN_PYPY
static PyObject* __Pyx_PyInt_SubtractObjC(PyObject *op1, PyObject *op2, CYTHON_UNUSED long intval, CYTHON_UNUSED int inplace) {
    #if PY_MAJOR_VERSION < 3
    if (likely(PyInt_CheckExact(op1))) {
        const long b = intval;
        long x;
        long a = PyInt_AS_LONG(op1);
            x = (long)((unsigned long)a - b);
            if (likely((x^a) >= 0 || (x^~b) >= 0))
                return PyInt_FromLong(x);
            return PyLong_Type.tp_as_number->nb_subtract(op1, op2);
    }
    #endif
    #if CYTHON_USE_PYLONG_INTERNALS
    if (likely(PyLong_CheckExact(op1))) {
        const long b = intval;
        long a, x;
#ifdef HAVE_LONG_LONG
        const PY_LONG_LONG llb = intval;
        PY_LONG_LONG lla, llx;
#endif
        const digit* digits = ((PyLongObject*)op1)->ob_digit;
        const Py_ssize_t size = Py_SIZE(op1);
        if (likely(__Pyx_sst_abs(size) <= 1)) {
            a = likely(size) ? digits[0] : 0;
            if (size == -1) a = -a;
        } else {
            switch (size) {
                case -2:
                    if (8 * sizeof(long) - 1 > 2 * PyLong_SHIFT) {
                        a = -(long) (((((unsigned long)digits[1]) << PyLong_SHIFT) | (unsigned long)digits[0]));
                        break;
#ifdef HAVE_LONG_LONG
                    } else if (8 * sizeof(PY_LONG_LONG) - 1 > 2 * PyLong_SHIFT) {
                        lla = -(PY_LONG_LONG) (((((unsigned PY_LONG_LONG)digits[1]) << PyLong_SHIFT) | (unsigned PY_LONG_LONG)digits[0]));
                        goto long_long;
#endif
                    }
                    CYTHON_FALLTHROUGH;
                case 2:
                    if (8 * sizeof(long) - 1 > 2 * PyLong_SHIFT) {
                        a = (long) (((((unsigned long)digits[1]) << PyLong_SHIFT) | (unsigned long)digits[0]));
                        break;
#ifdef HAVE_LONG_LONG
                    } else if (8 * sizeof(PY_LONG_LONG) - 1 > 2 * PyLong_SHIFT) {
                        lla = (PY_LONG_LONG) (((((unsigned PY_LONG_LONG)digits[1]) << PyLong_SHIFT) | (unsigned PY_LONG_LONG)digits[0]));
                        goto long_long;
#endif
                    }
                    CYTHON_FALLTHROUGH;
                case -3:
                    if (8 * sizeof(long) - 1 > 3 * PyLong_SHIFT) {
                        a = -(long) (((((((unsigned long)digits[2]) << PyLong_SHIFT) | (unsigned long)digits[1]) << PyLong_SHIFT) | (unsigned long)digits[0]));
                        break;
#ifdef HAVE_LONG_LONG
                    } else if (8 * sizeof(PY_LONG_LONG) - 1 > 3 * PyLong_SHIFT) {
                        lla = -(PY_LONG_LONG) (((((((unsigned PY_LONG_LONG)digits[2]) << PyLong_SHIFT) | (unsigned PY_LONG_LONG)digits[1]) << PyLong_SHIFT) | (unsigned PY_LONG_LONG)digits[0]));
                        goto long_long;
#endif
                    }
                    CYTHON_FALLTHROUGH;
                case 3:
                    if (8 * sizeof(long) - 1 > 3 * PyLong_SHIFT) {
                        a = (long) (((((((unsigned long)digits[2]) << PyLong_SHIFT) | (unsigned long)digits[1]) << PyLong_SHIFT) | (unsigned long)digits[0]));
                        break;
#ifdef HAVE_LONG_LONG
                    } else if (8 * sizeof(PY_LONG_LONG) - 1 > 3 * PyLong_SHIFT) {
                        lla = (PY_LONG_LONG) (((((((unsigned PY_LONG_LONG)digits[2]) << PyLong_SHIFT) | (unsigned PY_LONG_LONG)digits[1]) << PyLong_SHIFT) | (unsigned PY_LONG_LONG)digits[0]));
                        goto long_long;
#endif
                    }
                    CYTHON_FALLTHROUGH;
                case -4:
                    if (8 * sizeof(long) - 1 > 4 * PyLong_SHIFT) {
                        a = -(long) (((((((((unsigned long)digits[3]) << PyLong_SHIFT) | (unsigned long)digits[2]) << PyLong_SHIFT) | (unsigned long)digits[1]) << PyLong_SHIFT) | (unsigned long)digits[0]));
                        break;
#ifdef HAVE_LONG_LONG
                    } else if (8 * sizeof(PY_LONG_LONG) - 1 > 4 * PyLong_SHIFT) {
                        lla = -(PY_LONG_LONG) (((((((((unsigned PY_LONG_LONG)digits[3]) << PyLong_SHIFT) | (unsigned PY_LONG_LONG)digits[2]) << PyLong_SHIFT) | (unsigned PY_LONG_LONG)digits[1]) << PyLong_SHIFT) | (unsigned PY_LONG_LONG)digits[0]));
                        goto long_long;
#endif
                    }
                    CYTHON_FALLTHROUGH;
                case 4:
                    if (8 * sizeof(long) - 1 > 4 * PyLong_SHIFT) {
                        a = (long) (((((((((unsigned long)digits[3]) << PyLong_SHIFT) | (unsigned long)digits[2]) << PyLong_SHIFT) | (unsigned long)digits[1]) << PyLong_SHIFT) | (unsigned long)digits[0]));
                        break;
#ifdef HAVE_LONG_LONG
                    } else if (8 * sizeof(PY_LONG_LONG) - 1 > 4 * PyLong_SHIFT) {
                        lla = (PY_LONG_LONG) (((((((((unsigned PY_LONG_LONG)digits[3]) << PyLong_SHIFT) | (unsigned PY_LONG_LONG)digits[2]) << PyLong_SHIFT) | (unsigned PY_LONG_LONG)digits[1]) << PyLong_SHIFT) | (unsigned PY_LONG_LONG)digits[0]));
                        goto long_long;
#endif
                    }
                    CYTHON_FALLTHROUGH;
                default: return PyLong_Type.tp_as_number->nb_subtract(op1, op2);
            }
        }
                x = a - b;
            return PyLong_FromLong(x);
#ifdef HAVE_LONG_LONG
        long_long:
                llx = lla - llb;
            return PyLong_FromLongLong(llx);
#endif
        
        
    }
    #endif
    if (PyFloat_CheckExact(op1)) {
        const long b = intval;
        double a = PyFloat_AS_DOUBLE(op1);
            double result;
            PyFPE_START_PROTECT("subtract", return NULL)
            result = ((double)a) - (double)b;
            PyFPE_END_PROTECT(result)
            return PyFloat_FromDouble(result);
    }
    return (inplace ? PyNumber_InPlaceSubtract : PyNumber_Subtract)(op1, op2);
}
#endif

/* ImportFrom */
                static PyObject* __Pyx_ImportFrom(PyObject* module, PyObject* name) {
    PyObject* value = __Pyx_PyObject_GetAttrStr(module, name);
    if (unlikely(!value) && PyErr_ExceptionMatches(PyExc_AttributeError)) {
        PyErr_Format(PyExc_ImportError,
        #if PY_MAJOR_VERSION < 3
            "cannot import name %.230s", PyString_AS_STRING(name));
        #else
            "cannot import name %S", name);
        #endif
    }
    return value;
}

/* CallNextTpTraverse */
                static int __Pyx_call_next_tp_traverse(PyObject* obj, visitproc v, void *a, traverseproc current_tp_traverse) {
    PyTypeObject* type = Py_TYPE(obj);
    while (type && type->tp_traverse != current_tp_traverse)
        type = type->tp_base;
    while (type && type->tp_traverse == current_tp_traverse)
        type = type->tp_base;
    if (type && type->tp_traverse)
        return type->tp_traverse(obj, v, a);
    return 0;
}

/* CallNextTpClear */
                static void __Pyx_call_next_tp_clear(PyObject* obj, inquiry current_tp_clear) {
    PyTypeObject* type = Py_TYPE(obj);
    while (type && type->tp_clear != current_tp_clear)
        type = type->tp_base;
    while (type && type->tp_clear == current_tp_clear)
        type = type->tp_base;
    if (type && type->tp_clear)
        type->tp_clear(obj);
}

/* PyObject_GenericGetAttrNoDict */
                #if CYTHON_USE_TYPE_SLOTS && CYTHON_USE_PYTYPE_LOOKUP && PY_VERSION_HEX < 0x03070000
static PyObject *__Pyx_RaiseGenericGetAttributeError(PyTypeObject *tp, PyObject *attr_name) {
    PyErr_Format(PyExc_AttributeError,
#if PY_MAJOR_VERSION >= 3
                 "'%.50s' object has no attribute '%U'",
                 tp->tp_name, attr_name);
#else
                 "'%.50s' object has no attribute '%.400s'",
                 tp->tp_name, PyString_AS_STRING(attr_name));
#endif
    return NULL;
}
static CYTHON_INLINE PyObject* __Pyx_PyObject_GenericGetAttrNoDict(PyObject* obj, PyObject* attr_name) {
    PyObject *descr;
    PyTypeObject *tp = Py_TYPE(obj);
    if (unlikely(!PyString_Check(attr_name))) {
        return PyObject_GenericGetAttr(obj, attr_name);
    }
    assert(!tp->tp_dictoffset);
    descr = _PyType_Lookup(tp, attr_name);
    if (unlikely(!descr)) {
        return __Pyx_RaiseGenericGetAttributeError(tp, attr_name);
    }
    Py_INCREF(descr);
    #if PY_MAJOR_VERSION < 3
    if (likely(PyType_HasFeature(Py_TYPE(descr), Py_TPFLAGS_HAVE_CLASS)))
    #endif
    {
        descrgetfunc f = Py_TYPE(descr)->tp_descr_get;
        if (unlikely(f)) {
            PyObject *res = f(descr, obj, (PyObject *)tp);
            Py_DECREF(descr);
            return res;
        }
    }
    return descr;
}
#endif

/* PyObject_GenericGetAttr */
                #if CYTHON_USE_TYPE_SLOTS && CYTHON_USE_PYTYPE_LOOKUP && PY_VERSION_HEX < 0x03070000
static PyObject* __Pyx_PyObject_GenericGetAttr(PyObject* obj, PyObject* attr_name) {
    if (unlikely(Py_TYPE(obj)->tp_dictoffset)) {
        return PyObject_GenericGetAttr(obj, attr_name);
    }
    return __Pyx_PyObject_GenericGetAttrNoDict(obj, attr_name);
}
#endif

/* SetVTable */
                static int __Pyx_SetVtable(PyObject *dict, void *vtable) {
#if PY_VERSION_HEX >= 0x02070000
    PyObject *ob = PyCapsule_New(vtable, 0, 0);
#else
    PyObject *ob = PyCObject_FromVoidPtr(vtable, 0);
#endif
    if (!ob)
        goto bad;
    if (PyDict_SetItem(dict, __pyx_n_s_pyx_vtable, ob) < 0)
        goto bad;
    Py_DECREF(ob);
    return 0;
bad:
    Py_XDECREF(ob);
    return -1;
}

/* SetupReduce */
                static int __Pyx_setup_reduce_is_named(PyObject* meth, PyObject* name) {
  int ret;
  PyObject *name_attr;
  name_attr = __Pyx_PyObject_GetAttrStr(meth, __pyx_n_s_name_2);
  if (likely(name_attr)) {
      ret = PyObject_RichCompareBool(name_attr, name, Py_EQ);
  } else {
      ret = -1;
  }
  if (unlikely(ret < 0)) {
      PyErr_Clear();
      ret = 0;
  }
  Py_XDECREF(name_attr);
  return ret;
}
static int __Pyx_setup_reduce(PyObject* type_obj) {
    int ret = 0;
    PyObject *object_reduce = NULL;
    PyObject *object_reduce_ex = NULL;
    PyObject *reduce = NULL;
    PyObject *reduce_ex = NULL;
    PyObject *reduce_cython = NULL;
    PyObject *setstate = NULL;
    PyObject *setstate_cython = NULL;
#if CYTHON_USE_PYTYPE_LOOKUP
    if (_PyType_Lookup((PyTypeObject*)type_obj, __pyx_n_s_getstate)) goto GOOD;
#else
    if (PyObject_HasAttr(type_obj, __pyx_n_s_getstate)) goto GOOD;
#endif
#if CYTHON_USE_PYTYPE_LOOKUP
    object_reduce_ex = _PyType_Lookup(&PyBaseObject_Type, __pyx_n_s_reduce_ex); if (!object_reduce_ex) goto BAD;
#else
    object_reduce_ex = __Pyx_PyObject_GetAttrStr((PyObject*)&PyBaseObject_Type, __pyx_n_s_reduce_ex); if (!object_reduce_ex) goto BAD;
#endif
    reduce_ex = __Pyx_PyObject_GetAttrStr(type_obj, __pyx_n_s_reduce_ex); if (unlikely(!reduce_ex)) goto BAD;
    if (reduce_ex == object_reduce_ex) {
#if CYTHON_USE_PYTYPE_LOOKUP
        object_reduce = _PyType_Lookup(&PyBaseObject_Type, __pyx_n_s_reduce); if (!object_reduce) goto BAD;
#else
        object_reduce = __Pyx_PyObject_GetAttrStr((PyObject*)&PyBaseObject_Type, __pyx_n_s_reduce); if (!object_reduce) goto BAD;
#endif
        reduce = __Pyx_PyObject_GetAttrStr(type_obj, __pyx_n_s_reduce); if (unlikely(!reduce)) goto BAD;
        if (reduce == object_reduce || __Pyx_setup_reduce_is_named(reduce, __pyx_n_s_reduce_cython)) {
            reduce_cython = __Pyx_PyObject_GetAttrStr(type_obj, __pyx_n_s_reduce_cython); if (unlikely(!reduce_cython)) goto BAD;
            ret = PyDict_SetItem(((PyTypeObject*)type_obj)->tp_dict, __pyx_n_s_reduce, reduce_cython); if (unlikely(ret < 0)) goto BAD;
            ret = PyDict_DelItem(((PyTypeObject*)type_obj)->tp_dict, __pyx_n_s_reduce_cython); if (unlikely(ret < 0)) goto BAD;
            setstate = __Pyx_PyObject_GetAttrStr(type_obj, __pyx_n_s_setstate);
            if (!setstate) PyErr_Clear();
            if (!setstate || __Pyx_setup_reduce_is_named(setstate, __pyx_n_s_setstate_cython)) {
                setstate_cython = __Pyx_PyObject_GetAttrStr(type_obj, __pyx_n_s_setstate_cython); if (unlikely(!setstate_cython)) goto BAD;
                ret = PyDict_SetItem(((PyTypeObject*)type_obj)->tp_dict, __pyx_n_s_setstate, setstate_cython); if (unlikely(ret < 0)) goto BAD;
                ret = PyDict_DelItem(((PyTypeObject*)type_obj)->tp_dict, __pyx_n_s_setstate_cython); if (unlikely(ret < 0)) goto BAD;
            }
            PyType_Modified((PyTypeObject*)type_obj);
        }
    }
    goto GOOD;
BAD:
    if (!PyErr_Occurred())
        PyErr_Format(PyExc_RuntimeError, "Unable to initialize pickling for %s", ((PyTypeObject*)type_obj)->tp_name);
    ret = -1;
GOOD:
#if !CYTHON_USE_PYTYPE_LOOKUP
    Py_XDECREF(object_reduce);
    Py_XDECREF(object_reduce_ex);
#endif
    Py_XDECREF(reduce);
    Py_XDECREF(reduce_ex);
    Py_XDECREF(reduce_cython);
    Py_XDECREF(setstate);
    Py_XDECREF(setstate_cython);
    return ret;
}

/* CalculateMetaclass */
                static PyObject *__Pyx_CalculateMetaclass(PyTypeObject *metaclass, PyObject *bases) {
    Py_ssize_t i, nbases = PyTuple_GET_SIZE(bases);
    for (i=0; i < nbases; i++) {
        PyTypeObject *tmptype;
        PyObject *tmp = PyTuple_GET_ITEM(bases, i);
        tmptype = Py_TYPE(tmp);
#if PY_MAJOR_VERSION < 3
        if (tmptype == &PyClass_Type)
            continue;
#endif
        if (!metaclass) {
            metaclass = tmptype;
            continue;
        }
        if (PyType_IsSubtype(metaclass, tmptype))
            continue;
        if (PyType_IsSubtype(tmptype, metaclass)) {
            metaclass = tmptype;
            continue;
        }
        PyErr_SetString(PyExc_TypeError,
                        "metaclass conflict: "
                        "the metaclass of a derived class "
                        "must be a (non-strict) subclass "
                        "of the metaclasses of all its bases");
        return NULL;
    }
    if (!metaclass) {
#if PY_MAJOR_VERSION < 3
        metaclass = &PyClass_Type;
#else
        metaclass = &PyType_Type;
#endif
    }
    Py_INCREF((PyObject*) metaclass);
    return (PyObject*) metaclass;
}

/* Py3ClassCreate */
                static PyObject *__Pyx_Py3MetaclassPrepare(PyObject *metaclass, PyObject *bases, PyObject *name,
                                           PyObject *qualname, PyObject *mkw, PyObject *modname, PyObject *doc) {
    PyObject *ns;
    if (metaclass) {
        PyObject *prep = __Pyx_PyObject_GetAttrStr(metaclass, __pyx_n_s_prepare);
        if (prep) {
            PyObject *pargs = PyTuple_Pack(2, name, bases);
            if (unlikely(!pargs)) {
                Py_DECREF(prep);
                return NULL;
            }
            ns = PyObject_Call(prep, pargs, mkw);
            Py_DECREF(prep);
            Py_DECREF(pargs);
        } else {
            if (unlikely(!PyErr_ExceptionMatches(PyExc_AttributeError)))
                return NULL;
            PyErr_Clear();
            ns = PyDict_New();
        }
    } else {
        ns = PyDict_New();
    }
    if (unlikely(!ns))
        return NULL;
    if (unlikely(PyObject_SetItem(ns, __pyx_n_s_module, modname) < 0)) goto bad;
    if (unlikely(PyObject_SetItem(ns, __pyx_n_s_qualname, qualname) < 0)) goto bad;
    if (unlikely(doc && PyObject_SetItem(ns, __pyx_n_s_doc, doc) < 0)) goto bad;
    return ns;
bad:
    Py_DECREF(ns);
    return NULL;
}
static PyObject *__Pyx_Py3ClassCreate(PyObject *metaclass, PyObject *name, PyObject *bases,
                                      PyObject *dict, PyObject *mkw,
                                      int calculate_metaclass, int allow_py2_metaclass) {
    PyObject *result, *margs;
    PyObject *owned_metaclass = NULL;
    if (allow_py2_metaclass) {
        owned_metaclass = PyObject_GetItem(dict, __pyx_n_s_metaclass);
        if (owned_metaclass) {
            metaclass = owned_metaclass;
        } else if (likely(PyErr_ExceptionMatches(PyExc_KeyError))) {
            PyErr_Clear();
        } else {
            return NULL;
        }
    }
    if (calculate_metaclass && (!metaclass || PyType_Check(metaclass))) {
        metaclass = __Pyx_CalculateMetaclass((PyTypeObject*) metaclass, bases);
        Py_XDECREF(owned_metaclass);
        if (unlikely(!metaclass))
            return NULL;
        owned_metaclass = metaclass;
    }
    margs = PyTuple_Pack(3, name, bases, dict);
    if (unlikely(!margs)) {
        result = NULL;
    } else {
        result = PyObject_Call(metaclass, margs, mkw);
        Py_DECREF(margs);
    }
    Py_XDECREF(owned_metaclass);
    return result;
}

/* CLineInTraceback */
                #ifndef CYTHON_CLINE_IN_TRACEBACK
static int __Pyx_CLineForTraceback(CYTHON_UNUSED PyThreadState *tstate, int c_line) {
    PyObject *use_cline;
    PyObject *ptype, *pvalue, *ptraceback;
#if CYTHON_COMPILING_IN_CPYTHON
    PyObject **cython_runtime_dict;
#endif
    if (unlikely(!__pyx_cython_runtime)) {
        return c_line;
    }
    __Pyx_ErrFetchInState(tstate, &ptype, &pvalue, &ptraceback);
#if CYTHON_COMPILING_IN_CPYTHON
    cython_runtime_dict = _PyObject_GetDictPtr(__pyx_cython_runtime);
    if (likely(cython_runtime_dict)) {
      use_cline = __Pyx_PyDict_GetItemStr(*cython_runtime_dict, __pyx_n_s_cline_in_traceback);
    } else
#endif
    {
      PyObject *use_cline_obj = __Pyx_PyObject_GetAttrStr(__pyx_cython_runtime, __pyx_n_s_cline_in_traceback);
      if (use_cline_obj) {
        use_cline = PyObject_Not(use_cline_obj) ? Py_False : Py_True;
        Py_DECREF(use_cline_obj);
      } else {
        PyErr_Clear();
        use_cline = NULL;
      }
    }
    if (!use_cline) {
        c_line = 0;
        PyObject_SetAttr(__pyx_cython_runtime, __pyx_n_s_cline_in_traceback, Py_False);
    }
    else if (PyObject_Not(use_cline) != 0) {
        c_line = 0;
    }
    __Pyx_ErrRestoreInState(tstate, ptype, pvalue, ptraceback);
    return c_line;
}
#endif

/* CodeObjectCache */
                static int __pyx_bisect_code_objects(__Pyx_CodeObjectCacheEntry* entries, int count, int code_line) {
    int start = 0, mid = 0, end = count - 1;
    if (end >= 0 && code_line > entries[end].code_line) {
        return count;
    }
    while (start < end) {
        mid = start + (end - start) / 2;
        if (code_line < entries[mid].code_line) {
            end = mid;
        } else if (code_line > entries[mid].code_line) {
             start = mid + 1;
        } else {
            return mid;
        }
    }
    if (code_line <= entries[mid].code_line) {
        return mid;
    } else {
        return mid + 1;
    }
}
static PyCodeObject *__pyx_find_code_object(int code_line) {
    PyCodeObject* code_object;
    int pos;
    if (unlikely(!code_line) || unlikely(!__pyx_code_cache.entries)) {
        return NULL;
    }
    pos = __pyx_bisect_code_objects(__pyx_code_cache.entries, __pyx_code_cache.count, code_line);
    if (unlikely(pos >= __pyx_code_cache.count) || unlikely(__pyx_code_cache.entries[pos].code_line != code_line)) {
        return NULL;
    }
    code_object = __pyx_code_cache.entries[pos].code_object;
    Py_INCREF(code_object);
    return code_object;
}
static void __pyx_insert_code_object(int code_line, PyCodeObject* code_object) {
    int pos, i;
    __Pyx_CodeObjectCacheEntry* entries = __pyx_code_cache.entries;
    if (unlikely(!code_line)) {
        return;
    }
    if (unlikely(!entries)) {
        entries = (__Pyx_CodeObjectCacheEntry*)PyMem_Malloc(64*sizeof(__Pyx_CodeObjectCacheEntry));
        if (likely(entries)) {
            __pyx_code_cache.entries = entries;
            __pyx_code_cache.max_count = 64;
            __pyx_code_cache.count = 1;
            entries[0].code_line = code_line;
            entries[0].code_object = code_object;
            Py_INCREF(code_object);
        }
        return;
    }
    pos = __pyx_bisect_code_objects(__pyx_code_cache.entries, __pyx_code_cache.count, code_line);
    if ((pos < __pyx_code_cache.count) && unlikely(__pyx_code_cache.entries[pos].code_line == code_line)) {
        PyCodeObject* tmp = entries[pos].code_object;
        entries[pos].code_object = code_object;
        Py_DECREF(tmp);
        return;
    }
    if (__pyx_code_cache.count == __pyx_code_cache.max_count) {
        int new_max = __pyx_code_cache.max_count + 64;
        entries = (__Pyx_CodeObjectCacheEntry*)PyMem_Realloc(
            __pyx_code_cache.entries, (size_t)new_max*sizeof(__Pyx_CodeObjectCacheEntry));
        if (unlikely(!entries)) {
            return;
        }
        __pyx_code_cache.entries = entries;
        __pyx_code_cache.max_count = new_max;
    }
    for (i=__pyx_code_cache.count; i>pos; i--) {
        entries[i] = entries[i-1];
    }
    entries[pos].code_line = code_line;
    entries[pos].code_object = code_object;
    __pyx_code_cache.count++;
    Py_INCREF(code_object);
}

/* AddTraceback */
                #include "compile.h"
#include "frameobject.h"
#include "traceback.h"
static PyCodeObject* __Pyx_CreateCodeObjectForTraceback(
            const char *funcname, int c_line,
            int py_line, const char *filename) {
    PyCodeObject *py_code = 0;
    PyObject *py_srcfile = 0;
    PyObject *py_funcname = 0;
    #if PY_MAJOR_VERSION < 3
    py_srcfile = PyString_FromString(filename);
    #else
    py_srcfile = PyUnicode_FromString(filename);
    #endif
    if (!py_srcfile) goto bad;
    if (c_line) {
        #if PY_MAJOR_VERSION < 3
        py_funcname = PyString_FromFormat( "%s (%s:%d)", funcname, __pyx_cfilenm, c_line);
        #else
        py_funcname = PyUnicode_FromFormat( "%s (%s:%d)", funcname, __pyx_cfilenm, c_line);
        #endif
    }
    else {
        #if PY_MAJOR_VERSION < 3
        py_funcname = PyString_FromString(funcname);
        #else
        py_funcname = PyUnicode_FromString(funcname);
        #endif
    }
    if (!py_funcname) goto bad;
    py_code = __Pyx_PyCode_New(
        0,
        0,
        0,
        0,
        0,
        __pyx_empty_bytes, /*PyObject *code,*/
        __pyx_empty_tuple, /*PyObject *consts,*/
        __pyx_empty_tuple, /*PyObject *names,*/
        __pyx_empty_tuple, /*PyObject *varnames,*/
        __pyx_empty_tuple, /*PyObject *freevars,*/
        __pyx_empty_tuple, /*PyObject *cellvars,*/
        py_srcfile,   /*PyObject *filename,*/
        py_funcname,  /*PyObject *name,*/
        py_line,
        __pyx_empty_bytes  /*PyObject *lnotab*/
    );
    Py_DECREF(py_srcfile);
    Py_DECREF(py_funcname);
    return py_code;
bad:
    Py_XDECREF(py_srcfile);
    Py_XDECREF(py_funcname);
    return NULL;
}
static void __Pyx_AddTraceback(const char *funcname, int c_line,
                               int py_line, const char *filename) {
    PyCodeObject *py_code = 0;
    PyFrameObject *py_frame = 0;
    PyThreadState *tstate = __Pyx_PyThreadState_Current;
    if (c_line) {
        c_line = __Pyx_CLineForTraceback(tstate, c_line);
    }
    py_code = __pyx_find_code_object(c_line ? -c_line : py_line);
    if (!py_code) {
        py_code = __Pyx_CreateCodeObjectForTraceback(
            funcname, c_line, py_line, filename);
        if (!py_code) goto bad;
        __pyx_insert_code_object(c_line ? -c_line : py_line, py_code);
    }
    py_frame = PyFrame_New(
        tstate,            /*PyThreadState *tstate,*/
        py_code,           /*PyCodeObject *code,*/
        __pyx_d,    /*PyObject *globals,*/
        0                  /*PyObject *locals*/
    );
    if (!py_frame) goto bad;
    __Pyx_PyFrame_SetLineNumber(py_frame, py_line);
    PyTraceBack_Here(py_frame);
bad:
    Py_XDECREF(py_code);
    Py_XDECREF(py_frame);
}

/* CIntFromPyVerify */
                #define __PYX_VERIFY_RETURN_INT(target_type, func_type, func_value)\
    __PYX__VERIFY_RETURN_INT(target_type, func_type, func_value, 0)
#define __PYX_VERIFY_RETURN_INT_EXC(target_type, func_type, func_value)\
    __PYX__VERIFY_RETURN_INT(target_type, func_type, func_value, 1)
#define __PYX__VERIFY_RETURN_INT(target_type, func_type, func_value, exc)\
    {\
        func_type value = func_value;\
        if (sizeof(target_type) < sizeof(func_type)) {\
            if (unlikely(value != (func_type) (target_type) value)) {\
                func_type zero = 0;\
                if (exc && unlikely(value == (func_type)-1 && PyErr_Occurred()))\
                    return (target_type) -1;\
                if (is_unsigned && unlikely(value < zero))\
                    goto raise_neg_overflow;\
                else\
                    goto raise_overflow;\
            }\
        }\
        return (target_type) value;\
    }

/* CIntToPy */
                static CYTHON_INLINE PyObject* __Pyx_PyInt_From_int(int value) {
    const int neg_one = (int) -1, const_zero = (int) 0;
    const int is_unsigned = neg_one > const_zero;
    if (is_unsigned) {
        if (sizeof(int) < sizeof(long)) {
            return PyInt_FromLong((long) value);
        } else if (sizeof(int) <= sizeof(unsigned long)) {
            return PyLong_FromUnsignedLong((unsigned long) value);
#ifdef HAVE_LONG_LONG
        } else if (sizeof(int) <= sizeof(unsigned PY_LONG_LONG)) {
            return PyLong_FromUnsignedLongLong((unsigned PY_LONG_LONG) value);
#endif
        }
    } else {
        if (sizeof(int) <= sizeof(long)) {
            return PyInt_FromLong((long) value);
#ifdef HAVE_LONG_LONG
        } else if (sizeof(int) <= sizeof(PY_LONG_LONG)) {
            return PyLong_FromLongLong((PY_LONG_LONG) value);
#endif
        }
    }
    {
        int one = 1; int little = (int)*(unsigned char *)&one;
        unsigned char *bytes = (unsigned char *)&value;
        return _PyLong_FromByteArray(bytes, sizeof(int),
                                     little, !is_unsigned);
    }
}

/* CIntToPy */
                static CYTHON_INLINE PyObject* __Pyx_PyInt_From_grpc_connectivity_state(grpc_connectivity_state value) {
    const grpc_connectivity_state neg_one = (grpc_connectivity_state) -1, const_zero = (grpc_connectivity_state) 0;
    const int is_unsigned = neg_one > const_zero;
    if (is_unsigned) {
        if (sizeof(grpc_connectivity_state) < sizeof(long)) {
            return PyInt_FromLong((long) value);
        } else if (sizeof(grpc_connectivity_state) <= sizeof(unsigned long)) {
            return PyLong_FromUnsignedLong((unsigned long) value);
#ifdef HAVE_LONG_LONG
        } else if (sizeof(grpc_connectivity_state) <= sizeof(unsigned PY_LONG_LONG)) {
            return PyLong_FromUnsignedLongLong((unsigned PY_LONG_LONG) value);
#endif
        }
    } else {
        if (sizeof(grpc_connectivity_state) <= sizeof(long)) {
            return PyInt_FromLong((long) value);
#ifdef HAVE_LONG_LONG
        } else if (sizeof(grpc_connectivity_state) <= sizeof(PY_LONG_LONG)) {
            return PyLong_FromLongLong((PY_LONG_LONG) value);
#endif
        }
    }
    {
        int one = 1; int little = (int)*(unsigned char *)&one;
        unsigned char *bytes = (unsigned char *)&value;
        return _PyLong_FromByteArray(bytes, sizeof(grpc_connectivity_state),
                                     little, !is_unsigned);
    }
}

/* CIntToPy */
                static CYTHON_INLINE PyObject* __Pyx_PyInt_From_grpc_status_code(grpc_status_code value) {
    const grpc_status_code neg_one = (grpc_status_code) -1, const_zero = (grpc_status_code) 0;
    const int is_unsigned = neg_one > const_zero;
    if (is_unsigned) {
        if (sizeof(grpc_status_code) < sizeof(long)) {
            return PyInt_FromLong((long) value);
        } else if (sizeof(grpc_status_code) <= sizeof(unsigned long)) {
            return PyLong_FromUnsignedLong((unsigned long) value);
#ifdef HAVE_LONG_LONG
        } else if (sizeof(grpc_status_code) <= sizeof(unsigned PY_LONG_LONG)) {
            return PyLong_FromUnsignedLongLong((unsigned PY_LONG_LONG) value);
#endif
        }
    } else {
        if (sizeof(grpc_status_code) <= sizeof(long)) {
            return PyInt_FromLong((long) value);
#ifdef HAVE_LONG_LONG
        } else if (sizeof(grpc_status_code) <= sizeof(PY_LONG_LONG)) {
            return PyLong_FromLongLong((PY_LONG_LONG) value);
#endif
        }
    }
    {
        int one = 1; int little = (int)*(unsigned char *)&one;
        unsigned char *bytes = (unsigned char *)&value;
        return _PyLong_FromByteArray(bytes, sizeof(grpc_status_code),
                                     little, !is_unsigned);
    }
}

/* CIntToPy */
                static CYTHON_INLINE PyObject* __Pyx_PyInt_From_grpc_call_error(grpc_call_error value) {
    const grpc_call_error neg_one = (grpc_call_error) -1, const_zero = (grpc_call_error) 0;
    const int is_unsigned = neg_one > const_zero;
    if (is_unsigned) {
        if (sizeof(grpc_call_error) < sizeof(long)) {
            return PyInt_FromLong((long) value);
        } else if (sizeof(grpc_call_error) <= sizeof(unsigned long)) {
            return PyLong_FromUnsignedLong((unsigned long) value);
#ifdef HAVE_LONG_LONG
        } else if (sizeof(grpc_call_error) <= sizeof(unsigned PY_LONG_LONG)) {
            return PyLong_FromUnsignedLongLong((unsigned PY_LONG_LONG) value);
#endif
        }
    } else {
        if (sizeof(grpc_call_error) <= sizeof(long)) {
            return PyInt_FromLong((long) value);
#ifdef HAVE_LONG_LONG
        } else if (sizeof(grpc_call_error) <= sizeof(PY_LONG_LONG)) {
            return PyLong_FromLongLong((PY_LONG_LONG) value);
#endif
        }
    }
    {
        int one = 1; int little = (int)*(unsigned char *)&one;
        unsigned char *bytes = (unsigned char *)&value;
        return _PyLong_FromByteArray(bytes, sizeof(grpc_call_error),
                                     little, !is_unsigned);
    }
}

/* CIntToPy */
                static CYTHON_INLINE PyObject* __Pyx_PyInt_From_grpc_completion_type(grpc_completion_type value) {
    const grpc_completion_type neg_one = (grpc_completion_type) -1, const_zero = (grpc_completion_type) 0;
    const int is_unsigned = neg_one > const_zero;
    if (is_unsigned) {
        if (sizeof(grpc_completion_type) < sizeof(long)) {
            return PyInt_FromLong((long) value);
        } else if (sizeof(grpc_completion_type) <= sizeof(unsigned long)) {
            return PyLong_FromUnsignedLong((unsigned long) value);
#ifdef HAVE_LONG_LONG
        } else if (sizeof(grpc_completion_type) <= sizeof(unsigned PY_LONG_LONG)) {
            return PyLong_FromUnsignedLongLong((unsigned PY_LONG_LONG) value);
#endif
        }
    } else {
        if (sizeof(grpc_completion_type) <= sizeof(long)) {
            return PyInt_FromLong((long) value);
#ifdef HAVE_LONG_LONG
        } else if (sizeof(grpc_completion_type) <= sizeof(PY_LONG_LONG)) {
            return PyLong_FromLongLong((PY_LONG_LONG) value);
#endif
        }
    }
    {
        int one = 1; int little = (int)*(unsigned char *)&one;
        unsigned char *bytes = (unsigned char *)&value;
        return _PyLong_FromByteArray(bytes, sizeof(grpc_completion_type),
                                     little, !is_unsigned);
    }
}

/* CIntToPy */
                static CYTHON_INLINE PyObject* __Pyx_PyInt_From_grpc_op_type(grpc_op_type value) {
    const grpc_op_type neg_one = (grpc_op_type) -1, const_zero = (grpc_op_type) 0;
    const int is_unsigned = neg_one > const_zero;
    if (is_unsigned) {
        if (sizeof(grpc_op_type) < sizeof(long)) {
            return PyInt_FromLong((long) value);
        } else if (sizeof(grpc_op_type) <= sizeof(unsigned long)) {
            return PyLong_FromUnsignedLong((unsigned long) value);
#ifdef HAVE_LONG_LONG
        } else if (sizeof(grpc_op_type) <= sizeof(unsigned PY_LONG_LONG)) {
            return PyLong_FromUnsignedLongLong((unsigned PY_LONG_LONG) value);
#endif
        }
    } else {
        if (sizeof(grpc_op_type) <= sizeof(long)) {
            return PyInt_FromLong((long) value);
#ifdef HAVE_LONG_LONG
        } else if (sizeof(grpc_op_type) <= sizeof(PY_LONG_LONG)) {
            return PyLong_FromLongLong((PY_LONG_LONG) value);
#endif
        }
    }
    {
        int one = 1; int little = (int)*(unsigned char *)&one;
        unsigned char *bytes = (unsigned char *)&value;
        return _PyLong_FromByteArray(bytes, sizeof(grpc_op_type),
                                     little, !is_unsigned);
    }
}

/* CIntToPy */
                static CYTHON_INLINE PyObject* __Pyx_PyInt_From_grpc_compression_algorithm(grpc_compression_algorithm value) {
    const grpc_compression_algorithm neg_one = (grpc_compression_algorithm) -1, const_zero = (grpc_compression_algorithm) 0;
    const int is_unsigned = neg_one > const_zero;
    if (is_unsigned) {
        if (sizeof(grpc_compression_algorithm) < sizeof(long)) {
            return PyInt_FromLong((long) value);
        } else if (sizeof(grpc_compression_algorithm) <= sizeof(unsigned long)) {
            return PyLong_FromUnsignedLong((unsigned long) value);
#ifdef HAVE_LONG_LONG
        } else if (sizeof(grpc_compression_algorithm) <= sizeof(unsigned PY_LONG_LONG)) {
            return PyLong_FromUnsignedLongLong((unsigned PY_LONG_LONG) value);
#endif
        }
    } else {
        if (sizeof(grpc_compression_algorithm) <= sizeof(long)) {
            return PyInt_FromLong((long) value);
#ifdef HAVE_LONG_LONG
        } else if (sizeof(grpc_compression_algorithm) <= sizeof(PY_LONG_LONG)) {
            return PyLong_FromLongLong((PY_LONG_LONG) value);
#endif
        }
    }
    {
        int one = 1; int little = (int)*(unsigned char *)&one;
        unsigned char *bytes = (unsigned char *)&value;
        return _PyLong_FromByteArray(bytes, sizeof(grpc_compression_algorithm),
                                     little, !is_unsigned);
    }
}

/* CIntToPy */
                static CYTHON_INLINE PyObject* __Pyx_PyInt_From_grpc_compression_level(grpc_compression_level value) {
    const grpc_compression_level neg_one = (grpc_compression_level) -1, const_zero = (grpc_compression_level) 0;
    const int is_unsigned = neg_one > const_zero;
    if (is_unsigned) {
        if (sizeof(grpc_compression_level) < sizeof(long)) {
            return PyInt_FromLong((long) value);
        } else if (sizeof(grpc_compression_level) <= sizeof(unsigned long)) {
            return PyLong_FromUnsignedLong((unsigned long) value);
#ifdef HAVE_LONG_LONG
        } else if (sizeof(grpc_compression_level) <= sizeof(unsigned PY_LONG_LONG)) {
            return PyLong_FromUnsignedLongLong((unsigned PY_LONG_LONG) value);
#endif
        }
    } else {
        if (sizeof(grpc_compression_level) <= sizeof(long)) {
            return PyInt_FromLong((long) value);
#ifdef HAVE_LONG_LONG
        } else if (sizeof(grpc_compression_level) <= sizeof(PY_LONG_LONG)) {
            return PyLong_FromLongLong((PY_LONG_LONG) value);
#endif
        }
    }
    {
        int one = 1; int little = (int)*(unsigned char *)&one;
        unsigned char *bytes = (unsigned char *)&value;
        return _PyLong_FromByteArray(bytes, sizeof(grpc_compression_level),
                                     little, !is_unsigned);
    }
}

/* CIntToPy */
                static CYTHON_INLINE PyObject* __Pyx_PyInt_From_long(long value) {
    const long neg_one = (long) -1, const_zero = (long) 0;
    const int is_unsigned = neg_one > const_zero;
    if (is_unsigned) {
        if (sizeof(long) < sizeof(long)) {
            return PyInt_FromLong((long) value);
        } else if (sizeof(long) <= sizeof(unsigned long)) {
            return PyLong_FromUnsignedLong((unsigned long) value);
#ifdef HAVE_LONG_LONG
        } else if (sizeof(long) <= sizeof(unsigned PY_LONG_LONG)) {
            return PyLong_FromUnsignedLongLong((unsigned PY_LONG_LONG) value);
#endif
        }
    } else {
        if (sizeof(long) <= sizeof(long)) {
            return PyInt_FromLong((long) value);
#ifdef HAVE_LONG_LONG
        } else if (sizeof(long) <= sizeof(PY_LONG_LONG)) {
            return PyLong_FromLongLong((PY_LONG_LONG) value);
#endif
        }
    }
    {
        int one = 1; int little = (int)*(unsigned char *)&one;
        unsigned char *bytes = (unsigned char *)&value;
        return _PyLong_FromByteArray(bytes, sizeof(long),
                                     little, !is_unsigned);
    }
}

/* CIntToPy */
                static CYTHON_INLINE PyObject* __Pyx_PyInt_From_unsigned_int(unsigned int value) {
    const unsigned int neg_one = (unsigned int) -1, const_zero = (unsigned int) 0;
    const int is_unsigned = neg_one > const_zero;
    if (is_unsigned) {
        if (sizeof(unsigned int) < sizeof(long)) {
            return PyInt_FromLong((long) value);
        } else if (sizeof(unsigned int) <= sizeof(unsigned long)) {
            return PyLong_FromUnsignedLong((unsigned long) value);
#ifdef HAVE_LONG_LONG
        } else if (sizeof(unsigned int) <= sizeof(unsigned PY_LONG_LONG)) {
            return PyLong_FromUnsignedLongLong((unsigned PY_LONG_LONG) value);
#endif
        }
    } else {
        if (sizeof(unsigned int) <= sizeof(long)) {
            return PyInt_FromLong((long) value);
#ifdef HAVE_LONG_LONG
        } else if (sizeof(unsigned int) <= sizeof(PY_LONG_LONG)) {
            return PyLong_FromLongLong((PY_LONG_LONG) value);
#endif
        }
    }
    {
        int one = 1; int little = (int)*(unsigned char *)&one;
        unsigned char *bytes = (unsigned char *)&value;
        return _PyLong_FromByteArray(bytes, sizeof(unsigned int),
                                     little, !is_unsigned);
    }
}

/* CIntFromPy */
                static CYTHON_INLINE grpc_status_code __Pyx_PyInt_As_grpc_status_code(PyObject *x) {
    const grpc_status_code neg_one = (grpc_status_code) -1, const_zero = (grpc_status_code) 0;
    const int is_unsigned = neg_one > const_zero;
#if PY_MAJOR_VERSION < 3
    if (likely(PyInt_Check(x))) {
        if (sizeof(grpc_status_code) < sizeof(long)) {
            __PYX_VERIFY_RETURN_INT(grpc_status_code, long, PyInt_AS_LONG(x))
        } else {
            long val = PyInt_AS_LONG(x);
            if (is_unsigned && unlikely(val < 0)) {
                goto raise_neg_overflow;
            }
            return (grpc_status_code) val;
        }
    } else
#endif
    if (likely(PyLong_Check(x))) {
        if (is_unsigned) {
#if CYTHON_USE_PYLONG_INTERNALS
            const digit* digits = ((PyLongObject*)x)->ob_digit;
            switch (Py_SIZE(x)) {
                case  0: return (grpc_status_code) 0;
                case  1: __PYX_VERIFY_RETURN_INT(grpc_status_code, digit, digits[0])
                case 2:
                    if (8 * sizeof(grpc_status_code) > 1 * PyLong_SHIFT) {
                        if (8 * sizeof(unsigned long) > 2 * PyLong_SHIFT) {
                            __PYX_VERIFY_RETURN_INT(grpc_status_code, unsigned long, (((((unsigned long)digits[1]) << PyLong_SHIFT) | (unsigned long)digits[0])))
                        } else if (8 * sizeof(grpc_status_code) >= 2 * PyLong_SHIFT) {
                            return (grpc_status_code) (((((grpc_status_code)digits[1]) << PyLong_SHIFT) | (grpc_status_code)digits[0]));
                        }
                    }
                    break;
                case 3:
                    if (8 * sizeof(grpc_status_code) > 2 * PyLong_SHIFT) {
                        if (8 * sizeof(unsigned long) > 3 * PyLong_SHIFT) {
                            __PYX_VERIFY_RETURN_INT(grpc_status_code, unsigned long, (((((((unsigned long)digits[2]) << PyLong_SHIFT) | (unsigned long)digits[1]) << PyLong_SHIFT) | (unsigned long)digits[0])))
                        } else if (8 * sizeof(grpc_status_code) >= 3 * PyLong_SHIFT) {
                            return (grpc_status_code) (((((((grpc_status_code)digits[2]) << PyLong_SHIFT) | (grpc_status_code)digits[1]) << PyLong_SHIFT) | (grpc_status_code)digits[0]));
                        }
                    }
                    break;
                case 4:
                    if (8 * sizeof(grpc_status_code) > 3 * PyLong_SHIFT) {
                        if (8 * sizeof(unsigned long) > 4 * PyLong_SHIFT) {
                            __PYX_VERIFY_RETURN_INT(grpc_status_code, unsigned long, (((((((((unsigned long)digits[3]) << PyLong_SHIFT) | (unsigned long)digits[2]) << PyLong_SHIFT) | (unsigned long)digits[1]) << PyLong_SHIFT) | (unsigned long)digits[0])))
                        } else if (8 * sizeof(grpc_status_code) >= 4 * PyLong_SHIFT) {
                            return (grpc_status_code) (((((((((grpc_status_code)digits[3]) << PyLong_SHIFT) | (grpc_status_code)digits[2]) << PyLong_SHIFT) | (grpc_status_code)digits[1]) << PyLong_SHIFT) | (grpc_status_code)digits[0]));
                        }
                    }
                    break;
            }
#endif
#if CYTHON_COMPILING_IN_CPYTHON
            if (unlikely(Py_SIZE(x) < 0)) {
                goto raise_neg_overflow;
            }
#else
            {
                int result = PyObject_RichCompareBool(x, Py_False, Py_LT);
                if (unlikely(result < 0))
                    return (grpc_status_code) -1;
                if (unlikely(result == 1))
                    goto raise_neg_overflow;
            }
#endif
            if (sizeof(grpc_status_code) <= sizeof(unsigned long)) {
                __PYX_VERIFY_RETURN_INT_EXC(grpc_status_code, unsigned long, PyLong_AsUnsignedLong(x))
#ifdef HAVE_LONG_LONG
            } else if (sizeof(grpc_status_code) <= sizeof(unsigned PY_LONG_LONG)) {
                __PYX_VERIFY_RETURN_INT_EXC(grpc_status_code, unsigned PY_LONG_LONG, PyLong_AsUnsignedLongLong(x))
#endif
            }
        } else {
#if CYTHON_USE_PYLONG_INTERNALS
            const digit* digits = ((PyLongObject*)x)->ob_digit;
            switch (Py_SIZE(x)) {
                case  0: return (grpc_status_code) 0;
                case -1: __PYX_VERIFY_RETURN_INT(grpc_status_code, sdigit, (sdigit) (-(sdigit)digits[0]))
                case  1: __PYX_VERIFY_RETURN_INT(grpc_status_code,  digit, +digits[0])
                case -2:
                    if (8 * sizeof(grpc_status_code) - 1 > 1 * PyLong_SHIFT) {
                        if (8 * sizeof(unsigned long) > 2 * PyLong_SHIFT) {
                            __PYX_VERIFY_RETURN_INT(grpc_status_code, long, -(long) (((((unsigned long)digits[1]) << PyLong_SHIFT) | (unsigned long)digits[0])))
                        } else if (8 * sizeof(grpc_status_code) - 1 > 2 * PyLong_SHIFT) {
                            return (grpc_status_code) (((grpc_status_code)-1)*(((((grpc_status_code)digits[1]) << PyLong_SHIFT) | (grpc_status_code)digits[0])));
                        }
                    }
                    break;
                case 2:
                    if (8 * sizeof(grpc_status_code) > 1 * PyLong_SHIFT) {
                        if (8 * sizeof(unsigned long) > 2 * PyLong_SHIFT) {
                            __PYX_VERIFY_RETURN_INT(grpc_status_code, unsigned long, (((((unsigned long)digits[1]) << PyLong_SHIFT) | (unsigned long)digits[0])))
                        } else if (8 * sizeof(grpc_status_code) - 1 > 2 * PyLong_SHIFT) {
                            return (grpc_status_code) ((((((grpc_status_code)digits[1]) << PyLong_SHIFT) | (grpc_status_code)digits[0])));
                        }
                    }
                    break;
                case -3:
                    if (8 * sizeof(grpc_status_code) - 1 > 2 * PyLong_SHIFT) {
                        if (8 * sizeof(unsigned long) > 3 * PyLong_SHIFT) {
                            __PYX_VERIFY_RETURN_INT(grpc_status_code, long, -(long) (((((((unsigned long)digits[2]) << PyLong_SHIFT) | (unsigned long)digits[1]) << PyLong_SHIFT) | (unsigned long)digits[0])))
                        } else if (8 * sizeof(grpc_status_code) - 1 > 3 * PyLong_SHIFT) {
                            return (grpc_status_code) (((grpc_status_code)-1)*(((((((grpc_status_code)digits[2]) << PyLong_SHIFT) | (grpc_status_code)digits[1]) << PyLong_SHIFT) | (grpc_status_code)digits[0])));
                        }
                    }
                    break;
                case 3:
                    if (8 * sizeof(grpc_status_code) > 2 * PyLong_SHIFT) {
                        if (8 * sizeof(unsigned long) > 3 * PyLong_SHIFT) {
                            __PYX_VERIFY_RETURN_INT(grpc_status_code, unsigned long, (((((((unsigned long)digits[2]) << PyLong_SHIFT) | (unsigned long)digits[1]) << PyLong_SHIFT) | (unsigned long)digits[0])))
                        } else if (8 * sizeof(grpc_status_code) - 1 > 3 * PyLong_SHIFT) {
                            return (grpc_status_code) ((((((((grpc_status_code)digits[2]) << PyLong_SHIFT) | (grpc_status_code)digits[1]) << PyLong_SHIFT) | (grpc_status_code)digits[0])));
                        }
                    }
                    break;
                case -4:
                    if (8 * sizeof(grpc_status_code) - 1 > 3 * PyLong_SHIFT) {
                        if (8 * sizeof(unsigned long) > 4 * PyLong_SHIFT) {
                            __PYX_VERIFY_RETURN_INT(grpc_status_code, long, -(long) (((((((((unsigned long)digits[3]) << PyLong_SHIFT) | (unsigned long)digits[2]) << PyLong_SHIFT) | (unsigned long)digits[1]) << PyLong_SHIFT) | (unsigned long)digits[0])))
                        } else if (8 * sizeof(grpc_status_code) - 1 > 4 * PyLong_SHIFT) {
                            return (grpc_status_code) (((grpc_status_code)-1)*(((((((((grpc_status_code)digits[3]) << PyLong_SHIFT) | (grpc_status_code)digits[2]) << PyLong_SHIFT) | (grpc_status_code)digits[1]) << PyLong_SHIFT) | (grpc_status_code)digits[0])));
                        }
                    }
                    break;
                case 4:
                    if (8 * sizeof(grpc_status_code) > 3 * PyLong_SHIFT) {
                        if (8 * sizeof(unsigned long) > 4 * PyLong_SHIFT) {
                            __PYX_VERIFY_RETURN_INT(grpc_status_code, unsigned long, (((((((((unsigned long)digits[3]) << PyLong_SHIFT) | (unsigned long)digits[2]) << PyLong_SHIFT) | (unsigned long)digits[1]) << PyLong_SHIFT) | (unsigned long)digits[0])))
                        } else if (8 * sizeof(grpc_status_code) - 1 > 4 * PyLong_SHIFT) {
                            return (grpc_status_code) ((((((((((grpc_status_code)digits[3]) << PyLong_SHIFT) | (grpc_status_code)digits[2]) << PyLong_SHIFT) | (grpc_status_code)digits[1]) << PyLong_SHIFT) | (grpc_status_code)digits[0])));
                        }
                    }
                    break;
            }
#endif
            if (sizeof(grpc_status_code) <= sizeof(long)) {
                __PYX_VERIFY_RETURN_INT_EXC(grpc_status_code, long, PyLong_AsLong(x))
#ifdef HAVE_LONG_LONG
            } else if (sizeof(grpc_status_code) <= sizeof(PY_LONG_LONG)) {
                __PYX_VERIFY_RETURN_INT_EXC(grpc_status_code, PY_LONG_LONG, PyLong_AsLongLong(x))
#endif
            }
        }
        {
#if CYTHON_COMPILING_IN_PYPY && !defined(_PyLong_AsByteArray)
            PyErr_SetString(PyExc_RuntimeError,
                            "_PyLong_AsByteArray() not available in PyPy, cannot convert large numbers");
#else
            grpc_status_code val;
            PyObject *v = __Pyx_PyNumber_IntOrLong(x);
 #if PY_MAJOR_VERSION < 3
            if (likely(v) && !PyLong_Check(v)) {
                PyObject *tmp = v;
                v = PyNumber_Long(tmp);
                Py_DECREF(tmp);
            }
 #endif
            if (likely(v)) {
                int one = 1; int is_little = (int)*(unsigned char *)&one;
                unsigned char *bytes = (unsigned char *)&val;
                int ret = _PyLong_AsByteArray((PyLongObject *)v,
                                              bytes, sizeof(val),
                                              is_little, !is_unsigned);
                Py_DECREF(v);
                if (likely(!ret))
                    return val;
            }
#endif
            return (grpc_status_code) -1;
        }
    } else {
        grpc_status_code val;
        PyObject *tmp = __Pyx_PyNumber_IntOrLong(x);
        if (!tmp) return (grpc_status_code) -1;
        val = __Pyx_PyInt_As_grpc_status_code(tmp);
        Py_DECREF(tmp);
        return val;
    }
raise_overflow:
    PyErr_SetString(PyExc_OverflowError,
        "value too large to convert to grpc_status_code");
    return (grpc_status_code) -1;
raise_neg_overflow:
    PyErr_SetString(PyExc_OverflowError,
        "can't convert negative value to grpc_status_code");
    return (grpc_status_code) -1;
}

/* CIntFromPy */
                static CYTHON_INLINE int __Pyx_PyInt_As_int(PyObject *x) {
    const int neg_one = (int) -1, const_zero = (int) 0;
    const int is_unsigned = neg_one > const_zero;
#if PY_MAJOR_VERSION < 3
    if (likely(PyInt_Check(x))) {
        if (sizeof(int) < sizeof(long)) {
            __PYX_VERIFY_RETURN_INT(int, long, PyInt_AS_LONG(x))
        } else {
            long val = PyInt_AS_LONG(x);
            if (is_unsigned && unlikely(val < 0)) {
                goto raise_neg_overflow;
            }
            return (int) val;
        }
    } else
#endif
    if (likely(PyLong_Check(x))) {
        if (is_unsigned) {
#if CYTHON_USE_PYLONG_INTERNALS
            const digit* digits = ((PyLongObject*)x)->ob_digit;
            switch (Py_SIZE(x)) {
                case  0: return (int) 0;
                case  1: __PYX_VERIFY_RETURN_INT(int, digit, digits[0])
                case 2:
                    if (8 * sizeof(int) > 1 * PyLong_SHIFT) {
                        if (8 * sizeof(unsigned long) > 2 * PyLong_SHIFT) {
                            __PYX_VERIFY_RETURN_INT(int, unsigned long, (((((unsigned long)digits[1]) << PyLong_SHIFT) | (unsigned long)digits[0])))
                        } else if (8 * sizeof(int) >= 2 * PyLong_SHIFT) {
                            return (int) (((((int)digits[1]) << PyLong_SHIFT) | (int)digits[0]));
                        }
                    }
                    break;
                case 3:
                    if (8 * sizeof(int) > 2 * PyLong_SHIFT) {
                        if (8 * sizeof(unsigned long) > 3 * PyLong_SHIFT) {
                            __PYX_VERIFY_RETURN_INT(int, unsigned long, (((((((unsigned long)digits[2]) << PyLong_SHIFT) | (unsigned long)digits[1]) << PyLong_SHIFT) | (unsigned long)digits[0])))
                        } else if (8 * sizeof(int) >= 3 * PyLong_SHIFT) {
                            return (int) (((((((int)digits[2]) << PyLong_SHIFT) | (int)digits[1]) << PyLong_SHIFT) | (int)digits[0]));
                        }
                    }
                    break;
                case 4:
                    if (8 * sizeof(int) > 3 * PyLong_SHIFT) {
                        if (8 * sizeof(unsigned long) > 4 * PyLong_SHIFT) {
                            __PYX_VERIFY_RETURN_INT(int, unsigned long, (((((((((unsigned long)digits[3]) << PyLong_SHIFT) | (unsigned long)digits[2]) << PyLong_SHIFT) | (unsigned long)digits[1]) << PyLong_SHIFT) | (unsigned long)digits[0])))
                        } else if (8 * sizeof(int) >= 4 * PyLong_SHIFT) {
                            return (int) (((((((((int)digits[3]) << PyLong_SHIFT) | (int)digits[2]) << PyLong_SHIFT) | (int)digits[1]) << PyLong_SHIFT) | (int)digits[0]));
                        }
                    }
                    break;
            }
#endif
#if CYTHON_COMPILING_IN_CPYTHON
            if (unlikely(Py_SIZE(x) < 0)) {
                goto raise_neg_overflow;
            }
#else
            {
                int result = PyObject_RichCompareBool(x, Py_False, Py_LT);
                if (unlikely(result < 0))
                    return (int) -1;
                if (unlikely(result == 1))
                    goto raise_neg_overflow;
            }
#endif
            if (sizeof(int) <= sizeof(unsigned long)) {
                __PYX_VERIFY_RETURN_INT_EXC(int, unsigned long, PyLong_AsUnsignedLong(x))
#ifdef HAVE_LONG_LONG
            } else if (sizeof(int) <= sizeof(unsigned PY_LONG_LONG)) {
                __PYX_VERIFY_RETURN_INT_EXC(int, unsigned PY_LONG_LONG, PyLong_AsUnsignedLongLong(x))
#endif
            }
        } else {
#if CYTHON_USE_PYLONG_INTERNALS
            const digit* digits = ((PyLongObject*)x)->ob_digit;
            switch (Py_SIZE(x)) {
                case  0: return (int) 0;
                case -1: __PYX_VERIFY_RETURN_INT(int, sdigit, (sdigit) (-(sdigit)digits[0]))
                case  1: __PYX_VERIFY_RETURN_INT(int,  digit, +digits[0])
                case -2:
                    if (8 * sizeof(int) - 1 > 1 * PyLong_SHIFT) {
                        if (8 * sizeof(unsigned long) > 2 * PyLong_SHIFT) {
                            __PYX_VERIFY_RETURN_INT(int, long, -(long) (((((unsigned long)digits[1]) << PyLong_SHIFT) | (unsigned long)digits[0])))
                        } else if (8 * sizeof(int) - 1 > 2 * PyLong_SHIFT) {
                            return (int) (((int)-1)*(((((int)digits[1]) << PyLong_SHIFT) | (int)digits[0])));
                        }
                    }
                    break;
                case 2:
                    if (8 * sizeof(int) > 1 * PyLong_SHIFT) {
                        if (8 * sizeof(unsigned long) > 2 * PyLong_SHIFT) {
                            __PYX_VERIFY_RETURN_INT(int, unsigned long, (((((unsigned long)digits[1]) << PyLong_SHIFT) | (unsigned long)digits[0])))
                        } else if (8 * sizeof(int) - 1 > 2 * PyLong_SHIFT) {
                            return (int) ((((((int)digits[1]) << PyLong_SHIFT) | (int)digits[0])));
                        }
                    }
                    break;
                case -3:
                    if (8 * sizeof(int) - 1 > 2 * PyLong_SHIFT) {
                        if (8 * sizeof(unsigned long) > 3 * PyLong_SHIFT) {
                            __PYX_VERIFY_RETURN_INT(int, long, -(long) (((((((unsigned long)digits[2]) << PyLong_SHIFT) | (unsigned long)digits[1]) << PyLong_SHIFT) | (unsigned long)digits[0])))
                        } else if (8 * sizeof(int) - 1 > 3 * PyLong_SHIFT) {
                            return (int) (((int)-1)*(((((((int)digits[2]) << PyLong_SHIFT) | (int)digits[1]) << PyLong_SHIFT) | (int)digits[0])));
                        }
                    }
                    break;
                case 3:
                    if (8 * sizeof(int) > 2 * PyLong_SHIFT) {
                        if (8 * sizeof(unsigned long) > 3 * PyLong_SHIFT) {
                            __PYX_VERIFY_RETURN_INT(int, unsigned long, (((((((unsigned long)digits[2]) << PyLong_SHIFT) | (unsigned long)digits[1]) << PyLong_SHIFT) | (unsigned long)digits[0])))
                        } else if (8 * sizeof(int) - 1 > 3 * PyLong_SHIFT) {
                            return (int) ((((((((int)digits[2]) << PyLong_SHIFT) | (int)digits[1]) << PyLong_SHIFT) | (int)digits[0])));
                        }
                    }
                    break;
                case -4:
                    if (8 * sizeof(int) - 1 > 3 * PyLong_SHIFT) {
                        if (8 * sizeof(unsigned long) > 4 * PyLong_SHIFT) {
                            __PYX_VERIFY_RETURN_INT(int, long, -(long) (((((((((unsigned long)digits[3]) << PyLong_SHIFT) | (unsigned long)digits[2]) << PyLong_SHIFT) | (unsigned long)digits[1]) << PyLong_SHIFT) | (unsigned long)digits[0])))
                        } else if (8 * sizeof(int) - 1 > 4 * PyLong_SHIFT) {
                            return (int) (((int)-1)*(((((((((int)digits[3]) << PyLong_SHIFT) | (int)digits[2]) << PyLong_SHIFT) | (int)digits[1]) << PyLong_SHIFT) | (int)digits[0])));
                        }
                    }
                    break;
                case 4:
                    if (8 * sizeof(int) > 3 * PyLong_SHIFT) {
                        if (8 * sizeof(unsigned long) > 4 * PyLong_SHIFT) {
                            __PYX_VERIFY_RETURN_INT(int, unsigned long, (((((((((unsigned long)digits[3]) << PyLong_SHIFT) | (unsigned long)digits[2]) << PyLong_SHIFT) | (unsigned long)digits[1]) << PyLong_SHIFT) | (unsigned long)digits[0])))
                        } else if (8 * sizeof(int) - 1 > 4 * PyLong_SHIFT) {
                            return (int) ((((((((((int)digits[3]) << PyLong_SHIFT) | (int)digits[2]) << PyLong_SHIFT) | (int)digits[1]) << PyLong_SHIFT) | (int)digits[0])));
                        }
                    }
                    break;
            }
#endif
            if (sizeof(int) <= sizeof(long)) {
                __PYX_VERIFY_RETURN_INT_EXC(int, long, PyLong_AsLong(x))
#ifdef HAVE_LONG_LONG
            } else if (sizeof(int) <= sizeof(PY_LONG_LONG)) {
                __PYX_VERIFY_RETURN_INT_EXC(int, PY_LONG_LONG, PyLong_AsLongLong(x))
#endif
            }
        }
        {
#if CYTHON_COMPILING_IN_PYPY && !defined(_PyLong_AsByteArray)
            PyErr_SetString(PyExc_RuntimeError,
                            "_PyLong_AsByteArray() not available in PyPy, cannot convert large numbers");
#else
            int val;
            PyObject *v = __Pyx_PyNumber_IntOrLong(x);
 #if PY_MAJOR_VERSION < 3
            if (likely(v) && !PyLong_Check(v)) {
                PyObject *tmp = v;
                v = PyNumber_Long(tmp);
                Py_DECREF(tmp);
            }
 #endif
            if (likely(v)) {
                int one = 1; int is_little = (int)*(unsigned char *)&one;
                unsigned char *bytes = (unsigned char *)&val;
                int ret = _PyLong_AsByteArray((PyLongObject *)v,
                                              bytes, sizeof(val),
                                              is_little, !is_unsigned);
                Py_DECREF(v);
                if (likely(!ret))
                    return val;
            }
#endif
            return (int) -1;
        }
    } else {
        int val;
        PyObject *tmp = __Pyx_PyNumber_IntOrLong(x);
        if (!tmp) return (int) -1;
        val = __Pyx_PyInt_As_int(tmp);
        Py_DECREF(tmp);
        return val;
    }
raise_overflow:
    PyErr_SetString(PyExc_OverflowError,
        "value too large to convert to int");
    return (int) -1;
raise_neg_overflow:
    PyErr_SetString(PyExc_OverflowError,
        "can't convert negative value to int");
    return (int) -1;
}

/* CIntFromPy */
                static CYTHON_INLINE grpc_connectivity_state __Pyx_PyInt_As_grpc_connectivity_state(PyObject *x) {
    const grpc_connectivity_state neg_one = (grpc_connectivity_state) -1, const_zero = (grpc_connectivity_state) 0;
    const int is_unsigned = neg_one > const_zero;
#if PY_MAJOR_VERSION < 3
    if (likely(PyInt_Check(x))) {
        if (sizeof(grpc_connectivity_state) < sizeof(long)) {
            __PYX_VERIFY_RETURN_INT(grpc_connectivity_state, long, PyInt_AS_LONG(x))
        } else {
            long val = PyInt_AS_LONG(x);
            if (is_unsigned && unlikely(val < 0)) {
                goto raise_neg_overflow;
            }
            return (grpc_connectivity_state) val;
        }
    } else
#endif
    if (likely(PyLong_Check(x))) {
        if (is_unsigned) {
#if CYTHON_USE_PYLONG_INTERNALS
            const digit* digits = ((PyLongObject*)x)->ob_digit;
            switch (Py_SIZE(x)) {
                case  0: return (grpc_connectivity_state) 0;
                case  1: __PYX_VERIFY_RETURN_INT(grpc_connectivity_state, digit, digits[0])
                case 2:
                    if (8 * sizeof(grpc_connectivity_state) > 1 * PyLong_SHIFT) {
                        if (8 * sizeof(unsigned long) > 2 * PyLong_SHIFT) {
                            __PYX_VERIFY_RETURN_INT(grpc_connectivity_state, unsigned long, (((((unsigned long)digits[1]) << PyLong_SHIFT) | (unsigned long)digits[0])))
                        } else if (8 * sizeof(grpc_connectivity_state) >= 2 * PyLong_SHIFT) {
                            return (grpc_connectivity_state) (((((grpc_connectivity_state)digits[1]) << PyLong_SHIFT) | (grpc_connectivity_state)digits[0]));
                        }
                    }
                    break;
                case 3:
                    if (8 * sizeof(grpc_connectivity_state) > 2 * PyLong_SHIFT) {
                        if (8 * sizeof(unsigned long) > 3 * PyLong_SHIFT) {
                            __PYX_VERIFY_RETURN_INT(grpc_connectivity_state, unsigned long, (((((((unsigned long)digits[2]) << PyLong_SHIFT) | (unsigned long)digits[1]) << PyLong_SHIFT) | (unsigned long)digits[0])))
                        } else if (8 * sizeof(grpc_connectivity_state) >= 3 * PyLong_SHIFT) {
                            return (grpc_connectivity_state) (((((((grpc_connectivity_state)digits[2]) << PyLong_SHIFT) | (grpc_connectivity_state)digits[1]) << PyLong_SHIFT) | (grpc_connectivity_state)digits[0]));
                        }
                    }
                    break;
                case 4:
                    if (8 * sizeof(grpc_connectivity_state) > 3 * PyLong_SHIFT) {
                        if (8 * sizeof(unsigned long) > 4 * PyLong_SHIFT) {
                            __PYX_VERIFY_RETURN_INT(grpc_connectivity_state, unsigned long, (((((((((unsigned long)digits[3]) << PyLong_SHIFT) | (unsigned long)digits[2]) << PyLong_SHIFT) | (unsigned long)digits[1]) << PyLong_SHIFT) | (unsigned long)digits[0])))
                        } else if (8 * sizeof(grpc_connectivity_state) >= 4 * PyLong_SHIFT) {
                            return (grpc_connectivity_state) (((((((((grpc_connectivity_state)digits[3]) << PyLong_SHIFT) | (grpc_connectivity_state)digits[2]) << PyLong_SHIFT) | (grpc_connectivity_state)digits[1]) << PyLong_SHIFT) | (grpc_connectivity_state)digits[0]));
                        }
                    }
                    break;
            }
#endif
#if CYTHON_COMPILING_IN_CPYTHON
            if (unlikely(Py_SIZE(x) < 0)) {
                goto raise_neg_overflow;
            }
#else
            {
                int result = PyObject_RichCompareBool(x, Py_False, Py_LT);
                if (unlikely(result < 0))
                    return (grpc_connectivity_state) -1;
                if (unlikely(result == 1))
                    goto raise_neg_overflow;
            }
#endif
            if (sizeof(grpc_connectivity_state) <= sizeof(unsigned long)) {
                __PYX_VERIFY_RETURN_INT_EXC(grpc_connectivity_state, unsigned long, PyLong_AsUnsignedLong(x))
#ifdef HAVE_LONG_LONG
            } else if (sizeof(grpc_connectivity_state) <= sizeof(unsigned PY_LONG_LONG)) {
                __PYX_VERIFY_RETURN_INT_EXC(grpc_connectivity_state, unsigned PY_LONG_LONG, PyLong_AsUnsignedLongLong(x))
#endif
            }
        } else {
#if CYTHON_USE_PYLONG_INTERNALS
            const digit* digits = ((PyLongObject*)x)->ob_digit;
            switch (Py_SIZE(x)) {
                case  0: return (grpc_connectivity_state) 0;
                case -1: __PYX_VERIFY_RETURN_INT(grpc_connectivity_state, sdigit, (sdigit) (-(sdigit)digits[0]))
                case  1: __PYX_VERIFY_RETURN_INT(grpc_connectivity_state,  digit, +digits[0])
                case -2:
                    if (8 * sizeof(grpc_connectivity_state) - 1 > 1 * PyLong_SHIFT) {
                        if (8 * sizeof(unsigned long) > 2 * PyLong_SHIFT) {
                            __PYX_VERIFY_RETURN_INT(grpc_connectivity_state, long, -(long) (((((unsigned long)digits[1]) << PyLong_SHIFT) | (unsigned long)digits[0])))
                        } else if (8 * sizeof(grpc_connectivity_state) - 1 > 2 * PyLong_SHIFT) {
                            return (grpc_connectivity_state) (((grpc_connectivity_state)-1)*(((((grpc_connectivity_state)digits[1]) << PyLong_SHIFT) | (grpc_connectivity_state)digits[0])));
                        }
                    }
                    break;
                case 2:
                    if (8 * sizeof(grpc_connectivity_state) > 1 * PyLong_SHIFT) {
                        if (8 * sizeof(unsigned long) > 2 * PyLong_SHIFT) {
                            __PYX_VERIFY_RETURN_INT(grpc_connectivity_state, unsigned long, (((((unsigned long)digits[1]) << PyLong_SHIFT) | (unsigned long)digits[0])))
                        } else if (8 * sizeof(grpc_connectivity_state) - 1 > 2 * PyLong_SHIFT) {
                            return (grpc_connectivity_state) ((((((grpc_connectivity_state)digits[1]) << PyLong_SHIFT) | (grpc_connectivity_state)digits[0])));
                        }
                    }
                    break;
                case -3:
                    if (8 * sizeof(grpc_connectivity_state) - 1 > 2 * PyLong_SHIFT) {
                        if (8 * sizeof(unsigned long) > 3 * PyLong_SHIFT) {
                            __PYX_VERIFY_RETURN_INT(grpc_connectivity_state, long, -(long) (((((((unsigned long)digits[2]) << PyLong_SHIFT) | (unsigned long)digits[1]) << PyLong_SHIFT) | (unsigned long)digits[0])))
                        } else if (8 * sizeof(grpc_connectivity_state) - 1 > 3 * PyLong_SHIFT) {
                            return (grpc_connectivity_state) (((grpc_connectivity_state)-1)*(((((((grpc_connectivity_state)digits[2]) << PyLong_SHIFT) | (grpc_connectivity_state)digits[1]) << PyLong_SHIFT) | (grpc_connectivity_state)digits[0])));
                        }
                    }
                    break;
                case 3:
                    if (8 * sizeof(grpc_connectivity_state) > 2 * PyLong_SHIFT) {
                        if (8 * sizeof(unsigned long) > 3 * PyLong_SHIFT) {
                            __PYX_VERIFY_RETURN_INT(grpc_connectivity_state, unsigned long, (((((((unsigned long)digits[2]) << PyLong_SHIFT) | (unsigned long)digits[1]) << PyLong_SHIFT) | (unsigned long)digits[0])))
                        } else if (8 * sizeof(grpc_connectivity_state) - 1 > 3 * PyLong_SHIFT) {
                            return (grpc_connectivity_state) ((((((((grpc_connectivity_state)digits[2]) << PyLong_SHIFT) | (grpc_connectivity_state)digits[1]) << PyLong_SHIFT) | (grpc_connectivity_state)digits[0])));
                        }
                    }
                    break;
                case -4:
                    if (8 * sizeof(grpc_connectivity_state) - 1 > 3 * PyLong_SHIFT) {
                        if (8 * sizeof(unsigned long) > 4 * PyLong_SHIFT) {
                            __PYX_VERIFY_RETURN_INT(grpc_connectivity_state, long, -(long) (((((((((unsigned long)digits[3]) << PyLong_SHIFT) | (unsigned long)digits[2]) << PyLong_SHIFT) | (unsigned long)digits[1]) << PyLong_SHIFT) | (unsigned long)digits[0])))
                        } else if (8 * sizeof(grpc_connectivity_state) - 1 > 4 * PyLong_SHIFT) {
                            return (grpc_connectivity_state) (((grpc_connectivity_state)-1)*(((((((((grpc_connectivity_state)digits[3]) << PyLong_SHIFT) | (grpc_connectivity_state)digits[2]) << PyLong_SHIFT) | (grpc_connectivity_state)digits[1]) << PyLong_SHIFT) | (grpc_connectivity_state)digits[0])));
                        }
                    }
                    break;
                case 4:
                    if (8 * sizeof(grpc_connectivity_state) > 3 * PyLong_SHIFT) {
                        if (8 * sizeof(unsigned long) > 4 * PyLong_SHIFT) {
                            __PYX_VERIFY_RETURN_INT(grpc_connectivity_state, unsigned long, (((((((((unsigned long)digits[3]) << PyLong_SHIFT) | (unsigned long)digits[2]) << PyLong_SHIFT) | (unsigned long)digits[1]) << PyLong_SHIFT) | (unsigned long)digits[0])))
                        } else if (8 * sizeof(grpc_connectivity_state) - 1 > 4 * PyLong_SHIFT) {
                            return (grpc_connectivity_state) ((((((((((grpc_connectivity_state)digits[3]) << PyLong_SHIFT) | (grpc_connectivity_state)digits[2]) << PyLong_SHIFT) | (grpc_connectivity_state)digits[1]) << PyLong_SHIFT) | (grpc_connectivity_state)digits[0])));
                        }
                    }
                    break;
            }
#endif
            if (sizeof(grpc_connectivity_state) <= sizeof(long)) {
                __PYX_VERIFY_RETURN_INT_EXC(grpc_connectivity_state, long, PyLong_AsLong(x))
#ifdef HAVE_LONG_LONG
            } else if (sizeof(grpc_connectivity_state) <= sizeof(PY_LONG_LONG)) {
                __PYX_VERIFY_RETURN_INT_EXC(grpc_connectivity_state, PY_LONG_LONG, PyLong_AsLongLong(x))
#endif
            }
        }
        {
#if CYTHON_COMPILING_IN_PYPY && !defined(_PyLong_AsByteArray)
            PyErr_SetString(PyExc_RuntimeError,
                            "_PyLong_AsByteArray() not available in PyPy, cannot convert large numbers");
#else
            grpc_connectivity_state val;
            PyObject *v = __Pyx_PyNumber_IntOrLong(x);
 #if PY_MAJOR_VERSION < 3
            if (likely(v) && !PyLong_Check(v)) {
                PyObject *tmp = v;
                v = PyNumber_Long(tmp);
                Py_DECREF(tmp);
            }
 #endif
            if (likely(v)) {
                int one = 1; int is_little = (int)*(unsigned char *)&one;
                unsigned char *bytes = (unsigned char *)&val;
                int ret = _PyLong_AsByteArray((PyLongObject *)v,
                                              bytes, sizeof(val),
                                              is_little, !is_unsigned);
                Py_DECREF(v);
                if (likely(!ret))
                    return val;
            }
#endif
            return (grpc_connectivity_state) -1;
        }
    } else {
        grpc_connectivity_state val;
        PyObject *tmp = __Pyx_PyNumber_IntOrLong(x);
        if (!tmp) return (grpc_connectivity_state) -1;
        val = __Pyx_PyInt_As_grpc_connectivity_state(tmp);
        Py_DECREF(tmp);
        return val;
    }
raise_overflow:
    PyErr_SetString(PyExc_OverflowError,
        "value too large to convert to grpc_connectivity_state");
    return (grpc_connectivity_state) -1;
raise_neg_overflow:
    PyErr_SetString(PyExc_OverflowError,
        "can't convert negative value to grpc_connectivity_state");
    return (grpc_connectivity_state) -1;
}

/* CIntFromPy */
                static CYTHON_INLINE grpc_completion_type __Pyx_PyInt_As_grpc_completion_type(PyObject *x) {
    const grpc_completion_type neg_one = (grpc_completion_type) -1, const_zero = (grpc_completion_type) 0;
    const int is_unsigned = neg_one > const_zero;
#if PY_MAJOR_VERSION < 3
    if (likely(PyInt_Check(x))) {
        if (sizeof(grpc_completion_type) < sizeof(long)) {
            __PYX_VERIFY_RETURN_INT(grpc_completion_type, long, PyInt_AS_LONG(x))
        } else {
            long val = PyInt_AS_LONG(x);
            if (is_unsigned && unlikely(val < 0)) {
                goto raise_neg_overflow;
            }
            return (grpc_completion_type) val;
        }
    } else
#endif
    if (likely(PyLong_Check(x))) {
        if (is_unsigned) {
#if CYTHON_USE_PYLONG_INTERNALS
            const digit* digits = ((PyLongObject*)x)->ob_digit;
            switch (Py_SIZE(x)) {
                case  0: return (grpc_completion_type) 0;
                case  1: __PYX_VERIFY_RETURN_INT(grpc_completion_type, digit, digits[0])
                case 2:
                    if (8 * sizeof(grpc_completion_type) > 1 * PyLong_SHIFT) {
                        if (8 * sizeof(unsigned long) > 2 * PyLong_SHIFT) {
                            __PYX_VERIFY_RETURN_INT(grpc_completion_type, unsigned long, (((((unsigned long)digits[1]) << PyLong_SHIFT) | (unsigned long)digits[0])))
                        } else if (8 * sizeof(grpc_completion_type) >= 2 * PyLong_SHIFT) {
                            return (grpc_completion_type) (((((grpc_completion_type)digits[1]) << PyLong_SHIFT) | (grpc_completion_type)digits[0]));
                        }
                    }
                    break;
                case 3:
                    if (8 * sizeof(grpc_completion_type) > 2 * PyLong_SHIFT) {
                        if (8 * sizeof(unsigned long) > 3 * PyLong_SHIFT) {
                            __PYX_VERIFY_RETURN_INT(grpc_completion_type, unsigned long, (((((((unsigned long)digits[2]) << PyLong_SHIFT) | (unsigned long)digits[1]) << PyLong_SHIFT) | (unsigned long)digits[0])))
                        } else if (8 * sizeof(grpc_completion_type) >= 3 * PyLong_SHIFT) {
                            return (grpc_completion_type) (((((((grpc_completion_type)digits[2]) << PyLong_SHIFT) | (grpc_completion_type)digits[1]) << PyLong_SHIFT) | (grpc_completion_type)digits[0]));
                        }
                    }
                    break;
                case 4:
                    if (8 * sizeof(grpc_completion_type) > 3 * PyLong_SHIFT) {
                        if (8 * sizeof(unsigned long) > 4 * PyLong_SHIFT) {
                            __PYX_VERIFY_RETURN_INT(grpc_completion_type, unsigned long, (((((((((unsigned long)digits[3]) << PyLong_SHIFT) | (unsigned long)digits[2]) << PyLong_SHIFT) | (unsigned long)digits[1]) << PyLong_SHIFT) | (unsigned long)digits[0])))
                        } else if (8 * sizeof(grpc_completion_type) >= 4 * PyLong_SHIFT) {
                            return (grpc_completion_type) (((((((((grpc_completion_type)digits[3]) << PyLong_SHIFT) | (grpc_completion_type)digits[2]) << PyLong_SHIFT) | (grpc_completion_type)digits[1]) << PyLong_SHIFT) | (grpc_completion_type)digits[0]));
                        }
                    }
                    break;
            }
#endif
#if CYTHON_COMPILING_IN_CPYTHON
            if (unlikely(Py_SIZE(x) < 0)) {
                goto raise_neg_overflow;
            }
#else
            {
                int result = PyObject_RichCompareBool(x, Py_False, Py_LT);
                if (unlikely(result < 0))
                    return (grpc_completion_type) -1;
                if (unlikely(result == 1))
                    goto raise_neg_overflow;
            }
#endif
            if (sizeof(grpc_completion_type) <= sizeof(unsigned long)) {
                __PYX_VERIFY_RETURN_INT_EXC(grpc_completion_type, unsigned long, PyLong_AsUnsignedLong(x))
#ifdef HAVE_LONG_LONG
            } else if (sizeof(grpc_completion_type) <= sizeof(unsigned PY_LONG_LONG)) {
                __PYX_VERIFY_RETURN_INT_EXC(grpc_completion_type, unsigned PY_LONG_LONG, PyLong_AsUnsignedLongLong(x))
#endif
            }
        } else {
#if CYTHON_USE_PYLONG_INTERNALS
            const digit* digits = ((PyLongObject*)x)->ob_digit;
            switch (Py_SIZE(x)) {
                case  0: return (grpc_completion_type) 0;
                case -1: __PYX_VERIFY_RETURN_INT(grpc_completion_type, sdigit, (sdigit) (-(sdigit)digits[0]))
                case  1: __PYX_VERIFY_RETURN_INT(grpc_completion_type,  digit, +digits[0])
                case -2:
                    if (8 * sizeof(grpc_completion_type) - 1 > 1 * PyLong_SHIFT) {
                        if (8 * sizeof(unsigned long) > 2 * PyLong_SHIFT) {
                            __PYX_VERIFY_RETURN_INT(grpc_completion_type, long, -(long) (((((unsigned long)digits[1]) << PyLong_SHIFT) | (unsigned long)digits[0])))
                        } else if (8 * sizeof(grpc_completion_type) - 1 > 2 * PyLong_SHIFT) {
                            return (grpc_completion_type) (((grpc_completion_type)-1)*(((((grpc_completion_type)digits[1]) << PyLong_SHIFT) | (grpc_completion_type)digits[0])));
                        }
                    }
                    break;
                case 2:
                    if (8 * sizeof(grpc_completion_type) > 1 * PyLong_SHIFT) {
                        if (8 * sizeof(unsigned long) > 2 * PyLong_SHIFT) {
                            __PYX_VERIFY_RETURN_INT(grpc_completion_type, unsigned long, (((((unsigned long)digits[1]) << PyLong_SHIFT) | (unsigned long)digits[0])))
                        } else if (8 * sizeof(grpc_completion_type) - 1 > 2 * PyLong_SHIFT) {
                            return (grpc_completion_type) ((((((grpc_completion_type)digits[1]) << PyLong_SHIFT) | (grpc_completion_type)digits[0])));
                        }
                    }
                    break;
                case -3:
                    if (8 * sizeof(grpc_completion_type) - 1 > 2 * PyLong_SHIFT) {
                        if (8 * sizeof(unsigned long) > 3 * PyLong_SHIFT) {
                            __PYX_VERIFY_RETURN_INT(grpc_completion_type, long, -(long) (((((((unsigned long)digits[2]) << PyLong_SHIFT) | (unsigned long)digits[1]) << PyLong_SHIFT) | (unsigned long)digits[0])))
                        } else if (8 * sizeof(grpc_completion_type) - 1 > 3 * PyLong_SHIFT) {
                            return (grpc_completion_type) (((grpc_completion_type)-1)*(((((((grpc_completion_type)digits[2]) << PyLong_SHIFT) | (grpc_completion_type)digits[1]) << PyLong_SHIFT) | (grpc_completion_type)digits[0])));
                        }
                    }
                    break;
                case 3:
                    if (8 * sizeof(grpc_completion_type) > 2 * PyLong_SHIFT) {
                        if (8 * sizeof(unsigned long) > 3 * PyLong_SHIFT) {
                            __PYX_VERIFY_RETURN_INT(grpc_completion_type, unsigned long, (((((((unsigned long)digits[2]) << PyLong_SHIFT) | (unsigned long)digits[1]) << PyLong_SHIFT) | (unsigned long)digits[0])))
                        } else if (8 * sizeof(grpc_completion_type) - 1 > 3 * PyLong_SHIFT) {
                            return (grpc_completion_type) ((((((((grpc_completion_type)digits[2]) << PyLong_SHIFT) | (grpc_completion_type)digits[1]) << PyLong_SHIFT) | (grpc_completion_type)digits[0])));
                        }
                    }
                    break;
                case -4:
                    if (8 * sizeof(grpc_completion_type) - 1 > 3 * PyLong_SHIFT) {
                        if (8 * sizeof(unsigned long) > 4 * PyLong_SHIFT) {
                            __PYX_VERIFY_RETURN_INT(grpc_completion_type, long, -(long) (((((((((unsigned long)digits[3]) << PyLong_SHIFT) | (unsigned long)digits[2]) << PyLong_SHIFT) | (unsigned long)digits[1]) << PyLong_SHIFT) | (unsigned long)digits[0])))
                        } else if (8 * sizeof(grpc_completion_type) - 1 > 4 * PyLong_SHIFT) {
                            return (grpc_completion_type) (((grpc_completion_type)-1)*(((((((((grpc_completion_type)digits[3]) << PyLong_SHIFT) | (grpc_completion_type)digits[2]) << PyLong_SHIFT) | (grpc_completion_type)digits[1]) << PyLong_SHIFT) | (grpc_completion_type)digits[0])));
                        }
                    }
                    break;
                case 4:
                    if (8 * sizeof(grpc_completion_type) > 3 * PyLong_SHIFT) {
                        if (8 * sizeof(unsigned long) > 4 * PyLong_SHIFT) {
                            __PYX_VERIFY_RETURN_INT(grpc_completion_type, unsigned long, (((((((((unsigned long)digits[3]) << PyLong_SHIFT) | (unsigned long)digits[2]) << PyLong_SHIFT) | (unsigned long)digits[1]) << PyLong_SHIFT) | (unsigned long)digits[0])))
                        } else if (8 * sizeof(grpc_completion_type) - 1 > 4 * PyLong_SHIFT) {
                            return (grpc_completion_type) ((((((((((grpc_completion_type)digits[3]) << PyLong_SHIFT) | (grpc_completion_type)digits[2]) << PyLong_SHIFT) | (grpc_completion_type)digits[1]) << PyLong_SHIFT) | (grpc_completion_type)digits[0])));
                        }
                    }
                    break;
            }
#endif
            if (sizeof(grpc_completion_type) <= sizeof(long)) {
                __PYX_VERIFY_RETURN_INT_EXC(grpc_completion_type, long, PyLong_AsLong(x))
#ifdef HAVE_LONG_LONG
            } else if (sizeof(grpc_completion_type) <= sizeof(PY_LONG_LONG)) {
                __PYX_VERIFY_RETURN_INT_EXC(grpc_completion_type, PY_LONG_LONG, PyLong_AsLongLong(x))
#endif
            }
        }
        {
#if CYTHON_COMPILING_IN_PYPY && !defined(_PyLong_AsByteArray)
            PyErr_SetString(PyExc_RuntimeError,
                            "_PyLong_AsByteArray() not available in PyPy, cannot convert large numbers");
#else
            grpc_completion_type val;
            PyObject *v = __Pyx_PyNumber_IntOrLong(x);
 #if PY_MAJOR_VERSION < 3
            if (likely(v) && !PyLong_Check(v)) {
                PyObject *tmp = v;
                v = PyNumber_Long(tmp);
                Py_DECREF(tmp);
            }
 #endif
            if (likely(v)) {
                int one = 1; int is_little = (int)*(unsigned char *)&one;
                unsigned char *bytes = (unsigned char *)&val;
                int ret = _PyLong_AsByteArray((PyLongObject *)v,
                                              bytes, sizeof(val),
                                              is_little, !is_unsigned);
                Py_DECREF(v);
                if (likely(!ret))
                    return val;
            }
#endif
            return (grpc_completion_type) -1;
        }
    } else {
        grpc_completion_type val;
        PyObject *tmp = __Pyx_PyNumber_IntOrLong(x);
        if (!tmp) return (grpc_completion_type) -1;
        val = __Pyx_PyInt_As_grpc_completion_type(tmp);
        Py_DECREF(tmp);
        return val;
    }
raise_overflow:
    PyErr_SetString(PyExc_OverflowError,
        "value too large to convert to grpc_completion_type");
    return (grpc_completion_type) -1;
raise_neg_overflow:
    PyErr_SetString(PyExc_OverflowError,
        "can't convert negative value to grpc_completion_type");
    return (grpc_completion_type) -1;
}

/* CIntFromPy */
                static CYTHON_INLINE grpc_compression_algorithm __Pyx_PyInt_As_grpc_compression_algorithm(PyObject *x) {
    const grpc_compression_algorithm neg_one = (grpc_compression_algorithm) -1, const_zero = (grpc_compression_algorithm) 0;
    const int is_unsigned = neg_one > const_zero;
#if PY_MAJOR_VERSION < 3
    if (likely(PyInt_Check(x))) {
        if (sizeof(grpc_compression_algorithm) < sizeof(long)) {
            __PYX_VERIFY_RETURN_INT(grpc_compression_algorithm, long, PyInt_AS_LONG(x))
        } else {
            long val = PyInt_AS_LONG(x);
            if (is_unsigned && unlikely(val < 0)) {
                goto raise_neg_overflow;
            }
            return (grpc_compression_algorithm) val;
        }
    } else
#endif
    if (likely(PyLong_Check(x))) {
        if (is_unsigned) {
#if CYTHON_USE_PYLONG_INTERNALS
            const digit* digits = ((PyLongObject*)x)->ob_digit;
            switch (Py_SIZE(x)) {
                case  0: return (grpc_compression_algorithm) 0;
                case  1: __PYX_VERIFY_RETURN_INT(grpc_compression_algorithm, digit, digits[0])
                case 2:
                    if (8 * sizeof(grpc_compression_algorithm) > 1 * PyLong_SHIFT) {
                        if (8 * sizeof(unsigned long) > 2 * PyLong_SHIFT) {
                            __PYX_VERIFY_RETURN_INT(grpc_compression_algorithm, unsigned long, (((((unsigned long)digits[1]) << PyLong_SHIFT) | (unsigned long)digits[0])))
                        } else if (8 * sizeof(grpc_compression_algorithm) >= 2 * PyLong_SHIFT) {
                            return (grpc_compression_algorithm) (((((grpc_compression_algorithm)digits[1]) << PyLong_SHIFT) | (grpc_compression_algorithm)digits[0]));
                        }
                    }
                    break;
                case 3:
                    if (8 * sizeof(grpc_compression_algorithm) > 2 * PyLong_SHIFT) {
                        if (8 * sizeof(unsigned long) > 3 * PyLong_SHIFT) {
                            __PYX_VERIFY_RETURN_INT(grpc_compression_algorithm, unsigned long, (((((((unsigned long)digits[2]) << PyLong_SHIFT) | (unsigned long)digits[1]) << PyLong_SHIFT) | (unsigned long)digits[0])))
                        } else if (8 * sizeof(grpc_compression_algorithm) >= 3 * PyLong_SHIFT) {
                            return (grpc_compression_algorithm) (((((((grpc_compression_algorithm)digits[2]) << PyLong_SHIFT) | (grpc_compression_algorithm)digits[1]) << PyLong_SHIFT) | (grpc_compression_algorithm)digits[0]));
                        }
                    }
                    break;
                case 4:
                    if (8 * sizeof(grpc_compression_algorithm) > 3 * PyLong_SHIFT) {
                        if (8 * sizeof(unsigned long) > 4 * PyLong_SHIFT) {
                            __PYX_VERIFY_RETURN_INT(grpc_compression_algorithm, unsigned long, (((((((((unsigned long)digits[3]) << PyLong_SHIFT) | (unsigned long)digits[2]) << PyLong_SHIFT) | (unsigned long)digits[1]) << PyLong_SHIFT) | (unsigned long)digits[0])))
                        } else if (8 * sizeof(grpc_compression_algorithm) >= 4 * PyLong_SHIFT) {
                            return (grpc_compression_algorithm) (((((((((grpc_compression_algorithm)digits[3]) << PyLong_SHIFT) | (grpc_compression_algorithm)digits[2]) << PyLong_SHIFT) | (grpc_compression_algorithm)digits[1]) << PyLong_SHIFT) | (grpc_compression_algorithm)digits[0]));
                        }
                    }
                    break;
            }
#endif
#if CYTHON_COMPILING_IN_CPYTHON
            if (unlikely(Py_SIZE(x) < 0)) {
                goto raise_neg_overflow;
            }
#else
            {
                int result = PyObject_RichCompareBool(x, Py_False, Py_LT);
                if (unlikely(result < 0))
                    return (grpc_compression_algorithm) -1;
                if (unlikely(result == 1))
                    goto raise_neg_overflow;
            }
#endif
            if (sizeof(grpc_compression_algorithm) <= sizeof(unsigned long)) {
                __PYX_VERIFY_RETURN_INT_EXC(grpc_compression_algorithm, unsigned long, PyLong_AsUnsignedLong(x))
#ifdef HAVE_LONG_LONG
            } else if (sizeof(grpc_compression_algorithm) <= sizeof(unsigned PY_LONG_LONG)) {
                __PYX_VERIFY_RETURN_INT_EXC(grpc_compression_algorithm, unsigned PY_LONG_LONG, PyLong_AsUnsignedLongLong(x))
#endif
            }
        } else {
#if CYTHON_USE_PYLONG_INTERNALS
            const digit* digits = ((PyLongObject*)x)->ob_digit;
            switch (Py_SIZE(x)) {
                case  0: return (grpc_compression_algorithm) 0;
                case -1: __PYX_VERIFY_RETURN_INT(grpc_compression_algorithm, sdigit, (sdigit) (-(sdigit)digits[0]))
                case  1: __PYX_VERIFY_RETURN_INT(grpc_compression_algorithm,  digit, +digits[0])
                case -2:
                    if (8 * sizeof(grpc_compression_algorithm) - 1 > 1 * PyLong_SHIFT) {
                        if (8 * sizeof(unsigned long) > 2 * PyLong_SHIFT) {
                            __PYX_VERIFY_RETURN_INT(grpc_compression_algorithm, long, -(long) (((((unsigned long)digits[1]) << PyLong_SHIFT) | (unsigned long)digits[0])))
                        } else if (8 * sizeof(grpc_compression_algorithm) - 1 > 2 * PyLong_SHIFT) {
                            return (grpc_compression_algorithm) (((grpc_compression_algorithm)-1)*(((((grpc_compression_algorithm)digits[1]) << PyLong_SHIFT) | (grpc_compression_algorithm)digits[0])));
                        }
                    }
                    break;
                case 2:
                    if (8 * sizeof(grpc_compression_algorithm) > 1 * PyLong_SHIFT) {
                        if (8 * sizeof(unsigned long) > 2 * PyLong_SHIFT) {
                            __PYX_VERIFY_RETURN_INT(grpc_compression_algorithm, unsigned long, (((((unsigned long)digits[1]) << PyLong_SHIFT) | (unsigned long)digits[0])))
                        } else if (8 * sizeof(grpc_compression_algorithm) - 1 > 2 * PyLong_SHIFT) {
                            return (grpc_compression_algorithm) ((((((grpc_compression_algorithm)digits[1]) << PyLong_SHIFT) | (grpc_compression_algorithm)digits[0])));
                        }
                    }
                    break;
                case -3:
                    if (8 * sizeof(grpc_compression_algorithm) - 1 > 2 * PyLong_SHIFT) {
                        if (8 * sizeof(unsigned long) > 3 * PyLong_SHIFT) {
                            __PYX_VERIFY_RETURN_INT(grpc_compression_algorithm, long, -(long) (((((((unsigned long)digits[2]) << PyLong_SHIFT) | (unsigned long)digits[1]) << PyLong_SHIFT) | (unsigned long)digits[0])))
                        } else if (8 * sizeof(grpc_compression_algorithm) - 1 > 3 * PyLong_SHIFT) {
                            return (grpc_compression_algorithm) (((grpc_compression_algorithm)-1)*(((((((grpc_compression_algorithm)digits[2]) << PyLong_SHIFT) | (grpc_compression_algorithm)digits[1]) << PyLong_SHIFT) | (grpc_compression_algorithm)digits[0])));
                        }
                    }
                    break;
                case 3:
                    if (8 * sizeof(grpc_compression_algorithm) > 2 * PyLong_SHIFT) {
                        if (8 * sizeof(unsigned long) > 3 * PyLong_SHIFT) {
                            __PYX_VERIFY_RETURN_INT(grpc_compression_algorithm, unsigned long, (((((((unsigned long)digits[2]) << PyLong_SHIFT) | (unsigned long)digits[1]) << PyLong_SHIFT) | (unsigned long)digits[0])))
                        } else if (8 * sizeof(grpc_compression_algorithm) - 1 > 3 * PyLong_SHIFT) {
                            return (grpc_compression_algorithm) ((((((((grpc_compression_algorithm)digits[2]) << PyLong_SHIFT) | (grpc_compression_algorithm)digits[1]) << PyLong_SHIFT) | (grpc_compression_algorithm)digits[0])));
                        }
                    }
                    break;
                case -4:
                    if (8 * sizeof(grpc_compression_algorithm) - 1 > 3 * PyLong_SHIFT) {
                        if (8 * sizeof(unsigned long) > 4 * PyLong_SHIFT) {
                            __PYX_VERIFY_RETURN_INT(grpc_compression_algorithm, long, -(long) (((((((((unsigned long)digits[3]) << PyLong_SHIFT) | (unsigned long)digits[2]) << PyLong_SHIFT) | (unsigned long)digits[1]) << PyLong_SHIFT) | (unsigned long)digits[0])))
                        } else if (8 * sizeof(grpc_compression_algorithm) - 1 > 4 * PyLong_SHIFT) {
                            return (grpc_compression_algorithm) (((grpc_compression_algorithm)-1)*(((((((((grpc_compression_algorithm)digits[3]) << PyLong_SHIFT) | (grpc_compression_algorithm)digits[2]) << PyLong_SHIFT) | (grpc_compression_algorithm)digits[1]) << PyLong_SHIFT) | (grpc_compression_algorithm)digits[0])));
                        }
                    }
                    break;
                case 4:
                    if (8 * sizeof(grpc_compression_algorithm) > 3 * PyLong_SHIFT) {
                        if (8 * sizeof(unsigned long) > 4 * PyLong_SHIFT) {
                            __PYX_VERIFY_RETURN_INT(grpc_compression_algorithm, unsigned long, (((((((((unsigned long)digits[3]) << PyLong_SHIFT) | (unsigned long)digits[2]) << PyLong_SHIFT) | (unsigned long)digits[1]) << PyLong_SHIFT) | (unsigned long)digits[0])))
                        } else if (8 * sizeof(grpc_compression_algorithm) - 1 > 4 * PyLong_SHIFT) {
                            return (grpc_compression_algorithm) ((((((((((grpc_compression_algorithm)digits[3]) << PyLong_SHIFT) | (grpc_compression_algorithm)digits[2]) << PyLong_SHIFT) | (grpc_compression_algorithm)digits[1]) << PyLong_SHIFT) | (grpc_compression_algorithm)digits[0])));
                        }
                    }
                    break;
            }
#endif
            if (sizeof(grpc_compression_algorithm) <= sizeof(long)) {
                __PYX_VERIFY_RETURN_INT_EXC(grpc_compression_algorithm, long, PyLong_AsLong(x))
#ifdef HAVE_LONG_LONG
            } else if (sizeof(grpc_compression_algorithm) <= sizeof(PY_LONG_LONG)) {
                __PYX_VERIFY_RETURN_INT_EXC(grpc_compression_algorithm, PY_LONG_LONG, PyLong_AsLongLong(x))
#endif
            }
        }
        {
#if CYTHON_COMPILING_IN_PYPY && !defined(_PyLong_AsByteArray)
            PyErr_SetString(PyExc_RuntimeError,
                            "_PyLong_AsByteArray() not available in PyPy, cannot convert large numbers");
#else
            grpc_compression_algorithm val;
            PyObject *v = __Pyx_PyNumber_IntOrLong(x);
 #if PY_MAJOR_VERSION < 3
            if (likely(v) && !PyLong_Check(v)) {
                PyObject *tmp = v;
                v = PyNumber_Long(tmp);
                Py_DECREF(tmp);
            }
 #endif
            if (likely(v)) {
                int one = 1; int is_little = (int)*(unsigned char *)&one;
                unsigned char *bytes = (unsigned char *)&val;
                int ret = _PyLong_AsByteArray((PyLongObject *)v,
                                              bytes, sizeof(val),
                                              is_little, !is_unsigned);
                Py_DECREF(v);
                if (likely(!ret))
                    return val;
            }
#endif
            return (grpc_compression_algorithm) -1;
        }
    } else {
        grpc_compression_algorithm val;
        PyObject *tmp = __Pyx_PyNumber_IntOrLong(x);
        if (!tmp) return (grpc_compression_algorithm) -1;
        val = __Pyx_PyInt_As_grpc_compression_algorithm(tmp);
        Py_DECREF(tmp);
        return val;
    }
raise_overflow:
    PyErr_SetString(PyExc_OverflowError,
        "value too large to convert to grpc_compression_algorithm");
    return (grpc_compression_algorithm) -1;
raise_neg_overflow:
    PyErr_SetString(PyExc_OverflowError,
        "can't convert negative value to grpc_compression_algorithm");
    return (grpc_compression_algorithm) -1;
}

/* CIntFromPy */
                static CYTHON_INLINE long __Pyx_PyInt_As_long(PyObject *x) {
    const long neg_one = (long) -1, const_zero = (long) 0;
    const int is_unsigned = neg_one > const_zero;
#if PY_MAJOR_VERSION < 3
    if (likely(PyInt_Check(x))) {
        if (sizeof(long) < sizeof(long)) {
            __PYX_VERIFY_RETURN_INT(long, long, PyInt_AS_LONG(x))
        } else {
            long val = PyInt_AS_LONG(x);
            if (is_unsigned && unlikely(val < 0)) {
                goto raise_neg_overflow;
            }
            return (long) val;
        }
    } else
#endif
    if (likely(PyLong_Check(x))) {
        if (is_unsigned) {
#if CYTHON_USE_PYLONG_INTERNALS
            const digit* digits = ((PyLongObject*)x)->ob_digit;
            switch (Py_SIZE(x)) {
                case  0: return (long) 0;
                case  1: __PYX_VERIFY_RETURN_INT(long, digit, digits[0])
                case 2:
                    if (8 * sizeof(long) > 1 * PyLong_SHIFT) {
                        if (8 * sizeof(unsigned long) > 2 * PyLong_SHIFT) {
                            __PYX_VERIFY_RETURN_INT(long, unsigned long, (((((unsigned long)digits[1]) << PyLong_SHIFT) | (unsigned long)digits[0])))
                        } else if (8 * sizeof(long) >= 2 * PyLong_SHIFT) {
                            return (long) (((((long)digits[1]) << PyLong_SHIFT) | (long)digits[0]));
                        }
                    }
                    break;
                case 3:
                    if (8 * sizeof(long) > 2 * PyLong_SHIFT) {
                        if (8 * sizeof(unsigned long) > 3 * PyLong_SHIFT) {
                            __PYX_VERIFY_RETURN_INT(long, unsigned long, (((((((unsigned long)digits[2]) << PyLong_SHIFT) | (unsigned long)digits[1]) << PyLong_SHIFT) | (unsigned long)digits[0])))
                        } else if (8 * sizeof(long) >= 3 * PyLong_SHIFT) {
                            return (long) (((((((long)digits[2]) << PyLong_SHIFT) | (long)digits[1]) << PyLong_SHIFT) | (long)digits[0]));
                        }
                    }
                    break;
                case 4:
                    if (8 * sizeof(long) > 3 * PyLong_SHIFT) {
                        if (8 * sizeof(unsigned long) > 4 * PyLong_SHIFT) {
                            __PYX_VERIFY_RETURN_INT(long, unsigned long, (((((((((unsigned long)digits[3]) << PyLong_SHIFT) | (unsigned long)digits[2]) << PyLong_SHIFT) | (unsigned long)digits[1]) << PyLong_SHIFT) | (unsigned long)digits[0])))
                        } else if (8 * sizeof(long) >= 4 * PyLong_SHIFT) {
                            return (long) (((((((((long)digits[3]) << PyLong_SHIFT) | (long)digits[2]) << PyLong_SHIFT) | (long)digits[1]) << PyLong_SHIFT) | (long)digits[0]));
                        }
                    }
                    break;
            }
#endif
#if CYTHON_COMPILING_IN_CPYTHON
            if (unlikely(Py_SIZE(x) < 0)) {
                goto raise_neg_overflow;
            }
#else
            {
                int result = PyObject_RichCompareBool(x, Py_False, Py_LT);
                if (unlikely(result < 0))
                    return (long) -1;
                if (unlikely(result == 1))
                    goto raise_neg_overflow;
            }
#endif
            if (sizeof(long) <= sizeof(unsigned long)) {
                __PYX_VERIFY_RETURN_INT_EXC(long, unsigned long, PyLong_AsUnsignedLong(x))
#ifdef HAVE_LONG_LONG
            } else if (sizeof(long) <= sizeof(unsigned PY_LONG_LONG)) {
                __PYX_VERIFY_RETURN_INT_EXC(long, unsigned PY_LONG_LONG, PyLong_AsUnsignedLongLong(x))
#endif
            }
        } else {
#if CYTHON_USE_PYLONG_INTERNALS
            const digit* digits = ((PyLongObject*)x)->ob_digit;
            switch (Py_SIZE(x)) {
                case  0: return (long) 0;
                case -1: __PYX_VERIFY_RETURN_INT(long, sdigit, (sdigit) (-(sdigit)digits[0]))
                case  1: __PYX_VERIFY_RETURN_INT(long,  digit, +digits[0])
                case -2:
                    if (8 * sizeof(long) - 1 > 1 * PyLong_SHIFT) {
                        if (8 * sizeof(unsigned long) > 2 * PyLong_SHIFT) {
                            __PYX_VERIFY_RETURN_INT(long, long, -(long) (((((unsigned long)digits[1]) << PyLong_SHIFT) | (unsigned long)digits[0])))
                        } else if (8 * sizeof(long) - 1 > 2 * PyLong_SHIFT) {
                            return (long) (((long)-1)*(((((long)digits[1]) << PyLong_SHIFT) | (long)digits[0])));
                        }
                    }
                    break;
                case 2:
                    if (8 * sizeof(long) > 1 * PyLong_SHIFT) {
                        if (8 * sizeof(unsigned long) > 2 * PyLong_SHIFT) {
                            __PYX_VERIFY_RETURN_INT(long, unsigned long, (((((unsigned long)digits[1]) << PyLong_SHIFT) | (unsigned long)digits[0])))
                        } else if (8 * sizeof(long) - 1 > 2 * PyLong_SHIFT) {
                            return (long) ((((((long)digits[1]) << PyLong_SHIFT) | (long)digits[0])));
                        }
                    }
                    break;
                case -3:
                    if (8 * sizeof(long) - 1 > 2 * PyLong_SHIFT) {
                        if (8 * sizeof(unsigned long) > 3 * PyLong_SHIFT) {
                            __PYX_VERIFY_RETURN_INT(long, long, -(long) (((((((unsigned long)digits[2]) << PyLong_SHIFT) | (unsigned long)digits[1]) << PyLong_SHIFT) | (unsigned long)digits[0])))
                        } else if (8 * sizeof(long) - 1 > 3 * PyLong_SHIFT) {
                            return (long) (((long)-1)*(((((((long)digits[2]) << PyLong_SHIFT) | (long)digits[1]) << PyLong_SHIFT) | (long)digits[0])));
                        }
                    }
                    break;
                case 3:
                    if (8 * sizeof(long) > 2 * PyLong_SHIFT) {
                        if (8 * sizeof(unsigned long) > 3 * PyLong_SHIFT) {
                            __PYX_VERIFY_RETURN_INT(long, unsigned long, (((((((unsigned long)digits[2]) << PyLong_SHIFT) | (unsigned long)digits[1]) << PyLong_SHIFT) | (unsigned long)digits[0])))
                        } else if (8 * sizeof(long) - 1 > 3 * PyLong_SHIFT) {
                            return (long) ((((((((long)digits[2]) << PyLong_SHIFT) | (long)digits[1]) << PyLong_SHIFT) | (long)digits[0])));
                        }
                    }
                    break;
                case -4:
                    if (8 * sizeof(long) - 1 > 3 * PyLong_SHIFT) {
                        if (8 * sizeof(unsigned long) > 4 * PyLong_SHIFT) {
                            __PYX_VERIFY_RETURN_INT(long, long, -(long) (((((((((unsigned long)digits[3]) << PyLong_SHIFT) | (unsigned long)digits[2]) << PyLong_SHIFT) | (unsigned long)digits[1]) << PyLong_SHIFT) | (unsigned long)digits[0])))
                        } else if (8 * sizeof(long) - 1 > 4 * PyLong_SHIFT) {
                            return (long) (((long)-1)*(((((((((long)digits[3]) << PyLong_SHIFT) | (long)digits[2]) << PyLong_SHIFT) | (long)digits[1]) << PyLong_SHIFT) | (long)digits[0])));
                        }
                    }
                    break;
                case 4:
                    if (8 * sizeof(long) > 3 * PyLong_SHIFT) {
                        if (8 * sizeof(unsigned long) > 4 * PyLong_SHIFT) {
                            __PYX_VERIFY_RETURN_INT(long, unsigned long, (((((((((unsigned long)digits[3]) << PyLong_SHIFT) | (unsigned long)digits[2]) << PyLong_SHIFT) | (unsigned long)digits[1]) << PyLong_SHIFT) | (unsigned long)digits[0])))
                        } else if (8 * sizeof(long) - 1 > 4 * PyLong_SHIFT) {
                            return (long) ((((((((((long)digits[3]) << PyLong_SHIFT) | (long)digits[2]) << PyLong_SHIFT) | (long)digits[1]) << PyLong_SHIFT) | (long)digits[0])));
                        }
                    }
                    break;
            }
#endif
            if (sizeof(long) <= sizeof(long)) {
                __PYX_VERIFY_RETURN_INT_EXC(long, long, PyLong_AsLong(x))
#ifdef HAVE_LONG_LONG
            } else if (sizeof(long) <= sizeof(PY_LONG_LONG)) {
                __PYX_VERIFY_RETURN_INT_EXC(long, PY_LONG_LONG, PyLong_AsLongLong(x))
#endif
            }
        }
        {
#if CYTHON_COMPILING_IN_PYPY && !defined(_PyLong_AsByteArray)
            PyErr_SetString(PyExc_RuntimeError,
                            "_PyLong_AsByteArray() not available in PyPy, cannot convert large numbers");
#else
            long val;
            PyObject *v = __Pyx_PyNumber_IntOrLong(x);
 #if PY_MAJOR_VERSION < 3
            if (likely(v) && !PyLong_Check(v)) {
                PyObject *tmp = v;
                v = PyNumber_Long(tmp);
                Py_DECREF(tmp);
            }
 #endif
            if (likely(v)) {
                int one = 1; int is_little = (int)*(unsigned char *)&one;
                unsigned char *bytes = (unsigned char *)&val;
                int ret = _PyLong_AsByteArray((PyLongObject *)v,
                                              bytes, sizeof(val),
                                              is_little, !is_unsigned);
                Py_DECREF(v);
                if (likely(!ret))
                    return val;
            }
#endif
            return (long) -1;
        }
    } else {
        long val;
        PyObject *tmp = __Pyx_PyNumber_IntOrLong(x);
        if (!tmp) return (long) -1;
        val = __Pyx_PyInt_As_long(tmp);
        Py_DECREF(tmp);
        return val;
    }
raise_overflow:
    PyErr_SetString(PyExc_OverflowError,
        "value too large to convert to long");
    return (long) -1;
raise_neg_overflow:
    PyErr_SetString(PyExc_OverflowError,
        "can't convert negative value to long");
    return (long) -1;
}

/* CIntFromPy */
                static CYTHON_INLINE grpc_call_error __Pyx_PyInt_As_grpc_call_error(PyObject *x) {
    const grpc_call_error neg_one = (grpc_call_error) -1, const_zero = (grpc_call_error) 0;
    const int is_unsigned = neg_one > const_zero;
#if PY_MAJOR_VERSION < 3
    if (likely(PyInt_Check(x))) {
        if (sizeof(grpc_call_error) < sizeof(long)) {
            __PYX_VERIFY_RETURN_INT(grpc_call_error, long, PyInt_AS_LONG(x))
        } else {
            long val = PyInt_AS_LONG(x);
            if (is_unsigned && unlikely(val < 0)) {
                goto raise_neg_overflow;
            }
            return (grpc_call_error) val;
        }
    } else
#endif
    if (likely(PyLong_Check(x))) {
        if (is_unsigned) {
#if CYTHON_USE_PYLONG_INTERNALS
            const digit* digits = ((PyLongObject*)x)->ob_digit;
            switch (Py_SIZE(x)) {
                case  0: return (grpc_call_error) 0;
                case  1: __PYX_VERIFY_RETURN_INT(grpc_call_error, digit, digits[0])
                case 2:
                    if (8 * sizeof(grpc_call_error) > 1 * PyLong_SHIFT) {
                        if (8 * sizeof(unsigned long) > 2 * PyLong_SHIFT) {
                            __PYX_VERIFY_RETURN_INT(grpc_call_error, unsigned long, (((((unsigned long)digits[1]) << PyLong_SHIFT) | (unsigned long)digits[0])))
                        } else if (8 * sizeof(grpc_call_error) >= 2 * PyLong_SHIFT) {
                            return (grpc_call_error) (((((grpc_call_error)digits[1]) << PyLong_SHIFT) | (grpc_call_error)digits[0]));
                        }
                    }
                    break;
                case 3:
                    if (8 * sizeof(grpc_call_error) > 2 * PyLong_SHIFT) {
                        if (8 * sizeof(unsigned long) > 3 * PyLong_SHIFT) {
                            __PYX_VERIFY_RETURN_INT(grpc_call_error, unsigned long, (((((((unsigned long)digits[2]) << PyLong_SHIFT) | (unsigned long)digits[1]) << PyLong_SHIFT) | (unsigned long)digits[0])))
                        } else if (8 * sizeof(grpc_call_error) >= 3 * PyLong_SHIFT) {
                            return (grpc_call_error) (((((((grpc_call_error)digits[2]) << PyLong_SHIFT) | (grpc_call_error)digits[1]) << PyLong_SHIFT) | (grpc_call_error)digits[0]));
                        }
                    }
                    break;
                case 4:
                    if (8 * sizeof(grpc_call_error) > 3 * PyLong_SHIFT) {
                        if (8 * sizeof(unsigned long) > 4 * PyLong_SHIFT) {
                            __PYX_VERIFY_RETURN_INT(grpc_call_error, unsigned long, (((((((((unsigned long)digits[3]) << PyLong_SHIFT) | (unsigned long)digits[2]) << PyLong_SHIFT) | (unsigned long)digits[1]) << PyLong_SHIFT) | (unsigned long)digits[0])))
                        } else if (8 * sizeof(grpc_call_error) >= 4 * PyLong_SHIFT) {
                            return (grpc_call_error) (((((((((grpc_call_error)digits[3]) << PyLong_SHIFT) | (grpc_call_error)digits[2]) << PyLong_SHIFT) | (grpc_call_error)digits[1]) << PyLong_SHIFT) | (grpc_call_error)digits[0]));
                        }
                    }
                    break;
            }
#endif
#if CYTHON_COMPILING_IN_CPYTHON
            if (unlikely(Py_SIZE(x) < 0)) {
                goto raise_neg_overflow;
            }
#else
            {
                int result = PyObject_RichCompareBool(x, Py_False, Py_LT);
                if (unlikely(result < 0))
                    return (grpc_call_error) -1;
                if (unlikely(result == 1))
                    goto raise_neg_overflow;
            }
#endif
            if (sizeof(grpc_call_error) <= sizeof(unsigned long)) {
                __PYX_VERIFY_RETURN_INT_EXC(grpc_call_error, unsigned long, PyLong_AsUnsignedLong(x))
#ifdef HAVE_LONG_LONG
            } else if (sizeof(grpc_call_error) <= sizeof(unsigned PY_LONG_LONG)) {
                __PYX_VERIFY_RETURN_INT_EXC(grpc_call_error, unsigned PY_LONG_LONG, PyLong_AsUnsignedLongLong(x))
#endif
            }
        } else {
#if CYTHON_USE_PYLONG_INTERNALS
            const digit* digits = ((PyLongObject*)x)->ob_digit;
            switch (Py_SIZE(x)) {
                case  0: return (grpc_call_error) 0;
                case -1: __PYX_VERIFY_RETURN_INT(grpc_call_error, sdigit, (sdigit) (-(sdigit)digits[0]))
                case  1: __PYX_VERIFY_RETURN_INT(grpc_call_error,  digit, +digits[0])
                case -2:
                    if (8 * sizeof(grpc_call_error) - 1 > 1 * PyLong_SHIFT) {
                        if (8 * sizeof(unsigned long) > 2 * PyLong_SHIFT) {
                            __PYX_VERIFY_RETURN_INT(grpc_call_error, long, -(long) (((((unsigned long)digits[1]) << PyLong_SHIFT) | (unsigned long)digits[0])))
                        } else if (8 * sizeof(grpc_call_error) - 1 > 2 * PyLong_SHIFT) {
                            return (grpc_call_error) (((grpc_call_error)-1)*(((((grpc_call_error)digits[1]) << PyLong_SHIFT) | (grpc_call_error)digits[0])));
                        }
                    }
                    break;
                case 2:
                    if (8 * sizeof(grpc_call_error) > 1 * PyLong_SHIFT) {
                        if (8 * sizeof(unsigned long) > 2 * PyLong_SHIFT) {
                            __PYX_VERIFY_RETURN_INT(grpc_call_error, unsigned long, (((((unsigned long)digits[1]) << PyLong_SHIFT) | (unsigned long)digits[0])))
                        } else if (8 * sizeof(grpc_call_error) - 1 > 2 * PyLong_SHIFT) {
                            return (grpc_call_error) ((((((grpc_call_error)digits[1]) << PyLong_SHIFT) | (grpc_call_error)digits[0])));
                        }
                    }
                    break;
                case -3:
                    if (8 * sizeof(grpc_call_error) - 1 > 2 * PyLong_SHIFT) {
                        if (8 * sizeof(unsigned long) > 3 * PyLong_SHIFT) {
                            __PYX_VERIFY_RETURN_INT(grpc_call_error, long, -(long) (((((((unsigned long)digits[2]) << PyLong_SHIFT) | (unsigned long)digits[1]) << PyLong_SHIFT) | (unsigned long)digits[0])))
                        } else if (8 * sizeof(grpc_call_error) - 1 > 3 * PyLong_SHIFT) {
                            return (grpc_call_error) (((grpc_call_error)-1)*(((((((grpc_call_error)digits[2]) << PyLong_SHIFT) | (grpc_call_error)digits[1]) << PyLong_SHIFT) | (grpc_call_error)digits[0])));
                        }
                    }
                    break;
                case 3:
                    if (8 * sizeof(grpc_call_error) > 2 * PyLong_SHIFT) {
                        if (8 * sizeof(unsigned long) > 3 * PyLong_SHIFT) {
                            __PYX_VERIFY_RETURN_INT(grpc_call_error, unsigned long, (((((((unsigned long)digits[2]) << PyLong_SHIFT) | (unsigned long)digits[1]) << PyLong_SHIFT) | (unsigned long)digits[0])))
                        } else if (8 * sizeof(grpc_call_error) - 1 > 3 * PyLong_SHIFT) {
                            return (grpc_call_error) ((((((((grpc_call_error)digits[2]) << PyLong_SHIFT) | (grpc_call_error)digits[1]) << PyLong_SHIFT) | (grpc_call_error)digits[0])));
                        }
                    }
                    break;
                case -4:
                    if (8 * sizeof(grpc_call_error) - 1 > 3 * PyLong_SHIFT) {
                        if (8 * sizeof(unsigned long) > 4 * PyLong_SHIFT) {
                            __PYX_VERIFY_RETURN_INT(grpc_call_error, long, -(long) (((((((((unsigned long)digits[3]) << PyLong_SHIFT) | (unsigned long)digits[2]) << PyLong_SHIFT) | (unsigned long)digits[1]) << PyLong_SHIFT) | (unsigned long)digits[0])))
                        } else if (8 * sizeof(grpc_call_error) - 1 > 4 * PyLong_SHIFT) {
                            return (grpc_call_error) (((grpc_call_error)-1)*(((((((((grpc_call_error)digits[3]) << PyLong_SHIFT) | (grpc_call_error)digits[2]) << PyLong_SHIFT) | (grpc_call_error)digits[1]) << PyLong_SHIFT) | (grpc_call_error)digits[0])));
                        }
                    }
                    break;
                case 4:
                    if (8 * sizeof(grpc_call_error) > 3 * PyLong_SHIFT) {
                        if (8 * sizeof(unsigned long) > 4 * PyLong_SHIFT) {
                            __PYX_VERIFY_RETURN_INT(grpc_call_error, unsigned long, (((((((((unsigned long)digits[3]) << PyLong_SHIFT) | (unsigned long)digits[2]) << PyLong_SHIFT) | (unsigned long)digits[1]) << PyLong_SHIFT) | (unsigned long)digits[0])))
                        } else if (8 * sizeof(grpc_call_error) - 1 > 4 * PyLong_SHIFT) {
                            return (grpc_call_error) ((((((((((grpc_call_error)digits[3]) << PyLong_SHIFT) | (grpc_call_error)digits[2]) << PyLong_SHIFT) | (grpc_call_error)digits[1]) << PyLong_SHIFT) | (grpc_call_error)digits[0])));
                        }
                    }
                    break;
            }
#endif
            if (sizeof(grpc_call_error) <= sizeof(long)) {
                __PYX_VERIFY_RETURN_INT_EXC(grpc_call_error, long, PyLong_AsLong(x))
#ifdef HAVE_LONG_LONG
            } else if (sizeof(grpc_call_error) <= sizeof(PY_LONG_LONG)) {
                __PYX_VERIFY_RETURN_INT_EXC(grpc_call_error, PY_LONG_LONG, PyLong_AsLongLong(x))
#endif
            }
        }
        {
#if CYTHON_COMPILING_IN_PYPY && !defined(_PyLong_AsByteArray)
            PyErr_SetString(PyExc_RuntimeError,
                            "_PyLong_AsByteArray() not available in PyPy, cannot convert large numbers");
#else
            grpc_call_error val;
            PyObject *v = __Pyx_PyNumber_IntOrLong(x);
 #if PY_MAJOR_VERSION < 3
            if (likely(v) && !PyLong_Check(v)) {
                PyObject *tmp = v;
                v = PyNumber_Long(tmp);
                Py_DECREF(tmp);
            }
 #endif
            if (likely(v)) {
                int one = 1; int is_little = (int)*(unsigned char *)&one;
                unsigned char *bytes = (unsigned char *)&val;
                int ret = _PyLong_AsByteArray((PyLongObject *)v,
                                              bytes, sizeof(val),
                                              is_little, !is_unsigned);
                Py_DECREF(v);
                if (likely(!ret))
                    return val;
            }
#endif
            return (grpc_call_error) -1;
        }
    } else {
        grpc_call_error val;
        PyObject *tmp = __Pyx_PyNumber_IntOrLong(x);
        if (!tmp) return (grpc_call_error) -1;
        val = __Pyx_PyInt_As_grpc_call_error(tmp);
        Py_DECREF(tmp);
        return val;
    }
raise_overflow:
    PyErr_SetString(PyExc_OverflowError,
        "value too large to convert to grpc_call_error");
    return (grpc_call_error) -1;
raise_neg_overflow:
    PyErr_SetString(PyExc_OverflowError,
        "can't convert negative value to grpc_call_error");
    return (grpc_call_error) -1;
}

/* CIntFromPy */
                static CYTHON_INLINE size_t __Pyx_PyInt_As_size_t(PyObject *x) {
    const size_t neg_one = (size_t) -1, const_zero = (size_t) 0;
    const int is_unsigned = neg_one > const_zero;
#if PY_MAJOR_VERSION < 3
    if (likely(PyInt_Check(x))) {
        if (sizeof(size_t) < sizeof(long)) {
            __PYX_VERIFY_RETURN_INT(size_t, long, PyInt_AS_LONG(x))
        } else {
            long val = PyInt_AS_LONG(x);
            if (is_unsigned && unlikely(val < 0)) {
                goto raise_neg_overflow;
            }
            return (size_t) val;
        }
    } else
#endif
    if (likely(PyLong_Check(x))) {
        if (is_unsigned) {
#if CYTHON_USE_PYLONG_INTERNALS
            const digit* digits = ((PyLongObject*)x)->ob_digit;
            switch (Py_SIZE(x)) {
                case  0: return (size_t) 0;
                case  1: __PYX_VERIFY_RETURN_INT(size_t, digit, digits[0])
                case 2:
                    if (8 * sizeof(size_t) > 1 * PyLong_SHIFT) {
                        if (8 * sizeof(unsigned long) > 2 * PyLong_SHIFT) {
                            __PYX_VERIFY_RETURN_INT(size_t, unsigned long, (((((unsigned long)digits[1]) << PyLong_SHIFT) | (unsigned long)digits[0])))
                        } else if (8 * sizeof(size_t) >= 2 * PyLong_SHIFT) {
                            return (size_t) (((((size_t)digits[1]) << PyLong_SHIFT) | (size_t)digits[0]));
                        }
                    }
                    break;
                case 3:
                    if (8 * sizeof(size_t) > 2 * PyLong_SHIFT) {
                        if (8 * sizeof(unsigned long) > 3 * PyLong_SHIFT) {
                            __PYX_VERIFY_RETURN_INT(size_t, unsigned long, (((((((unsigned long)digits[2]) << PyLong_SHIFT) | (unsigned long)digits[1]) << PyLong_SHIFT) | (unsigned long)digits[0])))
                        } else if (8 * sizeof(size_t) >= 3 * PyLong_SHIFT) {
                            return (size_t) (((((((size_t)digits[2]) << PyLong_SHIFT) | (size_t)digits[1]) << PyLong_SHIFT) | (size_t)digits[0]));
                        }
                    }
                    break;
                case 4:
                    if (8 * sizeof(size_t) > 3 * PyLong_SHIFT) {
                        if (8 * sizeof(unsigned long) > 4 * PyLong_SHIFT) {
                            __PYX_VERIFY_RETURN_INT(size_t, unsigned long, (((((((((unsigned long)digits[3]) << PyLong_SHIFT) | (unsigned long)digits[2]) << PyLong_SHIFT) | (unsigned long)digits[1]) << PyLong_SHIFT) | (unsigned long)digits[0])))
                        } else if (8 * sizeof(size_t) >= 4 * PyLong_SHIFT) {
                            return (size_t) (((((((((size_t)digits[3]) << PyLong_SHIFT) | (size_t)digits[2]) << PyLong_SHIFT) | (size_t)digits[1]) << PyLong_SHIFT) | (size_t)digits[0]));
                        }
                    }
                    break;
            }
#endif
#if CYTHON_COMPILING_IN_CPYTHON
            if (unlikely(Py_SIZE(x) < 0)) {
                goto raise_neg_overflow;
            }
#else
            {
                int result = PyObject_RichCompareBool(x, Py_False, Py_LT);
                if (unlikely(result < 0))
                    return (size_t) -1;
                if (unlikely(result == 1))
                    goto raise_neg_overflow;
            }
#endif
            if (sizeof(size_t) <= sizeof(unsigned long)) {
                __PYX_VERIFY_RETURN_INT_EXC(size_t, unsigned long, PyLong_AsUnsignedLong(x))
#ifdef HAVE_LONG_LONG
            } else if (sizeof(size_t) <= sizeof(unsigned PY_LONG_LONG)) {
                __PYX_VERIFY_RETURN_INT_EXC(size_t, unsigned PY_LONG_LONG, PyLong_AsUnsignedLongLong(x))
#endif
            }
        } else {
#if CYTHON_USE_PYLONG_INTERNALS
            const digit* digits = ((PyLongObject*)x)->ob_digit;
            switch (Py_SIZE(x)) {
                case  0: return (size_t) 0;
                case -1: __PYX_VERIFY_RETURN_INT(size_t, sdigit, (sdigit) (-(sdigit)digits[0]))
                case  1: __PYX_VERIFY_RETURN_INT(size_t,  digit, +digits[0])
                case -2:
                    if (8 * sizeof(size_t) - 1 > 1 * PyLong_SHIFT) {
                        if (8 * sizeof(unsigned long) > 2 * PyLong_SHIFT) {
                            __PYX_VERIFY_RETURN_INT(size_t, long, -(long) (((((unsigned long)digits[1]) << PyLong_SHIFT) | (unsigned long)digits[0])))
                        } else if (8 * sizeof(size_t) - 1 > 2 * PyLong_SHIFT) {
                            return (size_t) (((size_t)-1)*(((((size_t)digits[1]) << PyLong_SHIFT) | (size_t)digits[0])));
                        }
                    }
                    break;
                case 2:
                    if (8 * sizeof(size_t) > 1 * PyLong_SHIFT) {
                        if (8 * sizeof(unsigned long) > 2 * PyLong_SHIFT) {
                            __PYX_VERIFY_RETURN_INT(size_t, unsigned long, (((((unsigned long)digits[1]) << PyLong_SHIFT) | (unsigned long)digits[0])))
                        } else if (8 * sizeof(size_t) - 1 > 2 * PyLong_SHIFT) {
                            return (size_t) ((((((size_t)digits[1]) << PyLong_SHIFT) | (size_t)digits[0])));
                        }
                    }
                    break;
                case -3:
                    if (8 * sizeof(size_t) - 1 > 2 * PyLong_SHIFT) {
                        if (8 * sizeof(unsigned long) > 3 * PyLong_SHIFT) {
                            __PYX_VERIFY_RETURN_INT(size_t, long, -(long) (((((((unsigned long)digits[2]) << PyLong_SHIFT) | (unsigned long)digits[1]) << PyLong_SHIFT) | (unsigned long)digits[0])))
                        } else if (8 * sizeof(size_t) - 1 > 3 * PyLong_SHIFT) {
                            return (size_t) (((size_t)-1)*(((((((size_t)digits[2]) << PyLong_SHIFT) | (size_t)digits[1]) << PyLong_SHIFT) | (size_t)digits[0])));
                        }
                    }
                    break;
                case 3:
                    if (8 * sizeof(size_t) > 2 * PyLong_SHIFT) {
                        if (8 * sizeof(unsigned long) > 3 * PyLong_SHIFT) {
                            __PYX_VERIFY_RETURN_INT(size_t, unsigned long, (((((((unsigned long)digits[2]) << PyLong_SHIFT) | (unsigned long)digits[1]) << PyLong_SHIFT) | (unsigned long)digits[0])))
                        } else if (8 * sizeof(size_t) - 1 > 3 * PyLong_SHIFT) {
                            return (size_t) ((((((((size_t)digits[2]) << PyLong_SHIFT) | (size_t)digits[1]) << PyLong_SHIFT) | (size_t)digits[0])));
                        }
                    }
                    break;
                case -4:
                    if (8 * sizeof(size_t) - 1 > 3 * PyLong_SHIFT) {
                        if (8 * sizeof(unsigned long) > 4 * PyLong_SHIFT) {
                            __PYX_VERIFY_RETURN_INT(size_t, long, -(long) (((((((((unsigned long)digits[3]) << PyLong_SHIFT) | (unsigned long)digits[2]) << PyLong_SHIFT) | (unsigned long)digits[1]) << PyLong_SHIFT) | (unsigned long)digits[0])))
                        } else if (8 * sizeof(size_t) - 1 > 4 * PyLong_SHIFT) {
                            return (size_t) (((size_t)-1)*(((((((((size_t)digits[3]) << PyLong_SHIFT) | (size_t)digits[2]) << PyLong_SHIFT) | (size_t)digits[1]) << PyLong_SHIFT) | (size_t)digits[0])));
                        }
                    }
                    break;
                case 4:
                    if (8 * sizeof(size_t) > 3 * PyLong_SHIFT) {
                        if (8 * sizeof(unsigned long) > 4 * PyLong_SHIFT) {
                            __PYX_VERIFY_RETURN_INT(size_t, unsigned long, (((((((((unsigned long)digits[3]) << PyLong_SHIFT) | (unsigned long)digits[2]) << PyLong_SHIFT) | (unsigned long)digits[1]) << PyLong_SHIFT) | (unsigned long)digits[0])))
                        } else if (8 * sizeof(size_t) - 1 > 4 * PyLong_SHIFT) {
                            return (size_t) ((((((((((size_t)digits[3]) << PyLong_SHIFT) | (size_t)digits[2]) << PyLong_SHIFT) | (size_t)digits[1]) << PyLong_SHIFT) | (size_t)digits[0])));
                        }
                    }
                    break;
            }
#endif
            if (sizeof(size_t) <= sizeof(long)) {
                __PYX_VERIFY_RETURN_INT_EXC(size_t, long, PyLong_AsLong(x))
#ifdef HAVE_LONG_LONG
            } else if (sizeof(size_t) <= sizeof(PY_LONG_LONG)) {
                __PYX_VERIFY_RETURN_INT_EXC(size_t, PY_LONG_LONG, PyLong_AsLongLong(x))
#endif
            }
        }
        {
#if CYTHON_COMPILING_IN_PYPY && !defined(_PyLong_AsByteArray)
            PyErr_SetString(PyExc_RuntimeError,
                            "_PyLong_AsByteArray() not available in PyPy, cannot convert large numbers");
#else
            size_t val;
            PyObject *v = __Pyx_PyNumber_IntOrLong(x);
 #if PY_MAJOR_VERSION < 3
            if (likely(v) && !PyLong_Check(v)) {
                PyObject *tmp = v;
                v = PyNumber_Long(tmp);
                Py_DECREF(tmp);
            }
 #endif
            if (likely(v)) {
                int one = 1; int is_little = (int)*(unsigned char *)&one;
                unsigned char *bytes = (unsigned char *)&val;
                int ret = _PyLong_AsByteArray((PyLongObject *)v,
                                              bytes, sizeof(val),
                                              is_little, !is_unsigned);
                Py_DECREF(v);
                if (likely(!ret))
                    return val;
            }
#endif
            return (size_t) -1;
        }
    } else {
        size_t val;
        PyObject *tmp = __Pyx_PyNumber_IntOrLong(x);
        if (!tmp) return (size_t) -1;
        val = __Pyx_PyInt_As_size_t(tmp);
        Py_DECREF(tmp);
        return val;
    }
raise_overflow:
    PyErr_SetString(PyExc_OverflowError,
        "value too large to convert to size_t");
    return (size_t) -1;
raise_neg_overflow:
    PyErr_SetString(PyExc_OverflowError,
        "can't convert negative value to size_t");
    return (size_t) -1;
}

/* CoroutineBase */
                #include <structmember.h>
#include <frameobject.h>
#define __Pyx_Coroutine_Undelegate(gen) Py_CLEAR((gen)->yieldfrom)
static int __Pyx_PyGen__FetchStopIterationValue(CYTHON_UNUSED PyThreadState *__pyx_tstate, PyObject **pvalue) {
    PyObject *et, *ev, *tb;
    PyObject *value = NULL;
    __Pyx_ErrFetch(&et, &ev, &tb);
    if (!et) {
        Py_XDECREF(tb);
        Py_XDECREF(ev);
        Py_INCREF(Py_None);
        *pvalue = Py_None;
        return 0;
    }
    if (likely(et == PyExc_StopIteration)) {
        if (!ev) {
            Py_INCREF(Py_None);
            value = Py_None;
        }
#if PY_VERSION_HEX >= 0x030300A0
        else if (Py_TYPE(ev) == (PyTypeObject*)PyExc_StopIteration) {
            value = ((PyStopIterationObject *)ev)->value;
            Py_INCREF(value);
            Py_DECREF(ev);
        }
#endif
        else if (unlikely(PyTuple_Check(ev))) {
            if (PyTuple_GET_SIZE(ev) >= 1) {
#if CYTHON_ASSUME_SAFE_MACROS && !CYTHON_AVOID_BORROWED_REFS
                value = PyTuple_GET_ITEM(ev, 0);
                Py_INCREF(value);
#else
                value = PySequence_ITEM(ev, 0);
#endif
            } else {
                Py_INCREF(Py_None);
                value = Py_None;
            }
            Py_DECREF(ev);
        }
        else if (!__Pyx_TypeCheck(ev, (PyTypeObject*)PyExc_StopIteration)) {
            value = ev;
        }
        if (likely(value)) {
            Py_XDECREF(tb);
            Py_DECREF(et);
            *pvalue = value;
            return 0;
        }
    } else if (!__Pyx_PyErr_GivenExceptionMatches(et, PyExc_StopIteration)) {
        __Pyx_ErrRestore(et, ev, tb);
        return -1;
    }
    PyErr_NormalizeException(&et, &ev, &tb);
    if (unlikely(!PyObject_TypeCheck(ev, (PyTypeObject*)PyExc_StopIteration))) {
        __Pyx_ErrRestore(et, ev, tb);
        return -1;
    }
    Py_XDECREF(tb);
    Py_DECREF(et);
#if PY_VERSION_HEX >= 0x030300A0
    value = ((PyStopIterationObject *)ev)->value;
    Py_INCREF(value);
    Py_DECREF(ev);
#else
    {
        PyObject* args = __Pyx_PyObject_GetAttrStr(ev, __pyx_n_s_args);
        Py_DECREF(ev);
        if (likely(args)) {
            value = PySequence_GetItem(args, 0);
            Py_DECREF(args);
        }
        if (unlikely(!value)) {
            __Pyx_ErrRestore(NULL, NULL, NULL);
            Py_INCREF(Py_None);
            value = Py_None;
        }
    }
#endif
    *pvalue = value;
    return 0;
}
static CYTHON_INLINE
void __Pyx_Coroutine_ExceptionClear(__pyx_CoroutineObject *self) {
    PyObject *exc_type = self->exc_type;
    PyObject *exc_value = self->exc_value;
    PyObject *exc_traceback = self->exc_traceback;
    self->exc_type = NULL;
    self->exc_value = NULL;
    self->exc_traceback = NULL;
    Py_XDECREF(exc_type);
    Py_XDECREF(exc_value);
    Py_XDECREF(exc_traceback);
}
#define __Pyx_Coroutine_AlreadyRunningError(gen)  (__Pyx__Coroutine_AlreadyRunningError(gen), (PyObject*)NULL)
static void __Pyx__Coroutine_AlreadyRunningError(CYTHON_UNUSED __pyx_CoroutineObject *gen) {
    const char *msg;
    if (0) {
    #ifdef __Pyx_Coroutine_USED
    } else if (__Pyx_Coroutine_Check((PyObject*)gen)) {
        msg = "coroutine already executing";
    #endif
    #ifdef __Pyx_AsyncGen_USED
    } else if (__Pyx_AsyncGen_CheckExact((PyObject*)gen)) {
        msg = "async generator already executing";
    #endif
    } else {
        msg = "generator already executing";
    }
    PyErr_SetString(PyExc_ValueError, msg);
}
#define __Pyx_Coroutine_NotStartedError(gen)  (__Pyx__Coroutine_NotStartedError(gen), (PyObject*)NULL)
static void __Pyx__Coroutine_NotStartedError(CYTHON_UNUSED PyObject *gen) {
    const char *msg;
    if (0) {
    #ifdef __Pyx_Coroutine_USED
    } else if (__Pyx_Coroutine_Check(gen)) {
        msg = "can't send non-None value to a just-started coroutine";
    #endif
    #ifdef __Pyx_AsyncGen_USED
    } else if (__Pyx_AsyncGen_CheckExact(gen)) {
        msg = "can't send non-None value to a just-started async generator";
    #endif
    } else {
        msg = "can't send non-None value to a just-started generator";
    }
    PyErr_SetString(PyExc_TypeError, msg);
}
#define __Pyx_Coroutine_AlreadyTerminatedError(gen, value, closing)  (__Pyx__Coroutine_AlreadyTerminatedError(gen, value, closing), (PyObject*)NULL)
static void __Pyx__Coroutine_AlreadyTerminatedError(CYTHON_UNUSED PyObject *gen, PyObject *value, CYTHON_UNUSED int closing) {
    #ifdef __Pyx_Coroutine_USED
    if (!closing && __Pyx_Coroutine_Check(gen)) {
        PyErr_SetString(PyExc_RuntimeError, "cannot reuse already awaited coroutine");
    } else
    #endif
    if (value) {
        #ifdef __Pyx_AsyncGen_USED
        if (__Pyx_AsyncGen_CheckExact(gen))
            PyErr_SetNone(__Pyx_PyExc_StopAsyncIteration);
        else
        #endif
        PyErr_SetNone(PyExc_StopIteration);
    }
}
static
PyObject *__Pyx_Coroutine_SendEx(__pyx_CoroutineObject *self, PyObject *value, int closing) {
    __Pyx_PyThreadState_declare
    PyThreadState *tstate;
    PyObject *retval;
    assert(!self->is_running);
    if (unlikely(self->resume_label == 0)) {
        if (unlikely(value && value != Py_None)) {
            return __Pyx_Coroutine_NotStartedError((PyObject*)self);
        }
    }
    if (unlikely(self->resume_label == -1)) {
        return __Pyx_Coroutine_AlreadyTerminatedError((PyObject*)self, value, closing);
    }
#if CYTHON_FAST_THREAD_STATE
    __Pyx_PyThreadState_assign
    tstate = __pyx_tstate;
#else
    tstate = __Pyx_PyThreadState_Current;
#endif
    if (self->exc_type) {
#if CYTHON_COMPILING_IN_PYPY || CYTHON_COMPILING_IN_PYSTON
#else
        if (self->exc_traceback) {
            PyTracebackObject *tb = (PyTracebackObject *) self->exc_traceback;
            PyFrameObject *f = tb->tb_frame;
            Py_XINCREF(tstate->frame);
            assert(f->f_back == NULL);
            f->f_back = tstate->frame;
        }
#endif
        __Pyx_ExceptionSwap(&self->exc_type, &self->exc_value,
                            &self->exc_traceback);
    } else {
        __Pyx_Coroutine_ExceptionClear(self);
        __Pyx_ExceptionSave(&self->exc_type, &self->exc_value, &self->exc_traceback);
    }
    self->is_running = 1;
    retval = self->body((PyObject *) self, tstate, value);
    self->is_running = 0;
    return retval;
}
static CYTHON_INLINE void __Pyx_Coroutine_ResetFrameBackpointer(__pyx_CoroutineObject *self) {
    if (likely(self->exc_traceback)) {
#if CYTHON_COMPILING_IN_PYPY || CYTHON_COMPILING_IN_PYSTON
#else
        PyTracebackObject *tb = (PyTracebackObject *) self->exc_traceback;
        PyFrameObject *f = tb->tb_frame;
        Py_CLEAR(f->f_back);
#endif
    }
}
static CYTHON_INLINE
PyObject *__Pyx_Coroutine_MethodReturn(CYTHON_UNUSED PyObject* gen, PyObject *retval) {
    if (unlikely(!retval)) {
        __Pyx_PyThreadState_declare
        __Pyx_PyThreadState_assign
        if (!__Pyx_PyErr_Occurred()) {
            PyObject *exc = PyExc_StopIteration;
            #ifdef __Pyx_AsyncGen_USED
            if (__Pyx_AsyncGen_CheckExact(gen))
                exc = __Pyx_PyExc_StopAsyncIteration;
            #endif
            __Pyx_PyErr_SetNone(exc);
        }
    }
    return retval;
}
static CYTHON_INLINE
PyObject *__Pyx_Coroutine_FinishDelegation(__pyx_CoroutineObject *gen) {
    PyObject *ret;
    PyObject *val = NULL;
    __Pyx_Coroutine_Undelegate(gen);
    __Pyx_PyGen__FetchStopIterationValue(__Pyx_PyThreadState_Current, &val);
    ret = __Pyx_Coroutine_SendEx(gen, val, 0);
    Py_XDECREF(val);
    return ret;
}
static PyObject *__Pyx_Coroutine_Send(PyObject *self, PyObject *value) {
    PyObject *retval;
    __pyx_CoroutineObject *gen = (__pyx_CoroutineObject*) self;
    PyObject *yf = gen->yieldfrom;
    if (unlikely(gen->is_running))
        return __Pyx_Coroutine_AlreadyRunningError(gen);
    if (yf) {
        PyObject *ret;
        gen->is_running = 1;
        #ifdef __Pyx_Generator_USED
        if (__Pyx_Generator_CheckExact(yf)) {
            ret = __Pyx_Coroutine_Send(yf, value);
        } else
        #endif
        #ifdef __Pyx_Coroutine_USED
        if (__Pyx_Coroutine_Check(yf)) {
            ret = __Pyx_Coroutine_Send(yf, value);
        } else
        #endif
        #ifdef __Pyx_AsyncGen_USED
        if (__pyx_PyAsyncGenASend_CheckExact(yf)) {
            ret = __Pyx_async_gen_asend_send(yf, value);
        } else
        #endif
        #if CYTHON_COMPILING_IN_CPYTHON && PY_VERSION_HEX >= 0x03030000 && (defined(__linux__) || PY_VERSION_HEX >= 0x030600B3)
        if (PyGen_CheckExact(yf)) {
            ret = _PyGen_Send((PyGenObject*)yf, value == Py_None ? NULL : value);
        } else
        #endif
        #if CYTHON_COMPILING_IN_CPYTHON && PY_VERSION_HEX >= 0x03050000 && defined(PyCoro_CheckExact) && (defined(__linux__) || PY_VERSION_HEX >= 0x030600B3)
        if (PyCoro_CheckExact(yf)) {
            ret = _PyGen_Send((PyGenObject*)yf, value == Py_None ? NULL : value);
        } else
        #endif
        {
            if (value == Py_None)
                ret = Py_TYPE(yf)->tp_iternext(yf);
            else
                ret = __Pyx_PyObject_CallMethod1(yf, __pyx_n_s_send, value);
        }
        gen->is_running = 0;
        if (likely(ret)) {
            return ret;
        }
        retval = __Pyx_Coroutine_FinishDelegation(gen);
    } else {
        retval = __Pyx_Coroutine_SendEx(gen, value, 0);
    }
    return __Pyx_Coroutine_MethodReturn(self, retval);
}
static int __Pyx_Coroutine_CloseIter(__pyx_CoroutineObject *gen, PyObject *yf) {
    PyObject *retval = NULL;
    int err = 0;
    #ifdef __Pyx_Generator_USED
    if (__Pyx_Generator_CheckExact(yf)) {
        retval = __Pyx_Coroutine_Close(yf);
        if (!retval)
            return -1;
    } else
    #endif
    #ifdef __Pyx_Coroutine_USED
    if (__Pyx_Coroutine_Check(yf)) {
        retval = __Pyx_Coroutine_Close(yf);
        if (!retval)
            return -1;
    } else
    if (__Pyx_CoroutineAwait_CheckExact(yf)) {
        retval = __Pyx_CoroutineAwait_Close((__pyx_CoroutineAwaitObject*)yf);
        if (!retval)
            return -1;
    } else
    #endif
    #ifdef __Pyx_AsyncGen_USED
    if (__pyx_PyAsyncGenASend_CheckExact(yf)) {
        retval = __Pyx_async_gen_asend_close(yf, NULL);
    } else
    if (__pyx_PyAsyncGenAThrow_CheckExact(yf)) {
        retval = __Pyx_async_gen_athrow_close(yf, NULL);
    } else
    #endif
    {
        PyObject *meth;
        gen->is_running = 1;
        meth = __Pyx_PyObject_GetAttrStr(yf, __pyx_n_s_close);
        if (unlikely(!meth)) {
            if (!PyErr_ExceptionMatches(PyExc_AttributeError)) {
                PyErr_WriteUnraisable(yf);
            }
            PyErr_Clear();
        } else {
            retval = PyObject_CallFunction(meth, NULL);
            Py_DECREF(meth);
            if (!retval)
                err = -1;
        }
        gen->is_running = 0;
    }
    Py_XDECREF(retval);
    return err;
}
static PyObject *__Pyx_Generator_Next(PyObject *self) {
    __pyx_CoroutineObject *gen = (__pyx_CoroutineObject*) self;
    PyObject *yf = gen->yieldfrom;
    if (unlikely(gen->is_running))
        return __Pyx_Coroutine_AlreadyRunningError(gen);
    if (yf) {
        PyObject *ret;
        gen->is_running = 1;
        #ifdef __Pyx_Generator_USED
        if (__Pyx_Generator_CheckExact(yf)) {
            ret = __Pyx_Generator_Next(yf);
        } else
        #endif
        #if CYTHON_COMPILING_IN_CPYTHON && PY_VERSION_HEX >= 0x03030000 && (defined(__linux__) || PY_VERSION_HEX >= 0x030600B3)
        if (PyGen_CheckExact(yf)) {
            ret = _PyGen_Send((PyGenObject*)yf, NULL);
        } else
        #endif
        #ifdef __Pyx_Coroutine_USED
        if (__Pyx_Coroutine_Check(yf)) {
            ret = __Pyx_Coroutine_Send(yf, Py_None);
        } else
        #endif
            ret = Py_TYPE(yf)->tp_iternext(yf);
        gen->is_running = 0;
        if (likely(ret)) {
            return ret;
        }
        return __Pyx_Coroutine_FinishDelegation(gen);
    }
    return __Pyx_Coroutine_SendEx(gen, Py_None, 0);
}
static PyObject *__Pyx_Coroutine_Close(PyObject *self) {
    __pyx_CoroutineObject *gen = (__pyx_CoroutineObject *) self;
    PyObject *retval, *raised_exception;
    PyObject *yf = gen->yieldfrom;
    int err = 0;
    if (unlikely(gen->is_running))
        return __Pyx_Coroutine_AlreadyRunningError(gen);
    if (yf) {
        Py_INCREF(yf);
        err = __Pyx_Coroutine_CloseIter(gen, yf);
        __Pyx_Coroutine_Undelegate(gen);
        Py_DECREF(yf);
    }
    if (err == 0)
        PyErr_SetNone(PyExc_GeneratorExit);
    retval = __Pyx_Coroutine_SendEx(gen, NULL, 1);
    if (unlikely(retval)) {
        const char *msg;
        Py_DECREF(retval);
        if ((0)) {
        #ifdef __Pyx_Coroutine_USED
        } else if (__Pyx_Coroutine_Check(self)) {
            msg = "coroutine ignored GeneratorExit";
        #endif
        #ifdef __Pyx_AsyncGen_USED
        } else if (__Pyx_AsyncGen_CheckExact(self)) {
#if PY_VERSION_HEX < 0x03060000
            msg = "async generator ignored GeneratorExit - might require Python 3.6+ finalisation (PEP 525)";
#else
            msg = "async generator ignored GeneratorExit";
#endif
        #endif
        } else {
            msg = "generator ignored GeneratorExit";
        }
        PyErr_SetString(PyExc_RuntimeError, msg);
        return NULL;
    }
    raised_exception = PyErr_Occurred();
    if (likely(!raised_exception || __Pyx_PyErr_GivenExceptionMatches2(raised_exception, PyExc_GeneratorExit, PyExc_StopIteration))) {
        if (raised_exception) PyErr_Clear();
        Py_INCREF(Py_None);
        return Py_None;
    }
    return NULL;
}
static PyObject *__Pyx__Coroutine_Throw(PyObject *self, PyObject *typ, PyObject *val, PyObject *tb,
                                        PyObject *args, int close_on_genexit) {
    __pyx_CoroutineObject *gen = (__pyx_CoroutineObject *) self;
    PyObject *yf = gen->yieldfrom;
    if (unlikely(gen->is_running))
        return __Pyx_Coroutine_AlreadyRunningError(gen);
    if (yf) {
        PyObject *ret;
        Py_INCREF(yf);
        if (__Pyx_PyErr_GivenExceptionMatches(typ, PyExc_GeneratorExit) && close_on_genexit) {
            int err = __Pyx_Coroutine_CloseIter(gen, yf);
            Py_DECREF(yf);
            __Pyx_Coroutine_Undelegate(gen);
            if (err < 0)
                return __Pyx_Coroutine_MethodReturn(self, __Pyx_Coroutine_SendEx(gen, NULL, 0));
            goto throw_here;
        }
        gen->is_running = 1;
        if (0
        #ifdef __Pyx_Generator_USED
            || __Pyx_Generator_CheckExact(yf)
        #endif
        #ifdef __Pyx_Coroutine_USED
            || __Pyx_Coroutine_Check(yf)
        #endif
            ) {
            ret = __Pyx__Coroutine_Throw(yf, typ, val, tb, args, close_on_genexit);
        #ifdef __Pyx_Coroutine_USED
        } else if (__Pyx_CoroutineAwait_CheckExact(yf)) {
            ret = __Pyx__Coroutine_Throw(((__pyx_CoroutineAwaitObject*)yf)->coroutine, typ, val, tb, args, close_on_genexit);
        #endif
        } else {
            PyObject *meth = __Pyx_PyObject_GetAttrStr(yf, __pyx_n_s_throw);
            if (unlikely(!meth)) {
                Py_DECREF(yf);
                if (!PyErr_ExceptionMatches(PyExc_AttributeError)) {
                    gen->is_running = 0;
                    return NULL;
                }
                PyErr_Clear();
                __Pyx_Coroutine_Undelegate(gen);
                gen->is_running = 0;
                goto throw_here;
            }
            if (likely(args)) {
                ret = PyObject_CallObject(meth, args);
            } else {
                ret = PyObject_CallFunctionObjArgs(meth, typ, val, tb, NULL);
            }
            Py_DECREF(meth);
        }
        gen->is_running = 0;
        Py_DECREF(yf);
        if (!ret) {
            ret = __Pyx_Coroutine_FinishDelegation(gen);
        }
        return __Pyx_Coroutine_MethodReturn(self, ret);
    }
throw_here:
    __Pyx_Raise(typ, val, tb, NULL);
    return __Pyx_Coroutine_MethodReturn(self, __Pyx_Coroutine_SendEx(gen, NULL, 0));
}
static PyObject *__Pyx_Coroutine_Throw(PyObject *self, PyObject *args) {
    PyObject *typ;
    PyObject *val = NULL;
    PyObject *tb = NULL;
    if (!PyArg_UnpackTuple(args, (char *)"throw", 1, 3, &typ, &val, &tb))
        return NULL;
    return __Pyx__Coroutine_Throw(self, typ, val, tb, args, 1);
}
static int __Pyx_Coroutine_traverse(__pyx_CoroutineObject *gen, visitproc visit, void *arg) {
    Py_VISIT(gen->closure);
    Py_VISIT(gen->classobj);
    Py_VISIT(gen->yieldfrom);
    Py_VISIT(gen->exc_type);
    Py_VISIT(gen->exc_value);
    Py_VISIT(gen->exc_traceback);
    return 0;
}
static int __Pyx_Coroutine_clear(PyObject *self) {
    __pyx_CoroutineObject *gen = (__pyx_CoroutineObject *) self;
    Py_CLEAR(gen->closure);
    Py_CLEAR(gen->classobj);
    Py_CLEAR(gen->yieldfrom);
    Py_CLEAR(gen->exc_type);
    Py_CLEAR(gen->exc_value);
    Py_CLEAR(gen->exc_traceback);
#ifdef __Pyx_AsyncGen_USED
    if (__Pyx_AsyncGen_CheckExact(self)) {
        Py_CLEAR(((__pyx_PyAsyncGenObject*)gen)->ag_finalizer);
    }
#endif
    Py_CLEAR(gen->gi_code);
    Py_CLEAR(gen->gi_name);
    Py_CLEAR(gen->gi_qualname);
    Py_CLEAR(gen->gi_modulename);
    return 0;
}
static void __Pyx_Coroutine_dealloc(PyObject *self) {
    __pyx_CoroutineObject *gen = (__pyx_CoroutineObject *) self;
    PyObject_GC_UnTrack(gen);
    if (gen->gi_weakreflist != NULL)
        PyObject_ClearWeakRefs(self);
    if (gen->resume_label >= 0) {
        PyObject_GC_Track(self);
#if PY_VERSION_HEX >= 0x030400a1 && CYTHON_USE_TP_FINALIZE
        if (PyObject_CallFinalizerFromDealloc(self))
#else
        Py_TYPE(gen)->tp_del(self);
        if (self->ob_refcnt > 0)
#endif
        {
            return;
        }
        PyObject_GC_UnTrack(self);
    }
#ifdef __Pyx_AsyncGen_USED
    if (__Pyx_AsyncGen_CheckExact(self)) {
        /* We have to handle this case for asynchronous generators
           right here, because this code has to be between UNTRACK
           and GC_Del. */
        Py_CLEAR(((__pyx_PyAsyncGenObject*)self)->ag_finalizer);
    }
#endif
    __Pyx_Coroutine_clear(self);
    PyObject_GC_Del(gen);
}
static void __Pyx_Coroutine_del(PyObject *self) {
    PyObject *error_type, *error_value, *error_traceback;
    __pyx_CoroutineObject *gen = (__pyx_CoroutineObject *) self;
    __Pyx_PyThreadState_declare
    if (gen->resume_label < 0) {
        return;
    }
#if !CYTHON_USE_TP_FINALIZE
    assert(self->ob_refcnt == 0);
    self->ob_refcnt = 1;
#endif
    __Pyx_PyThreadState_assign
    __Pyx_ErrFetch(&error_type, &error_value, &error_traceback);
#ifdef __Pyx_AsyncGen_USED
    if (__Pyx_AsyncGen_CheckExact(self)) {
        __pyx_PyAsyncGenObject *agen = (__pyx_PyAsyncGenObject*)self;
        PyObject *finalizer = agen->ag_finalizer;
        if (finalizer && !agen->ag_closed) {
            PyObject *res = __Pyx_PyObject_CallOneArg(finalizer, self);
            if (unlikely(!res)) {
                PyErr_WriteUnraisable(self);
            } else {
                Py_DECREF(res);
            }
            __Pyx_ErrRestore(error_type, error_value, error_traceback);
            return;
        }
    }
#endif
    if (unlikely(gen->resume_label == 0 && !error_value)) {
#ifdef __Pyx_Coroutine_USED
#ifdef __Pyx_Generator_USED
    if (!__Pyx_Generator_CheckExact(self))
#endif
        {
        PyObject_GC_UnTrack(self);
#if PY_MAJOR_VERSION >= 3  || defined(PyErr_WarnFormat)
        if (unlikely(PyErr_WarnFormat(PyExc_RuntimeWarning, 1, "coroutine '%.50S' was never awaited", gen->gi_qualname) < 0))
            PyErr_WriteUnraisable(self);
#else
        {PyObject *msg;
        char *cmsg;
        #if CYTHON_COMPILING_IN_PYPY
        msg = NULL;
        cmsg = (char*) "coroutine was never awaited";
        #else
        char *cname;
        PyObject *qualname;
        qualname = gen->gi_qualname;
        cname = PyString_AS_STRING(qualname);
        msg = PyString_FromFormat("coroutine '%.50s' was never awaited", cname);
        if (unlikely(!msg)) {
            PyErr_Clear();
            cmsg = (char*) "coroutine was never awaited";
        } else {
            cmsg = PyString_AS_STRING(msg);
        }
        #endif
        if (unlikely(PyErr_WarnEx(PyExc_RuntimeWarning, cmsg, 1) < 0))
            PyErr_WriteUnraisable(self);
        Py_XDECREF(msg);}
#endif
        PyObject_GC_Track(self);
        }
#endif
    } else {
        PyObject *res = __Pyx_Coroutine_Close(self);
        if (unlikely(!res)) {
            if (PyErr_Occurred())
                PyErr_WriteUnraisable(self);
        } else {
            Py_DECREF(res);
        }
    }
    __Pyx_ErrRestore(error_type, error_value, error_traceback);
#if !CYTHON_USE_TP_FINALIZE
    assert(self->ob_refcnt > 0);
    if (--self->ob_refcnt == 0) {
        return;
    }
    {
        Py_ssize_t refcnt = self->ob_refcnt;
        _Py_NewReference(self);
        self->ob_refcnt = refcnt;
    }
#if CYTHON_COMPILING_IN_CPYTHON
    assert(PyType_IS_GC(self->ob_type) &&
           _Py_AS_GC(self)->gc.gc_refs != _PyGC_REFS_UNTRACKED);
    _Py_DEC_REFTOTAL;
#endif
#ifdef COUNT_ALLOCS
    --Py_TYPE(self)->tp_frees;
    --Py_TYPE(self)->tp_allocs;
#endif
#endif
}
static PyObject *
__Pyx_Coroutine_get_name(__pyx_CoroutineObject *self)
{
    PyObject *name = self->gi_name;
    if (unlikely(!name)) name = Py_None;
    Py_INCREF(name);
    return name;
}
static int
__Pyx_Coroutine_set_name(__pyx_CoroutineObject *self, PyObject *value)
{
    PyObject *tmp;
#if PY_MAJOR_VERSION >= 3
    if (unlikely(value == NULL || !PyUnicode_Check(value))) {
#else
    if (unlikely(value == NULL || !PyString_Check(value))) {
#endif
        PyErr_SetString(PyExc_TypeError,
                        "__name__ must be set to a string object");
        return -1;
    }
    tmp = self->gi_name;
    Py_INCREF(value);
    self->gi_name = value;
    Py_XDECREF(tmp);
    return 0;
}
static PyObject *
__Pyx_Coroutine_get_qualname(__pyx_CoroutineObject *self)
{
    PyObject *name = self->gi_qualname;
    if (unlikely(!name)) name = Py_None;
    Py_INCREF(name);
    return name;
}
static int
__Pyx_Coroutine_set_qualname(__pyx_CoroutineObject *self, PyObject *value)
{
    PyObject *tmp;
#if PY_MAJOR_VERSION >= 3
    if (unlikely(value == NULL || !PyUnicode_Check(value))) {
#else
    if (unlikely(value == NULL || !PyString_Check(value))) {
#endif
        PyErr_SetString(PyExc_TypeError,
                        "__qualname__ must be set to a string object");
        return -1;
    }
    tmp = self->gi_qualname;
    Py_INCREF(value);
    self->gi_qualname = value;
    Py_XDECREF(tmp);
    return 0;
}
static __pyx_CoroutineObject *__Pyx__Coroutine_New(
            PyTypeObject* type, __pyx_coroutine_body_t body, PyObject *code, PyObject *closure,
            PyObject *name, PyObject *qualname, PyObject *module_name) {
    __pyx_CoroutineObject *gen = PyObject_GC_New(__pyx_CoroutineObject, type);
    if (unlikely(!gen))
        return NULL;
    return __Pyx__Coroutine_NewInit(gen, body, code, closure, name, qualname, module_name);
}
static __pyx_CoroutineObject *__Pyx__Coroutine_NewInit(
            __pyx_CoroutineObject *gen, __pyx_coroutine_body_t body, PyObject *code, PyObject *closure,
            PyObject *name, PyObject *qualname, PyObject *module_name) {
    gen->body = body;
    gen->closure = closure;
    Py_XINCREF(closure);
    gen->is_running = 0;
    gen->resume_label = 0;
    gen->classobj = NULL;
    gen->yieldfrom = NULL;
    gen->exc_type = NULL;
    gen->exc_value = NULL;
    gen->exc_traceback = NULL;
    gen->gi_weakreflist = NULL;
    Py_XINCREF(qualname);
    gen->gi_qualname = qualname;
    Py_XINCREF(name);
    gen->gi_name = name;
    Py_XINCREF(module_name);
    gen->gi_modulename = module_name;
    Py_XINCREF(code);
    gen->gi_code = code;
    PyObject_GC_Track(gen);
    return gen;
}

/* PatchModuleWithCoroutine */
                    static PyObject* __Pyx_Coroutine_patch_module(PyObject* module, const char* py_code) {
#if defined(__Pyx_Generator_USED) || defined(__Pyx_Coroutine_USED)
    int result;
    PyObject *globals, *result_obj;
    globals = PyDict_New();  if (unlikely(!globals)) goto ignore;
    result = PyDict_SetItemString(globals, "_cython_coroutine_type",
    #ifdef __Pyx_Coroutine_USED
        (PyObject*)__pyx_CoroutineType);
    #else
        Py_None);
    #endif
    if (unlikely(result < 0)) goto ignore;
    result = PyDict_SetItemString(globals, "_cython_generator_type",
    #ifdef __Pyx_Generator_USED
        (PyObject*)__pyx_GeneratorType);
    #else
        Py_None);
    #endif
    if (unlikely(result < 0)) goto ignore;
    if (unlikely(PyDict_SetItemString(globals, "_module", module) < 0)) goto ignore;
    if (unlikely(PyDict_SetItemString(globals, "__builtins__", __pyx_b) < 0)) goto ignore;
    result_obj = PyRun_String(py_code, Py_file_input, globals, globals);
    if (unlikely(!result_obj)) goto ignore;
    Py_DECREF(result_obj);
    Py_DECREF(globals);
    return module;
ignore:
    Py_XDECREF(globals);
    PyErr_WriteUnraisable(module);
    if (unlikely(PyErr_WarnEx(PyExc_RuntimeWarning, "Cython module failed to patch module with custom type", 1) < 0)) {
        Py_DECREF(module);
        module = NULL;
    }
#else
    py_code++;
#endif
    return module;
}

/* PatchGeneratorABC */
                    #ifndef CYTHON_REGISTER_ABCS
#define CYTHON_REGISTER_ABCS 1
#endif
#if defined(__Pyx_Generator_USED) || defined(__Pyx_Coroutine_USED)
static PyObject* __Pyx_patch_abc_module(PyObject *module);
static PyObject* __Pyx_patch_abc_module(PyObject *module) {
    module = __Pyx_Coroutine_patch_module(
        module, ""
"if _cython_generator_type is not None:\n"
"    try: Generator = _module.Generator\n"
"    except AttributeError: pass\n"
"    else: Generator.register(_cython_generator_type)\n"
"if _cython_coroutine_type is not None:\n"
"    try: Coroutine = _module.Coroutine\n"
"    except AttributeError: pass\n"
"    else: Coroutine.register(_cython_coroutine_type)\n"
    );
    return module;
}
#endif
static int __Pyx_patch_abc(void) {
#if defined(__Pyx_Generator_USED) || defined(__Pyx_Coroutine_USED)
    static int abc_patched = 0;
    if (CYTHON_REGISTER_ABCS && !abc_patched) {
        PyObject *module;
        module = PyImport_ImportModule((PY_MAJOR_VERSION >= 3) ? "collections.abc" : "collections");
        if (!module) {
            PyErr_WriteUnraisable(NULL);
            if (unlikely(PyErr_WarnEx(PyExc_RuntimeWarning,
                    ((PY_MAJOR_VERSION >= 3) ?
                        "Cython module failed to register with collections.abc module" :
                        "Cython module failed to register with collections module"), 1) < 0)) {
                return -1;
            }
        } else {
            module = __Pyx_patch_abc_module(module);
            abc_patched = 1;
            if (unlikely(!module))
                return -1;
            Py_DECREF(module);
        }
        module = PyImport_ImportModule("backports_abc");
        if (module) {
            module = __Pyx_patch_abc_module(module);
            Py_XDECREF(module);
        }
        if (!module) {
            PyErr_Clear();
        }
    }
#else
    if ((0)) __Pyx_Coroutine_patch_module(NULL, NULL);
#endif
    return 0;
}

/* Generator */
                    static PyMethodDef __pyx_Generator_methods[] = {
    {"send", (PyCFunction) __Pyx_Coroutine_Send, METH_O,
     (char*) PyDoc_STR("send(arg) -> send 'arg' into generator,\nreturn next yielded value or raise StopIteration.")},
    {"throw", (PyCFunction) __Pyx_Coroutine_Throw, METH_VARARGS,
     (char*) PyDoc_STR("throw(typ[,val[,tb]]) -> raise exception in generator,\nreturn next yielded value or raise StopIteration.")},
    {"close", (PyCFunction) __Pyx_Coroutine_Close, METH_NOARGS,
     (char*) PyDoc_STR("close() -> raise GeneratorExit inside generator.")},
    {0, 0, 0, 0}
};
static PyMemberDef __pyx_Generator_memberlist[] = {
    {(char *) "gi_running", T_BOOL, offsetof(__pyx_CoroutineObject, is_running), READONLY, NULL},
    {(char*) "gi_yieldfrom", T_OBJECT, offsetof(__pyx_CoroutineObject, yieldfrom), READONLY,
     (char*) PyDoc_STR("object being iterated by 'yield from', or None")},
    {(char*) "gi_code", T_OBJECT, offsetof(__pyx_CoroutineObject, gi_code), READONLY, NULL},
    {0, 0, 0, 0, 0}
};
static PyGetSetDef __pyx_Generator_getsets[] = {
    {(char *) "__name__", (getter)__Pyx_Coroutine_get_name, (setter)__Pyx_Coroutine_set_name,
     (char*) PyDoc_STR("name of the generator"), 0},
    {(char *) "__qualname__", (getter)__Pyx_Coroutine_get_qualname, (setter)__Pyx_Coroutine_set_qualname,
     (char*) PyDoc_STR("qualified name of the generator"), 0},
    {0, 0, 0, 0, 0}
};
static PyTypeObject __pyx_GeneratorType_type = {
    PyVarObject_HEAD_INIT(0, 0)
    "generator",
    sizeof(__pyx_CoroutineObject),
    0,
    (destructor) __Pyx_Coroutine_dealloc,
    0,
    0,
    0,
    0,
    0,
    0,
    0,
    0,
    0,
    0,
    0,
    0,
    0,
    0,
    Py_TPFLAGS_DEFAULT | Py_TPFLAGS_HAVE_GC | Py_TPFLAGS_HAVE_FINALIZE,
    0,
    (traverseproc) __Pyx_Coroutine_traverse,
    0,
    0,
    offsetof(__pyx_CoroutineObject, gi_weakreflist),
    0,
    (iternextfunc) __Pyx_Generator_Next,
    __pyx_Generator_methods,
    __pyx_Generator_memberlist,
    __pyx_Generator_getsets,
    0,
    0,
    0,
    0,
    0,
    0,
    0,
    0,
    0,
    0,
    0,
    0,
    0,
    0,
    0,
#if CYTHON_USE_TP_FINALIZE
    0,
#else
    __Pyx_Coroutine_del,
#endif
    0,
#if CYTHON_USE_TP_FINALIZE
    __Pyx_Coroutine_del,
#elif PY_VERSION_HEX >= 0x030400a1
    0,
#endif
};
static int __pyx_Generator_init(void) {
    __pyx_GeneratorType_type.tp_getattro = __Pyx_PyObject_GenericGetAttrNoDict;
    __pyx_GeneratorType_type.tp_iter = PyObject_SelfIter;
    __pyx_GeneratorType = __Pyx_FetchCommonType(&__pyx_GeneratorType_type);
    if (unlikely(!__pyx_GeneratorType)) {
        return -1;
    }
    return 0;
}

/* CheckBinaryVersion */
                    static int __Pyx_check_binary_version(void) {
    char ctversion[4], rtversion[4];
    PyOS_snprintf(ctversion, 4, "%d.%d", PY_MAJOR_VERSION, PY_MINOR_VERSION);
    PyOS_snprintf(rtversion, 4, "%s", Py_GetVersion());
    if (ctversion[0] != rtversion[0] || ctversion[2] != rtversion[2]) {
        char message[200];
        PyOS_snprintf(message, sizeof(message),
                      "compiletime version %s of module '%.100s' "
                      "does not match runtime version %s",
                      ctversion, __Pyx_MODULE_NAME, rtversion);
        return PyErr_WarnEx(NULL, message, 1);
    }
    return 0;
}

/* FunctionExport */
                    static int __Pyx_ExportFunction(const char *name, void (*f)(void), const char *sig) {
    PyObject *d = 0;
    PyObject *cobj = 0;
    union {
        void (*fp)(void);
        void *p;
    } tmp;
    d = PyObject_GetAttrString(__pyx_m, (char *)"__pyx_capi__");
    if (!d) {
        PyErr_Clear();
        d = PyDict_New();
        if (!d)
            goto bad;
        Py_INCREF(d);
        if (PyModule_AddObject(__pyx_m, (char *)"__pyx_capi__", d) < 0)
            goto bad;
    }
    tmp.fp = f;
#if PY_VERSION_HEX >= 0x02070000
    cobj = PyCapsule_New(tmp.p, sig, 0);
#else
    cobj = PyCObject_FromVoidPtrAndDesc(tmp.p, (void *)sig, 0);
#endif
    if (!cobj)
        goto bad;
    if (PyDict_SetItemString(d, name, cobj) < 0)
        goto bad;
    Py_DECREF(cobj);
    Py_DECREF(d);
    return 0;
bad:
    Py_XDECREF(cobj);
    Py_XDECREF(d);
    return -1;
}

/* ModuleImport */
                    #ifndef __PYX_HAVE_RT_ImportModule
#define __PYX_HAVE_RT_ImportModule
static PyObject *__Pyx_ImportModule(const char *name) {
    PyObject *py_name = 0;
    PyObject *py_module = 0;
    py_name = __Pyx_PyIdentifier_FromString(name);
    if (!py_name)
        goto bad;
    py_module = PyImport_Import(py_name);
    Py_DECREF(py_name);
    return py_module;
bad:
    Py_XDECREF(py_name);
    return 0;
}
#endif

/* TypeImport */
                    #ifndef __PYX_HAVE_RT_ImportType
#define __PYX_HAVE_RT_ImportType
static PyTypeObject *__Pyx_ImportType(const char *module_name, const char *class_name,
    size_t size, int strict)
{
    PyObject *py_module = 0;
    PyObject *result = 0;
    PyObject *py_name = 0;
    char warning[200];
    Py_ssize_t basicsize;
#ifdef Py_LIMITED_API
    PyObject *py_basicsize;
#endif
    py_module = __Pyx_ImportModule(module_name);
    if (!py_module)
        goto bad;
    py_name = __Pyx_PyIdentifier_FromString(class_name);
    if (!py_name)
        goto bad;
    result = PyObject_GetAttr(py_module, py_name);
    Py_DECREF(py_name);
    py_name = 0;
    Py_DECREF(py_module);
    py_module = 0;
    if (!result)
        goto bad;
    if (!PyType_Check(result)) {
        PyErr_Format(PyExc_TypeError,
            "%.200s.%.200s is not a type object",
            module_name, class_name);
        goto bad;
    }
#ifndef Py_LIMITED_API
    basicsize = ((PyTypeObject *)result)->tp_basicsize;
#else
    py_basicsize = PyObject_GetAttrString(result, "__basicsize__");
    if (!py_basicsize)
        goto bad;
    basicsize = PyLong_AsSsize_t(py_basicsize);
    Py_DECREF(py_basicsize);
    py_basicsize = 0;
    if (basicsize == (Py_ssize_t)-1 && PyErr_Occurred())
        goto bad;
#endif
    if (!strict && (size_t)basicsize > size) {
        PyOS_snprintf(warning, sizeof(warning),
            "%s.%s size changed, may indicate binary incompatibility. Expected %zd, got %zd",
            module_name, class_name, basicsize, size);
        if (PyErr_WarnEx(NULL, warning, 0) < 0) goto bad;
    }
    else if ((size_t)basicsize != size) {
        PyErr_Format(PyExc_ValueError,
            "%.200s.%.200s has the wrong size, try recompiling. Expected %zd, got %zd",
            module_name, class_name, basicsize, size);
        goto bad;
    }
    return (PyTypeObject *)result;
bad:
    Py_XDECREF(py_module);
    Py_XDECREF(result);
    return NULL;
}
#endif

/* InitStrings */
                    static int __Pyx_InitStrings(__Pyx_StringTabEntry *t) {
    while (t->p) {
        #if PY_MAJOR_VERSION < 3
        if (t->is_unicode) {
            *t->p = PyUnicode_DecodeUTF8(t->s, t->n - 1, NULL);
        } else if (t->intern) {
            *t->p = PyString_InternFromString(t->s);
        } else {
            *t->p = PyString_FromStringAndSize(t->s, t->n - 1);
        }
        #else
        if (t->is_unicode | t->is_str) {
            if (t->intern) {
                *t->p = PyUnicode_InternFromString(t->s);
            } else if (t->encoding) {
                *t->p = PyUnicode_Decode(t->s, t->n - 1, t->encoding, NULL);
            } else {
                *t->p = PyUnicode_FromStringAndSize(t->s, t->n - 1);
            }
        } else {
            *t->p = PyBytes_FromStringAndSize(t->s, t->n - 1);
        }
        #endif
        if (!*t->p)
            return -1;
        if (PyObject_Hash(*t->p) == -1)
            return -1;
        ++t;
    }
    return 0;
}

static CYTHON_INLINE PyObject* __Pyx_PyUnicode_FromString(const char* c_str) {
    return __Pyx_PyUnicode_FromStringAndSize(c_str, (Py_ssize_t)strlen(c_str));
}
static CYTHON_INLINE const char* __Pyx_PyObject_AsString(PyObject* o) {
    Py_ssize_t ignore;
    return __Pyx_PyObject_AsStringAndSize(o, &ignore);
}
#if __PYX_DEFAULT_STRING_ENCODING_IS_ASCII || __PYX_DEFAULT_STRING_ENCODING_IS_DEFAULT
#if !CYTHON_PEP393_ENABLED
static const char* __Pyx_PyUnicode_AsStringAndSize(PyObject* o, Py_ssize_t *length) {
    char* defenc_c;
    PyObject* defenc = _PyUnicode_AsDefaultEncodedString(o, NULL);
    if (!defenc) return NULL;
    defenc_c = PyBytes_AS_STRING(defenc);
#if __PYX_DEFAULT_STRING_ENCODING_IS_ASCII
    {
        char* end = defenc_c + PyBytes_GET_SIZE(defenc);
        char* c;
        for (c = defenc_c; c < end; c++) {
            if ((unsigned char) (*c) >= 128) {
                PyUnicode_AsASCIIString(o);
                return NULL;
            }
        }
    }
#endif
    *length = PyBytes_GET_SIZE(defenc);
    return defenc_c;
}
#else
static CYTHON_INLINE const char* __Pyx_PyUnicode_AsStringAndSize(PyObject* o, Py_ssize_t *length) {
    if (unlikely(__Pyx_PyUnicode_READY(o) == -1)) return NULL;
#if __PYX_DEFAULT_STRING_ENCODING_IS_ASCII
    if (likely(PyUnicode_IS_ASCII(o))) {
        *length = PyUnicode_GET_LENGTH(o);
        return PyUnicode_AsUTF8(o);
    } else {
        PyUnicode_AsASCIIString(o);
        return NULL;
    }
#else
    return PyUnicode_AsUTF8AndSize(o, length);
#endif
}
#endif
#endif
static CYTHON_INLINE const char* __Pyx_PyObject_AsStringAndSize(PyObject* o, Py_ssize_t *length) {
#if __PYX_DEFAULT_STRING_ENCODING_IS_ASCII || __PYX_DEFAULT_STRING_ENCODING_IS_DEFAULT
    if (
#if PY_MAJOR_VERSION < 3 && __PYX_DEFAULT_STRING_ENCODING_IS_ASCII
            __Pyx_sys_getdefaultencoding_not_ascii &&
#endif
            PyUnicode_Check(o)) {
        return __Pyx_PyUnicode_AsStringAndSize(o, length);
    } else
#endif
#if (!CYTHON_COMPILING_IN_PYPY) || (defined(PyByteArray_AS_STRING) && defined(PyByteArray_GET_SIZE))
    if (PyByteArray_Check(o)) {
        *length = PyByteArray_GET_SIZE(o);
        return PyByteArray_AS_STRING(o);
    } else
#endif
    {
        char* result;
        int r = PyBytes_AsStringAndSize(o, &result, length);
        if (unlikely(r < 0)) {
            return NULL;
        } else {
            return result;
        }
    }
}
static CYTHON_INLINE int __Pyx_PyObject_IsTrue(PyObject* x) {
   int is_true = x == Py_True;
   if (is_true | (x == Py_False) | (x == Py_None)) return is_true;
   else return PyObject_IsTrue(x);
}
static PyObject* __Pyx_PyNumber_IntOrLongWrongResultType(PyObject* result, const char* type_name) {
#if PY_MAJOR_VERSION >= 3
    if (PyLong_Check(result)) {
        if (PyErr_WarnFormat(PyExc_DeprecationWarning, 1,
                "__int__ returned non-int (type %.200s).  "
                "The ability to return an instance of a strict subclass of int "
                "is deprecated, and may be removed in a future version of Python.",
                Py_TYPE(result)->tp_name)) {
            Py_DECREF(result);
            return NULL;
        }
        return result;
    }
#endif
    PyErr_Format(PyExc_TypeError,
                 "__%.4s__ returned non-%.4s (type %.200s)",
                 type_name, type_name, Py_TYPE(result)->tp_name);
    Py_DECREF(result);
    return NULL;
}
static CYTHON_INLINE PyObject* __Pyx_PyNumber_IntOrLong(PyObject* x) {
#if CYTHON_USE_TYPE_SLOTS
  PyNumberMethods *m;
#endif
  const char *name = NULL;
  PyObject *res = NULL;
#if PY_MAJOR_VERSION < 3
  if (likely(PyInt_Check(x) || PyLong_Check(x)))
#else
  if (likely(PyLong_Check(x)))
#endif
    return __Pyx_NewRef(x);
#if CYTHON_USE_TYPE_SLOTS
  m = Py_TYPE(x)->tp_as_number;
  #if PY_MAJOR_VERSION < 3
  if (m && m->nb_int) {
    name = "int";
    res = m->nb_int(x);
  }
  else if (m && m->nb_long) {
    name = "long";
    res = m->nb_long(x);
  }
  #else
  if (likely(m && m->nb_int)) {
    name = "int";
    res = m->nb_int(x);
  }
  #endif
#else
  if (!PyBytes_CheckExact(x) && !PyUnicode_CheckExact(x)) {
    res = PyNumber_Int(x);
  }
#endif
  if (likely(res)) {
#if PY_MAJOR_VERSION < 3
    if (unlikely(!PyInt_Check(res) && !PyLong_Check(res))) {
#else
    if (unlikely(!PyLong_CheckExact(res))) {
#endif
        return __Pyx_PyNumber_IntOrLongWrongResultType(res, name);
    }
  }
  else if (!PyErr_Occurred()) {
    PyErr_SetString(PyExc_TypeError,
                    "an integer is required");
  }
  return res;
}
static CYTHON_INLINE Py_ssize_t __Pyx_PyIndex_AsSsize_t(PyObject* b) {
  Py_ssize_t ival;
  PyObject *x;
#if PY_MAJOR_VERSION < 3
  if (likely(PyInt_CheckExact(b))) {
    if (sizeof(Py_ssize_t) >= sizeof(long))
        return PyInt_AS_LONG(b);
    else
        return PyInt_AsSsize_t(x);
  }
#endif
  if (likely(PyLong_CheckExact(b))) {
    #if CYTHON_USE_PYLONG_INTERNALS
    const digit* digits = ((PyLongObject*)b)->ob_digit;
    const Py_ssize_t size = Py_SIZE(b);
    if (likely(__Pyx_sst_abs(size) <= 1)) {
        ival = likely(size) ? digits[0] : 0;
        if (size == -1) ival = -ival;
        return ival;
    } else {
      switch (size) {
         case 2:
           if (8 * sizeof(Py_ssize_t) > 2 * PyLong_SHIFT) {
             return (Py_ssize_t) (((((size_t)digits[1]) << PyLong_SHIFT) | (size_t)digits[0]));
           }
           break;
         case -2:
           if (8 * sizeof(Py_ssize_t) > 2 * PyLong_SHIFT) {
             return -(Py_ssize_t) (((((size_t)digits[1]) << PyLong_SHIFT) | (size_t)digits[0]));
           }
           break;
         case 3:
           if (8 * sizeof(Py_ssize_t) > 3 * PyLong_SHIFT) {
             return (Py_ssize_t) (((((((size_t)digits[2]) << PyLong_SHIFT) | (size_t)digits[1]) << PyLong_SHIFT) | (size_t)digits[0]));
           }
           break;
         case -3:
           if (8 * sizeof(Py_ssize_t) > 3 * PyLong_SHIFT) {
             return -(Py_ssize_t) (((((((size_t)digits[2]) << PyLong_SHIFT) | (size_t)digits[1]) << PyLong_SHIFT) | (size_t)digits[0]));
           }
           break;
         case 4:
           if (8 * sizeof(Py_ssize_t) > 4 * PyLong_SHIFT) {
             return (Py_ssize_t) (((((((((size_t)digits[3]) << PyLong_SHIFT) | (size_t)digits[2]) << PyLong_SHIFT) | (size_t)digits[1]) << PyLong_SHIFT) | (size_t)digits[0]));
           }
           break;
         case -4:
           if (8 * sizeof(Py_ssize_t) > 4 * PyLong_SHIFT) {
             return -(Py_ssize_t) (((((((((size_t)digits[3]) << PyLong_SHIFT) | (size_t)digits[2]) << PyLong_SHIFT) | (size_t)digits[1]) << PyLong_SHIFT) | (size_t)digits[0]));
           }
           break;
      }
    }
    #endif
    return PyLong_AsSsize_t(b);
  }
  x = PyNumber_Index(b);
  if (!x) return -1;
  ival = PyInt_AsSsize_t(x);
  Py_DECREF(x);
  return ival;
}
static CYTHON_INLINE PyObject * __Pyx_PyBool_FromLong(long b) {
  return b ? __Pyx_NewRef(Py_True) : __Pyx_NewRef(Py_False);
}
static CYTHON_INLINE PyObject * __Pyx_PyInt_FromSize_t(size_t ival) {
    return PyInt_FromSize_t(ival);
}


#endif /* Py_PYTHON_H */
